/* Do not modify this file.                                                   */
/* It is created automatically by the ASN.1 to Wireshark dissector compiler   */
/* packet-rrc.c                                                               */
/* ../../tools/asn2wrs.py -p rrc -c ./rrc.cnf -s ./packet-rrc-template -D . -O ../../epan/dissectors Class-definitions.asn PDU-definitions.asn InformationElements.asn Constant-definitions.asn Internode-definitions.asn */

/* Input file: packet-rrc-template.c */

#line 1 "../../asn1/rrc/packet-rrc-template.c"
/* packet-rrc.c
 * Routines for Universal Mobile Telecommunications System (UMTS);
 * Radio Resource Control (RRC) protocol specification
 * (3GPP TS 25.331  packet dissection
 * Copyright 2006-2010, Anders Broman <anders.broman@ericsson.com>
 *
 * $Id$
 *
 * Wireshark - Network traffic analyzer
 * By Gerald Combs <gerald@wireshark.org>
 * Copyright 1998 Gerald Combs
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 * Ref: 3GPP TS 25.331 V11.4.0 (2012-12)
 */

/**
 *
 * TODO:
 * - Fix ciphering information for circuit switched stuff
 */

#include "config.h"

#include <glib.h>
#include <epan/packet.h>
#include <epan/asn1.h>
#include <epan/conversation.h>
#include <epan/expert.h>

#include "packet-ber.h"
#include "packet-per.h"
#include "packet-rrc.h"
#include "packet-gsm_a_common.h"
#include "packet-umts_fp.h"

#ifdef _MSC_VER
/* disable: "warning C4049: compiler limit : terminating line number emission" */
#pragma warning(disable:4049)
/* disable: "warning C4146: unary minus operator applied to unsigned type, result still unsigned" */
#pragma warning(disable:4146)
#endif

#define PNAME  "Radio Resource Control (RRC) protocol"
#define PSNAME "RRC"
#define PFNAME "rrc"

extern int proto_fp;    /*Handler to FP*/

GTree * hsdsch_muxed_flows = NULL;
GTree * rrc_ciph_inf = NULL;
static int msg_type _U_;

static dissector_handle_t gsm_a_dtap_handle;
static dissector_handle_t rrc_ue_radio_access_cap_info_handle=NULL;
static dissector_handle_t rrc_pcch_handle=NULL;
static dissector_handle_t rrc_ul_ccch_handle=NULL;
static dissector_handle_t rrc_dl_ccch_handle=NULL;
static dissector_handle_t rrc_ul_dcch_handle=NULL;
static dissector_handle_t rrc_dl_dcch_handle=NULL;
static dissector_handle_t rrc_bcch_fach_handle=NULL;
static dissector_handle_t lte_rrc_ue_eutra_cap_handle=NULL;
static dissector_handle_t lte_rrc_dl_dcch_handle=NULL;
static dissector_handle_t gsm_rlcmac_dl_handle=NULL;

enum nas_sys_info_gsm_map {
  RRC_NAS_SYS_INFO_CS,
  RRC_NAS_SYS_INFO_PS,
  RRC_NAS_SYS_INFO_CN_COMMON
};

static guint32 rrc_nas_sys_info_gsm_map_type = RRC_NAS_SYS_INFO_CN_COMMON;

/* Forward declarations */
static int dissect_UE_RadioAccessCapabilityInfo_PDU(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree, void *);
static int dissect_SysInfoTypeSB1_PDU(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree, void *);
static int dissect_SysInfoTypeSB2_PDU(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree, void *);
static int dissect_SysInfoType5_PDU(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree, void *);
static int dissect_SysInfoType11_PDU(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree, void *);
static int dissect_SysInfoType11bis_PDU(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree, void *);
static int dissect_SysInfoType22_PDU(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree, void *);

/* Include constants */

/*--- Included file: packet-rrc-val.h ---*/
#line 1 "../../asn1/rrc/packet-rrc-val.h"
#define hiPDSCHidentities              64
#define hiPUSCHidentities              64
#define hiRM                           256
#define maxAC                          16
#define maxAdditionalMeas              4
#define maxASC                         8
#define maxASCmap                      7
#define maxASCpersist                  6
#define maxCCTrCH                      8
#define maxCellMeas                    32
#define maxCellMeasOnSecULFreq         32
#define maxCellMeas_1                  31
#define maxCellMeasOnSecULFreq_1       31
#define maxCNdomains                   4
#define maxCommonHRNTI                 4
#define maxCommonQueueID               2
#define maxCPCHsets                    16
#define maxDedicatedCSGFreq            4
#define maxDPCH_DLchan                 8
#define maxDPDCH_UL                    6
#define maxDRACclasses                 8
#define maxE_DCHMACdFlow               8
#define maxE_DCHMACdFlow_1             7
#define maxMultipleFrequencyBandsFDD   8
#define maxMultipleFrequencyBandsEUTRA 8
#define maxEUTRACellPerFreq            16
#define maxEUTRATargetFreqs            8
#define maxEDCHRL                      4
#define maxEDCHRL_1                    3
#define maxEDCHs                       32
#define maxEDCHs_1                     31
#define maxEDCHTxPattern_TDD128        4
#define maxEDCHTxPattern_TDD128_1      3
#define maxERNTIgroup                  32
#define maxERNTIperGroup               2
#define maxERUCCH                      256
#define maxFACHPCH                     8
#define maxFreq                        8
#define maxFreqBandsEUTRA              16
#define maxFreqBandsFDD                8
#define maxFreqBandsFDD2               22
#define maxFreqBandsFDD3               86
#define maxFreqBandsFDD_ext            15
#define maxFreqBandsFDD_ext2           64
#define maxFreqBandsFDD_ext3           78
#define maxFreqBandsIndicatorSupport   2
#define maxFreqBandsTDD                4
#define maxFreqBandsTDD_ext            16
#define maxFreqBandsGSM                16
#define maxFreqMeasWithoutCM           2
#define maxGANSS                       8
#define maxGANSS_1                     7
#define maxGANSSSat                    64
#define maxGANSSSat_1                  63
#define maxGERAN_SI                    8
#define maxGSMTargetCells              32
#define maxHNBNameSize                 48
#define maxHProcesses                  8
#define maxHS_SCCHLessTrBlk            4
#define maxHSDSCHTBIndex               64
#define maxHSDSCHTBIndex_tdd384        512
#define maxHSSCCHs                     4
#define maxHSSCCHs_1                   3
#define maxHSSICH_TDD128               4
#define maxHSSICH_TDD128_1             3
#define maxInterSysMessages            4
#define maxLoCHperRLC                  2
#define maxLoggedMeasReport            128
#define maxMAC_d_PDUsizes              8
#define maxMBMS_CommonCCTrCh           32
#define maxMBMS_CommonPhyCh            32
#define maxMBMS_CommonRB               32
#define maxMBMS_CommonTrCh             32
#define maxMBMS_Freq                   4
#define maxMBMS_L1CP                   4
#define maxMBMSservCount               8
#define maxMBMSservModif               32
#define maxMBMSservSched               16
#define maxMBMSservSelect              8
#define maxMBMSservUnmodif             64
#define maxMBMSTransmis                4
#define maxMBSFNClusters               16
#define maxMeasCSGRange                4
#define maxMeasEvent                   8
#define maxMeasEventOnSecULFreq        8
#define maxMeasIntervals               3
#define maxMeasOccasionPattern         5
#define maxMeasOccasionPattern_1       4
#define maxMeasParEvent                2
#define maxNonContiguousMultiCellCombinations 3
#define maxNumCDMA2000Freqs            8
#define maxNumE_AGCH                   4
#define maxNumE_HICH                   4
#define maxNumEUTRAFreqs               8
#define maxNumEUTRAFreqs_FACH          4
#define maxNumGSMCellGroup             16
#define maxNumGSMFreqRanges            32
#define maxNumFDDFreqs                 8
#define maxNumANRLoggedItems           4
#define maxnumLoggedMeas               8
#define maxNumTDDFreqs                 8
#define maxNoOfMeas                    16
#define maxOtherRAT                    15
#define maxOtherRAT_16                 16
#define maxPage1                       8
#define maxPCPCH_APsig                 16
#define maxPCPCH_APsubCh               12
#define maxPCPCH_CDsig                 16
#define maxPCPCH_CDsubCh               12
#define maxPCPCH_SF                    7
#define maxPCPCHs                      64
#define maxPDCPAlgoType                8
#define maxPDSCH                       8
#define maxPDSCH_TFCIgroups            256
#define maxPRACH                       16
#define maxPRACH_EUL                   16
#define maxPRACH_FPACH                 8
#define maxPredefConfig                16
#define maxPrio                        8
#define maxPrio_1                      7
#define maxPUSCH                       8
#define maxQueueIDs                    8
#define maxRABsetup                    16
#define maxRAT                         16
#define maxRB                          32
#define maxRBallRABs                   27
#define maxRBMuxOptions                8
#define maxRBperRAB                    8
#define maxRBperTrCh                   16
#define maxReportedEUTRACellPerFreq    4
#define maxReportedEUTRAFreqs          4
#define maxReportedGSMCells            8
#define maxRL                          8
#define maxRL_1                        7
#define maxRLCPDUsizePerLogChan        32
#define maxRFC3095_CID                 16384
#define maxROHC_PacketSizes_r4         16
#define maxROHC_Profile_r4             8
#define maxRxPatternForHSDSCH_TDD128   4
#define maxRxPatternForHSDSCH_TDD128_1 3
#define maxSat                         16
#define maxSatClockModels              4
#define maxSCCPCH                      16
#define maxSgnType                     8
#define maxSIB                         32
#define maxSIB_FACH                    8
#define maxSIBperMsg                   16
#define maxSIrequest                   4
#define maxSRBsetup                    8
#define maxSystemCapability            16
#define maxTDD128Carrier               6
#define maxTDD128Carrier_1             5
#define maxTbsForHSDSCH_TDD128         4
#define maxTbsForHSDSCH_TDD128_1       3
#define maxTF                          32
#define maxTF_CPCH                     16
#define maxTFC                         1024
#define maxTFCsub                      1024
#define maxTFCI_2_Combs                512
#define maxTGPS                        6
#define maxTrCH                        32
#define maxTrCHpreconf                 32
#define maxTS                          14
#define maxTS_1                        13
#define maxTS_2                        12
#define maxTS_LCR                      6
#define maxTS_LCR_1                    5
#define maxURA                         8
#define maxURNTI_Group                 8

/*--- End of included file: packet-rrc-val.h ---*/
#line 98 "../../asn1/rrc/packet-rrc-template.c"

/* Initialize the protocol and registered fields */
int proto_rrc = -1;
static int hf_test;

/*--- Included file: packet-rrc-hf.c ---*/
#line 1 "../../asn1/rrc/packet-rrc-hf.c"
static int hf_rrc_DL_DCCH_Message_PDU = -1;       /* DL_DCCH_Message */
static int hf_rrc_UL_DCCH_Message_PDU = -1;       /* UL_DCCH_Message */
static int hf_rrc_DL_CCCH_Message_PDU = -1;       /* DL_CCCH_Message */
static int hf_rrc_UL_CCCH_Message_PDU = -1;       /* UL_CCCH_Message */
static int hf_rrc_PCCH_Message_PDU = -1;          /* PCCH_Message */
static int hf_rrc_DL_SHCCH_Message_PDU = -1;      /* DL_SHCCH_Message */
static int hf_rrc_UL_SHCCH_Message_PDU = -1;      /* UL_SHCCH_Message */
static int hf_rrc_BCCH_FACH_Message_PDU = -1;     /* BCCH_FACH_Message */
static int hf_rrc_BCCH_BCH_Message_PDU = -1;      /* BCCH_BCH_Message */
static int hf_rrc_MCCH_Message_PDU = -1;          /* MCCH_Message */
static int hf_rrc_MSCH_Message_PDU = -1;          /* MSCH_Message */
static int hf_rrc_rrc_HandoverToUTRANCommand_PDU = -1;  /* HandoverToUTRANCommand */
static int hf_rrc_rrc_InterRATHandoverInfo_PDU = -1;  /* InterRATHandoverInfo */
static int hf_rrc_SystemInformation_BCH_PDU = -1;  /* SystemInformation_BCH */
static int hf_rrc_System_Information_Container_PDU = -1;  /* System_Information_Container */
static int hf_rrc_UE_RadioAccessCapabilityInfo_PDU = -1;  /* UE_RadioAccessCapabilityInfo */
static int hf_rrc_rrc_MasterInformationBlock_PDU = -1;  /* MasterInformationBlock */
static int hf_rrc_rrc_SysInfoType1_PDU = -1;      /* SysInfoType1 */
static int hf_rrc_rrc_SysInfoType2_PDU = -1;      /* SysInfoType2 */
static int hf_rrc_rrc_SysInfoType3_PDU = -1;      /* SysInfoType3 */
static int hf_rrc_SysInfoType4_PDU = -1;          /* SysInfoType4 */
static int hf_rrc_SysInfoType5_PDU = -1;          /* SysInfoType5 */
static int hf_rrc_SysInfoType5bis_PDU = -1;       /* SysInfoType5bis */
static int hf_rrc_SysInfoType6_PDU = -1;          /* SysInfoType6 */
static int hf_rrc_rrc_SysInfoType7_PDU = -1;      /* SysInfoType7 */
static int hf_rrc_SysInfoType8_PDU = -1;          /* SysInfoType8 */
static int hf_rrc_SysInfoType9_PDU = -1;          /* SysInfoType9 */
static int hf_rrc_SysInfoType10_PDU = -1;         /* SysInfoType10 */
static int hf_rrc_SysInfoType11_PDU = -1;         /* SysInfoType11 */
static int hf_rrc_SysInfoType11bis_PDU = -1;      /* SysInfoType11bis */
static int hf_rrc_rrc_SysInfoType12_PDU = -1;     /* SysInfoType12 */
static int hf_rrc_SysInfoType13_PDU = -1;         /* SysInfoType13 */
static int hf_rrc_SysInfoType13_1_PDU = -1;       /* SysInfoType13_1 */
static int hf_rrc_SysInfoType13_2_PDU = -1;       /* SysInfoType13_2 */
static int hf_rrc_SysInfoType13_3_PDU = -1;       /* SysInfoType13_3 */
static int hf_rrc_SysInfoType13_4_PDU = -1;       /* SysInfoType13_4 */
static int hf_rrc_SysInfoType14_PDU = -1;         /* SysInfoType14 */
static int hf_rrc_SysInfoType15_PDU = -1;         /* SysInfoType15 */
static int hf_rrc_SysInfoType15bis_PDU = -1;      /* SysInfoType15bis */
static int hf_rrc_SysInfoType15_1_PDU = -1;       /* SysInfoType15_1 */
static int hf_rrc_SysInfoType15_1bis_PDU = -1;    /* SysInfoType15_1bis */
static int hf_rrc_SysInfoType15_2_PDU = -1;       /* SysInfoType15_2 */
static int hf_rrc_SysInfoType15_2bis_PDU = -1;    /* SysInfoType15_2bis */
static int hf_rrc_SysInfoType15_2ter_PDU = -1;    /* SysInfoType15_2ter */
static int hf_rrc_SysInfoType15_3_PDU = -1;       /* SysInfoType15_3 */
static int hf_rrc_SysInfoType15_3bis_PDU = -1;    /* SysInfoType15_3bis */
static int hf_rrc_SysInfoType15_4_PDU = -1;       /* SysInfoType15_4 */
static int hf_rrc_SysInfoType15_5_PDU = -1;       /* SysInfoType15_5 */
static int hf_rrc_SysInfoType15_6_PDU = -1;       /* SysInfoType15_6 */
static int hf_rrc_SysInfoType15_7_PDU = -1;       /* SysInfoType15_7 */
static int hf_rrc_SysInfoType15_8_PDU = -1;       /* SysInfoType15_8 */
static int hf_rrc_SysInfoType16_PDU = -1;         /* SysInfoType16 */
static int hf_rrc_SysInfoType17_PDU = -1;         /* SysInfoType17 */
static int hf_rrc_SysInfoType18_PDU = -1;         /* SysInfoType18 */
static int hf_rrc_SysInfoType19_PDU = -1;         /* SysInfoType19 */
static int hf_rrc_SysInfoType20_PDU = -1;         /* SysInfoType20 */
static int hf_rrc_SysInfoType21_PDU = -1;         /* SysInfoType21 */
static int hf_rrc_SysInfoType22_PDU = -1;         /* SysInfoType22 */
static int hf_rrc_SysInfoTypeSB1_PDU = -1;        /* SysInfoTypeSB1 */
static int hf_rrc_SysInfoTypeSB2_PDU = -1;        /* SysInfoTypeSB2 */
static int hf_rrc_rrc_ToTargetRNC_Container_PDU = -1;  /* ToTargetRNC_Container */
static int hf_rrc_rrc_TargetRNC_ToSourceRNC_Container_PDU = -1;  /* TargetRNC_ToSourceRNC_Container */
static int hf_rrc_CellUpdate_r3_add_ext_IEs_PDU = -1;  /* CellUpdate_r3_add_ext_IEs */
static int hf_rrc_CellUpdateConfirm_r7_add_ext_IEs_PDU = -1;  /* CellUpdateConfirm_r7_add_ext_IEs */
static int hf_rrc_InitialDirectTransfer_r3_add_ext_IEs_PDU = -1;  /* InitialDirectTransfer_r3_add_ext_IEs */
static int hf_rrc_InterRATHandoverInfo_r3_add_ext_IEs_PDU = -1;  /* InterRATHandoverInfo_r3_add_ext_IEs */
static int hf_rrc_RadioBearerSetup_r7_add_ext_IEs_PDU = -1;  /* RadioBearerSetup_r7_add_ext_IEs */
static int hf_rrc_RRCConnectionSetupComplete_r3_add_ext_IEs_PDU = -1;  /* RRCConnectionSetupComplete_r3_add_ext_IEs */
static int hf_rrc_UE_CapabilityContainer_IEs_PDU = -1;  /* UE_CapabilityContainer_IEs */
static int hf_rrc_UECapabilityInformation_r3_add_ext_IEs_PDU = -1;  /* UECapabilityInformation_r3_add_ext_IEs */
static int hf_rrc_URAUpdate_r3_add_ext_IEs_PDU = -1;  /* URAUpdate_r3_add_ext_IEs */
static int hf_rrc_SRNC_RelocationInfo_v3h0ext_IEs_PDU = -1;  /* SRNC_RelocationInfo_v3h0ext_IEs */
static int hf_rrc_SRNC_RelocationInfo_r6_add_ext_IEs_PDU = -1;  /* SRNC_RelocationInfo_r6_add_ext_IEs */
static int hf_rrc_SRNC_RelocationInfo_r7_add_ext_IEs_PDU = -1;  /* SRNC_RelocationInfo_r7_add_ext_IEs */
static int hf_rrc_MeasurementReport_PDU = -1;     /* MeasurementReport */
static int hf_rrc_integrityCheckInfo = -1;        /* IntegrityCheckInfo */
static int hf_rrc_message = -1;                   /* DL_DCCH_MessageType */
static int hf_rrc_activeSetUpdate = -1;           /* ActiveSetUpdate */
static int hf_rrc_assistanceDataDelivery = -1;    /* AssistanceDataDelivery */
static int hf_rrc_cellChangeOrderFromUTRAN = -1;  /* CellChangeOrderFromUTRAN */
static int hf_rrc_cellUpdateConfirm = -1;         /* CellUpdateConfirm */
static int hf_rrc_counterCheck = -1;              /* CounterCheck */
static int hf_rrc_downlinkDirectTransfer = -1;    /* DownlinkDirectTransfer */
static int hf_rrc_handoverFromUTRANCommand_GSM = -1;  /* HandoverFromUTRANCommand_GSM */
static int hf_rrc_handoverFromUTRANCommand_CDMA2000 = -1;  /* HandoverFromUTRANCommand_CDMA2000 */
static int hf_rrc_measurementControl = -1;        /* MeasurementControl */
static int hf_rrc_pagingType2 = -1;               /* PagingType2 */
static int hf_rrc_physicalChannelReconfiguration = -1;  /* PhysicalChannelReconfiguration */
static int hf_rrc_physicalSharedChannelAllocation = -1;  /* PhysicalSharedChannelAllocation */
static int hf_rrc_radioBearerReconfiguration = -1;  /* RadioBearerReconfiguration */
static int hf_rrc_radioBearerRelease = -1;        /* RadioBearerRelease */
static int hf_rrc_radioBearerSetup = -1;          /* RadioBearerSetup */
static int hf_rrc_rrcConnectionRelease = -1;      /* RRCConnectionRelease */
static int hf_rrc_securityModeCommand = -1;       /* SecurityModeCommand */
static int hf_rrc_signallingConnectionRelease = -1;  /* SignallingConnectionRelease */
static int hf_rrc_transportChannelReconfiguration = -1;  /* TransportChannelReconfiguration */
static int hf_rrc_transportFormatCombinationControl = -1;  /* TransportFormatCombinationControl */
static int hf_rrc_ueCapabilityEnquiry = -1;       /* UECapabilityEnquiry */
static int hf_rrc_ueCapabilityInformationConfirm = -1;  /* UECapabilityInformationConfirm */
static int hf_rrc_uplinkPhysicalChannelControl = -1;  /* UplinkPhysicalChannelControl */
static int hf_rrc_uraUpdateConfirm = -1;          /* URAUpdateConfirm */
static int hf_rrc_utranMobilityInformation = -1;  /* UTRANMobilityInformation */
static int hf_rrc_handoverFromUTRANCommand_GERANIu = -1;  /* HandoverFromUTRANCommand_GERANIu */
static int hf_rrc_mbmsModifiedServicesInformation = -1;  /* MBMSModifiedServicesInformation */
static int hf_rrc_etwsPrimaryNotificationWithSecurity = -1;  /* ETWSPrimaryNotificationWithSecurity */
static int hf_rrc_handoverFromUTRANCommand_EUTRA = -1;  /* HandoverFromUTRANCommand_EUTRA */
static int hf_rrc_ueInformationRequest = -1;      /* UEInformationRequest */
static int hf_rrc_loggingMeasurementConfiguration = -1;  /* LoggingMeasurementConfiguration */
static int hf_rrc_spare1 = -1;                    /* NULL */
static int hf_rrc_message_01 = -1;                /* UL_DCCH_MessageType */
static int hf_rrc_activeSetUpdateComplete = -1;   /* ActiveSetUpdateComplete */
static int hf_rrc_activeSetUpdateFailure = -1;    /* ActiveSetUpdateFailure */
static int hf_rrc_cellChangeOrderFromUTRANFailure = -1;  /* CellChangeOrderFromUTRANFailure */
static int hf_rrc_counterCheckResponse = -1;      /* CounterCheckResponse */
static int hf_rrc_handoverToUTRANComplete = -1;   /* HandoverToUTRANComplete */
static int hf_rrc_initialDirectTransfer = -1;     /* InitialDirectTransfer */
static int hf_rrc_handoverFromUTRANFailure = -1;  /* HandoverFromUTRANFailure */
static int hf_rrc_measurementControlFailure = -1;  /* MeasurementControlFailure */
static int hf_rrc_measurementReport = -1;         /* MeasurementReport */
static int hf_rrc_physicalChannelReconfigurationComplete = -1;  /* PhysicalChannelReconfigurationComplete */
static int hf_rrc_physicalChannelReconfigurationFailure = -1;  /* PhysicalChannelReconfigurationFailure */
static int hf_rrc_radioBearerReconfigurationComplete = -1;  /* RadioBearerReconfigurationComplete */
static int hf_rrc_radioBearerReconfigurationFailure = -1;  /* RadioBearerReconfigurationFailure */
static int hf_rrc_radioBearerReleaseComplete = -1;  /* RadioBearerReleaseComplete */
static int hf_rrc_radioBearerReleaseFailure = -1;  /* RadioBearerReleaseFailure */
static int hf_rrc_radioBearerSetupComplete = -1;  /* RadioBearerSetupComplete */
static int hf_rrc_radioBearerSetupFailure = -1;   /* RadioBearerSetupFailure */
static int hf_rrc_rrcConnectionReleaseComplete = -1;  /* RRCConnectionReleaseComplete */
static int hf_rrc_rrcConnectionSetupComplete = -1;  /* RRCConnectionSetupComplete */
static int hf_rrc_rrcStatus = -1;                 /* RRCStatus */
static int hf_rrc_securityModeComplete = -1;      /* SecurityModeComplete */
static int hf_rrc_securityModeFailure = -1;       /* SecurityModeFailure */
static int hf_rrc_signallingConnectionReleaseIndication = -1;  /* SignallingConnectionReleaseIndication */
static int hf_rrc_transportChannelReconfigurationComplete = -1;  /* TransportChannelReconfigurationComplete */
static int hf_rrc_transportChannelReconfigurationFailure = -1;  /* TransportChannelReconfigurationFailure */
static int hf_rrc_transportFormatCombinationControlFailure = -1;  /* TransportFormatCombinationControlFailure */
static int hf_rrc_ueCapabilityInformation = -1;   /* UECapabilityInformation */
static int hf_rrc_uplinkDirectTransfer = -1;      /* UplinkDirectTransfer */
static int hf_rrc_utranMobilityInformationConfirm = -1;  /* UTRANMobilityInformationConfirm */
static int hf_rrc_utranMobilityInformationFailure = -1;  /* UTRANMobilityInformationFailure */
static int hf_rrc_mbmsModificationRequest = -1;   /* MBMSModificationRequest */
static int hf_rrc_ul_DCCH_MessageType_ext = -1;   /* UL_DCCH_MessageType_ext */
static int hf_rrc_ueInformationResponse = -1;     /* UEInformationResponse */
static int hf_rrc_spare15 = -1;                   /* NULL */
static int hf_rrc_spare14 = -1;                   /* NULL */
static int hf_rrc_spare13 = -1;                   /* NULL */
static int hf_rrc_spare12 = -1;                   /* NULL */
static int hf_rrc_spare11 = -1;                   /* NULL */
static int hf_rrc_spare10 = -1;                   /* NULL */
static int hf_rrc_spare9 = -1;                    /* NULL */
static int hf_rrc_spare8 = -1;                    /* NULL */
static int hf_rrc_spare7 = -1;                    /* NULL */
static int hf_rrc_spare6 = -1;                    /* NULL */
static int hf_rrc_spare5 = -1;                    /* NULL */
static int hf_rrc_spare4 = -1;                    /* NULL */
static int hf_rrc_spare3 = -1;                    /* NULL */
static int hf_rrc_spare2 = -1;                    /* NULL */
static int hf_rrc_message_02 = -1;                /* DL_CCCH_MessageType */
static int hf_rrc_cellUpdateConfirm_01 = -1;      /* CellUpdateConfirm_CCCH */
static int hf_rrc_rrcConnectionReject = -1;       /* RRCConnectionReject */
static int hf_rrc_rrcConnectionRelease_01 = -1;   /* RRCConnectionRelease_CCCH */
static int hf_rrc_rrcConnectionSetup = -1;        /* RRCConnectionSetup */
static int hf_rrc_uraUpdateConfirm_01 = -1;       /* URAUpdateConfirm_CCCH */
static int hf_rrc_message_03 = -1;                /* UL_CCCH_MessageType */
static int hf_rrc_cellUpdate = -1;                /* CellUpdate */
static int hf_rrc_rrcConnectionRequest = -1;      /* RRCConnectionRequest */
static int hf_rrc_uraUpdate = -1;                 /* URAUpdate */
static int hf_rrc_spare = -1;                     /* NULL */
static int hf_rrc_message_04 = -1;                /* PCCH_MessageType */
static int hf_rrc_pagingType1 = -1;               /* PagingType1 */
static int hf_rrc_message_05 = -1;                /* DL_SHCCH_MessageType */
static int hf_rrc_message_06 = -1;                /* UL_SHCCH_MessageType */
static int hf_rrc_puschCapacityRequest = -1;      /* PUSCHCapacityRequest */
static int hf_rrc_message_07 = -1;                /* BCCH_FACH_MessageType */
static int hf_rrc_dummy = -1;                     /* SystemInformation_FACH */
static int hf_rrc_systemInformationChangeIndication = -1;  /* SystemInformationChangeIndication */
static int hf_rrc_message_08 = -1;                /* SystemInformation_BCH */
static int hf_rrc_message_09 = -1;                /* MCCH_MessageType */
static int hf_rrc_mbmsAccessInformation = -1;     /* MBMSAccessInformation */
static int hf_rrc_mbmsCommonPTMRBInformation = -1;  /* MBMSCommonPTMRBInformation */
static int hf_rrc_mbmsCurrentCellPTMRBInformation = -1;  /* MBMSCurrentCellPTMRBInformation */
static int hf_rrc_mbmsGeneralInformation = -1;    /* MBMSGeneralInformation */
static int hf_rrc_mbmsNeighbouringCellPTMRBInformation = -1;  /* MBMSNeighbouringCellPTMRBInformation */
static int hf_rrc_mbmsUnmodifiedServicesInformation = -1;  /* MBMSUnmodifiedServicesInformation */
static int hf_rrc_message_10 = -1;                /* MSCH_MessageType */
static int hf_rrc_mbmsSchedulingInformation = -1;  /* MBMSSchedulingInformation */
static int hf_rrc_r3 = -1;                        /* T_r3 */
static int hf_rrc_activeSetUpdate_r3 = -1;        /* ActiveSetUpdate_r3_IEs */
static int hf_rrc_laterNonCriticalExtensions = -1;  /* T_laterNonCriticalExtensions */
static int hf_rrc_activeSetUpdate_r3_add_ext = -1;  /* BIT_STRING */
static int hf_rrc_v4b0NonCriticalExtensions = -1;  /* T_v4b0NonCriticalExtensions */
static int hf_rrc_activeSetUpdate_v4b0ext = -1;   /* ActiveSetUpdate_v4b0ext_IEs */
static int hf_rrc_v590NonCriticalExtensions = -1;  /* T_v590NonCriticalExtensions */
static int hf_rrc_activeSetUpdate_v590ext = -1;   /* ActiveSetUpdate_v590ext_IEs */
static int hf_rrc_v690NonCriticalExtensions = -1;  /* T_v690NonCriticalExtensions */
static int hf_rrc_activeSetUpdate_v690ext = -1;   /* ActiveSetUpdate_v690ext_IEs */
static int hf_rrc_nonCriticalExtensions = -1;     /* T_nonCriticalExtensions */
static int hf_rrc_later_than_r3 = -1;             /* T_later_than_r3 */
static int hf_rrc_rrc_TransactionIdentifier = -1;  /* RRC_TransactionIdentifier */
static int hf_rrc_criticalExtensions = -1;        /* T_criticalExtensions */
static int hf_rrc_r6 = -1;                        /* T_r6 */
static int hf_rrc_activeSetUpdate_r6 = -1;        /* ActiveSetUpdate_r6_IEs */
static int hf_rrc_activeSetUpdate_r6_add_ext = -1;  /* BIT_STRING */
static int hf_rrc_v6b0NonCriticalExtensions = -1;  /* T_v6b0NonCriticalExtensions */
static int hf_rrc_activeSetUpdate_v6b0ext = -1;   /* ActiveSetUpdate_v6b0ext_IEs */
static int hf_rrc_nonCriticalExtensions_01 = -1;  /* T_nonCriticalExtensions_01 */
static int hf_rrc_criticalExtensions_01 = -1;     /* T_criticalExtensions_01 */
static int hf_rrc_r7 = -1;                        /* T_r7 */
static int hf_rrc_activeSetUpdate_r7 = -1;        /* ActiveSetUpdate_r7_IEs */
static int hf_rrc_activeSetUpdate_r7_add_ext = -1;  /* BIT_STRING */
static int hf_rrc_v780NonCriticalExtensions = -1;  /* T_v780NonCriticalExtensions */
static int hf_rrc_activeSetUpdate_v780ext = -1;   /* ActiveSetUpdate_v780ext_IEs */
static int hf_rrc_v7f0NonCriticalExtensions = -1;  /* T_v7f0NonCriticalExtensions */
static int hf_rrc_activeSetUdpate_v7f0ext = -1;   /* ActiveSetUpdate_v7f0ext_IEs */
static int hf_rrc_v7g0NonCriticalExtensions = -1;  /* T_v7g0NonCriticalExtensions */
static int hf_rrc_activeSetUdpate_v7g0ext = -1;   /* ActiveSetUpdate_v7g0ext_IEs */
static int hf_rrc_nonCriticalExtensions_02 = -1;  /* T_nonCriticalExtensions_02 */
static int hf_rrc_criticalExtensions_02 = -1;     /* T_criticalExtensions_02 */
static int hf_rrc_r8 = -1;                        /* T_r8 */
static int hf_rrc_activeSetUpdate_r8 = -1;        /* ActiveSetUpdate_r8_IEs */
static int hf_rrc_activeSetUpdate_r8_add_ext = -1;  /* BIT_STRING */
static int hf_rrc_v890NonCriticalExtensions = -1;  /* T_v890NonCriticalExtensions */
static int hf_rrc_activeSetUpdate_v890ext = -1;   /* ActiveSetUpdate_v890ext_IEs */
static int hf_rrc_v7g0NonCriticalExtensions_01 = -1;  /* T_v7g0NonCriticalExtensions_01 */
static int hf_rrc_activeSetUpdate_v7g0ext = -1;   /* ActiveSetUpdate_v7g0ext_IEs */
static int hf_rrc_nonCriticalExtensions_03 = -1;  /* T_nonCriticalExtensions_03 */
static int hf_rrc_criticalExtensions_03 = -1;     /* T_criticalExtensions_03 */
static int hf_rrc_r9 = -1;                        /* T_r9 */
static int hf_rrc_activeSetUpdate_r9 = -1;        /* ActiveSetUpdate_r9_IEs */
static int hf_rrc_activeSetUpdate_r9_add_ext = -1;  /* BIT_STRING */
static int hf_rrc_v950NonCriticalExtensions = -1;  /* T_v950NonCriticalExtensions */
static int hf_rrc_activeSetUpdate_v950ext = -1;   /* ActiveSetUpdate_v950ext_IEs */
static int hf_rrc_nonCriticalExtensions_04 = -1;  /* T_nonCriticalExtensions_04 */
static int hf_rrc_criticalExtensions_04 = -1;     /* T_criticalExtensions_04 */
static int hf_rrc_r10 = -1;                       /* T_r10 */
static int hf_rrc_activeSetUpdate_r10 = -1;       /* ActiveSetUpdate_r10_IEs */
static int hf_rrc_activeSetUpdate_r10_add_ext = -1;  /* BIT_STRING */
static int hf_rrc_vbxyNonCriticalExtensions = -1;  /* T_vbxyNonCriticalExtensions */
static int hf_rrc_activeSetUpdate_vbxyext = -1;   /* ActiveSetUpdate_vbxyext_IEs */
static int hf_rrc_nonCriticalExtensions_05 = -1;  /* T_nonCriticalExtensions_05 */
static int hf_rrc_criticalExtensions_05 = -1;     /* T_criticalExtensions_05 */
static int hf_rrc_r11 = -1;                       /* T_r11 */
static int hf_rrc_activeSetUpdate_r11 = -1;       /* ActiveSetUpdate_r11_IEs */
static int hf_rrc_activeSetUpdate_r11_add_ext = -1;  /* BIT_STRING */
static int hf_rrc_nonCriticalExtensions_06 = -1;  /* T_nonCriticalExtensions_06 */
static int hf_rrc_criticalExtensions_06 = -1;     /* T_criticalExtensions_06 */
static int hf_rrc_dummy_01 = -1;                  /* IntegrityProtectionModeInfo */
static int hf_rrc_dummy2 = -1;                    /* CipheringModeInfo */
static int hf_rrc_activationTime = -1;            /* ActivationTime */
static int hf_rrc_newU_RNTI = -1;                 /* U_RNTI */
static int hf_rrc_cn_InformationInfo = -1;        /* CN_InformationInfo */
static int hf_rrc_dummy3 = -1;                    /* DL_CounterSynchronisationInfo */
static int hf_rrc_maxAllowedUL_TX_Power = -1;     /* MaxAllowedUL_TX_Power */
static int hf_rrc_rl_AdditionInformationList = -1;  /* RL_AdditionInformationList */
static int hf_rrc_rl_RemovalInformationList = -1;  /* RL_RemovalInformationList */
static int hf_rrc_tx_DiversityMode = -1;          /* TX_DiversityMode */
static int hf_rrc_dummy4 = -1;                    /* SSDT_Information */
static int hf_rrc_dummy_02 = -1;                  /* SSDT_UL */
static int hf_rrc_cell_id_PerRL_List = -1;        /* CellIdentity_PerRL_List */
static int hf_rrc_dpc_Mode = -1;                  /* DPC_Mode */
static int hf_rrc_dl_TPC_PowerOffsetPerRL_List = -1;  /* DL_TPC_PowerOffsetPerRL_List */
static int hf_rrc_primary_plmn_Identity = -1;     /* PLMN_Identity */
static int hf_rrc_newH_RNTI = -1;                 /* H_RNTI */
static int hf_rrc_newPrimary_E_RNTI = -1;         /* E_RNTI */
static int hf_rrc_newSecondary_E_RNTI = -1;       /* E_RNTI */
static int hf_rrc_cn_InformationInfo_01 = -1;     /* CN_InformationInfo_r6 */
static int hf_rrc_rl_AdditionInformationList_01 = -1;  /* RL_AdditionInformationList_r6 */
static int hf_rrc_serving_HSDSCH_CellInformation = -1;  /* Serving_HSDSCH_CellInformation */
static int hf_rrc_e_dch_ReconfigurationInfo = -1;  /* E_DCH_ReconfigurationInfo */
static int hf_rrc_rl_AdditionInformation_list_v6b0ext = -1;  /* RL_AdditionInformation_list_v6b0ext */
static int hf_rrc_dtx_drx_TimingInfo = -1;        /* DTX_DRX_TimingInfo_r7 */
static int hf_rrc_dtx_drx_Info = -1;              /* DTX_DRX_Info_r7 */
static int hf_rrc_hs_scch_LessInfo = -1;          /* HS_SCCH_LessInfo_r7 */
static int hf_rrc_mimoParameters = -1;            /* MIMO_Parameters_r7 */
static int hf_rrc_rl_AdditionInformationList_02 = -1;  /* RL_AdditionInformationList_r7 */
static int hf_rrc_serving_HSDSCH_CellInformation_01 = -1;  /* Serving_HSDSCH_CellInformation_r7 */
static int hf_rrc_e_dch_ReconfigurationInfo_01 = -1;  /* E_DCH_ReconfigurationInfo_r7 */
static int hf_rrc_ul_16QAM_Config = -1;           /* UL_16QAM_Config */
static int hf_rrc_e_dch_ReconfInfoSameCell = -1;  /* E_DCH_RL_InfoSameServingCell */
static int hf_rrc_e_TFC_Boost_Info = -1;          /* E_TFC_Boost_Info_r7 */
static int hf_rrc_e_DPDCH_PowerInterpolation = -1;  /* E_DPDCH_PowerInterpolation */
static int hf_rrc_hs_DSCH_TBSizeTable = -1;       /* HS_DSCH_TBSizeTable */
static int hf_rrc_mimoParameters_01 = -1;         /* MIMO_Parameters_v7f0ext */
static int hf_rrc_mimoParameters_02 = -1;         /* MIMO_Parameters_v7g0ext */
static int hf_rrc_mimoParameters_03 = -1;         /* MIMO_Parameters_r8 */
static int hf_rrc_rl_AdditionInformationList_03 = -1;  /* RL_AdditionInformationList_r8 */
static int hf_rrc_servingCellChangeParameters = -1;  /* ServingCellChangeParameters */
static int hf_rrc_serving_HSDSCH_CellInformation_02 = -1;  /* Serving_HSDSCH_CellInformation_r8 */
static int hf_rrc_dl_SecondaryCellInfoFDD = -1;   /* DL_SecondaryCellInfoFDD */
static int hf_rrc_dl_SecondaryCellInfoFDD_v890ext = -1;  /* DL_SecondaryCellInfoFDD_v890ext */
static int hf_rrc_rl_AdditionInformationList_04 = -1;  /* RL_AdditionInformationList_v890ext */
static int hf_rrc_mimoParameters_04 = -1;         /* MIMO_Parameters_r9 */
static int hf_rrc_ul_SecondaryCellInfoFDD = -1;   /* UL_SecondaryCellInfoFDD */
static int hf_rrc_e_dch_ReconfigurationInfo_SecULFrequency = -1;  /* E_DCH_ReconfigurationInfo_SecULFrequency */
static int hf_rrc_rl_AdditionInformationList_05 = -1;  /* RL_AdditionInformationList_r9 */
static int hf_rrc_rl_AdditionInformationList_SecULFreq = -1;  /* RL_AdditionInformationList_SecULFreq */
static int hf_rrc_rl_RemovalInformationList_SecULFreq = -1;  /* RL_RemovalInformationList_SecULFreq */
static int hf_rrc_serving_HSDSCH_CellInformation_03 = -1;  /* Serving_HSDSCH_CellInformation_r9 */
static int hf_rrc_dl_SecondaryCellInfoFDD_01 = -1;  /* DL_SecondaryCellInfoFDD_r9 */
static int hf_rrc_secondaryCellMimoParameters = -1;  /* SecondaryCellMIMOparametersFDD_v950ext */
static int hf_rrc_rl_AdditionInformationList_06 = -1;  /* RL_AdditionInformationList_v950ext */
static int hf_rrc_rl_AdditionInformationList_07 = -1;  /* RL_AdditionInformationList_r10 */
static int hf_rrc_dl_SecondaryCellInfoFDD_02 = -1;  /* DL_SecondaryCellInfoFDD_r10 */
static int hf_rrc_additionalDLSecCellInfoListFDD = -1;  /* AdditionalDLSecCellInfoListFDD */
static int hf_rrc_ulOLTDInfoFDD = -1;             /* UL_OLTD_InfoFDD */
static int hf_rrc_rl_AdditionInformationList_08 = -1;  /* RL_AdditionInformationList_vbxyext */
static int hf_rrc_mimo4x4Parameters = -1;         /* MIMO4x4_Parameters */
static int hf_rrc_ul_CLTD_InfoFDD = -1;           /* UL_CLTD_InfoFDD */
static int hf_rrc_f_tpich_ReconfigurationInfo = -1;  /* F_TPICH_ReconfigurationInfo */
static int hf_rrc_ul_OLTD_InfoFDD = -1;           /* UL_OLTD_InfoFDD */
static int hf_rrc_rl_AdditionInformationList_09 = -1;  /* RL_AdditionInformationList_r11 */
static int hf_rrc_serving_HSDSCH_CellInformation_04 = -1;  /* Serving_HSDSCH_CellInformation_r11 */
static int hf_rrc_e_dch_ReconfigurationInfo_02 = -1;  /* E_DCH_ReconfigurationInfo_r11 */
static int hf_rrc_ul_64QAM_Config = -1;           /* UL_64QAM_Config */
static int hf_rrc_ul_MIMO_InfoFDD = -1;           /* UL_MIMO_InfoFDD */
static int hf_rrc_dl_SecondaryCellInfoFDD_03 = -1;  /* DL_SecondaryCellInfoFDD_r11 */
static int hf_rrc_additionalDLSecCellInfoListFDD_01 = -1;  /* AdditionalDLSecCellInfoListFDD_r11 */
static int hf_rrc_additionalDLSecCellInfoListFDD2 = -1;  /* AdditionalDLSecCellInfoListFDD2_r11 */
static int hf_rrc_dummy_03 = -1;                  /* IntegrityProtActivationInfo */
static int hf_rrc_dummy2_01 = -1;                 /* RB_ActivationTimeInfoList */
static int hf_rrc_dummy3_01 = -1;                 /* UL_CounterSynchronisationInfo */
static int hf_rrc_laterNonCriticalExtensions_01 = -1;  /* T_laterNonCriticalExtensions_01 */
static int hf_rrc_activeSetUpdateComplete_r3_add_ext = -1;  /* BIT_STRING */
static int hf_rrc_nonCriticalExtensions_07 = -1;  /* T_nonCriticalExtensions_07 */
static int hf_rrc_failureCause = -1;              /* FailureCauseWithProtErr */
static int hf_rrc_laterNonCriticalExtensions_02 = -1;  /* T_laterNonCriticalExtensions_02 */
static int hf_rrc_activeSetUpdateFailure_r3_add_ext = -1;  /* BIT_STRING */
static int hf_rrc_nonCriticalExtensions_08 = -1;  /* T_nonCriticalExtensions_08 */
static int hf_rrc_r3_01 = -1;                     /* T_r3_01 */
static int hf_rrc_assistanceDataDelivery_r3 = -1;  /* AssistanceDataDelivery_r3_IEs */
static int hf_rrc_v3a0NonCriticalExtensions = -1;  /* T_v3a0NonCriticalExtensions */
static int hf_rrc_assistanceDataDelivery_v3a0ext = -1;  /* AssistanceDataDelivery_v3a0ext */
static int hf_rrc_laterNonCriticalExtensions_03 = -1;  /* T_laterNonCriticalExtensions_03 */
static int hf_rrc_assistanceDataDelivery_r3_add_ext = -1;  /* BIT_STRING */
static int hf_rrc_v4b0NonCriticalExtensions_01 = -1;  /* T_v4b0NonCriticalExtensions_01 */
static int hf_rrc_assistanceDataDelivery_v4b0ext = -1;  /* AssistanceDataDelivery_v4b0ext_IEs */
static int hf_rrc_v770NonCriticalExtension = -1;  /* T_v770NonCriticalExtension */
static int hf_rrc_assistanceDataDelivery_v770ext = -1;  /* AssistanceDataDelivery_v770ext_IEs */
static int hf_rrc_v860NonCriticalExtension = -1;  /* T_v860NonCriticalExtension */
static int hf_rrc_assistanceDataDelivery_v860ext = -1;  /* AssistanceDataDelivery_v860ext_IEs */
static int hf_rrc_v920NonCriticalExtension = -1;  /* T_v920NonCriticalExtension */
static int hf_rrc_assistanceDataDelivery_v920ext = -1;  /* AssistanceDataDelivery_v920ext_IEs */
static int hf_rrc_va40NonCriticalExtension = -1;  /* T_va40NonCriticalExtension */
static int hf_rrc_assistanceDataDelivery_va40ext = -1;  /* AssistanceDataDelivery_va40ext_IEs */
static int hf_rrc_nonCriticalExtensions_09 = -1;  /* T_nonCriticalExtensions_09 */
static int hf_rrc_later_than_r3_01 = -1;          /* T_later_than_r3_01 */
static int hf_rrc_criticalExtensions_07 = -1;     /* T_criticalExtensions_07 */
static int hf_rrc_ue_positioning_GPS_AssistanceData = -1;  /* UE_Positioning_GPS_AssistanceData */
static int hf_rrc_ue_positioning_OTDOA_AssistanceData_UEB = -1;  /* UE_Positioning_OTDOA_AssistanceData_UEB */
static int hf_rrc_sfn_Offset_Validity = -1;       /* SFN_Offset_Validity */
static int hf_rrc_ue_Positioning_OTDOA_AssistanceData_r4ext = -1;  /* UE_Positioning_OTDOA_AssistanceData_r4ext */
static int hf_rrc_ue_Positioning_OTDOA_AssistanceData_UEB_ext = -1;  /* UE_Positioning_OTDOA_AssistanceData_UEB_ext */
static int hf_rrc_ue_Positioning_GPS_AssistanceData = -1;  /* UE_Positioning_GPS_AssistanceData_v770ext */
static int hf_rrc_ue_positioning_GANSS_AssistanceData = -1;  /* UE_Positioning_GANSS_AssistanceData */
static int hf_rrc_ue_positioning_GANSS_AssistanceData_v860ext = -1;  /* UE_Positioning_GANSS_AssistanceData_v860ext */
static int hf_rrc_ue_positioning_GPS_AssistanceData_v920ext = -1;  /* UE_Positioning_GPS_AssistanceData_v920ext */
static int hf_rrc_ue_positioning_GANSS_AssistanceData_v920ext = -1;  /* UE_Positioning_GANSS_AssistanceData_v920ext */
static int hf_rrc_ue_positioning_GPS_AssistanceData_va40ext = -1;  /* UE_Positioning_GPS_AssistanceData_va40ext */
static int hf_rrc_ue_positioning_GANSS_AssistanceData_va40ext = -1;  /* UE_Positioning_GANSS_AssistanceData_va40ext */
static int hf_rrc_r3_02 = -1;                     /* T_r3_02 */
static int hf_rrc_cellChangeOrderFromUTRAN_IEs = -1;  /* CellChangeOrderFromUTRAN_r3_IEs */
static int hf_rrc_laterNonCriticalExtensions_04 = -1;  /* T_laterNonCriticalExtensions_04 */
static int hf_rrc_cellChangeOrderFromUTRAN_r3_add_ext = -1;  /* BIT_STRING */
static int hf_rrc_v590NonCriticalExtensions_01 = -1;  /* T_v590NonCriticalExtensions_01 */
static int hf_rrc_cellChangeOrderFromUTRAN_v590ext = -1;  /* CellChangeOrderFromUTRAN_v590ext_IEs */
static int hf_rrc_nonCriticalExtensions_10 = -1;  /* T_nonCriticalExtensions_10 */
static int hf_rrc_later_than_r3_02 = -1;          /* T_later_than_r3_02 */
static int hf_rrc_criticalExtensions_08 = -1;     /* T_criticalExtensions_08 */
static int hf_rrc_rab_InformationList = -1;       /* RAB_InformationList */
static int hf_rrc_interRAT_TargetCellDescription = -1;  /* InterRAT_TargetCellDescription */
static int hf_rrc_geran_SystemInfoType = -1;      /* T_geran_SystemInfoType */
static int hf_rrc_sI = -1;                        /* GERAN_SystemInformation */
static int hf_rrc_pSI = -1;                       /* GERAN_SystemInformation */
static int hf_rrc_r3_03 = -1;                     /* T_r3_03 */
static int hf_rrc_cellChangeOrderFromUTRANFailure_r3 = -1;  /* CellChangeOrderFromUTRANFailure_r3_IEs */
static int hf_rrc_laterNonCriticalExtensions_05 = -1;  /* T_laterNonCriticalExtensions_05 */
static int hf_rrc_cellChangeOrderFromUTRANFailure_r3_add_ext = -1;  /* BIT_STRING */
static int hf_rrc_nonCriticalExtensions_11 = -1;  /* T_nonCriticalExtensions_11 */
static int hf_rrc_dummy_04 = -1;                  /* T_dummy */
static int hf_rrc_criticalExtensions_09 = -1;     /* T_criticalExtensions_09 */
static int hf_rrc_interRAT_ChangeFailureCause = -1;  /* InterRAT_ChangeFailureCause */
static int hf_rrc_u_RNTI = -1;                    /* U_RNTI */
static int hf_rrc_startList = -1;                 /* STARTList */
static int hf_rrc_am_RLC_ErrorIndicationRb2_3or4 = -1;  /* BOOLEAN */
static int hf_rrc_am_RLC_ErrorIndicationRb5orAbove = -1;  /* BOOLEAN */
static int hf_rrc_cellUpdateCause = -1;           /* CellUpdateCause */
static int hf_rrc_failureCause_01 = -1;           /* FailureCauseWithProtErrTrId */
static int hf_rrc_rb_timer_indicator = -1;        /* Rb_timer_indicator */
static int hf_rrc_measuredResultsOnRACH = -1;     /* MeasuredResultsOnRACH */
static int hf_rrc_laterNonCriticalExtensions_06 = -1;  /* T_laterNonCriticalExtensions_06 */
static int hf_rrc_cellUpdate_r3_add_ext = -1;     /* T_cellUpdate_r3_add_ext */
static int hf_rrc_v590NonCriticalExtensions_02 = -1;  /* T_v590NonCriticalExtensions_02 */
static int hf_rrc_cellUpdate_v590ext = -1;        /* CellUpdate_v590ext */
static int hf_rrc_v690NonCriticalExtensions_01 = -1;  /* T_v690NonCriticalExtensions_01 */
static int hf_rrc_cellUpdate_v690ext = -1;        /* CellUpdate_v690ext_IEs */
static int hf_rrc_v6b0NonCriticalExtensions_01 = -1;  /* T_v6b0NonCriticalExtensions_01 */
static int hf_rrc_cellUpdate_v6b0ext = -1;        /* CellUpdate_v6b0ext_IEs */
static int hf_rrc_v770NonCriticalExtensions = -1;  /* T_v770NonCriticalExtensions */
static int hf_rrc_cellUpdate_v770ext = -1;        /* CellUpdate_v770ext_IEs */
static int hf_rrc_v860NonCriticalExtensions = -1;  /* T_v860NonCriticalExtensions */
static int hf_rrc_cellUpdate_v860ext = -1;        /* CellUpdate_v860ext_IEs */
static int hf_rrc_va40NonCriticalExtensions = -1;  /* T_va40NonCriticalExtensions */
static int hf_rrc_cellUpdate_va40ext = -1;        /* CellUpdate_va40ext_IEs */
static int hf_rrc_vbxyNonCriticalExtensions_01 = -1;  /* T_vbxyNonCriticalExtensions_01 */
static int hf_rrc_cellUpdate_vbxyext = -1;        /* CellUpdate_vbxyext_IEs */
static int hf_rrc_nonCriticalExtensions_12 = -1;  /* T_nonCriticalExtensions_12 */
static int hf_rrc_cellUpdate_v7e0ext = -1;        /* CellUpdate_v7e0ext_IEs */
static int hf_rrc_v7g0NonCriticalExtensions_02 = -1;  /* T_v7g0NonCriticalExtensions_02 */
static int hf_rrc_cellUpdate_v7g0ext = -1;        /* CellUpdate_v7g0ext_IEs */
static int hf_rrc_nonCriticalExtensions_13 = -1;  /* T_nonCriticalExtensions_13 */
static int hf_rrc_establishmentCause = -1;        /* EstablishmentCause */
static int hf_rrc_cellUpdateCause_ext = -1;       /* CellUpdateCause_ext */
static int hf_rrc_trafficVolumeIndicator = -1;    /* T_trafficVolumeIndicator */
static int hf_rrc_measuredResultsOnRACHinterFreq = -1;  /* MeasuredResultsOnRACHinterFreq */
static int hf_rrc_reconfigurationStatusIndicator = -1;  /* T_reconfigurationStatusIndicator */
static int hf_rrc_mbmsSelectedServices = -1;      /* MBMS_SelectedServicesShort */
static int hf_rrc_csCallType = -1;                /* T_csCallType */
static int hf_rrc_hspdschReception_CellFach = -1;  /* T_hspdschReception_CellFach */
static int hf_rrc_ueMobilityStateIndicator = -1;  /* High_MobilityDetected */
static int hf_rrc_capabilityChangeIndicator = -1;  /* T_capabilityChangeIndicator */
static int hf_rrc_supportForTwoDRXSchemesInPCH = -1;  /* T_supportForTwoDRXSchemesInPCH */
static int hf_rrc_measuredResultsOnRACH_v7g0ext = -1;  /* MeasuredResultsOnRACH_v7g0ext */
static int hf_rrc_supportOfCommonEDCH = -1;       /* T_supportOfCommonEDCH */
static int hf_rrc_supportOfHS_DSCHDRXOperation = -1;  /* T_supportOfHS_DSCHDRXOperation */
static int hf_rrc_supportOfMACiis = -1;           /* T_supportOfMACiis */
static int hf_rrc_supportOfSPSOperation = -1;     /* T_supportOfSPSOperation */
static int hf_rrc_supportOfControlChannelDRXOperation = -1;  /* T_supportOfControlChannelDRXOperation */
static int hf_rrc_securityRevertStatusIndicator = -1;  /* T_securityRevertStatusIndicator */
static int hf_rrc_loggedMeasAvailable = -1;       /* T_loggedMeasAvailable */
static int hf_rrc_loggedANRResultsAvailable = -1;  /* T_loggedANRResultsAvailable */
static int hf_rrc_measuredResultsOnRACHeutraFreq = -1;  /* MeasuredResultsOnRACHeutraFreq */
static int hf_rrc_r3_04 = -1;                     /* T_r3_04 */
static int hf_rrc_cellUpdateConfirm_r3 = -1;      /* CellUpdateConfirm_r3_IEs */
static int hf_rrc_v3a0NonCriticalExtensions_01 = -1;  /* T_v3a0NonCriticalExtensions_01 */
static int hf_rrc_cellUpdateConfirm_v3a0ext = -1;  /* CellUpdateConfirm_v3a0ext */
static int hf_rrc_laterNonCriticalExtensions_07 = -1;  /* T_laterNonCriticalExtensions_07 */
static int hf_rrc_cellUpdateConfirm_r3_add_ext = -1;  /* BIT_STRING */
static int hf_rrc_v4b0NonCriticalExtensions_02 = -1;  /* T_v4b0NonCriticalExtensions_02 */
static int hf_rrc_cellUpdateConfirm_v4b0ext = -1;  /* CellUpdateConfirm_v4b0ext_IEs */
static int hf_rrc_v590NonCriticalExtenstions = -1;  /* T_v590NonCriticalExtenstions */
static int hf_rrc_cellUpdateConfirm_v590ext = -1;  /* CellUpdateConfirm_v590ext_IEs */
static int hf_rrc_v5d0NonCriticalExtenstions = -1;  /* T_v5d0NonCriticalExtenstions */
static int hf_rrc_cellUpdateConfirm_v5d0ext = -1;  /* CellUpdateConfirm_v5d0ext_IEs */
static int hf_rrc_v690NonCriticalExtensions_02 = -1;  /* T_v690NonCriticalExtensions_02 */
static int hf_rrc_cellUpdateConfirm_v690ext = -1;  /* CellUpdateConfirm_v690ext_IEs */
static int hf_rrc_nonCriticalExtensions_14 = -1;  /* T_nonCriticalExtensions_14 */
static int hf_rrc_later_than_r3_03 = -1;          /* T_later_than_r3_03 */
static int hf_rrc_criticalExtensions_10 = -1;     /* T_criticalExtensions_10 */
static int hf_rrc_r4 = -1;                        /* T_r4 */
static int hf_rrc_cellUpdateConfirm_r4 = -1;      /* CellUpdateConfirm_r4_IEs */
static int hf_rrc_v4d0NonCriticalExtensions = -1;  /* T_v4d0NonCriticalExtensions */
static int hf_rrc_cellUpdateConfirm_r4_add_ext = -1;  /* BIT_STRING */
static int hf_rrc_v590NonCriticalExtenstions_01 = -1;  /* T_v590NonCriticalExtenstions_01 */
static int hf_rrc_v5d0NonCriticalExtenstions_01 = -1;  /* T_v5d0NonCriticalExtenstions_01 */
static int hf_rrc_v690NonCriticalExtensions_03 = -1;  /* T_v690NonCriticalExtensions_03 */
static int hf_rrc_nonCriticalExtensions_15 = -1;  /* T_nonCriticalExtensions_15 */
static int hf_rrc_criticalExtensions_11 = -1;     /* T_criticalExtensions_11 */
static int hf_rrc_r5 = -1;                        /* T_r5 */
static int hf_rrc_cellUpdateConfirm_r5 = -1;      /* CellUpdateConfirm_r5_IEs */
static int hf_rrc_cellUpdateConfirm_r5_add_ext = -1;  /* BIT_STRING */
static int hf_rrc_v5d0NonCriticalExtenstions_02 = -1;  /* T_v5d0NonCriticalExtenstions_02 */
static int hf_rrc_v690NonCriticalExtensions_04 = -1;  /* T_v690NonCriticalExtensions_04 */
static int hf_rrc_nonCriticalExtensions_16 = -1;  /* T_nonCriticalExtensions_16 */
static int hf_rrc_criticalExtensions_12 = -1;     /* T_criticalExtensions_12 */
static int hf_rrc_r6_01 = -1;                     /* T_r6_01 */
static int hf_rrc_cellUpdateConfirm_r6 = -1;      /* CellUpdateConfirm_r6_IEs */
static int hf_rrc_cellUpdateConfirm_r6_add_ext = -1;  /* BIT_STRING */
static int hf_rrc_v6b0NonCriticalExtensions_02 = -1;  /* T_v6b0NonCriticalExtensions_02 */
static int hf_rrc_cellUpdateConfirm_v6b0ext = -1;  /* CellUpdateConfirm_v6b0ext_IEs */
static int hf_rrc_nonCriticalExtensions_17 = -1;  /* T_nonCriticalExtensions_17 */
static int hf_rrc_criticalExtensions_13 = -1;     /* T_criticalExtensions_13 */
static int hf_rrc_r7_01 = -1;                     /* T_r7_01 */
static int hf_rrc_cellUpdateConfirm_r7 = -1;      /* CellUpdateConfirm_r7_IEs */
static int hf_rrc_cellUpdateConfirm_r7_add_ext = -1;  /* T_cellUpdateConfirm_r7_add_ext */
static int hf_rrc_v780NonCriticalExtensions_01 = -1;  /* T_v780NonCriticalExtensions_01 */
static int hf_rrc_cellUpdateConfirm_v780ext = -1;  /* CellUpdateConfirm_v780ext_IEs */
static int hf_rrc_v860NonCriticalExtensions_01 = -1;  /* T_v860NonCriticalExtensions_01 */
static int hf_rrc_cellUpdateConfirm_v860ext = -1;  /* CellUpdateConfirm_v860ext_IEs */
static int hf_rrc_nonCriticalExtensions_18 = -1;  /* T_nonCriticalExtensions_18 */
static int hf_rrc_criticalExtensions_14 = -1;     /* T_criticalExtensions_14 */
static int hf_rrc_r8_01 = -1;                     /* T_r8_01 */
static int hf_rrc_cellUpdateConfirm_r8 = -1;      /* CellUpdateConfirm_r8_IEs */
static int hf_rrc_cellUpdateConfirm_r8_add_ext = -1;  /* BIT_STRING */
static int hf_rrc_v7d0NonCriticalExtensions = -1;  /* T_v7d0NonCriticalExtensions */
static int hf_rrc_cellUpdateConfirm_v7d0ext = -1;  /* CellUpdateConfirm_v7d0ext_IEs */
static int hf_rrc_v7f0NonCriticalExtensions_01 = -1;  /* T_v7f0NonCriticalExtensions_01 */
static int hf_rrc_cellUpdateConfirm_v7f0ext = -1;  /* CellUpdateConfirm_v7f0ext_IEs */
static int hf_rrc_v890NonCriticalExtensions_01 = -1;  /* T_v890NonCriticalExtensions_01 */
static int hf_rrc_cellUpdateConfirm_v890ext = -1;  /* CellUpdateConfirm_v890ext_IEs */
static int hf_rrc_v7g0NonCriticalExtensions_03 = -1;  /* T_v7g0NonCriticalExtensions_03 */
static int hf_rrc_cellUpdateConfirm_v7g0ext = -1;  /* CellUpdateConfirm_v7g0ext_IEs */
static int hf_rrc_v8a0NonCriticalExtensions = -1;  /* T_v8a0NonCriticalExtensions */
static int hf_rrc_cellUpdateConfirm_v8a0ext = -1;  /* CellUpdateConfirm_v8a0ext_IEs */
static int hf_rrc_nonCriticalExtensions_19 = -1;  /* T_nonCriticalExtensions_19 */
static int hf_rrc_criticalExtensions_15 = -1;     /* T_criticalExtensions_15 */
static int hf_rrc_r9_01 = -1;                     /* T_r9_01 */
static int hf_rrc_cellUpdateConfirm_r9 = -1;      /* CellUpdateConfirm_r9_IEs */
static int hf_rrc_cellUpdateConfirm_r9_add_ext = -1;  /* BIT_STRING */
static int hf_rrc_v950NonCriticalExtensions_01 = -1;  /* T_v950NonCriticalExtensions_01 */
static int hf_rrc_cellUpdateConfirm_v950ext = -1;  /* CellUpdateConfirm_v950ext_IEs */
static int hf_rrc_v9c0NonCriticalExtensions = -1;  /* T_v9c0NonCriticalExtensions */
static int hf_rrc_cellUpdateConfirm_v9c0ext = -1;  /* CellUpdateConfirm_v9c0ext_IEs */
static int hf_rrc_nonCriticalExtensions_20 = -1;  /* T_nonCriticalExtensions_20 */
static int hf_rrc_criticalExtensions_16 = -1;     /* T_criticalExtensions_16 */
static int hf_rrc_r10_01 = -1;                    /* T_r10_01 */
static int hf_rrc_cellUpdateConfirm_r10 = -1;     /* CellUpdateConfirm_r10_IEs */
static int hf_rrc_cellUpdateConfirm_r10_add_ext = -1;  /* BIT_STRING */
static int hf_rrc_v9c0NonCriticalExtensions_01 = -1;  /* T_v9c0NonCriticalExtensions_01 */
static int hf_rrc_vbxyNonCriticalExtensions_02 = -1;  /* T_vbxyNonCriticalExtensions_02 */
static int hf_rrc_cellUpdateConfirm_vbxyext = -1;  /* CellUpdateConfirm_vbxyext_IEs */
static int hf_rrc_nonCriticalExtensions_21 = -1;  /* T_nonCriticalExtensions_21 */
static int hf_rrc_criticalExtensions_17 = -1;     /* T_criticalExtensions_17 */
static int hf_rrc_r11_01 = -1;                    /* T_r11_01 */
static int hf_rrc_cellUpdateConfirm_r11 = -1;     /* CellUpdateConfirm_r11_IEs */
static int hf_rrc_cellUpdateConfirm_r11_add_ext = -1;  /* BIT_STRING */
static int hf_rrc_nonCriticalExtensions_22 = -1;  /* T_nonCriticalExtensions_22 */
static int hf_rrc_criticalExtensions_18 = -1;     /* T_criticalExtensions_18 */
static int hf_rrc_integrityProtectionModeInfo = -1;  /* IntegrityProtectionModeInfo */
static int hf_rrc_cipheringModeInfo = -1;         /* CipheringModeInfo */
static int hf_rrc_new_U_RNTI = -1;                /* U_RNTI */
static int hf_rrc_new_C_RNTI = -1;                /* C_RNTI */
static int hf_rrc_rrc_StateIndicator = -1;        /* RRC_StateIndicator */
static int hf_rrc_utran_DRX_CycleLengthCoeff = -1;  /* UTRAN_DRX_CycleLengthCoefficient */
static int hf_rrc_rlc_Re_establishIndicatorRb2_3or4 = -1;  /* BOOLEAN */
static int hf_rrc_rlc_Re_establishIndicatorRb5orAbove = -1;  /* BOOLEAN */
static int hf_rrc_ura_Identity = -1;              /* URA_Identity */
static int hf_rrc_rb_InformationReleaseList = -1;  /* RB_InformationReleaseList */
static int hf_rrc_rb_InformationReconfigList = -1;  /* RB_InformationReconfigList */
static int hf_rrc_rb_InformationAffectedList = -1;  /* RB_InformationAffectedList */
static int hf_rrc_dl_CounterSynchronisationInfo = -1;  /* DL_CounterSynchronisationInfo */
static int hf_rrc_ul_CommonTransChInfo = -1;      /* UL_CommonTransChInfo */
static int hf_rrc_ul_deletedTransChInfoList = -1;  /* UL_DeletedTransChInfoList */
static int hf_rrc_ul_AddReconfTransChInfoList = -1;  /* UL_AddReconfTransChInfoList */
static int hf_rrc_modeSpecificTransChInfo = -1;   /* T_modeSpecificTransChInfo */
static int hf_rrc_fdd = -1;                       /* T_fdd */
static int hf_rrc_dummy_05 = -1;                  /* CPCH_SetID */
static int hf_rrc_dummy2_02 = -1;                 /* DRAC_StaticInformationList */
static int hf_rrc_tdd = -1;                       /* NULL */
static int hf_rrc_dl_CommonTransChInfo = -1;      /* DL_CommonTransChInfo */
static int hf_rrc_dl_DeletedTransChInfoList = -1;  /* DL_DeletedTransChInfoList */
static int hf_rrc_dl_AddReconfTransChInfoList = -1;  /* DL_AddReconfTransChInfoList */
static int hf_rrc_frequencyInfo = -1;             /* FrequencyInfo */
static int hf_rrc_ul_ChannelRequirement = -1;     /* UL_ChannelRequirement */
static int hf_rrc_modeSpecificPhysChInfo = -1;    /* T_modeSpecificPhysChInfo */
static int hf_rrc_fdd_01 = -1;                    /* T_fdd_01 */
static int hf_rrc_dummy_06 = -1;                  /* DL_PDSCH_Information */
static int hf_rrc_dl_CommonInformation = -1;      /* DL_CommonInformation */
static int hf_rrc_dl_InformationPerRL_List = -1;  /* DL_InformationPerRL_List */
static int hf_rrc_new_DSCH_RNTI = -1;             /* DSCH_RNTI */
static int hf_rrc_pdcp_ROHC_TargetMode = -1;      /* PDCP_ROHC_TargetMode */
static int hf_rrc_rb_InformationReconfigList_01 = -1;  /* RB_InformationReconfigList_r4 */
static int hf_rrc_ul_CommonTransChInfo_01 = -1;   /* UL_CommonTransChInfo_r4 */
static int hf_rrc_modeSpecificTransChInfo_01 = -1;  /* T_modeSpecificTransChInfo_01 */
static int hf_rrc_fdd_02 = -1;                    /* T_fdd_02 */
static int hf_rrc_dl_CommonTransChInfo_01 = -1;   /* DL_CommonTransChInfo_r4 */
static int hf_rrc_dl_AddReconfTransChInfoList_01 = -1;  /* DL_AddReconfTransChInfoList_r4 */
static int hf_rrc_ul_ChannelRequirement_01 = -1;  /* UL_ChannelRequirement_r4 */
static int hf_rrc_modeSpecificPhysChInfo_01 = -1;  /* T_modeSpecificPhysChInfo_01 */
static int hf_rrc_fdd_03 = -1;                    /* T_fdd_03 */
static int hf_rrc_dl_CommonInformation_01 = -1;   /* DL_CommonInformation_r4 */
static int hf_rrc_dl_InformationPerRL_List_01 = -1;  /* DL_InformationPerRL_List_r4 */
static int hf_rrc_new_H_RNTI = -1;                /* H_RNTI */
static int hf_rrc_rb_InformationReconfigList_02 = -1;  /* RB_InformationReconfigList_r5 */
static int hf_rrc_rb_InformationAffectedList_01 = -1;  /* RB_InformationAffectedList_r5 */
static int hf_rrc_dl_CounterSynchronisationInfo_01 = -1;  /* DL_CounterSynchronisationInfo_r5 */
static int hf_rrc_modeSpecificTransChInfo_02 = -1;  /* T_modeSpecificTransChInfo_02 */
static int hf_rrc_fdd_04 = -1;                    /* T_fdd_04 */
static int hf_rrc_dl_DeletedTransChInfoList_01 = -1;  /* DL_DeletedTransChInfoList_r5 */
static int hf_rrc_dl_AddReconfTransChInfoList_02 = -1;  /* DL_AddReconfTransChInfoList_r5 */
static int hf_rrc_ul_ChannelRequirement_02 = -1;  /* UL_ChannelRequirement_r5 */
static int hf_rrc_modeSpecificPhysChInfo_02 = -1;  /* T_modeSpecificPhysChInfo_02 */
static int hf_rrc_fdd_05 = -1;                    /* T_fdd_05 */
static int hf_rrc_dl_HSPDSCH_Information = -1;    /* DL_HSPDSCH_Information */
static int hf_rrc_dl_CommonInformation_02 = -1;   /* DL_CommonInformation_r5 */
static int hf_rrc_dl_InformationPerRL_List_02 = -1;  /* DL_InformationPerRL_List_r5 */
static int hf_rrc_rb_InformationReconfigList_03 = -1;  /* RB_InformationReconfigList_r6 */
static int hf_rrc_rb_InformationAffectedList_02 = -1;  /* RB_InformationAffectedList_r6 */
static int hf_rrc_ul_deletedTransChInfoList_01 = -1;  /* UL_DeletedTransChInfoList_r6 */
static int hf_rrc_ul_AddReconfTransChInfoList_01 = -1;  /* UL_AddReconfTransChInfoList_r6 */
static int hf_rrc_ul_DPCH_Info = -1;              /* UL_DPCH_Info_r6 */
static int hf_rrc_ul_EDCH_Information = -1;       /* UL_EDCH_Information_r6 */
static int hf_rrc_dl_HSPDSCH_Information_01 = -1;  /* DL_HSPDSCH_Information_r6 */
static int hf_rrc_dl_CommonInformation_03 = -1;   /* DL_CommonInformation_r6 */
static int hf_rrc_dl_InformationPerRL_List_03 = -1;  /* DL_InformationPerRL_List_r6 */
static int hf_rrc_mbms_PL_ServiceRestrictInfo = -1;  /* MBMS_PL_ServiceRestrictInfo_r6 */
static int hf_rrc_harq_Preamble_Mode = -1;        /* HARQ_Preamble_Mode */
static int hf_rrc_beaconPLEst = -1;               /* BEACON_PL_Est */
static int hf_rrc_postVerificationPeriod = -1;    /* T_postVerificationPeriod */
static int hf_rrc_dhs_sync = -1;                  /* DHS_Sync */
static int hf_rrc_dl_InformationPerRL_List_v6b0ext = -1;  /* DL_InformationPerRL_List_v6b0ext */
static int hf_rrc_integrityProtectionModeInfo_01 = -1;  /* IntegrityProtectionModeInfo_r7 */
static int hf_rrc_cipheringModeInfo_01 = -1;      /* CipheringModeInfo_r7 */
static int hf_rrc_utran_DRX_CycleLengthCoeff_01 = -1;  /* UTRAN_DRX_CycleLengthCoefficient_r7 */
static int hf_rrc_waitTime = -1;                  /* WaitTime */
static int hf_rrc_supportForChangeOfUE_Capability = -1;  /* BOOLEAN */
static int hf_rrc_rb_InformationReconfigList_04 = -1;  /* RB_InformationReconfigList_r7 */
static int hf_rrc_rb_InformationAffectedList_03 = -1;  /* RB_InformationAffectedList_r7 */
static int hf_rrc_ul_AddReconfTransChInfoList_02 = -1;  /* UL_AddReconfTransChInfoList_r7 */
static int hf_rrc_dl_DeletedTransChInfoList_02 = -1;  /* DL_DeletedTransChInfoList_r7 */
static int hf_rrc_dl_AddReconfTransChInfoList_03 = -1;  /* DL_AddReconfTransChInfoList_r7 */
static int hf_rrc_multi_frequencyInfo = -1;       /* Multi_frequencyInfo_LCR_r7 */
static int hf_rrc_ul_DPCH_Info_01 = -1;           /* UL_DPCH_Info_r7 */
static int hf_rrc_ul_EDCH_Information_01 = -1;    /* UL_EDCH_Information_r7 */
static int hf_rrc_dl_HSPDSCH_Information_02 = -1;  /* DL_HSPDSCH_Information_r7 */
static int hf_rrc_dl_CommonInformation_04 = -1;   /* DL_CommonInformation_r7 */
static int hf_rrc_dl_InformationPerRL_List_04 = -1;  /* DL_InformationPerRL_List_r7 */
static int hf_rrc_v7f0NonCriticalExtensions_02 = -1;  /* T_v7f0NonCriticalExtensions_02 */
static int hf_rrc_v7g0NonCriticalExtensions_04 = -1;  /* T_v7g0NonCriticalExtensions_04 */
static int hf_rrc_nonCriticalExtensions_23 = -1;  /* T_nonCriticalExtensions_23 */
static int hf_rrc_ul_EDCH_Information_02 = -1;    /* UL_EDCH_Information_ext */
static int hf_rrc_dummy_07 = -1;                  /* DefaultConfigForCellFACH */
static int hf_rrc_rab_InformationSetup = -1;      /* RAB_InformationSetup_r8 */
static int hf_rrc_rb_InformationReconfigList_05 = -1;  /* RB_InformationReconfigList_r8 */
static int hf_rrc_rb_InformationAffectedList_04 = -1;  /* RB_InformationAffectedList_r8 */
static int hf_rrc_ul_AddReconfTransChInfoList_03 = -1;  /* UL_AddReconfTransChInfoList_r8 */
static int hf_rrc_ul_EDCH_Information_03 = -1;    /* UL_EDCH_Information_r8 */
static int hf_rrc_dl_HSPDSCH_Information_03 = -1;  /* DL_HSPDSCH_Information_r8 */
static int hf_rrc_dl_CommonInformation_05 = -1;   /* DL_CommonInformation_r8 */
static int hf_rrc_dl_InformationPerRL_List_05 = -1;  /* DL_InformationPerRL_List_r8 */
static int hf_rrc_controlChannelDRXInfo_TDD128 = -1;  /* ControlChannelDRXInfo_TDD128_r8 */
static int hf_rrc_sps_Information_TDD128 = -1;    /* SPS_Information_TDD128_r8 */
static int hf_rrc_rab_InformationSetup_01 = -1;   /* T_rab_InformationSetup */
static int hf_rrc_rab_InformationSetup_r7 = -1;   /* RAB_InformationSetup_r7 */
static int hf_rrc_rab_InformationSetup_v820ext = -1;  /* RAB_InformationSetup_v820ext */
static int hf_rrc_dl_HSPDSCH_Information_04 = -1;  /* DL_HSPDSCH_Information_r8_ext */
static int hf_rrc_dl_HSPDSCH_Information_05 = -1;  /* DL_HSPDSCH_Information_r8_ext2 */
static int hf_rrc_dl_AddReconfTransChInfoList_04 = -1;  /* DL_AddReconfTransChInfoList_r9 */
static int hf_rrc_ul_EDCH_Information_04 = -1;    /* UL_EDCH_Information_r9 */
static int hf_rrc_dl_HSPDSCH_Information_06 = -1;  /* DL_HSPDSCH_Information_r9 */
static int hf_rrc_cellDCHMeasOccasionInfo_TDD128 = -1;  /* CellDCHMeasOccasionInfo_TDD128_r9 */
static int hf_rrc_dl_AddReconfTransChInfoList_05 = -1;  /* DL_AddReconfTransChInfoList_TDD128_v9c0ext */
static int hf_rrc_dummy2_03 = -1;                 /* SR_VCC_Info */
static int hf_rrc_ul_MulticarrierEDCHInfo_TDD128 = -1;  /* UL_MulticarrierEDCHInfo_TDD128 */
static int hf_rrc_dl_CommonInformation_06 = -1;   /* DL_CommonInformation_r10 */
static int hf_rrc_mu_MIMO_Info_TDD128 = -1;       /* MU_MIMO_Info_TDD128 */
static int hf_rrc_rb_InformationReconfigList_06 = -1;  /* RB_InformationReconfigList_r11 */
static int hf_rrc_dl_AddReconfTransChInfoList_06 = -1;  /* DL_AddReconfTransChInfoList_r11 */
static int hf_rrc_ul_DPCH_Info_02 = -1;           /* UL_DPCH_Info_r11 */
static int hf_rrc_ul_EDCH_Information_05 = -1;    /* UL_EDCH_Information_r11 */
static int hf_rrc_dl_HSPDSCH_Information_07 = -1;  /* DL_HSPDSCH_Information_r11 */
static int hf_rrc_dl_CommonInformation_07 = -1;   /* DL_CommonInformation_r11 */
static int hf_rrc_dl_InformationPerRL_List_06 = -1;  /* DL_InformationPerRL_List_r11 */
static int hf_rrc_commonERGCHInfoFDD = -1;        /* CommonERGCHInfoFDD */
static int hf_rrc_r3_05 = -1;                     /* T_r3_05 */
static int hf_rrc_laterNonCriticalExtensions_08 = -1;  /* T_laterNonCriticalExtensions_08 */
static int hf_rrc_cellUpdateConfirm_CCCH_r3_add_ext = -1;  /* BIT_STRING */
static int hf_rrc_v4b0NonCriticalExtensions_03 = -1;  /* T_v4b0NonCriticalExtensions_03 */
static int hf_rrc_v590NonCriticalExtensions_03 = -1;  /* T_v590NonCriticalExtensions_03 */
static int hf_rrc_v5d0NonCriticalExtenstions_03 = -1;  /* T_v5d0NonCriticalExtenstions_03 */
static int hf_rrc_v690NonCriticalExtensions_05 = -1;  /* T_v690NonCriticalExtensions_05 */
static int hf_rrc_nonCriticalExtensions_24 = -1;  /* T_nonCriticalExtensions_24 */
static int hf_rrc_later_than_r3_04 = -1;          /* T_later_than_r3_04 */
static int hf_rrc_criticalExtensions_19 = -1;     /* T_criticalExtensions_19 */
static int hf_rrc_r4_01 = -1;                     /* T_r4_01 */
static int hf_rrc_v4d0NonCriticalExtensions_01 = -1;  /* T_v4d0NonCriticalExtensions_01 */
static int hf_rrc_cellUpdateConfirm_CCCH_r4_add_ext = -1;  /* BIT_STRING */
static int hf_rrc_v590NonCriticalExtensions_04 = -1;  /* T_v590NonCriticalExtensions_04 */
static int hf_rrc_v5d0NonCriticalExtenstions_04 = -1;  /* T_v5d0NonCriticalExtenstions_04 */
static int hf_rrc_v690NonCriticalExtensions_06 = -1;  /* T_v690NonCriticalExtensions_06 */
static int hf_rrc_nonCriticalExtensions_25 = -1;  /* T_nonCriticalExtensions_25 */
static int hf_rrc_criticalExtensions_20 = -1;     /* T_criticalExtensions_20 */
static int hf_rrc_r5_01 = -1;                     /* T_r5_01 */
static int hf_rrc_cellUpdateConfirm_CCCH_r5_add_ext = -1;  /* BIT_STRING */
static int hf_rrc_v5d0NonCriticalExtenstions_05 = -1;  /* T_v5d0NonCriticalExtenstions_05 */
static int hf_rrc_v690NonCriticalExtensions_07 = -1;  /* T_v690NonCriticalExtensions_07 */
static int hf_rrc_nonCriticalExtensions_26 = -1;  /* T_nonCriticalExtensions_26 */
static int hf_rrc_criticalExtensions_21 = -1;     /* T_criticalExtensions_21 */
static int hf_rrc_r6_02 = -1;                     /* T_r6_02 */
static int hf_rrc_v6b0NonCriticalExtensions_03 = -1;  /* T_v6b0NonCriticalExtensions_03 */
static int hf_rrc_nonCriticalExtensions_27 = -1;  /* T_nonCriticalExtensions_27 */
static int hf_rrc_criticalExtensions_22 = -1;     /* T_criticalExtensions_22 */
static int hf_rrc_r7_02 = -1;                     /* T_r7_02 */
static int hf_rrc_cellUpdateConfirm_r7_add_ext_01 = -1;  /* T_cellUpdateConfirm_r7_add_ext_01 */
static int hf_rrc_v780NonCriticalExtensions_02 = -1;  /* T_v780NonCriticalExtensions_02 */
static int hf_rrc_v860NonCriticalExtensions_02 = -1;  /* T_v860NonCriticalExtensions_02 */
static int hf_rrc_nonCriticalExtensions_28 = -1;  /* T_nonCriticalExtensions_28 */
static int hf_rrc_criticalExtensions_23 = -1;     /* T_criticalExtensions_23 */
static int hf_rrc_r8_02 = -1;                     /* T_r8_02 */
static int hf_rrc_v7d0NonCriticalExtensions_01 = -1;  /* T_v7d0NonCriticalExtensions_01 */
static int hf_rrc_v890NonCriticalExtensions_02 = -1;  /* T_v890NonCriticalExtensions_02 */
static int hf_rrc_v8a0NonCriticalExtensions_01 = -1;  /* T_v8a0NonCriticalExtensions_01 */
static int hf_rrc_v7f0NonCriticalExtensions_03 = -1;  /* T_v7f0NonCriticalExtensions_03 */
static int hf_rrc_v7g0NonCriticalExtensions_05 = -1;  /* T_v7g0NonCriticalExtensions_05 */
static int hf_rrc_nonCriticalExtensions_29 = -1;  /* T_nonCriticalExtensions_29 */
static int hf_rrc_criticalExtensions_24 = -1;     /* T_criticalExtensions_24 */
static int hf_rrc_r9_02 = -1;                     /* T_r9_02 */
static int hf_rrc_nonCriticalExtensions_30 = -1;  /* T_nonCriticalExtensions_30 */
static int hf_rrc_criticalExtensions_25 = -1;     /* T_criticalExtensions_25 */
static int hf_rrc_r10_02 = -1;                    /* T_r10_02 */
static int hf_rrc_vbxyNonCriticalExtensions_03 = -1;  /* T_vbxyNonCriticalExtensions_03 */
static int hf_rrc_nonCriticalExtensions_31 = -1;  /* T_nonCriticalExtensions_31 */
static int hf_rrc_criticalExtensions_26 = -1;     /* T_criticalExtensions_26 */
static int hf_rrc_r11_02 = -1;                    /* T_r11_02 */
static int hf_rrc_nonCriticalExtensions_32 = -1;  /* T_nonCriticalExtensions_32 */
static int hf_rrc_criticalExtensions_27 = -1;     /* T_criticalExtensions_27 */
static int hf_rrc_r3_06 = -1;                     /* T_r3_06 */
static int hf_rrc_counterCheck_r3 = -1;           /* CounterCheck_r3_IEs */
static int hf_rrc_laterNonCriticalExtensions_09 = -1;  /* T_laterNonCriticalExtensions_09 */
static int hf_rrc_counterCheck_r3_add_ext = -1;   /* BIT_STRING */
static int hf_rrc_nonCriticalExtensions_33 = -1;  /* T_nonCriticalExtensions_33 */
static int hf_rrc_later_than_r3_05 = -1;          /* T_later_than_r3_05 */
static int hf_rrc_criticalExtensions_28 = -1;     /* T_criticalExtensions_28 */
static int hf_rrc_rb_COUNT_C_MSB_InformationList = -1;  /* RB_COUNT_C_MSB_InformationList */
static int hf_rrc_rb_COUNT_C_InformationList = -1;  /* RB_COUNT_C_InformationList */
static int hf_rrc_laterNonCriticalExtensions_10 = -1;  /* T_laterNonCriticalExtensions_10 */
static int hf_rrc_counterCheckResponse_r3_add_ext = -1;  /* BIT_STRING */
static int hf_rrc_nonCriticalExtensions_34 = -1;  /* T_nonCriticalExtensions_34 */
static int hf_rrc_r3_07 = -1;                     /* T_r3_07 */
static int hf_rrc_downlinkDirectTransfer_r3 = -1;  /* DownlinkDirectTransfer_r3_IEs */
static int hf_rrc_laterNonCriticalExtensions_11 = -1;  /* T_laterNonCriticalExtensions_11 */
static int hf_rrc_downlinkDirectTransfer_r3_add_ext = -1;  /* BIT_STRING */
static int hf_rrc_nonCriticalExtensions_35 = -1;  /* T_nonCriticalExtensions_35 */
static int hf_rrc_later_than_r3_06 = -1;          /* T_later_than_r3_06 */
static int hf_rrc_criticalExtensions_29 = -1;     /* T_criticalExtensions_29 */
static int hf_rrc_cn_DomainIdentity = -1;         /* CN_DomainIdentity */
static int hf_rrc_nas_Message = -1;               /* NAS_Message */
static int hf_rrc_etws_Information = -1;          /* ETWS_Information */
static int hf_rrc_etws_WarningSecurityInfo = -1;  /* ETWS_WarningSecurityInfo */
static int hf_rrc_nonCriticalExtensions_36 = -1;  /* T_nonCriticalExtensions_36 */
static int hf_rrc_r3_08 = -1;                     /* T_r3_08 */
static int hf_rrc_handoverToUTRANCommand_r3 = -1;  /* HandoverToUTRANCommand_r3_IEs */
static int hf_rrc_nonCriticalExtensions_37 = -1;  /* T_nonCriticalExtensions_37 */
static int hf_rrc_criticalExtensions_30 = -1;     /* T_criticalExtensions_30 */
static int hf_rrc_r4_02 = -1;                     /* T_r4_02 */
static int hf_rrc_handoverToUTRANCommand_r4 = -1;  /* HandoverToUTRANCommand_r4_IEs */
static int hf_rrc_nonCriticalExtensions_38 = -1;  /* T_nonCriticalExtensions_38 */
static int hf_rrc_criticalExtensions_31 = -1;     /* T_criticalExtensions_31 */
static int hf_rrc_r5_02 = -1;                     /* T_r5_02 */
static int hf_rrc_handoverToUTRANCommand_r5 = -1;  /* HandoverToUTRANCommand_r5_IEs */
static int hf_rrc_nonCriticalExtensions_39 = -1;  /* T_nonCriticalExtensions_39 */
static int hf_rrc_criticalExtensions_32 = -1;     /* T_criticalExtensions_32 */
static int hf_rrc_r6_03 = -1;                     /* T_r6_03 */
static int hf_rrc_handoverToUTRANCommand_r6 = -1;  /* HandoverToUTRANCommand_r6_IEs */
static int hf_rrc_v6b0NonCriticalExtensions_04 = -1;  /* T_v6b0NonCriticalExtensions_04 */
static int hf_rrc_handoverToUTRANCommand_v6b0ext = -1;  /* HandoverToUTRANCommand_v6b0ext_IEs */
static int hf_rrc_nonCriticalExtensions_40 = -1;  /* T_nonCriticalExtensions_40 */
static int hf_rrc_criticalExtensions_33 = -1;     /* T_criticalExtensions_33 */
static int hf_rrc_r7_03 = -1;                     /* T_r7_03 */
static int hf_rrc_handoverToUTRANCommand_r7 = -1;  /* HandoverToUTRANCommand_r7_IEs */
static int hf_rrc_v780NonCriticalExtensions_03 = -1;  /* T_v780NonCriticalExtensions_03 */
static int hf_rrc_handoverToUTRANCommand_v780ext = -1;  /* HandoverToUTRANCommand_v780ext_IEs */
static int hf_rrc_v820NonCriticalExtensions = -1;  /* T_v820NonCriticalExtensions */
static int hf_rrc_handoverToUTRANCommand_v820ext = -1;  /* HandoverToUTRANCommand_v820ext_IEs */
static int hf_rrc_v7d0NonCriticalExtensions_02 = -1;  /* T_v7d0NonCriticalExtensions_02 */
static int hf_rrc_handoverToUTRANCommand_v7d0ext = -1;  /* HandoverToUTRANCommand_v7d0ext_IEs */
static int hf_rrc_nonCriticalExtensions_41 = -1;  /* T_nonCriticalExtensions_41 */
static int hf_rrc_criticalExtensions_34 = -1;     /* T_criticalExtensions_34 */
static int hf_rrc_r8_03 = -1;                     /* T_r8_03 */
static int hf_rrc_handoverToUTRANCommand_r8 = -1;  /* HandoverToUTRANCommand_r8_IEs */
static int hf_rrc_v7d0NonCriticalExtensions_03 = -1;  /* T_v7d0NonCriticalExtensions_03 */
static int hf_rrc_v890NonCriticalExtensions_03 = -1;  /* T_v890NonCriticalExtensions_03 */
static int hf_rrc_handoverToUTRANCommand_v890ext = -1;  /* HandoverToUTRANCommand_v890ext_IEs */
static int hf_rrc_v8a0NonCriticalExtensions_02 = -1;  /* T_v8a0NonCriticalExtensions_02 */
static int hf_rrc_handoverToUTRANCommand_v8a0ext = -1;  /* HandoverToUTRANCommand_v8a0ext_IEs */
static int hf_rrc_nonCriticalExtensions_42 = -1;  /* T_nonCriticalExtensions_42 */
static int hf_rrc_criticalExtensions_35 = -1;     /* T_criticalExtensions_35 */
static int hf_rrc_r9_03 = -1;                     /* T_r9_03 */
static int hf_rrc_handoverToUTRANCommand_r9 = -1;  /* HandoverToUTRANCommand_r9_IEs */
static int hf_rrc_v9c0NonCriticalExtensions_02 = -1;  /* T_v9c0NonCriticalExtensions_02 */
static int hf_rrc_handoverToUTRANCommand_v9c0ext = -1;  /* HandoverToUTRANCommand_v9c0ext_IEs */
static int hf_rrc_nonCriticalExtensions_43 = -1;  /* T_nonCriticalExtensions_43 */
static int hf_rrc_criticalExtensions_36 = -1;     /* HandoverToUTRANCommand_LaterCriticalExtensions */
static int hf_rrc_new_U_RNTI_01 = -1;             /* U_RNTI_Short */
static int hf_rrc_dummy_08 = -1;                  /* ActivationTime */
static int hf_rrc_cipheringAlgorithm = -1;        /* CipheringAlgorithm */
static int hf_rrc_specificationMode = -1;         /* T_specificationMode */
static int hf_rrc_complete = -1;                  /* T_complete */
static int hf_rrc_srb_InformationSetupList = -1;  /* SRB_InformationSetupList */
static int hf_rrc_rab_InformationSetupList = -1;  /* RAB_InformationSetupList */
static int hf_rrc_ul_DPCH_Info_03 = -1;           /* UL_DPCH_Info */
static int hf_rrc_modeSpecificInfo = -1;          /* T_modeSpecificInfo */
static int hf_rrc_fdd_06 = -1;                    /* T_fdd_06 */
static int hf_rrc_dummy2_04 = -1;                 /* CPCH_SetInfo */
static int hf_rrc_preconfiguration = -1;          /* T_preconfiguration */
static int hf_rrc_preConfigMode = -1;             /* T_preConfigMode */
static int hf_rrc_predefinedConfigIdentity = -1;  /* PredefinedConfigIdentity */
static int hf_rrc_defaultConfig = -1;             /* T_defaultConfig */
static int hf_rrc_defaultConfigMode = -1;         /* DefaultConfigMode */
static int hf_rrc_defaultConfigIdentity = -1;     /* DefaultConfigIdentity */
static int hf_rrc_rab_Info = -1;                  /* RAB_Info_Post */
static int hf_rrc_modeSpecificInfo_01 = -1;       /* T_modeSpecificInfo_01 */
static int hf_rrc_fdd_07 = -1;                    /* T_fdd_07 */
static int hf_rrc_ul_DPCH_Info_04 = -1;           /* UL_DPCH_InfoPostFDD */
static int hf_rrc_dl_CommonInformationPost = -1;  /* DL_CommonInformationPost */
static int hf_rrc_dl_InformationPerRL_List_07 = -1;  /* DL_InformationPerRL_ListPostFDD */
static int hf_rrc_frequencyInfo_01 = -1;          /* FrequencyInfoFDD */
static int hf_rrc_tdd_01 = -1;                    /* T_tdd */
static int hf_rrc_ul_DPCH_Info_05 = -1;           /* UL_DPCH_InfoPostTDD */
static int hf_rrc_dl_InformationPerRL = -1;       /* DL_InformationPerRL_PostTDD */
static int hf_rrc_frequencyInfo_02 = -1;          /* FrequencyInfoTDD */
static int hf_rrc_primaryCCPCH_TX_Power = -1;     /* PrimaryCCPCH_TX_Power */
static int hf_rrc_specificationMode_01 = -1;      /* T_specificationMode_01 */
static int hf_rrc_complete_01 = -1;               /* T_complete_01 */
static int hf_rrc_rab_InformationSetupList_01 = -1;  /* RAB_InformationSetupList_r4 */
static int hf_rrc_ul_DPCH_Info_06 = -1;           /* UL_DPCH_Info_r4 */
static int hf_rrc_modeSpecificInfo_02 = -1;       /* T_modeSpecificInfo_02 */
static int hf_rrc_fdd_08 = -1;                    /* T_fdd_08 */
static int hf_rrc_preconfiguration_01 = -1;       /* T_preconfiguration_01 */
static int hf_rrc_preConfigMode_01 = -1;          /* T_preConfigMode_01 */
static int hf_rrc_defaultConfig_01 = -1;          /* T_defaultConfig_01 */
static int hf_rrc_defaultConfigIdentity_01 = -1;  /* DefaultConfigIdentity_r4 */
static int hf_rrc_modeSpecificInfo_03 = -1;       /* T_modeSpecificInfo_03 */
static int hf_rrc_fdd_09 = -1;                    /* T_fdd_09 */
static int hf_rrc_tdd_02 = -1;                    /* T_tdd_01 */
static int hf_rrc_tdd384 = -1;                    /* T_tdd384 */
static int hf_rrc_tdd128 = -1;                    /* T_tdd128 */
static int hf_rrc_ul_DPCH_Info_07 = -1;           /* UL_DPCH_InfoPostTDD_LCR_r4 */
static int hf_rrc_dl_InformationPerRL_01 = -1;    /* DL_InformationPerRL_PostTDD_LCR_r4 */
static int hf_rrc_specificationMode_02 = -1;      /* T_specificationMode_02 */
static int hf_rrc_complete_02 = -1;               /* T_complete_02 */
static int hf_rrc_srb_InformationSetupList_01 = -1;  /* SRB_InformationSetupList_r5 */
static int hf_rrc_rab_InformationSetupList_02 = -1;  /* RAB_InformationSetupList_r5 */
static int hf_rrc_ul_DPCH_Info_08 = -1;           /* UL_DPCH_Info_r5 */
static int hf_rrc_modeSpecificInfo_04 = -1;       /* T_modeSpecificInfo_04 */
static int hf_rrc_fdd_10 = -1;                    /* T_fdd_10 */
static int hf_rrc_preconfiguration_02 = -1;       /* T_preconfiguration_02 */
static int hf_rrc_preConfigMode_02 = -1;          /* T_preConfigMode_02 */
static int hf_rrc_defaultConfig_02 = -1;          /* T_defaultConfig_02 */
static int hf_rrc_defaultConfigIdentity_02 = -1;  /* DefaultConfigIdentity_r5 */
static int hf_rrc_modeSpecificInfo_05 = -1;       /* T_modeSpecificInfo_05 */
static int hf_rrc_fdd_11 = -1;                    /* T_fdd_11 */
static int hf_rrc_tdd_03 = -1;                    /* T_tdd_02 */
static int hf_rrc_tdd384_01 = -1;                 /* T_tdd384_01 */
static int hf_rrc_tdd128_01 = -1;                 /* T_tdd128_01 */
static int hf_rrc_specificationMode_03 = -1;      /* T_specificationMode_03 */
static int hf_rrc_complete_03 = -1;               /* T_complete_03 */
static int hf_rrc_srb_InformationSetupList_02 = -1;  /* SRB_InformationSetupList_r6 */
static int hf_rrc_rab_InformationSetupList_03 = -1;  /* RAB_InformationSetupList_r6 */
static int hf_rrc_preconfiguration_03 = -1;       /* T_preconfiguration_03 */
static int hf_rrc_preConfigMode_03 = -1;          /* T_preConfigMode_03 */
static int hf_rrc_defaultConfig_03 = -1;          /* T_defaultConfig_03 */
static int hf_rrc_defaultConfigIdentity_03 = -1;  /* DefaultConfigIdentity_r6 */
static int hf_rrc_modeSpecificInfo_06 = -1;       /* T_modeSpecificInfo_06 */
static int hf_rrc_fdd_12 = -1;                    /* T_fdd_12 */
static int hf_rrc_tdd_04 = -1;                    /* T_tdd_03 */
static int hf_rrc_tdd384_02 = -1;                 /* T_tdd384_02 */
static int hf_rrc_tdd128_02 = -1;                 /* T_tdd128_02 */
static int hf_rrc_ue_hspa_identities = -1;        /* UE_HSPA_Identities_r6 */
static int hf_rrc_cipheringAlgorithm_01 = -1;     /* CipheringAlgorithm_r7 */
static int hf_rrc_specificationMode_04 = -1;      /* T_specificationMode_04 */
static int hf_rrc_complete_04 = -1;               /* T_complete_04 */
static int hf_rrc_srb_InformationSetupList_03 = -1;  /* SRB_InformationSetupList_r7 */
static int hf_rrc_rab_InformationSetupList_04 = -1;  /* RAB_InformationSetupList_r7 */
static int hf_rrc_preconfiguration_04 = -1;       /* T_preconfiguration_04 */
static int hf_rrc_preConfigMode_04 = -1;          /* T_preConfigMode_04 */
static int hf_rrc_defaultConfig_04 = -1;          /* T_defaultConfig_04 */
static int hf_rrc_modeSpecificInfo_07 = -1;       /* T_modeSpecificInfo_07 */
static int hf_rrc_fdd_13 = -1;                    /* T_fdd_13 */
static int hf_rrc_tdd_05 = -1;                    /* T_tdd_04 */
static int hf_rrc_tdd384_03 = -1;                 /* T_tdd384_03 */
static int hf_rrc_tdd128_03 = -1;                 /* T_tdd128_03 */
static int hf_rrc_tdd768 = -1;                    /* T_tdd768 */
static int hf_rrc_dl_InformationPerRL_02 = -1;    /* DL_InformationPerRL_List_r7 */
static int hf_rrc_specificationMode_05 = -1;      /* T_specificationMode_05 */
static int hf_rrc_complete_05 = -1;               /* T_complete_05 */
static int hf_rrc_srb_InformationSetupList_04 = -1;  /* SRB_InformationSetupList_r8 */
static int hf_rrc_rab_InformationSetupList_05 = -1;  /* RAB_InformationSetupList_r8 */
static int hf_rrc_preconfiguration_05 = -1;       /* T_preconfiguration_05 */
static int hf_rrc_preConfigMode_05 = -1;          /* T_preConfigMode_05 */
static int hf_rrc_defaultConfig_05 = -1;          /* T_defaultConfig_05 */
static int hf_rrc_dummy_09 = -1;                  /* DLUL_HSPA_Information_r8 */
static int hf_rrc_modeSpecificInfo_08 = -1;       /* T_modeSpecificInfo_08 */
static int hf_rrc_fdd_14 = -1;                    /* T_fdd_14 */
static int hf_rrc_tdd_06 = -1;                    /* T_tdd_05 */
static int hf_rrc_tdd384_04 = -1;                 /* T_tdd384_04 */
static int hf_rrc_tdd128_04 = -1;                 /* T_tdd128_04 */
static int hf_rrc_tdd768_01 = -1;                 /* T_tdd768_01 */
static int hf_rrc_rab_InformationSetupList_06 = -1;  /* RAB_InformationSetupList_v820ext */
static int hf_rrc_defaultConfigForCellFACH = -1;  /* DefaultConfigForCellFACH */
static int hf_rrc_specificationMode_06 = -1;      /* T_specificationMode_06 */
static int hf_rrc_complete_06 = -1;               /* T_complete_06 */
static int hf_rrc_preconfiguration_06 = -1;       /* T_preconfiguration_06 */
static int hf_rrc_preConfigMode_06 = -1;          /* T_preConfigMode_06 */
static int hf_rrc_defaultConfig_06 = -1;          /* T_defaultConfig_06 */
static int hf_rrc_dummy_10 = -1;                  /* DLUL_HSPA_Information_r9 */
static int hf_rrc_modeSpecificInfo_09 = -1;       /* T_modeSpecificInfo_09 */
static int hf_rrc_fdd_15 = -1;                    /* T_fdd_15 */
static int hf_rrc_tdd_07 = -1;                    /* T_tdd_06 */
static int hf_rrc_tdd384_05 = -1;                 /* T_tdd384_05 */
static int hf_rrc_tdd128_05 = -1;                 /* T_tdd128_05 */
static int hf_rrc_tdd768_02 = -1;                 /* T_tdd768_02 */
static int hf_rrc_r10_03 = -1;                    /* T_r10_03 */
static int hf_rrc_handoverToUTRANCommand_r10 = -1;  /* HandoverToUTRANCommand_r10_IEs */
static int hf_rrc_v9c0NonCriticalExtensions_03 = -1;  /* T_v9c0NonCriticalExtensions_03 */
static int hf_rrc_nonCriticalExtensions_44 = -1;  /* T_nonCriticalExtensions_44 */
static int hf_rrc_criticalExtensions_37 = -1;     /* T_criticalExtensions_36 */
static int hf_rrc_r11_03 = -1;                    /* T_r11_03 */
static int hf_rrc_handoverToUTRANCommand_r11 = -1;  /* HandoverToUTRANCommand_r11_IEs */
static int hf_rrc_handoverToUTRANCommand_r11_add_ext = -1;  /* BIT_STRING */
static int hf_rrc_nonCriticalExtensions_45 = -1;  /* T_nonCriticalExtensions_45 */
static int hf_rrc_criticalExtensions_38 = -1;     /* T_criticalExtensions_37 */
static int hf_rrc_specificationMode_07 = -1;      /* T_specificationMode_07 */
static int hf_rrc_complete_07 = -1;               /* T_complete_07 */
static int hf_rrc_preconfiguration_07 = -1;       /* T_preconfiguration_07 */
static int hf_rrc_preConfigMode_07 = -1;          /* T_preConfigMode_07 */
static int hf_rrc_defaultConfig_07 = -1;          /* T_defaultConfig_07 */
static int hf_rrc_dummy_11 = -1;                  /* DLUL_HSPA_Information_r10 */
static int hf_rrc_modeSpecificInfo_10 = -1;       /* T_modeSpecificInfo_10 */
static int hf_rrc_fdd_16 = -1;                    /* T_fdd_16 */
static int hf_rrc_tdd_08 = -1;                    /* T_tdd_07 */
static int hf_rrc_tdd384_06 = -1;                 /* T_tdd384_06 */
static int hf_rrc_tdd128_06 = -1;                 /* T_tdd128_06 */
static int hf_rrc_tdd768_03 = -1;                 /* T_tdd768_03 */
static int hf_rrc_specificationMode_08 = -1;      /* T_specificationMode_08 */
static int hf_rrc_complete_08 = -1;               /* T_complete_08 */
static int hf_rrc_rab_InformationSetupList_07 = -1;  /* RAB_InformationSetupList_r11 */
static int hf_rrc_preconfiguration_08 = -1;       /* T_preconfiguration_08 */
static int hf_rrc_preConfigMode_08 = -1;          /* T_preConfigMode_08 */
static int hf_rrc_defaultConfig_08 = -1;          /* T_defaultConfig_08 */
static int hf_rrc_modeSpecificInfo_11 = -1;       /* T_modeSpecificInfo_11 */
static int hf_rrc_fdd_17 = -1;                    /* T_fdd_17 */
static int hf_rrc_tdd_09 = -1;                    /* T_tdd_08 */
static int hf_rrc_tdd384_07 = -1;                 /* T_tdd384_07 */
static int hf_rrc_tdd128_07 = -1;                 /* T_tdd128_07 */
static int hf_rrc_tdd768_04 = -1;                 /* T_tdd768_04 */
static int hf_rrc_count_C_ActivationTime = -1;    /* ActivationTime */
static int hf_rrc_laterNonCriticalExtensions_12 = -1;  /* T_laterNonCriticalExtensions_12 */
static int hf_rrc_handoverToUTRANComplete_r3_add_ext = -1;  /* BIT_STRING */
static int hf_rrc_va40NonCriticalExtensions_01 = -1;  /* T_va40NonCriticalExtensions_01 */
static int hf_rrc_handoverToUTRANComplete_va40ext = -1;  /* HandoverToUTRANComplete_va40ext */
static int hf_rrc_vbxyNonCriticalExtensions_04 = -1;  /* T_vbxyNonCriticalExtensions_04 */
static int hf_rrc_handoverToUTRANComplete_vbxyext = -1;  /* HandoverToUTRANComplete_vbxyext_IEs */
static int hf_rrc_nonCriticalExtensions_46 = -1;  /* T_nonCriticalExtensions_46 */
static int hf_rrc_loggedMeasAvailable_01 = -1;    /* T_loggedMeasAvailable_01 */
static int hf_rrc_loggedConnectionEstablishmentFailureInfoAvailable = -1;  /* T_loggedConnectionEstablishmentFailureInfoAvailable */
static int hf_rrc_intraDomainNasNodeSelector = -1;  /* IntraDomainNasNodeSelector */
static int hf_rrc_v3a0NonCriticalExtensions_02 = -1;  /* T_v3a0NonCriticalExtensions_02 */
static int hf_rrc_initialDirectTransfer_v3a0ext = -1;  /* InitialDirectTransfer_v3a0ext */
static int hf_rrc_laterNonCriticalExtensions_13 = -1;  /* T_laterNonCriticalExtensions_13 */
static int hf_rrc_initialDirectTransfer_r3_add_ext = -1;  /* T_initialDirectTransfer_r3_add_ext */
static int hf_rrc_v590NonCriticalExtensions_05 = -1;  /* T_v590NonCriticalExtensions_05 */
static int hf_rrc_initialDirectTransfer_v590ext = -1;  /* InitialDirectTransfer_v590ext */
static int hf_rrc_v690NonCriticalExtensions_08 = -1;  /* T_v690NonCriticalExtensions_08 */
static int hf_rrc_initialDirectTransfer_v690ext = -1;  /* InitialDirectTransfer_v690ext_IEs */
static int hf_rrc_v770NonCriticalExtensions_01 = -1;  /* T_v770NonCriticalExtensions_01 */
static int hf_rrc_initialDirectTransfer_v770ext = -1;  /* InitialDirectTransfer_v770ext_IEs */
static int hf_rrc_v860NonCriticalExtensions_03 = -1;  /* T_v860NonCriticalExtensions_03 */
static int hf_rrc_initialDirectTransfer_v860ext = -1;  /* InitialDirectTransfer_v860ext_IEs */
static int hf_rrc_vbxyNonCriticalExtensions_05 = -1;  /* T_vbxyNonCriticalExtensions_05 */
static int hf_rrc_initialDirectTransfer_vbxyext = -1;  /* InitialDirectTransfer_vbxyext_IEs */
static int hf_rrc_nonCriticalExtensions_47 = -1;  /* T_nonCriticalExtensions_47 */
static int hf_rrc_start_Value = -1;               /* START_Value */
static int hf_rrc_plmn_Identity = -1;             /* PLMN_Identity */
static int hf_rrc_mbms_JoinedInformation = -1;    /* MBMS_JoinedInformation_r6 */
static int hf_rrc_csCallType_01 = -1;             /* T_csCallType_01 */
static int hf_rrc_supportOfCSG = -1;              /* T_supportOfCSG */
static int hf_rrc_initialDirectTransfer_v7g0ext = -1;  /* InitialDirectTransfer_v7g0ext_IEs */
static int hf_rrc_nonCriticalExtensions_48 = -1;  /* T_nonCriticalExtensions_48 */
static int hf_rrc_r3_09 = -1;                     /* T_r3_09 */
static int hf_rrc_handoverFromUTRANCommand_GSM_r3 = -1;  /* HandoverFromUTRANCommand_GSM_r3_IEs */
static int hf_rrc_laterNonCriticalExtensions_14 = -1;  /* T_laterNonCriticalExtensions_14 */
static int hf_rrc_handoverFromUTRANCommand_GSM_r3_add_ext = -1;  /* BIT_STRING */
static int hf_rrc_v690NonCriticalExtensions_09 = -1;  /* T_v690NonCriticalExtensions_09 */
static int hf_rrc_handoverFromUTRANCommand_GSM_v690ext = -1;  /* HandoverFromUTRANCommand_GSM_v690ext_IEs */
static int hf_rrc_v860NonCriticalExtensions_04 = -1;  /* T_v860NonCriticalExtensions_04 */
static int hf_rrc_handoverFromUTRANCommand_GSM_v860ext = -1;  /* HandoverFromUTRANCommand_GSM_v860ext_IEs */
static int hf_rrc_nonCriticalExtensions_49 = -1;  /* T_nonCriticalExtensions_49 */
static int hf_rrc_later_than_r3_07 = -1;          /* T_later_than_r3_07 */
static int hf_rrc_criticalExtensions_39 = -1;     /* T_criticalExtensions_38 */
static int hf_rrc_r6_04 = -1;                     /* T_r6_04 */
static int hf_rrc_handoverFromUTRANCommand_GSM_r6 = -1;  /* HandoverFromUTRANCommand_GSM_r6_IEs */
static int hf_rrc_handoverFromUTRANCommand_GSM_r6_add_ext = -1;  /* BIT_STRING */
static int hf_rrc_v860NonCriticalExtensions_05 = -1;  /* T_v860NonCriticalExtensions_05 */
static int hf_rrc_nonCriticalExtensions_50 = -1;  /* T_nonCriticalExtensions_50 */
static int hf_rrc_criticalExtensions_40 = -1;     /* T_criticalExtensions_39 */
static int hf_rrc_toHandoverRAB_Info = -1;        /* RAB_Info */
static int hf_rrc_frequency_band = -1;            /* Frequency_Band */
static int hf_rrc_gsm_message = -1;               /* T_gsm_message */
static int hf_rrc_single_GSM_Message_r3 = -1;     /* T_single_GSM_Message_r3 */
static int hf_rrc_gsm_MessageList_r3 = -1;        /* T_gsm_MessageList_r3 */
static int hf_rrc_gsm_Messages = -1;              /* GSM_MessageList */
static int hf_rrc_toHandoverRAB_Info_01 = -1;     /* RAB_InformationList_r6 */
static int hf_rrc_gsm_message_01 = -1;            /* T_gsm_message_01 */
static int hf_rrc_single_GSM_Message_r6 = -1;     /* T_single_GSM_Message_r6 */
static int hf_rrc_gsm_MessageList_r6 = -1;        /* T_gsm_MessageList_r6 */
static int hf_rrc_geran_SystemInfoType_01 = -1;   /* T_geran_SystemInfoType_01 */
static int hf_rrc_geran_SystemInfoType_02 = -1;   /* T_geran_SystemInfoType_02 */
static int hf_rrc_sr_vcc_SecurityRABInfo = -1;    /* SR_VCC_SecurityRABInfo_v860ext */
static int hf_rrc_handoverFromUTRANCommand_GERANIu_01 = -1;  /* T_handoverFromUTRANCommand_GERANIu */
static int hf_rrc_r5_03 = -1;                     /* T_r5_03 */
static int hf_rrc_handoverFromUTRANCommand_GERANIu_r5 = -1;  /* HandoverFromUTRANCommand_GERANIu_r5_IEs */
static int hf_rrc_nonCriticalExtensions_51 = -1;  /* T_nonCriticalExtensions_51 */
static int hf_rrc_later_than_r5 = -1;             /* T_later_than_r5 */
static int hf_rrc_criticalExtensions_41 = -1;     /* T_criticalExtensions_40 */
static int hf_rrc_frequency_Band = -1;            /* Frequency_Band */
static int hf_rrc_geranIu_Message = -1;           /* T_geranIu_Message */
static int hf_rrc_single_GERANIu_Message = -1;    /* T_single_GERANIu_Message */
static int hf_rrc_geranIu_MessageList = -1;       /* T_geranIu_MessageList */
static int hf_rrc_geranIu_Messages = -1;          /* GERANIu_MessageList */
static int hf_rrc_r3_10 = -1;                     /* T_r3_10 */
static int hf_rrc_handoverFromUTRANCommand_CDMA2000_r3 = -1;  /* HandoverFromUTRANCommand_CDMA2000_r3_IEs */
static int hf_rrc_laterNonCriticalExtensions_15 = -1;  /* T_laterNonCriticalExtensions_15 */
static int hf_rrc_handoverFromUTRANCommand_CDMA2000_r3_add_ext = -1;  /* BIT_STRING */
static int hf_rrc_nonCriticalExtensions_52 = -1;  /* T_nonCriticalExtensions_52 */
static int hf_rrc_later_than_r3_08 = -1;          /* T_later_than_r3_08 */
static int hf_rrc_criticalExtensions_42 = -1;     /* T_criticalExtensions_41 */
static int hf_rrc_cdma2000_MessageList = -1;      /* CDMA2000_MessageList */
static int hf_rrc_criticalExtensions_43 = -1;     /* T_criticalExtensions_42 */
static int hf_rrc_r8_04 = -1;                     /* T_r8_04 */
static int hf_rrc_handoverFromUTRANCommand_EUTRA_r8 = -1;  /* HandoverFromUTRANCommand_EUTRA_r8_IEs */
static int hf_rrc_handoverFromUTRANCommand_EUTRA_r8_add_ext = -1;  /* BIT_STRING */
static int hf_rrc_nonCriticalExtensions_53 = -1;  /* T_nonCriticalExtensions_53 */
static int hf_rrc_criticalExtensions_44 = -1;     /* T_criticalExtensions_43 */
static int hf_rrc_r11_04 = -1;                    /* T_r11_04 */
static int hf_rrc_handoverFromUTRANCommand_EUTRA_r11 = -1;  /* HandoverFromUTRANCommand_EUTRA_r11_IEs */
static int hf_rrc_handoverFromUTRANCommand_EUTRA_r11_add_ext = -1;  /* BIT_STRING */
static int hf_rrc_nonCriticalExtensions_54 = -1;  /* T_nonCriticalExtensions_54 */
static int hf_rrc_criticalExtensions_45 = -1;     /* T_criticalExtensions_44 */
static int hf_rrc_toHandoverRAB_Info_02 = -1;     /* RAB_InformationList */
static int hf_rrc_eutra_Message = -1;             /* T_eutra_Message */
static int hf_rrc_rsr_vcc_Info = -1;              /* RSR_VCC_Info */
static int hf_rrc_eutra_Message_01 = -1;          /* T_eutra_Message_01 */
static int hf_rrc_interRAT_HO_FailureCause = -1;  /* InterRAT_HO_FailureCause */
static int hf_rrc_interRATMessage = -1;           /* T_interRATMessage */
static int hf_rrc_gsm = -1;                       /* T_gsm */
static int hf_rrc_gsm_MessageList_gsm = -1;       /* GSM_MessageList */
static int hf_rrc_cdma2000 = -1;                  /* T_cdma2000 */
static int hf_rrc_laterNonCriticalExtensions_16 = -1;  /* T_laterNonCriticalExtensions_16 */
static int hf_rrc_handoverFromUTRANFailure_r3_add_ext = -1;  /* BIT_STRING */
static int hf_rrc_v590NonCriticalExtensions_06 = -1;  /* T_v590NonCriticalExtensions_06 */
static int hf_rrc_handoverFromUTRANFailure_v590ext = -1;  /* HandoverFromUtranFailure_v590ext_IEs */
static int hf_rrc_v860NonCriticalExtensions_06 = -1;  /* T_v860NonCriticalExtensions_06 */
static int hf_rrc_handoverFromUTRANFailure_v860ext = -1;  /* HandoverFromUtranFailure_v860ext_IEs */
static int hf_rrc_nonCriticalExtensions_55 = -1;  /* T_nonCriticalExtensions_55 */
static int hf_rrc_geranIu_MessageList_01 = -1;    /* GERANIu_MessageList */
static int hf_rrc_eutra_Message_02 = -1;          /* T_eutra_Message_02 */
static int hf_rrc_predefinedConfigStatusList = -1;  /* T_predefinedConfigStatusList */
static int hf_rrc_absent = -1;                    /* NULL */
static int hf_rrc_present = -1;                   /* PredefinedConfigStatusList */
static int hf_rrc_uE_SecurityInformation = -1;    /* T_uE_SecurityInformation */
static int hf_rrc_present_01 = -1;                /* UE_SecurityInformation */
static int hf_rrc_ue_CapabilityContainer = -1;    /* T_ue_CapabilityContainer */
static int hf_rrc_present_02 = -1;                /* T_present */
static int hf_rrc_v390NonCriticalExtensions = -1;  /* T_v390NonCriticalExtensions */
static int hf_rrc_present_03 = -1;                /* T_present_01 */
static int hf_rrc_interRATHandoverInfo_v390ext = -1;  /* InterRATHandoverInfo_v390ext_IEs */
static int hf_rrc_v3a0NonCriticalExtensions_03 = -1;  /* T_v3a0NonCriticalExtensions_03 */
static int hf_rrc_interRATHandoverInfo_v3a0ext = -1;  /* InterRATHandoverInfo_v3a0ext_IEs */
static int hf_rrc_laterNonCriticalExtensions_17 = -1;  /* T_laterNonCriticalExtensions_17 */
static int hf_rrc_interRATHandoverInfo_v3d0ext = -1;  /* InterRATHandoverInfo_v3d0ext_IEs */
static int hf_rrc_interRATHandoverInfo_r3_add_ext = -1;  /* T_interRATHandoverInfo_r3_add_ext */
static int hf_rrc_v3g0NonCriticalExtensions = -1;  /* T_v3g0NonCriticalExtensions */
static int hf_rrc_interRATHandoverInfo_v3g0ext = -1;  /* InterRATHandoverInfo_v3g0ext_IEs */
static int hf_rrc_v4b0NonCriticalExtensions_04 = -1;  /* T_v4b0NonCriticalExtensions_04 */
static int hf_rrc_interRATHandoverInfo_v4b0ext = -1;  /* InterRATHandoverInfo_v4b0ext_IEs */
static int hf_rrc_v4d0NonCriticalExtensions_02 = -1;  /* T_v4d0NonCriticalExtensions_02 */
static int hf_rrc_interRATHandoverInfo_v4d0ext = -1;  /* InterRATHandoverInfo_v4d0ext_IEs */
static int hf_rrc_v590NonCriticalExtensions_07 = -1;  /* T_v590NonCriticalExtensions_07 */
static int hf_rrc_interRATHandoverInfo_v590ext = -1;  /* InterRATHandoverInfo_v590ext_IEs */
static int hf_rrc_v690NonCriticalExtensions_10 = -1;  /* T_v690NonCriticalExtensions_10 */
static int hf_rrc_interRATHandoverInfo_v690ext = -1;  /* InterRATHandoverInfo_v690ext_IEs */
static int hf_rrc_v6b0NonCriticalExtensions_05 = -1;  /* InterRATHandoverInfo_NonCriticalExtension_v6b0_IEs */
static int hf_rrc_ue_RadioAccessCapability_v380ext = -1;  /* UE_RadioAccessCapability_v380ext */
static int hf_rrc_dl_PhysChCapabilityFDD_v380ext = -1;  /* DL_PhysChCapabilityFDD_v380ext */
static int hf_rrc_ue_RadioAccessCapability_v3a0ext = -1;  /* UE_RadioAccessCapability_v3a0ext */
static int hf_rrc_uESpecificBehaviourInformation1interRAT = -1;  /* UESpecificBehaviourInformation1interRAT */
static int hf_rrc_ue_RadioAccessCapability_v3g0ext = -1;  /* UE_RadioAccessCapability_v3g0ext */
static int hf_rrc_interRATHandoverInfo_v690ext1 = -1;  /* InterRATHandoverInfo_v690ext1_IEs */
static int hf_rrc_v7e0NonCriticalExtensions = -1;  /* T_v7e0NonCriticalExtensions */
static int hf_rrc_interRATHandoverInfo_v7e0ext = -1;  /* InterRATHandoverInfo_v7e0ext_IEs */
static int hf_rrc_v7f0NonCriticalExtensions_04 = -1;  /* T_v7f0NonCriticalExtensions_04 */
static int hf_rrc_interRATHandoverInfo_v7f0ext = -1;  /* InterRATHandoverInfo_v7f0ext_IEs */
static int hf_rrc_va40NonCriticalExtensions_02 = -1;  /* T_va40NonCriticalExtensions_02 */
static int hf_rrc_interRATHandoverInfo_va40ext = -1;  /* InterRATHandoverInfoBand_va40ext_IEs */
static int hf_rrc_nonCriticalExtensions_56 = -1;  /* T_nonCriticalExtensions_56 */
static int hf_rrc_accessStratumReleaseIndicator = -1;  /* AccessStratumReleaseIndicator */
static int hf_rrc_tdd128_RF_Capability = -1;      /* RadioFrequencyBandTDDList */
static int hf_rrc_predefinedConfigStatusListComp = -1;  /* PredefinedConfigStatusListComp */
static int hf_rrc_ue_RadioAccessCapabilityComp = -1;  /* UE_RadioAccessCapabilityComp */
static int hf_rrc_ue_RadioAccessCapability_v650ext = -1;  /* UE_RadioAccessCapability_v650ext */
static int hf_rrc_ue_SecurityInformation2 = -1;   /* UE_SecurityInformation2 */
static int hf_rrc_ue_RadioAccessCapabilityComp_01 = -1;  /* UE_RadioAccessCapabilityComp_ext */
static int hf_rrc_ue_RadioAccessCapabilityComp2 = -1;  /* UE_RadioAccessCapabilityComp2 */
static int hf_rrc_interRATHandoverInfo_v6b0ext = -1;  /* InterRATHandoverInfo_v6b0ext_IEs */
static int hf_rrc_v6e0NonCriticalExtensions = -1;  /* T_v6e0NonCriticalExtensions */
static int hf_rrc_interRATHandoverInfo_v6e0ext = -1;  /* InterRATHandoverInfo_v6e0ext_IEs */
static int hf_rrc_v770NonCriticalExtensions_02 = -1;  /* T_v770NonCriticalExtensions_02 */
static int hf_rrc_interRATHandoverInfo_v770ext = -1;  /* InterRATHandoverInfo_v770ext_IEs */
static int hf_rrc_v790nonCriticalExtensions = -1;  /* T_v790nonCriticalExtensions */
static int hf_rrc_interRATHandoverInfo_v790ext = -1;  /* InterRATHandoverInfo_v790ext_IEs */
static int hf_rrc_v860NonCriticalExtensions_07 = -1;  /* T_v860NonCriticalExtensions_07 */
static int hf_rrc_interRATHandoverInfo_v860ext = -1;  /* InterRATHandoverInfo_v860ext_IEs */
static int hf_rrc_v880NonCriticalExtensions = -1;  /* T_v880NonCriticalExtensions */
static int hf_rrc_interRATHandoverInfo_v880ext = -1;  /* InterRATHandoverInfo_v880ext_IEs */
static int hf_rrc_v920NonCriticalExtensions = -1;  /* T_v920NonCriticalExtensions */
static int hf_rrc_interRATHandoverInfo_v920ext = -1;  /* InterRATHandoverInfo_v920ext_IEs */
static int hf_rrc_v8b0NonCriticalExtensions = -1;  /* T_v8b0NonCriticalExtensions */
static int hf_rrc_interRATHandoverInfo_v8b0ext = -1;  /* InterRATHandoverInfo_v8b0ext_IEs */
static int hf_rrc_v950NonCriticalExtensions_02 = -1;  /* T_v950NonCriticalExtensions_02 */
static int hf_rrc_interRATHandoverInfo_v950ext = -1;  /* InterRATHandoverInfo_v950ext_IEs */
static int hf_rrc_va40NonCriticalExtensions_03 = -1;  /* T_va40NonCriticalExtensions_03 */
static int hf_rrc_interRATHandoverInfo_va40ext_01 = -1;  /* InterRATHandoverInfo_va40ext_IEs */
static int hf_rrc_va80NonCriticalExtensions = -1;  /* T_va80NonCriticalExtensions */
static int hf_rrc_interRATHandoverInfo_va80ext = -1;  /* InterRATHandoverInfo_va80ext_IEs */
static int hf_rrc_vbxyNonCriticalExtensions_06 = -1;  /* T_vbxyNonCriticalExtensions_06 */
static int hf_rrc_interRATHandoverInfo_vbxyext = -1;  /* InterRATHandoverInfo_vbxyext_IEs */
static int hf_rrc_nonCriticalExtensions_57 = -1;  /* T_nonCriticalExtensions_57 */
static int hf_rrc_supportForSIB11bis = -1;        /* T_supportForSIB11bis */
static int hf_rrc_supportForFDPCH = -1;           /* T_supportForFDPCH */
static int hf_rrc_ue_RadioAccessCapabilityInfo = -1;  /* UE_RadioAccessCapabilityInfo_v770ext */
static int hf_rrc_ue_RadioAccessCapabilityComp_02 = -1;  /* UE_RadioAccessCapabilityComp_v770ext */
static int hf_rrc_ue_RadioAccessCapabilityComp2_01 = -1;  /* UE_RadioAccessCapabilityComp2_v770ext */
static int hf_rrc_supportForEDPCCHPowerBoosting = -1;  /* T_supportForEDPCCHPowerBoosting */
static int hf_rrc_ue_RadioAccessCapability = -1;  /* UE_RadioAccessCapability_v7e0ext */
static int hf_rrc_ue_RadioAccessCapability_01 = -1;  /* UE_RadioAccessCapability_v7f0ext */
static int hf_rrc_ue_RadioAccessCapabilityComp2_02 = -1;  /* UE_RadioAccessCapabilityComp2_v7f0ext */
static int hf_rrc_ue_RadioAccessCapabilityInfo_01 = -1;  /* UE_RadioAccessCapability_v860ext */
static int hf_rrc_ue_RadioAccessCapabilityComp2_03 = -1;  /* UE_RadioAccessCapabilityComp2_v860ext */
static int hf_rrc_ue_RadioAccessCapabilityInfo_02 = -1;  /* UE_RadioAccessCapability_v880ext */
static int hf_rrc_ue_RadioAccessCapabilityInfo_TDD128 = -1;  /* UE_RadioAccessCapabilityComp_TDD128 */
static int hf_rrc_ue_RadioAccessCapabilityInfo_03 = -1;  /* UE_RadioAccessCapability_v920ext */
static int hf_rrc_ue_RadioAccessCapabilityComp2_04 = -1;  /* UE_RadioAccessCapabilityComp2_v920ext */
static int hf_rrc_ue_RadioAccessCapabilityInfo_04 = -1;  /* UE_RadioAccessCapabilityInfo_TDD128_v8b0ext */
static int hf_rrc_pdcp_Capability = -1;           /* PDCP_Capability_r4_ext */
static int hf_rrc_ue_RadioAccessCapabilityInfo_05 = -1;  /* UE_RadioAccessCapability_va40ext */
static int hf_rrc_ue_RadioAccessCapabilityComp2_05 = -1;  /* UE_RadioAccessCapabilityComp2_va40ext */
static int hf_rrc_ue_RadioAccessCapabBand = -1;   /* UE_RadioAccessCapabBand_va40ext */
static int hf_rrc_rf_CapabilityFDDComp = -1;      /* RF_CapabBandListFDDComp_ext2 */
static int hf_rrc_ue_RadioAccessCapabilityInfo_va80ext = -1;  /* UE_RadioAccessCapability_va80ext */
static int hf_rrc_ue_RadioAccessCapabilityInfo_vbxyext = -1;  /* UE_RadioAccessCapability_vbxyext */
static int hf_rrc_ue_RadioAccessCapabilityComp2_06 = -1;  /* UE_RadioAccessCapabilityComp2_vbxyext */
static int hf_rrc_r3_11 = -1;                     /* T_r3_11 */
static int hf_rrc_measurementControl_r3 = -1;     /* MeasurementControl_r3_IEs */
static int hf_rrc_v390nonCriticalExtensions = -1;  /* T_v390nonCriticalExtensions */
static int hf_rrc_measurementControl_v390ext = -1;  /* MeasurementControl_v390ext */
static int hf_rrc_v3a0NonCriticalExtensions_04 = -1;  /* T_v3a0NonCriticalExtensions_04 */
static int hf_rrc_measurementControl_v3a0ext = -1;  /* MeasurementControl_v3a0ext */
static int hf_rrc_laterNonCriticalExtensions_18 = -1;  /* T_laterNonCriticalExtensions_18 */
static int hf_rrc_measurementControl_r3_add_ext = -1;  /* BIT_STRING */
static int hf_rrc_v4b0NonCriticalExtensions_05 = -1;  /* T_v4b0NonCriticalExtensions_05 */
static int hf_rrc_v590NonCriticalExtensions_08 = -1;  /* T_v590NonCriticalExtensions_08 */
static int hf_rrc_measurementControl_v590ext = -1;  /* MeasurementControl_v590ext_IEs */
static int hf_rrc_v5b0NonCriticalExtensions = -1;  /* T_v5b0NonCriticalExtensions */
static int hf_rrc_measurementControl_v5b0ext = -1;  /* MeasurementControl_v5b0ext_IEs */
static int hf_rrc_nonCriticalExtensions_58 = -1;  /* T_nonCriticalExtensions_58 */
static int hf_rrc_later_than_r3_09 = -1;          /* T_later_than_r3_09 */
static int hf_rrc_criticalExtensions_46 = -1;     /* T_criticalExtensions_45 */
static int hf_rrc_r4_03 = -1;                     /* T_r4_03 */
static int hf_rrc_measurementControl_r4 = -1;     /* MeasurementControl_r4_IEs */
static int hf_rrc_v4d0NonCriticalExtensions_03 = -1;  /* T_v4d0NonCriticalExtensions_03 */
static int hf_rrc_measurementControl_r4_add_ext = -1;  /* BIT_STRING */
static int hf_rrc_v590NonCriticalExtensions_09 = -1;  /* T_v590NonCriticalExtensions_09 */
static int hf_rrc_v5b0NonCriticalExtensions_01 = -1;  /* T_v5b0NonCriticalExtensions_01 */
static int hf_rrc_nonCriticalExtensions_59 = -1;  /* T_nonCriticalExtensions_59 */
static int hf_rrc_later_than_r4 = -1;             /* T_later_than_r4 */
static int hf_rrc_rrc_TransactionIdentifier_MSP = -1;  /* RRC_TransactionIdentifier */
static int hf_rrc_criticalExtensions_47 = -1;     /* T_criticalExtensions_46 */
static int hf_rrc_r6_05 = -1;                     /* T_r6_05 */
static int hf_rrc_measurementControl_r6 = -1;     /* MeasurementControl_r6_IEs */
static int hf_rrc_v6a0NonCriticalExtensions = -1;  /* T_v6a0NonCriticalExtensions */
static int hf_rrc_measurementControl_v6a0ext = -1;  /* MeasurementControl_v6a0ext_IEs */
static int hf_rrc_nonCriticalExtensions_60 = -1;  /* T_nonCriticalExtensions_60 */
static int hf_rrc_criticalExtensions_48 = -1;     /* T_criticalExtensions_47 */
static int hf_rrc_r7_04 = -1;                     /* T_r7_04 */
static int hf_rrc_measurementControl_r7 = -1;     /* MeasurementControl_r7_IEs */
static int hf_rrc_measurementControl_r7_add_ext = -1;  /* BIT_STRING */
static int hf_rrc_v7b0NonCriticalExtensions = -1;  /* T_v7b0NonCriticalExtensions */
static int hf_rrc_measurementControl_v7b0ext = -1;  /* MeasurementControl_v7b0ext_IEs */
static int hf_rrc_nonCriticalExtensions_61 = -1;  /* T_nonCriticalExtensions_61 */
static int hf_rrc_criticalExtensions_49 = -1;     /* T_criticalExtensions_48 */
static int hf_rrc_r8_05 = -1;                     /* T_r8_05 */
static int hf_rrc_measurementControl_r8 = -1;     /* MeasurementControl_r8_IEs */
static int hf_rrc_measurementControl_r8_add_ext = -1;  /* BIT_STRING */
static int hf_rrc_v8a0NonCriticalExtensions_03 = -1;  /* T_v8a0NonCriticalExtensions_03 */
static int hf_rrc_measurementControl_v8a0ext = -1;  /* MeasurementControl_v8a0ext_IEs */
static int hf_rrc_nonCriticalExtensions_62 = -1;  /* T_nonCriticalExtensions_62 */
static int hf_rrc_criticalExtensions_50 = -1;     /* T_criticalExtensions_49 */
static int hf_rrc_r9_04 = -1;                     /* T_r9_04 */
static int hf_rrc_measurementControl_r9 = -1;     /* MeasurementControl_r9_IEs */
static int hf_rrc_measurementControl_r9_add_ext = -1;  /* BIT_STRING */
static int hf_rrc_v970NonCriticalExtensions = -1;  /* T_v970NonCriticalExtensions */
static int hf_rrc_measurementControl_v970ext = -1;  /* MeasurementControl_v970ext_IEs */
static int hf_rrc_nonCriticalExtensions_63 = -1;  /* T_nonCriticalExtensions_63 */
static int hf_rrc_criticalExtensions_51 = -1;     /* T_criticalExtensions_50 */
static int hf_rrc_r10_04 = -1;                    /* T_r10_04 */
static int hf_rrc_measurementControl_r10 = -1;    /* MeasurementControl_r10_IEs */
static int hf_rrc_measurementControl_r10_add_ext = -1;  /* BIT_STRING */
static int hf_rrc_vaa0NonCriticalExtensions = -1;  /* T_vaa0NonCriticalExtensions */
static int hf_rrc_measurementControl_vaa0ext = -1;  /* MeasurementControl_vaa0ext_IEs */
static int hf_rrc_nonCriticalExtensions_64 = -1;  /* T_nonCriticalExtensions_64 */
static int hf_rrc_criticalExtensions_52 = -1;     /* T_criticalExtensions_51 */
static int hf_rrc_r11_05 = -1;                    /* T_r11_05 */
static int hf_rrc_measurementControl_r11 = -1;    /* MeasurementControl_r11_IEs */
static int hf_rrc_measurementControl_r11_add_ext = -1;  /* BIT_STRING */
static int hf_rrc_nonCriticalExtensions_65 = -1;  /* T_nonCriticalExtensions_65 */
static int hf_rrc_criticalExtensions_53 = -1;     /* T_criticalExtensions_52 */
static int hf_rrc_measurementIdentity = -1;       /* MeasurementIdentity */
static int hf_rrc_measurementCommand = -1;        /* MeasurementCommand */
static int hf_rrc_measurementReportingMode = -1;  /* MeasurementReportingMode */
static int hf_rrc_additionalMeasurementList = -1;  /* AdditionalMeasurementID_List */
static int hf_rrc_dpch_CompressedModeStatusInfo = -1;  /* DPCH_CompressedModeStatusInfo */
static int hf_rrc_ue_Positioning_Measurement_v390ext = -1;  /* UE_Positioning_Measurement_v390ext */
static int hf_rrc_measurementCommand_01 = -1;     /* MeasurementCommand_r4 */
static int hf_rrc_measurementCommand_v590ext = -1;  /* T_measurementCommand_v590ext */
static int hf_rrc_intra_frequency = -1;           /* Intra_FreqEventCriteriaList_v590ext */
static int hf_rrc_inter_frequency = -1;           /* Inter_FreqEventCriteriaList_v590ext */
static int hf_rrc_intraFreqReportingCriteria_1b_r5 = -1;  /* IntraFreqReportingCriteria_1b_r5 */
static int hf_rrc_intraFreqEvent_1d_r5 = -1;      /* IntraFreqEvent_1d_r5 */
static int hf_rrc_rrc_TransactionIdentifier_MSP_v590ext = -1;  /* RRC_TransactionIdentifier */
static int hf_rrc_interRATCellInfoIndication = -1;  /* InterRATCellInfoIndication */
static int hf_rrc_measurementCommand_02 = -1;     /* MeasurementCommand_r6 */
static int hf_rrc_measurementCommand_03 = -1;     /* MeasurementCommand_r7 */
static int hf_rrc_newInterFreqCellList = -1;      /* NewInterFreqCellList_v7b0ext */
static int hf_rrc_measurementCommand_04 = -1;     /* MeasurementCommand_r8 */
static int hf_rrc_newIntraFreqCellList = -1;      /* NewIntraFreqCellList_LCR_v8a0ext */
static int hf_rrc_newInterFreqCellList_01 = -1;   /* NewInterFreqCellList_LCR_v8a0ext */
static int hf_rrc_measurementCommand_05 = -1;     /* MeasurementCommand_r9 */
static int hf_rrc_measurementIdentity_01 = -1;    /* MeasurementIdentity_r9 */
static int hf_rrc_additionalMeasurementList_01 = -1;  /* AdditionalMeasurementID_List_r9 */
static int hf_rrc_measurementCommand_06 = -1;     /* MeasurementCommand_r10 */
static int hf_rrc_dpch_CompressedModeStatusInfo_01 = -1;  /* DPCH_CompressedModeStatusInfo_r10 */
static int hf_rrc_periodicalReportingCriteriaOnSecULFreq = -1;  /* PeriodicalWithReportingCellStatusOnSecULFreq */
static int hf_rrc_measurementCommand_07 = -1;     /* MeasurementCommand_r11 */
static int hf_rrc_laterNonCriticalExtensions_19 = -1;  /* T_laterNonCriticalExtensions_19 */
static int hf_rrc_measurementControlFailure_r3_add_ext = -1;  /* BIT_STRING */
static int hf_rrc_v590NonCriticalExtensions_10 = -1;  /* T_v590NonCriticalExtensions_10 */
static int hf_rrc_measurementControlFailure_v590ext = -1;  /* MeasurementControlFailure_v590ext_IEs */
static int hf_rrc_nonCriticalExtensions_66 = -1;  /* T_nonCriticalExtensions_66 */
static int hf_rrc_measuredResults = -1;           /* MeasuredResults */
static int hf_rrc_additionalMeasuredResults = -1;  /* MeasuredResultsList */
static int hf_rrc_eventResults = -1;              /* EventResults */
static int hf_rrc_v390nonCriticalExtensions_01 = -1;  /* T_v390nonCriticalExtensions_01 */
static int hf_rrc_measurementReport_v390ext = -1;  /* MeasurementReport_v390ext */
static int hf_rrc_laterNonCriticalExtensions_20 = -1;  /* T_laterNonCriticalExtensions_20 */
static int hf_rrc_measurementReport_r3_add_ext = -1;  /* BIT_STRING */
static int hf_rrc_v4b0NonCriticalExtensions_06 = -1;  /* T_v4b0NonCriticalExtensions_06 */
static int hf_rrc_measurementReport_v4b0ext = -1;  /* MeasurementReport_v4b0ext_IEs */
static int hf_rrc_v590NonCriticalExtensions_11 = -1;  /* T_v590NonCriticalExtensions_11 */
static int hf_rrc_measurementReport_v590ext = -1;  /* MeasurementReport_v590ext_IEs */
static int hf_rrc_v5b0NonCriticalExtensions_02 = -1;  /* T_v5b0NonCriticalExtensions_02 */
static int hf_rrc_measurementReport_v5b0ext = -1;  /* MeasurementReport_v5b0ext_IEs */
static int hf_rrc_v690NonCriticalExtensions_11 = -1;  /* T_v690NonCriticalExtensions_11 */
static int hf_rrc_measurementReport_v690ext = -1;  /* MeasurementReport_v690ext_IEs */
static int hf_rrc_v770NonCriticalExtensions_03 = -1;  /* T_v770NonCriticalExtensions_03 */
static int hf_rrc_measurementReport_v770ext = -1;  /* MeasurementReport_v770ext_IEs */
static int hf_rrc_v860NonCriticalExtensions_08 = -1;  /* T_v860NonCriticalExtensions_08 */
static int hf_rrc_measurementReport_v860ext = -1;  /* MeasurementReport_v860ext_IEs */
static int hf_rrc_v920NonCriticalExtensions_01 = -1;  /* T_v920NonCriticalExtensions_01 */
static int hf_rrc_measurementReport_v920ext = -1;  /* MeasurementReport_v920ext_IEs */
static int hf_rrc_v970NonCriticalExtensions_01 = -1;  /* T_v970NonCriticalExtensions_01 */
static int hf_rrc_measurementReport_v970ext_IE = -1;  /* MeasurementReport_v970ext_IEs */
static int hf_rrc_va40NonCriticalExtensions_04 = -1;  /* T_va40NonCriticalExtensions_04 */
static int hf_rrc_measurementReport_va40ext = -1;  /* MeasurementReport_va40ext_IEs */
static int hf_rrc_vbxyNonCriticalExtensions_07 = -1;  /* T_vbxyNonCriticalExtensions_07 */
static int hf_rrc_measurementReport_vbxyext = -1;  /* MeasurementReport_vbxyext_IEs */
static int hf_rrc_nonCriticalExtensions_67 = -1;  /* T_nonCriticalExtensions_67 */
static int hf_rrc_measuredResults_v390ext = -1;   /* MeasuredResults_v390ext */
static int hf_rrc_interFreqEventResults_LCR = -1;  /* InterFreqEventResults_LCR_r4_ext */
static int hf_rrc_additionalMeasuredResults_LCR = -1;  /* MeasuredResultsList_LCR_r4_ext */
static int hf_rrc_dummy_12 = -1;                  /* PrimaryCPICH_Info */
static int hf_rrc_measuredResults_v590ext = -1;   /* MeasuredResults_v590ext */
static int hf_rrc_measuredResults_01 = -1;        /* MeasuredResults_v770ext */
static int hf_rrc_additionalMeasuredResults_01 = -1;  /* MeasuredResultsList_v770xet */
static int hf_rrc_eventResults_01 = -1;           /* EventResults_v770ext */
static int hf_rrc_ue_Positioning_OTDOA_MeasuredResults = -1;  /* UE_Positioning_OTDOA_MeasuredResultsTDD_ext */
static int hf_rrc_measuredResults_02 = -1;        /* MeasuredResults_v860ext */
static int hf_rrc_additionalMeasuredResults_02 = -1;  /* MeasuredResultsList_v860ext */
static int hf_rrc_eventResults_02 = -1;           /* EventResults_v860ext */
static int hf_rrc_eutra_MeasuredResults = -1;     /* EUTRA_MeasuredResults */
static int hf_rrc_eutra_EventResults = -1;        /* EUTRA_EventResults */
static int hf_rrc_cSGProximityIndication = -1;    /* CSGProximityIndication */
static int hf_rrc_measuredResults_03 = -1;        /* MeasuredResults_v920ext */
static int hf_rrc_measuredResultsOnSecUlFreq = -1;  /* MeasuredResultsOnSecUlFreq */
static int hf_rrc_additionalMeasuredResults_03 = -1;  /* MeasuredResultsList_v920ext */
static int hf_rrc_additionalMeasuredResultsOnSecUlFreq = -1;  /* MeasuredResultsListOnSecUlFreq */
static int hf_rrc_eventResultsOnSecUlFreq = -1;   /* EventResultsOnSecUlFreq */
static int hf_rrc_eutra_MeasuredResults_01 = -1;  /* EUTRA_MeasuredResults_v920ext */
static int hf_rrc_measurementIdentity_02 = -1;    /* MeasurementIdentityExt */
static int hf_rrc_eventResults_03 = -1;           /* EventResults_va40ext */
static int hf_rrc_loggedMeasAvailable_02 = -1;    /* T_loggedMeasAvailable_02 */
static int hf_rrc_loggedANRResultsAvailable_01 = -1;  /* T_loggedANRResultsAvailable_01 */
static int hf_rrc_eutra_EventResultsForCELLFACH = -1;  /* EUTRA_EventResultsForCELLFACH */
static int hf_rrc_pagingRecordList = -1;          /* PagingRecordList */
static int hf_rrc_bcch_ModificationInfo = -1;     /* BCCH_ModificationInfo */
static int hf_rrc_laterNonCriticalExtensions_21 = -1;  /* T_laterNonCriticalExtensions_21 */
static int hf_rrc_pagingType1_r3_add_ext = -1;    /* BIT_STRING */
static int hf_rrc_v590NonCriticalExtensions_12 = -1;  /* T_v590NonCriticalExtensions_12 */
static int hf_rrc_pagingType1_v590ext = -1;       /* PagingType1_v590ext_IEs */
static int hf_rrc_v860NonCriticalExtensions_09 = -1;  /* T_v860NonCriticalExtensions_09 */
static int hf_rrc_pagingType1_v860ext = -1;       /* PagingType1_v860ext_IEs */
static int hf_rrc_nonCriticalExtensions_68 = -1;  /* T_nonCriticalExtensions_68 */
static int hf_rrc_pagingRecord2List = -1;         /* PagingRecord2List_r5 */
static int hf_rrc_pagingCause = -1;               /* PagingCause */
static int hf_rrc_pagingRecordTypeID = -1;        /* PagingRecordTypeID */
static int hf_rrc_laterNonCriticalExtensions_22 = -1;  /* T_laterNonCriticalExtensions_22 */
static int hf_rrc_pagingType2_r3_add_ext = -1;    /* BIT_STRING */
static int hf_rrc_nonCriticalExtensions_69 = -1;  /* T_nonCriticalExtensions_69 */
static int hf_rrc_r3_12 = -1;                     /* T_r3_12 */
static int hf_rrc_physicalChannelReconfiguration_r3 = -1;  /* PhysicalChannelReconfiguration_r3_IEs */
static int hf_rrc_v3a0NonCriticalExtensions_05 = -1;  /* T_v3a0NonCriticalExtensions_05 */
static int hf_rrc_physicalChannelReconfiguration_v3a0ext = -1;  /* PhysicalChannelReconfiguration_v3a0ext */
static int hf_rrc_laterNonCriticalExtensions_23 = -1;  /* T_laterNonCriticalExtensions_23 */
static int hf_rrc_physicalChannelReconfiguration_r3_add_ext = -1;  /* BIT_STRING */
static int hf_rrc_v4b0NonCriticalExtenstions = -1;  /* T_v4b0NonCriticalExtenstions */
static int hf_rrc_physicalChannelReconfiguration_v4b0ext = -1;  /* PhysicalChannelReconfiguration_v4b0ext_IEs */
static int hf_rrc_v590NonCriticalExtenstions_02 = -1;  /* T_v590NonCriticalExtenstions_02 */
static int hf_rrc_physicalChannelReconfiguration_v590ext = -1;  /* PhysicalChannelReconfiguration_v590ext_IEs */
static int hf_rrc_v690NonCriticalExtensions_12 = -1;  /* T_v690NonCriticalExtensions_12 */
static int hf_rrc_physicalChannelReconfiguration_v690ext = -1;  /* PhysicalChannelReconfiguration_v690ext_IEs */
static int hf_rrc_v770NonCriticalExtensions_04 = -1;  /* T_v770NonCriticalExtensions_04 */
static int hf_rrc_physicalChannelReconfiguration_v770ext = -1;  /* PhysicalChannelReconfiguration_v770ext_IEs */
static int hf_rrc_nonCriticalExtensions_70 = -1;  /* T_nonCriticalExtensions_70 */
static int hf_rrc_later_than_r3_10 = -1;          /* T_later_than_r3_10 */
static int hf_rrc_criticalExtensions_54 = -1;     /* T_criticalExtensions_53 */
static int hf_rrc_r4_04 = -1;                     /* T_r4_04 */
static int hf_rrc_physicalChannelReconfiguration_r4 = -1;  /* PhysicalChannelReconfiguration_r4_IEs */
static int hf_rrc_v4d0NonCriticalExtensions_04 = -1;  /* T_v4d0NonCriticalExtensions_04 */
static int hf_rrc_physicalChannelReconfiguration_r4_add_ext = -1;  /* BIT_STRING */
static int hf_rrc_v590NonCriticalExtenstions_03 = -1;  /* T_v590NonCriticalExtenstions_03 */
static int hf_rrc_v690NonCriticalExtensions_13 = -1;  /* T_v690NonCriticalExtensions_13 */
static int hf_rrc_v770NonCriticalExtensions_05 = -1;  /* T_v770NonCriticalExtensions_05 */
static int hf_rrc_nonCriticalExtensions_71 = -1;  /* T_nonCriticalExtensions_71 */
static int hf_rrc_criticalExtensions_55 = -1;     /* T_criticalExtensions_54 */
static int hf_rrc_r5_04 = -1;                     /* T_r5_04 */
static int hf_rrc_physicalChannelReconfiguration_r5 = -1;  /* PhysicalChannelReconfiguration_r5_IEs */
static int hf_rrc_physicalChannelReconfiguration_r5_add_ext = -1;  /* BIT_STRING */
static int hf_rrc_v690NonCriticalExtensions_14 = -1;  /* T_v690NonCriticalExtensions_14 */
static int hf_rrc_v770NonCriticalExtensions_06 = -1;  /* T_v770NonCriticalExtensions_06 */
static int hf_rrc_nonCriticalExtensions_72 = -1;  /* T_nonCriticalExtensions_72 */
static int hf_rrc_criticalExtensions_56 = -1;     /* T_criticalExtensions_55 */
static int hf_rrc_r6_06 = -1;                     /* T_r6_06 */
static int hf_rrc_physicalChannelReconfiguration_r6 = -1;  /* PhysicalChannelReconfiguration_r6_IEs */
static int hf_rrc_physicalChannelReconfiguration_r6_add_ext = -1;  /* BIT_STRING */
static int hf_rrc_v6b0NonCriticalExtensions_06 = -1;  /* T_v6b0NonCriticalExtensions_05 */
static int hf_rrc_physicalChannelReconfiguration_v6b0ext = -1;  /* PhysicalChannelReconfiguration_v6b0ext_IEs */
static int hf_rrc_v770NonCriticalExtensions_07 = -1;  /* T_v770NonCriticalExtensions_07 */
static int hf_rrc_nonCriticalExtensions_73 = -1;  /* T_nonCriticalExtensions_73 */
static int hf_rrc_criticalExtensions_57 = -1;     /* T_criticalExtensions_56 */
static int hf_rrc_r7_05 = -1;                     /* T_r7_05 */
static int hf_rrc_physicalChannelReconfiguration_r7 = -1;  /* PhysicalChannelReconfiguration_r7_IEs */
static int hf_rrc_physicalChannelReconfiguration_r7_add_ext = -1;  /* BIT_STRING */
static int hf_rrc_v780NonCriticalExtensions_04 = -1;  /* T_v780NonCriticalExtensions_04 */
static int hf_rrc_physicalChannelReconfiguration_v780ext = -1;  /* PhysicalChannelReconfiguration_v780ext_IEs */
static int hf_rrc_v7d0NonCriticalExtensions_04 = -1;  /* T_v7d0NonCriticalExtensions_04 */
static int hf_rrc_physicalChannelReconfiguration_v7d0ext = -1;  /* PhysicalChannelReconfiguration_v7d0ext_IEs */
static int hf_rrc_v7f0NonCriticalExtensions_05 = -1;  /* T_v7f0NonCriticalExtensions_05 */
static int hf_rrc_physicalChannelReconfiguration_v7f0ext = -1;  /* PhysicalChannelReconfiguration_v7f0ext_IEs */
static int hf_rrc_v7g0NonCriticalExtensions_06 = -1;  /* T_v7g0NonCriticalExtensions_06 */
static int hf_rrc_physicalChannelReconfiguration_v7g0ext = -1;  /* PhysicalChannelReconfiguration_v7g0ext_IEs */
static int hf_rrc_nonCriticalExtensions_74 = -1;  /* T_nonCriticalExtensions_74 */
static int hf_rrc_criticalExtensions_58 = -1;     /* T_criticalExtensions_57 */
static int hf_rrc_r8_06 = -1;                     /* T_r8_06 */
static int hf_rrc_physicalChannelReconfiguration_r8 = -1;  /* PhysicalChannelReconfiguration_r8_IEs */
static int hf_rrc_physicalChannelReconfiguration_r8_add_ext = -1;  /* BIT_STRING */
static int hf_rrc_v7d0NonCriticalExtensions_05 = -1;  /* T_v7d0NonCriticalExtensions_05 */
static int hf_rrc_v7f0NonCriticalExtensions_06 = -1;  /* T_v7f0NonCriticalExtensions_06 */
static int hf_rrc_v890NonCriticalExtensions_04 = -1;  /* T_v890NonCriticalExtensions_04 */
static int hf_rrc_physicalChannelReconfiguration_v890ext = -1;  /* PhysicalChannelReconfiguration_v890ext_IEs */
static int hf_rrc_v7g0NonCriticalExtensions_07 = -1;  /* T_v7g0NonCriticalExtensions_07 */
static int hf_rrc_v8a0NonCriticalExtensions_04 = -1;  /* T_v8a0NonCriticalExtensions_04 */
static int hf_rrc_physicalChannelReconfiguration_v8a0ext = -1;  /* PhysicalChannelReconfiguration_v8a0ext_IEs */
static int hf_rrc_nonCriticalExtensions_75 = -1;  /* T_nonCriticalExtensions_75 */
static int hf_rrc_criticalExtensions_59 = -1;     /* T_criticalExtensions_58 */
static int hf_rrc_r9_05 = -1;                     /* T_r9_05 */
static int hf_rrc_physicalChannelReconfiguration_r9 = -1;  /* PhysicalChannelReconfiguration_r9_IEs */
static int hf_rrc_physicalChannelReconfiguration_r9_add_ext = -1;  /* BIT_STRING */
static int hf_rrc_v950NonCriticalExtensions_03 = -1;  /* T_v950NonCriticalExtensions_03 */
static int hf_rrc_physicalChannelReconfiguration_v950ext = -1;  /* PhysicalChannelReconfiguration_v950ext_IEs */
static int hf_rrc_nonCriticalExtensions_76 = -1;  /* T_nonCriticalExtensions_76 */
static int hf_rrc_criticalExtensions_60 = -1;     /* T_criticalExtensions_59 */
static int hf_rrc_r10_05 = -1;                    /* T_r10_05 */
static int hf_rrc_physicalChannelReconfiguration_r10 = -1;  /* PhysicalChannelReconfiguration_r10_IEs */
static int hf_rrc_physicalChannelReconfiguration_r10_add_ext = -1;  /* BIT_STRING */
static int hf_rrc_vbxyNonCriticalExtensions_08 = -1;  /* T_vbxyNonCriticalExtensions_08 */
static int hf_rrc_physicalChannelReconfiguration_vbxyext = -1;  /* PhysicalChannelReconfiguration_vbxyext_IEs */
static int hf_rrc_nonCriticalExtensions_77 = -1;  /* T_nonCriticalExtensions_77 */
static int hf_rrc_criticalExtensions_61 = -1;     /* T_criticalExtensions_60 */
static int hf_rrc_r11_06 = -1;                    /* T_r11_06 */
static int hf_rrc_physicalChannelReconfiguration_r11 = -1;  /* PhysicalChannelReconfiguration_r11_IEs */
static int hf_rrc_physicalChannelReconfiguration_r11_add_ext = -1;  /* BIT_STRING */
static int hf_rrc_nonCriticalExtensions_78 = -1;  /* T_nonCriticalExtensions_78 */
static int hf_rrc_criticalExtensions_62 = -1;     /* T_criticalExtensions_61 */
static int hf_rrc_ul_ChannelRequirement_03 = -1;  /* UL_ChannelRequirementWithCPCH_SetID */
static int hf_rrc_modeSpecificInfo_12 = -1;       /* T_modeSpecificInfo_12 */
static int hf_rrc_fdd_18 = -1;                    /* T_fdd_18 */
static int hf_rrc_ul_ChannelRequirement_04 = -1;  /* UL_ChannelRequirementWithCPCH_SetID_r4 */
static int hf_rrc_modeSpecificInfo_13 = -1;       /* T_modeSpecificInfo_13 */
static int hf_rrc_fdd_19 = -1;                    /* T_fdd_19 */
static int hf_rrc_ul_ChannelRequirement_05 = -1;  /* UL_ChannelRequirementWithCPCH_SetID_r5 */
static int hf_rrc_modeSpecificInfo_14 = -1;       /* T_modeSpecificInfo_14 */
static int hf_rrc_fdd_20 = -1;                    /* T_fdd_20 */
static int hf_rrc_delayRestrictionFlag = -1;      /* DelayRestrictionFlag */
static int hf_rrc_postVerificationPeriod_01 = -1;  /* T_postVerificationPeriod_01 */
static int hf_rrc_timingMaintainedSynchInd = -1;  /* TimingMaintainedSynchInd */
static int hf_rrc_responseToChangeOfUE_Capability = -1;  /* T_responseToChangeOfUE_Capability */
static int hf_rrc_harqInfo = -1;                  /* HARQ_Info_r7 */
static int hf_rrc_responseToChangeOfUE_Capability_01 = -1;  /* T_responseToChangeOfUE_Capability_01 */
static int hf_rrc_responseToChangeOfUE_Capability_02 = -1;  /* T_responseToChangeOfUE_Capability_02 */
static int hf_rrc_responseToChangeOfUE_Capability_03 = -1;  /* T_responseToChangeOfUE_Capability_03 */
static int hf_rrc_responseToChangeOfUE_Capability_04 = -1;  /* T_responseToChangeOfUE_Capability_04 */
static int hf_rrc_harqInfo_01 = -1;               /* HARQ_Info_r11 */
static int hf_rrc_ul_IntegProtActivationInfo = -1;  /* IntegrityProtActivationInfo */
static int hf_rrc_ul_TimingAdvance = -1;          /* UL_TimingAdvance */
static int hf_rrc_dummy_13 = -1;                  /* RB_ActivationTimeInfoList */
static int hf_rrc_ul_CounterSynchronisationInfo = -1;  /* UL_CounterSynchronisationInfo */
static int hf_rrc_laterNonCriticalExtensions_24 = -1;  /* T_laterNonCriticalExtensions_24 */
static int hf_rrc_physicalChannelReconfigurationComplete_r3_add_ext = -1;  /* BIT_STRING */
static int hf_rrc_v770NonCriticalExtensions_08 = -1;  /* T_v770NonCriticalExtensions_08 */
static int hf_rrc_physicalChannelReconfigurationComplete_v770ext = -1;  /* PhysicalChannelReconfigurationComplete_v770ext_IEs */
static int hf_rrc_nonCriticalExtensions_79 = -1;  /* T_nonCriticalExtensions_79 */
static int hf_rrc_ext_UL_TimingAdvance = -1;      /* EXT_UL_TimingAdvance */
static int hf_rrc_deferredMeasurementControlReading = -1;  /* T_deferredMeasurementControlReading */
static int hf_rrc_laterNonCriticalExtensions_25 = -1;  /* T_laterNonCriticalExtensions_25 */
static int hf_rrc_physicalChannelReconfigurationFailure_r3_add_ext = -1;  /* BIT_STRING */
static int hf_rrc_nonCriticalExtensions_80 = -1;  /* T_nonCriticalExtensions_80 */
static int hf_rrc_r3_13 = -1;                     /* T_r3_13 */
static int hf_rrc_physicalSharedChannelAllocation_r3 = -1;  /* PhysicalSharedChannelAllocation_r3_IEs */
static int hf_rrc_laterNonCriticalExtensions_26 = -1;  /* T_laterNonCriticalExtensions_26 */
static int hf_rrc_physicalSharedChannelAllocation_r3_add_ext = -1;  /* BIT_STRING */
static int hf_rrc_nonCriticalExtensions_81 = -1;  /* T_nonCriticalExtensions_81 */
static int hf_rrc_later_than_r3_11 = -1;          /* T_later_than_r3_11 */
static int hf_rrc_dsch_RNTI = -1;                 /* DSCH_RNTI */
static int hf_rrc_criticalExtensions_63 = -1;     /* T_criticalExtensions_62 */
static int hf_rrc_r4_05 = -1;                     /* T_r4_05 */
static int hf_rrc_physicalSharedChannelAllocation_r4 = -1;  /* PhysicalSharedChannelAllocation_r4_IEs */
static int hf_rrc_v4d0NonCriticalExtensions_05 = -1;  /* T_v4d0NonCriticalExtensions_05 */
static int hf_rrc_physicalSharedChannelAllocation_r4_add_ext = -1;  /* BIT_STRING */
static int hf_rrc_v690NonCriticalExtensions_15 = -1;  /* T_v690NonCriticalExtensions_15 */
static int hf_rrc_physicalSharedChannelAllocation_v690ext = -1;  /* PhysicalSharedChannelAllocation_v690ext_IEs */
static int hf_rrc_v770NonCriticalExtensions_09 = -1;  /* T_v770NonCriticalExtensions_09 */
static int hf_rrc_physicalSharedChannelAllocation_v770ext = -1;  /* PhysicalSharedChannelAllocation_v770ext_IEs */
static int hf_rrc_nonCriticalExtensions_82 = -1;  /* T_nonCriticalExtensions_82 */
static int hf_rrc_criticalExtensions_64 = -1;     /* T_criticalExtensions_63 */
static int hf_rrc_ul_TimingAdvance_01 = -1;       /* UL_TimingAdvanceControl */
static int hf_rrc_pusch_CapacityAllocationInfo = -1;  /* PUSCH_CapacityAllocationInfo */
static int hf_rrc_pdsch_CapacityAllocationInfo = -1;  /* PDSCH_CapacityAllocationInfo */
static int hf_rrc_confirmRequest = -1;            /* T_confirmRequest */
static int hf_rrc_trafficVolumeReportRequest = -1;  /* INTEGER_0_255 */
static int hf_rrc_iscpTimeslotList = -1;          /* TimeslotList */
static int hf_rrc_requestPCCPCHRSCP = -1;         /* BOOLEAN */
static int hf_rrc_ul_TimingAdvance_02 = -1;       /* UL_TimingAdvanceControl_r4 */
static int hf_rrc_pusch_CapacityAllocationInfo_01 = -1;  /* PUSCH_CapacityAllocationInfo_r4 */
static int hf_rrc_pdsch_CapacityAllocationInfo_01 = -1;  /* PDSCH_CapacityAllocationInfo_r4 */
static int hf_rrc_confirmRequest_01 = -1;         /* T_confirmRequest_01 */
static int hf_rrc_iscpTimeslotList_01 = -1;       /* TimeslotList_r4 */
static int hf_rrc_ul_TimingAdvance_03 = -1;       /* UL_TimingAdvanceControl_r7 */
static int hf_rrc_pusch_CapacityAllocationInfo_02 = -1;  /* PUSCH_CapacityAllocationInfo_r7 */
static int hf_rrc_pdsch_CapacityAllocationInfo_02 = -1;  /* PDSCH_CapacityAllocationInfo_r7 */
static int hf_rrc_trafficVolume = -1;             /* TrafficVolumeMeasuredResultsList */
static int hf_rrc_timeslotListWithISCP = -1;      /* TimeslotListWithISCP */
static int hf_rrc_primaryCCPCH_RSCP = -1;         /* PrimaryCCPCH_RSCP */
static int hf_rrc_allocationConfirmation = -1;    /* T_allocationConfirmation */
static int hf_rrc_pdschConfirmation = -1;         /* PDSCH_Identity */
static int hf_rrc_puschConfirmation = -1;         /* PUSCH_Identity */
static int hf_rrc_protocolErrorIndicator = -1;    /* ProtocolErrorIndicatorWithMoreInfo */
static int hf_rrc_laterNonCriticalExtensions_27 = -1;  /* T_laterNonCriticalExtensions_27 */
static int hf_rrc_puschCapacityRequest_r3_add_ext = -1;  /* BIT_STRING */
static int hf_rrc_v590NonCriticalExtensions_13 = -1;  /* T_v590NonCriticalExtensions_13 */
static int hf_rrc_puschCapacityRequest_v590ext = -1;  /* PUSCHCapacityRequest_v590ext */
static int hf_rrc_nonCriticalExtensions_83 = -1;  /* T_nonCriticalExtensions_83 */
static int hf_rrc_primaryCCPCH_RSCP_delta = -1;   /* DeltaRSCP */
static int hf_rrc_r3_14 = -1;                     /* T_r3_14 */
static int hf_rrc_radioBearerReconfiguration_r3 = -1;  /* RadioBearerReconfiguration_r3_IEs */
static int hf_rrc_v3aoNonCriticalExtensions = -1;  /* T_v3aoNonCriticalExtensions */
static int hf_rrc_radioBearerReconfiguration_v3a0ext = -1;  /* RadioBearerReconfiguration_v3a0ext */
static int hf_rrc_laterNonCriticalExtensions_28 = -1;  /* T_laterNonCriticalExtensions_28 */
static int hf_rrc_radioBearerReconfiguration_r3_add_ext = -1;  /* BIT_STRING */
static int hf_rrc_v4b0NonCriticalExtensions_07 = -1;  /* T_v4b0NonCriticalExtensions_07 */
static int hf_rrc_radioBearerReconfiguration_v4b0ext = -1;  /* RadioBearerReconfiguration_v4b0ext_IEs */
static int hf_rrc_v590NonCriticalExtensions_14 = -1;  /* T_v590NonCriticalExtensions_14 */
static int hf_rrc_radioBearerReconfiguration_v590ext = -1;  /* RadioBearerReconfiguration_v590ext_IEs */
static int hf_rrc_v5d0NonCriticalExtenstions_06 = -1;  /* T_v5d0NonCriticalExtenstions_06 */
static int hf_rrc_radioBearerReconfiguration_v5d0ext = -1;  /* RadioBearerReconfiguration_v5d0ext_IEs */
static int hf_rrc_v690NonCriticalExtensions_16 = -1;  /* T_v690NonCriticalExtensions_16 */
static int hf_rrc_radioBearerReconfiguration_v690ext = -1;  /* RadioBearerReconfiguration_v690ext_IEs */
static int hf_rrc_v770NonCriticalExtensions_10 = -1;  /* T_v770NonCriticalExtensions_10 */
static int hf_rrc_radioBearerReconfiguration_v770ext = -1;  /* RadioBearerReconfiguration_v770ext_IEs */
static int hf_rrc_nonCriticalExtensions_84 = -1;  /* T_nonCriticalExtensions_84 */
static int hf_rrc_later_than_r3_12 = -1;          /* T_later_than_r3_12 */
static int hf_rrc_criticalExtensions_65 = -1;     /* T_criticalExtensions_64 */
static int hf_rrc_r4_06 = -1;                     /* T_r4_06 */
static int hf_rrc_radioBearerReconfiguration_r4 = -1;  /* RadioBearerReconfiguration_r4_IEs */
static int hf_rrc_v4d0NonCriticalExtensions_06 = -1;  /* T_v4d0NonCriticalExtensions_06 */
static int hf_rrc_radioBearerReconfiguration_r4_add_ext = -1;  /* BIT_STRING */
static int hf_rrc_v590NonCriticalExtensions_15 = -1;  /* T_v590NonCriticalExtensions_15 */
static int hf_rrc_v5d0NonCriticalExtenstions_07 = -1;  /* T_v5d0NonCriticalExtenstions_07 */
static int hf_rrc_v690NonCriticalExtensions_17 = -1;  /* T_v690NonCriticalExtensions_17 */
static int hf_rrc_v770NonCriticalExtensions_11 = -1;  /* T_v770NonCriticalExtensions_11 */
static int hf_rrc_nonCriticalExtensions_85 = -1;  /* T_nonCriticalExtensions_85 */
static int hf_rrc_criticalExtensions_66 = -1;     /* T_criticalExtensions_65 */
static int hf_rrc_r5_05 = -1;                     /* T_r5_05 */
static int hf_rrc_radioBearerReconfiguration_r5 = -1;  /* RadioBearerReconfiguration_r5_IEs */
static int hf_rrc_radioBearerReconfiguration_r5_add_ext = -1;  /* BIT_STRING */
static int hf_rrc_v5d0NonCriticalExtenstions_08 = -1;  /* T_v5d0NonCriticalExtenstions_08 */
static int hf_rrc_v690NonCriticalExtensions_18 = -1;  /* T_v690NonCriticalExtensions_18 */
static int hf_rrc_v770NonCriticalExtensions_12 = -1;  /* T_v770NonCriticalExtensions_12 */
static int hf_rrc_nonCriticalExtensions_86 = -1;  /* T_nonCriticalExtensions_86 */
static int hf_rrc_criticalExtensions_67 = -1;     /* T_criticalExtensions_66 */
static int hf_rrc_r6_07 = -1;                     /* T_r6_07 */
static int hf_rrc_radioBearerReconfiguration_r6 = -1;  /* RadioBearerReconfiguration_r6_IEs */
static int hf_rrc_radioBearerReconfiguration_r6_add_ext = -1;  /* BIT_STRING */
static int hf_rrc_v6b0NonCriticalExtensions_07 = -1;  /* T_v6b0NonCriticalExtensions_06 */
static int hf_rrc_radioBearerReconfiguration_v6b0ext = -1;  /* RadioBearerReconfiguration_v6b0ext_IEs */
static int hf_rrc_v6f0NonCriticalExtensions = -1;  /* T_v6f0NonCriticalExtensions */
static int hf_rrc_radioBearerRconfiguration_v6f0ext = -1;  /* RadioBearerReconfiguration_v6f0ext_IEs */
static int hf_rrc_v770NonCriticalExtensions_13 = -1;  /* T_v770NonCriticalExtensions_13 */
static int hf_rrc_nonCriticalExtensions_87 = -1;  /* T_nonCriticalExtensions_87 */
static int hf_rrc_criticalExtensions_68 = -1;     /* T_criticalExtensions_67 */
static int hf_rrc_r7_06 = -1;                     /* T_r7_06 */
static int hf_rrc_radioBearerReconfiguration_r7 = -1;  /* RadioBearerReconfiguration_r7_IEs */
static int hf_rrc_v780NonCriticalExtensions_05 = -1;  /* T_v780NonCriticalExtensions_05 */
static int hf_rrc_radioBearerReconfiguration_v780ext = -1;  /* RadioBearerReconfiguration_v780ext_IEs */
static int hf_rrc_v790NonCriticalExtensions = -1;  /* T_v790NonCriticalExtensions */
static int hf_rrc_radioBearerReconfiguration_v790ext = -1;  /* RadioBearerReconfiguration_v790ext_IEs */
static int hf_rrc_v7d0NonCriticalExtensions_06 = -1;  /* T_v7d0NonCriticalExtensions_06 */
static int hf_rrc_radioBearerReconfiguration_v7d0ext = -1;  /* RadioBearerReconfiguration_v7d0ext_IEs */
static int hf_rrc_v7f0NonCriticalExtensions_07 = -1;  /* T_v7f0NonCriticalExtensions_07 */
static int hf_rrc_radioBearerReconfiguration_v7f0ext = -1;  /* RadioBearerReconfiguration_v7f0ext_IEs */
static int hf_rrc_v7g0NonCriticalExtensions_08 = -1;  /* T_v7g0NonCriticalExtensions_08 */
static int hf_rrc_radioBearerReconfiguration_v7g0ext = -1;  /* RadioBearerReconfiguration_v7g0ext_IEs */
static int hf_rrc_nonCriticalExtensions_88 = -1;  /* T_nonCriticalExtensions_88 */
static int hf_rrc_criticalExtensions_69 = -1;     /* T_criticalExtensions_68 */
static int hf_rrc_r8_07 = -1;                     /* T_r8_07 */
static int hf_rrc_radioBearerReconfiguration_r8 = -1;  /* RadioBearerReconfiguration_r8_IEs */
static int hf_rrc_radioBearerReconfiguration_r8_add_ext = -1;  /* BIT_STRING */
static int hf_rrc_v7d0NonCriticalExtensions_07 = -1;  /* T_v7d0NonCriticalExtensions_07 */
static int hf_rrc_v7f0NonCriticalExtensions_08 = -1;  /* T_v7f0NonCriticalExtensions_08 */
static int hf_rrc_v890NonCriticalExtensions_05 = -1;  /* T_v890NonCriticalExtensions_05 */
static int hf_rrc_radioBearerReconfiguration_v890ext = -1;  /* RadioBearerReconfiguration_v890ext_IEs */
static int hf_rrc_v7g0NonCriticalExtensions_09 = -1;  /* T_v7g0NonCriticalExtensions_09 */
static int hf_rrc_v8a0NonCriticalExtensions_05 = -1;  /* T_v8a0NonCriticalExtensions_05 */
static int hf_rrc_radioBearerReconfiguration_v8a0ext = -1;  /* RadioBearerReconfiguration_v8a0ext_IEs */
static int hf_rrc_nonCriticalExtensions_89 = -1;  /* T_nonCriticalExtensions_89 */
static int hf_rrc_criticalExtensions_70 = -1;     /* T_criticalExtensions_69 */
static int hf_rrc_r9_06 = -1;                     /* T_r9_06 */
static int hf_rrc_radioBearerReconfiguration_r9 = -1;  /* RadioBearerReconfiguration_r9_IEs */
static int hf_rrc_radioBearerReconfiguration_r9_add_ext = -1;  /* BIT_STRING */
static int hf_rrc_v950NonCriticalExtensions_04 = -1;  /* T_v950NonCriticalExtensions_04 */
static int hf_rrc_radioBearerReconfiguration_v950ext = -1;  /* RadioBearerReconfiguration_v950ext_IEs */
static int hf_rrc_v9c0NonCriticalExtensions_04 = -1;  /* T_v9c0NonCriticalExtensions_04 */
static int hf_rrc_radioBearerReconfiguration_v9c0ext = -1;  /* RadioBearerReconfiguration_v9c0ext_IEs */
static int hf_rrc_nonCriticalExtensions_90 = -1;  /* T_nonCriticalExtensions_90 */
static int hf_rrc_criticalExtensions_71 = -1;     /* T_criticalExtensions_70 */
static int hf_rrc_r10_06 = -1;                    /* T_r10_06 */
static int hf_rrc_radioBearerReconfiguration_r10 = -1;  /* RadioBearerReconfiguration_r10_IEs */
static int hf_rrc_radioBearerReconfiguration_r10_add_ext = -1;  /* BIT_STRING */
static int hf_rrc_v9c0NonCriticalExtensions_05 = -1;  /* T_v9c0NonCriticalExtensions_05 */
static int hf_rrc_vaa0NonCriticalExtensions_01 = -1;  /* T_vaa0NonCriticalExtensions_01 */
static int hf_rrc_radioBearerReconfiguration_vaa0ext = -1;  /* RadioBearerReconfiguration_vaa0ext_IEs */
static int hf_rrc_vbxyNonCriticalExtensions_09 = -1;  /* T_vbxyNonCriticalExtensions_09 */
static int hf_rrc_radioBearerReconfiguration_vbxyext = -1;  /* RadioBearerReconfiguration_vbxyext_IEs */
static int hf_rrc_nonCriticalExtensions_91 = -1;  /* T_nonCriticalExtensions_91 */
static int hf_rrc_criticalExtensions_72 = -1;     /* T_criticalExtensions_71 */
static int hf_rrc_r11_07 = -1;                    /* T_r11_07 */
static int hf_rrc_radioBearerReconfiguration_r11 = -1;  /* RadioBearerReconfiguration_r11_IEs */
static int hf_rrc_radioBearerReconfiguration_r11_add_ext = -1;  /* BIT_STRING */
static int hf_rrc_nonCriticalExtensions_92 = -1;  /* T_nonCriticalExtensions_92 */
static int hf_rrc_criticalExtensions_73 = -1;     /* T_criticalExtensions_72 */
static int hf_rrc_umRlc_ReEstablishmentRBList = -1;  /* RB_IdentityList */
static int hf_rrc_rab_InformationReconfigList = -1;  /* RAB_InformationReconfigList */
static int hf_rrc_dummy_14 = -1;                  /* T_dummy_01 */
static int hf_rrc_fdd_21 = -1;                    /* T_fdd_21 */
static int hf_rrc_dummy1 = -1;                    /* CPCH_SetID */
static int hf_rrc_dl_AddReconfTransChInfoList_07 = -1;  /* DL_AddReconfTransChInfo2List */
static int hf_rrc_modeSpecificPhysChInfo_03 = -1;  /* T_modeSpecificPhysChInfo_03 */
static int hf_rrc_fdd_22 = -1;                    /* T_fdd_22 */
static int hf_rrc_dummy_15 = -1;                  /* T_dummy_02 */
static int hf_rrc_fdd_23 = -1;                    /* T_fdd_23 */
static int hf_rrc_modeSpecificPhysChInfo_04 = -1;  /* T_modeSpecificPhysChInfo_04 */
static int hf_rrc_fdd_24 = -1;                    /* T_fdd_24 */
static int hf_rrc_specificationMode_09 = -1;      /* T_specificationMode_09 */
static int hf_rrc_complete_09 = -1;               /* T_complete_09 */
static int hf_rrc_rb_PDCPContextRelocationList = -1;  /* RB_PDCPContextRelocationList */
static int hf_rrc_dummy_16 = -1;                  /* T_dummy_03 */
static int hf_rrc_fdd_25 = -1;                    /* T_fdd_25 */
static int hf_rrc_preconfiguration_09 = -1;       /* T_preconfiguration_09 */
static int hf_rrc_preConfigMode_09 = -1;          /* T_preConfigMode_09 */
static int hf_rrc_defaultConfig_09 = -1;          /* T_defaultConfig_09 */
static int hf_rrc_modeSpecificPhysChInfo_05 = -1;  /* T_modeSpecificPhysChInfo_05 */
static int hf_rrc_fdd_26 = -1;                    /* T_fdd_26 */
static int hf_rrc_specificationMode_10 = -1;      /* T_specificationMode_10 */
static int hf_rrc_complete_10 = -1;               /* T_complete_10 */
static int hf_rrc_preconfiguration_10 = -1;       /* T_preconfiguration_10 */
static int hf_rrc_preConfigMode_10 = -1;          /* T_preConfigMode_10 */
static int hf_rrc_defaultConfig_10 = -1;          /* T_defaultConfig_10 */
static int hf_rrc_postVerificationPeriod_02 = -1;  /* T_postVerificationPeriod_02 */
static int hf_rrc_specificationMode_11 = -1;      /* T_specificationMode_11 */
static int hf_rrc_complete_11 = -1;               /* T_complete_11 */
static int hf_rrc_rab_InformationMBMSPtpList = -1;  /* RAB_InformationMBMSPtpList */
static int hf_rrc_responseToChangeOfUE_Capability_05 = -1;  /* T_responseToChangeOfUE_Capability_05 */
static int hf_rrc_specificationMode_12 = -1;      /* T_specificationMode_12 */
static int hf_rrc_complete_12 = -1;               /* T_complete_12 */
static int hf_rrc_preconfiguration_11 = -1;       /* T_preconfiguration_11 */
static int hf_rrc_preConfigMode_11 = -1;          /* T_preConfigMode_11 */
static int hf_rrc_defaultConfig_11 = -1;          /* T_defaultConfig_11 */
static int hf_rrc_radioBearerReconfiguration_r7_add_ext = -1;  /* BIT_STRING */
static int hf_rrc_responseToChangeOfUE_Capability_06 = -1;  /* T_responseToChangeOfUE_Capability_06 */
static int hf_rrc_specificationMode_13 = -1;      /* T_specificationMode_13 */
static int hf_rrc_complete_13 = -1;               /* T_complete_13 */
static int hf_rrc_rab_InformationReconfigList_01 = -1;  /* RAB_InformationReconfigList_r8 */
static int hf_rrc_preconfiguration_12 = -1;       /* T_preconfiguration_12 */
static int hf_rrc_preConfigMode_12 = -1;          /* T_preConfigMode_12 */
static int hf_rrc_defaultConfig_12 = -1;          /* T_defaultConfig_12 */
static int hf_rrc_responseToChangeOfUE_Capability_07 = -1;  /* T_responseToChangeOfUE_Capability_07 */
static int hf_rrc_specificationMode_14 = -1;      /* T_specificationMode_14 */
static int hf_rrc_complete_14 = -1;               /* T_complete_14 */
static int hf_rrc_preconfiguration_13 = -1;       /* T_preconfiguration_13 */
static int hf_rrc_preConfigMode_13 = -1;          /* T_preConfigMode_13 */
static int hf_rrc_defaultConfig_13 = -1;          /* T_defaultConfig_13 */
static int hf_rrc_responseToChangeOfUE_Capability_08 = -1;  /* T_responseToChangeOfUE_Capability_08 */
static int hf_rrc_specificationMode_15 = -1;      /* T_specificationMode_15 */
static int hf_rrc_complete_15 = -1;               /* T_complete_15 */
static int hf_rrc_preconfiguration_14 = -1;       /* T_preconfiguration_14 */
static int hf_rrc_preConfigMode_14 = -1;          /* T_preConfigMode_14 */
static int hf_rrc_defaultConfig_14 = -1;          /* T_defaultConfig_14 */
static int hf_rrc_responseToChangeOfUE_Capability_09 = -1;  /* T_responseToChangeOfUE_Capability_09 */
static int hf_rrc_specificationMode_16 = -1;      /* T_specificationMode_16 */
static int hf_rrc_complete_16 = -1;               /* T_complete_16 */
static int hf_rrc_preconfiguration_15 = -1;       /* T_preconfiguration_15 */
static int hf_rrc_preConfigMode_15 = -1;          /* T_preConfigMode_15 */
static int hf_rrc_defaultConfig_15 = -1;          /* T_defaultConfig_15 */
static int hf_rrc_laterNonCriticalExtensions_29 = -1;  /* T_laterNonCriticalExtensions_29 */
static int hf_rrc_radioBearerReconfigurationComplete_r3_add_ext = -1;  /* BIT_STRING */
static int hf_rrc_v770NonCriticalExtensions_14 = -1;  /* T_v770NonCriticalExtensions_14 */
static int hf_rrc_radioBearerReconfigurationComplete_v770ext = -1;  /* RadioBearerReconfigurationComplete_v770ext_IEs */
static int hf_rrc_nonCriticalExtensions_93 = -1;  /* T_nonCriticalExtensions_93 */
static int hf_rrc_deferredMeasurementControlReading_01 = -1;  /* T_deferredMeasurementControlReading_01 */
static int hf_rrc_potentiallySuccesfulBearerList = -1;  /* RB_IdentityList */
static int hf_rrc_laterNonCriticalExtensions_30 = -1;  /* T_laterNonCriticalExtensions_30 */
static int hf_rrc_radioBearerReconfigurationFailure_r3_add_ext = -1;  /* BIT_STRING */
static int hf_rrc_nonCriticalExtensions_94 = -1;  /* T_nonCriticalExtensions_94 */
static int hf_rrc_r3_15 = -1;                     /* T_r3_15 */
static int hf_rrc_radioBearerRelease_r3 = -1;     /* RadioBearerRelease_r3_IEs */
static int hf_rrc_v3a0NonCriticalExtensions_06 = -1;  /* T_v3a0NonCriticalExtensions_06 */
static int hf_rrc_radioBearerRelease_v3a0ext = -1;  /* RadioBearerRelease_v3a0ext */
static int hf_rrc_laterNonCriticalExtensions_31 = -1;  /* T_laterNonCriticalExtensions_31 */
static int hf_rrc_radioBearerRelease_r3_add_ext = -1;  /* BIT_STRING */
static int hf_rrc_v4b0NonCriticalExtensions_08 = -1;  /* T_v4b0NonCriticalExtensions_08 */
static int hf_rrc_radioBearerRelease_v4b0ext = -1;  /* RadioBearerRelease_v4b0ext_IEs */
static int hf_rrc_v590NonCriticalExtensions_16 = -1;  /* T_v590NonCriticalExtensions_16 */
static int hf_rrc_radioBearerRelease_v590ext = -1;  /* RadioBearerRelease_v590ext_IEs */
static int hf_rrc_v690NonCriticalExtensions_19 = -1;  /* T_v690NonCriticalExtensions_19 */
static int hf_rrc_radioBearerRelease_v690ext = -1;  /* RadioBearerRelease_v690ext_IEs */
static int hf_rrc_v770NonCriticalExtensions_15 = -1;  /* T_v770NonCriticalExtensions_15 */
static int hf_rrc_radioBearerRelease_v770ext = -1;  /* RadioBearerRelease_v770ext_IEs */
static int hf_rrc_nonCriticalExtensions_95 = -1;  /* T_nonCriticalExtensions_95 */
static int hf_rrc_later_than_r3_13 = -1;          /* T_later_than_r3_13 */
static int hf_rrc_criticalExtensions_74 = -1;     /* T_criticalExtensions_73 */
static int hf_rrc_r4_07 = -1;                     /* T_r4_07 */
static int hf_rrc_radioBearerRelease_r4 = -1;     /* RadioBearerRelease_r4_IEs */
static int hf_rrc_v4d0NonCriticalExtensions_07 = -1;  /* T_v4d0NonCriticalExtensions_07 */
static int hf_rrc_radioBearerRelease_r4_add_ext = -1;  /* BIT_STRING */
static int hf_rrc_v590NonCriticalExtensions_17 = -1;  /* T_v590NonCriticalExtensions_17 */
static int hf_rrc_v690NonCriticalExtensions_20 = -1;  /* T_v690NonCriticalExtensions_20 */
static int hf_rrc_v770NonCriticalExtensions_16 = -1;  /* T_v770NonCriticalExtensions_16 */
static int hf_rrc_nonCriticalExtensions_96 = -1;  /* T_nonCriticalExtensions_96 */
static int hf_rrc_criticalExtensions_75 = -1;     /* T_criticalExtensions_74 */
static int hf_rrc_r5_06 = -1;                     /* T_r5_06 */
static int hf_rrc_radioBearerRelease_r5 = -1;     /* RadioBearerRelease_r5_IEs */
static int hf_rrc_radioBearerRelease_r5_add_ext = -1;  /* BIT_STRING */
static int hf_rrc_v690NonCriticalExtensions_21 = -1;  /* T_v690NonCriticalExtensions_21 */
static int hf_rrc_v770NonCriticalExtensions_17 = -1;  /* T_v770NonCriticalExtensions_17 */
static int hf_rrc_nonCriticalExtensions_97 = -1;  /* T_nonCriticalExtensions_97 */
static int hf_rrc_criticalExtensions_76 = -1;     /* T_criticalExtensions_75 */
static int hf_rrc_r6_08 = -1;                     /* T_r6_08 */
static int hf_rrc_radioBearerRelease_r6 = -1;     /* RadioBearerRelease_r6_IEs */
static int hf_rrc_radioBearerRelease_r6_add_ext = -1;  /* BIT_STRING */
static int hf_rrc_v6b0NonCriticalExtensions_08 = -1;  /* T_v6b0NonCriticalExtensions_07 */
static int hf_rrc_radioBearerRelease_v6b0ext = -1;  /* RadioBearerRelease_v6b0ext_IEs */
static int hf_rrc_v770NonCriticalExtensions_18 = -1;  /* T_v770NonCriticalExtensions_18 */
static int hf_rrc_nonCriticalExtensions_98 = -1;  /* T_nonCriticalExtensions_98 */
static int hf_rrc_criticalExtensions_77 = -1;     /* T_criticalExtensions_76 */
static int hf_rrc_r7_07 = -1;                     /* T_r7_07 */
static int hf_rrc_radioBearerRelease_r7 = -1;     /* RadioBearerRelease_r7_IEs */
static int hf_rrc_radioBearerRelease_r7_add_ext = -1;  /* BIT_STRING */
static int hf_rrc_v780NonCriticalExtensions_06 = -1;  /* T_v780NonCriticalExtensions_06 */
static int hf_rrc_radioBearerRelease_v780ext = -1;  /* RadioBearerRelease_v780ext_IEs */
static int hf_rrc_v7d0NonCriticalExtensions_08 = -1;  /* T_v7d0NonCriticalExtensions_08 */
static int hf_rrc_radioBearerRelease_v7d0ext = -1;  /* RadioBearerRelease_v7d0ext_IEs */
static int hf_rrc_v7f0NonCriticalExtensions_09 = -1;  /* T_v7f0NonCriticalExtensions_09 */
static int hf_rrc_radioBearerRelease_v7f0ext = -1;  /* RadioBearerRelease_v7f0ext_IEs */
static int hf_rrc_v7g0NonCriticalExtensions_10 = -1;  /* T_v7g0NonCriticalExtensions_10 */
static int hf_rrc_radioBearerRelease_v7g0ext = -1;  /* RadioBearerRelease_v7g0ext_IEs */
static int hf_rrc_nonCriticalExtensions_99 = -1;  /* T_nonCriticalExtensions_99 */
static int hf_rrc_criticalExtensions_78 = -1;     /* T_criticalExtensions_77 */
static int hf_rrc_r8_08 = -1;                     /* T_r8_08 */
static int hf_rrc_radioBearerRelease_r8 = -1;     /* RadioBearerRelease_r8_IEs */
static int hf_rrc_radioBearerRelease_r8_add_ext = -1;  /* BIT_STRING */
static int hf_rrc_v7d0NonCriticalExtensions_09 = -1;  /* T_v7d0NonCriticalExtensions_09 */
static int hf_rrc_v7f0NonCriticalExtensions_10 = -1;  /* T_v7f0NonCriticalExtensions_10 */
static int hf_rrc_v890NonCriticalExtensions_06 = -1;  /* T_v890NonCriticalExtensions_06 */
static int hf_rrc_radioBearerRelease_v890ext = -1;  /* RadioBearerRelease_v890ext_IEs */
static int hf_rrc_v7g0NonCriticalExtensions_11 = -1;  /* T_v7g0NonCriticalExtensions_11 */
static int hf_rrc_v8a0NonCriticalExtensions_06 = -1;  /* T_v8a0NonCriticalExtensions_06 */
static int hf_rrc_radioBearerRelease_v8a0ext = -1;  /* RadioBearerRelease_v8a0ext_IEs */
static int hf_rrc_nonCriticalExtensions_100 = -1;  /* T_nonCriticalExtensions_100 */
static int hf_rrc_criticalExtensions_79 = -1;     /* T_criticalExtensions_78 */
static int hf_rrc_r9_07 = -1;                     /* T_r9_07 */
static int hf_rrc_radioBearerRelease_r9 = -1;     /* RadioBearerRelease_r9_IEs */
static int hf_rrc_radioBearerRelease_r9_add_ext = -1;  /* BIT_STRING */
static int hf_rrc_v950NonCriticalExtensions_05 = -1;  /* T_v950NonCriticalExtensions_05 */
static int hf_rrc_radioBearerRelease_v950ext = -1;  /* RadioBearerRelease_v950ext_IEs */
static int hf_rrc_v970NonCriticalExtensions_02 = -1;  /* T_v970NonCriticalExtensions_02 */
static int hf_rrc_radioBearerRelease_v970ext = -1;  /* RadioBearerRelease_v970ext_IEs */
static int hf_rrc_v9c0NonCriticalExtensions_06 = -1;  /* T_v9c0NonCriticalExtensions_06 */
static int hf_rrc_radioBearerRelease_v9c0ext = -1;  /* RadioBearerRelease_v9c0ext_IEs */
static int hf_rrc_nonCriticalExtensions_101 = -1;  /* T_nonCriticalExtensions_101 */
static int hf_rrc_criticalExtensions_80 = -1;     /* T_criticalExtensions_79 */
static int hf_rrc_r10_07 = -1;                    /* T_r10_07 */
static int hf_rrc_radioBearerRelease_r10 = -1;    /* RadioBearerRelease_r10_IEs */
static int hf_rrc_radioBearerRelease_r10_add_ext = -1;  /* BIT_STRING */
static int hf_rrc_v9c0NonCriticalExtensions_07 = -1;  /* T_v9c0NonCriticalExtensions_07 */
static int hf_rrc_vbxyNonCriticalExtensions_10 = -1;  /* T_vbxyNonCriticalExtensions_10 */
static int hf_rrc_radioBearerRelease_vbxyext = -1;  /* RadioBearerRelease_vbxyext_IEs */
static int hf_rrc_nonCriticalExtensions_102 = -1;  /* T_nonCriticalExtensions_102 */
static int hf_rrc_criticalExtensions_81 = -1;     /* T_criticalExtensions_80 */
static int hf_rrc_r11_08 = -1;                    /* T_r11_08 */
static int hf_rrc_radioBearerRelease_r11 = -1;    /* RadioBearerRelease_r11_IEs */
static int hf_rrc_radioBearerRelease_r11_add_ext = -1;  /* BIT_STRING */
static int hf_rrc_nonCriticalExtensions_103 = -1;  /* T_nonCriticalExtensions_103 */
static int hf_rrc_criticalExtensions_82 = -1;     /* T_criticalExtensions_81 */
static int hf_rrc_signallingConnectionRelIndication = -1;  /* CN_DomainIdentity */
static int hf_rrc_dummy_17 = -1;                  /* T_dummy_04 */
static int hf_rrc_fdd_27 = -1;                    /* T_fdd_27 */
static int hf_rrc_modeSpecificPhysChInfo_06 = -1;  /* T_modeSpecificPhysChInfo_06 */
static int hf_rrc_fdd_28 = -1;                    /* T_fdd_28 */
static int hf_rrc_dummy_18 = -1;                  /* T_dummy_05 */
static int hf_rrc_fdd_29 = -1;                    /* T_fdd_29 */
static int hf_rrc_modeSpecificPhysChInfo_07 = -1;  /* T_modeSpecificPhysChInfo_07 */
static int hf_rrc_fdd_30 = -1;                    /* T_fdd_30 */
static int hf_rrc_dummy_19 = -1;                  /* T_dummy_06 */
static int hf_rrc_fdd_31 = -1;                    /* T_fdd_31 */
static int hf_rrc_modeSpecificPhysChInfo_08 = -1;  /* T_modeSpecificPhysChInfo_08 */
static int hf_rrc_fdd_32 = -1;                    /* T_fdd_32 */
static int hf_rrc_postVerificationPeriod_03 = -1;  /* T_postVerificationPeriod_03 */
static int hf_rrc_mbms_RB_ListReleasedToChangeTransferMode = -1;  /* RB_InformationReleaseList */
static int hf_rrc_laterNonCriticalExtensions_32 = -1;  /* T_laterNonCriticalExtensions_32 */
static int hf_rrc_radioBearerReleaseComplete_r3_add_ext = -1;  /* BIT_STRING */
static int hf_rrc_v770NonCriticalExtensions_19 = -1;  /* T_v770NonCriticalExtensions_19 */
static int hf_rrc_radioBearerReleaseComplete_v770ext = -1;  /* RadioBearerReleaseComplete_v770ext_IEs */
static int hf_rrc_nonCriticalExtensions_104 = -1;  /* T_nonCriticalExtensions_104 */
static int hf_rrc_deferredMeasurementControlReading_02 = -1;  /* T_deferredMeasurementControlReading_02 */
static int hf_rrc_laterNonCriticalExtensions_33 = -1;  /* T_laterNonCriticalExtensions_33 */
static int hf_rrc_radioBearerReleaseFailure_r3_add_ext = -1;  /* BIT_STRING */
static int hf_rrc_nonCriticalExtensions_105 = -1;  /* T_nonCriticalExtensions_105 */
static int hf_rrc_r3_16 = -1;                     /* T_r3_16 */
static int hf_rrc_radioBearerSetup_r3 = -1;       /* RadioBearerSetup_r3_IEs */
static int hf_rrc_v3a0NonCriticalExtensions_07 = -1;  /* T_v3a0NonCriticalExtensions_07 */
static int hf_rrc_radioBearerSetup_v3a0ext = -1;  /* RadioBearerSetup_v3a0ext */
static int hf_rrc_laterNonCriticalExtensions_34 = -1;  /* T_laterNonCriticalExtensions_34 */
static int hf_rrc_radioBearerSetup_r3_add_ext = -1;  /* BIT_STRING */
static int hf_rrc_v4b0NonCriticalExtensions_09 = -1;  /* T_v4b0NonCriticalExtensions_09 */
static int hf_rrc_radioBearerSetup_v4b0ext = -1;  /* RadioBearerSetup_v4b0ext_IEs */
static int hf_rrc_v590NonCriticalExtensions_18 = -1;  /* T_v590NonCriticalExtensions_18 */
static int hf_rrc_radioBearerSetup_v590ext = -1;  /* RadioBearerSetup_v590ext_IEs */
static int hf_rrc_v5d0NonCriticalExtenstions_09 = -1;  /* T_v5d0NonCriticalExtenstions_09 */
static int hf_rrc_radioBearerSetup_v5d0ext = -1;  /* RadioBearerSetup_v5d0ext_IEs */
static int hf_rrc_v690NonCriticalExtensions_22 = -1;  /* T_v690NonCriticalExtensions_22 */
static int hf_rrc_radioBearerSetup_v690ext = -1;  /* RadioBearerSetup_v690ext_IEs */
static int hf_rrc_nonCriticalExtensions_106 = -1;  /* T_nonCriticalExtensions_106 */
static int hf_rrc_later_than_r3_14 = -1;          /* T_later_than_r3_14 */
static int hf_rrc_criticalExtensions_83 = -1;     /* T_criticalExtensions_82 */
static int hf_rrc_r4_08 = -1;                     /* T_r4_08 */
static int hf_rrc_radioBearerSetup_r4 = -1;       /* RadioBearerSetup_r4_IEs */
static int hf_rrc_v4d0NonCriticalExtensions_08 = -1;  /* T_v4d0NonCriticalExtensions_08 */
static int hf_rrc_radioBearerSetup_r4_add_ext = -1;  /* BIT_STRING */
static int hf_rrc_v590NonCriticalExtensions_19 = -1;  /* T_v590NonCriticalExtensions_19 */
static int hf_rrc_v5d0NonCriticalExtenstions_10 = -1;  /* T_v5d0NonCriticalExtenstions_10 */
static int hf_rrc_v690NonCriticalExtensions_23 = -1;  /* T_v690NonCriticalExtensions_23 */
static int hf_rrc_nonCriticalExtensions_107 = -1;  /* T_nonCriticalExtensions_107 */
static int hf_rrc_criticalExtensions_84 = -1;     /* T_criticalExtensions_83 */
static int hf_rrc_r5_07 = -1;                     /* T_r5_07 */
static int hf_rrc_radioBearerSetup_r5 = -1;       /* RadioBearerSetup_r5_IEs */
static int hf_rrc_radioBearerSetup_r5_add_ext = -1;  /* BIT_STRING */
static int hf_rrc_v5d0NonCriticalExtenstions_11 = -1;  /* T_v5d0NonCriticalExtenstions_11 */
static int hf_rrc_v690NonCriticalExtensions_24 = -1;  /* T_v690NonCriticalExtensions_24 */
static int hf_rrc_nonCriticalExtensions_108 = -1;  /* T_nonCriticalExtensions_108 */
static int hf_rrc_criticalExtensions_85 = -1;     /* T_criticalExtensions_84 */
static int hf_rrc_r6_09 = -1;                     /* T_r6_09 */
static int hf_rrc_radioBearerSetup_r6 = -1;       /* RadioBearerSetup_r6_IEs */
static int hf_rrc_radioBearerSetup_r6_add_ext = -1;  /* BIT_STRING */
static int hf_rrc_v6b0NonCriticalExtensions_09 = -1;  /* T_v6b0NonCriticalExtensions_08 */
static int hf_rrc_radioBearerSetup_v6b0ext = -1;  /* RadioBearerSetup_v6b0ext_IEs */
static int hf_rrc_nonCriticalExtensions_109 = -1;  /* T_nonCriticalExtensions_109 */
static int hf_rrc_criticalExtensions_86 = -1;     /* T_criticalExtensions_85 */
static int hf_rrc_r7_08 = -1;                     /* T_r7_08 */
static int hf_rrc_radioBearerSetup_r7 = -1;       /* RadioBearerSetup_r7_IEs */
static int hf_rrc_radioBearerSetup_r7_add_ext = -1;  /* T_radioBearerSetup_r7_add_ext */
static int hf_rrc_v780NonCriticalExtensions_07 = -1;  /* T_v780NonCriticalExtensions_07 */
static int hf_rrc_radioBearerSetup_v780ext = -1;  /* RadioBearerSetup_v780ext_IEs */
static int hf_rrc_v820NonCriticalExtensions_01 = -1;  /* T_v820NonCriticalExtensions_01 */
static int hf_rrc_radioBearerSetup_v820ext = -1;  /* RadioBearerSetup_v820ext_IEs */
static int hf_rrc_nonCriticalExtensions_110 = -1;  /* T_nonCriticalExtensions_110 */
static int hf_rrc_criticalExtensions_87 = -1;     /* T_criticalExtensions_86 */
static int hf_rrc_r8_09 = -1;                     /* T_r8_09 */
static int hf_rrc_radioBearerSetup_r8 = -1;       /* RadioBearerSetup_r8_IEs */
static int hf_rrc_radioBearerSetup_r8_add_ext = -1;  /* BIT_STRING */
static int hf_rrc_v7d0NonCriticalExtensions_10 = -1;  /* T_v7d0NonCriticalExtensions_10 */
static int hf_rrc_radioBearerSetup_v7d0ext = -1;  /* RadioBearerSetup_v7d0ext_IEs */
static int hf_rrc_v7f0NonCriticalExtensions_11 = -1;  /* T_v7f0NonCriticalExtensions_11 */
static int hf_rrc_radioBearerSetup_v7f0ext = -1;  /* RadioBearerSetup_v7f0ext_IEs */
static int hf_rrc_v890NonCriticalExtensions_07 = -1;  /* T_v890NonCriticalExtensions_07 */
static int hf_rrc_radioBearerSetup_v890ext = -1;  /* RadioBearerSetup_v890ext_IEs */
static int hf_rrc_v7g0NonCriticalExtensions_12 = -1;  /* T_v7g0NonCriticalExtensions_12 */
static int hf_rrc_radioBearerSetup_v7g0ext = -1;  /* RadioBearerSetup_v7g0ext_IEs */
static int hf_rrc_v8a0NonCriticalExtensions_07 = -1;  /* T_v8a0NonCriticalExtensions_07 */
static int hf_rrc_radioBearerSetup_v8a0ext = -1;  /* RadioBearerSetup_v8a0ext_IEs */
static int hf_rrc_nonCriticalExtensions_111 = -1;  /* T_nonCriticalExtensions_111 */
static int hf_rrc_criticalExtensions_88 = -1;     /* T_criticalExtensions_87 */
static int hf_rrc_r9_08 = -1;                     /* T_r9_08 */
static int hf_rrc_radioBearerSetup_r9 = -1;       /* RadioBearerSetup_r9_IEs */
static int hf_rrc_radioBearerSetup_r9_add_ext = -1;  /* BIT_STRING */
static int hf_rrc_v950NonCriticalExtensions_06 = -1;  /* T_v950NonCriticalExtensions_06 */
static int hf_rrc_radioBearerSetup_v950ext = -1;  /* RadioBearerSetup_v950ext_IEs */
static int hf_rrc_v9c0NonCriticalExtensions_08 = -1;  /* T_v9c0NonCriticalExtensions_08 */
static int hf_rrc_radioBearerSetup_v9c0ext = -1;  /* RadioBearerSetup_v9c0ext_IEs */
static int hf_rrc_nonCriticalExtensions_112 = -1;  /* T_nonCriticalExtensions_112 */
static int hf_rrc_criticalExtensions_89 = -1;     /* T_criticalExtensions_88 */
static int hf_rrc_r10_08 = -1;                    /* T_r10_08 */
static int hf_rrc_radioBearerSetup_r10 = -1;      /* RadioBearerSetup_r10_IEs */
static int hf_rrc_radioBearerSetup_r10_add_ext = -1;  /* BIT_STRING */
static int hf_rrc_v9c0NonCriticalExtensions_09 = -1;  /* T_v9c0NonCriticalExtensions_09 */
static int hf_rrc_vbxyNonCriticalExtensions_11 = -1;  /* T_vbxyNonCriticalExtensions_11 */
static int hf_rrc_radioBearerSetup_vbxyext = -1;  /* RadioBearerSetup_vbxyext_IEs */
static int hf_rrc_nonCriticalExtensions_113 = -1;  /* T_nonCriticalExtensions_113 */
static int hf_rrc_criticalExtensions_90 = -1;     /* T_criticalExtensions_89 */
static int hf_rrc_r11_09 = -1;                    /* T_r11_09 */
static int hf_rrc_radioBearerSetup_r11 = -1;      /* RadioBearerSetup_r11_IEs */
static int hf_rrc_radioBearerSetup_r11_add_ext = -1;  /* BIT_STRING */
static int hf_rrc_nonCriticalExtensions_114 = -1;  /* T_nonCriticalExtensions_114 */
static int hf_rrc_criticalExtensions_91 = -1;     /* T_criticalExtensions_90 */
static int hf_rrc_dummy_20 = -1;                  /* T_dummy_07 */
static int hf_rrc_fdd_33 = -1;                    /* T_fdd_33 */
static int hf_rrc_modeSpecificPhysChInfo_09 = -1;  /* T_modeSpecificPhysChInfo_09 */
static int hf_rrc_fdd_34 = -1;                    /* T_fdd_34 */
static int hf_rrc_dummy_21 = -1;                  /* T_dummy_08 */
static int hf_rrc_fdd_35 = -1;                    /* T_fdd_35 */
static int hf_rrc_modeSpecificPhysChInfo_10 = -1;  /* T_modeSpecificPhysChInfo_10 */
static int hf_rrc_fdd_36 = -1;                    /* T_fdd_36 */
static int hf_rrc_dummy_22 = -1;                  /* T_dummy_09 */
static int hf_rrc_fdd_37 = -1;                    /* T_fdd_37 */
static int hf_rrc_modeSpecificPhysChInfo_11 = -1;  /* T_modeSpecificPhysChInfo_11 */
static int hf_rrc_fdd_38 = -1;                    /* T_fdd_38 */
static int hf_rrc_postVerificationPeriod_04 = -1;  /* T_postVerificationPeriod_04 */
static int hf_rrc_rab_InformationSetupList_08 = -1;  /* RAB_InformationSetupList_r6_ext */
static int hf_rrc_specificationMode_17 = -1;      /* T_specificationMode_17 */
static int hf_rrc_complete_17 = -1;               /* T_complete_17 */
static int hf_rrc_dummy_23 = -1;                  /* T_dummy_10 */
static int hf_rrc_rab_Info_01 = -1;               /* RAB_Info_r6 */
static int hf_rrc_rb_InformationChangedList = -1;  /* RB_InformationChangedList_r6 */
static int hf_rrc_powerOffsetInfoShort = -1;      /* PowerOffsetInfoShort */
static int hf_rrc_rab_InformationSetupListExt = -1;  /* RAB_InformationSetupList_v6b0ext */
static int hf_rrc_specificationMode_18 = -1;      /* T_specificationMode_18 */
static int hf_rrc_complete_18 = -1;               /* T_complete_18 */
static int hf_rrc_dummy_24 = -1;                  /* NULL */
static int hf_rrc_v7f0NonCriticalExtensions_12 = -1;  /* T_v7f0NonCriticalExtensions_12 */
static int hf_rrc_v7g0NonCriticalExtensions_13 = -1;  /* T_v7g0NonCriticalExtensions_13 */
static int hf_rrc_nonCriticalExtensions_115 = -1;  /* T_nonCriticalExtensions_115 */
static int hf_rrc_sr_vcc_Info = -1;               /* SR_VCC_Info */
static int hf_rrc_laterNonCriticalExtensions_35 = -1;  /* T_laterNonCriticalExtensions_35 */
static int hf_rrc_radioBearerSetupComplete_r3_add_ext = -1;  /* BIT_STRING */
static int hf_rrc_v770NonCriticalExtensions_20 = -1;  /* T_v770NonCriticalExtensions_20 */
static int hf_rrc_radioBearerSetupComplete_v770ext = -1;  /* RadioBearerSetupComplete_v770ext_IEs */
static int hf_rrc_nonCriticalExtensions_116 = -1;  /* T_nonCriticalExtensions_116 */
static int hf_rrc_deferredMeasurementControlReading_03 = -1;  /* T_deferredMeasurementControlReading_03 */
static int hf_rrc_laterNonCriticalExtensions_36 = -1;  /* T_laterNonCriticalExtensions_36 */
static int hf_rrc_radioBearerSetupFailure_r3_add_ext = -1;  /* BIT_STRING */
static int hf_rrc_nonCriticalExtensions_117 = -1;  /* T_nonCriticalExtensions_117 */
static int hf_rrc_r3_17 = -1;                     /* T_r3_17 */
static int hf_rrc_rrcConnectionReject_r3 = -1;    /* RRCConnectionReject_r3_IEs */
static int hf_rrc_laterNonCriticalExtensions_37 = -1;  /* T_laterNonCriticalExtensions_37 */
static int hf_rrc_rrcConnectionReject_r3_add_ext = -1;  /* BIT_STRING */
static int hf_rrc_v690NonCriticalExtensions_25 = -1;  /* T_v690NonCriticalExtensions_25 */
static int hf_rrc_rrcConnectionReject_v690ext = -1;  /* RRCConnectionReject_v690ext_IEs */
static int hf_rrc_v6f0NonCriticalExtensions_01 = -1;  /* T_v6f0NonCriticalExtensions_01 */
static int hf_rrc_rrcConnectionReject_v6f0ext = -1;  /* RRCConnectionReject_v6f0ext_IEs */
static int hf_rrc_v860NonCriticalExtensions_10 = -1;  /* T_v860NonCriticalExtensions_10 */
static int hf_rrc_rrcConnectionReject_v860ext = -1;  /* RRCConnectionReject_v860ext_IEs */
static int hf_rrc_va40NonCriticalExtensions_05 = -1;  /* T_va40NonCriticalExtensions_05 */
static int hf_rrc_rrcConnectionReject_va40ext = -1;  /* RRCConnectionReject_va40ext_IEs */
static int hf_rrc_nonCriticalExtensions_118 = -1;  /* T_nonCriticalExtensions_118 */
static int hf_rrc_later_than_r3_15 = -1;          /* T_later_than_r3_15 */
static int hf_rrc_initialUE_Identity = -1;        /* InitialUE_Identity */
static int hf_rrc_criticalExtensions_92 = -1;     /* T_criticalExtensions_91 */
static int hf_rrc_rejectionCause = -1;            /* RejectionCause */
static int hf_rrc_redirectionInfo = -1;           /* RedirectionInfo */
static int hf_rrc_redirectionInfo_v690ext = -1;   /* GSM_TargetCellInfoList */
static int hf_rrc_countingCompletion = -1;        /* T_countingCompletion */
static int hf_rrc_redirectionInfo_01 = -1;        /* RedirectionInfo_v860ext */
static int hf_rrc_extendedWaitTime = -1;          /* ExtendedWaitTime */
static int hf_rrc_r3_18 = -1;                     /* T_r3_18 */
static int hf_rrc_rrcConnectionRelease_r3 = -1;   /* RRCConnectionRelease_r3_IEs */
static int hf_rrc_laterNonCriticalExtensions_38 = -1;  /* T_laterNonCriticalExtensions_38 */
static int hf_rrc_rrcConnectionRelease_r3_add_ext = -1;  /* BIT_STRING */
static int hf_rrc_v690NonCriticalExtensions_26 = -1;  /* T_v690NonCriticalExtensions_26 */
static int hf_rrc_rrcConnectionRelease_v690ext = -1;  /* RRCConnectionRelease_v690ext_IEs */
static int hf_rrc_v770NonCriticalExtensions_21 = -1;  /* T_v770NonCriticalExtensions_21 */
static int hf_rrc_rrcConnectionRelease_v770ext = -1;  /* RRCConnectionRelease_v770ext_IEs */
static int hf_rrc_v860NonCriticalExtensions_11 = -1;  /* T_v860NonCriticalExtensions_11 */
static int hf_rrc_rrcConnectionRelease_v860ext = -1;  /* RRCConnectionRelease_v860ext_IEs */
static int hf_rrc_va40NonCriticalExtensions_06 = -1;  /* T_va40NonCriticalExtensions_06 */
static int hf_rrc_rrcConnectionRelease_va40ext = -1;  /* RRCConnectionRelease_va40ext_IEs */
static int hf_rrc_nonCriticalExtensions_119 = -1;  /* T_nonCriticalExtensions_119 */
static int hf_rrc_later_than_r3_16 = -1;          /* T_later_than_r3_16 */
static int hf_rrc_criticalExtensions_93 = -1;     /* T_criticalExtensions_92 */
static int hf_rrc_r4_09 = -1;                     /* T_r4_09 */
static int hf_rrc_rrcConnectionRelease_r4 = -1;   /* RRCConnectionRelease_r4_IEs */
static int hf_rrc_v4d0NonCriticalExtensions_09 = -1;  /* T_v4d0NonCriticalExtensions_09 */
static int hf_rrc_rrcConnectionRelease_r4_add_ext = -1;  /* BIT_STRING */
static int hf_rrc_v690NonCriticalExtensions_27 = -1;  /* T_v690NonCriticalExtensions_27 */
static int hf_rrc_v770NonCriticalExtensions_22 = -1;  /* T_v770NonCriticalExtensions_22 */
static int hf_rrc_v860NonCriticalExtensions_12 = -1;  /* T_v860NonCriticalExtensions_12 */
static int hf_rrc_va40NonCriticalExtensions_07 = -1;  /* T_va40NonCriticalExtensions_07 */
static int hf_rrc_nonCriticalExtensions_120 = -1;  /* T_nonCriticalExtensions_120 */
static int hf_rrc_criticalExtensions_94 = -1;     /* T_criticalExtensions_93 */
static int hf_rrc_n_308 = -1;                     /* N_308 */
static int hf_rrc_releaseCause = -1;              /* ReleaseCause */
static int hf_rrc_rplmn_information = -1;         /* Rplmn_Information */
static int hf_rrc_rplmn_information_01 = -1;      /* Rplmn_Information_r4 */
static int hf_rrc_redirectionInfo_v690ext_01 = -1;  /* RedirectionInfo_r6 */
static int hf_rrc_r3_19 = -1;                     /* T_r3_19 */
static int hf_rrc_rrcConnectionRelease_CCCH_r3 = -1;  /* RRCConnectionRelease_CCCH_r3_IEs */
static int hf_rrc_laterNonCriticalExtensions_39 = -1;  /* T_laterNonCriticalExtensions_39 */
static int hf_rrc_rrcConnectionRelease_CCCH_r3_add_ext = -1;  /* BIT_STRING */
static int hf_rrc_v690NonCriticalExtensions_28 = -1;  /* T_v690NonCriticalExtensions_28 */
static int hf_rrc_rrcConnectionRelease_v690ext_01 = -1;  /* RRCConnectionRelease_CCCH_v690ext_IEs */
static int hf_rrc_v860NonCriticalExtensions_13 = -1;  /* T_v860NonCriticalExtensions_13 */
static int hf_rrc_rrcConnectionRelease_v860ext_01 = -1;  /* RRCConnectionRelease_CCCH_v860ext_IEs */
static int hf_rrc_va40NonCriticalExtensions_08 = -1;  /* T_va40NonCriticalExtensions_08 */
static int hf_rrc_nonCriticalExtensions_121 = -1;  /* T_nonCriticalExtensions_121 */
static int hf_rrc_later_than_r3_17 = -1;          /* T_later_than_r3_17 */
static int hf_rrc_criticalExtensions_95 = -1;     /* T_criticalExtensions_94 */
static int hf_rrc_r4_10 = -1;                     /* T_r4_10 */
static int hf_rrc_rrcConnectionRelease_CCCH_r4 = -1;  /* RRCConnectionRelease_CCCH_r4_IEs */
static int hf_rrc_v4d0NonCriticalExtensions_10 = -1;  /* T_v4d0NonCriticalExtensions_10 */
static int hf_rrc_rrcConnectionRelease_CCCH_r4_add_ext = -1;  /* BIT_STRING */
static int hf_rrc_v690NonCriticalExtensions_29 = -1;  /* T_v690NonCriticalExtensions_29 */
static int hf_rrc_v860NonCriticalExtensions_14 = -1;  /* T_v860NonCriticalExtensions_14 */
static int hf_rrc_va40NonCriticalExtensions_09 = -1;  /* T_va40NonCriticalExtensions_09 */
static int hf_rrc_nonCriticalExtensions_122 = -1;  /* T_nonCriticalExtensions_122 */
static int hf_rrc_criticalExtensions_96 = -1;     /* T_criticalExtensions_95 */
static int hf_rrc_groupIdentity = -1;             /* SEQUENCE_SIZE_1_maxURNTI_Group_OF_GroupReleaseInformation */
static int hf_rrc_groupIdentity_item = -1;        /* GroupReleaseInformation */
static int hf_rrc_criticalExtensions_97 = -1;     /* T_criticalExtensions_96 */
static int hf_rrc_r5_08 = -1;                     /* T_r5_08 */
static int hf_rrc_rrcConnectionRelease_CCCH_r5 = -1;  /* RRCConnectionRelease_CCCH_r5_IEs */
static int hf_rrc_rrcConnectionRelease_CCCH_r5_add_ext = -1;  /* BIT_STRING */
static int hf_rrc_v690NonCriticalExtensions_30 = -1;  /* T_v690NonCriticalExtensions_30 */
static int hf_rrc_v860NonCriticalExtensions_15 = -1;  /* T_v860NonCriticalExtensions_15 */
static int hf_rrc_va40NonCriticalExtensions_10 = -1;  /* T_va40NonCriticalExtensions_10 */
static int hf_rrc_nonCriticalExtensions_123 = -1;  /* T_nonCriticalExtensions_123 */
static int hf_rrc_criticalExtensions_98 = -1;     /* T_criticalExtensions_97 */
static int hf_rrc_rrcConnectionRelease_02 = -1;   /* RRCConnectionRelease_r3_IEs */
static int hf_rrc_rrcConnectionRelease_03 = -1;   /* RRCConnectionRelease_r4_IEs */
static int hf_rrc_errorIndication = -1;           /* FailureCauseWithProtErr */
static int hf_rrc_laterNonCriticalExtensions_40 = -1;  /* T_laterNonCriticalExtensions_40 */
static int hf_rrc_rrcConnectionReleaseComplete_r3_add_ext = -1;  /* BIT_STRING */
static int hf_rrc_nonCriticalExtensions_124 = -1;  /* T_nonCriticalExtensions_124 */
static int hf_rrc_protocolErrorIndicator_01 = -1;  /* ProtocolErrorIndicator */
static int hf_rrc_v3d0NonCriticalExtensions = -1;  /* T_v3d0NonCriticalExtensions */
static int hf_rrc_rRCConnectionRequest_v3d0ext = -1;  /* RRCConnectionRequest_v3d0ext_IEs */
static int hf_rrc_v4b0NonCriticalExtensions_10 = -1;  /* T_v4b0NonCriticalExtensions_10 */
static int hf_rrc_rrcConnectionRequest_v4b0ext = -1;  /* RRCConnectionRequest_v4b0ext_IEs */
static int hf_rrc_v590NonCriticalExtensions_20 = -1;  /* T_v590NonCriticalExtensions_20 */
static int hf_rrc_rrcConnectionRequest_v590ext = -1;  /* RRCConnectionRequest_v590ext_IEs */
static int hf_rrc_v690NonCriticalExtensions_31 = -1;  /* T_v690NonCriticalExtensions_31 */
static int hf_rrc_rrcConnectionRequest_v690ext = -1;  /* RRCConnectionRequest_v690ext_IEs */
static int hf_rrc_v6b0NonCriticalExtensions_10 = -1;  /* T_v6b0NonCriticalExtensions_09 */
static int hf_rrc_rrcConnectionRequest_v6b0ext = -1;  /* RRCConnectionRequest_v6b0ext_IEs */
static int hf_rrc_v6e0NonCriticalExtensions_01 = -1;  /* T_v6e0NonCriticalExtensions_01 */
static int hf_rrc_rrcConnectionRequest_v6e0ext = -1;  /* RRCConnectionRequest_v6e0ext_IEs */
static int hf_rrc_v770NonCriticalExtensions_23 = -1;  /* T_v770NonCriticalExtensions_23 */
static int hf_rrc_rrcConnectionRequest_v770ext = -1;  /* RRCConnectionRequest_v770ext_IEs */
static int hf_rrc_v7b0NonCriticalExtensions_01 = -1;  /* T_v7b0NonCriticalExtensions_01 */
static int hf_rrc_rrcConnectionRequest_v7b0ext = -1;  /* RRCConnectionRequest_v7b0ext_IEs */
static int hf_rrc_v860NonCriticalExtensions_16 = -1;  /* T_v860NonCriticalExtensions_16 */
static int hf_rrc_rrcConnectionRequest_v860ext = -1;  /* RRCConnectionRequest_v860ext_IEs */
static int hf_rrc_v7e0NonCriticalExtensions_01 = -1;  /* T_v7e0NonCriticalExtensions_01 */
static int hf_rrc_rrcConnectionRequest_v7e0ext = -1;  /* RRCConnectionRequest_v7e0ext_IEs */
static int hf_rrc_v7g0NonCriticalExtensions_14 = -1;  /* T_v7g0NonCriticalExtensions_14 */
static int hf_rrc_rrcConnectionRequest_v7g0ext = -1;  /* RRCConnectionRequest_v7g0ext_IEs */
static int hf_rrc_v920NonCriticalExtensions_02 = -1;  /* T_v920NonCriticalExtensions_02 */
static int hf_rrc_rrcConnectionRequest_v920ext = -1;  /* RRCConnectionRequest_v920ext_IEs */
static int hf_rrc_v940NonCriticalExtensions = -1;  /* T_v940NonCriticalExtensions */
static int hf_rrc_rrcConnectionRequest_v940ext = -1;  /* RRCConnectionRequest_v940ext_IEs */
static int hf_rrc_va40NonCriticalExtensions_11 = -1;  /* RRCConnectionRequest_NonCriticalExts_va40_IEs */
static int hf_rrc_uESpecificBehaviourInformation1idle = -1;  /* UESpecificBehaviourInformation1idle */
static int hf_rrc_predefinedConfigStatusInfo = -1;  /* BOOLEAN */
static int hf_rrc_ueCapabilityIndication = -1;    /* T_ueCapabilityIndication */
static int hf_rrc_domainIndicator = -1;           /* T_domainIndicator */
static int hf_rrc_cs_domain = -1;                 /* T_cs_domain */
static int hf_rrc_csCallType_02 = -1;             /* T_csCallType_02 */
static int hf_rrc_ps_domain = -1;                 /* NULL */
static int hf_rrc_supportForFDPCH_01 = -1;        /* T_supportForFDPCH_01 */
static int hf_rrc_hspdschReception_CellFach_01 = -1;  /* T_hspdschReception_CellFach_01 */
static int hf_rrc_mac_ehsSupport = -1;            /* T_mac_ehsSupport */
static int hf_rrc_discontinuousDpcchTransmission = -1;  /* T_discontinuousDpcchTransmission */
static int hf_rrc_supportForE_FDPCH = -1;         /* T_supportForE_FDPCH */
static int hf_rrc_supportForCSVoiceoverHSPA = -1;  /* T_supportForCSVoiceoverHSPA */
static int hf_rrc_supportOfCommonEDCH_01 = -1;    /* T_supportOfCommonEDCH_01 */
static int hf_rrc_multiCellSupport = -1;          /* T_multiCellSupport */
static int hf_rrc_pre_redirectionInfo = -1;       /* Pre_RedirectionInfo */
static int hf_rrc_supportOfMACiis_01 = -1;        /* T_supportOfMACiis_01 */
static int hf_rrc_supportOfSPSOperation_01 = -1;  /* T_supportOfSPSOperation_01 */
static int hf_rrc_supportOfDualCellMIMO = -1;     /* T_supportOfDualCellMIMO */
static int hf_rrc_systemInformationContainerStoredIndicator = -1;  /* T_systemInformationContainerStoredIndicator */
static int hf_rrc_cSFBIndication = -1;            /* T_cSFBIndication */
static int hf_rrc_rrcConnectionRequest_va40ext = -1;  /* RRCConnectionRequest_va40ext_IEs */
static int hf_rrc_v9b0NonCriticalExtensions = -1;  /* T_v9b0NonCriticalExtensions */
static int hf_rrc_rrcConnectionRequest_v9b0ext = -1;  /* RRCConnectionRequest_v9b0ext_IEs */
static int hf_rrc_vbxyNonCriticalExtensions_12 = -1;  /* T_vbxyNonCriticalExtensions_12 */
static int hf_rrc_rrcConnectionRequest_vbxyext = -1;  /* RRCConnectionRequest_vbxyext_IEs */
static int hf_rrc_nonCriticalExtensions_125 = -1;  /* T_nonCriticalExtensions_125 */
static int hf_rrc_supportOfMoreThanTwoCells = -1;  /* T_supportOfMoreThanTwoCells */
static int hf_rrc_supportOf1stFrequencyBand = -1;  /* T_supportOf1stFrequencyBand */
static int hf_rrc_supportOf2ndFrequencyBand = -1;  /* T_supportOf2ndFrequencyBand */
static int hf_rrc_r3_20 = -1;                     /* T_r3_20 */
static int hf_rrc_rrcConnectionSetup_r3 = -1;     /* RRCConnectionSetup_r3_IEs */
static int hf_rrc_laterNonCriticalExtensions_41 = -1;  /* T_laterNonCriticalExtensions_41 */
static int hf_rrc_rrcConnectionSetup_r3_add_ext = -1;  /* BIT_STRING */
static int hf_rrc_v4b0NonCriticalExtensions_11 = -1;  /* T_v4b0NonCriticalExtensions_11 */
static int hf_rrc_rrcConnectionSetup_v4b0ext = -1;  /* RRCConnectionSetup_v4b0ext_IEs */
static int hf_rrc_v590NonCriticalExtensions_21 = -1;  /* T_v590NonCriticalExtensions_21 */
static int hf_rrc_rrcConnectionSetup_v590ext = -1;  /* RRCConnectionSetup_v590ext_IEs */
static int hf_rrc_v690NonCriticalExtensions_32 = -1;  /* T_v690NonCriticalExtensions_32 */
static int hf_rrc_rrcConnectionSetup_v690ext = -1;  /* RRCConnectionSetup_v690ext_IEs */
static int hf_rrc_nonCriticalExtensions_126 = -1;  /* T_nonCriticalExtensions_126 */
static int hf_rrc_later_than_r3_18 = -1;          /* T_later_than_r3_18 */
static int hf_rrc_criticalExtensions_99 = -1;     /* T_criticalExtensions_98 */
static int hf_rrc_r4_11 = -1;                     /* T_r4_11 */
static int hf_rrc_rrcConnectionSetup_r4 = -1;     /* RRCConnectionSetup_r4_IEs */
static int hf_rrc_v4d0NonCriticalExtensions_11 = -1;  /* T_v4d0NonCriticalExtensions_11 */
static int hf_rrc_rrcConnectionSetup_r4_add_ext = -1;  /* BIT_STRING */
static int hf_rrc_v590NonCriticalExtensions_22 = -1;  /* T_v590NonCriticalExtensions_22 */
static int hf_rrc_v690NonCriticalExtensions_33 = -1;  /* T_v690NonCriticalExtensions_33 */
static int hf_rrc_nonCriticalExtensions_127 = -1;  /* T_nonCriticalExtensions_127 */
static int hf_rrc_criticalExtensions_100 = -1;    /* T_criticalExtensions_99 */
static int hf_rrc_r5_09 = -1;                     /* T_r5_09 */
static int hf_rrc_rrcConnectionSetup_r5 = -1;     /* RRCConnectionSetup_r5_IEs */
static int hf_rrc_rrcConnectionSetup_r5_add_ext = -1;  /* BIT_STRING */
static int hf_rrc_v690NonCriticalExtensions_34 = -1;  /* T_v690NonCriticalExtensions_34 */
static int hf_rrc_nonCriticalExtensions_128 = -1;  /* T_nonCriticalExtensions_128 */
static int hf_rrc_criticalExtensions_101 = -1;    /* T_criticalExtensions_100 */
static int hf_rrc_r6_10 = -1;                     /* T_r6_10 */
static int hf_rrc_rrcConnectionSetup_r6 = -1;     /* RRCConnectionSetup_r6_IEs */
static int hf_rrc_rrcConnectionSetup_r6_add_ext = -1;  /* BIT_STRING */
static int hf_rrc_v6b0NonCriticalExtensions_11 = -1;  /* T_v6b0NonCriticalExtensions_10 */
static int hf_rrc_rrcConnectionSetup_v6b0ext = -1;  /* RRCConnectionSetup_v6b0ext_IEs */
static int hf_rrc_nonCriticalExtensions_129 = -1;  /* T_nonCriticalExtensions_129 */
static int hf_rrc_criticalExtensions_102 = -1;    /* T_criticalExtensions_101 */
static int hf_rrc_r7_09 = -1;                     /* T_r7_09 */
static int hf_rrc_rrcConnectionSetup_r7 = -1;     /* RRCConnectionSetup_r7_IEs */
static int hf_rrc_rrcConnectionSetup_r7_add_ext = -1;  /* BIT_STRING */
static int hf_rrc_v780NonCriticalExtensions_08 = -1;  /* T_v780NonCriticalExtensions_08 */
static int hf_rrc_rrcConnectionSetup_v780ext = -1;  /* RRCConnectionSetup_v780ext_IEs */
static int hf_rrc_v7d0NonCriticalExtensions_11 = -1;  /* T_v7d0NonCriticalExtensions_11 */
static int hf_rrc_rrcConnectionSetup_v7d0ext = -1;  /* RRCConnectionSetup_v7d0ext_IEs */
static int hf_rrc_nonCriticalExtensions_130 = -1;  /* T_nonCriticalExtensions_130 */
static int hf_rrc_criticalExtensions_103 = -1;    /* T_criticalExtensions_102 */
static int hf_rrc_r8_10 = -1;                     /* T_r8_10 */
static int hf_rrc_rrcConnectionSetup_r8 = -1;     /* RRCConnectionSetup_r8_IEs */
static int hf_rrc_rrcConnectionSetup_r8_add_ext = -1;  /* BIT_STRING */
static int hf_rrc_v7d0NonCriticalExtensions_12 = -1;  /* T_v7d0NonCriticalExtensions_12 */
static int hf_rrc_v890NonCriticalExtensions_08 = -1;  /* T_v890NonCriticalExtensions_08 */
static int hf_rrc_rrcConnectionSetup_v890ext = -1;  /* RRCConnectionSetup_v890ext_IEs */
static int hf_rrc_v8a0NonCriticalExtensions_08 = -1;  /* T_v8a0NonCriticalExtensions_08 */
static int hf_rrc_rrcConnectionSetup_v8a0ext = -1;  /* RRCConnectionSetup_v8a0ext_IEs */
static int hf_rrc_nonCriticalExtensions_131 = -1;  /* T_nonCriticalExtensions_131 */
static int hf_rrc_criticalExtensions_104 = -1;    /* T_criticalExtensions_103 */
static int hf_rrc_r9_09 = -1;                     /* T_r9_09 */
static int hf_rrc_rrcConnectionSetup_r9 = -1;     /* RRCConnectionSetup_r9_IEs */
static int hf_rrc_rrcConnectionSetup_r9_add_ext = -1;  /* BIT_STRING */
static int hf_rrc_v950NonCriticalExtensions_07 = -1;  /* T_v950NonCriticalExtensions_07 */
static int hf_rrc_rrcConnectionSetup_v950ext = -1;  /* RRCConnectionSetup_v950ext_IEs */
static int hf_rrc_nonCriticalExtensions_132 = -1;  /* T_nonCriticalExtensions_132 */
static int hf_rrc_criticalExtensions_105 = -1;    /* T_criticalExtensions_104 */
static int hf_rrc_r10_09 = -1;                    /* T_r10_09 */
static int hf_rrc_rrcConnectionSetup_r10 = -1;    /* RRCConnectionSetup_r10_IEs */
static int hf_rrc_rrcConnectionSetup_r10_add_ext = -1;  /* BIT_STRING */
static int hf_rrc_nonCriticalExtensions_133 = -1;  /* T_nonCriticalExtensions_133 */
static int hf_rrc_criticalExtensions_106 = -1;    /* T_criticalExtensions_105 */
static int hf_rrc_r11_10 = -1;                    /* T_r11_10 */
static int hf_rrc_rrcConnectionSetup_r11 = -1;    /* RRCConnectionSetup_r11_IEs */
static int hf_rrc_rrcConnectionSetup_r11_add_ext = -1;  /* BIT_STRING */
static int hf_rrc_nonCriticalExtensions_134 = -1;  /* T_nonCriticalExtensions_134 */
static int hf_rrc_criticalExtensions_107 = -1;    /* T_criticalExtensions_106 */
static int hf_rrc_new_c_RNTI = -1;                /* C_RNTI */
static int hf_rrc_capabilityUpdateRequirement = -1;  /* CapabilityUpdateRequirement */
static int hf_rrc_srb_InformationSetupList_05 = -1;  /* SRB_InformationSetupList2 */
static int hf_rrc_capabilityUpdateRequirement_r4_ext = -1;  /* CapabilityUpdateRequirement_r4_ext */
static int hf_rrc_systemSpecificCapUpdateReq = -1;  /* SystemSpecificCapUpdateReq_v590ext */
static int hf_rrc_capabilityUpdateRequirement_01 = -1;  /* CapabilityUpdateRequirement_r4 */
static int hf_rrc_capabilityUpdateRequirement_02 = -1;  /* CapabilityUpdateRequirement_r5 */
static int hf_rrc_specificationMode_19 = -1;      /* T_specificationMode_19 */
static int hf_rrc_complete_19 = -1;               /* T_complete_19 */
static int hf_rrc_preconfiguration_16 = -1;       /* T_preconfiguration_16 */
static int hf_rrc_preConfigMode_16 = -1;          /* T_preConfigMode_16 */
static int hf_rrc_defaultConfig_16 = -1;          /* T_defaultConfig_16 */
static int hf_rrc_dl_InformationPerRL_List_08 = -1;  /* DL_InformationPerRL_List_r5bis */
static int hf_rrc_postVerificationPeriod_05 = -1;  /* T_postVerificationPeriod_05 */
static int hf_rrc_specificationMode_20 = -1;      /* T_specificationMode_20 */
static int hf_rrc_complete_20 = -1;               /* T_complete_20 */
static int hf_rrc_srb_InformationSetupList_06 = -1;  /* SRB_InformationSetupList2_r6 */
static int hf_rrc_preconfiguration_17 = -1;       /* T_preconfiguration_17 */
static int hf_rrc_preConfigMode_17 = -1;          /* T_preConfigMode_17 */
static int hf_rrc_defaultConfig_17 = -1;          /* T_defaultConfig_17 */
static int hf_rrc_capabilityUpdateRequirement_03 = -1;  /* CapabilityUpdateRequirement_r7 */
static int hf_rrc_specificationMode_21 = -1;      /* T_specificationMode_21 */
static int hf_rrc_complete_21 = -1;               /* T_complete_21 */
static int hf_rrc_srb_InformationSetupList_07 = -1;  /* SRB_InformationSetupList2_r7 */
static int hf_rrc_preconfiguration_18 = -1;       /* T_preconfiguration_18 */
static int hf_rrc_preConfigMode_18 = -1;          /* T_preConfigMode_18 */
static int hf_rrc_defaultConfig_18 = -1;          /* T_defaultConfig_18 */
static int hf_rrc_capabilityUpdateRequirement_04 = -1;  /* CapabilityUpdateRequirement_r8 */
static int hf_rrc_specificationMode_22 = -1;      /* T_specificationMode_22 */
static int hf_rrc_complete_22 = -1;               /* T_complete_22 */
static int hf_rrc_srb_InformationSetupList_08 = -1;  /* SRB_InformationSetupList2_r8 */
static int hf_rrc_preconfiguration_19 = -1;       /* T_preconfiguration_19 */
static int hf_rrc_preConfigMode_19 = -1;          /* T_preConfigMode_19 */
static int hf_rrc_defaultConfig_19 = -1;          /* T_defaultConfig_19 */
static int hf_rrc_specificationMode_23 = -1;      /* T_specificationMode_23 */
static int hf_rrc_complete_23 = -1;               /* T_complete_23 */
static int hf_rrc_preconfiguration_20 = -1;       /* T_preconfiguration_20 */
static int hf_rrc_preConfigMode_20 = -1;          /* T_preConfigMode_20 */
static int hf_rrc_defaultConfig_20 = -1;          /* T_defaultConfig_20 */
static int hf_rrc_specificationMode_24 = -1;      /* T_specificationMode_24 */
static int hf_rrc_complete_24 = -1;               /* T_complete_24 */
static int hf_rrc_preconfiguration_21 = -1;       /* T_preconfiguration_21 */
static int hf_rrc_preConfigMode_21 = -1;          /* T_preConfigMode_21 */
static int hf_rrc_defaultConfig_21 = -1;          /* T_defaultConfig_21 */
static int hf_rrc_specificationMode_25 = -1;      /* T_specificationMode_25 */
static int hf_rrc_complete_25 = -1;               /* T_complete_25 */
static int hf_rrc_preconfiguration_22 = -1;       /* T_preconfiguration_22 */
static int hf_rrc_preConfigMode_22 = -1;          /* T_preConfigMode_22 */
static int hf_rrc_defaultConfig_22 = -1;          /* T_defaultConfig_22 */
static int hf_rrc_ue_RadioAccessCapability_02 = -1;  /* UE_RadioAccessCapability */
static int hf_rrc_ue_RATSpecificCapability = -1;  /* InterRAT_UE_RadioAccessCapabilityList */
static int hf_rrc_v370NonCriticalExtensions = -1;  /* T_v370NonCriticalExtensions */
static int hf_rrc_rrcConnectionSetupComplete_v370ext = -1;  /* RRCConnectionSetupComplete_v370ext */
static int hf_rrc_v380NonCriticalExtensions = -1;  /* T_v380NonCriticalExtensions */
static int hf_rrc_rrcConnectionSetupComplete_v380ext = -1;  /* RRCConnectionSetupComplete_v380ext_IEs */
static int hf_rrc_v3a0NonCriticalExtensions_08 = -1;  /* T_v3a0NonCriticalExtensions_08 */
static int hf_rrc_rrcConnectionSetupComplete_v3a0ext = -1;  /* RRCConnectionSetupComplete_v3a0ext_IEs */
static int hf_rrc_laterNonCriticalExtensions_42 = -1;  /* T_laterNonCriticalExtensions_42 */
static int hf_rrc_rrcConnectionSetupComplete_r3_add_ext = -1;  /* T_rrcConnectionSetupComplete_r3_add_ext */
static int hf_rrc_v3g0NonCriticalExtensions_01 = -1;  /* T_v3g0NonCriticalExtensions_01 */
static int hf_rrc_rrcConnectionSetupComplete_v3g0ext = -1;  /* RRCConnectionSetupComplete_v3g0ext_IEs */
static int hf_rrc_v4b0NonCriticalExtensions_12 = -1;  /* T_v4b0NonCriticalExtensions_12 */
static int hf_rrc_rrcConnectionSetupComplete_v4b0ext = -1;  /* RRCConnectionSetupComplete_v4b0ext_IEs */
static int hf_rrc_v590NonCriticalExtensions_23 = -1;  /* T_v590NonCriticalExtensions_23 */
static int hf_rrc_rrcConnectionSetupComplete_v590ext = -1;  /* RRCConnectionSetupComplete_v590ext_IEs */
static int hf_rrc_v5c0NonCriticalExtensions = -1;  /* T_v5c0NonCriticalExtensions */
static int hf_rrc_rrcConnectionSetupComplete_v5c0ext = -1;  /* RRCConnectionSetupComplete_v5c0ext_IEs */
static int hf_rrc_v690NonCriticalExtensions_35 = -1;  /* T_v690NonCriticalExtensions_35 */
static int hf_rrc_rrcConnectionSetupComplete_v690ext = -1;  /* RRCConnectionSetupComplete_v690ext_IEs */
static int hf_rrc_v770NonCriticalExtensions_24 = -1;  /* T_v770NonCriticalExtensions_24 */
static int hf_rrc_rrcConectionSetupComplete_v770ext = -1;  /* RRCConnectionSetupComplete_v770ext_IEs */
static int hf_rrc_va40NonCriticalExtensions_12 = -1;  /* T_va40NonCriticalExtensions_11 */
static int hf_rrc_rrcConectionSetupComplete_va40ext = -1;  /* RRCConnectionSetupComplete_va40ext_IEs */
static int hf_rrc_vbxyNonCriticalExtensions_13 = -1;  /* T_vbxyNonCriticalExtensions_13 */
static int hf_rrc_rrcConnectionSetupComplete_vbxyext = -1;  /* RRCConnectionSetupComplete_vbxyext_IEs */
static int hf_rrc_nonCriticalExtensions_135 = -1;  /* T_nonCriticalExtensions_135 */
static int hf_rrc_ue_RadioAccessCapability_v370ext = -1;  /* UE_RadioAccessCapability_v370ext */
static int hf_rrc_rrcConnectionSetupComplete_v650ext = -1;  /* RRCConnectionSetupComplete_v650ext_IEs */
static int hf_rrc_v680NonCriticalExtensions = -1;  /* T_v680NonCriticalExtensions */
static int hf_rrc_rrcConnectionSetupComplete_v680ext = -1;  /* RRCConnectionSetupComplete_v680ext_IEs */
static int hf_rrc_v7e0NonCriticalExtensions_02 = -1;  /* T_v7e0NonCriticalExtensions_02 */
static int hf_rrc_rrcConnectionSetupComplete_v7e0ext = -1;  /* RRCConnectionSetupComplete_v7e0ext_IEs */
static int hf_rrc_v7f0NonCriticalExtensions_13 = -1;  /* T_v7f0NonCriticalExtensions_13 */
static int hf_rrc_rrcConnectionSetupComplete_v7f0ext = -1;  /* RRCConnectionSetupComplete_v7f0ext_IEs */
static int hf_rrc_va40NonCriticalExtensions_13 = -1;  /* T_va40NonCriticalExtensions_12 */
static int hf_rrc_rrcConnectionSetupCompleteBand_va40ext = -1;  /* RRCConnectionSetupCompleteBand_va40ext_IEs */
static int hf_rrc_nonCriticalExtensions_136 = -1;  /* T_nonCriticalExtensions_136 */
static int hf_rrc_ue_RadioAccessCapability_v4b0ext = -1;  /* UE_RadioAccessCapability_v4b0ext */
static int hf_rrc_ue_RadioAccessCapability_v590ext = -1;  /* UE_RadioAccessCapability_v590ext */
static int hf_rrc_ue_RATSpecificCapability_v590ext = -1;  /* InterRAT_UE_RadioAccessCapability_v590ext */
static int hf_rrc_ue_RadioAccessCapability_v5c0ext = -1;  /* UE_RadioAccessCapability_v5c0ext */
static int hf_rrc_ue_RadioAccessCapability_v680ext = -1;  /* UE_RadioAccessCapability_v680ext */
static int hf_rrc_ueCapabilityContainer = -1;     /* T_ueCapabilityContainer */
static int hf_rrc_deferredMeasurementControlReading_04 = -1;  /* T_deferredMeasurementControlReading_04 */
static int hf_rrc_loggedMeasAvailable_03 = -1;    /* T_loggedMeasAvailable_03 */
static int hf_rrc_loggedANRResultsAvailable_02 = -1;  /* T_loggedANRResultsAvailable_02 */
static int hf_rrc_loggedConnectionEstablishmentFailureInfoAvailable_01 = -1;  /* T_loggedConnectionEstablishmentFailureInfoAvailable_01 */
static int hf_rrc_r3_21 = -1;                     /* T_r3_21 */
static int hf_rrc_rRC_FailureInfo_r3 = -1;        /* RRC_FailureInfo_r3_IEs */
static int hf_rrc_laterNonCriticalExtensions_43 = -1;  /* T_laterNonCriticalExtensions_43 */
static int hf_rrc_rrc_FailureInfo_r3_add_ext = -1;  /* BIT_STRING */
static int hf_rrc_nonCriticalExtensions_137 = -1;  /* T_nonCriticalExtensions_137 */
static int hf_rrc_criticalExtensions_108 = -1;    /* T_criticalExtensions_107 */
static int hf_rrc_failureCauseWithProtErr = -1;   /* FailureCauseWithProtErr */
static int hf_rrc_protocolErrorInformation = -1;  /* ProtocolErrorMoreInformation */
static int hf_rrc_laterNonCriticalExtensions_44 = -1;  /* T_laterNonCriticalExtensions_44 */
static int hf_rrc_rrcStatus_r3_add_ext = -1;      /* BIT_STRING */
static int hf_rrc_nonCriticalExtensions_138 = -1;  /* T_nonCriticalExtensions_138 */
static int hf_rrc_r3_22 = -1;                     /* T_r3_22 */
static int hf_rrc_securityModeCommand_r3 = -1;    /* SecurityModeCommand_r3_IEs */
static int hf_rrc_laterNonCriticalExtensions_45 = -1;  /* T_laterNonCriticalExtensions_45 */
static int hf_rrc_securityModeCommand_r3_add_ext = -1;  /* BIT_STRING */
static int hf_rrc_nonCriticalExtensions_139 = -1;  /* T_nonCriticalExtensions_139 */
static int hf_rrc_later_than_r3_19 = -1;          /* T_later_than_r3_19 */
static int hf_rrc_criticalExtensions_109 = -1;    /* T_criticalExtensions_108 */
static int hf_rrc_r7_10 = -1;                     /* T_r7_10 */
static int hf_rrc_securityModeCommand_r7 = -1;    /* SecurityModeCommand_r7_IEs */
static int hf_rrc_securityModeCommand_r7_add_ext = -1;  /* BIT_STRING */
static int hf_rrc_nonCriticalExtensions_140 = -1;  /* T_nonCriticalExtensions_140 */
static int hf_rrc_criticalExtensions_110 = -1;    /* T_criticalExtensions_109 */
static int hf_rrc_securityCapability = -1;        /* SecurityCapability */
static int hf_rrc_ue_SystemSpecificSecurityCap = -1;  /* InterRAT_UE_SecurityCapList */
static int hf_rrc_rb_UL_CiphActivationTimeInfo = -1;  /* RB_ActivationTimeInfoList */
static int hf_rrc_laterNonCriticalExtensions_46 = -1;  /* T_laterNonCriticalExtensions_46 */
static int hf_rrc_securityModeComplete_r3_add_ext = -1;  /* BIT_STRING */
static int hf_rrc_nonCriticalExtensions_141 = -1;  /* T_nonCriticalExtensions_141 */
static int hf_rrc_laterNonCriticalExtensions_47 = -1;  /* T_laterNonCriticalExtensions_47 */
static int hf_rrc_securityModeFailure_r3_add_ext = -1;  /* BIT_STRING */
static int hf_rrc_nonCriticalExtensions_142 = -1;  /* T_nonCriticalExtensions_142 */
static int hf_rrc_r3_23 = -1;                     /* T_r3_23 */
static int hf_rrc_signallingConnectionRelease_r3 = -1;  /* SignallingConnectionRelease_r3_IEs */
static int hf_rrc_laterNonCriticalExtensions_48 = -1;  /* T_laterNonCriticalExtensions_48 */
static int hf_rrc_signallingConnectionRelease_r3_add_ext = -1;  /* BIT_STRING */
static int hf_rrc_va40NonCriticalExtensions_14 = -1;  /* T_va40NonCriticalExtensions_13 */
static int hf_rrc_signallingConnectionRelease_va40ext = -1;  /* SignallingConnectionRelease_va40ext_IEs */
static int hf_rrc_nonCriticalExtensions_143 = -1;  /* T_nonCriticalExtensions_143 */
static int hf_rrc_later_than_r3_20 = -1;          /* T_later_than_r3_20 */
static int hf_rrc_criticalExtensions_111 = -1;    /* T_criticalExtensions_110 */
static int hf_rrc_laterNonCriticalExtensions_49 = -1;  /* T_laterNonCriticalExtensions_49 */
static int hf_rrc_signallingConnectionReleaseIndication_r3_add_ext = -1;  /* BIT_STRING */
static int hf_rrc_v860nonCriticalExtentions = -1;  /* T_v860nonCriticalExtentions */
static int hf_rrc_signallingConnectionReleaseIndication_v860ext = -1;  /* SignallingConnectionReleaseIndication_v860ext */
static int hf_rrc_nonCriticalExtensions_144 = -1;  /* T_nonCriticalExtensions_144 */
static int hf_rrc_signallingConnectionReleaseIndicationCause = -1;  /* T_signallingConnectionReleaseIndicationCause */
static int hf_rrc_sfn_Prime = -1;                 /* SFN_Prime */
static int hf_rrc_payload = -1;                   /* T_payload */
static int hf_rrc_noSegment = -1;                 /* NULL */
static int hf_rrc_firstSegment = -1;              /* FirstSegment */
static int hf_rrc_subsequentSegment = -1;         /* SubsequentSegment */
static int hf_rrc_lastSegmentShort = -1;          /* LastSegmentShort */
static int hf_rrc_lastAndFirst = -1;              /* T_lastAndFirst */
static int hf_rrc_firstSegment_01 = -1;           /* FirstSegmentShort */
static int hf_rrc_lastAndComplete = -1;           /* T_lastAndComplete */
static int hf_rrc_completeSIB_List = -1;          /* CompleteSIB_List */
static int hf_rrc_lastAndCompleteAndFirst = -1;   /* T_lastAndCompleteAndFirst */
static int hf_rrc_completeAndFirst = -1;          /* T_completeAndFirst */
static int hf_rrc_completeSIB = -1;               /* CompleteSIB */
static int hf_rrc_lastSegment = -1;               /* LastSegment */
static int hf_rrc_payload_01 = -1;                /* T_payload_01 */
static int hf_rrc_lastAndFirst_01 = -1;           /* T_lastAndFirst_01 */
static int hf_rrc_lastAndComplete_01 = -1;        /* T_lastAndComplete_01 */
static int hf_rrc_lastAndCompleteAndFirst_01 = -1;  /* T_lastAndCompleteAndFirst_01 */
static int hf_rrc_completeAndFirst_01 = -1;       /* T_completeAndFirst_01 */
static int hf_rrc_sib_Type = -1;                  /* SIB_Type */
static int hf_rrc_seg_Count = -1;                 /* SegCount */
static int hf_rrc_sib_Data_fixed = -1;            /* SIB_Data_fixed */
static int hf_rrc_sib_Data_variable = -1;         /* SIB_Data_variable */
static int hf_rrc_segmentIndex = -1;              /* SegmentIndex */
static int hf_rrc_CompleteSIB_List_item = -1;     /* CompleteSIBshort */
static int hf_rrc_sib_Data_fixed_01 = -1;         /* BIT_STRING_SIZE_226 */
static int hf_rrc_laterNonCriticalExtensions_50 = -1;  /* T_laterNonCriticalExtensions_50 */
static int hf_rrc_systemInformationChangeIndication_r3_add_ext = -1;  /* BIT_STRING */
static int hf_rrc_v860NonCriticalExtensions_17 = -1;  /* T_v860NonCriticalExtensions_17 */
static int hf_rrc_systemInformationChangeIndication_v860ext = -1;  /* SystemInformationChangeIndication_v860ext_IEs */
static int hf_rrc_nonCriticalExtensions_145 = -1;  /* T_nonCriticalExtensions_145 */
static int hf_rrc_r3_24 = -1;                     /* T_r3_24 */
static int hf_rrc_transportChannelReconfiguration_r3 = -1;  /* TransportChannelReconfiguration_r3_IEs */
static int hf_rrc_v3a0NonCriticalExtensions_09 = -1;  /* T_v3a0NonCriticalExtensions_09 */
static int hf_rrc_transportChannelReconfiguration_v3a0ext = -1;  /* TransportChannelReconfiguration_v3a0ext */
static int hf_rrc_laterNonCriticalExtensions_51 = -1;  /* T_laterNonCriticalExtensions_51 */
static int hf_rrc_transportChannelReconfiguration_r3_add_ext = -1;  /* BIT_STRING */
static int hf_rrc_v4b0NonCriticalExtensions_13 = -1;  /* T_v4b0NonCriticalExtensions_13 */
static int hf_rrc_transportChannelReconfiguration_v4b0ext = -1;  /* TransportChannelReconfiguration_v4b0ext_IEs */
static int hf_rrc_v590NonCriticalExtensions_24 = -1;  /* T_v590NonCriticalExtensions_24 */
static int hf_rrc_transportChannelReconfiguration_v590ext = -1;  /* TransportChannelReconfiguration_v590ext_IEs */
static int hf_rrc_v690NonCriticalExtensions_36 = -1;  /* T_v690NonCriticalExtensions_36 */
static int hf_rrc_transportChannelReconfiguration_v690ext = -1;  /* TransportChannelReconfiguration_v690ext_IEs */
static int hf_rrc_v770NonCriticalExtensions_25 = -1;  /* T_v770NonCriticalExtensions_25 */
static int hf_rrc_transportChannelReconfiguration_v770ext = -1;  /* TransportChannelReconfiguration_v770ext_IEs */
static int hf_rrc_nonCriticalExtensions_146 = -1;  /* T_nonCriticalExtensions_146 */
static int hf_rrc_later_than_r3_21 = -1;          /* T_later_than_r3_21 */
static int hf_rrc_criticalExtensions_112 = -1;    /* T_criticalExtensions_111 */
static int hf_rrc_r4_12 = -1;                     /* T_r4_12 */
static int hf_rrc_transportChannelReconfiguration_r4 = -1;  /* TransportChannelReconfiguration_r4_IEs */
static int hf_rrc_v4d0NonCriticalExtensions_12 = -1;  /* T_v4d0NonCriticalExtensions_12 */
static int hf_rrc_transportChannelReconfiguration_r4_add_ext = -1;  /* BIT_STRING */
static int hf_rrc_v590NonCriticalExtensions_25 = -1;  /* T_v590NonCriticalExtensions_25 */
static int hf_rrc_v690NonCriticalExtensions_37 = -1;  /* T_v690NonCriticalExtensions_37 */
static int hf_rrc_v770NonCriticalExtensions_26 = -1;  /* T_v770NonCriticalExtensions_26 */
static int hf_rrc_nonCriticalExtensions_147 = -1;  /* T_nonCriticalExtensions_147 */
static int hf_rrc_criticalExtensions_113 = -1;    /* T_criticalExtensions_112 */
static int hf_rrc_r5_10 = -1;                     /* T_r5_10 */
static int hf_rrc_transportChannelReconfiguration_r5 = -1;  /* TransportChannelReconfiguration_r5_IEs */
static int hf_rrc_transportChannelReconfiguration_r5_add_ext = -1;  /* BIT_STRING */
static int hf_rrc_v690NonCriticalExtensions_38 = -1;  /* T_v690NonCriticalExtensions_38 */
static int hf_rrc_v770NonCriticalExtensions_27 = -1;  /* T_v770NonCriticalExtensions_27 */
static int hf_rrc_nonCriticalExtensions_148 = -1;  /* T_nonCriticalExtensions_148 */
static int hf_rrc_criticalExtensions_114 = -1;    /* T_criticalExtensions_113 */
static int hf_rrc_r6_11 = -1;                     /* T_r6_11 */
static int hf_rrc_transportChannelReconfiguration_r6 = -1;  /* TransportChannelReconfiguration_r6_IEs */
static int hf_rrc_transportChannelReconfiguration_r6_add_ext = -1;  /* BIT_STRING */
static int hf_rrc_v6b0NonCriticalExtensions_12 = -1;  /* T_v6b0NonCriticalExtensions_11 */
static int hf_rrc_transportChannelReconfiguration_v6b0ext = -1;  /* TransportChannelReconfiguration_v6b0ext_IEs */
static int hf_rrc_v770NonCriticalExtensions_28 = -1;  /* T_v770NonCriticalExtensions_28 */
static int hf_rrc_nonCriticalExtensions_149 = -1;  /* T_nonCriticalExtensions_149 */
static int hf_rrc_criticalExtensions_115 = -1;    /* T_criticalExtensions_114 */
static int hf_rrc_r7_11 = -1;                     /* T_r7_11 */
static int hf_rrc_transportChannelReconfiguration_r7 = -1;  /* TransportChannelReconfiguration_r7_IEs */
static int hf_rrc_transportChannelReconfiguration_r7_add_ext = -1;  /* BIT_STRING */
static int hf_rrc_v780NonCriticalExtensions_09 = -1;  /* T_v780NonCriticalExtensions_09 */
static int hf_rrc_transportChannelReconfiguration_v780ext = -1;  /* TransportChannelReconfiguration_v780ext_IEs */
static int hf_rrc_v7d0NonCriticalExtensions_13 = -1;  /* T_v7d0NonCriticalExtensions_13 */
static int hf_rrc_transportChannelReconfiguration_v7d0ext = -1;  /* TransportChannelReconfiguration_v7d0ext_IEs */
static int hf_rrc_v7f0NonCriticalExtensions_14 = -1;  /* T_v7f0NonCriticalExtensions_14 */
static int hf_rrc_transportChannelReconfiguration_v7f0ext = -1;  /* TransportChannelReconfiguration_v7f0ext_IEs */
static int hf_rrc_v7g0NonCriticalExtensions_15 = -1;  /* T_v7g0NonCriticalExtensions_15 */
static int hf_rrc_transportChannelReconfiguration_v7g0ext = -1;  /* TransportChannelReconfiguration_v7g0ext_IEs */
static int hf_rrc_nonCriticalExtensions_150 = -1;  /* T_nonCriticalExtensions_150 */
static int hf_rrc_criticalExtensions_116 = -1;    /* T_criticalExtensions_115 */
static int hf_rrc_r8_11 = -1;                     /* T_r8_11 */
static int hf_rrc_transportChannelReconfiguration_r8 = -1;  /* TransportChannelReconfiguration_r8_IEs */
static int hf_rrc_transportChannelReconfiguration_r8_add_ext = -1;  /* BIT_STRING */
static int hf_rrc_v7d0NonCriticalExtensions_14 = -1;  /* T_v7d0NonCriticalExtensions_14 */
static int hf_rrc_v7f0NonCriticalExtensions_15 = -1;  /* T_v7f0NonCriticalExtensions_15 */
static int hf_rrc_v890NonCriticalExtensions_09 = -1;  /* T_v890NonCriticalExtensions_09 */
static int hf_rrc_transportChannelReconfiguration_v890ext = -1;  /* TransportChannelReconfiguration_v890ext_IEs */
static int hf_rrc_v7g0NonCriticalExtensions_16 = -1;  /* T_v7g0NonCriticalExtensions_16 */
static int hf_rrc_v8a0NonCriticalExtensions_09 = -1;  /* T_v8a0NonCriticalExtensions_09 */
static int hf_rrc_transportChannelReconfiguration_v8a0ext = -1;  /* TransportChannelReconfiguration_v8a0ext_IEs */
static int hf_rrc_nonCriticalExtensions_151 = -1;  /* T_nonCriticalExtensions_151 */
static int hf_rrc_criticalExtensions_117 = -1;    /* T_criticalExtensions_116 */
static int hf_rrc_r9_10 = -1;                     /* T_r9_10 */
static int hf_rrc_transportChannelReconfiguration_r9 = -1;  /* TransportChannelReconfiguration_r9_IEs */
static int hf_rrc_transportChannelReconfiguration_r9_add_ext = -1;  /* BIT_STRING */
static int hf_rrc_v950NonCriticalExtensions_08 = -1;  /* T_v950NonCriticalExtensions_08 */
static int hf_rrc_transportChannelReconfiguration_v950ext = -1;  /* TransportChannelReconfiguration_v950ext_IEs */
static int hf_rrc_v9c0NonCriticalExtensions_10 = -1;  /* T_v9c0NonCriticalExtensions_10 */
static int hf_rrc_transportChannelReconfiguration_v9c0ext = -1;  /* TransportChannelReconfiguration_v9c0ext_IEs */
static int hf_rrc_nonCriticalExtensions_152 = -1;  /* T_nonCriticalExtensions_152 */
static int hf_rrc_criticalExtensions_118 = -1;    /* T_criticalExtensions_117 */
static int hf_rrc_r10_10 = -1;                    /* T_r10_10 */
static int hf_rrc_transportChannelReconfiguration_r10 = -1;  /* TransportChannelReconfiguration_r10_IEs */
static int hf_rrc_transportChannelReconfiguration_r10_add_ext = -1;  /* BIT_STRING */
static int hf_rrc_v9c0NonCriticalExtensions_11 = -1;  /* T_v9c0NonCriticalExtensions_11 */
static int hf_rrc_vbxyNonCriticalExtensions_14 = -1;  /* T_vbxyNonCriticalExtensions_14 */
static int hf_rrc_transportChannelReconfiguration_vbxyext = -1;  /* TransportChannelReconfiguration_vbxyext_IEs */
static int hf_rrc_nonCriticalExtensions_153 = -1;  /* T_nonCriticalExtensions_153 */
static int hf_rrc_criticalExtensions_119 = -1;    /* T_criticalExtensions_118 */
static int hf_rrc_r11_11 = -1;                    /* T_r11_11 */
static int hf_rrc_transportChannelReconfiguration_r11 = -1;  /* TransportChannelReconfiguration_r11_IEs */
static int hf_rrc_transportChannelReconfiguration_r11_add_ext = -1;  /* BIT_STRING */
static int hf_rrc_nonCriticalExtensions_154 = -1;  /* T_nonCriticalExtensions_154 */
static int hf_rrc_criticalExtensions_120 = -1;    /* T_criticalExtensions_119 */
static int hf_rrc_dummy_25 = -1;                  /* T_dummy_11 */
static int hf_rrc_fdd_39 = -1;                    /* T_fdd_39 */
static int hf_rrc_modeSpecificPhysChInfo_12 = -1;  /* T_modeSpecificPhysChInfo_12 */
static int hf_rrc_fdd_40 = -1;                    /* T_fdd_40 */
static int hf_rrc_dummy_26 = -1;                  /* T_dummy_12 */
static int hf_rrc_fdd_41 = -1;                    /* T_fdd_41 */
static int hf_rrc_modeSpecificPhysChInfo_13 = -1;  /* T_modeSpecificPhysChInfo_13 */
static int hf_rrc_fdd_42 = -1;                    /* T_fdd_42 */
static int hf_rrc_dummy_27 = -1;                  /* T_dummy_13 */
static int hf_rrc_fdd_43 = -1;                    /* T_fdd_43 */
static int hf_rrc_modeSpecificPhysChInfo_14 = -1;  /* T_modeSpecificPhysChInfo_14 */
static int hf_rrc_fdd_44 = -1;                    /* T_fdd_44 */
static int hf_rrc_postVerificationPeriod_06 = -1;  /* T_postVerificationPeriod_06 */
static int hf_rrc_responseToChangeOfUE_Capability_10 = -1;  /* T_responseToChangeOfUE_Capability_10 */
static int hf_rrc_responseToChangeOfUE_Capability_11 = -1;  /* T_responseToChangeOfUE_Capability_11 */
static int hf_rrc_responseToChangeOfUE_Capability_12 = -1;  /* T_responseToChangeOfUE_Capability_12 */
static int hf_rrc_responseToChangeOfUE_Capability_13 = -1;  /* T_responseToChangeOfUE_Capability_13 */
static int hf_rrc_responseToChangeOfUE_Capability_14 = -1;  /* T_responseToChangeOfUE_Capability_14 */
static int hf_rrc_laterNonCriticalExtensions_52 = -1;  /* T_laterNonCriticalExtensions_52 */
static int hf_rrc_transportChannelReconfigurationComplete_r3_add_ext = -1;  /* BIT_STRING */
static int hf_rrc_v770NonCriticalExtensions_29 = -1;  /* T_v770NonCriticalExtensions_29 */
static int hf_rrc_transportChannelReconfigurationComplete_v770ext = -1;  /* TransportChannelReconfigurationComplete_v770ext_IEs */
static int hf_rrc_nonCriticalExtensions_155 = -1;  /* T_nonCriticalExtensions_155 */
static int hf_rrc_deferredMeasurementControlReading_05 = -1;  /* T_deferredMeasurementControlReading_05 */
static int hf_rrc_laterNonCriticalExtensions_53 = -1;  /* T_laterNonCriticalExtensions_53 */
static int hf_rrc_transportChannelReconfigurationFailure_r3_add_ext = -1;  /* BIT_STRING */
static int hf_rrc_nonCriticalExtensions_156 = -1;  /* T_nonCriticalExtensions_156 */
static int hf_rrc_modeSpecificInfo_15 = -1;       /* T_modeSpecificInfo_15 */
static int hf_rrc_fdd_45 = -1;                    /* NULL */
static int hf_rrc_tdd_10 = -1;                    /* T_tdd_09 */
static int hf_rrc_tfcs_ID = -1;                   /* TFCS_Identity */
static int hf_rrc_dpch_TFCS_InUplink = -1;        /* TFC_Subset */
static int hf_rrc_activationTimeForTFCSubset = -1;  /* ActivationTime */
static int hf_rrc_tfc_ControlDuration = -1;       /* TFC_ControlDuration */
static int hf_rrc_laterNonCriticalExtensions_54 = -1;  /* T_laterNonCriticalExtensions_54 */
static int hf_rrc_transportFormatCombinationControl_r3_add_ext = -1;  /* BIT_STRING */
static int hf_rrc_v820NonCriticalExtensions_02 = -1;  /* T_v820NonCriticalExtensions_02 */
static int hf_rrc_transportformatcombinationcontrol_v820ext = -1;  /* TransportFormatCombinationControl_v820ext_IEs */
static int hf_rrc_nonCriticalExtensions_157 = -1;  /* T_nonCriticalExtensions_157 */
static int hf_rrc_ul_AMR_Rate = -1;               /* UL_AMR_Rate */
static int hf_rrc_laterNonCriticalExtensions_55 = -1;  /* T_laterNonCriticalExtensions_55 */
static int hf_rrc_transportFormatCombinationControlFailure_r3_add_ext = -1;  /* BIT_STRING */
static int hf_rrc_nonCriticalExtensions_158 = -1;  /* T_nonCriticalExtensions_158 */
static int hf_rrc_r3_25 = -1;                     /* T_r3_25 */
static int hf_rrc_ueCapabilityEnquiry_r3 = -1;    /* UECapabilityEnquiry_r3_IEs */
static int hf_rrc_laterNonCriticalExtensions_56 = -1;  /* T_laterNonCriticalExtensions_56 */
static int hf_rrc_ueCapabilityEnquiry_r3_add_ext = -1;  /* BIT_STRING */
static int hf_rrc_v4b0NonCriticalExtensions_14 = -1;  /* T_v4b0NonCriticalExtensions_14 */
static int hf_rrc_ueCapabilityEnquiry_v4b0ext = -1;  /* UECapabilityEnquiry_v4b0ext_IEs */
static int hf_rrc_v590NonCriticalExtensions_26 = -1;  /* T_v590NonCriticalExtensions_26 */
static int hf_rrc_ueCapabilityEnquiry_v590ext = -1;  /* UECapabilityEnquiry_v590ext_IEs */
static int hf_rrc_v770NonCriticalExtensions_30 = -1;  /* T_v770NonCriticalExtensions_30 */
static int hf_rrc_ueCapabilityEnquiry_v770ext = -1;  /* UECapabilityEnquiry_v770ext_IEs */
static int hf_rrc_v860NonCriticalExtensions_18 = -1;  /* T_v860NonCriticalExtensions_18 */
static int hf_rrc_ueCapabilityEnquiry_v860ext = -1;  /* UECapabilityEnquiry_v860ext_IEs */
static int hf_rrc_nonCriticalExtensions_159 = -1;  /* T_nonCriticalExtensions_159 */
static int hf_rrc_later_than_r3_22 = -1;          /* T_later_than_r3_22 */
static int hf_rrc_criticalExtensions_121 = -1;    /* T_criticalExtensions_120 */
static int hf_rrc_capabilityUpdateRequirement_05 = -1;  /* CapabilityUpdateRequirement_v770ext */
static int hf_rrc_capabilityUpdateRequirement_06 = -1;  /* CapabilityUpdateRequirement_v860ext */
static int hf_rrc_v370NonCriticalExtensions_01 = -1;  /* T_v370NonCriticalExtensions_01 */
static int hf_rrc_ueCapabilityInformation_v370ext = -1;  /* UECapabilityInformation_v370ext */
static int hf_rrc_v380NonCriticalExtensions_01 = -1;  /* T_v380NonCriticalExtensions_01 */
static int hf_rrc_ueCapabilityInformation_v380ext = -1;  /* UECapabilityInformation_v380ext_IEs */
static int hf_rrc_v3a0NonCriticalExtensions_10 = -1;  /* T_v3a0NonCriticalExtensions_10 */
static int hf_rrc_ueCapabilityInformation_v3a0ext = -1;  /* UECapabilityInformation_v3a0ext_IEs */
static int hf_rrc_laterNonCriticalExtensions_57 = -1;  /* T_laterNonCriticalExtensions_57 */
static int hf_rrc_ueCapabilityInformation_r3_add_ext = -1;  /* T_ueCapabilityInformation_r3_add_ext */
static int hf_rrc_v4b0NonCriticalExtensions_15 = -1;  /* T_v4b0NonCriticalExtensions_15 */
static int hf_rrc_ueCapabilityInformation_v4b0ext = -1;  /* UECapabilityInformation_v4b0ext */
static int hf_rrc_v590NonCriticalExtensions_27 = -1;  /* T_v590NonCriticalExtensions_27 */
static int hf_rrc_ueCapabilityInformation_v590ext = -1;  /* UECapabilityInformation_v590ext */
static int hf_rrc_v5c0NonCriticalExtensions_01 = -1;  /* T_v5c0NonCriticalExtensions_01 */
static int hf_rrc_ueCapabilityInformation_v5c0ext = -1;  /* UECapabilityInformation_v5c0ext */
static int hf_rrc_v690NonCriticalExtensions_39 = -1;  /* T_v690NonCriticalExtensions_39 */
static int hf_rrc_ueCapabilityInformation_v690ext = -1;  /* UECapabilityInformation_v690ext_IEs */
static int hf_rrc_nonCriticalExtensions_160 = -1;  /* T_nonCriticalExtensions_160 */
static int hf_rrc_ueCapabilityInformation_v650ext = -1;  /* UECapabilityInformation_v650ext_IEs */
static int hf_rrc_v680NonCriticalExtensions_01 = -1;  /* T_v680NonCriticalExtensions_01 */
static int hf_rrc_ueCapabilityInformation_v680ext = -1;  /* UECapabilityInformation_v680ext_IEs */
static int hf_rrc_v7e0NonCriticalExtensions_03 = -1;  /* T_v7e0NonCriticalExtensions_03 */
static int hf_rrc_ueCapabilityInformation_v7e0ext = -1;  /* UECapabilityInformation_v7e0ext_IEs */
static int hf_rrc_v7f0NonCriticalExtensions_16 = -1;  /* T_v7f0NonCriticalExtensions_16 */
static int hf_rrc_ueCapabilityInformation_v7f0ext = -1;  /* UECapabilityInformation_v7f0ext_IEs */
static int hf_rrc_va40NonCriticalExtensions_15 = -1;  /* T_va40NonCriticalExtensions_14 */
static int hf_rrc_ueCapabilityInformation_va40ext = -1;  /* UECapabilityInformation_va40ext_IEs */
static int hf_rrc_nonCriticalExtensions_161 = -1;  /* T_nonCriticalExtensions_161 */
static int hf_rrc_ueCapabilityContainer_01 = -1;  /* T_ueCapabilityContainer_01 */
static int hf_rrc_r3_26 = -1;                     /* T_r3_26 */
static int hf_rrc_ueCapabilityInformationConfirm_r3 = -1;  /* UECapabilityInformationConfirm_r3_IEs */
static int hf_rrc_laterNonCriticalExtensions_58 = -1;  /* T_laterNonCriticalExtensions_58 */
static int hf_rrc_ueCapabilityInformationConfirm_r3_add_ext = -1;  /* BIT_STRING */
static int hf_rrc_v770NonCriticalExtensions_31 = -1;  /* T_v770NonCriticalExtensions_31 */
static int hf_rrc_ueCapabilityInformationConfirm_v770ext = -1;  /* UECapabilityInformationConfirm_v770ext_IEs */
static int hf_rrc_nonCriticalExtensions_162 = -1;  /* T_nonCriticalExtensions_162 */
static int hf_rrc_later_than_r3_23 = -1;          /* T_later_than_r3_23 */
static int hf_rrc_criticalExtensions_122 = -1;    /* T_criticalExtensions_121 */
static int hf_rrc_acceptanceOfChangeOfCapability = -1;  /* T_acceptanceOfChangeOfCapability */
static int hf_rrc_laterNonCriticalExtensions_59 = -1;  /* T_laterNonCriticalExtensions_59 */
static int hf_rrc_uplinkDirectTransfer_r3_add_ext = -1;  /* BIT_STRING */
static int hf_rrc_v690NonCriticalExtensions_40 = -1;  /* T_v690NonCriticalExtensions_40 */
static int hf_rrc_uplinkDirectTransfer_v690ext = -1;  /* UplinkDirectTransfer_v690ext_IEs */
static int hf_rrc_v7g0NonCriticalExtensions_17 = -1;  /* T_v7g0NonCriticalExtensions_17 */
static int hf_rrc_uplinkDirectTransfer_v7g0ext = -1;  /* UplinkDirectTransfer_v7g0ext_IEs */
static int hf_rrc_vbxyNonCriticalExtensions_15 = -1;  /* T_vbxyNonCriticalExtensions_15 */
static int hf_rrc_uplinkDirectTransfer_vbxyext = -1;  /* UplinkDirectTransfer_vbxyext_IEs */
static int hf_rrc_nonCriticalExtensions_163 = -1;  /* T_nonCriticalExtensions_163 */
static int hf_rrc_r3_27 = -1;                     /* T_r3_27 */
static int hf_rrc_uplinkPhysicalChannelControl_r3 = -1;  /* UplinkPhysicalChannelControl_r3_IEs */
static int hf_rrc_laterNonCriticalExtensions_60 = -1;  /* T_laterNonCriticalExtensions_60 */
static int hf_rrc_uplinkPhysicalChannelControl_r3_add_ext = -1;  /* BIT_STRING */
static int hf_rrc_v4b0NonCriticalExtensions_16 = -1;  /* T_v4b0NonCriticalExtensions_16 */
static int hf_rrc_uplinkPysicalChannelControl_v4b0ext = -1;  /* UplinkPhysicalChannelControl_v4b0ext_IEs */
static int hf_rrc_noncriticalExtensions = -1;     /* T_noncriticalExtensions */
static int hf_rrc_later_than_r3_24 = -1;          /* T_later_than_r3_24 */
static int hf_rrc_criticalExtensions_123 = -1;    /* T_criticalExtensions_122 */
static int hf_rrc_r4_13 = -1;                     /* T_r4_13 */
static int hf_rrc_uplinkPhysicalChannelControl_r4 = -1;  /* UplinkPhysicalChannelControl_r4_IEs */
static int hf_rrc_v4d0NonCriticalExtensions_13 = -1;  /* T_v4d0NonCriticalExtensions_13 */
static int hf_rrc_uplinkPhysicalChannelControl_r4_add_ext = -1;  /* BIT_STRING */
static int hf_rrc_v690NonCriticalExtensions_41 = -1;  /* T_v690NonCriticalExtensions_41 */
static int hf_rrc_uplinkPhysicalChannelControl_v690ext = -1;  /* UplinkPhysicalChannelControl_v690ext_IEs */
static int hf_rrc_nonCriticalExtensions_164 = -1;  /* T_nonCriticalExtensions_164 */
static int hf_rrc_criticalExtensions_124 = -1;    /* T_criticalExtensions_123 */
static int hf_rrc_r5_11 = -1;                     /* T_r5_11 */
static int hf_rrc_uplinkPhysicalChannelControl_r5 = -1;  /* UplinkPhysicalChannelControl_r5_IEs */
static int hf_rrc_uplinkPhysicalChannelControl_r5_add_ext = -1;  /* BIT_STRING */
static int hf_rrc_v690NonCriticalExtensions_42 = -1;  /* T_v690NonCriticalExtensions_42 */
static int hf_rrc_v6a0NonCriticalExtensions_01 = -1;  /* T_v6a0NonCriticalExtensions_01 */
static int hf_rrc_uplinkPhysicalChannelControl_v6a0ext = -1;  /* UplinkPhysicalChannelControl_v6a0ext_IEs */
static int hf_rrc_nonCriticalExtensions_165 = -1;  /* T_nonCriticalExtensions_165 */
static int hf_rrc_criticalExtensions_125 = -1;    /* T_criticalExtensions_124 */
static int hf_rrc_r7_12 = -1;                     /* T_r7_12 */
static int hf_rrc_uplinkPhysicalChannelControl_r7 = -1;  /* UplinkPhysicalChannelControl_r7_IEs */
static int hf_rrc_uplinkPhysicalChannelControl_r7_add_ext = -1;  /* BIT_STRING */
static int hf_rrc_nonCriticalExtensions_166 = -1;  /* T_nonCriticalExtensions_166 */
static int hf_rrc_criticalExtensions_126 = -1;    /* T_criticalExtensions_125 */
static int hf_rrc_ccTrCH_PowerControlInfo = -1;   /* CCTrCH_PowerControlInfo */
static int hf_rrc_timingAdvance = -1;             /* UL_TimingAdvanceControl */
static int hf_rrc_alpha = -1;                     /* Alpha */
static int hf_rrc_specialBurstScheduling = -1;    /* SpecialBurstScheduling */
static int hf_rrc_prach_ConstantValue = -1;       /* ConstantValueTdd */
static int hf_rrc_pusch_ConstantValue = -1;       /* ConstantValueTdd */
static int hf_rrc_openLoopPowerControl_IPDL_TDD = -1;  /* OpenLoopPowerControl_IPDL_TDD_r4 */
static int hf_rrc_ccTrCH_PowerControlInfo_01 = -1;  /* CCTrCH_PowerControlInfo_r4 */
static int hf_rrc_tddOption = -1;                 /* T_tddOption */
static int hf_rrc_tdd384_08 = -1;                 /* T_tdd384_08 */
static int hf_rrc_timingAdvance_01 = -1;          /* UL_TimingAdvanceControl_r4 */
static int hf_rrc_tdd128_08 = -1;                 /* T_tdd128_08 */
static int hf_rrc_ul_SynchronisationParameters = -1;  /* UL_SynchronisationParameters_r4 */
static int hf_rrc_ccTrCH_PowerControlInfo_02 = -1;  /* CCTrCH_PowerControlInfo_r5 */
static int hf_rrc_tddOption_01 = -1;              /* T_tddOption_01 */
static int hf_rrc_tdd384_09 = -1;                 /* T_tdd384_09 */
static int hf_rrc_hs_SICH_PowerControl = -1;      /* HS_SICH_Power_Control_Info_TDD384 */
static int hf_rrc_tdd128_09 = -1;                 /* T_tdd128_09 */
static int hf_rrc_desired_HS_SICH_PowerLevel = -1;  /* INTEGER_M120_M58 */
static int hf_rrc_tpc_Step_Size = -1;             /* T_tpc_Step_Size */
static int hf_rrc_ccTrCH_PowerControlInfo_03 = -1;  /* CCTrCH_PowerControlInfo_r7 */
static int hf_rrc_tddOption_02 = -1;              /* T_tddOption_02 */
static int hf_rrc_tdd384_10 = -1;                 /* T_tdd384_10 */
static int hf_rrc_tdd768_05 = -1;                 /* T_tdd768_05 */
static int hf_rrc_timingAdvance_02 = -1;          /* UL_TimingAdvanceControl_r7 */
static int hf_rrc_hs_SICH_PowerControl_01 = -1;   /* HS_SICH_Power_Control_Info_TDD768 */
static int hf_rrc_tdd128_10 = -1;                 /* T_tdd128_10 */
static int hf_rrc_tpc_Step_Size_01 = -1;          /* T_tpc_Step_Size_01 */
static int hf_rrc_ura_UpdateCause = -1;           /* URA_UpdateCause */
static int hf_rrc_laterNonCriticalExtensions_61 = -1;  /* T_laterNonCriticalExtensions_61 */
static int hf_rrc_uraUpdate_r3_add_ext = -1;      /* T_uraUpdate_r3_add_ext */
static int hf_rrc_v770NonCriticalExtensions_32 = -1;  /* T_v770NonCriticalExtensions_32 */
static int hf_rrc_uraUpdate_v770ext = -1;         /* UraUpdate_v770ext_IEs */
static int hf_rrc_v860NonCriticalExtensions_19 = -1;  /* T_v860NonCriticalExtensions_19 */
static int hf_rrc_uraUpdate_v860ext = -1;         /* URAUpdate_v860ext_IEs */
static int hf_rrc_va40NonCriticalExtensions_16 = -1;  /* T_va40NonCriticalExtensions_15 */
static int hf_rrc_uraUpdate_va40ext = -1;         /* URAUpdate_va40ext_IEs */
static int hf_rrc_nonCriticalExtensions_167 = -1;  /* T_nonCriticalExtensions_167 */
static int hf_rrc_uraUpdate_v7e0ext = -1;         /* URAUpdate_v7e0ext_IEs */
static int hf_rrc_nonCriticalExtensions_168 = -1;  /* T_nonCriticalExtensions_168 */
static int hf_rrc_support_hsdschReception_CellUraPch = -1;  /* T_support_hsdschReception_CellUraPch */
static int hf_rrc_support_hsdschReception_CellFach = -1;  /* T_support_hsdschReception_CellFach */
static int hf_rrc_supportForTwoDRXSchemesInPCH_01 = -1;  /* T_supportForTwoDRXSchemesInPCH_01 */
static int hf_rrc_supportOfHS_DSCHDRXOperation_01 = -1;  /* T_supportOfHS_DSCHDRXOperation_01 */
static int hf_rrc_supportOfCommonEDCH_02 = -1;    /* T_supportOfCommonEDCH_02 */
static int hf_rrc_supportOfMACiis_02 = -1;        /* T_supportOfMACiis_02 */
static int hf_rrc_loggedMeasAvailable_04 = -1;    /* T_loggedMeasAvailable_04 */
static int hf_rrc_loggedANRResultsAvailable_03 = -1;  /* T_loggedANRResultsAvailable_03 */
static int hf_rrc_r3_28 = -1;                     /* T_r3_28 */
static int hf_rrc_uraUpdateConfirm_r3 = -1;       /* URAUpdateConfirm_r3_IEs */
static int hf_rrc_laterNonCriticalExtensions_62 = -1;  /* T_laterNonCriticalExtensions_62 */
static int hf_rrc_uraUpdateConfirm_r3_add_ext = -1;  /* BIT_STRING */
static int hf_rrc_v690NonCriticalExtensions_43 = -1;  /* T_v690NonCriticalExtensions_43 */
static int hf_rrc_uraUpdateConfirm_v690ext = -1;  /* URAUpdateConfirm_v690ext_IEs */
static int hf_rrc_nonCriticalExtensions_169 = -1;  /* T_nonCriticalExtensions_169 */
static int hf_rrc_later_than_r3_25 = -1;          /* T_later_than_r3_25 */
static int hf_rrc_criticalExtensions_127 = -1;    /* T_criticalExtensions_126 */
static int hf_rrc_r5_12 = -1;                     /* T_r5_12 */
static int hf_rrc_uraUpdateConfirm_r5 = -1;       /* URAUpdateConfirm_r5_IEs */
static int hf_rrc_v690NonCriticalExtensions_44 = -1;  /* T_v690NonCriticalExtensions_44 */
static int hf_rrc_nonCriticalExtensions_170 = -1;  /* T_nonCriticalExtensions_170 */
static int hf_rrc_criticalExtensions_128 = -1;    /* T_criticalExtensions_127 */
static int hf_rrc_r7_13 = -1;                     /* T_r7_13 */
static int hf_rrc_uraUpdateConfirm_r7 = -1;       /* URAUpdateConfirm_r7_IEs */
static int hf_rrc_uraUpdateConfirm_r7_add_ext = -1;  /* BIT_STRING */
static int hf_rrc_v860NonCriticalExtensions_20 = -1;  /* T_v860NonCriticalExtensions_20 */
static int hf_rrc_uraUpdateConfirm_v860ext = -1;  /* URAUpdateConfirm_v860ext_IEs */
static int hf_rrc_nonCriticalExtensions_171 = -1;  /* T_nonCriticalExtensions_171 */
static int hf_rrc_criticalExtensions_129 = -1;    /* T_criticalExtensions_128 */
static int hf_rrc_r3_29 = -1;                     /* T_r3_29 */
static int hf_rrc_uraUpdateConfirm_CCCH_r3 = -1;  /* URAUpdateConfirm_CCCH_r3_IEs */
static int hf_rrc_laterNonCriticalExtensions_63 = -1;  /* T_laterNonCriticalExtensions_63 */
static int hf_rrc_uraUpdateConfirm_CCCH_r3_add_ext = -1;  /* BIT_STRING */
static int hf_rrc_v690NonCriticalExtensions_45 = -1;  /* T_v690NonCriticalExtensions_45 */
static int hf_rrc_v860NonCriticalExtensions_21 = -1;  /* T_v860NonCriticalExtensions_21 */
static int hf_rrc_nonCriticalExtensions_172 = -1;  /* T_nonCriticalExtensions_172 */
static int hf_rrc_later_than_r3_26 = -1;          /* T_later_than_r3_26 */
static int hf_rrc_criticalExtensions_130 = -1;    /* T_criticalExtensions_129 */
static int hf_rrc_uraUpdateConfirm_02 = -1;       /* URAUpdateConfirm_r3_IEs */
static int hf_rrc_r3_30 = -1;                     /* T_r3_30 */
static int hf_rrc_utranMobilityInformation_r3 = -1;  /* UTRANMobilityInformation_r3_IEs */
static int hf_rrc_v3a0NonCriticalExtensions_11 = -1;  /* T_v3a0NonCriticalExtensions_11 */
static int hf_rrc_utranMobilityInformation_v3a0ext = -1;  /* UTRANMobilityInformation_v3a0ext_IEs */
static int hf_rrc_laterNonCriticalExtensions_64 = -1;  /* T_laterNonCriticalExtensions_64 */
static int hf_rrc_utranMobilityInformation_r3_add_ext = -1;  /* BIT_STRING */
static int hf_rrc_v690NonCriticalExtensions_46 = -1;  /* T_v690NonCriticalExtensions_46 */
static int hf_rrc_utranMobilityInformation_v690ext = -1;  /* UtranMobilityInformation_v690ext_IEs */
static int hf_rrc_v860NonCriticalExtentions = -1;  /* T_v860NonCriticalExtentions */
static int hf_rrc_utranMobilityInformation_v860ext = -1;  /* UTRANMobilityInformation_v860ext1_IEs */
static int hf_rrc_nonCriticalExtensions_173 = -1;  /* T_nonCriticalExtensions_173 */
static int hf_rrc_later_than_r3_27 = -1;          /* T_later_than_r3_27 */
static int hf_rrc_criticalExtensions_131 = -1;    /* T_criticalExtensions_130 */
static int hf_rrc_r5_13 = -1;                     /* T_r5_13 */
static int hf_rrc_utranMobilityInformation_r5 = -1;  /* UTRANMobilityInformation_r5_IEs */
static int hf_rrc_v690NonCriticalExtensions_47 = -1;  /* T_v690NonCriticalExtensions_47 */
static int hf_rrc_v860NonCriticalExtentions_01 = -1;  /* T_v860NonCriticalExtentions_01 */
static int hf_rrc_nonCriticalExtensions_174 = -1;  /* T_nonCriticalExtensions_174 */
static int hf_rrc_criticalExtensions_132 = -1;    /* T_criticalExtensions_131 */
static int hf_rrc_r7_14 = -1;                     /* T_r7_14 */
static int hf_rrc_utranMobilityInformation_r7 = -1;  /* UTRANMobilityInformation_r7_IEs */
static int hf_rrc_utranMobilityInformation_r7_add_ext = -1;  /* BIT_STRING */
static int hf_rrc_v860NonCriticalExtensions_22 = -1;  /* T_v860NonCriticalExtensions_22 */
static int hf_rrc_utranMobilityInformation_v860ext_01 = -1;  /* UTRANMobilityInformation_v860ext2_IEs */
static int hf_rrc_nonCriticalExtensions_175 = -1;  /* T_nonCriticalExtensions_175 */
static int hf_rrc_criticalExtensions_133 = -1;    /* T_criticalExtensions_132 */
static int hf_rrc_ue_ConnTimersAndConstants = -1;  /* UE_ConnTimersAndConstants */
static int hf_rrc_cn_InformationInfo_02 = -1;     /* CN_InformationInfoFull */
static int hf_rrc_nonCriticalExtensions_176 = -1;  /* T_nonCriticalExtensions_176 */
static int hf_rrc_ue_ConnTimersAndConstants_v3a0ext = -1;  /* UE_ConnTimersAndConstants_v3a0ext */
static int hf_rrc_ue_ConnTimersAndConstants_01 = -1;  /* UE_ConnTimersAndConstants_r5 */
static int hf_rrc_ue_ConnTimersAndConstants_02 = -1;  /* UE_ConnTimersAndConstants_v860ext */
static int hf_rrc_dedicatedPriorityInformation = -1;  /* DedicatedPriorityInformation */
static int hf_rrc_laterNonCriticalExtensions_65 = -1;  /* T_laterNonCriticalExtensions_65 */
static int hf_rrc_utranMobilityInformationConfirm_r3_add_ext = -1;  /* BIT_STRING */
static int hf_rrc_v770NonCriticalExtension_01 = -1;  /* T_v770NonCriticalExtension_01 */
static int hf_rrc_utranMobilityInformationConfirm_v770ext = -1;  /* UTRANMobilityInformationConfirm_v770ext_IEs */
static int hf_rrc_va40NonCriticalExtensions_17 = -1;  /* T_va40NonCriticalExtensions_16 */
static int hf_rrc_utranMobilityInformationConfirm_va40ext = -1;  /* UTRANMobilityInformationConfirm_va40ext_IEs */
static int hf_rrc_nonCriticalExtensions_177 = -1;  /* T_nonCriticalExtensions_177 */
static int hf_rrc_deferredMeasurementControlReading_06 = -1;  /* T_deferredMeasurementControlReading_06 */
static int hf_rrc_loggedMeasAvailable_05 = -1;    /* T_loggedMeasAvailable_05 */
static int hf_rrc_laterNonCriticalExtensions_66 = -1;  /* T_laterNonCriticalExtensions_66 */
static int hf_rrc_utranMobilityInformationFailure_r3_add_ext = -1;  /* BIT_STRING */
static int hf_rrc_nonCriticalExtensions_178 = -1;  /* T_nonCriticalExtensions_178 */
static int hf_rrc_mbms_ServiceAccessInfoList = -1;  /* MBMS_ServiceAccessInfoList_r6 */
static int hf_rrc_nonCriticalExtensions_179 = -1;  /* T_nonCriticalExtensions_179 */
static int hf_rrc_mbms_CommonRBInformationList = -1;  /* MBMS_CommonRBInformationList_r6 */
static int hf_rrc_mbms_TranspChInfoForEachTrCh = -1;  /* MBMS_TranspChInfoForEachTrCh_r6 */
static int hf_rrc_mbms_TranspChInfoForEachCCTrCh = -1;  /* MBMS_TranspChInfoForEachCCTrCh_r6 */
static int hf_rrc_mbms_PhyChInformationList = -1;  /* MBMS_PhyChInformationList_r6 */
static int hf_rrc_v770NonCriticalExtensions_33 = -1;  /* T_v770NonCriticalExtensions_33 */
static int hf_rrc_mbmsCommonPTMRBInformation_v770ext = -1;  /* MBMSCommonPTMRBInformation_v770ext_IEs */
static int hf_rrc_v780NonCriticalExtensions_10 = -1;  /* T_v780NonCriticalExtensions_10 */
static int hf_rrc_mbmsCommonPTMRBInformation_v780ext = -1;  /* MBMSCommonPTMRBInformation_v780ext_IEs */
static int hf_rrc_v860NonCriticalExtensions_23 = -1;  /* T_v860NonCriticalExtensions_23 */
static int hf_rrc_mbmsCommonPTMRBInformation_v860ext = -1;  /* MBMSCommonPTMRBInformation_v860ext_IEs */
static int hf_rrc_nonCriticalExtensions_180 = -1;  /* T_nonCriticalExtensions_180 */
static int hf_rrc_mbms_PhyChInformationList_r7 = -1;  /* MBMS_PhyChInformationList_r7 */
static int hf_rrc_mbsfn_TDDInformation_LCR = -1;  /* MBSFN_TDDInformation_LCR */
static int hf_rrc_mbms_PhyChInformationList_01 = -1;  /* MBMS_PhyChInformationList_IMB384 */
static int hf_rrc_mbms_CurrentCell_SCCPCHList = -1;  /* MBMS_CurrentCell_SCCPCHList_r6 */
static int hf_rrc_mbms_SIBType5_SCCPCHList = -1;  /* MBMS_SIBType5_SCCPCHList_r6 */
static int hf_rrc_v770NonCriticalExtensions_34 = -1;  /* T_v770NonCriticalExtensions_34 */
static int hf_rrc_mbmsCurrentCellPTMRBInfo_v770ext = -1;  /* MBMSCurrentCellPTMRBInfo_v770ext_IEs */
static int hf_rrc_nonCriticalExtensions_181 = -1;  /* T_nonCriticalExtensions_181 */
static int hf_rrc_mbsfn_TDM_Info_List = -1;       /* MBSFN_TDM_Info_List */
static int hf_rrc_mbms_PreferredFrequencyInfo = -1;  /* MBMS_PreferredFrequencyList_r6 */
static int hf_rrc_mbms_TimersAndCounters = -1;    /* MBMS_TimersAndCounters_r6 */
static int hf_rrc_michConfigurationInfo = -1;     /* MBMS_MICHConfigurationInfo_r6 */
static int hf_rrc_cellGroupIdentity = -1;         /* MBMS_CellGroupIdentity_r6 */
static int hf_rrc_mschDefaultConfigurationInfo = -1;  /* MBMS_MSCH_ConfigurationInfo_r6 */
static int hf_rrc_v6b0NonCriticalExtensions_13 = -1;  /* T_v6b0NonCriticalExtensions_12 */
static int hf_rrc_mbmsGeneralInformation_v6b0ext = -1;  /* MBMSGeneralInformation_v6b0ext_IEs */
static int hf_rrc_v770NonCriticalExtensions_35 = -1;  /* T_v770NonCriticalExtensions_35 */
static int hf_rrc_mbmsGeneralInformation_v770ext = -1;  /* MBMSGeneralInformation_v770ext_IEs */
static int hf_rrc_v860NonCriticalExtensions_24 = -1;  /* T_v860NonCriticalExtensions_24 */
static int hf_rrc_mbmsGeneralInformation_v860ext = -1;  /* MBMSGeneralInformation_v860ext_IEs */
static int hf_rrc_v890NoncriticalExtensions = -1;  /* T_v890NoncriticalExtensions */
static int hf_rrc_mbmsGeneralInformation_v890ext = -1;  /* MBMSGeneralInformation_v890ext_IEs */
static int hf_rrc_nonCriticalExtensions_182 = -1;  /* T_nonCriticalExtensions_182 */
static int hf_rrc_indicateChangeInSelectedServices = -1;  /* BOOLEAN */
static int hf_rrc_mbmsMICHConfiguration = -1;     /* MBMS_MICHConfigurationInfo_v770ext */
static int hf_rrc_mbsfnInterFrequencyNeighbourList = -1;  /* MBSFN_InterFrequencyNeighbourList_r7 */
static int hf_rrc_mbsfnInterFrequencyNeighbourList_01 = -1;  /* MBSFN_InterFrequencyNeighbourList_v860ext */
static int hf_rrc_mbmsNetworkStandardTimeInformation_LCR = -1;  /* MBMS_NetworkStandardTimeInformation_LCR_v890ext */
static int hf_rrc_mbmsMICHConfiguration_01 = -1;  /* MBMS_MICHConfigurationInfo_v890ext */
static int hf_rrc_mbms_PreferredFreqRequest = -1;  /* MBMS_ServiceIdentity_r6 */
static int hf_rrc_v6b0NonCriticalExtensions_14 = -1;  /* T_v6b0NonCriticalExtensions_13 */
static int hf_rrc_mbmsModificationRequest_v6b0ext = -1;  /* MBMSModificationRequest_v6b0ext_IEs */
static int hf_rrc_v6f0NonCriticalExtensions_02 = -1;  /* T_v6f0NonCriticalExtensions_02 */
static int hf_rrc_mbmsModificationRequest_v6f0ext = -1;  /* MBMSModificationRequest_v6f0ext_IEs */
static int hf_rrc_nonCriticalExtensions_183 = -1;  /* T_nonCriticalExtensions_183 */
static int hf_rrc_mbmsSelectedServiceInfo = -1;   /* MBMS_SelectedServiceInfo */
static int hf_rrc_mbmsSupportOfServiceChangeForAPtpRB = -1;  /* T_mbmsSupportOfServiceChangeForAPtpRB */
static int hf_rrc_modifedServiceList = -1;        /* MBMS_ModifedServiceList_r6 */
static int hf_rrc_mbms_ReacquireMCCH = -1;        /* T_mbms_ReacquireMCCH */
static int hf_rrc_mbms_DynamicPersistenceLevel = -1;  /* DynamicPersistenceLevel */
static int hf_rrc_endOfModifiedMCCHInformation = -1;  /* INTEGER_1_16 */
static int hf_rrc_mbmsNumberOfNeighbourCells = -1;  /* MBMS_NumberOfNeighbourCells_r6 */
static int hf_rrc_mbms_AllUnmodifiedPTMServices = -1;  /* T_mbms_AllUnmodifiedPTMServices */
static int hf_rrc_mbms_PTMActivationTime = -1;    /* MBMS_PTMActivationTime_r6 */
static int hf_rrc_v770NonCriticalExtensions_36 = -1;  /* T_v770NonCriticalExtensions_36 */
static int hf_rrc_mbmsModifiedServicesInformation_v770ext = -1;  /* MBMSModifiedServicesInformation_v770ext_IEs */
static int hf_rrc_v7c0NonCriticalExtensions = -1;  /* T_v7c0NonCriticalExtensions */
static int hf_rrc_mbmsModifiedServicesInformation_v7c0ext = -1;  /* MBMSModifiedServicesInformation_v7c0ext_IEs */
static int hf_rrc_nonCriticalExtensions_184 = -1;  /* T_nonCriticalExtensions_184 */
static int hf_rrc_modifiedServiceList = -1;       /* MBMS_ModifiedServiceList_v770ext */
static int hf_rrc_mib_ValueTag = -1;              /* MIB_ValueTag */
static int hf_rrc_modifiedServiceList_01 = -1;    /* MBMS_ModifiedServiceList_LCR_v7c0ext */
static int hf_rrc_neighbouringCellIdentity = -1;  /* IntraFreqCellID */
static int hf_rrc_neighbouringCellSCCPCHList = -1;  /* MBMS_NeighbouringCellSCCPCHList_r6 */
static int hf_rrc_v770NonCriticalExtensions_37 = -1;  /* T_v770NonCriticalExtensions_37 */
static int hf_rrc_mbmsNeighbouringCellPTMRBInformation_v770ext = -1;  /* MBMSNeighbouringCellPTMRBInformation_v770ext_IEs */
static int hf_rrc_nonCriticalExtensions_185 = -1;  /* T_nonCriticalExtensions_185 */
static int hf_rrc_neighbouringCellSCCPCHList_01 = -1;  /* MBMS_NeighbouringCellSCCPCHList_v770ext */
static int hf_rrc_serviceSchedulingInfoList = -1;  /* MBMS_ServiceSchedulingInfoList_r6 */
static int hf_rrc_nonCriticalExtensions_186 = -1;  /* T_nonCriticalExtensions_186 */
static int hf_rrc_unmodifiedServiceList = -1;     /* MBMS_UnmodifiedServiceList_r6 */
static int hf_rrc_v770NonCriticalExtensions_38 = -1;  /* T_v770NonCriticalExtensions_38 */
static int hf_rrc_mbmsUnmodifiedServicesInformation_v770ext = -1;  /* MBMSUnmodifiedServicesInformation_v770ext_IEs */
static int hf_rrc_nonCriticalExtensions_187 = -1;  /* T_nonCriticalExtensions_187 */
static int hf_rrc_unmodifiedServiceList_01 = -1;  /* MBMS_UnmodifiedServiceList_v770ext */
static int hf_rrc_mib = -1;                       /* T_mib */
static int hf_rrc_sysInfoTypeSB1 = -1;            /* T_sysInfoTypeSB1 */
static int hf_rrc_sysInfoTypeSB2 = -1;            /* T_sysInfoTypeSB2 */
static int hf_rrc_sysInfoType1 = -1;              /* T_sysInfoType1 */
static int hf_rrc_sysInfoType3 = -1;              /* T_sysInfoType3 */
static int hf_rrc_sysInfoType5 = -1;              /* T_sysInfoType5 */
static int hf_rrc_sysInfoType7 = -1;              /* T_sysInfoType7 */
static int hf_rrc_sysInfoType11 = -1;             /* T_sysInfoType11 */
static int hf_rrc_sysInfoType11bis = -1;          /* T_sysInfoType11bis */
static int hf_rrc_sysInfoType12 = -1;             /* T_sysInfoType12 */
static int hf_rrc_vbxyNonCriticalExtensions_16 = -1;  /* T_vbxyNonCriticalExtensions_16 */
static int hf_rrc_system_Information_Container_vbxyext = -1;  /* System_Information_Container_vbxyext_IEs */
static int hf_rrc_nonCriticalExtensions_188 = -1;  /* T_nonCriticalExtensions_188 */
static int hf_rrc_sysInfoType22 = -1;             /* T_sysInfoType22 */
static int hf_rrc_criticalExtensions_134 = -1;    /* T_criticalExtensions_133 */
static int hf_rrc_r10_11 = -1;                    /* T_r10_11 */
static int hf_rrc_loggingMeasurementConfiguration_r10_IEs = -1;  /* LoggingMeasurementConfiguration_r10_IEs */
static int hf_rrc_loggingMeasurementConfiguration_r10_add_ext = -1;  /* BIT_STRING */
static int hf_rrc_nonCriticalExtensions_189 = -1;  /* T_nonCriticalExtensions_189 */
static int hf_rrc_criticalExtensions_135 = -1;    /* T_criticalExtensions_134 */
static int hf_rrc_r11_12 = -1;                    /* T_r11_12 */
static int hf_rrc_loggingMeasurementConfiguration_r11_IEs = -1;  /* LoggingMeasurementConfiguration_r11_IEs */
static int hf_rrc_loggingMeasurementConfiguration_r11_add_ext = -1;  /* BIT_STRING */
static int hf_rrc_nonCriticalExtensions_190 = -1;  /* T_nonCriticalExtensions_190 */
static int hf_rrc_criticalExtensions_136 = -1;    /* T_criticalExtensions_135 */
static int hf_rrc_loggedMeasurementsConfigurationInfo = -1;  /* LoggedMeasurementsConfigurationInfo */
static int hf_rrc_loggedANRConfigurationInfo = -1;  /* LoggedANRConfigurationInfo */
static int hf_rrc_loggedMeasurementsConfigurationInfo_01 = -1;  /* LoggedMeasurementsConfigurationInfo_r11 */
static int hf_rrc_loggedMeasReport = -1;          /* LoggedMeasReport */
static int hf_rrc_loggedANRReportInfoList = -1;   /* LoggedANRReportInfoList */
static int hf_rrc_vbxyNonCriticalExtensions_17 = -1;  /* T_vbxyNonCriticalExtensions_17 */
static int hf_rrc_ueInformationResponse_vbxyext = -1;  /* UEInformationResponse_vbxyext_IEs */
static int hf_rrc_nonCriticalExtensions_191 = -1;  /* T_nonCriticalExtensions_191 */
static int hf_rrc_loggedMeasReport_01 = -1;       /* LoggedMeasReport_vbxyext */
static int hf_rrc_loggedConnectionEstablishmentFailureReport = -1;  /* LoggedConnectionEstablishmentFailureReport */
static int hf_rrc_criticalExtensions_137 = -1;    /* T_criticalExtensions_136 */
static int hf_rrc_r10_12 = -1;                    /* T_r10_12 */
static int hf_rrc_ueInformationRequest_r10 = -1;  /* UEInformationRequest_r10_IEs */
static int hf_rrc_ueInformationRequest_r10_add_ext = -1;  /* BIT_STRING */
static int hf_rrc_nonCriticalExtensions_192 = -1;  /* T_nonCriticalExtensions_192 */
static int hf_rrc_criticalExtensions_138 = -1;    /* T_criticalExtensions_137 */
static int hf_rrc_r11_13 = -1;                    /* T_r11_13 */
static int hf_rrc_ueInformationRequest_r11 = -1;  /* UEInformationRequest_r11_IEs */
static int hf_rrc_ueInformationRequest_r11_add_ext = -1;  /* BIT_STRING */
static int hf_rrc_nonCriticalExtensions_193 = -1;  /* T_nonCriticalExtensions_193 */
static int hf_rrc_criticalExtensions_139 = -1;    /* T_criticalExtensions_138 */
static int hf_rrc_loggedMeasReportRequest = -1;   /* T_loggedMeasReportRequest */
static int hf_rrc_loggedANRReportRequest = -1;    /* T_loggedANRReportRequest */
static int hf_rrc_loggedMeasReportRequest_01 = -1;  /* T_loggedMeasReportRequest_01 */
static int hf_rrc_loggedANRReportRequest_01 = -1;  /* T_loggedANRReportRequest_01 */
static int hf_rrc_loggedConnectionEstablishmentFailureRequest = -1;  /* T_loggedConnectionEstablishmentFailureRequest */
static int hf_rrc_cn_DomainSpecificNAS_Info = -1;  /* NAS_SystemInformationGSM_MAP */
static int hf_rrc_cn_DRX_CycleLengthCoeff = -1;   /* CN_DRX_CycleLengthCoefficient */
static int hf_rrc_CN_DomainInformationList_item = -1;  /* CN_DomainInformation */
static int hf_rrc_CN_DomainInformationListFull_item = -1;  /* CN_DomainInformationFull */
static int hf_rrc_cn_Type = -1;                   /* T_cn_Type */
static int hf_rrc_gsm_MAP = -1;                   /* NAS_SystemInformationGSM_MAP */
static int hf_rrc_ansi_41 = -1;                   /* NAS_SystemInformationANSI_41 */
static int hf_rrc_CN_DomainSysInfoList_item = -1;  /* CN_DomainSysInfo */
static int hf_rrc_cn_CommonGSM_MAP_NAS_SysInfo = -1;  /* T_cn_CommonGSM_MAP_NAS_SysInfo */
static int hf_rrc_cn_DomainInformationList = -1;  /* CN_DomainInformationList */
static int hf_rrc_cn_CommonGSM_MAP_NAS_SysInfo_01 = -1;  /* T_cn_CommonGSM_MAP_NAS_SysInfo_01 */
static int hf_rrc_cn_CommonGSM_MAP_NAS_SysInfo_02 = -1;  /* T_cn_CommonGSM_MAP_NAS_SysInfo_02 */
static int hf_rrc_cn_DomainInformationListFull = -1;  /* CN_DomainInformationListFull */
static int hf_rrc_cs_DomainSpecificEAB_Parameters = -1;  /* EAB_Configuration */
static int hf_rrc_ps_DomainSpecificEAB_Parameters = -1;  /* EAB_Configuration */
static int hf_rrc_commonEAB_Parameters = -1;      /* EAB_Configuration */
static int hf_rrc_EAB_AccessClassBarredList_item = -1;  /* AccessClassBarred */
static int hf_rrc_eab_Category = -1;              /* T_eab_Category */
static int hf_rrc_eab_AccessClassBarredList = -1;  /* EAB_AccessClassBarredList */
static int hf_rrc_eab_ParametersPerPLMN_List = -1;  /* EAB_ParametersPerPLMN_List */
static int hf_rrc_eab_ParametersForAll = -1;      /* DomainSpecificEAB_Parameters */
static int hf_rrc_EAB_ParametersPerPLMN_List_item = -1;  /* EAB_ParametersPerPLMN_List_item */
static int hf_rrc_domainSpecificEAB_Parameters = -1;  /* DomainSpecificEAB_Parameters */
static int hf_rrc_ec_N0 = -1;                     /* INTEGER_M24_0 */
static int hf_rrc_routingbasis = -1;              /* T_routingbasis */
static int hf_rrc_localPTMSI = -1;                /* T_localPTMSI */
static int hf_rrc_routingparameter = -1;          /* RoutingParameter */
static int hf_rrc_tMSIofsamePLMN = -1;            /* T_tMSIofsamePLMN */
static int hf_rrc_tMSIofdifferentPLMN = -1;       /* T_tMSIofdifferentPLMN */
static int hf_rrc_iMSIresponsetopaging = -1;      /* T_iMSIresponsetopaging */
static int hf_rrc_iMSIcauseUEinitiatedEvent = -1;  /* T_iMSIcauseUEinitiatedEvent */
static int hf_rrc_iMEI = -1;                      /* T_iMEI */
static int hf_rrc_spare2_01 = -1;                 /* T_spare2 */
static int hf_rrc_spare1_01 = -1;                 /* T_spare1 */
static int hf_rrc_dummy_28 = -1;                  /* BOOLEAN */
static int hf_rrc_IMEI_item = -1;                 /* IMEI_Digit */
static int hf_rrc_IMSI_GSM_MAP_item = -1;         /* Digit */
static int hf_rrc_version = -1;                   /* T_version */
static int hf_rrc_release99 = -1;                 /* T_release99 */
static int hf_rrc_cn_Type_01 = -1;                /* T_cn_Type_01 */
static int hf_rrc_gsm_Map_IDNNS = -1;             /* Gsm_map_IDNNS */
static int hf_rrc_ansi_41_IDNNS = -1;             /* Ansi_41_IDNNS */
static int hf_rrc_later = -1;                     /* T_later */
static int hf_rrc_futurecoding = -1;              /* BIT_STRING_SIZE_15 */
static int hf_rrc_lac = -1;                       /* BIT_STRING_SIZE_16 */
static int hf_rrc_LocationRegistrationAccessClassBarredList_item = -1;  /* AccessClassBarred */
static int hf_rrc_noRestriction = -1;             /* NULL */
static int hf_rrc_restriction = -1;               /* LocationRegistrationAccessClassBarredList */
static int hf_rrc_MCC_item = -1;                  /* Digit */
static int hf_rrc_MNC_item = -1;                  /* Digit */
static int hf_rrc_mibPLMN_Identity = -1;          /* BOOLEAN */
static int hf_rrc_multiplePLMNs = -1;             /* SEQUENCE_SIZE_1_5_OF_PLMN_IdentityWithOptionalMCC_r6 */
static int hf_rrc_multiplePLMNs_item = -1;        /* PLMN_IdentityWithOptionalMCC_r6 */
static int hf_rrc_pagingPermissionWithAccessControlList = -1;  /* PagingPermissionWithAccessControlList */
static int hf_rrc_pagingPermissionWithAccessControlForAll = -1;  /* PagingPermissionWithAccessControlParameters */
static int hf_rrc_pagingPermissionWithAccessControlParametersForOperator1 = -1;  /* PagingPermissionWithAccessControlParameters */
static int hf_rrc_pagingPermissionWithAccessControlParametersForOperator2 = -1;  /* PagingPermissionWithAccessControlParameters */
static int hf_rrc_pagingPermissionWithAccessControlParametersForOperator3 = -1;  /* PagingPermissionWithAccessControlParameters */
static int hf_rrc_pagingPermissionWithAccessControlParametersForOperator4 = -1;  /* PagingPermissionWithAccessControlParameters */
static int hf_rrc_pagingPermissionWithAccessControlParametersForOperator5 = -1;  /* PagingPermissionWithAccessControlParameters */
static int hf_rrc_pagingResponseRestrictionIndicator = -1;  /* T_pagingResponseRestrictionIndicator */
static int hf_rrc_locationRegistrationRestrictionIndicator = -1;  /* T_locationRegistrationRestrictionIndicator */
static int hf_rrc_locationRegistration = -1;      /* LocationRegistrationParameters */
static int hf_rrc_mcc = -1;                       /* MCC */
static int hf_rrc_mnc = -1;                       /* MNC */
static int hf_rrc_gsm_MAP_01 = -1;                /* T_gsm_MAP */
static int hf_rrc_ansi_41_01 = -1;                /* T_ansi_41 */
static int hf_rrc_p_REV = -1;                     /* P_REV */
static int hf_rrc_min_P_REV = -1;                 /* Min_P_REV */
static int hf_rrc_sid = -1;                       /* SID */
static int hf_rrc_nid = -1;                       /* NID */
static int hf_rrc_gsm_MAP_and_ANSI_41 = -1;       /* T_gsm_MAP_and_ANSI_41 */
static int hf_rrc_gsm_MAP_RAB_Identity = -1;      /* BIT_STRING_SIZE_8 */
static int hf_rrc_ansi_41_RAB_Identity = -1;      /* BIT_STRING_SIZE_8 */
static int hf_rrc_lai = -1;                       /* LAI */
static int hf_rrc_rac = -1;                       /* RoutingAreaCode */
static int hf_rrc_AccessClassBarredList_item = -1;  /* AccessClassBarred */
static int hf_rrc_cellBarred = -1;                /* CellBarred */
static int hf_rrc_cellReservedForOperatorUse = -1;  /* ReservedIndicator */
static int hf_rrc_cellReservationExtension = -1;  /* ReservedIndicator */
static int hf_rrc_accessClassBarredList = -1;     /* AccessClassBarredList */
static int hf_rrc_cellReservedForCSG = -1;        /* T_cellReservedForCSG */
static int hf_rrc_barred = -1;                    /* T_barred */
static int hf_rrc_intraFreqCellReselectionInd = -1;  /* AllowedIndicator */
static int hf_rrc_t_Barred = -1;                  /* T_Barred */
static int hf_rrc_notBarred = -1;                 /* NULL */
static int hf_rrc_CellIdentity_PerRL_List_item = -1;  /* CellIdentity */
static int hf_rrc_cell_Identity = -1;             /* CellIdentity */
static int hf_rrc_mappingInfo = -1;               /* MappingInfo */
static int hf_rrc_cellSelectQualityMeasure = -1;  /* T_cellSelectQualityMeasure */
static int hf_rrc_cpich_Ec_N0 = -1;               /* T_cpich_Ec_N0 */
static int hf_rrc_q_HYST_2_S = -1;                /* Q_Hyst_S */
static int hf_rrc_cpich_RSCP = -1;                /* NULL */
static int hf_rrc_modeSpecificInfo_16 = -1;       /* T_modeSpecificInfo_16 */
static int hf_rrc_fdd_46 = -1;                    /* T_fdd_45 */
static int hf_rrc_s_Intrasearch = -1;             /* S_SearchQual */
static int hf_rrc_s_Intersearch = -1;             /* S_SearchQual */
static int hf_rrc_s_SearchHCS = -1;               /* S_SearchRXLEV */
static int hf_rrc_rat_List = -1;                  /* RAT_FDD_InfoList */
static int hf_rrc_q_QualMin = -1;                 /* Q_QualMin */
static int hf_rrc_q_RxlevMin = -1;                /* Q_RxlevMin */
static int hf_rrc_tdd_11 = -1;                    /* T_tdd_10 */
static int hf_rrc_s_Intrasearch_01 = -1;          /* S_SearchRXLEV */
static int hf_rrc_s_Intersearch_01 = -1;          /* S_SearchRXLEV */
static int hf_rrc_rat_List_01 = -1;               /* RAT_TDD_InfoList */
static int hf_rrc_q_Hyst_l_S = -1;                /* Q_Hyst_S */
static int hf_rrc_t_Reselection_S = -1;           /* T_Reselection_S */
static int hf_rrc_hcs_ServingCellInformation = -1;  /* HCS_ServingCellInformation */
static int hf_rrc_CSG_DedicatedFrequencyInfoList_item = -1;  /* FrequencyInfo */
static int hf_rrc_startPSC = -1;                  /* INTEGER_0_63 */
static int hf_rrc_numberOfPSCs = -1;              /* T_numberOfPSCs */
static int hf_rrc_pscRange2Offset = -1;           /* INTEGER_1_63 */
static int hf_rrc_domainSpecificAccessRestictionList = -1;  /* DomainSpecificAccessRestrictionList_v670ext */
static int hf_rrc_domainSpecificAccessRestictionParametersForAll = -1;  /* DomainSpecificAccessRestrictionParam_v670ext */
static int hf_rrc_domainSpecificAccessRestrictionParametersForOperator1 = -1;  /* DomainSpecificAccessRestrictionParam_v670ext */
static int hf_rrc_domainSpecificAccessRestrictionParametersForOperator2 = -1;  /* DomainSpecificAccessRestrictionParam_v670ext */
static int hf_rrc_domainSpecificAccessRestrictionParametersForOperator3 = -1;  /* DomainSpecificAccessRestrictionParam_v670ext */
static int hf_rrc_domainSpecificAccessRestrictionParametersForOperator4 = -1;  /* DomainSpecificAccessRestrictionParam_v670ext */
static int hf_rrc_domainSpecificAccessRestrictionParametersForOperator5 = -1;  /* DomainSpecificAccessRestrictionParam_v670ext */
static int hf_rrc_cSDomainSpecificAccessRestriction = -1;  /* DomainSpecificAccessRestriction_v670ext */
static int hf_rrc_pSDomainSpecificAccessRestriction = -1;  /* DomainSpecificAccessRestriction_v670ext */
static int hf_rrc_restriction_01 = -1;            /* T_restriction */
static int hf_rrc_domainSpecficAccessClassBarredList = -1;  /* AccessClassBarredList */
static int hf_rrc_rat = -1;                       /* RAT */
static int hf_rrc_mappingFunctionParameterList = -1;  /* MappingFunctionParameterList */
static int hf_rrc_functionType = -1;              /* MappingFunctionType */
static int hf_rrc_mapParameter1 = -1;             /* MapParameter */
static int hf_rrc_mapParameter2 = -1;             /* MapParameter */
static int hf_rrc_upperLimit = -1;                /* UpperLimit */
static int hf_rrc_MappingFunctionParameterList_item = -1;  /* MappingFunctionParameter */
static int hf_rrc_MappingInfo_item = -1;          /* Mapping */
static int hf_rrc_action = -1;                    /* T_action */
static int hf_rrc_clearDedicatedPriorities = -1;  /* NULL */
static int hf_rrc_configureDedicatedPriorities = -1;  /* T_configureDedicatedPriorities */
static int hf_rrc_t_322 = -1;                     /* T_322 */
static int hf_rrc_priorityLevelList = -1;         /* PriorityLevelList */
static int hf_rrc_eutraDetection = -1;            /* BOOLEAN */
static int hf_rrc_priority = -1;                  /* INTEGER_0_maxPrio_1 */
static int hf_rrc_radioAccessTechnology = -1;     /* T_radioAccessTechnology */
static int hf_rrc_utraFDD = -1;                   /* T_utraFDD */
static int hf_rrc_utraFDD_item = -1;              /* T_utraFDD_item */
static int hf_rrc_uarfcn = -1;                    /* UARFCN */
static int hf_rrc_utraTDD = -1;                   /* T_utraTDD */
static int hf_rrc_utraTDD_item = -1;              /* T_utraTDD_item */
static int hf_rrc_eutra = -1;                     /* T_eutra */
static int hf_rrc_eutra_item = -1;                /* T_eutra_item */
static int hf_rrc_earfcn = -1;                    /* EARFCN */
static int hf_rrc_gsm_01 = -1;                    /* T_gsm_01 */
static int hf_rrc_gsm_CellGroup = -1;             /* GSM_CellGroup */
static int hf_rrc_PriorityLevelList_item = -1;    /* PriorityLevel */
static int hf_rrc_rat_Identifier = -1;            /* RAT_Identifier */
static int hf_rrc_s_SearchRAT = -1;               /* S_SearchQual */
static int hf_rrc_s_HCS_RAT = -1;                 /* S_SearchRXLEV */
static int hf_rrc_s_Limit_SearchRAT = -1;         /* S_SearchQual */
static int hf_rrc_RAT_FDD_InfoList_item = -1;     /* RAT_FDD_Info */
static int hf_rrc_s_SearchRAT_01 = -1;            /* S_SearchRXLEV */
static int hf_rrc_s_Limit_SearchRAT_01 = -1;      /* S_SearchRXLEV */
static int hf_rrc_RAT_TDD_InfoList_item = -1;     /* RAT_TDD_Info */
static int hf_rrc_URA_IdentityList_item = -1;     /* URA_Identity */
static int hf_rrc_n_AP_RetransMax = -1;           /* N_AP_RetransMax */
static int hf_rrc_n_AccessFails = -1;             /* N_AccessFails */
static int hf_rrc_nf_BO_NoAICH = -1;              /* NF_BO_NoAICH */
static int hf_rrc_ns_BO_Busy = -1;                /* NS_BO_Busy */
static int hf_rrc_nf_BO_AllBusy = -1;             /* NF_BO_AllBusy */
static int hf_rrc_nf_BO_Mismatch = -1;            /* NF_BO_Mismatch */
static int hf_rrc_t_CPCH = -1;                    /* T_CPCH */
static int hf_rrc_ue_RadioCapabilityFDDUpdateRequirement = -1;  /* BOOLEAN */
static int hf_rrc_ue_RadioCapabilityTDDUpdateRequirement = -1;  /* BOOLEAN */
static int hf_rrc_systemSpecificCapUpdateReqList = -1;  /* SystemSpecificCapUpdateReqList */
static int hf_rrc_ue_RadioCapabilityUpdateRequirement_TDD128 = -1;  /* BOOLEAN */
static int hf_rrc_ue_RadioCapabilityFDDUpdateRequirement_FDD = -1;  /* BOOLEAN */
static int hf_rrc_ue_RadioCapabilityTDDUpdateRequirement_TDD384 = -1;  /* BOOLEAN */
static int hf_rrc_ue_RadioCapabilityTDDUpdateRequirement_TDD128 = -1;  /* BOOLEAN */
static int hf_rrc_systemSpecificCapUpdateReqList_01 = -1;  /* SystemSpecificCapUpdateReqList_r5 */
static int hf_rrc_ue_RadioCapabilityTDDUpdateRequirement_TDD768 = -1;  /* BOOLEAN */
static int hf_rrc_systemSpecificCapUpdateReqList_02 = -1;  /* SystemSpecificCapUpdateReqList_r8 */
static int hf_rrc_startRestart = -1;              /* CipheringAlgorithm */
static int hf_rrc_startRestart_01 = -1;           /* CipheringAlgorithm_r7 */
static int hf_rrc_cipheringModeCommand = -1;      /* CipheringModeCommand */
static int hf_rrc_activationTimeForDPCH = -1;     /* ActivationTime */
static int hf_rrc_rb_DL_CiphActivationTimeInfo = -1;  /* RB_ActivationTimeInfoList */
static int hf_rrc_cipheringModeCommand_01 = -1;   /* CipheringModeCommand_r7 */
static int hf_rrc_imsi_GSM_MAP = -1;              /* IMSI_GSM_MAP */
static int hf_rrc_tmsi_GSM_MAP = -1;              /* TMSI_GSM_MAP */
static int hf_rrc_p_TMSI_GSM_MAP = -1;            /* P_TMSI_GSM_MAP */
static int hf_rrc_imsi_DS_41 = -1;                /* IMSI_DS_41 */
static int hf_rrc_tmsi_DS_41 = -1;                /* TMSI_DS_41 */
static int hf_rrc_fdd_Measurements = -1;          /* BOOLEAN */
static int hf_rrc_tdd_Measurements = -1;          /* BOOLEAN */
static int hf_rrc_gsm_Measurements = -1;          /* GSM_Measurements */
static int hf_rrc_multiCarrierMeasurements = -1;  /* BOOLEAN */
static int hf_rrc_tdd128_Measurements = -1;       /* BOOLEAN */
static int hf_rrc_adjacentFrequencyMeasurements = -1;  /* T_adjacentFrequencyMeasurements */
static int hf_rrc_interBandMeasurements = -1;     /* T_interBandMeasurements */
static int hf_rrc_enhancedInterFrequencyMeasurements = -1;  /* T_enhancedInterFrequencyMeasurements */
static int hf_rrc_freqSpecificCompressedMode = -1;  /* T_freqSpecificCompressedMode */
static int hf_rrc_radioFrequencyBandEUTRA = -1;   /* RadioFrequencyBandEUTRA */
static int hf_rrc_compressedMode = -1;            /* BOOLEAN */
static int hf_rrc_CompressedModeMeasCapabEUTRAList_item = -1;  /* CompressedModeMeasCapabEUTRA */
static int hf_rrc_CompressedModeMeasCapabFDDList_item = -1;  /* CompressedModeMeasCapabFDD */
static int hf_rrc_CompressedModeMeasCapabFDDList2_item = -1;  /* CompressedModeMeasCapabFDD2 */
static int hf_rrc_CompressedModeMeasCapabFDDList3_item = -1;  /* CompressedModeMeasCapabFDD3 */
static int hf_rrc_CompressedModeMeasCapabFDDList_ext_item = -1;  /* CompressedModeMeasCapabFDD_ext */
static int hf_rrc_radioFrequencyBandFDD = -1;     /* RadioFrequencyBandFDD */
static int hf_rrc_dl_MeasurementsFDD = -1;        /* BOOLEAN */
static int hf_rrc_ul_MeasurementsFDD = -1;        /* BOOLEAN */
static int hf_rrc_radioFrequencyBandFDD2 = -1;    /* RadioFrequencyBandFDD2 */
static int hf_rrc_radioFrequencyBandFDD3 = -1;    /* RadioFrequencyBandFDD3 */
static int hf_rrc_CompressedModeMeasCapabTDDList_item = -1;  /* CompressedModeMeasCapabTDD */
static int hf_rrc_radioFrequencyBandTDD = -1;     /* RadioFrequencyBandTDD */
static int hf_rrc_dl_MeasurementsTDD = -1;        /* BOOLEAN */
static int hf_rrc_ul_MeasurementsTDD = -1;        /* BOOLEAN */
static int hf_rrc_CompressedModeMeasCapabGSMList_item = -1;  /* CompressedModeMeasCapabGSM */
static int hf_rrc_radioFrequencyBandGSM = -1;     /* RadioFrequencyBandGSM */
static int hf_rrc_dl_MeasurementsGSM = -1;        /* BOOLEAN */
static int hf_rrc_ul_MeasurementsGSM = -1;        /* BOOLEAN */
static int hf_rrc_dl_MeasurementsMC = -1;         /* BOOLEAN */
static int hf_rrc_ul_MeasurementsMC = -1;         /* BOOLEAN */
static int hf_rrc_initialPriorityDelayList = -1;  /* InitialPriorityDelayList */
static int hf_rrc_backoffControlParams = -1;      /* BackoffControlParams */
static int hf_rrc_powerControlAlgorithm = -1;     /* PowerControlAlgorithm */
static int hf_rrc_dl_DPCCH_BER = -1;              /* DL_DPCCH_BER */
static int hf_rrc_supportOfIntraFreqProximityIndication = -1;  /* T_supportOfIntraFreqProximityIndication */
static int hf_rrc_supportOfInterFreqProximityIndication = -1;  /* T_supportOfInterFreqProximityIndication */
static int hf_rrc_supportOfE_UtraProximityIndication = -1;  /* T_supportOfE_UtraProximityIndication */
static int hf_rrc_DL_InformationPerSecondaryRL_List_item = -1;  /* DL_InformationPerSecondaryRL */
static int hf_rrc_primaryCPICH_Info = -1;         /* PrimaryCPICH_Info */
static int hf_rrc_cell_id = -1;                   /* CellIdentity */
static int hf_rrc_dl_FDPCHInfoPerRL_SecULFreq = -1;  /* DL_FDPCH_InfoPerRL_r7 */
static int hf_rrc_e_AGCH_Information = -1;        /* E_AGCH_Information */
static int hf_rrc_e_HICH_Information = -1;        /* E_HICH_Information */
static int hf_rrc_e_RGCH_Info = -1;               /* T_e_RGCH_Info */
static int hf_rrc_e_RGCH_Information = -1;        /* E_RGCH_Information */
static int hf_rrc_releaseIndicator = -1;          /* NULL */
static int hf_rrc_maxNoDPCH_PDSCH_Codes = -1;     /* INTEGER_1_8 */
static int hf_rrc_maxNoPhysChBitsReceived = -1;   /* MaxNoPhysChBitsReceived */
static int hf_rrc_supportForSF_512 = -1;          /* BOOLEAN */
static int hf_rrc_dummy2_05 = -1;                 /* SimultaneousSCCPCH_DPCH_Reception */
static int hf_rrc_dummy_29 = -1;                  /* SupportOfDedicatedPilotsForChEstimation */
static int hf_rrc_hsdsch_physical_layer_category_ext = -1;  /* HSDSCH_physical_layer_category_ext */
static int hf_rrc_hsscchlessHsdschOperation = -1;  /* T_hsscchlessHsdschOperation */
static int hf_rrc_enhancedFdpch = -1;             /* T_enhancedFdpch */
static int hf_rrc_hsdschReception_CellFach = -1;  /* T_hsdschReception_CellFach */
static int hf_rrc_hsdschReception_CellUraPch = -1;  /* T_hsdschReception_CellUraPch */
static int hf_rrc_hsdsch_physical_layer_category_ext2 = -1;  /* HSDSCH_physical_layer_category_ext2 */
static int hf_rrc_supportOfTargetCellPreConfig = -1;  /* T_supportOfTargetCellPreConfig */
static int hf_rrc_supportOfHsdschDrxOperation = -1;  /* T_supportOfHsdschDrxOperation */
static int hf_rrc_hsdsch_physical_layer_category_ext3 = -1;  /* HSDSCH_physical_layer_category_ext3 */
static int hf_rrc_supportOfMimoOnlySingleStream = -1;  /* T_supportOfMimoOnlySingleStream */
static int hf_rrc_hsdsch_physical_layer_category_ext4 = -1;  /* HSDSCH_physical_layer_category_ext4 */
static int hf_rrc_hsdsch_physical_layer_category_ext5 = -1;  /* HSDSCH_physical_layer_category_ext5 */
static int hf_rrc_hsdsch_physical_layer_category_ext6 = -1;  /* HSDSCH_physical_layer_category_ext6 */
static int hf_rrc_hsdsch_physical_layer_category_ext7 = -1;  /* HSDSCH_physical_layer_category_ext7 */
static int hf_rrc_hsdsch_physical_layer_category_ext8 = -1;  /* HSDSCH_physical_layer_category_ext8 */
static int hf_rrc_supportOf4x4MimoDualStreamRestriction = -1;  /* T_supportOf4x4MimoDualStreamRestriction */
static int hf_rrc_supportOfNodeBTrigHS_DPCCHTransmission = -1;  /* T_supportOfNodeBTrigHS_DPCCHTransmission */
static int hf_rrc_supportOfHS_DSCHSecondDRXOperation = -1;  /* T_supportOfHS_DSCHSecondDRXOperation */
static int hf_rrc_maxTS_PerFrame = -1;            /* MaxTS_PerFrame */
static int hf_rrc_maxPhysChPerFrame = -1;         /* MaxPhysChPerFrame */
static int hf_rrc_minimumSF = -1;                 /* MinimumSF_DL */
static int hf_rrc_supportOfPDSCH = -1;            /* BOOLEAN */
static int hf_rrc_maxPhysChPerTS = -1;            /* MaxPhysChPerTS */
static int hf_rrc_maxTS_PerSubFrame = -1;         /* MaxTS_PerSubFrame_r4 */
static int hf_rrc_maxPhysChPerFrame_01 = -1;      /* MaxPhysChPerSubFrame_r4 */
static int hf_rrc_supportOf8PSK = -1;             /* BOOLEAN */
static int hf_rrc_multiCarrier_physical_layer_category = -1;  /* MultiCarrier_HSDSCH_physical_layer_category */
static int hf_rrc_hSDSCH_physical_layer_category_extension = -1;  /* HSDSCH_physical_layer_category_extension */
static int hf_rrc_multiCarrier_physical_layer_category_extension = -1;  /* MultiCarrier_HSDSCH_physical_layer_category_extension */
static int hf_rrc_supportOfSFModeForHSPDSCHDualStream = -1;  /* T_supportOfSFModeForHSPDSCHDualStream */
static int hf_rrc_multiCarrier_physical_layer_category_extension2 = -1;  /* MultiCarrier_HSDSCH_physical_layer_category_extension2 */
static int hf_rrc_maxPhysChPerFrame_02 = -1;      /* MaxPhysChPerFrame_768 */
static int hf_rrc_minimumSF_01 = -1;              /* MinimumSF_DL_768 */
static int hf_rrc_tdd768_hspdsch = -1;            /* T_tdd768_hspdsch */
static int hf_rrc_supported = -1;                 /* HSDSCH_physical_layer_category */
static int hf_rrc_unsupported = -1;               /* NULL */
static int hf_rrc_maxPhysChPerTS_01 = -1;         /* MaxPhysChPerTS_768 */
static int hf_rrc_maxNoBitsReceived = -1;         /* MaxNoBits */
static int hf_rrc_maxConvCodeBitsReceived = -1;   /* MaxNoBits */
static int hf_rrc_turboDecodingSupport = -1;      /* TurboSupport */
static int hf_rrc_maxSimultaneousTransChs = -1;   /* MaxSimultaneousTransChsDL */
static int hf_rrc_maxSimultaneousCCTrCH_Count = -1;  /* MaxSimultaneousCCTrCH_Count */
static int hf_rrc_maxReceivedTransportBlocks = -1;  /* MaxTransportBlocksDL */
static int hf_rrc_maxNumberOfTFC = -1;            /* MaxNumberOfTFC_DL */
static int hf_rrc_maxNumberOfTF = -1;             /* MaxNumberOfTF */
static int hf_rrc_transmissionProbability = -1;   /* TransmissionProbability */
static int hf_rrc_maximumBitRate = -1;            /* MaximumBitRate */
static int hf_rrc_DRAC_SysInfoList_item = -1;     /* DRAC_SysInfo */
static int hf_rrc_configurationUnsupported = -1;  /* NULL */
static int hf_rrc_physicalChannelFailure = -1;    /* NULL */
static int hf_rrc_incompatibleSimultaneousReconfiguration = -1;  /* NULL */
static int hf_rrc_compressedModeRuntimeError = -1;  /* TGPSI */
static int hf_rrc_protocolError = -1;             /* ProtocolErrorInformation */
static int hf_rrc_cellUpdateOccurred = -1;        /* NULL */
static int hf_rrc_invalidConfiguration = -1;      /* NULL */
static int hf_rrc_configurationIncomplete = -1;   /* NULL */
static int hf_rrc_unsupportedMeasurement = -1;    /* NULL */
static int hf_rrc_mbmsSessionAlreadyReceivedCorrectly = -1;  /* NULL */
static int hf_rrc_lowerPriorityMBMSService = -1;  /* NULL */
static int hf_rrc_modeSpecificInfo_17 = -1;       /* T_modeSpecificInfo_17 */
static int hf_rrc_fdd_47 = -1;                    /* T_fdd_46 */
static int hf_rrc_frequencyBandsSupportFDD = -1;  /* FrequencyBandsRedirectionFDD */
static int hf_rrc_tdd_12 = -1;                    /* T_tdd_11 */
static int hf_rrc_frequencyBandsSupportTDD128 = -1;  /* FrequencyBandsIndicatorTDD128 */
static int hf_rrc_FrequencyBandsIndicatorTDD128_item = -1;  /* FrequencyBandsIndicatorSupportTDD128 */
static int hf_rrc_frequencyBandsIndicatorTDD = -1;  /* RadioFrequencyBandTDD_r10 */
static int hf_rrc_FrequencyBandsRedirectionFDD_item = -1;  /* FrequencyBandsIndicatorFDD */
static int hf_rrc_frequencyBandsIndicator1 = -1;  /* RadioFrequencyBandFDD */
static int hf_rrc_frequencyBandsIndicator2 = -1;  /* RadioFrequencyBandFDD2 */
static int hf_rrc_frequencyBandsIndicator3 = -1;  /* RadioFrequencyBandFDD3 */
static int hf_rrc_rrc_ConnectionReleaseInformation = -1;  /* RRC_ConnectionReleaseInformation */
static int hf_rrc_groupReleaseInformation = -1;   /* GroupReleaseInformation */
static int hf_rrc_uRNTI_Group = -1;               /* U_RNTI_Group */
static int hf_rrc_gsm900 = -1;                    /* BOOLEAN */
static int hf_rrc_dcs1800 = -1;                   /* BOOLEAN */
static int hf_rrc_gsm1900 = -1;                   /* BOOLEAN */
static int hf_rrc_numberOfCellsBandA = -1;        /* INTEGER_2_6 */
static int hf_rrc_numberOfFrequenciesBandA = -1;  /* INTEGER_1_3 */
static int hf_rrc_numberOfCellsBandB = -1;        /* INTEGER_2_6 */
static int hf_rrc_numberOfFrequenciesBandB = -1;  /* INTEGER_1_3 */
static int hf_rrc_MultiflowBandCombList_item = -1;  /* MultiflowBandCombination */
static int hf_rrc_mimoInDifferentBands = -1;      /* MultiflowMIMOCapability */
static int hf_rrc_longerHARQProcessing = -1;      /* T_longerHARQProcessing */
static int hf_rrc_numberOfCells = -1;             /* INTEGER_2_8 */
static int hf_rrc_numberOfFrequencies = -1;       /* INTEGER_1_4 */
static int hf_rrc_mimo = -1;                      /* MultiflowMIMOCapability */
static int hf_rrc_nonContiguousCapability = -1;   /* MultiflowNonContiguousCapability */
static int hf_rrc_gapSize = -1;                   /* T_gapSize */
static int hf_rrc_idleInterval = -1;              /* BOOLEAN */
static int hf_rrc_IdleIntervalMeasCapabEUTRAList_item = -1;  /* IdleIntervalMeasCapabEUTRA */
static int hf_rrc_esn_DS_41 = -1;                 /* ESN_DS_41 */
static int hf_rrc_InitialPriorityDelayList_item = -1;  /* NS_IP */
static int hf_rrc_imsi = -1;                      /* IMSI_GSM_MAP */
static int hf_rrc_tmsi_and_LAI = -1;              /* TMSI_and_LAI_GSM_MAP */
static int hf_rrc_p_TMSI_and_RAI = -1;            /* P_TMSI_and_RAI_GSM_MAP */
static int hf_rrc_imei = -1;                      /* IMEI */
static int hf_rrc_imsi_and_ESN_DS_41 = -1;        /* IMSI_and_ESN_DS_41 */
static int hf_rrc_messageAuthenticationCode = -1;  /* MessageAuthenticationCode */
static int hf_rrc_rrc_MessageSequenceNumber = -1;  /* RRC_MessageSequenceNumber */
static int hf_rrc_rrc_MessageSequenceNumberList = -1;  /* RRC_MessageSequenceNumberList */
static int hf_rrc_startIntegrityProtection = -1;  /* T_startIntegrityProtection */
static int hf_rrc_integrityProtInitNumber = -1;   /* IntegrityProtInitNumber */
static int hf_rrc_modify = -1;                    /* T_modify */
static int hf_rrc_dl_IntegrityProtActivationInfo = -1;  /* IntegrityProtActivationInfo */
static int hf_rrc_integrityProtectionModeCommand = -1;  /* IntegrityProtectionModeCommand */
static int hf_rrc_integrityProtectionAlgorithm = -1;  /* IntegrityProtectionAlgorithm */
static int hf_rrc_integrityProtectionAlgorithm_01 = -1;  /* IntegrityProtectionAlgorithm_r7 */
static int hf_rrc_downlinkCompressedMode = -1;    /* CompressedModeMeasCapability */
static int hf_rrc_uplinkCompressedMode = -1;      /* CompressedModeMeasCapability */
static int hf_rrc_compressedModeMeasCapabFDDList = -1;  /* CompressedModeMeasCapabFDDList */
static int hf_rrc_compressedModeMeasCapabTDDList = -1;  /* CompressedModeMeasCapabTDDList */
static int hf_rrc_compressedModeMeasCapabGSMList = -1;  /* CompressedModeMeasCapabGSMList */
static int hf_rrc_compressedModeMeasCapabMC = -1;  /* CompressedModeMeasCapabMC */
static int hf_rrc_compressedModeMeasCapabFDDList_01 = -1;  /* CompressedModeMeasCapabFDDList2 */
static int hf_rrc_compressedModeMeasCapabEUTRAList = -1;  /* CompressedModeMeasCapabEUTRAList */
static int hf_rrc_compressedModeMeasCapabFDDList2 = -1;  /* CompressedModeMeasCapabFDDList2 */
static int hf_rrc_compressedModeMeasCapabFDDList3 = -1;  /* CompressedModeMeasCapabFDDList3 */
static int hf_rrc_downlinkCompressedMode_LCR = -1;  /* CompressedModeMeasCapability_LCR_r4 */
static int hf_rrc_uplinkCompressedMode_LCR = -1;  /* CompressedModeMeasCapability_LCR_r4 */
static int hf_rrc_downlinkCompressedMode_01 = -1;  /* CompressedModeMeasCapability_v860ext */
static int hf_rrc_downlinkCompressedMode_02 = -1;  /* CompressedModeMeasCapability_v920ext */
static int hf_rrc_interFrequencyDetectedSetMeasurements = -1;  /* T_interFrequencyDetectedSetMeasurements */
static int hf_rrc_downlinkCompressedMode_03 = -1;  /* CompressedModeMeasCapability_va40ext */
static int hf_rrc_interFrequencyMeasOnConfigCarriersWithoutCompMode = -1;  /* T_interFrequencyMeasOnConfigCarriersWithoutCompMode */
static int hf_rrc_idleIntervalMeasCapabEUTRAList = -1;  /* IdleIntervalMeasCapabEUTRAList */
static int hf_rrc_NonContiguousMultiCellCombinationList_item = -1;  /* NonContiguousMultiCell */
static int hf_rrc_aggregatedCells = -1;           /* T_aggregatedCells */
static int hf_rrc_gapSize_01 = -1;                /* T_gapSize_01 */
static int hf_rrc_nonContiguousMultiCellCombination22 = -1;  /* T_nonContiguousMultiCellCombination22 */
static int hf_rrc_nonContiguousMultiCellCombination31_13 = -1;  /* T_nonContiguousMultiCellCombination31_13 */
static int hf_rrc_supportOfGSM = -1;              /* BOOLEAN */
static int hf_rrc_supportOfMulticarrier = -1;     /* BOOLEAN */
static int hf_rrc_supportOfUTRAN_ToGERAN_NACC = -1;  /* BOOLEAN */
static int hf_rrc_supportOfHandoverToGAN = -1;    /* T_supportOfHandoverToGAN */
static int hf_rrc_supportOfPSHandoverToGAN = -1;  /* T_supportOfPSHandoverToGAN */
static int hf_rrc_supportOfEUTRAFDD = -1;         /* T_supportOfEUTRAFDD */
static int hf_rrc_supportOfInterRATHOToEUTRAFDD = -1;  /* T_supportOfInterRATHOToEUTRAFDD */
static int hf_rrc_supportOfEUTRATDD = -1;         /* T_supportOfEUTRATDD */
static int hf_rrc_supportOfInterRATHOToEUTRATDD = -1;  /* T_supportOfInterRATHOToEUTRATDD */
static int hf_rrc_eutraFeatureGroupIndicators = -1;  /* T_eutraFeatureGroupIndicators */
static int hf_rrc_supportOfEUTRAFDDMeasurementReportingInCELLFACH = -1;  /* T_supportOfEUTRAFDDMeasurementReportingInCELLFACH */
static int hf_rrc_supportOfEUTRATDDMeasurementReportingInCELLFACH = -1;  /* T_supportOfEUTRATDDMeasurementReportingInCELLFACH */
static int hf_rrc_supportOfIntraFreqSIAcquisitionForHO = -1;  /* T_supportOfIntraFreqSIAcquisitionForHO */
static int hf_rrc_supportOfInterFreqSIAcquisitionForHO = -1;  /* T_supportOfInterFreqSIAcquisitionForHO */
static int hf_rrc_supportOfE_UtraSIAcquisitionForHO = -1;  /* T_supportOfE_UtraSIAcquisitionForHO */
static int hf_rrc_NetworkAssistedGANSS_Supported_List_item = -1;  /* NetworkAssistedGANSS_Supported_List_item */
static int hf_rrc_gANSS_Id = -1;                  /* T_gANSS_Id */
static int hf_rrc_gANSS_Mode = -1;                /* GANSS_Mode */
static int hf_rrc_gANSS_SignalId = -1;            /* GANSS_Signal_Id */
static int hf_rrc_supportForUE_GANSS_TimingOfCellFrames = -1;  /* BOOLEAN */
static int hf_rrc_supportForUE_GANSS_CarrierPhaseMeasurement = -1;  /* BOOLEAN */
static int hf_rrc_NetworkAssistedGANSS_Supported_List_v860ext_item = -1;  /* NetworkAssistedGANSS_Supported_List_v860ext_item */
static int hf_rrc_sbas_Ids = -1;                  /* BIT_STRING_SIZE_8 */
static int hf_rrc_gANSS_SignalIds = -1;           /* BIT_STRING_SIZE_8 */
static int hf_rrc_non_native_AD_choices_supported = -1;  /* T_non_native_AD_choices_supported */
static int hf_rrc_p_TMSI = -1;                    /* P_TMSI_GSM_MAP */
static int hf_rrc_rai = -1;                       /* RAI */
static int hf_rrc_cn_Identity = -1;               /* T_cn_Identity */
static int hf_rrc_cn_pagedUE_Identity = -1;       /* CN_PagedUE_Identity */
static int hf_rrc_utran_Identity = -1;            /* T_utran_Identity */
static int hf_rrc_cn_OriginatedPage_connectedMode_UE = -1;  /* T_cn_OriginatedPage_connectedMode_UE */
static int hf_rrc_utran_SingleUE_Identity = -1;   /* T_utran_SingleUE_Identity */
static int hf_rrc_cn_OriginatedPage_connectedMode_UE_01 = -1;  /* T_cn_OriginatedPage_connectedMode_UE_01 */
static int hf_rrc_utran_GroupIdentity = -1;       /* SEQUENCE_SIZE_1_maxURNTI_Group_OF_GroupIdentityWithReleaseInformation */
static int hf_rrc_utran_GroupIdentity_item = -1;  /* GroupIdentityWithReleaseInformation */
static int hf_rrc_PagingRecordList_item = -1;     /* PagingRecord */
static int hf_rrc_PagingRecord2List_r5_item = -1;  /* PagingRecord2_r5 */
static int hf_rrc_losslessSRNS_RelocationSupport = -1;  /* BOOLEAN */
static int hf_rrc_supportForRfc2507 = -1;         /* T_supportForRfc2507 */
static int hf_rrc_notSupported = -1;              /* NULL */
static int hf_rrc_supported_01 = -1;              /* MaxHcContextSpace */
static int hf_rrc_supportForRfc3095 = -1;         /* T_supportForRfc3095 */
static int hf_rrc_supported_02 = -1;              /* T_supported */
static int hf_rrc_maxROHC_ContextSessions = -1;   /* MaxROHC_ContextSessions_r4 */
static int hf_rrc_reverseCompressionDepth = -1;   /* INTEGER_0_65535 */
static int hf_rrc_supportForRfc3095ContextRelocation = -1;  /* BOOLEAN */
static int hf_rrc_maxHcContextSpace = -1;         /* MaxHcContextSpace_r5_ext */
static int hf_rrc_losslessDLRLC_PDUSizeChange = -1;  /* T_losslessDLRLC_PDUSizeChange */
static int hf_rrc_supportForCSVoiceoverHSPA_01 = -1;  /* T_supportForCSVoiceoverHSPA_01 */
static int hf_rrc_fddPhysChCapability = -1;       /* T_fddPhysChCapability */
static int hf_rrc_downlinkPhysChCapability = -1;  /* DL_PhysChCapabilityFDD */
static int hf_rrc_uplinkPhysChCapability = -1;    /* UL_PhysChCapabilityFDD */
static int hf_rrc_tddPhysChCapability = -1;       /* T_tddPhysChCapability */
static int hf_rrc_downlinkPhysChCapability_01 = -1;  /* DL_PhysChCapabilityTDD */
static int hf_rrc_uplinkPhysChCapability_01 = -1;  /* UL_PhysChCapabilityTDD */
static int hf_rrc_fddPhysChCapability_01 = -1;    /* T_fddPhysChCapability_01 */
static int hf_rrc_downlinkPhysChCapability_02 = -1;  /* DL_PhysChCapabilityFDD_v770ext */
static int hf_rrc_uplinkPhysChCapability_02 = -1;  /* UL_PhysChCapabilityFDD_v770ext */
static int hf_rrc_tddPhysChCapability_384 = -1;   /* T_tddPhysChCapability_384 */
static int hf_rrc_uplinkPhysChCapability_03 = -1;  /* UL_PhysChCapabilityTDD_384_v770ext */
static int hf_rrc_tddPhysChCapability_768 = -1;   /* T_tddPhysChCapability_768 */
static int hf_rrc_downlinkPhysChCapability_03 = -1;  /* DL_PhysChCapabilityTDD_768 */
static int hf_rrc_uplinkPhysChCapability_04 = -1;  /* UL_PhysChCapabilityTDD_768 */
static int hf_rrc_tddPhysChCapability_128 = -1;   /* T_tddPhysChCapability_128 */
static int hf_rrc_downlinkPhysChCapability_04 = -1;  /* DL_PhysChCapabilityTDD_128_v770ext */
static int hf_rrc_uplinkPhysChCapability_05 = -1;  /* UL_PhysChCapabilityTDD_128_v770ext */
static int hf_rrc_fddPhysChCapability_02 = -1;    /* T_fddPhysChCapability_02 */
static int hf_rrc_downlinkPhysChCapability_05 = -1;  /* DL_PhysChCapabilityFDD_v860ext */
static int hf_rrc_tddPhysChCapability_128_01 = -1;  /* T_tddPhysChCapability_128_01 */
static int hf_rrc_downlinkPhysChCapability_06 = -1;  /* DL_PhysChCapabilityTDD_128_v860ext */
static int hf_rrc_fddPhysChCapability_03 = -1;    /* T_fddPhysChCapability_03 */
static int hf_rrc_downlinkPhysChCapability_07 = -1;  /* DL_PhysChCapabilityFDD_v920ext */
static int hf_rrc_uplinkPhysChCapability_06 = -1;  /* UL_PhysChCapabilityFDD_v920ext */
static int hf_rrc_fddPhysChCapability_04 = -1;    /* T_fddPhysChCapability_04 */
static int hf_rrc_downlinkPhysChCapability_08 = -1;  /* DL_PhysChCapabilityFDD_va40ext */
static int hf_rrc_tddPhysChCapability_128_02 = -1;  /* T_tddPhysChCapability_128_02 */
static int hf_rrc_downlinkPhysChCapability_09 = -1;  /* DL_PhysChCapabilityInfoTDD_128_va40ext */
static int hf_rrc_uplinkPhysChCapability_07 = -1;  /* UL_PhysChCapabilityInfoTDD_128_va40ext */
static int hf_rrc_fddPhysChCapability_05 = -1;    /* T_fddPhysChCapability_05 */
static int hf_rrc_downlinkPhysChCapability_10 = -1;  /* DL_PhysChCapabilityFDD_vbxyext */
static int hf_rrc_tddPhysChCapability_768_01 = -1;  /* T_tddPhysChCapability_768_01 */
static int hf_rrc_downlinkPhysChCapability_11 = -1;  /* DL_PhysChCapabilityInfoTDD_768 */
static int hf_rrc_tddPhysChCapability_128_03 = -1;  /* T_tddPhysChCapability_128_03 */
static int hf_rrc_uplinkPhysChCapability_08 = -1;  /* UL_PhysChCapabilityInfoTDD_128_v770ext */
static int hf_rrc_tdd128_PhysChCapability = -1;   /* T_tdd128_PhysChCapability */
static int hf_rrc_downlinkPhysChCapability_12 = -1;  /* DL_PhysChCapabilityTDD_LCR_r4 */
static int hf_rrc_uplinkPhysChCapability_09 = -1;  /* UL_PhysChCapabilityTDD_LCR_r4 */
static int hf_rrc_fdd_hspdsch = -1;               /* T_fdd_hspdsch */
static int hf_rrc_supported_03 = -1;              /* T_supported_01 */
static int hf_rrc_hsdsch_physical_layer_category = -1;  /* HSDSCH_physical_layer_category */
static int hf_rrc_dummy2_06 = -1;                 /* BOOLEAN */
static int hf_rrc_tdd384_hspdsch = -1;            /* T_tdd384_hspdsch */
static int hf_rrc_tdd128_hspdsch = -1;            /* T_tdd128_hspdsch */
static int hf_rrc_numberOfRepetitionsPerSFNPeriod = -1;  /* T_numberOfRepetitionsPerSFNPeriod */
static int hf_rrc_supportEUTRA_FDD = -1;          /* BOOLEAN */
static int hf_rrc_supportEUTRA_TDD = -1;          /* BOOLEAN */
static int hf_rrc_noError = -1;                   /* NULL */
static int hf_rrc_errorOccurred = -1;             /* T_errorOccurred */
static int hf_rrc_protocolErrorInformation_01 = -1;  /* ProtocolErrorInformation */
static int hf_rrc_diagnosticsType = -1;           /* T_diagnosticsType */
static int hf_rrc_type1 = -1;                     /* T_type1 */
static int hf_rrc_asn1_ViolationOrEncodingError = -1;  /* NULL */
static int hf_rrc_messageTypeNonexistent = -1;    /* NULL */
static int hf_rrc_messageNotCompatibleWithReceiverState = -1;  /* IdentificationOfReceivedMessage */
static int hf_rrc_ie_ValueNotComprehended = -1;   /* IdentificationOfReceivedMessage */
static int hf_rrc_conditionalInformationElementError = -1;  /* IdentificationOfReceivedMessage */
static int hf_rrc_messageExtensionNotComprehended = -1;  /* IdentificationOfReceivedMessage */
static int hf_rrc_RadioFrequencyBandTDDextList_item = -1;  /* RadioFrequencyBandTDDext */
static int hf_rrc_t314_expired = -1;              /* BOOLEAN */
static int hf_rrc_t315_expired = -1;              /* BOOLEAN */
static int hf_rrc_interRATInfo = -1;              /* InterRATInfo */
static int hf_rrc_interRATInfo_01 = -1;           /* InterRATInfo_r6 */
static int hf_rrc_frequencyInfo_03 = -1;          /* NULL */
static int hf_rrc_interRATInfo_02 = -1;           /* InterRATInfo_v860ext */
static int hf_rrc_fddRF_Capability = -1;          /* T_fddRF_Capability */
static int hf_rrc_ue_PowerClass = -1;             /* UE_PowerClass */
static int hf_rrc_txRxFrequencySeparation = -1;   /* TxRxFrequencySeparation */
static int hf_rrc_tddRF_Capability = -1;          /* T_tddRF_Capability */
static int hf_rrc_radioFrequencyTDDBandList = -1;  /* RadioFrequencyBandTDDList */
static int hf_rrc_chipRateCapability = -1;        /* ChipRateCapability */
static int hf_rrc_tddRF_Capability_01 = -1;       /* T_tddRF_Capability_01 */
static int hf_rrc_radioFrequencyBandTDDList = -1;  /* RadioFrequencyBandTDDList */
static int hf_rrc_tdd384RF_Capability = -1;       /* RadioFrequencyBandTDDList_r7 */
static int hf_rrc_tdd768RF_Capability = -1;       /* T_tdd768RF_Capability */
static int hf_rrc_radioFrequencyBandTDDList_01 = -1;  /* RadioFrequencyBandTDDList_r7 */
static int hf_rrc_tdd128RF_Capability = -1;       /* RadioFrequencyBandTDDList_r7 */
static int hf_rrc_tdd128RF_Capability_01 = -1;    /* RadioFrequencyBandTDDextList */
static int hf_rrc_tdd384RF_Capability_01 = -1;    /* RadioFrequencyBandTDDextList */
static int hf_rrc_tdd768RF_Capability_01 = -1;    /* RadioFrequencyBandTDDextList */
static int hf_rrc_supportOfMFBI = -1;             /* T_supportOfMFBI */
static int hf_rrc_totalRLC_AM_BufferSize = -1;    /* TotalRLC_AM_BufferSize */
static int hf_rrc_maximumRLC_WindowSize = -1;     /* MaximumRLC_WindowSize */
static int hf_rrc_maximumAM_EntityNumber = -1;    /* MaximumAM_EntityNumberRLC_Cap */
static int hf_rrc_totalRLC_AM_BufferSize_01 = -1;  /* TotalRLC_AM_BufferSize_r5_ext */
static int hf_rrc_supportOfTwoLogicalChannel = -1;  /* BOOLEAN */
static int hf_rrc_totalRLC_AM_BufferSize_02 = -1;  /* TotalRLC_AM_BufferSize_v920ext */
static int hf_rrc_totalRLC_AM_BufferSize_03 = -1;  /* TotalRLC_AM_BufferSize_va40ext */
static int hf_rrc_totalRLC_AM_BufferSize_04 = -1;  /* TotalRLC_AM_BufferSize_vbxyext */
static int hf_rrc_noRelease = -1;                 /* NULL */
static int hf_rrc_release = -1;                   /* T_release */
static int hf_rrc_RRC_MessageSequenceNumberList_item = -1;  /* RRC_MessageSequenceNumber */
static int hf_rrc_nonce = -1;                     /* BIT_STRING_SIZE_128 */
static int hf_rrc_ims_Information = -1;           /* OCTET_STRING_SIZE_1_32 */
static int hf_rrc_cipheringAlgorithmCap = -1;     /* T_cipheringAlgorithmCap */
static int hf_rrc_integrityProtectionAlgorithmCap = -1;  /* T_integrityProtectionAlgorithmCap */
static int hf_rrc_primary_E_RNTI = -1;            /* E_RNTI */
static int hf_rrc_secondary_E_RNTI = -1;          /* E_RNTI */
static int hf_rrc_scramblingCodeType = -1;        /* ScramblingCodeType */
static int hf_rrc_scramblingCodeNumber = -1;      /* UL_ScramblingCode */
static int hf_rrc_ms2_SchedTransmGrantHARQAlloc = -1;  /* BIT_STRING_SIZE_8 */
static int hf_rrc_servingGrant = -1;              /* T_servingGrant */
static int hf_rrc_primary_Secondary_GrantSelector = -1;  /* T_primary_Secondary_GrantSelector */
static int hf_rrc_minReduced_E_DPDCH_GainFactor = -1;  /* MinReduced_E_DPDCH_GainFactor */
static int hf_rrc_e_DCH_minimumSet_E_TFCI = -1;   /* E_DCH_MinimumSet_E_TFCI */
static int hf_rrc_dpcchPowerOffset_SecondaryULFrequency = -1;  /* INTEGER_0_7 */
static int hf_rrc_pc_Preamble = -1;               /* PC_Preamble */
static int hf_rrc_deltaACK = -1;                  /* DeltaACK */
static int hf_rrc_deltaNACK = -1;                 /* DeltaNACK */
static int hf_rrc_dl_hspdsch_Information = -1;    /* DL_HSPDSCH_Information */
static int hf_rrc_harqInfo_02 = -1;               /* HARQ_Info */
static int hf_rrc_mac_hsResetIndicator = -1;      /* T_mac_hsResetIndicator */
static int hf_rrc_dl_hspdsch_Information_01 = -1;  /* DL_HSPDSCH_Information_r7 */
static int hf_rrc_mac_hsResetIndicator_01 = -1;   /* T_mac_hsResetIndicator_01 */
static int hf_rrc_dl_hspdsch_Information_02 = -1;  /* DL_HSPDSCH_Information_r8 */
static int hf_rrc_mac_hsResetIndicator_02 = -1;   /* T_mac_hsResetIndicator_02 */
static int hf_rrc_dl_hspdsch_Information_03 = -1;  /* DL_HSPDSCH_Information_r9 */
static int hf_rrc_mac_hsResetIndicator_03 = -1;   /* T_mac_hsResetIndicator_03 */
static int hf_rrc_deltaACK_01 = -1;               /* DeltaACK_r11 */
static int hf_rrc_deltaNACK_01 = -1;              /* DeltaNACK_r11 */
static int hf_rrc_dl_hspdsch_Information_04 = -1;  /* DL_HSPDSCH_Information_r11 */
static int hf_rrc_mac_hsResetIndicator_04 = -1;   /* T_mac_hsResetIndicator_04 */
static int hf_rrc_mac_hsResetIndicator_assisting = -1;  /* T_mac_hsResetIndicator_assisting */
static int hf_rrc_supported_04 = -1;              /* T_supported_02 */
static int hf_rrc_maxNoSCCPCH_RL = -1;            /* MaxNoSCCPCH_RL */
static int hf_rrc_simultaneousSCCPCH_DPCH_DPDCH_Reception = -1;  /* BOOLEAN */
static int hf_rrc_rab_InfoReplace = -1;           /* RAB_InfoReplace */
static int hf_rrc_STARTList_item = -1;            /* STARTSingle */
static int hf_rrc_SystemSpecificCapUpdateReqList_item = -1;  /* SystemSpecificCapUpdateReq */
static int hf_rrc_SystemSpecificCapUpdateReqList_r5_item = -1;  /* SystemSpecificCapUpdateReq_r5 */
static int hf_rrc_SystemSpecificCapUpdateReqList_r8_item = -1;  /* SystemSpecificCapUpdateReq_r8 */
static int hf_rrc_tmsi = -1;                      /* TMSI_GSM_MAP */
static int hf_rrc_dl_TransChCapability = -1;      /* DL_TransChCapability */
static int hf_rrc_ul_TransChCapability = -1;      /* UL_TransChCapability */
static int hf_rrc_supported_05 = -1;              /* MaxNoBits */
static int hf_rrc_srnc_Identity = -1;             /* SRNC_Identity */
static int hf_rrc_s_RNTI = -1;                    /* S_RNTI */
static int hf_rrc_all = -1;                       /* NULL */
static int hf_rrc_u_RNTI_BitMaskIndex_b1 = -1;    /* BIT_STRING_SIZE_31 */
static int hf_rrc_u_RNTI_BitMaskIndex_b2 = -1;    /* BIT_STRING_SIZE_30 */
static int hf_rrc_u_RNTI_BitMaskIndex_b3 = -1;    /* BIT_STRING_SIZE_29 */
static int hf_rrc_u_RNTI_BitMaskIndex_b4 = -1;    /* BIT_STRING_SIZE_28 */
static int hf_rrc_u_RNTI_BitMaskIndex_b5 = -1;    /* BIT_STRING_SIZE_27 */
static int hf_rrc_u_RNTI_BitMaskIndex_b6 = -1;    /* BIT_STRING_SIZE_26 */
static int hf_rrc_u_RNTI_BitMaskIndex_b7 = -1;    /* BIT_STRING_SIZE_25 */
static int hf_rrc_u_RNTI_BitMaskIndex_b8 = -1;    /* BIT_STRING_SIZE_24 */
static int hf_rrc_u_RNTI_BitMaskIndex_b9 = -1;    /* BIT_STRING_SIZE_23 */
static int hf_rrc_u_RNTI_BitMaskIndex_b10 = -1;   /* BIT_STRING_SIZE_22 */
static int hf_rrc_u_RNTI_BitMaskIndex_b11 = -1;   /* BIT_STRING_SIZE_21 */
static int hf_rrc_u_RNTI_BitMaskIndex_b12 = -1;   /* BIT_STRING_SIZE_20 */
static int hf_rrc_u_RNTI_BitMaskIndex_b13 = -1;   /* BIT_STRING_SIZE_19 */
static int hf_rrc_u_RNTI_BitMaskIndex_b14 = -1;   /* BIT_STRING_SIZE_18 */
static int hf_rrc_u_RNTI_BitMaskIndex_b15 = -1;   /* BIT_STRING_SIZE_17 */
static int hf_rrc_u_RNTI_BitMaskIndex_b16 = -1;   /* BIT_STRING_SIZE_16 */
static int hf_rrc_u_RNTI_BitMaskIndex_b17 = -1;   /* BIT_STRING_SIZE_15 */
static int hf_rrc_u_RNTI_BitMaskIndex_b18 = -1;   /* BIT_STRING_SIZE_14 */
static int hf_rrc_u_RNTI_BitMaskIndex_b19 = -1;   /* BIT_STRING_SIZE_13 */
static int hf_rrc_u_RNTI_BitMaskIndex_b20 = -1;   /* BIT_STRING_SIZE_12 */
static int hf_rrc_u_RNTI_BitMaskIndex_b21 = -1;   /* BIT_STRING_SIZE_11 */
static int hf_rrc_u_RNTI_BitMaskIndex_b22 = -1;   /* BIT_STRING_SIZE_10 */
static int hf_rrc_u_RNTI_BitMaskIndex_b23 = -1;   /* BIT_STRING_SIZE_9 */
static int hf_rrc_u_RNTI_BitMaskIndex_b24 = -1;   /* BIT_STRING_SIZE_8 */
static int hf_rrc_u_RNTI_BitMaskIndex_b25 = -1;   /* BIT_STRING_SIZE_7 */
static int hf_rrc_u_RNTI_BitMaskIndex_b26 = -1;   /* BIT_STRING_SIZE_6 */
static int hf_rrc_u_RNTI_BitMaskIndex_b27 = -1;   /* BIT_STRING_SIZE_5 */
static int hf_rrc_u_RNTI_BitMaskIndex_b28 = -1;   /* BIT_STRING_SIZE_4 */
static int hf_rrc_u_RNTI_BitMaskIndex_b29 = -1;   /* BIT_STRING_SIZE_3 */
static int hf_rrc_u_RNTI_BitMaskIndex_b30 = -1;   /* BIT_STRING_SIZE_2 */
static int hf_rrc_u_RNTI_BitMaskIndex_b31 = -1;   /* BIT_STRING_SIZE_1 */
static int hf_rrc_s_RNTI_2 = -1;                  /* S_RNTI_2 */
static int hf_rrc_supportOfLoggedMeasurementsIdlePCH = -1;  /* T_supportOfLoggedMeasurementsIdlePCH */
static int hf_rrc_ue_RadioAccessCapability_v690ext = -1;  /* UE_RadioAccessCapability_v690ext */
static int hf_rrc_ue_RATSpecificCapability_v690ext = -1;  /* InterRAT_UE_RadioAccessCapability_v690ext */
static int hf_rrc_v6b0NonCriticalExtensions_15 = -1;  /* T_v6b0NonCriticalExtensions_14 */
static int hf_rrc_ue_RadioAccessCapability_v6b0ext = -1;  /* UE_RadioAccessCapability_v6b0ext_IEs */
static int hf_rrc_v6e0NonCriticalExtensions_02 = -1;  /* T_v6e0NonCriticalExtensions_02 */
static int hf_rrc_ue_RadioAccessCapability_v6e0ext = -1;  /* UE_RadioAccessCapability_v6e0ext_IEs */
static int hf_rrc_v770NonCriticalExtensions_39 = -1;  /* T_v770NonCriticalExtensions_39 */
static int hf_rrc_ue_RadioAccessCapability_v770ext = -1;  /* UE_RadioAccessCapability_v770ext_IEs */
static int hf_rrc_v790NonCriticalExtensions_01 = -1;  /* T_v790NonCriticalExtensions_01 */
static int hf_rrc_ue_RadioAccessCapability_v790ext = -1;  /* UE_RadioAccessCapability_v790ext_IEs */
static int hf_rrc_v860NonCriticalExtensions_25 = -1;  /* T_v860NonCriticalExtensions_25 */
static int hf_rrc_ue_RadioAccessCapability_v860ext = -1;  /* UE_RadioAccessCapability_v860ext_IEs */
static int hf_rrc_ue_RATSpecificCapability_v860ext = -1;  /* InterRAT_UE_RadioAccessCapability_v860ext */
static int hf_rrc_v880NonCriticalExtensions_01 = -1;  /* T_v880NonCriticalExtensions_01 */
static int hf_rrc_ue_RadioAccessCapability_v880ext = -1;  /* UE_RadioAccessCapability_v880ext_IEs */
static int hf_rrc_v890NonCriticalExtensions_10 = -1;  /* T_v890NonCriticalExtensions_10 */
static int hf_rrc_ue_RadioAccessCapability_v890ext = -1;  /* UE_RadioAccessCapability_v890ext_IEs */
static int hf_rrc_v920NonCriticalExtensions_03 = -1;  /* T_v920NonCriticalExtensions_03 */
static int hf_rrc_ue_RadioAccessCapability_v920ext = -1;  /* UE_RadioAccessCapability_v920ext_IEs */
static int hf_rrc_v970NonCriticalExtensions_03 = -1;  /* T_v970NonCriticalExtensions_03 */
static int hf_rrc_ue_RadioAccessCapability_v970ext = -1;  /* UE_RadioAccessCapability_v970ext_IEs */
static int hf_rrc_va40NonCriticalExtensions_18 = -1;  /* T_va40NonCriticalExtensions_17 */
static int hf_rrc_ue_RadioAccessCapability_va40ext = -1;  /* UE_RadioAccessCapability_va40ext_IEs */
static int hf_rrc_va60NonCriticalExtensions = -1;  /* T_va60NonCriticalExtensions */
static int hf_rrc_ue_RadioAccessCapability_va60ext = -1;  /* UE_RadioAccessCapability_va60ext_IEs */
static int hf_rrc_va80NonCriticalExtensions_01 = -1;  /* T_va80NonCriticalExtensions_01 */
static int hf_rrc_ue_RadioAccessCapability_va80ext = -1;  /* UE_RadioAccessCapability_va80ext_IEs */
static int hf_rrc_laterNonCriticalExtensions_67 = -1;  /* UE_RadioAccessCapability_LaterNonCriticalExtensions */
static int hf_rrc_supportForSIB11bis_01 = -1;     /* T_supportForSIB11bis_01 */
static int hf_rrc_supportForFDPCH_02 = -1;        /* T_supportForFDPCH_02 */
static int hf_rrc_pdcp_Capability_01 = -1;        /* PDCP_Capability_v770ext */
static int hf_rrc_rlc_Capability = -1;            /* RLC_Capability_v770ext */
static int hf_rrc_rf_Capability = -1;             /* RF_Capability_v770ext */
static int hf_rrc_physicalChannelCapability = -1;  /* PhysicalChannelCapability_v770ext */
static int hf_rrc_multiModeRAT_Capability = -1;   /* MultiModeRAT_Capability_v770ext */
static int hf_rrc_ue_PositioningCapability = -1;  /* UE_PositioningCapability_v770ext */
static int hf_rrc_mac_ehsSupport_01 = -1;         /* T_mac_ehsSupport_01 */
static int hf_rrc_ue_specificCapabilityInformation = -1;  /* UE_SpecificCapabilityInformation_LCRTDD */
static int hf_rrc_supportForEDPCCHPowerBoosting_01 = -1;  /* T_supportForEDPCCHPowerBoosting_01 */
static int hf_rrc_rf_Capability_01 = -1;          /* RF_Capability_v860ext */
static int hf_rrc_physicalChannelCapability_01 = -1;  /* PhysicalChannelCapability_v860ext */
static int hf_rrc_multiModeRAT_Capability_01 = -1;  /* MultiModeRAT_Capability_v860ext */
static int hf_rrc_ue_PositioningCapability_01 = -1;  /* UE_PositioningCapability_v860ext */
static int hf_rrc_measurementCapability = -1;     /* MeasurementCapability_v860ext */
static int hf_rrc_measurementCapabilityTDD = -1;  /* MeasurementCapabilityTDD */
static int hf_rrc_supportOfCommonEDCH_03 = -1;    /* T_supportOfCommonEDCH_03 */
static int hf_rrc_supportOfMACiis_03 = -1;        /* T_supportOfMACiis_03 */
static int hf_rrc_supportOfSPSOperation_02 = -1;  /* T_supportOfSPSOperation_02 */
static int hf_rrc_supportOfControlChannelDRXOperation_01 = -1;  /* T_supportOfControlChannelDRXOperation_01 */
static int hf_rrc_supportOfCSG_01 = -1;           /* T_supportOfCSG_01 */
static int hf_rrc_ue_RadioAccessCapabBandFDDList3 = -1;  /* UE_RadioAccessCapabBandFDDList3 */
static int hf_rrc_supportForPriorityReselectionInUTRAN = -1;  /* T_supportForPriorityReselectionInUTRAN */
static int hf_rrc_supportCellSpecificTxDiversityinDC_Operation = -1;  /* T_supportCellSpecificTxDiversityinDC_Operation */
static int hf_rrc_ue_RadioAccessCapabBandCombList = -1;  /* UE_RadioAccessCapabBandCombList */
static int hf_rrc_physicalChannelCapability_02 = -1;  /* PhysicalChannelCapability_v920ext */
static int hf_rrc_rlc_Capability_01 = -1;         /* RLC_Capability_v920ext */
static int hf_rrc_supportOfenhancedTS0 = -1;      /* T_supportOfenhancedTS0 */
static int hf_rrc_measurementCapability_01 = -1;  /* MeasurementCapability_v920ext */
static int hf_rrc_csgProximityIndicationCapability = -1;  /* CSG_ProximityIndicationCapability */
static int hf_rrc_neighCellSI_AcquisitionCapability = -1;  /* NeighCellSI_AcquisitionCapability */
static int hf_rrc_ue_ExtendedMeasurementSupport = -1;  /* T_ue_ExtendedMeasurementSupport */
static int hf_rrc_voiceOverUTRA_PS_HS_Support = -1;  /* T_voiceOverUTRA_PS_HS_Support */
static int hf_rrc_srvcc_SupportFromUTRA_to_UTRA = -1;  /* T_srvcc_SupportFromUTRA_to_UTRA */
static int hf_rrc_srvcc_SupportFromUTRA_to_GERAN = -1;  /* T_srvcc_SupportFromUTRA_to_GERAN */
static int hf_rrc_ue_supportMac_ehsWindowSizeExtension = -1;  /* T_ue_supportMac_ehsWindowSizeExtension */
static int hf_rrc_v9c0NonCriticalExtensions_12 = -1;  /* T_v9c0NonCriticalExtensions_12 */
static int hf_rrc_ue_RadioAccessCapability_v9c0ext = -1;  /* UE_RadioAccessCapability_v9c0ext_IEs */
static int hf_rrc_vaa0NonCriticalExtensions_02 = -1;  /* T_vaa0NonCriticalExtensions_02 */
static int hf_rrc_ue_RadioAccessCapability_vaa0ext = -1;  /* UE_RadioAccessCapability_vaa0ext_IEs */
static int hf_rrc_vbxyNonCriticalExtensions_18 = -1;  /* T_vbxyNonCriticalExtensions_18 */
static int hf_rrc_ue_RadioAccessCapability_vbxyext = -1;  /* UE_RadioAccessCapability_vbxyext_IEs */
static int hf_rrc_nonCriticalExtensions_194 = -1;  /* T_nonCriticalExtensions_194 */
static int hf_rrc_ue_RadioAccessCapabBandCombList_va40ext = -1;  /* UE_RadioAccessCapabBandCombList_va40ext */
static int hf_rrc_physicalChannelCapability_03 = -1;  /* PhysicalChannelCapability_va40ext */
static int hf_rrc_rlc_Capability_02 = -1;         /* RLC_Capability_va40ext */
static int hf_rrc_measurementCapability_02 = -1;  /* MeasurementCapability_va40ext */
static int hf_rrc_supportOfMUMIMO = -1;           /* T_supportOfMUMIMO */
static int hf_rrc_supportForDualCellMimoInDifferentBands = -1;  /* T_supportForDualCellMimoInDifferentBands */
static int hf_rrc_ue_BasedNetworkPerformanceMeasurementsParameters = -1;  /* UE_BasedNetworkPerformanceMeasurementsParameters */
static int hf_rrc_supportOfUTRANANR = -1;         /* T_supportOfUTRANANR */
static int hf_rrc_ue_RadioAccessCapabBandFDDList4 = -1;  /* UE_RadioAccessCapabBandFDDList4 */
static int hf_rrc_ue_RadioAccessCapabBandFDDList4_01 = -1;  /* UE_RadioAccessCapabBandFDDList4_va60ext */
static int hf_rrc_ue_RadioAccessCapabBandFDDList4_02 = -1;  /* UE_RadioAccessCapabBandFDDList4_va80ext */
static int hf_rrc_rf_capability = -1;             /* RF_Capability_vaa0ext */
static int hf_rrc_supportOfUMRLCre_establish_via_reconfig = -1;  /* T_supportOfUMRLCre_establish_via_reconfig */
static int hf_rrc_multiModeRAT_Capability_02 = -1;  /* MultiModeRAT_Capability_vbxyext */
static int hf_rrc_supportOfCommonERGCH = -1;      /* T_supportOfCommonERGCH */
static int hf_rrc_supportOfFallbackR99PRACH = -1;  /* T_supportOfFallbackR99PRACH */
static int hf_rrc_supportOfConcurrent2ms10msTTI = -1;  /* T_supportOfConcurrent2ms10msTTI */
static int hf_rrc_supportOfTTIalignmentAndPerHARQActivation = -1;  /* T_supportOfTTIalignmentAndPerHARQActivation */
static int hf_rrc_ue_RadioAccessCapabBandCombList_vbxyext = -1;  /* UE_RadioAccessCapabBandCombList_vbxyext */
static int hf_rrc_multiflowBandCombList = -1;     /* MultiflowBandCombList */
static int hf_rrc_physicalChannelCapability_04 = -1;  /* PhysicalChannelCapability_vbxyext */
static int hf_rrc_rlc_Capability_03 = -1;         /* RLC_Capability_vbxyext */
static int hf_rrc_multiflowCapability = -1;       /* MultiflowCapability */
static int hf_rrc_ue_RadioAccessCapabBandFDDList6 = -1;  /* UE_RadioAccessCapabBandFDDList6 */
static int hf_rrc_measurementCapability_03 = -1;  /* MeasurementCapability_vbxyext */
static int hf_rrc_rsrvcc_SupportFromUTRA_CS_HS_to_EUTRA_FDD = -1;  /* T_rsrvcc_SupportFromUTRA_CS_HS_to_EUTRA_FDD */
static int hf_rrc_rsrvcc_SupportFromUTRA_CS_HS_to_EUTRA_TDD = -1;  /* T_rsrvcc_SupportFromUTRA_CS_HS_to_EUTRA_TDD */
static int hf_rrc_t_301 = -1;                     /* T_301 */
static int hf_rrc_n_301 = -1;                     /* N_301 */
static int hf_rrc_t_302 = -1;                     /* T_302 */
static int hf_rrc_n_302 = -1;                     /* N_302 */
static int hf_rrc_t_304 = -1;                     /* T_304 */
static int hf_rrc_n_304 = -1;                     /* N_304 */
static int hf_rrc_t_305 = -1;                     /* T_305 */
static int hf_rrc_t_307 = -1;                     /* T_307 */
static int hf_rrc_t_308 = -1;                     /* T_308 */
static int hf_rrc_t_309 = -1;                     /* T_309 */
static int hf_rrc_t_310 = -1;                     /* T_310 */
static int hf_rrc_n_310 = -1;                     /* N_310 */
static int hf_rrc_t_311 = -1;                     /* T_311 */
static int hf_rrc_t_312 = -1;                     /* T_312 */
static int hf_rrc_n_312 = -1;                     /* N_312 */
static int hf_rrc_t_313 = -1;                     /* T_313 */
static int hf_rrc_n_313 = -1;                     /* N_313 */
static int hf_rrc_t_314 = -1;                     /* T_314 */
static int hf_rrc_t_315 = -1;                     /* T_315 */
static int hf_rrc_n_315 = -1;                     /* N_315 */
static int hf_rrc_t_316 = -1;                     /* T_316 */
static int hf_rrc_t_317 = -1;                     /* T_317 */
static int hf_rrc_n_312_01 = -1;                  /* N_312ext */
static int hf_rrc_n_315_01 = -1;                  /* N_315ext */
static int hf_rrc_n_312_02 = -1;                  /* N_312_r5 */
static int hf_rrc_n_315_02 = -1;                  /* N_315_r5 */
static int hf_rrc_t_323 = -1;                     /* T_323 */
static int hf_rrc_networkAssistedGANSS_supportedList = -1;  /* NetworkAssistedGANSS_Supported_List */
static int hf_rrc_networkAssistedGANSS_supportedList_01 = -1;  /* NetworkAssistedGANSS_Supported_List_v860ext */
static int hf_rrc_t_300 = -1;                     /* T_300 */
static int hf_rrc_n_300 = -1;                     /* N_300 */
static int hf_rrc_multiRAT_CapabilityList = -1;   /* MultiRAT_Capability */
static int hf_rrc_multiModeCapability = -1;       /* MultiModeCapability */
static int hf_rrc_pdcp_Capability_02 = -1;        /* PDCP_Capability */
static int hf_rrc_rlc_Capability_04 = -1;         /* RLC_Capability */
static int hf_rrc_transportChannelCapability = -1;  /* TransportChannelCapability */
static int hf_rrc_rf_Capability_02 = -1;          /* RF_Capability */
static int hf_rrc_physicalChannelCapability_05 = -1;  /* PhysicalChannelCapability */
static int hf_rrc_ue_MultiModeRAT_Capability = -1;  /* UE_MultiModeRAT_Capability */
static int hf_rrc_ue_positioning_Capability = -1;  /* UE_Positioning_Capability */
static int hf_rrc_measurementCapability_04 = -1;  /* MeasurementCapability */
static int hf_rrc_ue_RadioAccessCapabBandFDDList = -1;  /* UE_RadioAccessCapabBandFDDList */
static int hf_rrc_ue_PositioningCapabilityExt_v380 = -1;  /* UE_PositioningCapabilityExt_v380 */
static int hf_rrc_ue_PositioningCapabilityExt_v3a0 = -1;  /* UE_PositioningCapabilityExt_v3a0 */
static int hf_rrc_ue_PositioningCapabilityExt_v3g0 = -1;  /* UE_PositioningCapabilityExt_v3g0 */
static int hf_rrc_ue_RadioAccessCapabBandFDDList2 = -1;  /* UE_RadioAccessCapabBandFDDList2 */
static int hf_rrc_ue_RadioAccessCapabBandFDDList_ext = -1;  /* UE_RadioAccessCapabBandFDDList_ext */
static int hf_rrc_physicalchannelcapability_edch = -1;  /* PhysicalChannelCapability_edch_r6 */
static int hf_rrc_deviceType = -1;                /* T_deviceType */
static int hf_rrc_supportForTwoDRXSchemesInPCH_02 = -1;  /* T_supportForTwoDRXSchemesInPCH_02 */
static int hf_rrc_supportEDPDCHPowerInterpolation = -1;  /* T_supportEDPDCHPowerInterpolation */
static int hf_rrc_supportofTxDivOnNonMIMOChannel = -1;  /* T_supportofTxDivOnNonMIMOChannel */
static int hf_rrc_physicalChannelCapability_LCR = -1;  /* PhysicalChannelCapability_LCR_r4 */
static int hf_rrc_ue_RadioAccessCapabilityComp_TDD128 = -1;  /* UE_RadioAccessCapabilityComp_TDD128_v7f0ext */
static int hf_rrc_physicalChannelCapability_06 = -1;  /* PhysicalChannelCapabilityInfo_v770ext */
static int hf_rrc_ganssSupportIndication = -1;    /* T_ganssSupportIndication */
static int hf_rrc_mac_ehsSupport_02 = -1;         /* T_mac_ehsSupport_02 */
static int hf_rrc_ue_SpecificCapabilityInformation = -1;  /* UE_SpecificCapabilityInformation_LCRTDD */
static int hf_rrc_supportForPriorityReselectionInUTRAN_01 = -1;  /* T_supportForPriorityReselectionInUTRAN_01 */
static int hf_rrc_supportOfenhancedTS0_01 = -1;   /* T_supportOfenhancedTS0_01 */
static int hf_rrc_ue_RadioAccessCapabBandFDDList4_va60ext = -1;  /* UE_RadioAccessCapabBandFDDList4_va60ext */
static int hf_rrc_ue_RadioAccessCapabBandFDDList4_va80ext = -1;  /* UE_RadioAccessCapabBandFDDList4_va80ext */
static int hf_rrc_ue_RadioAccessCapabBandFDDList5 = -1;  /* UE_RadioAccessCapabBandFDDList5 */
static int hf_rrc_ue_RadioAccessCapabBandFDDList_ext2 = -1;  /* UE_RadioAccessCapabBandFDDList_ext2 */
static int hf_rrc_UE_RadioAccessCapabBandFDDList2_item = -1;  /* UE_RadioAccessCapabBandFDD2 */
static int hf_rrc_UE_RadioAccessCapabBandFDDList3_item = -1;  /* UE_RadioAccessCapabBandFDD3 */
static int hf_rrc_UE_RadioAccessCapabBandFDDList4_item = -1;  /* UE_RadioAccessCapabBandFDD4 */
static int hf_rrc_UE_RadioAccessCapabBandFDDList4_va60ext_item = -1;  /* UE_RadioAccessCapabBandFDD4_va60ext */
static int hf_rrc_UE_RadioAccessCapabBandFDDList4_va80ext_item = -1;  /* UE_RadioAccessCapabBandFDD4_va80ext */
static int hf_rrc_UE_RadioAccessCapabBandFDDList5_item = -1;  /* UE_RadioAccessCapabBandFDD5 */
static int hf_rrc_UE_RadioAccessCapabBandFDDList6_item = -1;  /* UE_RadioAccessCapabBandFDD6 */
static int hf_rrc_fddRF_Capability_01 = -1;       /* T_fddRF_Capability_01 */
static int hf_rrc_ue_PowerClass_01 = -1;          /* UE_PowerClassExt */
static int hf_rrc_measurementCapability2 = -1;    /* MeasurementCapabilityExt2 */
static int hf_rrc_measurementCapability3 = -1;    /* MeasurementCapabilityExt3 */
static int hf_rrc_additionalSecondaryCells = -1;  /* T_additionalSecondaryCells */
static int hf_rrc_nonContiguousMultiCellCombinationList = -1;  /* NonContiguousMultiCellCombinationList */
static int hf_rrc_supportofULOLTD = -1;           /* T_supportofULOLTD */
static int hf_rrc_additionalSecondaryCells_01 = -1;  /* T_additionalSecondaryCells_01 */
static int hf_rrc_supportofULOLTD_01 = -1;        /* T_supportofULOLTD_01 */
static int hf_rrc_fddRF_Capability_02 = -1;       /* T_fddRF_Capability_02 */
static int hf_rrc_measurementCapability4 = -1;    /* MeasurementCapabilityExt4 */
static int hf_rrc_additionalSecondaryCells2 = -1;  /* T_additionalSecondaryCells2 */
static int hf_rrc_supportOfULCLTD = -1;           /* T_supportOfULCLTD */
static int hf_rrc_multiflowCapabilityBand = -1;   /* MultiflowCapabilityBand */
static int hf_rrc_mimo4x4CapabilityBand = -1;     /* T_mimo4x4CapabilityBand */
static int hf_rrc_supportOfULMIMO = -1;           /* T_supportOfULMIMO */
static int hf_rrc_rx_tx_TimeDifferenceType2Capable = -1;  /* BOOLEAN */
static int hf_rrc_validity_CellPCH_UraPCH = -1;   /* T_validity_CellPCH_UraPCH */
static int hf_rrc_sfn_sfnType2Capability = -1;    /* T_sfn_sfnType2Capability */
static int hf_rrc_ue_GANSSPositioning_Capability = -1;  /* UE_GANSSPositioning_Capability */
static int hf_rrc_ue_GANSSPositioning_Capability_v860ext = -1;  /* UE_GANSSPositioning_Capability_v860ext_IEs */
static int hf_rrc_UE_RadioAccessCapabBandCombList_item = -1;  /* BandComb */
static int hf_rrc_UE_RadioAccessCapabBandFDDList_item = -1;  /* UE_RadioAccessCapabBandFDD */
static int hf_rrc_UE_RadioAccessCapabBandFDDList_ext_item = -1;  /* UE_RadioAccessCapabBandFDD_ext */
static int hf_rrc_UE_RadioAccessCapabBandFDDList_ext2_item = -1;  /* UE_RadioAccessCapabBandFDD_ext2 */
static int hf_rrc_fddRF_Capability_03 = -1;       /* T_fddRF_Capability_03 */
static int hf_rrc_measurementCapability_05 = -1;  /* MeasurementCapabilityExt */
static int hf_rrc_compressedModeMeasCapabFDDList_ext = -1;  /* CompressedModeMeasCapabFDDList_ext */
static int hf_rrc_pdcp_Capability_r4_ext = -1;    /* PDCP_Capability_r4_ext */
static int hf_rrc_tdd_CapabilityExt = -1;         /* T_tdd_CapabilityExt */
static int hf_rrc_rf_Capability_03 = -1;          /* RF_Capability_r4_ext */
static int hf_rrc_measurementCapability_r4_ext = -1;  /* MeasurementCapability_r4_ext */
static int hf_rrc_totalAM_RLCMemoryExceeds10kB = -1;  /* BOOLEAN */
static int hf_rrc_rf_CapabilityComp = -1;         /* RF_CapabilityComp */
static int hf_rrc_rf_CapabilityFDDComp_01 = -1;   /* RF_CapabBandListFDDComp_ext */
static int hf_rrc_hSDSCH_physical_layer_category = -1;  /* HSDSCH_physical_layer_category */
static int hf_rrc_tdd_edch_PhysicalLayerCategory = -1;  /* INTEGER_1_6 */
static int hf_rrc_dl_CapabilityWithSimultaneousHS_DSCHConfig = -1;  /* DL_CapabilityWithSimultaneousHS_DSCHConfig */
static int hf_rrc_supportForCSVoiceoverHSPA_02 = -1;  /* T_supportForCSVoiceoverHSPA_02 */
static int hf_rrc_rf_CapabilityComp_01 = -1;      /* RF_CapabilityComp_v770ext */
static int hf_rrc_securityCapabilityIndication = -1;  /* T_securityCapabilityIndication */
static int hf_rrc_ganssSupportIndication_01 = -1;  /* T_ganssSupportIndication_01 */
static int hf_rrc_mac_ehsSupport_03 = -1;         /* T_mac_ehsSupport_03 */
static int hf_rrc_fddPhysicalChannelCapab_hspdsch_edch = -1;  /* T_fddPhysicalChannelCapab_hspdsch_edch */
static int hf_rrc_physicalChannelCapabComp_hspdsch_r6 = -1;  /* HSDSCH_physical_layer_category */
static int hf_rrc_physicalChannelCapability_edch_r6 = -1;  /* PhysicalChannelCapability_edch_r6 */
static int hf_rrc_edch_PhysicalLayerCategory_extension = -1;  /* INTEGER_7 */
static int hf_rrc_hsscchlessHsdschOperation_01 = -1;  /* T_hsscchlessHsdschOperation_01 */
static int hf_rrc_enhancedFdpch_01 = -1;          /* T_enhancedFdpch_01 */
static int hf_rrc_hsdschReception_CellFach_01 = -1;  /* T_hsdschReception_CellFach_01 */
static int hf_rrc_hsdschReception_CellUraPch_01 = -1;  /* T_hsdschReception_CellUraPch_01 */
static int hf_rrc_discontinuousDpcchTransmission_01 = -1;  /* T_discontinuousDpcchTransmission_01 */
static int hf_rrc_slotFormat4 = -1;               /* T_slotFormat4 */
static int hf_rrc_supportofTxDivOnNonMIMOChannel_01 = -1;  /* T_supportofTxDivOnNonMIMOChannel_01 */
static int hf_rrc_supportOfMACiis_04 = -1;        /* T_supportOfMACiis_04 */
static int hf_rrc_edch_PhysicalLayerCategory_extension2 = -1;  /* INTEGER_8_9 */
static int hf_rrc_edch_PhysicalLayerCategory_extension3 = -1;  /* INTEGER_10_12 */
static int hf_rrc_supportOfSFModeForHSPDSCHDualStream_01 = -1;  /* T_supportOfSFModeForHSPDSCHDualStream_01 */
static int hf_rrc_supportOfCommonEDCH_04 = -1;    /* T_supportOfCommonEDCH_04 */
static int hf_rrc_supportOfMACiis_05 = -1;        /* T_supportOfMACiis_05 */
static int hf_rrc_supportOfSPSOperation_03 = -1;  /* T_supportOfSPSOperation_03 */
static int hf_rrc_supportOfControlChannelDRXOperation_02 = -1;  /* T_supportOfControlChannelDRXOperation_02 */
static int hf_rrc_fdd_48 = -1;                    /* T_fdd_47 */
static int hf_rrc_supported_06 = -1;              /* RF_CapabBandListFDDComp */
static int hf_rrc_tdd384_RF_Capability = -1;      /* T_tdd384_RF_Capability */
static int hf_rrc_supported_07 = -1;              /* RadioFrequencyBandTDDList */
static int hf_rrc_tdd128_RF_Capability_01 = -1;   /* T_tdd128_RF_Capability */
static int hf_rrc_tdd384_RF_Capability_01 = -1;   /* RadioFrequencyBandTDDList_r7 */
static int hf_rrc_tdd768_RF_Capability = -1;      /* T_tdd768_RF_Capability */
static int hf_rrc_supported_08 = -1;              /* RadioFrequencyBandTDDList_r7 */
static int hf_rrc_tdd128_RF_Capability_02 = -1;   /* RadioFrequencyBandTDDList_r7 */
static int hf_rrc_RF_CapabBandListFDDComp_item = -1;  /* RF_CapabBandFDDComp */
static int hf_rrc_RF_CapabBandListFDDComp_ext_item = -1;  /* RF_CapabBandFDDComp */
static int hf_rrc_RF_CapabBandListFDDComp_ext2_item = -1;  /* RF_CapabBandFDDComp */
static int hf_rrc_UE_RadioAccessCapabBandCombList_va40ext_item = -1;  /* SupportedCarrierCombination */
static int hf_rrc_UE_RadioAccessCapabBandCombList_vbxyext_item = -1;  /* SupportedCarrierCombination_ext */
static int hf_rrc_carrierCombination12 = -1;      /* BOOLEAN */
static int hf_rrc_carrierCombination21 = -1;      /* BOOLEAN */
static int hf_rrc_carrierCombination13 = -1;      /* BOOLEAN */
static int hf_rrc_carrierCombination31 = -1;      /* BOOLEAN */
static int hf_rrc_carrierCombination22 = -1;      /* BOOLEAN */
static int hf_rrc_carrierCombination14 = -1;      /* BOOLEAN */
static int hf_rrc_carrierCombination41 = -1;      /* BOOLEAN */
static int hf_rrc_carrierCombination15 = -1;      /* BOOLEAN */
static int hf_rrc_carrierCombination51 = -1;      /* BOOLEAN */
static int hf_rrc_carrierCombination16 = -1;      /* BOOLEAN */
static int hf_rrc_carrierCombination61 = -1;      /* BOOLEAN */
static int hf_rrc_carrierCombination17 = -1;      /* BOOLEAN */
static int hf_rrc_carrierCombination71 = -1;      /* BOOLEAN */
static int hf_rrc_carrierCombination23 = -1;      /* BOOLEAN */
static int hf_rrc_carrierCombination32 = -1;      /* BOOLEAN */
static int hf_rrc_carrierCombination24 = -1;      /* BOOLEAN */
static int hf_rrc_carrierCombination42 = -1;      /* BOOLEAN */
static int hf_rrc_carrierCombination25 = -1;      /* BOOLEAN */
static int hf_rrc_carrierCombination52 = -1;      /* BOOLEAN */
static int hf_rrc_carrierCombination26 = -1;      /* BOOLEAN */
static int hf_rrc_carrierCombination62 = -1;      /* BOOLEAN */
static int hf_rrc_carrierCombination33 = -1;      /* BOOLEAN */
static int hf_rrc_carrierCombination34 = -1;      /* BOOLEAN */
static int hf_rrc_carrierCombination43 = -1;      /* BOOLEAN */
static int hf_rrc_carrierCombination44 = -1;      /* BOOLEAN */
static int hf_rrc_carrierCombination35 = -1;      /* BOOLEAN */
static int hf_rrc_carrierCombination53 = -1;      /* BOOLEAN */
static int hf_rrc_pdcp_Capability_r5_ext = -1;    /* PDCP_Capability_r5_ext */
static int hf_rrc_rlc_Capability_r5_ext = -1;     /* RLC_Capability_r5_ext */
static int hf_rrc_physicalChannelCapability_07 = -1;  /* PhysicalChannelCapability_hspdsch_r5 */
static int hf_rrc_multiModeRAT_Capability_v590ext = -1;  /* MultiModeRAT_Capability_v590ext */
static int hf_rrc_pdcp_Capability_r5_ext2 = -1;   /* PDCP_Capability_r5_ext2 */
static int hf_rrc_multiModeRAT_Capability_v680ext = -1;  /* MultiModeRAT_Capability_v680ext */
static int hf_rrc_ulOLTDActivation = -1;          /* T_ulOLTDActivation */
static int hf_rrc_maxNoDPDCH_BitsTransmitted = -1;  /* MaxNoDPDCH_BitsTransmitted */
static int hf_rrc_discontinuousDpcchTransmission_02 = -1;  /* T_discontinuousDpcchTransmission_02 */
static int hf_rrc_slotFormat4_01 = -1;            /* T_slotFormat4_01 */
static int hf_rrc_maxPhysChPerTimeslot = -1;      /* MaxPhysChPerTimeslot */
static int hf_rrc_minimumSF_02 = -1;              /* MinimumSF_UL */
static int hf_rrc_supportOfPUSCH = -1;            /* BOOLEAN */
static int hf_rrc_tdd384_edch = -1;               /* T_tdd384_edch */
static int hf_rrc_supported_09 = -1;              /* T_supported_03 */
static int hf_rrc_tdd_edch_PhysicalLayerCategory_01 = -1;  /* INTEGER_1_16 */
static int hf_rrc_maxPhysChPerTimeslot_01 = -1;   /* MaxPhysChPerTimeslot_LCR_r7 */
static int hf_rrc_tdd128_edch = -1;               /* T_tdd128_edch */
static int hf_rrc_supported_10 = -1;              /* T_supported_04 */
static int hf_rrc_multiCarrier_EdchPhysicalLayerCategory_128 = -1;  /* INTEGER_1_8 */
static int hf_rrc_maxNumberCarrierForMCHSUPA_TDD = -1;  /* T_maxNumberCarrierForMCHSUPA_TDD */
static int hf_rrc_tdd384_edch_01 = -1;            /* T_tdd384_edch_01 */
static int hf_rrc_supported_11 = -1;              /* T_supported_05 */
static int hf_rrc_continue = -1;                  /* NULL */
static int hf_rrc_newConfiguration = -1;          /* T_newConfiguration */
static int hf_rrc_secondaryServingEDCHCell_Info = -1;  /* SecondaryServingEDCHCell_Info */
static int hf_rrc_secondaryEDCH_Info_Common = -1;  /* SecondaryEDCH_Info_Common */
static int hf_rrc_dl_InformationPerSecondaryRL_List = -1;  /* DL_InformationPerSecondaryRL_List */
static int hf_rrc_fdd_edch = -1;                  /* T_fdd_edch */
static int hf_rrc_supported_12 = -1;              /* T_supported_06 */
static int hf_rrc_edch_PhysicalLayerCategory = -1;  /* INTEGER_1_16 */
static int hf_rrc_maxNoBitsTransmitted = -1;      /* MaxNoBits */
static int hf_rrc_maxConvCodeBitsTransmitted = -1;  /* MaxNoBits */
static int hf_rrc_turboEncodingSupport = -1;      /* TurboSupport */
static int hf_rrc_maxSimultaneousTransChs_01 = -1;  /* MaxSimultaneousTransChsUL */
static int hf_rrc_modeSpecificInfo_18 = -1;       /* T_modeSpecificInfo_18 */
static int hf_rrc_tdd_13 = -1;                    /* T_tdd_12 */
static int hf_rrc_maxTransmittedBlocks = -1;      /* MaxTransportBlocksUL */
static int hf_rrc_maxNumberOfTFC_01 = -1;         /* MaxNumberOfTFC_UL */
static int hf_rrc_standaloneLocMethodsSupported = -1;  /* BOOLEAN */
static int hf_rrc_ue_BasedOTDOA_Supported = -1;   /* BOOLEAN */
static int hf_rrc_networkAssistedGPS_Supported = -1;  /* NetworkAssistedGPS_Supported */
static int hf_rrc_supportForUE_GPS_TimingOfCellFrames = -1;  /* BOOLEAN */
static int hf_rrc_supportForIPDL = -1;            /* BOOLEAN */
static int hf_rrc_start_CS = -1;                  /* START_Value */
static int hf_rrc_start_PS = -1;                  /* START_Value */
static int hf_rrc_drx_CycleLengthCoefficient = -1;  /* INTEGER_3_9 */
static int hf_rrc_drx_CycleLengthCoefficient2 = -1;  /* INTEGER_3_9 */
static int hf_rrc_timeForDRXCycle2 = -1;          /* T_319 */
static int hf_rrc_rfc2507_Info = -1;              /* RFC2507_Info */
static int hf_rrc_rfc3095_Info = -1;              /* RFC3095_Info_r4 */
static int hf_rrc_logicalChannelIdentity = -1;    /* LogicalChannelIdentity */
static int hf_rrc_mac_ehs_QueueId = -1;           /* MAC_ehs_QueueId */
static int hf_rrc_maxCS_Delay = -1;               /* MaxCS_Delay */
static int hf_rrc_defaultConfigIdForCellFACH = -1;  /* DefaultConfigIdForCellFACH */
static int hf_rrc_inSequenceDelivery = -1;        /* BOOLEAN */
static int hf_rrc_receivingWindowSize = -1;       /* ReceivingWindowSize */
static int hf_rrc_dl_RLC_StatusInfo = -1;         /* DL_RLC_StatusInfo */
static int hf_rrc_dl_RLC_PDU_size = -1;           /* OctetModeRLC_SizeInfoType1 */
static int hf_rrc_dl_RLC_PDU_size_01 = -1;        /* T_dl_RLC_PDU_size */
static int hf_rrc_fixedSize = -1;                 /* OctetModeRLC_SizeInfoType1 */
static int hf_rrc_flexibleSize = -1;              /* T_flexibleSize */
static int hf_rrc_dl_RLC_PDU_size_02 = -1;        /* T_dl_RLC_PDU_size_01 */
static int hf_rrc_flexibleSize_01 = -1;           /* T_flexibleSize_01 */
static int hf_rrc_dl_RLC_StatusInfo_01 = -1;      /* DL_RLC_StatusInfo_r11 */
static int hf_rrc_rB_WithPDCP_InfoList = -1;      /* RB_WithPDCP_InfoList */
static int hf_rrc_rb_WithPDCP_InfoList = -1;      /* RB_WithPDCP_InfoList */
static int hf_rrc_dl_TransportChannelType = -1;   /* DL_TransportChannelType */
static int hf_rrc_dl_TransportChannelType_01 = -1;  /* DL_TransportChannelType_r5 */
static int hf_rrc_dl_TransportChannelType_02 = -1;  /* DL_TransportChannelType_r7 */
static int hf_rrc_DL_LogicalChannelMappingList_item = -1;  /* DL_LogicalChannelMapping */
static int hf_rrc_DL_LogicalChannelMappingList_r5_item = -1;  /* DL_LogicalChannelMapping_r5 */
static int hf_rrc_DL_LogicalChannelMappingList_r7_item = -1;  /* DL_LogicalChannelMapping_r7 */
static int hf_rrc_dummy_30 = -1;                  /* CID_InclusionInfo_r4 */
static int hf_rrc_max_CID = -1;                   /* INTEGER_1_16383 */
static int hf_rrc_reverseDecompressionDepth = -1;  /* INTEGER_0_65535 */
static int hf_rrc_dl_AM_RLC_Mode = -1;            /* DL_AM_RLC_Mode */
static int hf_rrc_dl_UM_RLC_Mode = -1;            /* NULL */
static int hf_rrc_dl_TM_RLC_Mode = -1;            /* DL_TM_RLC_Mode */
static int hf_rrc_dl_AM_RLC_Mode_01 = -1;         /* DL_AM_RLC_Mode_r5 */
static int hf_rrc_dl_UM_RLC_Mode_01 = -1;         /* DL_UM_RLC_Mode_r5 */
static int hf_rrc_dl_UM_RLC_Mode_02 = -1;         /* DL_UM_RLC_Mode_r6 */
static int hf_rrc_dl_AM_RLC_Mode_02 = -1;         /* DL_AM_RLC_Mode_r7 */
static int hf_rrc_dl_AM_RLC_Mode_03 = -1;         /* DL_AM_RLC_Mode_r11 */
static int hf_rrc_timerStatusProhibit = -1;       /* TimerStatusProhibit */
static int hf_rrc_dummy_31 = -1;                  /* TimerEPC */
static int hf_rrc_missingPDU_Indicator = -1;      /* BOOLEAN */
static int hf_rrc_timerStatusPeriodic = -1;       /* TimerStatusPeriodic */
static int hf_rrc_timerReordering = -1;           /* TimerReordering */
static int hf_rrc_segmentationIndication = -1;    /* BOOLEAN */
static int hf_rrc_dch = -1;                       /* TransportChannelIdentity */
static int hf_rrc_fach = -1;                      /* NULL */
static int hf_rrc_dsch = -1;                      /* TransportChannelIdentity */
static int hf_rrc_dch_and_dsch = -1;              /* TransportChannelIdentityDCHandDSCH */
static int hf_rrc_hsdsch = -1;                    /* MAC_d_FlowIdentity */
static int hf_rrc_dch_and_hsdsch = -1;            /* MAC_d_FlowIdentityDCHandHSDSCH */
static int hf_rrc_hsdsch_01 = -1;                 /* T_hsdsch */
static int hf_rrc_mac_hs = -1;                    /* MAC_d_FlowIdentity */
static int hf_rrc_mac_ehs = -1;                   /* MAC_ehs_QueueId */
static int hf_rrc_dch_and_hsdsch_01 = -1;         /* T_dch_and_hsdsch */
static int hf_rrc_mac_hs_01 = -1;                 /* MAC_d_FlowIdentityDCHandHSDSCH */
static int hf_rrc_mac_ehs_01 = -1;                /* MAC_ehs_QueueIdDCHandHSDSCH */
static int hf_rrc_dl_UM_RLC_LI_size = -1;         /* DL_UM_RLC_LI_size */
static int hf_rrc_dl_Reception_Window_Size = -1;  /* DL_Reception_Window_Size_r6 */
static int hf_rrc_timerMRW = -1;                  /* TimerMRW */
static int hf_rrc_timerDiscard = -1;              /* TimerDiscard */
static int hf_rrc_maxMRW = -1;                    /* MaxMRW */
static int hf_rrc_algorithmSpecificInfo = -1;     /* AlgorithmSpecificInfo */
static int hf_rrc_HeaderCompressionInfoList_item = -1;  /* HeaderCompressionInfo */
static int hf_rrc_algorithmSpecificInfo_01 = -1;  /* AlgorithmSpecificInfo_r4 */
static int hf_rrc_HeaderCompressionInfoList_r4_item = -1;  /* HeaderCompressionInfo_r4 */
static int hf_rrc_supported_13 = -1;              /* MaxPDCP_SN_WindowSize */
static int hf_rrc_maxDAT = -1;                    /* MaxDAT */
static int hf_rrc_losslessSRNS_RelocSupport = -1;  /* LosslessSRNS_RelocSupport */
static int hf_rrc_pdcp_PDU_Header = -1;           /* PDCP_PDU_Header */
static int hf_rrc_headerCompressionInfoList = -1;  /* HeaderCompressionInfoList */
static int hf_rrc_headerCompressionInfoList_01 = -1;  /* HeaderCompressionInfoList_r4 */
static int hf_rrc_pdcp_Info = -1;                 /* PDCP_Info */
static int hf_rrc_dummy_32 = -1;                  /* INTEGER_0_65535 */
static int hf_rrc_pdcp_Info_01 = -1;              /* PDCP_Info_r4 */
static int hf_rrc_timerPollProhibit = -1;         /* TimerPollProhibit */
static int hf_rrc_timerPoll = -1;                 /* TimerPoll */
static int hf_rrc_poll_PDU = -1;                  /* Poll_PDU */
static int hf_rrc_poll_SDU = -1;                  /* Poll_SDU */
static int hf_rrc_lastTransmissionPDU_Poll = -1;  /* BOOLEAN */
static int hf_rrc_lastRetransmissionPDU_Poll = -1;  /* BOOLEAN */
static int hf_rrc_pollWindow = -1;                /* PollWindow */
static int hf_rrc_timerPollPeriodic = -1;         /* TimerPollPeriodic */
static int hf_rrc_re_EstablishmentTimer = -1;     /* Re_EstablishmentTimer */
static int hf_rrc_srb_InformationList = -1;       /* SRB_InformationSetupList */
static int hf_rrc_rb_InformationList = -1;        /* RB_InformationSetupList */
static int hf_rrc_predefinedRB_Configuration = -1;  /* PredefinedRB_Configuration */
static int hf_rrc_preDefTransChConfiguration = -1;  /* PreDefTransChConfiguration */
static int hf_rrc_preDefPhyChConfiguration = -1;  /* PreDefPhyChConfiguration */
static int hf_rrc_preDefPhyChConfiguration_01 = -1;  /* PreDefPhyChConfiguration_v770ext */
static int hf_rrc_mac_ehsWindowSize = -1;         /* MAC_hs_WindowSize_r9 */
static int hf_rrc_PredefinedConfigStatusList_item = -1;  /* PredefinedConfigStatusInfo */
static int hf_rrc_storedWithValueTagSameAsPrevius = -1;  /* NULL */
static int hf_rrc_other = -1;                     /* T_other */
static int hf_rrc_notStored = -1;                 /* NULL */
static int hf_rrc_storedWithDifferentValueTag = -1;  /* PredefinedConfigValueTag */
static int hf_rrc_setsWithDifferentValueTag = -1;  /* PredefinedConfigSetsWithDifferentValueTag */
static int hf_rrc_otherEntries = -1;              /* PredefinedConfigStatusListVarSz */
static int hf_rrc_PredefinedConfigSetsWithDifferentValueTag_item = -1;  /* PredefinedConfigSetWithDifferentValueTag */
static int hf_rrc_startPosition = -1;             /* INTEGER_0_10 */
static int hf_rrc_valueTagList = -1;              /* PredefinedConfigValueTagList */
static int hf_rrc_PredefinedConfigValueTagList_item = -1;  /* PredefinedConfigValueTag */
static int hf_rrc_PredefinedConfigStatusListVarSz_item = -1;  /* PredefinedConfigStatusInfo */
static int hf_rrc_rab_Identity = -1;              /* RAB_Identity */
static int hf_rrc_nas_Synchronisation_Indicator = -1;  /* NAS_Synchronisation_Indicator */
static int hf_rrc_mbms_SessionIdentity = -1;      /* MBMS_SessionIdentity */
static int hf_rrc_mbms_ServiceIdentity = -1;      /* OCTET_STRING_SIZE_3 */
static int hf_rrc_RAB_InformationList_item = -1;  /* RAB_Info */
static int hf_rrc_RAB_InformationList_r6_item = -1;  /* RAB_Info_r6 */
static int hf_rrc_RAB_InformationReconfigList_item = -1;  /* RAB_InformationReconfig */
static int hf_rrc_RAB_InformationReconfigList_r8_item = -1;  /* RAB_InformationReconfig_r8 */
static int hf_rrc_cs_HSPA_Information = -1;       /* CS_HSPA_Information */
static int hf_rrc_rb_Identity = -1;               /* RB_Identity */
static int hf_rrc_RAB_InformationMBMSPtpList_item = -1;  /* RAB_InformationMBMSPtp */
static int hf_rrc_rab_Info_02 = -1;               /* RAB_Info */
static int hf_rrc_rb_InformationSetupList = -1;   /* RB_InformationSetupList */
static int hf_rrc_rb_InformationSetupList_01 = -1;  /* RB_InformationSetupList_r4 */
static int hf_rrc_rb_InformationSetupList_02 = -1;  /* RB_InformationSetupList_r5 */
static int hf_rrc_rab_Info_r6_ext = -1;           /* RAB_Info_r6_ext */
static int hf_rrc_rb_InformationSetupList_03 = -1;  /* RB_InformationSetupList_r6 */
static int hf_rrc_rab_Info_v6b0ext = -1;          /* RAB_Info_v6b0ext */
static int hf_rrc_rab_Info_03 = -1;               /* RAB_Info_r7 */
static int hf_rrc_rb_InformationSetupList_04 = -1;  /* RB_InformationSetupList_r7 */
static int hf_rrc_rb_InformationSetupList_05 = -1;  /* RB_InformationSetupList_r8 */
static int hf_rrc_rb_InformationSetupList_06 = -1;  /* RB_InformationSetupList_r11 */
static int hf_rrc_RAB_InformationSetupList_item = -1;  /* RAB_InformationSetup */
static int hf_rrc_RAB_InformationSetupList_r4_item = -1;  /* RAB_InformationSetup_r4 */
static int hf_rrc_RAB_InformationSetupList_r5_item = -1;  /* RAB_InformationSetup_r5 */
static int hf_rrc_RAB_InformationSetupList_r6_item = -1;  /* RAB_InformationSetup_r6 */
static int hf_rrc_RAB_InformationSetupList_r6_ext_item = -1;  /* RAB_InformationSetup_r6_ext */
static int hf_rrc_RAB_InformationSetupList_v6b0ext_item = -1;  /* RAB_InformationSetup_v6b0ext */
static int hf_rrc_RAB_InformationSetupList_r7_item = -1;  /* RAB_InformationSetup_r7 */
static int hf_rrc_RAB_InformationSetupList_r8_item = -1;  /* RAB_InformationSetup_r8 */
static int hf_rrc_RAB_InformationSetupList_v820ext_item = -1;  /* RAB_InformationSetup_v820ext */
static int hf_rrc_RAB_InformationSetupList_r11_item = -1;  /* RAB_InformationSetup_r11 */
static int hf_rrc_rlc_SequenceNumber = -1;        /* RLC_SequenceNumber */
static int hf_rrc_RB_ActivationTimeInfoList_item = -1;  /* RB_ActivationTimeInfo */
static int hf_rrc_count_C_UL = -1;                /* COUNT_C */
static int hf_rrc_count_C_DL = -1;                /* COUNT_C */
static int hf_rrc_RB_COUNT_C_InformationList_item = -1;  /* RB_COUNT_C_Information */
static int hf_rrc_count_C_MSB_UL = -1;            /* COUNT_C_MSB */
static int hf_rrc_count_C_MSB_DL = -1;            /* COUNT_C_MSB */
static int hf_rrc_RB_COUNT_C_MSB_InformationList_item = -1;  /* RB_COUNT_C_MSB_Information */
static int hf_rrc_RB_IdentityList_item = -1;      /* RB_Identity */
static int hf_rrc_rb_MappingInfo = -1;            /* RB_MappingInfo */
static int hf_rrc_rb_MappingInfo_01 = -1;         /* RB_MappingInfo_r5 */
static int hf_rrc_rb_MappingInfo_02 = -1;         /* RB_MappingInfo_r6 */
static int hf_rrc_rb_MappingInfo_03 = -1;         /* RB_MappingInfo_r7 */
static int hf_rrc_rb_MappingInfo_04 = -1;         /* RB_MappingInfo_r8 */
static int hf_rrc_RB_InformationAffectedList_item = -1;  /* RB_InformationAffected */
static int hf_rrc_RB_InformationAffectedList_r5_item = -1;  /* RB_InformationAffected_r5 */
static int hf_rrc_RB_InformationAffectedList_r6_item = -1;  /* RB_InformationAffected_r6 */
static int hf_rrc_RB_InformationAffectedList_r7_item = -1;  /* RB_InformationAffected_r7 */
static int hf_rrc_RB_InformationAffectedList_r8_item = -1;  /* RB_InformationAffected_r8 */
static int hf_rrc_rb_Change = -1;                 /* T_rb_Change */
static int hf_rrc_release_01 = -1;                /* NULL */
static int hf_rrc_re_mapToDefaultRb = -1;         /* RB_Identity */
static int hf_rrc_RB_InformationChangedList_r6_item = -1;  /* RB_InformationChanged_r6 */
static int hf_rrc_pdcp_Info_02 = -1;              /* PDCP_InfoReconfig */
static int hf_rrc_pdcp_SN_Info = -1;              /* PDCP_SN_Info */
static int hf_rrc_rlc_Info = -1;                  /* RLC_Info */
static int hf_rrc_rb_StopContinue = -1;           /* RB_StopContinue */
static int hf_rrc_pdcp_Info_03 = -1;              /* PDCP_InfoReconfig_r4 */
static int hf_rrc_rlc_Info_01 = -1;               /* RLC_Info_r5 */
static int hf_rrc_rlc_Info_02 = -1;               /* RLC_Info_r6 */
static int hf_rrc_rlc_Info_03 = -1;               /* RLC_Info_r7 */
static int hf_rrc_rlc_Info_04 = -1;               /* RLC_Info_r11 */
static int hf_rrc_RB_InformationReconfigList_item = -1;  /* RB_InformationReconfig */
static int hf_rrc_RB_InformationReconfigList_r4_item = -1;  /* RB_InformationReconfig_r4 */
static int hf_rrc_RB_InformationReconfigList_r5_item = -1;  /* RB_InformationReconfig_r5 */
static int hf_rrc_RB_InformationReconfigList_r6_item = -1;  /* RB_InformationReconfig_r6 */
static int hf_rrc_RB_InformationReconfigList_r7_item = -1;  /* RB_InformationReconfig_r7 */
static int hf_rrc_RB_InformationReconfigList_r8_item = -1;  /* RB_InformationReconfig_r8 */
static int hf_rrc_RB_InformationReconfigList_r11_item = -1;  /* RB_InformationReconfig_r11 */
static int hf_rrc_RB_InformationReleaseList_item = -1;  /* RB_Identity */
static int hf_rrc_rlc_InfoChoice = -1;            /* RLC_InfoChoice */
static int hf_rrc_rlc_InfoChoice_01 = -1;         /* RLC_InfoChoice_r5 */
static int hf_rrc_rlc_InfoChoice_02 = -1;         /* RLC_InfoChoice_r6 */
static int hf_rrc_rlc_InfoChoice_03 = -1;         /* RLC_InfoChoice_r7 */
static int hf_rrc_rlc_InfoChoice_04 = -1;         /* RLC_InfoChoice_r11 */
static int hf_rrc_RB_InformationSetupList_item = -1;  /* RB_InformationSetup */
static int hf_rrc_RB_InformationSetupList_r4_item = -1;  /* RB_InformationSetup_r4 */
static int hf_rrc_RB_InformationSetupList_r5_item = -1;  /* RB_InformationSetup_r5 */
static int hf_rrc_RB_InformationSetupList_r6_item = -1;  /* RB_InformationSetup_r6 */
static int hf_rrc_RB_InformationSetupList_r7_item = -1;  /* RB_InformationSetup_r7 */
static int hf_rrc_RB_InformationSetupList_r8_item = -1;  /* RB_InformationSetup_r8 */
static int hf_rrc_RB_InformationSetupList_r11_item = -1;  /* RB_InformationSetup_r11 */
static int hf_rrc_RB_MappingInfo_item = -1;       /* RB_MappingOption */
static int hf_rrc_RB_MappingInfo_r5_item = -1;    /* RB_MappingOption_r5 */
static int hf_rrc_RB_MappingInfo_r6_item = -1;    /* RB_MappingOption_r6 */
static int hf_rrc_RB_MappingInfo_r7_item = -1;    /* RB_MappingOption_r7 */
static int hf_rrc_RB_MappingInfo_r8_item = -1;    /* RB_MappingOption_r8 */
static int hf_rrc_ul_LogicalChannelMappings = -1;  /* UL_LogicalChannelMappings */
static int hf_rrc_dl_LogicalChannelMappingList = -1;  /* DL_LogicalChannelMappingList */
static int hf_rrc_dl_LogicalChannelMappingList_01 = -1;  /* DL_LogicalChannelMappingList_r5 */
static int hf_rrc_ul_LogicalChannelMappings_01 = -1;  /* UL_LogicalChannelMappings_r6 */
static int hf_rrc_dl_LogicalChannelMappingList_02 = -1;  /* DL_LogicalChannelMappingList_r7 */
static int hf_rrc_ul_LogicalChannelMappings_02 = -1;  /* UL_LogicalChannelMappings_r8 */
static int hf_rrc_dl_RFC3095_Context_Relocation = -1;  /* BOOLEAN */
static int hf_rrc_ul_RFC3095_Context_Relocation = -1;  /* BOOLEAN */
static int hf_rrc_RB_PDCPContextRelocationList_item = -1;  /* RB_PDCPContextRelocation */
static int hf_rrc_RB_WithPDCP_InfoList_item = -1;  /* RB_WithPDCP_Info */
static int hf_rrc_f_MAX_PERIOD = -1;              /* INTEGER_1_65535 */
static int hf_rrc_f_MAX_TIME = -1;                /* INTEGER_1_255 */
static int hf_rrc_max_HEADER = -1;                /* INTEGER_60_65535 */
static int hf_rrc_tcp_SPACE = -1;                 /* INTEGER_3_255 */
static int hf_rrc_non_TCP_SPACE = -1;             /* INTEGER_3_65535 */
static int hf_rrc_expectReordering = -1;          /* ExpectReordering */
static int hf_rrc_rohcProfileList = -1;           /* ROHC_ProfileList_r4 */
static int hf_rrc_ul_RFC3095 = -1;                /* UL_RFC3095_r4 */
static int hf_rrc_dl_RFC3095 = -1;                /* DL_RFC3095_r4 */
static int hf_rrc_ul_RLC_Mode = -1;               /* UL_RLC_Mode */
static int hf_rrc_dl_RLC_Mode = -1;               /* DL_RLC_Mode */
static int hf_rrc_dl_RLC_Mode_01 = -1;            /* DL_RLC_Mode_r5 */
static int hf_rrc_rlc_OneSidedReEst = -1;         /* BOOLEAN */
static int hf_rrc_dl_RLC_Mode_02 = -1;            /* DL_RLC_Mode_r6 */
static int hf_rrc_altE_bitInterpretation = -1;    /* T_altE_bitInterpretation */
static int hf_rrc_dl_RLC_Mode_03 = -1;            /* DL_RLC_Mode_r7 */
static int hf_rrc_altE_bitInterpretation_01 = -1;  /* T_altE_bitInterpretation_01 */
static int hf_rrc_useSpecialValueOfHEField = -1;  /* T_useSpecialValueOfHEField */
static int hf_rrc_dl_RLC_Mode_04 = -1;            /* DL_RLC_Mode_r11 */
static int hf_rrc_altE_bitInterpretation_02 = -1;  /* T_altE_bitInterpretation_02 */
static int hf_rrc_useSpecialValueOfHEField_01 = -1;  /* T_useSpecialValueOfHEField_01 */
static int hf_rrc_dl_UM_RLC_OutOSeqDelivery_Info = -1;  /* UM_RLC_OutOSeqDelivery_Info_r6 */
static int hf_rrc_dl_UM_RLC_DuplAvoid_Reord_Info = -1;  /* UM_RLC_DuplAvoid_Reord_Info_r6 */
static int hf_rrc_same_as_RB = -1;                /* RB_Identity */
static int hf_rrc_lengthIndicatorSize = -1;       /* T_lengthIndicatorSize */
static int hf_rrc_minRLC_PDU_Size = -1;           /* INTEGER_0_1503 */
static int hf_rrc_largestRLC_PDU_Size = -1;       /* INTEGER_0_1503 */
static int hf_rrc_RLC_PDU_SizeList_item = -1;     /* RLC_PDU_Size */
static int hf_rrc_rlc_SizeIndex = -1;             /* INTEGER_1_maxTF */
static int hf_rrc_RLC_SizeExplicitList_item = -1;  /* RLC_SizeInfo */
static int hf_rrc_ROHC_ProfileList_r4_item = -1;  /* ROHC_Profile_r4 */
static int hf_rrc_ROHC_PacketSizeList_r4_item = -1;  /* ROHC_PacketSize_r4 */
static int hf_rrc_SRB_InformationSetupList_item = -1;  /* SRB_InformationSetup */
static int hf_rrc_SRB_InformationSetupList_r5_item = -1;  /* SRB_InformationSetup_r5 */
static int hf_rrc_SRB_InformationSetupList_r6_item = -1;  /* SRB_InformationSetup_r6 */
static int hf_rrc_SRB_InformationSetupList_r7_item = -1;  /* SRB_InformationSetup_r7 */
static int hf_rrc_SRB_InformationSetupList_r8_item = -1;  /* SRB_InformationSetup_r8 */
static int hf_rrc_SRB_InformationSetupList2_item = -1;  /* SRB_InformationSetup */
static int hf_rrc_SRB_InformationSetupList2_r6_item = -1;  /* SRB_InformationSetup_r6 */
static int hf_rrc_SRB_InformationSetupList2_r7_item = -1;  /* SRB_InformationSetup_r7 */
static int hf_rrc_SRB_InformationSetupList2_r8_item = -1;  /* SRB_InformationSetup_r8 */
static int hf_rrc_timerBasedExplicit = -1;        /* ExplicitDiscard */
static int hf_rrc_timerBasedNoExplicit = -1;      /* NoExplicitDiscard */
static int hf_rrc_maxDAT_Retransmissions = -1;    /* MaxDAT_Retransmissions */
static int hf_rrc_noDiscard = -1;                 /* MaxDAT */
static int hf_rrc_transmissionRLC_Discard = -1;   /* TransmissionRLC_Discard */
static int hf_rrc_transmissionWindowSize = -1;    /* TransmissionWindowSize */
static int hf_rrc_timerRST = -1;                  /* TimerRST */
static int hf_rrc_max_RST = -1;                   /* MaxRST */
static int hf_rrc_pollingInfo = -1;               /* PollingInfo */
static int hf_rrc_ul_TransportChannelType = -1;   /* UL_TransportChannelType */
static int hf_rrc_rlc_SizeList = -1;              /* T_rlc_SizeList */
static int hf_rrc_allSizes = -1;                  /* NULL */
static int hf_rrc_configured = -1;                /* NULL */
static int hf_rrc_explicitList = -1;              /* RLC_SizeExplicitList */
static int hf_rrc_mac_LogicalChannelPriority = -1;  /* MAC_LogicalChannelPriority */
static int hf_rrc_ul_TrCH_Type = -1;              /* T_ul_TrCH_Type */
static int hf_rrc_dch_rach_usch = -1;             /* T_dch_rach_usch */
static int hf_rrc_rlc_SizeList_01 = -1;           /* T_rlc_SizeList_01 */
static int hf_rrc_e_dch = -1;                     /* T_e_dch */
static int hf_rrc_e_DCH_MAC_d_FlowIdentity = -1;  /* E_DCH_MAC_d_FlowIdentity */
static int hf_rrc_ddi = -1;                       /* DDI */
static int hf_rrc_rlc_PDU_SizeList = -1;          /* RLC_PDU_SizeList */
static int hf_rrc_includeInSchedulingInfo = -1;   /* BOOLEAN */
static int hf_rrc_ul_TrCH_Type_01 = -1;           /* T_ul_TrCH_Type_01 */
static int hf_rrc_dch_rach_usch_01 = -1;          /* T_dch_rach_usch_01 */
static int hf_rrc_rlc_SizeList_02 = -1;           /* T_rlc_SizeList_02 */
static int hf_rrc_e_dch_01 = -1;                  /* T_e_dch_01 */
static int hf_rrc_rlc_PDU_Size = -1;              /* T_rlc_PDU_Size */
static int hf_rrc_fixedSize_01 = -1;              /* T_fixedSize */
static int hf_rrc_flexibleSize_02 = -1;           /* RLC_PDU_SizeConstraint */
static int hf_rrc_rlc_LogicalChannelMappingIndicator = -1;  /* BOOLEAN */
static int hf_rrc_ul_LogicalChannelMapping = -1;  /* SEQUENCE_SIZE_maxLoCHperRLC_OF_UL_LogicalChannelMapping */
static int hf_rrc_ul_LogicalChannelMapping_item = -1;  /* UL_LogicalChannelMapping */
static int hf_rrc_ul_LogicalChannelMapping_01 = -1;  /* SEQUENCE_SIZE_maxLoCHperRLC_OF_UL_LogicalChannelMapping_r6 */
static int hf_rrc_ul_LogicalChannelMapping_item_01 = -1;  /* UL_LogicalChannelMapping_r6 */
static int hf_rrc_ul_LogicalChannelMapping_02 = -1;  /* SEQUENCE_SIZE_maxLoCHperRLC_OF_UL_LogicalChannelMapping_r8 */
static int hf_rrc_ul_LogicalChannelMapping_item_02 = -1;  /* UL_LogicalChannelMapping_r8 */
static int hf_rrc_oneLogicalChannel = -1;         /* UL_LogicalChannelMapping */
static int hf_rrc_twoLogicalChannels = -1;        /* UL_LogicalChannelMappingList */
static int hf_rrc_oneLogicalChannel_01 = -1;      /* UL_LogicalChannelMapping_r6 */
static int hf_rrc_twoLogicalChannels_01 = -1;     /* UL_LogicalChannelMappingList_r6 */
static int hf_rrc_oneLogicalChannel_02 = -1;      /* UL_LogicalChannelMapping_r8 */
static int hf_rrc_twoLogicalChannels_02 = -1;     /* UL_LogicalChannelMappingList_r8 */
static int hf_rrc_dummy1_01 = -1;                 /* CID_InclusionInfo_r4 */
static int hf_rrc_dummy_33 = -1;                  /* ROHC_PacketSizeList_r4 */
static int hf_rrc_ul_AM_RLC_Mode = -1;            /* UL_AM_RLC_Mode */
static int hf_rrc_ul_UM_RLC_Mode = -1;            /* UL_UM_RLC_Mode */
static int hf_rrc_ul_TM_RLC_Mode = -1;            /* UL_TM_RLC_Mode */
static int hf_rrc_rach = -1;                      /* NULL */
static int hf_rrc_usch = -1;                      /* TransportChannelIdentity */
static int hf_rrc_timer_DAR = -1;                 /* TimerDAR_r6 */
static int hf_rrc_widowSize_DAR = -1;             /* WindowSizeDAR_r6 */
static int hf_rrc_timer_OSD = -1;                 /* TimerOSD_r6 */
static int hf_rrc_windowSize_OSD = -1;            /* WindowSizeOSD_r6 */
static int hf_rrc_mac_hs_AddReconfQueue_List = -1;  /* MAC_hs_AddReconfQueue_List */
static int hf_rrc_mac_hs_DelQueue_List = -1;      /* MAC_hs_DelQueue_List */
static int hf_rrc_mac_ehs_AddReconfQueue_List = -1;  /* MAC_ehs_AddReconfReordQ_List */
static int hf_rrc_dummy_34 = -1;                  /* MAC_ehs_DelReordQ_List */
static int hf_rrc_mac_ehs_AddReconfQueue_List_01 = -1;  /* MAC_ehs_AddReconfReordQ_List_r9 */
static int hf_rrc_mac_ehs_AddReconfQueue_List_02 = -1;  /* MAC_ehs_AddReconfReordQ_List_TDD128_v9c0ext */
static int hf_rrc_mac_ehs_AddReconfQueue_List_03 = -1;  /* MAC_ehs_AddReconfReordQ_List_r11 */
static int hf_rrc_AllowedTFC_List_item = -1;      /* TFC_Value */
static int hf_rrc_AllowedTFI_List_item = -1;      /* INTEGER_0_31 */
static int hf_rrc_sizeType1 = -1;                 /* INTEGER_0_127 */
static int hf_rrc_sizeType2 = -1;                 /* T_sizeType2 */
static int hf_rrc_part1 = -1;                     /* INTEGER_0_15 */
static int hf_rrc_part2 = -1;                     /* INTEGER_1_7 */
static int hf_rrc_sizeType3 = -1;                 /* T_sizeType3 */
static int hf_rrc_part1_01 = -1;                  /* INTEGER_0_47 */
static int hf_rrc_part2_01 = -1;                  /* INTEGER_1_15 */
static int hf_rrc_sizeType4 = -1;                 /* T_sizeType4 */
static int hf_rrc_part1_02 = -1;                  /* INTEGER_0_62 */
static int hf_rrc_part2_02 = -1;                  /* INTEGER_1_63 */
static int hf_rrc_noCoding = -1;                  /* NULL */
static int hf_rrc_convolutional = -1;             /* CodingRate */
static int hf_rrc_turbo = -1;                     /* NULL */
static int hf_rrc_mac_d_FlowIdentity = -1;        /* E_DCH_MAC_d_FlowIdentity */
static int hf_rrc_mac_d_FlowPowerOffset = -1;     /* E_DCH_MAC_d_FlowPowerOffset */
static int hf_rrc_mac_d_FlowMaxRetrans = -1;      /* E_DCH_MAC_d_FlowMaxRetrans */
static int hf_rrc_mac_d_FlowMultiplexingList = -1;  /* E_DCH_MAC_d_FlowMultiplexingList */
static int hf_rrc_e_dch_mac_d_flow_retransmission_timer = -1;  /* E_DCH_MAC_d_FlowRetransTimer */
static int hf_rrc_Common_E_DCH_MAC_d_FlowList_item = -1;  /* Common_E_DCH_MAC_d_Flow */
static int hf_rrc_Common_E_DCH_MAC_d_Flow_Info_List_ConcurrentTTI_item = -1;  /* Common_E_DCH_MAC_d_Flow_Info_ConcurrentTTI */
static int hf_rrc_rlc_Size = -1;                  /* T_rlc_Size */
static int hf_rrc_fdd_49 = -1;                    /* T_fdd_48 */
static int hf_rrc_octetModeRLC_SizeInfoType2 = -1;  /* OctetModeRLC_SizeInfoType2 */
static int hf_rrc_tdd_14 = -1;                    /* T_tdd_13 */
static int hf_rrc_commonTDD_Choice = -1;          /* T_commonTDD_Choice */
static int hf_rrc_bitModeRLC_SizeInfo = -1;       /* BitModeRLC_SizeInfo */
static int hf_rrc_octetModeRLC_SizeInfoType1 = -1;  /* OctetModeRLC_SizeInfoType1 */
static int hf_rrc_numberOfTbSizeList = -1;        /* SEQUENCE_SIZE_1_maxTF_OF_NumberOfTransportBlocks */
static int hf_rrc_numberOfTbSizeList_item = -1;   /* NumberOfTransportBlocks */
static int hf_rrc_logicalChannelList = -1;        /* LogicalChannelList */
static int hf_rrc_commonTDD_Choice_01 = -1;       /* T_commonTDD_Choice_01 */
static int hf_rrc_numberOfTbSizeAndTTIList = -1;  /* NumberOfTbSizeAndTTIList */
static int hf_rrc_CommonDynamicTF_InfoList_item = -1;  /* CommonDynamicTF_Info */
static int hf_rrc_CommonDynamicTF_InfoList_DynamicTTI_item = -1;  /* CommonDynamicTF_Info_DynamicTTI */
static int hf_rrc_tti = -1;                       /* T_tti */
static int hf_rrc_tti10 = -1;                     /* CommonDynamicTF_InfoList */
static int hf_rrc_tti20 = -1;                     /* CommonDynamicTF_InfoList */
static int hf_rrc_tti40 = -1;                     /* CommonDynamicTF_InfoList */
static int hf_rrc_tti80 = -1;                     /* CommonDynamicTF_InfoList */
static int hf_rrc_dynamic = -1;                   /* CommonDynamicTF_InfoList_DynamicTTI */
static int hf_rrc_semistaticTF_Information = -1;  /* SemistaticTF_Information */
static int hf_rrc_tti_01 = -1;                    /* T_tti_01 */
static int hf_rrc_tti5 = -1;                      /* CommonDynamicTF_InfoList */
static int hf_rrc_t1_ReleaseTimer = -1;           /* T1_ReleaseTimer */
static int hf_rrc_reorderingResetTimer = -1;      /* Treset_ResetTimer */
static int hf_rrc_mac_ehsWindowSize_01 = -1;      /* MAC_hs_WindowSize */
static int hf_rrc_Common_MAC_ehs_ReorderingQueueList_item = -1;  /* Common_MAC_ehs_ReorderingQueue */
static int hf_rrc_tti_selection = -1;             /* T_tti_selection */
static int hf_rrc_fixed = -1;                     /* T_fixed */
static int hf_rrc_fixed_TTI_Selection = -1;       /* T_fixed_TTI_Selection */
static int hf_rrc_threshold_based = -1;           /* T_threshold_based */
static int hf_rrc_common_E_DCH_TTI_Selection_Thresh = -1;  /* INTEGER_0_15 */
static int hf_rrc_concurrent_TTI_Partition_Index = -1;  /* INTEGER_0_maxEDCHs_1 */
static int hf_rrc_commonEDCHSystemInfoParamConcurrentTTI = -1;  /* CommonEDCHSystemInfoParamConcurrentTTI */
static int hf_rrc_rlc_Size_01 = -1;               /* T_rlc_Size_01 */
static int hf_rrc_bitMode = -1;                   /* BitModeRLC_SizeInfo */
static int hf_rrc_octetModeType1 = -1;            /* OctetModeRLC_SizeInfoType1 */
static int hf_rrc_rlc_Size_02 = -1;               /* T_rlc_Size_02 */
static int hf_rrc_DedicatedDynamicTF_InfoList_item = -1;  /* DedicatedDynamicTF_Info */
static int hf_rrc_DedicatedDynamicTF_InfoList_DynamicTTI_item = -1;  /* DedicatedDynamicTF_Info_DynamicTTI */
static int hf_rrc_tti_02 = -1;                    /* T_tti_02 */
static int hf_rrc_tti10_01 = -1;                  /* DedicatedDynamicTF_InfoList */
static int hf_rrc_tti20_01 = -1;                  /* DedicatedDynamicTF_InfoList */
static int hf_rrc_tti40_01 = -1;                  /* DedicatedDynamicTF_InfoList */
static int hf_rrc_tti80_01 = -1;                  /* DedicatedDynamicTF_InfoList */
static int hf_rrc_dynamic_01 = -1;                /* DedicatedDynamicTF_InfoList_DynamicTTI */
static int hf_rrc_DL_AddReconfTransChInfo2List_item = -1;  /* DL_AddReconfTransChInformation2 */
static int hf_rrc_DL_AddReconfTransChInfoList_item = -1;  /* DL_AddReconfTransChInformation */
static int hf_rrc_DL_AddReconfTransChInfoList_r4_item = -1;  /* DL_AddReconfTransChInformation_r4 */
static int hf_rrc_DL_AddReconfTransChInfoList_r5_item = -1;  /* DL_AddReconfTransChInformation_r5 */
static int hf_rrc_DL_AddReconfTransChInfoList_r7_item = -1;  /* DL_AddReconfTransChInformation_r7 */
static int hf_rrc_DL_AddReconfTransChInfoList_r9_item = -1;  /* DL_AddReconfTransChInformation_r9 */
static int hf_rrc_DL_AddReconfTransChInfoList_TDD128_v9c0ext_item = -1;  /* DL_AddReconfTransChInformation_TDD128_v9c0ext */
static int hf_rrc_DL_AddReconfTransChInfoList_r11_item = -1;  /* DL_AddReconfTransChInformation_r11 */
static int hf_rrc_dl_TransportChannelType_03 = -1;  /* DL_TrCH_Type */
static int hf_rrc_dl_transportChannelIdentity = -1;  /* TransportChannelIdentity */
static int hf_rrc_tfs_SignallingMode = -1;        /* T_tfs_SignallingMode */
static int hf_rrc_explicit_config = -1;           /* TransportFormatSet */
static int hf_rrc_sameAsULTrCH = -1;              /* UL_TransportChannelIdentity */
static int hf_rrc_dch_QualityTarget = -1;         /* QualityTarget */
static int hf_rrc_dummy_35 = -1;                  /* TM_SignallingInfo */
static int hf_rrc_tfs_SignallingMode_01 = -1;     /* T_tfs_SignallingMode_01 */
static int hf_rrc_dl_TransportChannelType_04 = -1;  /* DL_TrCH_TypeId1_r5 */
static int hf_rrc_tfs_SignallingMode_02 = -1;     /* T_tfs_SignallingMode_02 */
static int hf_rrc_hsdsch_02 = -1;                 /* HSDSCH_Info */
static int hf_rrc_tfs_SignallingMode_03 = -1;     /* T_tfs_SignallingMode_03 */
static int hf_rrc_hsdsch_03 = -1;                 /* HSDSCH_Info_r7 */
static int hf_rrc_tfs_SignallingMode_04 = -1;     /* T_tfs_SignallingMode_04 */
static int hf_rrc_hsdsch_04 = -1;                 /* HSDSCH_Info_r9 */
static int hf_rrc_hsdsch_05 = -1;                 /* HSDSCH_Info_TDD128_v9c0ext */
static int hf_rrc_tfs_SignallingMode_05 = -1;     /* T_tfs_SignallingMode_05 */
static int hf_rrc_hsdsch_06 = -1;                 /* HSDSCH_Info_r11 */
static int hf_rrc_transportChannelIdentity = -1;  /* TransportChannelIdentity */
static int hf_rrc_tfs_SignallingMode_06 = -1;     /* T_tfs_SignallingMode_06 */
static int hf_rrc_qualityTarget = -1;             /* QualityTarget */
static int hf_rrc_sccpch_TFCS = -1;               /* TFCS */
static int hf_rrc_modeSpecificInfo_19 = -1;       /* T_modeSpecificInfo_19 */
static int hf_rrc_fdd_50 = -1;                    /* T_fdd_49 */
static int hf_rrc_dl_Parameters = -1;             /* T_dl_Parameters */
static int hf_rrc_dl_DCH_TFCS = -1;               /* TFCS */
static int hf_rrc_sameAsUL = -1;                  /* NULL */
static int hf_rrc_tdd_15 = -1;                    /* T_tdd_14 */
static int hf_rrc_individualDL_CCTrCH_InfoList = -1;  /* IndividualDL_CCTrCH_InfoList */
static int hf_rrc_modeSpecificInfo_20 = -1;       /* T_modeSpecificInfo_20 */
static int hf_rrc_fdd_51 = -1;                    /* T_fdd_50 */
static int hf_rrc_dl_Parameters_01 = -1;          /* T_dl_Parameters_01 */
static int hf_rrc_dl_DCH_TFCS_01 = -1;            /* T_dl_DCH_TFCS */
static int hf_rrc_tfcs = -1;                      /* TFCS */
static int hf_rrc_tdd_16 = -1;                    /* T_tdd_15 */
static int hf_rrc_DL_DeletedTransChInfoList_item = -1;  /* DL_TransportChannelIdentity */
static int hf_rrc_DL_DeletedTransChInfoList_r5_item = -1;  /* DL_TransportChannelIdentity_r5 */
static int hf_rrc_DL_DeletedTransChInfoList_r7_item = -1;  /* DL_TransportChannelIdentity_r7 */
static int hf_rrc_dl_TransportChannelIdentity = -1;  /* TransportChannelIdentity */
static int hf_rrc_dl_TransportChannelType_05 = -1;  /* DL_TrCH_TypeId2_r5 */
static int hf_rrc_dl_TransportChannelType_06 = -1;  /* DL_TrCH_TypeId2_r7 */
static int hf_rrc_hsdsch_07 = -1;                 /* NULL */
static int hf_rrc_hsdsch_08 = -1;                 /* T_hsdsch_01 */
static int hf_rrc_transmissionTimeValidity = -1;  /* TransmissionTimeValidity */
static int hf_rrc_timeDurationBeforeRetry = -1;   /* TimeDurationBeforeRetry */
static int hf_rrc_drac_ClassIdentity = -1;        /* DRAC_ClassIdentity */
static int hf_rrc_DRAC_StaticInformationList_item = -1;  /* DRAC_StaticInformation */
static int hf_rrc_transmissionGrantType = -1;     /* T_transmissionGrantType */
static int hf_rrc_non_ScheduledTransGrantInfo = -1;  /* T_non_ScheduledTransGrantInfo */
static int hf_rrc_maxMAC_e_PDUContents = -1;      /* INTEGER_1_19982 */
static int hf_rrc_ms2_NonSchedTransmGrantHARQAlloc = -1;  /* BIT_STRING_SIZE_8 */
static int hf_rrc_scheduledTransmissionGrantInfo = -1;  /* NULL */
static int hf_rrc_mac_d_FlowRetransTimer = -1;    /* E_DCH_MAC_d_FlowRetransTimer */
static int hf_rrc_transmissionGrantType_01 = -1;  /* T_transmissionGrantType_01 */
static int hf_rrc_non_ScheduledTransGrantInfo_01 = -1;  /* T_non_ScheduledTransGrantInfo_01 */
static int hf_rrc_modeSpecificInfo_21 = -1;       /* T_modeSpecificInfo_21 */
static int hf_rrc_fdd_52 = -1;                    /* T_fdd_51 */
static int hf_rrc_E_DCH_AddReconf_MAC_d_FlowList_item = -1;  /* E_DCH_AddReconf_MAC_d_Flow */
static int hf_rrc_E_DCH_AddReconf_MAC_d_FlowList_r7_item = -1;  /* E_DCH_AddReconf_MAC_d_Flow_r7 */
static int hf_rrc_complete_26 = -1;               /* TFCS_ReconfAdd */
static int hf_rrc_addition = -1;                  /* TFCS_ReconfAdd */
static int hf_rrc_removal = -1;                   /* TFCS_RemovalList */
static int hf_rrc_replacement = -1;               /* T_replacement */
static int hf_rrc_tfcsRemoval = -1;               /* TFCS_RemovalList */
static int hf_rrc_tfcsAdd = -1;                   /* TFCS_ReconfAdd */
static int hf_rrc_signalledGainFactors = -1;      /* SignalledGainFactors */
static int hf_rrc_computedGainFactors = -1;       /* ReferenceTFC_ID */
static int hf_rrc_addOrReconfMAC_dFlow = -1;      /* AddOrReconfMAC_dFlow */
static int hf_rrc_dl_MAC_HeaderType = -1;         /* T_dl_MAC_HeaderType */
static int hf_rrc_mac_hs_02 = -1;                 /* AddOrReconfMAC_dFlow */
static int hf_rrc_mac_ehs_02 = -1;                /* AddOrReconfMAC_ehs_ReordQ */
static int hf_rrc_dl_MAC_HeaderType_01 = -1;      /* T_dl_MAC_HeaderType_01 */
static int hf_rrc_mac_ehs_03 = -1;                /* AddOrReconfMAC_ehs_ReordQ_r9 */
static int hf_rrc_mac_ehs_04 = -1;                /* AddOrReconfMAC_ehs_ReordQ_TDD128_v9c0ext */
static int hf_rrc_dl_MAC_HeaderType_02 = -1;      /* T_dl_MAC_HeaderType_02 */
static int hf_rrc_mac_ehs_05 = -1;                /* AddOrReconfMAC_ehs_ReordQ_r11 */
static int hf_rrc_numberOfProcesses = -1;         /* INTEGER_1_8 */
static int hf_rrc_memoryPartitioning = -1;        /* T_memoryPartitioning */
static int hf_rrc_implicit = -1;                  /* NULL */
static int hf_rrc_explicit = -1;                  /* SEQUENCE_SIZE_1_maxHProcesses_OF_HARQMemorySize */
static int hf_rrc_explicit_item = -1;             /* HARQMemorySize */
static int hf_rrc_numberOfProcesses_01 = -1;      /* T_numberOfProcesses */
static int hf_rrc_memoryPartitioning_01 = -1;     /* T_memoryPartitioning_01 */
static int hf_rrc_explicit_01 = -1;               /* T_explicit */
static int hf_rrc_memorySize = -1;                /* SEQUENCE_SIZE_1_maxHProcesses_OF_HARQMemorySize */
static int hf_rrc_memorySize_item = -1;           /* HARQMemorySize */
static int hf_rrc_additionalMemorySizesForMIMO = -1;  /* SEQUENCE_SIZE_1_maxHProcesses_OF_HARQMemorySize */
static int hf_rrc_additionalMemorySizesForMIMO_item = -1;  /* HARQMemorySize */
static int hf_rrc_numberOfProcesses_02 = -1;      /* T_numberOfProcesses_01 */
static int hf_rrc_memoryPartitioning_02 = -1;     /* T_memoryPartitioning_02 */
static int hf_rrc_explicit_02 = -1;               /* T_explicit_01 */
static int hf_rrc_additionalMemorySizesThirdMIMOStream = -1;  /* SEQUENCE_SIZE_1_maxHProcesses_OF_HARQMemorySize */
static int hf_rrc_additionalMemorySizesThirdMIMOStream_item = -1;  /* HARQMemorySize */
static int hf_rrc_additionalMemorySizesFourthMIMOStream = -1;  /* SEQUENCE_SIZE_1_maxHProcesses_OF_HARQMemorySize */
static int hf_rrc_additionalMemorySizesFourthMIMOStream_item = -1;  /* HARQMemorySize */
static int hf_rrc_dl_TFCS_Identity = -1;          /* TFCS_Identity */
static int hf_rrc_tfcs_SignallingMode = -1;       /* T_tfcs_SignallingMode */
static int hf_rrc_explicit_config_01 = -1;        /* TFCS */
static int hf_rrc_sameAsUL_01 = -1;               /* TFCS_Identity */
static int hf_rrc_IndividualDL_CCTrCH_InfoList_item = -1;  /* IndividualDL_CCTrCH_Info */
static int hf_rrc_ul_TFCS_Identity = -1;          /* TFCS_Identity */
static int hf_rrc_ul_TFCS = -1;                   /* TFCS */
static int hf_rrc_tfc_Subset = -1;                /* TFC_Subset */
static int hf_rrc_IndividualUL_CCTrCH_InfoList_item = -1;  /* IndividualUL_CCTrCH_Info */
static int hf_rrc_logChOfRb = -1;                 /* INTEGER_0_1 */
static int hf_rrc_explicitList_01 = -1;           /* SEQUENCE_SIZE_1_15_OF_LogicalChannelByRB */
static int hf_rrc_explicitList_item = -1;         /* LogicalChannelByRB */
static int hf_rrc_dch_transport_ch_id = -1;       /* TransportChannelIdentity */
static int hf_rrc_hsdsch_mac_d_flow_id = -1;      /* MAC_d_FlowIdentity */
static int hf_rrc_MAC_d_PDU_SizeInfo_List_item = -1;  /* MAC_d_PDUsizeInfo */
static int hf_rrc_mac_d_PDU_Size = -1;            /* INTEGER_1_5000 */
static int hf_rrc_mac_d_PDU_Index = -1;           /* INTEGER_0_7 */
static int hf_rrc_MAC_hs_AddReconfQueue_List_item = -1;  /* MAC_hs_AddReconfQueue */
static int hf_rrc_mac_hsQueueId = -1;             /* INTEGER_0_7 */
static int hf_rrc_mac_dFlowId = -1;               /* MAC_d_FlowIdentity */
static int hf_rrc_reorderingReleaseTimer = -1;    /* T1_ReleaseTimer */
static int hf_rrc_mac_hsWindowSize = -1;          /* MAC_hs_WindowSize */
static int hf_rrc_mac_d_PDU_SizeInfo_List = -1;   /* MAC_d_PDU_SizeInfo_List */
static int hf_rrc_MAC_hs_DelQueue_List_item = -1;  /* MAC_hs_DelQueue */
static int hf_rrc_MAC_ehs_AddReconfReordQ_List_item = -1;  /* MAC_ehs_AddReconfReordQ */
static int hf_rrc_MAC_ehs_AddReconfReordQ_List_r9_item = -1;  /* MAC_ehs_AddReconfReordQ_r9 */
static int hf_rrc_MAC_ehs_AddReconfReordQ_List_TDD128_v9c0ext_item = -1;  /* MAC_ehs_AddReconfReordQ_TDD128_v9c0ext */
static int hf_rrc_MAC_ehs_AddReconfReordQ_List_r11_item = -1;  /* MAC_ehs_AddReconfReordQ_r11 */
static int hf_rrc_mac_ehsWindowSize_extension = -1;  /* MAC_ehs_WindowSize_TDD128_v9c0ext */
static int hf_rrc_mac_ehsWindowSize_02 = -1;      /* MAC_hs_WindowSize_r11 */
static int hf_rrc_MAC_ehs_DelReordQ_List_item = -1;  /* MAC_ehs_DelReordQ */
static int hf_rrc_hsdsch_mac_ehs_QueueId = -1;    /* MAC_ehs_QueueId */
static int hf_rrc_NumberOfTbSizeAndTTIList_item = -1;  /* NumberOfTbSizeAndTTIList_item */
static int hf_rrc_numberOfTransportBlocks = -1;   /* NumberOfTransportBlocks */
static int hf_rrc_transmissionTimeInterval = -1;  /* TransmissionTimeInterval */
static int hf_rrc_Non_allowedTFC_List_item = -1;  /* TFC_Value */
static int hf_rrc_zero = -1;                      /* NULL */
static int hf_rrc_one = -1;                       /* NULL */
static int hf_rrc_small = -1;                     /* INTEGER_2_17 */
static int hf_rrc_large = -1;                     /* INTEGER_18_512 */
static int hf_rrc_sizeType1_01 = -1;              /* INTEGER_0_31 */
static int hf_rrc_sizeType2_01 = -1;              /* T_sizeType2_01 */
static int hf_rrc_part1_03 = -1;                  /* INTEGER_0_23 */
static int hf_rrc_part2_03 = -1;                  /* INTEGER_1_3 */
static int hf_rrc_sizeType3_01 = -1;              /* T_sizeType3_01 */
static int hf_rrc_part1_04 = -1;                  /* INTEGER_0_61 */
static int hf_rrc_sizeType2_02 = -1;              /* INTEGER_0_63 */
static int hf_rrc_sizeType3_02 = -1;              /* INTEGER_0_56 */
static int hf_rrc_referenceTFC = -1;              /* TFC_Value */
static int hf_rrc_modeSpecificInfo_22 = -1;       /* T_modeSpecificInfo_22 */
static int hf_rrc_fdd_53 = -1;                    /* T_fdd_52 */
static int hf_rrc_gainFactorBetaC = -1;           /* GainFactor */
static int hf_rrc_gainFactorBetaD = -1;           /* GainFactor */
static int hf_rrc_gainFactorInformation = -1;     /* GainFactorInformation */
static int hf_rrc_powerOffsetPp_m = -1;           /* PowerOffsetPp_m */
static int hf_rrc_ul_AddReconfTrChInfoList = -1;  /* UL_AddReconfTransChInfoList */
static int hf_rrc_dl_TrChInfoList = -1;           /* DL_AddReconfTransChInfoList */
static int hf_rrc_bler_QualityValue = -1;         /* BLER_QualityValue */
static int hf_rrc_ul_TransportChannelType_01 = -1;  /* UL_TrCH_Type */
static int hf_rrc_restrictedTrChIdentity = -1;    /* TransportChannelIdentity */
static int hf_rrc_allowedTFI_List = -1;           /* AllowedTFI_List */
static int hf_rrc_RestrictedTrChInfoList_item = -1;  /* RestrictedTrChInfo */
static int hf_rrc_channelCodingType = -1;         /* ChannelCodingType */
static int hf_rrc_rateMatchingAttribute = -1;     /* RateMatchingAttribute */
static int hf_rrc_crc_Size = -1;                  /* CRC_Size */
static int hf_rrc_modeSpecificInfo_23 = -1;       /* T_modeSpecificInfo_23 */
static int hf_rrc_fdd_54 = -1;                    /* T_fdd_53 */
static int hf_rrc_referenceTFC_ID = -1;           /* ReferenceTFC_ID */
static int hf_rrc_splitType = -1;                 /* SplitType */
static int hf_rrc_tfci_Field2_Length = -1;        /* INTEGER_1_10 */
static int hf_rrc_tfci_Field1_Information = -1;   /* ExplicitTFCS_Configuration */
static int hf_rrc_tfci_Field2_Information = -1;   /* TFCI_Field2_Information */
static int hf_rrc_minimumAllowedTFC_Number = -1;  /* TFC_Value */
static int hf_rrc_allowedTFC_List = -1;           /* AllowedTFC_List */
static int hf_rrc_non_allowedTFC_List = -1;       /* Non_allowedTFC_List */
static int hf_rrc_restrictedTrChInfoList = -1;    /* RestrictedTrChInfoList */
static int hf_rrc_fullTFCS = -1;                  /* NULL */
static int hf_rrc_TFC_SubsetList_item = -1;       /* TFC_SubsetList_item */
static int hf_rrc_modeSpecificInfo_24 = -1;       /* T_modeSpecificInfo_24 */
static int hf_rrc_tdd_17 = -1;                    /* T_tdd_16 */
static int hf_rrc_tfci_Range = -1;                /* TFCI_RangeList */
static int hf_rrc_explicit_config_02 = -1;        /* ExplicitTFCS_Configuration */
static int hf_rrc_maxTFCIField2Value = -1;        /* INTEGER_1_1023 */
static int hf_rrc_tfcs_InfoForDSCH = -1;          /* TFCS_InfoForDSCH */
static int hf_rrc_TFCI_RangeList_item = -1;       /* TFCI_Range */
static int hf_rrc_normalTFCI_Signalling = -1;     /* ExplicitTFCS_Configuration */
static int hf_rrc_dummy_36 = -1;                  /* SplitTFCI_Signalling */
static int hf_rrc_tfcs_ID_01 = -1;                /* TFCS_IdentityPlain */
static int hf_rrc_sharedChannelIndicator = -1;    /* BOOLEAN */
static int hf_rrc_ctfc2bit = -1;                  /* INTEGER_0_3 */
static int hf_rrc_ctfc4bit = -1;                  /* INTEGER_0_15 */
static int hf_rrc_ctfc6bit = -1;                  /* INTEGER_0_63 */
static int hf_rrc_ctfc8bit = -1;                  /* INTEGER_0_255 */
static int hf_rrc_ctfc12bit = -1;                 /* INTEGER_0_4095 */
static int hf_rrc_ctfc16bit = -1;                 /* INTEGER_0_65535 */
static int hf_rrc_ctfc24bit = -1;                 /* INTEGER_0_16777215 */
static int hf_rrc_ctfcSize = -1;                  /* T_ctfcSize */
static int hf_rrc_ctfc2Bit = -1;                  /* T_ctfc2Bit */
static int hf_rrc_ctfc2Bit_item = -1;             /* T_ctfc2Bit_item */
static int hf_rrc_ctfc2 = -1;                     /* INTEGER_0_3 */
static int hf_rrc_powerOffsetInformation = -1;    /* PowerOffsetInformation */
static int hf_rrc_ctfc4Bit = -1;                  /* T_ctfc4Bit */
static int hf_rrc_ctfc4Bit_item = -1;             /* T_ctfc4Bit_item */
static int hf_rrc_ctfc4 = -1;                     /* INTEGER_0_15 */
static int hf_rrc_ctfc6Bit = -1;                  /* T_ctfc6Bit */
static int hf_rrc_ctfc6Bit_item = -1;             /* T_ctfc6Bit_item */
static int hf_rrc_ctfc6 = -1;                     /* INTEGER_0_63 */
static int hf_rrc_ctfc8Bit = -1;                  /* T_ctfc8Bit */
static int hf_rrc_ctfc8Bit_item = -1;             /* T_ctfc8Bit_item */
static int hf_rrc_ctfc8 = -1;                     /* INTEGER_0_255 */
static int hf_rrc_ctfc12Bit = -1;                 /* T_ctfc12Bit */
static int hf_rrc_ctfc12Bit_item = -1;            /* T_ctfc12Bit_item */
static int hf_rrc_ctfc12 = -1;                    /* INTEGER_0_4095 */
static int hf_rrc_ctfc16Bit = -1;                 /* T_ctfc16Bit */
static int hf_rrc_ctfc16Bit_item = -1;            /* T_ctfc16Bit_item */
static int hf_rrc_ctfc16 = -1;                    /* INTEGER_0_65535 */
static int hf_rrc_ctfc24Bit = -1;                 /* T_ctfc24Bit */
static int hf_rrc_ctfc24Bit_item = -1;            /* T_ctfc24Bit_item */
static int hf_rrc_ctfc24 = -1;                    /* INTEGER_0_16777215 */
static int hf_rrc_tfci = -1;                      /* INTEGER_0_1023 */
static int hf_rrc_TFCS_RemovalList_item = -1;     /* TFCS_Removal */
static int hf_rrc_messType = -1;                  /* MessType */
static int hf_rrc_tm_SignallingMode = -1;         /* T_tm_SignallingMode */
static int hf_rrc_mode1 = -1;                     /* NULL */
static int hf_rrc_mode2 = -1;                     /* T_mode2 */
static int hf_rrc_ul_controlledTrChList = -1;     /* UL_ControlledTrChList */
static int hf_rrc_dsch_transport_ch_id = -1;      /* TransportChannelIdentity */
static int hf_rrc_dedicatedTransChTFS = -1;       /* DedicatedTransChTFS */
static int hf_rrc_commonTransChTFS = -1;          /* CommonTransChTFS */
static int hf_rrc_commonTransChTFS_LCR = -1;      /* CommonTransChTFS_LCR */
static int hf_rrc_UL_AddReconfTransChInfoList_item = -1;  /* UL_AddReconfTransChInformation */
static int hf_rrc_UL_AddReconfTransChInfoList_r6_item = -1;  /* UL_AddReconfTransChInformation_r6 */
static int hf_rrc_UL_AddReconfTransChInfoList_r7_item = -1;  /* UL_AddReconfTransChInformation_r7 */
static int hf_rrc_UL_AddReconfTransChInfoList_r8_item = -1;  /* UL_AddReconfTransChInformation_r8 */
static int hf_rrc_transportFormatSet = -1;        /* TransportFormatSet */
static int hf_rrc_dch_usch = -1;                  /* T_dch_usch */
static int hf_rrc_e_dch_02 = -1;                  /* T_e_dch_02 */
static int hf_rrc_tti_03 = -1;                    /* E_DCH_TTI */
static int hf_rrc_harq_Info = -1;                 /* T_harq_Info */
static int hf_rrc_addReconf_MAC_d_FlowList = -1;  /* E_DCH_AddReconf_MAC_d_FlowList */
static int hf_rrc_dch_usch_01 = -1;               /* T_dch_usch_01 */
static int hf_rrc_e_dch_03 = -1;                  /* T_e_dch_03 */
static int hf_rrc_modeSpecific = -1;              /* T_modeSpecific */
static int hf_rrc_fdd_55 = -1;                    /* T_fdd_54 */
static int hf_rrc_harq_Info_01 = -1;              /* T_harq_Info_01 */
static int hf_rrc_addReconf_MAC_d_FlowList_01 = -1;  /* E_DCH_AddReconf_MAC_d_FlowList_r7 */
static int hf_rrc_dch_usch_02 = -1;               /* T_dch_usch_02 */
static int hf_rrc_e_dch_04 = -1;                  /* T_e_dch_04 */
static int hf_rrc_ul_MAC_HeaderType = -1;         /* T_ul_MAC_HeaderType */
static int hf_rrc_modeSpecific_01 = -1;           /* T_modeSpecific_01 */
static int hf_rrc_fdd_56 = -1;                    /* T_fdd_55 */
static int hf_rrc_harq_Info_02 = -1;              /* T_harq_Info_02 */
static int hf_rrc_prach_TFCS = -1;                /* TFCS */
static int hf_rrc_modeSpecificInfo_25 = -1;       /* T_modeSpecificInfo_25 */
static int hf_rrc_fdd_57 = -1;                    /* T_fdd_56 */
static int hf_rrc_tdd_18 = -1;                    /* T_tdd_17 */
static int hf_rrc_individualUL_CCTrCH_InfoList = -1;  /* IndividualUL_CCTrCH_InfoList */
static int hf_rrc_modeSpecificInfo_26 = -1;       /* T_modeSpecificInfo_26 */
static int hf_rrc_fdd_58 = -1;                    /* T_fdd_57 */
static int hf_rrc_tdd_19 = -1;                    /* T_tdd_18 */
static int hf_rrc_tfc_SubsetList = -1;            /* TFC_SubsetList */
static int hf_rrc_UL_ControlledTrChList_item = -1;  /* TransportChannelIdentity */
static int hf_rrc_UL_DeletedTransChInfoList_item = -1;  /* UL_TransportChannelIdentity */
static int hf_rrc_UL_DeletedTransChInfoList_r6_item = -1;  /* UL_TransportChannelIdentity_r6 */
static int hf_rrc_ul_TransportChannelIdentity = -1;  /* TransportChannelIdentity */
static int hf_rrc_dch_usch_03 = -1;               /* T_dch_usch_03 */
static int hf_rrc_e_dch_05 = -1;                  /* E_DCH_MAC_d_FlowIdentity */
static int hf_rrc_USCH_TransportChannelsInfo_item = -1;  /* USCH_TransportChannelsInfo_item */
static int hf_rrc_usch_TransportChannelIdentity = -1;  /* TransportChannelIdentity */
static int hf_rrc_usch_TFS = -1;                  /* TransportFormatSet */
static int hf_rrc_AC_To_ASC_MappingTable_item = -1;  /* AC_To_ASC_Mapping */
static int hf_rrc_availableSignatureStartIndex = -1;  /* INTEGER_0_15 */
static int hf_rrc_availableSignatureEndIndex = -1;  /* INTEGER_0_15 */
static int hf_rrc_assignedSubChannelNumber = -1;  /* T_assignedSubChannelNumber */
static int hf_rrc_channelisationCodeIndices = -1;  /* T_channelisationCodeIndices */
static int hf_rrc_subchannelSize = -1;            /* T_subchannelSize */
static int hf_rrc_size1 = -1;                     /* NULL */
static int hf_rrc_size2 = -1;                     /* T_size2 */
static int hf_rrc_subchannels = -1;               /* T_subchannels */
static int hf_rrc_size4 = -1;                     /* T_size4 */
static int hf_rrc_subchannels_01 = -1;            /* T_subchannels_01 */
static int hf_rrc_size8 = -1;                     /* T_size8 */
static int hf_rrc_subchannels_02 = -1;            /* T_subchannels_02 */
static int hf_rrc_channelisationCodeIndices_01 = -1;  /* T_channelisationCodeIndices_01 */
static int hf_rrc_subchannelSize_01 = -1;         /* T_subchannelSize_01 */
static int hf_rrc_size2_01 = -1;                  /* T_size2_01 */
static int hf_rrc_subchannels_03 = -1;            /* T_subchannels_03 */
static int hf_rrc_size4_01 = -1;                  /* T_size4_01 */
static int hf_rrc_subchannels_04 = -1;            /* T_subchannels_04 */
static int hf_rrc_size8_01 = -1;                  /* T_size8_01 */
static int hf_rrc_subchannels_05 = -1;            /* T_subchannels_05 */
static int hf_rrc_size16 = -1;                    /* T_size16 */
static int hf_rrc_subchannels_06 = -1;            /* T_subchannels_06 */
static int hf_rrc_availableSYNC_UlCodesIndics = -1;  /* T_availableSYNC_UlCodesIndics */
static int hf_rrc_subchannelSize_02 = -1;         /* T_subchannelSize_02 */
static int hf_rrc_size2_02 = -1;                  /* T_size2_02 */
static int hf_rrc_subchannels_07 = -1;            /* T_subchannels_07 */
static int hf_rrc_size4_02 = -1;                  /* T_size4_02 */
static int hf_rrc_subchannels_08 = -1;            /* T_subchannels_08 */
static int hf_rrc_size8_02 = -1;                  /* T_size8_02 */
static int hf_rrc_subchannels_09 = -1;            /* T_subchannels_09 */
static int hf_rrc_dynamicTFInformationCCCH = -1;  /* DynamicTFInformationCCCH */
static int hf_rrc_additionalPRACH_TF_and_TFCS_CCCH_IEs = -1;  /* AdditionalPRACH_TF_and_TFCS_CCCH_IEs */
static int hf_rrc_AdditionalPRACH_TF_and_TFCS_CCCH_List_item = -1;  /* AdditionalPRACH_TF_and_TFCS_CCCH */
static int hf_rrc_AdditionalDLSecCellInfoListFDD_item = -1;  /* AdditionalDLSecCellInfoFDD */
static int hf_rrc_AdditionalDLSecCellInfoListFDD_r11_item = -1;  /* AdditionalDLSecCellInfoFDD_r11 */
static int hf_rrc_AdditionalDLSecCellInfoListFDD2_r11_item = -1;  /* AdditionalDLSecCellInfoFDD_r11 */
static int hf_rrc_channelisationCode256 = -1;     /* ChannelisationCode256 */
static int hf_rrc_sttd_Indicator = -1;            /* BOOLEAN */
static int hf_rrc_aich_TransmissionTiming = -1;   /* AICH_TransmissionTiming */
static int hf_rrc_allocationActivationTime = -1;  /* INTEGER_0_255 */
static int hf_rrc_allocationDuration = -1;        /* INTEGER_1_256 */
static int hf_rrc_antenna3And4_S_CPICH = -1;      /* Antenna3And4_S_CPICH */
static int hf_rrc_antenna3And4_D_CPICH = -1;      /* Antenna3And4_D_CPICH */
static int hf_rrc_antenna3_S_CPICH = -1;          /* Antenna3_S_CPICH */
static int hf_rrc_antenna4_S_CPICH = -1;          /* Antenna4_S_CPICH */
static int hf_rrc_s_cpich_PowerOffset_Mimo = -1;  /* S_CPICH_PowerOffset_4x4MIMO */
static int hf_rrc_antenna3_D_CPICH = -1;          /* Antenna3_D_CPICH */
static int hf_rrc_antenna4_D_CPICH = -1;          /* Antenna4_D_CPICH */
static int hf_rrc_d_cpich_PowerOffset_Mimo = -1;  /* D_CPICH_PowerOffset_4x4MIMO */
static int hf_rrc_initialD_CPICHStatus = -1;      /* T_initialD_CPICHStatus */
static int hf_rrc_channelisationCode = -1;        /* ChannelisationCode256 */
static int hf_rrc_ap_Signature = -1;              /* AP_Signature */
static int hf_rrc_availableAP_SubchannelList = -1;  /* AvailableAP_SubchannelList */
static int hf_rrc_accessServiceClass_FDD = -1;    /* AccessServiceClass_FDD */
static int hf_rrc_accessServiceClass_TDD = -1;    /* AccessServiceClass_TDD */
static int hf_rrc_accessServiceClass_TDD_01 = -1;  /* AccessServiceClass_TDD_r7 */
static int hf_rrc_accessServiceClass_TDD_LCR = -1;  /* AccessServiceClass_TDD_LCR_r4 */
static int hf_rrc_AvailableAP_Signature_VCAMList_item = -1;  /* AP_Signature_VCAM */
static int hf_rrc_AvailableAP_SignatureList_item = -1;  /* AP_Signature */
static int hf_rrc_AvailableAP_SubchannelList_item = -1;  /* AP_Subchannel */
static int hf_rrc_AvailableMinimumSF_ListVCAM_item = -1;  /* AvailableMinimumSF_VCAM */
static int hf_rrc_minimumSpreadingFactor = -1;    /* MinimumSpreadingFactor */
static int hf_rrc_nf_Max = -1;                    /* NF_Max */
static int hf_rrc_maxAvailablePCPCH_Number = -1;  /* MaxAvailablePCPCH_Number */
static int hf_rrc_availableAP_Signature_VCAMList = -1;  /* AvailableAP_Signature_VCAMList */
static int hf_rrc_tfcs_Identity = -1;             /* TFCS_Identity */
static int hf_rrc_ul_DPCH_PowerControlInfo = -1;  /* UL_DPCH_PowerControlInfo */
static int hf_rrc_ul_DPCH_PowerControlInfo_01 = -1;  /* UL_DPCH_PowerControlInfo_r4 */
static int hf_rrc_ul_DPCH_PowerControlInfo_02 = -1;  /* UL_DPCH_PowerControlInfo_r5 */
static int hf_rrc_ul_DPCH_PowerControlInfo_03 = -1;  /* UL_DPCH_PowerControlInfo_r7 */
static int hf_rrc_CD_AccessSlotSubchannelList_item = -1;  /* CD_AccessSlotSubchannel */
static int hf_rrc_CD_SignatureCodeList_item = -1;  /* CD_SignatureCode */
static int hf_rrc_burstType = -1;                 /* BurstType */
static int hf_rrc_midambleShift = -1;             /* MidambleShiftLong */
static int hf_rrc_timeslot = -1;                  /* TimeslotNumber */
static int hf_rrc_cellParametersID = -1;          /* CellParametersID */
static int hf_rrc_notActive = -1;                 /* NULL */
static int hf_rrc_isActive = -1;                  /* AvailableMinimumSF_ListVCAM */
static int hf_rrc_availableAP_SignatureList = -1;  /* AvailableAP_SignatureList */
static int hf_rrc_pdsch_CodeMapList = -1;         /* PDSCH_CodeMapList */
static int hf_rrc_start_code = -1;                /* HS_ChannelisationCode_LCR */
static int hf_rrc_stop_code = -1;                 /* HS_ChannelisationCode_LCR */
static int hf_rrc_s_offset = -1;                  /* INTEGER_0_9 */
static int hf_rrc_f_dpch_ChannelisationCodeNumber = -1;  /* INTEGER_0_255 */
static int hf_rrc_e_RGCH_Information_01 = -1;     /* E_RGCH_Information_CommonEdch */
static int hf_rrc_e_hich_Info = -1;               /* E_HICH_Information_CommonEdch */
static int hf_rrc_ul_DPCH_CodeInfoForCommonEDCH = -1;  /* UL_DPCH_CodeInfoForCommonEDCH */
static int hf_rrc_twoMsHarqConfiguration = -1;    /* TwoMsHarqConfiguration */
static int hf_rrc_Common_E_RNTI_Info_item = -1;   /* Common_E_RNTI_Info_item */
static int hf_rrc_starting_E_RNTI = -1;           /* E_RNTI */
static int hf_rrc_number_of_group = -1;           /* INTEGER_1_maxERNTIgroup */
static int hf_rrc_number_of_ENRTI_per_group = -1;  /* INTEGER_1_maxERNTIperGroup */
static int hf_rrc_scheduledTransmissionConfiguration = -1;  /* ScheduledTransmissionConfiguration */
static int hf_rrc_cOffset = -1;                   /* INTEGER_0_29 */
static int hf_rrc_ul_InterferenceForCommonEDCH = -1;  /* UL_Interference */
static int hf_rrc_common_E_DCH_MAC_d_FlowList = -1;  /* Common_E_DCH_MAC_d_FlowList */
static int hf_rrc_modeSpecificInfo_27 = -1;       /* T_modeSpecificInfo_27 */
static int hf_rrc_dummy_37 = -1;                  /* T_dummy_14 */
static int hf_rrc_tdd_20 = -1;                    /* T_tdd_19 */
static int hf_rrc_tdd768_06 = -1;                 /* NULL */
static int hf_rrc_tdd384_11 = -1;                 /* NULL */
static int hf_rrc_tdd128_11 = -1;                 /* T_tdd128_11 */
static int hf_rrc_e_RUCCH_Info = -1;              /* E_RUCCH_Info_TDD128 */
static int hf_rrc_e_PUCH_Info = -1;               /* E_PUCH_Info_TDD128 */
static int hf_rrc_e_hich_Information = -1;        /* E_HICH_Information_TDD128 */
static int hf_rrc_e_agch_Information = -1;        /* E_AGCH_Information_TDD128 */
static int hf_rrc_harq_Info_03 = -1;              /* T_harq_Info_03 */
static int hf_rrc_ccch_transmission_Info = -1;    /* T_ccch_transmission_Info */
static int hf_rrc_common_e_rnti_Info = -1;        /* Common_E_RNTI_Info */
static int hf_rrc_harq_MaximumNumberOfRetransmissions = -1;  /* INTEGER_0_7 */
static int hf_rrc_harq_retransmission_timer = -1;  /* T_harq_retransmission_timer */
static int hf_rrc_harq_power_offset = -1;         /* INTEGER_0_6 */
static int hf_rrc_prach_PreambleForEnhancedUplink = -1;  /* PRACH_PreambleForEnhancedUplink */
static int hf_rrc_initialServingGrantValue = -1;  /* INTEGER_0_37 */
static int hf_rrc_e_dch_TTI = -1;                 /* E_DCH_TTI */
static int hf_rrc_e_agch_Information_01 = -1;     /* E_AGCH_Information */
static int hf_rrc_harq_Info_04 = -1;              /* T_harq_Info_04 */
static int hf_rrc_ul_DPCHpowerControlInfoForCommonEDCH = -1;  /* UL_DPCHpowerControlInfoForCommonEDCH */
static int hf_rrc_e_dpcch_Info = -1;              /* E_DPCCH_Info_r7 */
static int hf_rrc_e_dpdch_Info = -1;              /* E_DPDCH_Info_r8 */
static int hf_rrc_dl_FDPCH_TPCcommandErrorRate = -1;  /* INTEGER_1_10 */
static int hf_rrc_additional_E_DCH_TransmitBackoff = -1;  /* INTEGER_0_15 */
static int hf_rrc_max_CCCH_ResourceAllocation = -1;  /* T_max_CCCH_ResourceAllocation */
static int hf_rrc_max_PeriodForCollisionResolution = -1;  /* INTEGER_8_24 */
static int hf_rrc_e_dch_TransmitContinuationOffset = -1;  /* T_e_dch_TransmitContinuationOffset */
static int hf_rrc_ack_nack_support_on_HS_DPCCH = -1;  /* BOOLEAN */
static int hf_rrc_measurement_Feedback_Info = -1;  /* Measurement_Feedback_Info_r7 */
static int hf_rrc_common_E_DCH_ResourceInfoList = -1;  /* SEQUENCE_SIZE_1_maxEDCHs_OF_Common_E_DCH_ResourceInfoList */
static int hf_rrc_common_E_DCH_ResourceInfoList_item = -1;  /* Common_E_DCH_ResourceInfoList */
static int hf_rrc_common_E_DCH_MAC_d_Flow_Info_List_ConcurrentTTI = -1;  /* Common_E_DCH_MAC_d_Flow_Info_List_ConcurrentTTI */
static int hf_rrc_ul_DPCHpowerControlInfoConcurrentTTI = -1;  /* UL_DPCHpowerControlInfoConcurrentTTI */
static int hf_rrc_e_DPCCH_DPCCH_PowerOffset = -1;  /* E_DPCCH_DPCCH_PowerOffset */
static int hf_rrc_max_CCCH_ResourceAllocation_01 = -1;  /* T_max_CCCH_ResourceAllocation_01 */
static int hf_rrc_e_dch_TransmitContinuationOffset_01 = -1;  /* T_e_dch_TransmitContinuationOffset_01 */
static int hf_rrc_measurement_Feedback_Info_ConcurrentTTI = -1;  /* Measurement_Feedback_Info_ConcurrentTTI */
static int hf_rrc_channelisationCode_01 = -1;     /* INTEGER_0_127 */
static int hf_rrc_signatureSequence = -1;         /* INTEGER_0_39 */
static int hf_rrc_configurationInfo = -1;         /* T_configurationInfo */
static int hf_rrc_newConfiguration_01 = -1;       /* T_newConfiguration_01 */
static int hf_rrc_eRGCHNeighbourCellList = -1;    /* BIT_STRING_SIZE_maxCellMeas */
static int hf_rrc_commonERGCHChannelConfigList = -1;  /* SEQUENCE_SIZE_1_maxCellMeas_OF_CommonERGCHChannelConfig */
static int hf_rrc_commonERGCHChannelConfigList_item = -1;  /* CommonERGCHChannelConfig */
static int hf_rrc_minimumServingGrantValue = -1;  /* INTEGER_0_37 */
static int hf_rrc_reportingRange = -1;            /* ReportingRange */
static int hf_rrc_filterCoefficient = -1;         /* FilterCoefficient */
static int hf_rrc_secondInterleavingMode = -1;    /* SecondInterleavingMode */
static int hf_rrc_tfci_Coding = -1;               /* TFCI_Coding */
static int hf_rrc_puncturingLimit = -1;           /* PuncturingLimit */
static int hf_rrc_repetitionPeriodAndLength = -1;  /* RepetitionPeriodAndLength */
static int hf_rrc_repetitionPeriodLengthAndOffset = -1;  /* RepetitionPeriodLengthAndOffset */
static int hf_rrc_controlChannelDrxOperation = -1;  /* T_controlChannelDrxOperation */
static int hf_rrc_continue_01 = -1;               /* T_continue */
static int hf_rrc_enablingDelay = -1;             /* EnablingDelay_TDD128 */
static int hf_rrc_newOperation = -1;              /* T_newOperation */
static int hf_rrc_hS_SCCH_Drx_Info = -1;          /* HS_SCCH_DRX_Info_TDD128 */
static int hf_rrc_e_AGCH_Drx_Info = -1;           /* E_AGCH_DRX_Info_TDD128 */
static int hf_rrc_cpch_SetID = -1;                /* CPCH_SetID */
static int hf_rrc_dynamicPersistenceLevelTF_List = -1;  /* DynamicPersistenceLevelTF_List */
static int hf_rrc_CPCH_PersistenceLevelsList_item = -1;  /* CPCH_PersistenceLevels */
static int hf_rrc_ap_PreambleScramblingCode = -1;  /* AP_PreambleScramblingCode */
static int hf_rrc_ap_AICH_ChannelisationCode = -1;  /* AP_AICH_ChannelisationCode */
static int hf_rrc_cd_PreambleScramblingCode = -1;  /* CD_PreambleScramblingCode */
static int hf_rrc_cd_CA_ICH_ChannelisationCode = -1;  /* CD_CA_ICH_ChannelisationCode */
static int hf_rrc_cd_AccessSlotSubchannelList = -1;  /* CD_AccessSlotSubchannelList */
static int hf_rrc_cd_SignatureCodeList = -1;      /* CD_SignatureCodeList */
static int hf_rrc_deltaPp_m = -1;                 /* DeltaPp_m */
static int hf_rrc_ul_DPCCH_SlotFormat = -1;       /* UL_DPCCH_SlotFormat */
static int hf_rrc_n_StartMessage = -1;            /* N_StartMessage */
static int hf_rrc_n_EOT = -1;                     /* N_EOT */
static int hf_rrc_channelAssignmentActive = -1;   /* ChannelAssignmentActive */
static int hf_rrc_cpch_StatusIndicationMode = -1;  /* CPCH_StatusIndicationMode */
static int hf_rrc_pcpch_ChannelInfoList = -1;     /* PCPCH_ChannelInfoList */
static int hf_rrc_CPCH_SetInfoList_item = -1;     /* CPCH_SetInfo */
static int hf_rrc_timeInfo = -1;                  /* TimeInfo */
static int hf_rrc_commonTimeslotInfo = -1;        /* CommonTimeslotInfo */
static int hf_rrc_dl_CCTrCH_TimeslotsCodes = -1;  /* DownlinkTimeslotsCodes */
static int hf_rrc_ul_CCTrChTPCList = -1;          /* UL_CCTrChTPCList */
static int hf_rrc_tddOption_03 = -1;              /* T_tddOption_03 */
static int hf_rrc_tdd384_12 = -1;                 /* T_tdd384_11 */
static int hf_rrc_tdd128_12 = -1;                 /* T_tdd128_12 */
static int hf_rrc_dl_CCTrCH_TimeslotsCodes_01 = -1;  /* DownlinkTimeslotsCodes_LCR_r4 */
static int hf_rrc_tddOption_04 = -1;              /* T_tddOption_04 */
static int hf_rrc_tdd384_13 = -1;                 /* T_tdd384_12 */
static int hf_rrc_dl_CCTrCH_TimeslotsCodes_02 = -1;  /* DownlinkTimeslotsCodes_r7 */
static int hf_rrc_tdd768_07 = -1;                 /* T_tdd768_06 */
static int hf_rrc_dl_CCTrCH_TimeslotsCodes_03 = -1;  /* DownlinkTimeslotsCodes_VHCR */
static int hf_rrc_tdd128_13 = -1;                 /* T_tdd128_13 */
static int hf_rrc_DL_CCTrChList_item = -1;        /* DL_CCTrCh */
static int hf_rrc_DL_CCTrChList_r7_item = -1;     /* DL_CCTrCh_r7 */
static int hf_rrc_DL_CCTrChList_r4_item = -1;     /* DL_CCTrCh_r4 */
static int hf_rrc_DL_CCTrChListToRemove_item = -1;  /* TFCS_IdentityPlain */
static int hf_rrc_secondaryScramblingCode = -1;   /* SecondaryScramblingCode */
static int hf_rrc_sf_AndCodeNumber = -1;          /* SF512_AndCodeNumber */
static int hf_rrc_scramblingCodeChange = -1;      /* ScramblingCodeChange */
static int hf_rrc_DL_ChannelisationCodeList_item = -1;  /* DL_ChannelisationCode */
static int hf_rrc_dl_DPCH_InfoCommon = -1;        /* DL_DPCH_InfoCommon */
static int hf_rrc_modeSpecificInfo_28 = -1;       /* T_modeSpecificInfo_28 */
static int hf_rrc_fdd_59 = -1;                    /* T_fdd_58 */
static int hf_rrc_defaultDPCH_OffsetValue = -1;   /* DefaultDPCH_OffsetValueFDD */
static int hf_rrc_dpch_CompressedModeInfo = -1;   /* DPCH_CompressedModeInfo */
static int hf_rrc_dummy_38 = -1;                  /* SSDT_Information */
static int hf_rrc_tdd_21 = -1;                    /* T_tdd_20 */
static int hf_rrc_defaultDPCH_OffsetValue_01 = -1;  /* DefaultDPCH_OffsetValueTDD */
static int hf_rrc_dl_DPCH_InfoCommon_01 = -1;     /* DL_DPCH_InfoCommon_r4 */
static int hf_rrc_modeSpecificInfo_29 = -1;       /* T_modeSpecificInfo_29 */
static int hf_rrc_fdd_60 = -1;                    /* T_fdd_59 */
static int hf_rrc_dummy_39 = -1;                  /* SSDT_Information_r4 */
static int hf_rrc_tdd_22 = -1;                    /* T_tdd_21 */
static int hf_rrc_tddOption_05 = -1;              /* T_tddOption_05 */
static int hf_rrc_tdd128_14 = -1;                 /* T_tdd128_14 */
static int hf_rrc_tstd_Indicator = -1;            /* BOOLEAN */
static int hf_rrc_modeSpecificInfo_30 = -1;       /* T_modeSpecificInfo_30 */
static int hf_rrc_fdd_61 = -1;                    /* T_fdd_60 */
static int hf_rrc_tdd_23 = -1;                    /* T_tdd_22 */
static int hf_rrc_tddOption_06 = -1;              /* T_tddOption_06 */
static int hf_rrc_tdd128_15 = -1;                 /* T_tdd128_15 */
static int hf_rrc_mac_hsResetIndicator_05 = -1;   /* T_mac_hsResetIndicator_05 */
static int hf_rrc_dl_dpchInfoCommon = -1;         /* T_dl_dpchInfoCommon */
static int hf_rrc_dl_DPCH_InfoCommon_02 = -1;     /* DL_DPCH_InfoCommon_r6 */
static int hf_rrc_dl_FDPCH_InfoCommon = -1;       /* DL_FDPCH_InfoCommon_r6 */
static int hf_rrc_modeSpecificInfo_31 = -1;       /* T_modeSpecificInfo_31 */
static int hf_rrc_fdd_62 = -1;                    /* T_fdd_61 */
static int hf_rrc_tdd_24 = -1;                    /* T_tdd_23 */
static int hf_rrc_tddOption_07 = -1;              /* T_tddOption_07 */
static int hf_rrc_tdd128_16 = -1;                 /* T_tdd128_16 */
static int hf_rrc_mac_hsResetIndicator_06 = -1;   /* T_mac_hsResetIndicator_06 */
static int hf_rrc_postVerificationPeriod_07 = -1;  /* T_postVerificationPeriod_07 */
static int hf_rrc_dl_dpchInfoCommon_01 = -1;      /* T_dl_dpchInfoCommon_01 */
static int hf_rrc_modeSpecificInfo_32 = -1;       /* T_modeSpecificInfo_32 */
static int hf_rrc_fdd_63 = -1;                    /* T_fdd_62 */
static int hf_rrc_tdd_25 = -1;                    /* T_tdd_24 */
static int hf_rrc_tddOption_08 = -1;              /* T_tddOption_08 */
static int hf_rrc_tdd128_17 = -1;                 /* T_tdd128_17 */
static int hf_rrc_mac_hsResetIndicator_07 = -1;   /* T_mac_hsResetIndicator_07 */
static int hf_rrc_postVerificationPeriod_08 = -1;  /* T_postVerificationPeriod_08 */
static int hf_rrc_dl_dpchInfoCommon_02 = -1;      /* T_dl_dpchInfoCommon_02 */
static int hf_rrc_modeSpecificInfo_33 = -1;       /* T_modeSpecificInfo_33 */
static int hf_rrc_fdd_64 = -1;                    /* T_fdd_63 */
static int hf_rrc_dpch_CompressedModeInfo_01 = -1;  /* DPCH_CompressedModeInfo_r8 */
static int hf_rrc_tdd_26 = -1;                    /* T_tdd_25 */
static int hf_rrc_tddOption_09 = -1;              /* T_tddOption_09 */
static int hf_rrc_tdd128_18 = -1;                 /* T_tdd128_18 */
static int hf_rrc_mac_hsResetIndicator_08 = -1;   /* T_mac_hsResetIndicator_08 */
static int hf_rrc_postVerificationPeriod_09 = -1;  /* T_postVerificationPeriod_09 */
static int hf_rrc_dl_dpchInfoCommon_03 = -1;      /* T_dl_dpchInfoCommon_03 */
static int hf_rrc_modeSpecificInfo_34 = -1;       /* T_modeSpecificInfo_34 */
static int hf_rrc_fdd_65 = -1;                    /* T_fdd_64 */
static int hf_rrc_dpch_CompressedModeInfo_02 = -1;  /* DPCH_CompressedModeInfo_r10 */
static int hf_rrc_tdd_27 = -1;                    /* T_tdd_26 */
static int hf_rrc_tddOption_10 = -1;              /* T_tddOption_10 */
static int hf_rrc_tdd128_19 = -1;                 /* T_tdd128_19 */
static int hf_rrc_mac_hsResetIndicator_09 = -1;   /* T_mac_hsResetIndicator_09 */
static int hf_rrc_postVerificationPeriod_10 = -1;  /* T_postVerificationPeriod_10 */
static int hf_rrc_dl_dpchInfoCommon_04 = -1;      /* T_dl_dpchInfoCommon_04 */
static int hf_rrc_modeSpecificInfo_35 = -1;       /* T_modeSpecificInfo_35 */
static int hf_rrc_fdd_66 = -1;                    /* T_fdd_65 */
static int hf_rrc_tdd_28 = -1;                    /* T_tdd_27 */
static int hf_rrc_tddOption_11 = -1;              /* T_tddOption_11 */
static int hf_rrc_tdd128_20 = -1;                 /* T_tdd128_20 */
static int hf_rrc_mac_hsResetIndicator_10 = -1;   /* T_mac_hsResetIndicator_10 */
static int hf_rrc_postVerificationPeriod_11 = -1;  /* T_postVerificationPeriod_11 */
static int hf_rrc_mac_hsResetIndicator_assisting_01 = -1;  /* T_mac_hsResetIndicator_assisting_01 */
static int hf_rrc_dl_DPCH_InfoCommon_03 = -1;     /* DL_DPCH_InfoCommonPost */
static int hf_rrc_dl_DPCH_InfoCommon_04 = -1;     /* DL_DPCH_InfoCommonPredef */
static int hf_rrc_cfnHandling = -1;               /* T_cfnHandling */
static int hf_rrc_maintain = -1;                  /* NULL */
static int hf_rrc_initialise = -1;                /* T_initialise */
static int hf_rrc_dummy_40 = -1;                  /* Cfntargetsfnframeoffset */
static int hf_rrc_modeSpecificInfo_36 = -1;       /* T_modeSpecificInfo_36 */
static int hf_rrc_fdd_67 = -1;                    /* T_fdd_66 */
static int hf_rrc_dl_DPCH_PowerControlInfo = -1;  /* DL_DPCH_PowerControlInfo */
static int hf_rrc_powerOffsetPilot_pdpdch = -1;   /* PowerOffsetPilot_pdpdch */
static int hf_rrc_dl_rate_matching_restriction = -1;  /* Dl_rate_matching_restriction */
static int hf_rrc_spreadingFactorAndPilot = -1;   /* SF512_AndPilot */
static int hf_rrc_positionFixedOrFlexible = -1;   /* PositionFixedOrFlexible */
static int hf_rrc_tfci_Existence = -1;            /* BOOLEAN */
static int hf_rrc_tdd_29 = -1;                    /* T_tdd_28 */
static int hf_rrc_cfnHandling_01 = -1;            /* T_cfnHandling_01 */
static int hf_rrc_initialise_01 = -1;             /* T_initialise_01 */
static int hf_rrc_modeSpecificInfo_37 = -1;       /* T_modeSpecificInfo_37 */
static int hf_rrc_fdd_68 = -1;                    /* T_fdd_67 */
static int hf_rrc_tdd_30 = -1;                    /* T_tdd_29 */
static int hf_rrc_mac_d_HFN_initial_value = -1;   /* MAC_d_HFN_initial_value */
static int hf_rrc_cfnHandling_02 = -1;            /* T_cfnHandling_02 */
static int hf_rrc_maintain_01 = -1;               /* T_maintain */
static int hf_rrc_timingmaintainedsynchind = -1;  /* TimingMaintainedSynchInd */
static int hf_rrc_initialise_02 = -1;             /* NULL */
static int hf_rrc_modeSpecificInfo_38 = -1;       /* T_modeSpecificInfo_38 */
static int hf_rrc_fdd_69 = -1;                    /* T_fdd_68 */
static int hf_rrc_tdd_31 = -1;                    /* T_tdd_30 */
static int hf_rrc_modeSpecificInfo_39 = -1;       /* T_modeSpecificInfo_39 */
static int hf_rrc_fdd_70 = -1;                    /* T_fdd_69 */
static int hf_rrc_tdd_32 = -1;                    /* T_tdd_31 */
static int hf_rrc_fdd_71 = -1;                    /* T_fdd_70 */
static int hf_rrc_pCPICH_UsageForChannelEst = -1;  /* PCPICH_UsageForChannelEst */
static int hf_rrc_dpch_FrameOffset = -1;          /* DPCH_FrameOffset */
static int hf_rrc_secondaryCPICH_Info = -1;       /* SecondaryCPICH_Info */
static int hf_rrc_dl_ChannelisationCodeList = -1;  /* DL_ChannelisationCodeList */
static int hf_rrc_tpc_CombinationIndex = -1;      /* TPC_CombinationIndex */
static int hf_rrc_dummy_41 = -1;                  /* SSDT_CellIdentity */
static int hf_rrc_closedLoopTimingAdjMode = -1;   /* ClosedLoopTimingAdjMode */
static int hf_rrc_tdd_33 = -1;                    /* T_tdd_32 */
static int hf_rrc_dl_CCTrChListToEstablish = -1;  /* DL_CCTrChList */
static int hf_rrc_dl_CCTrChListToRemove = -1;     /* DL_CCTrChListToRemove */
static int hf_rrc_fdd_72 = -1;                    /* T_fdd_71 */
static int hf_rrc_tdd_34 = -1;                    /* T_tdd_33 */
static int hf_rrc_dl_CCTrChListToEstablish_01 = -1;  /* DL_CCTrChList_r4 */
static int hf_rrc_fdd_73 = -1;                    /* T_fdd_72 */
static int hf_rrc_powerOffsetTPC_pdpdch = -1;     /* PowerOffsetTPC_pdpdch */
static int hf_rrc_tdd_35 = -1;                    /* T_tdd_34 */
static int hf_rrc_fdd_74 = -1;                    /* T_fdd_73 */
static int hf_rrc_tdd_36 = -1;                    /* T_tdd_35 */
static int hf_rrc_fdd_75 = -1;                    /* T_fdd_74 */
static int hf_rrc_tdd_37 = -1;                    /* T_tdd_36 */
static int hf_rrc_dl_CCTrChListToEstablish_02 = -1;  /* DL_CCTrChList_r7 */
static int hf_rrc_fdpch_FrameOffset = -1;         /* DPCH_FrameOffset */
static int hf_rrc_dl_ChannelisationCode = -1;     /* INTEGER_0_255 */
static int hf_rrc_fdpch_SlotFormat = -1;          /* FDPCH_SlotFormat */
static int hf_rrc_sttdIndication = -1;            /* STTDIndication */
static int hf_rrc_dl_ChannelisationCode_01 = -1;  /* DL_ChannelisationCode */
static int hf_rrc_dl_DPCH_TimeslotsCodes = -1;    /* DownlinkTimeslotsCodes */
static int hf_rrc_modeSpecificInfo_40 = -1;       /* T_modeSpecificInfo_40 */
static int hf_rrc_fdd_76 = -1;                    /* T_fdd_75 */
static int hf_rrc_tdd_38 = -1;                    /* T_tdd_37 */
static int hf_rrc_tpc_StepSizeTDD = -1;           /* TPC_StepSizeTDD */
static int hf_rrc_cfnHandling_03 = -1;            /* T_cfnHandling_03 */
static int hf_rrc_maintain_02 = -1;               /* T_maintain_01 */
static int hf_rrc_dl_FDPCH_PowerControlInfo = -1;  /* DL_DPCH_PowerControlInfo */
static int hf_rrc_dl_FDPCH_TPCcommandErrorRate_01 = -1;  /* INTEGER_1_16 */
static int hf_rrc_hs_scch_Info = -1;              /* HS_SCCH_Info */
static int hf_rrc_measurement_feedback_Info = -1;  /* Measurement_Feedback_Info */
static int hf_rrc_modeSpecificInfo_41 = -1;       /* T_modeSpecificInfo_41 */
static int hf_rrc_tdd_39 = -1;                    /* T_tdd_38 */
static int hf_rrc_tdd384_14 = -1;                 /* T_tdd384_13 */
static int hf_rrc_dl_HSPDSCH_TS_Configuration = -1;  /* DL_HSPDSCH_TS_Configuration */
static int hf_rrc_tdd128_21 = -1;                 /* T_tdd128_21 */
static int hf_rrc_hs_PDSCH_Midamble_Configuration_tdd128 = -1;  /* HS_PDSCH_Midamble_Configuration_TDD128 */
static int hf_rrc_hs_scch_Info_01 = -1;           /* HS_SCCH_Info_r6 */
static int hf_rrc_modeSpecificInfo_42 = -1;       /* T_modeSpecificInfo_42 */
static int hf_rrc_tdd_40 = -1;                    /* T_tdd_39 */
static int hf_rrc_tdd384_15 = -1;                 /* T_tdd384_14 */
static int hf_rrc_tdd128_22 = -1;                 /* T_tdd128_22 */
static int hf_rrc_hs_scch_Info_02 = -1;           /* HS_SCCH_Info_r7 */
static int hf_rrc_measurement_feedback_Info_01 = -1;  /* Measurement_Feedback_Info_r7 */
static int hf_rrc_modeSpecificInfo_43 = -1;       /* T_modeSpecificInfo_43 */
static int hf_rrc_tdd_41 = -1;                    /* T_tdd_40 */
static int hf_rrc_tdd384_16 = -1;                 /* T_tdd384_15 */
static int hf_rrc_tdd768_08 = -1;                 /* T_tdd768_07 */
static int hf_rrc_dl_HSPDSCH_TS_Configuration_01 = -1;  /* DL_HSPDSCH_TS_Configuration_VHCR */
static int hf_rrc_tdd128_23 = -1;                 /* T_tdd128_23 */
static int hf_rrc_dl_MultiCarrier_Information = -1;  /* DL_MultiCarrier_Information */
static int hf_rrc_fdd_77 = -1;                    /* T_fdd_76 */
static int hf_rrc_dl_64QAM_Configured = -1;       /* T_dl_64QAM_Configured */
static int hf_rrc_modeSpecificInfo_44 = -1;       /* T_modeSpecificInfo_44 */
static int hf_rrc_tdd_42 = -1;                    /* T_tdd_41 */
static int hf_rrc_tdd384_17 = -1;                 /* T_tdd384_16 */
static int hf_rrc_tdd768_09 = -1;                 /* T_tdd768_08 */
static int hf_rrc_tdd128_24 = -1;                 /* T_tdd128_24 */
static int hf_rrc_fdd_78 = -1;                    /* T_fdd_77 */
static int hf_rrc_dl_64QAM_Configured_01 = -1;    /* T_dl_64QAM_Configured_01 */
static int hf_rrc_modeSpecificInfo_45 = -1;       /* T_modeSpecificInfo_45 */
static int hf_rrc_tdd_43 = -1;                    /* T_tdd_42 */
static int hf_rrc_tdd128_25 = -1;                 /* T_tdd128_25 */
static int hf_rrc_outofSyncWindow = -1;           /* OutofSyncWindow */
static int hf_rrc_hs_scch_Info_03 = -1;           /* HS_SCCH_Info_r8_ext */
static int hf_rrc_hs_scch_Info_04 = -1;           /* HS_SCCH_Info_r9 */
static int hf_rrc_modeSpecificInfo_46 = -1;       /* T_modeSpecificInfo_46 */
static int hf_rrc_tdd_44 = -1;                    /* T_tdd_43 */
static int hf_rrc_tdd384_18 = -1;                 /* T_tdd384_17 */
static int hf_rrc_tdd768_10 = -1;                 /* T_tdd768_09 */
static int hf_rrc_tdd128_26 = -1;                 /* T_tdd128_26 */
static int hf_rrc_tS0_Indicator = -1;             /* T_tS0_Indicator */
static int hf_rrc_fdd_79 = -1;                    /* T_fdd_78 */
static int hf_rrc_dl_64QAM_Configured_02 = -1;    /* T_dl_64QAM_Configured_02 */
static int hf_rrc_measurement_feedback_Info_02 = -1;  /* Measurement_Feedback_Info_r11 */
static int hf_rrc_modeSpecificInfo_47 = -1;       /* T_modeSpecificInfo_47 */
static int hf_rrc_tdd_45 = -1;                    /* T_tdd_44 */
static int hf_rrc_tdd384_19 = -1;                 /* T_tdd384_18 */
static int hf_rrc_tdd768_11 = -1;                 /* T_tdd768_10 */
static int hf_rrc_tdd128_27 = -1;                 /* T_tdd128_27 */
static int hf_rrc_tS0_Indicator_01 = -1;          /* T_tS0_Indicator_01 */
static int hf_rrc_fdd_80 = -1;                    /* T_fdd_79 */
static int hf_rrc_dl_64QAM_Configured_03 = -1;    /* T_dl_64QAM_Configured_03 */
static int hf_rrc_DL_HSPDSCH_MultiCarrier_Information_item = -1;  /* DL_HSPDSCH_MultiCarrier_Information_item */
static int hf_rrc_uarfcn_Carrier = -1;            /* UARFCN */
static int hf_rrc_hs_PDSCH_Midamble_Configuration = -1;  /* HS_PDSCH_Midamble_Configuration_TDD128 */
static int hf_rrc_hs_SCCH_TDD128_MultiCarrier = -1;  /* SEQUENCE_SIZE_1_maxHSSCCHs_OF_HS_SCCH_TDD128_MultiCarrier */
static int hf_rrc_hs_SCCH_TDD128_MultiCarrier_item = -1;  /* HS_SCCH_TDD128_MultiCarrier */
static int hf_rrc_DL_HSPDSCH_TS_Configuration_item = -1;  /* DL_HSPDSCH_TS_Configuration_item */
static int hf_rrc_midambleShiftAndBurstType = -1;  /* MidambleShiftAndBurstType_DL */
static int hf_rrc_DL_HSPDSCH_TS_Configuration_VHCR_item = -1;  /* DL_HSPDSCH_TS_Configuration_VHCR_item */
static int hf_rrc_midambleShiftAndBurstType_01 = -1;  /* MidambleShiftAndBurstType_DL_VHCR */
static int hf_rrc_modeSpecificInfo_48 = -1;       /* T_modeSpecificInfo_48 */
static int hf_rrc_fdd_81 = -1;                    /* T_fdd_80 */
static int hf_rrc_dummy1_02 = -1;                 /* PDSCH_SHO_DCH_Info */
static int hf_rrc_dummy2_07 = -1;                 /* PDSCH_CodeMapping */
static int hf_rrc_tdd_46 = -1;                    /* PrimaryCCPCH_Info */
static int hf_rrc_dl_DPCH_InfoPerRL = -1;         /* DL_DPCH_InfoPerRL */
static int hf_rrc_dummy_42 = -1;                  /* SCCPCH_InfoForFACH */
static int hf_rrc_modeSpecificInfo_49 = -1;       /* T_modeSpecificInfo_49 */
static int hf_rrc_fdd_82 = -1;                    /* T_fdd_81 */
static int hf_rrc_tdd_47 = -1;                    /* PrimaryCCPCH_Info_r4 */
static int hf_rrc_dl_DPCH_InfoPerRL_01 = -1;      /* DL_DPCH_InfoPerRL_r4 */
static int hf_rrc_dummy_43 = -1;                  /* SCCPCH_InfoForFACH_r4 */
static int hf_rrc_modeSpecificInfo_50 = -1;       /* T_modeSpecificInfo_50 */
static int hf_rrc_fdd_83 = -1;                    /* T_fdd_82 */
static int hf_rrc_servingHSDSCH_RL_indicator = -1;  /* BOOLEAN */
static int hf_rrc_dl_DPCH_InfoPerRL_02 = -1;      /* DL_DPCH_InfoPerRL_r5 */
static int hf_rrc_modeSpecificInfo_51 = -1;       /* T_modeSpecificInfo_51 */
static int hf_rrc_fdd_84 = -1;                    /* T_fdd_83 */
static int hf_rrc_modeSpecificInfo_52 = -1;       /* T_modeSpecificInfo_52 */
static int hf_rrc_fdd_85 = -1;                    /* T_fdd_84 */
static int hf_rrc_servingEDCH_RL_indicator = -1;  /* BOOLEAN */
static int hf_rrc_dl_dpchInfo = -1;               /* T_dl_dpchInfo */
static int hf_rrc_dl_DPCH_InfoPerRL_03 = -1;      /* DL_DPCH_InfoPerRL_r6 */
static int hf_rrc_dl_FDPCH_InfoPerRL = -1;        /* DL_FDPCH_InfoPerRL_r6 */
static int hf_rrc_e_HICH_Info = -1;               /* T_e_HICH_Info */
static int hf_rrc_e_RGCH_Info_01 = -1;            /* T_e_RGCH_Info_01 */
static int hf_rrc_modeSpecificInfo_53 = -1;       /* T_modeSpecificInfo_53 */
static int hf_rrc_fdd_86 = -1;                    /* T_fdd_85 */
static int hf_rrc_dl_dpchInfo_01 = -1;            /* T_dl_dpchInfo_01 */
static int hf_rrc_dl_DPCH_InfoPerRL_04 = -1;      /* DL_DPCH_InfoPerRL_r7 */
static int hf_rrc_dl_FDPCH_InfoPerRL_01 = -1;     /* DL_FDPCH_InfoPerRL_r7 */
static int hf_rrc_e_AGCH_Information_01 = -1;     /* E_AGCH_Information_r7 */
static int hf_rrc_modeSpecificInfo2 = -1;         /* T_modeSpecificInfo2 */
static int hf_rrc_fdd_87 = -1;                    /* T_fdd_86 */
static int hf_rrc_e_HICH_Info_01 = -1;            /* T_e_HICH_Info_01 */
static int hf_rrc_e_RGCH_Info_02 = -1;            /* T_e_RGCH_Info_02 */
static int hf_rrc_tdd_48 = -1;                    /* T_tdd_45 */
static int hf_rrc_tdd384_tdd768 = -1;             /* T_tdd384_tdd768 */
static int hf_rrc_e_HICH_Info_02 = -1;            /* E_HICH_Information_TDD384_768 */
static int hf_rrc_tdd128_28 = -1;                 /* T_tdd128_28 */
static int hf_rrc_e_HICH_Info_03 = -1;            /* E_HICH_Information_TDD128 */
static int hf_rrc_modeSpecificInfo_54 = -1;       /* T_modeSpecificInfo_54 */
static int hf_rrc_fdd_88 = -1;                    /* T_fdd_87 */
static int hf_rrc_dl_dpchInfo_02 = -1;            /* T_dl_dpchInfo_02 */
static int hf_rrc_e_AGCH_Information_02 = -1;     /* E_AGCH_Information_r8 */
static int hf_rrc_modeSpecificInfo2_01 = -1;      /* T_modeSpecificInfo2_01 */
static int hf_rrc_fdd_89 = -1;                    /* T_fdd_88 */
static int hf_rrc_e_HICH_Info_04 = -1;            /* T_e_HICH_Info_02 */
static int hf_rrc_e_RGCH_Info_03 = -1;            /* T_e_RGCH_Info_03 */
static int hf_rrc_tdd_49 = -1;                    /* T_tdd_46 */
static int hf_rrc_tdd384_tdd768_01 = -1;          /* T_tdd384_tdd768_01 */
static int hf_rrc_tdd128_29 = -1;                 /* T_tdd128_29 */
static int hf_rrc_modeSpecificInfo_55 = -1;       /* T_modeSpecificInfo_55 */
static int hf_rrc_fdd_90 = -1;                    /* T_fdd_89 */
static int hf_rrc_dl_dpchInfo_03 = -1;            /* T_dl_dpchInfo_03 */
static int hf_rrc_e_ROCH_Information = -1;        /* E_ROCH_Information */
static int hf_rrc_modeSpecificInfo2_02 = -1;      /* T_modeSpecificInfo2_02 */
static int hf_rrc_fdd_91 = -1;                    /* T_fdd_90 */
static int hf_rrc_e_HICH_Info_05 = -1;            /* T_e_HICH_Info_03 */
static int hf_rrc_e_HICH_Information_01 = -1;     /* E_HICH_Information_r11 */
static int hf_rrc_secondaryReleaseIndicator = -1;  /* NULL */
static int hf_rrc_e_RGCH_Info_04 = -1;            /* T_e_RGCH_Info_04 */
static int hf_rrc_f_TPICH_Info = -1;              /* T_f_TPICH_Info */
static int hf_rrc_f_TPICH_Information = -1;       /* F_TPICH_Information */
static int hf_rrc_tdd_50 = -1;                    /* T_tdd_47 */
static int hf_rrc_tdd384_tdd768_02 = -1;          /* T_tdd384_tdd768_02 */
static int hf_rrc_tdd128_30 = -1;                 /* T_tdd128_30 */
static int hf_rrc_DL_InformationPerRL_List_item = -1;  /* DL_InformationPerRL */
static int hf_rrc_DL_InformationPerRL_List_r4_item = -1;  /* DL_InformationPerRL_r4 */
static int hf_rrc_DL_InformationPerRL_List_r5_item = -1;  /* DL_InformationPerRL_r5 */
static int hf_rrc_DL_InformationPerRL_List_r6_item = -1;  /* DL_InformationPerRL_r6 */
static int hf_rrc_DL_InformationPerRL_List_v6b0ext_item = -1;  /* DL_InformationPerRL_v6b0ext */
static int hf_rrc_DL_InformationPerRL_List_r5bis_item = -1;  /* DL_InformationPerRL_r5bis */
static int hf_rrc_DL_InformationPerRL_List_r7_item = -1;  /* DL_InformationPerRL_r7 */
static int hf_rrc_DL_InformationPerRL_List_r8_item = -1;  /* DL_InformationPerRL_r8 */
static int hf_rrc_DL_InformationPerRL_List_r11_item = -1;  /* DL_InformationPerRL_r11 */
static int hf_rrc_DL_InformationPerRL_ListPostFDD_item = -1;  /* DL_InformationPerRL_PostFDD */
static int hf_rrc_dl_DPCH_InfoPerRL_05 = -1;      /* DL_DPCH_InfoPerRL_PostFDD */
static int hf_rrc_primaryCCPCH_Info = -1;         /* PrimaryCCPCH_InfoPost */
static int hf_rrc_dl_DPCH_InfoPerRL_06 = -1;      /* DL_DPCH_InfoPerRL_PostTDD */
static int hf_rrc_primaryCCPCH_Info_01 = -1;      /* PrimaryCCPCH_InfoPostTDD_LCR_r4 */
static int hf_rrc_dl_DPCH_InfoPerRL_07 = -1;      /* DL_DPCH_InfoPerRL_PostTDD_LCR_r4 */
static int hf_rrc_tsn_Length = -1;                /* T_tsn_Length */
static int hf_rrc_multiCarrierNumber = -1;        /* INTEGER_1_maxTDD128Carrier */
static int hf_rrc_dl_HSPDSCH_MultiCarrier_Information = -1;  /* DL_HSPDSCH_MultiCarrier_Information */
static int hf_rrc_restrictedTrCH_InfoList = -1;   /* RestrictedTrCH_InfoList */
static int hf_rrc_newConfiguration_02 = -1;       /* T_newConfiguration_02 */
static int hf_rrc_dl_64QAM_Configured_04 = -1;    /* T_dl_64QAM_Configured_04 */
static int hf_rrc_dl_ScramblingCode = -1;         /* SecondaryScramblingCode */
static int hf_rrc_hS_SCCHChannelisationCodeInfo = -1;  /* SEQUENCE_SIZE_1_maxHSSCCHs_OF_HS_SCCH_Codes */
static int hf_rrc_hS_SCCHChannelisationCodeInfo_item = -1;  /* HS_SCCH_Codes */
static int hf_rrc_measurementPowerOffset = -1;    /* MeasurementPowerOffset */
static int hf_rrc_uarfcn_DL = -1;                 /* UARFCN */
static int hf_rrc_differentTxModeFromServingHS_DSCHCell = -1;  /* T_differentTxModeFromServingHS_DSCHCell */
static int hf_rrc_configurationInfo_01 = -1;      /* T_configurationInfo_01 */
static int hf_rrc_newConfiguration_03 = -1;       /* T_newConfiguration_03 */
static int hf_rrc_dl_64QAM_Configured_05 = -1;    /* T_dl_64QAM_Configured_05 */
static int hf_rrc_differentTxModeFromServingHS_DSCHCell_01 = -1;  /* T_differentTxModeFromServingHS_DSCHCell_01 */
static int hf_rrc_secondaryCellMIMOparameters = -1;  /* SecondaryCellMIMOparametersFDD */
static int hf_rrc_configurationInfo_02 = -1;      /* T_configurationInfo_02 */
static int hf_rrc_newConfiguration_04 = -1;       /* T_newConfiguration_04 */
static int hf_rrc_dl_64QAM_Configured_06 = -1;    /* T_dl_64QAM_Configured_06 */
static int hf_rrc_differentTxModeFromServingHS_DSCHCell_02 = -1;  /* T_differentTxModeFromServingHS_DSCHCell_02 */
static int hf_rrc_secondaryCellMIMOparameters_01 = -1;  /* SecondaryCellMIMOparametersFDD_r10 */
static int hf_rrc_configurationInfo_03 = -1;      /* T_configurationInfo_03 */
static int hf_rrc_newConfiguration_05 = -1;       /* T_newConfiguration_05 */
static int hf_rrc_dl_64QAM_Configured_07 = -1;    /* T_dl_64QAM_Configured_07 */
static int hf_rrc_measurementPowerOffset1 = -1;   /* MeasurementPowerOffset */
static int hf_rrc_differentTxModeFromServingHS_DSCHCell_03 = -1;  /* T_differentTxModeFromServingHS_DSCHCell_03 */
static int hf_rrc_multiflowConfigurationInfo = -1;  /* MultiflowConfigurationInfo */
static int hf_rrc_secondaryCell4x4MIMOparameters = -1;  /* SecondaryCell4x4MIMOparametersFDD */
static int hf_rrc_DL_TPC_PowerOffsetPerRL_List_item = -1;  /* DL_TPC_PowerOffsetPerRL */
static int hf_rrc_codesRepresentation = -1;       /* T_codesRepresentation */
static int hf_rrc_consecutive = -1;               /* T_consecutive */
static int hf_rrc_firstChannelisationCode = -1;   /* DL_TS_ChannelisationCode */
static int hf_rrc_lastChannelisationCode = -1;    /* DL_TS_ChannelisationCode */
static int hf_rrc_bitmap = -1;                    /* T_bitmap */
static int hf_rrc_codesRepresentation_01 = -1;    /* T_codesRepresentation_01 */
static int hf_rrc_consecutive_01 = -1;            /* T_consecutive_01 */
static int hf_rrc_firstChannelisationCode_01 = -1;  /* DL_TS_ChannelisationCode_VHCR */
static int hf_rrc_lastChannelisationCode_01 = -1;  /* DL_TS_ChannelisationCode_VHCR */
static int hf_rrc_bitmap_01 = -1;                 /* T_bitmap_01 */
static int hf_rrc_firstChannelisationCode_02 = -1;  /* INTEGER_1_15 */
static int hf_rrc_lastChannelisationCode_02 = -1;  /* INTEGER_1_15 */
static int hf_rrc_parameters = -1;                /* T_parameters */
static int hf_rrc_sameAsLast = -1;                /* T_sameAsLast */
static int hf_rrc_timeslotNumber = -1;            /* TimeslotNumber */
static int hf_rrc_newParameters = -1;             /* T_newParameters */
static int hf_rrc_individualTimeslotInfo = -1;    /* IndividualTimeslotInfo */
static int hf_rrc_dl_TS_ChannelisationCodesShort = -1;  /* DL_TS_ChannelisationCodesShort */
static int hf_rrc_parameters_01 = -1;             /* T_parameters_01 */
static int hf_rrc_sameAsLast_01 = -1;             /* T_sameAsLast_01 */
static int hf_rrc_newParameters_01 = -1;          /* T_newParameters_01 */
static int hf_rrc_individualTimeslotInfo_01 = -1;  /* IndividualTimeslotInfo_VHCR */
static int hf_rrc_dl_TS_ChannelisationCodesShort_01 = -1;  /* DL_TS_ChannelisationCodesShort_VHCR */
static int hf_rrc_parameters_02 = -1;             /* T_parameters_02 */
static int hf_rrc_sameAsLast_02 = -1;             /* T_sameAsLast_02 */
static int hf_rrc_timeslotNumber_01 = -1;         /* TimeslotNumber_LCR_r4 */
static int hf_rrc_newParameters_02 = -1;          /* T_newParameters_02 */
static int hf_rrc_individualTimeslotInfo_02 = -1;  /* IndividualTimeslotInfo_LCR_r4 */
static int hf_rrc_parameters_03 = -1;             /* T_parameters_03 */
static int hf_rrc_sameAsLast_03 = -1;             /* T_sameAsLast_03 */
static int hf_rrc_newParameters_03 = -1;          /* T_newParameters_03 */
static int hf_rrc_individualTimeslotInfo_03 = -1;  /* IndividualTimeslotInfo_r7 */
static int hf_rrc_firstIndividualTimeslotInfo = -1;  /* IndividualTimeslotInfo */
static int hf_rrc_moreTimeslots = -1;             /* T_moreTimeslots */
static int hf_rrc_noMore = -1;                    /* NULL */
static int hf_rrc_additionalTimeslots = -1;       /* T_additionalTimeslots */
static int hf_rrc_consecutive_02 = -1;            /* INTEGER_1_maxTS_1 */
static int hf_rrc_timeslotList = -1;              /* SEQUENCE_SIZE_1_maxTS_1_OF_DownlinkAdditionalTimeslots */
static int hf_rrc_timeslotList_item = -1;         /* DownlinkAdditionalTimeslots */
static int hf_rrc_firstIndividualTimeslotInfo_01 = -1;  /* IndividualTimeslotInfo_VHCR */
static int hf_rrc_moreTimeslots_01 = -1;          /* T_moreTimeslots_01 */
static int hf_rrc_additionalTimeslots_01 = -1;    /* T_additionalTimeslots_01 */
static int hf_rrc_timeslotList_01 = -1;           /* SEQUENCE_SIZE_1_maxTS_1_OF_DownlinkAdditionalTimeslots_VHCR */
static int hf_rrc_timeslotList_item_01 = -1;      /* DownlinkAdditionalTimeslots_VHCR */
static int hf_rrc_firstIndividualTimeslotInfo_02 = -1;  /* IndividualTimeslotInfo_LCR_r4 */
static int hf_rrc_moreTimeslots_02 = -1;          /* T_moreTimeslots_02 */
static int hf_rrc_additionalTimeslots_02 = -1;    /* T_additionalTimeslots_02 */
static int hf_rrc_consecutive_03 = -1;            /* INTEGER_1_maxTS_LCR_1 */
static int hf_rrc_timeslotList_02 = -1;           /* SEQUENCE_SIZE_1_maxTS_LCR_1_OF_DownlinkAdditionalTimeslots_LCR_r4 */
static int hf_rrc_timeslotList_item_02 = -1;      /* DownlinkAdditionalTimeslots_LCR_r4 */
static int hf_rrc_firstIndividualTimeslotInfo_03 = -1;  /* IndividualTimeslotInfo_r7 */
static int hf_rrc_moreTimeslots_03 = -1;          /* T_moreTimeslots_03 */
static int hf_rrc_additionalTimeslots_03 = -1;    /* T_additionalTimeslots_03 */
static int hf_rrc_timeslotList_03 = -1;           /* SEQUENCE_SIZE_1_maxTS_1_OF_DownlinkAdditionalTimeslots_r7 */
static int hf_rrc_timeslotList_item_03 = -1;      /* DownlinkAdditionalTimeslots_r7 */
static int hf_rrc_tgp_SequenceList = -1;          /* TGP_SequenceList */
static int hf_rrc_tgp_SequenceList_01 = -1;       /* TGP_SequenceList_r8 */
static int hf_rrc_tgp_SequenceList_02 = -1;       /* TGP_SequenceList_r10 */
static int hf_rrc_tgps_Reconfiguration_CFN = -1;  /* TGPS_Reconfiguration_CFN */
static int hf_rrc_tgp_SequenceShortList = -1;     /* SEQUENCE_SIZE_1_maxTGPS_OF_TGP_SequenceShort */
static int hf_rrc_tgp_SequenceShortList_item = -1;  /* TGP_SequenceShort */
static int hf_rrc_tgp_SequenceShortList_01 = -1;  /* SEQUENCE_SIZE_1_maxTGPS_OF_TGP_SequenceShort_r10 */
static int hf_rrc_tgp_SequenceShortList_item_01 = -1;  /* TGP_SequenceShort_r10 */
static int hf_rrc_ue_drx_Cycle = -1;              /* UE_DRX_Cycle */
static int hf_rrc_ue_drx_Cycle_InactivityThreshold = -1;  /* UE_DRX_Cycle_InactivityThreshold */
static int hf_rrc_ue_GrantMonitoring_InactivityThreshold = -1;  /* UE_GrantMonitoring_InactivityThreshold */
static int hf_rrc_ue_drx_GrantMonitoring = -1;    /* BOOLEAN */
static int hf_rrc_maxTFCI_Field2Value = -1;       /* MaxTFCI_Field2Value */
static int hf_rrc_spreadingFactor = -1;           /* SF_PDSCH */
static int hf_rrc_codeNumber = -1;                /* CodeNumberDSCH */
static int hf_rrc_multiCodeInfo = -1;             /* MultiCodeInfo */
static int hf_rrc_DSCH_MappingList_item = -1;     /* DSCH_Mapping */
static int hf_rrc_DSCH_TransportChannelsInfo_item = -1;  /* DSCH_TransportChannelsInfo_item */
static int hf_rrc_dsch_transport_channel_identity = -1;  /* TransportChannelIdentity */
static int hf_rrc_dsch_TFS = -1;                  /* TransportFormatSet */
static int hf_rrc_dtx_Info = -1;                  /* DTX_Info */
static int hf_rrc_drx_Info = -1;                  /* DRX_Info */
static int hf_rrc_uplink_DPCCHSlotFormatInformation = -1;  /* Uplink_DPCCH_Slot_Format_Information */
static int hf_rrc_ue_dtx_Cycle1_10ms = -1;        /* UE_DTX_Cycle1_10ms */
static int hf_rrc_ue_dtx_Cycle2_10ms = -1;        /* UE_DTX_Cycle2_10ms */
static int hf_rrc_mac_dtx_Cycle_10ms = -1;        /* MAC_DTX_Cycle_10ms */
static int hf_rrc_ue_dtx_Cycle1_2ms = -1;         /* UE_DTX_Cycle1_2ms */
static int hf_rrc_ue_dtx_Cycle2_2ms = -1;         /* UE_DTX_Cycle2_2ms */
static int hf_rrc_mac_dtx_Cycle_2ms = -1;         /* MAC_DTX_Cycle_2ms */
static int hf_rrc_e_dch_TTI_Length = -1;          /* T_e_dch_TTI_Length */
static int hf_rrc_dtx_e_dch_TTI_10ms = -1;        /* DTX_E_DCH_TTI_10ms */
static int hf_rrc_dtx_e_dch_TTI_2ms = -1;         /* DTX_E_DCH_TTI_2ms */
static int hf_rrc_ue_dtx_cycle2InactivityThreshold = -1;  /* UE_DTX_Cycle2InactivityThreshold */
static int hf_rrc_ue_dtx_cycle2DefaultSG = -1;    /* INTEGER_0_38 */
static int hf_rrc_ue_dtx_long_preamble_length = -1;  /* UE_DTX_long_preamble_length */
static int hf_rrc_mac_InactivityThreshold = -1;   /* MAC_InactivityThreshold */
static int hf_rrc_cqi_dtx_Timer = -1;             /* CQI_DTX_Timer */
static int hf_rrc_ue_dpcch_Burst1 = -1;           /* UE_DPCCH_Burst */
static int hf_rrc_ue_dpcch_Burst2 = -1;           /* UE_DPCCH_Burst */
static int hf_rrc_DynamicPersistenceLevelList_item = -1;  /* DynamicPersistenceLevel */
static int hf_rrc_DynamicPersistenceLevelTF_List_item = -1;  /* DynamicPersistenceLevel */
static int hf_rrc_e_AGCH_DRX_InfoType = -1;       /* T_e_AGCH_DRX_InfoType */
static int hf_rrc_sameAsHS_SCCH = -1;             /* NULL */
static int hf_rrc_e_AGCH_DRX_Parameters = -1;     /* T_e_AGCH_DRX_Parameters */
static int hf_rrc_e_AGCH_DRX_Cycle = -1;          /* ControlChannelDRXCycle_TDD128 */
static int hf_rrc_e_AGCH_InactivityMonitorThreshold = -1;  /* E_AGCH_InactivityMonitorThreshold_TDD128 */
static int hf_rrc_e_AGCH_DRX_Offset = -1;         /* INTEGER_0_63 */
static int hf_rrc_tS_number = -1;                 /* INTEGER_0_14 */
static int hf_rrc_channelisation_code = -1;       /* DL_TS_ChannelisationCode */
static int hf_rrc_midambleShiftAndBurstType_02 = -1;  /* MidambleShiftAndBurstType_EDCH */
static int hf_rrc_channelisation_code_01 = -1;    /* DL_TS_ChannelisationCode_VHCR */
static int hf_rrc_firstChannelisationCode_03 = -1;  /* HS_ChannelisationCode_LCR */
static int hf_rrc_secondChannelisationCode = -1;  /* HS_ChannelisationCode_LCR */
static int hf_rrc_midambleAllocationMode = -1;    /* T_midambleAllocationMode */
static int hf_rrc_defaultMidamble = -1;           /* NULL */
static int hf_rrc_commonMidamble = -1;            /* NULL */
static int hf_rrc_ueSpecificMidamble = -1;        /* INTEGER_0_15 */
static int hf_rrc_midambleConfiguration = -1;     /* INTEGER_1_8 */
static int hf_rrc_e_AGCH_ChannelisationCode = -1;  /* E_AGCH_ChannelisationCode */
static int hf_rrc_modeSpecific_02 = -1;           /* T_modeSpecific_02 */
static int hf_rrc_fdd_92 = -1;                    /* T_fdd_91 */
static int hf_rrc_tdd_51 = -1;                    /* T_tdd_48 */
static int hf_rrc_tdd384_20 = -1;                 /* T_tdd384_19 */
static int hf_rrc_long_Term_Grant_Indicator = -1;  /* BOOLEAN */
static int hf_rrc_length_of_TTRI_field = -1;      /* INTEGER_1_12 */
static int hf_rrc_e_AGCH_Set_Config = -1;         /* E_AGCH_Set_Config */
static int hf_rrc_e_AGCH_BLER_Target = -1;        /* Bler_Target */
static int hf_rrc_tdd768_12 = -1;                 /* T_tdd768_11 */
static int hf_rrc_e_AGCH_Set_Config_01 = -1;      /* E_AGCH_Set_Config_VHCR */
static int hf_rrc_tdd128_31 = -1;                 /* T_tdd128_31 */
static int hf_rrc_rdi_Indicator = -1;             /* BOOLEAN */
static int hf_rrc_tpc_StepSize = -1;              /* TPC_StepSizeTDD */
static int hf_rrc_e_AGCH_Set_Config_02 = -1;      /* E_AGCH_Set_Config_LCR */
static int hf_rrc_modeSpecific_03 = -1;           /* T_modeSpecific_03 */
static int hf_rrc_fdd_93 = -1;                    /* T_fdd_92 */
static int hf_rrc_tdd_52 = -1;                    /* T_tdd_49 */
static int hf_rrc_tdd384_21 = -1;                 /* T_tdd384_20 */
static int hf_rrc_tdd768_13 = -1;                 /* T_tdd768_12 */
static int hf_rrc_tdd128_32 = -1;                 /* T_tdd128_32 */
static int hf_rrc_E_AGCH_Set_Config_item = -1;    /* E_AGCH_Individual */
static int hf_rrc_E_AGCH_Set_Config_VHCR_item = -1;  /* E_AGCH_Individual_VHCR */
static int hf_rrc_E_AGCH_Set_Config_LCR_item = -1;  /* E_AGCH_Individual_LCR */
static int hf_rrc_e_ROCH_ChannelisationCode = -1;  /* E_ROCH_ChannelisationCode */
static int hf_rrc_e_roch_e_rnti = -1;             /* E_RNTI */
static int hf_rrc_e_DCH_RL_InfoNewServingCell = -1;  /* E_DCH_RL_InfoNewServingCell */
static int hf_rrc_e_DCH_RL_InfoOtherCellList = -1;  /* SEQUENCE_SIZE_1_maxEDCHRL_OF_E_DCH_RL_InfoOtherCell */
static int hf_rrc_e_DCH_RL_InfoOtherCellList_item = -1;  /* E_DCH_RL_InfoOtherCell */
static int hf_rrc_e_DCH_RL_InfoNewServingCell_01 = -1;  /* E_DCH_RL_InfoNewServingCell_r7 */
static int hf_rrc_e_DCH_RL_InfoNewServingCell_02 = -1;  /* E_DCH_RL_InfoNewServingCell_r11 */
static int hf_rrc_e_DCH_RL_InfoOtherCellList_01 = -1;  /* SEQUENCE_SIZE_1_maxEDCHRL_OF_E_DCH_RL_InfoOtherCell_r11 */
static int hf_rrc_e_DCH_RL_InfoOtherCellList_item_01 = -1;  /* E_DCH_RL_InfoOtherCell_r11 */
static int hf_rrc_e_DCH_RL_InfoNewSecServingCell = -1;  /* E_DCH_RL_InfoNewSecServingCell */
static int hf_rrc_e_DCH_RL_InfoOtherCellList_SecULFreq = -1;  /* SEQUENCE_SIZE_1_maxEDCHRL_OF_E_DCH_RL_InfoOtherCell_SecULFreq */
static int hf_rrc_e_DCH_RL_InfoOtherCellList_SecULFreq_item = -1;  /* E_DCH_RL_InfoOtherCell_SecULFreq */
static int hf_rrc_servingGrant_01 = -1;           /* T_servingGrant_01 */
static int hf_rrc_value = -1;                     /* INTEGER_0_38 */
static int hf_rrc_primary_Secondary_GrantSelector_01 = -1;  /* T_primary_Secondary_GrantSelector_01 */
static int hf_rrc_reference_E_TFCIs = -1;         /* E_DPDCH_Reference_E_TFCIList */
static int hf_rrc_powerOffsetForSchedInfo = -1;   /* INTEGER_0_6 */
static int hf_rrc_threeIndexStepThreshold = -1;   /* INTEGER_0_37 */
static int hf_rrc_twoIndexStepThreshold = -1;     /* INTEGER_0_37 */
static int hf_rrc_e_RGCH_Info_05 = -1;            /* T_e_RGCH_Info_05 */
static int hf_rrc_servingGrant_02 = -1;           /* T_servingGrant_02 */
static int hf_rrc_primary_Secondary_GrantSelector_02 = -1;  /* T_primary_Secondary_GrantSelector_02 */
static int hf_rrc_reference_E_TFCIs_01 = -1;      /* E_DPDCH_Reference_E_TFCIList_r7 */
static int hf_rrc_e_RGCH_Info_06 = -1;            /* T_e_RGCH_Info_06 */
static int hf_rrc_servingGrant_03 = -1;           /* T_servingGrant_03 */
static int hf_rrc_primary_Secondary_GrantSelector_03 = -1;  /* T_primary_Secondary_GrantSelector_03 */
static int hf_rrc_e_RGCH_Info_07 = -1;            /* T_e_RGCH_Info_07 */
static int hf_rrc_e_RGCH_Info_08 = -1;            /* T_e_RGCH_Info_08 */
static int hf_rrc_e_HICH_Info_06 = -1;            /* T_e_HICH_Info_04 */
static int hf_rrc_e_RGCH_Info_09 = -1;            /* T_e_RGCH_Info_09 */
static int hf_rrc_e_HICH_Info_07 = -1;            /* T_e_HICH_Info_05 */
static int hf_rrc_e_RGCH_Info_10 = -1;            /* T_e_RGCH_Info_10 */
static int hf_rrc_e_RGCH_Info_11 = -1;            /* T_e_RGCH_Info_11 */
static int hf_rrc_e_dch_SPS_Operation = -1;       /* T_e_dch_SPS_Operation */
static int hf_rrc_newOperation_01 = -1;           /* E_DCH_SPS_NewOperation_TDD128 */
static int hf_rrc_e_hich_Info_01 = -1;            /* E_HICH_Information_For_SPS_TDD128 */
static int hf_rrc_e_dch_TxPattern = -1;           /* E_DCH_TxPatternList_TDD128 */
static int hf_rrc_initialSPSInfoForEDCH = -1;     /* T_initialSPSInfoForEDCH */
static int hf_rrc_n_E_UCCH = -1;                  /* INTEGER_1_8 */
static int hf_rrc_codeResourceInfo = -1;          /* UL_TS_ChannelisationCode */
static int hf_rrc_timeslotResourceRelatedInfo = -1;  /* BIT_STRING_SIZE_5 */
static int hf_rrc_powerResourceRelatedInfo = -1;  /* INTEGER_1_32 */
static int hf_rrc_subframeNum = -1;               /* INTEGER_0_1 */
static int hf_rrc_initialTxPatternIndex = -1;     /* INTEGER_0_maxEDCHTxPattern_TDD128_1 */
static int hf_rrc_E_DCH_TxPatternList_TDD128_item = -1;  /* E_DCH_TxPatternList_TDD128_item */
static int hf_rrc_repetitionPeriodAndLength_01 = -1;  /* RepetitionPeriodAndLengthForSPS */
static int hf_rrc_happyBit_DelayCondition = -1;   /* HappyBit_DelayCondition */
static int hf_rrc_e_TFCI_TableIndex = -1;         /* E_TFCI_TableIndex */
static int hf_rrc_e_DCH_MinimumSet_E_TFCI = -1;   /* E_DCH_MinimumSet_E_TFCI */
static int hf_rrc_maxChannelisationCodes = -1;    /* E_DPDCH_MaxChannelisationCodes */
static int hf_rrc_pl_NonMax = -1;                 /* E_DPDCH_PL_NonMax */
static int hf_rrc_schedulingInfoConfiguration = -1;  /* E_DPDCH_SchedulingInfoConfiguration */
static int hf_rrc_reference_E_TFCI = -1;          /* INTEGER_0_127 */
static int hf_rrc_reference_E_TFCI_PO = -1;       /* INTEGER_0_29 */
static int hf_rrc_reference_E_TFCI_PO_r7 = -1;    /* INTEGER_0_31 */
static int hf_rrc_E_DPDCH_Reference_E_TFCIList_item = -1;  /* E_DPDCH_Reference_E_TFCI */
static int hf_rrc_E_DPDCH_Reference_E_TFCIList_r7_item = -1;  /* E_DPDCH_Reference_E_TFCI_r7 */
static int hf_rrc_periodicityOfSchedInfo_NoGrant = -1;  /* E_DPDCH_PeriodicyOfSchedInfo */
static int hf_rrc_periodicityOfSchedInfo_Grant = -1;  /* E_DPDCH_PeriodicyOfSchedInfo */
static int hf_rrc_servingGrant_04 = -1;           /* T_servingGrant_04 */
static int hf_rrc_primary_Secondary_GrantSelector_04 = -1;  /* T_primary_Secondary_GrantSelector_04 */
static int hf_rrc_channelisationCode_02 = -1;     /* E_HICH_ChannelisationCode */
static int hf_rrc_signatureSequence_01 = -1;      /* E_HICH_RGCH_SignatureSequence */
static int hf_rrc_secondarySignatureSequence = -1;  /* E_HICH_RGCH_SignatureSequence */
static int hf_rrc_n_E_HICH = -1;                  /* INTEGER_4_44 */
static int hf_rrc_tS_Number = -1;                 /* INTEGER_0_14 */
static int hf_rrc_modeSpecificInfo_56 = -1;       /* T_modeSpecificInfo_56 */
static int hf_rrc_tdd384_22 = -1;                 /* DL_TS_ChannelisationCode */
static int hf_rrc_tdd768_14 = -1;                 /* DL_TS_ChannelisationCode_VHCR */
static int hf_rrc_burst_Type = -1;                /* T_burst_Type */
static int hf_rrc_midamble_Allocation_Mode = -1;  /* T_midamble_Allocation_Mode */
static int hf_rrc_ei = -1;                        /* INTEGER_0_3 */
static int hf_rrc_channelisation_Code = -1;       /* HS_ChannelisationCode_LCR */
static int hf_rrc_midambleAllocationMode_01 = -1;  /* T_midambleAllocationMode_01 */
static int hf_rrc_n_E_HICH_01 = -1;               /* INTEGER_4_15 */
static int hf_rrc_e_HICH_InfoList = -1;           /* E_HICH_Information_LCR_List */
static int hf_rrc_E_HICH_Information_LCR_List_item = -1;  /* E_HICH_Information_LCR */
static int hf_rrc_configurationmode = -1;         /* T_configurationmode */
static int hf_rrc_implicit_01 = -1;               /* T_implicit */
static int hf_rrc_explicit_03 = -1;               /* T_explicit_02 */
static int hf_rrc_midambleAllocationMode_02 = -1;  /* T_midambleAllocationMode_02 */
static int hf_rrc_signatureSequenceGroupIndex = -1;  /* INTEGER_0_19 */
static int hf_rrc_e_TFCS_Info = -1;               /* E_TFCS_Info */
static int hf_rrc_modeSpecificInfo_57 = -1;       /* T_modeSpecificInfo_57 */
static int hf_rrc_tdd348_tdd768 = -1;             /* T_tdd348_tdd768 */
static int hf_rrc_noSlotsForTFCIandTPC = -1;      /* INTEGER_1_12 */
static int hf_rrc_e_PUCH_ContantValue = -1;       /* INTEGER_M35_10 */
static int hf_rrc_e_PUCH_TS_ConfigurationList = -1;  /* SEQUENCE_SIZE_1_maxTS_2_OF_E_PUCH_TS_Slots */
static int hf_rrc_e_PUCH_TS_ConfigurationList_item = -1;  /* E_PUCH_TS_Slots */
static int hf_rrc_e_PUCH_CodeHopping = -1;        /* BOOLEAN */
static int hf_rrc_e_PUCH_TPC_Step_Size = -1;      /* INTEGER_1_3 */
static int hf_rrc_minimum_Allowed_Code_Rate = -1;  /* INTEGER_0_63 */
static int hf_rrc_maximum_Allowed_Code_Rate = -1;  /* INTEGER_0_63 */
static int hf_rrc_tdd128_33 = -1;                 /* T_tdd128_33 */
static int hf_rrc_snpl_ReportType = -1;           /* T_snpl_ReportType */
static int hf_rrc_prxdes_base = -1;               /* INTEGER_M112_M50 */
static int hf_rrc_beaconPLEst_01 = -1;            /* BOOLEAN */
static int hf_rrc_pebase_PowerControlGAP = -1;    /* PowerControlGAP */
static int hf_rrc_e_PUCH_TS_ConfigurationList_01 = -1;  /* SEQUENCE_SIZE_1_maxTS_LCR_1_OF_E_PUCH_TS_Slots_LCR */
static int hf_rrc_e_PUCH_TS_ConfigurationList_item_01 = -1;  /* E_PUCH_TS_Slots_LCR */
static int hf_rrc_maximumNumOfRetransSchedInfo = -1;  /* INTEGER_0_15 */
static int hf_rrc_retransTimerForSchedInfo = -1;  /* T_retransTimerForSchedInfo */
static int hf_rrc_snpl_ReportType_01 = -1;        /* T_snpl_ReportType_01 */
static int hf_rrc_pebase_PowerControlGAP_01 = -1;  /* INTEGER_1_255 */
static int hf_rrc_retransTimerForSchedInfo_01 = -1;  /* T_retransTimerForSchedInfo_01 */
static int hf_rrc_dummy_44 = -1;                  /* INTEGER_0_6 */
static int hf_rrc_ts_Number = -1;                 /* INTEGER_0_14 */
static int hf_rrc_timeslotNumber_02 = -1;         /* INTEGER_1_5 */
static int hf_rrc_midambleAllocationMode_03 = -1;  /* T_midambleAllocationMode_03 */
static int hf_rrc_rg_CombinationIndex = -1;       /* E_RGCH_CombinationIndex */
static int hf_rrc_modeSpecificInfo_58 = -1;       /* T_modeSpecificInfo_58 */
static int hf_rrc_tdd384_tdd768_03 = -1;          /* T_tdd384_tdd768_03 */
static int hf_rrc_e_RUCCH_ConstantValue = -1;     /* INTEGER_M35_10 */
static int hf_rrc_e_RUCCH_PersistenceScalingFactor = -1;  /* PersistenceScalingFactor */
static int hf_rrc_t_RUCCH = -1;                   /* T_t_RUCCH */
static int hf_rrc_e_RUCCH_TS_Number = -1;         /* INTEGER_0_14 */
static int hf_rrc_e_RUCCH_Midamble = -1;          /* T_e_RUCCH_Midamble */
static int hf_rrc_t_adv = -1;                     /* T_t_adv */
static int hf_rrc_t_SCHED = -1;                   /* T_t_SCHED */
static int hf_rrc_modeSpecificInfo_59 = -1;       /* T_modeSpecificInfo_59 */
static int hf_rrc_tdd384_23 = -1;                 /* T_tdd384_21 */
static int hf_rrc_sF16 = -1;                      /* SEQUENCE_SIZE_1_8_OF_SF16Codes */
static int hf_rrc_sF16_item = -1;                 /* SF16Codes */
static int hf_rrc_sF8 = -1;                       /* SEQUENCE_SIZE_1_8_OF_SF8Codes */
static int hf_rrc_sF8_item = -1;                  /* SF8Codes */
static int hf_rrc_tdd768_15 = -1;                 /* T_tdd768_13 */
static int hf_rrc_sF32 = -1;                      /* SEQUENCE_SIZE_1_16_OF_SF32Codes */
static int hf_rrc_sF32_item = -1;                 /* SF32Codes */
static int hf_rrc_sF816 = -1;                     /* SEQUENCE_SIZE_1_16_OF_SF16Codes2 */
static int hf_rrc_sF816_item = -1;                /* SF16Codes2 */
static int hf_rrc_tdd128_34 = -1;                 /* T_tdd128_34 */
static int hf_rrc_t_RUCCH_01 = -1;                /* T_t_RUCCH_01 */
static int hf_rrc_n_RUCCH = -1;                   /* INTEGER_0_7 */
static int hf_rrc_t_WAIT = -1;                    /* T_t_WAIT */
static int hf_rrc_t_SI = -1;                      /* T_t_SI */
static int hf_rrc_extendedEstimationWindow = -1;  /* INTEGER_2_5 */
static int hf_rrc_e_RUCCH_AccessServiceClass = -1;  /* E_RUCCH_AccessServiceClass */
static int hf_rrc_e_RUCCH_PersistenceScalingFactor_01 = -1;  /* PersistenceScalingFactorList */
static int hf_rrc_sync_UL_InfoForE_RUCCH = -1;    /* SYNC_UL_InfoForE_RUCCH */
static int hf_rrc_prach_information = -1;         /* PRACH_Information_LCR_List */
static int hf_rrc_t_RUCCH_02 = -1;                /* T_t_RUCCH_02 */
static int hf_rrc_t_WAIT_01 = -1;                 /* T_t_WAIT_01 */
static int hf_rrc_t_SI_01 = -1;                   /* T_t_SI_01 */
static int hf_rrc_e_TFCI_Boost = -1;              /* INTEGER_0_127 */
static int hf_rrc_delta_T2TP = -1;                /* INTEGER_0_6 */
static int hf_rrc_reference_Beta_QPSK_List = -1;  /* SEQUENCE_SIZE_1_8_OF_Reference_Beta_QPSK */
static int hf_rrc_reference_Beta_QPSK_List_item = -1;  /* Reference_Beta_QPSK */
static int hf_rrc_reference_Beta_16QAM_List = -1;  /* SEQUENCE_SIZE_1_8_OF_Reference_Beta_16QAM */
static int hf_rrc_reference_Beta_16QAM_List_item = -1;  /* Reference_Beta_16QAM */
static int hf_rrc_modeSpecificInfo_60 = -1;       /* T_modeSpecificInfo_60 */
static int hf_rrc_tdd384_24 = -1;                 /* T_tdd384_22 */
static int hf_rrc_ex_ul_TimingAdvance = -1;       /* INTEGER_0_255 */
static int hf_rrc_tdd768_16 = -1;                 /* T_tdd768_14 */
static int hf_rrc_ex_ul_TimingAdvance_01 = -1;    /* INTEGER_0_511 */
static int hf_rrc_ctch_Indicator = -1;            /* BOOLEAN */
static int hf_rrc_FACH_PCH_InformationList_item = -1;  /* FACH_PCH_Information */
static int hf_rrc_cCCH_Fallback = -1;             /* BOOLEAN */
static int hf_rrc_dCCH_Fallback = -1;             /* BOOLEAN */
static int hf_rrc_timeslot_01 = -1;               /* TimeslotNumber_LCR_r4 */
static int hf_rrc_channelisationCode_03 = -1;     /* TDD_FPACH_CCode16_r4 */
static int hf_rrc_midambleShiftAndBurstType_03 = -1;  /* MidambleShiftAndBurstType_LCR_r4 */
static int hf_rrc_wi = -1;                        /* Wi_LCR */
static int hf_rrc_f_tpich_Information = -1;       /* F_TPICH_Information */
static int hf_rrc_f_tpich_Info = -1;              /* T_f_tpich_Info */
static int hf_rrc_ftpich_SlotFormat = -1;         /* FTPICH_SlotFormat */
static int hf_rrc_ftpich_CodeNumber = -1;         /* INTEGER_0_255 */
static int hf_rrc_ftpich_FrameOffset = -1;        /* FTPICH_FrameOffset */
static int hf_rrc_f_tpich_InfoNewServingCell = -1;  /* F_TPICH_InfoNewServingCell */
static int hf_rrc_f_tpich_InfoOtherCellList = -1;  /* SEQUENCE_SIZE_1_maxRL_OF_F_TPICH_InfoOtherCell */
static int hf_rrc_f_tpich_InfoOtherCellList_item = -1;  /* F_TPICH_InfoOtherCell */
static int hf_rrc_FreqIndexListForEnhancedMeas_item = -1;  /* FrequencyIndexForEnhancedMeas */
static int hf_rrc_modeSpecificInfo_61 = -1;       /* T_modeSpecificInfo_61 */
static int hf_rrc_fdd_94 = -1;                    /* FrequencyInfoFDD */
static int hf_rrc_tdd_53 = -1;                    /* FrequencyInfoTDD */
static int hf_rrc_uarfcn_UL = -1;                 /* UARFCN */
static int hf_rrc_uarfcn_Nt = -1;                 /* UARFCN */
static int hf_rrc_startCode = -1;                 /* HS_ChannelisationCode_LCR */
static int hf_rrc_stopCode = -1;                  /* HS_ChannelisationCode_LCR */
static int hf_rrc_ccch_MappingInfo = -1;          /* CommonRBMappingInfo */
static int hf_rrc_srb1_MappingInfo = -1;          /* CommonRBMappingInfo */
static int hf_rrc_common_MAC_ehs_ReorderingQueueList = -1;  /* Common_MAC_ehs_ReorderingQueueList */
static int hf_rrc_hs_scch_SystemInfo = -1;        /* HS_SCCH_SystemInfo */
static int hf_rrc_harq_SystemInfo = -1;           /* HARQ_Info */
static int hf_rrc_common_H_RNTI_information = -1;  /* SEQUENCE_SIZE_1_maxCommonHRNTI_OF_H_RNTI */
static int hf_rrc_common_H_RNTI_information_item = -1;  /* H_RNTI */
static int hf_rrc_bcchSpecific_H_RNTI = -1;       /* H_RNTI */
static int hf_rrc_hs_scch_SystemInfo_tdd128 = -1;  /* HS_SCCH_SystemInfo_TDD128 */
static int hf_rrc_hs_pdsch_MidambleConfiguration = -1;  /* HS_PDSCH_Midamble_Configuration_TDD128 */
static int hf_rrc_t_321 = -1;                     /* T_321 */
static int hf_rrc_hs_dsch_DrxCycleFach = -1;      /* HS_DSCH_DrxCycleFach */
static int hf_rrc_hs_dsch_DrxBurstFach = -1;      /* HS_DSCH_DrxBurstFach */
static int hf_rrc_drxInterruption_hs_dsch = -1;   /* BOOLEAN */
static int hf_rrc_hs_dsch_SecondDrx_CycleFach = -1;  /* HS_DSCH_SecondDrx_CycleFach */
static int hf_rrc_drx_level = -1;                 /* T_drx_level */
static int hf_rrc_one_level_DRX = -1;             /* T_one_level_DRX */
static int hf_rrc_hs_dsch_Second_Rx_BurstFach = -1;  /* HS_DSCH_Second_Rx_BurstFach */
static int hf_rrc_t329 = -1;                      /* T_329 */
static int hf_rrc_two_level_DRX = -1;             /* T_two_level_DRX */
static int hf_rrc_t328 = -1;                      /* T_328 */
static int hf_rrc_hs_dsch_First_Rx_BurstFach = -1;  /* HS_DSCH_First_Rx_BurstFach */
static int hf_rrc_hs_dsch_First_Drx_CycleFach = -1;  /* HS_DSCH_First_Drx_CycleFach */
static int hf_rrc_dlScramblingCode = -1;          /* SecondaryScramblingCode */
static int hf_rrc_pich_ForHSDPASupportedPagingList = -1;  /* SEQUENCE_SIZE_1_maxSCCPCH_OF_PICH_ForHSDPASupportedPaging */
static int hf_rrc_pich_ForHSDPASupportedPagingList_item = -1;  /* PICH_ForHSDPASupportedPaging */
static int hf_rrc_numberOfPcchTransmissions = -1;  /* INTEGER_1_5 */
static int hf_rrc_transportBlockSizeList = -1;    /* SEQUENCE_SIZE_1_2_OF_TransportBlockSizeIndex */
static int hf_rrc_transportBlockSizeList_item = -1;  /* TransportBlockSizeIndex */
static int hf_rrc_pich_ForHsdschList = -1;        /* SEQUENCE_SIZE_1_maxSCCPCH_OF_PICH_ForHSDPASupportedPaging_TDD128 */
static int hf_rrc_pich_ForHsdschList_item = -1;   /* PICH_ForHSDPASupportedPaging_TDD128 */
static int hf_rrc_dtch_DCCH_reception_window_size = -1;  /* INTEGER_1_16 */
static int hf_rrc_pcch_InformationList = -1;      /* PCCH_InformationList */
static int hf_rrc_HS_DSCH_RxPatternList_TDD128_item = -1;  /* HS_DSCH_RxPatternList_TDD128_item */
static int hf_rrc_HS_DSCH_TbsList_TDD128_item = -1;  /* HS_DSCH_TbsList_TDD128_item */
static int hf_rrc_hs_dsch_TBSizeIndex = -1;       /* INTEGER_1_63 */
static int hf_rrc_hs_dsch_SPS_Operation = -1;     /* T_hs_dsch_SPS_Operation */
static int hf_rrc_newOperation_02 = -1;           /* HS_DSCH_SPS_NewOperation_TDD128 */
static int hf_rrc_hs_dsch_TbsList = -1;           /* HS_DSCH_TbsList_TDD128 */
static int hf_rrc_hs_dsch_RxPatternList = -1;     /* HS_DSCH_RxPatternList_TDD128 */
static int hf_rrc_hARQInfoForSPS = -1;            /* T_hARQInfoForSPS */
static int hf_rrc_memorySize_01 = -1;             /* HARQMemorySize */
static int hf_rrc_hs_sich_List = -1;              /* HS_SICH_List_TDD128 */
static int hf_rrc_initialSPSInfoForHSDSCH = -1;   /* T_initialSPSInfoForHSDSCH */
static int hf_rrc_timeslotInfo = -1;              /* BIT_STRING_SIZE_5 */
static int hf_rrc_codeResourceInfo_01 = -1;       /* HS_ChannelisationCodeSetInfo_LCR */
static int hf_rrc_sfnNum = -1;                    /* INTEGER_0_1 */
static int hf_rrc_initialRxPatternIndex = -1;     /* INTEGER_0_maxRxPatternForHSDSCH_TDD128_1 */
static int hf_rrc_initialTfsIndex = -1;           /* INTEGER_0_maxTbsForHSDSCH_TDD128_1 */
static int hf_rrc_modulation = -1;                /* T_modulation */
static int hf_rrc_hs_sich_Index = -1;             /* INTEGER_0_maxHSSICH_TDD128_1 */
static int hf_rrc_midambleAllocationMode_04 = -1;  /* T_midambleAllocationMode_04 */
static int hf_rrc_modeSpecificInfo_62 = -1;       /* T_modeSpecificInfo_62 */
static int hf_rrc_fdd_95 = -1;                    /* T_fdd_93 */
static int hf_rrc_tdd_54 = -1;                    /* T_tdd_50 */
static int hf_rrc_tdd384_25 = -1;                 /* T_tdd384_23 */
static int hf_rrc_nack_ack_power_offset = -1;     /* INTEGER_M7_8 */
static int hf_rrc_hs_SICH_PowerControl_Info = -1;  /* HS_SICH_Power_Control_Info_TDD384 */
static int hf_rrc_hS_SCCH_SetConfiguration = -1;  /* SEQUENCE_SIZE_1_maxHSSCCHs_OF_HS_SCCH_TDD384 */
static int hf_rrc_hS_SCCH_SetConfiguration_item = -1;  /* HS_SCCH_TDD384 */
static int hf_rrc_tdd128_35 = -1;                 /* SEQUENCE_SIZE_1_maxHSSCCHs_OF_HS_SCCH_TDD128 */
static int hf_rrc_tdd128_item = -1;               /* HS_SCCH_TDD128 */
static int hf_rrc_modeSpecificInfo_63 = -1;       /* T_modeSpecificInfo_63 */
static int hf_rrc_fdd_96 = -1;                    /* T_fdd_94 */
static int hf_rrc_tdd_55 = -1;                    /* T_tdd_51 */
static int hf_rrc_tdd384_26 = -1;                 /* T_tdd384_24 */
static int hf_rrc_bler_target = -1;               /* Bler_Target */
static int hf_rrc_hS_SCCH_SetConfiguration_01 = -1;  /* SEQUENCE_SIZE_1_maxHSSCCHs_OF_HS_SCCH_TDD384_r6 */
static int hf_rrc_hS_SCCH_SetConfiguration_item_01 = -1;  /* HS_SCCH_TDD384_r6 */
static int hf_rrc_tdd128_36 = -1;                 /* T_tdd128_35 */
static int hf_rrc_power_level_HSSICH = -1;        /* INTEGER_M120_M58 */
static int hf_rrc_tpc_step_size = -1;             /* T_tpc_step_size */
static int hf_rrc_hS_SCCH_SetConfiguration_02 = -1;  /* SEQUENCE_SIZE_1_maxHSSCCHs_OF_HS_SCCH_TDD128_r6 */
static int hf_rrc_hS_SCCH_SetConfiguration_item_02 = -1;  /* HS_SCCH_TDD128_r6 */
static int hf_rrc_modeSpecificInfo_64 = -1;       /* T_modeSpecificInfo_64 */
static int hf_rrc_fdd_97 = -1;                    /* T_fdd_95 */
static int hf_rrc_tdd_56 = -1;                    /* T_tdd_52 */
static int hf_rrc_tdd384_27 = -1;                 /* T_tdd384_25 */
static int hf_rrc_tdd768_17 = -1;                 /* T_tdd768_15 */
static int hf_rrc_hs_SICH_PowerControl_Info_01 = -1;  /* HS_SICH_Power_Control_Info_TDD768 */
static int hf_rrc_hS_SCCH_SetConfiguration_03 = -1;  /* SEQUENCE_SIZE_1_maxHSSCCHs_OF_HS_SCCH_TDD768 */
static int hf_rrc_hS_SCCH_SetConfiguration_item_03 = -1;  /* HS_SCCH_TDD768 */
static int hf_rrc_tdd128_37 = -1;                 /* T_tdd128_36 */
static int hf_rrc_tpc_step_size_01 = -1;          /* T_tpc_step_size_01 */
static int hf_rrc_powerControlGAP = -1;           /* PowerControlGAP */
static int hf_rrc_pathlossCompensationSwitch = -1;  /* BOOLEAN */
static int hf_rrc_modeSpecificInfo_65 = -1;       /* T_modeSpecificInfo_65 */
static int hf_rrc_tdd_57 = -1;                    /* T_tdd_53 */
static int hf_rrc_tdd128_38 = -1;                 /* T_tdd128_37 */
static int hf_rrc_hS_SCCH_tpc_step_size = -1;     /* T_hS_SCCH_tpc_step_size */
static int hf_rrc_modeSpecificInfo_66 = -1;       /* T_modeSpecificInfo_66 */
static int hf_rrc_fdd_98 = -1;                    /* T_fdd_96 */
static int hf_rrc_tdd_58 = -1;                    /* T_tdd_54 */
static int hf_rrc_tdd384_28 = -1;                 /* T_tdd384_26 */
static int hf_rrc_tdd768_18 = -1;                 /* T_tdd768_16 */
static int hf_rrc_tdd128_39 = -1;                 /* T_tdd128_38 */
static int hf_rrc_tpc_step_size_02 = -1;          /* T_tpc_step_size_02 */
static int hf_rrc_hS_SCCH_tpc_step_size_01 = -1;  /* T_hS_SCCH_tpc_step_size_01 */
static int hf_rrc_hS_SCCH_DRX_Cycle = -1;         /* ControlChannelDRXCycle_TDD128 */
static int hf_rrc_hS_SCCH_DRX_InactivityThreshold = -1;  /* HS_SCCH_DRX_InactivityThreshold_TDD128 */
static int hf_rrc_hS_SCCH_DRX_Offset = -1;        /* INTEGER_0_63 */
static int hf_rrc_hs_pdsch_CodeIndex = -1;        /* INTEGER_1_15 */
static int hf_rrc_hs_scch_LessTFS = -1;           /* HS_SCCH_LessTFSList */
static int hf_rrc_hs_scchLessOperation = -1;      /* T_hs_scchLessOperation */
static int hf_rrc_newOperation_03 = -1;           /* HS_SCCH_Less_NewOperation */
static int hf_rrc_HS_SCCH_LessTFSList_item = -1;  /* HS_SCCH_LessTFSList_item */
static int hf_rrc_hs_scch_LessTFSI = -1;          /* INTEGER_1_90 */
static int hf_rrc_hs_scch_LessSecondCodeSupport = -1;  /* BOOLEAN */
static int hf_rrc_hs_SCCH_SetConfiguration = -1;  /* SEQUENCE_SIZE_1_maxHSSCCHs_OF_HS_SCCH_TDD128_r6 */
static int hf_rrc_hs_SCCH_SetConfiguration_item = -1;  /* HS_SCCH_TDD128_r6 */
static int hf_rrc_tpc_step_size_03 = -1;          /* T_tpc_step_size_03 */
static int hf_rrc_power_control_gap = -1;         /* INTEGER_1_255 */
static int hf_rrc_pathloss_compensation_switch = -1;  /* BOOLEAN */
static int hf_rrc_midambleAllocationMode_05 = -1;  /* T_midambleAllocationMode_05 */
static int hf_rrc_hs_sich_configuration = -1;     /* HS_SICH_Configuration_TDD128 */
static int hf_rrc_midambleAllocationMode_06 = -1;  /* T_midambleAllocationMode_06 */
static int hf_rrc_hs_sich_configuration_01 = -1;  /* HS_SICH_Configuration_TDD128_r6 */
static int hf_rrc_uarfcn_HS_SCCH_Rx = -1;         /* UARFCN */
static int hf_rrc_midambleAllocationMode_07 = -1;  /* T_midambleAllocationMode_07 */
static int hf_rrc_channelisationCode_04 = -1;     /* HS_ChannelisationCode_LCR */
static int hf_rrc_midambleAllocationMode_08 = -1;  /* T_midambleAllocationMode_08 */
static int hf_rrc_ueSpecificMidamble_01 = -1;     /* T_ueSpecificMidamble */
static int hf_rrc_tpc_step_size_04 = -1;          /* T_tpc_step_size_04 */
static int hf_rrc_midambleAllocationMode_09 = -1;  /* T_midambleAllocationMode_09 */
static int hf_rrc_ueSpecificMidamble_02 = -1;     /* T_ueSpecificMidamble_01 */
static int hf_rrc_HS_SICH_List_TDD128_item = -1;  /* HS_SICH_List_TDD128_item */
static int hf_rrc_implicit_02 = -1;               /* T_implicit_01 */
static int hf_rrc_hS_SCCH_Index = -1;             /* INTEGER_0_maxHSSCCHs_1 */
static int hf_rrc_explicit_04 = -1;               /* T_explicit_03 */
static int hf_rrc_hS_SICH_Info = -1;              /* HS_SICH_Configuration_TDD128_r6 */
static int hf_rrc_HS_SICH_ReferenceSignalInfoList_item = -1;  /* HS_SICH_ReferenceSignalInfoList_item */
static int hf_rrc_midambleShift_01 = -1;          /* INTEGER_0_15 */
static int hf_rrc_timeSlot = -1;                  /* INTEGER_1_5 */
static int hf_rrc_channelisationCode_05 = -1;     /* DL_TS_ChannelisationCode */
static int hf_rrc_midambleAllocationMode_10 = -1;  /* T_midambleAllocationMode_10 */
static int hf_rrc_ueSpecificMidamble_03 = -1;     /* T_ueSpecificMidamble_02 */
static int hf_rrc_midambleconfiguration = -1;     /* MidambleConfigurationBurstType1and3 */
static int hf_rrc_hs_sich_configuration_02 = -1;  /* HS_SICH_Configuration_TDD384 */
static int hf_rrc_midambleAllocationMode_11 = -1;  /* T_midambleAllocationMode_11 */
static int hf_rrc_ueSpecificMidamble_04 = -1;     /* T_ueSpecificMidamble_03 */
static int hf_rrc_channelisationCode_06 = -1;     /* DL_TS_ChannelisationCode_VHCR */
static int hf_rrc_midambleAllocationMode_12 = -1;  /* T_midambleAllocationMode_12 */
static int hf_rrc_ueSpecificMidamble_05 = -1;     /* T_ueSpecificMidamble_04 */
static int hf_rrc_hs_sich_configuration_03 = -1;  /* HS_SICH_Configuration_TDD768 */
static int hf_rrc_midambleAllocationMode_13 = -1;  /* T_midambleAllocationMode_13 */
static int hf_rrc_ueSpecificMidamble_06 = -1;     /* T_ueSpecificMidamble_05 */
static int hf_rrc_midambleAllocationMode_14 = -1;  /* T_midambleAllocationMode_14 */
static int hf_rrc_ueSpecificMidamble_07 = -1;     /* T_ueSpecificMidamble_06 */
static int hf_rrc_ul_target_SIR = -1;             /* INTEGER_M22_40 */
static int hf_rrc_hs_sich_ConstantValue = -1;     /* ConstantValue */
static int hf_rrc_midambleShiftAndBurstType_04 = -1;  /* MidambleShiftAndBurstType */
static int hf_rrc_midambleShiftAndBurstType_VHCR = -1;  /* MidambleShiftAndBurstType_VHCR */
static int hf_rrc_modulation_01 = -1;             /* T_modulation_01 */
static int hf_rrc_ss_TPC_Symbols = -1;            /* T_ss_TPC_Symbols */
static int hf_rrc_additionalSS_TPC_Symbols = -1;  /* INTEGER_1_15 */
static int hf_rrc_modulation_02 = -1;             /* T_modulation_02 */
static int hf_rrc_ss_TPC_Symbols_01 = -1;         /* T_ss_TPC_Symbols_01 */
static int hf_rrc_midambleShiftAndBurstType_05 = -1;  /* MidambleShiftAndBurstType_r7 */
static int hf_rrc_ul_TimeslotInterference = -1;   /* TDD_UL_Interference */
static int hf_rrc_IndividualTS_InterferenceList_item = -1;  /* IndividualTS_Interference */
static int hf_rrc_modeSpecificInfo_67 = -1;       /* T_modeSpecificInfo_67 */
static int hf_rrc_fdd_99 = -1;                    /* T_fdd_97 */
static int hf_rrc_feedback_cycle = -1;            /* Feedback_cycle */
static int hf_rrc_cqi_RepetitionFactor = -1;      /* CQI_RepetitionFactor */
static int hf_rrc_deltaCQI = -1;                  /* DeltaCQI */
static int hf_rrc_modeSpecificInfo_68 = -1;       /* T_modeSpecificInfo_68 */
static int hf_rrc_fdd_100 = -1;                   /* T_fdd_98 */
static int hf_rrc_feedback_cycle_01 = -1;         /* Feedback_cycle_r7 */
static int hf_rrc_modeSpecificInfo_69 = -1;       /* T_modeSpecificInfo_69 */
static int hf_rrc_fdd_101 = -1;                   /* T_fdd_99 */
static int hf_rrc_cqi_RepetitionFactor_Assisting = -1;  /* CQI_RepetitionFactor */
static int hf_rrc_deltaCQI_01 = -1;               /* DeltaCQI_r11 */
static int hf_rrc_burstType_01 = -1;              /* T_burstType */
static int hf_rrc_type1_01 = -1;                  /* T_type1_01 */
static int hf_rrc_midambleConfigurationBurstType1and3 = -1;  /* MidambleConfigurationBurstType1and3 */
static int hf_rrc_midambleAllocationMode_15 = -1;  /* T_midambleAllocationMode_15 */
static int hf_rrc_ueSpecificMidamble_08 = -1;     /* T_ueSpecificMidamble_07 */
static int hf_rrc_type2 = -1;                     /* T_type2 */
static int hf_rrc_midambleConfigurationBurstType2 = -1;  /* MidambleConfigurationBurstType2 */
static int hf_rrc_midambleAllocationMode_16 = -1;  /* T_midambleAllocationMode_16 */
static int hf_rrc_ueSpecificMidamble_09 = -1;     /* T_ueSpecificMidamble_08 */
static int hf_rrc_midambleShift_02 = -1;          /* MidambleShiftShort */
static int hf_rrc_type3 = -1;                     /* T_type3 */
static int hf_rrc_midambleAllocationMode_17 = -1;  /* T_midambleAllocationMode_17 */
static int hf_rrc_ueSpecificMidamble_10 = -1;     /* T_ueSpecificMidamble_09 */
static int hf_rrc_burstType_02 = -1;              /* T_burstType_01 */
static int hf_rrc_type1_02 = -1;                  /* T_type1_02 */
static int hf_rrc_midambleAllocationMode_18 = -1;  /* T_midambleAllocationMode_18 */
static int hf_rrc_ueSpecificMidamble_11 = -1;     /* T_ueSpecificMidamble_10 */
static int hf_rrc_type2_01 = -1;                  /* T_type2_01 */
static int hf_rrc_midambleConfigurationBurstType2_01 = -1;  /* MidambleConfigurationBurstType2_VHCR */
static int hf_rrc_midambleAllocationMode_19 = -1;  /* T_midambleAllocationMode_19 */
static int hf_rrc_ueSpecificMidamble_12 = -1;     /* T_ueSpecificMidamble_11 */
static int hf_rrc_midambleShift_03 = -1;          /* MidambleShiftShort_VHCR */
static int hf_rrc_type3_01 = -1;                  /* T_type3_01 */
static int hf_rrc_midambleAllocationMode_20 = -1;  /* T_midambleAllocationMode_20 */
static int hf_rrc_ueSpecificMidamble_13 = -1;     /* T_ueSpecificMidamble_12 */
static int hf_rrc_mbsfnBurstType4 = -1;           /* NULL */
static int hf_rrc_burstType_03 = -1;              /* T_burstType_02 */
static int hf_rrc_type1_03 = -1;                  /* T_type1_03 */
static int hf_rrc_midambleAllocationMode_21 = -1;  /* T_midambleAllocationMode_21 */
static int hf_rrc_ueSpecificMidamble_14 = -1;     /* T_ueSpecificMidamble_13 */
static int hf_rrc_type2_02 = -1;                  /* T_type2_02 */
static int hf_rrc_midambleAllocationMode_22 = -1;  /* T_midambleAllocationMode_22 */
static int hf_rrc_ueSpecificMidamble_15 = -1;     /* T_ueSpecificMidamble_14 */
static int hf_rrc_type3_02 = -1;                  /* T_type3_02 */
static int hf_rrc_midambleAllocationMode_23 = -1;  /* T_midambleAllocationMode_23 */
static int hf_rrc_ueSpecificMidamble_16 = -1;     /* T_ueSpecificMidamble_15 */
static int hf_rrc_burstType_04 = -1;              /* T_burstType_03 */
static int hf_rrc_type1_04 = -1;                  /* T_type1_04 */
static int hf_rrc_midambleAllocationMode_24 = -1;  /* T_midambleAllocationMode_24 */
static int hf_rrc_ueSpecificMidamble_17 = -1;     /* T_ueSpecificMidamble_16 */
static int hf_rrc_type2_03 = -1;                  /* T_type2_03 */
static int hf_rrc_midambleAllocationMode_25 = -1;  /* T_midambleAllocationMode_25 */
static int hf_rrc_ueSpecificMidamble_18 = -1;     /* T_ueSpecificMidamble_17 */
static int hf_rrc_burstType_05 = -1;              /* T_burstType_04 */
static int hf_rrc_type1_05 = -1;                  /* T_type1_05 */
static int hf_rrc_midambleAllocationMode_26 = -1;  /* T_midambleAllocationMode_26 */
static int hf_rrc_ueSpecificMidamble_19 = -1;     /* T_ueSpecificMidamble_18 */
static int hf_rrc_type2_04 = -1;                  /* T_type2_04 */
static int hf_rrc_midambleAllocationMode_27 = -1;  /* T_midambleAllocationMode_27 */
static int hf_rrc_ueSpecificMidamble_20 = -1;     /* T_ueSpecificMidamble_19 */
static int hf_rrc_midambleAllocationMode_28 = -1;  /* T_midambleAllocationMode_28 */
static int hf_rrc_ueSpecificMidamble_21 = -1;     /* T_ueSpecificMidamble_20 */
static int hf_rrc_burstType_06 = -1;              /* T_burstType_05 */
static int hf_rrc_type1_06 = -1;                  /* T_type1_06 */
static int hf_rrc_midambleConfigurationBurstType1 = -1;  /* MidambleConfigurationBurstType1 */
static int hf_rrc_midambleAllocationMode_29 = -1;  /* T_midambleAllocationMode_29 */
static int hf_rrc_ueSpecificMidamble_22 = -1;     /* T_ueSpecificMidamble_21 */
static int hf_rrc_type2_05 = -1;                  /* T_type2_05 */
static int hf_rrc_midambleAllocationMode_30 = -1;  /* T_midambleAllocationMode_30 */
static int hf_rrc_ueSpecificMidamble_23 = -1;     /* T_ueSpecificMidamble_22 */
static int hf_rrc_mimoOperation = -1;             /* MIMO_Operation */
static int hf_rrc_mimoN_M_Ratio = -1;             /* MIMO_N_M_Ratio */
static int hf_rrc_mimoPilotConfiguration = -1;    /* MIMO_PilotConfiguration */
static int hf_rrc_newConfiguration_06 = -1;       /* T_newConfiguration_06 */
static int hf_rrc_mimo4x4N_M_Ratio = -1;          /* MIMO_N_M_Ratio */
static int hf_rrc_mimo4x4PilotConfiguration = -1;  /* MIMO4x4_PilotConfiguration */
static int hf_rrc_precodingWeightSetRestriction = -1;  /* BIT_STRING_SIZE_64 */
static int hf_rrc_precodingWeightSetRestriction_01 = -1;  /* T_precodingWeightSetRestriction */
static int hf_rrc_modeSpecificInfo_70 = -1;       /* T_modeSpecificInfo_70 */
static int hf_rrc_fdd_102 = -1;                   /* T_fdd_100 */
static int hf_rrc_tdd_59 = -1;                    /* T_tdd_55 */
static int hf_rrc_tdd128_40 = -1;                 /* T_tdd128_39 */
static int hf_rrc_mimoSFModeForHSPDSCHDualStream = -1;  /* T_mimoSFModeForHSPDSCHDualStream */
static int hf_rrc_hs_sich_ReferenceSignalInfoList = -1;  /* HS_SICH_ReferenceSignalInfoList */
static int hf_rrc_tdd384_tdd768_04 = -1;          /* NULL */
static int hf_rrc_modeSpecificInfo_71 = -1;       /* T_modeSpecificInfo_71 */
static int hf_rrc_fdd_103 = -1;                   /* T_fdd_101 */
static int hf_rrc_mimoPilotConfiguration_01 = -1;  /* MIMO_PilotConfiguration_r9 */
static int hf_rrc_precodingWeightSetRestriction_02 = -1;  /* T_precodingWeightSetRestriction_01 */
static int hf_rrc_tdd_60 = -1;                    /* T_tdd_56 */
static int hf_rrc_tdd128_41 = -1;                 /* T_tdd128_40 */
static int hf_rrc_mimoSFModeForHSPDSCHDualStream_01 = -1;  /* T_mimoSFModeForHSPDSCHDualStream_01 */
static int hf_rrc_secondCPICH_Pattern = -1;       /* T_secondCPICH_Pattern */
static int hf_rrc_normalPattern = -1;             /* NULL */
static int hf_rrc_diversityPattern = -1;          /* T_diversityPattern */
static int hf_rrc_s_cpich_PowerOffset_Mimo_01 = -1;  /* S_CPICH_PowerOffset_MIMO */
static int hf_rrc_secondCPICH_Pattern_01 = -1;    /* T_secondCPICH_Pattern_01 */
static int hf_rrc_diversityPattern_01 = -1;       /* T_diversityPattern_01 */
static int hf_rrc_antenna2 = -1;                  /* MIMO_PilotConfiguration_r9 */
static int hf_rrc_antenna3And4 = -1;              /* Antenna3And4 */
static int hf_rrc_secondFrequencyInfo = -1;       /* FrequencyInfoTDD */
static int hf_rrc_fPachFrequencyInfo = -1;        /* FrequencyInfoTDD */
static int hf_rrc_upPCHpositionInfo = -1;         /* UpPCHposition_LCR */
static int hf_rrc_cellType = -1;                  /* MultiflowCellType */
static int hf_rrc_timeReferenceCell = -1;         /* T_timeReferenceCell */
static int hf_rrc_newConfiguration_07 = -1;       /* T_newConfiguration_07 */
static int hf_rrc_mu_MIMO_Operation = -1;         /* MU_MIMO_Operation */
static int hf_rrc_standaloneMidambleInfo = -1;    /* StandaloneMidambleInfo_TDD128 */
static int hf_rrc_enablingDelay_01 = -1;          /* EnablingDelay */
static int hf_rrc_ue_dtx_drx_Offset = -1;         /* UE_DTX_DRX_Offset */
static int hf_rrc_hs_dpcch_TransmitContinuationOffset = -1;  /* T_hs_dpcch_TransmitContinuationOffset */
static int hf_rrc_tdd384_768 = -1;                /* T_tdd384_768 */
static int hf_rrc_timeslotResourceRelatedInfo_01 = -1;  /* BIT_STRING_SIZE_13 */
static int hf_rrc_tdd128_42 = -1;                 /* T_tdd128_41 */
static int hf_rrc_e_HICH_Info_08 = -1;            /* T_e_HICH_Info_06 */
static int hf_rrc_midambleAllocationMode_31 = -1;  /* T_midambleAllocationMode_31 */
static int hf_rrc_tdd384_768_01 = -1;             /* NULL */
static int hf_rrc_tdd128_43 = -1;                 /* T_tdd128_42 */
static int hf_rrc_t_SI_nst = -1;                  /* T_t_SI_nst */
static int hf_rrc_tdd384_768_02 = -1;             /* T_tdd384_768_01 */
static int hf_rrc_tdd128_44 = -1;                 /* T_tdd128_43 */
static int hf_rrc_e_HICH_Info_09 = -1;            /* T_e_HICH_Info_07 */
static int hf_rrc_midambleAllocationMode_32 = -1;  /* T_midambleAllocationMode_32 */
static int hf_rrc_t_SI_nst_01 = -1;               /* T_t_SI_nst_01 */
static int hf_rrc_dpch_ConstantValue = -1;        /* ConstantValueTdd */
static int hf_rrc_ipdl_alpha = -1;                /* Alpha */
static int hf_rrc_maxPowerIncrease = -1;          /* MaxPowerIncrease_r4 */
static int hf_rrc_paging_associatedHspdschInfo = -1;  /* T_paging_associatedHspdschInfo */
static int hf_rrc_paging_associatedHspdschInfo_item = -1;  /* T_paging_associatedHspdschInfo_item */
static int hf_rrc_timeslotResourceRelatedInfo_02 = -1;  /* BIT_STRING_SIZE_6 */
static int hf_rrc_codeResourceInformation = -1;   /* CodeResourceInformation_TDD128 */
static int hf_rrc_paging_sub_Channel_size = -1;   /* INTEGER_1_3 */
static int hf_rrc_pcpch_UL_ScramblingCode = -1;   /* INTEGER_0_79 */
static int hf_rrc_pcpch_DL_ChannelisationCode = -1;  /* INTEGER_0_511 */
static int hf_rrc_pcpch_DL_ScramblingCode = -1;   /* SecondaryScramblingCode */
static int hf_rrc_pcp_Length = -1;                /* PCP_Length */
static int hf_rrc_ucsm_Info = -1;                 /* UCSM_Info */
static int hf_rrc_PCPCH_ChannelInfoList_item = -1;  /* PCPCH_ChannelInfo */
static int hf_rrc_pdsch_PowerControlInfo = -1;    /* PDSCH_PowerControlInfo */
static int hf_rrc_pdsch_AllocationPeriodInfo = -1;  /* AllocationPeriodInfo */
static int hf_rrc_configuration = -1;             /* T_configuration */
static int hf_rrc_old_Configuration = -1;         /* T_old_Configuration */
static int hf_rrc_pdsch_Identity = -1;            /* PDSCH_Identity */
static int hf_rrc_new_Configuration = -1;         /* T_new_Configuration */
static int hf_rrc_pdsch_Info = -1;                /* PDSCH_Info */
static int hf_rrc_configuration_01 = -1;          /* T_configuration_01 */
static int hf_rrc_old_Configuration_01 = -1;      /* T_old_Configuration_01 */
static int hf_rrc_new_Configuration_01 = -1;      /* T_new_Configuration_01 */
static int hf_rrc_pdsch_Info_01 = -1;             /* PDSCH_Info_r4 */
static int hf_rrc_configuration_02 = -1;          /* T_configuration_02 */
static int hf_rrc_old_Configuration_02 = -1;      /* T_old_Configuration_02 */
static int hf_rrc_new_Configuration_02 = -1;      /* T_new_Configuration_02 */
static int hf_rrc_pdsch_Info_02 = -1;             /* PDSCH_Info_r7 */
static int hf_rrc_PDSCH_CodeInfoList_item = -1;   /* PDSCH_CodeInfo */
static int hf_rrc_codeNumberStart = -1;           /* CodeNumberDSCH */
static int hf_rrc_codeNumberStop = -1;            /* CodeNumberDSCH */
static int hf_rrc_PDSCH_CodeMapList_item = -1;    /* PDSCH_CodeMap */
static int hf_rrc_signallingMethod = -1;          /* T_signallingMethod */
static int hf_rrc_codeRange = -1;                 /* CodeRange */
static int hf_rrc_tfci_Range_01 = -1;             /* DSCH_MappingList */
static int hf_rrc_explicit_config_03 = -1;        /* PDSCH_CodeInfoList */
static int hf_rrc_replace = -1;                   /* ReplacedPDSCH_CodeInfoList */
static int hf_rrc_pdsch_TimeslotsCodes = -1;      /* DownlinkTimeslotsCodes */
static int hf_rrc_tddOption_12 = -1;              /* T_tddOption_12 */
static int hf_rrc_tdd384_29 = -1;                 /* T_tdd384_27 */
static int hf_rrc_tdd128_45 = -1;                 /* T_tdd128_44 */
static int hf_rrc_pdsch_TimeslotsCodes_01 = -1;   /* DownlinkTimeslotsCodes_LCR_r4 */
static int hf_rrc_tddOption_13 = -1;              /* T_tddOption_13 */
static int hf_rrc_tdd384_30 = -1;                 /* T_tdd384_28 */
static int hf_rrc_tdd768_19 = -1;                 /* T_tdd768_17 */
static int hf_rrc_pdsch_TimeslotsCodes_02 = -1;   /* DownlinkTimeslotsCodes_VHCR */
static int hf_rrc_tdd128_46 = -1;                 /* T_tdd128_45 */
static int hf_rrc_dsch_RadioLinkIdentifier = -1;  /* DSCH_RadioLinkIdentifier */
static int hf_rrc_rl_IdentifierList = -1;         /* RL_IdentifierList */
static int hf_rrc_dsch_TFCS = -1;                 /* TFCS */
static int hf_rrc_dsch_TransportChannelsInfo = -1;  /* DSCH_TransportChannelsInfo */
static int hf_rrc_pdsch_Info_03 = -1;             /* PDSCH_Info_LCR_r4 */
static int hf_rrc_PDSCH_SysInfoList_item = -1;    /* PDSCH_SysInfo */
static int hf_rrc_PDSCH_SysInfoList_VHCR_r7_item = -1;  /* PDSCH_SysInfo_VHCR_r7 */
static int hf_rrc_PDSCH_SysInfoList_HCR_r5_item = -1;  /* PDSCH_SysInfo_HCR_r5 */
static int hf_rrc_PDSCH_SysInfoList_LCR_r4_item = -1;  /* PDSCH_SysInfo_LCR_r4 */
static int hf_rrc_PDSCH_SysInfoList_SFN_item = -1;  /* PDSCH_SysInfoList_SFN_item */
static int hf_rrc_pdsch_SysInfo = -1;             /* PDSCH_SysInfo */
static int hf_rrc_sfn_TimeInfo = -1;              /* SFN_TimeInfo */
static int hf_rrc_PDSCH_SysInfoList_SFN_HCR_r5_item = -1;  /* PDSCH_SysInfoList_SFN_HCR_r5_item */
static int hf_rrc_pdsch_SysInfo_01 = -1;          /* PDSCH_SysInfo_HCR_r5 */
static int hf_rrc_PDSCH_SysInfoList_SFN_LCR_r4_item = -1;  /* PDSCH_SysInfoList_SFN_LCR_r4_item */
static int hf_rrc_pdsch_SysInfo_02 = -1;          /* PDSCH_SysInfo_LCR_r4 */
static int hf_rrc_PersistenceScalingFactorList_item = -1;  /* PersistenceScalingFactor */
static int hf_rrc_PichChannelisationCodeList_LCR_r4_item = -1;  /* DL_TS_ChannelisationCode */
static int hf_rrc_hsdpa_AssociatedPichInfo = -1;  /* PICH_Info */
static int hf_rrc_hs_pdschChannelisationCode = -1;  /* INTEGER_1_15 */
static int hf_rrc_implicit_03 = -1;               /* T_implicit_02 */
static int hf_rrc_occurrenceSequenceNumberOfPICH = -1;  /* OccurrenceSequenceNumberOfPICH */
static int hf_rrc_explicit_05 = -1;               /* PICH_Info_LCR_r4 */
static int hf_rrc_fdd_104 = -1;                   /* T_fdd_102 */
static int hf_rrc_pi_CountPerFrame = -1;          /* PI_CountPerFrame */
static int hf_rrc_tdd_61 = -1;                    /* T_tdd_57 */
static int hf_rrc_channelisationCode_07 = -1;     /* TDD_PICH_CCode */
static int hf_rrc_repetitionPeriodLengthOffset = -1;  /* RepPerLengthOffset_PICH */
static int hf_rrc_pagingIndicatorLength = -1;     /* PagingIndicatorLength */
static int hf_rrc_n_GAP = -1;                     /* N_GAP */
static int hf_rrc_n_PCH = -1;                     /* N_PCH */
static int hf_rrc_channelisationCode_08 = -1;     /* T_channelisationCode */
static int hf_rrc_tdd384_31 = -1;                 /* TDD_PICH_CCode */
static int hf_rrc_tdd768_20 = -1;                 /* TDD768_PICH_CCode */
static int hf_rrc_pichChannelisationCodeList_LCR_r4 = -1;  /* PichChannelisationCodeList_LCR_r4 */
static int hf_rrc_plcchSequenceNumber = -1;       /* INTEGER_1_14 */
static int hf_rrc_tpcCommandTargetRate = -1;      /* TPC_CommandTargetRate */
static int hf_rrc_algorithm1 = -1;                /* TPC_StepSizeFDD */
static int hf_rrc_algorithm2 = -1;                /* NULL */
static int hf_rrc_PRACH_ChanCodes_LCR_r4_item = -1;  /* TDD_PRACH_CCode_LCR_r4 */
static int hf_rrc_PRACH_ChanCodes_List_LCR_item = -1;  /* TDD_PRACH_CCode_LCR_r4 */
static int hf_rrc_timeslot_02 = -1;               /* TimeslotNumber_PRACH_LCR_r4 */
static int hf_rrc_prach_ChanCodes_LCR = -1;       /* PRACH_ChanCodes_LCR_r4 */
static int hf_rrc_fpach_Info = -1;                /* FPACH_Info_r4 */
static int hf_rrc_prach_ChanCodes_list_LCR = -1;  /* PRACH_ChanCodes_List_LCR */
static int hf_rrc_PRACH_Information_LCR_List_item = -1;  /* PRACH_Information_LCR */
static int hf_rrc_fdd_105 = -1;                   /* SEQUENCE_SIZE_1_maxASC_OF_ASCSetting_FDD */
static int hf_rrc_fdd_item = -1;                  /* ASCSetting_FDD */
static int hf_rrc_tdd_62 = -1;                    /* SEQUENCE_SIZE_1_maxASC_OF_ASCSetting_TDD */
static int hf_rrc_tdd_item = -1;                  /* ASCSetting_TDD */
static int hf_rrc_tdd_63 = -1;                    /* SEQUENCE_SIZE_1_maxASC_OF_ASCSetting_TDD_r7 */
static int hf_rrc_tdd_item_01 = -1;               /* ASCSetting_TDD_r7 */
static int hf_rrc_PRACH_Partitioning_LCR_r4_item = -1;  /* ASCSetting_TDD_LCR_r4 */
static int hf_rrc_powerRampStep = -1;             /* PowerRampStep */
static int hf_rrc_preambleRetransMax = -1;        /* PreambleRetransMax */
static int hf_rrc_availableSignatures = -1;       /* AvailableSignatures */
static int hf_rrc_e_ai_Indication = -1;           /* BOOLEAN */
static int hf_rrc_preambleScramblingCodeWordNumber = -1;  /* PreambleScramblingCodeWordNumber */
static int hf_rrc_availableSubChannelNumbers = -1;  /* AvailableSubChannelNumbers */
static int hf_rrc_prach_Partitioning = -1;        /* PRACH_Partitioning_r7 */
static int hf_rrc_persistenceScalingFactorList = -1;  /* PersistenceScalingFactorList */
static int hf_rrc_ac_To_ASC_MappingTable = -1;    /* AC_To_ASC_MappingTable */
static int hf_rrc_primaryCPICH_TX_Power = -1;     /* PrimaryCPICH_TX_Power */
static int hf_rrc_constantValue = -1;             /* ConstantValue */
static int hf_rrc_prach_PowerOffset = -1;         /* PRACH_PowerOffset */
static int hf_rrc_rach_TransmissionParameters = -1;  /* RACH_TransmissionParameters */
static int hf_rrc_aich_Info = -1;                 /* AICH_Info */
static int hf_rrc_powerOffsetPp_e = -1;           /* INTEGER_M5_10 */
static int hf_rrc_aich_Info_01 = -1;              /* AICH_Info_Compressed */
static int hf_rrc_prach_PreambleForEnhancedUplinkList = -1;  /* SEQUENCE_SIZE_1_maxPRACH_EUL_OF_PRACH_PreambleForEnhancedUplinkExtWithWeight */
static int hf_rrc_prach_PreambleForEnhancedUplinkList_item = -1;  /* PRACH_PreambleForEnhancedUplinkExtWithWeight */
static int hf_rrc_prach_PreambleForEnhancedUplinkExt = -1;  /* PRACH_PreambleForEnhancedUplinkExt */
static int hf_rrc_weight = -1;                    /* INTEGER_0_5 */
static int hf_rrc_modeSpecificInfo_72 = -1;       /* T_modeSpecificInfo_72 */
static int hf_rrc_fdd_106 = -1;                   /* T_fdd_103 */
static int hf_rrc_availableSF = -1;               /* SF_PRACH */
static int hf_rrc_tdd_64 = -1;                    /* T_tdd_58 */
static int hf_rrc_channelisationCodeList = -1;    /* TDD_PRACH_CCodeList */
static int hf_rrc_prach_Midamble = -1;            /* PRACH_Midamble */
static int hf_rrc_channelisationCodeList_01 = -1;  /* TDD768_PRACH_CCodeList */
static int hf_rrc_e_RUCCH_Sync_UL_Codes_Bitmap = -1;  /* Sync_UL_Codes_Bitmap */
static int hf_rrc_sync_UL_Info = -1;              /* SYNC_UL_Info_r4 */
static int hf_rrc_prach_DefinitionList = -1;      /* SEQUENCE_SIZE_1_maxPRACH_FPACH_OF_PRACH_Definition_LCR_r4 */
static int hf_rrc_prach_DefinitionList_item = -1;  /* PRACH_Definition_LCR_r4 */
static int hf_rrc_prach_RACH_Info = -1;           /* PRACH_RACH_Info */
static int hf_rrc_rach_TransportFormatSet = -1;   /* TransportFormatSet */
static int hf_rrc_rach_TFCS = -1;                 /* TFCS */
static int hf_rrc_prach_Partitioning_01 = -1;     /* PRACH_Partitioning */
static int hf_rrc_modeSpecificInfo_73 = -1;       /* T_modeSpecificInfo_73 */
static int hf_rrc_fdd_107 = -1;                   /* T_fdd_104 */
static int hf_rrc_prach_RACH_Info_01 = -1;        /* PRACH_RACH_Info_VHCR_r7 */
static int hf_rrc_prach_RACH_Info_LCR = -1;       /* PRACH_RACH_Info_LCR_r4 */
static int hf_rrc_rach_TransportFormatSet_LCR = -1;  /* TransportFormatSet_LCR */
static int hf_rrc_prach_Partitioning_LCR = -1;    /* PRACH_Partitioning_LCR_r4 */
static int hf_rrc_prach_RACH_Info_LCR_01 = -1;    /* PRACH_RACH_Info_LCR_v770ext */
static int hf_rrc_PRACH_SystemInformationList_item = -1;  /* PRACH_SystemInformation */
static int hf_rrc_PRACH_SystemInformationList_VHCR_r7_item = -1;  /* PRACH_SystemInformation_VHCR_r7 */
static int hf_rrc_PRACH_SystemInformationList_LCR_r4_item = -1;  /* PRACH_SystemInformation_LCR_r4 */
static int hf_rrc_PRACH_SystemInformationList_LCR_v770ext_item = -1;  /* PRACH_SystemInformation_LCR_v770ext */
static int hf_rrc_ul_DPCH_InfoPredef = -1;        /* UL_DPCH_InfoPredef */
static int hf_rrc_dl_CommonInformationPredef = -1;  /* DL_CommonInformationPredef */
static int hf_rrc_ul_DPCH_InfoPredef_01 = -1;     /* UL_DPCH_InfoPredef_v770ext */
static int hf_rrc_fdd_108 = -1;                   /* T_fdd_105 */
static int hf_rrc_tx_DiversityIndicator = -1;     /* BOOLEAN */
static int hf_rrc_tdd_65 = -1;                    /* T_tdd_59 */
static int hf_rrc_syncCase = -1;                  /* T_syncCase */
static int hf_rrc_syncCase1 = -1;                 /* T_syncCase1 */
static int hf_rrc_syncCase2 = -1;                 /* T_syncCase2 */
static int hf_rrc_timeslotSync2 = -1;             /* TimeslotSync2 */
static int hf_rrc_sctd_Indicator = -1;            /* BOOLEAN */
static int hf_rrc_fdd_109 = -1;                   /* T_fdd_106 */
static int hf_rrc_tdd_66 = -1;                    /* T_tdd_60 */
static int hf_rrc_tddOption_14 = -1;              /* T_tddOption_14 */
static int hf_rrc_tdd384_tdd768_05 = -1;          /* T_tdd384_tdd768_04 */
static int hf_rrc_syncCase_01 = -1;               /* T_syncCase_01 */
static int hf_rrc_syncCase1_01 = -1;              /* T_syncCase1_01 */
static int hf_rrc_syncCase2_01 = -1;              /* T_syncCase2_01 */
static int hf_rrc_tdd128_47 = -1;                 /* T_tdd128_46 */
static int hf_rrc_syncCase_02 = -1;               /* T_syncCase_02 */
static int hf_rrc_syncCase1_02 = -1;              /* T_syncCase1_02 */
static int hf_rrc_syncCase2_02 = -1;              /* T_syncCase2_02 */
static int hf_rrc_primaryScramblingCode = -1;     /* PrimaryScramblingCode */
static int hf_rrc_pusch_Allocation = -1;          /* T_pusch_Allocation */
static int hf_rrc_pusch_AllocationPending = -1;   /* NULL */
static int hf_rrc_pusch_AllocationAssignment = -1;  /* T_pusch_AllocationAssignment */
static int hf_rrc_pusch_AllocationPeriodInfo = -1;  /* AllocationPeriodInfo */
static int hf_rrc_pusch_PowerControlInfo = -1;    /* UL_TargetSIR */
static int hf_rrc_configuration_03 = -1;          /* T_configuration_03 */
static int hf_rrc_old_Configuration_03 = -1;      /* T_old_Configuration_03 */
static int hf_rrc_pusch_Identity = -1;            /* PUSCH_Identity */
static int hf_rrc_new_Configuration_03 = -1;      /* T_new_Configuration_03 */
static int hf_rrc_pusch_Info = -1;                /* PUSCH_Info */
static int hf_rrc_pusch_Allocation_01 = -1;       /* T_pusch_Allocation_01 */
static int hf_rrc_pusch_AllocationAssignment_01 = -1;  /* T_pusch_AllocationAssignment_01 */
static int hf_rrc_pusch_PowerControlInfo_01 = -1;  /* PUSCH_PowerControlInfo_r4 */
static int hf_rrc_configuration_04 = -1;          /* T_configuration_04 */
static int hf_rrc_old_Configuration_04 = -1;      /* T_old_Configuration_04 */
static int hf_rrc_new_Configuration_04 = -1;      /* T_new_Configuration_04 */
static int hf_rrc_pusch_Info_01 = -1;             /* PUSCH_Info_r4 */
static int hf_rrc_pusch_Allocation_02 = -1;       /* T_pusch_Allocation_02 */
static int hf_rrc_pusch_AllocationAssignment_02 = -1;  /* T_pusch_AllocationAssignment_02 */
static int hf_rrc_pusch_PowerControlInfo_02 = -1;  /* PUSCH_PowerControlInfo_r7 */
static int hf_rrc_configuration_05 = -1;          /* T_configuration_05 */
static int hf_rrc_old_Configuration_05 = -1;      /* T_old_Configuration_05 */
static int hf_rrc_new_Configuration_05 = -1;      /* T_new_Configuration_05 */
static int hf_rrc_pusch_Info_02 = -1;             /* PUSCH_Info_VHCR */
static int hf_rrc_pusch_TimeslotsCodes = -1;      /* UplinkTimeslotsCodes */
static int hf_rrc_tddOption_15 = -1;              /* T_tddOption_15 */
static int hf_rrc_tdd384_32 = -1;                 /* T_tdd384_29 */
static int hf_rrc_tdd128_48 = -1;                 /* T_tdd128_47 */
static int hf_rrc_pusch_TimeslotsCodes_01 = -1;   /* UplinkTimeslotsCodes_LCR_r4 */
static int hf_rrc_pusch_TimeslotsCodes_VHCR = -1;  /* UplinkTimeslotsCodes_VHCR */
static int hf_rrc_ul_TargetSIR = -1;              /* UL_TargetSIR */
static int hf_rrc_tddOption_16 = -1;              /* T_tddOption_16 */
static int hf_rrc_tdd128_49 = -1;                 /* T_tdd128_48 */
static int hf_rrc_tddOption_17 = -1;              /* T_tddOption_17 */
static int hf_rrc_tdd128_50 = -1;                 /* T_tdd128_49 */
static int hf_rrc_usch_TFCS = -1;                 /* TFCS */
static int hf_rrc_pusch_Info_VHCR = -1;           /* PUSCH_Info_VHCR */
static int hf_rrc_usch_TransportChannelsInfo = -1;  /* USCH_TransportChannelsInfo */
static int hf_rrc_pusch_Info_03 = -1;             /* PUSCH_Info_LCR_r4 */
static int hf_rrc_PUSCH_SysInfoList_item = -1;    /* PUSCH_SysInfo */
static int hf_rrc_PUSCH_SysInfoList_HCR_r5_item = -1;  /* PUSCH_SysInfo_HCR_r5 */
static int hf_rrc_PUSCH_SysInfoList_LCR_r4_item = -1;  /* PUSCH_SysInfo_LCR_r4 */
static int hf_rrc_PUSCH_SysInfoList_SFN_item = -1;  /* PUSCH_SysInfoList_SFN_item */
static int hf_rrc_pusch_SysInfo = -1;             /* PUSCH_SysInfo */
static int hf_rrc_PUSCH_SysInfoList_SFN_HCR_r5_item = -1;  /* PUSCH_SysInfoList_SFN_HCR_r5_item */
static int hf_rrc_pusch_SysInfo_01 = -1;          /* PUSCH_SysInfo_HCR_r5 */
static int hf_rrc_PUSCH_SysInfoList_SFN_LCR_r4_item = -1;  /* PUSCH_SysInfoList_SFN_LCR_r4_item */
static int hf_rrc_pusch_SysInfo_02 = -1;          /* PUSCH_SysInfo_LCR_r4 */
static int hf_rrc_PUSCH_SysInfoList_SFN_VHCR_item = -1;  /* PUSCH_SysInfoList_SFN_VHCR_item */
static int hf_rrc_pusch_SysInfo_VHCR = -1;        /* PUSCH_SysInfo_VHCR */
static int hf_rrc_mmax = -1;                      /* INTEGER_1_32 */
static int hf_rrc_nb01Min = -1;                   /* NB01 */
static int hf_rrc_nb01Max = -1;                   /* NB01 */
static int hf_rrc_reference_Code_Rate = -1;       /* INTEGER_0_10 */
static int hf_rrc_reference_Beta = -1;            /* INTEGER_M15_16 */
static int hf_rrc_repetitionPeriod1 = -1;         /* NULL */
static int hf_rrc_repetitionPeriod2 = -1;         /* INTEGER_1_1 */
static int hf_rrc_repetitionPeriod4 = -1;         /* INTEGER_1_3 */
static int hf_rrc_repetitionPeriod8 = -1;         /* INTEGER_1_7 */
static int hf_rrc_repetitionPeriod16 = -1;        /* INTEGER_1_15 */
static int hf_rrc_repetitionPeriod32 = -1;        /* INTEGER_1_31 */
static int hf_rrc_repetitionPeriod64 = -1;        /* INTEGER_1_63 */
static int hf_rrc_repetitionPeriod2_01 = -1;      /* T_repetitionPeriod2 */
static int hf_rrc_length = -1;                    /* NULL */
static int hf_rrc_offset = -1;                    /* INTEGER_0_1 */
static int hf_rrc_repetitionPeriod4_01 = -1;      /* T_repetitionPeriod4 */
static int hf_rrc_length_01 = -1;                 /* INTEGER_1_3 */
static int hf_rrc_offset_01 = -1;                 /* INTEGER_0_3 */
static int hf_rrc_repetitionPeriod8_01 = -1;      /* T_repetitionPeriod8 */
static int hf_rrc_length_02 = -1;                 /* INTEGER_1_7 */
static int hf_rrc_offset_02 = -1;                 /* INTEGER_0_7 */
static int hf_rrc_repetitionPeriod16_01 = -1;     /* T_repetitionPeriod16 */
static int hf_rrc_length_03 = -1;                 /* INTEGER_1_15 */
static int hf_rrc_offset_03 = -1;                 /* INTEGER_0_15 */
static int hf_rrc_repetitionPeriod32_01 = -1;     /* T_repetitionPeriod32 */
static int hf_rrc_length_04 = -1;                 /* INTEGER_1_31 */
static int hf_rrc_offset_04 = -1;                 /* INTEGER_0_31 */
static int hf_rrc_repetitionPeriod64_01 = -1;     /* T_repetitionPeriod64 */
static int hf_rrc_length_05 = -1;                 /* INTEGER_1_63 */
static int hf_rrc_offset_05 = -1;                 /* INTEGER_0_63 */
static int hf_rrc_tfci_Field2 = -1;               /* MaxTFCI_Field2Value */
static int hf_rrc_ReplacedPDSCH_CodeInfoList_item = -1;  /* ReplacedPDSCH_CodeInfo */
static int hf_rrc_rpp4_2 = -1;                    /* INTEGER_0_3 */
static int hf_rrc_rpp8_2 = -1;                    /* INTEGER_0_7 */
static int hf_rrc_rpp8_4 = -1;                    /* INTEGER_0_7 */
static int hf_rrc_rpp16_2 = -1;                   /* INTEGER_0_15 */
static int hf_rrc_rpp16_4 = -1;                   /* INTEGER_0_15 */
static int hf_rrc_rpp32_2 = -1;                   /* INTEGER_0_31 */
static int hf_rrc_rpp32_4 = -1;                   /* INTEGER_0_31 */
static int hf_rrc_rpp64_2 = -1;                   /* INTEGER_0_63 */
static int hf_rrc_rpp64_4 = -1;                   /* INTEGER_0_63 */
static int hf_rrc_dl_restrictedTrCh_Type = -1;    /* DL_TrCH_Type */
static int hf_rrc_restrictedDL_TrCH_Identity = -1;  /* TransportChannelIdentity */
static int hf_rrc_allowedTFIList = -1;            /* AllowedTFI_List */
static int hf_rrc_RestrictedTrCH_InfoList_item = -1;  /* RestrictedTrCH */
static int hf_rrc_dummy2_08 = -1;                 /* SCCPCH_InfoForFACH */
static int hf_rrc_cell_Id = -1;                   /* CellIdentity */
static int hf_rrc_dl_dpchInfo_04 = -1;            /* T_dl_dpchInfo_04 */
static int hf_rrc_dl_dpchInfo_05 = -1;            /* T_dl_dpchInfo_05 */
static int hf_rrc_dl_dpchInfo_06 = -1;            /* T_dl_dpchInfo_06 */
static int hf_rrc_targetCellPreconfigInfo = -1;   /* TargetCellPreconfigInfo */
static int hf_rrc_dl_dpchInfo_07 = -1;            /* T_dl_dpchInfo_07 */
static int hf_rrc_targetCellPreconfigInfo_01 = -1;  /* TargetCellPreconfigInfo_r9 */
static int hf_rrc_dl_dpchInfo_08 = -1;            /* T_dl_dpchInfo_08 */
static int hf_rrc_targetCellPreconfigInfo_02 = -1;  /* TargetCellPreconfigInfo_r10 */
static int hf_rrc_targetCellPreconfigInfo_03 = -1;  /* TargetCellPreconfigInfo_vbxyext */
static int hf_rrc_dl_dpchInfo_09 = -1;            /* T_dl_dpchInfo_09 */
static int hf_rrc_targetCellPreconfigInfo_04 = -1;  /* TargetCellPreconfigInfo_r11 */
static int hf_rrc_targetCellPreconfigInfo_05 = -1;  /* TargetCellPreconfigInfo_v890ext */
static int hf_rrc_targetCellPreconfigInfo_06 = -1;  /* TargetCellPreconfigInfo_v950ext */
static int hf_rrc_RL_AdditionInformationList_item = -1;  /* RL_AdditionInformation */
static int hf_rrc_RL_AdditionInformationList_r6_item = -1;  /* RL_AdditionInformation_r6 */
static int hf_rrc_RL_AdditionInformationList_r7_item = -1;  /* RL_AdditionInformation_r7 */
static int hf_rrc_RL_AdditionInformation_list_v6b0ext_item = -1;  /* RL_AdditionInformation_v6b0ext */
static int hf_rrc_RL_AdditionInformationList_r8_item = -1;  /* RL_AdditionInformation_r8 */
static int hf_rrc_RL_AdditionInformationList_v890ext_item = -1;  /* RL_AdditionInformation_v890ext */
static int hf_rrc_RL_AdditionInformationList_r9_item = -1;  /* RL_AdditionInformation_r9 */
static int hf_rrc_RL_AdditionInformationList_v950ext_item = -1;  /* RL_AdditionInformation_v950ext */
static int hf_rrc_RL_AdditionInformationList_r10_item = -1;  /* RL_AdditionInformation_r10 */
static int hf_rrc_RL_AdditionInformationList_vbxyext_item = -1;  /* RL_AdditionInformation_vbxyext */
static int hf_rrc_RL_AdditionInformationList_r11_item = -1;  /* RL_AdditionInformation_r11 */
static int hf_rrc_RL_AdditionInformationList_SecULFreq_item = -1;  /* RL_AdditionInformation_SecULFreq */
static int hf_rrc_RL_IdentifierList_item = -1;    /* PrimaryCPICH_Info */
static int hf_rrc_RL_RemovalInformationList_item = -1;  /* PrimaryCPICH_Info */
static int hf_rrc_RL_RemovalInformationList_SecULFreq_item = -1;  /* PrimaryCPICH_Info */
static int hf_rrc_SCCPCH_ChannelisationCodeList_item = -1;  /* SCCPCH_ChannelisationCode */
static int hf_rrc_SCCPCH_ChannelisationCodeList_VHCR_item = -1;  /* SCCPCH_ChannelisationCode_VHCR */
static int hf_rrc_secondaryCCPCH_Info = -1;       /* SecondaryCCPCH_Info */
static int hf_rrc_modeSpecificInfo_74 = -1;       /* T_modeSpecificInfo_74 */
static int hf_rrc_fdd_110 = -1;                   /* T_fdd_107 */
static int hf_rrc_fach_PCH_InformationList = -1;  /* FACH_PCH_InformationList */
static int hf_rrc_sib_ReferenceListFACH = -1;     /* SIB_ReferenceListFACH */
static int hf_rrc_tdd_67 = -1;                    /* T_tdd_61 */
static int hf_rrc_secondaryCCPCH_Info_01 = -1;    /* SecondaryCCPCH_Info_r4 */
static int hf_rrc_modeSpecificInfo_75 = -1;       /* T_modeSpecificInfo_75 */
static int hf_rrc_fdd_111 = -1;                   /* T_fdd_108 */
static int hf_rrc_pich_Info = -1;                 /* PICH_Info */
static int hf_rrc_secondaryCCPCH_LCR_Extensions = -1;  /* SecondaryCCPCH_Info_LCR_r4_ext */
static int hf_rrc_pich_Info_01 = -1;              /* PICH_Info_LCR_r4 */
static int hf_rrc_secondaryCCPCH_Info_02 = -1;    /* SecondaryCCPCH_Info_HCR_VHCR_r7 */
static int hf_rrc_pich_Info_02 = -1;              /* PICH_Info_HCR_VHCR_r7 */
static int hf_rrc_SCCPCH_SystemInformationList_item = -1;  /* SCCPCH_SystemInformation */
static int hf_rrc_SCCPCH_SystemInformationList_HCR_VHCR_r7_item = -1;  /* SCCPCH_SystemInformation_HCR_VHCR_r7 */
static int hf_rrc_SCCPCH_SystemInformationList_LCR_r4_ext_item = -1;  /* SCCPCH_SystemInformation_LCR_r4_ext */
static int hf_rrc_secondaryCCPCHInfo_MBMS = -1;   /* SecondaryCCPCHInfo_MBMS_r6 */
static int hf_rrc_transportFormatCombinationSet = -1;  /* TFCS */
static int hf_rrc_fachCarryingMCCH = -1;          /* T_fachCarryingMCCH */
static int hf_rrc_mcch_transportFormatSet = -1;   /* TransportFormatSet */
static int hf_rrc_mcch_ConfigurationInfo = -1;    /* MBMS_MCCH_ConfigurationInfo_r6 */
static int hf_rrc_fachCarryingMTCH_List = -1;     /* MBMS_FACHCarryingMTCH_List */
static int hf_rrc_fachCarryingMSCH = -1;          /* T_fachCarryingMSCH */
static int hf_rrc_msch_transportFormatSet = -1;   /* TransportFormatSet */
static int hf_rrc_dummy_45 = -1;                  /* MBMS_MSCH_ConfigurationInfo_r6 */
static int hf_rrc_secondaryCCPCHInfo_MBMS_01 = -1;  /* SecondaryCCPCHInfo_MBMS_r7 */
static int hf_rrc_fachCarryingMCCH_01 = -1;       /* T_fachCarryingMCCH_01 */
static int hf_rrc_fachCarryingMSCH_01 = -1;       /* T_fachCarryingMSCH_01 */
static int hf_rrc_common_E_DCH_ResourceInfoListExt = -1;  /* SEQUENCE_SIZE_1_maxEDCHs_OF_Common_E_DCH_ResourceInfoListExt */
static int hf_rrc_common_E_DCH_ResourceInfoListExt_item = -1;  /* Common_E_DCH_ResourceInfoListExt */
static int hf_rrc_modeSpecificInfo_76 = -1;       /* T_modeSpecificInfo_76 */
static int hf_rrc_fdd_112 = -1;                   /* T_fdd_109 */
static int hf_rrc_dummy1_03 = -1;                 /* PCPICH_UsageForChannelEst */
static int hf_rrc_dummy2_09 = -1;                 /* SecondaryCPICH_Info */
static int hf_rrc_sf_AndCodeNumber_01 = -1;       /* SF256_AndCodeNumber */
static int hf_rrc_pilotSymbolExistence = -1;      /* BOOLEAN */
static int hf_rrc_timingOffset = -1;              /* TimingOffset */
static int hf_rrc_tdd_68 = -1;                    /* T_tdd_62 */
static int hf_rrc_commonTimeslotInfo_01 = -1;     /* CommonTimeslotInfoSCCPCH */
static int hf_rrc_channelisationCode_09 = -1;     /* SCCPCH_ChannelisationCodeList */
static int hf_rrc_modeSpecificInfo_77 = -1;       /* T_modeSpecificInfo_77 */
static int hf_rrc_fdd_113 = -1;                   /* T_fdd_110 */
static int hf_rrc_tdd_69 = -1;                    /* T_tdd_63 */
static int hf_rrc_tddOption_18 = -1;              /* T_tddOption_18 */
static int hf_rrc_tdd384_33 = -1;                 /* T_tdd384_30 */
static int hf_rrc_tdd128_51 = -1;                 /* T_tdd128_50 */
static int hf_rrc_modeSpecificInfo_78 = -1;       /* T_modeSpecificInfo_78 */
static int hf_rrc_tdd384_34 = -1;                 /* T_tdd384_31 */
static int hf_rrc_tdd768_21 = -1;                 /* T_tdd768_18 */
static int hf_rrc_channelisationCode_10 = -1;     /* SCCPCH_ChannelisationCodeList_VHCR */
static int hf_rrc_individualTimeslotLCR_Ext = -1;  /* IndividualTimeslotInfo_LCR_r4_ext */
static int hf_rrc_subFrameNumber = -1;            /* INTEGER_0_4 */
static int hf_rrc_dl_ChannelisationCodes = -1;    /* DL_ChannelCodes_MBSFN_IMB384 */
static int hf_rrc_modulation_03 = -1;             /* T_modulation_03 */
static int hf_rrc_modQPSK = -1;                   /* NULL */
static int hf_rrc_mod16QAM = -1;                  /* T_mod16QAM */
static int hf_rrc_cpich_SecCCPCH_PowerOffset = -1;  /* INTEGER_M11_4 */
static int hf_rrc_modeSpecificInfo_79 = -1;       /* T_modeSpecificInfo_79 */
static int hf_rrc_fdd_114 = -1;                   /* T_fdd_111 */
static int hf_rrc_tdd384_35 = -1;                 /* T_tdd384_32 */
static int hf_rrc_commonTimeslotInfoMBMS = -1;    /* CommonTimeslotInfoMBMS */
static int hf_rrc_downlinkTimeslotsCodes = -1;    /* DownlinkTimeslotsCodes */
static int hf_rrc_tdd128_52 = -1;                 /* T_tdd128_51 */
static int hf_rrc_downlinkTimeslotsCodes_01 = -1;  /* DownlinkTimeslotsCodes_LCR_r4 */
static int hf_rrc_modeSpecificInfo_80 = -1;       /* T_modeSpecificInfo_80 */
static int hf_rrc_fdd_115 = -1;                   /* T_fdd_112 */
static int hf_rrc_modulation_04 = -1;             /* T_modulation_04 */
static int hf_rrc_mod16QAM_01 = -1;               /* INTEGER_M11_4 */
static int hf_rrc_tdd384_36 = -1;                 /* T_tdd384_33 */
static int hf_rrc_downlinkTimeslotsCodes_02 = -1;  /* DownlinkTimeslotsCodes_r7 */
static int hf_rrc_modulation_05 = -1;             /* T_modulation_05 */
static int hf_rrc_tdd768_22 = -1;                 /* T_tdd768_19 */
static int hf_rrc_downlinkTimeslotsCodes_03 = -1;  /* DownlinkTimeslotsCodes_VHCR */
static int hf_rrc_modulation_06 = -1;             /* T_modulation_06 */
static int hf_rrc_tdd128_53 = -1;                 /* T_tdd128_52 */
static int hf_rrc_mbsfnSpecialTimeSlot = -1;      /* TimeSlotLCR_ext */
static int hf_rrc_modulation_07 = -1;             /* T_modulation_07 */
static int hf_rrc_secondaryDL_ScramblingCode = -1;  /* SecondaryScramblingCode */
static int hf_rrc_newConfiguration_08 = -1;       /* T_newConfiguration_08 */
static int hf_rrc_precodingWeightSetRestriction_03 = -1;  /* T_precodingWeightSetRestriction_02 */
static int hf_rrc_newConfiguration_09 = -1;       /* T_newConfiguration_09 */
static int hf_rrc_precodingWeightSetRestriction_04 = -1;  /* T_precodingWeightSetRestriction_03 */
static int hf_rrc_newConfiguration_10 = -1;       /* T_newConfiguration_10 */
static int hf_rrc_mimoPilotConfiguration_02 = -1;  /* MIMO4x4_PilotConfiguration */
static int hf_rrc_servingCellChangeMACreset = -1;  /* ServingCellChangeMACreset */
static int hf_rrc_servingCellChangeMsgType = -1;  /* ServingCellChangeMsgType */
static int hf_rrc_servingCellChangeTrId = -1;     /* ServingCellChangeTrId */
static int hf_rrc_sf4 = -1;                       /* INTEGER_0_3 */
static int hf_rrc_sf8 = -1;                       /* INTEGER_0_7 */
static int hf_rrc_sf16 = -1;                      /* INTEGER_0_15 */
static int hf_rrc_sf32 = -1;                      /* INTEGER_0_31 */
static int hf_rrc_sf64 = -1;                      /* INTEGER_0_63 */
static int hf_rrc_sf128 = -1;                     /* INTEGER_0_127 */
static int hf_rrc_sf256 = -1;                     /* INTEGER_0_255 */
static int hf_rrc_sf512 = -1;                     /* INTEGER_0_511 */
static int hf_rrc_sfd4 = -1;                      /* NULL */
static int hf_rrc_sfd8 = -1;                      /* NULL */
static int hf_rrc_sfd16 = -1;                     /* NULL */
static int hf_rrc_sfd32 = -1;                     /* NULL */
static int hf_rrc_sfd64 = -1;                     /* NULL */
static int hf_rrc_sfd128 = -1;                    /* PilotBits128 */
static int hf_rrc_sfd256 = -1;                    /* PilotBits256 */
static int hf_rrc_sfd512 = -1;                    /* NULL */
static int hf_rrc_activationTimeSFN = -1;         /* INTEGER_0_4095 */
static int hf_rrc_physChDuration = -1;            /* DurationTimeInfo */
static int hf_rrc_e_dch_SPS_Info = -1;            /* E_DCH_SPS_Information_TDD128 */
static int hf_rrc_hs_dsch_SPS_Info = -1;          /* HS_DSCH_SPS_Information_TDD128 */
static int hf_rrc_s_Field = -1;                   /* S_Field */
static int hf_rrc_codeWordSet = -1;               /* CodeWordSet */
static int hf_rrc_ssdt_UL_r4 = -1;                /* SSDT_UL */
static int hf_rrc_timeSlotInformation = -1;       /* BIT_STRING_SIZE_5 */
static int hf_rrc_repetitionPeriod = -1;          /* T_repetitionPeriod */
static int hf_rrc_referenceBeta = -1;             /* INTEGER_M15_16 */
static int hf_rrc_sync_UL_CodesBitmap = -1;       /* T_sync_UL_CodesBitmap */
static int hf_rrc_prxUpPCHdes = -1;               /* INTEGER_0_62 */
static int hf_rrc_sync_UL_Procedure = -1;         /* SYNC_UL_Procedure_r4 */
static int hf_rrc_max_SYNC_UL_Transmissions = -1;  /* T_max_SYNC_UL_Transmissions */
static int hf_rrc_powerRampStep_01 = -1;          /* INTEGER_0_3 */
static int hf_rrc_sync_UL_Codes_Bitmap = -1;      /* Sync_UL_Codes_Bitmap */
static int hf_rrc_max_SYNC_UL_Transmissions_01 = -1;  /* T_max_SYNC_UL_Transmissions_01 */
static int hf_rrc_max_SYNC_UL_Transmissions_02 = -1;  /* T_max_SYNC_UL_Transmissions_02 */
static int hf_rrc_activationTimeOffset = -1;      /* ActivationTimeOffset */
static int hf_rrc_uL_OLTD_InfoFDD = -1;           /* UL_OLTD_InfoFDD */
static int hf_rrc_ul_MIMO_Info = -1;              /* UL_MIMO_InfoFDD */
static int hf_rrc_TDD_MBSFNInformation_item = -1;  /* TDD_MBSFNTSlotInfo */
static int hf_rrc_timeSlotNumber = -1;            /* TimeslotNumber */
static int hf_rrc_sf8_01 = -1;                    /* SEQUENCE_SIZE_1_8_OF_TDD_PRACH_CCode8 */
static int hf_rrc_sf8_item = -1;                  /* TDD_PRACH_CCode8 */
static int hf_rrc_sf16_01 = -1;                   /* SEQUENCE_SIZE_1_8_OF_TDD_PRACH_CCode16 */
static int hf_rrc_sf16_item = -1;                 /* TDD_PRACH_CCode16 */
static int hf_rrc_sf32_01 = -1;                   /* SEQUENCE_SIZE_1_16_OF_TDD768_PRACH_CCode32 */
static int hf_rrc_sf32_item = -1;                 /* TDD768_PRACH_CCode32 */
static int hf_rrc_sf16_02 = -1;                   /* SEQUENCE_SIZE_1_16_OF_TDD768_PRACH_CCode16 */
static int hf_rrc_sf16_item_01 = -1;              /* TDD768_PRACH_CCode16 */
static int hf_rrc_tgpsi = -1;                     /* TGPSI */
static int hf_rrc_tgps_Status = -1;               /* T_tgps_Status */
static int hf_rrc_activate = -1;                  /* T_activate */
static int hf_rrc_tgcfn = -1;                     /* TGCFN */
static int hf_rrc_deactivate = -1;                /* NULL */
static int hf_rrc_tgps_ConfigurationParams = -1;  /* TGPS_ConfigurationParams */
static int hf_rrc_tgps_Status_01 = -1;            /* T_tgps_Status_01 */
static int hf_rrc_activate_01 = -1;               /* T_activate_01 */
static int hf_rrc_tgps_ConfigurationParams_01 = -1;  /* TGPS_ConfigurationParams_r8 */
static int hf_rrc_tgps_Status_02 = -1;            /* T_tgps_Status_02 */
static int hf_rrc_activate_02 = -1;               /* T_activate_02 */
static int hf_rrc_tgps_ConfigurationParams_02 = -1;  /* TGPS_ConfigurationParams_r10 */
static int hf_rrc_TGP_SequenceList_item = -1;     /* TGP_Sequence */
static int hf_rrc_TGP_SequenceList_r8_item = -1;  /* TGP_Sequence_r8 */
static int hf_rrc_TGP_SequenceList_r10_item = -1;  /* TGP_Sequence_r10 */
static int hf_rrc_tgps_Status_03 = -1;            /* T_tgps_Status_03 */
static int hf_rrc_activate_03 = -1;               /* T_activate_03 */
static int hf_rrc_tgps_Status_04 = -1;            /* T_tgps_Status_04 */
static int hf_rrc_activate_04 = -1;               /* T_activate_04 */
static int hf_rrc_freqSpecificCompressedMode_01 = -1;  /* T_freqSpecificCompressedMode_01 */
static int hf_rrc_tgmp = -1;                      /* TGMP */
static int hf_rrc_tgprc = -1;                     /* TGPRC */
static int hf_rrc_tgsn = -1;                      /* TGSN */
static int hf_rrc_tgl1 = -1;                      /* TGL */
static int hf_rrc_tgl2 = -1;                      /* TGL */
static int hf_rrc_tgd = -1;                       /* TGD */
static int hf_rrc_tgpl1 = -1;                     /* TGPL */
static int hf_rrc_dummy_46 = -1;                  /* TGPL */
static int hf_rrc_rpp = -1;                       /* RPP */
static int hf_rrc_itp = -1;                       /* ITP */
static int hf_rrc_ul_DL_Mode = -1;                /* UL_DL_Mode */
static int hf_rrc_dl_FrameType = -1;              /* DL_FrameType */
static int hf_rrc_deltaSIR1 = -1;                 /* DeltaSIR */
static int hf_rrc_deltaSIRAfter1 = -1;            /* DeltaSIR */
static int hf_rrc_deltaSIR2 = -1;                 /* DeltaSIR */
static int hf_rrc_deltaSIRAfter2 = -1;            /* DeltaSIR */
static int hf_rrc_nidentifyAbort = -1;            /* NidentifyAbort */
static int hf_rrc_treconfirmAbort = -1;           /* TreconfirmAbort */
static int hf_rrc_tgmp_01 = -1;                   /* TGMP_r8 */
static int hf_rrc_freqSpecificCompressedMode_02 = -1;  /* T_freqSpecificCompressedMode_02 */
static int hf_rrc_durationTimeInfo = -1;          /* DurationTimeInfo */
static int hf_rrc_TimeslotList_item = -1;         /* TimeslotNumber */
static int hf_rrc_tdd384_37 = -1;                 /* SEQUENCE_SIZE_1_maxTS_OF_TimeslotNumber */
static int hf_rrc_tdd384_item = -1;               /* TimeslotNumber */
static int hf_rrc_tdd128_54 = -1;                 /* SEQUENCE_SIZE_1_maxTS_LCR_OF_TimeslotNumber_LCR_r4 */
static int hf_rrc_tdd128_item_01 = -1;            /* TimeslotNumber_LCR_r4 */
static int hf_rrc_timing = -1;                    /* T_timing */
static int hf_rrc_newTiming = -1;                 /* NewTiming */
static int hf_rrc_harqProcessAllocation = -1;     /* BIT_STRING_SIZE_8 */
static int hf_rrc_channelReqParamsForUCSM = -1;   /* ChannelReqParamsForUCSM */
static int hf_rrc_ul_16QAM_Settings = -1;         /* UL_16QAM_Settings */
static int hf_rrc_mac_es_e_resetIndicator = -1;   /* T_mac_es_e_resetIndicator */
static int hf_rrc_beta_Ed_Gain_E_AGCH_Table_Selection = -1;  /* INTEGER_0_1 */
static int hf_rrc_ul_64QAM_Settings = -1;         /* UL_64QAM_Settings */
static int hf_rrc_mac_is_i_resetIndicator = -1;   /* T_mac_is_i_resetIndicator */
static int hf_rrc_ul_CCTrCH_TimeslotsCodes = -1;  /* UplinkTimeslotsCodes */
static int hf_rrc_tddOption_19 = -1;              /* T_tddOption_19 */
static int hf_rrc_tdd384_38 = -1;                 /* T_tdd384_34 */
static int hf_rrc_tdd128_55 = -1;                 /* T_tdd128_53 */
static int hf_rrc_ul_CCTrCH_TimeslotsCodes_01 = -1;  /* UplinkTimeslotsCodes_LCR_r4 */
static int hf_rrc_tddOption_20 = -1;              /* T_tddOption_20 */
static int hf_rrc_tdd384_39 = -1;                 /* T_tdd384_35 */
static int hf_rrc_tdd768_23 = -1;                 /* T_tdd768_20 */
static int hf_rrc_ul_CCTrCH_TimeslotsCodes_02 = -1;  /* UplinkTimeslotsCodes_VHCR */
static int hf_rrc_tdd128_56 = -1;                 /* T_tdd128_54 */
static int hf_rrc_ul_CCTrCH_TimeslotsCodes_03 = -1;  /* UplinkTimeslotsCodes_LCR_r7 */
static int hf_rrc_UL_CCTrCHList_item = -1;        /* UL_CCTrCH */
static int hf_rrc_UL_CCTrCHList_r4_item = -1;     /* UL_CCTrCH_r4 */
static int hf_rrc_UL_CCTrCHList_r7_item = -1;     /* UL_CCTrCH_r7 */
static int hf_rrc_UL_CCTrCHListToRemove_item = -1;  /* TFCS_IdentityPlain */
static int hf_rrc_UL_CCTrChTPCList_item = -1;     /* TFCS_Identity */
static int hf_rrc_dummy_47 = -1;                  /* CPCH_SetInfo */
static int hf_rrc_dummy1_04 = -1;                 /* CPCH_SetInfo */
static int hf_rrc_dummy2_10 = -1;                 /* CPCH_SetID */
static int hf_rrc_configurationInfo_04 = -1;      /* T_configurationInfo_04 */
static int hf_rrc_newConfiguration_11 = -1;       /* T_newConfiguration_11 */
static int hf_rrc_s_dpcch_Info = -1;              /* S_DPCCH_Information */
static int hf_rrc_initial_CLTD_ActivationState = -1;  /* T_initial_CLTD_ActivationState */
static int hf_rrc_configurationInfo_05 = -1;      /* T_configurationInfo_05 */
static int hf_rrc_newConfiguration_12 = -1;       /* T_newConfiguration_12 */
static int hf_rrc_s_e_dpcch_power_offset = -1;    /* INTEGER_0_17 */
static int hf_rrc_rank2_minimum_E_TFCI = -1;      /* E_DCH_MinimumSet_E_TFCI */
static int hf_rrc_inter_stream_compensation = -1;  /* INTEGER_0_15 */
static int hf_rrc_ul = -1;                        /* UL_CompressedModeMethod */
static int hf_rrc_dl = -1;                        /* DL_CompressedModeMethod */
static int hf_rrc_ul_and_dl = -1;                 /* T_ul_and_dl */
static int hf_rrc_ul_DPCCHscramblingCodeType = -1;  /* ScramblingCodeType */
static int hf_rrc_ul_DPCCHscramblingCode = -1;    /* UL_ScramblingCode */
static int hf_rrc_modeSpecificInfo_81 = -1;       /* T_modeSpecificInfo_81 */
static int hf_rrc_fdd_116 = -1;                   /* T_fdd_113 */
static int hf_rrc_scramblingCode = -1;            /* UL_ScramblingCode */
static int hf_rrc_numberOfDPDCH = -1;             /* NumberOfDPDCH */
static int hf_rrc_spreadingFactor_01 = -1;        /* SpreadingFactor */
static int hf_rrc_numberOfFBI_Bits = -1;          /* NumberOfFBI_Bits */
static int hf_rrc_tdd_70 = -1;                    /* T_tdd_64 */
static int hf_rrc_ul_CCTrCHList = -1;             /* UL_CCTrCHList */
static int hf_rrc_ul_CCTrCHListToRemove = -1;     /* UL_CCTrCHListToRemove */
static int hf_rrc_modeSpecificInfo_82 = -1;       /* T_modeSpecificInfo_82 */
static int hf_rrc_fdd_117 = -1;                   /* T_fdd_114 */
static int hf_rrc_tdd_71 = -1;                    /* T_tdd_65 */
static int hf_rrc_ul_CCTrCHList_01 = -1;          /* UL_CCTrCHList_r4 */
static int hf_rrc_modeSpecificInfo_83 = -1;       /* T_modeSpecificInfo_83 */
static int hf_rrc_fdd_118 = -1;                   /* T_fdd_115 */
static int hf_rrc_tdd_72 = -1;                    /* T_tdd_66 */
static int hf_rrc_ul_DPCH_PowerControlInfo_04 = -1;  /* UL_DPCH_PowerControlInfo_r6 */
static int hf_rrc_modeSpecificInfo_84 = -1;       /* T_modeSpecificInfo_84 */
static int hf_rrc_fdd_119 = -1;                   /* T_fdd_116 */
static int hf_rrc_dpdchPresence = -1;             /* T_dpdchPresence */
static int hf_rrc_present_04 = -1;                /* T_present_02 */
static int hf_rrc_notPresent = -1;                /* T_notPresent */
static int hf_rrc_tdd_73 = -1;                    /* T_tdd_67 */
static int hf_rrc_modeSpecificInfo_85 = -1;       /* T_modeSpecificInfo_85 */
static int hf_rrc_fdd_120 = -1;                   /* T_fdd_117 */
static int hf_rrc_dpdchPresence_01 = -1;          /* T_dpdchPresence_01 */
static int hf_rrc_present_05 = -1;                /* T_present_03 */
static int hf_rrc_numberOfTPC_Bits = -1;          /* NumberOfTPC_Bits */
static int hf_rrc_notPresent_01 = -1;             /* T_notPresent_01 */
static int hf_rrc_tdd_74 = -1;                    /* T_tdd_68 */
static int hf_rrc_ul_CCTrCHList_02 = -1;          /* UL_CCTrCHList_r7 */
static int hf_rrc_ul_DPCH_PowerControlInfo_05 = -1;  /* UL_DPCH_PowerControlInfo_r11 */
static int hf_rrc_modeSpecificInfo_86 = -1;       /* T_modeSpecificInfo_86 */
static int hf_rrc_fdd_121 = -1;                   /* T_fdd_118 */
static int hf_rrc_dpdchPresence_02 = -1;          /* T_dpdchPresence_02 */
static int hf_rrc_present_06 = -1;                /* T_present_04 */
static int hf_rrc_notPresent_02 = -1;             /* T_notPresent_02 */
static int hf_rrc_tdd_75 = -1;                    /* T_tdd_69 */
static int hf_rrc_ul_DPCH_PowerControlInfo_06 = -1;  /* UL_DPCH_PowerControlInfoPostFDD */
static int hf_rrc_reducedScramblingCodeNumber = -1;  /* ReducedScramblingCodeNumber */
static int hf_rrc_ul_DPCH_PowerControlInfo_07 = -1;  /* UL_DPCH_PowerControlInfoPostTDD */
static int hf_rrc_ul_DPCH_PowerControlInfo_08 = -1;  /* UL_DPCH_PowerControlInfoPostTDD_LCR_r4 */
static int hf_rrc_ul_TimingAdvance_04 = -1;       /* UL_TimingAdvanceControl_LCR_r4 */
static int hf_rrc_ul_DPCH_PowerControlInfo_09 = -1;  /* UL_DPCH_PowerControlInfoPredef */
static int hf_rrc_modeSpecificInfo_87 = -1;       /* T_modeSpecificInfo_87 */
static int hf_rrc_fdd_122 = -1;                   /* T_fdd_119 */
static int hf_rrc_tdd_76 = -1;                    /* T_tdd_70 */
static int hf_rrc_modeSpecificInfo_88 = -1;       /* T_modeSpecificInfo_88 */
static int hf_rrc_fdd_123 = -1;                   /* T_fdd_120 */
static int hf_rrc_fdd_124 = -1;                   /* T_fdd_121 */
static int hf_rrc_dpcch_PowerOffset = -1;         /* DPCCH_PowerOffset */
static int hf_rrc_sRB_delay = -1;                 /* SRB_delay */
static int hf_rrc_tdd_77 = -1;                    /* T_tdd_71 */
static int hf_rrc_ul_OL_PC_Signalling = -1;       /* T_ul_OL_PC_Signalling */
static int hf_rrc_broadcast_UL_OL_PC_info = -1;   /* NULL */
static int hf_rrc_individuallySignalled = -1;     /* T_individuallySignalled */
static int hf_rrc_individualTS_InterferenceList = -1;  /* IndividualTS_InterferenceList */
static int hf_rrc_fdd_125 = -1;                   /* T_fdd_122 */
static int hf_rrc_tdd_78 = -1;                    /* T_tdd_72 */
static int hf_rrc_ul_OL_PC_Signalling_01 = -1;    /* T_ul_OL_PC_Signalling_01 */
static int hf_rrc_individuallySignalled_01 = -1;  /* T_individuallySignalled_01 */
static int hf_rrc_tddOption_21 = -1;              /* T_tddOption_21 */
static int hf_rrc_tdd384_40 = -1;                 /* T_tdd384_36 */
static int hf_rrc_dpch_ConstantValue_01 = -1;     /* ConstantValue */
static int hf_rrc_tdd128_57 = -1;                 /* T_tdd128_55 */
static int hf_rrc_fdd_126 = -1;                   /* T_fdd_123 */
static int hf_rrc_ack_NACK_repetition_factor = -1;  /* ACK_NACK_repetitionFactor */
static int hf_rrc_tdd_79 = -1;                    /* T_tdd_73 */
static int hf_rrc_ul_OL_PC_Signalling_02 = -1;    /* T_ul_OL_PC_Signalling_02 */
static int hf_rrc_individuallySignalled_02 = -1;  /* T_individuallySignalled_02 */
static int hf_rrc_tddOption_22 = -1;              /* T_tddOption_22 */
static int hf_rrc_tdd384_41 = -1;                 /* T_tdd384_37 */
static int hf_rrc_tdd128_58 = -1;                 /* T_tdd128_56 */
static int hf_rrc_fdd_127 = -1;                   /* T_fdd_124 */
static int hf_rrc_tdd_80 = -1;                    /* T_tdd_74 */
static int hf_rrc_ul_OL_PC_Signalling_03 = -1;    /* T_ul_OL_PC_Signalling_03 */
static int hf_rrc_individuallySignalled_03 = -1;  /* T_individuallySignalled_03 */
static int hf_rrc_tddOption_23 = -1;              /* T_tddOption_23 */
static int hf_rrc_tdd384_42 = -1;                 /* T_tdd384_38 */
static int hf_rrc_tdd128_59 = -1;                 /* T_tdd128_57 */
static int hf_rrc_fdd_128 = -1;                   /* T_fdd_125 */
static int hf_rrc_tdd_81 = -1;                    /* T_tdd_75 */
static int hf_rrc_ul_OL_PC_Signalling_04 = -1;    /* T_ul_OL_PC_Signalling_04 */
static int hf_rrc_individuallySignalled_04 = -1;  /* T_individuallySignalled_04 */
static int hf_rrc_tddOption_24 = -1;              /* T_tddOption_24 */
static int hf_rrc_tdd384_43 = -1;                 /* T_tdd384_39 */
static int hf_rrc_tdd768_24 = -1;                 /* T_tdd768_21 */
static int hf_rrc_tdd128_60 = -1;                 /* T_tdd128_58 */
static int hf_rrc_fdd_129 = -1;                   /* T_fdd_126 */
static int hf_rrc_ack_NACK_repetition_factor_assisting = -1;  /* ACK_NACK_repetitionFactor */
static int hf_rrc_tdd_82 = -1;                    /* T_tdd_76 */
static int hf_rrc_ul_OL_PC_Signalling_05 = -1;    /* T_ul_OL_PC_Signalling_05 */
static int hf_rrc_individuallySignalled_05 = -1;  /* T_individuallySignalled_05 */
static int hf_rrc_tddOption_25 = -1;              /* T_tddOption_25 */
static int hf_rrc_tdd384_44 = -1;                 /* T_tdd384_40 */
static int hf_rrc_tdd768_25 = -1;                 /* T_tdd768_22 */
static int hf_rrc_tdd128_61 = -1;                 /* T_tdd128_59 */
static int hf_rrc_dpcch_PowerOffset_01 = -1;      /* DPCCH_PowerOffset2 */
static int hf_rrc_fdd_130 = -1;                   /* T_fdd_127 */
static int hf_rrc_tdd_83 = -1;                    /* T_tdd_77 */
static int hf_rrc_tdd_84 = -1;                    /* T_tdd_78 */
static int hf_rrc_non_ScheduledTransGrantInfo_02 = -1;  /* Non_ScheduledTransGrantInfoTDD_ext */
static int hf_rrc_mac_es_e_resetIndicator_01 = -1;  /* T_mac_es_e_resetIndicator_01 */
static int hf_rrc_e_DPCCH_Info = -1;              /* E_DPCCH_Info */
static int hf_rrc_e_DPDCH_Info = -1;              /* E_DPDCH_Info */
static int hf_rrc_schedulingTransmConfiguration = -1;  /* E_DPDCH_SchedulingTransmConfiguration */
static int hf_rrc_mac_es_e_resetIndicator_02 = -1;  /* T_mac_es_e_resetIndicator_02 */
static int hf_rrc_modeSpecificInfo_89 = -1;       /* T_modeSpecificInfo_89 */
static int hf_rrc_fdd_131 = -1;                   /* T_fdd_128 */
static int hf_rrc_e_DPCCH_Info_01 = -1;           /* E_DPCCH_Info_r7 */
static int hf_rrc_e_DPDCH_Info_01 = -1;           /* E_DPDCH_Info_r7 */
static int hf_rrc_tdd_85 = -1;                    /* T_tdd_79 */
static int hf_rrc_e_RUCCH_Info_01 = -1;           /* E_RUCCH_Info */
static int hf_rrc_e_PUCH_Info_01 = -1;            /* E_PUCH_Info */
static int hf_rrc_non_ScheduledTransGrantInfo_03 = -1;  /* Non_ScheduledTransGrantInfoTDD */
static int hf_rrc_mac_es_e_resetIndicator_03 = -1;  /* T_mac_es_e_resetIndicator_03 */
static int hf_rrc_modeSpecificInfo_90 = -1;       /* T_modeSpecificInfo_90 */
static int hf_rrc_fdd_132 = -1;                   /* T_fdd_129 */
static int hf_rrc_e_DPDCH_Info_02 = -1;           /* E_DPDCH_Info_r8 */
static int hf_rrc_tdd_86 = -1;                    /* T_tdd_80 */
static int hf_rrc_mac_es_e_resetIndicator_04 = -1;  /* T_mac_es_e_resetIndicator_04 */
static int hf_rrc_modeSpecificInfo_91 = -1;       /* T_modeSpecificInfo_91 */
static int hf_rrc_fdd_133 = -1;                   /* T_fdd_130 */
static int hf_rrc_tdd_87 = -1;                    /* T_tdd_81 */
static int hf_rrc_non_ScheduledTransGrantInfo_04 = -1;  /* Non_ScheduledTransGrantInfoTDD_r9 */
static int hf_rrc_mac_es_e_resetIndicator_05 = -1;  /* T_mac_es_e_resetIndicator_05 */
static int hf_rrc_modeSpecificInfo_92 = -1;       /* T_modeSpecificInfo_92 */
static int hf_rrc_fdd_134 = -1;                   /* T_fdd_131 */
static int hf_rrc_tdd_88 = -1;                    /* T_tdd_82 */
static int hf_rrc_e_PUCH_info = -1;               /* E_PUCH_Info_MulticarrierEDCH_TDD128 */
static int hf_rrc_e_AGCH_info = -1;               /* E_AGCH_Information_TDD128 */
static int hf_rrc_e_HICH_info = -1;               /* E_HICH_Information_TDD128 */
static int hf_rrc_UL_MulticarrierEDCH_Infolist_TDD128_item = -1;  /* UL_MulticarrierEDCH_InfolistItem_TDD128 */
static int hf_rrc_UL_MulticarrierEDCH_Deletelist_TDD128_item = -1;  /* UARFCN */
static int hf_rrc_configuration_info = -1;        /* T_configuration_info */
static int hf_rrc_newConfiguration_13 = -1;       /* UL_MulticarrierEDCH_NewConfigurationInfo_TDD128 */
static int hf_rrc_tsn_Length_01 = -1;             /* T_tsn_Length_01 */
static int hf_rrc_ul_MulticarrierEDCH_Infolist = -1;  /* UL_MulticarrierEDCH_Infolist_TDD128 */
static int hf_rrc_ul_MulticarrierEDCH_Deletelist = -1;  /* UL_MulticarrierEDCH_Deletelist_TDD128 */
static int hf_rrc_stepSize = -1;                  /* INTEGER_1_8 */
static int hf_rrc_frequency = -1;                 /* INTEGER_1_8 */
static int hf_rrc_disabled = -1;                  /* NULL */
static int hf_rrc_enabled = -1;                   /* T_enabled */
static int hf_rrc_enabled_01 = -1;                /* T_enabled_01 */
static int hf_rrc_tddOption_26 = -1;              /* T_tddOption_26 */
static int hf_rrc_tdd384_45 = -1;                 /* T_tdd384_41 */
static int hf_rrc_tdd128_62 = -1;                 /* T_tdd128_60 */
static int hf_rrc_synchronisationParameters = -1;  /* SynchronisationParameters_r4 */
static int hf_rrc_enabled_02 = -1;                /* T_enabled_02 */
static int hf_rrc_tddOption_27 = -1;              /* T_tddOption_27 */
static int hf_rrc_tdd384_46 = -1;                 /* T_tdd384_42 */
static int hf_rrc_tdd768_26 = -1;                 /* T_tdd768_23 */
static int hf_rrc_tdd128_63 = -1;                 /* T_tdd128_61 */
static int hf_rrc_enabled_03 = -1;                /* T_enabled_03 */
static int hf_rrc_UL_TS_ChannelisationCodeList_item = -1;  /* UL_TS_ChannelisationCode */
static int hf_rrc_UL_TS_ChannelisationCodeList_r7_item = -1;  /* UL_TS_ChannelisationCodeList_r7_item */
static int hf_rrc_ul_TS_Channelisation_Code = -1;  /* UL_TS_ChannelisationCode */
static int hf_rrc_plcch_info = -1;                /* PLCCH_Info */
static int hf_rrc_UL_TS_ChannelisationCodeList_VHCR_item = -1;  /* UL_TS_ChannelisationCode_VHCR */
static int hf_rrc_parameters_04 = -1;             /* T_parameters_04 */
static int hf_rrc_sameAsLast_04 = -1;             /* T_sameAsLast_04 */
static int hf_rrc_newParameters_04 = -1;          /* T_newParameters_04 */
static int hf_rrc_ul_TS_ChannelisationCodeList = -1;  /* UL_TS_ChannelisationCodeList */
static int hf_rrc_parameters_05 = -1;             /* T_parameters_05 */
static int hf_rrc_sameAsLast_05 = -1;             /* T_sameAsLast_05 */
static int hf_rrc_newParameters_05 = -1;          /* T_newParameters_05 */
static int hf_rrc_parameters_06 = -1;             /* T_parameters_06 */
static int hf_rrc_sameAsLast_06 = -1;             /* T_sameAsLast_06 */
static int hf_rrc_newParameters_06 = -1;          /* T_newParameters_06 */
static int hf_rrc_ul_TS_ChannelisationCodeList_01 = -1;  /* UL_TS_ChannelisationCodeList_r7 */
static int hf_rrc_parameters_07 = -1;             /* T_parameters_07 */
static int hf_rrc_sameAsLast_07 = -1;             /* T_sameAsLast_07 */
static int hf_rrc_newParameters_07 = -1;          /* T_newParameters_07 */
static int hf_rrc_dynamicSFusage = -1;            /* BOOLEAN */
static int hf_rrc_moreTimeslots_04 = -1;          /* T_moreTimeslots_04 */
static int hf_rrc_additionalTimeslots_04 = -1;    /* T_additionalTimeslots_04 */
static int hf_rrc_consecutive_04 = -1;            /* T_consecutive_02 */
static int hf_rrc_numAdditionalTimeslots = -1;    /* INTEGER_1_maxTS_1 */
static int hf_rrc_timeslotList_04 = -1;           /* SEQUENCE_SIZE_1_maxTS_1_OF_UplinkAdditionalTimeslots */
static int hf_rrc_timeslotList_item_04 = -1;      /* UplinkAdditionalTimeslots */
static int hf_rrc_ul_TS_ChannelisationCodeList_02 = -1;  /* UL_TS_ChannelisationCodeList_VHCR */
static int hf_rrc_moreTimeslots_05 = -1;          /* T_moreTimeslots_05 */
static int hf_rrc_additionalTimeslots_05 = -1;    /* T_additionalTimeslots_05 */
static int hf_rrc_consecutive_05 = -1;            /* T_consecutive_03 */
static int hf_rrc_timeslotList_05 = -1;           /* SEQUENCE_SIZE_1_maxTS_1_OF_UplinkAdditionalTimeslots_VHCR */
static int hf_rrc_timeslotList_item_05 = -1;      /* UplinkAdditionalTimeslots_VHCR */
static int hf_rrc_moreTimeslots_06 = -1;          /* T_moreTimeslots_06 */
static int hf_rrc_additionalTimeslots_06 = -1;    /* T_additionalTimeslots_06 */
static int hf_rrc_consecutive_06 = -1;            /* T_consecutive_04 */
static int hf_rrc_numAdditionalTimeslots_01 = -1;  /* INTEGER_1_maxTS_LCR_1 */
static int hf_rrc_timeslotList_06 = -1;           /* SEQUENCE_SIZE_1_maxTS_LCR_1_OF_UplinkAdditionalTimeslots_LCR_r4 */
static int hf_rrc_timeslotList_item_06 = -1;      /* UplinkAdditionalTimeslots_LCR_r4 */
static int hf_rrc_moreTimeslots_07 = -1;          /* T_moreTimeslots_07 */
static int hf_rrc_additionalTimeslots_07 = -1;    /* T_additionalTimeslots_07 */
static int hf_rrc_consecutive_07 = -1;            /* T_consecutive_05 */
static int hf_rrc_timeslotList_07 = -1;           /* SEQUENCE_SIZE_1_maxTS_LCR_1_OF_UplinkAdditionalTimeslots_LCR_r7 */
static int hf_rrc_timeslotList_item_07 = -1;      /* UplinkAdditionalTimeslots_LCR_r7 */
static int hf_rrc_satID = -1;                     /* SatID */
static int hf_rrc_doppler0thOrder = -1;           /* INTEGER_M2048_2047 */
static int hf_rrc_extraDopplerInfo = -1;          /* ExtraDopplerInfo */
static int hf_rrc_codePhase = -1;                 /* INTEGER_0_1022 */
static int hf_rrc_integerCodePhase = -1;          /* INTEGER_0_19 */
static int hf_rrc_gps_BitNumber = -1;             /* INTEGER_0_3 */
static int hf_rrc_codePhaseSearchWindow = -1;     /* CodePhaseSearchWindow */
static int hf_rrc_azimuthAndElevation = -1;       /* AzimuthAndElevation */
static int hf_rrc_azimuthAndElevation_01 = -1;    /* AzimuthAndElevation_va40ext */
static int hf_rrc_azimuthAndElevation_02 = -1;    /* AzimuthAndElevation_r10 */
static int hf_rrc_AcquisitionSatInfoList_item = -1;  /* AcquisitionSatInfo */
static int hf_rrc_AcquisitionSatInfoList_va40ext_item = -1;  /* AcquisitionSatInfo_va40ext */
static int hf_rrc_AcquisitionSatInfoList_r10_item = -1;  /* AcquisitionSatInfo_r10 */
static int hf_rrc_AdditionalMeasurementID_List_item = -1;  /* MeasurementIdentity */
static int hf_rrc_AdditionalMeasurementID_List_r9_item = -1;  /* MeasurementIdentity_r9 */
static int hf_rrc_dataID = -1;                    /* INTEGER_0_3 */
static int hf_rrc_e = -1;                         /* BIT_STRING_SIZE_16 */
static int hf_rrc_t_oa = -1;                      /* BIT_STRING_SIZE_8 */
static int hf_rrc_deltaI = -1;                    /* BIT_STRING_SIZE_16 */
static int hf_rrc_omegaDot = -1;                  /* BIT_STRING_SIZE_16 */
static int hf_rrc_satHealth = -1;                 /* BIT_STRING_SIZE_8 */
static int hf_rrc_a_Sqrt = -1;                    /* BIT_STRING_SIZE_24 */
static int hf_rrc_omega0 = -1;                    /* BIT_STRING_SIZE_24 */
static int hf_rrc_m0 = -1;                        /* BIT_STRING_SIZE_24 */
static int hf_rrc_omega = -1;                     /* BIT_STRING_SIZE_24 */
static int hf_rrc_af0 = -1;                       /* BIT_STRING_SIZE_11 */
static int hf_rrc_af1 = -1;                       /* BIT_STRING_SIZE_11 */
static int hf_rrc_AlmanacSatInfoList_item = -1;   /* AlmanacSatInfo */
static int hf_rrc_sat_info_SBASecefList = -1;     /* GANSS_SAT_Info_Almanac_SBASecefList */
static int hf_rrc_sat_info_GLOkpList = -1;        /* GANSS_SAT_Info_Almanac_GLOkpList */
static int hf_rrc_t_oa_01 = -1;                   /* INTEGER_0_147 */
static int hf_rrc_iod_a = -1;                     /* INTEGER_0_3 */
static int hf_rrc_sat_info_kpList = -1;           /* GANSS_SAT_Info_Almanac_KpList */
static int hf_rrc_sat_info_MIDIkpList = -1;       /* GANSS_SAT_Info_Almanac_MIDIkpList */
static int hf_rrc_sat_info_NAVkpList = -1;        /* GANSS_SAT_Info_Almanac_NAVkpList */
static int hf_rrc_sat_info_REDkpList = -1;        /* GANSS_SAT_Info_Almanac_REDkpList */
static int hf_rrc_AuxInfoGANSS_ID1_item = -1;     /* AuxInfoGANSS_ID1_element */
static int hf_rrc_svID = -1;                      /* INTEGER_0_63 */
static int hf_rrc_signalsAvailable = -1;          /* BIT_STRING_SIZE_8 */
static int hf_rrc_AuxInfoGANSS_ID3_item = -1;     /* AuxInfoGANSS_ID3_element */
static int hf_rrc_channelNumber = -1;             /* INTEGER_M7_13 */
static int hf_rrc_azimuth = -1;                   /* INTEGER_0_31 */
static int hf_rrc_elevation = -1;                 /* INTEGER_0_7 */
static int hf_rrc_azimuthLSB = -1;                /* INTEGER_0_15 */
static int hf_rrc_elevationLSB = -1;              /* INTEGER_0_15 */
static int hf_rrc_BadSatList_item = -1;           /* INTEGER_0_63 */
static int hf_rrc_dl_TransportChannelBLER = -1;   /* DL_TransportChannelBLER */
static int hf_rrc_BLER_MeasurementResultsList_item = -1;  /* BLER_MeasurementResults */
static int hf_rrc_BLER_TransChIdList_item = -1;   /* TransportChannelIdentity */
static int hf_rrc_verifiedBSIC = -1;              /* INTEGER_0_maxCellMeas */
static int hf_rrc_nonVerifiedBSIC = -1;           /* BCCH_ARFCN */
static int hf_rrc_burstStart = -1;                /* INTEGER_0_15 */
static int hf_rrc_burstLength = -1;               /* INTEGER_10_25 */
static int hf_rrc_burstFreq = -1;                 /* INTEGER_1_16 */
static int hf_rrc_intraFreqReportingCriteria = -1;  /* IntraFreqReportingCriteria */
static int hf_rrc_periodicalReportingCriteria = -1;  /* PeriodicalReportingCriteria */
static int hf_rrc_intraFreqReportingCriteria_01 = -1;  /* IntraFreqReportingCriteria_LCR_r4 */
static int hf_rrc_patternIdentifier = -1;         /* INTEGER_0_maxMeasOccasionPattern_1 */
static int hf_rrc_statusFlag = -1;                /* T_statusFlag */
static int hf_rrc_measurementPurpose = -1;        /* BIT_STRING_SIZE_5 */
static int hf_rrc_measurementOccasionPatternParameter = -1;  /* MeasurementOccasionPatternParameter */
static int hf_rrc_cellDCHMeasOccasionSequenceList = -1;  /* SEQUENCE_SIZE_1_maxMeasOccasionPattern_OF_CellDCHMeasOccasionPattern_LCR */
static int hf_rrc_cellDCHMeasOccasionSequenceList_item = -1;  /* CellDCHMeasOccasionPattern_LCR */
static int hf_rrc_cellIndividualOffset = -1;      /* CellIndividualOffset */
static int hf_rrc_referenceTimeDifferenceToCell = -1;  /* ReferenceTimeDifferenceToCell */
static int hf_rrc_modeSpecificInfo_93 = -1;       /* T_modeSpecificInfo_93 */
static int hf_rrc_fdd_135 = -1;                   /* T_fdd_132 */
static int hf_rrc_readSFN_Indicator = -1;         /* BOOLEAN */
static int hf_rrc_tdd_89 = -1;                    /* T_tdd_83 */
static int hf_rrc_primaryCCPCH_Info_02 = -1;      /* PrimaryCCPCH_Info */
static int hf_rrc_timeslotInfoList = -1;          /* TimeslotInfoList */
static int hf_rrc_modeSpecificInfo_94 = -1;       /* T_modeSpecificInfo_94 */
static int hf_rrc_fdd_136 = -1;                   /* T_fdd_133 */
static int hf_rrc_tdd_90 = -1;                    /* T_tdd_84 */
static int hf_rrc_primaryCCPCH_Info_03 = -1;      /* PrimaryCCPCH_Info_r4 */
static int hf_rrc_timeslotInfoList_01 = -1;       /* TimeslotInfoList_r4 */
static int hf_rrc_cellSelectionReselectionInfo = -1;  /* CellSelectReselectInfoMC_RSCP */
static int hf_rrc_modeSpecificInfo_95 = -1;       /* T_modeSpecificInfo_95 */
static int hf_rrc_fdd_137 = -1;                   /* T_fdd_134 */
static int hf_rrc_tdd_91 = -1;                    /* T_tdd_85 */
static int hf_rrc_CellInfoListToBeExcluded_r11_item = -1;  /* PrimaryCPICH_Info */
static int hf_rrc_modeSpecificInfo_96 = -1;       /* T_modeSpecificInfo_96 */
static int hf_rrc_fdd_138 = -1;                   /* T_fdd_135 */
static int hf_rrc_tdd_92 = -1;                    /* T_tdd_86 */
static int hf_rrc_cellSelectionReselectionInfo_01 = -1;  /* CellSelectReselectInfoSIB_11_12_RSCP */
static int hf_rrc_primaryCCPCH_Info_04 = -1;      /* PrimaryCCPCH_Info_LCR_r4 */
static int hf_rrc_timeslotInfoList_02 = -1;       /* TimeslotInfoList_LCR_r4 */
static int hf_rrc_modeSpecificInfo_97 = -1;       /* T_modeSpecificInfo_97 */
static int hf_rrc_fdd_139 = -1;                   /* T_fdd_136 */
static int hf_rrc_tdd_93 = -1;                    /* T_tdd_87 */
static int hf_rrc_cellSelectionReselectionInfo_02 = -1;  /* CellSelectReselectInfoSIB_11_12_ECN0 */
static int hf_rrc_modeSpecificInfo_98 = -1;       /* T_modeSpecificInfo_98 */
static int hf_rrc_fdd_140 = -1;                   /* T_fdd_137 */
static int hf_rrc_tdd_94 = -1;                    /* T_tdd_88 */
static int hf_rrc_cellSelectionReselectionInfo_03 = -1;  /* CellSelectReselectInfoSIB_11_12_HCS_RSCP */
static int hf_rrc_modeSpecificInfo_99 = -1;       /* T_modeSpecificInfo_99 */
static int hf_rrc_fdd_141 = -1;                   /* T_fdd_138 */
static int hf_rrc_tdd_95 = -1;                    /* T_tdd_89 */
static int hf_rrc_cellSelectionReselectionInfo_04 = -1;  /* CellSelectReselectInfoSIB_11_12_HCS_ECN0 */
static int hf_rrc_cellIdentity = -1;              /* CellIdentity */
static int hf_rrc_dummy_48 = -1;                  /* SFN_SFN_ObsTimeDifference */
static int hf_rrc_cellSynchronisationInfo = -1;   /* CellSynchronisationInfo */
static int hf_rrc_modeSpecificInfo_100 = -1;      /* T_modeSpecificInfo_100 */
static int hf_rrc_fdd_142 = -1;                   /* T_fdd_139 */
static int hf_rrc_cpich_Ec_N0_01 = -1;            /* CPICH_Ec_N0 */
static int hf_rrc_cpich_RSCP_01 = -1;             /* CPICH_RSCP */
static int hf_rrc_pathloss = -1;                  /* Pathloss */
static int hf_rrc_tdd_96 = -1;                    /* T_tdd_90 */
static int hf_rrc_proposedTGSN = -1;              /* TGSN */
static int hf_rrc_timeslotISCP_List = -1;         /* TimeslotISCP_List */
static int hf_rrc_modeSpecificInfo_101 = -1;      /* T_modeSpecificInfo_101 */
static int hf_rrc_fdd_143 = -1;                   /* T_fdd_140 */
static int hf_rrc_deltaRSCPPerCell = -1;          /* DeltaRSCPPerCell */
static int hf_rrc_tdd_97 = -1;                    /* T_tdd_91 */
static int hf_rrc_csgIdentity = -1;               /* CSG_Identity */
static int hf_rrc_csgMemberIndication = -1;       /* T_csgMemberIndication */
static int hf_rrc_fdd_144 = -1;                   /* SEQUENCE_SIZE_1_maxCellMeas_OF_PrimaryCPICH_Info */
static int hf_rrc_fdd_item_01 = -1;               /* PrimaryCPICH_Info */
static int hf_rrc_tdd_98 = -1;                    /* SEQUENCE_SIZE_1_maxCellMeas_OF_PrimaryCCPCH_Info */
static int hf_rrc_tdd_item_02 = -1;               /* PrimaryCCPCH_Info */
static int hf_rrc_fdd_145 = -1;                   /* SEQUENCE_SIZE_1_maxCellMeasOnSecULFreq_OF_PrimaryCPICH_Info */
static int hf_rrc_CellMeasurementEventResults_LCR_r4_item = -1;  /* PrimaryCCPCH_Info_LCR_r4 */
static int hf_rrc_dummy_49 = -1;                  /* SFN_SFN_OTD_Type */
static int hf_rrc_cellIdentity_reportingIndicator = -1;  /* BOOLEAN */
static int hf_rrc_cellSynchronisationInfoReportingIndicator = -1;  /* BOOLEAN */
static int hf_rrc_modeSpecificInfo_102 = -1;      /* T_modeSpecificInfo_102 */
static int hf_rrc_fdd_146 = -1;                   /* T_fdd_141 */
static int hf_rrc_cpich_Ec_N0_reportingIndicator = -1;  /* BOOLEAN */
static int hf_rrc_cpich_RSCP_reportingIndicator = -1;  /* BOOLEAN */
static int hf_rrc_pathloss_reportingIndicator = -1;  /* BOOLEAN */
static int hf_rrc_tdd_99 = -1;                    /* T_tdd_92 */
static int hf_rrc_timeslotISCP_reportingIndicator = -1;  /* BOOLEAN */
static int hf_rrc_proposedTGSN_ReportingRequired = -1;  /* BOOLEAN */
static int hf_rrc_primaryCCPCH_RSCP_reportingIndicator = -1;  /* BOOLEAN */
static int hf_rrc_q_OffsetS_N = -1;               /* Q_OffsetS_N */
static int hf_rrc_hcs_NeighbouringCellInformation_RSCP = -1;  /* HCS_NeighbouringCellInformation_RSCP */
static int hf_rrc_modeSpecificInfo_103 = -1;      /* T_modeSpecificInfo_103 */
static int hf_rrc_fdd_147 = -1;                   /* T_fdd_142 */
static int hf_rrc_tdd_100 = -1;                   /* T_tdd_93 */
static int hf_rrc_gsm_02 = -1;                    /* T_gsm_02 */
static int hf_rrc_q_Offset1S_N = -1;              /* Q_OffsetS_N */
static int hf_rrc_q_Offset2S_N = -1;              /* Q_OffsetS_N */
static int hf_rrc_modeSpecificInfo_104 = -1;      /* T_modeSpecificInfo_104 */
static int hf_rrc_fdd_148 = -1;                   /* T_fdd_143 */
static int hf_rrc_tdd_101 = -1;                   /* T_tdd_94 */
static int hf_rrc_gsm_03 = -1;                    /* T_gsm_03 */
static int hf_rrc_modeSpecificInfo_105 = -1;      /* T_modeSpecificInfo_105 */
static int hf_rrc_fdd_149 = -1;                   /* T_fdd_144 */
static int hf_rrc_tdd_102 = -1;                   /* T_tdd_95 */
static int hf_rrc_gsm_04 = -1;                    /* T_gsm_04 */
static int hf_rrc_modeSpecificInfo_106 = -1;      /* T_modeSpecificInfo_106 */
static int hf_rrc_fdd_150 = -1;                   /* T_fdd_145 */
static int hf_rrc_tdd_103 = -1;                   /* T_tdd_96 */
static int hf_rrc_gsm_05 = -1;                    /* T_gsm_05 */
static int hf_rrc_modeSpecificInfo_107 = -1;      /* T_modeSpecificInfo_107 */
static int hf_rrc_fdd_151 = -1;                   /* T_fdd_146 */
static int hf_rrc_tdd_104 = -1;                   /* T_tdd_97 */
static int hf_rrc_gsm_06 = -1;                    /* T_gsm_06 */
static int hf_rrc_hcs_NeighbouringCellInformation_ECN0 = -1;  /* HCS_NeighbouringCellInformation_ECN0 */
static int hf_rrc_modeSpecificInfo_108 = -1;      /* T_modeSpecificInfo_108 */
static int hf_rrc_fdd_152 = -1;                   /* T_fdd_147 */
static int hf_rrc_tdd_105 = -1;                   /* T_tdd_98 */
static int hf_rrc_gsm_07 = -1;                    /* T_gsm_07 */
static int hf_rrc_deltaQrxlevmin = -1;            /* DeltaQrxlevmin */
static int hf_rrc_deltaQhcs = -1;                 /* DeltaRSCP */
static int hf_rrc_q_Hyst_l_S_PCH = -1;            /* Q_Hyst_S_Fine */
static int hf_rrc_q_Hyst_l_S_FACH = -1;           /* Q_Hyst_S_Fine */
static int hf_rrc_q_Hyst_2_S_PCH = -1;            /* Q_Hyst_S_Fine */
static int hf_rrc_q_Hyst_2_S_FACH = -1;           /* Q_Hyst_S_Fine */
static int hf_rrc_t_Reselection_S_PCH = -1;       /* T_Reselection_S */
static int hf_rrc_t_Reselection_S_FACH = -1;      /* T_Reselection_S_Fine */
static int hf_rrc_non_HCS_t_CR_Max = -1;          /* T_CRMax */
static int hf_rrc_speedDependentScalingFactor = -1;  /* SpeedDependentScalingFactor */
static int hf_rrc_interFrequencyTreselectionScalingFactor = -1;  /* TreselectionScalingFactor */
static int hf_rrc_interRATTreselectionScalingFactor = -1;  /* TreselectionScalingFactor */
static int hf_rrc_CellsForInterFreqMeasList_item = -1;  /* InterFreqCellID */
static int hf_rrc_CellsForInterRATMeasList_item = -1;  /* InterRATCellID */
static int hf_rrc_CellsForIntraFreqMeasList_item = -1;  /* IntraFreqCellID */
static int hf_rrc_CellsForIntraFreqMeasListOnSecULFreq_item = -1;  /* IntraFreqCellIDOnSecULFreq */
static int hf_rrc_modeSpecificInfo_109 = -1;      /* T_modeSpecificInfo_109 */
static int hf_rrc_fdd_153 = -1;                   /* T_fdd_148 */
static int hf_rrc_countC_SFN_Frame_difference = -1;  /* CountC_SFN_Frame_difference */
static int hf_rrc_tm = -1;                        /* INTEGER_0_38399 */
static int hf_rrc_tdd_106 = -1;                   /* T_tdd_99 */
static int hf_rrc_bsicReported = -1;              /* BSICReported */
static int hf_rrc_CellToReportList_item = -1;     /* CellToReport */
static int hf_rrc_cnavToc = -1;                   /* BIT_STRING_SIZE_11 */
static int hf_rrc_cnavTop = -1;                   /* BIT_STRING_SIZE_11 */
static int hf_rrc_cnavURA0 = -1;                  /* BIT_STRING_SIZE_5 */
static int hf_rrc_cnavURA1 = -1;                  /* BIT_STRING_SIZE_3 */
static int hf_rrc_cnavURA2 = -1;                  /* BIT_STRING_SIZE_3 */
static int hf_rrc_cnavAf2 = -1;                   /* BIT_STRING_SIZE_10 */
static int hf_rrc_cnavAf1 = -1;                   /* BIT_STRING_SIZE_20 */
static int hf_rrc_cnavAf0 = -1;                   /* BIT_STRING_SIZE_26 */
static int hf_rrc_cnavTgd = -1;                   /* BIT_STRING_SIZE_13 */
static int hf_rrc_cnavISCl1cp = -1;               /* BIT_STRING_SIZE_13 */
static int hf_rrc_cnavISCl1cd = -1;               /* BIT_STRING_SIZE_13 */
static int hf_rrc_cnavISCl1ca = -1;               /* BIT_STRING_SIZE_13 */
static int hf_rrc_cnavISCl2c = -1;                /* BIT_STRING_SIZE_13 */
static int hf_rrc_cnavISCl5i5 = -1;               /* BIT_STRING_SIZE_13 */
static int hf_rrc_cnavISCl5q5 = -1;               /* BIT_STRING_SIZE_13 */
static int hf_rrc_countC_SFN_High = -1;           /* INTEGER_0_15 */
static int hf_rrc_off = -1;                       /* INTEGER_0_255 */
static int hf_rrc_modeSpecificInfo_110 = -1;      /* T_modeSpecificInfo_110 */
static int hf_rrc_fdd_154 = -1;                   /* T_fdd_149 */
static int hf_rrc_startPSC_01 = -1;               /* PrimaryCPICH_Info */
static int hf_rrc_numberOfPSCs_01 = -1;           /* INTEGER_1_512 */
static int hf_rrc_CSGCellInfoList_item = -1;      /* CSGCellInfo */
static int hf_rrc_cSGInterFreqCellInfoListperFreq = -1;  /* CSGCellInfoList */
static int hf_rrc_CSGInterFreqCellInfoList_item = -1;  /* CSGInterFreqCellInfo */
static int hf_rrc_uTRACSGProximityDetec = -1;     /* T_uTRACSGProximityDetec */
static int hf_rrc_e_UTRACSGProximityDetec = -1;   /* T_e_UTRACSGProximityDetec */
static int hf_rrc_cSGproximityInd = -1;           /* T_cSGproximityInd */
static int hf_rrc_radioAccessTechnology_01 = -1;  /* T_radioAccessTechnology_01 */
static int hf_rrc_uTRA = -1;                      /* T_uTRA */
static int hf_rrc_cSGFrequencyInfoUTRA = -1;      /* FrequencyInfo */
static int hf_rrc_e_UTRA = -1;                    /* T_e_UTRA */
static int hf_rrc_cSGFrequencyInfoEUTRA = -1;     /* EARFCN */
static int hf_rrc_ganss_signal_id = -1;           /* GANSS_Signal_Id */
static int hf_rrc_data_bits = -1;                 /* BIT_STRING_SIZE_1_1024 */
static int hf_rrc_DataBitAssistanceList_item = -1;  /* DataBitAssistanceSat */
static int hf_rrc_satID_01 = -1;                  /* INTEGER_0_63 */
static int hf_rrc_dataBitAssistanceSgnList = -1;  /* DataBitAssistanceSgnList */
static int hf_rrc_DataBitAssistanceSgnList_item = -1;  /* DataBitAssistance */
static int hf_rrc_deltaRSCP = -1;                 /* DeltaRSCP */
static int hf_rrc_b1 = -1;                        /* BIT_STRING_SIZE_11 */
static int hf_rrc_b2 = -1;                        /* BIT_STRING_SIZE_10 */
static int hf_rrc_ganssSignalId = -1;             /* GANSS_Signal_Id */
static int hf_rrc_ganssStatusHealth = -1;         /* GANSS_Status_Health */
static int hf_rrc_dgansssignalInformationList = -1;  /* DGANSSSignalInformationList */
static int hf_rrc_dgansssignalInformationList_01 = -1;  /* DGANSSSignalInformationList_r9 */
static int hf_rrc_dgansssignalInformationList_02 = -1;  /* DGANSSSignalInformationList_v920ext */
static int hf_rrc_DGANSSInfoList_item = -1;       /* DGANSSInfo */
static int hf_rrc_DGANSSInfoList_r9_item = -1;    /* DGANSSInfo_r9 */
static int hf_rrc_DGANSSInfoList_v920ext_item = -1;  /* DGANSSInfo_v920ext */
static int hf_rrc_satId = -1;                     /* INTEGER_0_63 */
static int hf_rrc_iode_dganss = -1;               /* BIT_STRING_SIZE_10 */
static int hf_rrc_udre = -1;                      /* UDRE */
static int hf_rrc_ganss_prc = -1;                 /* INTEGER_M2047_2047 */
static int hf_rrc_ganss_rrc = -1;                 /* INTEGER_M127_127 */
static int hf_rrc_udreGrowthRate = -1;            /* UDREGrowthRate */
static int hf_rrc_udreValidityTime = -1;          /* UDREValidityTime */
static int hf_rrc_DGANSSSignalInformationList_item = -1;  /* DGANSSSignalInformation */
static int hf_rrc_DGANSSSignalInformationList_r9_item = -1;  /* DGANSSSignalInformation_r9 */
static int hf_rrc_DGANSSSignalInformationList_v920ext_item = -1;  /* DGANSSSignalInformation_v920ext */
static int hf_rrc_iode = -1;                      /* IODE */
static int hf_rrc_prc = -1;                       /* PRC */
static int hf_rrc_rrc = -1;                       /* RRC */
static int hf_rrc_dummy1_05 = -1;                 /* DeltaPRC */
static int hf_rrc_dummy2_11 = -1;                 /* DeltaRRC */
static int hf_rrc_dummy3_02 = -1;                 /* DeltaPRC */
static int hf_rrc_dummy4_01 = -1;                 /* DeltaRRC */
static int hf_rrc_DGPS_CorrectionSatInfoList_item = -1;  /* DGPS_CorrectionSatInfo */
static int hf_rrc_DGPS_CorrectionSatInfoList_r9_item = -1;  /* DGPS_CorrectionSatInfo_r9 */
static int hf_rrc_DGPS_CorrectionSatInfoList_v920ext_item = -1;  /* DGPS_CorrectionSatInfo_v920ext */
static int hf_rrc_latitudeSign = -1;              /* T_latitudeSign */
static int hf_rrc_latitude = -1;                  /* INTEGER_0_8388607 */
static int hf_rrc_longitude = -1;                 /* INTEGER_M8388608_8388607 */
static int hf_rrc_latitudeSign_01 = -1;           /* T_latitudeSign_01 */
static int hf_rrc_altitudeDirection = -1;         /* T_altitudeDirection */
static int hf_rrc_altitude = -1;                  /* INTEGER_0_32767 */
static int hf_rrc_latitudeSign_02 = -1;           /* T_latitudeSign_02 */
static int hf_rrc_altitudeDirection_01 = -1;      /* T_altitudeDirection_01 */
static int hf_rrc_uncertaintySemiMajor = -1;      /* INTEGER_0_127 */
static int hf_rrc_uncertaintySemiMinor = -1;      /* INTEGER_0_127 */
static int hf_rrc_orientationMajorAxis = -1;      /* INTEGER_0_89 */
static int hf_rrc_uncertaintyAltitude = -1;       /* INTEGER_0_127 */
static int hf_rrc_confidence = -1;                /* INTEGER_0_100 */
static int hf_rrc_latitudeSign_03 = -1;           /* T_latitudeSign_03 */
static int hf_rrc_uncertaintyCode = -1;           /* INTEGER_0_127 */
static int hf_rrc_latitudeSign_04 = -1;           /* T_latitudeSign_04 */
static int hf_rrc_reportedCells = -1;             /* SEQUENCE_SIZE_1_maxReportedEUTRACellPerFreq_OF_EUTRA_PhysicalCellIdentity */
static int hf_rrc_reportedCells_item = -1;        /* EUTRA_PhysicalCellIdentity */
static int hf_rrc_Eutra_EventResultList_item = -1;  /* Eutra_EventResult */
static int hf_rrc_eventID = -1;                   /* EventIDInterRAT */
static int hf_rrc_eutra_EventResultsList = -1;    /* Eutra_EventResultList */
static int hf_rrc_eutraFrequencyIndicator = -1;   /* BIT_STRING_SIZE_maxNumEUTRAFreqs_FACH */
static int hf_rrc_measurementBandwidth = -1;      /* EUTRA_MeasurementBandwidth */
static int hf_rrc_qRxLevMinEUTRA = -1;            /* INTEGER_M70_M22 */
static int hf_rrc_threshXhigh = -1;               /* INTEGER_0_31 */
static int hf_rrc_threshXlow = -1;                /* INTEGER_0_31 */
static int hf_rrc_eutra_blackListedCellList = -1;  /* EUTRA_BlacklistedCellPerFreqList */
static int hf_rrc_qqualMinEUTRA = -1;             /* INTEGER_M34_M3 */
static int hf_rrc_threshXhigh2 = -1;              /* INTEGER_0_31 */
static int hf_rrc_threshXlow2 = -1;               /* INTEGER_0_31 */
static int hf_rrc_EUTRA_FrequencyAndPriorityInfoList_item = -1;  /* EUTRA_FrequencyAndPriorityInfo */
static int hf_rrc_EUTRA_FrequencyAndPriorityInfoList_v920ext_item = -1;  /* EUTRA_FrequencyAndPriorityInfo_v920ext */
static int hf_rrc_eutraTreselectionScalingFactor = -1;  /* TreselectionScalingFactor2 */
static int hf_rrc_eutraFrequencyRemoval = -1;     /* EUTRA_FrequencyRemoval */
static int hf_rrc_eutraNewFrequencies = -1;       /* EUTRA_FrequencyInfoList */
static int hf_rrc_eutraSIAcquisition = -1;        /* EUTRA_SIAcquisition */
static int hf_rrc_eutraNewFrequencies_01 = -1;    /* EUTRA_FrequencyInfoList_r11 */
static int hf_rrc_physicalCellIdentity = -1;      /* EUTRA_PhysicalCellIdentity */
static int hf_rrc_rSRP = -1;                      /* INTEGER_0_97 */
static int hf_rrc_rSRQ = -1;                      /* INTEGER_0_33 */
static int hf_rrc_measuredEUTRACells = -1;        /* SEQUENCE_SIZE_1_maxReportedEUTRACellPerFreq_OF_EUTRA_MeasuredCells */
static int hf_rrc_measuredEUTRACells_item = -1;   /* EUTRA_MeasuredCells */
static int hf_rrc_Eutra_MeasuredResultList_item = -1;  /* Eutra_MeasuredResult */
static int hf_rrc_eutraMeasuredResultList = -1;   /* Eutra_MeasuredResultList */
static int hf_rrc_eutraSIacquisitionResults = -1;  /* EUTRA_SIacquisitionResults */
static int hf_rrc_measuredEUTRACells_v920ext = -1;  /* SEQUENCE_SIZE_1_maxReportedEUTRACellPerFreq_OF_EUTRA_MeasuredCells_v920ext */
static int hf_rrc_measuredEUTRACells_v920ext_item = -1;  /* EUTRA_MeasuredCells_v920ext */
static int hf_rrc_Eutra_MeasuredResultList_v920ext_item = -1;  /* Eutra_MeasuredResult_v920ext */
static int hf_rrc_eutraMeasuredResultList_v920ext = -1;  /* Eutra_MeasuredResultList_v920ext */
static int hf_rrc_cgiInfo = -1;                   /* T_cgiInfo */
static int hf_rrc_cellIdentity_01 = -1;           /* BIT_STRING_SIZE_28 */
static int hf_rrc_trackingAreaCode = -1;          /* BIT_STRING_SIZE_16 */
static int hf_rrc_csgMemberIndication_01 = -1;    /* T_csgMemberIndication_01 */
static int hf_rrc_removeAllFrequencies = -1;      /* NULL */
static int hf_rrc_removeSomeFrequencies = -1;     /* SEQUENCE_SIZE_1_maxNumEUTRAFreqs_OF_EARFCN */
static int hf_rrc_removeSomeFrequencies_item = -1;  /* EARFCN */
static int hf_rrc_removeNoFrequencies = -1;       /* NULL */
static int hf_rrc_EUTRA_FrequencyInfoList_item = -1;  /* EUTRA_FrequencyInfo */
static int hf_rrc_EUTRA_FrequencyInfoList_r11_item = -1;  /* EUTRA_FrequencyInfo_r11 */
static int hf_rrc_widebandRSRQMeasurement = -1;   /* T_widebandRSRQMeasurement */
static int hf_rrc_eutra_FrequencyListIndicator = -1;  /* BIT_STRING_SIZE_maxNumEUTRAFreqs */
static int hf_rrc_eutra_FrequencyRepQuantityRACH = -1;  /* T_eutra_FrequencyRepQuantityRACH */
static int hf_rrc_eutra_FrequencyRACHReportingThreshold = -1;  /* INTEGER_0_97 */
static int hf_rrc_rachReportingPriority = -1;     /* T_rachReportingPriority */
static int hf_rrc_eutraFrequenciesForMeasurement = -1;  /* SEQUENCE_SIZE_1_maxNumEUTRAFreqs_FACH_OF_EARFCN */
static int hf_rrc_eutraFrequenciesForMeasurement_item = -1;  /* EARFCN */
static int hf_rrc_reportCriteria = -1;            /* T_reportCriteria */
static int hf_rrc_measurementQuantity = -1;       /* T_measurementQuantity */
static int hf_rrc_reportingThreshold = -1;        /* INTEGER_0_97 */
static int hf_rrc_triggeringCondition = -1;       /* TriggeringCondition2 */
static int hf_rrc_forbiddenAffectCellList = -1;   /* ForbiddenAffectCellList */
static int hf_rrc_w = -1;                         /* W */
static int hf_rrc_reportDeactivationThreshold = -1;  /* ReportDeactivationThreshold */
static int hf_rrc_reportingAmount = -1;           /* ReportingAmount */
static int hf_rrc_reportingInterval = -1;         /* ReportingInterval */
static int hf_rrc_forbiddenAffectCellList_01 = -1;  /* ForbiddenAffectCellList_r4 */
static int hf_rrc_forbiddenAffectCellListOnSecULFreq = -1;  /* ForbiddenAffectCellListOnSecULFreq */
static int hf_rrc_forbiddenAffectCellList_02 = -1;  /* ForbiddenAffectCellList_LCR_r4 */
static int hf_rrc_triggeringCondition_01 = -1;    /* TriggeringCondition1 */
static int hf_rrc_periodicReportingInfo_1b = -1;  /* PeriodicReportingInfo_1b */
static int hf_rrc_replacementActivationThreshold = -1;  /* ReplacementActivationThreshold */
static int hf_rrc_useCIO = -1;                    /* BOOLEAN */
static int hf_rrc_thresholdUsedFrequency = -1;    /* ThresholdUsedFrequency */
static int hf_rrc_thresholdUsedFrequency_01 = -1;  /* ThresholdUsedFrequency_r6 */
static int hf_rrc_dummy_50 = -1;                  /* Threshold */
static int hf_rrc_usedFreqW = -1;                 /* W */
static int hf_rrc_hysteresis = -1;                /* HysteresisInterFreq */
static int hf_rrc_timeToTrigger = -1;             /* TimeToTrigger */
static int hf_rrc_reportingCellStatus = -1;       /* ReportingCellStatus */
static int hf_rrc_nonUsedFreqParameterList = -1;  /* NonUsedFreqParameterList */
static int hf_rrc_nonUsedFreqParameterList_01 = -1;  /* NonUsedFreqWList_r6 */
static int hf_rrc_reportingCellStatus_01 = -1;    /* ReportingCellStatus_r10 */
static int hf_rrc_nonUsedFreqParameterList_02 = -1;  /* NonUsedFreq2aParameterList_r10 */
static int hf_rrc_nonUsedFreqParameterList_03 = -1;  /* NonUsedFreq2aParameterList_r11 */
static int hf_rrc_usedFreqThreshold = -1;         /* Threshold */
static int hf_rrc_usedFreqThreshold_01 = -1;      /* Threshold_r6 */
static int hf_rrc_nonUsedFreqParameterList_04 = -1;  /* NonUsedFreqParameterList_r6 */
static int hf_rrc_nonUsedFreqParameterList_05 = -1;  /* NonUsedFreqParameterList_r10 */
static int hf_rrc_nonUsedFreqParameterList_06 = -1;  /* NonUsedFreqParameterList_r11 */
static int hf_rrc_thresholdOwnSystem = -1;        /* Threshold */
static int hf_rrc_thresholdOtherSystem = -1;      /* Threshold */
static int hf_rrc_hysteresis_01 = -1;             /* Hysteresis */
static int hf_rrc_intraFreqEventResults = -1;     /* IntraFreqEventResults */
static int hf_rrc_interFreqEventResults = -1;     /* InterFreqEventResults */
static int hf_rrc_interRATEventResults = -1;      /* InterRATEventResults */
static int hf_rrc_trafficVolumeEventResults = -1;  /* TrafficVolumeEventResults */
static int hf_rrc_qualityEventResults = -1;       /* QualityEventResults */
static int hf_rrc_ue_InternalEventResults = -1;   /* UE_InternalEventResults */
static int hf_rrc_ue_positioning_MeasurementEventResults = -1;  /* UE_Positioning_MeasurementEventResults */
static int hf_rrc_ue_positioning_MeasurementEventResults_01 = -1;  /* UE_Positioning_MeasurementEventResults_v770ext */
static int hf_rrc_ue_positioning_MeasurementEventResults_02 = -1;  /* UE_Positioning_MeasurementEventResults_v860ext */
static int hf_rrc_interFreqEventResults_01 = -1;  /* InterFreqEventResults_va40ext */
static int hf_rrc_intraFreqEventResults_01 = -1;  /* IntraFreqEventResultsOnSecUlFreq */
static int hf_rrc_dopplerFirstOrder = -1;         /* INTEGER_M42_21 */
static int hf_rrc_dopplerUncertainty = -1;        /* T_dopplerUncertainty */
static int hf_rrc_doppler1stOrder = -1;           /* INTEGER_M42_21 */
static int hf_rrc_dopplerUncertainty_01 = -1;     /* DopplerUncertainty */
static int hf_rrc_fACH_meas_occasion_coeff = -1;  /* INTEGER_1_12 */
static int hf_rrc_inter_freq_FDD_meas_ind = -1;   /* BOOLEAN */
static int hf_rrc_inter_freq_TDD_meas_ind = -1;   /* BOOLEAN */
static int hf_rrc_inter_RAT_meas_ind = -1;        /* SEQUENCE_SIZE_1_maxOtherRAT_OF_RAT_Type */
static int hf_rrc_inter_RAT_meas_ind_item = -1;   /* RAT_Type */
static int hf_rrc_inter_freq_TDD128_meas_ind = -1;  /* BOOLEAN */
static int hf_rrc_fdd_155 = -1;                   /* PrimaryCPICH_Info */
static int hf_rrc_tdd_107 = -1;                   /* PrimaryCCPCH_Info_LCR_r4 */
static int hf_rrc_ForbiddenAffectCellList_item = -1;  /* ForbiddenAffectCell */
static int hf_rrc_ForbiddenAffectCellList_r4_item = -1;  /* ForbiddenAffectCell_r4 */
static int hf_rrc_ForbiddenAffectCellList_LCR_r4_item = -1;  /* ForbiddenAffectCell_LCR_r4 */
static int hf_rrc_ForbiddenAffectCellListOnSecULFreq_item = -1;  /* ForbiddenAffectCellOnSecULFreq */
static int hf_rrc_primaryCPICH = -1;              /* PrimaryCPICH_Info */
static int hf_rrc_ganssTod = -1;                  /* INTEGER_0_86399 */
static int hf_rrc_dataBitAssistance = -1;         /* ReqDataBitAssistance */
static int hf_rrc_ganssId = -1;                   /* INTEGER_0_7 */
static int hf_rrc_ganssTimeModelsList = -1;       /* GANSSTimeModelsList */
static int hf_rrc_uePositioningDGANSSCorrections = -1;  /* UE_Positioning_DGANSSCorrections */
static int hf_rrc_uePositioningGANSSNavigationModel = -1;  /* UE_Positioning_GANSS_NavigationModel */
static int hf_rrc_uePositioningGANSSRealTimeIntegrity = -1;  /* UE_Positioning_GANSS_RealTimeIntegrity */
static int hf_rrc_uePositioningGANSSDataBitAssistance = -1;  /* UE_Positioning_GANSS_Data_Bit_Assistance */
static int hf_rrc_uePositioningGANSSReferenceMeasurementInfo = -1;  /* UE_Positioning_GANSS_ReferenceMeasurementInfo */
static int hf_rrc_uePositioningGANSSAlmanac = -1;  /* UE_Positioning_GANSS_Almanac */
static int hf_rrc_uePositioningGANSSUTCModel = -1;  /* UE_Positioning_GANSS_UTCModel */
static int hf_rrc_uePositiningGANSSsbasID = -1;   /* UE_Positioning_GANSS_SBAS_ID */
static int hf_rrc_uePositioningGANSSAddNavigationModels = -1;  /* UE_Positioning_GANSS_AddNavigationModels */
static int hf_rrc_uePositioningGANSSAlmanac_01 = -1;  /* UE_Positioning_GANSS_Almanac_v860ext */
static int hf_rrc_uePositioningGANSSAddUTCModels = -1;  /* UE_Positioning_GANSS_AddUTCModels */
static int hf_rrc_uePositioningGANSSAuxiliaryInfo = -1;  /* UE_Positioning_GANSS_AuxiliaryInfo */
static int hf_rrc_uePositioningGANSSAlmanac_02 = -1;  /* UE_Positioning_GANSS_Almanac_r8 */
static int hf_rrc_uePositioningDGANSSCorrections_01 = -1;  /* UE_Positioning_DGANSSCorrections_r9 */
static int hf_rrc_uePositioningDGANSSCorrections_02 = -1;  /* UE_Positioning_DGANSSCorrections_v920ext */
static int hf_rrc_ganssTimeModelsList_01 = -1;    /* GANSSTimeModelsList_va40ext */
static int hf_rrc_uePositioningGANSSReferenceMeasurementInfo_01 = -1;  /* UE_Positioning_GANSS_ReferenceMeasurementInfo_va40ext */
static int hf_rrc_uePositioningGANSSAlmanac_03 = -1;  /* UE_Positioning_GANSS_Almanac_va40ext */
static int hf_rrc_ganssTimeModelsList_02 = -1;    /* GANSSTimeModelsList_r10 */
static int hf_rrc_uePositioningGANSSReferenceMeasurementInfo_02 = -1;  /* UE_Positioning_GANSS_ReferenceMeasurementInfo_r10 */
static int hf_rrc_uePositioningGANSSAlmanac_04 = -1;  /* UE_Positioning_GANSS_Almanac_r10 */
static int hf_rrc_GANSSGenericDataList_item = -1;  /* GANSSGenericData */
static int hf_rrc_GANSSGenericDataList_v860ext_item = -1;  /* GANSSGenericData_v860ext */
static int hf_rrc_GANSSGenericDataList_r8_item = -1;  /* GANSSGenericData_r8 */
static int hf_rrc_GANSSGenericDataList_r9_item = -1;  /* GANSSGenericData_r9 */
static int hf_rrc_GANSSGenericDataList_v920ext_item = -1;  /* GANSSGenericData_v920ext */
static int hf_rrc_GANSSGenericDataList_va40ext_item = -1;  /* GANSSGenericData_va40ext */
static int hf_rrc_GANSSGenericDataList_r10_item = -1;  /* GANSSGenericData_r10 */
static int hf_rrc_GANSSGenericMeasurementInfo_item = -1;  /* GANSSGenericMeasurementInfo_item */
static int hf_rrc_ganssMeasurementSignalList = -1;  /* GANSSMeasurementSignalList */
static int hf_rrc_GANSSGenericMeasurementInfo_v860ext_item = -1;  /* GANSSGenericMeasurementInfo_v860ext_item */
static int hf_rrc_ganssMeasurementSignalList_01 = -1;  /* GANSSMeasurementSignalList_v860ext */
static int hf_rrc_GANSSMeasurementParameters_item = -1;  /* GANSSMeasurementParameters_item */
static int hf_rrc_cSurNzero = -1;                 /* INTEGER_0_63 */
static int hf_rrc_multipathIndicator = -1;        /* T_multipathIndicator */
static int hf_rrc_carrierQualityIndication = -1;  /* BIT_STRING_SIZE_2 */
static int hf_rrc_ganssCodePhase = -1;            /* INTEGER_0_2097151 */
static int hf_rrc_ganssIntegerCodePhase = -1;     /* INTEGER_0_63 */
static int hf_rrc_codePhaseRmsError = -1;         /* INTEGER_0_63 */
static int hf_rrc_doppler = -1;                   /* INTEGER_M32768_32767 */
static int hf_rrc_adr = -1;                       /* INTEGER_0_33554431 */
static int hf_rrc_GANSSMeasurementParameters_v860ext_item = -1;  /* GANSSMeasurementParameters_v860ext_item */
static int hf_rrc_ganssIntegerCodePhaseExt = -1;  /* INTEGER_64_127 */
static int hf_rrc_GANSSMeasurementSignalList_item = -1;  /* GANSSMeasurementSignalList_item */
static int hf_rrc_ganssCodePhaseAmbiguity = -1;   /* INTEGER_0_31 */
static int hf_rrc_ganssMeasurementParameters = -1;  /* GANSSMeasurementParameters */
static int hf_rrc_GANSSMeasurementSignalList_v860ext_item = -1;  /* GANSSMeasurementSignalList_v860ext_item */
static int hf_rrc_ganssCodePhaseAmbiguityExt = -1;  /* INTEGER_32_127 */
static int hf_rrc_ganssMeasurementParameters_01 = -1;  /* GANSSMeasurementParameters_v860ext */
static int hf_rrc_ganssWeek = -1;                 /* INTEGER_0_4095 */
static int hf_rrc_ganssToe = -1;                  /* INTEGER_0_167 */
static int hf_rrc_t_toeLimit = -1;                /* INTEGER_0_10 */
static int hf_rrc_satellitesListRelatedDataList = -1;  /* SatellitesListRelatedDataList */
static int hf_rrc_gANSS_tod = -1;                 /* INTEGER_0_3599999 */
static int hf_rrc_gANSS_timeId = -1;              /* INTEGER_0_7 */
static int hf_rrc_gANSS_tod_uncertainty = -1;     /* INTEGER_0_127 */
static int hf_rrc_ganssRealTimeIntegrity = -1;    /* BOOLEAN */
static int hf_rrc_ganssDifferentialCorrection = -1;  /* DGANSS_Sig_Id_Req */
static int hf_rrc_ganssAlmanac = -1;              /* BOOLEAN */
static int hf_rrc_ganssNavigationModel = -1;      /* BOOLEAN */
static int hf_rrc_ganssTimeModelGNSS_GNSS = -1;   /* BIT_STRING_SIZE_8 */
static int hf_rrc_ganssReferenceMeasurementInfo = -1;  /* BOOLEAN */
static int hf_rrc_ganssDataBits = -1;             /* GanssDataBits */
static int hf_rrc_ganssUTCModel = -1;             /* BOOLEAN */
static int hf_rrc_ganssNavigationModelAdditionalData = -1;  /* GanssNavigationModelAdditionalData */
static int hf_rrc_sbasId = -1;                    /* UE_Positioning_GANSS_SBAS_ID */
static int hf_rrc_ganssAddNavigationModel = -1;   /* T_ganssAddNavigationModel */
static int hf_rrc_ganssAddUTCmodel = -1;          /* T_ganssAddUTCmodel */
static int hf_rrc_ganssAuxInfo = -1;              /* T_ganssAuxInfo */
static int hf_rrc_ganssAddADchoices = -1;         /* T_ganssAddADchoices */
static int hf_rrc_orbitModelID = -1;              /* INTEGER_0_7 */
static int hf_rrc_clockModelID = -1;              /* INTEGER_0_7 */
static int hf_rrc_utcModelID = -1;                /* INTEGER_0_7 */
static int hf_rrc_almanacModelID = -1;            /* INTEGER_0_7 */
static int hf_rrc_GanssRequestedGenericAssistanceDataList_item = -1;  /* GanssReqGenericData */
static int hf_rrc_GanssRequestedGenericAssistanceDataList_v860ext_item = -1;  /* GanssReqGenericData_v860ext */
static int hf_rrc_ganssSatId = -1;                /* INTEGER_0_63 */
static int hf_rrc_dopplerZeroOrder = -1;          /* INTEGER_M2048_2047 */
static int hf_rrc_extraDoppler = -1;              /* ExtraDoppler */
static int hf_rrc_codePhase_01 = -1;              /* INTEGER_0_1023 */
static int hf_rrc_integerCodePhase_01 = -1;       /* INTEGER_0_127 */
static int hf_rrc_codePhaseSearchWindow_01 = -1;  /* BIT_STRING_SIZE_5 */
static int hf_rrc_azimuthandElevation = -1;       /* AzimuthAndElevation */
static int hf_rrc_azimuthandElevation_01 = -1;    /* AzimuthAndElevation_r10 */
static int hf_rrc_GANSSSatelliteInformationList_item = -1;  /* GANSSSatelliteInformation */
static int hf_rrc_GANSSSatelliteInformationList_va40ext_item = -1;  /* GANSSSatelliteInformation_va40ext */
static int hf_rrc_GANSSSatelliteInformationList_r10_item = -1;  /* GANSSSatelliteInformation_r10 */
static int hf_rrc_GANSSTimeModelsList_item = -1;  /* UE_Positioning_GANSS_TimeModel */
static int hf_rrc_GANSSTimeModelsList_va40ext_item = -1;  /* UE_Positioning_GANSS_TimeModel_va40ext */
static int hf_rrc_GANSSTimeModelsList_r10_item = -1;  /* UE_Positioning_GANSS_TimeModel_r10 */
static int hf_rrc_gloAlmNA = -1;                  /* BIT_STRING_SIZE_11 */
static int hf_rrc_gloAlmnA = -1;                  /* BIT_STRING_SIZE_5 */
static int hf_rrc_gloAlmHA = -1;                  /* BIT_STRING_SIZE_5 */
static int hf_rrc_gloAlmLambdaA = -1;             /* BIT_STRING_SIZE_21 */
static int hf_rrc_gloAlmTlambdaA = -1;            /* BIT_STRING_SIZE_21 */
static int hf_rrc_gloAlmDeltaIA = -1;             /* BIT_STRING_SIZE_18 */
static int hf_rrc_gloAkmDeltaTA = -1;             /* BIT_STRING_SIZE_22 */
static int hf_rrc_gloAlmDeltaTdotA = -1;          /* BIT_STRING_SIZE_7 */
static int hf_rrc_gloAlmEpsilonA = -1;            /* BIT_STRING_SIZE_15 */
static int hf_rrc_gloAlmOmegaA = -1;              /* BIT_STRING_SIZE_16 */
static int hf_rrc_gloAlmTauA = -1;                /* BIT_STRING_SIZE_10 */
static int hf_rrc_gloAlmCA = -1;                  /* BIT_STRING_SIZE_1 */
static int hf_rrc_gloAlmMA = -1;                  /* BIT_STRING_SIZE_2 */
static int hf_rrc_svId = -1;                      /* INTEGER_0_63 */
static int hf_rrc_ganss_alm_e = -1;               /* BIT_STRING_SIZE_11 */
static int hf_rrc_ganss_delta_I_alm = -1;         /* BIT_STRING_SIZE_11 */
static int hf_rrc_ganss_omegadot_alm = -1;        /* BIT_STRING_SIZE_11 */
static int hf_rrc_ganss_svhealth_alm = -1;        /* BIT_STRING_SIZE_4 */
static int hf_rrc_ganss_delta_a_sqrt_alm = -1;    /* BIT_STRING_SIZE_17 */
static int hf_rrc_ganss_omegazero_alm = -1;       /* BIT_STRING_SIZE_16 */
static int hf_rrc_ganss_m_zero_alm = -1;          /* BIT_STRING_SIZE_16 */
static int hf_rrc_ganss_omega_alm = -1;           /* BIT_STRING_SIZE_16 */
static int hf_rrc_ganss_af_zero_alm = -1;         /* BIT_STRING_SIZE_14 */
static int hf_rrc_ganss_af_one_alm = -1;          /* BIT_STRING_SIZE_11 */
static int hf_rrc_midiAlmE = -1;                  /* BIT_STRING_SIZE_11 */
static int hf_rrc_midiAlmDeltaI = -1;             /* BIT_STRING_SIZE_11 */
static int hf_rrc_midiAlmOmegaDot = -1;           /* BIT_STRING_SIZE_11 */
static int hf_rrc_midiAlmSqrtA = -1;              /* BIT_STRING_SIZE_17 */
static int hf_rrc_midiAlmOmega0 = -1;             /* BIT_STRING_SIZE_16 */
static int hf_rrc_midiAlmOmega = -1;              /* BIT_STRING_SIZE_16 */
static int hf_rrc_midiAlmMo = -1;                 /* BIT_STRING_SIZE_16 */
static int hf_rrc_midiAlmaf0 = -1;                /* BIT_STRING_SIZE_11 */
static int hf_rrc_midiAlmaf1 = -1;                /* BIT_STRING_SIZE_10 */
static int hf_rrc_midiAlmL1Health = -1;           /* BIT_STRING_SIZE_1 */
static int hf_rrc_midiAlmL2Health = -1;           /* BIT_STRING_SIZE_1 */
static int hf_rrc_midiAlmL5Health = -1;           /* BIT_STRING_SIZE_1 */
static int hf_rrc_navAlmE = -1;                   /* BIT_STRING_SIZE_16 */
static int hf_rrc_navAlmDeltaI = -1;              /* BIT_STRING_SIZE_16 */
static int hf_rrc_navAlmOMEGADOT = -1;            /* BIT_STRING_SIZE_16 */
static int hf_rrc_navAlmSVHealth = -1;            /* BIT_STRING_SIZE_8 */
static int hf_rrc_navAlmSqrtA = -1;               /* BIT_STRING_SIZE_24 */
static int hf_rrc_navAlmOMEGAo = -1;              /* BIT_STRING_SIZE_24 */
static int hf_rrc_navAlmOmega = -1;               /* BIT_STRING_SIZE_24 */
static int hf_rrc_navAlmMo = -1;                  /* BIT_STRING_SIZE_24 */
static int hf_rrc_navAlmaf0 = -1;                 /* BIT_STRING_SIZE_11 */
static int hf_rrc_navAlmaf1 = -1;                 /* BIT_STRING_SIZE_11 */
static int hf_rrc_redAlmDeltaA = -1;              /* BIT_STRING_SIZE_8 */
static int hf_rrc_redAlmOmega0 = -1;              /* BIT_STRING_SIZE_7 */
static int hf_rrc_redAlmPhi0 = -1;                /* BIT_STRING_SIZE_7 */
static int hf_rrc_redAlmL1Health = -1;            /* BIT_STRING_SIZE_1 */
static int hf_rrc_redAlmL2Health = -1;            /* BIT_STRING_SIZE_1 */
static int hf_rrc_redAlmL5Health = -1;            /* BIT_STRING_SIZE_1 */
static int hf_rrc_sbasAlmDataID = -1;             /* BIT_STRING_SIZE_2 */
static int hf_rrc_sbasAlmHealth = -1;             /* BIT_STRING_SIZE_8 */
static int hf_rrc_sbasAlmXg = -1;                 /* BIT_STRING_SIZE_15 */
static int hf_rrc_sbasAlmYg = -1;                 /* BIT_STRING_SIZE_15 */
static int hf_rrc_sbasAlmZg = -1;                 /* BIT_STRING_SIZE_9 */
static int hf_rrc_sbasAlmXgdot = -1;              /* BIT_STRING_SIZE_3 */
static int hf_rrc_sbasAlmYgDot = -1;              /* BIT_STRING_SIZE_3 */
static int hf_rrc_sbasAlmZgDot = -1;              /* BIT_STRING_SIZE_4 */
static int hf_rrc_sbasAlmTo = -1;                 /* BIT_STRING_SIZE_11 */
static int hf_rrc_GANSS_SAT_Info_Almanac_GLOkpList_item = -1;  /* GANSS_SAT_Info_Almanac_GLOkp */
static int hf_rrc_GANSS_SAT_Info_Almanac_KpList_item = -1;  /* GANSS_SAT_Info_Almanac_Kp */
static int hf_rrc_GANSS_SAT_Info_Almanac_MIDIkpList_item = -1;  /* GANSS_SAT_Info_Almanac_MIDIkp */
static int hf_rrc_GANSS_SAT_Info_Almanac_NAVkpList_item = -1;  /* GANSS_SAT_Info_Almanac_NAVkp */
static int hf_rrc_GANSS_SAT_Info_Almanac_REDkpList_item = -1;  /* GANSS_SAT_Info_Almanac_REDkp */
static int hf_rrc_GANSS_SAT_Info_Almanac_SBASecefList_item = -1;  /* GANSS_SAT_Info_Almanac_SBASecef */
static int hf_rrc_svHealth = -1;                  /* BIT_STRING_SIZE_6 */
static int hf_rrc_iod = -1;                       /* BIT_STRING_SIZE_11 */
static int hf_rrc_ganssClockModel = -1;           /* UE_Positioning_GANSS_AddClockModels */
static int hf_rrc_ganssOrbitModel = -1;           /* UE_Positioning_GANSS_AddOrbitModels */
static int hf_rrc_svHealth_01 = -1;               /* BIT_STRING_SIZE_5 */
static int hf_rrc_iod_01 = -1;                    /* BIT_STRING_SIZE_10 */
static int hf_rrc_ganssClockModel_01 = -1;        /* UE_Positioning_GANSS_ClockModel */
static int hf_rrc_ganssOrbitModel_01 = -1;        /* UE_Positioning_GANSS_OrbitModel */
static int hf_rrc_Ganss_Sat_Info_AddNavList_item = -1;  /* Ganss_Sat_Info_AddNav */
static int hf_rrc_Ganss_Sat_Info_NavList_item = -1;  /* Ganss_Sat_Info_Nav */
static int hf_rrc_storm_flag_one = -1;            /* BOOLEAN */
static int hf_rrc_storm_flag_two = -1;            /* BOOLEAN */
static int hf_rrc_storm_flag_three = -1;          /* BOOLEAN */
static int hf_rrc_storm_flag_four = -1;           /* BOOLEAN */
static int hf_rrc_storm_flag_five = -1;           /* BOOLEAN */
static int hf_rrc_gloTau = -1;                    /* BIT_STRING_SIZE_22 */
static int hf_rrc_gloGamma = -1;                  /* BIT_STRING_SIZE_11 */
static int hf_rrc_gloDeltaTau = -1;               /* BIT_STRING_SIZE_5 */
static int hf_rrc_satelliteID = -1;               /* INTEGER_0_63 */
static int hf_rrc_c_N0 = -1;                      /* INTEGER_0_63 */
static int hf_rrc_doppler_01 = -1;                /* INTEGER_M32768_32768 */
static int hf_rrc_wholeGPS_Chips = -1;            /* INTEGER_0_1022 */
static int hf_rrc_fractionalGPS_Chips = -1;       /* INTEGER_0_1023 */
static int hf_rrc_multipathIndicator_01 = -1;     /* MultipathIndicator */
static int hf_rrc_pseudorangeRMS_Error = -1;      /* INTEGER_0_63 */
static int hf_rrc_GPS_MeasurementParamList_item = -1;  /* GPS_MeasurementParam */
static int hf_rrc_startingARFCN = -1;             /* BCCH_ARFCN */
static int hf_rrc_bandIndicator = -1;             /* T_bandIndicator */
static int hf_rrc_followingARFCNs = -1;           /* T_followingARFCNs */
static int hf_rrc_explicitListOfARFCNs = -1;      /* SEQUENCE_SIZE_0_31_OF_BCCH_ARFCN */
static int hf_rrc_explicitListOfARFCNs_item = -1;  /* BCCH_ARFCN */
static int hf_rrc_equallySpacedARFCNs = -1;       /* T_equallySpacedARFCNs */
static int hf_rrc_arfcn_Spacing = -1;             /* INTEGER_1_8 */
static int hf_rrc_numberOfFollowingARFCNs = -1;   /* INTEGER_0_31 */
static int hf_rrc_variableBitMapOfARFCNs = -1;    /* OCTET_STRING_SIZE_1_16 */
static int hf_rrc_continuousRangeOfARFCNs = -1;   /* T_continuousRangeOfARFCNs */
static int hf_rrc_endingARFCN = -1;               /* BCCH_ARFCN */
static int hf_rrc_gsm_CarrierRSSI = -1;           /* GSM_CarrierRSSI */
static int hf_rrc_dummy_51 = -1;                  /* INTEGER_46_173 */
static int hf_rrc_dummy2_12 = -1;                 /* ObservedTimeDifferenceToGSM */
static int hf_rrc_GSM_MeasuredResultsList_item = -1;  /* GSM_MeasuredResults */
static int hf_rrc_tlm_Message = -1;               /* BIT_STRING_SIZE_14 */
static int hf_rrc_tlm_Reserved = -1;              /* BIT_STRING_SIZE_2 */
static int hf_rrc_alert = -1;                     /* BOOLEAN */
static int hf_rrc_antiSpoof = -1;                 /* BOOLEAN */
static int hf_rrc_GPS_TOW_AssistList_item = -1;   /* GPS_TOW_Assist */
static int hf_rrc_gsmCellGroup = -1;              /* GSM_CellGroup */
static int hf_rrc_qRxLevMinGSM = -1;              /* INTEGER_M58_M13 */
static int hf_rrc_GSM_PriorityInfoList_item = -1;  /* GSM_PriorityInfo */
static int hf_rrc_gsmTreselectionScalingFactor = -1;  /* TreselectionScalingFactor2 */
static int hf_rrc_penaltyTime = -1;               /* PenaltyTime_RSCP */
static int hf_rrc_penaltyTime_01 = -1;            /* PenaltyTime_ECN0 */
static int hf_rrc_hcs_PRIO = -1;                  /* HCS_PRIO */
static int hf_rrc_q_HCS = -1;                     /* Q_HCS */
static int hf_rrc_hcs_CellReselectInformation = -1;  /* HCS_CellReselectInformation_RSCP */
static int hf_rrc_hcs_CellReselectInformation_01 = -1;  /* HCS_CellReselectInformation_ECN0 */
static int hf_rrc_t_CR_Max = -1;                  /* T_CRMax */
static int hf_rrc_bearing = -1;                   /* INTEGER_0_359 */
static int hf_rrc_horizontalSpeed = -1;           /* INTEGER_0_2047 */
static int hf_rrc_verticalSpeedDirection = -1;    /* T_verticalSpeedDirection */
static int hf_rrc_verticalSpeed = -1;             /* INTEGER_0_255 */
static int hf_rrc_horizontalSpeedUncertainty = -1;  /* INTEGER_0_255 */
static int hf_rrc_verticalSpeedDirection_01 = -1;  /* T_verticalSpeedDirection_01 */
static int hf_rrc_horizontalUncertaintySpeed = -1;  /* INTEGER_0_255 */
static int hf_rrc_verticalUncertaintySpeed = -1;  /* INTEGER_0_255 */
static int hf_rrc_k = -1;                         /* INTEGER_2_3 */
static int hf_rrc_nonFreqRelatedEventResults = -1;  /* CellMeasurementEventResults */
static int hf_rrc_nonFreqRelatedEventResults_01 = -1;  /* CellMeasurementEventResults_LCR_r4 */
static int hf_rrc_removedInterFreqCellList = -1;  /* RemovedInterFreqCellList */
static int hf_rrc_newInterFreqCellList_02 = -1;   /* NewInterFreqCellList */
static int hf_rrc_cellsForInterFreqMeasList = -1;  /* CellsForInterFreqMeasList */
static int hf_rrc_newInterFreqCellList_03 = -1;   /* NewInterFreqCellList_r4 */
static int hf_rrc_newInterFreqCellList_04 = -1;   /* NewInterFreqCellList_r8 */
static int hf_rrc_newInterFreqCellList_05 = -1;   /* NewInterFreqCellList_r9 */
static int hf_rrc_cSGInterFreqCellInfoList = -1;  /* CSGInterFreqCellInfoList */
static int hf_rrc_interFreqSIAcquisition = -1;    /* InterFreqSIAcquisition */
static int hf_rrc_newInterFreqCellList_06 = -1;   /* NewInterFreqCellList_r10 */
static int hf_rrc_modeSpecificInfo_111 = -1;      /* T_modeSpecificInfo_111 */
static int hf_rrc_fdd_156 = -1;                   /* T_fdd_150 */
static int hf_rrc_primaryScramblingCode_01 = -1;  /* PrimaryCPICH_Info */
static int hf_rrc_newInterFreqCellList_07 = -1;   /* NewInterFreqCellSI_List_RSCP */
static int hf_rrc_newInterFreqCellList_08 = -1;   /* NewInterFreqCellSI_List_ECN0 */
static int hf_rrc_newInterFreqCellList_09 = -1;   /* NewInterFreqCellSI_List_HCS_RSCP */
static int hf_rrc_newInterFreqCellList_10 = -1;   /* NewInterFreqCellSI_List_HCS_ECN0 */
static int hf_rrc_newInterFreqCellList_11 = -1;   /* NewInterFreqCellSI_List_RSCP_LCR_r4 */
static int hf_rrc_newInterFreqCellList_12 = -1;   /* NewInterFreqCellSI_List_ECN0_LCR_r4 */
static int hf_rrc_newInterFreqCellList_13 = -1;   /* NewInterFreqCellSI_List_HCS_RSCP_LCR_r4 */
static int hf_rrc_newInterFreqCellList_14 = -1;   /* NewInterFreqCellSI_List_HCS_ECN0_LCR_r4 */
static int hf_rrc_InterFreqCellList_item = -1;    /* InterFreqCell */
static int hf_rrc_InterFreqCellList_LCR_r4_ext_item = -1;  /* InterFreqCell_LCR_r4 */
static int hf_rrc_InterFreqCellMeasuredResultsList_item = -1;  /* CellMeasuredResults */
static int hf_rrc_InterFreqCellMeasuredResultsList_v920ext_item = -1;  /* CellMeasuredResults_v920ext */
static int hf_rrc_event2a = -1;                   /* Event2a */
static int hf_rrc_event2b = -1;                   /* Event2b */
static int hf_rrc_event2c = -1;                   /* Event2c */
static int hf_rrc_event2d = -1;                   /* Event2d */
static int hf_rrc_event2e = -1;                   /* Event2e */
static int hf_rrc_event2f = -1;                   /* Event2f */
static int hf_rrc_event2a_01 = -1;                /* Event2a_r6 */
static int hf_rrc_event2b_01 = -1;                /* Event2b_r6 */
static int hf_rrc_event2c_01 = -1;                /* Event2c_r6 */
static int hf_rrc_event2d_01 = -1;                /* Event2d_r6 */
static int hf_rrc_event2e_01 = -1;                /* Event2e_r6 */
static int hf_rrc_event2f_01 = -1;                /* Event2f_r6 */
static int hf_rrc_event2a_02 = -1;                /* Event2a_r10 */
static int hf_rrc_event2b_02 = -1;                /* Event2b_r10 */
static int hf_rrc_event2c_02 = -1;                /* Event2c_r10 */
static int hf_rrc_event2d_02 = -1;                /* Event2d_r10 */
static int hf_rrc_event2e_02 = -1;                /* Event2e_r10 */
static int hf_rrc_event2f_02 = -1;                /* Event2f_r10 */
static int hf_rrc_event2a_03 = -1;                /* Event2a_r11 */
static int hf_rrc_event2b_03 = -1;                /* Event2b_r11 */
static int hf_rrc_event2c_03 = -1;                /* Event2c_r11 */
static int hf_rrc_event2e_03 = -1;                /* Event2e_r11 */
static int hf_rrc_InterFreqEventList_item = -1;   /* InterFreqEvent */
static int hf_rrc_InterFreqEventList_r6_item = -1;  /* InterFreqEvent_r6 */
static int hf_rrc_InterFreqEventList_r10_item = -1;  /* InterFreqEvent_r10 */
static int hf_rrc_InterFreqEventList_r11_item = -1;  /* InterFreqEvent_r11 */
static int hf_rrc_InterFrequencyMeasuredResultsList_v590ext_item = -1;  /* DeltaRSCPPerCell */
static int hf_rrc_threholdUsedFrequency_delta = -1;  /* DeltaRSCP */
static int hf_rrc_threholdNonUsedFrequency_deltaList = -1;  /* ThreholdNonUsedFrequency_deltaList */
static int hf_rrc_Inter_FreqEventCriteriaList_v590ext_item = -1;  /* Inter_FreqEventCriteria_v590ext */
static int hf_rrc_Intra_FreqEventCriteriaList_v590ext_item = -1;  /* DeltaRSCP */
static int hf_rrc_IntraFrequencyMeasuredResultsList_v590ext_item = -1;  /* DeltaRSCPPerCell */
static int hf_rrc_eventID_01 = -1;                /* EventIDInterFreq */
static int hf_rrc_interFreqCellList = -1;         /* InterFreqCellList */
static int hf_rrc_detectedSetTrigger = -1;        /* T_detectedSetTrigger */
static int hf_rrc_interFreqCellList_01 = -1;      /* InterFreqCellList_LCR_r4_ext */
static int hf_rrc_reportingCriteria = -1;         /* T_reportingCriteria */
static int hf_rrc_intraFreqReportingCriteria_02 = -1;  /* T_intraFreqReportingCriteria */
static int hf_rrc_intraFreqMeasQuantity = -1;     /* IntraFreqMeasQuantity */
static int hf_rrc_interFreqReportingCriteria = -1;  /* T_interFreqReportingCriteria */
static int hf_rrc_modeSpecificInfo_112 = -1;      /* T_modeSpecificInfo_112 */
static int hf_rrc_fdd_157 = -1;                   /* T_fdd_151 */
static int hf_rrc_freqQualityEstimateQuantity_FDD = -1;  /* FreqQualityEstimateQuantity_FDD */
static int hf_rrc_tdd_108 = -1;                   /* T_tdd_100 */
static int hf_rrc_freqQualityEstimateQuantity_TDD = -1;  /* FreqQualityEstimateQuantity_TDD */
static int hf_rrc_utra_CarrierRSSI = -1;          /* UTRA_CarrierRSSI */
static int hf_rrc_interFreqCellMeasuredResultsList = -1;  /* InterFreqCellMeasuredResultsList */
static int hf_rrc_interFreqCellMeasuredResultsList_01 = -1;  /* InterFreqCellMeasuredResultsList_v920ext */
static int hf_rrc_InterFreqMeasuredResultsList_item = -1;  /* InterFreqMeasuredResults */
static int hf_rrc_InterFreqMeasuredResultsList_v920ext_item = -1;  /* InterFreqMeasuredResults_v920ext */
static int hf_rrc_interFreqCellInfoSI_List = -1;  /* InterFreqCellInfoSI_List_RSCP */
static int hf_rrc_interFreqCellInfoSI_List_01 = -1;  /* InterFreqCellInfoSI_List_ECN0 */
static int hf_rrc_interFreqCellInfoSI_List_02 = -1;  /* InterFreqCellInfoSI_List_HCS_RSCP */
static int hf_rrc_interFreqCellInfoSI_List_03 = -1;  /* InterFreqCellInfoSI_List_HCS_ECN0 */
static int hf_rrc_interFreqCellInfoSI_List_04 = -1;  /* InterFreqCellInfoSI_List_RSCP_LCR */
static int hf_rrc_interFreqCellInfoSI_List_05 = -1;  /* InterFreqCellInfoSI_List_ECN0_LCR */
static int hf_rrc_interFreqCellInfoSI_List_06 = -1;  /* InterFreqCellInfoSI_List_HCS_RSCP_LCR */
static int hf_rrc_interFreqCellInfoSI_List_07 = -1;  /* InterFreqCellInfoSI_List_HCS_ECN0_LCR */
static int hf_rrc_InterFreqRACHRepCellsList_item = -1;  /* InterFreqCellID */
static int hf_rrc_modeSpecificInfo_113 = -1;      /* T_modeSpecificInfo_113 */
static int hf_rrc_fdd_158 = -1;                   /* T_fdd_152 */
static int hf_rrc_interFreqRepQuantityRACH_FDD = -1;  /* InterFreqRepQuantityRACH_FDD */
static int hf_rrc_tdd_109 = -1;                   /* T_tdd_101 */
static int hf_rrc_interFreqRepQuantityRACH_TDDList = -1;  /* InterFreqRepQuantityRACH_TDDList */
static int hf_rrc_interFreqRACHReportingThreshold = -1;  /* Threshold */
static int hf_rrc_maxReportedCellsOnRACHinterFreq = -1;  /* MaxReportedCellsOnRACHinterFreq */
static int hf_rrc_modeSpecificInfo_114 = -1;      /* T_modeSpecificInfo_114 */
static int hf_rrc_fdd_159 = -1;                   /* T_fdd_153 */
static int hf_rrc_tdd_110 = -1;                   /* T_tdd_102 */
static int hf_rrc_interFreqReportingCriteria_01 = -1;  /* InterFreqReportingCriteria */
static int hf_rrc_periodicalReportingCriteria_01 = -1;  /* PeriodicalWithReportingCellStatus */
static int hf_rrc_noReporting = -1;               /* ReportingCellStatusOpt */
static int hf_rrc_intraFreqReportingCriteria_03 = -1;  /* IntraFreqReportingCriteria_r4 */
static int hf_rrc_intraFreqReportingCriteria_04 = -1;  /* IntraFreqReportingCriteria_r6 */
static int hf_rrc_interFreqReportingCriteria_02 = -1;  /* InterFreqReportingCriteria_r6 */
static int hf_rrc_intraFreqReportingCriteria_05 = -1;  /* IntraFreqReportingCriteria_r7 */
static int hf_rrc_intraFreqReportingCriteria_06 = -1;  /* IntraFreqReportingCriteria_r9 */
static int hf_rrc_interFreqReportingCriteria_03 = -1;  /* InterFreqReportingCriteria_r10 */
static int hf_rrc_periodicalReportingCriteria_02 = -1;  /* PeriodicalWithReportingCellStatus_r10 */
static int hf_rrc_noReporting_01 = -1;            /* ReportingCellStatusOpt_r10 */
static int hf_rrc_intraFreqReportingCriteria_07 = -1;  /* IntraFreqReportingCriteria_r11 */
static int hf_rrc_interFreqReportingCriteria_04 = -1;  /* InterFreqReportingCriteria_r11 */
static int hf_rrc_interFreqEventList = -1;        /* InterFreqEventList */
static int hf_rrc_interFreqEventList_01 = -1;     /* InterFreqEventList_r6 */
static int hf_rrc_interFreqEventList_02 = -1;     /* InterFreqEventList_r10 */
static int hf_rrc_interFreqEventList_03 = -1;     /* InterFreqEventList_r11 */
static int hf_rrc_utra_Carrier_RSSI = -1;         /* BOOLEAN */
static int hf_rrc_frequencyQualityEstimate = -1;  /* BOOLEAN */
static int hf_rrc_nonFreqRelatedQuantities = -1;  /* CellReportingQuantities */
static int hf_rrc_InterFreqRepQuantityRACH_TDDList_item = -1;  /* InterFreqRepQuantityRACH_TDD */
static int hf_rrc_interFreqCellInfoList = -1;     /* InterFreqCellInfoList */
static int hf_rrc_interFreqMeasQuantity = -1;     /* InterFreqMeasQuantity */
static int hf_rrc_interFreqReportingQuantity = -1;  /* InterFreqReportingQuantity */
static int hf_rrc_measurementValidity = -1;       /* MeasurementValidity */
static int hf_rrc_interFreqSetUpdate = -1;        /* UE_AutonomousUpdateMode */
static int hf_rrc_reportCriteria_01 = -1;         /* InterFreqReportCriteria */
static int hf_rrc_interFreqCellInfoList_01 = -1;  /* InterFreqCellInfoList_r4 */
static int hf_rrc_reportCriteria_02 = -1;         /* InterFreqReportCriteria_r4 */
static int hf_rrc_reportCriteria_03 = -1;         /* InterFreqReportCriteria_r6 */
static int hf_rrc_reportCriteria_04 = -1;         /* InterFreqReportCriteria_r7 */
static int hf_rrc_interFreqCellInfoList_02 = -1;  /* InterFreqCellInfoList_r8 */
static int hf_rrc_adjacentFrequencyIndex = -1;    /* INTEGER_0_31 */
static int hf_rrc_interFreqCellInfoList_03 = -1;  /* InterFreqCellInfoList_r9 */
static int hf_rrc_interBandFrequencyIndex = -1;   /* INTEGER_0_31 */
static int hf_rrc_reportCriteria_05 = -1;         /* InterFreqReportCriteria_r9 */
static int hf_rrc_interFreqCellInfoList_04 = -1;  /* InterFreqCellInfoList_r10 */
static int hf_rrc_freqIndexListForEnhancedMeas = -1;  /* FreqIndexListForEnhancedMeas */
static int hf_rrc_reportCriteria_06 = -1;         /* InterFreqReportCriteria_r10 */
static int hf_rrc_reportCriteria_07 = -1;         /* InterFreqReportCriteria_r11 */
static int hf_rrc_technologySpecificInfo = -1;    /* T_technologySpecificInfo */
static int hf_rrc_gsm_08 = -1;                    /* T_gsm_08 */
static int hf_rrc_bsic = -1;                      /* BSIC */
static int hf_rrc_bcch_ARFCN = -1;                /* BCCH_ARFCN */
static int hf_rrc_ncMode = -1;                    /* NC_Mode */
static int hf_rrc_is_2000 = -1;                   /* NULL */
static int hf_rrc_removedInterRATCellList = -1;   /* RemovedInterRATCellList */
static int hf_rrc_newInterRATCellList = -1;       /* NewInterRATCellList */
static int hf_rrc_cellsForInterRATMeasList = -1;  /* CellsForInterRATMeasList */
static int hf_rrc_newInterRATCellList_01 = -1;    /* NewInterRATCellList_B */
static int hf_rrc_interRATCellInfoIndication_r6 = -1;  /* InterRATCellInfoIndication */
static int hf_rrc_event3a = -1;                   /* Event3a */
static int hf_rrc_event3b = -1;                   /* Event3b */
static int hf_rrc_event3c = -1;                   /* Event3c */
static int hf_rrc_event3d = -1;                   /* Event3d */
static int hf_rrc_InterRATEventList_item = -1;    /* InterRATEvent */
static int hf_rrc_cellToReportList = -1;          /* CellToReportList */
static int hf_rrc_rat_01 = -1;                    /* InterRATInfo */
static int hf_rrc_gsm_TargetCellInfoList = -1;    /* GSM_TargetCellInfoList */
static int hf_rrc_gsm_09 = -1;                    /* NULL */
static int hf_rrc_eutra_01 = -1;                  /* T_eutra_01 */
static int hf_rrc_eutra_TargetFreqInfoList = -1;  /* EUTRA_TargetFreqInfoList */
static int hf_rrc_measQuantityUTRAN_QualityEstimate = -1;  /* IntraFreqMeasQuantity */
static int hf_rrc_ratSpecificInfo = -1;           /* T_ratSpecificInfo */
static int hf_rrc_gsm_10 = -1;                    /* T_gsm_09 */
static int hf_rrc_measurementQuantity_01 = -1;    /* MeasurementQuantityGSM */
static int hf_rrc_bsic_VerificationRequired = -1;  /* BSIC_VerificationRequired */
static int hf_rrc_is_2000_01 = -1;                /* T_is_2000 */
static int hf_rrc_tadd_EcIo = -1;                 /* INTEGER_0_63 */
static int hf_rrc_tcomp_EcIo = -1;                /* INTEGER_0_15 */
static int hf_rrc_softSlope = -1;                 /* INTEGER_0_63 */
static int hf_rrc_addIntercept = -1;              /* INTEGER_0_63 */
static int hf_rrc_ratSpecificInfo_01 = -1;        /* T_ratSpecificInfo_01 */
static int hf_rrc_gsm_11 = -1;                    /* T_gsm_10 */
static int hf_rrc_is_2000_02 = -1;                /* T_is_2000_01 */
static int hf_rrc_e_UTRA_01 = -1;                 /* T_e_UTRA_01 */
static int hf_rrc_measurementQuantity_02 = -1;    /* MeasurementQuantityEUTRA */
static int hf_rrc_gsm_12 = -1;                    /* GSM_MeasuredResultsList */
static int hf_rrc_InterRATMeasuredResultsList_item = -1;  /* InterRATMeasuredResults */
static int hf_rrc_interRATCellInfoList = -1;      /* InterRATCellInfoList */
static int hf_rrc_interRATMeasQuantity = -1;      /* InterRATMeasQuantity */
static int hf_rrc_interRATReportingQuantity = -1;  /* InterRATReportingQuantity */
static int hf_rrc_reportCriteria_08 = -1;         /* InterRATReportCriteria */
static int hf_rrc_interRATCellInfoList_01 = -1;   /* InterRATCellInfoList_r4 */
static int hf_rrc_interRATCellInfoList_02 = -1;   /* InterRATCellInfoList_r6 */
static int hf_rrc_interRATMeasurementObjects = -1;  /* T_interRATMeasurementObjects */
static int hf_rrc_eutra_FrequencyList = -1;       /* EUTRA_FrequencyList */
static int hf_rrc_interRATMeasQuantity_01 = -1;   /* InterRATMeasQuantity_r8 */
static int hf_rrc_interRATReportingQuantity_01 = -1;  /* InterRATReportingQuantity_r8 */
static int hf_rrc_idleIntervalInfo = -1;          /* IdleIntervalInfo */
static int hf_rrc_interRATMeasurementObjects_01 = -1;  /* T_interRATMeasurementObjects_01 */
static int hf_rrc_eutra_FrequencyList_01 = -1;    /* EUTRA_FrequencyList_r9 */
static int hf_rrc_interRATMeasurementObjects_02 = -1;  /* T_interRATMeasurementObjects_02 */
static int hf_rrc_eutra_FrequencyList_02 = -1;    /* EUTRA_FrequencyList_r11 */
static int hf_rrc_interRATCellInfoList_03 = -1;   /* InterRATCellInfoList_B */
static int hf_rrc_interRATReportingCriteria = -1;  /* InterRATReportingCriteria */
static int hf_rrc_interRATEventList = -1;         /* InterRATEventList */
static int hf_rrc_utran_EstimatedQuality = -1;    /* BOOLEAN */
static int hf_rrc_ratSpecificInfo_02 = -1;        /* T_ratSpecificInfo_02 */
static int hf_rrc_gsm_13 = -1;                    /* T_gsm_11 */
static int hf_rrc_gsm_Carrier_RSSI = -1;          /* BOOLEAN */
static int hf_rrc_ratSpecificInfo_03 = -1;        /* T_ratSpecificInfo_03 */
static int hf_rrc_gsm_14 = -1;                    /* T_gsm_12 */
static int hf_rrc_eutra_02 = -1;                  /* T_eutra_02 */
static int hf_rrc_reportingQuantity = -1;         /* T_reportingQuantity */
static int hf_rrc_removedIntraFreqCellList = -1;  /* RemovedIntraFreqCellList */
static int hf_rrc_newIntraFreqCellList_01 = -1;   /* NewIntraFreqCellList */
static int hf_rrc_cellsForIntraFreqMeasList = -1;  /* CellsForIntraFreqMeasList */
static int hf_rrc_newIntraFreqCellList_02 = -1;   /* NewIntraFreqCellList_r4 */
static int hf_rrc_removedIntraFreqCellList_01 = -1;  /* RemovedIntraFreqCellListOnSecULFreq */
static int hf_rrc_newIntraFreqCellList_03 = -1;   /* NewIntraFreqCellListOnSecULFreq */
static int hf_rrc_cellsForIntraFreqMeasList_01 = -1;  /* CellsForIntraFreqMeasListOnSecULFreq */
static int hf_rrc_intraFreqCellInfoListOnSecULFreq = -1;  /* IntraFreqCellInfoListInfoOnSecULFreq */
static int hf_rrc_newIntraFreqCellList_04 = -1;   /* NewIntraFreqCellList_r9 */
static int hf_rrc_cSGIntraFreqCellInfoList = -1;  /* CSGIntraFreqCellInfoList */
static int hf_rrc_intraFreqSIAcquisition = -1;    /* IntraFreqSIAcquisition */
static int hf_rrc_newIntraFreqCellList_05 = -1;   /* NewIntraFreqCellList_r10 */
static int hf_rrc_IntraFreqSIAcquisitionInfo_item = -1;  /* PrimaryCPICH_Info */
static int hf_rrc_modeSpecificInfo_115 = -1;      /* T_modeSpecificInfo_115 */
static int hf_rrc_fdd_160 = -1;                   /* T_fdd_154 */
static int hf_rrc_intraFreqSIAcquisitionInfo = -1;  /* IntraFreqSIAcquisitionInfo */
static int hf_rrc_newIntraFreqCellList_06 = -1;   /* NewIntraFreqCellSI_List_RSCP */
static int hf_rrc_newIntraFreqCellList_07 = -1;   /* NewIntraFreqCellSI_List_ECN0 */
static int hf_rrc_newIntraFreqCellList_08 = -1;   /* NewIntraFreqCellSI_List_HCS_RSCP */
static int hf_rrc_newIntraFreqCellList_09 = -1;   /* NewIntraFreqCellSI_List_HCS_ECN0 */
static int hf_rrc_newIntraFreqCellList_10 = -1;   /* NewIntraFreqCellSI_List_RSCP_LCR_r4 */
static int hf_rrc_newIntraFreqCellList_11 = -1;   /* NewIntraFreqCellSI_List_ECN0_LCR_r4 */
static int hf_rrc_newIntraFreqCellList_12 = -1;   /* NewIntraFreqCellSI_List_HCS_RSCP_LCR_r4 */
static int hf_rrc_newIntraFreqCellList_13 = -1;   /* NewIntraFreqCellSI_List_HCS_ECN0_LCR_r4 */
static int hf_rrc_e1a = -1;                       /* Event1a */
static int hf_rrc_e1b = -1;                       /* Event1b */
static int hf_rrc_e1c = -1;                       /* Event1c */
static int hf_rrc_e1d = -1;                       /* NULL */
static int hf_rrc_e1e = -1;                       /* Event1e */
static int hf_rrc_e1f = -1;                       /* Event1f */
static int hf_rrc_e1g = -1;                       /* NULL */
static int hf_rrc_e1h = -1;                       /* ThresholdUsedFrequency */
static int hf_rrc_e1i = -1;                       /* ThresholdUsedFrequency */
static int hf_rrc_e1a_01 = -1;                    /* Event1a_r4 */
static int hf_rrc_e1b_01 = -1;                    /* Event1b_r4 */
static int hf_rrc_e1a_02 = -1;                    /* Event1a_LCR_r4 */
static int hf_rrc_e1b_02 = -1;                    /* Event1b_LCR_r4 */
static int hf_rrc_e1d_01 = -1;                    /* Event1d */
static int hf_rrc_e1e_01 = -1;                    /* Event1e_r6 */
static int hf_rrc_e1f_01 = -1;                    /* Event1f_r6 */
static int hf_rrc_e1h_01 = -1;                    /* ThresholdUsedFrequency_r6 */
static int hf_rrc_e1i_01 = -1;                    /* ThresholdUsedFrequency_r6 */
static int hf_rrc_e1j = -1;                       /* Event1j_r6 */
static int hf_rrc_e1b_03 = -1;                    /* Event1b_r7 */
static int hf_rrc_e1a_03 = -1;                    /* Event1a_OnSecUlFreq_r9 */
static int hf_rrc_e1b_04 = -1;                    /* Event1b_OnSecUlFreq_r9 */
static int hf_rrc_event = -1;                     /* IntraFreqEvent */
static int hf_rrc_event_01 = -1;                  /* IntraFreqEvent_r4 */
static int hf_rrc_event_02 = -1;                  /* IntraFreqEvent_LCR_r4 */
static int hf_rrc_event_03 = -1;                  /* IntraFreqEvent_r6 */
static int hf_rrc_event_04 = -1;                  /* IntraFreqEvent_r7 */
static int hf_rrc_event_05 = -1;                  /* IntraFreqEventOnSecULFreq */
static int hf_rrc_IntraFreqEventCriteriaList_item = -1;  /* IntraFreqEventCriteria */
static int hf_rrc_IntraFreqEventCriteriaList_r4_item = -1;  /* IntraFreqEventCriteria_r4 */
static int hf_rrc_IntraFreqEventCriteriaList_LCR_r4_item = -1;  /* IntraFreqEventCriteria_LCR_r4 */
static int hf_rrc_IntraFreqEventCriteriaList_r6_item = -1;  /* IntraFreqEventCriteria_r6 */
static int hf_rrc_IntraFreqEventCriteriaList_r7_item = -1;  /* IntraFreqEventCriteria_r7 */
static int hf_rrc_intraFreqEventCriteria = -1;    /* SEQUENCE_SIZE_1_maxMeasEventOnSecULFreq_OF_IntraFreqEventCriteriaOnSecULFreq */
static int hf_rrc_intraFreqEventCriteria_item = -1;  /* IntraFreqEventCriteriaOnSecULFreq */
static int hf_rrc_eventID_02 = -1;                /* EventIDIntraFreq */
static int hf_rrc_cellMeasurementEventResults = -1;  /* CellMeasurementEventResults */
static int hf_rrc_cellMeasurementEventResultsOnSecUlFreq = -1;  /* CellMeasurementEventResultsOnSecUlFreq */
static int hf_rrc_modeSpecificInfo_116 = -1;      /* T_modeSpecificInfo_116 */
static int hf_rrc_fdd_161 = -1;                   /* T_fdd_155 */
static int hf_rrc_intraFreqMeasQuantity_FDD = -1;  /* IntraFreqMeasQuantity_FDD */
static int hf_rrc_tdd_111 = -1;                   /* T_tdd_103 */
static int hf_rrc_intraFreqMeasQuantity_TDDList = -1;  /* IntraFreqMeasQuantity_TDDList */
static int hf_rrc_IntraFreqMeasQuantity_TDDList_item = -1;  /* IntraFreqMeasQuantity_TDD */
static int hf_rrc_IntraFreqMeasQuantity_TDD_sib3List_item = -1;  /* IntraFreqMeasQuantity_TDD_sib3List_item */
static int hf_rrc_IntraFreqMeasuredResultsList_item = -1;  /* CellMeasuredResults */
static int hf_rrc_IntraFreqMeasuredResultsList_v920ext_item = -1;  /* CellMeasuredResults_v920ext */
static int hf_rrc_intraFreqMeasurementID = -1;    /* MeasurementIdentity */
static int hf_rrc_intraFreqCellInfoSI_List = -1;  /* IntraFreqCellInfoSI_List_RSCP */
static int hf_rrc_intraFreqReportingQuantityForRACH = -1;  /* IntraFreqReportingQuantityForRACH */
static int hf_rrc_maxReportedCellsOnRACH = -1;    /* MaxReportedCellsOnRACH */
static int hf_rrc_reportingInfoForCellDCH = -1;   /* ReportingInfoForCellDCH */
static int hf_rrc_intraFreqCellInfoSI_List_01 = -1;  /* IntraFreqCellInfoSI_List_ECN0 */
static int hf_rrc_intraFreqCellInfoSI_List_02 = -1;  /* IntraFreqCellInfoSI_List_HCS_RSCP */
static int hf_rrc_intraFreqCellInfoSI_List_03 = -1;  /* IntraFreqCellInfoSI_List_HCS_ECN0 */
static int hf_rrc_intraFreqCellInfoSI_List_04 = -1;  /* IntraFreqCellInfoSI_List_RSCP_LCR_r4 */
static int hf_rrc_reportingInfoForCellDCH_01 = -1;  /* ReportingInfoForCellDCH_LCR_r4 */
static int hf_rrc_intraFreqCellInfoSI_List_05 = -1;  /* IntraFreqCellInfoSI_List_ECN0_LCR_r4 */
static int hf_rrc_intraFreqCellInfoSI_List_06 = -1;  /* IntraFreqCellInfoSI_List_HCS_RSCP_LCR_r4 */
static int hf_rrc_intraFreqCellInfoSI_List_07 = -1;  /* IntraFreqCellInfoSI_List_HCS_ECN0_LCR_r4 */
static int hf_rrc_eventCriteriaList = -1;         /* IntraFreqEventCriteriaList */
static int hf_rrc_eventCriteriaList_01 = -1;      /* IntraFreqEventCriteriaList_r4 */
static int hf_rrc_eventCriteriaList_02 = -1;      /* IntraFreqEventCriteriaList_LCR_r4 */
static int hf_rrc_eventCriteriaList_03 = -1;      /* IntraFreqEventCriteriaList_r6 */
static int hf_rrc_eventCriteriaList_04 = -1;      /* IntraFreqEventCriteriaList_r7 */
static int hf_rrc_eventCriteriaListOnSecULFreq = -1;  /* IntraFreqEventCriteriaListOnSecULFreq */
static int hf_rrc_excludedCellInfoList = -1;      /* CellInfoListToBeExcluded_r11 */
static int hf_rrc_activeSetReportingQuantities = -1;  /* CellReportingQuantities */
static int hf_rrc_monitoredSetReportingQuantities = -1;  /* CellReportingQuantities */
static int hf_rrc_detectedSetReportingQuantities = -1;  /* CellReportingQuantities */
static int hf_rrc_sfn_SFN_OTD_Type = -1;          /* SFN_SFN_OTD_Type */
static int hf_rrc_modeSpecificInfo_117 = -1;      /* T_modeSpecificInfo_117 */
static int hf_rrc_fdd_162 = -1;                   /* T_fdd_156 */
static int hf_rrc_intraFreqRepQuantityRACH_FDD = -1;  /* IntraFreqRepQuantityRACH_FDD */
static int hf_rrc_tdd_112 = -1;                   /* T_tdd_104 */
static int hf_rrc_intraFreqRepQuantityRACH_TDDList = -1;  /* IntraFreqRepQuantityRACH_TDDList */
static int hf_rrc_IntraFreqRepQuantityRACH_TDDList_item = -1;  /* IntraFreqRepQuantityRACH_TDD */
static int hf_rrc_intraFreqCellInfoList = -1;     /* IntraFreqCellInfoList */
static int hf_rrc_intraFreqReportingQuantity = -1;  /* IntraFreqReportingQuantity */
static int hf_rrc_reportCriteria_09 = -1;         /* IntraFreqReportCriteria */
static int hf_rrc_intraFreqCellInfoList_01 = -1;  /* IntraFreqCellInfoList_r4 */
static int hf_rrc_reportCriteria_10 = -1;         /* IntraFreqReportCriteria_r4 */
static int hf_rrc_reportCriteria_11 = -1;         /* IntraFreqReportCriteria_r6 */
static int hf_rrc_reportCriteria_12 = -1;         /* IntraFreqReportCriteria_r7 */
static int hf_rrc_intraFreqCellInfoList_02 = -1;  /* IntraFreqCellInfoList_r9 */
static int hf_rrc_intraFreqCellInfoListOnSecULFreq_01 = -1;  /* IntraFreqCellInfoListOnSecULFreq */
static int hf_rrc_reportCriteria_13 = -1;         /* IntraFreqReportCriteria_r9 */
static int hf_rrc_intraFreqCellInfoList_03 = -1;  /* IntraFreqCellInfoList_r10 */
static int hf_rrc_reportCriteria_14 = -1;         /* IntraFreqReportCriteria_r11 */
static int hf_rrc_IntraFrequencyMeasuredResultsListOnSecULFreq_item = -1;  /* CellMeasuredResults_r9 */
static int hf_rrc_loggingAbsoluteThreshold = -1;  /* LoggingAbsoluteThreshold */
static int hf_rrc_loggingRelativeThreshold = -1;  /* LoggingRelativeThreshold */
static int hf_rrc_loggingDuration = -1;           /* T_loggingDuration */
static int hf_rrc_intraUTRAANR = -1;              /* IntraUTRAANR */
static int hf_rrc_interRATANRforEUTRAIndicator = -1;  /* T_interRATANRforEUTRAIndicator */
static int hf_rrc_interRATANRforGSMIndicator = -1;  /* T_interRATANRforGSMIndicator */
static int hf_rrc_LoggedANRReportInfoList_item = -1;  /* LoggedANRReportInfo */
static int hf_rrc_servingCell = -1;               /* CellIdentity */
static int hf_rrc_loggedCellInfo = -1;            /* LoggedCellInfo */
static int hf_rrc_loggedUTRACellInfo = -1;        /* LoggedUTRACellInfo */
static int hf_rrc_loggedEUTRACellInfo = -1;       /* LoggedEUTRACellInfo */
static int hf_rrc_loggedGSMCellInfo = -1;         /* LoggedGSMCellInfo */
static int hf_rrc_loggedConnectionEstablishmentFailureAccessInfo_FDD = -1;  /* LoggedConnectionEstablishmentFailureAccessInfo_FDD */
static int hf_rrc_loggedMeasFailedCellMeas = -1;  /* LoggedMeasServingCellMeas_FDD */
static int hf_rrc_loggedMeasIntrafreqNeighbourList = -1;  /* LoggedMeasIntrafreqNeighbourList_FDD */
static int hf_rrc_loggedMeasInterfreqList = -1;   /* LoggedMeasInterfreqList_FDD */
static int hf_rrc_loggedMeasInterRATNeighbourMeas = -1;  /* LoggedMeasInterRATNeighbourMeas */
static int hf_rrc_loggedConnectionEstablishmentFailureLocationInfo = -1;  /* LoggedConnectionEstablishmentFailureLocationInfo */
static int hf_rrc_loggedConnectionEstablishmentFailureAccessInfo_TDD = -1;  /* LoggedConnectionEstablishmentFailureAccessInfo_TDD */
static int hf_rrc_loggedMeasFailedCellMeas_01 = -1;  /* LoggedMeasServingCellMeas_TDD128 */
static int hf_rrc_loggedMeasIntrafreqNeighbourList_01 = -1;  /* LoggedMeasIntrafreqNeighbourMeasList_TDD128 */
static int hf_rrc_loggedMeasInterfreqList_01 = -1;  /* LoggedMeasInterfreqList_TDD128 */
static int hf_rrc_numberOfRRCMsgTransmitted = -1;  /* INTEGER_0_8 */
static int hf_rrc_fpachReceived = -1;             /* BOOLEAN */
static int hf_rrc_e_RUCCHFailure = -1;            /* BOOLEAN */
static int hf_rrc_location = -1;                  /* T_location */
static int hf_rrc_ellipsoidPoint = -1;            /* EllipsoidPoint */
static int hf_rrc_ellipsoidPointUncertCircle = -1;  /* EllipsoidPointUncertCircle */
static int hf_rrc_ellipsoidPointUncertEllipse = -1;  /* EllipsoidPointUncertEllipse */
static int hf_rrc_ellipsoidPointAltitude = -1;    /* EllipsoidPointAltitude */
static int hf_rrc_ellipsoidPointAltitudeEllipse = -1;  /* EllipsoidPointAltitudeEllipsoide */
static int hf_rrc_horizontalVelocity = -1;        /* HorizontalVelocity */
static int hf_rrc_timeSinceFailure = -1;          /* INTEGER_1_172800 */
static int hf_rrc_modeSpecificInfo_118 = -1;      /* T_modeSpecificInfo_118 */
static int hf_rrc_fdd_163 = -1;                   /* T_fdd_157 */
static int hf_rrc_loggedConnectionEstablishmentFailureInfo = -1;  /* LoggedConnectionEstablishmentFailureInfo_FDD */
static int hf_rrc_tdd_113 = -1;                   /* T_tdd_105 */
static int hf_rrc_loggedConnectionEstablishmentFailureInfo_01 = -1;  /* LoggedConnectionEstablishmentFailureInfo_TDD128 */
static int hf_rrc_eutraCellIdentity = -1;         /* BIT_STRING_SIZE_28 */
static int hf_rrc_gsmCellIdentity = -1;           /* BIT_STRING_SIZE_16 */
static int hf_rrc_bandIndicator_01 = -1;          /* Frequency_Band */
static int hf_rrc_absoluteTimeInfo = -1;          /* BIT_STRING_SIZE_48 */
static int hf_rrc_loggingDuration_01 = -1;        /* T_loggingDuration_01 */
static int hf_rrc_loggingInterval = -1;           /* T_loggingInterval */
static int hf_rrc_traceReference = -1;            /* TraceReference */
static int hf_rrc_traceRecordingSession = -1;     /* TraceRecordingSession */
static int hf_rrc_tce_Id = -1;                    /* TCE_Id */
static int hf_rrc_areaConfiguration = -1;         /* T_areaConfiguration */
static int hf_rrc_cellIDList = -1;                /* SEQUENCE_SIZE_1_32_OF_CellIdentity */
static int hf_rrc_cellIDList_item = -1;           /* CellIdentity */
static int hf_rrc_locationAreaList = -1;          /* SEQUENCE_SIZE_1_8_OF_LAI */
static int hf_rrc_locationAreaList_item = -1;     /* LAI */
static int hf_rrc_routingAreaList = -1;           /* SEQUENCE_SIZE_1_8_OF_RAI */
static int hf_rrc_routingAreaList_item = -1;      /* RAI */
static int hf_rrc_loggingDuration_02 = -1;        /* T_loggingDuration_02 */
static int hf_rrc_loggingInterval_01 = -1;        /* T_loggingInterval_01 */
static int hf_rrc_plmnList = -1;                  /* SEQUENCE_SIZE_1_15_OF_PLMN_Identity */
static int hf_rrc_plmnList_item = -1;             /* PLMN_Identity */
static int hf_rrc_areaConfiguration_01 = -1;      /* T_areaConfiguration_01 */
static int hf_rrc_cellIDList_01 = -1;             /* SEQUENCE_SIZE_1_32_OF_CellIDListItem */
static int hf_rrc_cellIDList_item_01 = -1;        /* CellIDListItem */
static int hf_rrc_LoggedMeasInterfreqList_FDD_item = -1;  /* LoggedMeasInterfreqInfo_FDD */
static int hf_rrc_loggedMeasInterfreqNeighbourList = -1;  /* SEQUENCE_SIZE_1_maxnumLoggedMeas_OF_LoggedMeasNeighbourInfo_FDD */
static int hf_rrc_loggedMeasInterfreqNeighbourList_item = -1;  /* LoggedMeasNeighbourInfo_FDD */
static int hf_rrc_loggedMeasInterfreqNeighbourList_01 = -1;  /* LoggedMeasInterfreqNeighbourList_TDD128 */
static int hf_rrc_loggedMeasEUTRAFreqList = -1;   /* LoggedMeasEUTRAFreqList */
static int hf_rrc_loggedMeasGSMNeighbourCellsList = -1;  /* LoggedMeasGSMNeighbourCellsList */
static int hf_rrc_LoggedMeasIntrafreqNeighbourList_FDD_item = -1;  /* LoggedMeasNeighbourInfo_FDD */
static int hf_rrc_LoggedMeasIntrafreqNeighbourMeasList_TDD128_item = -1;  /* LoggedMeasNeighbourMeas_TDD128 */
static int hf_rrc_LoggedMeasGSMNeighbourCellsList_item = -1;  /* LoggedMeasGSMNeighbourCellsinfo */
static int hf_rrc_LoggedMeasEUTRAFreqList_item = -1;  /* LoggedMeasEUTRAFreqInfo */
static int hf_rrc_loggedMeasEUTRAFreqNeighbourInfoList = -1;  /* SEQUENCE_SIZE_1_maxnumLoggedMeas_OF_LoggedMeasEUTRAFreqNeighbourInfo */
static int hf_rrc_loggedMeasEUTRAFreqNeighbourInfoList_item = -1;  /* LoggedMeasEUTRAFreqNeighbourInfo */
static int hf_rrc_relativeTimeStamp = -1;         /* INTEGER_0_7200 */
static int hf_rrc_loggedMeasServingCellMeas = -1;  /* LoggedMeasServingCellMeas_FDD */
static int hf_rrc_loggedMeasLocationInfo = -1;    /* LoggedMeasLocationInfo */
static int hf_rrc_loggedMeasServingCellMeas_01 = -1;  /* LoggedMeasServingCellMeas_FDD_vbxyext */
static int hf_rrc_loggedMeasLocationInfo_01 = -1;  /* LoggedMeasLocationInfo_vbxyext */
static int hf_rrc_loggedMeasServingCellMeas_02 = -1;  /* LoggedMeasServingCellMeas_TDD128 */
static int hf_rrc_loggedMeasServingCellMeas_03 = -1;  /* LoggedMeasServingCellMeas_TDD128_vbxyext */
static int hf_rrc_LoggedMeasInterfreqList_TDD128_item = -1;  /* LoggedMeasInterfreqInfo_TDD128 */
static int hf_rrc_LoggedMeasInterfreqNeighbourList_TDD128_item = -1;  /* LoggedMeasNeighbourMeas_TDD128 */
static int hf_rrc_loggedMeasInfoList = -1;        /* LoggedMeasInfoList */
static int hf_rrc_loggedMeasAvailable_06 = -1;    /* T_loggedMeasAvailable_06 */
static int hf_rrc_loggedMeasInfoList_01 = -1;     /* LoggedMeasInfoList_vbxyext */
static int hf_rrc_LoggedMeasInfoList_FDD_item = -1;  /* LoggedMeasInfo_FDD */
static int hf_rrc_LoggedMeasInfoList_FDD_vbxyext_item = -1;  /* LoggedMeasInfo_FDD_vbxyext */
static int hf_rrc_LoggedMeasInfoList_TDD128_item = -1;  /* LoggedMeasInfo_TDD128 */
static int hf_rrc_LoggedMeasInfoList_TDD128_vbxyext_item = -1;  /* LoggedMeasInfo_TDD128_vbxyext */
static int hf_rrc_modeSpecificInfo_119 = -1;      /* T_modeSpecificInfo_119 */
static int hf_rrc_fdd_164 = -1;                   /* T_fdd_158 */
static int hf_rrc_loggedMeasInfoList_02 = -1;     /* LoggedMeasInfoList_FDD */
static int hf_rrc_tdd_114 = -1;                   /* T_tdd_106 */
static int hf_rrc_loggedMeasInfoList_03 = -1;     /* LoggedMeasInfoList_TDD128 */
static int hf_rrc_modeSpecificInfo_120 = -1;      /* T_modeSpecificInfo_120 */
static int hf_rrc_fdd_165 = -1;                   /* T_fdd_159 */
static int hf_rrc_loggedMeasInfoList_04 = -1;     /* LoggedMeasInfoList_FDD_vbxyext */
static int hf_rrc_tdd_115 = -1;                   /* T_tdd_107 */
static int hf_rrc_loggedMeasInfoList_05 = -1;     /* LoggedMeasInfoList_TDD128_vbxyext */
static int hf_rrc_plmnIdentity = -1;              /* PLMN_Identity */
static int hf_rrc_rscpforANR = -1;                /* RSCPforANR */
static int hf_rrc_ec_N0forANR = -1;               /* Ec_N0forANR */
static int hf_rrc_toe_nav = -1;                   /* BIT_STRING_SIZE_14 */
static int hf_rrc_ganss_omega_nav = -1;           /* BIT_STRING_SIZE_32 */
static int hf_rrc_delta_n_nav = -1;               /* BIT_STRING_SIZE_16 */
static int hf_rrc_m_zero_nav = -1;                /* BIT_STRING_SIZE_32 */
static int hf_rrc_omegadot_nav = -1;              /* BIT_STRING_SIZE_24 */
static int hf_rrc_ganss_e_nav = -1;               /* BIT_STRING_SIZE_32 */
static int hf_rrc_idot_nav = -1;                  /* BIT_STRING_SIZE_14 */
static int hf_rrc_a_sqrt_nav = -1;                /* BIT_STRING_SIZE_32 */
static int hf_rrc_i_zero_nav = -1;                /* BIT_STRING_SIZE_32 */
static int hf_rrc_omega_zero_nav = -1;            /* BIT_STRING_SIZE_32 */
static int hf_rrc_c_rs_nav = -1;                  /* BIT_STRING_SIZE_16 */
static int hf_rrc_c_is_nav = -1;                  /* BIT_STRING_SIZE_16 */
static int hf_rrc_c_us_nav = -1;                  /* BIT_STRING_SIZE_16 */
static int hf_rrc_c_rc_nav = -1;                  /* BIT_STRING_SIZE_16 */
static int hf_rrc_c_ic_nav = -1;                  /* BIT_STRING_SIZE_16 */
static int hf_rrc_c_uc_nav = -1;                  /* BIT_STRING_SIZE_16 */
static int hf_rrc_intraFreqMeasuredResultsList = -1;  /* IntraFreqMeasuredResultsList */
static int hf_rrc_interFreqMeasuredResultsList = -1;  /* InterFreqMeasuredResultsList */
static int hf_rrc_interRATMeasuredResultsList = -1;  /* InterRATMeasuredResultsList */
static int hf_rrc_trafficVolumeMeasuredResultsList = -1;  /* TrafficVolumeMeasuredResultsList */
static int hf_rrc_qualityMeasuredResults = -1;    /* QualityMeasuredResults */
static int hf_rrc_ue_InternalMeasuredResults = -1;  /* UE_InternalMeasuredResults */
static int hf_rrc_ue_positioning_MeasuredResults = -1;  /* UE_Positioning_MeasuredResults */
static int hf_rrc_intraFreqMeasuredResultsList_01 = -1;  /* IntraFrequencyMeasuredResultsListOnSecULFreq */
static int hf_rrc_ue_positioning_MeasuredResults_v390ext = -1;  /* UE_Positioning_MeasuredResults_v390ext */
static int hf_rrc_intraFrequencyMeasuredResultsList = -1;  /* IntraFrequencyMeasuredResultsList_v590ext */
static int hf_rrc_interFrequencyMeasuredResultsList = -1;  /* InterFrequencyMeasuredResultsList_v590ext */
static int hf_rrc_ueInternalMeasuredResults = -1;  /* UE_InternalMeasuredResults_v770ext */
static int hf_rrc_ue_positioning_MeasuredResults_01 = -1;  /* UE_Positioning_MeasuredResults_v770ext */
static int hf_rrc_ue_positioning_MeasuredResults_02 = -1;  /* UE_Positioning_MeasuredResults_v860ext */
static int hf_rrc_MeasuredResultsList_v920ext_item = -1;  /* MeasuredResults_v920ext */
static int hf_rrc_MeasuredResultsListOnSecUlFreq_item = -1;  /* MeasuredResultsOnSecUlFreq */
static int hf_rrc_intraFreqMeasuredResultsList_02 = -1;  /* IntraFreqMeasuredResultsList_v920ext */
static int hf_rrc_interFreqMeasuredResultsList_01 = -1;  /* InterFreqMeasuredResultsList_v920ext */
static int hf_rrc_ue_InternalMeasuredResults_01 = -1;  /* UE_InternalMeasuredResults_LCR_r4 */
static int hf_rrc_ue_positioniing_MeasuredResults = -1;  /* UE_Positioning_MeasuredResults */
static int hf_rrc_MeasuredResultsList_item = -1;  /* MeasuredResults */
static int hf_rrc_MeasuredResultsList_LCR_r4_ext_item = -1;  /* MeasuredResults_LCR_r4 */
static int hf_rrc_MeasuredResultsList_v770xet_item = -1;  /* MeasuredResultsList_v770xet_item */
static int hf_rrc_MeasuredResultsList_v860ext_item = -1;  /* MeasuredResultsList_v860ext_item */
static int hf_rrc_currentCell = -1;               /* T_currentCell */
static int hf_rrc_modeSpecificInfo_121 = -1;      /* T_modeSpecificInfo_121 */
static int hf_rrc_fdd_166 = -1;                   /* T_fdd_160 */
static int hf_rrc_measurementQuantity_03 = -1;    /* T_measurementQuantity_01 */
static int hf_rrc_tdd_116 = -1;                   /* T_tdd_108 */
static int hf_rrc_timeslotISCP = -1;              /* TimeslotISCP_List */
static int hf_rrc_monitoredCells = -1;            /* MonitoredCellRACH_List */
static int hf_rrc_currentCell_DeltaRSCP = -1;     /* DeltaRSCPPerCell */
static int hf_rrc_monitoredCellRACH_List_v7g0ext = -1;  /* MonitoredCellRACH_List_v7g0ext */
static int hf_rrc_MonitoredCellRACH_List_v7g0ext_item = -1;  /* DeltaRSCPPerCell */
static int hf_rrc_interFreqCellIndication_SIB11 = -1;  /* INTEGER_0_1 */
static int hf_rrc_interFreqCellIndication_SIB12 = -1;  /* INTEGER_0_1 */
static int hf_rrc_interFreqRACHRepCellsList = -1;  /* InterFreqRACHRepCellsList */
static int hf_rrc_eutraFrequencyIndication_SIB19 = -1;  /* INTEGER_0_1 */
static int hf_rrc_setup = -1;                     /* MeasurementType */
static int hf_rrc_modify_01 = -1;                 /* T_modify_01 */
static int hf_rrc_measurementType = -1;           /* MeasurementType */
static int hf_rrc_setup_01 = -1;                  /* MeasurementType_r4 */
static int hf_rrc_modify_02 = -1;                 /* T_modify_02 */
static int hf_rrc_measurementType_01 = -1;        /* MeasurementType_r4 */
static int hf_rrc_setup_02 = -1;                  /* MeasurementType_r6 */
static int hf_rrc_modify_03 = -1;                 /* T_modify_03 */
static int hf_rrc_measurementType_02 = -1;        /* MeasurementType_r6 */
static int hf_rrc_setup_03 = -1;                  /* MeasurementType_r7 */
static int hf_rrc_modify_04 = -1;                 /* T_modify_04 */
static int hf_rrc_measurementType_03 = -1;        /* MeasurementType_r7 */
static int hf_rrc_setup_04 = -1;                  /* MeasurementType_r8 */
static int hf_rrc_modify_05 = -1;                 /* T_modify_05 */
static int hf_rrc_measurementType_04 = -1;        /* MeasurementType_r8 */
static int hf_rrc_setup_05 = -1;                  /* MeasurementType_r9 */
static int hf_rrc_modify_06 = -1;                 /* T_modify_06 */
static int hf_rrc_measurementType_05 = -1;        /* MeasurementType_r9 */
static int hf_rrc_setup_06 = -1;                  /* MeasurementType_r10 */
static int hf_rrc_modify_07 = -1;                 /* T_modify_07 */
static int hf_rrc_measurementType_06 = -1;        /* MeasurementType_r10 */
static int hf_rrc_setup_07 = -1;                  /* MeasurementType_r11 */
static int hf_rrc_modify_08 = -1;                 /* T_modify_08 */
static int hf_rrc_measurementType_07 = -1;        /* MeasurementType_r11 */
static int hf_rrc_use_of_HCS = -1;                /* T_use_of_HCS */
static int hf_rrc_hcs_not_used = -1;              /* T_hcs_not_used */
static int hf_rrc_cellSelectQualityMeasure_01 = -1;  /* T_cellSelectQualityMeasure_01 */
static int hf_rrc_cpich_RSCP_02 = -1;             /* T_cpich_RSCP */
static int hf_rrc_intraFreqMeasurementSysInfo = -1;  /* IntraFreqMeasurementSysInfo_RSCP */
static int hf_rrc_interFreqMeasurementSysInfo = -1;  /* InterFreqMeasurementSysInfo_RSCP */
static int hf_rrc_cpich_Ec_N0_02 = -1;            /* T_cpich_Ec_N0_01 */
static int hf_rrc_intraFreqMeasurementSysInfo_01 = -1;  /* IntraFreqMeasurementSysInfo_ECN0 */
static int hf_rrc_interFreqMeasurementSysInfo_01 = -1;  /* InterFreqMeasurementSysInfo_ECN0 */
static int hf_rrc_interRATMeasurementSysInfo = -1;  /* InterRATMeasurementSysInfo_B */
static int hf_rrc_hcs_used = -1;                  /* T_hcs_used */
static int hf_rrc_cellSelectQualityMeasure_02 = -1;  /* T_cellSelectQualityMeasure_02 */
static int hf_rrc_cpich_RSCP_03 = -1;             /* T_cpich_RSCP_01 */
static int hf_rrc_intraFreqMeasurementSysInfo_02 = -1;  /* IntraFreqMeasurementSysInfo_HCS_RSCP */
static int hf_rrc_interFreqMeasurementSysInfo_02 = -1;  /* InterFreqMeasurementSysInfo_HCS_RSCP */
static int hf_rrc_cpich_Ec_N0_03 = -1;            /* T_cpich_Ec_N0_02 */
static int hf_rrc_intraFreqMeasurementSysInfo_03 = -1;  /* IntraFreqMeasurementSysInfo_HCS_ECN0 */
static int hf_rrc_interFreqMeasurementSysInfo_03 = -1;  /* InterFreqMeasurementSysInfo_HCS_ECN0 */
static int hf_rrc_interRATMeasurementSysInfo_01 = -1;  /* InterRATMeasurementSysInfo */
static int hf_rrc_trafficVolumeMeasSysInfo = -1;  /* TrafficVolumeMeasSysInfo */
static int hf_rrc_dummy_52 = -1;                  /* UE_InternalMeasurementSysInfo */
static int hf_rrc_use_of_HCS_01 = -1;             /* T_use_of_HCS_01 */
static int hf_rrc_hcs_not_used_01 = -1;           /* T_hcs_not_used_01 */
static int hf_rrc_cellSelectQualityMeasure_03 = -1;  /* T_cellSelectQualityMeasure_03 */
static int hf_rrc_cpich_RSCP_04 = -1;             /* T_cpich_RSCP_02 */
static int hf_rrc_cpich_Ec_N0_04 = -1;            /* T_cpich_Ec_N0_03 */
static int hf_rrc_hcs_used_01 = -1;               /* T_hcs_used_01 */
static int hf_rrc_cellSelectQualityMeasure_04 = -1;  /* T_cellSelectQualityMeasure_04 */
static int hf_rrc_cpich_RSCP_05 = -1;             /* T_cpich_RSCP_03 */
static int hf_rrc_cpich_Ec_N0_05 = -1;            /* T_cpich_Ec_N0_04 */
static int hf_rrc_newIntraFrequencyCellInfoListAddon_r5 = -1;  /* SEQUENCE_SIZE_1_maxCellMeas_OF_CellSelectReselectInfo_v590ext */
static int hf_rrc_newIntraFrequencyCellInfoListAddon_r5_item = -1;  /* CellSelectReselectInfo_v590ext */
static int hf_rrc_newInterFrequencyCellInfoListAddon_r5 = -1;  /* SEQUENCE_SIZE_1_maxCellMeas_OF_CellSelectReselectInfo_v590ext */
static int hf_rrc_newInterFrequencyCellInfoListAddon_r5_item = -1;  /* CellSelectReselectInfo_v590ext */
static int hf_rrc_newInterRATCellInfoListAddon_r5 = -1;  /* SEQUENCE_SIZE_1_maxCellMeas_OF_CellSelectReselectInfo_v590ext */
static int hf_rrc_newInterRATCellInfoListAddon_r5_item = -1;  /* CellSelectReselectInfo_v590ext */
static int hf_rrc_use_of_HCS_02 = -1;             /* T_use_of_HCS_02 */
static int hf_rrc_hcs_not_used_02 = -1;           /* T_hcs_not_used_02 */
static int hf_rrc_cellSelectQualityMeasure_05 = -1;  /* T_cellSelectQualityMeasure_05 */
static int hf_rrc_cpich_RSCP_06 = -1;             /* T_cpich_RSCP_04 */
static int hf_rrc_cpich_Ec_N0_06 = -1;            /* T_cpich_Ec_N0_05 */
static int hf_rrc_hcs_used_02 = -1;               /* T_hcs_used_02 */
static int hf_rrc_cellSelectQualityMeasure_06 = -1;  /* T_cellSelectQualityMeasure_06 */
static int hf_rrc_cpich_RSCP_07 = -1;             /* T_cpich_RSCP_05 */
static int hf_rrc_cpich_Ec_N0_07 = -1;            /* T_cpich_Ec_N0_06 */
static int hf_rrc_use_of_HCS_03 = -1;             /* T_use_of_HCS_03 */
static int hf_rrc_hcs_not_used_03 = -1;           /* T_hcs_not_used_03 */
static int hf_rrc_cellSelectQualityMeasure_07 = -1;  /* T_cellSelectQualityMeasure_07 */
static int hf_rrc_cpich_RSCP_08 = -1;             /* T_cpich_RSCP_06 */
static int hf_rrc_intraFreqMeasurementSysInfo_04 = -1;  /* IntraFreqMeasurementSysInfo_RSCP_LCR_r4 */
static int hf_rrc_interFreqMeasurementSysInfo_04 = -1;  /* InterFreqMeasurementSysInfo_RSCP_LCR_r4 */
static int hf_rrc_cpich_Ec_N0_08 = -1;            /* T_cpich_Ec_N0_07 */
static int hf_rrc_intraFreqMeasurementSysInfo_05 = -1;  /* IntraFreqMeasurementSysInfo_ECN0_LCR_r4 */
static int hf_rrc_interFreqMeasurementSysInfo_05 = -1;  /* InterFreqMeasurementSysInfo_ECN0_LCR_r4 */
static int hf_rrc_hcs_used_03 = -1;               /* T_hcs_used_03 */
static int hf_rrc_cellSelectQualityMeasure_08 = -1;  /* T_cellSelectQualityMeasure_08 */
static int hf_rrc_cpich_RSCP_09 = -1;             /* T_cpich_RSCP_07 */
static int hf_rrc_intraFreqMeasurementSysInfo_06 = -1;  /* IntraFreqMeasurementSysInfo_HCS_RSCP_LCR_r4 */
static int hf_rrc_interFreqMeasurementSysInfo_06 = -1;  /* InterFreqMeasurementSysInfo_HCS_RSCP_LCR_r4 */
static int hf_rrc_cpich_Ec_N0_09 = -1;            /* T_cpich_Ec_N0_08 */
static int hf_rrc_intraFreqMeasurementSysInfo_07 = -1;  /* IntraFreqMeasurementSysInfo_HCS_ECN0_LCR_r4 */
static int hf_rrc_interFreqMeasurementSysInfo_07 = -1;  /* InterFreqMeasurementSysInfo_HCS_ECN0_LCR_r4 */
static int hf_rrc_measurement_Occasion_Coeff = -1;  /* INTEGER_1_9 */
static int hf_rrc_measurement_Occasion_Offset = -1;  /* INTEGER_0_511 */
static int hf_rrc_measurement_Occasion_Length = -1;  /* INTEGER_1_512 */
static int hf_rrc_timeslotBitmap = -1;            /* BIT_STRING_SIZE_7 */
static int hf_rrc_measurementReportTransferMode = -1;  /* TransferMode */
static int hf_rrc_periodicalOrEventTrigger = -1;  /* PeriodicalOrEventTrigger */
static int hf_rrc_intraFrequencyMeasurement = -1;  /* IntraFrequencyMeasurement */
static int hf_rrc_interFrequencyMeasurement = -1;  /* InterFrequencyMeasurement */
static int hf_rrc_interRATMeasurement = -1;       /* InterRATMeasurement */
static int hf_rrc_ue_positioning_Measurement = -1;  /* UE_Positioning_Measurement */
static int hf_rrc_trafficVolumeMeasurement = -1;  /* TrafficVolumeMeasurement */
static int hf_rrc_qualityMeasurement = -1;        /* QualityMeasurement */
static int hf_rrc_ue_InternalMeasurement = -1;    /* UE_InternalMeasurement */
static int hf_rrc_intraFrequencyMeasurement_01 = -1;  /* IntraFrequencyMeasurement_r4 */
static int hf_rrc_interFrequencyMeasurement_01 = -1;  /* InterFrequencyMeasurement_r4 */
static int hf_rrc_interRATMeasurement_01 = -1;    /* InterRATMeasurement_r4 */
static int hf_rrc_up_Measurement = -1;            /* UE_Positioning_Measurement_r4 */
static int hf_rrc_ue_InternalMeasurement_01 = -1;  /* UE_InternalMeasurement_r4 */
static int hf_rrc_intraFrequencyMeasurement_02 = -1;  /* IntraFrequencyMeasurement_r6 */
static int hf_rrc_interFrequencyMeasurement_02 = -1;  /* InterFrequencyMeasurement_r6 */
static int hf_rrc_interRATMeasurement_02 = -1;    /* InterRATMeasurement_r6 */
static int hf_rrc_intraFrequencyMeasurement_03 = -1;  /* IntraFrequencyMeasurement_r7 */
static int hf_rrc_interFrequencyMeasurement_03 = -1;  /* InterFrequencyMeasurement_r7 */
static int hf_rrc_up_Measurement_01 = -1;         /* UE_Positioning_Measurement_r7 */
static int hf_rrc_interFrequencyMeasurement_04 = -1;  /* InterFrequencyMeasurement_r8 */
static int hf_rrc_interRATMeasurement_03 = -1;    /* InterRATMeasurement_r8 */
static int hf_rrc_up_Measurement_02 = -1;         /* UE_Positioning_Measurement_r8 */
static int hf_rrc_intraFrequencyMeasurement_04 = -1;  /* IntraFrequencyMeasurement_r9 */
static int hf_rrc_interFrequencyMeasurement_05 = -1;  /* InterFrequencyMeasurement_r9 */
static int hf_rrc_interRATMeasurement_04 = -1;    /* InterRATMeasurement_r9 */
static int hf_rrc_up_Measurement_03 = -1;         /* UE_Positioning_Measurement_r9 */
static int hf_rrc_csgProximityDetection = -1;     /* CSGProximityDetection */
static int hf_rrc_intraFrequencyMeasurement_05 = -1;  /* IntraFrequencyMeasurement_r10 */
static int hf_rrc_interFrequencyMeasurement_06 = -1;  /* InterFrequencyMeasurement_r10 */
static int hf_rrc_up_Measurement_04 = -1;         /* UE_Positioning_Measurement_r10 */
static int hf_rrc_intraFrequencyMeasurement_06 = -1;  /* IntraFrequencyMeasurement_r11 */
static int hf_rrc_interFrequencyMeasurement_07 = -1;  /* InterFrequencyMeasurement_r11 */
static int hf_rrc_interRATMeasurement_05 = -1;    /* InterRATMeasurement_r11 */
static int hf_rrc_eutraMeasurementForCELLFACH = -1;  /* EutraMeasurementForCELLFACH */
static int hf_rrc_ue_State = -1;                  /* T_ue_State */
static int hf_rrc_MonitoredCellRACH_List_item = -1;  /* MonitoredCellRACH_Result */
static int hf_rrc_sfn_SFN_ObsTimeDifference = -1;  /* SFN_SFN_ObsTimeDifference */
static int hf_rrc_modeSpecificInfo_122 = -1;      /* T_modeSpecificInfo_122 */
static int hf_rrc_fdd_167 = -1;                   /* T_fdd_161 */
static int hf_rrc_measurementQuantity_04 = -1;    /* T_measurementQuantity_02 */
static int hf_rrc_tdd_117 = -1;                   /* T_tdd_109 */
static int hf_rrc_MultipleEUTRAFrequencyBandIndicatorList_item = -1;  /* RadioFrequencyBandEUTRA */
static int hf_rrc_multipleEUTRAFrequencyBandIndicatorlist = -1;  /* MultipleEUTRAFrequencyBandIndicatorList */
static int hf_rrc_MultipleEUTRAFrequencyInfoList_item = -1;  /* MultipleEUTRAFrequencyBandInfo */
static int hf_rrc_MultipleFrequencyBandIndicatorListFDD_item = -1;  /* FrequencyBandsIndicatorFDD */
static int hf_rrc_multipleFrequencyBandIndicatorListFDD = -1;  /* MultipleFrequencyBandIndicatorListFDD */
static int hf_rrc_MultipleFrequencyInfoListFDD_item = -1;  /* MultipleFrequencyBandInfo */
static int hf_rrc_n_CR = -1;                      /* INTEGER_1_16 */
static int hf_rrc_t_CRMaxHyst = -1;               /* T_CRMaxHyst */
static int hf_rrc_navToc = -1;                    /* BIT_STRING_SIZE_16 */
static int hf_rrc_navaf2 = -1;                    /* BIT_STRING_SIZE_8 */
static int hf_rrc_navaf1 = -1;                    /* BIT_STRING_SIZE_16 */
static int hf_rrc_navaf0 = -1;                    /* BIT_STRING_SIZE_22 */
static int hf_rrc_navTgd = -1;                    /* BIT_STRING_SIZE_8 */
static int hf_rrc_satelliteStatus = -1;           /* SatelliteStatus */
static int hf_rrc_ephemerisParameter = -1;        /* EphemerisParameter */
static int hf_rrc_NavigationModelSatInfoList_item = -1;  /* NavigationModelSatInfo */
static int hf_rrc_cnavURAindex = -1;              /* BIT_STRING_SIZE_5 */
static int hf_rrc_cnavDeltaA = -1;                /* BIT_STRING_SIZE_26 */
static int hf_rrc_cnavAdot = -1;                  /* BIT_STRING_SIZE_25 */
static int hf_rrc_cnavDeltaNo = -1;               /* BIT_STRING_SIZE_17 */
static int hf_rrc_cnavDeltaNoDot = -1;            /* BIT_STRING_SIZE_23 */
static int hf_rrc_cnavMo = -1;                    /* BIT_STRING_SIZE_33 */
static int hf_rrc_cnavE = -1;                     /* BIT_STRING_SIZE_33 */
static int hf_rrc_cnavOmega = -1;                 /* BIT_STRING_SIZE_33 */
static int hf_rrc_cnavOMEGA0 = -1;                /* BIT_STRING_SIZE_33 */
static int hf_rrc_cnavDeltaOmegaDot = -1;         /* BIT_STRING_SIZE_17 */
static int hf_rrc_cnavIo = -1;                    /* BIT_STRING_SIZE_33 */
static int hf_rrc_cnavIoDot = -1;                 /* BIT_STRING_SIZE_15 */
static int hf_rrc_cnavCis = -1;                   /* BIT_STRING_SIZE_16 */
static int hf_rrc_cnavCic = -1;                   /* BIT_STRING_SIZE_16 */
static int hf_rrc_cnavCrs = -1;                   /* BIT_STRING_SIZE_24 */
static int hf_rrc_cnavCrc = -1;                   /* BIT_STRING_SIZE_24 */
static int hf_rrc_cnavCus = -1;                   /* BIT_STRING_SIZE_21 */
static int hf_rrc_cnavCuc = -1;                   /* BIT_STRING_SIZE_21 */
static int hf_rrc_gloEn = -1;                     /* BIT_STRING_SIZE_5 */
static int hf_rrc_gloP1 = -1;                     /* BIT_STRING_SIZE_2 */
static int hf_rrc_gloP2 = -1;                     /* BIT_STRING_SIZE_1 */
static int hf_rrc_gloM = -1;                      /* BIT_STRING_SIZE_2 */
static int hf_rrc_gloX = -1;                      /* BIT_STRING_SIZE_27 */
static int hf_rrc_gloXdot = -1;                   /* BIT_STRING_SIZE_24 */
static int hf_rrc_gloXdotdot = -1;                /* BIT_STRING_SIZE_5 */
static int hf_rrc_gloY = -1;                      /* BIT_STRING_SIZE_27 */
static int hf_rrc_gloYdot = -1;                   /* BIT_STRING_SIZE_24 */
static int hf_rrc_gloYdotdot = -1;                /* BIT_STRING_SIZE_5 */
static int hf_rrc_gloZ = -1;                      /* BIT_STRING_SIZE_27 */
static int hf_rrc_gloZdot = -1;                   /* BIT_STRING_SIZE_24 */
static int hf_rrc_gloZdotdot = -1;                /* BIT_STRING_SIZE_5 */
static int hf_rrc_navURA = -1;                    /* BIT_STRING_SIZE_4 */
static int hf_rrc_navFitFlag = -1;                /* BIT_STRING_SIZE_1 */
static int hf_rrc_navToe = -1;                    /* BIT_STRING_SIZE_16 */
static int hf_rrc_navOmega = -1;                  /* BIT_STRING_SIZE_32 */
static int hf_rrc_navDeltaN = -1;                 /* BIT_STRING_SIZE_16 */
static int hf_rrc_navM0 = -1;                     /* BIT_STRING_SIZE_32 */
static int hf_rrc_navOmegaADot = -1;              /* BIT_STRING_SIZE_24 */
static int hf_rrc_navE = -1;                      /* BIT_STRING_SIZE_32 */
static int hf_rrc_navIDot = -1;                   /* BIT_STRING_SIZE_14 */
static int hf_rrc_navAPowerHalf = -1;             /* BIT_STRING_SIZE_32 */
static int hf_rrc_navI0 = -1;                     /* BIT_STRING_SIZE_32 */
static int hf_rrc_navOmegaA0 = -1;                /* BIT_STRING_SIZE_32 */
static int hf_rrc_navCrs = -1;                    /* BIT_STRING_SIZE_16 */
static int hf_rrc_navCis = -1;                    /* BIT_STRING_SIZE_16 */
static int hf_rrc_navCus = -1;                    /* BIT_STRING_SIZE_16 */
static int hf_rrc_navCrc = -1;                    /* BIT_STRING_SIZE_16 */
static int hf_rrc_navCic = -1;                    /* BIT_STRING_SIZE_16 */
static int hf_rrc_navCuc = -1;                    /* BIT_STRING_SIZE_16 */
static int hf_rrc_sbasTo = -1;                    /* BIT_STRING_SIZE_13 */
static int hf_rrc_sbasAccuracy = -1;              /* BIT_STRING_SIZE_4 */
static int hf_rrc_sbasXg = -1;                    /* BIT_STRING_SIZE_30 */
static int hf_rrc_sbasYg = -1;                    /* BIT_STRING_SIZE_30 */
static int hf_rrc_sbasZg = -1;                    /* BIT_STRING_SIZE_25 */
static int hf_rrc_sbasXgDot = -1;                 /* BIT_STRING_SIZE_17 */
static int hf_rrc_sbasYgDot = -1;                 /* BIT_STRING_SIZE_17 */
static int hf_rrc_sbasZgDot = -1;                 /* BIT_STRING_SIZE_18 */
static int hf_rrc_sbasXgDotDot = -1;              /* BIT_STRING_SIZE_10 */
static int hf_rrc_sbagYgDotDot = -1;              /* BIT_STRING_SIZE_10 */
static int hf_rrc_sbasZgDotDot = -1;              /* BIT_STRING_SIZE_10 */
static int hf_rrc_codeOnL2 = -1;                  /* BIT_STRING_SIZE_2 */
static int hf_rrc_uraIndex = -1;                  /* BIT_STRING_SIZE_4 */
static int hf_rrc_satHealth_01 = -1;              /* BIT_STRING_SIZE_6 */
static int hf_rrc_iodc = -1;                      /* BIT_STRING_SIZE_10 */
static int hf_rrc_l2Pflag = -1;                   /* BIT_STRING_SIZE_1 */
static int hf_rrc_sf1Revd = -1;                   /* SubFrame1Reserved */
static int hf_rrc_t_GD = -1;                      /* BIT_STRING_SIZE_8 */
static int hf_rrc_t_oc = -1;                      /* BIT_STRING_SIZE_16 */
static int hf_rrc_af2 = -1;                       /* BIT_STRING_SIZE_8 */
static int hf_rrc_af1_01 = -1;                    /* BIT_STRING_SIZE_16 */
static int hf_rrc_af0_01 = -1;                    /* BIT_STRING_SIZE_22 */
static int hf_rrc_c_rs = -1;                      /* BIT_STRING_SIZE_16 */
static int hf_rrc_delta_n = -1;                   /* BIT_STRING_SIZE_16 */
static int hf_rrc_m0_01 = -1;                     /* BIT_STRING_SIZE_32 */
static int hf_rrc_c_uc = -1;                      /* BIT_STRING_SIZE_16 */
static int hf_rrc_e_01 = -1;                      /* BIT_STRING_SIZE_32 */
static int hf_rrc_c_us = -1;                      /* BIT_STRING_SIZE_16 */
static int hf_rrc_a_Sqrt_01 = -1;                 /* BIT_STRING_SIZE_32 */
static int hf_rrc_t_oe = -1;                      /* BIT_STRING_SIZE_16 */
static int hf_rrc_fitInterval = -1;               /* BIT_STRING_SIZE_1 */
static int hf_rrc_aodo = -1;                      /* BIT_STRING_SIZE_5 */
static int hf_rrc_c_ic = -1;                      /* BIT_STRING_SIZE_16 */
static int hf_rrc_omega0_01 = -1;                 /* BIT_STRING_SIZE_32 */
static int hf_rrc_c_is = -1;                      /* BIT_STRING_SIZE_16 */
static int hf_rrc_i0 = -1;                        /* BIT_STRING_SIZE_32 */
static int hf_rrc_c_rc = -1;                      /* BIT_STRING_SIZE_16 */
static int hf_rrc_omega_01 = -1;                  /* BIT_STRING_SIZE_32 */
static int hf_rrc_omegaDot_01 = -1;               /* BIT_STRING_SIZE_24 */
static int hf_rrc_iDot = -1;                      /* BIT_STRING_SIZE_14 */
static int hf_rrc_modeSpecificInfo_123 = -1;      /* T_modeSpecificInfo_123 */
static int hf_rrc_fdd_168 = -1;                   /* T_fdd_162 */
static int hf_rrc_neighbourIdentity = -1;         /* PrimaryCPICH_Info */
static int hf_rrc_uE_RX_TX_TimeDifferenceType2Info = -1;  /* UE_RX_TX_TimeDifferenceType2Info */
static int hf_rrc_tdd_118 = -1;                   /* T_tdd_110 */
static int hf_rrc_neighbourAndChannelIdentity = -1;  /* CellAndChannelIdentity */
static int hf_rrc_neighbourQuality = -1;          /* NeighbourQuality */
static int hf_rrc_sfn_SFN_ObsTimeDifference2 = -1;  /* SFN_SFN_ObsTimeDifference2 */
static int hf_rrc_modeSpecificInfo_124 = -1;      /* T_modeSpecificInfo_124 */
static int hf_rrc_fdd_169 = -1;                   /* T_fdd_163 */
static int hf_rrc_NeighbourList_item = -1;        /* Neighbour */
static int hf_rrc_NeighbourList_TDD_r7_item = -1;  /* Neighbour_TDD_r7 */
static int hf_rrc_NeighbourList_v390ext_item = -1;  /* Neighbour_v390ext */
static int hf_rrc_ue_Positioning_OTDOA_Quality = -1;  /* UE_Positioning_OTDOA_Quality */
static int hf_rrc_interFreqCellID = -1;           /* InterFreqCellID */
static int hf_rrc_cellInfo = -1;                  /* CellInfo */
static int hf_rrc_cellInfo_01 = -1;               /* CellInfo_r4 */
static int hf_rrc_intraSecondaryFreqIndicator = -1;  /* BOOLEAN */
static int hf_rrc_cellInfo_LCR_r8 = -1;           /* CellInfo_LCR_r8_ext */
static int hf_rrc_modeSpecificInfo_125 = -1;      /* T_modeSpecificInfo_125 */
static int hf_rrc_noInfo = -1;                    /* NULL */
static int hf_rrc_tdd128_64 = -1;                 /* T_tdd128_62 */
static int hf_rrc_cellInfo_02 = -1;               /* CellInfo_r9 */
static int hf_rrc_modeSpecificInfo_126 = -1;      /* T_modeSpecificInfo_126 */
static int hf_rrc_tdd128_65 = -1;                 /* T_tdd128_63 */
static int hf_rrc_modeSpecificInfo_127 = -1;      /* T_modeSpecificInfo_127 */
static int hf_rrc_tdd128_66 = -1;                 /* T_tdd128_64 */
static int hf_rrc_sNPLMonitorSetIndicator_TDD128 = -1;  /* BIT_STRING_SIZE_5 */
static int hf_rrc_NewInterFreqCellList_item = -1;  /* NewInterFreqCell */
static int hf_rrc_NewInterFreqCellList_r4_item = -1;  /* NewInterFreqCell_r4 */
static int hf_rrc_NewInterFreqCellList_v7b0ext_item = -1;  /* NewInterFreqCell_v7b0ext */
static int hf_rrc_NewInterFreqCellList_LCR_v8a0ext_item = -1;  /* NewInterFreqCell_LCR_v8a0ext */
static int hf_rrc_NewInterFreqCellList_r8_item = -1;  /* NewInterFreqCell_r8 */
static int hf_rrc_NewInterFreqCellList_r9_item = -1;  /* NewInterFreqCell_r9 */
static int hf_rrc_NewInterFreqCellList_r10_item = -1;  /* NewInterFreqCell_r10 */
static int hf_rrc_cellInfo_03 = -1;               /* CellInfoSI_RSCP */
static int hf_rrc_cellInfo_04 = -1;               /* CellInfoSI_ECN0 */
static int hf_rrc_cellInfo_05 = -1;               /* CellInfoSI_HCS_RSCP */
static int hf_rrc_cellInfo_06 = -1;               /* CellInfoSI_HCS_ECN0 */
static int hf_rrc_cellInfo_07 = -1;               /* CellInfoSI_RSCP_LCR_r4 */
static int hf_rrc_cellInfo_08 = -1;               /* CellInfoSI_ECN0_LCR_r4 */
static int hf_rrc_cellInfo_09 = -1;               /* CellInfoSI_HCS_RSCP_LCR_r4 */
static int hf_rrc_cellInfo_10 = -1;               /* CellInfoSI_HCS_ECN0_LCR_r4 */
static int hf_rrc_NewInterFreqCellSI_List_ECN0_item = -1;  /* NewInterFreqCellSI_ECN0 */
static int hf_rrc_NewInterFreqCellSI_List_HCS_RSCP_item = -1;  /* NewInterFreqCellSI_HCS_RSCP */
static int hf_rrc_NewInterFreqCellSI_List_HCS_ECN0_item = -1;  /* NewInterFreqCellSI_HCS_ECN0 */
static int hf_rrc_NewInterFreqCellSI_List_RSCP_item = -1;  /* NewInterFreqCellSI_RSCP */
static int hf_rrc_NewInterFreqCellSI_List_ECN0_LCR_r4_item = -1;  /* NewInterFreqCellSI_ECN0_LCR_r4 */
static int hf_rrc_NewInterFreqCellSI_List_HCS_RSCP_LCR_r4_item = -1;  /* NewInterFreqCellSI_HCS_RSCP_LCR_r4 */
static int hf_rrc_NewInterFreqCellSI_List_HCS_ECN0_LCR_r4_item = -1;  /* NewInterFreqCellSI_HCS_ECN0_LCR_r4 */
static int hf_rrc_NewInterFreqCellSI_List_RSCP_LCR_r4_item = -1;  /* NewInterFreqCellSI_RSCP_LCR_r4 */
static int hf_rrc_interRATCellID = -1;            /* InterRATCellID */
static int hf_rrc_technologySpecificInfo_01 = -1;  /* T_technologySpecificInfo_01 */
static int hf_rrc_gsm_15 = -1;                    /* T_gsm_13 */
static int hf_rrc_cellSelectionReselectionInfo_05 = -1;  /* CellSelectReselectInfoSIB_11_12 */
static int hf_rrc_interRATCellIndividualOffset = -1;  /* InterRATCellIndividualOffset */
static int hf_rrc_is_2000_03 = -1;                /* T_is_2000_02 */
static int hf_rrc_is_2000SpecificMeasInfo = -1;   /* IS_2000SpecificMeasInfo */
static int hf_rrc_technologySpecificInfo_02 = -1;  /* T_technologySpecificInfo_02 */
static int hf_rrc_gsm_16 = -1;                    /* T_gsm_14 */
static int hf_rrc_is_2000_04 = -1;                /* T_is_2000_03 */
static int hf_rrc_NewInterRATCellList_item = -1;  /* NewInterRATCell */
static int hf_rrc_NewInterRATCellList_B_item = -1;  /* NewInterRATCell_B */
static int hf_rrc_intraFreqCellID = -1;           /* IntraFreqCellID */
static int hf_rrc_modeSpecificInfo_128 = -1;      /* T_modeSpecificInfo_128 */
static int hf_rrc_tdd128_67 = -1;                 /* T_tdd128_65 */
static int hf_rrc_intraFreqCellIDOnSecULFreq = -1;  /* IntraFreqCellIDOnSecULFreq */
static int hf_rrc_NewIntraFreqCellList_item = -1;  /* NewIntraFreqCell */
static int hf_rrc_NewIntraFreqCellList_r4_item = -1;  /* NewIntraFreqCell_r4 */
static int hf_rrc_NewIntraFreqCellList_r9_item = -1;  /* NewIntraFreqCell_r9 */
static int hf_rrc_NewIntraFreqCellList_r10_item = -1;  /* NewIntraFreqCell_r10 */
static int hf_rrc_NewIntraFreqCellList_LCR_v8a0ext_item = -1;  /* NewIntraFreqCell_LCR_v8a0ext */
static int hf_rrc_NewIntraFreqCellListOnSecULFreq_item = -1;  /* NewIntraFreqCellOnSecULFreq */
static int hf_rrc_NewIntraFreqCellSI_List_RSCP_item = -1;  /* NewIntraFreqCellSI_RSCP */
static int hf_rrc_NewIntraFreqCellSI_List_ECN0_item = -1;  /* NewIntraFreqCellSI_ECN0 */
static int hf_rrc_NewIntraFreqCellSI_List_HCS_RSCP_item = -1;  /* NewIntraFreqCellSI_HCS_RSCP */
static int hf_rrc_NewIntraFreqCellSI_List_HCS_ECN0_item = -1;  /* NewIntraFreqCellSI_HCS_ECN0 */
static int hf_rrc_NewIntraFreqCellSI_List_RSCP_LCR_r4_item = -1;  /* NewIntraFreqCellSI_RSCP_LCR_r4 */
static int hf_rrc_NewIntraFreqCellSI_List_ECN0_LCR_r4_item = -1;  /* NewIntraFreqCellSI_ECN0_LCR_r4 */
static int hf_rrc_NewIntraFreqCellSI_List_HCS_RSCP_LCR_r4_item = -1;  /* NewIntraFreqCellSI_HCS_RSCP_LCR_r4 */
static int hf_rrc_NewIntraFreqCellSI_List_HCS_ECN0_LCR_r4_item = -1;  /* NewIntraFreqCellSI_HCS_ECN0_LCR_r4 */
static int hf_rrc_nonUsedFreqThreshold = -1;      /* Threshold */
static int hf_rrc_nonUsedFreqW = -1;              /* W */
static int hf_rrc_nonUsedFreqThreshold_01 = -1;   /* Threshold_r6 */
static int hf_rrc_nonUsedFreqTriggeringConditionDetectedCells = -1;  /* T_nonUsedFreqTriggeringConditionDetectedCells */
static int hf_rrc_nonUsedFreqTriggeringConditionDetectedCells_01 = -1;  /* T_nonUsedFreqTriggeringConditionDetectedCells_01 */
static int hf_rrc_nonUsedFreqTriggeringConditionDetectedCells_02 = -1;  /* T_nonUsedFreqTriggeringConditionDetectedCells_02 */
static int hf_rrc_nonUsedFreqTriggeringConditionDetectedCells_03 = -1;  /* T_nonUsedFreqTriggeringConditionDetectedCells_03 */
static int hf_rrc_NonUsedFreqParameterList_item = -1;  /* NonUsedFreqParameter */
static int hf_rrc_NonUsedFreqParameterList_r6_item = -1;  /* NonUsedFreqParameter_r6 */
static int hf_rrc_NonUsedFreqParameterList_r10_item = -1;  /* NonUsedFreqParameter_r10 */
static int hf_rrc_NonUsedFreqParameterList_r11_item = -1;  /* NonUsedFreqParameter_r11 */
static int hf_rrc_NonUsedFreq2aParameterList_r10_item = -1;  /* NonUsedFreq2aParameter_r10 */
static int hf_rrc_NonUsedFreq2aParameterList_r11_item = -1;  /* NonUsedFreq2aParameter_r11 */
static int hf_rrc_NonUsedFreqWList_r6_item = -1;  /* W */
static int hf_rrc_notUsed = -1;                   /* NULL */
static int hf_rrc_pt10 = -1;                      /* TemporaryOffset1 */
static int hf_rrc_pt20 = -1;                      /* TemporaryOffset1 */
static int hf_rrc_pt30 = -1;                      /* TemporaryOffset1 */
static int hf_rrc_pt40 = -1;                      /* TemporaryOffset1 */
static int hf_rrc_pt50 = -1;                      /* TemporaryOffset1 */
static int hf_rrc_pt60 = -1;                      /* TemporaryOffset1 */
static int hf_rrc_pt10_01 = -1;                   /* TemporaryOffsetList */
static int hf_rrc_pt20_01 = -1;                   /* TemporaryOffsetList */
static int hf_rrc_pt30_01 = -1;                   /* TemporaryOffsetList */
static int hf_rrc_pt40_01 = -1;                   /* TemporaryOffsetList */
static int hf_rrc_pt50_01 = -1;                   /* TemporaryOffsetList */
static int hf_rrc_pt60_01 = -1;                   /* TemporaryOffsetList */
static int hf_rrc_reportingInterval_01 = -1;      /* ReportingIntervalLong */
static int hf_rrc_periodicalWithReportingCellStatus = -1;  /* PeriodicalWithReportingCellStatus_r10 */
static int hf_rrc_plmnsOfIntraFreqCellsList = -1;  /* PLMNsOfIntraFreqCellsList */
static int hf_rrc_plmnsOfInterFreqCellsList = -1;  /* PLMNsOfInterFreqCellsList */
static int hf_rrc_plmnsOfInterRATCellsList = -1;  /* PLMNsOfInterRATCellsList */
static int hf_rrc_multipleplmnsOfIntraFreqCellsList = -1;  /* MultiplePLMNsOfIntraFreqCellsList */
static int hf_rrc_multipleplmnsOfInterFreqCellsList = -1;  /* MultiplePLMNsOfInterFreqCellsList */
static int hf_rrc_PLMNsOfInterFreqCellsList_item = -1;  /* PLMNsOfInterFreqCellsList_item */
static int hf_rrc_MultiplePLMNsOfInterFreqCellsList_item = -1;  /* MultiplePLMNsOfInterFreqCellsList_item */
static int hf_rrc_multiplePLMN_list = -1;         /* SEQUENCE_SIZE_1_6_OF_PLMN_IdentityWithOptionalMCC_r6 */
static int hf_rrc_multiplePLMN_list_item = -1;    /* PLMN_IdentityWithOptionalMCC_r6 */
static int hf_rrc_PLMNsOfIntraFreqCellsList_item = -1;  /* PLMNsOfIntraFreqCellsList_item */
static int hf_rrc_MultiplePLMNsOfIntraFreqCellsList_item = -1;  /* MultiplePLMNsOfIntraFreqCellsList_item */
static int hf_rrc_PLMNsOfInterRATCellsList_item = -1;  /* PLMNsOfInterRATCellsList_item */
static int hf_rrc_QualityEventResults_item = -1;  /* TransportChannelIdentity */
static int hf_rrc_blerMeasurementResultsList = -1;  /* BLER_MeasurementResultsList */
static int hf_rrc_modeSpecificInfo_129 = -1;      /* T_modeSpecificInfo_129 */
static int hf_rrc_tdd_119 = -1;                   /* T_tdd_111 */
static int hf_rrc_sir_MeasurementResults = -1;    /* SIR_MeasurementList */
static int hf_rrc_qualityReportingQuantity = -1;  /* QualityReportingQuantity */
static int hf_rrc_reportCriteria_15 = -1;         /* QualityReportCriteria */
static int hf_rrc_qualityReportingCriteria = -1;  /* QualityReportingCriteria */
static int hf_rrc_noReporting_02 = -1;            /* NULL */
static int hf_rrc_QualityReportingCriteria_item = -1;  /* QualityReportingCriteriaSingle */
static int hf_rrc_totalCRC = -1;                  /* INTEGER_1_512 */
static int hf_rrc_badCRC = -1;                    /* INTEGER_1_512 */
static int hf_rrc_pendingAfterTrigger = -1;       /* INTEGER_1_512 */
static int hf_rrc_dl_TransChBLER = -1;            /* BOOLEAN */
static int hf_rrc_bler_dl_TransChIdList = -1;     /* BLER_TransChIdList */
static int hf_rrc_modeSpecificInfo_130 = -1;      /* T_modeSpecificInfo_130 */
static int hf_rrc_tdd_120 = -1;                   /* T_tdd_112 */
static int hf_rrc_sir_TFCS_List = -1;             /* SIR_TFCS_List */
static int hf_rrc_ellipsoidPointWithAltitude = -1;  /* EllipsoidPointAltitude */
static int hf_rrc_ellipsoidPointAltitudeEllipsoide = -1;  /* EllipsoidPointAltitudeEllipsoide */
static int hf_rrc_accuracy40 = -1;                /* INTEGER_0_960 */
static int hf_rrc_accuracy256 = -1;               /* INTEGER_0_150 */
static int hf_rrc_accuracy2560 = -1;              /* INTEGER_0_15 */
static int hf_rrc_removeAllInterFreqCells = -1;   /* NULL */
static int hf_rrc_removeSomeInterFreqCells = -1;  /* SEQUENCE_SIZE_1_maxCellMeas_OF_InterFreqCellID */
static int hf_rrc_removeSomeInterFreqCells_item = -1;  /* InterFreqCellID */
static int hf_rrc_removeNoInterFreqCells = -1;    /* NULL */
static int hf_rrc_removeAllInterRATCells = -1;    /* NULL */
static int hf_rrc_removeSomeInterRATCells = -1;   /* SEQUENCE_SIZE_1_maxCellMeas_OF_InterRATCellID */
static int hf_rrc_removeSomeInterRATCells_item = -1;  /* InterRATCellID */
static int hf_rrc_removeNoInterRATCells = -1;     /* NULL */
static int hf_rrc_removeAllIntraFreqCells = -1;   /* NULL */
static int hf_rrc_removeSomeIntraFreqCells = -1;  /* SEQUENCE_SIZE_1_maxCellMeas_OF_IntraFreqCellID */
static int hf_rrc_removeSomeIntraFreqCells_item = -1;  /* IntraFreqCellID */
static int hf_rrc_removeNoIntraFreqCells = -1;    /* NULL */
static int hf_rrc_removeSomeIntraFreqCells_01 = -1;  /* SEQUENCE_SIZE_1_maxCellMeasOnSecULFreq_OF_IntraFreqCellIDOnSecULFreq */
static int hf_rrc_removeSomeIntraFreqCells_item_01 = -1;  /* IntraFreqCellIDOnSecULFreq */
static int hf_rrc_withinActiveSet = -1;           /* MaxNumberOfReportingCellsType1 */
static int hf_rrc_withinMonitoredSetUsedFreq = -1;  /* MaxNumberOfReportingCellsType1 */
static int hf_rrc_withinActiveAndOrMonitoredUsedFreq = -1;  /* MaxNumberOfReportingCellsType1 */
static int hf_rrc_withinDetectedSetUsedFreq = -1;  /* MaxNumberOfReportingCellsType1 */
static int hf_rrc_withinMonitoredAndOrDetectedUsedFreq = -1;  /* MaxNumberOfReportingCellsType1 */
static int hf_rrc_allActiveplusMonitoredSet = -1;  /* MaxNumberOfReportingCellsType3 */
static int hf_rrc_allActivePlusDetectedSet = -1;  /* MaxNumberOfReportingCellsType3 */
static int hf_rrc_allActivePlusMonitoredAndOrDetectedSet = -1;  /* MaxNumberOfReportingCellsType3 */
static int hf_rrc_withinVirtualActSet = -1;       /* MaxNumberOfReportingCellsType1 */
static int hf_rrc_withinMonitoredSetNonUsedFreq = -1;  /* MaxNumberOfReportingCellsType1 */
static int hf_rrc_withinMonitoredAndOrVirtualActiveSetNonUsedFreq = -1;  /* MaxNumberOfReportingCellsType1 */
static int hf_rrc_allVirtualActSetplusMonitoredSetNonUsedFreq = -1;  /* MaxNumberOfReportingCellsType3 */
static int hf_rrc_withinActSetOrVirtualActSet_InterRATcells = -1;  /* MaxNumberOfReportingCellsType2 */
static int hf_rrc_withinActSetAndOrMonitoredUsedFreqOrVirtualActSetAndOrMonitoredNonUsedFreq = -1;  /* MaxNumberOfReportingCellsType2 */
static int hf_rrc_allVirtualActSetplusMonitoredSetplusDetectedSetNonUsedFreq = -1;  /* MaxNumberOfReportingCellsType3 */
static int hf_rrc_reportCriteria_16 = -1;         /* CellDCH_ReportCriteria */
static int hf_rrc_reportCriteria_17 = -1;         /* CellDCH_ReportCriteria_LCR_r4 */
static int hf_rrc_ganssSignalID = -1;             /* DGANSS_Sig_Id_Req */
static int hf_rrc_ganssDataBitInterval = -1;      /* INTEGER_0_15 */
static int hf_rrc_ganssSatelliteInfo = -1;        /* T_ganssSatelliteInfo */
static int hf_rrc_ganssSatelliteInfo_item = -1;   /* INTEGER_0_63 */
static int hf_rrc_RL_AdditionInfoList_item = -1;  /* PrimaryCPICH_Info */
static int hf_rrc_rl_AdditionInfoList = -1;       /* RL_AdditionInfoList */
static int hf_rrc_rL_RemovalInformationList = -1;  /* RL_RemovalInformationList */
static int hf_rrc_rSCP = -1;                      /* INTEGER_M120_M25 */
static int hf_rrc_SatDataList_item = -1;          /* SatData */
static int hf_rrc_iod_02 = -1;                    /* INTEGER_0_1023 */
static int hf_rrc_SatellitesListRelatedDataList_item = -1;  /* SatellitesListRelatedData */
static int hf_rrc_t_oc_01 = -1;                   /* BIT_STRING_SIZE_14 */
static int hf_rrc_af2_01 = -1;                    /* BIT_STRING_SIZE_12 */
static int hf_rrc_af1_02 = -1;                    /* BIT_STRING_SIZE_18 */
static int hf_rrc_af0_02 = -1;                    /* BIT_STRING_SIZE_28 */
static int hf_rrc_tgd_01 = -1;                    /* BIT_STRING_SIZE_10 */
static int hf_rrc_model_id = -1;                  /* INTEGER_0_1 */
static int hf_rrc_Satellite_clock_modelList_item = -1;  /* Satellite_clock_model */
static int hf_rrc_sbasAgfo = -1;                  /* BIT_STRING_SIZE_12 */
static int hf_rrc_sbasAgf1 = -1;                  /* BIT_STRING_SIZE_8 */
static int hf_rrc_s_DPCCH_DPCCH_PowerOffset = -1;  /* S_DPCCH_DPCCH_PowerOffset */
static int hf_rrc_type1_07 = -1;                  /* SFN_SFN_ObsTimeDifference1 */
static int hf_rrc_type2_06 = -1;                  /* SFN_SFN_ObsTimeDifference2 */
static int hf_rrc_sfn_Offset = -1;                /* INTEGER_0_4095 */
static int hf_rrc_sfn_sfn_Reltimedifference = -1;  /* INTEGER_0_38399 */
static int hf_rrc_SIR_MeasurementList_item = -1;  /* SIR_MeasurementResults */
static int hf_rrc_sir_TimeslotList = -1;          /* SIR_TimeslotList */
static int hf_rrc_SIR_TFCS_List_item = -1;        /* SIR_TFCS */
static int hf_rrc_SIR_TimeslotList_item = -1;     /* SIR */
static int hf_rrc_reserved1 = -1;                 /* BIT_STRING_SIZE_23 */
static int hf_rrc_reserved2 = -1;                 /* BIT_STRING_SIZE_24 */
static int hf_rrc_reserved3 = -1;                 /* BIT_STRING_SIZE_24 */
static int hf_rrc_reserved4 = -1;                 /* BIT_STRING_SIZE_16 */
static int hf_rrc_t_ADV = -1;                     /* INTEGER_0_2047 */
static int hf_rrc_sfn = -1;                       /* INTEGER_0_4095 */
static int hf_rrc_t_ADV_01 = -1;                  /* INTEGER_0_8191 */
static int hf_rrc_t30 = -1;                       /* N_CR_T_CRMaxHyst */
static int hf_rrc_t60 = -1;                       /* N_CR_T_CRMaxHyst */
static int hf_rrc_t120 = -1;                      /* N_CR_T_CRMaxHyst */
static int hf_rrc_t180 = -1;                      /* N_CR_T_CRMaxHyst */
static int hf_rrc_t240 = -1;                      /* N_CR_T_CRMaxHyst */
static int hf_rrc_tce_Id_01 = -1;                 /* OCTET_STRING_SIZE_1 */
static int hf_rrc_temporaryOffset1 = -1;          /* TemporaryOffset1 */
static int hf_rrc_temporaryOffset2 = -1;          /* TemporaryOffset2 */
static int hf_rrc_ThreholdNonUsedFrequency_deltaList_item = -1;  /* DeltaRSCPPerCell */
static int hf_rrc_TimeslotInfoList_item = -1;     /* TimeslotInfo */
static int hf_rrc_TimeslotInfoList_LCR_r4_item = -1;  /* TimeslotInfo_LCR_r4 */
static int hf_rrc_tdd384_47 = -1;                 /* SEQUENCE_SIZE_1_maxTS_OF_TimeslotInfo */
static int hf_rrc_tdd384_item_01 = -1;            /* TimeslotInfo */
static int hf_rrc_tdd128_68 = -1;                 /* SEQUENCE_SIZE_1_maxTS_LCR_OF_TimeslotInfo_LCR_r4 */
static int hf_rrc_tdd128_item_02 = -1;            /* TimeslotInfo_LCR_r4 */
static int hf_rrc_TimeslotISCP_List_item = -1;    /* TimeslotISCP */
static int hf_rrc_TimeslotListWithISCP_item = -1;  /* TimeslotWithISCP */
static int hf_rrc_timeslotISCP_01 = -1;           /* TimeslotISCP */
static int hf_rrc_traceId = -1;                   /* OCTET_STRING_SIZE_3 */
static int hf_rrc_traceRecordingSession_01 = -1;  /* OCTET_STRING_SIZE_2 */
static int hf_rrc_eventID_03 = -1;                /* TrafficVolumeEventType */
static int hf_rrc_reportingThreshold_01 = -1;     /* TrafficVolumeThreshold */
static int hf_rrc_pendingTimeAfterTrigger = -1;   /* PendingTimeAfterTrigger */
static int hf_rrc_tx_InterruptionAfterTrigger = -1;  /* TX_InterruptionAfterTrigger */
static int hf_rrc_ul_transportChannelCausingEvent = -1;  /* UL_TrCH_Identity */
static int hf_rrc_trafficVolumeEventIdentity = -1;  /* TrafficVolumeEventType */
static int hf_rrc_rlc_BufferPayload = -1;         /* NULL */
static int hf_rrc_averageRLC_BufferPayload = -1;  /* TimeInterval */
static int hf_rrc_varianceOfRLC_BufferPayload = -1;  /* TimeInterval */
static int hf_rrc_trafficVolumeMeasurementID = -1;  /* MeasurementIdentity */
static int hf_rrc_trafficVolumeMeasurementObjectList = -1;  /* TrafficVolumeMeasurementObjectList */
static int hf_rrc_trafficVolumeMeasQuantity = -1;  /* TrafficVolumeMeasQuantity */
static int hf_rrc_trafficVolumeReportingQuantity = -1;  /* TrafficVolumeReportingQuantity */
static int hf_rrc_dummy_53 = -1;                  /* TrafficVolumeReportingCriteria */
static int hf_rrc_reportCriteriaSysInf = -1;      /* TrafficVolumeReportCriteriaSysInfo */
static int hf_rrc_rlc_BuffersPayload = -1;        /* RLC_BuffersPayload */
static int hf_rrc_averageRLC_BufferPayload_01 = -1;  /* AverageRLC_BufferPayload */
static int hf_rrc_varianceOfRLC_BufferPayload_01 = -1;  /* VarianceOfRLC_BufferPayload */
static int hf_rrc_TrafficVolumeMeasuredResultsList_item = -1;  /* TrafficVolumeMeasuredResults */
static int hf_rrc_reportCriteria_18 = -1;         /* TrafficVolumeReportCriteria */
static int hf_rrc_TrafficVolumeMeasurementObjectList_item = -1;  /* UL_TrCH_Identity */
static int hf_rrc_trafficVolumeReportingCriteria = -1;  /* TrafficVolumeReportingCriteria */
static int hf_rrc_transChCriteriaList = -1;       /* TransChCriteriaList */
static int hf_rrc_rlc_RB_BufferPayload = -1;      /* BOOLEAN */
static int hf_rrc_rlc_RB_BufferPayloadAverage = -1;  /* BOOLEAN */
static int hf_rrc_rlc_RB_BufferPayloadVariance = -1;  /* BOOLEAN */
static int hf_rrc_ul_transportChannelID = -1;     /* UL_TrCH_Identity */
static int hf_rrc_eventSpecificParameters = -1;   /* SEQUENCE_SIZE_1_maxMeasParEvent_OF_TrafficVolumeEventParam */
static int hf_rrc_eventSpecificParameters_item = -1;  /* TrafficVolumeEventParam */
static int hf_rrc_TransChCriteriaList_item = -1;  /* TransChCriteria */
static int hf_rrc_transmittedPowerThreshold = -1;  /* TransmittedPowerThreshold */
static int hf_rrc_ue_RX_TX_TimeDifferenceThreshold = -1;  /* UE_RX_TX_TimeDifferenceThreshold */
static int hf_rrc_onWithNoReporting = -1;         /* NULL */
static int hf_rrc_dummy2_13 = -1;                 /* RL_InformationLists */
static int hf_rrc_event6a = -1;                   /* UE_6AB_Event */
static int hf_rrc_event6b = -1;                   /* UE_6AB_Event */
static int hf_rrc_event6c = -1;                   /* TimeToTrigger */
static int hf_rrc_event6d = -1;                   /* TimeToTrigger */
static int hf_rrc_event6e = -1;                   /* TimeToTrigger */
static int hf_rrc_event6f = -1;                   /* UE_6FG_Event */
static int hf_rrc_event6g = -1;                   /* UE_6FG_Event */
static int hf_rrc_UE_InternalEventParamList_item = -1;  /* UE_InternalEventParam */
static int hf_rrc_event6a_01 = -1;                /* NULL */
static int hf_rrc_event6b_01 = -1;                /* NULL */
static int hf_rrc_event6c_01 = -1;                /* NULL */
static int hf_rrc_event6d_01 = -1;                /* NULL */
static int hf_rrc_event6e_01 = -1;                /* NULL */
static int hf_rrc_event6f_01 = -1;                /* PrimaryCPICH_Info */
static int hf_rrc_event6g_01 = -1;                /* PrimaryCPICH_Info */
static int hf_rrc_measurementQuantity_05 = -1;    /* UE_MeasurementQuantity */
static int hf_rrc_modeSpecificInfo_131 = -1;      /* T_modeSpecificInfo_131 */
static int hf_rrc_fdd_170 = -1;                   /* T_fdd_164 */
static int hf_rrc_ue_TransmittedPowerFDD = -1;    /* UE_TransmittedPower */
static int hf_rrc_ue_RX_TX_ReportEntryList = -1;  /* UE_RX_TX_ReportEntryList */
static int hf_rrc_tdd_121 = -1;                   /* T_tdd_113 */
static int hf_rrc_ue_TransmittedPowerTDD_List = -1;  /* UE_TransmittedPowerTDD_List */
static int hf_rrc_appliedTA = -1;                 /* UL_TimingAdvance */
static int hf_rrc_t_ADVinfo = -1;                 /* T_ADVinfo */
static int hf_rrc_modeSpecificInfo_132 = -1;      /* T_modeSpecificInfo_132 */
static int hf_rrc_tdd384_768_03 = -1;             /* T_tdd384_768_02 */
static int hf_rrc_appliedTA_01 = -1;              /* EXT_UL_TimingAdvance */
static int hf_rrc_tdd128_69 = -1;                 /* T_tdd128_66 */
static int hf_rrc_t_ADVinfo_01 = -1;              /* T_ADVinfo_ext */
static int hf_rrc_ue_InternalMeasQuantity = -1;   /* UE_InternalMeasQuantity */
static int hf_rrc_ue_InternalReportingQuantity = -1;  /* UE_InternalReportingQuantity */
static int hf_rrc_reportCriteria_19 = -1;         /* UE_InternalReportCriteria */
static int hf_rrc_ue_InternalReportingQuantity_01 = -1;  /* UE_InternalReportingQuantity_r4 */
static int hf_rrc_ue_InternalMeasurementID = -1;  /* MeasurementIdentity */
static int hf_rrc_ue_InternalReportingCriteria = -1;  /* UE_InternalReportingCriteria */
static int hf_rrc_ue_InternalEventParamList = -1;  /* UE_InternalEventParamList */
static int hf_rrc_ue_TransmittedPower = -1;       /* BOOLEAN */
static int hf_rrc_modeSpecificInfo_133 = -1;      /* T_modeSpecificInfo_133 */
static int hf_rrc_fdd_171 = -1;                   /* T_fdd_165 */
static int hf_rrc_ue_RX_TX_TimeDifference = -1;   /* BOOLEAN */
static int hf_rrc_tdd_122 = -1;                   /* T_tdd_114 */
static int hf_rrc_appliedTA_02 = -1;              /* BOOLEAN */
static int hf_rrc_modeSpecificInfo_134 = -1;      /* T_modeSpecificInfo_134 */
static int hf_rrc_fdd_172 = -1;                   /* T_fdd_166 */
static int hf_rrc_tdd_123 = -1;                   /* T_tdd_115 */
static int hf_rrc_tddOption_28 = -1;              /* T_tddOption_28 */
static int hf_rrc_tdd384_48 = -1;                 /* T_tdd384_43 */
static int hf_rrc_tdd128_70 = -1;                 /* T_tdd128_67 */
static int hf_rrc_t_ADVinfo_02 = -1;              /* BOOLEAN */
static int hf_rrc_ue_RX_TX_TimeDifferenceType1 = -1;  /* UE_RX_TX_TimeDifferenceType1 */
static int hf_rrc_UE_RX_TX_ReportEntryList_item = -1;  /* UE_RX_TX_ReportEntry */
static int hf_rrc_ue_RX_TX_TimeDifferenceType2 = -1;  /* UE_RX_TX_TimeDifferenceType2 */
static int hf_rrc_UE_TransmittedPowerTDD_List_item = -1;  /* UE_TransmittedPower */
static int hf_rrc_rachorcpch = -1;                /* NULL */
static int hf_rrc_cipheringKeyFlag = -1;          /* BIT_STRING_SIZE_1 */
static int hf_rrc_cipheringSerialNumber = -1;     /* INTEGER_0_65535 */
static int hf_rrc_dganssreferencetime = -1;       /* INTEGER_0_119 */
static int hf_rrc_dganssInfoList = -1;            /* DGANSSInfoList */
static int hf_rrc_dganssInfoList_01 = -1;         /* DGANSSInfoList_r9 */
static int hf_rrc_dganssInfoList_02 = -1;         /* DGANSSInfoList_v920ext */
static int hf_rrc_errorReason = -1;               /* UE_Positioning_ErrorCause */
static int hf_rrc_ue_positioning_GPS_additionalAssistanceDataRequest = -1;  /* UE_Positioning_GPS_AdditionalAssistanceDataRequest */
static int hf_rrc_errorReason_01 = -1;            /* UE_Positioning_ErrorCause_r7 */
static int hf_rrc_ue_positioning_GANSS_additionalAssistanceDataRequest = -1;  /* UE_Positioning_GANSS_AdditionalAssistanceDataRequest */
static int hf_rrc_ue_positioning_GANSS_additionalAssistanceDataRequest_01 = -1;  /* UE_Positioning_GANSS_AdditionalAssistanceDataRequest_v860ext */
static int hf_rrc_reportFirstFix = -1;            /* BOOLEAN */
static int hf_rrc_measurementInterval = -1;       /* UE_Positioning_MeasurementInterval */
static int hf_rrc_eventSpecificInfo = -1;         /* UE_Positioning_EventSpecificInfo */
static int hf_rrc_eventSpecificInfo_01 = -1;      /* UE_Positioning_EventSpecificInfo_r7 */
static int hf_rrc_UE_Positioning_EventParamList_item = -1;  /* UE_Positioning_EventParam */
static int hf_rrc_UE_Positioning_EventParamList_r7_item = -1;  /* UE_Positioning_EventParam_r7 */
static int hf_rrc_e7a = -1;                       /* ThresholdPositionChange */
static int hf_rrc_e7b = -1;                       /* ThresholdSFN_SFN_Change */
static int hf_rrc_e7c = -1;                       /* ThresholdSFN_GPS_TOW */
static int hf_rrc_e7c_01 = -1;                    /* ThresholdSFN_GPS_TOW_us */
static int hf_rrc_e7d = -1;                       /* ThresholdSFN_GANSS_TOW */
static int hf_rrc_utcModel1 = -1;                 /* UTCmodelSet1 */
static int hf_rrc_utcModel2 = -1;                 /* UTCmodelSet2 */
static int hf_rrc_utcModel3 = -1;                 /* UTCmodelSet3 */
static int hf_rrc_ganss_wk_number = -1;           /* INTEGER_0_255 */
static int hf_rrc_alm_keplerianParameters = -1;   /* ALM_keplerianParameters */
static int hf_rrc_alm_keplerianNAVAlmanac = -1;   /* ALM_NAVKeplerianSet */
static int hf_rrc_alm_keplerianReducedAlmanac = -1;  /* ALM_ReducedKeplerianSet */
static int hf_rrc_alm_keplerianMidiAlmanac = -1;  /* ALM_MidiAlmanacSet */
static int hf_rrc_alm_keplerianGLONASS = -1;      /* ALM_GlonassAlmanacSet */
static int hf_rrc_alm_ecefSBASAlmanac = -1;       /* ALM_ECEFsbasAlmanacSet */
static int hf_rrc_complete_Almanac_Provided = -1;  /* BOOLEAN */
static int hf_rrc_ganssReferenceTime = -1;        /* BOOLEAN */
static int hf_rrc_ganssreferenceLocation = -1;    /* BOOLEAN */
static int hf_rrc_ganssIonosphericModel = -1;     /* BOOLEAN */
static int hf_rrc_ganssRequestedGenericAssistanceDataList = -1;  /* GanssRequestedGenericAssistanceDataList */
static int hf_rrc_ganssAddIonoModelReq = -1;      /* BIT_STRING_SIZE_2 */
static int hf_rrc_ganssEOPreq = -1;               /* T_ganssEOPreq */
static int hf_rrc_ganssRequestedGenericAssistanceDataList_01 = -1;  /* GanssRequestedGenericAssistanceDataList_v860ext */
static int hf_rrc_dataID_01 = -1;                 /* BIT_STRING_SIZE_2 */
static int hf_rrc_alpha_beta_parameters = -1;     /* UE_Positioning_GPS_IonosphericModel */
static int hf_rrc_non_broadcastIndication = -1;   /* T_non_broadcastIndication */
static int hf_rrc_ganssSatInfoNavList = -1;       /* Ganss_Sat_Info_AddNavList */
static int hf_rrc_ue_positioning_GANSS_ReferenceTime = -1;  /* UE_Positioning_GANSS_ReferenceTime */
static int hf_rrc_uePositioningGanssReferencePosition = -1;  /* ReferenceLocationGANSS */
static int hf_rrc_uePositioningGanssIonosphericModel = -1;  /* UE_Positioning_GANSS_IonosphericModel */
static int hf_rrc_ganssGenericDataList = -1;      /* GANSSGenericDataList */
static int hf_rrc_uePositioningGanssAddIonoModel = -1;  /* UE_Positioning_GANSS_AddIonoModel */
static int hf_rrc_uePositioningGanssEarthOrientationPara = -1;  /* UE_Positioning_GANSS_EarthOrientPara */
static int hf_rrc_ganssGenericDataList_01 = -1;   /* GANSSGenericDataList_v860ext */
static int hf_rrc_ganssGenericDataList_02 = -1;   /* GANSSGenericDataList_r8 */
static int hf_rrc_ganssGenericDataList_03 = -1;   /* GANSSGenericDataList_r9 */
static int hf_rrc_ganssGenericDataList_04 = -1;   /* GANSSGenericDataList_v920ext */
static int hf_rrc_ue_positioning_GANSS_ReferenceTime_01 = -1;  /* UE_Positioning_GANSS_ReferenceTime_va40ext */
static int hf_rrc_ganssGenericDataList_05 = -1;   /* GANSSGenericDataList_va40ext */
static int hf_rrc_ue_positioning_GANSS_ReferenceTime_02 = -1;  /* UE_Positioning_GANSS_ReferenceTime_r10 */
static int hf_rrc_ganssGenericDataList_06 = -1;   /* GANSSGenericDataList_r10 */
static int hf_rrc_ganssID1 = -1;                  /* AuxInfoGANSS_ID1 */
static int hf_rrc_ganssID3 = -1;                  /* AuxInfoGANSS_ID3 */
static int hf_rrc_satellite_clock_modelList = -1;  /* Satellite_clock_modelList */
static int hf_rrc_navClockModel = -1;             /* NAVclockModel */
static int hf_rrc_cnavClockModel = -1;            /* CNAVclockModel */
static int hf_rrc_glonassClockModel = -1;         /* GLONASSclockModel */
static int hf_rrc_sbasClockModel = -1;            /* SBASclockModel */
static int hf_rrc_navKeplerianSet = -1;           /* NavModel_NAVKeplerianSet */
static int hf_rrc_cnavKeplerianSet = -1;          /* NavModel_CNAVKeplerianSet */
static int hf_rrc_glonassECEF = -1;               /* NavModel_GLONASSecef */
static int hf_rrc_sbasECEF = -1;                  /* NavModel_SBASecef */
static int hf_rrc_ganss_tod = -1;                 /* INTEGER_0_59 */
static int hf_rrc_dataBitAssistanceList = -1;     /* DataBitAssistanceList */
static int hf_rrc_teop = -1;                      /* BIT_STRING_SIZE_16 */
static int hf_rrc_pmX = -1;                       /* BIT_STRING_SIZE_21 */
static int hf_rrc_pmXdot = -1;                    /* BIT_STRING_SIZE_15 */
static int hf_rrc_pmY = -1;                       /* BIT_STRING_SIZE_21 */
static int hf_rrc_pmYdot = -1;                    /* BIT_STRING_SIZE_15 */
static int hf_rrc_deltaUT1 = -1;                  /* BIT_STRING_SIZE_31 */
static int hf_rrc_deltaUT1dot = -1;               /* BIT_STRING_SIZE_19 */
static int hf_rrc_alpha_zero_ionos = -1;          /* BIT_STRING_SIZE_12 */
static int hf_rrc_alpha_one_ionos = -1;           /* BIT_STRING_SIZE_12 */
static int hf_rrc_alpha_two_ionos = -1;           /* BIT_STRING_SIZE_12 */
static int hf_rrc_gANSS_storm_flags = -1;         /* GANSS_Storm_Flag */
static int hf_rrc_referenceTime = -1;             /* T_referenceTime */
static int hf_rrc_utran_GANSSReferenceTimeResult = -1;  /* UTRAN_GANSSReferenceTime */
static int hf_rrc_ganssReferenceTimeOnly = -1;    /* GANSSReferenceTimeOnly */
static int hf_rrc_ganssGenericMeasurementInfo = -1;  /* GANSSGenericMeasurementInfo */
static int hf_rrc_ganssGenericMeasurementInfo_01 = -1;  /* GANSSGenericMeasurementInfo_v860ext */
static int hf_rrc_non_broadcastIndication_01 = -1;  /* T_non_broadcastIndication_01 */
static int hf_rrc_ganssSatInfoNavList_01 = -1;    /* Ganss_Sat_Info_NavList */
static int hf_rrc_keplerianParameters = -1;       /* KeplerianParameters */
static int hf_rrc_UE_Positioning_GANSS_RealTimeIntegrity_item = -1;  /* UE_Positioning_GANSS_RealTimeIntegrity_item */
static int hf_rrc_bad_ganss_satId = -1;           /* INTEGER_0_63 */
static int hf_rrc_bad_ganss_signalId = -1;        /* BIT_STRING_SIZE_8 */
static int hf_rrc_satelliteInformationList = -1;  /* GANSSSatelliteInformationList */
static int hf_rrc_satelliteInformationList_01 = -1;  /* GANSSSatelliteInformationList_va40ext */
static int hf_rrc_satelliteInformationList_02 = -1;  /* GANSSSatelliteInformationList_r10 */
static int hf_rrc_ganssDay = -1;                  /* INTEGER_0_8191 */
static int hf_rrc_ganssTodUncertainty = -1;       /* INTEGER_0_127 */
static int hf_rrc_ganssTimeId = -1;               /* INTEGER_0_7 */
static int hf_rrc_utran_ganssreferenceTime = -1;  /* T_utran_ganssreferenceTime */
static int hf_rrc_timingOfCellFrames = -1;        /* INTEGER_0_3999999 */
static int hf_rrc_mode = -1;                      /* T_mode */
static int hf_rrc_fdd_173 = -1;                   /* T_fdd_167 */
static int hf_rrc_primary_CPICH_Info = -1;        /* PrimaryCPICH_Info */
static int hf_rrc_tdd_124 = -1;                   /* T_tdd_116 */
static int hf_rrc_cellParameters = -1;            /* CellParametersID */
static int hf_rrc_referenceSfn = -1;              /* INTEGER_0_4095 */
static int hf_rrc_tutran_ganss_driftRate = -1;    /* Tutran_Ganss_DriftRate */
static int hf_rrc_ganss_Day_Cycle_Number = -1;    /* INTEGER_0_7 */
static int hf_rrc_utran_ganssreferenceTime_01 = -1;  /* T_utran_ganssreferenceTime_01 */
static int hf_rrc_mode_01 = -1;                   /* T_mode_01 */
static int hf_rrc_fdd_174 = -1;                   /* T_fdd_168 */
static int hf_rrc_tdd_125 = -1;                   /* T_tdd_117 */
static int hf_rrc_ganss_timeModelreferenceTime = -1;  /* INTEGER_0_37799 */
static int hf_rrc_ganss_t_a0 = -1;                /* INTEGER_M2147483648_2147483647 */
static int hf_rrc_ganss_t_a1 = -1;                /* INTEGER_M8388608_8388607 */
static int hf_rrc_ganss_t_a2 = -1;                /* INTEGER_M64_63 */
static int hf_rrc_gnss_to_id = -1;                /* T_gnss_to_id */
static int hf_rrc_ganss_wk_number_01 = -1;        /* INTEGER_0_8191 */
static int hf_rrc_deltaT = -1;                    /* INTEGER_M128_127 */
static int hf_rrc_gnss_to_id_01 = -1;             /* T_gnss_to_id_01 */
static int hf_rrc_UE_Positioning_GANSS_TimeModels_item = -1;  /* UE_Positioning_GANSS_TimeModel */
static int hf_rrc_UE_Positioning_GANSS_TimeModels_va40ext_item = -1;  /* UE_Positioning_GANSS_TimeModel_va40ext */
static int hf_rrc_a_one_utc = -1;                 /* BIT_STRING_SIZE_24 */
static int hf_rrc_a_zero_utc = -1;                /* BIT_STRING_SIZE_32 */
static int hf_rrc_t_ot_utc = -1;                  /* BIT_STRING_SIZE_8 */
static int hf_rrc_delta_t_ls_utc = -1;            /* BIT_STRING_SIZE_8 */
static int hf_rrc_w_n_t_utc = -1;                 /* BIT_STRING_SIZE_8 */
static int hf_rrc_w_n_lsf_utc = -1;               /* BIT_STRING_SIZE_8 */
static int hf_rrc_dn_utc = -1;                    /* BIT_STRING_SIZE_8 */
static int hf_rrc_delta_t_lsf_utc = -1;           /* BIT_STRING_SIZE_8 */
static int hf_rrc_gps_ReferenceTime = -1;         /* GPS_TOW_1msec */
static int hf_rrc_utran_GPSReferenceTime = -1;    /* UTRAN_GPSReferenceTime */
static int hf_rrc_satelliteInformationList_03 = -1;  /* AcquisitionSatInfoList */
static int hf_rrc_ue_Positioning_GPS_ReferenceTimeUncertainty = -1;  /* UE_Positioning_GPS_ReferenceTimeUncertainty */
static int hf_rrc_satelliteInformationList_04 = -1;  /* AcquisitionSatInfoList_va40ext */
static int hf_rrc_satelliteInformationList_05 = -1;  /* AcquisitionSatInfoList_r10 */
static int hf_rrc_almanacRequest = -1;            /* BOOLEAN */
static int hf_rrc_utcModelRequest = -1;           /* BOOLEAN */
static int hf_rrc_ionosphericModelRequest = -1;   /* BOOLEAN */
static int hf_rrc_navigationModelRequest = -1;    /* BOOLEAN */
static int hf_rrc_dgpsCorrectionsRequest = -1;    /* BOOLEAN */
static int hf_rrc_referenceLocationRequest = -1;  /* BOOLEAN */
static int hf_rrc_referenceTimeRequest = -1;      /* BOOLEAN */
static int hf_rrc_aquisitionAssistanceRequest = -1;  /* BOOLEAN */
static int hf_rrc_realTimeIntegrityRequest = -1;  /* BOOLEAN */
static int hf_rrc_navModelAddDataRequest = -1;    /* UE_Positioning_GPS_NavModelAddDataReq */
static int hf_rrc_wn_a = -1;                      /* BIT_STRING_SIZE_8 */
static int hf_rrc_almanacSatInfoList = -1;        /* AlmanacSatInfoList */
static int hf_rrc_sv_GlobalHealth = -1;           /* BIT_STRING_SIZE_364 */
static int hf_rrc_ue_positioning_GPS_ReferenceTime = -1;  /* UE_Positioning_GPS_ReferenceTime */
static int hf_rrc_ue_positioning_GPS_ReferenceLocation = -1;  /* ReferenceLocation */
static int hf_rrc_ue_positioning_GPS_DGPS_Corrections = -1;  /* UE_Positioning_GPS_DGPS_Corrections */
static int hf_rrc_ue_positioning_GPS_NavigationModel = -1;  /* UE_Positioning_GPS_NavigationModel */
static int hf_rrc_ue_positioning_GPS_IonosphericModel = -1;  /* UE_Positioning_GPS_IonosphericModel */
static int hf_rrc_ue_positioning_GPS_UTC_Model = -1;  /* UE_Positioning_GPS_UTC_Model */
static int hf_rrc_ue_positioning_GPS_Almanac = -1;  /* UE_Positioning_GPS_Almanac */
static int hf_rrc_ue_positioning_GPS_AcquisitionAssistance = -1;  /* UE_Positioning_GPS_AcquisitionAssistance */
static int hf_rrc_ue_positioning_GPS_Real_timeIntegrity = -1;  /* BadSatList */
static int hf_rrc_dummy_54 = -1;                  /* UE_Positioning_GPS_ReferenceCellInfo */
static int hf_rrc_ue_positioning_GPS_ReferenceTime_01 = -1;  /* UE_Positioning_GPS_ReferenceTime_v770ext */
static int hf_rrc_ue_positioning_GPS_AcquisitionAssistance_01 = -1;  /* UE_Positioning_GPS_AcquisitionAssistance_v770ext */
static int hf_rrc_ue_positioning_GPS_ReferenceTime_02 = -1;  /* UE_Positioning_GPS_ReferenceTime_r7 */
static int hf_rrc_ue_positioning_GPS_AcquisitionAssistance_02 = -1;  /* UE_Positioning_GPS_AcquisitionAssistance_r7 */
static int hf_rrc_ue_positioning_GPS_DGPS_Corrections_01 = -1;  /* UE_Positioning_GPS_DGPS_Corrections_r9 */
static int hf_rrc_ue_positioning_GPS_DGPS_Corrections_02 = -1;  /* UE_Positioning_GPS_DGPS_Corrections_v920ext */
static int hf_rrc_ue_positioning_GPS_ReferenceTime_03 = -1;  /* UE_Positioning_GPS_ReferenceTime_va40ext */
static int hf_rrc_ue_positioning_GPS_Almanac_01 = -1;  /* UE_Positioning_GPS_Almanac_va40ext */
static int hf_rrc_ue_positioning_GPS_AcquisitionAssistance_03 = -1;  /* UE_Positioning_GPS_AcquisitionAssistance_va40ext */
static int hf_rrc_ue_positioning_GPS_ReferenceTime_04 = -1;  /* UE_Positioning_GPS_ReferenceTime_r10 */
static int hf_rrc_ue_positioning_GPS_Almanac_02 = -1;  /* UE_Positioning_GPS_Almanac_r10 */
static int hf_rrc_ue_positioning_GPS_AcquisitionAssistance_04 = -1;  /* UE_Positioning_GPS_AcquisitionAssistance_r10 */
static int hf_rrc_gps_TOW = -1;                   /* GPS_TOW_1sec */
static int hf_rrc_statusHealth = -1;              /* DiffCorrectionStatus */
static int hf_rrc_dgps_CorrectionSatInfoList = -1;  /* DGPS_CorrectionSatInfoList */
static int hf_rrc_dgps_CorrectionSatInfoList_01 = -1;  /* DGPS_CorrectionSatInfoList_r9 */
static int hf_rrc_dgps_CorrectionSatInfoList_02 = -1;  /* DGPS_CorrectionSatInfoList_v920ext */
static int hf_rrc_alpha0 = -1;                    /* BIT_STRING_SIZE_8 */
static int hf_rrc_alpha1 = -1;                    /* BIT_STRING_SIZE_8 */
static int hf_rrc_alpha2 = -1;                    /* BIT_STRING_SIZE_8 */
static int hf_rrc_alpha3 = -1;                    /* BIT_STRING_SIZE_8 */
static int hf_rrc_beta0 = -1;                     /* BIT_STRING_SIZE_8 */
static int hf_rrc_beta1 = -1;                     /* BIT_STRING_SIZE_8 */
static int hf_rrc_beta2 = -1;                     /* BIT_STRING_SIZE_8 */
static int hf_rrc_beta3 = -1;                     /* BIT_STRING_SIZE_8 */
static int hf_rrc_referenceTime_01 = -1;          /* T_referenceTime_01 */
static int hf_rrc_utran_GPSReferenceTimeResult = -1;  /* UTRAN_GPSReferenceTimeResult */
static int hf_rrc_gps_ReferenceTimeOnly = -1;     /* GPS_TOW_1msec */
static int hf_rrc_gps_MeasurementParamList = -1;  /* GPS_MeasurementParamList */
static int hf_rrc_navigationModelSatInfoList = -1;  /* NavigationModelSatInfoList */
static int hf_rrc_gps_Week = -1;                  /* INTEGER_0_1023 */
static int hf_rrc_gps_Toe = -1;                   /* INTEGER_0_255 */
static int hf_rrc_tToeLimit = -1;                 /* INTEGER_0_15 */
static int hf_rrc_satDataList = -1;               /* SatDataList */
static int hf_rrc_modeSpecificInfo_135 = -1;      /* T_modeSpecificInfo_135 */
static int hf_rrc_fdd_175 = -1;                   /* T_fdd_169 */
static int hf_rrc_referenceIdentity = -1;         /* PrimaryCPICH_Info */
static int hf_rrc_tdd_126 = -1;                   /* T_tdd_118 */
static int hf_rrc_referenceIdentity_01 = -1;      /* CellParametersID */
static int hf_rrc_gps_tow_1msec = -1;             /* GPS_TOW_1msec */
static int hf_rrc_sfn_tow_Uncertainty = -1;       /* SFN_TOW_Uncertainty */
static int hf_rrc_utran_GPS_DriftRate = -1;       /* UTRAN_GPS_DriftRate */
static int hf_rrc_gps_TOW_AssistList = -1;        /* GPS_TOW_AssistList */
static int hf_rrc_gps_Week_Cycle_Number = -1;     /* INTEGER_0_7 */
static int hf_rrc_a1 = -1;                        /* BIT_STRING_SIZE_24 */
static int hf_rrc_a0 = -1;                        /* BIT_STRING_SIZE_32 */
static int hf_rrc_t_ot = -1;                      /* BIT_STRING_SIZE_8 */
static int hf_rrc_wn_t = -1;                      /* BIT_STRING_SIZE_8 */
static int hf_rrc_delta_t_LS = -1;                /* BIT_STRING_SIZE_8 */
static int hf_rrc_wn_lsf = -1;                    /* BIT_STRING_SIZE_8 */
static int hf_rrc_dn = -1;                        /* BIT_STRING_SIZE_8 */
static int hf_rrc_delta_t_LSF = -1;               /* BIT_STRING_SIZE_8 */
static int hf_rrc_ip_Spacing = -1;                /* IP_Spacing */
static int hf_rrc_ip_Length = -1;                 /* IP_Length */
static int hf_rrc_ip_Offset = -1;                 /* INTEGER_0_9 */
static int hf_rrc_seed = -1;                      /* INTEGER_0_63 */
static int hf_rrc_burstModeParameters = -1;       /* BurstModeParameters */
static int hf_rrc_modeSpecificInfo_136 = -1;      /* T_modeSpecificInfo_136 */
static int hf_rrc_fdd_176 = -1;                   /* T_fdd_170 */
static int hf_rrc_tdd_127 = -1;                   /* T_tdd_119 */
static int hf_rrc_ip_Spacing_TDD = -1;            /* IP_Spacing_TDD */
static int hf_rrc_ip_slot = -1;                   /* INTEGER_0_14 */
static int hf_rrc_ip_Start = -1;                  /* INTEGER_0_4095 */
static int hf_rrc_ip_PCCPCG = -1;                 /* IP_PCCPCH_r4 */
static int hf_rrc_ip_Spacing_01 = -1;             /* IP_Spacing_TDD */
static int hf_rrc_ue_positioning_OTDOA_Measurement = -1;  /* UE_Positioning_OTDOA_Measurement */
static int hf_rrc_ue_positioning_PositionEstimateInfo = -1;  /* UE_Positioning_PositionEstimateInfo */
static int hf_rrc_ue_positioning_GPS_Measurement = -1;  /* UE_Positioning_GPS_MeasurementResults */
static int hf_rrc_ue_positioning_Error = -1;      /* UE_Positioning_Error */
static int hf_rrc_ue_positioning_PositionEstimateInfo_01 = -1;  /* UE_Positioning_PositionEstimateInfo_v770ext */
static int hf_rrc_ue_positioning_GPS_Measurement_01 = -1;  /* UE_Positioning_GPS_MeasurementResults_v770ext */
static int hf_rrc_ue_positioning_Error_01 = -1;   /* UE_Positioning_Error_r7 */
static int hf_rrc_ue_positioning_Ganss_MeasuredResults = -1;  /* UE_Positioning_GANSS_MeasuredResults */
static int hf_rrc_ue_Positioning_OTDOA_Measurement_v390ext = -1;  /* UE_Positioning_OTDOA_Measurement_v390ext */
static int hf_rrc_ue_positioning_Error_02 = -1;   /* UE_Positioning_Error_v860ext */
static int hf_rrc_ue_positioning_Ganss_MeasurementResults = -1;  /* UE_Positioning_GANSS_MeasuredResults_v860ext */
static int hf_rrc_ue_positioning_ReportingQuantity = -1;  /* UE_Positioning_ReportingQuantity */
static int hf_rrc_reportCriteria_20 = -1;         /* UE_Positioning_ReportCriteria */
static int hf_rrc_ue_positioning_OTDOA_AssistanceData = -1;  /* UE_Positioning_OTDOA_AssistanceData */
static int hf_rrc_ue_positioning_ReportingQuantity_v390ext = -1;  /* UE_Positioning_ReportingQuantity_v390ext */
static int hf_rrc_ue_positioning_ReportingQuantity_01 = -1;  /* UE_Positioning_ReportingQuantity_r4 */
static int hf_rrc_ue_positioning_OTDOA_AssistanceData_01 = -1;  /* UE_Positioning_OTDOA_AssistanceData_r4 */
static int hf_rrc_ue_positioning_ReportingQuantity_02 = -1;  /* UE_Positioning_ReportingQuantity_r7 */
static int hf_rrc_reportCriteria_21 = -1;         /* UE_Positioning_ReportCriteria_r7 */
static int hf_rrc_ue_positioning_OTDOA_AssistanceData_02 = -1;  /* UE_Positioning_OTDOA_AssistanceData_r7 */
static int hf_rrc_ue_positioning_GPS_AssistanceData_01 = -1;  /* UE_Positioning_GPS_AssistanceData_r7 */
static int hf_rrc_ue_positioning_ReportingQuantity_03 = -1;  /* UE_Positioning_ReportingQuantity_r8 */
static int hf_rrc_ue_positioning_GANSS_AssistanceData_01 = -1;  /* UE_Positioning_GANSS_AssistanceData_r8 */
static int hf_rrc_ue_positioning_GPS_AssistanceData_02 = -1;  /* UE_Positioning_GPS_AssistanceData_r9 */
static int hf_rrc_ue_positioning_GANSS_AssistanceData_02 = -1;  /* UE_Positioning_GANSS_AssistanceData_r9 */
static int hf_rrc_ue_positioning_GPS_AssistanceData_03 = -1;  /* UE_Positioning_GPS_AssistanceData_r10 */
static int hf_rrc_ue_positioning_GANSS_AssistanceData_03 = -1;  /* UE_Positioning_GANSS_AssistanceData_r10 */
static int hf_rrc_event7a = -1;                   /* UE_Positioning_PositionEstimateInfo */
static int hf_rrc_event7b = -1;                   /* UE_Positioning_OTDOA_Measurement */
static int hf_rrc_event7c = -1;                   /* UE_Positioning_GPS_MeasurementResults */
static int hf_rrc_additionalOrReplacedPosMeasEvent = -1;  /* NULL */
static int hf_rrc_event7a_01 = -1;                /* UE_Positioning_PositionEstimateInfo_v770ext */
static int hf_rrc_event7c_01 = -1;                /* UE_Positioning_GPS_MeasurementResults_v770ext */
static int hf_rrc_event7d = -1;                   /* UE_Positioning_GANSS_MeasuredResults */
static int hf_rrc_event7d_01 = -1;                /* UE_Positioning_GANSS_MeasuredResults_v860ext */
static int hf_rrc_ue_positioning_OTDOA_ReferenceCellInfo = -1;  /* UE_Positioning_OTDOA_ReferenceCellInfo */
static int hf_rrc_ue_positioning_OTDOA_NeighbourCellList = -1;  /* UE_Positioning_OTDOA_NeighbourCellList */
static int hf_rrc_ue_positioning_OTDOA_ReferenceCellInfo_01 = -1;  /* UE_Positioning_OTDOA_ReferenceCellInfo_r4 */
static int hf_rrc_ue_positioning_OTDOA_NeighbourCellList_01 = -1;  /* UE_Positioning_OTDOA_NeighbourCellList_r4 */
static int hf_rrc_ue_Positioning_IPDL_Parameters_TDD_r4_ext = -1;  /* UE_Positioning_IPDL_Parameters_TDD_r4_ext */
static int hf_rrc_ue_Positioning_IPDL_Parameters_TDDList_r4_ext = -1;  /* UE_Positioning_IPDL_Parameters_TDDList_r4_ext */
static int hf_rrc_ue_positioning_OTDOA_ReferenceCellInfo_UEB = -1;  /* UE_Positioning_OTDOA_ReferenceCellInfo_UEB */
static int hf_rrc_ue_positioning_OTDOA_NeighbourCellList_UEB = -1;  /* UE_Positioning_OTDOA_NeighbourCellList_UEB */
static int hf_rrc_ue_positioning_OTDOA_ReferenceCellInfo_UEB_ext = -1;  /* UE_Positioning_OTDOA_ReferenceCellInfo_UEB_ext */
static int hf_rrc_ue_positioning_OTDOA_NeighbourCellList_UEB_ext = -1;  /* UE_Positioning_OTDOA_NeighbourCellList_UEB_ext */
static int hf_rrc_ue_positioning_OTDOA_ReferenceCellInfo_02 = -1;  /* UE_Positioning_OTDOA_ReferenceCellInfo_r7 */
static int hf_rrc_ue_positioning_OTDOA_NeighbourCellList_02 = -1;  /* UE_Positioning_OTDOA_NeighbourCellList_r7 */
static int hf_rrc_UE_Positioning_IPDL_Parameters_TDDList_r4_ext_item = -1;  /* UE_Positioning_IPDL_Parameters_TDD_r4_ext */
static int hf_rrc_neighbourList = -1;             /* NeighbourList_TDD_r7 */
static int hf_rrc_modeSpecificInfo_137 = -1;      /* T_modeSpecificInfo_137 */
static int hf_rrc_fdd_177 = -1;                   /* T_fdd_171 */
static int hf_rrc_referenceCellIDentity = -1;     /* PrimaryCPICH_Info */
static int hf_rrc_ue_RX_TX_TimeDifferenceType2Info = -1;  /* UE_RX_TX_TimeDifferenceType2Info */
static int hf_rrc_tdd_128 = -1;                   /* T_tdd_120 */
static int hf_rrc_referenceCellIdentity = -1;     /* CellParametersID */
static int hf_rrc_neighbourList_01 = -1;          /* NeighbourList */
static int hf_rrc_neighbourList_v390ext = -1;     /* NeighbourList_v390ext */
static int hf_rrc_modeSpecificInfo_138 = -1;      /* T_modeSpecificInfo_138 */
static int hf_rrc_fdd_178 = -1;                   /* T_fdd_172 */
static int hf_rrc_tdd_129 = -1;                   /* T_tdd_121 */
static int hf_rrc_cellAndChannelIdentity = -1;    /* CellAndChannelIdentity */
static int hf_rrc_ue_positioning_IPDL_Paremeters = -1;  /* UE_Positioning_IPDL_Parameters */
static int hf_rrc_sfn_SFN_RelTimeDifference = -1;  /* SFN_SFN_RelTimeDifference1 */
static int hf_rrc_sfn_SFN_Drift = -1;             /* SFN_SFN_Drift */
static int hf_rrc_searchWindowSize = -1;          /* OTDOA_SearchWindowSize */
static int hf_rrc_positioningMode = -1;           /* T_positioningMode */
static int hf_rrc_ueBased = -1;                   /* T_ueBased */
static int hf_rrc_ueAssisted = -1;                /* T_ueAssisted */
static int hf_rrc_modeSpecificInfo_139 = -1;      /* T_modeSpecificInfo_139 */
static int hf_rrc_fdd_179 = -1;                   /* T_fdd_173 */
static int hf_rrc_tdd_130 = -1;                   /* T_tdd_122 */
static int hf_rrc_ue_positioning_IPDL_Paremeters_01 = -1;  /* UE_Positioning_IPDL_Parameters_r4 */
static int hf_rrc_positioningMode_01 = -1;        /* T_positioningMode_01 */
static int hf_rrc_ueBased_01 = -1;                /* T_ueBased_01 */
static int hf_rrc_relativeNorth = -1;             /* INTEGER_M20000_20000 */
static int hf_rrc_relativeEast = -1;              /* INTEGER_M20000_20000 */
static int hf_rrc_relativeAltitude = -1;          /* INTEGER_M4000_4000 */
static int hf_rrc_fineSFN_SFN = -1;               /* FineSFN_SFN */
static int hf_rrc_roundTripTime = -1;             /* INTEGER_0_32766 */
static int hf_rrc_ueAssisted_01 = -1;             /* T_ueAssisted_01 */
static int hf_rrc_modeSpecificInfo_140 = -1;      /* T_modeSpecificInfo_140 */
static int hf_rrc_fdd_180 = -1;                   /* T_fdd_174 */
static int hf_rrc_tdd_131 = -1;                   /* T_tdd_123 */
static int hf_rrc_roundTripTimeExtension = -1;    /* INTEGER_0_70274 */
static int hf_rrc_modeSpecificInfo_141 = -1;      /* T_modeSpecificInfo_141 */
static int hf_rrc_fdd_181 = -1;                   /* T_fdd_175 */
static int hf_rrc_tdd_132 = -1;                   /* T_tdd_124 */
static int hf_rrc_positioningMode_02 = -1;        /* T_positioningMode_02 */
static int hf_rrc_ueBased_02 = -1;                /* T_ueBased_02 */
static int hf_rrc_ueAssisted_02 = -1;             /* T_ueAssisted_02 */
static int hf_rrc_UE_Positioning_OTDOA_NeighbourCellList_item = -1;  /* UE_Positioning_OTDOA_NeighbourCellInfo */
static int hf_rrc_UE_Positioning_OTDOA_NeighbourCellList_r4_item = -1;  /* UE_Positioning_OTDOA_NeighbourCellInfo_r4 */
static int hf_rrc_UE_Positioning_OTDOA_NeighbourCellList_UEB_item = -1;  /* UE_Positioning_OTDOA_NeighbourCellInfo_UEB */
static int hf_rrc_UE_Positioning_OTDOA_NeighbourCellList_UEB_ext_item = -1;  /* UE_Positioning_OTDOA_NeighbourCellInfo_UEB_ext */
static int hf_rrc_UE_Positioning_OTDOA_NeighbourCellList_r7_item = -1;  /* UE_Positioning_OTDOA_NeighbourCellInfo_r7 */
static int hf_rrc_stdResolution = -1;             /* BIT_STRING_SIZE_2 */
static int hf_rrc_numberOfOTDOA_Measurements = -1;  /* BIT_STRING_SIZE_3 */
static int hf_rrc_stdOfOTDOA_Measurements = -1;   /* BIT_STRING_SIZE_5 */
static int hf_rrc_modeSpecificInfo_142 = -1;      /* T_modeSpecificInfo_142 */
static int hf_rrc_fdd_182 = -1;                   /* T_fdd_176 */
static int hf_rrc_tdd_133 = -1;                   /* T_tdd_125 */
static int hf_rrc_positioningMode_03 = -1;        /* T_positioningMode_03 */
static int hf_rrc_ueBased_03 = -1;                /* T_ueBased_03 */
static int hf_rrc_ueAssisted_03 = -1;             /* T_ueAssisted_03 */
static int hf_rrc_modeSpecificInfo_143 = -1;      /* T_modeSpecificInfo_143 */
static int hf_rrc_fdd_183 = -1;                   /* T_fdd_177 */
static int hf_rrc_tdd_134 = -1;                   /* T_tdd_126 */
static int hf_rrc_positioningMode_04 = -1;        /* T_positioningMode_04 */
static int hf_rrc_ueBased_04 = -1;                /* T_ueBased_04 */
static int hf_rrc_cellPosition = -1;              /* ReferenceCellPosition */
static int hf_rrc_ueAssisted_04 = -1;             /* T_ueAssisted_04 */
static int hf_rrc_modeSpecificInfo_144 = -1;      /* T_modeSpecificInfo_144 */
static int hf_rrc_fdd_184 = -1;                   /* T_fdd_178 */
static int hf_rrc_tdd_135 = -1;                   /* T_tdd_127 */
static int hf_rrc_modeSpecificInfo_145 = -1;      /* T_modeSpecificInfo_145 */
static int hf_rrc_fdd_185 = -1;                   /* T_fdd_179 */
static int hf_rrc_tdd_136 = -1;                   /* T_tdd_128 */
static int hf_rrc_positioningMode_05 = -1;        /* T_positioningMode_05 */
static int hf_rrc_ueBased_05 = -1;                /* T_ueBased_05 */
static int hf_rrc_ueAssisted_05 = -1;             /* T_ueAssisted_05 */
static int hf_rrc_referenceTime_02 = -1;          /* T_referenceTime_02 */
static int hf_rrc_cell_Timing = -1;               /* T_cell_Timing */
static int hf_rrc_modeSpecificInfo_146 = -1;      /* T_modeSpecificInfo_146 */
static int hf_rrc_fdd_186 = -1;                   /* T_fdd_180 */
static int hf_rrc_tdd_137 = -1;                   /* T_tdd_129 */
static int hf_rrc_positionEstimate = -1;          /* PositionEstimate */
static int hf_rrc_referenceTimeOptions = -1;      /* T_referenceTimeOptions */
static int hf_rrc_earlier_than_r7 = -1;           /* NULL */
static int hf_rrc_r7_15 = -1;                     /* T_r7_15 */
static int hf_rrc_referenceTime_03 = -1;          /* T_referenceTime_03 */
static int hf_rrc_positionData = -1;              /* BIT_STRING_SIZE_16 */
static int hf_rrc_velocityEstimate = -1;          /* VelocityEstimate */
static int hf_rrc_ue_positioning_ReportingCriteria = -1;  /* UE_Positioning_EventParamList */
static int hf_rrc_ue_positioning_ReportingCriteria_01 = -1;  /* UE_Positioning_EventParamList_r7 */
static int hf_rrc_methodType = -1;                /* UE_Positioning_MethodType */
static int hf_rrc_positioningMethod = -1;         /* PositioningMethod */
static int hf_rrc_dummy1_06 = -1;                 /* UE_Positioning_ResponseTime */
static int hf_rrc_horizontal_Accuracy = -1;       /* UE_Positioning_Accuracy */
static int hf_rrc_gps_TimingOfCellWanted = -1;    /* BOOLEAN */
static int hf_rrc_additionalAssistanceDataRequest = -1;  /* BOOLEAN */
static int hf_rrc_environmentCharacterisation = -1;  /* EnvironmentCharacterisation */
static int hf_rrc_vertical_Accuracy = -1;         /* UE_Positioning_Accuracy */
static int hf_rrc_horizontalAccuracy = -1;        /* UE_Positioning_Accuracy */
static int hf_rrc_verticalAccuracy = -1;          /* UE_Positioning_Accuracy */
static int hf_rrc_additionalAssistanceDataReq = -1;  /* BOOLEAN */
static int hf_rrc_velocityRequested = -1;         /* T_velocityRequested */
static int hf_rrc_gANSSPositioningMethods = -1;   /* BIT_STRING_SIZE_16 */
static int hf_rrc_gANSSTimingOfCellWanted = -1;   /* BIT_STRING_SIZE_8 */
static int hf_rrc_gANSSCarrierPhaseMeasurementRequested = -1;  /* BIT_STRING_SIZE_8 */
static int hf_rrc_velocityRequested_01 = -1;      /* T_velocityRequested_01 */
static int hf_rrc_gANSSMultiFreqMeasurementRequested = -1;  /* BIT_STRING_SIZE_8 */
static int hf_rrc_utcA0 = -1;                     /* BIT_STRING_SIZE_16 */
static int hf_rrc_utcA1 = -1;                     /* BIT_STRING_SIZE_13 */
static int hf_rrc_utcA2 = -1;                     /* BIT_STRING_SIZE_7 */
static int hf_rrc_utcDeltaTls = -1;               /* BIT_STRING_SIZE_8 */
static int hf_rrc_utcTot = -1;                    /* BIT_STRING_SIZE_16 */
static int hf_rrc_utcWNot = -1;                   /* BIT_STRING_SIZE_13 */
static int hf_rrc_utcWNlsf = -1;                  /* BIT_STRING_SIZE_8 */
static int hf_rrc_utcDN = -1;                     /* BIT_STRING_SIZE_4 */
static int hf_rrc_utcDeltaTlsf = -1;              /* BIT_STRING_SIZE_8 */
static int hf_rrc_nA = -1;                        /* BIT_STRING_SIZE_11 */
static int hf_rrc_tauC = -1;                      /* BIT_STRING_SIZE_32 */
static int hf_rrc_deltaUT1_01 = -1;               /* DeltaUT1 */
static int hf_rrc_kp = -1;                        /* BIT_STRING_SIZE_2 */
static int hf_rrc_utcA1wnt = -1;                  /* BIT_STRING_SIZE_24 */
static int hf_rrc_utcA0wnt = -1;                  /* BIT_STRING_SIZE_32 */
static int hf_rrc_utcTot_01 = -1;                 /* BIT_STRING_SIZE_8 */
static int hf_rrc_utcWNt = -1;                    /* BIT_STRING_SIZE_8 */
static int hf_rrc_utcDN_01 = -1;                  /* BIT_STRING_SIZE_8 */
static int hf_rrc_utcStandardID = -1;             /* BIT_STRING_SIZE_3 */
static int hf_rrc_ue_GANSSTimingOfCellFrames = -1;  /* T_ue_GANSSTimingOfCellFrames */
static int hf_rrc_ms_Part = -1;                   /* INTEGER_0_80 */
static int hf_rrc_ls_Part = -1;                   /* INTEGER_0_4294967295 */
static int hf_rrc_gANSS_TimeId = -1;              /* INTEGER_0_7 */
static int hf_rrc_gANSS_TimeUncertainty = -1;     /* INTEGER_0_127 */
static int hf_rrc_mode_02 = -1;                   /* T_mode_02 */
static int hf_rrc_fdd_187 = -1;                   /* T_fdd_181 */
static int hf_rrc_tdd_138 = -1;                   /* T_tdd_130 */
static int hf_rrc_UTRAN_FDD_FrequencyList_item = -1;  /* UTRAN_FDD_Frequency */
static int hf_rrc_qQualMinFDD = -1;               /* INTEGER_M24_0 */
static int hf_rrc_qRxLevMinFDD = -1;              /* INTEGER_M60_M13 */
static int hf_rrc_UTRAN_TDD_FrequencyList_item = -1;  /* UTRAN_TDD_Frequency */
static int hf_rrc_qRxLevMinTDD = -1;              /* INTEGER_M60_M13 */
static int hf_rrc_utran_GPSTimingOfCell = -1;     /* T_utran_GPSTimingOfCell */
static int hf_rrc_ms_part = -1;                   /* INTEGER_0_1023 */
static int hf_rrc_ls_part = -1;                   /* INTEGER_0_4294967295 */
static int hf_rrc_modeSpecificInfo_147 = -1;      /* T_modeSpecificInfo_147 */
static int hf_rrc_fdd_188 = -1;                   /* T_fdd_182 */
static int hf_rrc_tdd_139 = -1;                   /* T_tdd_131 */
static int hf_rrc_ue_GPSTimingOfCell = -1;        /* T_ue_GPSTimingOfCell */
static int hf_rrc_ms_part_01 = -1;                /* INTEGER_0_16383 */
static int hf_rrc_modeSpecificInfo_148 = -1;      /* T_modeSpecificInfo_148 */
static int hf_rrc_fdd_189 = -1;                   /* T_fdd_183 */
static int hf_rrc_tdd_140 = -1;                   /* T_tdd_132 */
static int hf_rrc_utra_ServingCell = -1;          /* T_utra_ServingCell */
static int hf_rrc_s_PrioritySearch1 = -1;         /* INTEGER_0_31 */
static int hf_rrc_s_PrioritySearch2 = -1;         /* INTEGER_0_7 */
static int hf_rrc_threshServingLow = -1;          /* INTEGER_0_31 */
static int hf_rrc_utran_FDD_FrequencyList = -1;   /* UTRAN_FDD_FrequencyList */
static int hf_rrc_utran_TDD_FrequencyList = -1;   /* UTRAN_TDD_FrequencyList */
static int hf_rrc_threshServingLow2 = -1;         /* INTEGER_0_31 */
static int hf_rrc_horizontalWithVerticalVelocity = -1;  /* HorizontalWithVerticalVelocity */
static int hf_rrc_horizontalVelocityWithUncertainty = -1;  /* HorizontalVelocityWithUncertainty */
static int hf_rrc_horizontalWithVerticalVelocityAndUncertainty = -1;  /* HorizontalWithVerticalVelocityAndUncertainty */
static int hf_rrc_bcch_ModificationTime = -1;     /* BCCH_ModificationTime */
static int hf_rrc_ncc = -1;                       /* NCC */
static int hf_rrc_bcc = -1;                       /* BCC */
static int hf_rrc_ctch_AllocationPeriod = -1;     /* INTEGER_1_256 */
static int hf_rrc_cbs_FrameOffset = -1;           /* INTEGER_0_255 */
static int hf_rrc_msg_Type = -1;                  /* BIT_STRING_SIZE_8 */
static int hf_rrc_payload_02 = -1;                /* BIT_STRING_SIZE_1_512 */
static int hf_rrc_CDMA2000_MessageList_item = -1;  /* CDMA2000_Message */
static int hf_rrc_CDMA2000_UMTS_Frequency_List_item = -1;  /* FrequencyInfoCDMA2000 */
static int hf_rrc_dataTransmFreqGranularity = -1;  /* T_dataTransmFreqGranularity */
static int hf_rrc_numberOfDataTransmOcc = -1;     /* INTEGER_1_610 */
static int hf_rrc_dataVolumeMontoringWindow = -1;  /* INTEGER_1_120 */
static int hf_rrc_dataTransmFrequency = -1;       /* DataTransmFrequency */
static int hf_rrc_dataVolumePerRB = -1;           /* DataVolumePerRB_List */
static int hf_rrc_dataVolume = -1;                /* INTEGER_0_4294967295 */
static int hf_rrc_DataVolumePerRB_List_item = -1;  /* DataVolumePerRB */
static int hf_rrc_modeSpecificInfo_149 = -1;      /* T_modeSpecificInfo_149 */
static int hf_rrc_fdd_190 = -1;                   /* T_fdd_184 */
static int hf_rrc_intraFreqMeasQuantity_FDD_01 = -1;  /* IntraFreqMeasQuantity_FDD_sib3 */
static int hf_rrc_tdd_141 = -1;                   /* T_tdd_133 */
static int hf_rrc_intraFreqMeasQuantity_TDDList_01 = -1;  /* IntraFreqMeasQuantity_TDD_sib3List */
static int hf_rrc_warningType = -1;               /* OCTET_STRING_SIZE_1_2 */
static int hf_rrc_messageIdentifier = -1;         /* OCTET_STRING_SIZE_2 */
static int hf_rrc_serialNumber = -1;              /* OCTET_STRING_SIZE_2 */
static int hf_rrc_EUTRA_BlacklistedCellPerFreqList_item = -1;  /* EUTRA_BlacklistedCell */
static int hf_rrc_ue_EUTRA_Capability = -1;       /* T_ue_EUTRA_Capability */
static int hf_rrc_EUTRA_TargetFreqInfoList_item = -1;  /* EUTRA_TargetFreqInfo */
static int hf_rrc_dlEUTRACarrierFreq = -1;        /* EARFCN */
static int hf_rrc_eutraBlacklistedCellPerFreqList = -1;  /* EUTRA_BlacklistedCellPerFreqList */
static int hf_rrc_extensionGANSS_SIBType = -1;    /* SIB_TypeExtGANSS */
static int hf_rrc_schedulingInfo = -1;            /* SchedulingInformation */
static int hf_rrc_valueTagInfo = -1;              /* ValueTagInfo */
static int hf_rrc_ExtGANSS_SIBTypeInfoSchedulingInfoList_item = -1;  /* ExtGANSS_SIBTypeInfoSchedulingInfo */
static int hf_rrc_ganssID = -1;                   /* INTEGER_0_7 */
static int hf_rrc_sbasID = -1;                    /* UE_Positioning_GANSS_SBAS_ID */
static int hf_rrc_ganssScheduling = -1;           /* SEQUENCE_SIZE_1_maxSIB_OF_ExtGANSS_SchedulingInfo */
static int hf_rrc_ganssScheduling_item = -1;      /* ExtGANSS_SchedulingInfo */
static int hf_rrc_ExtSIBTypeInfoSchedulingInfo_List_item = -1;  /* ExtSIBTypeInfoSchedulingInfo */
static int hf_rrc_extensionSIB_Type = -1;         /* SIB_TypeExt */
static int hf_rrc_ExtSIBTypeInfoSchedulingInfo_List2_item = -1;  /* ExtSIBTypeInfoSchedulingInfo2 */
static int hf_rrc_extensionSIB_Type2 = -1;        /* SIB_TypeExt2 */
static int hf_rrc_FDD_UMTS_Frequency_List_item = -1;  /* FrequencyInfoFDD */
static int hf_rrc_band_Class = -1;                /* BIT_STRING_SIZE_5 */
static int hf_rrc_cdma_Freq = -1;                 /* BIT_STRING_SIZE_11 */
static int hf_rrc_GERAN_SystemInformation_item = -1;  /* GERAN_SystemInfoBlock */
static int hf_rrc_gsmLowRangeUARFCN = -1;         /* UARFCN */
static int hf_rrc_gsmUpRangeUARFCN = -1;          /* UARFCN */
static int hf_rrc_GSM_BA_Range_List_item = -1;    /* GSM_BA_Range */
static int hf_rrc_GSM_MessageList_item = -1;      /* GSM_MessageList_item */
static int hf_rrc_GSM_TargetCellInfoList_item = -1;  /* GSM_TargetCellInfo */
static int hf_rrc_receivedMessageType = -1;       /* ReceivedMessageType */
static int hf_rrc_configurationUnacceptable = -1;  /* NULL */
static int hf_rrc_unspecified = -1;               /* NULL */
static int hf_rrc_GERANIu_MessageList_item = -1;  /* BIT_STRING_SIZE_1_32768 */
static int hf_rrc_gsm_17 = -1;                    /* T_gsm_15 */
static int hf_rrc_gsm_Classmark2 = -1;            /* GSM_Classmark2 */
static int hf_rrc_gsm_Classmark3 = -1;            /* GSM_Classmark3 */
static int hf_rrc_cdma2000_01 = -1;               /* T_cdma2000_01 */
static int hf_rrc_InterRAT_UE_RadioAccessCapabilityList_item = -1;  /* InterRAT_UE_RadioAccessCapability */
static int hf_rrc_geranIu_RadioAccessCapability = -1;  /* GERANIu_RadioAccessCapability */
static int hf_rrc_supportOfInter_RAT_PS_Handover = -1;  /* T_supportOfInter_RAT_PS_Handover */
static int hf_rrc_gsm_MS_RadioAccessCapability = -1;  /* GSM_MS_RadioAccessCapability */
static int hf_rrc_eutra_RadioAccessCapability = -1;  /* EUTRA_RadioAccessCapability */
static int hf_rrc_gsm_18 = -1;                    /* T_gsm_16 */
static int hf_rrc_gsmSecurityCapability = -1;     /* GsmSecurityCapability */
static int hf_rrc_InterRAT_UE_SecurityCapList_item = -1;  /* InterRAT_UE_SecurityCapability */
static int hf_rrc_interRAT_ProtocolError = -1;    /* NULL */
static int hf_rrc_plmn_Type = -1;                 /* PLMN_Type */
static int hf_rrc_sibSb_ReferenceList = -1;       /* SIBSb_ReferenceList */
static int hf_rrc_v690NonCriticalExtensions_48 = -1;  /* T_v690NonCriticalExtensions_48 */
static int hf_rrc_masterInformationBlock_v690ext = -1;  /* MasterInformationBlock_v690ext */
static int hf_rrc_v6b0NonCriticalExtensions_16 = -1;  /* T_v6b0NonCriticalExtensions_15 */
static int hf_rrc_masterInformationBlock_v6b0ext = -1;  /* MasterInformationBlock_v6b0ext_IEs */
static int hf_rrc_v860NonCriticalExtensions_26 = -1;  /* T_v860NonCriticalExtensions_26 */
static int hf_rrc_masterInformationBlock_v860ext = -1;  /* MasterInformationBlock_v860ext_IEs */
static int hf_rrc_nonCriticalExtensions_195 = -1;  /* T_nonCriticalExtensions_195 */
static int hf_rrc_multiplePLMN_List = -1;         /* MultiplePLMN_List_r6 */
static int hf_rrc_extSIBTypeInfoSchedulingInfo_List = -1;  /* ExtSIBTypeInfoSchedulingInfo_List */
static int hf_rrc_extSIBTypeInfoSchedulingInfo_List_01 = -1;  /* ExtSIBTypeInfoSchedulingInfo_List2 */
static int hf_rrc_extGANSS_SIBTypeInfoSchedulingInfoList = -1;  /* ExtGANSS_SIBTypeInfoSchedulingInfoList */
static int hf_rrc_csg_Indicator = -1;             /* T_csg_Indicator */
static int hf_rrc_predefinedConfigValueTag = -1;  /* PredefinedConfigValueTag */
static int hf_rrc_diagnosticsType_01 = -1;        /* T_diagnosticsType_01 */
static int hf_rrc_type1_08 = -1;                  /* T_type1_07 */
static int hf_rrc_protocolErrorCause = -1;        /* ProtocolErrorCause */
static int hf_rrc_gsm_BA_Range_List = -1;         /* GSM_BA_Range_List */
static int hf_rrc_fdd_UMTS_Frequency_List = -1;   /* FDD_UMTS_Frequency_List */
static int hf_rrc_tdd_UMTS_Frequency_List = -1;   /* TDD_UMTS_Frequency_List */
static int hf_rrc_cdma2000_UMTS_Frequency_List = -1;  /* CDMA2000_UMTS_Frequency_List */
static int hf_rrc_tdd384_UMTS_Frequency_List = -1;  /* TDD_UMTS_Frequency_List */
static int hf_rrc_tdd128_UMTS_Frequency_List = -1;  /* TDD_UMTS_Frequency_List */
static int hf_rrc_scheduling = -1;                /* T_scheduling */
static int hf_rrc_segCount = -1;                  /* SegCount */
static int hf_rrc_sib_Pos = -1;                   /* T_sib_Pos */
static int hf_rrc_rep4 = -1;                      /* INTEGER_0_1 */
static int hf_rrc_rep8 = -1;                      /* INTEGER_0_3 */
static int hf_rrc_rep16 = -1;                     /* INTEGER_0_7 */
static int hf_rrc_rep32 = -1;                     /* INTEGER_0_15 */
static int hf_rrc_rep64 = -1;                     /* INTEGER_0_31 */
static int hf_rrc_rep128 = -1;                    /* INTEGER_0_63 */
static int hf_rrc_rep256 = -1;                    /* INTEGER_0_127 */
static int hf_rrc_rep512 = -1;                    /* INTEGER_0_255 */
static int hf_rrc_rep1024 = -1;                   /* INTEGER_0_511 */
static int hf_rrc_rep2048 = -1;                   /* INTEGER_0_1023 */
static int hf_rrc_rep4096 = -1;                   /* INTEGER_0_2047 */
static int hf_rrc_sib_PosOffsetInfo = -1;         /* SibOFF_List */
static int hf_rrc_sib_Type_01 = -1;               /* SIB_TypeAndTag */
static int hf_rrc_scheduling_01 = -1;             /* SchedulingInformation */
static int hf_rrc_sibSb_Type = -1;                /* SIBSb_TypeAndTag */
static int hf_rrc_sibOccurIdentity = -1;          /* SIBOccurIdentity */
static int hf_rrc_sibOccurValueTag = -1;          /* SIBOccurValueTag */
static int hf_rrc_SIB_ReferenceList_item = -1;    /* SchedulingInformationSIB */
static int hf_rrc_SIBSb_ReferenceList_item = -1;  /* SchedulingInformationSIBSb */
static int hf_rrc_SIB_ReferenceListFACH_item = -1;  /* SchedulingInformationSIB */
static int hf_rrc_sysInfoType1_01 = -1;           /* PLMN_ValueTag */
static int hf_rrc_sysInfoType2 = -1;              /* CellValueTag */
static int hf_rrc_sysInfoType3_01 = -1;           /* CellValueTag */
static int hf_rrc_sysInfoType4 = -1;              /* CellValueTag */
static int hf_rrc_sysInfoType5_01 = -1;           /* CellValueTag */
static int hf_rrc_sysInfoType6 = -1;              /* CellValueTag */
static int hf_rrc_sysInfoType7_01 = -1;           /* NULL */
static int hf_rrc_dummy_55 = -1;                  /* CellValueTag */
static int hf_rrc_dummy2_14 = -1;                 /* NULL */
static int hf_rrc_dummy3_03 = -1;                 /* NULL */
static int hf_rrc_sysInfoType11_01 = -1;          /* CellValueTag */
static int hf_rrc_sysInfoType12_01 = -1;          /* CellValueTag */
static int hf_rrc_sysInfoType13 = -1;             /* CellValueTag */
static int hf_rrc_sysInfoType13_1 = -1;           /* CellValueTag */
static int hf_rrc_sysInfoType13_2 = -1;           /* CellValueTag */
static int hf_rrc_sysInfoType13_3 = -1;           /* CellValueTag */
static int hf_rrc_sysInfoType13_4 = -1;           /* CellValueTag */
static int hf_rrc_sysInfoType14 = -1;             /* NULL */
static int hf_rrc_sysInfoType15 = -1;             /* CellValueTag */
static int hf_rrc_sysInfoType16 = -1;             /* PredefinedConfigIdentityAndValueTag */
static int hf_rrc_sysInfoType17 = -1;             /* NULL */
static int hf_rrc_sysInfoType15_1 = -1;           /* CellValueTag */
static int hf_rrc_sysInfoType15_2 = -1;           /* SIBOccurrenceIdentityAndValueTag */
static int hf_rrc_sysInfoType15_3 = -1;           /* SIBOccurrenceIdentityAndValueTag */
static int hf_rrc_sysInfoType15_4 = -1;           /* CellValueTag */
static int hf_rrc_sysInfoType18 = -1;             /* CellValueTag */
static int hf_rrc_sysInfoType15_5 = -1;           /* CellValueTag */
static int hf_rrc_sysInfoType5bis = -1;           /* CellValueTag */
static int hf_rrc_sysInfoTypeSB1_01 = -1;         /* CellValueTag */
static int hf_rrc_sysInfoTypeSB2_01 = -1;         /* CellValueTag */
static int hf_rrc_systemInfoType11bis = -1;       /* NULL */
static int hf_rrc_systemInfoType15bis = -1;       /* NULL */
static int hf_rrc_systemInfoType15_1bis = -1;     /* NULL */
static int hf_rrc_systemInfoType15_2bis = -1;     /* NULL */
static int hf_rrc_systemInfoType15_3bis = -1;     /* NULL */
static int hf_rrc_systemInfoType15_6 = -1;        /* NULL */
static int hf_rrc_systemInfoType15_7 = -1;        /* NULL */
static int hf_rrc_systemInfoType15_8 = -1;        /* NULL */
static int hf_rrc_systemInfoType19 = -1;          /* NULL */
static int hf_rrc_systemInfoType15_2ter = -1;     /* NULL */
static int hf_rrc_systemInfoType20 = -1;          /* NULL */
static int hf_rrc_systemInfoType21 = -1;          /* NULL */
static int hf_rrc_systemInfoType22 = -1;          /* NULL */
static int hf_rrc_SibOFF_List_item = -1;          /* SibOFF */
static int hf_rrc_cn_CommonGSM_MAP_NAS_SysInfo_03 = -1;  /* T_cn_CommonGSM_MAP_NAS_SysInfo_03 */
static int hf_rrc_cn_DomainSysInfoList = -1;      /* CN_DomainSysInfoList */
static int hf_rrc_ue_IdleTimersAndConstants = -1;  /* UE_IdleTimersAndConstants */
static int hf_rrc_v3a0NonCriticalExtensions_12 = -1;  /* T_v3a0NonCriticalExtensions_12 */
static int hf_rrc_sysInfoType1_v3a0ext = -1;      /* SysInfoType1_v3a0ext_IEs */
static int hf_rrc_v860NonCriticalExtentions_02 = -1;  /* T_v860NonCriticalExtentions_02 */
static int hf_rrc_sysInfoType1_v860ext = -1;      /* SysInfoType1_v860ext_IEs */
static int hf_rrc_nonCriticalExtensions_196 = -1;  /* T_nonCriticalExtensions_196 */
static int hf_rrc_ue_IdleTimersAndConstants_v3a0ext = -1;  /* UE_IdleTimersAndConstants_v3a0ext */
static int hf_rrc_ura_IdentityList = -1;          /* URA_IdentityList */
static int hf_rrc_nonCriticalExtensions_197 = -1;  /* T_nonCriticalExtensions_197 */
static int hf_rrc_sib4indicator = -1;             /* BOOLEAN */
static int hf_rrc_cellSelectReselectInfo = -1;    /* CellSelectReselectInfoSIB_3_4 */
static int hf_rrc_cellAccessRestriction = -1;     /* CellAccessRestriction */
static int hf_rrc_v4b0NonCriticalExtensions_17 = -1;  /* T_v4b0NonCriticalExtensions_17 */
static int hf_rrc_sysInfoType3_v4b0ext = -1;      /* SysInfoType3_v4b0ext_IEs */
static int hf_rrc_v590NonCriticalExtension = -1;  /* T_v590NonCriticalExtension */
static int hf_rrc_sysInfoType3_v590ext = -1;      /* SysInfoType3_v590ext */
static int hf_rrc_v5c0NoncriticalExtension = -1;  /* T_v5c0NoncriticalExtension */
static int hf_rrc_sysInfoType3_v5c0ext = -1;      /* SysInfoType3_v5c0ext_IEs */
static int hf_rrc_v670NonCriticalExtension = -1;  /* T_v670NonCriticalExtension */
static int hf_rrc_sysInfoType3_v670ext = -1;      /* SysInfoType3_v670ext */
static int hf_rrc_v770NonCriticalExtension_02 = -1;  /* T_v770NonCriticalExtension_02 */
static int hf_rrc_sysInfoType3_v770ext = -1;      /* SysInfoType3_v770ext_IEs */
static int hf_rrc_v830NonCriticalExtension = -1;  /* T_v830NonCriticalExtension */
static int hf_rrc_sysInfoType3_v830ext = -1;      /* SysInfoType3_v830ext_IEs */
static int hf_rrc_v860NonCriticalExtension_01 = -1;  /* T_v860NonCriticalExtension_01 */
static int hf_rrc_sysInfoType3_v860ext = -1;      /* SysInfoType3_v860ext_IEs */
static int hf_rrc_v870NonCriticalExtension = -1;  /* T_v870NonCriticalExtension */
static int hf_rrc_sysInfoType3_v870ext = -1;      /* SysInfoType3_v870ext_IEs */
static int hf_rrc_v900NonCriticalExtension = -1;  /* T_v900NonCriticalExtension */
static int hf_rrc_sysInfoType3_v920ext = -1;      /* SysInfoType3_v920ext_IEs */
static int hf_rrc_nonCriticalExtensions_198 = -1;  /* T_nonCriticalExtensions_198 */
static int hf_rrc_mapping_LCR = -1;               /* Mapping_LCR_r4 */
static int hf_rrc_cellSelectReselectInfo_v590ext = -1;  /* CellSelectReselectInfo_v590ext */
static int hf_rrc_cellSelectReselectInfoTreselectionScaling_v5c0ext = -1;  /* CellSelectReselectInfoTreselectionScaling_v5c0ext */
static int hf_rrc_domainSpecificAccessRestrictionParametersForPLMNOfMIB = -1;  /* DomainSpecificAccessRestrictionParam_v670ext */
static int hf_rrc_domainSpecificAccessRestictionForSharedNetwork = -1;  /* DomainSpecificAccessRestrictionForSharedNetwork_v670ext */
static int hf_rrc_deferredMeasurementControlReadingSupport = -1;  /* DeferredMeasurementControlReadingSupport */
static int hf_rrc_q_QualMin_Offset = -1;          /* Q_QualMin_Offset */
static int hf_rrc_q_RxlevMin_Offset = -1;         /* Q_RxlevMin_Offset */
static int hf_rrc_mbsfnOnlyService = -1;          /* MBSFNOnlyService */
static int hf_rrc_pagingPermissionWithAccessControlParametersForPLMNOfMIB = -1;  /* PagingPermissionWithAccessControlParameters */
static int hf_rrc_pagingPermissionWithAccessControlParametersForSharedNetwork = -1;  /* PagingPermissionWithAccessControlForSharedNetwork */
static int hf_rrc_csg_PSCSplitInfo = -1;          /* CSG_PSCSplitInfo */
static int hf_rrc_cellAccessRestriction_01 = -1;  /* CellAccessRestriction_v870ext */
static int hf_rrc_imsEmergencySupportIndicator = -1;  /* T_imsEmergencySupportIndicator */
static int hf_rrc_v4b0NonCriticalExtensions_18 = -1;  /* T_v4b0NonCriticalExtensions_18 */
static int hf_rrc_sysInfoType4_v4b0ext = -1;      /* SysInfoType4_v4b0ext_IEs */
static int hf_rrc_v590NonCriticalExtension_01 = -1;  /* T_v590NonCriticalExtension_01 */
static int hf_rrc_sysInfoType4_v590ext = -1;      /* SysInfoType4_v590ext */
static int hf_rrc_v5b0NonCriticalExtension = -1;  /* T_v5b0NonCriticalExtension */
static int hf_rrc_sysInfoType4_v5b0ext = -1;      /* SysInfoType4_v5b0ext_IEs */
static int hf_rrc_v5c0NonCriticalExtension = -1;  /* T_v5c0NonCriticalExtension */
static int hf_rrc_sysInfoType4_v5c0ext = -1;      /* SysInfoType4_v5c0ext_IEs */
static int hf_rrc_nonCriticalExtensions_199 = -1;  /* T_nonCriticalExtensions_199 */
static int hf_rrc_cellSelectReselectInfoPCHFACH_v5b0ext = -1;  /* CellSelectReselectInfoPCHFACH_v5b0ext */
static int hf_rrc_sib6indicator = -1;             /* BOOLEAN */
static int hf_rrc_pich_PowerOffset = -1;          /* PICH_PowerOffset */
static int hf_rrc_modeSpecificInfo_150 = -1;      /* T_modeSpecificInfo_150 */
static int hf_rrc_fdd_191 = -1;                   /* T_fdd_185 */
static int hf_rrc_aich_PowerOffset = -1;          /* AICH_PowerOffset */
static int hf_rrc_tdd_142 = -1;                   /* T_tdd_134 */
static int hf_rrc_pusch_SysInfoList_SFN = -1;     /* PUSCH_SysInfoList_SFN */
static int hf_rrc_pdsch_SysInfoList_SFN = -1;     /* PDSCH_SysInfoList_SFN */
static int hf_rrc_openLoopPowerControl_TDD = -1;  /* OpenLoopPowerControl_TDD */
static int hf_rrc_prach_SystemInformationList = -1;  /* PRACH_SystemInformationList */
static int hf_rrc_sCCPCH_SystemInformationList = -1;  /* SCCPCH_SystemInformationList */
static int hf_rrc_cbs_DRX_Level1Information = -1;  /* CBS_DRX_Level1Information */
static int hf_rrc_v4b0NonCriticalExtensions_19 = -1;  /* T_v4b0NonCriticalExtensions_19 */
static int hf_rrc_sysInfoType5_v4b0ext = -1;      /* SysInfoType5_v4b0ext_IEs */
static int hf_rrc_v590NonCriticalExtensions_28 = -1;  /* T_v590NonCriticalExtensions_28 */
static int hf_rrc_sysInfoType5_v590ext = -1;      /* SysInfoType5_v590ext_IEs */
static int hf_rrc_v650NonCriticalExtensions = -1;  /* T_v650NonCriticalExtensions */
static int hf_rrc_sysInfoType5_v650ext = -1;      /* SysInfoType5_v650ext_IEs */
static int hf_rrc_v680NonCriticalExtensions_02 = -1;  /* T_v680NonCriticalExtensions_02 */
static int hf_rrc_sysInfoType5_v680ext = -1;      /* SysInfoType5_v680ext_IEs */
static int hf_rrc_v690NonCriticalExtensions_49 = -1;  /* T_v690NonCriticalExtensions_49 */
static int hf_rrc_sysInfoType5_v690ext = -1;      /* SysInfoType5_v690ext_IEs */
static int hf_rrc_v770NonCriticalExtensions_40 = -1;  /* T_v770NonCriticalExtensions_40 */
static int hf_rrc_sysInfoType5_v770ext = -1;      /* SysInfoType5_v770ext_IEs */
static int hf_rrc_v860NonCriticalExtensions_27 = -1;  /* T_v860NonCriticalExtensions_27 */
static int hf_rrc_sysInfoType5_v860ext = -1;      /* SysInfoType5_v860ext_IEs */
static int hf_rrc_v890NonCriticalExtensions_11 = -1;  /* T_v890NonCriticalExtensions_11 */
static int hf_rrc_sysInfoType5_v890ext = -1;      /* SysInfoType5_v890ext_IEs */
static int hf_rrc_v8b0NonCriticalExtensions_01 = -1;  /* T_v8b0NonCriticalExtensions_01 */
static int hf_rrc_sysInfoType5_v8b0ext = -1;      /* SysInfoType5_v8b0ext_IEs */
static int hf_rrc_v8d0NonCriticalExtensions = -1;  /* T_v8d0NonCriticalExtensions */
static int hf_rrc_sysInfoType5_v8d0ext = -1;      /* SysInfoType5_v8d0ext_IEs */
static int hf_rrc_va40NonCriticalExtensions_19 = -1;  /* T_va40NonCriticalExtensions_18 */
static int hf_rrc_sysInfoType5_va40ext = -1;      /* SysInfoType5_va40ext_IEs */
static int hf_rrc_va80NonCriticalExtensions_02 = -1;  /* T_va80NonCriticalExtensions_02 */
static int hf_rrc_sysInfoType5_va80ext = -1;      /* SysInfoType5_va80ext_IEs */
static int hf_rrc_vbxyNonCriticalExtensions_19 = -1;  /* SysInfoType5_NonCriticalExtensions_vbxy_IEs */
static int hf_rrc_pNBSCH_Allocation_r4 = -1;      /* PNBSCH_Allocation_r4 */
static int hf_rrc_prach_SystemInformationList_LCR_r4 = -1;  /* PRACH_SystemInformationList_LCR_r4 */
static int hf_rrc_tdd128SpecificInfo = -1;        /* T_tdd128SpecificInfo */
static int hf_rrc_pusch_SysInfoList_SFN_01 = -1;  /* PUSCH_SysInfoList_SFN_LCR_r4 */
static int hf_rrc_pdsch_SysInfoList_SFN_01 = -1;  /* PDSCH_SysInfoList_SFN_LCR_r4 */
static int hf_rrc_pCCPCH_LCR_Extensions = -1;     /* PrimaryCCPCH_Info_LCR_r4_ext */
static int hf_rrc_sCCPCH_LCR_ExtensionsList = -1;  /* SCCPCH_SystemInformationList_LCR_r4_ext */
static int hf_rrc_frequencyBandIndicator = -1;    /* RadioFrequencyBandFDD */
static int hf_rrc_hcr_r5_SpecificInfo = -1;       /* T_hcr_r5_SpecificInfo */
static int hf_rrc_pusch_SysInfoList_SFN_02 = -1;  /* PUSCH_SysInfoList_SFN_HCR_r5 */
static int hf_rrc_pdsch_SysInfoList_SFN_02 = -1;  /* PDSCH_SysInfoList_SFN_HCR_r5 */
static int hf_rrc_frequencyBandIndicator2 = -1;   /* RadioFrequencyBandFDD2 */
static int hf_rrc_hsdpa_CellIndicator = -1;       /* T_hsdpa_CellIndicator */
static int hf_rrc_edch_CellIndicator = -1;        /* T_edch_CellIndicator */
static int hf_rrc_sccpch_SystemInformation_MBMS = -1;  /* T_sccpch_SystemInformation_MBMS */
static int hf_rrc_mcchOnSCCPCHusedForNonMBMS = -1;  /* MBMS_MCCH_ConfigurationInfo_r6 */
static int hf_rrc_mcchOnSCCPCHusedOnlyForMBMS = -1;  /* SCCPCH_SystemInformation_MBMS_r6 */
static int hf_rrc_additionalPRACH_TF_and_TFCS_CCCH_List = -1;  /* AdditionalPRACH_TF_and_TFCS_CCCH_List */
static int hf_rrc_cBS_DRX_Level1Information_extension = -1;  /* CBS_DRX_Level1Information_extension_r6 */
static int hf_rrc_modeSpecificInfo_151 = -1;      /* T_modeSpecificInfo_151 */
static int hf_rrc_fdd_192 = -1;                   /* T_fdd_186 */
static int hf_rrc_hs_dsch_CommonSystemInformation = -1;  /* HS_DSCH_CommonSystemInformation */
static int hf_rrc_hs_dsch_PagingSystemInformation = -1;  /* HS_DSCH_PagingSystemInformation */
static int hf_rrc_tdd768_27 = -1;                 /* T_tdd768_24 */
static int hf_rrc_pusch_SysInfoList_SFN_03 = -1;  /* PUSCH_SysInfoList_SFN_VHCR */
static int hf_rrc_pdsch_SysInfoList_SFN_03 = -1;  /* PDSCH_SysInfoList_VHCR_r7 */
static int hf_rrc_prach_SystemInformationList_01 = -1;  /* PRACH_SystemInformationList_VHCR_r7 */
static int hf_rrc_tdd128_71 = -1;                 /* T_tdd128_68 */
static int hf_rrc_prach_SystemInformationList_02 = -1;  /* PRACH_SystemInformationList_LCR_v770ext */
static int hf_rrc_sccpch_SystemInformationList = -1;  /* SCCPCH_SystemInformationList_HCR_VHCR_r7 */
static int hf_rrc_sccpch_SystemInformation_MBMS_01 = -1;  /* T_sccpch_SystemInformation_MBMS_01 */
static int hf_rrc_mcchOnSCCPCHusedOnlyForMBMS_01 = -1;  /* SCCPCH_SystemInformation_MBMS_r7 */
static int hf_rrc_tDD_MBSFNInformation = -1;      /* TDD_MBSFNInformation */
static int hf_rrc_dummy_56 = -1;                  /* T_dummy_15 */
static int hf_rrc_modeSpecificInfo_152 = -1;      /* T_modeSpecificInfo_152 */
static int hf_rrc_fdd_193 = -1;                   /* T_fdd_187 */
static int hf_rrc_hs_dsch_DrxCellfach_info = -1;  /* HS_DSCH_DrxCellfach_info */
static int hf_rrc_tdd128_72 = -1;                 /* T_tdd128_69 */
static int hf_rrc_hs_dsch_CommonSysInfo = -1;     /* T_hs_dsch_CommonSysInfo */
static int hf_rrc_hs_dsch_CommonSystemInformation_01 = -1;  /* HS_DSCH_CommonSystemInformation_TDD128 */
static int hf_rrc_hs_dsch_PagingSystemInformation_01 = -1;  /* HS_DSCH_PagingSystemInformation_TDD128 */
static int hf_rrc_hs_dsch_DrxCellfach_info_01 = -1;  /* HS_DSCH_DrxCellfach_info_TDD128 */
static int hf_rrc_commonEDCHSystemInfo = -1;      /* CommonEDCHSystemInfo */
static int hf_rrc_tresetUsageIndicator = -1;      /* T_tresetUsageIndicator */
static int hf_rrc_commonEDCHSystemInfoFDD = -1;   /* CommonEDCHSystemInfoFDD */
static int hf_rrc_frequencyBandsIndicatorSupport = -1;  /* FrequencyBandsIndicatorSupport */
static int hf_rrc_frequencyBandIndicator3 = -1;   /* RadioFrequencyBandFDD3 */
static int hf_rrc_sysInfoType5_vbxyext = -1;      /* SysInfoType5_vbxyext_IEs */
static int hf_rrc_nonCriticalExtensions_200 = -1;  /* T_nonCriticalExtensions_200 */
static int hf_rrc_sib22indicator = -1;            /* BOOLEAN */
static int hf_rrc_modeSpecificInfo_153 = -1;      /* T_modeSpecificInfo_153 */
static int hf_rrc_fdd_194 = -1;                   /* T_fdd_188 */
static int hf_rrc_dummy_57 = -1;                  /* CSICH_PowerOffset */
static int hf_rrc_tdd_143 = -1;                   /* T_tdd_135 */
static int hf_rrc_v4b0NonCriticalExtensions_20 = -1;  /* T_v4b0NonCriticalExtensions_20 */
static int hf_rrc_sysInfoType6_v4b0ext = -1;      /* SysInfoType6_v4b0ext_IEs */
static int hf_rrc_v590NonCriticalExtensions_29 = -1;  /* T_v590NonCriticalExtensions_29 */
static int hf_rrc_sysInfoType6_v590ext = -1;      /* SysInfoType6_v590ext_IEs */
static int hf_rrc_v650nonCriticalExtensions = -1;  /* T_v650nonCriticalExtensions */
static int hf_rrc_sysInfoType6_v650ext = -1;      /* SysInfoType6_v650ext_IEs */
static int hf_rrc_v690nonCriticalExtensions = -1;  /* T_v690nonCriticalExtensions */
static int hf_rrc_sysInfoType6_v690ext = -1;      /* SysInfoType6_v690ext_IEs */
static int hf_rrc_v770NonCriticalExtensions_41 = -1;  /* T_v770NonCriticalExtensions_41 */
static int hf_rrc_sysInfoType6_v770ext = -1;      /* SysInfoType6_v770ext_IEs */
static int hf_rrc_va40NonCriticalExtensions_20 = -1;  /* T_va40NonCriticalExtensions_19 */
static int hf_rrc_sysInfoType6_va40ext = -1;      /* SysInfoType6_va40ext_IEs */
static int hf_rrc_va80NonCriticalExtensions_03 = -1;  /* T_va80NonCriticalExtensions_03 */
static int hf_rrc_sysInfoType6_va80ext = -1;      /* SysInfoType6_va80ext_IEs */
static int hf_rrc_nonCriticalExtensions_201 = -1;  /* T_nonCriticalExtensions_201 */
static int hf_rrc_tdd128SpecificInfo_01 = -1;     /* T_tdd128SpecificInfo_01 */
static int hf_rrc_hcr_r5_SpecificInfo_01 = -1;    /* T_hcr_r5_SpecificInfo_01 */
static int hf_rrc_tdd768SpecificInfo = -1;        /* T_tdd768SpecificInfo */
static int hf_rrc_modeSpecificInfo_154 = -1;      /* T_modeSpecificInfo_154 */
static int hf_rrc_fdd_195 = -1;                   /* T_fdd_189 */
static int hf_rrc_ul_Interference = -1;           /* UL_Interference */
static int hf_rrc_prach_Information_SIB5_List = -1;  /* DynamicPersistenceLevelList */
static int hf_rrc_prach_Information_SIB6_List = -1;  /* DynamicPersistenceLevelList */
static int hf_rrc_expirationTimeFactor = -1;      /* ExpirationTimeFactor */
static int hf_rrc_nonCriticalExtensions_202 = -1;  /* T_nonCriticalExtensions_202 */
static int hf_rrc_dummy1_07 = -1;                 /* CPCH_Parameters */
static int hf_rrc_dummy2_15 = -1;                 /* CPCH_SetInfoList */
static int hf_rrc_dummy3_04 = -1;                 /* CSICH_PowerOffset */
static int hf_rrc_nonCriticalExtensions_203 = -1;  /* T_nonCriticalExtensions_203 */
static int hf_rrc_dummy_58 = -1;                  /* CPCH_PersistenceLevelsList */
static int hf_rrc_nonCriticalExtensions_204 = -1;  /* T_nonCriticalExtensions_204 */
static int hf_rrc_dummy_59 = -1;                  /* DRAC_SysInfoList */
static int hf_rrc_nonCriticalExtensions_205 = -1;  /* T_nonCriticalExtensions_205 */
static int hf_rrc_sib12indicator = -1;            /* BOOLEAN */
static int hf_rrc_fach_MeasurementOccasionInfo = -1;  /* FACH_MeasurementOccasionInfo */
static int hf_rrc_measurementControlSysInfo = -1;  /* MeasurementControlSysInfo */
static int hf_rrc_v4b0NonCriticalExtensions_21 = -1;  /* T_v4b0NonCriticalExtensions_21 */
static int hf_rrc_sysInfoType11_v4b0ext = -1;     /* SysInfoType11_v4b0ext_IEs */
static int hf_rrc_v590NonCriticalExtension_02 = -1;  /* T_v590NonCriticalExtension_02 */
static int hf_rrc_sysInfoType11_v590ext = -1;     /* SysInfoType11_v590ext_IEs */
static int hf_rrc_v690NonCriticalExtensions_50 = -1;  /* T_v690NonCriticalExtensions_50 */
static int hf_rrc_sysInfoType11_v690ext = -1;     /* SysInfoType11_v690ext_IEs */
static int hf_rrc_v6b0NonCriticalExtensions_17 = -1;  /* T_v6b0NonCriticalExtensions_16 */
static int hf_rrc_sysInfoType11_v6b0ext = -1;     /* SysInfoType11_v6b0ext_IEs */
static int hf_rrc_v770NonCriticalExtensions_42 = -1;  /* T_v770NonCriticalExtensions_42 */
static int hf_rrc_sysInfoType11_v770ext = -1;     /* SysInfoType11_v770ext_IEs */
static int hf_rrc_v7b0NonCriticalExtensions_02 = -1;  /* T_v7b0NonCriticalExtensions_02 */
static int hf_rrc_sysInfoType11_v7b0ext = -1;     /* SysInfoType11_v7b0ext_IEs */
static int hf_rrc_v860NonCriticalExtensions_28 = -1;  /* T_v860NonCriticalExtensions_28 */
static int hf_rrc_sysInfoType11_v860ext = -1;     /* SysInfoType11_v860ext_IEs */
static int hf_rrc_va80NonCriticalExtensions_04 = -1;  /* T_va80NonCriticalExtensions_04 */
static int hf_rrc_sysInfoType11_va80ext = -1;     /* SysInfoType11_va80ext_IEs */
static int hf_rrc_nonCriticalExtensions_206 = -1;  /* T_nonCriticalExtensions_206 */
static int hf_rrc_fach_MeasurementOccasionInfo_LCR_Ext = -1;  /* FACH_MeasurementOccasionInfo_LCR_r4_ext */
static int hf_rrc_measurementControlSysInfo_LCR = -1;  /* MeasurementControlSysInfo_LCR_r4_ext */
static int hf_rrc_newIntraFrequencyCellInfoList_v590ext = -1;  /* SEQUENCE_SIZE_1_maxCellMeas_OF_CellSelectReselectInfo_v590ext */
static int hf_rrc_newIntraFrequencyCellInfoList_v590ext_item = -1;  /* CellSelectReselectInfo_v590ext */
static int hf_rrc_newInterFrequencyCellInfoList_v590ext = -1;  /* SEQUENCE_SIZE_1_maxCellMeas_OF_CellSelectReselectInfo_v590ext */
static int hf_rrc_newInterFrequencyCellInfoList_v590ext_item = -1;  /* CellSelectReselectInfo_v590ext */
static int hf_rrc_newInterRATCellInfoList_v590ext = -1;  /* SEQUENCE_SIZE_1_maxCellMeas_OF_CellSelectReselectInfo_v590ext */
static int hf_rrc_newInterRATCellInfoList_v590ext_item = -1;  /* CellSelectReselectInfo_v590ext */
static int hf_rrc_intraFreqEventCriteriaList_v590ext = -1;  /* Intra_FreqEventCriteriaList_v590ext */
static int hf_rrc_dummy_60 = -1;                  /* Dummy_InterFreqRACHReportingInfo */
static int hf_rrc_interFreqRACHReportingInfo = -1;  /* InterFreqRACHReportingInfo */
static int hf_rrc_mbsfnFrequencyList = -1;        /* MBSFNFrequencyList */
static int hf_rrc_mbsfnFrequencyList_01 = -1;     /* MBSFNFrequencyList_v860ext */
static int hf_rrc_multipleFrequencyInfoListFDD = -1;  /* MultipleFrequencyInfoListFDD */
static int hf_rrc_measurementControlSysInfo_01 = -1;  /* MeasurementControlSysInfoExtension */
static int hf_rrc_measurementControlSysInfo_LCR_01 = -1;  /* MeasurementControlSysInfoExtension_LCR_r4 */
static int hf_rrc_measurementControlSysInfoExtensionAddon_r5 = -1;  /* MeasurementControlSysInfoExtensionAddon_r5 */
static int hf_rrc_v7b0NonCriticalExtensions_03 = -1;  /* T_v7b0NonCriticalExtensions_03 */
static int hf_rrc_sysInfoType11bis_v7b0ext = -1;  /* SysInfoType11bis_v7b0ext_IEs */
static int hf_rrc_v860NonCriticalExtensions_29 = -1;  /* T_v860NonCriticalExtensions_29 */
static int hf_rrc_sysInfoType11bis_v860ext = -1;  /* SysInfoType11bis_v860ext_IEs */
static int hf_rrc_va80NonCriticalExtensions_05 = -1;  /* T_va80NonCriticalExtensions_05 */
static int hf_rrc_sysInfoType11bis_va80ext = -1;  /* SysInfoType11bis_va80ext_IEs */
static int hf_rrc_nonCriticalExtensions_207 = -1;  /* T_nonCriticalExtensions_207 */
static int hf_rrc_csg_DedicatedFrequencyInfoList = -1;  /* CSG_DedicatedFrequencyInfoList */
static int hf_rrc_v4b0NonCriticalExtensions_22 = -1;  /* T_v4b0NonCriticalExtensions_22 */
static int hf_rrc_sysInfoType12_v4b0ext = -1;     /* SysInfoType12_v4b0ext_IEs */
static int hf_rrc_v590NonCriticalExtension_03 = -1;  /* T_v590NonCriticalExtension_03 */
static int hf_rrc_sysInfoType12_v590ext = -1;     /* SysInfoType12_v590ext_IEs */
static int hf_rrc_v690NonCriticalExtensions_51 = -1;  /* T_v690NonCriticalExtensions_51 */
static int hf_rrc_sysInfoType12_v690ext = -1;     /* SysInfoType12_v690ext_IEs */
static int hf_rrc_v6b0NonCriticalExtensions_18 = -1;  /* T_v6b0NonCriticalExtensions_17 */
static int hf_rrc_sysInfoType12_v6b0ext = -1;     /* SysInfoType12_v6b0ext_IEs */
static int hf_rrc_v7b0NonCriticalExtensions_04 = -1;  /* T_v7b0NonCriticalExtensions_04 */
static int hf_rrc_sysInfoType12_v7b0ext = -1;     /* SysInfoType12_v7b0ext_IEs */
static int hf_rrc_va80NonCriticalExtensions_06 = -1;  /* T_va80NonCriticalExtensions_06 */
static int hf_rrc_sysInfoType12_va80ext = -1;     /* SysInfoType12_va80ext_IEs */
static int hf_rrc_nonCriticalExtensions_208 = -1;  /* T_nonCriticalExtensions_208 */
static int hf_rrc_v3a0NonCriticalExtensions_13 = -1;  /* T_v3a0NonCriticalExtensions_13 */
static int hf_rrc_sysInfoType13_v3a0ext = -1;     /* SysInfoType13_v3a0ext_IEs */
static int hf_rrc_v4b0NonCriticalExtensions_23 = -1;  /* T_v4b0NonCriticalExtensions_23 */
static int hf_rrc_sysInfoType13_v4b0ext = -1;     /* SysInfoType13_v4b0ext_IEs */
static int hf_rrc_v770NonCriticalExtensions_43 = -1;  /* T_v770NonCriticalExtensions_43 */
static int hf_rrc_sysInfoType13_v770ext = -1;     /* SysInfoType13_v770ext_IEs */
static int hf_rrc_nonCriticalExtensions_209 = -1;  /* T_nonCriticalExtensions_209 */
static int hf_rrc_capabilityUpdateRequirement_r4Ext = -1;  /* CapabilityUpdateRequirement_r4_ext */
static int hf_rrc_ansi_41_RAND_Information = -1;  /* ANSI_41_RAND_Information */
static int hf_rrc_nonCriticalExtensions_210 = -1;  /* T_nonCriticalExtensions_210 */
static int hf_rrc_ansi_41_UserZoneID_Information = -1;  /* ANSI_41_UserZoneID_Information */
static int hf_rrc_nonCriticalExtensions_211 = -1;  /* T_nonCriticalExtensions_211 */
static int hf_rrc_ansi_41_PrivateNeighbourListInfo = -1;  /* ANSI_41_PrivateNeighbourListInfo */
static int hf_rrc_nonCriticalExtensions_212 = -1;  /* T_nonCriticalExtensions_212 */
static int hf_rrc_ansi_41_GlobalServiceRedirectInfo = -1;  /* ANSI_41_GlobalServiceRedirectInfo */
static int hf_rrc_nonCriticalExtensions_213 = -1;  /* T_nonCriticalExtensions_213 */
static int hf_rrc_nonCriticalExtensions_214 = -1;  /* T_nonCriticalExtensions_214 */
static int hf_rrc_ue_positioning_GPS_CipherParameters = -1;  /* UE_Positioning_CipherParameters */
static int hf_rrc_v4b0NonCriticalExtensions_24 = -1;  /* T_v4b0NonCriticalExtensions_24 */
static int hf_rrc_sysInfoType15_v4b0ext = -1;     /* SysInfoType15_v4b0ext_IEs */
static int hf_rrc_v770NonCriticalExtensions_44 = -1;  /* T_v770NonCriticalExtensions_44 */
static int hf_rrc_sysInfoType15_v770ext = -1;     /* SysInfoType15_v770ext_IEs */
static int hf_rrc_va40NonCriticalExtensions_21 = -1;  /* T_va40NonCriticalExtensions_20 */
static int hf_rrc_sysInfoType15_va40ext = -1;     /* SysInfoType15_va40ext_IEs */
static int hf_rrc_nonCriticalExtensions_215 = -1;  /* T_nonCriticalExtensions_215 */
static int hf_rrc_up_Ipdl_Parameters_TDD = -1;    /* UE_Positioning_IPDL_Parameters_TDD_r4_ext */
static int hf_rrc_ue_Positioning_GPS_ReferenceTime = -1;  /* UE_Positioning_GPS_ReferenceTime_v770ext */
static int hf_rrc_ue_Positioning_GPS_ReferenceTime_01 = -1;  /* UE_Positioning_GPS_ReferenceTime_va40ext */
static int hf_rrc_ue_positioning_GANSS_ReferencePosition = -1;  /* ReferenceLocationGANSS */
static int hf_rrc_ue_positioning_GANSS_IonosphericModel = -1;  /* UE_Positioning_GANSS_IonosphericModel */
static int hf_rrc_v860NonCriticalExtensions_30 = -1;  /* T_v860NonCriticalExtensions_30 */
static int hf_rrc_sysInfoType15bis_v860ext = -1;  /* SysInfoType15bis_v860ext_IEs */
static int hf_rrc_va40NonCriticalExtensions_22 = -1;  /* T_va40NonCriticalExtensions_21 */
static int hf_rrc_sysInfoType15bis_va40ext = -1;  /* SysInfoType15bis_va40ext_IEs */
static int hf_rrc_nonCriticalExtensions_216 = -1;  /* T_nonCriticalExtensions_216 */
static int hf_rrc_ue_Positioning_GANSS_AddIonoModel = -1;  /* UE_Positioning_GANSS_AddIonoModel */
static int hf_rrc_ue_Positioning_GANSS_EarthOrientationPara = -1;  /* UE_Positioning_GANSS_EarthOrientPara */
static int hf_rrc_v920NonCriticalExtensions_04 = -1;  /* T_v920NonCriticalExtensions_04 */
static int hf_rrc_sysInfoType15_1_v920ext = -1;   /* SysInfoType15_1_v920ext_IEs */
static int hf_rrc_nonCriticalExtensions_217 = -1;  /* T_nonCriticalExtensions_217 */
static int hf_rrc_ue_positioning_GANSS_DGANSS_Corrections = -1;  /* UE_Positioning_DGANSSCorrections */
static int hf_rrc_v920NonCriticalExtensions_05 = -1;  /* T_v920NonCriticalExtensions_05 */
static int hf_rrc_sysInfoType15_1bis_v920ext = -1;  /* SysInfoType15_1bis_v920ext_IEs */
static int hf_rrc_nonCriticalExtensions_218 = -1;  /* T_nonCriticalExtensions_218 */
static int hf_rrc_ue_positioning_GANSS_DGANSS_Corrections_01 = -1;  /* UE_Positioning_DGANSSCorrections_v920ext */
static int hf_rrc_transmissionTOW = -1;           /* GPS_TOW_1sec */
static int hf_rrc_nonCriticalExtensions_219 = -1;  /* T_nonCriticalExtensions_219 */
static int hf_rrc_ue_positioning_GANSS_navigationModel = -1;  /* UE_Positioning_GANSS_NavigationModel */
static int hf_rrc_nonCriticalExtensions_220 = -1;  /* T_nonCriticalExtensions_220 */
static int hf_rrc_ue_positioning_GANSS_AddNavigationModels = -1;  /* UE_Positioning_GANSS_AddNavigationModels */
static int hf_rrc_nonCriticalExtensions_221 = -1;  /* T_nonCriticalExtensions_221 */
static int hf_rrc_satMask = -1;                   /* BIT_STRING_SIZE_1_32 */
static int hf_rrc_lsbTOW = -1;                    /* BIT_STRING_SIZE_8 */
static int hf_rrc_va40NonCriticalExtensions_23 = -1;  /* T_va40NonCriticalExtensions_22 */
static int hf_rrc_sysInfoType15_3_va40ext = -1;   /* SysInfoType15_3_va40ext_IEs */
static int hf_rrc_nonCriticalExtensions_222 = -1;  /* T_nonCriticalExtensions_222 */
static int hf_rrc_ue_positioning_GANSS_Almanac = -1;  /* UE_Positioning_GANSS_Almanac */
static int hf_rrc_ue_positioning_GANSS_TimeModels = -1;  /* UE_Positioning_GANSS_TimeModels */
static int hf_rrc_ue_positioning_GANSS_UTC_Model = -1;  /* UE_Positioning_GANSS_UTCModel */
static int hf_rrc_v860NonCriticalExtensions_31 = -1;  /* T_v860NonCriticalExtensions_31 */
static int hf_rrc_sysInfoType15_3bis_v860ext = -1;  /* SysInfoType15_3bis_v860ext_IEs */
static int hf_rrc_va40NonCriticalExtensions_24 = -1;  /* T_va40NonCriticalExtensions_23 */
static int hf_rrc_sysInfoType15_3bis_va40ext = -1;  /* SysInfoType15_3bis_va40ext_IEs */
static int hf_rrc_nonCriticalExtensions_223 = -1;  /* T_nonCriticalExtensions_223 */
static int hf_rrc_ue_Positioning_GANSS_Almanac = -1;  /* UE_Positioning_GANSS_Almanac_v860ext */
static int hf_rrc_ue_Positioning_GANSS_AddUTCModels = -1;  /* UE_Positioning_GANSS_AddUTCModels */
static int hf_rrc_ue_Positioning_GANSS_AuxiliaryInfo = -1;  /* UE_Positioning_GANSS_AuxiliaryInfo */
static int hf_rrc_ue_Positioning_GANSS_Almanac_01 = -1;  /* UE_Positioning_GANSS_Almanac_va40ext */
static int hf_rrc_ue_positioning_GANSS_TimeModels_01 = -1;  /* UE_Positioning_GANSS_TimeModels_va40ext */
static int hf_rrc_ue_positioning_OTDOA_CipherParameters = -1;  /* UE_Positioning_CipherParameters */
static int hf_rrc_v3a0NonCriticalExtensions_14 = -1;  /* T_v3a0NonCriticalExtensions_14 */
static int hf_rrc_sysInfoType15_4_v3a0ext = -1;   /* SysInfoType15_4_v3a0ext */
static int hf_rrc_v4b0NonCriticalExtensions_25 = -1;  /* T_v4b0NonCriticalExtensions_25 */
static int hf_rrc_sysInfoType15_4_v4b0ext = -1;   /* SysInfoType15_4_v4b0ext */
static int hf_rrc_nonCriticalExtensions_224 = -1;  /* T_nonCriticalExtensions_224 */
static int hf_rrc_v3a0NonCriticalExtensions_15 = -1;  /* T_v3a0NonCriticalExtensions_15 */
static int hf_rrc_sysInfoType15_5_v3a0ext = -1;   /* SysInfoType15_5_v3a0ext */
static int hf_rrc_v770NonCriticalExtensions_45 = -1;  /* T_v770NonCriticalExtensions_45 */
static int hf_rrc_sysInfoType15_5_v770ext = -1;   /* SysInfoType15_5_v770ext_IEs */
static int hf_rrc_nonCriticalExtensions_225 = -1;  /* T_nonCriticalExtensions_225 */
static int hf_rrc_ue_positioning_GANSS_TOD = -1;  /* INTEGER_0_86399 */
static int hf_rrc_ue_positioning_GANSS_ReferenceMeasurementInformation = -1;  /* UE_Positioning_GANSS_ReferenceMeasurementInfo */
static int hf_rrc_va40NonCriticalExtensions_25 = -1;  /* T_va40NonCriticalExtensions_24 */
static int hf_rrc_sysInfoType15_6_va40ext = -1;   /* SysInfoType15_6_va40ext_IEs */
static int hf_rrc_nonCriticalExtensions_226 = -1;  /* T_nonCriticalExtensions_226 */
static int hf_rrc_ue_positioning_GANSS_ReferenceMeasurementInformation_01 = -1;  /* UE_Positioning_GANSS_ReferenceMeasurementInfo_va40ext */
static int hf_rrc_ue_positioning_GANSS_DataBitAssistance = -1;  /* UE_Positioning_GANSS_Data_Bit_Assistance */
static int hf_rrc_nonCriticalExtensions_227 = -1;  /* T_nonCriticalExtensions_227 */
static int hf_rrc_ue_positioning_GANSS_DataCipheringInfo = -1;  /* UE_Positioning_CipherParameters */
static int hf_rrc_ue_positioning_GANSS_realTimeIntegrity = -1;  /* UE_Positioning_GANSS_RealTimeIntegrity */
static int hf_rrc_nonCriticalExtensions_228 = -1;  /* T_nonCriticalExtensions_228 */
static int hf_rrc_preDefinedRadioConfiguration = -1;  /* PreDefRadioConfiguration */
static int hf_rrc_v770NonCriticalExtensions_46 = -1;  /* T_v770NonCriticalExtensions_46 */
static int hf_rrc_sysInfoType16_v770ext = -1;     /* SysInfoType16_v770ext_IEs */
static int hf_rrc_v920NonCriticalExtensions_06 = -1;  /* T_v920NonCriticalExtensions_06 */
static int hf_rrc_sysInfoType16_v920ext = -1;     /* SysInfoType16_v920ext_IEs */
static int hf_rrc_nonCriticalExtensions_229 = -1;  /* T_nonCriticalExtensions_229 */
static int hf_rrc_preDefinedRadioConfiguration_01 = -1;  /* PreDefRadioConfiguration_v770ext */
static int hf_rrc_preDefinedRadioConfiguration_02 = -1;  /* PreDefRadioConfiguration_v920ext */
static int hf_rrc_pusch_SysInfoList = -1;         /* PUSCH_SysInfoList */
static int hf_rrc_pdsch_SysInfoList = -1;         /* PDSCH_SysInfoList */
static int hf_rrc_v4b0NonCriticalExtensions_26 = -1;  /* T_v4b0NonCriticalExtensions_26 */
static int hf_rrc_sysInfoType17_v4b0ext = -1;     /* SysInfoType17_v4b0ext_IEs */
static int hf_rrc_v590NonCriticalExtensions_30 = -1;  /* T_v590NonCriticalExtensions_30 */
static int hf_rrc_sysInfoType17_v590ext = -1;     /* SysInfoType17_v590ext_IEs */
static int hf_rrc_v770NonCriticalExtensions_47 = -1;  /* T_v770NonCriticalExtensions_47 */
static int hf_rrc_sysInfoType17_v770ext = -1;     /* SysInfoType17_v770ext_IEs */
static int hf_rrc_nonCriticalExtensions_230 = -1;  /* T_nonCriticalExtensions_230 */
static int hf_rrc_tdd128SpecificInfo_02 = -1;     /* T_tdd128SpecificInfo_02 */
static int hf_rrc_pusch_SysInfoList_01 = -1;      /* PUSCH_SysInfoList_LCR_r4 */
static int hf_rrc_pdsch_SysInfoList_01 = -1;      /* PDSCH_SysInfoList_LCR_r4 */
static int hf_rrc_hcr_r5_SpecificInfo_02 = -1;    /* T_hcr_r5_SpecificInfo_02 */
static int hf_rrc_pusch_SysInfoList_02 = -1;      /* PUSCH_SysInfoList_HCR_r5 */
static int hf_rrc_pdsch_SysInfoList_02 = -1;      /* PDSCH_SysInfoList_HCR_r5 */
static int hf_rrc_tdd768SpecificInfo_01 = -1;     /* T_tdd768SpecificInfo_01 */
static int hf_rrc_idleModePLMNIdentities = -1;    /* PLMNIdentitiesOfNeighbourCells */
static int hf_rrc_connectedModePLMNIdentities = -1;  /* PLMNIdentitiesOfNeighbourCells */
static int hf_rrc_v6b0NonCriticalExtensions_19 = -1;  /* T_v6b0NonCriticalExtensions_18 */
static int hf_rrc_sysInfoType18_v6b0ext = -1;     /* SysInfoType18_v6b0ext */
static int hf_rrc_v860NonCriticalExtensions_32 = -1;  /* T_v860NonCriticalExtensions_32 */
static int hf_rrc_sysInfoType18_v860ext = -1;     /* SysInfoType18_v860ext */
static int hf_rrc_nonCriticalExtensions_231 = -1;  /* T_nonCriticalExtensions_231 */
static int hf_rrc_idleModePLMNIdentitiesSIB11bis = -1;  /* PLMNIdentitiesOfNeighbourCells */
static int hf_rrc_connectedModePLMNIdentitiesSIB11bis = -1;  /* PLMNIdentitiesOfNeighbourCells */
static int hf_rrc_idleModePLMNIdentities_01 = -1;  /* PLMNIdentitiesOfNeighbourCells_v860ext */
static int hf_rrc_connectedModePLMNIdentities_01 = -1;  /* PLMNIdentitiesOfNeighbourCells_v860ext */
static int hf_rrc_utra_PriorityInfoList = -1;     /* UTRA_PriorityInfoList */
static int hf_rrc_gsm_PriorityInfoList = -1;      /* GSM_PriorityInfoList */
static int hf_rrc_eutra_FrequencyAndPriorityInfoList = -1;  /* EUTRA_FrequencyAndPriorityInfoList */
static int hf_rrc_v920NonCriticalExtensions_07 = -1;  /* T_v920NonCriticalExtensions_07 */
static int hf_rrc_sysInfoType19_v920ext = -1;     /* SysInfoType19_v920ext */
static int hf_rrc_va80NonCriticalExtensions_07 = -1;  /* T_va80NonCriticalExtensions_07 */
static int hf_rrc_sysInfoType19_va80ext = -1;     /* SysInfoType19_va80ext */
static int hf_rrc_vb30NonCriticalExtensions = -1;  /* T_vb30NonCriticalExtensions */
static int hf_rrc_sysInfoType19_vb30ext = -1;     /* SysInfoType19_vb30ext */
static int hf_rrc_vbxyNonCriticalExtensions_20 = -1;  /* T_vbxyNonCriticalExtensions_19 */
static int hf_rrc_sysInfoType19_vbxyext = -1;     /* SysInfoType19_vbxyext */
static int hf_rrc_nonCriticalExtensions_232 = -1;  /* T_nonCriticalExtensions_232 */
static int hf_rrc_utra_PriorityInfoList_v920ext = -1;  /* UTRA_PriorityInfoList_v920ext */
static int hf_rrc_eutra_FrequencyAndPriorityInfoList_v920ext = -1;  /* EUTRA_FrequencyAndPriorityInfoList_v920ext */
static int hf_rrc_multipleEutraFrequencyInfoList = -1;  /* MultipleEUTRAFrequencyInfoList */
static int hf_rrc_cell_fach_meas_ind = -1;        /* T_cell_fach_meas_ind */
static int hf_rrc_gsm_PriorityInfoList_vbxyext = -1;  /* GSM_PriorityInfoList_vbxyext */
static int hf_rrc_eutra_FrequencyAndPriorityInfoList_vbxyext = -1;  /* EUTRA_FrequencyAndPriorityInfoList_vbxyext */
static int hf_rrc_eutra_FrequencyRACHReportingInfo = -1;  /* EUTRA_FrequencyRACHReportingInfo */
static int hf_rrc_hNBName = -1;                   /* HNBName */
static int hf_rrc_nonCriticalExtensions_233 = -1;  /* T_nonCriticalExtensions_233 */
static int hf_rrc_eab_Parameters = -1;            /* EAB_Parameters */
static int hf_rrc_nonCriticalExtensions_234 = -1;  /* T_nonCriticalExtensions_234 */
static int hf_rrc_modeSpecificInfo_155 = -1;      /* T_modeSpecificInfo_155 */
static int hf_rrc_fdd_196 = -1;                   /* T_fdd_190 */
static int hf_rrc_prach_PreambleForEnhancedUplinkListType1Ext = -1;  /* PRACH_PreambleForEnhancedUplinkListType1Ext */
static int hf_rrc_prach_PreambleForEnhancedUplinkListType2Ext = -1;  /* PRACH_PreambleForEnhancedUplinkListType2Ext */
static int hf_rrc_prach_PreambleForEnhancedUplinkListType3Ext = -1;  /* PRACH_PreambleForEnhancedUplinkListType3Ext */
static int hf_rrc_concurrent_Deployment_2ms_10ms_TTI = -1;  /* Concurrent_Deployment_2ms_10ms_TTI */
static int hf_rrc_nodeB_Trigger_HS_DPCCH_Transmission = -1;  /* NodeB_Trigger_HS_DPCCH_Transmission */
static int hf_rrc_fallback_R99_PRACH_info = -1;   /* Fallback_R99_PRACH_info_IEs */
static int hf_rrc_commonEDCHResourceConfigInfoListExt = -1;  /* CommonEDCHResourceConfigInfoListExt */
static int hf_rrc_hs_dsch_DrxCellfach_SecondDrx_info = -1;  /* HS_DSCH_DrxCellfach_SecondDrx_info */
static int hf_rrc_nonCriticalExtensions_235 = -1;  /* T_nonCriticalExtensions_235 */
static int hf_rrc_sib_ReferenceList = -1;         /* SIB_ReferenceList */
static int hf_rrc_v6b0NonCriticalExtensions_20 = -1;  /* T_v6b0NonCriticalExtensions_19 */
static int hf_rrc_sysInfoTypeSB1_v6b0ext = -1;    /* SysInfoTypeSB1_v6b0ext */
static int hf_rrc_v860NonCriticalExtensions_33 = -1;  /* T_v860NonCriticalExtensions_33 */
static int hf_rrc_sysInfoTypeSB1_v860ext = -1;    /* SysInfoTypeSB1_v860ext */
static int hf_rrc_nonCriticalExtensions_236 = -1;  /* T_nonCriticalExtensions_236 */
static int hf_rrc_v6b0NonCriticalExtensions_21 = -1;  /* T_v6b0NonCriticalExtensions_20 */
static int hf_rrc_sysInfoTypeSB2_v6b0ext = -1;    /* SysInfoTypeSB2_v6b0ext */
static int hf_rrc_v860NonCriticalExtensions_34 = -1;  /* T_v860NonCriticalExtensions_34 */
static int hf_rrc_sysInfoTypeSB2_v860ext = -1;    /* SysInfoTypeSB2_v860ext */
static int hf_rrc_nonCriticalExtensions_237 = -1;  /* T_nonCriticalExtensions_237 */
static int hf_rrc_TDD_UMTS_Frequency_List_item = -1;  /* FrequencyInfoTDD */
static int hf_rrc_ue_InactivityPeriod = -1;       /* INTEGER_1_120 */
static int hf_rrc_ul_dataVolumeHistory = -1;      /* DataVolumeHistory */
static int hf_rrc_dl_dataVolumeHistory = -1;      /* DataVolumeHistory */
static int hf_rrc_none = -1;                      /* NULL */
static int hf_rrc_cellValueTag = -1;              /* CellValueTag */
static int hf_rrc_plmn_ValueTag = -1;             /* PLMN_ValueTag */
static int hf_rrc_sIBOccurrenceIdentityAndValueTag = -1;  /* SIBOccurrenceIdentityAndValueTag */
static int hf_rrc_commonRBIdentity = -1;          /* MBMS_CommonRBIdentity */
static int hf_rrc_rlc_Info_05 = -1;               /* RLC_Info_MTCH_r6 */
static int hf_rrc_MBMS_CommonRBInformationList_r6_item = -1;  /* MBMS_CommonRBInformation_r6 */
static int hf_rrc_countingForUraPCH = -1;         /* BOOLEAN */
static int hf_rrc_countingForCellPCH = -1;        /* BOOLEAN */
static int hf_rrc_countingForCellFACH = -1;       /* BOOLEAN */
static int hf_rrc_sccpchIdentity = -1;            /* MBMS_SCCPCHIdentity */
static int hf_rrc_secondaryCCPCH_Info_03 = -1;    /* MBMS_CommonPhyChIdentity */
static int hf_rrc_softComb_TimingOffset = -1;     /* MBMS_SoftComb_TimingOffset */
static int hf_rrc_transpCh_InfoCommonForAllTrCh = -1;  /* MBMS_CommonCCTrChIdentity */
static int hf_rrc_transpCHInformation = -1;       /* MBMS_TrCHInformation_CurrList */
static int hf_rrc_MBMS_CurrentCell_SCCPCHList_r6_item = -1;  /* MBMS_CurrentCell_SCCPCH_r6 */
static int hf_rrc_MBMS_FACHCarryingMTCH_List_item = -1;  /* TransportFormatSet */
static int hf_rrc_cycleOffset = -1;               /* INTEGER_0_7 */
static int hf_rrc_mtch_L1CombiningPeriodList = -1;  /* T_mtch_L1CombiningPeriodList */
static int hf_rrc_mtch_L1CombiningPeriodList_item = -1;  /* T_mtch_L1CombiningPeriodList_item */
static int hf_rrc_periodStart = -1;               /* INTEGER_0_7 */
static int hf_rrc_periodDuration = -1;            /* INTEGER_1_8 */
static int hf_rrc_cycleOffset_01 = -1;            /* INTEGER_0_15 */
static int hf_rrc_mtch_L1CombiningPeriodList_01 = -1;  /* T_mtch_L1CombiningPeriodList_01 */
static int hf_rrc_mtch_L1CombiningPeriodList_item_01 = -1;  /* T_mtch_L1CombiningPeriodList_item_01 */
static int hf_rrc_periodStart_01 = -1;            /* INTEGER_0_15 */
static int hf_rrc_periodDuration_01 = -1;         /* INTEGER_1_16 */
static int hf_rrc_cycleOffset_02 = -1;            /* INTEGER_0_31 */
static int hf_rrc_mtch_L1CombiningPeriodList_02 = -1;  /* T_mtch_L1CombiningPeriodList_02 */
static int hf_rrc_mtch_L1CombiningPeriodList_item_02 = -1;  /* T_mtch_L1CombiningPeriodList_item_02 */
static int hf_rrc_periodStart_02 = -1;            /* INTEGER_0_31 */
static int hf_rrc_periodDuration_02 = -1;         /* INTEGER_1_32 */
static int hf_rrc_cycleOffset_03 = -1;            /* INTEGER_0_63 */
static int hf_rrc_mtch_L1CombiningPeriodList_03 = -1;  /* T_mtch_L1CombiningPeriodList_03 */
static int hf_rrc_mtch_L1CombiningPeriodList_item_03 = -1;  /* T_mtch_L1CombiningPeriodList_item_03 */
static int hf_rrc_periodStart_03 = -1;            /* INTEGER_0_63 */
static int hf_rrc_periodDuration_03 = -1;         /* INTEGER_1_64 */
static int hf_rrc_cycleOffset_04 = -1;            /* INTEGER_0_127 */
static int hf_rrc_mtch_L1CombiningPeriodList_04 = -1;  /* T_mtch_L1CombiningPeriodList_04 */
static int hf_rrc_mtch_L1CombiningPeriodList_item_04 = -1;  /* T_mtch_L1CombiningPeriodList_item_04 */
static int hf_rrc_periodStart_04 = -1;            /* INTEGER_0_127 */
static int hf_rrc_periodDuration_04 = -1;         /* INTEGER_1_128 */
static int hf_rrc_cycleOffset_05 = -1;            /* INTEGER_0_255 */
static int hf_rrc_mtch_L1CombiningPeriodList_05 = -1;  /* T_mtch_L1CombiningPeriodList_05 */
static int hf_rrc_mtch_L1CombiningPeriodList_item_05 = -1;  /* T_mtch_L1CombiningPeriodList_item_05 */
static int hf_rrc_periodStart_05 = -1;            /* INTEGER_0_255 */
static int hf_rrc_periodDuration_05 = -1;         /* INTEGER_1_256 */
static int hf_rrc_cycleLength_32 = -1;            /* MBMS_L1CombiningSchedule_32 */
static int hf_rrc_cycleLength_64 = -1;            /* MBMS_L1CombiningSchedule_64 */
static int hf_rrc_cycleLength_128 = -1;           /* MBMS_L1CombiningSchedule_128 */
static int hf_rrc_cycleLength_256 = -1;           /* MBMS_L1CombiningSchedule_256 */
static int hf_rrc_cycleLength_512 = -1;           /* MBMS_L1CombiningSchedule_512 */
static int hf_rrc_cycleLength_1024 = -1;          /* MBMS_L1CombiningSchedule_1024 */
static int hf_rrc_sameAsCurrent = -1;             /* T_sameAsCurrent */
static int hf_rrc_currentCell_SCCPCH = -1;        /* MBMS_SCCPCHIdentity */
static int hf_rrc_msch_ConfigurationInfo = -1;    /* MBMS_MSCH_ConfigurationInfo_r6 */
static int hf_rrc_different = -1;                 /* T_different */
static int hf_rrc_transpCHInformation_01 = -1;    /* MBMS_TrCHInformation_NeighbList */
static int hf_rrc_accessInfoPeriodCoefficient = -1;  /* INTEGER_0_3 */
static int hf_rrc_repetitionPeriodCoefficient = -1;  /* INTEGER_0_3 */
static int hf_rrc_modificationPeriodCoefficient = -1;  /* INTEGER_7_10 */
static int hf_rrc_rlc_Info_06 = -1;               /* RLC_Info_MCCH_r6 */
static int hf_rrc_tctf_Presence = -1;             /* MBMS_TCTF_Presence */
static int hf_rrc_michPowerOffset = -1;           /* MBMS_MICHPowerOffset */
static int hf_rrc_mode_03 = -1;                   /* T_mode_03 */
static int hf_rrc_fdd_197 = -1;                   /* T_fdd_191 */
static int hf_rrc_ni_CountPerFrame = -1;          /* MBMS_NI_CountPerFrame */
static int hf_rrc_tdd384_49 = -1;                 /* T_tdd384_44 */
static int hf_rrc_repetitionPeriodLengthOffset_01 = -1;  /* RepPerLengthOffset_MICH */
static int hf_rrc_mbmsNotificationIndLength = -1;  /* MBMS_MICHNotificationIndLength */
static int hf_rrc_tdd128_73 = -1;                 /* T_tdd128_70 */
static int hf_rrc_channelisationCodeList_02 = -1;  /* SEQUENCE_SIZE_1_2_OF_DL_TS_ChannelisationCode */
static int hf_rrc_channelisationCodeList_item = -1;  /* DL_TS_ChannelisationCode */
static int hf_rrc_mode_04 = -1;                   /* T_mode_04 */
static int hf_rrc_tdd384_50 = -1;                 /* T_tdd384_45 */
static int hf_rrc_tdd768_28 = -1;                 /* T_tdd768_25 */
static int hf_rrc_midambleShiftAndBurstType_06 = -1;  /* MidambleShiftAndBurstType_VHCR */
static int hf_rrc_tdd128_74 = -1;                 /* T_tdd128_71 */
static int hf_rrc_mode_05 = -1;                   /* T_mode_05 */
static int hf_rrc_imb384 = -1;                    /* T_imb384 */
static int hf_rrc_ni_CountPerFrame_01 = -1;       /* MBMS_NI_CountPerFrame_IMB384 */
static int hf_rrc_mbms_TransmissionIdentity = -1;  /* MBMS_TransmissionIdentity */
static int hf_rrc_mbms_RequiredUEAction = -1;     /* MBMS_RequiredUEAction_Mod */
static int hf_rrc_mbms_PreferredFrequency = -1;   /* T_mbms_PreferredFrequency */
static int hf_rrc_mcch = -1;                      /* MBMS_PFLIndex */
static int hf_rrc_dcch = -1;                      /* MBMS_PFLInfo */
static int hf_rrc_dummy_61 = -1;                  /* T_dummy_16 */
static int hf_rrc_continueMCCHReading = -1;       /* BOOLEAN */
static int hf_rrc_MBMS_ModifedServiceList_r6_item = -1;  /* MBMS_ModifedService_r6 */
static int hf_rrc_mbsfnClusterFrequency = -1;     /* MBSFN_ClusterFrequency_r7 */
static int hf_rrc_rbReleaseCause = -1;            /* MBMS_PTM_RBReleaseCause_LCR_r7 */
static int hf_rrc_MBMS_ModifiedServiceList_v770ext_item = -1;  /* MBMS_ModifiedService_v770ext */
static int hf_rrc_MBMS_ModifiedServiceList_LCR_v7c0ext_item = -1;  /* MBMS_ModifiedService_LCR_v7c0ext */
static int hf_rrc_mschShedulingInfo = -1;         /* MBMS_MSCHSchedulingInfo */
static int hf_rrc_rlc_Info_07 = -1;               /* RLC_Info_MSCH_r6 */
static int hf_rrc_schedulingPeriod_32_Offset = -1;  /* INTEGER_0_31 */
static int hf_rrc_schedulingPeriod_64_Offset = -1;  /* INTEGER_0_63 */
static int hf_rrc_schedulingPeriod_128_Offset = -1;  /* INTEGER_0_127 */
static int hf_rrc_schedulingPeriod_256_Offset = -1;  /* INTEGER_0_255 */
static int hf_rrc_schedulingPeriod_512_Offset = -1;  /* INTEGER_0_511 */
static int hf_rrc_schedulingPeriod_1024_Offset = -1;  /* INTEGER_0_1023 */
static int hf_rrc_secondaryCCPCHPwrOffsetDiff = -1;  /* MBMS_SCCPCHPwrOffsetDiff */
static int hf_rrc_layer1Combining = -1;           /* T_layer1Combining */
static int hf_rrc_fdd_198 = -1;                   /* T_fdd_192 */
static int hf_rrc_mbms_L1CombiningTransmTimeDiff = -1;  /* MBMS_L1CombiningTransmTimeDiff */
static int hf_rrc_mbms_L1CombiningSchedule = -1;  /* MBMS_L1CombiningSchedule */
static int hf_rrc_mbms_L23Configuration = -1;     /* MBMS_L23Configuration */
static int hf_rrc_secondaryCCPCH_InfoDiff = -1;   /* SecondaryCCPCHInfoDiff_MBMS */
static int hf_rrc_layer1Combining_01 = -1;        /* T_layer1Combining_01 */
static int hf_rrc_fdd_199 = -1;                   /* T_fdd_193 */
static int hf_rrc_MBMS_NeighbouringCellSCCPCHList_r6_item = -1;  /* MBMS_NeighbouringCellSCCPCH_r6 */
static int hf_rrc_MBMS_NeighbouringCellSCCPCHList_v770ext_item = -1;  /* MBMS_NeighbouringCellSCCPCH_v770ext */
static int hf_rrc_networkStandardTime = -1;       /* BIT_STRING_SIZE_40 */
static int hf_rrc_correlativeSFN = -1;            /* INTEGER_0_4095 */
static int hf_rrc_mbms_CommonPhyChIdentity = -1;  /* MBMS_CommonPhyChIdentity */
static int hf_rrc_secondaryCCPCHInfo_MBMS_02 = -1;  /* SecondaryCCPCHFrameType2Info */
static int hf_rrc_MBMS_PhyChInformationList_IMB384_item = -1;  /* MBMS_PhyChInformation_IMB384 */
static int hf_rrc_MBMS_PhyChInformationList_r6_item = -1;  /* MBMS_PhyChInformation_r6 */
static int hf_rrc_MBMS_PhyChInformationList_r7_item = -1;  /* MBMS_PhyChInformation_r7 */
static int hf_rrc_mbmsPreferredFrequency = -1;    /* INTEGER_1_maxMBMS_Freq */
static int hf_rrc_layerConvergenceInformation = -1;  /* T_layerConvergenceInformation */
static int hf_rrc_mbms_Qoffset = -1;              /* MBMS_Qoffset */
static int hf_rrc_mbms_HCSoffset = -1;            /* INTEGER_0_7 */
static int hf_rrc_MBMS_PreferredFrequencyList_r6_item = -1;  /* MBMS_PreferredFrequencyInfo_r6 */
static int hf_rrc_rbInformation = -1;             /* MBMS_CommonRBIdentity */
static int hf_rrc_shortTransmissionID = -1;       /* MBMS_ShortTransmissionID */
static int hf_rrc_logicalChIdentity = -1;         /* MBMS_LogicalChIdentity */
static int hf_rrc_MBMS_PTM_RBInformation_CList_item = -1;  /* MBMS_PTM_RBInformation_C */
static int hf_rrc_layer1_CombiningStatus = -1;    /* BOOLEAN */
static int hf_rrc_MBMS_PTM_RBInformation_NList_item = -1;  /* MBMS_PTM_RBInformation_N */
static int hf_rrc_status = -1;                    /* T_status */
static int hf_rrc_some = -1;                      /* MBMS_SelectedServicesListFull */
static int hf_rrc_MBMS_SelectedServicesListFull_item = -1;  /* MBMS_ServiceIdentity_r6 */
static int hf_rrc_MBMS_SelectedServicesListShort_item = -1;  /* MBMS_ShortTransmissionID */
static int hf_rrc_mbms_SelectedServicesList = -1;  /* MBMS_SelectedServicesListShort */
static int hf_rrc_modificationPeriodIdentity = -1;  /* INTEGER_0_1 */
static int hf_rrc_accessprobabilityFactor_Idle = -1;  /* MBMS_AccessProbabilityFactor */
static int hf_rrc_accessprobabilityFactor_Connected = -1;  /* MBMS_AccessProbabilityFactor */
static int hf_rrc_mbms_ConnectedModeCountingScope = -1;  /* MBMS_ConnectedModeCountingScope */
static int hf_rrc_MBMS_ServiceAccessInfoList_r6_item = -1;  /* MBMS_ServiceAccessInfo_r6 */
static int hf_rrc_serviceIdentity = -1;           /* OCTET_STRING_SIZE_3 */
static int hf_rrc_plmn_Identity_01 = -1;          /* T_plmn_Identity */
static int hf_rrc_sameAsMIB_PLMN_Id = -1;         /* NULL */
static int hf_rrc_other_01 = -1;                  /* T_other_01 */
static int hf_rrc_sameAsMIB_MultiPLMN_Id = -1;    /* INTEGER_1_5 */
static int hf_rrc_explicitPLMN_Id = -1;           /* PLMN_Identity */
static int hf_rrc_mbms_ServiceTransmInfoList = -1;  /* MBMS_ServiceTransmInfoList */
static int hf_rrc_nextSchedulingperiod = -1;      /* INTEGER_0_31 */
static int hf_rrc_MBMS_ServiceSchedulingInfoList_r6_item = -1;  /* MBMS_ServiceSchedulingInfo_r6 */
static int hf_rrc_start = -1;                     /* INTEGER_0_255 */
static int hf_rrc_duration = -1;                  /* INTEGER_1_256 */
static int hf_rrc_MBMS_ServiceTransmInfoList_item = -1;  /* MBMS_ServiceTransmInfo */
static int hf_rrc_transpCHInformation_02 = -1;    /* MBMS_TrCHInformation_SIB5List */
static int hf_rrc_MBMS_SIBType5_SCCPCHList_r6_item = -1;  /* MBMS_SIBType5_SCCPCH_r6 */
static int hf_rrc_t_318 = -1;                     /* T_318 */
static int hf_rrc_mbms_ServiceIdentity_01 = -1;   /* MBMS_ServiceIdentity_r6 */
static int hf_rrc_commonCCTrChIdentity = -1;      /* MBMS_CommonCCTrChIdentity */
static int hf_rrc_MBMS_TranspChInfoForEachCCTrCh_r6_item = -1;  /* MBMS_TranspChInfoForCCTrCh_r6 */
static int hf_rrc_MBMS_TranspChInfoForEachTrCh_r6_item = -1;  /* MBMS_TranspChInfoForTrCh_r6 */
static int hf_rrc_commonTrChIdentity = -1;        /* MBMS_CommonTrChIdentity */
static int hf_rrc_transpCh_Info = -1;             /* MBMS_CommonTrChIdentity */
static int hf_rrc_rbInformation_01 = -1;          /* MBMS_PTM_RBInformation_CList */
static int hf_rrc_MBMS_TrCHInformation_CurrList_item = -1;  /* MBMS_TrCHInformation_Curr */
static int hf_rrc_transpCh_CombiningStatus = -1;  /* BOOLEAN */
static int hf_rrc_rbInformation_02 = -1;          /* MBMS_PTM_RBInformation_NList */
static int hf_rrc_MBMS_TrCHInformation_NeighbList_item = -1;  /* MBMS_TrCHInformation_Neighb */
static int hf_rrc_transpCh_Identity = -1;         /* INTEGER_1_maxFACHPCH */
static int hf_rrc_MBMS_TrCHInformation_SIB5List_item = -1;  /* MBMS_TrCHInformation_SIB5 */
static int hf_rrc_mbms_RequiredUEAction_01 = -1;  /* MBMS_RequiredUEAction_UMod */
static int hf_rrc_mbms_PreferredFrequency_01 = -1;  /* MBMS_PFLIndex */
static int hf_rrc_MBMS_UnmodifiedServiceList_r6_item = -1;  /* MBMS_UnmodifiedService_r6 */
static int hf_rrc_MBMS_UnmodifiedServiceList_v770ext_item = -1;  /* MBMS_UnmodifiedService_v770ext */
static int hf_rrc_imb_Indication = -1;            /* T_imb_Indication */
static int hf_rrc_MBSFNFrequencyList_item = -1;   /* MBSFNFrequency */
static int hf_rrc_MBSFNFrequencyList_v860ext_item = -1;  /* MBSFNFrequency_v860ext */
static int hf_rrc_notificationOfAllMBSFNServicesInTheBand = -1;  /* T_notificationOfAllMBSFNServicesInTheBand */
static int hf_rrc_mbsfnFrequency = -1;            /* FrequencyInfo */
static int hf_rrc_mbsfnServicesNotification = -1;  /* T_mbsfnServicesNotification */
static int hf_rrc_mbsfnServicesNotified = -1;     /* NULL */
static int hf_rrc_mbsfnServicesNotNotified = -1;  /* MBSFNservicesNotNotified_r7 */
static int hf_rrc_imb_Indication_01 = -1;         /* T_imb_Indication_01 */
static int hf_rrc_MBSFN_InterFrequencyNeighbourList_r7_item = -1;  /* MBSFNInterFrequencyNeighbour_r7 */
static int hf_rrc_MBSFN_InterFrequencyNeighbourList_v860ext_item = -1;  /* MBSFNInterFrequencyNeighbour_v860ext */
static int hf_rrc_MBSFN_TDDInformation_item = -1;  /* MBSFN_TDDTimeSlotInfo */
static int hf_rrc_MBSFN_TDDInformation_LCR_item = -1;  /* MBSFN_TDDTimeSlotInfo_LCR */
static int hf_rrc_timeSlotNumber_01 = -1;         /* TimeslotNumber_LCR_r4 */
static int hf_rrc_frequencyIndex = -1;            /* INTEGER_1_maxMBSFNClusters */
static int hf_rrc_timeSlotList = -1;              /* MBSFN_TDDInformation */
static int hf_rrc_tDMPeriod = -1;                 /* INTEGER_2_9 */
static int hf_rrc_tDMOffset = -1;                 /* INTEGER_0_8 */
static int hf_rrc_tDMLength = -1;                 /* INTEGER_1_8 */
static int hf_rrc_MBSFN_TDM_Info_List_item = -1;  /* MBSFN_TDM_Info */
static int hf_rrc_interRATHandoverInfo = -1;      /* InterRATHandoverInfoWithInterRATCapabilities_r3 */
static int hf_rrc_srncRelocation = -1;            /* SRNC_RelocationInfo_r3 */
static int hf_rrc_rfc3095_ContextInfo = -1;       /* RFC3095_ContextInfo_r5 */
static int hf_rrc_extension = -1;                 /* NULL */
static int hf_rrc_rrc_FailureInfo = -1;           /* RRC_FailureInfo */
static int hf_rrc_dL_DCCHmessage = -1;            /* T_dL_DCCHmessage */
static int hf_rrc_r3_31 = -1;                     /* T_r3_31 */
static int hf_rrc_interRATHandoverInfo_r3 = -1;   /* InterRATHandoverInfoWithInterRATCapabilities_r3_IEs */
static int hf_rrc_v390NonCriticalExtensions_01 = -1;  /* T_v390NonCriticalExtensions_01 */
static int hf_rrc_interRATHandoverInfoWithInterRATCapabilities_v390ext = -1;  /* InterRATHandoverInfoWithInterRATCapabilities_v390ext_IEs */
static int hf_rrc_v690NonCriticalExtensions_52 = -1;  /* T_v690NonCriticalExtensions_52 */
static int hf_rrc_interRATHandoverInfoWithInterRATCapabilities_v690ext = -1;  /* InterRATHandoverInfoWithInterRATCapabilities_v690ext_IEs */
static int hf_rrc_v860NonCriticalExtensions_35 = -1;  /* T_v860NonCriticalExtensions_35 */
static int hf_rrc_interRATHandoverInfoWithInterRATCapabilities_v860ext = -1;  /* InterRATHandoverInfoWithInterRATCapabilities_v860ext_IEs */
static int hf_rrc_v920NonCriticalExtensions_08 = -1;  /* T_v920NonCriticalExtensions_08 */
static int hf_rrc_interRATHandoverInfoWithInterRATCapabilities_v920ext = -1;  /* InterRATHandoverInfoWithInterRATCapabilities_v920ext_IEs */
static int hf_rrc_nonCriticalExtensions_238 = -1;  /* T_nonCriticalExtensions_238 */
static int hf_rrc_criticalExtensions_140 = -1;    /* T_criticalExtensions_139 */
static int hf_rrc_interRATHandoverInfo_01 = -1;   /* T_interRATHandoverInfo */
static int hf_rrc_ue_RATSpecificCapability_01 = -1;  /* InterRAT_UE_RadioAccessCapability_v860ext */
static int hf_rrc_ue_Inactivity_Period = -1;      /* INTEGER_1_120 */
static int hf_rrc_r5_14 = -1;                     /* T_r5_14 */
static int hf_rrc_rFC3095_ContextInfoList_r5 = -1;  /* RFC3095_ContextInfoList_r5 */
static int hf_rrc_nonCriticalExtensions_239 = -1;  /* T_nonCriticalExtensions_239 */
static int hf_rrc_criticalExtensions_141 = -1;    /* T_criticalExtensions_140 */
static int hf_rrc_RFC3095_ContextInfoList_r5_item = -1;  /* RFC3095_ContextInfo */
static int hf_rrc_r3_32 = -1;                     /* T_r3_32 */
static int hf_rrc_sRNC_RelocationInfo_r3 = -1;    /* SRNC_RelocationInfo_r3_IEs */
static int hf_rrc_v380NonCriticalExtensions_02 = -1;  /* T_v380NonCriticalExtensions_02 */
static int hf_rrc_sRNC_RelocationInfo_v380ext = -1;  /* SRNC_RelocationInfo_v380ext_IEs */
static int hf_rrc_v390NonCriticalExtensions_02 = -1;  /* T_v390NonCriticalExtensions_02 */
static int hf_rrc_sRNC_RelocationInfo_v390ext = -1;  /* SRNC_RelocationInfo_v390ext_IEs */
static int hf_rrc_v3a0NonCriticalExtensions_16 = -1;  /* T_v3a0NonCriticalExtensions_16 */
static int hf_rrc_sRNC_RelocationInfo_v3a0ext = -1;  /* SRNC_RelocationInfo_v3a0ext_IEs */
static int hf_rrc_v3b0NonCriticalExtensions = -1;  /* T_v3b0NonCriticalExtensions */
static int hf_rrc_sRNC_RelocationInfo_v3b0ext = -1;  /* SRNC_RelocationInfo_v3b0ext_IEs */
static int hf_rrc_v3c0NonCriticalExtensions = -1;  /* T_v3c0NonCriticalExtensions */
static int hf_rrc_sRNC_RelocationInfo_v3c0ext = -1;  /* SRNC_RelocationInfo_v3c0ext_IEs */
static int hf_rrc_laterNonCriticalExtensions_68 = -1;  /* T_laterNonCriticalExtensions_67 */
static int hf_rrc_sRNC_RelocationInfo_v3d0ext = -1;  /* SRNC_RelocationInfo_v3d0ext_IEs */
static int hf_rrc_sRNC_RelocationInfo_r3_add_ext = -1;  /* T_sRNC_RelocationInfo_r3_add_ext */
static int hf_rrc_v3g0NonCriticalExtensions_02 = -1;  /* T_v3g0NonCriticalExtensions_02 */
static int hf_rrc_sRNC_RelocationInfo_v3g0ext = -1;  /* SRNC_RelocationInfo_v3g0ext_IEs */
static int hf_rrc_v4b0NonCriticalExtensions_27 = -1;  /* T_v4b0NonCriticalExtensions_27 */
static int hf_rrc_sRNC_RelocationInfo_v4b0ext = -1;  /* SRNC_RelocationInfo_v4b0ext_IEs */
static int hf_rrc_v590NonCriticalExtensions_31 = -1;  /* T_v590NonCriticalExtensions_31 */
static int hf_rrc_sRNC_RelocationInfo_v590ext = -1;  /* SRNC_RelocationInfo_v590ext_IEs */
static int hf_rrc_v5a0NonCriticalExtensions = -1;  /* T_v5a0NonCriticalExtensions */
static int hf_rrc_sRNC_RelocationInfo_v5a0ext = -1;  /* SRNC_RelocationInfo_v5a0ext_IEs */
static int hf_rrc_v5b0NonCriticalExtensions_03 = -1;  /* T_v5b0NonCriticalExtensions_03 */
static int hf_rrc_sRNC_RelocationInfo_v5b0ext = -1;  /* SRNC_RelocationInfo_v5b0ext_IEs */
static int hf_rrc_v5c0NonCriticalExtensions_02 = -1;  /* T_v5c0NonCriticalExtensions_02 */
static int hf_rrc_sRNC_RelocationInfo_v5c0ext = -1;  /* SRNC_RelocationInfo_v5c0ext_IEs */
static int hf_rrc_v690NonCriticalExtensions_53 = -1;  /* T_v690NonCriticalExtensions_53 */
static int hf_rrc_sRNC_RelocationInfo_v690ext = -1;  /* SRNC_RelocationInfo_v690ext_IEs */
static int hf_rrc_v6b0NonCriticalExtensions_22 = -1;  /* T_v6b0NonCriticalExtensions_21 */
static int hf_rrc_sRNC_RelocationInfo_v6b0ext = -1;  /* SRNC_RelocationInfo_v6b0ext_IEs */
static int hf_rrc_v770NonCriticalExtensions_48 = -1;  /* T_v770NonCriticalExtensions_48 */
static int hf_rrc_sRNC_RelocationInfo_v770ext = -1;  /* SRNC_RelocationInfo_v770ext_IEs */
static int hf_rrc_v7e0NonCriticalExtensions_04 = -1;  /* T_v7e0NonCriticalExtensions_04 */
static int hf_rrc_sRNC_RelocationInfo_v7e0ext = -1;  /* SRNC_RelocationInfo_v7e0ext_IEs */
static int hf_rrc_v7f0NonCriticalExtensions_17 = -1;  /* T_v7f0NonCriticalExtensions_17 */
static int hf_rrc_sRNC_RelocationInfo_v7f0ext = -1;  /* SRNC_RelocationInfo_v7f0ext_IEs */
static int hf_rrc_nonCriticalExtensions_240 = -1;  /* T_nonCriticalExtensions_240 */
static int hf_rrc_later_than_r3_28 = -1;          /* T_later_than_r3_28 */
static int hf_rrc_r4_14 = -1;                     /* T_r4_14 */
static int hf_rrc_sRNC_RelocationInfo_r4 = -1;    /* SRNC_RelocationInfo_r4_IEs */
static int hf_rrc_v4d0NonCriticalExtensions_14 = -1;  /* T_v4d0NonCriticalExtensions_14 */
static int hf_rrc_sRNC_RelocationInfo_v4d0ext = -1;  /* SRNC_RelocationInfo_v4d0ext_IEs */
static int hf_rrc_sRNC_RelocationInfo_r4_add_ext = -1;  /* BIT_STRING */
static int hf_rrc_v590NonCriticalExtensions_32 = -1;  /* T_v590NonCriticalExtensions_32 */
static int hf_rrc_v5a0NonCriticalExtensions_01 = -1;  /* T_v5a0NonCriticalExtensions_01 */
static int hf_rrc_v5b0NonCriticalExtensions_04 = -1;  /* T_v5b0NonCriticalExtensions_04 */
static int hf_rrc_v5c0NonCriticalExtensions_03 = -1;  /* T_v5c0NonCriticalExtensions_03 */
static int hf_rrc_v690NonCriticalExtensions_54 = -1;  /* T_v690NonCriticalExtensions_54 */
static int hf_rrc_v6b0NonCriticalExtensions_23 = -1;  /* T_v6b0NonCriticalExtensions_22 */
static int hf_rrc_v770NonCriticalExtensions_49 = -1;  /* T_v770NonCriticalExtensions_49 */
static int hf_rrc_v7e0NonCriticalExtensions_05 = -1;  /* T_v7e0NonCriticalExtensions_05 */
static int hf_rrc_v7f0NonCriticalExtensions_18 = -1;  /* T_v7f0NonCriticalExtensions_18 */
static int hf_rrc_nonCriticalExtensions_241 = -1;  /* T_nonCriticalExtensions_241 */
static int hf_rrc_criticalExtensions_142 = -1;    /* T_criticalExtensions_141 */
static int hf_rrc_r5_15 = -1;                     /* T_r5_15 */
static int hf_rrc_sRNC_RelocationInfo_r5 = -1;    /* SRNC_RelocationInfo_r5_IEs */
static int hf_rrc_sRNC_RelocationInfo_r5_add_ext = -1;  /* BIT_STRING */
static int hf_rrc_v5a0NonCriticalExtensions_02 = -1;  /* T_v5a0NonCriticalExtensions_02 */
static int hf_rrc_v5b0NonCriticalExtensions_05 = -1;  /* T_v5b0NonCriticalExtensions_05 */
static int hf_rrc_v5c0NonCriticalExtensions_04 = -1;  /* T_v5c0NonCriticalExtensions_04 */
static int hf_rrc_v690NonCriticalExtensions_55 = -1;  /* T_v690NonCriticalExtensions_55 */
static int hf_rrc_v6b0NonCriticalExtensions_24 = -1;  /* T_v6b0NonCriticalExtensions_23 */
static int hf_rrc_v770NonCriticalExtensions_50 = -1;  /* T_v770NonCriticalExtensions_50 */
static int hf_rrc_v7e0NonCriticalExtensions_06 = -1;  /* T_v7e0NonCriticalExtensions_06 */
static int hf_rrc_v7f0NonCriticalExtensions_19 = -1;  /* T_v7f0NonCriticalExtensions_19 */
static int hf_rrc_nonCriticalExtensions_242 = -1;  /* T_nonCriticalExtensions_242 */
static int hf_rrc_criticalExtensions_143 = -1;    /* T_criticalExtensions_142 */
static int hf_rrc_r6_12 = -1;                     /* T_r6_12 */
static int hf_rrc_sRNC_RelocationInfo_r6 = -1;    /* SRNC_RelocationInfo_r6_IEs */
static int hf_rrc_sRNC_RelocationInfo_r6_add_ext = -1;  /* T_sRNC_RelocationInfo_r6_add_ext */
static int hf_rrc_v6b0NonCriticalExtensions_25 = -1;  /* T_v6b0NonCriticalExtensions_24 */
static int hf_rrc_v770NonCriticalExtensions_51 = -1;  /* T_v770NonCriticalExtensions_51 */
static int hf_rrc_v820NonCriticalExtensions_03 = -1;  /* T_v820NonCriticalExtensions_03 */
static int hf_rrc_srnc_RelocationInfo_v820ext = -1;  /* SRNC_RelocationInfo_v820ext_IEs */
static int hf_rrc_v860NonCriticalExtensions_36 = -1;  /* T_v860NonCriticalExtensions_36 */
static int hf_rrc_sRNC_RelocationInfo_v860ext = -1;  /* SRNC_RelocationInfo_v860ext_IEs */
static int hf_rrc_nonCriticalExtensions_243 = -1;  /* T_nonCriticalExtensions_243 */
static int hf_rrc_criticalExtensions_144 = -1;    /* T_criticalExtensions_143 */
static int hf_rrc_r7_16 = -1;                     /* T_r7_16 */
static int hf_rrc_sRNC_RelocationInfo_r7 = -1;    /* SRNC_RelocationInfo_r7_IEs */
static int hf_rrc_sRNC_RelocationInfo_r7_add_ext = -1;  /* T_sRNC_RelocationInfo_r7_add_ext */
static int hf_rrc_v820NonCriticalExtensions_04 = -1;  /* T_v820NonCriticalExtensions_04 */
static int hf_rrc_v860NonCriticalExtensions_37 = -1;  /* T_v860NonCriticalExtensions_37 */
static int hf_rrc_nonCriticalExtensions_244 = -1;  /* T_nonCriticalExtensions_244 */
static int hf_rrc_criticalExtensions_145 = -1;    /* T_criticalExtensions_144 */
static int hf_rrc_r8_12 = -1;                     /* T_r8_12 */
static int hf_rrc_sRNC_RelocationInfo_r8 = -1;    /* SRNC_RelocationInfo_r8_IEs */
static int hf_rrc_sRNC_RelocationInfo_r8_add_ext = -1;  /* BIT_STRING */
static int hf_rrc_v7e0NonCriticalExtensions_07 = -1;  /* T_v7e0NonCriticalExtensions_07 */
static int hf_rrc_v7f0NonCriticalExtensions_20 = -1;  /* T_v7f0NonCriticalExtensions_20 */
static int hf_rrc_v8d0NonCriticalExtensions_01 = -1;  /* T_v8d0NonCriticalExtensions_01 */
static int hf_rrc_sRNC_RelocationInfo_v8d0ext = -1;  /* SRNC_RelocationInfo_v8d0ext_IEs */
static int hf_rrc_nonCriticalExtensions_245 = -1;  /* T_nonCriticalExtensions_245 */
static int hf_rrc_criticalExtensions_146 = -1;    /* T_criticalExtensions_145 */
static int hf_rrc_r9_11 = -1;                     /* T_r9_11 */
static int hf_rrc_sRNC_RelocationInfo_r9 = -1;    /* SRNC_RelocationInfo_r9_IEs */
static int hf_rrc_sRNC_RelocationInfo_r9_add_ext = -1;  /* BIT_STRING */
static int hf_rrc_v8d0NonCriticalExtensions_02 = -1;  /* T_v8d0NonCriticalExtensions_02 */
static int hf_rrc_v970NonCriticalExtensions_04 = -1;  /* T_v970NonCriticalExtensions_04 */
static int hf_rrc_sRNC_RelocationInfo_v970ext = -1;  /* SRNC_RelocationInfo_v970ext_IEs */
static int hf_rrc_v9c0NonCriticalExtensions_13 = -1;  /* T_v9c0NonCriticalExtensions_13 */
static int hf_rrc_sRNC_RelocationInfo_v9c0ext = -1;  /* SRNC_RelocationInfo_v9c0ext_IEs */
static int hf_rrc_nonCriticalExtensions_246 = -1;  /* T_nonCriticalExtensions_246 */
static int hf_rrc_criticalExtensions_147 = -1;    /* T_criticalExtensions_146 */
static int hf_rrc_r10_13 = -1;                    /* T_r10_13 */
static int hf_rrc_sRNC_RelocationInfo_r10 = -1;   /* SRNC_RelocationInfo_r10_IEs */
static int hf_rrc_sRNC_RelocationInfo_r10_add_ext = -1;  /* BIT_STRING */
static int hf_rrc_nonCriticalExtensions_247 = -1;  /* T_nonCriticalExtensions_247 */
static int hf_rrc_criticalExtensions_148 = -1;    /* T_criticalExtensions_147 */
static int hf_rrc_r11_14 = -1;                    /* T_r11_14 */
static int hf_rrc_sRNC_RelocationInfo_r11 = -1;   /* SRNC_RelocationInfo_r11_IEs */
static int hf_rrc_sRNC_RelocationInfo_r11_add_ext = -1;  /* BIT_STRING */
static int hf_rrc_v9c0NonCriticalExtensions_14 = -1;  /* T_v9c0NonCriticalExtensions_14 */
static int hf_rrc_nonCriticalExtensions_248 = -1;  /* T_nonCriticalExtensions_248 */
static int hf_rrc_criticalExtensions_149 = -1;    /* T_criticalExtensions_148 */
static int hf_rrc_stateOfRRC = -1;                /* StateOfRRC */
static int hf_rrc_stateOfRRC_Procedure = -1;      /* StateOfRRC_Procedure */
static int hf_rrc_cipheringStatus = -1;           /* CipheringStatus */
static int hf_rrc_calculationTimeForCiphering = -1;  /* CalculationTimeForCiphering */
static int hf_rrc_cipheringInfoPerRB_List = -1;   /* CipheringInfoPerRB_List */
static int hf_rrc_count_C_List = -1;              /* COUNT_C_List */
static int hf_rrc_integrityProtectionStatus = -1;  /* IntegrityProtectionStatus */
static int hf_rrc_srb_SpecificIntegrityProtInfo = -1;  /* SRB_SpecificIntegrityProtInfoList */
static int hf_rrc_implementationSpecificParams = -1;  /* ImplementationSpecificParams */
static int hf_rrc_c_RNTI = -1;                    /* C_RNTI */
static int hf_rrc_ue_Positioning_LastKnownPos = -1;  /* UE_Positioning_LastKnownPos */
static int hf_rrc_cn_CommonGSM_MAP_NAS_SysInfo_04 = -1;  /* NAS_SystemInformationGSM_MAP */
static int hf_rrc_ongoingMeasRepList = -1;        /* OngoingMeasRepList */
static int hf_rrc_predefinedConfigStatusList_01 = -1;  /* PredefinedConfigStatusList */
static int hf_rrc_rab_InformationList_01 = -1;    /* RAB_InformationSetupList */
static int hf_rrc_ul_TransChInfoList = -1;        /* UL_AddReconfTransChInfoList */
static int hf_rrc_modeSpecificInfo_156 = -1;      /* T_modeSpecificInfo_156 */
static int hf_rrc_fdd_200 = -1;                   /* T_fdd_194 */
static int hf_rrc_dl_TransChInfoList = -1;        /* DL_AddReconfTransChInfoList */
static int hf_rrc_cipheringStatusList = -1;       /* CipheringStatusList */
static int hf_rrc_cn_DomainInformationList_v390ext = -1;  /* CN_DomainInformationList_v390ext */
static int hf_rrc_cipheringInfoForSRB1_v3a0ext = -1;  /* CipheringInfoPerRB_List_v3a0ext */
static int hf_rrc_startValueForCiphering_v3a0ext = -1;  /* START_Value */
static int hf_rrc_startValueForCiphering_v3b0ext = -1;  /* STARTList2 */
static int hf_rrc_rb_IdentityForHOMessage = -1;   /* RB_Identity */
static int hf_rrc_tpc_CombinationInfoList = -1;   /* TPC_CombinationInfoList */
static int hf_rrc_nonCriticalExtension = -1;      /* T_nonCriticalExtension */
static int hf_rrc_TPC_CombinationInfoList_item = -1;  /* TPC_Combination_Info */
static int hf_rrc_TPC_CombinationInfoList_r9_item = -1;  /* TPC_Combination_Info_r9 */
static int hf_rrc_STARTList2_item = -1;           /* STARTSingle */
static int hf_rrc_storedCompressedModeInfo = -1;  /* StoredCompressedModeInfo */
static int hf_rrc_dl_UM_SN = -1;                  /* BIT_STRING_SIZE_7 */
static int hf_rrc_CipheringStatusList_item = -1;  /* CipheringStatusCNdomain */
static int hf_rrc_CodeChangeStatusList_item = -1;  /* CodeChangeStatus */
static int hf_rrc_storedTGP_SequenceList = -1;    /* StoredTGP_SequenceList */
static int hf_rrc_codeChangeStatusList = -1;      /* CodeChangeStatusList */
static int hf_rrc_storedTGP_SequenceList_01 = -1;  /* StoredTGP_SequenceList_r8 */
static int hf_rrc_StoredTGP_SequenceList_item = -1;  /* StoredTGP_Sequence */
static int hf_rrc_StoredTGP_SequenceList_r8_item = -1;  /* StoredTGP_Sequence_r8 */
static int hf_rrc_current_tgps_Status = -1;       /* T_current_tgps_Status */
static int hf_rrc_active = -1;                    /* T_active */
static int hf_rrc_inactive = -1;                  /* NULL */
static int hf_rrc_current_tgps_Status_01 = -1;    /* T_current_tgps_Status_01 */
static int hf_rrc_active_01 = -1;                 /* T_active_01 */
static int hf_rrc_cipheringStatusList_01 = -1;    /* CipheringStatusList_r4 */
static int hf_rrc_latestConfiguredCN_Domain = -1;  /* CN_DomainIdentity */
static int hf_rrc_cipheringInfoPerRB_List_01 = -1;  /* CipheringInfoPerRB_List_r4 */
static int hf_rrc_ue_RadioAccessCapability_03 = -1;  /* UE_RadioAccessCapability_r4 */
static int hf_rrc_ue_RadioAccessCapability_ext = -1;  /* UE_RadioAccessCapabBandFDDList */
static int hf_rrc_cn_DomainInformationList_01 = -1;  /* CN_DomainInformationListFull */
static int hf_rrc_ongoingMeasRepList_01 = -1;     /* OngoingMeasRepList_r4 */
static int hf_rrc_rab_InformationList_02 = -1;    /* RAB_InformationSetupList_r4 */
static int hf_rrc_dummy_62 = -1;                  /* T_dummy_17 */
static int hf_rrc_fdd_201 = -1;                   /* T_fdd_195 */
static int hf_rrc_dl_TransChInfoList_01 = -1;     /* DL_AddReconfTransChInfoList_r4 */
static int hf_rrc_ue_RadioAccessCapability_04 = -1;  /* UE_RadioAccessCapability_r5 */
static int hf_rrc_ue_RATSpecificCapability_02 = -1;  /* InterRAT_UE_RadioAccessCapabilityList_r5 */
static int hf_rrc_ongoingMeasRepList_02 = -1;     /* OngoingMeasRepList_r5 */
static int hf_rrc_srb_InformationList_01 = -1;    /* SRB_InformationSetupList_r5 */
static int hf_rrc_rab_InformationList_03 = -1;    /* RAB_InformationSetupList_r5 */
static int hf_rrc_dummy_63 = -1;                  /* T_dummy_18 */
static int hf_rrc_fdd_202 = -1;                   /* T_fdd_196 */
static int hf_rrc_dl_TransChInfoList_02 = -1;     /* DL_AddReconfTransChInfoList_r5 */
static int hf_rrc_ueCapabilityContainer_02 = -1;  /* T_ueCapabilityContainer_02 */
static int hf_rrc_ueCapabilityContainer_RSC = -1;  /* T_ueCapabilityContainer_RSC */
static int hf_rrc_ueCapabilityContainer_UCI = -1;  /* T_ueCapabilityContainer_UCI */
static int hf_rrc_intraFreqReportingCriteria_08 = -1;  /* IntraFreqReportingCriteria_r6_ext */
static int hf_rrc_ue_RadioAccessCapability_05 = -1;  /* UE_RadioAccessCapability_r6 */
static int hf_rrc_ueCapabilityContainer_03 = -1;  /* T_ueCapabilityContainer_03 */
static int hf_rrc_ueCapabilityContainer_RSC_01 = -1;  /* T_ueCapabilityContainer_RSC_01 */
static int hf_rrc_ueCapabilityContainer_UCI_01 = -1;  /* T_ueCapabilityContainer_UCI_01 */
static int hf_rrc_ongoingMeasRepList_03 = -1;     /* OngoingMeasRepList_r6 */
static int hf_rrc_srb_InformationList_02 = -1;    /* SRB_InformationSetupList_r6 */
static int hf_rrc_rab_InformationList_04 = -1;    /* RAB_InformationSetupList_r6 */
static int hf_rrc_ul_TransChInfoList_01 = -1;     /* UL_AddReconfTransChInfoList_r6 */
static int hf_rrc_measurementReport_01 = -1;      /* T_measurementReport */
static int hf_rrc_v7f0NonCriticalExtensions_21 = -1;  /* T_v7f0NonCriticalExtensions_21 */
static int hf_rrc_nonCriticalExtensions_249 = -1;  /* T_nonCriticalExtensions_249 */
static int hf_rrc_ueCapabilityContainer_04 = -1;  /* T_ueCapabilityContainer_04 */
static int hf_rrc_ueCapabilityContainer_RSC_02 = -1;  /* T_ueCapabilityContainer_RSC_02 */
static int hf_rrc_ueCapabilityContainer_UCI_02 = -1;  /* T_ueCapabilityContainer_UCI_02 */
static int hf_rrc_srns_t_305 = -1;                /* T_305 */
static int hf_rrc_ongoingMeasRepList_04 = -1;     /* OngoingMeasRepList_r7 */
static int hf_rrc_srb_InformationList_03 = -1;    /* SRB_InformationSetupList_r7 */
static int hf_rrc_rab_InformationList_05 = -1;    /* RAB_InformationSetupList_r7 */
static int hf_rrc_ul_TransChInfoList_02 = -1;     /* UL_AddReconfTransChInfoList_r7 */
static int hf_rrc_dl_TransChInfoList_03 = -1;     /* DL_AddReconfTransChInfoList_r7 */
static int hf_rrc_measurementReport_02 = -1;      /* T_measurementReport_01 */
static int hf_rrc_v7f0NonCriticalExtensions_22 = -1;  /* T_v7f0NonCriticalExtensions_22 */
static int hf_rrc_nonCriticalExtensions_250 = -1;  /* T_nonCriticalExtensions_250 */
static int hf_rrc_thresholdSFN_GPS_TOW_us = -1;   /* ThresholdSFN_GPS_TOW_us */
static int hf_rrc_ueCapabilityContainer_05 = -1;  /* T_ueCapabilityContainer_05 */
static int hf_rrc_ueCapabilityContainer_RSC_03 = -1;  /* T_ueCapabilityContainer_RSC_03 */
static int hf_rrc_ueCapabilityContainer_UCI_03 = -1;  /* T_ueCapabilityContainer_UCI_03 */
static int hf_rrc_ongoingMeasRepList_05 = -1;     /* OngoingMeasRepList_r8 */
static int hf_rrc_srb_InformationList_04 = -1;    /* SRB_InformationSetupList_r8 */
static int hf_rrc_rab_InformationList_06 = -1;    /* RAB_InformationSetupList_r8 */
static int hf_rrc_ul_TransChInfoList_03 = -1;     /* UL_AddReconfTransChInfoList_r8 */
static int hf_rrc_e_RGCH_CombinationInfoList = -1;  /* E_RGCH_CombinationInfoList */
static int hf_rrc_storedCompressedModeInfo_01 = -1;  /* StoredCompressedModeInfo_r8 */
static int hf_rrc_measurementReport_03 = -1;      /* T_measurementReport_02 */
static int hf_rrc_ue_HistoryInformation = -1;     /* UE_HistoryInformation */
static int hf_rrc_rab_InformationList_07 = -1;    /* RAB_InformationSetupList_v820ext */
static int hf_rrc_ueCapabilityContainer_06 = -1;  /* T_ueCapabilityContainer_06 */
static int hf_rrc_ueCapabilityContainer_RSC_04 = -1;  /* T_ueCapabilityContainer_RSC_04 */
static int hf_rrc_ueCapabilityContainer_UCI_04 = -1;  /* T_ueCapabilityContainer_UCI_04 */
static int hf_rrc_ongoingMeasRepList_06 = -1;     /* OngoingMeasRepList_r9 */
static int hf_rrc_dl_TransChInfoList_04 = -1;     /* DL_AddReconfTransChInfoList_r9 */
static int hf_rrc_secondary_tpc_CombinationInfoList = -1;  /* TPC_CombinationInfoList_r9 */
static int hf_rrc_secondary_e_RGCH_CombinationInfoList = -1;  /* E_RGCH_CombinationInfoList_r9 */
static int hf_rrc_measurementReport_04 = -1;      /* T_measurementReport_03 */
static int hf_rrc_ongoingMeasRepList_07 = -1;     /* OngoingMeasRepList_v970ext_IE */
static int hf_rrc_ueCapabilityContainer_07 = -1;  /* T_ueCapabilityContainer_07 */
static int hf_rrc_ueCapabilityContainer_RSC_05 = -1;  /* T_ueCapabilityContainer_RSC_05 */
static int hf_rrc_ueCapabilityContainer_UCI_05 = -1;  /* T_ueCapabilityContainer_UCI_05 */
static int hf_rrc_ongoingMeasRepList_08 = -1;     /* OngoingMeasRepList_r10 */
static int hf_rrc_measurementReport_05 = -1;      /* T_measurementReport_04 */
static int hf_rrc_ueCapabilityContainer_08 = -1;  /* T_ueCapabilityContainer_08 */
static int hf_rrc_ueCapabilityContainer_RSC_06 = -1;  /* T_ueCapabilityContainer_RSC_06 */
static int hf_rrc_ueCapabilityContainer_UCI_06 = -1;  /* T_ueCapabilityContainer_UCI_06 */
static int hf_rrc_ongoingMeasRepList_09 = -1;     /* OngoingMeasRepList_r11 */
static int hf_rrc_rab_InformationList_08 = -1;    /* RAB_InformationSetupList_r11 */
static int hf_rrc_dl_TransChInfoList_05 = -1;     /* DL_AddReconfTransChInfoList_r11 */
static int hf_rrc_measurementReport_06 = -1;      /* T_measurementReport_05 */
static int hf_rrc_dl_HFN = -1;                    /* BIT_STRING_SIZE_20_25 */
static int hf_rrc_ul_HFN = -1;                    /* BIT_STRING_SIZE_20_25 */
static int hf_rrc_CipheringInfoPerRB_List_item = -1;  /* CipheringInfoPerRB */
static int hf_rrc_CipheringInfoPerRB_List_r4_item = -1;  /* CipheringInfoPerRB_r4 */
static int hf_rrc_CipheringStatusList_r4_item = -1;  /* CipheringStatusCNdomain_r4 */
static int hf_rrc_CN_DomainInformationList_v390ext_item = -1;  /* CN_DomainInformation_v390ext */
static int hf_rrc_tdd384_Measurements = -1;       /* BOOLEAN */
static int hf_rrc_COUNT_C_List_item = -1;         /* COUNT_CSingle */
static int hf_rrc_count_C = -1;                   /* BIT_STRING_SIZE_32 */
static int hf_rrc_dummy3_05 = -1;                 /* SupportOfDedicatedPilotsForChEstimation */
static int hf_rrc_fdd_hspdsch_01 = -1;            /* T_fdd_hspdsch_01 */
static int hf_rrc_supported_14 = -1;              /* T_supported_07 */
static int hf_rrc_tdd384_hspdsch_01 = -1;         /* T_tdd384_hspdsch_01 */
static int hf_rrc_tdd128_hspdsch_01 = -1;         /* T_tdd128_hspdsch_01 */
static int hf_rrc_rfc3095_Context_Identity = -1;  /* INTEGER_0_16383 */
static int hf_rrc_dl_mode = -1;                   /* T_dl_mode */
static int hf_rrc_dl_ref_ir = -1;                 /* OCTET_STRING_SIZE_1_3000 */
static int hf_rrc_dl_ref_time = -1;               /* INTEGER_0_4294967295 */
static int hf_rrc_dl_curr_time = -1;              /* INTEGER_0_4294967295 */
static int hf_rrc_dl_syn_offset_id = -1;          /* INTEGER_0_65535 */
static int hf_rrc_dl_syn_slope_ts = -1;           /* INTEGER_0_4294967295 */
static int hf_rrc_dl_dyn_changed = -1;            /* BOOLEAN */
static int hf_rrc_rg_CombinationIndex_01 = -1;    /* INTEGER_0_5 */
static int hf_rrc_E_RGCH_CombinationInfoList_item = -1;  /* E_RGCH_Combination_Info */
static int hf_rrc_E_RGCH_CombinationInfoList_r9_item = -1;  /* E_RGCH_Combination_Info_r9 */
static int hf_rrc_interRAT_UE_RadioAccessCapability = -1;  /* InterRAT_UE_RadioAccessCapabilityList */
static int hf_rrc_event_06 = -1;                  /* Event1j_r6 */
static int hf_rrc_downlinkCompressedMode_04 = -1;  /* CompressedModeMeasCapability_r4 */
static int hf_rrc_uplinkCompressedMode_01 = -1;   /* CompressedModeMeasCapability_r4 */
static int hf_rrc_modify_09 = -1;                 /* NULL */
static int hf_rrc_measurementCommandWithType = -1;  /* MeasurementCommandWithType */
static int hf_rrc_additionalMeasurementID_List = -1;  /* AdditionalMeasurementID_List */
static int hf_rrc_measurementCommandWithType_01 = -1;  /* MeasurementCommandWithType_r4 */
static int hf_rrc_measurementCommand_v590ext_01 = -1;  /* T_measurementCommand_v590ext_01 */
static int hf_rrc_measurementCommandWithType_02 = -1;  /* MeasurementCommandWithType_r6 */
static int hf_rrc_measurementCommandWithType_03 = -1;  /* MeasurementCommandWithType_r7 */
static int hf_rrc_measurementCommandWithType_04 = -1;  /* MeasurementCommandWithType_r8 */
static int hf_rrc_measurementCommandWithType_05 = -1;  /* MeasurementCommandWithType_r9 */
static int hf_rrc_additionalMeasurementID_List_01 = -1;  /* AdditionalMeasurementID_List_r9 */
static int hf_rrc_measurementCommandWithType_06 = -1;  /* MeasurementCommandWithType_r10 */
static int hf_rrc_measurementCommandWithType_07 = -1;  /* MeasurementCommandWithType_r11 */
static int hf_rrc_OngoingMeasRepList_item = -1;   /* OngoingMeasRep */
static int hf_rrc_OngoingMeasRepList_r4_item = -1;  /* OngoingMeasRep_r4 */
static int hf_rrc_OngoingMeasRepList_r5_item = -1;  /* OngoingMeasRep_r5 */
static int hf_rrc_OngoingMeasRepList_r6_item = -1;  /* OngoingMeasRep_r6 */
static int hf_rrc_OngoingMeasRepList_r7_item = -1;  /* OngoingMeasRep_r7 */
static int hf_rrc_OngoingMeasRepList_r8_item = -1;  /* OngoingMeasRep_r8 */
static int hf_rrc_OngoingMeasRepList_r9_item = -1;  /* OngoingMeasRep_r9 */
static int hf_rrc_OngoingMeasRepList_v970ext_IE_item = -1;  /* OngoingMeasRep_v970ext_IEs */
static int hf_rrc_OngoingMeasRepList_r10_item = -1;  /* OngoingMeasRep_r10 */
static int hf_rrc_OngoingMeasRepList_r11_item = -1;  /* OngoingMeasRep_r11 */
static int hf_rrc_supportForRfc2507_01 = -1;      /* T_supportForRfc2507_01 */
static int hf_rrc_supportForRfc3095_01 = -1;      /* T_supportForRfc3095_01 */
static int hf_rrc_supported_15 = -1;              /* T_supported_08 */
static int hf_rrc_supportForRfc2507_02 = -1;      /* T_supportForRfc2507_02 */
static int hf_rrc_supported_16 = -1;              /* MaxHcContextSpace_r5 */
static int hf_rrc_supportForRfc3095_02 = -1;      /* T_supportForRfc3095_02 */
static int hf_rrc_supported_17 = -1;              /* T_supported_09 */
static int hf_rrc_losslessDLRLC_PDUSizeChange_01 = -1;  /* T_losslessDLRLC_PDUSizeChange_01 */
static int hf_rrc_supportForRfc2507_03 = -1;      /* T_supportForRfc2507_03 */
static int hf_rrc_supportForRfc3095_03 = -1;      /* T_supportForRfc3095_03 */
static int hf_rrc_supported_18 = -1;              /* T_supported_10 */
static int hf_rrc_fddPhysChCapability_06 = -1;    /* T_fddPhysChCapability_06 */
static int hf_rrc_downlinkPhysChCapability_13 = -1;  /* DL_PhysChCapabilityFDD_r4 */
static int hf_rrc_tdd384_PhysChCapability = -1;   /* T_tdd384_PhysChCapability */
static int hf_rrc_tdd128_PhysChCapability_01 = -1;  /* T_tdd128_PhysChCapability_01 */
static int hf_rrc_fddPhysChCapability_07 = -1;    /* T_fddPhysChCapability_07 */
static int hf_rrc_downlinkPhysChCapability_14 = -1;  /* DL_PhysChCapabilityFDD_r5 */
static int hf_rrc_tdd384_PhysChCapability_01 = -1;  /* T_tdd384_PhysChCapability_01 */
static int hf_rrc_downlinkPhysChCapability_15 = -1;  /* DL_PhysChCapabilityTDD_r5 */
static int hf_rrc_tdd128_PhysChCapability_02 = -1;  /* T_tdd128_PhysChCapability_02 */
static int hf_rrc_downlinkPhysChCapability_16 = -1;  /* DL_PhysChCapabilityTDD_LCR_r5 */
static int hf_rrc_fddRF_Capability_04 = -1;       /* T_fddRF_Capability_04 */
static int hf_rrc_tdd384_RF_Capability_02 = -1;   /* T_tdd384_RF_Capability_01 */
static int hf_rrc_tdd128_RF_Capability_03 = -1;   /* T_tdd128_RF_Capability_01 */
static int hf_rrc_rfc3095_Context_List = -1;      /* RFC3095_Context_List */
static int hf_rrc_RFC3095_Context_List_item = -1;  /* RFC3095_Context_List_item */
static int hf_rrc_dl_RFC3095_Context = -1;        /* DL_RFC3095_Context */
static int hf_rrc_ul_RFC3095_Context = -1;        /* UL_RFC3095_Context */
static int hf_rrc_totalRLC_AM_BufferSize_05 = -1;  /* TotalRLC_AM_BufferSize_r5 */
static int hf_rrc_ul_RRC_HFN = -1;                /* BIT_STRING_SIZE_28 */
static int hf_rrc_dl_RRC_HFN = -1;                /* BIT_STRING_SIZE_28 */
static int hf_rrc_ul_RRC_SequenceNumber = -1;     /* RRC_MessageSequenceNumber */
static int hf_rrc_dl_RRC_SequenceNumber = -1;     /* RRC_MessageSequenceNumber */
static int hf_rrc_SRB_SpecificIntegrityProtInfoList_item = -1;  /* SRB_SpecificIntegrityProtInfo */
static int hf_rrc_validity_CellPCH_UraPCH_01 = -1;  /* T_validity_CellPCH_UraPCH_01 */
static int hf_rrc_sfn_sfnType2Capability_01 = -1;  /* T_sfn_sfnType2Capability_01 */
static int hf_rrc_pdcp_Capability_03 = -1;        /* PDCP_Capability_r4 */
static int hf_rrc_rf_Capability_04 = -1;          /* RF_Capability_r4 */
static int hf_rrc_physicalChannelCapability_08 = -1;  /* PhysicalChannelCapability_r4 */
static int hf_rrc_ue_positioning_Capability_01 = -1;  /* UE_Positioning_Capability_r4 */
static int hf_rrc_measurementCapability_06 = -1;  /* MeasurementCapability_r4 */
static int hf_rrc_pdcp_Capability_04 = -1;        /* PDCP_Capability_r5 */
static int hf_rrc_rlc_Capability_05 = -1;         /* RLC_Capability_r5 */
static int hf_rrc_physicalChannelCapability_09 = -1;  /* PhysicalChannelCapability_r5 */
static int hf_rrc_ue_MultiModeRAT_Capability_01 = -1;  /* UE_MultiModeRAT_Capability_r5 */
static int hf_rrc_pdcp_Capability_05 = -1;        /* PDCP_Capability_r6 */
static int hf_rrc_ul_mode = -1;                   /* T_ul_mode */
static int hf_rrc_ul_ref_ir = -1;                 /* OCTET_STRING_SIZE_1_3000 */
static int hf_rrc_ul_ref_time = -1;               /* INTEGER_0_4294967295 */
static int hf_rrc_ul_curr_time = -1;              /* INTEGER_0_4294967295 */
static int hf_rrc_ul_syn_offset_id = -1;          /* INTEGER_0_65535 */
static int hf_rrc_ul_syn_slope_ts = -1;           /* INTEGER_0_4294967295 */
static int hf_rrc_ul_ref_sn_1 = -1;               /* INTEGER_0_65535 */
/* named bits */
static int hf_rrc_T_cipheringAlgorithmCap_spare15 = -1;
static int hf_rrc_T_cipheringAlgorithmCap_spare14 = -1;
static int hf_rrc_T_cipheringAlgorithmCap_spare13 = -1;
static int hf_rrc_T_cipheringAlgorithmCap_spare12 = -1;
static int hf_rrc_T_cipheringAlgorithmCap_spare11 = -1;
static int hf_rrc_T_cipheringAlgorithmCap_spare10 = -1;
static int hf_rrc_T_cipheringAlgorithmCap_spare9 = -1;
static int hf_rrc_T_cipheringAlgorithmCap_spare8 = -1;
static int hf_rrc_T_cipheringAlgorithmCap_spare7 = -1;
static int hf_rrc_T_cipheringAlgorithmCap_spare6 = -1;
static int hf_rrc_T_cipheringAlgorithmCap_spare5 = -1;
static int hf_rrc_T_cipheringAlgorithmCap_spare4 = -1;
static int hf_rrc_T_cipheringAlgorithmCap_spare3 = -1;
static int hf_rrc_T_cipheringAlgorithmCap_uea2 = -1;
static int hf_rrc_T_cipheringAlgorithmCap_uea1 = -1;
static int hf_rrc_T_cipheringAlgorithmCap_uea0 = -1;
static int hf_rrc_T_integrityProtectionAlgorithmCap_spare15 = -1;
static int hf_rrc_T_integrityProtectionAlgorithmCap_spare14 = -1;
static int hf_rrc_T_integrityProtectionAlgorithmCap_spare13 = -1;
static int hf_rrc_T_integrityProtectionAlgorithmCap_spare12 = -1;
static int hf_rrc_T_integrityProtectionAlgorithmCap_spare11 = -1;
static int hf_rrc_T_integrityProtectionAlgorithmCap_spare10 = -1;
static int hf_rrc_T_integrityProtectionAlgorithmCap_spare9 = -1;
static int hf_rrc_T_integrityProtectionAlgorithmCap_spare8 = -1;
static int hf_rrc_T_integrityProtectionAlgorithmCap_spare7 = -1;
static int hf_rrc_T_integrityProtectionAlgorithmCap_spare6 = -1;
static int hf_rrc_T_integrityProtectionAlgorithmCap_spare5 = -1;
static int hf_rrc_T_integrityProtectionAlgorithmCap_spare4 = -1;
static int hf_rrc_T_integrityProtectionAlgorithmCap_spare3 = -1;
static int hf_rrc_T_integrityProtectionAlgorithmCap_uia2 = -1;
static int hf_rrc_T_integrityProtectionAlgorithmCap_uia1 = -1;
static int hf_rrc_T_integrityProtectionAlgorithmCap_spare0 = -1;
static int hf_rrc_T_assignedSubChannelNumber_b3 = -1;
static int hf_rrc_T_assignedSubChannelNumber_b2 = -1;
static int hf_rrc_T_assignedSubChannelNumber_b1 = -1;
static int hf_rrc_T_assignedSubChannelNumber_b0 = -1;
static int hf_rrc_T_channelisationCodeIndices_chCodeIndex7 = -1;
static int hf_rrc_T_channelisationCodeIndices_chCodeIndex6 = -1;
static int hf_rrc_T_channelisationCodeIndices_chCodeIndex5 = -1;
static int hf_rrc_T_channelisationCodeIndices_chCodeIndex4 = -1;
static int hf_rrc_T_channelisationCodeIndices_chCodeIndex3 = -1;
static int hf_rrc_T_channelisationCodeIndices_chCodeIndex2 = -1;
static int hf_rrc_T_channelisationCodeIndices_chCodeIndex1 = -1;
static int hf_rrc_T_channelisationCodeIndices_chCodeIndex0 = -1;
static int hf_rrc_T_subchannels_01_subCh3 = -1;
static int hf_rrc_T_subchannels_01_subCh2 = -1;
static int hf_rrc_T_subchannels_01_subCh1 = -1;
static int hf_rrc_T_subchannels_01_subCh0 = -1;
static int hf_rrc_T_subchannels_02_subCh7 = -1;
static int hf_rrc_T_subchannels_02_subCh6 = -1;
static int hf_rrc_T_subchannels_02_subCh5 = -1;
static int hf_rrc_T_subchannels_02_subCh4 = -1;
static int hf_rrc_T_subchannels_02_subCh3 = -1;
static int hf_rrc_T_subchannels_02_subCh2 = -1;
static int hf_rrc_T_subchannels_02_subCh1 = -1;
static int hf_rrc_T_subchannels_02_subCh0 = -1;
static int hf_rrc_T_channelisationCodeIndices_01_chCodeIndex15 = -1;
static int hf_rrc_T_channelisationCodeIndices_01_chCodeIndex14 = -1;
static int hf_rrc_T_channelisationCodeIndices_01_chCodeIndex13 = -1;
static int hf_rrc_T_channelisationCodeIndices_01_chCodeIndex12 = -1;
static int hf_rrc_T_channelisationCodeIndices_01_chCodeIndex11 = -1;
static int hf_rrc_T_channelisationCodeIndices_01_chCodeIndex10 = -1;
static int hf_rrc_T_channelisationCodeIndices_01_chCodeIndex9 = -1;
static int hf_rrc_T_channelisationCodeIndices_01_chCodeIndex8 = -1;
static int hf_rrc_T_channelisationCodeIndices_01_chCodeIndex7 = -1;
static int hf_rrc_T_channelisationCodeIndices_01_chCodeIndex6 = -1;
static int hf_rrc_T_channelisationCodeIndices_01_chCodeIndex5 = -1;
static int hf_rrc_T_channelisationCodeIndices_01_chCodeIndex4 = -1;
static int hf_rrc_T_channelisationCodeIndices_01_chCodeIndex3 = -1;
static int hf_rrc_T_channelisationCodeIndices_01_chCodeIndex2 = -1;
static int hf_rrc_T_channelisationCodeIndices_01_chCodeIndex1 = -1;
static int hf_rrc_T_channelisationCodeIndices_01_chCodeIndex0 = -1;
static int hf_rrc_T_subchannels_04_subCh3 = -1;
static int hf_rrc_T_subchannels_04_subCh2 = -1;
static int hf_rrc_T_subchannels_04_subCh1 = -1;
static int hf_rrc_T_subchannels_04_subCh0 = -1;
static int hf_rrc_T_subchannels_05_subCh7 = -1;
static int hf_rrc_T_subchannels_05_subCh6 = -1;
static int hf_rrc_T_subchannels_05_subCh5 = -1;
static int hf_rrc_T_subchannels_05_subCh4 = -1;
static int hf_rrc_T_subchannels_05_subCh3 = -1;
static int hf_rrc_T_subchannels_05_subCh2 = -1;
static int hf_rrc_T_subchannels_05_subCh1 = -1;
static int hf_rrc_T_subchannels_05_subCh0 = -1;
static int hf_rrc_T_subchannels_06_subCh15 = -1;
static int hf_rrc_T_subchannels_06_subCh14 = -1;
static int hf_rrc_T_subchannels_06_subCh13 = -1;
static int hf_rrc_T_subchannels_06_subCh12 = -1;
static int hf_rrc_T_subchannels_06_subCh11 = -1;
static int hf_rrc_T_subchannels_06_subCh10 = -1;
static int hf_rrc_T_subchannels_06_subCh9 = -1;
static int hf_rrc_T_subchannels_06_subCh8 = -1;
static int hf_rrc_T_subchannels_06_subCh7 = -1;
static int hf_rrc_T_subchannels_06_subCh6 = -1;
static int hf_rrc_T_subchannels_06_subCh5 = -1;
static int hf_rrc_T_subchannels_06_subCh4 = -1;
static int hf_rrc_T_subchannels_06_subCh3 = -1;
static int hf_rrc_T_subchannels_06_subCh2 = -1;
static int hf_rrc_T_subchannels_06_subCh1 = -1;
static int hf_rrc_T_subchannels_06_subCh0 = -1;
static int hf_rrc_T_availableSYNC_UlCodesIndics_sulCodeIndex7 = -1;
static int hf_rrc_T_availableSYNC_UlCodesIndics_sulCodeIndex6 = -1;
static int hf_rrc_T_availableSYNC_UlCodesIndics_sulCodeIndex5 = -1;
static int hf_rrc_T_availableSYNC_UlCodesIndics_sulCodeIndex4 = -1;
static int hf_rrc_T_availableSYNC_UlCodesIndics_sulCodeIndex3 = -1;
static int hf_rrc_T_availableSYNC_UlCodesIndics_sulCodeIndex2 = -1;
static int hf_rrc_T_availableSYNC_UlCodesIndics_sulCodeIndex1 = -1;
static int hf_rrc_T_availableSYNC_UlCodesIndics_sulCodeIndex0 = -1;
static int hf_rrc_T_subchannels_08_subCh3 = -1;
static int hf_rrc_T_subchannels_08_subCh2 = -1;
static int hf_rrc_T_subchannels_08_subCh1 = -1;
static int hf_rrc_T_subchannels_08_subCh0 = -1;
static int hf_rrc_T_subchannels_09_subCh7 = -1;
static int hf_rrc_T_subchannels_09_subCh6 = -1;
static int hf_rrc_T_subchannels_09_subCh5 = -1;
static int hf_rrc_T_subchannels_09_subCh4 = -1;
static int hf_rrc_T_subchannels_09_subCh3 = -1;
static int hf_rrc_T_subchannels_09_subCh2 = -1;
static int hf_rrc_T_subchannels_09_subCh1 = -1;
static int hf_rrc_T_subchannels_09_subCh0 = -1;
static int hf_rrc_AvailableSignatures_signature15 = -1;
static int hf_rrc_AvailableSignatures_signature14 = -1;
static int hf_rrc_AvailableSignatures_signature13 = -1;
static int hf_rrc_AvailableSignatures_signature12 = -1;
static int hf_rrc_AvailableSignatures_signature11 = -1;
static int hf_rrc_AvailableSignatures_signature10 = -1;
static int hf_rrc_AvailableSignatures_signature9 = -1;
static int hf_rrc_AvailableSignatures_signature8 = -1;
static int hf_rrc_AvailableSignatures_signature7 = -1;
static int hf_rrc_AvailableSignatures_signature6 = -1;
static int hf_rrc_AvailableSignatures_signature5 = -1;
static int hf_rrc_AvailableSignatures_signature4 = -1;
static int hf_rrc_AvailableSignatures_signature3 = -1;
static int hf_rrc_AvailableSignatures_signature2 = -1;
static int hf_rrc_AvailableSignatures_signature1 = -1;
static int hf_rrc_AvailableSignatures_signature0 = -1;
static int hf_rrc_AvailableSubChannelNumbers_subCh11 = -1;
static int hf_rrc_AvailableSubChannelNumbers_subCh10 = -1;
static int hf_rrc_AvailableSubChannelNumbers_subCh9 = -1;
static int hf_rrc_AvailableSubChannelNumbers_subCh8 = -1;
static int hf_rrc_AvailableSubChannelNumbers_subCh7 = -1;
static int hf_rrc_AvailableSubChannelNumbers_subCh6 = -1;
static int hf_rrc_AvailableSubChannelNumbers_subCh5 = -1;
static int hf_rrc_AvailableSubChannelNumbers_subCh4 = -1;
static int hf_rrc_AvailableSubChannelNumbers_subCh3 = -1;
static int hf_rrc_AvailableSubChannelNumbers_subCh2 = -1;
static int hf_rrc_AvailableSubChannelNumbers_subCh1 = -1;
static int hf_rrc_AvailableSubChannelNumbers_subCh0 = -1;
static int hf_rrc_T_bitmap_chCode16_SF16 = -1;
static int hf_rrc_T_bitmap_chCode15_SF16 = -1;
static int hf_rrc_T_bitmap_chCode14_SF16 = -1;
static int hf_rrc_T_bitmap_chCode13_SF16 = -1;
static int hf_rrc_T_bitmap_chCode12_SF16 = -1;
static int hf_rrc_T_bitmap_chCode11_SF16 = -1;
static int hf_rrc_T_bitmap_chCode10_SF16 = -1;
static int hf_rrc_T_bitmap_chCode9_SF16 = -1;
static int hf_rrc_T_bitmap_chCode8_SF16 = -1;
static int hf_rrc_T_bitmap_chCode7_SF16 = -1;
static int hf_rrc_T_bitmap_chCode6_SF16 = -1;
static int hf_rrc_T_bitmap_chCode5_SF16 = -1;
static int hf_rrc_T_bitmap_chCode4_SF16 = -1;
static int hf_rrc_T_bitmap_chCode3_SF16 = -1;
static int hf_rrc_T_bitmap_chCode2_SF16 = -1;
static int hf_rrc_T_bitmap_chCode1_SF16 = -1;
static int hf_rrc_T_bitmap_01_chCode32_SF32 = -1;
static int hf_rrc_T_bitmap_01_chCode31_SF32 = -1;
static int hf_rrc_T_bitmap_01_chCode30_SF32 = -1;
static int hf_rrc_T_bitmap_01_chCode29_SF32 = -1;
static int hf_rrc_T_bitmap_01_chCode28_SF32 = -1;
static int hf_rrc_T_bitmap_01_chCode27_SF32 = -1;
static int hf_rrc_T_bitmap_01_chCode26_SF32 = -1;
static int hf_rrc_T_bitmap_01_chCode25_SF32 = -1;
static int hf_rrc_T_bitmap_01_chCode24_SF32 = -1;
static int hf_rrc_T_bitmap_01_chCode23_SF32 = -1;
static int hf_rrc_T_bitmap_01_chCode22_SF32 = -1;
static int hf_rrc_T_bitmap_01_chCode21_SF32 = -1;
static int hf_rrc_T_bitmap_01_chCode20_SF32 = -1;
static int hf_rrc_T_bitmap_01_chCode19_SF32 = -1;
static int hf_rrc_T_bitmap_01_chCode18_SF32 = -1;
static int hf_rrc_T_bitmap_01_chCode17_SF32 = -1;
static int hf_rrc_T_bitmap_01_chCode16_SF32 = -1;
static int hf_rrc_T_bitmap_01_chCode15_SF32 = -1;
static int hf_rrc_T_bitmap_01_chCode14_SF32 = -1;
static int hf_rrc_T_bitmap_01_chCode13_SF32 = -1;
static int hf_rrc_T_bitmap_01_chCode12_SF32 = -1;
static int hf_rrc_T_bitmap_01_chCode11_SF32 = -1;
static int hf_rrc_T_bitmap_01_chCode10_SF32 = -1;
static int hf_rrc_T_bitmap_01_chCode9_SF32 = -1;
static int hf_rrc_T_bitmap_01_chCode8_SF32 = -1;
static int hf_rrc_T_bitmap_01_chCode7_SF32 = -1;
static int hf_rrc_T_bitmap_01_chCode6_SF32 = -1;
static int hf_rrc_T_bitmap_01_chCode5_SF32 = -1;
static int hf_rrc_T_bitmap_01_chCode4_SF32 = -1;
static int hf_rrc_T_bitmap_01_chCode3_SF32 = -1;
static int hf_rrc_T_bitmap_01_chCode2_SF32 = -1;
static int hf_rrc_T_bitmap_01_chCode1_SF32 = -1;
static int hf_rrc_T_sync_UL_CodesBitmap_code7 = -1;
static int hf_rrc_T_sync_UL_CodesBitmap_code6 = -1;
static int hf_rrc_T_sync_UL_CodesBitmap_code5 = -1;
static int hf_rrc_T_sync_UL_CodesBitmap_code4 = -1;
static int hf_rrc_T_sync_UL_CodesBitmap_code3 = -1;
static int hf_rrc_T_sync_UL_CodesBitmap_code2 = -1;
static int hf_rrc_T_sync_UL_CodesBitmap_code1 = -1;
static int hf_rrc_T_sync_UL_CodesBitmap_code0 = -1;
static int hf_rrc_Sync_UL_Codes_Bitmap_code7 = -1;
static int hf_rrc_Sync_UL_Codes_Bitmap_code6 = -1;
static int hf_rrc_Sync_UL_Codes_Bitmap_code5 = -1;
static int hf_rrc_Sync_UL_Codes_Bitmap_code4 = -1;
static int hf_rrc_Sync_UL_Codes_Bitmap_code3 = -1;
static int hf_rrc_Sync_UL_Codes_Bitmap_code2 = -1;
static int hf_rrc_Sync_UL_Codes_Bitmap_code1 = -1;
static int hf_rrc_Sync_UL_Codes_Bitmap_code0 = -1;
static int hf_rrc_GsmSecurityCapability_a5_7 = -1;
static int hf_rrc_GsmSecurityCapability_a5_6 = -1;
static int hf_rrc_GsmSecurityCapability_a5_5 = -1;
static int hf_rrc_GsmSecurityCapability_a5_4 = -1;
static int hf_rrc_GsmSecurityCapability_a5_3 = -1;
static int hf_rrc_GsmSecurityCapability_a5_2 = -1;
static int hf_rrc_GsmSecurityCapability_a5_1 = -1;

/*--- End of included file: packet-rrc-hf.c ---*/
#line 103 "../../asn1/rrc/packet-rrc-template.c"

/* Initialize the subtree pointers */
static int ett_rrc = -1;


/*--- Included file: packet-rrc-ett.c ---*/
#line 1 "../../asn1/rrc/packet-rrc-ett.c"
static gint ett_rrc_DL_DCCH_Message = -1;
static gint ett_rrc_DL_DCCH_MessageType = -1;
static gint ett_rrc_UL_DCCH_Message = -1;
static gint ett_rrc_UL_DCCH_MessageType = -1;
static gint ett_rrc_UL_DCCH_MessageType_ext = -1;
static gint ett_rrc_DL_CCCH_Message = -1;
static gint ett_rrc_DL_CCCH_MessageType = -1;
static gint ett_rrc_UL_CCCH_Message = -1;
static gint ett_rrc_UL_CCCH_MessageType = -1;
static gint ett_rrc_PCCH_Message = -1;
static gint ett_rrc_PCCH_MessageType = -1;
static gint ett_rrc_DL_SHCCH_Message = -1;
static gint ett_rrc_DL_SHCCH_MessageType = -1;
static gint ett_rrc_UL_SHCCH_Message = -1;
static gint ett_rrc_UL_SHCCH_MessageType = -1;
static gint ett_rrc_BCCH_FACH_Message = -1;
static gint ett_rrc_BCCH_FACH_MessageType = -1;
static gint ett_rrc_BCCH_BCH_Message = -1;
static gint ett_rrc_MCCH_Message = -1;
static gint ett_rrc_MCCH_MessageType = -1;
static gint ett_rrc_MSCH_Message = -1;
static gint ett_rrc_MSCH_MessageType = -1;
static gint ett_rrc_ActiveSetUpdate = -1;
static gint ett_rrc_T_r3 = -1;
static gint ett_rrc_T_laterNonCriticalExtensions = -1;
static gint ett_rrc_T_v4b0NonCriticalExtensions = -1;
static gint ett_rrc_T_v590NonCriticalExtensions = -1;
static gint ett_rrc_T_v690NonCriticalExtensions = -1;
static gint ett_rrc_T_nonCriticalExtensions = -1;
static gint ett_rrc_T_later_than_r3 = -1;
static gint ett_rrc_T_criticalExtensions = -1;
static gint ett_rrc_T_r6 = -1;
static gint ett_rrc_T_v6b0NonCriticalExtensions = -1;
static gint ett_rrc_T_nonCriticalExtensions_01 = -1;
static gint ett_rrc_T_criticalExtensions_01 = -1;
static gint ett_rrc_T_r7 = -1;
static gint ett_rrc_T_v780NonCriticalExtensions = -1;
static gint ett_rrc_T_v7f0NonCriticalExtensions = -1;
static gint ett_rrc_T_v7g0NonCriticalExtensions = -1;
static gint ett_rrc_T_nonCriticalExtensions_02 = -1;
static gint ett_rrc_T_criticalExtensions_02 = -1;
static gint ett_rrc_T_r8 = -1;
static gint ett_rrc_T_v890NonCriticalExtensions = -1;
static gint ett_rrc_T_v7g0NonCriticalExtensions_01 = -1;
static gint ett_rrc_T_nonCriticalExtensions_03 = -1;
static gint ett_rrc_T_criticalExtensions_03 = -1;
static gint ett_rrc_T_r9 = -1;
static gint ett_rrc_T_v950NonCriticalExtensions = -1;
static gint ett_rrc_T_nonCriticalExtensions_04 = -1;
static gint ett_rrc_T_criticalExtensions_04 = -1;
static gint ett_rrc_T_r10 = -1;
static gint ett_rrc_T_vbxyNonCriticalExtensions = -1;
static gint ett_rrc_T_nonCriticalExtensions_05 = -1;
static gint ett_rrc_T_criticalExtensions_05 = -1;
static gint ett_rrc_T_r11 = -1;
static gint ett_rrc_T_nonCriticalExtensions_06 = -1;
static gint ett_rrc_T_criticalExtensions_06 = -1;
static gint ett_rrc_ActiveSetUpdate_r3_IEs = -1;
static gint ett_rrc_ActiveSetUpdate_v4b0ext_IEs = -1;
static gint ett_rrc_ActiveSetUpdate_v590ext_IEs = -1;
static gint ett_rrc_ActiveSetUpdate_v690ext_IEs = -1;
static gint ett_rrc_ActiveSetUpdate_r6_IEs = -1;
static gint ett_rrc_ActiveSetUpdate_v6b0ext_IEs = -1;
static gint ett_rrc_ActiveSetUpdate_r7_IEs = -1;
static gint ett_rrc_ActiveSetUpdate_v780ext_IEs = -1;
static gint ett_rrc_ActiveSetUpdate_v7f0ext_IEs = -1;
static gint ett_rrc_ActiveSetUpdate_v7g0ext_IEs = -1;
static gint ett_rrc_ActiveSetUpdate_r8_IEs = -1;
static gint ett_rrc_ActiveSetUpdate_v890ext_IEs = -1;
static gint ett_rrc_ActiveSetUpdate_r9_IEs = -1;
static gint ett_rrc_ActiveSetUpdate_v950ext_IEs = -1;
static gint ett_rrc_ActiveSetUpdate_r10_IEs = -1;
static gint ett_rrc_ActiveSetUpdate_vbxyext_IEs = -1;
static gint ett_rrc_ActiveSetUpdate_r11_IEs = -1;
static gint ett_rrc_ActiveSetUpdateComplete = -1;
static gint ett_rrc_T_laterNonCriticalExtensions_01 = -1;
static gint ett_rrc_T_nonCriticalExtensions_07 = -1;
static gint ett_rrc_ActiveSetUpdateFailure = -1;
static gint ett_rrc_T_laterNonCriticalExtensions_02 = -1;
static gint ett_rrc_T_nonCriticalExtensions_08 = -1;
static gint ett_rrc_AssistanceDataDelivery = -1;
static gint ett_rrc_T_r3_01 = -1;
static gint ett_rrc_T_v3a0NonCriticalExtensions = -1;
static gint ett_rrc_T_laterNonCriticalExtensions_03 = -1;
static gint ett_rrc_T_v4b0NonCriticalExtensions_01 = -1;
static gint ett_rrc_T_v770NonCriticalExtension = -1;
static gint ett_rrc_T_v860NonCriticalExtension = -1;
static gint ett_rrc_T_v920NonCriticalExtension = -1;
static gint ett_rrc_T_va40NonCriticalExtension = -1;
static gint ett_rrc_T_nonCriticalExtensions_09 = -1;
static gint ett_rrc_T_later_than_r3_01 = -1;
static gint ett_rrc_T_criticalExtensions_07 = -1;
static gint ett_rrc_AssistanceDataDelivery_r3_IEs = -1;
static gint ett_rrc_AssistanceDataDelivery_v3a0ext = -1;
static gint ett_rrc_AssistanceDataDelivery_v4b0ext_IEs = -1;
static gint ett_rrc_AssistanceDataDelivery_v770ext_IEs = -1;
static gint ett_rrc_AssistanceDataDelivery_v860ext_IEs = -1;
static gint ett_rrc_AssistanceDataDelivery_v920ext_IEs = -1;
static gint ett_rrc_AssistanceDataDelivery_va40ext_IEs = -1;
static gint ett_rrc_CellChangeOrderFromUTRAN = -1;
static gint ett_rrc_T_r3_02 = -1;
static gint ett_rrc_T_laterNonCriticalExtensions_04 = -1;
static gint ett_rrc_T_v590NonCriticalExtensions_01 = -1;
static gint ett_rrc_T_nonCriticalExtensions_10 = -1;
static gint ett_rrc_T_later_than_r3_02 = -1;
static gint ett_rrc_T_criticalExtensions_08 = -1;
static gint ett_rrc_CellChangeOrderFromUTRAN_r3_IEs = -1;
static gint ett_rrc_CellChangeOrderFromUTRAN_v590ext_IEs = -1;
static gint ett_rrc_T_geran_SystemInfoType = -1;
static gint ett_rrc_CellChangeOrderFromUTRANFailure = -1;
static gint ett_rrc_T_r3_03 = -1;
static gint ett_rrc_T_laterNonCriticalExtensions_05 = -1;
static gint ett_rrc_T_nonCriticalExtensions_11 = -1;
static gint ett_rrc_T_dummy = -1;
static gint ett_rrc_T_criticalExtensions_09 = -1;
static gint ett_rrc_CellChangeOrderFromUTRANFailure_r3_IEs = -1;
static gint ett_rrc_CellUpdate = -1;
static gint ett_rrc_T_laterNonCriticalExtensions_06 = -1;
static gint ett_rrc_T_v590NonCriticalExtensions_02 = -1;
static gint ett_rrc_T_v690NonCriticalExtensions_01 = -1;
static gint ett_rrc_T_v6b0NonCriticalExtensions_01 = -1;
static gint ett_rrc_T_v770NonCriticalExtensions = -1;
static gint ett_rrc_T_v860NonCriticalExtensions = -1;
static gint ett_rrc_T_va40NonCriticalExtensions = -1;
static gint ett_rrc_T_vbxyNonCriticalExtensions_01 = -1;
static gint ett_rrc_T_nonCriticalExtensions_12 = -1;
static gint ett_rrc_CellUpdate_r3_add_ext_IEs = -1;
static gint ett_rrc_T_v7g0NonCriticalExtensions_02 = -1;
static gint ett_rrc_T_nonCriticalExtensions_13 = -1;
static gint ett_rrc_CellUpdate_v590ext = -1;
static gint ett_rrc_CellUpdate_v690ext_IEs = -1;
static gint ett_rrc_CellUpdate_v6b0ext_IEs = -1;
static gint ett_rrc_CellUpdate_v770ext_IEs = -1;
static gint ett_rrc_CellUpdate_v7e0ext_IEs = -1;
static gint ett_rrc_CellUpdate_v7g0ext_IEs = -1;
static gint ett_rrc_CellUpdate_v860ext_IEs = -1;
static gint ett_rrc_CellUpdate_va40ext_IEs = -1;
static gint ett_rrc_CellUpdate_vbxyext_IEs = -1;
static gint ett_rrc_CellUpdateConfirm = -1;
static gint ett_rrc_T_r3_04 = -1;
static gint ett_rrc_T_v3a0NonCriticalExtensions_01 = -1;
static gint ett_rrc_T_laterNonCriticalExtensions_07 = -1;
static gint ett_rrc_T_v4b0NonCriticalExtensions_02 = -1;
static gint ett_rrc_T_v590NonCriticalExtenstions = -1;
static gint ett_rrc_T_v5d0NonCriticalExtenstions = -1;
static gint ett_rrc_T_v690NonCriticalExtensions_02 = -1;
static gint ett_rrc_T_nonCriticalExtensions_14 = -1;
static gint ett_rrc_T_later_than_r3_03 = -1;
static gint ett_rrc_T_criticalExtensions_10 = -1;
static gint ett_rrc_T_r4 = -1;
static gint ett_rrc_T_v4d0NonCriticalExtensions = -1;
static gint ett_rrc_T_v590NonCriticalExtenstions_01 = -1;
static gint ett_rrc_T_v5d0NonCriticalExtenstions_01 = -1;
static gint ett_rrc_T_v690NonCriticalExtensions_03 = -1;
static gint ett_rrc_T_nonCriticalExtensions_15 = -1;
static gint ett_rrc_T_criticalExtensions_11 = -1;
static gint ett_rrc_T_r5 = -1;
static gint ett_rrc_T_v5d0NonCriticalExtenstions_02 = -1;
static gint ett_rrc_T_v690NonCriticalExtensions_04 = -1;
static gint ett_rrc_T_nonCriticalExtensions_16 = -1;
static gint ett_rrc_T_criticalExtensions_12 = -1;
static gint ett_rrc_T_r6_01 = -1;
static gint ett_rrc_T_v6b0NonCriticalExtensions_02 = -1;
static gint ett_rrc_T_nonCriticalExtensions_17 = -1;
static gint ett_rrc_T_criticalExtensions_13 = -1;
static gint ett_rrc_T_r7_01 = -1;
static gint ett_rrc_T_v780NonCriticalExtensions_01 = -1;
static gint ett_rrc_T_v860NonCriticalExtensions_01 = -1;
static gint ett_rrc_T_nonCriticalExtensions_18 = -1;
static gint ett_rrc_T_criticalExtensions_14 = -1;
static gint ett_rrc_T_r8_01 = -1;
static gint ett_rrc_T_v7d0NonCriticalExtensions = -1;
static gint ett_rrc_T_v7f0NonCriticalExtensions_01 = -1;
static gint ett_rrc_T_v890NonCriticalExtensions_01 = -1;
static gint ett_rrc_T_v7g0NonCriticalExtensions_03 = -1;
static gint ett_rrc_T_v8a0NonCriticalExtensions = -1;
static gint ett_rrc_T_nonCriticalExtensions_19 = -1;
static gint ett_rrc_T_criticalExtensions_15 = -1;
static gint ett_rrc_T_r9_01 = -1;
static gint ett_rrc_T_v950NonCriticalExtensions_01 = -1;
static gint ett_rrc_T_v9c0NonCriticalExtensions = -1;
static gint ett_rrc_T_nonCriticalExtensions_20 = -1;
static gint ett_rrc_T_criticalExtensions_16 = -1;
static gint ett_rrc_T_r10_01 = -1;
static gint ett_rrc_T_v9c0NonCriticalExtensions_01 = -1;
static gint ett_rrc_T_vbxyNonCriticalExtensions_02 = -1;
static gint ett_rrc_T_nonCriticalExtensions_21 = -1;
static gint ett_rrc_T_criticalExtensions_17 = -1;
static gint ett_rrc_T_r11_01 = -1;
static gint ett_rrc_T_nonCriticalExtensions_22 = -1;
static gint ett_rrc_T_criticalExtensions_18 = -1;
static gint ett_rrc_CellUpdateConfirm_r3_IEs = -1;
static gint ett_rrc_T_modeSpecificTransChInfo = -1;
static gint ett_rrc_T_fdd = -1;
static gint ett_rrc_T_modeSpecificPhysChInfo = -1;
static gint ett_rrc_T_fdd_01 = -1;
static gint ett_rrc_CellUpdateConfirm_v3a0ext = -1;
static gint ett_rrc_CellUpdateConfirm_v4b0ext_IEs = -1;
static gint ett_rrc_CellUpdateConfirm_v590ext_IEs = -1;
static gint ett_rrc_CellUpdateConfirm_v5d0ext_IEs = -1;
static gint ett_rrc_CellUpdateConfirm_r4_IEs = -1;
static gint ett_rrc_T_modeSpecificTransChInfo_01 = -1;
static gint ett_rrc_T_fdd_02 = -1;
static gint ett_rrc_T_modeSpecificPhysChInfo_01 = -1;
static gint ett_rrc_T_fdd_03 = -1;
static gint ett_rrc_CellUpdateConfirm_r5_IEs = -1;
static gint ett_rrc_T_modeSpecificTransChInfo_02 = -1;
static gint ett_rrc_T_fdd_04 = -1;
static gint ett_rrc_T_modeSpecificPhysChInfo_02 = -1;
static gint ett_rrc_T_fdd_05 = -1;
static gint ett_rrc_CellUpdateConfirm_r6_IEs = -1;
static gint ett_rrc_CellUpdateConfirm_v690ext_IEs = -1;
static gint ett_rrc_CellUpdateConfirm_v6b0ext_IEs = -1;
static gint ett_rrc_CellUpdateConfirm_r7_IEs = -1;
static gint ett_rrc_CellUpdateConfirm_r7_add_ext_IEs = -1;
static gint ett_rrc_T_v7f0NonCriticalExtensions_02 = -1;
static gint ett_rrc_T_v7g0NonCriticalExtensions_04 = -1;
static gint ett_rrc_T_nonCriticalExtensions_23 = -1;
static gint ett_rrc_CellUpdateConfirm_v780ext_IEs = -1;
static gint ett_rrc_CellUpdateConfirm_v7d0ext_IEs = -1;
static gint ett_rrc_CellUpdateConfirm_v7f0ext_IEs = -1;
static gint ett_rrc_CellUpdateConfirm_v7g0ext_IEs = -1;
static gint ett_rrc_CellUpdateConfirm_r8_IEs = -1;
static gint ett_rrc_CellUpdateConfirm_v860ext_IEs = -1;
static gint ett_rrc_T_rab_InformationSetup = -1;
static gint ett_rrc_CellUpdateConfirm_v890ext_IEs = -1;
static gint ett_rrc_CellUpdateConfirm_v8a0ext_IEs = -1;
static gint ett_rrc_CellUpdateConfirm_r9_IEs = -1;
static gint ett_rrc_CellUpdateConfirm_v950ext_IEs = -1;
static gint ett_rrc_CellUpdateConfirm_v9c0ext_IEs = -1;
static gint ett_rrc_CellUpdateConfirm_r10_IEs = -1;
static gint ett_rrc_CellUpdateConfirm_vbxyext_IEs = -1;
static gint ett_rrc_CellUpdateConfirm_r11_IEs = -1;
static gint ett_rrc_CellUpdateConfirm_CCCH = -1;
static gint ett_rrc_T_r3_05 = -1;
static gint ett_rrc_T_laterNonCriticalExtensions_08 = -1;
static gint ett_rrc_T_v4b0NonCriticalExtensions_03 = -1;
static gint ett_rrc_T_v590NonCriticalExtensions_03 = -1;
static gint ett_rrc_T_v5d0NonCriticalExtenstions_03 = -1;
static gint ett_rrc_T_v690NonCriticalExtensions_05 = -1;
static gint ett_rrc_T_nonCriticalExtensions_24 = -1;
static gint ett_rrc_T_later_than_r3_04 = -1;
static gint ett_rrc_T_criticalExtensions_19 = -1;
static gint ett_rrc_T_r4_01 = -1;
static gint ett_rrc_T_v4d0NonCriticalExtensions_01 = -1;
static gint ett_rrc_T_v590NonCriticalExtensions_04 = -1;
static gint ett_rrc_T_v5d0NonCriticalExtenstions_04 = -1;
static gint ett_rrc_T_v690NonCriticalExtensions_06 = -1;
static gint ett_rrc_T_nonCriticalExtensions_25 = -1;
static gint ett_rrc_T_criticalExtensions_20 = -1;
static gint ett_rrc_T_r5_01 = -1;
static gint ett_rrc_T_v5d0NonCriticalExtenstions_05 = -1;
static gint ett_rrc_T_v690NonCriticalExtensions_07 = -1;
static gint ett_rrc_T_nonCriticalExtensions_26 = -1;
static gint ett_rrc_T_criticalExtensions_21 = -1;
static gint ett_rrc_T_r6_02 = -1;
static gint ett_rrc_T_v6b0NonCriticalExtensions_03 = -1;
static gint ett_rrc_T_nonCriticalExtensions_27 = -1;
static gint ett_rrc_T_criticalExtensions_22 = -1;
static gint ett_rrc_T_r7_02 = -1;
static gint ett_rrc_T_v780NonCriticalExtensions_02 = -1;
static gint ett_rrc_T_v860NonCriticalExtensions_02 = -1;
static gint ett_rrc_T_nonCriticalExtensions_28 = -1;
static gint ett_rrc_T_criticalExtensions_23 = -1;
static gint ett_rrc_T_r8_02 = -1;
static gint ett_rrc_T_v7d0NonCriticalExtensions_01 = -1;
static gint ett_rrc_T_v890NonCriticalExtensions_02 = -1;
static gint ett_rrc_T_v8a0NonCriticalExtensions_01 = -1;
static gint ett_rrc_T_v7f0NonCriticalExtensions_03 = -1;
static gint ett_rrc_T_v7g0NonCriticalExtensions_05 = -1;
static gint ett_rrc_T_nonCriticalExtensions_29 = -1;
static gint ett_rrc_T_criticalExtensions_24 = -1;
static gint ett_rrc_T_r9_02 = -1;
static gint ett_rrc_T_nonCriticalExtensions_30 = -1;
static gint ett_rrc_T_criticalExtensions_25 = -1;
static gint ett_rrc_T_r10_02 = -1;
static gint ett_rrc_T_vbxyNonCriticalExtensions_03 = -1;
static gint ett_rrc_T_nonCriticalExtensions_31 = -1;
static gint ett_rrc_T_criticalExtensions_26 = -1;
static gint ett_rrc_T_r11_02 = -1;
static gint ett_rrc_T_nonCriticalExtensions_32 = -1;
static gint ett_rrc_T_criticalExtensions_27 = -1;
static gint ett_rrc_CounterCheck = -1;
static gint ett_rrc_T_r3_06 = -1;
static gint ett_rrc_T_laterNonCriticalExtensions_09 = -1;
static gint ett_rrc_T_nonCriticalExtensions_33 = -1;
static gint ett_rrc_T_later_than_r3_05 = -1;
static gint ett_rrc_T_criticalExtensions_28 = -1;
static gint ett_rrc_CounterCheck_r3_IEs = -1;
static gint ett_rrc_CounterCheckResponse = -1;
static gint ett_rrc_T_laterNonCriticalExtensions_10 = -1;
static gint ett_rrc_T_nonCriticalExtensions_34 = -1;
static gint ett_rrc_DownlinkDirectTransfer = -1;
static gint ett_rrc_T_r3_07 = -1;
static gint ett_rrc_T_laterNonCriticalExtensions_11 = -1;
static gint ett_rrc_T_nonCriticalExtensions_35 = -1;
static gint ett_rrc_T_later_than_r3_06 = -1;
static gint ett_rrc_T_criticalExtensions_29 = -1;
static gint ett_rrc_DownlinkDirectTransfer_r3_IEs = -1;
static gint ett_rrc_ETWSPrimaryNotificationWithSecurity = -1;
static gint ett_rrc_T_nonCriticalExtensions_36 = -1;
static gint ett_rrc_HandoverToUTRANCommand = -1;
static gint ett_rrc_T_r3_08 = -1;
static gint ett_rrc_T_nonCriticalExtensions_37 = -1;
static gint ett_rrc_T_criticalExtensions_30 = -1;
static gint ett_rrc_T_r4_02 = -1;
static gint ett_rrc_T_nonCriticalExtensions_38 = -1;
static gint ett_rrc_T_criticalExtensions_31 = -1;
static gint ett_rrc_T_r5_02 = -1;
static gint ett_rrc_T_nonCriticalExtensions_39 = -1;
static gint ett_rrc_T_criticalExtensions_32 = -1;
static gint ett_rrc_T_r6_03 = -1;
static gint ett_rrc_T_v6b0NonCriticalExtensions_04 = -1;
static gint ett_rrc_T_nonCriticalExtensions_40 = -1;
static gint ett_rrc_T_criticalExtensions_33 = -1;
static gint ett_rrc_T_r7_03 = -1;
static gint ett_rrc_T_v780NonCriticalExtensions_03 = -1;
static gint ett_rrc_T_v820NonCriticalExtensions = -1;
static gint ett_rrc_T_v7d0NonCriticalExtensions_02 = -1;
static gint ett_rrc_T_nonCriticalExtensions_41 = -1;
static gint ett_rrc_T_criticalExtensions_34 = -1;
static gint ett_rrc_T_r8_03 = -1;
static gint ett_rrc_T_v7d0NonCriticalExtensions_03 = -1;
static gint ett_rrc_T_v890NonCriticalExtensions_03 = -1;
static gint ett_rrc_T_v8a0NonCriticalExtensions_02 = -1;
static gint ett_rrc_T_nonCriticalExtensions_42 = -1;
static gint ett_rrc_T_criticalExtensions_35 = -1;
static gint ett_rrc_T_r9_03 = -1;
static gint ett_rrc_T_v9c0NonCriticalExtensions_02 = -1;
static gint ett_rrc_T_nonCriticalExtensions_43 = -1;
static gint ett_rrc_HandoverToUTRANCommand_r3_IEs = -1;
static gint ett_rrc_T_specificationMode = -1;
static gint ett_rrc_T_complete = -1;
static gint ett_rrc_T_modeSpecificInfo = -1;
static gint ett_rrc_T_fdd_06 = -1;
static gint ett_rrc_T_preconfiguration = -1;
static gint ett_rrc_T_preConfigMode = -1;
static gint ett_rrc_T_defaultConfig = -1;
static gint ett_rrc_T_modeSpecificInfo_01 = -1;
static gint ett_rrc_T_fdd_07 = -1;
static gint ett_rrc_T_tdd = -1;
static gint ett_rrc_HandoverToUTRANCommand_r4_IEs = -1;
static gint ett_rrc_T_specificationMode_01 = -1;
static gint ett_rrc_T_complete_01 = -1;
static gint ett_rrc_T_modeSpecificInfo_02 = -1;
static gint ett_rrc_T_fdd_08 = -1;
static gint ett_rrc_T_preconfiguration_01 = -1;
static gint ett_rrc_T_preConfigMode_01 = -1;
static gint ett_rrc_T_defaultConfig_01 = -1;
static gint ett_rrc_T_modeSpecificInfo_03 = -1;
static gint ett_rrc_T_fdd_09 = -1;
static gint ett_rrc_T_tdd_01 = -1;
static gint ett_rrc_T_tdd384 = -1;
static gint ett_rrc_T_tdd128 = -1;
static gint ett_rrc_HandoverToUTRANCommand_r5_IEs = -1;
static gint ett_rrc_T_specificationMode_02 = -1;
static gint ett_rrc_T_complete_02 = -1;
static gint ett_rrc_T_modeSpecificInfo_04 = -1;
static gint ett_rrc_T_fdd_10 = -1;
static gint ett_rrc_T_preconfiguration_02 = -1;
static gint ett_rrc_T_preConfigMode_02 = -1;
static gint ett_rrc_T_defaultConfig_02 = -1;
static gint ett_rrc_T_modeSpecificInfo_05 = -1;
static gint ett_rrc_T_fdd_11 = -1;
static gint ett_rrc_T_tdd_02 = -1;
static gint ett_rrc_T_tdd384_01 = -1;
static gint ett_rrc_T_tdd128_01 = -1;
static gint ett_rrc_HandoverToUTRANCommand_r6_IEs = -1;
static gint ett_rrc_T_specificationMode_03 = -1;
static gint ett_rrc_T_complete_03 = -1;
static gint ett_rrc_T_preconfiguration_03 = -1;
static gint ett_rrc_T_preConfigMode_03 = -1;
static gint ett_rrc_T_defaultConfig_03 = -1;
static gint ett_rrc_T_modeSpecificInfo_06 = -1;
static gint ett_rrc_T_fdd_12 = -1;
static gint ett_rrc_T_tdd_03 = -1;
static gint ett_rrc_T_tdd384_02 = -1;
static gint ett_rrc_T_tdd128_02 = -1;
static gint ett_rrc_HandoverToUTRANCommand_v6b0ext_IEs = -1;
static gint ett_rrc_HandoverToUTRANCommand_r7_IEs = -1;
static gint ett_rrc_T_specificationMode_04 = -1;
static gint ett_rrc_T_complete_04 = -1;
static gint ett_rrc_T_preconfiguration_04 = -1;
static gint ett_rrc_T_preConfigMode_04 = -1;
static gint ett_rrc_T_defaultConfig_04 = -1;
static gint ett_rrc_T_modeSpecificInfo_07 = -1;
static gint ett_rrc_T_fdd_13 = -1;
static gint ett_rrc_T_tdd_04 = -1;
static gint ett_rrc_T_tdd384_03 = -1;
static gint ett_rrc_T_tdd128_03 = -1;
static gint ett_rrc_T_tdd768 = -1;
static gint ett_rrc_HandoverToUTRANCommand_v780ext_IEs = -1;
static gint ett_rrc_HandoverToUTRANCommand_v7d0ext_IEs = -1;
static gint ett_rrc_HandoverToUTRANCommand_r8_IEs = -1;
static gint ett_rrc_T_specificationMode_05 = -1;
static gint ett_rrc_T_complete_05 = -1;
static gint ett_rrc_T_preconfiguration_05 = -1;
static gint ett_rrc_T_preConfigMode_05 = -1;
static gint ett_rrc_T_defaultConfig_05 = -1;
static gint ett_rrc_T_modeSpecificInfo_08 = -1;
static gint ett_rrc_T_fdd_14 = -1;
static gint ett_rrc_T_tdd_05 = -1;
static gint ett_rrc_T_tdd384_04 = -1;
static gint ett_rrc_T_tdd128_04 = -1;
static gint ett_rrc_T_tdd768_01 = -1;
static gint ett_rrc_HandoverToUTRANCommand_v820ext_IEs = -1;
static gint ett_rrc_HandoverToUTRANCommand_v890ext_IEs = -1;
static gint ett_rrc_HandoverToUTRANCommand_v8a0ext_IEs = -1;
static gint ett_rrc_HandoverToUTRANCommand_r9_IEs = -1;
static gint ett_rrc_T_specificationMode_06 = -1;
static gint ett_rrc_T_complete_06 = -1;
static gint ett_rrc_T_preconfiguration_06 = -1;
static gint ett_rrc_T_preConfigMode_06 = -1;
static gint ett_rrc_T_defaultConfig_06 = -1;
static gint ett_rrc_T_modeSpecificInfo_09 = -1;
static gint ett_rrc_T_fdd_15 = -1;
static gint ett_rrc_T_tdd_06 = -1;
static gint ett_rrc_T_tdd384_05 = -1;
static gint ett_rrc_T_tdd128_05 = -1;
static gint ett_rrc_T_tdd768_02 = -1;
static gint ett_rrc_HandoverToUTRANCommand_v9c0ext_IEs = -1;
static gint ett_rrc_HandoverToUTRANCommand_LaterCriticalExtensions = -1;
static gint ett_rrc_T_r10_03 = -1;
static gint ett_rrc_T_v9c0NonCriticalExtensions_03 = -1;
static gint ett_rrc_T_nonCriticalExtensions_44 = -1;
static gint ett_rrc_T_criticalExtensions_36 = -1;
static gint ett_rrc_T_r11_03 = -1;
static gint ett_rrc_T_nonCriticalExtensions_45 = -1;
static gint ett_rrc_T_criticalExtensions_37 = -1;
static gint ett_rrc_HandoverToUTRANCommand_r10_IEs = -1;
static gint ett_rrc_T_specificationMode_07 = -1;
static gint ett_rrc_T_complete_07 = -1;
static gint ett_rrc_T_preconfiguration_07 = -1;
static gint ett_rrc_T_preConfigMode_07 = -1;
static gint ett_rrc_T_defaultConfig_07 = -1;
static gint ett_rrc_T_modeSpecificInfo_10 = -1;
static gint ett_rrc_T_fdd_16 = -1;
static gint ett_rrc_T_tdd_07 = -1;
static gint ett_rrc_T_tdd384_06 = -1;
static gint ett_rrc_T_tdd128_06 = -1;
static gint ett_rrc_T_tdd768_03 = -1;
static gint ett_rrc_HandoverToUTRANCommand_r11_IEs = -1;
static gint ett_rrc_T_specificationMode_08 = -1;
static gint ett_rrc_T_complete_08 = -1;
static gint ett_rrc_T_preconfiguration_08 = -1;
static gint ett_rrc_T_preConfigMode_08 = -1;
static gint ett_rrc_T_defaultConfig_08 = -1;
static gint ett_rrc_T_modeSpecificInfo_11 = -1;
static gint ett_rrc_T_fdd_17 = -1;
static gint ett_rrc_T_tdd_08 = -1;
static gint ett_rrc_T_tdd384_07 = -1;
static gint ett_rrc_T_tdd128_07 = -1;
static gint ett_rrc_T_tdd768_04 = -1;
static gint ett_rrc_HandoverToUTRANComplete = -1;
static gint ett_rrc_T_laterNonCriticalExtensions_12 = -1;
static gint ett_rrc_T_va40NonCriticalExtensions_01 = -1;
static gint ett_rrc_T_vbxyNonCriticalExtensions_04 = -1;
static gint ett_rrc_T_nonCriticalExtensions_46 = -1;
static gint ett_rrc_HandoverToUTRANComplete_va40ext = -1;
static gint ett_rrc_HandoverToUTRANComplete_vbxyext_IEs = -1;
static gint ett_rrc_InitialDirectTransfer = -1;
static gint ett_rrc_T_v3a0NonCriticalExtensions_02 = -1;
static gint ett_rrc_T_laterNonCriticalExtensions_13 = -1;
static gint ett_rrc_T_v590NonCriticalExtensions_05 = -1;
static gint ett_rrc_T_v690NonCriticalExtensions_08 = -1;
static gint ett_rrc_T_v770NonCriticalExtensions_01 = -1;
static gint ett_rrc_T_v860NonCriticalExtensions_03 = -1;
static gint ett_rrc_T_vbxyNonCriticalExtensions_05 = -1;
static gint ett_rrc_T_nonCriticalExtensions_47 = -1;
static gint ett_rrc_InitialDirectTransfer_v3a0ext = -1;
static gint ett_rrc_InitialDirectTransfer_v590ext = -1;
static gint ett_rrc_InitialDirectTransfer_v690ext_IEs = -1;
static gint ett_rrc_InitialDirectTransfer_v770ext_IEs = -1;
static gint ett_rrc_InitialDirectTransfer_v860ext_IEs = -1;
static gint ett_rrc_InitialDirectTransfer_r3_add_ext_IEs = -1;
static gint ett_rrc_T_nonCriticalExtensions_48 = -1;
static gint ett_rrc_InitialDirectTransfer_v7g0ext_IEs = -1;
static gint ett_rrc_InitialDirectTransfer_vbxyext_IEs = -1;
static gint ett_rrc_HandoverFromUTRANCommand_GSM = -1;
static gint ett_rrc_T_r3_09 = -1;
static gint ett_rrc_T_laterNonCriticalExtensions_14 = -1;
static gint ett_rrc_T_v690NonCriticalExtensions_09 = -1;
static gint ett_rrc_T_v860NonCriticalExtensions_04 = -1;
static gint ett_rrc_T_nonCriticalExtensions_49 = -1;
static gint ett_rrc_T_later_than_r3_07 = -1;
static gint ett_rrc_T_criticalExtensions_38 = -1;
static gint ett_rrc_T_r6_04 = -1;
static gint ett_rrc_T_v860NonCriticalExtensions_05 = -1;
static gint ett_rrc_T_nonCriticalExtensions_50 = -1;
static gint ett_rrc_T_criticalExtensions_39 = -1;
static gint ett_rrc_HandoverFromUTRANCommand_GSM_r3_IEs = -1;
static gint ett_rrc_T_gsm_message = -1;
static gint ett_rrc_T_single_GSM_Message_r3 = -1;
static gint ett_rrc_T_gsm_MessageList_r3 = -1;
static gint ett_rrc_HandoverFromUTRANCommand_GSM_r6_IEs = -1;
static gint ett_rrc_T_gsm_message_01 = -1;
static gint ett_rrc_T_single_GSM_Message_r6 = -1;
static gint ett_rrc_T_gsm_MessageList_r6 = -1;
static gint ett_rrc_T_geran_SystemInfoType_01 = -1;
static gint ett_rrc_HandoverFromUTRANCommand_GSM_v690ext_IEs = -1;
static gint ett_rrc_T_geran_SystemInfoType_02 = -1;
static gint ett_rrc_HandoverFromUTRANCommand_GSM_v860ext_IEs = -1;
static gint ett_rrc_HandoverFromUTRANCommand_GERANIu = -1;
static gint ett_rrc_T_handoverFromUTRANCommand_GERANIu = -1;
static gint ett_rrc_T_r5_03 = -1;
static gint ett_rrc_T_nonCriticalExtensions_51 = -1;
static gint ett_rrc_T_later_than_r5 = -1;
static gint ett_rrc_T_criticalExtensions_40 = -1;
static gint ett_rrc_HandoverFromUTRANCommand_GERANIu_r5_IEs = -1;
static gint ett_rrc_T_geranIu_Message = -1;
static gint ett_rrc_T_single_GERANIu_Message = -1;
static gint ett_rrc_T_geranIu_MessageList = -1;
static gint ett_rrc_HandoverFromUTRANCommand_CDMA2000 = -1;
static gint ett_rrc_T_r3_10 = -1;
static gint ett_rrc_T_laterNonCriticalExtensions_15 = -1;
static gint ett_rrc_T_nonCriticalExtensions_52 = -1;
static gint ett_rrc_T_later_than_r3_08 = -1;
static gint ett_rrc_T_criticalExtensions_41 = -1;
static gint ett_rrc_HandoverFromUTRANCommand_CDMA2000_r3_IEs = -1;
static gint ett_rrc_HandoverFromUTRANCommand_EUTRA = -1;
static gint ett_rrc_T_criticalExtensions_42 = -1;
static gint ett_rrc_T_r8_04 = -1;
static gint ett_rrc_T_nonCriticalExtensions_53 = -1;
static gint ett_rrc_T_criticalExtensions_43 = -1;
static gint ett_rrc_T_r11_04 = -1;
static gint ett_rrc_T_nonCriticalExtensions_54 = -1;
static gint ett_rrc_T_criticalExtensions_44 = -1;
static gint ett_rrc_HandoverFromUTRANCommand_EUTRA_r8_IEs = -1;
static gint ett_rrc_HandoverFromUTRANCommand_EUTRA_r11_IEs = -1;
static gint ett_rrc_HandoverFromUTRANFailure = -1;
static gint ett_rrc_T_interRATMessage = -1;
static gint ett_rrc_T_gsm = -1;
static gint ett_rrc_T_cdma2000 = -1;
static gint ett_rrc_T_laterNonCriticalExtensions_16 = -1;
static gint ett_rrc_T_v590NonCriticalExtensions_06 = -1;
static gint ett_rrc_T_v860NonCriticalExtensions_06 = -1;
static gint ett_rrc_T_nonCriticalExtensions_55 = -1;
static gint ett_rrc_HandoverFromUtranFailure_v590ext_IEs = -1;
static gint ett_rrc_HandoverFromUtranFailure_v860ext_IEs = -1;
static gint ett_rrc_InterRATHandoverInfo = -1;
static gint ett_rrc_T_predefinedConfigStatusList = -1;
static gint ett_rrc_T_uE_SecurityInformation = -1;
static gint ett_rrc_T_ue_CapabilityContainer = -1;
static gint ett_rrc_T_v390NonCriticalExtensions = -1;
static gint ett_rrc_T_present_01 = -1;
static gint ett_rrc_T_v3a0NonCriticalExtensions_03 = -1;
static gint ett_rrc_T_laterNonCriticalExtensions_17 = -1;
static gint ett_rrc_T_v3g0NonCriticalExtensions = -1;
static gint ett_rrc_T_v4b0NonCriticalExtensions_04 = -1;
static gint ett_rrc_T_v4d0NonCriticalExtensions_02 = -1;
static gint ett_rrc_T_v590NonCriticalExtensions_07 = -1;
static gint ett_rrc_T_v690NonCriticalExtensions_10 = -1;
static gint ett_rrc_InterRATHandoverInfo_v390ext_IEs = -1;
static gint ett_rrc_InterRATHandoverInfo_v3a0ext_IEs = -1;
static gint ett_rrc_InterRATHandoverInfo_v3d0ext_IEs = -1;
static gint ett_rrc_InterRATHandoverInfo_v3g0ext_IEs = -1;
static gint ett_rrc_InterRATHandoverInfo_r3_add_ext_IEs = -1;
static gint ett_rrc_T_v7e0NonCriticalExtensions = -1;
static gint ett_rrc_T_v7f0NonCriticalExtensions_04 = -1;
static gint ett_rrc_T_va40NonCriticalExtensions_02 = -1;
static gint ett_rrc_T_nonCriticalExtensions_56 = -1;
static gint ett_rrc_InterRATHandoverInfo_v4b0ext_IEs = -1;
static gint ett_rrc_InterRATHandoverInfo_v4d0ext_IEs = -1;
static gint ett_rrc_InterRATHandoverInfo_v590ext_IEs = -1;
static gint ett_rrc_InterRATHandoverInfo_v690ext1_IEs = -1;
static gint ett_rrc_InterRATHandoverInfo_v690ext_IEs = -1;
static gint ett_rrc_InterRATHandoverInfo_NonCriticalExtension_v6b0_IEs = -1;
static gint ett_rrc_T_v6e0NonCriticalExtensions = -1;
static gint ett_rrc_T_v770NonCriticalExtensions_02 = -1;
static gint ett_rrc_T_v790nonCriticalExtensions = -1;
static gint ett_rrc_T_v860NonCriticalExtensions_07 = -1;
static gint ett_rrc_T_v880NonCriticalExtensions = -1;
static gint ett_rrc_T_v920NonCriticalExtensions = -1;
static gint ett_rrc_T_v8b0NonCriticalExtensions = -1;
static gint ett_rrc_T_v950NonCriticalExtensions_02 = -1;
static gint ett_rrc_T_va40NonCriticalExtensions_03 = -1;
static gint ett_rrc_T_va80NonCriticalExtensions = -1;
static gint ett_rrc_T_vbxyNonCriticalExtensions_06 = -1;
static gint ett_rrc_T_nonCriticalExtensions_57 = -1;
static gint ett_rrc_InterRATHandoverInfo_v6b0ext_IEs = -1;
static gint ett_rrc_InterRATHandoverInfo_v6e0ext_IEs = -1;
static gint ett_rrc_InterRATHandoverInfo_v770ext_IEs = -1;
static gint ett_rrc_InterRATHandoverInfo_v790ext_IEs = -1;
static gint ett_rrc_InterRATHandoverInfo_v7e0ext_IEs = -1;
static gint ett_rrc_InterRATHandoverInfo_v7f0ext_IEs = -1;
static gint ett_rrc_InterRATHandoverInfo_v860ext_IEs = -1;
static gint ett_rrc_InterRATHandoverInfo_v880ext_IEs = -1;
static gint ett_rrc_InterRATHandoverInfo_v920ext_IEs = -1;
static gint ett_rrc_InterRATHandoverInfo_v8b0ext_IEs = -1;
static gint ett_rrc_InterRATHandoverInfo_v950ext_IEs = -1;
static gint ett_rrc_InterRATHandoverInfo_va40ext_IEs = -1;
static gint ett_rrc_InterRATHandoverInfoBand_va40ext_IEs = -1;
static gint ett_rrc_InterRATHandoverInfo_va80ext_IEs = -1;
static gint ett_rrc_InterRATHandoverInfo_vbxyext_IEs = -1;
static gint ett_rrc_MeasurementControl = -1;
static gint ett_rrc_T_r3_11 = -1;
static gint ett_rrc_T_v390nonCriticalExtensions = -1;
static gint ett_rrc_T_v3a0NonCriticalExtensions_04 = -1;
static gint ett_rrc_T_laterNonCriticalExtensions_18 = -1;
static gint ett_rrc_T_v4b0NonCriticalExtensions_05 = -1;
static gint ett_rrc_T_v590NonCriticalExtensions_08 = -1;
static gint ett_rrc_T_v5b0NonCriticalExtensions = -1;
static gint ett_rrc_T_nonCriticalExtensions_58 = -1;
static gint ett_rrc_T_later_than_r3_09 = -1;
static gint ett_rrc_T_criticalExtensions_45 = -1;
static gint ett_rrc_T_r4_03 = -1;
static gint ett_rrc_T_v4d0NonCriticalExtensions_03 = -1;
static gint ett_rrc_T_v590NonCriticalExtensions_09 = -1;
static gint ett_rrc_T_v5b0NonCriticalExtensions_01 = -1;
static gint ett_rrc_T_nonCriticalExtensions_59 = -1;
static gint ett_rrc_T_later_than_r4 = -1;
static gint ett_rrc_T_criticalExtensions_46 = -1;
static gint ett_rrc_T_r6_05 = -1;
static gint ett_rrc_T_v6a0NonCriticalExtensions = -1;
static gint ett_rrc_T_nonCriticalExtensions_60 = -1;
static gint ett_rrc_T_criticalExtensions_47 = -1;
static gint ett_rrc_T_r7_04 = -1;
static gint ett_rrc_T_v7b0NonCriticalExtensions = -1;
static gint ett_rrc_T_nonCriticalExtensions_61 = -1;
static gint ett_rrc_T_criticalExtensions_48 = -1;
static gint ett_rrc_T_r8_05 = -1;
static gint ett_rrc_T_v8a0NonCriticalExtensions_03 = -1;
static gint ett_rrc_T_nonCriticalExtensions_62 = -1;
static gint ett_rrc_T_criticalExtensions_49 = -1;
static gint ett_rrc_T_r9_04 = -1;
static gint ett_rrc_T_v970NonCriticalExtensions = -1;
static gint ett_rrc_T_nonCriticalExtensions_63 = -1;
static gint ett_rrc_T_criticalExtensions_50 = -1;
static gint ett_rrc_T_r10_04 = -1;
static gint ett_rrc_T_vaa0NonCriticalExtensions = -1;
static gint ett_rrc_T_nonCriticalExtensions_64 = -1;
static gint ett_rrc_T_criticalExtensions_51 = -1;
static gint ett_rrc_T_r11_05 = -1;
static gint ett_rrc_T_nonCriticalExtensions_65 = -1;
static gint ett_rrc_T_criticalExtensions_52 = -1;
static gint ett_rrc_MeasurementControl_r3_IEs = -1;
static gint ett_rrc_MeasurementControl_v390ext = -1;
static gint ett_rrc_MeasurementControl_v3a0ext = -1;
static gint ett_rrc_MeasurementControl_r4_IEs = -1;
static gint ett_rrc_MeasurementControl_v590ext_IEs = -1;
static gint ett_rrc_T_measurementCommand_v590ext = -1;
static gint ett_rrc_MeasurementControl_v5b0ext_IEs = -1;
static gint ett_rrc_MeasurementControl_r6_IEs = -1;
static gint ett_rrc_MeasurementControl_v6a0ext_IEs = -1;
static gint ett_rrc_MeasurementControl_r7_IEs = -1;
static gint ett_rrc_MeasurementControl_v7b0ext_IEs = -1;
static gint ett_rrc_MeasurementControl_r8_IEs = -1;
static gint ett_rrc_MeasurementControl_v8a0ext_IEs = -1;
static gint ett_rrc_MeasurementControl_r9_IEs = -1;
static gint ett_rrc_MeasurementControl_v970ext_IEs = -1;
static gint ett_rrc_MeasurementControl_r10_IEs = -1;
static gint ett_rrc_MeasurementControl_vaa0ext_IEs = -1;
static gint ett_rrc_MeasurementControl_r11_IEs = -1;
static gint ett_rrc_MeasurementControlFailure = -1;
static gint ett_rrc_T_laterNonCriticalExtensions_19 = -1;
static gint ett_rrc_T_v590NonCriticalExtensions_10 = -1;
static gint ett_rrc_T_nonCriticalExtensions_66 = -1;
static gint ett_rrc_MeasurementControlFailure_v590ext_IEs = -1;
static gint ett_rrc_MeasurementReport = -1;
static gint ett_rrc_T_v390nonCriticalExtensions_01 = -1;
static gint ett_rrc_T_laterNonCriticalExtensions_20 = -1;
static gint ett_rrc_T_v4b0NonCriticalExtensions_06 = -1;
static gint ett_rrc_T_v590NonCriticalExtensions_11 = -1;
static gint ett_rrc_T_v5b0NonCriticalExtensions_02 = -1;
static gint ett_rrc_T_v690NonCriticalExtensions_11 = -1;
static gint ett_rrc_T_v770NonCriticalExtensions_03 = -1;
static gint ett_rrc_T_v860NonCriticalExtensions_08 = -1;
static gint ett_rrc_T_v920NonCriticalExtensions_01 = -1;
static gint ett_rrc_T_v970NonCriticalExtensions_01 = -1;
static gint ett_rrc_T_va40NonCriticalExtensions_04 = -1;
static gint ett_rrc_T_vbxyNonCriticalExtensions_07 = -1;
static gint ett_rrc_T_nonCriticalExtensions_67 = -1;
static gint ett_rrc_MeasurementReport_v390ext = -1;
static gint ett_rrc_MeasurementReport_v4b0ext_IEs = -1;
static gint ett_rrc_MeasurementReport_v590ext_IEs = -1;
static gint ett_rrc_MeasurementReport_v5b0ext_IEs = -1;
static gint ett_rrc_MeasurementReport_v690ext_IEs = -1;
static gint ett_rrc_MeasurementReport_v770ext_IEs = -1;
static gint ett_rrc_MeasurementReport_v860ext_IEs = -1;
static gint ett_rrc_MeasurementReport_v920ext_IEs = -1;
static gint ett_rrc_MeasurementReport_v970ext_IEs = -1;
static gint ett_rrc_MeasurementReport_va40ext_IEs = -1;
static gint ett_rrc_MeasurementReport_vbxyext_IEs = -1;
static gint ett_rrc_PagingType1 = -1;
static gint ett_rrc_T_laterNonCriticalExtensions_21 = -1;
static gint ett_rrc_T_v590NonCriticalExtensions_12 = -1;
static gint ett_rrc_T_v860NonCriticalExtensions_09 = -1;
static gint ett_rrc_T_nonCriticalExtensions_68 = -1;
static gint ett_rrc_PagingType1_v590ext_IEs = -1;
static gint ett_rrc_PagingType1_v860ext_IEs = -1;
static gint ett_rrc_PagingType2 = -1;
static gint ett_rrc_T_laterNonCriticalExtensions_22 = -1;
static gint ett_rrc_T_nonCriticalExtensions_69 = -1;
static gint ett_rrc_PhysicalChannelReconfiguration = -1;
static gint ett_rrc_T_r3_12 = -1;
static gint ett_rrc_T_v3a0NonCriticalExtensions_05 = -1;
static gint ett_rrc_T_laterNonCriticalExtensions_23 = -1;
static gint ett_rrc_T_v4b0NonCriticalExtenstions = -1;
static gint ett_rrc_T_v590NonCriticalExtenstions_02 = -1;
static gint ett_rrc_T_v690NonCriticalExtensions_12 = -1;
static gint ett_rrc_T_v770NonCriticalExtensions_04 = -1;
static gint ett_rrc_T_nonCriticalExtensions_70 = -1;
static gint ett_rrc_T_later_than_r3_10 = -1;
static gint ett_rrc_T_criticalExtensions_53 = -1;
static gint ett_rrc_T_r4_04 = -1;
static gint ett_rrc_T_v4d0NonCriticalExtensions_04 = -1;
static gint ett_rrc_T_v590NonCriticalExtenstions_03 = -1;
static gint ett_rrc_T_v690NonCriticalExtensions_13 = -1;
static gint ett_rrc_T_v770NonCriticalExtensions_05 = -1;
static gint ett_rrc_T_nonCriticalExtensions_71 = -1;
static gint ett_rrc_T_criticalExtensions_54 = -1;
static gint ett_rrc_T_r5_04 = -1;
static gint ett_rrc_T_v690NonCriticalExtensions_14 = -1;
static gint ett_rrc_T_v770NonCriticalExtensions_06 = -1;
static gint ett_rrc_T_nonCriticalExtensions_72 = -1;
static gint ett_rrc_T_criticalExtensions_55 = -1;
static gint ett_rrc_T_r6_06 = -1;
static gint ett_rrc_T_v6b0NonCriticalExtensions_05 = -1;
static gint ett_rrc_T_v770NonCriticalExtensions_07 = -1;
static gint ett_rrc_T_nonCriticalExtensions_73 = -1;
static gint ett_rrc_T_criticalExtensions_56 = -1;
static gint ett_rrc_T_r7_05 = -1;
static gint ett_rrc_T_v780NonCriticalExtensions_04 = -1;
static gint ett_rrc_T_v7d0NonCriticalExtensions_04 = -1;
static gint ett_rrc_T_v7f0NonCriticalExtensions_05 = -1;
static gint ett_rrc_T_v7g0NonCriticalExtensions_06 = -1;
static gint ett_rrc_T_nonCriticalExtensions_74 = -1;
static gint ett_rrc_T_criticalExtensions_57 = -1;
static gint ett_rrc_T_r8_06 = -1;
static gint ett_rrc_T_v7d0NonCriticalExtensions_05 = -1;
static gint ett_rrc_T_v7f0NonCriticalExtensions_06 = -1;
static gint ett_rrc_T_v890NonCriticalExtensions_04 = -1;
static gint ett_rrc_T_v7g0NonCriticalExtensions_07 = -1;
static gint ett_rrc_T_v8a0NonCriticalExtensions_04 = -1;
static gint ett_rrc_T_nonCriticalExtensions_75 = -1;
static gint ett_rrc_T_criticalExtensions_58 = -1;
static gint ett_rrc_T_r9_05 = -1;
static gint ett_rrc_T_v950NonCriticalExtensions_03 = -1;
static gint ett_rrc_T_nonCriticalExtensions_76 = -1;
static gint ett_rrc_T_criticalExtensions_59 = -1;
static gint ett_rrc_T_r10_05 = -1;
static gint ett_rrc_T_vbxyNonCriticalExtensions_08 = -1;
static gint ett_rrc_T_nonCriticalExtensions_77 = -1;
static gint ett_rrc_T_criticalExtensions_60 = -1;
static gint ett_rrc_T_r11_06 = -1;
static gint ett_rrc_T_nonCriticalExtensions_78 = -1;
static gint ett_rrc_T_criticalExtensions_61 = -1;
static gint ett_rrc_PhysicalChannelReconfiguration_r3_IEs = -1;
static gint ett_rrc_T_modeSpecificInfo_12 = -1;
static gint ett_rrc_T_fdd_18 = -1;
static gint ett_rrc_PhysicalChannelReconfiguration_v3a0ext = -1;
static gint ett_rrc_PhysicalChannelReconfiguration_v4b0ext_IEs = -1;
static gint ett_rrc_PhysicalChannelReconfiguration_v590ext_IEs = -1;
static gint ett_rrc_PhysicalChannelReconfiguration_r4_IEs = -1;
static gint ett_rrc_T_modeSpecificInfo_13 = -1;
static gint ett_rrc_T_fdd_19 = -1;
static gint ett_rrc_PhysicalChannelReconfiguration_r5_IEs = -1;
static gint ett_rrc_T_modeSpecificInfo_14 = -1;
static gint ett_rrc_T_fdd_20 = -1;
static gint ett_rrc_PhysicalChannelReconfiguration_r6_IEs = -1;
static gint ett_rrc_PhysicalChannelReconfiguration_v690ext_IEs = -1;
static gint ett_rrc_PhysicalChannelReconfiguration_v6b0ext_IEs = -1;
static gint ett_rrc_PhysicalChannelReconfiguration_r7_IEs = -1;
static gint ett_rrc_PhysicalChannelReconfiguration_v770ext_IEs = -1;
static gint ett_rrc_PhysicalChannelReconfiguration_v780ext_IEs = -1;
static gint ett_rrc_PhysicalChannelReconfiguration_v7d0ext_IEs = -1;
static gint ett_rrc_PhysicalChannelReconfiguration_v7f0ext_IEs = -1;
static gint ett_rrc_PhysicalChannelReconfiguration_v7g0ext_IEs = -1;
static gint ett_rrc_PhysicalChannelReconfiguration_r8_IEs = -1;
static gint ett_rrc_PhysicalChannelReconfiguration_v890ext_IEs = -1;
static gint ett_rrc_PhysicalChannelReconfiguration_v8a0ext_IEs = -1;
static gint ett_rrc_PhysicalChannelReconfiguration_r9_IEs = -1;
static gint ett_rrc_PhysicalChannelReconfiguration_v950ext_IEs = -1;
static gint ett_rrc_PhysicalChannelReconfiguration_r10_IEs = -1;
static gint ett_rrc_PhysicalChannelReconfiguration_vbxyext_IEs = -1;
static gint ett_rrc_PhysicalChannelReconfiguration_r11_IEs = -1;
static gint ett_rrc_PhysicalChannelReconfigurationComplete = -1;
static gint ett_rrc_T_laterNonCriticalExtensions_24 = -1;
static gint ett_rrc_T_v770NonCriticalExtensions_08 = -1;
static gint ett_rrc_T_nonCriticalExtensions_79 = -1;
static gint ett_rrc_PhysicalChannelReconfigurationComplete_v770ext_IEs = -1;
static gint ett_rrc_PhysicalChannelReconfigurationFailure = -1;
static gint ett_rrc_T_laterNonCriticalExtensions_25 = -1;
static gint ett_rrc_T_nonCriticalExtensions_80 = -1;
static gint ett_rrc_PhysicalSharedChannelAllocation = -1;
static gint ett_rrc_T_r3_13 = -1;
static gint ett_rrc_T_laterNonCriticalExtensions_26 = -1;
static gint ett_rrc_T_nonCriticalExtensions_81 = -1;
static gint ett_rrc_T_later_than_r3_11 = -1;
static gint ett_rrc_T_criticalExtensions_62 = -1;
static gint ett_rrc_T_r4_05 = -1;
static gint ett_rrc_T_v4d0NonCriticalExtensions_05 = -1;
static gint ett_rrc_T_v690NonCriticalExtensions_15 = -1;
static gint ett_rrc_T_v770NonCriticalExtensions_09 = -1;
static gint ett_rrc_T_nonCriticalExtensions_82 = -1;
static gint ett_rrc_T_criticalExtensions_63 = -1;
static gint ett_rrc_PhysicalSharedChannelAllocation_r3_IEs = -1;
static gint ett_rrc_PhysicalSharedChannelAllocation_r4_IEs = -1;
static gint ett_rrc_PhysicalSharedChannelAllocation_v690ext_IEs = -1;
static gint ett_rrc_PhysicalSharedChannelAllocation_v770ext_IEs = -1;
static gint ett_rrc_PUSCHCapacityRequest = -1;
static gint ett_rrc_T_allocationConfirmation = -1;
static gint ett_rrc_T_laterNonCriticalExtensions_27 = -1;
static gint ett_rrc_T_v590NonCriticalExtensions_13 = -1;
static gint ett_rrc_T_nonCriticalExtensions_83 = -1;
static gint ett_rrc_PUSCHCapacityRequest_v590ext = -1;
static gint ett_rrc_RadioBearerReconfiguration = -1;
static gint ett_rrc_T_r3_14 = -1;
static gint ett_rrc_T_v3aoNonCriticalExtensions = -1;
static gint ett_rrc_T_laterNonCriticalExtensions_28 = -1;
static gint ett_rrc_T_v4b0NonCriticalExtensions_07 = -1;
static gint ett_rrc_T_v590NonCriticalExtensions_14 = -1;
static gint ett_rrc_T_v5d0NonCriticalExtenstions_06 = -1;
static gint ett_rrc_T_v690NonCriticalExtensions_16 = -1;
static gint ett_rrc_T_v770NonCriticalExtensions_10 = -1;
static gint ett_rrc_T_nonCriticalExtensions_84 = -1;
static gint ett_rrc_T_later_than_r3_12 = -1;
static gint ett_rrc_T_criticalExtensions_64 = -1;
static gint ett_rrc_T_r4_06 = -1;
static gint ett_rrc_T_v4d0NonCriticalExtensions_06 = -1;
static gint ett_rrc_T_v590NonCriticalExtensions_15 = -1;
static gint ett_rrc_T_v5d0NonCriticalExtenstions_07 = -1;
static gint ett_rrc_T_v690NonCriticalExtensions_17 = -1;
static gint ett_rrc_T_v770NonCriticalExtensions_11 = -1;
static gint ett_rrc_T_nonCriticalExtensions_85 = -1;
static gint ett_rrc_T_criticalExtensions_65 = -1;
static gint ett_rrc_T_r5_05 = -1;
static gint ett_rrc_T_v5d0NonCriticalExtenstions_08 = -1;
static gint ett_rrc_T_v690NonCriticalExtensions_18 = -1;
static gint ett_rrc_T_v770NonCriticalExtensions_12 = -1;
static gint ett_rrc_T_nonCriticalExtensions_86 = -1;
static gint ett_rrc_T_criticalExtensions_66 = -1;
static gint ett_rrc_T_r6_07 = -1;
static gint ett_rrc_T_v6b0NonCriticalExtensions_06 = -1;
static gint ett_rrc_T_v6f0NonCriticalExtensions = -1;
static gint ett_rrc_T_v770NonCriticalExtensions_13 = -1;
static gint ett_rrc_T_nonCriticalExtensions_87 = -1;
static gint ett_rrc_T_criticalExtensions_67 = -1;
static gint ett_rrc_T_r7_06 = -1;
static gint ett_rrc_T_v780NonCriticalExtensions_05 = -1;
static gint ett_rrc_T_v790NonCriticalExtensions = -1;
static gint ett_rrc_T_v7d0NonCriticalExtensions_06 = -1;
static gint ett_rrc_T_v7f0NonCriticalExtensions_07 = -1;
static gint ett_rrc_T_v7g0NonCriticalExtensions_08 = -1;
static gint ett_rrc_T_nonCriticalExtensions_88 = -1;
static gint ett_rrc_T_criticalExtensions_68 = -1;
static gint ett_rrc_T_r8_07 = -1;
static gint ett_rrc_T_v7d0NonCriticalExtensions_07 = -1;
static gint ett_rrc_T_v7f0NonCriticalExtensions_08 = -1;
static gint ett_rrc_T_v890NonCriticalExtensions_05 = -1;
static gint ett_rrc_T_v7g0NonCriticalExtensions_09 = -1;
static gint ett_rrc_T_v8a0NonCriticalExtensions_05 = -1;
static gint ett_rrc_T_nonCriticalExtensions_89 = -1;
static gint ett_rrc_T_criticalExtensions_69 = -1;
static gint ett_rrc_T_r9_06 = -1;
static gint ett_rrc_T_v950NonCriticalExtensions_04 = -1;
static gint ett_rrc_T_v9c0NonCriticalExtensions_04 = -1;
static gint ett_rrc_T_nonCriticalExtensions_90 = -1;
static gint ett_rrc_T_criticalExtensions_70 = -1;
static gint ett_rrc_T_r10_06 = -1;
static gint ett_rrc_T_v9c0NonCriticalExtensions_05 = -1;
static gint ett_rrc_T_vaa0NonCriticalExtensions_01 = -1;
static gint ett_rrc_T_vbxyNonCriticalExtensions_09 = -1;
static gint ett_rrc_T_nonCriticalExtensions_91 = -1;
static gint ett_rrc_T_criticalExtensions_71 = -1;
static gint ett_rrc_T_r11_07 = -1;
static gint ett_rrc_T_nonCriticalExtensions_92 = -1;
static gint ett_rrc_T_criticalExtensions_72 = -1;
static gint ett_rrc_RadioBearerReconfiguration_vaa0ext_IEs = -1;
static gint ett_rrc_RadioBearerReconfiguration_r3_IEs = -1;
static gint ett_rrc_T_dummy_01 = -1;
static gint ett_rrc_T_fdd_21 = -1;
static gint ett_rrc_T_modeSpecificPhysChInfo_03 = -1;
static gint ett_rrc_T_fdd_22 = -1;
static gint ett_rrc_RadioBearerReconfiguration_v3a0ext = -1;
static gint ett_rrc_RadioBearerReconfiguration_v4b0ext_IEs = -1;
static gint ett_rrc_RadioBearerReconfiguration_v590ext_IEs = -1;
static gint ett_rrc_RadioBearerReconfiguration_v5d0ext_IEs = -1;
static gint ett_rrc_RadioBearerReconfiguration_r4_IEs = -1;
static gint ett_rrc_T_dummy_02 = -1;
static gint ett_rrc_T_fdd_23 = -1;
static gint ett_rrc_T_modeSpecificPhysChInfo_04 = -1;
static gint ett_rrc_T_fdd_24 = -1;
static gint ett_rrc_RadioBearerReconfiguration_r5_IEs = -1;
static gint ett_rrc_T_specificationMode_09 = -1;
static gint ett_rrc_T_complete_09 = -1;
static gint ett_rrc_T_dummy_03 = -1;
static gint ett_rrc_T_fdd_25 = -1;
static gint ett_rrc_T_preconfiguration_09 = -1;
static gint ett_rrc_T_preConfigMode_09 = -1;
static gint ett_rrc_T_defaultConfig_09 = -1;
static gint ett_rrc_T_modeSpecificPhysChInfo_05 = -1;
static gint ett_rrc_T_fdd_26 = -1;
static gint ett_rrc_RadioBearerReconfiguration_r6_IEs = -1;
static gint ett_rrc_T_specificationMode_10 = -1;
static gint ett_rrc_T_complete_10 = -1;
static gint ett_rrc_T_preconfiguration_10 = -1;
static gint ett_rrc_T_preConfigMode_10 = -1;
static gint ett_rrc_T_defaultConfig_10 = -1;
static gint ett_rrc_RadioBearerReconfiguration_v690ext_IEs = -1;
static gint ett_rrc_RadioBearerReconfiguration_v6b0ext_IEs = -1;
static gint ett_rrc_RadioBearerReconfiguration_v6f0ext_IEs = -1;
static gint ett_rrc_T_specificationMode_11 = -1;
static gint ett_rrc_T_complete_11 = -1;
static gint ett_rrc_RadioBearerReconfiguration_r7_IEs = -1;
static gint ett_rrc_T_specificationMode_12 = -1;
static gint ett_rrc_T_complete_12 = -1;
static gint ett_rrc_T_preconfiguration_11 = -1;
static gint ett_rrc_T_preConfigMode_11 = -1;
static gint ett_rrc_T_defaultConfig_11 = -1;
static gint ett_rrc_RadioBearerReconfiguration_v770ext_IEs = -1;
static gint ett_rrc_RadioBearerReconfiguration_v780ext_IEs = -1;
static gint ett_rrc_RadioBearerReconfiguration_v790ext_IEs = -1;
static gint ett_rrc_RadioBearerReconfiguration_v7d0ext_IEs = -1;
static gint ett_rrc_RadioBearerReconfiguration_v7f0ext_IEs = -1;
static gint ett_rrc_RadioBearerReconfiguration_v7g0ext_IEs = -1;
static gint ett_rrc_RadioBearerReconfiguration_r8_IEs = -1;
static gint ett_rrc_T_specificationMode_13 = -1;
static gint ett_rrc_T_complete_13 = -1;
static gint ett_rrc_T_preconfiguration_12 = -1;
static gint ett_rrc_T_preConfigMode_12 = -1;
static gint ett_rrc_T_defaultConfig_12 = -1;
static gint ett_rrc_RadioBearerReconfiguration_v890ext_IEs = -1;
static gint ett_rrc_RadioBearerReconfiguration_v8a0ext_IEs = -1;
static gint ett_rrc_RadioBearerReconfiguration_r9_IEs = -1;
static gint ett_rrc_T_specificationMode_14 = -1;
static gint ett_rrc_T_complete_14 = -1;
static gint ett_rrc_T_preconfiguration_13 = -1;
static gint ett_rrc_T_preConfigMode_13 = -1;
static gint ett_rrc_T_defaultConfig_13 = -1;
static gint ett_rrc_RadioBearerReconfiguration_v950ext_IEs = -1;
static gint ett_rrc_RadioBearerReconfiguration_v9c0ext_IEs = -1;
static gint ett_rrc_RadioBearerReconfiguration_r10_IEs = -1;
static gint ett_rrc_T_specificationMode_15 = -1;
static gint ett_rrc_T_complete_15 = -1;
static gint ett_rrc_T_preconfiguration_14 = -1;
static gint ett_rrc_T_preConfigMode_14 = -1;
static gint ett_rrc_T_defaultConfig_14 = -1;
static gint ett_rrc_RadioBearerReconfiguration_vbxyext_IEs = -1;
static gint ett_rrc_RadioBearerReconfiguration_r11_IEs = -1;
static gint ett_rrc_T_specificationMode_16 = -1;
static gint ett_rrc_T_complete_16 = -1;
static gint ett_rrc_T_preconfiguration_15 = -1;
static gint ett_rrc_T_preConfigMode_15 = -1;
static gint ett_rrc_T_defaultConfig_15 = -1;
static gint ett_rrc_RadioBearerReconfigurationComplete = -1;
static gint ett_rrc_T_laterNonCriticalExtensions_29 = -1;
static gint ett_rrc_T_v770NonCriticalExtensions_14 = -1;
static gint ett_rrc_T_nonCriticalExtensions_93 = -1;
static gint ett_rrc_RadioBearerReconfigurationComplete_v770ext_IEs = -1;
static gint ett_rrc_RadioBearerReconfigurationFailure = -1;
static gint ett_rrc_T_laterNonCriticalExtensions_30 = -1;
static gint ett_rrc_T_nonCriticalExtensions_94 = -1;
static gint ett_rrc_RadioBearerRelease = -1;
static gint ett_rrc_T_r3_15 = -1;
static gint ett_rrc_T_v3a0NonCriticalExtensions_06 = -1;
static gint ett_rrc_T_laterNonCriticalExtensions_31 = -1;
static gint ett_rrc_T_v4b0NonCriticalExtensions_08 = -1;
static gint ett_rrc_T_v590NonCriticalExtensions_16 = -1;
static gint ett_rrc_T_v690NonCriticalExtensions_19 = -1;
static gint ett_rrc_T_v770NonCriticalExtensions_15 = -1;
static gint ett_rrc_T_nonCriticalExtensions_95 = -1;
static gint ett_rrc_T_later_than_r3_13 = -1;
static gint ett_rrc_T_criticalExtensions_73 = -1;
static gint ett_rrc_T_r4_07 = -1;
static gint ett_rrc_T_v4d0NonCriticalExtensions_07 = -1;
static gint ett_rrc_T_v590NonCriticalExtensions_17 = -1;
static gint ett_rrc_T_v690NonCriticalExtensions_20 = -1;
static gint ett_rrc_T_v770NonCriticalExtensions_16 = -1;
static gint ett_rrc_T_nonCriticalExtensions_96 = -1;
static gint ett_rrc_T_criticalExtensions_74 = -1;
static gint ett_rrc_T_r5_06 = -1;
static gint ett_rrc_T_v690NonCriticalExtensions_21 = -1;
static gint ett_rrc_T_v770NonCriticalExtensions_17 = -1;
static gint ett_rrc_T_nonCriticalExtensions_97 = -1;
static gint ett_rrc_T_criticalExtensions_75 = -1;
static gint ett_rrc_T_r6_08 = -1;
static gint ett_rrc_T_v6b0NonCriticalExtensions_07 = -1;
static gint ett_rrc_T_v770NonCriticalExtensions_18 = -1;
static gint ett_rrc_T_nonCriticalExtensions_98 = -1;
static gint ett_rrc_T_criticalExtensions_76 = -1;
static gint ett_rrc_T_r7_07 = -1;
static gint ett_rrc_T_v780NonCriticalExtensions_06 = -1;
static gint ett_rrc_T_v7d0NonCriticalExtensions_08 = -1;
static gint ett_rrc_T_v7f0NonCriticalExtensions_09 = -1;
static gint ett_rrc_T_v7g0NonCriticalExtensions_10 = -1;
static gint ett_rrc_T_nonCriticalExtensions_99 = -1;
static gint ett_rrc_T_criticalExtensions_77 = -1;
static gint ett_rrc_T_r8_08 = -1;
static gint ett_rrc_T_v7d0NonCriticalExtensions_09 = -1;
static gint ett_rrc_T_v7f0NonCriticalExtensions_10 = -1;
static gint ett_rrc_T_v890NonCriticalExtensions_06 = -1;
static gint ett_rrc_T_v7g0NonCriticalExtensions_11 = -1;
static gint ett_rrc_T_v8a0NonCriticalExtensions_06 = -1;
static gint ett_rrc_T_nonCriticalExtensions_100 = -1;
static gint ett_rrc_T_criticalExtensions_78 = -1;
static gint ett_rrc_T_r9_07 = -1;
static gint ett_rrc_T_v950NonCriticalExtensions_05 = -1;
static gint ett_rrc_T_v970NonCriticalExtensions_02 = -1;
static gint ett_rrc_T_v9c0NonCriticalExtensions_06 = -1;
static gint ett_rrc_T_nonCriticalExtensions_101 = -1;
static gint ett_rrc_T_criticalExtensions_79 = -1;
static gint ett_rrc_T_r10_07 = -1;
static gint ett_rrc_T_v9c0NonCriticalExtensions_07 = -1;
static gint ett_rrc_T_vbxyNonCriticalExtensions_10 = -1;
static gint ett_rrc_T_nonCriticalExtensions_102 = -1;
static gint ett_rrc_T_criticalExtensions_80 = -1;
static gint ett_rrc_T_r11_08 = -1;
static gint ett_rrc_T_nonCriticalExtensions_103 = -1;
static gint ett_rrc_T_criticalExtensions_81 = -1;
static gint ett_rrc_RadioBearerRelease_r3_IEs = -1;
static gint ett_rrc_T_dummy_04 = -1;
static gint ett_rrc_T_fdd_27 = -1;
static gint ett_rrc_T_modeSpecificPhysChInfo_06 = -1;
static gint ett_rrc_T_fdd_28 = -1;
static gint ett_rrc_RadioBearerRelease_v3a0ext = -1;
static gint ett_rrc_RadioBearerRelease_v4b0ext_IEs = -1;
static gint ett_rrc_RadioBearerRelease_v590ext_IEs = -1;
static gint ett_rrc_RadioBearerRelease_r4_IEs = -1;
static gint ett_rrc_T_dummy_05 = -1;
static gint ett_rrc_T_fdd_29 = -1;
static gint ett_rrc_T_modeSpecificPhysChInfo_07 = -1;
static gint ett_rrc_T_fdd_30 = -1;
static gint ett_rrc_RadioBearerRelease_r5_IEs = -1;
static gint ett_rrc_T_dummy_06 = -1;
static gint ett_rrc_T_fdd_31 = -1;
static gint ett_rrc_T_modeSpecificPhysChInfo_08 = -1;
static gint ett_rrc_T_fdd_32 = -1;
static gint ett_rrc_RadioBearerRelease_v690ext_IEs = -1;
static gint ett_rrc_RadioBearerRelease_r6_IEs = -1;
static gint ett_rrc_RadioBearerRelease_v6b0ext_IEs = -1;
static gint ett_rrc_RadioBearerRelease_r7_IEs = -1;
static gint ett_rrc_RadioBearerRelease_v770ext_IEs = -1;
static gint ett_rrc_RadioBearerRelease_v780ext_IEs = -1;
static gint ett_rrc_RadioBearerRelease_v7d0ext_IEs = -1;
static gint ett_rrc_RadioBearerRelease_v7f0ext_IEs = -1;
static gint ett_rrc_RadioBearerRelease_v7g0ext_IEs = -1;
static gint ett_rrc_RadioBearerRelease_r8_IEs = -1;
static gint ett_rrc_RadioBearerRelease_v890ext_IEs = -1;
static gint ett_rrc_RadioBearerRelease_v8a0ext_IEs = -1;
static gint ett_rrc_RadioBearerRelease_r9_IEs = -1;
static gint ett_rrc_RadioBearerRelease_v950ext_IEs = -1;
static gint ett_rrc_RadioBearerRelease_v970ext_IEs = -1;
static gint ett_rrc_RadioBearerRelease_v9c0ext_IEs = -1;
static gint ett_rrc_RadioBearerRelease_r10_IEs = -1;
static gint ett_rrc_RadioBearerRelease_vbxyext_IEs = -1;
static gint ett_rrc_RadioBearerRelease_r11_IEs = -1;
static gint ett_rrc_RadioBearerReleaseComplete = -1;
static gint ett_rrc_T_laterNonCriticalExtensions_32 = -1;
static gint ett_rrc_T_v770NonCriticalExtensions_19 = -1;
static gint ett_rrc_T_nonCriticalExtensions_104 = -1;
static gint ett_rrc_RadioBearerReleaseComplete_v770ext_IEs = -1;
static gint ett_rrc_RadioBearerReleaseFailure = -1;
static gint ett_rrc_T_laterNonCriticalExtensions_33 = -1;
static gint ett_rrc_T_nonCriticalExtensions_105 = -1;
static gint ett_rrc_RadioBearerSetup = -1;
static gint ett_rrc_T_r3_16 = -1;
static gint ett_rrc_T_v3a0NonCriticalExtensions_07 = -1;
static gint ett_rrc_T_laterNonCriticalExtensions_34 = -1;
static gint ett_rrc_T_v4b0NonCriticalExtensions_09 = -1;
static gint ett_rrc_T_v590NonCriticalExtensions_18 = -1;
static gint ett_rrc_T_v5d0NonCriticalExtenstions_09 = -1;
static gint ett_rrc_T_v690NonCriticalExtensions_22 = -1;
static gint ett_rrc_T_nonCriticalExtensions_106 = -1;
static gint ett_rrc_T_later_than_r3_14 = -1;
static gint ett_rrc_T_criticalExtensions_82 = -1;
static gint ett_rrc_T_r4_08 = -1;
static gint ett_rrc_T_v4d0NonCriticalExtensions_08 = -1;
static gint ett_rrc_T_v590NonCriticalExtensions_19 = -1;
static gint ett_rrc_T_v5d0NonCriticalExtenstions_10 = -1;
static gint ett_rrc_T_v690NonCriticalExtensions_23 = -1;
static gint ett_rrc_T_nonCriticalExtensions_107 = -1;
static gint ett_rrc_T_criticalExtensions_83 = -1;
static gint ett_rrc_T_r5_07 = -1;
static gint ett_rrc_T_v5d0NonCriticalExtenstions_11 = -1;
static gint ett_rrc_T_v690NonCriticalExtensions_24 = -1;
static gint ett_rrc_T_nonCriticalExtensions_108 = -1;
static gint ett_rrc_T_criticalExtensions_84 = -1;
static gint ett_rrc_T_r6_09 = -1;
static gint ett_rrc_T_v6b0NonCriticalExtensions_08 = -1;
static gint ett_rrc_T_nonCriticalExtensions_109 = -1;
static gint ett_rrc_T_criticalExtensions_85 = -1;
static gint ett_rrc_T_r7_08 = -1;
static gint ett_rrc_T_v780NonCriticalExtensions_07 = -1;
static gint ett_rrc_T_v820NonCriticalExtensions_01 = -1;
static gint ett_rrc_T_nonCriticalExtensions_110 = -1;
static gint ett_rrc_T_criticalExtensions_86 = -1;
static gint ett_rrc_T_r8_09 = -1;
static gint ett_rrc_T_v7d0NonCriticalExtensions_10 = -1;
static gint ett_rrc_T_v7f0NonCriticalExtensions_11 = -1;
static gint ett_rrc_T_v890NonCriticalExtensions_07 = -1;
static gint ett_rrc_T_v7g0NonCriticalExtensions_12 = -1;
static gint ett_rrc_T_v8a0NonCriticalExtensions_07 = -1;
static gint ett_rrc_T_nonCriticalExtensions_111 = -1;
static gint ett_rrc_T_criticalExtensions_87 = -1;
static gint ett_rrc_T_r9_08 = -1;
static gint ett_rrc_T_v950NonCriticalExtensions_06 = -1;
static gint ett_rrc_T_v9c0NonCriticalExtensions_08 = -1;
static gint ett_rrc_T_nonCriticalExtensions_112 = -1;
static gint ett_rrc_T_criticalExtensions_88 = -1;
static gint ett_rrc_T_r10_08 = -1;
static gint ett_rrc_T_v9c0NonCriticalExtensions_09 = -1;
static gint ett_rrc_T_vbxyNonCriticalExtensions_11 = -1;
static gint ett_rrc_T_nonCriticalExtensions_113 = -1;
static gint ett_rrc_T_criticalExtensions_89 = -1;
static gint ett_rrc_T_r11_09 = -1;
static gint ett_rrc_T_nonCriticalExtensions_114 = -1;
static gint ett_rrc_T_criticalExtensions_90 = -1;
static gint ett_rrc_RadioBearerSetup_r3_IEs = -1;
static gint ett_rrc_T_dummy_07 = -1;
static gint ett_rrc_T_fdd_33 = -1;
static gint ett_rrc_T_modeSpecificPhysChInfo_09 = -1;
static gint ett_rrc_T_fdd_34 = -1;
static gint ett_rrc_RadioBearerSetup_v3a0ext = -1;
static gint ett_rrc_RadioBearerSetup_v4b0ext_IEs = -1;
static gint ett_rrc_RadioBearerSetup_v590ext_IEs = -1;
static gint ett_rrc_RadioBearerSetup_v5d0ext_IEs = -1;
static gint ett_rrc_RadioBearerSetup_r4_IEs = -1;
static gint ett_rrc_T_dummy_08 = -1;
static gint ett_rrc_T_fdd_35 = -1;
static gint ett_rrc_T_modeSpecificPhysChInfo_10 = -1;
static gint ett_rrc_T_fdd_36 = -1;
static gint ett_rrc_RadioBearerSetup_r5_IEs = -1;
static gint ett_rrc_T_dummy_09 = -1;
static gint ett_rrc_T_fdd_37 = -1;
static gint ett_rrc_T_modeSpecificPhysChInfo_11 = -1;
static gint ett_rrc_T_fdd_38 = -1;
static gint ett_rrc_RadioBearerSetup_v690ext_IEs = -1;
static gint ett_rrc_RadioBearerSetup_r6_IEs = -1;
static gint ett_rrc_T_specificationMode_17 = -1;
static gint ett_rrc_T_complete_17 = -1;
static gint ett_rrc_T_dummy_10 = -1;
static gint ett_rrc_RadioBearerSetup_v6b0ext_IEs = -1;
static gint ett_rrc_RadioBearerSetup_r7_IEs = -1;
static gint ett_rrc_T_specificationMode_18 = -1;
static gint ett_rrc_T_complete_18 = -1;
static gint ett_rrc_RadioBearerSetup_r7_add_ext_IEs = -1;
static gint ett_rrc_T_v7f0NonCriticalExtensions_12 = -1;
static gint ett_rrc_T_v7g0NonCriticalExtensions_13 = -1;
static gint ett_rrc_T_nonCriticalExtensions_115 = -1;
static gint ett_rrc_RadioBearerSetup_v780ext_IEs = -1;
static gint ett_rrc_RadioBearerSetup_v7d0ext_IEs = -1;
static gint ett_rrc_RadioBearerSetup_v7f0ext_IEs = -1;
static gint ett_rrc_RadioBearerSetup_v7g0ext_IEs = -1;
static gint ett_rrc_RadioBearerSetup_r8_IEs = -1;
static gint ett_rrc_RadioBearerSetup_v820ext_IEs = -1;
static gint ett_rrc_RadioBearerSetup_v890ext_IEs = -1;
static gint ett_rrc_RadioBearerSetup_v8a0ext_IEs = -1;
static gint ett_rrc_RadioBearerSetup_r9_IEs = -1;
static gint ett_rrc_RadioBearerSetup_v950ext_IEs = -1;
static gint ett_rrc_RadioBearerSetup_v9c0ext_IEs = -1;
static gint ett_rrc_RadioBearerSetup_r10_IEs = -1;
static gint ett_rrc_RadioBearerSetup_vbxyext_IEs = -1;
static gint ett_rrc_RadioBearerSetup_r11_IEs = -1;
static gint ett_rrc_RadioBearerSetupComplete = -1;
static gint ett_rrc_T_laterNonCriticalExtensions_35 = -1;
static gint ett_rrc_T_v770NonCriticalExtensions_20 = -1;
static gint ett_rrc_T_nonCriticalExtensions_116 = -1;
static gint ett_rrc_RadioBearerSetupComplete_v770ext_IEs = -1;
static gint ett_rrc_RadioBearerSetupFailure = -1;
static gint ett_rrc_T_laterNonCriticalExtensions_36 = -1;
static gint ett_rrc_T_nonCriticalExtensions_117 = -1;
static gint ett_rrc_RRCConnectionReject = -1;
static gint ett_rrc_T_r3_17 = -1;
static gint ett_rrc_T_laterNonCriticalExtensions_37 = -1;
static gint ett_rrc_T_v690NonCriticalExtensions_25 = -1;
static gint ett_rrc_T_v6f0NonCriticalExtensions_01 = -1;
static gint ett_rrc_T_v860NonCriticalExtensions_10 = -1;
static gint ett_rrc_T_va40NonCriticalExtensions_05 = -1;
static gint ett_rrc_T_nonCriticalExtensions_118 = -1;
static gint ett_rrc_T_later_than_r3_15 = -1;
static gint ett_rrc_T_criticalExtensions_91 = -1;
static gint ett_rrc_RRCConnectionReject_r3_IEs = -1;
static gint ett_rrc_RRCConnectionReject_v690ext_IEs = -1;
static gint ett_rrc_RRCConnectionReject_v6f0ext_IEs = -1;
static gint ett_rrc_RRCConnectionReject_v860ext_IEs = -1;
static gint ett_rrc_RRCConnectionReject_va40ext_IEs = -1;
static gint ett_rrc_RRCConnectionRelease = -1;
static gint ett_rrc_T_r3_18 = -1;
static gint ett_rrc_T_laterNonCriticalExtensions_38 = -1;
static gint ett_rrc_T_v690NonCriticalExtensions_26 = -1;
static gint ett_rrc_T_v770NonCriticalExtensions_21 = -1;
static gint ett_rrc_T_v860NonCriticalExtensions_11 = -1;
static gint ett_rrc_T_va40NonCriticalExtensions_06 = -1;
static gint ett_rrc_T_nonCriticalExtensions_119 = -1;
static gint ett_rrc_T_later_than_r3_16 = -1;
static gint ett_rrc_T_criticalExtensions_92 = -1;
static gint ett_rrc_T_r4_09 = -1;
static gint ett_rrc_T_v4d0NonCriticalExtensions_09 = -1;
static gint ett_rrc_T_v690NonCriticalExtensions_27 = -1;
static gint ett_rrc_T_v770NonCriticalExtensions_22 = -1;
static gint ett_rrc_T_v860NonCriticalExtensions_12 = -1;
static gint ett_rrc_T_va40NonCriticalExtensions_07 = -1;
static gint ett_rrc_T_nonCriticalExtensions_120 = -1;
static gint ett_rrc_T_criticalExtensions_93 = -1;
static gint ett_rrc_RRCConnectionRelease_r3_IEs = -1;
static gint ett_rrc_RRCConnectionRelease_r4_IEs = -1;
static gint ett_rrc_RRCConnectionRelease_v690ext_IEs = -1;
static gint ett_rrc_RRCConnectionRelease_v770ext_IEs = -1;
static gint ett_rrc_RRCConnectionRelease_v860ext_IEs = -1;
static gint ett_rrc_RRCConnectionRelease_va40ext_IEs = -1;
static gint ett_rrc_RRCConnectionRelease_CCCH = -1;
static gint ett_rrc_T_r3_19 = -1;
static gint ett_rrc_T_laterNonCriticalExtensions_39 = -1;
static gint ett_rrc_T_v690NonCriticalExtensions_28 = -1;
static gint ett_rrc_T_v860NonCriticalExtensions_13 = -1;
static gint ett_rrc_T_va40NonCriticalExtensions_08 = -1;
static gint ett_rrc_T_nonCriticalExtensions_121 = -1;
static gint ett_rrc_T_later_than_r3_17 = -1;
static gint ett_rrc_T_criticalExtensions_94 = -1;
static gint ett_rrc_T_r4_10 = -1;
static gint ett_rrc_T_v4d0NonCriticalExtensions_10 = -1;
static gint ett_rrc_T_v690NonCriticalExtensions_29 = -1;
static gint ett_rrc_T_v860NonCriticalExtensions_14 = -1;
static gint ett_rrc_T_va40NonCriticalExtensions_09 = -1;
static gint ett_rrc_T_nonCriticalExtensions_122 = -1;
static gint ett_rrc_T_criticalExtensions_95 = -1;
static gint ett_rrc_SEQUENCE_SIZE_1_maxURNTI_Group_OF_GroupReleaseInformation = -1;
static gint ett_rrc_T_criticalExtensions_96 = -1;
static gint ett_rrc_T_r5_08 = -1;
static gint ett_rrc_T_v690NonCriticalExtensions_30 = -1;
static gint ett_rrc_T_v860NonCriticalExtensions_15 = -1;
static gint ett_rrc_T_va40NonCriticalExtensions_10 = -1;
static gint ett_rrc_T_nonCriticalExtensions_123 = -1;
static gint ett_rrc_T_criticalExtensions_97 = -1;
static gint ett_rrc_RRCConnectionRelease_CCCH_r3_IEs = -1;
static gint ett_rrc_RRCConnectionRelease_CCCH_r4_IEs = -1;
static gint ett_rrc_RRCConnectionReleaseComplete = -1;
static gint ett_rrc_T_laterNonCriticalExtensions_40 = -1;
static gint ett_rrc_T_nonCriticalExtensions_124 = -1;
static gint ett_rrc_RRCConnectionRequest = -1;
static gint ett_rrc_T_v3d0NonCriticalExtensions = -1;
static gint ett_rrc_T_v4b0NonCriticalExtensions_10 = -1;
static gint ett_rrc_T_v590NonCriticalExtensions_20 = -1;
static gint ett_rrc_T_v690NonCriticalExtensions_31 = -1;
static gint ett_rrc_T_v6b0NonCriticalExtensions_09 = -1;
static gint ett_rrc_T_v6e0NonCriticalExtensions_01 = -1;
static gint ett_rrc_T_v770NonCriticalExtensions_23 = -1;
static gint ett_rrc_T_v7b0NonCriticalExtensions_01 = -1;
static gint ett_rrc_T_v860NonCriticalExtensions_16 = -1;
static gint ett_rrc_T_v7e0NonCriticalExtensions_01 = -1;
static gint ett_rrc_T_v7g0NonCriticalExtensions_14 = -1;
static gint ett_rrc_T_v920NonCriticalExtensions_02 = -1;
static gint ett_rrc_T_v940NonCriticalExtensions = -1;
static gint ett_rrc_RRCConnectionRequest_v3d0ext_IEs = -1;
static gint ett_rrc_RRCConnectionRequest_v4b0ext_IEs = -1;
static gint ett_rrc_RRCConnectionRequest_v590ext_IEs = -1;
static gint ett_rrc_RRCConnectionRequest_v690ext_IEs = -1;
static gint ett_rrc_T_domainIndicator = -1;
static gint ett_rrc_T_cs_domain = -1;
static gint ett_rrc_RRCConnectionRequest_v6b0ext_IEs = -1;
static gint ett_rrc_RRCConnectionRequest_v6e0ext_IEs = -1;
static gint ett_rrc_RRCConnectionRequest_v770ext_IEs = -1;
static gint ett_rrc_RRCConnectionRequest_v7b0ext_IEs = -1;
static gint ett_rrc_RRCConnectionRequest_v7e0ext_IEs = -1;
static gint ett_rrc_RRCConnectionRequest_v7g0ext_IEs = -1;
static gint ett_rrc_RRCConnectionRequest_v860ext_IEs = -1;
static gint ett_rrc_RRCConnectionRequest_v920ext_IEs = -1;
static gint ett_rrc_RRCConnectionRequest_v940ext_IEs = -1;
static gint ett_rrc_RRCConnectionRequest_v9b0ext_IEs = -1;
static gint ett_rrc_RRCConnectionRequest_NonCriticalExts_va40_IEs = -1;
static gint ett_rrc_T_v9b0NonCriticalExtensions = -1;
static gint ett_rrc_T_vbxyNonCriticalExtensions_12 = -1;
static gint ett_rrc_T_nonCriticalExtensions_125 = -1;
static gint ett_rrc_RRCConnectionRequest_va40ext_IEs = -1;
static gint ett_rrc_RRCConnectionRequest_vbxyext_IEs = -1;
static gint ett_rrc_RRCConnectionSetup = -1;
static gint ett_rrc_T_r3_20 = -1;
static gint ett_rrc_T_laterNonCriticalExtensions_41 = -1;
static gint ett_rrc_T_v4b0NonCriticalExtensions_11 = -1;
static gint ett_rrc_T_v590NonCriticalExtensions_21 = -1;
static gint ett_rrc_T_v690NonCriticalExtensions_32 = -1;
static gint ett_rrc_T_nonCriticalExtensions_126 = -1;
static gint ett_rrc_T_later_than_r3_18 = -1;
static gint ett_rrc_T_criticalExtensions_98 = -1;
static gint ett_rrc_T_r4_11 = -1;
static gint ett_rrc_T_v4d0NonCriticalExtensions_11 = -1;
static gint ett_rrc_T_v590NonCriticalExtensions_22 = -1;
static gint ett_rrc_T_v690NonCriticalExtensions_33 = -1;
static gint ett_rrc_T_nonCriticalExtensions_127 = -1;
static gint ett_rrc_T_criticalExtensions_99 = -1;
static gint ett_rrc_T_r5_09 = -1;
static gint ett_rrc_T_v690NonCriticalExtensions_34 = -1;
static gint ett_rrc_T_nonCriticalExtensions_128 = -1;
static gint ett_rrc_T_criticalExtensions_100 = -1;
static gint ett_rrc_T_r6_10 = -1;
static gint ett_rrc_T_v6b0NonCriticalExtensions_10 = -1;
static gint ett_rrc_T_nonCriticalExtensions_129 = -1;
static gint ett_rrc_T_criticalExtensions_101 = -1;
static gint ett_rrc_T_r7_09 = -1;
static gint ett_rrc_T_v780NonCriticalExtensions_08 = -1;
static gint ett_rrc_T_v7d0NonCriticalExtensions_11 = -1;
static gint ett_rrc_T_nonCriticalExtensions_130 = -1;
static gint ett_rrc_T_criticalExtensions_102 = -1;
static gint ett_rrc_T_r8_10 = -1;
static gint ett_rrc_T_v7d0NonCriticalExtensions_12 = -1;
static gint ett_rrc_T_v890NonCriticalExtensions_08 = -1;
static gint ett_rrc_T_v8a0NonCriticalExtensions_08 = -1;
static gint ett_rrc_T_nonCriticalExtensions_131 = -1;
static gint ett_rrc_T_criticalExtensions_103 = -1;
static gint ett_rrc_T_r9_09 = -1;
static gint ett_rrc_T_v950NonCriticalExtensions_07 = -1;
static gint ett_rrc_T_nonCriticalExtensions_132 = -1;
static gint ett_rrc_T_criticalExtensions_104 = -1;
static gint ett_rrc_T_r10_09 = -1;
static gint ett_rrc_T_nonCriticalExtensions_133 = -1;
static gint ett_rrc_T_criticalExtensions_105 = -1;
static gint ett_rrc_T_r11_10 = -1;
static gint ett_rrc_T_nonCriticalExtensions_134 = -1;
static gint ett_rrc_T_criticalExtensions_106 = -1;
static gint ett_rrc_RRCConnectionSetup_r3_IEs = -1;
static gint ett_rrc_RRCConnectionSetup_v4b0ext_IEs = -1;
static gint ett_rrc_RRCConnectionSetup_v590ext_IEs = -1;
static gint ett_rrc_RRCConnectionSetup_r4_IEs = -1;
static gint ett_rrc_RRCConnectionSetup_r5_IEs = -1;
static gint ett_rrc_T_specificationMode_19 = -1;
static gint ett_rrc_T_complete_19 = -1;
static gint ett_rrc_T_preconfiguration_16 = -1;
static gint ett_rrc_T_preConfigMode_16 = -1;
static gint ett_rrc_T_defaultConfig_16 = -1;
static gint ett_rrc_RRCConnectionSetup_v690ext_IEs = -1;
static gint ett_rrc_RRCConnectionSetup_r6_IEs = -1;
static gint ett_rrc_T_specificationMode_20 = -1;
static gint ett_rrc_T_complete_20 = -1;
static gint ett_rrc_T_preconfiguration_17 = -1;
static gint ett_rrc_T_preConfigMode_17 = -1;
static gint ett_rrc_T_defaultConfig_17 = -1;
static gint ett_rrc_RRCConnectionSetup_v6b0ext_IEs = -1;
static gint ett_rrc_RRCConnectionSetup_r7_IEs = -1;
static gint ett_rrc_T_specificationMode_21 = -1;
static gint ett_rrc_T_complete_21 = -1;
static gint ett_rrc_T_preconfiguration_18 = -1;
static gint ett_rrc_T_preConfigMode_18 = -1;
static gint ett_rrc_T_defaultConfig_18 = -1;
static gint ett_rrc_RRCConnectionSetup_v780ext_IEs = -1;
static gint ett_rrc_RRCConnectionSetup_v7d0ext_IEs = -1;
static gint ett_rrc_RRCConnectionSetup_r8_IEs = -1;
static gint ett_rrc_T_specificationMode_22 = -1;
static gint ett_rrc_T_complete_22 = -1;
static gint ett_rrc_T_preconfiguration_19 = -1;
static gint ett_rrc_T_preConfigMode_19 = -1;
static gint ett_rrc_T_defaultConfig_19 = -1;
static gint ett_rrc_RRCConnectionSetup_v890ext_IEs = -1;
static gint ett_rrc_RRCConnectionSetup_v8a0ext_IEs = -1;
static gint ett_rrc_RRCConnectionSetup_r9_IEs = -1;
static gint ett_rrc_T_specificationMode_23 = -1;
static gint ett_rrc_T_complete_23 = -1;
static gint ett_rrc_T_preconfiguration_20 = -1;
static gint ett_rrc_T_preConfigMode_20 = -1;
static gint ett_rrc_T_defaultConfig_20 = -1;
static gint ett_rrc_RRCConnectionSetup_v950ext_IEs = -1;
static gint ett_rrc_RRCConnectionSetup_r10_IEs = -1;
static gint ett_rrc_T_specificationMode_24 = -1;
static gint ett_rrc_T_complete_24 = -1;
static gint ett_rrc_T_preconfiguration_21 = -1;
static gint ett_rrc_T_preConfigMode_21 = -1;
static gint ett_rrc_T_defaultConfig_21 = -1;
static gint ett_rrc_RRCConnectionSetup_r11_IEs = -1;
static gint ett_rrc_T_specificationMode_25 = -1;
static gint ett_rrc_T_complete_25 = -1;
static gint ett_rrc_T_preconfiguration_22 = -1;
static gint ett_rrc_T_preConfigMode_22 = -1;
static gint ett_rrc_T_defaultConfig_22 = -1;
static gint ett_rrc_RRCConnectionSetupComplete = -1;
static gint ett_rrc_T_v370NonCriticalExtensions = -1;
static gint ett_rrc_T_v380NonCriticalExtensions = -1;
static gint ett_rrc_T_v3a0NonCriticalExtensions_08 = -1;
static gint ett_rrc_T_laterNonCriticalExtensions_42 = -1;
static gint ett_rrc_T_v3g0NonCriticalExtensions_01 = -1;
static gint ett_rrc_T_v4b0NonCriticalExtensions_12 = -1;
static gint ett_rrc_T_v590NonCriticalExtensions_23 = -1;
static gint ett_rrc_T_v5c0NonCriticalExtensions = -1;
static gint ett_rrc_T_v690NonCriticalExtensions_35 = -1;
static gint ett_rrc_T_v770NonCriticalExtensions_24 = -1;
static gint ett_rrc_T_va40NonCriticalExtensions_11 = -1;
static gint ett_rrc_T_vbxyNonCriticalExtensions_13 = -1;
static gint ett_rrc_T_nonCriticalExtensions_135 = -1;
static gint ett_rrc_RRCConnectionSetupComplete_v370ext = -1;
static gint ett_rrc_RRCConnectionSetupComplete_v380ext_IEs = -1;
static gint ett_rrc_RRCConnectionSetupComplete_v3a0ext_IEs = -1;
static gint ett_rrc_RRCConnectionSetupComplete_v3g0ext_IEs = -1;
static gint ett_rrc_RRCConnectionSetupComplete_r3_add_ext_IEs = -1;
static gint ett_rrc_T_v680NonCriticalExtensions = -1;
static gint ett_rrc_T_v7e0NonCriticalExtensions_02 = -1;
static gint ett_rrc_T_v7f0NonCriticalExtensions_13 = -1;
static gint ett_rrc_T_va40NonCriticalExtensions_12 = -1;
static gint ett_rrc_T_nonCriticalExtensions_136 = -1;
static gint ett_rrc_RRCConnectionSetupComplete_v4b0ext_IEs = -1;
static gint ett_rrc_RRCConnectionSetupComplete_v590ext_IEs = -1;
static gint ett_rrc_RRCConnectionSetupComplete_v5c0ext_IEs = -1;
static gint ett_rrc_RRCConnectionSetupComplete_v650ext_IEs = -1;
static gint ett_rrc_RRCConnectionSetupComplete_v680ext_IEs = -1;
static gint ett_rrc_RRCConnectionSetupComplete_v690ext_IEs = -1;
static gint ett_rrc_RRCConnectionSetupComplete_v770ext_IEs = -1;
static gint ett_rrc_RRCConnectionSetupComplete_v7e0ext_IEs = -1;
static gint ett_rrc_RRCConnectionSetupComplete_v7f0ext_IEs = -1;
static gint ett_rrc_RRCConnectionSetupComplete_va40ext_IEs = -1;
static gint ett_rrc_RRCConnectionSetupCompleteBand_va40ext_IEs = -1;
static gint ett_rrc_RRCConnectionSetupComplete_vbxyext_IEs = -1;
static gint ett_rrc_RRC_FailureInfo = -1;
static gint ett_rrc_T_r3_21 = -1;
static gint ett_rrc_T_laterNonCriticalExtensions_43 = -1;
static gint ett_rrc_T_nonCriticalExtensions_137 = -1;
static gint ett_rrc_T_criticalExtensions_107 = -1;
static gint ett_rrc_RRC_FailureInfo_r3_IEs = -1;
static gint ett_rrc_RRCStatus = -1;
static gint ett_rrc_T_laterNonCriticalExtensions_44 = -1;
static gint ett_rrc_T_nonCriticalExtensions_138 = -1;
static gint ett_rrc_SecurityModeCommand = -1;
static gint ett_rrc_T_r3_22 = -1;
static gint ett_rrc_T_laterNonCriticalExtensions_45 = -1;
static gint ett_rrc_T_nonCriticalExtensions_139 = -1;
static gint ett_rrc_T_later_than_r3_19 = -1;
static gint ett_rrc_T_criticalExtensions_108 = -1;
static gint ett_rrc_T_r7_10 = -1;
static gint ett_rrc_T_nonCriticalExtensions_140 = -1;
static gint ett_rrc_T_criticalExtensions_109 = -1;
static gint ett_rrc_SecurityModeCommand_r3_IEs = -1;
static gint ett_rrc_SecurityModeCommand_r7_IEs = -1;
static gint ett_rrc_SecurityModeComplete = -1;
static gint ett_rrc_T_laterNonCriticalExtensions_46 = -1;
static gint ett_rrc_T_nonCriticalExtensions_141 = -1;
static gint ett_rrc_SecurityModeFailure = -1;
static gint ett_rrc_T_laterNonCriticalExtensions_47 = -1;
static gint ett_rrc_T_nonCriticalExtensions_142 = -1;
static gint ett_rrc_SignallingConnectionRelease = -1;
static gint ett_rrc_T_r3_23 = -1;
static gint ett_rrc_T_laterNonCriticalExtensions_48 = -1;
static gint ett_rrc_T_va40NonCriticalExtensions_13 = -1;
static gint ett_rrc_T_nonCriticalExtensions_143 = -1;
static gint ett_rrc_T_later_than_r3_20 = -1;
static gint ett_rrc_T_criticalExtensions_110 = -1;
static gint ett_rrc_SignallingConnectionRelease_r3_IEs = -1;
static gint ett_rrc_SignallingConnectionRelease_va40ext_IEs = -1;
static gint ett_rrc_SignallingConnectionReleaseIndication = -1;
static gint ett_rrc_T_laterNonCriticalExtensions_49 = -1;
static gint ett_rrc_T_v860nonCriticalExtentions = -1;
static gint ett_rrc_T_nonCriticalExtensions_144 = -1;
static gint ett_rrc_SignallingConnectionReleaseIndication_v860ext = -1;
static gint ett_rrc_SystemInformation_BCH = -1;
static gint ett_rrc_T_payload = -1;
static gint ett_rrc_T_lastAndFirst = -1;
static gint ett_rrc_T_lastAndComplete = -1;
static gint ett_rrc_T_lastAndCompleteAndFirst = -1;
static gint ett_rrc_T_completeAndFirst = -1;
static gint ett_rrc_SystemInformation_FACH = -1;
static gint ett_rrc_T_payload_01 = -1;
static gint ett_rrc_T_lastAndFirst_01 = -1;
static gint ett_rrc_T_lastAndComplete_01 = -1;
static gint ett_rrc_T_lastAndCompleteAndFirst_01 = -1;
static gint ett_rrc_T_completeAndFirst_01 = -1;
static gint ett_rrc_FirstSegment = -1;
static gint ett_rrc_FirstSegmentShort = -1;
static gint ett_rrc_SubsequentSegment = -1;
static gint ett_rrc_LastSegment = -1;
static gint ett_rrc_LastSegmentShort = -1;
static gint ett_rrc_CompleteSIB_List = -1;
static gint ett_rrc_CompleteSIB = -1;
static gint ett_rrc_CompleteSIBshort = -1;
static gint ett_rrc_SystemInformationChangeIndication = -1;
static gint ett_rrc_T_laterNonCriticalExtensions_50 = -1;
static gint ett_rrc_T_v860NonCriticalExtensions_17 = -1;
static gint ett_rrc_T_nonCriticalExtensions_145 = -1;
static gint ett_rrc_SystemInformationChangeIndication_v860ext_IEs = -1;
static gint ett_rrc_TransportChannelReconfiguration = -1;
static gint ett_rrc_T_r3_24 = -1;
static gint ett_rrc_T_v3a0NonCriticalExtensions_09 = -1;
static gint ett_rrc_T_laterNonCriticalExtensions_51 = -1;
static gint ett_rrc_T_v4b0NonCriticalExtensions_13 = -1;
static gint ett_rrc_T_v590NonCriticalExtensions_24 = -1;
static gint ett_rrc_T_v690NonCriticalExtensions_36 = -1;
static gint ett_rrc_T_v770NonCriticalExtensions_25 = -1;
static gint ett_rrc_T_nonCriticalExtensions_146 = -1;
static gint ett_rrc_T_later_than_r3_21 = -1;
static gint ett_rrc_T_criticalExtensions_111 = -1;
static gint ett_rrc_T_r4_12 = -1;
static gint ett_rrc_T_v4d0NonCriticalExtensions_12 = -1;
static gint ett_rrc_T_v590NonCriticalExtensions_25 = -1;
static gint ett_rrc_T_v690NonCriticalExtensions_37 = -1;
static gint ett_rrc_T_v770NonCriticalExtensions_26 = -1;
static gint ett_rrc_T_nonCriticalExtensions_147 = -1;
static gint ett_rrc_T_criticalExtensions_112 = -1;
static gint ett_rrc_T_r5_10 = -1;
static gint ett_rrc_T_v690NonCriticalExtensions_38 = -1;
static gint ett_rrc_T_v770NonCriticalExtensions_27 = -1;
static gint ett_rrc_T_nonCriticalExtensions_148 = -1;
static gint ett_rrc_T_criticalExtensions_113 = -1;
static gint ett_rrc_T_r6_11 = -1;
static gint ett_rrc_T_v6b0NonCriticalExtensions_11 = -1;
static gint ett_rrc_T_v770NonCriticalExtensions_28 = -1;
static gint ett_rrc_T_nonCriticalExtensions_149 = -1;
static gint ett_rrc_T_criticalExtensions_114 = -1;
static gint ett_rrc_T_r7_11 = -1;
static gint ett_rrc_T_v780NonCriticalExtensions_09 = -1;
static gint ett_rrc_T_v7d0NonCriticalExtensions_13 = -1;
static gint ett_rrc_T_v7f0NonCriticalExtensions_14 = -1;
static gint ett_rrc_T_v7g0NonCriticalExtensions_15 = -1;
static gint ett_rrc_T_nonCriticalExtensions_150 = -1;
static gint ett_rrc_T_criticalExtensions_115 = -1;
static gint ett_rrc_T_r8_11 = -1;
static gint ett_rrc_T_v7d0NonCriticalExtensions_14 = -1;
static gint ett_rrc_T_v7f0NonCriticalExtensions_15 = -1;
static gint ett_rrc_T_v890NonCriticalExtensions_09 = -1;
static gint ett_rrc_T_v7g0NonCriticalExtensions_16 = -1;
static gint ett_rrc_T_v8a0NonCriticalExtensions_09 = -1;
static gint ett_rrc_T_nonCriticalExtensions_151 = -1;
static gint ett_rrc_T_criticalExtensions_116 = -1;
static gint ett_rrc_T_r9_10 = -1;
static gint ett_rrc_T_v950NonCriticalExtensions_08 = -1;
static gint ett_rrc_T_v9c0NonCriticalExtensions_10 = -1;
static gint ett_rrc_T_nonCriticalExtensions_152 = -1;
static gint ett_rrc_T_criticalExtensions_117 = -1;
static gint ett_rrc_T_r10_10 = -1;
static gint ett_rrc_T_v9c0NonCriticalExtensions_11 = -1;
static gint ett_rrc_T_vbxyNonCriticalExtensions_14 = -1;
static gint ett_rrc_T_nonCriticalExtensions_153 = -1;
static gint ett_rrc_T_criticalExtensions_118 = -1;
static gint ett_rrc_T_r11_11 = -1;
static gint ett_rrc_T_nonCriticalExtensions_154 = -1;
static gint ett_rrc_T_criticalExtensions_119 = -1;
static gint ett_rrc_TransportChannelReconfiguration_r3_IEs = -1;
static gint ett_rrc_T_dummy_11 = -1;
static gint ett_rrc_T_fdd_39 = -1;
static gint ett_rrc_T_modeSpecificPhysChInfo_12 = -1;
static gint ett_rrc_T_fdd_40 = -1;
static gint ett_rrc_TransportChannelReconfiguration_v3a0ext = -1;
static gint ett_rrc_TransportChannelReconfiguration_v4b0ext_IEs = -1;
static gint ett_rrc_TransportChannelReconfiguration_v590ext_IEs = -1;
static gint ett_rrc_TransportChannelReconfiguration_r4_IEs = -1;
static gint ett_rrc_T_dummy_12 = -1;
static gint ett_rrc_T_fdd_41 = -1;
static gint ett_rrc_T_modeSpecificPhysChInfo_13 = -1;
static gint ett_rrc_T_fdd_42 = -1;
static gint ett_rrc_TransportChannelReconfiguration_r5_IEs = -1;
static gint ett_rrc_T_dummy_13 = -1;
static gint ett_rrc_T_fdd_43 = -1;
static gint ett_rrc_T_modeSpecificPhysChInfo_14 = -1;
static gint ett_rrc_T_fdd_44 = -1;
static gint ett_rrc_TransportChannelReconfiguration_v690ext_IEs = -1;
static gint ett_rrc_TransportChannelReconfiguration_r6_IEs = -1;
static gint ett_rrc_TransportChannelReconfiguration_v6b0ext_IEs = -1;
static gint ett_rrc_TransportChannelReconfiguration_r7_IEs = -1;
static gint ett_rrc_TransportChannelReconfiguration_v770ext_IEs = -1;
static gint ett_rrc_TransportChannelReconfiguration_v780ext_IEs = -1;
static gint ett_rrc_TransportChannelReconfiguration_v7d0ext_IEs = -1;
static gint ett_rrc_TransportChannelReconfiguration_v7f0ext_IEs = -1;
static gint ett_rrc_TransportChannelReconfiguration_v7g0ext_IEs = -1;
static gint ett_rrc_TransportChannelReconfiguration_r8_IEs = -1;
static gint ett_rrc_TransportChannelReconfiguration_v890ext_IEs = -1;
static gint ett_rrc_TransportChannelReconfiguration_v8a0ext_IEs = -1;
static gint ett_rrc_TransportChannelReconfiguration_r9_IEs = -1;
static gint ett_rrc_TransportChannelReconfiguration_v950ext_IEs = -1;
static gint ett_rrc_TransportChannelReconfiguration_v9c0ext_IEs = -1;
static gint ett_rrc_TransportChannelReconfiguration_r10_IEs = -1;
static gint ett_rrc_TransportChannelReconfiguration_vbxyext_IEs = -1;
static gint ett_rrc_TransportChannelReconfiguration_r11_IEs = -1;
static gint ett_rrc_TransportChannelReconfigurationComplete = -1;
static gint ett_rrc_T_laterNonCriticalExtensions_52 = -1;
static gint ett_rrc_T_v770NonCriticalExtensions_29 = -1;
static gint ett_rrc_T_nonCriticalExtensions_155 = -1;
static gint ett_rrc_TransportChannelReconfigurationComplete_v770ext_IEs = -1;
static gint ett_rrc_TransportChannelReconfigurationFailure = -1;
static gint ett_rrc_T_laterNonCriticalExtensions_53 = -1;
static gint ett_rrc_T_nonCriticalExtensions_156 = -1;
static gint ett_rrc_TransportFormatCombinationControl = -1;
static gint ett_rrc_T_modeSpecificInfo_15 = -1;
static gint ett_rrc_T_tdd_09 = -1;
static gint ett_rrc_T_laterNonCriticalExtensions_54 = -1;
static gint ett_rrc_T_v820NonCriticalExtensions_02 = -1;
static gint ett_rrc_T_nonCriticalExtensions_157 = -1;
static gint ett_rrc_TransportFormatCombinationControl_v820ext_IEs = -1;
static gint ett_rrc_TransportFormatCombinationControlFailure = -1;
static gint ett_rrc_T_laterNonCriticalExtensions_55 = -1;
static gint ett_rrc_T_nonCriticalExtensions_158 = -1;
static gint ett_rrc_UECapabilityEnquiry = -1;
static gint ett_rrc_T_r3_25 = -1;
static gint ett_rrc_T_laterNonCriticalExtensions_56 = -1;
static gint ett_rrc_T_v4b0NonCriticalExtensions_14 = -1;
static gint ett_rrc_T_v590NonCriticalExtensions_26 = -1;
static gint ett_rrc_T_v770NonCriticalExtensions_30 = -1;
static gint ett_rrc_T_v860NonCriticalExtensions_18 = -1;
static gint ett_rrc_T_nonCriticalExtensions_159 = -1;
static gint ett_rrc_T_later_than_r3_22 = -1;
static gint ett_rrc_T_criticalExtensions_120 = -1;
static gint ett_rrc_UECapabilityEnquiry_r3_IEs = -1;
static gint ett_rrc_UECapabilityEnquiry_v4b0ext_IEs = -1;
static gint ett_rrc_UECapabilityEnquiry_v590ext_IEs = -1;
static gint ett_rrc_UECapabilityEnquiry_v770ext_IEs = -1;
static gint ett_rrc_UECapabilityEnquiry_v860ext_IEs = -1;
static gint ett_rrc_UECapabilityInformation = -1;
static gint ett_rrc_T_v370NonCriticalExtensions_01 = -1;
static gint ett_rrc_T_v380NonCriticalExtensions_01 = -1;
static gint ett_rrc_T_v3a0NonCriticalExtensions_10 = -1;
static gint ett_rrc_T_laterNonCriticalExtensions_57 = -1;
static gint ett_rrc_T_v4b0NonCriticalExtensions_15 = -1;
static gint ett_rrc_T_v590NonCriticalExtensions_27 = -1;
static gint ett_rrc_T_v5c0NonCriticalExtensions_01 = -1;
static gint ett_rrc_T_v690NonCriticalExtensions_39 = -1;
static gint ett_rrc_T_nonCriticalExtensions_160 = -1;
static gint ett_rrc_UECapabilityInformation_v370ext = -1;
static gint ett_rrc_UECapabilityInformation_v380ext_IEs = -1;
static gint ett_rrc_UECapabilityInformation_v3a0ext_IEs = -1;
static gint ett_rrc_UECapabilityInformation_r3_add_ext_IEs = -1;
static gint ett_rrc_T_v680NonCriticalExtensions_01 = -1;
static gint ett_rrc_T_v7e0NonCriticalExtensions_03 = -1;
static gint ett_rrc_T_v7f0NonCriticalExtensions_16 = -1;
static gint ett_rrc_T_va40NonCriticalExtensions_14 = -1;
static gint ett_rrc_T_nonCriticalExtensions_161 = -1;
static gint ett_rrc_UECapabilityInformation_v4b0ext = -1;
static gint ett_rrc_UECapabilityInformation_v590ext = -1;
static gint ett_rrc_UECapabilityInformation_v5c0ext = -1;
static gint ett_rrc_UECapabilityInformation_v650ext_IEs = -1;
static gint ett_rrc_UECapabilityInformation_v680ext_IEs = -1;
static gint ett_rrc_UECapabilityInformation_v690ext_IEs = -1;
static gint ett_rrc_UECapabilityInformation_v7e0ext_IEs = -1;
static gint ett_rrc_UECapabilityInformation_v7f0ext_IEs = -1;
static gint ett_rrc_UECapabilityInformation_va40ext_IEs = -1;
static gint ett_rrc_UECapabilityInformationConfirm = -1;
static gint ett_rrc_T_r3_26 = -1;
static gint ett_rrc_T_laterNonCriticalExtensions_58 = -1;
static gint ett_rrc_T_v770NonCriticalExtensions_31 = -1;
static gint ett_rrc_T_nonCriticalExtensions_162 = -1;
static gint ett_rrc_T_later_than_r3_23 = -1;
static gint ett_rrc_T_criticalExtensions_121 = -1;
static gint ett_rrc_UECapabilityInformationConfirm_r3_IEs = -1;
static gint ett_rrc_UECapabilityInformationConfirm_v770ext_IEs = -1;
static gint ett_rrc_UplinkDirectTransfer = -1;
static gint ett_rrc_T_laterNonCriticalExtensions_59 = -1;
static gint ett_rrc_T_v690NonCriticalExtensions_40 = -1;
static gint ett_rrc_T_v7g0NonCriticalExtensions_17 = -1;
static gint ett_rrc_T_vbxyNonCriticalExtensions_15 = -1;
static gint ett_rrc_T_nonCriticalExtensions_163 = -1;
static gint ett_rrc_UplinkDirectTransfer_v690ext_IEs = -1;
static gint ett_rrc_UplinkDirectTransfer_v7g0ext_IEs = -1;
static gint ett_rrc_UplinkDirectTransfer_vbxyext_IEs = -1;
static gint ett_rrc_UplinkPhysicalChannelControl = -1;
static gint ett_rrc_T_r3_27 = -1;
static gint ett_rrc_T_laterNonCriticalExtensions_60 = -1;
static gint ett_rrc_T_v4b0NonCriticalExtensions_16 = -1;
static gint ett_rrc_T_noncriticalExtensions = -1;
static gint ett_rrc_T_later_than_r3_24 = -1;
static gint ett_rrc_T_criticalExtensions_122 = -1;
static gint ett_rrc_T_r4_13 = -1;
static gint ett_rrc_T_v4d0NonCriticalExtensions_13 = -1;
static gint ett_rrc_T_v690NonCriticalExtensions_41 = -1;
static gint ett_rrc_T_nonCriticalExtensions_164 = -1;
static gint ett_rrc_T_criticalExtensions_123 = -1;
static gint ett_rrc_T_r5_11 = -1;
static gint ett_rrc_T_v690NonCriticalExtensions_42 = -1;
static gint ett_rrc_T_v6a0NonCriticalExtensions_01 = -1;
static gint ett_rrc_T_nonCriticalExtensions_165 = -1;
static gint ett_rrc_T_criticalExtensions_124 = -1;
static gint ett_rrc_T_r7_12 = -1;
static gint ett_rrc_T_nonCriticalExtensions_166 = -1;
static gint ett_rrc_T_criticalExtensions_125 = -1;
static gint ett_rrc_UplinkPhysicalChannelControl_r3_IEs = -1;
static gint ett_rrc_UplinkPhysicalChannelControl_v4b0ext_IEs = -1;
static gint ett_rrc_UplinkPhysicalChannelControl_r4_IEs = -1;
static gint ett_rrc_T_tddOption = -1;
static gint ett_rrc_T_tdd384_08 = -1;
static gint ett_rrc_T_tdd128_08 = -1;
static gint ett_rrc_UplinkPhysicalChannelControl_r5_IEs = -1;
static gint ett_rrc_T_tddOption_01 = -1;
static gint ett_rrc_T_tdd384_09 = -1;
static gint ett_rrc_T_tdd128_09 = -1;
static gint ett_rrc_UplinkPhysicalChannelControl_v690ext_IEs = -1;
static gint ett_rrc_UplinkPhysicalChannelControl_v6a0ext_IEs = -1;
static gint ett_rrc_UplinkPhysicalChannelControl_r7_IEs = -1;
static gint ett_rrc_T_tddOption_02 = -1;
static gint ett_rrc_T_tdd384_10 = -1;
static gint ett_rrc_T_tdd768_05 = -1;
static gint ett_rrc_T_tdd128_10 = -1;
static gint ett_rrc_URAUpdate = -1;
static gint ett_rrc_T_laterNonCriticalExtensions_61 = -1;
static gint ett_rrc_T_v770NonCriticalExtensions_32 = -1;
static gint ett_rrc_T_v860NonCriticalExtensions_19 = -1;
static gint ett_rrc_T_va40NonCriticalExtensions_15 = -1;
static gint ett_rrc_T_nonCriticalExtensions_167 = -1;
static gint ett_rrc_URAUpdate_r3_add_ext_IEs = -1;
static gint ett_rrc_T_nonCriticalExtensions_168 = -1;
static gint ett_rrc_UraUpdate_v770ext_IEs = -1;
static gint ett_rrc_URAUpdate_v7e0ext_IEs = -1;
static gint ett_rrc_URAUpdate_v860ext_IEs = -1;
static gint ett_rrc_URAUpdate_va40ext_IEs = -1;
static gint ett_rrc_URAUpdateConfirm = -1;
static gint ett_rrc_T_r3_28 = -1;
static gint ett_rrc_T_laterNonCriticalExtensions_62 = -1;
static gint ett_rrc_T_v690NonCriticalExtensions_43 = -1;
static gint ett_rrc_T_nonCriticalExtensions_169 = -1;
static gint ett_rrc_T_later_than_r3_25 = -1;
static gint ett_rrc_T_criticalExtensions_126 = -1;
static gint ett_rrc_T_r5_12 = -1;
static gint ett_rrc_T_v690NonCriticalExtensions_44 = -1;
static gint ett_rrc_T_nonCriticalExtensions_170 = -1;
static gint ett_rrc_T_criticalExtensions_127 = -1;
static gint ett_rrc_T_r7_13 = -1;
static gint ett_rrc_T_v860NonCriticalExtensions_20 = -1;
static gint ett_rrc_T_nonCriticalExtensions_171 = -1;
static gint ett_rrc_T_criticalExtensions_128 = -1;
static gint ett_rrc_URAUpdateConfirm_r3_IEs = -1;
static gint ett_rrc_URAUpdateConfirm_r5_IEs = -1;
static gint ett_rrc_URAUpdateConfirm_v690ext_IEs = -1;
static gint ett_rrc_URAUpdateConfirm_r7_IEs = -1;
static gint ett_rrc_URAUpdateConfirm_v860ext_IEs = -1;
static gint ett_rrc_URAUpdateConfirm_CCCH = -1;
static gint ett_rrc_T_r3_29 = -1;
static gint ett_rrc_T_laterNonCriticalExtensions_63 = -1;
static gint ett_rrc_T_v690NonCriticalExtensions_45 = -1;
static gint ett_rrc_T_v860NonCriticalExtensions_21 = -1;
static gint ett_rrc_T_nonCriticalExtensions_172 = -1;
static gint ett_rrc_T_later_than_r3_26 = -1;
static gint ett_rrc_T_criticalExtensions_129 = -1;
static gint ett_rrc_URAUpdateConfirm_CCCH_r3_IEs = -1;
static gint ett_rrc_UTRANMobilityInformation = -1;
static gint ett_rrc_T_r3_30 = -1;
static gint ett_rrc_T_v3a0NonCriticalExtensions_11 = -1;
static gint ett_rrc_T_laterNonCriticalExtensions_64 = -1;
static gint ett_rrc_T_v690NonCriticalExtensions_46 = -1;
static gint ett_rrc_T_v860NonCriticalExtentions = -1;
static gint ett_rrc_T_nonCriticalExtensions_173 = -1;
static gint ett_rrc_T_later_than_r3_27 = -1;
static gint ett_rrc_T_criticalExtensions_130 = -1;
static gint ett_rrc_T_r5_13 = -1;
static gint ett_rrc_T_v690NonCriticalExtensions_47 = -1;
static gint ett_rrc_T_v860NonCriticalExtentions_01 = -1;
static gint ett_rrc_T_nonCriticalExtensions_174 = -1;
static gint ett_rrc_T_criticalExtensions_131 = -1;
static gint ett_rrc_T_r7_14 = -1;
static gint ett_rrc_T_v860NonCriticalExtensions_22 = -1;
static gint ett_rrc_T_nonCriticalExtensions_175 = -1;
static gint ett_rrc_T_criticalExtensions_132 = -1;
static gint ett_rrc_UTRANMobilityInformation_r3_IEs = -1;
static gint ett_rrc_T_nonCriticalExtensions_176 = -1;
static gint ett_rrc_UTRANMobilityInformation_v3a0ext_IEs = -1;
static gint ett_rrc_UTRANMobilityInformation_r5_IEs = -1;
static gint ett_rrc_UtranMobilityInformation_v690ext_IEs = -1;
static gint ett_rrc_UTRANMobilityInformation_r7_IEs = -1;
static gint ett_rrc_UTRANMobilityInformation_v860ext1_IEs = -1;
static gint ett_rrc_UTRANMobilityInformation_v860ext2_IEs = -1;
static gint ett_rrc_UTRANMobilityInformationConfirm = -1;
static gint ett_rrc_T_laterNonCriticalExtensions_65 = -1;
static gint ett_rrc_T_v770NonCriticalExtension_01 = -1;
static gint ett_rrc_T_va40NonCriticalExtensions_16 = -1;
static gint ett_rrc_T_nonCriticalExtensions_177 = -1;
static gint ett_rrc_UTRANMobilityInformationConfirm_v770ext_IEs = -1;
static gint ett_rrc_UTRANMobilityInformationConfirm_va40ext_IEs = -1;
static gint ett_rrc_UTRANMobilityInformationFailure = -1;
static gint ett_rrc_T_laterNonCriticalExtensions_66 = -1;
static gint ett_rrc_T_nonCriticalExtensions_178 = -1;
static gint ett_rrc_MBMSAccessInformation = -1;
static gint ett_rrc_T_nonCriticalExtensions_179 = -1;
static gint ett_rrc_MBMSCommonPTMRBInformation = -1;
static gint ett_rrc_T_v770NonCriticalExtensions_33 = -1;
static gint ett_rrc_T_v780NonCriticalExtensions_10 = -1;
static gint ett_rrc_T_v860NonCriticalExtensions_23 = -1;
static gint ett_rrc_T_nonCriticalExtensions_180 = -1;
static gint ett_rrc_MBMSCommonPTMRBInformation_v770ext_IEs = -1;
static gint ett_rrc_MBMSCommonPTMRBInformation_v780ext_IEs = -1;
static gint ett_rrc_MBMSCommonPTMRBInformation_v860ext_IEs = -1;
static gint ett_rrc_MBMSCurrentCellPTMRBInformation = -1;
static gint ett_rrc_T_v770NonCriticalExtensions_34 = -1;
static gint ett_rrc_T_nonCriticalExtensions_181 = -1;
static gint ett_rrc_MBMSCurrentCellPTMRBInfo_v770ext_IEs = -1;
static gint ett_rrc_MBMSGeneralInformation = -1;
static gint ett_rrc_T_v6b0NonCriticalExtensions_12 = -1;
static gint ett_rrc_T_v770NonCriticalExtensions_35 = -1;
static gint ett_rrc_T_v860NonCriticalExtensions_24 = -1;
static gint ett_rrc_T_v890NoncriticalExtensions = -1;
static gint ett_rrc_T_nonCriticalExtensions_182 = -1;
static gint ett_rrc_MBMSGeneralInformation_v6b0ext_IEs = -1;
static gint ett_rrc_MBMSGeneralInformation_v770ext_IEs = -1;
static gint ett_rrc_MBMSGeneralInformation_v860ext_IEs = -1;
static gint ett_rrc_MBMSGeneralInformation_v890ext_IEs = -1;
static gint ett_rrc_MBMSModificationRequest = -1;
static gint ett_rrc_T_v6b0NonCriticalExtensions_13 = -1;
static gint ett_rrc_T_v6f0NonCriticalExtensions_02 = -1;
static gint ett_rrc_T_nonCriticalExtensions_183 = -1;
static gint ett_rrc_MBMSModificationRequest_v6b0ext_IEs = -1;
static gint ett_rrc_MBMSModificationRequest_v6f0ext_IEs = -1;
static gint ett_rrc_MBMSModifiedServicesInformation = -1;
static gint ett_rrc_T_v770NonCriticalExtensions_36 = -1;
static gint ett_rrc_T_v7c0NonCriticalExtensions = -1;
static gint ett_rrc_T_nonCriticalExtensions_184 = -1;
static gint ett_rrc_MBMSModifiedServicesInformation_v770ext_IEs = -1;
static gint ett_rrc_MBMSModifiedServicesInformation_v7c0ext_IEs = -1;
static gint ett_rrc_MBMSNeighbouringCellPTMRBInformation = -1;
static gint ett_rrc_T_v770NonCriticalExtensions_37 = -1;
static gint ett_rrc_T_nonCriticalExtensions_185 = -1;
static gint ett_rrc_MBMSNeighbouringCellPTMRBInformation_v770ext_IEs = -1;
static gint ett_rrc_MBMSSchedulingInformation = -1;
static gint ett_rrc_T_nonCriticalExtensions_186 = -1;
static gint ett_rrc_MBMSUnmodifiedServicesInformation = -1;
static gint ett_rrc_T_v770NonCriticalExtensions_38 = -1;
static gint ett_rrc_T_nonCriticalExtensions_187 = -1;
static gint ett_rrc_MBMSUnmodifiedServicesInformation_v770ext_IEs = -1;
static gint ett_rrc_System_Information_Container = -1;
static gint ett_rrc_T_vbxyNonCriticalExtensions_16 = -1;
static gint ett_rrc_T_nonCriticalExtensions_188 = -1;
static gint ett_rrc_System_Information_Container_vbxyext_IEs = -1;
static gint ett_rrc_LoggingMeasurementConfiguration = -1;
static gint ett_rrc_T_criticalExtensions_133 = -1;
static gint ett_rrc_T_r10_11 = -1;
static gint ett_rrc_T_nonCriticalExtensions_189 = -1;
static gint ett_rrc_T_criticalExtensions_134 = -1;
static gint ett_rrc_T_r11_12 = -1;
static gint ett_rrc_T_nonCriticalExtensions_190 = -1;
static gint ett_rrc_T_criticalExtensions_135 = -1;
static gint ett_rrc_LoggingMeasurementConfiguration_r10_IEs = -1;
static gint ett_rrc_LoggingMeasurementConfiguration_r11_IEs = -1;
static gint ett_rrc_UEInformationResponse = -1;
static gint ett_rrc_T_vbxyNonCriticalExtensions_17 = -1;
static gint ett_rrc_T_nonCriticalExtensions_191 = -1;
static gint ett_rrc_UEInformationResponse_vbxyext_IEs = -1;
static gint ett_rrc_UEInformationRequest = -1;
static gint ett_rrc_T_criticalExtensions_136 = -1;
static gint ett_rrc_T_r10_12 = -1;
static gint ett_rrc_T_nonCriticalExtensions_192 = -1;
static gint ett_rrc_T_criticalExtensions_137 = -1;
static gint ett_rrc_T_r11_13 = -1;
static gint ett_rrc_T_nonCriticalExtensions_193 = -1;
static gint ett_rrc_T_criticalExtensions_138 = -1;
static gint ett_rrc_UEInformationRequest_r10_IEs = -1;
static gint ett_rrc_UEInformationRequest_r11_IEs = -1;
static gint ett_rrc_CN_DomainInformation = -1;
static gint ett_rrc_CN_DomainInformationFull = -1;
static gint ett_rrc_CN_DomainInformationList = -1;
static gint ett_rrc_CN_DomainInformationListFull = -1;
static gint ett_rrc_CN_DomainSysInfo = -1;
static gint ett_rrc_T_cn_Type = -1;
static gint ett_rrc_CN_DomainSysInfoList = -1;
static gint ett_rrc_CN_InformationInfo = -1;
static gint ett_rrc_CN_InformationInfo_r6 = -1;
static gint ett_rrc_CN_InformationInfoFull = -1;
static gint ett_rrc_DomainSpecificEAB_Parameters = -1;
static gint ett_rrc_EAB_AccessClassBarredList = -1;
static gint ett_rrc_EAB_Configuration = -1;
static gint ett_rrc_EAB_Parameters = -1;
static gint ett_rrc_EAB_ParametersPerPLMN_List = -1;
static gint ett_rrc_EAB_ParametersPerPLMN_List_item = -1;
static gint ett_rrc_Ec_N0forANR = -1;
static gint ett_rrc_Gsm_map_IDNNS = -1;
static gint ett_rrc_T_routingbasis = -1;
static gint ett_rrc_T_localPTMSI = -1;
static gint ett_rrc_T_tMSIofsamePLMN = -1;
static gint ett_rrc_T_tMSIofdifferentPLMN = -1;
static gint ett_rrc_T_iMSIresponsetopaging = -1;
static gint ett_rrc_T_iMSIcauseUEinitiatedEvent = -1;
static gint ett_rrc_T_iMEI = -1;
static gint ett_rrc_T_spare2 = -1;
static gint ett_rrc_T_spare1 = -1;
static gint ett_rrc_IMEI = -1;
static gint ett_rrc_IMSI_GSM_MAP = -1;
static gint ett_rrc_IntraDomainNasNodeSelector = -1;
static gint ett_rrc_T_version = -1;
static gint ett_rrc_T_release99 = -1;
static gint ett_rrc_T_cn_Type_01 = -1;
static gint ett_rrc_T_later = -1;
static gint ett_rrc_LAI = -1;
static gint ett_rrc_LocationRegistrationAccessClassBarredList = -1;
static gint ett_rrc_LocationRegistrationParameters = -1;
static gint ett_rrc_MCC = -1;
static gint ett_rrc_MNC = -1;
static gint ett_rrc_MultiplePLMN_List_r6 = -1;
static gint ett_rrc_SEQUENCE_SIZE_1_5_OF_PLMN_IdentityWithOptionalMCC_r6 = -1;
static gint ett_rrc_PagingPermissionWithAccessControlForSharedNetwork = -1;
static gint ett_rrc_PagingPermissionWithAccessControlList = -1;
static gint ett_rrc_PagingPermissionWithAccessControlParameters = -1;
static gint ett_rrc_PLMN_Identity = -1;
static gint ett_rrc_PLMN_IdentityWithOptionalMCC_r6 = -1;
static gint ett_rrc_PLMN_Type = -1;
static gint ett_rrc_T_gsm_MAP = -1;
static gint ett_rrc_T_ansi_41 = -1;
static gint ett_rrc_T_gsm_MAP_and_ANSI_41 = -1;
static gint ett_rrc_RAB_Identity = -1;
static gint ett_rrc_RAI = -1;
static gint ett_rrc_AccessClassBarredList = -1;
static gint ett_rrc_CellAccessRestriction = -1;
static gint ett_rrc_CellAccessRestriction_v870ext = -1;
static gint ett_rrc_CellBarred = -1;
static gint ett_rrc_T_barred = -1;
static gint ett_rrc_CellIdentity_PerRL_List = -1;
static gint ett_rrc_CellIDListItem = -1;
static gint ett_rrc_CellSelectReselectInfoSIB_3_4 = -1;
static gint ett_rrc_T_cellSelectQualityMeasure = -1;
static gint ett_rrc_T_cpich_Ec_N0 = -1;
static gint ett_rrc_T_modeSpecificInfo_16 = -1;
static gint ett_rrc_T_fdd_45 = -1;
static gint ett_rrc_T_tdd_10 = -1;
static gint ett_rrc_CSG_DedicatedFrequencyInfoList = -1;
static gint ett_rrc_CSG_PSCSplitInfo = -1;
static gint ett_rrc_DomainSpecificAccessRestrictionForSharedNetwork_v670ext = -1;
static gint ett_rrc_DomainSpecificAccessRestrictionList_v670ext = -1;
static gint ett_rrc_DomainSpecificAccessRestrictionParam_v670ext = -1;
static gint ett_rrc_DomainSpecificAccessRestriction_v670ext = -1;
static gint ett_rrc_T_restriction = -1;
static gint ett_rrc_Mapping = -1;
static gint ett_rrc_Mapping_LCR_r4 = -1;
static gint ett_rrc_MappingFunctionParameter = -1;
static gint ett_rrc_MappingFunctionParameterList = -1;
static gint ett_rrc_MappingInfo = -1;
static gint ett_rrc_DedicatedPriorityInformation = -1;
static gint ett_rrc_T_action = -1;
static gint ett_rrc_T_configureDedicatedPriorities = -1;
static gint ett_rrc_PriorityLevel = -1;
static gint ett_rrc_T_radioAccessTechnology = -1;
static gint ett_rrc_T_utraFDD = -1;
static gint ett_rrc_T_utraFDD_item = -1;
static gint ett_rrc_T_utraTDD = -1;
static gint ett_rrc_T_utraTDD_item = -1;
static gint ett_rrc_T_eutra = -1;
static gint ett_rrc_T_eutra_item = -1;
static gint ett_rrc_T_gsm_01 = -1;
static gint ett_rrc_PriorityLevelList = -1;
static gint ett_rrc_RAT_FDD_Info = -1;
static gint ett_rrc_RAT_FDD_InfoList = -1;
static gint ett_rrc_RAT_TDD_Info = -1;
static gint ett_rrc_RAT_TDD_InfoList = -1;
static gint ett_rrc_URA_IdentityList = -1;
static gint ett_rrc_BackoffControlParams = -1;
static gint ett_rrc_CapabilityUpdateRequirement = -1;
static gint ett_rrc_CapabilityUpdateRequirement_r4_ext = -1;
static gint ett_rrc_CapabilityUpdateRequirement_r4 = -1;
static gint ett_rrc_CapabilityUpdateRequirement_r5 = -1;
static gint ett_rrc_CapabilityUpdateRequirement_v770ext = -1;
static gint ett_rrc_CapabilityUpdateRequirement_r7 = -1;
static gint ett_rrc_CapabilityUpdateRequirement_v860ext = -1;
static gint ett_rrc_CapabilityUpdateRequirement_r8 = -1;
static gint ett_rrc_CipheringModeCommand = -1;
static gint ett_rrc_CipheringModeCommand_r7 = -1;
static gint ett_rrc_CipheringModeInfo = -1;
static gint ett_rrc_CipheringModeInfo_r7 = -1;
static gint ett_rrc_CN_PagedUE_Identity = -1;
static gint ett_rrc_CompressedModeMeasCapability = -1;
static gint ett_rrc_CompressedModeMeasCapability_LCR_r4 = -1;
static gint ett_rrc_CompressedModeMeasCapability_v860ext = -1;
static gint ett_rrc_CompressedModeMeasCapability_v920ext = -1;
static gint ett_rrc_CompressedModeMeasCapability_va40ext = -1;
static gint ett_rrc_CompressedModeMeasCapabEUTRA = -1;
static gint ett_rrc_CompressedModeMeasCapabEUTRAList = -1;
static gint ett_rrc_CompressedModeMeasCapabFDDList = -1;
static gint ett_rrc_CompressedModeMeasCapabFDDList2 = -1;
static gint ett_rrc_CompressedModeMeasCapabFDDList3 = -1;
static gint ett_rrc_CompressedModeMeasCapabFDDList_ext = -1;
static gint ett_rrc_CompressedModeMeasCapabFDD = -1;
static gint ett_rrc_CompressedModeMeasCapabFDD2 = -1;
static gint ett_rrc_CompressedModeMeasCapabFDD3 = -1;
static gint ett_rrc_CompressedModeMeasCapabFDD_ext = -1;
static gint ett_rrc_CompressedModeMeasCapabTDDList = -1;
static gint ett_rrc_CompressedModeMeasCapabTDD = -1;
static gint ett_rrc_CompressedModeMeasCapabGSMList = -1;
static gint ett_rrc_CompressedModeMeasCapabGSM = -1;
static gint ett_rrc_CompressedModeMeasCapabMC = -1;
static gint ett_rrc_CPCH_Parameters = -1;
static gint ett_rrc_CSG_ProximityIndicationCapability = -1;
static gint ett_rrc_DL_InformationPerSecondaryRL_List = -1;
static gint ett_rrc_DL_InformationPerSecondaryRL = -1;
static gint ett_rrc_T_e_RGCH_Info = -1;
static gint ett_rrc_DL_PhysChCapabilityFDD = -1;
static gint ett_rrc_DL_PhysChCapabilityFDD_v380ext = -1;
static gint ett_rrc_DL_PhysChCapabilityFDD_v770ext = -1;
static gint ett_rrc_DL_PhysChCapabilityFDD_v860ext = -1;
static gint ett_rrc_DL_PhysChCapabilityFDD_v920ext = -1;
static gint ett_rrc_DL_PhysChCapabilityFDD_va40ext = -1;
static gint ett_rrc_DL_PhysChCapabilityFDD_vbxyext = -1;
static gint ett_rrc_DL_PhysChCapabilityTDD = -1;
static gint ett_rrc_DL_PhysChCapabilityTDD_LCR_r4 = -1;
static gint ett_rrc_DL_PhysChCapabilityTDD_128_v770ext = -1;
static gint ett_rrc_DL_PhysChCapabilityTDD_128_v860ext = -1;
static gint ett_rrc_DL_PhysChCapabilityInfoTDD_128_va40ext = -1;
static gint ett_rrc_DL_PhysChCapabilityTDD_768 = -1;
static gint ett_rrc_T_tdd768_hspdsch = -1;
static gint ett_rrc_DL_PhysChCapabilityInfoTDD_768 = -1;
static gint ett_rrc_DL_TransChCapability = -1;
static gint ett_rrc_DRAC_SysInfo = -1;
static gint ett_rrc_DRAC_SysInfoList = -1;
static gint ett_rrc_FailureCauseWithProtErr = -1;
static gint ett_rrc_FailureCauseWithProtErrTrId = -1;
static gint ett_rrc_FrequencyBandsIndicatorSupport = -1;
static gint ett_rrc_T_modeSpecificInfo_17 = -1;
static gint ett_rrc_T_fdd_46 = -1;
static gint ett_rrc_T_tdd_11 = -1;
static gint ett_rrc_FrequencyBandsIndicatorTDD128 = -1;
static gint ett_rrc_FrequencyBandsIndicatorSupportTDD128 = -1;
static gint ett_rrc_FrequencyBandsRedirectionFDD = -1;
static gint ett_rrc_FrequencyBandsIndicatorFDD = -1;
static gint ett_rrc_GroupIdentityWithReleaseInformation = -1;
static gint ett_rrc_GroupReleaseInformation = -1;
static gint ett_rrc_GSM_Measurements = -1;
static gint ett_rrc_MultiflowBandCombination = -1;
static gint ett_rrc_MultiflowBandCombList = -1;
static gint ett_rrc_MultiflowCapability = -1;
static gint ett_rrc_MultiflowCapabilityBand = -1;
static gint ett_rrc_MultiflowNonContiguousCapability = -1;
static gint ett_rrc_IdleIntervalMeasCapabEUTRA = -1;
static gint ett_rrc_IdleIntervalMeasCapabEUTRAList = -1;
static gint ett_rrc_IMSI_and_ESN_DS_41 = -1;
static gint ett_rrc_InitialPriorityDelayList = -1;
static gint ett_rrc_InitialUE_Identity = -1;
static gint ett_rrc_IntegrityCheckInfo = -1;
static gint ett_rrc_IntegrityProtActivationInfo = -1;
static gint ett_rrc_IntegrityProtectionModeCommand = -1;
static gint ett_rrc_T_startIntegrityProtection = -1;
static gint ett_rrc_T_modify = -1;
static gint ett_rrc_IntegrityProtectionModeInfo = -1;
static gint ett_rrc_IntegrityProtectionModeInfo_r7 = -1;
static gint ett_rrc_MeasurementCapability = -1;
static gint ett_rrc_MeasurementCapabilityExt = -1;
static gint ett_rrc_MeasurementCapabilityExt2 = -1;
static gint ett_rrc_MeasurementCapabilityExt3 = -1;
static gint ett_rrc_MeasurementCapabilityExt4 = -1;
static gint ett_rrc_MeasurementCapability_r4_ext = -1;
static gint ett_rrc_MeasurementCapability_v860ext = -1;
static gint ett_rrc_MeasurementCapability_v920ext = -1;
static gint ett_rrc_MeasurementCapability_va40ext = -1;
static gint ett_rrc_MeasurementCapability_vbxyext = -1;
static gint ett_rrc_MeasurementCapabilityTDD = -1;
static gint ett_rrc_NonContiguousMultiCellCombinationList = -1;
static gint ett_rrc_NonContiguousMultiCell = -1;
static gint ett_rrc_MultiRAT_Capability = -1;
static gint ett_rrc_MultiModeRAT_Capability_v590ext = -1;
static gint ett_rrc_MultiModeRAT_Capability_v680ext = -1;
static gint ett_rrc_MultiModeRAT_Capability_v770ext = -1;
static gint ett_rrc_MultiModeRAT_Capability_v860ext = -1;
static gint ett_rrc_MultiModeRAT_Capability_vbxyext = -1;
static gint ett_rrc_NeighCellSI_AcquisitionCapability = -1;
static gint ett_rrc_NetworkAssistedGANSS_Supported_List = -1;
static gint ett_rrc_NetworkAssistedGANSS_Supported_List_item = -1;
static gint ett_rrc_NetworkAssistedGANSS_Supported_List_v860ext = -1;
static gint ett_rrc_NetworkAssistedGANSS_Supported_List_v860ext_item = -1;
static gint ett_rrc_P_TMSI_and_RAI_GSM_MAP = -1;
static gint ett_rrc_PagingRecord = -1;
static gint ett_rrc_T_cn_Identity = -1;
static gint ett_rrc_T_utran_Identity = -1;
static gint ett_rrc_T_cn_OriginatedPage_connectedMode_UE = -1;
static gint ett_rrc_PagingRecord2_r5 = -1;
static gint ett_rrc_T_utran_SingleUE_Identity = -1;
static gint ett_rrc_T_cn_OriginatedPage_connectedMode_UE_01 = -1;
static gint ett_rrc_SEQUENCE_SIZE_1_maxURNTI_Group_OF_GroupIdentityWithReleaseInformation = -1;
static gint ett_rrc_PagingRecordList = -1;
static gint ett_rrc_PagingRecord2List_r5 = -1;
static gint ett_rrc_PDCP_Capability = -1;
static gint ett_rrc_T_supportForRfc2507 = -1;
static gint ett_rrc_PDCP_Capability_r4_ext = -1;
static gint ett_rrc_T_supportForRfc3095 = -1;
static gint ett_rrc_T_supported = -1;
static gint ett_rrc_PDCP_Capability_r5_ext = -1;
static gint ett_rrc_PDCP_Capability_r5_ext2 = -1;
static gint ett_rrc_PDCP_Capability_v770ext = -1;
static gint ett_rrc_PhysicalChannelCapability = -1;
static gint ett_rrc_T_fddPhysChCapability = -1;
static gint ett_rrc_T_tddPhysChCapability = -1;
static gint ett_rrc_PhysicalChannelCapability_v770ext = -1;
static gint ett_rrc_T_fddPhysChCapability_01 = -1;
static gint ett_rrc_T_tddPhysChCapability_384 = -1;
static gint ett_rrc_T_tddPhysChCapability_768 = -1;
static gint ett_rrc_T_tddPhysChCapability_128 = -1;
static gint ett_rrc_PhysicalChannelCapability_v860ext = -1;
static gint ett_rrc_T_fddPhysChCapability_02 = -1;
static gint ett_rrc_T_tddPhysChCapability_128_01 = -1;
static gint ett_rrc_PhysicalChannelCapability_v920ext = -1;
static gint ett_rrc_T_fddPhysChCapability_03 = -1;
static gint ett_rrc_PhysicalChannelCapability_va40ext = -1;
static gint ett_rrc_T_fddPhysChCapability_04 = -1;
static gint ett_rrc_T_tddPhysChCapability_128_02 = -1;
static gint ett_rrc_PhysicalChannelCapability_vbxyext = -1;
static gint ett_rrc_T_fddPhysChCapability_05 = -1;
static gint ett_rrc_PhysicalChannelCapabilityInfo_v770ext = -1;
static gint ett_rrc_T_tddPhysChCapability_768_01 = -1;
static gint ett_rrc_T_tddPhysChCapability_128_03 = -1;
static gint ett_rrc_PhysicalChannelCapability_LCR_r4 = -1;
static gint ett_rrc_T_tdd128_PhysChCapability = -1;
static gint ett_rrc_PhysicalChannelCapability_hspdsch_r5 = -1;
static gint ett_rrc_T_fdd_hspdsch = -1;
static gint ett_rrc_T_supported_01 = -1;
static gint ett_rrc_T_tdd384_hspdsch = -1;
static gint ett_rrc_T_tdd128_hspdsch = -1;
static gint ett_rrc_PNBSCH_Allocation_r4 = -1;
static gint ett_rrc_Pre_RedirectionInfo = -1;
static gint ett_rrc_ProtocolErrorIndicatorWithMoreInfo = -1;
static gint ett_rrc_T_errorOccurred = -1;
static gint ett_rrc_ProtocolErrorMoreInformation = -1;
static gint ett_rrc_T_diagnosticsType = -1;
static gint ett_rrc_T_type1 = -1;
static gint ett_rrc_RadioFrequencyBandTDDextList = -1;
static gint ett_rrc_Rb_timer_indicator = -1;
static gint ett_rrc_RedirectionInfo = -1;
static gint ett_rrc_RedirectionInfo_r6 = -1;
static gint ett_rrc_RedirectionInfo_v860ext = -1;
static gint ett_rrc_RF_Capability = -1;
static gint ett_rrc_T_fddRF_Capability = -1;
static gint ett_rrc_T_tddRF_Capability = -1;
static gint ett_rrc_RF_Capability_r4_ext = -1;
static gint ett_rrc_T_tddRF_Capability_01 = -1;
static gint ett_rrc_RF_Capability_v770ext = -1;
static gint ett_rrc_T_tdd768RF_Capability = -1;
static gint ett_rrc_RF_Capability_v860ext = -1;
static gint ett_rrc_RF_Capability_vaa0ext = -1;
static gint ett_rrc_RLC_Capability = -1;
static gint ett_rrc_RLC_Capability_r5_ext = -1;
static gint ett_rrc_RLC_Capability_v770ext = -1;
static gint ett_rrc_RLC_Capability_v920ext = -1;
static gint ett_rrc_RLC_Capability_va40ext = -1;
static gint ett_rrc_RLC_Capability_vbxyext = -1;
static gint ett_rrc_RRC_ConnectionReleaseInformation = -1;
static gint ett_rrc_T_release = -1;
static gint ett_rrc_RRC_MessageSequenceNumberList = -1;
static gint ett_rrc_RSR_VCC_Info = -1;
static gint ett_rrc_SecurityCapability = -1;
static gint ett_rrc_T_cipheringAlgorithmCap = -1;
static gint ett_rrc_T_integrityProtectionAlgorithmCap = -1;
static gint ett_rrc_SecondaryServingEDCHCell_Info = -1;
static gint ett_rrc_SecondaryEDCH_Info_Common = -1;
static gint ett_rrc_T_servingGrant = -1;
static gint ett_rrc_Serving_HSDSCH_CellInformation = -1;
static gint ett_rrc_Serving_HSDSCH_CellInformation_r7 = -1;
static gint ett_rrc_Serving_HSDSCH_CellInformation_r8 = -1;
static gint ett_rrc_Serving_HSDSCH_CellInformation_r9 = -1;
static gint ett_rrc_Serving_HSDSCH_CellInformation_r11 = -1;
static gint ett_rrc_SimultaneousSCCPCH_DPCH_Reception = -1;
static gint ett_rrc_T_supported_02 = -1;
static gint ett_rrc_SR_VCC_Info = -1;
static gint ett_rrc_SR_VCC_SecurityRABInfo_v860ext = -1;
static gint ett_rrc_STARTList = -1;
static gint ett_rrc_STARTSingle = -1;
static gint ett_rrc_SystemSpecificCapUpdateReqList = -1;
static gint ett_rrc_SystemSpecificCapUpdateReqList_r5 = -1;
static gint ett_rrc_SystemSpecificCapUpdateReqList_r8 = -1;
static gint ett_rrc_TMSI_and_LAI_GSM_MAP = -1;
static gint ett_rrc_TransportChannelCapability = -1;
static gint ett_rrc_TurboSupport = -1;
static gint ett_rrc_U_RNTI = -1;
static gint ett_rrc_U_RNTI_Group = -1;
static gint ett_rrc_U_RNTI_Short = -1;
static gint ett_rrc_UE_BasedNetworkPerformanceMeasurementsParameters = -1;
static gint ett_rrc_UE_CapabilityContainer_IEs = -1;
static gint ett_rrc_T_v6b0NonCriticalExtensions_14 = -1;
static gint ett_rrc_T_v6e0NonCriticalExtensions_02 = -1;
static gint ett_rrc_T_v770NonCriticalExtensions_39 = -1;
static gint ett_rrc_T_v790NonCriticalExtensions_01 = -1;
static gint ett_rrc_T_v860NonCriticalExtensions_25 = -1;
static gint ett_rrc_T_v880NonCriticalExtensions_01 = -1;
static gint ett_rrc_T_v890NonCriticalExtensions_10 = -1;
static gint ett_rrc_T_v920NonCriticalExtensions_03 = -1;
static gint ett_rrc_T_v970NonCriticalExtensions_03 = -1;
static gint ett_rrc_T_va40NonCriticalExtensions_17 = -1;
static gint ett_rrc_T_va60NonCriticalExtensions = -1;
static gint ett_rrc_T_va80NonCriticalExtensions_01 = -1;
static gint ett_rrc_UE_RadioAccessCapability_v6b0ext_IEs = -1;
static gint ett_rrc_UE_RadioAccessCapability_v6e0ext_IEs = -1;
static gint ett_rrc_UE_RadioAccessCapability_v770ext_IEs = -1;
static gint ett_rrc_UE_RadioAccessCapability_v790ext_IEs = -1;
static gint ett_rrc_UE_RadioAccessCapability_v860ext_IEs = -1;
static gint ett_rrc_UE_RadioAccessCapability_v880ext_IEs = -1;
static gint ett_rrc_UE_RadioAccessCapability_v890ext_IEs = -1;
static gint ett_rrc_UE_RadioAccessCapability_v920ext_IEs = -1;
static gint ett_rrc_UE_RadioAccessCapability_v970ext_IEs = -1;
static gint ett_rrc_UE_RadioAccessCapability_v9c0ext_IEs = -1;
static gint ett_rrc_UE_RadioAccessCapability_LaterNonCriticalExtensions = -1;
static gint ett_rrc_T_v9c0NonCriticalExtensions_12 = -1;
static gint ett_rrc_T_vaa0NonCriticalExtensions_02 = -1;
static gint ett_rrc_T_vbxyNonCriticalExtensions_18 = -1;
static gint ett_rrc_T_nonCriticalExtensions_194 = -1;
static gint ett_rrc_UE_RadioAccessCapability_va40ext_IEs = -1;
static gint ett_rrc_UE_RadioAccessCapability_va60ext_IEs = -1;
static gint ett_rrc_UE_RadioAccessCapability_va80ext_IEs = -1;
static gint ett_rrc_UE_RadioAccessCapability_vaa0ext_IEs = -1;
static gint ett_rrc_UE_RadioAccessCapability_vbxyext_IEs = -1;
static gint ett_rrc_UE_ConnTimersAndConstants = -1;
static gint ett_rrc_UE_ConnTimersAndConstants_v3a0ext = -1;
static gint ett_rrc_UE_ConnTimersAndConstants_r5 = -1;
static gint ett_rrc_UE_ConnTimersAndConstants_v860ext = -1;
static gint ett_rrc_UE_GANSSPositioning_Capability = -1;
static gint ett_rrc_UE_GANSSPositioning_Capability_v860ext_IEs = -1;
static gint ett_rrc_UE_IdleTimersAndConstants = -1;
static gint ett_rrc_UE_HSPA_Identities_r6 = -1;
static gint ett_rrc_UE_IdleTimersAndConstants_v3a0ext = -1;
static gint ett_rrc_UE_MultiModeRAT_Capability = -1;
static gint ett_rrc_UE_RadioAccessCapability = -1;
static gint ett_rrc_UE_RadioAccessCapabilityInfo = -1;
static gint ett_rrc_UE_RadioAccessCapability_v370ext = -1;
static gint ett_rrc_UE_RadioAccessCapability_v380ext = -1;
static gint ett_rrc_UE_RadioAccessCapability_v3a0ext = -1;
static gint ett_rrc_UE_RadioAccessCapability_v3g0ext = -1;
static gint ett_rrc_UE_RadioAccessCapability_v650ext = -1;
static gint ett_rrc_UE_RadioAccessCapability_v690ext = -1;
static gint ett_rrc_UE_RadioAccessCapability_v7e0ext = -1;
static gint ett_rrc_UE_RadioAccessCapability_v7f0ext = -1;
static gint ett_rrc_UE_RadioAccessCapabilityInfo_v770ext = -1;
static gint ett_rrc_UE_RadioAccessCapabilityInfo_TDD128_v8b0ext = -1;
static gint ett_rrc_UE_RadioAccessCapability_v860ext = -1;
static gint ett_rrc_UE_RadioAccessCapability_v880ext = -1;
static gint ett_rrc_UE_RadioAccessCapability_v920ext = -1;
static gint ett_rrc_UE_RadioAccessCapability_va40ext = -1;
static gint ett_rrc_UE_RadioAccessCapability_va80ext = -1;
static gint ett_rrc_UE_RadioAccessCapability_vbxyext = -1;
static gint ett_rrc_UE_RadioAccessCapabBand_va40ext = -1;
static gint ett_rrc_UE_RadioAccessCapabBandFDDList2 = -1;
static gint ett_rrc_UE_RadioAccessCapabBandFDDList3 = -1;
static gint ett_rrc_UE_RadioAccessCapabBandFDDList4 = -1;
static gint ett_rrc_UE_RadioAccessCapabBandFDDList4_va60ext = -1;
static gint ett_rrc_UE_RadioAccessCapabBandFDDList4_va80ext = -1;
static gint ett_rrc_UE_RadioAccessCapabBandFDDList5 = -1;
static gint ett_rrc_UE_RadioAccessCapabBandFDDList6 = -1;
static gint ett_rrc_UE_RadioAccessCapabBandFDD2 = -1;
static gint ett_rrc_T_fddRF_Capability_01 = -1;
static gint ett_rrc_UE_RadioAccessCapabBandFDD3 = -1;
static gint ett_rrc_UE_RadioAccessCapabBandFDD4 = -1;
static gint ett_rrc_UE_RadioAccessCapabBandFDD4_va60ext = -1;
static gint ett_rrc_UE_RadioAccessCapabBandFDD4_va80ext = -1;
static gint ett_rrc_UE_RadioAccessCapabBandFDD5 = -1;
static gint ett_rrc_T_fddRF_Capability_02 = -1;
static gint ett_rrc_UE_RadioAccessCapabBandFDD6 = -1;
static gint ett_rrc_UE_PositioningCapabilityExt_v380 = -1;
static gint ett_rrc_UE_PositioningCapabilityExt_v3a0 = -1;
static gint ett_rrc_UE_PositioningCapabilityExt_v3g0 = -1;
static gint ett_rrc_UE_PositioningCapability_v770ext = -1;
static gint ett_rrc_UE_PositioningCapability_v860ext = -1;
static gint ett_rrc_UE_RadioAccessCapabBandCombList = -1;
static gint ett_rrc_UE_RadioAccessCapabBandFDDList = -1;
static gint ett_rrc_UE_RadioAccessCapabBandFDDList_ext = -1;
static gint ett_rrc_UE_RadioAccessCapabBandFDDList_ext2 = -1;
static gint ett_rrc_UE_RadioAccessCapabBandFDD = -1;
static gint ett_rrc_T_fddRF_Capability_03 = -1;
static gint ett_rrc_UE_RadioAccessCapabBandFDD_ext = -1;
static gint ett_rrc_UE_RadioAccessCapabBandFDD_ext2 = -1;
static gint ett_rrc_UE_RadioAccessCapability_v4b0ext = -1;
static gint ett_rrc_T_tdd_CapabilityExt = -1;
static gint ett_rrc_UE_RadioAccessCapabilityComp = -1;
static gint ett_rrc_UE_RadioAccessCapabilityComp_ext = -1;
static gint ett_rrc_UE_RadioAccessCapabilityComp_TDD128_v7f0ext = -1;
static gint ett_rrc_UE_RadioAccessCapabilityComp_v770ext = -1;
static gint ett_rrc_UE_RadioAccessCapabilityComp2 = -1;
static gint ett_rrc_T_fddPhysicalChannelCapab_hspdsch_edch = -1;
static gint ett_rrc_UE_RadioAccessCapabilityComp2_v770ext = -1;
static gint ett_rrc_UE_RadioAccessCapabilityComp2_v7f0ext = -1;
static gint ett_rrc_UE_RadioAccessCapabilityComp2_v860ext = -1;
static gint ett_rrc_UE_RadioAccessCapabilityComp2_v920ext = -1;
static gint ett_rrc_UE_RadioAccessCapabilityComp2_va40ext = -1;
static gint ett_rrc_UE_RadioAccessCapabilityComp2_vbxyext = -1;
static gint ett_rrc_UE_RadioAccessCapabilityComp_TDD128 = -1;
static gint ett_rrc_RF_CapabilityComp = -1;
static gint ett_rrc_T_fdd_47 = -1;
static gint ett_rrc_T_tdd384_RF_Capability = -1;
static gint ett_rrc_T_tdd128_RF_Capability = -1;
static gint ett_rrc_RF_CapabilityComp_v770ext = -1;
static gint ett_rrc_T_tdd768_RF_Capability = -1;
static gint ett_rrc_RF_CapabBandListFDDComp = -1;
static gint ett_rrc_RF_CapabBandListFDDComp_ext = -1;
static gint ett_rrc_RF_CapabBandListFDDComp_ext2 = -1;
static gint ett_rrc_UE_RadioAccessCapabBandCombList_va40ext = -1;
static gint ett_rrc_UE_RadioAccessCapabBandCombList_vbxyext = -1;
static gint ett_rrc_SupportedCarrierCombination = -1;
static gint ett_rrc_SupportedCarrierCombination_ext = -1;
static gint ett_rrc_UE_RadioAccessCapability_v590ext = -1;
static gint ett_rrc_UE_RadioAccessCapability_v5c0ext = -1;
static gint ett_rrc_UE_RadioAccessCapability_v680ext = -1;
static gint ett_rrc_UL_OLTD_InfoFDD = -1;
static gint ett_rrc_UL_PhysChCapabilityFDD = -1;
static gint ett_rrc_UL_PhysChCapabilityFDD_v770ext = -1;
static gint ett_rrc_UL_PhysChCapabilityFDD_v920ext = -1;
static gint ett_rrc_UL_PhysChCapabilityTDD = -1;
static gint ett_rrc_UL_PhysChCapabilityTDD_384_v770ext = -1;
static gint ett_rrc_T_tdd384_edch = -1;
static gint ett_rrc_T_supported_03 = -1;
static gint ett_rrc_UL_PhysChCapabilityTDD_LCR_r4 = -1;
static gint ett_rrc_UL_PhysChCapabilityTDD_128_v770ext = -1;
static gint ett_rrc_T_tdd128_edch = -1;
static gint ett_rrc_T_supported_04 = -1;
static gint ett_rrc_UL_PhysChCapabilityInfoTDD_128_v770ext = -1;
static gint ett_rrc_UL_PhysChCapabilityInfoTDD_128_va40ext = -1;
static gint ett_rrc_UL_PhysChCapabilityTDD_768 = -1;
static gint ett_rrc_T_tdd384_edch_01 = -1;
static gint ett_rrc_T_supported_05 = -1;
static gint ett_rrc_UL_SecondaryCellInfoFDD = -1;
static gint ett_rrc_T_newConfiguration = -1;
static gint ett_rrc_PhysicalChannelCapability_edch_r6 = -1;
static gint ett_rrc_T_fdd_edch = -1;
static gint ett_rrc_T_supported_06 = -1;
static gint ett_rrc_UL_TransChCapability = -1;
static gint ett_rrc_T_modeSpecificInfo_18 = -1;
static gint ett_rrc_T_tdd_12 = -1;
static gint ett_rrc_UE_Positioning_Capability = -1;
static gint ett_rrc_UE_SecurityInformation = -1;
static gint ett_rrc_UE_SecurityInformation2 = -1;
static gint ett_rrc_UTRAN_DRX_CycleLengthCoefficient_r7 = -1;
static gint ett_rrc_AlgorithmSpecificInfo = -1;
static gint ett_rrc_AlgorithmSpecificInfo_r4 = -1;
static gint ett_rrc_CommonRBMappingInfo = -1;
static gint ett_rrc_CS_HSPA_Information = -1;
static gint ett_rrc_DefaultConfigForCellFACH = -1;
static gint ett_rrc_DL_AM_RLC_Mode = -1;
static gint ett_rrc_DL_AM_RLC_Mode_r5 = -1;
static gint ett_rrc_DL_AM_RLC_Mode_r7 = -1;
static gint ett_rrc_T_dl_RLC_PDU_size = -1;
static gint ett_rrc_DL_AM_RLC_Mode_r11 = -1;
static gint ett_rrc_T_dl_RLC_PDU_size_01 = -1;
static gint ett_rrc_DL_CounterSynchronisationInfo = -1;
static gint ett_rrc_DL_CounterSynchronisationInfo_r5 = -1;
static gint ett_rrc_DL_LogicalChannelMapping = -1;
static gint ett_rrc_DL_LogicalChannelMapping_r5 = -1;
static gint ett_rrc_DL_LogicalChannelMapping_r7 = -1;
static gint ett_rrc_DL_LogicalChannelMappingList = -1;
static gint ett_rrc_DL_LogicalChannelMappingList_r5 = -1;
static gint ett_rrc_DL_LogicalChannelMappingList_r7 = -1;
static gint ett_rrc_DL_RFC3095_r4 = -1;
static gint ett_rrc_DL_RLC_Mode = -1;
static gint ett_rrc_DL_RLC_Mode_r5 = -1;
static gint ett_rrc_DL_RLC_Mode_r6 = -1;
static gint ett_rrc_DL_RLC_Mode_r7 = -1;
static gint ett_rrc_DL_RLC_Mode_r11 = -1;
static gint ett_rrc_DL_RLC_StatusInfo = -1;
static gint ett_rrc_DL_RLC_StatusInfo_r11 = -1;
static gint ett_rrc_DL_TM_RLC_Mode = -1;
static gint ett_rrc_DL_TransportChannelType = -1;
static gint ett_rrc_DL_TransportChannelType_r5 = -1;
static gint ett_rrc_DL_TransportChannelType_r7 = -1;
static gint ett_rrc_T_hsdsch = -1;
static gint ett_rrc_T_dch_and_hsdsch = -1;
static gint ett_rrc_DL_UM_RLC_Mode_r5 = -1;
static gint ett_rrc_DL_UM_RLC_Mode_r6 = -1;
static gint ett_rrc_ExplicitDiscard = -1;
static gint ett_rrc_HeaderCompressionInfo = -1;
static gint ett_rrc_HeaderCompressionInfoList = -1;
static gint ett_rrc_HeaderCompressionInfo_r4 = -1;
static gint ett_rrc_HeaderCompressionInfoList_r4 = -1;
static gint ett_rrc_LosslessSRNS_RelocSupport = -1;
static gint ett_rrc_MaxDAT_Retransmissions = -1;
static gint ett_rrc_PDCP_Info = -1;
static gint ett_rrc_PDCP_Info_r4 = -1;
static gint ett_rrc_PDCP_InfoReconfig = -1;
static gint ett_rrc_PDCP_InfoReconfig_r4 = -1;
static gint ett_rrc_PollingInfo = -1;
static gint ett_rrc_PredefinedRB_Configuration = -1;
static gint ett_rrc_PreDefRadioConfiguration = -1;
static gint ett_rrc_PreDefRadioConfiguration_v770ext = -1;
static gint ett_rrc_PreDefRadioConfiguration_v920ext = -1;
static gint ett_rrc_PredefinedConfigStatusList = -1;
static gint ett_rrc_PredefinedConfigStatusInfo = -1;
static gint ett_rrc_T_other = -1;
static gint ett_rrc_PredefinedConfigStatusListComp = -1;
static gint ett_rrc_PredefinedConfigSetsWithDifferentValueTag = -1;
static gint ett_rrc_PredefinedConfigSetWithDifferentValueTag = -1;
static gint ett_rrc_PredefinedConfigValueTagList = -1;
static gint ett_rrc_PredefinedConfigStatusListVarSz = -1;
static gint ett_rrc_RAB_Info = -1;
static gint ett_rrc_RAB_Info_r6_ext = -1;
static gint ett_rrc_RAB_Info_v6b0ext = -1;
static gint ett_rrc_RAB_Info_r6 = -1;
static gint ett_rrc_RAB_Info_r7 = -1;
static gint ett_rrc_RAB_InfoReplace = -1;
static gint ett_rrc_RAB_InformationList = -1;
static gint ett_rrc_RAB_InformationList_r6 = -1;
static gint ett_rrc_RAB_InformationReconfigList = -1;
static gint ett_rrc_RAB_InformationReconfigList_r8 = -1;
static gint ett_rrc_RAB_InformationReconfig = -1;
static gint ett_rrc_RAB_InformationReconfig_r8 = -1;
static gint ett_rrc_RAB_Info_Post = -1;
static gint ett_rrc_RAB_InformationMBMSPtp = -1;
static gint ett_rrc_RAB_InformationMBMSPtpList = -1;
static gint ett_rrc_RAB_InformationSetup = -1;
static gint ett_rrc_RAB_InformationSetup_r4 = -1;
static gint ett_rrc_RAB_InformationSetup_r5 = -1;
static gint ett_rrc_RAB_InformationSetup_r6_ext = -1;
static gint ett_rrc_RAB_InformationSetup_r6 = -1;
static gint ett_rrc_RAB_InformationSetup_v6b0ext = -1;
static gint ett_rrc_RAB_InformationSetup_r7 = -1;
static gint ett_rrc_RAB_InformationSetup_r8 = -1;
static gint ett_rrc_RAB_InformationSetup_v820ext = -1;
static gint ett_rrc_RAB_InformationSetup_r11 = -1;
static gint ett_rrc_RAB_InformationSetupList = -1;
static gint ett_rrc_RAB_InformationSetupList_r4 = -1;
static gint ett_rrc_RAB_InformationSetupList_r5 = -1;
static gint ett_rrc_RAB_InformationSetupList_r6 = -1;
static gint ett_rrc_RAB_InformationSetupList_r6_ext = -1;
static gint ett_rrc_RAB_InformationSetupList_v6b0ext = -1;
static gint ett_rrc_RAB_InformationSetupList_r7 = -1;
static gint ett_rrc_RAB_InformationSetupList_r8 = -1;
static gint ett_rrc_RAB_InformationSetupList_v820ext = -1;
static gint ett_rrc_RAB_InformationSetupList_r11 = -1;
static gint ett_rrc_RB_ActivationTimeInfo = -1;
static gint ett_rrc_RB_ActivationTimeInfoList = -1;
static gint ett_rrc_RB_COUNT_C_Information = -1;
static gint ett_rrc_RB_COUNT_C_InformationList = -1;
static gint ett_rrc_RB_COUNT_C_MSB_Information = -1;
static gint ett_rrc_RB_COUNT_C_MSB_InformationList = -1;
static gint ett_rrc_RB_IdentityList = -1;
static gint ett_rrc_RB_InformationAffected = -1;
static gint ett_rrc_RB_InformationAffected_r5 = -1;
static gint ett_rrc_RB_InformationAffected_r6 = -1;
static gint ett_rrc_RB_InformationAffected_r7 = -1;
static gint ett_rrc_RB_InformationAffected_r8 = -1;
static gint ett_rrc_RB_InformationAffectedList = -1;
static gint ett_rrc_RB_InformationAffectedList_r5 = -1;
static gint ett_rrc_RB_InformationAffectedList_r6 = -1;
static gint ett_rrc_RB_InformationAffectedList_r7 = -1;
static gint ett_rrc_RB_InformationAffectedList_r8 = -1;
static gint ett_rrc_RB_InformationChanged_r6 = -1;
static gint ett_rrc_T_rb_Change = -1;
static gint ett_rrc_RB_InformationChangedList_r6 = -1;
static gint ett_rrc_RB_InformationReconfig = -1;
static gint ett_rrc_RB_InformationReconfig_r4 = -1;
static gint ett_rrc_RB_InformationReconfig_r5 = -1;
static gint ett_rrc_RB_InformationReconfig_r6 = -1;
static gint ett_rrc_RB_InformationReconfig_r7 = -1;
static gint ett_rrc_RB_InformationReconfig_r8 = -1;
static gint ett_rrc_RB_InformationReconfig_r11 = -1;
static gint ett_rrc_RB_InformationReconfigList = -1;
static gint ett_rrc_RB_InformationReconfigList_r4 = -1;
static gint ett_rrc_RB_InformationReconfigList_r5 = -1;
static gint ett_rrc_RB_InformationReconfigList_r6 = -1;
static gint ett_rrc_RB_InformationReconfigList_r7 = -1;
static gint ett_rrc_RB_InformationReconfigList_r8 = -1;
static gint ett_rrc_RB_InformationReconfigList_r11 = -1;
static gint ett_rrc_RB_InformationReleaseList = -1;
static gint ett_rrc_RB_InformationSetup = -1;
static gint ett_rrc_RB_InformationSetup_r4 = -1;
static gint ett_rrc_RB_InformationSetup_r5 = -1;
static gint ett_rrc_RB_InformationSetup_r6 = -1;
static gint ett_rrc_RB_InformationSetup_r7 = -1;
static gint ett_rrc_RB_InformationSetup_r8 = -1;
static gint ett_rrc_RB_InformationSetup_r11 = -1;
static gint ett_rrc_RB_InformationSetupList = -1;
static gint ett_rrc_RB_InformationSetupList_r4 = -1;
static gint ett_rrc_RB_InformationSetupList_r5 = -1;
static gint ett_rrc_RB_InformationSetupList_r6 = -1;
static gint ett_rrc_RB_InformationSetupList_r7 = -1;
static gint ett_rrc_RB_InformationSetupList_r8 = -1;
static gint ett_rrc_RB_InformationSetupList_r11 = -1;
static gint ett_rrc_RB_MappingInfo = -1;
static gint ett_rrc_RB_MappingInfo_r5 = -1;
static gint ett_rrc_RB_MappingInfo_r6 = -1;
static gint ett_rrc_RB_MappingInfo_r7 = -1;
static gint ett_rrc_RB_MappingInfo_r8 = -1;
static gint ett_rrc_RB_MappingOption = -1;
static gint ett_rrc_RB_MappingOption_r5 = -1;
static gint ett_rrc_RB_MappingOption_r6 = -1;
static gint ett_rrc_RB_MappingOption_r7 = -1;
static gint ett_rrc_RB_MappingOption_r8 = -1;
static gint ett_rrc_RB_PDCPContextRelocation = -1;
static gint ett_rrc_RB_PDCPContextRelocationList = -1;
static gint ett_rrc_RB_WithPDCP_Info = -1;
static gint ett_rrc_RB_WithPDCP_InfoList = -1;
static gint ett_rrc_RFC2507_Info = -1;
static gint ett_rrc_RFC3095_Info_r4 = -1;
static gint ett_rrc_RLC_Info = -1;
static gint ett_rrc_RLC_Info_r5 = -1;
static gint ett_rrc_RLC_Info_r6 = -1;
static gint ett_rrc_RLC_Info_r7 = -1;
static gint ett_rrc_RLC_Info_r11 = -1;
static gint ett_rrc_RLC_Info_MCCH_r6 = -1;
static gint ett_rrc_RLC_Info_MSCH_r6 = -1;
static gint ett_rrc_RLC_Info_MTCH_r6 = -1;
static gint ett_rrc_RLC_InfoChoice = -1;
static gint ett_rrc_RLC_InfoChoice_r5 = -1;
static gint ett_rrc_RLC_InfoChoice_r6 = -1;
static gint ett_rrc_RLC_InfoChoice_r7 = -1;
static gint ett_rrc_RLC_InfoChoice_r11 = -1;
static gint ett_rrc_RLC_PDU_SizeConstraint = -1;
static gint ett_rrc_RLC_PDU_SizeList = -1;
static gint ett_rrc_RLC_SizeInfo = -1;
static gint ett_rrc_RLC_SizeExplicitList = -1;
static gint ett_rrc_ROHC_ProfileList_r4 = -1;
static gint ett_rrc_ROHC_PacketSizeList_r4 = -1;
static gint ett_rrc_SRB_InformationSetup = -1;
static gint ett_rrc_SRB_InformationSetup_r5 = -1;
static gint ett_rrc_SRB_InformationSetup_r6 = -1;
static gint ett_rrc_SRB_InformationSetup_r7 = -1;
static gint ett_rrc_SRB_InformationSetup_r8 = -1;
static gint ett_rrc_SRB_InformationSetupList = -1;
static gint ett_rrc_SRB_InformationSetupList_r5 = -1;
static gint ett_rrc_SRB_InformationSetupList_r6 = -1;
static gint ett_rrc_SRB_InformationSetupList_r7 = -1;
static gint ett_rrc_SRB_InformationSetupList_r8 = -1;
static gint ett_rrc_SRB_InformationSetupList2 = -1;
static gint ett_rrc_SRB_InformationSetupList2_r6 = -1;
static gint ett_rrc_SRB_InformationSetupList2_r7 = -1;
static gint ett_rrc_SRB_InformationSetupList2_r8 = -1;
static gint ett_rrc_TransmissionRLC_Discard = -1;
static gint ett_rrc_UL_AM_RLC_Mode = -1;
static gint ett_rrc_UL_CounterSynchronisationInfo = -1;
static gint ett_rrc_UL_LogicalChannelMapping = -1;
static gint ett_rrc_T_rlc_SizeList = -1;
static gint ett_rrc_UL_LogicalChannelMapping_r6 = -1;
static gint ett_rrc_T_ul_TrCH_Type = -1;
static gint ett_rrc_T_dch_rach_usch = -1;
static gint ett_rrc_T_rlc_SizeList_01 = -1;
static gint ett_rrc_T_e_dch = -1;
static gint ett_rrc_UL_LogicalChannelMapping_r8 = -1;
static gint ett_rrc_T_ul_TrCH_Type_01 = -1;
static gint ett_rrc_T_dch_rach_usch_01 = -1;
static gint ett_rrc_T_rlc_SizeList_02 = -1;
static gint ett_rrc_T_e_dch_01 = -1;
static gint ett_rrc_T_rlc_PDU_Size = -1;
static gint ett_rrc_T_fixedSize = -1;
static gint ett_rrc_UL_LogicalChannelMappingList = -1;
static gint ett_rrc_SEQUENCE_SIZE_maxLoCHperRLC_OF_UL_LogicalChannelMapping = -1;
static gint ett_rrc_UL_LogicalChannelMappingList_r6 = -1;
static gint ett_rrc_SEQUENCE_SIZE_maxLoCHperRLC_OF_UL_LogicalChannelMapping_r6 = -1;
static gint ett_rrc_UL_LogicalChannelMappingList_r8 = -1;
static gint ett_rrc_SEQUENCE_SIZE_maxLoCHperRLC_OF_UL_LogicalChannelMapping_r8 = -1;
static gint ett_rrc_UL_LogicalChannelMappings = -1;
static gint ett_rrc_UL_LogicalChannelMappings_r6 = -1;
static gint ett_rrc_UL_LogicalChannelMappings_r8 = -1;
static gint ett_rrc_UL_RFC3095_r4 = -1;
static gint ett_rrc_UL_RLC_Mode = -1;
static gint ett_rrc_UL_TM_RLC_Mode = -1;
static gint ett_rrc_UL_UM_RLC_Mode = -1;
static gint ett_rrc_UL_TransportChannelType = -1;
static gint ett_rrc_UM_RLC_DuplAvoid_Reord_Info_r6 = -1;
static gint ett_rrc_UM_RLC_OutOSeqDelivery_Info_r6 = -1;
static gint ett_rrc_AddOrReconfMAC_dFlow = -1;
static gint ett_rrc_AddOrReconfMAC_ehs_ReordQ = -1;
static gint ett_rrc_AddOrReconfMAC_ehs_ReordQ_r9 = -1;
static gint ett_rrc_AddOrReconfMAC_ehs_ReordQ_TDD128_v9c0ext = -1;
static gint ett_rrc_AddOrReconfMAC_ehs_ReordQ_r11 = -1;
static gint ett_rrc_AllowedTFC_List = -1;
static gint ett_rrc_AllowedTFI_List = -1;
static gint ett_rrc_BitModeRLC_SizeInfo = -1;
static gint ett_rrc_T_sizeType2 = -1;
static gint ett_rrc_T_sizeType3 = -1;
static gint ett_rrc_T_sizeType4 = -1;
static gint ett_rrc_ChannelCodingType = -1;
static gint ett_rrc_Common_E_DCH_MAC_d_Flow = -1;
static gint ett_rrc_Common_E_DCH_MAC_d_FlowList = -1;
static gint ett_rrc_Common_E_DCH_MAC_d_Flow_Info_ConcurrentTTI = -1;
static gint ett_rrc_Common_E_DCH_MAC_d_Flow_Info_List_ConcurrentTTI = -1;
static gint ett_rrc_CommonDynamicTF_Info = -1;
static gint ett_rrc_T_rlc_Size = -1;
static gint ett_rrc_T_fdd_48 = -1;
static gint ett_rrc_T_tdd_13 = -1;
static gint ett_rrc_T_commonTDD_Choice = -1;
static gint ett_rrc_SEQUENCE_SIZE_1_maxTF_OF_NumberOfTransportBlocks = -1;
static gint ett_rrc_CommonDynamicTF_Info_DynamicTTI = -1;
static gint ett_rrc_T_commonTDD_Choice_01 = -1;
static gint ett_rrc_CommonDynamicTF_InfoList = -1;
static gint ett_rrc_CommonDynamicTF_InfoList_DynamicTTI = -1;
static gint ett_rrc_CommonTransChTFS = -1;
static gint ett_rrc_T_tti = -1;
static gint ett_rrc_CommonTransChTFS_LCR = -1;
static gint ett_rrc_T_tti_01 = -1;
static gint ett_rrc_Common_MAC_ehs_ReorderingQueue = -1;
static gint ett_rrc_Common_MAC_ehs_ReorderingQueueList = -1;
static gint ett_rrc_Concurrent_Deployment_2ms_10ms_TTI = -1;
static gint ett_rrc_T_tti_selection = -1;
static gint ett_rrc_T_fixed = -1;
static gint ett_rrc_T_threshold_based = -1;
static gint ett_rrc_DedicatedDynamicTF_Info = -1;
static gint ett_rrc_T_rlc_Size_01 = -1;
static gint ett_rrc_DedicatedDynamicTF_Info_DynamicTTI = -1;
static gint ett_rrc_T_rlc_Size_02 = -1;
static gint ett_rrc_DedicatedDynamicTF_InfoList = -1;
static gint ett_rrc_DedicatedDynamicTF_InfoList_DynamicTTI = -1;
static gint ett_rrc_DedicatedTransChTFS = -1;
static gint ett_rrc_T_tti_02 = -1;
static gint ett_rrc_DL_AddReconfTransChInfo2List = -1;
static gint ett_rrc_DL_AddReconfTransChInfoList = -1;
static gint ett_rrc_DL_AddReconfTransChInfoList_r4 = -1;
static gint ett_rrc_DL_AddReconfTransChInfoList_r5 = -1;
static gint ett_rrc_DL_AddReconfTransChInfoList_r7 = -1;
static gint ett_rrc_DL_AddReconfTransChInfoList_r9 = -1;
static gint ett_rrc_DL_AddReconfTransChInfoList_TDD128_v9c0ext = -1;
static gint ett_rrc_DL_AddReconfTransChInfoList_r11 = -1;
static gint ett_rrc_DL_AddReconfTransChInformation = -1;
static gint ett_rrc_T_tfs_SignallingMode = -1;
static gint ett_rrc_DL_AddReconfTransChInformation_r4 = -1;
static gint ett_rrc_T_tfs_SignallingMode_01 = -1;
static gint ett_rrc_DL_AddReconfTransChInformation_r5 = -1;
static gint ett_rrc_T_tfs_SignallingMode_02 = -1;
static gint ett_rrc_DL_AddReconfTransChInformation_r7 = -1;
static gint ett_rrc_T_tfs_SignallingMode_03 = -1;
static gint ett_rrc_DL_AddReconfTransChInformation_r9 = -1;
static gint ett_rrc_T_tfs_SignallingMode_04 = -1;
static gint ett_rrc_DL_AddReconfTransChInformation_TDD128_v9c0ext = -1;
static gint ett_rrc_DL_AddReconfTransChInformation_r11 = -1;
static gint ett_rrc_T_tfs_SignallingMode_05 = -1;
static gint ett_rrc_DL_AddReconfTransChInformation2 = -1;
static gint ett_rrc_T_tfs_SignallingMode_06 = -1;
static gint ett_rrc_DL_CommonTransChInfo = -1;
static gint ett_rrc_T_modeSpecificInfo_19 = -1;
static gint ett_rrc_T_fdd_49 = -1;
static gint ett_rrc_T_dl_Parameters = -1;
static gint ett_rrc_T_tdd_14 = -1;
static gint ett_rrc_DL_CommonTransChInfo_r4 = -1;
static gint ett_rrc_T_modeSpecificInfo_20 = -1;
static gint ett_rrc_T_fdd_50 = -1;
static gint ett_rrc_T_dl_Parameters_01 = -1;
static gint ett_rrc_T_dl_DCH_TFCS = -1;
static gint ett_rrc_T_tdd_15 = -1;
static gint ett_rrc_DL_DeletedTransChInfoList = -1;
static gint ett_rrc_DL_DeletedTransChInfoList_r5 = -1;
static gint ett_rrc_DL_DeletedTransChInfoList_r7 = -1;
static gint ett_rrc_DL_TransportChannelIdentity = -1;
static gint ett_rrc_DL_TransportChannelIdentity_r5 = -1;
static gint ett_rrc_DL_TransportChannelIdentity_r7 = -1;
static gint ett_rrc_DL_TrCH_TypeId1_r5 = -1;
static gint ett_rrc_DL_TrCH_TypeId2_r5 = -1;
static gint ett_rrc_DL_TrCH_TypeId2_r7 = -1;
static gint ett_rrc_T_hsdsch_01 = -1;
static gint ett_rrc_DRAC_StaticInformation = -1;
static gint ett_rrc_DRAC_StaticInformationList = -1;
static gint ett_rrc_E_DCH_AddReconf_MAC_d_Flow = -1;
static gint ett_rrc_T_transmissionGrantType = -1;
static gint ett_rrc_T_non_ScheduledTransGrantInfo = -1;
static gint ett_rrc_E_DCH_AddReconf_MAC_d_Flow_r7 = -1;
static gint ett_rrc_T_transmissionGrantType_01 = -1;
static gint ett_rrc_T_non_ScheduledTransGrantInfo_01 = -1;
static gint ett_rrc_T_modeSpecificInfo_21 = -1;
static gint ett_rrc_T_fdd_51 = -1;
static gint ett_rrc_E_DCH_AddReconf_MAC_d_FlowList = -1;
static gint ett_rrc_E_DCH_AddReconf_MAC_d_FlowList_r7 = -1;
static gint ett_rrc_ExplicitTFCS_Configuration = -1;
static gint ett_rrc_T_replacement = -1;
static gint ett_rrc_GainFactorInformation = -1;
static gint ett_rrc_HSDSCH_Info = -1;
static gint ett_rrc_HSDSCH_Info_r7 = -1;
static gint ett_rrc_T_dl_MAC_HeaderType = -1;
static gint ett_rrc_HSDSCH_Info_r9 = -1;
static gint ett_rrc_T_dl_MAC_HeaderType_01 = -1;
static gint ett_rrc_HSDSCH_Info_TDD128_v9c0ext = -1;
static gint ett_rrc_HSDSCH_Info_r11 = -1;
static gint ett_rrc_T_dl_MAC_HeaderType_02 = -1;
static gint ett_rrc_HARQ_Info = -1;
static gint ett_rrc_T_memoryPartitioning = -1;
static gint ett_rrc_SEQUENCE_SIZE_1_maxHProcesses_OF_HARQMemorySize = -1;
static gint ett_rrc_HARQ_Info_r7 = -1;
static gint ett_rrc_T_memoryPartitioning_01 = -1;
static gint ett_rrc_T_explicit = -1;
static gint ett_rrc_HARQ_Info_r11 = -1;
static gint ett_rrc_T_memoryPartitioning_02 = -1;
static gint ett_rrc_T_explicit_01 = -1;
static gint ett_rrc_IndividualDL_CCTrCH_Info = -1;
static gint ett_rrc_T_tfcs_SignallingMode = -1;
static gint ett_rrc_IndividualDL_CCTrCH_InfoList = -1;
static gint ett_rrc_IndividualUL_CCTrCH_Info = -1;
static gint ett_rrc_IndividualUL_CCTrCH_InfoList = -1;
static gint ett_rrc_LogicalChannelByRB = -1;
static gint ett_rrc_LogicalChannelList = -1;
static gint ett_rrc_SEQUENCE_SIZE_1_15_OF_LogicalChannelByRB = -1;
static gint ett_rrc_MAC_d_FlowIdentityDCHandHSDSCH = -1;
static gint ett_rrc_MAC_d_PDU_SizeInfo_List = -1;
static gint ett_rrc_MAC_d_PDUsizeInfo = -1;
static gint ett_rrc_MAC_hs_AddReconfQueue_List = -1;
static gint ett_rrc_MAC_hs_AddReconfQueue = -1;
static gint ett_rrc_MAC_hs_DelQueue_List = -1;
static gint ett_rrc_MAC_hs_DelQueue = -1;
static gint ett_rrc_MAC_ehs_AddReconfReordQ_List = -1;
static gint ett_rrc_MAC_ehs_AddReconfReordQ_List_r9 = -1;
static gint ett_rrc_MAC_ehs_AddReconfReordQ_List_TDD128_v9c0ext = -1;
static gint ett_rrc_MAC_ehs_AddReconfReordQ_List_r11 = -1;
static gint ett_rrc_MAC_ehs_AddReconfReordQ = -1;
static gint ett_rrc_MAC_ehs_AddReconfReordQ_r9 = -1;
static gint ett_rrc_MAC_ehs_AddReconfReordQ_TDD128_v9c0ext = -1;
static gint ett_rrc_MAC_ehs_AddReconfReordQ_r11 = -1;
static gint ett_rrc_MAC_ehs_DelReordQ_List = -1;
static gint ett_rrc_MAC_ehs_DelReordQ = -1;
static gint ett_rrc_MAC_ehs_QueueIdDCHandHSDSCH = -1;
static gint ett_rrc_NumberOfTbSizeAndTTIList = -1;
static gint ett_rrc_NumberOfTbSizeAndTTIList_item = -1;
static gint ett_rrc_Non_allowedTFC_List = -1;
static gint ett_rrc_NumberOfTransportBlocks = -1;
static gint ett_rrc_OctetModeRLC_SizeInfoType1 = -1;
static gint ett_rrc_T_sizeType2_01 = -1;
static gint ett_rrc_T_sizeType3_01 = -1;
static gint ett_rrc_OctetModeRLC_SizeInfoType2 = -1;
static gint ett_rrc_PowerOffsetInfoShort = -1;
static gint ett_rrc_T_modeSpecificInfo_22 = -1;
static gint ett_rrc_T_fdd_52 = -1;
static gint ett_rrc_PowerOffsetInformation = -1;
static gint ett_rrc_PreDefTransChConfiguration = -1;
static gint ett_rrc_QualityTarget = -1;
static gint ett_rrc_RestrictedTrChInfo = -1;
static gint ett_rrc_RestrictedTrChInfoList = -1;
static gint ett_rrc_SemistaticTF_Information = -1;
static gint ett_rrc_SignalledGainFactors = -1;
static gint ett_rrc_T_modeSpecificInfo_23 = -1;
static gint ett_rrc_T_fdd_53 = -1;
static gint ett_rrc_SplitTFCI_Signalling = -1;
static gint ett_rrc_TFC_Subset = -1;
static gint ett_rrc_TFC_SubsetList = -1;
static gint ett_rrc_TFC_SubsetList_item = -1;
static gint ett_rrc_T_modeSpecificInfo_24 = -1;
static gint ett_rrc_T_tdd_16 = -1;
static gint ett_rrc_TFCI_Field2_Information = -1;
static gint ett_rrc_TFCI_Range = -1;
static gint ett_rrc_TFCI_RangeList = -1;
static gint ett_rrc_TFCS = -1;
static gint ett_rrc_TFCS_Identity = -1;
static gint ett_rrc_TFCS_InfoForDSCH = -1;
static gint ett_rrc_TFCS_ReconfAdd = -1;
static gint ett_rrc_T_ctfcSize = -1;
static gint ett_rrc_T_ctfc2Bit = -1;
static gint ett_rrc_T_ctfc2Bit_item = -1;
static gint ett_rrc_T_ctfc4Bit = -1;
static gint ett_rrc_T_ctfc4Bit_item = -1;
static gint ett_rrc_T_ctfc6Bit = -1;
static gint ett_rrc_T_ctfc6Bit_item = -1;
static gint ett_rrc_T_ctfc8Bit = -1;
static gint ett_rrc_T_ctfc8Bit_item = -1;
static gint ett_rrc_T_ctfc12Bit = -1;
static gint ett_rrc_T_ctfc12Bit_item = -1;
static gint ett_rrc_T_ctfc16Bit = -1;
static gint ett_rrc_T_ctfc16Bit_item = -1;
static gint ett_rrc_T_ctfc24Bit = -1;
static gint ett_rrc_T_ctfc24Bit_item = -1;
static gint ett_rrc_TFCS_Removal = -1;
static gint ett_rrc_TFCS_RemovalList = -1;
static gint ett_rrc_TM_SignallingInfo = -1;
static gint ett_rrc_T_tm_SignallingMode = -1;
static gint ett_rrc_T_mode2 = -1;
static gint ett_rrc_TransportChannelIdentityDCHandDSCH = -1;
static gint ett_rrc_TransportFormatSet = -1;
static gint ett_rrc_TransportFormatSet_LCR = -1;
static gint ett_rrc_UL_AddReconfTransChInfoList = -1;
static gint ett_rrc_UL_AddReconfTransChInfoList_r6 = -1;
static gint ett_rrc_UL_AddReconfTransChInfoList_r7 = -1;
static gint ett_rrc_UL_AddReconfTransChInfoList_r8 = -1;
static gint ett_rrc_UL_AddReconfTransChInformation = -1;
static gint ett_rrc_UL_AddReconfTransChInformation_r6 = -1;
static gint ett_rrc_T_dch_usch = -1;
static gint ett_rrc_T_e_dch_02 = -1;
static gint ett_rrc_UL_AddReconfTransChInformation_r7 = -1;
static gint ett_rrc_T_dch_usch_01 = -1;
static gint ett_rrc_T_e_dch_03 = -1;
static gint ett_rrc_T_modeSpecific = -1;
static gint ett_rrc_T_fdd_54 = -1;
static gint ett_rrc_UL_AddReconfTransChInformation_r8 = -1;
static gint ett_rrc_T_dch_usch_02 = -1;
static gint ett_rrc_T_e_dch_04 = -1;
static gint ett_rrc_T_modeSpecific_01 = -1;
static gint ett_rrc_T_fdd_55 = -1;
static gint ett_rrc_UL_CommonTransChInfo = -1;
static gint ett_rrc_T_modeSpecificInfo_25 = -1;
static gint ett_rrc_T_fdd_56 = -1;
static gint ett_rrc_T_tdd_17 = -1;
static gint ett_rrc_UL_CommonTransChInfo_r4 = -1;
static gint ett_rrc_T_modeSpecificInfo_26 = -1;
static gint ett_rrc_T_fdd_57 = -1;
static gint ett_rrc_T_tdd_18 = -1;
static gint ett_rrc_UL_ControlledTrChList = -1;
static gint ett_rrc_UL_DeletedTransChInfoList = -1;
static gint ett_rrc_UL_DeletedTransChInfoList_r6 = -1;
static gint ett_rrc_UL_TransportChannelIdentity = -1;
static gint ett_rrc_UL_TransportChannelIdentity_r6 = -1;
static gint ett_rrc_T_dch_usch_03 = -1;
static gint ett_rrc_USCH_TransportChannelsInfo = -1;
static gint ett_rrc_USCH_TransportChannelsInfo_item = -1;
static gint ett_rrc_AC_To_ASC_MappingTable = -1;
static gint ett_rrc_AccessServiceClass_FDD = -1;
static gint ett_rrc_T_assignedSubChannelNumber = -1;
static gint ett_rrc_AccessServiceClass_TDD = -1;
static gint ett_rrc_T_channelisationCodeIndices = -1;
static gint ett_rrc_T_subchannelSize = -1;
static gint ett_rrc_T_size2 = -1;
static gint ett_rrc_T_size4 = -1;
static gint ett_rrc_T_subchannels_01 = -1;
static gint ett_rrc_T_size8 = -1;
static gint ett_rrc_T_subchannels_02 = -1;
static gint ett_rrc_AccessServiceClass_TDD_r7 = -1;
static gint ett_rrc_T_channelisationCodeIndices_01 = -1;
static gint ett_rrc_T_subchannelSize_01 = -1;
static gint ett_rrc_T_size2_01 = -1;
static gint ett_rrc_T_size4_01 = -1;
static gint ett_rrc_T_subchannels_04 = -1;
static gint ett_rrc_T_size8_01 = -1;
static gint ett_rrc_T_subchannels_05 = -1;
static gint ett_rrc_T_size16 = -1;
static gint ett_rrc_T_subchannels_06 = -1;
static gint ett_rrc_AccessServiceClass_TDD_LCR_r4 = -1;
static gint ett_rrc_T_availableSYNC_UlCodesIndics = -1;
static gint ett_rrc_T_subchannelSize_02 = -1;
static gint ett_rrc_T_size2_02 = -1;
static gint ett_rrc_T_size4_02 = -1;
static gint ett_rrc_T_subchannels_08 = -1;
static gint ett_rrc_T_size8_02 = -1;
static gint ett_rrc_T_subchannels_09 = -1;
static gint ett_rrc_AdditionalPRACH_TF_and_TFCS_CCCH_IEs = -1;
static gint ett_rrc_AdditionalPRACH_TF_and_TFCS_CCCH = -1;
static gint ett_rrc_AdditionalPRACH_TF_and_TFCS_CCCH_List = -1;
static gint ett_rrc_AdditionalDLSecCellInfoListFDD = -1;
static gint ett_rrc_AdditionalDLSecCellInfoListFDD_r11 = -1;
static gint ett_rrc_AdditionalDLSecCellInfoListFDD2_r11 = -1;
static gint ett_rrc_AdditionalDLSecCellInfoFDD = -1;
static gint ett_rrc_AdditionalDLSecCellInfoFDD_r11 = -1;
static gint ett_rrc_AICH_Info = -1;
static gint ett_rrc_AICH_Info_Compressed = -1;
static gint ett_rrc_AllocationPeriodInfo = -1;
static gint ett_rrc_Antenna3And4 = -1;
static gint ett_rrc_Antenna3And4_S_CPICH = -1;
static gint ett_rrc_Antenna3And4_D_CPICH = -1;
static gint ett_rrc_Antenna3_D_CPICH = -1;
static gint ett_rrc_Antenna4_D_CPICH = -1;
static gint ett_rrc_Antenna3_S_CPICH = -1;
static gint ett_rrc_Antenna4_S_CPICH = -1;
static gint ett_rrc_AP_Signature_VCAM = -1;
static gint ett_rrc_ASCSetting_FDD = -1;
static gint ett_rrc_ASCSetting_TDD = -1;
static gint ett_rrc_ASCSetting_TDD_r7 = -1;
static gint ett_rrc_ASCSetting_TDD_LCR_r4 = -1;
static gint ett_rrc_AvailableAP_Signature_VCAMList = -1;
static gint ett_rrc_AvailableAP_SignatureList = -1;
static gint ett_rrc_AvailableAP_SubchannelList = -1;
static gint ett_rrc_AvailableMinimumSF_ListVCAM = -1;
static gint ett_rrc_AvailableMinimumSF_VCAM = -1;
static gint ett_rrc_AvailableSignatures = -1;
static gint ett_rrc_AvailableSubChannelNumbers = -1;
static gint ett_rrc_CCTrCH_PowerControlInfo = -1;
static gint ett_rrc_CCTrCH_PowerControlInfo_r4 = -1;
static gint ett_rrc_CCTrCH_PowerControlInfo_r5 = -1;
static gint ett_rrc_CCTrCH_PowerControlInfo_r7 = -1;
static gint ett_rrc_CD_AccessSlotSubchannelList = -1;
static gint ett_rrc_CD_SignatureCodeList = -1;
static gint ett_rrc_CellAndChannelIdentity = -1;
static gint ett_rrc_ChannelAssignmentActive = -1;
static gint ett_rrc_ChannelReqParamsForUCSM = -1;
static gint ett_rrc_CodeRange = -1;
static gint ett_rrc_CodeResourceInformation_TDD128 = -1;
static gint ett_rrc_Common_E_DCH_ResourceInfoList = -1;
static gint ett_rrc_Common_E_DCH_ResourceInfoListExt = -1;
static gint ett_rrc_Common_E_RNTI_Info = -1;
static gint ett_rrc_Common_E_RNTI_Info_item = -1;
static gint ett_rrc_CommonEDCHResourceConfigInfoListExt = -1;
static gint ett_rrc_CommonEDCHSystemInfo = -1;
static gint ett_rrc_T_modeSpecificInfo_27 = -1;
static gint ett_rrc_T_dummy_14 = -1;
static gint ett_rrc_T_tdd_19 = -1;
static gint ett_rrc_T_tdd128_11 = -1;
static gint ett_rrc_T_ccch_transmission_Info = -1;
static gint ett_rrc_CommonEDCHSystemInfoFDD = -1;
static gint ett_rrc_SEQUENCE_SIZE_1_maxEDCHs_OF_Common_E_DCH_ResourceInfoList = -1;
static gint ett_rrc_CommonEDCHSystemInfoParamConcurrentTTI = -1;
static gint ett_rrc_CommonERGCHChannelConfig = -1;
static gint ett_rrc_CommonERGCHInfoFDD = -1;
static gint ett_rrc_T_configurationInfo = -1;
static gint ett_rrc_T_newConfiguration_01 = -1;
static gint ett_rrc_SEQUENCE_SIZE_1_maxCellMeas_OF_CommonERGCHChannelConfig = -1;
static gint ett_rrc_CommonTimeslotInfo = -1;
static gint ett_rrc_CommonTimeslotInfoMBMS = -1;
static gint ett_rrc_CommonTimeslotInfoSCCPCH = -1;
static gint ett_rrc_ControlChannelDRXInfo_TDD128_r8 = -1;
static gint ett_rrc_T_controlChannelDrxOperation = -1;
static gint ett_rrc_T_continue = -1;
static gint ett_rrc_T_newOperation = -1;
static gint ett_rrc_CPCH_PersistenceLevels = -1;
static gint ett_rrc_CPCH_PersistenceLevelsList = -1;
static gint ett_rrc_CPCH_SetInfo = -1;
static gint ett_rrc_CPCH_SetInfoList = -1;
static gint ett_rrc_DL_CCTrCh = -1;
static gint ett_rrc_DL_CCTrCh_r4 = -1;
static gint ett_rrc_T_tddOption_03 = -1;
static gint ett_rrc_T_tdd384_11 = -1;
static gint ett_rrc_T_tdd128_12 = -1;
static gint ett_rrc_DL_CCTrCh_r7 = -1;
static gint ett_rrc_T_tddOption_04 = -1;
static gint ett_rrc_T_tdd384_12 = -1;
static gint ett_rrc_T_tdd768_06 = -1;
static gint ett_rrc_T_tdd128_13 = -1;
static gint ett_rrc_DL_CCTrChList = -1;
static gint ett_rrc_DL_CCTrChList_r7 = -1;
static gint ett_rrc_DL_CCTrChList_r4 = -1;
static gint ett_rrc_DL_CCTrChListToRemove = -1;
static gint ett_rrc_DL_ChannelisationCode = -1;
static gint ett_rrc_DL_ChannelisationCodeList = -1;
static gint ett_rrc_DL_CommonInformation = -1;
static gint ett_rrc_T_modeSpecificInfo_28 = -1;
static gint ett_rrc_T_fdd_58 = -1;
static gint ett_rrc_T_tdd_20 = -1;
static gint ett_rrc_DL_CommonInformation_r4 = -1;
static gint ett_rrc_T_modeSpecificInfo_29 = -1;
static gint ett_rrc_T_fdd_59 = -1;
static gint ett_rrc_T_tdd_21 = -1;
static gint ett_rrc_T_tddOption_05 = -1;
static gint ett_rrc_T_tdd128_14 = -1;
static gint ett_rrc_DL_CommonInformation_r5 = -1;
static gint ett_rrc_T_modeSpecificInfo_30 = -1;
static gint ett_rrc_T_fdd_60 = -1;
static gint ett_rrc_T_tdd_22 = -1;
static gint ett_rrc_T_tddOption_06 = -1;
static gint ett_rrc_T_tdd128_15 = -1;
static gint ett_rrc_DL_CommonInformation_r6 = -1;
static gint ett_rrc_T_dl_dpchInfoCommon = -1;
static gint ett_rrc_T_modeSpecificInfo_31 = -1;
static gint ett_rrc_T_fdd_61 = -1;
static gint ett_rrc_T_tdd_23 = -1;
static gint ett_rrc_T_tddOption_07 = -1;
static gint ett_rrc_T_tdd128_16 = -1;
static gint ett_rrc_DL_CommonInformation_r7 = -1;
static gint ett_rrc_T_dl_dpchInfoCommon_01 = -1;
static gint ett_rrc_T_modeSpecificInfo_32 = -1;
static gint ett_rrc_T_fdd_62 = -1;
static gint ett_rrc_T_tdd_24 = -1;
static gint ett_rrc_T_tddOption_08 = -1;
static gint ett_rrc_T_tdd128_17 = -1;
static gint ett_rrc_DL_CommonInformation_r8 = -1;
static gint ett_rrc_T_dl_dpchInfoCommon_02 = -1;
static gint ett_rrc_T_modeSpecificInfo_33 = -1;
static gint ett_rrc_T_fdd_63 = -1;
static gint ett_rrc_T_tdd_25 = -1;
static gint ett_rrc_T_tddOption_09 = -1;
static gint ett_rrc_T_tdd128_18 = -1;
static gint ett_rrc_DL_CommonInformation_r10 = -1;
static gint ett_rrc_T_dl_dpchInfoCommon_03 = -1;
static gint ett_rrc_T_modeSpecificInfo_34 = -1;
static gint ett_rrc_T_fdd_64 = -1;
static gint ett_rrc_T_tdd_26 = -1;
static gint ett_rrc_T_tddOption_10 = -1;
static gint ett_rrc_T_tdd128_19 = -1;
static gint ett_rrc_DL_CommonInformation_r11 = -1;
static gint ett_rrc_T_dl_dpchInfoCommon_04 = -1;
static gint ett_rrc_T_modeSpecificInfo_35 = -1;
static gint ett_rrc_T_fdd_65 = -1;
static gint ett_rrc_T_tdd_27 = -1;
static gint ett_rrc_T_tddOption_11 = -1;
static gint ett_rrc_T_tdd128_20 = -1;
static gint ett_rrc_DL_CommonInformationPost = -1;
static gint ett_rrc_DL_CommonInformationPredef = -1;
static gint ett_rrc_DL_DPCH_InfoCommon = -1;
static gint ett_rrc_T_cfnHandling = -1;
static gint ett_rrc_T_initialise = -1;
static gint ett_rrc_T_modeSpecificInfo_36 = -1;
static gint ett_rrc_T_fdd_66 = -1;
static gint ett_rrc_T_tdd_28 = -1;
static gint ett_rrc_DL_DPCH_InfoCommon_r4 = -1;
static gint ett_rrc_T_cfnHandling_01 = -1;
static gint ett_rrc_T_initialise_01 = -1;
static gint ett_rrc_T_modeSpecificInfo_37 = -1;
static gint ett_rrc_T_fdd_67 = -1;
static gint ett_rrc_T_tdd_29 = -1;
static gint ett_rrc_DL_DPCH_InfoCommon_r6 = -1;
static gint ett_rrc_T_cfnHandling_02 = -1;
static gint ett_rrc_T_maintain = -1;
static gint ett_rrc_T_modeSpecificInfo_38 = -1;
static gint ett_rrc_T_fdd_68 = -1;
static gint ett_rrc_T_tdd_30 = -1;
static gint ett_rrc_DL_DPCH_InfoCommonPost = -1;
static gint ett_rrc_DL_DPCH_InfoCommonPredef = -1;
static gint ett_rrc_T_modeSpecificInfo_39 = -1;
static gint ett_rrc_T_fdd_69 = -1;
static gint ett_rrc_T_tdd_31 = -1;
static gint ett_rrc_DL_DPCH_InfoPerRL = -1;
static gint ett_rrc_T_fdd_70 = -1;
static gint ett_rrc_T_tdd_32 = -1;
static gint ett_rrc_DL_DPCH_InfoPerRL_r4 = -1;
static gint ett_rrc_T_fdd_71 = -1;
static gint ett_rrc_T_tdd_33 = -1;
static gint ett_rrc_DL_DPCH_InfoPerRL_r5 = -1;
static gint ett_rrc_T_fdd_72 = -1;
static gint ett_rrc_T_tdd_34 = -1;
static gint ett_rrc_DL_DPCH_InfoPerRL_r6 = -1;
static gint ett_rrc_T_fdd_73 = -1;
static gint ett_rrc_T_tdd_35 = -1;
static gint ett_rrc_DL_DPCH_InfoPerRL_r7 = -1;
static gint ett_rrc_T_fdd_74 = -1;
static gint ett_rrc_T_tdd_36 = -1;
static gint ett_rrc_DL_FDPCH_InfoPerRL_r6 = -1;
static gint ett_rrc_DL_FDPCH_InfoPerRL_r7 = -1;
static gint ett_rrc_DL_DPCH_InfoPerRL_PostFDD = -1;
static gint ett_rrc_DL_DPCH_InfoPerRL_PostTDD = -1;
static gint ett_rrc_DL_DPCH_InfoPerRL_PostTDD_LCR_r4 = -1;
static gint ett_rrc_DL_DPCH_PowerControlInfo = -1;
static gint ett_rrc_T_modeSpecificInfo_40 = -1;
static gint ett_rrc_T_fdd_75 = -1;
static gint ett_rrc_T_tdd_37 = -1;
static gint ett_rrc_DL_FDPCH_InfoCommon_r6 = -1;
static gint ett_rrc_T_cfnHandling_03 = -1;
static gint ett_rrc_T_maintain_01 = -1;
static gint ett_rrc_DL_HSPDSCH_Information = -1;
static gint ett_rrc_T_modeSpecificInfo_41 = -1;
static gint ett_rrc_T_tdd_38 = -1;
static gint ett_rrc_T_tdd384_13 = -1;
static gint ett_rrc_T_tdd128_21 = -1;
static gint ett_rrc_DL_HSPDSCH_Information_r6 = -1;
static gint ett_rrc_T_modeSpecificInfo_42 = -1;
static gint ett_rrc_T_tdd_39 = -1;
static gint ett_rrc_T_tdd384_14 = -1;
static gint ett_rrc_T_tdd128_22 = -1;
static gint ett_rrc_DL_HSPDSCH_Information_r7 = -1;
static gint ett_rrc_T_modeSpecificInfo_43 = -1;
static gint ett_rrc_T_tdd_40 = -1;
static gint ett_rrc_T_tdd384_15 = -1;
static gint ett_rrc_T_tdd768_07 = -1;
static gint ett_rrc_T_tdd128_23 = -1;
static gint ett_rrc_T_fdd_76 = -1;
static gint ett_rrc_DL_HSPDSCH_Information_r8 = -1;
static gint ett_rrc_T_modeSpecificInfo_44 = -1;
static gint ett_rrc_T_tdd_41 = -1;
static gint ett_rrc_T_tdd384_16 = -1;
static gint ett_rrc_T_tdd768_08 = -1;
static gint ett_rrc_T_tdd128_24 = -1;
static gint ett_rrc_T_fdd_77 = -1;
static gint ett_rrc_DL_HSPDSCH_Information_r8_ext = -1;
static gint ett_rrc_T_modeSpecificInfo_45 = -1;
static gint ett_rrc_T_tdd_42 = -1;
static gint ett_rrc_T_tdd128_25 = -1;
static gint ett_rrc_DL_HSPDSCH_Information_r8_ext2 = -1;
static gint ett_rrc_DL_HSPDSCH_Information_r9 = -1;
static gint ett_rrc_T_modeSpecificInfo_46 = -1;
static gint ett_rrc_T_tdd_43 = -1;
static gint ett_rrc_T_tdd384_17 = -1;
static gint ett_rrc_T_tdd768_09 = -1;
static gint ett_rrc_T_tdd128_26 = -1;
static gint ett_rrc_T_fdd_78 = -1;
static gint ett_rrc_DL_HSPDSCH_Information_r11 = -1;
static gint ett_rrc_T_modeSpecificInfo_47 = -1;
static gint ett_rrc_T_tdd_44 = -1;
static gint ett_rrc_T_tdd384_18 = -1;
static gint ett_rrc_T_tdd768_10 = -1;
static gint ett_rrc_T_tdd128_27 = -1;
static gint ett_rrc_T_fdd_79 = -1;
static gint ett_rrc_DL_HSPDSCH_MultiCarrier_Information = -1;
static gint ett_rrc_DL_HSPDSCH_MultiCarrier_Information_item = -1;
static gint ett_rrc_SEQUENCE_SIZE_1_maxHSSCCHs_OF_HS_SCCH_TDD128_MultiCarrier = -1;
static gint ett_rrc_DL_HSPDSCH_TS_Configuration = -1;
static gint ett_rrc_DL_HSPDSCH_TS_Configuration_item = -1;
static gint ett_rrc_DL_HSPDSCH_TS_Configuration_VHCR = -1;
static gint ett_rrc_DL_HSPDSCH_TS_Configuration_VHCR_item = -1;
static gint ett_rrc_DL_InformationPerRL = -1;
static gint ett_rrc_T_modeSpecificInfo_48 = -1;
static gint ett_rrc_T_fdd_80 = -1;
static gint ett_rrc_DL_InformationPerRL_r4 = -1;
static gint ett_rrc_T_modeSpecificInfo_49 = -1;
static gint ett_rrc_T_fdd_81 = -1;
static gint ett_rrc_DL_InformationPerRL_r5 = -1;
static gint ett_rrc_T_modeSpecificInfo_50 = -1;
static gint ett_rrc_T_fdd_82 = -1;
static gint ett_rrc_DL_InformationPerRL_r5bis = -1;
static gint ett_rrc_T_modeSpecificInfo_51 = -1;
static gint ett_rrc_T_fdd_83 = -1;
static gint ett_rrc_DL_InformationPerRL_r6 = -1;
static gint ett_rrc_T_modeSpecificInfo_52 = -1;
static gint ett_rrc_T_fdd_84 = -1;
static gint ett_rrc_T_dl_dpchInfo = -1;
static gint ett_rrc_T_e_HICH_Info = -1;
static gint ett_rrc_T_e_RGCH_Info_01 = -1;
static gint ett_rrc_DL_InformationPerRL_v6b0ext = -1;
static gint ett_rrc_DL_InformationPerRL_r7 = -1;
static gint ett_rrc_T_modeSpecificInfo_53 = -1;
static gint ett_rrc_T_fdd_85 = -1;
static gint ett_rrc_T_dl_dpchInfo_01 = -1;
static gint ett_rrc_T_modeSpecificInfo2 = -1;
static gint ett_rrc_T_fdd_86 = -1;
static gint ett_rrc_T_e_HICH_Info_01 = -1;
static gint ett_rrc_T_e_RGCH_Info_02 = -1;
static gint ett_rrc_T_tdd_45 = -1;
static gint ett_rrc_T_tdd384_tdd768 = -1;
static gint ett_rrc_T_tdd128_28 = -1;
static gint ett_rrc_DL_InformationPerRL_r8 = -1;
static gint ett_rrc_T_modeSpecificInfo_54 = -1;
static gint ett_rrc_T_fdd_87 = -1;
static gint ett_rrc_T_dl_dpchInfo_02 = -1;
static gint ett_rrc_T_modeSpecificInfo2_01 = -1;
static gint ett_rrc_T_fdd_88 = -1;
static gint ett_rrc_T_e_HICH_Info_02 = -1;
static gint ett_rrc_T_e_RGCH_Info_03 = -1;
static gint ett_rrc_T_tdd_46 = -1;
static gint ett_rrc_T_tdd384_tdd768_01 = -1;
static gint ett_rrc_T_tdd128_29 = -1;
static gint ett_rrc_DL_InformationPerRL_r11 = -1;
static gint ett_rrc_T_modeSpecificInfo_55 = -1;
static gint ett_rrc_T_fdd_89 = -1;
static gint ett_rrc_T_dl_dpchInfo_03 = -1;
static gint ett_rrc_T_modeSpecificInfo2_02 = -1;
static gint ett_rrc_T_fdd_90 = -1;
static gint ett_rrc_T_e_HICH_Info_03 = -1;
static gint ett_rrc_T_e_RGCH_Info_04 = -1;
static gint ett_rrc_T_f_TPICH_Info = -1;
static gint ett_rrc_T_tdd_47 = -1;
static gint ett_rrc_T_tdd384_tdd768_02 = -1;
static gint ett_rrc_T_tdd128_30 = -1;
static gint ett_rrc_DL_InformationPerRL_List = -1;
static gint ett_rrc_DL_InformationPerRL_List_r4 = -1;
static gint ett_rrc_DL_InformationPerRL_List_r5 = -1;
static gint ett_rrc_DL_InformationPerRL_List_r6 = -1;
static gint ett_rrc_DL_InformationPerRL_List_v6b0ext = -1;
static gint ett_rrc_DL_InformationPerRL_List_r5bis = -1;
static gint ett_rrc_DL_InformationPerRL_List_r7 = -1;
static gint ett_rrc_DL_InformationPerRL_List_r8 = -1;
static gint ett_rrc_DL_InformationPerRL_List_r11 = -1;
static gint ett_rrc_DL_InformationPerRL_ListPostFDD = -1;
static gint ett_rrc_DL_InformationPerRL_PostFDD = -1;
static gint ett_rrc_DL_InformationPerRL_PostTDD = -1;
static gint ett_rrc_DL_InformationPerRL_PostTDD_LCR_r4 = -1;
static gint ett_rrc_DL_MultiCarrier_Information = -1;
static gint ett_rrc_DL_PDSCH_Information = -1;
static gint ett_rrc_Dl_rate_matching_restriction = -1;
static gint ett_rrc_DL_SecondaryCellInfoFDD = -1;
static gint ett_rrc_T_newConfiguration_02 = -1;
static gint ett_rrc_SEQUENCE_SIZE_1_maxHSSCCHs_OF_HS_SCCH_Codes = -1;
static gint ett_rrc_DL_SecondaryCellInfoFDD_v890ext = -1;
static gint ett_rrc_DL_SecondaryCellInfoFDD_r9 = -1;
static gint ett_rrc_T_configurationInfo_01 = -1;
static gint ett_rrc_T_newConfiguration_03 = -1;
static gint ett_rrc_DL_SecondaryCellInfoFDD_r10 = -1;
static gint ett_rrc_T_configurationInfo_02 = -1;
static gint ett_rrc_T_newConfiguration_04 = -1;
static gint ett_rrc_DL_SecondaryCellInfoFDD_r11 = -1;
static gint ett_rrc_T_configurationInfo_03 = -1;
static gint ett_rrc_T_newConfiguration_05 = -1;
static gint ett_rrc_DL_TPC_PowerOffsetPerRL = -1;
static gint ett_rrc_DL_TPC_PowerOffsetPerRL_List = -1;
static gint ett_rrc_DL_TS_ChannelisationCodesShort = -1;
static gint ett_rrc_T_codesRepresentation = -1;
static gint ett_rrc_T_consecutive = -1;
static gint ett_rrc_T_bitmap = -1;
static gint ett_rrc_DL_TS_ChannelisationCodesShort_VHCR = -1;
static gint ett_rrc_T_codesRepresentation_01 = -1;
static gint ett_rrc_T_consecutive_01 = -1;
static gint ett_rrc_T_bitmap_01 = -1;
static gint ett_rrc_DL_ChannelCodes_MBSFN_IMB384 = -1;
static gint ett_rrc_DLUL_HSPA_Information_r8 = -1;
static gint ett_rrc_DLUL_HSPA_Information_r9 = -1;
static gint ett_rrc_DLUL_HSPA_Information_r10 = -1;
static gint ett_rrc_DownlinkAdditionalTimeslots = -1;
static gint ett_rrc_T_parameters = -1;
static gint ett_rrc_T_sameAsLast = -1;
static gint ett_rrc_T_newParameters = -1;
static gint ett_rrc_DownlinkAdditionalTimeslots_VHCR = -1;
static gint ett_rrc_T_parameters_01 = -1;
static gint ett_rrc_T_sameAsLast_01 = -1;
static gint ett_rrc_T_newParameters_01 = -1;
static gint ett_rrc_DownlinkAdditionalTimeslots_LCR_r4 = -1;
static gint ett_rrc_T_parameters_02 = -1;
static gint ett_rrc_T_sameAsLast_02 = -1;
static gint ett_rrc_T_newParameters_02 = -1;
static gint ett_rrc_DownlinkAdditionalTimeslots_r7 = -1;
static gint ett_rrc_T_parameters_03 = -1;
static gint ett_rrc_T_sameAsLast_03 = -1;
static gint ett_rrc_T_newParameters_03 = -1;
static gint ett_rrc_DownlinkTimeslotsCodes = -1;
static gint ett_rrc_T_moreTimeslots = -1;
static gint ett_rrc_T_additionalTimeslots = -1;
static gint ett_rrc_SEQUENCE_SIZE_1_maxTS_1_OF_DownlinkAdditionalTimeslots = -1;
static gint ett_rrc_DownlinkTimeslotsCodes_VHCR = -1;
static gint ett_rrc_T_moreTimeslots_01 = -1;
static gint ett_rrc_T_additionalTimeslots_01 = -1;
static gint ett_rrc_SEQUENCE_SIZE_1_maxTS_1_OF_DownlinkAdditionalTimeslots_VHCR = -1;
static gint ett_rrc_DownlinkTimeslotsCodes_LCR_r4 = -1;
static gint ett_rrc_T_moreTimeslots_02 = -1;
static gint ett_rrc_T_additionalTimeslots_02 = -1;
static gint ett_rrc_SEQUENCE_SIZE_1_maxTS_LCR_1_OF_DownlinkAdditionalTimeslots_LCR_r4 = -1;
static gint ett_rrc_DownlinkTimeslotsCodes_r7 = -1;
static gint ett_rrc_T_moreTimeslots_03 = -1;
static gint ett_rrc_T_additionalTimeslots_03 = -1;
static gint ett_rrc_SEQUENCE_SIZE_1_maxTS_1_OF_DownlinkAdditionalTimeslots_r7 = -1;
static gint ett_rrc_DPCH_CompressedModeInfo = -1;
static gint ett_rrc_DPCH_CompressedModeInfo_r8 = -1;
static gint ett_rrc_DPCH_CompressedModeInfo_r10 = -1;
static gint ett_rrc_DPCH_CompressedModeStatusInfo = -1;
static gint ett_rrc_SEQUENCE_SIZE_1_maxTGPS_OF_TGP_SequenceShort = -1;
static gint ett_rrc_DPCH_CompressedModeStatusInfo_r10 = -1;
static gint ett_rrc_SEQUENCE_SIZE_1_maxTGPS_OF_TGP_SequenceShort_r10 = -1;
static gint ett_rrc_DRX_Info = -1;
static gint ett_rrc_DSCH_Mapping = -1;
static gint ett_rrc_DSCH_MappingList = -1;
static gint ett_rrc_DSCH_TransportChannelsInfo = -1;
static gint ett_rrc_DSCH_TransportChannelsInfo_item = -1;
static gint ett_rrc_DTX_DRX_Info_r7 = -1;
static gint ett_rrc_DTX_E_DCH_TTI_10ms = -1;
static gint ett_rrc_DTX_E_DCH_TTI_2ms = -1;
static gint ett_rrc_DTX_Info = -1;
static gint ett_rrc_T_e_dch_TTI_Length = -1;
static gint ett_rrc_DynamicPersistenceLevelList = -1;
static gint ett_rrc_DynamicPersistenceLevelTF_List = -1;
static gint ett_rrc_DynamicTFInformationCCCH = -1;
static gint ett_rrc_E_AGCH_DRX_Info_TDD128 = -1;
static gint ett_rrc_T_e_AGCH_DRX_InfoType = -1;
static gint ett_rrc_T_e_AGCH_DRX_Parameters = -1;
static gint ett_rrc_E_AGCH_Individual = -1;
static gint ett_rrc_E_AGCH_Individual_VHCR = -1;
static gint ett_rrc_E_AGCH_Individual_LCR = -1;
static gint ett_rrc_T_midambleAllocationMode = -1;
static gint ett_rrc_E_AGCH_Information = -1;
static gint ett_rrc_E_AGCH_Information_r7 = -1;
static gint ett_rrc_T_modeSpecific_02 = -1;
static gint ett_rrc_T_fdd_91 = -1;
static gint ett_rrc_T_tdd_48 = -1;
static gint ett_rrc_T_tdd384_19 = -1;
static gint ett_rrc_T_tdd768_11 = -1;
static gint ett_rrc_T_tdd128_31 = -1;
static gint ett_rrc_E_AGCH_Information_r8 = -1;
static gint ett_rrc_T_modeSpecific_03 = -1;
static gint ett_rrc_T_fdd_92 = -1;
static gint ett_rrc_T_tdd_49 = -1;
static gint ett_rrc_T_tdd384_20 = -1;
static gint ett_rrc_T_tdd768_12 = -1;
static gint ett_rrc_T_tdd128_32 = -1;
static gint ett_rrc_E_AGCH_Information_TDD128 = -1;
static gint ett_rrc_E_AGCH_Set_Config = -1;
static gint ett_rrc_E_AGCH_Set_Config_VHCR = -1;
static gint ett_rrc_E_AGCH_Set_Config_LCR = -1;
static gint ett_rrc_E_ROCH_Information = -1;
static gint ett_rrc_E_DCH_ReconfigurationInfo = -1;
static gint ett_rrc_SEQUENCE_SIZE_1_maxEDCHRL_OF_E_DCH_RL_InfoOtherCell = -1;
static gint ett_rrc_E_DCH_ReconfigurationInfo_r7 = -1;
static gint ett_rrc_E_DCH_ReconfigurationInfo_r11 = -1;
static gint ett_rrc_SEQUENCE_SIZE_1_maxEDCHRL_OF_E_DCH_RL_InfoOtherCell_r11 = -1;
static gint ett_rrc_E_DCH_ReconfigurationInfo_SecULFrequency = -1;
static gint ett_rrc_SEQUENCE_SIZE_1_maxEDCHRL_OF_E_DCH_RL_InfoOtherCell_SecULFreq = -1;
static gint ett_rrc_E_DCH_RL_InfoNewServingCell = -1;
static gint ett_rrc_T_servingGrant_01 = -1;
static gint ett_rrc_T_e_RGCH_Info_05 = -1;
static gint ett_rrc_E_DCH_RL_InfoNewServingCell_r7 = -1;
static gint ett_rrc_T_servingGrant_02 = -1;
static gint ett_rrc_T_e_RGCH_Info_06 = -1;
static gint ett_rrc_E_DCH_RL_InfoNewServingCell_r11 = -1;
static gint ett_rrc_T_servingGrant_03 = -1;
static gint ett_rrc_T_e_RGCH_Info_07 = -1;
static gint ett_rrc_E_DCH_RL_InfoNewSecServingCell = -1;
static gint ett_rrc_T_e_RGCH_Info_08 = -1;
static gint ett_rrc_E_DCH_RL_InfoOtherCell = -1;
static gint ett_rrc_T_e_HICH_Info_04 = -1;
static gint ett_rrc_T_e_RGCH_Info_09 = -1;
static gint ett_rrc_E_DCH_RL_InfoOtherCell_r11 = -1;
static gint ett_rrc_T_e_HICH_Info_05 = -1;
static gint ett_rrc_T_e_RGCH_Info_10 = -1;
static gint ett_rrc_E_DCH_RL_InfoOtherCell_SecULFreq = -1;
static gint ett_rrc_T_e_RGCH_Info_11 = -1;
static gint ett_rrc_E_DCH_RL_InfoSameServingCell = -1;
static gint ett_rrc_E_DCH_SPS_Information_TDD128 = -1;
static gint ett_rrc_T_e_dch_SPS_Operation = -1;
static gint ett_rrc_E_DCH_SPS_NewOperation_TDD128 = -1;
static gint ett_rrc_T_initialSPSInfoForEDCH = -1;
static gint ett_rrc_E_DCH_TxPatternList_TDD128 = -1;
static gint ett_rrc_E_DCH_TxPatternList_TDD128_item = -1;
static gint ett_rrc_E_DPCCH_Info = -1;
static gint ett_rrc_E_DPCCH_Info_r7 = -1;
static gint ett_rrc_E_DPDCH_Info = -1;
static gint ett_rrc_E_DPDCH_Info_r7 = -1;
static gint ett_rrc_E_DPDCH_Info_r8 = -1;
static gint ett_rrc_E_DPDCH_Reference_E_TFCI = -1;
static gint ett_rrc_E_DPDCH_Reference_E_TFCI_r7 = -1;
static gint ett_rrc_E_DPDCH_Reference_E_TFCIList = -1;
static gint ett_rrc_E_DPDCH_Reference_E_TFCIList_r7 = -1;
static gint ett_rrc_E_DPDCH_SchedulingInfoConfiguration = -1;
static gint ett_rrc_E_DPDCH_SchedulingTransmConfiguration = -1;
static gint ett_rrc_T_servingGrant_04 = -1;
static gint ett_rrc_E_HICH_Information = -1;
static gint ett_rrc_E_HICH_Information_r11 = -1;
static gint ett_rrc_E_HICH_Information_CommonEdch = -1;
static gint ett_rrc_E_HICH_Information_TDD384_768 = -1;
static gint ett_rrc_T_modeSpecificInfo_56 = -1;
static gint ett_rrc_E_HICH_Information_LCR = -1;
static gint ett_rrc_T_midambleAllocationMode_01 = -1;
static gint ett_rrc_E_HICH_Information_TDD128 = -1;
static gint ett_rrc_E_HICH_Information_LCR_List = -1;
static gint ett_rrc_E_HICH_Information_For_SPS_TDD128 = -1;
static gint ett_rrc_T_configurationmode = -1;
static gint ett_rrc_T_implicit = -1;
static gint ett_rrc_T_explicit_02 = -1;
static gint ett_rrc_T_midambleAllocationMode_02 = -1;
static gint ett_rrc_E_PUCH_Info = -1;
static gint ett_rrc_T_modeSpecificInfo_57 = -1;
static gint ett_rrc_T_tdd348_tdd768 = -1;
static gint ett_rrc_SEQUENCE_SIZE_1_maxTS_2_OF_E_PUCH_TS_Slots = -1;
static gint ett_rrc_T_tdd128_33 = -1;
static gint ett_rrc_SEQUENCE_SIZE_1_maxTS_LCR_1_OF_E_PUCH_TS_Slots_LCR = -1;
static gint ett_rrc_E_PUCH_Info_TDD128 = -1;
static gint ett_rrc_E_PUCH_Info_MulticarrierEDCH_TDD128 = -1;
static gint ett_rrc_E_PUCH_TS_Slots = -1;
static gint ett_rrc_E_PUCH_TS_Slots_LCR = -1;
static gint ett_rrc_T_midambleAllocationMode_03 = -1;
static gint ett_rrc_E_RGCH_Information = -1;
static gint ett_rrc_E_RGCH_Information_CommonEdch = -1;
static gint ett_rrc_E_RUCCH_Info = -1;
static gint ett_rrc_T_modeSpecificInfo_58 = -1;
static gint ett_rrc_T_tdd384_tdd768_03 = -1;
static gint ett_rrc_T_modeSpecificInfo_59 = -1;
static gint ett_rrc_T_tdd384_21 = -1;
static gint ett_rrc_SEQUENCE_SIZE_1_8_OF_SF16Codes = -1;
static gint ett_rrc_SEQUENCE_SIZE_1_8_OF_SF8Codes = -1;
static gint ett_rrc_T_tdd768_13 = -1;
static gint ett_rrc_SEQUENCE_SIZE_1_16_OF_SF32Codes = -1;
static gint ett_rrc_SEQUENCE_SIZE_1_16_OF_SF16Codes2 = -1;
static gint ett_rrc_T_tdd128_34 = -1;
static gint ett_rrc_E_RUCCH_Info_TDD128 = -1;
static gint ett_rrc_E_TFC_Boost_Info_r7 = -1;
static gint ett_rrc_E_TFCS_Info = -1;
static gint ett_rrc_SEQUENCE_SIZE_1_8_OF_Reference_Beta_QPSK = -1;
static gint ett_rrc_SEQUENCE_SIZE_1_8_OF_Reference_Beta_16QAM = -1;
static gint ett_rrc_EXT_UL_TimingAdvance = -1;
static gint ett_rrc_T_modeSpecificInfo_60 = -1;
static gint ett_rrc_T_tdd384_22 = -1;
static gint ett_rrc_T_tdd768_14 = -1;
static gint ett_rrc_FACH_PCH_Information = -1;
static gint ett_rrc_FACH_PCH_InformationList = -1;
static gint ett_rrc_Fallback_R99_PRACH_info_IEs = -1;
static gint ett_rrc_FPACH_Info_r4 = -1;
static gint ett_rrc_F_TPICH_InfoNewServingCell = -1;
static gint ett_rrc_F_TPICH_InfoOtherCell = -1;
static gint ett_rrc_T_f_tpich_Info = -1;
static gint ett_rrc_F_TPICH_Information = -1;
static gint ett_rrc_F_TPICH_ReconfigurationInfo = -1;
static gint ett_rrc_SEQUENCE_SIZE_1_maxRL_OF_F_TPICH_InfoOtherCell = -1;
static gint ett_rrc_FreqIndexListForEnhancedMeas = -1;
static gint ett_rrc_FrequencyInfo = -1;
static gint ett_rrc_T_modeSpecificInfo_61 = -1;
static gint ett_rrc_FrequencyInfoFDD = -1;
static gint ett_rrc_FrequencyInfoTDD = -1;
static gint ett_rrc_HS_ChannelisationCodeSetInfo_LCR = -1;
static gint ett_rrc_HS_DSCH_CommonSystemInformation = -1;
static gint ett_rrc_SEQUENCE_SIZE_1_maxCommonHRNTI_OF_H_RNTI = -1;
static gint ett_rrc_HS_DSCH_CommonSystemInformation_TDD128 = -1;
static gint ett_rrc_HS_DSCH_DrxCellfach_info = -1;
static gint ett_rrc_HS_DSCH_DrxCellfach_info_TDD128 = -1;
static gint ett_rrc_HS_DSCH_DrxCellfach_SecondDrx_info = -1;
static gint ett_rrc_T_drx_level = -1;
static gint ett_rrc_T_one_level_DRX = -1;
static gint ett_rrc_T_two_level_DRX = -1;
static gint ett_rrc_HS_DSCH_PagingSystemInformation = -1;
static gint ett_rrc_SEQUENCE_SIZE_1_maxSCCPCH_OF_PICH_ForHSDPASupportedPaging = -1;
static gint ett_rrc_SEQUENCE_SIZE_1_2_OF_TransportBlockSizeIndex = -1;
static gint ett_rrc_HS_DSCH_PagingSystemInformation_TDD128 = -1;
static gint ett_rrc_SEQUENCE_SIZE_1_maxSCCPCH_OF_PICH_ForHSDPASupportedPaging_TDD128 = -1;
static gint ett_rrc_HS_DSCH_RxPatternList_TDD128 = -1;
static gint ett_rrc_HS_DSCH_RxPatternList_TDD128_item = -1;
static gint ett_rrc_HS_DSCH_TbsList_TDD128 = -1;
static gint ett_rrc_HS_DSCH_TbsList_TDD128_item = -1;
static gint ett_rrc_HS_DSCH_SPS_Information_TDD128 = -1;
static gint ett_rrc_T_hs_dsch_SPS_Operation = -1;
static gint ett_rrc_HS_DSCH_SPS_NewOperation_TDD128 = -1;
static gint ett_rrc_T_hARQInfoForSPS = -1;
static gint ett_rrc_T_initialSPSInfoForHSDSCH = -1;
static gint ett_rrc_HS_PDSCH_Midamble_Configuration_TDD128 = -1;
static gint ett_rrc_T_midambleAllocationMode_04 = -1;
static gint ett_rrc_HS_SCCH_Info = -1;
static gint ett_rrc_T_modeSpecificInfo_62 = -1;
static gint ett_rrc_T_fdd_93 = -1;
static gint ett_rrc_T_tdd_50 = -1;
static gint ett_rrc_T_tdd384_23 = -1;
static gint ett_rrc_SEQUENCE_SIZE_1_maxHSSCCHs_OF_HS_SCCH_TDD384 = -1;
static gint ett_rrc_SEQUENCE_SIZE_1_maxHSSCCHs_OF_HS_SCCH_TDD128 = -1;
static gint ett_rrc_HS_SCCH_Info_r6 = -1;
static gint ett_rrc_T_modeSpecificInfo_63 = -1;
static gint ett_rrc_T_fdd_94 = -1;
static gint ett_rrc_T_tdd_51 = -1;
static gint ett_rrc_T_tdd384_24 = -1;
static gint ett_rrc_SEQUENCE_SIZE_1_maxHSSCCHs_OF_HS_SCCH_TDD384_r6 = -1;
static gint ett_rrc_T_tdd128_35 = -1;
static gint ett_rrc_SEQUENCE_SIZE_1_maxHSSCCHs_OF_HS_SCCH_TDD128_r6 = -1;
static gint ett_rrc_HS_SCCH_Info_r7 = -1;
static gint ett_rrc_T_modeSpecificInfo_64 = -1;
static gint ett_rrc_T_fdd_95 = -1;
static gint ett_rrc_T_tdd_52 = -1;
static gint ett_rrc_T_tdd384_25 = -1;
static gint ett_rrc_T_tdd768_15 = -1;
static gint ett_rrc_SEQUENCE_SIZE_1_maxHSSCCHs_OF_HS_SCCH_TDD768 = -1;
static gint ett_rrc_T_tdd128_36 = -1;
static gint ett_rrc_HS_SCCH_Info_r8_ext = -1;
static gint ett_rrc_T_modeSpecificInfo_65 = -1;
static gint ett_rrc_T_tdd_53 = -1;
static gint ett_rrc_T_tdd128_37 = -1;
static gint ett_rrc_HS_SCCH_Info_r9 = -1;
static gint ett_rrc_T_modeSpecificInfo_66 = -1;
static gint ett_rrc_T_fdd_96 = -1;
static gint ett_rrc_T_tdd_54 = -1;
static gint ett_rrc_T_tdd384_26 = -1;
static gint ett_rrc_T_tdd768_16 = -1;
static gint ett_rrc_T_tdd128_38 = -1;
static gint ett_rrc_HS_SCCH_DRX_Info_TDD128 = -1;
static gint ett_rrc_HS_SCCH_Less_NewOperation = -1;
static gint ett_rrc_HS_SCCH_LessInfo_r7 = -1;
static gint ett_rrc_T_hs_scchLessOperation = -1;
static gint ett_rrc_HS_SCCH_LessTFSList = -1;
static gint ett_rrc_HS_SCCH_LessTFSList_item = -1;
static gint ett_rrc_HS_SCCH_SystemInfo = -1;
static gint ett_rrc_HS_SCCH_SystemInfo_TDD128 = -1;
static gint ett_rrc_HS_SCCH_TDD128 = -1;
static gint ett_rrc_T_midambleAllocationMode_05 = -1;
static gint ett_rrc_HS_SCCH_TDD128_r6 = -1;
static gint ett_rrc_T_midambleAllocationMode_06 = -1;
static gint ett_rrc_HS_SCCH_TDD128_MultiCarrier = -1;
static gint ett_rrc_T_midambleAllocationMode_07 = -1;
static gint ett_rrc_HS_SICH_Configuration_TDD128 = -1;
static gint ett_rrc_T_midambleAllocationMode_08 = -1;
static gint ett_rrc_T_ueSpecificMidamble = -1;
static gint ett_rrc_HS_SICH_Configuration_TDD128_r6 = -1;
static gint ett_rrc_T_midambleAllocationMode_09 = -1;
static gint ett_rrc_T_ueSpecificMidamble_01 = -1;
static gint ett_rrc_HS_SICH_List_TDD128 = -1;
static gint ett_rrc_HS_SICH_List_TDD128_item = -1;
static gint ett_rrc_T_implicit_01 = -1;
static gint ett_rrc_T_explicit_03 = -1;
static gint ett_rrc_HS_SICH_ReferenceSignalInfoList = -1;
static gint ett_rrc_HS_SICH_ReferenceSignalInfoList_item = -1;
static gint ett_rrc_HS_SCCH_TDD384 = -1;
static gint ett_rrc_T_midambleAllocationMode_10 = -1;
static gint ett_rrc_T_ueSpecificMidamble_02 = -1;
static gint ett_rrc_HS_SCCH_TDD384_r6 = -1;
static gint ett_rrc_T_midambleAllocationMode_11 = -1;
static gint ett_rrc_T_ueSpecificMidamble_03 = -1;
static gint ett_rrc_HS_SCCH_TDD768 = -1;
static gint ett_rrc_T_midambleAllocationMode_12 = -1;
static gint ett_rrc_T_ueSpecificMidamble_04 = -1;
static gint ett_rrc_HS_SICH_Configuration_TDD384 = -1;
static gint ett_rrc_T_midambleAllocationMode_13 = -1;
static gint ett_rrc_T_ueSpecificMidamble_05 = -1;
static gint ett_rrc_HS_SICH_Configuration_TDD768 = -1;
static gint ett_rrc_T_midambleAllocationMode_14 = -1;
static gint ett_rrc_T_ueSpecificMidamble_06 = -1;
static gint ett_rrc_HS_SICH_Power_Control_Info_TDD384 = -1;
static gint ett_rrc_HS_SICH_Power_Control_Info_TDD768 = -1;
static gint ett_rrc_IndividualTimeslotInfo = -1;
static gint ett_rrc_IndividualTimeslotInfo_VHCR = -1;
static gint ett_rrc_IndividualTimeslotInfo_LCR_r4 = -1;
static gint ett_rrc_IndividualTimeslotInfo_LCR_r4_ext = -1;
static gint ett_rrc_IndividualTimeslotInfo_r7 = -1;
static gint ett_rrc_IndividualTS_Interference = -1;
static gint ett_rrc_IndividualTS_InterferenceList = -1;
static gint ett_rrc_Measurement_Feedback_Info = -1;
static gint ett_rrc_T_modeSpecificInfo_67 = -1;
static gint ett_rrc_T_fdd_97 = -1;
static gint ett_rrc_Measurement_Feedback_Info_r7 = -1;
static gint ett_rrc_T_modeSpecificInfo_68 = -1;
static gint ett_rrc_T_fdd_98 = -1;
static gint ett_rrc_Measurement_Feedback_Info_r11 = -1;
static gint ett_rrc_T_modeSpecificInfo_69 = -1;
static gint ett_rrc_T_fdd_99 = -1;
static gint ett_rrc_Measurement_Feedback_Info_ConcurrentTTI = -1;
static gint ett_rrc_MidambleShiftAndBurstType = -1;
static gint ett_rrc_T_burstType = -1;
static gint ett_rrc_T_type1_01 = -1;
static gint ett_rrc_T_midambleAllocationMode_15 = -1;
static gint ett_rrc_T_ueSpecificMidamble_07 = -1;
static gint ett_rrc_T_type2 = -1;
static gint ett_rrc_T_midambleAllocationMode_16 = -1;
static gint ett_rrc_T_ueSpecificMidamble_08 = -1;
static gint ett_rrc_T_type3 = -1;
static gint ett_rrc_T_midambleAllocationMode_17 = -1;
static gint ett_rrc_T_ueSpecificMidamble_09 = -1;
static gint ett_rrc_MidambleShiftAndBurstType_VHCR = -1;
static gint ett_rrc_T_burstType_01 = -1;
static gint ett_rrc_T_type1_02 = -1;
static gint ett_rrc_T_midambleAllocationMode_18 = -1;
static gint ett_rrc_T_ueSpecificMidamble_10 = -1;
static gint ett_rrc_T_type2_01 = -1;
static gint ett_rrc_T_midambleAllocationMode_19 = -1;
static gint ett_rrc_T_ueSpecificMidamble_11 = -1;
static gint ett_rrc_T_type3_01 = -1;
static gint ett_rrc_T_midambleAllocationMode_20 = -1;
static gint ett_rrc_T_ueSpecificMidamble_12 = -1;
static gint ett_rrc_MidambleShiftAndBurstType_r7 = -1;
static gint ett_rrc_T_burstType_02 = -1;
static gint ett_rrc_T_type1_03 = -1;
static gint ett_rrc_T_midambleAllocationMode_21 = -1;
static gint ett_rrc_T_ueSpecificMidamble_13 = -1;
static gint ett_rrc_T_type2_02 = -1;
static gint ett_rrc_T_midambleAllocationMode_22 = -1;
static gint ett_rrc_T_ueSpecificMidamble_14 = -1;
static gint ett_rrc_T_type3_02 = -1;
static gint ett_rrc_T_midambleAllocationMode_23 = -1;
static gint ett_rrc_T_ueSpecificMidamble_15 = -1;
static gint ett_rrc_MidambleShiftAndBurstType_DL = -1;
static gint ett_rrc_T_burstType_03 = -1;
static gint ett_rrc_T_type1_04 = -1;
static gint ett_rrc_T_midambleAllocationMode_24 = -1;
static gint ett_rrc_T_ueSpecificMidamble_16 = -1;
static gint ett_rrc_T_type2_03 = -1;
static gint ett_rrc_T_midambleAllocationMode_25 = -1;
static gint ett_rrc_T_ueSpecificMidamble_17 = -1;
static gint ett_rrc_MidambleShiftAndBurstType_DL_VHCR = -1;
static gint ett_rrc_T_burstType_04 = -1;
static gint ett_rrc_T_type1_05 = -1;
static gint ett_rrc_T_midambleAllocationMode_26 = -1;
static gint ett_rrc_T_ueSpecificMidamble_18 = -1;
static gint ett_rrc_T_type2_04 = -1;
static gint ett_rrc_T_midambleAllocationMode_27 = -1;
static gint ett_rrc_T_ueSpecificMidamble_19 = -1;
static gint ett_rrc_MidambleShiftAndBurstType_LCR_r4 = -1;
static gint ett_rrc_T_midambleAllocationMode_28 = -1;
static gint ett_rrc_T_ueSpecificMidamble_20 = -1;
static gint ett_rrc_MidambleShiftAndBurstType_EDCH = -1;
static gint ett_rrc_T_burstType_05 = -1;
static gint ett_rrc_T_type1_06 = -1;
static gint ett_rrc_T_midambleAllocationMode_29 = -1;
static gint ett_rrc_T_ueSpecificMidamble_21 = -1;
static gint ett_rrc_T_type2_05 = -1;
static gint ett_rrc_T_midambleAllocationMode_30 = -1;
static gint ett_rrc_T_ueSpecificMidamble_22 = -1;
static gint ett_rrc_MIMO_Parameters_r7 = -1;
static gint ett_rrc_MIMO4x4_Parameters = -1;
static gint ett_rrc_T_newConfiguration_06 = -1;
static gint ett_rrc_MIMO_Parameters_v7g0ext = -1;
static gint ett_rrc_MIMO_Parameters_r8 = -1;
static gint ett_rrc_T_modeSpecificInfo_70 = -1;
static gint ett_rrc_T_fdd_100 = -1;
static gint ett_rrc_T_tdd_55 = -1;
static gint ett_rrc_T_tdd128_39 = -1;
static gint ett_rrc_MIMO_Parameters_r9 = -1;
static gint ett_rrc_T_modeSpecificInfo_71 = -1;
static gint ett_rrc_T_fdd_101 = -1;
static gint ett_rrc_T_tdd_56 = -1;
static gint ett_rrc_T_tdd128_40 = -1;
static gint ett_rrc_MIMO_PilotConfiguration = -1;
static gint ett_rrc_T_secondCPICH_Pattern = -1;
static gint ett_rrc_T_diversityPattern = -1;
static gint ett_rrc_MIMO_PilotConfiguration_v7f0ext = -1;
static gint ett_rrc_MIMO_PilotConfiguration_r9 = -1;
static gint ett_rrc_T_secondCPICH_Pattern_01 = -1;
static gint ett_rrc_T_diversityPattern_01 = -1;
static gint ett_rrc_MIMO4x4_PilotConfiguration = -1;
static gint ett_rrc_Multi_frequencyInfo_LCR_r7 = -1;
static gint ett_rrc_MultiflowConfigurationInfo = -1;
static gint ett_rrc_MU_MIMO_Info_TDD128 = -1;
static gint ett_rrc_T_newConfiguration_07 = -1;
static gint ett_rrc_NewTiming = -1;
static gint ett_rrc_NodeB_Trigger_HS_DPCCH_Transmission = -1;
static gint ett_rrc_Non_ScheduledTransGrantInfoTDD = -1;
static gint ett_rrc_T_tdd384_768 = -1;
static gint ett_rrc_T_tdd128_41 = -1;
static gint ett_rrc_T_e_HICH_Info_06 = -1;
static gint ett_rrc_T_midambleAllocationMode_31 = -1;
static gint ett_rrc_Non_ScheduledTransGrantInfoTDD_ext = -1;
static gint ett_rrc_T_tdd128_42 = -1;
static gint ett_rrc_Non_ScheduledTransGrantInfoTDD_r9 = -1;
static gint ett_rrc_T_tdd384_768_01 = -1;
static gint ett_rrc_T_tdd128_43 = -1;
static gint ett_rrc_T_e_HICH_Info_07 = -1;
static gint ett_rrc_T_midambleAllocationMode_32 = -1;
static gint ett_rrc_OpenLoopPowerControl_TDD = -1;
static gint ett_rrc_OpenLoopPowerControl_IPDL_TDD_r4 = -1;
static gint ett_rrc_PCCH_InformationList = -1;
static gint ett_rrc_T_paging_associatedHspdschInfo = -1;
static gint ett_rrc_T_paging_associatedHspdschInfo_item = -1;
static gint ett_rrc_PCPCH_ChannelInfo = -1;
static gint ett_rrc_PCPCH_ChannelInfoList = -1;
static gint ett_rrc_PDSCH_CapacityAllocationInfo = -1;
static gint ett_rrc_T_configuration = -1;
static gint ett_rrc_T_old_Configuration = -1;
static gint ett_rrc_T_new_Configuration = -1;
static gint ett_rrc_PDSCH_CapacityAllocationInfo_r4 = -1;
static gint ett_rrc_T_configuration_01 = -1;
static gint ett_rrc_T_old_Configuration_01 = -1;
static gint ett_rrc_T_new_Configuration_01 = -1;
static gint ett_rrc_PDSCH_CapacityAllocationInfo_r7 = -1;
static gint ett_rrc_T_configuration_02 = -1;
static gint ett_rrc_T_old_Configuration_02 = -1;
static gint ett_rrc_T_new_Configuration_02 = -1;
static gint ett_rrc_PDSCH_CodeInfo = -1;
static gint ett_rrc_PDSCH_CodeInfoList = -1;
static gint ett_rrc_PDSCH_CodeMap = -1;
static gint ett_rrc_PDSCH_CodeMapList = -1;
static gint ett_rrc_PDSCH_CodeMapping = -1;
static gint ett_rrc_T_signallingMethod = -1;
static gint ett_rrc_PDSCH_Info = -1;
static gint ett_rrc_PDSCH_Info_r4 = -1;
static gint ett_rrc_T_tddOption_12 = -1;
static gint ett_rrc_T_tdd384_27 = -1;
static gint ett_rrc_T_tdd128_44 = -1;
static gint ett_rrc_PDSCH_Info_r7 = -1;
static gint ett_rrc_T_tddOption_13 = -1;
static gint ett_rrc_T_tdd384_28 = -1;
static gint ett_rrc_T_tdd768_17 = -1;
static gint ett_rrc_T_tdd128_45 = -1;
static gint ett_rrc_PDSCH_Info_LCR_r4 = -1;
static gint ett_rrc_PDSCH_PowerControlInfo = -1;
static gint ett_rrc_PDSCH_SHO_DCH_Info = -1;
static gint ett_rrc_PDSCH_SysInfo = -1;
static gint ett_rrc_PDSCH_SysInfo_VHCR_r7 = -1;
static gint ett_rrc_PDSCH_SysInfo_HCR_r5 = -1;
static gint ett_rrc_PDSCH_SysInfo_LCR_r4 = -1;
static gint ett_rrc_PDSCH_SysInfoList = -1;
static gint ett_rrc_PDSCH_SysInfoList_VHCR_r7 = -1;
static gint ett_rrc_PDSCH_SysInfoList_HCR_r5 = -1;
static gint ett_rrc_PDSCH_SysInfoList_LCR_r4 = -1;
static gint ett_rrc_PDSCH_SysInfoList_SFN = -1;
static gint ett_rrc_PDSCH_SysInfoList_SFN_item = -1;
static gint ett_rrc_PDSCH_SysInfoList_SFN_HCR_r5 = -1;
static gint ett_rrc_PDSCH_SysInfoList_SFN_HCR_r5_item = -1;
static gint ett_rrc_PDSCH_SysInfoList_SFN_LCR_r4 = -1;
static gint ett_rrc_PDSCH_SysInfoList_SFN_LCR_r4_item = -1;
static gint ett_rrc_PersistenceScalingFactorList = -1;
static gint ett_rrc_PichChannelisationCodeList_LCR_r4 = -1;
static gint ett_rrc_PICH_ForHSDPASupportedPaging = -1;
static gint ett_rrc_PICH_ForHSDPASupportedPaging_TDD128 = -1;
static gint ett_rrc_T_implicit_02 = -1;
static gint ett_rrc_PICH_Info = -1;
static gint ett_rrc_T_fdd_102 = -1;
static gint ett_rrc_T_tdd_57 = -1;
static gint ett_rrc_PICH_Info_HCR_VHCR_r7 = -1;
static gint ett_rrc_T_channelisationCode = -1;
static gint ett_rrc_PICH_Info_LCR_r4 = -1;
static gint ett_rrc_PLCCH_Info = -1;
static gint ett_rrc_PowerControlAlgorithm = -1;
static gint ett_rrc_PRACH_ChanCodes_LCR_r4 = -1;
static gint ett_rrc_PRACH_ChanCodes_List_LCR = -1;
static gint ett_rrc_PRACH_Definition_LCR_r4 = -1;
static gint ett_rrc_PRACH_Information_LCR = -1;
static gint ett_rrc_PRACH_Information_LCR_List = -1;
static gint ett_rrc_PRACH_Partitioning = -1;
static gint ett_rrc_SEQUENCE_SIZE_1_maxASC_OF_ASCSetting_FDD = -1;
static gint ett_rrc_SEQUENCE_SIZE_1_maxASC_OF_ASCSetting_TDD = -1;
static gint ett_rrc_PRACH_Partitioning_r7 = -1;
static gint ett_rrc_SEQUENCE_SIZE_1_maxASC_OF_ASCSetting_TDD_r7 = -1;
static gint ett_rrc_PRACH_Partitioning_LCR_r4 = -1;
static gint ett_rrc_PRACH_PowerOffset = -1;
static gint ett_rrc_PRACH_PreambleForEnhancedUplink = -1;
static gint ett_rrc_PRACH_PreambleForEnhancedUplinkExt = -1;
static gint ett_rrc_PRACH_PreambleForEnhancedUplinkExtList = -1;
static gint ett_rrc_SEQUENCE_SIZE_1_maxPRACH_EUL_OF_PRACH_PreambleForEnhancedUplinkExtWithWeight = -1;
static gint ett_rrc_PRACH_PreambleForEnhancedUplinkExtWithWeight = -1;
static gint ett_rrc_PRACH_RACH_Info = -1;
static gint ett_rrc_T_modeSpecificInfo_72 = -1;
static gint ett_rrc_T_fdd_103 = -1;
static gint ett_rrc_T_tdd_58 = -1;
static gint ett_rrc_PRACH_RACH_Info_VHCR_r7 = -1;
static gint ett_rrc_PRACH_RACH_Info_LCR_v770ext = -1;
static gint ett_rrc_PRACH_RACH_Info_LCR_r4 = -1;
static gint ett_rrc_SEQUENCE_SIZE_1_maxPRACH_FPACH_OF_PRACH_Definition_LCR_r4 = -1;
static gint ett_rrc_PRACH_SystemInformation = -1;
static gint ett_rrc_T_modeSpecificInfo_73 = -1;
static gint ett_rrc_T_fdd_104 = -1;
static gint ett_rrc_PRACH_SystemInformation_VHCR_r7 = -1;
static gint ett_rrc_PRACH_SystemInformation_LCR_r4 = -1;
static gint ett_rrc_PRACH_SystemInformation_LCR_v770ext = -1;
static gint ett_rrc_PRACH_SystemInformationList = -1;
static gint ett_rrc_PRACH_SystemInformationList_VHCR_r7 = -1;
static gint ett_rrc_PRACH_SystemInformationList_LCR_r4 = -1;
static gint ett_rrc_PRACH_SystemInformationList_LCR_v770ext = -1;
static gint ett_rrc_PreDefPhyChConfiguration = -1;
static gint ett_rrc_PreDefPhyChConfiguration_v770ext = -1;
static gint ett_rrc_PrimaryCCPCH_Info = -1;
static gint ett_rrc_T_fdd_105 = -1;
static gint ett_rrc_T_tdd_59 = -1;
static gint ett_rrc_T_syncCase = -1;
static gint ett_rrc_T_syncCase1 = -1;
static gint ett_rrc_T_syncCase2 = -1;
static gint ett_rrc_PrimaryCCPCH_Info_r4 = -1;
static gint ett_rrc_T_fdd_106 = -1;
static gint ett_rrc_T_tdd_60 = -1;
static gint ett_rrc_T_tddOption_14 = -1;
static gint ett_rrc_T_tdd384_tdd768_04 = -1;
static gint ett_rrc_T_syncCase_01 = -1;
static gint ett_rrc_T_syncCase1_01 = -1;
static gint ett_rrc_T_syncCase2_01 = -1;
static gint ett_rrc_T_tdd128_46 = -1;
static gint ett_rrc_PrimaryCCPCH_Info_LCR_r4 = -1;
static gint ett_rrc_PrimaryCCPCH_Info_LCR_r4_ext = -1;
static gint ett_rrc_PrimaryCCPCH_InfoPost = -1;
static gint ett_rrc_T_syncCase_02 = -1;
static gint ett_rrc_T_syncCase1_02 = -1;
static gint ett_rrc_T_syncCase2_02 = -1;
static gint ett_rrc_PrimaryCCPCH_InfoPostTDD_LCR_r4 = -1;
static gint ett_rrc_PrimaryCPICH_Info = -1;
static gint ett_rrc_PUSCH_CapacityAllocationInfo = -1;
static gint ett_rrc_T_pusch_Allocation = -1;
static gint ett_rrc_T_pusch_AllocationAssignment = -1;
static gint ett_rrc_T_configuration_03 = -1;
static gint ett_rrc_T_old_Configuration_03 = -1;
static gint ett_rrc_T_new_Configuration_03 = -1;
static gint ett_rrc_PUSCH_CapacityAllocationInfo_r4 = -1;
static gint ett_rrc_T_pusch_Allocation_01 = -1;
static gint ett_rrc_T_pusch_AllocationAssignment_01 = -1;
static gint ett_rrc_T_configuration_04 = -1;
static gint ett_rrc_T_old_Configuration_04 = -1;
static gint ett_rrc_T_new_Configuration_04 = -1;
static gint ett_rrc_PUSCH_CapacityAllocationInfo_r7 = -1;
static gint ett_rrc_T_pusch_Allocation_02 = -1;
static gint ett_rrc_T_pusch_AllocationAssignment_02 = -1;
static gint ett_rrc_T_configuration_05 = -1;
static gint ett_rrc_T_old_Configuration_05 = -1;
static gint ett_rrc_T_new_Configuration_05 = -1;
static gint ett_rrc_PUSCH_Info = -1;
static gint ett_rrc_PUSCH_Info_r4 = -1;
static gint ett_rrc_T_tddOption_15 = -1;
static gint ett_rrc_T_tdd384_29 = -1;
static gint ett_rrc_T_tdd128_47 = -1;
static gint ett_rrc_PUSCH_Info_VHCR = -1;
static gint ett_rrc_PUSCH_Info_LCR_r4 = -1;
static gint ett_rrc_PUSCH_PowerControlInfo_r4 = -1;
static gint ett_rrc_T_tddOption_16 = -1;
static gint ett_rrc_T_tdd128_48 = -1;
static gint ett_rrc_PUSCH_PowerControlInfo_r7 = -1;
static gint ett_rrc_T_tddOption_17 = -1;
static gint ett_rrc_T_tdd128_49 = -1;
static gint ett_rrc_PUSCH_SysInfo = -1;
static gint ett_rrc_PUSCH_SysInfo_VHCR = -1;
static gint ett_rrc_PUSCH_SysInfo_HCR_r5 = -1;
static gint ett_rrc_PUSCH_SysInfo_LCR_r4 = -1;
static gint ett_rrc_PUSCH_SysInfoList = -1;
static gint ett_rrc_PUSCH_SysInfoList_HCR_r5 = -1;
static gint ett_rrc_PUSCH_SysInfoList_LCR_r4 = -1;
static gint ett_rrc_PUSCH_SysInfoList_SFN = -1;
static gint ett_rrc_PUSCH_SysInfoList_SFN_item = -1;
static gint ett_rrc_PUSCH_SysInfoList_SFN_HCR_r5 = -1;
static gint ett_rrc_PUSCH_SysInfoList_SFN_HCR_r5_item = -1;
static gint ett_rrc_PUSCH_SysInfoList_SFN_LCR_r4 = -1;
static gint ett_rrc_PUSCH_SysInfoList_SFN_LCR_r4_item = -1;
static gint ett_rrc_PUSCH_SysInfoList_SFN_VHCR = -1;
static gint ett_rrc_PUSCH_SysInfoList_SFN_VHCR_item = -1;
static gint ett_rrc_RACH_TransmissionParameters = -1;
static gint ett_rrc_Reference_Beta_QPSK = -1;
static gint ett_rrc_Reference_Beta_16QAM = -1;
static gint ett_rrc_RepetitionPeriodAndLength = -1;
static gint ett_rrc_RepetitionPeriodLengthAndOffset = -1;
static gint ett_rrc_T_repetitionPeriod2 = -1;
static gint ett_rrc_T_repetitionPeriod4 = -1;
static gint ett_rrc_T_repetitionPeriod8 = -1;
static gint ett_rrc_T_repetitionPeriod16 = -1;
static gint ett_rrc_T_repetitionPeriod32 = -1;
static gint ett_rrc_T_repetitionPeriod64 = -1;
static gint ett_rrc_RepetitionPeriodAndLengthForSPS = -1;
static gint ett_rrc_ReplacedPDSCH_CodeInfo = -1;
static gint ett_rrc_ReplacedPDSCH_CodeInfoList = -1;
static gint ett_rrc_RepPerLengthOffset_PICH = -1;
static gint ett_rrc_RepPerLengthOffset_MICH = -1;
static gint ett_rrc_RestrictedTrCH = -1;
static gint ett_rrc_RestrictedTrCH_InfoList = -1;
static gint ett_rrc_RL_AdditionInformation = -1;
static gint ett_rrc_RL_AdditionInformation_r6 = -1;
static gint ett_rrc_T_dl_dpchInfo_04 = -1;
static gint ett_rrc_RL_AdditionInformation_r7 = -1;
static gint ett_rrc_T_dl_dpchInfo_05 = -1;
static gint ett_rrc_RL_AdditionInformation_r8 = -1;
static gint ett_rrc_T_dl_dpchInfo_06 = -1;
static gint ett_rrc_RL_AdditionInformation_r9 = -1;
static gint ett_rrc_T_dl_dpchInfo_07 = -1;
static gint ett_rrc_RL_AdditionInformation_r10 = -1;
static gint ett_rrc_T_dl_dpchInfo_08 = -1;
static gint ett_rrc_RL_AdditionInformation_vbxyext = -1;
static gint ett_rrc_RL_AdditionInformation_r11 = -1;
static gint ett_rrc_T_dl_dpchInfo_09 = -1;
static gint ett_rrc_RL_AdditionInformation_v6b0ext = -1;
static gint ett_rrc_RL_AdditionInformation_v890ext = -1;
static gint ett_rrc_RL_AdditionInformation_v950ext = -1;
static gint ett_rrc_RL_AdditionInformationList = -1;
static gint ett_rrc_RL_AdditionInformationList_r6 = -1;
static gint ett_rrc_RL_AdditionInformationList_r7 = -1;
static gint ett_rrc_RL_AdditionInformation_list_v6b0ext = -1;
static gint ett_rrc_RL_AdditionInformationList_r8 = -1;
static gint ett_rrc_RL_AdditionInformationList_v890ext = -1;
static gint ett_rrc_RL_AdditionInformationList_r9 = -1;
static gint ett_rrc_RL_AdditionInformationList_v950ext = -1;
static gint ett_rrc_RL_AdditionInformationList_r10 = -1;
static gint ett_rrc_RL_AdditionInformationList_vbxyext = -1;
static gint ett_rrc_RL_AdditionInformationList_r11 = -1;
static gint ett_rrc_RL_AdditionInformationList_SecULFreq = -1;
static gint ett_rrc_RL_AdditionInformation_SecULFreq = -1;
static gint ett_rrc_RL_IdentifierList = -1;
static gint ett_rrc_RL_RemovalInformationList = -1;
static gint ett_rrc_RL_RemovalInformationList_SecULFreq = -1;
static gint ett_rrc_SCCPCH_ChannelisationCodeList = -1;
static gint ett_rrc_SCCPCH_ChannelisationCodeList_VHCR = -1;
static gint ett_rrc_SCCPCH_InfoForFACH = -1;
static gint ett_rrc_T_modeSpecificInfo_74 = -1;
static gint ett_rrc_T_fdd_107 = -1;
static gint ett_rrc_T_tdd_61 = -1;
static gint ett_rrc_SCCPCH_InfoForFACH_r4 = -1;
static gint ett_rrc_T_modeSpecificInfo_75 = -1;
static gint ett_rrc_T_fdd_108 = -1;
static gint ett_rrc_SCCPCH_SystemInformation = -1;
static gint ett_rrc_SCCPCH_SystemInformation_LCR_r4_ext = -1;
static gint ett_rrc_SCCPCH_SystemInformation_HCR_VHCR_r7 = -1;
static gint ett_rrc_SCCPCH_SystemInformationList = -1;
static gint ett_rrc_SCCPCH_SystemInformationList_HCR_VHCR_r7 = -1;
static gint ett_rrc_SCCPCH_SystemInformationList_LCR_r4_ext = -1;
static gint ett_rrc_SCCPCH_SystemInformation_MBMS_r6 = -1;
static gint ett_rrc_T_fachCarryingMCCH = -1;
static gint ett_rrc_T_fachCarryingMSCH = -1;
static gint ett_rrc_SCCPCH_SystemInformation_MBMS_r7 = -1;
static gint ett_rrc_T_fachCarryingMCCH_01 = -1;
static gint ett_rrc_T_fachCarryingMSCH_01 = -1;
static gint ett_rrc_ScheduledTransmissionConfiguration = -1;
static gint ett_rrc_SEQUENCE_SIZE_1_maxEDCHs_OF_Common_E_DCH_ResourceInfoListExt = -1;
static gint ett_rrc_SecondaryCCPCH_Info = -1;
static gint ett_rrc_T_modeSpecificInfo_76 = -1;
static gint ett_rrc_T_fdd_109 = -1;
static gint ett_rrc_T_tdd_62 = -1;
static gint ett_rrc_SecondaryCCPCH_Info_r4 = -1;
static gint ett_rrc_T_modeSpecificInfo_77 = -1;
static gint ett_rrc_T_fdd_110 = -1;
static gint ett_rrc_T_tdd_63 = -1;
static gint ett_rrc_T_tddOption_18 = -1;
static gint ett_rrc_T_tdd384_30 = -1;
static gint ett_rrc_T_tdd128_50 = -1;
static gint ett_rrc_SecondaryCCPCH_Info_HCR_VHCR_r7 = -1;
static gint ett_rrc_T_modeSpecificInfo_78 = -1;
static gint ett_rrc_T_tdd384_31 = -1;
static gint ett_rrc_T_tdd768_18 = -1;
static gint ett_rrc_SecondaryCCPCH_Info_LCR_r4_ext = -1;
static gint ett_rrc_SecondaryCCPCHFrameType2Info = -1;
static gint ett_rrc_T_modulation_03 = -1;
static gint ett_rrc_T_mod16QAM = -1;
static gint ett_rrc_SecondaryCCPCHInfo_MBMS_r6 = -1;
static gint ett_rrc_T_modeSpecificInfo_79 = -1;
static gint ett_rrc_T_fdd_111 = -1;
static gint ett_rrc_T_tdd384_32 = -1;
static gint ett_rrc_T_tdd128_51 = -1;
static gint ett_rrc_SecondaryCCPCHInfo_MBMS_r7 = -1;
static gint ett_rrc_T_modeSpecificInfo_80 = -1;
static gint ett_rrc_T_fdd_112 = -1;
static gint ett_rrc_T_modulation_04 = -1;
static gint ett_rrc_T_tdd384_33 = -1;
static gint ett_rrc_T_tdd768_19 = -1;
static gint ett_rrc_T_tdd128_52 = -1;
static gint ett_rrc_SecondaryCCPCHInfoDiff_MBMS = -1;
static gint ett_rrc_SecondaryCPICH_Info = -1;
static gint ett_rrc_SecondaryCellMIMOparametersFDD = -1;
static gint ett_rrc_T_newConfiguration_08 = -1;
static gint ett_rrc_SecondaryCellMIMOparametersFDD_v950ext = -1;
static gint ett_rrc_SecondaryCellMIMOparametersFDD_r10 = -1;
static gint ett_rrc_T_newConfiguration_09 = -1;
static gint ett_rrc_SecondaryCell4x4MIMOparametersFDD = -1;
static gint ett_rrc_T_newConfiguration_10 = -1;
static gint ett_rrc_ServingCellChangeParameters = -1;
static gint ett_rrc_SF256_AndCodeNumber = -1;
static gint ett_rrc_SF512_AndCodeNumber = -1;
static gint ett_rrc_SF512_AndPilot = -1;
static gint ett_rrc_SFN_TimeInfo = -1;
static gint ett_rrc_SPS_Information_TDD128_r8 = -1;
static gint ett_rrc_SSDT_Information = -1;
static gint ett_rrc_SSDT_Information_r4 = -1;
static gint ett_rrc_StandaloneMidambleInfo_TDD128 = -1;
static gint ett_rrc_SynchronisationParameters_r4 = -1;
static gint ett_rrc_T_sync_UL_CodesBitmap = -1;
static gint ett_rrc_Sync_UL_Codes_Bitmap = -1;
static gint ett_rrc_SYNC_UL_Procedure_r4 = -1;
static gint ett_rrc_SYNC_UL_Info_r4 = -1;
static gint ett_rrc_SYNC_UL_InfoForE_RUCCH = -1;
static gint ett_rrc_TargetCellPreconfigInfo = -1;
static gint ett_rrc_TargetCellPreconfigInfo_r9 = -1;
static gint ett_rrc_TargetCellPreconfigInfo_r10 = -1;
static gint ett_rrc_TargetCellPreconfigInfo_vbxyext = -1;
static gint ett_rrc_TargetCellPreconfigInfo_r11 = -1;
static gint ett_rrc_TDD_MBSFNInformation = -1;
static gint ett_rrc_TDD_MBSFNTSlotInfo = -1;
static gint ett_rrc_TDD_PRACH_CCodeList = -1;
static gint ett_rrc_SEQUENCE_SIZE_1_8_OF_TDD_PRACH_CCode8 = -1;
static gint ett_rrc_SEQUENCE_SIZE_1_8_OF_TDD_PRACH_CCode16 = -1;
static gint ett_rrc_TDD768_PRACH_CCodeList = -1;
static gint ett_rrc_SEQUENCE_SIZE_1_16_OF_TDD768_PRACH_CCode32 = -1;
static gint ett_rrc_SEQUENCE_SIZE_1_16_OF_TDD768_PRACH_CCode16 = -1;
static gint ett_rrc_TGP_Sequence = -1;
static gint ett_rrc_T_tgps_Status = -1;
static gint ett_rrc_T_activate = -1;
static gint ett_rrc_TGP_Sequence_r8 = -1;
static gint ett_rrc_T_tgps_Status_01 = -1;
static gint ett_rrc_T_activate_01 = -1;
static gint ett_rrc_TGP_Sequence_r10 = -1;
static gint ett_rrc_T_tgps_Status_02 = -1;
static gint ett_rrc_T_activate_02 = -1;
static gint ett_rrc_TGP_SequenceList = -1;
static gint ett_rrc_TGP_SequenceList_r8 = -1;
static gint ett_rrc_TGP_SequenceList_r10 = -1;
static gint ett_rrc_TGP_SequenceShort = -1;
static gint ett_rrc_T_tgps_Status_03 = -1;
static gint ett_rrc_T_activate_03 = -1;
static gint ett_rrc_TGP_SequenceShort_r10 = -1;
static gint ett_rrc_T_tgps_Status_04 = -1;
static gint ett_rrc_T_activate_04 = -1;
static gint ett_rrc_TGPS_ConfigurationParams = -1;
static gint ett_rrc_TGPS_ConfigurationParams_r8 = -1;
static gint ett_rrc_TGPS_ConfigurationParams_r10 = -1;
static gint ett_rrc_TimeInfo = -1;
static gint ett_rrc_TimeslotList = -1;
static gint ett_rrc_TimeslotList_r4 = -1;
static gint ett_rrc_SEQUENCE_SIZE_1_maxTS_OF_TimeslotNumber = -1;
static gint ett_rrc_SEQUENCE_SIZE_1_maxTS_LCR_OF_TimeslotNumber_LCR_r4 = -1;
static gint ett_rrc_DTX_DRX_TimingInfo_r7 = -1;
static gint ett_rrc_T_timing = -1;
static gint ett_rrc_TwoMsHarqConfiguration = -1;
static gint ett_rrc_UCSM_Info = -1;
static gint ett_rrc_UL_16QAM_Config = -1;
static gint ett_rrc_UL_16QAM_Settings = -1;
static gint ett_rrc_UL_64QAM_Config = -1;
static gint ett_rrc_UL_64QAM_Settings = -1;
static gint ett_rrc_UL_CCTrCH = -1;
static gint ett_rrc_UL_CCTrCH_r4 = -1;
static gint ett_rrc_T_tddOption_19 = -1;
static gint ett_rrc_T_tdd384_34 = -1;
static gint ett_rrc_T_tdd128_53 = -1;
static gint ett_rrc_UL_CCTrCH_r7 = -1;
static gint ett_rrc_T_tddOption_20 = -1;
static gint ett_rrc_T_tdd384_35 = -1;
static gint ett_rrc_T_tdd768_20 = -1;
static gint ett_rrc_T_tdd128_54 = -1;
static gint ett_rrc_UL_CCTrCHList = -1;
static gint ett_rrc_UL_CCTrCHList_r4 = -1;
static gint ett_rrc_UL_CCTrCHList_r7 = -1;
static gint ett_rrc_UL_CCTrCHListToRemove = -1;
static gint ett_rrc_UL_CCTrChTPCList = -1;
static gint ett_rrc_UL_ChannelRequirement = -1;
static gint ett_rrc_UL_ChannelRequirement_r4 = -1;
static gint ett_rrc_UL_ChannelRequirement_r5 = -1;
static gint ett_rrc_UL_ChannelRequirementWithCPCH_SetID = -1;
static gint ett_rrc_UL_ChannelRequirementWithCPCH_SetID_r4 = -1;
static gint ett_rrc_UL_ChannelRequirementWithCPCH_SetID_r5 = -1;
static gint ett_rrc_UL_CLTD_InfoFDD = -1;
static gint ett_rrc_T_configurationInfo_04 = -1;
static gint ett_rrc_T_newConfiguration_11 = -1;
static gint ett_rrc_UL_MIMO_InfoFDD = -1;
static gint ett_rrc_T_configurationInfo_05 = -1;
static gint ett_rrc_T_newConfiguration_12 = -1;
static gint ett_rrc_UL_DL_Mode = -1;
static gint ett_rrc_T_ul_and_dl = -1;
static gint ett_rrc_UL_DPCH_CodeInfoForCommonEDCH = -1;
static gint ett_rrc_UL_DPCH_Info = -1;
static gint ett_rrc_T_modeSpecificInfo_81 = -1;
static gint ett_rrc_T_fdd_113 = -1;
static gint ett_rrc_T_tdd_64 = -1;
static gint ett_rrc_UL_DPCH_Info_r4 = -1;
static gint ett_rrc_T_modeSpecificInfo_82 = -1;
static gint ett_rrc_T_fdd_114 = -1;
static gint ett_rrc_T_tdd_65 = -1;
static gint ett_rrc_UL_DPCH_Info_r5 = -1;
static gint ett_rrc_T_modeSpecificInfo_83 = -1;
static gint ett_rrc_T_fdd_115 = -1;
static gint ett_rrc_T_tdd_66 = -1;
static gint ett_rrc_UL_DPCH_Info_r6 = -1;
static gint ett_rrc_T_modeSpecificInfo_84 = -1;
static gint ett_rrc_T_fdd_116 = -1;
static gint ett_rrc_T_dpdchPresence = -1;
static gint ett_rrc_T_present_02 = -1;
static gint ett_rrc_T_notPresent = -1;
static gint ett_rrc_T_tdd_67 = -1;
static gint ett_rrc_UL_DPCH_Info_r7 = -1;
static gint ett_rrc_T_modeSpecificInfo_85 = -1;
static gint ett_rrc_T_fdd_117 = -1;
static gint ett_rrc_T_dpdchPresence_01 = -1;
static gint ett_rrc_T_present_03 = -1;
static gint ett_rrc_T_notPresent_01 = -1;
static gint ett_rrc_T_tdd_68 = -1;
static gint ett_rrc_UL_DPCH_Info_r11 = -1;
static gint ett_rrc_T_modeSpecificInfo_86 = -1;
static gint ett_rrc_T_fdd_118 = -1;
static gint ett_rrc_T_dpdchPresence_02 = -1;
static gint ett_rrc_T_present_04 = -1;
static gint ett_rrc_T_notPresent_02 = -1;
static gint ett_rrc_T_tdd_69 = -1;
static gint ett_rrc_UL_DPCH_InfoPostFDD = -1;
static gint ett_rrc_UL_DPCH_InfoPostTDD = -1;
static gint ett_rrc_UL_DPCH_InfoPostTDD_LCR_r4 = -1;
static gint ett_rrc_UL_DPCH_InfoPredef = -1;
static gint ett_rrc_T_modeSpecificInfo_87 = -1;
static gint ett_rrc_T_fdd_119 = -1;
static gint ett_rrc_T_tdd_70 = -1;
static gint ett_rrc_UL_DPCH_InfoPredef_v770ext = -1;
static gint ett_rrc_T_modeSpecificInfo_88 = -1;
static gint ett_rrc_T_fdd_120 = -1;
static gint ett_rrc_UL_DPCH_PowerControlInfo = -1;
static gint ett_rrc_T_fdd_121 = -1;
static gint ett_rrc_T_tdd_71 = -1;
static gint ett_rrc_T_ul_OL_PC_Signalling = -1;
static gint ett_rrc_T_individuallySignalled = -1;
static gint ett_rrc_UL_DPCH_PowerControlInfo_r4 = -1;
static gint ett_rrc_T_fdd_122 = -1;
static gint ett_rrc_T_tdd_72 = -1;
static gint ett_rrc_T_ul_OL_PC_Signalling_01 = -1;
static gint ett_rrc_T_individuallySignalled_01 = -1;
static gint ett_rrc_T_tddOption_21 = -1;
static gint ett_rrc_T_tdd384_36 = -1;
static gint ett_rrc_T_tdd128_55 = -1;
static gint ett_rrc_UL_DPCH_PowerControlInfo_r5 = -1;
static gint ett_rrc_T_fdd_123 = -1;
static gint ett_rrc_T_tdd_73 = -1;
static gint ett_rrc_T_ul_OL_PC_Signalling_02 = -1;
static gint ett_rrc_T_individuallySignalled_02 = -1;
static gint ett_rrc_T_tddOption_22 = -1;
static gint ett_rrc_T_tdd384_37 = -1;
static gint ett_rrc_T_tdd128_56 = -1;
static gint ett_rrc_UL_DPCH_PowerControlInfo_r6 = -1;
static gint ett_rrc_T_fdd_124 = -1;
static gint ett_rrc_T_tdd_74 = -1;
static gint ett_rrc_T_ul_OL_PC_Signalling_03 = -1;
static gint ett_rrc_T_individuallySignalled_03 = -1;
static gint ett_rrc_T_tddOption_23 = -1;
static gint ett_rrc_T_tdd384_38 = -1;
static gint ett_rrc_T_tdd128_57 = -1;
static gint ett_rrc_UL_DPCH_PowerControlInfo_r7 = -1;
static gint ett_rrc_T_fdd_125 = -1;
static gint ett_rrc_T_tdd_75 = -1;
static gint ett_rrc_T_ul_OL_PC_Signalling_04 = -1;
static gint ett_rrc_T_individuallySignalled_04 = -1;
static gint ett_rrc_T_tddOption_24 = -1;
static gint ett_rrc_T_tdd384_39 = -1;
static gint ett_rrc_T_tdd768_21 = -1;
static gint ett_rrc_T_tdd128_58 = -1;
static gint ett_rrc_UL_DPCH_PowerControlInfo_r11 = -1;
static gint ett_rrc_T_fdd_126 = -1;
static gint ett_rrc_T_tdd_76 = -1;
static gint ett_rrc_T_ul_OL_PC_Signalling_05 = -1;
static gint ett_rrc_T_individuallySignalled_05 = -1;
static gint ett_rrc_T_tddOption_25 = -1;
static gint ett_rrc_T_tdd384_40 = -1;
static gint ett_rrc_T_tdd768_22 = -1;
static gint ett_rrc_T_tdd128_59 = -1;
static gint ett_rrc_UL_DPCHpowerControlInfoConcurrentTTI = -1;
static gint ett_rrc_UL_DPCHpowerControlInfoForCommonEDCH = -1;
static gint ett_rrc_UL_DPCH_PowerControlInfoPostFDD = -1;
static gint ett_rrc_UL_DPCH_PowerControlInfoPostTDD = -1;
static gint ett_rrc_UL_DPCH_PowerControlInfoPostTDD_LCR_r4 = -1;
static gint ett_rrc_UL_DPCH_PowerControlInfoPredef = -1;
static gint ett_rrc_T_fdd_127 = -1;
static gint ett_rrc_T_tdd_77 = -1;
static gint ett_rrc_UL_EDCH_Information_ext = -1;
static gint ett_rrc_T_tdd_78 = -1;
static gint ett_rrc_UL_EDCH_Information_r6 = -1;
static gint ett_rrc_UL_EDCH_Information_r7 = -1;
static gint ett_rrc_T_modeSpecificInfo_89 = -1;
static gint ett_rrc_T_fdd_128 = -1;
static gint ett_rrc_T_tdd_79 = -1;
static gint ett_rrc_UL_EDCH_Information_r8 = -1;
static gint ett_rrc_T_modeSpecificInfo_90 = -1;
static gint ett_rrc_T_fdd_129 = -1;
static gint ett_rrc_T_tdd_80 = -1;
static gint ett_rrc_UL_EDCH_Information_r9 = -1;
static gint ett_rrc_T_modeSpecificInfo_91 = -1;
static gint ett_rrc_T_fdd_130 = -1;
static gint ett_rrc_T_tdd_81 = -1;
static gint ett_rrc_UL_EDCH_Information_r11 = -1;
static gint ett_rrc_T_modeSpecificInfo_92 = -1;
static gint ett_rrc_T_fdd_131 = -1;
static gint ett_rrc_T_tdd_82 = -1;
static gint ett_rrc_UL_MulticarrierEDCH_InfolistItem_TDD128 = -1;
static gint ett_rrc_UL_MulticarrierEDCH_Infolist_TDD128 = -1;
static gint ett_rrc_UL_MulticarrierEDCH_Deletelist_TDD128 = -1;
static gint ett_rrc_UL_MulticarrierEDCHInfo_TDD128 = -1;
static gint ett_rrc_T_configuration_info = -1;
static gint ett_rrc_UL_MulticarrierEDCH_NewConfigurationInfo_TDD128 = -1;
static gint ett_rrc_UL_SynchronisationParameters_r4 = -1;
static gint ett_rrc_UL_TimingAdvanceControl = -1;
static gint ett_rrc_T_enabled = -1;
static gint ett_rrc_UL_TimingAdvanceControl_r4 = -1;
static gint ett_rrc_T_enabled_01 = -1;
static gint ett_rrc_T_tddOption_26 = -1;
static gint ett_rrc_T_tdd384_41 = -1;
static gint ett_rrc_T_tdd128_60 = -1;
static gint ett_rrc_UL_TimingAdvanceControl_r7 = -1;
static gint ett_rrc_T_enabled_02 = -1;
static gint ett_rrc_T_tddOption_27 = -1;
static gint ett_rrc_T_tdd384_42 = -1;
static gint ett_rrc_T_tdd768_23 = -1;
static gint ett_rrc_T_tdd128_61 = -1;
static gint ett_rrc_UL_TimingAdvanceControl_LCR_r4 = -1;
static gint ett_rrc_T_enabled_03 = -1;
static gint ett_rrc_UL_TS_ChannelisationCodeList = -1;
static gint ett_rrc_UL_TS_ChannelisationCodeList_r7 = -1;
static gint ett_rrc_UL_TS_ChannelisationCodeList_r7_item = -1;
static gint ett_rrc_UL_TS_ChannelisationCodeList_VHCR = -1;
static gint ett_rrc_UplinkAdditionalTimeslots = -1;
static gint ett_rrc_T_parameters_04 = -1;
static gint ett_rrc_T_sameAsLast_04 = -1;
static gint ett_rrc_T_newParameters_04 = -1;
static gint ett_rrc_UplinkAdditionalTimeslots_LCR_r4 = -1;
static gint ett_rrc_T_parameters_05 = -1;
static gint ett_rrc_T_sameAsLast_05 = -1;
static gint ett_rrc_T_newParameters_05 = -1;
static gint ett_rrc_UplinkAdditionalTimeslots_LCR_r7 = -1;
static gint ett_rrc_T_parameters_06 = -1;
static gint ett_rrc_T_sameAsLast_06 = -1;
static gint ett_rrc_T_newParameters_06 = -1;
static gint ett_rrc_UplinkAdditionalTimeslots_VHCR = -1;
static gint ett_rrc_T_parameters_07 = -1;
static gint ett_rrc_T_sameAsLast_07 = -1;
static gint ett_rrc_T_newParameters_07 = -1;
static gint ett_rrc_UplinkTimeslotsCodes = -1;
static gint ett_rrc_T_moreTimeslots_04 = -1;
static gint ett_rrc_T_additionalTimeslots_04 = -1;
static gint ett_rrc_T_consecutive_02 = -1;
static gint ett_rrc_SEQUENCE_SIZE_1_maxTS_1_OF_UplinkAdditionalTimeslots = -1;
static gint ett_rrc_UplinkTimeslotsCodes_VHCR = -1;
static gint ett_rrc_T_moreTimeslots_05 = -1;
static gint ett_rrc_T_additionalTimeslots_05 = -1;
static gint ett_rrc_T_consecutive_03 = -1;
static gint ett_rrc_SEQUENCE_SIZE_1_maxTS_1_OF_UplinkAdditionalTimeslots_VHCR = -1;
static gint ett_rrc_UplinkTimeslotsCodes_LCR_r4 = -1;
static gint ett_rrc_T_moreTimeslots_06 = -1;
static gint ett_rrc_T_additionalTimeslots_06 = -1;
static gint ett_rrc_T_consecutive_04 = -1;
static gint ett_rrc_SEQUENCE_SIZE_1_maxTS_LCR_1_OF_UplinkAdditionalTimeslots_LCR_r4 = -1;
static gint ett_rrc_UplinkTimeslotsCodes_LCR_r7 = -1;
static gint ett_rrc_T_moreTimeslots_07 = -1;
static gint ett_rrc_T_additionalTimeslots_07 = -1;
static gint ett_rrc_T_consecutive_05 = -1;
static gint ett_rrc_SEQUENCE_SIZE_1_maxTS_LCR_1_OF_UplinkAdditionalTimeslots_LCR_r7 = -1;
static gint ett_rrc_AcquisitionSatInfo = -1;
static gint ett_rrc_AcquisitionSatInfo_va40ext = -1;
static gint ett_rrc_AcquisitionSatInfo_r10 = -1;
static gint ett_rrc_AcquisitionSatInfoList = -1;
static gint ett_rrc_AcquisitionSatInfoList_va40ext = -1;
static gint ett_rrc_AcquisitionSatInfoList_r10 = -1;
static gint ett_rrc_AdditionalMeasurementID_List = -1;
static gint ett_rrc_AdditionalMeasurementID_List_r9 = -1;
static gint ett_rrc_AlmanacSatInfo = -1;
static gint ett_rrc_AlmanacSatInfoList = -1;
static gint ett_rrc_ALM_ECEFsbasAlmanacSet = -1;
static gint ett_rrc_ALM_GlonassAlmanacSet = -1;
static gint ett_rrc_ALM_keplerianParameters = -1;
static gint ett_rrc_ALM_MidiAlmanacSet = -1;
static gint ett_rrc_ALM_NAVKeplerianSet = -1;
static gint ett_rrc_ALM_ReducedKeplerianSet = -1;
static gint ett_rrc_AuxInfoGANSS_ID1 = -1;
static gint ett_rrc_AuxInfoGANSS_ID1_element = -1;
static gint ett_rrc_AuxInfoGANSS_ID3 = -1;
static gint ett_rrc_AuxInfoGANSS_ID3_element = -1;
static gint ett_rrc_AzimuthAndElevation = -1;
static gint ett_rrc_AzimuthAndElevation_va40ext = -1;
static gint ett_rrc_AzimuthAndElevation_r10 = -1;
static gint ett_rrc_BadSatList = -1;
static gint ett_rrc_BLER_MeasurementResults = -1;
static gint ett_rrc_BLER_MeasurementResultsList = -1;
static gint ett_rrc_BLER_TransChIdList = -1;
static gint ett_rrc_BSICReported = -1;
static gint ett_rrc_BurstModeParameters = -1;
static gint ett_rrc_CellDCH_ReportCriteria = -1;
static gint ett_rrc_CellDCH_ReportCriteria_LCR_r4 = -1;
static gint ett_rrc_CellDCHMeasOccasionPattern_LCR = -1;
static gint ett_rrc_CellDCHMeasOccasionInfo_TDD128_r9 = -1;
static gint ett_rrc_SEQUENCE_SIZE_1_maxMeasOccasionPattern_OF_CellDCHMeasOccasionPattern_LCR = -1;
static gint ett_rrc_CellInfo = -1;
static gint ett_rrc_T_modeSpecificInfo_93 = -1;
static gint ett_rrc_T_fdd_132 = -1;
static gint ett_rrc_T_tdd_83 = -1;
static gint ett_rrc_CellInfo_r4 = -1;
static gint ett_rrc_T_modeSpecificInfo_94 = -1;
static gint ett_rrc_T_fdd_133 = -1;
static gint ett_rrc_T_tdd_84 = -1;
static gint ett_rrc_CellInfo_LCR_r8_ext = -1;
static gint ett_rrc_CellInfo_r9 = -1;
static gint ett_rrc_T_modeSpecificInfo_95 = -1;
static gint ett_rrc_T_fdd_134 = -1;
static gint ett_rrc_T_tdd_85 = -1;
static gint ett_rrc_CellInfoListToBeExcluded_r11 = -1;
static gint ett_rrc_CellInfoSI_RSCP = -1;
static gint ett_rrc_T_modeSpecificInfo_96 = -1;
static gint ett_rrc_T_fdd_135 = -1;
static gint ett_rrc_T_tdd_86 = -1;
static gint ett_rrc_CellInfoSI_RSCP_LCR_r4 = -1;
static gint ett_rrc_CellInfoSI_ECN0 = -1;
static gint ett_rrc_T_modeSpecificInfo_97 = -1;
static gint ett_rrc_T_fdd_136 = -1;
static gint ett_rrc_T_tdd_87 = -1;
static gint ett_rrc_CellInfoSI_ECN0_LCR_r4 = -1;
static gint ett_rrc_CellInfoSI_HCS_RSCP = -1;
static gint ett_rrc_T_modeSpecificInfo_98 = -1;
static gint ett_rrc_T_fdd_137 = -1;
static gint ett_rrc_T_tdd_88 = -1;
static gint ett_rrc_CellInfoSI_HCS_RSCP_LCR_r4 = -1;
static gint ett_rrc_CellInfoSI_HCS_ECN0 = -1;
static gint ett_rrc_T_modeSpecificInfo_99 = -1;
static gint ett_rrc_T_fdd_138 = -1;
static gint ett_rrc_T_tdd_89 = -1;
static gint ett_rrc_CellInfoSI_HCS_ECN0_LCR_r4 = -1;
static gint ett_rrc_CellMeasuredResults = -1;
static gint ett_rrc_T_modeSpecificInfo_100 = -1;
static gint ett_rrc_T_fdd_139 = -1;
static gint ett_rrc_T_tdd_90 = -1;
static gint ett_rrc_CellMeasuredResults_r9 = -1;
static gint ett_rrc_T_modeSpecificInfo_101 = -1;
static gint ett_rrc_T_fdd_140 = -1;
static gint ett_rrc_T_tdd_91 = -1;
static gint ett_rrc_CellMeasuredResults_v920ext = -1;
static gint ett_rrc_CellMeasurementEventResults = -1;
static gint ett_rrc_SEQUENCE_SIZE_1_maxCellMeas_OF_PrimaryCPICH_Info = -1;
static gint ett_rrc_SEQUENCE_SIZE_1_maxCellMeas_OF_PrimaryCCPCH_Info = -1;
static gint ett_rrc_CellMeasurementEventResultsOnSecUlFreq = -1;
static gint ett_rrc_SEQUENCE_SIZE_1_maxCellMeasOnSecULFreq_OF_PrimaryCPICH_Info = -1;
static gint ett_rrc_CellMeasurementEventResults_LCR_r4 = -1;
static gint ett_rrc_CellReportingQuantities = -1;
static gint ett_rrc_T_modeSpecificInfo_102 = -1;
static gint ett_rrc_T_fdd_141 = -1;
static gint ett_rrc_T_tdd_92 = -1;
static gint ett_rrc_CellSelectReselectInfoMC_RSCP = -1;
static gint ett_rrc_T_modeSpecificInfo_103 = -1;
static gint ett_rrc_T_fdd_142 = -1;
static gint ett_rrc_T_tdd_93 = -1;
static gint ett_rrc_T_gsm_02 = -1;
static gint ett_rrc_CellSelectReselectInfoSIB_11_12 = -1;
static gint ett_rrc_T_modeSpecificInfo_104 = -1;
static gint ett_rrc_T_fdd_143 = -1;
static gint ett_rrc_T_tdd_94 = -1;
static gint ett_rrc_T_gsm_03 = -1;
static gint ett_rrc_CellSelectReselectInfoSIB_11_12_RSCP = -1;
static gint ett_rrc_T_modeSpecificInfo_105 = -1;
static gint ett_rrc_T_fdd_144 = -1;
static gint ett_rrc_T_tdd_95 = -1;
static gint ett_rrc_T_gsm_04 = -1;
static gint ett_rrc_CellSelectReselectInfoSIB_11_12_ECN0 = -1;
static gint ett_rrc_T_modeSpecificInfo_106 = -1;
static gint ett_rrc_T_fdd_145 = -1;
static gint ett_rrc_T_tdd_96 = -1;
static gint ett_rrc_T_gsm_05 = -1;
static gint ett_rrc_CellSelectReselectInfoSIB_11_12_HCS_RSCP = -1;
static gint ett_rrc_T_modeSpecificInfo_107 = -1;
static gint ett_rrc_T_fdd_146 = -1;
static gint ett_rrc_T_tdd_97 = -1;
static gint ett_rrc_T_gsm_06 = -1;
static gint ett_rrc_CellSelectReselectInfoSIB_11_12_HCS_ECN0 = -1;
static gint ett_rrc_T_modeSpecificInfo_108 = -1;
static gint ett_rrc_T_fdd_147 = -1;
static gint ett_rrc_T_tdd_98 = -1;
static gint ett_rrc_T_gsm_07 = -1;
static gint ett_rrc_CellSelectReselectInfo_v590ext = -1;
static gint ett_rrc_CellSelectReselectInfoPCHFACH_v5b0ext = -1;
static gint ett_rrc_CellSelectReselectInfoTreselectionScaling_v5c0ext = -1;
static gint ett_rrc_CellsForInterFreqMeasList = -1;
static gint ett_rrc_CellsForInterRATMeasList = -1;
static gint ett_rrc_CellsForIntraFreqMeasList = -1;
static gint ett_rrc_CellsForIntraFreqMeasListOnSecULFreq = -1;
static gint ett_rrc_CellSynchronisationInfo = -1;
static gint ett_rrc_T_modeSpecificInfo_109 = -1;
static gint ett_rrc_T_fdd_148 = -1;
static gint ett_rrc_T_tdd_99 = -1;
static gint ett_rrc_CellToReport = -1;
static gint ett_rrc_CellToReportList = -1;
static gint ett_rrc_CNAVclockModel = -1;
static gint ett_rrc_CountC_SFN_Frame_difference = -1;
static gint ett_rrc_CSGCellInfo = -1;
static gint ett_rrc_T_modeSpecificInfo_110 = -1;
static gint ett_rrc_T_fdd_149 = -1;
static gint ett_rrc_CSGCellInfoList = -1;
static gint ett_rrc_CSGInterFreqCellInfo = -1;
static gint ett_rrc_CSGInterFreqCellInfoList = -1;
static gint ett_rrc_CSGProximityDetection = -1;
static gint ett_rrc_CSGProximityIndication = -1;
static gint ett_rrc_T_radioAccessTechnology_01 = -1;
static gint ett_rrc_T_uTRA = -1;
static gint ett_rrc_T_e_UTRA = -1;
static gint ett_rrc_DataBitAssistance = -1;
static gint ett_rrc_DataBitAssistanceList = -1;
static gint ett_rrc_DataBitAssistanceSat = -1;
static gint ett_rrc_DataBitAssistanceSgnList = -1;
static gint ett_rrc_DeltaRSCPPerCell = -1;
static gint ett_rrc_DeltaUT1 = -1;
static gint ett_rrc_DGANSSInfo = -1;
static gint ett_rrc_DGANSSInfo_r9 = -1;
static gint ett_rrc_DGANSSInfo_v920ext = -1;
static gint ett_rrc_DGANSSInfoList = -1;
static gint ett_rrc_DGANSSInfoList_r9 = -1;
static gint ett_rrc_DGANSSInfoList_v920ext = -1;
static gint ett_rrc_DGANSSSignalInformation = -1;
static gint ett_rrc_DGANSSSignalInformation_r9 = -1;
static gint ett_rrc_DGANSSSignalInformation_v920ext = -1;
static gint ett_rrc_DGANSSSignalInformationList = -1;
static gint ett_rrc_DGANSSSignalInformationList_r9 = -1;
static gint ett_rrc_DGANSSSignalInformationList_v920ext = -1;
static gint ett_rrc_DGPS_CorrectionSatInfo = -1;
static gint ett_rrc_DGPS_CorrectionSatInfo_r9 = -1;
static gint ett_rrc_DGPS_CorrectionSatInfo_v920ext = -1;
static gint ett_rrc_DGPS_CorrectionSatInfoList = -1;
static gint ett_rrc_DGPS_CorrectionSatInfoList_r9 = -1;
static gint ett_rrc_DGPS_CorrectionSatInfoList_v920ext = -1;
static gint ett_rrc_EllipsoidPoint = -1;
static gint ett_rrc_EllipsoidPointAltitude = -1;
static gint ett_rrc_EllipsoidPointAltitudeEllipsoide = -1;
static gint ett_rrc_EllipsoidPointUncertCircle = -1;
static gint ett_rrc_EllipsoidPointUncertEllipse = -1;
static gint ett_rrc_Eutra_EventResult = -1;
static gint ett_rrc_SEQUENCE_SIZE_1_maxReportedEUTRACellPerFreq_OF_EUTRA_PhysicalCellIdentity = -1;
static gint ett_rrc_Eutra_EventResultList = -1;
static gint ett_rrc_EUTRA_EventResults = -1;
static gint ett_rrc_EUTRA_EventResultsForCELLFACH = -1;
static gint ett_rrc_EUTRA_FrequencyAndPriorityInfo = -1;
static gint ett_rrc_EUTRA_FrequencyAndPriorityInfo_v920ext = -1;
static gint ett_rrc_EUTRA_FrequencyAndPriorityInfoList = -1;
static gint ett_rrc_EUTRA_FrequencyAndPriorityInfoList_v920ext = -1;
static gint ett_rrc_EUTRA_FrequencyAndPriorityInfoList_vbxyext = -1;
static gint ett_rrc_EUTRA_FrequencyList = -1;
static gint ett_rrc_EUTRA_FrequencyList_r9 = -1;
static gint ett_rrc_EUTRA_FrequencyList_r11 = -1;
static gint ett_rrc_EUTRA_SIAcquisition = -1;
static gint ett_rrc_EUTRA_MeasuredCells = -1;
static gint ett_rrc_Eutra_MeasuredResult = -1;
static gint ett_rrc_SEQUENCE_SIZE_1_maxReportedEUTRACellPerFreq_OF_EUTRA_MeasuredCells = -1;
static gint ett_rrc_Eutra_MeasuredResultList = -1;
static gint ett_rrc_EUTRA_MeasuredResults = -1;
static gint ett_rrc_EUTRA_MeasuredCells_v920ext = -1;
static gint ett_rrc_Eutra_MeasuredResult_v920ext = -1;
static gint ett_rrc_SEQUENCE_SIZE_1_maxReportedEUTRACellPerFreq_OF_EUTRA_MeasuredCells_v920ext = -1;
static gint ett_rrc_Eutra_MeasuredResultList_v920ext = -1;
static gint ett_rrc_EUTRA_MeasuredResults_v920ext = -1;
static gint ett_rrc_EUTRA_SIacquisitionResults = -1;
static gint ett_rrc_T_cgiInfo = -1;
static gint ett_rrc_EUTRA_FrequencyRemoval = -1;
static gint ett_rrc_SEQUENCE_SIZE_1_maxNumEUTRAFreqs_OF_EARFCN = -1;
static gint ett_rrc_EUTRA_FrequencyInfoList = -1;
static gint ett_rrc_EUTRA_FrequencyInfoList_r11 = -1;
static gint ett_rrc_EUTRA_FrequencyInfo = -1;
static gint ett_rrc_EUTRA_FrequencyInfo_r11 = -1;
static gint ett_rrc_EUTRA_FrequencyRACHReportingInfo = -1;
static gint ett_rrc_EutraMeasurementForCELLFACH = -1;
static gint ett_rrc_SEQUENCE_SIZE_1_maxNumEUTRAFreqs_FACH_OF_EARFCN = -1;
static gint ett_rrc_Event1a = -1;
static gint ett_rrc_Event1a_r4 = -1;
static gint ett_rrc_Event1a_OnSecUlFreq_r9 = -1;
static gint ett_rrc_Event1a_LCR_r4 = -1;
static gint ett_rrc_Event1b = -1;
static gint ett_rrc_Event1b_r4 = -1;
static gint ett_rrc_Event1b_r7 = -1;
static gint ett_rrc_Event1b_OnSecUlFreq_r9 = -1;
static gint ett_rrc_Event1b_LCR_r4 = -1;
static gint ett_rrc_Event1c = -1;
static gint ett_rrc_Event1d = -1;
static gint ett_rrc_Event1e = -1;
static gint ett_rrc_Event1e_r6 = -1;
static gint ett_rrc_Event1f = -1;
static gint ett_rrc_Event1f_r6 = -1;
static gint ett_rrc_Event1j_r6 = -1;
static gint ett_rrc_Event2a = -1;
static gint ett_rrc_Event2a_r6 = -1;
static gint ett_rrc_Event2a_r10 = -1;
static gint ett_rrc_Event2a_r11 = -1;
static gint ett_rrc_Event2b = -1;
static gint ett_rrc_Event2b_r6 = -1;
static gint ett_rrc_Event2b_r10 = -1;
static gint ett_rrc_Event2b_r11 = -1;
static gint ett_rrc_Event2c = -1;
static gint ett_rrc_Event2c_r6 = -1;
static gint ett_rrc_Event2c_r10 = -1;
static gint ett_rrc_Event2c_r11 = -1;
static gint ett_rrc_Event2d = -1;
static gint ett_rrc_Event2d_r6 = -1;
static gint ett_rrc_Event2d_r10 = -1;
static gint ett_rrc_Event2e = -1;
static gint ett_rrc_Event2e_r6 = -1;
static gint ett_rrc_Event2e_r10 = -1;
static gint ett_rrc_Event2e_r11 = -1;
static gint ett_rrc_Event2f = -1;
static gint ett_rrc_Event2f_r6 = -1;
static gint ett_rrc_Event2f_r10 = -1;
static gint ett_rrc_Event3a = -1;
static gint ett_rrc_Event3b = -1;
static gint ett_rrc_Event3c = -1;
static gint ett_rrc_Event3d = -1;
static gint ett_rrc_EventResults = -1;
static gint ett_rrc_EventResults_v770ext = -1;
static gint ett_rrc_EventResults_v860ext = -1;
static gint ett_rrc_EventResults_va40ext = -1;
static gint ett_rrc_EventResultsOnSecUlFreq = -1;
static gint ett_rrc_ExtraDoppler = -1;
static gint ett_rrc_ExtraDopplerInfo = -1;
static gint ett_rrc_FACH_MeasurementOccasionInfo = -1;
static gint ett_rrc_SEQUENCE_SIZE_1_maxOtherRAT_OF_RAT_Type = -1;
static gint ett_rrc_FACH_MeasurementOccasionInfo_LCR_r4_ext = -1;
static gint ett_rrc_ForbiddenAffectCell = -1;
static gint ett_rrc_ForbiddenAffectCell_r4 = -1;
static gint ett_rrc_ForbiddenAffectCell_LCR_r4 = -1;
static gint ett_rrc_ForbiddenAffectCellList = -1;
static gint ett_rrc_ForbiddenAffectCellList_r4 = -1;
static gint ett_rrc_ForbiddenAffectCellList_LCR_r4 = -1;
static gint ett_rrc_ForbiddenAffectCellListOnSecULFreq = -1;
static gint ett_rrc_ForbiddenAffectCellOnSecULFreq = -1;
static gint ett_rrc_GanssDataBits = -1;
static gint ett_rrc_GANSSGenericData = -1;
static gint ett_rrc_GANSSGenericData_v860ext = -1;
static gint ett_rrc_GANSSGenericData_r8 = -1;
static gint ett_rrc_GANSSGenericData_r9 = -1;
static gint ett_rrc_GANSSGenericData_v920ext = -1;
static gint ett_rrc_GANSSGenericData_va40ext = -1;
static gint ett_rrc_GANSSGenericData_r10 = -1;
static gint ett_rrc_GANSSGenericDataList = -1;
static gint ett_rrc_GANSSGenericDataList_v860ext = -1;
static gint ett_rrc_GANSSGenericDataList_r8 = -1;
static gint ett_rrc_GANSSGenericDataList_r9 = -1;
static gint ett_rrc_GANSSGenericDataList_v920ext = -1;
static gint ett_rrc_GANSSGenericDataList_va40ext = -1;
static gint ett_rrc_GANSSGenericDataList_r10 = -1;
static gint ett_rrc_GANSSGenericMeasurementInfo = -1;
static gint ett_rrc_GANSSGenericMeasurementInfo_item = -1;
static gint ett_rrc_GANSSGenericMeasurementInfo_v860ext = -1;
static gint ett_rrc_GANSSGenericMeasurementInfo_v860ext_item = -1;
static gint ett_rrc_GANSSMeasurementParameters = -1;
static gint ett_rrc_GANSSMeasurementParameters_item = -1;
static gint ett_rrc_GANSSMeasurementParameters_v860ext = -1;
static gint ett_rrc_GANSSMeasurementParameters_v860ext_item = -1;
static gint ett_rrc_GANSSMeasurementSignalList = -1;
static gint ett_rrc_GANSSMeasurementSignalList_item = -1;
static gint ett_rrc_GANSSMeasurementSignalList_v860ext = -1;
static gint ett_rrc_GANSSMeasurementSignalList_v860ext_item = -1;
static gint ett_rrc_GanssNavigationModelAdditionalData = -1;
static gint ett_rrc_GANSSReferenceTimeOnly = -1;
static gint ett_rrc_GanssReqGenericData = -1;
static gint ett_rrc_GanssReqGenericData_v860ext = -1;
static gint ett_rrc_T_ganssAddADchoices = -1;
static gint ett_rrc_GanssRequestedGenericAssistanceDataList = -1;
static gint ett_rrc_GanssRequestedGenericAssistanceDataList_v860ext = -1;
static gint ett_rrc_GANSSSatelliteInformation = -1;
static gint ett_rrc_GANSSSatelliteInformation_va40ext = -1;
static gint ett_rrc_GANSSSatelliteInformation_r10 = -1;
static gint ett_rrc_GANSSSatelliteInformationList = -1;
static gint ett_rrc_GANSSSatelliteInformationList_va40ext = -1;
static gint ett_rrc_GANSSSatelliteInformationList_r10 = -1;
static gint ett_rrc_GANSSTimeModelsList = -1;
static gint ett_rrc_GANSSTimeModelsList_va40ext = -1;
static gint ett_rrc_GANSSTimeModelsList_r10 = -1;
static gint ett_rrc_GANSS_SAT_Info_Almanac_GLOkp = -1;
static gint ett_rrc_GANSS_SAT_Info_Almanac_Kp = -1;
static gint ett_rrc_GANSS_SAT_Info_Almanac_MIDIkp = -1;
static gint ett_rrc_GANSS_SAT_Info_Almanac_NAVkp = -1;
static gint ett_rrc_GANSS_SAT_Info_Almanac_REDkp = -1;
static gint ett_rrc_GANSS_SAT_Info_Almanac_SBASecef = -1;
static gint ett_rrc_GANSS_SAT_Info_Almanac_GLOkpList = -1;
static gint ett_rrc_GANSS_SAT_Info_Almanac_KpList = -1;
static gint ett_rrc_GANSS_SAT_Info_Almanac_MIDIkpList = -1;
static gint ett_rrc_GANSS_SAT_Info_Almanac_NAVkpList = -1;
static gint ett_rrc_GANSS_SAT_Info_Almanac_REDkpList = -1;
static gint ett_rrc_GANSS_SAT_Info_Almanac_SBASecefList = -1;
static gint ett_rrc_Ganss_Sat_Info_AddNav = -1;
static gint ett_rrc_Ganss_Sat_Info_Nav = -1;
static gint ett_rrc_Ganss_Sat_Info_AddNavList = -1;
static gint ett_rrc_Ganss_Sat_Info_NavList = -1;
static gint ett_rrc_GANSS_Storm_Flag = -1;
static gint ett_rrc_GLONASSclockModel = -1;
static gint ett_rrc_GPS_MeasurementParam = -1;
static gint ett_rrc_GPS_MeasurementParamList = -1;
static gint ett_rrc_GSM_CellGroup = -1;
static gint ett_rrc_T_followingARFCNs = -1;
static gint ett_rrc_SEQUENCE_SIZE_0_31_OF_BCCH_ARFCN = -1;
static gint ett_rrc_T_equallySpacedARFCNs = -1;
static gint ett_rrc_T_continuousRangeOfARFCNs = -1;
static gint ett_rrc_GSM_MeasuredResults = -1;
static gint ett_rrc_GSM_MeasuredResultsList = -1;
static gint ett_rrc_GPS_TOW_Assist = -1;
static gint ett_rrc_GPS_TOW_AssistList = -1;
static gint ett_rrc_GSM_PriorityInfo = -1;
static gint ett_rrc_GSM_PriorityInfoList = -1;
static gint ett_rrc_GSM_PriorityInfoList_vbxyext = -1;
static gint ett_rrc_HCS_CellReselectInformation_RSCP = -1;
static gint ett_rrc_HCS_CellReselectInformation_ECN0 = -1;
static gint ett_rrc_HCS_NeighbouringCellInformation_RSCP = -1;
static gint ett_rrc_HCS_NeighbouringCellInformation_ECN0 = -1;
static gint ett_rrc_HCS_ServingCellInformation = -1;
static gint ett_rrc_HorizontalVelocity = -1;
static gint ett_rrc_HorizontalWithVerticalVelocity = -1;
static gint ett_rrc_HorizontalVelocityWithUncertainty = -1;
static gint ett_rrc_HorizontalWithVerticalVelocityAndUncertainty = -1;
static gint ett_rrc_IdleIntervalInfo = -1;
static gint ett_rrc_InterFreqCell = -1;
static gint ett_rrc_InterFreqCell_LCR_r4 = -1;
static gint ett_rrc_InterFreqCellInfoList = -1;
static gint ett_rrc_InterFreqCellInfoList_r4 = -1;
static gint ett_rrc_InterFreqCellInfoList_r8 = -1;
static gint ett_rrc_InterFreqCellInfoList_r9 = -1;
static gint ett_rrc_InterFreqCellInfoList_r10 = -1;
static gint ett_rrc_InterFreqSIAcquisition = -1;
static gint ett_rrc_T_modeSpecificInfo_111 = -1;
static gint ett_rrc_T_fdd_150 = -1;
static gint ett_rrc_InterFreqCellInfoSI_List_RSCP = -1;
static gint ett_rrc_InterFreqCellInfoSI_List_ECN0 = -1;
static gint ett_rrc_InterFreqCellInfoSI_List_HCS_RSCP = -1;
static gint ett_rrc_InterFreqCellInfoSI_List_HCS_ECN0 = -1;
static gint ett_rrc_InterFreqCellInfoSI_List_RSCP_LCR = -1;
static gint ett_rrc_InterFreqCellInfoSI_List_ECN0_LCR = -1;
static gint ett_rrc_InterFreqCellInfoSI_List_HCS_RSCP_LCR = -1;
static gint ett_rrc_InterFreqCellInfoSI_List_HCS_ECN0_LCR = -1;
static gint ett_rrc_InterFreqCellList = -1;
static gint ett_rrc_InterFreqCellList_LCR_r4_ext = -1;
static gint ett_rrc_InterFreqCellMeasuredResultsList = -1;
static gint ett_rrc_InterFreqCellMeasuredResultsList_v920ext = -1;
static gint ett_rrc_InterFreqEvent = -1;
static gint ett_rrc_InterFreqEvent_r6 = -1;
static gint ett_rrc_InterFreqEvent_r10 = -1;
static gint ett_rrc_InterFreqEvent_r11 = -1;
static gint ett_rrc_InterFreqEventList = -1;
static gint ett_rrc_InterFreqEventList_r6 = -1;
static gint ett_rrc_InterFreqEventList_r10 = -1;
static gint ett_rrc_InterFreqEventList_r11 = -1;
static gint ett_rrc_InterFrequencyMeasuredResultsList_v590ext = -1;
static gint ett_rrc_Inter_FreqEventCriteria_v590ext = -1;
static gint ett_rrc_Inter_FreqEventCriteriaList_v590ext = -1;
static gint ett_rrc_Intra_FreqEventCriteriaList_v590ext = -1;
static gint ett_rrc_IntraFrequencyMeasuredResultsList_v590ext = -1;
static gint ett_rrc_InterFreqEventResults = -1;
static gint ett_rrc_InterFreqEventResults_va40ext = -1;
static gint ett_rrc_InterFreqEventResults_LCR_r4_ext = -1;
static gint ett_rrc_InterFreqMeasQuantity = -1;
static gint ett_rrc_T_reportingCriteria = -1;
static gint ett_rrc_T_intraFreqReportingCriteria = -1;
static gint ett_rrc_T_interFreqReportingCriteria = -1;
static gint ett_rrc_T_modeSpecificInfo_112 = -1;
static gint ett_rrc_T_fdd_151 = -1;
static gint ett_rrc_T_tdd_100 = -1;
static gint ett_rrc_InterFreqMeasuredResults = -1;
static gint ett_rrc_InterFreqMeasuredResults_v920ext = -1;
static gint ett_rrc_InterFreqMeasuredResultsList = -1;
static gint ett_rrc_InterFreqMeasuredResultsList_v920ext = -1;
static gint ett_rrc_InterFreqMeasurementSysInfo_RSCP = -1;
static gint ett_rrc_InterFreqMeasurementSysInfo_ECN0 = -1;
static gint ett_rrc_InterFreqMeasurementSysInfo_HCS_RSCP = -1;
static gint ett_rrc_InterFreqMeasurementSysInfo_HCS_ECN0 = -1;
static gint ett_rrc_InterFreqMeasurementSysInfo_RSCP_LCR_r4 = -1;
static gint ett_rrc_InterFreqMeasurementSysInfo_ECN0_LCR_r4 = -1;
static gint ett_rrc_InterFreqMeasurementSysInfo_HCS_RSCP_LCR_r4 = -1;
static gint ett_rrc_InterFreqMeasurementSysInfo_HCS_ECN0_LCR_r4 = -1;
static gint ett_rrc_InterFreqRACHRepCellsList = -1;
static gint ett_rrc_Dummy_InterFreqRACHReportingInfo = -1;
static gint ett_rrc_T_modeSpecificInfo_113 = -1;
static gint ett_rrc_T_fdd_152 = -1;
static gint ett_rrc_T_tdd_101 = -1;
static gint ett_rrc_InterFreqRACHReportingInfo = -1;
static gint ett_rrc_T_modeSpecificInfo_114 = -1;
static gint ett_rrc_T_fdd_153 = -1;
static gint ett_rrc_T_tdd_102 = -1;
static gint ett_rrc_InterFreqReportCriteria = -1;
static gint ett_rrc_InterFreqReportCriteria_r4 = -1;
static gint ett_rrc_InterFreqReportCriteria_r6 = -1;
static gint ett_rrc_InterFreqReportCriteria_r7 = -1;
static gint ett_rrc_InterFreqReportCriteria_r9 = -1;
static gint ett_rrc_InterFreqReportCriteria_r10 = -1;
static gint ett_rrc_InterFreqReportCriteria_r11 = -1;
static gint ett_rrc_InterFreqReportingCriteria = -1;
static gint ett_rrc_InterFreqReportingCriteria_r6 = -1;
static gint ett_rrc_InterFreqReportingCriteria_r10 = -1;
static gint ett_rrc_InterFreqReportingCriteria_r11 = -1;
static gint ett_rrc_InterFreqReportingQuantity = -1;
static gint ett_rrc_InterFreqRepQuantityRACH_TDDList = -1;
static gint ett_rrc_InterFrequencyMeasurement = -1;
static gint ett_rrc_InterFrequencyMeasurement_r4 = -1;
static gint ett_rrc_InterFrequencyMeasurement_r6 = -1;
static gint ett_rrc_InterFrequencyMeasurement_r7 = -1;
static gint ett_rrc_InterFrequencyMeasurement_r8 = -1;
static gint ett_rrc_InterFrequencyMeasurement_r9 = -1;
static gint ett_rrc_InterFrequencyMeasurement_r10 = -1;
static gint ett_rrc_InterFrequencyMeasurement_r11 = -1;
static gint ett_rrc_InterRAT_TargetCellDescription = -1;
static gint ett_rrc_T_technologySpecificInfo = -1;
static gint ett_rrc_T_gsm_08 = -1;
static gint ett_rrc_InterRATCellInfoList = -1;
static gint ett_rrc_InterRATCellInfoList_B = -1;
static gint ett_rrc_InterRATCellInfoList_r4 = -1;
static gint ett_rrc_InterRATCellInfoList_r6 = -1;
static gint ett_rrc_InterRATEvent = -1;
static gint ett_rrc_InterRATEventList = -1;
static gint ett_rrc_InterRATEventResults = -1;
static gint ett_rrc_InterRATInfo_r6 = -1;
static gint ett_rrc_InterRATInfo_v860ext = -1;
static gint ett_rrc_T_eutra_01 = -1;
static gint ett_rrc_InterRATMeasQuantity = -1;
static gint ett_rrc_T_ratSpecificInfo = -1;
static gint ett_rrc_T_gsm_09 = -1;
static gint ett_rrc_T_is_2000 = -1;
static gint ett_rrc_InterRATMeasQuantity_r8 = -1;
static gint ett_rrc_T_ratSpecificInfo_01 = -1;
static gint ett_rrc_T_gsm_10 = -1;
static gint ett_rrc_T_is_2000_01 = -1;
static gint ett_rrc_T_e_UTRA_01 = -1;
static gint ett_rrc_InterRATMeasuredResults = -1;
static gint ett_rrc_InterRATMeasuredResultsList = -1;
static gint ett_rrc_InterRATMeasurement = -1;
static gint ett_rrc_InterRATMeasurement_r4 = -1;
static gint ett_rrc_InterRATMeasurement_r6 = -1;
static gint ett_rrc_InterRATMeasurement_r8 = -1;
static gint ett_rrc_T_interRATMeasurementObjects = -1;
static gint ett_rrc_InterRATMeasurement_r9 = -1;
static gint ett_rrc_T_interRATMeasurementObjects_01 = -1;
static gint ett_rrc_InterRATMeasurement_r11 = -1;
static gint ett_rrc_T_interRATMeasurementObjects_02 = -1;
static gint ett_rrc_InterRATMeasurementSysInfo = -1;
static gint ett_rrc_InterRATMeasurementSysInfo_B = -1;
static gint ett_rrc_InterRATReportCriteria = -1;
static gint ett_rrc_InterRATReportingCriteria = -1;
static gint ett_rrc_InterRATReportingQuantity = -1;
static gint ett_rrc_T_ratSpecificInfo_02 = -1;
static gint ett_rrc_T_gsm_11 = -1;
static gint ett_rrc_InterRATReportingQuantity_r8 = -1;
static gint ett_rrc_T_ratSpecificInfo_03 = -1;
static gint ett_rrc_T_gsm_12 = -1;
static gint ett_rrc_T_eutra_02 = -1;
static gint ett_rrc_IntraFreqCellInfoList = -1;
static gint ett_rrc_IntraFreqCellInfoList_r4 = -1;
static gint ett_rrc_IntraFreqCellInfoListInfoOnSecULFreq = -1;
static gint ett_rrc_IntraFreqCellInfoListOnSecULFreq = -1;
static gint ett_rrc_IntraFreqCellInfoList_r9 = -1;
static gint ett_rrc_IntraFreqCellInfoList_r10 = -1;
static gint ett_rrc_IntraFreqSIAcquisitionInfo = -1;
static gint ett_rrc_IntraFreqSIAcquisition = -1;
static gint ett_rrc_T_modeSpecificInfo_115 = -1;
static gint ett_rrc_T_fdd_154 = -1;
static gint ett_rrc_IntraFreqCellInfoSI_List_RSCP = -1;
static gint ett_rrc_IntraFreqCellInfoSI_List_ECN0 = -1;
static gint ett_rrc_IntraFreqCellInfoSI_List_HCS_RSCP = -1;
static gint ett_rrc_IntraFreqCellInfoSI_List_HCS_ECN0 = -1;
static gint ett_rrc_IntraFreqCellInfoSI_List_RSCP_LCR_r4 = -1;
static gint ett_rrc_IntraFreqCellInfoSI_List_ECN0_LCR_r4 = -1;
static gint ett_rrc_IntraFreqCellInfoSI_List_HCS_RSCP_LCR_r4 = -1;
static gint ett_rrc_IntraFreqCellInfoSI_List_HCS_ECN0_LCR_r4 = -1;
static gint ett_rrc_IntraFreqEvent = -1;
static gint ett_rrc_IntraFreqEvent_r4 = -1;
static gint ett_rrc_IntraFreqEvent_LCR_r4 = -1;
static gint ett_rrc_IntraFreqEvent_r6 = -1;
static gint ett_rrc_IntraFreqEvent_r7 = -1;
static gint ett_rrc_IntraFreqEventOnSecULFreq = -1;
static gint ett_rrc_IntraFreqEvent_1d_r5 = -1;
static gint ett_rrc_IntraFreqEventCriteria = -1;
static gint ett_rrc_IntraFreqEventCriteria_r4 = -1;
static gint ett_rrc_IntraFreqEventCriteria_LCR_r4 = -1;
static gint ett_rrc_IntraFreqEventCriteria_r6 = -1;
static gint ett_rrc_IntraFreqEventCriteria_r7 = -1;
static gint ett_rrc_IntraFreqEventCriteriaOnSecULFreq = -1;
static gint ett_rrc_IntraFreqEventCriteriaList = -1;
static gint ett_rrc_IntraFreqEventCriteriaList_r4 = -1;
static gint ett_rrc_IntraFreqEventCriteriaList_LCR_r4 = -1;
static gint ett_rrc_IntraFreqEventCriteriaList_r6 = -1;
static gint ett_rrc_IntraFreqEventCriteriaList_r7 = -1;
static gint ett_rrc_IntraFreqEventCriteriaListOnSecULFreq = -1;
static gint ett_rrc_SEQUENCE_SIZE_1_maxMeasEventOnSecULFreq_OF_IntraFreqEventCriteriaOnSecULFreq = -1;
static gint ett_rrc_IntraFreqEventResults = -1;
static gint ett_rrc_IntraFreqEventResultsOnSecUlFreq = -1;
static gint ett_rrc_IntraFreqMeasQuantity = -1;
static gint ett_rrc_T_modeSpecificInfo_116 = -1;
static gint ett_rrc_T_fdd_155 = -1;
static gint ett_rrc_T_tdd_103 = -1;
static gint ett_rrc_IntraFreqMeasQuantity_TDDList = -1;
static gint ett_rrc_IntraFreqMeasQuantity_TDD_sib3List = -1;
static gint ett_rrc_IntraFreqMeasuredResultsList = -1;
static gint ett_rrc_IntraFreqMeasuredResultsList_v920ext = -1;
static gint ett_rrc_IntraFreqMeasurementSysInfo_RSCP = -1;
static gint ett_rrc_IntraFreqMeasurementSysInfo_ECN0 = -1;
static gint ett_rrc_IntraFreqMeasurementSysInfo_HCS_RSCP = -1;
static gint ett_rrc_IntraFreqMeasurementSysInfo_HCS_ECN0 = -1;
static gint ett_rrc_IntraFreqMeasurementSysInfo_RSCP_LCR_r4 = -1;
static gint ett_rrc_IntraFreqMeasurementSysInfo_ECN0_LCR_r4 = -1;
static gint ett_rrc_IntraFreqMeasurementSysInfo_HCS_RSCP_LCR_r4 = -1;
static gint ett_rrc_IntraFreqMeasurementSysInfo_HCS_ECN0_LCR_r4 = -1;
static gint ett_rrc_IntraFreqReportCriteria = -1;
static gint ett_rrc_IntraFreqReportCriteria_r4 = -1;
static gint ett_rrc_IntraFreqReportCriteria_r6 = -1;
static gint ett_rrc_IntraFreqReportCriteria_r7 = -1;
static gint ett_rrc_IntraFreqReportCriteria_r9 = -1;
static gint ett_rrc_IntraFreqReportCriteria_r11 = -1;
static gint ett_rrc_IntraFreqReportingCriteria = -1;
static gint ett_rrc_IntraFreqReportingCriteria_r4 = -1;
static gint ett_rrc_IntraFreqReportingCriteria_LCR_r4 = -1;
static gint ett_rrc_IntraFreqReportingCriteria_r6 = -1;
static gint ett_rrc_IntraFreqReportingCriteria_r7 = -1;
static gint ett_rrc_IntraFreqReportingCriteria_r9 = -1;
static gint ett_rrc_IntraFreqReportingCriteria_r11 = -1;
static gint ett_rrc_IntraFreqReportingCriteria_1b_r5 = -1;
static gint ett_rrc_IntraFreqReportingQuantity = -1;
static gint ett_rrc_IntraFreqReportingQuantityForRACH = -1;
static gint ett_rrc_T_modeSpecificInfo_117 = -1;
static gint ett_rrc_T_fdd_156 = -1;
static gint ett_rrc_T_tdd_104 = -1;
static gint ett_rrc_IntraFreqRepQuantityRACH_TDDList = -1;
static gint ett_rrc_IntraFrequencyMeasurement = -1;
static gint ett_rrc_IntraFrequencyMeasurement_r4 = -1;
static gint ett_rrc_IntraFrequencyMeasurement_r6 = -1;
static gint ett_rrc_IntraFrequencyMeasurement_r7 = -1;
static gint ett_rrc_IntraFrequencyMeasurement_r9 = -1;
static gint ett_rrc_IntraFrequencyMeasurement_r10 = -1;
static gint ett_rrc_IntraFrequencyMeasurement_r11 = -1;
static gint ett_rrc_IntraFrequencyMeasuredResultsListOnSecULFreq = -1;
static gint ett_rrc_IntraUTRAANR = -1;
static gint ett_rrc_LoggedANRConfigurationInfo = -1;
static gint ett_rrc_LoggedANRReportInfoList = -1;
static gint ett_rrc_LoggedANRReportInfo = -1;
static gint ett_rrc_LoggedCellInfo = -1;
static gint ett_rrc_LoggedConnectionEstablishmentFailureInfo_FDD = -1;
static gint ett_rrc_LoggedConnectionEstablishmentFailureInfo_TDD128 = -1;
static gint ett_rrc_LoggedConnectionEstablishmentFailureAccessInfo_FDD = -1;
static gint ett_rrc_LoggedConnectionEstablishmentFailureAccessInfo_TDD = -1;
static gint ett_rrc_LoggedConnectionEstablishmentFailureLocationInfo = -1;
static gint ett_rrc_T_location = -1;
static gint ett_rrc_LoggedConnectionEstablishmentFailureReport = -1;
static gint ett_rrc_T_modeSpecificInfo_118 = -1;
static gint ett_rrc_T_fdd_157 = -1;
static gint ett_rrc_T_tdd_105 = -1;
static gint ett_rrc_LoggedEUTRACellInfo = -1;
static gint ett_rrc_LoggedGSMCellInfo = -1;
static gint ett_rrc_LoggedMeasurementsConfigurationInfo = -1;
static gint ett_rrc_T_areaConfiguration = -1;
static gint ett_rrc_SEQUENCE_SIZE_1_32_OF_CellIdentity = -1;
static gint ett_rrc_SEQUENCE_SIZE_1_8_OF_LAI = -1;
static gint ett_rrc_SEQUENCE_SIZE_1_8_OF_RAI = -1;
static gint ett_rrc_LoggedMeasurementsConfigurationInfo_r11 = -1;
static gint ett_rrc_SEQUENCE_SIZE_1_15_OF_PLMN_Identity = -1;
static gint ett_rrc_T_areaConfiguration_01 = -1;
static gint ett_rrc_SEQUENCE_SIZE_1_32_OF_CellIDListItem = -1;
static gint ett_rrc_LoggedMeasInterfreqList_FDD = -1;
static gint ett_rrc_LoggedMeasInterfreqInfo_FDD = -1;
static gint ett_rrc_SEQUENCE_SIZE_1_maxnumLoggedMeas_OF_LoggedMeasNeighbourInfo_FDD = -1;
static gint ett_rrc_LoggedMeasInterfreqInfo_TDD128 = -1;
static gint ett_rrc_LoggedMeasInterRATNeighbourMeas = -1;
static gint ett_rrc_LoggedMeasIntrafreqNeighbourList_FDD = -1;
static gint ett_rrc_LoggedMeasNeighbourInfo_FDD = -1;
static gint ett_rrc_LoggedMeasIntrafreqNeighbourMeasList_TDD128 = -1;
static gint ett_rrc_LoggedMeasNeighbourMeas_TDD128 = -1;
static gint ett_rrc_LoggedMeasGSMNeighbourCellsList = -1;
static gint ett_rrc_LoggedMeasGSMNeighbourCellsinfo = -1;
static gint ett_rrc_LoggedMeasEUTRAFreqList = -1;
static gint ett_rrc_LoggedMeasEUTRAFreqInfo = -1;
static gint ett_rrc_SEQUENCE_SIZE_1_maxnumLoggedMeas_OF_LoggedMeasEUTRAFreqNeighbourInfo = -1;
static gint ett_rrc_LoggedMeasEUTRAFreqNeighbourInfo = -1;
static gint ett_rrc_LoggedMeasLocationInfo = -1;
static gint ett_rrc_LoggedMeasLocationInfo_vbxyext = -1;
static gint ett_rrc_LoggedMeasInfo_FDD = -1;
static gint ett_rrc_LoggedMeasInfo_FDD_vbxyext = -1;
static gint ett_rrc_LoggedMeasInfo_TDD128 = -1;
static gint ett_rrc_LoggedMeasInfo_TDD128_vbxyext = -1;
static gint ett_rrc_LoggedMeasInterfreqList_TDD128 = -1;
static gint ett_rrc_LoggedMeasInterfreqNeighbourList_TDD128 = -1;
static gint ett_rrc_LoggedMeasReport = -1;
static gint ett_rrc_LoggedMeasReport_vbxyext = -1;
static gint ett_rrc_LoggedMeasInfoList_FDD = -1;
static gint ett_rrc_LoggedMeasInfoList_FDD_vbxyext = -1;
static gint ett_rrc_LoggedMeasInfoList_TDD128 = -1;
static gint ett_rrc_LoggedMeasInfoList_TDD128_vbxyext = -1;
static gint ett_rrc_LoggedMeasInfoList = -1;
static gint ett_rrc_T_modeSpecificInfo_119 = -1;
static gint ett_rrc_T_fdd_158 = -1;
static gint ett_rrc_T_tdd_106 = -1;
static gint ett_rrc_LoggedMeasInfoList_vbxyext = -1;
static gint ett_rrc_T_modeSpecificInfo_120 = -1;
static gint ett_rrc_T_fdd_159 = -1;
static gint ett_rrc_T_tdd_107 = -1;
static gint ett_rrc_LoggedMeasServingCellMeas_FDD = -1;
static gint ett_rrc_LoggedMeasServingCellMeas_FDD_vbxyext = -1;
static gint ett_rrc_LoggedMeasServingCellMeas_TDD128 = -1;
static gint ett_rrc_LoggedMeasServingCellMeas_TDD128_vbxyext = -1;
static gint ett_rrc_LoggedUTRACellInfo = -1;
static gint ett_rrc_LoggingAbsoluteThreshold = -1;
static gint ett_rrc_KeplerianParameters = -1;
static gint ett_rrc_MeasuredResults = -1;
static gint ett_rrc_MeasuredResultsOnSecUlFreq = -1;
static gint ett_rrc_MeasuredResults_v390ext = -1;
static gint ett_rrc_MeasuredResults_v590ext = -1;
static gint ett_rrc_MeasuredResults_v770ext = -1;
static gint ett_rrc_MeasuredResults_v860ext = -1;
static gint ett_rrc_MeasuredResultsList_v920ext = -1;
static gint ett_rrc_MeasuredResultsListOnSecUlFreq = -1;
static gint ett_rrc_MeasuredResults_v920ext = -1;
static gint ett_rrc_MeasuredResults_LCR_r4 = -1;
static gint ett_rrc_MeasuredResultsList = -1;
static gint ett_rrc_MeasuredResultsList_LCR_r4_ext = -1;
static gint ett_rrc_MeasuredResultsList_v770xet = -1;
static gint ett_rrc_MeasuredResultsList_v770xet_item = -1;
static gint ett_rrc_MeasuredResultsList_v860ext = -1;
static gint ett_rrc_MeasuredResultsList_v860ext_item = -1;
static gint ett_rrc_MeasuredResultsOnRACH = -1;
static gint ett_rrc_T_currentCell = -1;
static gint ett_rrc_T_modeSpecificInfo_121 = -1;
static gint ett_rrc_T_fdd_160 = -1;
static gint ett_rrc_T_measurementQuantity_01 = -1;
static gint ett_rrc_T_tdd_108 = -1;
static gint ett_rrc_MeasuredResultsOnRACH_v7g0ext = -1;
static gint ett_rrc_MonitoredCellRACH_List_v7g0ext = -1;
static gint ett_rrc_MeasuredResultsOnRACHinterFreq = -1;
static gint ett_rrc_MeasuredResultsOnRACHeutraFreq = -1;
static gint ett_rrc_MeasurementCommand = -1;
static gint ett_rrc_T_modify_01 = -1;
static gint ett_rrc_MeasurementCommand_r4 = -1;
static gint ett_rrc_T_modify_02 = -1;
static gint ett_rrc_MeasurementCommand_r6 = -1;
static gint ett_rrc_T_modify_03 = -1;
static gint ett_rrc_MeasurementCommand_r7 = -1;
static gint ett_rrc_T_modify_04 = -1;
static gint ett_rrc_MeasurementCommand_r8 = -1;
static gint ett_rrc_T_modify_05 = -1;
static gint ett_rrc_MeasurementCommand_r9 = -1;
static gint ett_rrc_T_modify_06 = -1;
static gint ett_rrc_MeasurementCommand_r10 = -1;
static gint ett_rrc_T_modify_07 = -1;
static gint ett_rrc_MeasurementCommand_r11 = -1;
static gint ett_rrc_T_modify_08 = -1;
static gint ett_rrc_MeasurementControlSysInfo = -1;
static gint ett_rrc_T_use_of_HCS = -1;
static gint ett_rrc_T_hcs_not_used = -1;
static gint ett_rrc_T_cellSelectQualityMeasure_01 = -1;
static gint ett_rrc_T_cpich_RSCP = -1;
static gint ett_rrc_T_cpich_Ec_N0_01 = -1;
static gint ett_rrc_T_hcs_used = -1;
static gint ett_rrc_T_cellSelectQualityMeasure_02 = -1;
static gint ett_rrc_T_cpich_RSCP_01 = -1;
static gint ett_rrc_T_cpich_Ec_N0_02 = -1;
static gint ett_rrc_MeasurementControlSysInfoExtension = -1;
static gint ett_rrc_T_use_of_HCS_01 = -1;
static gint ett_rrc_T_hcs_not_used_01 = -1;
static gint ett_rrc_T_cellSelectQualityMeasure_03 = -1;
static gint ett_rrc_T_cpich_RSCP_02 = -1;
static gint ett_rrc_T_cpich_Ec_N0_03 = -1;
static gint ett_rrc_T_hcs_used_01 = -1;
static gint ett_rrc_T_cellSelectQualityMeasure_04 = -1;
static gint ett_rrc_T_cpich_RSCP_03 = -1;
static gint ett_rrc_T_cpich_Ec_N0_04 = -1;
static gint ett_rrc_MeasurementControlSysInfoExtensionAddon_r5 = -1;
static gint ett_rrc_SEQUENCE_SIZE_1_maxCellMeas_OF_CellSelectReselectInfo_v590ext = -1;
static gint ett_rrc_MeasurementControlSysInfoExtension_LCR_r4 = -1;
static gint ett_rrc_T_use_of_HCS_02 = -1;
static gint ett_rrc_T_hcs_not_used_02 = -1;
static gint ett_rrc_T_cellSelectQualityMeasure_05 = -1;
static gint ett_rrc_T_cpich_RSCP_04 = -1;
static gint ett_rrc_T_cpich_Ec_N0_05 = -1;
static gint ett_rrc_T_hcs_used_02 = -1;
static gint ett_rrc_T_cellSelectQualityMeasure_06 = -1;
static gint ett_rrc_T_cpich_RSCP_05 = -1;
static gint ett_rrc_T_cpich_Ec_N0_06 = -1;
static gint ett_rrc_MeasurementControlSysInfo_LCR_r4_ext = -1;
static gint ett_rrc_T_use_of_HCS_03 = -1;
static gint ett_rrc_T_hcs_not_used_03 = -1;
static gint ett_rrc_T_cellSelectQualityMeasure_07 = -1;
static gint ett_rrc_T_cpich_RSCP_06 = -1;
static gint ett_rrc_T_cpich_Ec_N0_07 = -1;
static gint ett_rrc_T_hcs_used_03 = -1;
static gint ett_rrc_T_cellSelectQualityMeasure_08 = -1;
static gint ett_rrc_T_cpich_RSCP_07 = -1;
static gint ett_rrc_T_cpich_Ec_N0_08 = -1;
static gint ett_rrc_MeasurementOccasionPatternParameter = -1;
static gint ett_rrc_MeasurementReportingMode = -1;
static gint ett_rrc_MeasurementType = -1;
static gint ett_rrc_MeasurementType_r4 = -1;
static gint ett_rrc_MeasurementType_r6 = -1;
static gint ett_rrc_MeasurementType_r7 = -1;
static gint ett_rrc_MeasurementType_r8 = -1;
static gint ett_rrc_MeasurementType_r9 = -1;
static gint ett_rrc_MeasurementType_r10 = -1;
static gint ett_rrc_MeasurementType_r11 = -1;
static gint ett_rrc_MeasurementValidity = -1;
static gint ett_rrc_MonitoredCellRACH_List = -1;
static gint ett_rrc_MonitoredCellRACH_Result = -1;
static gint ett_rrc_T_modeSpecificInfo_122 = -1;
static gint ett_rrc_T_fdd_161 = -1;
static gint ett_rrc_T_measurementQuantity_02 = -1;
static gint ett_rrc_T_tdd_109 = -1;
static gint ett_rrc_MultipleEUTRAFrequencyBandIndicatorList = -1;
static gint ett_rrc_MultipleEUTRAFrequencyBandInfo = -1;
static gint ett_rrc_MultipleEUTRAFrequencyInfoList = -1;
static gint ett_rrc_MultipleFrequencyBandIndicatorListFDD = -1;
static gint ett_rrc_MultipleFrequencyBandInfo = -1;
static gint ett_rrc_MultipleFrequencyInfoListFDD = -1;
static gint ett_rrc_N_CR_T_CRMaxHyst = -1;
static gint ett_rrc_NAVclockModel = -1;
static gint ett_rrc_NavigationModelSatInfo = -1;
static gint ett_rrc_NavigationModelSatInfoList = -1;
static gint ett_rrc_NavModel_CNAVKeplerianSet = -1;
static gint ett_rrc_NavModel_GLONASSecef = -1;
static gint ett_rrc_NavModel_NAVKeplerianSet = -1;
static gint ett_rrc_NavModel_SBASecef = -1;
static gint ett_rrc_EphemerisParameter = -1;
static gint ett_rrc_Neighbour = -1;
static gint ett_rrc_T_modeSpecificInfo_123 = -1;
static gint ett_rrc_T_fdd_162 = -1;
static gint ett_rrc_T_tdd_110 = -1;
static gint ett_rrc_Neighbour_v390ext = -1;
static gint ett_rrc_T_modeSpecificInfo_124 = -1;
static gint ett_rrc_T_fdd_163 = -1;
static gint ett_rrc_NeighbourList = -1;
static gint ett_rrc_Neighbour_TDD_r7 = -1;
static gint ett_rrc_NeighbourList_TDD_r7 = -1;
static gint ett_rrc_NeighbourList_v390ext = -1;
static gint ett_rrc_NeighbourQuality = -1;
static gint ett_rrc_NewInterFreqCell = -1;
static gint ett_rrc_NewInterFreqCell_r4 = -1;
static gint ett_rrc_NewInterFreqCell_v7b0ext = -1;
static gint ett_rrc_NewInterFreqCell_LCR_v8a0ext = -1;
static gint ett_rrc_NewInterFreqCell_r8 = -1;
static gint ett_rrc_T_modeSpecificInfo_125 = -1;
static gint ett_rrc_T_tdd128_62 = -1;
static gint ett_rrc_NewInterFreqCell_r9 = -1;
static gint ett_rrc_T_modeSpecificInfo_126 = -1;
static gint ett_rrc_T_tdd128_63 = -1;
static gint ett_rrc_NewInterFreqCell_r10 = -1;
static gint ett_rrc_T_modeSpecificInfo_127 = -1;
static gint ett_rrc_T_tdd128_64 = -1;
static gint ett_rrc_NewInterFreqCellList = -1;
static gint ett_rrc_NewInterFreqCellList_r4 = -1;
static gint ett_rrc_NewInterFreqCellList_v7b0ext = -1;
static gint ett_rrc_NewInterFreqCellList_LCR_v8a0ext = -1;
static gint ett_rrc_NewInterFreqCellList_r8 = -1;
static gint ett_rrc_NewInterFreqCellList_r9 = -1;
static gint ett_rrc_NewInterFreqCellList_r10 = -1;
static gint ett_rrc_NewInterFreqCellSI_RSCP = -1;
static gint ett_rrc_NewInterFreqCellSI_ECN0 = -1;
static gint ett_rrc_NewInterFreqCellSI_HCS_RSCP = -1;
static gint ett_rrc_NewInterFreqCellSI_HCS_ECN0 = -1;
static gint ett_rrc_NewInterFreqCellSI_RSCP_LCR_r4 = -1;
static gint ett_rrc_NewInterFreqCellSI_ECN0_LCR_r4 = -1;
static gint ett_rrc_NewInterFreqCellSI_HCS_RSCP_LCR_r4 = -1;
static gint ett_rrc_NewInterFreqCellSI_HCS_ECN0_LCR_r4 = -1;
static gint ett_rrc_NewInterFreqCellSI_List_ECN0 = -1;
static gint ett_rrc_NewInterFreqCellSI_List_HCS_RSCP = -1;
static gint ett_rrc_NewInterFreqCellSI_List_HCS_ECN0 = -1;
static gint ett_rrc_NewInterFreqCellSI_List_RSCP = -1;
static gint ett_rrc_NewInterFreqCellSI_List_ECN0_LCR_r4 = -1;
static gint ett_rrc_NewInterFreqCellSI_List_HCS_RSCP_LCR_r4 = -1;
static gint ett_rrc_NewInterFreqCellSI_List_HCS_ECN0_LCR_r4 = -1;
static gint ett_rrc_NewInterFreqCellSI_List_RSCP_LCR_r4 = -1;
static gint ett_rrc_NewInterRATCell = -1;
static gint ett_rrc_T_technologySpecificInfo_01 = -1;
static gint ett_rrc_T_gsm_13 = -1;
static gint ett_rrc_T_is_2000_02 = -1;
static gint ett_rrc_NewInterRATCell_B = -1;
static gint ett_rrc_T_technologySpecificInfo_02 = -1;
static gint ett_rrc_T_gsm_14 = -1;
static gint ett_rrc_T_is_2000_03 = -1;
static gint ett_rrc_NewInterRATCellList = -1;
static gint ett_rrc_NewInterRATCellList_B = -1;
static gint ett_rrc_NewIntraFreqCell = -1;
static gint ett_rrc_NewIntraFreqCell_r4 = -1;
static gint ett_rrc_NewIntraFreqCell_r9 = -1;
static gint ett_rrc_NewIntraFreqCell_r10 = -1;
static gint ett_rrc_T_modeSpecificInfo_128 = -1;
static gint ett_rrc_T_tdd128_65 = -1;
static gint ett_rrc_NewIntraFreqCellOnSecULFreq = -1;
static gint ett_rrc_NewIntraFreqCellList = -1;
static gint ett_rrc_NewIntraFreqCellList_r4 = -1;
static gint ett_rrc_NewIntraFreqCellList_r9 = -1;
static gint ett_rrc_NewIntraFreqCellList_r10 = -1;
static gint ett_rrc_NewIntraFreqCell_LCR_v8a0ext = -1;
static gint ett_rrc_NewIntraFreqCellList_LCR_v8a0ext = -1;
static gint ett_rrc_NewIntraFreqCellListOnSecULFreq = -1;
static gint ett_rrc_NewIntraFreqCellSI_RSCP = -1;
static gint ett_rrc_NewIntraFreqCellSI_ECN0 = -1;
static gint ett_rrc_NewIntraFreqCellSI_HCS_RSCP = -1;
static gint ett_rrc_NewIntraFreqCellSI_HCS_ECN0 = -1;
static gint ett_rrc_NewIntraFreqCellSI_RSCP_LCR_r4 = -1;
static gint ett_rrc_NewIntraFreqCellSI_ECN0_LCR_r4 = -1;
static gint ett_rrc_NewIntraFreqCellSI_HCS_RSCP_LCR_r4 = -1;
static gint ett_rrc_NewIntraFreqCellSI_HCS_ECN0_LCR_r4 = -1;
static gint ett_rrc_NewIntraFreqCellSI_List_RSCP = -1;
static gint ett_rrc_NewIntraFreqCellSI_List_ECN0 = -1;
static gint ett_rrc_NewIntraFreqCellSI_List_HCS_RSCP = -1;
static gint ett_rrc_NewIntraFreqCellSI_List_HCS_ECN0 = -1;
static gint ett_rrc_NewIntraFreqCellSI_List_RSCP_LCR_r4 = -1;
static gint ett_rrc_NewIntraFreqCellSI_List_ECN0_LCR_r4 = -1;
static gint ett_rrc_NewIntraFreqCellSI_List_HCS_RSCP_LCR_r4 = -1;
static gint ett_rrc_NewIntraFreqCellSI_List_HCS_ECN0_LCR_r4 = -1;
static gint ett_rrc_NonUsedFreqParameter = -1;
static gint ett_rrc_NonUsedFreqParameter_r6 = -1;
static gint ett_rrc_NonUsedFreqParameter_r10 = -1;
static gint ett_rrc_NonUsedFreqParameter_r11 = -1;
static gint ett_rrc_NonUsedFreq2aParameter_r10 = -1;
static gint ett_rrc_NonUsedFreq2aParameter_r11 = -1;
static gint ett_rrc_NonUsedFreqParameterList = -1;
static gint ett_rrc_NonUsedFreqParameterList_r6 = -1;
static gint ett_rrc_NonUsedFreqParameterList_r10 = -1;
static gint ett_rrc_NonUsedFreqParameterList_r11 = -1;
static gint ett_rrc_NonUsedFreq2aParameterList_r10 = -1;
static gint ett_rrc_NonUsedFreq2aParameterList_r11 = -1;
static gint ett_rrc_NonUsedFreqWList_r6 = -1;
static gint ett_rrc_PenaltyTime_RSCP = -1;
static gint ett_rrc_PenaltyTime_ECN0 = -1;
static gint ett_rrc_PeriodicalReportingCriteria = -1;
static gint ett_rrc_PeriodicReportingInfo_1b = -1;
static gint ett_rrc_PeriodicalWithReportingCellStatus = -1;
static gint ett_rrc_PeriodicalWithReportingCellStatus_r10 = -1;
static gint ett_rrc_PeriodicalWithReportingCellStatusOnSecULFreq = -1;
static gint ett_rrc_PLMNIdentitiesOfNeighbourCells = -1;
static gint ett_rrc_PLMNIdentitiesOfNeighbourCells_v860ext = -1;
static gint ett_rrc_PLMNsOfInterFreqCellsList = -1;
static gint ett_rrc_PLMNsOfInterFreqCellsList_item = -1;
static gint ett_rrc_MultiplePLMNsOfInterFreqCellsList = -1;
static gint ett_rrc_MultiplePLMNsOfInterFreqCellsList_item = -1;
static gint ett_rrc_SEQUENCE_SIZE_1_6_OF_PLMN_IdentityWithOptionalMCC_r6 = -1;
static gint ett_rrc_PLMNsOfIntraFreqCellsList = -1;
static gint ett_rrc_PLMNsOfIntraFreqCellsList_item = -1;
static gint ett_rrc_MultiplePLMNsOfIntraFreqCellsList = -1;
static gint ett_rrc_MultiplePLMNsOfIntraFreqCellsList_item = -1;
static gint ett_rrc_PLMNsOfInterRATCellsList = -1;
static gint ett_rrc_PLMNsOfInterRATCellsList_item = -1;
static gint ett_rrc_PositionEstimate = -1;
static gint ett_rrc_QualityEventResults = -1;
static gint ett_rrc_QualityMeasuredResults = -1;
static gint ett_rrc_T_modeSpecificInfo_129 = -1;
static gint ett_rrc_T_tdd_111 = -1;
static gint ett_rrc_QualityMeasurement = -1;
static gint ett_rrc_QualityReportCriteria = -1;
static gint ett_rrc_QualityReportingCriteria = -1;
static gint ett_rrc_QualityReportingCriteriaSingle = -1;
static gint ett_rrc_QualityReportingQuantity = -1;
static gint ett_rrc_T_modeSpecificInfo_130 = -1;
static gint ett_rrc_T_tdd_112 = -1;
static gint ett_rrc_ReferenceCellPosition = -1;
static gint ett_rrc_ReferenceLocation = -1;
static gint ett_rrc_ReferenceLocationGANSS = -1;
static gint ett_rrc_ReferenceTimeDifferenceToCell = -1;
static gint ett_rrc_RemovedInterFreqCellList = -1;
static gint ett_rrc_SEQUENCE_SIZE_1_maxCellMeas_OF_InterFreqCellID = -1;
static gint ett_rrc_RemovedInterRATCellList = -1;
static gint ett_rrc_SEQUENCE_SIZE_1_maxCellMeas_OF_InterRATCellID = -1;
static gint ett_rrc_RemovedIntraFreqCellList = -1;
static gint ett_rrc_SEQUENCE_SIZE_1_maxCellMeas_OF_IntraFreqCellID = -1;
static gint ett_rrc_RemovedIntraFreqCellListOnSecULFreq = -1;
static gint ett_rrc_SEQUENCE_SIZE_1_maxCellMeasOnSecULFreq_OF_IntraFreqCellIDOnSecULFreq = -1;
static gint ett_rrc_ReportingCellStatus = -1;
static gint ett_rrc_ReportingCellStatus_r10 = -1;
static gint ett_rrc_ReportingCellStatusOpt = -1;
static gint ett_rrc_ReportingCellStatusOpt_r10 = -1;
static gint ett_rrc_ReportingInfoForCellDCH = -1;
static gint ett_rrc_ReportingInfoForCellDCH_LCR_r4 = -1;
static gint ett_rrc_ReqDataBitAssistance = -1;
static gint ett_rrc_T_ganssSatelliteInfo = -1;
static gint ett_rrc_RL_AdditionInfoList = -1;
static gint ett_rrc_RL_InformationLists = -1;
static gint ett_rrc_RSCPforANR = -1;
static gint ett_rrc_SatData = -1;
static gint ett_rrc_SatDataList = -1;
static gint ett_rrc_SatellitesListRelatedData = -1;
static gint ett_rrc_SatellitesListRelatedDataList = -1;
static gint ett_rrc_Satellite_clock_model = -1;
static gint ett_rrc_Satellite_clock_modelList = -1;
static gint ett_rrc_SBASclockModel = -1;
static gint ett_rrc_S_DPCCH_Information = -1;
static gint ett_rrc_SFN_SFN_ObsTimeDifference = -1;
static gint ett_rrc_SFN_SFN_RelTimeDifference1 = -1;
static gint ett_rrc_SIR_MeasurementList = -1;
static gint ett_rrc_SIR_MeasurementResults = -1;
static gint ett_rrc_SIR_TFCS_List = -1;
static gint ett_rrc_SIR_TimeslotList = -1;
static gint ett_rrc_SubFrame1Reserved = -1;
static gint ett_rrc_T_ADVinfo = -1;
static gint ett_rrc_T_ADVinfo_ext = -1;
static gint ett_rrc_T_CRMax = -1;
static gint ett_rrc_TCE_Id = -1;
static gint ett_rrc_TemporaryOffsetList = -1;
static gint ett_rrc_ThreholdNonUsedFrequency_deltaList = -1;
static gint ett_rrc_TimeslotInfo = -1;
static gint ett_rrc_TimeslotInfo_LCR_r4 = -1;
static gint ett_rrc_TimeslotInfoList = -1;
static gint ett_rrc_TimeslotInfoList_LCR_r4 = -1;
static gint ett_rrc_TimeslotInfoList_r4 = -1;
static gint ett_rrc_SEQUENCE_SIZE_1_maxTS_OF_TimeslotInfo = -1;
static gint ett_rrc_SEQUENCE_SIZE_1_maxTS_LCR_OF_TimeslotInfo_LCR_r4 = -1;
static gint ett_rrc_TimeslotISCP_List = -1;
static gint ett_rrc_TimeslotListWithISCP = -1;
static gint ett_rrc_TimeslotWithISCP = -1;
static gint ett_rrc_TraceReference = -1;
static gint ett_rrc_TraceRecordingSession = -1;
static gint ett_rrc_TrafficVolumeEventParam = -1;
static gint ett_rrc_TrafficVolumeEventResults = -1;
static gint ett_rrc_TrafficVolumeMeasQuantity = -1;
static gint ett_rrc_TrafficVolumeMeasSysInfo = -1;
static gint ett_rrc_TrafficVolumeMeasuredResults = -1;
static gint ett_rrc_TrafficVolumeMeasuredResultsList = -1;
static gint ett_rrc_TrafficVolumeMeasurement = -1;
static gint ett_rrc_TrafficVolumeMeasurementObjectList = -1;
static gint ett_rrc_TrafficVolumeReportCriteria = -1;
static gint ett_rrc_TrafficVolumeReportCriteriaSysInfo = -1;
static gint ett_rrc_TrafficVolumeReportingCriteria = -1;
static gint ett_rrc_TrafficVolumeReportingQuantity = -1;
static gint ett_rrc_TransChCriteria = -1;
static gint ett_rrc_SEQUENCE_SIZE_1_maxMeasParEvent_OF_TrafficVolumeEventParam = -1;
static gint ett_rrc_TransChCriteriaList = -1;
static gint ett_rrc_UE_6AB_Event = -1;
static gint ett_rrc_UE_6FG_Event = -1;
static gint ett_rrc_UE_AutonomousUpdateMode = -1;
static gint ett_rrc_UE_InternalEventParam = -1;
static gint ett_rrc_UE_InternalEventParamList = -1;
static gint ett_rrc_UE_InternalEventResults = -1;
static gint ett_rrc_UE_InternalMeasQuantity = -1;
static gint ett_rrc_UE_InternalMeasuredResults = -1;
static gint ett_rrc_T_modeSpecificInfo_131 = -1;
static gint ett_rrc_T_fdd_164 = -1;
static gint ett_rrc_T_tdd_113 = -1;
static gint ett_rrc_UE_InternalMeasuredResults_LCR_r4 = -1;
static gint ett_rrc_UE_InternalMeasuredResults_v770ext = -1;
static gint ett_rrc_T_modeSpecificInfo_132 = -1;
static gint ett_rrc_T_tdd384_768_02 = -1;
static gint ett_rrc_T_tdd128_66 = -1;
static gint ett_rrc_UE_InternalMeasurement = -1;
static gint ett_rrc_UE_InternalMeasurement_r4 = -1;
static gint ett_rrc_UE_InternalMeasurementSysInfo = -1;
static gint ett_rrc_UE_InternalReportCriteria = -1;
static gint ett_rrc_UE_InternalReportingCriteria = -1;
static gint ett_rrc_UE_InternalReportingQuantity = -1;
static gint ett_rrc_T_modeSpecificInfo_133 = -1;
static gint ett_rrc_T_fdd_165 = -1;
static gint ett_rrc_T_tdd_114 = -1;
static gint ett_rrc_UE_InternalReportingQuantity_r4 = -1;
static gint ett_rrc_T_modeSpecificInfo_134 = -1;
static gint ett_rrc_T_fdd_166 = -1;
static gint ett_rrc_T_tdd_115 = -1;
static gint ett_rrc_T_tddOption_28 = -1;
static gint ett_rrc_T_tdd384_43 = -1;
static gint ett_rrc_T_tdd128_67 = -1;
static gint ett_rrc_UE_RX_TX_ReportEntry = -1;
static gint ett_rrc_UE_RX_TX_ReportEntryList = -1;
static gint ett_rrc_UE_RX_TX_TimeDifferenceType2Info = -1;
static gint ett_rrc_UE_TransmittedPowerTDD_List = -1;
static gint ett_rrc_UL_TrCH_Identity = -1;
static gint ett_rrc_UE_Positioning_CipherParameters = -1;
static gint ett_rrc_UE_Positioning_DGANSSCorrections = -1;
static gint ett_rrc_UE_Positioning_DGANSSCorrections_r9 = -1;
static gint ett_rrc_UE_Positioning_DGANSSCorrections_v920ext = -1;
static gint ett_rrc_UE_Positioning_Error = -1;
static gint ett_rrc_UE_Positioning_Error_r7 = -1;
static gint ett_rrc_UE_Positioning_Error_v860ext = -1;
static gint ett_rrc_UE_Positioning_EventParam = -1;
static gint ett_rrc_UE_Positioning_EventParam_r7 = -1;
static gint ett_rrc_UE_Positioning_EventParamList = -1;
static gint ett_rrc_UE_Positioning_EventParamList_r7 = -1;
static gint ett_rrc_UE_Positioning_EventSpecificInfo = -1;
static gint ett_rrc_UE_Positioning_EventSpecificInfo_r7 = -1;
static gint ett_rrc_UE_Positioning_GANSS_AddUTCModels = -1;
static gint ett_rrc_UE_Positioning_GANSS_Almanac = -1;
static gint ett_rrc_UE_Positioning_GANSS_Almanac_v860ext = -1;
static gint ett_rrc_UE_Positioning_GANSS_Almanac_r8 = -1;
static gint ett_rrc_UE_Positioning_GANSS_Almanac_va40ext = -1;
static gint ett_rrc_UE_Positioning_GANSS_Almanac_r10 = -1;
static gint ett_rrc_UE_Positioning_GANSS_AdditionalAssistanceDataRequest = -1;
static gint ett_rrc_UE_Positioning_GANSS_AdditionalAssistanceDataRequest_v860ext = -1;
static gint ett_rrc_UE_Positioning_GANSS_AddIonoModel = -1;
static gint ett_rrc_UE_Positioning_GANSS_AddNavigationModels = -1;
static gint ett_rrc_UE_Positioning_GANSS_AssistanceData = -1;
static gint ett_rrc_UE_Positioning_GANSS_AssistanceData_v860ext = -1;
static gint ett_rrc_UE_Positioning_GANSS_AssistanceData_r8 = -1;
static gint ett_rrc_UE_Positioning_GANSS_AssistanceData_r9 = -1;
static gint ett_rrc_UE_Positioning_GANSS_AssistanceData_v920ext = -1;
static gint ett_rrc_UE_Positioning_GANSS_AssistanceData_va40ext = -1;
static gint ett_rrc_UE_Positioning_GANSS_AssistanceData_r10 = -1;
static gint ett_rrc_UE_Positioning_GANSS_AuxiliaryInfo = -1;
static gint ett_rrc_UE_Positioning_GANSS_ClockModel = -1;
static gint ett_rrc_UE_Positioning_GANSS_AddClockModels = -1;
static gint ett_rrc_UE_Positioning_GANSS_AddOrbitModels = -1;
static gint ett_rrc_UE_Positioning_GANSS_Data_Bit_Assistance = -1;
static gint ett_rrc_UE_Positioning_GANSS_EarthOrientPara = -1;
static gint ett_rrc_UE_Positioning_GANSS_IonosphericModel = -1;
static gint ett_rrc_UE_Positioning_GANSS_MeasuredResults = -1;
static gint ett_rrc_T_referenceTime = -1;
static gint ett_rrc_UE_Positioning_GANSS_MeasuredResults_v860ext = -1;
static gint ett_rrc_UE_Positioning_GANSS_NavigationModel = -1;
static gint ett_rrc_UE_Positioning_GANSS_OrbitModel = -1;
static gint ett_rrc_UE_Positioning_GANSS_RealTimeIntegrity = -1;
static gint ett_rrc_UE_Positioning_GANSS_RealTimeIntegrity_item = -1;
static gint ett_rrc_UE_Positioning_GANSS_ReferenceMeasurementInfo = -1;
static gint ett_rrc_UE_Positioning_GANSS_ReferenceMeasurementInfo_va40ext = -1;
static gint ett_rrc_UE_Positioning_GANSS_ReferenceMeasurementInfo_r10 = -1;
static gint ett_rrc_UE_Positioning_GANSS_ReferenceTime = -1;
static gint ett_rrc_T_utran_ganssreferenceTime = -1;
static gint ett_rrc_T_mode = -1;
static gint ett_rrc_T_fdd_167 = -1;
static gint ett_rrc_T_tdd_116 = -1;
static gint ett_rrc_UE_Positioning_GANSS_ReferenceTime_va40ext = -1;
static gint ett_rrc_UE_Positioning_GANSS_ReferenceTime_r10 = -1;
static gint ett_rrc_T_utran_ganssreferenceTime_01 = -1;
static gint ett_rrc_T_mode_01 = -1;
static gint ett_rrc_T_fdd_168 = -1;
static gint ett_rrc_T_tdd_117 = -1;
static gint ett_rrc_UE_Positioning_GANSS_TimeModel = -1;
static gint ett_rrc_UE_Positioning_GANSS_TimeModel_va40ext = -1;
static gint ett_rrc_UE_Positioning_GANSS_TimeModel_r10 = -1;
static gint ett_rrc_UE_Positioning_GANSS_TimeModels = -1;
static gint ett_rrc_UE_Positioning_GANSS_TimeModels_va40ext = -1;
static gint ett_rrc_UE_Positioning_GANSS_UTCModel = -1;
static gint ett_rrc_UE_Positioning_GPS_AcquisitionAssistance = -1;
static gint ett_rrc_UE_Positioning_GPS_AcquisitionAssistance_v770ext = -1;
static gint ett_rrc_UE_Positioning_GPS_AcquisitionAssistance_r7 = -1;
static gint ett_rrc_UE_Positioning_GPS_AcquisitionAssistance_va40ext = -1;
static gint ett_rrc_UE_Positioning_GPS_AcquisitionAssistance_r10 = -1;
static gint ett_rrc_UE_Positioning_GPS_AdditionalAssistanceDataRequest = -1;
static gint ett_rrc_UE_Positioning_GPS_Almanac = -1;
static gint ett_rrc_UE_Positioning_GPS_Almanac_va40ext = -1;
static gint ett_rrc_UE_Positioning_GPS_Almanac_r10 = -1;
static gint ett_rrc_UE_Positioning_GPS_AssistanceData = -1;
static gint ett_rrc_UE_Positioning_GPS_AssistanceData_v770ext = -1;
static gint ett_rrc_UE_Positioning_GPS_AssistanceData_r7 = -1;
static gint ett_rrc_UE_Positioning_GPS_AssistanceData_r9 = -1;
static gint ett_rrc_UE_Positioning_GPS_AssistanceData_v920ext = -1;
static gint ett_rrc_UE_Positioning_GPS_AssistanceData_va40ext = -1;
static gint ett_rrc_UE_Positioning_GPS_AssistanceData_r10 = -1;
static gint ett_rrc_UE_Positioning_GPS_DGPS_Corrections = -1;
static gint ett_rrc_UE_Positioning_GPS_DGPS_Corrections_r9 = -1;
static gint ett_rrc_UE_Positioning_GPS_DGPS_Corrections_v920ext = -1;
static gint ett_rrc_UE_Positioning_GPS_IonosphericModel = -1;
static gint ett_rrc_UE_Positioning_GPS_MeasurementResults = -1;
static gint ett_rrc_T_referenceTime_01 = -1;
static gint ett_rrc_UE_Positioning_GPS_MeasurementResults_v770ext = -1;
static gint ett_rrc_UE_Positioning_GPS_NavigationModel = -1;
static gint ett_rrc_UE_Positioning_GPS_NavModelAddDataReq = -1;
static gint ett_rrc_UE_Positioning_GPS_ReferenceCellInfo = -1;
static gint ett_rrc_T_modeSpecificInfo_135 = -1;
static gint ett_rrc_T_fdd_169 = -1;
static gint ett_rrc_T_tdd_118 = -1;
static gint ett_rrc_UE_Positioning_GPS_ReferenceTime = -1;
static gint ett_rrc_UE_Positioning_GPS_ReferenceTime_v770ext = -1;
static gint ett_rrc_UE_Positioning_GPS_ReferenceTime_r7 = -1;
static gint ett_rrc_UE_Positioning_GPS_ReferenceTime_va40ext = -1;
static gint ett_rrc_UE_Positioning_GPS_ReferenceTime_r10 = -1;
static gint ett_rrc_UE_Positioning_GPS_UTC_Model = -1;
static gint ett_rrc_UE_Positioning_IPDL_Parameters = -1;
static gint ett_rrc_UE_Positioning_IPDL_Parameters_r4 = -1;
static gint ett_rrc_T_modeSpecificInfo_136 = -1;
static gint ett_rrc_T_fdd_170 = -1;
static gint ett_rrc_T_tdd_119 = -1;
static gint ett_rrc_UE_Positioning_IPDL_Parameters_TDD_r4_ext = -1;
static gint ett_rrc_UE_Positioning_MeasuredResults = -1;
static gint ett_rrc_UE_Positioning_MeasuredResults_v770ext = -1;
static gint ett_rrc_UE_Positioning_MeasuredResults_v390ext = -1;
static gint ett_rrc_UE_Positioning_MeasuredResults_v860ext = -1;
static gint ett_rrc_UE_Positioning_Measurement = -1;
static gint ett_rrc_UE_Positioning_Measurement_v390ext = -1;
static gint ett_rrc_UE_Positioning_Measurement_r4 = -1;
static gint ett_rrc_UE_Positioning_Measurement_r7 = -1;
static gint ett_rrc_UE_Positioning_Measurement_r8 = -1;
static gint ett_rrc_UE_Positioning_Measurement_r9 = -1;
static gint ett_rrc_UE_Positioning_Measurement_r10 = -1;
static gint ett_rrc_UE_Positioning_MeasurementEventResults = -1;
static gint ett_rrc_UE_Positioning_MeasurementEventResults_v770ext = -1;
static gint ett_rrc_UE_Positioning_MeasurementEventResults_v860ext = -1;
static gint ett_rrc_UE_Positioning_OTDOA_AssistanceData = -1;
static gint ett_rrc_UE_Positioning_OTDOA_AssistanceData_r4 = -1;
static gint ett_rrc_UE_Positioning_OTDOA_AssistanceData_r4ext = -1;
static gint ett_rrc_UE_Positioning_OTDOA_AssistanceData_UEB = -1;
static gint ett_rrc_UE_Positioning_OTDOA_AssistanceData_UEB_ext = -1;
static gint ett_rrc_UE_Positioning_OTDOA_AssistanceData_r7 = -1;
static gint ett_rrc_UE_Positioning_IPDL_Parameters_TDDList_r4_ext = -1;
static gint ett_rrc_UE_Positioning_OTDOA_MeasuredResultsTDD_ext = -1;
static gint ett_rrc_UE_Positioning_OTDOA_Measurement = -1;
static gint ett_rrc_T_modeSpecificInfo_137 = -1;
static gint ett_rrc_T_fdd_171 = -1;
static gint ett_rrc_T_tdd_120 = -1;
static gint ett_rrc_UE_Positioning_OTDOA_Measurement_v390ext = -1;
static gint ett_rrc_UE_Positioning_OTDOA_NeighbourCellInfo = -1;
static gint ett_rrc_T_modeSpecificInfo_138 = -1;
static gint ett_rrc_T_fdd_172 = -1;
static gint ett_rrc_T_tdd_121 = -1;
static gint ett_rrc_T_positioningMode = -1;
static gint ett_rrc_T_ueBased = -1;
static gint ett_rrc_T_ueAssisted = -1;
static gint ett_rrc_UE_Positioning_OTDOA_NeighbourCellInfo_r4 = -1;
static gint ett_rrc_T_modeSpecificInfo_139 = -1;
static gint ett_rrc_T_fdd_173 = -1;
static gint ett_rrc_T_tdd_122 = -1;
static gint ett_rrc_T_positioningMode_01 = -1;
static gint ett_rrc_T_ueBased_01 = -1;
static gint ett_rrc_T_ueAssisted_01 = -1;
static gint ett_rrc_UE_Positioning_OTDOA_NeighbourCellInfo_UEB = -1;
static gint ett_rrc_T_modeSpecificInfo_140 = -1;
static gint ett_rrc_T_fdd_174 = -1;
static gint ett_rrc_T_tdd_123 = -1;
static gint ett_rrc_UE_Positioning_OTDOA_NeighbourCellInfo_UEB_ext = -1;
static gint ett_rrc_UE_Positioning_OTDOA_NeighbourCellInfo_r7 = -1;
static gint ett_rrc_T_modeSpecificInfo_141 = -1;
static gint ett_rrc_T_fdd_175 = -1;
static gint ett_rrc_T_tdd_124 = -1;
static gint ett_rrc_T_positioningMode_02 = -1;
static gint ett_rrc_T_ueBased_02 = -1;
static gint ett_rrc_T_ueAssisted_02 = -1;
static gint ett_rrc_UE_Positioning_OTDOA_NeighbourCellList = -1;
static gint ett_rrc_UE_Positioning_OTDOA_NeighbourCellList_r4 = -1;
static gint ett_rrc_UE_Positioning_OTDOA_NeighbourCellList_UEB = -1;
static gint ett_rrc_UE_Positioning_OTDOA_NeighbourCellList_UEB_ext = -1;
static gint ett_rrc_UE_Positioning_OTDOA_NeighbourCellList_r7 = -1;
static gint ett_rrc_UE_Positioning_OTDOA_Quality = -1;
static gint ett_rrc_UE_Positioning_OTDOA_ReferenceCellInfo = -1;
static gint ett_rrc_T_modeSpecificInfo_142 = -1;
static gint ett_rrc_T_fdd_176 = -1;
static gint ett_rrc_T_tdd_125 = -1;
static gint ett_rrc_T_positioningMode_03 = -1;
static gint ett_rrc_T_ueBased_03 = -1;
static gint ett_rrc_T_ueAssisted_03 = -1;
static gint ett_rrc_UE_Positioning_OTDOA_ReferenceCellInfo_r4 = -1;
static gint ett_rrc_T_modeSpecificInfo_143 = -1;
static gint ett_rrc_T_fdd_177 = -1;
static gint ett_rrc_T_tdd_126 = -1;
static gint ett_rrc_T_positioningMode_04 = -1;
static gint ett_rrc_T_ueBased_04 = -1;
static gint ett_rrc_T_ueAssisted_04 = -1;
static gint ett_rrc_UE_Positioning_OTDOA_ReferenceCellInfo_UEB = -1;
static gint ett_rrc_T_modeSpecificInfo_144 = -1;
static gint ett_rrc_T_fdd_178 = -1;
static gint ett_rrc_T_tdd_127 = -1;
static gint ett_rrc_UE_Positioning_OTDOA_ReferenceCellInfo_UEB_ext = -1;
static gint ett_rrc_UE_Positioning_OTDOA_ReferenceCellInfo_r7 = -1;
static gint ett_rrc_T_modeSpecificInfo_145 = -1;
static gint ett_rrc_T_fdd_179 = -1;
static gint ett_rrc_T_tdd_128 = -1;
static gint ett_rrc_T_positioningMode_05 = -1;
static gint ett_rrc_T_ueBased_05 = -1;
static gint ett_rrc_T_ueAssisted_05 = -1;
static gint ett_rrc_UE_Positioning_PositionEstimateInfo = -1;
static gint ett_rrc_T_referenceTime_02 = -1;
static gint ett_rrc_T_cell_Timing = -1;
static gint ett_rrc_T_modeSpecificInfo_146 = -1;
static gint ett_rrc_T_fdd_180 = -1;
static gint ett_rrc_T_tdd_129 = -1;
static gint ett_rrc_UE_Positioning_PositionEstimateInfo_v770ext = -1;
static gint ett_rrc_T_referenceTimeOptions = -1;
static gint ett_rrc_T_r7_15 = -1;
static gint ett_rrc_T_referenceTime_03 = -1;
static gint ett_rrc_UE_Positioning_ReportCriteria = -1;
static gint ett_rrc_UE_Positioning_ReportCriteria_r7 = -1;
static gint ett_rrc_UE_Positioning_ReportingQuantity = -1;
static gint ett_rrc_UE_Positioning_ReportingQuantity_v390ext = -1;
static gint ett_rrc_UE_Positioning_ReportingQuantity_r4 = -1;
static gint ett_rrc_UE_Positioning_ReportingQuantity_r7 = -1;
static gint ett_rrc_UE_Positioning_ReportingQuantity_r8 = -1;
static gint ett_rrc_UTCmodelSet1 = -1;
static gint ett_rrc_UTCmodelSet2 = -1;
static gint ett_rrc_UTCmodelSet3 = -1;
static gint ett_rrc_UTRAN_GANSSReferenceTime = -1;
static gint ett_rrc_T_ue_GANSSTimingOfCellFrames = -1;
static gint ett_rrc_T_mode_02 = -1;
static gint ett_rrc_T_fdd_181 = -1;
static gint ett_rrc_T_tdd_130 = -1;
static gint ett_rrc_UTRAN_FDD_FrequencyList = -1;
static gint ett_rrc_UTRAN_FDD_Frequency = -1;
static gint ett_rrc_UTRAN_TDD_FrequencyList = -1;
static gint ett_rrc_UTRAN_TDD_Frequency = -1;
static gint ett_rrc_UTRAN_GPSReferenceTime = -1;
static gint ett_rrc_T_utran_GPSTimingOfCell = -1;
static gint ett_rrc_T_modeSpecificInfo_147 = -1;
static gint ett_rrc_T_fdd_182 = -1;
static gint ett_rrc_T_tdd_131 = -1;
static gint ett_rrc_UTRAN_GPSReferenceTimeResult = -1;
static gint ett_rrc_T_ue_GPSTimingOfCell = -1;
static gint ett_rrc_T_modeSpecificInfo_148 = -1;
static gint ett_rrc_T_fdd_183 = -1;
static gint ett_rrc_T_tdd_132 = -1;
static gint ett_rrc_UTRA_PriorityInfoList = -1;
static gint ett_rrc_T_utra_ServingCell = -1;
static gint ett_rrc_UTRA_PriorityInfoList_v920ext = -1;
static gint ett_rrc_VelocityEstimate = -1;
static gint ett_rrc_BCCH_ModificationInfo = -1;
static gint ett_rrc_BSIC = -1;
static gint ett_rrc_CBS_DRX_Level1Information = -1;
static gint ett_rrc_CDMA2000_Message = -1;
static gint ett_rrc_CDMA2000_MessageList = -1;
static gint ett_rrc_CDMA2000_UMTS_Frequency_List = -1;
static gint ett_rrc_DataTransmFrequency = -1;
static gint ett_rrc_DataVolumeHistory = -1;
static gint ett_rrc_DataVolumePerRB = -1;
static gint ett_rrc_DataVolumePerRB_List = -1;
static gint ett_rrc_DeferredMeasurementControlReadingSupport = -1;
static gint ett_rrc_T_modeSpecificInfo_149 = -1;
static gint ett_rrc_T_fdd_184 = -1;
static gint ett_rrc_T_tdd_133 = -1;
static gint ett_rrc_ETWS_Information = -1;
static gint ett_rrc_EUTRA_BlacklistedCell = -1;
static gint ett_rrc_EUTRA_BlacklistedCellPerFreqList = -1;
static gint ett_rrc_EUTRA_RadioAccessCapability = -1;
static gint ett_rrc_EUTRA_TargetFreqInfoList = -1;
static gint ett_rrc_EUTRA_TargetFreqInfo = -1;
static gint ett_rrc_ExtGANSS_SchedulingInfo = -1;
static gint ett_rrc_ExtGANSS_SIBTypeInfoSchedulingInfoList = -1;
static gint ett_rrc_ExtGANSS_SIBTypeInfoSchedulingInfo = -1;
static gint ett_rrc_SEQUENCE_SIZE_1_maxSIB_OF_ExtGANSS_SchedulingInfo = -1;
static gint ett_rrc_ExtSIBTypeInfoSchedulingInfo_List = -1;
static gint ett_rrc_ExtSIBTypeInfoSchedulingInfo = -1;
static gint ett_rrc_ExtSIBTypeInfoSchedulingInfo_List2 = -1;
static gint ett_rrc_ExtSIBTypeInfoSchedulingInfo2 = -1;
static gint ett_rrc_FDD_UMTS_Frequency_List = -1;
static gint ett_rrc_FrequencyInfoCDMA2000 = -1;
static gint ett_rrc_GERAN_SystemInformation = -1;
static gint ett_rrc_GSM_BA_Range = -1;
static gint ett_rrc_GSM_BA_Range_List = -1;
static gint ett_rrc_GSM_MessageList = -1;
static gint ett_rrc_GsmSecurityCapability = -1;
static gint ett_rrc_GSM_TargetCellInfoList = -1;
static gint ett_rrc_GSM_TargetCellInfo = -1;
static gint ett_rrc_IdentificationOfReceivedMessage = -1;
static gint ett_rrc_InterRAT_ChangeFailureCause = -1;
static gint ett_rrc_GERANIu_MessageList = -1;
static gint ett_rrc_InterRAT_UE_RadioAccessCapability = -1;
static gint ett_rrc_T_gsm_15 = -1;
static gint ett_rrc_T_cdma2000_01 = -1;
static gint ett_rrc_InterRAT_UE_RadioAccessCapabilityList = -1;
static gint ett_rrc_InterRAT_UE_RadioAccessCapability_v590ext = -1;
static gint ett_rrc_InterRAT_UE_RadioAccessCapability_v690ext = -1;
static gint ett_rrc_InterRAT_UE_RadioAccessCapability_v860ext = -1;
static gint ett_rrc_InterRAT_UE_SecurityCapability = -1;
static gint ett_rrc_T_gsm_16 = -1;
static gint ett_rrc_InterRAT_UE_SecurityCapList = -1;
static gint ett_rrc_InterRAT_HO_FailureCause = -1;
static gint ett_rrc_MasterInformationBlock = -1;
static gint ett_rrc_T_v690NonCriticalExtensions_48 = -1;
static gint ett_rrc_T_v6b0NonCriticalExtensions_15 = -1;
static gint ett_rrc_T_v860NonCriticalExtensions_26 = -1;
static gint ett_rrc_T_nonCriticalExtensions_195 = -1;
static gint ett_rrc_MasterInformationBlock_v690ext = -1;
static gint ett_rrc_MasterInformationBlock_v6b0ext_IEs = -1;
static gint ett_rrc_MasterInformationBlock_v860ext_IEs = -1;
static gint ett_rrc_PredefinedConfigIdentityAndValueTag = -1;
static gint ett_rrc_ProtocolErrorInformation = -1;
static gint ett_rrc_T_diagnosticsType_01 = -1;
static gint ett_rrc_T_type1_07 = -1;
static gint ett_rrc_Rplmn_Information = -1;
static gint ett_rrc_Rplmn_Information_r4 = -1;
static gint ett_rrc_SchedulingInformation = -1;
static gint ett_rrc_T_scheduling = -1;
static gint ett_rrc_T_sib_Pos = -1;
static gint ett_rrc_SchedulingInformationSIB = -1;
static gint ett_rrc_SchedulingInformationSIBSb = -1;
static gint ett_rrc_SIBOccurrenceIdentityAndValueTag = -1;
static gint ett_rrc_SIB_ReferenceList = -1;
static gint ett_rrc_SIBSb_ReferenceList = -1;
static gint ett_rrc_SIB_ReferenceListFACH = -1;
static gint ett_rrc_SIB_TypeAndTag = -1;
static gint ett_rrc_SIBSb_TypeAndTag = -1;
static gint ett_rrc_SIB_TypeExt = -1;
static gint ett_rrc_SIB_TypeExt2 = -1;
static gint ett_rrc_SIB_TypeExtGANSS = -1;
static gint ett_rrc_SibOFF_List = -1;
static gint ett_rrc_SysInfoType1 = -1;
static gint ett_rrc_T_v3a0NonCriticalExtensions_12 = -1;
static gint ett_rrc_T_v860NonCriticalExtentions_02 = -1;
static gint ett_rrc_T_nonCriticalExtensions_196 = -1;
static gint ett_rrc_SysInfoType1_v3a0ext_IEs = -1;
static gint ett_rrc_SysInfoType1_v860ext_IEs = -1;
static gint ett_rrc_SysInfoType2 = -1;
static gint ett_rrc_T_nonCriticalExtensions_197 = -1;
static gint ett_rrc_SysInfoType3 = -1;
static gint ett_rrc_T_v4b0NonCriticalExtensions_17 = -1;
static gint ett_rrc_T_v590NonCriticalExtension = -1;
static gint ett_rrc_T_v5c0NoncriticalExtension = -1;
static gint ett_rrc_T_v670NonCriticalExtension = -1;
static gint ett_rrc_T_v770NonCriticalExtension_02 = -1;
static gint ett_rrc_T_v830NonCriticalExtension = -1;
static gint ett_rrc_T_v860NonCriticalExtension_01 = -1;
static gint ett_rrc_T_v870NonCriticalExtension = -1;
static gint ett_rrc_T_v900NonCriticalExtension = -1;
static gint ett_rrc_T_nonCriticalExtensions_198 = -1;
static gint ett_rrc_SysInfoType3_v4b0ext_IEs = -1;
static gint ett_rrc_SysInfoType3_v590ext = -1;
static gint ett_rrc_SysInfoType3_v5c0ext_IEs = -1;
static gint ett_rrc_SysInfoType3_v670ext = -1;
static gint ett_rrc_SysInfoType3_v770ext_IEs = -1;
static gint ett_rrc_SysInfoType3_v830ext_IEs = -1;
static gint ett_rrc_SysInfoType3_v860ext_IEs = -1;
static gint ett_rrc_SysInfoType3_v870ext_IEs = -1;
static gint ett_rrc_SysInfoType3_v920ext_IEs = -1;
static gint ett_rrc_SysInfoType4 = -1;
static gint ett_rrc_T_v4b0NonCriticalExtensions_18 = -1;
static gint ett_rrc_T_v590NonCriticalExtension_01 = -1;
static gint ett_rrc_T_v5b0NonCriticalExtension = -1;
static gint ett_rrc_T_v5c0NonCriticalExtension = -1;
static gint ett_rrc_T_nonCriticalExtensions_199 = -1;
static gint ett_rrc_SysInfoType4_v4b0ext_IEs = -1;
static gint ett_rrc_SysInfoType4_v590ext = -1;
static gint ett_rrc_SysInfoType4_v5b0ext_IEs = -1;
static gint ett_rrc_SysInfoType4_v5c0ext_IEs = -1;
static gint ett_rrc_SysInfoType5 = -1;
static gint ett_rrc_T_modeSpecificInfo_150 = -1;
static gint ett_rrc_T_fdd_185 = -1;
static gint ett_rrc_T_tdd_134 = -1;
static gint ett_rrc_T_v4b0NonCriticalExtensions_19 = -1;
static gint ett_rrc_T_v590NonCriticalExtensions_28 = -1;
static gint ett_rrc_T_v650NonCriticalExtensions = -1;
static gint ett_rrc_T_v680NonCriticalExtensions_02 = -1;
static gint ett_rrc_T_v690NonCriticalExtensions_49 = -1;
static gint ett_rrc_T_v770NonCriticalExtensions_40 = -1;
static gint ett_rrc_T_v860NonCriticalExtensions_27 = -1;
static gint ett_rrc_T_v890NonCriticalExtensions_11 = -1;
static gint ett_rrc_T_v8b0NonCriticalExtensions_01 = -1;
static gint ett_rrc_T_v8d0NonCriticalExtensions = -1;
static gint ett_rrc_T_va40NonCriticalExtensions_18 = -1;
static gint ett_rrc_T_va80NonCriticalExtensions_02 = -1;
static gint ett_rrc_SysInfoType5_v4b0ext_IEs = -1;
static gint ett_rrc_T_tdd128SpecificInfo = -1;
static gint ett_rrc_SysInfoType5_v590ext_IEs = -1;
static gint ett_rrc_T_hcr_r5_SpecificInfo = -1;
static gint ett_rrc_SysInfoType5_v650ext_IEs = -1;
static gint ett_rrc_SysInfoType5_v680ext_IEs = -1;
static gint ett_rrc_SysInfoType5_v690ext_IEs = -1;
static gint ett_rrc_T_sccpch_SystemInformation_MBMS = -1;
static gint ett_rrc_SysInfoType5_v770ext_IEs = -1;
static gint ett_rrc_T_modeSpecificInfo_151 = -1;
static gint ett_rrc_T_fdd_186 = -1;
static gint ett_rrc_T_tdd768_24 = -1;
static gint ett_rrc_T_tdd128_68 = -1;
static gint ett_rrc_T_sccpch_SystemInformation_MBMS_01 = -1;
static gint ett_rrc_SysInfoType5_v860ext_IEs = -1;
static gint ett_rrc_T_dummy_15 = -1;
static gint ett_rrc_T_modeSpecificInfo_152 = -1;
static gint ett_rrc_T_fdd_187 = -1;
static gint ett_rrc_T_tdd128_69 = -1;
static gint ett_rrc_T_hs_dsch_CommonSysInfo = -1;
static gint ett_rrc_SysInfoType5_v890ext_IEs = -1;
static gint ett_rrc_SysInfoType5_v8b0ext_IEs = -1;
static gint ett_rrc_SysInfoType5_v8d0ext_IEs = -1;
static gint ett_rrc_SysInfoType5_va40ext_IEs = -1;
static gint ett_rrc_SysInfoType5_va80ext_IEs = -1;
static gint ett_rrc_SysInfoType5_NonCriticalExtensions_vbxy_IEs = -1;
static gint ett_rrc_T_nonCriticalExtensions_200 = -1;
static gint ett_rrc_SysInfoType5_vbxyext_IEs = -1;
static gint ett_rrc_SysInfoType6 = -1;
static gint ett_rrc_T_modeSpecificInfo_153 = -1;
static gint ett_rrc_T_fdd_188 = -1;
static gint ett_rrc_T_tdd_135 = -1;
static gint ett_rrc_T_v4b0NonCriticalExtensions_20 = -1;
static gint ett_rrc_T_v590NonCriticalExtensions_29 = -1;
static gint ett_rrc_T_v650nonCriticalExtensions = -1;
static gint ett_rrc_T_v690nonCriticalExtensions = -1;
static gint ett_rrc_T_v770NonCriticalExtensions_41 = -1;
static gint ett_rrc_T_va40NonCriticalExtensions_19 = -1;
static gint ett_rrc_T_va80NonCriticalExtensions_03 = -1;
static gint ett_rrc_T_nonCriticalExtensions_201 = -1;
static gint ett_rrc_SysInfoType6_v4b0ext_IEs = -1;
static gint ett_rrc_T_tdd128SpecificInfo_01 = -1;
static gint ett_rrc_SysInfoType6_v590ext_IEs = -1;
static gint ett_rrc_T_hcr_r5_SpecificInfo_01 = -1;
static gint ett_rrc_SysInfoType6_v650ext_IEs = -1;
static gint ett_rrc_SysInfoType6_v690ext_IEs = -1;
static gint ett_rrc_SysInfoType6_v770ext_IEs = -1;
static gint ett_rrc_T_tdd768SpecificInfo = -1;
static gint ett_rrc_SysInfoType6_va40ext_IEs = -1;
static gint ett_rrc_SysInfoType6_va80ext_IEs = -1;
static gint ett_rrc_SysInfoType7 = -1;
static gint ett_rrc_T_modeSpecificInfo_154 = -1;
static gint ett_rrc_T_fdd_189 = -1;
static gint ett_rrc_T_nonCriticalExtensions_202 = -1;
static gint ett_rrc_SysInfoType8 = -1;
static gint ett_rrc_T_nonCriticalExtensions_203 = -1;
static gint ett_rrc_SysInfoType9 = -1;
static gint ett_rrc_T_nonCriticalExtensions_204 = -1;
static gint ett_rrc_SysInfoType10 = -1;
static gint ett_rrc_T_nonCriticalExtensions_205 = -1;
static gint ett_rrc_SysInfoType11 = -1;
static gint ett_rrc_T_v4b0NonCriticalExtensions_21 = -1;
static gint ett_rrc_T_v590NonCriticalExtension_02 = -1;
static gint ett_rrc_T_v690NonCriticalExtensions_50 = -1;
static gint ett_rrc_T_v6b0NonCriticalExtensions_16 = -1;
static gint ett_rrc_T_v770NonCriticalExtensions_42 = -1;
static gint ett_rrc_T_v7b0NonCriticalExtensions_02 = -1;
static gint ett_rrc_T_v860NonCriticalExtensions_28 = -1;
static gint ett_rrc_T_va80NonCriticalExtensions_04 = -1;
static gint ett_rrc_T_nonCriticalExtensions_206 = -1;
static gint ett_rrc_SysInfoType11_v4b0ext_IEs = -1;
static gint ett_rrc_SysInfoType11_v590ext_IEs = -1;
static gint ett_rrc_SysInfoType11_v690ext_IEs = -1;
static gint ett_rrc_SysInfoType11_v6b0ext_IEs = -1;
static gint ett_rrc_SysInfoType11_v770ext_IEs = -1;
static gint ett_rrc_SysInfoType11_v7b0ext_IEs = -1;
static gint ett_rrc_SysInfoType11_v860ext_IEs = -1;
static gint ett_rrc_SysInfoType11_va80ext_IEs = -1;
static gint ett_rrc_SysInfoType11bis = -1;
static gint ett_rrc_T_v7b0NonCriticalExtensions_03 = -1;
static gint ett_rrc_T_v860NonCriticalExtensions_29 = -1;
static gint ett_rrc_T_va80NonCriticalExtensions_05 = -1;
static gint ett_rrc_T_nonCriticalExtensions_207 = -1;
static gint ett_rrc_SysInfoType11bis_v7b0ext_IEs = -1;
static gint ett_rrc_SysInfoType11bis_v860ext_IEs = -1;
static gint ett_rrc_SysInfoType11bis_va80ext_IEs = -1;
static gint ett_rrc_SysInfoType12 = -1;
static gint ett_rrc_T_v4b0NonCriticalExtensions_22 = -1;
static gint ett_rrc_T_v590NonCriticalExtension_03 = -1;
static gint ett_rrc_T_v690NonCriticalExtensions_51 = -1;
static gint ett_rrc_T_v6b0NonCriticalExtensions_17 = -1;
static gint ett_rrc_T_v7b0NonCriticalExtensions_04 = -1;
static gint ett_rrc_T_va80NonCriticalExtensions_06 = -1;
static gint ett_rrc_T_nonCriticalExtensions_208 = -1;
static gint ett_rrc_SysInfoType12_v4b0ext_IEs = -1;
static gint ett_rrc_SysInfoType12_v590ext_IEs = -1;
static gint ett_rrc_SysInfoType12_v690ext_IEs = -1;
static gint ett_rrc_SysInfoType12_v6b0ext_IEs = -1;
static gint ett_rrc_SysInfoType12_v7b0ext_IEs = -1;
static gint ett_rrc_SysInfoType12_va80ext_IEs = -1;
static gint ett_rrc_SysInfoType13 = -1;
static gint ett_rrc_T_v3a0NonCriticalExtensions_13 = -1;
static gint ett_rrc_T_v4b0NonCriticalExtensions_23 = -1;
static gint ett_rrc_T_v770NonCriticalExtensions_43 = -1;
static gint ett_rrc_T_nonCriticalExtensions_209 = -1;
static gint ett_rrc_SysInfoType13_v3a0ext_IEs = -1;
static gint ett_rrc_SysInfoType13_v4b0ext_IEs = -1;
static gint ett_rrc_SysInfoType13_v770ext_IEs = -1;
static gint ett_rrc_SysInfoType13_1 = -1;
static gint ett_rrc_T_nonCriticalExtensions_210 = -1;
static gint ett_rrc_SysInfoType13_2 = -1;
static gint ett_rrc_T_nonCriticalExtensions_211 = -1;
static gint ett_rrc_SysInfoType13_3 = -1;
static gint ett_rrc_T_nonCriticalExtensions_212 = -1;
static gint ett_rrc_SysInfoType13_4 = -1;
static gint ett_rrc_T_nonCriticalExtensions_213 = -1;
static gint ett_rrc_SysInfoType14 = -1;
static gint ett_rrc_T_nonCriticalExtensions_214 = -1;
static gint ett_rrc_SysInfoType15 = -1;
static gint ett_rrc_T_v4b0NonCriticalExtensions_24 = -1;
static gint ett_rrc_T_v770NonCriticalExtensions_44 = -1;
static gint ett_rrc_T_va40NonCriticalExtensions_20 = -1;
static gint ett_rrc_T_nonCriticalExtensions_215 = -1;
static gint ett_rrc_SysInfoType15_v4b0ext_IEs = -1;
static gint ett_rrc_SysInfoType15_v770ext_IEs = -1;
static gint ett_rrc_SysInfoType15_va40ext_IEs = -1;
static gint ett_rrc_SysInfoType15bis = -1;
static gint ett_rrc_T_v860NonCriticalExtensions_30 = -1;
static gint ett_rrc_T_va40NonCriticalExtensions_21 = -1;
static gint ett_rrc_T_nonCriticalExtensions_216 = -1;
static gint ett_rrc_SysInfoType15bis_v860ext_IEs = -1;
static gint ett_rrc_SysInfoType15bis_va40ext_IEs = -1;
static gint ett_rrc_SysInfoType15_1 = -1;
static gint ett_rrc_T_v920NonCriticalExtensions_04 = -1;
static gint ett_rrc_T_nonCriticalExtensions_217 = -1;
static gint ett_rrc_SysInfoType15_1_v920ext_IEs = -1;
static gint ett_rrc_SysInfoType15_1bis = -1;
static gint ett_rrc_T_v920NonCriticalExtensions_05 = -1;
static gint ett_rrc_T_nonCriticalExtensions_218 = -1;
static gint ett_rrc_SysInfoType15_1bis_v920ext_IEs = -1;
static gint ett_rrc_SysInfoType15_2 = -1;
static gint ett_rrc_T_nonCriticalExtensions_219 = -1;
static gint ett_rrc_SysInfoType15_2bis = -1;
static gint ett_rrc_T_nonCriticalExtensions_220 = -1;
static gint ett_rrc_SysInfoType15_2ter = -1;
static gint ett_rrc_T_nonCriticalExtensions_221 = -1;
static gint ett_rrc_SysInfoType15_3 = -1;
static gint ett_rrc_T_va40NonCriticalExtensions_22 = -1;
static gint ett_rrc_T_nonCriticalExtensions_222 = -1;
static gint ett_rrc_SysInfoType15_3_va40ext_IEs = -1;
static gint ett_rrc_SysInfoType15_3bis = -1;
static gint ett_rrc_T_v860NonCriticalExtensions_31 = -1;
static gint ett_rrc_T_va40NonCriticalExtensions_23 = -1;
static gint ett_rrc_T_nonCriticalExtensions_223 = -1;
static gint ett_rrc_SysInfoType15_3bis_v860ext_IEs = -1;
static gint ett_rrc_SysInfoType15_3bis_va40ext_IEs = -1;
static gint ett_rrc_SysInfoType15_4 = -1;
static gint ett_rrc_T_v3a0NonCriticalExtensions_14 = -1;
static gint ett_rrc_T_v4b0NonCriticalExtensions_25 = -1;
static gint ett_rrc_T_nonCriticalExtensions_224 = -1;
static gint ett_rrc_SysInfoType15_4_v3a0ext = -1;
static gint ett_rrc_SysInfoType15_4_v4b0ext = -1;
static gint ett_rrc_SysInfoType15_5 = -1;
static gint ett_rrc_T_v3a0NonCriticalExtensions_15 = -1;
static gint ett_rrc_T_v770NonCriticalExtensions_45 = -1;
static gint ett_rrc_T_nonCriticalExtensions_225 = -1;
static gint ett_rrc_SysInfoType15_5_v3a0ext = -1;
static gint ett_rrc_SysInfoType15_5_v770ext_IEs = -1;
static gint ett_rrc_SysInfoType15_6 = -1;
static gint ett_rrc_T_va40NonCriticalExtensions_24 = -1;
static gint ett_rrc_T_nonCriticalExtensions_226 = -1;
static gint ett_rrc_SysInfoType15_6_va40ext_IEs = -1;
static gint ett_rrc_SysInfoType15_7 = -1;
static gint ett_rrc_T_nonCriticalExtensions_227 = -1;
static gint ett_rrc_SysInfoType15_8 = -1;
static gint ett_rrc_T_nonCriticalExtensions_228 = -1;
static gint ett_rrc_SysInfoType16 = -1;
static gint ett_rrc_T_v770NonCriticalExtensions_46 = -1;
static gint ett_rrc_T_v920NonCriticalExtensions_06 = -1;
static gint ett_rrc_T_nonCriticalExtensions_229 = -1;
static gint ett_rrc_SysInfoType16_v770ext_IEs = -1;
static gint ett_rrc_SysInfoType16_v920ext_IEs = -1;
static gint ett_rrc_SysInfoType17 = -1;
static gint ett_rrc_T_v4b0NonCriticalExtensions_26 = -1;
static gint ett_rrc_T_v590NonCriticalExtensions_30 = -1;
static gint ett_rrc_T_v770NonCriticalExtensions_47 = -1;
static gint ett_rrc_T_nonCriticalExtensions_230 = -1;
static gint ett_rrc_SysInfoType17_v4b0ext_IEs = -1;
static gint ett_rrc_T_tdd128SpecificInfo_02 = -1;
static gint ett_rrc_SysInfoType17_v590ext_IEs = -1;
static gint ett_rrc_T_hcr_r5_SpecificInfo_02 = -1;
static gint ett_rrc_SysInfoType17_v770ext_IEs = -1;
static gint ett_rrc_T_tdd768SpecificInfo_01 = -1;
static gint ett_rrc_SysInfoType18 = -1;
static gint ett_rrc_T_v6b0NonCriticalExtensions_18 = -1;
static gint ett_rrc_T_v860NonCriticalExtensions_32 = -1;
static gint ett_rrc_T_nonCriticalExtensions_231 = -1;
static gint ett_rrc_SysInfoType18_v6b0ext = -1;
static gint ett_rrc_SysInfoType18_v860ext = -1;
static gint ett_rrc_SysInfoType19 = -1;
static gint ett_rrc_T_v920NonCriticalExtensions_07 = -1;
static gint ett_rrc_T_va80NonCriticalExtensions_07 = -1;
static gint ett_rrc_T_vb30NonCriticalExtensions = -1;
static gint ett_rrc_T_vbxyNonCriticalExtensions_19 = -1;
static gint ett_rrc_T_nonCriticalExtensions_232 = -1;
static gint ett_rrc_SysInfoType19_v920ext = -1;
static gint ett_rrc_SysInfoType19_va80ext = -1;
static gint ett_rrc_SysInfoType19_vb30ext = -1;
static gint ett_rrc_SysInfoType19_vbxyext = -1;
static gint ett_rrc_SysInfoType20 = -1;
static gint ett_rrc_T_nonCriticalExtensions_233 = -1;
static gint ett_rrc_SysInfoType21 = -1;
static gint ett_rrc_T_nonCriticalExtensions_234 = -1;
static gint ett_rrc_SysInfoType22 = -1;
static gint ett_rrc_T_modeSpecificInfo_155 = -1;
static gint ett_rrc_T_fdd_190 = -1;
static gint ett_rrc_T_nonCriticalExtensions_235 = -1;
static gint ett_rrc_SysInfoTypeSB1 = -1;
static gint ett_rrc_T_v6b0NonCriticalExtensions_19 = -1;
static gint ett_rrc_T_v860NonCriticalExtensions_33 = -1;
static gint ett_rrc_T_nonCriticalExtensions_236 = -1;
static gint ett_rrc_SysInfoTypeSB1_v6b0ext = -1;
static gint ett_rrc_SysInfoTypeSB1_v860ext = -1;
static gint ett_rrc_SysInfoTypeSB2 = -1;
static gint ett_rrc_T_v6b0NonCriticalExtensions_20 = -1;
static gint ett_rrc_T_v860NonCriticalExtensions_34 = -1;
static gint ett_rrc_T_nonCriticalExtensions_237 = -1;
static gint ett_rrc_SysInfoTypeSB2_v6b0ext = -1;
static gint ett_rrc_SysInfoTypeSB2_v860ext = -1;
static gint ett_rrc_TDD_UMTS_Frequency_List = -1;
static gint ett_rrc_UE_HistoryInformation = -1;
static gint ett_rrc_ValueTagInfo = -1;
static gint ett_rrc_MBMS_CommonRBInformation_r6 = -1;
static gint ett_rrc_MBMS_CommonRBInformationList_r6 = -1;
static gint ett_rrc_MBMS_ConnectedModeCountingScope = -1;
static gint ett_rrc_MBMS_CurrentCell_SCCPCH_r6 = -1;
static gint ett_rrc_MBMS_CurrentCell_SCCPCHList_r6 = -1;
static gint ett_rrc_MBMS_FACHCarryingMTCH_List = -1;
static gint ett_rrc_MBMS_JoinedInformation_r6 = -1;
static gint ett_rrc_MBMS_L1CombiningSchedule_32 = -1;
static gint ett_rrc_T_mtch_L1CombiningPeriodList = -1;
static gint ett_rrc_T_mtch_L1CombiningPeriodList_item = -1;
static gint ett_rrc_MBMS_L1CombiningSchedule_64 = -1;
static gint ett_rrc_T_mtch_L1CombiningPeriodList_01 = -1;
static gint ett_rrc_T_mtch_L1CombiningPeriodList_item_01 = -1;
static gint ett_rrc_MBMS_L1CombiningSchedule_128 = -1;
static gint ett_rrc_T_mtch_L1CombiningPeriodList_02 = -1;
static gint ett_rrc_T_mtch_L1CombiningPeriodList_item_02 = -1;
static gint ett_rrc_MBMS_L1CombiningSchedule_256 = -1;
static gint ett_rrc_T_mtch_L1CombiningPeriodList_03 = -1;
static gint ett_rrc_T_mtch_L1CombiningPeriodList_item_03 = -1;
static gint ett_rrc_MBMS_L1CombiningSchedule_512 = -1;
static gint ett_rrc_T_mtch_L1CombiningPeriodList_04 = -1;
static gint ett_rrc_T_mtch_L1CombiningPeriodList_item_04 = -1;
static gint ett_rrc_MBMS_L1CombiningSchedule_1024 = -1;
static gint ett_rrc_T_mtch_L1CombiningPeriodList_05 = -1;
static gint ett_rrc_T_mtch_L1CombiningPeriodList_item_05 = -1;
static gint ett_rrc_MBMS_L1CombiningSchedule = -1;
static gint ett_rrc_MBMS_L23Configuration = -1;
static gint ett_rrc_T_sameAsCurrent = -1;
static gint ett_rrc_T_different = -1;
static gint ett_rrc_MBMS_MCCH_ConfigurationInfo_r6 = -1;
static gint ett_rrc_MBMS_MICHConfigurationInfo_r6 = -1;
static gint ett_rrc_T_mode_03 = -1;
static gint ett_rrc_T_fdd_191 = -1;
static gint ett_rrc_T_tdd384_44 = -1;
static gint ett_rrc_T_tdd128_70 = -1;
static gint ett_rrc_SEQUENCE_SIZE_1_2_OF_DL_TS_ChannelisationCode = -1;
static gint ett_rrc_MBMS_MICHConfigurationInfo_v770ext = -1;
static gint ett_rrc_T_mode_04 = -1;
static gint ett_rrc_T_tdd384_45 = -1;
static gint ett_rrc_T_tdd768_25 = -1;
static gint ett_rrc_T_tdd128_71 = -1;
static gint ett_rrc_MBMS_MICHConfigurationInfo_v890ext = -1;
static gint ett_rrc_T_mode_05 = -1;
static gint ett_rrc_T_imb384 = -1;
static gint ett_rrc_MBMS_ModifedService_r6 = -1;
static gint ett_rrc_T_mbms_PreferredFrequency = -1;
static gint ett_rrc_MBMS_ModifedServiceList_r6 = -1;
static gint ett_rrc_MBMS_ModifiedService_v770ext = -1;
static gint ett_rrc_MBMS_ModifiedService_LCR_v7c0ext = -1;
static gint ett_rrc_MBMS_ModifiedServiceList_v770ext = -1;
static gint ett_rrc_MBMS_ModifiedServiceList_LCR_v7c0ext = -1;
static gint ett_rrc_MBMS_MSCH_ConfigurationInfo_r6 = -1;
static gint ett_rrc_MBMS_MSCHSchedulingInfo = -1;
static gint ett_rrc_MBMS_NeighbouringCellSCCPCH_r6 = -1;
static gint ett_rrc_T_layer1Combining = -1;
static gint ett_rrc_T_fdd_192 = -1;
static gint ett_rrc_MBMS_NeighbouringCellSCCPCH_v770ext = -1;
static gint ett_rrc_T_layer1Combining_01 = -1;
static gint ett_rrc_T_fdd_193 = -1;
static gint ett_rrc_MBMS_NeighbouringCellSCCPCHList_r6 = -1;
static gint ett_rrc_MBMS_NeighbouringCellSCCPCHList_v770ext = -1;
static gint ett_rrc_MBMS_NetworkStandardTimeInformation_LCR_v890ext = -1;
static gint ett_rrc_MBMS_PhyChInformation_IMB384 = -1;
static gint ett_rrc_MBMS_PhyChInformationList_IMB384 = -1;
static gint ett_rrc_MBMS_PhyChInformation_r6 = -1;
static gint ett_rrc_MBMS_PhyChInformation_r7 = -1;
static gint ett_rrc_MBMS_PhyChInformationList_r6 = -1;
static gint ett_rrc_MBMS_PhyChInformationList_r7 = -1;
static gint ett_rrc_MBMS_PreferredFrequencyInfo_r6 = -1;
static gint ett_rrc_T_layerConvergenceInformation = -1;
static gint ett_rrc_MBMS_PreferredFrequencyList_r6 = -1;
static gint ett_rrc_MBMS_PTM_RBInformation_C = -1;
static gint ett_rrc_MBMS_PTM_RBInformation_CList = -1;
static gint ett_rrc_MBMS_PTM_RBInformation_N = -1;
static gint ett_rrc_MBMS_PTM_RBInformation_NList = -1;
static gint ett_rrc_MBMS_SelectedServiceInfo = -1;
static gint ett_rrc_T_status = -1;
static gint ett_rrc_MBMS_SelectedServicesListFull = -1;
static gint ett_rrc_MBMS_SelectedServicesListShort = -1;
static gint ett_rrc_MBMS_SelectedServicesShort = -1;
static gint ett_rrc_MBMS_ServiceAccessInfo_r6 = -1;
static gint ett_rrc_MBMS_ServiceAccessInfoList_r6 = -1;
static gint ett_rrc_MBMS_ServiceIdentity_r6 = -1;
static gint ett_rrc_T_plmn_Identity = -1;
static gint ett_rrc_T_other_01 = -1;
static gint ett_rrc_MBMS_ServiceSchedulingInfo_r6 = -1;
static gint ett_rrc_MBMS_ServiceSchedulingInfoList_r6 = -1;
static gint ett_rrc_MBMS_ServiceTransmInfo = -1;
static gint ett_rrc_MBMS_ServiceTransmInfoList = -1;
static gint ett_rrc_MBMS_SIBType5_SCCPCH_r6 = -1;
static gint ett_rrc_MBMS_SIBType5_SCCPCHList_r6 = -1;
static gint ett_rrc_MBMS_TimersAndCounters_r6 = -1;
static gint ett_rrc_MBMS_TransmissionIdentity = -1;
static gint ett_rrc_MBMS_TranspChInfoForCCTrCh_r6 = -1;
static gint ett_rrc_MBMS_TranspChInfoForEachCCTrCh_r6 = -1;
static gint ett_rrc_MBMS_TranspChInfoForEachTrCh_r6 = -1;
static gint ett_rrc_MBMS_TranspChInfoForTrCh_r6 = -1;
static gint ett_rrc_MBMS_TrCHInformation_Curr = -1;
static gint ett_rrc_MBMS_TrCHInformation_CurrList = -1;
static gint ett_rrc_MBMS_TrCHInformation_Neighb = -1;
static gint ett_rrc_MBMS_TrCHInformation_NeighbList = -1;
static gint ett_rrc_MBMS_TrCHInformation_SIB5 = -1;
static gint ett_rrc_MBMS_TrCHInformation_SIB5List = -1;
static gint ett_rrc_MBMS_UnmodifiedService_r6 = -1;
static gint ett_rrc_MBMS_UnmodifiedServiceList_r6 = -1;
static gint ett_rrc_MBMS_UnmodifiedService_v770ext = -1;
static gint ett_rrc_MBMS_UnmodifiedServiceList_v770ext = -1;
static gint ett_rrc_MBSFNFrequency = -1;
static gint ett_rrc_MBSFNFrequency_v860ext = -1;
static gint ett_rrc_MBSFNFrequencyList = -1;
static gint ett_rrc_MBSFNFrequencyList_v860ext = -1;
static gint ett_rrc_MBSFNservicesNotNotified_r7 = -1;
static gint ett_rrc_MBSFNInterFrequencyNeighbour_r7 = -1;
static gint ett_rrc_T_mbsfnServicesNotification = -1;
static gint ett_rrc_MBSFNInterFrequencyNeighbour_v860ext = -1;
static gint ett_rrc_MBSFN_InterFrequencyNeighbourList_r7 = -1;
static gint ett_rrc_MBSFN_InterFrequencyNeighbourList_v860ext = -1;
static gint ett_rrc_MBSFN_TDDInformation = -1;
static gint ett_rrc_MBSFN_TDDInformation_LCR = -1;
static gint ett_rrc_MBSFN_TDDTimeSlotInfo = -1;
static gint ett_rrc_MBSFN_TDDTimeSlotInfo_LCR = -1;
static gint ett_rrc_MBSFN_TDM_Info = -1;
static gint ett_rrc_MBSFN_TDM_Info_List = -1;
static gint ett_rrc_ToTargetRNC_Container = -1;
static gint ett_rrc_TargetRNC_ToSourceRNC_Container = -1;
static gint ett_rrc_InterRATHandoverInfoWithInterRATCapabilities_r3 = -1;
static gint ett_rrc_T_r3_31 = -1;
static gint ett_rrc_T_v390NonCriticalExtensions_01 = -1;
static gint ett_rrc_T_v690NonCriticalExtensions_52 = -1;
static gint ett_rrc_T_v860NonCriticalExtensions_35 = -1;
static gint ett_rrc_T_v920NonCriticalExtensions_08 = -1;
static gint ett_rrc_T_nonCriticalExtensions_238 = -1;
static gint ett_rrc_T_criticalExtensions_139 = -1;
static gint ett_rrc_InterRATHandoverInfoWithInterRATCapabilities_r3_IEs = -1;
static gint ett_rrc_InterRATHandoverInfoWithInterRATCapabilities_v390ext_IEs = -1;
static gint ett_rrc_InterRATHandoverInfoWithInterRATCapabilities_v690ext_IEs = -1;
static gint ett_rrc_InterRATHandoverInfoWithInterRATCapabilities_v860ext_IEs = -1;
static gint ett_rrc_InterRATHandoverInfoWithInterRATCapabilities_v920ext_IEs = -1;
static gint ett_rrc_RFC3095_ContextInfo_r5 = -1;
static gint ett_rrc_T_r5_14 = -1;
static gint ett_rrc_T_nonCriticalExtensions_239 = -1;
static gint ett_rrc_T_criticalExtensions_140 = -1;
static gint ett_rrc_RFC3095_ContextInfoList_r5 = -1;
static gint ett_rrc_SRNC_RelocationInfo_r3 = -1;
static gint ett_rrc_T_r3_32 = -1;
static gint ett_rrc_T_v380NonCriticalExtensions_02 = -1;
static gint ett_rrc_T_v390NonCriticalExtensions_02 = -1;
static gint ett_rrc_T_v3a0NonCriticalExtensions_16 = -1;
static gint ett_rrc_T_v3b0NonCriticalExtensions = -1;
static gint ett_rrc_T_v3c0NonCriticalExtensions = -1;
static gint ett_rrc_T_laterNonCriticalExtensions_67 = -1;
static gint ett_rrc_T_v3g0NonCriticalExtensions_02 = -1;
static gint ett_rrc_T_v4b0NonCriticalExtensions_27 = -1;
static gint ett_rrc_T_v590NonCriticalExtensions_31 = -1;
static gint ett_rrc_T_v5a0NonCriticalExtensions = -1;
static gint ett_rrc_T_v5b0NonCriticalExtensions_03 = -1;
static gint ett_rrc_T_v5c0NonCriticalExtensions_02 = -1;
static gint ett_rrc_T_v690NonCriticalExtensions_53 = -1;
static gint ett_rrc_T_v6b0NonCriticalExtensions_21 = -1;
static gint ett_rrc_T_v770NonCriticalExtensions_48 = -1;
static gint ett_rrc_T_v7e0NonCriticalExtensions_04 = -1;
static gint ett_rrc_T_v7f0NonCriticalExtensions_17 = -1;
static gint ett_rrc_T_nonCriticalExtensions_240 = -1;
static gint ett_rrc_T_later_than_r3_28 = -1;
static gint ett_rrc_T_r4_14 = -1;
static gint ett_rrc_T_v4d0NonCriticalExtensions_14 = -1;
static gint ett_rrc_T_v590NonCriticalExtensions_32 = -1;
static gint ett_rrc_T_v5a0NonCriticalExtensions_01 = -1;
static gint ett_rrc_T_v5b0NonCriticalExtensions_04 = -1;
static gint ett_rrc_T_v5c0NonCriticalExtensions_03 = -1;
static gint ett_rrc_T_v690NonCriticalExtensions_54 = -1;
static gint ett_rrc_T_v6b0NonCriticalExtensions_22 = -1;
static gint ett_rrc_T_v770NonCriticalExtensions_49 = -1;
static gint ett_rrc_T_v7e0NonCriticalExtensions_05 = -1;
static gint ett_rrc_T_v7f0NonCriticalExtensions_18 = -1;
static gint ett_rrc_T_nonCriticalExtensions_241 = -1;
static gint ett_rrc_T_criticalExtensions_141 = -1;
static gint ett_rrc_T_r5_15 = -1;
static gint ett_rrc_T_v5a0NonCriticalExtensions_02 = -1;
static gint ett_rrc_T_v5b0NonCriticalExtensions_05 = -1;
static gint ett_rrc_T_v5c0NonCriticalExtensions_04 = -1;
static gint ett_rrc_T_v690NonCriticalExtensions_55 = -1;
static gint ett_rrc_T_v6b0NonCriticalExtensions_23 = -1;
static gint ett_rrc_T_v770NonCriticalExtensions_50 = -1;
static gint ett_rrc_T_v7e0NonCriticalExtensions_06 = -1;
static gint ett_rrc_T_v7f0NonCriticalExtensions_19 = -1;
static gint ett_rrc_T_nonCriticalExtensions_242 = -1;
static gint ett_rrc_T_criticalExtensions_142 = -1;
static gint ett_rrc_T_r6_12 = -1;
static gint ett_rrc_T_v6b0NonCriticalExtensions_24 = -1;
static gint ett_rrc_T_v770NonCriticalExtensions_51 = -1;
static gint ett_rrc_T_v820NonCriticalExtensions_03 = -1;
static gint ett_rrc_T_v860NonCriticalExtensions_36 = -1;
static gint ett_rrc_T_nonCriticalExtensions_243 = -1;
static gint ett_rrc_T_criticalExtensions_143 = -1;
static gint ett_rrc_T_r7_16 = -1;
static gint ett_rrc_T_v820NonCriticalExtensions_04 = -1;
static gint ett_rrc_T_v860NonCriticalExtensions_37 = -1;
static gint ett_rrc_T_nonCriticalExtensions_244 = -1;
static gint ett_rrc_T_criticalExtensions_144 = -1;
static gint ett_rrc_T_r8_12 = -1;
static gint ett_rrc_T_v7e0NonCriticalExtensions_07 = -1;
static gint ett_rrc_T_v7f0NonCriticalExtensions_20 = -1;
static gint ett_rrc_T_v8d0NonCriticalExtensions_01 = -1;
static gint ett_rrc_T_nonCriticalExtensions_245 = -1;
static gint ett_rrc_T_criticalExtensions_145 = -1;
static gint ett_rrc_T_r9_11 = -1;
static gint ett_rrc_T_v8d0NonCriticalExtensions_02 = -1;
static gint ett_rrc_T_v970NonCriticalExtensions_04 = -1;
static gint ett_rrc_T_v9c0NonCriticalExtensions_13 = -1;
static gint ett_rrc_T_nonCriticalExtensions_246 = -1;
static gint ett_rrc_T_criticalExtensions_146 = -1;
static gint ett_rrc_T_r10_13 = -1;
static gint ett_rrc_T_nonCriticalExtensions_247 = -1;
static gint ett_rrc_T_criticalExtensions_147 = -1;
static gint ett_rrc_T_r11_14 = -1;
static gint ett_rrc_T_v9c0NonCriticalExtensions_14 = -1;
static gint ett_rrc_T_nonCriticalExtensions_248 = -1;
static gint ett_rrc_T_criticalExtensions_148 = -1;
static gint ett_rrc_SRNC_RelocationInfo_r3_IEs = -1;
static gint ett_rrc_T_modeSpecificInfo_156 = -1;
static gint ett_rrc_T_fdd_194 = -1;
static gint ett_rrc_SRNC_RelocationInfo_v380ext_IEs = -1;
static gint ett_rrc_SRNC_RelocationInfo_v390ext_IEs = -1;
static gint ett_rrc_SRNC_RelocationInfo_v3a0ext_IEs = -1;
static gint ett_rrc_SRNC_RelocationInfo_v3b0ext_IEs = -1;
static gint ett_rrc_SRNC_RelocationInfo_v3c0ext_IEs = -1;
static gint ett_rrc_SRNC_RelocationInfo_v3d0ext_IEs = -1;
static gint ett_rrc_SRNC_RelocationInfo_v3g0ext_IEs = -1;
static gint ett_rrc_SRNC_RelocationInfo_v3h0ext_IEs = -1;
static gint ett_rrc_T_nonCriticalExtension = -1;
static gint ett_rrc_SRNC_RelocationInfo_v4d0ext_IEs = -1;
static gint ett_rrc_TPC_CombinationInfoList = -1;
static gint ett_rrc_TPC_CombinationInfoList_r9 = -1;
static gint ett_rrc_STARTList2 = -1;
static gint ett_rrc_SRNC_RelocationInfo_v4b0ext_IEs = -1;
static gint ett_rrc_SRNC_RelocationInfo_v590ext_IEs = -1;
static gint ett_rrc_SRNC_RelocationInfo_v5a0ext_IEs = -1;
static gint ett_rrc_SRNC_RelocationInfo_v5b0ext_IEs = -1;
static gint ett_rrc_SRNC_RelocationInfo_v5c0ext_IEs = -1;
static gint ett_rrc_CipheringInfoPerRB_List_v3a0ext = -1;
static gint ett_rrc_CipheringStatusList = -1;
static gint ett_rrc_CipheringStatusCNdomain = -1;
static gint ett_rrc_CodeChangeStatusList = -1;
static gint ett_rrc_CodeChangeStatus = -1;
static gint ett_rrc_StoredCompressedModeInfo = -1;
static gint ett_rrc_StoredCompressedModeInfo_r8 = -1;
static gint ett_rrc_StoredTGP_SequenceList = -1;
static gint ett_rrc_StoredTGP_SequenceList_r8 = -1;
static gint ett_rrc_StoredTGP_Sequence = -1;
static gint ett_rrc_T_current_tgps_Status = -1;
static gint ett_rrc_T_active = -1;
static gint ett_rrc_StoredTGP_Sequence_r8 = -1;
static gint ett_rrc_T_current_tgps_Status_01 = -1;
static gint ett_rrc_T_active_01 = -1;
static gint ett_rrc_SRNC_RelocationInfo_r4_IEs = -1;
static gint ett_rrc_T_dummy_17 = -1;
static gint ett_rrc_T_fdd_195 = -1;
static gint ett_rrc_SRNC_RelocationInfo_r5_IEs = -1;
static gint ett_rrc_T_dummy_18 = -1;
static gint ett_rrc_T_fdd_196 = -1;
static gint ett_rrc_SRNC_RelocationInfo_v690ext_IEs = -1;
static gint ett_rrc_SRNC_RelocationInfo_r6_IEs = -1;
static gint ett_rrc_SRNC_RelocationInfo_r6_add_ext_IEs = -1;
static gint ett_rrc_T_v7f0NonCriticalExtensions_21 = -1;
static gint ett_rrc_T_nonCriticalExtensions_249 = -1;
static gint ett_rrc_SRNC_RelocationInfo_v6b0ext_IEs = -1;
static gint ett_rrc_SRNC_RelocationInfo_r7_IEs = -1;
static gint ett_rrc_SRNC_RelocationInfo_r7_add_ext_IEs = -1;
static gint ett_rrc_T_v7f0NonCriticalExtensions_22 = -1;
static gint ett_rrc_T_nonCriticalExtensions_250 = -1;
static gint ett_rrc_SRNC_RelocationInfo_v770ext_IEs = -1;
static gint ett_rrc_SRNC_RelocationInfo_v7e0ext_IEs = -1;
static gint ett_rrc_SRNC_RelocationInfo_v7f0ext_IEs = -1;
static gint ett_rrc_SRNC_RelocationInfo_r8_IEs = -1;
static gint ett_rrc_SRNC_RelocationInfo_v820ext_IEs = -1;
static gint ett_rrc_SRNC_RelocationInfo_v860ext_IEs = -1;
static gint ett_rrc_SRNC_RelocationInfo_v8d0ext_IEs = -1;
static gint ett_rrc_SRNC_RelocationInfo_r9_IEs = -1;
static gint ett_rrc_SRNC_RelocationInfo_v970ext_IEs = -1;
static gint ett_rrc_SRNC_RelocationInfo_v9c0ext_IEs = -1;
static gint ett_rrc_SRNC_RelocationInfo_r10_IEs = -1;
static gint ett_rrc_SRNC_RelocationInfo_r11_IEs = -1;
static gint ett_rrc_CalculationTimeForCiphering = -1;
static gint ett_rrc_CipheringInfoPerRB = -1;
static gint ett_rrc_CipheringInfoPerRB_r4 = -1;
static gint ett_rrc_CipheringInfoPerRB_List = -1;
static gint ett_rrc_CipheringInfoPerRB_List_r4 = -1;
static gint ett_rrc_CipheringStatusList_r4 = -1;
static gint ett_rrc_CipheringStatusCNdomain_r4 = -1;
static gint ett_rrc_CN_DomainInformation_v390ext = -1;
static gint ett_rrc_CN_DomainInformationList_v390ext = -1;
static gint ett_rrc_CompressedModeMeasCapability_r4 = -1;
static gint ett_rrc_COUNT_C_List = -1;
static gint ett_rrc_COUNT_CSingle = -1;
static gint ett_rrc_DL_PhysChCapabilityFDD_r4 = -1;
static gint ett_rrc_DL_PhysChCapabilityFDD_r5 = -1;
static gint ett_rrc_T_fdd_hspdsch_01 = -1;
static gint ett_rrc_T_supported_07 = -1;
static gint ett_rrc_DL_PhysChCapabilityTDD_r5 = -1;
static gint ett_rrc_T_tdd384_hspdsch_01 = -1;
static gint ett_rrc_DL_PhysChCapabilityTDD_LCR_r5 = -1;
static gint ett_rrc_T_tdd128_hspdsch_01 = -1;
static gint ett_rrc_DL_RFC3095_Context = -1;
static gint ett_rrc_E_RGCH_Combination_Info = -1;
static gint ett_rrc_E_RGCH_Combination_Info_r9 = -1;
static gint ett_rrc_E_RGCH_CombinationInfoList = -1;
static gint ett_rrc_E_RGCH_CombinationInfoList_r9 = -1;
static gint ett_rrc_InterRAT_UE_RadioAccessCapabilityList_r5 = -1;
static gint ett_rrc_IntraFreqReportingCriteria_r6_ext = -1;
static gint ett_rrc_MeasurementCapability_r4 = -1;
static gint ett_rrc_MeasurementCommandWithType = -1;
static gint ett_rrc_MeasurementCommandWithType_r4 = -1;
static gint ett_rrc_MeasurementCommandWithType_r6 = -1;
static gint ett_rrc_MeasurementCommandWithType_r7 = -1;
static gint ett_rrc_MeasurementCommandWithType_r8 = -1;
static gint ett_rrc_MeasurementCommandWithType_r9 = -1;
static gint ett_rrc_MeasurementCommandWithType_r10 = -1;
static gint ett_rrc_MeasurementCommandWithType_r11 = -1;
static gint ett_rrc_OngoingMeasRep = -1;
static gint ett_rrc_OngoingMeasRep_r4 = -1;
static gint ett_rrc_OngoingMeasRep_r5 = -1;
static gint ett_rrc_T_measurementCommand_v590ext_01 = -1;
static gint ett_rrc_OngoingMeasRep_r6 = -1;
static gint ett_rrc_OngoingMeasRep_r7 = -1;
static gint ett_rrc_OngoingMeasRep_r8 = -1;
static gint ett_rrc_OngoingMeasRep_r9 = -1;
static gint ett_rrc_OngoingMeasRep_v970ext_IEs = -1;
static gint ett_rrc_OngoingMeasRep_r10 = -1;
static gint ett_rrc_OngoingMeasRep_r11 = -1;
static gint ett_rrc_OngoingMeasRepList = -1;
static gint ett_rrc_OngoingMeasRepList_r4 = -1;
static gint ett_rrc_OngoingMeasRepList_r5 = -1;
static gint ett_rrc_OngoingMeasRepList_r6 = -1;
static gint ett_rrc_OngoingMeasRepList_r7 = -1;
static gint ett_rrc_OngoingMeasRepList_r8 = -1;
static gint ett_rrc_OngoingMeasRepList_r9 = -1;
static gint ett_rrc_OngoingMeasRepList_v970ext_IE = -1;
static gint ett_rrc_OngoingMeasRepList_r10 = -1;
static gint ett_rrc_OngoingMeasRepList_r11 = -1;
static gint ett_rrc_PDCP_Capability_r4 = -1;
static gint ett_rrc_T_supportForRfc2507_01 = -1;
static gint ett_rrc_T_supportForRfc3095_01 = -1;
static gint ett_rrc_T_supported_08 = -1;
static gint ett_rrc_PDCP_Capability_r5 = -1;
static gint ett_rrc_T_supportForRfc2507_02 = -1;
static gint ett_rrc_T_supportForRfc3095_02 = -1;
static gint ett_rrc_T_supported_09 = -1;
static gint ett_rrc_PDCP_Capability_r6 = -1;
static gint ett_rrc_T_supportForRfc2507_03 = -1;
static gint ett_rrc_T_supportForRfc3095_03 = -1;
static gint ett_rrc_T_supported_10 = -1;
static gint ett_rrc_PhysicalChannelCapability_r4 = -1;
static gint ett_rrc_T_fddPhysChCapability_06 = -1;
static gint ett_rrc_T_tdd384_PhysChCapability = -1;
static gint ett_rrc_T_tdd128_PhysChCapability_01 = -1;
static gint ett_rrc_PhysicalChannelCapability_r5 = -1;
static gint ett_rrc_T_fddPhysChCapability_07 = -1;
static gint ett_rrc_T_tdd384_PhysChCapability_01 = -1;
static gint ett_rrc_T_tdd128_PhysChCapability_02 = -1;
static gint ett_rrc_RF_Capability_r4 = -1;
static gint ett_rrc_T_fddRF_Capability_04 = -1;
static gint ett_rrc_T_tdd384_RF_Capability_01 = -1;
static gint ett_rrc_T_tdd128_RF_Capability_01 = -1;
static gint ett_rrc_RFC3095_ContextInfo = -1;
static gint ett_rrc_RFC3095_Context_List = -1;
static gint ett_rrc_RFC3095_Context_List_item = -1;
static gint ett_rrc_RLC_Capability_r5 = -1;
static gint ett_rrc_SRB_SpecificIntegrityProtInfo = -1;
static gint ett_rrc_SRB_SpecificIntegrityProtInfoList = -1;
static gint ett_rrc_TPC_Combination_Info = -1;
static gint ett_rrc_TPC_Combination_Info_r9 = -1;
static gint ett_rrc_UE_MultiModeRAT_Capability_r5 = -1;
static gint ett_rrc_UE_Positioning_Capability_r4 = -1;
static gint ett_rrc_UE_Positioning_LastKnownPos = -1;
static gint ett_rrc_UE_RadioAccessCapability_r4 = -1;
static gint ett_rrc_UE_RadioAccessCapability_r5 = -1;
static gint ett_rrc_UE_RadioAccessCapability_r6 = -1;
static gint ett_rrc_UL_RFC3095_Context = -1;

/*--- End of included file: packet-rrc-ett.c ---*/
#line 108 "../../asn1/rrc/packet-rrc-template.c"

static gint ett_rrc_eutraFeatureGroupIndicators = -1;
static gint ett_rrc_cn_CommonGSM_MAP_NAS_SysInfo = -1;

/* Global variables */
static proto_tree *top_tree;

static int hf_rrc_eutra_feat_group_ind_1 = -1;
static int hf_rrc_eutra_feat_group_ind_2 = -1;
static int hf_rrc_eutra_feat_group_ind_3 = -1;
static int hf_rrc_eutra_feat_group_ind_4 = -1;

static const true_false_string rrc_eutra_feat_group_ind_1_val = {
  "UTRA CELL_PCH to EUTRA RRC_IDLE cell reselection - Supported",
  "UTRA CELL_PCH to EUTRA RRC_IDLE cell reselection - Not supported"
};
static const true_false_string rrc_eutra_feat_group_ind_2_val = {
  "EUTRAN measurements and reporting in connected mode - Supported",
  "EUTRAN measurements and reporting in connected mode - Not supported"
};
static const true_false_string rrc_eutra_feat_group_ind_3_val = {
  "UTRA CELL_FACH absolute priority cell reselection for high priority layers - Supported",
  "UTRA CELL_FACH absolute priority cell reselection for high priority layers - Not supported"
};
static const true_false_string rrc_eutra_feat_group_ind_4_val = {
  "UTRA CELL_FACH absolute priority cell reselection for all layers - Supported",
  "UTRA CELL_FACH absolute priority cell reselection for all layers - Not supported"
};
static int flowd,type;

static int cipher_start_val[2] _U_;

/*Stores how many channels we have detected for a HS-DSCH MAC-flow*/
#define    RRC_MAX_NUM_HSDHSCH_MACDFLOW 8
static guint8 num_chans_per_flow[RRC_MAX_NUM_HSDHSCH_MACDFLOW];
static int rbid;
static int activation_frame;


/**
 * Return the maximum counter, useful for initiating counters
 */
#if 0
static int get_max_counter(int com_context){
    int i;
    guint32 max = 0;
    rrc_ciphering_info * c_inf;

    if( (c_inf = g_tree_lookup(rrc_ciph_inf, GINT_TO_POINTER((gint)com_context))) == NULL ){
        return 0;
    }
    for(i = 0; i<31; i++){
        max = MAX(c_inf->ps_conf_counters[i][0], max);
        max = MAX(c_inf->ps_conf_counters[i][1], max);
    }
    return max;
}
#endif
/** Utility functions used for various comparisons/cleanups in tree **/
gint rrc_key_cmp(gconstpointer b_ptr, gconstpointer a_ptr, gpointer ignore _U_){
    if( GPOINTER_TO_INT(a_ptr) > GPOINTER_TO_INT(b_ptr) ){
        return  -1;
    }
    return GPOINTER_TO_INT(a_ptr) < GPOINTER_TO_INT(b_ptr);
}
void rrc_free_key(gpointer key _U_){
            /*Keys should be de allocated elsewhere.*/

    }
void rrc_free_value(gpointer value ){
            g_free(value);
    }

/*--- Included file: packet-rrc-fn.c ---*/
#line 1 "../../asn1/rrc/packet-rrc-fn.c"
/*--- PDUs declarations ---*/
static int dissect_CellUpdate_r3_add_ext_IEs_PDU(tvbuff_t *tvb _U_, packet_info *pinfo _U_, proto_tree *tree _U_, void *data _U_);
static int dissect_CellUpdateConfirm_r7_add_ext_IEs_PDU(tvbuff_t *tvb _U_, packet_info *pinfo _U_, proto_tree *tree _U_, void *data _U_);
static int dissect_InitialDirectTransfer_r3_add_ext_IEs_PDU(tvbuff_t *tvb _U_, packet_info *pinfo _U_, proto_tree *tree _U_, void *data _U_);
static int dissect_InterRATHandoverInfo_r3_add_ext_IEs_PDU(tvbuff_t *tvb _U_, packet_info *pinfo _U_, proto_tree *tree _U_, void *data _U_);
static int dissect_RadioBearerSetup_r7_add_ext_IEs_PDU(tvbuff_t *tvb _U_, packet_info *pinfo _U_, proto_tree *tree _U_, void *data _U_);
static int dissect_RRCConnectionSetupComplete_r3_add_ext_IEs_PDU(tvbuff_t *tvb _U_, packet_info *pinfo _U_, proto_tree *tree _U_, void *data _U_);
static int dissect_UE_CapabilityContainer_IEs_PDU(tvbuff_t *tvb _U_, packet_info *pinfo _U_, proto_tree *tree _U_, void *data _U_);
static int dissect_UECapabilityInformation_r3_add_ext_IEs_PDU(tvbuff_t *tvb _U_, packet_info *pinfo _U_, proto_tree *tree _U_, void *data _U_);
static int dissect_URAUpdate_r3_add_ext_IEs_PDU(tvbuff_t *tvb _U_, packet_info *pinfo _U_, proto_tree *tree _U_, void *data _U_);
static int dissect_SRNC_RelocationInfo_v3h0ext_IEs_PDU(tvbuff_t *tvb _U_, packet_info *pinfo _U_, proto_tree *tree _U_, void *data _U_);
static int dissect_SRNC_RelocationInfo_r6_add_ext_IEs_PDU(tvbuff_t *tvb _U_, packet_info *pinfo _U_, proto_tree *tree _U_, void *data _U_);
static int dissect_SRNC_RelocationInfo_r7_add_ext_IEs_PDU(tvbuff_t *tvb _U_, packet_info *pinfo _U_, proto_tree *tree _U_, void *data _U_);
static int dissect_MeasurementReport_PDU(tvbuff_t *tvb _U_, packet_info *pinfo _U_, proto_tree *tree _U_, void *data _U_);



static int
dissect_rrc_MessageAuthenticationCode(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_bit_string(tvb, offset, actx, tree, hf_index,
                                     32, 32, FALSE, NULL);

  return offset;
}



static int
dissect_rrc_RRC_MessageSequenceNumber(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            0U, 15U, NULL, FALSE);

  return offset;
}


static const per_sequence_t IntegrityCheckInfo_sequence[] = {
  { &hf_rrc_messageAuthenticationCode, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MessageAuthenticationCode },
  { &hf_rrc_rrc_MessageSequenceNumber, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RRC_MessageSequenceNumber },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_IntegrityCheckInfo(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_IntegrityCheckInfo, IntegrityCheckInfo_sequence);

  return offset;
}



static int
dissect_rrc_RRC_TransactionIdentifier(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            0U, 3U, NULL, FALSE);

  return offset;
}



static int
dissect_rrc_IntegrityProtInitNumber(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_bit_string(tvb, offset, actx, tree, hf_index,
                                     32, 32, FALSE, NULL);

  return offset;
}


static const per_sequence_t T_startIntegrityProtection_sequence[] = {
  { &hf_rrc_integrityProtInitNumber, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_IntegrityProtInitNumber },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_startIntegrityProtection(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_startIntegrityProtection, T_startIntegrityProtection_sequence);

  return offset;
}


static const per_sequence_t RRC_MessageSequenceNumberList_sequence_of[1] = {
  { &hf_rrc_RRC_MessageSequenceNumberList_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RRC_MessageSequenceNumber },
};

static int
dissect_rrc_RRC_MessageSequenceNumberList(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_RRC_MessageSequenceNumberList, RRC_MessageSequenceNumberList_sequence_of,
                                                  4, 5, FALSE);

  return offset;
}


static const per_sequence_t IntegrityProtActivationInfo_sequence[] = {
  { &hf_rrc_rrc_MessageSequenceNumberList, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RRC_MessageSequenceNumberList },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_IntegrityProtActivationInfo(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_IntegrityProtActivationInfo, IntegrityProtActivationInfo_sequence);

  return offset;
}


static const per_sequence_t T_modify_sequence[] = {
  { &hf_rrc_dl_IntegrityProtActivationInfo, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_IntegrityProtActivationInfo },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_modify(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_modify, T_modify_sequence);

  return offset;
}


static const value_string rrc_IntegrityProtectionModeCommand_vals[] = {
  {   0, "startIntegrityProtection" },
  {   1, "modify" },
  { 0, NULL }
};

static const per_choice_t IntegrityProtectionModeCommand_choice[] = {
  {   0, &hf_rrc_startIntegrityProtection, ASN1_NO_EXTENSIONS     , dissect_rrc_T_startIntegrityProtection },
  {   1, &hf_rrc_modify          , ASN1_NO_EXTENSIONS     , dissect_rrc_T_modify },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_IntegrityProtectionModeCommand(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_IntegrityProtectionModeCommand, IntegrityProtectionModeCommand_choice,
                                 NULL);

  return offset;
}


static const value_string rrc_IntegrityProtectionAlgorithm_vals[] = {
  {   0, "uia1" },
  { 0, NULL }
};


static int
dissect_rrc_IntegrityProtectionAlgorithm(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t IntegrityProtectionModeInfo_sequence[] = {
  { &hf_rrc_integrityProtectionModeCommand, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_IntegrityProtectionModeCommand },
  { &hf_rrc_integrityProtectionAlgorithm, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_IntegrityProtectionAlgorithm },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_IntegrityProtectionModeInfo(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_IntegrityProtectionModeInfo, IntegrityProtectionModeInfo_sequence);

  return offset;
}


static const value_string rrc_CipheringAlgorithm_vals[] = {
  {   0, "uea0" },
  {   1, "uea1" },
  { 0, NULL }
};


static int
dissect_rrc_CipheringAlgorithm(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     2, NULL, FALSE, 0, NULL);

  return offset;
}



static int
dissect_rrc_NULL(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_null(tvb, offset, actx, tree, hf_index);

  return offset;
}


static const value_string rrc_CipheringModeCommand_vals[] = {
  {   0, "startRestart" },
  {   1, "dummy" },
  { 0, NULL }
};

static const per_choice_t CipheringModeCommand_choice[] = {
  {   0, &hf_rrc_startRestart    , ASN1_NO_EXTENSIONS     , dissect_rrc_CipheringAlgorithm },
  {   1, &hf_rrc_dummy_24        , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_CipheringModeCommand(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_CipheringModeCommand, CipheringModeCommand_choice,
                                 NULL);

  return offset;
}



static int
dissect_rrc_ActivationTime(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            0U, 255U, NULL, FALSE);

  return offset;
}



static int
dissect_rrc_RB_Identity(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
#line 870 "../../asn1/rrc/rrc.cnf"

  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            1U, 32U, &rbid, FALSE);






  return offset;
}



static int
dissect_rrc_RLC_SequenceNumber(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
#line 876 "../../asn1/rrc/rrc.cnf"

  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            0U, 4095U, &activation_frame, FALSE);




  return offset;
}


static const per_sequence_t RB_ActivationTimeInfo_sequence[] = {
  { &hf_rrc_rb_Identity     , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RB_Identity },
  { &hf_rrc_rlc_SequenceNumber, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RLC_SequenceNumber },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_RB_ActivationTimeInfo(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
#line 849 "../../asn1/rrc/rrc.cnf"
  fp_info       *fpinf;
  rrc_ciphering_info * c_inf;


  fpinf = p_get_proto_data(actx->pinfo->fd, proto_fp);

  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_RB_ActivationTimeInfo, RB_ActivationTimeInfo_sequence);


  /*If no info found, skip all this*/
  if(fpinf == NULL){
    return offset;
  }
  /*This should not happen*/
  if( (c_inf = g_tree_lookup(rrc_ciph_inf, GINT_TO_POINTER(fpinf->com_context_id))) == NULL ){
    return offset;
  }
  /*Set the ciphering activation frame information*/
  c_inf->seq_no[rbid][fpinf->is_uplink] = activation_frame;




  return offset;
}


static const per_sequence_t RB_ActivationTimeInfoList_sequence_of[1] = {
  { &hf_rrc_RB_ActivationTimeInfoList_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RB_ActivationTimeInfo },
};

static int
dissect_rrc_RB_ActivationTimeInfoList(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_RB_ActivationTimeInfoList, RB_ActivationTimeInfoList_sequence_of,
                                                  1, maxRB, FALSE);

  return offset;
}


static const per_sequence_t CipheringModeInfo_sequence[] = {
  { &hf_rrc_cipheringModeCommand, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CipheringModeCommand },
  { &hf_rrc_activationTimeForDPCH, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_ActivationTime },
  { &hf_rrc_rb_DL_CiphActivationTimeInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RB_ActivationTimeInfoList },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_CipheringModeInfo(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_CipheringModeInfo, CipheringModeInfo_sequence);

  return offset;
}



static int
dissect_rrc_SRNC_Identity(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_bit_string(tvb, offset, actx, tree, hf_index,
                                     12, 12, FALSE, NULL);

  return offset;
}



static int
dissect_rrc_S_RNTI(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_bit_string(tvb, offset, actx, tree, hf_index,
                                     20, 20, FALSE, NULL);

  return offset;
}


static const per_sequence_t U_RNTI_sequence[] = {
  { &hf_rrc_srnc_Identity   , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SRNC_Identity },
  { &hf_rrc_s_RNTI          , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_S_RNTI },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_U_RNTI(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_U_RNTI, U_RNTI_sequence);

  return offset;
}



static int
dissect_rrc_Digit(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            0U, 9U, NULL, FALSE);

  return offset;
}


static const per_sequence_t MCC_sequence_of[1] = {
  { &hf_rrc_MCC_item        , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_Digit },
};

static int
dissect_rrc_MCC(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_MCC, MCC_sequence_of,
                                                  3, 3, FALSE);

  return offset;
}


static const per_sequence_t MNC_sequence_of[1] = {
  { &hf_rrc_MNC_item        , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_Digit },
};

static int
dissect_rrc_MNC(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_MNC, MNC_sequence_of,
                                                  2, 3, FALSE);

  return offset;
}


static const per_sequence_t PLMN_Identity_sequence[] = {
  { &hf_rrc_mcc             , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MCC },
  { &hf_rrc_mnc             , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MNC },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_PLMN_Identity(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_PLMN_Identity, PLMN_Identity_sequence);

  return offset;
}



static int
dissect_rrc_NAS_SystemInformationGSM_MAP(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
#line 674 "../../asn1/rrc/rrc.cnf"
  tvbuff_t *nas_sys_info_gsm_map_tvb = NULL;
  guint32 length;
  proto_item *item;
  proto_tree *subtree;

  offset = dissect_per_octet_string(tvb, offset, actx, tree, hf_index,
                                       1, 8, FALSE, &nas_sys_info_gsm_map_tvb);

  length = tvb_length(nas_sys_info_gsm_map_tvb);
  if (length) {
    switch (rrc_nas_sys_info_gsm_map_type) {
      case RRC_NAS_SYS_INFO_CN_COMMON:
        item = proto_tree_add_text(tree, nas_sys_info_gsm_map_tvb, 0, length, "CN Common GSM-MAP NAS system information");
        subtree = proto_item_add_subtree(item, ett_rrc_cn_CommonGSM_MAP_NAS_SysInfo);
        de_cn_common_gsm_map_nas_sys_info(nas_sys_info_gsm_map_tvb, subtree, actx->pinfo, 0, length, NULL, 0);
        break;
      case RRC_NAS_SYS_INFO_CS:
        item = proto_tree_add_text(tree, nas_sys_info_gsm_map_tvb, 0, length, "CS domain specific system information");
        subtree = proto_item_add_subtree(item, ett_rrc_cn_CommonGSM_MAP_NAS_SysInfo);
        de_cs_domain_spec_sys_info(nas_sys_info_gsm_map_tvb, subtree, actx->pinfo, 0, length, NULL, 0);
        break;
      case RRC_NAS_SYS_INFO_PS:
        item = proto_tree_add_text(tree, nas_sys_info_gsm_map_tvb, 0, length, "PS domain specific system information");
        subtree = proto_item_add_subtree(item, ett_rrc_cn_CommonGSM_MAP_NAS_SysInfo);
        de_ps_domain_spec_sys_info(nas_sys_info_gsm_map_tvb, subtree, actx->pinfo, 0, length, NULL, 0);
        break;
      default:
        break;
    }
  }
  rrc_nas_sys_info_gsm_map_type = RRC_NAS_SYS_INFO_CN_COMMON;

/*Here we try to figure out which HS-DSCH channels are multiplexed*/


  return offset;
}



static int
dissect_rrc_T_cn_CommonGSM_MAP_NAS_SysInfo(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
#line 658 "../../asn1/rrc/rrc.cnf"
  rrc_nas_sys_info_gsm_map_type = RRC_NAS_SYS_INFO_CN_COMMON;
  offset = dissect_rrc_NAS_SystemInformationGSM_MAP(tvb, offset, actx, tree, hf_index);




  return offset;
}


static const value_string rrc_CN_DomainIdentity_vals[] = {
  {   0, "cs-domain" },
  {   1, "ps-domain" },
  { 0, NULL }
};


static int
dissect_rrc_CN_DomainIdentity(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
#line 655 "../../asn1/rrc/rrc.cnf"
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     2, &rrc_nas_sys_info_gsm_map_type, FALSE, 0, NULL);




  return offset;
}


static const per_sequence_t CN_DomainInformation_sequence[] = {
  { &hf_rrc_cn_DomainIdentity, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CN_DomainIdentity },
  { &hf_rrc_cn_DomainSpecificNAS_Info, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_NAS_SystemInformationGSM_MAP },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_CN_DomainInformation(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_CN_DomainInformation, CN_DomainInformation_sequence);

  return offset;
}


static const per_sequence_t CN_DomainInformationList_sequence_of[1] = {
  { &hf_rrc_CN_DomainInformationList_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CN_DomainInformation },
};

static int
dissect_rrc_CN_DomainInformationList(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_CN_DomainInformationList, CN_DomainInformationList_sequence_of,
                                                  1, maxCNdomains, FALSE);

  return offset;
}


static const per_sequence_t CN_InformationInfo_sequence[] = {
  { &hf_rrc_plmn_Identity   , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_PLMN_Identity },
  { &hf_rrc_cn_CommonGSM_MAP_NAS_SysInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_cn_CommonGSM_MAP_NAS_SysInfo },
  { &hf_rrc_cn_DomainInformationList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CN_DomainInformationList },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_CN_InformationInfo(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_CN_InformationInfo, CN_InformationInfo_sequence);

  return offset;
}



static int
dissect_rrc_PDCP_SN_Info(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            0U, 65535U, NULL, FALSE);

  return offset;
}


static const per_sequence_t RB_WithPDCP_Info_sequence[] = {
  { &hf_rrc_rb_Identity     , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RB_Identity },
  { &hf_rrc_pdcp_SN_Info    , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PDCP_SN_Info },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_RB_WithPDCP_Info(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_RB_WithPDCP_Info, RB_WithPDCP_Info_sequence);

  return offset;
}


static const per_sequence_t RB_WithPDCP_InfoList_sequence_of[1] = {
  { &hf_rrc_RB_WithPDCP_InfoList_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RB_WithPDCP_Info },
};

static int
dissect_rrc_RB_WithPDCP_InfoList(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_RB_WithPDCP_InfoList, RB_WithPDCP_InfoList_sequence_of,
                                                  1, maxRBallRABs, FALSE);

  return offset;
}


static const per_sequence_t DL_CounterSynchronisationInfo_sequence[] = {
  { &hf_rrc_rB_WithPDCP_InfoList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RB_WithPDCP_InfoList },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_DL_CounterSynchronisationInfo(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_DL_CounterSynchronisationInfo, DL_CounterSynchronisationInfo_sequence);

  return offset;
}



static int
dissect_rrc_MaxAllowedUL_TX_Power(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            -50, 33U, NULL, FALSE);

  return offset;
}



static int
dissect_rrc_PrimaryScramblingCode(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            0U, 511U, NULL, FALSE);

  return offset;
}


static const per_sequence_t PrimaryCPICH_Info_sequence[] = {
  { &hf_rrc_primaryScramblingCode, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PrimaryScramblingCode },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_PrimaryCPICH_Info(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_PrimaryCPICH_Info, PrimaryCPICH_Info_sequence);

  return offset;
}


static const value_string rrc_PCPICH_UsageForChannelEst_vals[] = {
  {   0, "mayBeUsed" },
  {   1, "shallNotBeUsed" },
  { 0, NULL }
};


static int
dissect_rrc_PCPICH_UsageForChannelEst(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     2, NULL, FALSE, 0, NULL);

  return offset;
}



static int
dissect_rrc_DPCH_FrameOffset(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            0U, 149U, NULL, FALSE);

  return offset;
}



static int
dissect_rrc_SecondaryScramblingCode(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            1U, 15U, NULL, FALSE);

  return offset;
}



static int
dissect_rrc_ChannelisationCode256(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            0U, 255U, NULL, FALSE);

  return offset;
}


static const per_sequence_t SecondaryCPICH_Info_sequence[] = {
  { &hf_rrc_secondaryDL_ScramblingCode, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_SecondaryScramblingCode },
  { &hf_rrc_channelisationCode, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_ChannelisationCode256 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_SecondaryCPICH_Info(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_SecondaryCPICH_Info, SecondaryCPICH_Info_sequence);

  return offset;
}



static int
dissect_rrc_INTEGER_0_3(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            0U, 3U, NULL, FALSE);

  return offset;
}



static int
dissect_rrc_INTEGER_0_7(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            0U, 7U, NULL, FALSE);

  return offset;
}



static int
dissect_rrc_INTEGER_0_15(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            0U, 15U, NULL, FALSE);

  return offset;
}



static int
dissect_rrc_INTEGER_0_31(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            0U, 31U, NULL, FALSE);

  return offset;
}



static int
dissect_rrc_INTEGER_0_63(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            0U, 63U, NULL, FALSE);

  return offset;
}



static int
dissect_rrc_INTEGER_0_127(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            0U, 127U, NULL, FALSE);

  return offset;
}



static int
dissect_rrc_INTEGER_0_255(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            0U, 255U, NULL, FALSE);

  return offset;
}



static int
dissect_rrc_INTEGER_0_511(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            0U, 511U, NULL, FALSE);

  return offset;
}


static const value_string rrc_SF512_AndCodeNumber_vals[] = {
  {   0, "sf4" },
  {   1, "sf8" },
  {   2, "sf16" },
  {   3, "sf32" },
  {   4, "sf64" },
  {   5, "sf128" },
  {   6, "sf256" },
  {   7, "sf512" },
  { 0, NULL }
};

static const per_choice_t SF512_AndCodeNumber_choice[] = {
  {   0, &hf_rrc_sf4             , ASN1_NO_EXTENSIONS     , dissect_rrc_INTEGER_0_3 },
  {   1, &hf_rrc_sf8             , ASN1_NO_EXTENSIONS     , dissect_rrc_INTEGER_0_7 },
  {   2, &hf_rrc_sf16            , ASN1_NO_EXTENSIONS     , dissect_rrc_INTEGER_0_15 },
  {   3, &hf_rrc_sf32            , ASN1_NO_EXTENSIONS     , dissect_rrc_INTEGER_0_31 },
  {   4, &hf_rrc_sf64            , ASN1_NO_EXTENSIONS     , dissect_rrc_INTEGER_0_63 },
  {   5, &hf_rrc_sf128           , ASN1_NO_EXTENSIONS     , dissect_rrc_INTEGER_0_127 },
  {   6, &hf_rrc_sf256           , ASN1_NO_EXTENSIONS     , dissect_rrc_INTEGER_0_255 },
  {   7, &hf_rrc_sf512           , ASN1_NO_EXTENSIONS     , dissect_rrc_INTEGER_0_511 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_SF512_AndCodeNumber(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_SF512_AndCodeNumber, SF512_AndCodeNumber_choice,
                                 NULL);

  return offset;
}


static const value_string rrc_ScramblingCodeChange_vals[] = {
  {   0, "codeChange" },
  {   1, "noCodeChange" },
  { 0, NULL }
};


static int
dissect_rrc_ScramblingCodeChange(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     2, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t DL_ChannelisationCode_sequence[] = {
  { &hf_rrc_secondaryScramblingCode, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_SecondaryScramblingCode },
  { &hf_rrc_sf_AndCodeNumber, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SF512_AndCodeNumber },
  { &hf_rrc_scramblingCodeChange, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_ScramblingCodeChange },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_DL_ChannelisationCode(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_DL_ChannelisationCode, DL_ChannelisationCode_sequence);

  return offset;
}


static const per_sequence_t DL_ChannelisationCodeList_sequence_of[1] = {
  { &hf_rrc_DL_ChannelisationCodeList_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DL_ChannelisationCode },
};

static int
dissect_rrc_DL_ChannelisationCodeList(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_DL_ChannelisationCodeList, DL_ChannelisationCodeList_sequence_of,
                                                  1, maxDPCH_DLchan, FALSE);

  return offset;
}



static int
dissect_rrc_TPC_CombinationIndex(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            0U, 5U, NULL, FALSE);

  return offset;
}


static const value_string rrc_SSDT_CellIdentity_vals[] = {
  {   0, "ssdt-id-a" },
  {   1, "ssdt-id-b" },
  {   2, "ssdt-id-c" },
  {   3, "ssdt-id-d" },
  {   4, "ssdt-id-e" },
  {   5, "ssdt-id-f" },
  {   6, "ssdt-id-g" },
  {   7, "ssdt-id-h" },
  { 0, NULL }
};


static int
dissect_rrc_SSDT_CellIdentity(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     8, NULL, FALSE, 0, NULL);

  return offset;
}


static const value_string rrc_ClosedLoopTimingAdjMode_vals[] = {
  {   0, "slot1" },
  {   1, "slot2" },
  { 0, NULL }
};


static int
dissect_rrc_ClosedLoopTimingAdjMode(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     2, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t T_fdd_70_sequence[] = {
  { &hf_rrc_pCPICH_UsageForChannelEst, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PCPICH_UsageForChannelEst },
  { &hf_rrc_dpch_FrameOffset, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DPCH_FrameOffset },
  { &hf_rrc_secondaryCPICH_Info, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_SecondaryCPICH_Info },
  { &hf_rrc_dl_ChannelisationCodeList, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DL_ChannelisationCodeList },
  { &hf_rrc_tpc_CombinationIndex, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TPC_CombinationIndex },
  { &hf_rrc_dummy_41        , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_SSDT_CellIdentity },
  { &hf_rrc_closedLoopTimingAdjMode, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_ClosedLoopTimingAdjMode },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_fdd_70(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_fdd_70, T_fdd_70_sequence);

  return offset;
}



static int
dissect_rrc_TFCS_IdentityPlain(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            1U, 8U, NULL, FALSE);

  return offset;
}



static int
dissect_rrc_DurationTimeInfo(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            1U, 4096U, NULL, FALSE);

  return offset;
}


static const per_sequence_t TimeInfo_sequence[] = {
  { &hf_rrc_activationTime  , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_ActivationTime },
  { &hf_rrc_durationTimeInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DurationTimeInfo },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_TimeInfo(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_TimeInfo, TimeInfo_sequence);

  return offset;
}


static const value_string rrc_SecondInterleavingMode_vals[] = {
  {   0, "frameRelated" },
  {   1, "timeslotRelated" },
  { 0, NULL }
};


static int
dissect_rrc_SecondInterleavingMode(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     2, NULL, FALSE, 0, NULL);

  return offset;
}


static const value_string rrc_TFCI_Coding_vals[] = {
  {   0, "tfci-bits-4" },
  {   1, "tfci-bits-8" },
  {   2, "tfci-bits-16" },
  {   3, "tfci-bits-32" },
  { 0, NULL }
};


static int
dissect_rrc_TFCI_Coding(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     4, NULL, FALSE, 0, NULL);

  return offset;
}


static const value_string rrc_PuncturingLimit_vals[] = {
  {   0, "pl0-40" },
  {   1, "pl0-44" },
  {   2, "pl0-48" },
  {   3, "pl0-52" },
  {   4, "pl0-56" },
  {   5, "pl0-60" },
  {   6, "pl0-64" },
  {   7, "pl0-68" },
  {   8, "pl0-72" },
  {   9, "pl0-76" },
  {  10, "pl0-80" },
  {  11, "pl0-84" },
  {  12, "pl0-88" },
  {  13, "pl0-92" },
  {  14, "pl0-96" },
  {  15, "pl1" },
  { 0, NULL }
};


static int
dissect_rrc_PuncturingLimit(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     16, NULL, FALSE, 0, NULL);

  return offset;
}



static int
dissect_rrc_INTEGER_1_1(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            1U, 1U, NULL, FALSE);

  return offset;
}



static int
dissect_rrc_INTEGER_1_3(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            1U, 3U, NULL, FALSE);

  return offset;
}



static int
dissect_rrc_INTEGER_1_7(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            1U, 7U, NULL, FALSE);

  return offset;
}



static int
dissect_rrc_INTEGER_1_15(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            1U, 15U, NULL, FALSE);

  return offset;
}



static int
dissect_rrc_INTEGER_1_31(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            1U, 31U, NULL, FALSE);

  return offset;
}



static int
dissect_rrc_INTEGER_1_63(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            1U, 63U, NULL, FALSE);

  return offset;
}


static const value_string rrc_RepetitionPeriodAndLength_vals[] = {
  {   0, "repetitionPeriod1" },
  {   1, "repetitionPeriod2" },
  {   2, "repetitionPeriod4" },
  {   3, "repetitionPeriod8" },
  {   4, "repetitionPeriod16" },
  {   5, "repetitionPeriod32" },
  {   6, "repetitionPeriod64" },
  { 0, NULL }
};

static const per_choice_t RepetitionPeriodAndLength_choice[] = {
  {   0, &hf_rrc_repetitionPeriod1, ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   1, &hf_rrc_repetitionPeriod2, ASN1_NO_EXTENSIONS     , dissect_rrc_INTEGER_1_1 },
  {   2, &hf_rrc_repetitionPeriod4, ASN1_NO_EXTENSIONS     , dissect_rrc_INTEGER_1_3 },
  {   3, &hf_rrc_repetitionPeriod8, ASN1_NO_EXTENSIONS     , dissect_rrc_INTEGER_1_7 },
  {   4, &hf_rrc_repetitionPeriod16, ASN1_NO_EXTENSIONS     , dissect_rrc_INTEGER_1_15 },
  {   5, &hf_rrc_repetitionPeriod32, ASN1_NO_EXTENSIONS     , dissect_rrc_INTEGER_1_31 },
  {   6, &hf_rrc_repetitionPeriod64, ASN1_NO_EXTENSIONS     , dissect_rrc_INTEGER_1_63 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_RepetitionPeriodAndLength(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_RepetitionPeriodAndLength, RepetitionPeriodAndLength_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t CommonTimeslotInfo_sequence[] = {
  { &hf_rrc_secondInterleavingMode, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SecondInterleavingMode },
  { &hf_rrc_tfci_Coding     , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_TFCI_Coding },
  { &hf_rrc_puncturingLimit , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PuncturingLimit },
  { &hf_rrc_repetitionPeriodAndLength, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RepetitionPeriodAndLength },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_CommonTimeslotInfo(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_CommonTimeslotInfo, CommonTimeslotInfo_sequence);

  return offset;
}



static int
dissect_rrc_TimeslotNumber(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            0U, 14U, NULL, FALSE);

  return offset;
}



static int
dissect_rrc_BOOLEAN(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_boolean(tvb, offset, actx, tree, hf_index, NULL);

  return offset;
}


static const value_string rrc_MidambleConfigurationBurstType1and3_vals[] = {
  {   0, "ms4" },
  {   1, "ms8" },
  {   2, "ms16" },
  { 0, NULL }
};


static int
dissect_rrc_MidambleConfigurationBurstType1and3(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     3, NULL, FALSE, 0, NULL);

  return offset;
}



static int
dissect_rrc_MidambleShiftLong(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            0U, 15U, NULL, FALSE);

  return offset;
}


static const per_sequence_t T_ueSpecificMidamble_07_sequence[] = {
  { &hf_rrc_midambleShift   , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MidambleShiftLong },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_ueSpecificMidamble_07(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_ueSpecificMidamble_07, T_ueSpecificMidamble_07_sequence);

  return offset;
}


static const value_string rrc_T_midambleAllocationMode_15_vals[] = {
  {   0, "defaultMidamble" },
  {   1, "commonMidamble" },
  {   2, "ueSpecificMidamble" },
  { 0, NULL }
};

static const per_choice_t T_midambleAllocationMode_15_choice[] = {
  {   0, &hf_rrc_defaultMidamble , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   1, &hf_rrc_commonMidamble  , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   2, &hf_rrc_ueSpecificMidamble_08, ASN1_NO_EXTENSIONS     , dissect_rrc_T_ueSpecificMidamble_07 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_midambleAllocationMode_15(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_midambleAllocationMode_15, T_midambleAllocationMode_15_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t T_type1_01_sequence[] = {
  { &hf_rrc_midambleConfigurationBurstType1and3, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MidambleConfigurationBurstType1and3 },
  { &hf_rrc_midambleAllocationMode_15, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_midambleAllocationMode_15 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_type1_01(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_type1_01, T_type1_01_sequence);

  return offset;
}


static const value_string rrc_MidambleConfigurationBurstType2_vals[] = {
  {   0, "ms3" },
  {   1, "ms6" },
  { 0, NULL }
};


static int
dissect_rrc_MidambleConfigurationBurstType2(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     2, NULL, FALSE, 0, NULL);

  return offset;
}



static int
dissect_rrc_MidambleShiftShort(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            0U, 5U, NULL, FALSE);

  return offset;
}


static const per_sequence_t T_ueSpecificMidamble_08_sequence[] = {
  { &hf_rrc_midambleShift_02, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MidambleShiftShort },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_ueSpecificMidamble_08(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_ueSpecificMidamble_08, T_ueSpecificMidamble_08_sequence);

  return offset;
}


static const value_string rrc_T_midambleAllocationMode_16_vals[] = {
  {   0, "defaultMidamble" },
  {   1, "commonMidamble" },
  {   2, "ueSpecificMidamble" },
  { 0, NULL }
};

static const per_choice_t T_midambleAllocationMode_16_choice[] = {
  {   0, &hf_rrc_defaultMidamble , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   1, &hf_rrc_commonMidamble  , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   2, &hf_rrc_ueSpecificMidamble_09, ASN1_NO_EXTENSIONS     , dissect_rrc_T_ueSpecificMidamble_08 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_midambleAllocationMode_16(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_midambleAllocationMode_16, T_midambleAllocationMode_16_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t T_type2_sequence[] = {
  { &hf_rrc_midambleConfigurationBurstType2, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MidambleConfigurationBurstType2 },
  { &hf_rrc_midambleAllocationMode_16, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_midambleAllocationMode_16 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_type2(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_type2, T_type2_sequence);

  return offset;
}


static const per_sequence_t T_ueSpecificMidamble_09_sequence[] = {
  { &hf_rrc_midambleShift   , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MidambleShiftLong },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_ueSpecificMidamble_09(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_ueSpecificMidamble_09, T_ueSpecificMidamble_09_sequence);

  return offset;
}


static const value_string rrc_T_midambleAllocationMode_17_vals[] = {
  {   0, "defaultMidamble" },
  {   1, "ueSpecificMidamble" },
  { 0, NULL }
};

static const per_choice_t T_midambleAllocationMode_17_choice[] = {
  {   0, &hf_rrc_defaultMidamble , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   1, &hf_rrc_ueSpecificMidamble_10, ASN1_NO_EXTENSIONS     , dissect_rrc_T_ueSpecificMidamble_09 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_midambleAllocationMode_17(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_midambleAllocationMode_17, T_midambleAllocationMode_17_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t T_type3_sequence[] = {
  { &hf_rrc_midambleConfigurationBurstType1and3, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MidambleConfigurationBurstType1and3 },
  { &hf_rrc_midambleAllocationMode_17, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_midambleAllocationMode_17 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_type3(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_type3, T_type3_sequence);

  return offset;
}


static const value_string rrc_T_burstType_vals[] = {
  {   0, "type1" },
  {   1, "type2" },
  {   2, "type3" },
  { 0, NULL }
};

static const per_choice_t T_burstType_choice[] = {
  {   0, &hf_rrc_type1_01        , ASN1_NO_EXTENSIONS     , dissect_rrc_T_type1_01 },
  {   1, &hf_rrc_type2           , ASN1_NO_EXTENSIONS     , dissect_rrc_T_type2 },
  {   2, &hf_rrc_type3           , ASN1_NO_EXTENSIONS     , dissect_rrc_T_type3 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_burstType(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_burstType, T_burstType_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t MidambleShiftAndBurstType_sequence[] = {
  { &hf_rrc_burstType_01    , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_burstType },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_MidambleShiftAndBurstType(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_MidambleShiftAndBurstType, MidambleShiftAndBurstType_sequence);

  return offset;
}


static const per_sequence_t IndividualTimeslotInfo_sequence[] = {
  { &hf_rrc_timeslotNumber  , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TimeslotNumber },
  { &hf_rrc_tfci_Existence  , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { &hf_rrc_midambleShiftAndBurstType_04, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MidambleShiftAndBurstType },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_IndividualTimeslotInfo(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_IndividualTimeslotInfo, IndividualTimeslotInfo_sequence);

  return offset;
}


static const value_string rrc_DL_TS_ChannelisationCode_vals[] = {
  {   0, "cc16-1" },
  {   1, "cc16-2" },
  {   2, "cc16-3" },
  {   3, "cc16-4" },
  {   4, "cc16-5" },
  {   5, "cc16-6" },
  {   6, "cc16-7" },
  {   7, "cc16-8" },
  {   8, "cc16-9" },
  {   9, "cc16-10" },
  {  10, "cc16-11" },
  {  11, "cc16-12" },
  {  12, "cc16-13" },
  {  13, "cc16-14" },
  {  14, "cc16-15" },
  {  15, "cc16-16" },
  { 0, NULL }
};


static int
dissect_rrc_DL_TS_ChannelisationCode(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     16, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t T_consecutive_sequence[] = {
  { &hf_rrc_firstChannelisationCode, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DL_TS_ChannelisationCode },
  { &hf_rrc_lastChannelisationCode, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DL_TS_ChannelisationCode },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_consecutive(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_consecutive, T_consecutive_sequence);

  return offset;
}



static int
dissect_rrc_T_bitmap(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_bit_string(tvb, offset, actx, tree, hf_index,
                                     16, 16, FALSE, NULL);

  return offset;
}


static const value_string rrc_T_codesRepresentation_vals[] = {
  {   0, "consecutive" },
  {   1, "bitmap" },
  { 0, NULL }
};

static const per_choice_t T_codesRepresentation_choice[] = {
  {   0, &hf_rrc_consecutive     , ASN1_NO_EXTENSIONS     , dissect_rrc_T_consecutive },
  {   1, &hf_rrc_bitmap          , ASN1_NO_EXTENSIONS     , dissect_rrc_T_bitmap },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_codesRepresentation(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_codesRepresentation, T_codesRepresentation_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t DL_TS_ChannelisationCodesShort_sequence[] = {
  { &hf_rrc_codesRepresentation, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_codesRepresentation },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_DL_TS_ChannelisationCodesShort(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_DL_TS_ChannelisationCodesShort, DL_TS_ChannelisationCodesShort_sequence);

  return offset;
}



static int
dissect_rrc_INTEGER_1_maxTS_1(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            1U, maxTS_1, NULL, FALSE);

  return offset;
}


static const per_sequence_t T_sameAsLast_sequence[] = {
  { &hf_rrc_timeslotNumber  , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TimeslotNumber },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_sameAsLast(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_sameAsLast, T_sameAsLast_sequence);

  return offset;
}


static const per_sequence_t T_newParameters_sequence[] = {
  { &hf_rrc_individualTimeslotInfo, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_IndividualTimeslotInfo },
  { &hf_rrc_dl_TS_ChannelisationCodesShort, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DL_TS_ChannelisationCodesShort },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_newParameters(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_newParameters, T_newParameters_sequence);

  return offset;
}


static const value_string rrc_T_parameters_vals[] = {
  {   0, "sameAsLast" },
  {   1, "newParameters" },
  { 0, NULL }
};

static const per_choice_t T_parameters_choice[] = {
  {   0, &hf_rrc_sameAsLast      , ASN1_NO_EXTENSIONS     , dissect_rrc_T_sameAsLast },
  {   1, &hf_rrc_newParameters   , ASN1_NO_EXTENSIONS     , dissect_rrc_T_newParameters },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_parameters(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_parameters, T_parameters_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t DownlinkAdditionalTimeslots_sequence[] = {
  { &hf_rrc_parameters      , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_parameters },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_DownlinkAdditionalTimeslots(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_DownlinkAdditionalTimeslots, DownlinkAdditionalTimeslots_sequence);

  return offset;
}


static const per_sequence_t SEQUENCE_SIZE_1_maxTS_1_OF_DownlinkAdditionalTimeslots_sequence_of[1] = {
  { &hf_rrc_timeslotList_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DownlinkAdditionalTimeslots },
};

static int
dissect_rrc_SEQUENCE_SIZE_1_maxTS_1_OF_DownlinkAdditionalTimeslots(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_SEQUENCE_SIZE_1_maxTS_1_OF_DownlinkAdditionalTimeslots, SEQUENCE_SIZE_1_maxTS_1_OF_DownlinkAdditionalTimeslots_sequence_of,
                                                  1, maxTS_1, FALSE);

  return offset;
}


static const value_string rrc_T_additionalTimeslots_vals[] = {
  {   0, "consecutive" },
  {   1, "timeslotList" },
  { 0, NULL }
};

static const per_choice_t T_additionalTimeslots_choice[] = {
  {   0, &hf_rrc_consecutive_02  , ASN1_NO_EXTENSIONS     , dissect_rrc_INTEGER_1_maxTS_1 },
  {   1, &hf_rrc_timeslotList    , ASN1_NO_EXTENSIONS     , dissect_rrc_SEQUENCE_SIZE_1_maxTS_1_OF_DownlinkAdditionalTimeslots },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_additionalTimeslots(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_additionalTimeslots, T_additionalTimeslots_choice,
                                 NULL);

  return offset;
}


static const value_string rrc_T_moreTimeslots_vals[] = {
  {   0, "noMore" },
  {   1, "additionalTimeslots" },
  { 0, NULL }
};

static const per_choice_t T_moreTimeslots_choice[] = {
  {   0, &hf_rrc_noMore          , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   1, &hf_rrc_additionalTimeslots, ASN1_NO_EXTENSIONS     , dissect_rrc_T_additionalTimeslots },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_moreTimeslots(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_moreTimeslots, T_moreTimeslots_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t DownlinkTimeslotsCodes_sequence[] = {
  { &hf_rrc_firstIndividualTimeslotInfo, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_IndividualTimeslotInfo },
  { &hf_rrc_dl_TS_ChannelisationCodesShort, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DL_TS_ChannelisationCodesShort },
  { &hf_rrc_moreTimeslots   , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_moreTimeslots },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_DownlinkTimeslotsCodes(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_DownlinkTimeslotsCodes, DownlinkTimeslotsCodes_sequence);

  return offset;
}


static const per_sequence_t TFCS_Identity_sequence[] = {
  { &hf_rrc_tfcs_ID_01      , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_TFCS_IdentityPlain },
  { &hf_rrc_sharedChannelIndicator, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_TFCS_Identity(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_TFCS_Identity, TFCS_Identity_sequence);

  return offset;
}


static const per_sequence_t UL_CCTrChTPCList_sequence_of[1] = {
  { &hf_rrc_UL_CCTrChTPCList_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TFCS_Identity },
};

static int
dissect_rrc_UL_CCTrChTPCList(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_UL_CCTrChTPCList, UL_CCTrChTPCList_sequence_of,
                                                  0, maxCCTrCH, FALSE);

  return offset;
}


static const per_sequence_t DL_CCTrCh_sequence[] = {
  { &hf_rrc_tfcs_ID_01      , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_TFCS_IdentityPlain },
  { &hf_rrc_timeInfo        , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TimeInfo },
  { &hf_rrc_commonTimeslotInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CommonTimeslotInfo },
  { &hf_rrc_dl_CCTrCH_TimeslotsCodes, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DownlinkTimeslotsCodes },
  { &hf_rrc_ul_CCTrChTPCList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_CCTrChTPCList },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_DL_CCTrCh(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_DL_CCTrCh, DL_CCTrCh_sequence);

  return offset;
}


static const per_sequence_t DL_CCTrChList_sequence_of[1] = {
  { &hf_rrc_DL_CCTrChList_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DL_CCTrCh },
};

static int
dissect_rrc_DL_CCTrChList(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_DL_CCTrChList, DL_CCTrChList_sequence_of,
                                                  1, maxCCTrCH, FALSE);

  return offset;
}


static const per_sequence_t DL_CCTrChListToRemove_sequence_of[1] = {
  { &hf_rrc_DL_CCTrChListToRemove_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TFCS_IdentityPlain },
};

static int
dissect_rrc_DL_CCTrChListToRemove(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_DL_CCTrChListToRemove, DL_CCTrChListToRemove_sequence_of,
                                                  1, maxCCTrCH, FALSE);

  return offset;
}


static const per_sequence_t T_tdd_32_sequence[] = {
  { &hf_rrc_dl_CCTrChListToEstablish, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_CCTrChList },
  { &hf_rrc_dl_CCTrChListToRemove, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_CCTrChListToRemove },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_tdd_32(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_tdd_32, T_tdd_32_sequence);

  return offset;
}


static const value_string rrc_DL_DPCH_InfoPerRL_vals[] = {
  {   0, "fdd" },
  {   1, "tdd" },
  { 0, NULL }
};

static const per_choice_t DL_DPCH_InfoPerRL_choice[] = {
  {   0, &hf_rrc_fdd_71          , ASN1_NO_EXTENSIONS     , dissect_rrc_T_fdd_70 },
  {   1, &hf_rrc_tdd_33          , ASN1_NO_EXTENSIONS     , dissect_rrc_T_tdd_32 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_DL_DPCH_InfoPerRL(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_DL_DPCH_InfoPerRL, DL_DPCH_InfoPerRL_choice,
                                 NULL);

  return offset;
}


static const value_string rrc_SF256_AndCodeNumber_vals[] = {
  {   0, "sf4" },
  {   1, "sf8" },
  {   2, "sf16" },
  {   3, "sf32" },
  {   4, "sf64" },
  {   5, "sf128" },
  {   6, "sf256" },
  { 0, NULL }
};

static const per_choice_t SF256_AndCodeNumber_choice[] = {
  {   0, &hf_rrc_sf4             , ASN1_NO_EXTENSIONS     , dissect_rrc_INTEGER_0_3 },
  {   1, &hf_rrc_sf8             , ASN1_NO_EXTENSIONS     , dissect_rrc_INTEGER_0_7 },
  {   2, &hf_rrc_sf16            , ASN1_NO_EXTENSIONS     , dissect_rrc_INTEGER_0_15 },
  {   3, &hf_rrc_sf32            , ASN1_NO_EXTENSIONS     , dissect_rrc_INTEGER_0_31 },
  {   4, &hf_rrc_sf64            , ASN1_NO_EXTENSIONS     , dissect_rrc_INTEGER_0_63 },
  {   5, &hf_rrc_sf128           , ASN1_NO_EXTENSIONS     , dissect_rrc_INTEGER_0_127 },
  {   6, &hf_rrc_sf256           , ASN1_NO_EXTENSIONS     , dissect_rrc_INTEGER_0_255 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_SF256_AndCodeNumber(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_SF256_AndCodeNumber, SF256_AndCodeNumber_choice,
                                 NULL);

  return offset;
}


static const value_string rrc_PositionFixedOrFlexible_vals[] = {
  {   0, "fixed" },
  {   1, "flexible" },
  { 0, NULL }
};


static int
dissect_rrc_PositionFixedOrFlexible(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     2, NULL, FALSE, 0, NULL);

  return offset;
}



static int
dissect_rrc_TimingOffset(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            0U, 149U, NULL, FALSE);

  return offset;
}


static const per_sequence_t T_fdd_109_sequence[] = {
  { &hf_rrc_dummy1_03       , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PCPICH_UsageForChannelEst },
  { &hf_rrc_dummy2_09       , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_SecondaryCPICH_Info },
  { &hf_rrc_secondaryScramblingCode, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_SecondaryScramblingCode },
  { &hf_rrc_sttd_Indicator  , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { &hf_rrc_sf_AndCodeNumber_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SF256_AndCodeNumber },
  { &hf_rrc_pilotSymbolExistence, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { &hf_rrc_tfci_Existence  , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { &hf_rrc_positionFixedOrFlexible, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PositionFixedOrFlexible },
  { &hf_rrc_timingOffset    , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_TimingOffset },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_fdd_109(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_fdd_109, T_fdd_109_sequence);

  return offset;
}



static int
dissect_rrc_INTEGER_0_1(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            0U, 1U, NULL, FALSE);

  return offset;
}


static const per_sequence_t T_repetitionPeriod2_sequence[] = {
  { &hf_rrc_length          , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_NULL },
  { &hf_rrc_offset          , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_0_1 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_repetitionPeriod2(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_repetitionPeriod2, T_repetitionPeriod2_sequence);

  return offset;
}


static const per_sequence_t T_repetitionPeriod4_sequence[] = {
  { &hf_rrc_length_01       , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_1_3 },
  { &hf_rrc_offset_01       , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_0_3 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_repetitionPeriod4(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_repetitionPeriod4, T_repetitionPeriod4_sequence);

  return offset;
}


static const per_sequence_t T_repetitionPeriod8_sequence[] = {
  { &hf_rrc_length_02       , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_1_7 },
  { &hf_rrc_offset_02       , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_0_7 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_repetitionPeriod8(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_repetitionPeriod8, T_repetitionPeriod8_sequence);

  return offset;
}


static const per_sequence_t T_repetitionPeriod16_sequence[] = {
  { &hf_rrc_length_03       , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_1_15 },
  { &hf_rrc_offset_03       , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_0_15 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_repetitionPeriod16(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_repetitionPeriod16, T_repetitionPeriod16_sequence);

  return offset;
}


static const per_sequence_t T_repetitionPeriod32_sequence[] = {
  { &hf_rrc_length_04       , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_1_31 },
  { &hf_rrc_offset_04       , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_0_31 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_repetitionPeriod32(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_repetitionPeriod32, T_repetitionPeriod32_sequence);

  return offset;
}


static const per_sequence_t T_repetitionPeriod64_sequence[] = {
  { &hf_rrc_length_05       , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_1_63 },
  { &hf_rrc_offset_05       , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_0_63 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_repetitionPeriod64(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_repetitionPeriod64, T_repetitionPeriod64_sequence);

  return offset;
}


static const value_string rrc_RepetitionPeriodLengthAndOffset_vals[] = {
  {   0, "repetitionPeriod1" },
  {   1, "repetitionPeriod2" },
  {   2, "repetitionPeriod4" },
  {   3, "repetitionPeriod8" },
  {   4, "repetitionPeriod16" },
  {   5, "repetitionPeriod32" },
  {   6, "repetitionPeriod64" },
  { 0, NULL }
};

static const per_choice_t RepetitionPeriodLengthAndOffset_choice[] = {
  {   0, &hf_rrc_repetitionPeriod1, ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   1, &hf_rrc_repetitionPeriod2_01, ASN1_NO_EXTENSIONS     , dissect_rrc_T_repetitionPeriod2 },
  {   2, &hf_rrc_repetitionPeriod4_01, ASN1_NO_EXTENSIONS     , dissect_rrc_T_repetitionPeriod4 },
  {   3, &hf_rrc_repetitionPeriod8_01, ASN1_NO_EXTENSIONS     , dissect_rrc_T_repetitionPeriod8 },
  {   4, &hf_rrc_repetitionPeriod16_01, ASN1_NO_EXTENSIONS     , dissect_rrc_T_repetitionPeriod16 },
  {   5, &hf_rrc_repetitionPeriod32_01, ASN1_NO_EXTENSIONS     , dissect_rrc_T_repetitionPeriod32 },
  {   6, &hf_rrc_repetitionPeriod64_01, ASN1_NO_EXTENSIONS     , dissect_rrc_T_repetitionPeriod64 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_RepetitionPeriodLengthAndOffset(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_RepetitionPeriodLengthAndOffset, RepetitionPeriodLengthAndOffset_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t CommonTimeslotInfoSCCPCH_sequence[] = {
  { &hf_rrc_secondInterleavingMode, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SecondInterleavingMode },
  { &hf_rrc_tfci_Coding     , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_TFCI_Coding },
  { &hf_rrc_puncturingLimit , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PuncturingLimit },
  { &hf_rrc_repetitionPeriodLengthAndOffset, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RepetitionPeriodLengthAndOffset },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_CommonTimeslotInfoSCCPCH(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_CommonTimeslotInfoSCCPCH, CommonTimeslotInfoSCCPCH_sequence);

  return offset;
}


static const value_string rrc_SCCPCH_ChannelisationCode_vals[] = {
  {   0, "cc16-1" },
  {   1, "cc16-2" },
  {   2, "cc16-3" },
  {   3, "cc16-4" },
  {   4, "cc16-5" },
  {   5, "cc16-6" },
  {   6, "cc16-7" },
  {   7, "cc16-8" },
  {   8, "cc16-9" },
  {   9, "cc16-10" },
  {  10, "cc16-11" },
  {  11, "cc16-12" },
  {  12, "cc16-13" },
  {  13, "cc16-14" },
  {  14, "cc16-15" },
  {  15, "cc16-16" },
  { 0, NULL }
};


static int
dissect_rrc_SCCPCH_ChannelisationCode(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     16, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t SCCPCH_ChannelisationCodeList_sequence_of[1] = {
  { &hf_rrc_SCCPCH_ChannelisationCodeList_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SCCPCH_ChannelisationCode },
};

static int
dissect_rrc_SCCPCH_ChannelisationCodeList(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_SCCPCH_ChannelisationCodeList, SCCPCH_ChannelisationCodeList_sequence_of,
                                                  1, 16, FALSE);

  return offset;
}


static const per_sequence_t T_tdd_62_sequence[] = {
  { &hf_rrc_commonTimeslotInfo_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CommonTimeslotInfoSCCPCH },
  { &hf_rrc_individualTimeslotInfo, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_IndividualTimeslotInfo },
  { &hf_rrc_channelisationCode_09, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SCCPCH_ChannelisationCodeList },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_tdd_62(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_tdd_62, T_tdd_62_sequence);

  return offset;
}


static const value_string rrc_T_modeSpecificInfo_76_vals[] = {
  {   0, "fdd" },
  {   1, "tdd" },
  { 0, NULL }
};

static const per_choice_t T_modeSpecificInfo_76_choice[] = {
  {   0, &hf_rrc_fdd_112         , ASN1_NO_EXTENSIONS     , dissect_rrc_T_fdd_109 },
  {   1, &hf_rrc_tdd_68          , ASN1_NO_EXTENSIONS     , dissect_rrc_T_tdd_62 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_modeSpecificInfo_76(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_modeSpecificInfo_76, T_modeSpecificInfo_76_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t SecondaryCCPCH_Info_sequence[] = {
  { &hf_rrc_modeSpecificInfo_76, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_modeSpecificInfo_76 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_SecondaryCCPCH_Info(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_SecondaryCCPCH_Info, SecondaryCCPCH_Info_sequence);

  return offset;
}



static int
dissect_rrc_GainFactor(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            0U, 15U, NULL, FALSE);

  return offset;
}


static const per_sequence_t T_fdd_53_sequence[] = {
  { &hf_rrc_gainFactorBetaC , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_GainFactor },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_fdd_53(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_fdd_53, T_fdd_53_sequence);

  return offset;
}


static const value_string rrc_T_modeSpecificInfo_23_vals[] = {
  {   0, "fdd" },
  {   1, "tdd" },
  { 0, NULL }
};

static const per_choice_t T_modeSpecificInfo_23_choice[] = {
  {   0, &hf_rrc_fdd_54          , ASN1_NO_EXTENSIONS     , dissect_rrc_T_fdd_53 },
  {   1, &hf_rrc_tdd             , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_modeSpecificInfo_23(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_modeSpecificInfo_23, T_modeSpecificInfo_23_choice,
                                 NULL);

  return offset;
}



static int
dissect_rrc_ReferenceTFC_ID(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            0U, 3U, NULL, FALSE);

  return offset;
}


static const per_sequence_t SignalledGainFactors_sequence[] = {
  { &hf_rrc_modeSpecificInfo_23, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_modeSpecificInfo_23 },
  { &hf_rrc_gainFactorBetaD , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_GainFactor },
  { &hf_rrc_referenceTFC_ID , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_ReferenceTFC_ID },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_SignalledGainFactors(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_SignalledGainFactors, SignalledGainFactors_sequence);

  return offset;
}


static const value_string rrc_GainFactorInformation_vals[] = {
  {   0, "signalledGainFactors" },
  {   1, "computedGainFactors" },
  { 0, NULL }
};

static const per_choice_t GainFactorInformation_choice[] = {
  {   0, &hf_rrc_signalledGainFactors, ASN1_NO_EXTENSIONS     , dissect_rrc_SignalledGainFactors },
  {   1, &hf_rrc_computedGainFactors, ASN1_NO_EXTENSIONS     , dissect_rrc_ReferenceTFC_ID },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_GainFactorInformation(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_GainFactorInformation, GainFactorInformation_choice,
                                 NULL);

  return offset;
}



static int
dissect_rrc_PowerOffsetPp_m(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            -5, 10U, NULL, FALSE);

  return offset;
}


static const per_sequence_t PowerOffsetInformation_sequence[] = {
  { &hf_rrc_gainFactorInformation, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_GainFactorInformation },
  { &hf_rrc_powerOffsetPp_m , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_PowerOffsetPp_m },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_PowerOffsetInformation(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_PowerOffsetInformation, PowerOffsetInformation_sequence);

  return offset;
}


static const per_sequence_t T_ctfc2Bit_item_sequence[] = {
  { &hf_rrc_ctfc2           , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_0_3 },
  { &hf_rrc_powerOffsetInformation, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_PowerOffsetInformation },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_ctfc2Bit_item(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_ctfc2Bit_item, T_ctfc2Bit_item_sequence);

  return offset;
}


static const per_sequence_t T_ctfc2Bit_sequence_of[1] = {
  { &hf_rrc_ctfc2Bit_item   , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_ctfc2Bit_item },
};

static int
dissect_rrc_T_ctfc2Bit(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_T_ctfc2Bit, T_ctfc2Bit_sequence_of,
                                                  1, maxTFC, FALSE);

  return offset;
}


static const per_sequence_t T_ctfc4Bit_item_sequence[] = {
  { &hf_rrc_ctfc4           , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_0_15 },
  { &hf_rrc_powerOffsetInformation, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_PowerOffsetInformation },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_ctfc4Bit_item(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_ctfc4Bit_item, T_ctfc4Bit_item_sequence);

  return offset;
}


static const per_sequence_t T_ctfc4Bit_sequence_of[1] = {
  { &hf_rrc_ctfc4Bit_item   , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_ctfc4Bit_item },
};

static int
dissect_rrc_T_ctfc4Bit(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_T_ctfc4Bit, T_ctfc4Bit_sequence_of,
                                                  1, maxTFC, FALSE);

  return offset;
}


static const per_sequence_t T_ctfc6Bit_item_sequence[] = {
  { &hf_rrc_ctfc6           , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_0_63 },
  { &hf_rrc_powerOffsetInformation, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_PowerOffsetInformation },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_ctfc6Bit_item(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_ctfc6Bit_item, T_ctfc6Bit_item_sequence);

  return offset;
}


static const per_sequence_t T_ctfc6Bit_sequence_of[1] = {
  { &hf_rrc_ctfc6Bit_item   , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_ctfc6Bit_item },
};

static int
dissect_rrc_T_ctfc6Bit(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_T_ctfc6Bit, T_ctfc6Bit_sequence_of,
                                                  1, maxTFC, FALSE);

  return offset;
}


static const per_sequence_t T_ctfc8Bit_item_sequence[] = {
  { &hf_rrc_ctfc8           , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_0_255 },
  { &hf_rrc_powerOffsetInformation, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_PowerOffsetInformation },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_ctfc8Bit_item(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_ctfc8Bit_item, T_ctfc8Bit_item_sequence);

  return offset;
}


static const per_sequence_t T_ctfc8Bit_sequence_of[1] = {
  { &hf_rrc_ctfc8Bit_item   , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_ctfc8Bit_item },
};

static int
dissect_rrc_T_ctfc8Bit(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_T_ctfc8Bit, T_ctfc8Bit_sequence_of,
                                                  1, maxTFC, FALSE);

  return offset;
}



static int
dissect_rrc_INTEGER_0_4095(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            0U, 4095U, NULL, FALSE);

  return offset;
}


static const per_sequence_t T_ctfc12Bit_item_sequence[] = {
  { &hf_rrc_ctfc12          , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_0_4095 },
  { &hf_rrc_powerOffsetInformation, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_PowerOffsetInformation },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_ctfc12Bit_item(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_ctfc12Bit_item, T_ctfc12Bit_item_sequence);

  return offset;
}


static const per_sequence_t T_ctfc12Bit_sequence_of[1] = {
  { &hf_rrc_ctfc12Bit_item  , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_ctfc12Bit_item },
};

static int
dissect_rrc_T_ctfc12Bit(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_T_ctfc12Bit, T_ctfc12Bit_sequence_of,
                                                  1, maxTFC, FALSE);

  return offset;
}



static int
dissect_rrc_INTEGER_0_65535(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            0U, 65535U, NULL, FALSE);

  return offset;
}


static const per_sequence_t T_ctfc16Bit_item_sequence[] = {
  { &hf_rrc_ctfc16          , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_0_65535 },
  { &hf_rrc_powerOffsetInformation, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_PowerOffsetInformation },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_ctfc16Bit_item(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_ctfc16Bit_item, T_ctfc16Bit_item_sequence);

  return offset;
}


static const per_sequence_t T_ctfc16Bit_sequence_of[1] = {
  { &hf_rrc_ctfc16Bit_item  , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_ctfc16Bit_item },
};

static int
dissect_rrc_T_ctfc16Bit(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_T_ctfc16Bit, T_ctfc16Bit_sequence_of,
                                                  1, maxTFC, FALSE);

  return offset;
}



static int
dissect_rrc_INTEGER_0_16777215(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            0U, 16777215U, NULL, FALSE);

  return offset;
}


static const per_sequence_t T_ctfc24Bit_item_sequence[] = {
  { &hf_rrc_ctfc24          , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_0_16777215 },
  { &hf_rrc_powerOffsetInformation, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_PowerOffsetInformation },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_ctfc24Bit_item(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_ctfc24Bit_item, T_ctfc24Bit_item_sequence);

  return offset;
}


static const per_sequence_t T_ctfc24Bit_sequence_of[1] = {
  { &hf_rrc_ctfc24Bit_item  , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_ctfc24Bit_item },
};

static int
dissect_rrc_T_ctfc24Bit(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_T_ctfc24Bit, T_ctfc24Bit_sequence_of,
                                                  1, maxTFC, FALSE);

  return offset;
}


static const value_string rrc_T_ctfcSize_vals[] = {
  {   0, "ctfc2Bit" },
  {   1, "ctfc4Bit" },
  {   2, "ctfc6Bit" },
  {   3, "ctfc8Bit" },
  {   4, "ctfc12Bit" },
  {   5, "ctfc16Bit" },
  {   6, "ctfc24Bit" },
  { 0, NULL }
};

static const per_choice_t T_ctfcSize_choice[] = {
  {   0, &hf_rrc_ctfc2Bit        , ASN1_NO_EXTENSIONS     , dissect_rrc_T_ctfc2Bit },
  {   1, &hf_rrc_ctfc4Bit        , ASN1_NO_EXTENSIONS     , dissect_rrc_T_ctfc4Bit },
  {   2, &hf_rrc_ctfc6Bit        , ASN1_NO_EXTENSIONS     , dissect_rrc_T_ctfc6Bit },
  {   3, &hf_rrc_ctfc8Bit        , ASN1_NO_EXTENSIONS     , dissect_rrc_T_ctfc8Bit },
  {   4, &hf_rrc_ctfc12Bit       , ASN1_NO_EXTENSIONS     , dissect_rrc_T_ctfc12Bit },
  {   5, &hf_rrc_ctfc16Bit       , ASN1_NO_EXTENSIONS     , dissect_rrc_T_ctfc16Bit },
  {   6, &hf_rrc_ctfc24Bit       , ASN1_NO_EXTENSIONS     , dissect_rrc_T_ctfc24Bit },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_ctfcSize(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_ctfcSize, T_ctfcSize_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t TFCS_ReconfAdd_sequence[] = {
  { &hf_rrc_ctfcSize        , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_ctfcSize },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_TFCS_ReconfAdd(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_TFCS_ReconfAdd, TFCS_ReconfAdd_sequence);

  return offset;
}



static int
dissect_rrc_INTEGER_0_1023(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            0U, 1023U, NULL, FALSE);

  return offset;
}


static const per_sequence_t TFCS_Removal_sequence[] = {
  { &hf_rrc_tfci            , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_0_1023 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_TFCS_Removal(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_TFCS_Removal, TFCS_Removal_sequence);

  return offset;
}


static const per_sequence_t TFCS_RemovalList_sequence_of[1] = {
  { &hf_rrc_TFCS_RemovalList_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TFCS_Removal },
};

static int
dissect_rrc_TFCS_RemovalList(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_TFCS_RemovalList, TFCS_RemovalList_sequence_of,
                                                  1, maxTFC, FALSE);

  return offset;
}


static const per_sequence_t T_replacement_sequence[] = {
  { &hf_rrc_tfcsRemoval     , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TFCS_RemovalList },
  { &hf_rrc_tfcsAdd         , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TFCS_ReconfAdd },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_replacement(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_replacement, T_replacement_sequence);

  return offset;
}


static const value_string rrc_ExplicitTFCS_Configuration_vals[] = {
  {   0, "complete" },
  {   1, "addition" },
  {   2, "removal" },
  {   3, "replacement" },
  { 0, NULL }
};

static const per_choice_t ExplicitTFCS_Configuration_choice[] = {
  {   0, &hf_rrc_complete_26     , ASN1_NO_EXTENSIONS     , dissect_rrc_TFCS_ReconfAdd },
  {   1, &hf_rrc_addition        , ASN1_NO_EXTENSIONS     , dissect_rrc_TFCS_ReconfAdd },
  {   2, &hf_rrc_removal         , ASN1_NO_EXTENSIONS     , dissect_rrc_TFCS_RemovalList },
  {   3, &hf_rrc_replacement     , ASN1_NO_EXTENSIONS     , dissect_rrc_T_replacement },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_ExplicitTFCS_Configuration(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_ExplicitTFCS_Configuration, ExplicitTFCS_Configuration_choice,
                                 NULL);

  return offset;
}


static const value_string rrc_SplitType_vals[] = {
  {   0, "hardSplit" },
  {   1, "logicalSplit" },
  { 0, NULL }
};


static int
dissect_rrc_SplitType(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     2, NULL, FALSE, 0, NULL);

  return offset;
}



static int
dissect_rrc_INTEGER_1_10(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            1U, 10U, NULL, FALSE);

  return offset;
}



static int
dissect_rrc_INTEGER_1_1023(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            1U, 1023U, NULL, FALSE);

  return offset;
}


static const value_string rrc_TFCS_InfoForDSCH_vals[] = {
  {   0, "ctfc2bit" },
  {   1, "ctfc4bit" },
  {   2, "ctfc6bit" },
  {   3, "ctfc8bit" },
  {   4, "ctfc12bit" },
  {   5, "ctfc16bit" },
  {   6, "ctfc24bit" },
  { 0, NULL }
};

static const per_choice_t TFCS_InfoForDSCH_choice[] = {
  {   0, &hf_rrc_ctfc2bit        , ASN1_NO_EXTENSIONS     , dissect_rrc_INTEGER_0_3 },
  {   1, &hf_rrc_ctfc4bit        , ASN1_NO_EXTENSIONS     , dissect_rrc_INTEGER_0_15 },
  {   2, &hf_rrc_ctfc6bit        , ASN1_NO_EXTENSIONS     , dissect_rrc_INTEGER_0_63 },
  {   3, &hf_rrc_ctfc8bit        , ASN1_NO_EXTENSIONS     , dissect_rrc_INTEGER_0_255 },
  {   4, &hf_rrc_ctfc12bit       , ASN1_NO_EXTENSIONS     , dissect_rrc_INTEGER_0_4095 },
  {   5, &hf_rrc_ctfc16bit       , ASN1_NO_EXTENSIONS     , dissect_rrc_INTEGER_0_65535 },
  {   6, &hf_rrc_ctfc24bit       , ASN1_NO_EXTENSIONS     , dissect_rrc_INTEGER_0_16777215 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_TFCS_InfoForDSCH(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_TFCS_InfoForDSCH, TFCS_InfoForDSCH_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t TFCI_Range_sequence[] = {
  { &hf_rrc_maxTFCIField2Value, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_1_1023 },
  { &hf_rrc_tfcs_InfoForDSCH, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TFCS_InfoForDSCH },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_TFCI_Range(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_TFCI_Range, TFCI_Range_sequence);

  return offset;
}


static const per_sequence_t TFCI_RangeList_sequence_of[1] = {
  { &hf_rrc_TFCI_RangeList_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TFCI_Range },
};

static int
dissect_rrc_TFCI_RangeList(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_TFCI_RangeList, TFCI_RangeList_sequence_of,
                                                  1, maxPDSCH_TFCIgroups, FALSE);

  return offset;
}


static const value_string rrc_TFCI_Field2_Information_vals[] = {
  {   0, "tfci-Range" },
  {   1, "explicit-config" },
  { 0, NULL }
};

static const per_choice_t TFCI_Field2_Information_choice[] = {
  {   0, &hf_rrc_tfci_Range      , ASN1_NO_EXTENSIONS     , dissect_rrc_TFCI_RangeList },
  {   1, &hf_rrc_explicit_config_02, ASN1_NO_EXTENSIONS     , dissect_rrc_ExplicitTFCS_Configuration },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_TFCI_Field2_Information(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_TFCI_Field2_Information, TFCI_Field2_Information_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t SplitTFCI_Signalling_sequence[] = {
  { &hf_rrc_splitType       , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_SplitType },
  { &hf_rrc_tfci_Field2_Length, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_INTEGER_1_10 },
  { &hf_rrc_tfci_Field1_Information, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_ExplicitTFCS_Configuration },
  { &hf_rrc_tfci_Field2_Information, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_TFCI_Field2_Information },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_SplitTFCI_Signalling(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_SplitTFCI_Signalling, SplitTFCI_Signalling_sequence);

  return offset;
}


static const value_string rrc_TFCS_vals[] = {
  {   0, "normalTFCI-Signalling" },
  {   1, "dummy" },
  { 0, NULL }
};

static const per_choice_t TFCS_choice[] = {
  {   0, &hf_rrc_normalTFCI_Signalling, ASN1_NO_EXTENSIONS     , dissect_rrc_ExplicitTFCS_Configuration },
  {   1, &hf_rrc_dummy_36        , ASN1_NO_EXTENSIONS     , dissect_rrc_SplitTFCI_Signalling },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_TFCS(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_TFCS, TFCS_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t T_sizeType2_sequence[] = {
  { &hf_rrc_part1           , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_0_15 },
  { &hf_rrc_part2           , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_INTEGER_1_7 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_sizeType2(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_sizeType2, T_sizeType2_sequence);

  return offset;
}



static int
dissect_rrc_INTEGER_0_47(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            0U, 47U, NULL, FALSE);

  return offset;
}


static const per_sequence_t T_sizeType3_sequence[] = {
  { &hf_rrc_part1_01        , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_0_47 },
  { &hf_rrc_part2_01        , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_INTEGER_1_15 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_sizeType3(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_sizeType3, T_sizeType3_sequence);

  return offset;
}



static int
dissect_rrc_INTEGER_0_62(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            0U, 62U, NULL, FALSE);

  return offset;
}


static const per_sequence_t T_sizeType4_sequence[] = {
  { &hf_rrc_part1_02        , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_0_62 },
  { &hf_rrc_part2_02        , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_INTEGER_1_63 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_sizeType4(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_sizeType4, T_sizeType4_sequence);

  return offset;
}


static const value_string rrc_BitModeRLC_SizeInfo_vals[] = {
  {   0, "sizeType1" },
  {   1, "sizeType2" },
  {   2, "sizeType3" },
  {   3, "sizeType4" },
  { 0, NULL }
};

static const per_choice_t BitModeRLC_SizeInfo_choice[] = {
  {   0, &hf_rrc_sizeType1       , ASN1_NO_EXTENSIONS     , dissect_rrc_INTEGER_0_127 },
  {   1, &hf_rrc_sizeType2       , ASN1_NO_EXTENSIONS     , dissect_rrc_T_sizeType2 },
  {   2, &hf_rrc_sizeType3       , ASN1_NO_EXTENSIONS     , dissect_rrc_T_sizeType3 },
  {   3, &hf_rrc_sizeType4       , ASN1_NO_EXTENSIONS     , dissect_rrc_T_sizeType4 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_BitModeRLC_SizeInfo(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_BitModeRLC_SizeInfo, BitModeRLC_SizeInfo_choice,
                                 NULL);

  return offset;
}



static int
dissect_rrc_INTEGER_0_23(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            0U, 23U, NULL, FALSE);

  return offset;
}


static const per_sequence_t T_sizeType2_01_sequence[] = {
  { &hf_rrc_part1_03        , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_0_23 },
  { &hf_rrc_part2_03        , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_INTEGER_1_3 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_sizeType2_01(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_sizeType2_01, T_sizeType2_01_sequence);

  return offset;
}



static int
dissect_rrc_INTEGER_0_61(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            0U, 61U, NULL, FALSE);

  return offset;
}


static const per_sequence_t T_sizeType3_01_sequence[] = {
  { &hf_rrc_part1_04        , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_0_61 },
  { &hf_rrc_part2           , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_INTEGER_1_7 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_sizeType3_01(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_sizeType3_01, T_sizeType3_01_sequence);

  return offset;
}


static const value_string rrc_OctetModeRLC_SizeInfoType1_vals[] = {
  {   0, "sizeType1" },
  {   1, "sizeType2" },
  {   2, "sizeType3" },
  { 0, NULL }
};

static const per_choice_t OctetModeRLC_SizeInfoType1_choice[] = {
  {   0, &hf_rrc_sizeType1_01    , ASN1_NO_EXTENSIONS     , dissect_rrc_INTEGER_0_31 },
  {   1, &hf_rrc_sizeType2_01    , ASN1_NO_EXTENSIONS     , dissect_rrc_T_sizeType2_01 },
  {   2, &hf_rrc_sizeType3_01    , ASN1_NO_EXTENSIONS     , dissect_rrc_T_sizeType3_01 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_OctetModeRLC_SizeInfoType1(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_OctetModeRLC_SizeInfoType1, OctetModeRLC_SizeInfoType1_choice,
                                 NULL);

  return offset;
}


static const value_string rrc_T_rlc_Size_01_vals[] = {
  {   0, "bitMode" },
  {   1, "octetModeType1" },
  { 0, NULL }
};

static const per_choice_t T_rlc_Size_01_choice[] = {
  {   0, &hf_rrc_bitMode         , ASN1_NO_EXTENSIONS     , dissect_rrc_BitModeRLC_SizeInfo },
  {   1, &hf_rrc_octetModeType1  , ASN1_NO_EXTENSIONS     , dissect_rrc_OctetModeRLC_SizeInfoType1 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_rlc_Size_01(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_rlc_Size_01, T_rlc_Size_01_choice,
                                 NULL);

  return offset;
}



static int
dissect_rrc_INTEGER_2_17(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            2U, 17U, NULL, FALSE);

  return offset;
}



static int
dissect_rrc_INTEGER_18_512(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            18U, 512U, NULL, FALSE);

  return offset;
}


static const value_string rrc_NumberOfTransportBlocks_vals[] = {
  {   0, "zero" },
  {   1, "one" },
  {   2, "small" },
  {   3, "large" },
  { 0, NULL }
};

static const per_choice_t NumberOfTransportBlocks_choice[] = {
  {   0, &hf_rrc_zero            , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   1, &hf_rrc_one             , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   2, &hf_rrc_small           , ASN1_NO_EXTENSIONS     , dissect_rrc_INTEGER_2_17 },
  {   3, &hf_rrc_large           , ASN1_NO_EXTENSIONS     , dissect_rrc_INTEGER_18_512 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_NumberOfTransportBlocks(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_NumberOfTransportBlocks, NumberOfTransportBlocks_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t SEQUENCE_SIZE_1_maxTF_OF_NumberOfTransportBlocks_sequence_of[1] = {
  { &hf_rrc_numberOfTbSizeList_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_NumberOfTransportBlocks },
};

static int
dissect_rrc_SEQUENCE_SIZE_1_maxTF_OF_NumberOfTransportBlocks(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_SEQUENCE_SIZE_1_maxTF_OF_NumberOfTransportBlocks, SEQUENCE_SIZE_1_maxTF_OF_NumberOfTransportBlocks_sequence_of,
                                                  1, maxTF, FALSE);

  return offset;
}


static const per_sequence_t LogicalChannelByRB_sequence[] = {
  { &hf_rrc_rb_Identity     , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RB_Identity },
  { &hf_rrc_logChOfRb       , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_INTEGER_0_1 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_LogicalChannelByRB(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_LogicalChannelByRB, LogicalChannelByRB_sequence);

  return offset;
}


static const per_sequence_t SEQUENCE_SIZE_1_15_OF_LogicalChannelByRB_sequence_of[1] = {
  { &hf_rrc_explicitList_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_LogicalChannelByRB },
};

static int
dissect_rrc_SEQUENCE_SIZE_1_15_OF_LogicalChannelByRB(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_SEQUENCE_SIZE_1_15_OF_LogicalChannelByRB, SEQUENCE_SIZE_1_15_OF_LogicalChannelByRB_sequence_of,
                                                  1, 15, FALSE);

  return offset;
}


static const value_string rrc_LogicalChannelList_vals[] = {
  {   0, "allSizes" },
  {   1, "configured" },
  {   2, "explicitList" },
  { 0, NULL }
};

static const per_choice_t LogicalChannelList_choice[] = {
  {   0, &hf_rrc_allSizes        , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   1, &hf_rrc_configured      , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   2, &hf_rrc_explicitList_01 , ASN1_NO_EXTENSIONS     , dissect_rrc_SEQUENCE_SIZE_1_15_OF_LogicalChannelByRB },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_LogicalChannelList(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_LogicalChannelList, LogicalChannelList_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t DedicatedDynamicTF_Info_sequence[] = {
  { &hf_rrc_rlc_Size_01     , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_rlc_Size_01 },
  { &hf_rrc_numberOfTbSizeList, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SEQUENCE_SIZE_1_maxTF_OF_NumberOfTransportBlocks },
  { &hf_rrc_logicalChannelList, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_LogicalChannelList },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_DedicatedDynamicTF_Info(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_DedicatedDynamicTF_Info, DedicatedDynamicTF_Info_sequence);

  return offset;
}


static const per_sequence_t DedicatedDynamicTF_InfoList_sequence_of[1] = {
  { &hf_rrc_DedicatedDynamicTF_InfoList_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DedicatedDynamicTF_Info },
};

static int
dissect_rrc_DedicatedDynamicTF_InfoList(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_DedicatedDynamicTF_InfoList, DedicatedDynamicTF_InfoList_sequence_of,
                                                  1, maxTF, FALSE);

  return offset;
}


static const value_string rrc_T_rlc_Size_02_vals[] = {
  {   0, "bitMode" },
  {   1, "octetModeType1" },
  { 0, NULL }
};

static const per_choice_t T_rlc_Size_02_choice[] = {
  {   0, &hf_rrc_bitMode         , ASN1_NO_EXTENSIONS     , dissect_rrc_BitModeRLC_SizeInfo },
  {   1, &hf_rrc_octetModeType1  , ASN1_NO_EXTENSIONS     , dissect_rrc_OctetModeRLC_SizeInfoType1 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_rlc_Size_02(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_rlc_Size_02, T_rlc_Size_02_choice,
                                 NULL);

  return offset;
}


static const value_string rrc_TransmissionTimeInterval_vals[] = {
  {   0, "tti10" },
  {   1, "tti20" },
  {   2, "tti40" },
  {   3, "tti80" },
  { 0, NULL }
};


static int
dissect_rrc_TransmissionTimeInterval(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     4, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t NumberOfTbSizeAndTTIList_item_sequence[] = {
  { &hf_rrc_numberOfTransportBlocks, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_NumberOfTransportBlocks },
  { &hf_rrc_transmissionTimeInterval, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TransmissionTimeInterval },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_NumberOfTbSizeAndTTIList_item(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_NumberOfTbSizeAndTTIList_item, NumberOfTbSizeAndTTIList_item_sequence);

  return offset;
}


static const per_sequence_t NumberOfTbSizeAndTTIList_sequence_of[1] = {
  { &hf_rrc_NumberOfTbSizeAndTTIList_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_NumberOfTbSizeAndTTIList_item },
};

static int
dissect_rrc_NumberOfTbSizeAndTTIList(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_NumberOfTbSizeAndTTIList, NumberOfTbSizeAndTTIList_sequence_of,
                                                  1, maxTF, FALSE);

  return offset;
}


static const per_sequence_t DedicatedDynamicTF_Info_DynamicTTI_sequence[] = {
  { &hf_rrc_rlc_Size_02     , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_rlc_Size_02 },
  { &hf_rrc_numberOfTbSizeAndTTIList, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_NumberOfTbSizeAndTTIList },
  { &hf_rrc_logicalChannelList, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_LogicalChannelList },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_DedicatedDynamicTF_Info_DynamicTTI(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_DedicatedDynamicTF_Info_DynamicTTI, DedicatedDynamicTF_Info_DynamicTTI_sequence);

  return offset;
}


static const per_sequence_t DedicatedDynamicTF_InfoList_DynamicTTI_sequence_of[1] = {
  { &hf_rrc_DedicatedDynamicTF_InfoList_DynamicTTI_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DedicatedDynamicTF_Info_DynamicTTI },
};

static int
dissect_rrc_DedicatedDynamicTF_InfoList_DynamicTTI(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_DedicatedDynamicTF_InfoList_DynamicTTI, DedicatedDynamicTF_InfoList_DynamicTTI_sequence_of,
                                                  1, maxTF, FALSE);

  return offset;
}


static const value_string rrc_T_tti_02_vals[] = {
  {   0, "tti10" },
  {   1, "tti20" },
  {   2, "tti40" },
  {   3, "tti80" },
  {   4, "dynamic" },
  { 0, NULL }
};

static const per_choice_t T_tti_02_choice[] = {
  {   0, &hf_rrc_tti10_01        , ASN1_NO_EXTENSIONS     , dissect_rrc_DedicatedDynamicTF_InfoList },
  {   1, &hf_rrc_tti20_01        , ASN1_NO_EXTENSIONS     , dissect_rrc_DedicatedDynamicTF_InfoList },
  {   2, &hf_rrc_tti40_01        , ASN1_NO_EXTENSIONS     , dissect_rrc_DedicatedDynamicTF_InfoList },
  {   3, &hf_rrc_tti80_01        , ASN1_NO_EXTENSIONS     , dissect_rrc_DedicatedDynamicTF_InfoList },
  {   4, &hf_rrc_dynamic_01      , ASN1_NO_EXTENSIONS     , dissect_rrc_DedicatedDynamicTF_InfoList_DynamicTTI },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_tti_02(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_tti_02, T_tti_02_choice,
                                 NULL);

  return offset;
}


static const value_string rrc_CodingRate_vals[] = {
  {   0, "half" },
  {   1, "third" },
  { 0, NULL }
};


static int
dissect_rrc_CodingRate(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     2, NULL, FALSE, 0, NULL);

  return offset;
}


static const value_string rrc_ChannelCodingType_vals[] = {
  {   0, "noCoding" },
  {   1, "convolutional" },
  {   2, "turbo" },
  { 0, NULL }
};

static const per_choice_t ChannelCodingType_choice[] = {
  {   0, &hf_rrc_noCoding        , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   1, &hf_rrc_convolutional   , ASN1_NO_EXTENSIONS     , dissect_rrc_CodingRate },
  {   2, &hf_rrc_turbo           , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_ChannelCodingType(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_ChannelCodingType, ChannelCodingType_choice,
                                 NULL);

  return offset;
}



static int
dissect_rrc_RateMatchingAttribute(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            1U, hiRM, NULL, FALSE);

  return offset;
}


static const value_string rrc_CRC_Size_vals[] = {
  {   0, "crc0" },
  {   1, "crc8" },
  {   2, "crc12" },
  {   3, "crc16" },
  {   4, "crc24" },
  { 0, NULL }
};


static int
dissect_rrc_CRC_Size(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     5, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t SemistaticTF_Information_sequence[] = {
  { &hf_rrc_channelCodingType, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_ChannelCodingType },
  { &hf_rrc_rateMatchingAttribute, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RateMatchingAttribute },
  { &hf_rrc_crc_Size        , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CRC_Size },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_SemistaticTF_Information(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_SemistaticTF_Information, SemistaticTF_Information_sequence);

  return offset;
}


static const per_sequence_t DedicatedTransChTFS_sequence[] = {
  { &hf_rrc_tti_02          , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_tti_02 },
  { &hf_rrc_semistaticTF_Information, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SemistaticTF_Information },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_DedicatedTransChTFS(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_DedicatedTransChTFS, DedicatedTransChTFS_sequence);

  return offset;
}



static int
dissect_rrc_INTEGER_0_56(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            0U, 56U, NULL, FALSE);

  return offset;
}


static const value_string rrc_OctetModeRLC_SizeInfoType2_vals[] = {
  {   0, "sizeType1" },
  {   1, "sizeType2" },
  {   2, "sizeType3" },
  { 0, NULL }
};

static const per_choice_t OctetModeRLC_SizeInfoType2_choice[] = {
  {   0, &hf_rrc_sizeType1_01    , ASN1_NO_EXTENSIONS     , dissect_rrc_INTEGER_0_31 },
  {   1, &hf_rrc_sizeType2_02    , ASN1_NO_EXTENSIONS     , dissect_rrc_INTEGER_0_63 },
  {   2, &hf_rrc_sizeType3_02    , ASN1_NO_EXTENSIONS     , dissect_rrc_INTEGER_0_56 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_OctetModeRLC_SizeInfoType2(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_OctetModeRLC_SizeInfoType2, OctetModeRLC_SizeInfoType2_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t T_fdd_48_sequence[] = {
  { &hf_rrc_octetModeRLC_SizeInfoType2, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_OctetModeRLC_SizeInfoType2 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_fdd_48(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_fdd_48, T_fdd_48_sequence);

  return offset;
}


static const value_string rrc_T_commonTDD_Choice_vals[] = {
  {   0, "bitModeRLC-SizeInfo" },
  {   1, "octetModeRLC-SizeInfoType1" },
  { 0, NULL }
};

static const per_choice_t T_commonTDD_Choice_choice[] = {
  {   0, &hf_rrc_bitModeRLC_SizeInfo, ASN1_NO_EXTENSIONS     , dissect_rrc_BitModeRLC_SizeInfo },
  {   1, &hf_rrc_octetModeRLC_SizeInfoType1, ASN1_NO_EXTENSIONS     , dissect_rrc_OctetModeRLC_SizeInfoType1 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_commonTDD_Choice(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_commonTDD_Choice, T_commonTDD_Choice_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t T_tdd_13_sequence[] = {
  { &hf_rrc_commonTDD_Choice, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_commonTDD_Choice },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_tdd_13(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_tdd_13, T_tdd_13_sequence);

  return offset;
}


static const value_string rrc_T_rlc_Size_vals[] = {
  {   0, "fdd" },
  {   1, "tdd" },
  { 0, NULL }
};

static const per_choice_t T_rlc_Size_choice[] = {
  {   0, &hf_rrc_fdd_49          , ASN1_NO_EXTENSIONS     , dissect_rrc_T_fdd_48 },
  {   1, &hf_rrc_tdd_14          , ASN1_NO_EXTENSIONS     , dissect_rrc_T_tdd_13 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_rlc_Size(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_rlc_Size, T_rlc_Size_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t CommonDynamicTF_Info_sequence[] = {
  { &hf_rrc_rlc_Size        , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_rlc_Size },
  { &hf_rrc_numberOfTbSizeList, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SEQUENCE_SIZE_1_maxTF_OF_NumberOfTransportBlocks },
  { &hf_rrc_logicalChannelList, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_LogicalChannelList },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_CommonDynamicTF_Info(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_CommonDynamicTF_Info, CommonDynamicTF_Info_sequence);

  return offset;
}


static const per_sequence_t CommonDynamicTF_InfoList_sequence_of[1] = {
  { &hf_rrc_CommonDynamicTF_InfoList_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CommonDynamicTF_Info },
};

static int
dissect_rrc_CommonDynamicTF_InfoList(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_CommonDynamicTF_InfoList, CommonDynamicTF_InfoList_sequence_of,
                                                  1, maxTF, FALSE);

  return offset;
}


static const value_string rrc_T_commonTDD_Choice_01_vals[] = {
  {   0, "bitModeRLC-SizeInfo" },
  {   1, "octetModeRLC-SizeInfoType1" },
  { 0, NULL }
};

static const per_choice_t T_commonTDD_Choice_01_choice[] = {
  {   0, &hf_rrc_bitModeRLC_SizeInfo, ASN1_NO_EXTENSIONS     , dissect_rrc_BitModeRLC_SizeInfo },
  {   1, &hf_rrc_octetModeRLC_SizeInfoType1, ASN1_NO_EXTENSIONS     , dissect_rrc_OctetModeRLC_SizeInfoType1 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_commonTDD_Choice_01(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_commonTDD_Choice_01, T_commonTDD_Choice_01_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t CommonDynamicTF_Info_DynamicTTI_sequence[] = {
  { &hf_rrc_commonTDD_Choice_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_commonTDD_Choice_01 },
  { &hf_rrc_numberOfTbSizeAndTTIList, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_NumberOfTbSizeAndTTIList },
  { &hf_rrc_logicalChannelList, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_LogicalChannelList },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_CommonDynamicTF_Info_DynamicTTI(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_CommonDynamicTF_Info_DynamicTTI, CommonDynamicTF_Info_DynamicTTI_sequence);

  return offset;
}


static const per_sequence_t CommonDynamicTF_InfoList_DynamicTTI_sequence_of[1] = {
  { &hf_rrc_CommonDynamicTF_InfoList_DynamicTTI_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CommonDynamicTF_Info_DynamicTTI },
};

static int
dissect_rrc_CommonDynamicTF_InfoList_DynamicTTI(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_CommonDynamicTF_InfoList_DynamicTTI, CommonDynamicTF_InfoList_DynamicTTI_sequence_of,
                                                  1, maxTF, FALSE);

  return offset;
}


static const value_string rrc_T_tti_vals[] = {
  {   0, "tti10" },
  {   1, "tti20" },
  {   2, "tti40" },
  {   3, "tti80" },
  {   4, "dynamic" },
  { 0, NULL }
};

static const per_choice_t T_tti_choice[] = {
  {   0, &hf_rrc_tti10           , ASN1_NO_EXTENSIONS     , dissect_rrc_CommonDynamicTF_InfoList },
  {   1, &hf_rrc_tti20           , ASN1_NO_EXTENSIONS     , dissect_rrc_CommonDynamicTF_InfoList },
  {   2, &hf_rrc_tti40           , ASN1_NO_EXTENSIONS     , dissect_rrc_CommonDynamicTF_InfoList },
  {   3, &hf_rrc_tti80           , ASN1_NO_EXTENSIONS     , dissect_rrc_CommonDynamicTF_InfoList },
  {   4, &hf_rrc_dynamic         , ASN1_NO_EXTENSIONS     , dissect_rrc_CommonDynamicTF_InfoList_DynamicTTI },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_tti(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_tti, T_tti_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t CommonTransChTFS_sequence[] = {
  { &hf_rrc_tti             , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_tti },
  { &hf_rrc_semistaticTF_Information, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SemistaticTF_Information },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_CommonTransChTFS(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_CommonTransChTFS, CommonTransChTFS_sequence);

  return offset;
}


static const value_string rrc_TransportFormatSet_vals[] = {
  {   0, "dedicatedTransChTFS" },
  {   1, "commonTransChTFS" },
  { 0, NULL }
};

static const per_choice_t TransportFormatSet_choice[] = {
  {   0, &hf_rrc_dedicatedTransChTFS, ASN1_NO_EXTENSIONS     , dissect_rrc_DedicatedTransChTFS },
  {   1, &hf_rrc_commonTransChTFS, ASN1_NO_EXTENSIONS     , dissect_rrc_CommonTransChTFS },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_TransportFormatSet(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_TransportFormatSet, TransportFormatSet_choice,
                                 NULL);

  return offset;
}



static int
dissect_rrc_TransportChannelIdentity(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            1U, 32U, NULL, FALSE);

  return offset;
}


static const per_sequence_t FACH_PCH_Information_sequence[] = {
  { &hf_rrc_transportFormatSet, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TransportFormatSet },
  { &hf_rrc_transportChannelIdentity, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TransportChannelIdentity },
  { &hf_rrc_ctch_Indicator  , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_FACH_PCH_Information(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_FACH_PCH_Information, FACH_PCH_Information_sequence);

  return offset;
}


static const per_sequence_t FACH_PCH_InformationList_sequence_of[1] = {
  { &hf_rrc_FACH_PCH_InformationList_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_FACH_PCH_Information },
};

static int
dissect_rrc_FACH_PCH_InformationList(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_FACH_PCH_InformationList, FACH_PCH_InformationList_sequence_of,
                                                  1, maxFACHPCH, FALSE);

  return offset;
}



static int
dissect_rrc_PLMN_ValueTag(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            1U, 256U, NULL, FALSE);

  return offset;
}



static int
dissect_rrc_CellValueTag(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            1U, 4U, NULL, FALSE);

  return offset;
}



static int
dissect_rrc_PredefinedConfigIdentity(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            0U, 15U, NULL, FALSE);

  return offset;
}



static int
dissect_rrc_PredefinedConfigValueTag(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            0U, 15U, NULL, FALSE);

  return offset;
}


static const per_sequence_t PredefinedConfigIdentityAndValueTag_sequence[] = {
  { &hf_rrc_predefinedConfigIdentity, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PredefinedConfigIdentity },
  { &hf_rrc_predefinedConfigValueTag, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PredefinedConfigValueTag },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_PredefinedConfigIdentityAndValueTag(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_PredefinedConfigIdentityAndValueTag, PredefinedConfigIdentityAndValueTag_sequence);

  return offset;
}



static int
dissect_rrc_SIBOccurIdentity(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            0U, 15U, NULL, FALSE);

  return offset;
}



static int
dissect_rrc_SIBOccurValueTag(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            0U, 15U, NULL, FALSE);

  return offset;
}


static const per_sequence_t SIBOccurrenceIdentityAndValueTag_sequence[] = {
  { &hf_rrc_sibOccurIdentity, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SIBOccurIdentity },
  { &hf_rrc_sibOccurValueTag, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SIBOccurValueTag },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_SIBOccurrenceIdentityAndValueTag(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_SIBOccurrenceIdentityAndValueTag, SIBOccurrenceIdentityAndValueTag_sequence);

  return offset;
}


static const value_string rrc_SIB_TypeAndTag_vals[] = {
  {   0, "sysInfoType1" },
  {   1, "sysInfoType2" },
  {   2, "sysInfoType3" },
  {   3, "sysInfoType4" },
  {   4, "sysInfoType5" },
  {   5, "sysInfoType6" },
  {   6, "sysInfoType7" },
  {   7, "dummy" },
  {   8, "dummy2" },
  {   9, "dummy3" },
  {  10, "sysInfoType11" },
  {  11, "sysInfoType12" },
  {  12, "sysInfoType13" },
  {  13, "sysInfoType13-1" },
  {  14, "sysInfoType13-2" },
  {  15, "sysInfoType13-3" },
  {  16, "sysInfoType13-4" },
  {  17, "sysInfoType14" },
  {  18, "sysInfoType15" },
  {  19, "sysInfoType16" },
  {  20, "sysInfoType17" },
  {  21, "sysInfoType15-1" },
  {  22, "sysInfoType15-2" },
  {  23, "sysInfoType15-3" },
  {  24, "sysInfoType15-4" },
  {  25, "sysInfoType18" },
  {  26, "sysInfoType15-5" },
  {  27, "sysInfoType5bis" },
  {  28, "spare4" },
  {  29, "spare3" },
  {  30, "spare2" },
  {  31, "spare1" },
  { 0, NULL }
};

static value_string_ext rrc_SIB_TypeAndTag_vals_ext = VALUE_STRING_EXT_INIT(rrc_SIB_TypeAndTag_vals);

static const per_choice_t SIB_TypeAndTag_choice[] = {
  {   0, &hf_rrc_sysInfoType1_01 , ASN1_NO_EXTENSIONS     , dissect_rrc_PLMN_ValueTag },
  {   1, &hf_rrc_sysInfoType2    , ASN1_NO_EXTENSIONS     , dissect_rrc_CellValueTag },
  {   2, &hf_rrc_sysInfoType3_01 , ASN1_NO_EXTENSIONS     , dissect_rrc_CellValueTag },
  {   3, &hf_rrc_sysInfoType4    , ASN1_NO_EXTENSIONS     , dissect_rrc_CellValueTag },
  {   4, &hf_rrc_sysInfoType5_01 , ASN1_NO_EXTENSIONS     , dissect_rrc_CellValueTag },
  {   5, &hf_rrc_sysInfoType6    , ASN1_NO_EXTENSIONS     , dissect_rrc_CellValueTag },
  {   6, &hf_rrc_sysInfoType7_01 , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   7, &hf_rrc_dummy_55        , ASN1_NO_EXTENSIONS     , dissect_rrc_CellValueTag },
  {   8, &hf_rrc_dummy2_14       , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   9, &hf_rrc_dummy3_03       , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {  10, &hf_rrc_sysInfoType11_01, ASN1_NO_EXTENSIONS     , dissect_rrc_CellValueTag },
  {  11, &hf_rrc_sysInfoType12_01, ASN1_NO_EXTENSIONS     , dissect_rrc_CellValueTag },
  {  12, &hf_rrc_sysInfoType13   , ASN1_NO_EXTENSIONS     , dissect_rrc_CellValueTag },
  {  13, &hf_rrc_sysInfoType13_1 , ASN1_NO_EXTENSIONS     , dissect_rrc_CellValueTag },
  {  14, &hf_rrc_sysInfoType13_2 , ASN1_NO_EXTENSIONS     , dissect_rrc_CellValueTag },
  {  15, &hf_rrc_sysInfoType13_3 , ASN1_NO_EXTENSIONS     , dissect_rrc_CellValueTag },
  {  16, &hf_rrc_sysInfoType13_4 , ASN1_NO_EXTENSIONS     , dissect_rrc_CellValueTag },
  {  17, &hf_rrc_sysInfoType14   , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {  18, &hf_rrc_sysInfoType15   , ASN1_NO_EXTENSIONS     , dissect_rrc_CellValueTag },
  {  19, &hf_rrc_sysInfoType16   , ASN1_NO_EXTENSIONS     , dissect_rrc_PredefinedConfigIdentityAndValueTag },
  {  20, &hf_rrc_sysInfoType17   , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {  21, &hf_rrc_sysInfoType15_1 , ASN1_NO_EXTENSIONS     , dissect_rrc_CellValueTag },
  {  22, &hf_rrc_sysInfoType15_2 , ASN1_NO_EXTENSIONS     , dissect_rrc_SIBOccurrenceIdentityAndValueTag },
  {  23, &hf_rrc_sysInfoType15_3 , ASN1_NO_EXTENSIONS     , dissect_rrc_SIBOccurrenceIdentityAndValueTag },
  {  24, &hf_rrc_sysInfoType15_4 , ASN1_NO_EXTENSIONS     , dissect_rrc_CellValueTag },
  {  25, &hf_rrc_sysInfoType18   , ASN1_NO_EXTENSIONS     , dissect_rrc_CellValueTag },
  {  26, &hf_rrc_sysInfoType15_5 , ASN1_NO_EXTENSIONS     , dissect_rrc_CellValueTag },
  {  27, &hf_rrc_sysInfoType5bis , ASN1_NO_EXTENSIONS     , dissect_rrc_CellValueTag },
  {  28, &hf_rrc_spare4          , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {  29, &hf_rrc_spare3          , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {  30, &hf_rrc_spare2          , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {  31, &hf_rrc_spare1          , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_SIB_TypeAndTag(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_SIB_TypeAndTag, SIB_TypeAndTag_choice,
                                 NULL);

  return offset;
}



static int
dissect_rrc_SegCount(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            1U, 16U, NULL, FALSE);

  return offset;
}



static int
dissect_rrc_INTEGER_0_2047(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            0U, 2047U, NULL, FALSE);

  return offset;
}


static const value_string rrc_T_sib_Pos_vals[] = {
  {   0, "rep4" },
  {   1, "rep8" },
  {   2, "rep16" },
  {   3, "rep32" },
  {   4, "rep64" },
  {   5, "rep128" },
  {   6, "rep256" },
  {   7, "rep512" },
  {   8, "rep1024" },
  {   9, "rep2048" },
  {  10, "rep4096" },
  { 0, NULL }
};

static const per_choice_t T_sib_Pos_choice[] = {
  {   0, &hf_rrc_rep4            , ASN1_NO_EXTENSIONS     , dissect_rrc_INTEGER_0_1 },
  {   1, &hf_rrc_rep8            , ASN1_NO_EXTENSIONS     , dissect_rrc_INTEGER_0_3 },
  {   2, &hf_rrc_rep16           , ASN1_NO_EXTENSIONS     , dissect_rrc_INTEGER_0_7 },
  {   3, &hf_rrc_rep32           , ASN1_NO_EXTENSIONS     , dissect_rrc_INTEGER_0_15 },
  {   4, &hf_rrc_rep64           , ASN1_NO_EXTENSIONS     , dissect_rrc_INTEGER_0_31 },
  {   5, &hf_rrc_rep128          , ASN1_NO_EXTENSIONS     , dissect_rrc_INTEGER_0_63 },
  {   6, &hf_rrc_rep256          , ASN1_NO_EXTENSIONS     , dissect_rrc_INTEGER_0_127 },
  {   7, &hf_rrc_rep512          , ASN1_NO_EXTENSIONS     , dissect_rrc_INTEGER_0_255 },
  {   8, &hf_rrc_rep1024         , ASN1_NO_EXTENSIONS     , dissect_rrc_INTEGER_0_511 },
  {   9, &hf_rrc_rep2048         , ASN1_NO_EXTENSIONS     , dissect_rrc_INTEGER_0_1023 },
  {  10, &hf_rrc_rep4096         , ASN1_NO_EXTENSIONS     , dissect_rrc_INTEGER_0_2047 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_sib_Pos(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_sib_Pos, T_sib_Pos_choice,
                                 NULL);

  return offset;
}


static const value_string rrc_SibOFF_vals[] = {
  {   0, "so2" },
  {   1, "so4" },
  {   2, "so6" },
  {   3, "so8" },
  {   4, "so10" },
  {   5, "so12" },
  {   6, "so14" },
  {   7, "so16" },
  {   8, "so18" },
  {   9, "so20" },
  {  10, "so22" },
  {  11, "so24" },
  {  12, "so26" },
  {  13, "so28" },
  {  14, "so30" },
  {  15, "so32" },
  { 0, NULL }
};


static int
dissect_rrc_SibOFF(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     16, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t SibOFF_List_sequence_of[1] = {
  { &hf_rrc_SibOFF_List_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SibOFF },
};

static int
dissect_rrc_SibOFF_List(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_SibOFF_List, SibOFF_List_sequence_of,
                                                  1, 15, FALSE);

  return offset;
}


static const per_sequence_t T_scheduling_sequence[] = {
  { &hf_rrc_segCount        , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_SegCount },
  { &hf_rrc_sib_Pos         , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_sib_Pos },
  { &hf_rrc_sib_PosOffsetInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_SibOFF_List },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_scheduling(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_scheduling, T_scheduling_sequence);

  return offset;
}


static const per_sequence_t SchedulingInformation_sequence[] = {
  { &hf_rrc_scheduling      , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_scheduling },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_SchedulingInformation(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_SchedulingInformation, SchedulingInformation_sequence);

  return offset;
}


static const per_sequence_t SchedulingInformationSIB_sequence[] = {
  { &hf_rrc_sib_Type_01     , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SIB_TypeAndTag },
  { &hf_rrc_scheduling_01   , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SchedulingInformation },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_SchedulingInformationSIB(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_SchedulingInformationSIB, SchedulingInformationSIB_sequence);

  return offset;
}


static const per_sequence_t SIB_ReferenceListFACH_sequence_of[1] = {
  { &hf_rrc_SIB_ReferenceListFACH_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SchedulingInformationSIB },
};

static int
dissect_rrc_SIB_ReferenceListFACH(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_SIB_ReferenceListFACH, SIB_ReferenceListFACH_sequence_of,
                                                  1, maxSIB_FACH, FALSE);

  return offset;
}


static const per_sequence_t T_fdd_107_sequence[] = {
  { &hf_rrc_fach_PCH_InformationList, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_FACH_PCH_InformationList },
  { &hf_rrc_sib_ReferenceListFACH, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SIB_ReferenceListFACH },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_fdd_107(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_fdd_107, T_fdd_107_sequence);

  return offset;
}


static const per_sequence_t T_tdd_61_sequence[] = {
  { &hf_rrc_fach_PCH_InformationList, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_FACH_PCH_InformationList },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_tdd_61(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_tdd_61, T_tdd_61_sequence);

  return offset;
}


static const value_string rrc_T_modeSpecificInfo_74_vals[] = {
  {   0, "fdd" },
  {   1, "tdd" },
  { 0, NULL }
};

static const per_choice_t T_modeSpecificInfo_74_choice[] = {
  {   0, &hf_rrc_fdd_110         , ASN1_NO_EXTENSIONS     , dissect_rrc_T_fdd_107 },
  {   1, &hf_rrc_tdd_67          , ASN1_NO_EXTENSIONS     , dissect_rrc_T_tdd_61 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_modeSpecificInfo_74(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_modeSpecificInfo_74, T_modeSpecificInfo_74_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t SCCPCH_InfoForFACH_sequence[] = {
  { &hf_rrc_secondaryCCPCH_Info, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SecondaryCCPCH_Info },
  { &hf_rrc_tfcs            , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TFCS },
  { &hf_rrc_modeSpecificInfo_74, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_modeSpecificInfo_74 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_SCCPCH_InfoForFACH(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_SCCPCH_InfoForFACH, SCCPCH_InfoForFACH_sequence);

  return offset;
}


static const per_sequence_t RL_AdditionInformation_sequence[] = {
  { &hf_rrc_primaryCPICH_Info, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PrimaryCPICH_Info },
  { &hf_rrc_dl_DPCH_InfoPerRL, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DL_DPCH_InfoPerRL },
  { &hf_rrc_dummy_28        , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { &hf_rrc_dummy2_08       , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_SCCPCH_InfoForFACH },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_RL_AdditionInformation(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_RL_AdditionInformation, RL_AdditionInformation_sequence);

  return offset;
}


static const per_sequence_t RL_AdditionInformationList_sequence_of[1] = {
  { &hf_rrc_RL_AdditionInformationList_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RL_AdditionInformation },
};

static int
dissect_rrc_RL_AdditionInformationList(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_RL_AdditionInformationList, RL_AdditionInformationList_sequence_of,
                                                  1, maxRL_1, FALSE);

  return offset;
}


static const per_sequence_t RL_RemovalInformationList_sequence_of[1] = {
  { &hf_rrc_RL_RemovalInformationList_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PrimaryCPICH_Info },
};

static int
dissect_rrc_RL_RemovalInformationList(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_RL_RemovalInformationList, RL_RemovalInformationList_sequence_of,
                                                  1, maxRL, FALSE);

  return offset;
}


static const value_string rrc_TX_DiversityMode_vals[] = {
  {   0, "noDiversity" },
  {   1, "sttd" },
  {   2, "closedLoopMode1" },
  {   3, "dummy" },
  { 0, NULL }
};


static int
dissect_rrc_TX_DiversityMode(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     4, NULL, FALSE, 0, NULL);

  return offset;
}


static const value_string rrc_S_Field_vals[] = {
  {   0, "e1bit" },
  {   1, "e2bits" },
  { 0, NULL }
};


static int
dissect_rrc_S_Field(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     2, NULL, FALSE, 0, NULL);

  return offset;
}


static const value_string rrc_CodeWordSet_vals[] = {
  {   0, "longCWS" },
  {   1, "mediumCWS" },
  {   2, "shortCWS" },
  {   3, "ssdtOff" },
  { 0, NULL }
};


static int
dissect_rrc_CodeWordSet(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     4, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t SSDT_Information_sequence[] = {
  { &hf_rrc_s_Field         , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_S_Field },
  { &hf_rrc_codeWordSet     , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CodeWordSet },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_SSDT_Information(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_SSDT_Information, SSDT_Information_sequence);

  return offset;
}


static const per_sequence_t ActiveSetUpdate_r3_IEs_sequence[] = {
  { &hf_rrc_rrc_TransactionIdentifier, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RRC_TransactionIdentifier },
  { &hf_rrc_dummy_01        , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_IntegrityProtectionModeInfo },
  { &hf_rrc_dummy2          , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CipheringModeInfo },
  { &hf_rrc_activationTime  , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_ActivationTime },
  { &hf_rrc_newU_RNTI       , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_U_RNTI },
  { &hf_rrc_cn_InformationInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CN_InformationInfo },
  { &hf_rrc_dummy3          , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_CounterSynchronisationInfo },
  { &hf_rrc_maxAllowedUL_TX_Power, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MaxAllowedUL_TX_Power },
  { &hf_rrc_rl_AdditionInformationList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RL_AdditionInformationList },
  { &hf_rrc_rl_RemovalInformationList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RL_RemovalInformationList },
  { &hf_rrc_tx_DiversityMode, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_TX_DiversityMode },
  { &hf_rrc_dummy4          , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_SSDT_Information },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_ActiveSetUpdate_r3_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_ActiveSetUpdate_r3_IEs, ActiveSetUpdate_r3_IEs_sequence);

  return offset;
}



static int
dissect_rrc_BIT_STRING(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_bit_string(tvb, offset, actx, tree, hf_index,
                                     NO_BOUND, NO_BOUND, FALSE, NULL);

  return offset;
}


static const value_string rrc_SSDT_UL_vals[] = {
  {   0, "ul" },
  {   1, "ul-AndDL" },
  { 0, NULL }
};


static int
dissect_rrc_SSDT_UL(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     2, NULL, FALSE, 0, NULL);

  return offset;
}



static int
dissect_rrc_CellIdentity(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_bit_string(tvb, offset, actx, tree, hf_index,
                                     28, 28, FALSE, NULL);

  return offset;
}


static const per_sequence_t CellIdentity_PerRL_List_sequence_of[1] = {
  { &hf_rrc_CellIdentity_PerRL_List_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CellIdentity },
};

static int
dissect_rrc_CellIdentity_PerRL_List(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_CellIdentity_PerRL_List, CellIdentity_PerRL_List_sequence_of,
                                                  1, maxRL, FALSE);

  return offset;
}


static const per_sequence_t ActiveSetUpdate_v4b0ext_IEs_sequence[] = {
  { &hf_rrc_dummy_02        , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_SSDT_UL },
  { &hf_rrc_cell_id_PerRL_List, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CellIdentity_PerRL_List },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_ActiveSetUpdate_v4b0ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_ActiveSetUpdate_v4b0ext_IEs, ActiveSetUpdate_v4b0ext_IEs_sequence);

  return offset;
}


static const value_string rrc_DPC_Mode_vals[] = {
  {   0, "singleTPC" },
  {   1, "tpcTripletInSoft" },
  { 0, NULL }
};


static int
dissect_rrc_DPC_Mode(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     2, NULL, FALSE, 0, NULL);

  return offset;
}



static int
dissect_rrc_PowerOffsetTPC_pdpdch(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            0U, 24U, NULL, FALSE);

  return offset;
}


static const per_sequence_t DL_TPC_PowerOffsetPerRL_sequence[] = {
  { &hf_rrc_powerOffsetTPC_pdpdch, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_PowerOffsetTPC_pdpdch },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_DL_TPC_PowerOffsetPerRL(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_DL_TPC_PowerOffsetPerRL, DL_TPC_PowerOffsetPerRL_sequence);

  return offset;
}


static const per_sequence_t DL_TPC_PowerOffsetPerRL_List_sequence_of[1] = {
  { &hf_rrc_DL_TPC_PowerOffsetPerRL_List_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DL_TPC_PowerOffsetPerRL },
};

static int
dissect_rrc_DL_TPC_PowerOffsetPerRL_List(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_DL_TPC_PowerOffsetPerRL_List, DL_TPC_PowerOffsetPerRL_List_sequence_of,
                                                  1, maxRL, FALSE);

  return offset;
}


static const per_sequence_t ActiveSetUpdate_v590ext_IEs_sequence[] = {
  { &hf_rrc_dpc_Mode        , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DPC_Mode },
  { &hf_rrc_dl_TPC_PowerOffsetPerRL_List, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_TPC_PowerOffsetPerRL_List },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_ActiveSetUpdate_v590ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_ActiveSetUpdate_v590ext_IEs, ActiveSetUpdate_v590ext_IEs_sequence);

  return offset;
}


static const per_sequence_t ActiveSetUpdate_v690ext_IEs_sequence[] = {
  { &hf_rrc_primary_plmn_Identity, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_PLMN_Identity },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_ActiveSetUpdate_v690ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_ActiveSetUpdate_v690ext_IEs, ActiveSetUpdate_v690ext_IEs_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_nonCriticalExtensions(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_nonCriticalExtensions, T_nonCriticalExtensions_sequence);

  return offset;
}


static const per_sequence_t T_v690NonCriticalExtensions_sequence[] = {
  { &hf_rrc_activeSetUpdate_v690ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_ActiveSetUpdate_v690ext_IEs },
  { &hf_rrc_nonCriticalExtensions, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_nonCriticalExtensions },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v690NonCriticalExtensions(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v690NonCriticalExtensions, T_v690NonCriticalExtensions_sequence);

  return offset;
}


static const per_sequence_t T_v590NonCriticalExtensions_sequence[] = {
  { &hf_rrc_activeSetUpdate_v590ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_ActiveSetUpdate_v590ext_IEs },
  { &hf_rrc_v690NonCriticalExtensions, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v690NonCriticalExtensions },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v590NonCriticalExtensions(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v590NonCriticalExtensions, T_v590NonCriticalExtensions_sequence);

  return offset;
}


static const per_sequence_t T_v4b0NonCriticalExtensions_sequence[] = {
  { &hf_rrc_activeSetUpdate_v4b0ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_ActiveSetUpdate_v4b0ext_IEs },
  { &hf_rrc_v590NonCriticalExtensions, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v590NonCriticalExtensions },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v4b0NonCriticalExtensions(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v4b0NonCriticalExtensions, T_v4b0NonCriticalExtensions_sequence);

  return offset;
}


static const per_sequence_t T_laterNonCriticalExtensions_sequence[] = {
  { &hf_rrc_activeSetUpdate_r3_add_ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_BIT_STRING },
  { &hf_rrc_v4b0NonCriticalExtensions, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v4b0NonCriticalExtensions },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_laterNonCriticalExtensions(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_laterNonCriticalExtensions, T_laterNonCriticalExtensions_sequence);

  return offset;
}


static const per_sequence_t T_r3_sequence[] = {
  { &hf_rrc_activeSetUpdate_r3, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_ActiveSetUpdate_r3_IEs },
  { &hf_rrc_laterNonCriticalExtensions, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_laterNonCriticalExtensions },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_r3(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_r3, T_r3_sequence);

  return offset;
}



static int
dissect_rrc_H_RNTI(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
#line 780 "../../asn1/rrc/rrc.cnf"
  tvbuff_t *hrnti_tvb;
  struct rrc_info *rrcinf;
  offset = dissect_per_bit_string(tvb, offset, actx, tree, hf_index,
                                     16, 16, FALSE, &hrnti_tvb);




#line 785 "../../asn1/rrc/rrc.cnf"
  rrcinf = p_get_proto_data(actx->pinfo->fd, proto_rrc);
  if (!rrcinf) {
    rrcinf = se_alloc0(sizeof(struct rrc_info));
    p_add_proto_data(actx->pinfo->fd, proto_rrc, rrcinf);
  }
  rrcinf->hrnti[actx->pinfo->fd->subnum] = tvb_get_ntohs(hrnti_tvb, 0);


  return offset;
}



static int
dissect_rrc_E_RNTI(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_bit_string(tvb, offset, actx, tree, hf_index,
                                     16, 16, FALSE, NULL);

  return offset;
}



static int
dissect_rrc_T_cn_CommonGSM_MAP_NAS_SysInfo_01(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
#line 662 "../../asn1/rrc/rrc.cnf"
  rrc_nas_sys_info_gsm_map_type = RRC_NAS_SYS_INFO_CN_COMMON;
  offset = dissect_rrc_NAS_SystemInformationGSM_MAP(tvb, offset, actx, tree, hf_index);




  return offset;
}


static const per_sequence_t CN_InformationInfo_r6_sequence[] = {
  { &hf_rrc_plmn_Identity   , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_PLMN_Identity },
  { &hf_rrc_cn_CommonGSM_MAP_NAS_SysInfo_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_cn_CommonGSM_MAP_NAS_SysInfo_01 },
  { &hf_rrc_cn_DomainInformationList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CN_DomainInformationList },
  { &hf_rrc_primary_plmn_Identity, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_PLMN_Identity },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_CN_InformationInfo_r6(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_CN_InformationInfo_r6, CN_InformationInfo_r6_sequence);

  return offset;
}


static const per_sequence_t T_fdd_72_sequence[] = {
  { &hf_rrc_pCPICH_UsageForChannelEst, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PCPICH_UsageForChannelEst },
  { &hf_rrc_dpch_FrameOffset, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DPCH_FrameOffset },
  { &hf_rrc_secondaryCPICH_Info, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_SecondaryCPICH_Info },
  { &hf_rrc_dl_ChannelisationCodeList, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DL_ChannelisationCodeList },
  { &hf_rrc_tpc_CombinationIndex, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TPC_CombinationIndex },
  { &hf_rrc_powerOffsetTPC_pdpdch, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_PowerOffsetTPC_pdpdch },
  { &hf_rrc_dummy_41        , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_SSDT_CellIdentity },
  { &hf_rrc_closedLoopTimingAdjMode, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_ClosedLoopTimingAdjMode },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_fdd_72(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_fdd_72, T_fdd_72_sequence);

  return offset;
}


static const per_sequence_t T_tdd384_11_sequence[] = {
  { &hf_rrc_dl_CCTrCH_TimeslotsCodes, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DownlinkTimeslotsCodes },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_tdd384_11(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_tdd384_11, T_tdd384_11_sequence);

  return offset;
}



static int
dissect_rrc_TimeslotNumber_LCR_r4(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            0U, 6U, NULL, FALSE);

  return offset;
}


static const per_sequence_t T_ueSpecificMidamble_20_sequence[] = {
  { &hf_rrc_midambleShift_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_0_15 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_ueSpecificMidamble_20(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_ueSpecificMidamble_20, T_ueSpecificMidamble_20_sequence);

  return offset;
}


static const value_string rrc_T_midambleAllocationMode_28_vals[] = {
  {   0, "defaultMidamble" },
  {   1, "commonMidamble" },
  {   2, "ueSpecificMidamble" },
  { 0, NULL }
};

static const per_choice_t T_midambleAllocationMode_28_choice[] = {
  {   0, &hf_rrc_defaultMidamble , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   1, &hf_rrc_commonMidamble  , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   2, &hf_rrc_ueSpecificMidamble_21, ASN1_NO_EXTENSIONS     , dissect_rrc_T_ueSpecificMidamble_20 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_midambleAllocationMode_28(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_midambleAllocationMode_28, T_midambleAllocationMode_28_choice,
                                 NULL);

  return offset;
}



static int
dissect_rrc_INTEGER_1_8(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            1U, 8U, NULL, FALSE);

  return offset;
}


static const per_sequence_t MidambleShiftAndBurstType_LCR_r4_sequence[] = {
  { &hf_rrc_midambleAllocationMode_28, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_midambleAllocationMode_28 },
  { &hf_rrc_midambleConfiguration, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_1_8 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_MidambleShiftAndBurstType_LCR_r4(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_MidambleShiftAndBurstType_LCR_r4, MidambleShiftAndBurstType_LCR_r4_sequence);

  return offset;
}


static const value_string rrc_T_modulation_01_vals[] = {
  {   0, "mod-QPSK" },
  {   1, "mod-8PSK" },
  { 0, NULL }
};


static int
dissect_rrc_T_modulation_01(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     2, NULL, FALSE, 0, NULL);

  return offset;
}


static const value_string rrc_T_ss_TPC_Symbols_vals[] = {
  {   0, "zero" },
  {   1, "one" },
  {   2, "sixteenOverSF" },
  { 0, NULL }
};


static int
dissect_rrc_T_ss_TPC_Symbols(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     3, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t IndividualTimeslotInfo_LCR_r4_sequence[] = {
  { &hf_rrc_timeslotNumber_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TimeslotNumber_LCR_r4 },
  { &hf_rrc_tfci_Existence  , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { &hf_rrc_midambleShiftAndBurstType_03, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MidambleShiftAndBurstType_LCR_r4 },
  { &hf_rrc_modulation_01   , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_modulation_01 },
  { &hf_rrc_ss_TPC_Symbols  , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_ss_TPC_Symbols },
  { &hf_rrc_additionalSS_TPC_Symbols, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_INTEGER_1_15 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_IndividualTimeslotInfo_LCR_r4(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_IndividualTimeslotInfo_LCR_r4, IndividualTimeslotInfo_LCR_r4_sequence);

  return offset;
}



static int
dissect_rrc_INTEGER_1_maxTS_LCR_1(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            1U, maxTS_LCR_1, NULL, FALSE);

  return offset;
}


static const per_sequence_t T_sameAsLast_02_sequence[] = {
  { &hf_rrc_timeslotNumber_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TimeslotNumber_LCR_r4 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_sameAsLast_02(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_sameAsLast_02, T_sameAsLast_02_sequence);

  return offset;
}


static const per_sequence_t T_newParameters_02_sequence[] = {
  { &hf_rrc_individualTimeslotInfo_02, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_IndividualTimeslotInfo_LCR_r4 },
  { &hf_rrc_dl_TS_ChannelisationCodesShort, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DL_TS_ChannelisationCodesShort },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_newParameters_02(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_newParameters_02, T_newParameters_02_sequence);

  return offset;
}


static const value_string rrc_T_parameters_02_vals[] = {
  {   0, "sameAsLast" },
  {   1, "newParameters" },
  { 0, NULL }
};

static const per_choice_t T_parameters_02_choice[] = {
  {   0, &hf_rrc_sameAsLast_02   , ASN1_NO_EXTENSIONS     , dissect_rrc_T_sameAsLast_02 },
  {   1, &hf_rrc_newParameters_02, ASN1_NO_EXTENSIONS     , dissect_rrc_T_newParameters_02 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_parameters_02(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_parameters_02, T_parameters_02_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t DownlinkAdditionalTimeslots_LCR_r4_sequence[] = {
  { &hf_rrc_parameters_02   , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_parameters_02 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_DownlinkAdditionalTimeslots_LCR_r4(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_DownlinkAdditionalTimeslots_LCR_r4, DownlinkAdditionalTimeslots_LCR_r4_sequence);

  return offset;
}


static const per_sequence_t SEQUENCE_SIZE_1_maxTS_LCR_1_OF_DownlinkAdditionalTimeslots_LCR_r4_sequence_of[1] = {
  { &hf_rrc_timeslotList_item_02, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DownlinkAdditionalTimeslots_LCR_r4 },
};

static int
dissect_rrc_SEQUENCE_SIZE_1_maxTS_LCR_1_OF_DownlinkAdditionalTimeslots_LCR_r4(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_SEQUENCE_SIZE_1_maxTS_LCR_1_OF_DownlinkAdditionalTimeslots_LCR_r4, SEQUENCE_SIZE_1_maxTS_LCR_1_OF_DownlinkAdditionalTimeslots_LCR_r4_sequence_of,
                                                  1, maxTS_LCR_1, FALSE);

  return offset;
}


static const value_string rrc_T_additionalTimeslots_02_vals[] = {
  {   0, "consecutive" },
  {   1, "timeslotList" },
  { 0, NULL }
};

static const per_choice_t T_additionalTimeslots_02_choice[] = {
  {   0, &hf_rrc_consecutive_03  , ASN1_NO_EXTENSIONS     , dissect_rrc_INTEGER_1_maxTS_LCR_1 },
  {   1, &hf_rrc_timeslotList_02 , ASN1_NO_EXTENSIONS     , dissect_rrc_SEQUENCE_SIZE_1_maxTS_LCR_1_OF_DownlinkAdditionalTimeslots_LCR_r4 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_additionalTimeslots_02(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_additionalTimeslots_02, T_additionalTimeslots_02_choice,
                                 NULL);

  return offset;
}


static const value_string rrc_T_moreTimeslots_02_vals[] = {
  {   0, "noMore" },
  {   1, "additionalTimeslots" },
  { 0, NULL }
};

static const per_choice_t T_moreTimeslots_02_choice[] = {
  {   0, &hf_rrc_noMore          , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   1, &hf_rrc_additionalTimeslots_02, ASN1_NO_EXTENSIONS     , dissect_rrc_T_additionalTimeslots_02 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_moreTimeslots_02(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_moreTimeslots_02, T_moreTimeslots_02_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t DownlinkTimeslotsCodes_LCR_r4_sequence[] = {
  { &hf_rrc_firstIndividualTimeslotInfo_02, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_IndividualTimeslotInfo_LCR_r4 },
  { &hf_rrc_dl_TS_ChannelisationCodesShort, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DL_TS_ChannelisationCodesShort },
  { &hf_rrc_moreTimeslots_02, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_moreTimeslots_02 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_DownlinkTimeslotsCodes_LCR_r4(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_DownlinkTimeslotsCodes_LCR_r4, DownlinkTimeslotsCodes_LCR_r4_sequence);

  return offset;
}


static const per_sequence_t T_tdd128_12_sequence[] = {
  { &hf_rrc_dl_CCTrCH_TimeslotsCodes_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DownlinkTimeslotsCodes_LCR_r4 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_tdd128_12(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_tdd128_12, T_tdd128_12_sequence);

  return offset;
}


static const value_string rrc_T_tddOption_03_vals[] = {
  {   0, "tdd384" },
  {   1, "tdd128" },
  { 0, NULL }
};

static const per_choice_t T_tddOption_03_choice[] = {
  {   0, &hf_rrc_tdd384_12       , ASN1_NO_EXTENSIONS     , dissect_rrc_T_tdd384_11 },
  {   1, &hf_rrc_tdd128_12       , ASN1_NO_EXTENSIONS     , dissect_rrc_T_tdd128_12 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_tddOption_03(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_tddOption_03, T_tddOption_03_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t DL_CCTrCh_r4_sequence[] = {
  { &hf_rrc_tfcs_ID_01      , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_TFCS_IdentityPlain },
  { &hf_rrc_timeInfo        , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TimeInfo },
  { &hf_rrc_commonTimeslotInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CommonTimeslotInfo },
  { &hf_rrc_tddOption_03    , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_tddOption_03 },
  { &hf_rrc_ul_CCTrChTPCList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_CCTrChTPCList },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_DL_CCTrCh_r4(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_DL_CCTrCh_r4, DL_CCTrCh_r4_sequence);

  return offset;
}


static const per_sequence_t DL_CCTrChList_r4_sequence_of[1] = {
  { &hf_rrc_DL_CCTrChList_r4_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DL_CCTrCh_r4 },
};

static int
dissect_rrc_DL_CCTrChList_r4(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_DL_CCTrChList_r4, DL_CCTrChList_r4_sequence_of,
                                                  1, maxCCTrCH, FALSE);

  return offset;
}


static const per_sequence_t T_tdd_34_sequence[] = {
  { &hf_rrc_dl_CCTrChListToEstablish_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_CCTrChList_r4 },
  { &hf_rrc_dl_CCTrChListToRemove, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_CCTrChListToRemove },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_tdd_34(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_tdd_34, T_tdd_34_sequence);

  return offset;
}


static const value_string rrc_DL_DPCH_InfoPerRL_r5_vals[] = {
  {   0, "fdd" },
  {   1, "tdd" },
  { 0, NULL }
};

static const per_choice_t DL_DPCH_InfoPerRL_r5_choice[] = {
  {   0, &hf_rrc_fdd_73          , ASN1_NO_EXTENSIONS     , dissect_rrc_T_fdd_72 },
  {   1, &hf_rrc_tdd_35          , ASN1_NO_EXTENSIONS     , dissect_rrc_T_tdd_34 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_DL_DPCH_InfoPerRL_r5(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_DL_DPCH_InfoPerRL_r5, DL_DPCH_InfoPerRL_r5_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t DL_FDPCH_InfoPerRL_r6_sequence[] = {
  { &hf_rrc_pCPICH_UsageForChannelEst, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PCPICH_UsageForChannelEst },
  { &hf_rrc_fdpch_FrameOffset, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DPCH_FrameOffset },
  { &hf_rrc_secondaryCPICH_Info, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_SecondaryCPICH_Info },
  { &hf_rrc_secondaryScramblingCode, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_SecondaryScramblingCode },
  { &hf_rrc_dl_ChannelisationCode, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_0_255 },
  { &hf_rrc_tpc_CombinationIndex, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TPC_CombinationIndex },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_DL_FDPCH_InfoPerRL_r6(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_DL_FDPCH_InfoPerRL_r6, DL_FDPCH_InfoPerRL_r6_sequence);

  return offset;
}


static const value_string rrc_T_dl_dpchInfo_04_vals[] = {
  {   0, "dl-DPCH-InfoPerRL" },
  {   1, "dl-FDPCH-InfoPerRL" },
  { 0, NULL }
};

static const per_choice_t T_dl_dpchInfo_04_choice[] = {
  {   0, &hf_rrc_dl_DPCH_InfoPerRL_02, ASN1_NO_EXTENSIONS     , dissect_rrc_DL_DPCH_InfoPerRL_r5 },
  {   1, &hf_rrc_dl_FDPCH_InfoPerRL, ASN1_NO_EXTENSIONS     , dissect_rrc_DL_FDPCH_InfoPerRL_r6 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_dl_dpchInfo_04(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_dl_dpchInfo_04, T_dl_dpchInfo_04_choice,
                                 NULL);

  return offset;
}



static int
dissect_rrc_E_HICH_ChannelisationCode(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            0U, 127U, NULL, FALSE);

  return offset;
}



static int
dissect_rrc_E_HICH_RGCH_SignatureSequence(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            0U, 39U, NULL, FALSE);

  return offset;
}


static const per_sequence_t E_HICH_Information_sequence[] = {
  { &hf_rrc_channelisationCode_02, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_E_HICH_ChannelisationCode },
  { &hf_rrc_signatureSequence_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_E_HICH_RGCH_SignatureSequence },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_E_HICH_Information(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_E_HICH_Information, E_HICH_Information_sequence);

  return offset;
}



static int
dissect_rrc_E_RGCH_CombinationIndex(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            0U, 5U, NULL, FALSE);

  return offset;
}


static const per_sequence_t E_RGCH_Information_sequence[] = {
  { &hf_rrc_signatureSequence_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_E_HICH_RGCH_SignatureSequence },
  { &hf_rrc_rg_CombinationIndex, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_E_RGCH_CombinationIndex },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_E_RGCH_Information(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_E_RGCH_Information, E_RGCH_Information_sequence);

  return offset;
}


static const per_sequence_t RL_AdditionInformation_r6_sequence[] = {
  { &hf_rrc_primaryCPICH_Info, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PrimaryCPICH_Info },
  { &hf_rrc_cell_Id         , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CellIdentity },
  { &hf_rrc_dl_dpchInfo_04  , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_dl_dpchInfo_04 },
  { &hf_rrc_e_HICH_Information, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_E_HICH_Information },
  { &hf_rrc_e_RGCH_Information, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_E_RGCH_Information },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_RL_AdditionInformation_r6(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_RL_AdditionInformation_r6, RL_AdditionInformation_r6_sequence);

  return offset;
}


static const per_sequence_t RL_AdditionInformationList_r6_sequence_of[1] = {
  { &hf_rrc_RL_AdditionInformationList_r6_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RL_AdditionInformation_r6 },
};

static int
dissect_rrc_RL_AdditionInformationList_r6(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_RL_AdditionInformationList_r6, RL_AdditionInformationList_r6_sequence_of,
                                                  1, maxRL_1, FALSE);

  return offset;
}



static int
dissect_rrc_DeltaACK(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            0U, 8U, NULL, FALSE);

  return offset;
}



static int
dissect_rrc_DeltaNACK(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            0U, 8U, NULL, FALSE);

  return offset;
}



static int
dissect_rrc_HARQ_Preamble_Mode(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            0U, 1U, NULL, FALSE);

  return offset;
}



static int
dissect_rrc_HS_SCCH_Codes(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            0U, 127U, NULL, FALSE);

  return offset;
}


static const per_sequence_t SEQUENCE_SIZE_1_maxHSSCCHs_OF_HS_SCCH_Codes_sequence_of[1] = {
  { &hf_rrc_hS_SCCHChannelisationCodeInfo_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_HS_SCCH_Codes },
};

static int
dissect_rrc_SEQUENCE_SIZE_1_maxHSSCCHs_OF_HS_SCCH_Codes(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_SEQUENCE_SIZE_1_maxHSSCCHs_OF_HS_SCCH_Codes, SEQUENCE_SIZE_1_maxHSSCCHs_OF_HS_SCCH_Codes_sequence_of,
                                                  1, maxHSSCCHs, FALSE);

  return offset;
}


static const per_sequence_t T_fdd_93_sequence[] = {
  { &hf_rrc_hS_SCCHChannelisationCodeInfo, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SEQUENCE_SIZE_1_maxHSSCCHs_OF_HS_SCCH_Codes },
  { &hf_rrc_dl_ScramblingCode, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_SecondaryScramblingCode },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_fdd_93(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_fdd_93, T_fdd_93_sequence);

  return offset;
}



static int
dissect_rrc_INTEGER_M7_8(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            -7, 8U, NULL, FALSE);

  return offset;
}



static int
dissect_rrc_INTEGER_M22_40(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            -22, 40U, NULL, FALSE);

  return offset;
}



static int
dissect_rrc_ConstantValue(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            -35, -10, NULL, FALSE);

  return offset;
}


static const per_sequence_t HS_SICH_Power_Control_Info_TDD384_sequence[] = {
  { &hf_rrc_ul_target_SIR   , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_M22_40 },
  { &hf_rrc_hs_sich_ConstantValue, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_ConstantValue },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_HS_SICH_Power_Control_Info_TDD384(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_HS_SICH_Power_Control_Info_TDD384, HS_SICH_Power_Control_Info_TDD384_sequence);

  return offset;
}


static const per_sequence_t T_ueSpecificMidamble_02_sequence[] = {
  { &hf_rrc_midambleShift   , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MidambleShiftLong },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_ueSpecificMidamble_02(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_ueSpecificMidamble_02, T_ueSpecificMidamble_02_sequence);

  return offset;
}


static const value_string rrc_T_midambleAllocationMode_10_vals[] = {
  {   0, "defaultMidamble" },
  {   1, "commonMidamble" },
  {   2, "ueSpecificMidamble" },
  { 0, NULL }
};

static const per_choice_t T_midambleAllocationMode_10_choice[] = {
  {   0, &hf_rrc_defaultMidamble , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   1, &hf_rrc_commonMidamble  , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   2, &hf_rrc_ueSpecificMidamble_03, ASN1_NO_EXTENSIONS     , dissect_rrc_T_ueSpecificMidamble_02 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_midambleAllocationMode_10(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_midambleAllocationMode_10, T_midambleAllocationMode_10_choice,
                                 NULL);

  return offset;
}



static int
dissect_rrc_Bler_Target(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            -63, 0U, NULL, FALSE);

  return offset;
}


static const per_sequence_t T_ueSpecificMidamble_05_sequence[] = {
  { &hf_rrc_midambleShift   , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MidambleShiftLong },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_ueSpecificMidamble_05(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_ueSpecificMidamble_05, T_ueSpecificMidamble_05_sequence);

  return offset;
}


static const value_string rrc_T_midambleAllocationMode_13_vals[] = {
  {   0, "defaultMidamble" },
  {   1, "ueSpecificMidamble" },
  { 0, NULL }
};

static const per_choice_t T_midambleAllocationMode_13_choice[] = {
  {   0, &hf_rrc_defaultMidamble , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   1, &hf_rrc_ueSpecificMidamble_06, ASN1_NO_EXTENSIONS     , dissect_rrc_T_ueSpecificMidamble_05 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_midambleAllocationMode_13(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_midambleAllocationMode_13, T_midambleAllocationMode_13_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t HS_SICH_Configuration_TDD384_sequence[] = {
  { &hf_rrc_timeslotNumber  , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TimeslotNumber },
  { &hf_rrc_channelisationCode_05, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DL_TS_ChannelisationCode },
  { &hf_rrc_midambleAllocationMode_13, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_midambleAllocationMode_13 },
  { &hf_rrc_midambleconfiguration, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MidambleConfigurationBurstType1and3 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_HS_SICH_Configuration_TDD384(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_HS_SICH_Configuration_TDD384, HS_SICH_Configuration_TDD384_sequence);

  return offset;
}


static const per_sequence_t HS_SCCH_TDD384_sequence[] = {
  { &hf_rrc_timeslotNumber  , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TimeslotNumber },
  { &hf_rrc_channelisationCode_05, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DL_TS_ChannelisationCode },
  { &hf_rrc_midambleAllocationMode_10, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_midambleAllocationMode_10 },
  { &hf_rrc_midambleconfiguration, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MidambleConfigurationBurstType1and3 },
  { &hf_rrc_bler_target     , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_Bler_Target },
  { &hf_rrc_hs_sich_configuration_02, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_HS_SICH_Configuration_TDD384 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_HS_SCCH_TDD384(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_HS_SCCH_TDD384, HS_SCCH_TDD384_sequence);

  return offset;
}


static const per_sequence_t SEQUENCE_SIZE_1_maxHSSCCHs_OF_HS_SCCH_TDD384_sequence_of[1] = {
  { &hf_rrc_hS_SCCH_SetConfiguration_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_HS_SCCH_TDD384 },
};

static int
dissect_rrc_SEQUENCE_SIZE_1_maxHSSCCHs_OF_HS_SCCH_TDD384(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_SEQUENCE_SIZE_1_maxHSSCCHs_OF_HS_SCCH_TDD384, SEQUENCE_SIZE_1_maxHSSCCHs_OF_HS_SCCH_TDD384_sequence_of,
                                                  1, maxHSSCCHs, FALSE);

  return offset;
}


static const per_sequence_t T_tdd384_23_sequence[] = {
  { &hf_rrc_nack_ack_power_offset, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_M7_8 },
  { &hf_rrc_hs_SICH_PowerControl_Info, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_HS_SICH_Power_Control_Info_TDD384 },
  { &hf_rrc_hS_SCCH_SetConfiguration, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SEQUENCE_SIZE_1_maxHSSCCHs_OF_HS_SCCH_TDD384 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_tdd384_23(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_tdd384_23, T_tdd384_23_sequence);

  return offset;
}


static const value_string rrc_HS_ChannelisationCode_LCR_vals[] = {
  {   0, "cc16-1" },
  {   1, "cc16-2" },
  {   2, "cc16-3" },
  {   3, "cc16-4" },
  {   4, "cc16-5" },
  {   5, "cc16-6" },
  {   6, "cc16-7" },
  {   7, "cc16-8" },
  {   8, "cc16-9" },
  {   9, "cc16-10" },
  {  10, "cc16-11" },
  {  11, "cc16-12" },
  {  12, "cc16-13" },
  {  13, "cc16-14" },
  {  14, "cc16-15" },
  {  15, "cc16-16" },
  { 0, NULL }
};


static int
dissect_rrc_HS_ChannelisationCode_LCR(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     16, NULL, FALSE, 0, NULL);

  return offset;
}


static const value_string rrc_T_midambleAllocationMode_05_vals[] = {
  {   0, "defaultMidamble" },
  {   1, "commonMidamble" },
  {   2, "ueSpecificMidamble" },
  { 0, NULL }
};

static const per_choice_t T_midambleAllocationMode_05_choice[] = {
  {   0, &hf_rrc_defaultMidamble , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   1, &hf_rrc_commonMidamble  , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   2, &hf_rrc_ueSpecificMidamble, ASN1_NO_EXTENSIONS     , dissect_rrc_INTEGER_0_15 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_midambleAllocationMode_05(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_midambleAllocationMode_05, T_midambleAllocationMode_05_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t T_ueSpecificMidamble_sequence[] = {
  { &hf_rrc_midambleShift   , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MidambleShiftLong },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_ueSpecificMidamble(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_ueSpecificMidamble, T_ueSpecificMidamble_sequence);

  return offset;
}


static const value_string rrc_T_midambleAllocationMode_08_vals[] = {
  {   0, "defaultMidamble" },
  {   1, "ueSpecificMidamble" },
  { 0, NULL }
};

static const per_choice_t T_midambleAllocationMode_08_choice[] = {
  {   0, &hf_rrc_defaultMidamble , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   1, &hf_rrc_ueSpecificMidamble_01, ASN1_NO_EXTENSIONS     , dissect_rrc_T_ueSpecificMidamble },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_midambleAllocationMode_08(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_midambleAllocationMode_08, T_midambleAllocationMode_08_choice,
                                 NULL);

  return offset;
}



static int
dissect_rrc_INTEGER_M120_M58(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            -120, -58, NULL, FALSE);

  return offset;
}


static const value_string rrc_T_tpc_step_size_04_vals[] = {
  {   0, "s1" },
  {   1, "s2" },
  {   2, "s3" },
  {   3, "spare1" },
  { 0, NULL }
};


static int
dissect_rrc_T_tpc_step_size_04(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     4, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t HS_SICH_Configuration_TDD128_sequence[] = {
  { &hf_rrc_timeslotNumber_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TimeslotNumber_LCR_r4 },
  { &hf_rrc_channelisationCode_04, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_HS_ChannelisationCode_LCR },
  { &hf_rrc_midambleAllocationMode_08, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_midambleAllocationMode_08 },
  { &hf_rrc_midambleConfiguration, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_1_8 },
  { &hf_rrc_nack_ack_power_offset, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_M7_8 },
  { &hf_rrc_power_level_HSSICH, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_M120_M58 },
  { &hf_rrc_tpc_step_size_04, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_tpc_step_size_04 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_HS_SICH_Configuration_TDD128(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_HS_SICH_Configuration_TDD128, HS_SICH_Configuration_TDD128_sequence);

  return offset;
}


static const per_sequence_t HS_SCCH_TDD128_sequence[] = {
  { &hf_rrc_timeslotNumber_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TimeslotNumber_LCR_r4 },
  { &hf_rrc_firstChannelisationCode_03, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_HS_ChannelisationCode_LCR },
  { &hf_rrc_secondChannelisationCode, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_HS_ChannelisationCode_LCR },
  { &hf_rrc_midambleAllocationMode_05, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_midambleAllocationMode_05 },
  { &hf_rrc_midambleConfiguration, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_1_8 },
  { &hf_rrc_bler_target     , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_Bler_Target },
  { &hf_rrc_hs_sich_configuration, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_HS_SICH_Configuration_TDD128 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_HS_SCCH_TDD128(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_HS_SCCH_TDD128, HS_SCCH_TDD128_sequence);

  return offset;
}


static const per_sequence_t SEQUENCE_SIZE_1_maxHSSCCHs_OF_HS_SCCH_TDD128_sequence_of[1] = {
  { &hf_rrc_tdd128_item     , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_HS_SCCH_TDD128 },
};

static int
dissect_rrc_SEQUENCE_SIZE_1_maxHSSCCHs_OF_HS_SCCH_TDD128(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_SEQUENCE_SIZE_1_maxHSSCCHs_OF_HS_SCCH_TDD128, SEQUENCE_SIZE_1_maxHSSCCHs_OF_HS_SCCH_TDD128_sequence_of,
                                                  1, maxHSSCCHs, FALSE);

  return offset;
}


static const value_string rrc_T_tdd_50_vals[] = {
  {   0, "tdd384" },
  {   1, "tdd128" },
  { 0, NULL }
};

static const per_choice_t T_tdd_50_choice[] = {
  {   0, &hf_rrc_tdd384_25       , ASN1_NO_EXTENSIONS     , dissect_rrc_T_tdd384_23 },
  {   1, &hf_rrc_tdd128_35       , ASN1_NO_EXTENSIONS     , dissect_rrc_SEQUENCE_SIZE_1_maxHSSCCHs_OF_HS_SCCH_TDD128 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_tdd_50(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_tdd_50, T_tdd_50_choice,
                                 NULL);

  return offset;
}


static const value_string rrc_T_modeSpecificInfo_62_vals[] = {
  {   0, "fdd" },
  {   1, "tdd" },
  { 0, NULL }
};

static const per_choice_t T_modeSpecificInfo_62_choice[] = {
  {   0, &hf_rrc_fdd_95          , ASN1_NO_EXTENSIONS     , dissect_rrc_T_fdd_93 },
  {   1, &hf_rrc_tdd_54          , ASN1_NO_EXTENSIONS     , dissect_rrc_T_tdd_50 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_modeSpecificInfo_62(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_modeSpecificInfo_62, T_modeSpecificInfo_62_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t HS_SCCH_Info_sequence[] = {
  { &hf_rrc_modeSpecificInfo_62, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_modeSpecificInfo_62 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_HS_SCCH_Info(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_HS_SCCH_Info, HS_SCCH_Info_sequence);

  return offset;
}



static int
dissect_rrc_MeasurementPowerOffset(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            -12, 26U, NULL, FALSE);

  return offset;
}


static const value_string rrc_Feedback_cycle_vals[] = {
  {   0, "fc0" },
  {   1, "fc2" },
  {   2, "fc4" },
  {   3, "fc8" },
  {   4, "fc10" },
  {   5, "fc20" },
  {   6, "fc40" },
  {   7, "fc80" },
  {   8, "fc160" },
  { 0, NULL }
};


static int
dissect_rrc_Feedback_cycle(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     9, NULL, FALSE, 0, NULL);

  return offset;
}



static int
dissect_rrc_CQI_RepetitionFactor(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            1U, 4U, NULL, FALSE);

  return offset;
}



static int
dissect_rrc_DeltaCQI(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            0U, 8U, NULL, FALSE);

  return offset;
}


static const per_sequence_t T_fdd_97_sequence[] = {
  { &hf_rrc_measurementPowerOffset, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MeasurementPowerOffset },
  { &hf_rrc_feedback_cycle  , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_Feedback_cycle },
  { &hf_rrc_cqi_RepetitionFactor, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CQI_RepetitionFactor },
  { &hf_rrc_deltaCQI        , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DeltaCQI },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_fdd_97(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_fdd_97, T_fdd_97_sequence);

  return offset;
}


static const value_string rrc_T_modeSpecificInfo_67_vals[] = {
  {   0, "fdd" },
  {   1, "tdd" },
  { 0, NULL }
};

static const per_choice_t T_modeSpecificInfo_67_choice[] = {
  {   0, &hf_rrc_fdd_99          , ASN1_NO_EXTENSIONS     , dissect_rrc_T_fdd_97 },
  {   1, &hf_rrc_tdd             , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_modeSpecificInfo_67(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_modeSpecificInfo_67, T_modeSpecificInfo_67_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t Measurement_Feedback_Info_sequence[] = {
  { &hf_rrc_modeSpecificInfo_67, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_modeSpecificInfo_67 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_Measurement_Feedback_Info(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_Measurement_Feedback_Info, Measurement_Feedback_Info_sequence);

  return offset;
}


static const per_sequence_t T_ueSpecificMidamble_16_sequence[] = {
  { &hf_rrc_midambleShift   , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MidambleShiftLong },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_ueSpecificMidamble_16(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_ueSpecificMidamble_16, T_ueSpecificMidamble_16_sequence);

  return offset;
}


static const value_string rrc_T_midambleAllocationMode_24_vals[] = {
  {   0, "defaultMidamble" },
  {   1, "commonMidamble" },
  {   2, "ueSpecificMidamble" },
  { 0, NULL }
};

static const per_choice_t T_midambleAllocationMode_24_choice[] = {
  {   0, &hf_rrc_defaultMidamble , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   1, &hf_rrc_commonMidamble  , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   2, &hf_rrc_ueSpecificMidamble_17, ASN1_NO_EXTENSIONS     , dissect_rrc_T_ueSpecificMidamble_16 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_midambleAllocationMode_24(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_midambleAllocationMode_24, T_midambleAllocationMode_24_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t T_type1_04_sequence[] = {
  { &hf_rrc_midambleConfigurationBurstType1and3, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MidambleConfigurationBurstType1and3 },
  { &hf_rrc_midambleAllocationMode_24, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_midambleAllocationMode_24 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_type1_04(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_type1_04, T_type1_04_sequence);

  return offset;
}


static const per_sequence_t T_ueSpecificMidamble_17_sequence[] = {
  { &hf_rrc_midambleShift_02, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MidambleShiftShort },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_ueSpecificMidamble_17(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_ueSpecificMidamble_17, T_ueSpecificMidamble_17_sequence);

  return offset;
}


static const value_string rrc_T_midambleAllocationMode_25_vals[] = {
  {   0, "defaultMidamble" },
  {   1, "commonMidamble" },
  {   2, "ueSpecificMidamble" },
  { 0, NULL }
};

static const per_choice_t T_midambleAllocationMode_25_choice[] = {
  {   0, &hf_rrc_defaultMidamble , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   1, &hf_rrc_commonMidamble  , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   2, &hf_rrc_ueSpecificMidamble_18, ASN1_NO_EXTENSIONS     , dissect_rrc_T_ueSpecificMidamble_17 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_midambleAllocationMode_25(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_midambleAllocationMode_25, T_midambleAllocationMode_25_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t T_type2_03_sequence[] = {
  { &hf_rrc_midambleConfigurationBurstType2, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MidambleConfigurationBurstType2 },
  { &hf_rrc_midambleAllocationMode_25, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_midambleAllocationMode_25 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_type2_03(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_type2_03, T_type2_03_sequence);

  return offset;
}


static const value_string rrc_T_burstType_03_vals[] = {
  {   0, "type1" },
  {   1, "type2" },
  { 0, NULL }
};

static const per_choice_t T_burstType_03_choice[] = {
  {   0, &hf_rrc_type1_04        , ASN1_NO_EXTENSIONS     , dissect_rrc_T_type1_04 },
  {   1, &hf_rrc_type2_03        , ASN1_NO_EXTENSIONS     , dissect_rrc_T_type2_03 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_burstType_03(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_burstType_03, T_burstType_03_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t MidambleShiftAndBurstType_DL_sequence[] = {
  { &hf_rrc_burstType_04    , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_burstType_03 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_MidambleShiftAndBurstType_DL(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_MidambleShiftAndBurstType_DL, MidambleShiftAndBurstType_DL_sequence);

  return offset;
}


static const per_sequence_t DL_HSPDSCH_TS_Configuration_item_sequence[] = {
  { &hf_rrc_timeslot        , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TimeslotNumber },
  { &hf_rrc_midambleShiftAndBurstType, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MidambleShiftAndBurstType_DL },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_DL_HSPDSCH_TS_Configuration_item(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_DL_HSPDSCH_TS_Configuration_item, DL_HSPDSCH_TS_Configuration_item_sequence);

  return offset;
}


static const per_sequence_t DL_HSPDSCH_TS_Configuration_sequence_of[1] = {
  { &hf_rrc_DL_HSPDSCH_TS_Configuration_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DL_HSPDSCH_TS_Configuration_item },
};

static int
dissect_rrc_DL_HSPDSCH_TS_Configuration(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_DL_HSPDSCH_TS_Configuration, DL_HSPDSCH_TS_Configuration_sequence_of,
                                                  1, maxTS_1, FALSE);

  return offset;
}


static const per_sequence_t T_tdd384_13_sequence[] = {
  { &hf_rrc_dl_HSPDSCH_TS_Configuration, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_HSPDSCH_TS_Configuration },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_tdd384_13(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_tdd384_13, T_tdd384_13_sequence);

  return offset;
}


static const value_string rrc_T_midambleAllocationMode_04_vals[] = {
  {   0, "defaultMidamble" },
  {   1, "commonMidamble" },
  {   2, "ueSpecificMidamble" },
  { 0, NULL }
};

static const per_choice_t T_midambleAllocationMode_04_choice[] = {
  {   0, &hf_rrc_defaultMidamble , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   1, &hf_rrc_commonMidamble  , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   2, &hf_rrc_ueSpecificMidamble, ASN1_NO_EXTENSIONS     , dissect_rrc_INTEGER_0_15 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_midambleAllocationMode_04(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_midambleAllocationMode_04, T_midambleAllocationMode_04_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t HS_PDSCH_Midamble_Configuration_TDD128_sequence[] = {
  { &hf_rrc_midambleAllocationMode_04, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_midambleAllocationMode_04 },
  { &hf_rrc_midambleConfiguration, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_1_8 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_HS_PDSCH_Midamble_Configuration_TDD128(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_HS_PDSCH_Midamble_Configuration_TDD128, HS_PDSCH_Midamble_Configuration_TDD128_sequence);

  return offset;
}


static const per_sequence_t T_tdd128_21_sequence[] = {
  { &hf_rrc_hs_PDSCH_Midamble_Configuration_tdd128, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_HS_PDSCH_Midamble_Configuration_TDD128 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_tdd128_21(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_tdd128_21, T_tdd128_21_sequence);

  return offset;
}


static const value_string rrc_T_tdd_38_vals[] = {
  {   0, "tdd384" },
  {   1, "tdd128" },
  { 0, NULL }
};

static const per_choice_t T_tdd_38_choice[] = {
  {   0, &hf_rrc_tdd384_14       , ASN1_NO_EXTENSIONS     , dissect_rrc_T_tdd384_13 },
  {   1, &hf_rrc_tdd128_21       , ASN1_NO_EXTENSIONS     , dissect_rrc_T_tdd128_21 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_tdd_38(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_tdd_38, T_tdd_38_choice,
                                 NULL);

  return offset;
}


static const value_string rrc_T_modeSpecificInfo_41_vals[] = {
  {   0, "tdd" },
  {   1, "fdd" },
  { 0, NULL }
};

static const per_choice_t T_modeSpecificInfo_41_choice[] = {
  {   0, &hf_rrc_tdd_39          , ASN1_NO_EXTENSIONS     , dissect_rrc_T_tdd_38 },
  {   1, &hf_rrc_fdd_45          , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_modeSpecificInfo_41(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_modeSpecificInfo_41, T_modeSpecificInfo_41_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t DL_HSPDSCH_Information_sequence[] = {
  { &hf_rrc_hs_scch_Info    , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_HS_SCCH_Info },
  { &hf_rrc_measurement_feedback_Info, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_Measurement_Feedback_Info },
  { &hf_rrc_modeSpecificInfo_41, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_modeSpecificInfo_41 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_DL_HSPDSCH_Information(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_DL_HSPDSCH_Information, DL_HSPDSCH_Information_sequence);

  return offset;
}


static const value_string rrc_HARQMemorySize_vals[] = {
  {   0, "hms800" },
  {   1, "hms1600" },
  {   2, "hms2400" },
  {   3, "hms3200" },
  {   4, "hms4000" },
  {   5, "hms4800" },
  {   6, "hms5600" },
  {   7, "hms6400" },
  {   8, "hms7200" },
  {   9, "hms8000" },
  {  10, "hms8800" },
  {  11, "hms9600" },
  {  12, "hms10400" },
  {  13, "hms11200" },
  {  14, "hms12000" },
  {  15, "hms12800" },
  {  16, "hms13600" },
  {  17, "hms14400" },
  {  18, "hms15200" },
  {  19, "hms16000" },
  {  20, "hms17600" },
  {  21, "hms19200" },
  {  22, "hms20800" },
  {  23, "hms22400" },
  {  24, "hms24000" },
  {  25, "hms25600" },
  {  26, "hms27200" },
  {  27, "hms28800" },
  {  28, "hms30400" },
  {  29, "hms32000" },
  {  30, "hms36000" },
  {  31, "hms40000" },
  {  32, "hms44000" },
  {  33, "hms48000" },
  {  34, "hms52000" },
  {  35, "hms56000" },
  {  36, "hms60000" },
  {  37, "hms64000" },
  {  38, "hms68000" },
  {  39, "hms72000" },
  {  40, "hms76000" },
  {  41, "hms80000" },
  {  42, "hms88000" },
  {  43, "hms96000" },
  {  44, "hms104000" },
  {  45, "hms112000" },
  {  46, "hms120000" },
  {  47, "hms128000" },
  {  48, "hms136000" },
  {  49, "hms144000" },
  {  50, "hms152000" },
  {  51, "hms160000" },
  {  52, "hms176000" },
  {  53, "hms192000" },
  {  54, "hms208000" },
  {  55, "hms224000" },
  {  56, "hms240000" },
  {  57, "hms256000" },
  {  58, "hms272000" },
  {  59, "hms288000" },
  {  60, "hms304000" },
  { 0, NULL }
};

static value_string_ext rrc_HARQMemorySize_vals_ext = VALUE_STRING_EXT_INIT(rrc_HARQMemorySize_vals);


static int
dissect_rrc_HARQMemorySize(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     61, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t SEQUENCE_SIZE_1_maxHProcesses_OF_HARQMemorySize_sequence_of[1] = {
  { &hf_rrc_explicit_item   , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_HARQMemorySize },
};

static int
dissect_rrc_SEQUENCE_SIZE_1_maxHProcesses_OF_HARQMemorySize(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_SEQUENCE_SIZE_1_maxHProcesses_OF_HARQMemorySize, SEQUENCE_SIZE_1_maxHProcesses_OF_HARQMemorySize_sequence_of,
                                                  1, maxHProcesses, FALSE);

  return offset;
}


static const value_string rrc_T_memoryPartitioning_vals[] = {
  {   0, "implicit" },
  {   1, "explicit" },
  { 0, NULL }
};

static const per_choice_t T_memoryPartitioning_choice[] = {
  {   0, &hf_rrc_implicit        , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   1, &hf_rrc_explicit        , ASN1_NO_EXTENSIONS     , dissect_rrc_SEQUENCE_SIZE_1_maxHProcesses_OF_HARQMemorySize },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_memoryPartitioning(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_memoryPartitioning, T_memoryPartitioning_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t HARQ_Info_sequence[] = {
  { &hf_rrc_numberOfProcesses, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_1_8 },
  { &hf_rrc_memoryPartitioning, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_memoryPartitioning },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_HARQ_Info(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_HARQ_Info, HARQ_Info_sequence);

  return offset;
}


static const value_string rrc_T_mac_hsResetIndicator_vals[] = {
  {   0, "true" },
  { 0, NULL }
};


static int
dissect_rrc_T_mac_hsResetIndicator(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t Serving_HSDSCH_CellInformation_sequence[] = {
  { &hf_rrc_deltaACK        , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DeltaACK },
  { &hf_rrc_deltaNACK       , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DeltaNACK },
  { &hf_rrc_harq_Preamble_Mode, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_HARQ_Preamble_Mode },
  { &hf_rrc_primaryCPICH_Info, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_PrimaryCPICH_Info },
  { &hf_rrc_dl_hspdsch_Information, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_HSPDSCH_Information },
  { &hf_rrc_harqInfo_02     , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_HARQ_Info },
  { &hf_rrc_mac_hsResetIndicator, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_mac_hsResetIndicator },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_Serving_HSDSCH_CellInformation(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_Serving_HSDSCH_CellInformation, Serving_HSDSCH_CellInformation_sequence);

  return offset;
}



static int
dissect_rrc_E_AGCH_ChannelisationCode(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            0U, 255U, NULL, FALSE);

  return offset;
}


static const per_sequence_t E_AGCH_Information_sequence[] = {
  { &hf_rrc_e_AGCH_ChannelisationCode, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_E_AGCH_ChannelisationCode },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_E_AGCH_Information(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_E_AGCH_Information, E_AGCH_Information_sequence);

  return offset;
}



static int
dissect_rrc_INTEGER_0_38(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            0U, 38U, NULL, FALSE);

  return offset;
}


static const value_string rrc_T_primary_Secondary_GrantSelector_01_vals[] = {
  {   0, "primary" },
  {   1, "secondary" },
  { 0, NULL }
};


static int
dissect_rrc_T_primary_Secondary_GrantSelector_01(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     2, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t T_servingGrant_01_sequence[] = {
  { &hf_rrc_value           , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_INTEGER_0_38 },
  { &hf_rrc_primary_Secondary_GrantSelector_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_primary_Secondary_GrantSelector_01 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_servingGrant_01(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_servingGrant_01, T_servingGrant_01_sequence);

  return offset;
}



static int
dissect_rrc_E_DPCCH_DPCCH_PowerOffset(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            0U, 8U, NULL, FALSE);

  return offset;
}



static int
dissect_rrc_INTEGER_0_29(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            0U, 29U, NULL, FALSE);

  return offset;
}


static const per_sequence_t E_DPDCH_Reference_E_TFCI_sequence[] = {
  { &hf_rrc_reference_E_TFCI, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_0_127 },
  { &hf_rrc_reference_E_TFCI_PO, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_0_29 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_E_DPDCH_Reference_E_TFCI(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_E_DPDCH_Reference_E_TFCI, E_DPDCH_Reference_E_TFCI_sequence);

  return offset;
}


static const per_sequence_t E_DPDCH_Reference_E_TFCIList_sequence_of[1] = {
  { &hf_rrc_E_DPDCH_Reference_E_TFCIList_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_E_DPDCH_Reference_E_TFCI },
};

static int
dissect_rrc_E_DPDCH_Reference_E_TFCIList(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_E_DPDCH_Reference_E_TFCIList, E_DPDCH_Reference_E_TFCIList_sequence_of,
                                                  1, 8, FALSE);

  return offset;
}



static int
dissect_rrc_INTEGER_0_6(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            0U, 6U, NULL, FALSE);

  return offset;
}



static int
dissect_rrc_INTEGER_0_37(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            0U, 37U, NULL, FALSE);

  return offset;
}


static const value_string rrc_T_e_RGCH_Info_05_vals[] = {
  {   0, "e-RGCH-Information" },
  {   1, "releaseIndicator" },
  { 0, NULL }
};

static const per_choice_t T_e_RGCH_Info_05_choice[] = {
  {   0, &hf_rrc_e_RGCH_Information, ASN1_NO_EXTENSIONS     , dissect_rrc_E_RGCH_Information },
  {   1, &hf_rrc_releaseIndicator, ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_e_RGCH_Info_05(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_e_RGCH_Info_05, T_e_RGCH_Info_05_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t E_DCH_RL_InfoNewServingCell_sequence[] = {
  { &hf_rrc_primaryCPICH_Info, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PrimaryCPICH_Info },
  { &hf_rrc_e_AGCH_Information, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_E_AGCH_Information },
  { &hf_rrc_servingGrant_01 , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_servingGrant_01 },
  { &hf_rrc_e_DPCCH_DPCCH_PowerOffset, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_E_DPCCH_DPCCH_PowerOffset },
  { &hf_rrc_reference_E_TFCIs, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_E_DPDCH_Reference_E_TFCIList },
  { &hf_rrc_powerOffsetForSchedInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_INTEGER_0_6 },
  { &hf_rrc_threeIndexStepThreshold, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_INTEGER_0_37 },
  { &hf_rrc_twoIndexStepThreshold, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_INTEGER_0_37 },
  { &hf_rrc_e_HICH_Information, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_E_HICH_Information },
  { &hf_rrc_e_RGCH_Info_05  , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_e_RGCH_Info_05 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_E_DCH_RL_InfoNewServingCell(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_E_DCH_RL_InfoNewServingCell, E_DCH_RL_InfoNewServingCell_sequence);

  return offset;
}


static const value_string rrc_T_e_HICH_Info_04_vals[] = {
  {   0, "e-HICH-Information" },
  {   1, "releaseIndicator" },
  { 0, NULL }
};

static const per_choice_t T_e_HICH_Info_04_choice[] = {
  {   0, &hf_rrc_e_HICH_Information, ASN1_NO_EXTENSIONS     , dissect_rrc_E_HICH_Information },
  {   1, &hf_rrc_releaseIndicator, ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_e_HICH_Info_04(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_e_HICH_Info_04, T_e_HICH_Info_04_choice,
                                 NULL);

  return offset;
}


static const value_string rrc_T_e_RGCH_Info_09_vals[] = {
  {   0, "e-RGCH-Information" },
  {   1, "releaseIndicator" },
  { 0, NULL }
};

static const per_choice_t T_e_RGCH_Info_09_choice[] = {
  {   0, &hf_rrc_e_RGCH_Information, ASN1_NO_EXTENSIONS     , dissect_rrc_E_RGCH_Information },
  {   1, &hf_rrc_releaseIndicator, ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_e_RGCH_Info_09(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_e_RGCH_Info_09, T_e_RGCH_Info_09_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t E_DCH_RL_InfoOtherCell_sequence[] = {
  { &hf_rrc_primaryCPICH_Info, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PrimaryCPICH_Info },
  { &hf_rrc_e_HICH_Info_06  , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_e_HICH_Info_04 },
  { &hf_rrc_e_RGCH_Info_09  , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_e_RGCH_Info_09 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_E_DCH_RL_InfoOtherCell(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_E_DCH_RL_InfoOtherCell, E_DCH_RL_InfoOtherCell_sequence);

  return offset;
}


static const per_sequence_t SEQUENCE_SIZE_1_maxEDCHRL_OF_E_DCH_RL_InfoOtherCell_sequence_of[1] = {
  { &hf_rrc_e_DCH_RL_InfoOtherCellList_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_E_DCH_RL_InfoOtherCell },
};

static int
dissect_rrc_SEQUENCE_SIZE_1_maxEDCHRL_OF_E_DCH_RL_InfoOtherCell(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_SEQUENCE_SIZE_1_maxEDCHRL_OF_E_DCH_RL_InfoOtherCell, SEQUENCE_SIZE_1_maxEDCHRL_OF_E_DCH_RL_InfoOtherCell_sequence_of,
                                                  1, maxEDCHRL, FALSE);

  return offset;
}


static const per_sequence_t E_DCH_ReconfigurationInfo_sequence[] = {
  { &hf_rrc_e_DCH_RL_InfoNewServingCell, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_E_DCH_RL_InfoNewServingCell },
  { &hf_rrc_e_DCH_RL_InfoOtherCellList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_SEQUENCE_SIZE_1_maxEDCHRL_OF_E_DCH_RL_InfoOtherCell },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_E_DCH_ReconfigurationInfo(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_E_DCH_ReconfigurationInfo, E_DCH_ReconfigurationInfo_sequence);

  return offset;
}


static const per_sequence_t ActiveSetUpdate_r6_IEs_sequence[] = {
  { &hf_rrc_activationTime  , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_ActivationTime },
  { &hf_rrc_newU_RNTI       , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_U_RNTI },
  { &hf_rrc_newH_RNTI       , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_H_RNTI },
  { &hf_rrc_newPrimary_E_RNTI, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_E_RNTI },
  { &hf_rrc_newSecondary_E_RNTI, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_E_RNTI },
  { &hf_rrc_cn_InformationInfo_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CN_InformationInfo_r6 },
  { &hf_rrc_maxAllowedUL_TX_Power, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MaxAllowedUL_TX_Power },
  { &hf_rrc_rl_AdditionInformationList_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RL_AdditionInformationList_r6 },
  { &hf_rrc_rl_RemovalInformationList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RL_RemovalInformationList },
  { &hf_rrc_tx_DiversityMode, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_TX_DiversityMode },
  { &hf_rrc_dpc_Mode        , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DPC_Mode },
  { &hf_rrc_serving_HSDSCH_CellInformation, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_Serving_HSDSCH_CellInformation },
  { &hf_rrc_e_dch_ReconfigurationInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_E_DCH_ReconfigurationInfo },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_ActiveSetUpdate_r6_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_ActiveSetUpdate_r6_IEs, ActiveSetUpdate_r6_IEs_sequence);

  return offset;
}


static const value_string rrc_STTDIndication_vals[] = {
  {   0, "true" },
  { 0, NULL }
};


static int
dissect_rrc_STTDIndication(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t RL_AdditionInformation_v6b0ext_sequence[] = {
  { &hf_rrc_sttdIndication  , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_STTDIndication },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_RL_AdditionInformation_v6b0ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_RL_AdditionInformation_v6b0ext, RL_AdditionInformation_v6b0ext_sequence);

  return offset;
}


static const per_sequence_t RL_AdditionInformation_list_v6b0ext_sequence_of[1] = {
  { &hf_rrc_RL_AdditionInformation_list_v6b0ext_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RL_AdditionInformation_v6b0ext },
};

static int
dissect_rrc_RL_AdditionInformation_list_v6b0ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_RL_AdditionInformation_list_v6b0ext, RL_AdditionInformation_list_v6b0ext_sequence_of,
                                                  1, maxRL, FALSE);

  return offset;
}


static const per_sequence_t ActiveSetUpdate_v6b0ext_IEs_sequence[] = {
  { &hf_rrc_rl_AdditionInformation_list_v6b0ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RL_AdditionInformation_list_v6b0ext },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_ActiveSetUpdate_v6b0ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_ActiveSetUpdate_v6b0ext_IEs, ActiveSetUpdate_v6b0ext_IEs_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions_01_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_nonCriticalExtensions_01(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_nonCriticalExtensions_01, T_nonCriticalExtensions_01_sequence);

  return offset;
}


static const per_sequence_t T_v6b0NonCriticalExtensions_sequence[] = {
  { &hf_rrc_activeSetUpdate_v6b0ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_ActiveSetUpdate_v6b0ext_IEs },
  { &hf_rrc_nonCriticalExtensions_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_nonCriticalExtensions_01 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v6b0NonCriticalExtensions(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v6b0NonCriticalExtensions, T_v6b0NonCriticalExtensions_sequence);

  return offset;
}


static const per_sequence_t T_r6_sequence[] = {
  { &hf_rrc_activeSetUpdate_r6, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_ActiveSetUpdate_r6_IEs },
  { &hf_rrc_activeSetUpdate_r6_add_ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_BIT_STRING },
  { &hf_rrc_v6b0NonCriticalExtensions, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v6b0NonCriticalExtensions },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_r6(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_r6, T_r6_sequence);

  return offset;
}


static const value_string rrc_EnablingDelay_vals[] = {
  {   0, "radio-frames-0" },
  {   1, "radio-frames-1" },
  {   2, "radio-frames-2" },
  {   3, "radio-frames-4" },
  {   4, "radio-frames-8" },
  {   5, "radio-frames-16" },
  {   6, "radio-frames-32" },
  {   7, "radio-frames-64" },
  {   8, "radio-frames-128" },
  {   9, "spare7" },
  {  10, "spare6" },
  {  11, "spare5" },
  {  12, "spare4" },
  {  13, "spare3" },
  {  14, "spare2" },
  {  15, "spare1" },
  { 0, NULL }
};


static int
dissect_rrc_EnablingDelay(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     16, NULL, FALSE, 0, NULL);

  return offset;
}



static int
dissect_rrc_UE_DTX_DRX_Offset(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            0U, 159U, NULL, FALSE);

  return offset;
}


static const per_sequence_t NewTiming_sequence[] = {
  { &hf_rrc_enablingDelay_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_EnablingDelay },
  { &hf_rrc_ue_dtx_drx_Offset, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UE_DTX_DRX_Offset },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_NewTiming(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_NewTiming, NewTiming_sequence);

  return offset;
}


static const value_string rrc_T_timing_vals[] = {
  {   0, "continue" },
  {   1, "newTiming" },
  { 0, NULL }
};

static const per_choice_t T_timing_choice[] = {
  {   0, &hf_rrc_continue        , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   1, &hf_rrc_newTiming       , ASN1_NO_EXTENSIONS     , dissect_rrc_NewTiming },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_timing(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_timing, T_timing_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t DTX_DRX_TimingInfo_r7_sequence[] = {
  { &hf_rrc_timing          , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_timing },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_DTX_DRX_TimingInfo_r7(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_DTX_DRX_TimingInfo_r7, DTX_DRX_TimingInfo_r7_sequence);

  return offset;
}


static const value_string rrc_UE_DTX_Cycle1_10ms_vals[] = {
  {   0, "sub-frames-1" },
  {   1, "sub-frames-5" },
  {   2, "sub-frames-10" },
  {   3, "sub-frames-20" },
  { 0, NULL }
};


static int
dissect_rrc_UE_DTX_Cycle1_10ms(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     4, NULL, FALSE, 0, NULL);

  return offset;
}


static const value_string rrc_UE_DTX_Cycle2_10ms_vals[] = {
  {   0, "sub-frames-5" },
  {   1, "sub-frames-10" },
  {   2, "sub-frames-20" },
  {   3, "sub-frames-40" },
  {   4, "sub-frames-80" },
  {   5, "sub-frames-160" },
  {   6, "spare2" },
  {   7, "spare1" },
  { 0, NULL }
};


static int
dissect_rrc_UE_DTX_Cycle2_10ms(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     8, NULL, FALSE, 0, NULL);

  return offset;
}


static const value_string rrc_MAC_DTX_Cycle_10ms_vals[] = {
  {   0, "sub-frames-5" },
  {   1, "sub-frames-10" },
  {   2, "sub-frames-20" },
  {   3, "spare1" },
  { 0, NULL }
};


static int
dissect_rrc_MAC_DTX_Cycle_10ms(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     4, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t DTX_E_DCH_TTI_10ms_sequence[] = {
  { &hf_rrc_ue_dtx_Cycle1_10ms, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UE_DTX_Cycle1_10ms },
  { &hf_rrc_ue_dtx_Cycle2_10ms, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UE_DTX_Cycle2_10ms },
  { &hf_rrc_mac_dtx_Cycle_10ms, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MAC_DTX_Cycle_10ms },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_DTX_E_DCH_TTI_10ms(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_DTX_E_DCH_TTI_10ms, DTX_E_DCH_TTI_10ms_sequence);

  return offset;
}


static const value_string rrc_UE_DTX_Cycle1_2ms_vals[] = {
  {   0, "sub-frames-1" },
  {   1, "sub-frames-4" },
  {   2, "sub-frames-5" },
  {   3, "sub-frames-8" },
  {   4, "sub-frames-10" },
  {   5, "sub-frames-16" },
  {   6, "sub-frames-20" },
  {   7, "spare1" },
  { 0, NULL }
};


static int
dissect_rrc_UE_DTX_Cycle1_2ms(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     8, NULL, FALSE, 0, NULL);

  return offset;
}


static const value_string rrc_UE_DTX_Cycle2_2ms_vals[] = {
  {   0, "sub-frames-4" },
  {   1, "sub-frames-5" },
  {   2, "sub-frames-8" },
  {   3, "sub-frames-10" },
  {   4, "sub-frames-16" },
  {   5, "sub-frames-20" },
  {   6, "sub-frames-32" },
  {   7, "sub-frames-40" },
  {   8, "sub-frames-64" },
  {   9, "sub-frames-80" },
  {  10, "sub-frames-128" },
  {  11, "sub-frames-160" },
  {  12, "spare4" },
  {  13, "spare3" },
  {  14, "spare2" },
  {  15, "spare1" },
  { 0, NULL }
};


static int
dissect_rrc_UE_DTX_Cycle2_2ms(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     16, NULL, FALSE, 0, NULL);

  return offset;
}


static const value_string rrc_MAC_DTX_Cycle_2ms_vals[] = {
  {   0, "sub-frames-1" },
  {   1, "sub-frames-4" },
  {   2, "sub-frames-5" },
  {   3, "sub-frames-8" },
  {   4, "sub-frames-10" },
  {   5, "sub-frames-16" },
  {   6, "sub-frames-20" },
  {   7, "spare1" },
  { 0, NULL }
};


static int
dissect_rrc_MAC_DTX_Cycle_2ms(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     8, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t DTX_E_DCH_TTI_2ms_sequence[] = {
  { &hf_rrc_ue_dtx_Cycle1_2ms, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UE_DTX_Cycle1_2ms },
  { &hf_rrc_ue_dtx_Cycle2_2ms, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UE_DTX_Cycle2_2ms },
  { &hf_rrc_mac_dtx_Cycle_2ms, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MAC_DTX_Cycle_2ms },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_DTX_E_DCH_TTI_2ms(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_DTX_E_DCH_TTI_2ms, DTX_E_DCH_TTI_2ms_sequence);

  return offset;
}


static const value_string rrc_T_e_dch_TTI_Length_vals[] = {
  {   0, "dtx-e-dch-TTI-10ms" },
  {   1, "dtx-e-dch-TTI-2ms" },
  { 0, NULL }
};

static const per_choice_t T_e_dch_TTI_Length_choice[] = {
  {   0, &hf_rrc_dtx_e_dch_TTI_10ms, ASN1_NO_EXTENSIONS     , dissect_rrc_DTX_E_DCH_TTI_10ms },
  {   1, &hf_rrc_dtx_e_dch_TTI_2ms, ASN1_NO_EXTENSIONS     , dissect_rrc_DTX_E_DCH_TTI_2ms },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_e_dch_TTI_Length(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_e_dch_TTI_Length, T_e_dch_TTI_Length_choice,
                                 NULL);

  return offset;
}


static const value_string rrc_UE_DTX_Cycle2InactivityThreshold_vals[] = {
  {   0, "e-dch-tti-1" },
  {   1, "e-dch-tti-4" },
  {   2, "e-dch-tti-8" },
  {   3, "e-dch-tti-16" },
  {   4, "e-dch-tti-32" },
  {   5, "e-dch-tti-64" },
  {   6, "e-dch-tti-128" },
  {   7, "e-dch-tti-256" },
  {   8, "spare8" },
  {   9, "spare7" },
  {  10, "spare6" },
  {  11, "spare5" },
  {  12, "spare4" },
  {  13, "spare3" },
  {  14, "spare2" },
  {  15, "spare1" },
  { 0, NULL }
};


static int
dissect_rrc_UE_DTX_Cycle2InactivityThreshold(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     16, NULL, FALSE, 0, NULL);

  return offset;
}


static const value_string rrc_UE_DTX_long_preamble_length_vals[] = {
  {   0, "slots-4" },
  {   1, "slots-15" },
  { 0, NULL }
};


static int
dissect_rrc_UE_DTX_long_preamble_length(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     2, NULL, FALSE, 0, NULL);

  return offset;
}


static const value_string rrc_MAC_InactivityThreshold_vals[] = {
  {   0, "e-dch-tti-1" },
  {   1, "e-dch-tti-2" },
  {   2, "e-dch-tti-4" },
  {   3, "e-dch-tti-8" },
  {   4, "e-dch-tti-16" },
  {   5, "e-dch-tti-32" },
  {   6, "e-dch-tti-64" },
  {   7, "e-dch-tti-128" },
  {   8, "e-dch-tti-256" },
  {   9, "e-dch-tti-512" },
  {  10, "e-dch-tti-Infinity" },
  {  11, "spare5" },
  {  12, "spare4" },
  {  13, "spare3" },
  {  14, "spare2" },
  {  15, "spare1" },
  { 0, NULL }
};


static int
dissect_rrc_MAC_InactivityThreshold(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     16, NULL, FALSE, 0, NULL);

  return offset;
}


static const value_string rrc_CQI_DTX_Timer_vals[] = {
  {   0, "sub-frames-0" },
  {   1, "sub-frames-1" },
  {   2, "sub-frames-2" },
  {   3, "sub-frames-4" },
  {   4, "sub-frames-8" },
  {   5, "sub-frames-16" },
  {   6, "sub-frames-32" },
  {   7, "sub-frames-64" },
  {   8, "sub-frames-128" },
  {   9, "sub-frames-256" },
  {  10, "sub-frames-512" },
  {  11, "sub-frames-Infinity" },
  {  12, "spare4" },
  {  13, "spare3" },
  {  14, "spare2" },
  {  15, "spare1" },
  { 0, NULL }
};


static int
dissect_rrc_CQI_DTX_Timer(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     16, NULL, FALSE, 0, NULL);

  return offset;
}


static const value_string rrc_UE_DPCCH_Burst_vals[] = {
  {   0, "sub-frames-1" },
  {   1, "sub-frames-2" },
  {   2, "sub-frames-5" },
  {   3, "spare1" },
  { 0, NULL }
};


static int
dissect_rrc_UE_DPCCH_Burst(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     4, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t DTX_Info_sequence[] = {
  { &hf_rrc_e_dch_TTI_Length, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_e_dch_TTI_Length },
  { &hf_rrc_ue_dtx_cycle2InactivityThreshold, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UE_DTX_Cycle2InactivityThreshold },
  { &hf_rrc_ue_dtx_cycle2DefaultSG, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_INTEGER_0_38 },
  { &hf_rrc_ue_dtx_long_preamble_length, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_DTX_long_preamble_length },
  { &hf_rrc_mac_InactivityThreshold, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MAC_InactivityThreshold },
  { &hf_rrc_cqi_dtx_Timer   , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CQI_DTX_Timer },
  { &hf_rrc_ue_dpcch_Burst1 , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UE_DPCCH_Burst },
  { &hf_rrc_ue_dpcch_Burst2 , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UE_DPCCH_Burst },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_DTX_Info(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_DTX_Info, DTX_Info_sequence);

  return offset;
}


static const value_string rrc_UE_DRX_Cycle_vals[] = {
  {   0, "sub-frames-4" },
  {   1, "sub-frames-5" },
  {   2, "sub-frames-8" },
  {   3, "sub-frames-10" },
  {   4, "sub-frames-16" },
  {   5, "sub-frames-20" },
  {   6, "spare2" },
  {   7, "spare1" },
  { 0, NULL }
};


static int
dissect_rrc_UE_DRX_Cycle(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     8, NULL, FALSE, 0, NULL);

  return offset;
}


static const value_string rrc_UE_DRX_Cycle_InactivityThreshold_vals[] = {
  {   0, "sub-frames-0" },
  {   1, "sub-frames-1" },
  {   2, "sub-frames-2" },
  {   3, "sub-frames-4" },
  {   4, "sub-frames-8" },
  {   5, "sub-frames-16" },
  {   6, "sub-frames-32" },
  {   7, "sub-frames-64" },
  {   8, "sub-frames-128" },
  {   9, "sub-frames-256" },
  {  10, "sub-frames-512" },
  {  11, "spare5" },
  {  12, "spare4" },
  {  13, "spare3" },
  {  14, "spare2" },
  {  15, "spare1" },
  { 0, NULL }
};


static int
dissect_rrc_UE_DRX_Cycle_InactivityThreshold(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     16, NULL, FALSE, 0, NULL);

  return offset;
}


static const value_string rrc_UE_GrantMonitoring_InactivityThreshold_vals[] = {
  {   0, "e-dch-tti-0" },
  {   1, "e-dch-tti-1" },
  {   2, "e-dch-tti-2" },
  {   3, "e-dch-tti-4" },
  {   4, "e-dch-tti-8" },
  {   5, "e-dch-tti-16" },
  {   6, "e-dch-tti-32" },
  {   7, "e-dch-tti-64" },
  {   8, "e-dch-tti-128" },
  {   9, "e-dch-tti-256" },
  {  10, "spare6" },
  {  11, "spare5" },
  {  12, "spare4" },
  {  13, "spare3" },
  {  14, "spare2" },
  {  15, "spare1" },
  { 0, NULL }
};


static int
dissect_rrc_UE_GrantMonitoring_InactivityThreshold(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     16, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t DRX_Info_sequence[] = {
  { &hf_rrc_ue_drx_Cycle    , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UE_DRX_Cycle },
  { &hf_rrc_ue_drx_Cycle_InactivityThreshold, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UE_DRX_Cycle_InactivityThreshold },
  { &hf_rrc_ue_GrantMonitoring_InactivityThreshold, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UE_GrantMonitoring_InactivityThreshold },
  { &hf_rrc_ue_drx_GrantMonitoring, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_DRX_Info(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_DRX_Info, DRX_Info_sequence);

  return offset;
}


static const value_string rrc_Uplink_DPCCH_Slot_Format_Information_vals[] = {
  {   0, "slot-format-1" },
  {   1, "dummy" },
  {   2, "slot-format-4" },
  {   3, "spare1" },
  { 0, NULL }
};


static int
dissect_rrc_Uplink_DPCCH_Slot_Format_Information(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     4, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t DTX_DRX_Info_r7_sequence[] = {
  { &hf_rrc_dtx_Info        , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DTX_Info },
  { &hf_rrc_drx_Info        , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DRX_Info },
  { &hf_rrc_uplink_DPCCHSlotFormatInformation, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_Uplink_DPCCH_Slot_Format_Information },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_DTX_DRX_Info_r7(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_DTX_DRX_Info_r7, DTX_DRX_Info_r7_sequence);

  return offset;
}



static int
dissect_rrc_INTEGER_1_90(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            1U, 90U, NULL, FALSE);

  return offset;
}


static const per_sequence_t HS_SCCH_LessTFSList_item_sequence[] = {
  { &hf_rrc_hs_scch_LessTFSI, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_1_90 },
  { &hf_rrc_hs_scch_LessSecondCodeSupport, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_HS_SCCH_LessTFSList_item(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_HS_SCCH_LessTFSList_item, HS_SCCH_LessTFSList_item_sequence);

  return offset;
}


static const per_sequence_t HS_SCCH_LessTFSList_sequence_of[1] = {
  { &hf_rrc_HS_SCCH_LessTFSList_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_HS_SCCH_LessTFSList_item },
};

static int
dissect_rrc_HS_SCCH_LessTFSList(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_HS_SCCH_LessTFSList, HS_SCCH_LessTFSList_sequence_of,
                                                  1, maxHS_SCCHLessTrBlk, FALSE);

  return offset;
}


static const per_sequence_t HS_SCCH_Less_NewOperation_sequence[] = {
  { &hf_rrc_hs_pdsch_CodeIndex, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_1_15 },
  { &hf_rrc_hs_scch_LessTFS , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_HS_SCCH_LessTFSList },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_HS_SCCH_Less_NewOperation(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_HS_SCCH_Less_NewOperation, HS_SCCH_Less_NewOperation_sequence);

  return offset;
}


static const value_string rrc_T_hs_scchLessOperation_vals[] = {
  {   0, "continue" },
  {   1, "newOperation" },
  { 0, NULL }
};

static const per_choice_t T_hs_scchLessOperation_choice[] = {
  {   0, &hf_rrc_continue        , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   1, &hf_rrc_newOperation_03 , ASN1_NO_EXTENSIONS     , dissect_rrc_HS_SCCH_Less_NewOperation },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_hs_scchLessOperation(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_hs_scchLessOperation, T_hs_scchLessOperation_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t HS_SCCH_LessInfo_r7_sequence[] = {
  { &hf_rrc_hs_scchLessOperation, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_hs_scchLessOperation },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_HS_SCCH_LessInfo_r7(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_HS_SCCH_LessInfo_r7, HS_SCCH_LessInfo_r7_sequence);

  return offset;
}


static const value_string rrc_MIMO_Operation_vals[] = {
  {   0, "start" },
  {   1, "continue" },
  { 0, NULL }
};


static int
dissect_rrc_MIMO_Operation(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     2, NULL, FALSE, 0, NULL);

  return offset;
}


static const value_string rrc_MIMO_N_M_Ratio_vals[] = {
  {   0, "mnm1-2" },
  {   1, "mnm2-3" },
  {   2, "mnm3-4" },
  {   3, "mnm4-5" },
  {   4, "mnm5-6" },
  {   5, "mnm6-7" },
  {   6, "mnm7-8" },
  {   7, "mnm8-9" },
  {   8, "mnm9-10" },
  {   9, "mnm1-1" },
  { 0, NULL }
};


static int
dissect_rrc_MIMO_N_M_Ratio(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     10, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t T_diversityPattern_sequence[] = {
  { &hf_rrc_channelisationCode, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_ChannelisationCode256 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_diversityPattern(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_diversityPattern, T_diversityPattern_sequence);

  return offset;
}


static const value_string rrc_T_secondCPICH_Pattern_vals[] = {
  {   0, "normalPattern" },
  {   1, "diversityPattern" },
  { 0, NULL }
};

static const per_choice_t T_secondCPICH_Pattern_choice[] = {
  {   0, &hf_rrc_normalPattern   , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   1, &hf_rrc_diversityPattern, ASN1_NO_EXTENSIONS     , dissect_rrc_T_diversityPattern },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_secondCPICH_Pattern(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_secondCPICH_Pattern, T_secondCPICH_Pattern_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t MIMO_PilotConfiguration_sequence[] = {
  { &hf_rrc_secondCPICH_Pattern, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_secondCPICH_Pattern },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_MIMO_PilotConfiguration(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_MIMO_PilotConfiguration, MIMO_PilotConfiguration_sequence);

  return offset;
}


static const per_sequence_t MIMO_Parameters_r7_sequence[] = {
  { &hf_rrc_mimoOperation   , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MIMO_Operation },
  { &hf_rrc_mimoN_M_Ratio   , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MIMO_N_M_Ratio },
  { &hf_rrc_mimoPilotConfiguration, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MIMO_PilotConfiguration },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_MIMO_Parameters_r7(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_MIMO_Parameters_r7, MIMO_Parameters_r7_sequence);

  return offset;
}



static int
dissect_rrc_FDPCH_SlotFormat(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            0U, 9U, NULL, FALSE);

  return offset;
}


static const per_sequence_t DL_FDPCH_InfoPerRL_r7_sequence[] = {
  { &hf_rrc_pCPICH_UsageForChannelEst, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PCPICH_UsageForChannelEst },
  { &hf_rrc_fdpch_FrameOffset, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DPCH_FrameOffset },
  { &hf_rrc_fdpch_SlotFormat, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_FDPCH_SlotFormat },
  { &hf_rrc_secondaryCPICH_Info, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_SecondaryCPICH_Info },
  { &hf_rrc_secondaryScramblingCode, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_SecondaryScramblingCode },
  { &hf_rrc_dl_ChannelisationCode, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_0_255 },
  { &hf_rrc_tpc_CombinationIndex, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TPC_CombinationIndex },
  { &hf_rrc_sttdIndication  , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_STTDIndication },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_DL_FDPCH_InfoPerRL_r7(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_DL_FDPCH_InfoPerRL_r7, DL_FDPCH_InfoPerRL_r7_sequence);

  return offset;
}


static const value_string rrc_T_dl_dpchInfo_05_vals[] = {
  {   0, "dl-DPCH-InfoPerRL" },
  {   1, "dl-FDPCH-InfoPerRL" },
  { 0, NULL }
};

static const per_choice_t T_dl_dpchInfo_05_choice[] = {
  {   0, &hf_rrc_dl_DPCH_InfoPerRL_02, ASN1_NO_EXTENSIONS     , dissect_rrc_DL_DPCH_InfoPerRL_r5 },
  {   1, &hf_rrc_dl_FDPCH_InfoPerRL_01, ASN1_NO_EXTENSIONS     , dissect_rrc_DL_FDPCH_InfoPerRL_r7 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_dl_dpchInfo_05(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_dl_dpchInfo_05, T_dl_dpchInfo_05_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t RL_AdditionInformation_r7_sequence[] = {
  { &hf_rrc_primaryCPICH_Info, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PrimaryCPICH_Info },
  { &hf_rrc_cell_Id         , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CellIdentity },
  { &hf_rrc_dl_dpchInfo_05  , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_dl_dpchInfo_05 },
  { &hf_rrc_e_HICH_Information, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_E_HICH_Information },
  { &hf_rrc_e_RGCH_Information, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_E_RGCH_Information },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_RL_AdditionInformation_r7(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_RL_AdditionInformation_r7, RL_AdditionInformation_r7_sequence);

  return offset;
}


static const per_sequence_t RL_AdditionInformationList_r7_sequence_of[1] = {
  { &hf_rrc_RL_AdditionInformationList_r7_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RL_AdditionInformation_r7 },
};

static int
dissect_rrc_RL_AdditionInformationList_r7(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_RL_AdditionInformationList_r7, RL_AdditionInformationList_r7_sequence_of,
                                                  1, maxRL_1, FALSE);

  return offset;
}


static const per_sequence_t T_fdd_95_sequence[] = {
  { &hf_rrc_hS_SCCHChannelisationCodeInfo, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SEQUENCE_SIZE_1_maxHSSCCHs_OF_HS_SCCH_Codes },
  { &hf_rrc_dl_ScramblingCode, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_SecondaryScramblingCode },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_fdd_95(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_fdd_95, T_fdd_95_sequence);

  return offset;
}



static int
dissect_rrc_DHS_Sync(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            -20, 10U, NULL, FALSE);

  return offset;
}


static const per_sequence_t T_ueSpecificMidamble_03_sequence[] = {
  { &hf_rrc_midambleShift   , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MidambleShiftLong },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_ueSpecificMidamble_03(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_ueSpecificMidamble_03, T_ueSpecificMidamble_03_sequence);

  return offset;
}


static const value_string rrc_T_midambleAllocationMode_11_vals[] = {
  {   0, "defaultMidamble" },
  {   1, "commonMidamble" },
  {   2, "ueSpecificMidamble" },
  { 0, NULL }
};

static const per_choice_t T_midambleAllocationMode_11_choice[] = {
  {   0, &hf_rrc_defaultMidamble , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   1, &hf_rrc_commonMidamble  , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   2, &hf_rrc_ueSpecificMidamble_04, ASN1_NO_EXTENSIONS     , dissect_rrc_T_ueSpecificMidamble_03 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_midambleAllocationMode_11(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_midambleAllocationMode_11, T_midambleAllocationMode_11_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t HS_SCCH_TDD384_r6_sequence[] = {
  { &hf_rrc_timeslotNumber  , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TimeslotNumber },
  { &hf_rrc_channelisationCode_05, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DL_TS_ChannelisationCode },
  { &hf_rrc_midambleAllocationMode_11, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_midambleAllocationMode_11 },
  { &hf_rrc_midambleconfiguration, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MidambleConfigurationBurstType1and3 },
  { &hf_rrc_hs_sich_configuration_02, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_HS_SICH_Configuration_TDD384 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_HS_SCCH_TDD384_r6(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_HS_SCCH_TDD384_r6, HS_SCCH_TDD384_r6_sequence);

  return offset;
}


static const per_sequence_t SEQUENCE_SIZE_1_maxHSSCCHs_OF_HS_SCCH_TDD384_r6_sequence_of[1] = {
  { &hf_rrc_hS_SCCH_SetConfiguration_item_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_HS_SCCH_TDD384_r6 },
};

static int
dissect_rrc_SEQUENCE_SIZE_1_maxHSSCCHs_OF_HS_SCCH_TDD384_r6(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_SEQUENCE_SIZE_1_maxHSSCCHs_OF_HS_SCCH_TDD384_r6, SEQUENCE_SIZE_1_maxHSSCCHs_OF_HS_SCCH_TDD384_r6_sequence_of,
                                                  1, maxHSSCCHs, FALSE);

  return offset;
}


static const per_sequence_t T_tdd384_25_sequence[] = {
  { &hf_rrc_nack_ack_power_offset, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_M7_8 },
  { &hf_rrc_hs_SICH_PowerControl_Info, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_HS_SICH_Power_Control_Info_TDD384 },
  { &hf_rrc_dhs_sync        , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DHS_Sync },
  { &hf_rrc_bler_target     , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_Bler_Target },
  { &hf_rrc_hS_SCCH_SetConfiguration_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SEQUENCE_SIZE_1_maxHSSCCHs_OF_HS_SCCH_TDD384_r6 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_tdd384_25(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_tdd384_25, T_tdd384_25_sequence);

  return offset;
}


static const per_sequence_t HS_SICH_Power_Control_Info_TDD768_sequence[] = {
  { &hf_rrc_ul_target_SIR   , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_M22_40 },
  { &hf_rrc_hs_sich_ConstantValue, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_ConstantValue },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_HS_SICH_Power_Control_Info_TDD768(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_HS_SICH_Power_Control_Info_TDD768, HS_SICH_Power_Control_Info_TDD768_sequence);

  return offset;
}


static const value_string rrc_DL_TS_ChannelisationCode_VHCR_vals[] = {
  {   0, "cc32-1" },
  {   1, "cc32-2" },
  {   2, "cc32-3" },
  {   3, "cc32-4" },
  {   4, "cc132-5" },
  {   5, "cc32-6" },
  {   6, "cc32-7" },
  {   7, "cc32-8" },
  {   8, "cc32-9" },
  {   9, "cc32-10" },
  {  10, "cc32-11" },
  {  11, "cc32-12" },
  {  12, "cc32-13" },
  {  13, "cc32-14" },
  {  14, "cc32-15" },
  {  15, "cc32-16" },
  {  16, "cc32-17" },
  {  17, "cc32-18" },
  {  18, "cc32-19" },
  {  19, "cc32-20" },
  {  20, "cc32-21" },
  {  21, "cc32-22" },
  {  22, "cc32-23" },
  {  23, "cc32-24" },
  {  24, "cc32-25" },
  {  25, "cc32-26" },
  {  26, "cc32-27" },
  {  27, "cc32-28" },
  {  28, "cc32-29" },
  {  29, "cc32-30" },
  {  30, "cc32-31" },
  {  31, "cc32-32" },
  { 0, NULL }
};

static value_string_ext rrc_DL_TS_ChannelisationCode_VHCR_vals_ext = VALUE_STRING_EXT_INIT(rrc_DL_TS_ChannelisationCode_VHCR_vals);


static int
dissect_rrc_DL_TS_ChannelisationCode_VHCR(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     32, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t T_ueSpecificMidamble_04_sequence[] = {
  { &hf_rrc_midambleShift   , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MidambleShiftLong },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_ueSpecificMidamble_04(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_ueSpecificMidamble_04, T_ueSpecificMidamble_04_sequence);

  return offset;
}


static const value_string rrc_T_midambleAllocationMode_12_vals[] = {
  {   0, "defaultMidamble" },
  {   1, "commonMidamble" },
  {   2, "ueSpecificMidamble" },
  { 0, NULL }
};

static const per_choice_t T_midambleAllocationMode_12_choice[] = {
  {   0, &hf_rrc_defaultMidamble , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   1, &hf_rrc_commonMidamble  , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   2, &hf_rrc_ueSpecificMidamble_05, ASN1_NO_EXTENSIONS     , dissect_rrc_T_ueSpecificMidamble_04 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_midambleAllocationMode_12(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_midambleAllocationMode_12, T_midambleAllocationMode_12_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t T_ueSpecificMidamble_06_sequence[] = {
  { &hf_rrc_midambleShift   , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MidambleShiftLong },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_ueSpecificMidamble_06(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_ueSpecificMidamble_06, T_ueSpecificMidamble_06_sequence);

  return offset;
}


static const value_string rrc_T_midambleAllocationMode_14_vals[] = {
  {   0, "defaultMidamble" },
  {   1, "ueSpecificMidamble" },
  { 0, NULL }
};

static const per_choice_t T_midambleAllocationMode_14_choice[] = {
  {   0, &hf_rrc_defaultMidamble , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   1, &hf_rrc_ueSpecificMidamble_07, ASN1_NO_EXTENSIONS     , dissect_rrc_T_ueSpecificMidamble_06 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_midambleAllocationMode_14(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_midambleAllocationMode_14, T_midambleAllocationMode_14_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t HS_SICH_Configuration_TDD768_sequence[] = {
  { &hf_rrc_timeslotNumber  , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TimeslotNumber },
  { &hf_rrc_channelisationCode_06, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DL_TS_ChannelisationCode_VHCR },
  { &hf_rrc_midambleAllocationMode_14, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_midambleAllocationMode_14 },
  { &hf_rrc_midambleconfiguration, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MidambleConfigurationBurstType1and3 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_HS_SICH_Configuration_TDD768(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_HS_SICH_Configuration_TDD768, HS_SICH_Configuration_TDD768_sequence);

  return offset;
}


static const per_sequence_t HS_SCCH_TDD768_sequence[] = {
  { &hf_rrc_timeslotNumber  , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TimeslotNumber },
  { &hf_rrc_channelisationCode_06, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DL_TS_ChannelisationCode_VHCR },
  { &hf_rrc_midambleAllocationMode_12, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_midambleAllocationMode_12 },
  { &hf_rrc_midambleconfiguration, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MidambleConfigurationBurstType1and3 },
  { &hf_rrc_hs_sich_configuration_03, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_HS_SICH_Configuration_TDD768 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_HS_SCCH_TDD768(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_HS_SCCH_TDD768, HS_SCCH_TDD768_sequence);

  return offset;
}


static const per_sequence_t SEQUENCE_SIZE_1_maxHSSCCHs_OF_HS_SCCH_TDD768_sequence_of[1] = {
  { &hf_rrc_hS_SCCH_SetConfiguration_item_03, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_HS_SCCH_TDD768 },
};

static int
dissect_rrc_SEQUENCE_SIZE_1_maxHSSCCHs_OF_HS_SCCH_TDD768(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_SEQUENCE_SIZE_1_maxHSSCCHs_OF_HS_SCCH_TDD768, SEQUENCE_SIZE_1_maxHSSCCHs_OF_HS_SCCH_TDD768_sequence_of,
                                                  1, maxHSSCCHs, FALSE);

  return offset;
}


static const per_sequence_t T_tdd768_15_sequence[] = {
  { &hf_rrc_nack_ack_power_offset, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_M7_8 },
  { &hf_rrc_hs_SICH_PowerControl_Info_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_HS_SICH_Power_Control_Info_TDD768 },
  { &hf_rrc_dhs_sync        , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DHS_Sync },
  { &hf_rrc_bler_target     , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_Bler_Target },
  { &hf_rrc_hS_SCCH_SetConfiguration_03, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SEQUENCE_SIZE_1_maxHSSCCHs_OF_HS_SCCH_TDD768 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_tdd768_15(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_tdd768_15, T_tdd768_15_sequence);

  return offset;
}


static const value_string rrc_T_tpc_step_size_01_vals[] = {
  {   0, "s1" },
  {   1, "s2" },
  {   2, "s3" },
  {   3, "spare1" },
  { 0, NULL }
};


static int
dissect_rrc_T_tpc_step_size_01(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     4, NULL, FALSE, 0, NULL);

  return offset;
}



static int
dissect_rrc_PowerControlGAP(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            0U, 255U, NULL, FALSE);

  return offset;
}


static const value_string rrc_T_midambleAllocationMode_06_vals[] = {
  {   0, "defaultMidamble" },
  {   1, "commonMidamble" },
  {   2, "ueSpecificMidamble" },
  { 0, NULL }
};

static const per_choice_t T_midambleAllocationMode_06_choice[] = {
  {   0, &hf_rrc_defaultMidamble , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   1, &hf_rrc_commonMidamble  , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   2, &hf_rrc_ueSpecificMidamble, ASN1_NO_EXTENSIONS     , dissect_rrc_INTEGER_0_15 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_midambleAllocationMode_06(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_midambleAllocationMode_06, T_midambleAllocationMode_06_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t T_ueSpecificMidamble_01_sequence[] = {
  { &hf_rrc_midambleShift   , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MidambleShiftLong },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_ueSpecificMidamble_01(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_ueSpecificMidamble_01, T_ueSpecificMidamble_01_sequence);

  return offset;
}


static const value_string rrc_T_midambleAllocationMode_09_vals[] = {
  {   0, "defaultMidamble" },
  {   1, "ueSpecificMidamble" },
  { 0, NULL }
};

static const per_choice_t T_midambleAllocationMode_09_choice[] = {
  {   0, &hf_rrc_defaultMidamble , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   1, &hf_rrc_ueSpecificMidamble_02, ASN1_NO_EXTENSIONS     , dissect_rrc_T_ueSpecificMidamble_01 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_midambleAllocationMode_09(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_midambleAllocationMode_09, T_midambleAllocationMode_09_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t HS_SICH_Configuration_TDD128_r6_sequence[] = {
  { &hf_rrc_timeslotNumber_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TimeslotNumber_LCR_r4 },
  { &hf_rrc_channelisationCode_04, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_HS_ChannelisationCode_LCR },
  { &hf_rrc_midambleAllocationMode_09, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_midambleAllocationMode_09 },
  { &hf_rrc_midambleConfiguration, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_1_8 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_HS_SICH_Configuration_TDD128_r6(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_HS_SICH_Configuration_TDD128_r6, HS_SICH_Configuration_TDD128_r6_sequence);

  return offset;
}


static const per_sequence_t HS_SCCH_TDD128_r6_sequence[] = {
  { &hf_rrc_timeslotNumber_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TimeslotNumber_LCR_r4 },
  { &hf_rrc_firstChannelisationCode_03, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_HS_ChannelisationCode_LCR },
  { &hf_rrc_secondChannelisationCode, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_HS_ChannelisationCode_LCR },
  { &hf_rrc_midambleAllocationMode_06, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_midambleAllocationMode_06 },
  { &hf_rrc_midambleConfiguration, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_1_8 },
  { &hf_rrc_hs_sich_configuration_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_HS_SICH_Configuration_TDD128_r6 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_HS_SCCH_TDD128_r6(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_HS_SCCH_TDD128_r6, HS_SCCH_TDD128_r6_sequence);

  return offset;
}


static const per_sequence_t SEQUENCE_SIZE_1_maxHSSCCHs_OF_HS_SCCH_TDD128_r6_sequence_of[1] = {
  { &hf_rrc_hS_SCCH_SetConfiguration_item_02, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_HS_SCCH_TDD128_r6 },
};

static int
dissect_rrc_SEQUENCE_SIZE_1_maxHSSCCHs_OF_HS_SCCH_TDD128_r6(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_SEQUENCE_SIZE_1_maxHSSCCHs_OF_HS_SCCH_TDD128_r6, SEQUENCE_SIZE_1_maxHSSCCHs_OF_HS_SCCH_TDD128_r6_sequence_of,
                                                  1, maxHSSCCHs, FALSE);

  return offset;
}


static const per_sequence_t T_tdd128_36_sequence[] = {
  { &hf_rrc_nack_ack_power_offset, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_M7_8 },
  { &hf_rrc_power_level_HSSICH, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_M120_M58 },
  { &hf_rrc_tpc_step_size_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_tpc_step_size_01 },
  { &hf_rrc_bler_target     , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_Bler_Target },
  { &hf_rrc_powerControlGAP , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_PowerControlGAP },
  { &hf_rrc_pathlossCompensationSwitch, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_BOOLEAN },
  { &hf_rrc_hS_SCCH_SetConfiguration_02, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SEQUENCE_SIZE_1_maxHSSCCHs_OF_HS_SCCH_TDD128_r6 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_tdd128_36(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_tdd128_36, T_tdd128_36_sequence);

  return offset;
}


static const value_string rrc_T_tdd_52_vals[] = {
  {   0, "tdd384" },
  {   1, "tdd768" },
  {   2, "tdd128" },
  { 0, NULL }
};

static const per_choice_t T_tdd_52_choice[] = {
  {   0, &hf_rrc_tdd384_27       , ASN1_NO_EXTENSIONS     , dissect_rrc_T_tdd384_25 },
  {   1, &hf_rrc_tdd768_17       , ASN1_NO_EXTENSIONS     , dissect_rrc_T_tdd768_15 },
  {   2, &hf_rrc_tdd128_37       , ASN1_NO_EXTENSIONS     , dissect_rrc_T_tdd128_36 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_tdd_52(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_tdd_52, T_tdd_52_choice,
                                 NULL);

  return offset;
}


static const value_string rrc_T_modeSpecificInfo_64_vals[] = {
  {   0, "fdd" },
  {   1, "tdd" },
  { 0, NULL }
};

static const per_choice_t T_modeSpecificInfo_64_choice[] = {
  {   0, &hf_rrc_fdd_97          , ASN1_NO_EXTENSIONS     , dissect_rrc_T_fdd_95 },
  {   1, &hf_rrc_tdd_56          , ASN1_NO_EXTENSIONS     , dissect_rrc_T_tdd_52 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_modeSpecificInfo_64(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_modeSpecificInfo_64, T_modeSpecificInfo_64_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t HS_SCCH_Info_r7_sequence[] = {
  { &hf_rrc_modeSpecificInfo_64, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_modeSpecificInfo_64 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_HS_SCCH_Info_r7(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_HS_SCCH_Info_r7, HS_SCCH_Info_r7_sequence);

  return offset;
}


static const value_string rrc_Feedback_cycle_r7_vals[] = {
  {   0, "fc0" },
  {   1, "fc2" },
  {   2, "fc4" },
  {   3, "fc8" },
  {   4, "fc10" },
  {   5, "fc20" },
  {   6, "fc40" },
  {   7, "fc80" },
  {   8, "fc160" },
  {   9, "fc16" },
  {  10, "fc32" },
  {  11, "fc64" },
  {  12, "spare4" },
  {  13, "spare3" },
  {  14, "spare2" },
  {  15, "spare1" },
  { 0, NULL }
};


static int
dissect_rrc_Feedback_cycle_r7(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     16, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t T_fdd_98_sequence[] = {
  { &hf_rrc_measurementPowerOffset, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MeasurementPowerOffset },
  { &hf_rrc_feedback_cycle_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_Feedback_cycle_r7 },
  { &hf_rrc_cqi_RepetitionFactor, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CQI_RepetitionFactor },
  { &hf_rrc_deltaCQI        , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DeltaCQI },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_fdd_98(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_fdd_98, T_fdd_98_sequence);

  return offset;
}


static const value_string rrc_T_modeSpecificInfo_68_vals[] = {
  {   0, "fdd" },
  {   1, "tdd" },
  { 0, NULL }
};

static const per_choice_t T_modeSpecificInfo_68_choice[] = {
  {   0, &hf_rrc_fdd_100         , ASN1_NO_EXTENSIONS     , dissect_rrc_T_fdd_98 },
  {   1, &hf_rrc_tdd             , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_modeSpecificInfo_68(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_modeSpecificInfo_68, T_modeSpecificInfo_68_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t Measurement_Feedback_Info_r7_sequence[] = {
  { &hf_rrc_modeSpecificInfo_68, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_modeSpecificInfo_68 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_Measurement_Feedback_Info_r7(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_Measurement_Feedback_Info_r7, Measurement_Feedback_Info_r7_sequence);

  return offset;
}


static const per_sequence_t T_tdd384_15_sequence[] = {
  { &hf_rrc_dl_HSPDSCH_TS_Configuration, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_HSPDSCH_TS_Configuration },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_tdd384_15(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_tdd384_15, T_tdd384_15_sequence);

  return offset;
}


static const per_sequence_t T_ueSpecificMidamble_18_sequence[] = {
  { &hf_rrc_midambleShift   , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MidambleShiftLong },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_ueSpecificMidamble_18(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_ueSpecificMidamble_18, T_ueSpecificMidamble_18_sequence);

  return offset;
}


static const value_string rrc_T_midambleAllocationMode_26_vals[] = {
  {   0, "defaultMidamble" },
  {   1, "commonMidamble" },
  {   2, "ueSpecificMidamble" },
  { 0, NULL }
};

static const per_choice_t T_midambleAllocationMode_26_choice[] = {
  {   0, &hf_rrc_defaultMidamble , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   1, &hf_rrc_commonMidamble  , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   2, &hf_rrc_ueSpecificMidamble_19, ASN1_NO_EXTENSIONS     , dissect_rrc_T_ueSpecificMidamble_18 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_midambleAllocationMode_26(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_midambleAllocationMode_26, T_midambleAllocationMode_26_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t T_type1_05_sequence[] = {
  { &hf_rrc_midambleConfigurationBurstType1and3, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MidambleConfigurationBurstType1and3 },
  { &hf_rrc_midambleAllocationMode_26, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_midambleAllocationMode_26 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_type1_05(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_type1_05, T_type1_05_sequence);

  return offset;
}


static const value_string rrc_MidambleConfigurationBurstType2_VHCR_vals[] = {
  {   0, "ms4" },
  {   1, "ms8" },
  { 0, NULL }
};


static int
dissect_rrc_MidambleConfigurationBurstType2_VHCR(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     2, NULL, FALSE, 0, NULL);

  return offset;
}



static int
dissect_rrc_MidambleShiftShort_VHCR(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            0U, 7U, NULL, FALSE);

  return offset;
}


static const per_sequence_t T_ueSpecificMidamble_19_sequence[] = {
  { &hf_rrc_midambleShift_03, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MidambleShiftShort_VHCR },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_ueSpecificMidamble_19(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_ueSpecificMidamble_19, T_ueSpecificMidamble_19_sequence);

  return offset;
}


static const value_string rrc_T_midambleAllocationMode_27_vals[] = {
  {   0, "defaultMidamble" },
  {   1, "commonMidamble" },
  {   2, "ueSpecificMidamble" },
  { 0, NULL }
};

static const per_choice_t T_midambleAllocationMode_27_choice[] = {
  {   0, &hf_rrc_defaultMidamble , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   1, &hf_rrc_commonMidamble  , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   2, &hf_rrc_ueSpecificMidamble_20, ASN1_NO_EXTENSIONS     , dissect_rrc_T_ueSpecificMidamble_19 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_midambleAllocationMode_27(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_midambleAllocationMode_27, T_midambleAllocationMode_27_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t T_type2_04_sequence[] = {
  { &hf_rrc_midambleConfigurationBurstType2_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MidambleConfigurationBurstType2_VHCR },
  { &hf_rrc_midambleAllocationMode_27, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_midambleAllocationMode_27 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_type2_04(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_type2_04, T_type2_04_sequence);

  return offset;
}


static const value_string rrc_T_burstType_04_vals[] = {
  {   0, "type1" },
  {   1, "type2" },
  { 0, NULL }
};

static const per_choice_t T_burstType_04_choice[] = {
  {   0, &hf_rrc_type1_05        , ASN1_NO_EXTENSIONS     , dissect_rrc_T_type1_05 },
  {   1, &hf_rrc_type2_04        , ASN1_NO_EXTENSIONS     , dissect_rrc_T_type2_04 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_burstType_04(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_burstType_04, T_burstType_04_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t MidambleShiftAndBurstType_DL_VHCR_sequence[] = {
  { &hf_rrc_burstType_05    , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_burstType_04 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_MidambleShiftAndBurstType_DL_VHCR(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_MidambleShiftAndBurstType_DL_VHCR, MidambleShiftAndBurstType_DL_VHCR_sequence);

  return offset;
}


static const per_sequence_t DL_HSPDSCH_TS_Configuration_VHCR_item_sequence[] = {
  { &hf_rrc_timeslot        , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TimeslotNumber },
  { &hf_rrc_midambleShiftAndBurstType_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MidambleShiftAndBurstType_DL_VHCR },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_DL_HSPDSCH_TS_Configuration_VHCR_item(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_DL_HSPDSCH_TS_Configuration_VHCR_item, DL_HSPDSCH_TS_Configuration_VHCR_item_sequence);

  return offset;
}


static const per_sequence_t DL_HSPDSCH_TS_Configuration_VHCR_sequence_of[1] = {
  { &hf_rrc_DL_HSPDSCH_TS_Configuration_VHCR_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DL_HSPDSCH_TS_Configuration_VHCR_item },
};

static int
dissect_rrc_DL_HSPDSCH_TS_Configuration_VHCR(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_DL_HSPDSCH_TS_Configuration_VHCR, DL_HSPDSCH_TS_Configuration_VHCR_sequence_of,
                                                  1, maxTS_1, FALSE);

  return offset;
}


static const per_sequence_t T_tdd768_07_sequence[] = {
  { &hf_rrc_dl_HSPDSCH_TS_Configuration_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_HSPDSCH_TS_Configuration_VHCR },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_tdd768_07(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_tdd768_07, T_tdd768_07_sequence);

  return offset;
}


static const value_string rrc_T_tsn_Length_vals[] = {
  {   0, "tsn-6bits" },
  {   1, "tsn-9bits" },
  { 0, NULL }
};


static int
dissect_rrc_T_tsn_Length(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     2, NULL, FALSE, 0, NULL);

  return offset;
}



static int
dissect_rrc_INTEGER_1_maxTDD128Carrier(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            1U, maxTDD128Carrier, NULL, FALSE);

  return offset;
}



static int
dissect_rrc_UARFCN(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            0U, 16383U, NULL, FALSE);

  return offset;
}


static const value_string rrc_T_midambleAllocationMode_07_vals[] = {
  {   0, "defaultMidamble" },
  {   1, "commonMidamble" },
  {   2, "ueSpecificMidamble" },
  { 0, NULL }
};

static const per_choice_t T_midambleAllocationMode_07_choice[] = {
  {   0, &hf_rrc_defaultMidamble , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   1, &hf_rrc_commonMidamble  , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   2, &hf_rrc_ueSpecificMidamble, ASN1_NO_EXTENSIONS     , dissect_rrc_INTEGER_0_15 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_midambleAllocationMode_07(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_midambleAllocationMode_07, T_midambleAllocationMode_07_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t HS_SCCH_TDD128_MultiCarrier_sequence[] = {
  { &hf_rrc_uarfcn_HS_SCCH_Rx, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UARFCN },
  { &hf_rrc_timeslotNumber_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TimeslotNumber_LCR_r4 },
  { &hf_rrc_firstChannelisationCode_03, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_HS_ChannelisationCode_LCR },
  { &hf_rrc_secondChannelisationCode, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_HS_ChannelisationCode_LCR },
  { &hf_rrc_midambleAllocationMode_07, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_midambleAllocationMode_07 },
  { &hf_rrc_midambleConfiguration, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_1_8 },
  { &hf_rrc_bler_target     , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_Bler_Target },
  { &hf_rrc_hs_sich_configuration, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_HS_SICH_Configuration_TDD128 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_HS_SCCH_TDD128_MultiCarrier(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_HS_SCCH_TDD128_MultiCarrier, HS_SCCH_TDD128_MultiCarrier_sequence);

  return offset;
}


static const per_sequence_t SEQUENCE_SIZE_1_maxHSSCCHs_OF_HS_SCCH_TDD128_MultiCarrier_sequence_of[1] = {
  { &hf_rrc_hs_SCCH_TDD128_MultiCarrier_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_HS_SCCH_TDD128_MultiCarrier },
};

static int
dissect_rrc_SEQUENCE_SIZE_1_maxHSSCCHs_OF_HS_SCCH_TDD128_MultiCarrier(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_SEQUENCE_SIZE_1_maxHSSCCHs_OF_HS_SCCH_TDD128_MultiCarrier, SEQUENCE_SIZE_1_maxHSSCCHs_OF_HS_SCCH_TDD128_MultiCarrier_sequence_of,
                                                  1, maxHSSCCHs, FALSE);

  return offset;
}


static const per_sequence_t DL_HSPDSCH_MultiCarrier_Information_item_sequence[] = {
  { &hf_rrc_uarfcn_Carrier  , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UARFCN },
  { &hf_rrc_harqInfo_02     , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_HARQ_Info },
  { &hf_rrc_hs_PDSCH_Midamble_Configuration, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_HS_PDSCH_Midamble_Configuration_TDD128 },
  { &hf_rrc_hs_SCCH_TDD128_MultiCarrier, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_SEQUENCE_SIZE_1_maxHSSCCHs_OF_HS_SCCH_TDD128_MultiCarrier },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_DL_HSPDSCH_MultiCarrier_Information_item(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_DL_HSPDSCH_MultiCarrier_Information_item, DL_HSPDSCH_MultiCarrier_Information_item_sequence);

  return offset;
}


static const per_sequence_t DL_HSPDSCH_MultiCarrier_Information_sequence_of[1] = {
  { &hf_rrc_DL_HSPDSCH_MultiCarrier_Information_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DL_HSPDSCH_MultiCarrier_Information_item },
};

static int
dissect_rrc_DL_HSPDSCH_MultiCarrier_Information(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_DL_HSPDSCH_MultiCarrier_Information, DL_HSPDSCH_MultiCarrier_Information_sequence_of,
                                                  1, maxTDD128Carrier, FALSE);

  return offset;
}


static const per_sequence_t DL_MultiCarrier_Information_sequence[] = {
  { &hf_rrc_tsn_Length      , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_tsn_Length },
  { &hf_rrc_multiCarrierNumber, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_INTEGER_1_maxTDD128Carrier },
  { &hf_rrc_dl_HSPDSCH_MultiCarrier_Information, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_HSPDSCH_MultiCarrier_Information },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_DL_MultiCarrier_Information(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_DL_MultiCarrier_Information, DL_MultiCarrier_Information_sequence);

  return offset;
}


static const per_sequence_t T_tdd128_23_sequence[] = {
  { &hf_rrc_hs_PDSCH_Midamble_Configuration_tdd128, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_HS_PDSCH_Midamble_Configuration_TDD128 },
  { &hf_rrc_dl_MultiCarrier_Information, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_MultiCarrier_Information },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_tdd128_23(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_tdd128_23, T_tdd128_23_sequence);

  return offset;
}


static const value_string rrc_T_tdd_40_vals[] = {
  {   0, "tdd384" },
  {   1, "tdd768" },
  {   2, "tdd128" },
  { 0, NULL }
};

static const per_choice_t T_tdd_40_choice[] = {
  {   0, &hf_rrc_tdd384_16       , ASN1_NO_EXTENSIONS     , dissect_rrc_T_tdd384_15 },
  {   1, &hf_rrc_tdd768_08       , ASN1_NO_EXTENSIONS     , dissect_rrc_T_tdd768_07 },
  {   2, &hf_rrc_tdd128_23       , ASN1_NO_EXTENSIONS     , dissect_rrc_T_tdd128_23 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_tdd_40(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_tdd_40, T_tdd_40_choice,
                                 NULL);

  return offset;
}


static const value_string rrc_T_dl_64QAM_Configured_vals[] = {
  {   0, "true" },
  { 0, NULL }
};


static int
dissect_rrc_T_dl_64QAM_Configured(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t T_fdd_76_sequence[] = {
  { &hf_rrc_dl_64QAM_Configured, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_dl_64QAM_Configured },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_fdd_76(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_fdd_76, T_fdd_76_sequence);

  return offset;
}


static const value_string rrc_T_modeSpecificInfo_43_vals[] = {
  {   0, "tdd" },
  {   1, "fdd" },
  { 0, NULL }
};

static const per_choice_t T_modeSpecificInfo_43_choice[] = {
  {   0, &hf_rrc_tdd_41          , ASN1_NO_EXTENSIONS     , dissect_rrc_T_tdd_40 },
  {   1, &hf_rrc_fdd_77          , ASN1_NO_EXTENSIONS     , dissect_rrc_T_fdd_76 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_modeSpecificInfo_43(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_modeSpecificInfo_43, T_modeSpecificInfo_43_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t DL_HSPDSCH_Information_r7_sequence[] = {
  { &hf_rrc_hs_scch_Info_02 , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_HS_SCCH_Info_r7 },
  { &hf_rrc_measurement_feedback_Info_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_Measurement_Feedback_Info_r7 },
  { &hf_rrc_modeSpecificInfo_43, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_modeSpecificInfo_43 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_DL_HSPDSCH_Information_r7(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_DL_HSPDSCH_Information_r7, DL_HSPDSCH_Information_r7_sequence);

  return offset;
}


static const value_string rrc_T_numberOfProcesses_vals[] = {
  {   0, "n1" },
  {   1, "n2" },
  {   2, "n3" },
  {   3, "n4" },
  {   4, "n5" },
  {   5, "n6" },
  {   6, "n7" },
  {   7, "n8" },
  {   8, "n12" },
  {   9, "n14" },
  {  10, "n16" },
  { 0, NULL }
};


static int
dissect_rrc_T_numberOfProcesses(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     11, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t T_explicit_sequence[] = {
  { &hf_rrc_memorySize      , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SEQUENCE_SIZE_1_maxHProcesses_OF_HARQMemorySize },
  { &hf_rrc_additionalMemorySizesForMIMO, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_SEQUENCE_SIZE_1_maxHProcesses_OF_HARQMemorySize },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_explicit(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_explicit, T_explicit_sequence);

  return offset;
}


static const value_string rrc_T_memoryPartitioning_01_vals[] = {
  {   0, "implicit" },
  {   1, "explicit" },
  { 0, NULL }
};

static const per_choice_t T_memoryPartitioning_01_choice[] = {
  {   0, &hf_rrc_implicit        , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   1, &hf_rrc_explicit_01     , ASN1_NO_EXTENSIONS     , dissect_rrc_T_explicit },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_memoryPartitioning_01(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_memoryPartitioning_01, T_memoryPartitioning_01_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t HARQ_Info_r7_sequence[] = {
  { &hf_rrc_numberOfProcesses_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_numberOfProcesses },
  { &hf_rrc_memoryPartitioning_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_memoryPartitioning_01 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_HARQ_Info_r7(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_HARQ_Info_r7, HARQ_Info_r7_sequence);

  return offset;
}


static const value_string rrc_T_mac_hsResetIndicator_01_vals[] = {
  {   0, "true" },
  { 0, NULL }
};


static int
dissect_rrc_T_mac_hsResetIndicator_01(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t Serving_HSDSCH_CellInformation_r7_sequence[] = {
  { &hf_rrc_deltaACK        , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DeltaACK },
  { &hf_rrc_deltaNACK       , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DeltaNACK },
  { &hf_rrc_harq_Preamble_Mode, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_HARQ_Preamble_Mode },
  { &hf_rrc_primaryCPICH_Info, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_PrimaryCPICH_Info },
  { &hf_rrc_dl_hspdsch_Information_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_HSPDSCH_Information_r7 },
  { &hf_rrc_harqInfo        , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_HARQ_Info_r7 },
  { &hf_rrc_mac_hsResetIndicator_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_mac_hsResetIndicator_01 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_Serving_HSDSCH_CellInformation_r7(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_Serving_HSDSCH_CellInformation_r7, Serving_HSDSCH_CellInformation_r7_sequence);

  return offset;
}


static const value_string rrc_T_primary_Secondary_GrantSelector_02_vals[] = {
  {   0, "primary" },
  {   1, "secondary" },
  { 0, NULL }
};


static int
dissect_rrc_T_primary_Secondary_GrantSelector_02(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     2, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t T_servingGrant_02_sequence[] = {
  { &hf_rrc_value           , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_INTEGER_0_38 },
  { &hf_rrc_primary_Secondary_GrantSelector_02, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_primary_Secondary_GrantSelector_02 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_servingGrant_02(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_servingGrant_02, T_servingGrant_02_sequence);

  return offset;
}


static const per_sequence_t E_DPDCH_Reference_E_TFCI_r7_sequence[] = {
  { &hf_rrc_reference_E_TFCI, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_0_127 },
  { &hf_rrc_reference_E_TFCI_PO_r7, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_0_31 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_E_DPDCH_Reference_E_TFCI_r7(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_E_DPDCH_Reference_E_TFCI_r7, E_DPDCH_Reference_E_TFCI_r7_sequence);

  return offset;
}


static const per_sequence_t E_DPDCH_Reference_E_TFCIList_r7_sequence_of[1] = {
  { &hf_rrc_E_DPDCH_Reference_E_TFCIList_r7_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_E_DPDCH_Reference_E_TFCI_r7 },
};

static int
dissect_rrc_E_DPDCH_Reference_E_TFCIList_r7(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_E_DPDCH_Reference_E_TFCIList_r7, E_DPDCH_Reference_E_TFCIList_r7_sequence_of,
                                                  1, 8, FALSE);

  return offset;
}


static const value_string rrc_T_e_RGCH_Info_06_vals[] = {
  {   0, "e-RGCH-Information" },
  {   1, "releaseIndicator" },
  { 0, NULL }
};

static const per_choice_t T_e_RGCH_Info_06_choice[] = {
  {   0, &hf_rrc_e_RGCH_Information, ASN1_NO_EXTENSIONS     , dissect_rrc_E_RGCH_Information },
  {   1, &hf_rrc_releaseIndicator, ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_e_RGCH_Info_06(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_e_RGCH_Info_06, T_e_RGCH_Info_06_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t E_DCH_RL_InfoNewServingCell_r7_sequence[] = {
  { &hf_rrc_primaryCPICH_Info, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PrimaryCPICH_Info },
  { &hf_rrc_e_AGCH_Information, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_E_AGCH_Information },
  { &hf_rrc_servingGrant_02 , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_servingGrant_02 },
  { &hf_rrc_e_DPCCH_DPCCH_PowerOffset, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_E_DPCCH_DPCCH_PowerOffset },
  { &hf_rrc_reference_E_TFCIs_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_E_DPDCH_Reference_E_TFCIList_r7 },
  { &hf_rrc_powerOffsetForSchedInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_INTEGER_0_6 },
  { &hf_rrc_threeIndexStepThreshold, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_INTEGER_0_37 },
  { &hf_rrc_twoIndexStepThreshold, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_INTEGER_0_37 },
  { &hf_rrc_e_HICH_Information, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_E_HICH_Information },
  { &hf_rrc_e_RGCH_Info_06  , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_e_RGCH_Info_06 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_E_DCH_RL_InfoNewServingCell_r7(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_E_DCH_RL_InfoNewServingCell_r7, E_DCH_RL_InfoNewServingCell_r7_sequence);

  return offset;
}


static const per_sequence_t E_DCH_ReconfigurationInfo_r7_sequence[] = {
  { &hf_rrc_e_DCH_RL_InfoNewServingCell_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_E_DCH_RL_InfoNewServingCell_r7 },
  { &hf_rrc_e_DCH_RL_InfoOtherCellList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_SEQUENCE_SIZE_1_maxEDCHRL_OF_E_DCH_RL_InfoOtherCell },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_E_DCH_ReconfigurationInfo_r7(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_E_DCH_ReconfigurationInfo_r7, E_DCH_ReconfigurationInfo_r7_sequence);

  return offset;
}


static const per_sequence_t UL_16QAM_Settings_sequence[] = {
  { &hf_rrc_beta_Ed_Gain_E_AGCH_Table_Selection, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_0_1 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UL_16QAM_Settings(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UL_16QAM_Settings, UL_16QAM_Settings_sequence);

  return offset;
}



static int
dissect_rrc_E_TFCI_TableIndex(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            0U, 1U, NULL, FALSE);

  return offset;
}


static const value_string rrc_T_mac_es_e_resetIndicator_vals[] = {
  {   0, "true" },
  { 0, NULL }
};


static int
dissect_rrc_T_mac_es_e_resetIndicator(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t UL_16QAM_Config_sequence[] = {
  { &hf_rrc_ul_16QAM_Settings, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_16QAM_Settings },
  { &hf_rrc_e_TFCI_TableIndex, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_E_TFCI_TableIndex },
  { &hf_rrc_mac_es_e_resetIndicator, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_mac_es_e_resetIndicator },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UL_16QAM_Config(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UL_16QAM_Config, UL_16QAM_Config_sequence);

  return offset;
}


static const per_sequence_t E_DCH_RL_InfoSameServingCell_sequence[] = {
  { &hf_rrc_e_DPCCH_DPCCH_PowerOffset, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_E_DPCCH_DPCCH_PowerOffset },
  { &hf_rrc_reference_E_TFCIs_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_E_DPDCH_Reference_E_TFCIList_r7 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_E_DCH_RL_InfoSameServingCell(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_E_DCH_RL_InfoSameServingCell, E_DCH_RL_InfoSameServingCell_sequence);

  return offset;
}


static const per_sequence_t E_TFC_Boost_Info_r7_sequence[] = {
  { &hf_rrc_e_TFCI_Boost    , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_0_127 },
  { &hf_rrc_delta_T2TP      , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_INTEGER_0_6 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_E_TFC_Boost_Info_r7(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_E_TFC_Boost_Info_r7, E_TFC_Boost_Info_r7_sequence);

  return offset;
}



static int
dissect_rrc_E_DPDCH_PowerInterpolation(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_boolean(tvb, offset, actx, tree, hf_index, NULL);

  return offset;
}


static const per_sequence_t ActiveSetUpdate_r7_IEs_sequence[] = {
  { &hf_rrc_activationTime  , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_ActivationTime },
  { &hf_rrc_newU_RNTI       , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_U_RNTI },
  { &hf_rrc_newH_RNTI       , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_H_RNTI },
  { &hf_rrc_newPrimary_E_RNTI, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_E_RNTI },
  { &hf_rrc_newSecondary_E_RNTI, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_E_RNTI },
  { &hf_rrc_cn_InformationInfo_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CN_InformationInfo_r6 },
  { &hf_rrc_dtx_drx_TimingInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DTX_DRX_TimingInfo_r7 },
  { &hf_rrc_dtx_drx_Info    , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DTX_DRX_Info_r7 },
  { &hf_rrc_hs_scch_LessInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_HS_SCCH_LessInfo_r7 },
  { &hf_rrc_mimoParameters  , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MIMO_Parameters_r7 },
  { &hf_rrc_maxAllowedUL_TX_Power, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MaxAllowedUL_TX_Power },
  { &hf_rrc_rl_AdditionInformationList_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RL_AdditionInformationList_r7 },
  { &hf_rrc_rl_RemovalInformationList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RL_RemovalInformationList },
  { &hf_rrc_tx_DiversityMode, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_TX_DiversityMode },
  { &hf_rrc_dpc_Mode        , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DPC_Mode },
  { &hf_rrc_serving_HSDSCH_CellInformation_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_Serving_HSDSCH_CellInformation_r7 },
  { &hf_rrc_e_dch_ReconfigurationInfo_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_E_DCH_ReconfigurationInfo_r7 },
  { &hf_rrc_ul_16QAM_Config , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_16QAM_Config },
  { &hf_rrc_e_dch_ReconfInfoSameCell, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_E_DCH_RL_InfoSameServingCell },
  { &hf_rrc_e_TFC_Boost_Info, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_E_TFC_Boost_Info_r7 },
  { &hf_rrc_e_DPDCH_PowerInterpolation, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_E_DPDCH_PowerInterpolation },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_ActiveSetUpdate_r7_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_ActiveSetUpdate_r7_IEs, ActiveSetUpdate_r7_IEs_sequence);

  return offset;
}


static const value_string rrc_HS_DSCH_TBSizeTable_vals[] = {
  {   0, "octetAligned" },
  { 0, NULL }
};


static int
dissect_rrc_HS_DSCH_TBSizeTable(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t ActiveSetUpdate_v780ext_IEs_sequence[] = {
  { &hf_rrc_hs_DSCH_TBSizeTable, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_HS_DSCH_TBSizeTable },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_ActiveSetUpdate_v780ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_ActiveSetUpdate_v780ext_IEs, ActiveSetUpdate_v780ext_IEs_sequence);

  return offset;
}



static int
dissect_rrc_S_CPICH_PowerOffset_MIMO(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            -6, 0U, NULL, FALSE);

  return offset;
}


static const per_sequence_t MIMO_PilotConfiguration_v7f0ext_sequence[] = {
  { &hf_rrc_s_cpich_PowerOffset_Mimo_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_S_CPICH_PowerOffset_MIMO },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_MIMO_PilotConfiguration_v7f0ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_MIMO_PilotConfiguration_v7f0ext, MIMO_PilotConfiguration_v7f0ext_sequence);

  return offset;
}



static int
dissect_rrc_MIMO_Parameters_v7f0ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_rrc_MIMO_PilotConfiguration_v7f0ext(tvb, offset, actx, tree, hf_index);

  return offset;
}


static const per_sequence_t ActiveSetUpdate_v7f0ext_IEs_sequence[] = {
  { &hf_rrc_mimoParameters_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MIMO_Parameters_v7f0ext },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_ActiveSetUpdate_v7f0ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_ActiveSetUpdate_v7f0ext_IEs, ActiveSetUpdate_v7f0ext_IEs_sequence);

  return offset;
}


static const value_string rrc_T_precodingWeightSetRestriction_vals[] = {
  {   0, "true" },
  { 0, NULL }
};


static int
dissect_rrc_T_precodingWeightSetRestriction(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t MIMO_Parameters_v7g0ext_sequence[] = {
  { &hf_rrc_precodingWeightSetRestriction_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_precodingWeightSetRestriction },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_MIMO_Parameters_v7g0ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_MIMO_Parameters_v7g0ext, MIMO_Parameters_v7g0ext_sequence);

  return offset;
}


static const per_sequence_t ActiveSetUpdate_v7g0ext_IEs_sequence[] = {
  { &hf_rrc_mimoParameters_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MIMO_Parameters_v7g0ext },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_ActiveSetUpdate_v7g0ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_ActiveSetUpdate_v7g0ext_IEs, ActiveSetUpdate_v7g0ext_IEs_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions_02_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_nonCriticalExtensions_02(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_nonCriticalExtensions_02, T_nonCriticalExtensions_02_sequence);

  return offset;
}


static const per_sequence_t T_v7g0NonCriticalExtensions_sequence[] = {
  { &hf_rrc_activeSetUdpate_v7g0ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_ActiveSetUpdate_v7g0ext_IEs },
  { &hf_rrc_nonCriticalExtensions_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_nonCriticalExtensions_02 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v7g0NonCriticalExtensions(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v7g0NonCriticalExtensions, T_v7g0NonCriticalExtensions_sequence);

  return offset;
}


static const per_sequence_t T_v7f0NonCriticalExtensions_sequence[] = {
  { &hf_rrc_activeSetUdpate_v7f0ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_ActiveSetUpdate_v7f0ext_IEs },
  { &hf_rrc_v7g0NonCriticalExtensions, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v7g0NonCriticalExtensions },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v7f0NonCriticalExtensions(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v7f0NonCriticalExtensions, T_v7f0NonCriticalExtensions_sequence);

  return offset;
}


static const per_sequence_t T_v780NonCriticalExtensions_sequence[] = {
  { &hf_rrc_activeSetUpdate_v780ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_ActiveSetUpdate_v780ext_IEs },
  { &hf_rrc_v7f0NonCriticalExtensions, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v7f0NonCriticalExtensions },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v780NonCriticalExtensions(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v780NonCriticalExtensions, T_v780NonCriticalExtensions_sequence);

  return offset;
}


static const per_sequence_t T_r7_sequence[] = {
  { &hf_rrc_activeSetUpdate_r7, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_ActiveSetUpdate_r7_IEs },
  { &hf_rrc_activeSetUpdate_r7_add_ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_BIT_STRING },
  { &hf_rrc_v780NonCriticalExtensions, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v780NonCriticalExtensions },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_r7(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_r7, T_r7_sequence);

  return offset;
}


static const per_sequence_t T_fdd_100_sequence[] = {
  { &hf_rrc_mimoN_M_Ratio   , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MIMO_N_M_Ratio },
  { &hf_rrc_mimoPilotConfiguration, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MIMO_PilotConfiguration },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_fdd_100(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_fdd_100, T_fdd_100_sequence);

  return offset;
}


static const value_string rrc_T_mimoSFModeForHSPDSCHDualStream_vals[] = {
  {   0, "sf1" },
  {   1, "sf1sf16" },
  { 0, NULL }
};


static int
dissect_rrc_T_mimoSFModeForHSPDSCHDualStream(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     2, NULL, FALSE, 0, NULL);

  return offset;
}



static int
dissect_rrc_INTEGER_1_5(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            1U, 5U, NULL, FALSE);

  return offset;
}


static const per_sequence_t HS_SICH_ReferenceSignalInfoList_item_sequence[] = {
  { &hf_rrc_midambleConfiguration, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_1_8 },
  { &hf_rrc_midambleShift_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_0_15 },
  { &hf_rrc_timeSlot        , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_1_5 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_HS_SICH_ReferenceSignalInfoList_item(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_HS_SICH_ReferenceSignalInfoList_item, HS_SICH_ReferenceSignalInfoList_item_sequence);

  return offset;
}


static const per_sequence_t HS_SICH_ReferenceSignalInfoList_sequence_of[1] = {
  { &hf_rrc_HS_SICH_ReferenceSignalInfoList_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_HS_SICH_ReferenceSignalInfoList_item },
};

static int
dissect_rrc_HS_SICH_ReferenceSignalInfoList(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_HS_SICH_ReferenceSignalInfoList, HS_SICH_ReferenceSignalInfoList_sequence_of,
                                                  1, maxHSSCCHs, FALSE);

  return offset;
}


static const per_sequence_t T_tdd128_39_sequence[] = {
  { &hf_rrc_mimoSFModeForHSPDSCHDualStream, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_mimoSFModeForHSPDSCHDualStream },
  { &hf_rrc_hs_sich_ReferenceSignalInfoList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_HS_SICH_ReferenceSignalInfoList },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_tdd128_39(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_tdd128_39, T_tdd128_39_sequence);

  return offset;
}


static const value_string rrc_T_tdd_55_vals[] = {
  {   0, "tdd128" },
  {   1, "tdd384-tdd768" },
  { 0, NULL }
};

static const per_choice_t T_tdd_55_choice[] = {
  {   0, &hf_rrc_tdd128_40       , ASN1_NO_EXTENSIONS     , dissect_rrc_T_tdd128_39 },
  {   1, &hf_rrc_tdd384_tdd768_04, ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_tdd_55(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_tdd_55, T_tdd_55_choice,
                                 NULL);

  return offset;
}


static const value_string rrc_T_modeSpecificInfo_70_vals[] = {
  {   0, "fdd" },
  {   1, "tdd" },
  { 0, NULL }
};

static const per_choice_t T_modeSpecificInfo_70_choice[] = {
  {   0, &hf_rrc_fdd_102         , ASN1_NO_EXTENSIONS     , dissect_rrc_T_fdd_100 },
  {   1, &hf_rrc_tdd_59          , ASN1_NO_EXTENSIONS     , dissect_rrc_T_tdd_55 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_modeSpecificInfo_70(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_modeSpecificInfo_70, T_modeSpecificInfo_70_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t MIMO_Parameters_r8_sequence[] = {
  { &hf_rrc_mimoOperation   , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MIMO_Operation },
  { &hf_rrc_modeSpecificInfo_70, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_modeSpecificInfo_70 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_MIMO_Parameters_r8(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_MIMO_Parameters_r8, MIMO_Parameters_r8_sequence);

  return offset;
}


static const value_string rrc_T_dl_dpchInfo_06_vals[] = {
  {   0, "dl-DPCH-InfoPerRL" },
  {   1, "dl-FDPCH-InfoPerRL" },
  { 0, NULL }
};

static const per_choice_t T_dl_dpchInfo_06_choice[] = {
  {   0, &hf_rrc_dl_DPCH_InfoPerRL_02, ASN1_NO_EXTENSIONS     , dissect_rrc_DL_DPCH_InfoPerRL_r5 },
  {   1, &hf_rrc_dl_FDPCH_InfoPerRL_01, ASN1_NO_EXTENSIONS     , dissect_rrc_DL_FDPCH_InfoPerRL_r7 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_dl_dpchInfo_06(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_dl_dpchInfo_06, T_dl_dpchInfo_06_choice,
                                 NULL);

  return offset;
}



static int
dissect_rrc_ActivationTimeOffset(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            0U, 255U, NULL, FALSE);

  return offset;
}


static const per_sequence_t T_tdd384_16_sequence[] = {
  { &hf_rrc_dl_HSPDSCH_TS_Configuration, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_HSPDSCH_TS_Configuration },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_tdd384_16(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_tdd384_16, T_tdd384_16_sequence);

  return offset;
}


static const per_sequence_t T_tdd768_08_sequence[] = {
  { &hf_rrc_dl_HSPDSCH_TS_Configuration_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_HSPDSCH_TS_Configuration_VHCR },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_tdd768_08(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_tdd768_08, T_tdd768_08_sequence);

  return offset;
}


static const per_sequence_t T_tdd128_24_sequence[] = {
  { &hf_rrc_hs_PDSCH_Midamble_Configuration_tdd128, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_HS_PDSCH_Midamble_Configuration_TDD128 },
  { &hf_rrc_dl_MultiCarrier_Information, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_MultiCarrier_Information },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_tdd128_24(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_tdd128_24, T_tdd128_24_sequence);

  return offset;
}


static const value_string rrc_T_tdd_41_vals[] = {
  {   0, "tdd384" },
  {   1, "tdd768" },
  {   2, "tdd128" },
  { 0, NULL }
};

static const per_choice_t T_tdd_41_choice[] = {
  {   0, &hf_rrc_tdd384_17       , ASN1_NO_EXTENSIONS     , dissect_rrc_T_tdd384_16 },
  {   1, &hf_rrc_tdd768_09       , ASN1_NO_EXTENSIONS     , dissect_rrc_T_tdd768_08 },
  {   2, &hf_rrc_tdd128_24       , ASN1_NO_EXTENSIONS     , dissect_rrc_T_tdd128_24 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_tdd_41(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_tdd_41, T_tdd_41_choice,
                                 NULL);

  return offset;
}


static const value_string rrc_T_dl_64QAM_Configured_01_vals[] = {
  {   0, "true" },
  { 0, NULL }
};


static int
dissect_rrc_T_dl_64QAM_Configured_01(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t T_fdd_77_sequence[] = {
  { &hf_rrc_dl_64QAM_Configured_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_dl_64QAM_Configured_01 },
  { &hf_rrc_hs_DSCH_TBSizeTable, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_HS_DSCH_TBSizeTable },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_fdd_77(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_fdd_77, T_fdd_77_sequence);

  return offset;
}


static const value_string rrc_T_modeSpecificInfo_44_vals[] = {
  {   0, "tdd" },
  {   1, "fdd" },
  { 0, NULL }
};

static const per_choice_t T_modeSpecificInfo_44_choice[] = {
  {   0, &hf_rrc_tdd_42          , ASN1_NO_EXTENSIONS     , dissect_rrc_T_tdd_41 },
  {   1, &hf_rrc_fdd_78          , ASN1_NO_EXTENSIONS     , dissect_rrc_T_fdd_77 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_modeSpecificInfo_44(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_modeSpecificInfo_44, T_modeSpecificInfo_44_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t DL_HSPDSCH_Information_r8_sequence[] = {
  { &hf_rrc_hs_scch_Info_02 , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_HS_SCCH_Info_r7 },
  { &hf_rrc_measurement_feedback_Info_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_Measurement_Feedback_Info_r7 },
  { &hf_rrc_modeSpecificInfo_44, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_modeSpecificInfo_44 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_DL_HSPDSCH_Information_r8(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_DL_HSPDSCH_Information_r8, DL_HSPDSCH_Information_r8_sequence);

  return offset;
}


static const value_string rrc_T_mac_hsResetIndicator_02_vals[] = {
  {   0, "true" },
  { 0, NULL }
};


static int
dissect_rrc_T_mac_hsResetIndicator_02(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t Serving_HSDSCH_CellInformation_r8_sequence[] = {
  { &hf_rrc_deltaACK        , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DeltaACK },
  { &hf_rrc_deltaNACK       , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DeltaNACK },
  { &hf_rrc_harq_Preamble_Mode, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_HARQ_Preamble_Mode },
  { &hf_rrc_primaryCPICH_Info, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_PrimaryCPICH_Info },
  { &hf_rrc_dl_hspdsch_Information_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_HSPDSCH_Information_r8 },
  { &hf_rrc_harqInfo        , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_HARQ_Info_r7 },
  { &hf_rrc_mac_hsResetIndicator_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_mac_hsResetIndicator_02 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_Serving_HSDSCH_CellInformation_r8(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_Serving_HSDSCH_CellInformation_r8, Serving_HSDSCH_CellInformation_r8_sequence);

  return offset;
}


static const value_string rrc_T_dl_64QAM_Configured_04_vals[] = {
  {   0, "true" },
  { 0, NULL }
};


static int
dissect_rrc_T_dl_64QAM_Configured_04(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t T_newConfiguration_02_sequence[] = {
  { &hf_rrc_new_H_RNTI      , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_H_RNTI },
  { &hf_rrc_dl_64QAM_Configured_04, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_dl_64QAM_Configured_04 },
  { &hf_rrc_hs_DSCH_TBSizeTable, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_HS_DSCH_TBSizeTable },
  { &hf_rrc_primaryCPICH_Info, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PrimaryCPICH_Info },
  { &hf_rrc_dl_ScramblingCode, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_SecondaryScramblingCode },
  { &hf_rrc_hS_SCCHChannelisationCodeInfo, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SEQUENCE_SIZE_1_maxHSSCCHs_OF_HS_SCCH_Codes },
  { &hf_rrc_measurementPowerOffset, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MeasurementPowerOffset },
  { &hf_rrc_uarfcn_DL       , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UARFCN },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_newConfiguration_02(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_newConfiguration_02, T_newConfiguration_02_sequence);

  return offset;
}


static const value_string rrc_DL_SecondaryCellInfoFDD_vals[] = {
  {   0, "continue" },
  {   1, "newConfiguration" },
  { 0, NULL }
};

static const per_choice_t DL_SecondaryCellInfoFDD_choice[] = {
  {   0, &hf_rrc_continue        , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   1, &hf_rrc_newConfiguration_02, ASN1_NO_EXTENSIONS     , dissect_rrc_T_newConfiguration_02 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_DL_SecondaryCellInfoFDD(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_DL_SecondaryCellInfoFDD, DL_SecondaryCellInfoFDD_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t TargetCellPreconfigInfo_sequence[] = {
  { &hf_rrc_activationTimeOffset, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_ActivationTimeOffset },
  { &hf_rrc_new_H_RNTI      , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_H_RNTI },
  { &hf_rrc_newPrimary_E_RNTI, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_E_RNTI },
  { &hf_rrc_newSecondary_E_RNTI, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_E_RNTI },
  { &hf_rrc_serving_HSDSCH_CellInformation_02, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_Serving_HSDSCH_CellInformation_r8 },
  { &hf_rrc_e_dch_ReconfigurationInfo_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_E_DCH_ReconfigurationInfo_r7 },
  { &hf_rrc_dtx_drx_TimingInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DTX_DRX_TimingInfo_r7 },
  { &hf_rrc_dtx_drx_Info    , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DTX_DRX_Info_r7 },
  { &hf_rrc_hs_scch_LessInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_HS_SCCH_LessInfo_r7 },
  { &hf_rrc_mimoParameters_03, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MIMO_Parameters_r8 },
  { &hf_rrc_dl_SecondaryCellInfoFDD, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_SecondaryCellInfoFDD },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_TargetCellPreconfigInfo(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_TargetCellPreconfigInfo, TargetCellPreconfigInfo_sequence);

  return offset;
}


static const per_sequence_t RL_AdditionInformation_r8_sequence[] = {
  { &hf_rrc_primaryCPICH_Info, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PrimaryCPICH_Info },
  { &hf_rrc_cell_Id         , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CellIdentity },
  { &hf_rrc_dl_dpchInfo_06  , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_dl_dpchInfo_06 },
  { &hf_rrc_e_HICH_Information, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_E_HICH_Information },
  { &hf_rrc_e_RGCH_Information, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_E_RGCH_Information },
  { &hf_rrc_targetCellPreconfigInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_TargetCellPreconfigInfo },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_RL_AdditionInformation_r8(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_RL_AdditionInformation_r8, RL_AdditionInformation_r8_sequence);

  return offset;
}


static const per_sequence_t RL_AdditionInformationList_r8_sequence_of[1] = {
  { &hf_rrc_RL_AdditionInformationList_r8_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RL_AdditionInformation_r8 },
};

static int
dissect_rrc_RL_AdditionInformationList_r8(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_RL_AdditionInformationList_r8, RL_AdditionInformationList_r8_sequence_of,
                                                  1, maxRL_1, FALSE);

  return offset;
}



static int
dissect_rrc_ServingCellChangeMACreset(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_boolean(tvb, offset, actx, tree, hf_index, NULL);

  return offset;
}


static const value_string rrc_ServingCellChangeMsgType_vals[] = {
  {   0, "radioBearerSetup" },
  {   1, "radioBearerReconfiguration" },
  {   2, "transportChannelReconfiguration" },
  {   3, "physicalChannelReconfiguration" },
  { 0, NULL }
};


static int
dissect_rrc_ServingCellChangeMsgType(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     4, NULL, FALSE, 0, NULL);

  return offset;
}



static int
dissect_rrc_ServingCellChangeTrId(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            0U, 3U, NULL, FALSE);

  return offset;
}


static const per_sequence_t ServingCellChangeParameters_sequence[] = {
  { &hf_rrc_servingCellChangeMACreset, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_ServingCellChangeMACreset },
  { &hf_rrc_servingCellChangeMsgType, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_ServingCellChangeMsgType },
  { &hf_rrc_servingCellChangeTrId, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_ServingCellChangeTrId },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_ServingCellChangeParameters(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_ServingCellChangeParameters, ServingCellChangeParameters_sequence);

  return offset;
}


static const per_sequence_t ActiveSetUpdate_r8_IEs_sequence[] = {
  { &hf_rrc_activationTime  , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_ActivationTime },
  { &hf_rrc_newU_RNTI       , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_U_RNTI },
  { &hf_rrc_newH_RNTI       , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_H_RNTI },
  { &hf_rrc_newPrimary_E_RNTI, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_E_RNTI },
  { &hf_rrc_newSecondary_E_RNTI, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_E_RNTI },
  { &hf_rrc_cn_InformationInfo_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CN_InformationInfo_r6 },
  { &hf_rrc_dtx_drx_TimingInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DTX_DRX_TimingInfo_r7 },
  { &hf_rrc_dtx_drx_Info    , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DTX_DRX_Info_r7 },
  { &hf_rrc_hs_scch_LessInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_HS_SCCH_LessInfo_r7 },
  { &hf_rrc_mimoParameters_03, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MIMO_Parameters_r8 },
  { &hf_rrc_maxAllowedUL_TX_Power, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MaxAllowedUL_TX_Power },
  { &hf_rrc_rl_AdditionInformationList_03, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RL_AdditionInformationList_r8 },
  { &hf_rrc_servingCellChangeParameters, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_ServingCellChangeParameters },
  { &hf_rrc_rl_RemovalInformationList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RL_RemovalInformationList },
  { &hf_rrc_tx_DiversityMode, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_TX_DiversityMode },
  { &hf_rrc_dpc_Mode        , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DPC_Mode },
  { &hf_rrc_serving_HSDSCH_CellInformation_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_Serving_HSDSCH_CellInformation_r8 },
  { &hf_rrc_e_dch_ReconfigurationInfo_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_E_DCH_ReconfigurationInfo_r7 },
  { &hf_rrc_ul_16QAM_Config , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_16QAM_Config },
  { &hf_rrc_e_dch_ReconfInfoSameCell, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_E_DCH_RL_InfoSameServingCell },
  { &hf_rrc_e_TFC_Boost_Info, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_E_TFC_Boost_Info_r7 },
  { &hf_rrc_e_DPDCH_PowerInterpolation, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_E_DPDCH_PowerInterpolation },
  { &hf_rrc_dl_SecondaryCellInfoFDD, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_SecondaryCellInfoFDD },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_ActiveSetUpdate_r8_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_ActiveSetUpdate_r8_IEs, ActiveSetUpdate_r8_IEs_sequence);

  return offset;
}


static const value_string rrc_T_differentTxModeFromServingHS_DSCHCell_vals[] = {
  {   0, "different" },
  { 0, NULL }
};


static int
dissect_rrc_T_differentTxModeFromServingHS_DSCHCell(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t DL_SecondaryCellInfoFDD_v890ext_sequence[] = {
  { &hf_rrc_differentTxModeFromServingHS_DSCHCell, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_differentTxModeFromServingHS_DSCHCell },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_DL_SecondaryCellInfoFDD_v890ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_DL_SecondaryCellInfoFDD_v890ext, DL_SecondaryCellInfoFDD_v890ext_sequence);

  return offset;
}



static int
dissect_rrc_TargetCellPreconfigInfo_v890ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_rrc_MIMO_Parameters_v7f0ext(tvb, offset, actx, tree, hf_index);

  return offset;
}


static const per_sequence_t RL_AdditionInformation_v890ext_sequence[] = {
  { &hf_rrc_targetCellPreconfigInfo_05, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_TargetCellPreconfigInfo_v890ext },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_RL_AdditionInformation_v890ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_RL_AdditionInformation_v890ext, RL_AdditionInformation_v890ext_sequence);

  return offset;
}


static const per_sequence_t RL_AdditionInformationList_v890ext_sequence_of[1] = {
  { &hf_rrc_RL_AdditionInformationList_v890ext_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RL_AdditionInformation_v890ext },
};

static int
dissect_rrc_RL_AdditionInformationList_v890ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_RL_AdditionInformationList_v890ext, RL_AdditionInformationList_v890ext_sequence_of,
                                                  1, maxRL_1, FALSE);

  return offset;
}


static const per_sequence_t ActiveSetUpdate_v890ext_IEs_sequence[] = {
  { &hf_rrc_dl_SecondaryCellInfoFDD_v890ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_SecondaryCellInfoFDD_v890ext },
  { &hf_rrc_mimoParameters_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MIMO_Parameters_v7f0ext },
  { &hf_rrc_rl_AdditionInformationList_04, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RL_AdditionInformationList_v890ext },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_ActiveSetUpdate_v890ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_ActiveSetUpdate_v890ext_IEs, ActiveSetUpdate_v890ext_IEs_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions_03_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_nonCriticalExtensions_03(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_nonCriticalExtensions_03, T_nonCriticalExtensions_03_sequence);

  return offset;
}


static const per_sequence_t T_v7g0NonCriticalExtensions_01_sequence[] = {
  { &hf_rrc_activeSetUpdate_v7g0ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_ActiveSetUpdate_v7g0ext_IEs },
  { &hf_rrc_nonCriticalExtensions_03, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_nonCriticalExtensions_03 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v7g0NonCriticalExtensions_01(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v7g0NonCriticalExtensions_01, T_v7g0NonCriticalExtensions_01_sequence);

  return offset;
}


static const per_sequence_t T_v890NonCriticalExtensions_sequence[] = {
  { &hf_rrc_activeSetUpdate_v890ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_ActiveSetUpdate_v890ext_IEs },
  { &hf_rrc_v7g0NonCriticalExtensions_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v7g0NonCriticalExtensions_01 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v890NonCriticalExtensions(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v890NonCriticalExtensions, T_v890NonCriticalExtensions_sequence);

  return offset;
}


static const per_sequence_t T_r8_sequence[] = {
  { &hf_rrc_activeSetUpdate_r8, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_ActiveSetUpdate_r8_IEs },
  { &hf_rrc_activeSetUpdate_r8_add_ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_BIT_STRING },
  { &hf_rrc_v890NonCriticalExtensions, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v890NonCriticalExtensions },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_r8(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_r8, T_r8_sequence);

  return offset;
}


static const per_sequence_t T_diversityPattern_01_sequence[] = {
  { &hf_rrc_channelisationCode, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_ChannelisationCode256 },
  { &hf_rrc_s_cpich_PowerOffset_Mimo_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_S_CPICH_PowerOffset_MIMO },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_diversityPattern_01(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_diversityPattern_01, T_diversityPattern_01_sequence);

  return offset;
}


static const value_string rrc_T_secondCPICH_Pattern_01_vals[] = {
  {   0, "normalPattern" },
  {   1, "diversityPattern" },
  { 0, NULL }
};

static const per_choice_t T_secondCPICH_Pattern_01_choice[] = {
  {   0, &hf_rrc_normalPattern   , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   1, &hf_rrc_diversityPattern_01, ASN1_NO_EXTENSIONS     , dissect_rrc_T_diversityPattern_01 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_secondCPICH_Pattern_01(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_secondCPICH_Pattern_01, T_secondCPICH_Pattern_01_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t MIMO_PilotConfiguration_r9_sequence[] = {
  { &hf_rrc_secondCPICH_Pattern_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_secondCPICH_Pattern_01 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_MIMO_PilotConfiguration_r9(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_MIMO_PilotConfiguration_r9, MIMO_PilotConfiguration_r9_sequence);

  return offset;
}


static const value_string rrc_T_precodingWeightSetRestriction_01_vals[] = {
  {   0, "true" },
  { 0, NULL }
};


static int
dissect_rrc_T_precodingWeightSetRestriction_01(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t T_fdd_101_sequence[] = {
  { &hf_rrc_mimoN_M_Ratio   , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MIMO_N_M_Ratio },
  { &hf_rrc_mimoPilotConfiguration_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MIMO_PilotConfiguration_r9 },
  { &hf_rrc_precodingWeightSetRestriction_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_precodingWeightSetRestriction_01 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_fdd_101(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_fdd_101, T_fdd_101_sequence);

  return offset;
}


static const value_string rrc_T_mimoSFModeForHSPDSCHDualStream_01_vals[] = {
  {   0, "sf1" },
  {   1, "sf1sf16" },
  { 0, NULL }
};


static int
dissect_rrc_T_mimoSFModeForHSPDSCHDualStream_01(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     2, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t T_tdd128_40_sequence[] = {
  { &hf_rrc_mimoSFModeForHSPDSCHDualStream_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_mimoSFModeForHSPDSCHDualStream_01 },
  { &hf_rrc_hs_sich_ReferenceSignalInfoList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_HS_SICH_ReferenceSignalInfoList },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_tdd128_40(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_tdd128_40, T_tdd128_40_sequence);

  return offset;
}


static const value_string rrc_T_tdd_56_vals[] = {
  {   0, "tdd128" },
  {   1, "tdd384-tdd768" },
  { 0, NULL }
};

static const per_choice_t T_tdd_56_choice[] = {
  {   0, &hf_rrc_tdd128_41       , ASN1_NO_EXTENSIONS     , dissect_rrc_T_tdd128_40 },
  {   1, &hf_rrc_tdd384_tdd768_04, ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_tdd_56(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_tdd_56, T_tdd_56_choice,
                                 NULL);

  return offset;
}


static const value_string rrc_T_modeSpecificInfo_71_vals[] = {
  {   0, "fdd" },
  {   1, "tdd" },
  { 0, NULL }
};

static const per_choice_t T_modeSpecificInfo_71_choice[] = {
  {   0, &hf_rrc_fdd_103         , ASN1_NO_EXTENSIONS     , dissect_rrc_T_fdd_101 },
  {   1, &hf_rrc_tdd_60          , ASN1_NO_EXTENSIONS     , dissect_rrc_T_tdd_56 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_modeSpecificInfo_71(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_modeSpecificInfo_71, T_modeSpecificInfo_71_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t MIMO_Parameters_r9_sequence[] = {
  { &hf_rrc_mimoOperation   , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MIMO_Operation },
  { &hf_rrc_modeSpecificInfo_71, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_modeSpecificInfo_71 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_MIMO_Parameters_r9(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_MIMO_Parameters_r9, MIMO_Parameters_r9_sequence);

  return offset;
}


static const per_sequence_t SecondaryServingEDCHCell_Info_sequence[] = {
  { &hf_rrc_primary_E_RNTI  , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_E_RNTI },
  { &hf_rrc_secondary_E_RNTI, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_E_RNTI },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_SecondaryServingEDCHCell_Info(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_SecondaryServingEDCHCell_Info, SecondaryServingEDCHCell_Info_sequence);

  return offset;
}


static const per_sequence_t FrequencyInfoFDD_sequence[] = {
  { &hf_rrc_uarfcn_UL       , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UARFCN },
  { &hf_rrc_uarfcn_DL       , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UARFCN },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_FrequencyInfoFDD(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_FrequencyInfoFDD, FrequencyInfoFDD_sequence);

  return offset;
}


static const per_sequence_t FrequencyInfoTDD_sequence[] = {
  { &hf_rrc_uarfcn_Nt       , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UARFCN },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_FrequencyInfoTDD(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_FrequencyInfoTDD, FrequencyInfoTDD_sequence);

  return offset;
}


static const value_string rrc_T_modeSpecificInfo_61_vals[] = {
  {   0, "fdd" },
  {   1, "tdd" },
  { 0, NULL }
};

static const per_choice_t T_modeSpecificInfo_61_choice[] = {
  {   0, &hf_rrc_fdd_94          , ASN1_NO_EXTENSIONS     , dissect_rrc_FrequencyInfoFDD },
  {   1, &hf_rrc_tdd_53          , ASN1_NO_EXTENSIONS     , dissect_rrc_FrequencyInfoTDD },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_modeSpecificInfo_61(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_modeSpecificInfo_61, T_modeSpecificInfo_61_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t FrequencyInfo_sequence[] = {
  { &hf_rrc_modeSpecificInfo_61, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_modeSpecificInfo_61 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_FrequencyInfo(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_FrequencyInfo, FrequencyInfo_sequence);

  return offset;
}


static const value_string rrc_ScramblingCodeType_vals[] = {
  {   0, "shortSC" },
  {   1, "longSC" },
  { 0, NULL }
};


static int
dissect_rrc_ScramblingCodeType(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     2, NULL, FALSE, 0, NULL);

  return offset;
}



static int
dissect_rrc_UL_ScramblingCode(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            0U, 16777215U, NULL, FALSE);

  return offset;
}



static int
dissect_rrc_BIT_STRING_SIZE_8(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_bit_string(tvb, offset, actx, tree, hf_index,
                                     8, 8, FALSE, NULL);

  return offset;
}


static const value_string rrc_T_primary_Secondary_GrantSelector_vals[] = {
  {   0, "primary" },
  {   1, "secondary" },
  { 0, NULL }
};


static int
dissect_rrc_T_primary_Secondary_GrantSelector(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     2, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t T_servingGrant_sequence[] = {
  { &hf_rrc_primary_Secondary_GrantSelector, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_primary_Secondary_GrantSelector },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_servingGrant(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_servingGrant, T_servingGrant_sequence);

  return offset;
}


static const value_string rrc_MinReduced_E_DPDCH_GainFactor_vals[] = {
  {   0, "m8-15" },
  {   1, "m11-15" },
  {   2, "m15-15" },
  {   3, "m21-15" },
  {   4, "m30-15" },
  {   5, "m42-15" },
  {   6, "m60-15" },
  {   7, "m84-15" },
  { 0, NULL }
};


static int
dissect_rrc_MinReduced_E_DPDCH_GainFactor(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     8, NULL, FALSE, 0, NULL);

  return offset;
}



static int
dissect_rrc_E_DCH_MinimumSet_E_TFCI(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            0U, 127U, NULL, FALSE);

  return offset;
}



static int
dissect_rrc_PC_Preamble(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            0U, 7U, NULL, FALSE);

  return offset;
}


static const per_sequence_t SecondaryEDCH_Info_Common_sequence[] = {
  { &hf_rrc_frequencyInfo   , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_FrequencyInfo },
  { &hf_rrc_scramblingCodeType, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_ScramblingCodeType },
  { &hf_rrc_scramblingCodeNumber, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UL_ScramblingCode },
  { &hf_rrc_ms2_SchedTransmGrantHARQAlloc, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_BIT_STRING_SIZE_8 },
  { &hf_rrc_servingGrant    , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_servingGrant },
  { &hf_rrc_minReduced_E_DPDCH_GainFactor, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MinReduced_E_DPDCH_GainFactor },
  { &hf_rrc_e_DCH_minimumSet_E_TFCI, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_E_DCH_MinimumSet_E_TFCI },
  { &hf_rrc_dpcchPowerOffset_SecondaryULFrequency, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_0_7 },
  { &hf_rrc_pc_Preamble     , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PC_Preamble },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_SecondaryEDCH_Info_Common(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_SecondaryEDCH_Info_Common, SecondaryEDCH_Info_Common_sequence);

  return offset;
}


static const value_string rrc_T_e_RGCH_Info_vals[] = {
  {   0, "e-RGCH-Information" },
  {   1, "releaseIndicator" },
  { 0, NULL }
};

static const per_choice_t T_e_RGCH_Info_choice[] = {
  {   0, &hf_rrc_e_RGCH_Information, ASN1_NO_EXTENSIONS     , dissect_rrc_E_RGCH_Information },
  {   1, &hf_rrc_releaseIndicator, ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_e_RGCH_Info(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_e_RGCH_Info, T_e_RGCH_Info_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t DL_InformationPerSecondaryRL_sequence[] = {
  { &hf_rrc_primaryCPICH_Info, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PrimaryCPICH_Info },
  { &hf_rrc_cell_id         , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CellIdentity },
  { &hf_rrc_dl_FDPCHInfoPerRL_SecULFreq, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DL_FDPCH_InfoPerRL_r7 },
  { &hf_rrc_e_AGCH_Information, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_E_AGCH_Information },
  { &hf_rrc_e_HICH_Information, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_E_HICH_Information },
  { &hf_rrc_e_RGCH_Info     , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_e_RGCH_Info },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_DL_InformationPerSecondaryRL(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_DL_InformationPerSecondaryRL, DL_InformationPerSecondaryRL_sequence);

  return offset;
}


static const per_sequence_t DL_InformationPerSecondaryRL_List_sequence_of[1] = {
  { &hf_rrc_DL_InformationPerSecondaryRL_List_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DL_InformationPerSecondaryRL },
};

static int
dissect_rrc_DL_InformationPerSecondaryRL_List(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_DL_InformationPerSecondaryRL_List, DL_InformationPerSecondaryRL_List_sequence_of,
                                                  1, maxEDCHRL, FALSE);

  return offset;
}


static const per_sequence_t T_newConfiguration_sequence[] = {
  { &hf_rrc_secondaryServingEDCHCell_Info, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_SecondaryServingEDCHCell_Info },
  { &hf_rrc_secondaryEDCH_Info_Common, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_SecondaryEDCH_Info_Common },
  { &hf_rrc_dl_InformationPerSecondaryRL_List, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_InformationPerSecondaryRL_List },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_newConfiguration(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_newConfiguration, T_newConfiguration_sequence);

  return offset;
}


static const value_string rrc_UL_SecondaryCellInfoFDD_vals[] = {
  {   0, "continue" },
  {   1, "newConfiguration" },
  { 0, NULL }
};

static const per_choice_t UL_SecondaryCellInfoFDD_choice[] = {
  {   0, &hf_rrc_continue        , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   1, &hf_rrc_newConfiguration, ASN1_NO_EXTENSIONS     , dissect_rrc_T_newConfiguration },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_UL_SecondaryCellInfoFDD(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_UL_SecondaryCellInfoFDD, UL_SecondaryCellInfoFDD_choice,
                                 NULL);

  return offset;
}


static const value_string rrc_T_e_RGCH_Info_08_vals[] = {
  {   0, "e-RGCH-Information" },
  {   1, "releaseIndicator" },
  { 0, NULL }
};

static const per_choice_t T_e_RGCH_Info_08_choice[] = {
  {   0, &hf_rrc_e_RGCH_Information, ASN1_NO_EXTENSIONS     , dissect_rrc_E_RGCH_Information },
  {   1, &hf_rrc_releaseIndicator, ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_e_RGCH_Info_08(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_e_RGCH_Info_08, T_e_RGCH_Info_08_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t E_DCH_RL_InfoNewSecServingCell_sequence[] = {
  { &hf_rrc_e_AGCH_Information, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_E_AGCH_Information },
  { &hf_rrc_e_HICH_Information, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_E_HICH_Information },
  { &hf_rrc_e_RGCH_Info_08  , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_e_RGCH_Info_08 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_E_DCH_RL_InfoNewSecServingCell(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_E_DCH_RL_InfoNewSecServingCell, E_DCH_RL_InfoNewSecServingCell_sequence);

  return offset;
}


static const value_string rrc_T_e_RGCH_Info_11_vals[] = {
  {   0, "e-RGCH-Information" },
  {   1, "releaseIndicator" },
  { 0, NULL }
};

static const per_choice_t T_e_RGCH_Info_11_choice[] = {
  {   0, &hf_rrc_e_RGCH_Information, ASN1_NO_EXTENSIONS     , dissect_rrc_E_RGCH_Information },
  {   1, &hf_rrc_releaseIndicator, ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_e_RGCH_Info_11(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_e_RGCH_Info_11, T_e_RGCH_Info_11_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t E_DCH_RL_InfoOtherCell_SecULFreq_sequence[] = {
  { &hf_rrc_primaryCPICH_Info, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PrimaryCPICH_Info },
  { &hf_rrc_e_HICH_Information, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_E_HICH_Information },
  { &hf_rrc_e_RGCH_Info_11  , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_e_RGCH_Info_11 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_E_DCH_RL_InfoOtherCell_SecULFreq(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_E_DCH_RL_InfoOtherCell_SecULFreq, E_DCH_RL_InfoOtherCell_SecULFreq_sequence);

  return offset;
}


static const per_sequence_t SEQUENCE_SIZE_1_maxEDCHRL_OF_E_DCH_RL_InfoOtherCell_SecULFreq_sequence_of[1] = {
  { &hf_rrc_e_DCH_RL_InfoOtherCellList_SecULFreq_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_E_DCH_RL_InfoOtherCell_SecULFreq },
};

static int
dissect_rrc_SEQUENCE_SIZE_1_maxEDCHRL_OF_E_DCH_RL_InfoOtherCell_SecULFreq(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_SEQUENCE_SIZE_1_maxEDCHRL_OF_E_DCH_RL_InfoOtherCell_SecULFreq, SEQUENCE_SIZE_1_maxEDCHRL_OF_E_DCH_RL_InfoOtherCell_SecULFreq_sequence_of,
                                                  1, maxEDCHRL, FALSE);

  return offset;
}


static const per_sequence_t E_DCH_ReconfigurationInfo_SecULFrequency_sequence[] = {
  { &hf_rrc_e_DCH_RL_InfoNewSecServingCell, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_E_DCH_RL_InfoNewSecServingCell },
  { &hf_rrc_e_DCH_RL_InfoOtherCellList_SecULFreq, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_SEQUENCE_SIZE_1_maxEDCHRL_OF_E_DCH_RL_InfoOtherCell_SecULFreq },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_E_DCH_ReconfigurationInfo_SecULFrequency(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_E_DCH_ReconfigurationInfo_SecULFrequency, E_DCH_ReconfigurationInfo_SecULFrequency_sequence);

  return offset;
}


static const value_string rrc_T_dl_dpchInfo_07_vals[] = {
  {   0, "dl-DPCH-InfoPerRL" },
  {   1, "dl-FDPCH-InfoPerRL" },
  { 0, NULL }
};

static const per_choice_t T_dl_dpchInfo_07_choice[] = {
  {   0, &hf_rrc_dl_DPCH_InfoPerRL_02, ASN1_NO_EXTENSIONS     , dissect_rrc_DL_DPCH_InfoPerRL_r5 },
  {   1, &hf_rrc_dl_FDPCH_InfoPerRL_01, ASN1_NO_EXTENSIONS     , dissect_rrc_DL_FDPCH_InfoPerRL_r7 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_dl_dpchInfo_07(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_dl_dpchInfo_07, T_dl_dpchInfo_07_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t T_fdd_96_sequence[] = {
  { &hf_rrc_hS_SCCHChannelisationCodeInfo, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SEQUENCE_SIZE_1_maxHSSCCHs_OF_HS_SCCH_Codes },
  { &hf_rrc_dl_ScramblingCode, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_SecondaryScramblingCode },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_fdd_96(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_fdd_96, T_fdd_96_sequence);

  return offset;
}


static const per_sequence_t T_tdd384_26_sequence[] = {
  { &hf_rrc_nack_ack_power_offset, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_M7_8 },
  { &hf_rrc_hs_SICH_PowerControl_Info, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_HS_SICH_Power_Control_Info_TDD384 },
  { &hf_rrc_dhs_sync        , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DHS_Sync },
  { &hf_rrc_bler_target     , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_Bler_Target },
  { &hf_rrc_hS_SCCH_SetConfiguration_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SEQUENCE_SIZE_1_maxHSSCCHs_OF_HS_SCCH_TDD384_r6 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_tdd384_26(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_tdd384_26, T_tdd384_26_sequence);

  return offset;
}


static const per_sequence_t T_tdd768_16_sequence[] = {
  { &hf_rrc_nack_ack_power_offset, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_M7_8 },
  { &hf_rrc_hs_SICH_PowerControl_Info_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_HS_SICH_Power_Control_Info_TDD768 },
  { &hf_rrc_dhs_sync        , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DHS_Sync },
  { &hf_rrc_bler_target     , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_Bler_Target },
  { &hf_rrc_hS_SCCH_SetConfiguration_03, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SEQUENCE_SIZE_1_maxHSSCCHs_OF_HS_SCCH_TDD768 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_tdd768_16(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_tdd768_16, T_tdd768_16_sequence);

  return offset;
}


static const value_string rrc_T_tpc_step_size_02_vals[] = {
  {   0, "s1" },
  {   1, "s2" },
  {   2, "s3" },
  {   3, "spare1" },
  { 0, NULL }
};


static int
dissect_rrc_T_tpc_step_size_02(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     4, NULL, FALSE, 0, NULL);

  return offset;
}


static const value_string rrc_T_hS_SCCH_tpc_step_size_01_vals[] = {
  {   0, "s1" },
  {   1, "s2" },
  {   2, "s3" },
  {   3, "spare1" },
  { 0, NULL }
};


static int
dissect_rrc_T_hS_SCCH_tpc_step_size_01(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     4, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t T_tdd128_38_sequence[] = {
  { &hf_rrc_nack_ack_power_offset, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_M7_8 },
  { &hf_rrc_power_level_HSSICH, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_M120_M58 },
  { &hf_rrc_tpc_step_size_02, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_tpc_step_size_02 },
  { &hf_rrc_bler_target     , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_Bler_Target },
  { &hf_rrc_hS_SCCH_tpc_step_size_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_hS_SCCH_tpc_step_size_01 },
  { &hf_rrc_powerControlGAP , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_PowerControlGAP },
  { &hf_rrc_pathlossCompensationSwitch, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_BOOLEAN },
  { &hf_rrc_hS_SCCH_SetConfiguration_02, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SEQUENCE_SIZE_1_maxHSSCCHs_OF_HS_SCCH_TDD128_r6 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_tdd128_38(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_tdd128_38, T_tdd128_38_sequence);

  return offset;
}


static const value_string rrc_T_tdd_54_vals[] = {
  {   0, "tdd384" },
  {   1, "tdd768" },
  {   2, "tdd128" },
  { 0, NULL }
};

static const per_choice_t T_tdd_54_choice[] = {
  {   0, &hf_rrc_tdd384_28       , ASN1_NO_EXTENSIONS     , dissect_rrc_T_tdd384_26 },
  {   1, &hf_rrc_tdd768_18       , ASN1_NO_EXTENSIONS     , dissect_rrc_T_tdd768_16 },
  {   2, &hf_rrc_tdd128_39       , ASN1_NO_EXTENSIONS     , dissect_rrc_T_tdd128_38 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_tdd_54(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_tdd_54, T_tdd_54_choice,
                                 NULL);

  return offset;
}


static const value_string rrc_T_modeSpecificInfo_66_vals[] = {
  {   0, "fdd" },
  {   1, "tdd" },
  { 0, NULL }
};

static const per_choice_t T_modeSpecificInfo_66_choice[] = {
  {   0, &hf_rrc_fdd_98          , ASN1_NO_EXTENSIONS     , dissect_rrc_T_fdd_96 },
  {   1, &hf_rrc_tdd_58          , ASN1_NO_EXTENSIONS     , dissect_rrc_T_tdd_54 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_modeSpecificInfo_66(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_modeSpecificInfo_66, T_modeSpecificInfo_66_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t HS_SCCH_Info_r9_sequence[] = {
  { &hf_rrc_modeSpecificInfo_66, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_modeSpecificInfo_66 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_HS_SCCH_Info_r9(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_HS_SCCH_Info_r9, HS_SCCH_Info_r9_sequence);

  return offset;
}


static const per_sequence_t T_tdd384_17_sequence[] = {
  { &hf_rrc_dl_HSPDSCH_TS_Configuration, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_HSPDSCH_TS_Configuration },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_tdd384_17(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_tdd384_17, T_tdd384_17_sequence);

  return offset;
}


static const per_sequence_t T_tdd768_09_sequence[] = {
  { &hf_rrc_dl_HSPDSCH_TS_Configuration_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_HSPDSCH_TS_Configuration_VHCR },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_tdd768_09(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_tdd768_09, T_tdd768_09_sequence);

  return offset;
}


static const value_string rrc_T_tS0_Indicator_vals[] = {
  {   0, "true" },
  { 0, NULL }
};


static int
dissect_rrc_T_tS0_Indicator(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}


static const value_string rrc_OutofSyncWindow_vals[] = {
  {   0, "ms40" },
  {   1, "ms80" },
  {   2, "ms160" },
  {   3, "ms320" },
  {   4, "ms640" },
  {   5, "spare3" },
  {   6, "spare2" },
  {   7, "spare1" },
  { 0, NULL }
};


static int
dissect_rrc_OutofSyncWindow(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     8, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t T_tdd128_26_sequence[] = {
  { &hf_rrc_hs_PDSCH_Midamble_Configuration_tdd128, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_HS_PDSCH_Midamble_Configuration_TDD128 },
  { &hf_rrc_dl_MultiCarrier_Information, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_MultiCarrier_Information },
  { &hf_rrc_tS0_Indicator   , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_tS0_Indicator },
  { &hf_rrc_outofSyncWindow , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_OutofSyncWindow },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_tdd128_26(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_tdd128_26, T_tdd128_26_sequence);

  return offset;
}


static const value_string rrc_T_tdd_43_vals[] = {
  {   0, "tdd384" },
  {   1, "tdd768" },
  {   2, "tdd128" },
  { 0, NULL }
};

static const per_choice_t T_tdd_43_choice[] = {
  {   0, &hf_rrc_tdd384_18       , ASN1_NO_EXTENSIONS     , dissect_rrc_T_tdd384_17 },
  {   1, &hf_rrc_tdd768_10       , ASN1_NO_EXTENSIONS     , dissect_rrc_T_tdd768_09 },
  {   2, &hf_rrc_tdd128_26       , ASN1_NO_EXTENSIONS     , dissect_rrc_T_tdd128_26 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_tdd_43(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_tdd_43, T_tdd_43_choice,
                                 NULL);

  return offset;
}


static const value_string rrc_T_dl_64QAM_Configured_02_vals[] = {
  {   0, "true" },
  { 0, NULL }
};


static int
dissect_rrc_T_dl_64QAM_Configured_02(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t T_fdd_78_sequence[] = {
  { &hf_rrc_dl_64QAM_Configured_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_dl_64QAM_Configured_02 },
  { &hf_rrc_hs_DSCH_TBSizeTable, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_HS_DSCH_TBSizeTable },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_fdd_78(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_fdd_78, T_fdd_78_sequence);

  return offset;
}


static const value_string rrc_T_modeSpecificInfo_46_vals[] = {
  {   0, "tdd" },
  {   1, "fdd" },
  { 0, NULL }
};

static const per_choice_t T_modeSpecificInfo_46_choice[] = {
  {   0, &hf_rrc_tdd_44          , ASN1_NO_EXTENSIONS     , dissect_rrc_T_tdd_43 },
  {   1, &hf_rrc_fdd_79          , ASN1_NO_EXTENSIONS     , dissect_rrc_T_fdd_78 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_modeSpecificInfo_46(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_modeSpecificInfo_46, T_modeSpecificInfo_46_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t DL_HSPDSCH_Information_r9_sequence[] = {
  { &hf_rrc_hs_scch_Info_04 , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_HS_SCCH_Info_r9 },
  { &hf_rrc_measurement_feedback_Info_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_Measurement_Feedback_Info_r7 },
  { &hf_rrc_modeSpecificInfo_46, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_modeSpecificInfo_46 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_DL_HSPDSCH_Information_r9(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_DL_HSPDSCH_Information_r9, DL_HSPDSCH_Information_r9_sequence);

  return offset;
}


static const value_string rrc_T_mac_hsResetIndicator_03_vals[] = {
  {   0, "true" },
  { 0, NULL }
};


static int
dissect_rrc_T_mac_hsResetIndicator_03(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t Serving_HSDSCH_CellInformation_r9_sequence[] = {
  { &hf_rrc_deltaACK        , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DeltaACK },
  { &hf_rrc_deltaNACK       , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DeltaNACK },
  { &hf_rrc_harq_Preamble_Mode, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_HARQ_Preamble_Mode },
  { &hf_rrc_primaryCPICH_Info, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_PrimaryCPICH_Info },
  { &hf_rrc_dl_hspdsch_Information_03, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_HSPDSCH_Information_r9 },
  { &hf_rrc_harqInfo        , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_HARQ_Info_r7 },
  { &hf_rrc_mac_hsResetIndicator_03, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_mac_hsResetIndicator_03 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_Serving_HSDSCH_CellInformation_r9(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_Serving_HSDSCH_CellInformation_r9, Serving_HSDSCH_CellInformation_r9_sequence);

  return offset;
}


static const value_string rrc_T_dl_64QAM_Configured_05_vals[] = {
  {   0, "true" },
  { 0, NULL }
};


static int
dissect_rrc_T_dl_64QAM_Configured_05(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}


static const value_string rrc_T_differentTxModeFromServingHS_DSCHCell_01_vals[] = {
  {   0, "different" },
  { 0, NULL }
};


static int
dissect_rrc_T_differentTxModeFromServingHS_DSCHCell_01(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t T_newConfiguration_03_sequence[] = {
  { &hf_rrc_new_H_RNTI      , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_H_RNTI },
  { &hf_rrc_dl_64QAM_Configured_05, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_dl_64QAM_Configured_05 },
  { &hf_rrc_hs_DSCH_TBSizeTable, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_HS_DSCH_TBSizeTable },
  { &hf_rrc_primaryCPICH_Info, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PrimaryCPICH_Info },
  { &hf_rrc_dl_ScramblingCode, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_SecondaryScramblingCode },
  { &hf_rrc_hS_SCCHChannelisationCodeInfo, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SEQUENCE_SIZE_1_maxHSSCCHs_OF_HS_SCCH_Codes },
  { &hf_rrc_measurementPowerOffset, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MeasurementPowerOffset },
  { &hf_rrc_uarfcn_DL       , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UARFCN },
  { &hf_rrc_differentTxModeFromServingHS_DSCHCell_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_differentTxModeFromServingHS_DSCHCell_01 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_newConfiguration_03(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_newConfiguration_03, T_newConfiguration_03_sequence);

  return offset;
}


static const value_string rrc_T_configurationInfo_01_vals[] = {
  {   0, "continue" },
  {   1, "newConfiguration" },
  { 0, NULL }
};

static const per_choice_t T_configurationInfo_01_choice[] = {
  {   0, &hf_rrc_continue        , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   1, &hf_rrc_newConfiguration_03, ASN1_NO_EXTENSIONS     , dissect_rrc_T_newConfiguration_03 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_configurationInfo_01(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_configurationInfo_01, T_configurationInfo_01_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t T_newConfiguration_08_sequence[] = {
  { &hf_rrc_mimoN_M_Ratio   , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MIMO_N_M_Ratio },
  { &hf_rrc_mimoPilotConfiguration_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MIMO_PilotConfiguration_r9 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_newConfiguration_08(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_newConfiguration_08, T_newConfiguration_08_sequence);

  return offset;
}


static const value_string rrc_SecondaryCellMIMOparametersFDD_vals[] = {
  {   0, "continue" },
  {   1, "newConfiguration" },
  { 0, NULL }
};

static const per_choice_t SecondaryCellMIMOparametersFDD_choice[] = {
  {   0, &hf_rrc_continue        , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   1, &hf_rrc_newConfiguration_08, ASN1_NO_EXTENSIONS     , dissect_rrc_T_newConfiguration_08 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_SecondaryCellMIMOparametersFDD(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_SecondaryCellMIMOparametersFDD, SecondaryCellMIMOparametersFDD_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t DL_SecondaryCellInfoFDD_r9_sequence[] = {
  { &hf_rrc_configurationInfo_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_configurationInfo_01 },
  { &hf_rrc_secondaryCellMIMOparameters, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_SecondaryCellMIMOparametersFDD },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_DL_SecondaryCellInfoFDD_r9(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_DL_SecondaryCellInfoFDD_r9, DL_SecondaryCellInfoFDD_r9_sequence);

  return offset;
}


static const per_sequence_t TargetCellPreconfigInfo_r9_sequence[] = {
  { &hf_rrc_activationTimeOffset, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_ActivationTimeOffset },
  { &hf_rrc_new_H_RNTI      , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_H_RNTI },
  { &hf_rrc_newPrimary_E_RNTI, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_E_RNTI },
  { &hf_rrc_newSecondary_E_RNTI, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_E_RNTI },
  { &hf_rrc_serving_HSDSCH_CellInformation_03, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_Serving_HSDSCH_CellInformation_r9 },
  { &hf_rrc_e_dch_ReconfigurationInfo_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_E_DCH_ReconfigurationInfo_r7 },
  { &hf_rrc_dtx_drx_TimingInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DTX_DRX_TimingInfo_r7 },
  { &hf_rrc_dtx_drx_Info    , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DTX_DRX_Info_r7 },
  { &hf_rrc_hs_scch_LessInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_HS_SCCH_LessInfo_r7 },
  { &hf_rrc_mimoParameters_04, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MIMO_Parameters_r9 },
  { &hf_rrc_dl_SecondaryCellInfoFDD_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_SecondaryCellInfoFDD_r9 },
  { &hf_rrc_ul_SecondaryCellInfoFDD, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_SecondaryCellInfoFDD },
  { &hf_rrc_e_dch_ReconfigurationInfo_SecULFrequency, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_E_DCH_ReconfigurationInfo_SecULFrequency },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_TargetCellPreconfigInfo_r9(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_TargetCellPreconfigInfo_r9, TargetCellPreconfigInfo_r9_sequence);

  return offset;
}


static const per_sequence_t RL_AdditionInformation_r9_sequence[] = {
  { &hf_rrc_primaryCPICH_Info, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PrimaryCPICH_Info },
  { &hf_rrc_cell_Id         , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CellIdentity },
  { &hf_rrc_dl_dpchInfo_07  , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_dl_dpchInfo_07 },
  { &hf_rrc_e_HICH_Information, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_E_HICH_Information },
  { &hf_rrc_e_RGCH_Information, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_E_RGCH_Information },
  { &hf_rrc_targetCellPreconfigInfo_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_TargetCellPreconfigInfo_r9 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_RL_AdditionInformation_r9(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_RL_AdditionInformation_r9, RL_AdditionInformation_r9_sequence);

  return offset;
}


static const per_sequence_t RL_AdditionInformationList_r9_sequence_of[1] = {
  { &hf_rrc_RL_AdditionInformationList_r9_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RL_AdditionInformation_r9 },
};

static int
dissect_rrc_RL_AdditionInformationList_r9(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_RL_AdditionInformationList_r9, RL_AdditionInformationList_r9_sequence_of,
                                                  1, maxRL_1, FALSE);

  return offset;
}


static const per_sequence_t RL_AdditionInformation_SecULFreq_sequence[] = {
  { &hf_rrc_primaryCPICH_Info, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PrimaryCPICH_Info },
  { &hf_rrc_cell_id         , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CellIdentity },
  { &hf_rrc_dl_FDPCH_InfoPerRL_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DL_FDPCH_InfoPerRL_r7 },
  { &hf_rrc_e_HICH_Information, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_E_HICH_Information },
  { &hf_rrc_e_RGCH_Information, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_E_RGCH_Information },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_RL_AdditionInformation_SecULFreq(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_RL_AdditionInformation_SecULFreq, RL_AdditionInformation_SecULFreq_sequence);

  return offset;
}


static const per_sequence_t RL_AdditionInformationList_SecULFreq_sequence_of[1] = {
  { &hf_rrc_RL_AdditionInformationList_SecULFreq_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RL_AdditionInformation_SecULFreq },
};

static int
dissect_rrc_RL_AdditionInformationList_SecULFreq(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_RL_AdditionInformationList_SecULFreq, RL_AdditionInformationList_SecULFreq_sequence_of,
                                                  1, maxEDCHRL_1, FALSE);

  return offset;
}


static const per_sequence_t RL_RemovalInformationList_SecULFreq_sequence_of[1] = {
  { &hf_rrc_RL_RemovalInformationList_SecULFreq_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PrimaryCPICH_Info },
};

static int
dissect_rrc_RL_RemovalInformationList_SecULFreq(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_RL_RemovalInformationList_SecULFreq, RL_RemovalInformationList_SecULFreq_sequence_of,
                                                  1, maxEDCHRL, FALSE);

  return offset;
}


static const per_sequence_t ActiveSetUpdate_r9_IEs_sequence[] = {
  { &hf_rrc_activationTime  , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_ActivationTime },
  { &hf_rrc_newU_RNTI       , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_U_RNTI },
  { &hf_rrc_newH_RNTI       , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_H_RNTI },
  { &hf_rrc_newPrimary_E_RNTI, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_E_RNTI },
  { &hf_rrc_newSecondary_E_RNTI, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_E_RNTI },
  { &hf_rrc_cn_InformationInfo_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CN_InformationInfo_r6 },
  { &hf_rrc_dtx_drx_TimingInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DTX_DRX_TimingInfo_r7 },
  { &hf_rrc_dtx_drx_Info    , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DTX_DRX_Info_r7 },
  { &hf_rrc_hs_scch_LessInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_HS_SCCH_LessInfo_r7 },
  { &hf_rrc_mimoParameters_04, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MIMO_Parameters_r9 },
  { &hf_rrc_maxAllowedUL_TX_Power, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MaxAllowedUL_TX_Power },
  { &hf_rrc_ul_SecondaryCellInfoFDD, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_SecondaryCellInfoFDD },
  { &hf_rrc_e_dch_ReconfigurationInfo_SecULFrequency, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_E_DCH_ReconfigurationInfo_SecULFrequency },
  { &hf_rrc_rl_AdditionInformationList_05, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RL_AdditionInformationList_r9 },
  { &hf_rrc_rl_AdditionInformationList_SecULFreq, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RL_AdditionInformationList_SecULFreq },
  { &hf_rrc_servingCellChangeParameters, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_ServingCellChangeParameters },
  { &hf_rrc_rl_RemovalInformationList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RL_RemovalInformationList },
  { &hf_rrc_rl_RemovalInformationList_SecULFreq, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RL_RemovalInformationList_SecULFreq },
  { &hf_rrc_tx_DiversityMode, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_TX_DiversityMode },
  { &hf_rrc_dpc_Mode        , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DPC_Mode },
  { &hf_rrc_serving_HSDSCH_CellInformation_03, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_Serving_HSDSCH_CellInformation_r9 },
  { &hf_rrc_e_dch_ReconfigurationInfo_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_E_DCH_ReconfigurationInfo_r7 },
  { &hf_rrc_ul_16QAM_Config , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_16QAM_Config },
  { &hf_rrc_e_dch_ReconfInfoSameCell, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_E_DCH_RL_InfoSameServingCell },
  { &hf_rrc_e_TFC_Boost_Info, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_E_TFC_Boost_Info_r7 },
  { &hf_rrc_e_DPDCH_PowerInterpolation, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_E_DPDCH_PowerInterpolation },
  { &hf_rrc_dl_SecondaryCellInfoFDD_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_SecondaryCellInfoFDD_r9 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_ActiveSetUpdate_r9_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_ActiveSetUpdate_r9_IEs, ActiveSetUpdate_r9_IEs_sequence);

  return offset;
}


static const value_string rrc_T_precodingWeightSetRestriction_02_vals[] = {
  {   0, "true" },
  { 0, NULL }
};


static int
dissect_rrc_T_precodingWeightSetRestriction_02(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t SecondaryCellMIMOparametersFDD_v950ext_sequence[] = {
  { &hf_rrc_precodingWeightSetRestriction_03, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_precodingWeightSetRestriction_02 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_SecondaryCellMIMOparametersFDD_v950ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_SecondaryCellMIMOparametersFDD_v950ext, SecondaryCellMIMOparametersFDD_v950ext_sequence);

  return offset;
}



static int
dissect_rrc_TargetCellPreconfigInfo_v950ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_rrc_SecondaryCellMIMOparametersFDD_v950ext(tvb, offset, actx, tree, hf_index);

  return offset;
}


static const per_sequence_t RL_AdditionInformation_v950ext_sequence[] = {
  { &hf_rrc_targetCellPreconfigInfo_06, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_TargetCellPreconfigInfo_v950ext },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_RL_AdditionInformation_v950ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_RL_AdditionInformation_v950ext, RL_AdditionInformation_v950ext_sequence);

  return offset;
}


static const per_sequence_t RL_AdditionInformationList_v950ext_sequence_of[1] = {
  { &hf_rrc_RL_AdditionInformationList_v950ext_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RL_AdditionInformation_v950ext },
};

static int
dissect_rrc_RL_AdditionInformationList_v950ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_RL_AdditionInformationList_v950ext, RL_AdditionInformationList_v950ext_sequence_of,
                                                  1, maxRL_1, FALSE);

  return offset;
}


static const per_sequence_t ActiveSetUpdate_v950ext_IEs_sequence[] = {
  { &hf_rrc_secondaryCellMimoParameters, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_SecondaryCellMIMOparametersFDD_v950ext },
  { &hf_rrc_rl_AdditionInformationList_06, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RL_AdditionInformationList_v950ext },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_ActiveSetUpdate_v950ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_ActiveSetUpdate_v950ext_IEs, ActiveSetUpdate_v950ext_IEs_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions_04_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_nonCriticalExtensions_04(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_nonCriticalExtensions_04, T_nonCriticalExtensions_04_sequence);

  return offset;
}


static const per_sequence_t T_v950NonCriticalExtensions_sequence[] = {
  { &hf_rrc_activeSetUpdate_v950ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_ActiveSetUpdate_v950ext_IEs },
  { &hf_rrc_nonCriticalExtensions_04, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_nonCriticalExtensions_04 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v950NonCriticalExtensions(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v950NonCriticalExtensions, T_v950NonCriticalExtensions_sequence);

  return offset;
}


static const per_sequence_t T_r9_sequence[] = {
  { &hf_rrc_activeSetUpdate_r9, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_ActiveSetUpdate_r9_IEs },
  { &hf_rrc_activeSetUpdate_r9_add_ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_BIT_STRING },
  { &hf_rrc_v950NonCriticalExtensions, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v950NonCriticalExtensions },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_r9(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_r9, T_r9_sequence);

  return offset;
}


static const value_string rrc_T_dl_dpchInfo_08_vals[] = {
  {   0, "dl-DPCH-InfoPerRL" },
  {   1, "dl-FDPCH-InfoPerRL" },
  { 0, NULL }
};

static const per_choice_t T_dl_dpchInfo_08_choice[] = {
  {   0, &hf_rrc_dl_DPCH_InfoPerRL_02, ASN1_NO_EXTENSIONS     , dissect_rrc_DL_DPCH_InfoPerRL_r5 },
  {   1, &hf_rrc_dl_FDPCH_InfoPerRL_01, ASN1_NO_EXTENSIONS     , dissect_rrc_DL_FDPCH_InfoPerRL_r7 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_dl_dpchInfo_08(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_dl_dpchInfo_08, T_dl_dpchInfo_08_choice,
                                 NULL);

  return offset;
}


static const value_string rrc_T_dl_64QAM_Configured_06_vals[] = {
  {   0, "true" },
  { 0, NULL }
};


static int
dissect_rrc_T_dl_64QAM_Configured_06(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}


static const value_string rrc_T_differentTxModeFromServingHS_DSCHCell_02_vals[] = {
  {   0, "different" },
  { 0, NULL }
};


static int
dissect_rrc_T_differentTxModeFromServingHS_DSCHCell_02(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t T_newConfiguration_04_sequence[] = {
  { &hf_rrc_new_H_RNTI      , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_H_RNTI },
  { &hf_rrc_dl_64QAM_Configured_06, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_dl_64QAM_Configured_06 },
  { &hf_rrc_hs_DSCH_TBSizeTable, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_HS_DSCH_TBSizeTable },
  { &hf_rrc_primaryCPICH_Info, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PrimaryCPICH_Info },
  { &hf_rrc_dl_ScramblingCode, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_SecondaryScramblingCode },
  { &hf_rrc_hS_SCCHChannelisationCodeInfo, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SEQUENCE_SIZE_1_maxHSSCCHs_OF_HS_SCCH_Codes },
  { &hf_rrc_measurementPowerOffset, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MeasurementPowerOffset },
  { &hf_rrc_uarfcn_DL       , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UARFCN },
  { &hf_rrc_differentTxModeFromServingHS_DSCHCell_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_differentTxModeFromServingHS_DSCHCell_02 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_newConfiguration_04(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_newConfiguration_04, T_newConfiguration_04_sequence);

  return offset;
}


static const value_string rrc_T_configurationInfo_02_vals[] = {
  {   0, "continue" },
  {   1, "newConfiguration" },
  { 0, NULL }
};

static const per_choice_t T_configurationInfo_02_choice[] = {
  {   0, &hf_rrc_continue        , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   1, &hf_rrc_newConfiguration_04, ASN1_NO_EXTENSIONS     , dissect_rrc_T_newConfiguration_04 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_configurationInfo_02(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_configurationInfo_02, T_configurationInfo_02_choice,
                                 NULL);

  return offset;
}


static const value_string rrc_T_precodingWeightSetRestriction_03_vals[] = {
  {   0, "true" },
  { 0, NULL }
};


static int
dissect_rrc_T_precodingWeightSetRestriction_03(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t T_newConfiguration_09_sequence[] = {
  { &hf_rrc_mimoN_M_Ratio   , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MIMO_N_M_Ratio },
  { &hf_rrc_mimoPilotConfiguration_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MIMO_PilotConfiguration_r9 },
  { &hf_rrc_precodingWeightSetRestriction_04, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_precodingWeightSetRestriction_03 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_newConfiguration_09(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_newConfiguration_09, T_newConfiguration_09_sequence);

  return offset;
}


static const value_string rrc_SecondaryCellMIMOparametersFDD_r10_vals[] = {
  {   0, "continue" },
  {   1, "newConfiguration" },
  { 0, NULL }
};

static const per_choice_t SecondaryCellMIMOparametersFDD_r10_choice[] = {
  {   0, &hf_rrc_continue        , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   1, &hf_rrc_newConfiguration_09, ASN1_NO_EXTENSIONS     , dissect_rrc_T_newConfiguration_09 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_SecondaryCellMIMOparametersFDD_r10(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_SecondaryCellMIMOparametersFDD_r10, SecondaryCellMIMOparametersFDD_r10_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t DL_SecondaryCellInfoFDD_r10_sequence[] = {
  { &hf_rrc_configurationInfo_02, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_configurationInfo_02 },
  { &hf_rrc_secondaryCellMIMOparameters_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_SecondaryCellMIMOparametersFDD_r10 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_DL_SecondaryCellInfoFDD_r10(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_DL_SecondaryCellInfoFDD_r10, DL_SecondaryCellInfoFDD_r10_sequence);

  return offset;
}


static const per_sequence_t AdditionalDLSecCellInfoFDD_sequence[] = {
  { &hf_rrc_dl_SecondaryCellInfoFDD_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_SecondaryCellInfoFDD_r10 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_AdditionalDLSecCellInfoFDD(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_AdditionalDLSecCellInfoFDD, AdditionalDLSecCellInfoFDD_sequence);

  return offset;
}


static const per_sequence_t AdditionalDLSecCellInfoListFDD_sequence_of[1] = {
  { &hf_rrc_AdditionalDLSecCellInfoListFDD_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_AdditionalDLSecCellInfoFDD },
};

static int
dissect_rrc_AdditionalDLSecCellInfoListFDD(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_AdditionalDLSecCellInfoListFDD, AdditionalDLSecCellInfoListFDD_sequence_of,
                                                  2, 2, FALSE);

  return offset;
}


static const per_sequence_t TargetCellPreconfigInfo_r10_sequence[] = {
  { &hf_rrc_activationTimeOffset, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_ActivationTimeOffset },
  { &hf_rrc_new_H_RNTI      , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_H_RNTI },
  { &hf_rrc_newPrimary_E_RNTI, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_E_RNTI },
  { &hf_rrc_newSecondary_E_RNTI, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_E_RNTI },
  { &hf_rrc_serving_HSDSCH_CellInformation_03, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_Serving_HSDSCH_CellInformation_r9 },
  { &hf_rrc_e_dch_ReconfigurationInfo_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_E_DCH_ReconfigurationInfo_r7 },
  { &hf_rrc_dtx_drx_TimingInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DTX_DRX_TimingInfo_r7 },
  { &hf_rrc_dtx_drx_Info    , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DTX_DRX_Info_r7 },
  { &hf_rrc_hs_scch_LessInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_HS_SCCH_LessInfo_r7 },
  { &hf_rrc_mimoParameters_04, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MIMO_Parameters_r9 },
  { &hf_rrc_dl_SecondaryCellInfoFDD_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_SecondaryCellInfoFDD_r10 },
  { &hf_rrc_additionalDLSecCellInfoListFDD, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_AdditionalDLSecCellInfoListFDD },
  { &hf_rrc_ul_SecondaryCellInfoFDD, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_SecondaryCellInfoFDD },
  { &hf_rrc_e_dch_ReconfigurationInfo_SecULFrequency, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_E_DCH_ReconfigurationInfo_SecULFrequency },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_TargetCellPreconfigInfo_r10(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_TargetCellPreconfigInfo_r10, TargetCellPreconfigInfo_r10_sequence);

  return offset;
}


static const per_sequence_t RL_AdditionInformation_r10_sequence[] = {
  { &hf_rrc_primaryCPICH_Info, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PrimaryCPICH_Info },
  { &hf_rrc_cell_Id         , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CellIdentity },
  { &hf_rrc_dl_dpchInfo_08  , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_dl_dpchInfo_08 },
  { &hf_rrc_e_HICH_Information, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_E_HICH_Information },
  { &hf_rrc_e_RGCH_Information, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_E_RGCH_Information },
  { &hf_rrc_targetCellPreconfigInfo_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_TargetCellPreconfigInfo_r10 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_RL_AdditionInformation_r10(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_RL_AdditionInformation_r10, RL_AdditionInformation_r10_sequence);

  return offset;
}


static const per_sequence_t RL_AdditionInformationList_r10_sequence_of[1] = {
  { &hf_rrc_RL_AdditionInformationList_r10_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RL_AdditionInformation_r10 },
};

static int
dissect_rrc_RL_AdditionInformationList_r10(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_RL_AdditionInformationList_r10, RL_AdditionInformationList_r10_sequence_of,
                                                  1, maxRL_1, FALSE);

  return offset;
}


static const per_sequence_t ActiveSetUpdate_r10_IEs_sequence[] = {
  { &hf_rrc_activationTime  , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_ActivationTime },
  { &hf_rrc_newU_RNTI       , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_U_RNTI },
  { &hf_rrc_newH_RNTI       , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_H_RNTI },
  { &hf_rrc_newPrimary_E_RNTI, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_E_RNTI },
  { &hf_rrc_newSecondary_E_RNTI, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_E_RNTI },
  { &hf_rrc_cn_InformationInfo_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CN_InformationInfo_r6 },
  { &hf_rrc_dtx_drx_TimingInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DTX_DRX_TimingInfo_r7 },
  { &hf_rrc_dtx_drx_Info    , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DTX_DRX_Info_r7 },
  { &hf_rrc_hs_scch_LessInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_HS_SCCH_LessInfo_r7 },
  { &hf_rrc_mimoParameters_04, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MIMO_Parameters_r9 },
  { &hf_rrc_maxAllowedUL_TX_Power, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MaxAllowedUL_TX_Power },
  { &hf_rrc_ul_SecondaryCellInfoFDD, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_SecondaryCellInfoFDD },
  { &hf_rrc_e_dch_ReconfigurationInfo_SecULFrequency, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_E_DCH_ReconfigurationInfo_SecULFrequency },
  { &hf_rrc_rl_AdditionInformationList_07, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RL_AdditionInformationList_r10 },
  { &hf_rrc_rl_AdditionInformationList_SecULFreq, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RL_AdditionInformationList_SecULFreq },
  { &hf_rrc_servingCellChangeParameters, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_ServingCellChangeParameters },
  { &hf_rrc_rl_RemovalInformationList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RL_RemovalInformationList },
  { &hf_rrc_rl_RemovalInformationList_SecULFreq, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RL_RemovalInformationList_SecULFreq },
  { &hf_rrc_tx_DiversityMode, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_TX_DiversityMode },
  { &hf_rrc_dpc_Mode        , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DPC_Mode },
  { &hf_rrc_serving_HSDSCH_CellInformation_03, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_Serving_HSDSCH_CellInformation_r9 },
  { &hf_rrc_e_dch_ReconfigurationInfo_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_E_DCH_ReconfigurationInfo_r7 },
  { &hf_rrc_ul_16QAM_Config , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_16QAM_Config },
  { &hf_rrc_e_dch_ReconfInfoSameCell, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_E_DCH_RL_InfoSameServingCell },
  { &hf_rrc_e_TFC_Boost_Info, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_E_TFC_Boost_Info_r7 },
  { &hf_rrc_e_DPDCH_PowerInterpolation, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_E_DPDCH_PowerInterpolation },
  { &hf_rrc_dl_SecondaryCellInfoFDD_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_SecondaryCellInfoFDD_r10 },
  { &hf_rrc_additionalDLSecCellInfoListFDD, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_AdditionalDLSecCellInfoListFDD },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_ActiveSetUpdate_r10_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_ActiveSetUpdate_r10_IEs, ActiveSetUpdate_r10_IEs_sequence);

  return offset;
}


static const value_string rrc_T_ulOLTDActivation_vals[] = {
  {   0, "true" },
  { 0, NULL }
};


static int
dissect_rrc_T_ulOLTDActivation(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t UL_OLTD_InfoFDD_sequence[] = {
  { &hf_rrc_ulOLTDActivation, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_ulOLTDActivation },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UL_OLTD_InfoFDD(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UL_OLTD_InfoFDD, UL_OLTD_InfoFDD_sequence);

  return offset;
}


static const per_sequence_t TargetCellPreconfigInfo_vbxyext_sequence[] = {
  { &hf_rrc_ulOLTDInfoFDD   , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_OLTD_InfoFDD },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_TargetCellPreconfigInfo_vbxyext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_TargetCellPreconfigInfo_vbxyext, TargetCellPreconfigInfo_vbxyext_sequence);

  return offset;
}


static const per_sequence_t RL_AdditionInformation_vbxyext_sequence[] = {
  { &hf_rrc_targetCellPreconfigInfo_03, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_TargetCellPreconfigInfo_vbxyext },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_RL_AdditionInformation_vbxyext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_RL_AdditionInformation_vbxyext, RL_AdditionInformation_vbxyext_sequence);

  return offset;
}


static const per_sequence_t RL_AdditionInformationList_vbxyext_sequence_of[1] = {
  { &hf_rrc_RL_AdditionInformationList_vbxyext_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RL_AdditionInformation_vbxyext },
};

static int
dissect_rrc_RL_AdditionInformationList_vbxyext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_RL_AdditionInformationList_vbxyext, RL_AdditionInformationList_vbxyext_sequence_of,
                                                  1, maxRL_1, FALSE);

  return offset;
}


static const per_sequence_t ActiveSetUpdate_vbxyext_IEs_sequence[] = {
  { &hf_rrc_ulOLTDInfoFDD   , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_OLTD_InfoFDD },
  { &hf_rrc_rl_AdditionInformationList_08, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RL_AdditionInformationList_vbxyext },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_ActiveSetUpdate_vbxyext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_ActiveSetUpdate_vbxyext_IEs, ActiveSetUpdate_vbxyext_IEs_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions_05_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_nonCriticalExtensions_05(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_nonCriticalExtensions_05, T_nonCriticalExtensions_05_sequence);

  return offset;
}


static const per_sequence_t T_vbxyNonCriticalExtensions_sequence[] = {
  { &hf_rrc_activeSetUpdate_vbxyext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_ActiveSetUpdate_vbxyext_IEs },
  { &hf_rrc_nonCriticalExtensions_05, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_nonCriticalExtensions_05 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_vbxyNonCriticalExtensions(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_vbxyNonCriticalExtensions, T_vbxyNonCriticalExtensions_sequence);

  return offset;
}


static const per_sequence_t T_r10_sequence[] = {
  { &hf_rrc_activeSetUpdate_r10, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_ActiveSetUpdate_r10_IEs },
  { &hf_rrc_activeSetUpdate_r10_add_ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_BIT_STRING },
  { &hf_rrc_vbxyNonCriticalExtensions, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_vbxyNonCriticalExtensions },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_r10(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_r10, T_r10_sequence);

  return offset;
}


static const per_sequence_t Antenna3_S_CPICH_sequence[] = {
  { &hf_rrc_channelisationCode, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_ChannelisationCode256 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_Antenna3_S_CPICH(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_Antenna3_S_CPICH, Antenna3_S_CPICH_sequence);

  return offset;
}


static const per_sequence_t Antenna4_S_CPICH_sequence[] = {
  { &hf_rrc_channelisationCode, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_ChannelisationCode256 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_Antenna4_S_CPICH(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_Antenna4_S_CPICH, Antenna4_S_CPICH_sequence);

  return offset;
}



static int
dissect_rrc_S_CPICH_PowerOffset_4x4MIMO(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            -12, 0U, NULL, FALSE);

  return offset;
}


static const per_sequence_t Antenna3And4_S_CPICH_sequence[] = {
  { &hf_rrc_antenna3_S_CPICH, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_Antenna3_S_CPICH },
  { &hf_rrc_antenna4_S_CPICH, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_Antenna4_S_CPICH },
  { &hf_rrc_s_cpich_PowerOffset_Mimo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_S_CPICH_PowerOffset_4x4MIMO },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_Antenna3And4_S_CPICH(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_Antenna3And4_S_CPICH, Antenna3And4_S_CPICH_sequence);

  return offset;
}


static const per_sequence_t Antenna3_D_CPICH_sequence[] = {
  { &hf_rrc_channelisationCode, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_ChannelisationCode256 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_Antenna3_D_CPICH(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_Antenna3_D_CPICH, Antenna3_D_CPICH_sequence);

  return offset;
}


static const per_sequence_t Antenna4_D_CPICH_sequence[] = {
  { &hf_rrc_channelisationCode, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_ChannelisationCode256 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_Antenna4_D_CPICH(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_Antenna4_D_CPICH, Antenna4_D_CPICH_sequence);

  return offset;
}



static int
dissect_rrc_D_CPICH_PowerOffset_4x4MIMO(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            -12, 0U, NULL, FALSE);

  return offset;
}


static const value_string rrc_T_initialD_CPICHStatus_vals[] = {
  {   0, "activated" },
  { 0, NULL }
};


static int
dissect_rrc_T_initialD_CPICHStatus(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t Antenna3And4_D_CPICH_sequence[] = {
  { &hf_rrc_antenna3_D_CPICH, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_Antenna3_D_CPICH },
  { &hf_rrc_antenna4_D_CPICH, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_Antenna4_D_CPICH },
  { &hf_rrc_d_cpich_PowerOffset_Mimo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_D_CPICH_PowerOffset_4x4MIMO },
  { &hf_rrc_initialD_CPICHStatus, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_initialD_CPICHStatus },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_Antenna3And4_D_CPICH(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_Antenna3And4_D_CPICH, Antenna3And4_D_CPICH_sequence);

  return offset;
}


static const per_sequence_t Antenna3And4_sequence[] = {
  { &hf_rrc_antenna3And4_S_CPICH, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_Antenna3And4_S_CPICH },
  { &hf_rrc_antenna3And4_D_CPICH, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_Antenna3And4_D_CPICH },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_Antenna3And4(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_Antenna3And4, Antenna3And4_sequence);

  return offset;
}


static const per_sequence_t MIMO4x4_PilotConfiguration_sequence[] = {
  { &hf_rrc_antenna2        , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MIMO_PilotConfiguration_r9 },
  { &hf_rrc_antenna3And4    , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_Antenna3And4 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_MIMO4x4_PilotConfiguration(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_MIMO4x4_PilotConfiguration, MIMO4x4_PilotConfiguration_sequence);

  return offset;
}



static int
dissect_rrc_BIT_STRING_SIZE_64(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_bit_string(tvb, offset, actx, tree, hf_index,
                                     64, 64, FALSE, NULL);

  return offset;
}


static const per_sequence_t T_newConfiguration_06_sequence[] = {
  { &hf_rrc_mimo4x4N_M_Ratio, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MIMO_N_M_Ratio },
  { &hf_rrc_mimo4x4PilotConfiguration, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MIMO4x4_PilotConfiguration },
  { &hf_rrc_precodingWeightSetRestriction, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_BIT_STRING_SIZE_64 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_newConfiguration_06(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_newConfiguration_06, T_newConfiguration_06_sequence);

  return offset;
}


static const value_string rrc_MIMO4x4_Parameters_vals[] = {
  {   0, "continue" },
  {   1, "newConfiguration" },
  { 0, NULL }
};

static const per_choice_t MIMO4x4_Parameters_choice[] = {
  {   0, &hf_rrc_continue        , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   1, &hf_rrc_newConfiguration_06, ASN1_NO_EXTENSIONS     , dissect_rrc_T_newConfiguration_06 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_MIMO4x4_Parameters(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_MIMO4x4_Parameters, MIMO4x4_Parameters_choice,
                                 NULL);

  return offset;
}



static int
dissect_rrc_S_DPCCH_DPCCH_PowerOffset(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            0U, 6U, NULL, FALSE);

  return offset;
}


static const per_sequence_t S_DPCCH_Information_sequence[] = {
  { &hf_rrc_s_DPCCH_DPCCH_PowerOffset, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_S_DPCCH_DPCCH_PowerOffset },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_S_DPCCH_Information(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_S_DPCCH_Information, S_DPCCH_Information_sequence);

  return offset;
}


static const value_string rrc_T_initial_CLTD_ActivationState_vals[] = {
  {   0, "first-state" },
  {   1, "second-state" },
  { 0, NULL }
};


static int
dissect_rrc_T_initial_CLTD_ActivationState(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     2, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t T_newConfiguration_11_sequence[] = {
  { &hf_rrc_s_dpcch_Info    , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_S_DPCCH_Information },
  { &hf_rrc_initial_CLTD_ActivationState, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_initial_CLTD_ActivationState },
  { &hf_rrc_primaryCPICH_Info, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_PrimaryCPICH_Info },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_newConfiguration_11(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_newConfiguration_11, T_newConfiguration_11_sequence);

  return offset;
}


static const value_string rrc_T_configurationInfo_04_vals[] = {
  {   0, "continue" },
  {   1, "newConfiguration" },
  { 0, NULL }
};

static const per_choice_t T_configurationInfo_04_choice[] = {
  {   0, &hf_rrc_continue        , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   1, &hf_rrc_newConfiguration_11, ASN1_NO_EXTENSIONS     , dissect_rrc_T_newConfiguration_11 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_configurationInfo_04(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_configurationInfo_04, T_configurationInfo_04_choice,
                                 NULL);

  return offset;
}


static const value_string rrc_UL_CLTD_InfoFDD_vals[] = {
  {   0, "configurationInfo" },
  { 0, NULL }
};

static const per_choice_t UL_CLTD_InfoFDD_choice[] = {
  {   0, &hf_rrc_configurationInfo_04, ASN1_NO_EXTENSIONS     , dissect_rrc_T_configurationInfo_04 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_UL_CLTD_InfoFDD(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_UL_CLTD_InfoFDD, UL_CLTD_InfoFDD_choice,
                                 NULL);

  return offset;
}



static int
dissect_rrc_FTPICH_SlotFormat(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            0U, 9U, NULL, FALSE);

  return offset;
}



static int
dissect_rrc_FTPICH_FrameOffset(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            0U, 149U, NULL, FALSE);

  return offset;
}


static const per_sequence_t F_TPICH_Information_sequence[] = {
  { &hf_rrc_ftpich_SlotFormat, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_FTPICH_SlotFormat },
  { &hf_rrc_ftpich_CodeNumber, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_0_255 },
  { &hf_rrc_ftpich_FrameOffset, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_FTPICH_FrameOffset },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_F_TPICH_Information(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_F_TPICH_Information, F_TPICH_Information_sequence);

  return offset;
}


static const per_sequence_t F_TPICH_InfoNewServingCell_sequence[] = {
  { &hf_rrc_f_tpich_Information, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_F_TPICH_Information },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_F_TPICH_InfoNewServingCell(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_F_TPICH_InfoNewServingCell, F_TPICH_InfoNewServingCell_sequence);

  return offset;
}


static const value_string rrc_T_f_tpich_Info_vals[] = {
  {   0, "f-tpich-Information" },
  {   1, "releaseIndicator" },
  { 0, NULL }
};

static const per_choice_t T_f_tpich_Info_choice[] = {
  {   0, &hf_rrc_f_tpich_Information, ASN1_NO_EXTENSIONS     , dissect_rrc_F_TPICH_Information },
  {   1, &hf_rrc_releaseIndicator, ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_f_tpich_Info(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_f_tpich_Info, T_f_tpich_Info_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t F_TPICH_InfoOtherCell_sequence[] = {
  { &hf_rrc_primaryCPICH_Info, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PrimaryCPICH_Info },
  { &hf_rrc_f_tpich_Info    , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_f_tpich_Info },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_F_TPICH_InfoOtherCell(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_F_TPICH_InfoOtherCell, F_TPICH_InfoOtherCell_sequence);

  return offset;
}


static const per_sequence_t SEQUENCE_SIZE_1_maxRL_OF_F_TPICH_InfoOtherCell_sequence_of[1] = {
  { &hf_rrc_f_tpich_InfoOtherCellList_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_F_TPICH_InfoOtherCell },
};

static int
dissect_rrc_SEQUENCE_SIZE_1_maxRL_OF_F_TPICH_InfoOtherCell(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_SEQUENCE_SIZE_1_maxRL_OF_F_TPICH_InfoOtherCell, SEQUENCE_SIZE_1_maxRL_OF_F_TPICH_InfoOtherCell_sequence_of,
                                                  1, maxRL, FALSE);

  return offset;
}


static const per_sequence_t F_TPICH_ReconfigurationInfo_sequence[] = {
  { &hf_rrc_f_tpich_InfoNewServingCell, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_F_TPICH_InfoNewServingCell },
  { &hf_rrc_f_tpich_InfoOtherCellList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_SEQUENCE_SIZE_1_maxRL_OF_F_TPICH_InfoOtherCell },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_F_TPICH_ReconfigurationInfo(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_F_TPICH_ReconfigurationInfo, F_TPICH_ReconfigurationInfo_sequence);

  return offset;
}


static const value_string rrc_T_dl_dpchInfo_09_vals[] = {
  {   0, "dl-DPCH-InfoPerRL" },
  {   1, "dl-FDPCH-InfoPerRL" },
  { 0, NULL }
};

static const per_choice_t T_dl_dpchInfo_09_choice[] = {
  {   0, &hf_rrc_dl_DPCH_InfoPerRL_02, ASN1_NO_EXTENSIONS     , dissect_rrc_DL_DPCH_InfoPerRL_r5 },
  {   1, &hf_rrc_dl_FDPCH_InfoPerRL_01, ASN1_NO_EXTENSIONS     , dissect_rrc_DL_FDPCH_InfoPerRL_r7 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_dl_dpchInfo_09(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_dl_dpchInfo_09, T_dl_dpchInfo_09_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t E_HICH_Information_r11_sequence[] = {
  { &hf_rrc_channelisationCode_02, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_E_HICH_ChannelisationCode },
  { &hf_rrc_signatureSequence_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_E_HICH_RGCH_SignatureSequence },
  { &hf_rrc_secondarySignatureSequence, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_E_HICH_RGCH_SignatureSequence },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_E_HICH_Information_r11(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_E_HICH_Information_r11, E_HICH_Information_r11_sequence);

  return offset;
}



static int
dissect_rrc_DeltaACK_r11(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            0U, 10U, NULL, FALSE);

  return offset;
}



static int
dissect_rrc_DeltaNACK_r11(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            0U, 10U, NULL, FALSE);

  return offset;
}



static int
dissect_rrc_DeltaCQI_r11(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            0U, 10U, NULL, FALSE);

  return offset;
}


static const per_sequence_t T_fdd_99_sequence[] = {
  { &hf_rrc_measurementPowerOffset, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MeasurementPowerOffset },
  { &hf_rrc_feedback_cycle_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_Feedback_cycle_r7 },
  { &hf_rrc_cqi_RepetitionFactor, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CQI_RepetitionFactor },
  { &hf_rrc_cqi_RepetitionFactor_Assisting, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CQI_RepetitionFactor },
  { &hf_rrc_deltaCQI_01     , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DeltaCQI_r11 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_fdd_99(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_fdd_99, T_fdd_99_sequence);

  return offset;
}


static const value_string rrc_T_modeSpecificInfo_69_vals[] = {
  {   0, "fdd" },
  {   1, "tdd" },
  { 0, NULL }
};

static const per_choice_t T_modeSpecificInfo_69_choice[] = {
  {   0, &hf_rrc_fdd_101         , ASN1_NO_EXTENSIONS     , dissect_rrc_T_fdd_99 },
  {   1, &hf_rrc_tdd             , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_modeSpecificInfo_69(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_modeSpecificInfo_69, T_modeSpecificInfo_69_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t Measurement_Feedback_Info_r11_sequence[] = {
  { &hf_rrc_modeSpecificInfo_69, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_modeSpecificInfo_69 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_Measurement_Feedback_Info_r11(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_Measurement_Feedback_Info_r11, Measurement_Feedback_Info_r11_sequence);

  return offset;
}


static const per_sequence_t T_tdd384_18_sequence[] = {
  { &hf_rrc_dl_HSPDSCH_TS_Configuration, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_HSPDSCH_TS_Configuration },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_tdd384_18(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_tdd384_18, T_tdd384_18_sequence);

  return offset;
}


static const per_sequence_t T_tdd768_10_sequence[] = {
  { &hf_rrc_dl_HSPDSCH_TS_Configuration_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_HSPDSCH_TS_Configuration_VHCR },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_tdd768_10(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_tdd768_10, T_tdd768_10_sequence);

  return offset;
}


static const value_string rrc_T_tS0_Indicator_01_vals[] = {
  {   0, "true" },
  { 0, NULL }
};


static int
dissect_rrc_T_tS0_Indicator_01(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t T_tdd128_27_sequence[] = {
  { &hf_rrc_hs_PDSCH_Midamble_Configuration_tdd128, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_HS_PDSCH_Midamble_Configuration_TDD128 },
  { &hf_rrc_dl_MultiCarrier_Information, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_MultiCarrier_Information },
  { &hf_rrc_tS0_Indicator_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_tS0_Indicator_01 },
  { &hf_rrc_outofSyncWindow , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_OutofSyncWindow },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_tdd128_27(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_tdd128_27, T_tdd128_27_sequence);

  return offset;
}


static const value_string rrc_T_tdd_44_vals[] = {
  {   0, "tdd384" },
  {   1, "tdd768" },
  {   2, "tdd128" },
  { 0, NULL }
};

static const per_choice_t T_tdd_44_choice[] = {
  {   0, &hf_rrc_tdd384_19       , ASN1_NO_EXTENSIONS     , dissect_rrc_T_tdd384_18 },
  {   1, &hf_rrc_tdd768_11       , ASN1_NO_EXTENSIONS     , dissect_rrc_T_tdd768_10 },
  {   2, &hf_rrc_tdd128_27       , ASN1_NO_EXTENSIONS     , dissect_rrc_T_tdd128_27 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_tdd_44(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_tdd_44, T_tdd_44_choice,
                                 NULL);

  return offset;
}


static const value_string rrc_T_dl_64QAM_Configured_03_vals[] = {
  {   0, "true" },
  { 0, NULL }
};


static int
dissect_rrc_T_dl_64QAM_Configured_03(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t T_fdd_79_sequence[] = {
  { &hf_rrc_dl_64QAM_Configured_03, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_dl_64QAM_Configured_03 },
  { &hf_rrc_hs_DSCH_TBSizeTable, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_HS_DSCH_TBSizeTable },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_fdd_79(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_fdd_79, T_fdd_79_sequence);

  return offset;
}


static const value_string rrc_T_modeSpecificInfo_47_vals[] = {
  {   0, "tdd" },
  {   1, "fdd" },
  { 0, NULL }
};

static const per_choice_t T_modeSpecificInfo_47_choice[] = {
  {   0, &hf_rrc_tdd_45          , ASN1_NO_EXTENSIONS     , dissect_rrc_T_tdd_44 },
  {   1, &hf_rrc_fdd_80          , ASN1_NO_EXTENSIONS     , dissect_rrc_T_fdd_79 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_modeSpecificInfo_47(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_modeSpecificInfo_47, T_modeSpecificInfo_47_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t DL_HSPDSCH_Information_r11_sequence[] = {
  { &hf_rrc_hs_scch_Info_04 , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_HS_SCCH_Info_r9 },
  { &hf_rrc_measurement_feedback_Info_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_Measurement_Feedback_Info_r11 },
  { &hf_rrc_modeSpecificInfo_47, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_modeSpecificInfo_47 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_DL_HSPDSCH_Information_r11(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_DL_HSPDSCH_Information_r11, DL_HSPDSCH_Information_r11_sequence);

  return offset;
}


static const value_string rrc_T_numberOfProcesses_01_vals[] = {
  {   0, "n1" },
  {   1, "n2" },
  {   2, "n3" },
  {   3, "n4" },
  {   4, "n5" },
  {   5, "n6" },
  {   6, "n7" },
  {   7, "n8" },
  {   8, "n12" },
  {   9, "n14" },
  {  10, "n16" },
  {  11, "n24" },
  {  12, "n28" },
  {  13, "n32" },
  { 0, NULL }
};


static int
dissect_rrc_T_numberOfProcesses_01(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     14, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t T_explicit_01_sequence[] = {
  { &hf_rrc_memorySize      , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SEQUENCE_SIZE_1_maxHProcesses_OF_HARQMemorySize },
  { &hf_rrc_additionalMemorySizesForMIMO, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_SEQUENCE_SIZE_1_maxHProcesses_OF_HARQMemorySize },
  { &hf_rrc_additionalMemorySizesThirdMIMOStream, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_SEQUENCE_SIZE_1_maxHProcesses_OF_HARQMemorySize },
  { &hf_rrc_additionalMemorySizesFourthMIMOStream, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_SEQUENCE_SIZE_1_maxHProcesses_OF_HARQMemorySize },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_explicit_01(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_explicit_01, T_explicit_01_sequence);

  return offset;
}


static const value_string rrc_T_memoryPartitioning_02_vals[] = {
  {   0, "implicit" },
  {   1, "explicit" },
  { 0, NULL }
};

static const per_choice_t T_memoryPartitioning_02_choice[] = {
  {   0, &hf_rrc_implicit        , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   1, &hf_rrc_explicit_02     , ASN1_NO_EXTENSIONS     , dissect_rrc_T_explicit_01 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_memoryPartitioning_02(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_memoryPartitioning_02, T_memoryPartitioning_02_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t HARQ_Info_r11_sequence[] = {
  { &hf_rrc_numberOfProcesses_02, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_numberOfProcesses_01 },
  { &hf_rrc_memoryPartitioning_02, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_memoryPartitioning_02 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_HARQ_Info_r11(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_HARQ_Info_r11, HARQ_Info_r11_sequence);

  return offset;
}


static const value_string rrc_T_mac_hsResetIndicator_04_vals[] = {
  {   0, "true" },
  { 0, NULL }
};


static int
dissect_rrc_T_mac_hsResetIndicator_04(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}


static const value_string rrc_T_mac_hsResetIndicator_assisting_vals[] = {
  {   0, "true" },
  { 0, NULL }
};


static int
dissect_rrc_T_mac_hsResetIndicator_assisting(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t Serving_HSDSCH_CellInformation_r11_sequence[] = {
  { &hf_rrc_deltaACK_01     , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DeltaACK_r11 },
  { &hf_rrc_deltaNACK_01    , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DeltaNACK_r11 },
  { &hf_rrc_harq_Preamble_Mode, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_HARQ_Preamble_Mode },
  { &hf_rrc_primaryCPICH_Info, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_PrimaryCPICH_Info },
  { &hf_rrc_dl_hspdsch_Information_04, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_HSPDSCH_Information_r11 },
  { &hf_rrc_harqInfo_01     , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_HARQ_Info_r11 },
  { &hf_rrc_mac_hsResetIndicator_04, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_mac_hsResetIndicator_04 },
  { &hf_rrc_mac_hsResetIndicator_assisting, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_mac_hsResetIndicator_assisting },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_Serving_HSDSCH_CellInformation_r11(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_Serving_HSDSCH_CellInformation_r11, Serving_HSDSCH_CellInformation_r11_sequence);

  return offset;
}



static int
dissect_rrc_E_ROCH_ChannelisationCode(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            0U, 255U, NULL, FALSE);

  return offset;
}


static const per_sequence_t E_ROCH_Information_sequence[] = {
  { &hf_rrc_e_ROCH_ChannelisationCode, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_E_ROCH_ChannelisationCode },
  { &hf_rrc_e_roch_e_rnti   , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_E_RNTI },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_E_ROCH_Information(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_E_ROCH_Information, E_ROCH_Information_sequence);

  return offset;
}


static const value_string rrc_T_primary_Secondary_GrantSelector_03_vals[] = {
  {   0, "primary" },
  {   1, "secondary" },
  { 0, NULL }
};


static int
dissect_rrc_T_primary_Secondary_GrantSelector_03(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     2, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t T_servingGrant_03_sequence[] = {
  { &hf_rrc_value           , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_INTEGER_0_38 },
  { &hf_rrc_primary_Secondary_GrantSelector_03, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_primary_Secondary_GrantSelector_03 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_servingGrant_03(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_servingGrant_03, T_servingGrant_03_sequence);

  return offset;
}


static const value_string rrc_T_e_RGCH_Info_07_vals[] = {
  {   0, "e-RGCH-Information" },
  {   1, "releaseIndicator" },
  { 0, NULL }
};

static const per_choice_t T_e_RGCH_Info_07_choice[] = {
  {   0, &hf_rrc_e_RGCH_Information, ASN1_NO_EXTENSIONS     , dissect_rrc_E_RGCH_Information },
  {   1, &hf_rrc_releaseIndicator, ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_e_RGCH_Info_07(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_e_RGCH_Info_07, T_e_RGCH_Info_07_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t E_DCH_RL_InfoNewServingCell_r11_sequence[] = {
  { &hf_rrc_primaryCPICH_Info, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PrimaryCPICH_Info },
  { &hf_rrc_e_AGCH_Information, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_E_AGCH_Information },
  { &hf_rrc_e_ROCH_Information, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_E_ROCH_Information },
  { &hf_rrc_servingGrant_03 , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_servingGrant_03 },
  { &hf_rrc_e_DPCCH_DPCCH_PowerOffset, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_E_DPCCH_DPCCH_PowerOffset },
  { &hf_rrc_reference_E_TFCIs_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_E_DPDCH_Reference_E_TFCIList_r7 },
  { &hf_rrc_powerOffsetForSchedInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_INTEGER_0_6 },
  { &hf_rrc_threeIndexStepThreshold, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_INTEGER_0_37 },
  { &hf_rrc_twoIndexStepThreshold, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_INTEGER_0_37 },
  { &hf_rrc_e_HICH_Information_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_E_HICH_Information_r11 },
  { &hf_rrc_e_RGCH_Info_07  , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_e_RGCH_Info_07 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_E_DCH_RL_InfoNewServingCell_r11(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_E_DCH_RL_InfoNewServingCell_r11, E_DCH_RL_InfoNewServingCell_r11_sequence);

  return offset;
}


static const value_string rrc_T_e_HICH_Info_05_vals[] = {
  {   0, "e-HICH-Information" },
  {   1, "releaseIndicator" },
  {   2, "secondaryReleaseIndicator" },
  { 0, NULL }
};

static const per_choice_t T_e_HICH_Info_05_choice[] = {
  {   0, &hf_rrc_e_HICH_Information, ASN1_NO_EXTENSIONS     , dissect_rrc_E_HICH_Information },
  {   1, &hf_rrc_releaseIndicator, ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   2, &hf_rrc_secondaryReleaseIndicator, ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_e_HICH_Info_05(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_e_HICH_Info_05, T_e_HICH_Info_05_choice,
                                 NULL);

  return offset;
}


static const value_string rrc_T_e_RGCH_Info_10_vals[] = {
  {   0, "e-RGCH-Information" },
  {   1, "releaseIndicator" },
  { 0, NULL }
};

static const per_choice_t T_e_RGCH_Info_10_choice[] = {
  {   0, &hf_rrc_e_RGCH_Information, ASN1_NO_EXTENSIONS     , dissect_rrc_E_RGCH_Information },
  {   1, &hf_rrc_releaseIndicator, ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_e_RGCH_Info_10(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_e_RGCH_Info_10, T_e_RGCH_Info_10_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t E_DCH_RL_InfoOtherCell_r11_sequence[] = {
  { &hf_rrc_primaryCPICH_Info, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PrimaryCPICH_Info },
  { &hf_rrc_e_HICH_Info_07  , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_e_HICH_Info_05 },
  { &hf_rrc_e_RGCH_Info_10  , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_e_RGCH_Info_10 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_E_DCH_RL_InfoOtherCell_r11(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_E_DCH_RL_InfoOtherCell_r11, E_DCH_RL_InfoOtherCell_r11_sequence);

  return offset;
}


static const per_sequence_t SEQUENCE_SIZE_1_maxEDCHRL_OF_E_DCH_RL_InfoOtherCell_r11_sequence_of[1] = {
  { &hf_rrc_e_DCH_RL_InfoOtherCellList_item_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_E_DCH_RL_InfoOtherCell_r11 },
};

static int
dissect_rrc_SEQUENCE_SIZE_1_maxEDCHRL_OF_E_DCH_RL_InfoOtherCell_r11(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_SEQUENCE_SIZE_1_maxEDCHRL_OF_E_DCH_RL_InfoOtherCell_r11, SEQUENCE_SIZE_1_maxEDCHRL_OF_E_DCH_RL_InfoOtherCell_r11_sequence_of,
                                                  1, maxEDCHRL, FALSE);

  return offset;
}


static const per_sequence_t E_DCH_ReconfigurationInfo_r11_sequence[] = {
  { &hf_rrc_e_DCH_RL_InfoNewServingCell_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_E_DCH_RL_InfoNewServingCell_r11 },
  { &hf_rrc_e_DCH_RL_InfoOtherCellList_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_SEQUENCE_SIZE_1_maxEDCHRL_OF_E_DCH_RL_InfoOtherCell_r11 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_E_DCH_ReconfigurationInfo_r11(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_E_DCH_ReconfigurationInfo_r11, E_DCH_ReconfigurationInfo_r11_sequence);

  return offset;
}


static const value_string rrc_T_dl_64QAM_Configured_07_vals[] = {
  {   0, "true" },
  { 0, NULL }
};


static int
dissect_rrc_T_dl_64QAM_Configured_07(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}


static const value_string rrc_T_differentTxModeFromServingHS_DSCHCell_03_vals[] = {
  {   0, "different" },
  { 0, NULL }
};


static int
dissect_rrc_T_differentTxModeFromServingHS_DSCHCell_03(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}


static const value_string rrc_MultiflowCellType_vals[] = {
  {   0, "intra-nodeB" },
  {   1, "inter-nodeB" },
  { 0, NULL }
};


static int
dissect_rrc_MultiflowCellType(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     2, NULL, FALSE, 0, NULL);

  return offset;
}


static const value_string rrc_T_timeReferenceCell_vals[] = {
  {   0, "true" },
  { 0, NULL }
};


static int
dissect_rrc_T_timeReferenceCell(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t MultiflowConfigurationInfo_sequence[] = {
  { &hf_rrc_cellType        , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MultiflowCellType },
  { &hf_rrc_timeReferenceCell, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_timeReferenceCell },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_MultiflowConfigurationInfo(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_MultiflowConfigurationInfo, MultiflowConfigurationInfo_sequence);

  return offset;
}


static const per_sequence_t T_newConfiguration_05_sequence[] = {
  { &hf_rrc_new_H_RNTI      , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_H_RNTI },
  { &hf_rrc_dl_64QAM_Configured_07, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_dl_64QAM_Configured_07 },
  { &hf_rrc_hs_DSCH_TBSizeTable, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_HS_DSCH_TBSizeTable },
  { &hf_rrc_primaryCPICH_Info, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PrimaryCPICH_Info },
  { &hf_rrc_dl_ScramblingCode, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_SecondaryScramblingCode },
  { &hf_rrc_hS_SCCHChannelisationCodeInfo, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SEQUENCE_SIZE_1_maxHSSCCHs_OF_HS_SCCH_Codes },
  { &hf_rrc_measurementPowerOffset, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MeasurementPowerOffset },
  { &hf_rrc_measurementPowerOffset1, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MeasurementPowerOffset },
  { &hf_rrc_uarfcn_DL       , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UARFCN },
  { &hf_rrc_differentTxModeFromServingHS_DSCHCell_03, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_differentTxModeFromServingHS_DSCHCell_03 },
  { &hf_rrc_multiflowConfigurationInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MultiflowConfigurationInfo },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_newConfiguration_05(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_newConfiguration_05, T_newConfiguration_05_sequence);

  return offset;
}


static const value_string rrc_T_configurationInfo_03_vals[] = {
  {   0, "continue" },
  {   1, "newConfiguration" },
  { 0, NULL }
};

static const per_choice_t T_configurationInfo_03_choice[] = {
  {   0, &hf_rrc_continue        , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   1, &hf_rrc_newConfiguration_05, ASN1_NO_EXTENSIONS     , dissect_rrc_T_newConfiguration_05 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_configurationInfo_03(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_configurationInfo_03, T_configurationInfo_03_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t T_newConfiguration_10_sequence[] = {
  { &hf_rrc_mimoN_M_Ratio   , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MIMO_N_M_Ratio },
  { &hf_rrc_mimoPilotConfiguration_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MIMO4x4_PilotConfiguration },
  { &hf_rrc_precodingWeightSetRestriction, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_BIT_STRING_SIZE_64 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_newConfiguration_10(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_newConfiguration_10, T_newConfiguration_10_sequence);

  return offset;
}


static const value_string rrc_SecondaryCell4x4MIMOparametersFDD_vals[] = {
  {   0, "continue" },
  {   1, "newConfiguration" },
  { 0, NULL }
};

static const per_choice_t SecondaryCell4x4MIMOparametersFDD_choice[] = {
  {   0, &hf_rrc_continue        , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   1, &hf_rrc_newConfiguration_10, ASN1_NO_EXTENSIONS     , dissect_rrc_T_newConfiguration_10 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_SecondaryCell4x4MIMOparametersFDD(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_SecondaryCell4x4MIMOparametersFDD, SecondaryCell4x4MIMOparametersFDD_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t DL_SecondaryCellInfoFDD_r11_sequence[] = {
  { &hf_rrc_configurationInfo_03, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_configurationInfo_03 },
  { &hf_rrc_secondaryCellMIMOparameters_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_SecondaryCellMIMOparametersFDD_r10 },
  { &hf_rrc_secondaryCell4x4MIMOparameters, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_SecondaryCell4x4MIMOparametersFDD },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_DL_SecondaryCellInfoFDD_r11(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_DL_SecondaryCellInfoFDD_r11, DL_SecondaryCellInfoFDD_r11_sequence);

  return offset;
}


static const per_sequence_t AdditionalDLSecCellInfoFDD_r11_sequence[] = {
  { &hf_rrc_dl_SecondaryCellInfoFDD_03, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_SecondaryCellInfoFDD_r11 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_AdditionalDLSecCellInfoFDD_r11(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_AdditionalDLSecCellInfoFDD_r11, AdditionalDLSecCellInfoFDD_r11_sequence);

  return offset;
}


static const per_sequence_t AdditionalDLSecCellInfoListFDD_r11_sequence_of[1] = {
  { &hf_rrc_AdditionalDLSecCellInfoListFDD_r11_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_AdditionalDLSecCellInfoFDD_r11 },
};

static int
dissect_rrc_AdditionalDLSecCellInfoListFDD_r11(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_AdditionalDLSecCellInfoListFDD_r11, AdditionalDLSecCellInfoListFDD_r11_sequence_of,
                                                  2, 2, FALSE);

  return offset;
}


static const per_sequence_t AdditionalDLSecCellInfoListFDD2_r11_sequence_of[1] = {
  { &hf_rrc_AdditionalDLSecCellInfoListFDD2_r11_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_AdditionalDLSecCellInfoFDD_r11 },
};

static int
dissect_rrc_AdditionalDLSecCellInfoListFDD2_r11(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_AdditionalDLSecCellInfoListFDD2_r11, AdditionalDLSecCellInfoListFDD2_r11_sequence_of,
                                                  4, 4, FALSE);

  return offset;
}


static const per_sequence_t UL_64QAM_Settings_sequence[] = {
  { &hf_rrc_beta_Ed_Gain_E_AGCH_Table_Selection, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_0_1 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UL_64QAM_Settings(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UL_64QAM_Settings, UL_64QAM_Settings_sequence);

  return offset;
}


static const value_string rrc_T_mac_is_i_resetIndicator_vals[] = {
  {   0, "true" },
  { 0, NULL }
};


static int
dissect_rrc_T_mac_is_i_resetIndicator(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t UL_64QAM_Config_sequence[] = {
  { &hf_rrc_ul_64QAM_Settings, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_64QAM_Settings },
  { &hf_rrc_mac_is_i_resetIndicator, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_mac_is_i_resetIndicator },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UL_64QAM_Config(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UL_64QAM_Config, UL_64QAM_Config_sequence);

  return offset;
}



static int
dissect_rrc_INTEGER_0_17(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            0U, 17U, NULL, FALSE);

  return offset;
}


static const per_sequence_t T_newConfiguration_12_sequence[] = {
  { &hf_rrc_s_e_dpcch_power_offset, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_0_17 },
  { &hf_rrc_rank2_minimum_E_TFCI, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_E_DCH_MinimumSet_E_TFCI },
  { &hf_rrc_inter_stream_compensation, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_0_15 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_newConfiguration_12(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_newConfiguration_12, T_newConfiguration_12_sequence);

  return offset;
}


static const value_string rrc_T_configurationInfo_05_vals[] = {
  {   0, "continue" },
  {   1, "newConfiguration" },
  { 0, NULL }
};

static const per_choice_t T_configurationInfo_05_choice[] = {
  {   0, &hf_rrc_continue        , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   1, &hf_rrc_newConfiguration_12, ASN1_NO_EXTENSIONS     , dissect_rrc_T_newConfiguration_12 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_configurationInfo_05(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_configurationInfo_05, T_configurationInfo_05_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t UL_MIMO_InfoFDD_sequence[] = {
  { &hf_rrc_configurationInfo_05, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_configurationInfo_05 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UL_MIMO_InfoFDD(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UL_MIMO_InfoFDD, UL_MIMO_InfoFDD_sequence);

  return offset;
}


static const per_sequence_t TargetCellPreconfigInfo_r11_sequence[] = {
  { &hf_rrc_activationTimeOffset, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_ActivationTimeOffset },
  { &hf_rrc_new_H_RNTI      , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_H_RNTI },
  { &hf_rrc_newPrimary_E_RNTI, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_E_RNTI },
  { &hf_rrc_newSecondary_E_RNTI, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_E_RNTI },
  { &hf_rrc_serving_HSDSCH_CellInformation_04, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_Serving_HSDSCH_CellInformation_r11 },
  { &hf_rrc_e_dch_ReconfigurationInfo_02, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_E_DCH_ReconfigurationInfo_r11 },
  { &hf_rrc_dtx_drx_TimingInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DTX_DRX_TimingInfo_r7 },
  { &hf_rrc_dtx_drx_Info    , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DTX_DRX_Info_r7 },
  { &hf_rrc_hs_scch_LessInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_HS_SCCH_LessInfo_r7 },
  { &hf_rrc_mimoParameters_04, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MIMO_Parameters_r9 },
  { &hf_rrc_mimo4x4Parameters, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MIMO4x4_Parameters },
  { &hf_rrc_dl_SecondaryCellInfoFDD_03, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_SecondaryCellInfoFDD_r11 },
  { &hf_rrc_additionalDLSecCellInfoListFDD_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_AdditionalDLSecCellInfoListFDD_r11 },
  { &hf_rrc_additionalDLSecCellInfoListFDD2, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_AdditionalDLSecCellInfoListFDD2_r11 },
  { &hf_rrc_ul_SecondaryCellInfoFDD, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_SecondaryCellInfoFDD },
  { &hf_rrc_e_dch_ReconfigurationInfo_SecULFrequency, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_E_DCH_ReconfigurationInfo_SecULFrequency },
  { &hf_rrc_ul_CLTD_InfoFDD , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_CLTD_InfoFDD },
  { &hf_rrc_f_tpich_ReconfigurationInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_F_TPICH_ReconfigurationInfo },
  { &hf_rrc_uL_OLTD_InfoFDD , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_OLTD_InfoFDD },
  { &hf_rrc_ul_16QAM_Config , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_16QAM_Config },
  { &hf_rrc_ul_64QAM_Config , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_64QAM_Config },
  { &hf_rrc_ul_MIMO_Info    , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_MIMO_InfoFDD },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_TargetCellPreconfigInfo_r11(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_TargetCellPreconfigInfo_r11, TargetCellPreconfigInfo_r11_sequence);

  return offset;
}


static const per_sequence_t RL_AdditionInformation_r11_sequence[] = {
  { &hf_rrc_primaryCPICH_Info, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PrimaryCPICH_Info },
  { &hf_rrc_cell_Id         , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CellIdentity },
  { &hf_rrc_dl_dpchInfo_09  , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_dl_dpchInfo_09 },
  { &hf_rrc_e_HICH_Information_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_E_HICH_Information_r11 },
  { &hf_rrc_e_RGCH_Information, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_E_RGCH_Information },
  { &hf_rrc_f_TPICH_Information, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_F_TPICH_Information },
  { &hf_rrc_targetCellPreconfigInfo_04, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_TargetCellPreconfigInfo_r11 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_RL_AdditionInformation_r11(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_RL_AdditionInformation_r11, RL_AdditionInformation_r11_sequence);

  return offset;
}


static const per_sequence_t RL_AdditionInformationList_r11_sequence_of[1] = {
  { &hf_rrc_RL_AdditionInformationList_r11_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RL_AdditionInformation_r11 },
};

static int
dissect_rrc_RL_AdditionInformationList_r11(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_RL_AdditionInformationList_r11, RL_AdditionInformationList_r11_sequence_of,
                                                  1, maxRL_1, FALSE);

  return offset;
}


static const per_sequence_t ActiveSetUpdate_r11_IEs_sequence[] = {
  { &hf_rrc_activationTime  , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_ActivationTime },
  { &hf_rrc_newU_RNTI       , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_U_RNTI },
  { &hf_rrc_newH_RNTI       , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_H_RNTI },
  { &hf_rrc_newPrimary_E_RNTI, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_E_RNTI },
  { &hf_rrc_newSecondary_E_RNTI, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_E_RNTI },
  { &hf_rrc_cn_InformationInfo_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CN_InformationInfo_r6 },
  { &hf_rrc_dtx_drx_TimingInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DTX_DRX_TimingInfo_r7 },
  { &hf_rrc_dtx_drx_Info    , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DTX_DRX_Info_r7 },
  { &hf_rrc_hs_scch_LessInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_HS_SCCH_LessInfo_r7 },
  { &hf_rrc_mimoParameters_04, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MIMO_Parameters_r9 },
  { &hf_rrc_mimo4x4Parameters, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MIMO4x4_Parameters },
  { &hf_rrc_maxAllowedUL_TX_Power, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MaxAllowedUL_TX_Power },
  { &hf_rrc_ul_SecondaryCellInfoFDD, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_SecondaryCellInfoFDD },
  { &hf_rrc_e_dch_ReconfigurationInfo_SecULFrequency, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_E_DCH_ReconfigurationInfo_SecULFrequency },
  { &hf_rrc_ul_CLTD_InfoFDD , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_CLTD_InfoFDD },
  { &hf_rrc_f_tpich_ReconfigurationInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_F_TPICH_ReconfigurationInfo },
  { &hf_rrc_ul_OLTD_InfoFDD , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_OLTD_InfoFDD },
  { &hf_rrc_rl_AdditionInformationList_09, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RL_AdditionInformationList_r11 },
  { &hf_rrc_rl_AdditionInformationList_SecULFreq, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RL_AdditionInformationList_SecULFreq },
  { &hf_rrc_servingCellChangeParameters, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_ServingCellChangeParameters },
  { &hf_rrc_rl_RemovalInformationList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RL_RemovalInformationList },
  { &hf_rrc_rl_RemovalInformationList_SecULFreq, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RL_RemovalInformationList_SecULFreq },
  { &hf_rrc_tx_DiversityMode, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_TX_DiversityMode },
  { &hf_rrc_dpc_Mode        , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DPC_Mode },
  { &hf_rrc_serving_HSDSCH_CellInformation_04, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_Serving_HSDSCH_CellInformation_r11 },
  { &hf_rrc_e_dch_ReconfigurationInfo_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_E_DCH_ReconfigurationInfo_r11 },
  { &hf_rrc_ul_16QAM_Config , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_16QAM_Config },
  { &hf_rrc_ul_64QAM_Config , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_64QAM_Config },
  { &hf_rrc_ul_MIMO_InfoFDD , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_MIMO_InfoFDD },
  { &hf_rrc_e_dch_ReconfInfoSameCell, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_E_DCH_RL_InfoSameServingCell },
  { &hf_rrc_e_TFC_Boost_Info, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_E_TFC_Boost_Info_r7 },
  { &hf_rrc_e_DPDCH_PowerInterpolation, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_E_DPDCH_PowerInterpolation },
  { &hf_rrc_dl_SecondaryCellInfoFDD_03, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_SecondaryCellInfoFDD_r11 },
  { &hf_rrc_additionalDLSecCellInfoListFDD_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_AdditionalDLSecCellInfoListFDD_r11 },
  { &hf_rrc_additionalDLSecCellInfoListFDD2, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_AdditionalDLSecCellInfoListFDD2_r11 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_ActiveSetUpdate_r11_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_ActiveSetUpdate_r11_IEs, ActiveSetUpdate_r11_IEs_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions_06_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_nonCriticalExtensions_06(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_nonCriticalExtensions_06, T_nonCriticalExtensions_06_sequence);

  return offset;
}


static const per_sequence_t T_r11_sequence[] = {
  { &hf_rrc_activeSetUpdate_r11, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_ActiveSetUpdate_r11_IEs },
  { &hf_rrc_activeSetUpdate_r11_add_ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_BIT_STRING },
  { &hf_rrc_nonCriticalExtensions_06, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_nonCriticalExtensions_06 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_r11(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_r11, T_r11_sequence);

  return offset;
}


static const per_sequence_t T_criticalExtensions_06_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_criticalExtensions_06(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_criticalExtensions_06, T_criticalExtensions_06_sequence);

  return offset;
}


static const value_string rrc_T_criticalExtensions_05_vals[] = {
  {   0, "r11" },
  {   1, "criticalExtensions" },
  { 0, NULL }
};

static const per_choice_t T_criticalExtensions_05_choice[] = {
  {   0, &hf_rrc_r11             , ASN1_NO_EXTENSIONS     , dissect_rrc_T_r11 },
  {   1, &hf_rrc_criticalExtensions_06, ASN1_NO_EXTENSIONS     , dissect_rrc_T_criticalExtensions_06 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_criticalExtensions_05(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_criticalExtensions_05, T_criticalExtensions_05_choice,
                                 NULL);

  return offset;
}


static const value_string rrc_T_criticalExtensions_04_vals[] = {
  {   0, "r10" },
  {   1, "criticalExtensions" },
  { 0, NULL }
};

static const per_choice_t T_criticalExtensions_04_choice[] = {
  {   0, &hf_rrc_r10             , ASN1_NO_EXTENSIONS     , dissect_rrc_T_r10 },
  {   1, &hf_rrc_criticalExtensions_05, ASN1_NO_EXTENSIONS     , dissect_rrc_T_criticalExtensions_05 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_criticalExtensions_04(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_criticalExtensions_04, T_criticalExtensions_04_choice,
                                 NULL);

  return offset;
}


static const value_string rrc_T_criticalExtensions_03_vals[] = {
  {   0, "r9" },
  {   1, "criticalExtensions" },
  { 0, NULL }
};

static const per_choice_t T_criticalExtensions_03_choice[] = {
  {   0, &hf_rrc_r9              , ASN1_NO_EXTENSIONS     , dissect_rrc_T_r9 },
  {   1, &hf_rrc_criticalExtensions_04, ASN1_NO_EXTENSIONS     , dissect_rrc_T_criticalExtensions_04 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_criticalExtensions_03(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_criticalExtensions_03, T_criticalExtensions_03_choice,
                                 NULL);

  return offset;
}


static const value_string rrc_T_criticalExtensions_02_vals[] = {
  {   0, "r8" },
  {   1, "criticalExtensions" },
  { 0, NULL }
};

static const per_choice_t T_criticalExtensions_02_choice[] = {
  {   0, &hf_rrc_r8              , ASN1_NO_EXTENSIONS     , dissect_rrc_T_r8 },
  {   1, &hf_rrc_criticalExtensions_03, ASN1_NO_EXTENSIONS     , dissect_rrc_T_criticalExtensions_03 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_criticalExtensions_02(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_criticalExtensions_02, T_criticalExtensions_02_choice,
                                 NULL);

  return offset;
}


static const value_string rrc_T_criticalExtensions_01_vals[] = {
  {   0, "r7" },
  {   1, "criticalExtensions" },
  { 0, NULL }
};

static const per_choice_t T_criticalExtensions_01_choice[] = {
  {   0, &hf_rrc_r7              , ASN1_NO_EXTENSIONS     , dissect_rrc_T_r7 },
  {   1, &hf_rrc_criticalExtensions_02, ASN1_NO_EXTENSIONS     , dissect_rrc_T_criticalExtensions_02 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_criticalExtensions_01(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_criticalExtensions_01, T_criticalExtensions_01_choice,
                                 NULL);

  return offset;
}


static const value_string rrc_T_criticalExtensions_vals[] = {
  {   0, "r6" },
  {   1, "criticalExtensions" },
  { 0, NULL }
};

static const per_choice_t T_criticalExtensions_choice[] = {
  {   0, &hf_rrc_r6              , ASN1_NO_EXTENSIONS     , dissect_rrc_T_r6 },
  {   1, &hf_rrc_criticalExtensions_01, ASN1_NO_EXTENSIONS     , dissect_rrc_T_criticalExtensions_01 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_criticalExtensions(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_criticalExtensions, T_criticalExtensions_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t T_later_than_r3_sequence[] = {
  { &hf_rrc_rrc_TransactionIdentifier, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RRC_TransactionIdentifier },
  { &hf_rrc_criticalExtensions, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_criticalExtensions },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_later_than_r3(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_later_than_r3, T_later_than_r3_sequence);

  return offset;
}


static const value_string rrc_ActiveSetUpdate_vals[] = {
  {   0, "r3" },
  {   1, "later-than-r3" },
  { 0, NULL }
};

static const per_choice_t ActiveSetUpdate_choice[] = {
  {   0, &hf_rrc_r3              , ASN1_NO_EXTENSIONS     , dissect_rrc_T_r3 },
  {   1, &hf_rrc_later_than_r3   , ASN1_NO_EXTENSIONS     , dissect_rrc_T_later_than_r3 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_ActiveSetUpdate(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
#line 165 "../../asn1/rrc/rrc.cnf"
  col_append_str(actx->pinfo->cinfo, COL_INFO, "ActiveSetUpdate");
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_ActiveSetUpdate, ActiveSetUpdate_choice,
                                 NULL);




  return offset;
}



static int
dissect_rrc_GPS_TOW_1msec(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            0U, 604799999U, NULL, FALSE);

  return offset;
}



static int
dissect_rrc_INTEGER_0_4294967295(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            0U, 4294967295U, NULL, FALSE);

  return offset;
}


static const per_sequence_t T_utran_GPSTimingOfCell_sequence[] = {
  { &hf_rrc_ms_part         , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_0_1023 },
  { &hf_rrc_ls_part         , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_0_4294967295 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_utran_GPSTimingOfCell(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_utran_GPSTimingOfCell, T_utran_GPSTimingOfCell_sequence);

  return offset;
}


static const per_sequence_t T_fdd_182_sequence[] = {
  { &hf_rrc_referenceIdentity, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PrimaryCPICH_Info },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_fdd_182(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_fdd_182, T_fdd_182_sequence);

  return offset;
}



static int
dissect_rrc_CellParametersID(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            0U, 127U, NULL, FALSE);

  return offset;
}


static const per_sequence_t T_tdd_131_sequence[] = {
  { &hf_rrc_referenceIdentity_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CellParametersID },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_tdd_131(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_tdd_131, T_tdd_131_sequence);

  return offset;
}


static const value_string rrc_T_modeSpecificInfo_147_vals[] = {
  {   0, "fdd" },
  {   1, "tdd" },
  { 0, NULL }
};

static const per_choice_t T_modeSpecificInfo_147_choice[] = {
  {   0, &hf_rrc_fdd_188         , ASN1_NO_EXTENSIONS     , dissect_rrc_T_fdd_182 },
  {   1, &hf_rrc_tdd_139         , ASN1_NO_EXTENSIONS     , dissect_rrc_T_tdd_131 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_modeSpecificInfo_147(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_modeSpecificInfo_147, T_modeSpecificInfo_147_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t UTRAN_GPSReferenceTime_sequence[] = {
  { &hf_rrc_utran_GPSTimingOfCell, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_utran_GPSTimingOfCell },
  { &hf_rrc_modeSpecificInfo_147, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_modeSpecificInfo_147 },
  { &hf_rrc_sfn             , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_0_4095 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UTRAN_GPSReferenceTime(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UTRAN_GPSReferenceTime, UTRAN_GPSReferenceTime_sequence);

  return offset;
}


static const value_string rrc_SFN_TOW_Uncertainty_vals[] = {
  {   0, "lessThan10" },
  {   1, "moreThan10" },
  { 0, NULL }
};


static int
dissect_rrc_SFN_TOW_Uncertainty(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     2, NULL, FALSE, 0, NULL);

  return offset;
}


static const value_string rrc_UTRAN_GPS_DriftRate_vals[] = {
  {   0, "utran-GPSDrift0" },
  {   1, "utran-GPSDrift1" },
  {   2, "utran-GPSDrift2" },
  {   3, "utran-GPSDrift5" },
  {   4, "utran-GPSDrift10" },
  {   5, "utran-GPSDrift15" },
  {   6, "utran-GPSDrift25" },
  {   7, "utran-GPSDrift50" },
  {   8, "utran-GPSDrift-1" },
  {   9, "utran-GPSDrift-2" },
  {  10, "utran-GPSDrift-5" },
  {  11, "utran-GPSDrift-10" },
  {  12, "utran-GPSDrift-15" },
  {  13, "utran-GPSDrift-25" },
  {  14, "utran-GPSDrift-50" },
  { 0, NULL }
};


static int
dissect_rrc_UTRAN_GPS_DriftRate(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     15, NULL, FALSE, 0, NULL);

  return offset;
}



static int
dissect_rrc_SatID(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            0U, 63U, NULL, FALSE);

  return offset;
}



static int
dissect_rrc_BIT_STRING_SIZE_14(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_bit_string(tvb, offset, actx, tree, hf_index,
                                     14, 14, FALSE, NULL);

  return offset;
}



static int
dissect_rrc_BIT_STRING_SIZE_2(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_bit_string(tvb, offset, actx, tree, hf_index,
                                     2, 2, FALSE, NULL);

  return offset;
}


static const per_sequence_t GPS_TOW_Assist_sequence[] = {
  { &hf_rrc_satID           , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SatID },
  { &hf_rrc_tlm_Message     , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_14 },
  { &hf_rrc_tlm_Reserved    , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_2 },
  { &hf_rrc_alert           , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { &hf_rrc_antiSpoof       , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_GPS_TOW_Assist(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_GPS_TOW_Assist, GPS_TOW_Assist_sequence);

  return offset;
}


static const per_sequence_t GPS_TOW_AssistList_sequence_of[1] = {
  { &hf_rrc_GPS_TOW_AssistList_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_GPS_TOW_Assist },
};

static int
dissect_rrc_GPS_TOW_AssistList(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_GPS_TOW_AssistList, GPS_TOW_AssistList_sequence_of,
                                                  1, maxSat, FALSE);

  return offset;
}


static const per_sequence_t UE_Positioning_GPS_ReferenceTime_sequence[] = {
  { &hf_rrc_gps_Week        , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_0_1023 },
  { &hf_rrc_gps_tow_1msec   , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_GPS_TOW_1msec },
  { &hf_rrc_utran_GPSReferenceTime, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UTRAN_GPSReferenceTime },
  { &hf_rrc_sfn_tow_Uncertainty, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_SFN_TOW_Uncertainty },
  { &hf_rrc_utran_GPS_DriftRate, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UTRAN_GPS_DriftRate },
  { &hf_rrc_gps_TOW_AssistList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_GPS_TOW_AssistList },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UE_Positioning_GPS_ReferenceTime(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UE_Positioning_GPS_ReferenceTime, UE_Positioning_GPS_ReferenceTime_sequence);

  return offset;
}


static const value_string rrc_T_latitudeSign_02_vals[] = {
  {   0, "north" },
  {   1, "south" },
  { 0, NULL }
};


static int
dissect_rrc_T_latitudeSign_02(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     2, NULL, FALSE, 0, NULL);

  return offset;
}



static int
dissect_rrc_INTEGER_0_8388607(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            0U, 8388607U, NULL, FALSE);

  return offset;
}



static int
dissect_rrc_INTEGER_M8388608_8388607(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            -8388608, 8388607U, NULL, FALSE);

  return offset;
}


static const value_string rrc_T_altitudeDirection_01_vals[] = {
  {   0, "height" },
  {   1, "depth" },
  { 0, NULL }
};


static int
dissect_rrc_T_altitudeDirection_01(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     2, NULL, FALSE, 0, NULL);

  return offset;
}



static int
dissect_rrc_INTEGER_0_32767(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            0U, 32767U, NULL, FALSE);

  return offset;
}



static int
dissect_rrc_INTEGER_0_89(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            0U, 89U, NULL, FALSE);

  return offset;
}



static int
dissect_rrc_INTEGER_0_100(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            0U, 100U, NULL, FALSE);

  return offset;
}


static const per_sequence_t EllipsoidPointAltitudeEllipsoide_sequence[] = {
  { &hf_rrc_latitudeSign_02 , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_latitudeSign_02 },
  { &hf_rrc_latitude        , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_0_8388607 },
  { &hf_rrc_longitude       , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_M8388608_8388607 },
  { &hf_rrc_altitudeDirection_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_altitudeDirection_01 },
  { &hf_rrc_altitude        , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_0_32767 },
  { &hf_rrc_uncertaintySemiMajor, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_0_127 },
  { &hf_rrc_uncertaintySemiMinor, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_0_127 },
  { &hf_rrc_orientationMajorAxis, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_0_89 },
  { &hf_rrc_uncertaintyAltitude, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_0_127 },
  { &hf_rrc_confidence      , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_0_100 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_EllipsoidPointAltitudeEllipsoide(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_EllipsoidPointAltitudeEllipsoide, EllipsoidPointAltitudeEllipsoide_sequence);

  return offset;
}


static const per_sequence_t ReferenceLocation_sequence[] = {
  { &hf_rrc_ellipsoidPointAltitudeEllipsoide, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_EllipsoidPointAltitudeEllipsoide },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_ReferenceLocation(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_ReferenceLocation, ReferenceLocation_sequence);

  return offset;
}



static int
dissect_rrc_GPS_TOW_1sec(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            0U, 604799U, NULL, FALSE);

  return offset;
}


static const value_string rrc_DiffCorrectionStatus_vals[] = {
  {   0, "udre-1-0" },
  {   1, "udre-0-75" },
  {   2, "udre-0-5" },
  {   3, "udre-0-3" },
  {   4, "udre-0-2" },
  {   5, "udre-0-1" },
  {   6, "noData" },
  {   7, "invalidData" },
  { 0, NULL }
};


static int
dissect_rrc_DiffCorrectionStatus(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     8, NULL, FALSE, 0, NULL);

  return offset;
}



static int
dissect_rrc_IODE(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            0U, 255U, NULL, FALSE);

  return offset;
}


static const value_string rrc_UDRE_vals[] = {
  {   0, "lessThan1" },
  {   1, "between1-and-4" },
  {   2, "between4-and-8" },
  {   3, "over8" },
  { 0, NULL }
};


static int
dissect_rrc_UDRE(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     4, NULL, FALSE, 0, NULL);

  return offset;
}



static int
dissect_rrc_PRC(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            -2047, 2047U, NULL, FALSE);

  return offset;
}



static int
dissect_rrc_RRC(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            -127, 127U, NULL, FALSE);

  return offset;
}



static int
dissect_rrc_DeltaPRC(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            -127, 127U, NULL, FALSE);

  return offset;
}



static int
dissect_rrc_DeltaRRC(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            -7, 7U, NULL, FALSE);

  return offset;
}


static const per_sequence_t DGPS_CorrectionSatInfo_sequence[] = {
  { &hf_rrc_satID           , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SatID },
  { &hf_rrc_iode            , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_IODE },
  { &hf_rrc_udre            , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UDRE },
  { &hf_rrc_prc             , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PRC },
  { &hf_rrc_rrc             , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RRC },
  { &hf_rrc_dummy1_05       , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DeltaPRC },
  { &hf_rrc_dummy2_11       , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DeltaRRC },
  { &hf_rrc_dummy3_02       , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DeltaPRC },
  { &hf_rrc_dummy4_01       , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DeltaRRC },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_DGPS_CorrectionSatInfo(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_DGPS_CorrectionSatInfo, DGPS_CorrectionSatInfo_sequence);

  return offset;
}


static const per_sequence_t DGPS_CorrectionSatInfoList_sequence_of[1] = {
  { &hf_rrc_DGPS_CorrectionSatInfoList_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DGPS_CorrectionSatInfo },
};

static int
dissect_rrc_DGPS_CorrectionSatInfoList(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_DGPS_CorrectionSatInfoList, DGPS_CorrectionSatInfoList_sequence_of,
                                                  1, maxSat, FALSE);

  return offset;
}


static const per_sequence_t UE_Positioning_GPS_DGPS_Corrections_sequence[] = {
  { &hf_rrc_gps_TOW         , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_GPS_TOW_1sec },
  { &hf_rrc_statusHealth    , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DiffCorrectionStatus },
  { &hf_rrc_dgps_CorrectionSatInfoList, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DGPS_CorrectionSatInfoList },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UE_Positioning_GPS_DGPS_Corrections(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UE_Positioning_GPS_DGPS_Corrections, UE_Positioning_GPS_DGPS_Corrections_sequence);

  return offset;
}


static const value_string rrc_SatelliteStatus_vals[] = {
  {   0, "ns-NN-U" },
  {   1, "es-SN" },
  {   2, "es-NN-U" },
  {   3, "rev2" },
  {   4, "rev" },
  { 0, NULL }
};


static int
dissect_rrc_SatelliteStatus(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     5, NULL, FALSE, 0, NULL);

  return offset;
}



static int
dissect_rrc_BIT_STRING_SIZE_4(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_bit_string(tvb, offset, actx, tree, hf_index,
                                     4, 4, FALSE, NULL);

  return offset;
}



static int
dissect_rrc_BIT_STRING_SIZE_6(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_bit_string(tvb, offset, actx, tree, hf_index,
                                     6, 6, FALSE, NULL);

  return offset;
}



static int
dissect_rrc_BIT_STRING_SIZE_10(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_bit_string(tvb, offset, actx, tree, hf_index,
                                     10, 10, FALSE, NULL);

  return offset;
}



static int
dissect_rrc_BIT_STRING_SIZE_1(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_bit_string(tvb, offset, actx, tree, hf_index,
                                     1, 1, FALSE, NULL);

  return offset;
}



static int
dissect_rrc_BIT_STRING_SIZE_23(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_bit_string(tvb, offset, actx, tree, hf_index,
                                     23, 23, FALSE, NULL);

  return offset;
}



static int
dissect_rrc_BIT_STRING_SIZE_24(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_bit_string(tvb, offset, actx, tree, hf_index,
                                     24, 24, FALSE, NULL);

  return offset;
}



static int
dissect_rrc_BIT_STRING_SIZE_16(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_bit_string(tvb, offset, actx, tree, hf_index,
                                     16, 16, FALSE, NULL);

  return offset;
}


static const per_sequence_t SubFrame1Reserved_sequence[] = {
  { &hf_rrc_reserved1       , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_23 },
  { &hf_rrc_reserved2       , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_24 },
  { &hf_rrc_reserved3       , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_24 },
  { &hf_rrc_reserved4       , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_16 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_SubFrame1Reserved(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_SubFrame1Reserved, SubFrame1Reserved_sequence);

  return offset;
}



static int
dissect_rrc_BIT_STRING_SIZE_22(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_bit_string(tvb, offset, actx, tree, hf_index,
                                     22, 22, FALSE, NULL);

  return offset;
}



static int
dissect_rrc_BIT_STRING_SIZE_32(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_bit_string(tvb, offset, actx, tree, hf_index,
                                     32, 32, FALSE, NULL);

  return offset;
}



static int
dissect_rrc_BIT_STRING_SIZE_5(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_bit_string(tvb, offset, actx, tree, hf_index,
                                     5, 5, FALSE, NULL);

  return offset;
}


static const per_sequence_t EphemerisParameter_sequence[] = {
  { &hf_rrc_codeOnL2        , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_2 },
  { &hf_rrc_uraIndex        , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_4 },
  { &hf_rrc_satHealth_01    , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_6 },
  { &hf_rrc_iodc            , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_10 },
  { &hf_rrc_l2Pflag         , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_1 },
  { &hf_rrc_sf1Revd         , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SubFrame1Reserved },
  { &hf_rrc_t_GD            , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_8 },
  { &hf_rrc_t_oc            , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_16 },
  { &hf_rrc_af2             , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_8 },
  { &hf_rrc_af1_01          , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_16 },
  { &hf_rrc_af0_01          , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_22 },
  { &hf_rrc_c_rs            , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_16 },
  { &hf_rrc_delta_n         , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_16 },
  { &hf_rrc_m0_01           , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_32 },
  { &hf_rrc_c_uc            , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_16 },
  { &hf_rrc_e_01            , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_32 },
  { &hf_rrc_c_us            , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_16 },
  { &hf_rrc_a_Sqrt_01       , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_32 },
  { &hf_rrc_t_oe            , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_16 },
  { &hf_rrc_fitInterval     , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_1 },
  { &hf_rrc_aodo            , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_5 },
  { &hf_rrc_c_ic            , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_16 },
  { &hf_rrc_omega0_01       , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_32 },
  { &hf_rrc_c_is            , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_16 },
  { &hf_rrc_i0              , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_32 },
  { &hf_rrc_c_rc            , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_16 },
  { &hf_rrc_omega_01        , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_32 },
  { &hf_rrc_omegaDot_01     , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_24 },
  { &hf_rrc_iDot            , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_14 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_EphemerisParameter(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_EphemerisParameter, EphemerisParameter_sequence);

  return offset;
}


static const per_sequence_t NavigationModelSatInfo_sequence[] = {
  { &hf_rrc_satID           , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SatID },
  { &hf_rrc_satelliteStatus , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SatelliteStatus },
  { &hf_rrc_ephemerisParameter, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_EphemerisParameter },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_NavigationModelSatInfo(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_NavigationModelSatInfo, NavigationModelSatInfo_sequence);

  return offset;
}


static const per_sequence_t NavigationModelSatInfoList_sequence_of[1] = {
  { &hf_rrc_NavigationModelSatInfoList_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_NavigationModelSatInfo },
};

static int
dissect_rrc_NavigationModelSatInfoList(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_NavigationModelSatInfoList, NavigationModelSatInfoList_sequence_of,
                                                  1, maxSat, FALSE);

  return offset;
}


static const per_sequence_t UE_Positioning_GPS_NavigationModel_sequence[] = {
  { &hf_rrc_navigationModelSatInfoList, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_NavigationModelSatInfoList },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UE_Positioning_GPS_NavigationModel(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UE_Positioning_GPS_NavigationModel, UE_Positioning_GPS_NavigationModel_sequence);

  return offset;
}


static const per_sequence_t UE_Positioning_GPS_IonosphericModel_sequence[] = {
  { &hf_rrc_alpha0          , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_8 },
  { &hf_rrc_alpha1          , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_8 },
  { &hf_rrc_alpha2          , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_8 },
  { &hf_rrc_alpha3          , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_8 },
  { &hf_rrc_beta0           , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_8 },
  { &hf_rrc_beta1           , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_8 },
  { &hf_rrc_beta2           , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_8 },
  { &hf_rrc_beta3           , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_8 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UE_Positioning_GPS_IonosphericModel(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UE_Positioning_GPS_IonosphericModel, UE_Positioning_GPS_IonosphericModel_sequence);

  return offset;
}


static const per_sequence_t UE_Positioning_GPS_UTC_Model_sequence[] = {
  { &hf_rrc_a1              , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_24 },
  { &hf_rrc_a0              , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_32 },
  { &hf_rrc_t_ot            , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_8 },
  { &hf_rrc_wn_t            , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_8 },
  { &hf_rrc_delta_t_LS      , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_8 },
  { &hf_rrc_wn_lsf          , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_8 },
  { &hf_rrc_dn              , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_8 },
  { &hf_rrc_delta_t_LSF     , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_8 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UE_Positioning_GPS_UTC_Model(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UE_Positioning_GPS_UTC_Model, UE_Positioning_GPS_UTC_Model_sequence);

  return offset;
}



static int
dissect_rrc_BIT_STRING_SIZE_11(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_bit_string(tvb, offset, actx, tree, hf_index,
                                     11, 11, FALSE, NULL);

  return offset;
}


static const per_sequence_t AlmanacSatInfo_sequence[] = {
  { &hf_rrc_dataID          , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_0_3 },
  { &hf_rrc_satID           , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SatID },
  { &hf_rrc_e               , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_16 },
  { &hf_rrc_t_oa            , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_8 },
  { &hf_rrc_deltaI          , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_16 },
  { &hf_rrc_omegaDot        , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_16 },
  { &hf_rrc_satHealth       , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_8 },
  { &hf_rrc_a_Sqrt          , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_24 },
  { &hf_rrc_omega0          , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_24 },
  { &hf_rrc_m0              , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_24 },
  { &hf_rrc_omega           , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_24 },
  { &hf_rrc_af0             , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_11 },
  { &hf_rrc_af1             , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_11 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_AlmanacSatInfo(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_AlmanacSatInfo, AlmanacSatInfo_sequence);

  return offset;
}


static const per_sequence_t AlmanacSatInfoList_sequence_of[1] = {
  { &hf_rrc_AlmanacSatInfoList_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_AlmanacSatInfo },
};

static int
dissect_rrc_AlmanacSatInfoList(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_AlmanacSatInfoList, AlmanacSatInfoList_sequence_of,
                                                  1, maxSat, FALSE);

  return offset;
}



static int
dissect_rrc_BIT_STRING_SIZE_364(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_bit_string(tvb, offset, actx, tree, hf_index,
                                     364, 364, FALSE, NULL);

  return offset;
}


static const per_sequence_t UE_Positioning_GPS_Almanac_sequence[] = {
  { &hf_rrc_wn_a            , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_8 },
  { &hf_rrc_almanacSatInfoList, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_AlmanacSatInfoList },
  { &hf_rrc_sv_GlobalHealth , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_BIT_STRING_SIZE_364 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UE_Positioning_GPS_Almanac(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UE_Positioning_GPS_Almanac, UE_Positioning_GPS_Almanac_sequence);

  return offset;
}



static int
dissect_rrc_INTEGER_M2048_2047(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            -2048, 2047U, NULL, FALSE);

  return offset;
}



static int
dissect_rrc_INTEGER_M42_21(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            -42, 21U, NULL, FALSE);

  return offset;
}


static const value_string rrc_DopplerUncertainty_vals[] = {
  {   0, "hz12-5" },
  {   1, "hz25" },
  {   2, "hz50" },
  {   3, "hz100" },
  {   4, "hz200" },
  {   5, "spare3" },
  {   6, "spare2" },
  {   7, "spare1" },
  { 0, NULL }
};


static int
dissect_rrc_DopplerUncertainty(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     8, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t ExtraDopplerInfo_sequence[] = {
  { &hf_rrc_doppler1stOrder , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_M42_21 },
  { &hf_rrc_dopplerUncertainty_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DopplerUncertainty },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_ExtraDopplerInfo(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_ExtraDopplerInfo, ExtraDopplerInfo_sequence);

  return offset;
}



static int
dissect_rrc_INTEGER_0_1022(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            0U, 1022U, NULL, FALSE);

  return offset;
}



static int
dissect_rrc_INTEGER_0_19(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            0U, 19U, NULL, FALSE);

  return offset;
}


static const value_string rrc_CodePhaseSearchWindow_vals[] = {
  {   0, "w1023" },
  {   1, "w1" },
  {   2, "w2" },
  {   3, "w3" },
  {   4, "w4" },
  {   5, "w6" },
  {   6, "w8" },
  {   7, "w12" },
  {   8, "w16" },
  {   9, "w24" },
  {  10, "w32" },
  {  11, "w48" },
  {  12, "w64" },
  {  13, "w96" },
  {  14, "w128" },
  {  15, "w192" },
  { 0, NULL }
};


static int
dissect_rrc_CodePhaseSearchWindow(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     16, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t AzimuthAndElevation_sequence[] = {
  { &hf_rrc_azimuth         , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_0_31 },
  { &hf_rrc_elevation       , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_0_7 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_AzimuthAndElevation(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_AzimuthAndElevation, AzimuthAndElevation_sequence);

  return offset;
}


static const per_sequence_t AcquisitionSatInfo_sequence[] = {
  { &hf_rrc_satID           , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SatID },
  { &hf_rrc_doppler0thOrder , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_M2048_2047 },
  { &hf_rrc_extraDopplerInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_ExtraDopplerInfo },
  { &hf_rrc_codePhase       , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_0_1022 },
  { &hf_rrc_integerCodePhase, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_0_19 },
  { &hf_rrc_gps_BitNumber   , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_0_3 },
  { &hf_rrc_codePhaseSearchWindow, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CodePhaseSearchWindow },
  { &hf_rrc_azimuthAndElevation, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_AzimuthAndElevation },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_AcquisitionSatInfo(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_AcquisitionSatInfo, AcquisitionSatInfo_sequence);

  return offset;
}


static const per_sequence_t AcquisitionSatInfoList_sequence_of[1] = {
  { &hf_rrc_AcquisitionSatInfoList_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_AcquisitionSatInfo },
};

static int
dissect_rrc_AcquisitionSatInfoList(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_AcquisitionSatInfoList, AcquisitionSatInfoList_sequence_of,
                                                  1, maxSat, FALSE);

  return offset;
}


static const per_sequence_t UE_Positioning_GPS_AcquisitionAssistance_sequence[] = {
  { &hf_rrc_gps_ReferenceTime, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_GPS_TOW_1msec },
  { &hf_rrc_utran_GPSReferenceTime, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UTRAN_GPSReferenceTime },
  { &hf_rrc_satelliteInformationList_03, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_AcquisitionSatInfoList },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UE_Positioning_GPS_AcquisitionAssistance(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UE_Positioning_GPS_AcquisitionAssistance, UE_Positioning_GPS_AcquisitionAssistance_sequence);

  return offset;
}


static const per_sequence_t BadSatList_sequence_of[1] = {
  { &hf_rrc_BadSatList_item , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_0_63 },
};

static int
dissect_rrc_BadSatList(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_BadSatList, BadSatList_sequence_of,
                                                  1, maxSat, FALSE);

  return offset;
}


static const per_sequence_t T_fdd_169_sequence[] = {
  { &hf_rrc_referenceIdentity, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PrimaryCPICH_Info },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_fdd_169(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_fdd_169, T_fdd_169_sequence);

  return offset;
}


static const per_sequence_t T_tdd_118_sequence[] = {
  { &hf_rrc_referenceIdentity_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CellParametersID },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_tdd_118(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_tdd_118, T_tdd_118_sequence);

  return offset;
}


static const value_string rrc_T_modeSpecificInfo_135_vals[] = {
  {   0, "fdd" },
  {   1, "tdd" },
  { 0, NULL }
};

static const per_choice_t T_modeSpecificInfo_135_choice[] = {
  {   0, &hf_rrc_fdd_175         , ASN1_NO_EXTENSIONS     , dissect_rrc_T_fdd_169 },
  {   1, &hf_rrc_tdd_126         , ASN1_NO_EXTENSIONS     , dissect_rrc_T_tdd_118 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_modeSpecificInfo_135(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_modeSpecificInfo_135, T_modeSpecificInfo_135_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t UE_Positioning_GPS_ReferenceCellInfo_sequence[] = {
  { &hf_rrc_modeSpecificInfo_135, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_modeSpecificInfo_135 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UE_Positioning_GPS_ReferenceCellInfo(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UE_Positioning_GPS_ReferenceCellInfo, UE_Positioning_GPS_ReferenceCellInfo_sequence);

  return offset;
}


static const per_sequence_t UE_Positioning_GPS_AssistanceData_sequence[] = {
  { &hf_rrc_ue_positioning_GPS_ReferenceTime, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_Positioning_GPS_ReferenceTime },
  { &hf_rrc_ue_positioning_GPS_ReferenceLocation, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_ReferenceLocation },
  { &hf_rrc_ue_positioning_GPS_DGPS_Corrections, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_Positioning_GPS_DGPS_Corrections },
  { &hf_rrc_ue_positioning_GPS_NavigationModel, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_Positioning_GPS_NavigationModel },
  { &hf_rrc_ue_positioning_GPS_IonosphericModel, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_Positioning_GPS_IonosphericModel },
  { &hf_rrc_ue_positioning_GPS_UTC_Model, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_Positioning_GPS_UTC_Model },
  { &hf_rrc_ue_positioning_GPS_Almanac, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_Positioning_GPS_Almanac },
  { &hf_rrc_ue_positioning_GPS_AcquisitionAssistance, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_Positioning_GPS_AcquisitionAssistance },
  { &hf_rrc_ue_positioning_GPS_Real_timeIntegrity, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_BadSatList },
  { &hf_rrc_dummy_54        , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_Positioning_GPS_ReferenceCellInfo },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UE_Positioning_GPS_AssistanceData(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UE_Positioning_GPS_AssistanceData, UE_Positioning_GPS_AssistanceData_sequence);

  return offset;
}


static const per_sequence_t T_fdd_178_sequence[] = {
  { &hf_rrc_primaryCPICH_Info, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PrimaryCPICH_Info },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_fdd_178(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_fdd_178, T_fdd_178_sequence);

  return offset;
}


static const value_string rrc_BurstType_vals[] = {
  {   0, "type1" },
  {   1, "type2" },
  { 0, NULL }
};


static int
dissect_rrc_BurstType(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     2, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t CellAndChannelIdentity_sequence[] = {
  { &hf_rrc_burstType       , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BurstType },
  { &hf_rrc_midambleShift   , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MidambleShiftLong },
  { &hf_rrc_timeslot        , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TimeslotNumber },
  { &hf_rrc_cellParametersID, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CellParametersID },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_CellAndChannelIdentity(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_CellAndChannelIdentity, CellAndChannelIdentity_sequence);

  return offset;
}


static const per_sequence_t T_tdd_127_sequence[] = {
  { &hf_rrc_cellAndChannelIdentity, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CellAndChannelIdentity },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_tdd_127(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_tdd_127, T_tdd_127_sequence);

  return offset;
}


static const value_string rrc_T_modeSpecificInfo_144_vals[] = {
  {   0, "fdd" },
  {   1, "tdd" },
  { 0, NULL }
};

static const per_choice_t T_modeSpecificInfo_144_choice[] = {
  {   0, &hf_rrc_fdd_184         , ASN1_NO_EXTENSIONS     , dissect_rrc_T_fdd_178 },
  {   1, &hf_rrc_tdd_135         , ASN1_NO_EXTENSIONS     , dissect_rrc_T_tdd_127 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_modeSpecificInfo_144(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_modeSpecificInfo_144, T_modeSpecificInfo_144_choice,
                                 NULL);

  return offset;
}


static const value_string rrc_T_latitudeSign_vals[] = {
  {   0, "north" },
  {   1, "south" },
  { 0, NULL }
};


static int
dissect_rrc_T_latitudeSign(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     2, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t EllipsoidPoint_sequence[] = {
  { &hf_rrc_latitudeSign    , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_latitudeSign },
  { &hf_rrc_latitude        , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_0_8388607 },
  { &hf_rrc_longitude       , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_M8388608_8388607 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_EllipsoidPoint(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_EllipsoidPoint, EllipsoidPoint_sequence);

  return offset;
}


static const value_string rrc_T_latitudeSign_01_vals[] = {
  {   0, "north" },
  {   1, "south" },
  { 0, NULL }
};


static int
dissect_rrc_T_latitudeSign_01(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     2, NULL, FALSE, 0, NULL);

  return offset;
}


static const value_string rrc_T_altitudeDirection_vals[] = {
  {   0, "height" },
  {   1, "depth" },
  { 0, NULL }
};


static int
dissect_rrc_T_altitudeDirection(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     2, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t EllipsoidPointAltitude_sequence[] = {
  { &hf_rrc_latitudeSign_01 , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_latitudeSign_01 },
  { &hf_rrc_latitude        , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_0_8388607 },
  { &hf_rrc_longitude       , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_M8388608_8388607 },
  { &hf_rrc_altitudeDirection, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_altitudeDirection },
  { &hf_rrc_altitude        , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_0_32767 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_EllipsoidPointAltitude(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_EllipsoidPointAltitude, EllipsoidPointAltitude_sequence);

  return offset;
}


static const value_string rrc_ReferenceCellPosition_vals[] = {
  {   0, "ellipsoidPoint" },
  {   1, "ellipsoidPointWithAltitude" },
  { 0, NULL }
};

static const per_choice_t ReferenceCellPosition_choice[] = {
  {   0, &hf_rrc_ellipsoidPoint  , ASN1_NO_EXTENSIONS     , dissect_rrc_EllipsoidPoint },
  {   1, &hf_rrc_ellipsoidPointWithAltitude, ASN1_NO_EXTENSIONS     , dissect_rrc_EllipsoidPointAltitude },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_ReferenceCellPosition(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_ReferenceCellPosition, ReferenceCellPosition_choice,
                                 NULL);

  return offset;
}



static int
dissect_rrc_INTEGER_0_32766(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            0U, 32766U, NULL, FALSE);

  return offset;
}


static const value_string rrc_IP_Spacing_vals[] = {
  {   0, "e5" },
  {   1, "e7" },
  {   2, "e10" },
  {   3, "e15" },
  {   4, "e20" },
  {   5, "e30" },
  {   6, "e40" },
  {   7, "e50" },
  { 0, NULL }
};


static int
dissect_rrc_IP_Spacing(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     8, NULL, FALSE, 0, NULL);

  return offset;
}


static const value_string rrc_IP_Length_vals[] = {
  {   0, "ipl5" },
  {   1, "ipl10" },
  { 0, NULL }
};


static int
dissect_rrc_IP_Length(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     2, NULL, FALSE, 0, NULL);

  return offset;
}



static int
dissect_rrc_INTEGER_0_9(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            0U, 9U, NULL, FALSE);

  return offset;
}



static int
dissect_rrc_INTEGER_10_25(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            10U, 25U, NULL, FALSE);

  return offset;
}



static int
dissect_rrc_INTEGER_1_16(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            1U, 16U, NULL, FALSE);

  return offset;
}


static const per_sequence_t BurstModeParameters_sequence[] = {
  { &hf_rrc_burstStart      , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_0_15 },
  { &hf_rrc_burstLength     , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_10_25 },
  { &hf_rrc_burstFreq       , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_1_16 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_BurstModeParameters(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_BurstModeParameters, BurstModeParameters_sequence);

  return offset;
}


static const per_sequence_t UE_Positioning_IPDL_Parameters_sequence[] = {
  { &hf_rrc_ip_Spacing      , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_IP_Spacing },
  { &hf_rrc_ip_Length       , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_IP_Length },
  { &hf_rrc_ip_Offset       , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_0_9 },
  { &hf_rrc_seed            , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_0_63 },
  { &hf_rrc_burstModeParameters, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_BurstModeParameters },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UE_Positioning_IPDL_Parameters(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UE_Positioning_IPDL_Parameters, UE_Positioning_IPDL_Parameters_sequence);

  return offset;
}


static const per_sequence_t UE_Positioning_OTDOA_ReferenceCellInfo_UEB_sequence[] = {
  { &hf_rrc_sfn             , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_INTEGER_0_4095 },
  { &hf_rrc_modeSpecificInfo_144, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_modeSpecificInfo_144 },
  { &hf_rrc_frequencyInfo   , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_FrequencyInfo },
  { &hf_rrc_cellPosition    , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_ReferenceCellPosition },
  { &hf_rrc_roundTripTime   , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_INTEGER_0_32766 },
  { &hf_rrc_ue_positioning_IPDL_Paremeters, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_Positioning_IPDL_Parameters },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UE_Positioning_OTDOA_ReferenceCellInfo_UEB(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UE_Positioning_OTDOA_ReferenceCellInfo_UEB, UE_Positioning_OTDOA_ReferenceCellInfo_UEB_sequence);

  return offset;
}


static const per_sequence_t T_fdd_174_sequence[] = {
  { &hf_rrc_primaryCPICH_Info, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PrimaryCPICH_Info },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_fdd_174(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_fdd_174, T_fdd_174_sequence);

  return offset;
}


static const per_sequence_t T_tdd_123_sequence[] = {
  { &hf_rrc_cellAndChannelIdentity, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CellAndChannelIdentity },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_tdd_123(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_tdd_123, T_tdd_123_sequence);

  return offset;
}


static const value_string rrc_T_modeSpecificInfo_140_vals[] = {
  {   0, "fdd" },
  {   1, "tdd" },
  { 0, NULL }
};

static const per_choice_t T_modeSpecificInfo_140_choice[] = {
  {   0, &hf_rrc_fdd_180         , ASN1_NO_EXTENSIONS     , dissect_rrc_T_fdd_174 },
  {   1, &hf_rrc_tdd_131         , ASN1_NO_EXTENSIONS     , dissect_rrc_T_tdd_123 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_modeSpecificInfo_140(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_modeSpecificInfo_140, T_modeSpecificInfo_140_choice,
                                 NULL);

  return offset;
}



static int
dissect_rrc_INTEGER_0_38399(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            0U, 38399U, NULL, FALSE);

  return offset;
}


static const per_sequence_t SFN_SFN_RelTimeDifference1_sequence[] = {
  { &hf_rrc_sfn_Offset      , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_0_4095 },
  { &hf_rrc_sfn_sfn_Reltimedifference, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_0_38399 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_SFN_SFN_RelTimeDifference1(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_SFN_SFN_RelTimeDifference1, SFN_SFN_RelTimeDifference1_sequence);

  return offset;
}


static const value_string rrc_SFN_SFN_Drift_vals[] = {
  {   0, "sfnsfndrift0" },
  {   1, "sfnsfndrift1" },
  {   2, "sfnsfndrift2" },
  {   3, "sfnsfndrift3" },
  {   4, "sfnsfndrift4" },
  {   5, "sfnsfndrift5" },
  {   6, "sfnsfndrift8" },
  {   7, "sfnsfndrift10" },
  {   8, "sfnsfndrift15" },
  {   9, "sfnsfndrift25" },
  {  10, "sfnsfndrift35" },
  {  11, "sfnsfndrift50" },
  {  12, "sfnsfndrift65" },
  {  13, "sfnsfndrift80" },
  {  14, "sfnsfndrift100" },
  {  15, "sfnsfndrift-1" },
  {  16, "sfnsfndrift-2" },
  {  17, "sfnsfndrift-3" },
  {  18, "sfnsfndrift-4" },
  {  19, "sfnsfndrift-5" },
  {  20, "sfnsfndrift-8" },
  {  21, "sfnsfndrift-10" },
  {  22, "sfnsfndrift-15" },
  {  23, "sfnsfndrift-25" },
  {  24, "sfnsfndrift-35" },
  {  25, "sfnsfndrift-50" },
  {  26, "sfnsfndrift-65" },
  {  27, "sfnsfndrift-80" },
  {  28, "sfnsfndrift-100" },
  { 0, NULL }
};

static value_string_ext rrc_SFN_SFN_Drift_vals_ext = VALUE_STRING_EXT_INIT(rrc_SFN_SFN_Drift_vals);


static int
dissect_rrc_SFN_SFN_Drift(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     29, NULL, FALSE, 0, NULL);

  return offset;
}


static const value_string rrc_OTDOA_SearchWindowSize_vals[] = {
  {   0, "c20" },
  {   1, "c40" },
  {   2, "c80" },
  {   3, "c160" },
  {   4, "c320" },
  {   5, "c640" },
  {   6, "c1280" },
  {   7, "moreThan1280" },
  { 0, NULL }
};


static int
dissect_rrc_OTDOA_SearchWindowSize(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     8, NULL, FALSE, 0, NULL);

  return offset;
}



static int
dissect_rrc_INTEGER_M20000_20000(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            -20000, 20000U, NULL, FALSE);

  return offset;
}



static int
dissect_rrc_INTEGER_M4000_4000(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            -4000, 4000U, NULL, FALSE);

  return offset;
}



static int
dissect_rrc_FineSFN_SFN(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            0U, 15U, NULL, FALSE);

  return offset;
}


static const per_sequence_t UE_Positioning_OTDOA_NeighbourCellInfo_UEB_sequence[] = {
  { &hf_rrc_modeSpecificInfo_140, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_modeSpecificInfo_140 },
  { &hf_rrc_frequencyInfo   , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_FrequencyInfo },
  { &hf_rrc_ue_positioning_IPDL_Paremeters, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_Positioning_IPDL_Parameters },
  { &hf_rrc_sfn_SFN_RelTimeDifference, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SFN_SFN_RelTimeDifference1 },
  { &hf_rrc_sfn_SFN_Drift   , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_SFN_SFN_Drift },
  { &hf_rrc_searchWindowSize, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_OTDOA_SearchWindowSize },
  { &hf_rrc_relativeNorth   , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_INTEGER_M20000_20000 },
  { &hf_rrc_relativeEast    , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_INTEGER_M20000_20000 },
  { &hf_rrc_relativeAltitude, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_INTEGER_M4000_4000 },
  { &hf_rrc_fineSFN_SFN     , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_FineSFN_SFN },
  { &hf_rrc_roundTripTime   , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_INTEGER_0_32766 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UE_Positioning_OTDOA_NeighbourCellInfo_UEB(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UE_Positioning_OTDOA_NeighbourCellInfo_UEB, UE_Positioning_OTDOA_NeighbourCellInfo_UEB_sequence);

  return offset;
}


static const per_sequence_t UE_Positioning_OTDOA_NeighbourCellList_UEB_sequence_of[1] = {
  { &hf_rrc_UE_Positioning_OTDOA_NeighbourCellList_UEB_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UE_Positioning_OTDOA_NeighbourCellInfo_UEB },
};

static int
dissect_rrc_UE_Positioning_OTDOA_NeighbourCellList_UEB(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_UE_Positioning_OTDOA_NeighbourCellList_UEB, UE_Positioning_OTDOA_NeighbourCellList_UEB_sequence_of,
                                                  1, maxCellMeas, FALSE);

  return offset;
}


static const per_sequence_t UE_Positioning_OTDOA_AssistanceData_UEB_sequence[] = {
  { &hf_rrc_ue_positioning_OTDOA_ReferenceCellInfo_UEB, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_Positioning_OTDOA_ReferenceCellInfo_UEB },
  { &hf_rrc_ue_positioning_OTDOA_NeighbourCellList_UEB, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_Positioning_OTDOA_NeighbourCellList_UEB },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UE_Positioning_OTDOA_AssistanceData_UEB(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UE_Positioning_OTDOA_AssistanceData_UEB, UE_Positioning_OTDOA_AssistanceData_UEB_sequence);

  return offset;
}


static const per_sequence_t AssistanceDataDelivery_r3_IEs_sequence[] = {
  { &hf_rrc_rrc_TransactionIdentifier, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RRC_TransactionIdentifier },
  { &hf_rrc_ue_positioning_GPS_AssistanceData, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_Positioning_GPS_AssistanceData },
  { &hf_rrc_ue_positioning_OTDOA_AssistanceData_UEB, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_Positioning_OTDOA_AssistanceData_UEB },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_AssistanceDataDelivery_r3_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_AssistanceDataDelivery_r3_IEs, AssistanceDataDelivery_r3_IEs_sequence);

  return offset;
}


static const value_string rrc_SFN_Offset_Validity_vals[] = {
  {   0, "false" },
  { 0, NULL }
};


static int
dissect_rrc_SFN_Offset_Validity(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t AssistanceDataDelivery_v3a0ext_sequence[] = {
  { &hf_rrc_sfn_Offset_Validity, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_SFN_Offset_Validity },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_AssistanceDataDelivery_v3a0ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_AssistanceDataDelivery_v3a0ext, AssistanceDataDelivery_v3a0ext_sequence);

  return offset;
}


static const value_string rrc_IP_Spacing_TDD_vals[] = {
  {   0, "e30" },
  {   1, "e40" },
  {   2, "e50" },
  {   3, "e70" },
  {   4, "e100" },
  { 0, NULL }
};


static int
dissect_rrc_IP_Spacing_TDD(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     5, NULL, FALSE, 0, NULL);

  return offset;
}



static int
dissect_rrc_INTEGER_0_14(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            0U, 14U, NULL, FALSE);

  return offset;
}



static int
dissect_rrc_IP_PCCPCH_r4(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_boolean(tvb, offset, actx, tree, hf_index, NULL);

  return offset;
}


static const per_sequence_t UE_Positioning_IPDL_Parameters_TDD_r4_ext_sequence[] = {
  { &hf_rrc_ip_Spacing_01   , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_IP_Spacing_TDD },
  { &hf_rrc_ip_slot         , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_0_14 },
  { &hf_rrc_ip_Start        , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_0_4095 },
  { &hf_rrc_ip_PCCPCG       , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_IP_PCCPCH_r4 },
  { &hf_rrc_burstModeParameters, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BurstModeParameters },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UE_Positioning_IPDL_Parameters_TDD_r4_ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UE_Positioning_IPDL_Parameters_TDD_r4_ext, UE_Positioning_IPDL_Parameters_TDD_r4_ext_sequence);

  return offset;
}


static const per_sequence_t UE_Positioning_IPDL_Parameters_TDDList_r4_ext_sequence_of[1] = {
  { &hf_rrc_UE_Positioning_IPDL_Parameters_TDDList_r4_ext_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UE_Positioning_IPDL_Parameters_TDD_r4_ext },
};

static int
dissect_rrc_UE_Positioning_IPDL_Parameters_TDDList_r4_ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_UE_Positioning_IPDL_Parameters_TDDList_r4_ext, UE_Positioning_IPDL_Parameters_TDDList_r4_ext_sequence_of,
                                                  1, maxCellMeas, FALSE);

  return offset;
}


static const per_sequence_t UE_Positioning_OTDOA_AssistanceData_r4ext_sequence[] = {
  { &hf_rrc_ue_Positioning_IPDL_Parameters_TDD_r4_ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_Positioning_IPDL_Parameters_TDD_r4_ext },
  { &hf_rrc_ue_Positioning_IPDL_Parameters_TDDList_r4_ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_Positioning_IPDL_Parameters_TDDList_r4_ext },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UE_Positioning_OTDOA_AssistanceData_r4ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UE_Positioning_OTDOA_AssistanceData_r4ext, UE_Positioning_OTDOA_AssistanceData_r4ext_sequence);

  return offset;
}


static const per_sequence_t AssistanceDataDelivery_v4b0ext_IEs_sequence[] = {
  { &hf_rrc_ue_Positioning_OTDOA_AssistanceData_r4ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_Positioning_OTDOA_AssistanceData_r4ext },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_AssistanceDataDelivery_v4b0ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_AssistanceDataDelivery_v4b0ext_IEs, AssistanceDataDelivery_v4b0ext_IEs_sequence);

  return offset;
}



static int
dissect_rrc_INTEGER_0_70274(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            0U, 70274U, NULL, FALSE);

  return offset;
}


static const per_sequence_t UE_Positioning_OTDOA_ReferenceCellInfo_UEB_ext_sequence[] = {
  { &hf_rrc_roundTripTimeExtension, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_INTEGER_0_70274 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UE_Positioning_OTDOA_ReferenceCellInfo_UEB_ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UE_Positioning_OTDOA_ReferenceCellInfo_UEB_ext, UE_Positioning_OTDOA_ReferenceCellInfo_UEB_ext_sequence);

  return offset;
}


static const per_sequence_t UE_Positioning_OTDOA_NeighbourCellInfo_UEB_ext_sequence[] = {
  { &hf_rrc_roundTripTimeExtension, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_INTEGER_0_70274 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UE_Positioning_OTDOA_NeighbourCellInfo_UEB_ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UE_Positioning_OTDOA_NeighbourCellInfo_UEB_ext, UE_Positioning_OTDOA_NeighbourCellInfo_UEB_ext_sequence);

  return offset;
}


static const per_sequence_t UE_Positioning_OTDOA_NeighbourCellList_UEB_ext_sequence_of[1] = {
  { &hf_rrc_UE_Positioning_OTDOA_NeighbourCellList_UEB_ext_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UE_Positioning_OTDOA_NeighbourCellInfo_UEB_ext },
};

static int
dissect_rrc_UE_Positioning_OTDOA_NeighbourCellList_UEB_ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_UE_Positioning_OTDOA_NeighbourCellList_UEB_ext, UE_Positioning_OTDOA_NeighbourCellList_UEB_ext_sequence_of,
                                                  1, maxCellMeas, FALSE);

  return offset;
}


static const per_sequence_t UE_Positioning_OTDOA_AssistanceData_UEB_ext_sequence[] = {
  { &hf_rrc_ue_positioning_OTDOA_ReferenceCellInfo_UEB_ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_Positioning_OTDOA_ReferenceCellInfo_UEB_ext },
  { &hf_rrc_ue_positioning_OTDOA_NeighbourCellList_UEB_ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_Positioning_OTDOA_NeighbourCellList_UEB_ext },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UE_Positioning_OTDOA_AssistanceData_UEB_ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UE_Positioning_OTDOA_AssistanceData_UEB_ext, UE_Positioning_OTDOA_AssistanceData_UEB_ext_sequence);

  return offset;
}



static int
dissect_rrc_UE_Positioning_GPS_ReferenceTimeUncertainty(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            0U, 127U, NULL, FALSE);

  return offset;
}


static const per_sequence_t UE_Positioning_GPS_ReferenceTime_v770ext_sequence[] = {
  { &hf_rrc_ue_Positioning_GPS_ReferenceTimeUncertainty, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_Positioning_GPS_ReferenceTimeUncertainty },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UE_Positioning_GPS_ReferenceTime_v770ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UE_Positioning_GPS_ReferenceTime_v770ext, UE_Positioning_GPS_ReferenceTime_v770ext_sequence);

  return offset;
}


static const per_sequence_t UE_Positioning_GPS_AcquisitionAssistance_v770ext_sequence[] = {
  { &hf_rrc_ue_Positioning_GPS_ReferenceTimeUncertainty, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_Positioning_GPS_ReferenceTimeUncertainty },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UE_Positioning_GPS_AcquisitionAssistance_v770ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UE_Positioning_GPS_AcquisitionAssistance_v770ext, UE_Positioning_GPS_AcquisitionAssistance_v770ext_sequence);

  return offset;
}


static const per_sequence_t UE_Positioning_GPS_AssistanceData_v770ext_sequence[] = {
  { &hf_rrc_ue_positioning_GPS_ReferenceTime_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_Positioning_GPS_ReferenceTime_v770ext },
  { &hf_rrc_ue_positioning_GPS_AcquisitionAssistance_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_Positioning_GPS_AcquisitionAssistance_v770ext },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UE_Positioning_GPS_AssistanceData_v770ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UE_Positioning_GPS_AssistanceData_v770ext, UE_Positioning_GPS_AssistanceData_v770ext_sequence);

  return offset;
}



static int
dissect_rrc_INTEGER_0_8191(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            0U, 8191U, NULL, FALSE);

  return offset;
}



static int
dissect_rrc_INTEGER_0_86399(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            0U, 86399U, NULL, FALSE);

  return offset;
}



static int
dissect_rrc_INTEGER_0_3999999(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            0U, 3999999U, NULL, FALSE);

  return offset;
}


static const per_sequence_t T_fdd_167_sequence[] = {
  { &hf_rrc_primary_CPICH_Info, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PrimaryCPICH_Info },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_fdd_167(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_fdd_167, T_fdd_167_sequence);

  return offset;
}


static const per_sequence_t T_tdd_116_sequence[] = {
  { &hf_rrc_cellParameters  , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CellParametersID },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_tdd_116(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_tdd_116, T_tdd_116_sequence);

  return offset;
}


static const value_string rrc_T_mode_vals[] = {
  {   0, "fdd" },
  {   1, "tdd" },
  { 0, NULL }
};

static const per_choice_t T_mode_choice[] = {
  {   0, &hf_rrc_fdd_173         , ASN1_NO_EXTENSIONS     , dissect_rrc_T_fdd_167 },
  {   1, &hf_rrc_tdd_124         , ASN1_NO_EXTENSIONS     , dissect_rrc_T_tdd_116 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_mode(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_mode, T_mode_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t T_utran_ganssreferenceTime_sequence[] = {
  { &hf_rrc_timingOfCellFrames, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_0_3999999 },
  { &hf_rrc_mode            , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_mode },
  { &hf_rrc_referenceSfn    , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_0_4095 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_utran_ganssreferenceTime(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_utran_ganssreferenceTime, T_utran_ganssreferenceTime_sequence);

  return offset;
}


static const value_string rrc_Tutran_Ganss_DriftRate_vals[] = {
  {   0, "ugdr0" },
  {   1, "ugdr1" },
  {   2, "ugdr2" },
  {   3, "ugdr5" },
  {   4, "ugdr10" },
  {   5, "ugdr15" },
  {   6, "ugdr25" },
  {   7, "ugdr50" },
  {   8, "ugdr-1" },
  {   9, "ugdr-2" },
  {  10, "ugdr-5" },
  {  11, "ugdr-10" },
  {  12, "ugdr-15" },
  {  13, "ugdr-25" },
  {  14, "ugdr-50" },
  {  15, "spare" },
  { 0, NULL }
};


static int
dissect_rrc_Tutran_Ganss_DriftRate(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     16, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t UE_Positioning_GANSS_ReferenceTime_sequence[] = {
  { &hf_rrc_ganssDay        , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_INTEGER_0_8191 },
  { &hf_rrc_ganssTod        , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_0_86399 },
  { &hf_rrc_ganssTodUncertainty, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_INTEGER_0_127 },
  { &hf_rrc_ganssTimeId     , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_INTEGER_0_7 },
  { &hf_rrc_utran_ganssreferenceTime, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_utran_ganssreferenceTime },
  { &hf_rrc_tutran_ganss_driftRate, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_Tutran_Ganss_DriftRate },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UE_Positioning_GANSS_ReferenceTime(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UE_Positioning_GANSS_ReferenceTime, UE_Positioning_GANSS_ReferenceTime_sequence);

  return offset;
}


static const per_sequence_t ReferenceLocationGANSS_sequence[] = {
  { &hf_rrc_ellipsoidPointAltitudeEllipsoide, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_EllipsoidPointAltitudeEllipsoide },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_ReferenceLocationGANSS(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_ReferenceLocationGANSS, ReferenceLocationGANSS_sequence);

  return offset;
}



static int
dissect_rrc_BIT_STRING_SIZE_12(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_bit_string(tvb, offset, actx, tree, hf_index,
                                     12, 12, FALSE, NULL);

  return offset;
}


static const per_sequence_t GANSS_Storm_Flag_sequence[] = {
  { &hf_rrc_storm_flag_one  , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { &hf_rrc_storm_flag_two  , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { &hf_rrc_storm_flag_three, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { &hf_rrc_storm_flag_four , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { &hf_rrc_storm_flag_five , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_GANSS_Storm_Flag(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_GANSS_Storm_Flag, GANSS_Storm_Flag_sequence);

  return offset;
}


static const per_sequence_t UE_Positioning_GANSS_IonosphericModel_sequence[] = {
  { &hf_rrc_alpha_zero_ionos, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_12 },
  { &hf_rrc_alpha_one_ionos , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_12 },
  { &hf_rrc_alpha_two_ionos , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_12 },
  { &hf_rrc_gANSS_storm_flags, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_GANSS_Storm_Flag },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UE_Positioning_GANSS_IonosphericModel(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UE_Positioning_GANSS_IonosphericModel, UE_Positioning_GANSS_IonosphericModel_sequence);

  return offset;
}



static int
dissect_rrc_INTEGER_0_37799(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            0U, 37799U, NULL, FALSE);

  return offset;
}



static int
dissect_rrc_INTEGER_M2147483648_2147483647(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            G_MININT32, 2147483647U, NULL, FALSE);

  return offset;
}



static int
dissect_rrc_INTEGER_M64_63(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            -64, 63U, NULL, FALSE);

  return offset;
}


static const value_string rrc_T_gnss_to_id_vals[] = {
  {   0, "gps" },
  {   1, "galileo" },
  {   2, "qzss" },
  {   3, "glonass" },
  {   4, "spare4" },
  {   5, "spare3" },
  {   6, "spare2" },
  {   7, "spare1" },
  { 0, NULL }
};


static int
dissect_rrc_T_gnss_to_id(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     8, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t UE_Positioning_GANSS_TimeModel_sequence[] = {
  { &hf_rrc_ganss_timeModelreferenceTime, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_0_37799 },
  { &hf_rrc_ganss_t_a0      , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_M2147483648_2147483647 },
  { &hf_rrc_ganss_t_a1      , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_INTEGER_M8388608_8388607 },
  { &hf_rrc_ganss_t_a2      , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_INTEGER_M64_63 },
  { &hf_rrc_gnss_to_id      , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_gnss_to_id },
  { &hf_rrc_ganss_wk_number_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_INTEGER_0_8191 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UE_Positioning_GANSS_TimeModel(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UE_Positioning_GANSS_TimeModel, UE_Positioning_GANSS_TimeModel_sequence);

  return offset;
}


static const per_sequence_t GANSSTimeModelsList_sequence_of[1] = {
  { &hf_rrc_GANSSTimeModelsList_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UE_Positioning_GANSS_TimeModel },
};

static int
dissect_rrc_GANSSTimeModelsList(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_GANSSTimeModelsList, GANSSTimeModelsList_sequence_of,
                                                  1, maxGANSS_1, FALSE);

  return offset;
}



static int
dissect_rrc_INTEGER_0_119(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            0U, 119U, NULL, FALSE);

  return offset;
}



static int
dissect_rrc_GANSS_Signal_Id(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            0U, 7U, NULL, FALSE);

  return offset;
}


static const value_string rrc_GANSS_Status_Health_vals[] = {
  {   0, "udre-scale-1dot0" },
  {   1, "udre-scale-0dot75" },
  {   2, "udre-scale-0dot5" },
  {   3, "udre-scale-0dot3" },
  {   4, "udre-scale-0dot2" },
  {   5, "udre-scale-0dot1" },
  {   6, "no-data" },
  {   7, "invalid-data" },
  { 0, NULL }
};


static int
dissect_rrc_GANSS_Status_Health(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     8, NULL, FALSE, 0, NULL);

  return offset;
}



static int
dissect_rrc_INTEGER_M2047_2047(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            -2047, 2047U, NULL, FALSE);

  return offset;
}



static int
dissect_rrc_INTEGER_M127_127(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            -127, 127U, NULL, FALSE);

  return offset;
}


static const per_sequence_t DGANSSSignalInformation_sequence[] = {
  { &hf_rrc_satId           , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_0_63 },
  { &hf_rrc_iode_dganss     , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_10 },
  { &hf_rrc_udre            , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UDRE },
  { &hf_rrc_ganss_prc       , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_M2047_2047 },
  { &hf_rrc_ganss_rrc       , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_M127_127 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_DGANSSSignalInformation(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_DGANSSSignalInformation, DGANSSSignalInformation_sequence);

  return offset;
}


static const per_sequence_t DGANSSSignalInformationList_sequence_of[1] = {
  { &hf_rrc_DGANSSSignalInformationList_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DGANSSSignalInformation },
};

static int
dissect_rrc_DGANSSSignalInformationList(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_DGANSSSignalInformationList, DGANSSSignalInformationList_sequence_of,
                                                  1, maxGANSSSat, FALSE);

  return offset;
}


static const per_sequence_t DGANSSInfo_sequence[] = {
  { &hf_rrc_ganssSignalId   , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_GANSS_Signal_Id },
  { &hf_rrc_ganssStatusHealth, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_GANSS_Status_Health },
  { &hf_rrc_dgansssignalInformationList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DGANSSSignalInformationList },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_DGANSSInfo(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_DGANSSInfo, DGANSSInfo_sequence);

  return offset;
}


static const per_sequence_t DGANSSInfoList_sequence_of[1] = {
  { &hf_rrc_DGANSSInfoList_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DGANSSInfo },
};

static int
dissect_rrc_DGANSSInfoList(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_DGANSSInfoList, DGANSSInfoList_sequence_of,
                                                  1, maxSgnType, FALSE);

  return offset;
}


static const per_sequence_t UE_Positioning_DGANSSCorrections_sequence[] = {
  { &hf_rrc_dganssreferencetime, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_0_119 },
  { &hf_rrc_dganssInfoList  , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DGANSSInfoList },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UE_Positioning_DGANSSCorrections(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UE_Positioning_DGANSSCorrections, UE_Positioning_DGANSSCorrections_sequence);

  return offset;
}


static const value_string rrc_T_non_broadcastIndication_01_vals[] = {
  {   0, "true" },
  { 0, NULL }
};


static int
dissect_rrc_T_non_broadcastIndication_01(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}



static int
dissect_rrc_BIT_STRING_SIZE_18(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_bit_string(tvb, offset, actx, tree, hf_index,
                                     18, 18, FALSE, NULL);

  return offset;
}



static int
dissect_rrc_BIT_STRING_SIZE_28(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_bit_string(tvb, offset, actx, tree, hf_index,
                                     28, 28, FALSE, NULL);

  return offset;
}


static const per_sequence_t Satellite_clock_model_sequence[] = {
  { &hf_rrc_t_oc_01         , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_14 },
  { &hf_rrc_af2_01          , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_12 },
  { &hf_rrc_af1_02          , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_18 },
  { &hf_rrc_af0_02          , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_28 },
  { &hf_rrc_tgd_01          , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_BIT_STRING_SIZE_10 },
  { &hf_rrc_model_id        , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_INTEGER_0_1 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_Satellite_clock_model(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_Satellite_clock_model, Satellite_clock_model_sequence);

  return offset;
}


static const per_sequence_t Satellite_clock_modelList_sequence_of[1] = {
  { &hf_rrc_Satellite_clock_modelList_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_Satellite_clock_model },
};

static int
dissect_rrc_Satellite_clock_modelList(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_Satellite_clock_modelList, Satellite_clock_modelList_sequence_of,
                                                  1, maxSatClockModels, FALSE);

  return offset;
}


static const per_sequence_t UE_Positioning_GANSS_ClockModel_sequence[] = {
  { &hf_rrc_satellite_clock_modelList, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_Satellite_clock_modelList },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UE_Positioning_GANSS_ClockModel(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UE_Positioning_GANSS_ClockModel, UE_Positioning_GANSS_ClockModel_sequence);

  return offset;
}


static const per_sequence_t KeplerianParameters_sequence[] = {
  { &hf_rrc_toe_nav         , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_14 },
  { &hf_rrc_ganss_omega_nav , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_32 },
  { &hf_rrc_delta_n_nav     , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_16 },
  { &hf_rrc_m_zero_nav      , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_32 },
  { &hf_rrc_omegadot_nav    , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_24 },
  { &hf_rrc_ganss_e_nav     , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_32 },
  { &hf_rrc_idot_nav        , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_14 },
  { &hf_rrc_a_sqrt_nav      , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_32 },
  { &hf_rrc_i_zero_nav      , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_32 },
  { &hf_rrc_omega_zero_nav  , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_32 },
  { &hf_rrc_c_rs_nav        , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_16 },
  { &hf_rrc_c_is_nav        , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_16 },
  { &hf_rrc_c_us_nav        , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_16 },
  { &hf_rrc_c_rc_nav        , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_16 },
  { &hf_rrc_c_ic_nav        , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_16 },
  { &hf_rrc_c_uc_nav        , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_16 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_KeplerianParameters(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_KeplerianParameters, KeplerianParameters_sequence);

  return offset;
}


static const per_sequence_t UE_Positioning_GANSS_OrbitModel_sequence[] = {
  { &hf_rrc_keplerianParameters, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_KeplerianParameters },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UE_Positioning_GANSS_OrbitModel(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UE_Positioning_GANSS_OrbitModel, UE_Positioning_GANSS_OrbitModel_sequence);

  return offset;
}


static const per_sequence_t Ganss_Sat_Info_Nav_sequence[] = {
  { &hf_rrc_satId           , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_0_63 },
  { &hf_rrc_svHealth_01     , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_5 },
  { &hf_rrc_iod_01          , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_10 },
  { &hf_rrc_ganssClockModel_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UE_Positioning_GANSS_ClockModel },
  { &hf_rrc_ganssOrbitModel_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UE_Positioning_GANSS_OrbitModel },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_Ganss_Sat_Info_Nav(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_Ganss_Sat_Info_Nav, Ganss_Sat_Info_Nav_sequence);

  return offset;
}


static const per_sequence_t Ganss_Sat_Info_NavList_sequence_of[1] = {
  { &hf_rrc_Ganss_Sat_Info_NavList_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_Ganss_Sat_Info_Nav },
};

static int
dissect_rrc_Ganss_Sat_Info_NavList(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_Ganss_Sat_Info_NavList, Ganss_Sat_Info_NavList_sequence_of,
                                                  1, maxGANSSSat, FALSE);

  return offset;
}


static const per_sequence_t UE_Positioning_GANSS_NavigationModel_sequence[] = {
  { &hf_rrc_non_broadcastIndication_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_non_broadcastIndication_01 },
  { &hf_rrc_ganssSatInfoNavList_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_Ganss_Sat_Info_NavList },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UE_Positioning_GANSS_NavigationModel(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UE_Positioning_GANSS_NavigationModel, UE_Positioning_GANSS_NavigationModel_sequence);

  return offset;
}


static const per_sequence_t UE_Positioning_GANSS_RealTimeIntegrity_item_sequence[] = {
  { &hf_rrc_bad_ganss_satId , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_0_63 },
  { &hf_rrc_bad_ganss_signalId, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_BIT_STRING_SIZE_8 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UE_Positioning_GANSS_RealTimeIntegrity_item(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UE_Positioning_GANSS_RealTimeIntegrity_item, UE_Positioning_GANSS_RealTimeIntegrity_item_sequence);

  return offset;
}


static const per_sequence_t UE_Positioning_GANSS_RealTimeIntegrity_sequence_of[1] = {
  { &hf_rrc_UE_Positioning_GANSS_RealTimeIntegrity_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UE_Positioning_GANSS_RealTimeIntegrity_item },
};

static int
dissect_rrc_UE_Positioning_GANSS_RealTimeIntegrity(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_UE_Positioning_GANSS_RealTimeIntegrity, UE_Positioning_GANSS_RealTimeIntegrity_sequence_of,
                                                  1, maxGANSSSat, FALSE);

  return offset;
}



static int
dissect_rrc_INTEGER_0_59(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            0U, 59U, NULL, FALSE);

  return offset;
}



static int
dissect_rrc_BIT_STRING_SIZE_1_1024(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_bit_string(tvb, offset, actx, tree, hf_index,
                                     1, 1024, FALSE, NULL);

  return offset;
}


static const per_sequence_t DataBitAssistance_sequence[] = {
  { &hf_rrc_ganss_signal_id , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_GANSS_Signal_Id },
  { &hf_rrc_data_bits       , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_1_1024 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_DataBitAssistance(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_DataBitAssistance, DataBitAssistance_sequence);

  return offset;
}


static const per_sequence_t DataBitAssistanceSgnList_sequence_of[1] = {
  { &hf_rrc_DataBitAssistanceSgnList_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DataBitAssistance },
};

static int
dissect_rrc_DataBitAssistanceSgnList(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_DataBitAssistanceSgnList, DataBitAssistanceSgnList_sequence_of,
                                                  1, maxSgnType, FALSE);

  return offset;
}


static const per_sequence_t DataBitAssistanceSat_sequence[] = {
  { &hf_rrc_satID_01        , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_0_63 },
  { &hf_rrc_dataBitAssistanceSgnList, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DataBitAssistanceSgnList },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_DataBitAssistanceSat(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_DataBitAssistanceSat, DataBitAssistanceSat_sequence);

  return offset;
}


static const per_sequence_t DataBitAssistanceList_sequence_of[1] = {
  { &hf_rrc_DataBitAssistanceList_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DataBitAssistanceSat },
};

static int
dissect_rrc_DataBitAssistanceList(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_DataBitAssistanceList, DataBitAssistanceList_sequence_of,
                                                  1, maxGANSSSat, FALSE);

  return offset;
}


static const per_sequence_t UE_Positioning_GANSS_Data_Bit_Assistance_sequence[] = {
  { &hf_rrc_ganss_tod       , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_0_59 },
  { &hf_rrc_dataBitAssistanceList, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DataBitAssistanceList },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UE_Positioning_GANSS_Data_Bit_Assistance(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UE_Positioning_GANSS_Data_Bit_Assistance, UE_Positioning_GANSS_Data_Bit_Assistance_sequence);

  return offset;
}


static const value_string rrc_T_dopplerUncertainty_vals[] = {
  {   0, "dopU40" },
  {   1, "dopU20" },
  {   2, "dopU10" },
  {   3, "dopU5" },
  {   4, "dopU2-5" },
  {   5, "spare3" },
  {   6, "spare2" },
  {   7, "spare1" },
  { 0, NULL }
};


static int
dissect_rrc_T_dopplerUncertainty(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     8, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t ExtraDoppler_sequence[] = {
  { &hf_rrc_dopplerFirstOrder, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_M42_21 },
  { &hf_rrc_dopplerUncertainty, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_dopplerUncertainty },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_ExtraDoppler(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_ExtraDoppler, ExtraDoppler_sequence);

  return offset;
}


static const per_sequence_t GANSSSatelliteInformation_sequence[] = {
  { &hf_rrc_ganssSatId      , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_0_63 },
  { &hf_rrc_dopplerZeroOrder, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_M2048_2047 },
  { &hf_rrc_extraDoppler    , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_ExtraDoppler },
  { &hf_rrc_codePhase_01    , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_0_1023 },
  { &hf_rrc_integerCodePhase_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_0_127 },
  { &hf_rrc_codePhaseSearchWindow_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_5 },
  { &hf_rrc_azimuthandElevation, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_AzimuthAndElevation },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_GANSSSatelliteInformation(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_GANSSSatelliteInformation, GANSSSatelliteInformation_sequence);

  return offset;
}


static const per_sequence_t GANSSSatelliteInformationList_sequence_of[1] = {
  { &hf_rrc_GANSSSatelliteInformationList_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_GANSSSatelliteInformation },
};

static int
dissect_rrc_GANSSSatelliteInformationList(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_GANSSSatelliteInformationList, GANSSSatelliteInformationList_sequence_of,
                                                  1, maxGANSSSat, FALSE);

  return offset;
}


static const per_sequence_t UE_Positioning_GANSS_ReferenceMeasurementInfo_sequence[] = {
  { &hf_rrc_ganssSignalId   , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_GANSS_Signal_Id },
  { &hf_rrc_satelliteInformationList, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_GANSSSatelliteInformationList },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UE_Positioning_GANSS_ReferenceMeasurementInfo(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UE_Positioning_GANSS_ReferenceMeasurementInfo, UE_Positioning_GANSS_ReferenceMeasurementInfo_sequence);

  return offset;
}



static int
dissect_rrc_INTEGER_0_147(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            0U, 147U, NULL, FALSE);

  return offset;
}



static int
dissect_rrc_BIT_STRING_SIZE_17(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_bit_string(tvb, offset, actx, tree, hf_index,
                                     17, 17, FALSE, NULL);

  return offset;
}


static const per_sequence_t GANSS_SAT_Info_Almanac_Kp_sequence[] = {
  { &hf_rrc_svId            , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_0_63 },
  { &hf_rrc_ganss_alm_e     , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_11 },
  { &hf_rrc_ganss_delta_I_alm, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_11 },
  { &hf_rrc_ganss_omegadot_alm, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_11 },
  { &hf_rrc_ganss_svhealth_alm, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_4 },
  { &hf_rrc_ganss_delta_a_sqrt_alm, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_17 },
  { &hf_rrc_ganss_omegazero_alm, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_16 },
  { &hf_rrc_ganss_m_zero_alm, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_16 },
  { &hf_rrc_ganss_omega_alm , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_16 },
  { &hf_rrc_ganss_af_zero_alm, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_14 },
  { &hf_rrc_ganss_af_one_alm, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_11 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_GANSS_SAT_Info_Almanac_Kp(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_GANSS_SAT_Info_Almanac_Kp, GANSS_SAT_Info_Almanac_Kp_sequence);

  return offset;
}


static const per_sequence_t GANSS_SAT_Info_Almanac_KpList_sequence_of[1] = {
  { &hf_rrc_GANSS_SAT_Info_Almanac_KpList_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_GANSS_SAT_Info_Almanac_Kp },
};

static int
dissect_rrc_GANSS_SAT_Info_Almanac_KpList(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_GANSS_SAT_Info_Almanac_KpList, GANSS_SAT_Info_Almanac_KpList_sequence_of,
                                                  1, maxGANSSSat, FALSE);

  return offset;
}


static const per_sequence_t ALM_keplerianParameters_sequence[] = {
  { &hf_rrc_t_oa_01         , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_0_147 },
  { &hf_rrc_iod_a           , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_0_3 },
  { &hf_rrc_sat_info_kpList , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_GANSS_SAT_Info_Almanac_KpList },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_ALM_keplerianParameters(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_ALM_keplerianParameters, ALM_keplerianParameters_sequence);

  return offset;
}


static const per_sequence_t UE_Positioning_GANSS_Almanac_sequence[] = {
  { &hf_rrc_ganss_wk_number , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_0_255 },
  { &hf_rrc_alm_keplerianParameters, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_ALM_keplerianParameters },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UE_Positioning_GANSS_Almanac(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UE_Positioning_GANSS_Almanac, UE_Positioning_GANSS_Almanac_sequence);

  return offset;
}


static const per_sequence_t UE_Positioning_GANSS_UTCModel_sequence[] = {
  { &hf_rrc_a_one_utc       , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_24 },
  { &hf_rrc_a_zero_utc      , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_32 },
  { &hf_rrc_t_ot_utc        , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_8 },
  { &hf_rrc_delta_t_ls_utc  , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_8 },
  { &hf_rrc_w_n_t_utc       , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_8 },
  { &hf_rrc_w_n_lsf_utc     , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_8 },
  { &hf_rrc_dn_utc          , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_8 },
  { &hf_rrc_delta_t_lsf_utc , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_8 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UE_Positioning_GANSS_UTCModel(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UE_Positioning_GANSS_UTCModel, UE_Positioning_GANSS_UTCModel_sequence);

  return offset;
}


static const per_sequence_t GANSSGenericData_sequence[] = {
  { &hf_rrc_ganssId         , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_INTEGER_0_7 },
  { &hf_rrc_ganssTimeModelsList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_GANSSTimeModelsList },
  { &hf_rrc_uePositioningDGANSSCorrections, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_Positioning_DGANSSCorrections },
  { &hf_rrc_uePositioningGANSSNavigationModel, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_Positioning_GANSS_NavigationModel },
  { &hf_rrc_uePositioningGANSSRealTimeIntegrity, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_Positioning_GANSS_RealTimeIntegrity },
  { &hf_rrc_uePositioningGANSSDataBitAssistance, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_Positioning_GANSS_Data_Bit_Assistance },
  { &hf_rrc_uePositioningGANSSReferenceMeasurementInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_Positioning_GANSS_ReferenceMeasurementInfo },
  { &hf_rrc_uePositioningGANSSAlmanac, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_Positioning_GANSS_Almanac },
  { &hf_rrc_uePositioningGANSSUTCModel, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_Positioning_GANSS_UTCModel },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_GANSSGenericData(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_GANSSGenericData, GANSSGenericData_sequence);

  return offset;
}


static const per_sequence_t GANSSGenericDataList_sequence_of[1] = {
  { &hf_rrc_GANSSGenericDataList_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_GANSSGenericData },
};

static int
dissect_rrc_GANSSGenericDataList(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_GANSSGenericDataList, GANSSGenericDataList_sequence_of,
                                                  1, maxGANSS, FALSE);

  return offset;
}


static const per_sequence_t UE_Positioning_GANSS_AssistanceData_sequence[] = {
  { &hf_rrc_ue_positioning_GANSS_ReferenceTime, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_Positioning_GANSS_ReferenceTime },
  { &hf_rrc_uePositioningGanssReferencePosition, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_ReferenceLocationGANSS },
  { &hf_rrc_uePositioningGanssIonosphericModel, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_Positioning_GANSS_IonosphericModel },
  { &hf_rrc_ganssGenericDataList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_GANSSGenericDataList },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UE_Positioning_GANSS_AssistanceData(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UE_Positioning_GANSS_AssistanceData, UE_Positioning_GANSS_AssistanceData_sequence);

  return offset;
}


static const per_sequence_t AssistanceDataDelivery_v770ext_IEs_sequence[] = {
  { &hf_rrc_ue_Positioning_OTDOA_AssistanceData_UEB_ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_Positioning_OTDOA_AssistanceData_UEB_ext },
  { &hf_rrc_ue_Positioning_GPS_AssistanceData, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_Positioning_GPS_AssistanceData_v770ext },
  { &hf_rrc_ue_positioning_GANSS_AssistanceData, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_Positioning_GANSS_AssistanceData },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_AssistanceDataDelivery_v770ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_AssistanceDataDelivery_v770ext_IEs, AssistanceDataDelivery_v770ext_IEs_sequence);

  return offset;
}


static const per_sequence_t UE_Positioning_GANSS_AddIonoModel_sequence[] = {
  { &hf_rrc_dataID_01       , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_2 },
  { &hf_rrc_alpha_beta_parameters, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UE_Positioning_GPS_IonosphericModel },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UE_Positioning_GANSS_AddIonoModel(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UE_Positioning_GANSS_AddIonoModel, UE_Positioning_GANSS_AddIonoModel_sequence);

  return offset;
}



static int
dissect_rrc_BIT_STRING_SIZE_21(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_bit_string(tvb, offset, actx, tree, hf_index,
                                     21, 21, FALSE, NULL);

  return offset;
}



static int
dissect_rrc_BIT_STRING_SIZE_15(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_bit_string(tvb, offset, actx, tree, hf_index,
                                     15, 15, FALSE, NULL);

  return offset;
}



static int
dissect_rrc_BIT_STRING_SIZE_31(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_bit_string(tvb, offset, actx, tree, hf_index,
                                     31, 31, FALSE, NULL);

  return offset;
}



static int
dissect_rrc_BIT_STRING_SIZE_19(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_bit_string(tvb, offset, actx, tree, hf_index,
                                     19, 19, FALSE, NULL);

  return offset;
}


static const per_sequence_t UE_Positioning_GANSS_EarthOrientPara_sequence[] = {
  { &hf_rrc_teop            , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_16 },
  { &hf_rrc_pmX             , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_21 },
  { &hf_rrc_pmXdot          , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_15 },
  { &hf_rrc_pmY             , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_21 },
  { &hf_rrc_pmYdot          , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_15 },
  { &hf_rrc_deltaUT1        , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_31 },
  { &hf_rrc_deltaUT1dot     , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_19 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UE_Positioning_GANSS_EarthOrientPara(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UE_Positioning_GANSS_EarthOrientPara, UE_Positioning_GANSS_EarthOrientPara_sequence);

  return offset;
}


static const value_string rrc_UE_Positioning_GANSS_SBAS_ID_vals[] = {
  {   0, "waas" },
  {   1, "egnos" },
  {   2, "msas" },
  {   3, "gagan" },
  {   4, "spare4" },
  {   5, "spare3" },
  {   6, "spare2" },
  {   7, "spare1" },
  { 0, NULL }
};


static int
dissect_rrc_UE_Positioning_GANSS_SBAS_ID(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     8, NULL, FALSE, 0, NULL);

  return offset;
}


static const value_string rrc_T_non_broadcastIndication_vals[] = {
  {   0, "true" },
  { 0, NULL }
};


static int
dissect_rrc_T_non_broadcastIndication(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t NAVclockModel_sequence[] = {
  { &hf_rrc_navToc          , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_16 },
  { &hf_rrc_navaf2          , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_8 },
  { &hf_rrc_navaf1          , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_16 },
  { &hf_rrc_navaf0          , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_22 },
  { &hf_rrc_navTgd          , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_8 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_NAVclockModel(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_NAVclockModel, NAVclockModel_sequence);

  return offset;
}



static int
dissect_rrc_BIT_STRING_SIZE_3(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_bit_string(tvb, offset, actx, tree, hf_index,
                                     3, 3, FALSE, NULL);

  return offset;
}



static int
dissect_rrc_BIT_STRING_SIZE_20(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_bit_string(tvb, offset, actx, tree, hf_index,
                                     20, 20, FALSE, NULL);

  return offset;
}



static int
dissect_rrc_BIT_STRING_SIZE_26(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_bit_string(tvb, offset, actx, tree, hf_index,
                                     26, 26, FALSE, NULL);

  return offset;
}



static int
dissect_rrc_BIT_STRING_SIZE_13(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_bit_string(tvb, offset, actx, tree, hf_index,
                                     13, 13, FALSE, NULL);

  return offset;
}


static const per_sequence_t CNAVclockModel_sequence[] = {
  { &hf_rrc_cnavToc         , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_11 },
  { &hf_rrc_cnavTop         , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_11 },
  { &hf_rrc_cnavURA0        , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_5 },
  { &hf_rrc_cnavURA1        , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_3 },
  { &hf_rrc_cnavURA2        , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_3 },
  { &hf_rrc_cnavAf2         , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_10 },
  { &hf_rrc_cnavAf1         , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_20 },
  { &hf_rrc_cnavAf0         , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_26 },
  { &hf_rrc_cnavTgd         , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_13 },
  { &hf_rrc_cnavISCl1cp     , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_BIT_STRING_SIZE_13 },
  { &hf_rrc_cnavISCl1cd     , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_BIT_STRING_SIZE_13 },
  { &hf_rrc_cnavISCl1ca     , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_BIT_STRING_SIZE_13 },
  { &hf_rrc_cnavISCl2c      , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_BIT_STRING_SIZE_13 },
  { &hf_rrc_cnavISCl5i5     , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_BIT_STRING_SIZE_13 },
  { &hf_rrc_cnavISCl5q5     , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_BIT_STRING_SIZE_13 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_CNAVclockModel(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_CNAVclockModel, CNAVclockModel_sequence);

  return offset;
}


static const per_sequence_t GLONASSclockModel_sequence[] = {
  { &hf_rrc_gloTau          , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_22 },
  { &hf_rrc_gloGamma        , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_11 },
  { &hf_rrc_gloDeltaTau     , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_BIT_STRING_SIZE_5 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_GLONASSclockModel(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_GLONASSclockModel, GLONASSclockModel_sequence);

  return offset;
}


static const per_sequence_t SBASclockModel_sequence[] = {
  { &hf_rrc_sbasTo          , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_13 },
  { &hf_rrc_sbasAgfo        , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_12 },
  { &hf_rrc_sbasAgf1        , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_8 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_SBASclockModel(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_SBASclockModel, SBASclockModel_sequence);

  return offset;
}


static const per_sequence_t UE_Positioning_GANSS_AddClockModels_sequence[] = {
  { &hf_rrc_navClockModel   , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_NAVclockModel },
  { &hf_rrc_cnavClockModel  , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CNAVclockModel },
  { &hf_rrc_glonassClockModel, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_GLONASSclockModel },
  { &hf_rrc_sbasClockModel  , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_SBASclockModel },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UE_Positioning_GANSS_AddClockModels(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UE_Positioning_GANSS_AddClockModels, UE_Positioning_GANSS_AddClockModels_sequence);

  return offset;
}


static const per_sequence_t NavModel_NAVKeplerianSet_sequence[] = {
  { &hf_rrc_navURA          , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_4 },
  { &hf_rrc_navFitFlag      , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_1 },
  { &hf_rrc_navToe          , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_16 },
  { &hf_rrc_navOmega        , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_32 },
  { &hf_rrc_navDeltaN       , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_16 },
  { &hf_rrc_navM0           , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_32 },
  { &hf_rrc_navOmegaADot    , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_24 },
  { &hf_rrc_navE            , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_32 },
  { &hf_rrc_navIDot         , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_14 },
  { &hf_rrc_navAPowerHalf   , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_32 },
  { &hf_rrc_navI0           , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_32 },
  { &hf_rrc_navOmegaA0      , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_32 },
  { &hf_rrc_navCrs          , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_16 },
  { &hf_rrc_navCis          , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_16 },
  { &hf_rrc_navCus          , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_16 },
  { &hf_rrc_navCrc          , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_16 },
  { &hf_rrc_navCic          , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_16 },
  { &hf_rrc_navCuc          , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_16 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_NavModel_NAVKeplerianSet(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_NavModel_NAVKeplerianSet, NavModel_NAVKeplerianSet_sequence);

  return offset;
}



static int
dissect_rrc_BIT_STRING_SIZE_25(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_bit_string(tvb, offset, actx, tree, hf_index,
                                     25, 25, FALSE, NULL);

  return offset;
}



static int
dissect_rrc_BIT_STRING_SIZE_33(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_bit_string(tvb, offset, actx, tree, hf_index,
                                     33, 33, FALSE, NULL);

  return offset;
}


static const per_sequence_t NavModel_CNAVKeplerianSet_sequence[] = {
  { &hf_rrc_cnavTop         , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_11 },
  { &hf_rrc_cnavURAindex    , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_5 },
  { &hf_rrc_cnavDeltaA      , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_26 },
  { &hf_rrc_cnavAdot        , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_25 },
  { &hf_rrc_cnavDeltaNo     , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_17 },
  { &hf_rrc_cnavDeltaNoDot  , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_23 },
  { &hf_rrc_cnavMo          , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_33 },
  { &hf_rrc_cnavE           , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_33 },
  { &hf_rrc_cnavOmega       , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_33 },
  { &hf_rrc_cnavOMEGA0      , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_33 },
  { &hf_rrc_cnavDeltaOmegaDot, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_17 },
  { &hf_rrc_cnavIo          , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_33 },
  { &hf_rrc_cnavIoDot       , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_15 },
  { &hf_rrc_cnavCis         , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_16 },
  { &hf_rrc_cnavCic         , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_16 },
  { &hf_rrc_cnavCrs         , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_24 },
  { &hf_rrc_cnavCrc         , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_24 },
  { &hf_rrc_cnavCus         , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_21 },
  { &hf_rrc_cnavCuc         , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_21 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_NavModel_CNAVKeplerianSet(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_NavModel_CNAVKeplerianSet, NavModel_CNAVKeplerianSet_sequence);

  return offset;
}



static int
dissect_rrc_BIT_STRING_SIZE_27(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_bit_string(tvb, offset, actx, tree, hf_index,
                                     27, 27, FALSE, NULL);

  return offset;
}


static const per_sequence_t NavModel_GLONASSecef_sequence[] = {
  { &hf_rrc_gloEn           , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_5 },
  { &hf_rrc_gloP1           , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_2 },
  { &hf_rrc_gloP2           , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_1 },
  { &hf_rrc_gloM            , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_BIT_STRING_SIZE_2 },
  { &hf_rrc_gloX            , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_27 },
  { &hf_rrc_gloXdot         , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_24 },
  { &hf_rrc_gloXdotdot      , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_5 },
  { &hf_rrc_gloY            , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_27 },
  { &hf_rrc_gloYdot         , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_24 },
  { &hf_rrc_gloYdotdot      , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_5 },
  { &hf_rrc_gloZ            , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_27 },
  { &hf_rrc_gloZdot         , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_24 },
  { &hf_rrc_gloZdotdot      , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_5 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_NavModel_GLONASSecef(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_NavModel_GLONASSecef, NavModel_GLONASSecef_sequence);

  return offset;
}



static int
dissect_rrc_BIT_STRING_SIZE_30(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_bit_string(tvb, offset, actx, tree, hf_index,
                                     30, 30, FALSE, NULL);

  return offset;
}


static const per_sequence_t NavModel_SBASecef_sequence[] = {
  { &hf_rrc_sbasTo          , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_BIT_STRING_SIZE_13 },
  { &hf_rrc_sbasAccuracy    , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_4 },
  { &hf_rrc_sbasXg          , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_30 },
  { &hf_rrc_sbasYg          , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_30 },
  { &hf_rrc_sbasZg          , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_25 },
  { &hf_rrc_sbasXgDot       , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_17 },
  { &hf_rrc_sbasYgDot       , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_17 },
  { &hf_rrc_sbasZgDot       , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_18 },
  { &hf_rrc_sbasXgDotDot    , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_10 },
  { &hf_rrc_sbagYgDotDot    , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_10 },
  { &hf_rrc_sbasZgDotDot    , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_10 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_NavModel_SBASecef(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_NavModel_SBASecef, NavModel_SBASecef_sequence);

  return offset;
}


static const per_sequence_t UE_Positioning_GANSS_AddOrbitModels_sequence[] = {
  { &hf_rrc_navKeplerianSet , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_NavModel_NAVKeplerianSet },
  { &hf_rrc_cnavKeplerianSet, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_NavModel_CNAVKeplerianSet },
  { &hf_rrc_glonassECEF     , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_NavModel_GLONASSecef },
  { &hf_rrc_sbasECEF        , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_NavModel_SBASecef },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UE_Positioning_GANSS_AddOrbitModels(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UE_Positioning_GANSS_AddOrbitModels, UE_Positioning_GANSS_AddOrbitModels_sequence);

  return offset;
}


static const per_sequence_t Ganss_Sat_Info_AddNav_sequence[] = {
  { &hf_rrc_satId           , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_0_63 },
  { &hf_rrc_svHealth        , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_6 },
  { &hf_rrc_iod             , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_11 },
  { &hf_rrc_ganssClockModel , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UE_Positioning_GANSS_AddClockModels },
  { &hf_rrc_ganssOrbitModel , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UE_Positioning_GANSS_AddOrbitModels },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_Ganss_Sat_Info_AddNav(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_Ganss_Sat_Info_AddNav, Ganss_Sat_Info_AddNav_sequence);

  return offset;
}


static const per_sequence_t Ganss_Sat_Info_AddNavList_sequence_of[1] = {
  { &hf_rrc_Ganss_Sat_Info_AddNavList_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_Ganss_Sat_Info_AddNav },
};

static int
dissect_rrc_Ganss_Sat_Info_AddNavList(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_Ganss_Sat_Info_AddNavList, Ganss_Sat_Info_AddNavList_sequence_of,
                                                  1, maxGANSSSat, FALSE);

  return offset;
}


static const per_sequence_t UE_Positioning_GANSS_AddNavigationModels_sequence[] = {
  { &hf_rrc_non_broadcastIndication, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_non_broadcastIndication },
  { &hf_rrc_ganssSatInfoNavList, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_Ganss_Sat_Info_AddNavList },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UE_Positioning_GANSS_AddNavigationModels(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UE_Positioning_GANSS_AddNavigationModels, UE_Positioning_GANSS_AddNavigationModels_sequence);

  return offset;
}


static const per_sequence_t GANSS_SAT_Info_Almanac_NAVkp_sequence[] = {
  { &hf_rrc_svID            , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_0_63 },
  { &hf_rrc_navAlmE         , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_16 },
  { &hf_rrc_navAlmDeltaI    , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_16 },
  { &hf_rrc_navAlmOMEGADOT  , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_16 },
  { &hf_rrc_navAlmSVHealth  , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_8 },
  { &hf_rrc_navAlmSqrtA     , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_24 },
  { &hf_rrc_navAlmOMEGAo    , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_24 },
  { &hf_rrc_navAlmOmega     , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_24 },
  { &hf_rrc_navAlmMo        , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_24 },
  { &hf_rrc_navAlmaf0       , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_11 },
  { &hf_rrc_navAlmaf1       , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_11 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_GANSS_SAT_Info_Almanac_NAVkp(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_GANSS_SAT_Info_Almanac_NAVkp, GANSS_SAT_Info_Almanac_NAVkp_sequence);

  return offset;
}


static const per_sequence_t GANSS_SAT_Info_Almanac_NAVkpList_sequence_of[1] = {
  { &hf_rrc_GANSS_SAT_Info_Almanac_NAVkpList_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_GANSS_SAT_Info_Almanac_NAVkp },
};

static int
dissect_rrc_GANSS_SAT_Info_Almanac_NAVkpList(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_GANSS_SAT_Info_Almanac_NAVkpList, GANSS_SAT_Info_Almanac_NAVkpList_sequence_of,
                                                  1, maxGANSSSat, FALSE);

  return offset;
}


static const per_sequence_t ALM_NAVKeplerianSet_sequence[] = {
  { &hf_rrc_t_oa_01         , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_0_147 },
  { &hf_rrc_sat_info_NAVkpList, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_GANSS_SAT_Info_Almanac_NAVkpList },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_ALM_NAVKeplerianSet(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_ALM_NAVKeplerianSet, ALM_NAVKeplerianSet_sequence);

  return offset;
}



static int
dissect_rrc_BIT_STRING_SIZE_7(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_bit_string(tvb, offset, actx, tree, hf_index,
                                     7, 7, FALSE, NULL);

  return offset;
}


static const per_sequence_t GANSS_SAT_Info_Almanac_REDkp_sequence[] = {
  { &hf_rrc_svID            , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_0_63 },
  { &hf_rrc_redAlmDeltaA    , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_8 },
  { &hf_rrc_redAlmOmega0    , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_7 },
  { &hf_rrc_redAlmPhi0      , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_7 },
  { &hf_rrc_redAlmL1Health  , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_1 },
  { &hf_rrc_redAlmL2Health  , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_1 },
  { &hf_rrc_redAlmL5Health  , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_1 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_GANSS_SAT_Info_Almanac_REDkp(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_GANSS_SAT_Info_Almanac_REDkp, GANSS_SAT_Info_Almanac_REDkp_sequence);

  return offset;
}


static const per_sequence_t GANSS_SAT_Info_Almanac_REDkpList_sequence_of[1] = {
  { &hf_rrc_GANSS_SAT_Info_Almanac_REDkpList_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_GANSS_SAT_Info_Almanac_REDkp },
};

static int
dissect_rrc_GANSS_SAT_Info_Almanac_REDkpList(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_GANSS_SAT_Info_Almanac_REDkpList, GANSS_SAT_Info_Almanac_REDkpList_sequence_of,
                                                  1, maxGANSSSat, FALSE);

  return offset;
}


static const per_sequence_t ALM_ReducedKeplerianSet_sequence[] = {
  { &hf_rrc_t_oa_01         , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_0_147 },
  { &hf_rrc_sat_info_REDkpList, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_GANSS_SAT_Info_Almanac_REDkpList },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_ALM_ReducedKeplerianSet(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_ALM_ReducedKeplerianSet, ALM_ReducedKeplerianSet_sequence);

  return offset;
}


static const per_sequence_t GANSS_SAT_Info_Almanac_MIDIkp_sequence[] = {
  { &hf_rrc_svID            , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_0_63 },
  { &hf_rrc_midiAlmE        , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_11 },
  { &hf_rrc_midiAlmDeltaI   , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_11 },
  { &hf_rrc_midiAlmOmegaDot , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_11 },
  { &hf_rrc_midiAlmSqrtA    , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_17 },
  { &hf_rrc_midiAlmOmega0   , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_16 },
  { &hf_rrc_midiAlmOmega    , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_16 },
  { &hf_rrc_midiAlmMo       , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_16 },
  { &hf_rrc_midiAlmaf0      , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_11 },
  { &hf_rrc_midiAlmaf1      , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_10 },
  { &hf_rrc_midiAlmL1Health , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_1 },
  { &hf_rrc_midiAlmL2Health , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_1 },
  { &hf_rrc_midiAlmL5Health , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_1 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_GANSS_SAT_Info_Almanac_MIDIkp(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_GANSS_SAT_Info_Almanac_MIDIkp, GANSS_SAT_Info_Almanac_MIDIkp_sequence);

  return offset;
}


static const per_sequence_t GANSS_SAT_Info_Almanac_MIDIkpList_sequence_of[1] = {
  { &hf_rrc_GANSS_SAT_Info_Almanac_MIDIkpList_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_GANSS_SAT_Info_Almanac_MIDIkp },
};

static int
dissect_rrc_GANSS_SAT_Info_Almanac_MIDIkpList(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_GANSS_SAT_Info_Almanac_MIDIkpList, GANSS_SAT_Info_Almanac_MIDIkpList_sequence_of,
                                                  1, maxGANSSSat, FALSE);

  return offset;
}


static const per_sequence_t ALM_MidiAlmanacSet_sequence[] = {
  { &hf_rrc_t_oa_01         , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_0_147 },
  { &hf_rrc_sat_info_MIDIkpList, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_GANSS_SAT_Info_Almanac_MIDIkpList },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_ALM_MidiAlmanacSet(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_ALM_MidiAlmanacSet, ALM_MidiAlmanacSet_sequence);

  return offset;
}


static const per_sequence_t GANSS_SAT_Info_Almanac_GLOkp_sequence[] = {
  { &hf_rrc_gloAlmNA        , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_11 },
  { &hf_rrc_gloAlmnA        , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_5 },
  { &hf_rrc_gloAlmHA        , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_5 },
  { &hf_rrc_gloAlmLambdaA   , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_21 },
  { &hf_rrc_gloAlmTlambdaA  , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_21 },
  { &hf_rrc_gloAlmDeltaIA   , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_18 },
  { &hf_rrc_gloAkmDeltaTA   , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_22 },
  { &hf_rrc_gloAlmDeltaTdotA, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_7 },
  { &hf_rrc_gloAlmEpsilonA  , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_15 },
  { &hf_rrc_gloAlmOmegaA    , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_16 },
  { &hf_rrc_gloAlmTauA      , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_10 },
  { &hf_rrc_gloAlmCA        , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_1 },
  { &hf_rrc_gloAlmMA        , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_BIT_STRING_SIZE_2 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_GANSS_SAT_Info_Almanac_GLOkp(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_GANSS_SAT_Info_Almanac_GLOkp, GANSS_SAT_Info_Almanac_GLOkp_sequence);

  return offset;
}


static const per_sequence_t GANSS_SAT_Info_Almanac_GLOkpList_sequence_of[1] = {
  { &hf_rrc_GANSS_SAT_Info_Almanac_GLOkpList_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_GANSS_SAT_Info_Almanac_GLOkp },
};

static int
dissect_rrc_GANSS_SAT_Info_Almanac_GLOkpList(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_GANSS_SAT_Info_Almanac_GLOkpList, GANSS_SAT_Info_Almanac_GLOkpList_sequence_of,
                                                  1, maxGANSSSat, FALSE);

  return offset;
}


static const per_sequence_t ALM_GlonassAlmanacSet_sequence[] = {
  { &hf_rrc_sat_info_GLOkpList, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_GANSS_SAT_Info_Almanac_GLOkpList },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_ALM_GlonassAlmanacSet(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_ALM_GlonassAlmanacSet, ALM_GlonassAlmanacSet_sequence);

  return offset;
}



static int
dissect_rrc_BIT_STRING_SIZE_9(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_bit_string(tvb, offset, actx, tree, hf_index,
                                     9, 9, FALSE, NULL);

  return offset;
}


static const per_sequence_t GANSS_SAT_Info_Almanac_SBASecef_sequence[] = {
  { &hf_rrc_sbasAlmDataID   , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_2 },
  { &hf_rrc_svID            , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_0_63 },
  { &hf_rrc_sbasAlmHealth   , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_8 },
  { &hf_rrc_sbasAlmXg       , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_15 },
  { &hf_rrc_sbasAlmYg       , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_15 },
  { &hf_rrc_sbasAlmZg       , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_9 },
  { &hf_rrc_sbasAlmXgdot    , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_3 },
  { &hf_rrc_sbasAlmYgDot    , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_3 },
  { &hf_rrc_sbasAlmZgDot    , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_4 },
  { &hf_rrc_sbasAlmTo       , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_11 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_GANSS_SAT_Info_Almanac_SBASecef(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_GANSS_SAT_Info_Almanac_SBASecef, GANSS_SAT_Info_Almanac_SBASecef_sequence);

  return offset;
}


static const per_sequence_t GANSS_SAT_Info_Almanac_SBASecefList_sequence_of[1] = {
  { &hf_rrc_GANSS_SAT_Info_Almanac_SBASecefList_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_GANSS_SAT_Info_Almanac_SBASecef },
};

static int
dissect_rrc_GANSS_SAT_Info_Almanac_SBASecefList(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_GANSS_SAT_Info_Almanac_SBASecefList, GANSS_SAT_Info_Almanac_SBASecefList_sequence_of,
                                                  1, maxGANSSSat, FALSE);

  return offset;
}


static const per_sequence_t ALM_ECEFsbasAlmanacSet_sequence[] = {
  { &hf_rrc_sat_info_SBASecefList, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_GANSS_SAT_Info_Almanac_SBASecefList },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_ALM_ECEFsbasAlmanacSet(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_ALM_ECEFsbasAlmanacSet, ALM_ECEFsbasAlmanacSet_sequence);

  return offset;
}


static const per_sequence_t UE_Positioning_GANSS_Almanac_v860ext_sequence[] = {
  { &hf_rrc_alm_keplerianNAVAlmanac, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_ALM_NAVKeplerianSet },
  { &hf_rrc_alm_keplerianReducedAlmanac, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_ALM_ReducedKeplerianSet },
  { &hf_rrc_alm_keplerianMidiAlmanac, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_ALM_MidiAlmanacSet },
  { &hf_rrc_alm_keplerianGLONASS, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_ALM_GlonassAlmanacSet },
  { &hf_rrc_alm_ecefSBASAlmanac, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_ALM_ECEFsbasAlmanacSet },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UE_Positioning_GANSS_Almanac_v860ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UE_Positioning_GANSS_Almanac_v860ext, UE_Positioning_GANSS_Almanac_v860ext_sequence);

  return offset;
}


static const per_sequence_t UTCmodelSet1_sequence[] = {
  { &hf_rrc_utcA0           , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_16 },
  { &hf_rrc_utcA1           , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_13 },
  { &hf_rrc_utcA2           , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_7 },
  { &hf_rrc_utcDeltaTls     , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_8 },
  { &hf_rrc_utcTot          , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_16 },
  { &hf_rrc_utcWNot         , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_13 },
  { &hf_rrc_utcWNlsf        , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_8 },
  { &hf_rrc_utcDN           , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_4 },
  { &hf_rrc_utcDeltaTlsf    , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_8 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UTCmodelSet1(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UTCmodelSet1, UTCmodelSet1_sequence);

  return offset;
}


static const per_sequence_t DeltaUT1_sequence[] = {
  { &hf_rrc_b1              , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_11 },
  { &hf_rrc_b2              , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_10 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_DeltaUT1(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_DeltaUT1, DeltaUT1_sequence);

  return offset;
}


static const per_sequence_t UTCmodelSet2_sequence[] = {
  { &hf_rrc_nA              , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_11 },
  { &hf_rrc_tauC            , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_32 },
  { &hf_rrc_deltaUT1_01     , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DeltaUT1 },
  { &hf_rrc_kp              , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_BIT_STRING_SIZE_2 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UTCmodelSet2(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UTCmodelSet2, UTCmodelSet2_sequence);

  return offset;
}


static const per_sequence_t UTCmodelSet3_sequence[] = {
  { &hf_rrc_utcA1wnt        , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_24 },
  { &hf_rrc_utcA0wnt        , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_32 },
  { &hf_rrc_utcTot_01       , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_8 },
  { &hf_rrc_utcWNt          , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_8 },
  { &hf_rrc_utcDeltaTls     , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_8 },
  { &hf_rrc_utcWNlsf        , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_8 },
  { &hf_rrc_utcDN_01        , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_8 },
  { &hf_rrc_utcDeltaTlsf    , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_8 },
  { &hf_rrc_utcStandardID   , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_3 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UTCmodelSet3(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UTCmodelSet3, UTCmodelSet3_sequence);

  return offset;
}


static const per_sequence_t UE_Positioning_GANSS_AddUTCModels_sequence[] = {
  { &hf_rrc_utcModel1       , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UTCmodelSet1 },
  { &hf_rrc_utcModel2       , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UTCmodelSet2 },
  { &hf_rrc_utcModel3       , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UTCmodelSet3 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UE_Positioning_GANSS_AddUTCModels(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UE_Positioning_GANSS_AddUTCModels, UE_Positioning_GANSS_AddUTCModels_sequence);

  return offset;
}


static const per_sequence_t AuxInfoGANSS_ID1_element_sequence[] = {
  { &hf_rrc_svID            , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_0_63 },
  { &hf_rrc_signalsAvailable, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_8 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_AuxInfoGANSS_ID1_element(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_AuxInfoGANSS_ID1_element, AuxInfoGANSS_ID1_element_sequence);

  return offset;
}


static const per_sequence_t AuxInfoGANSS_ID1_sequence_of[1] = {
  { &hf_rrc_AuxInfoGANSS_ID1_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_AuxInfoGANSS_ID1_element },
};

static int
dissect_rrc_AuxInfoGANSS_ID1(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_AuxInfoGANSS_ID1, AuxInfoGANSS_ID1_sequence_of,
                                                  1, maxGANSSSat, FALSE);

  return offset;
}



static int
dissect_rrc_INTEGER_M7_13(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            -7, 13U, NULL, FALSE);

  return offset;
}


static const per_sequence_t AuxInfoGANSS_ID3_element_sequence[] = {
  { &hf_rrc_svID            , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_0_63 },
  { &hf_rrc_signalsAvailable, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_8 },
  { &hf_rrc_channelNumber   , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_M7_13 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_AuxInfoGANSS_ID3_element(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_AuxInfoGANSS_ID3_element, AuxInfoGANSS_ID3_element_sequence);

  return offset;
}


static const per_sequence_t AuxInfoGANSS_ID3_sequence_of[1] = {
  { &hf_rrc_AuxInfoGANSS_ID3_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_AuxInfoGANSS_ID3_element },
};

static int
dissect_rrc_AuxInfoGANSS_ID3(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_AuxInfoGANSS_ID3, AuxInfoGANSS_ID3_sequence_of,
                                                  1, maxGANSSSat, FALSE);

  return offset;
}


static const value_string rrc_UE_Positioning_GANSS_AuxiliaryInfo_vals[] = {
  {   0, "ganssID1" },
  {   1, "ganssID3" },
  { 0, NULL }
};

static const per_choice_t UE_Positioning_GANSS_AuxiliaryInfo_choice[] = {
  {   0, &hf_rrc_ganssID1        , ASN1_NO_EXTENSIONS     , dissect_rrc_AuxInfoGANSS_ID1 },
  {   1, &hf_rrc_ganssID3        , ASN1_NO_EXTENSIONS     , dissect_rrc_AuxInfoGANSS_ID3 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_UE_Positioning_GANSS_AuxiliaryInfo(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_UE_Positioning_GANSS_AuxiliaryInfo, UE_Positioning_GANSS_AuxiliaryInfo_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t GANSSGenericData_v860ext_sequence[] = {
  { &hf_rrc_uePositiningGANSSsbasID, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_Positioning_GANSS_SBAS_ID },
  { &hf_rrc_uePositioningGANSSAddNavigationModels, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_Positioning_GANSS_AddNavigationModels },
  { &hf_rrc_uePositioningGANSSAlmanac_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_Positioning_GANSS_Almanac_v860ext },
  { &hf_rrc_uePositioningGANSSAddUTCModels, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_Positioning_GANSS_AddUTCModels },
  { &hf_rrc_uePositioningGANSSAuxiliaryInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_Positioning_GANSS_AuxiliaryInfo },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_GANSSGenericData_v860ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_GANSSGenericData_v860ext, GANSSGenericData_v860ext_sequence);

  return offset;
}


static const per_sequence_t GANSSGenericDataList_v860ext_sequence_of[1] = {
  { &hf_rrc_GANSSGenericDataList_v860ext_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_GANSSGenericData_v860ext },
};

static int
dissect_rrc_GANSSGenericDataList_v860ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_GANSSGenericDataList_v860ext, GANSSGenericDataList_v860ext_sequence_of,
                                                  1, maxGANSS, FALSE);

  return offset;
}


static const per_sequence_t UE_Positioning_GANSS_AssistanceData_v860ext_sequence[] = {
  { &hf_rrc_uePositioningGanssAddIonoModel, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_Positioning_GANSS_AddIonoModel },
  { &hf_rrc_uePositioningGanssEarthOrientationPara, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_Positioning_GANSS_EarthOrientPara },
  { &hf_rrc_ganssGenericDataList_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_GANSSGenericDataList_v860ext },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UE_Positioning_GANSS_AssistanceData_v860ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UE_Positioning_GANSS_AssistanceData_v860ext, UE_Positioning_GANSS_AssistanceData_v860ext_sequence);

  return offset;
}


static const per_sequence_t AssistanceDataDelivery_v860ext_IEs_sequence[] = {
  { &hf_rrc_ue_positioning_GANSS_AssistanceData_v860ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_Positioning_GANSS_AssistanceData_v860ext },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_AssistanceDataDelivery_v860ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_AssistanceDataDelivery_v860ext_IEs, AssistanceDataDelivery_v860ext_IEs_sequence);

  return offset;
}


static const value_string rrc_UDREGrowthRate_vals[] = {
  {   0, "growth-1-5" },
  {   1, "growth-2" },
  {   2, "growth-4" },
  {   3, "growth-6" },
  {   4, "growth-8" },
  {   5, "growth-10" },
  {   6, "growth-12" },
  {   7, "growth-16" },
  { 0, NULL }
};


static int
dissect_rrc_UDREGrowthRate(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     8, NULL, FALSE, 0, NULL);

  return offset;
}


static const value_string rrc_UDREValidityTime_vals[] = {
  {   0, "val-20sec" },
  {   1, "val-40sec" },
  {   2, "val-80sec" },
  {   3, "val-160sec" },
  {   4, "val-320sec" },
  {   5, "val-640sec" },
  {   6, "val-1280sec" },
  {   7, "val-2560sec" },
  { 0, NULL }
};


static int
dissect_rrc_UDREValidityTime(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     8, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t DGPS_CorrectionSatInfo_v920ext_sequence[] = {
  { &hf_rrc_udreGrowthRate  , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UDREGrowthRate },
  { &hf_rrc_udreValidityTime, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UDREValidityTime },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_DGPS_CorrectionSatInfo_v920ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_DGPS_CorrectionSatInfo_v920ext, DGPS_CorrectionSatInfo_v920ext_sequence);

  return offset;
}


static const per_sequence_t DGPS_CorrectionSatInfoList_v920ext_sequence_of[1] = {
  { &hf_rrc_DGPS_CorrectionSatInfoList_v920ext_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DGPS_CorrectionSatInfo_v920ext },
};

static int
dissect_rrc_DGPS_CorrectionSatInfoList_v920ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_DGPS_CorrectionSatInfoList_v920ext, DGPS_CorrectionSatInfoList_v920ext_sequence_of,
                                                  1, maxSat, FALSE);

  return offset;
}


static const per_sequence_t UE_Positioning_GPS_DGPS_Corrections_v920ext_sequence[] = {
  { &hf_rrc_dgps_CorrectionSatInfoList_02, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DGPS_CorrectionSatInfoList_v920ext },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UE_Positioning_GPS_DGPS_Corrections_v920ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UE_Positioning_GPS_DGPS_Corrections_v920ext, UE_Positioning_GPS_DGPS_Corrections_v920ext_sequence);

  return offset;
}


static const per_sequence_t UE_Positioning_GPS_AssistanceData_v920ext_sequence[] = {
  { &hf_rrc_ue_positioning_GPS_DGPS_Corrections_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_Positioning_GPS_DGPS_Corrections_v920ext },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UE_Positioning_GPS_AssistanceData_v920ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UE_Positioning_GPS_AssistanceData_v920ext, UE_Positioning_GPS_AssistanceData_v920ext_sequence);

  return offset;
}


static const per_sequence_t DGANSSSignalInformation_v920ext_sequence[] = {
  { &hf_rrc_udreGrowthRate  , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UDREGrowthRate },
  { &hf_rrc_udreValidityTime, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UDREValidityTime },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_DGANSSSignalInformation_v920ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_DGANSSSignalInformation_v920ext, DGANSSSignalInformation_v920ext_sequence);

  return offset;
}


static const per_sequence_t DGANSSSignalInformationList_v920ext_sequence_of[1] = {
  { &hf_rrc_DGANSSSignalInformationList_v920ext_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DGANSSSignalInformation_v920ext },
};

static int
dissect_rrc_DGANSSSignalInformationList_v920ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_DGANSSSignalInformationList_v920ext, DGANSSSignalInformationList_v920ext_sequence_of,
                                                  1, maxGANSSSat, FALSE);

  return offset;
}


static const per_sequence_t DGANSSInfo_v920ext_sequence[] = {
  { &hf_rrc_dgansssignalInformationList_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DGANSSSignalInformationList_v920ext },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_DGANSSInfo_v920ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_DGANSSInfo_v920ext, DGANSSInfo_v920ext_sequence);

  return offset;
}


static const per_sequence_t DGANSSInfoList_v920ext_sequence_of[1] = {
  { &hf_rrc_DGANSSInfoList_v920ext_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DGANSSInfo_v920ext },
};

static int
dissect_rrc_DGANSSInfoList_v920ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_DGANSSInfoList_v920ext, DGANSSInfoList_v920ext_sequence_of,
                                                  1, maxSgnType, FALSE);

  return offset;
}


static const per_sequence_t UE_Positioning_DGANSSCorrections_v920ext_sequence[] = {
  { &hf_rrc_dganssInfoList_02, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DGANSSInfoList_v920ext },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UE_Positioning_DGANSSCorrections_v920ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UE_Positioning_DGANSSCorrections_v920ext, UE_Positioning_DGANSSCorrections_v920ext_sequence);

  return offset;
}


static const per_sequence_t GANSSGenericData_v920ext_sequence[] = {
  { &hf_rrc_uePositioningDGANSSCorrections_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_Positioning_DGANSSCorrections_v920ext },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_GANSSGenericData_v920ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_GANSSGenericData_v920ext, GANSSGenericData_v920ext_sequence);

  return offset;
}


static const per_sequence_t GANSSGenericDataList_v920ext_sequence_of[1] = {
  { &hf_rrc_GANSSGenericDataList_v920ext_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_GANSSGenericData_v920ext },
};

static int
dissect_rrc_GANSSGenericDataList_v920ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_GANSSGenericDataList_v920ext, GANSSGenericDataList_v920ext_sequence_of,
                                                  1, maxGANSS, FALSE);

  return offset;
}


static const per_sequence_t UE_Positioning_GANSS_AssistanceData_v920ext_sequence[] = {
  { &hf_rrc_ganssGenericDataList_04, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_GANSSGenericDataList_v920ext },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UE_Positioning_GANSS_AssistanceData_v920ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UE_Positioning_GANSS_AssistanceData_v920ext, UE_Positioning_GANSS_AssistanceData_v920ext_sequence);

  return offset;
}


static const per_sequence_t AssistanceDataDelivery_v920ext_IEs_sequence[] = {
  { &hf_rrc_ue_positioning_GPS_AssistanceData_v920ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_Positioning_GPS_AssistanceData_v920ext },
  { &hf_rrc_ue_positioning_GANSS_AssistanceData_v920ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_Positioning_GANSS_AssistanceData_v920ext },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_AssistanceDataDelivery_v920ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_AssistanceDataDelivery_v920ext_IEs, AssistanceDataDelivery_v920ext_IEs_sequence);

  return offset;
}


static const per_sequence_t UE_Positioning_GPS_ReferenceTime_va40ext_sequence[] = {
  { &hf_rrc_gps_Week_Cycle_Number, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_INTEGER_0_7 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UE_Positioning_GPS_ReferenceTime_va40ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UE_Positioning_GPS_ReferenceTime_va40ext, UE_Positioning_GPS_ReferenceTime_va40ext_sequence);

  return offset;
}


static const per_sequence_t UE_Positioning_GPS_Almanac_va40ext_sequence[] = {
  { &hf_rrc_complete_Almanac_Provided, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_BOOLEAN },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UE_Positioning_GPS_Almanac_va40ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UE_Positioning_GPS_Almanac_va40ext, UE_Positioning_GPS_Almanac_va40ext_sequence);

  return offset;
}


static const per_sequence_t AzimuthAndElevation_va40ext_sequence[] = {
  { &hf_rrc_azimuthLSB      , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_0_15 },
  { &hf_rrc_elevationLSB    , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_0_15 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_AzimuthAndElevation_va40ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_AzimuthAndElevation_va40ext, AzimuthAndElevation_va40ext_sequence);

  return offset;
}


static const per_sequence_t AcquisitionSatInfo_va40ext_sequence[] = {
  { &hf_rrc_azimuthAndElevation_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_AzimuthAndElevation_va40ext },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_AcquisitionSatInfo_va40ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_AcquisitionSatInfo_va40ext, AcquisitionSatInfo_va40ext_sequence);

  return offset;
}


static const per_sequence_t AcquisitionSatInfoList_va40ext_sequence_of[1] = {
  { &hf_rrc_AcquisitionSatInfoList_va40ext_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_AcquisitionSatInfo_va40ext },
};

static int
dissect_rrc_AcquisitionSatInfoList_va40ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_AcquisitionSatInfoList_va40ext, AcquisitionSatInfoList_va40ext_sequence_of,
                                                  1, maxSat, FALSE);

  return offset;
}


static const per_sequence_t UE_Positioning_GPS_AcquisitionAssistance_va40ext_sequence[] = {
  { &hf_rrc_satelliteInformationList_04, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_AcquisitionSatInfoList_va40ext },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UE_Positioning_GPS_AcquisitionAssistance_va40ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UE_Positioning_GPS_AcquisitionAssistance_va40ext, UE_Positioning_GPS_AcquisitionAssistance_va40ext_sequence);

  return offset;
}


static const per_sequence_t UE_Positioning_GPS_AssistanceData_va40ext_sequence[] = {
  { &hf_rrc_ue_positioning_GPS_ReferenceTime_03, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_Positioning_GPS_ReferenceTime_va40ext },
  { &hf_rrc_ue_positioning_GPS_Almanac_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_Positioning_GPS_Almanac_va40ext },
  { &hf_rrc_ue_positioning_GPS_AcquisitionAssistance_03, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_Positioning_GPS_AcquisitionAssistance_va40ext },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UE_Positioning_GPS_AssistanceData_va40ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UE_Positioning_GPS_AssistanceData_va40ext, UE_Positioning_GPS_AssistanceData_va40ext_sequence);

  return offset;
}


static const per_sequence_t UE_Positioning_GANSS_ReferenceTime_va40ext_sequence[] = {
  { &hf_rrc_ganss_Day_Cycle_Number, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_INTEGER_0_7 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UE_Positioning_GANSS_ReferenceTime_va40ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UE_Positioning_GANSS_ReferenceTime_va40ext, UE_Positioning_GANSS_ReferenceTime_va40ext_sequence);

  return offset;
}



static int
dissect_rrc_INTEGER_M128_127(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            -128, 127U, NULL, FALSE);

  return offset;
}


static const per_sequence_t UE_Positioning_GANSS_TimeModel_va40ext_sequence[] = {
  { &hf_rrc_deltaT          , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_INTEGER_M128_127 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UE_Positioning_GANSS_TimeModel_va40ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UE_Positioning_GANSS_TimeModel_va40ext, UE_Positioning_GANSS_TimeModel_va40ext_sequence);

  return offset;
}


static const per_sequence_t GANSSTimeModelsList_va40ext_sequence_of[1] = {
  { &hf_rrc_GANSSTimeModelsList_va40ext_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UE_Positioning_GANSS_TimeModel_va40ext },
};

static int
dissect_rrc_GANSSTimeModelsList_va40ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_GANSSTimeModelsList_va40ext, GANSSTimeModelsList_va40ext_sequence_of,
                                                  1, maxGANSS_1, FALSE);

  return offset;
}


static const per_sequence_t GANSSSatelliteInformation_va40ext_sequence[] = {
  { &hf_rrc_azimuthAndElevation_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_AzimuthAndElevation_va40ext },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_GANSSSatelliteInformation_va40ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_GANSSSatelliteInformation_va40ext, GANSSSatelliteInformation_va40ext_sequence);

  return offset;
}


static const per_sequence_t GANSSSatelliteInformationList_va40ext_sequence_of[1] = {
  { &hf_rrc_GANSSSatelliteInformationList_va40ext_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_GANSSSatelliteInformation_va40ext },
};

static int
dissect_rrc_GANSSSatelliteInformationList_va40ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_GANSSSatelliteInformationList_va40ext, GANSSSatelliteInformationList_va40ext_sequence_of,
                                                  1, maxGANSSSat, FALSE);

  return offset;
}


static const per_sequence_t UE_Positioning_GANSS_ReferenceMeasurementInfo_va40ext_sequence[] = {
  { &hf_rrc_satelliteInformationList_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_GANSSSatelliteInformationList_va40ext },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UE_Positioning_GANSS_ReferenceMeasurementInfo_va40ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UE_Positioning_GANSS_ReferenceMeasurementInfo_va40ext, UE_Positioning_GANSS_ReferenceMeasurementInfo_va40ext_sequence);

  return offset;
}


static const per_sequence_t UE_Positioning_GANSS_Almanac_va40ext_sequence[] = {
  { &hf_rrc_complete_Almanac_Provided, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_BOOLEAN },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UE_Positioning_GANSS_Almanac_va40ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UE_Positioning_GANSS_Almanac_va40ext, UE_Positioning_GANSS_Almanac_va40ext_sequence);

  return offset;
}


static const per_sequence_t GANSSGenericData_va40ext_sequence[] = {
  { &hf_rrc_ganssTimeModelsList_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_GANSSTimeModelsList_va40ext },
  { &hf_rrc_uePositioningGANSSReferenceMeasurementInfo_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_Positioning_GANSS_ReferenceMeasurementInfo_va40ext },
  { &hf_rrc_uePositioningGANSSAlmanac_03, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_Positioning_GANSS_Almanac_va40ext },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_GANSSGenericData_va40ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_GANSSGenericData_va40ext, GANSSGenericData_va40ext_sequence);

  return offset;
}


static const per_sequence_t GANSSGenericDataList_va40ext_sequence_of[1] = {
  { &hf_rrc_GANSSGenericDataList_va40ext_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_GANSSGenericData_va40ext },
};

static int
dissect_rrc_GANSSGenericDataList_va40ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_GANSSGenericDataList_va40ext, GANSSGenericDataList_va40ext_sequence_of,
                                                  1, maxGANSS, FALSE);

  return offset;
}


static const per_sequence_t UE_Positioning_GANSS_AssistanceData_va40ext_sequence[] = {
  { &hf_rrc_ue_positioning_GANSS_ReferenceTime_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_Positioning_GANSS_ReferenceTime_va40ext },
  { &hf_rrc_ganssGenericDataList_05, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_GANSSGenericDataList_va40ext },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UE_Positioning_GANSS_AssistanceData_va40ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UE_Positioning_GANSS_AssistanceData_va40ext, UE_Positioning_GANSS_AssistanceData_va40ext_sequence);

  return offset;
}


static const per_sequence_t AssistanceDataDelivery_va40ext_IEs_sequence[] = {
  { &hf_rrc_ue_positioning_GPS_AssistanceData_va40ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_Positioning_GPS_AssistanceData_va40ext },
  { &hf_rrc_ue_positioning_GANSS_AssistanceData_va40ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_Positioning_GANSS_AssistanceData_va40ext },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_AssistanceDataDelivery_va40ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_AssistanceDataDelivery_va40ext_IEs, AssistanceDataDelivery_va40ext_IEs_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions_09_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_nonCriticalExtensions_09(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_nonCriticalExtensions_09, T_nonCriticalExtensions_09_sequence);

  return offset;
}


static const per_sequence_t T_va40NonCriticalExtension_sequence[] = {
  { &hf_rrc_assistanceDataDelivery_va40ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_AssistanceDataDelivery_va40ext_IEs },
  { &hf_rrc_nonCriticalExtensions_09, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_nonCriticalExtensions_09 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_va40NonCriticalExtension(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_va40NonCriticalExtension, T_va40NonCriticalExtension_sequence);

  return offset;
}


static const per_sequence_t T_v920NonCriticalExtension_sequence[] = {
  { &hf_rrc_assistanceDataDelivery_v920ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_AssistanceDataDelivery_v920ext_IEs },
  { &hf_rrc_va40NonCriticalExtension, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_va40NonCriticalExtension },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v920NonCriticalExtension(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v920NonCriticalExtension, T_v920NonCriticalExtension_sequence);

  return offset;
}


static const per_sequence_t T_v860NonCriticalExtension_sequence[] = {
  { &hf_rrc_assistanceDataDelivery_v860ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_AssistanceDataDelivery_v860ext_IEs },
  { &hf_rrc_v920NonCriticalExtension, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v920NonCriticalExtension },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v860NonCriticalExtension(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v860NonCriticalExtension, T_v860NonCriticalExtension_sequence);

  return offset;
}


static const per_sequence_t T_v770NonCriticalExtension_sequence[] = {
  { &hf_rrc_assistanceDataDelivery_v770ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_AssistanceDataDelivery_v770ext_IEs },
  { &hf_rrc_v860NonCriticalExtension, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v860NonCriticalExtension },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v770NonCriticalExtension(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v770NonCriticalExtension, T_v770NonCriticalExtension_sequence);

  return offset;
}


static const per_sequence_t T_v4b0NonCriticalExtensions_01_sequence[] = {
  { &hf_rrc_assistanceDataDelivery_v4b0ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_AssistanceDataDelivery_v4b0ext_IEs },
  { &hf_rrc_v770NonCriticalExtension, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v770NonCriticalExtension },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v4b0NonCriticalExtensions_01(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v4b0NonCriticalExtensions_01, T_v4b0NonCriticalExtensions_01_sequence);

  return offset;
}


static const per_sequence_t T_laterNonCriticalExtensions_03_sequence[] = {
  { &hf_rrc_assistanceDataDelivery_r3_add_ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_BIT_STRING },
  { &hf_rrc_v4b0NonCriticalExtensions_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v4b0NonCriticalExtensions_01 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_laterNonCriticalExtensions_03(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_laterNonCriticalExtensions_03, T_laterNonCriticalExtensions_03_sequence);

  return offset;
}


static const per_sequence_t T_v3a0NonCriticalExtensions_sequence[] = {
  { &hf_rrc_assistanceDataDelivery_v3a0ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_AssistanceDataDelivery_v3a0ext },
  { &hf_rrc_laterNonCriticalExtensions_03, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_laterNonCriticalExtensions_03 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v3a0NonCriticalExtensions(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v3a0NonCriticalExtensions, T_v3a0NonCriticalExtensions_sequence);

  return offset;
}


static const per_sequence_t T_r3_01_sequence[] = {
  { &hf_rrc_assistanceDataDelivery_r3, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_AssistanceDataDelivery_r3_IEs },
  { &hf_rrc_v3a0NonCriticalExtensions, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v3a0NonCriticalExtensions },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_r3_01(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_r3_01, T_r3_01_sequence);

  return offset;
}


static const per_sequence_t T_criticalExtensions_07_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_criticalExtensions_07(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_criticalExtensions_07, T_criticalExtensions_07_sequence);

  return offset;
}


static const per_sequence_t T_later_than_r3_01_sequence[] = {
  { &hf_rrc_rrc_TransactionIdentifier, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RRC_TransactionIdentifier },
  { &hf_rrc_criticalExtensions_07, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_criticalExtensions_07 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_later_than_r3_01(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_later_than_r3_01, T_later_than_r3_01_sequence);

  return offset;
}


static const value_string rrc_AssistanceDataDelivery_vals[] = {
  {   0, "r3" },
  {   1, "later-than-r3" },
  { 0, NULL }
};

static const per_choice_t AssistanceDataDelivery_choice[] = {
  {   0, &hf_rrc_r3_01           , ASN1_NO_EXTENSIONS     , dissect_rrc_T_r3_01 },
  {   1, &hf_rrc_later_than_r3_01, ASN1_NO_EXTENSIONS     , dissect_rrc_T_later_than_r3_01 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_AssistanceDataDelivery(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
#line 177 "../../asn1/rrc/rrc.cnf"
  col_append_str(actx->pinfo->cinfo, COL_INFO, "AssistanceDataDelivery");
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_AssistanceDataDelivery, AssistanceDataDelivery_choice,
                                 NULL);




  return offset;
}


static const value_string rrc_RAB_Identity_vals[] = {
  {   0, "gsm-MAP-RAB-Identity" },
  {   1, "ansi-41-RAB-Identity" },
  { 0, NULL }
};

static const per_choice_t RAB_Identity_choice[] = {
  {   0, &hf_rrc_gsm_MAP_RAB_Identity, ASN1_NO_EXTENSIONS     , dissect_rrc_BIT_STRING_SIZE_8 },
  {   1, &hf_rrc_ansi_41_RAB_Identity, ASN1_NO_EXTENSIONS     , dissect_rrc_BIT_STRING_SIZE_8 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_RAB_Identity(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_RAB_Identity, RAB_Identity_choice,
                                 NULL);

  return offset;
}



static int
dissect_rrc_NAS_Synchronisation_Indicator(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_bit_string(tvb, offset, actx, tree, hf_index,
                                     4, 4, FALSE, NULL);

  return offset;
}


static const value_string rrc_Re_EstablishmentTimer_vals[] = {
  {   0, "useT314" },
  {   1, "useT315" },
  { 0, NULL }
};


static int
dissect_rrc_Re_EstablishmentTimer(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     2, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t RAB_Info_sequence[] = {
  { &hf_rrc_rab_Identity    , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RAB_Identity },
  { &hf_rrc_cn_DomainIdentity, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CN_DomainIdentity },
  { &hf_rrc_nas_Synchronisation_Indicator, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_NAS_Synchronisation_Indicator },
  { &hf_rrc_re_EstablishmentTimer, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_Re_EstablishmentTimer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_RAB_Info(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_RAB_Info, RAB_Info_sequence);

  return offset;
}


static const per_sequence_t RAB_InformationList_sequence_of[1] = {
  { &hf_rrc_RAB_InformationList_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RAB_Info },
};

static int
dissect_rrc_RAB_InformationList(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_RAB_InformationList, RAB_InformationList_sequence_of,
                                                  1, maxRABsetup, FALSE);

  return offset;
}



static int
dissect_rrc_NCC(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            0U, 7U, NULL, FALSE);

  return offset;
}



static int
dissect_rrc_BCC(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            0U, 7U, NULL, FALSE);

  return offset;
}


static const per_sequence_t BSIC_sequence[] = {
  { &hf_rrc_ncc             , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_NCC },
  { &hf_rrc_bcc             , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BCC },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_BSIC(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_BSIC, BSIC_sequence);

  return offset;
}


static const value_string rrc_Frequency_Band_vals[] = {
  {   0, "dcs1800BandUsed" },
  {   1, "pcs1900BandUsed" },
  { 0, NULL }
};


static int
dissect_rrc_Frequency_Band(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     2, NULL, FALSE, 0, NULL);

  return offset;
}



static int
dissect_rrc_BCCH_ARFCN(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            0U, 1023U, NULL, FALSE);

  return offset;
}



static int
dissect_rrc_NC_Mode(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_bit_string(tvb, offset, actx, tree, hf_index,
                                     3, 3, FALSE, NULL);

  return offset;
}


static const per_sequence_t T_gsm_08_sequence[] = {
  { &hf_rrc_bsic            , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BSIC },
  { &hf_rrc_frequency_band  , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_Frequency_Band },
  { &hf_rrc_bcch_ARFCN      , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BCCH_ARFCN },
  { &hf_rrc_ncMode          , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_NC_Mode },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_gsm_08(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_gsm_08, T_gsm_08_sequence);

  return offset;
}


static const value_string rrc_T_technologySpecificInfo_vals[] = {
  {   0, "gsm" },
  {   1, "is-2000" },
  {   2, "spare2" },
  {   3, "spare1" },
  { 0, NULL }
};

static const per_choice_t T_technologySpecificInfo_choice[] = {
  {   0, &hf_rrc_gsm_08          , ASN1_NO_EXTENSIONS     , dissect_rrc_T_gsm_08 },
  {   1, &hf_rrc_is_2000         , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   2, &hf_rrc_spare2          , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   3, &hf_rrc_spare1          , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_technologySpecificInfo(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_technologySpecificInfo, T_technologySpecificInfo_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t InterRAT_TargetCellDescription_sequence[] = {
  { &hf_rrc_technologySpecificInfo, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_technologySpecificInfo },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_InterRAT_TargetCellDescription(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_InterRAT_TargetCellDescription, InterRAT_TargetCellDescription_sequence);

  return offset;
}


static const per_sequence_t CellChangeOrderFromUTRAN_r3_IEs_sequence[] = {
  { &hf_rrc_rrc_TransactionIdentifier, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RRC_TransactionIdentifier },
  { &hf_rrc_dummy_01        , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_IntegrityProtectionModeInfo },
  { &hf_rrc_activationTime  , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_ActivationTime },
  { &hf_rrc_rab_InformationList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RAB_InformationList },
  { &hf_rrc_interRAT_TargetCellDescription, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_InterRAT_TargetCellDescription },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_CellChangeOrderFromUTRAN_r3_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_CellChangeOrderFromUTRAN_r3_IEs, CellChangeOrderFromUTRAN_r3_IEs_sequence);

  return offset;
}



static int
dissect_rrc_GERAN_SystemInfoBlock(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
#line 554 "../../asn1/rrc/rrc.cnf"
  tvbuff_t *parameter_tvb=NULL;

  offset = dissect_per_octet_string(tvb, offset, actx, tree, hf_index,
                                       1, 23, FALSE, &parameter_tvb);

  if (parameter_tvb){
    guint8 byte = tvb_get_guint8(parameter_tvb, 0);
    if (byte == 0x06) {
      if (gsm_a_dtap_handle) {
        call_dissector(gsm_a_dtap_handle, parameter_tvb, actx->pinfo, tree);
      }
    } else {
      if (gsm_rlcmac_dl_handle) {
        call_dissector(gsm_rlcmac_dl_handle, parameter_tvb, actx->pinfo, tree);
      }
    }
  }



  return offset;
}


static const per_sequence_t GERAN_SystemInformation_sequence_of[1] = {
  { &hf_rrc_GERAN_SystemInformation_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_GERAN_SystemInfoBlock },
};

static int
dissect_rrc_GERAN_SystemInformation(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_GERAN_SystemInformation, GERAN_SystemInformation_sequence_of,
                                                  1, maxGERAN_SI, FALSE);

  return offset;
}


static const value_string rrc_T_geran_SystemInfoType_vals[] = {
  {   0, "sI" },
  {   1, "pSI" },
  { 0, NULL }
};

static const per_choice_t T_geran_SystemInfoType_choice[] = {
  {   0, &hf_rrc_sI              , ASN1_NO_EXTENSIONS     , dissect_rrc_GERAN_SystemInformation },
  {   1, &hf_rrc_pSI             , ASN1_NO_EXTENSIONS     , dissect_rrc_GERAN_SystemInformation },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_geran_SystemInfoType(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_geran_SystemInfoType, T_geran_SystemInfoType_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t CellChangeOrderFromUTRAN_v590ext_IEs_sequence[] = {
  { &hf_rrc_geran_SystemInfoType, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_geran_SystemInfoType },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_CellChangeOrderFromUTRAN_v590ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_CellChangeOrderFromUTRAN_v590ext_IEs, CellChangeOrderFromUTRAN_v590ext_IEs_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions_10_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_nonCriticalExtensions_10(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_nonCriticalExtensions_10, T_nonCriticalExtensions_10_sequence);

  return offset;
}


static const per_sequence_t T_v590NonCriticalExtensions_01_sequence[] = {
  { &hf_rrc_cellChangeOrderFromUTRAN_v590ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CellChangeOrderFromUTRAN_v590ext_IEs },
  { &hf_rrc_nonCriticalExtensions_10, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_nonCriticalExtensions_10 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v590NonCriticalExtensions_01(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v590NonCriticalExtensions_01, T_v590NonCriticalExtensions_01_sequence);

  return offset;
}


static const per_sequence_t T_laterNonCriticalExtensions_04_sequence[] = {
  { &hf_rrc_cellChangeOrderFromUTRAN_r3_add_ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_BIT_STRING },
  { &hf_rrc_v590NonCriticalExtensions_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v590NonCriticalExtensions_01 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_laterNonCriticalExtensions_04(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_laterNonCriticalExtensions_04, T_laterNonCriticalExtensions_04_sequence);

  return offset;
}


static const per_sequence_t T_r3_02_sequence[] = {
  { &hf_rrc_cellChangeOrderFromUTRAN_IEs, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CellChangeOrderFromUTRAN_r3_IEs },
  { &hf_rrc_laterNonCriticalExtensions_04, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_laterNonCriticalExtensions_04 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_r3_02(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_r3_02, T_r3_02_sequence);

  return offset;
}


static const per_sequence_t T_criticalExtensions_08_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_criticalExtensions_08(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_criticalExtensions_08, T_criticalExtensions_08_sequence);

  return offset;
}


static const per_sequence_t T_later_than_r3_02_sequence[] = {
  { &hf_rrc_rrc_TransactionIdentifier, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RRC_TransactionIdentifier },
  { &hf_rrc_criticalExtensions_08, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_criticalExtensions_08 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_later_than_r3_02(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_later_than_r3_02, T_later_than_r3_02_sequence);

  return offset;
}


static const value_string rrc_CellChangeOrderFromUTRAN_vals[] = {
  {   0, "r3" },
  {   1, "later-than-r3" },
  { 0, NULL }
};

static const per_choice_t CellChangeOrderFromUTRAN_choice[] = {
  {   0, &hf_rrc_r3_02           , ASN1_NO_EXTENSIONS     , dissect_rrc_T_r3_02 },
  {   1, &hf_rrc_later_than_r3_02, ASN1_NO_EXTENSIONS     , dissect_rrc_T_later_than_r3_02 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_CellChangeOrderFromUTRAN(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
#line 181 "../../asn1/rrc/rrc.cnf"
  col_append_str(actx->pinfo->cinfo, COL_INFO, "CellChangeOrderFromUTRAN");
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_CellChangeOrderFromUTRAN, CellChangeOrderFromUTRAN_choice,
                                 NULL);




  return offset;
}



static int
dissect_rrc_C_RNTI(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_bit_string(tvb, offset, actx, tree, hf_index,
                                     16, 16, FALSE, NULL);

  return offset;
}


static const value_string rrc_RRC_StateIndicator_vals[] = {
  {   0, "cell-DCH" },
  {   1, "cell-FACH" },
  {   2, "cell-PCH" },
  {   3, "ura-PCH" },
  { 0, NULL }
};


static int
dissect_rrc_RRC_StateIndicator(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     4, NULL, FALSE, 0, NULL);

  return offset;
}



static int
dissect_rrc_UTRAN_DRX_CycleLengthCoefficient(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            3U, 9U, NULL, FALSE);

  return offset;
}



static int
dissect_rrc_URA_Identity(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_bit_string(tvb, offset, actx, tree, hf_index,
                                     16, 16, FALSE, NULL);

  return offset;
}


static const per_sequence_t RB_InformationReleaseList_sequence_of[1] = {
  { &hf_rrc_RB_InformationReleaseList_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RB_Identity },
};

static int
dissect_rrc_RB_InformationReleaseList(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_RB_InformationReleaseList, RB_InformationReleaseList_sequence_of,
                                                  1, maxRB, FALSE);

  return offset;
}


static const value_string rrc_MaxPDCP_SN_WindowSize_vals[] = {
  {   0, "sn255" },
  {   1, "sn65535" },
  { 0, NULL }
};


static int
dissect_rrc_MaxPDCP_SN_WindowSize(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     2, NULL, FALSE, 0, NULL);

  return offset;
}


static const value_string rrc_LosslessSRNS_RelocSupport_vals[] = {
  {   0, "supported" },
  {   1, "notSupported" },
  { 0, NULL }
};

static const per_choice_t LosslessSRNS_RelocSupport_choice[] = {
  {   0, &hf_rrc_supported_13    , ASN1_NO_EXTENSIONS     , dissect_rrc_MaxPDCP_SN_WindowSize },
  {   1, &hf_rrc_notSupported    , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_LosslessSRNS_RelocSupport(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_LosslessSRNS_RelocSupport, LosslessSRNS_RelocSupport_choice,
                                 NULL);

  return offset;
}


static const value_string rrc_PDCP_PDU_Header_vals[] = {
  {   0, "present" },
  {   1, "absent" },
  { 0, NULL }
};


static int
dissect_rrc_PDCP_PDU_Header(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     2, NULL, FALSE, 0, NULL);

  return offset;
}



static int
dissect_rrc_INTEGER_1_65535(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            1U, 65535U, NULL, FALSE);

  return offset;
}



static int
dissect_rrc_INTEGER_1_255(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            1U, 255U, NULL, FALSE);

  return offset;
}



static int
dissect_rrc_INTEGER_60_65535(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            60U, 65535U, NULL, FALSE);

  return offset;
}



static int
dissect_rrc_INTEGER_3_255(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            3U, 255U, NULL, FALSE);

  return offset;
}



static int
dissect_rrc_INTEGER_3_65535(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            3U, 65535U, NULL, FALSE);

  return offset;
}


static const value_string rrc_ExpectReordering_vals[] = {
  {   0, "reorderingNotExpected" },
  {   1, "reorderingExpected" },
  { 0, NULL }
};


static int
dissect_rrc_ExpectReordering(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     2, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t RFC2507_Info_sequence[] = {
  { &hf_rrc_f_MAX_PERIOD    , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_INTEGER_1_65535 },
  { &hf_rrc_f_MAX_TIME      , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_INTEGER_1_255 },
  { &hf_rrc_max_HEADER      , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_INTEGER_60_65535 },
  { &hf_rrc_tcp_SPACE       , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_INTEGER_3_255 },
  { &hf_rrc_non_TCP_SPACE   , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_INTEGER_3_65535 },
  { &hf_rrc_expectReordering, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_ExpectReordering },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_RFC2507_Info(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_RFC2507_Info, RFC2507_Info_sequence);

  return offset;
}


static const value_string rrc_AlgorithmSpecificInfo_vals[] = {
  {   0, "rfc2507-Info" },
  { 0, NULL }
};

static const per_choice_t AlgorithmSpecificInfo_choice[] = {
  {   0, &hf_rrc_rfc2507_Info    , ASN1_NO_EXTENSIONS     , dissect_rrc_RFC2507_Info },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_AlgorithmSpecificInfo(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_AlgorithmSpecificInfo, AlgorithmSpecificInfo_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t HeaderCompressionInfo_sequence[] = {
  { &hf_rrc_algorithmSpecificInfo, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_AlgorithmSpecificInfo },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_HeaderCompressionInfo(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_HeaderCompressionInfo, HeaderCompressionInfo_sequence);

  return offset;
}


static const per_sequence_t HeaderCompressionInfoList_sequence_of[1] = {
  { &hf_rrc_HeaderCompressionInfoList_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_HeaderCompressionInfo },
};

static int
dissect_rrc_HeaderCompressionInfoList(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_HeaderCompressionInfoList, HeaderCompressionInfoList_sequence_of,
                                                  1, maxPDCPAlgoType, FALSE);

  return offset;
}


static const per_sequence_t PDCP_Info_sequence[] = {
  { &hf_rrc_losslessSRNS_RelocSupport, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_LosslessSRNS_RelocSupport },
  { &hf_rrc_pdcp_PDU_Header , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PDCP_PDU_Header },
  { &hf_rrc_headerCompressionInfoList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_HeaderCompressionInfoList },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_PDCP_Info(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_PDCP_Info, PDCP_Info_sequence);

  return offset;
}


static const per_sequence_t PDCP_InfoReconfig_sequence[] = {
  { &hf_rrc_pdcp_Info       , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PDCP_Info },
  { &hf_rrc_dummy_32        , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_0_65535 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_PDCP_InfoReconfig(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_PDCP_InfoReconfig, PDCP_InfoReconfig_sequence);

  return offset;
}


static const value_string rrc_TimerMRW_vals[] = {
  {   0, "te50" },
  {   1, "te60" },
  {   2, "te70" },
  {   3, "te80" },
  {   4, "te90" },
  {   5, "te100" },
  {   6, "te120" },
  {   7, "te140" },
  {   8, "te160" },
  {   9, "te180" },
  {  10, "te200" },
  {  11, "te300" },
  {  12, "te400" },
  {  13, "te500" },
  {  14, "te700" },
  {  15, "te900" },
  { 0, NULL }
};


static int
dissect_rrc_TimerMRW(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     16, NULL, FALSE, 0, NULL);

  return offset;
}


static const value_string rrc_TimerDiscard_vals[] = {
  {   0, "td0-1" },
  {   1, "td0-25" },
  {   2, "td0-5" },
  {   3, "td0-75" },
  {   4, "td1" },
  {   5, "td1-25" },
  {   6, "td1-5" },
  {   7, "td1-75" },
  {   8, "td2" },
  {   9, "td2-5" },
  {  10, "td3" },
  {  11, "td3-5" },
  {  12, "td4" },
  {  13, "td4-5" },
  {  14, "td5" },
  {  15, "td7-5" },
  { 0, NULL }
};


static int
dissect_rrc_TimerDiscard(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     16, NULL, FALSE, 0, NULL);

  return offset;
}


static const value_string rrc_MaxMRW_vals[] = {
  {   0, "mm1" },
  {   1, "mm4" },
  {   2, "mm6" },
  {   3, "mm8" },
  {   4, "mm12" },
  {   5, "mm16" },
  {   6, "mm24" },
  {   7, "mm32" },
  { 0, NULL }
};


static int
dissect_rrc_MaxMRW(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     8, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t ExplicitDiscard_sequence[] = {
  { &hf_rrc_timerMRW        , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TimerMRW },
  { &hf_rrc_timerDiscard    , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TimerDiscard },
  { &hf_rrc_maxMRW          , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MaxMRW },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_ExplicitDiscard(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_ExplicitDiscard, ExplicitDiscard_sequence);

  return offset;
}


static const value_string rrc_NoExplicitDiscard_vals[] = {
  {   0, "dt10" },
  {   1, "dt20" },
  {   2, "dt30" },
  {   3, "dt40" },
  {   4, "dt50" },
  {   5, "dt60" },
  {   6, "dt70" },
  {   7, "dt80" },
  {   8, "dt90" },
  {   9, "dt100" },
  { 0, NULL }
};


static int
dissect_rrc_NoExplicitDiscard(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     10, NULL, FALSE, 0, NULL);

  return offset;
}


static const value_string rrc_MaxDAT_vals[] = {
  {   0, "dat1" },
  {   1, "dat2" },
  {   2, "dat3" },
  {   3, "dat4" },
  {   4, "dat5" },
  {   5, "dat6" },
  {   6, "dat7" },
  {   7, "dat8" },
  {   8, "dat9" },
  {   9, "dat10" },
  {  10, "dat15" },
  {  11, "dat20" },
  {  12, "dat25" },
  {  13, "dat30" },
  {  14, "dat35" },
  {  15, "dat40" },
  { 0, NULL }
};


static int
dissect_rrc_MaxDAT(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     16, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t MaxDAT_Retransmissions_sequence[] = {
  { &hf_rrc_maxDAT          , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MaxDAT },
  { &hf_rrc_timerMRW        , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TimerMRW },
  { &hf_rrc_maxMRW          , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MaxMRW },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_MaxDAT_Retransmissions(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_MaxDAT_Retransmissions, MaxDAT_Retransmissions_sequence);

  return offset;
}


static const value_string rrc_TransmissionRLC_Discard_vals[] = {
  {   0, "timerBasedExplicit" },
  {   1, "timerBasedNoExplicit" },
  {   2, "maxDAT-Retransmissions" },
  {   3, "noDiscard" },
  { 0, NULL }
};

static const per_choice_t TransmissionRLC_Discard_choice[] = {
  {   0, &hf_rrc_timerBasedExplicit, ASN1_NO_EXTENSIONS     , dissect_rrc_ExplicitDiscard },
  {   1, &hf_rrc_timerBasedNoExplicit, ASN1_NO_EXTENSIONS     , dissect_rrc_NoExplicitDiscard },
  {   2, &hf_rrc_maxDAT_Retransmissions, ASN1_NO_EXTENSIONS     , dissect_rrc_MaxDAT_Retransmissions },
  {   3, &hf_rrc_noDiscard       , ASN1_NO_EXTENSIONS     , dissect_rrc_MaxDAT },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_TransmissionRLC_Discard(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_TransmissionRLC_Discard, TransmissionRLC_Discard_choice,
                                 NULL);

  return offset;
}


static const value_string rrc_TransmissionWindowSize_vals[] = {
  {   0, "tw1" },
  {   1, "tw8" },
  {   2, "tw16" },
  {   3, "tw32" },
  {   4, "tw64" },
  {   5, "tw128" },
  {   6, "tw256" },
  {   7, "tw512" },
  {   8, "tw768" },
  {   9, "tw1024" },
  {  10, "tw1536" },
  {  11, "tw2047" },
  {  12, "tw2560" },
  {  13, "tw3072" },
  {  14, "tw3584" },
  {  15, "tw4095" },
  { 0, NULL }
};


static int
dissect_rrc_TransmissionWindowSize(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     16, NULL, FALSE, 0, NULL);

  return offset;
}


static const value_string rrc_TimerRST_vals[] = {
  {   0, "tr50" },
  {   1, "tr100" },
  {   2, "tr150" },
  {   3, "tr200" },
  {   4, "tr250" },
  {   5, "tr300" },
  {   6, "tr350" },
  {   7, "tr400" },
  {   8, "tr450" },
  {   9, "tr500" },
  {  10, "tr550" },
  {  11, "tr600" },
  {  12, "tr700" },
  {  13, "tr800" },
  {  14, "tr900" },
  {  15, "tr1000" },
  { 0, NULL }
};


static int
dissect_rrc_TimerRST(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     16, NULL, FALSE, 0, NULL);

  return offset;
}


static const value_string rrc_MaxRST_vals[] = {
  {   0, "rst1" },
  {   1, "rst4" },
  {   2, "rst6" },
  {   3, "rst8" },
  {   4, "rst12" },
  {   5, "rst16" },
  {   6, "rst24" },
  {   7, "rst32" },
  { 0, NULL }
};


static int
dissect_rrc_MaxRST(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     8, NULL, FALSE, 0, NULL);

  return offset;
}


static const value_string rrc_TimerPollProhibit_vals[] = {
  {   0, "tpp10" },
  {   1, "tpp20" },
  {   2, "tpp30" },
  {   3, "tpp40" },
  {   4, "tpp50" },
  {   5, "tpp60" },
  {   6, "tpp70" },
  {   7, "tpp80" },
  {   8, "tpp90" },
  {   9, "tpp100" },
  {  10, "tpp110" },
  {  11, "tpp120" },
  {  12, "tpp130" },
  {  13, "tpp140" },
  {  14, "tpp150" },
  {  15, "tpp160" },
  {  16, "tpp170" },
  {  17, "tpp180" },
  {  18, "tpp190" },
  {  19, "tpp200" },
  {  20, "tpp210" },
  {  21, "tpp220" },
  {  22, "tpp230" },
  {  23, "tpp240" },
  {  24, "tpp250" },
  {  25, "tpp260" },
  {  26, "tpp270" },
  {  27, "tpp280" },
  {  28, "tpp290" },
  {  29, "tpp300" },
  {  30, "tpp310" },
  {  31, "tpp320" },
  {  32, "tpp330" },
  {  33, "tpp340" },
  {  34, "tpp350" },
  {  35, "tpp360" },
  {  36, "tpp370" },
  {  37, "tpp380" },
  {  38, "tpp390" },
  {  39, "tpp400" },
  {  40, "tpp410" },
  {  41, "tpp420" },
  {  42, "tpp430" },
  {  43, "tpp440" },
  {  44, "tpp450" },
  {  45, "tpp460" },
  {  46, "tpp470" },
  {  47, "tpp480" },
  {  48, "tpp490" },
  {  49, "tpp500" },
  {  50, "tpp510" },
  {  51, "tpp520" },
  {  52, "tpp530" },
  {  53, "tpp540" },
  {  54, "tpp550" },
  {  55, "tpp600" },
  {  56, "tpp650" },
  {  57, "tpp700" },
  {  58, "tpp750" },
  {  59, "tpp800" },
  {  60, "tpp850" },
  {  61, "tpp900" },
  {  62, "tpp950" },
  {  63, "tpp1000" },
  { 0, NULL }
};

static value_string_ext rrc_TimerPollProhibit_vals_ext = VALUE_STRING_EXT_INIT(rrc_TimerPollProhibit_vals);


static int
dissect_rrc_TimerPollProhibit(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     64, NULL, FALSE, 0, NULL);

  return offset;
}


static const value_string rrc_TimerPoll_vals[] = {
  {   0, "tp10" },
  {   1, "tp20" },
  {   2, "tp30" },
  {   3, "tp40" },
  {   4, "tp50" },
  {   5, "tp60" },
  {   6, "tp70" },
  {   7, "tp80" },
  {   8, "tp90" },
  {   9, "tp100" },
  {  10, "tp110" },
  {  11, "tp120" },
  {  12, "tp130" },
  {  13, "tp140" },
  {  14, "tp150" },
  {  15, "tp160" },
  {  16, "tp170" },
  {  17, "tp180" },
  {  18, "tp190" },
  {  19, "tp200" },
  {  20, "tp210" },
  {  21, "tp220" },
  {  22, "tp230" },
  {  23, "tp240" },
  {  24, "tp250" },
  {  25, "tp260" },
  {  26, "tp270" },
  {  27, "tp280" },
  {  28, "tp290" },
  {  29, "tp300" },
  {  30, "tp310" },
  {  31, "tp320" },
  {  32, "tp330" },
  {  33, "tp340" },
  {  34, "tp350" },
  {  35, "tp360" },
  {  36, "tp370" },
  {  37, "tp380" },
  {  38, "tp390" },
  {  39, "tp400" },
  {  40, "tp410" },
  {  41, "tp420" },
  {  42, "tp430" },
  {  43, "tp440" },
  {  44, "tp450" },
  {  45, "tp460" },
  {  46, "tp470" },
  {  47, "tp480" },
  {  48, "tp490" },
  {  49, "tp500" },
  {  50, "tp510" },
  {  51, "tp520" },
  {  52, "tp530" },
  {  53, "tp540" },
  {  54, "tp550" },
  {  55, "tp600" },
  {  56, "tp650" },
  {  57, "tp700" },
  {  58, "tp750" },
  {  59, "tp800" },
  {  60, "tp850" },
  {  61, "tp900" },
  {  62, "tp950" },
  {  63, "tp1000" },
  { 0, NULL }
};

static value_string_ext rrc_TimerPoll_vals_ext = VALUE_STRING_EXT_INIT(rrc_TimerPoll_vals);


static int
dissect_rrc_TimerPoll(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     64, NULL, FALSE, 0, NULL);

  return offset;
}


static const value_string rrc_Poll_PDU_vals[] = {
  {   0, "pdu1" },
  {   1, "pdu2" },
  {   2, "pdu4" },
  {   3, "pdu8" },
  {   4, "pdu16" },
  {   5, "pdu32" },
  {   6, "pdu64" },
  {   7, "pdu128" },
  { 0, NULL }
};


static int
dissect_rrc_Poll_PDU(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     8, NULL, FALSE, 0, NULL);

  return offset;
}


static const value_string rrc_Poll_SDU_vals[] = {
  {   0, "sdu1" },
  {   1, "sdu4" },
  {   2, "sdu16" },
  {   3, "sdu64" },
  { 0, NULL }
};


static int
dissect_rrc_Poll_SDU(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     4, NULL, FALSE, 0, NULL);

  return offset;
}


static const value_string rrc_PollWindow_vals[] = {
  {   0, "pw50" },
  {   1, "pw60" },
  {   2, "pw70" },
  {   3, "pw80" },
  {   4, "pw85" },
  {   5, "pw90" },
  {   6, "pw95" },
  {   7, "pw99" },
  { 0, NULL }
};


static int
dissect_rrc_PollWindow(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     8, NULL, FALSE, 0, NULL);

  return offset;
}


static const value_string rrc_TimerPollPeriodic_vals[] = {
  {   0, "tper100" },
  {   1, "tper200" },
  {   2, "tper300" },
  {   3, "tper400" },
  {   4, "tper500" },
  {   5, "tper750" },
  {   6, "tper1000" },
  {   7, "tper2000" },
  { 0, NULL }
};


static int
dissect_rrc_TimerPollPeriodic(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     8, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t PollingInfo_sequence[] = {
  { &hf_rrc_timerPollProhibit, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_TimerPollProhibit },
  { &hf_rrc_timerPoll       , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_TimerPoll },
  { &hf_rrc_poll_PDU        , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_Poll_PDU },
  { &hf_rrc_poll_SDU        , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_Poll_SDU },
  { &hf_rrc_lastTransmissionPDU_Poll, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { &hf_rrc_lastRetransmissionPDU_Poll, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { &hf_rrc_pollWindow      , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_PollWindow },
  { &hf_rrc_timerPollPeriodic, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_TimerPollPeriodic },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_PollingInfo(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_PollingInfo, PollingInfo_sequence);

  return offset;
}


static const per_sequence_t UL_AM_RLC_Mode_sequence[] = {
  { &hf_rrc_transmissionRLC_Discard, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TransmissionRLC_Discard },
  { &hf_rrc_transmissionWindowSize, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TransmissionWindowSize },
  { &hf_rrc_timerRST        , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TimerRST },
  { &hf_rrc_max_RST         , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MaxRST },
  { &hf_rrc_pollingInfo     , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_PollingInfo },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UL_AM_RLC_Mode(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UL_AM_RLC_Mode, UL_AM_RLC_Mode_sequence);

  return offset;
}


static const per_sequence_t UL_UM_RLC_Mode_sequence[] = {
  { &hf_rrc_transmissionRLC_Discard, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_TransmissionRLC_Discard },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UL_UM_RLC_Mode(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UL_UM_RLC_Mode, UL_UM_RLC_Mode_sequence);

  return offset;
}


static const per_sequence_t UL_TM_RLC_Mode_sequence[] = {
  { &hf_rrc_transmissionRLC_Discard, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_TransmissionRLC_Discard },
  { &hf_rrc_segmentationIndication, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UL_TM_RLC_Mode(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UL_TM_RLC_Mode, UL_TM_RLC_Mode_sequence);

  return offset;
}


static const value_string rrc_UL_RLC_Mode_vals[] = {
  {   0, "ul-AM-RLC-Mode" },
  {   1, "ul-UM-RLC-Mode" },
  {   2, "ul-TM-RLC-Mode" },
  {   3, "spare" },
  { 0, NULL }
};

static const per_choice_t UL_RLC_Mode_choice[] = {
  {   0, &hf_rrc_ul_AM_RLC_Mode  , ASN1_NO_EXTENSIONS     , dissect_rrc_UL_AM_RLC_Mode },
  {   1, &hf_rrc_ul_UM_RLC_Mode  , ASN1_NO_EXTENSIONS     , dissect_rrc_UL_UM_RLC_Mode },
  {   2, &hf_rrc_ul_TM_RLC_Mode  , ASN1_NO_EXTENSIONS     , dissect_rrc_UL_TM_RLC_Mode },
  {   3, &hf_rrc_spare           , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_UL_RLC_Mode(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_UL_RLC_Mode, UL_RLC_Mode_choice,
                                 NULL);

  return offset;
}


static const value_string rrc_ReceivingWindowSize_vals[] = {
  {   0, "rw1" },
  {   1, "rw8" },
  {   2, "rw16" },
  {   3, "rw32" },
  {   4, "rw64" },
  {   5, "rw128" },
  {   6, "rw256" },
  {   7, "rw512" },
  {   8, "rw768" },
  {   9, "rw1024" },
  {  10, "rw1536" },
  {  11, "rw2047" },
  {  12, "rw2560" },
  {  13, "rw3072" },
  {  14, "rw3584" },
  {  15, "rw4095" },
  { 0, NULL }
};


static int
dissect_rrc_ReceivingWindowSize(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     16, NULL, FALSE, 0, NULL);

  return offset;
}


static const value_string rrc_TimerStatusProhibit_vals[] = {
  {   0, "tsp10" },
  {   1, "tsp20" },
  {   2, "tsp30" },
  {   3, "tsp40" },
  {   4, "tsp50" },
  {   5, "tsp60" },
  {   6, "tsp70" },
  {   7, "tsp80" },
  {   8, "tsp90" },
  {   9, "tsp100" },
  {  10, "tsp110" },
  {  11, "tsp120" },
  {  12, "tsp130" },
  {  13, "tsp140" },
  {  14, "tsp150" },
  {  15, "tsp160" },
  {  16, "tsp170" },
  {  17, "tsp180" },
  {  18, "tsp190" },
  {  19, "tsp200" },
  {  20, "tsp210" },
  {  21, "tsp220" },
  {  22, "tsp230" },
  {  23, "tsp240" },
  {  24, "tsp250" },
  {  25, "tsp260" },
  {  26, "tsp270" },
  {  27, "tsp280" },
  {  28, "tsp290" },
  {  29, "tsp300" },
  {  30, "tsp310" },
  {  31, "tsp320" },
  {  32, "tsp330" },
  {  33, "tsp340" },
  {  34, "tsp350" },
  {  35, "tsp360" },
  {  36, "tsp370" },
  {  37, "tsp380" },
  {  38, "tsp390" },
  {  39, "tsp400" },
  {  40, "tsp410" },
  {  41, "tsp420" },
  {  42, "tsp430" },
  {  43, "tsp440" },
  {  44, "tsp450" },
  {  45, "tsp460" },
  {  46, "tsp470" },
  {  47, "tsp480" },
  {  48, "tsp490" },
  {  49, "tsp500" },
  {  50, "tsp510" },
  {  51, "tsp520" },
  {  52, "tsp530" },
  {  53, "tsp540" },
  {  54, "tsp550" },
  {  55, "tsp600" },
  {  56, "tsp650" },
  {  57, "tsp700" },
  {  58, "tsp750" },
  {  59, "tsp800" },
  {  60, "tsp850" },
  {  61, "tsp900" },
  {  62, "tsp950" },
  {  63, "tsp1000" },
  { 0, NULL }
};

static value_string_ext rrc_TimerStatusProhibit_vals_ext = VALUE_STRING_EXT_INIT(rrc_TimerStatusProhibit_vals);


static int
dissect_rrc_TimerStatusProhibit(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     64, NULL, FALSE, 0, NULL);

  return offset;
}


static const value_string rrc_TimerEPC_vals[] = {
  {   0, "te50" },
  {   1, "te60" },
  {   2, "te70" },
  {   3, "te80" },
  {   4, "te90" },
  {   5, "te100" },
  {   6, "te120" },
  {   7, "te140" },
  {   8, "te160" },
  {   9, "te180" },
  {  10, "te200" },
  {  11, "te300" },
  {  12, "te400" },
  {  13, "te500" },
  {  14, "te700" },
  {  15, "te900" },
  { 0, NULL }
};


static int
dissect_rrc_TimerEPC(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     16, NULL, FALSE, 0, NULL);

  return offset;
}


static const value_string rrc_TimerStatusPeriodic_vals[] = {
  {   0, "tsp100" },
  {   1, "tsp200" },
  {   2, "tsp300" },
  {   3, "tsp400" },
  {   4, "tsp500" },
  {   5, "tsp750" },
  {   6, "tsp1000" },
  {   7, "tsp2000" },
  { 0, NULL }
};


static int
dissect_rrc_TimerStatusPeriodic(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     8, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t DL_RLC_StatusInfo_sequence[] = {
  { &hf_rrc_timerStatusProhibit, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_TimerStatusProhibit },
  { &hf_rrc_dummy_31        , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_TimerEPC },
  { &hf_rrc_missingPDU_Indicator, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { &hf_rrc_timerStatusPeriodic, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_TimerStatusPeriodic },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_DL_RLC_StatusInfo(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_DL_RLC_StatusInfo, DL_RLC_StatusInfo_sequence);

  return offset;
}


static const per_sequence_t DL_AM_RLC_Mode_sequence[] = {
  { &hf_rrc_inSequenceDelivery, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { &hf_rrc_receivingWindowSize, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_ReceivingWindowSize },
  { &hf_rrc_dl_RLC_StatusInfo, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DL_RLC_StatusInfo },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_DL_AM_RLC_Mode(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_DL_AM_RLC_Mode, DL_AM_RLC_Mode_sequence);

  return offset;
}


static const per_sequence_t DL_TM_RLC_Mode_sequence[] = {
  { &hf_rrc_segmentationIndication, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_DL_TM_RLC_Mode(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_DL_TM_RLC_Mode, DL_TM_RLC_Mode_sequence);

  return offset;
}


static const value_string rrc_DL_RLC_Mode_vals[] = {
  {   0, "dl-AM-RLC-Mode" },
  {   1, "dl-UM-RLC-Mode" },
  {   2, "dl-TM-RLC-Mode" },
  { 0, NULL }
};

static const per_choice_t DL_RLC_Mode_choice[] = {
  {   0, &hf_rrc_dl_AM_RLC_Mode  , ASN1_NO_EXTENSIONS     , dissect_rrc_DL_AM_RLC_Mode },
  {   1, &hf_rrc_dl_UM_RLC_Mode  , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   2, &hf_rrc_dl_TM_RLC_Mode  , ASN1_NO_EXTENSIONS     , dissect_rrc_DL_TM_RLC_Mode },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_DL_RLC_Mode(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_DL_RLC_Mode, DL_RLC_Mode_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t RLC_Info_sequence[] = {
  { &hf_rrc_ul_RLC_Mode     , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_RLC_Mode },
  { &hf_rrc_dl_RLC_Mode     , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_RLC_Mode },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_RLC_Info(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_RLC_Info, RLC_Info_sequence);

  return offset;
}


static const value_string rrc_UL_TransportChannelType_vals[] = {
  {   0, "dch" },
  {   1, "rach" },
  {   2, "dummy" },
  {   3, "usch" },
  { 0, NULL }
};

static const per_choice_t UL_TransportChannelType_choice[] = {
  {   0, &hf_rrc_dch             , ASN1_NO_EXTENSIONS     , dissect_rrc_TransportChannelIdentity },
  {   1, &hf_rrc_rach            , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   2, &hf_rrc_dummy_24        , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   3, &hf_rrc_usch            , ASN1_NO_EXTENSIONS     , dissect_rrc_TransportChannelIdentity },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_UL_TransportChannelType(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_UL_TransportChannelType, UL_TransportChannelType_choice,
                                 NULL);

  return offset;
}



static int
dissect_rrc_LogicalChannelIdentity(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            1U, 15U, NULL, FALSE);

  return offset;
}



static int
dissect_rrc_INTEGER_1_maxTF(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            1U, maxTF, NULL, FALSE);

  return offset;
}


static const per_sequence_t RLC_SizeInfo_sequence[] = {
  { &hf_rrc_rlc_SizeIndex   , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_1_maxTF },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_RLC_SizeInfo(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_RLC_SizeInfo, RLC_SizeInfo_sequence);

  return offset;
}


static const per_sequence_t RLC_SizeExplicitList_sequence_of[1] = {
  { &hf_rrc_RLC_SizeExplicitList_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RLC_SizeInfo },
};

static int
dissect_rrc_RLC_SizeExplicitList(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_RLC_SizeExplicitList, RLC_SizeExplicitList_sequence_of,
                                                  1, maxTF, FALSE);

  return offset;
}


static const value_string rrc_T_rlc_SizeList_vals[] = {
  {   0, "allSizes" },
  {   1, "configured" },
  {   2, "explicitList" },
  { 0, NULL }
};

static const per_choice_t T_rlc_SizeList_choice[] = {
  {   0, &hf_rrc_allSizes        , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   1, &hf_rrc_configured      , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   2, &hf_rrc_explicitList    , ASN1_NO_EXTENSIONS     , dissect_rrc_RLC_SizeExplicitList },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_rlc_SizeList(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_rlc_SizeList, T_rlc_SizeList_choice,
                                 NULL);

  return offset;
}



static int
dissect_rrc_MAC_LogicalChannelPriority(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            1U, 8U, NULL, FALSE);

  return offset;
}


static const per_sequence_t UL_LogicalChannelMapping_sequence[] = {
  { &hf_rrc_ul_TransportChannelType, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UL_TransportChannelType },
  { &hf_rrc_logicalChannelIdentity, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_LogicalChannelIdentity },
  { &hf_rrc_rlc_SizeList    , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_rlc_SizeList },
  { &hf_rrc_mac_LogicalChannelPriority, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MAC_LogicalChannelPriority },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UL_LogicalChannelMapping(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UL_LogicalChannelMapping, UL_LogicalChannelMapping_sequence);

  return offset;
}


static const per_sequence_t SEQUENCE_SIZE_maxLoCHperRLC_OF_UL_LogicalChannelMapping_sequence_of[1] = {
  { &hf_rrc_ul_LogicalChannelMapping_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UL_LogicalChannelMapping },
};

static int
dissect_rrc_SEQUENCE_SIZE_maxLoCHperRLC_OF_UL_LogicalChannelMapping(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_SEQUENCE_SIZE_maxLoCHperRLC_OF_UL_LogicalChannelMapping, SEQUENCE_SIZE_maxLoCHperRLC_OF_UL_LogicalChannelMapping_sequence_of,
                                                  maxLoCHperRLC, maxLoCHperRLC, FALSE);

  return offset;
}


static const per_sequence_t UL_LogicalChannelMappingList_sequence[] = {
  { &hf_rrc_rlc_LogicalChannelMappingIndicator, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { &hf_rrc_ul_LogicalChannelMapping, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SEQUENCE_SIZE_maxLoCHperRLC_OF_UL_LogicalChannelMapping },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UL_LogicalChannelMappingList(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UL_LogicalChannelMappingList, UL_LogicalChannelMappingList_sequence);

  return offset;
}


static const value_string rrc_UL_LogicalChannelMappings_vals[] = {
  {   0, "oneLogicalChannel" },
  {   1, "twoLogicalChannels" },
  { 0, NULL }
};

static const per_choice_t UL_LogicalChannelMappings_choice[] = {
  {   0, &hf_rrc_oneLogicalChannel, ASN1_NO_EXTENSIONS     , dissect_rrc_UL_LogicalChannelMapping },
  {   1, &hf_rrc_twoLogicalChannels, ASN1_NO_EXTENSIONS     , dissect_rrc_UL_LogicalChannelMappingList },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_UL_LogicalChannelMappings(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_UL_LogicalChannelMappings, UL_LogicalChannelMappings_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t TransportChannelIdentityDCHandDSCH_sequence[] = {
  { &hf_rrc_dch_transport_ch_id, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TransportChannelIdentity },
  { &hf_rrc_dsch_transport_ch_id, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TransportChannelIdentity },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_TransportChannelIdentityDCHandDSCH(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_TransportChannelIdentityDCHandDSCH, TransportChannelIdentityDCHandDSCH_sequence);

  return offset;
}


static const value_string rrc_DL_TransportChannelType_vals[] = {
  {   0, "dch" },
  {   1, "fach" },
  {   2, "dsch" },
  {   3, "dch-and-dsch" },
  { 0, NULL }
};

static const per_choice_t DL_TransportChannelType_choice[] = {
  {   0, &hf_rrc_dch             , ASN1_NO_EXTENSIONS     , dissect_rrc_TransportChannelIdentity },
  {   1, &hf_rrc_fach            , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   2, &hf_rrc_dsch            , ASN1_NO_EXTENSIONS     , dissect_rrc_TransportChannelIdentity },
  {   3, &hf_rrc_dch_and_dsch    , ASN1_NO_EXTENSIONS     , dissect_rrc_TransportChannelIdentityDCHandDSCH },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_DL_TransportChannelType(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_DL_TransportChannelType, DL_TransportChannelType_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t DL_LogicalChannelMapping_sequence[] = {
  { &hf_rrc_dl_TransportChannelType, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DL_TransportChannelType },
  { &hf_rrc_logicalChannelIdentity, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_LogicalChannelIdentity },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_DL_LogicalChannelMapping(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_DL_LogicalChannelMapping, DL_LogicalChannelMapping_sequence);

  return offset;
}


static const per_sequence_t DL_LogicalChannelMappingList_sequence_of[1] = {
  { &hf_rrc_DL_LogicalChannelMappingList_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DL_LogicalChannelMapping },
};

static int
dissect_rrc_DL_LogicalChannelMappingList(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_DL_LogicalChannelMappingList, DL_LogicalChannelMappingList_sequence_of,
                                                  1, maxLoCHperRLC, FALSE);

  return offset;
}


static const per_sequence_t RB_MappingOption_sequence[] = {
  { &hf_rrc_ul_LogicalChannelMappings, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_LogicalChannelMappings },
  { &hf_rrc_dl_LogicalChannelMappingList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_LogicalChannelMappingList },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_RB_MappingOption(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_RB_MappingOption, RB_MappingOption_sequence);

  return offset;
}


static const per_sequence_t RB_MappingInfo_sequence_of[1] = {
  { &hf_rrc_RB_MappingInfo_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RB_MappingOption },
};

static int
dissect_rrc_RB_MappingInfo(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_RB_MappingInfo, RB_MappingInfo_sequence_of,
                                                  1, maxRBMuxOptions, FALSE);

  return offset;
}


static const value_string rrc_RB_StopContinue_vals[] = {
  {   0, "stopRB" },
  {   1, "continueRB" },
  { 0, NULL }
};


static int
dissect_rrc_RB_StopContinue(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     2, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t RB_InformationReconfig_sequence[] = {
  { &hf_rrc_rb_Identity     , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RB_Identity },
  { &hf_rrc_pdcp_Info_02    , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_PDCP_InfoReconfig },
  { &hf_rrc_pdcp_SN_Info    , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_PDCP_SN_Info },
  { &hf_rrc_rlc_Info        , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RLC_Info },
  { &hf_rrc_rb_MappingInfo  , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RB_MappingInfo },
  { &hf_rrc_rb_StopContinue , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RB_StopContinue },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_RB_InformationReconfig(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_RB_InformationReconfig, RB_InformationReconfig_sequence);

  return offset;
}


static const per_sequence_t RB_InformationReconfigList_sequence_of[1] = {
  { &hf_rrc_RB_InformationReconfigList_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RB_InformationReconfig },
};

static int
dissect_rrc_RB_InformationReconfigList(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_RB_InformationReconfigList, RB_InformationReconfigList_sequence_of,
                                                  1, maxRB, FALSE);

  return offset;
}


static const per_sequence_t RB_InformationAffected_sequence[] = {
  { &hf_rrc_rb_Identity     , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RB_Identity },
  { &hf_rrc_rb_MappingInfo  , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RB_MappingInfo },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_RB_InformationAffected(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_RB_InformationAffected, RB_InformationAffected_sequence);

  return offset;
}


static const per_sequence_t RB_InformationAffectedList_sequence_of[1] = {
  { &hf_rrc_RB_InformationAffectedList_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RB_InformationAffected },
};

static int
dissect_rrc_RB_InformationAffectedList(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_RB_InformationAffectedList, RB_InformationAffectedList_sequence_of,
                                                  1, maxRB, FALSE);

  return offset;
}



static int
dissect_rrc_TFC_Value(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            0U, 1023U, NULL, FALSE);

  return offset;
}


static const per_sequence_t AllowedTFC_List_sequence_of[1] = {
  { &hf_rrc_AllowedTFC_List_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TFC_Value },
};

static int
dissect_rrc_AllowedTFC_List(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_AllowedTFC_List, AllowedTFC_List_sequence_of,
                                                  1, maxTFC, FALSE);

  return offset;
}


static const per_sequence_t Non_allowedTFC_List_sequence_of[1] = {
  { &hf_rrc_Non_allowedTFC_List_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TFC_Value },
};

static int
dissect_rrc_Non_allowedTFC_List(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_Non_allowedTFC_List, Non_allowedTFC_List_sequence_of,
                                                  1, maxTFC, FALSE);

  return offset;
}


static const value_string rrc_UL_TrCH_Type_vals[] = {
  {   0, "dch" },
  {   1, "usch" },
  { 0, NULL }
};


static int
dissect_rrc_UL_TrCH_Type(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     2, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t AllowedTFI_List_sequence_of[1] = {
  { &hf_rrc_AllowedTFI_List_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_0_31 },
};

static int
dissect_rrc_AllowedTFI_List(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_AllowedTFI_List, AllowedTFI_List_sequence_of,
                                                  1, maxTF, FALSE);

  return offset;
}


static const per_sequence_t RestrictedTrChInfo_sequence[] = {
  { &hf_rrc_ul_TransportChannelType_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UL_TrCH_Type },
  { &hf_rrc_restrictedTrChIdentity, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TransportChannelIdentity },
  { &hf_rrc_allowedTFI_List , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_AllowedTFI_List },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_RestrictedTrChInfo(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_RestrictedTrChInfo, RestrictedTrChInfo_sequence);

  return offset;
}


static const per_sequence_t RestrictedTrChInfoList_sequence_of[1] = {
  { &hf_rrc_RestrictedTrChInfoList_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RestrictedTrChInfo },
};

static int
dissect_rrc_RestrictedTrChInfoList(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_RestrictedTrChInfoList, RestrictedTrChInfoList_sequence_of,
                                                  1, maxTrCH, FALSE);

  return offset;
}


static const value_string rrc_TFC_Subset_vals[] = {
  {   0, "minimumAllowedTFC-Number" },
  {   1, "allowedTFC-List" },
  {   2, "non-allowedTFC-List" },
  {   3, "restrictedTrChInfoList" },
  {   4, "fullTFCS" },
  { 0, NULL }
};

static const per_choice_t TFC_Subset_choice[] = {
  {   0, &hf_rrc_minimumAllowedTFC_Number, ASN1_NO_EXTENSIONS     , dissect_rrc_TFC_Value },
  {   1, &hf_rrc_allowedTFC_List , ASN1_NO_EXTENSIONS     , dissect_rrc_AllowedTFC_List },
  {   2, &hf_rrc_non_allowedTFC_List, ASN1_NO_EXTENSIONS     , dissect_rrc_Non_allowedTFC_List },
  {   3, &hf_rrc_restrictedTrChInfoList, ASN1_NO_EXTENSIONS     , dissect_rrc_RestrictedTrChInfoList },
  {   4, &hf_rrc_fullTFCS        , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_TFC_Subset(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_TFC_Subset, TFC_Subset_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t T_fdd_56_sequence[] = {
  { &hf_rrc_ul_TFCS         , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TFCS },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_fdd_56(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_fdd_56, T_fdd_56_sequence);

  return offset;
}


static const per_sequence_t IndividualUL_CCTrCH_Info_sequence[] = {
  { &hf_rrc_ul_TFCS_Identity, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TFCS_Identity },
  { &hf_rrc_ul_TFCS         , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TFCS },
  { &hf_rrc_tfc_Subset      , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TFC_Subset },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_IndividualUL_CCTrCH_Info(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_IndividualUL_CCTrCH_Info, IndividualUL_CCTrCH_Info_sequence);

  return offset;
}


static const per_sequence_t IndividualUL_CCTrCH_InfoList_sequence_of[1] = {
  { &hf_rrc_IndividualUL_CCTrCH_InfoList_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_IndividualUL_CCTrCH_Info },
};

static int
dissect_rrc_IndividualUL_CCTrCH_InfoList(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_IndividualUL_CCTrCH_InfoList, IndividualUL_CCTrCH_InfoList_sequence_of,
                                                  1, maxCCTrCH, FALSE);

  return offset;
}


static const per_sequence_t T_tdd_17_sequence[] = {
  { &hf_rrc_individualUL_CCTrCH_InfoList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_IndividualUL_CCTrCH_InfoList },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_tdd_17(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_tdd_17, T_tdd_17_sequence);

  return offset;
}


static const value_string rrc_T_modeSpecificInfo_25_vals[] = {
  {   0, "fdd" },
  {   1, "tdd" },
  { 0, NULL }
};

static const per_choice_t T_modeSpecificInfo_25_choice[] = {
  {   0, &hf_rrc_fdd_57          , ASN1_NO_EXTENSIONS     , dissect_rrc_T_fdd_56 },
  {   1, &hf_rrc_tdd_18          , ASN1_NO_EXTENSIONS     , dissect_rrc_T_tdd_17 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_modeSpecificInfo_25(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_modeSpecificInfo_25, T_modeSpecificInfo_25_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t UL_CommonTransChInfo_sequence[] = {
  { &hf_rrc_tfc_Subset      , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_TFC_Subset },
  { &hf_rrc_prach_TFCS      , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_TFCS },
  { &hf_rrc_modeSpecificInfo_25, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_modeSpecificInfo_25 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UL_CommonTransChInfo(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UL_CommonTransChInfo, UL_CommonTransChInfo_sequence);

  return offset;
}


static const per_sequence_t UL_TransportChannelIdentity_sequence[] = {
  { &hf_rrc_ul_TransportChannelType_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UL_TrCH_Type },
  { &hf_rrc_ul_TransportChannelIdentity, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TransportChannelIdentity },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UL_TransportChannelIdentity(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UL_TransportChannelIdentity, UL_TransportChannelIdentity_sequence);

  return offset;
}


static const per_sequence_t UL_DeletedTransChInfoList_sequence_of[1] = {
  { &hf_rrc_UL_DeletedTransChInfoList_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UL_TransportChannelIdentity },
};

static int
dissect_rrc_UL_DeletedTransChInfoList(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_UL_DeletedTransChInfoList, UL_DeletedTransChInfoList_sequence_of,
                                                  1, maxTrCH, FALSE);

  return offset;
}


static const per_sequence_t UL_AddReconfTransChInformation_sequence[] = {
  { &hf_rrc_ul_TransportChannelType_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UL_TrCH_Type },
  { &hf_rrc_transportChannelIdentity, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TransportChannelIdentity },
  { &hf_rrc_transportFormatSet, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TransportFormatSet },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UL_AddReconfTransChInformation(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UL_AddReconfTransChInformation, UL_AddReconfTransChInformation_sequence);

  return offset;
}


static const per_sequence_t UL_AddReconfTransChInfoList_sequence_of[1] = {
  { &hf_rrc_UL_AddReconfTransChInfoList_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UL_AddReconfTransChInformation },
};

static int
dissect_rrc_UL_AddReconfTransChInfoList(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_UL_AddReconfTransChInfoList, UL_AddReconfTransChInfoList_sequence_of,
                                                  1, maxTrCHpreconf, FALSE);

  return offset;
}



static int
dissect_rrc_CPCH_SetID(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            1U, maxCPCHsets, NULL, FALSE);

  return offset;
}



static int
dissect_rrc_TransmissionTimeValidity(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            1U, 256U, NULL, FALSE);

  return offset;
}



static int
dissect_rrc_TimeDurationBeforeRetry(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            1U, 256U, NULL, FALSE);

  return offset;
}



static int
dissect_rrc_DRAC_ClassIdentity(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            1U, maxDRACclasses, NULL, FALSE);

  return offset;
}


static const per_sequence_t DRAC_StaticInformation_sequence[] = {
  { &hf_rrc_transmissionTimeValidity, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TransmissionTimeValidity },
  { &hf_rrc_timeDurationBeforeRetry, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TimeDurationBeforeRetry },
  { &hf_rrc_drac_ClassIdentity, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DRAC_ClassIdentity },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_DRAC_StaticInformation(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_DRAC_StaticInformation, DRAC_StaticInformation_sequence);

  return offset;
}


static const per_sequence_t DRAC_StaticInformationList_sequence_of[1] = {
  { &hf_rrc_DRAC_StaticInformationList_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DRAC_StaticInformation },
};

static int
dissect_rrc_DRAC_StaticInformationList(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_DRAC_StaticInformationList, DRAC_StaticInformationList_sequence_of,
                                                  1, maxTrCH, FALSE);

  return offset;
}


static const per_sequence_t T_fdd_sequence[] = {
  { &hf_rrc_dummy_05        , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CPCH_SetID },
  { &hf_rrc_dummy2_02       , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DRAC_StaticInformationList },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_fdd(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_fdd, T_fdd_sequence);

  return offset;
}


static const value_string rrc_T_modeSpecificTransChInfo_vals[] = {
  {   0, "fdd" },
  {   1, "tdd" },
  { 0, NULL }
};

static const per_choice_t T_modeSpecificTransChInfo_choice[] = {
  {   0, &hf_rrc_fdd             , ASN1_NO_EXTENSIONS     , dissect_rrc_T_fdd },
  {   1, &hf_rrc_tdd             , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_modeSpecificTransChInfo(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_modeSpecificTransChInfo, T_modeSpecificTransChInfo_choice,
                                 NULL);

  return offset;
}


static const value_string rrc_T_dl_Parameters_vals[] = {
  {   0, "dl-DCH-TFCS" },
  {   1, "sameAsUL" },
  { 0, NULL }
};

static const per_choice_t T_dl_Parameters_choice[] = {
  {   0, &hf_rrc_dl_DCH_TFCS     , ASN1_NO_EXTENSIONS     , dissect_rrc_TFCS },
  {   1, &hf_rrc_sameAsUL        , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_dl_Parameters(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_dl_Parameters, T_dl_Parameters_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t T_fdd_49_sequence[] = {
  { &hf_rrc_dl_Parameters   , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_dl_Parameters },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_fdd_49(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_fdd_49, T_fdd_49_sequence);

  return offset;
}


static const value_string rrc_T_tfcs_SignallingMode_vals[] = {
  {   0, "explicit-config" },
  {   1, "sameAsUL" },
  { 0, NULL }
};

static const per_choice_t T_tfcs_SignallingMode_choice[] = {
  {   0, &hf_rrc_explicit_config_01, ASN1_NO_EXTENSIONS     , dissect_rrc_TFCS },
  {   1, &hf_rrc_sameAsUL_01     , ASN1_NO_EXTENSIONS     , dissect_rrc_TFCS_Identity },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_tfcs_SignallingMode(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_tfcs_SignallingMode, T_tfcs_SignallingMode_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t IndividualDL_CCTrCH_Info_sequence[] = {
  { &hf_rrc_dl_TFCS_Identity, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TFCS_Identity },
  { &hf_rrc_tfcs_SignallingMode, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_tfcs_SignallingMode },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_IndividualDL_CCTrCH_Info(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_IndividualDL_CCTrCH_Info, IndividualDL_CCTrCH_Info_sequence);

  return offset;
}


static const per_sequence_t IndividualDL_CCTrCH_InfoList_sequence_of[1] = {
  { &hf_rrc_IndividualDL_CCTrCH_InfoList_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_IndividualDL_CCTrCH_Info },
};

static int
dissect_rrc_IndividualDL_CCTrCH_InfoList(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_IndividualDL_CCTrCH_InfoList, IndividualDL_CCTrCH_InfoList_sequence_of,
                                                  1, maxCCTrCH, FALSE);

  return offset;
}


static const per_sequence_t T_tdd_14_sequence[] = {
  { &hf_rrc_individualDL_CCTrCH_InfoList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_IndividualDL_CCTrCH_InfoList },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_tdd_14(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_tdd_14, T_tdd_14_sequence);

  return offset;
}


static const value_string rrc_T_modeSpecificInfo_19_vals[] = {
  {   0, "fdd" },
  {   1, "tdd" },
  { 0, NULL }
};

static const per_choice_t T_modeSpecificInfo_19_choice[] = {
  {   0, &hf_rrc_fdd_50          , ASN1_NO_EXTENSIONS     , dissect_rrc_T_fdd_49 },
  {   1, &hf_rrc_tdd_15          , ASN1_NO_EXTENSIONS     , dissect_rrc_T_tdd_14 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_modeSpecificInfo_19(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_modeSpecificInfo_19, T_modeSpecificInfo_19_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t DL_CommonTransChInfo_sequence[] = {
  { &hf_rrc_sccpch_TFCS     , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_TFCS },
  { &hf_rrc_modeSpecificInfo_19, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_modeSpecificInfo_19 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_DL_CommonTransChInfo(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_DL_CommonTransChInfo, DL_CommonTransChInfo_sequence);

  return offset;
}


static const value_string rrc_DL_TrCH_Type_vals[] = {
  {   0, "dch" },
  {   1, "dsch" },
  { 0, NULL }
};


static int
dissect_rrc_DL_TrCH_Type(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     2, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t DL_TransportChannelIdentity_sequence[] = {
  { &hf_rrc_dl_TransportChannelType_03, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DL_TrCH_Type },
  { &hf_rrc_dl_TransportChannelIdentity, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TransportChannelIdentity },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_DL_TransportChannelIdentity(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_DL_TransportChannelIdentity, DL_TransportChannelIdentity_sequence);

  return offset;
}


static const per_sequence_t DL_DeletedTransChInfoList_sequence_of[1] = {
  { &hf_rrc_DL_DeletedTransChInfoList_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DL_TransportChannelIdentity },
};

static int
dissect_rrc_DL_DeletedTransChInfoList(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_DL_DeletedTransChInfoList, DL_DeletedTransChInfoList_sequence_of,
                                                  1, maxTrCH, FALSE);

  return offset;
}


static const value_string rrc_T_tfs_SignallingMode_vals[] = {
  {   0, "explicit-config" },
  {   1, "sameAsULTrCH" },
  { 0, NULL }
};

static const per_choice_t T_tfs_SignallingMode_choice[] = {
  {   0, &hf_rrc_explicit_config , ASN1_NO_EXTENSIONS     , dissect_rrc_TransportFormatSet },
  {   1, &hf_rrc_sameAsULTrCH    , ASN1_NO_EXTENSIONS     , dissect_rrc_UL_TransportChannelIdentity },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_tfs_SignallingMode(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_tfs_SignallingMode, T_tfs_SignallingMode_choice,
                                 NULL);

  return offset;
}



static int
dissect_rrc_BLER_QualityValue(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            -63, 0U, NULL, FALSE);

  return offset;
}


static const per_sequence_t QualityTarget_sequence[] = {
  { &hf_rrc_bler_QualityValue, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BLER_QualityValue },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_QualityTarget(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_QualityTarget, QualityTarget_sequence);

  return offset;
}


static const value_string rrc_MessType_vals[] = {
  {   0, "transportFormatCombinationControl" },
  { 0, NULL }
};


static int
dissect_rrc_MessType(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t UL_ControlledTrChList_sequence_of[1] = {
  { &hf_rrc_UL_ControlledTrChList_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TransportChannelIdentity },
};

static int
dissect_rrc_UL_ControlledTrChList(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_UL_ControlledTrChList, UL_ControlledTrChList_sequence_of,
                                                  1, maxTrCH, FALSE);

  return offset;
}


static const per_sequence_t T_mode2_sequence[] = {
  { &hf_rrc_ul_controlledTrChList, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UL_ControlledTrChList },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_mode2(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_mode2, T_mode2_sequence);

  return offset;
}


static const value_string rrc_T_tm_SignallingMode_vals[] = {
  {   0, "mode1" },
  {   1, "mode2" },
  { 0, NULL }
};

static const per_choice_t T_tm_SignallingMode_choice[] = {
  {   0, &hf_rrc_mode1           , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   1, &hf_rrc_mode2           , ASN1_NO_EXTENSIONS     , dissect_rrc_T_mode2 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_tm_SignallingMode(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_tm_SignallingMode, T_tm_SignallingMode_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t TM_SignallingInfo_sequence[] = {
  { &hf_rrc_messType        , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MessType },
  { &hf_rrc_tm_SignallingMode, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_tm_SignallingMode },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_TM_SignallingInfo(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_TM_SignallingInfo, TM_SignallingInfo_sequence);

  return offset;
}


static const per_sequence_t DL_AddReconfTransChInformation_sequence[] = {
  { &hf_rrc_dl_TransportChannelType_03, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DL_TrCH_Type },
  { &hf_rrc_dl_transportChannelIdentity, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TransportChannelIdentity },
  { &hf_rrc_tfs_SignallingMode, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_tfs_SignallingMode },
  { &hf_rrc_dch_QualityTarget, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_QualityTarget },
  { &hf_rrc_dummy_35        , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_TM_SignallingInfo },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_DL_AddReconfTransChInformation(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_DL_AddReconfTransChInformation, DL_AddReconfTransChInformation_sequence);

  return offset;
}


static const per_sequence_t DL_AddReconfTransChInfoList_sequence_of[1] = {
  { &hf_rrc_DL_AddReconfTransChInfoList_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DL_AddReconfTransChInformation },
};

static int
dissect_rrc_DL_AddReconfTransChInfoList(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_DL_AddReconfTransChInfoList, DL_AddReconfTransChInfoList_sequence_of,
                                                  1, maxTrCHpreconf, FALSE);

  return offset;
}



static int
dissect_rrc_DPCCH_PowerOffset(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            -82, -3, NULL, FALSE);

  return offset;
}



static int
dissect_rrc_SRB_delay(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            0U, 7U, NULL, FALSE);

  return offset;
}



static int
dissect_rrc_TPC_StepSizeFDD(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            0U, 1U, NULL, FALSE);

  return offset;
}


static const value_string rrc_PowerControlAlgorithm_vals[] = {
  {   0, "algorithm1" },
  {   1, "algorithm2" },
  { 0, NULL }
};

static const per_choice_t PowerControlAlgorithm_choice[] = {
  {   0, &hf_rrc_algorithm1      , ASN1_NO_EXTENSIONS     , dissect_rrc_TPC_StepSizeFDD },
  {   1, &hf_rrc_algorithm2      , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_PowerControlAlgorithm(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_PowerControlAlgorithm, PowerControlAlgorithm_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t T_fdd_121_sequence[] = {
  { &hf_rrc_dpcch_PowerOffset, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DPCCH_PowerOffset },
  { &hf_rrc_pc_Preamble     , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PC_Preamble },
  { &hf_rrc_sRB_delay       , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SRB_delay },
  { &hf_rrc_powerControlAlgorithm, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PowerControlAlgorithm },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_fdd_121(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_fdd_121, T_fdd_121_sequence);

  return offset;
}



static int
dissect_rrc_UL_TargetSIR(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            0U, 62U, NULL, FALSE);

  return offset;
}



static int
dissect_rrc_TDD_UL_Interference(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            -110, -52, NULL, FALSE);

  return offset;
}


static const per_sequence_t IndividualTS_Interference_sequence[] = {
  { &hf_rrc_timeslot        , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TimeslotNumber },
  { &hf_rrc_ul_TimeslotInterference, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TDD_UL_Interference },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_IndividualTS_Interference(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_IndividualTS_Interference, IndividualTS_Interference_sequence);

  return offset;
}


static const per_sequence_t IndividualTS_InterferenceList_sequence_of[1] = {
  { &hf_rrc_IndividualTS_InterferenceList_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_IndividualTS_Interference },
};

static int
dissect_rrc_IndividualTS_InterferenceList(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_IndividualTS_InterferenceList, IndividualTS_InterferenceList_sequence_of,
                                                  1, maxTS, FALSE);

  return offset;
}



static int
dissect_rrc_ConstantValueTdd(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            -35, 10U, NULL, FALSE);

  return offset;
}



static int
dissect_rrc_PrimaryCCPCH_TX_Power(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            6U, 43U, NULL, FALSE);

  return offset;
}


static const per_sequence_t T_individuallySignalled_sequence[] = {
  { &hf_rrc_individualTS_InterferenceList, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_IndividualTS_InterferenceList },
  { &hf_rrc_dpch_ConstantValue, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_ConstantValueTdd },
  { &hf_rrc_primaryCCPCH_TX_Power, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PrimaryCCPCH_TX_Power },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_individuallySignalled(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_individuallySignalled, T_individuallySignalled_sequence);

  return offset;
}


static const value_string rrc_T_ul_OL_PC_Signalling_vals[] = {
  {   0, "broadcast-UL-OL-PC-info" },
  {   1, "individuallySignalled" },
  { 0, NULL }
};

static const per_choice_t T_ul_OL_PC_Signalling_choice[] = {
  {   0, &hf_rrc_broadcast_UL_OL_PC_info, ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   1, &hf_rrc_individuallySignalled, ASN1_NO_EXTENSIONS     , dissect_rrc_T_individuallySignalled },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_ul_OL_PC_Signalling(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_ul_OL_PC_Signalling, T_ul_OL_PC_Signalling_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t T_tdd_71_sequence[] = {
  { &hf_rrc_ul_TargetSIR    , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_TargetSIR },
  { &hf_rrc_ul_OL_PC_Signalling, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_ul_OL_PC_Signalling },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_tdd_71(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_tdd_71, T_tdd_71_sequence);

  return offset;
}


static const value_string rrc_UL_DPCH_PowerControlInfo_vals[] = {
  {   0, "fdd" },
  {   1, "tdd" },
  { 0, NULL }
};

static const per_choice_t UL_DPCH_PowerControlInfo_choice[] = {
  {   0, &hf_rrc_fdd_124         , ASN1_NO_EXTENSIONS     , dissect_rrc_T_fdd_121 },
  {   1, &hf_rrc_tdd_77          , ASN1_NO_EXTENSIONS     , dissect_rrc_T_tdd_71 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_UL_DPCH_PowerControlInfo(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_UL_DPCH_PowerControlInfo, UL_DPCH_PowerControlInfo_choice,
                                 NULL);

  return offset;
}



static int
dissect_rrc_NumberOfDPDCH(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            1U, maxDPDCH_UL, NULL, FALSE);

  return offset;
}


static const value_string rrc_SpreadingFactor_vals[] = {
  {   0, "sf4" },
  {   1, "sf8" },
  {   2, "sf16" },
  {   3, "sf32" },
  {   4, "sf64" },
  {   5, "sf128" },
  {   6, "sf256" },
  { 0, NULL }
};


static int
dissect_rrc_SpreadingFactor(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     7, NULL, FALSE, 0, NULL);

  return offset;
}



static int
dissect_rrc_NumberOfFBI_Bits(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            1U, 2U, NULL, FALSE);

  return offset;
}


static const per_sequence_t T_fdd_113_sequence[] = {
  { &hf_rrc_scramblingCodeType, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_ScramblingCodeType },
  { &hf_rrc_scramblingCode  , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UL_ScramblingCode },
  { &hf_rrc_numberOfDPDCH   , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_NumberOfDPDCH },
  { &hf_rrc_spreadingFactor_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SpreadingFactor },
  { &hf_rrc_tfci_Existence  , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { &hf_rrc_numberOfFBI_Bits, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_NumberOfFBI_Bits },
  { &hf_rrc_puncturingLimit , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PuncturingLimit },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_fdd_113(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_fdd_113, T_fdd_113_sequence);

  return offset;
}



static int
dissect_rrc_UL_TimingAdvance(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            0U, 63U, NULL, FALSE);

  return offset;
}


static const per_sequence_t T_enabled_sequence[] = {
  { &hf_rrc_ul_TimingAdvance, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_TimingAdvance },
  { &hf_rrc_activationTime  , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_ActivationTime },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_enabled(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_enabled, T_enabled_sequence);

  return offset;
}


static const value_string rrc_UL_TimingAdvanceControl_vals[] = {
  {   0, "disabled" },
  {   1, "enabled" },
  { 0, NULL }
};

static const per_choice_t UL_TimingAdvanceControl_choice[] = {
  {   0, &hf_rrc_disabled        , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   1, &hf_rrc_enabled         , ASN1_NO_EXTENSIONS     , dissect_rrc_T_enabled },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_UL_TimingAdvanceControl(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_UL_TimingAdvanceControl, UL_TimingAdvanceControl_choice,
                                 NULL);

  return offset;
}


static const value_string rrc_UL_TS_ChannelisationCode_vals[] = {
  {   0, "cc1-1" },
  {   1, "cc2-1" },
  {   2, "cc2-2" },
  {   3, "cc4-1" },
  {   4, "cc4-2" },
  {   5, "cc4-3" },
  {   6, "cc4-4" },
  {   7, "cc8-1" },
  {   8, "cc8-2" },
  {   9, "cc8-3" },
  {  10, "cc8-4" },
  {  11, "cc8-5" },
  {  12, "cc8-6" },
  {  13, "cc8-7" },
  {  14, "cc8-8" },
  {  15, "cc16-1" },
  {  16, "cc16-2" },
  {  17, "cc16-3" },
  {  18, "cc16-4" },
  {  19, "cc16-5" },
  {  20, "cc16-6" },
  {  21, "cc16-7" },
  {  22, "cc16-8" },
  {  23, "cc16-9" },
  {  24, "cc16-10" },
  {  25, "cc16-11" },
  {  26, "cc16-12" },
  {  27, "cc16-13" },
  {  28, "cc16-14" },
  {  29, "cc16-15" },
  {  30, "cc16-16" },
  { 0, NULL }
};

static value_string_ext rrc_UL_TS_ChannelisationCode_vals_ext = VALUE_STRING_EXT_INIT(rrc_UL_TS_ChannelisationCode_vals);


static int
dissect_rrc_UL_TS_ChannelisationCode(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     31, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t UL_TS_ChannelisationCodeList_sequence_of[1] = {
  { &hf_rrc_UL_TS_ChannelisationCodeList_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UL_TS_ChannelisationCode },
};

static int
dissect_rrc_UL_TS_ChannelisationCodeList(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_UL_TS_ChannelisationCodeList, UL_TS_ChannelisationCodeList_sequence_of,
                                                  1, 2, FALSE);

  return offset;
}


static const per_sequence_t T_consecutive_02_sequence[] = {
  { &hf_rrc_numAdditionalTimeslots, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_1_maxTS_1 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_consecutive_02(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_consecutive_02, T_consecutive_02_sequence);

  return offset;
}


static const per_sequence_t T_sameAsLast_04_sequence[] = {
  { &hf_rrc_timeslotNumber  , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TimeslotNumber },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_sameAsLast_04(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_sameAsLast_04, T_sameAsLast_04_sequence);

  return offset;
}


static const per_sequence_t T_newParameters_04_sequence[] = {
  { &hf_rrc_individualTimeslotInfo, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_IndividualTimeslotInfo },
  { &hf_rrc_ul_TS_ChannelisationCodeList, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UL_TS_ChannelisationCodeList },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_newParameters_04(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_newParameters_04, T_newParameters_04_sequence);

  return offset;
}


static const value_string rrc_T_parameters_04_vals[] = {
  {   0, "sameAsLast" },
  {   1, "newParameters" },
  { 0, NULL }
};

static const per_choice_t T_parameters_04_choice[] = {
  {   0, &hf_rrc_sameAsLast_04   , ASN1_NO_EXTENSIONS     , dissect_rrc_T_sameAsLast_04 },
  {   1, &hf_rrc_newParameters_04, ASN1_NO_EXTENSIONS     , dissect_rrc_T_newParameters_04 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_parameters_04(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_parameters_04, T_parameters_04_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t UplinkAdditionalTimeslots_sequence[] = {
  { &hf_rrc_parameters_04   , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_parameters_04 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UplinkAdditionalTimeslots(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UplinkAdditionalTimeslots, UplinkAdditionalTimeslots_sequence);

  return offset;
}


static const per_sequence_t SEQUENCE_SIZE_1_maxTS_1_OF_UplinkAdditionalTimeslots_sequence_of[1] = {
  { &hf_rrc_timeslotList_item_04, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UplinkAdditionalTimeslots },
};

static int
dissect_rrc_SEQUENCE_SIZE_1_maxTS_1_OF_UplinkAdditionalTimeslots(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_SEQUENCE_SIZE_1_maxTS_1_OF_UplinkAdditionalTimeslots, SEQUENCE_SIZE_1_maxTS_1_OF_UplinkAdditionalTimeslots_sequence_of,
                                                  1, maxTS_1, FALSE);

  return offset;
}


static const value_string rrc_T_additionalTimeslots_04_vals[] = {
  {   0, "consecutive" },
  {   1, "timeslotList" },
  { 0, NULL }
};

static const per_choice_t T_additionalTimeslots_04_choice[] = {
  {   0, &hf_rrc_consecutive_04  , ASN1_NO_EXTENSIONS     , dissect_rrc_T_consecutive_02 },
  {   1, &hf_rrc_timeslotList_04 , ASN1_NO_EXTENSIONS     , dissect_rrc_SEQUENCE_SIZE_1_maxTS_1_OF_UplinkAdditionalTimeslots },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_additionalTimeslots_04(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_additionalTimeslots_04, T_additionalTimeslots_04_choice,
                                 NULL);

  return offset;
}


static const value_string rrc_T_moreTimeslots_04_vals[] = {
  {   0, "noMore" },
  {   1, "additionalTimeslots" },
  { 0, NULL }
};

static const per_choice_t T_moreTimeslots_04_choice[] = {
  {   0, &hf_rrc_noMore          , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   1, &hf_rrc_additionalTimeslots_04, ASN1_NO_EXTENSIONS     , dissect_rrc_T_additionalTimeslots_04 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_moreTimeslots_04(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_moreTimeslots_04, T_moreTimeslots_04_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t UplinkTimeslotsCodes_sequence[] = {
  { &hf_rrc_dynamicSFusage  , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { &hf_rrc_firstIndividualTimeslotInfo, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_IndividualTimeslotInfo },
  { &hf_rrc_ul_TS_ChannelisationCodeList, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UL_TS_ChannelisationCodeList },
  { &hf_rrc_moreTimeslots_04, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_moreTimeslots_04 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UplinkTimeslotsCodes(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UplinkTimeslotsCodes, UplinkTimeslotsCodes_sequence);

  return offset;
}


static const per_sequence_t UL_CCTrCH_sequence[] = {
  { &hf_rrc_tfcs_ID_01      , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_TFCS_IdentityPlain },
  { &hf_rrc_ul_TargetSIR    , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UL_TargetSIR },
  { &hf_rrc_timeInfo        , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TimeInfo },
  { &hf_rrc_commonTimeslotInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CommonTimeslotInfo },
  { &hf_rrc_ul_CCTrCH_TimeslotsCodes, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UplinkTimeslotsCodes },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UL_CCTrCH(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UL_CCTrCH, UL_CCTrCH_sequence);

  return offset;
}


static const per_sequence_t UL_CCTrCHList_sequence_of[1] = {
  { &hf_rrc_UL_CCTrCHList_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UL_CCTrCH },
};

static int
dissect_rrc_UL_CCTrCHList(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_UL_CCTrCHList, UL_CCTrCHList_sequence_of,
                                                  1, maxCCTrCH, FALSE);

  return offset;
}


static const per_sequence_t UL_CCTrCHListToRemove_sequence_of[1] = {
  { &hf_rrc_UL_CCTrCHListToRemove_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TFCS_IdentityPlain },
};

static int
dissect_rrc_UL_CCTrCHListToRemove(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_UL_CCTrCHListToRemove, UL_CCTrCHListToRemove_sequence_of,
                                                  1, maxCCTrCH, FALSE);

  return offset;
}


static const per_sequence_t T_tdd_64_sequence[] = {
  { &hf_rrc_ul_TimingAdvance_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_TimingAdvanceControl },
  { &hf_rrc_ul_CCTrCHList   , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_CCTrCHList },
  { &hf_rrc_ul_CCTrCHListToRemove, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_CCTrCHListToRemove },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_tdd_64(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_tdd_64, T_tdd_64_sequence);

  return offset;
}


static const value_string rrc_T_modeSpecificInfo_81_vals[] = {
  {   0, "fdd" },
  {   1, "tdd" },
  { 0, NULL }
};

static const per_choice_t T_modeSpecificInfo_81_choice[] = {
  {   0, &hf_rrc_fdd_116         , ASN1_NO_EXTENSIONS     , dissect_rrc_T_fdd_113 },
  {   1, &hf_rrc_tdd_70          , ASN1_NO_EXTENSIONS     , dissect_rrc_T_tdd_64 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_modeSpecificInfo_81(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_modeSpecificInfo_81, T_modeSpecificInfo_81_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t UL_DPCH_Info_sequence[] = {
  { &hf_rrc_ul_DPCH_PowerControlInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_DPCH_PowerControlInfo },
  { &hf_rrc_modeSpecificInfo_81, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_modeSpecificInfo_81 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UL_DPCH_Info(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UL_DPCH_Info, UL_DPCH_Info_sequence);

  return offset;
}



static int
dissect_rrc_AP_PreambleScramblingCode(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            0U, 79U, NULL, FALSE);

  return offset;
}



static int
dissect_rrc_AP_AICH_ChannelisationCode(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            0U, 255U, NULL, FALSE);

  return offset;
}



static int
dissect_rrc_CD_PreambleScramblingCode(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            0U, 79U, NULL, FALSE);

  return offset;
}



static int
dissect_rrc_CD_CA_ICH_ChannelisationCode(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            0U, 255U, NULL, FALSE);

  return offset;
}



static int
dissect_rrc_CD_AccessSlotSubchannel(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            0U, 11U, NULL, FALSE);

  return offset;
}


static const per_sequence_t CD_AccessSlotSubchannelList_sequence_of[1] = {
  { &hf_rrc_CD_AccessSlotSubchannelList_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CD_AccessSlotSubchannel },
};

static int
dissect_rrc_CD_AccessSlotSubchannelList(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_CD_AccessSlotSubchannelList, CD_AccessSlotSubchannelList_sequence_of,
                                                  1, maxPCPCH_CDsubCh, FALSE);

  return offset;
}



static int
dissect_rrc_CD_SignatureCode(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            0U, 15U, NULL, FALSE);

  return offset;
}


static const per_sequence_t CD_SignatureCodeList_sequence_of[1] = {
  { &hf_rrc_CD_SignatureCodeList_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CD_SignatureCode },
};

static int
dissect_rrc_CD_SignatureCodeList(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_CD_SignatureCodeList, CD_SignatureCodeList_sequence_of,
                                                  1, maxPCPCH_CDsig, FALSE);

  return offset;
}



static int
dissect_rrc_DeltaPp_m(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            -10, 10U, NULL, FALSE);

  return offset;
}


static const value_string rrc_UL_DPCCH_SlotFormat_vals[] = {
  {   0, "slf0" },
  {   1, "slf1" },
  {   2, "slf2" },
  { 0, NULL }
};


static int
dissect_rrc_UL_DPCCH_SlotFormat(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     3, NULL, FALSE, 0, NULL);

  return offset;
}



static int
dissect_rrc_N_StartMessage(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            1U, 8U, NULL, FALSE);

  return offset;
}



static int
dissect_rrc_N_EOT(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            0U, 7U, NULL, FALSE);

  return offset;
}


static const value_string rrc_MinimumSpreadingFactor_vals[] = {
  {   0, "sf4" },
  {   1, "sf8" },
  {   2, "sf16" },
  {   3, "sf32" },
  {   4, "sf64" },
  {   5, "sf128" },
  {   6, "sf256" },
  { 0, NULL }
};


static int
dissect_rrc_MinimumSpreadingFactor(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     7, NULL, FALSE, 0, NULL);

  return offset;
}



static int
dissect_rrc_NF_Max(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            1U, 64U, NULL, FALSE);

  return offset;
}



static int
dissect_rrc_MaxAvailablePCPCH_Number(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            1U, 64U, NULL, FALSE);

  return offset;
}



static int
dissect_rrc_AP_Signature(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            0U, 15U, NULL, FALSE);

  return offset;
}



static int
dissect_rrc_AP_Subchannel(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            0U, 11U, NULL, FALSE);

  return offset;
}


static const per_sequence_t AvailableAP_SubchannelList_sequence_of[1] = {
  { &hf_rrc_AvailableAP_SubchannelList_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_AP_Subchannel },
};

static int
dissect_rrc_AvailableAP_SubchannelList(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_AvailableAP_SubchannelList, AvailableAP_SubchannelList_sequence_of,
                                                  1, maxPCPCH_APsubCh, FALSE);

  return offset;
}


static const per_sequence_t AP_Signature_VCAM_sequence[] = {
  { &hf_rrc_ap_Signature    , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_AP_Signature },
  { &hf_rrc_availableAP_SubchannelList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_AvailableAP_SubchannelList },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_AP_Signature_VCAM(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_AP_Signature_VCAM, AP_Signature_VCAM_sequence);

  return offset;
}


static const per_sequence_t AvailableAP_Signature_VCAMList_sequence_of[1] = {
  { &hf_rrc_AvailableAP_Signature_VCAMList_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_AP_Signature_VCAM },
};

static int
dissect_rrc_AvailableAP_Signature_VCAMList(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_AvailableAP_Signature_VCAMList, AvailableAP_Signature_VCAMList_sequence_of,
                                                  1, maxPCPCH_APsig, FALSE);

  return offset;
}


static const per_sequence_t AvailableMinimumSF_VCAM_sequence[] = {
  { &hf_rrc_minimumSpreadingFactor, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MinimumSpreadingFactor },
  { &hf_rrc_nf_Max          , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_NF_Max },
  { &hf_rrc_maxAvailablePCPCH_Number, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MaxAvailablePCPCH_Number },
  { &hf_rrc_availableAP_Signature_VCAMList, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_AvailableAP_Signature_VCAMList },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_AvailableMinimumSF_VCAM(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_AvailableMinimumSF_VCAM, AvailableMinimumSF_VCAM_sequence);

  return offset;
}


static const per_sequence_t AvailableMinimumSF_ListVCAM_sequence_of[1] = {
  { &hf_rrc_AvailableMinimumSF_ListVCAM_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_AvailableMinimumSF_VCAM },
};

static int
dissect_rrc_AvailableMinimumSF_ListVCAM(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_AvailableMinimumSF_ListVCAM, AvailableMinimumSF_ListVCAM_sequence_of,
                                                  1, maxPCPCH_SF, FALSE);

  return offset;
}


static const value_string rrc_ChannelAssignmentActive_vals[] = {
  {   0, "notActive" },
  {   1, "isActive" },
  { 0, NULL }
};

static const per_choice_t ChannelAssignmentActive_choice[] = {
  {   0, &hf_rrc_notActive       , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   1, &hf_rrc_isActive        , ASN1_NO_EXTENSIONS     , dissect_rrc_AvailableMinimumSF_ListVCAM },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_ChannelAssignmentActive(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_ChannelAssignmentActive, ChannelAssignmentActive_choice,
                                 NULL);

  return offset;
}


static const value_string rrc_CPCH_StatusIndicationMode_vals[] = {
  {   0, "pa-mode" },
  {   1, "pamsf-mode" },
  { 0, NULL }
};


static int
dissect_rrc_CPCH_StatusIndicationMode(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     2, NULL, FALSE, 0, NULL);

  return offset;
}



static int
dissect_rrc_INTEGER_0_79(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            0U, 79U, NULL, FALSE);

  return offset;
}


static const value_string rrc_PCP_Length_vals[] = {
  {   0, "as0" },
  {   1, "as8" },
  { 0, NULL }
};


static int
dissect_rrc_PCP_Length(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     2, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t AvailableAP_SignatureList_sequence_of[1] = {
  { &hf_rrc_AvailableAP_SignatureList_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_AP_Signature },
};

static int
dissect_rrc_AvailableAP_SignatureList(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_AvailableAP_SignatureList, AvailableAP_SignatureList_sequence_of,
                                                  1, maxPCPCH_APsig, FALSE);

  return offset;
}


static const per_sequence_t ChannelReqParamsForUCSM_sequence[] = {
  { &hf_rrc_availableAP_SignatureList, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_AvailableAP_SignatureList },
  { &hf_rrc_availableAP_SubchannelList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_AvailableAP_SubchannelList },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_ChannelReqParamsForUCSM(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_ChannelReqParamsForUCSM, ChannelReqParamsForUCSM_sequence);

  return offset;
}


static const per_sequence_t UCSM_Info_sequence[] = {
  { &hf_rrc_minimumSpreadingFactor, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MinimumSpreadingFactor },
  { &hf_rrc_nf_Max          , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_NF_Max },
  { &hf_rrc_channelReqParamsForUCSM, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_ChannelReqParamsForUCSM },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UCSM_Info(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UCSM_Info, UCSM_Info_sequence);

  return offset;
}


static const per_sequence_t PCPCH_ChannelInfo_sequence[] = {
  { &hf_rrc_pcpch_UL_ScramblingCode, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_0_79 },
  { &hf_rrc_pcpch_DL_ChannelisationCode, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_0_511 },
  { &hf_rrc_pcpch_DL_ScramblingCode, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_SecondaryScramblingCode },
  { &hf_rrc_pcp_Length      , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PCP_Length },
  { &hf_rrc_ucsm_Info       , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UCSM_Info },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_PCPCH_ChannelInfo(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_PCPCH_ChannelInfo, PCPCH_ChannelInfo_sequence);

  return offset;
}


static const per_sequence_t PCPCH_ChannelInfoList_sequence_of[1] = {
  { &hf_rrc_PCPCH_ChannelInfoList_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PCPCH_ChannelInfo },
};

static int
dissect_rrc_PCPCH_ChannelInfoList(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_PCPCH_ChannelInfoList, PCPCH_ChannelInfoList_sequence_of,
                                                  1, maxPCPCHs, FALSE);

  return offset;
}


static const per_sequence_t CPCH_SetInfo_sequence[] = {
  { &hf_rrc_cpch_SetID      , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CPCH_SetID },
  { &hf_rrc_transportFormatSet, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TransportFormatSet },
  { &hf_rrc_tfcs            , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TFCS },
  { &hf_rrc_ap_PreambleScramblingCode, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_AP_PreambleScramblingCode },
  { &hf_rrc_ap_AICH_ChannelisationCode, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_AP_AICH_ChannelisationCode },
  { &hf_rrc_cd_PreambleScramblingCode, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CD_PreambleScramblingCode },
  { &hf_rrc_cd_CA_ICH_ChannelisationCode, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CD_CA_ICH_ChannelisationCode },
  { &hf_rrc_cd_AccessSlotSubchannelList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CD_AccessSlotSubchannelList },
  { &hf_rrc_cd_SignatureCodeList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CD_SignatureCodeList },
  { &hf_rrc_deltaPp_m       , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DeltaPp_m },
  { &hf_rrc_ul_DPCCH_SlotFormat, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UL_DPCCH_SlotFormat },
  { &hf_rrc_n_StartMessage  , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_N_StartMessage },
  { &hf_rrc_n_EOT           , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_N_EOT },
  { &hf_rrc_channelAssignmentActive, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_ChannelAssignmentActive },
  { &hf_rrc_cpch_StatusIndicationMode, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CPCH_StatusIndicationMode },
  { &hf_rrc_pcpch_ChannelInfoList, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PCPCH_ChannelInfoList },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_CPCH_SetInfo(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_CPCH_SetInfo, CPCH_SetInfo_sequence);

  return offset;
}


static const value_string rrc_UL_ChannelRequirement_vals[] = {
  {   0, "ul-DPCH-Info" },
  {   1, "dummy" },
  { 0, NULL }
};

static const per_choice_t UL_ChannelRequirement_choice[] = {
  {   0, &hf_rrc_ul_DPCH_Info_03 , ASN1_NO_EXTENSIONS     , dissect_rrc_UL_DPCH_Info },
  {   1, &hf_rrc_dummy_47        , ASN1_NO_EXTENSIONS     , dissect_rrc_CPCH_SetInfo },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_UL_ChannelRequirement(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_UL_ChannelRequirement, UL_ChannelRequirement_choice,
                                 NULL);

  return offset;
}



static int
dissect_rrc_DSCH_RadioLinkIdentifier(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            0U, 511U, NULL, FALSE);

  return offset;
}


static const per_sequence_t RL_IdentifierList_sequence_of[1] = {
  { &hf_rrc_RL_IdentifierList_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PrimaryCPICH_Info },
};

static int
dissect_rrc_RL_IdentifierList(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_RL_IdentifierList, RL_IdentifierList_sequence_of,
                                                  1, maxRL, FALSE);

  return offset;
}


static const per_sequence_t PDSCH_SHO_DCH_Info_sequence[] = {
  { &hf_rrc_dsch_RadioLinkIdentifier, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DSCH_RadioLinkIdentifier },
  { &hf_rrc_rl_IdentifierList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RL_IdentifierList },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_PDSCH_SHO_DCH_Info(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_PDSCH_SHO_DCH_Info, PDSCH_SHO_DCH_Info_sequence);

  return offset;
}


static const value_string rrc_SF_PDSCH_vals[] = {
  {   0, "sfp4" },
  {   1, "sfp8" },
  {   2, "sfp16" },
  {   3, "sfp32" },
  {   4, "sfp64" },
  {   5, "sfp128" },
  {   6, "sfp256" },
  { 0, NULL }
};


static int
dissect_rrc_SF_PDSCH(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     7, NULL, FALSE, 0, NULL);

  return offset;
}



static int
dissect_rrc_MultiCodeInfo(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            1U, 16U, NULL, FALSE);

  return offset;
}



static int
dissect_rrc_CodeNumberDSCH(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            0U, 255U, NULL, FALSE);

  return offset;
}


static const per_sequence_t PDSCH_CodeMap_sequence[] = {
  { &hf_rrc_spreadingFactor , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SF_PDSCH },
  { &hf_rrc_multiCodeInfo   , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MultiCodeInfo },
  { &hf_rrc_codeNumberStart , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CodeNumberDSCH },
  { &hf_rrc_codeNumberStop  , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CodeNumberDSCH },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_PDSCH_CodeMap(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_PDSCH_CodeMap, PDSCH_CodeMap_sequence);

  return offset;
}


static const per_sequence_t PDSCH_CodeMapList_sequence_of[1] = {
  { &hf_rrc_PDSCH_CodeMapList_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PDSCH_CodeMap },
};

static int
dissect_rrc_PDSCH_CodeMapList(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_PDSCH_CodeMapList, PDSCH_CodeMapList_sequence_of,
                                                  1, maxPDSCH_TFCIgroups, FALSE);

  return offset;
}


static const per_sequence_t CodeRange_sequence[] = {
  { &hf_rrc_pdsch_CodeMapList, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PDSCH_CodeMapList },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_CodeRange(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_CodeRange, CodeRange_sequence);

  return offset;
}



static int
dissect_rrc_MaxTFCI_Field2Value(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            1U, 1023U, NULL, FALSE);

  return offset;
}


static const per_sequence_t DSCH_Mapping_sequence[] = {
  { &hf_rrc_maxTFCI_Field2Value, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MaxTFCI_Field2Value },
  { &hf_rrc_spreadingFactor , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SF_PDSCH },
  { &hf_rrc_codeNumber      , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CodeNumberDSCH },
  { &hf_rrc_multiCodeInfo   , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MultiCodeInfo },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_DSCH_Mapping(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_DSCH_Mapping, DSCH_Mapping_sequence);

  return offset;
}


static const per_sequence_t DSCH_MappingList_sequence_of[1] = {
  { &hf_rrc_DSCH_MappingList_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DSCH_Mapping },
};

static int
dissect_rrc_DSCH_MappingList(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_DSCH_MappingList, DSCH_MappingList_sequence_of,
                                                  1, maxPDSCH_TFCIgroups, FALSE);

  return offset;
}


static const per_sequence_t PDSCH_CodeInfo_sequence[] = {
  { &hf_rrc_spreadingFactor , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SF_PDSCH },
  { &hf_rrc_codeNumber      , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CodeNumberDSCH },
  { &hf_rrc_multiCodeInfo   , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MultiCodeInfo },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_PDSCH_CodeInfo(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_PDSCH_CodeInfo, PDSCH_CodeInfo_sequence);

  return offset;
}


static const per_sequence_t PDSCH_CodeInfoList_sequence_of[1] = {
  { &hf_rrc_PDSCH_CodeInfoList_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PDSCH_CodeInfo },
};

static int
dissect_rrc_PDSCH_CodeInfoList(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_PDSCH_CodeInfoList, PDSCH_CodeInfoList_sequence_of,
                                                  1, maxTFCI_2_Combs, FALSE);

  return offset;
}


static const per_sequence_t ReplacedPDSCH_CodeInfo_sequence[] = {
  { &hf_rrc_tfci_Field2     , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MaxTFCI_Field2Value },
  { &hf_rrc_spreadingFactor , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SF_PDSCH },
  { &hf_rrc_codeNumber      , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CodeNumberDSCH },
  { &hf_rrc_multiCodeInfo   , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MultiCodeInfo },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_ReplacedPDSCH_CodeInfo(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_ReplacedPDSCH_CodeInfo, ReplacedPDSCH_CodeInfo_sequence);

  return offset;
}


static const per_sequence_t ReplacedPDSCH_CodeInfoList_sequence_of[1] = {
  { &hf_rrc_ReplacedPDSCH_CodeInfoList_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_ReplacedPDSCH_CodeInfo },
};

static int
dissect_rrc_ReplacedPDSCH_CodeInfoList(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_ReplacedPDSCH_CodeInfoList, ReplacedPDSCH_CodeInfoList_sequence_of,
                                                  1, maxTFCI_2_Combs, FALSE);

  return offset;
}


static const value_string rrc_T_signallingMethod_vals[] = {
  {   0, "codeRange" },
  {   1, "tfci-Range" },
  {   2, "explicit-config" },
  {   3, "replace" },
  { 0, NULL }
};

static const per_choice_t T_signallingMethod_choice[] = {
  {   0, &hf_rrc_codeRange       , ASN1_NO_EXTENSIONS     , dissect_rrc_CodeRange },
  {   1, &hf_rrc_tfci_Range_01   , ASN1_NO_EXTENSIONS     , dissect_rrc_DSCH_MappingList },
  {   2, &hf_rrc_explicit_config_03, ASN1_NO_EXTENSIONS     , dissect_rrc_PDSCH_CodeInfoList },
  {   3, &hf_rrc_replace         , ASN1_NO_EXTENSIONS     , dissect_rrc_ReplacedPDSCH_CodeInfoList },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_signallingMethod(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_signallingMethod, T_signallingMethod_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t PDSCH_CodeMapping_sequence[] = {
  { &hf_rrc_dl_ScramblingCode, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_SecondaryScramblingCode },
  { &hf_rrc_signallingMethod, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_signallingMethod },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_PDSCH_CodeMapping(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_PDSCH_CodeMapping, PDSCH_CodeMapping_sequence);

  return offset;
}


static const per_sequence_t DL_PDSCH_Information_sequence[] = {
  { &hf_rrc_dummy1_02       , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_PDSCH_SHO_DCH_Info },
  { &hf_rrc_dummy2_07       , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_PDSCH_CodeMapping },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_DL_PDSCH_Information(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_DL_PDSCH_Information, DL_PDSCH_Information_sequence);

  return offset;
}


static const per_sequence_t T_fdd_01_sequence[] = {
  { &hf_rrc_dummy_06        , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_PDSCH_Information },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_fdd_01(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_fdd_01, T_fdd_01_sequence);

  return offset;
}


static const value_string rrc_T_modeSpecificPhysChInfo_vals[] = {
  {   0, "fdd" },
  {   1, "tdd" },
  { 0, NULL }
};

static const per_choice_t T_modeSpecificPhysChInfo_choice[] = {
  {   0, &hf_rrc_fdd_01          , ASN1_NO_EXTENSIONS     , dissect_rrc_T_fdd_01 },
  {   1, &hf_rrc_tdd             , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_modeSpecificPhysChInfo(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_modeSpecificPhysChInfo, T_modeSpecificPhysChInfo_choice,
                                 NULL);

  return offset;
}



static int
dissect_rrc_Cfntargetsfnframeoffset(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            0U, 255U, NULL, FALSE);

  return offset;
}


static const per_sequence_t T_initialise_sequence[] = {
  { &hf_rrc_dummy_40        , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_Cfntargetsfnframeoffset },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_initialise(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_initialise, T_initialise_sequence);

  return offset;
}


static const value_string rrc_T_cfnHandling_vals[] = {
  {   0, "maintain" },
  {   1, "initialise" },
  { 0, NULL }
};

static const per_choice_t T_cfnHandling_choice[] = {
  {   0, &hf_rrc_maintain        , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   1, &hf_rrc_initialise      , ASN1_NO_EXTENSIONS     , dissect_rrc_T_initialise },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_cfnHandling(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_cfnHandling, T_cfnHandling_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t T_fdd_75_sequence[] = {
  { &hf_rrc_dpc_Mode        , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DPC_Mode },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_fdd_75(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_fdd_75, T_fdd_75_sequence);

  return offset;
}



static int
dissect_rrc_TPC_StepSizeTDD(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            1U, 3U, NULL, FALSE);

  return offset;
}


static const per_sequence_t T_tdd_37_sequence[] = {
  { &hf_rrc_tpc_StepSizeTDD , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_TPC_StepSizeTDD },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_tdd_37(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_tdd_37, T_tdd_37_sequence);

  return offset;
}


static const value_string rrc_T_modeSpecificInfo_40_vals[] = {
  {   0, "fdd" },
  {   1, "tdd" },
  { 0, NULL }
};

static const per_choice_t T_modeSpecificInfo_40_choice[] = {
  {   0, &hf_rrc_fdd_76          , ASN1_NO_EXTENSIONS     , dissect_rrc_T_fdd_75 },
  {   1, &hf_rrc_tdd_38          , ASN1_NO_EXTENSIONS     , dissect_rrc_T_tdd_37 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_modeSpecificInfo_40(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_modeSpecificInfo_40, T_modeSpecificInfo_40_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t DL_DPCH_PowerControlInfo_sequence[] = {
  { &hf_rrc_modeSpecificInfo_40, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_modeSpecificInfo_40 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_DL_DPCH_PowerControlInfo(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_DL_DPCH_PowerControlInfo, DL_DPCH_PowerControlInfo_sequence);

  return offset;
}



static int
dissect_rrc_PowerOffsetPilot_pdpdch(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            0U, 24U, NULL, FALSE);

  return offset;
}


static const per_sequence_t RestrictedTrCH_sequence[] = {
  { &hf_rrc_dl_restrictedTrCh_Type, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DL_TrCH_Type },
  { &hf_rrc_restrictedDL_TrCH_Identity, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TransportChannelIdentity },
  { &hf_rrc_allowedTFIList  , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_AllowedTFI_List },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_RestrictedTrCH(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_RestrictedTrCH, RestrictedTrCH_sequence);

  return offset;
}


static const per_sequence_t RestrictedTrCH_InfoList_sequence_of[1] = {
  { &hf_rrc_RestrictedTrCH_InfoList_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RestrictedTrCH },
};

static int
dissect_rrc_RestrictedTrCH_InfoList(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_RestrictedTrCH_InfoList, RestrictedTrCH_InfoList_sequence_of,
                                                  1, maxTrCH, FALSE);

  return offset;
}


static const per_sequence_t Dl_rate_matching_restriction_sequence[] = {
  { &hf_rrc_restrictedTrCH_InfoList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RestrictedTrCH_InfoList },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_Dl_rate_matching_restriction(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_Dl_rate_matching_restriction, Dl_rate_matching_restriction_sequence);

  return offset;
}


static const value_string rrc_PilotBits128_vals[] = {
  {   0, "pb4" },
  {   1, "pb8" },
  { 0, NULL }
};


static int
dissect_rrc_PilotBits128(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     2, NULL, FALSE, 0, NULL);

  return offset;
}


static const value_string rrc_PilotBits256_vals[] = {
  {   0, "pb2" },
  {   1, "pb4" },
  {   2, "pb8" },
  { 0, NULL }
};


static int
dissect_rrc_PilotBits256(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     3, NULL, FALSE, 0, NULL);

  return offset;
}


static const value_string rrc_SF512_AndPilot_vals[] = {
  {   0, "sfd4" },
  {   1, "sfd8" },
  {   2, "sfd16" },
  {   3, "sfd32" },
  {   4, "sfd64" },
  {   5, "sfd128" },
  {   6, "sfd256" },
  {   7, "sfd512" },
  { 0, NULL }
};

static const per_choice_t SF512_AndPilot_choice[] = {
  {   0, &hf_rrc_sfd4            , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   1, &hf_rrc_sfd8            , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   2, &hf_rrc_sfd16           , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   3, &hf_rrc_sfd32           , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   4, &hf_rrc_sfd64           , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   5, &hf_rrc_sfd128          , ASN1_NO_EXTENSIONS     , dissect_rrc_PilotBits128 },
  {   6, &hf_rrc_sfd256          , ASN1_NO_EXTENSIONS     , dissect_rrc_PilotBits256 },
  {   7, &hf_rrc_sfd512          , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_SF512_AndPilot(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_SF512_AndPilot, SF512_AndPilot_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t T_fdd_66_sequence[] = {
  { &hf_rrc_dl_DPCH_PowerControlInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_DPCH_PowerControlInfo },
  { &hf_rrc_powerOffsetPilot_pdpdch, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PowerOffsetPilot_pdpdch },
  { &hf_rrc_dl_rate_matching_restriction, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_Dl_rate_matching_restriction },
  { &hf_rrc_spreadingFactorAndPilot, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SF512_AndPilot },
  { &hf_rrc_positionFixedOrFlexible, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PositionFixedOrFlexible },
  { &hf_rrc_tfci_Existence  , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_fdd_66(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_fdd_66, T_fdd_66_sequence);

  return offset;
}


static const per_sequence_t T_tdd_28_sequence[] = {
  { &hf_rrc_dl_DPCH_PowerControlInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_DPCH_PowerControlInfo },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_tdd_28(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_tdd_28, T_tdd_28_sequence);

  return offset;
}


static const value_string rrc_T_modeSpecificInfo_36_vals[] = {
  {   0, "fdd" },
  {   1, "tdd" },
  { 0, NULL }
};

static const per_choice_t T_modeSpecificInfo_36_choice[] = {
  {   0, &hf_rrc_fdd_67          , ASN1_NO_EXTENSIONS     , dissect_rrc_T_fdd_66 },
  {   1, &hf_rrc_tdd_29          , ASN1_NO_EXTENSIONS     , dissect_rrc_T_tdd_28 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_modeSpecificInfo_36(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_modeSpecificInfo_36, T_modeSpecificInfo_36_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t DL_DPCH_InfoCommon_sequence[] = {
  { &hf_rrc_cfnHandling     , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_cfnHandling },
  { &hf_rrc_modeSpecificInfo_36, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_modeSpecificInfo_36 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_DL_DPCH_InfoCommon(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_DL_DPCH_InfoCommon, DL_DPCH_InfoCommon_sequence);

  return offset;
}



static int
dissect_rrc_DefaultDPCH_OffsetValueFDD(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            0U, 599U, NULL, FALSE);

  return offset;
}



static int
dissect_rrc_TGPSI(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            1U, maxTGPS, NULL, FALSE);

  return offset;
}



static int
dissect_rrc_TGCFN(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            0U, 255U, NULL, FALSE);

  return offset;
}


static const per_sequence_t T_activate_sequence[] = {
  { &hf_rrc_tgcfn           , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TGCFN },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_activate(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_activate, T_activate_sequence);

  return offset;
}


static const value_string rrc_T_tgps_Status_vals[] = {
  {   0, "activate" },
  {   1, "deactivate" },
  { 0, NULL }
};

static const per_choice_t T_tgps_Status_choice[] = {
  {   0, &hf_rrc_activate        , ASN1_NO_EXTENSIONS     , dissect_rrc_T_activate },
  {   1, &hf_rrc_deactivate      , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_tgps_Status(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_tgps_Status, T_tgps_Status_choice,
                                 NULL);

  return offset;
}


static const value_string rrc_TGMP_vals[] = {
  {   0, "tdd-Measurement" },
  {   1, "fdd-Measurement" },
  {   2, "gsm-CarrierRSSIMeasurement" },
  {   3, "gsm-initialBSICIdentification" },
  {   4, "gsmBSICReconfirmation" },
  {   5, "multi-carrier" },
  { 0, NULL }
};


static int
dissect_rrc_TGMP(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     6, NULL, FALSE, 0, NULL);

  return offset;
}



static int
dissect_rrc_TGPRC(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            0U, 511U, NULL, FALSE);

  return offset;
}



static int
dissect_rrc_TGSN(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            0U, 14U, NULL, FALSE);

  return offset;
}



static int
dissect_rrc_TGL(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            1U, 14U, NULL, FALSE);

  return offset;
}



static int
dissect_rrc_TGD(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            15U, 270U, NULL, FALSE);

  return offset;
}



static int
dissect_rrc_TGPL(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            1U, 144U, NULL, FALSE);

  return offset;
}


static const value_string rrc_RPP_vals[] = {
  {   0, "mode0" },
  {   1, "mode1" },
  { 0, NULL }
};


static int
dissect_rrc_RPP(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     2, NULL, FALSE, 0, NULL);

  return offset;
}


static const value_string rrc_ITP_vals[] = {
  {   0, "mode0" },
  {   1, "mode1" },
  { 0, NULL }
};


static int
dissect_rrc_ITP(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     2, NULL, FALSE, 0, NULL);

  return offset;
}


static const value_string rrc_UL_CompressedModeMethod_vals[] = {
  {   0, "sf-2" },
  {   1, "higherLayerScheduling" },
  { 0, NULL }
};


static int
dissect_rrc_UL_CompressedModeMethod(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     2, NULL, FALSE, 0, NULL);

  return offset;
}


static const value_string rrc_DL_CompressedModeMethod_vals[] = {
  {   0, "dummy" },
  {   1, "sf-2" },
  {   2, "higherLayerScheduling" },
  { 0, NULL }
};


static int
dissect_rrc_DL_CompressedModeMethod(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     3, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t T_ul_and_dl_sequence[] = {
  { &hf_rrc_ul              , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UL_CompressedModeMethod },
  { &hf_rrc_dl              , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DL_CompressedModeMethod },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_ul_and_dl(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_ul_and_dl, T_ul_and_dl_sequence);

  return offset;
}


static const value_string rrc_UL_DL_Mode_vals[] = {
  {   0, "ul" },
  {   1, "dl" },
  {   2, "ul-and-dl" },
  { 0, NULL }
};

static const per_choice_t UL_DL_Mode_choice[] = {
  {   0, &hf_rrc_ul              , ASN1_NO_EXTENSIONS     , dissect_rrc_UL_CompressedModeMethod },
  {   1, &hf_rrc_dl              , ASN1_NO_EXTENSIONS     , dissect_rrc_DL_CompressedModeMethod },
  {   2, &hf_rrc_ul_and_dl       , ASN1_NO_EXTENSIONS     , dissect_rrc_T_ul_and_dl },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_UL_DL_Mode(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_UL_DL_Mode, UL_DL_Mode_choice,
                                 NULL);

  return offset;
}


static const value_string rrc_DL_FrameType_vals[] = {
  {   0, "dl-FrameTypeA" },
  {   1, "dl-FrameTypeB" },
  { 0, NULL }
};


static int
dissect_rrc_DL_FrameType(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     2, NULL, FALSE, 0, NULL);

  return offset;
}



static int
dissect_rrc_DeltaSIR(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            0U, 30U, NULL, FALSE);

  return offset;
}



static int
dissect_rrc_NidentifyAbort(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            1U, 128U, NULL, FALSE);

  return offset;
}



static int
dissect_rrc_TreconfirmAbort(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            1U, 20U, NULL, FALSE);

  return offset;
}


static const per_sequence_t TGPS_ConfigurationParams_sequence[] = {
  { &hf_rrc_tgmp            , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TGMP },
  { &hf_rrc_tgprc           , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TGPRC },
  { &hf_rrc_tgsn            , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TGSN },
  { &hf_rrc_tgl1            , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TGL },
  { &hf_rrc_tgl2            , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_TGL },
  { &hf_rrc_tgd             , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TGD },
  { &hf_rrc_tgpl1           , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TGPL },
  { &hf_rrc_dummy_46        , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_TGPL },
  { &hf_rrc_rpp             , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RPP },
  { &hf_rrc_itp             , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_ITP },
  { &hf_rrc_ul_DL_Mode      , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UL_DL_Mode },
  { &hf_rrc_dl_FrameType    , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DL_FrameType },
  { &hf_rrc_deltaSIR1       , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DeltaSIR },
  { &hf_rrc_deltaSIRAfter1  , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DeltaSIR },
  { &hf_rrc_deltaSIR2       , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DeltaSIR },
  { &hf_rrc_deltaSIRAfter2  , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DeltaSIR },
  { &hf_rrc_nidentifyAbort  , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_NidentifyAbort },
  { &hf_rrc_treconfirmAbort , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_TreconfirmAbort },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_TGPS_ConfigurationParams(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_TGPS_ConfigurationParams, TGPS_ConfigurationParams_sequence);

  return offset;
}


static const per_sequence_t TGP_Sequence_sequence[] = {
  { &hf_rrc_tgpsi           , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TGPSI },
  { &hf_rrc_tgps_Status     , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_tgps_Status },
  { &hf_rrc_tgps_ConfigurationParams, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_TGPS_ConfigurationParams },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_TGP_Sequence(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_TGP_Sequence, TGP_Sequence_sequence);

  return offset;
}


static const per_sequence_t TGP_SequenceList_sequence_of[1] = {
  { &hf_rrc_TGP_SequenceList_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TGP_Sequence },
};

static int
dissect_rrc_TGP_SequenceList(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_TGP_SequenceList, TGP_SequenceList_sequence_of,
                                                  1, maxTGPS, FALSE);

  return offset;
}


static const per_sequence_t DPCH_CompressedModeInfo_sequence[] = {
  { &hf_rrc_tgp_SequenceList, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TGP_SequenceList },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_DPCH_CompressedModeInfo(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_DPCH_CompressedModeInfo, DPCH_CompressedModeInfo_sequence);

  return offset;
}


static const per_sequence_t T_fdd_58_sequence[] = {
  { &hf_rrc_defaultDPCH_OffsetValue, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DefaultDPCH_OffsetValueFDD },
  { &hf_rrc_dpch_CompressedModeInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DPCH_CompressedModeInfo },
  { &hf_rrc_tx_DiversityMode, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_TX_DiversityMode },
  { &hf_rrc_dummy_38        , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_SSDT_Information },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_fdd_58(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_fdd_58, T_fdd_58_sequence);

  return offset;
}



static int
dissect_rrc_DefaultDPCH_OffsetValueTDD(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            0U, 7U, NULL, FALSE);

  return offset;
}


static const per_sequence_t T_tdd_20_sequence[] = {
  { &hf_rrc_defaultDPCH_OffsetValue_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DefaultDPCH_OffsetValueTDD },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_tdd_20(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_tdd_20, T_tdd_20_sequence);

  return offset;
}


static const value_string rrc_T_modeSpecificInfo_28_vals[] = {
  {   0, "fdd" },
  {   1, "tdd" },
  { 0, NULL }
};

static const per_choice_t T_modeSpecificInfo_28_choice[] = {
  {   0, &hf_rrc_fdd_59          , ASN1_NO_EXTENSIONS     , dissect_rrc_T_fdd_58 },
  {   1, &hf_rrc_tdd_21          , ASN1_NO_EXTENSIONS     , dissect_rrc_T_tdd_20 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_modeSpecificInfo_28(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_modeSpecificInfo_28, T_modeSpecificInfo_28_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t DL_CommonInformation_sequence[] = {
  { &hf_rrc_dl_DPCH_InfoCommon, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_DPCH_InfoCommon },
  { &hf_rrc_modeSpecificInfo_28, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_modeSpecificInfo_28 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_DL_CommonInformation(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_DL_CommonInformation, DL_CommonInformation_sequence);

  return offset;
}


static const per_sequence_t T_fdd_80_sequence[] = {
  { &hf_rrc_primaryCPICH_Info, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PrimaryCPICH_Info },
  { &hf_rrc_dummy1_02       , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_PDSCH_SHO_DCH_Info },
  { &hf_rrc_dummy2_07       , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_PDSCH_CodeMapping },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_fdd_80(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_fdd_80, T_fdd_80_sequence);

  return offset;
}


static const per_sequence_t T_fdd_105_sequence[] = {
  { &hf_rrc_tx_DiversityIndicator, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_fdd_105(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_fdd_105, T_fdd_105_sequence);

  return offset;
}


static const per_sequence_t T_syncCase1_sequence[] = {
  { &hf_rrc_timeslot        , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TimeslotNumber },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_syncCase1(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_syncCase1, T_syncCase1_sequence);

  return offset;
}



static int
dissect_rrc_TimeslotSync2(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            0U, 6U, NULL, FALSE);

  return offset;
}


static const per_sequence_t T_syncCase2_sequence[] = {
  { &hf_rrc_timeslotSync2   , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TimeslotSync2 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_syncCase2(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_syncCase2, T_syncCase2_sequence);

  return offset;
}


static const value_string rrc_T_syncCase_vals[] = {
  {   0, "syncCase1" },
  {   1, "syncCase2" },
  { 0, NULL }
};

static const per_choice_t T_syncCase_choice[] = {
  {   0, &hf_rrc_syncCase1       , ASN1_NO_EXTENSIONS     , dissect_rrc_T_syncCase1 },
  {   1, &hf_rrc_syncCase2       , ASN1_NO_EXTENSIONS     , dissect_rrc_T_syncCase2 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_syncCase(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_syncCase, T_syncCase_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t T_tdd_59_sequence[] = {
  { &hf_rrc_syncCase        , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_syncCase },
  { &hf_rrc_cellParametersID, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CellParametersID },
  { &hf_rrc_sctd_Indicator  , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_tdd_59(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_tdd_59, T_tdd_59_sequence);

  return offset;
}


static const value_string rrc_PrimaryCCPCH_Info_vals[] = {
  {   0, "fdd" },
  {   1, "tdd" },
  { 0, NULL }
};

static const per_choice_t PrimaryCCPCH_Info_choice[] = {
  {   0, &hf_rrc_fdd_108         , ASN1_NO_EXTENSIONS     , dissect_rrc_T_fdd_105 },
  {   1, &hf_rrc_tdd_65          , ASN1_NO_EXTENSIONS     , dissect_rrc_T_tdd_59 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_PrimaryCCPCH_Info(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_PrimaryCCPCH_Info, PrimaryCCPCH_Info_choice,
                                 NULL);

  return offset;
}


static const value_string rrc_T_modeSpecificInfo_48_vals[] = {
  {   0, "fdd" },
  {   1, "tdd" },
  { 0, NULL }
};

static const per_choice_t T_modeSpecificInfo_48_choice[] = {
  {   0, &hf_rrc_fdd_81          , ASN1_NO_EXTENSIONS     , dissect_rrc_T_fdd_80 },
  {   1, &hf_rrc_tdd_46          , ASN1_NO_EXTENSIONS     , dissect_rrc_PrimaryCCPCH_Info },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_modeSpecificInfo_48(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_modeSpecificInfo_48, T_modeSpecificInfo_48_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t DL_InformationPerRL_sequence[] = {
  { &hf_rrc_modeSpecificInfo_48, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_modeSpecificInfo_48 },
  { &hf_rrc_dl_DPCH_InfoPerRL, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_DPCH_InfoPerRL },
  { &hf_rrc_dummy_42        , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_SCCPCH_InfoForFACH },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_DL_InformationPerRL(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_DL_InformationPerRL, DL_InformationPerRL_sequence);

  return offset;
}


static const per_sequence_t DL_InformationPerRL_List_sequence_of[1] = {
  { &hf_rrc_DL_InformationPerRL_List_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DL_InformationPerRL },
};

static int
dissect_rrc_DL_InformationPerRL_List(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_DL_InformationPerRL_List, DL_InformationPerRL_List_sequence_of,
                                                  1, maxRL, FALSE);

  return offset;
}


static const per_sequence_t CellUpdateConfirm_r3_IEs_sequence[] = {
  { &hf_rrc_rrc_TransactionIdentifier, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RRC_TransactionIdentifier },
  { &hf_rrc_integrityProtectionModeInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_IntegrityProtectionModeInfo },
  { &hf_rrc_cipheringModeInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CipheringModeInfo },
  { &hf_rrc_activationTime  , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_ActivationTime },
  { &hf_rrc_new_U_RNTI      , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_U_RNTI },
  { &hf_rrc_new_C_RNTI      , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_C_RNTI },
  { &hf_rrc_rrc_StateIndicator, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RRC_StateIndicator },
  { &hf_rrc_utran_DRX_CycleLengthCoeff, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UTRAN_DRX_CycleLengthCoefficient },
  { &hf_rrc_rlc_Re_establishIndicatorRb2_3or4, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { &hf_rrc_rlc_Re_establishIndicatorRb5orAbove, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { &hf_rrc_cn_InformationInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CN_InformationInfo },
  { &hf_rrc_ura_Identity    , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_URA_Identity },
  { &hf_rrc_rb_InformationReleaseList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RB_InformationReleaseList },
  { &hf_rrc_rb_InformationReconfigList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RB_InformationReconfigList },
  { &hf_rrc_rb_InformationAffectedList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RB_InformationAffectedList },
  { &hf_rrc_dl_CounterSynchronisationInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_CounterSynchronisationInfo },
  { &hf_rrc_ul_CommonTransChInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_CommonTransChInfo },
  { &hf_rrc_ul_deletedTransChInfoList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_DeletedTransChInfoList },
  { &hf_rrc_ul_AddReconfTransChInfoList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_AddReconfTransChInfoList },
  { &hf_rrc_modeSpecificTransChInfo, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_modeSpecificTransChInfo },
  { &hf_rrc_dl_CommonTransChInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_CommonTransChInfo },
  { &hf_rrc_dl_DeletedTransChInfoList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_DeletedTransChInfoList },
  { &hf_rrc_dl_AddReconfTransChInfoList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_AddReconfTransChInfoList },
  { &hf_rrc_frequencyInfo   , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_FrequencyInfo },
  { &hf_rrc_maxAllowedUL_TX_Power, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MaxAllowedUL_TX_Power },
  { &hf_rrc_ul_ChannelRequirement, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_ChannelRequirement },
  { &hf_rrc_modeSpecificPhysChInfo, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_modeSpecificPhysChInfo },
  { &hf_rrc_dl_CommonInformation, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_CommonInformation },
  { &hf_rrc_dl_InformationPerRL_List, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_InformationPerRL_List },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_CellUpdateConfirm_r3_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_CellUpdateConfirm_r3_IEs, CellUpdateConfirm_r3_IEs_sequence);

  return offset;
}



static int
dissect_rrc_DSCH_RNTI(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_bit_string(tvb, offset, actx, tree, hf_index,
                                     16, 16, FALSE, NULL);

  return offset;
}


static const per_sequence_t CellUpdateConfirm_v3a0ext_sequence[] = {
  { &hf_rrc_new_DSCH_RNTI   , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DSCH_RNTI },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_CellUpdateConfirm_v3a0ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_CellUpdateConfirm_v3a0ext, CellUpdateConfirm_v3a0ext_sequence);

  return offset;
}


static const per_sequence_t CellUpdateConfirm_v4b0ext_IEs_sequence[] = {
  { &hf_rrc_dummy_02        , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_SSDT_UL },
  { &hf_rrc_cell_id_PerRL_List, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CellIdentity_PerRL_List },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_CellUpdateConfirm_v4b0ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_CellUpdateConfirm_v4b0ext_IEs, CellUpdateConfirm_v4b0ext_IEs_sequence);

  return offset;
}


static const per_sequence_t CellUpdateConfirm_v590ext_IEs_sequence[] = {
  { &hf_rrc_dl_TPC_PowerOffsetPerRL_List, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_TPC_PowerOffsetPerRL_List },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_CellUpdateConfirm_v590ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_CellUpdateConfirm_v590ext_IEs, CellUpdateConfirm_v590ext_IEs_sequence);

  return offset;
}


static const value_string rrc_PDCP_ROHC_TargetMode_vals[] = {
  {   0, "o-Mode" },
  {   1, "r-Mode" },
  { 0, NULL }
};


static int
dissect_rrc_PDCP_ROHC_TargetMode(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     2, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t CellUpdateConfirm_v5d0ext_IEs_sequence[] = {
  { &hf_rrc_pdcp_ROHC_TargetMode, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_PDCP_ROHC_TargetMode },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_CellUpdateConfirm_v5d0ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_CellUpdateConfirm_v5d0ext_IEs, CellUpdateConfirm_v5d0ext_IEs_sequence);

  return offset;
}


static const value_string rrc_BEACON_PL_Est_vals[] = {
  {   0, "true" },
  { 0, NULL }
};


static int
dissect_rrc_BEACON_PL_Est(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}


static const value_string rrc_T_postVerificationPeriod_vals[] = {
  {   0, "true" },
  { 0, NULL }
};


static int
dissect_rrc_T_postVerificationPeriod(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}


static const value_string rrc_MBMS_PL_ServiceRestrictInfo_r6_vals[] = {
  {   0, "true" },
  { 0, NULL }
};


static int
dissect_rrc_MBMS_PL_ServiceRestrictInfo_r6(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t CellUpdateConfirm_v690ext_IEs_sequence[] = {
  { &hf_rrc_primary_plmn_Identity, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_PLMN_Identity },
  { &hf_rrc_harq_Preamble_Mode, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_HARQ_Preamble_Mode },
  { &hf_rrc_beaconPLEst     , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_BEACON_PL_Est },
  { &hf_rrc_postVerificationPeriod, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_postVerificationPeriod },
  { &hf_rrc_dhs_sync        , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DHS_Sync },
  { &hf_rrc_mbms_PL_ServiceRestrictInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MBMS_PL_ServiceRestrictInfo_r6 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_CellUpdateConfirm_v690ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_CellUpdateConfirm_v690ext_IEs, CellUpdateConfirm_v690ext_IEs_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions_14_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_nonCriticalExtensions_14(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_nonCriticalExtensions_14, T_nonCriticalExtensions_14_sequence);

  return offset;
}


static const per_sequence_t T_v690NonCriticalExtensions_02_sequence[] = {
  { &hf_rrc_cellUpdateConfirm_v690ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CellUpdateConfirm_v690ext_IEs },
  { &hf_rrc_nonCriticalExtensions_14, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_nonCriticalExtensions_14 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v690NonCriticalExtensions_02(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v690NonCriticalExtensions_02, T_v690NonCriticalExtensions_02_sequence);

  return offset;
}


static const per_sequence_t T_v5d0NonCriticalExtenstions_sequence[] = {
  { &hf_rrc_cellUpdateConfirm_v5d0ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CellUpdateConfirm_v5d0ext_IEs },
  { &hf_rrc_v690NonCriticalExtensions_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v690NonCriticalExtensions_02 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v5d0NonCriticalExtenstions(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v5d0NonCriticalExtenstions, T_v5d0NonCriticalExtenstions_sequence);

  return offset;
}


static const per_sequence_t T_v590NonCriticalExtenstions_sequence[] = {
  { &hf_rrc_cellUpdateConfirm_v590ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CellUpdateConfirm_v590ext_IEs },
  { &hf_rrc_v5d0NonCriticalExtenstions, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v5d0NonCriticalExtenstions },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v590NonCriticalExtenstions(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v590NonCriticalExtenstions, T_v590NonCriticalExtenstions_sequence);

  return offset;
}


static const per_sequence_t T_v4b0NonCriticalExtensions_02_sequence[] = {
  { &hf_rrc_cellUpdateConfirm_v4b0ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CellUpdateConfirm_v4b0ext_IEs },
  { &hf_rrc_v590NonCriticalExtenstions, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v590NonCriticalExtenstions },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v4b0NonCriticalExtensions_02(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v4b0NonCriticalExtensions_02, T_v4b0NonCriticalExtensions_02_sequence);

  return offset;
}


static const per_sequence_t T_laterNonCriticalExtensions_07_sequence[] = {
  { &hf_rrc_cellUpdateConfirm_r3_add_ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_BIT_STRING },
  { &hf_rrc_v4b0NonCriticalExtensions_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v4b0NonCriticalExtensions_02 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_laterNonCriticalExtensions_07(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_laterNonCriticalExtensions_07, T_laterNonCriticalExtensions_07_sequence);

  return offset;
}


static const per_sequence_t T_v3a0NonCriticalExtensions_01_sequence[] = {
  { &hf_rrc_cellUpdateConfirm_v3a0ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CellUpdateConfirm_v3a0ext },
  { &hf_rrc_laterNonCriticalExtensions_07, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_laterNonCriticalExtensions_07 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v3a0NonCriticalExtensions_01(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v3a0NonCriticalExtensions_01, T_v3a0NonCriticalExtensions_01_sequence);

  return offset;
}


static const per_sequence_t T_r3_04_sequence[] = {
  { &hf_rrc_cellUpdateConfirm_r3, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CellUpdateConfirm_r3_IEs },
  { &hf_rrc_v3a0NonCriticalExtensions_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v3a0NonCriticalExtensions_01 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_r3_04(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_r3_04, T_r3_04_sequence);

  return offset;
}



static int
dissect_rrc_ROHC_Profile_r4(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            1U, 3U, NULL, FALSE);

  return offset;
}


static const per_sequence_t ROHC_ProfileList_r4_sequence_of[1] = {
  { &hf_rrc_ROHC_ProfileList_r4_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_ROHC_Profile_r4 },
};

static int
dissect_rrc_ROHC_ProfileList_r4(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_ROHC_ProfileList_r4, ROHC_ProfileList_r4_sequence_of,
                                                  1, maxROHC_Profile_r4, FALSE);

  return offset;
}


static const value_string rrc_CID_InclusionInfo_r4_vals[] = {
  {   0, "pdcp-Header" },
  {   1, "rfc3095-PacketFormat" },
  { 0, NULL }
};


static int
dissect_rrc_CID_InclusionInfo_r4(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     2, NULL, FALSE, 0, NULL);

  return offset;
}



static int
dissect_rrc_INTEGER_1_16383(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            1U, 16383U, NULL, FALSE);

  return offset;
}



static int
dissect_rrc_ROHC_PacketSize_r4(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            2U, 1500U, NULL, FALSE);

  return offset;
}


static const per_sequence_t ROHC_PacketSizeList_r4_sequence_of[1] = {
  { &hf_rrc_ROHC_PacketSizeList_r4_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_ROHC_PacketSize_r4 },
};

static int
dissect_rrc_ROHC_PacketSizeList_r4(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_ROHC_PacketSizeList_r4, ROHC_PacketSizeList_r4_sequence_of,
                                                  1, maxROHC_PacketSizes_r4, FALSE);

  return offset;
}


static const per_sequence_t UL_RFC3095_r4_sequence[] = {
  { &hf_rrc_dummy1_01       , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CID_InclusionInfo_r4 },
  { &hf_rrc_max_CID         , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_INTEGER_1_16383 },
  { &hf_rrc_dummy_33        , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_ROHC_PacketSizeList_r4 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UL_RFC3095_r4(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UL_RFC3095_r4, UL_RFC3095_r4_sequence);

  return offset;
}


static const per_sequence_t DL_RFC3095_r4_sequence[] = {
  { &hf_rrc_dummy_30        , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CID_InclusionInfo_r4 },
  { &hf_rrc_max_CID         , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_INTEGER_1_16383 },
  { &hf_rrc_reverseDecompressionDepth, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_INTEGER_0_65535 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_DL_RFC3095_r4(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_DL_RFC3095_r4, DL_RFC3095_r4_sequence);

  return offset;
}


static const per_sequence_t RFC3095_Info_r4_sequence[] = {
  { &hf_rrc_rohcProfileList , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_ROHC_ProfileList_r4 },
  { &hf_rrc_ul_RFC3095      , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_RFC3095_r4 },
  { &hf_rrc_dl_RFC3095      , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_RFC3095_r4 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_RFC3095_Info_r4(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_RFC3095_Info_r4, RFC3095_Info_r4_sequence);

  return offset;
}


static const value_string rrc_AlgorithmSpecificInfo_r4_vals[] = {
  {   0, "rfc2507-Info" },
  {   1, "rfc3095-Info" },
  { 0, NULL }
};

static const per_choice_t AlgorithmSpecificInfo_r4_choice[] = {
  {   0, &hf_rrc_rfc2507_Info    , ASN1_NO_EXTENSIONS     , dissect_rrc_RFC2507_Info },
  {   1, &hf_rrc_rfc3095_Info    , ASN1_NO_EXTENSIONS     , dissect_rrc_RFC3095_Info_r4 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_AlgorithmSpecificInfo_r4(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_AlgorithmSpecificInfo_r4, AlgorithmSpecificInfo_r4_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t HeaderCompressionInfo_r4_sequence[] = {
  { &hf_rrc_algorithmSpecificInfo_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_AlgorithmSpecificInfo_r4 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_HeaderCompressionInfo_r4(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_HeaderCompressionInfo_r4, HeaderCompressionInfo_r4_sequence);

  return offset;
}


static const per_sequence_t HeaderCompressionInfoList_r4_sequence_of[1] = {
  { &hf_rrc_HeaderCompressionInfoList_r4_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_HeaderCompressionInfo_r4 },
};

static int
dissect_rrc_HeaderCompressionInfoList_r4(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_HeaderCompressionInfoList_r4, HeaderCompressionInfoList_r4_sequence_of,
                                                  1, maxPDCPAlgoType, FALSE);

  return offset;
}


static const per_sequence_t PDCP_Info_r4_sequence[] = {
  { &hf_rrc_losslessSRNS_RelocSupport, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_LosslessSRNS_RelocSupport },
  { &hf_rrc_pdcp_PDU_Header , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PDCP_PDU_Header },
  { &hf_rrc_headerCompressionInfoList_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_HeaderCompressionInfoList_r4 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_PDCP_Info_r4(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_PDCP_Info_r4, PDCP_Info_r4_sequence);

  return offset;
}


static const per_sequence_t PDCP_InfoReconfig_r4_sequence[] = {
  { &hf_rrc_pdcp_Info_01    , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PDCP_Info_r4 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_PDCP_InfoReconfig_r4(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_PDCP_InfoReconfig_r4, PDCP_InfoReconfig_r4_sequence);

  return offset;
}


static const per_sequence_t RB_InformationReconfig_r4_sequence[] = {
  { &hf_rrc_rb_Identity     , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RB_Identity },
  { &hf_rrc_pdcp_Info_03    , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_PDCP_InfoReconfig_r4 },
  { &hf_rrc_pdcp_SN_Info    , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_PDCP_SN_Info },
  { &hf_rrc_rlc_Info        , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RLC_Info },
  { &hf_rrc_rb_MappingInfo  , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RB_MappingInfo },
  { &hf_rrc_rb_StopContinue , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RB_StopContinue },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_RB_InformationReconfig_r4(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_RB_InformationReconfig_r4, RB_InformationReconfig_r4_sequence);

  return offset;
}


static const per_sequence_t RB_InformationReconfigList_r4_sequence_of[1] = {
  { &hf_rrc_RB_InformationReconfigList_r4_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RB_InformationReconfig_r4 },
};

static int
dissect_rrc_RB_InformationReconfigList_r4(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_RB_InformationReconfigList_r4, RB_InformationReconfigList_r4_sequence_of,
                                                  1, maxRB, FALSE);

  return offset;
}


static const per_sequence_t T_fdd_57_sequence[] = {
  { &hf_rrc_ul_TFCS         , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TFCS },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_fdd_57(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_fdd_57, T_fdd_57_sequence);

  return offset;
}


static const per_sequence_t T_tdd_18_sequence[] = {
  { &hf_rrc_individualUL_CCTrCH_InfoList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_IndividualUL_CCTrCH_InfoList },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_tdd_18(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_tdd_18, T_tdd_18_sequence);

  return offset;
}


static const value_string rrc_T_modeSpecificInfo_26_vals[] = {
  {   0, "fdd" },
  {   1, "tdd" },
  { 0, NULL }
};

static const per_choice_t T_modeSpecificInfo_26_choice[] = {
  {   0, &hf_rrc_fdd_58          , ASN1_NO_EXTENSIONS     , dissect_rrc_T_fdd_57 },
  {   1, &hf_rrc_tdd_19          , ASN1_NO_EXTENSIONS     , dissect_rrc_T_tdd_18 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_modeSpecificInfo_26(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_modeSpecificInfo_26, T_modeSpecificInfo_26_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t T_tdd_16_sequence[] = {
  { &hf_rrc_tfcs_ID         , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_TFCS_Identity },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_tdd_16(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_tdd_16, T_tdd_16_sequence);

  return offset;
}


static const value_string rrc_T_modeSpecificInfo_24_vals[] = {
  {   0, "fdd" },
  {   1, "tdd" },
  { 0, NULL }
};

static const per_choice_t T_modeSpecificInfo_24_choice[] = {
  {   0, &hf_rrc_fdd_45          , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   1, &hf_rrc_tdd_17          , ASN1_NO_EXTENSIONS     , dissect_rrc_T_tdd_16 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_modeSpecificInfo_24(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_modeSpecificInfo_24, T_modeSpecificInfo_24_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t TFC_SubsetList_item_sequence[] = {
  { &hf_rrc_modeSpecificInfo_24, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_modeSpecificInfo_24 },
  { &hf_rrc_tfc_Subset      , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TFC_Subset },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_TFC_SubsetList_item(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_TFC_SubsetList_item, TFC_SubsetList_item_sequence);

  return offset;
}


static const per_sequence_t TFC_SubsetList_sequence_of[1] = {
  { &hf_rrc_TFC_SubsetList_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TFC_SubsetList_item },
};

static int
dissect_rrc_TFC_SubsetList(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_TFC_SubsetList, TFC_SubsetList_sequence_of,
                                                  1, maxTFCsub, FALSE);

  return offset;
}


static const per_sequence_t UL_CommonTransChInfo_r4_sequence[] = {
  { &hf_rrc_tfc_Subset      , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_TFC_Subset },
  { &hf_rrc_prach_TFCS      , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_TFCS },
  { &hf_rrc_modeSpecificInfo_26, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_modeSpecificInfo_26 },
  { &hf_rrc_tfc_SubsetList  , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_TFC_SubsetList },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UL_CommonTransChInfo_r4(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UL_CommonTransChInfo_r4, UL_CommonTransChInfo_r4_sequence);

  return offset;
}


static const per_sequence_t T_fdd_02_sequence[] = {
  { &hf_rrc_dummy_05        , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CPCH_SetID },
  { &hf_rrc_dummy2_02       , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DRAC_StaticInformationList },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_fdd_02(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_fdd_02, T_fdd_02_sequence);

  return offset;
}


static const value_string rrc_T_modeSpecificTransChInfo_01_vals[] = {
  {   0, "fdd" },
  {   1, "tdd" },
  { 0, NULL }
};

static const per_choice_t T_modeSpecificTransChInfo_01_choice[] = {
  {   0, &hf_rrc_fdd_02          , ASN1_NO_EXTENSIONS     , dissect_rrc_T_fdd_02 },
  {   1, &hf_rrc_tdd             , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_modeSpecificTransChInfo_01(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_modeSpecificTransChInfo_01, T_modeSpecificTransChInfo_01_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t T_dl_DCH_TFCS_sequence[] = {
  { &hf_rrc_tfcs            , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_TFCS },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_dl_DCH_TFCS(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_dl_DCH_TFCS, T_dl_DCH_TFCS_sequence);

  return offset;
}


static const value_string rrc_T_dl_Parameters_01_vals[] = {
  {   0, "dl-DCH-TFCS" },
  {   1, "sameAsUL" },
  { 0, NULL }
};

static const per_choice_t T_dl_Parameters_01_choice[] = {
  {   0, &hf_rrc_dl_DCH_TFCS_01  , ASN1_NO_EXTENSIONS     , dissect_rrc_T_dl_DCH_TFCS },
  {   1, &hf_rrc_sameAsUL        , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_dl_Parameters_01(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_dl_Parameters_01, T_dl_Parameters_01_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t T_fdd_50_sequence[] = {
  { &hf_rrc_dl_Parameters_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_dl_Parameters_01 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_fdd_50(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_fdd_50, T_fdd_50_sequence);

  return offset;
}


static const per_sequence_t T_tdd_15_sequence[] = {
  { &hf_rrc_individualDL_CCTrCH_InfoList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_IndividualDL_CCTrCH_InfoList },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_tdd_15(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_tdd_15, T_tdd_15_sequence);

  return offset;
}


static const value_string rrc_T_modeSpecificInfo_20_vals[] = {
  {   0, "fdd" },
  {   1, "tdd" },
  { 0, NULL }
};

static const per_choice_t T_modeSpecificInfo_20_choice[] = {
  {   0, &hf_rrc_fdd_51          , ASN1_NO_EXTENSIONS     , dissect_rrc_T_fdd_50 },
  {   1, &hf_rrc_tdd_16          , ASN1_NO_EXTENSIONS     , dissect_rrc_T_tdd_15 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_modeSpecificInfo_20(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_modeSpecificInfo_20, T_modeSpecificInfo_20_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t DL_CommonTransChInfo_r4_sequence[] = {
  { &hf_rrc_sccpch_TFCS     , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_TFCS },
  { &hf_rrc_modeSpecificInfo_20, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_modeSpecificInfo_20 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_DL_CommonTransChInfo_r4(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_DL_CommonTransChInfo_r4, DL_CommonTransChInfo_r4_sequence);

  return offset;
}


static const value_string rrc_T_tfs_SignallingMode_01_vals[] = {
  {   0, "explicit-config" },
  {   1, "sameAsULTrCH" },
  { 0, NULL }
};

static const per_choice_t T_tfs_SignallingMode_01_choice[] = {
  {   0, &hf_rrc_explicit_config , ASN1_NO_EXTENSIONS     , dissect_rrc_TransportFormatSet },
  {   1, &hf_rrc_sameAsULTrCH    , ASN1_NO_EXTENSIONS     , dissect_rrc_UL_TransportChannelIdentity },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_tfs_SignallingMode_01(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_tfs_SignallingMode_01, T_tfs_SignallingMode_01_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t DL_AddReconfTransChInformation_r4_sequence[] = {
  { &hf_rrc_dl_TransportChannelType_03, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DL_TrCH_Type },
  { &hf_rrc_dl_transportChannelIdentity, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TransportChannelIdentity },
  { &hf_rrc_tfs_SignallingMode_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_tfs_SignallingMode_01 },
  { &hf_rrc_dch_QualityTarget, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_QualityTarget },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_DL_AddReconfTransChInformation_r4(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_DL_AddReconfTransChInformation_r4, DL_AddReconfTransChInformation_r4_sequence);

  return offset;
}


static const per_sequence_t DL_AddReconfTransChInfoList_r4_sequence_of[1] = {
  { &hf_rrc_DL_AddReconfTransChInfoList_r4_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DL_AddReconfTransChInformation_r4 },
};

static int
dissect_rrc_DL_AddReconfTransChInfoList_r4(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_DL_AddReconfTransChInfoList_r4, DL_AddReconfTransChInfoList_r4_sequence_of,
                                                  1, maxTrCHpreconf, FALSE);

  return offset;
}


static const per_sequence_t T_fdd_122_sequence[] = {
  { &hf_rrc_dpcch_PowerOffset, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DPCCH_PowerOffset },
  { &hf_rrc_pc_Preamble     , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PC_Preamble },
  { &hf_rrc_sRB_delay       , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SRB_delay },
  { &hf_rrc_powerControlAlgorithm, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PowerControlAlgorithm },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_fdd_122(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_fdd_122, T_fdd_122_sequence);

  return offset;
}


static const per_sequence_t T_tdd384_36_sequence[] = {
  { &hf_rrc_individualTS_InterferenceList, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_IndividualTS_InterferenceList },
  { &hf_rrc_dpch_ConstantValue_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_ConstantValue },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_tdd384_36(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_tdd384_36, T_tdd384_36_sequence);

  return offset;
}


static const per_sequence_t T_tdd128_55_sequence[] = {
  { &hf_rrc_tpc_StepSize    , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TPC_StepSizeTDD },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_tdd128_55(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_tdd128_55, T_tdd128_55_sequence);

  return offset;
}


static const value_string rrc_T_tddOption_21_vals[] = {
  {   0, "tdd384" },
  {   1, "tdd128" },
  { 0, NULL }
};

static const per_choice_t T_tddOption_21_choice[] = {
  {   0, &hf_rrc_tdd384_40       , ASN1_NO_EXTENSIONS     , dissect_rrc_T_tdd384_36 },
  {   1, &hf_rrc_tdd128_57       , ASN1_NO_EXTENSIONS     , dissect_rrc_T_tdd128_55 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_tddOption_21(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_tddOption_21, T_tddOption_21_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t T_individuallySignalled_01_sequence[] = {
  { &hf_rrc_tddOption_21    , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_tddOption_21 },
  { &hf_rrc_primaryCCPCH_TX_Power, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PrimaryCCPCH_TX_Power },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_individuallySignalled_01(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_individuallySignalled_01, T_individuallySignalled_01_sequence);

  return offset;
}


static const value_string rrc_T_ul_OL_PC_Signalling_01_vals[] = {
  {   0, "broadcast-UL-OL-PC-info" },
  {   1, "individuallySignalled" },
  { 0, NULL }
};

static const per_choice_t T_ul_OL_PC_Signalling_01_choice[] = {
  {   0, &hf_rrc_broadcast_UL_OL_PC_info, ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   1, &hf_rrc_individuallySignalled_01, ASN1_NO_EXTENSIONS     , dissect_rrc_T_individuallySignalled_01 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_ul_OL_PC_Signalling_01(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_ul_OL_PC_Signalling_01, T_ul_OL_PC_Signalling_01_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t T_tdd_72_sequence[] = {
  { &hf_rrc_ul_TargetSIR    , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_TargetSIR },
  { &hf_rrc_ul_OL_PC_Signalling_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_ul_OL_PC_Signalling_01 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_tdd_72(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_tdd_72, T_tdd_72_sequence);

  return offset;
}


static const value_string rrc_UL_DPCH_PowerControlInfo_r4_vals[] = {
  {   0, "fdd" },
  {   1, "tdd" },
  { 0, NULL }
};

static const per_choice_t UL_DPCH_PowerControlInfo_r4_choice[] = {
  {   0, &hf_rrc_fdd_125         , ASN1_NO_EXTENSIONS     , dissect_rrc_T_fdd_122 },
  {   1, &hf_rrc_tdd_78          , ASN1_NO_EXTENSIONS     , dissect_rrc_T_tdd_72 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_UL_DPCH_PowerControlInfo_r4(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_UL_DPCH_PowerControlInfo_r4, UL_DPCH_PowerControlInfo_r4_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t T_fdd_114_sequence[] = {
  { &hf_rrc_scramblingCodeType, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_ScramblingCodeType },
  { &hf_rrc_scramblingCode  , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UL_ScramblingCode },
  { &hf_rrc_numberOfDPDCH   , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_NumberOfDPDCH },
  { &hf_rrc_spreadingFactor_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SpreadingFactor },
  { &hf_rrc_tfci_Existence  , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { &hf_rrc_numberOfFBI_Bits, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_NumberOfFBI_Bits },
  { &hf_rrc_puncturingLimit , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PuncturingLimit },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_fdd_114(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_fdd_114, T_fdd_114_sequence);

  return offset;
}


static const per_sequence_t T_tdd384_41_sequence[] = {
  { &hf_rrc_ul_TimingAdvance, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_TimingAdvance },
  { &hf_rrc_activationTime  , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_ActivationTime },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_tdd384_41(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_tdd384_41, T_tdd384_41_sequence);

  return offset;
}


static const per_sequence_t UL_SynchronisationParameters_r4_sequence[] = {
  { &hf_rrc_stepSize        , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_1_8 },
  { &hf_rrc_frequency       , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_1_8 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UL_SynchronisationParameters_r4(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UL_SynchronisationParameters_r4, UL_SynchronisationParameters_r4_sequence);

  return offset;
}



static int
dissect_rrc_T_sync_UL_CodesBitmap(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_bit_string(tvb, offset, actx, tree, hf_index,
                                     8, 8, FALSE, NULL);

  return offset;
}


static const value_string rrc_TDD_FPACH_CCode16_r4_vals[] = {
  {   0, "cc16-1" },
  {   1, "cc16-2" },
  {   2, "cc16-3" },
  {   3, "cc16-4" },
  {   4, "cc16-5" },
  {   5, "cc16-6" },
  {   6, "cc16-7" },
  {   7, "cc16-8" },
  {   8, "cc16-9" },
  {   9, "cc16-10" },
  {  10, "cc16-11" },
  {  11, "cc16-12" },
  {  12, "cc16-13" },
  {  13, "cc16-14" },
  {  14, "cc16-15" },
  {  15, "cc16-16" },
  { 0, NULL }
};


static int
dissect_rrc_TDD_FPACH_CCode16_r4(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     16, NULL, FALSE, 0, NULL);

  return offset;
}



static int
dissect_rrc_Wi_LCR(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            1U, 4U, NULL, FALSE);

  return offset;
}


static const per_sequence_t FPACH_Info_r4_sequence[] = {
  { &hf_rrc_timeslot_01     , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TimeslotNumber_LCR_r4 },
  { &hf_rrc_channelisationCode_03, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TDD_FPACH_CCode16_r4 },
  { &hf_rrc_midambleShiftAndBurstType_03, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MidambleShiftAndBurstType_LCR_r4 },
  { &hf_rrc_wi              , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_Wi_LCR },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_FPACH_Info_r4(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_FPACH_Info_r4, FPACH_Info_r4_sequence);

  return offset;
}


static const value_string rrc_T_max_SYNC_UL_Transmissions_vals[] = {
  {   0, "tr1" },
  {   1, "tr2" },
  {   2, "tr4" },
  {   3, "tr8" },
  { 0, NULL }
};


static int
dissect_rrc_T_max_SYNC_UL_Transmissions(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     4, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t SYNC_UL_Procedure_r4_sequence[] = {
  { &hf_rrc_max_SYNC_UL_Transmissions, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_max_SYNC_UL_Transmissions },
  { &hf_rrc_powerRampStep_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_0_3 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_SYNC_UL_Procedure_r4(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_SYNC_UL_Procedure_r4, SYNC_UL_Procedure_r4_sequence);

  return offset;
}


static const per_sequence_t SynchronisationParameters_r4_sequence[] = {
  { &hf_rrc_sync_UL_CodesBitmap, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_sync_UL_CodesBitmap },
  { &hf_rrc_fpach_Info      , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_FPACH_Info_r4 },
  { &hf_rrc_prxUpPCHdes     , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_0_62 },
  { &hf_rrc_sync_UL_Procedure, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_SYNC_UL_Procedure_r4 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_SynchronisationParameters_r4(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_SynchronisationParameters_r4, SynchronisationParameters_r4_sequence);

  return offset;
}


static const per_sequence_t T_tdd128_60_sequence[] = {
  { &hf_rrc_ul_SynchronisationParameters, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_SynchronisationParameters_r4 },
  { &hf_rrc_synchronisationParameters, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_SynchronisationParameters_r4 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_tdd128_60(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_tdd128_60, T_tdd128_60_sequence);

  return offset;
}


static const value_string rrc_T_tddOption_26_vals[] = {
  {   0, "tdd384" },
  {   1, "tdd128" },
  { 0, NULL }
};

static const per_choice_t T_tddOption_26_choice[] = {
  {   0, &hf_rrc_tdd384_45       , ASN1_NO_EXTENSIONS     , dissect_rrc_T_tdd384_41 },
  {   1, &hf_rrc_tdd128_62       , ASN1_NO_EXTENSIONS     , dissect_rrc_T_tdd128_60 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_tddOption_26(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_tddOption_26, T_tddOption_26_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t T_enabled_01_sequence[] = {
  { &hf_rrc_tddOption_26    , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_tddOption_26 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_enabled_01(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_enabled_01, T_enabled_01_sequence);

  return offset;
}


static const value_string rrc_UL_TimingAdvanceControl_r4_vals[] = {
  {   0, "disabled" },
  {   1, "enabled" },
  { 0, NULL }
};

static const per_choice_t UL_TimingAdvanceControl_r4_choice[] = {
  {   0, &hf_rrc_disabled        , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   1, &hf_rrc_enabled_01      , ASN1_NO_EXTENSIONS     , dissect_rrc_T_enabled_01 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_UL_TimingAdvanceControl_r4(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_UL_TimingAdvanceControl_r4, UL_TimingAdvanceControl_r4_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t T_tdd384_34_sequence[] = {
  { &hf_rrc_ul_CCTrCH_TimeslotsCodes, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UplinkTimeslotsCodes },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_tdd384_34(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_tdd384_34, T_tdd384_34_sequence);

  return offset;
}


static const per_sequence_t T_consecutive_04_sequence[] = {
  { &hf_rrc_numAdditionalTimeslots_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_1_maxTS_LCR_1 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_consecutive_04(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_consecutive_04, T_consecutive_04_sequence);

  return offset;
}


static const per_sequence_t T_sameAsLast_05_sequence[] = {
  { &hf_rrc_timeslotNumber  , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TimeslotNumber },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_sameAsLast_05(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_sameAsLast_05, T_sameAsLast_05_sequence);

  return offset;
}


static const per_sequence_t T_newParameters_05_sequence[] = {
  { &hf_rrc_individualTimeslotInfo_02, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_IndividualTimeslotInfo_LCR_r4 },
  { &hf_rrc_ul_TS_ChannelisationCodeList, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UL_TS_ChannelisationCodeList },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_newParameters_05(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_newParameters_05, T_newParameters_05_sequence);

  return offset;
}


static const value_string rrc_T_parameters_05_vals[] = {
  {   0, "sameAsLast" },
  {   1, "newParameters" },
  { 0, NULL }
};

static const per_choice_t T_parameters_05_choice[] = {
  {   0, &hf_rrc_sameAsLast_05   , ASN1_NO_EXTENSIONS     , dissect_rrc_T_sameAsLast_05 },
  {   1, &hf_rrc_newParameters_05, ASN1_NO_EXTENSIONS     , dissect_rrc_T_newParameters_05 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_parameters_05(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_parameters_05, T_parameters_05_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t UplinkAdditionalTimeslots_LCR_r4_sequence[] = {
  { &hf_rrc_parameters_05   , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_parameters_05 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UplinkAdditionalTimeslots_LCR_r4(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UplinkAdditionalTimeslots_LCR_r4, UplinkAdditionalTimeslots_LCR_r4_sequence);

  return offset;
}


static const per_sequence_t SEQUENCE_SIZE_1_maxTS_LCR_1_OF_UplinkAdditionalTimeslots_LCR_r4_sequence_of[1] = {
  { &hf_rrc_timeslotList_item_06, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UplinkAdditionalTimeslots_LCR_r4 },
};

static int
dissect_rrc_SEQUENCE_SIZE_1_maxTS_LCR_1_OF_UplinkAdditionalTimeslots_LCR_r4(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_SEQUENCE_SIZE_1_maxTS_LCR_1_OF_UplinkAdditionalTimeslots_LCR_r4, SEQUENCE_SIZE_1_maxTS_LCR_1_OF_UplinkAdditionalTimeslots_LCR_r4_sequence_of,
                                                  1, maxTS_LCR_1, FALSE);

  return offset;
}


static const value_string rrc_T_additionalTimeslots_06_vals[] = {
  {   0, "consecutive" },
  {   1, "timeslotList" },
  { 0, NULL }
};

static const per_choice_t T_additionalTimeslots_06_choice[] = {
  {   0, &hf_rrc_consecutive_06  , ASN1_NO_EXTENSIONS     , dissect_rrc_T_consecutive_04 },
  {   1, &hf_rrc_timeslotList_06 , ASN1_NO_EXTENSIONS     , dissect_rrc_SEQUENCE_SIZE_1_maxTS_LCR_1_OF_UplinkAdditionalTimeslots_LCR_r4 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_additionalTimeslots_06(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_additionalTimeslots_06, T_additionalTimeslots_06_choice,
                                 NULL);

  return offset;
}


static const value_string rrc_T_moreTimeslots_06_vals[] = {
  {   0, "noMore" },
  {   1, "additionalTimeslots" },
  { 0, NULL }
};

static const per_choice_t T_moreTimeslots_06_choice[] = {
  {   0, &hf_rrc_noMore          , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   1, &hf_rrc_additionalTimeslots_06, ASN1_NO_EXTENSIONS     , dissect_rrc_T_additionalTimeslots_06 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_moreTimeslots_06(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_moreTimeslots_06, T_moreTimeslots_06_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t UplinkTimeslotsCodes_LCR_r4_sequence[] = {
  { &hf_rrc_dynamicSFusage  , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { &hf_rrc_firstIndividualTimeslotInfo_02, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_IndividualTimeslotInfo_LCR_r4 },
  { &hf_rrc_ul_TS_ChannelisationCodeList, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UL_TS_ChannelisationCodeList },
  { &hf_rrc_moreTimeslots_06, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_moreTimeslots_06 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UplinkTimeslotsCodes_LCR_r4(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UplinkTimeslotsCodes_LCR_r4, UplinkTimeslotsCodes_LCR_r4_sequence);

  return offset;
}


static const per_sequence_t T_tdd128_53_sequence[] = {
  { &hf_rrc_ul_CCTrCH_TimeslotsCodes_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UplinkTimeslotsCodes_LCR_r4 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_tdd128_53(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_tdd128_53, T_tdd128_53_sequence);

  return offset;
}


static const value_string rrc_T_tddOption_19_vals[] = {
  {   0, "tdd384" },
  {   1, "tdd128" },
  { 0, NULL }
};

static const per_choice_t T_tddOption_19_choice[] = {
  {   0, &hf_rrc_tdd384_38       , ASN1_NO_EXTENSIONS     , dissect_rrc_T_tdd384_34 },
  {   1, &hf_rrc_tdd128_55       , ASN1_NO_EXTENSIONS     , dissect_rrc_T_tdd128_53 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_tddOption_19(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_tddOption_19, T_tddOption_19_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t UL_CCTrCH_r4_sequence[] = {
  { &hf_rrc_tfcs_ID_01      , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_TFCS_IdentityPlain },
  { &hf_rrc_ul_TargetSIR    , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UL_TargetSIR },
  { &hf_rrc_timeInfo        , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TimeInfo },
  { &hf_rrc_commonTimeslotInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CommonTimeslotInfo },
  { &hf_rrc_tddOption_19    , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_tddOption_19 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UL_CCTrCH_r4(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UL_CCTrCH_r4, UL_CCTrCH_r4_sequence);

  return offset;
}


static const per_sequence_t UL_CCTrCHList_r4_sequence_of[1] = {
  { &hf_rrc_UL_CCTrCHList_r4_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UL_CCTrCH_r4 },
};

static int
dissect_rrc_UL_CCTrCHList_r4(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_UL_CCTrCHList_r4, UL_CCTrCHList_r4_sequence_of,
                                                  1, maxCCTrCH, FALSE);

  return offset;
}


static const per_sequence_t T_tdd_65_sequence[] = {
  { &hf_rrc_ul_TimingAdvance_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_TimingAdvanceControl_r4 },
  { &hf_rrc_ul_CCTrCHList_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_CCTrCHList_r4 },
  { &hf_rrc_ul_CCTrCHListToRemove, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_CCTrCHListToRemove },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_tdd_65(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_tdd_65, T_tdd_65_sequence);

  return offset;
}


static const value_string rrc_T_modeSpecificInfo_82_vals[] = {
  {   0, "fdd" },
  {   1, "tdd" },
  { 0, NULL }
};

static const per_choice_t T_modeSpecificInfo_82_choice[] = {
  {   0, &hf_rrc_fdd_117         , ASN1_NO_EXTENSIONS     , dissect_rrc_T_fdd_114 },
  {   1, &hf_rrc_tdd_71          , ASN1_NO_EXTENSIONS     , dissect_rrc_T_tdd_65 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_modeSpecificInfo_82(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_modeSpecificInfo_82, T_modeSpecificInfo_82_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t UL_DPCH_Info_r4_sequence[] = {
  { &hf_rrc_ul_DPCH_PowerControlInfo_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_DPCH_PowerControlInfo_r4 },
  { &hf_rrc_modeSpecificInfo_82, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_modeSpecificInfo_82 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UL_DPCH_Info_r4(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UL_DPCH_Info_r4, UL_DPCH_Info_r4_sequence);

  return offset;
}


static const value_string rrc_UL_ChannelRequirement_r4_vals[] = {
  {   0, "ul-DPCH-Info" },
  {   1, "dummy" },
  { 0, NULL }
};

static const per_choice_t UL_ChannelRequirement_r4_choice[] = {
  {   0, &hf_rrc_ul_DPCH_Info_06 , ASN1_NO_EXTENSIONS     , dissect_rrc_UL_DPCH_Info_r4 },
  {   1, &hf_rrc_dummy_47        , ASN1_NO_EXTENSIONS     , dissect_rrc_CPCH_SetInfo },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_UL_ChannelRequirement_r4(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_UL_ChannelRequirement_r4, UL_ChannelRequirement_r4_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t T_fdd_03_sequence[] = {
  { &hf_rrc_dummy_06        , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_PDSCH_Information },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_fdd_03(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_fdd_03, T_fdd_03_sequence);

  return offset;
}


static const value_string rrc_T_modeSpecificPhysChInfo_01_vals[] = {
  {   0, "fdd" },
  {   1, "tdd" },
  { 0, NULL }
};

static const per_choice_t T_modeSpecificPhysChInfo_01_choice[] = {
  {   0, &hf_rrc_fdd_03          , ASN1_NO_EXTENSIONS     , dissect_rrc_T_fdd_03 },
  {   1, &hf_rrc_tdd             , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_modeSpecificPhysChInfo_01(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_modeSpecificPhysChInfo_01, T_modeSpecificPhysChInfo_01_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t T_initialise_01_sequence[] = {
  { &hf_rrc_dummy_40        , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_Cfntargetsfnframeoffset },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_initialise_01(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_initialise_01, T_initialise_01_sequence);

  return offset;
}


static const value_string rrc_T_cfnHandling_01_vals[] = {
  {   0, "maintain" },
  {   1, "initialise" },
  { 0, NULL }
};

static const per_choice_t T_cfnHandling_01_choice[] = {
  {   0, &hf_rrc_maintain        , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   1, &hf_rrc_initialise_01   , ASN1_NO_EXTENSIONS     , dissect_rrc_T_initialise_01 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_cfnHandling_01(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_cfnHandling_01, T_cfnHandling_01_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t T_fdd_67_sequence[] = {
  { &hf_rrc_dl_DPCH_PowerControlInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_DPCH_PowerControlInfo },
  { &hf_rrc_powerOffsetPilot_pdpdch, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PowerOffsetPilot_pdpdch },
  { &hf_rrc_dl_rate_matching_restriction, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_Dl_rate_matching_restriction },
  { &hf_rrc_spreadingFactorAndPilot, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SF512_AndPilot },
  { &hf_rrc_positionFixedOrFlexible, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PositionFixedOrFlexible },
  { &hf_rrc_tfci_Existence  , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_fdd_67(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_fdd_67, T_fdd_67_sequence);

  return offset;
}


static const per_sequence_t T_tdd_29_sequence[] = {
  { &hf_rrc_dl_DPCH_PowerControlInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_DPCH_PowerControlInfo },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_tdd_29(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_tdd_29, T_tdd_29_sequence);

  return offset;
}


static const value_string rrc_T_modeSpecificInfo_37_vals[] = {
  {   0, "fdd" },
  {   1, "tdd" },
  { 0, NULL }
};

static const per_choice_t T_modeSpecificInfo_37_choice[] = {
  {   0, &hf_rrc_fdd_68          , ASN1_NO_EXTENSIONS     , dissect_rrc_T_fdd_67 },
  {   1, &hf_rrc_tdd_30          , ASN1_NO_EXTENSIONS     , dissect_rrc_T_tdd_29 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_modeSpecificInfo_37(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_modeSpecificInfo_37, T_modeSpecificInfo_37_choice,
                                 NULL);

  return offset;
}



static int
dissect_rrc_MAC_d_HFN_initial_value(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_bit_string(tvb, offset, actx, tree, hf_index,
                                     24, 24, FALSE, NULL);

  return offset;
}


static const per_sequence_t DL_DPCH_InfoCommon_r4_sequence[] = {
  { &hf_rrc_cfnHandling_01  , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_cfnHandling_01 },
  { &hf_rrc_modeSpecificInfo_37, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_modeSpecificInfo_37 },
  { &hf_rrc_mac_d_HFN_initial_value, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MAC_d_HFN_initial_value },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_DL_DPCH_InfoCommon_r4(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_DL_DPCH_InfoCommon_r4, DL_DPCH_InfoCommon_r4_sequence);

  return offset;
}


static const per_sequence_t SSDT_Information_r4_sequence[] = {
  { &hf_rrc_s_Field         , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_S_Field },
  { &hf_rrc_codeWordSet     , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CodeWordSet },
  { &hf_rrc_ssdt_UL_r4      , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_SSDT_UL },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_SSDT_Information_r4(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_SSDT_Information_r4, SSDT_Information_r4_sequence);

  return offset;
}


static const per_sequence_t T_fdd_59_sequence[] = {
  { &hf_rrc_defaultDPCH_OffsetValue, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DefaultDPCH_OffsetValueFDD },
  { &hf_rrc_dpch_CompressedModeInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DPCH_CompressedModeInfo },
  { &hf_rrc_tx_DiversityMode, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_TX_DiversityMode },
  { &hf_rrc_dummy_39        , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_SSDT_Information_r4 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_fdd_59(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_fdd_59, T_fdd_59_sequence);

  return offset;
}


static const per_sequence_t T_tdd128_14_sequence[] = {
  { &hf_rrc_tstd_Indicator  , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_tdd128_14(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_tdd128_14, T_tdd128_14_sequence);

  return offset;
}


static const value_string rrc_T_tddOption_05_vals[] = {
  {   0, "tdd384" },
  {   1, "tdd128" },
  { 0, NULL }
};

static const per_choice_t T_tddOption_05_choice[] = {
  {   0, &hf_rrc_tdd384_11       , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   1, &hf_rrc_tdd128_14       , ASN1_NO_EXTENSIONS     , dissect_rrc_T_tdd128_14 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_tddOption_05(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_tddOption_05, T_tddOption_05_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t T_tdd_21_sequence[] = {
  { &hf_rrc_tddOption_05    , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_tddOption_05 },
  { &hf_rrc_defaultDPCH_OffsetValue_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DefaultDPCH_OffsetValueTDD },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_tdd_21(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_tdd_21, T_tdd_21_sequence);

  return offset;
}


static const value_string rrc_T_modeSpecificInfo_29_vals[] = {
  {   0, "fdd" },
  {   1, "tdd" },
  { 0, NULL }
};

static const per_choice_t T_modeSpecificInfo_29_choice[] = {
  {   0, &hf_rrc_fdd_60          , ASN1_NO_EXTENSIONS     , dissect_rrc_T_fdd_59 },
  {   1, &hf_rrc_tdd_22          , ASN1_NO_EXTENSIONS     , dissect_rrc_T_tdd_21 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_modeSpecificInfo_29(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_modeSpecificInfo_29, T_modeSpecificInfo_29_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t DL_CommonInformation_r4_sequence[] = {
  { &hf_rrc_dl_DPCH_InfoCommon_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_DPCH_InfoCommon_r4 },
  { &hf_rrc_modeSpecificInfo_29, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_modeSpecificInfo_29 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_DL_CommonInformation_r4(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_DL_CommonInformation_r4, DL_CommonInformation_r4_sequence);

  return offset;
}


static const per_sequence_t T_fdd_81_sequence[] = {
  { &hf_rrc_primaryCPICH_Info, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PrimaryCPICH_Info },
  { &hf_rrc_dummy1_02       , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_PDSCH_SHO_DCH_Info },
  { &hf_rrc_dummy2_07       , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_PDSCH_CodeMapping },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_fdd_81(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_fdd_81, T_fdd_81_sequence);

  return offset;
}


static const per_sequence_t T_fdd_106_sequence[] = {
  { &hf_rrc_tx_DiversityIndicator, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_fdd_106(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_fdd_106, T_fdd_106_sequence);

  return offset;
}


static const per_sequence_t T_syncCase1_01_sequence[] = {
  { &hf_rrc_timeslot        , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TimeslotNumber },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_syncCase1_01(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_syncCase1_01, T_syncCase1_01_sequence);

  return offset;
}


static const per_sequence_t T_syncCase2_01_sequence[] = {
  { &hf_rrc_timeslotSync2   , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TimeslotSync2 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_syncCase2_01(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_syncCase2_01, T_syncCase2_01_sequence);

  return offset;
}


static const value_string rrc_T_syncCase_01_vals[] = {
  {   0, "syncCase1" },
  {   1, "syncCase2" },
  { 0, NULL }
};

static const per_choice_t T_syncCase_01_choice[] = {
  {   0, &hf_rrc_syncCase1_01    , ASN1_NO_EXTENSIONS     , dissect_rrc_T_syncCase1_01 },
  {   1, &hf_rrc_syncCase2_01    , ASN1_NO_EXTENSIONS     , dissect_rrc_T_syncCase2_01 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_syncCase_01(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_syncCase_01, T_syncCase_01_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t T_tdd384_tdd768_04_sequence[] = {
  { &hf_rrc_syncCase_01     , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_syncCase_01 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_tdd384_tdd768_04(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_tdd384_tdd768_04, T_tdd384_tdd768_04_sequence);

  return offset;
}


static const per_sequence_t T_tdd128_46_sequence[] = {
  { &hf_rrc_tstd_Indicator  , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_tdd128_46(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_tdd128_46, T_tdd128_46_sequence);

  return offset;
}


static const value_string rrc_T_tddOption_14_vals[] = {
  {   0, "tdd384-tdd768" },
  {   1, "tdd128" },
  { 0, NULL }
};

static const per_choice_t T_tddOption_14_choice[] = {
  {   0, &hf_rrc_tdd384_tdd768_05, ASN1_NO_EXTENSIONS     , dissect_rrc_T_tdd384_tdd768_04 },
  {   1, &hf_rrc_tdd128_47       , ASN1_NO_EXTENSIONS     , dissect_rrc_T_tdd128_46 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_tddOption_14(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_tddOption_14, T_tddOption_14_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t T_tdd_60_sequence[] = {
  { &hf_rrc_tddOption_14    , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_tddOption_14 },
  { &hf_rrc_cellParametersID, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CellParametersID },
  { &hf_rrc_sctd_Indicator  , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_tdd_60(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_tdd_60, T_tdd_60_sequence);

  return offset;
}


static const value_string rrc_PrimaryCCPCH_Info_r4_vals[] = {
  {   0, "fdd" },
  {   1, "tdd" },
  { 0, NULL }
};

static const per_choice_t PrimaryCCPCH_Info_r4_choice[] = {
  {   0, &hf_rrc_fdd_109         , ASN1_NO_EXTENSIONS     , dissect_rrc_T_fdd_106 },
  {   1, &hf_rrc_tdd_66          , ASN1_NO_EXTENSIONS     , dissect_rrc_T_tdd_60 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_PrimaryCCPCH_Info_r4(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_PrimaryCCPCH_Info_r4, PrimaryCCPCH_Info_r4_choice,
                                 NULL);

  return offset;
}


static const value_string rrc_T_modeSpecificInfo_49_vals[] = {
  {   0, "fdd" },
  {   1, "tdd" },
  { 0, NULL }
};

static const per_choice_t T_modeSpecificInfo_49_choice[] = {
  {   0, &hf_rrc_fdd_82          , ASN1_NO_EXTENSIONS     , dissect_rrc_T_fdd_81 },
  {   1, &hf_rrc_tdd_47          , ASN1_NO_EXTENSIONS     , dissect_rrc_PrimaryCCPCH_Info_r4 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_modeSpecificInfo_49(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_modeSpecificInfo_49, T_modeSpecificInfo_49_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t T_fdd_71_sequence[] = {
  { &hf_rrc_pCPICH_UsageForChannelEst, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PCPICH_UsageForChannelEst },
  { &hf_rrc_dpch_FrameOffset, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DPCH_FrameOffset },
  { &hf_rrc_secondaryCPICH_Info, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_SecondaryCPICH_Info },
  { &hf_rrc_dl_ChannelisationCodeList, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DL_ChannelisationCodeList },
  { &hf_rrc_tpc_CombinationIndex, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TPC_CombinationIndex },
  { &hf_rrc_dummy_41        , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_SSDT_CellIdentity },
  { &hf_rrc_closedLoopTimingAdjMode, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_ClosedLoopTimingAdjMode },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_fdd_71(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_fdd_71, T_fdd_71_sequence);

  return offset;
}


static const per_sequence_t T_tdd_33_sequence[] = {
  { &hf_rrc_dl_CCTrChListToEstablish_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_CCTrChList_r4 },
  { &hf_rrc_dl_CCTrChListToRemove, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_CCTrChListToRemove },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_tdd_33(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_tdd_33, T_tdd_33_sequence);

  return offset;
}


static const value_string rrc_DL_DPCH_InfoPerRL_r4_vals[] = {
  {   0, "fdd" },
  {   1, "tdd" },
  { 0, NULL }
};

static const per_choice_t DL_DPCH_InfoPerRL_r4_choice[] = {
  {   0, &hf_rrc_fdd_72          , ASN1_NO_EXTENSIONS     , dissect_rrc_T_fdd_71 },
  {   1, &hf_rrc_tdd_34          , ASN1_NO_EXTENSIONS     , dissect_rrc_T_tdd_33 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_DL_DPCH_InfoPerRL_r4(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_DL_DPCH_InfoPerRL_r4, DL_DPCH_InfoPerRL_r4_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t T_fdd_110_sequence[] = {
  { &hf_rrc_secondaryScramblingCode, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_SecondaryScramblingCode },
  { &hf_rrc_sttd_Indicator  , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { &hf_rrc_sf_AndCodeNumber_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SF256_AndCodeNumber },
  { &hf_rrc_pilotSymbolExistence, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { &hf_rrc_tfci_Existence  , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { &hf_rrc_positionFixedOrFlexible, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PositionFixedOrFlexible },
  { &hf_rrc_timingOffset    , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_TimingOffset },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_fdd_110(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_fdd_110, T_fdd_110_sequence);

  return offset;
}


static const per_sequence_t T_tdd384_30_sequence[] = {
  { &hf_rrc_individualTimeslotInfo, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_IndividualTimeslotInfo },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_tdd384_30(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_tdd384_30, T_tdd384_30_sequence);

  return offset;
}


static const per_sequence_t T_tdd128_50_sequence[] = {
  { &hf_rrc_individualTimeslotInfo_02, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_IndividualTimeslotInfo_LCR_r4 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_tdd128_50(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_tdd128_50, T_tdd128_50_sequence);

  return offset;
}


static const value_string rrc_T_tddOption_18_vals[] = {
  {   0, "tdd384" },
  {   1, "tdd128" },
  { 0, NULL }
};

static const per_choice_t T_tddOption_18_choice[] = {
  {   0, &hf_rrc_tdd384_33       , ASN1_NO_EXTENSIONS     , dissect_rrc_T_tdd384_30 },
  {   1, &hf_rrc_tdd128_51       , ASN1_NO_EXTENSIONS     , dissect_rrc_T_tdd128_50 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_tddOption_18(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_tddOption_18, T_tddOption_18_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t T_tdd_63_sequence[] = {
  { &hf_rrc_commonTimeslotInfo_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CommonTimeslotInfoSCCPCH },
  { &hf_rrc_tddOption_18    , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_tddOption_18 },
  { &hf_rrc_channelisationCode_09, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SCCPCH_ChannelisationCodeList },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_tdd_63(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_tdd_63, T_tdd_63_sequence);

  return offset;
}


static const value_string rrc_T_modeSpecificInfo_77_vals[] = {
  {   0, "fdd" },
  {   1, "tdd" },
  { 0, NULL }
};

static const per_choice_t T_modeSpecificInfo_77_choice[] = {
  {   0, &hf_rrc_fdd_113         , ASN1_NO_EXTENSIONS     , dissect_rrc_T_fdd_110 },
  {   1, &hf_rrc_tdd_69          , ASN1_NO_EXTENSIONS     , dissect_rrc_T_tdd_63 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_modeSpecificInfo_77(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_modeSpecificInfo_77, T_modeSpecificInfo_77_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t SecondaryCCPCH_Info_r4_sequence[] = {
  { &hf_rrc_modeSpecificInfo_77, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_modeSpecificInfo_77 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_SecondaryCCPCH_Info_r4(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_SecondaryCCPCH_Info_r4, SecondaryCCPCH_Info_r4_sequence);

  return offset;
}


static const per_sequence_t T_fdd_108_sequence[] = {
  { &hf_rrc_sib_ReferenceListFACH, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SIB_ReferenceListFACH },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_fdd_108(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_fdd_108, T_fdd_108_sequence);

  return offset;
}


static const value_string rrc_T_modeSpecificInfo_75_vals[] = {
  {   0, "fdd" },
  {   1, "tdd" },
  { 0, NULL }
};

static const per_choice_t T_modeSpecificInfo_75_choice[] = {
  {   0, &hf_rrc_fdd_111         , ASN1_NO_EXTENSIONS     , dissect_rrc_T_fdd_108 },
  {   1, &hf_rrc_tdd             , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_modeSpecificInfo_75(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_modeSpecificInfo_75, T_modeSpecificInfo_75_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t SCCPCH_InfoForFACH_r4_sequence[] = {
  { &hf_rrc_secondaryCCPCH_Info_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SecondaryCCPCH_Info_r4 },
  { &hf_rrc_tfcs            , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TFCS },
  { &hf_rrc_fach_PCH_InformationList, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_FACH_PCH_InformationList },
  { &hf_rrc_modeSpecificInfo_75, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_modeSpecificInfo_75 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_SCCPCH_InfoForFACH_r4(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_SCCPCH_InfoForFACH_r4, SCCPCH_InfoForFACH_r4_sequence);

  return offset;
}


static const per_sequence_t DL_InformationPerRL_r4_sequence[] = {
  { &hf_rrc_modeSpecificInfo_49, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_modeSpecificInfo_49 },
  { &hf_rrc_dl_DPCH_InfoPerRL_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_DPCH_InfoPerRL_r4 },
  { &hf_rrc_dummy_43        , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_SCCPCH_InfoForFACH_r4 },
  { &hf_rrc_cell_id         , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CellIdentity },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_DL_InformationPerRL_r4(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_DL_InformationPerRL_r4, DL_InformationPerRL_r4_sequence);

  return offset;
}


static const per_sequence_t DL_InformationPerRL_List_r4_sequence_of[1] = {
  { &hf_rrc_DL_InformationPerRL_List_r4_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DL_InformationPerRL_r4 },
};

static int
dissect_rrc_DL_InformationPerRL_List_r4(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_DL_InformationPerRL_List_r4, DL_InformationPerRL_List_r4_sequence_of,
                                                  1, maxRL, FALSE);

  return offset;
}


static const per_sequence_t CellUpdateConfirm_r4_IEs_sequence[] = {
  { &hf_rrc_integrityProtectionModeInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_IntegrityProtectionModeInfo },
  { &hf_rrc_cipheringModeInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CipheringModeInfo },
  { &hf_rrc_activationTime  , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_ActivationTime },
  { &hf_rrc_new_U_RNTI      , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_U_RNTI },
  { &hf_rrc_new_C_RNTI      , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_C_RNTI },
  { &hf_rrc_new_DSCH_RNTI   , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DSCH_RNTI },
  { &hf_rrc_rrc_StateIndicator, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RRC_StateIndicator },
  { &hf_rrc_utran_DRX_CycleLengthCoeff, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UTRAN_DRX_CycleLengthCoefficient },
  { &hf_rrc_rlc_Re_establishIndicatorRb2_3or4, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { &hf_rrc_rlc_Re_establishIndicatorRb5orAbove, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { &hf_rrc_cn_InformationInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CN_InformationInfo },
  { &hf_rrc_ura_Identity    , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_URA_Identity },
  { &hf_rrc_rb_InformationReleaseList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RB_InformationReleaseList },
  { &hf_rrc_rb_InformationReconfigList_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RB_InformationReconfigList_r4 },
  { &hf_rrc_rb_InformationAffectedList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RB_InformationAffectedList },
  { &hf_rrc_dl_CounterSynchronisationInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_CounterSynchronisationInfo },
  { &hf_rrc_ul_CommonTransChInfo_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_CommonTransChInfo_r4 },
  { &hf_rrc_ul_deletedTransChInfoList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_DeletedTransChInfoList },
  { &hf_rrc_ul_AddReconfTransChInfoList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_AddReconfTransChInfoList },
  { &hf_rrc_modeSpecificTransChInfo_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_modeSpecificTransChInfo_01 },
  { &hf_rrc_dl_CommonTransChInfo_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_CommonTransChInfo_r4 },
  { &hf_rrc_dl_DeletedTransChInfoList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_DeletedTransChInfoList },
  { &hf_rrc_dl_AddReconfTransChInfoList_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_AddReconfTransChInfoList_r4 },
  { &hf_rrc_frequencyInfo   , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_FrequencyInfo },
  { &hf_rrc_maxAllowedUL_TX_Power, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MaxAllowedUL_TX_Power },
  { &hf_rrc_ul_ChannelRequirement_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_ChannelRequirement_r4 },
  { &hf_rrc_modeSpecificPhysChInfo_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_modeSpecificPhysChInfo_01 },
  { &hf_rrc_dl_CommonInformation_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_CommonInformation_r4 },
  { &hf_rrc_dl_InformationPerRL_List_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_InformationPerRL_List_r4 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_CellUpdateConfirm_r4_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_CellUpdateConfirm_r4_IEs, CellUpdateConfirm_r4_IEs_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions_15_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_nonCriticalExtensions_15(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_nonCriticalExtensions_15, T_nonCriticalExtensions_15_sequence);

  return offset;
}


static const per_sequence_t T_v690NonCriticalExtensions_03_sequence[] = {
  { &hf_rrc_cellUpdateConfirm_v690ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CellUpdateConfirm_v690ext_IEs },
  { &hf_rrc_nonCriticalExtensions_15, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_nonCriticalExtensions_15 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v690NonCriticalExtensions_03(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v690NonCriticalExtensions_03, T_v690NonCriticalExtensions_03_sequence);

  return offset;
}


static const per_sequence_t T_v5d0NonCriticalExtenstions_01_sequence[] = {
  { &hf_rrc_cellUpdateConfirm_v5d0ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CellUpdateConfirm_v5d0ext_IEs },
  { &hf_rrc_v690NonCriticalExtensions_03, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v690NonCriticalExtensions_03 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v5d0NonCriticalExtenstions_01(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v5d0NonCriticalExtenstions_01, T_v5d0NonCriticalExtenstions_01_sequence);

  return offset;
}


static const per_sequence_t T_v590NonCriticalExtenstions_01_sequence[] = {
  { &hf_rrc_cellUpdateConfirm_v590ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CellUpdateConfirm_v590ext_IEs },
  { &hf_rrc_v5d0NonCriticalExtenstions_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v5d0NonCriticalExtenstions_01 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v590NonCriticalExtenstions_01(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v590NonCriticalExtenstions_01, T_v590NonCriticalExtenstions_01_sequence);

  return offset;
}


static const per_sequence_t T_v4d0NonCriticalExtensions_sequence[] = {
  { &hf_rrc_cellUpdateConfirm_r4_add_ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_BIT_STRING },
  { &hf_rrc_v590NonCriticalExtenstions_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v590NonCriticalExtenstions_01 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v4d0NonCriticalExtensions(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v4d0NonCriticalExtensions, T_v4d0NonCriticalExtensions_sequence);

  return offset;
}


static const per_sequence_t T_r4_sequence[] = {
  { &hf_rrc_cellUpdateConfirm_r4, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CellUpdateConfirm_r4_IEs },
  { &hf_rrc_v4d0NonCriticalExtensions, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v4d0NonCriticalExtensions },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_r4(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_r4, T_r4_sequence);

  return offset;
}


static const per_sequence_t DL_AM_RLC_Mode_r5_sequence[] = {
  { &hf_rrc_dl_RLC_PDU_size , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_OctetModeRLC_SizeInfoType1 },
  { &hf_rrc_inSequenceDelivery, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { &hf_rrc_receivingWindowSize, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_ReceivingWindowSize },
  { &hf_rrc_dl_RLC_StatusInfo, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DL_RLC_StatusInfo },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_DL_AM_RLC_Mode_r5(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_DL_AM_RLC_Mode_r5, DL_AM_RLC_Mode_r5_sequence);

  return offset;
}


static const value_string rrc_DL_UM_RLC_LI_size_vals[] = {
  {   0, "size7" },
  {   1, "size15" },
  { 0, NULL }
};


static int
dissect_rrc_DL_UM_RLC_LI_size(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     2, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t DL_UM_RLC_Mode_r5_sequence[] = {
  { &hf_rrc_dl_UM_RLC_LI_size, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DL_UM_RLC_LI_size },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_DL_UM_RLC_Mode_r5(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_DL_UM_RLC_Mode_r5, DL_UM_RLC_Mode_r5_sequence);

  return offset;
}


static const value_string rrc_DL_RLC_Mode_r5_vals[] = {
  {   0, "dl-AM-RLC-Mode" },
  {   1, "dl-UM-RLC-Mode" },
  {   2, "dl-TM-RLC-Mode" },
  { 0, NULL }
};

static const per_choice_t DL_RLC_Mode_r5_choice[] = {
  {   0, &hf_rrc_dl_AM_RLC_Mode_01, ASN1_NO_EXTENSIONS     , dissect_rrc_DL_AM_RLC_Mode_r5 },
  {   1, &hf_rrc_dl_UM_RLC_Mode_01, ASN1_NO_EXTENSIONS     , dissect_rrc_DL_UM_RLC_Mode_r5 },
  {   2, &hf_rrc_dl_TM_RLC_Mode  , ASN1_NO_EXTENSIONS     , dissect_rrc_DL_TM_RLC_Mode },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_DL_RLC_Mode_r5(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_DL_RLC_Mode_r5, DL_RLC_Mode_r5_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t RLC_Info_r5_sequence[] = {
  { &hf_rrc_ul_RLC_Mode     , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_RLC_Mode },
  { &hf_rrc_dl_RLC_Mode_01  , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_RLC_Mode_r5 },
  { &hf_rrc_rlc_OneSidedReEst, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_RLC_Info_r5(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_RLC_Info_r5, RLC_Info_r5_sequence);

  return offset;
}



static int
dissect_rrc_MAC_d_FlowIdentity(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
#line 777 "../../asn1/rrc/rrc.cnf"
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            0U, 7U, &flowd, FALSE);




  return offset;
}


static const per_sequence_t MAC_d_FlowIdentityDCHandHSDSCH_sequence[] = {
  { &hf_rrc_dch_transport_ch_id, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TransportChannelIdentity },
  { &hf_rrc_hsdsch_mac_d_flow_id, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MAC_d_FlowIdentity },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_MAC_d_FlowIdentityDCHandHSDSCH(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_MAC_d_FlowIdentityDCHandHSDSCH, MAC_d_FlowIdentityDCHandHSDSCH_sequence);

  return offset;
}


static const value_string rrc_DL_TransportChannelType_r5_vals[] = {
  {   0, "dch" },
  {   1, "fach" },
  {   2, "dsch" },
  {   3, "dch-and-dsch" },
  {   4, "hsdsch" },
  {   5, "dch-and-hsdsch" },
  { 0, NULL }
};

static const per_choice_t DL_TransportChannelType_r5_choice[] = {
  {   0, &hf_rrc_dch             , ASN1_NO_EXTENSIONS     , dissect_rrc_TransportChannelIdentity },
  {   1, &hf_rrc_fach            , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   2, &hf_rrc_dsch            , ASN1_NO_EXTENSIONS     , dissect_rrc_TransportChannelIdentity },
  {   3, &hf_rrc_dch_and_dsch    , ASN1_NO_EXTENSIONS     , dissect_rrc_TransportChannelIdentityDCHandDSCH },
  {   4, &hf_rrc_hsdsch          , ASN1_NO_EXTENSIONS     , dissect_rrc_MAC_d_FlowIdentity },
  {   5, &hf_rrc_dch_and_hsdsch  , ASN1_NO_EXTENSIONS     , dissect_rrc_MAC_d_FlowIdentityDCHandHSDSCH },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_DL_TransportChannelType_r5(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
#line 706 "../../asn1/rrc/rrc.cnf"

    gint *flowd_p;
    gint *cur_val=NULL;
    struct rrc_info *rrcinf;

      offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_DL_TransportChannelType_r5, DL_TransportChannelType_r5_choice,
                                 &type);


    /*TODO: This should probably be done better*/
    if(type == 4){    /*If this is type HS-DSCH*/
        num_chans_per_flow[flowd]++;

        if(num_chans_per_flow[flowd] > 1 ){
            rrcinf = p_get_proto_data(actx->pinfo->fd, proto_rrc);
            if((rrcinf == NULL) || (rrcinf->hrnti[actx->pinfo->fd->subnum] == 0)){
                expert_add_info_format(actx->pinfo, actx->created_item, PI_SEQUENCE, PI_NOTE, "Did not detect any H-RNTI");
            }
            else{
                /*If it doesnt exists, insert it*/
                if( (cur_val=g_tree_lookup(hsdsch_muxed_flows, GUINT_TO_POINTER((guint)rrcinf->hrnti[actx->pinfo->fd->subnum]))) == NULL ){

                    flowd_p = (guint*)g_malloc0(sizeof(gint));
                    *flowd_p = (1<<flowd);    /*Set the bit to mark it as true*/
                    g_tree_insert(hsdsch_muxed_flows, GUINT_TO_POINTER((guint)rrcinf->hrnti[actx->pinfo->fd->subnum]), flowd_p);

                }else{
                    *cur_val = (1<<flowd) | *cur_val;

                }
            }

        }

    }
/*Here we try to figure out which HS-DSCH channels are multiplexed*/


  return offset;
}


static const per_sequence_t DL_LogicalChannelMapping_r5_sequence[] = {
  { &hf_rrc_dl_TransportChannelType_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DL_TransportChannelType_r5 },
  { &hf_rrc_logicalChannelIdentity, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_LogicalChannelIdentity },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_DL_LogicalChannelMapping_r5(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_DL_LogicalChannelMapping_r5, DL_LogicalChannelMapping_r5_sequence);

  return offset;
}


static const per_sequence_t DL_LogicalChannelMappingList_r5_sequence_of[1] = {
  { &hf_rrc_DL_LogicalChannelMappingList_r5_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DL_LogicalChannelMapping_r5 },
};

static int
dissect_rrc_DL_LogicalChannelMappingList_r5(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_DL_LogicalChannelMappingList_r5, DL_LogicalChannelMappingList_r5_sequence_of,
                                                  1, maxLoCHperRLC, FALSE);

  return offset;
}


static const per_sequence_t RB_MappingOption_r5_sequence[] = {
  { &hf_rrc_ul_LogicalChannelMappings, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_LogicalChannelMappings },
  { &hf_rrc_dl_LogicalChannelMappingList_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_LogicalChannelMappingList_r5 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_RB_MappingOption_r5(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_RB_MappingOption_r5, RB_MappingOption_r5_sequence);

  return offset;
}


static const per_sequence_t RB_MappingInfo_r5_sequence_of[1] = {
  { &hf_rrc_RB_MappingInfo_r5_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RB_MappingOption_r5 },
};

static int
dissect_rrc_RB_MappingInfo_r5(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_RB_MappingInfo_r5, RB_MappingInfo_r5_sequence_of,
                                                  1, maxRBMuxOptions, FALSE);

  return offset;
}


static const per_sequence_t RB_InformationReconfig_r5_sequence[] = {
  { &hf_rrc_rb_Identity     , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RB_Identity },
  { &hf_rrc_pdcp_Info_03    , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_PDCP_InfoReconfig_r4 },
  { &hf_rrc_pdcp_SN_Info    , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_PDCP_SN_Info },
  { &hf_rrc_rlc_Info_01     , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RLC_Info_r5 },
  { &hf_rrc_rb_MappingInfo_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RB_MappingInfo_r5 },
  { &hf_rrc_rb_StopContinue , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RB_StopContinue },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_RB_InformationReconfig_r5(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_RB_InformationReconfig_r5, RB_InformationReconfig_r5_sequence);

  return offset;
}


static const per_sequence_t RB_InformationReconfigList_r5_sequence_of[1] = {
  { &hf_rrc_RB_InformationReconfigList_r5_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RB_InformationReconfig_r5 },
};

static int
dissect_rrc_RB_InformationReconfigList_r5(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_RB_InformationReconfigList_r5, RB_InformationReconfigList_r5_sequence_of,
                                                  1, maxRB, FALSE);

  return offset;
}


static const per_sequence_t RB_InformationAffected_r5_sequence[] = {
  { &hf_rrc_rb_Identity     , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RB_Identity },
  { &hf_rrc_rb_MappingInfo_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RB_MappingInfo_r5 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_RB_InformationAffected_r5(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_RB_InformationAffected_r5, RB_InformationAffected_r5_sequence);

  return offset;
}


static const per_sequence_t RB_InformationAffectedList_r5_sequence_of[1] = {
  { &hf_rrc_RB_InformationAffectedList_r5_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RB_InformationAffected_r5 },
};

static int
dissect_rrc_RB_InformationAffectedList_r5(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_RB_InformationAffectedList_r5, RB_InformationAffectedList_r5_sequence_of,
                                                  1, maxRB, FALSE);

  return offset;
}


static const per_sequence_t RB_PDCPContextRelocation_sequence[] = {
  { &hf_rrc_rb_Identity     , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RB_Identity },
  { &hf_rrc_dl_RFC3095_Context_Relocation, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { &hf_rrc_ul_RFC3095_Context_Relocation, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_RB_PDCPContextRelocation(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_RB_PDCPContextRelocation, RB_PDCPContextRelocation_sequence);

  return offset;
}


static const per_sequence_t RB_PDCPContextRelocationList_sequence_of[1] = {
  { &hf_rrc_RB_PDCPContextRelocationList_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RB_PDCPContextRelocation },
};

static int
dissect_rrc_RB_PDCPContextRelocationList(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_RB_PDCPContextRelocationList, RB_PDCPContextRelocationList_sequence_of,
                                                  1, maxRBallRABs, FALSE);

  return offset;
}


static const per_sequence_t DL_CounterSynchronisationInfo_r5_sequence[] = {
  { &hf_rrc_rb_WithPDCP_InfoList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RB_WithPDCP_InfoList },
  { &hf_rrc_rb_PDCPContextRelocationList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RB_PDCPContextRelocationList },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_DL_CounterSynchronisationInfo_r5(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_DL_CounterSynchronisationInfo_r5, DL_CounterSynchronisationInfo_r5_sequence);

  return offset;
}


static const per_sequence_t T_fdd_04_sequence[] = {
  { &hf_rrc_dummy_05        , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CPCH_SetID },
  { &hf_rrc_dummy2_02       , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DRAC_StaticInformationList },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_fdd_04(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_fdd_04, T_fdd_04_sequence);

  return offset;
}


static const value_string rrc_T_modeSpecificTransChInfo_02_vals[] = {
  {   0, "fdd" },
  {   1, "tdd" },
  { 0, NULL }
};

static const per_choice_t T_modeSpecificTransChInfo_02_choice[] = {
  {   0, &hf_rrc_fdd_04          , ASN1_NO_EXTENSIONS     , dissect_rrc_T_fdd_04 },
  {   1, &hf_rrc_tdd             , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_modeSpecificTransChInfo_02(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_modeSpecificTransChInfo_02, T_modeSpecificTransChInfo_02_choice,
                                 NULL);

  return offset;
}


static const value_string rrc_DL_TrCH_TypeId2_r5_vals[] = {
  {   0, "dch" },
  {   1, "dsch" },
  {   2, "hsdsch" },
  { 0, NULL }
};

static const per_choice_t DL_TrCH_TypeId2_r5_choice[] = {
  {   0, &hf_rrc_dch             , ASN1_NO_EXTENSIONS     , dissect_rrc_TransportChannelIdentity },
  {   1, &hf_rrc_dsch            , ASN1_NO_EXTENSIONS     , dissect_rrc_TransportChannelIdentity },
  {   2, &hf_rrc_hsdsch          , ASN1_NO_EXTENSIONS     , dissect_rrc_MAC_d_FlowIdentity },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_DL_TrCH_TypeId2_r5(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_DL_TrCH_TypeId2_r5, DL_TrCH_TypeId2_r5_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t DL_TransportChannelIdentity_r5_sequence[] = {
  { &hf_rrc_dl_TransportChannelType_05, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DL_TrCH_TypeId2_r5 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_DL_TransportChannelIdentity_r5(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_DL_TransportChannelIdentity_r5, DL_TransportChannelIdentity_r5_sequence);

  return offset;
}


static const per_sequence_t DL_DeletedTransChInfoList_r5_sequence_of[1] = {
  { &hf_rrc_DL_DeletedTransChInfoList_r5_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DL_TransportChannelIdentity_r5 },
};

static int
dissect_rrc_DL_DeletedTransChInfoList_r5(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_DL_DeletedTransChInfoList_r5, DL_DeletedTransChInfoList_r5_sequence_of,
                                                  1, maxTrCH, FALSE);

  return offset;
}


static const value_string rrc_DL_TrCH_TypeId1_r5_vals[] = {
  {   0, "dch" },
  {   1, "dsch" },
  {   2, "hsdsch" },
  { 0, NULL }
};

static const per_choice_t DL_TrCH_TypeId1_r5_choice[] = {
  {   0, &hf_rrc_dch             , ASN1_NO_EXTENSIONS     , dissect_rrc_TransportChannelIdentity },
  {   1, &hf_rrc_dsch            , ASN1_NO_EXTENSIONS     , dissect_rrc_TransportChannelIdentity },
  {   2, &hf_rrc_hsdsch_07       , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_DL_TrCH_TypeId1_r5(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_DL_TrCH_TypeId1_r5, DL_TrCH_TypeId1_r5_choice,
                                 NULL);

  return offset;
}


static const value_string rrc_T1_ReleaseTimer_vals[] = {
  {   0, "rt10" },
  {   1, "rt20" },
  {   2, "rt30" },
  {   3, "rt40" },
  {   4, "rt50" },
  {   5, "rt60" },
  {   6, "rt70" },
  {   7, "rt80" },
  {   8, "rt90" },
  {   9, "rt100" },
  {  10, "rt120" },
  {  11, "rt140" },
  {  12, "rt160" },
  {  13, "rt200" },
  {  14, "rt300" },
  {  15, "rt400" },
  { 0, NULL }
};


static int
dissect_rrc_T1_ReleaseTimer(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     16, NULL, FALSE, 0, NULL);

  return offset;
}


static const value_string rrc_MAC_hs_WindowSize_vals[] = {
  {   0, "mws4" },
  {   1, "mws6" },
  {   2, "mws8" },
  {   3, "mws12" },
  {   4, "mws16" },
  {   5, "mws24" },
  {   6, "mws32" },
  { 0, NULL }
};


static int
dissect_rrc_MAC_hs_WindowSize(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     7, NULL, FALSE, 0, NULL);

  return offset;
}



static int
dissect_rrc_INTEGER_1_5000(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            1U, 5000U, NULL, FALSE);

  return offset;
}


static const per_sequence_t MAC_d_PDUsizeInfo_sequence[] = {
  { &hf_rrc_mac_d_PDU_Size  , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_1_5000 },
  { &hf_rrc_mac_d_PDU_Index , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_0_7 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_MAC_d_PDUsizeInfo(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_MAC_d_PDUsizeInfo, MAC_d_PDUsizeInfo_sequence);

  return offset;
}


static const per_sequence_t MAC_d_PDU_SizeInfo_List_sequence_of[1] = {
  { &hf_rrc_MAC_d_PDU_SizeInfo_List_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MAC_d_PDUsizeInfo },
};

static int
dissect_rrc_MAC_d_PDU_SizeInfo_List(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_MAC_d_PDU_SizeInfo_List, MAC_d_PDU_SizeInfo_List_sequence_of,
                                                  1, maxMAC_d_PDUsizes, FALSE);

  return offset;
}


static const per_sequence_t MAC_hs_AddReconfQueue_sequence[] = {
  { &hf_rrc_mac_hsQueueId   , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_0_7 },
  { &hf_rrc_mac_dFlowId     , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MAC_d_FlowIdentity },
  { &hf_rrc_reorderingReleaseTimer, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T1_ReleaseTimer },
  { &hf_rrc_mac_hsWindowSize, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MAC_hs_WindowSize },
  { &hf_rrc_mac_d_PDU_SizeInfo_List, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MAC_d_PDU_SizeInfo_List },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_MAC_hs_AddReconfQueue(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_MAC_hs_AddReconfQueue, MAC_hs_AddReconfQueue_sequence);

  return offset;
}


static const per_sequence_t MAC_hs_AddReconfQueue_List_sequence_of[1] = {
  { &hf_rrc_MAC_hs_AddReconfQueue_List_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MAC_hs_AddReconfQueue },
};

static int
dissect_rrc_MAC_hs_AddReconfQueue_List(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_MAC_hs_AddReconfQueue_List, MAC_hs_AddReconfQueue_List_sequence_of,
                                                  1, maxQueueIDs, FALSE);

  return offset;
}


static const per_sequence_t MAC_hs_DelQueue_sequence[] = {
  { &hf_rrc_mac_hsQueueId   , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_0_7 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_MAC_hs_DelQueue(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_MAC_hs_DelQueue, MAC_hs_DelQueue_sequence);

  return offset;
}


static const per_sequence_t MAC_hs_DelQueue_List_sequence_of[1] = {
  { &hf_rrc_MAC_hs_DelQueue_List_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MAC_hs_DelQueue },
};

static int
dissect_rrc_MAC_hs_DelQueue_List(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_MAC_hs_DelQueue_List, MAC_hs_DelQueue_List_sequence_of,
                                                  1, maxQueueIDs, FALSE);

  return offset;
}


static const per_sequence_t AddOrReconfMAC_dFlow_sequence[] = {
  { &hf_rrc_mac_hs_AddReconfQueue_List, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MAC_hs_AddReconfQueue_List },
  { &hf_rrc_mac_hs_DelQueue_List, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MAC_hs_DelQueue_List },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_AddOrReconfMAC_dFlow(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_AddOrReconfMAC_dFlow, AddOrReconfMAC_dFlow_sequence);

  return offset;
}


static const per_sequence_t HSDSCH_Info_sequence[] = {
  { &hf_rrc_harqInfo_02     , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_HARQ_Info },
  { &hf_rrc_addOrReconfMAC_dFlow, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_AddOrReconfMAC_dFlow },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_HSDSCH_Info(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_HSDSCH_Info, HSDSCH_Info_sequence);

  return offset;
}


static const value_string rrc_T_tfs_SignallingMode_02_vals[] = {
  {   0, "explicit-config" },
  {   1, "sameAsULTrCH" },
  {   2, "hsdsch" },
  { 0, NULL }
};

static const per_choice_t T_tfs_SignallingMode_02_choice[] = {
  {   0, &hf_rrc_explicit_config , ASN1_NO_EXTENSIONS     , dissect_rrc_TransportFormatSet },
  {   1, &hf_rrc_sameAsULTrCH    , ASN1_NO_EXTENSIONS     , dissect_rrc_UL_TransportChannelIdentity },
  {   2, &hf_rrc_hsdsch_02       , ASN1_NO_EXTENSIONS     , dissect_rrc_HSDSCH_Info },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_tfs_SignallingMode_02(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_tfs_SignallingMode_02, T_tfs_SignallingMode_02_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t DL_AddReconfTransChInformation_r5_sequence[] = {
  { &hf_rrc_dl_TransportChannelType_04, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DL_TrCH_TypeId1_r5 },
  { &hf_rrc_tfs_SignallingMode_02, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_tfs_SignallingMode_02 },
  { &hf_rrc_dch_QualityTarget, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_QualityTarget },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_DL_AddReconfTransChInformation_r5(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_DL_AddReconfTransChInformation_r5, DL_AddReconfTransChInformation_r5_sequence);

  return offset;
}


static const per_sequence_t DL_AddReconfTransChInfoList_r5_sequence_of[1] = {
  { &hf_rrc_DL_AddReconfTransChInfoList_r5_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DL_AddReconfTransChInformation_r5 },
};

static int
dissect_rrc_DL_AddReconfTransChInfoList_r5(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_DL_AddReconfTransChInfoList_r5, DL_AddReconfTransChInfoList_r5_sequence_of,
                                                  1, maxTrCHpreconf, FALSE);

  return offset;
}



static int
dissect_rrc_ACK_NACK_repetitionFactor(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            1U, 4U, NULL, FALSE);

  return offset;
}


static const per_sequence_t T_fdd_123_sequence[] = {
  { &hf_rrc_dpcch_PowerOffset, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DPCCH_PowerOffset },
  { &hf_rrc_pc_Preamble     , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PC_Preamble },
  { &hf_rrc_sRB_delay       , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SRB_delay },
  { &hf_rrc_powerControlAlgorithm, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PowerControlAlgorithm },
  { &hf_rrc_deltaACK        , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DeltaACK },
  { &hf_rrc_deltaNACK       , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DeltaNACK },
  { &hf_rrc_ack_NACK_repetition_factor, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_ACK_NACK_repetitionFactor },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_fdd_123(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_fdd_123, T_fdd_123_sequence);

  return offset;
}


static const per_sequence_t T_tdd384_37_sequence[] = {
  { &hf_rrc_individualTS_InterferenceList, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_IndividualTS_InterferenceList },
  { &hf_rrc_dpch_ConstantValue_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_ConstantValue },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_tdd384_37(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_tdd384_37, T_tdd384_37_sequence);

  return offset;
}


static const per_sequence_t T_tdd128_56_sequence[] = {
  { &hf_rrc_tpc_StepSize    , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TPC_StepSizeTDD },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_tdd128_56(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_tdd128_56, T_tdd128_56_sequence);

  return offset;
}


static const value_string rrc_T_tddOption_22_vals[] = {
  {   0, "tdd384" },
  {   1, "tdd128" },
  { 0, NULL }
};

static const per_choice_t T_tddOption_22_choice[] = {
  {   0, &hf_rrc_tdd384_41       , ASN1_NO_EXTENSIONS     , dissect_rrc_T_tdd384_37 },
  {   1, &hf_rrc_tdd128_58       , ASN1_NO_EXTENSIONS     , dissect_rrc_T_tdd128_56 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_tddOption_22(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_tddOption_22, T_tddOption_22_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t T_individuallySignalled_02_sequence[] = {
  { &hf_rrc_tddOption_22    , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_tddOption_22 },
  { &hf_rrc_primaryCCPCH_TX_Power, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PrimaryCCPCH_TX_Power },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_individuallySignalled_02(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_individuallySignalled_02, T_individuallySignalled_02_sequence);

  return offset;
}


static const value_string rrc_T_ul_OL_PC_Signalling_02_vals[] = {
  {   0, "broadcast-UL-OL-PC-info" },
  {   1, "individuallySignalled" },
  { 0, NULL }
};

static const per_choice_t T_ul_OL_PC_Signalling_02_choice[] = {
  {   0, &hf_rrc_broadcast_UL_OL_PC_info, ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   1, &hf_rrc_individuallySignalled_02, ASN1_NO_EXTENSIONS     , dissect_rrc_T_individuallySignalled_02 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_ul_OL_PC_Signalling_02(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_ul_OL_PC_Signalling_02, T_ul_OL_PC_Signalling_02_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t T_tdd_73_sequence[] = {
  { &hf_rrc_ul_TargetSIR    , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_TargetSIR },
  { &hf_rrc_ul_OL_PC_Signalling_02, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_ul_OL_PC_Signalling_02 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_tdd_73(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_tdd_73, T_tdd_73_sequence);

  return offset;
}


static const value_string rrc_UL_DPCH_PowerControlInfo_r5_vals[] = {
  {   0, "fdd" },
  {   1, "tdd" },
  { 0, NULL }
};

static const per_choice_t UL_DPCH_PowerControlInfo_r5_choice[] = {
  {   0, &hf_rrc_fdd_126         , ASN1_NO_EXTENSIONS     , dissect_rrc_T_fdd_123 },
  {   1, &hf_rrc_tdd_79          , ASN1_NO_EXTENSIONS     , dissect_rrc_T_tdd_73 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_UL_DPCH_PowerControlInfo_r5(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_UL_DPCH_PowerControlInfo_r5, UL_DPCH_PowerControlInfo_r5_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t T_fdd_115_sequence[] = {
  { &hf_rrc_scramblingCodeType, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_ScramblingCodeType },
  { &hf_rrc_scramblingCode  , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UL_ScramblingCode },
  { &hf_rrc_numberOfDPDCH   , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_NumberOfDPDCH },
  { &hf_rrc_spreadingFactor_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SpreadingFactor },
  { &hf_rrc_tfci_Existence  , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { &hf_rrc_numberOfFBI_Bits, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_NumberOfFBI_Bits },
  { &hf_rrc_puncturingLimit , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PuncturingLimit },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_fdd_115(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_fdd_115, T_fdd_115_sequence);

  return offset;
}


static const per_sequence_t T_tdd_66_sequence[] = {
  { &hf_rrc_ul_TimingAdvance_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_TimingAdvanceControl_r4 },
  { &hf_rrc_ul_CCTrCHList_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_CCTrCHList_r4 },
  { &hf_rrc_ul_CCTrCHListToRemove, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_CCTrCHListToRemove },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_tdd_66(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_tdd_66, T_tdd_66_sequence);

  return offset;
}


static const value_string rrc_T_modeSpecificInfo_83_vals[] = {
  {   0, "fdd" },
  {   1, "tdd" },
  { 0, NULL }
};

static const per_choice_t T_modeSpecificInfo_83_choice[] = {
  {   0, &hf_rrc_fdd_118         , ASN1_NO_EXTENSIONS     , dissect_rrc_T_fdd_115 },
  {   1, &hf_rrc_tdd_72          , ASN1_NO_EXTENSIONS     , dissect_rrc_T_tdd_66 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_modeSpecificInfo_83(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_modeSpecificInfo_83, T_modeSpecificInfo_83_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t UL_DPCH_Info_r5_sequence[] = {
  { &hf_rrc_ul_DPCH_PowerControlInfo_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_DPCH_PowerControlInfo_r5 },
  { &hf_rrc_modeSpecificInfo_83, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_modeSpecificInfo_83 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UL_DPCH_Info_r5(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UL_DPCH_Info_r5, UL_DPCH_Info_r5_sequence);

  return offset;
}


static const value_string rrc_UL_ChannelRequirement_r5_vals[] = {
  {   0, "ul-DPCH-Info" },
  {   1, "dummy" },
  { 0, NULL }
};

static const per_choice_t UL_ChannelRequirement_r5_choice[] = {
  {   0, &hf_rrc_ul_DPCH_Info_08 , ASN1_NO_EXTENSIONS     , dissect_rrc_UL_DPCH_Info_r5 },
  {   1, &hf_rrc_dummy_47        , ASN1_NO_EXTENSIONS     , dissect_rrc_CPCH_SetInfo },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_UL_ChannelRequirement_r5(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_UL_ChannelRequirement_r5, UL_ChannelRequirement_r5_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t T_fdd_05_sequence[] = {
  { &hf_rrc_dummy_06        , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_PDSCH_Information },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_fdd_05(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_fdd_05, T_fdd_05_sequence);

  return offset;
}


static const value_string rrc_T_modeSpecificPhysChInfo_02_vals[] = {
  {   0, "fdd" },
  {   1, "tdd" },
  { 0, NULL }
};

static const per_choice_t T_modeSpecificPhysChInfo_02_choice[] = {
  {   0, &hf_rrc_fdd_05          , ASN1_NO_EXTENSIONS     , dissect_rrc_T_fdd_05 },
  {   1, &hf_rrc_tdd             , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_modeSpecificPhysChInfo_02(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_modeSpecificPhysChInfo_02, T_modeSpecificPhysChInfo_02_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t T_fdd_60_sequence[] = {
  { &hf_rrc_defaultDPCH_OffsetValue, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DefaultDPCH_OffsetValueFDD },
  { &hf_rrc_dpch_CompressedModeInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DPCH_CompressedModeInfo },
  { &hf_rrc_tx_DiversityMode, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_TX_DiversityMode },
  { &hf_rrc_dummy_39        , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_SSDT_Information_r4 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_fdd_60(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_fdd_60, T_fdd_60_sequence);

  return offset;
}


static const per_sequence_t T_tdd128_15_sequence[] = {
  { &hf_rrc_tstd_Indicator  , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_tdd128_15(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_tdd128_15, T_tdd128_15_sequence);

  return offset;
}


static const value_string rrc_T_tddOption_06_vals[] = {
  {   0, "tdd384" },
  {   1, "tdd128" },
  { 0, NULL }
};

static const per_choice_t T_tddOption_06_choice[] = {
  {   0, &hf_rrc_tdd384_11       , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   1, &hf_rrc_tdd128_15       , ASN1_NO_EXTENSIONS     , dissect_rrc_T_tdd128_15 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_tddOption_06(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_tddOption_06, T_tddOption_06_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t T_tdd_22_sequence[] = {
  { &hf_rrc_tddOption_06    , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_tddOption_06 },
  { &hf_rrc_defaultDPCH_OffsetValue_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DefaultDPCH_OffsetValueTDD },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_tdd_22(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_tdd_22, T_tdd_22_sequence);

  return offset;
}


static const value_string rrc_T_modeSpecificInfo_30_vals[] = {
  {   0, "fdd" },
  {   1, "tdd" },
  { 0, NULL }
};

static const per_choice_t T_modeSpecificInfo_30_choice[] = {
  {   0, &hf_rrc_fdd_61          , ASN1_NO_EXTENSIONS     , dissect_rrc_T_fdd_60 },
  {   1, &hf_rrc_tdd_23          , ASN1_NO_EXTENSIONS     , dissect_rrc_T_tdd_22 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_modeSpecificInfo_30(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_modeSpecificInfo_30, T_modeSpecificInfo_30_choice,
                                 NULL);

  return offset;
}


static const value_string rrc_T_mac_hsResetIndicator_05_vals[] = {
  {   0, "true" },
  { 0, NULL }
};


static int
dissect_rrc_T_mac_hsResetIndicator_05(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t DL_CommonInformation_r5_sequence[] = {
  { &hf_rrc_dl_DPCH_InfoCommon_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_DPCH_InfoCommon_r4 },
  { &hf_rrc_modeSpecificInfo_30, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_modeSpecificInfo_30 },
  { &hf_rrc_mac_hsResetIndicator_05, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_mac_hsResetIndicator_05 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_DL_CommonInformation_r5(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_DL_CommonInformation_r5, DL_CommonInformation_r5_sequence);

  return offset;
}


static const per_sequence_t T_fdd_82_sequence[] = {
  { &hf_rrc_primaryCPICH_Info, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PrimaryCPICH_Info },
  { &hf_rrc_dummy1_02       , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_PDSCH_SHO_DCH_Info },
  { &hf_rrc_dummy2_07       , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_PDSCH_CodeMapping },
  { &hf_rrc_servingHSDSCH_RL_indicator, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_fdd_82(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_fdd_82, T_fdd_82_sequence);

  return offset;
}


static const value_string rrc_T_modeSpecificInfo_50_vals[] = {
  {   0, "fdd" },
  {   1, "tdd" },
  { 0, NULL }
};

static const per_choice_t T_modeSpecificInfo_50_choice[] = {
  {   0, &hf_rrc_fdd_83          , ASN1_NO_EXTENSIONS     , dissect_rrc_T_fdd_82 },
  {   1, &hf_rrc_tdd_47          , ASN1_NO_EXTENSIONS     , dissect_rrc_PrimaryCCPCH_Info_r4 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_modeSpecificInfo_50(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_modeSpecificInfo_50, T_modeSpecificInfo_50_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t DL_InformationPerRL_r5_sequence[] = {
  { &hf_rrc_modeSpecificInfo_50, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_modeSpecificInfo_50 },
  { &hf_rrc_dl_DPCH_InfoPerRL_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_DPCH_InfoPerRL_r5 },
  { &hf_rrc_dummy_43        , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_SCCPCH_InfoForFACH_r4 },
  { &hf_rrc_cell_id         , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CellIdentity },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_DL_InformationPerRL_r5(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_DL_InformationPerRL_r5, DL_InformationPerRL_r5_sequence);

  return offset;
}


static const per_sequence_t DL_InformationPerRL_List_r5_sequence_of[1] = {
  { &hf_rrc_DL_InformationPerRL_List_r5_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DL_InformationPerRL_r5 },
};

static int
dissect_rrc_DL_InformationPerRL_List_r5(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_DL_InformationPerRL_List_r5, DL_InformationPerRL_List_r5_sequence_of,
                                                  1, maxRL, FALSE);

  return offset;
}


static const per_sequence_t CellUpdateConfirm_r5_IEs_sequence[] = {
  { &hf_rrc_integrityProtectionModeInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_IntegrityProtectionModeInfo },
  { &hf_rrc_cipheringModeInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CipheringModeInfo },
  { &hf_rrc_activationTime  , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_ActivationTime },
  { &hf_rrc_new_U_RNTI      , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_U_RNTI },
  { &hf_rrc_new_C_RNTI      , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_C_RNTI },
  { &hf_rrc_new_DSCH_RNTI   , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DSCH_RNTI },
  { &hf_rrc_new_H_RNTI      , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_H_RNTI },
  { &hf_rrc_rrc_StateIndicator, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RRC_StateIndicator },
  { &hf_rrc_utran_DRX_CycleLengthCoeff, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UTRAN_DRX_CycleLengthCoefficient },
  { &hf_rrc_rlc_Re_establishIndicatorRb2_3or4, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { &hf_rrc_rlc_Re_establishIndicatorRb5orAbove, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { &hf_rrc_cn_InformationInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CN_InformationInfo },
  { &hf_rrc_ura_Identity    , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_URA_Identity },
  { &hf_rrc_rb_InformationReleaseList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RB_InformationReleaseList },
  { &hf_rrc_rb_InformationReconfigList_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RB_InformationReconfigList_r5 },
  { &hf_rrc_rb_InformationAffectedList_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RB_InformationAffectedList_r5 },
  { &hf_rrc_dl_CounterSynchronisationInfo_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_CounterSynchronisationInfo_r5 },
  { &hf_rrc_ul_CommonTransChInfo_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_CommonTransChInfo_r4 },
  { &hf_rrc_ul_deletedTransChInfoList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_DeletedTransChInfoList },
  { &hf_rrc_ul_AddReconfTransChInfoList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_AddReconfTransChInfoList },
  { &hf_rrc_modeSpecificTransChInfo_02, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_modeSpecificTransChInfo_02 },
  { &hf_rrc_dl_CommonTransChInfo_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_CommonTransChInfo_r4 },
  { &hf_rrc_dl_DeletedTransChInfoList_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_DeletedTransChInfoList_r5 },
  { &hf_rrc_dl_AddReconfTransChInfoList_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_AddReconfTransChInfoList_r5 },
  { &hf_rrc_frequencyInfo   , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_FrequencyInfo },
  { &hf_rrc_maxAllowedUL_TX_Power, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MaxAllowedUL_TX_Power },
  { &hf_rrc_ul_ChannelRequirement_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_ChannelRequirement_r5 },
  { &hf_rrc_modeSpecificPhysChInfo_02, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_modeSpecificPhysChInfo_02 },
  { &hf_rrc_dl_HSPDSCH_Information, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_HSPDSCH_Information },
  { &hf_rrc_dl_CommonInformation_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_CommonInformation_r5 },
  { &hf_rrc_dl_InformationPerRL_List_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_InformationPerRL_List_r5 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_CellUpdateConfirm_r5_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_CellUpdateConfirm_r5_IEs, CellUpdateConfirm_r5_IEs_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions_16_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_nonCriticalExtensions_16(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_nonCriticalExtensions_16, T_nonCriticalExtensions_16_sequence);

  return offset;
}


static const per_sequence_t T_v690NonCriticalExtensions_04_sequence[] = {
  { &hf_rrc_cellUpdateConfirm_v690ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CellUpdateConfirm_v690ext_IEs },
  { &hf_rrc_nonCriticalExtensions_16, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_nonCriticalExtensions_16 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v690NonCriticalExtensions_04(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v690NonCriticalExtensions_04, T_v690NonCriticalExtensions_04_sequence);

  return offset;
}


static const per_sequence_t T_v5d0NonCriticalExtenstions_02_sequence[] = {
  { &hf_rrc_cellUpdateConfirm_v5d0ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CellUpdateConfirm_v5d0ext_IEs },
  { &hf_rrc_v690NonCriticalExtensions_04, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v690NonCriticalExtensions_04 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v5d0NonCriticalExtenstions_02(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v5d0NonCriticalExtenstions_02, T_v5d0NonCriticalExtenstions_02_sequence);

  return offset;
}


static const per_sequence_t T_r5_sequence[] = {
  { &hf_rrc_cellUpdateConfirm_r5, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CellUpdateConfirm_r5_IEs },
  { &hf_rrc_cellUpdateConfirm_r5_add_ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_BIT_STRING },
  { &hf_rrc_v5d0NonCriticalExtenstions_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v5d0NonCriticalExtenstions_02 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_r5(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_r5, T_r5_sequence);

  return offset;
}


static const value_string rrc_DL_Reception_Window_Size_r6_vals[] = {
  {   0, "size32" },
  {   1, "size48" },
  {   2, "size64" },
  {   3, "size80" },
  {   4, "size96" },
  {   5, "size112" },
  { 0, NULL }
};


static int
dissect_rrc_DL_Reception_Window_Size_r6(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     6, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t DL_UM_RLC_Mode_r6_sequence[] = {
  { &hf_rrc_dl_UM_RLC_LI_size, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DL_UM_RLC_LI_size },
  { &hf_rrc_dl_Reception_Window_Size, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_Reception_Window_Size_r6 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_DL_UM_RLC_Mode_r6(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_DL_UM_RLC_Mode_r6, DL_UM_RLC_Mode_r6_sequence);

  return offset;
}


static const value_string rrc_DL_RLC_Mode_r6_vals[] = {
  {   0, "dl-AM-RLC-Mode" },
  {   1, "dl-UM-RLC-Mode" },
  {   2, "dl-TM-RLC-Mode" },
  { 0, NULL }
};

static const per_choice_t DL_RLC_Mode_r6_choice[] = {
  {   0, &hf_rrc_dl_AM_RLC_Mode_01, ASN1_NO_EXTENSIONS     , dissect_rrc_DL_AM_RLC_Mode_r5 },
  {   1, &hf_rrc_dl_UM_RLC_Mode_02, ASN1_NO_EXTENSIONS     , dissect_rrc_DL_UM_RLC_Mode_r6 },
  {   2, &hf_rrc_dl_TM_RLC_Mode  , ASN1_NO_EXTENSIONS     , dissect_rrc_DL_TM_RLC_Mode },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_DL_RLC_Mode_r6(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_DL_RLC_Mode_r6, DL_RLC_Mode_r6_choice,
                                 NULL);

  return offset;
}


static const value_string rrc_T_altE_bitInterpretation_vals[] = {
  {   0, "true" },
  { 0, NULL }
};


static int
dissect_rrc_T_altE_bitInterpretation(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t RLC_Info_r6_sequence[] = {
  { &hf_rrc_ul_RLC_Mode     , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_RLC_Mode },
  { &hf_rrc_dl_RLC_Mode_02  , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_RLC_Mode_r6 },
  { &hf_rrc_rlc_OneSidedReEst, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { &hf_rrc_altE_bitInterpretation, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_altE_bitInterpretation },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_RLC_Info_r6(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_RLC_Info_r6, RLC_Info_r6_sequence);

  return offset;
}


static const value_string rrc_T_rlc_SizeList_01_vals[] = {
  {   0, "allSizes" },
  {   1, "configured" },
  {   2, "explicitList" },
  { 0, NULL }
};

static const per_choice_t T_rlc_SizeList_01_choice[] = {
  {   0, &hf_rrc_allSizes        , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   1, &hf_rrc_configured      , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   2, &hf_rrc_explicitList    , ASN1_NO_EXTENSIONS     , dissect_rrc_RLC_SizeExplicitList },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_rlc_SizeList_01(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_rlc_SizeList_01, T_rlc_SizeList_01_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t T_dch_rach_usch_sequence[] = {
  { &hf_rrc_ul_TransportChannelType, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UL_TransportChannelType },
  { &hf_rrc_logicalChannelIdentity, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_LogicalChannelIdentity },
  { &hf_rrc_rlc_SizeList_01 , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_rlc_SizeList_01 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_dch_rach_usch(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_dch_rach_usch, T_dch_rach_usch_sequence);

  return offset;
}



static int
dissect_rrc_E_DCH_MAC_d_FlowIdentity(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            0U, maxE_DCHMACdFlow_1, NULL, FALSE);

  return offset;
}



static int
dissect_rrc_DDI(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            0U, 62U, NULL, FALSE);

  return offset;
}



static int
dissect_rrc_RLC_PDU_Size(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_rrc_OctetModeRLC_SizeInfoType1(tvb, offset, actx, tree, hf_index);

  return offset;
}


static const per_sequence_t RLC_PDU_SizeList_sequence_of[1] = {
  { &hf_rrc_RLC_PDU_SizeList_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RLC_PDU_Size },
};

static int
dissect_rrc_RLC_PDU_SizeList(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_RLC_PDU_SizeList, RLC_PDU_SizeList_sequence_of,
                                                  1, maxRLCPDUsizePerLogChan, FALSE);

  return offset;
}


static const per_sequence_t T_e_dch_sequence[] = {
  { &hf_rrc_logicalChannelIdentity, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_LogicalChannelIdentity },
  { &hf_rrc_e_DCH_MAC_d_FlowIdentity, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_E_DCH_MAC_d_FlowIdentity },
  { &hf_rrc_ddi             , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DDI },
  { &hf_rrc_rlc_PDU_SizeList, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RLC_PDU_SizeList },
  { &hf_rrc_includeInSchedulingInfo, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_e_dch(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_e_dch, T_e_dch_sequence);

  return offset;
}


static const value_string rrc_T_ul_TrCH_Type_vals[] = {
  {   0, "dch-rach-usch" },
  {   1, "e-dch" },
  { 0, NULL }
};

static const per_choice_t T_ul_TrCH_Type_choice[] = {
  {   0, &hf_rrc_dch_rach_usch   , ASN1_NO_EXTENSIONS     , dissect_rrc_T_dch_rach_usch },
  {   1, &hf_rrc_e_dch           , ASN1_NO_EXTENSIONS     , dissect_rrc_T_e_dch },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_ul_TrCH_Type(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_ul_TrCH_Type, T_ul_TrCH_Type_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t UL_LogicalChannelMapping_r6_sequence[] = {
  { &hf_rrc_ul_TrCH_Type    , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_ul_TrCH_Type },
  { &hf_rrc_mac_LogicalChannelPriority, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MAC_LogicalChannelPriority },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UL_LogicalChannelMapping_r6(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UL_LogicalChannelMapping_r6, UL_LogicalChannelMapping_r6_sequence);

  return offset;
}


static const per_sequence_t SEQUENCE_SIZE_maxLoCHperRLC_OF_UL_LogicalChannelMapping_r6_sequence_of[1] = {
  { &hf_rrc_ul_LogicalChannelMapping_item_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UL_LogicalChannelMapping_r6 },
};

static int
dissect_rrc_SEQUENCE_SIZE_maxLoCHperRLC_OF_UL_LogicalChannelMapping_r6(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_SEQUENCE_SIZE_maxLoCHperRLC_OF_UL_LogicalChannelMapping_r6, SEQUENCE_SIZE_maxLoCHperRLC_OF_UL_LogicalChannelMapping_r6_sequence_of,
                                                  maxLoCHperRLC, maxLoCHperRLC, FALSE);

  return offset;
}


static const per_sequence_t UL_LogicalChannelMappingList_r6_sequence[] = {
  { &hf_rrc_rlc_LogicalChannelMappingIndicator, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { &hf_rrc_ul_LogicalChannelMapping_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SEQUENCE_SIZE_maxLoCHperRLC_OF_UL_LogicalChannelMapping_r6 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UL_LogicalChannelMappingList_r6(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UL_LogicalChannelMappingList_r6, UL_LogicalChannelMappingList_r6_sequence);

  return offset;
}


static const value_string rrc_UL_LogicalChannelMappings_r6_vals[] = {
  {   0, "oneLogicalChannel" },
  {   1, "twoLogicalChannels" },
  { 0, NULL }
};

static const per_choice_t UL_LogicalChannelMappings_r6_choice[] = {
  {   0, &hf_rrc_oneLogicalChannel_01, ASN1_NO_EXTENSIONS     , dissect_rrc_UL_LogicalChannelMapping_r6 },
  {   1, &hf_rrc_twoLogicalChannels_01, ASN1_NO_EXTENSIONS     , dissect_rrc_UL_LogicalChannelMappingList_r6 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_UL_LogicalChannelMappings_r6(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_UL_LogicalChannelMappings_r6, UL_LogicalChannelMappings_r6_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t RB_MappingOption_r6_sequence[] = {
  { &hf_rrc_ul_LogicalChannelMappings_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_LogicalChannelMappings_r6 },
  { &hf_rrc_dl_LogicalChannelMappingList_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_LogicalChannelMappingList_r5 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_RB_MappingOption_r6(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_RB_MappingOption_r6, RB_MappingOption_r6_sequence);

  return offset;
}


static const per_sequence_t RB_MappingInfo_r6_sequence_of[1] = {
  { &hf_rrc_RB_MappingInfo_r6_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RB_MappingOption_r6 },
};

static int
dissect_rrc_RB_MappingInfo_r6(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_RB_MappingInfo_r6, RB_MappingInfo_r6_sequence_of,
                                                  1, maxRBMuxOptions, FALSE);

  return offset;
}


static const per_sequence_t RB_InformationReconfig_r6_sequence[] = {
  { &hf_rrc_rb_Identity     , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RB_Identity },
  { &hf_rrc_pdcp_Info_03    , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_PDCP_InfoReconfig_r4 },
  { &hf_rrc_pdcp_SN_Info    , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_PDCP_SN_Info },
  { &hf_rrc_rlc_Info_02     , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RLC_Info_r6 },
  { &hf_rrc_rb_MappingInfo_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RB_MappingInfo_r6 },
  { &hf_rrc_rb_StopContinue , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RB_StopContinue },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_RB_InformationReconfig_r6(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_RB_InformationReconfig_r6, RB_InformationReconfig_r6_sequence);

  return offset;
}


static const per_sequence_t RB_InformationReconfigList_r6_sequence_of[1] = {
  { &hf_rrc_RB_InformationReconfigList_r6_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RB_InformationReconfig_r6 },
};

static int
dissect_rrc_RB_InformationReconfigList_r6(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_RB_InformationReconfigList_r6, RB_InformationReconfigList_r6_sequence_of,
                                                  1, maxRB, FALSE);

  return offset;
}


static const per_sequence_t RB_InformationAffected_r6_sequence[] = {
  { &hf_rrc_rb_Identity     , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RB_Identity },
  { &hf_rrc_rb_MappingInfo_02, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RB_MappingInfo_r6 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_RB_InformationAffected_r6(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_RB_InformationAffected_r6, RB_InformationAffected_r6_sequence);

  return offset;
}


static const per_sequence_t RB_InformationAffectedList_r6_sequence_of[1] = {
  { &hf_rrc_RB_InformationAffectedList_r6_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RB_InformationAffected_r6 },
};

static int
dissect_rrc_RB_InformationAffectedList_r6(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_RB_InformationAffectedList_r6, RB_InformationAffectedList_r6_sequence_of,
                                                  1, maxRB, FALSE);

  return offset;
}


static const per_sequence_t T_dch_usch_03_sequence[] = {
  { &hf_rrc_ul_TransportChannelType_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UL_TrCH_Type },
  { &hf_rrc_ul_TransportChannelIdentity, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TransportChannelIdentity },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_dch_usch_03(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_dch_usch_03, T_dch_usch_03_sequence);

  return offset;
}


static const value_string rrc_UL_TransportChannelIdentity_r6_vals[] = {
  {   0, "dch-usch" },
  {   1, "e-dch" },
  { 0, NULL }
};

static const per_choice_t UL_TransportChannelIdentity_r6_choice[] = {
  {   0, &hf_rrc_dch_usch_03     , ASN1_NO_EXTENSIONS     , dissect_rrc_T_dch_usch_03 },
  {   1, &hf_rrc_e_dch_05        , ASN1_NO_EXTENSIONS     , dissect_rrc_E_DCH_MAC_d_FlowIdentity },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_UL_TransportChannelIdentity_r6(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_UL_TransportChannelIdentity_r6, UL_TransportChannelIdentity_r6_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t UL_DeletedTransChInfoList_r6_sequence_of[1] = {
  { &hf_rrc_UL_DeletedTransChInfoList_r6_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UL_TransportChannelIdentity_r6 },
};

static int
dissect_rrc_UL_DeletedTransChInfoList_r6(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_UL_DeletedTransChInfoList_r6, UL_DeletedTransChInfoList_r6_sequence_of,
                                                  1, maxTrCH, FALSE);

  return offset;
}


static const per_sequence_t T_dch_usch_sequence[] = {
  { &hf_rrc_ul_TransportChannelType_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UL_TrCH_Type },
  { &hf_rrc_transportChannelIdentity, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TransportChannelIdentity },
  { &hf_rrc_transportFormatSet, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TransportFormatSet },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_dch_usch(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_dch_usch, T_dch_usch_sequence);

  return offset;
}


static const value_string rrc_E_DCH_TTI_vals[] = {
  {   0, "tti2" },
  {   1, "tti10" },
  { 0, NULL }
};


static int
dissect_rrc_E_DCH_TTI(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     2, NULL, FALSE, 0, NULL);

  return offset;
}


static const value_string rrc_T_harq_Info_vals[] = {
  {   0, "rv0" },
  {   1, "rvtable" },
  { 0, NULL }
};


static int
dissect_rrc_T_harq_Info(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     2, NULL, FALSE, 0, NULL);

  return offset;
}



static int
dissect_rrc_E_DCH_MAC_d_FlowPowerOffset(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            0U, 6U, NULL, FALSE);

  return offset;
}



static int
dissect_rrc_E_DCH_MAC_d_FlowMaxRetrans(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            0U, 15U, NULL, FALSE);

  return offset;
}



static int
dissect_rrc_E_DCH_MAC_d_FlowMultiplexingList(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_bit_string(tvb, offset, actx, tree, hf_index,
                                     maxE_DCHMACdFlow, maxE_DCHMACdFlow, FALSE, NULL);

  return offset;
}



static int
dissect_rrc_INTEGER_1_19982(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            1U, 19982U, NULL, FALSE);

  return offset;
}


static const per_sequence_t T_non_ScheduledTransGrantInfo_sequence[] = {
  { &hf_rrc_maxMAC_e_PDUContents, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_1_19982 },
  { &hf_rrc_ms2_NonSchedTransmGrantHARQAlloc, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_BIT_STRING_SIZE_8 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_non_ScheduledTransGrantInfo(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_non_ScheduledTransGrantInfo, T_non_ScheduledTransGrantInfo_sequence);

  return offset;
}


static const value_string rrc_T_transmissionGrantType_vals[] = {
  {   0, "non-ScheduledTransGrantInfo" },
  {   1, "scheduledTransmissionGrantInfo" },
  { 0, NULL }
};

static const per_choice_t T_transmissionGrantType_choice[] = {
  {   0, &hf_rrc_non_ScheduledTransGrantInfo, ASN1_NO_EXTENSIONS     , dissect_rrc_T_non_ScheduledTransGrantInfo },
  {   1, &hf_rrc_scheduledTransmissionGrantInfo, ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_transmissionGrantType(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_transmissionGrantType, T_transmissionGrantType_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t E_DCH_AddReconf_MAC_d_Flow_sequence[] = {
  { &hf_rrc_mac_d_FlowIdentity, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_E_DCH_MAC_d_FlowIdentity },
  { &hf_rrc_mac_d_FlowPowerOffset, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_E_DCH_MAC_d_FlowPowerOffset },
  { &hf_rrc_mac_d_FlowMaxRetrans, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_E_DCH_MAC_d_FlowMaxRetrans },
  { &hf_rrc_mac_d_FlowMultiplexingList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_E_DCH_MAC_d_FlowMultiplexingList },
  { &hf_rrc_transmissionGrantType, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_transmissionGrantType },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_E_DCH_AddReconf_MAC_d_Flow(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_E_DCH_AddReconf_MAC_d_Flow, E_DCH_AddReconf_MAC_d_Flow_sequence);

  return offset;
}


static const per_sequence_t E_DCH_AddReconf_MAC_d_FlowList_sequence_of[1] = {
  { &hf_rrc_E_DCH_AddReconf_MAC_d_FlowList_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_E_DCH_AddReconf_MAC_d_Flow },
};

static int
dissect_rrc_E_DCH_AddReconf_MAC_d_FlowList(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_E_DCH_AddReconf_MAC_d_FlowList, E_DCH_AddReconf_MAC_d_FlowList_sequence_of,
                                                  1, maxE_DCHMACdFlow, FALSE);

  return offset;
}


static const per_sequence_t T_e_dch_02_sequence[] = {
  { &hf_rrc_tti_03          , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_E_DCH_TTI },
  { &hf_rrc_harq_Info       , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_harq_Info },
  { &hf_rrc_addReconf_MAC_d_FlowList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_E_DCH_AddReconf_MAC_d_FlowList },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_e_dch_02(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_e_dch_02, T_e_dch_02_sequence);

  return offset;
}


static const value_string rrc_UL_AddReconfTransChInformation_r6_vals[] = {
  {   0, "dch-usch" },
  {   1, "e-dch" },
  { 0, NULL }
};

static const per_choice_t UL_AddReconfTransChInformation_r6_choice[] = {
  {   0, &hf_rrc_dch_usch        , ASN1_NO_EXTENSIONS     , dissect_rrc_T_dch_usch },
  {   1, &hf_rrc_e_dch_02        , ASN1_NO_EXTENSIONS     , dissect_rrc_T_e_dch_02 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_UL_AddReconfTransChInformation_r6(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_UL_AddReconfTransChInformation_r6, UL_AddReconfTransChInformation_r6_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t UL_AddReconfTransChInfoList_r6_sequence_of[1] = {
  { &hf_rrc_UL_AddReconfTransChInfoList_r6_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UL_AddReconfTransChInformation_r6 },
};

static int
dissect_rrc_UL_AddReconfTransChInfoList_r6(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_UL_AddReconfTransChInfoList_r6, UL_AddReconfTransChInfoList_r6_sequence_of,
                                                  1, maxTrCH, FALSE);

  return offset;
}


static const per_sequence_t T_fdd_124_sequence[] = {
  { &hf_rrc_dpcch_PowerOffset, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DPCCH_PowerOffset },
  { &hf_rrc_pc_Preamble     , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PC_Preamble },
  { &hf_rrc_sRB_delay       , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SRB_delay },
  { &hf_rrc_powerControlAlgorithm, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PowerControlAlgorithm },
  { &hf_rrc_deltaACK        , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DeltaACK },
  { &hf_rrc_deltaNACK       , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DeltaNACK },
  { &hf_rrc_ack_NACK_repetition_factor, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_ACK_NACK_repetitionFactor },
  { &hf_rrc_harq_Preamble_Mode, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_HARQ_Preamble_Mode },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_fdd_124(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_fdd_124, T_fdd_124_sequence);

  return offset;
}


static const per_sequence_t T_tdd384_38_sequence[] = {
  { &hf_rrc_individualTS_InterferenceList, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_IndividualTS_InterferenceList },
  { &hf_rrc_dpch_ConstantValue_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_ConstantValue },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_tdd384_38(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_tdd384_38, T_tdd384_38_sequence);

  return offset;
}


static const per_sequence_t T_tdd128_57_sequence[] = {
  { &hf_rrc_beaconPLEst     , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_BEACON_PL_Est },
  { &hf_rrc_tpc_StepSize    , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TPC_StepSizeTDD },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_tdd128_57(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_tdd128_57, T_tdd128_57_sequence);

  return offset;
}


static const value_string rrc_T_tddOption_23_vals[] = {
  {   0, "tdd384" },
  {   1, "tdd128" },
  { 0, NULL }
};

static const per_choice_t T_tddOption_23_choice[] = {
  {   0, &hf_rrc_tdd384_42       , ASN1_NO_EXTENSIONS     , dissect_rrc_T_tdd384_38 },
  {   1, &hf_rrc_tdd128_59       , ASN1_NO_EXTENSIONS     , dissect_rrc_T_tdd128_57 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_tddOption_23(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_tddOption_23, T_tddOption_23_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t T_individuallySignalled_03_sequence[] = {
  { &hf_rrc_tddOption_23    , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_tddOption_23 },
  { &hf_rrc_primaryCCPCH_TX_Power, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PrimaryCCPCH_TX_Power },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_individuallySignalled_03(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_individuallySignalled_03, T_individuallySignalled_03_sequence);

  return offset;
}


static const value_string rrc_T_ul_OL_PC_Signalling_03_vals[] = {
  {   0, "broadcast-UL-OL-PC-info" },
  {   1, "individuallySignalled" },
  { 0, NULL }
};

static const per_choice_t T_ul_OL_PC_Signalling_03_choice[] = {
  {   0, &hf_rrc_broadcast_UL_OL_PC_info, ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   1, &hf_rrc_individuallySignalled_03, ASN1_NO_EXTENSIONS     , dissect_rrc_T_individuallySignalled_03 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_ul_OL_PC_Signalling_03(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_ul_OL_PC_Signalling_03, T_ul_OL_PC_Signalling_03_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t T_tdd_74_sequence[] = {
  { &hf_rrc_ul_TargetSIR    , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_TargetSIR },
  { &hf_rrc_ul_OL_PC_Signalling_03, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_ul_OL_PC_Signalling_03 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_tdd_74(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_tdd_74, T_tdd_74_sequence);

  return offset;
}


static const value_string rrc_UL_DPCH_PowerControlInfo_r6_vals[] = {
  {   0, "fdd" },
  {   1, "tdd" },
  { 0, NULL }
};

static const per_choice_t UL_DPCH_PowerControlInfo_r6_choice[] = {
  {   0, &hf_rrc_fdd_127         , ASN1_NO_EXTENSIONS     , dissect_rrc_T_fdd_124 },
  {   1, &hf_rrc_tdd_80          , ASN1_NO_EXTENSIONS     , dissect_rrc_T_tdd_74 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_UL_DPCH_PowerControlInfo_r6(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_UL_DPCH_PowerControlInfo_r6, UL_DPCH_PowerControlInfo_r6_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t T_present_02_sequence[] = {
  { &hf_rrc_numberOfDPDCH   , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_NumberOfDPDCH },
  { &hf_rrc_spreadingFactor_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SpreadingFactor },
  { &hf_rrc_tfci_Existence  , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { &hf_rrc_numberOfFBI_Bits, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_NumberOfFBI_Bits },
  { &hf_rrc_puncturingLimit , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PuncturingLimit },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_present_02(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_present_02, T_present_02_sequence);

  return offset;
}


static const per_sequence_t T_notPresent_sequence[] = {
  { &hf_rrc_tfci_Existence  , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { &hf_rrc_numberOfFBI_Bits, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_NumberOfFBI_Bits },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_notPresent(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_notPresent, T_notPresent_sequence);

  return offset;
}


static const value_string rrc_T_dpdchPresence_vals[] = {
  {   0, "present" },
  {   1, "notPresent" },
  { 0, NULL }
};

static const per_choice_t T_dpdchPresence_choice[] = {
  {   0, &hf_rrc_present_04      , ASN1_NO_EXTENSIONS     , dissect_rrc_T_present_02 },
  {   1, &hf_rrc_notPresent      , ASN1_NO_EXTENSIONS     , dissect_rrc_T_notPresent },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_dpdchPresence(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_dpdchPresence, T_dpdchPresence_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t T_fdd_116_sequence[] = {
  { &hf_rrc_scramblingCodeType, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_ScramblingCodeType },
  { &hf_rrc_scramblingCode  , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UL_ScramblingCode },
  { &hf_rrc_dpdchPresence   , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_dpdchPresence },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_fdd_116(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_fdd_116, T_fdd_116_sequence);

  return offset;
}


static const per_sequence_t T_tdd_67_sequence[] = {
  { &hf_rrc_ul_TimingAdvance_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_TimingAdvanceControl_r4 },
  { &hf_rrc_ul_CCTrCHList_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_CCTrCHList_r4 },
  { &hf_rrc_ul_CCTrCHListToRemove, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_CCTrCHListToRemove },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_tdd_67(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_tdd_67, T_tdd_67_sequence);

  return offset;
}


static const value_string rrc_T_modeSpecificInfo_84_vals[] = {
  {   0, "fdd" },
  {   1, "tdd" },
  { 0, NULL }
};

static const per_choice_t T_modeSpecificInfo_84_choice[] = {
  {   0, &hf_rrc_fdd_119         , ASN1_NO_EXTENSIONS     , dissect_rrc_T_fdd_116 },
  {   1, &hf_rrc_tdd_73          , ASN1_NO_EXTENSIONS     , dissect_rrc_T_tdd_67 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_modeSpecificInfo_84(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_modeSpecificInfo_84, T_modeSpecificInfo_84_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t UL_DPCH_Info_r6_sequence[] = {
  { &hf_rrc_ul_DPCH_PowerControlInfo_04, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_DPCH_PowerControlInfo_r6 },
  { &hf_rrc_modeSpecificInfo_84, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_modeSpecificInfo_84 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UL_DPCH_Info_r6(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UL_DPCH_Info_r6, UL_DPCH_Info_r6_sequence);

  return offset;
}


static const value_string rrc_T_mac_es_e_resetIndicator_01_vals[] = {
  {   0, "true" },
  { 0, NULL }
};


static int
dissect_rrc_T_mac_es_e_resetIndicator_01(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}


static const value_string rrc_HappyBit_DelayCondition_vals[] = {
  {   0, "ms2" },
  {   1, "ms10" },
  {   2, "ms20" },
  {   3, "ms50" },
  {   4, "ms100" },
  {   5, "ms200" },
  {   6, "ms500" },
  {   7, "ms1000" },
  { 0, NULL }
};


static int
dissect_rrc_HappyBit_DelayCondition(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     8, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t E_DPCCH_Info_sequence[] = {
  { &hf_rrc_e_DPCCH_DPCCH_PowerOffset, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_E_DPCCH_DPCCH_PowerOffset },
  { &hf_rrc_happyBit_DelayCondition, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_HappyBit_DelayCondition },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_E_DPCCH_Info(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_E_DPCCH_Info, E_DPCCH_Info_sequence);

  return offset;
}


static const value_string rrc_E_DPDCH_MaxChannelisationCodes_vals[] = {
  {   0, "sf256" },
  {   1, "sf128" },
  {   2, "sf64" },
  {   3, "sf32" },
  {   4, "sf16" },
  {   5, "sf8" },
  {   6, "sf4" },
  {   7, "sf4x2" },
  {   8, "sf2x2" },
  {   9, "sf4x2-and-sf2x2" },
  { 0, NULL }
};


static int
dissect_rrc_E_DPDCH_MaxChannelisationCodes(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     10, NULL, FALSE, 0, NULL);

  return offset;
}



static int
dissect_rrc_E_DPDCH_PL_NonMax(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            11U, 25U, NULL, FALSE);

  return offset;
}


static const value_string rrc_E_DPDCH_PeriodicyOfSchedInfo_vals[] = {
  {   0, "everyEDCHTTI" },
  {   1, "ms4" },
  {   2, "ms10" },
  {   3, "ms20" },
  {   4, "ms50" },
  {   5, "ms100" },
  {   6, "ms200" },
  {   7, "ms500" },
  {   8, "ms1000" },
  { 0, NULL }
};


static int
dissect_rrc_E_DPDCH_PeriodicyOfSchedInfo(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     9, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t E_DPDCH_SchedulingInfoConfiguration_sequence[] = {
  { &hf_rrc_periodicityOfSchedInfo_NoGrant, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_E_DPDCH_PeriodicyOfSchedInfo },
  { &hf_rrc_periodicityOfSchedInfo_Grant, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_E_DPDCH_PeriodicyOfSchedInfo },
  { &hf_rrc_powerOffsetForSchedInfo, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_0_6 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_E_DPDCH_SchedulingInfoConfiguration(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_E_DPDCH_SchedulingInfoConfiguration, E_DPDCH_SchedulingInfoConfiguration_sequence);

  return offset;
}


static const per_sequence_t E_DPDCH_Info_sequence[] = {
  { &hf_rrc_e_TFCI_TableIndex, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_E_TFCI_TableIndex },
  { &hf_rrc_e_DCH_MinimumSet_E_TFCI, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_E_DCH_MinimumSet_E_TFCI },
  { &hf_rrc_reference_E_TFCIs, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_E_DPDCH_Reference_E_TFCIList },
  { &hf_rrc_maxChannelisationCodes, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_E_DPDCH_MaxChannelisationCodes },
  { &hf_rrc_pl_NonMax       , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_E_DPDCH_PL_NonMax },
  { &hf_rrc_schedulingInfoConfiguration, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_E_DPDCH_SchedulingInfoConfiguration },
  { &hf_rrc_threeIndexStepThreshold, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_INTEGER_0_37 },
  { &hf_rrc_twoIndexStepThreshold, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_INTEGER_0_37 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_E_DPDCH_Info(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_E_DPDCH_Info, E_DPDCH_Info_sequence);

  return offset;
}


static const value_string rrc_T_primary_Secondary_GrantSelector_04_vals[] = {
  {   0, "primary" },
  {   1, "secondary" },
  { 0, NULL }
};


static int
dissect_rrc_T_primary_Secondary_GrantSelector_04(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     2, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t T_servingGrant_04_sequence[] = {
  { &hf_rrc_value           , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_0_38 },
  { &hf_rrc_primary_Secondary_GrantSelector_04, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_primary_Secondary_GrantSelector_04 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_servingGrant_04(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_servingGrant_04, T_servingGrant_04_sequence);

  return offset;
}


static const per_sequence_t E_DPDCH_SchedulingTransmConfiguration_sequence[] = {
  { &hf_rrc_ms2_SchedTransmGrantHARQAlloc, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_BIT_STRING_SIZE_8 },
  { &hf_rrc_servingGrant_04 , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_servingGrant_04 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_E_DPDCH_SchedulingTransmConfiguration(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_E_DPDCH_SchedulingTransmConfiguration, E_DPDCH_SchedulingTransmConfiguration_sequence);

  return offset;
}


static const per_sequence_t UL_EDCH_Information_r6_sequence[] = {
  { &hf_rrc_mac_es_e_resetIndicator_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_mac_es_e_resetIndicator_01 },
  { &hf_rrc_e_DPCCH_Info    , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_E_DPCCH_Info },
  { &hf_rrc_e_DPDCH_Info    , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_E_DPDCH_Info },
  { &hf_rrc_schedulingTransmConfiguration, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_E_DPDCH_SchedulingTransmConfiguration },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UL_EDCH_Information_r6(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UL_EDCH_Information_r6, UL_EDCH_Information_r6_sequence);

  return offset;
}


static const per_sequence_t T_fdd_94_sequence[] = {
  { &hf_rrc_hS_SCCHChannelisationCodeInfo, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SEQUENCE_SIZE_1_maxHSSCCHs_OF_HS_SCCH_Codes },
  { &hf_rrc_dl_ScramblingCode, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_SecondaryScramblingCode },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_fdd_94(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_fdd_94, T_fdd_94_sequence);

  return offset;
}


static const per_sequence_t T_tdd384_24_sequence[] = {
  { &hf_rrc_nack_ack_power_offset, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_M7_8 },
  { &hf_rrc_hs_SICH_PowerControl_Info, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_HS_SICH_Power_Control_Info_TDD384 },
  { &hf_rrc_dhs_sync        , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DHS_Sync },
  { &hf_rrc_bler_target     , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_Bler_Target },
  { &hf_rrc_hS_SCCH_SetConfiguration_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SEQUENCE_SIZE_1_maxHSSCCHs_OF_HS_SCCH_TDD384_r6 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_tdd384_24(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_tdd384_24, T_tdd384_24_sequence);

  return offset;
}


static const value_string rrc_T_tpc_step_size_vals[] = {
  {   0, "s1" },
  {   1, "s2" },
  {   2, "s3" },
  {   3, "spare1" },
  { 0, NULL }
};


static int
dissect_rrc_T_tpc_step_size(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     4, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t T_tdd128_35_sequence[] = {
  { &hf_rrc_nack_ack_power_offset, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_M7_8 },
  { &hf_rrc_power_level_HSSICH, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_M120_M58 },
  { &hf_rrc_tpc_step_size   , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_tpc_step_size },
  { &hf_rrc_bler_target     , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_Bler_Target },
  { &hf_rrc_hS_SCCH_SetConfiguration_02, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SEQUENCE_SIZE_1_maxHSSCCHs_OF_HS_SCCH_TDD128_r6 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_tdd128_35(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_tdd128_35, T_tdd128_35_sequence);

  return offset;
}


static const value_string rrc_T_tdd_51_vals[] = {
  {   0, "tdd384" },
  {   1, "tdd128" },
  { 0, NULL }
};

static const per_choice_t T_tdd_51_choice[] = {
  {   0, &hf_rrc_tdd384_26       , ASN1_NO_EXTENSIONS     , dissect_rrc_T_tdd384_24 },
  {   1, &hf_rrc_tdd128_36       , ASN1_NO_EXTENSIONS     , dissect_rrc_T_tdd128_35 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_tdd_51(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_tdd_51, T_tdd_51_choice,
                                 NULL);

  return offset;
}


static const value_string rrc_T_modeSpecificInfo_63_vals[] = {
  {   0, "fdd" },
  {   1, "tdd" },
  { 0, NULL }
};

static const per_choice_t T_modeSpecificInfo_63_choice[] = {
  {   0, &hf_rrc_fdd_96          , ASN1_NO_EXTENSIONS     , dissect_rrc_T_fdd_94 },
  {   1, &hf_rrc_tdd_55          , ASN1_NO_EXTENSIONS     , dissect_rrc_T_tdd_51 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_modeSpecificInfo_63(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_modeSpecificInfo_63, T_modeSpecificInfo_63_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t HS_SCCH_Info_r6_sequence[] = {
  { &hf_rrc_modeSpecificInfo_63, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_modeSpecificInfo_63 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_HS_SCCH_Info_r6(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_HS_SCCH_Info_r6, HS_SCCH_Info_r6_sequence);

  return offset;
}


static const per_sequence_t T_tdd384_14_sequence[] = {
  { &hf_rrc_dl_HSPDSCH_TS_Configuration, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_HSPDSCH_TS_Configuration },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_tdd384_14(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_tdd384_14, T_tdd384_14_sequence);

  return offset;
}


static const per_sequence_t T_tdd128_22_sequence[] = {
  { &hf_rrc_hs_PDSCH_Midamble_Configuration_tdd128, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_HS_PDSCH_Midamble_Configuration_TDD128 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_tdd128_22(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_tdd128_22, T_tdd128_22_sequence);

  return offset;
}


static const value_string rrc_T_tdd_39_vals[] = {
  {   0, "tdd384" },
  {   1, "tdd128" },
  { 0, NULL }
};

static const per_choice_t T_tdd_39_choice[] = {
  {   0, &hf_rrc_tdd384_15       , ASN1_NO_EXTENSIONS     , dissect_rrc_T_tdd384_14 },
  {   1, &hf_rrc_tdd128_22       , ASN1_NO_EXTENSIONS     , dissect_rrc_T_tdd128_22 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_tdd_39(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_tdd_39, T_tdd_39_choice,
                                 NULL);

  return offset;
}


static const value_string rrc_T_modeSpecificInfo_42_vals[] = {
  {   0, "tdd" },
  {   1, "fdd" },
  { 0, NULL }
};

static const per_choice_t T_modeSpecificInfo_42_choice[] = {
  {   0, &hf_rrc_tdd_40          , ASN1_NO_EXTENSIONS     , dissect_rrc_T_tdd_39 },
  {   1, &hf_rrc_fdd_45          , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_modeSpecificInfo_42(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_modeSpecificInfo_42, T_modeSpecificInfo_42_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t DL_HSPDSCH_Information_r6_sequence[] = {
  { &hf_rrc_hs_scch_Info_01 , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_HS_SCCH_Info_r6 },
  { &hf_rrc_measurement_feedback_Info, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_Measurement_Feedback_Info },
  { &hf_rrc_modeSpecificInfo_42, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_modeSpecificInfo_42 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_DL_HSPDSCH_Information_r6(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_DL_HSPDSCH_Information_r6, DL_HSPDSCH_Information_r6_sequence);

  return offset;
}


static const value_string rrc_TimingMaintainedSynchInd_vals[] = {
  {   0, "false" },
  { 0, NULL }
};


static int
dissect_rrc_TimingMaintainedSynchInd(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t T_maintain_sequence[] = {
  { &hf_rrc_timingmaintainedsynchind, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_TimingMaintainedSynchInd },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_maintain(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_maintain, T_maintain_sequence);

  return offset;
}


static const value_string rrc_T_cfnHandling_02_vals[] = {
  {   0, "maintain" },
  {   1, "initialise" },
  { 0, NULL }
};

static const per_choice_t T_cfnHandling_02_choice[] = {
  {   0, &hf_rrc_maintain_01     , ASN1_NO_EXTENSIONS     , dissect_rrc_T_maintain },
  {   1, &hf_rrc_initialise_02   , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_cfnHandling_02(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_cfnHandling_02, T_cfnHandling_02_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t T_fdd_68_sequence[] = {
  { &hf_rrc_dl_DPCH_PowerControlInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_DPCH_PowerControlInfo },
  { &hf_rrc_powerOffsetPilot_pdpdch, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PowerOffsetPilot_pdpdch },
  { &hf_rrc_dl_rate_matching_restriction, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_Dl_rate_matching_restriction },
  { &hf_rrc_spreadingFactorAndPilot, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SF512_AndPilot },
  { &hf_rrc_positionFixedOrFlexible, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PositionFixedOrFlexible },
  { &hf_rrc_tfci_Existence  , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_fdd_68(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_fdd_68, T_fdd_68_sequence);

  return offset;
}


static const per_sequence_t T_tdd_30_sequence[] = {
  { &hf_rrc_dl_DPCH_PowerControlInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_DPCH_PowerControlInfo },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_tdd_30(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_tdd_30, T_tdd_30_sequence);

  return offset;
}


static const value_string rrc_T_modeSpecificInfo_38_vals[] = {
  {   0, "fdd" },
  {   1, "tdd" },
  { 0, NULL }
};

static const per_choice_t T_modeSpecificInfo_38_choice[] = {
  {   0, &hf_rrc_fdd_69          , ASN1_NO_EXTENSIONS     , dissect_rrc_T_fdd_68 },
  {   1, &hf_rrc_tdd_31          , ASN1_NO_EXTENSIONS     , dissect_rrc_T_tdd_30 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_modeSpecificInfo_38(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_modeSpecificInfo_38, T_modeSpecificInfo_38_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t DL_DPCH_InfoCommon_r6_sequence[] = {
  { &hf_rrc_cfnHandling_02  , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_cfnHandling_02 },
  { &hf_rrc_modeSpecificInfo_38, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_modeSpecificInfo_38 },
  { &hf_rrc_mac_d_HFN_initial_value, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MAC_d_HFN_initial_value },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_DL_DPCH_InfoCommon_r6(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_DL_DPCH_InfoCommon_r6, DL_DPCH_InfoCommon_r6_sequence);

  return offset;
}


static const per_sequence_t T_maintain_01_sequence[] = {
  { &hf_rrc_timingmaintainedsynchind, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_TimingMaintainedSynchInd },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_maintain_01(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_maintain_01, T_maintain_01_sequence);

  return offset;
}


static const value_string rrc_T_cfnHandling_03_vals[] = {
  {   0, "maintain" },
  {   1, "initialise" },
  { 0, NULL }
};

static const per_choice_t T_cfnHandling_03_choice[] = {
  {   0, &hf_rrc_maintain_02     , ASN1_NO_EXTENSIONS     , dissect_rrc_T_maintain_01 },
  {   1, &hf_rrc_initialise_02   , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_cfnHandling_03(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_cfnHandling_03, T_cfnHandling_03_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t DL_FDPCH_InfoCommon_r6_sequence[] = {
  { &hf_rrc_cfnHandling_03  , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_cfnHandling_03 },
  { &hf_rrc_dl_FDPCH_PowerControlInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_DPCH_PowerControlInfo },
  { &hf_rrc_dl_FDPCH_TPCcommandErrorRate_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_INTEGER_1_16 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_DL_FDPCH_InfoCommon_r6(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_DL_FDPCH_InfoCommon_r6, DL_FDPCH_InfoCommon_r6_sequence);

  return offset;
}


static const value_string rrc_T_dl_dpchInfoCommon_vals[] = {
  {   0, "dl-DPCH-InfoCommon" },
  {   1, "dl-FDPCH-InfoCommon" },
  { 0, NULL }
};

static const per_choice_t T_dl_dpchInfoCommon_choice[] = {
  {   0, &hf_rrc_dl_DPCH_InfoCommon_02, ASN1_NO_EXTENSIONS     , dissect_rrc_DL_DPCH_InfoCommon_r6 },
  {   1, &hf_rrc_dl_FDPCH_InfoCommon, ASN1_NO_EXTENSIONS     , dissect_rrc_DL_FDPCH_InfoCommon_r6 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_dl_dpchInfoCommon(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_dl_dpchInfoCommon, T_dl_dpchInfoCommon_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t T_fdd_61_sequence[] = {
  { &hf_rrc_defaultDPCH_OffsetValue, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DefaultDPCH_OffsetValueFDD },
  { &hf_rrc_dpch_CompressedModeInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DPCH_CompressedModeInfo },
  { &hf_rrc_tx_DiversityMode, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_TX_DiversityMode },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_fdd_61(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_fdd_61, T_fdd_61_sequence);

  return offset;
}


static const per_sequence_t T_tdd128_16_sequence[] = {
  { &hf_rrc_tstd_Indicator  , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_tdd128_16(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_tdd128_16, T_tdd128_16_sequence);

  return offset;
}


static const value_string rrc_T_tddOption_07_vals[] = {
  {   0, "tdd384" },
  {   1, "tdd128" },
  { 0, NULL }
};

static const per_choice_t T_tddOption_07_choice[] = {
  {   0, &hf_rrc_tdd384_11       , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   1, &hf_rrc_tdd128_16       , ASN1_NO_EXTENSIONS     , dissect_rrc_T_tdd128_16 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_tddOption_07(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_tddOption_07, T_tddOption_07_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t T_tdd_23_sequence[] = {
  { &hf_rrc_tddOption_07    , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_tddOption_07 },
  { &hf_rrc_defaultDPCH_OffsetValue_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DefaultDPCH_OffsetValueTDD },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_tdd_23(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_tdd_23, T_tdd_23_sequence);

  return offset;
}


static const value_string rrc_T_modeSpecificInfo_31_vals[] = {
  {   0, "fdd" },
  {   1, "tdd" },
  { 0, NULL }
};

static const per_choice_t T_modeSpecificInfo_31_choice[] = {
  {   0, &hf_rrc_fdd_62          , ASN1_NO_EXTENSIONS     , dissect_rrc_T_fdd_61 },
  {   1, &hf_rrc_tdd_24          , ASN1_NO_EXTENSIONS     , dissect_rrc_T_tdd_23 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_modeSpecificInfo_31(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_modeSpecificInfo_31, T_modeSpecificInfo_31_choice,
                                 NULL);

  return offset;
}


static const value_string rrc_T_mac_hsResetIndicator_06_vals[] = {
  {   0, "true" },
  { 0, NULL }
};


static int
dissect_rrc_T_mac_hsResetIndicator_06(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}


static const value_string rrc_T_postVerificationPeriod_07_vals[] = {
  {   0, "true" },
  { 0, NULL }
};


static int
dissect_rrc_T_postVerificationPeriod_07(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t DL_CommonInformation_r6_sequence[] = {
  { &hf_rrc_dl_dpchInfoCommon, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_dl_dpchInfoCommon },
  { &hf_rrc_modeSpecificInfo_31, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_modeSpecificInfo_31 },
  { &hf_rrc_mac_hsResetIndicator_06, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_mac_hsResetIndicator_06 },
  { &hf_rrc_postVerificationPeriod_07, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_postVerificationPeriod_07 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_DL_CommonInformation_r6(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_DL_CommonInformation_r6, DL_CommonInformation_r6_sequence);

  return offset;
}


static const per_sequence_t T_fdd_84_sequence[] = {
  { &hf_rrc_primaryCPICH_Info, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PrimaryCPICH_Info },
  { &hf_rrc_servingHSDSCH_RL_indicator, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { &hf_rrc_servingEDCH_RL_indicator, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_fdd_84(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_fdd_84, T_fdd_84_sequence);

  return offset;
}


static const value_string rrc_T_modeSpecificInfo_52_vals[] = {
  {   0, "fdd" },
  {   1, "tdd" },
  { 0, NULL }
};

static const per_choice_t T_modeSpecificInfo_52_choice[] = {
  {   0, &hf_rrc_fdd_85          , ASN1_NO_EXTENSIONS     , dissect_rrc_T_fdd_84 },
  {   1, &hf_rrc_tdd_47          , ASN1_NO_EXTENSIONS     , dissect_rrc_PrimaryCCPCH_Info_r4 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_modeSpecificInfo_52(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_modeSpecificInfo_52, T_modeSpecificInfo_52_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t T_fdd_73_sequence[] = {
  { &hf_rrc_pCPICH_UsageForChannelEst, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PCPICH_UsageForChannelEst },
  { &hf_rrc_dpch_FrameOffset, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DPCH_FrameOffset },
  { &hf_rrc_secondaryCPICH_Info, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_SecondaryCPICH_Info },
  { &hf_rrc_dl_ChannelisationCodeList, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DL_ChannelisationCodeList },
  { &hf_rrc_tpc_CombinationIndex, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TPC_CombinationIndex },
  { &hf_rrc_powerOffsetTPC_pdpdch, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_PowerOffsetTPC_pdpdch },
  { &hf_rrc_closedLoopTimingAdjMode, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_ClosedLoopTimingAdjMode },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_fdd_73(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_fdd_73, T_fdd_73_sequence);

  return offset;
}


static const per_sequence_t T_tdd_35_sequence[] = {
  { &hf_rrc_dl_CCTrChListToEstablish_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_CCTrChList_r4 },
  { &hf_rrc_dl_CCTrChListToRemove, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_CCTrChListToRemove },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_tdd_35(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_tdd_35, T_tdd_35_sequence);

  return offset;
}


static const value_string rrc_DL_DPCH_InfoPerRL_r6_vals[] = {
  {   0, "fdd" },
  {   1, "tdd" },
  { 0, NULL }
};

static const per_choice_t DL_DPCH_InfoPerRL_r6_choice[] = {
  {   0, &hf_rrc_fdd_74          , ASN1_NO_EXTENSIONS     , dissect_rrc_T_fdd_73 },
  {   1, &hf_rrc_tdd_36          , ASN1_NO_EXTENSIONS     , dissect_rrc_T_tdd_35 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_DL_DPCH_InfoPerRL_r6(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_DL_DPCH_InfoPerRL_r6, DL_DPCH_InfoPerRL_r6_choice,
                                 NULL);

  return offset;
}


static const value_string rrc_T_dl_dpchInfo_vals[] = {
  {   0, "dl-DPCH-InfoPerRL" },
  {   1, "dl-FDPCH-InfoPerRL" },
  { 0, NULL }
};

static const per_choice_t T_dl_dpchInfo_choice[] = {
  {   0, &hf_rrc_dl_DPCH_InfoPerRL_03, ASN1_NO_EXTENSIONS     , dissect_rrc_DL_DPCH_InfoPerRL_r6 },
  {   1, &hf_rrc_dl_FDPCH_InfoPerRL, ASN1_NO_EXTENSIONS     , dissect_rrc_DL_FDPCH_InfoPerRL_r6 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_dl_dpchInfo(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_dl_dpchInfo, T_dl_dpchInfo_choice,
                                 NULL);

  return offset;
}


static const value_string rrc_T_e_HICH_Info_vals[] = {
  {   0, "e-HICH-Information" },
  {   1, "releaseIndicator" },
  { 0, NULL }
};

static const per_choice_t T_e_HICH_Info_choice[] = {
  {   0, &hf_rrc_e_HICH_Information, ASN1_NO_EXTENSIONS     , dissect_rrc_E_HICH_Information },
  {   1, &hf_rrc_releaseIndicator, ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_e_HICH_Info(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_e_HICH_Info, T_e_HICH_Info_choice,
                                 NULL);

  return offset;
}


static const value_string rrc_T_e_RGCH_Info_01_vals[] = {
  {   0, "e-RGCH-Information" },
  {   1, "releaseIndicator" },
  { 0, NULL }
};

static const per_choice_t T_e_RGCH_Info_01_choice[] = {
  {   0, &hf_rrc_e_RGCH_Information, ASN1_NO_EXTENSIONS     , dissect_rrc_E_RGCH_Information },
  {   1, &hf_rrc_releaseIndicator, ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_e_RGCH_Info_01(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_e_RGCH_Info_01, T_e_RGCH_Info_01_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t DL_InformationPerRL_r6_sequence[] = {
  { &hf_rrc_modeSpecificInfo_52, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_modeSpecificInfo_52 },
  { &hf_rrc_dl_dpchInfo     , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_dl_dpchInfo },
  { &hf_rrc_e_AGCH_Information, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_E_AGCH_Information },
  { &hf_rrc_e_HICH_Info     , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_e_HICH_Info },
  { &hf_rrc_e_RGCH_Info_01  , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_e_RGCH_Info_01 },
  { &hf_rrc_cell_id         , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CellIdentity },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_DL_InformationPerRL_r6(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_DL_InformationPerRL_r6, DL_InformationPerRL_r6_sequence);

  return offset;
}


static const per_sequence_t DL_InformationPerRL_List_r6_sequence_of[1] = {
  { &hf_rrc_DL_InformationPerRL_List_r6_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DL_InformationPerRL_r6 },
};

static int
dissect_rrc_DL_InformationPerRL_List_r6(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_DL_InformationPerRL_List_r6, DL_InformationPerRL_List_r6_sequence_of,
                                                  1, maxRL, FALSE);

  return offset;
}


static const per_sequence_t CellUpdateConfirm_r6_IEs_sequence[] = {
  { &hf_rrc_integrityProtectionModeInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_IntegrityProtectionModeInfo },
  { &hf_rrc_cipheringModeInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CipheringModeInfo },
  { &hf_rrc_activationTime  , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_ActivationTime },
  { &hf_rrc_new_U_RNTI      , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_U_RNTI },
  { &hf_rrc_new_C_RNTI      , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_C_RNTI },
  { &hf_rrc_new_DSCH_RNTI   , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DSCH_RNTI },
  { &hf_rrc_new_H_RNTI      , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_H_RNTI },
  { &hf_rrc_newPrimary_E_RNTI, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_E_RNTI },
  { &hf_rrc_newSecondary_E_RNTI, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_E_RNTI },
  { &hf_rrc_rrc_StateIndicator, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RRC_StateIndicator },
  { &hf_rrc_utran_DRX_CycleLengthCoeff, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UTRAN_DRX_CycleLengthCoefficient },
  { &hf_rrc_rlc_Re_establishIndicatorRb2_3or4, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { &hf_rrc_rlc_Re_establishIndicatorRb5orAbove, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { &hf_rrc_cn_InformationInfo_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CN_InformationInfo_r6 },
  { &hf_rrc_ura_Identity    , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_URA_Identity },
  { &hf_rrc_rb_InformationReleaseList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RB_InformationReleaseList },
  { &hf_rrc_rb_InformationReconfigList_03, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RB_InformationReconfigList_r6 },
  { &hf_rrc_rb_InformationAffectedList_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RB_InformationAffectedList_r6 },
  { &hf_rrc_dl_CounterSynchronisationInfo_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_CounterSynchronisationInfo_r5 },
  { &hf_rrc_pdcp_ROHC_TargetMode, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_PDCP_ROHC_TargetMode },
  { &hf_rrc_ul_CommonTransChInfo_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_CommonTransChInfo_r4 },
  { &hf_rrc_ul_deletedTransChInfoList_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_DeletedTransChInfoList_r6 },
  { &hf_rrc_ul_AddReconfTransChInfoList_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_AddReconfTransChInfoList_r6 },
  { &hf_rrc_dl_CommonTransChInfo_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_CommonTransChInfo_r4 },
  { &hf_rrc_dl_DeletedTransChInfoList_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_DeletedTransChInfoList_r5 },
  { &hf_rrc_dl_AddReconfTransChInfoList_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_AddReconfTransChInfoList_r5 },
  { &hf_rrc_frequencyInfo   , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_FrequencyInfo },
  { &hf_rrc_maxAllowedUL_TX_Power, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MaxAllowedUL_TX_Power },
  { &hf_rrc_ul_DPCH_Info    , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_DPCH_Info_r6 },
  { &hf_rrc_ul_EDCH_Information, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_EDCH_Information_r6 },
  { &hf_rrc_dl_HSPDSCH_Information_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_HSPDSCH_Information_r6 },
  { &hf_rrc_dl_CommonInformation_03, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_CommonInformation_r6 },
  { &hf_rrc_dl_InformationPerRL_List_03, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_InformationPerRL_List_r6 },
  { &hf_rrc_mbms_PL_ServiceRestrictInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MBMS_PL_ServiceRestrictInfo_r6 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_CellUpdateConfirm_r6_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_CellUpdateConfirm_r6_IEs, CellUpdateConfirm_r6_IEs_sequence);

  return offset;
}


static const per_sequence_t DL_InformationPerRL_v6b0ext_sequence[] = {
  { &hf_rrc_sttdIndication  , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_STTDIndication },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_DL_InformationPerRL_v6b0ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_DL_InformationPerRL_v6b0ext, DL_InformationPerRL_v6b0ext_sequence);

  return offset;
}


static const per_sequence_t DL_InformationPerRL_List_v6b0ext_sequence_of[1] = {
  { &hf_rrc_DL_InformationPerRL_List_v6b0ext_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DL_InformationPerRL_v6b0ext },
};

static int
dissect_rrc_DL_InformationPerRL_List_v6b0ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_DL_InformationPerRL_List_v6b0ext, DL_InformationPerRL_List_v6b0ext_sequence_of,
                                                  1, maxRL, FALSE);

  return offset;
}


static const per_sequence_t CellUpdateConfirm_v6b0ext_IEs_sequence[] = {
  { &hf_rrc_dl_InformationPerRL_List_v6b0ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_InformationPerRL_List_v6b0ext },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_CellUpdateConfirm_v6b0ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_CellUpdateConfirm_v6b0ext_IEs, CellUpdateConfirm_v6b0ext_IEs_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions_17_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_nonCriticalExtensions_17(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_nonCriticalExtensions_17, T_nonCriticalExtensions_17_sequence);

  return offset;
}


static const per_sequence_t T_v6b0NonCriticalExtensions_02_sequence[] = {
  { &hf_rrc_cellUpdateConfirm_v6b0ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CellUpdateConfirm_v6b0ext_IEs },
  { &hf_rrc_nonCriticalExtensions_17, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_nonCriticalExtensions_17 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v6b0NonCriticalExtensions_02(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v6b0NonCriticalExtensions_02, T_v6b0NonCriticalExtensions_02_sequence);

  return offset;
}


static const per_sequence_t T_r6_01_sequence[] = {
  { &hf_rrc_cellUpdateConfirm_r6, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CellUpdateConfirm_r6_IEs },
  { &hf_rrc_cellUpdateConfirm_r6_add_ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_BIT_STRING },
  { &hf_rrc_v6b0NonCriticalExtensions_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v6b0NonCriticalExtensions_02 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_r6_01(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_r6_01, T_r6_01_sequence);

  return offset;
}


static const value_string rrc_IntegrityProtectionAlgorithm_r7_vals[] = {
  {   0, "uia1" },
  {   1, "uia2" },
  { 0, NULL }
};


static int
dissect_rrc_IntegrityProtectionAlgorithm_r7(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     2, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t IntegrityProtectionModeInfo_r7_sequence[] = {
  { &hf_rrc_integrityProtectionModeCommand, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_IntegrityProtectionModeCommand },
  { &hf_rrc_integrityProtectionAlgorithm_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_IntegrityProtectionAlgorithm_r7 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_IntegrityProtectionModeInfo_r7(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_IntegrityProtectionModeInfo_r7, IntegrityProtectionModeInfo_r7_sequence);

  return offset;
}


static const value_string rrc_CipheringAlgorithm_r7_vals[] = {
  {   0, "uea0" },
  {   1, "uea1" },
  {   2, "uea2" },
  { 0, NULL }
};


static int
dissect_rrc_CipheringAlgorithm_r7(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     3, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t CipheringModeCommand_r7_sequence[] = {
  { &hf_rrc_startRestart_01 , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CipheringAlgorithm_r7 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_CipheringModeCommand_r7(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_CipheringModeCommand_r7, CipheringModeCommand_r7_sequence);

  return offset;
}


static const per_sequence_t CipheringModeInfo_r7_sequence[] = {
  { &hf_rrc_cipheringModeCommand_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CipheringModeCommand_r7 },
  { &hf_rrc_activationTimeForDPCH, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_ActivationTime },
  { &hf_rrc_rb_DL_CiphActivationTimeInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RB_ActivationTimeInfoList },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_CipheringModeInfo_r7(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_CipheringModeInfo_r7, CipheringModeInfo_r7_sequence);

  return offset;
}



static int
dissect_rrc_INTEGER_3_9(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            3U, 9U, NULL, FALSE);

  return offset;
}


static const value_string rrc_T_319_vals[] = {
  {   0, "ms80" },
  {   1, "ms160" },
  {   2, "ms320" },
  {   3, "ms640" },
  {   4, "ms1280" },
  {   5, "ms2560" },
  {   6, "ms5120" },
  { 0, NULL }
};


static int
dissect_rrc_T_319(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     7, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t UTRAN_DRX_CycleLengthCoefficient_r7_sequence[] = {
  { &hf_rrc_drx_CycleLengthCoefficient, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_3_9 },
  { &hf_rrc_drx_CycleLengthCoefficient2, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_INTEGER_3_9 },
  { &hf_rrc_timeForDRXCycle2, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_319 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UTRAN_DRX_CycleLengthCoefficient_r7(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UTRAN_DRX_CycleLengthCoefficient_r7, UTRAN_DRX_CycleLengthCoefficient_r7_sequence);

  return offset;
}



static int
dissect_rrc_WaitTime(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            0U, 15U, NULL, FALSE);

  return offset;
}


static const value_string rrc_T_flexibleSize_vals[] = {
  {   0, "size7" },
  {   1, "size15" },
  { 0, NULL }
};


static int
dissect_rrc_T_flexibleSize(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     2, NULL, FALSE, 0, NULL);

  return offset;
}


static const value_string rrc_T_dl_RLC_PDU_size_vals[] = {
  {   0, "fixedSize" },
  {   1, "flexibleSize" },
  { 0, NULL }
};

static const per_choice_t T_dl_RLC_PDU_size_choice[] = {
  {   0, &hf_rrc_fixedSize       , ASN1_NO_EXTENSIONS     , dissect_rrc_OctetModeRLC_SizeInfoType1 },
  {   1, &hf_rrc_flexibleSize    , ASN1_NO_EXTENSIONS     , dissect_rrc_T_flexibleSize },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_dl_RLC_PDU_size(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_dl_RLC_PDU_size, T_dl_RLC_PDU_size_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t DL_AM_RLC_Mode_r7_sequence[] = {
  { &hf_rrc_dl_RLC_PDU_size_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_dl_RLC_PDU_size },
  { &hf_rrc_inSequenceDelivery, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { &hf_rrc_receivingWindowSize, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_ReceivingWindowSize },
  { &hf_rrc_dl_RLC_StatusInfo, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DL_RLC_StatusInfo },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_DL_AM_RLC_Mode_r7(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_DL_AM_RLC_Mode_r7, DL_AM_RLC_Mode_r7_sequence);

  return offset;
}


static const value_string rrc_DL_RLC_Mode_r7_vals[] = {
  {   0, "dl-AM-RLC-Mode" },
  {   1, "dl-UM-RLC-Mode" },
  {   2, "dl-TM-RLC-Mode" },
  { 0, NULL }
};

static const per_choice_t DL_RLC_Mode_r7_choice[] = {
  {   0, &hf_rrc_dl_AM_RLC_Mode_02, ASN1_NO_EXTENSIONS     , dissect_rrc_DL_AM_RLC_Mode_r7 },
  {   1, &hf_rrc_dl_UM_RLC_Mode_02, ASN1_NO_EXTENSIONS     , dissect_rrc_DL_UM_RLC_Mode_r6 },
  {   2, &hf_rrc_dl_TM_RLC_Mode  , ASN1_NO_EXTENSIONS     , dissect_rrc_DL_TM_RLC_Mode },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_DL_RLC_Mode_r7(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_DL_RLC_Mode_r7, DL_RLC_Mode_r7_choice,
                                 NULL);

  return offset;
}


static const value_string rrc_T_altE_bitInterpretation_01_vals[] = {
  {   0, "true" },
  { 0, NULL }
};


static int
dissect_rrc_T_altE_bitInterpretation_01(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}


static const value_string rrc_T_useSpecialValueOfHEField_vals[] = {
  {   0, "true" },
  { 0, NULL }
};


static int
dissect_rrc_T_useSpecialValueOfHEField(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t RLC_Info_r7_sequence[] = {
  { &hf_rrc_ul_RLC_Mode     , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_RLC_Mode },
  { &hf_rrc_dl_RLC_Mode_03  , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_RLC_Mode_r7 },
  { &hf_rrc_rlc_OneSidedReEst, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { &hf_rrc_altE_bitInterpretation_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_altE_bitInterpretation_01 },
  { &hf_rrc_useSpecialValueOfHEField, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_useSpecialValueOfHEField },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_RLC_Info_r7(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_RLC_Info_r7, RLC_Info_r7_sequence);

  return offset;
}



static int
dissect_rrc_MAC_ehs_QueueId(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            0U, 7U, NULL, FALSE);

  return offset;
}


static const value_string rrc_T_hsdsch_vals[] = {
  {   0, "mac-hs" },
  {   1, "mac-ehs" },
  { 0, NULL }
};

static const per_choice_t T_hsdsch_choice[] = {
  {   0, &hf_rrc_mac_hs          , ASN1_NO_EXTENSIONS     , dissect_rrc_MAC_d_FlowIdentity },
  {   1, &hf_rrc_mac_ehs         , ASN1_NO_EXTENSIONS     , dissect_rrc_MAC_ehs_QueueId },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_hsdsch(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_hsdsch, T_hsdsch_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t MAC_ehs_QueueIdDCHandHSDSCH_sequence[] = {
  { &hf_rrc_dch_transport_ch_id, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TransportChannelIdentity },
  { &hf_rrc_hsdsch_mac_ehs_QueueId, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MAC_ehs_QueueId },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_MAC_ehs_QueueIdDCHandHSDSCH(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_MAC_ehs_QueueIdDCHandHSDSCH, MAC_ehs_QueueIdDCHandHSDSCH_sequence);

  return offset;
}


static const value_string rrc_T_dch_and_hsdsch_vals[] = {
  {   0, "mac-hs" },
  {   1, "mac-ehs" },
  { 0, NULL }
};

static const per_choice_t T_dch_and_hsdsch_choice[] = {
  {   0, &hf_rrc_mac_hs_01       , ASN1_NO_EXTENSIONS     , dissect_rrc_MAC_d_FlowIdentityDCHandHSDSCH },
  {   1, &hf_rrc_mac_ehs_01      , ASN1_NO_EXTENSIONS     , dissect_rrc_MAC_ehs_QueueIdDCHandHSDSCH },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_dch_and_hsdsch(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_dch_and_hsdsch, T_dch_and_hsdsch_choice,
                                 NULL);

  return offset;
}


static const value_string rrc_DL_TransportChannelType_r7_vals[] = {
  {   0, "dch" },
  {   1, "fach" },
  {   2, "dsch" },
  {   3, "dch-and-dsch" },
  {   4, "hsdsch" },
  {   5, "dch-and-hsdsch" },
  { 0, NULL }
};

static const per_choice_t DL_TransportChannelType_r7_choice[] = {
  {   0, &hf_rrc_dch             , ASN1_NO_EXTENSIONS     , dissect_rrc_TransportChannelIdentity },
  {   1, &hf_rrc_fach            , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   2, &hf_rrc_dsch            , ASN1_NO_EXTENSIONS     , dissect_rrc_TransportChannelIdentity },
  {   3, &hf_rrc_dch_and_dsch    , ASN1_NO_EXTENSIONS     , dissect_rrc_TransportChannelIdentityDCHandDSCH },
  {   4, &hf_rrc_hsdsch_01       , ASN1_NO_EXTENSIONS     , dissect_rrc_T_hsdsch },
  {   5, &hf_rrc_dch_and_hsdsch_01, ASN1_NO_EXTENSIONS     , dissect_rrc_T_dch_and_hsdsch },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_DL_TransportChannelType_r7(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
#line 741 "../../asn1/rrc/rrc.cnf"

    gint *flowd_p;
    gint *cur_val=NULL;
    struct rrc_info *rrcinf;

      offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_DL_TransportChannelType_r7, DL_TransportChannelType_r7_choice,
                                 &type);


    /*TODO: This should probably be done better*/
    if(type == 4){    /*If this is type HS-DSCH*/
        num_chans_per_flow[flowd]++;


        if(num_chans_per_flow[flowd] > 1 ){
            rrcinf = p_get_proto_data(actx->pinfo->fd, proto_rrc);
            if((rrcinf == NULL) || (rrcinf->hrnti[actx->pinfo->fd->subnum] == 0)){
                expert_add_info_format(actx->pinfo, actx->created_item, PI_SEQUENCE, PI_NOTE, "Did not detect any H-RNTI");
            }
            else{
                /*If it doesnt exists, insert it*/
                if( (cur_val=g_tree_lookup(hsdsch_muxed_flows, GUINT_TO_POINTER((guint)rrcinf->hrnti[actx->pinfo->fd->subnum]))) == NULL ){

                    flowd_p = (guint*)g_malloc0(sizeof(gint));
                    *flowd_p = (1<<flowd);    /* Set the bit to mark it as true*/
                    g_tree_insert(hsdsch_muxed_flows, GUINT_TO_POINTER((guint)rrcinf->hrnti[actx->pinfo->fd->subnum]), flowd_p);

                }else{
                    *cur_val = (1<<flowd) | *cur_val;

                }
            }

        }

    }



  return offset;
}


static const per_sequence_t DL_LogicalChannelMapping_r7_sequence[] = {
  { &hf_rrc_dl_TransportChannelType_02, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DL_TransportChannelType_r7 },
  { &hf_rrc_logicalChannelIdentity, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_LogicalChannelIdentity },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_DL_LogicalChannelMapping_r7(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_DL_LogicalChannelMapping_r7, DL_LogicalChannelMapping_r7_sequence);

  return offset;
}


static const per_sequence_t DL_LogicalChannelMappingList_r7_sequence_of[1] = {
  { &hf_rrc_DL_LogicalChannelMappingList_r7_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DL_LogicalChannelMapping_r7 },
};

static int
dissect_rrc_DL_LogicalChannelMappingList_r7(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_DL_LogicalChannelMappingList_r7, DL_LogicalChannelMappingList_r7_sequence_of,
                                                  1, maxLoCHperRLC, FALSE);

  return offset;
}


static const per_sequence_t RB_MappingOption_r7_sequence[] = {
  { &hf_rrc_ul_LogicalChannelMappings_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_LogicalChannelMappings_r6 },
  { &hf_rrc_dl_LogicalChannelMappingList_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_LogicalChannelMappingList_r7 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_RB_MappingOption_r7(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_RB_MappingOption_r7, RB_MappingOption_r7_sequence);

  return offset;
}


static const per_sequence_t RB_MappingInfo_r7_sequence_of[1] = {
  { &hf_rrc_RB_MappingInfo_r7_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RB_MappingOption_r7 },
};

static int
dissect_rrc_RB_MappingInfo_r7(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_RB_MappingInfo_r7, RB_MappingInfo_r7_sequence_of,
                                                  1, maxRBMuxOptions, FALSE);

  return offset;
}


static const per_sequence_t RB_InformationReconfig_r7_sequence[] = {
  { &hf_rrc_rb_Identity     , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RB_Identity },
  { &hf_rrc_pdcp_Info_03    , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_PDCP_InfoReconfig_r4 },
  { &hf_rrc_pdcp_SN_Info    , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_PDCP_SN_Info },
  { &hf_rrc_rlc_Info_03     , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RLC_Info_r7 },
  { &hf_rrc_rb_MappingInfo_03, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RB_MappingInfo_r7 },
  { &hf_rrc_rb_StopContinue , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RB_StopContinue },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_RB_InformationReconfig_r7(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_RB_InformationReconfig_r7, RB_InformationReconfig_r7_sequence);

  return offset;
}


static const per_sequence_t RB_InformationReconfigList_r7_sequence_of[1] = {
  { &hf_rrc_RB_InformationReconfigList_r7_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RB_InformationReconfig_r7 },
};

static int
dissect_rrc_RB_InformationReconfigList_r7(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_RB_InformationReconfigList_r7, RB_InformationReconfigList_r7_sequence_of,
                                                  1, maxRB, FALSE);

  return offset;
}


static const per_sequence_t RB_InformationAffected_r7_sequence[] = {
  { &hf_rrc_rb_Identity     , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RB_Identity },
  { &hf_rrc_rb_MappingInfo_03, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RB_MappingInfo_r7 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_RB_InformationAffected_r7(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_RB_InformationAffected_r7, RB_InformationAffected_r7_sequence);

  return offset;
}


static const per_sequence_t RB_InformationAffectedList_r7_sequence_of[1] = {
  { &hf_rrc_RB_InformationAffectedList_r7_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RB_InformationAffected_r7 },
};

static int
dissect_rrc_RB_InformationAffectedList_r7(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_RB_InformationAffectedList_r7, RB_InformationAffectedList_r7_sequence_of,
                                                  1, maxRB, FALSE);

  return offset;
}


static const per_sequence_t T_dch_usch_01_sequence[] = {
  { &hf_rrc_ul_TransportChannelType_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UL_TrCH_Type },
  { &hf_rrc_transportChannelIdentity, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TransportChannelIdentity },
  { &hf_rrc_transportFormatSet, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TransportFormatSet },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_dch_usch_01(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_dch_usch_01, T_dch_usch_01_sequence);

  return offset;
}


static const per_sequence_t T_fdd_54_sequence[] = {
  { &hf_rrc_tti_03          , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_E_DCH_TTI },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_fdd_54(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_fdd_54, T_fdd_54_sequence);

  return offset;
}


static const value_string rrc_T_modeSpecific_vals[] = {
  {   0, "fdd" },
  {   1, "tdd" },
  { 0, NULL }
};

static const per_choice_t T_modeSpecific_choice[] = {
  {   0, &hf_rrc_fdd_55          , ASN1_NO_EXTENSIONS     , dissect_rrc_T_fdd_54 },
  {   1, &hf_rrc_tdd             , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_modeSpecific(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_modeSpecific, T_modeSpecific_choice,
                                 NULL);

  return offset;
}


static const value_string rrc_T_harq_Info_01_vals[] = {
  {   0, "rv0" },
  {   1, "rvtable" },
  { 0, NULL }
};


static int
dissect_rrc_T_harq_Info_01(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     2, NULL, FALSE, 0, NULL);

  return offset;
}


static const value_string rrc_E_DCH_MAC_d_FlowRetransTimer_vals[] = {
  {   0, "ms10" },
  {   1, "ms15" },
  {   2, "ms20" },
  {   3, "ms25" },
  {   4, "ms30" },
  {   5, "ms35" },
  {   6, "ms40" },
  {   7, "ms45" },
  {   8, "ms50" },
  {   9, "ms55" },
  {  10, "ms60" },
  {  11, "ms65" },
  {  12, "ms70" },
  {  13, "ms75" },
  {  14, "ms80" },
  {  15, "ms85" },
  {  16, "ms90" },
  {  17, "ms95" },
  {  18, "ms100" },
  {  19, "ms110" },
  {  20, "ms120" },
  {  21, "ms140" },
  {  22, "ms160" },
  {  23, "ms200" },
  {  24, "ms240" },
  {  25, "ms280" },
  {  26, "ms320" },
  {  27, "ms400" },
  {  28, "ms480" },
  {  29, "ms560" },
  { 0, NULL }
};

static value_string_ext rrc_E_DCH_MAC_d_FlowRetransTimer_vals_ext = VALUE_STRING_EXT_INIT(rrc_E_DCH_MAC_d_FlowRetransTimer_vals);


static int
dissect_rrc_E_DCH_MAC_d_FlowRetransTimer(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     30, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t T_fdd_51_sequence[] = {
  { &hf_rrc_maxMAC_e_PDUContents, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_1_19982 },
  { &hf_rrc_ms2_NonSchedTransmGrantHARQAlloc, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_BIT_STRING_SIZE_8 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_fdd_51(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_fdd_51, T_fdd_51_sequence);

  return offset;
}


static const value_string rrc_T_modeSpecificInfo_21_vals[] = {
  {   0, "fdd" },
  {   1, "tdd" },
  { 0, NULL }
};

static const per_choice_t T_modeSpecificInfo_21_choice[] = {
  {   0, &hf_rrc_fdd_52          , ASN1_NO_EXTENSIONS     , dissect_rrc_T_fdd_51 },
  {   1, &hf_rrc_tdd             , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_modeSpecificInfo_21(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_modeSpecificInfo_21, T_modeSpecificInfo_21_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t T_non_ScheduledTransGrantInfo_01_sequence[] = {
  { &hf_rrc_modeSpecificInfo_21, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_modeSpecificInfo_21 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_non_ScheduledTransGrantInfo_01(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_non_ScheduledTransGrantInfo_01, T_non_ScheduledTransGrantInfo_01_sequence);

  return offset;
}


static const value_string rrc_T_transmissionGrantType_01_vals[] = {
  {   0, "non-ScheduledTransGrantInfo" },
  {   1, "scheduledTransmissionGrantInfo" },
  { 0, NULL }
};

static const per_choice_t T_transmissionGrantType_01_choice[] = {
  {   0, &hf_rrc_non_ScheduledTransGrantInfo_01, ASN1_NO_EXTENSIONS     , dissect_rrc_T_non_ScheduledTransGrantInfo_01 },
  {   1, &hf_rrc_scheduledTransmissionGrantInfo, ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_transmissionGrantType_01(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_transmissionGrantType_01, T_transmissionGrantType_01_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t E_DCH_AddReconf_MAC_d_Flow_r7_sequence[] = {
  { &hf_rrc_mac_d_FlowIdentity, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_E_DCH_MAC_d_FlowIdentity },
  { &hf_rrc_mac_d_FlowPowerOffset, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_E_DCH_MAC_d_FlowPowerOffset },
  { &hf_rrc_mac_d_FlowMaxRetrans, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_E_DCH_MAC_d_FlowMaxRetrans },
  { &hf_rrc_mac_d_FlowRetransTimer, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_E_DCH_MAC_d_FlowRetransTimer },
  { &hf_rrc_mac_d_FlowMultiplexingList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_E_DCH_MAC_d_FlowMultiplexingList },
  { &hf_rrc_transmissionGrantType_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_transmissionGrantType_01 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_E_DCH_AddReconf_MAC_d_Flow_r7(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_E_DCH_AddReconf_MAC_d_Flow_r7, E_DCH_AddReconf_MAC_d_Flow_r7_sequence);

  return offset;
}


static const per_sequence_t E_DCH_AddReconf_MAC_d_FlowList_r7_sequence_of[1] = {
  { &hf_rrc_E_DCH_AddReconf_MAC_d_FlowList_r7_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_E_DCH_AddReconf_MAC_d_Flow_r7 },
};

static int
dissect_rrc_E_DCH_AddReconf_MAC_d_FlowList_r7(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_E_DCH_AddReconf_MAC_d_FlowList_r7, E_DCH_AddReconf_MAC_d_FlowList_r7_sequence_of,
                                                  1, maxE_DCHMACdFlow, FALSE);

  return offset;
}


static const per_sequence_t T_e_dch_03_sequence[] = {
  { &hf_rrc_modeSpecific    , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_modeSpecific },
  { &hf_rrc_harq_Info_01    , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_harq_Info_01 },
  { &hf_rrc_addReconf_MAC_d_FlowList_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_E_DCH_AddReconf_MAC_d_FlowList_r7 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_e_dch_03(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_e_dch_03, T_e_dch_03_sequence);

  return offset;
}


static const value_string rrc_UL_AddReconfTransChInformation_r7_vals[] = {
  {   0, "dch-usch" },
  {   1, "e-dch" },
  { 0, NULL }
};

static const per_choice_t UL_AddReconfTransChInformation_r7_choice[] = {
  {   0, &hf_rrc_dch_usch_01     , ASN1_NO_EXTENSIONS     , dissect_rrc_T_dch_usch_01 },
  {   1, &hf_rrc_e_dch_03        , ASN1_NO_EXTENSIONS     , dissect_rrc_T_e_dch_03 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_UL_AddReconfTransChInformation_r7(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_UL_AddReconfTransChInformation_r7, UL_AddReconfTransChInformation_r7_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t UL_AddReconfTransChInfoList_r7_sequence_of[1] = {
  { &hf_rrc_UL_AddReconfTransChInfoList_r7_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UL_AddReconfTransChInformation_r7 },
};

static int
dissect_rrc_UL_AddReconfTransChInfoList_r7(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_UL_AddReconfTransChInfoList_r7, UL_AddReconfTransChInfoList_r7_sequence_of,
                                                  1, maxTrCH, FALSE);

  return offset;
}


static const value_string rrc_T_hsdsch_01_vals[] = {
  {   0, "mac-hs" },
  {   1, "mac-ehs" },
  { 0, NULL }
};

static const per_choice_t T_hsdsch_01_choice[] = {
  {   0, &hf_rrc_mac_hs          , ASN1_NO_EXTENSIONS     , dissect_rrc_MAC_d_FlowIdentity },
  {   1, &hf_rrc_mac_ehs         , ASN1_NO_EXTENSIONS     , dissect_rrc_MAC_ehs_QueueId },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_hsdsch_01(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_hsdsch_01, T_hsdsch_01_choice,
                                 NULL);

  return offset;
}


static const value_string rrc_DL_TrCH_TypeId2_r7_vals[] = {
  {   0, "dch" },
  {   1, "dsch" },
  {   2, "hsdsch" },
  { 0, NULL }
};

static const per_choice_t DL_TrCH_TypeId2_r7_choice[] = {
  {   0, &hf_rrc_dch             , ASN1_NO_EXTENSIONS     , dissect_rrc_TransportChannelIdentity },
  {   1, &hf_rrc_dsch            , ASN1_NO_EXTENSIONS     , dissect_rrc_TransportChannelIdentity },
  {   2, &hf_rrc_hsdsch_08       , ASN1_NO_EXTENSIONS     , dissect_rrc_T_hsdsch_01 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_DL_TrCH_TypeId2_r7(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_DL_TrCH_TypeId2_r7, DL_TrCH_TypeId2_r7_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t DL_TransportChannelIdentity_r7_sequence[] = {
  { &hf_rrc_dl_TransportChannelType_06, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DL_TrCH_TypeId2_r7 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_DL_TransportChannelIdentity_r7(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_DL_TransportChannelIdentity_r7, DL_TransportChannelIdentity_r7_sequence);

  return offset;
}


static const per_sequence_t DL_DeletedTransChInfoList_r7_sequence_of[1] = {
  { &hf_rrc_DL_DeletedTransChInfoList_r7_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DL_TransportChannelIdentity_r7 },
};

static int
dissect_rrc_DL_DeletedTransChInfoList_r7(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_DL_DeletedTransChInfoList_r7, DL_DeletedTransChInfoList_r7_sequence_of,
                                                  1, maxTrCH, FALSE);

  return offset;
}


static const value_string rrc_Treset_ResetTimer_vals[] = {
  {   0, "rt1" },
  {   1, "rt2" },
  {   2, "rt3" },
  {   3, "rt4" },
  { 0, NULL }
};


static int
dissect_rrc_Treset_ResetTimer(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     4, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t MAC_ehs_AddReconfReordQ_sequence[] = {
  { &hf_rrc_mac_ehs_QueueId , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MAC_ehs_QueueId },
  { &hf_rrc_reorderingReleaseTimer, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T1_ReleaseTimer },
  { &hf_rrc_reorderingResetTimer, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_Treset_ResetTimer },
  { &hf_rrc_mac_ehsWindowSize_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MAC_hs_WindowSize },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_MAC_ehs_AddReconfReordQ(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_MAC_ehs_AddReconfReordQ, MAC_ehs_AddReconfReordQ_sequence);

  return offset;
}


static const per_sequence_t MAC_ehs_AddReconfReordQ_List_sequence_of[1] = {
  { &hf_rrc_MAC_ehs_AddReconfReordQ_List_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MAC_ehs_AddReconfReordQ },
};

static int
dissect_rrc_MAC_ehs_AddReconfReordQ_List(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_MAC_ehs_AddReconfReordQ_List, MAC_ehs_AddReconfReordQ_List_sequence_of,
                                                  1, maxQueueIDs, FALSE);

  return offset;
}


static const per_sequence_t MAC_ehs_DelReordQ_sequence[] = {
  { &hf_rrc_mac_ehs_QueueId , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MAC_ehs_QueueId },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_MAC_ehs_DelReordQ(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_MAC_ehs_DelReordQ, MAC_ehs_DelReordQ_sequence);

  return offset;
}


static const per_sequence_t MAC_ehs_DelReordQ_List_sequence_of[1] = {
  { &hf_rrc_MAC_ehs_DelReordQ_List_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MAC_ehs_DelReordQ },
};

static int
dissect_rrc_MAC_ehs_DelReordQ_List(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_MAC_ehs_DelReordQ_List, MAC_ehs_DelReordQ_List_sequence_of,
                                                  1, maxQueueIDs, FALSE);

  return offset;
}


static const per_sequence_t AddOrReconfMAC_ehs_ReordQ_sequence[] = {
  { &hf_rrc_mac_ehs_AddReconfQueue_List, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MAC_ehs_AddReconfReordQ_List },
  { &hf_rrc_dummy_34        , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MAC_ehs_DelReordQ_List },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_AddOrReconfMAC_ehs_ReordQ(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_AddOrReconfMAC_ehs_ReordQ, AddOrReconfMAC_ehs_ReordQ_sequence);

  return offset;
}


static const value_string rrc_T_dl_MAC_HeaderType_vals[] = {
  {   0, "mac-hs" },
  {   1, "mac-ehs" },
  { 0, NULL }
};

static const per_choice_t T_dl_MAC_HeaderType_choice[] = {
  {   0, &hf_rrc_mac_hs_02       , ASN1_NO_EXTENSIONS     , dissect_rrc_AddOrReconfMAC_dFlow },
  {   1, &hf_rrc_mac_ehs_02      , ASN1_NO_EXTENSIONS     , dissect_rrc_AddOrReconfMAC_ehs_ReordQ },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_dl_MAC_HeaderType(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_dl_MAC_HeaderType, T_dl_MAC_HeaderType_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t HSDSCH_Info_r7_sequence[] = {
  { &hf_rrc_harqInfo        , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_HARQ_Info_r7 },
  { &hf_rrc_dl_MAC_HeaderType, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_dl_MAC_HeaderType },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_HSDSCH_Info_r7(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_HSDSCH_Info_r7, HSDSCH_Info_r7_sequence);

  return offset;
}


static const value_string rrc_T_tfs_SignallingMode_03_vals[] = {
  {   0, "explicit-config" },
  {   1, "sameAsULTrCH" },
  {   2, "hsdsch" },
  { 0, NULL }
};

static const per_choice_t T_tfs_SignallingMode_03_choice[] = {
  {   0, &hf_rrc_explicit_config , ASN1_NO_EXTENSIONS     , dissect_rrc_TransportFormatSet },
  {   1, &hf_rrc_sameAsULTrCH    , ASN1_NO_EXTENSIONS     , dissect_rrc_UL_TransportChannelIdentity },
  {   2, &hf_rrc_hsdsch_03       , ASN1_NO_EXTENSIONS     , dissect_rrc_HSDSCH_Info_r7 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_tfs_SignallingMode_03(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_tfs_SignallingMode_03, T_tfs_SignallingMode_03_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t DL_AddReconfTransChInformation_r7_sequence[] = {
  { &hf_rrc_dl_TransportChannelType_04, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DL_TrCH_TypeId1_r5 },
  { &hf_rrc_tfs_SignallingMode_03, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_tfs_SignallingMode_03 },
  { &hf_rrc_dch_QualityTarget, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_QualityTarget },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_DL_AddReconfTransChInformation_r7(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_DL_AddReconfTransChInformation_r7, DL_AddReconfTransChInformation_r7_sequence);

  return offset;
}


static const per_sequence_t DL_AddReconfTransChInfoList_r7_sequence_of[1] = {
  { &hf_rrc_DL_AddReconfTransChInfoList_r7_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DL_AddReconfTransChInformation_r7 },
};

static int
dissect_rrc_DL_AddReconfTransChInfoList_r7(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_DL_AddReconfTransChInfoList_r7, DL_AddReconfTransChInfoList_r7_sequence_of,
                                                  1, maxTrCHpreconf, FALSE);

  return offset;
}



static int
dissect_rrc_UpPCHposition_LCR(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            0U, 127U, NULL, FALSE);

  return offset;
}


static const per_sequence_t Multi_frequencyInfo_LCR_r7_sequence[] = {
  { &hf_rrc_secondFrequencyInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_FrequencyInfoTDD },
  { &hf_rrc_fPachFrequencyInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_FrequencyInfoTDD },
  { &hf_rrc_upPCHpositionInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UpPCHposition_LCR },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_Multi_frequencyInfo_LCR_r7(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_Multi_frequencyInfo_LCR_r7, Multi_frequencyInfo_LCR_r7_sequence);

  return offset;
}


static const per_sequence_t T_fdd_125_sequence[] = {
  { &hf_rrc_dpcch_PowerOffset, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DPCCH_PowerOffset },
  { &hf_rrc_pc_Preamble     , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PC_Preamble },
  { &hf_rrc_sRB_delay       , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SRB_delay },
  { &hf_rrc_powerControlAlgorithm, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PowerControlAlgorithm },
  { &hf_rrc_deltaACK        , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DeltaACK },
  { &hf_rrc_deltaNACK       , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DeltaNACK },
  { &hf_rrc_ack_NACK_repetition_factor, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_ACK_NACK_repetitionFactor },
  { &hf_rrc_harq_Preamble_Mode, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_HARQ_Preamble_Mode },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_fdd_125(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_fdd_125, T_fdd_125_sequence);

  return offset;
}


static const per_sequence_t T_tdd384_39_sequence[] = {
  { &hf_rrc_individualTS_InterferenceList, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_IndividualTS_InterferenceList },
  { &hf_rrc_dpch_ConstantValue_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_ConstantValue },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_tdd384_39(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_tdd384_39, T_tdd384_39_sequence);

  return offset;
}


static const per_sequence_t T_tdd768_21_sequence[] = {
  { &hf_rrc_individualTS_InterferenceList, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_IndividualTS_InterferenceList },
  { &hf_rrc_dpch_ConstantValue_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_ConstantValue },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_tdd768_21(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_tdd768_21, T_tdd768_21_sequence);

  return offset;
}


static const per_sequence_t T_tdd128_58_sequence[] = {
  { &hf_rrc_beaconPLEst     , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_BEACON_PL_Est },
  { &hf_rrc_tpc_StepSize    , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TPC_StepSizeTDD },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_tdd128_58(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_tdd128_58, T_tdd128_58_sequence);

  return offset;
}


static const value_string rrc_T_tddOption_24_vals[] = {
  {   0, "tdd384" },
  {   1, "tdd768" },
  {   2, "tdd128" },
  { 0, NULL }
};

static const per_choice_t T_tddOption_24_choice[] = {
  {   0, &hf_rrc_tdd384_43       , ASN1_NO_EXTENSIONS     , dissect_rrc_T_tdd384_39 },
  {   1, &hf_rrc_tdd768_24       , ASN1_NO_EXTENSIONS     , dissect_rrc_T_tdd768_21 },
  {   2, &hf_rrc_tdd128_60       , ASN1_NO_EXTENSIONS     , dissect_rrc_T_tdd128_58 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_tddOption_24(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_tddOption_24, T_tddOption_24_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t T_individuallySignalled_04_sequence[] = {
  { &hf_rrc_tddOption_24    , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_tddOption_24 },
  { &hf_rrc_primaryCCPCH_TX_Power, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PrimaryCCPCH_TX_Power },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_individuallySignalled_04(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_individuallySignalled_04, T_individuallySignalled_04_sequence);

  return offset;
}


static const value_string rrc_T_ul_OL_PC_Signalling_04_vals[] = {
  {   0, "broadcast-UL-OL-PC-info" },
  {   1, "individuallySignalled" },
  { 0, NULL }
};

static const per_choice_t T_ul_OL_PC_Signalling_04_choice[] = {
  {   0, &hf_rrc_broadcast_UL_OL_PC_info, ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   1, &hf_rrc_individuallySignalled_04, ASN1_NO_EXTENSIONS     , dissect_rrc_T_individuallySignalled_04 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_ul_OL_PC_Signalling_04(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_ul_OL_PC_Signalling_04, T_ul_OL_PC_Signalling_04_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t T_tdd_75_sequence[] = {
  { &hf_rrc_ul_TargetSIR    , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_TargetSIR },
  { &hf_rrc_ul_OL_PC_Signalling_04, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_ul_OL_PC_Signalling_04 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_tdd_75(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_tdd_75, T_tdd_75_sequence);

  return offset;
}


static const value_string rrc_UL_DPCH_PowerControlInfo_r7_vals[] = {
  {   0, "fdd" },
  {   1, "tdd" },
  { 0, NULL }
};

static const per_choice_t UL_DPCH_PowerControlInfo_r7_choice[] = {
  {   0, &hf_rrc_fdd_128         , ASN1_NO_EXTENSIONS     , dissect_rrc_T_fdd_125 },
  {   1, &hf_rrc_tdd_81          , ASN1_NO_EXTENSIONS     , dissect_rrc_T_tdd_75 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_UL_DPCH_PowerControlInfo_r7(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_UL_DPCH_PowerControlInfo_r7, UL_DPCH_PowerControlInfo_r7_choice,
                                 NULL);

  return offset;
}


static const value_string rrc_NumberOfTPC_Bits_vals[] = {
  {   0, "tpc4" },
  { 0, NULL }
};


static int
dissect_rrc_NumberOfTPC_Bits(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t T_present_03_sequence[] = {
  { &hf_rrc_numberOfDPDCH   , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_NumberOfDPDCH },
  { &hf_rrc_spreadingFactor_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SpreadingFactor },
  { &hf_rrc_tfci_Existence  , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { &hf_rrc_numberOfFBI_Bits, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_NumberOfFBI_Bits },
  { &hf_rrc_numberOfTPC_Bits, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_NumberOfTPC_Bits },
  { &hf_rrc_puncturingLimit , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PuncturingLimit },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_present_03(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_present_03, T_present_03_sequence);

  return offset;
}


static const per_sequence_t T_notPresent_01_sequence[] = {
  { &hf_rrc_tfci_Existence  , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { &hf_rrc_numberOfFBI_Bits, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_NumberOfFBI_Bits },
  { &hf_rrc_numberOfTPC_Bits, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_NumberOfTPC_Bits },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_notPresent_01(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_notPresent_01, T_notPresent_01_sequence);

  return offset;
}


static const value_string rrc_T_dpdchPresence_01_vals[] = {
  {   0, "present" },
  {   1, "notPresent" },
  { 0, NULL }
};

static const per_choice_t T_dpdchPresence_01_choice[] = {
  {   0, &hf_rrc_present_05      , ASN1_NO_EXTENSIONS     , dissect_rrc_T_present_03 },
  {   1, &hf_rrc_notPresent_01   , ASN1_NO_EXTENSIONS     , dissect_rrc_T_notPresent_01 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_dpdchPresence_01(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_dpdchPresence_01, T_dpdchPresence_01_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t T_fdd_117_sequence[] = {
  { &hf_rrc_scramblingCodeType, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_ScramblingCodeType },
  { &hf_rrc_scramblingCode  , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UL_ScramblingCode },
  { &hf_rrc_dpdchPresence_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_dpdchPresence_01 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_fdd_117(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_fdd_117, T_fdd_117_sequence);

  return offset;
}


static const per_sequence_t T_tdd384_22_sequence[] = {
  { &hf_rrc_ex_ul_TimingAdvance, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_0_255 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_tdd384_22(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_tdd384_22, T_tdd384_22_sequence);

  return offset;
}


static const per_sequence_t T_tdd768_14_sequence[] = {
  { &hf_rrc_ex_ul_TimingAdvance_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_0_511 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_tdd768_14(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_tdd768_14, T_tdd768_14_sequence);

  return offset;
}


static const value_string rrc_T_modeSpecificInfo_60_vals[] = {
  {   0, "tdd384" },
  {   1, "tdd768" },
  { 0, NULL }
};

static const per_choice_t T_modeSpecificInfo_60_choice[] = {
  {   0, &hf_rrc_tdd384_24       , ASN1_NO_EXTENSIONS     , dissect_rrc_T_tdd384_22 },
  {   1, &hf_rrc_tdd768_16       , ASN1_NO_EXTENSIONS     , dissect_rrc_T_tdd768_14 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_modeSpecificInfo_60(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_modeSpecificInfo_60, T_modeSpecificInfo_60_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t EXT_UL_TimingAdvance_sequence[] = {
  { &hf_rrc_modeSpecificInfo_60, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_modeSpecificInfo_60 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_EXT_UL_TimingAdvance(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_EXT_UL_TimingAdvance, EXT_UL_TimingAdvance_sequence);

  return offset;
}


static const per_sequence_t T_tdd384_42_sequence[] = {
  { &hf_rrc_ul_TimingAdvance, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_TimingAdvance },
  { &hf_rrc_ext_UL_TimingAdvance, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_EXT_UL_TimingAdvance },
  { &hf_rrc_activationTime  , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_ActivationTime },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_tdd384_42(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_tdd384_42, T_tdd384_42_sequence);

  return offset;
}


static const per_sequence_t T_tdd768_23_sequence[] = {
  { &hf_rrc_ext_UL_TimingAdvance, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_EXT_UL_TimingAdvance },
  { &hf_rrc_activationTime  , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_ActivationTime },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_tdd768_23(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_tdd768_23, T_tdd768_23_sequence);

  return offset;
}


static const per_sequence_t T_tdd128_61_sequence[] = {
  { &hf_rrc_ul_SynchronisationParameters, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_SynchronisationParameters_r4 },
  { &hf_rrc_synchronisationParameters, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_SynchronisationParameters_r4 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_tdd128_61(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_tdd128_61, T_tdd128_61_sequence);

  return offset;
}


static const value_string rrc_T_tddOption_27_vals[] = {
  {   0, "tdd384" },
  {   1, "tdd768" },
  {   2, "tdd128" },
  { 0, NULL }
};

static const per_choice_t T_tddOption_27_choice[] = {
  {   0, &hf_rrc_tdd384_46       , ASN1_NO_EXTENSIONS     , dissect_rrc_T_tdd384_42 },
  {   1, &hf_rrc_tdd768_26       , ASN1_NO_EXTENSIONS     , dissect_rrc_T_tdd768_23 },
  {   2, &hf_rrc_tdd128_63       , ASN1_NO_EXTENSIONS     , dissect_rrc_T_tdd128_61 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_tddOption_27(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_tddOption_27, T_tddOption_27_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t T_enabled_02_sequence[] = {
  { &hf_rrc_tddOption_27    , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_tddOption_27 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_enabled_02(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_enabled_02, T_enabled_02_sequence);

  return offset;
}


static const value_string rrc_UL_TimingAdvanceControl_r7_vals[] = {
  {   0, "disabled" },
  {   1, "enabled" },
  { 0, NULL }
};

static const per_choice_t UL_TimingAdvanceControl_r7_choice[] = {
  {   0, &hf_rrc_disabled        , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   1, &hf_rrc_enabled_02      , ASN1_NO_EXTENSIONS     , dissect_rrc_T_enabled_02 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_UL_TimingAdvanceControl_r7(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_UL_TimingAdvanceControl_r7, UL_TimingAdvanceControl_r7_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t T_tdd384_35_sequence[] = {
  { &hf_rrc_ul_CCTrCH_TimeslotsCodes, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UplinkTimeslotsCodes },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_tdd384_35(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_tdd384_35, T_tdd384_35_sequence);

  return offset;
}


static const per_sequence_t T_ueSpecificMidamble_10_sequence[] = {
  { &hf_rrc_midambleShift   , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MidambleShiftLong },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_ueSpecificMidamble_10(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_ueSpecificMidamble_10, T_ueSpecificMidamble_10_sequence);

  return offset;
}


static const value_string rrc_T_midambleAllocationMode_18_vals[] = {
  {   0, "defaultMidamble" },
  {   1, "commonMidamble" },
  {   2, "ueSpecificMidamble" },
  { 0, NULL }
};

static const per_choice_t T_midambleAllocationMode_18_choice[] = {
  {   0, &hf_rrc_defaultMidamble , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   1, &hf_rrc_commonMidamble  , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   2, &hf_rrc_ueSpecificMidamble_11, ASN1_NO_EXTENSIONS     , dissect_rrc_T_ueSpecificMidamble_10 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_midambleAllocationMode_18(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_midambleAllocationMode_18, T_midambleAllocationMode_18_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t T_type1_02_sequence[] = {
  { &hf_rrc_midambleConfigurationBurstType1and3, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MidambleConfigurationBurstType1and3 },
  { &hf_rrc_midambleAllocationMode_18, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_midambleAllocationMode_18 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_type1_02(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_type1_02, T_type1_02_sequence);

  return offset;
}


static const per_sequence_t T_ueSpecificMidamble_11_sequence[] = {
  { &hf_rrc_midambleShift_03, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MidambleShiftShort_VHCR },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_ueSpecificMidamble_11(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_ueSpecificMidamble_11, T_ueSpecificMidamble_11_sequence);

  return offset;
}


static const value_string rrc_T_midambleAllocationMode_19_vals[] = {
  {   0, "defaultMidamble" },
  {   1, "commonMidamble" },
  {   2, "ueSpecificMidamble" },
  { 0, NULL }
};

static const per_choice_t T_midambleAllocationMode_19_choice[] = {
  {   0, &hf_rrc_defaultMidamble , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   1, &hf_rrc_commonMidamble  , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   2, &hf_rrc_ueSpecificMidamble_12, ASN1_NO_EXTENSIONS     , dissect_rrc_T_ueSpecificMidamble_11 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_midambleAllocationMode_19(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_midambleAllocationMode_19, T_midambleAllocationMode_19_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t T_type2_01_sequence[] = {
  { &hf_rrc_midambleConfigurationBurstType2_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MidambleConfigurationBurstType2_VHCR },
  { &hf_rrc_midambleAllocationMode_19, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_midambleAllocationMode_19 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_type2_01(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_type2_01, T_type2_01_sequence);

  return offset;
}


static const per_sequence_t T_ueSpecificMidamble_12_sequence[] = {
  { &hf_rrc_midambleShift   , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MidambleShiftLong },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_ueSpecificMidamble_12(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_ueSpecificMidamble_12, T_ueSpecificMidamble_12_sequence);

  return offset;
}


static const value_string rrc_T_midambleAllocationMode_20_vals[] = {
  {   0, "defaultMidamble" },
  {   1, "ueSpecificMidamble" },
  { 0, NULL }
};

static const per_choice_t T_midambleAllocationMode_20_choice[] = {
  {   0, &hf_rrc_defaultMidamble , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   1, &hf_rrc_ueSpecificMidamble_13, ASN1_NO_EXTENSIONS     , dissect_rrc_T_ueSpecificMidamble_12 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_midambleAllocationMode_20(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_midambleAllocationMode_20, T_midambleAllocationMode_20_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t T_type3_01_sequence[] = {
  { &hf_rrc_midambleConfigurationBurstType1and3, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MidambleConfigurationBurstType1and3 },
  { &hf_rrc_midambleAllocationMode_20, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_midambleAllocationMode_20 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_type3_01(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_type3_01, T_type3_01_sequence);

  return offset;
}


static const value_string rrc_T_burstType_01_vals[] = {
  {   0, "type1" },
  {   1, "type2" },
  {   2, "type3" },
  {   3, "mbsfnBurstType4" },
  { 0, NULL }
};

static const per_choice_t T_burstType_01_choice[] = {
  {   0, &hf_rrc_type1_02        , ASN1_NO_EXTENSIONS     , dissect_rrc_T_type1_02 },
  {   1, &hf_rrc_type2_01        , ASN1_NO_EXTENSIONS     , dissect_rrc_T_type2_01 },
  {   2, &hf_rrc_type3_01        , ASN1_NO_EXTENSIONS     , dissect_rrc_T_type3_01 },
  {   3, &hf_rrc_mbsfnBurstType4 , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_burstType_01(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_burstType_01, T_burstType_01_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t MidambleShiftAndBurstType_VHCR_sequence[] = {
  { &hf_rrc_burstType_02    , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_burstType_01 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_MidambleShiftAndBurstType_VHCR(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_MidambleShiftAndBurstType_VHCR, MidambleShiftAndBurstType_VHCR_sequence);

  return offset;
}


static const per_sequence_t IndividualTimeslotInfo_VHCR_sequence[] = {
  { &hf_rrc_timeslotNumber  , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TimeslotNumber },
  { &hf_rrc_tfci_Existence  , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { &hf_rrc_midambleShiftAndBurstType_VHCR, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MidambleShiftAndBurstType_VHCR },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_IndividualTimeslotInfo_VHCR(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_IndividualTimeslotInfo_VHCR, IndividualTimeslotInfo_VHCR_sequence);

  return offset;
}


static const value_string rrc_UL_TS_ChannelisationCode_VHCR_vals[] = {
  {   0, "cc1-1" },
  {   1, "cc2-1" },
  {   2, "cc2-2" },
  {   3, "cc4-1" },
  {   4, "cc4-2" },
  {   5, "cc4-3" },
  {   6, "cc4-4" },
  {   7, "cc8-1" },
  {   8, "cc8-2" },
  {   9, "cc8-3" },
  {  10, "cc8-4" },
  {  11, "cc8-5" },
  {  12, "cc8-6" },
  {  13, "cc8-7" },
  {  14, "cc8-8" },
  {  15, "cc16-1" },
  {  16, "cc16-2" },
  {  17, "cc16-3" },
  {  18, "cc16-4" },
  {  19, "cc16-5" },
  {  20, "cc16-6" },
  {  21, "cc16-7" },
  {  22, "cc16-8" },
  {  23, "cc16-9" },
  {  24, "cc16-10" },
  {  25, "cc16-11" },
  {  26, "cc16-12" },
  {  27, "cc16-13" },
  {  28, "cc16-14" },
  {  29, "cc16-15" },
  {  30, "cc16-16" },
  {  31, "cc32-1" },
  {  32, "cc32-2" },
  {  33, "cc32-3" },
  {  34, "cc32-4" },
  {  35, "cc32-5" },
  {  36, "cc32-6" },
  {  37, "cc32-7" },
  {  38, "cc32-8" },
  {  39, "cc32-9" },
  {  40, "cc32-10" },
  {  41, "cc32-11" },
  {  42, "cc32-12" },
  {  43, "cc32-13" },
  {  44, "cc32-14" },
  {  45, "cc32-15" },
  {  46, "cc32-16" },
  {  47, "cc32-17" },
  {  48, "cc32-18" },
  {  49, "cc32-19" },
  {  50, "cc32-20" },
  {  51, "cc32-21" },
  {  52, "cc32-22" },
  {  53, "cc32-23" },
  {  54, "cc32-24" },
  {  55, "cc32-25" },
  {  56, "cc32-26" },
  {  57, "cc32-27" },
  {  58, "cc32-28" },
  {  59, "cc32-29" },
  {  60, "cc32-30" },
  {  61, "cc32-31" },
  {  62, "cc32-32" },
  { 0, NULL }
};

static value_string_ext rrc_UL_TS_ChannelisationCode_VHCR_vals_ext = VALUE_STRING_EXT_INIT(rrc_UL_TS_ChannelisationCode_VHCR_vals);


static int
dissect_rrc_UL_TS_ChannelisationCode_VHCR(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     63, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t UL_TS_ChannelisationCodeList_VHCR_sequence_of[1] = {
  { &hf_rrc_UL_TS_ChannelisationCodeList_VHCR_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UL_TS_ChannelisationCode_VHCR },
};

static int
dissect_rrc_UL_TS_ChannelisationCodeList_VHCR(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_UL_TS_ChannelisationCodeList_VHCR, UL_TS_ChannelisationCodeList_VHCR_sequence_of,
                                                  1, 2, FALSE);

  return offset;
}


static const per_sequence_t T_consecutive_03_sequence[] = {
  { &hf_rrc_numAdditionalTimeslots, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_1_maxTS_1 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_consecutive_03(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_consecutive_03, T_consecutive_03_sequence);

  return offset;
}


static const per_sequence_t T_sameAsLast_07_sequence[] = {
  { &hf_rrc_timeslotNumber  , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TimeslotNumber },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_sameAsLast_07(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_sameAsLast_07, T_sameAsLast_07_sequence);

  return offset;
}


static const per_sequence_t T_newParameters_07_sequence[] = {
  { &hf_rrc_individualTimeslotInfo_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_IndividualTimeslotInfo_VHCR },
  { &hf_rrc_ul_TS_ChannelisationCodeList, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UL_TS_ChannelisationCodeList },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_newParameters_07(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_newParameters_07, T_newParameters_07_sequence);

  return offset;
}


static const value_string rrc_T_parameters_07_vals[] = {
  {   0, "sameAsLast" },
  {   1, "newParameters" },
  { 0, NULL }
};

static const per_choice_t T_parameters_07_choice[] = {
  {   0, &hf_rrc_sameAsLast_07   , ASN1_NO_EXTENSIONS     , dissect_rrc_T_sameAsLast_07 },
  {   1, &hf_rrc_newParameters_07, ASN1_NO_EXTENSIONS     , dissect_rrc_T_newParameters_07 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_parameters_07(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_parameters_07, T_parameters_07_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t UplinkAdditionalTimeslots_VHCR_sequence[] = {
  { &hf_rrc_parameters_07   , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_parameters_07 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UplinkAdditionalTimeslots_VHCR(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UplinkAdditionalTimeslots_VHCR, UplinkAdditionalTimeslots_VHCR_sequence);

  return offset;
}


static const per_sequence_t SEQUENCE_SIZE_1_maxTS_1_OF_UplinkAdditionalTimeslots_VHCR_sequence_of[1] = {
  { &hf_rrc_timeslotList_item_05, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UplinkAdditionalTimeslots_VHCR },
};

static int
dissect_rrc_SEQUENCE_SIZE_1_maxTS_1_OF_UplinkAdditionalTimeslots_VHCR(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_SEQUENCE_SIZE_1_maxTS_1_OF_UplinkAdditionalTimeslots_VHCR, SEQUENCE_SIZE_1_maxTS_1_OF_UplinkAdditionalTimeslots_VHCR_sequence_of,
                                                  1, maxTS_1, FALSE);

  return offset;
}


static const value_string rrc_T_additionalTimeslots_05_vals[] = {
  {   0, "consecutive" },
  {   1, "timeslotList" },
  { 0, NULL }
};

static const per_choice_t T_additionalTimeslots_05_choice[] = {
  {   0, &hf_rrc_consecutive_05  , ASN1_NO_EXTENSIONS     , dissect_rrc_T_consecutive_03 },
  {   1, &hf_rrc_timeslotList_05 , ASN1_NO_EXTENSIONS     , dissect_rrc_SEQUENCE_SIZE_1_maxTS_1_OF_UplinkAdditionalTimeslots_VHCR },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_additionalTimeslots_05(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_additionalTimeslots_05, T_additionalTimeslots_05_choice,
                                 NULL);

  return offset;
}


static const value_string rrc_T_moreTimeslots_05_vals[] = {
  {   0, "noMore" },
  {   1, "additionalTimeslots" },
  { 0, NULL }
};

static const per_choice_t T_moreTimeslots_05_choice[] = {
  {   0, &hf_rrc_noMore          , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   1, &hf_rrc_additionalTimeslots_05, ASN1_NO_EXTENSIONS     , dissect_rrc_T_additionalTimeslots_05 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_moreTimeslots_05(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_moreTimeslots_05, T_moreTimeslots_05_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t UplinkTimeslotsCodes_VHCR_sequence[] = {
  { &hf_rrc_dynamicSFusage  , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { &hf_rrc_firstIndividualTimeslotInfo_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_IndividualTimeslotInfo_VHCR },
  { &hf_rrc_ul_TS_ChannelisationCodeList_02, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UL_TS_ChannelisationCodeList_VHCR },
  { &hf_rrc_moreTimeslots_05, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_moreTimeslots_05 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UplinkTimeslotsCodes_VHCR(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UplinkTimeslotsCodes_VHCR, UplinkTimeslotsCodes_VHCR_sequence);

  return offset;
}


static const per_sequence_t T_tdd768_20_sequence[] = {
  { &hf_rrc_ul_CCTrCH_TimeslotsCodes_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UplinkTimeslotsCodes_VHCR },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_tdd768_20(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_tdd768_20, T_tdd768_20_sequence);

  return offset;
}



static int
dissect_rrc_INTEGER_1_14(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            1U, 14U, NULL, FALSE);

  return offset;
}



static int
dissect_rrc_TPC_CommandTargetRate(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            0U, 10U, NULL, FALSE);

  return offset;
}


static const per_sequence_t PLCCH_Info_sequence[] = {
  { &hf_rrc_plcchSequenceNumber, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_1_14 },
  { &hf_rrc_timeslotNumber_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TimeslotNumber_LCR_r4 },
  { &hf_rrc_channelisationCode_05, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DL_TS_ChannelisationCode },
  { &hf_rrc_tpcCommandTargetRate, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TPC_CommandTargetRate },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_PLCCH_Info(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_PLCCH_Info, PLCCH_Info_sequence);

  return offset;
}


static const per_sequence_t UL_TS_ChannelisationCodeList_r7_item_sequence[] = {
  { &hf_rrc_ul_TS_Channelisation_Code, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UL_TS_ChannelisationCode },
  { &hf_rrc_plcch_info      , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_PLCCH_Info },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UL_TS_ChannelisationCodeList_r7_item(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UL_TS_ChannelisationCodeList_r7_item, UL_TS_ChannelisationCodeList_r7_item_sequence);

  return offset;
}


static const per_sequence_t UL_TS_ChannelisationCodeList_r7_sequence_of[1] = {
  { &hf_rrc_UL_TS_ChannelisationCodeList_r7_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UL_TS_ChannelisationCodeList_r7_item },
};

static int
dissect_rrc_UL_TS_ChannelisationCodeList_r7(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_UL_TS_ChannelisationCodeList_r7, UL_TS_ChannelisationCodeList_r7_sequence_of,
                                                  1, 2, FALSE);

  return offset;
}


static const per_sequence_t T_consecutive_05_sequence[] = {
  { &hf_rrc_numAdditionalTimeslots_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_1_maxTS_LCR_1 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_consecutive_05(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_consecutive_05, T_consecutive_05_sequence);

  return offset;
}


static const per_sequence_t T_sameAsLast_06_sequence[] = {
  { &hf_rrc_timeslotNumber  , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TimeslotNumber },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_sameAsLast_06(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_sameAsLast_06, T_sameAsLast_06_sequence);

  return offset;
}


static const per_sequence_t T_newParameters_06_sequence[] = {
  { &hf_rrc_individualTimeslotInfo_02, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_IndividualTimeslotInfo_LCR_r4 },
  { &hf_rrc_ul_TS_ChannelisationCodeList_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UL_TS_ChannelisationCodeList_r7 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_newParameters_06(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_newParameters_06, T_newParameters_06_sequence);

  return offset;
}


static const value_string rrc_T_parameters_06_vals[] = {
  {   0, "sameAsLast" },
  {   1, "newParameters" },
  { 0, NULL }
};

static const per_choice_t T_parameters_06_choice[] = {
  {   0, &hf_rrc_sameAsLast_06   , ASN1_NO_EXTENSIONS     , dissect_rrc_T_sameAsLast_06 },
  {   1, &hf_rrc_newParameters_06, ASN1_NO_EXTENSIONS     , dissect_rrc_T_newParameters_06 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_parameters_06(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_parameters_06, T_parameters_06_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t UplinkAdditionalTimeslots_LCR_r7_sequence[] = {
  { &hf_rrc_parameters_06   , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_parameters_06 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UplinkAdditionalTimeslots_LCR_r7(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UplinkAdditionalTimeslots_LCR_r7, UplinkAdditionalTimeslots_LCR_r7_sequence);

  return offset;
}


static const per_sequence_t SEQUENCE_SIZE_1_maxTS_LCR_1_OF_UplinkAdditionalTimeslots_LCR_r7_sequence_of[1] = {
  { &hf_rrc_timeslotList_item_07, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UplinkAdditionalTimeslots_LCR_r7 },
};

static int
dissect_rrc_SEQUENCE_SIZE_1_maxTS_LCR_1_OF_UplinkAdditionalTimeslots_LCR_r7(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_SEQUENCE_SIZE_1_maxTS_LCR_1_OF_UplinkAdditionalTimeslots_LCR_r7, SEQUENCE_SIZE_1_maxTS_LCR_1_OF_UplinkAdditionalTimeslots_LCR_r7_sequence_of,
                                                  1, maxTS_LCR_1, FALSE);

  return offset;
}


static const value_string rrc_T_additionalTimeslots_07_vals[] = {
  {   0, "consecutive" },
  {   1, "timeslotList" },
  { 0, NULL }
};

static const per_choice_t T_additionalTimeslots_07_choice[] = {
  {   0, &hf_rrc_consecutive_07  , ASN1_NO_EXTENSIONS     , dissect_rrc_T_consecutive_05 },
  {   1, &hf_rrc_timeslotList_07 , ASN1_NO_EXTENSIONS     , dissect_rrc_SEQUENCE_SIZE_1_maxTS_LCR_1_OF_UplinkAdditionalTimeslots_LCR_r7 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_additionalTimeslots_07(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_additionalTimeslots_07, T_additionalTimeslots_07_choice,
                                 NULL);

  return offset;
}


static const value_string rrc_T_moreTimeslots_07_vals[] = {
  {   0, "noMore" },
  {   1, "additionalTimeslots" },
  { 0, NULL }
};

static const per_choice_t T_moreTimeslots_07_choice[] = {
  {   0, &hf_rrc_noMore          , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   1, &hf_rrc_additionalTimeslots_07, ASN1_NO_EXTENSIONS     , dissect_rrc_T_additionalTimeslots_07 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_moreTimeslots_07(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_moreTimeslots_07, T_moreTimeslots_07_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t UplinkTimeslotsCodes_LCR_r7_sequence[] = {
  { &hf_rrc_dynamicSFusage  , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { &hf_rrc_firstIndividualTimeslotInfo_02, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_IndividualTimeslotInfo_LCR_r4 },
  { &hf_rrc_ul_TS_ChannelisationCodeList_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UL_TS_ChannelisationCodeList_r7 },
  { &hf_rrc_moreTimeslots_07, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_moreTimeslots_07 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UplinkTimeslotsCodes_LCR_r7(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UplinkTimeslotsCodes_LCR_r7, UplinkTimeslotsCodes_LCR_r7_sequence);

  return offset;
}


static const per_sequence_t T_tdd128_54_sequence[] = {
  { &hf_rrc_ul_CCTrCH_TimeslotsCodes_03, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UplinkTimeslotsCodes_LCR_r7 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_tdd128_54(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_tdd128_54, T_tdd128_54_sequence);

  return offset;
}


static const value_string rrc_T_tddOption_20_vals[] = {
  {   0, "tdd384" },
  {   1, "tdd768" },
  {   2, "tdd128" },
  { 0, NULL }
};

static const per_choice_t T_tddOption_20_choice[] = {
  {   0, &hf_rrc_tdd384_39       , ASN1_NO_EXTENSIONS     , dissect_rrc_T_tdd384_35 },
  {   1, &hf_rrc_tdd768_23       , ASN1_NO_EXTENSIONS     , dissect_rrc_T_tdd768_20 },
  {   2, &hf_rrc_tdd128_56       , ASN1_NO_EXTENSIONS     , dissect_rrc_T_tdd128_54 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_tddOption_20(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_tddOption_20, T_tddOption_20_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t UL_CCTrCH_r7_sequence[] = {
  { &hf_rrc_tfcs_ID_01      , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_TFCS_IdentityPlain },
  { &hf_rrc_ul_TargetSIR    , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UL_TargetSIR },
  { &hf_rrc_timeInfo        , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TimeInfo },
  { &hf_rrc_commonTimeslotInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CommonTimeslotInfo },
  { &hf_rrc_tddOption_20    , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_tddOption_20 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UL_CCTrCH_r7(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UL_CCTrCH_r7, UL_CCTrCH_r7_sequence);

  return offset;
}


static const per_sequence_t UL_CCTrCHList_r7_sequence_of[1] = {
  { &hf_rrc_UL_CCTrCHList_r7_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UL_CCTrCH_r7 },
};

static int
dissect_rrc_UL_CCTrCHList_r7(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_UL_CCTrCHList_r7, UL_CCTrCHList_r7_sequence_of,
                                                  1, maxCCTrCH, FALSE);

  return offset;
}


static const per_sequence_t T_tdd_68_sequence[] = {
  { &hf_rrc_ul_TimingAdvance_03, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_TimingAdvanceControl_r7 },
  { &hf_rrc_ul_CCTrCHList_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_CCTrCHList_r7 },
  { &hf_rrc_ul_CCTrCHListToRemove, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_CCTrCHListToRemove },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_tdd_68(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_tdd_68, T_tdd_68_sequence);

  return offset;
}


static const value_string rrc_T_modeSpecificInfo_85_vals[] = {
  {   0, "fdd" },
  {   1, "tdd" },
  { 0, NULL }
};

static const per_choice_t T_modeSpecificInfo_85_choice[] = {
  {   0, &hf_rrc_fdd_120         , ASN1_NO_EXTENSIONS     , dissect_rrc_T_fdd_117 },
  {   1, &hf_rrc_tdd_74          , ASN1_NO_EXTENSIONS     , dissect_rrc_T_tdd_68 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_modeSpecificInfo_85(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_modeSpecificInfo_85, T_modeSpecificInfo_85_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t UL_DPCH_Info_r7_sequence[] = {
  { &hf_rrc_ul_DPCH_PowerControlInfo_03, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_DPCH_PowerControlInfo_r7 },
  { &hf_rrc_modeSpecificInfo_85, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_modeSpecificInfo_85 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UL_DPCH_Info_r7(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UL_DPCH_Info_r7, UL_DPCH_Info_r7_sequence);

  return offset;
}


static const value_string rrc_T_mac_es_e_resetIndicator_02_vals[] = {
  {   0, "true" },
  { 0, NULL }
};


static int
dissect_rrc_T_mac_es_e_resetIndicator_02(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t E_DPCCH_Info_r7_sequence[] = {
  { &hf_rrc_e_DPCCH_DPCCH_PowerOffset, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_E_DPCCH_DPCCH_PowerOffset },
  { &hf_rrc_happyBit_DelayCondition, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_HappyBit_DelayCondition },
  { &hf_rrc_e_TFC_Boost_Info, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_E_TFC_Boost_Info_r7 },
  { &hf_rrc_e_DPDCH_PowerInterpolation, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_E_DPDCH_PowerInterpolation },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_E_DPCCH_Info_r7(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_E_DPCCH_Info_r7, E_DPCCH_Info_r7_sequence);

  return offset;
}


static const per_sequence_t E_DPDCH_Info_r7_sequence[] = {
  { &hf_rrc_e_TFCI_TableIndex, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_E_TFCI_TableIndex },
  { &hf_rrc_e_DCH_MinimumSet_E_TFCI, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_E_DCH_MinimumSet_E_TFCI },
  { &hf_rrc_reference_E_TFCIs_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_E_DPDCH_Reference_E_TFCIList_r7 },
  { &hf_rrc_maxChannelisationCodes, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_E_DPDCH_MaxChannelisationCodes },
  { &hf_rrc_pl_NonMax       , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_E_DPDCH_PL_NonMax },
  { &hf_rrc_schedulingInfoConfiguration, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_E_DPDCH_SchedulingInfoConfiguration },
  { &hf_rrc_threeIndexStepThreshold, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_INTEGER_0_37 },
  { &hf_rrc_twoIndexStepThreshold, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_INTEGER_0_37 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_E_DPDCH_Info_r7(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_E_DPDCH_Info_r7, E_DPDCH_Info_r7_sequence);

  return offset;
}


static const per_sequence_t T_fdd_128_sequence[] = {
  { &hf_rrc_e_DPCCH_Info_01 , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_E_DPCCH_Info_r7 },
  { &hf_rrc_e_DPDCH_Info_01 , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_E_DPDCH_Info_r7 },
  { &hf_rrc_schedulingTransmConfiguration, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_E_DPDCH_SchedulingTransmConfiguration },
  { &hf_rrc_ul_16QAM_Settings, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_16QAM_Settings },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_fdd_128(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_fdd_128, T_fdd_128_sequence);

  return offset;
}



static int
dissect_rrc_INTEGER_M35_10(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            -35, 10U, NULL, FALSE);

  return offset;
}


static const value_string rrc_PersistenceScalingFactor_vals[] = {
  {   0, "psf0-9" },
  {   1, "psf0-8" },
  {   2, "psf0-7" },
  {   3, "psf0-6" },
  {   4, "psf0-5" },
  {   5, "psf0-4" },
  {   6, "psf0-3" },
  {   7, "psf0-2" },
  { 0, NULL }
};


static int
dissect_rrc_PersistenceScalingFactor(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     8, NULL, FALSE, 0, NULL);

  return offset;
}


static const value_string rrc_T_t_RUCCH_vals[] = {
  {   0, "ms40" },
  {   1, "ms80" },
  {   2, "ms120" },
  {   3, "ms160" },
  {   4, "ms200" },
  {   5, "ms240" },
  {   6, "ms280" },
  {   7, "ms320" },
  { 0, NULL }
};


static int
dissect_rrc_T_t_RUCCH(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     8, NULL, FALSE, 0, NULL);

  return offset;
}


static const value_string rrc_T_e_RUCCH_Midamble_vals[] = {
  {   0, "direct" },
  {   1, "direct-Inverted" },
  { 0, NULL }
};


static int
dissect_rrc_T_e_RUCCH_Midamble(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     2, NULL, FALSE, 0, NULL);

  return offset;
}


static const value_string rrc_T_t_adv_vals[] = {
  {   0, "sec1" },
  {   1, "sec2" },
  {   2, "sec4" },
  {   3, "sec8" },
  {   4, "sec16" },
  {   5, "sec32" },
  {   6, "sec128" },
  {   7, "infinity" },
  { 0, NULL }
};


static int
dissect_rrc_T_t_adv(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     8, NULL, FALSE, 0, NULL);

  return offset;
}


static const value_string rrc_T_t_SCHED_vals[] = {
  {   0, "zero" },
  {   1, "ms40" },
  {   2, "ms80" },
  {   3, "ms160" },
  {   4, "ms250" },
  {   5, "sec1" },
  {   6, "sec2" },
  {   7, "sec4" },
  { 0, NULL }
};


static int
dissect_rrc_T_t_SCHED(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     8, NULL, FALSE, 0, NULL);

  return offset;
}


static const value_string rrc_SF16Codes_vals[] = {
  {   0, "cc16-1" },
  {   1, "cc16-2" },
  {   2, "cc16-3" },
  {   3, "cc16-4" },
  {   4, "cc16-5" },
  {   5, "cc16-6" },
  {   6, "cc16-7" },
  {   7, "cc16-8" },
  { 0, NULL }
};


static int
dissect_rrc_SF16Codes(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     8, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t SEQUENCE_SIZE_1_8_OF_SF16Codes_sequence_of[1] = {
  { &hf_rrc_sF16_item       , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SF16Codes },
};

static int
dissect_rrc_SEQUENCE_SIZE_1_8_OF_SF16Codes(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_SEQUENCE_SIZE_1_8_OF_SF16Codes, SEQUENCE_SIZE_1_8_OF_SF16Codes_sequence_of,
                                                  1, 8, FALSE);

  return offset;
}


static const value_string rrc_SF8Codes_vals[] = {
  {   0, "cc8-1" },
  {   1, "cc8-2" },
  {   2, "cc8-3" },
  {   3, "cc8-4" },
  {   4, "cc8-5" },
  {   5, "cc8-6" },
  {   6, "cc8-7" },
  {   7, "cc8-8" },
  { 0, NULL }
};


static int
dissect_rrc_SF8Codes(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     8, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t SEQUENCE_SIZE_1_8_OF_SF8Codes_sequence_of[1] = {
  { &hf_rrc_sF8_item        , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SF8Codes },
};

static int
dissect_rrc_SEQUENCE_SIZE_1_8_OF_SF8Codes(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_SEQUENCE_SIZE_1_8_OF_SF8Codes, SEQUENCE_SIZE_1_8_OF_SF8Codes_sequence_of,
                                                  1, 8, FALSE);

  return offset;
}


static const value_string rrc_T_tdd384_21_vals[] = {
  {   0, "sF16" },
  {   1, "sF8" },
  { 0, NULL }
};

static const per_choice_t T_tdd384_21_choice[] = {
  {   0, &hf_rrc_sF16            , ASN1_NO_EXTENSIONS     , dissect_rrc_SEQUENCE_SIZE_1_8_OF_SF16Codes },
  {   1, &hf_rrc_sF8             , ASN1_NO_EXTENSIONS     , dissect_rrc_SEQUENCE_SIZE_1_8_OF_SF8Codes },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_tdd384_21(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_tdd384_21, T_tdd384_21_choice,
                                 NULL);

  return offset;
}


static const value_string rrc_SF32Codes_vals[] = {
  {   0, "cc32-1" },
  {   1, "cc32-2" },
  {   2, "cc32-3" },
  {   3, "cc32-4" },
  {   4, "cc32-5" },
  {   5, "cc32-6" },
  {   6, "cc32-7" },
  {   7, "cc32-8" },
  {   8, "cc32-9" },
  {   9, "cc32-10" },
  {  10, "cc32-11" },
  {  11, "cc32-12" },
  {  12, "cc32-13" },
  {  13, "cc32-14" },
  {  14, "cc32-15" },
  {  15, "cc32-16" },
  { 0, NULL }
};


static int
dissect_rrc_SF32Codes(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     16, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t SEQUENCE_SIZE_1_16_OF_SF32Codes_sequence_of[1] = {
  { &hf_rrc_sF32_item       , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SF32Codes },
};

static int
dissect_rrc_SEQUENCE_SIZE_1_16_OF_SF32Codes(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_SEQUENCE_SIZE_1_16_OF_SF32Codes, SEQUENCE_SIZE_1_16_OF_SF32Codes_sequence_of,
                                                  1, 16, FALSE);

  return offset;
}


static const value_string rrc_SF16Codes2_vals[] = {
  {   0, "cc16-1" },
  {   1, "cc16-2" },
  {   2, "cc16-3" },
  {   3, "cc16-4" },
  {   4, "cc16-5" },
  {   5, "cc16-6" },
  {   6, "cc16-7" },
  {   7, "cc16-8" },
  {   8, "cc16-9" },
  {   9, "cc16-10" },
  {  10, "cc16-11" },
  {  11, "cc16-12" },
  {  12, "cc16-13" },
  {  13, "cc16-14" },
  {  14, "cc16-15" },
  {  15, "cc16-16" },
  { 0, NULL }
};


static int
dissect_rrc_SF16Codes2(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     16, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t SEQUENCE_SIZE_1_16_OF_SF16Codes2_sequence_of[1] = {
  { &hf_rrc_sF816_item      , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SF16Codes2 },
};

static int
dissect_rrc_SEQUENCE_SIZE_1_16_OF_SF16Codes2(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_SEQUENCE_SIZE_1_16_OF_SF16Codes2, SEQUENCE_SIZE_1_16_OF_SF16Codes2_sequence_of,
                                                  1, 16, FALSE);

  return offset;
}


static const value_string rrc_T_tdd768_13_vals[] = {
  {   0, "sF32" },
  {   1, "sF816" },
  { 0, NULL }
};

static const per_choice_t T_tdd768_13_choice[] = {
  {   0, &hf_rrc_sF32            , ASN1_NO_EXTENSIONS     , dissect_rrc_SEQUENCE_SIZE_1_16_OF_SF32Codes },
  {   1, &hf_rrc_sF816           , ASN1_NO_EXTENSIONS     , dissect_rrc_SEQUENCE_SIZE_1_16_OF_SF16Codes2 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_tdd768_13(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_tdd768_13, T_tdd768_13_choice,
                                 NULL);

  return offset;
}


static const value_string rrc_T_modeSpecificInfo_59_vals[] = {
  {   0, "tdd384" },
  {   1, "tdd768" },
  { 0, NULL }
};

static const per_choice_t T_modeSpecificInfo_59_choice[] = {
  {   0, &hf_rrc_tdd384_23       , ASN1_NO_EXTENSIONS     , dissect_rrc_T_tdd384_21 },
  {   1, &hf_rrc_tdd768_15       , ASN1_NO_EXTENSIONS     , dissect_rrc_T_tdd768_13 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_modeSpecificInfo_59(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_modeSpecificInfo_59, T_modeSpecificInfo_59_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t T_tdd384_tdd768_03_sequence[] = {
  { &hf_rrc_e_RUCCH_ConstantValue, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_M35_10 },
  { &hf_rrc_e_RUCCH_PersistenceScalingFactor, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PersistenceScalingFactor },
  { &hf_rrc_t_RUCCH         , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_t_RUCCH },
  { &hf_rrc_e_RUCCH_TS_Number, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_INTEGER_0_14 },
  { &hf_rrc_e_RUCCH_Midamble, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_e_RUCCH_Midamble },
  { &hf_rrc_t_adv           , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_t_adv },
  { &hf_rrc_t_SCHED         , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_t_SCHED },
  { &hf_rrc_modeSpecificInfo_59, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_modeSpecificInfo_59 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_tdd384_tdd768_03(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_tdd384_tdd768_03, T_tdd384_tdd768_03_sequence);

  return offset;
}


static const value_string rrc_T_t_RUCCH_01_vals[] = {
  {   0, "ms20" },
  {   1, "ms40" },
  {   2, "ms60" },
  {   3, "ms80" },
  {   4, "ms120" },
  {   5, "ms160" },
  {   6, "ms200" },
  {   7, "ms240" },
  {   8, "ms280" },
  {   9, "ms320" },
  {  10, "ms400" },
  {  11, "ms500" },
  {  12, "ms600" },
  {  13, "ms800" },
  {  14, "ms1000" },
  {  15, "ms2000" },
  { 0, NULL }
};


static int
dissect_rrc_T_t_RUCCH_01(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     16, NULL, FALSE, 0, NULL);

  return offset;
}


static const value_string rrc_T_t_WAIT_vals[] = {
  {   0, "everyEDCHTTI" },
  {   1, "ms40" },
  {   2, "ms80" },
  {   3, "ms160" },
  {   4, "ms320" },
  {   5, "ms640" },
  {   6, "ms1000" },
  {   7, "ms2000" },
  { 0, NULL }
};


static int
dissect_rrc_T_t_WAIT(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     8, NULL, FALSE, 0, NULL);

  return offset;
}


static const value_string rrc_T_t_SI_vals[] = {
  {   0, "everyEDCHTTI" },
  {   1, "ms20" },
  {   2, "ms40" },
  {   3, "ms60" },
  {   4, "ms80" },
  {   5, "ms160" },
  {   6, "ms200" },
  { 0, NULL }
};


static int
dissect_rrc_T_t_SI(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     7, NULL, FALSE, 0, NULL);

  return offset;
}



static int
dissect_rrc_INTEGER_2_5(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            2U, 5U, NULL, FALSE);

  return offset;
}



static int
dissect_rrc_T_availableSYNC_UlCodesIndics(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_bit_string(tvb, offset, actx, tree, hf_index,
                                     8, 8, FALSE, NULL);

  return offset;
}


static const value_string rrc_T_subchannels_07_vals[] = {
  {   0, "subch0" },
  {   1, "subch1" },
  { 0, NULL }
};


static int
dissect_rrc_T_subchannels_07(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     2, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t T_size2_02_sequence[] = {
  { &hf_rrc_subchannels_07  , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_subchannels_07 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_size2_02(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_size2_02, T_size2_02_sequence);

  return offset;
}



static int
dissect_rrc_T_subchannels_08(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_bit_string(tvb, offset, actx, tree, hf_index,
                                     4, 4, FALSE, NULL);

  return offset;
}


static const per_sequence_t T_size4_02_sequence[] = {
  { &hf_rrc_subchannels_08  , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_subchannels_08 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_size4_02(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_size4_02, T_size4_02_sequence);

  return offset;
}



static int
dissect_rrc_T_subchannels_09(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_bit_string(tvb, offset, actx, tree, hf_index,
                                     8, 8, FALSE, NULL);

  return offset;
}


static const per_sequence_t T_size8_02_sequence[] = {
  { &hf_rrc_subchannels_09  , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_subchannels_09 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_size8_02(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_size8_02, T_size8_02_sequence);

  return offset;
}


static const value_string rrc_T_subchannelSize_02_vals[] = {
  {   0, "size1" },
  {   1, "size2" },
  {   2, "size4" },
  {   3, "size8" },
  { 0, NULL }
};

static const per_choice_t T_subchannelSize_02_choice[] = {
  {   0, &hf_rrc_size1           , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   1, &hf_rrc_size2_02        , ASN1_NO_EXTENSIONS     , dissect_rrc_T_size2_02 },
  {   2, &hf_rrc_size4_02        , ASN1_NO_EXTENSIONS     , dissect_rrc_T_size4_02 },
  {   3, &hf_rrc_size8_02        , ASN1_NO_EXTENSIONS     , dissect_rrc_T_size8_02 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_subchannelSize_02(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_subchannelSize_02, T_subchannelSize_02_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t AccessServiceClass_TDD_LCR_r4_sequence[] = {
  { &hf_rrc_availableSYNC_UlCodesIndics, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_availableSYNC_UlCodesIndics },
  { &hf_rrc_subchannelSize_02, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_subchannelSize_02 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_AccessServiceClass_TDD_LCR_r4(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_AccessServiceClass_TDD_LCR_r4, AccessServiceClass_TDD_LCR_r4_sequence);

  return offset;
}


static const per_sequence_t ASCSetting_TDD_LCR_r4_sequence[] = {
  { &hf_rrc_accessServiceClass_TDD_LCR, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_AccessServiceClass_TDD_LCR_r4 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_ASCSetting_TDD_LCR_r4(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_ASCSetting_TDD_LCR_r4, ASCSetting_TDD_LCR_r4_sequence);

  return offset;
}


static const per_sequence_t PRACH_Partitioning_LCR_r4_sequence_of[1] = {
  { &hf_rrc_PRACH_Partitioning_LCR_r4_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_ASCSetting_TDD_LCR_r4 },
};

static int
dissect_rrc_PRACH_Partitioning_LCR_r4(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_PRACH_Partitioning_LCR_r4, PRACH_Partitioning_LCR_r4_sequence_of,
                                                  1, maxASC, FALSE);

  return offset;
}



static int
dissect_rrc_E_RUCCH_AccessServiceClass(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_rrc_PRACH_Partitioning_LCR_r4(tvb, offset, actx, tree, hf_index);

  return offset;
}


static const per_sequence_t PersistenceScalingFactorList_sequence_of[1] = {
  { &hf_rrc_PersistenceScalingFactorList_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PersistenceScalingFactor },
};

static int
dissect_rrc_PersistenceScalingFactorList(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_PersistenceScalingFactorList, PersistenceScalingFactorList_sequence_of,
                                                  1, maxASCpersist, FALSE);

  return offset;
}


static const value_string rrc_T_max_SYNC_UL_Transmissions_02_vals[] = {
  {   0, "tr1" },
  {   1, "tr2" },
  {   2, "tr4" },
  {   3, "tr8" },
  { 0, NULL }
};


static int
dissect_rrc_T_max_SYNC_UL_Transmissions_02(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     4, NULL, FALSE, 0, NULL);

  return offset;
}



static int
dissect_rrc_Sync_UL_Codes_Bitmap(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_bit_string(tvb, offset, actx, tree, hf_index,
                                     8, 8, FALSE, NULL);

  return offset;
}



static int
dissect_rrc_INTEGER_1_32(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            1U, 32U, NULL, FALSE);

  return offset;
}


static const per_sequence_t SYNC_UL_InfoForE_RUCCH_sequence[] = {
  { &hf_rrc_prxUpPCHdes     , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_INTEGER_0_62 },
  { &hf_rrc_powerRampStep_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_INTEGER_0_3 },
  { &hf_rrc_max_SYNC_UL_Transmissions_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_max_SYNC_UL_Transmissions_02 },
  { &hf_rrc_e_RUCCH_Sync_UL_Codes_Bitmap, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_Sync_UL_Codes_Bitmap },
  { &hf_rrc_mmax            , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_1_32 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_SYNC_UL_InfoForE_RUCCH(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_SYNC_UL_InfoForE_RUCCH, SYNC_UL_InfoForE_RUCCH_sequence);

  return offset;
}



static int
dissect_rrc_TimeslotNumber_PRACH_LCR_r4(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            1U, 6U, NULL, FALSE);

  return offset;
}


static const value_string rrc_TDD_PRACH_CCode_LCR_r4_vals[] = {
  {   0, "cc4-1" },
  {   1, "cc4-2" },
  {   2, "cc4-3" },
  {   3, "cc4-4" },
  {   4, "cc8-1" },
  {   5, "cc8-2" },
  {   6, "cc8-3" },
  {   7, "cc8-4" },
  {   8, "cc8-5" },
  {   9, "cc8-6" },
  {  10, "cc8-7" },
  {  11, "cc8-8" },
  {  12, "cc16-1" },
  {  13, "cc16-2" },
  {  14, "cc16-3" },
  {  15, "cc16-4" },
  {  16, "cc16-5" },
  {  17, "cc16-6" },
  {  18, "cc16-7" },
  {  19, "cc16-8" },
  {  20, "cc16-9" },
  {  21, "cc16-10" },
  {  22, "cc16-11" },
  {  23, "cc16-12" },
  {  24, "cc16-13" },
  {  25, "cc16-14" },
  {  26, "cc16-15" },
  {  27, "cc16-16" },
  { 0, NULL }
};

static value_string_ext rrc_TDD_PRACH_CCode_LCR_r4_vals_ext = VALUE_STRING_EXT_INIT(rrc_TDD_PRACH_CCode_LCR_r4_vals);


static int
dissect_rrc_TDD_PRACH_CCode_LCR_r4(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     28, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t PRACH_ChanCodes_List_LCR_sequence_of[1] = {
  { &hf_rrc_PRACH_ChanCodes_List_LCR_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TDD_PRACH_CCode_LCR_r4 },
};

static int
dissect_rrc_PRACH_ChanCodes_List_LCR(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_PRACH_ChanCodes_List_LCR, PRACH_ChanCodes_List_LCR_sequence_of,
                                                  1, 2, FALSE);

  return offset;
}


static const per_sequence_t PRACH_Information_LCR_sequence[] = {
  { &hf_rrc_timeslot_02     , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TimeslotNumber_PRACH_LCR_r4 },
  { &hf_rrc_prach_ChanCodes_list_LCR, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PRACH_ChanCodes_List_LCR },
  { &hf_rrc_midambleShiftAndBurstType_03, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MidambleShiftAndBurstType_LCR_r4 },
  { &hf_rrc_fpach_Info      , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_FPACH_Info_r4 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_PRACH_Information_LCR(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_PRACH_Information_LCR, PRACH_Information_LCR_sequence);

  return offset;
}


static const per_sequence_t PRACH_Information_LCR_List_sequence_of[1] = {
  { &hf_rrc_PRACH_Information_LCR_List_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PRACH_Information_LCR },
};

static int
dissect_rrc_PRACH_Information_LCR_List(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_PRACH_Information_LCR_List, PRACH_Information_LCR_List_sequence_of,
                                                  1, maxPRACH_FPACH, FALSE);

  return offset;
}


static const per_sequence_t T_tdd128_34_sequence[] = {
  { &hf_rrc_t_RUCCH_01      , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_t_RUCCH_01 },
  { &hf_rrc_n_RUCCH         , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_INTEGER_0_7 },
  { &hf_rrc_t_WAIT          , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_t_WAIT },
  { &hf_rrc_t_SI            , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_t_SI },
  { &hf_rrc_extendedEstimationWindow, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_INTEGER_2_5 },
  { &hf_rrc_e_RUCCH_AccessServiceClass, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_E_RUCCH_AccessServiceClass },
  { &hf_rrc_e_RUCCH_PersistenceScalingFactor_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_PersistenceScalingFactorList },
  { &hf_rrc_sync_UL_InfoForE_RUCCH, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_SYNC_UL_InfoForE_RUCCH },
  { &hf_rrc_prach_information, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_PRACH_Information_LCR_List },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_tdd128_34(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_tdd128_34, T_tdd128_34_sequence);

  return offset;
}


static const value_string rrc_T_modeSpecificInfo_58_vals[] = {
  {   0, "tdd384-tdd768" },
  {   1, "tdd128" },
  { 0, NULL }
};

static const per_choice_t T_modeSpecificInfo_58_choice[] = {
  {   0, &hf_rrc_tdd384_tdd768_03, ASN1_NO_EXTENSIONS     , dissect_rrc_T_tdd384_tdd768_03 },
  {   1, &hf_rrc_tdd128_34       , ASN1_NO_EXTENSIONS     , dissect_rrc_T_tdd128_34 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_modeSpecificInfo_58(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_modeSpecificInfo_58, T_modeSpecificInfo_58_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t E_RUCCH_Info_sequence[] = {
  { &hf_rrc_modeSpecificInfo_58, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_modeSpecificInfo_58 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_E_RUCCH_Info(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_E_RUCCH_Info, E_RUCCH_Info_sequence);

  return offset;
}



static int
dissect_rrc_INTEGER_0_10(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            0U, 10U, NULL, FALSE);

  return offset;
}



static int
dissect_rrc_INTEGER_M15_16(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            -15, 16U, NULL, FALSE);

  return offset;
}


static const per_sequence_t Reference_Beta_QPSK_sequence[] = {
  { &hf_rrc_reference_Code_Rate, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_0_10 },
  { &hf_rrc_reference_Beta  , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_M15_16 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_Reference_Beta_QPSK(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_Reference_Beta_QPSK, Reference_Beta_QPSK_sequence);

  return offset;
}


static const per_sequence_t SEQUENCE_SIZE_1_8_OF_Reference_Beta_QPSK_sequence_of[1] = {
  { &hf_rrc_reference_Beta_QPSK_List_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_Reference_Beta_QPSK },
};

static int
dissect_rrc_SEQUENCE_SIZE_1_8_OF_Reference_Beta_QPSK(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_SEQUENCE_SIZE_1_8_OF_Reference_Beta_QPSK, SEQUENCE_SIZE_1_8_OF_Reference_Beta_QPSK_sequence_of,
                                                  1, 8, FALSE);

  return offset;
}


static const per_sequence_t Reference_Beta_16QAM_sequence[] = {
  { &hf_rrc_reference_Code_Rate, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_0_10 },
  { &hf_rrc_reference_Beta  , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_M15_16 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_Reference_Beta_16QAM(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_Reference_Beta_16QAM, Reference_Beta_16QAM_sequence);

  return offset;
}


static const per_sequence_t SEQUENCE_SIZE_1_8_OF_Reference_Beta_16QAM_sequence_of[1] = {
  { &hf_rrc_reference_Beta_16QAM_List_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_Reference_Beta_16QAM },
};

static int
dissect_rrc_SEQUENCE_SIZE_1_8_OF_Reference_Beta_16QAM(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_SEQUENCE_SIZE_1_8_OF_Reference_Beta_16QAM, SEQUENCE_SIZE_1_8_OF_Reference_Beta_16QAM_sequence_of,
                                                  1, 8, FALSE);

  return offset;
}


static const per_sequence_t E_TFCS_Info_sequence[] = {
  { &hf_rrc_reference_Beta_QPSK_List, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SEQUENCE_SIZE_1_8_OF_Reference_Beta_QPSK },
  { &hf_rrc_reference_Beta_16QAM_List, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SEQUENCE_SIZE_1_8_OF_Reference_Beta_16QAM },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_E_TFCS_Info(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_E_TFCS_Info, E_TFCS_Info_sequence);

  return offset;
}



static int
dissect_rrc_INTEGER_1_12(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            1U, 12U, NULL, FALSE);

  return offset;
}


static const value_string rrc_MidambleConfigurationBurstType1_vals[] = {
  {   0, "ms4" },
  {   1, "ms8" },
  {   2, "ms16" },
  { 0, NULL }
};


static int
dissect_rrc_MidambleConfigurationBurstType1(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     3, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t T_ueSpecificMidamble_21_sequence[] = {
  { &hf_rrc_midambleShift   , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MidambleShiftLong },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_ueSpecificMidamble_21(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_ueSpecificMidamble_21, T_ueSpecificMidamble_21_sequence);

  return offset;
}


static const value_string rrc_T_midambleAllocationMode_29_vals[] = {
  {   0, "defaultMidamble" },
  {   1, "commonMidamble" },
  {   2, "ueSpecificMidamble" },
  { 0, NULL }
};

static const per_choice_t T_midambleAllocationMode_29_choice[] = {
  {   0, &hf_rrc_defaultMidamble , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   1, &hf_rrc_commonMidamble  , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   2, &hf_rrc_ueSpecificMidamble_22, ASN1_NO_EXTENSIONS     , dissect_rrc_T_ueSpecificMidamble_21 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_midambleAllocationMode_29(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_midambleAllocationMode_29, T_midambleAllocationMode_29_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t T_type1_06_sequence[] = {
  { &hf_rrc_midambleConfigurationBurstType1, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MidambleConfigurationBurstType1 },
  { &hf_rrc_midambleAllocationMode_29, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_midambleAllocationMode_29 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_type1_06(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_type1_06, T_type1_06_sequence);

  return offset;
}


static const per_sequence_t T_ueSpecificMidamble_22_sequence[] = {
  { &hf_rrc_midambleShift_02, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MidambleShiftShort },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_ueSpecificMidamble_22(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_ueSpecificMidamble_22, T_ueSpecificMidamble_22_sequence);

  return offset;
}


static const value_string rrc_T_midambleAllocationMode_30_vals[] = {
  {   0, "defaultMidamble" },
  {   1, "commonMidamble" },
  {   2, "ueSpecificMidamble" },
  { 0, NULL }
};

static const per_choice_t T_midambleAllocationMode_30_choice[] = {
  {   0, &hf_rrc_defaultMidamble , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   1, &hf_rrc_commonMidamble  , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   2, &hf_rrc_ueSpecificMidamble_23, ASN1_NO_EXTENSIONS     , dissect_rrc_T_ueSpecificMidamble_22 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_midambleAllocationMode_30(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_midambleAllocationMode_30, T_midambleAllocationMode_30_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t T_type2_05_sequence[] = {
  { &hf_rrc_midambleConfigurationBurstType2, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MidambleConfigurationBurstType2 },
  { &hf_rrc_midambleAllocationMode_30, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_midambleAllocationMode_30 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_type2_05(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_type2_05, T_type2_05_sequence);

  return offset;
}


static const value_string rrc_T_burstType_05_vals[] = {
  {   0, "type1" },
  {   1, "type2" },
  { 0, NULL }
};

static const per_choice_t T_burstType_05_choice[] = {
  {   0, &hf_rrc_type1_06        , ASN1_NO_EXTENSIONS     , dissect_rrc_T_type1_06 },
  {   1, &hf_rrc_type2_05        , ASN1_NO_EXTENSIONS     , dissect_rrc_T_type2_05 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_burstType_05(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_burstType_05, T_burstType_05_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t MidambleShiftAndBurstType_EDCH_sequence[] = {
  { &hf_rrc_burstType_06    , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_burstType_05 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_MidambleShiftAndBurstType_EDCH(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_MidambleShiftAndBurstType_EDCH, MidambleShiftAndBurstType_EDCH_sequence);

  return offset;
}


static const per_sequence_t E_PUCH_TS_Slots_sequence[] = {
  { &hf_rrc_ts_Number       , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_0_14 },
  { &hf_rrc_midambleShiftAndBurstType_02, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MidambleShiftAndBurstType_EDCH },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_E_PUCH_TS_Slots(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_E_PUCH_TS_Slots, E_PUCH_TS_Slots_sequence);

  return offset;
}


static const per_sequence_t SEQUENCE_SIZE_1_maxTS_2_OF_E_PUCH_TS_Slots_sequence_of[1] = {
  { &hf_rrc_e_PUCH_TS_ConfigurationList_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_E_PUCH_TS_Slots },
};

static int
dissect_rrc_SEQUENCE_SIZE_1_maxTS_2_OF_E_PUCH_TS_Slots(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_SEQUENCE_SIZE_1_maxTS_2_OF_E_PUCH_TS_Slots, SEQUENCE_SIZE_1_maxTS_2_OF_E_PUCH_TS_Slots_sequence_of,
                                                  1, maxTS_2, FALSE);

  return offset;
}


static const per_sequence_t T_tdd348_tdd768_sequence[] = {
  { &hf_rrc_noSlotsForTFCIandTPC, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_1_12 },
  { &hf_rrc_e_PUCH_ContantValue, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_M35_10 },
  { &hf_rrc_e_PUCH_TS_ConfigurationList, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SEQUENCE_SIZE_1_maxTS_2_OF_E_PUCH_TS_Slots },
  { &hf_rrc_e_PUCH_CodeHopping, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { &hf_rrc_e_PUCH_TPC_Step_Size, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_1_3 },
  { &hf_rrc_minimum_Allowed_Code_Rate, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_0_63 },
  { &hf_rrc_maximum_Allowed_Code_Rate, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_0_63 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_tdd348_tdd768(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_tdd348_tdd768, T_tdd348_tdd768_sequence);

  return offset;
}


static const value_string rrc_T_snpl_ReportType_vals[] = {
  {   0, "type1" },
  {   1, "type2" },
  { 0, NULL }
};


static int
dissect_rrc_T_snpl_ReportType(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     2, NULL, FALSE, 0, NULL);

  return offset;
}



static int
dissect_rrc_INTEGER_M112_M50(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            -112, -50, NULL, FALSE);

  return offset;
}


static const value_string rrc_T_midambleAllocationMode_03_vals[] = {
  {   0, "defaultMidamble" },
  {   1, "ueSpecificMidamble" },
  { 0, NULL }
};

static const per_choice_t T_midambleAllocationMode_03_choice[] = {
  {   0, &hf_rrc_defaultMidamble , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   1, &hf_rrc_ueSpecificMidamble, ASN1_NO_EXTENSIONS     , dissect_rrc_INTEGER_0_15 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_midambleAllocationMode_03(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_midambleAllocationMode_03, T_midambleAllocationMode_03_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t E_PUCH_TS_Slots_LCR_sequence[] = {
  { &hf_rrc_timeslotNumber_02, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_1_5 },
  { &hf_rrc_midambleAllocationMode_03, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_midambleAllocationMode_03 },
  { &hf_rrc_midambleConfiguration, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_1_8 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_E_PUCH_TS_Slots_LCR(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_E_PUCH_TS_Slots_LCR, E_PUCH_TS_Slots_LCR_sequence);

  return offset;
}


static const per_sequence_t SEQUENCE_SIZE_1_maxTS_LCR_1_OF_E_PUCH_TS_Slots_LCR_sequence_of[1] = {
  { &hf_rrc_e_PUCH_TS_ConfigurationList_item_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_E_PUCH_TS_Slots_LCR },
};

static int
dissect_rrc_SEQUENCE_SIZE_1_maxTS_LCR_1_OF_E_PUCH_TS_Slots_LCR(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_SEQUENCE_SIZE_1_maxTS_LCR_1_OF_E_PUCH_TS_Slots_LCR, SEQUENCE_SIZE_1_maxTS_LCR_1_OF_E_PUCH_TS_Slots_LCR_sequence_of,
                                                  1, maxTS_LCR_1, FALSE);

  return offset;
}


static const value_string rrc_T_retransTimerForSchedInfo_vals[] = {
  {   0, "ms10" },
  {   1, "ms15" },
  {   2, "ms20" },
  {   3, "ms25" },
  {   4, "ms30" },
  {   5, "ms35" },
  {   6, "ms40" },
  {   7, "ms45" },
  {   8, "ms50" },
  {   9, "ms55" },
  {  10, "ms60" },
  {  11, "ms65" },
  {  12, "ms70" },
  {  13, "ms75" },
  {  14, "ms80" },
  {  15, "ms85" },
  {  16, "ms90" },
  {  17, "ms95" },
  {  18, "ms100" },
  {  19, "ms110" },
  {  20, "ms120" },
  {  21, "ms140" },
  {  22, "ms160" },
  {  23, "ms200" },
  {  24, "ms240" },
  {  25, "ms280" },
  {  26, "ms320" },
  {  27, "ms400" },
  {  28, "ms480" },
  {  29, "ms560" },
  { 0, NULL }
};

static value_string_ext rrc_T_retransTimerForSchedInfo_vals_ext = VALUE_STRING_EXT_INIT(rrc_T_retransTimerForSchedInfo_vals);


static int
dissect_rrc_T_retransTimerForSchedInfo(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     30, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t T_tdd128_33_sequence[] = {
  { &hf_rrc_snpl_ReportType , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_snpl_ReportType },
  { &hf_rrc_prxdes_base     , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_M112_M50 },
  { &hf_rrc_beaconPLEst_01  , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_BOOLEAN },
  { &hf_rrc_tpc_StepSize    , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TPC_StepSizeTDD },
  { &hf_rrc_pebase_PowerControlGAP, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_PowerControlGAP },
  { &hf_rrc_ul_SynchronisationParameters, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_SynchronisationParameters_r4 },
  { &hf_rrc_e_PUCH_TS_ConfigurationList_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SEQUENCE_SIZE_1_maxTS_LCR_1_OF_E_PUCH_TS_Slots_LCR },
  { &hf_rrc_minimum_Allowed_Code_Rate, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_0_63 },
  { &hf_rrc_maximum_Allowed_Code_Rate, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_0_63 },
  { &hf_rrc_maximumNumOfRetransSchedInfo, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_0_15 },
  { &hf_rrc_retransTimerForSchedInfo, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_retransTimerForSchedInfo },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_tdd128_33(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_tdd128_33, T_tdd128_33_sequence);

  return offset;
}


static const value_string rrc_T_modeSpecificInfo_57_vals[] = {
  {   0, "tdd348-tdd768" },
  {   1, "tdd128" },
  { 0, NULL }
};

static const per_choice_t T_modeSpecificInfo_57_choice[] = {
  {   0, &hf_rrc_tdd348_tdd768   , ASN1_NO_EXTENSIONS     , dissect_rrc_T_tdd348_tdd768 },
  {   1, &hf_rrc_tdd128_33       , ASN1_NO_EXTENSIONS     , dissect_rrc_T_tdd128_33 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_modeSpecificInfo_57(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_modeSpecificInfo_57, T_modeSpecificInfo_57_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t E_PUCH_Info_sequence[] = {
  { &hf_rrc_e_TFCS_Info     , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_E_TFCS_Info },
  { &hf_rrc_modeSpecificInfo_57, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_modeSpecificInfo_57 },
  { &hf_rrc_powerOffsetForSchedInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_INTEGER_0_6 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_E_PUCH_Info(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_E_PUCH_Info, E_PUCH_Info_sequence);

  return offset;
}


static const per_sequence_t T_tdd384_768_sequence[] = {
  { &hf_rrc_timeslotResourceRelatedInfo_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_13 },
  { &hf_rrc_powerResourceRelatedInfo, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_1_32 },
  { &hf_rrc_activationTime  , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_ActivationTime },
  { &hf_rrc_repetitionPeriodAndLength, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RepetitionPeriodAndLength },
  { &hf_rrc_codeResourceInfo, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UL_TS_ChannelisationCode },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_tdd384_768(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_tdd384_768, T_tdd384_768_sequence);

  return offset;
}



static int
dissect_rrc_INTEGER_4_15(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            4U, 15U, NULL, FALSE);

  return offset;
}


static const value_string rrc_T_midambleAllocationMode_31_vals[] = {
  {   0, "defaultMidamble" },
  {   1, "ueSpecificMidamble" },
  { 0, NULL }
};

static const per_choice_t T_midambleAllocationMode_31_choice[] = {
  {   0, &hf_rrc_defaultMidamble , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   1, &hf_rrc_ueSpecificMidamble, ASN1_NO_EXTENSIONS     , dissect_rrc_INTEGER_0_15 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_midambleAllocationMode_31(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_midambleAllocationMode_31, T_midambleAllocationMode_31_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t T_e_HICH_Info_06_sequence[] = {
  { &hf_rrc_timeslotNumber_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TimeslotNumber_LCR_r4 },
  { &hf_rrc_channelisation_Code, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_HS_ChannelisationCode_LCR },
  { &hf_rrc_midambleAllocationMode_31, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_midambleAllocationMode_31 },
  { &hf_rrc_midambleConfiguration, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_1_8 },
  { &hf_rrc_signatureSequenceGroupIndex, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_0_19 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_e_HICH_Info_06(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_e_HICH_Info_06, T_e_HICH_Info_06_sequence);

  return offset;
}


static const per_sequence_t T_tdd128_41_sequence[] = {
  { &hf_rrc_n_E_UCCH        , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_INTEGER_1_8 },
  { &hf_rrc_n_E_HICH_01     , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_INTEGER_4_15 },
  { &hf_rrc_timeslotResourceRelatedInfo, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_5 },
  { &hf_rrc_powerResourceRelatedInfo, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_1_32 },
  { &hf_rrc_activationTime  , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_ActivationTime },
  { &hf_rrc_sfnNum          , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_0_1 },
  { &hf_rrc_repetitionPeriodAndLength, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RepetitionPeriodAndLength },
  { &hf_rrc_codeResourceInfo, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UL_TS_ChannelisationCode },
  { &hf_rrc_e_HICH_Info_08  , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_e_HICH_Info_06 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_tdd128_41(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_tdd128_41, T_tdd128_41_sequence);

  return offset;
}


static const value_string rrc_Non_ScheduledTransGrantInfoTDD_vals[] = {
  {   0, "tdd384-768" },
  {   1, "tdd128" },
  { 0, NULL }
};

static const per_choice_t Non_ScheduledTransGrantInfoTDD_choice[] = {
  {   0, &hf_rrc_tdd384_768      , ASN1_NO_EXTENSIONS     , dissect_rrc_T_tdd384_768 },
  {   1, &hf_rrc_tdd128_42       , ASN1_NO_EXTENSIONS     , dissect_rrc_T_tdd128_41 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_Non_ScheduledTransGrantInfoTDD(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_Non_ScheduledTransGrantInfoTDD, Non_ScheduledTransGrantInfoTDD_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t T_tdd_79_sequence[] = {
  { &hf_rrc_e_RUCCH_Info_01 , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_E_RUCCH_Info },
  { &hf_rrc_e_PUCH_Info_01  , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_E_PUCH_Info },
  { &hf_rrc_non_ScheduledTransGrantInfo_03, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_Non_ScheduledTransGrantInfoTDD },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_tdd_79(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_tdd_79, T_tdd_79_sequence);

  return offset;
}


static const value_string rrc_T_modeSpecificInfo_89_vals[] = {
  {   0, "fdd" },
  {   1, "tdd" },
  { 0, NULL }
};

static const per_choice_t T_modeSpecificInfo_89_choice[] = {
  {   0, &hf_rrc_fdd_131         , ASN1_NO_EXTENSIONS     , dissect_rrc_T_fdd_128 },
  {   1, &hf_rrc_tdd_85          , ASN1_NO_EXTENSIONS     , dissect_rrc_T_tdd_79 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_modeSpecificInfo_89(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_modeSpecificInfo_89, T_modeSpecificInfo_89_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t UL_EDCH_Information_r7_sequence[] = {
  { &hf_rrc_mac_es_e_resetIndicator_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_mac_es_e_resetIndicator_02 },
  { &hf_rrc_modeSpecificInfo_89, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_modeSpecificInfo_89 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UL_EDCH_Information_r7(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UL_EDCH_Information_r7, UL_EDCH_Information_r7_sequence);

  return offset;
}


static const value_string rrc_T_dl_dpchInfoCommon_01_vals[] = {
  {   0, "dl-DPCH-InfoCommon" },
  {   1, "dl-FDPCH-InfoCommon" },
  { 0, NULL }
};

static const per_choice_t T_dl_dpchInfoCommon_01_choice[] = {
  {   0, &hf_rrc_dl_DPCH_InfoCommon_02, ASN1_NO_EXTENSIONS     , dissect_rrc_DL_DPCH_InfoCommon_r6 },
  {   1, &hf_rrc_dl_FDPCH_InfoCommon, ASN1_NO_EXTENSIONS     , dissect_rrc_DL_FDPCH_InfoCommon_r6 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_dl_dpchInfoCommon_01(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_dl_dpchInfoCommon_01, T_dl_dpchInfoCommon_01_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t T_fdd_62_sequence[] = {
  { &hf_rrc_defaultDPCH_OffsetValue, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DefaultDPCH_OffsetValueFDD },
  { &hf_rrc_dpch_CompressedModeInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DPCH_CompressedModeInfo },
  { &hf_rrc_tx_DiversityMode, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_TX_DiversityMode },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_fdd_62(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_fdd_62, T_fdd_62_sequence);

  return offset;
}


static const per_sequence_t T_tdd128_17_sequence[] = {
  { &hf_rrc_tstd_Indicator  , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_tdd128_17(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_tdd128_17, T_tdd128_17_sequence);

  return offset;
}


static const value_string rrc_T_tddOption_08_vals[] = {
  {   0, "tdd384" },
  {   1, "tdd768" },
  {   2, "tdd128" },
  { 0, NULL }
};

static const per_choice_t T_tddOption_08_choice[] = {
  {   0, &hf_rrc_tdd384_11       , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   1, &hf_rrc_tdd768_06       , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   2, &hf_rrc_tdd128_17       , ASN1_NO_EXTENSIONS     , dissect_rrc_T_tdd128_17 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_tddOption_08(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_tddOption_08, T_tddOption_08_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t T_tdd_24_sequence[] = {
  { &hf_rrc_tddOption_08    , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_tddOption_08 },
  { &hf_rrc_defaultDPCH_OffsetValue_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DefaultDPCH_OffsetValueTDD },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_tdd_24(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_tdd_24, T_tdd_24_sequence);

  return offset;
}


static const value_string rrc_T_modeSpecificInfo_32_vals[] = {
  {   0, "fdd" },
  {   1, "tdd" },
  { 0, NULL }
};

static const per_choice_t T_modeSpecificInfo_32_choice[] = {
  {   0, &hf_rrc_fdd_63          , ASN1_NO_EXTENSIONS     , dissect_rrc_T_fdd_62 },
  {   1, &hf_rrc_tdd_25          , ASN1_NO_EXTENSIONS     , dissect_rrc_T_tdd_24 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_modeSpecificInfo_32(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_modeSpecificInfo_32, T_modeSpecificInfo_32_choice,
                                 NULL);

  return offset;
}


static const value_string rrc_T_mac_hsResetIndicator_07_vals[] = {
  {   0, "true" },
  { 0, NULL }
};


static int
dissect_rrc_T_mac_hsResetIndicator_07(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}


static const value_string rrc_T_postVerificationPeriod_08_vals[] = {
  {   0, "true" },
  { 0, NULL }
};


static int
dissect_rrc_T_postVerificationPeriod_08(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t DL_CommonInformation_r7_sequence[] = {
  { &hf_rrc_dl_dpchInfoCommon_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_dl_dpchInfoCommon_01 },
  { &hf_rrc_modeSpecificInfo_32, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_modeSpecificInfo_32 },
  { &hf_rrc_mac_hsResetIndicator_07, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_mac_hsResetIndicator_07 },
  { &hf_rrc_postVerificationPeriod_08, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_postVerificationPeriod_08 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_DL_CommonInformation_r7(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_DL_CommonInformation_r7, DL_CommonInformation_r7_sequence);

  return offset;
}


static const per_sequence_t T_fdd_85_sequence[] = {
  { &hf_rrc_primaryCPICH_Info, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PrimaryCPICH_Info },
  { &hf_rrc_servingHSDSCH_RL_indicator, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { &hf_rrc_servingEDCH_RL_indicator, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_fdd_85(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_fdd_85, T_fdd_85_sequence);

  return offset;
}


static const value_string rrc_T_modeSpecificInfo_53_vals[] = {
  {   0, "fdd" },
  {   1, "tdd" },
  { 0, NULL }
};

static const per_choice_t T_modeSpecificInfo_53_choice[] = {
  {   0, &hf_rrc_fdd_86          , ASN1_NO_EXTENSIONS     , dissect_rrc_T_fdd_85 },
  {   1, &hf_rrc_tdd_47          , ASN1_NO_EXTENSIONS     , dissect_rrc_PrimaryCCPCH_Info_r4 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_modeSpecificInfo_53(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_modeSpecificInfo_53, T_modeSpecificInfo_53_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t T_fdd_74_sequence[] = {
  { &hf_rrc_pCPICH_UsageForChannelEst, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PCPICH_UsageForChannelEst },
  { &hf_rrc_dpch_FrameOffset, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DPCH_FrameOffset },
  { &hf_rrc_secondaryCPICH_Info, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_SecondaryCPICH_Info },
  { &hf_rrc_dl_ChannelisationCodeList, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DL_ChannelisationCodeList },
  { &hf_rrc_tpc_CombinationIndex, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TPC_CombinationIndex },
  { &hf_rrc_powerOffsetTPC_pdpdch, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_PowerOffsetTPC_pdpdch },
  { &hf_rrc_closedLoopTimingAdjMode, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_ClosedLoopTimingAdjMode },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_fdd_74(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_fdd_74, T_fdd_74_sequence);

  return offset;
}


static const per_sequence_t T_ueSpecificMidamble_13_sequence[] = {
  { &hf_rrc_midambleShift   , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MidambleShiftLong },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_ueSpecificMidamble_13(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_ueSpecificMidamble_13, T_ueSpecificMidamble_13_sequence);

  return offset;
}


static const value_string rrc_T_midambleAllocationMode_21_vals[] = {
  {   0, "defaultMidamble" },
  {   1, "commonMidamble" },
  {   2, "ueSpecificMidamble" },
  { 0, NULL }
};

static const per_choice_t T_midambleAllocationMode_21_choice[] = {
  {   0, &hf_rrc_defaultMidamble , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   1, &hf_rrc_commonMidamble  , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   2, &hf_rrc_ueSpecificMidamble_14, ASN1_NO_EXTENSIONS     , dissect_rrc_T_ueSpecificMidamble_13 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_midambleAllocationMode_21(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_midambleAllocationMode_21, T_midambleAllocationMode_21_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t T_type1_03_sequence[] = {
  { &hf_rrc_midambleConfigurationBurstType1and3, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MidambleConfigurationBurstType1and3 },
  { &hf_rrc_midambleAllocationMode_21, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_midambleAllocationMode_21 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_type1_03(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_type1_03, T_type1_03_sequence);

  return offset;
}


static const per_sequence_t T_ueSpecificMidamble_14_sequence[] = {
  { &hf_rrc_midambleShift_02, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MidambleShiftShort },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_ueSpecificMidamble_14(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_ueSpecificMidamble_14, T_ueSpecificMidamble_14_sequence);

  return offset;
}


static const value_string rrc_T_midambleAllocationMode_22_vals[] = {
  {   0, "defaultMidamble" },
  {   1, "commonMidamble" },
  {   2, "ueSpecificMidamble" },
  { 0, NULL }
};

static const per_choice_t T_midambleAllocationMode_22_choice[] = {
  {   0, &hf_rrc_defaultMidamble , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   1, &hf_rrc_commonMidamble  , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   2, &hf_rrc_ueSpecificMidamble_15, ASN1_NO_EXTENSIONS     , dissect_rrc_T_ueSpecificMidamble_14 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_midambleAllocationMode_22(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_midambleAllocationMode_22, T_midambleAllocationMode_22_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t T_type2_02_sequence[] = {
  { &hf_rrc_midambleConfigurationBurstType2, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MidambleConfigurationBurstType2 },
  { &hf_rrc_midambleAllocationMode_22, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_midambleAllocationMode_22 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_type2_02(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_type2_02, T_type2_02_sequence);

  return offset;
}


static const per_sequence_t T_ueSpecificMidamble_15_sequence[] = {
  { &hf_rrc_midambleShift   , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MidambleShiftLong },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_ueSpecificMidamble_15(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_ueSpecificMidamble_15, T_ueSpecificMidamble_15_sequence);

  return offset;
}


static const value_string rrc_T_midambleAllocationMode_23_vals[] = {
  {   0, "defaultMidamble" },
  {   1, "ueSpecificMidamble" },
  { 0, NULL }
};

static const per_choice_t T_midambleAllocationMode_23_choice[] = {
  {   0, &hf_rrc_defaultMidamble , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   1, &hf_rrc_ueSpecificMidamble_16, ASN1_NO_EXTENSIONS     , dissect_rrc_T_ueSpecificMidamble_15 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_midambleAllocationMode_23(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_midambleAllocationMode_23, T_midambleAllocationMode_23_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t T_type3_02_sequence[] = {
  { &hf_rrc_midambleConfigurationBurstType1and3, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MidambleConfigurationBurstType1and3 },
  { &hf_rrc_midambleAllocationMode_23, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_midambleAllocationMode_23 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_type3_02(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_type3_02, T_type3_02_sequence);

  return offset;
}


static const value_string rrc_T_burstType_02_vals[] = {
  {   0, "type1" },
  {   1, "type2" },
  {   2, "type3" },
  {   3, "mbsfnBurstType4" },
  { 0, NULL }
};

static const per_choice_t T_burstType_02_choice[] = {
  {   0, &hf_rrc_type1_03        , ASN1_NO_EXTENSIONS     , dissect_rrc_T_type1_03 },
  {   1, &hf_rrc_type2_02        , ASN1_NO_EXTENSIONS     , dissect_rrc_T_type2_02 },
  {   2, &hf_rrc_type3_02        , ASN1_NO_EXTENSIONS     , dissect_rrc_T_type3_02 },
  {   3, &hf_rrc_mbsfnBurstType4 , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_burstType_02(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_burstType_02, T_burstType_02_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t MidambleShiftAndBurstType_r7_sequence[] = {
  { &hf_rrc_burstType_03    , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_burstType_02 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_MidambleShiftAndBurstType_r7(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_MidambleShiftAndBurstType_r7, MidambleShiftAndBurstType_r7_sequence);

  return offset;
}


static const per_sequence_t IndividualTimeslotInfo_r7_sequence[] = {
  { &hf_rrc_timeslotNumber  , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TimeslotNumber },
  { &hf_rrc_tfci_Existence  , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { &hf_rrc_midambleShiftAndBurstType_05, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MidambleShiftAndBurstType_r7 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_IndividualTimeslotInfo_r7(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_IndividualTimeslotInfo_r7, IndividualTimeslotInfo_r7_sequence);

  return offset;
}


static const per_sequence_t T_sameAsLast_03_sequence[] = {
  { &hf_rrc_timeslotNumber  , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TimeslotNumber },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_sameAsLast_03(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_sameAsLast_03, T_sameAsLast_03_sequence);

  return offset;
}


static const per_sequence_t T_newParameters_03_sequence[] = {
  { &hf_rrc_individualTimeslotInfo_03, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_IndividualTimeslotInfo_r7 },
  { &hf_rrc_dl_TS_ChannelisationCodesShort, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DL_TS_ChannelisationCodesShort },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_newParameters_03(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_newParameters_03, T_newParameters_03_sequence);

  return offset;
}


static const value_string rrc_T_parameters_03_vals[] = {
  {   0, "sameAsLast" },
  {   1, "newParameters" },
  { 0, NULL }
};

static const per_choice_t T_parameters_03_choice[] = {
  {   0, &hf_rrc_sameAsLast_03   , ASN1_NO_EXTENSIONS     , dissect_rrc_T_sameAsLast_03 },
  {   1, &hf_rrc_newParameters_03, ASN1_NO_EXTENSIONS     , dissect_rrc_T_newParameters_03 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_parameters_03(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_parameters_03, T_parameters_03_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t DownlinkAdditionalTimeslots_r7_sequence[] = {
  { &hf_rrc_parameters_03   , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_parameters_03 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_DownlinkAdditionalTimeslots_r7(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_DownlinkAdditionalTimeslots_r7, DownlinkAdditionalTimeslots_r7_sequence);

  return offset;
}


static const per_sequence_t SEQUENCE_SIZE_1_maxTS_1_OF_DownlinkAdditionalTimeslots_r7_sequence_of[1] = {
  { &hf_rrc_timeslotList_item_03, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DownlinkAdditionalTimeslots_r7 },
};

static int
dissect_rrc_SEQUENCE_SIZE_1_maxTS_1_OF_DownlinkAdditionalTimeslots_r7(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_SEQUENCE_SIZE_1_maxTS_1_OF_DownlinkAdditionalTimeslots_r7, SEQUENCE_SIZE_1_maxTS_1_OF_DownlinkAdditionalTimeslots_r7_sequence_of,
                                                  1, maxTS_1, FALSE);

  return offset;
}


static const value_string rrc_T_additionalTimeslots_03_vals[] = {
  {   0, "consecutive" },
  {   1, "timeslotList" },
  { 0, NULL }
};

static const per_choice_t T_additionalTimeslots_03_choice[] = {
  {   0, &hf_rrc_consecutive_02  , ASN1_NO_EXTENSIONS     , dissect_rrc_INTEGER_1_maxTS_1 },
  {   1, &hf_rrc_timeslotList_03 , ASN1_NO_EXTENSIONS     , dissect_rrc_SEQUENCE_SIZE_1_maxTS_1_OF_DownlinkAdditionalTimeslots_r7 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_additionalTimeslots_03(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_additionalTimeslots_03, T_additionalTimeslots_03_choice,
                                 NULL);

  return offset;
}


static const value_string rrc_T_moreTimeslots_03_vals[] = {
  {   0, "noMore" },
  {   1, "additionalTimeslots" },
  { 0, NULL }
};

static const per_choice_t T_moreTimeslots_03_choice[] = {
  {   0, &hf_rrc_noMore          , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   1, &hf_rrc_additionalTimeslots_03, ASN1_NO_EXTENSIONS     , dissect_rrc_T_additionalTimeslots_03 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_moreTimeslots_03(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_moreTimeslots_03, T_moreTimeslots_03_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t DownlinkTimeslotsCodes_r7_sequence[] = {
  { &hf_rrc_firstIndividualTimeslotInfo_03, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_IndividualTimeslotInfo_r7 },
  { &hf_rrc_dl_TS_ChannelisationCodesShort, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DL_TS_ChannelisationCodesShort },
  { &hf_rrc_moreTimeslots_03, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_moreTimeslots_03 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_DownlinkTimeslotsCodes_r7(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_DownlinkTimeslotsCodes_r7, DownlinkTimeslotsCodes_r7_sequence);

  return offset;
}


static const per_sequence_t T_tdd384_12_sequence[] = {
  { &hf_rrc_dl_CCTrCH_TimeslotsCodes_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DownlinkTimeslotsCodes_r7 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_tdd384_12(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_tdd384_12, T_tdd384_12_sequence);

  return offset;
}


static const per_sequence_t T_consecutive_01_sequence[] = {
  { &hf_rrc_firstChannelisationCode_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DL_TS_ChannelisationCode_VHCR },
  { &hf_rrc_lastChannelisationCode_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DL_TS_ChannelisationCode_VHCR },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_consecutive_01(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_consecutive_01, T_consecutive_01_sequence);

  return offset;
}



static int
dissect_rrc_T_bitmap_01(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_bit_string(tvb, offset, actx, tree, hf_index,
                                     32, 32, FALSE, NULL);

  return offset;
}


static const value_string rrc_T_codesRepresentation_01_vals[] = {
  {   0, "consecutive" },
  {   1, "bitmap" },
  { 0, NULL }
};

static const per_choice_t T_codesRepresentation_01_choice[] = {
  {   0, &hf_rrc_consecutive_01  , ASN1_NO_EXTENSIONS     , dissect_rrc_T_consecutive_01 },
  {   1, &hf_rrc_bitmap_01       , ASN1_NO_EXTENSIONS     , dissect_rrc_T_bitmap_01 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_codesRepresentation_01(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_codesRepresentation_01, T_codesRepresentation_01_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t DL_TS_ChannelisationCodesShort_VHCR_sequence[] = {
  { &hf_rrc_codesRepresentation_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_codesRepresentation_01 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_DL_TS_ChannelisationCodesShort_VHCR(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_DL_TS_ChannelisationCodesShort_VHCR, DL_TS_ChannelisationCodesShort_VHCR_sequence);

  return offset;
}


static const per_sequence_t T_sameAsLast_01_sequence[] = {
  { &hf_rrc_timeslotNumber  , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TimeslotNumber },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_sameAsLast_01(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_sameAsLast_01, T_sameAsLast_01_sequence);

  return offset;
}


static const per_sequence_t T_newParameters_01_sequence[] = {
  { &hf_rrc_individualTimeslotInfo_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_IndividualTimeslotInfo_VHCR },
  { &hf_rrc_dl_TS_ChannelisationCodesShort_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DL_TS_ChannelisationCodesShort_VHCR },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_newParameters_01(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_newParameters_01, T_newParameters_01_sequence);

  return offset;
}


static const value_string rrc_T_parameters_01_vals[] = {
  {   0, "sameAsLast" },
  {   1, "newParameters" },
  { 0, NULL }
};

static const per_choice_t T_parameters_01_choice[] = {
  {   0, &hf_rrc_sameAsLast_01   , ASN1_NO_EXTENSIONS     , dissect_rrc_T_sameAsLast_01 },
  {   1, &hf_rrc_newParameters_01, ASN1_NO_EXTENSIONS     , dissect_rrc_T_newParameters_01 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_parameters_01(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_parameters_01, T_parameters_01_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t DownlinkAdditionalTimeslots_VHCR_sequence[] = {
  { &hf_rrc_parameters_01   , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_parameters_01 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_DownlinkAdditionalTimeslots_VHCR(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_DownlinkAdditionalTimeslots_VHCR, DownlinkAdditionalTimeslots_VHCR_sequence);

  return offset;
}


static const per_sequence_t SEQUENCE_SIZE_1_maxTS_1_OF_DownlinkAdditionalTimeslots_VHCR_sequence_of[1] = {
  { &hf_rrc_timeslotList_item_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DownlinkAdditionalTimeslots_VHCR },
};

static int
dissect_rrc_SEQUENCE_SIZE_1_maxTS_1_OF_DownlinkAdditionalTimeslots_VHCR(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_SEQUENCE_SIZE_1_maxTS_1_OF_DownlinkAdditionalTimeslots_VHCR, SEQUENCE_SIZE_1_maxTS_1_OF_DownlinkAdditionalTimeslots_VHCR_sequence_of,
                                                  1, maxTS_1, FALSE);

  return offset;
}


static const value_string rrc_T_additionalTimeslots_01_vals[] = {
  {   0, "consecutive" },
  {   1, "timeslotList" },
  { 0, NULL }
};

static const per_choice_t T_additionalTimeslots_01_choice[] = {
  {   0, &hf_rrc_consecutive_02  , ASN1_NO_EXTENSIONS     , dissect_rrc_INTEGER_1_maxTS_1 },
  {   1, &hf_rrc_timeslotList_01 , ASN1_NO_EXTENSIONS     , dissect_rrc_SEQUENCE_SIZE_1_maxTS_1_OF_DownlinkAdditionalTimeslots_VHCR },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_additionalTimeslots_01(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_additionalTimeslots_01, T_additionalTimeslots_01_choice,
                                 NULL);

  return offset;
}


static const value_string rrc_T_moreTimeslots_01_vals[] = {
  {   0, "noMore" },
  {   1, "additionalTimeslots" },
  { 0, NULL }
};

static const per_choice_t T_moreTimeslots_01_choice[] = {
  {   0, &hf_rrc_noMore          , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   1, &hf_rrc_additionalTimeslots_01, ASN1_NO_EXTENSIONS     , dissect_rrc_T_additionalTimeslots_01 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_moreTimeslots_01(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_moreTimeslots_01, T_moreTimeslots_01_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t DownlinkTimeslotsCodes_VHCR_sequence[] = {
  { &hf_rrc_firstIndividualTimeslotInfo_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_IndividualTimeslotInfo_VHCR },
  { &hf_rrc_dl_TS_ChannelisationCodesShort_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DL_TS_ChannelisationCodesShort_VHCR },
  { &hf_rrc_moreTimeslots_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_moreTimeslots_01 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_DownlinkTimeslotsCodes_VHCR(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_DownlinkTimeslotsCodes_VHCR, DownlinkTimeslotsCodes_VHCR_sequence);

  return offset;
}


static const per_sequence_t T_tdd768_06_sequence[] = {
  { &hf_rrc_dl_CCTrCH_TimeslotsCodes_03, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DownlinkTimeslotsCodes_VHCR },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_tdd768_06(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_tdd768_06, T_tdd768_06_sequence);

  return offset;
}


static const per_sequence_t T_tdd128_13_sequence[] = {
  { &hf_rrc_dl_CCTrCH_TimeslotsCodes_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DownlinkTimeslotsCodes_LCR_r4 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_tdd128_13(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_tdd128_13, T_tdd128_13_sequence);

  return offset;
}


static const value_string rrc_T_tddOption_04_vals[] = {
  {   0, "tdd384" },
  {   1, "tdd768" },
  {   2, "tdd128" },
  { 0, NULL }
};

static const per_choice_t T_tddOption_04_choice[] = {
  {   0, &hf_rrc_tdd384_13       , ASN1_NO_EXTENSIONS     , dissect_rrc_T_tdd384_12 },
  {   1, &hf_rrc_tdd768_07       , ASN1_NO_EXTENSIONS     , dissect_rrc_T_tdd768_06 },
  {   2, &hf_rrc_tdd128_13       , ASN1_NO_EXTENSIONS     , dissect_rrc_T_tdd128_13 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_tddOption_04(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_tddOption_04, T_tddOption_04_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t DL_CCTrCh_r7_sequence[] = {
  { &hf_rrc_tfcs_ID_01      , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_TFCS_IdentityPlain },
  { &hf_rrc_timeInfo        , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TimeInfo },
  { &hf_rrc_commonTimeslotInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CommonTimeslotInfo },
  { &hf_rrc_tddOption_04    , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_tddOption_04 },
  { &hf_rrc_ul_CCTrChTPCList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_CCTrChTPCList },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_DL_CCTrCh_r7(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_DL_CCTrCh_r7, DL_CCTrCh_r7_sequence);

  return offset;
}


static const per_sequence_t DL_CCTrChList_r7_sequence_of[1] = {
  { &hf_rrc_DL_CCTrChList_r7_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DL_CCTrCh_r7 },
};

static int
dissect_rrc_DL_CCTrChList_r7(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_DL_CCTrChList_r7, DL_CCTrChList_r7_sequence_of,
                                                  1, maxCCTrCH, FALSE);

  return offset;
}


static const per_sequence_t T_tdd_36_sequence[] = {
  { &hf_rrc_dl_CCTrChListToEstablish_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_CCTrChList_r7 },
  { &hf_rrc_dl_CCTrChListToRemove, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_CCTrChListToRemove },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_tdd_36(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_tdd_36, T_tdd_36_sequence);

  return offset;
}


static const value_string rrc_DL_DPCH_InfoPerRL_r7_vals[] = {
  {   0, "fdd" },
  {   1, "tdd" },
  { 0, NULL }
};

static const per_choice_t DL_DPCH_InfoPerRL_r7_choice[] = {
  {   0, &hf_rrc_fdd_75          , ASN1_NO_EXTENSIONS     , dissect_rrc_T_fdd_74 },
  {   1, &hf_rrc_tdd_37          , ASN1_NO_EXTENSIONS     , dissect_rrc_T_tdd_36 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_DL_DPCH_InfoPerRL_r7(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_DL_DPCH_InfoPerRL_r7, DL_DPCH_InfoPerRL_r7_choice,
                                 NULL);

  return offset;
}


static const value_string rrc_T_dl_dpchInfo_01_vals[] = {
  {   0, "dl-DPCH-InfoPerRL" },
  {   1, "dl-FDPCH-InfoPerRL" },
  { 0, NULL }
};

static const per_choice_t T_dl_dpchInfo_01_choice[] = {
  {   0, &hf_rrc_dl_DPCH_InfoPerRL_04, ASN1_NO_EXTENSIONS     , dissect_rrc_DL_DPCH_InfoPerRL_r7 },
  {   1, &hf_rrc_dl_FDPCH_InfoPerRL_01, ASN1_NO_EXTENSIONS     , dissect_rrc_DL_FDPCH_InfoPerRL_r7 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_dl_dpchInfo_01(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_dl_dpchInfo_01, T_dl_dpchInfo_01_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t T_fdd_91_sequence[] = {
  { &hf_rrc_e_AGCH_ChannelisationCode, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_E_AGCH_ChannelisationCode },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_fdd_91(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_fdd_91, T_fdd_91_sequence);

  return offset;
}


static const per_sequence_t E_AGCH_Individual_sequence[] = {
  { &hf_rrc_tS_number       , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_0_14 },
  { &hf_rrc_channelisation_code, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DL_TS_ChannelisationCode },
  { &hf_rrc_midambleShiftAndBurstType_02, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MidambleShiftAndBurstType_EDCH },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_E_AGCH_Individual(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_E_AGCH_Individual, E_AGCH_Individual_sequence);

  return offset;
}


static const per_sequence_t E_AGCH_Set_Config_sequence_of[1] = {
  { &hf_rrc_E_AGCH_Set_Config_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_E_AGCH_Individual },
};

static int
dissect_rrc_E_AGCH_Set_Config(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_E_AGCH_Set_Config, E_AGCH_Set_Config_sequence_of,
                                                  1, maxNumE_AGCH, FALSE);

  return offset;
}


static const per_sequence_t T_tdd384_19_sequence[] = {
  { &hf_rrc_long_Term_Grant_Indicator, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_BOOLEAN },
  { &hf_rrc_length_of_TTRI_field, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_1_12 },
  { &hf_rrc_e_AGCH_Set_Config, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_E_AGCH_Set_Config },
  { &hf_rrc_e_AGCH_BLER_Target, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_Bler_Target },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_tdd384_19(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_tdd384_19, T_tdd384_19_sequence);

  return offset;
}


static const per_sequence_t E_AGCH_Individual_VHCR_sequence[] = {
  { &hf_rrc_tS_number       , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_0_14 },
  { &hf_rrc_channelisation_code_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DL_TS_ChannelisationCode_VHCR },
  { &hf_rrc_midambleShiftAndBurstType_02, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MidambleShiftAndBurstType_EDCH },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_E_AGCH_Individual_VHCR(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_E_AGCH_Individual_VHCR, E_AGCH_Individual_VHCR_sequence);

  return offset;
}


static const per_sequence_t E_AGCH_Set_Config_VHCR_sequence_of[1] = {
  { &hf_rrc_E_AGCH_Set_Config_VHCR_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_E_AGCH_Individual_VHCR },
};

static int
dissect_rrc_E_AGCH_Set_Config_VHCR(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_E_AGCH_Set_Config_VHCR, E_AGCH_Set_Config_VHCR_sequence_of,
                                                  1, maxNumE_AGCH, FALSE);

  return offset;
}


static const per_sequence_t T_tdd768_11_sequence[] = {
  { &hf_rrc_long_Term_Grant_Indicator, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_BOOLEAN },
  { &hf_rrc_length_of_TTRI_field, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_1_12 },
  { &hf_rrc_e_AGCH_Set_Config_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_E_AGCH_Set_Config_VHCR },
  { &hf_rrc_e_AGCH_BLER_Target, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_Bler_Target },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_tdd768_11(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_tdd768_11, T_tdd768_11_sequence);

  return offset;
}


static const value_string rrc_T_midambleAllocationMode_vals[] = {
  {   0, "defaultMidamble" },
  {   1, "commonMidamble" },
  {   2, "ueSpecificMidamble" },
  { 0, NULL }
};

static const per_choice_t T_midambleAllocationMode_choice[] = {
  {   0, &hf_rrc_defaultMidamble , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   1, &hf_rrc_commonMidamble  , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   2, &hf_rrc_ueSpecificMidamble, ASN1_NO_EXTENSIONS     , dissect_rrc_INTEGER_0_15 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_midambleAllocationMode(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_midambleAllocationMode, T_midambleAllocationMode_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t E_AGCH_Individual_LCR_sequence[] = {
  { &hf_rrc_timeslotNumber_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TimeslotNumber_LCR_r4 },
  { &hf_rrc_firstChannelisationCode_03, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_HS_ChannelisationCode_LCR },
  { &hf_rrc_secondChannelisationCode, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_HS_ChannelisationCode_LCR },
  { &hf_rrc_midambleAllocationMode, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_midambleAllocationMode },
  { &hf_rrc_midambleConfiguration, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_1_8 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_E_AGCH_Individual_LCR(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_E_AGCH_Individual_LCR, E_AGCH_Individual_LCR_sequence);

  return offset;
}


static const per_sequence_t E_AGCH_Set_Config_LCR_sequence_of[1] = {
  { &hf_rrc_E_AGCH_Set_Config_LCR_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_E_AGCH_Individual_LCR },
};

static int
dissect_rrc_E_AGCH_Set_Config_LCR(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_E_AGCH_Set_Config_LCR, E_AGCH_Set_Config_LCR_sequence_of,
                                                  1, maxNumE_AGCH, FALSE);

  return offset;
}


static const per_sequence_t T_tdd128_31_sequence[] = {
  { &hf_rrc_rdi_Indicator   , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { &hf_rrc_tpc_StepSize    , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TPC_StepSizeTDD },
  { &hf_rrc_e_AGCH_Set_Config_02, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_E_AGCH_Set_Config_LCR },
  { &hf_rrc_e_AGCH_BLER_Target, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_Bler_Target },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_tdd128_31(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_tdd128_31, T_tdd128_31_sequence);

  return offset;
}


static const value_string rrc_T_tdd_48_vals[] = {
  {   0, "tdd384" },
  {   1, "tdd768" },
  {   2, "tdd128" },
  { 0, NULL }
};

static const per_choice_t T_tdd_48_choice[] = {
  {   0, &hf_rrc_tdd384_20       , ASN1_NO_EXTENSIONS     , dissect_rrc_T_tdd384_19 },
  {   1, &hf_rrc_tdd768_12       , ASN1_NO_EXTENSIONS     , dissect_rrc_T_tdd768_11 },
  {   2, &hf_rrc_tdd128_31       , ASN1_NO_EXTENSIONS     , dissect_rrc_T_tdd128_31 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_tdd_48(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_tdd_48, T_tdd_48_choice,
                                 NULL);

  return offset;
}


static const value_string rrc_T_modeSpecific_02_vals[] = {
  {   0, "fdd" },
  {   1, "tdd" },
  { 0, NULL }
};

static const per_choice_t T_modeSpecific_02_choice[] = {
  {   0, &hf_rrc_fdd_92          , ASN1_NO_EXTENSIONS     , dissect_rrc_T_fdd_91 },
  {   1, &hf_rrc_tdd_51          , ASN1_NO_EXTENSIONS     , dissect_rrc_T_tdd_48 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_modeSpecific_02(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_modeSpecific_02, T_modeSpecific_02_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t E_AGCH_Information_r7_sequence[] = {
  { &hf_rrc_modeSpecific_02 , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_modeSpecific_02 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_E_AGCH_Information_r7(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_E_AGCH_Information_r7, E_AGCH_Information_r7_sequence);

  return offset;
}


static const value_string rrc_T_e_HICH_Info_01_vals[] = {
  {   0, "e-HICH-Information" },
  {   1, "releaseIndicator" },
  { 0, NULL }
};

static const per_choice_t T_e_HICH_Info_01_choice[] = {
  {   0, &hf_rrc_e_HICH_Information, ASN1_NO_EXTENSIONS     , dissect_rrc_E_HICH_Information },
  {   1, &hf_rrc_releaseIndicator, ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_e_HICH_Info_01(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_e_HICH_Info_01, T_e_HICH_Info_01_choice,
                                 NULL);

  return offset;
}


static const value_string rrc_T_e_RGCH_Info_02_vals[] = {
  {   0, "e-RGCH-Information" },
  {   1, "releaseIndicator" },
  { 0, NULL }
};

static const per_choice_t T_e_RGCH_Info_02_choice[] = {
  {   0, &hf_rrc_e_RGCH_Information, ASN1_NO_EXTENSIONS     , dissect_rrc_E_RGCH_Information },
  {   1, &hf_rrc_releaseIndicator, ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_e_RGCH_Info_02(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_e_RGCH_Info_02, T_e_RGCH_Info_02_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t T_fdd_86_sequence[] = {
  { &hf_rrc_e_HICH_Info_01  , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_e_HICH_Info_01 },
  { &hf_rrc_e_RGCH_Info_02  , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_e_RGCH_Info_02 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_fdd_86(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_fdd_86, T_fdd_86_sequence);

  return offset;
}



static int
dissect_rrc_INTEGER_4_44(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            4U, 44U, NULL, FALSE);

  return offset;
}


static const value_string rrc_T_modeSpecificInfo_56_vals[] = {
  {   0, "tdd384" },
  {   1, "tdd768" },
  { 0, NULL }
};

static const per_choice_t T_modeSpecificInfo_56_choice[] = {
  {   0, &hf_rrc_tdd384_22       , ASN1_NO_EXTENSIONS     , dissect_rrc_DL_TS_ChannelisationCode },
  {   1, &hf_rrc_tdd768_14       , ASN1_NO_EXTENSIONS     , dissect_rrc_DL_TS_ChannelisationCode_VHCR },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_modeSpecificInfo_56(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_modeSpecificInfo_56, T_modeSpecificInfo_56_choice,
                                 NULL);

  return offset;
}


static const value_string rrc_T_burst_Type_vals[] = {
  {   0, "type1" },
  {   1, "type2" },
  { 0, NULL }
};


static int
dissect_rrc_T_burst_Type(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     2, NULL, FALSE, 0, NULL);

  return offset;
}


static const value_string rrc_T_midamble_Allocation_Mode_vals[] = {
  {   0, "default" },
  {   1, "common" },
  { 0, NULL }
};


static int
dissect_rrc_T_midamble_Allocation_Mode(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     2, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t E_HICH_Information_TDD384_768_sequence[] = {
  { &hf_rrc_n_E_HICH        , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_4_44 },
  { &hf_rrc_tS_Number       , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_0_14 },
  { &hf_rrc_modeSpecificInfo_56, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_modeSpecificInfo_56 },
  { &hf_rrc_burst_Type      , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_burst_Type },
  { &hf_rrc_midamble_Allocation_Mode, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_midamble_Allocation_Mode },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_E_HICH_Information_TDD384_768(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_E_HICH_Information_TDD384_768, E_HICH_Information_TDD384_768_sequence);

  return offset;
}


static const per_sequence_t T_tdd384_tdd768_sequence[] = {
  { &hf_rrc_e_HICH_Info_02  , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_E_HICH_Information_TDD384_768 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_tdd384_tdd768(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_tdd384_tdd768, T_tdd384_tdd768_sequence);

  return offset;
}


static const value_string rrc_T_midambleAllocationMode_01_vals[] = {
  {   0, "defaultMidamble" },
  {   1, "ueSpecificMidamble" },
  { 0, NULL }
};

static const per_choice_t T_midambleAllocationMode_01_choice[] = {
  {   0, &hf_rrc_defaultMidamble , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   1, &hf_rrc_ueSpecificMidamble, ASN1_NO_EXTENSIONS     , dissect_rrc_INTEGER_0_15 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_midambleAllocationMode_01(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_midambleAllocationMode_01, T_midambleAllocationMode_01_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t E_HICH_Information_LCR_sequence[] = {
  { &hf_rrc_ei              , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_0_3 },
  { &hf_rrc_timeslotNumber_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TimeslotNumber_LCR_r4 },
  { &hf_rrc_channelisation_Code, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_HS_ChannelisationCode_LCR },
  { &hf_rrc_midambleAllocationMode_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_midambleAllocationMode_01 },
  { &hf_rrc_midambleConfiguration, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_1_8 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_E_HICH_Information_LCR(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_E_HICH_Information_LCR, E_HICH_Information_LCR_sequence);

  return offset;
}


static const per_sequence_t E_HICH_Information_LCR_List_sequence_of[1] = {
  { &hf_rrc_E_HICH_Information_LCR_List_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_E_HICH_Information_LCR },
};

static int
dissect_rrc_E_HICH_Information_LCR_List(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_E_HICH_Information_LCR_List, E_HICH_Information_LCR_List_sequence_of,
                                                  1, maxNumE_HICH, FALSE);

  return offset;
}


static const per_sequence_t E_HICH_Information_TDD128_sequence[] = {
  { &hf_rrc_n_E_HICH_01     , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_4_15 },
  { &hf_rrc_e_HICH_InfoList , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_E_HICH_Information_LCR_List },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_E_HICH_Information_TDD128(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_E_HICH_Information_TDD128, E_HICH_Information_TDD128_sequence);

  return offset;
}


static const per_sequence_t T_tdd128_28_sequence[] = {
  { &hf_rrc_e_HICH_Info_03  , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_E_HICH_Information_TDD128 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_tdd128_28(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_tdd128_28, T_tdd128_28_sequence);

  return offset;
}


static const value_string rrc_T_tdd_45_vals[] = {
  {   0, "tdd384-tdd768" },
  {   1, "tdd128" },
  { 0, NULL }
};

static const per_choice_t T_tdd_45_choice[] = {
  {   0, &hf_rrc_tdd384_tdd768   , ASN1_NO_EXTENSIONS     , dissect_rrc_T_tdd384_tdd768 },
  {   1, &hf_rrc_tdd128_28       , ASN1_NO_EXTENSIONS     , dissect_rrc_T_tdd128_28 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_tdd_45(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_tdd_45, T_tdd_45_choice,
                                 NULL);

  return offset;
}


static const value_string rrc_T_modeSpecificInfo2_vals[] = {
  {   0, "fdd" },
  {   1, "tdd" },
  { 0, NULL }
};

static const per_choice_t T_modeSpecificInfo2_choice[] = {
  {   0, &hf_rrc_fdd_87          , ASN1_NO_EXTENSIONS     , dissect_rrc_T_fdd_86 },
  {   1, &hf_rrc_tdd_48          , ASN1_NO_EXTENSIONS     , dissect_rrc_T_tdd_45 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_modeSpecificInfo2(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_modeSpecificInfo2, T_modeSpecificInfo2_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t DL_InformationPerRL_r7_sequence[] = {
  { &hf_rrc_modeSpecificInfo_53, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_modeSpecificInfo_53 },
  { &hf_rrc_dl_dpchInfo_01  , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_dl_dpchInfo_01 },
  { &hf_rrc_e_AGCH_Information_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_E_AGCH_Information_r7 },
  { &hf_rrc_modeSpecificInfo2, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_modeSpecificInfo2 },
  { &hf_rrc_cell_id         , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CellIdentity },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_DL_InformationPerRL_r7(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_DL_InformationPerRL_r7, DL_InformationPerRL_r7_sequence);

  return offset;
}


static const per_sequence_t DL_InformationPerRL_List_r7_sequence_of[1] = {
  { &hf_rrc_DL_InformationPerRL_List_r7_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DL_InformationPerRL_r7 },
};

static int
dissect_rrc_DL_InformationPerRL_List_r7(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_DL_InformationPerRL_List_r7, DL_InformationPerRL_List_r7_sequence_of,
                                                  1, maxRL, FALSE);

  return offset;
}


static const per_sequence_t CellUpdateConfirm_r7_IEs_sequence[] = {
  { &hf_rrc_integrityProtectionModeInfo_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_IntegrityProtectionModeInfo_r7 },
  { &hf_rrc_cipheringModeInfo_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CipheringModeInfo_r7 },
  { &hf_rrc_activationTime  , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_ActivationTime },
  { &hf_rrc_new_U_RNTI      , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_U_RNTI },
  { &hf_rrc_new_C_RNTI      , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_C_RNTI },
  { &hf_rrc_new_DSCH_RNTI   , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DSCH_RNTI },
  { &hf_rrc_new_H_RNTI      , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_H_RNTI },
  { &hf_rrc_newPrimary_E_RNTI, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_E_RNTI },
  { &hf_rrc_newSecondary_E_RNTI, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_E_RNTI },
  { &hf_rrc_rrc_StateIndicator, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RRC_StateIndicator },
  { &hf_rrc_utran_DRX_CycleLengthCoeff_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UTRAN_DRX_CycleLengthCoefficient_r7 },
  { &hf_rrc_waitTime        , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_WaitTime },
  { &hf_rrc_rlc_Re_establishIndicatorRb2_3or4, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { &hf_rrc_rlc_Re_establishIndicatorRb5orAbove, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { &hf_rrc_cn_InformationInfo_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CN_InformationInfo_r6 },
  { &hf_rrc_ura_Identity    , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_URA_Identity },
  { &hf_rrc_supportForChangeOfUE_Capability, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_BOOLEAN },
  { &hf_rrc_rb_InformationReleaseList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RB_InformationReleaseList },
  { &hf_rrc_rb_InformationReconfigList_04, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RB_InformationReconfigList_r7 },
  { &hf_rrc_rb_InformationAffectedList_03, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RB_InformationAffectedList_r7 },
  { &hf_rrc_dl_CounterSynchronisationInfo_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_CounterSynchronisationInfo_r5 },
  { &hf_rrc_pdcp_ROHC_TargetMode, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_PDCP_ROHC_TargetMode },
  { &hf_rrc_ul_CommonTransChInfo_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_CommonTransChInfo_r4 },
  { &hf_rrc_ul_deletedTransChInfoList_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_DeletedTransChInfoList_r6 },
  { &hf_rrc_ul_AddReconfTransChInfoList_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_AddReconfTransChInfoList_r7 },
  { &hf_rrc_dl_CommonTransChInfo_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_CommonTransChInfo_r4 },
  { &hf_rrc_dl_DeletedTransChInfoList_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_DeletedTransChInfoList_r7 },
  { &hf_rrc_dl_AddReconfTransChInfoList_03, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_AddReconfTransChInfoList_r7 },
  { &hf_rrc_frequencyInfo   , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_FrequencyInfo },
  { &hf_rrc_multi_frequencyInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_Multi_frequencyInfo_LCR_r7 },
  { &hf_rrc_dtx_drx_TimingInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DTX_DRX_TimingInfo_r7 },
  { &hf_rrc_dtx_drx_Info    , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DTX_DRX_Info_r7 },
  { &hf_rrc_hs_scch_LessInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_HS_SCCH_LessInfo_r7 },
  { &hf_rrc_mimoParameters  , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MIMO_Parameters_r7 },
  { &hf_rrc_maxAllowedUL_TX_Power, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MaxAllowedUL_TX_Power },
  { &hf_rrc_ul_DPCH_Info_01 , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_DPCH_Info_r7 },
  { &hf_rrc_ul_EDCH_Information_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_EDCH_Information_r7 },
  { &hf_rrc_dl_HSPDSCH_Information_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_HSPDSCH_Information_r7 },
  { &hf_rrc_dl_CommonInformation_04, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_CommonInformation_r7 },
  { &hf_rrc_dl_InformationPerRL_List_04, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_InformationPerRL_List_r7 },
  { &hf_rrc_mbms_PL_ServiceRestrictInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MBMS_PL_ServiceRestrictInfo_r6 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_CellUpdateConfirm_r7_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_CellUpdateConfirm_r7_IEs, CellUpdateConfirm_r7_IEs_sequence);

  return offset;
}



static int
dissect_rrc_T_cellUpdateConfirm_r7_add_ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_bit_string_containing_pdu_new(tvb, offset, actx, tree, hf_index,
                                                              NO_BOUND, NO_BOUND, FALSE, dissect_CellUpdateConfirm_r7_add_ext_IEs_PDU);

  return offset;
}


static const per_sequence_t CellUpdateConfirm_v780ext_IEs_sequence[] = {
  { &hf_rrc_hs_DSCH_TBSizeTable, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_HS_DSCH_TBSizeTable },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_CellUpdateConfirm_v780ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_CellUpdateConfirm_v780ext_IEs, CellUpdateConfirm_v780ext_IEs_sequence);

  return offset;
}



static int
dissect_rrc_MBMS_SessionIdentity(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_octet_string(tvb, offset, actx, tree, hf_index,
                                       1, 1, FALSE, NULL);

  return offset;
}



static int
dissect_rrc_OCTET_STRING_SIZE_3(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_octet_string(tvb, offset, actx, tree, hf_index,
                                       3, 3, FALSE, NULL);

  return offset;
}


static const per_sequence_t RAB_Info_r7_sequence[] = {
  { &hf_rrc_rab_Identity    , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RAB_Identity },
  { &hf_rrc_mbms_SessionIdentity, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MBMS_SessionIdentity },
  { &hf_rrc_mbms_ServiceIdentity, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_OCTET_STRING_SIZE_3 },
  { &hf_rrc_cn_DomainIdentity, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CN_DomainIdentity },
  { &hf_rrc_nas_Synchronisation_Indicator, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_NAS_Synchronisation_Indicator },
  { &hf_rrc_re_EstablishmentTimer, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_Re_EstablishmentTimer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_RAB_Info_r7(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_RAB_Info_r7, RAB_Info_r7_sequence);

  return offset;
}


static const value_string rrc_RLC_InfoChoice_r7_vals[] = {
  {   0, "rlc-Info" },
  {   1, "same-as-RB" },
  { 0, NULL }
};

static const per_choice_t RLC_InfoChoice_r7_choice[] = {
  {   0, &hf_rrc_rlc_Info_03     , ASN1_NO_EXTENSIONS     , dissect_rrc_RLC_Info_r7 },
  {   1, &hf_rrc_same_as_RB      , ASN1_NO_EXTENSIONS     , dissect_rrc_RB_Identity },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_RLC_InfoChoice_r7(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_RLC_InfoChoice_r7, RLC_InfoChoice_r7_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t RB_InformationSetup_r7_sequence[] = {
  { &hf_rrc_rb_Identity     , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RB_Identity },
  { &hf_rrc_pdcp_Info_01    , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_PDCP_Info_r4 },
  { &hf_rrc_rlc_InfoChoice_03, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RLC_InfoChoice_r7 },
  { &hf_rrc_rb_MappingInfo_03, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RB_MappingInfo_r7 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_RB_InformationSetup_r7(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_RB_InformationSetup_r7, RB_InformationSetup_r7_sequence);

  return offset;
}


static const per_sequence_t RB_InformationSetupList_r7_sequence_of[1] = {
  { &hf_rrc_RB_InformationSetupList_r7_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RB_InformationSetup_r7 },
};

static int
dissect_rrc_RB_InformationSetupList_r7(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_RB_InformationSetupList_r7, RB_InformationSetupList_r7_sequence_of,
                                                  1, maxRBperRAB, FALSE);

  return offset;
}


static const per_sequence_t RAB_InformationSetup_r7_sequence[] = {
  { &hf_rrc_rab_Info_03     , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RAB_Info_r7 },
  { &hf_rrc_rb_InformationSetupList_04, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RB_InformationSetupList_r7 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_RAB_InformationSetup_r7(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_RAB_InformationSetup_r7, RAB_InformationSetup_r7_sequence);

  return offset;
}


static const value_string rrc_UL_AMR_Rate_vals[] = {
  {   0, "t0" },
  {   1, "t1" },
  {   2, "t2" },
  {   3, "t3" },
  {   4, "t4" },
  {   5, "t5" },
  {   6, "t6" },
  {   7, "t7" },
  {   8, "t8" },
  {   9, "spare7" },
  {  10, "spare6" },
  {  11, "spare5" },
  {  12, "spare4" },
  {  13, "spare3" },
  {  14, "spare2" },
  {  15, "spare1" },
  { 0, NULL }
};


static int
dissect_rrc_UL_AMR_Rate(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     16, NULL, FALSE, 0, NULL);

  return offset;
}



static int
dissect_rrc_MaxCS_Delay(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            0U, 18U, NULL, FALSE);

  return offset;
}


static const per_sequence_t CS_HSPA_Information_sequence[] = {
  { &hf_rrc_ul_AMR_Rate     , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_AMR_Rate },
  { &hf_rrc_maxCS_Delay     , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MaxCS_Delay },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_CS_HSPA_Information(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_CS_HSPA_Information, CS_HSPA_Information_sequence);

  return offset;
}


static const per_sequence_t RAB_InformationSetup_v820ext_sequence[] = {
  { &hf_rrc_cs_HSPA_Information, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CS_HSPA_Information },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_RAB_InformationSetup_v820ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_RAB_InformationSetup_v820ext, RAB_InformationSetup_v820ext_sequence);

  return offset;
}


static const per_sequence_t T_rab_InformationSetup_sequence[] = {
  { &hf_rrc_rab_InformationSetup_r7, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RAB_InformationSetup_r7 },
  { &hf_rrc_rab_InformationSetup_v820ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RAB_InformationSetup_v820ext },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_rab_InformationSetup(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_rab_InformationSetup, T_rab_InformationSetup_sequence);

  return offset;
}


static const per_sequence_t CellUpdateConfirm_v860ext_IEs_sequence[] = {
  { &hf_rrc_rab_InformationSetup_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_rab_InformationSetup },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_CellUpdateConfirm_v860ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_CellUpdateConfirm_v860ext_IEs, CellUpdateConfirm_v860ext_IEs_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions_18_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_nonCriticalExtensions_18(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_nonCriticalExtensions_18, T_nonCriticalExtensions_18_sequence);

  return offset;
}


static const per_sequence_t T_v860NonCriticalExtensions_01_sequence[] = {
  { &hf_rrc_cellUpdateConfirm_v860ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CellUpdateConfirm_v860ext_IEs },
  { &hf_rrc_nonCriticalExtensions_18, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_nonCriticalExtensions_18 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v860NonCriticalExtensions_01(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v860NonCriticalExtensions_01, T_v860NonCriticalExtensions_01_sequence);

  return offset;
}


static const per_sequence_t T_v780NonCriticalExtensions_01_sequence[] = {
  { &hf_rrc_cellUpdateConfirm_v780ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CellUpdateConfirm_v780ext_IEs },
  { &hf_rrc_v860NonCriticalExtensions_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v860NonCriticalExtensions_01 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v780NonCriticalExtensions_01(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v780NonCriticalExtensions_01, T_v780NonCriticalExtensions_01_sequence);

  return offset;
}


static const per_sequence_t T_r7_01_sequence[] = {
  { &hf_rrc_cellUpdateConfirm_r7, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CellUpdateConfirm_r7_IEs },
  { &hf_rrc_cellUpdateConfirm_r7_add_ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_cellUpdateConfirm_r7_add_ext },
  { &hf_rrc_v780NonCriticalExtensions_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v780NonCriticalExtensions_01 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_r7_01(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_r7_01, T_r7_01_sequence);

  return offset;
}


static const value_string rrc_DefaultConfigMode_vals[] = {
  {   0, "fdd" },
  {   1, "tdd" },
  { 0, NULL }
};


static int
dissect_rrc_DefaultConfigMode(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     2, NULL, FALSE, 0, NULL);

  return offset;
}



static int
dissect_rrc_DefaultConfigIdForCellFACH(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            0U, 15U, NULL, FALSE);

  return offset;
}


static const per_sequence_t DefaultConfigForCellFACH_sequence[] = {
  { &hf_rrc_defaultConfigMode, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DefaultConfigMode },
  { &hf_rrc_defaultConfigIdForCellFACH, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DefaultConfigIdForCellFACH },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_DefaultConfigForCellFACH(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_DefaultConfigForCellFACH, DefaultConfigForCellFACH_sequence);

  return offset;
}


static const per_sequence_t RAB_InfoReplace_sequence[] = {
  { &hf_rrc_rab_Identity    , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RAB_Identity },
  { &hf_rrc_cn_DomainIdentity, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CN_DomainIdentity },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_RAB_InfoReplace(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_RAB_InfoReplace, RAB_InfoReplace_sequence);

  return offset;
}


static const value_string rrc_T_rlc_SizeList_02_vals[] = {
  {   0, "allSizes" },
  {   1, "configured" },
  {   2, "explicitList" },
  { 0, NULL }
};

static const per_choice_t T_rlc_SizeList_02_choice[] = {
  {   0, &hf_rrc_allSizes        , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   1, &hf_rrc_configured      , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   2, &hf_rrc_explicitList    , ASN1_NO_EXTENSIONS     , dissect_rrc_RLC_SizeExplicitList },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_rlc_SizeList_02(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_rlc_SizeList_02, T_rlc_SizeList_02_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t T_dch_rach_usch_01_sequence[] = {
  { &hf_rrc_ul_TransportChannelType, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UL_TransportChannelType },
  { &hf_rrc_logicalChannelIdentity, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_LogicalChannelIdentity },
  { &hf_rrc_rlc_SizeList_02 , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_rlc_SizeList_02 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_dch_rach_usch_01(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_dch_rach_usch_01, T_dch_rach_usch_01_sequence);

  return offset;
}


static const per_sequence_t T_fixedSize_sequence[] = {
  { &hf_rrc_ddi             , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DDI },
  { &hf_rrc_rlc_PDU_SizeList, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RLC_PDU_SizeList },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_fixedSize(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_fixedSize, T_fixedSize_sequence);

  return offset;
}


static const value_string rrc_T_lengthIndicatorSize_vals[] = {
  {   0, "size7" },
  {   1, "size15" },
  { 0, NULL }
};


static int
dissect_rrc_T_lengthIndicatorSize(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     2, NULL, FALSE, 0, NULL);

  return offset;
}



static int
dissect_rrc_INTEGER_0_1503(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            0U, 1503U, NULL, FALSE);

  return offset;
}


static const per_sequence_t RLC_PDU_SizeConstraint_sequence[] = {
  { &hf_rrc_lengthIndicatorSize, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_lengthIndicatorSize },
  { &hf_rrc_minRLC_PDU_Size , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_0_1503 },
  { &hf_rrc_largestRLC_PDU_Size, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_0_1503 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_RLC_PDU_SizeConstraint(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_RLC_PDU_SizeConstraint, RLC_PDU_SizeConstraint_sequence);

  return offset;
}


static const value_string rrc_T_rlc_PDU_Size_vals[] = {
  {   0, "fixedSize" },
  {   1, "flexibleSize" },
  { 0, NULL }
};

static const per_choice_t T_rlc_PDU_Size_choice[] = {
  {   0, &hf_rrc_fixedSize_01    , ASN1_NO_EXTENSIONS     , dissect_rrc_T_fixedSize },
  {   1, &hf_rrc_flexibleSize_02 , ASN1_NO_EXTENSIONS     , dissect_rrc_RLC_PDU_SizeConstraint },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_rlc_PDU_Size(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_rlc_PDU_Size, T_rlc_PDU_Size_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t T_e_dch_01_sequence[] = {
  { &hf_rrc_logicalChannelIdentity, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_LogicalChannelIdentity },
  { &hf_rrc_e_DCH_MAC_d_FlowIdentity, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_E_DCH_MAC_d_FlowIdentity },
  { &hf_rrc_rlc_PDU_Size    , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_rlc_PDU_Size },
  { &hf_rrc_includeInSchedulingInfo, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_e_dch_01(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_e_dch_01, T_e_dch_01_sequence);

  return offset;
}


static const value_string rrc_T_ul_TrCH_Type_01_vals[] = {
  {   0, "dch-rach-usch" },
  {   1, "e-dch" },
  { 0, NULL }
};

static const per_choice_t T_ul_TrCH_Type_01_choice[] = {
  {   0, &hf_rrc_dch_rach_usch_01, ASN1_NO_EXTENSIONS     , dissect_rrc_T_dch_rach_usch_01 },
  {   1, &hf_rrc_e_dch_01        , ASN1_NO_EXTENSIONS     , dissect_rrc_T_e_dch_01 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_ul_TrCH_Type_01(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_ul_TrCH_Type_01, T_ul_TrCH_Type_01_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t UL_LogicalChannelMapping_r8_sequence[] = {
  { &hf_rrc_ul_TrCH_Type_01 , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_ul_TrCH_Type_01 },
  { &hf_rrc_mac_LogicalChannelPriority, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MAC_LogicalChannelPriority },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UL_LogicalChannelMapping_r8(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UL_LogicalChannelMapping_r8, UL_LogicalChannelMapping_r8_sequence);

  return offset;
}


static const per_sequence_t SEQUENCE_SIZE_maxLoCHperRLC_OF_UL_LogicalChannelMapping_r8_sequence_of[1] = {
  { &hf_rrc_ul_LogicalChannelMapping_item_02, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UL_LogicalChannelMapping_r8 },
};

static int
dissect_rrc_SEQUENCE_SIZE_maxLoCHperRLC_OF_UL_LogicalChannelMapping_r8(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_SEQUENCE_SIZE_maxLoCHperRLC_OF_UL_LogicalChannelMapping_r8, SEQUENCE_SIZE_maxLoCHperRLC_OF_UL_LogicalChannelMapping_r8_sequence_of,
                                                  maxLoCHperRLC, maxLoCHperRLC, FALSE);

  return offset;
}


static const per_sequence_t UL_LogicalChannelMappingList_r8_sequence[] = {
  { &hf_rrc_rlc_LogicalChannelMappingIndicator, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { &hf_rrc_ul_LogicalChannelMapping_02, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SEQUENCE_SIZE_maxLoCHperRLC_OF_UL_LogicalChannelMapping_r8 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UL_LogicalChannelMappingList_r8(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UL_LogicalChannelMappingList_r8, UL_LogicalChannelMappingList_r8_sequence);

  return offset;
}


static const value_string rrc_UL_LogicalChannelMappings_r8_vals[] = {
  {   0, "oneLogicalChannel" },
  {   1, "twoLogicalChannels" },
  { 0, NULL }
};

static const per_choice_t UL_LogicalChannelMappings_r8_choice[] = {
  {   0, &hf_rrc_oneLogicalChannel_02, ASN1_NO_EXTENSIONS     , dissect_rrc_UL_LogicalChannelMapping_r8 },
  {   1, &hf_rrc_twoLogicalChannels_02, ASN1_NO_EXTENSIONS     , dissect_rrc_UL_LogicalChannelMappingList_r8 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_UL_LogicalChannelMappings_r8(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_UL_LogicalChannelMappings_r8, UL_LogicalChannelMappings_r8_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t RB_MappingOption_r8_sequence[] = {
  { &hf_rrc_ul_LogicalChannelMappings_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_LogicalChannelMappings_r8 },
  { &hf_rrc_dl_LogicalChannelMappingList_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_LogicalChannelMappingList_r7 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_RB_MappingOption_r8(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_RB_MappingOption_r8, RB_MappingOption_r8_sequence);

  return offset;
}


static const per_sequence_t RB_MappingInfo_r8_sequence_of[1] = {
  { &hf_rrc_RB_MappingInfo_r8_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RB_MappingOption_r8 },
};

static int
dissect_rrc_RB_MappingInfo_r8(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_RB_MappingInfo_r8, RB_MappingInfo_r8_sequence_of,
                                                  1, maxRBMuxOptions, FALSE);

  return offset;
}


static const per_sequence_t RB_InformationSetup_r8_sequence[] = {
  { &hf_rrc_rb_Identity     , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RB_Identity },
  { &hf_rrc_pdcp_Info_01    , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_PDCP_Info_r4 },
  { &hf_rrc_rlc_InfoChoice_03, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RLC_InfoChoice_r7 },
  { &hf_rrc_rb_MappingInfo_04, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RB_MappingInfo_r8 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_RB_InformationSetup_r8(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_RB_InformationSetup_r8, RB_InformationSetup_r8_sequence);

  return offset;
}


static const per_sequence_t RB_InformationSetupList_r8_sequence_of[1] = {
  { &hf_rrc_RB_InformationSetupList_r8_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RB_InformationSetup_r8 },
};

static int
dissect_rrc_RB_InformationSetupList_r8(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_RB_InformationSetupList_r8, RB_InformationSetupList_r8_sequence_of,
                                                  1, maxRBperRAB, FALSE);

  return offset;
}


static const per_sequence_t RAB_InformationSetup_r8_sequence[] = {
  { &hf_rrc_rab_Info_03     , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RAB_Info_r7 },
  { &hf_rrc_cs_HSPA_Information, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CS_HSPA_Information },
  { &hf_rrc_rab_InfoReplace , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RAB_InfoReplace },
  { &hf_rrc_rb_InformationSetupList_05, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RB_InformationSetupList_r8 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_RAB_InformationSetup_r8(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_RAB_InformationSetup_r8, RAB_InformationSetup_r8_sequence);

  return offset;
}


static const per_sequence_t RB_InformationReconfig_r8_sequence[] = {
  { &hf_rrc_rb_Identity     , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RB_Identity },
  { &hf_rrc_pdcp_Info_03    , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_PDCP_InfoReconfig_r4 },
  { &hf_rrc_pdcp_SN_Info    , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_PDCP_SN_Info },
  { &hf_rrc_rlc_Info_03     , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RLC_Info_r7 },
  { &hf_rrc_rb_MappingInfo_04, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RB_MappingInfo_r8 },
  { &hf_rrc_rb_StopContinue , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RB_StopContinue },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_RB_InformationReconfig_r8(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_RB_InformationReconfig_r8, RB_InformationReconfig_r8_sequence);

  return offset;
}


static const per_sequence_t RB_InformationReconfigList_r8_sequence_of[1] = {
  { &hf_rrc_RB_InformationReconfigList_r8_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RB_InformationReconfig_r8 },
};

static int
dissect_rrc_RB_InformationReconfigList_r8(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_RB_InformationReconfigList_r8, RB_InformationReconfigList_r8_sequence_of,
                                                  1, maxRB, FALSE);

  return offset;
}


static const per_sequence_t RB_InformationAffected_r8_sequence[] = {
  { &hf_rrc_rb_Identity     , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RB_Identity },
  { &hf_rrc_rb_MappingInfo_04, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RB_MappingInfo_r8 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_RB_InformationAffected_r8(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_RB_InformationAffected_r8, RB_InformationAffected_r8_sequence);

  return offset;
}


static const per_sequence_t RB_InformationAffectedList_r8_sequence_of[1] = {
  { &hf_rrc_RB_InformationAffectedList_r8_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RB_InformationAffected_r8 },
};

static int
dissect_rrc_RB_InformationAffectedList_r8(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_RB_InformationAffectedList_r8, RB_InformationAffectedList_r8_sequence_of,
                                                  1, maxRB, FALSE);

  return offset;
}


static const per_sequence_t T_dch_usch_02_sequence[] = {
  { &hf_rrc_ul_TransportChannelType_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UL_TrCH_Type },
  { &hf_rrc_transportChannelIdentity, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TransportChannelIdentity },
  { &hf_rrc_transportFormatSet, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TransportFormatSet },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_dch_usch_02(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_dch_usch_02, T_dch_usch_02_sequence);

  return offset;
}


static const value_string rrc_T_ul_MAC_HeaderType_vals[] = {
  {   0, "mac-iis" },
  { 0, NULL }
};


static int
dissect_rrc_T_ul_MAC_HeaderType(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t T_fdd_55_sequence[] = {
  { &hf_rrc_tti_03          , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_E_DCH_TTI },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_fdd_55(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_fdd_55, T_fdd_55_sequence);

  return offset;
}


static const value_string rrc_T_modeSpecific_01_vals[] = {
  {   0, "fdd" },
  {   1, "tdd" },
  { 0, NULL }
};

static const per_choice_t T_modeSpecific_01_choice[] = {
  {   0, &hf_rrc_fdd_56          , ASN1_NO_EXTENSIONS     , dissect_rrc_T_fdd_55 },
  {   1, &hf_rrc_tdd             , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_modeSpecific_01(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_modeSpecific_01, T_modeSpecific_01_choice,
                                 NULL);

  return offset;
}


static const value_string rrc_T_harq_Info_02_vals[] = {
  {   0, "rv0" },
  {   1, "rvtable" },
  { 0, NULL }
};


static int
dissect_rrc_T_harq_Info_02(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     2, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t T_e_dch_04_sequence[] = {
  { &hf_rrc_ul_MAC_HeaderType, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_ul_MAC_HeaderType },
  { &hf_rrc_modeSpecific_01 , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_modeSpecific_01 },
  { &hf_rrc_harq_Info_02    , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_harq_Info_02 },
  { &hf_rrc_addReconf_MAC_d_FlowList_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_E_DCH_AddReconf_MAC_d_FlowList_r7 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_e_dch_04(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_e_dch_04, T_e_dch_04_sequence);

  return offset;
}


static const value_string rrc_UL_AddReconfTransChInformation_r8_vals[] = {
  {   0, "dch-usch" },
  {   1, "e-dch" },
  { 0, NULL }
};

static const per_choice_t UL_AddReconfTransChInformation_r8_choice[] = {
  {   0, &hf_rrc_dch_usch_02     , ASN1_NO_EXTENSIONS     , dissect_rrc_T_dch_usch_02 },
  {   1, &hf_rrc_e_dch_04        , ASN1_NO_EXTENSIONS     , dissect_rrc_T_e_dch_04 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_UL_AddReconfTransChInformation_r8(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_UL_AddReconfTransChInformation_r8, UL_AddReconfTransChInformation_r8_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t UL_AddReconfTransChInfoList_r8_sequence_of[1] = {
  { &hf_rrc_UL_AddReconfTransChInfoList_r8_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UL_AddReconfTransChInformation_r8 },
};

static int
dissect_rrc_UL_AddReconfTransChInfoList_r8(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_UL_AddReconfTransChInfoList_r8, UL_AddReconfTransChInfoList_r8_sequence_of,
                                                  1, maxTrCH, FALSE);

  return offset;
}


static const value_string rrc_T_mac_es_e_resetIndicator_03_vals[] = {
  {   0, "true" },
  { 0, NULL }
};


static int
dissect_rrc_T_mac_es_e_resetIndicator_03(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t E_DPDCH_Info_r8_sequence[] = {
  { &hf_rrc_e_TFCI_TableIndex, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_E_TFCI_TableIndex },
  { &hf_rrc_e_DCH_MinimumSet_E_TFCI, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_E_DCH_MinimumSet_E_TFCI },
  { &hf_rrc_reference_E_TFCIs_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_E_DPDCH_Reference_E_TFCIList_r7 },
  { &hf_rrc_minReduced_E_DPDCH_GainFactor, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MinReduced_E_DPDCH_GainFactor },
  { &hf_rrc_maxChannelisationCodes, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_E_DPDCH_MaxChannelisationCodes },
  { &hf_rrc_pl_NonMax       , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_E_DPDCH_PL_NonMax },
  { &hf_rrc_schedulingInfoConfiguration, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_E_DPDCH_SchedulingInfoConfiguration },
  { &hf_rrc_threeIndexStepThreshold, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_INTEGER_0_37 },
  { &hf_rrc_twoIndexStepThreshold, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_INTEGER_0_37 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_E_DPDCH_Info_r8(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_E_DPDCH_Info_r8, E_DPDCH_Info_r8_sequence);

  return offset;
}


static const per_sequence_t T_fdd_129_sequence[] = {
  { &hf_rrc_e_DPCCH_Info_01 , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_E_DPCCH_Info_r7 },
  { &hf_rrc_e_DPDCH_Info_02 , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_E_DPDCH_Info_r8 },
  { &hf_rrc_schedulingTransmConfiguration, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_E_DPDCH_SchedulingTransmConfiguration },
  { &hf_rrc_ul_16QAM_Settings, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_16QAM_Settings },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_fdd_129(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_fdd_129, T_fdd_129_sequence);

  return offset;
}


static const per_sequence_t T_tdd_80_sequence[] = {
  { &hf_rrc_e_RUCCH_Info_01 , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_E_RUCCH_Info },
  { &hf_rrc_e_PUCH_Info_01  , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_E_PUCH_Info },
  { &hf_rrc_non_ScheduledTransGrantInfo_03, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_Non_ScheduledTransGrantInfoTDD },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_tdd_80(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_tdd_80, T_tdd_80_sequence);

  return offset;
}


static const value_string rrc_T_modeSpecificInfo_90_vals[] = {
  {   0, "fdd" },
  {   1, "tdd" },
  { 0, NULL }
};

static const per_choice_t T_modeSpecificInfo_90_choice[] = {
  {   0, &hf_rrc_fdd_132         , ASN1_NO_EXTENSIONS     , dissect_rrc_T_fdd_129 },
  {   1, &hf_rrc_tdd_86          , ASN1_NO_EXTENSIONS     , dissect_rrc_T_tdd_80 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_modeSpecificInfo_90(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_modeSpecificInfo_90, T_modeSpecificInfo_90_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t UL_EDCH_Information_r8_sequence[] = {
  { &hf_rrc_mac_es_e_resetIndicator_03, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_mac_es_e_resetIndicator_03 },
  { &hf_rrc_modeSpecificInfo_90, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_modeSpecificInfo_90 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UL_EDCH_Information_r8(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UL_EDCH_Information_r8, UL_EDCH_Information_r8_sequence);

  return offset;
}


static const value_string rrc_T_dl_dpchInfoCommon_02_vals[] = {
  {   0, "dl-DPCH-InfoCommon" },
  {   1, "dl-FDPCH-InfoCommon" },
  { 0, NULL }
};

static const per_choice_t T_dl_dpchInfoCommon_02_choice[] = {
  {   0, &hf_rrc_dl_DPCH_InfoCommon_02, ASN1_NO_EXTENSIONS     , dissect_rrc_DL_DPCH_InfoCommon_r6 },
  {   1, &hf_rrc_dl_FDPCH_InfoCommon, ASN1_NO_EXTENSIONS     , dissect_rrc_DL_FDPCH_InfoCommon_r6 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_dl_dpchInfoCommon_02(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_dl_dpchInfoCommon_02, T_dl_dpchInfoCommon_02_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t T_activate_01_sequence[] = {
  { &hf_rrc_tgcfn           , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TGCFN },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_activate_01(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_activate_01, T_activate_01_sequence);

  return offset;
}


static const value_string rrc_T_tgps_Status_01_vals[] = {
  {   0, "activate" },
  {   1, "deactivate" },
  { 0, NULL }
};

static const per_choice_t T_tgps_Status_01_choice[] = {
  {   0, &hf_rrc_activate_01     , ASN1_NO_EXTENSIONS     , dissect_rrc_T_activate_01 },
  {   1, &hf_rrc_deactivate      , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_tgps_Status_01(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_tgps_Status_01, T_tgps_Status_01_choice,
                                 NULL);

  return offset;
}


static const value_string rrc_TGMP_r8_vals[] = {
  {   0, "tdd-Measurement" },
  {   1, "fdd-Measurement" },
  {   2, "gsm-CarrierRSSIMeasurement" },
  {   3, "gsm-initialBSICIdentification" },
  {   4, "gsmBSICReconfirmation" },
  {   5, "multi-carrier" },
  {   6, "e-UTRA" },
  {   7, "spare" },
  { 0, NULL }
};


static int
dissect_rrc_TGMP_r8(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     8, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t TGPS_ConfigurationParams_r8_sequence[] = {
  { &hf_rrc_tgmp_01         , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TGMP_r8 },
  { &hf_rrc_tgprc           , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TGPRC },
  { &hf_rrc_tgsn            , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TGSN },
  { &hf_rrc_tgl1            , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TGL },
  { &hf_rrc_tgl2            , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_TGL },
  { &hf_rrc_tgd             , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TGD },
  { &hf_rrc_tgpl1           , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TGPL },
  { &hf_rrc_rpp             , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RPP },
  { &hf_rrc_itp             , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_ITP },
  { &hf_rrc_ul_DL_Mode      , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UL_DL_Mode },
  { &hf_rrc_dl_FrameType    , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DL_FrameType },
  { &hf_rrc_deltaSIR1       , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DeltaSIR },
  { &hf_rrc_deltaSIRAfter1  , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DeltaSIR },
  { &hf_rrc_deltaSIR2       , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DeltaSIR },
  { &hf_rrc_deltaSIRAfter2  , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DeltaSIR },
  { &hf_rrc_nidentifyAbort  , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_NidentifyAbort },
  { &hf_rrc_treconfirmAbort , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_TreconfirmAbort },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_TGPS_ConfigurationParams_r8(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_TGPS_ConfigurationParams_r8, TGPS_ConfigurationParams_r8_sequence);

  return offset;
}


static const per_sequence_t TGP_Sequence_r8_sequence[] = {
  { &hf_rrc_tgpsi           , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TGPSI },
  { &hf_rrc_tgps_Status_01  , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_tgps_Status_01 },
  { &hf_rrc_tgps_ConfigurationParams_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_TGPS_ConfigurationParams_r8 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_TGP_Sequence_r8(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_TGP_Sequence_r8, TGP_Sequence_r8_sequence);

  return offset;
}


static const per_sequence_t TGP_SequenceList_r8_sequence_of[1] = {
  { &hf_rrc_TGP_SequenceList_r8_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TGP_Sequence_r8 },
};

static int
dissect_rrc_TGP_SequenceList_r8(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_TGP_SequenceList_r8, TGP_SequenceList_r8_sequence_of,
                                                  1, maxTGPS, FALSE);

  return offset;
}


static const per_sequence_t DPCH_CompressedModeInfo_r8_sequence[] = {
  { &hf_rrc_tgp_SequenceList_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TGP_SequenceList_r8 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_DPCH_CompressedModeInfo_r8(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_DPCH_CompressedModeInfo_r8, DPCH_CompressedModeInfo_r8_sequence);

  return offset;
}


static const per_sequence_t T_fdd_63_sequence[] = {
  { &hf_rrc_defaultDPCH_OffsetValue, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DefaultDPCH_OffsetValueFDD },
  { &hf_rrc_dpch_CompressedModeInfo_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DPCH_CompressedModeInfo_r8 },
  { &hf_rrc_tx_DiversityMode, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_TX_DiversityMode },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_fdd_63(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_fdd_63, T_fdd_63_sequence);

  return offset;
}


static const per_sequence_t T_tdd128_18_sequence[] = {
  { &hf_rrc_tstd_Indicator  , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_tdd128_18(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_tdd128_18, T_tdd128_18_sequence);

  return offset;
}


static const value_string rrc_T_tddOption_09_vals[] = {
  {   0, "tdd384" },
  {   1, "tdd768" },
  {   2, "tdd128" },
  { 0, NULL }
};

static const per_choice_t T_tddOption_09_choice[] = {
  {   0, &hf_rrc_tdd384_11       , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   1, &hf_rrc_tdd768_06       , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   2, &hf_rrc_tdd128_18       , ASN1_NO_EXTENSIONS     , dissect_rrc_T_tdd128_18 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_tddOption_09(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_tddOption_09, T_tddOption_09_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t T_tdd_25_sequence[] = {
  { &hf_rrc_tddOption_09    , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_tddOption_09 },
  { &hf_rrc_defaultDPCH_OffsetValue_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DefaultDPCH_OffsetValueTDD },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_tdd_25(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_tdd_25, T_tdd_25_sequence);

  return offset;
}


static const value_string rrc_T_modeSpecificInfo_33_vals[] = {
  {   0, "fdd" },
  {   1, "tdd" },
  { 0, NULL }
};

static const per_choice_t T_modeSpecificInfo_33_choice[] = {
  {   0, &hf_rrc_fdd_64          , ASN1_NO_EXTENSIONS     , dissect_rrc_T_fdd_63 },
  {   1, &hf_rrc_tdd_26          , ASN1_NO_EXTENSIONS     , dissect_rrc_T_tdd_25 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_modeSpecificInfo_33(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_modeSpecificInfo_33, T_modeSpecificInfo_33_choice,
                                 NULL);

  return offset;
}


static const value_string rrc_T_mac_hsResetIndicator_08_vals[] = {
  {   0, "true" },
  { 0, NULL }
};


static int
dissect_rrc_T_mac_hsResetIndicator_08(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}


static const value_string rrc_T_postVerificationPeriod_09_vals[] = {
  {   0, "true" },
  { 0, NULL }
};


static int
dissect_rrc_T_postVerificationPeriod_09(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t DL_CommonInformation_r8_sequence[] = {
  { &hf_rrc_dl_dpchInfoCommon_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_dl_dpchInfoCommon_02 },
  { &hf_rrc_modeSpecificInfo_33, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_modeSpecificInfo_33 },
  { &hf_rrc_mac_hsResetIndicator_08, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_mac_hsResetIndicator_08 },
  { &hf_rrc_postVerificationPeriod_09, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_postVerificationPeriod_09 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_DL_CommonInformation_r8(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_DL_CommonInformation_r8, DL_CommonInformation_r8_sequence);

  return offset;
}


static const per_sequence_t T_fdd_87_sequence[] = {
  { &hf_rrc_primaryCPICH_Info, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PrimaryCPICH_Info },
  { &hf_rrc_servingHSDSCH_RL_indicator, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { &hf_rrc_servingEDCH_RL_indicator, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_fdd_87(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_fdd_87, T_fdd_87_sequence);

  return offset;
}


static const value_string rrc_T_modeSpecificInfo_54_vals[] = {
  {   0, "fdd" },
  {   1, "tdd" },
  { 0, NULL }
};

static const per_choice_t T_modeSpecificInfo_54_choice[] = {
  {   0, &hf_rrc_fdd_88          , ASN1_NO_EXTENSIONS     , dissect_rrc_T_fdd_87 },
  {   1, &hf_rrc_tdd_47          , ASN1_NO_EXTENSIONS     , dissect_rrc_PrimaryCCPCH_Info_r4 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_modeSpecificInfo_54(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_modeSpecificInfo_54, T_modeSpecificInfo_54_choice,
                                 NULL);

  return offset;
}


static const value_string rrc_T_dl_dpchInfo_02_vals[] = {
  {   0, "dl-DPCH-InfoPerRL" },
  {   1, "dl-FDPCH-InfoPerRL" },
  { 0, NULL }
};

static const per_choice_t T_dl_dpchInfo_02_choice[] = {
  {   0, &hf_rrc_dl_DPCH_InfoPerRL_04, ASN1_NO_EXTENSIONS     , dissect_rrc_DL_DPCH_InfoPerRL_r7 },
  {   1, &hf_rrc_dl_FDPCH_InfoPerRL_01, ASN1_NO_EXTENSIONS     , dissect_rrc_DL_FDPCH_InfoPerRL_r7 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_dl_dpchInfo_02(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_dl_dpchInfo_02, T_dl_dpchInfo_02_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t T_fdd_92_sequence[] = {
  { &hf_rrc_e_AGCH_ChannelisationCode, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_E_AGCH_ChannelisationCode },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_fdd_92(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_fdd_92, T_fdd_92_sequence);

  return offset;
}


static const per_sequence_t T_tdd384_20_sequence[] = {
  { &hf_rrc_long_Term_Grant_Indicator, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_BOOLEAN },
  { &hf_rrc_length_of_TTRI_field, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_1_12 },
  { &hf_rrc_e_AGCH_Set_Config, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_E_AGCH_Set_Config },
  { &hf_rrc_e_AGCH_BLER_Target, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_Bler_Target },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_tdd384_20(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_tdd384_20, T_tdd384_20_sequence);

  return offset;
}


static const per_sequence_t T_tdd768_12_sequence[] = {
  { &hf_rrc_long_Term_Grant_Indicator, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_BOOLEAN },
  { &hf_rrc_length_of_TTRI_field, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_1_12 },
  { &hf_rrc_e_AGCH_Set_Config_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_E_AGCH_Set_Config_VHCR },
  { &hf_rrc_e_AGCH_BLER_Target, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_Bler_Target },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_tdd768_12(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_tdd768_12, T_tdd768_12_sequence);

  return offset;
}


static const value_string rrc_E_AGCH_InactivityMonitorThreshold_TDD128_vals[] = {
  {   0, "radio-frames-0" },
  {   1, "radio-frames-1" },
  {   2, "radio-frames-2" },
  {   3, "radio-frames-4" },
  {   4, "radio-frames-8" },
  {   5, "radio-frames-16" },
  {   6, "radio-frames-32" },
  {   7, "radio-frames-64" },
  {   8, "radio-frames-128" },
  {   9, "radio-frames-256" },
  {  10, "radio-frames-512" },
  {  11, "spare5" },
  {  12, "spare4" },
  {  13, "spare3" },
  {  14, "spare2" },
  {  15, "infinity" },
  { 0, NULL }
};


static int
dissect_rrc_E_AGCH_InactivityMonitorThreshold_TDD128(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     16, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t T_tdd128_32_sequence[] = {
  { &hf_rrc_rdi_Indicator   , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { &hf_rrc_tpc_StepSize    , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TPC_StepSizeTDD },
  { &hf_rrc_e_AGCH_Set_Config_02, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_E_AGCH_Set_Config_LCR },
  { &hf_rrc_e_AGCH_BLER_Target, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_Bler_Target },
  { &hf_rrc_e_AGCH_InactivityMonitorThreshold, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_E_AGCH_InactivityMonitorThreshold_TDD128 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_tdd128_32(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_tdd128_32, T_tdd128_32_sequence);

  return offset;
}


static const value_string rrc_T_tdd_49_vals[] = {
  {   0, "tdd384" },
  {   1, "tdd768" },
  {   2, "tdd128" },
  { 0, NULL }
};

static const per_choice_t T_tdd_49_choice[] = {
  {   0, &hf_rrc_tdd384_21       , ASN1_NO_EXTENSIONS     , dissect_rrc_T_tdd384_20 },
  {   1, &hf_rrc_tdd768_13       , ASN1_NO_EXTENSIONS     , dissect_rrc_T_tdd768_12 },
  {   2, &hf_rrc_tdd128_32       , ASN1_NO_EXTENSIONS     , dissect_rrc_T_tdd128_32 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_tdd_49(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_tdd_49, T_tdd_49_choice,
                                 NULL);

  return offset;
}


static const value_string rrc_T_modeSpecific_03_vals[] = {
  {   0, "fdd" },
  {   1, "tdd" },
  { 0, NULL }
};

static const per_choice_t T_modeSpecific_03_choice[] = {
  {   0, &hf_rrc_fdd_93          , ASN1_NO_EXTENSIONS     , dissect_rrc_T_fdd_92 },
  {   1, &hf_rrc_tdd_52          , ASN1_NO_EXTENSIONS     , dissect_rrc_T_tdd_49 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_modeSpecific_03(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_modeSpecific_03, T_modeSpecific_03_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t E_AGCH_Information_r8_sequence[] = {
  { &hf_rrc_modeSpecific_03 , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_modeSpecific_03 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_E_AGCH_Information_r8(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_E_AGCH_Information_r8, E_AGCH_Information_r8_sequence);

  return offset;
}


static const value_string rrc_T_e_HICH_Info_02_vals[] = {
  {   0, "e-HICH-Information" },
  {   1, "releaseIndicator" },
  { 0, NULL }
};

static const per_choice_t T_e_HICH_Info_02_choice[] = {
  {   0, &hf_rrc_e_HICH_Information, ASN1_NO_EXTENSIONS     , dissect_rrc_E_HICH_Information },
  {   1, &hf_rrc_releaseIndicator, ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_e_HICH_Info_02(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_e_HICH_Info_02, T_e_HICH_Info_02_choice,
                                 NULL);

  return offset;
}


static const value_string rrc_T_e_RGCH_Info_03_vals[] = {
  {   0, "e-RGCH-Information" },
  {   1, "releaseIndicator" },
  { 0, NULL }
};

static const per_choice_t T_e_RGCH_Info_03_choice[] = {
  {   0, &hf_rrc_e_RGCH_Information, ASN1_NO_EXTENSIONS     , dissect_rrc_E_RGCH_Information },
  {   1, &hf_rrc_releaseIndicator, ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_e_RGCH_Info_03(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_e_RGCH_Info_03, T_e_RGCH_Info_03_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t T_fdd_88_sequence[] = {
  { &hf_rrc_e_HICH_Info_04  , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_e_HICH_Info_02 },
  { &hf_rrc_e_RGCH_Info_03  , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_e_RGCH_Info_03 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_fdd_88(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_fdd_88, T_fdd_88_sequence);

  return offset;
}


static const per_sequence_t T_tdd384_tdd768_01_sequence[] = {
  { &hf_rrc_e_HICH_Info_02  , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_E_HICH_Information_TDD384_768 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_tdd384_tdd768_01(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_tdd384_tdd768_01, T_tdd384_tdd768_01_sequence);

  return offset;
}


static const per_sequence_t T_tdd128_29_sequence[] = {
  { &hf_rrc_e_HICH_Info_03  , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_E_HICH_Information_TDD128 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_tdd128_29(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_tdd128_29, T_tdd128_29_sequence);

  return offset;
}


static const value_string rrc_T_tdd_46_vals[] = {
  {   0, "tdd384-tdd768" },
  {   1, "tdd128" },
  { 0, NULL }
};

static const per_choice_t T_tdd_46_choice[] = {
  {   0, &hf_rrc_tdd384_tdd768_01, ASN1_NO_EXTENSIONS     , dissect_rrc_T_tdd384_tdd768_01 },
  {   1, &hf_rrc_tdd128_29       , ASN1_NO_EXTENSIONS     , dissect_rrc_T_tdd128_29 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_tdd_46(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_tdd_46, T_tdd_46_choice,
                                 NULL);

  return offset;
}


static const value_string rrc_T_modeSpecificInfo2_01_vals[] = {
  {   0, "fdd" },
  {   1, "tdd" },
  { 0, NULL }
};

static const per_choice_t T_modeSpecificInfo2_01_choice[] = {
  {   0, &hf_rrc_fdd_89          , ASN1_NO_EXTENSIONS     , dissect_rrc_T_fdd_88 },
  {   1, &hf_rrc_tdd_49          , ASN1_NO_EXTENSIONS     , dissect_rrc_T_tdd_46 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_modeSpecificInfo2_01(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_modeSpecificInfo2_01, T_modeSpecificInfo2_01_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t DL_InformationPerRL_r8_sequence[] = {
  { &hf_rrc_modeSpecificInfo_54, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_modeSpecificInfo_54 },
  { &hf_rrc_dl_dpchInfo_02  , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_dl_dpchInfo_02 },
  { &hf_rrc_e_AGCH_Information_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_E_AGCH_Information_r8 },
  { &hf_rrc_modeSpecificInfo2_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_modeSpecificInfo2_01 },
  { &hf_rrc_cell_id         , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CellIdentity },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_DL_InformationPerRL_r8(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_DL_InformationPerRL_r8, DL_InformationPerRL_r8_sequence);

  return offset;
}


static const per_sequence_t DL_InformationPerRL_List_r8_sequence_of[1] = {
  { &hf_rrc_DL_InformationPerRL_List_r8_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DL_InformationPerRL_r8 },
};

static int
dissect_rrc_DL_InformationPerRL_List_r8(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_DL_InformationPerRL_List_r8, DL_InformationPerRL_List_r8_sequence_of,
                                                  1, maxRL, FALSE);

  return offset;
}


static const value_string rrc_EnablingDelay_TDD128_vals[] = {
  {   0, "radio-frames-0" },
  {   1, "radio-frames-1" },
  {   2, "radio-frames-2" },
  {   3, "radio-frames-4" },
  {   4, "radio-frames-8" },
  {   5, "radio-frames-16" },
  {   6, "radio-frames-32" },
  {   7, "radio-frames-64" },
  {   8, "radio-frames-128" },
  {   9, "spare7" },
  {  10, "spare6" },
  {  11, "spare5" },
  {  12, "spare4" },
  {  13, "spare3" },
  {  14, "spare2" },
  {  15, "infinity" },
  { 0, NULL }
};


static int
dissect_rrc_EnablingDelay_TDD128(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     16, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t T_continue_sequence[] = {
  { &hf_rrc_enablingDelay   , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_EnablingDelay_TDD128 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_continue(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_continue, T_continue_sequence);

  return offset;
}


static const value_string rrc_ControlChannelDRXCycle_TDD128_vals[] = {
  {   0, "sub-frames-1" },
  {   1, "sub-frames-2" },
  {   2, "sub-frames-4" },
  {   3, "sub-frames-8" },
  {   4, "sub-frames-16" },
  {   5, "sub-frames-32" },
  {   6, "sub-frames-64" },
  {   7, "spare1" },
  { 0, NULL }
};


static int
dissect_rrc_ControlChannelDRXCycle_TDD128(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     8, NULL, FALSE, 0, NULL);

  return offset;
}


static const value_string rrc_HS_SCCH_DRX_InactivityThreshold_TDD128_vals[] = {
  {   0, "sub-frames-1" },
  {   1, "sub-frames-2" },
  {   2, "sub-frames-4" },
  {   3, "sub-frames-8" },
  {   4, "sub-frames-16" },
  {   5, "sub-frames-32" },
  {   6, "sub-frames-64" },
  {   7, "sub-frames-128" },
  {   8, "sub-frames-256" },
  {   9, "sub-frames-512" },
  {  10, "spare6" },
  {  11, "spare5" },
  {  12, "spare4" },
  {  13, "spare3" },
  {  14, "spare2" },
  {  15, "infinity" },
  { 0, NULL }
};


static int
dissect_rrc_HS_SCCH_DRX_InactivityThreshold_TDD128(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     16, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t HS_SCCH_DRX_Info_TDD128_sequence[] = {
  { &hf_rrc_hS_SCCH_DRX_Cycle, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_ControlChannelDRXCycle_TDD128 },
  { &hf_rrc_hS_SCCH_DRX_InactivityThreshold, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_HS_SCCH_DRX_InactivityThreshold_TDD128 },
  { &hf_rrc_hS_SCCH_DRX_Offset, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_0_63 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_HS_SCCH_DRX_Info_TDD128(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_HS_SCCH_DRX_Info_TDD128, HS_SCCH_DRX_Info_TDD128_sequence);

  return offset;
}


static const per_sequence_t T_e_AGCH_DRX_Parameters_sequence[] = {
  { &hf_rrc_e_AGCH_DRX_Cycle, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_ControlChannelDRXCycle_TDD128 },
  { &hf_rrc_e_AGCH_InactivityMonitorThreshold, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_E_AGCH_InactivityMonitorThreshold_TDD128 },
  { &hf_rrc_e_AGCH_DRX_Offset, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_0_63 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_e_AGCH_DRX_Parameters(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_e_AGCH_DRX_Parameters, T_e_AGCH_DRX_Parameters_sequence);

  return offset;
}


static const value_string rrc_T_e_AGCH_DRX_InfoType_vals[] = {
  {   0, "sameAsHS-SCCH" },
  {   1, "e-AGCH-DRX-Parameters" },
  { 0, NULL }
};

static const per_choice_t T_e_AGCH_DRX_InfoType_choice[] = {
  {   0, &hf_rrc_sameAsHS_SCCH   , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   1, &hf_rrc_e_AGCH_DRX_Parameters, ASN1_NO_EXTENSIONS     , dissect_rrc_T_e_AGCH_DRX_Parameters },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_e_AGCH_DRX_InfoType(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_e_AGCH_DRX_InfoType, T_e_AGCH_DRX_InfoType_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t E_AGCH_DRX_Info_TDD128_sequence[] = {
  { &hf_rrc_e_AGCH_DRX_InfoType, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_e_AGCH_DRX_InfoType },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_E_AGCH_DRX_Info_TDD128(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_E_AGCH_DRX_Info_TDD128, E_AGCH_DRX_Info_TDD128_sequence);

  return offset;
}


static const per_sequence_t T_newOperation_sequence[] = {
  { &hf_rrc_hS_SCCH_Drx_Info, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_HS_SCCH_DRX_Info_TDD128 },
  { &hf_rrc_e_AGCH_Drx_Info , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_E_AGCH_DRX_Info_TDD128 },
  { &hf_rrc_enablingDelay   , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_EnablingDelay_TDD128 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_newOperation(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_newOperation, T_newOperation_sequence);

  return offset;
}


static const value_string rrc_T_controlChannelDrxOperation_vals[] = {
  {   0, "continue" },
  {   1, "newOperation" },
  { 0, NULL }
};

static const per_choice_t T_controlChannelDrxOperation_choice[] = {
  {   0, &hf_rrc_continue_01     , ASN1_NO_EXTENSIONS     , dissect_rrc_T_continue },
  {   1, &hf_rrc_newOperation    , ASN1_NO_EXTENSIONS     , dissect_rrc_T_newOperation },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_controlChannelDrxOperation(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_controlChannelDrxOperation, T_controlChannelDrxOperation_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t ControlChannelDRXInfo_TDD128_r8_sequence[] = {
  { &hf_rrc_controlChannelDrxOperation, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_controlChannelDrxOperation },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_ControlChannelDRXInfo_TDD128_r8(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_ControlChannelDRXInfo_TDD128_r8, ControlChannelDRXInfo_TDD128_r8_sequence);

  return offset;
}


static const per_sequence_t T_implicit_sequence[] = {
  { &hf_rrc_ei              , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_INTEGER_0_3 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_implicit(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_implicit, T_implicit_sequence);

  return offset;
}


static const value_string rrc_T_midambleAllocationMode_02_vals[] = {
  {   0, "defaultMidamble" },
  {   1, "ueSpecificMidamble" },
  { 0, NULL }
};

static const per_choice_t T_midambleAllocationMode_02_choice[] = {
  {   0, &hf_rrc_defaultMidamble , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   1, &hf_rrc_ueSpecificMidamble, ASN1_NO_EXTENSIONS     , dissect_rrc_INTEGER_0_15 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_midambleAllocationMode_02(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_midambleAllocationMode_02, T_midambleAllocationMode_02_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t T_explicit_02_sequence[] = {
  { &hf_rrc_timeslotNumber_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TimeslotNumber_LCR_r4 },
  { &hf_rrc_channelisation_Code, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_HS_ChannelisationCode_LCR },
  { &hf_rrc_midambleAllocationMode_02, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_midambleAllocationMode_02 },
  { &hf_rrc_midambleConfiguration, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_1_8 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_explicit_02(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_explicit_02, T_explicit_02_sequence);

  return offset;
}


static const value_string rrc_T_configurationmode_vals[] = {
  {   0, "implicit" },
  {   1, "explicit" },
  { 0, NULL }
};

static const per_choice_t T_configurationmode_choice[] = {
  {   0, &hf_rrc_implicit_01     , ASN1_NO_EXTENSIONS     , dissect_rrc_T_implicit },
  {   1, &hf_rrc_explicit_03     , ASN1_NO_EXTENSIONS     , dissect_rrc_T_explicit_02 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_configurationmode(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_configurationmode, T_configurationmode_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t E_HICH_Information_For_SPS_TDD128_sequence[] = {
  { &hf_rrc_configurationmode, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_configurationmode },
  { &hf_rrc_signatureSequenceGroupIndex, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_0_19 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_E_HICH_Information_For_SPS_TDD128(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_E_HICH_Information_For_SPS_TDD128, E_HICH_Information_For_SPS_TDD128_sequence);

  return offset;
}


static const value_string rrc_RepetitionPeriodAndLengthForSPS_vals[] = {
  {   0, "repetitionPeriod1" },
  {   1, "repetitionPeriod2" },
  {   2, "repetitionPeriod4" },
  {   3, "repetitionPeriod8" },
  {   4, "repetitionPeriod16" },
  {   5, "repetitionPeriod32" },
  { 0, NULL }
};

static const per_choice_t RepetitionPeriodAndLengthForSPS_choice[] = {
  {   0, &hf_rrc_repetitionPeriod1, ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   1, &hf_rrc_repetitionPeriod2, ASN1_NO_EXTENSIONS     , dissect_rrc_INTEGER_1_1 },
  {   2, &hf_rrc_repetitionPeriod4, ASN1_NO_EXTENSIONS     , dissect_rrc_INTEGER_1_3 },
  {   3, &hf_rrc_repetitionPeriod8, ASN1_NO_EXTENSIONS     , dissect_rrc_INTEGER_1_7 },
  {   4, &hf_rrc_repetitionPeriod16, ASN1_NO_EXTENSIONS     , dissect_rrc_INTEGER_1_15 },
  {   5, &hf_rrc_repetitionPeriod32, ASN1_NO_EXTENSIONS     , dissect_rrc_INTEGER_1_31 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_RepetitionPeriodAndLengthForSPS(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_RepetitionPeriodAndLengthForSPS, RepetitionPeriodAndLengthForSPS_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t E_DCH_TxPatternList_TDD128_item_sequence[] = {
  { &hf_rrc_repetitionPeriodAndLength_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RepetitionPeriodAndLengthForSPS },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_E_DCH_TxPatternList_TDD128_item(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_E_DCH_TxPatternList_TDD128_item, E_DCH_TxPatternList_TDD128_item_sequence);

  return offset;
}


static const per_sequence_t E_DCH_TxPatternList_TDD128_sequence_of[1] = {
  { &hf_rrc_E_DCH_TxPatternList_TDD128_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_E_DCH_TxPatternList_TDD128_item },
};

static int
dissect_rrc_E_DCH_TxPatternList_TDD128(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_E_DCH_TxPatternList_TDD128, E_DCH_TxPatternList_TDD128_sequence_of,
                                                  1, maxEDCHTxPattern_TDD128, FALSE);

  return offset;
}



static int
dissect_rrc_INTEGER_0_maxEDCHTxPattern_TDD128_1(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            0U, maxEDCHTxPattern_TDD128_1, NULL, FALSE);

  return offset;
}


static const per_sequence_t T_initialSPSInfoForEDCH_sequence[] = {
  { &hf_rrc_n_E_UCCH        , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_1_8 },
  { &hf_rrc_codeResourceInfo, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UL_TS_ChannelisationCode },
  { &hf_rrc_timeslotResourceRelatedInfo, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_5 },
  { &hf_rrc_powerResourceRelatedInfo, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_1_32 },
  { &hf_rrc_activationTime  , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_ActivationTime },
  { &hf_rrc_subframeNum     , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_0_1 },
  { &hf_rrc_initialTxPatternIndex, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_0_maxEDCHTxPattern_TDD128_1 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_initialSPSInfoForEDCH(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_initialSPSInfoForEDCH, T_initialSPSInfoForEDCH_sequence);

  return offset;
}


static const per_sequence_t E_DCH_SPS_NewOperation_TDD128_sequence[] = {
  { &hf_rrc_e_hich_Info_01  , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_E_HICH_Information_For_SPS_TDD128 },
  { &hf_rrc_e_dch_TxPattern , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_E_DCH_TxPatternList_TDD128 },
  { &hf_rrc_initialSPSInfoForEDCH, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_initialSPSInfoForEDCH },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_E_DCH_SPS_NewOperation_TDD128(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_E_DCH_SPS_NewOperation_TDD128, E_DCH_SPS_NewOperation_TDD128_sequence);

  return offset;
}


static const value_string rrc_T_e_dch_SPS_Operation_vals[] = {
  {   0, "continue" },
  {   1, "newOperation" },
  { 0, NULL }
};

static const per_choice_t T_e_dch_SPS_Operation_choice[] = {
  {   0, &hf_rrc_continue        , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   1, &hf_rrc_newOperation_01 , ASN1_NO_EXTENSIONS     , dissect_rrc_E_DCH_SPS_NewOperation_TDD128 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_e_dch_SPS_Operation(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_e_dch_SPS_Operation, T_e_dch_SPS_Operation_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t E_DCH_SPS_Information_TDD128_sequence[] = {
  { &hf_rrc_e_dch_SPS_Operation, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_e_dch_SPS_Operation },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_E_DCH_SPS_Information_TDD128(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_E_DCH_SPS_Information_TDD128, E_DCH_SPS_Information_TDD128_sequence);

  return offset;
}


static const per_sequence_t HS_DSCH_TbsList_TDD128_item_sequence[] = {
  { &hf_rrc_hs_dsch_TBSizeIndex, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_1_63 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_HS_DSCH_TbsList_TDD128_item(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_HS_DSCH_TbsList_TDD128_item, HS_DSCH_TbsList_TDD128_item_sequence);

  return offset;
}


static const per_sequence_t HS_DSCH_TbsList_TDD128_sequence_of[1] = {
  { &hf_rrc_HS_DSCH_TbsList_TDD128_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_HS_DSCH_TbsList_TDD128_item },
};

static int
dissect_rrc_HS_DSCH_TbsList_TDD128(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_HS_DSCH_TbsList_TDD128, HS_DSCH_TbsList_TDD128_sequence_of,
                                                  1, maxTbsForHSDSCH_TDD128, FALSE);

  return offset;
}


static const per_sequence_t HS_DSCH_RxPatternList_TDD128_item_sequence[] = {
  { &hf_rrc_repetitionPeriodAndLength_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RepetitionPeriodAndLengthForSPS },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_HS_DSCH_RxPatternList_TDD128_item(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_HS_DSCH_RxPatternList_TDD128_item, HS_DSCH_RxPatternList_TDD128_item_sequence);

  return offset;
}


static const per_sequence_t HS_DSCH_RxPatternList_TDD128_sequence_of[1] = {
  { &hf_rrc_HS_DSCH_RxPatternList_TDD128_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_HS_DSCH_RxPatternList_TDD128_item },
};

static int
dissect_rrc_HS_DSCH_RxPatternList_TDD128(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_HS_DSCH_RxPatternList_TDD128, HS_DSCH_RxPatternList_TDD128_sequence_of,
                                                  1, maxRxPatternForHSDSCH_TDD128, FALSE);

  return offset;
}


static const per_sequence_t T_hARQInfoForSPS_sequence[] = {
  { &hf_rrc_numberOfProcesses, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_1_8 },
  { &hf_rrc_memorySize_01   , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_HARQMemorySize },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_hARQInfoForSPS(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_hARQInfoForSPS, T_hARQInfoForSPS_sequence);

  return offset;
}



static int
dissect_rrc_INTEGER_0_maxHSSCCHs_1(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            0U, maxHSSCCHs_1, NULL, FALSE);

  return offset;
}


static const per_sequence_t T_implicit_01_sequence[] = {
  { &hf_rrc_hS_SCCH_Index   , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_0_maxHSSCCHs_1 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_implicit_01(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_implicit_01, T_implicit_01_sequence);

  return offset;
}


static const per_sequence_t T_explicit_03_sequence[] = {
  { &hf_rrc_hS_SICH_Info    , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_HS_SICH_Configuration_TDD128_r6 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_explicit_03(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_explicit_03, T_explicit_03_sequence);

  return offset;
}


static const value_string rrc_HS_SICH_List_TDD128_item_vals[] = {
  {   0, "implicit" },
  {   1, "explicit" },
  { 0, NULL }
};

static const per_choice_t HS_SICH_List_TDD128_item_choice[] = {
  {   0, &hf_rrc_implicit_02     , ASN1_NO_EXTENSIONS     , dissect_rrc_T_implicit_01 },
  {   1, &hf_rrc_explicit_04     , ASN1_NO_EXTENSIONS     , dissect_rrc_T_explicit_03 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_HS_SICH_List_TDD128_item(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_HS_SICH_List_TDD128_item, HS_SICH_List_TDD128_item_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t HS_SICH_List_TDD128_sequence_of[1] = {
  { &hf_rrc_HS_SICH_List_TDD128_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_HS_SICH_List_TDD128_item },
};

static int
dissect_rrc_HS_SICH_List_TDD128(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_HS_SICH_List_TDD128, HS_SICH_List_TDD128_sequence_of,
                                                  1, maxHSSICH_TDD128, FALSE);

  return offset;
}


static const per_sequence_t HS_ChannelisationCodeSetInfo_LCR_sequence[] = {
  { &hf_rrc_startCode       , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_HS_ChannelisationCode_LCR },
  { &hf_rrc_stopCode        , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_HS_ChannelisationCode_LCR },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_HS_ChannelisationCodeSetInfo_LCR(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_HS_ChannelisationCodeSetInfo_LCR, HS_ChannelisationCodeSetInfo_LCR_sequence);

  return offset;
}



static int
dissect_rrc_INTEGER_0_maxRxPatternForHSDSCH_TDD128_1(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            0U, maxRxPatternForHSDSCH_TDD128_1, NULL, FALSE);

  return offset;
}



static int
dissect_rrc_INTEGER_0_maxTbsForHSDSCH_TDD128_1(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            0U, maxTbsForHSDSCH_TDD128_1, NULL, FALSE);

  return offset;
}


static const value_string rrc_T_modulation_vals[] = {
  {   0, "modQPSK" },
  {   1, "mod16QAM" },
  { 0, NULL }
};


static int
dissect_rrc_T_modulation(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     2, NULL, FALSE, 0, NULL);

  return offset;
}



static int
dissect_rrc_INTEGER_0_maxHSSICH_TDD128_1(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            0U, maxHSSICH_TDD128_1, NULL, FALSE);

  return offset;
}


static const per_sequence_t T_initialSPSInfoForHSDSCH_sequence[] = {
  { &hf_rrc_timeslotInfo    , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_5 },
  { &hf_rrc_codeResourceInfo_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_HS_ChannelisationCodeSetInfo_LCR },
  { &hf_rrc_activationTime  , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_ActivationTime },
  { &hf_rrc_sfnNum          , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_0_1 },
  { &hf_rrc_initialRxPatternIndex, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_0_maxRxPatternForHSDSCH_TDD128_1 },
  { &hf_rrc_initialTfsIndex , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_0_maxTbsForHSDSCH_TDD128_1 },
  { &hf_rrc_modulation      , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_modulation },
  { &hf_rrc_hs_sich_Index   , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_0_maxHSSICH_TDD128_1 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_initialSPSInfoForHSDSCH(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_initialSPSInfoForHSDSCH, T_initialSPSInfoForHSDSCH_sequence);

  return offset;
}


static const per_sequence_t HS_DSCH_SPS_NewOperation_TDD128_sequence[] = {
  { &hf_rrc_hs_dsch_TbsList , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_HS_DSCH_TbsList_TDD128 },
  { &hf_rrc_hs_dsch_RxPatternList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_HS_DSCH_RxPatternList_TDD128 },
  { &hf_rrc_hARQInfoForSPS  , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_hARQInfoForSPS },
  { &hf_rrc_hs_sich_List    , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_HS_SICH_List_TDD128 },
  { &hf_rrc_initialSPSInfoForHSDSCH, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_initialSPSInfoForHSDSCH },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_HS_DSCH_SPS_NewOperation_TDD128(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_HS_DSCH_SPS_NewOperation_TDD128, HS_DSCH_SPS_NewOperation_TDD128_sequence);

  return offset;
}


static const value_string rrc_T_hs_dsch_SPS_Operation_vals[] = {
  {   0, "continue" },
  {   1, "newOperation" },
  { 0, NULL }
};

static const per_choice_t T_hs_dsch_SPS_Operation_choice[] = {
  {   0, &hf_rrc_continue        , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   1, &hf_rrc_newOperation_02 , ASN1_NO_EXTENSIONS     , dissect_rrc_HS_DSCH_SPS_NewOperation_TDD128 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_hs_dsch_SPS_Operation(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_hs_dsch_SPS_Operation, T_hs_dsch_SPS_Operation_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t HS_DSCH_SPS_Information_TDD128_sequence[] = {
  { &hf_rrc_hs_dsch_SPS_Operation, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_hs_dsch_SPS_Operation },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_HS_DSCH_SPS_Information_TDD128(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_HS_DSCH_SPS_Information_TDD128, HS_DSCH_SPS_Information_TDD128_sequence);

  return offset;
}


static const per_sequence_t SPS_Information_TDD128_r8_sequence[] = {
  { &hf_rrc_e_dch_SPS_Info  , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_E_DCH_SPS_Information_TDD128 },
  { &hf_rrc_hs_dsch_SPS_Info, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_HS_DSCH_SPS_Information_TDD128 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_SPS_Information_TDD128_r8(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_SPS_Information_TDD128_r8, SPS_Information_TDD128_r8_sequence);

  return offset;
}


static const per_sequence_t CellUpdateConfirm_r8_IEs_sequence[] = {
  { &hf_rrc_integrityProtectionModeInfo_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_IntegrityProtectionModeInfo_r7 },
  { &hf_rrc_cipheringModeInfo_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CipheringModeInfo_r7 },
  { &hf_rrc_activationTime  , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_ActivationTime },
  { &hf_rrc_new_U_RNTI      , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_U_RNTI },
  { &hf_rrc_new_C_RNTI      , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_C_RNTI },
  { &hf_rrc_new_DSCH_RNTI   , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DSCH_RNTI },
  { &hf_rrc_new_H_RNTI      , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_H_RNTI },
  { &hf_rrc_newPrimary_E_RNTI, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_E_RNTI },
  { &hf_rrc_newSecondary_E_RNTI, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_E_RNTI },
  { &hf_rrc_rrc_StateIndicator, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RRC_StateIndicator },
  { &hf_rrc_utran_DRX_CycleLengthCoeff_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UTRAN_DRX_CycleLengthCoefficient_r7 },
  { &hf_rrc_waitTime        , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_WaitTime },
  { &hf_rrc_rlc_Re_establishIndicatorRb2_3or4, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { &hf_rrc_rlc_Re_establishIndicatorRb5orAbove, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { &hf_rrc_cn_InformationInfo_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CN_InformationInfo_r6 },
  { &hf_rrc_ura_Identity    , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_URA_Identity },
  { &hf_rrc_supportForChangeOfUE_Capability, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_BOOLEAN },
  { &hf_rrc_dummy_07        , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DefaultConfigForCellFACH },
  { &hf_rrc_rab_InformationSetup, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RAB_InformationSetup_r8 },
  { &hf_rrc_rb_InformationReleaseList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RB_InformationReleaseList },
  { &hf_rrc_rb_InformationReconfigList_05, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RB_InformationReconfigList_r8 },
  { &hf_rrc_rb_InformationAffectedList_04, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RB_InformationAffectedList_r8 },
  { &hf_rrc_dl_CounterSynchronisationInfo_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_CounterSynchronisationInfo_r5 },
  { &hf_rrc_pdcp_ROHC_TargetMode, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_PDCP_ROHC_TargetMode },
  { &hf_rrc_ul_CommonTransChInfo_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_CommonTransChInfo_r4 },
  { &hf_rrc_ul_deletedTransChInfoList_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_DeletedTransChInfoList_r6 },
  { &hf_rrc_ul_AddReconfTransChInfoList_03, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_AddReconfTransChInfoList_r8 },
  { &hf_rrc_dl_CommonTransChInfo_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_CommonTransChInfo_r4 },
  { &hf_rrc_dl_DeletedTransChInfoList_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_DeletedTransChInfoList_r7 },
  { &hf_rrc_dl_AddReconfTransChInfoList_03, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_AddReconfTransChInfoList_r7 },
  { &hf_rrc_frequencyInfo   , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_FrequencyInfo },
  { &hf_rrc_multi_frequencyInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_Multi_frequencyInfo_LCR_r7 },
  { &hf_rrc_dtx_drx_TimingInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DTX_DRX_TimingInfo_r7 },
  { &hf_rrc_dtx_drx_Info    , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DTX_DRX_Info_r7 },
  { &hf_rrc_hs_scch_LessInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_HS_SCCH_LessInfo_r7 },
  { &hf_rrc_mimoParameters_03, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MIMO_Parameters_r8 },
  { &hf_rrc_maxAllowedUL_TX_Power, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MaxAllowedUL_TX_Power },
  { &hf_rrc_ul_DPCH_Info_01 , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_DPCH_Info_r7 },
  { &hf_rrc_ul_EDCH_Information_03, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_EDCH_Information_r8 },
  { &hf_rrc_dl_HSPDSCH_Information_03, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_HSPDSCH_Information_r8 },
  { &hf_rrc_dl_CommonInformation_05, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_CommonInformation_r8 },
  { &hf_rrc_dl_InformationPerRL_List_05, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_InformationPerRL_List_r8 },
  { &hf_rrc_dl_SecondaryCellInfoFDD, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_SecondaryCellInfoFDD },
  { &hf_rrc_controlChannelDRXInfo_TDD128, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_ControlChannelDRXInfo_TDD128_r8 },
  { &hf_rrc_sps_Information_TDD128, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_SPS_Information_TDD128_r8 },
  { &hf_rrc_mbms_PL_ServiceRestrictInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MBMS_PL_ServiceRestrictInfo_r6 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_CellUpdateConfirm_r8_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_CellUpdateConfirm_r8_IEs, CellUpdateConfirm_r8_IEs_sequence);

  return offset;
}


static const value_string rrc_T_t_SI_nst_vals[] = {
  {   0, "everyEDCHTTI" },
  {   1, "ms20" },
  {   2, "ms40" },
  {   3, "ms60" },
  {   4, "ms80" },
  {   5, "ms160" },
  {   6, "ms200" },
  { 0, NULL }
};


static int
dissect_rrc_T_t_SI_nst(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     7, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t T_tdd128_42_sequence[] = {
  { &hf_rrc_t_SI_nst        , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_t_SI_nst },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_tdd128_42(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_tdd128_42, T_tdd128_42_sequence);

  return offset;
}


static const value_string rrc_Non_ScheduledTransGrantInfoTDD_ext_vals[] = {
  {   0, "tdd384-768" },
  {   1, "tdd128" },
  { 0, NULL }
};

static const per_choice_t Non_ScheduledTransGrantInfoTDD_ext_choice[] = {
  {   0, &hf_rrc_tdd384_768_01   , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   1, &hf_rrc_tdd128_43       , ASN1_NO_EXTENSIONS     , dissect_rrc_T_tdd128_42 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_Non_ScheduledTransGrantInfoTDD_ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_Non_ScheduledTransGrantInfoTDD_ext, Non_ScheduledTransGrantInfoTDD_ext_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t T_tdd_78_sequence[] = {
  { &hf_rrc_non_ScheduledTransGrantInfo_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_Non_ScheduledTransGrantInfoTDD_ext },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_tdd_78(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_tdd_78, T_tdd_78_sequence);

  return offset;
}


static const value_string rrc_UL_EDCH_Information_ext_vals[] = {
  {   0, "fdd" },
  {   1, "tdd" },
  { 0, NULL }
};

static const per_choice_t UL_EDCH_Information_ext_choice[] = {
  {   0, &hf_rrc_fdd_45          , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   1, &hf_rrc_tdd_84          , ASN1_NO_EXTENSIONS     , dissect_rrc_T_tdd_78 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_UL_EDCH_Information_ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_UL_EDCH_Information_ext, UL_EDCH_Information_ext_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t CellUpdateConfirm_v7d0ext_IEs_sequence[] = {
  { &hf_rrc_ul_EDCH_Information_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_EDCH_Information_ext },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_CellUpdateConfirm_v7d0ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_CellUpdateConfirm_v7d0ext_IEs, CellUpdateConfirm_v7d0ext_IEs_sequence);

  return offset;
}


static const per_sequence_t CellUpdateConfirm_v7f0ext_IEs_sequence[] = {
  { &hf_rrc_mimoParameters_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MIMO_Parameters_v7f0ext },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_CellUpdateConfirm_v7f0ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_CellUpdateConfirm_v7f0ext_IEs, CellUpdateConfirm_v7f0ext_IEs_sequence);

  return offset;
}


static const per_sequence_t T_tdd128_25_sequence[] = {
  { &hf_rrc_outofSyncWindow , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_OutofSyncWindow },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_tdd128_25(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_tdd128_25, T_tdd128_25_sequence);

  return offset;
}


static const value_string rrc_T_tdd_42_vals[] = {
  {   0, "tdd384" },
  {   1, "tdd768" },
  {   2, "tdd128" },
  { 0, NULL }
};

static const per_choice_t T_tdd_42_choice[] = {
  {   0, &hf_rrc_tdd384_11       , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   1, &hf_rrc_tdd768_06       , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   2, &hf_rrc_tdd128_25       , ASN1_NO_EXTENSIONS     , dissect_rrc_T_tdd128_25 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_tdd_42(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_tdd_42, T_tdd_42_choice,
                                 NULL);

  return offset;
}


static const value_string rrc_T_modeSpecificInfo_45_vals[] = {
  {   0, "tdd" },
  {   1, "fdd" },
  { 0, NULL }
};

static const per_choice_t T_modeSpecificInfo_45_choice[] = {
  {   0, &hf_rrc_tdd_43          , ASN1_NO_EXTENSIONS     , dissect_rrc_T_tdd_42 },
  {   1, &hf_rrc_fdd_45          , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_modeSpecificInfo_45(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_modeSpecificInfo_45, T_modeSpecificInfo_45_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t DL_HSPDSCH_Information_r8_ext_sequence[] = {
  { &hf_rrc_modeSpecificInfo_45, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_modeSpecificInfo_45 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_DL_HSPDSCH_Information_r8_ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_DL_HSPDSCH_Information_r8_ext, DL_HSPDSCH_Information_r8_ext_sequence);

  return offset;
}


static const per_sequence_t CellUpdateConfirm_v890ext_IEs_sequence[] = {
  { &hf_rrc_dl_SecondaryCellInfoFDD_v890ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_SecondaryCellInfoFDD_v890ext },
  { &hf_rrc_dl_HSPDSCH_Information_04, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_HSPDSCH_Information_r8_ext },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_CellUpdateConfirm_v890ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_CellUpdateConfirm_v890ext_IEs, CellUpdateConfirm_v890ext_IEs_sequence);

  return offset;
}


static const per_sequence_t CellUpdateConfirm_v7g0ext_IEs_sequence[] = {
  { &hf_rrc_mimoParameters_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MIMO_Parameters_v7g0ext },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_CellUpdateConfirm_v7g0ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_CellUpdateConfirm_v7g0ext_IEs, CellUpdateConfirm_v7g0ext_IEs_sequence);

  return offset;
}


static const value_string rrc_T_hS_SCCH_tpc_step_size_vals[] = {
  {   0, "s1" },
  {   1, "s2" },
  {   2, "s3" },
  {   3, "spare1" },
  { 0, NULL }
};


static int
dissect_rrc_T_hS_SCCH_tpc_step_size(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     4, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t T_tdd128_37_sequence[] = {
  { &hf_rrc_hS_SCCH_tpc_step_size, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_hS_SCCH_tpc_step_size },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_tdd128_37(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_tdd128_37, T_tdd128_37_sequence);

  return offset;
}


static const value_string rrc_T_tdd_53_vals[] = {
  {   0, "tdd384" },
  {   1, "tdd768" },
  {   2, "tdd128" },
  { 0, NULL }
};

static const per_choice_t T_tdd_53_choice[] = {
  {   0, &hf_rrc_tdd384_11       , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   1, &hf_rrc_tdd768_06       , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   2, &hf_rrc_tdd128_38       , ASN1_NO_EXTENSIONS     , dissect_rrc_T_tdd128_37 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_tdd_53(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_tdd_53, T_tdd_53_choice,
                                 NULL);

  return offset;
}


static const value_string rrc_T_modeSpecificInfo_65_vals[] = {
  {   0, "fdd" },
  {   1, "tdd" },
  { 0, NULL }
};

static const per_choice_t T_modeSpecificInfo_65_choice[] = {
  {   0, &hf_rrc_fdd_45          , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   1, &hf_rrc_tdd_57          , ASN1_NO_EXTENSIONS     , dissect_rrc_T_tdd_53 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_modeSpecificInfo_65(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_modeSpecificInfo_65, T_modeSpecificInfo_65_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t HS_SCCH_Info_r8_ext_sequence[] = {
  { &hf_rrc_modeSpecificInfo_65, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_modeSpecificInfo_65 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_HS_SCCH_Info_r8_ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_HS_SCCH_Info_r8_ext, HS_SCCH_Info_r8_ext_sequence);

  return offset;
}


static const per_sequence_t DL_HSPDSCH_Information_r8_ext2_sequence[] = {
  { &hf_rrc_hs_scch_Info_03 , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_HS_SCCH_Info_r8_ext },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_DL_HSPDSCH_Information_r8_ext2(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_DL_HSPDSCH_Information_r8_ext2, DL_HSPDSCH_Information_r8_ext2_sequence);

  return offset;
}


static const per_sequence_t CellUpdateConfirm_v8a0ext_IEs_sequence[] = {
  { &hf_rrc_dl_HSPDSCH_Information_05, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_HSPDSCH_Information_r8_ext2 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_CellUpdateConfirm_v8a0ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_CellUpdateConfirm_v8a0ext_IEs, CellUpdateConfirm_v8a0ext_IEs_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions_19_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_nonCriticalExtensions_19(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_nonCriticalExtensions_19, T_nonCriticalExtensions_19_sequence);

  return offset;
}


static const per_sequence_t T_v8a0NonCriticalExtensions_sequence[] = {
  { &hf_rrc_cellUpdateConfirm_v8a0ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CellUpdateConfirm_v8a0ext_IEs },
  { &hf_rrc_nonCriticalExtensions_19, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_nonCriticalExtensions_19 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v8a0NonCriticalExtensions(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v8a0NonCriticalExtensions, T_v8a0NonCriticalExtensions_sequence);

  return offset;
}


static const per_sequence_t T_v7g0NonCriticalExtensions_03_sequence[] = {
  { &hf_rrc_cellUpdateConfirm_v7g0ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CellUpdateConfirm_v7g0ext_IEs },
  { &hf_rrc_v8a0NonCriticalExtensions, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v8a0NonCriticalExtensions },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v7g0NonCriticalExtensions_03(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v7g0NonCriticalExtensions_03, T_v7g0NonCriticalExtensions_03_sequence);

  return offset;
}


static const per_sequence_t T_v890NonCriticalExtensions_01_sequence[] = {
  { &hf_rrc_cellUpdateConfirm_v890ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CellUpdateConfirm_v890ext_IEs },
  { &hf_rrc_v7g0NonCriticalExtensions_03, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v7g0NonCriticalExtensions_03 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v890NonCriticalExtensions_01(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v890NonCriticalExtensions_01, T_v890NonCriticalExtensions_01_sequence);

  return offset;
}


static const per_sequence_t T_v7f0NonCriticalExtensions_01_sequence[] = {
  { &hf_rrc_cellUpdateConfirm_v7f0ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CellUpdateConfirm_v7f0ext_IEs },
  { &hf_rrc_v890NonCriticalExtensions_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v890NonCriticalExtensions_01 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v7f0NonCriticalExtensions_01(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v7f0NonCriticalExtensions_01, T_v7f0NonCriticalExtensions_01_sequence);

  return offset;
}


static const per_sequence_t T_v7d0NonCriticalExtensions_sequence[] = {
  { &hf_rrc_cellUpdateConfirm_v7d0ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CellUpdateConfirm_v7d0ext_IEs },
  { &hf_rrc_v7f0NonCriticalExtensions_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v7f0NonCriticalExtensions_01 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v7d0NonCriticalExtensions(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v7d0NonCriticalExtensions, T_v7d0NonCriticalExtensions_sequence);

  return offset;
}


static const per_sequence_t T_r8_01_sequence[] = {
  { &hf_rrc_cellUpdateConfirm_r8, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CellUpdateConfirm_r8_IEs },
  { &hf_rrc_cellUpdateConfirm_r8_add_ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_BIT_STRING },
  { &hf_rrc_v7d0NonCriticalExtensions, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v7d0NonCriticalExtensions },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_r8_01(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_r8_01, T_r8_01_sequence);

  return offset;
}


static const value_string rrc_MAC_hs_WindowSize_r9_vals[] = {
  {   0, "mws4" },
  {   1, "mws6" },
  {   2, "mws8" },
  {   3, "mws12" },
  {   4, "mws16" },
  {   5, "mws24" },
  {   6, "mws32" },
  {   7, "mw64" },
  {   8, "mw128" },
  { 0, NULL }
};


static int
dissect_rrc_MAC_hs_WindowSize_r9(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     9, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t MAC_ehs_AddReconfReordQ_r9_sequence[] = {
  { &hf_rrc_mac_ehs_QueueId , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MAC_ehs_QueueId },
  { &hf_rrc_reorderingReleaseTimer, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T1_ReleaseTimer },
  { &hf_rrc_reorderingResetTimer, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_Treset_ResetTimer },
  { &hf_rrc_mac_ehsWindowSize, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MAC_hs_WindowSize_r9 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_MAC_ehs_AddReconfReordQ_r9(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_MAC_ehs_AddReconfReordQ_r9, MAC_ehs_AddReconfReordQ_r9_sequence);

  return offset;
}


static const per_sequence_t MAC_ehs_AddReconfReordQ_List_r9_sequence_of[1] = {
  { &hf_rrc_MAC_ehs_AddReconfReordQ_List_r9_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MAC_ehs_AddReconfReordQ_r9 },
};

static int
dissect_rrc_MAC_ehs_AddReconfReordQ_List_r9(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_MAC_ehs_AddReconfReordQ_List_r9, MAC_ehs_AddReconfReordQ_List_r9_sequence_of,
                                                  1, maxQueueIDs, FALSE);

  return offset;
}


static const per_sequence_t AddOrReconfMAC_ehs_ReordQ_r9_sequence[] = {
  { &hf_rrc_mac_ehs_AddReconfQueue_List_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MAC_ehs_AddReconfReordQ_List_r9 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_AddOrReconfMAC_ehs_ReordQ_r9(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_AddOrReconfMAC_ehs_ReordQ_r9, AddOrReconfMAC_ehs_ReordQ_r9_sequence);

  return offset;
}


static const value_string rrc_T_dl_MAC_HeaderType_01_vals[] = {
  {   0, "mac-hs" },
  {   1, "mac-ehs" },
  { 0, NULL }
};

static const per_choice_t T_dl_MAC_HeaderType_01_choice[] = {
  {   0, &hf_rrc_mac_hs_02       , ASN1_NO_EXTENSIONS     , dissect_rrc_AddOrReconfMAC_dFlow },
  {   1, &hf_rrc_mac_ehs_03      , ASN1_NO_EXTENSIONS     , dissect_rrc_AddOrReconfMAC_ehs_ReordQ_r9 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_dl_MAC_HeaderType_01(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_dl_MAC_HeaderType_01, T_dl_MAC_HeaderType_01_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t HSDSCH_Info_r9_sequence[] = {
  { &hf_rrc_harqInfo        , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_HARQ_Info_r7 },
  { &hf_rrc_dl_MAC_HeaderType_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_dl_MAC_HeaderType_01 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_HSDSCH_Info_r9(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_HSDSCH_Info_r9, HSDSCH_Info_r9_sequence);

  return offset;
}


static const value_string rrc_T_tfs_SignallingMode_04_vals[] = {
  {   0, "explicit-config" },
  {   1, "sameAsULTrCH" },
  {   2, "hsdsch" },
  { 0, NULL }
};

static const per_choice_t T_tfs_SignallingMode_04_choice[] = {
  {   0, &hf_rrc_explicit_config , ASN1_NO_EXTENSIONS     , dissect_rrc_TransportFormatSet },
  {   1, &hf_rrc_sameAsULTrCH    , ASN1_NO_EXTENSIONS     , dissect_rrc_UL_TransportChannelIdentity },
  {   2, &hf_rrc_hsdsch_04       , ASN1_NO_EXTENSIONS     , dissect_rrc_HSDSCH_Info_r9 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_tfs_SignallingMode_04(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_tfs_SignallingMode_04, T_tfs_SignallingMode_04_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t DL_AddReconfTransChInformation_r9_sequence[] = {
  { &hf_rrc_dl_TransportChannelType_04, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DL_TrCH_TypeId1_r5 },
  { &hf_rrc_tfs_SignallingMode_04, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_tfs_SignallingMode_04 },
  { &hf_rrc_dch_QualityTarget, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_QualityTarget },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_DL_AddReconfTransChInformation_r9(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_DL_AddReconfTransChInformation_r9, DL_AddReconfTransChInformation_r9_sequence);

  return offset;
}


static const per_sequence_t DL_AddReconfTransChInfoList_r9_sequence_of[1] = {
  { &hf_rrc_DL_AddReconfTransChInfoList_r9_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DL_AddReconfTransChInformation_r9 },
};

static int
dissect_rrc_DL_AddReconfTransChInfoList_r9(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_DL_AddReconfTransChInfoList_r9, DL_AddReconfTransChInfoList_r9_sequence_of,
                                                  1, maxTrCHpreconf, FALSE);

  return offset;
}


static const value_string rrc_T_mac_es_e_resetIndicator_04_vals[] = {
  {   0, "true" },
  { 0, NULL }
};


static int
dissect_rrc_T_mac_es_e_resetIndicator_04(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t T_fdd_130_sequence[] = {
  { &hf_rrc_e_DPCCH_Info_01 , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_E_DPCCH_Info_r7 },
  { &hf_rrc_e_DPDCH_Info_02 , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_E_DPDCH_Info_r8 },
  { &hf_rrc_schedulingTransmConfiguration, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_E_DPDCH_SchedulingTransmConfiguration },
  { &hf_rrc_ul_16QAM_Settings, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_16QAM_Settings },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_fdd_130(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_fdd_130, T_fdd_130_sequence);

  return offset;
}


static const per_sequence_t T_tdd384_768_01_sequence[] = {
  { &hf_rrc_timeslotResourceRelatedInfo_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_13 },
  { &hf_rrc_powerResourceRelatedInfo, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_1_32 },
  { &hf_rrc_activationTime  , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_ActivationTime },
  { &hf_rrc_repetitionPeriodAndLength, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RepetitionPeriodAndLength },
  { &hf_rrc_codeResourceInfo, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UL_TS_ChannelisationCode },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_tdd384_768_01(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_tdd384_768_01, T_tdd384_768_01_sequence);

  return offset;
}


static const value_string rrc_T_midambleAllocationMode_32_vals[] = {
  {   0, "defaultMidamble" },
  {   1, "ueSpecificMidamble" },
  { 0, NULL }
};

static const per_choice_t T_midambleAllocationMode_32_choice[] = {
  {   0, &hf_rrc_defaultMidamble , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   1, &hf_rrc_ueSpecificMidamble, ASN1_NO_EXTENSIONS     , dissect_rrc_INTEGER_0_15 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_midambleAllocationMode_32(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_midambleAllocationMode_32, T_midambleAllocationMode_32_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t T_e_HICH_Info_07_sequence[] = {
  { &hf_rrc_timeslotNumber_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TimeslotNumber_LCR_r4 },
  { &hf_rrc_channelisation_Code, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_HS_ChannelisationCode_LCR },
  { &hf_rrc_midambleAllocationMode_32, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_midambleAllocationMode_32 },
  { &hf_rrc_midambleConfiguration, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_1_8 },
  { &hf_rrc_signatureSequenceGroupIndex, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_0_19 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_e_HICH_Info_07(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_e_HICH_Info_07, T_e_HICH_Info_07_sequence);

  return offset;
}


static const value_string rrc_T_t_SI_nst_01_vals[] = {
  {   0, "everyEDCHTTI" },
  {   1, "ms20" },
  {   2, "ms40" },
  {   3, "ms60" },
  {   4, "ms80" },
  {   5, "ms160" },
  {   6, "ms200" },
  { 0, NULL }
};


static int
dissect_rrc_T_t_SI_nst_01(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     7, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t T_tdd128_43_sequence[] = {
  { &hf_rrc_n_E_UCCH        , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_INTEGER_1_8 },
  { &hf_rrc_n_E_HICH_01     , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_INTEGER_4_15 },
  { &hf_rrc_timeslotResourceRelatedInfo, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_5 },
  { &hf_rrc_powerResourceRelatedInfo, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_1_32 },
  { &hf_rrc_activationTime  , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_ActivationTime },
  { &hf_rrc_sfnNum          , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_0_1 },
  { &hf_rrc_repetitionPeriodAndLength, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RepetitionPeriodAndLength },
  { &hf_rrc_codeResourceInfo, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UL_TS_ChannelisationCode },
  { &hf_rrc_e_HICH_Info_09  , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_e_HICH_Info_07 },
  { &hf_rrc_t_SI_nst_01     , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_t_SI_nst_01 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_tdd128_43(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_tdd128_43, T_tdd128_43_sequence);

  return offset;
}


static const value_string rrc_Non_ScheduledTransGrantInfoTDD_r9_vals[] = {
  {   0, "tdd384-768" },
  {   1, "tdd128" },
  { 0, NULL }
};

static const per_choice_t Non_ScheduledTransGrantInfoTDD_r9_choice[] = {
  {   0, &hf_rrc_tdd384_768_02   , ASN1_NO_EXTENSIONS     , dissect_rrc_T_tdd384_768_01 },
  {   1, &hf_rrc_tdd128_44       , ASN1_NO_EXTENSIONS     , dissect_rrc_T_tdd128_43 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_Non_ScheduledTransGrantInfoTDD_r9(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_Non_ScheduledTransGrantInfoTDD_r9, Non_ScheduledTransGrantInfoTDD_r9_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t T_tdd_81_sequence[] = {
  { &hf_rrc_e_RUCCH_Info_01 , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_E_RUCCH_Info },
  { &hf_rrc_e_PUCH_Info_01  , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_E_PUCH_Info },
  { &hf_rrc_non_ScheduledTransGrantInfo_04, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_Non_ScheduledTransGrantInfoTDD_r9 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_tdd_81(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_tdd_81, T_tdd_81_sequence);

  return offset;
}


static const value_string rrc_T_modeSpecificInfo_91_vals[] = {
  {   0, "fdd" },
  {   1, "tdd" },
  { 0, NULL }
};

static const per_choice_t T_modeSpecificInfo_91_choice[] = {
  {   0, &hf_rrc_fdd_133         , ASN1_NO_EXTENSIONS     , dissect_rrc_T_fdd_130 },
  {   1, &hf_rrc_tdd_87          , ASN1_NO_EXTENSIONS     , dissect_rrc_T_tdd_81 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_modeSpecificInfo_91(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_modeSpecificInfo_91, T_modeSpecificInfo_91_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t UL_EDCH_Information_r9_sequence[] = {
  { &hf_rrc_mac_es_e_resetIndicator_04, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_mac_es_e_resetIndicator_04 },
  { &hf_rrc_modeSpecificInfo_91, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_modeSpecificInfo_91 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UL_EDCH_Information_r9(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UL_EDCH_Information_r9, UL_EDCH_Information_r9_sequence);

  return offset;
}



static int
dissect_rrc_INTEGER_0_maxMeasOccasionPattern_1(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            0U, maxMeasOccasionPattern_1, NULL, FALSE);

  return offset;
}


static const value_string rrc_T_statusFlag_vals[] = {
  {   0, "activate" },
  {   1, "deactivate" },
  { 0, NULL }
};


static int
dissect_rrc_T_statusFlag(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     2, NULL, FALSE, 0, NULL);

  return offset;
}



static int
dissect_rrc_INTEGER_1_9(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            1U, 9U, NULL, FALSE);

  return offset;
}



static int
dissect_rrc_INTEGER_1_512(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            1U, 512U, NULL, FALSE);

  return offset;
}


static const per_sequence_t MeasurementOccasionPatternParameter_sequence[] = {
  { &hf_rrc_measurement_Occasion_Coeff, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_1_9 },
  { &hf_rrc_measurement_Occasion_Offset, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_0_511 },
  { &hf_rrc_measurement_Occasion_Length, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_1_512 },
  { &hf_rrc_timeslotBitmap  , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_BIT_STRING_SIZE_7 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_MeasurementOccasionPatternParameter(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_MeasurementOccasionPatternParameter, MeasurementOccasionPatternParameter_sequence);

  return offset;
}


static const per_sequence_t CellDCHMeasOccasionPattern_LCR_sequence[] = {
  { &hf_rrc_patternIdentifier, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_0_maxMeasOccasionPattern_1 },
  { &hf_rrc_statusFlag      , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_statusFlag },
  { &hf_rrc_measurementPurpose, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_BIT_STRING_SIZE_5 },
  { &hf_rrc_measurementOccasionPatternParameter, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MeasurementOccasionPatternParameter },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_CellDCHMeasOccasionPattern_LCR(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_CellDCHMeasOccasionPattern_LCR, CellDCHMeasOccasionPattern_LCR_sequence);

  return offset;
}


static const per_sequence_t SEQUENCE_SIZE_1_maxMeasOccasionPattern_OF_CellDCHMeasOccasionPattern_LCR_sequence_of[1] = {
  { &hf_rrc_cellDCHMeasOccasionSequenceList_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CellDCHMeasOccasionPattern_LCR },
};

static int
dissect_rrc_SEQUENCE_SIZE_1_maxMeasOccasionPattern_OF_CellDCHMeasOccasionPattern_LCR(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_SEQUENCE_SIZE_1_maxMeasOccasionPattern_OF_CellDCHMeasOccasionPattern_LCR, SEQUENCE_SIZE_1_maxMeasOccasionPattern_OF_CellDCHMeasOccasionPattern_LCR_sequence_of,
                                                  1, maxMeasOccasionPattern, FALSE);

  return offset;
}


static const per_sequence_t CellDCHMeasOccasionInfo_TDD128_r9_sequence[] = {
  { &hf_rrc_cellDCHMeasOccasionSequenceList, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SEQUENCE_SIZE_1_maxMeasOccasionPattern_OF_CellDCHMeasOccasionPattern_LCR },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_CellDCHMeasOccasionInfo_TDD128_r9(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_CellDCHMeasOccasionInfo_TDD128_r9, CellDCHMeasOccasionInfo_TDD128_r9_sequence);

  return offset;
}


static const per_sequence_t CellUpdateConfirm_r9_IEs_sequence[] = {
  { &hf_rrc_integrityProtectionModeInfo_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_IntegrityProtectionModeInfo_r7 },
  { &hf_rrc_cipheringModeInfo_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CipheringModeInfo_r7 },
  { &hf_rrc_activationTime  , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_ActivationTime },
  { &hf_rrc_new_U_RNTI      , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_U_RNTI },
  { &hf_rrc_new_C_RNTI      , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_C_RNTI },
  { &hf_rrc_new_DSCH_RNTI   , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DSCH_RNTI },
  { &hf_rrc_new_H_RNTI      , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_H_RNTI },
  { &hf_rrc_newPrimary_E_RNTI, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_E_RNTI },
  { &hf_rrc_newSecondary_E_RNTI, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_E_RNTI },
  { &hf_rrc_rrc_StateIndicator, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RRC_StateIndicator },
  { &hf_rrc_utran_DRX_CycleLengthCoeff_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UTRAN_DRX_CycleLengthCoefficient_r7 },
  { &hf_rrc_waitTime        , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_WaitTime },
  { &hf_rrc_rlc_Re_establishIndicatorRb2_3or4, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { &hf_rrc_rlc_Re_establishIndicatorRb5orAbove, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { &hf_rrc_cn_InformationInfo_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CN_InformationInfo_r6 },
  { &hf_rrc_ura_Identity    , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_URA_Identity },
  { &hf_rrc_supportForChangeOfUE_Capability, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_BOOLEAN },
  { &hf_rrc_dummy_07        , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DefaultConfigForCellFACH },
  { &hf_rrc_rab_InformationSetup, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RAB_InformationSetup_r8 },
  { &hf_rrc_rb_InformationReleaseList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RB_InformationReleaseList },
  { &hf_rrc_rb_InformationReconfigList_05, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RB_InformationReconfigList_r8 },
  { &hf_rrc_rb_InformationAffectedList_04, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RB_InformationAffectedList_r8 },
  { &hf_rrc_dl_CounterSynchronisationInfo_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_CounterSynchronisationInfo_r5 },
  { &hf_rrc_pdcp_ROHC_TargetMode, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_PDCP_ROHC_TargetMode },
  { &hf_rrc_ul_CommonTransChInfo_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_CommonTransChInfo_r4 },
  { &hf_rrc_ul_deletedTransChInfoList_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_DeletedTransChInfoList_r6 },
  { &hf_rrc_ul_AddReconfTransChInfoList_03, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_AddReconfTransChInfoList_r8 },
  { &hf_rrc_dl_CommonTransChInfo_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_CommonTransChInfo_r4 },
  { &hf_rrc_dl_DeletedTransChInfoList_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_DeletedTransChInfoList_r7 },
  { &hf_rrc_dl_AddReconfTransChInfoList_04, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_AddReconfTransChInfoList_r9 },
  { &hf_rrc_frequencyInfo   , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_FrequencyInfo },
  { &hf_rrc_multi_frequencyInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_Multi_frequencyInfo_LCR_r7 },
  { &hf_rrc_dtx_drx_TimingInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DTX_DRX_TimingInfo_r7 },
  { &hf_rrc_dtx_drx_Info    , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DTX_DRX_Info_r7 },
  { &hf_rrc_hs_scch_LessInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_HS_SCCH_LessInfo_r7 },
  { &hf_rrc_mimoParameters_04, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MIMO_Parameters_r9 },
  { &hf_rrc_maxAllowedUL_TX_Power, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MaxAllowedUL_TX_Power },
  { &hf_rrc_ul_DPCH_Info_01 , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_DPCH_Info_r7 },
  { &hf_rrc_ul_EDCH_Information_04, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_EDCH_Information_r9 },
  { &hf_rrc_ul_SecondaryCellInfoFDD, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_SecondaryCellInfoFDD },
  { &hf_rrc_dl_HSPDSCH_Information_06, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_HSPDSCH_Information_r9 },
  { &hf_rrc_dl_CommonInformation_05, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_CommonInformation_r8 },
  { &hf_rrc_dl_InformationPerRL_List_05, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_InformationPerRL_List_r8 },
  { &hf_rrc_dl_SecondaryCellInfoFDD_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_SecondaryCellInfoFDD_r9 },
  { &hf_rrc_controlChannelDRXInfo_TDD128, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_ControlChannelDRXInfo_TDD128_r8 },
  { &hf_rrc_sps_Information_TDD128, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_SPS_Information_TDD128_r8 },
  { &hf_rrc_mbms_PL_ServiceRestrictInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MBMS_PL_ServiceRestrictInfo_r6 },
  { &hf_rrc_cellDCHMeasOccasionInfo_TDD128, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CellDCHMeasOccasionInfo_TDD128_r9 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_CellUpdateConfirm_r9_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_CellUpdateConfirm_r9_IEs, CellUpdateConfirm_r9_IEs_sequence);

  return offset;
}


static const per_sequence_t CellUpdateConfirm_v950ext_IEs_sequence[] = {
  { &hf_rrc_secondaryCellMimoParameters, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_SecondaryCellMIMOparametersFDD_v950ext },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_CellUpdateConfirm_v950ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_CellUpdateConfirm_v950ext_IEs, CellUpdateConfirm_v950ext_IEs_sequence);

  return offset;
}


static const value_string rrc_MAC_ehs_WindowSize_TDD128_v9c0ext_vals[] = {
  {   0, "mws96" },
  {   1, "mws160" },
  {   2, "mws192" },
  {   3, "mws256" },
  { 0, NULL }
};


static int
dissect_rrc_MAC_ehs_WindowSize_TDD128_v9c0ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     4, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t MAC_ehs_AddReconfReordQ_TDD128_v9c0ext_sequence[] = {
  { &hf_rrc_mac_ehsWindowSize_extension, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MAC_ehs_WindowSize_TDD128_v9c0ext },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_MAC_ehs_AddReconfReordQ_TDD128_v9c0ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_MAC_ehs_AddReconfReordQ_TDD128_v9c0ext, MAC_ehs_AddReconfReordQ_TDD128_v9c0ext_sequence);

  return offset;
}


static const per_sequence_t MAC_ehs_AddReconfReordQ_List_TDD128_v9c0ext_sequence_of[1] = {
  { &hf_rrc_MAC_ehs_AddReconfReordQ_List_TDD128_v9c0ext_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MAC_ehs_AddReconfReordQ_TDD128_v9c0ext },
};

static int
dissect_rrc_MAC_ehs_AddReconfReordQ_List_TDD128_v9c0ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_MAC_ehs_AddReconfReordQ_List_TDD128_v9c0ext, MAC_ehs_AddReconfReordQ_List_TDD128_v9c0ext_sequence_of,
                                                  1, maxQueueIDs, FALSE);

  return offset;
}


static const per_sequence_t AddOrReconfMAC_ehs_ReordQ_TDD128_v9c0ext_sequence[] = {
  { &hf_rrc_mac_ehs_AddReconfQueue_List_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MAC_ehs_AddReconfReordQ_List_TDD128_v9c0ext },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_AddOrReconfMAC_ehs_ReordQ_TDD128_v9c0ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_AddOrReconfMAC_ehs_ReordQ_TDD128_v9c0ext, AddOrReconfMAC_ehs_ReordQ_TDD128_v9c0ext_sequence);

  return offset;
}


static const per_sequence_t HSDSCH_Info_TDD128_v9c0ext_sequence[] = {
  { &hf_rrc_mac_ehs_04      , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_AddOrReconfMAC_ehs_ReordQ_TDD128_v9c0ext },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_HSDSCH_Info_TDD128_v9c0ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_HSDSCH_Info_TDD128_v9c0ext, HSDSCH_Info_TDD128_v9c0ext_sequence);

  return offset;
}


static const per_sequence_t DL_AddReconfTransChInformation_TDD128_v9c0ext_sequence[] = {
  { &hf_rrc_hsdsch_05       , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_HSDSCH_Info_TDD128_v9c0ext },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_DL_AddReconfTransChInformation_TDD128_v9c0ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_DL_AddReconfTransChInformation_TDD128_v9c0ext, DL_AddReconfTransChInformation_TDD128_v9c0ext_sequence);

  return offset;
}


static const per_sequence_t DL_AddReconfTransChInfoList_TDD128_v9c0ext_sequence_of[1] = {
  { &hf_rrc_DL_AddReconfTransChInfoList_TDD128_v9c0ext_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DL_AddReconfTransChInformation_TDD128_v9c0ext },
};

static int
dissect_rrc_DL_AddReconfTransChInfoList_TDD128_v9c0ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_DL_AddReconfTransChInfoList_TDD128_v9c0ext, DL_AddReconfTransChInfoList_TDD128_v9c0ext_sequence_of,
                                                  1, maxTrCHpreconf, FALSE);

  return offset;
}


static const per_sequence_t CellUpdateConfirm_v9c0ext_IEs_sequence[] = {
  { &hf_rrc_dl_AddReconfTransChInfoList_05, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_AddReconfTransChInfoList_TDD128_v9c0ext },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_CellUpdateConfirm_v9c0ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_CellUpdateConfirm_v9c0ext_IEs, CellUpdateConfirm_v9c0ext_IEs_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions_20_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_nonCriticalExtensions_20(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_nonCriticalExtensions_20, T_nonCriticalExtensions_20_sequence);

  return offset;
}


static const per_sequence_t T_v9c0NonCriticalExtensions_sequence[] = {
  { &hf_rrc_cellUpdateConfirm_v9c0ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CellUpdateConfirm_v9c0ext_IEs },
  { &hf_rrc_nonCriticalExtensions_20, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_nonCriticalExtensions_20 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v9c0NonCriticalExtensions(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v9c0NonCriticalExtensions, T_v9c0NonCriticalExtensions_sequence);

  return offset;
}


static const per_sequence_t T_v950NonCriticalExtensions_01_sequence[] = {
  { &hf_rrc_cellUpdateConfirm_v950ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CellUpdateConfirm_v950ext_IEs },
  { &hf_rrc_v9c0NonCriticalExtensions, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v9c0NonCriticalExtensions },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v950NonCriticalExtensions_01(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v950NonCriticalExtensions_01, T_v950NonCriticalExtensions_01_sequence);

  return offset;
}


static const per_sequence_t T_r9_01_sequence[] = {
  { &hf_rrc_cellUpdateConfirm_r9, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CellUpdateConfirm_r9_IEs },
  { &hf_rrc_cellUpdateConfirm_r9_add_ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_BIT_STRING },
  { &hf_rrc_v950NonCriticalExtensions_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v950NonCriticalExtensions_01 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_r9_01(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_r9_01, T_r9_01_sequence);

  return offset;
}



static int
dissect_rrc_BIT_STRING_SIZE_128(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_bit_string(tvb, offset, actx, tree, hf_index,
                                     128, 128, FALSE, NULL);

  return offset;
}


static const per_sequence_t SR_VCC_Info_sequence[] = {
  { &hf_rrc_nonce           , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_BIT_STRING_SIZE_128 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_SR_VCC_Info(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_SR_VCC_Info, SR_VCC_Info_sequence);

  return offset;
}


static const value_string rrc_T_tsn_Length_01_vals[] = {
  {   0, "tsn-14bits" },
  { 0, NULL }
};


static int
dissect_rrc_T_tsn_Length_01(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t E_PUCH_Info_MulticarrierEDCH_TDD128_sequence[] = {
  { &hf_rrc_e_PUCH_TS_ConfigurationList_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SEQUENCE_SIZE_1_maxTS_LCR_1_OF_E_PUCH_TS_Slots_LCR },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_E_PUCH_Info_MulticarrierEDCH_TDD128(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_E_PUCH_Info_MulticarrierEDCH_TDD128, E_PUCH_Info_MulticarrierEDCH_TDD128_sequence);

  return offset;
}


static const per_sequence_t E_AGCH_Information_TDD128_sequence[] = {
  { &hf_rrc_rdi_Indicator   , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { &hf_rrc_tpc_StepSize    , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TPC_StepSizeTDD },
  { &hf_rrc_e_AGCH_Set_Config_02, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_E_AGCH_Set_Config_LCR },
  { &hf_rrc_e_AGCH_BLER_Target, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_Bler_Target },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_E_AGCH_Information_TDD128(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_E_AGCH_Information_TDD128, E_AGCH_Information_TDD128_sequence);

  return offset;
}


static const per_sequence_t UL_MulticarrierEDCH_InfolistItem_TDD128_sequence[] = {
  { &hf_rrc_uarfcn          , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UARFCN },
  { &hf_rrc_prxdes_base     , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_M112_M50 },
  { &hf_rrc_e_PUCH_info     , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_E_PUCH_Info_MulticarrierEDCH_TDD128 },
  { &hf_rrc_e_AGCH_info     , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_E_AGCH_Information_TDD128 },
  { &hf_rrc_e_HICH_info     , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_E_HICH_Information_TDD128 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UL_MulticarrierEDCH_InfolistItem_TDD128(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UL_MulticarrierEDCH_InfolistItem_TDD128, UL_MulticarrierEDCH_InfolistItem_TDD128_sequence);

  return offset;
}


static const per_sequence_t UL_MulticarrierEDCH_Infolist_TDD128_sequence_of[1] = {
  { &hf_rrc_UL_MulticarrierEDCH_Infolist_TDD128_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UL_MulticarrierEDCH_InfolistItem_TDD128 },
};

static int
dissect_rrc_UL_MulticarrierEDCH_Infolist_TDD128(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_UL_MulticarrierEDCH_Infolist_TDD128, UL_MulticarrierEDCH_Infolist_TDD128_sequence_of,
                                                  1, maxTDD128Carrier_1, FALSE);

  return offset;
}


static const per_sequence_t UL_MulticarrierEDCH_Deletelist_TDD128_sequence_of[1] = {
  { &hf_rrc_UL_MulticarrierEDCH_Deletelist_TDD128_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UARFCN },
};

static int
dissect_rrc_UL_MulticarrierEDCH_Deletelist_TDD128(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_UL_MulticarrierEDCH_Deletelist_TDD128, UL_MulticarrierEDCH_Deletelist_TDD128_sequence_of,
                                                  1, maxTDD128Carrier_1, FALSE);

  return offset;
}


static const per_sequence_t UL_MulticarrierEDCH_NewConfigurationInfo_TDD128_sequence[] = {
  { &hf_rrc_tsn_Length_01   , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_tsn_Length_01 },
  { &hf_rrc_ul_MulticarrierEDCH_Infolist, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_MulticarrierEDCH_Infolist_TDD128 },
  { &hf_rrc_ul_MulticarrierEDCH_Deletelist, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_MulticarrierEDCH_Deletelist_TDD128 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UL_MulticarrierEDCH_NewConfigurationInfo_TDD128(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UL_MulticarrierEDCH_NewConfigurationInfo_TDD128, UL_MulticarrierEDCH_NewConfigurationInfo_TDD128_sequence);

  return offset;
}


static const value_string rrc_T_configuration_info_vals[] = {
  {   0, "continue" },
  {   1, "newConfiguration" },
  { 0, NULL }
};

static const per_choice_t T_configuration_info_choice[] = {
  {   0, &hf_rrc_continue        , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   1, &hf_rrc_newConfiguration_13, ASN1_NO_EXTENSIONS     , dissect_rrc_UL_MulticarrierEDCH_NewConfigurationInfo_TDD128 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_configuration_info(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_configuration_info, T_configuration_info_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t UL_MulticarrierEDCHInfo_TDD128_sequence[] = {
  { &hf_rrc_configuration_info, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_configuration_info },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UL_MulticarrierEDCHInfo_TDD128(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UL_MulticarrierEDCHInfo_TDD128, UL_MulticarrierEDCHInfo_TDD128_sequence);

  return offset;
}


static const value_string rrc_T_dl_dpchInfoCommon_03_vals[] = {
  {   0, "dl-DPCH-InfoCommon" },
  {   1, "dl-FDPCH-InfoCommon" },
  { 0, NULL }
};

static const per_choice_t T_dl_dpchInfoCommon_03_choice[] = {
  {   0, &hf_rrc_dl_DPCH_InfoCommon_02, ASN1_NO_EXTENSIONS     , dissect_rrc_DL_DPCH_InfoCommon_r6 },
  {   1, &hf_rrc_dl_FDPCH_InfoCommon, ASN1_NO_EXTENSIONS     , dissect_rrc_DL_FDPCH_InfoCommon_r6 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_dl_dpchInfoCommon_03(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_dl_dpchInfoCommon_03, T_dl_dpchInfoCommon_03_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t T_activate_02_sequence[] = {
  { &hf_rrc_tgcfn           , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TGCFN },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_activate_02(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_activate_02, T_activate_02_sequence);

  return offset;
}


static const value_string rrc_T_tgps_Status_02_vals[] = {
  {   0, "activate" },
  {   1, "deactivate" },
  { 0, NULL }
};

static const per_choice_t T_tgps_Status_02_choice[] = {
  {   0, &hf_rrc_activate_02     , ASN1_NO_EXTENSIONS     , dissect_rrc_T_activate_02 },
  {   1, &hf_rrc_deactivate      , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_tgps_Status_02(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_tgps_Status_02, T_tgps_Status_02_choice,
                                 NULL);

  return offset;
}


static const value_string rrc_T_freqSpecificCompressedMode_02_vals[] = {
  {   0, "true" },
  { 0, NULL }
};


static int
dissect_rrc_T_freqSpecificCompressedMode_02(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t TGPS_ConfigurationParams_r10_sequence[] = {
  { &hf_rrc_tgmp_01         , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TGMP_r8 },
  { &hf_rrc_freqSpecificCompressedMode_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_freqSpecificCompressedMode_02 },
  { &hf_rrc_tgprc           , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TGPRC },
  { &hf_rrc_tgsn            , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TGSN },
  { &hf_rrc_tgl1            , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TGL },
  { &hf_rrc_tgl2            , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_TGL },
  { &hf_rrc_tgd             , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TGD },
  { &hf_rrc_tgpl1           , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TGPL },
  { &hf_rrc_rpp             , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RPP },
  { &hf_rrc_itp             , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_ITP },
  { &hf_rrc_ul_DL_Mode      , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UL_DL_Mode },
  { &hf_rrc_dl_FrameType    , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DL_FrameType },
  { &hf_rrc_deltaSIR1       , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DeltaSIR },
  { &hf_rrc_deltaSIRAfter1  , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DeltaSIR },
  { &hf_rrc_deltaSIR2       , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DeltaSIR },
  { &hf_rrc_deltaSIRAfter2  , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DeltaSIR },
  { &hf_rrc_nidentifyAbort  , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_NidentifyAbort },
  { &hf_rrc_treconfirmAbort , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_TreconfirmAbort },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_TGPS_ConfigurationParams_r10(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_TGPS_ConfigurationParams_r10, TGPS_ConfigurationParams_r10_sequence);

  return offset;
}


static const per_sequence_t TGP_Sequence_r10_sequence[] = {
  { &hf_rrc_tgpsi           , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TGPSI },
  { &hf_rrc_tgps_Status_02  , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_tgps_Status_02 },
  { &hf_rrc_tgps_ConfigurationParams_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_TGPS_ConfigurationParams_r10 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_TGP_Sequence_r10(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_TGP_Sequence_r10, TGP_Sequence_r10_sequence);

  return offset;
}


static const per_sequence_t TGP_SequenceList_r10_sequence_of[1] = {
  { &hf_rrc_TGP_SequenceList_r10_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TGP_Sequence_r10 },
};

static int
dissect_rrc_TGP_SequenceList_r10(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_TGP_SequenceList_r10, TGP_SequenceList_r10_sequence_of,
                                                  1, maxTGPS, FALSE);

  return offset;
}


static const per_sequence_t DPCH_CompressedModeInfo_r10_sequence[] = {
  { &hf_rrc_tgp_SequenceList_02, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TGP_SequenceList_r10 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_DPCH_CompressedModeInfo_r10(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_DPCH_CompressedModeInfo_r10, DPCH_CompressedModeInfo_r10_sequence);

  return offset;
}


static const per_sequence_t T_fdd_64_sequence[] = {
  { &hf_rrc_defaultDPCH_OffsetValue, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DefaultDPCH_OffsetValueFDD },
  { &hf_rrc_dpch_CompressedModeInfo_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DPCH_CompressedModeInfo_r10 },
  { &hf_rrc_tx_DiversityMode, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_TX_DiversityMode },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_fdd_64(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_fdd_64, T_fdd_64_sequence);

  return offset;
}


static const per_sequence_t T_tdd128_19_sequence[] = {
  { &hf_rrc_tstd_Indicator  , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_tdd128_19(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_tdd128_19, T_tdd128_19_sequence);

  return offset;
}


static const value_string rrc_T_tddOption_10_vals[] = {
  {   0, "tdd384" },
  {   1, "tdd768" },
  {   2, "tdd128" },
  { 0, NULL }
};

static const per_choice_t T_tddOption_10_choice[] = {
  {   0, &hf_rrc_tdd384_11       , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   1, &hf_rrc_tdd768_06       , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   2, &hf_rrc_tdd128_19       , ASN1_NO_EXTENSIONS     , dissect_rrc_T_tdd128_19 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_tddOption_10(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_tddOption_10, T_tddOption_10_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t T_tdd_26_sequence[] = {
  { &hf_rrc_tddOption_10    , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_tddOption_10 },
  { &hf_rrc_defaultDPCH_OffsetValue_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DefaultDPCH_OffsetValueTDD },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_tdd_26(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_tdd_26, T_tdd_26_sequence);

  return offset;
}


static const value_string rrc_T_modeSpecificInfo_34_vals[] = {
  {   0, "fdd" },
  {   1, "tdd" },
  { 0, NULL }
};

static const per_choice_t T_modeSpecificInfo_34_choice[] = {
  {   0, &hf_rrc_fdd_65          , ASN1_NO_EXTENSIONS     , dissect_rrc_T_fdd_64 },
  {   1, &hf_rrc_tdd_27          , ASN1_NO_EXTENSIONS     , dissect_rrc_T_tdd_26 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_modeSpecificInfo_34(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_modeSpecificInfo_34, T_modeSpecificInfo_34_choice,
                                 NULL);

  return offset;
}


static const value_string rrc_T_mac_hsResetIndicator_09_vals[] = {
  {   0, "true" },
  { 0, NULL }
};


static int
dissect_rrc_T_mac_hsResetIndicator_09(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}


static const value_string rrc_T_postVerificationPeriod_10_vals[] = {
  {   0, "true" },
  { 0, NULL }
};


static int
dissect_rrc_T_postVerificationPeriod_10(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t DL_CommonInformation_r10_sequence[] = {
  { &hf_rrc_dl_dpchInfoCommon_03, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_dl_dpchInfoCommon_03 },
  { &hf_rrc_modeSpecificInfo_34, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_modeSpecificInfo_34 },
  { &hf_rrc_mac_hsResetIndicator_09, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_mac_hsResetIndicator_09 },
  { &hf_rrc_postVerificationPeriod_10, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_postVerificationPeriod_10 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_DL_CommonInformation_r10(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_DL_CommonInformation_r10, DL_CommonInformation_r10_sequence);

  return offset;
}


static const value_string rrc_MU_MIMO_Operation_vals[] = {
  {   0, "uL" },
  {   1, "dL" },
  {   2, "uLandDL" },
  {   3, "spare" },
  { 0, NULL }
};


static int
dissect_rrc_MU_MIMO_Operation(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     4, NULL, FALSE, 0, NULL);

  return offset;
}


static const value_string rrc_T_repetitionPeriod_vals[] = {
  {   0, "v1" },
  {   1, "v2" },
  {   2, "v4" },
  {   3, "v8" },
  {   4, "v16" },
  {   5, "v32" },
  {   6, "v64" },
  {   7, "spare" },
  { 0, NULL }
};


static int
dissect_rrc_T_repetitionPeriod(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     8, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t StandaloneMidambleInfo_TDD128_sequence[] = {
  { &hf_rrc_midambleConfiguration, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_1_8 },
  { &hf_rrc_midambleShift_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_0_15 },
  { &hf_rrc_timeSlotInformation, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_5 },
  { &hf_rrc_activationTime  , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_ActivationTime },
  { &hf_rrc_subframeNum     , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_0_1 },
  { &hf_rrc_repetitionPeriod, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_repetitionPeriod },
  { &hf_rrc_referenceBeta   , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_INTEGER_M15_16 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_StandaloneMidambleInfo_TDD128(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_StandaloneMidambleInfo_TDD128, StandaloneMidambleInfo_TDD128_sequence);

  return offset;
}


static const per_sequence_t T_newConfiguration_07_sequence[] = {
  { &hf_rrc_mu_MIMO_Operation, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MU_MIMO_Operation },
  { &hf_rrc_standaloneMidambleInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_StandaloneMidambleInfo_TDD128 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_newConfiguration_07(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_newConfiguration_07, T_newConfiguration_07_sequence);

  return offset;
}


static const value_string rrc_MU_MIMO_Info_TDD128_vals[] = {
  {   0, "continue" },
  {   1, "newConfiguration" },
  { 0, NULL }
};

static const per_choice_t MU_MIMO_Info_TDD128_choice[] = {
  {   0, &hf_rrc_continue        , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   1, &hf_rrc_newConfiguration_07, ASN1_NO_EXTENSIONS     , dissect_rrc_T_newConfiguration_07 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_MU_MIMO_Info_TDD128(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_MU_MIMO_Info_TDD128, MU_MIMO_Info_TDD128_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t CellUpdateConfirm_r10_IEs_sequence[] = {
  { &hf_rrc_integrityProtectionModeInfo_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_IntegrityProtectionModeInfo_r7 },
  { &hf_rrc_cipheringModeInfo_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CipheringModeInfo_r7 },
  { &hf_rrc_dummy2_03       , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_SR_VCC_Info },
  { &hf_rrc_activationTime  , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_ActivationTime },
  { &hf_rrc_new_U_RNTI      , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_U_RNTI },
  { &hf_rrc_new_C_RNTI      , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_C_RNTI },
  { &hf_rrc_new_DSCH_RNTI   , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DSCH_RNTI },
  { &hf_rrc_new_H_RNTI      , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_H_RNTI },
  { &hf_rrc_newPrimary_E_RNTI, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_E_RNTI },
  { &hf_rrc_newSecondary_E_RNTI, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_E_RNTI },
  { &hf_rrc_rrc_StateIndicator, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RRC_StateIndicator },
  { &hf_rrc_utran_DRX_CycleLengthCoeff_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UTRAN_DRX_CycleLengthCoefficient_r7 },
  { &hf_rrc_waitTime        , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_WaitTime },
  { &hf_rrc_rlc_Re_establishIndicatorRb2_3or4, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { &hf_rrc_rlc_Re_establishIndicatorRb5orAbove, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { &hf_rrc_cn_InformationInfo_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CN_InformationInfo_r6 },
  { &hf_rrc_ura_Identity    , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_URA_Identity },
  { &hf_rrc_supportForChangeOfUE_Capability, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_BOOLEAN },
  { &hf_rrc_dummy_07        , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DefaultConfigForCellFACH },
  { &hf_rrc_rab_InformationSetup, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RAB_InformationSetup_r8 },
  { &hf_rrc_rb_InformationReleaseList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RB_InformationReleaseList },
  { &hf_rrc_rb_InformationReconfigList_05, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RB_InformationReconfigList_r8 },
  { &hf_rrc_rb_InformationAffectedList_04, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RB_InformationAffectedList_r8 },
  { &hf_rrc_dl_CounterSynchronisationInfo_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_CounterSynchronisationInfo_r5 },
  { &hf_rrc_pdcp_ROHC_TargetMode, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_PDCP_ROHC_TargetMode },
  { &hf_rrc_ul_CommonTransChInfo_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_CommonTransChInfo_r4 },
  { &hf_rrc_ul_deletedTransChInfoList_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_DeletedTransChInfoList_r6 },
  { &hf_rrc_ul_AddReconfTransChInfoList_03, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_AddReconfTransChInfoList_r8 },
  { &hf_rrc_dl_CommonTransChInfo_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_CommonTransChInfo_r4 },
  { &hf_rrc_dl_DeletedTransChInfoList_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_DeletedTransChInfoList_r7 },
  { &hf_rrc_dl_AddReconfTransChInfoList_04, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_AddReconfTransChInfoList_r9 },
  { &hf_rrc_frequencyInfo   , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_FrequencyInfo },
  { &hf_rrc_multi_frequencyInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_Multi_frequencyInfo_LCR_r7 },
  { &hf_rrc_dtx_drx_TimingInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DTX_DRX_TimingInfo_r7 },
  { &hf_rrc_dtx_drx_Info    , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DTX_DRX_Info_r7 },
  { &hf_rrc_hs_scch_LessInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_HS_SCCH_LessInfo_r7 },
  { &hf_rrc_mimoParameters_04, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MIMO_Parameters_r9 },
  { &hf_rrc_maxAllowedUL_TX_Power, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MaxAllowedUL_TX_Power },
  { &hf_rrc_ul_DPCH_Info_01 , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_DPCH_Info_r7 },
  { &hf_rrc_ul_EDCH_Information_04, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_EDCH_Information_r9 },
  { &hf_rrc_ul_SecondaryCellInfoFDD, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_SecondaryCellInfoFDD },
  { &hf_rrc_ul_MulticarrierEDCHInfo_TDD128, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_MulticarrierEDCHInfo_TDD128 },
  { &hf_rrc_dl_HSPDSCH_Information_06, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_HSPDSCH_Information_r9 },
  { &hf_rrc_dl_CommonInformation_06, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_CommonInformation_r10 },
  { &hf_rrc_dl_InformationPerRL_List_05, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_InformationPerRL_List_r8 },
  { &hf_rrc_dl_SecondaryCellInfoFDD_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_SecondaryCellInfoFDD_r10 },
  { &hf_rrc_additionalDLSecCellInfoListFDD, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_AdditionalDLSecCellInfoListFDD },
  { &hf_rrc_controlChannelDRXInfo_TDD128, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_ControlChannelDRXInfo_TDD128_r8 },
  { &hf_rrc_sps_Information_TDD128, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_SPS_Information_TDD128_r8 },
  { &hf_rrc_mu_MIMO_Info_TDD128, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MU_MIMO_Info_TDD128 },
  { &hf_rrc_mbms_PL_ServiceRestrictInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MBMS_PL_ServiceRestrictInfo_r6 },
  { &hf_rrc_cellDCHMeasOccasionInfo_TDD128, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CellDCHMeasOccasionInfo_TDD128_r9 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_CellUpdateConfirm_r10_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_CellUpdateConfirm_r10_IEs, CellUpdateConfirm_r10_IEs_sequence);

  return offset;
}


static const per_sequence_t CellUpdateConfirm_vbxyext_IEs_sequence[] = {
  { &hf_rrc_ulOLTDInfoFDD   , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_OLTD_InfoFDD },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_CellUpdateConfirm_vbxyext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_CellUpdateConfirm_vbxyext_IEs, CellUpdateConfirm_vbxyext_IEs_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions_21_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_nonCriticalExtensions_21(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_nonCriticalExtensions_21, T_nonCriticalExtensions_21_sequence);

  return offset;
}


static const per_sequence_t T_vbxyNonCriticalExtensions_02_sequence[] = {
  { &hf_rrc_cellUpdateConfirm_vbxyext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CellUpdateConfirm_vbxyext_IEs },
  { &hf_rrc_nonCriticalExtensions_21, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_nonCriticalExtensions_21 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_vbxyNonCriticalExtensions_02(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_vbxyNonCriticalExtensions_02, T_vbxyNonCriticalExtensions_02_sequence);

  return offset;
}


static const per_sequence_t T_v9c0NonCriticalExtensions_01_sequence[] = {
  { &hf_rrc_cellUpdateConfirm_v9c0ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CellUpdateConfirm_v9c0ext_IEs },
  { &hf_rrc_vbxyNonCriticalExtensions_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_vbxyNonCriticalExtensions_02 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v9c0NonCriticalExtensions_01(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v9c0NonCriticalExtensions_01, T_v9c0NonCriticalExtensions_01_sequence);

  return offset;
}


static const per_sequence_t T_r10_01_sequence[] = {
  { &hf_rrc_cellUpdateConfirm_r10, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CellUpdateConfirm_r10_IEs },
  { &hf_rrc_cellUpdateConfirm_r10_add_ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_BIT_STRING },
  { &hf_rrc_v9c0NonCriticalExtensions_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v9c0NonCriticalExtensions_01 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_r10_01(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_r10_01, T_r10_01_sequence);

  return offset;
}


static const value_string rrc_T_flexibleSize_01_vals[] = {
  {   0, "size7" },
  {   1, "size15" },
  { 0, NULL }
};


static int
dissect_rrc_T_flexibleSize_01(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     2, NULL, FALSE, 0, NULL);

  return offset;
}


static const value_string rrc_T_dl_RLC_PDU_size_01_vals[] = {
  {   0, "fixedSize" },
  {   1, "flexibleSize" },
  { 0, NULL }
};

static const per_choice_t T_dl_RLC_PDU_size_01_choice[] = {
  {   0, &hf_rrc_fixedSize       , ASN1_NO_EXTENSIONS     , dissect_rrc_OctetModeRLC_SizeInfoType1 },
  {   1, &hf_rrc_flexibleSize_01 , ASN1_NO_EXTENSIONS     , dissect_rrc_T_flexibleSize_01 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_dl_RLC_PDU_size_01(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_dl_RLC_PDU_size_01, T_dl_RLC_PDU_size_01_choice,
                                 NULL);

  return offset;
}


static const value_string rrc_TimerReordering_vals[] = {
  {   0, "tr50" },
  {   1, "tr100" },
  {   2, "tr150" },
  {   3, "tr200" },
  {   4, "tr400" },
  {   5, "tr600" },
  {   6, "tr800" },
  {   7, "tr1000" },
  { 0, NULL }
};


static int
dissect_rrc_TimerReordering(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     8, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t DL_RLC_StatusInfo_r11_sequence[] = {
  { &hf_rrc_timerStatusProhibit, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_TimerStatusProhibit },
  { &hf_rrc_missingPDU_Indicator, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { &hf_rrc_timerStatusPeriodic, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_TimerStatusPeriodic },
  { &hf_rrc_timerReordering , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_TimerReordering },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_DL_RLC_StatusInfo_r11(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_DL_RLC_StatusInfo_r11, DL_RLC_StatusInfo_r11_sequence);

  return offset;
}


static const per_sequence_t DL_AM_RLC_Mode_r11_sequence[] = {
  { &hf_rrc_dl_RLC_PDU_size_02, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_dl_RLC_PDU_size_01 },
  { &hf_rrc_inSequenceDelivery, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { &hf_rrc_receivingWindowSize, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_ReceivingWindowSize },
  { &hf_rrc_dl_RLC_StatusInfo_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DL_RLC_StatusInfo_r11 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_DL_AM_RLC_Mode_r11(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_DL_AM_RLC_Mode_r11, DL_AM_RLC_Mode_r11_sequence);

  return offset;
}


static const value_string rrc_DL_RLC_Mode_r11_vals[] = {
  {   0, "dl-AM-RLC-Mode" },
  {   1, "dl-UM-RLC-Mode" },
  {   2, "dl-TM-RLC-Mode" },
  { 0, NULL }
};

static const per_choice_t DL_RLC_Mode_r11_choice[] = {
  {   0, &hf_rrc_dl_AM_RLC_Mode_03, ASN1_NO_EXTENSIONS     , dissect_rrc_DL_AM_RLC_Mode_r11 },
  {   1, &hf_rrc_dl_UM_RLC_Mode_02, ASN1_NO_EXTENSIONS     , dissect_rrc_DL_UM_RLC_Mode_r6 },
  {   2, &hf_rrc_dl_TM_RLC_Mode  , ASN1_NO_EXTENSIONS     , dissect_rrc_DL_TM_RLC_Mode },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_DL_RLC_Mode_r11(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_DL_RLC_Mode_r11, DL_RLC_Mode_r11_choice,
                                 NULL);

  return offset;
}


static const value_string rrc_T_altE_bitInterpretation_02_vals[] = {
  {   0, "true" },
  { 0, NULL }
};


static int
dissect_rrc_T_altE_bitInterpretation_02(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}


static const value_string rrc_T_useSpecialValueOfHEField_01_vals[] = {
  {   0, "true" },
  { 0, NULL }
};


static int
dissect_rrc_T_useSpecialValueOfHEField_01(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t RLC_Info_r11_sequence[] = {
  { &hf_rrc_ul_RLC_Mode     , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_RLC_Mode },
  { &hf_rrc_dl_RLC_Mode_04  , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_RLC_Mode_r11 },
  { &hf_rrc_rlc_OneSidedReEst, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { &hf_rrc_altE_bitInterpretation_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_altE_bitInterpretation_02 },
  { &hf_rrc_useSpecialValueOfHEField_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_useSpecialValueOfHEField_01 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_RLC_Info_r11(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_RLC_Info_r11, RLC_Info_r11_sequence);

  return offset;
}


static const per_sequence_t RB_InformationReconfig_r11_sequence[] = {
  { &hf_rrc_rb_Identity     , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RB_Identity },
  { &hf_rrc_pdcp_Info_03    , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_PDCP_InfoReconfig_r4 },
  { &hf_rrc_pdcp_SN_Info    , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_PDCP_SN_Info },
  { &hf_rrc_rlc_Info_04     , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RLC_Info_r11 },
  { &hf_rrc_rb_MappingInfo_04, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RB_MappingInfo_r8 },
  { &hf_rrc_rb_StopContinue , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RB_StopContinue },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_RB_InformationReconfig_r11(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_RB_InformationReconfig_r11, RB_InformationReconfig_r11_sequence);

  return offset;
}


static const per_sequence_t RB_InformationReconfigList_r11_sequence_of[1] = {
  { &hf_rrc_RB_InformationReconfigList_r11_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RB_InformationReconfig_r11 },
};

static int
dissect_rrc_RB_InformationReconfigList_r11(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_RB_InformationReconfigList_r11, RB_InformationReconfigList_r11_sequence_of,
                                                  1, maxRB, FALSE);

  return offset;
}


static const value_string rrc_MAC_hs_WindowSize_r11_vals[] = {
  {   0, "mws4" },
  {   1, "mws6" },
  {   2, "mws8" },
  {   3, "mws12" },
  {   4, "mws16" },
  {   5, "mws24" },
  {   6, "mws32" },
  {   7, "mw64" },
  {   8, "mw128" },
  {   9, "mw256" },
  { 0, NULL }
};


static int
dissect_rrc_MAC_hs_WindowSize_r11(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     10, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t MAC_ehs_AddReconfReordQ_r11_sequence[] = {
  { &hf_rrc_mac_ehs_QueueId , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MAC_ehs_QueueId },
  { &hf_rrc_reorderingReleaseTimer, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T1_ReleaseTimer },
  { &hf_rrc_reorderingResetTimer, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_Treset_ResetTimer },
  { &hf_rrc_mac_ehsWindowSize_02, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MAC_hs_WindowSize_r11 },
  { &hf_rrc_mac_ehsWindowSize_extension, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MAC_ehs_WindowSize_TDD128_v9c0ext },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_MAC_ehs_AddReconfReordQ_r11(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_MAC_ehs_AddReconfReordQ_r11, MAC_ehs_AddReconfReordQ_r11_sequence);

  return offset;
}


static const per_sequence_t MAC_ehs_AddReconfReordQ_List_r11_sequence_of[1] = {
  { &hf_rrc_MAC_ehs_AddReconfReordQ_List_r11_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MAC_ehs_AddReconfReordQ_r11 },
};

static int
dissect_rrc_MAC_ehs_AddReconfReordQ_List_r11(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_MAC_ehs_AddReconfReordQ_List_r11, MAC_ehs_AddReconfReordQ_List_r11_sequence_of,
                                                  1, maxQueueIDs, FALSE);

  return offset;
}


static const per_sequence_t AddOrReconfMAC_ehs_ReordQ_r11_sequence[] = {
  { &hf_rrc_mac_ehs_AddReconfQueue_List_03, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MAC_ehs_AddReconfReordQ_List_r11 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_AddOrReconfMAC_ehs_ReordQ_r11(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_AddOrReconfMAC_ehs_ReordQ_r11, AddOrReconfMAC_ehs_ReordQ_r11_sequence);

  return offset;
}


static const value_string rrc_T_dl_MAC_HeaderType_02_vals[] = {
  {   0, "mac-hs" },
  {   1, "mac-ehs" },
  { 0, NULL }
};

static const per_choice_t T_dl_MAC_HeaderType_02_choice[] = {
  {   0, &hf_rrc_mac_hs_02       , ASN1_NO_EXTENSIONS     , dissect_rrc_AddOrReconfMAC_dFlow },
  {   1, &hf_rrc_mac_ehs_05      , ASN1_NO_EXTENSIONS     , dissect_rrc_AddOrReconfMAC_ehs_ReordQ_r11 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_dl_MAC_HeaderType_02(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_dl_MAC_HeaderType_02, T_dl_MAC_HeaderType_02_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t HSDSCH_Info_r11_sequence[] = {
  { &hf_rrc_harqInfo        , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_HARQ_Info_r7 },
  { &hf_rrc_dl_MAC_HeaderType_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_dl_MAC_HeaderType_02 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_HSDSCH_Info_r11(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_HSDSCH_Info_r11, HSDSCH_Info_r11_sequence);

  return offset;
}


static const value_string rrc_T_tfs_SignallingMode_05_vals[] = {
  {   0, "explicit-config" },
  {   1, "sameAsULTrCH" },
  {   2, "hsdsch" },
  { 0, NULL }
};

static const per_choice_t T_tfs_SignallingMode_05_choice[] = {
  {   0, &hf_rrc_explicit_config , ASN1_NO_EXTENSIONS     , dissect_rrc_TransportFormatSet },
  {   1, &hf_rrc_sameAsULTrCH    , ASN1_NO_EXTENSIONS     , dissect_rrc_UL_TransportChannelIdentity },
  {   2, &hf_rrc_hsdsch_06       , ASN1_NO_EXTENSIONS     , dissect_rrc_HSDSCH_Info_r11 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_tfs_SignallingMode_05(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_tfs_SignallingMode_05, T_tfs_SignallingMode_05_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t DL_AddReconfTransChInformation_r11_sequence[] = {
  { &hf_rrc_dl_TransportChannelType_04, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DL_TrCH_TypeId1_r5 },
  { &hf_rrc_tfs_SignallingMode_05, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_tfs_SignallingMode_05 },
  { &hf_rrc_dch_QualityTarget, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_QualityTarget },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_DL_AddReconfTransChInformation_r11(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_DL_AddReconfTransChInformation_r11, DL_AddReconfTransChInformation_r11_sequence);

  return offset;
}


static const per_sequence_t DL_AddReconfTransChInfoList_r11_sequence_of[1] = {
  { &hf_rrc_DL_AddReconfTransChInfoList_r11_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DL_AddReconfTransChInformation_r11 },
};

static int
dissect_rrc_DL_AddReconfTransChInfoList_r11(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_DL_AddReconfTransChInfoList_r11, DL_AddReconfTransChInfoList_r11_sequence_of,
                                                  1, maxTrCHpreconf, FALSE);

  return offset;
}


static const per_sequence_t T_fdd_126_sequence[] = {
  { &hf_rrc_dpcch_PowerOffset, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DPCCH_PowerOffset },
  { &hf_rrc_pc_Preamble     , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PC_Preamble },
  { &hf_rrc_sRB_delay       , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SRB_delay },
  { &hf_rrc_powerControlAlgorithm, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PowerControlAlgorithm },
  { &hf_rrc_deltaACK_01     , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DeltaACK_r11 },
  { &hf_rrc_deltaNACK_01    , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DeltaNACK_r11 },
  { &hf_rrc_ack_NACK_repetition_factor, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_ACK_NACK_repetitionFactor },
  { &hf_rrc_ack_NACK_repetition_factor_assisting, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_ACK_NACK_repetitionFactor },
  { &hf_rrc_harq_Preamble_Mode, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_HARQ_Preamble_Mode },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_fdd_126(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_fdd_126, T_fdd_126_sequence);

  return offset;
}


static const per_sequence_t T_tdd384_40_sequence[] = {
  { &hf_rrc_individualTS_InterferenceList, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_IndividualTS_InterferenceList },
  { &hf_rrc_dpch_ConstantValue_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_ConstantValue },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_tdd384_40(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_tdd384_40, T_tdd384_40_sequence);

  return offset;
}


static const per_sequence_t T_tdd768_22_sequence[] = {
  { &hf_rrc_individualTS_InterferenceList, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_IndividualTS_InterferenceList },
  { &hf_rrc_dpch_ConstantValue_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_ConstantValue },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_tdd768_22(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_tdd768_22, T_tdd768_22_sequence);

  return offset;
}


static const per_sequence_t T_tdd128_59_sequence[] = {
  { &hf_rrc_beaconPLEst     , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_BEACON_PL_Est },
  { &hf_rrc_tpc_StepSize    , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TPC_StepSizeTDD },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_tdd128_59(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_tdd128_59, T_tdd128_59_sequence);

  return offset;
}


static const value_string rrc_T_tddOption_25_vals[] = {
  {   0, "tdd384" },
  {   1, "tdd768" },
  {   2, "tdd128" },
  { 0, NULL }
};

static const per_choice_t T_tddOption_25_choice[] = {
  {   0, &hf_rrc_tdd384_44       , ASN1_NO_EXTENSIONS     , dissect_rrc_T_tdd384_40 },
  {   1, &hf_rrc_tdd768_25       , ASN1_NO_EXTENSIONS     , dissect_rrc_T_tdd768_22 },
  {   2, &hf_rrc_tdd128_61       , ASN1_NO_EXTENSIONS     , dissect_rrc_T_tdd128_59 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_tddOption_25(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_tddOption_25, T_tddOption_25_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t T_individuallySignalled_05_sequence[] = {
  { &hf_rrc_tddOption_25    , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_tddOption_25 },
  { &hf_rrc_primaryCCPCH_TX_Power, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PrimaryCCPCH_TX_Power },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_individuallySignalled_05(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_individuallySignalled_05, T_individuallySignalled_05_sequence);

  return offset;
}


static const value_string rrc_T_ul_OL_PC_Signalling_05_vals[] = {
  {   0, "broadcast-UL-OL-PC-info" },
  {   1, "individuallySignalled" },
  { 0, NULL }
};

static const per_choice_t T_ul_OL_PC_Signalling_05_choice[] = {
  {   0, &hf_rrc_broadcast_UL_OL_PC_info, ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   1, &hf_rrc_individuallySignalled_05, ASN1_NO_EXTENSIONS     , dissect_rrc_T_individuallySignalled_05 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_ul_OL_PC_Signalling_05(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_ul_OL_PC_Signalling_05, T_ul_OL_PC_Signalling_05_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t T_tdd_76_sequence[] = {
  { &hf_rrc_ul_TargetSIR    , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_TargetSIR },
  { &hf_rrc_ul_OL_PC_Signalling_05, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_ul_OL_PC_Signalling_05 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_tdd_76(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_tdd_76, T_tdd_76_sequence);

  return offset;
}


static const value_string rrc_UL_DPCH_PowerControlInfo_r11_vals[] = {
  {   0, "fdd" },
  {   1, "tdd" },
  { 0, NULL }
};

static const per_choice_t UL_DPCH_PowerControlInfo_r11_choice[] = {
  {   0, &hf_rrc_fdd_129         , ASN1_NO_EXTENSIONS     , dissect_rrc_T_fdd_126 },
  {   1, &hf_rrc_tdd_82          , ASN1_NO_EXTENSIONS     , dissect_rrc_T_tdd_76 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_UL_DPCH_PowerControlInfo_r11(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_UL_DPCH_PowerControlInfo_r11, UL_DPCH_PowerControlInfo_r11_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t T_present_04_sequence[] = {
  { &hf_rrc_numberOfDPDCH   , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_NumberOfDPDCH },
  { &hf_rrc_spreadingFactor_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SpreadingFactor },
  { &hf_rrc_tfci_Existence  , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { &hf_rrc_numberOfFBI_Bits, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_NumberOfFBI_Bits },
  { &hf_rrc_numberOfTPC_Bits, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_NumberOfTPC_Bits },
  { &hf_rrc_puncturingLimit , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PuncturingLimit },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_present_04(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_present_04, T_present_04_sequence);

  return offset;
}


static const per_sequence_t T_notPresent_02_sequence[] = {
  { &hf_rrc_tfci_Existence  , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { &hf_rrc_numberOfFBI_Bits, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_NumberOfFBI_Bits },
  { &hf_rrc_numberOfTPC_Bits, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_NumberOfTPC_Bits },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_notPresent_02(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_notPresent_02, T_notPresent_02_sequence);

  return offset;
}


static const value_string rrc_T_dpdchPresence_02_vals[] = {
  {   0, "present" },
  {   1, "notPresent" },
  { 0, NULL }
};

static const per_choice_t T_dpdchPresence_02_choice[] = {
  {   0, &hf_rrc_present_06      , ASN1_NO_EXTENSIONS     , dissect_rrc_T_present_04 },
  {   1, &hf_rrc_notPresent_02   , ASN1_NO_EXTENSIONS     , dissect_rrc_T_notPresent_02 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_dpdchPresence_02(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_dpdchPresence_02, T_dpdchPresence_02_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t T_fdd_118_sequence[] = {
  { &hf_rrc_scramblingCodeType, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_ScramblingCodeType },
  { &hf_rrc_scramblingCode  , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UL_ScramblingCode },
  { &hf_rrc_dpdchPresence_02, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_dpdchPresence_02 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_fdd_118(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_fdd_118, T_fdd_118_sequence);

  return offset;
}


static const per_sequence_t T_tdd_69_sequence[] = {
  { &hf_rrc_ul_TimingAdvance_03, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_TimingAdvanceControl_r7 },
  { &hf_rrc_ul_CCTrCHList_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_CCTrCHList_r7 },
  { &hf_rrc_ul_CCTrCHListToRemove, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_CCTrCHListToRemove },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_tdd_69(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_tdd_69, T_tdd_69_sequence);

  return offset;
}


static const value_string rrc_T_modeSpecificInfo_86_vals[] = {
  {   0, "fdd" },
  {   1, "tdd" },
  { 0, NULL }
};

static const per_choice_t T_modeSpecificInfo_86_choice[] = {
  {   0, &hf_rrc_fdd_121         , ASN1_NO_EXTENSIONS     , dissect_rrc_T_fdd_118 },
  {   1, &hf_rrc_tdd_75          , ASN1_NO_EXTENSIONS     , dissect_rrc_T_tdd_69 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_modeSpecificInfo_86(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_modeSpecificInfo_86, T_modeSpecificInfo_86_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t UL_DPCH_Info_r11_sequence[] = {
  { &hf_rrc_ul_DPCH_PowerControlInfo_05, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_DPCH_PowerControlInfo_r11 },
  { &hf_rrc_modeSpecificInfo_86, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_modeSpecificInfo_86 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UL_DPCH_Info_r11(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UL_DPCH_Info_r11, UL_DPCH_Info_r11_sequence);

  return offset;
}


static const value_string rrc_T_mac_es_e_resetIndicator_05_vals[] = {
  {   0, "true" },
  { 0, NULL }
};


static int
dissect_rrc_T_mac_es_e_resetIndicator_05(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t T_fdd_131_sequence[] = {
  { &hf_rrc_e_DPCCH_Info_01 , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_E_DPCCH_Info_r7 },
  { &hf_rrc_e_DPDCH_Info_02 , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_E_DPDCH_Info_r8 },
  { &hf_rrc_ul_MIMO_Info    , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_MIMO_InfoFDD },
  { &hf_rrc_schedulingTransmConfiguration, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_E_DPDCH_SchedulingTransmConfiguration },
  { &hf_rrc_ul_16QAM_Settings, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_16QAM_Settings },
  { &hf_rrc_ul_64QAM_Settings, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_64QAM_Settings },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_fdd_131(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_fdd_131, T_fdd_131_sequence);

  return offset;
}


static const per_sequence_t T_tdd_82_sequence[] = {
  { &hf_rrc_e_RUCCH_Info_01 , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_E_RUCCH_Info },
  { &hf_rrc_e_PUCH_Info_01  , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_E_PUCH_Info },
  { &hf_rrc_non_ScheduledTransGrantInfo_04, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_Non_ScheduledTransGrantInfoTDD_r9 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_tdd_82(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_tdd_82, T_tdd_82_sequence);

  return offset;
}


static const value_string rrc_T_modeSpecificInfo_92_vals[] = {
  {   0, "fdd" },
  {   1, "tdd" },
  { 0, NULL }
};

static const per_choice_t T_modeSpecificInfo_92_choice[] = {
  {   0, &hf_rrc_fdd_134         , ASN1_NO_EXTENSIONS     , dissect_rrc_T_fdd_131 },
  {   1, &hf_rrc_tdd_88          , ASN1_NO_EXTENSIONS     , dissect_rrc_T_tdd_82 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_modeSpecificInfo_92(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_modeSpecificInfo_92, T_modeSpecificInfo_92_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t UL_EDCH_Information_r11_sequence[] = {
  { &hf_rrc_mac_es_e_resetIndicator_05, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_mac_es_e_resetIndicator_05 },
  { &hf_rrc_modeSpecificInfo_92, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_modeSpecificInfo_92 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UL_EDCH_Information_r11(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UL_EDCH_Information_r11, UL_EDCH_Information_r11_sequence);

  return offset;
}


static const value_string rrc_T_dl_dpchInfoCommon_04_vals[] = {
  {   0, "dl-DPCH-InfoCommon" },
  {   1, "dl-FDPCH-InfoCommon" },
  { 0, NULL }
};

static const per_choice_t T_dl_dpchInfoCommon_04_choice[] = {
  {   0, &hf_rrc_dl_DPCH_InfoCommon_02, ASN1_NO_EXTENSIONS     , dissect_rrc_DL_DPCH_InfoCommon_r6 },
  {   1, &hf_rrc_dl_FDPCH_InfoCommon, ASN1_NO_EXTENSIONS     , dissect_rrc_DL_FDPCH_InfoCommon_r6 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_dl_dpchInfoCommon_04(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_dl_dpchInfoCommon_04, T_dl_dpchInfoCommon_04_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t T_fdd_65_sequence[] = {
  { &hf_rrc_defaultDPCH_OffsetValue, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DefaultDPCH_OffsetValueFDD },
  { &hf_rrc_dpch_CompressedModeInfo_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DPCH_CompressedModeInfo_r10 },
  { &hf_rrc_tx_DiversityMode, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_TX_DiversityMode },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_fdd_65(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_fdd_65, T_fdd_65_sequence);

  return offset;
}


static const per_sequence_t T_tdd128_20_sequence[] = {
  { &hf_rrc_tstd_Indicator  , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_tdd128_20(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_tdd128_20, T_tdd128_20_sequence);

  return offset;
}


static const value_string rrc_T_tddOption_11_vals[] = {
  {   0, "tdd384" },
  {   1, "tdd768" },
  {   2, "tdd128" },
  { 0, NULL }
};

static const per_choice_t T_tddOption_11_choice[] = {
  {   0, &hf_rrc_tdd384_11       , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   1, &hf_rrc_tdd768_06       , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   2, &hf_rrc_tdd128_20       , ASN1_NO_EXTENSIONS     , dissect_rrc_T_tdd128_20 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_tddOption_11(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_tddOption_11, T_tddOption_11_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t T_tdd_27_sequence[] = {
  { &hf_rrc_tddOption_11    , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_tddOption_11 },
  { &hf_rrc_defaultDPCH_OffsetValue_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DefaultDPCH_OffsetValueTDD },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_tdd_27(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_tdd_27, T_tdd_27_sequence);

  return offset;
}


static const value_string rrc_T_modeSpecificInfo_35_vals[] = {
  {   0, "fdd" },
  {   1, "tdd" },
  { 0, NULL }
};

static const per_choice_t T_modeSpecificInfo_35_choice[] = {
  {   0, &hf_rrc_fdd_66          , ASN1_NO_EXTENSIONS     , dissect_rrc_T_fdd_65 },
  {   1, &hf_rrc_tdd_28          , ASN1_NO_EXTENSIONS     , dissect_rrc_T_tdd_27 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_modeSpecificInfo_35(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_modeSpecificInfo_35, T_modeSpecificInfo_35_choice,
                                 NULL);

  return offset;
}


static const value_string rrc_T_mac_hsResetIndicator_10_vals[] = {
  {   0, "true" },
  { 0, NULL }
};


static int
dissect_rrc_T_mac_hsResetIndicator_10(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}


static const value_string rrc_T_postVerificationPeriod_11_vals[] = {
  {   0, "true" },
  { 0, NULL }
};


static int
dissect_rrc_T_postVerificationPeriod_11(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}


static const value_string rrc_T_mac_hsResetIndicator_assisting_01_vals[] = {
  {   0, "true" },
  { 0, NULL }
};


static int
dissect_rrc_T_mac_hsResetIndicator_assisting_01(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t DL_CommonInformation_r11_sequence[] = {
  { &hf_rrc_dl_dpchInfoCommon_04, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_dl_dpchInfoCommon_04 },
  { &hf_rrc_modeSpecificInfo_35, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_modeSpecificInfo_35 },
  { &hf_rrc_mac_hsResetIndicator_10, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_mac_hsResetIndicator_10 },
  { &hf_rrc_postVerificationPeriod_11, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_postVerificationPeriod_11 },
  { &hf_rrc_mac_hsResetIndicator_assisting_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_mac_hsResetIndicator_assisting_01 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_DL_CommonInformation_r11(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_DL_CommonInformation_r11, DL_CommonInformation_r11_sequence);

  return offset;
}


static const per_sequence_t T_fdd_89_sequence[] = {
  { &hf_rrc_primaryCPICH_Info, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PrimaryCPICH_Info },
  { &hf_rrc_servingHSDSCH_RL_indicator, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { &hf_rrc_servingEDCH_RL_indicator, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_fdd_89(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_fdd_89, T_fdd_89_sequence);

  return offset;
}


static const value_string rrc_T_modeSpecificInfo_55_vals[] = {
  {   0, "fdd" },
  {   1, "tdd" },
  { 0, NULL }
};

static const per_choice_t T_modeSpecificInfo_55_choice[] = {
  {   0, &hf_rrc_fdd_90          , ASN1_NO_EXTENSIONS     , dissect_rrc_T_fdd_89 },
  {   1, &hf_rrc_tdd_47          , ASN1_NO_EXTENSIONS     , dissect_rrc_PrimaryCCPCH_Info_r4 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_modeSpecificInfo_55(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_modeSpecificInfo_55, T_modeSpecificInfo_55_choice,
                                 NULL);

  return offset;
}


static const value_string rrc_T_dl_dpchInfo_03_vals[] = {
  {   0, "dl-DPCH-InfoPerRL" },
  {   1, "dl-FDPCH-InfoPerRL" },
  { 0, NULL }
};

static const per_choice_t T_dl_dpchInfo_03_choice[] = {
  {   0, &hf_rrc_dl_DPCH_InfoPerRL_04, ASN1_NO_EXTENSIONS     , dissect_rrc_DL_DPCH_InfoPerRL_r7 },
  {   1, &hf_rrc_dl_FDPCH_InfoPerRL_01, ASN1_NO_EXTENSIONS     , dissect_rrc_DL_FDPCH_InfoPerRL_r7 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_dl_dpchInfo_03(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_dl_dpchInfo_03, T_dl_dpchInfo_03_choice,
                                 NULL);

  return offset;
}


static const value_string rrc_T_e_HICH_Info_03_vals[] = {
  {   0, "e-HICH-Information" },
  {   1, "releaseIndicator" },
  {   2, "secondaryReleaseIndicator" },
  { 0, NULL }
};

static const per_choice_t T_e_HICH_Info_03_choice[] = {
  {   0, &hf_rrc_e_HICH_Information_01, ASN1_NO_EXTENSIONS     , dissect_rrc_E_HICH_Information_r11 },
  {   1, &hf_rrc_releaseIndicator, ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   2, &hf_rrc_secondaryReleaseIndicator, ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_e_HICH_Info_03(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_e_HICH_Info_03, T_e_HICH_Info_03_choice,
                                 NULL);

  return offset;
}


static const value_string rrc_T_e_RGCH_Info_04_vals[] = {
  {   0, "e-RGCH-Information" },
  {   1, "releaseIndicator" },
  { 0, NULL }
};

static const per_choice_t T_e_RGCH_Info_04_choice[] = {
  {   0, &hf_rrc_e_RGCH_Information, ASN1_NO_EXTENSIONS     , dissect_rrc_E_RGCH_Information },
  {   1, &hf_rrc_releaseIndicator, ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_e_RGCH_Info_04(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_e_RGCH_Info_04, T_e_RGCH_Info_04_choice,
                                 NULL);

  return offset;
}


static const value_string rrc_T_f_TPICH_Info_vals[] = {
  {   0, "f-TPICH-Information" },
  {   1, "releaseIndicator" },
  { 0, NULL }
};

static const per_choice_t T_f_TPICH_Info_choice[] = {
  {   0, &hf_rrc_f_TPICH_Information, ASN1_NO_EXTENSIONS     , dissect_rrc_F_TPICH_Information },
  {   1, &hf_rrc_releaseIndicator, ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_f_TPICH_Info(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_f_TPICH_Info, T_f_TPICH_Info_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t T_fdd_90_sequence[] = {
  { &hf_rrc_e_HICH_Info_05  , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_e_HICH_Info_03 },
  { &hf_rrc_e_RGCH_Info_04  , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_e_RGCH_Info_04 },
  { &hf_rrc_f_TPICH_Info    , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_f_TPICH_Info },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_fdd_90(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_fdd_90, T_fdd_90_sequence);

  return offset;
}


static const per_sequence_t T_tdd384_tdd768_02_sequence[] = {
  { &hf_rrc_e_HICH_Info_02  , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_E_HICH_Information_TDD384_768 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_tdd384_tdd768_02(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_tdd384_tdd768_02, T_tdd384_tdd768_02_sequence);

  return offset;
}


static const per_sequence_t T_tdd128_30_sequence[] = {
  { &hf_rrc_e_HICH_Info_03  , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_E_HICH_Information_TDD128 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_tdd128_30(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_tdd128_30, T_tdd128_30_sequence);

  return offset;
}


static const value_string rrc_T_tdd_47_vals[] = {
  {   0, "tdd384-tdd768" },
  {   1, "tdd128" },
  { 0, NULL }
};

static const per_choice_t T_tdd_47_choice[] = {
  {   0, &hf_rrc_tdd384_tdd768_02, ASN1_NO_EXTENSIONS     , dissect_rrc_T_tdd384_tdd768_02 },
  {   1, &hf_rrc_tdd128_30       , ASN1_NO_EXTENSIONS     , dissect_rrc_T_tdd128_30 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_tdd_47(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_tdd_47, T_tdd_47_choice,
                                 NULL);

  return offset;
}


static const value_string rrc_T_modeSpecificInfo2_02_vals[] = {
  {   0, "fdd" },
  {   1, "tdd" },
  { 0, NULL }
};

static const per_choice_t T_modeSpecificInfo2_02_choice[] = {
  {   0, &hf_rrc_fdd_91          , ASN1_NO_EXTENSIONS     , dissect_rrc_T_fdd_90 },
  {   1, &hf_rrc_tdd_50          , ASN1_NO_EXTENSIONS     , dissect_rrc_T_tdd_47 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_modeSpecificInfo2_02(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_modeSpecificInfo2_02, T_modeSpecificInfo2_02_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t DL_InformationPerRL_r11_sequence[] = {
  { &hf_rrc_modeSpecificInfo_55, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_modeSpecificInfo_55 },
  { &hf_rrc_dl_dpchInfo_03  , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_dl_dpchInfo_03 },
  { &hf_rrc_e_AGCH_Information_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_E_AGCH_Information_r8 },
  { &hf_rrc_e_ROCH_Information, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_E_ROCH_Information },
  { &hf_rrc_modeSpecificInfo2_02, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_modeSpecificInfo2_02 },
  { &hf_rrc_cell_id         , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CellIdentity },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_DL_InformationPerRL_r11(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_DL_InformationPerRL_r11, DL_InformationPerRL_r11_sequence);

  return offset;
}


static const per_sequence_t DL_InformationPerRL_List_r11_sequence_of[1] = {
  { &hf_rrc_DL_InformationPerRL_List_r11_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DL_InformationPerRL_r11 },
};

static int
dissect_rrc_DL_InformationPerRL_List_r11(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_DL_InformationPerRL_List_r11, DL_InformationPerRL_List_r11_sequence_of,
                                                  1, maxRL, FALSE);

  return offset;
}



static int
dissect_rrc_BIT_STRING_SIZE_maxCellMeas(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_bit_string(tvb, offset, actx, tree, hf_index,
                                     maxCellMeas, maxCellMeas, FALSE, NULL);

  return offset;
}



static int
dissect_rrc_INTEGER_0_39(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            0U, 39U, NULL, FALSE);

  return offset;
}


static const per_sequence_t CommonERGCHChannelConfig_sequence[] = {
  { &hf_rrc_channelisationCode_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_INTEGER_0_127 },
  { &hf_rrc_signatureSequence, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_INTEGER_0_39 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_CommonERGCHChannelConfig(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_CommonERGCHChannelConfig, CommonERGCHChannelConfig_sequence);

  return offset;
}


static const per_sequence_t SEQUENCE_SIZE_1_maxCellMeas_OF_CommonERGCHChannelConfig_sequence_of[1] = {
  { &hf_rrc_commonERGCHChannelConfigList_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CommonERGCHChannelConfig },
};

static int
dissect_rrc_SEQUENCE_SIZE_1_maxCellMeas_OF_CommonERGCHChannelConfig(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_SEQUENCE_SIZE_1_maxCellMeas_OF_CommonERGCHChannelConfig, SEQUENCE_SIZE_1_maxCellMeas_OF_CommonERGCHChannelConfig_sequence_of,
                                                  1, maxCellMeas, FALSE);

  return offset;
}



static int
dissect_rrc_ReportingRange(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            0U, 29U, NULL, FALSE);

  return offset;
}


static const value_string rrc_FilterCoefficient_vals[] = {
  {   0, "fc0" },
  {   1, "fc1" },
  {   2, "fc2" },
  {   3, "fc3" },
  {   4, "fc4" },
  {   5, "fc5" },
  {   6, "fc6" },
  {   7, "fc7" },
  {   8, "fc8" },
  {   9, "fc9" },
  {  10, "fc11" },
  {  11, "fc13" },
  {  12, "fc15" },
  {  13, "fc17" },
  {  14, "fc19" },
  {  15, "spare1" },
  { 0, NULL }
};


static int
dissect_rrc_FilterCoefficient(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     16, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t T_newConfiguration_01_sequence[] = {
  { &hf_rrc_eRGCHNeighbourCellList, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_maxCellMeas },
  { &hf_rrc_commonERGCHChannelConfigList, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SEQUENCE_SIZE_1_maxCellMeas_OF_CommonERGCHChannelConfig },
  { &hf_rrc_minimumServingGrantValue, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_INTEGER_0_37 },
  { &hf_rrc_reportingRange  , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_ReportingRange },
  { &hf_rrc_filterCoefficient, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_FilterCoefficient },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_newConfiguration_01(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_newConfiguration_01, T_newConfiguration_01_sequence);

  return offset;
}


static const value_string rrc_T_configurationInfo_vals[] = {
  {   0, "continue" },
  {   1, "newConfiguration" },
  { 0, NULL }
};

static const per_choice_t T_configurationInfo_choice[] = {
  {   0, &hf_rrc_continue        , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   1, &hf_rrc_newConfiguration_01, ASN1_NO_EXTENSIONS     , dissect_rrc_T_newConfiguration_01 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_configurationInfo(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_configurationInfo, T_configurationInfo_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t CommonERGCHInfoFDD_sequence[] = {
  { &hf_rrc_configurationInfo, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_configurationInfo },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_CommonERGCHInfoFDD(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_CommonERGCHInfoFDD, CommonERGCHInfoFDD_sequence);

  return offset;
}


static const per_sequence_t CellUpdateConfirm_r11_IEs_sequence[] = {
  { &hf_rrc_integrityProtectionModeInfo_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_IntegrityProtectionModeInfo_r7 },
  { &hf_rrc_cipheringModeInfo_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CipheringModeInfo_r7 },
  { &hf_rrc_dummy2_03       , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_SR_VCC_Info },
  { &hf_rrc_activationTime  , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_ActivationTime },
  { &hf_rrc_new_U_RNTI      , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_U_RNTI },
  { &hf_rrc_new_C_RNTI      , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_C_RNTI },
  { &hf_rrc_new_DSCH_RNTI   , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DSCH_RNTI },
  { &hf_rrc_new_H_RNTI      , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_H_RNTI },
  { &hf_rrc_newPrimary_E_RNTI, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_E_RNTI },
  { &hf_rrc_newSecondary_E_RNTI, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_E_RNTI },
  { &hf_rrc_rrc_StateIndicator, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RRC_StateIndicator },
  { &hf_rrc_utran_DRX_CycleLengthCoeff_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UTRAN_DRX_CycleLengthCoefficient_r7 },
  { &hf_rrc_waitTime        , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_WaitTime },
  { &hf_rrc_rlc_Re_establishIndicatorRb2_3or4, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { &hf_rrc_rlc_Re_establishIndicatorRb5orAbove, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { &hf_rrc_cn_InformationInfo_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CN_InformationInfo_r6 },
  { &hf_rrc_ura_Identity    , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_URA_Identity },
  { &hf_rrc_supportForChangeOfUE_Capability, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_BOOLEAN },
  { &hf_rrc_dummy_07        , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DefaultConfigForCellFACH },
  { &hf_rrc_rab_InformationSetup, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RAB_InformationSetup_r8 },
  { &hf_rrc_rb_InformationReleaseList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RB_InformationReleaseList },
  { &hf_rrc_rb_InformationReconfigList_06, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RB_InformationReconfigList_r11 },
  { &hf_rrc_rb_InformationAffectedList_04, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RB_InformationAffectedList_r8 },
  { &hf_rrc_dl_CounterSynchronisationInfo_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_CounterSynchronisationInfo_r5 },
  { &hf_rrc_pdcp_ROHC_TargetMode, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_PDCP_ROHC_TargetMode },
  { &hf_rrc_ul_CommonTransChInfo_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_CommonTransChInfo_r4 },
  { &hf_rrc_ul_deletedTransChInfoList_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_DeletedTransChInfoList_r6 },
  { &hf_rrc_ul_AddReconfTransChInfoList_03, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_AddReconfTransChInfoList_r8 },
  { &hf_rrc_dl_CommonTransChInfo_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_CommonTransChInfo_r4 },
  { &hf_rrc_dl_DeletedTransChInfoList_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_DeletedTransChInfoList_r7 },
  { &hf_rrc_dl_AddReconfTransChInfoList_06, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_AddReconfTransChInfoList_r11 },
  { &hf_rrc_frequencyInfo   , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_FrequencyInfo },
  { &hf_rrc_multi_frequencyInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_Multi_frequencyInfo_LCR_r7 },
  { &hf_rrc_dtx_drx_TimingInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DTX_DRX_TimingInfo_r7 },
  { &hf_rrc_dtx_drx_Info    , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DTX_DRX_Info_r7 },
  { &hf_rrc_hs_scch_LessInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_HS_SCCH_LessInfo_r7 },
  { &hf_rrc_mimoParameters_04, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MIMO_Parameters_r9 },
  { &hf_rrc_mimo4x4Parameters, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MIMO4x4_Parameters },
  { &hf_rrc_maxAllowedUL_TX_Power, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MaxAllowedUL_TX_Power },
  { &hf_rrc_ul_DPCH_Info_02 , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_DPCH_Info_r11 },
  { &hf_rrc_ul_EDCH_Information_05, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_EDCH_Information_r11 },
  { &hf_rrc_ul_SecondaryCellInfoFDD, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_SecondaryCellInfoFDD },
  { &hf_rrc_ul_MulticarrierEDCHInfo_TDD128, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_MulticarrierEDCHInfo_TDD128 },
  { &hf_rrc_ul_CLTD_InfoFDD , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_CLTD_InfoFDD },
  { &hf_rrc_ul_OLTD_InfoFDD , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_OLTD_InfoFDD },
  { &hf_rrc_dl_HSPDSCH_Information_07, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_HSPDSCH_Information_r11 },
  { &hf_rrc_dl_CommonInformation_07, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_CommonInformation_r11 },
  { &hf_rrc_dl_InformationPerRL_List_06, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_InformationPerRL_List_r11 },
  { &hf_rrc_dl_SecondaryCellInfoFDD_03, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_SecondaryCellInfoFDD_r11 },
  { &hf_rrc_additionalDLSecCellInfoListFDD_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_AdditionalDLSecCellInfoListFDD_r11 },
  { &hf_rrc_additionalDLSecCellInfoListFDD2, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_AdditionalDLSecCellInfoListFDD2_r11 },
  { &hf_rrc_commonERGCHInfoFDD, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CommonERGCHInfoFDD },
  { &hf_rrc_controlChannelDRXInfo_TDD128, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_ControlChannelDRXInfo_TDD128_r8 },
  { &hf_rrc_sps_Information_TDD128, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_SPS_Information_TDD128_r8 },
  { &hf_rrc_mu_MIMO_Info_TDD128, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MU_MIMO_Info_TDD128 },
  { &hf_rrc_mbms_PL_ServiceRestrictInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MBMS_PL_ServiceRestrictInfo_r6 },
  { &hf_rrc_cellDCHMeasOccasionInfo_TDD128, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CellDCHMeasOccasionInfo_TDD128_r9 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_CellUpdateConfirm_r11_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_CellUpdateConfirm_r11_IEs, CellUpdateConfirm_r11_IEs_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions_22_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_nonCriticalExtensions_22(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_nonCriticalExtensions_22, T_nonCriticalExtensions_22_sequence);

  return offset;
}


static const per_sequence_t T_r11_01_sequence[] = {
  { &hf_rrc_cellUpdateConfirm_r11, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CellUpdateConfirm_r11_IEs },
  { &hf_rrc_cellUpdateConfirm_r11_add_ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_BIT_STRING },
  { &hf_rrc_nonCriticalExtensions_22, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_nonCriticalExtensions_22 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_r11_01(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_r11_01, T_r11_01_sequence);

  return offset;
}


static const per_sequence_t T_criticalExtensions_18_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_criticalExtensions_18(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_criticalExtensions_18, T_criticalExtensions_18_sequence);

  return offset;
}


static const value_string rrc_T_criticalExtensions_17_vals[] = {
  {   0, "r11" },
  {   1, "criticalExtensions" },
  { 0, NULL }
};

static const per_choice_t T_criticalExtensions_17_choice[] = {
  {   0, &hf_rrc_r11_01          , ASN1_NO_EXTENSIONS     , dissect_rrc_T_r11_01 },
  {   1, &hf_rrc_criticalExtensions_18, ASN1_NO_EXTENSIONS     , dissect_rrc_T_criticalExtensions_18 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_criticalExtensions_17(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_criticalExtensions_17, T_criticalExtensions_17_choice,
                                 NULL);

  return offset;
}


static const value_string rrc_T_criticalExtensions_16_vals[] = {
  {   0, "r10" },
  {   1, "criticalExtensions" },
  { 0, NULL }
};

static const per_choice_t T_criticalExtensions_16_choice[] = {
  {   0, &hf_rrc_r10_01          , ASN1_NO_EXTENSIONS     , dissect_rrc_T_r10_01 },
  {   1, &hf_rrc_criticalExtensions_17, ASN1_NO_EXTENSIONS     , dissect_rrc_T_criticalExtensions_17 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_criticalExtensions_16(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_criticalExtensions_16, T_criticalExtensions_16_choice,
                                 NULL);

  return offset;
}


static const value_string rrc_T_criticalExtensions_15_vals[] = {
  {   0, "r9" },
  {   1, "criticalExtensions" },
  { 0, NULL }
};

static const per_choice_t T_criticalExtensions_15_choice[] = {
  {   0, &hf_rrc_r9_01           , ASN1_NO_EXTENSIONS     , dissect_rrc_T_r9_01 },
  {   1, &hf_rrc_criticalExtensions_16, ASN1_NO_EXTENSIONS     , dissect_rrc_T_criticalExtensions_16 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_criticalExtensions_15(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_criticalExtensions_15, T_criticalExtensions_15_choice,
                                 NULL);

  return offset;
}


static const value_string rrc_T_criticalExtensions_14_vals[] = {
  {   0, "r8" },
  {   1, "criticalExtensions" },
  { 0, NULL }
};

static const per_choice_t T_criticalExtensions_14_choice[] = {
  {   0, &hf_rrc_r8_01           , ASN1_NO_EXTENSIONS     , dissect_rrc_T_r8_01 },
  {   1, &hf_rrc_criticalExtensions_15, ASN1_NO_EXTENSIONS     , dissect_rrc_T_criticalExtensions_15 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_criticalExtensions_14(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_criticalExtensions_14, T_criticalExtensions_14_choice,
                                 NULL);

  return offset;
}


static const value_string rrc_T_criticalExtensions_13_vals[] = {
  {   0, "r7" },
  {   1, "criticalExtensions" },
  { 0, NULL }
};

static const per_choice_t T_criticalExtensions_13_choice[] = {
  {   0, &hf_rrc_r7_01           , ASN1_NO_EXTENSIONS     , dissect_rrc_T_r7_01 },
  {   1, &hf_rrc_criticalExtensions_14, ASN1_NO_EXTENSIONS     , dissect_rrc_T_criticalExtensions_14 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_criticalExtensions_13(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_criticalExtensions_13, T_criticalExtensions_13_choice,
                                 NULL);

  return offset;
}


static const value_string rrc_T_criticalExtensions_12_vals[] = {
  {   0, "r6" },
  {   1, "criticalExtensions" },
  { 0, NULL }
};

static const per_choice_t T_criticalExtensions_12_choice[] = {
  {   0, &hf_rrc_r6_01           , ASN1_NO_EXTENSIONS     , dissect_rrc_T_r6_01 },
  {   1, &hf_rrc_criticalExtensions_13, ASN1_NO_EXTENSIONS     , dissect_rrc_T_criticalExtensions_13 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_criticalExtensions_12(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_criticalExtensions_12, T_criticalExtensions_12_choice,
                                 NULL);

  return offset;
}


static const value_string rrc_T_criticalExtensions_11_vals[] = {
  {   0, "r5" },
  {   1, "criticalExtensions" },
  { 0, NULL }
};

static const per_choice_t T_criticalExtensions_11_choice[] = {
  {   0, &hf_rrc_r5              , ASN1_NO_EXTENSIONS     , dissect_rrc_T_r5 },
  {   1, &hf_rrc_criticalExtensions_12, ASN1_NO_EXTENSIONS     , dissect_rrc_T_criticalExtensions_12 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_criticalExtensions_11(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_criticalExtensions_11, T_criticalExtensions_11_choice,
                                 NULL);

  return offset;
}


static const value_string rrc_T_criticalExtensions_10_vals[] = {
  {   0, "r4" },
  {   1, "criticalExtensions" },
  { 0, NULL }
};

static const per_choice_t T_criticalExtensions_10_choice[] = {
  {   0, &hf_rrc_r4              , ASN1_NO_EXTENSIONS     , dissect_rrc_T_r4 },
  {   1, &hf_rrc_criticalExtensions_11, ASN1_NO_EXTENSIONS     , dissect_rrc_T_criticalExtensions_11 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_criticalExtensions_10(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_criticalExtensions_10, T_criticalExtensions_10_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t T_later_than_r3_03_sequence[] = {
  { &hf_rrc_rrc_TransactionIdentifier, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RRC_TransactionIdentifier },
  { &hf_rrc_criticalExtensions_10, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_criticalExtensions_10 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_later_than_r3_03(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_later_than_r3_03, T_later_than_r3_03_sequence);

  return offset;
}


static const value_string rrc_CellUpdateConfirm_vals[] = {
  {   0, "r3" },
  {   1, "later-than-r3" },
  { 0, NULL }
};

static const per_choice_t CellUpdateConfirm_choice[] = {
  {   0, &hf_rrc_r3_04           , ASN1_NO_EXTENSIONS     , dissect_rrc_T_r3_04 },
  {   1, &hf_rrc_later_than_r3_03, ASN1_NO_EXTENSIONS     , dissect_rrc_T_later_than_r3_03 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_CellUpdateConfirm(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
#line 197 "../../asn1/rrc/rrc.cnf"
  col_append_str(actx->pinfo->cinfo, COL_INFO, "CellUpdateConfirm");
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_CellUpdateConfirm, CellUpdateConfirm_choice,
                                 NULL);




  return offset;
}



static int
dissect_rrc_COUNT_C_MSB(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            0U, 33554431U, NULL, FALSE);

  return offset;
}


static const per_sequence_t RB_COUNT_C_MSB_Information_sequence[] = {
  { &hf_rrc_rb_Identity     , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RB_Identity },
  { &hf_rrc_count_C_MSB_UL  , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_COUNT_C_MSB },
  { &hf_rrc_count_C_MSB_DL  , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_COUNT_C_MSB },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_RB_COUNT_C_MSB_Information(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_RB_COUNT_C_MSB_Information, RB_COUNT_C_MSB_Information_sequence);

  return offset;
}


static const per_sequence_t RB_COUNT_C_MSB_InformationList_sequence_of[1] = {
  { &hf_rrc_RB_COUNT_C_MSB_InformationList_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RB_COUNT_C_MSB_Information },
};

static int
dissect_rrc_RB_COUNT_C_MSB_InformationList(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_RB_COUNT_C_MSB_InformationList, RB_COUNT_C_MSB_InformationList_sequence_of,
                                                  1, maxRBallRABs, FALSE);

  return offset;
}


static const per_sequence_t CounterCheck_r3_IEs_sequence[] = {
  { &hf_rrc_rrc_TransactionIdentifier, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RRC_TransactionIdentifier },
  { &hf_rrc_rb_COUNT_C_MSB_InformationList, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RB_COUNT_C_MSB_InformationList },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_CounterCheck_r3_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_CounterCheck_r3_IEs, CounterCheck_r3_IEs_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions_33_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_nonCriticalExtensions_33(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_nonCriticalExtensions_33, T_nonCriticalExtensions_33_sequence);

  return offset;
}


static const per_sequence_t T_laterNonCriticalExtensions_09_sequence[] = {
  { &hf_rrc_counterCheck_r3_add_ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_BIT_STRING },
  { &hf_rrc_nonCriticalExtensions_33, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_nonCriticalExtensions_33 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_laterNonCriticalExtensions_09(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_laterNonCriticalExtensions_09, T_laterNonCriticalExtensions_09_sequence);

  return offset;
}


static const per_sequence_t T_r3_06_sequence[] = {
  { &hf_rrc_counterCheck_r3 , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CounterCheck_r3_IEs },
  { &hf_rrc_laterNonCriticalExtensions_09, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_laterNonCriticalExtensions_09 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_r3_06(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_r3_06, T_r3_06_sequence);

  return offset;
}


static const per_sequence_t T_criticalExtensions_28_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_criticalExtensions_28(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_criticalExtensions_28, T_criticalExtensions_28_sequence);

  return offset;
}


static const per_sequence_t T_later_than_r3_05_sequence[] = {
  { &hf_rrc_rrc_TransactionIdentifier, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RRC_TransactionIdentifier },
  { &hf_rrc_criticalExtensions_28, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_criticalExtensions_28 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_later_than_r3_05(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_later_than_r3_05, T_later_than_r3_05_sequence);

  return offset;
}


static const value_string rrc_CounterCheck_vals[] = {
  {   0, "r3" },
  {   1, "later-than-r3" },
  { 0, NULL }
};

static const per_choice_t CounterCheck_choice[] = {
  {   0, &hf_rrc_r3_06           , ASN1_NO_EXTENSIONS     , dissect_rrc_T_r3_06 },
  {   1, &hf_rrc_later_than_r3_05, ASN1_NO_EXTENSIONS     , dissect_rrc_T_later_than_r3_05 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_CounterCheck(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
#line 201 "../../asn1/rrc/rrc.cnf"
  col_append_str(actx->pinfo->cinfo, COL_INFO, "CounterCheck");
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_CounterCheck, CounterCheck_choice,
                                 NULL);




  return offset;
}



static int
dissect_rrc_NAS_Message(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
#line 147 "../../asn1/rrc/rrc.cnf"
  tvbuff_t *nas_message_tvb=NULL;

  offset = dissect_per_octet_string(tvb, offset, actx, tree, hf_index,
                                       1, 4095, FALSE, &nas_message_tvb);


    if (nas_message_tvb)
        call_dissector(gsm_a_dtap_handle,nas_message_tvb,actx->pinfo, tree);



  return offset;
}


static const per_sequence_t DownlinkDirectTransfer_r3_IEs_sequence[] = {
  { &hf_rrc_rrc_TransactionIdentifier, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RRC_TransactionIdentifier },
  { &hf_rrc_cn_DomainIdentity, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CN_DomainIdentity },
  { &hf_rrc_nas_Message     , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_NAS_Message },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_DownlinkDirectTransfer_r3_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_DownlinkDirectTransfer_r3_IEs, DownlinkDirectTransfer_r3_IEs_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions_35_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_nonCriticalExtensions_35(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_nonCriticalExtensions_35, T_nonCriticalExtensions_35_sequence);

  return offset;
}


static const per_sequence_t T_laterNonCriticalExtensions_11_sequence[] = {
  { &hf_rrc_downlinkDirectTransfer_r3_add_ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_BIT_STRING },
  { &hf_rrc_nonCriticalExtensions_35, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_nonCriticalExtensions_35 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_laterNonCriticalExtensions_11(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_laterNonCriticalExtensions_11, T_laterNonCriticalExtensions_11_sequence);

  return offset;
}


static const per_sequence_t T_r3_07_sequence[] = {
  { &hf_rrc_downlinkDirectTransfer_r3, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DownlinkDirectTransfer_r3_IEs },
  { &hf_rrc_laterNonCriticalExtensions_11, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_laterNonCriticalExtensions_11 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_r3_07(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_r3_07, T_r3_07_sequence);

  return offset;
}


static const per_sequence_t T_criticalExtensions_29_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_criticalExtensions_29(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_criticalExtensions_29, T_criticalExtensions_29_sequence);

  return offset;
}


static const per_sequence_t T_later_than_r3_06_sequence[] = {
  { &hf_rrc_rrc_TransactionIdentifier, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RRC_TransactionIdentifier },
  { &hf_rrc_criticalExtensions_29, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_criticalExtensions_29 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_later_than_r3_06(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_later_than_r3_06, T_later_than_r3_06_sequence);

  return offset;
}


static const value_string rrc_DownlinkDirectTransfer_vals[] = {
  {   0, "r3" },
  {   1, "later-than-r3" },
  { 0, NULL }
};

static const per_choice_t DownlinkDirectTransfer_choice[] = {
  {   0, &hf_rrc_r3_07           , ASN1_NO_EXTENSIONS     , dissect_rrc_T_r3_07 },
  {   1, &hf_rrc_later_than_r3_06, ASN1_NO_EXTENSIONS     , dissect_rrc_T_later_than_r3_06 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_DownlinkDirectTransfer(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
#line 209 "../../asn1/rrc/rrc.cnf"
  col_append_str(actx->pinfo->cinfo, COL_INFO, "DownlinkDirectTransfer");
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_DownlinkDirectTransfer, DownlinkDirectTransfer_choice,
                                 NULL);




  return offset;
}


static const per_sequence_t T_single_GSM_Message_r3_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_single_GSM_Message_r3(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
#line 523 "../../asn1/rrc/rrc.cnf"
  tvbuff_t *gsm_message_tvb=NULL;
  guint bits_remaining, whole_octets_remaining;

  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_single_GSM_Message_r3, T_single_GSM_Message_r3_sequence);


  bits_remaining = 8*tvb_length(tvb) - offset;
  whole_octets_remaining = bits_remaining / 8;

  gsm_message_tvb = tvb_new_octet_aligned(tvb, offset, 8*whole_octets_remaining);

  if (gsm_message_tvb) {
    add_new_data_source(actx->pinfo,gsm_message_tvb,"GSM Message(aligned)");
    call_dissector(gsm_a_dtap_handle,gsm_message_tvb,actx->pinfo, tree);
  }


  return offset;
}



static int
dissect_rrc_GSM_MessageList_item(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
#line 515 "../../asn1/rrc/rrc.cnf"
  tvbuff_t *gsm_messagelist_tvb=NULL;

  offset = dissect_per_bit_string(tvb, offset, actx, tree, hf_index,
                                     1, 512, FALSE, &gsm_messagelist_tvb);


  if (gsm_messagelist_tvb)
    call_dissector(gsm_a_dtap_handle,gsm_messagelist_tvb,actx->pinfo, tree);



  return offset;
}


static const per_sequence_t GSM_MessageList_sequence_of[1] = {
  { &hf_rrc_GSM_MessageList_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_GSM_MessageList_item },
};

static int
dissect_rrc_GSM_MessageList(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_GSM_MessageList, GSM_MessageList_sequence_of,
                                                  1, maxInterSysMessages, FALSE);

  return offset;
}


static const per_sequence_t T_gsm_MessageList_r3_sequence[] = {
  { &hf_rrc_gsm_Messages    , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_GSM_MessageList },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_gsm_MessageList_r3(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_gsm_MessageList_r3, T_gsm_MessageList_r3_sequence);

  return offset;
}


static const value_string rrc_T_gsm_message_vals[] = {
  {   0, "single-GSM-Message" },
  {   1, "gsm-MessageList" },
  { 0, NULL }
};

static const per_choice_t T_gsm_message_choice[] = {
  {   0, &hf_rrc_single_GSM_Message_r3, ASN1_NO_EXTENSIONS     , dissect_rrc_T_single_GSM_Message_r3 },
  {   1, &hf_rrc_gsm_MessageList_r3, ASN1_NO_EXTENSIONS     , dissect_rrc_T_gsm_MessageList_r3 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_gsm_message(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_gsm_message, T_gsm_message_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t HandoverFromUTRANCommand_GSM_r3_IEs_sequence[] = {
  { &hf_rrc_rrc_TransactionIdentifier, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RRC_TransactionIdentifier },
  { &hf_rrc_activationTime  , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_ActivationTime },
  { &hf_rrc_toHandoverRAB_Info, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RAB_Info },
  { &hf_rrc_frequency_band  , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_Frequency_Band },
  { &hf_rrc_gsm_message     , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_gsm_message },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_HandoverFromUTRANCommand_GSM_r3_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_HandoverFromUTRANCommand_GSM_r3_IEs, HandoverFromUTRANCommand_GSM_r3_IEs_sequence);

  return offset;
}


static const value_string rrc_T_geran_SystemInfoType_02_vals[] = {
  {   0, "sI" },
  {   1, "pSI" },
  { 0, NULL }
};

static const per_choice_t T_geran_SystemInfoType_02_choice[] = {
  {   0, &hf_rrc_sI              , ASN1_NO_EXTENSIONS     , dissect_rrc_GERAN_SystemInformation },
  {   1, &hf_rrc_pSI             , ASN1_NO_EXTENSIONS     , dissect_rrc_GERAN_SystemInformation },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_geran_SystemInfoType_02(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_geran_SystemInfoType_02, T_geran_SystemInfoType_02_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t HandoverFromUTRANCommand_GSM_v690ext_IEs_sequence[] = {
  { &hf_rrc_geran_SystemInfoType_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_geran_SystemInfoType_02 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_HandoverFromUTRANCommand_GSM_v690ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_HandoverFromUTRANCommand_GSM_v690ext_IEs, HandoverFromUTRANCommand_GSM_v690ext_IEs_sequence);

  return offset;
}


static const per_sequence_t SR_VCC_SecurityRABInfo_v860ext_sequence[] = {
  { &hf_rrc_sr_vcc_Info     , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SR_VCC_Info },
  { &hf_rrc_rab_InfoReplace , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RAB_InfoReplace },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_SR_VCC_SecurityRABInfo_v860ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_SR_VCC_SecurityRABInfo_v860ext, SR_VCC_SecurityRABInfo_v860ext_sequence);

  return offset;
}


static const per_sequence_t HandoverFromUTRANCommand_GSM_v860ext_IEs_sequence[] = {
  { &hf_rrc_sr_vcc_SecurityRABInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_SR_VCC_SecurityRABInfo_v860ext },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_HandoverFromUTRANCommand_GSM_v860ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_HandoverFromUTRANCommand_GSM_v860ext_IEs, HandoverFromUTRANCommand_GSM_v860ext_IEs_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions_49_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_nonCriticalExtensions_49(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_nonCriticalExtensions_49, T_nonCriticalExtensions_49_sequence);

  return offset;
}


static const per_sequence_t T_v860NonCriticalExtensions_04_sequence[] = {
  { &hf_rrc_handoverFromUTRANCommand_GSM_v860ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_HandoverFromUTRANCommand_GSM_v860ext_IEs },
  { &hf_rrc_nonCriticalExtensions_49, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_nonCriticalExtensions_49 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v860NonCriticalExtensions_04(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v860NonCriticalExtensions_04, T_v860NonCriticalExtensions_04_sequence);

  return offset;
}


static const per_sequence_t T_v690NonCriticalExtensions_09_sequence[] = {
  { &hf_rrc_handoverFromUTRANCommand_GSM_v690ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_HandoverFromUTRANCommand_GSM_v690ext_IEs },
  { &hf_rrc_v860NonCriticalExtensions_04, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v860NonCriticalExtensions_04 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v690NonCriticalExtensions_09(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v690NonCriticalExtensions_09, T_v690NonCriticalExtensions_09_sequence);

  return offset;
}


static const per_sequence_t T_laterNonCriticalExtensions_14_sequence[] = {
  { &hf_rrc_handoverFromUTRANCommand_GSM_r3_add_ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_BIT_STRING },
  { &hf_rrc_v690NonCriticalExtensions_09, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v690NonCriticalExtensions_09 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_laterNonCriticalExtensions_14(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_laterNonCriticalExtensions_14, T_laterNonCriticalExtensions_14_sequence);

  return offset;
}


static const per_sequence_t T_r3_09_sequence[] = {
  { &hf_rrc_handoverFromUTRANCommand_GSM_r3, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_HandoverFromUTRANCommand_GSM_r3_IEs },
  { &hf_rrc_laterNonCriticalExtensions_14, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_laterNonCriticalExtensions_14 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_r3_09(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_r3_09, T_r3_09_sequence);

  return offset;
}


static const per_sequence_t RAB_Info_r6_sequence[] = {
  { &hf_rrc_rab_Identity    , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RAB_Identity },
  { &hf_rrc_mbms_SessionIdentity, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MBMS_SessionIdentity },
  { &hf_rrc_cn_DomainIdentity, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CN_DomainIdentity },
  { &hf_rrc_nas_Synchronisation_Indicator, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_NAS_Synchronisation_Indicator },
  { &hf_rrc_re_EstablishmentTimer, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_Re_EstablishmentTimer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_RAB_Info_r6(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_RAB_Info_r6, RAB_Info_r6_sequence);

  return offset;
}


static const per_sequence_t RAB_InformationList_r6_sequence_of[1] = {
  { &hf_rrc_RAB_InformationList_r6_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RAB_Info_r6 },
};

static int
dissect_rrc_RAB_InformationList_r6(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_RAB_InformationList_r6, RAB_InformationList_r6_sequence_of,
                                                  1, maxRABsetup, FALSE);

  return offset;
}


static const per_sequence_t T_single_GSM_Message_r6_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_single_GSM_Message_r6(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
#line 538 "../../asn1/rrc/rrc.cnf"
  tvbuff_t *gsm_message_tvb=NULL;
  guint bits_remaining, whole_octets_remaining;

  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_single_GSM_Message_r6, T_single_GSM_Message_r6_sequence);


  bits_remaining = 8*tvb_length(tvb) - offset;
  whole_octets_remaining = bits_remaining / 8;

  gsm_message_tvb = tvb_new_octet_aligned(tvb, offset, 8*whole_octets_remaining);

  if (gsm_message_tvb) {
    add_new_data_source(actx->pinfo,gsm_message_tvb,"GSM Message(aligned)");
    call_dissector(gsm_a_dtap_handle,gsm_message_tvb,actx->pinfo, tree);
  }



  return offset;
}


static const per_sequence_t T_gsm_MessageList_r6_sequence[] = {
  { &hf_rrc_gsm_Messages    , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_GSM_MessageList },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_gsm_MessageList_r6(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_gsm_MessageList_r6, T_gsm_MessageList_r6_sequence);

  return offset;
}


static const value_string rrc_T_gsm_message_01_vals[] = {
  {   0, "single-GSM-Message" },
  {   1, "gsm-MessageList" },
  { 0, NULL }
};

static const per_choice_t T_gsm_message_01_choice[] = {
  {   0, &hf_rrc_single_GSM_Message_r6, ASN1_NO_EXTENSIONS     , dissect_rrc_T_single_GSM_Message_r6 },
  {   1, &hf_rrc_gsm_MessageList_r6, ASN1_NO_EXTENSIONS     , dissect_rrc_T_gsm_MessageList_r6 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_gsm_message_01(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_gsm_message_01, T_gsm_message_01_choice,
                                 NULL);

  return offset;
}


static const value_string rrc_T_geran_SystemInfoType_01_vals[] = {
  {   0, "sI" },
  {   1, "pSI" },
  { 0, NULL }
};

static const per_choice_t T_geran_SystemInfoType_01_choice[] = {
  {   0, &hf_rrc_sI              , ASN1_NO_EXTENSIONS     , dissect_rrc_GERAN_SystemInformation },
  {   1, &hf_rrc_pSI             , ASN1_NO_EXTENSIONS     , dissect_rrc_GERAN_SystemInformation },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_geran_SystemInfoType_01(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_geran_SystemInfoType_01, T_geran_SystemInfoType_01_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t HandoverFromUTRANCommand_GSM_r6_IEs_sequence[] = {
  { &hf_rrc_activationTime  , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_ActivationTime },
  { &hf_rrc_toHandoverRAB_Info_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RAB_InformationList_r6 },
  { &hf_rrc_frequency_band  , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_Frequency_Band },
  { &hf_rrc_gsm_message_01  , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_gsm_message_01 },
  { &hf_rrc_geran_SystemInfoType_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_geran_SystemInfoType_01 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_HandoverFromUTRANCommand_GSM_r6_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_HandoverFromUTRANCommand_GSM_r6_IEs, HandoverFromUTRANCommand_GSM_r6_IEs_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions_50_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_nonCriticalExtensions_50(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_nonCriticalExtensions_50, T_nonCriticalExtensions_50_sequence);

  return offset;
}


static const per_sequence_t T_v860NonCriticalExtensions_05_sequence[] = {
  { &hf_rrc_handoverFromUTRANCommand_GSM_v860ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_HandoverFromUTRANCommand_GSM_v860ext_IEs },
  { &hf_rrc_nonCriticalExtensions_50, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_nonCriticalExtensions_50 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v860NonCriticalExtensions_05(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v860NonCriticalExtensions_05, T_v860NonCriticalExtensions_05_sequence);

  return offset;
}


static const per_sequence_t T_r6_04_sequence[] = {
  { &hf_rrc_handoverFromUTRANCommand_GSM_r6, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_HandoverFromUTRANCommand_GSM_r6_IEs },
  { &hf_rrc_handoverFromUTRANCommand_GSM_r6_add_ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_BIT_STRING },
  { &hf_rrc_v860NonCriticalExtensions_05, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v860NonCriticalExtensions_05 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_r6_04(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_r6_04, T_r6_04_sequence);

  return offset;
}


static const per_sequence_t T_criticalExtensions_39_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_criticalExtensions_39(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_criticalExtensions_39, T_criticalExtensions_39_sequence);

  return offset;
}


static const value_string rrc_T_criticalExtensions_38_vals[] = {
  {   0, "r6" },
  {   1, "criticalExtensions" },
  { 0, NULL }
};

static const per_choice_t T_criticalExtensions_38_choice[] = {
  {   0, &hf_rrc_r6_04           , ASN1_NO_EXTENSIONS     , dissect_rrc_T_r6_04 },
  {   1, &hf_rrc_criticalExtensions_40, ASN1_NO_EXTENSIONS     , dissect_rrc_T_criticalExtensions_39 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_criticalExtensions_38(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_criticalExtensions_38, T_criticalExtensions_38_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t T_later_than_r3_07_sequence[] = {
  { &hf_rrc_rrc_TransactionIdentifier, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RRC_TransactionIdentifier },
  { &hf_rrc_criticalExtensions_39, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_criticalExtensions_38 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_later_than_r3_07(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_later_than_r3_07, T_later_than_r3_07_sequence);

  return offset;
}


static const value_string rrc_HandoverFromUTRANCommand_GSM_vals[] = {
  {   0, "r3" },
  {   1, "later-than-r3" },
  { 0, NULL }
};

static const per_choice_t HandoverFromUTRANCommand_GSM_choice[] = {
  {   0, &hf_rrc_r3_09           , ASN1_NO_EXTENSIONS     , dissect_rrc_T_r3_09 },
  {   1, &hf_rrc_later_than_r3_07, ASN1_NO_EXTENSIONS     , dissect_rrc_T_later_than_r3_07 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_HandoverFromUTRANCommand_GSM(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
#line 233 "../../asn1/rrc/rrc.cnf"
  col_append_str(actx->pinfo->cinfo, COL_INFO, "HandoverFromUTRANCommand-GSM");
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_HandoverFromUTRANCommand_GSM, HandoverFromUTRANCommand_GSM_choice,
                                 NULL);




  return offset;
}



static int
dissect_rrc_BIT_STRING_SIZE_1_512(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_bit_string(tvb, offset, actx, tree, hf_index,
                                     1, 512, FALSE, NULL);

  return offset;
}


static const per_sequence_t CDMA2000_Message_sequence[] = {
  { &hf_rrc_msg_Type        , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_8 },
  { &hf_rrc_payload_02      , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_1_512 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_CDMA2000_Message(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_CDMA2000_Message, CDMA2000_Message_sequence);

  return offset;
}


static const per_sequence_t CDMA2000_MessageList_sequence_of[1] = {
  { &hf_rrc_CDMA2000_MessageList_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CDMA2000_Message },
};

static int
dissect_rrc_CDMA2000_MessageList(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_CDMA2000_MessageList, CDMA2000_MessageList_sequence_of,
                                                  1, maxInterSysMessages, FALSE);

  return offset;
}


static const per_sequence_t HandoverFromUTRANCommand_CDMA2000_r3_IEs_sequence[] = {
  { &hf_rrc_rrc_TransactionIdentifier, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RRC_TransactionIdentifier },
  { &hf_rrc_activationTime  , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_ActivationTime },
  { &hf_rrc_toHandoverRAB_Info, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RAB_Info },
  { &hf_rrc_cdma2000_MessageList, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CDMA2000_MessageList },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_HandoverFromUTRANCommand_CDMA2000_r3_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_HandoverFromUTRANCommand_CDMA2000_r3_IEs, HandoverFromUTRANCommand_CDMA2000_r3_IEs_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions_52_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_nonCriticalExtensions_52(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_nonCriticalExtensions_52, T_nonCriticalExtensions_52_sequence);

  return offset;
}


static const per_sequence_t T_laterNonCriticalExtensions_15_sequence[] = {
  { &hf_rrc_handoverFromUTRANCommand_CDMA2000_r3_add_ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_BIT_STRING },
  { &hf_rrc_nonCriticalExtensions_52, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_nonCriticalExtensions_52 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_laterNonCriticalExtensions_15(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_laterNonCriticalExtensions_15, T_laterNonCriticalExtensions_15_sequence);

  return offset;
}


static const per_sequence_t T_r3_10_sequence[] = {
  { &hf_rrc_handoverFromUTRANCommand_CDMA2000_r3, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_HandoverFromUTRANCommand_CDMA2000_r3_IEs },
  { &hf_rrc_laterNonCriticalExtensions_15, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_laterNonCriticalExtensions_15 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_r3_10(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_r3_10, T_r3_10_sequence);

  return offset;
}


static const per_sequence_t T_criticalExtensions_41_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_criticalExtensions_41(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_criticalExtensions_41, T_criticalExtensions_41_sequence);

  return offset;
}


static const per_sequence_t T_later_than_r3_08_sequence[] = {
  { &hf_rrc_rrc_TransactionIdentifier, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RRC_TransactionIdentifier },
  { &hf_rrc_criticalExtensions_42, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_criticalExtensions_41 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_later_than_r3_08(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_later_than_r3_08, T_later_than_r3_08_sequence);

  return offset;
}


static const value_string rrc_HandoverFromUTRANCommand_CDMA2000_vals[] = {
  {   0, "r3" },
  {   1, "later-than-r3" },
  { 0, NULL }
};

static const per_choice_t HandoverFromUTRANCommand_CDMA2000_choice[] = {
  {   0, &hf_rrc_r3_10           , ASN1_NO_EXTENSIONS     , dissect_rrc_T_r3_10 },
  {   1, &hf_rrc_later_than_r3_08, ASN1_NO_EXTENSIONS     , dissect_rrc_T_later_than_r3_08 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_HandoverFromUTRANCommand_CDMA2000(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
#line 237 "../../asn1/rrc/rrc.cnf"
  col_append_str(actx->pinfo->cinfo, COL_INFO, "HandoverFromUTRANCommand-CDMA2000");
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_HandoverFromUTRANCommand_CDMA2000, HandoverFromUTRANCommand_CDMA2000_choice,
                                 NULL);




  return offset;
}



static int
dissect_rrc_MeasurementIdentity(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            1U, 16U, NULL, FALSE);

  return offset;
}



static int
dissect_rrc_IntraFreqCellID(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            0U, maxCellMeas_1, NULL, FALSE);

  return offset;
}


static const per_sequence_t SEQUENCE_SIZE_1_maxCellMeas_OF_IntraFreqCellID_sequence_of[1] = {
  { &hf_rrc_removeSomeIntraFreqCells_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_IntraFreqCellID },
};

static int
dissect_rrc_SEQUENCE_SIZE_1_maxCellMeas_OF_IntraFreqCellID(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_SEQUENCE_SIZE_1_maxCellMeas_OF_IntraFreqCellID, SEQUENCE_SIZE_1_maxCellMeas_OF_IntraFreqCellID_sequence_of,
                                                  1, maxCellMeas, FALSE);

  return offset;
}


static const value_string rrc_RemovedIntraFreqCellList_vals[] = {
  {   0, "removeAllIntraFreqCells" },
  {   1, "removeSomeIntraFreqCells" },
  {   2, "removeNoIntraFreqCells" },
  { 0, NULL }
};

static const per_choice_t RemovedIntraFreqCellList_choice[] = {
  {   0, &hf_rrc_removeAllIntraFreqCells, ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   1, &hf_rrc_removeSomeIntraFreqCells, ASN1_NO_EXTENSIONS     , dissect_rrc_SEQUENCE_SIZE_1_maxCellMeas_OF_IntraFreqCellID },
  {   2, &hf_rrc_removeNoIntraFreqCells, ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_RemovedIntraFreqCellList(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_RemovedIntraFreqCellList, RemovedIntraFreqCellList_choice,
                                 NULL);

  return offset;
}



static int
dissect_rrc_CellIndividualOffset(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            -20, 20U, NULL, FALSE);

  return offset;
}



static int
dissect_rrc_INTEGER_0_960(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            0U, 960U, NULL, FALSE);

  return offset;
}



static int
dissect_rrc_INTEGER_0_150(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            0U, 150U, NULL, FALSE);

  return offset;
}


static const value_string rrc_ReferenceTimeDifferenceToCell_vals[] = {
  {   0, "accuracy40" },
  {   1, "accuracy256" },
  {   2, "accuracy2560" },
  { 0, NULL }
};

static const per_choice_t ReferenceTimeDifferenceToCell_choice[] = {
  {   0, &hf_rrc_accuracy40      , ASN1_NO_EXTENSIONS     , dissect_rrc_INTEGER_0_960 },
  {   1, &hf_rrc_accuracy256     , ASN1_NO_EXTENSIONS     , dissect_rrc_INTEGER_0_150 },
  {   2, &hf_rrc_accuracy2560    , ASN1_NO_EXTENSIONS     , dissect_rrc_INTEGER_0_15 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_ReferenceTimeDifferenceToCell(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_ReferenceTimeDifferenceToCell, ReferenceTimeDifferenceToCell_choice,
                                 NULL);

  return offset;
}



static int
dissect_rrc_PrimaryCPICH_TX_Power(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            -10, 50U, NULL, FALSE);

  return offset;
}


static const per_sequence_t T_fdd_132_sequence[] = {
  { &hf_rrc_primaryCPICH_Info, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_PrimaryCPICH_Info },
  { &hf_rrc_primaryCPICH_TX_Power, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_PrimaryCPICH_TX_Power },
  { &hf_rrc_readSFN_Indicator, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { &hf_rrc_tx_DiversityIndicator, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_fdd_132(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_fdd_132, T_fdd_132_sequence);

  return offset;
}


static const per_sequence_t TimeslotInfo_sequence[] = {
  { &hf_rrc_timeslotNumber  , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TimeslotNumber },
  { &hf_rrc_burstType       , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BurstType },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_TimeslotInfo(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_TimeslotInfo, TimeslotInfo_sequence);

  return offset;
}


static const per_sequence_t TimeslotInfoList_sequence_of[1] = {
  { &hf_rrc_TimeslotInfoList_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TimeslotInfo },
};

static int
dissect_rrc_TimeslotInfoList(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_TimeslotInfoList, TimeslotInfoList_sequence_of,
                                                  1, maxTS, FALSE);

  return offset;
}


static const per_sequence_t T_tdd_83_sequence[] = {
  { &hf_rrc_primaryCCPCH_Info_02, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PrimaryCCPCH_Info },
  { &hf_rrc_primaryCCPCH_TX_Power, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_PrimaryCCPCH_TX_Power },
  { &hf_rrc_timeslotInfoList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_TimeslotInfoList },
  { &hf_rrc_readSFN_Indicator, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_tdd_83(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_tdd_83, T_tdd_83_sequence);

  return offset;
}


static const value_string rrc_T_modeSpecificInfo_93_vals[] = {
  {   0, "fdd" },
  {   1, "tdd" },
  { 0, NULL }
};

static const per_choice_t T_modeSpecificInfo_93_choice[] = {
  {   0, &hf_rrc_fdd_135         , ASN1_NO_EXTENSIONS     , dissect_rrc_T_fdd_132 },
  {   1, &hf_rrc_tdd_89          , ASN1_NO_EXTENSIONS     , dissect_rrc_T_tdd_83 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_modeSpecificInfo_93(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_modeSpecificInfo_93, T_modeSpecificInfo_93_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t CellInfo_sequence[] = {
  { &hf_rrc_cellIndividualOffset, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CellIndividualOffset },
  { &hf_rrc_referenceTimeDifferenceToCell, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_ReferenceTimeDifferenceToCell },
  { &hf_rrc_modeSpecificInfo_93, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_modeSpecificInfo_93 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_CellInfo(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_CellInfo, CellInfo_sequence);

  return offset;
}


static const per_sequence_t NewIntraFreqCell_sequence[] = {
  { &hf_rrc_intraFreqCellID , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_IntraFreqCellID },
  { &hf_rrc_cellInfo        , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CellInfo },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_NewIntraFreqCell(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_NewIntraFreqCell, NewIntraFreqCell_sequence);

  return offset;
}


static const per_sequence_t NewIntraFreqCellList_sequence_of[1] = {
  { &hf_rrc_NewIntraFreqCellList_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_NewIntraFreqCell },
};

static int
dissect_rrc_NewIntraFreqCellList(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_NewIntraFreqCellList, NewIntraFreqCellList_sequence_of,
                                                  1, maxCellMeas, FALSE);

  return offset;
}


static const per_sequence_t CellsForIntraFreqMeasList_sequence_of[1] = {
  { &hf_rrc_CellsForIntraFreqMeasList_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_IntraFreqCellID },
};

static int
dissect_rrc_CellsForIntraFreqMeasList(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_CellsForIntraFreqMeasList, CellsForIntraFreqMeasList_sequence_of,
                                                  1, maxCellMeas, FALSE);

  return offset;
}


static const per_sequence_t IntraFreqCellInfoList_sequence[] = {
  { &hf_rrc_removedIntraFreqCellList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RemovedIntraFreqCellList },
  { &hf_rrc_newIntraFreqCellList_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_NewIntraFreqCellList },
  { &hf_rrc_cellsForIntraFreqMeasList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CellsForIntraFreqMeasList },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_IntraFreqCellInfoList(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_IntraFreqCellInfoList, IntraFreqCellInfoList_sequence);

  return offset;
}


static const value_string rrc_IntraFreqMeasQuantity_FDD_vals[] = {
  {   0, "cpich-Ec-N0" },
  {   1, "cpich-RSCP" },
  {   2, "pathloss" },
  {   3, "dummy" },
  { 0, NULL }
};


static int
dissect_rrc_IntraFreqMeasQuantity_FDD(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     4, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t T_fdd_155_sequence[] = {
  { &hf_rrc_intraFreqMeasQuantity_FDD, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_IntraFreqMeasQuantity_FDD },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_fdd_155(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_fdd_155, T_fdd_155_sequence);

  return offset;
}


static const value_string rrc_IntraFreqMeasQuantity_TDD_vals[] = {
  {   0, "primaryCCPCH-RSCP" },
  {   1, "pathloss" },
  {   2, "timeslotISCP" },
  {   3, "dummy" },
  { 0, NULL }
};


static int
dissect_rrc_IntraFreqMeasQuantity_TDD(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     4, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t IntraFreqMeasQuantity_TDDList_sequence_of[1] = {
  { &hf_rrc_IntraFreqMeasQuantity_TDDList_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_IntraFreqMeasQuantity_TDD },
};

static int
dissect_rrc_IntraFreqMeasQuantity_TDDList(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_IntraFreqMeasQuantity_TDDList, IntraFreqMeasQuantity_TDDList_sequence_of,
                                                  1, 4, FALSE);

  return offset;
}


static const per_sequence_t T_tdd_103_sequence[] = {
  { &hf_rrc_intraFreqMeasQuantity_TDDList, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_IntraFreqMeasQuantity_TDDList },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_tdd_103(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_tdd_103, T_tdd_103_sequence);

  return offset;
}


static const value_string rrc_T_modeSpecificInfo_116_vals[] = {
  {   0, "fdd" },
  {   1, "tdd" },
  { 0, NULL }
};

static const per_choice_t T_modeSpecificInfo_116_choice[] = {
  {   0, &hf_rrc_fdd_161         , ASN1_NO_EXTENSIONS     , dissect_rrc_T_fdd_155 },
  {   1, &hf_rrc_tdd_111         , ASN1_NO_EXTENSIONS     , dissect_rrc_T_tdd_103 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_modeSpecificInfo_116(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_modeSpecificInfo_116, T_modeSpecificInfo_116_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t IntraFreqMeasQuantity_sequence[] = {
  { &hf_rrc_filterCoefficient, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_FilterCoefficient },
  { &hf_rrc_modeSpecificInfo_116, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_modeSpecificInfo_116 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_IntraFreqMeasQuantity(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_IntraFreqMeasQuantity, IntraFreqMeasQuantity_sequence);

  return offset;
}


static const value_string rrc_SFN_SFN_OTD_Type_vals[] = {
  {   0, "noReport" },
  {   1, "type1" },
  {   2, "type2" },
  { 0, NULL }
};


static int
dissect_rrc_SFN_SFN_OTD_Type(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     3, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t T_fdd_141_sequence[] = {
  { &hf_rrc_cpich_Ec_N0_reportingIndicator, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { &hf_rrc_cpich_RSCP_reportingIndicator, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { &hf_rrc_pathloss_reportingIndicator, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_fdd_141(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_fdd_141, T_fdd_141_sequence);

  return offset;
}


static const per_sequence_t T_tdd_92_sequence[] = {
  { &hf_rrc_timeslotISCP_reportingIndicator, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { &hf_rrc_proposedTGSN_ReportingRequired, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { &hf_rrc_primaryCCPCH_RSCP_reportingIndicator, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { &hf_rrc_pathloss_reportingIndicator, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_tdd_92(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_tdd_92, T_tdd_92_sequence);

  return offset;
}


static const value_string rrc_T_modeSpecificInfo_102_vals[] = {
  {   0, "fdd" },
  {   1, "tdd" },
  { 0, NULL }
};

static const per_choice_t T_modeSpecificInfo_102_choice[] = {
  {   0, &hf_rrc_fdd_146         , ASN1_NO_EXTENSIONS     , dissect_rrc_T_fdd_141 },
  {   1, &hf_rrc_tdd_99          , ASN1_NO_EXTENSIONS     , dissect_rrc_T_tdd_92 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_modeSpecificInfo_102(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_modeSpecificInfo_102, T_modeSpecificInfo_102_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t CellReportingQuantities_sequence[] = {
  { &hf_rrc_dummy_49        , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SFN_SFN_OTD_Type },
  { &hf_rrc_cellIdentity_reportingIndicator, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { &hf_rrc_cellSynchronisationInfoReportingIndicator, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { &hf_rrc_modeSpecificInfo_102, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_modeSpecificInfo_102 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_CellReportingQuantities(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_CellReportingQuantities, CellReportingQuantities_sequence);

  return offset;
}


static const per_sequence_t IntraFreqReportingQuantity_sequence[] = {
  { &hf_rrc_activeSetReportingQuantities, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CellReportingQuantities },
  { &hf_rrc_monitoredSetReportingQuantities, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CellReportingQuantities },
  { &hf_rrc_detectedSetReportingQuantities, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CellReportingQuantities },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_IntraFreqReportingQuantity(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_IntraFreqReportingQuantity, IntraFreqReportingQuantity_sequence);

  return offset;
}


static const value_string rrc_T_ue_State_vals[] = {
  {   0, "cell-DCH" },
  {   1, "all-But-Cell-DCH" },
  {   2, "all-States" },
  { 0, NULL }
};


static int
dissect_rrc_T_ue_State(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     3, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t MeasurementValidity_sequence[] = {
  { &hf_rrc_ue_State        , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_ue_State },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_MeasurementValidity(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_MeasurementValidity, MeasurementValidity_sequence);

  return offset;
}


static const value_string rrc_TriggeringCondition2_vals[] = {
  {   0, "activeSetCellsOnly" },
  {   1, "monitoredSetCellsOnly" },
  {   2, "activeSetAndMonitoredSetCells" },
  {   3, "detectedSetCellsOnly" },
  {   4, "detectedSetAndMonitoredSetCells" },
  { 0, NULL }
};


static int
dissect_rrc_TriggeringCondition2(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     5, NULL, FALSE, 0, NULL);

  return offset;
}


static const value_string rrc_ForbiddenAffectCell_vals[] = {
  {   0, "fdd" },
  {   1, "tdd" },
  { 0, NULL }
};

static const per_choice_t ForbiddenAffectCell_choice[] = {
  {   0, &hf_rrc_fdd_155         , ASN1_NO_EXTENSIONS     , dissect_rrc_PrimaryCPICH_Info },
  {   1, &hf_rrc_tdd_46          , ASN1_NO_EXTENSIONS     , dissect_rrc_PrimaryCCPCH_Info },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_ForbiddenAffectCell(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_ForbiddenAffectCell, ForbiddenAffectCell_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t ForbiddenAffectCellList_sequence_of[1] = {
  { &hf_rrc_ForbiddenAffectCellList_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_ForbiddenAffectCell },
};

static int
dissect_rrc_ForbiddenAffectCellList(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_ForbiddenAffectCellList, ForbiddenAffectCellList_sequence_of,
                                                  1, maxCellMeas, FALSE);

  return offset;
}



static int
dissect_rrc_W(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            0U, 20U, NULL, FALSE);

  return offset;
}


static const value_string rrc_ReportDeactivationThreshold_vals[] = {
  {   0, "notApplicable" },
  {   1, "t1" },
  {   2, "t2" },
  {   3, "t3" },
  {   4, "t4" },
  {   5, "t5" },
  {   6, "t6" },
  {   7, "t7" },
  { 0, NULL }
};


static int
dissect_rrc_ReportDeactivationThreshold(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     8, NULL, FALSE, 0, NULL);

  return offset;
}


static const value_string rrc_ReportingAmount_vals[] = {
  {   0, "ra1" },
  {   1, "ra2" },
  {   2, "ra4" },
  {   3, "ra8" },
  {   4, "ra16" },
  {   5, "ra32" },
  {   6, "ra64" },
  {   7, "ra-Infinity" },
  { 0, NULL }
};


static int
dissect_rrc_ReportingAmount(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     8, NULL, FALSE, 0, NULL);

  return offset;
}


static const value_string rrc_ReportingInterval_vals[] = {
  {   0, "noPeriodicalreporting" },
  {   1, "ri0-25" },
  {   2, "ri0-5" },
  {   3, "ri1" },
  {   4, "ri2" },
  {   5, "ri4" },
  {   6, "ri8" },
  {   7, "ri16" },
  { 0, NULL }
};


static int
dissect_rrc_ReportingInterval(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     8, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t Event1a_sequence[] = {
  { &hf_rrc_triggeringCondition, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TriggeringCondition2 },
  { &hf_rrc_reportingRange  , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_ReportingRange },
  { &hf_rrc_forbiddenAffectCellList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_ForbiddenAffectCellList },
  { &hf_rrc_w               , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_W },
  { &hf_rrc_reportDeactivationThreshold, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_ReportDeactivationThreshold },
  { &hf_rrc_reportingAmount , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_ReportingAmount },
  { &hf_rrc_reportingInterval, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_ReportingInterval },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_Event1a(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_Event1a, Event1a_sequence);

  return offset;
}


static const value_string rrc_TriggeringCondition1_vals[] = {
  {   0, "activeSetCellsOnly" },
  {   1, "monitoredSetCellsOnly" },
  {   2, "activeSetAndMonitoredSetCells" },
  { 0, NULL }
};


static int
dissect_rrc_TriggeringCondition1(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     3, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t Event1b_sequence[] = {
  { &hf_rrc_triggeringCondition_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TriggeringCondition1 },
  { &hf_rrc_reportingRange  , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_ReportingRange },
  { &hf_rrc_forbiddenAffectCellList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_ForbiddenAffectCellList },
  { &hf_rrc_w               , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_W },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_Event1b(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_Event1b, Event1b_sequence);

  return offset;
}


static const value_string rrc_ReplacementActivationThreshold_vals[] = {
  {   0, "notApplicable" },
  {   1, "t1" },
  {   2, "t2" },
  {   3, "t3" },
  {   4, "t4" },
  {   5, "t5" },
  {   6, "t6" },
  {   7, "t7" },
  { 0, NULL }
};


static int
dissect_rrc_ReplacementActivationThreshold(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     8, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t Event1c_sequence[] = {
  { &hf_rrc_replacementActivationThreshold, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_ReplacementActivationThreshold },
  { &hf_rrc_reportingAmount , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_ReportingAmount },
  { &hf_rrc_reportingInterval, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_ReportingInterval },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_Event1c(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_Event1c, Event1c_sequence);

  return offset;
}



static int
dissect_rrc_ThresholdUsedFrequency(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            -115, 165U, NULL, FALSE);

  return offset;
}


static const per_sequence_t Event1e_sequence[] = {
  { &hf_rrc_triggeringCondition, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TriggeringCondition2 },
  { &hf_rrc_thresholdUsedFrequency, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_ThresholdUsedFrequency },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_Event1e(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_Event1e, Event1e_sequence);

  return offset;
}


static const per_sequence_t Event1f_sequence[] = {
  { &hf_rrc_triggeringCondition_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TriggeringCondition1 },
  { &hf_rrc_thresholdUsedFrequency, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_ThresholdUsedFrequency },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_Event1f(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_Event1f, Event1f_sequence);

  return offset;
}


static const value_string rrc_IntraFreqEvent_vals[] = {
  {   0, "e1a" },
  {   1, "e1b" },
  {   2, "e1c" },
  {   3, "e1d" },
  {   4, "e1e" },
  {   5, "e1f" },
  {   6, "e1g" },
  {   7, "e1h" },
  {   8, "e1i" },
  { 0, NULL }
};

static const per_choice_t IntraFreqEvent_choice[] = {
  {   0, &hf_rrc_e1a             , ASN1_NO_EXTENSIONS     , dissect_rrc_Event1a },
  {   1, &hf_rrc_e1b             , ASN1_NO_EXTENSIONS     , dissect_rrc_Event1b },
  {   2, &hf_rrc_e1c             , ASN1_NO_EXTENSIONS     , dissect_rrc_Event1c },
  {   3, &hf_rrc_e1d             , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   4, &hf_rrc_e1e             , ASN1_NO_EXTENSIONS     , dissect_rrc_Event1e },
  {   5, &hf_rrc_e1f             , ASN1_NO_EXTENSIONS     , dissect_rrc_Event1f },
  {   6, &hf_rrc_e1g             , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   7, &hf_rrc_e1h             , ASN1_NO_EXTENSIONS     , dissect_rrc_ThresholdUsedFrequency },
  {   8, &hf_rrc_e1i             , ASN1_NO_EXTENSIONS     , dissect_rrc_ThresholdUsedFrequency },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_IntraFreqEvent(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_IntraFreqEvent, IntraFreqEvent_choice,
                                 NULL);

  return offset;
}



static int
dissect_rrc_Hysteresis(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            0U, 15U, NULL, FALSE);

  return offset;
}


static const value_string rrc_TimeToTrigger_vals[] = {
  {   0, "ttt0" },
  {   1, "ttt10" },
  {   2, "ttt20" },
  {   3, "ttt40" },
  {   4, "ttt60" },
  {   5, "ttt80" },
  {   6, "ttt100" },
  {   7, "ttt120" },
  {   8, "ttt160" },
  {   9, "ttt200" },
  {  10, "ttt240" },
  {  11, "tt320" },
  {  12, "ttt640" },
  {  13, "ttt1280" },
  {  14, "ttt2560" },
  {  15, "ttt5000" },
  { 0, NULL }
};


static int
dissect_rrc_TimeToTrigger(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     16, NULL, FALSE, 0, NULL);

  return offset;
}


static const value_string rrc_MaxNumberOfReportingCellsType1_vals[] = {
  {   0, "e1" },
  {   1, "e2" },
  {   2, "e3" },
  {   3, "e4" },
  {   4, "e5" },
  {   5, "e6" },
  { 0, NULL }
};


static int
dissect_rrc_MaxNumberOfReportingCellsType1(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     6, NULL, FALSE, 0, NULL);

  return offset;
}


static const value_string rrc_MaxNumberOfReportingCellsType3_vals[] = {
  {   0, "viactCellsPlus1" },
  {   1, "viactCellsPlus2" },
  {   2, "viactCellsPlus3" },
  {   3, "viactCellsPlus4" },
  {   4, "viactCellsPlus5" },
  {   5, "viactCellsPlus6" },
  { 0, NULL }
};


static int
dissect_rrc_MaxNumberOfReportingCellsType3(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     6, NULL, FALSE, 0, NULL);

  return offset;
}


static const value_string rrc_MaxNumberOfReportingCellsType2_vals[] = {
  {   0, "e1" },
  {   1, "e2" },
  {   2, "e3" },
  {   3, "e4" },
  {   4, "e5" },
  {   5, "e6" },
  {   6, "e7" },
  {   7, "e8" },
  {   8, "e9" },
  {   9, "e10" },
  {  10, "e11" },
  {  11, "e12" },
  { 0, NULL }
};


static int
dissect_rrc_MaxNumberOfReportingCellsType2(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     12, NULL, FALSE, 0, NULL);

  return offset;
}


static const value_string rrc_ReportingCellStatus_vals[] = {
  {   0, "withinActiveSet" },
  {   1, "withinMonitoredSetUsedFreq" },
  {   2, "withinActiveAndOrMonitoredUsedFreq" },
  {   3, "withinDetectedSetUsedFreq" },
  {   4, "withinMonitoredAndOrDetectedUsedFreq" },
  {   5, "allActiveplusMonitoredSet" },
  {   6, "allActivePlusDetectedSet" },
  {   7, "allActivePlusMonitoredAndOrDetectedSet" },
  {   8, "withinVirtualActSet" },
  {   9, "withinMonitoredSetNonUsedFreq" },
  {  10, "withinMonitoredAndOrVirtualActiveSetNonUsedFreq" },
  {  11, "allVirtualActSetplusMonitoredSetNonUsedFreq" },
  {  12, "withinActSetOrVirtualActSet-InterRATcells" },
  {  13, "withinActSetAndOrMonitoredUsedFreqOrVirtualActSetAndOrMonitoredNonUsedFreq" },
  { 0, NULL }
};

static const per_choice_t ReportingCellStatus_choice[] = {
  {   0, &hf_rrc_withinActiveSet , ASN1_NO_EXTENSIONS     , dissect_rrc_MaxNumberOfReportingCellsType1 },
  {   1, &hf_rrc_withinMonitoredSetUsedFreq, ASN1_NO_EXTENSIONS     , dissect_rrc_MaxNumberOfReportingCellsType1 },
  {   2, &hf_rrc_withinActiveAndOrMonitoredUsedFreq, ASN1_NO_EXTENSIONS     , dissect_rrc_MaxNumberOfReportingCellsType1 },
  {   3, &hf_rrc_withinDetectedSetUsedFreq, ASN1_NO_EXTENSIONS     , dissect_rrc_MaxNumberOfReportingCellsType1 },
  {   4, &hf_rrc_withinMonitoredAndOrDetectedUsedFreq, ASN1_NO_EXTENSIONS     , dissect_rrc_MaxNumberOfReportingCellsType1 },
  {   5, &hf_rrc_allActiveplusMonitoredSet, ASN1_NO_EXTENSIONS     , dissect_rrc_MaxNumberOfReportingCellsType3 },
  {   6, &hf_rrc_allActivePlusDetectedSet, ASN1_NO_EXTENSIONS     , dissect_rrc_MaxNumberOfReportingCellsType3 },
  {   7, &hf_rrc_allActivePlusMonitoredAndOrDetectedSet, ASN1_NO_EXTENSIONS     , dissect_rrc_MaxNumberOfReportingCellsType3 },
  {   8, &hf_rrc_withinVirtualActSet, ASN1_NO_EXTENSIONS     , dissect_rrc_MaxNumberOfReportingCellsType1 },
  {   9, &hf_rrc_withinMonitoredSetNonUsedFreq, ASN1_NO_EXTENSIONS     , dissect_rrc_MaxNumberOfReportingCellsType1 },
  {  10, &hf_rrc_withinMonitoredAndOrVirtualActiveSetNonUsedFreq, ASN1_NO_EXTENSIONS     , dissect_rrc_MaxNumberOfReportingCellsType1 },
  {  11, &hf_rrc_allVirtualActSetplusMonitoredSetNonUsedFreq, ASN1_NO_EXTENSIONS     , dissect_rrc_MaxNumberOfReportingCellsType3 },
  {  12, &hf_rrc_withinActSetOrVirtualActSet_InterRATcells, ASN1_NO_EXTENSIONS     , dissect_rrc_MaxNumberOfReportingCellsType2 },
  {  13, &hf_rrc_withinActSetAndOrMonitoredUsedFreqOrVirtualActSetAndOrMonitoredNonUsedFreq, ASN1_NO_EXTENSIONS     , dissect_rrc_MaxNumberOfReportingCellsType2 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_ReportingCellStatus(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_ReportingCellStatus, ReportingCellStatus_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t IntraFreqEventCriteria_sequence[] = {
  { &hf_rrc_event           , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_IntraFreqEvent },
  { &hf_rrc_hysteresis_01   , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_Hysteresis },
  { &hf_rrc_timeToTrigger   , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TimeToTrigger },
  { &hf_rrc_reportingCellStatus, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_ReportingCellStatus },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_IntraFreqEventCriteria(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_IntraFreqEventCriteria, IntraFreqEventCriteria_sequence);

  return offset;
}


static const per_sequence_t IntraFreqEventCriteriaList_sequence_of[1] = {
  { &hf_rrc_IntraFreqEventCriteriaList_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_IntraFreqEventCriteria },
};

static int
dissect_rrc_IntraFreqEventCriteriaList(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_IntraFreqEventCriteriaList, IntraFreqEventCriteriaList_sequence_of,
                                                  1, maxMeasEvent, FALSE);

  return offset;
}


static const per_sequence_t IntraFreqReportingCriteria_sequence[] = {
  { &hf_rrc_eventCriteriaList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_IntraFreqEventCriteriaList },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_IntraFreqReportingCriteria(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_IntraFreqReportingCriteria, IntraFreqReportingCriteria_sequence);

  return offset;
}


static const value_string rrc_ReportingIntervalLong_vals[] = {
  {   0, "ril0" },
  {   1, "ril0-25" },
  {   2, "ril0-5" },
  {   3, "ril1" },
  {   4, "ril2" },
  {   5, "ril3" },
  {   6, "ril4" },
  {   7, "ril6" },
  {   8, "ril8" },
  {   9, "ril12" },
  {  10, "ril16" },
  {  11, "ril20" },
  {  12, "ril24" },
  {  13, "ril28" },
  {  14, "ril32" },
  {  15, "ril64" },
  { 0, NULL }
};


static int
dissect_rrc_ReportingIntervalLong(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     16, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t PeriodicalReportingCriteria_sequence[] = {
  { &hf_rrc_reportingAmount , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_ReportingAmount },
  { &hf_rrc_reportingInterval_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_ReportingIntervalLong },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_PeriodicalReportingCriteria(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_PeriodicalReportingCriteria, PeriodicalReportingCriteria_sequence);

  return offset;
}


static const per_sequence_t PeriodicalWithReportingCellStatus_sequence[] = {
  { &hf_rrc_periodicalReportingCriteria, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PeriodicalReportingCriteria },
  { &hf_rrc_reportingCellStatus, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_ReportingCellStatus },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_PeriodicalWithReportingCellStatus(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_PeriodicalWithReportingCellStatus, PeriodicalWithReportingCellStatus_sequence);

  return offset;
}


static const per_sequence_t ReportingCellStatusOpt_sequence[] = {
  { &hf_rrc_reportingCellStatus, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_ReportingCellStatus },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_ReportingCellStatusOpt(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_ReportingCellStatusOpt, ReportingCellStatusOpt_sequence);

  return offset;
}


static const value_string rrc_IntraFreqReportCriteria_vals[] = {
  {   0, "intraFreqReportingCriteria" },
  {   1, "periodicalReportingCriteria" },
  {   2, "noReporting" },
  { 0, NULL }
};

static const per_choice_t IntraFreqReportCriteria_choice[] = {
  {   0, &hf_rrc_intraFreqReportingCriteria, ASN1_NO_EXTENSIONS     , dissect_rrc_IntraFreqReportingCriteria },
  {   1, &hf_rrc_periodicalReportingCriteria_01, ASN1_NO_EXTENSIONS     , dissect_rrc_PeriodicalWithReportingCellStatus },
  {   2, &hf_rrc_noReporting     , ASN1_NO_EXTENSIONS     , dissect_rrc_ReportingCellStatusOpt },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_IntraFreqReportCriteria(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_IntraFreqReportCriteria, IntraFreqReportCriteria_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t IntraFrequencyMeasurement_sequence[] = {
  { &hf_rrc_intraFreqCellInfoList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_IntraFreqCellInfoList },
  { &hf_rrc_intraFreqMeasQuantity, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_IntraFreqMeasQuantity },
  { &hf_rrc_intraFreqReportingQuantity, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_IntraFreqReportingQuantity },
  { &hf_rrc_measurementValidity, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MeasurementValidity },
  { &hf_rrc_reportCriteria_09, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_IntraFreqReportCriteria },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_IntraFrequencyMeasurement(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_IntraFrequencyMeasurement, IntraFrequencyMeasurement_sequence);

  return offset;
}



static int
dissect_rrc_InterFreqCellID(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            0U, maxCellMeas_1, NULL, FALSE);

  return offset;
}


static const per_sequence_t SEQUENCE_SIZE_1_maxCellMeas_OF_InterFreqCellID_sequence_of[1] = {
  { &hf_rrc_removeSomeInterFreqCells_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_InterFreqCellID },
};

static int
dissect_rrc_SEQUENCE_SIZE_1_maxCellMeas_OF_InterFreqCellID(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_SEQUENCE_SIZE_1_maxCellMeas_OF_InterFreqCellID, SEQUENCE_SIZE_1_maxCellMeas_OF_InterFreqCellID_sequence_of,
                                                  1, maxCellMeas, FALSE);

  return offset;
}


static const value_string rrc_RemovedInterFreqCellList_vals[] = {
  {   0, "removeAllInterFreqCells" },
  {   1, "removeSomeInterFreqCells" },
  {   2, "removeNoInterFreqCells" },
  { 0, NULL }
};

static const per_choice_t RemovedInterFreqCellList_choice[] = {
  {   0, &hf_rrc_removeAllInterFreqCells, ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   1, &hf_rrc_removeSomeInterFreqCells, ASN1_NO_EXTENSIONS     , dissect_rrc_SEQUENCE_SIZE_1_maxCellMeas_OF_InterFreqCellID },
  {   2, &hf_rrc_removeNoInterFreqCells, ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_RemovedInterFreqCellList(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_RemovedInterFreqCellList, RemovedInterFreqCellList_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t NewInterFreqCell_sequence[] = {
  { &hf_rrc_interFreqCellID , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_InterFreqCellID },
  { &hf_rrc_frequencyInfo   , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_FrequencyInfo },
  { &hf_rrc_cellInfo        , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CellInfo },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_NewInterFreqCell(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_NewInterFreqCell, NewInterFreqCell_sequence);

  return offset;
}


static const per_sequence_t NewInterFreqCellList_sequence_of[1] = {
  { &hf_rrc_NewInterFreqCellList_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_NewInterFreqCell },
};

static int
dissect_rrc_NewInterFreqCellList(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_NewInterFreqCellList, NewInterFreqCellList_sequence_of,
                                                  1, maxCellMeas, FALSE);

  return offset;
}


static const per_sequence_t CellsForInterFreqMeasList_sequence_of[1] = {
  { &hf_rrc_CellsForInterFreqMeasList_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_InterFreqCellID },
};

static int
dissect_rrc_CellsForInterFreqMeasList(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_CellsForInterFreqMeasList, CellsForInterFreqMeasList_sequence_of,
                                                  1, maxCellMeas, FALSE);

  return offset;
}


static const per_sequence_t InterFreqCellInfoList_sequence[] = {
  { &hf_rrc_removedInterFreqCellList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RemovedInterFreqCellList },
  { &hf_rrc_newInterFreqCellList_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_NewInterFreqCellList },
  { &hf_rrc_cellsForInterFreqMeasList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CellsForInterFreqMeasList },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_InterFreqCellInfoList(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_InterFreqCellInfoList, InterFreqCellInfoList_sequence);

  return offset;
}


static const per_sequence_t T_intraFreqReportingCriteria_sequence[] = {
  { &hf_rrc_intraFreqMeasQuantity, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_IntraFreqMeasQuantity },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_intraFreqReportingCriteria(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_intraFreqReportingCriteria, T_intraFreqReportingCriteria_sequence);

  return offset;
}


static const value_string rrc_FreqQualityEstimateQuantity_FDD_vals[] = {
  {   0, "cpich-Ec-N0" },
  {   1, "cpich-RSCP" },
  { 0, NULL }
};


static int
dissect_rrc_FreqQualityEstimateQuantity_FDD(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     2, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t T_fdd_151_sequence[] = {
  { &hf_rrc_freqQualityEstimateQuantity_FDD, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_FreqQualityEstimateQuantity_FDD },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_fdd_151(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_fdd_151, T_fdd_151_sequence);

  return offset;
}


static const value_string rrc_FreqQualityEstimateQuantity_TDD_vals[] = {
  {   0, "primaryCCPCH-RSCP" },
  { 0, NULL }
};


static int
dissect_rrc_FreqQualityEstimateQuantity_TDD(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t T_tdd_100_sequence[] = {
  { &hf_rrc_freqQualityEstimateQuantity_TDD, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_FreqQualityEstimateQuantity_TDD },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_tdd_100(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_tdd_100, T_tdd_100_sequence);

  return offset;
}


static const value_string rrc_T_modeSpecificInfo_112_vals[] = {
  {   0, "fdd" },
  {   1, "tdd" },
  { 0, NULL }
};

static const per_choice_t T_modeSpecificInfo_112_choice[] = {
  {   0, &hf_rrc_fdd_157         , ASN1_NO_EXTENSIONS     , dissect_rrc_T_fdd_151 },
  {   1, &hf_rrc_tdd_108         , ASN1_NO_EXTENSIONS     , dissect_rrc_T_tdd_100 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_modeSpecificInfo_112(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_modeSpecificInfo_112, T_modeSpecificInfo_112_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t T_interFreqReportingCriteria_sequence[] = {
  { &hf_rrc_filterCoefficient, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_FilterCoefficient },
  { &hf_rrc_modeSpecificInfo_112, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_modeSpecificInfo_112 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_interFreqReportingCriteria(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_interFreqReportingCriteria, T_interFreqReportingCriteria_sequence);

  return offset;
}


static const value_string rrc_T_reportingCriteria_vals[] = {
  {   0, "intraFreqReportingCriteria" },
  {   1, "interFreqReportingCriteria" },
  { 0, NULL }
};

static const per_choice_t T_reportingCriteria_choice[] = {
  {   0, &hf_rrc_intraFreqReportingCriteria_02, ASN1_NO_EXTENSIONS     , dissect_rrc_T_intraFreqReportingCriteria },
  {   1, &hf_rrc_interFreqReportingCriteria, ASN1_NO_EXTENSIONS     , dissect_rrc_T_interFreqReportingCriteria },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_reportingCriteria(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_reportingCriteria, T_reportingCriteria_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t InterFreqMeasQuantity_sequence[] = {
  { &hf_rrc_reportingCriteria, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_reportingCriteria },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_InterFreqMeasQuantity(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_InterFreqMeasQuantity, InterFreqMeasQuantity_sequence);

  return offset;
}


static const per_sequence_t InterFreqReportingQuantity_sequence[] = {
  { &hf_rrc_utra_Carrier_RSSI, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { &hf_rrc_frequencyQualityEstimate, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { &hf_rrc_nonFreqRelatedQuantities, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CellReportingQuantities },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_InterFreqReportingQuantity(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_InterFreqReportingQuantity, InterFreqReportingQuantity_sequence);

  return offset;
}


static const per_sequence_t RL_AdditionInfoList_sequence_of[1] = {
  { &hf_rrc_RL_AdditionInfoList_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PrimaryCPICH_Info },
};

static int
dissect_rrc_RL_AdditionInfoList(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_RL_AdditionInfoList, RL_AdditionInfoList_sequence_of,
                                                  1, maxRL, FALSE);

  return offset;
}


static const per_sequence_t RL_InformationLists_sequence[] = {
  { &hf_rrc_rl_AdditionInfoList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RL_AdditionInfoList },
  { &hf_rrc_rL_RemovalInformationList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RL_RemovalInformationList },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_RL_InformationLists(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_RL_InformationLists, RL_InformationLists_sequence);

  return offset;
}


static const value_string rrc_UE_AutonomousUpdateMode_vals[] = {
  {   0, "dummy" },
  {   1, "onWithNoReporting" },
  {   2, "dummy2" },
  { 0, NULL }
};

static const per_choice_t UE_AutonomousUpdateMode_choice[] = {
  {   0, &hf_rrc_dummy_24        , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   1, &hf_rrc_onWithNoReporting, ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   2, &hf_rrc_dummy2_13       , ASN1_NO_EXTENSIONS     , dissect_rrc_RL_InformationLists },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_UE_AutonomousUpdateMode(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_UE_AutonomousUpdateMode, UE_AutonomousUpdateMode_choice,
                                 NULL);

  return offset;
}



static int
dissect_rrc_Threshold(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            -115, 0U, NULL, FALSE);

  return offset;
}



static int
dissect_rrc_HysteresisInterFreq(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            0U, 29U, NULL, FALSE);

  return offset;
}


static const per_sequence_t NonUsedFreqParameter_sequence[] = {
  { &hf_rrc_nonUsedFreqThreshold, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_Threshold },
  { &hf_rrc_nonUsedFreqW    , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_W },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_NonUsedFreqParameter(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_NonUsedFreqParameter, NonUsedFreqParameter_sequence);

  return offset;
}


static const per_sequence_t NonUsedFreqParameterList_sequence_of[1] = {
  { &hf_rrc_NonUsedFreqParameterList_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_NonUsedFreqParameter },
};

static int
dissect_rrc_NonUsedFreqParameterList(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_NonUsedFreqParameterList, NonUsedFreqParameterList_sequence_of,
                                                  1, maxFreq, FALSE);

  return offset;
}


static const per_sequence_t Event2a_sequence[] = {
  { &hf_rrc_dummy_50        , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_Threshold },
  { &hf_rrc_usedFreqW       , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_W },
  { &hf_rrc_hysteresis      , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_HysteresisInterFreq },
  { &hf_rrc_timeToTrigger   , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TimeToTrigger },
  { &hf_rrc_reportingCellStatus, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_ReportingCellStatus },
  { &hf_rrc_nonUsedFreqParameterList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_NonUsedFreqParameterList },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_Event2a(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_Event2a, Event2a_sequence);

  return offset;
}


static const per_sequence_t Event2b_sequence[] = {
  { &hf_rrc_usedFreqThreshold, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_Threshold },
  { &hf_rrc_usedFreqW       , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_W },
  { &hf_rrc_hysteresis      , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_HysteresisInterFreq },
  { &hf_rrc_timeToTrigger   , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TimeToTrigger },
  { &hf_rrc_reportingCellStatus, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_ReportingCellStatus },
  { &hf_rrc_nonUsedFreqParameterList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_NonUsedFreqParameterList },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_Event2b(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_Event2b, Event2b_sequence);

  return offset;
}


static const per_sequence_t Event2c_sequence[] = {
  { &hf_rrc_hysteresis      , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_HysteresisInterFreq },
  { &hf_rrc_timeToTrigger   , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TimeToTrigger },
  { &hf_rrc_reportingCellStatus, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_ReportingCellStatus },
  { &hf_rrc_nonUsedFreqParameterList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_NonUsedFreqParameterList },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_Event2c(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_Event2c, Event2c_sequence);

  return offset;
}


static const per_sequence_t Event2d_sequence[] = {
  { &hf_rrc_usedFreqThreshold, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_Threshold },
  { &hf_rrc_usedFreqW       , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_W },
  { &hf_rrc_hysteresis      , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_HysteresisInterFreq },
  { &hf_rrc_timeToTrigger   , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TimeToTrigger },
  { &hf_rrc_reportingCellStatus, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_ReportingCellStatus },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_Event2d(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_Event2d, Event2d_sequence);

  return offset;
}


static const per_sequence_t Event2e_sequence[] = {
  { &hf_rrc_hysteresis      , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_HysteresisInterFreq },
  { &hf_rrc_timeToTrigger   , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TimeToTrigger },
  { &hf_rrc_reportingCellStatus, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_ReportingCellStatus },
  { &hf_rrc_nonUsedFreqParameterList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_NonUsedFreqParameterList },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_Event2e(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_Event2e, Event2e_sequence);

  return offset;
}


static const per_sequence_t Event2f_sequence[] = {
  { &hf_rrc_usedFreqThreshold, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_Threshold },
  { &hf_rrc_usedFreqW       , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_W },
  { &hf_rrc_hysteresis      , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_HysteresisInterFreq },
  { &hf_rrc_timeToTrigger   , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TimeToTrigger },
  { &hf_rrc_reportingCellStatus, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_ReportingCellStatus },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_Event2f(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_Event2f, Event2f_sequence);

  return offset;
}


static const value_string rrc_InterFreqEvent_vals[] = {
  {   0, "event2a" },
  {   1, "event2b" },
  {   2, "event2c" },
  {   3, "event2d" },
  {   4, "event2e" },
  {   5, "event2f" },
  { 0, NULL }
};

static const per_choice_t InterFreqEvent_choice[] = {
  {   0, &hf_rrc_event2a         , ASN1_NO_EXTENSIONS     , dissect_rrc_Event2a },
  {   1, &hf_rrc_event2b         , ASN1_NO_EXTENSIONS     , dissect_rrc_Event2b },
  {   2, &hf_rrc_event2c         , ASN1_NO_EXTENSIONS     , dissect_rrc_Event2c },
  {   3, &hf_rrc_event2d         , ASN1_NO_EXTENSIONS     , dissect_rrc_Event2d },
  {   4, &hf_rrc_event2e         , ASN1_NO_EXTENSIONS     , dissect_rrc_Event2e },
  {   5, &hf_rrc_event2f         , ASN1_NO_EXTENSIONS     , dissect_rrc_Event2f },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_InterFreqEvent(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_InterFreqEvent, InterFreqEvent_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t InterFreqEventList_sequence_of[1] = {
  { &hf_rrc_InterFreqEventList_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_InterFreqEvent },
};

static int
dissect_rrc_InterFreqEventList(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_InterFreqEventList, InterFreqEventList_sequence_of,
                                                  1, maxMeasEvent, FALSE);

  return offset;
}


static const per_sequence_t InterFreqReportingCriteria_sequence[] = {
  { &hf_rrc_interFreqEventList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_InterFreqEventList },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_InterFreqReportingCriteria(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_InterFreqReportingCriteria, InterFreqReportingCriteria_sequence);

  return offset;
}


static const value_string rrc_InterFreqReportCriteria_vals[] = {
  {   0, "intraFreqReportingCriteria" },
  {   1, "interFreqReportingCriteria" },
  {   2, "periodicalReportingCriteria" },
  {   3, "noReporting" },
  { 0, NULL }
};

static const per_choice_t InterFreqReportCriteria_choice[] = {
  {   0, &hf_rrc_intraFreqReportingCriteria, ASN1_NO_EXTENSIONS     , dissect_rrc_IntraFreqReportingCriteria },
  {   1, &hf_rrc_interFreqReportingCriteria_01, ASN1_NO_EXTENSIONS     , dissect_rrc_InterFreqReportingCriteria },
  {   2, &hf_rrc_periodicalReportingCriteria_01, ASN1_NO_EXTENSIONS     , dissect_rrc_PeriodicalWithReportingCellStatus },
  {   3, &hf_rrc_noReporting     , ASN1_NO_EXTENSIONS     , dissect_rrc_ReportingCellStatusOpt },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_InterFreqReportCriteria(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_InterFreqReportCriteria, InterFreqReportCriteria_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t InterFrequencyMeasurement_sequence[] = {
  { &hf_rrc_interFreqCellInfoList, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_InterFreqCellInfoList },
  { &hf_rrc_interFreqMeasQuantity, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_InterFreqMeasQuantity },
  { &hf_rrc_interFreqReportingQuantity, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_InterFreqReportingQuantity },
  { &hf_rrc_measurementValidity, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MeasurementValidity },
  { &hf_rrc_interFreqSetUpdate, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_AutonomousUpdateMode },
  { &hf_rrc_reportCriteria_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_InterFreqReportCriteria },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_InterFrequencyMeasurement(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_InterFrequencyMeasurement, InterFrequencyMeasurement_sequence);

  return offset;
}



static int
dissect_rrc_InterRATCellID(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            0U, maxCellMeas_1, NULL, FALSE);

  return offset;
}


static const per_sequence_t SEQUENCE_SIZE_1_maxCellMeas_OF_InterRATCellID_sequence_of[1] = {
  { &hf_rrc_removeSomeInterRATCells_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_InterRATCellID },
};

static int
dissect_rrc_SEQUENCE_SIZE_1_maxCellMeas_OF_InterRATCellID(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_SEQUENCE_SIZE_1_maxCellMeas_OF_InterRATCellID, SEQUENCE_SIZE_1_maxCellMeas_OF_InterRATCellID_sequence_of,
                                                  1, maxCellMeas, FALSE);

  return offset;
}


static const value_string rrc_RemovedInterRATCellList_vals[] = {
  {   0, "removeAllInterRATCells" },
  {   1, "removeSomeInterRATCells" },
  {   2, "removeNoInterRATCells" },
  { 0, NULL }
};

static const per_choice_t RemovedInterRATCellList_choice[] = {
  {   0, &hf_rrc_removeAllInterRATCells, ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   1, &hf_rrc_removeSomeInterRATCells, ASN1_NO_EXTENSIONS     , dissect_rrc_SEQUENCE_SIZE_1_maxCellMeas_OF_InterRATCellID },
  {   2, &hf_rrc_removeNoInterRATCells, ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_RemovedInterRATCellList(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_RemovedInterRATCellList, RemovedInterRATCellList_choice,
                                 NULL);

  return offset;
}



static int
dissect_rrc_Q_OffsetS_N(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            -50, 50U, NULL, FALSE);

  return offset;
}



static int
dissect_rrc_HCS_PRIO(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            0U, 7U, NULL, FALSE);

  return offset;
}



static int
dissect_rrc_Q_HCS(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            0U, 99U, NULL, FALSE);

  return offset;
}


static const value_string rrc_TemporaryOffset1_vals[] = {
  {   0, "to3" },
  {   1, "to6" },
  {   2, "to9" },
  {   3, "to12" },
  {   4, "to15" },
  {   5, "to18" },
  {   6, "to21" },
  {   7, "infinite" },
  { 0, NULL }
};


static int
dissect_rrc_TemporaryOffset1(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     8, NULL, FALSE, 0, NULL);

  return offset;
}


static const value_string rrc_PenaltyTime_RSCP_vals[] = {
  {   0, "notUsed" },
  {   1, "pt10" },
  {   2, "pt20" },
  {   3, "pt30" },
  {   4, "pt40" },
  {   5, "pt50" },
  {   6, "pt60" },
  { 0, NULL }
};

static const per_choice_t PenaltyTime_RSCP_choice[] = {
  {   0, &hf_rrc_notUsed         , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   1, &hf_rrc_pt10            , ASN1_NO_EXTENSIONS     , dissect_rrc_TemporaryOffset1 },
  {   2, &hf_rrc_pt20            , ASN1_NO_EXTENSIONS     , dissect_rrc_TemporaryOffset1 },
  {   3, &hf_rrc_pt30            , ASN1_NO_EXTENSIONS     , dissect_rrc_TemporaryOffset1 },
  {   4, &hf_rrc_pt40            , ASN1_NO_EXTENSIONS     , dissect_rrc_TemporaryOffset1 },
  {   5, &hf_rrc_pt50            , ASN1_NO_EXTENSIONS     , dissect_rrc_TemporaryOffset1 },
  {   6, &hf_rrc_pt60            , ASN1_NO_EXTENSIONS     , dissect_rrc_TemporaryOffset1 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_PenaltyTime_RSCP(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_PenaltyTime_RSCP, PenaltyTime_RSCP_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t HCS_CellReselectInformation_RSCP_sequence[] = {
  { &hf_rrc_penaltyTime     , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PenaltyTime_RSCP },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_HCS_CellReselectInformation_RSCP(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_HCS_CellReselectInformation_RSCP, HCS_CellReselectInformation_RSCP_sequence);

  return offset;
}


static const per_sequence_t HCS_NeighbouringCellInformation_RSCP_sequence[] = {
  { &hf_rrc_hcs_PRIO        , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_HCS_PRIO },
  { &hf_rrc_q_HCS           , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_Q_HCS },
  { &hf_rrc_hcs_CellReselectInformation, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_HCS_CellReselectInformation_RSCP },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_HCS_NeighbouringCellInformation_RSCP(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_HCS_NeighbouringCellInformation_RSCP, HCS_NeighbouringCellInformation_RSCP_sequence);

  return offset;
}



static int
dissect_rrc_Q_QualMin(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            -24, 0U, NULL, FALSE);

  return offset;
}



static int
dissect_rrc_Q_RxlevMin(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            -58, -13, NULL, FALSE);

  return offset;
}


static const per_sequence_t T_fdd_143_sequence[] = {
  { &hf_rrc_q_QualMin       , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_Q_QualMin },
  { &hf_rrc_q_RxlevMin      , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_Q_RxlevMin },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_fdd_143(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_fdd_143, T_fdd_143_sequence);

  return offset;
}


static const per_sequence_t T_tdd_94_sequence[] = {
  { &hf_rrc_q_RxlevMin      , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_Q_RxlevMin },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_tdd_94(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_tdd_94, T_tdd_94_sequence);

  return offset;
}


static const per_sequence_t T_gsm_03_sequence[] = {
  { &hf_rrc_q_RxlevMin      , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_Q_RxlevMin },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_gsm_03(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_gsm_03, T_gsm_03_sequence);

  return offset;
}


static const value_string rrc_T_modeSpecificInfo_104_vals[] = {
  {   0, "fdd" },
  {   1, "tdd" },
  {   2, "gsm" },
  { 0, NULL }
};

static const per_choice_t T_modeSpecificInfo_104_choice[] = {
  {   0, &hf_rrc_fdd_148         , ASN1_NO_EXTENSIONS     , dissect_rrc_T_fdd_143 },
  {   1, &hf_rrc_tdd_101         , ASN1_NO_EXTENSIONS     , dissect_rrc_T_tdd_94 },
  {   2, &hf_rrc_gsm_03          , ASN1_NO_EXTENSIONS     , dissect_rrc_T_gsm_03 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_modeSpecificInfo_104(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_modeSpecificInfo_104, T_modeSpecificInfo_104_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t CellSelectReselectInfoSIB_11_12_sequence[] = {
  { &hf_rrc_q_Offset1S_N    , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_Q_OffsetS_N },
  { &hf_rrc_q_Offset2S_N    , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_Q_OffsetS_N },
  { &hf_rrc_maxAllowedUL_TX_Power, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MaxAllowedUL_TX_Power },
  { &hf_rrc_hcs_NeighbouringCellInformation_RSCP, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_HCS_NeighbouringCellInformation_RSCP },
  { &hf_rrc_modeSpecificInfo_104, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_modeSpecificInfo_104 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_CellSelectReselectInfoSIB_11_12(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_CellSelectReselectInfoSIB_11_12, CellSelectReselectInfoSIB_11_12_sequence);

  return offset;
}



static int
dissect_rrc_InterRATCellIndividualOffset(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            -50, 50U, NULL, FALSE);

  return offset;
}


static const per_sequence_t T_gsm_13_sequence[] = {
  { &hf_rrc_cellSelectionReselectionInfo_05, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CellSelectReselectInfoSIB_11_12 },
  { &hf_rrc_interRATCellIndividualOffset, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_InterRATCellIndividualOffset },
  { &hf_rrc_bsic            , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BSIC },
  { &hf_rrc_frequency_band  , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_Frequency_Band },
  { &hf_rrc_bcch_ARFCN      , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BCCH_ARFCN },
  { &hf_rrc_dummy_24        , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_NULL },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_gsm_13(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_gsm_13, T_gsm_13_sequence);

  return offset;
}


static const value_string rrc_IS_2000SpecificMeasInfo_vals[] = {
  {   0, "frequency" },
  {   1, "timeslot" },
  {   2, "colourcode" },
  {   3, "outputpower" },
  {   4, "pn-Offset" },
  { 0, NULL }
};


static int
dissect_rrc_IS_2000SpecificMeasInfo(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     5, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t T_is_2000_02_sequence[] = {
  { &hf_rrc_is_2000SpecificMeasInfo, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_IS_2000SpecificMeasInfo },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_is_2000_02(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_is_2000_02, T_is_2000_02_sequence);

  return offset;
}


static const value_string rrc_T_technologySpecificInfo_01_vals[] = {
  {   0, "gsm" },
  {   1, "is-2000" },
  {   2, "absent" },
  {   3, "spare1" },
  { 0, NULL }
};

static const per_choice_t T_technologySpecificInfo_01_choice[] = {
  {   0, &hf_rrc_gsm_15          , ASN1_NO_EXTENSIONS     , dissect_rrc_T_gsm_13 },
  {   1, &hf_rrc_is_2000_03      , ASN1_NO_EXTENSIONS     , dissect_rrc_T_is_2000_02 },
  {   2, &hf_rrc_absent          , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   3, &hf_rrc_spare1          , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_technologySpecificInfo_01(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_technologySpecificInfo_01, T_technologySpecificInfo_01_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t NewInterRATCell_sequence[] = {
  { &hf_rrc_interRATCellID  , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_InterRATCellID },
  { &hf_rrc_technologySpecificInfo_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_technologySpecificInfo_01 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_NewInterRATCell(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_NewInterRATCell, NewInterRATCell_sequence);

  return offset;
}


static const per_sequence_t NewInterRATCellList_sequence_of[1] = {
  { &hf_rrc_NewInterRATCellList_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_NewInterRATCell },
};

static int
dissect_rrc_NewInterRATCellList(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_NewInterRATCellList, NewInterRATCellList_sequence_of,
                                                  1, maxCellMeas, FALSE);

  return offset;
}


static const per_sequence_t CellsForInterRATMeasList_sequence_of[1] = {
  { &hf_rrc_CellsForInterRATMeasList_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_InterRATCellID },
};

static int
dissect_rrc_CellsForInterRATMeasList(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_CellsForInterRATMeasList, CellsForInterRATMeasList_sequence_of,
                                                  1, maxCellMeas, FALSE);

  return offset;
}


static const per_sequence_t InterRATCellInfoList_sequence[] = {
  { &hf_rrc_removedInterRATCellList, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RemovedInterRATCellList },
  { &hf_rrc_newInterRATCellList, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_NewInterRATCellList },
  { &hf_rrc_cellsForInterRATMeasList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CellsForInterRATMeasList },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_InterRATCellInfoList(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_InterRATCellInfoList, InterRATCellInfoList_sequence);

  return offset;
}


static const value_string rrc_MeasurementQuantityGSM_vals[] = {
  {   0, "gsm-CarrierRSSI" },
  {   1, "dummy" },
  { 0, NULL }
};


static int
dissect_rrc_MeasurementQuantityGSM(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     2, NULL, FALSE, 0, NULL);

  return offset;
}


static const value_string rrc_BSIC_VerificationRequired_vals[] = {
  {   0, "required" },
  {   1, "notRequired" },
  { 0, NULL }
};


static int
dissect_rrc_BSIC_VerificationRequired(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     2, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t T_gsm_09_sequence[] = {
  { &hf_rrc_measurementQuantity_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MeasurementQuantityGSM },
  { &hf_rrc_filterCoefficient, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_FilterCoefficient },
  { &hf_rrc_bsic_VerificationRequired, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BSIC_VerificationRequired },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_gsm_09(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_gsm_09, T_gsm_09_sequence);

  return offset;
}


static const per_sequence_t T_is_2000_sequence[] = {
  { &hf_rrc_tadd_EcIo       , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_0_63 },
  { &hf_rrc_tcomp_EcIo      , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_0_15 },
  { &hf_rrc_softSlope       , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_INTEGER_0_63 },
  { &hf_rrc_addIntercept    , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_INTEGER_0_63 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_is_2000(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_is_2000, T_is_2000_sequence);

  return offset;
}


static const value_string rrc_T_ratSpecificInfo_vals[] = {
  {   0, "gsm" },
  {   1, "is-2000" },
  { 0, NULL }
};

static const per_choice_t T_ratSpecificInfo_choice[] = {
  {   0, &hf_rrc_gsm_10          , ASN1_NO_EXTENSIONS     , dissect_rrc_T_gsm_09 },
  {   1, &hf_rrc_is_2000_01      , ASN1_NO_EXTENSIONS     , dissect_rrc_T_is_2000 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_ratSpecificInfo(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_ratSpecificInfo, T_ratSpecificInfo_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t InterRATMeasQuantity_sequence[] = {
  { &hf_rrc_measQuantityUTRAN_QualityEstimate, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_IntraFreqMeasQuantity },
  { &hf_rrc_ratSpecificInfo , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_ratSpecificInfo },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_InterRATMeasQuantity(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_InterRATMeasQuantity, InterRATMeasQuantity_sequence);

  return offset;
}


static const per_sequence_t T_gsm_11_sequence[] = {
  { &hf_rrc_dummy_28        , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { &hf_rrc_dummy2_06       , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { &hf_rrc_gsm_Carrier_RSSI, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_gsm_11(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_gsm_11, T_gsm_11_sequence);

  return offset;
}


static const value_string rrc_T_ratSpecificInfo_02_vals[] = {
  {   0, "gsm" },
  { 0, NULL }
};

static const per_choice_t T_ratSpecificInfo_02_choice[] = {
  {   0, &hf_rrc_gsm_13          , ASN1_NO_EXTENSIONS     , dissect_rrc_T_gsm_11 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_ratSpecificInfo_02(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_ratSpecificInfo_02, T_ratSpecificInfo_02_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t InterRATReportingQuantity_sequence[] = {
  { &hf_rrc_utran_EstimatedQuality, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { &hf_rrc_ratSpecificInfo_02, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_ratSpecificInfo_02 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_InterRATReportingQuantity(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_InterRATReportingQuantity, InterRATReportingQuantity_sequence);

  return offset;
}


static const per_sequence_t Event3a_sequence[] = {
  { &hf_rrc_thresholdOwnSystem, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_Threshold },
  { &hf_rrc_w               , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_W },
  { &hf_rrc_thresholdOtherSystem, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_Threshold },
  { &hf_rrc_hysteresis_01   , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_Hysteresis },
  { &hf_rrc_timeToTrigger   , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TimeToTrigger },
  { &hf_rrc_reportingCellStatus, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_ReportingCellStatus },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_Event3a(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_Event3a, Event3a_sequence);

  return offset;
}


static const per_sequence_t Event3b_sequence[] = {
  { &hf_rrc_thresholdOtherSystem, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_Threshold },
  { &hf_rrc_hysteresis_01   , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_Hysteresis },
  { &hf_rrc_timeToTrigger   , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TimeToTrigger },
  { &hf_rrc_reportingCellStatus, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_ReportingCellStatus },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_Event3b(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_Event3b, Event3b_sequence);

  return offset;
}


static const per_sequence_t Event3c_sequence[] = {
  { &hf_rrc_thresholdOtherSystem, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_Threshold },
  { &hf_rrc_hysteresis_01   , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_Hysteresis },
  { &hf_rrc_timeToTrigger   , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TimeToTrigger },
  { &hf_rrc_reportingCellStatus, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_ReportingCellStatus },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_Event3c(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_Event3c, Event3c_sequence);

  return offset;
}


static const per_sequence_t Event3d_sequence[] = {
  { &hf_rrc_hysteresis_01   , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_Hysteresis },
  { &hf_rrc_timeToTrigger   , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TimeToTrigger },
  { &hf_rrc_reportingCellStatus, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_ReportingCellStatus },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_Event3d(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_Event3d, Event3d_sequence);

  return offset;
}


static const value_string rrc_InterRATEvent_vals[] = {
  {   0, "event3a" },
  {   1, "event3b" },
  {   2, "event3c" },
  {   3, "event3d" },
  { 0, NULL }
};

static const per_choice_t InterRATEvent_choice[] = {
  {   0, &hf_rrc_event3a         , ASN1_NO_EXTENSIONS     , dissect_rrc_Event3a },
  {   1, &hf_rrc_event3b         , ASN1_NO_EXTENSIONS     , dissect_rrc_Event3b },
  {   2, &hf_rrc_event3c         , ASN1_NO_EXTENSIONS     , dissect_rrc_Event3c },
  {   3, &hf_rrc_event3d         , ASN1_NO_EXTENSIONS     , dissect_rrc_Event3d },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_InterRATEvent(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_InterRATEvent, InterRATEvent_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t InterRATEventList_sequence_of[1] = {
  { &hf_rrc_InterRATEventList_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_InterRATEvent },
};

static int
dissect_rrc_InterRATEventList(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_InterRATEventList, InterRATEventList_sequence_of,
                                                  1, maxMeasEvent, FALSE);

  return offset;
}


static const per_sequence_t InterRATReportingCriteria_sequence[] = {
  { &hf_rrc_interRATEventList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_InterRATEventList },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_InterRATReportingCriteria(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_InterRATReportingCriteria, InterRATReportingCriteria_sequence);

  return offset;
}


static const value_string rrc_InterRATReportCriteria_vals[] = {
  {   0, "interRATReportingCriteria" },
  {   1, "periodicalReportingCriteria" },
  {   2, "noReporting" },
  { 0, NULL }
};

static const per_choice_t InterRATReportCriteria_choice[] = {
  {   0, &hf_rrc_interRATReportingCriteria, ASN1_NO_EXTENSIONS     , dissect_rrc_InterRATReportingCriteria },
  {   1, &hf_rrc_periodicalReportingCriteria_01, ASN1_NO_EXTENSIONS     , dissect_rrc_PeriodicalWithReportingCellStatus },
  {   2, &hf_rrc_noReporting     , ASN1_NO_EXTENSIONS     , dissect_rrc_ReportingCellStatusOpt },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_InterRATReportCriteria(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_InterRATReportCriteria, InterRATReportCriteria_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t InterRATMeasurement_sequence[] = {
  { &hf_rrc_interRATCellInfoList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_InterRATCellInfoList },
  { &hf_rrc_interRATMeasQuantity, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_InterRATMeasQuantity },
  { &hf_rrc_interRATReportingQuantity, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_InterRATReportingQuantity },
  { &hf_rrc_reportCriteria_08, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_InterRATReportCriteria },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_InterRATMeasurement(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_InterRATMeasurement, InterRATMeasurement_sequence);

  return offset;
}


static const value_string rrc_UE_Positioning_MethodType_vals[] = {
  {   0, "ue-Assisted" },
  {   1, "ue-Based" },
  {   2, "ue-BasedPreferred" },
  {   3, "ue-AssistedPreferred" },
  { 0, NULL }
};


static int
dissect_rrc_UE_Positioning_MethodType(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     4, NULL, FALSE, 0, NULL);

  return offset;
}


static const value_string rrc_PositioningMethod_vals[] = {
  {   0, "otdoa" },
  {   1, "gps" },
  {   2, "otdoaOrGPS" },
  {   3, "cellID" },
  { 0, NULL }
};


static int
dissect_rrc_PositioningMethod(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     4, NULL, FALSE, 0, NULL);

  return offset;
}


static const value_string rrc_UE_Positioning_ResponseTime_vals[] = {
  {   0, "s1" },
  {   1, "s2" },
  {   2, "s4" },
  {   3, "s8" },
  {   4, "s16" },
  {   5, "s32" },
  {   6, "s64" },
  {   7, "s128" },
  { 0, NULL }
};


static int
dissect_rrc_UE_Positioning_ResponseTime(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     8, NULL, FALSE, 0, NULL);

  return offset;
}



static int
dissect_rrc_UE_Positioning_Accuracy(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_bit_string(tvb, offset, actx, tree, hf_index,
                                     7, 7, FALSE, NULL);

  return offset;
}


static const value_string rrc_EnvironmentCharacterisation_vals[] = {
  {   0, "possibleHeavyMultipathNLOS" },
  {   1, "lightMultipathLOS" },
  {   2, "notDefined" },
  {   3, "spare" },
  { 0, NULL }
};


static int
dissect_rrc_EnvironmentCharacterisation(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     4, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t UE_Positioning_ReportingQuantity_sequence[] = {
  { &hf_rrc_methodType      , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UE_Positioning_MethodType },
  { &hf_rrc_positioningMethod, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PositioningMethod },
  { &hf_rrc_dummy1_06       , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UE_Positioning_ResponseTime },
  { &hf_rrc_horizontal_Accuracy, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_Positioning_Accuracy },
  { &hf_rrc_gps_TimingOfCellWanted, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { &hf_rrc_dummy2_06       , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { &hf_rrc_additionalAssistanceDataRequest, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { &hf_rrc_environmentCharacterisation, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_EnvironmentCharacterisation },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UE_Positioning_ReportingQuantity(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UE_Positioning_ReportingQuantity, UE_Positioning_ReportingQuantity_sequence);

  return offset;
}


static const value_string rrc_UE_Positioning_MeasurementInterval_vals[] = {
  {   0, "e5" },
  {   1, "e15" },
  {   2, "e60" },
  {   3, "e300" },
  {   4, "e900" },
  {   5, "e1800" },
  {   6, "e3600" },
  {   7, "e7200" },
  { 0, NULL }
};


static int
dissect_rrc_UE_Positioning_MeasurementInterval(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     8, NULL, FALSE, 0, NULL);

  return offset;
}


static const value_string rrc_ThresholdPositionChange_vals[] = {
  {   0, "pc10" },
  {   1, "pc20" },
  {   2, "pc30" },
  {   3, "pc40" },
  {   4, "pc50" },
  {   5, "pc100" },
  {   6, "pc200" },
  {   7, "pc300" },
  {   8, "pc500" },
  {   9, "pc1000" },
  {  10, "pc2000" },
  {  11, "pc5000" },
  {  12, "pc10000" },
  {  13, "pc20000" },
  {  14, "pc50000" },
  {  15, "pc100000" },
  { 0, NULL }
};


static int
dissect_rrc_ThresholdPositionChange(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     16, NULL, FALSE, 0, NULL);

  return offset;
}


static const value_string rrc_ThresholdSFN_SFN_Change_vals[] = {
  {   0, "c0-25" },
  {   1, "c0-5" },
  {   2, "c1" },
  {   3, "c2" },
  {   4, "c3" },
  {   5, "c4" },
  {   6, "c5" },
  {   7, "c10" },
  {   8, "c20" },
  {   9, "c50" },
  {  10, "c100" },
  {  11, "c200" },
  {  12, "c500" },
  {  13, "c1000" },
  {  14, "c2000" },
  {  15, "c5000" },
  { 0, NULL }
};


static int
dissect_rrc_ThresholdSFN_SFN_Change(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     16, NULL, FALSE, 0, NULL);

  return offset;
}


static const value_string rrc_ThresholdSFN_GPS_TOW_vals[] = {
  {   0, "ms1" },
  {   1, "ms2" },
  {   2, "ms3" },
  {   3, "ms5" },
  {   4, "ms10" },
  {   5, "ms20" },
  {   6, "ms50" },
  {   7, "ms100" },
  { 0, NULL }
};


static int
dissect_rrc_ThresholdSFN_GPS_TOW(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     8, NULL, FALSE, 0, NULL);

  return offset;
}


static const value_string rrc_UE_Positioning_EventSpecificInfo_vals[] = {
  {   0, "e7a" },
  {   1, "e7b" },
  {   2, "e7c" },
  { 0, NULL }
};

static const per_choice_t UE_Positioning_EventSpecificInfo_choice[] = {
  {   0, &hf_rrc_e7a             , ASN1_NO_EXTENSIONS     , dissect_rrc_ThresholdPositionChange },
  {   1, &hf_rrc_e7b             , ASN1_NO_EXTENSIONS     , dissect_rrc_ThresholdSFN_SFN_Change },
  {   2, &hf_rrc_e7c             , ASN1_NO_EXTENSIONS     , dissect_rrc_ThresholdSFN_GPS_TOW },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_UE_Positioning_EventSpecificInfo(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_UE_Positioning_EventSpecificInfo, UE_Positioning_EventSpecificInfo_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t UE_Positioning_EventParam_sequence[] = {
  { &hf_rrc_reportingAmount , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_ReportingAmount },
  { &hf_rrc_reportFirstFix  , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { &hf_rrc_measurementInterval, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UE_Positioning_MeasurementInterval },
  { &hf_rrc_eventSpecificInfo, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UE_Positioning_EventSpecificInfo },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UE_Positioning_EventParam(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UE_Positioning_EventParam, UE_Positioning_EventParam_sequence);

  return offset;
}


static const per_sequence_t UE_Positioning_EventParamList_sequence_of[1] = {
  { &hf_rrc_UE_Positioning_EventParamList_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UE_Positioning_EventParam },
};

static int
dissect_rrc_UE_Positioning_EventParamList(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_UE_Positioning_EventParamList, UE_Positioning_EventParamList_sequence_of,
                                                  1, maxMeasEvent, FALSE);

  return offset;
}


static const value_string rrc_UE_Positioning_ReportCriteria_vals[] = {
  {   0, "ue-positioning-ReportingCriteria" },
  {   1, "periodicalReportingCriteria" },
  {   2, "noReporting" },
  { 0, NULL }
};

static const per_choice_t UE_Positioning_ReportCriteria_choice[] = {
  {   0, &hf_rrc_ue_positioning_ReportingCriteria, ASN1_NO_EXTENSIONS     , dissect_rrc_UE_Positioning_EventParamList },
  {   1, &hf_rrc_periodicalReportingCriteria, ASN1_NO_EXTENSIONS     , dissect_rrc_PeriodicalReportingCriteria },
  {   2, &hf_rrc_noReporting_02  , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_UE_Positioning_ReportCriteria(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_UE_Positioning_ReportCriteria, UE_Positioning_ReportCriteria_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t T_fdd_176_sequence[] = {
  { &hf_rrc_primaryCPICH_Info, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PrimaryCPICH_Info },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_fdd_176(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_fdd_176, T_fdd_176_sequence);

  return offset;
}


static const per_sequence_t T_tdd_125_sequence[] = {
  { &hf_rrc_cellAndChannelIdentity, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CellAndChannelIdentity },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_tdd_125(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_tdd_125, T_tdd_125_sequence);

  return offset;
}


static const value_string rrc_T_modeSpecificInfo_142_vals[] = {
  {   0, "fdd" },
  {   1, "tdd" },
  { 0, NULL }
};

static const per_choice_t T_modeSpecificInfo_142_choice[] = {
  {   0, &hf_rrc_fdd_182         , ASN1_NO_EXTENSIONS     , dissect_rrc_T_fdd_176 },
  {   1, &hf_rrc_tdd_133         , ASN1_NO_EXTENSIONS     , dissect_rrc_T_tdd_125 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_modeSpecificInfo_142(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_modeSpecificInfo_142, T_modeSpecificInfo_142_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t T_ueBased_03_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_ueBased_03(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_ueBased_03, T_ueBased_03_sequence);

  return offset;
}


static const per_sequence_t T_ueAssisted_03_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_ueAssisted_03(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_ueAssisted_03, T_ueAssisted_03_sequence);

  return offset;
}


static const value_string rrc_T_positioningMode_03_vals[] = {
  {   0, "ueBased" },
  {   1, "ueAssisted" },
  { 0, NULL }
};

static const per_choice_t T_positioningMode_03_choice[] = {
  {   0, &hf_rrc_ueBased_03      , ASN1_NO_EXTENSIONS     , dissect_rrc_T_ueBased_03 },
  {   1, &hf_rrc_ueAssisted_03   , ASN1_NO_EXTENSIONS     , dissect_rrc_T_ueAssisted_03 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_positioningMode_03(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_positioningMode_03, T_positioningMode_03_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t UE_Positioning_OTDOA_ReferenceCellInfo_sequence[] = {
  { &hf_rrc_sfn             , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_INTEGER_0_4095 },
  { &hf_rrc_modeSpecificInfo_142, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_modeSpecificInfo_142 },
  { &hf_rrc_frequencyInfo   , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_FrequencyInfo },
  { &hf_rrc_positioningMode_03, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_positioningMode_03 },
  { &hf_rrc_ue_positioning_IPDL_Paremeters, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_Positioning_IPDL_Parameters },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UE_Positioning_OTDOA_ReferenceCellInfo(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UE_Positioning_OTDOA_ReferenceCellInfo, UE_Positioning_OTDOA_ReferenceCellInfo_sequence);

  return offset;
}


static const per_sequence_t T_fdd_172_sequence[] = {
  { &hf_rrc_primaryCPICH_Info, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PrimaryCPICH_Info },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_fdd_172(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_fdd_172, T_fdd_172_sequence);

  return offset;
}


static const per_sequence_t T_tdd_121_sequence[] = {
  { &hf_rrc_cellAndChannelIdentity, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CellAndChannelIdentity },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_tdd_121(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_tdd_121, T_tdd_121_sequence);

  return offset;
}


static const value_string rrc_T_modeSpecificInfo_138_vals[] = {
  {   0, "fdd" },
  {   1, "tdd" },
  { 0, NULL }
};

static const per_choice_t T_modeSpecificInfo_138_choice[] = {
  {   0, &hf_rrc_fdd_178         , ASN1_NO_EXTENSIONS     , dissect_rrc_T_fdd_172 },
  {   1, &hf_rrc_tdd_129         , ASN1_NO_EXTENSIONS     , dissect_rrc_T_tdd_121 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_modeSpecificInfo_138(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_modeSpecificInfo_138, T_modeSpecificInfo_138_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t T_ueBased_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_ueBased(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_ueBased, T_ueBased_sequence);

  return offset;
}


static const per_sequence_t T_ueAssisted_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_ueAssisted(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_ueAssisted, T_ueAssisted_sequence);

  return offset;
}


static const value_string rrc_T_positioningMode_vals[] = {
  {   0, "ueBased" },
  {   1, "ueAssisted" },
  { 0, NULL }
};

static const per_choice_t T_positioningMode_choice[] = {
  {   0, &hf_rrc_ueBased         , ASN1_NO_EXTENSIONS     , dissect_rrc_T_ueBased },
  {   1, &hf_rrc_ueAssisted      , ASN1_NO_EXTENSIONS     , dissect_rrc_T_ueAssisted },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_positioningMode(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_positioningMode, T_positioningMode_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t UE_Positioning_OTDOA_NeighbourCellInfo_sequence[] = {
  { &hf_rrc_modeSpecificInfo_138, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_modeSpecificInfo_138 },
  { &hf_rrc_frequencyInfo   , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_FrequencyInfo },
  { &hf_rrc_ue_positioning_IPDL_Paremeters, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_Positioning_IPDL_Parameters },
  { &hf_rrc_sfn_SFN_RelTimeDifference, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SFN_SFN_RelTimeDifference1 },
  { &hf_rrc_sfn_SFN_Drift   , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_SFN_SFN_Drift },
  { &hf_rrc_searchWindowSize, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_OTDOA_SearchWindowSize },
  { &hf_rrc_positioningMode , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_positioningMode },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UE_Positioning_OTDOA_NeighbourCellInfo(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UE_Positioning_OTDOA_NeighbourCellInfo, UE_Positioning_OTDOA_NeighbourCellInfo_sequence);

  return offset;
}


static const per_sequence_t UE_Positioning_OTDOA_NeighbourCellList_sequence_of[1] = {
  { &hf_rrc_UE_Positioning_OTDOA_NeighbourCellList_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UE_Positioning_OTDOA_NeighbourCellInfo },
};

static int
dissect_rrc_UE_Positioning_OTDOA_NeighbourCellList(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_UE_Positioning_OTDOA_NeighbourCellList, UE_Positioning_OTDOA_NeighbourCellList_sequence_of,
                                                  1, maxCellMeas, FALSE);

  return offset;
}


static const per_sequence_t UE_Positioning_OTDOA_AssistanceData_sequence[] = {
  { &hf_rrc_ue_positioning_OTDOA_ReferenceCellInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_Positioning_OTDOA_ReferenceCellInfo },
  { &hf_rrc_ue_positioning_OTDOA_NeighbourCellList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_Positioning_OTDOA_NeighbourCellList },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UE_Positioning_OTDOA_AssistanceData(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UE_Positioning_OTDOA_AssistanceData, UE_Positioning_OTDOA_AssistanceData_sequence);

  return offset;
}


static const per_sequence_t UE_Positioning_Measurement_sequence[] = {
  { &hf_rrc_ue_positioning_ReportingQuantity, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UE_Positioning_ReportingQuantity },
  { &hf_rrc_reportCriteria_20, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UE_Positioning_ReportCriteria },
  { &hf_rrc_ue_positioning_OTDOA_AssistanceData, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_Positioning_OTDOA_AssistanceData },
  { &hf_rrc_ue_positioning_GPS_AssistanceData, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_Positioning_GPS_AssistanceData },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UE_Positioning_Measurement(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UE_Positioning_Measurement, UE_Positioning_Measurement_sequence);

  return offset;
}


static const value_string rrc_UL_TrCH_Identity_vals[] = {
  {   0, "dch" },
  {   1, "rachorcpch" },
  {   2, "usch" },
  { 0, NULL }
};

static const per_choice_t UL_TrCH_Identity_choice[] = {
  {   0, &hf_rrc_dch             , ASN1_NO_EXTENSIONS     , dissect_rrc_TransportChannelIdentity },
  {   1, &hf_rrc_rachorcpch      , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   2, &hf_rrc_usch            , ASN1_NO_EXTENSIONS     , dissect_rrc_TransportChannelIdentity },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_UL_TrCH_Identity(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_UL_TrCH_Identity, UL_TrCH_Identity_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t TrafficVolumeMeasurementObjectList_sequence_of[1] = {
  { &hf_rrc_TrafficVolumeMeasurementObjectList_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UL_TrCH_Identity },
};

static int
dissect_rrc_TrafficVolumeMeasurementObjectList(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_TrafficVolumeMeasurementObjectList, TrafficVolumeMeasurementObjectList_sequence_of,
                                                  1, maxTrCH, FALSE);

  return offset;
}



static int
dissect_rrc_TimeInterval(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            1U, 13U, NULL, FALSE);

  return offset;
}


static const value_string rrc_TrafficVolumeMeasQuantity_vals[] = {
  {   0, "rlc-BufferPayload" },
  {   1, "averageRLC-BufferPayload" },
  {   2, "varianceOfRLC-BufferPayload" },
  { 0, NULL }
};

static const per_choice_t TrafficVolumeMeasQuantity_choice[] = {
  {   0, &hf_rrc_rlc_BufferPayload, ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   1, &hf_rrc_averageRLC_BufferPayload, ASN1_NO_EXTENSIONS     , dissect_rrc_TimeInterval },
  {   2, &hf_rrc_varianceOfRLC_BufferPayload, ASN1_NO_EXTENSIONS     , dissect_rrc_TimeInterval },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_TrafficVolumeMeasQuantity(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_TrafficVolumeMeasQuantity, TrafficVolumeMeasQuantity_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t TrafficVolumeReportingQuantity_sequence[] = {
  { &hf_rrc_rlc_RB_BufferPayload, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { &hf_rrc_rlc_RB_BufferPayloadAverage, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { &hf_rrc_rlc_RB_BufferPayloadVariance, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_TrafficVolumeReportingQuantity(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_TrafficVolumeReportingQuantity, TrafficVolumeReportingQuantity_sequence);

  return offset;
}


static const value_string rrc_TrafficVolumeEventType_vals[] = {
  {   0, "e4a" },
  {   1, "e4b" },
  { 0, NULL }
};


static int
dissect_rrc_TrafficVolumeEventType(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     2, NULL, FALSE, 0, NULL);

  return offset;
}


static const value_string rrc_TrafficVolumeThreshold_vals[] = {
  {   0, "th8" },
  {   1, "th16" },
  {   2, "th32" },
  {   3, "th64" },
  {   4, "th128" },
  {   5, "th256" },
  {   6, "th512" },
  {   7, "th1024" },
  {   8, "th2k" },
  {   9, "th3k" },
  {  10, "th4k" },
  {  11, "th6k" },
  {  12, "th8k" },
  {  13, "th12k" },
  {  14, "th16k" },
  {  15, "th24k" },
  {  16, "th32k" },
  {  17, "th48k" },
  {  18, "th64k" },
  {  19, "th96k" },
  {  20, "th128k" },
  {  21, "th192k" },
  {  22, "th256k" },
  {  23, "th384k" },
  {  24, "th512k" },
  {  25, "th768k" },
  { 0, NULL }
};

static value_string_ext rrc_TrafficVolumeThreshold_vals_ext = VALUE_STRING_EXT_INIT(rrc_TrafficVolumeThreshold_vals);


static int
dissect_rrc_TrafficVolumeThreshold(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     26, NULL, FALSE, 0, NULL);

  return offset;
}


static const value_string rrc_PendingTimeAfterTrigger_vals[] = {
  {   0, "ptat0-25" },
  {   1, "ptat0-5" },
  {   2, "ptat1" },
  {   3, "ptat2" },
  {   4, "ptat4" },
  {   5, "ptat8" },
  {   6, "ptat16" },
  { 0, NULL }
};


static int
dissect_rrc_PendingTimeAfterTrigger(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     7, NULL, FALSE, 0, NULL);

  return offset;
}


static const value_string rrc_TX_InterruptionAfterTrigger_vals[] = {
  {   0, "txiat0-25" },
  {   1, "txiat0-5" },
  {   2, "txiat1" },
  {   3, "txiat2" },
  {   4, "txiat4" },
  {   5, "txiat8" },
  {   6, "txiat16" },
  { 0, NULL }
};


static int
dissect_rrc_TX_InterruptionAfterTrigger(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     7, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t TrafficVolumeEventParam_sequence[] = {
  { &hf_rrc_eventID_03      , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TrafficVolumeEventType },
  { &hf_rrc_reportingThreshold_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TrafficVolumeThreshold },
  { &hf_rrc_timeToTrigger   , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_TimeToTrigger },
  { &hf_rrc_pendingTimeAfterTrigger, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_PendingTimeAfterTrigger },
  { &hf_rrc_tx_InterruptionAfterTrigger, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_TX_InterruptionAfterTrigger },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_TrafficVolumeEventParam(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_TrafficVolumeEventParam, TrafficVolumeEventParam_sequence);

  return offset;
}


static const per_sequence_t SEQUENCE_SIZE_1_maxMeasParEvent_OF_TrafficVolumeEventParam_sequence_of[1] = {
  { &hf_rrc_eventSpecificParameters_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TrafficVolumeEventParam },
};

static int
dissect_rrc_SEQUENCE_SIZE_1_maxMeasParEvent_OF_TrafficVolumeEventParam(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_SEQUENCE_SIZE_1_maxMeasParEvent_OF_TrafficVolumeEventParam, SEQUENCE_SIZE_1_maxMeasParEvent_OF_TrafficVolumeEventParam_sequence_of,
                                                  1, maxMeasParEvent, FALSE);

  return offset;
}


static const per_sequence_t TransChCriteria_sequence[] = {
  { &hf_rrc_ul_transportChannelID, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_TrCH_Identity },
  { &hf_rrc_eventSpecificParameters, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_SEQUENCE_SIZE_1_maxMeasParEvent_OF_TrafficVolumeEventParam },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_TransChCriteria(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_TransChCriteria, TransChCriteria_sequence);

  return offset;
}


static const per_sequence_t TransChCriteriaList_sequence_of[1] = {
  { &hf_rrc_TransChCriteriaList_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TransChCriteria },
};

static int
dissect_rrc_TransChCriteriaList(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_TransChCriteriaList, TransChCriteriaList_sequence_of,
                                                  1, maxTrCH, FALSE);

  return offset;
}


static const per_sequence_t TrafficVolumeReportingCriteria_sequence[] = {
  { &hf_rrc_transChCriteriaList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_TransChCriteriaList },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_TrafficVolumeReportingCriteria(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_TrafficVolumeReportingCriteria, TrafficVolumeReportingCriteria_sequence);

  return offset;
}


static const value_string rrc_TrafficVolumeReportCriteria_vals[] = {
  {   0, "trafficVolumeReportingCriteria" },
  {   1, "periodicalReportingCriteria" },
  {   2, "noReporting" },
  { 0, NULL }
};

static const per_choice_t TrafficVolumeReportCriteria_choice[] = {
  {   0, &hf_rrc_trafficVolumeReportingCriteria, ASN1_NO_EXTENSIONS     , dissect_rrc_TrafficVolumeReportingCriteria },
  {   1, &hf_rrc_periodicalReportingCriteria, ASN1_NO_EXTENSIONS     , dissect_rrc_PeriodicalReportingCriteria },
  {   2, &hf_rrc_noReporting_02  , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_TrafficVolumeReportCriteria(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_TrafficVolumeReportCriteria, TrafficVolumeReportCriteria_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t TrafficVolumeMeasurement_sequence[] = {
  { &hf_rrc_trafficVolumeMeasurementObjectList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_TrafficVolumeMeasurementObjectList },
  { &hf_rrc_trafficVolumeMeasQuantity, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_TrafficVolumeMeasQuantity },
  { &hf_rrc_trafficVolumeReportingQuantity, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_TrafficVolumeReportingQuantity },
  { &hf_rrc_measurementValidity, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MeasurementValidity },
  { &hf_rrc_reportCriteria_18, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TrafficVolumeReportCriteria },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_TrafficVolumeMeasurement(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_TrafficVolumeMeasurement, TrafficVolumeMeasurement_sequence);

  return offset;
}


static const per_sequence_t BLER_TransChIdList_sequence_of[1] = {
  { &hf_rrc_BLER_TransChIdList_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TransportChannelIdentity },
};

static int
dissect_rrc_BLER_TransChIdList(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_BLER_TransChIdList, BLER_TransChIdList_sequence_of,
                                                  1, maxTrCH, FALSE);

  return offset;
}



static int
dissect_rrc_SIR_TFCS(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_rrc_TFCS_IdentityPlain(tvb, offset, actx, tree, hf_index);

  return offset;
}


static const per_sequence_t SIR_TFCS_List_sequence_of[1] = {
  { &hf_rrc_SIR_TFCS_List_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SIR_TFCS },
};

static int
dissect_rrc_SIR_TFCS_List(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_SIR_TFCS_List, SIR_TFCS_List_sequence_of,
                                                  1, maxCCTrCH, FALSE);

  return offset;
}


static const per_sequence_t T_tdd_112_sequence[] = {
  { &hf_rrc_sir_TFCS_List   , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_SIR_TFCS_List },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_tdd_112(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_tdd_112, T_tdd_112_sequence);

  return offset;
}


static const value_string rrc_T_modeSpecificInfo_130_vals[] = {
  {   0, "fdd" },
  {   1, "tdd" },
  { 0, NULL }
};

static const per_choice_t T_modeSpecificInfo_130_choice[] = {
  {   0, &hf_rrc_fdd_45          , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   1, &hf_rrc_tdd_120         , ASN1_NO_EXTENSIONS     , dissect_rrc_T_tdd_112 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_modeSpecificInfo_130(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_modeSpecificInfo_130, T_modeSpecificInfo_130_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t QualityReportingQuantity_sequence[] = {
  { &hf_rrc_dl_TransChBLER  , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { &hf_rrc_bler_dl_TransChIdList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_BLER_TransChIdList },
  { &hf_rrc_modeSpecificInfo_130, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_modeSpecificInfo_130 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_QualityReportingQuantity(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_QualityReportingQuantity, QualityReportingQuantity_sequence);

  return offset;
}


static const per_sequence_t QualityReportingCriteriaSingle_sequence[] = {
  { &hf_rrc_transportChannelIdentity, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TransportChannelIdentity },
  { &hf_rrc_totalCRC        , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_1_512 },
  { &hf_rrc_badCRC          , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_1_512 },
  { &hf_rrc_pendingAfterTrigger, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_1_512 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_QualityReportingCriteriaSingle(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_QualityReportingCriteriaSingle, QualityReportingCriteriaSingle_sequence);

  return offset;
}


static const per_sequence_t QualityReportingCriteria_sequence_of[1] = {
  { &hf_rrc_QualityReportingCriteria_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_QualityReportingCriteriaSingle },
};

static int
dissect_rrc_QualityReportingCriteria(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_QualityReportingCriteria, QualityReportingCriteria_sequence_of,
                                                  1, maxTrCH, FALSE);

  return offset;
}


static const value_string rrc_QualityReportCriteria_vals[] = {
  {   0, "qualityReportingCriteria" },
  {   1, "periodicalReportingCriteria" },
  {   2, "noReporting" },
  { 0, NULL }
};

static const per_choice_t QualityReportCriteria_choice[] = {
  {   0, &hf_rrc_qualityReportingCriteria, ASN1_NO_EXTENSIONS     , dissect_rrc_QualityReportingCriteria },
  {   1, &hf_rrc_periodicalReportingCriteria, ASN1_NO_EXTENSIONS     , dissect_rrc_PeriodicalReportingCriteria },
  {   2, &hf_rrc_noReporting_02  , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_QualityReportCriteria(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_QualityReportCriteria, QualityReportCriteria_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t QualityMeasurement_sequence[] = {
  { &hf_rrc_qualityReportingQuantity, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_QualityReportingQuantity },
  { &hf_rrc_reportCriteria_15, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_QualityReportCriteria },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_QualityMeasurement(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_QualityMeasurement, QualityMeasurement_sequence);

  return offset;
}


static const value_string rrc_UE_MeasurementQuantity_vals[] = {
  {   0, "ue-TransmittedPower" },
  {   1, "utra-Carrier-RSSI" },
  {   2, "ue-RX-TX-TimeDifference" },
  { 0, NULL }
};


static int
dissect_rrc_UE_MeasurementQuantity(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     3, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t UE_InternalMeasQuantity_sequence[] = {
  { &hf_rrc_measurementQuantity_05, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UE_MeasurementQuantity },
  { &hf_rrc_filterCoefficient, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_FilterCoefficient },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UE_InternalMeasQuantity(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UE_InternalMeasQuantity, UE_InternalMeasQuantity_sequence);

  return offset;
}


static const per_sequence_t T_fdd_165_sequence[] = {
  { &hf_rrc_ue_RX_TX_TimeDifference, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_fdd_165(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_fdd_165, T_fdd_165_sequence);

  return offset;
}


static const per_sequence_t T_tdd_114_sequence[] = {
  { &hf_rrc_appliedTA_02    , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_tdd_114(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_tdd_114, T_tdd_114_sequence);

  return offset;
}


static const value_string rrc_T_modeSpecificInfo_133_vals[] = {
  {   0, "fdd" },
  {   1, "tdd" },
  { 0, NULL }
};

static const per_choice_t T_modeSpecificInfo_133_choice[] = {
  {   0, &hf_rrc_fdd_171         , ASN1_NO_EXTENSIONS     , dissect_rrc_T_fdd_165 },
  {   1, &hf_rrc_tdd_122         , ASN1_NO_EXTENSIONS     , dissect_rrc_T_tdd_114 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_modeSpecificInfo_133(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_modeSpecificInfo_133, T_modeSpecificInfo_133_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t UE_InternalReportingQuantity_sequence[] = {
  { &hf_rrc_ue_TransmittedPower, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { &hf_rrc_modeSpecificInfo_133, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_modeSpecificInfo_133 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UE_InternalReportingQuantity(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UE_InternalReportingQuantity, UE_InternalReportingQuantity_sequence);

  return offset;
}



static int
dissect_rrc_TransmittedPowerThreshold(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            -50, 33U, NULL, FALSE);

  return offset;
}


static const per_sequence_t UE_6AB_Event_sequence[] = {
  { &hf_rrc_timeToTrigger   , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TimeToTrigger },
  { &hf_rrc_transmittedPowerThreshold, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TransmittedPowerThreshold },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UE_6AB_Event(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UE_6AB_Event, UE_6AB_Event_sequence);

  return offset;
}



static int
dissect_rrc_UE_RX_TX_TimeDifferenceThreshold(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            768U, 1280U, NULL, FALSE);

  return offset;
}


static const per_sequence_t UE_6FG_Event_sequence[] = {
  { &hf_rrc_timeToTrigger   , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TimeToTrigger },
  { &hf_rrc_ue_RX_TX_TimeDifferenceThreshold, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UE_RX_TX_TimeDifferenceThreshold },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UE_6FG_Event(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UE_6FG_Event, UE_6FG_Event_sequence);

  return offset;
}


static const value_string rrc_UE_InternalEventParam_vals[] = {
  {   0, "event6a" },
  {   1, "event6b" },
  {   2, "event6c" },
  {   3, "event6d" },
  {   4, "event6e" },
  {   5, "event6f" },
  {   6, "event6g" },
  { 0, NULL }
};

static const per_choice_t UE_InternalEventParam_choice[] = {
  {   0, &hf_rrc_event6a         , ASN1_NO_EXTENSIONS     , dissect_rrc_UE_6AB_Event },
  {   1, &hf_rrc_event6b         , ASN1_NO_EXTENSIONS     , dissect_rrc_UE_6AB_Event },
  {   2, &hf_rrc_event6c         , ASN1_NO_EXTENSIONS     , dissect_rrc_TimeToTrigger },
  {   3, &hf_rrc_event6d         , ASN1_NO_EXTENSIONS     , dissect_rrc_TimeToTrigger },
  {   4, &hf_rrc_event6e         , ASN1_NO_EXTENSIONS     , dissect_rrc_TimeToTrigger },
  {   5, &hf_rrc_event6f         , ASN1_NO_EXTENSIONS     , dissect_rrc_UE_6FG_Event },
  {   6, &hf_rrc_event6g         , ASN1_NO_EXTENSIONS     , dissect_rrc_UE_6FG_Event },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_UE_InternalEventParam(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_UE_InternalEventParam, UE_InternalEventParam_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t UE_InternalEventParamList_sequence_of[1] = {
  { &hf_rrc_UE_InternalEventParamList_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UE_InternalEventParam },
};

static int
dissect_rrc_UE_InternalEventParamList(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_UE_InternalEventParamList, UE_InternalEventParamList_sequence_of,
                                                  1, maxMeasEvent, FALSE);

  return offset;
}


static const per_sequence_t UE_InternalReportingCriteria_sequence[] = {
  { &hf_rrc_ue_InternalEventParamList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_InternalEventParamList },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UE_InternalReportingCriteria(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UE_InternalReportingCriteria, UE_InternalReportingCriteria_sequence);

  return offset;
}


static const value_string rrc_UE_InternalReportCriteria_vals[] = {
  {   0, "ue-InternalReportingCriteria" },
  {   1, "periodicalReportingCriteria" },
  {   2, "noReporting" },
  { 0, NULL }
};

static const per_choice_t UE_InternalReportCriteria_choice[] = {
  {   0, &hf_rrc_ue_InternalReportingCriteria, ASN1_NO_EXTENSIONS     , dissect_rrc_UE_InternalReportingCriteria },
  {   1, &hf_rrc_periodicalReportingCriteria, ASN1_NO_EXTENSIONS     , dissect_rrc_PeriodicalReportingCriteria },
  {   2, &hf_rrc_noReporting_02  , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_UE_InternalReportCriteria(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_UE_InternalReportCriteria, UE_InternalReportCriteria_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t UE_InternalMeasurement_sequence[] = {
  { &hf_rrc_ue_InternalMeasQuantity, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_InternalMeasQuantity },
  { &hf_rrc_ue_InternalReportingQuantity, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_InternalReportingQuantity },
  { &hf_rrc_reportCriteria_19, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UE_InternalReportCriteria },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UE_InternalMeasurement(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UE_InternalMeasurement, UE_InternalMeasurement_sequence);

  return offset;
}


static const value_string rrc_MeasurementType_vals[] = {
  {   0, "intraFrequencyMeasurement" },
  {   1, "interFrequencyMeasurement" },
  {   2, "interRATMeasurement" },
  {   3, "ue-positioning-Measurement" },
  {   4, "trafficVolumeMeasurement" },
  {   5, "qualityMeasurement" },
  {   6, "ue-InternalMeasurement" },
  { 0, NULL }
};

static const per_choice_t MeasurementType_choice[] = {
  {   0, &hf_rrc_intraFrequencyMeasurement, ASN1_NO_EXTENSIONS     , dissect_rrc_IntraFrequencyMeasurement },
  {   1, &hf_rrc_interFrequencyMeasurement, ASN1_NO_EXTENSIONS     , dissect_rrc_InterFrequencyMeasurement },
  {   2, &hf_rrc_interRATMeasurement, ASN1_NO_EXTENSIONS     , dissect_rrc_InterRATMeasurement },
  {   3, &hf_rrc_ue_positioning_Measurement, ASN1_NO_EXTENSIONS     , dissect_rrc_UE_Positioning_Measurement },
  {   4, &hf_rrc_trafficVolumeMeasurement, ASN1_NO_EXTENSIONS     , dissect_rrc_TrafficVolumeMeasurement },
  {   5, &hf_rrc_qualityMeasurement, ASN1_NO_EXTENSIONS     , dissect_rrc_QualityMeasurement },
  {   6, &hf_rrc_ue_InternalMeasurement, ASN1_NO_EXTENSIONS     , dissect_rrc_UE_InternalMeasurement },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_MeasurementType(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_MeasurementType, MeasurementType_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t T_modify_01_sequence[] = {
  { &hf_rrc_measurementType , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MeasurementType },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_modify_01(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_modify_01, T_modify_01_sequence);

  return offset;
}


static const value_string rrc_MeasurementCommand_vals[] = {
  {   0, "setup" },
  {   1, "modify" },
  {   2, "release" },
  { 0, NULL }
};

static const per_choice_t MeasurementCommand_choice[] = {
  {   0, &hf_rrc_setup           , ASN1_NO_EXTENSIONS     , dissect_rrc_MeasurementType },
  {   1, &hf_rrc_modify_01       , ASN1_NO_EXTENSIONS     , dissect_rrc_T_modify_01 },
  {   2, &hf_rrc_release_01      , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_MeasurementCommand(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_MeasurementCommand, MeasurementCommand_choice,
                                 NULL);

  return offset;
}


static const value_string rrc_TransferMode_vals[] = {
  {   0, "acknowledgedModeRLC" },
  {   1, "unacknowledgedModeRLC" },
  { 0, NULL }
};


static int
dissect_rrc_TransferMode(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     2, NULL, FALSE, 0, NULL);

  return offset;
}


static const value_string rrc_PeriodicalOrEventTrigger_vals[] = {
  {   0, "periodical" },
  {   1, "eventTrigger" },
  { 0, NULL }
};


static int
dissect_rrc_PeriodicalOrEventTrigger(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     2, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t MeasurementReportingMode_sequence[] = {
  { &hf_rrc_measurementReportTransferMode, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TransferMode },
  { &hf_rrc_periodicalOrEventTrigger, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PeriodicalOrEventTrigger },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_MeasurementReportingMode(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_MeasurementReportingMode, MeasurementReportingMode_sequence);

  return offset;
}


static const per_sequence_t AdditionalMeasurementID_List_sequence_of[1] = {
  { &hf_rrc_AdditionalMeasurementID_List_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MeasurementIdentity },
};

static int
dissect_rrc_AdditionalMeasurementID_List(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_AdditionalMeasurementID_List, AdditionalMeasurementID_List_sequence_of,
                                                  1, maxAdditionalMeas, FALSE);

  return offset;
}



static int
dissect_rrc_TGPS_Reconfiguration_CFN(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            0U, 255U, NULL, FALSE);

  return offset;
}


static const per_sequence_t T_activate_03_sequence[] = {
  { &hf_rrc_tgcfn           , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TGCFN },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_activate_03(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_activate_03, T_activate_03_sequence);

  return offset;
}


static const value_string rrc_T_tgps_Status_03_vals[] = {
  {   0, "activate" },
  {   1, "deactivate" },
  { 0, NULL }
};

static const per_choice_t T_tgps_Status_03_choice[] = {
  {   0, &hf_rrc_activate_03     , ASN1_NO_EXTENSIONS     , dissect_rrc_T_activate_03 },
  {   1, &hf_rrc_deactivate      , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_tgps_Status_03(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_tgps_Status_03, T_tgps_Status_03_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t TGP_SequenceShort_sequence[] = {
  { &hf_rrc_tgpsi           , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TGPSI },
  { &hf_rrc_tgps_Status_03  , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_tgps_Status_03 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_TGP_SequenceShort(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_TGP_SequenceShort, TGP_SequenceShort_sequence);

  return offset;
}


static const per_sequence_t SEQUENCE_SIZE_1_maxTGPS_OF_TGP_SequenceShort_sequence_of[1] = {
  { &hf_rrc_tgp_SequenceShortList_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TGP_SequenceShort },
};

static int
dissect_rrc_SEQUENCE_SIZE_1_maxTGPS_OF_TGP_SequenceShort(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_SEQUENCE_SIZE_1_maxTGPS_OF_TGP_SequenceShort, SEQUENCE_SIZE_1_maxTGPS_OF_TGP_SequenceShort_sequence_of,
                                                  1, maxTGPS, FALSE);

  return offset;
}


static const per_sequence_t DPCH_CompressedModeStatusInfo_sequence[] = {
  { &hf_rrc_tgps_Reconfiguration_CFN, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TGPS_Reconfiguration_CFN },
  { &hf_rrc_tgp_SequenceShortList, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SEQUENCE_SIZE_1_maxTGPS_OF_TGP_SequenceShort },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_DPCH_CompressedModeStatusInfo(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_DPCH_CompressedModeStatusInfo, DPCH_CompressedModeStatusInfo_sequence);

  return offset;
}


static const per_sequence_t MeasurementControl_r3_IEs_sequence[] = {
  { &hf_rrc_rrc_TransactionIdentifier, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RRC_TransactionIdentifier },
  { &hf_rrc_measurementIdentity, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MeasurementIdentity },
  { &hf_rrc_measurementCommand, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MeasurementCommand },
  { &hf_rrc_measurementReportingMode, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MeasurementReportingMode },
  { &hf_rrc_additionalMeasurementList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_AdditionalMeasurementID_List },
  { &hf_rrc_dpch_CompressedModeStatusInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DPCH_CompressedModeStatusInfo },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_MeasurementControl_r3_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_MeasurementControl_r3_IEs, MeasurementControl_r3_IEs_sequence);

  return offset;
}


static const per_sequence_t UE_Positioning_ReportingQuantity_v390ext_sequence[] = {
  { &hf_rrc_vertical_Accuracy, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UE_Positioning_Accuracy },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UE_Positioning_ReportingQuantity_v390ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UE_Positioning_ReportingQuantity_v390ext, UE_Positioning_ReportingQuantity_v390ext_sequence);

  return offset;
}


static const per_sequence_t UE_Positioning_Measurement_v390ext_sequence[] = {
  { &hf_rrc_ue_positioning_ReportingQuantity_v390ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_Positioning_ReportingQuantity_v390ext },
  { &hf_rrc_measurementValidity, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MeasurementValidity },
  { &hf_rrc_ue_positioning_OTDOA_AssistanceData_UEB, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_Positioning_OTDOA_AssistanceData_UEB },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UE_Positioning_Measurement_v390ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UE_Positioning_Measurement_v390ext, UE_Positioning_Measurement_v390ext_sequence);

  return offset;
}


static const per_sequence_t MeasurementControl_v390ext_sequence[] = {
  { &hf_rrc_ue_Positioning_Measurement_v390ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_Positioning_Measurement_v390ext },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_MeasurementControl_v390ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_MeasurementControl_v390ext, MeasurementControl_v390ext_sequence);

  return offset;
}


static const per_sequence_t MeasurementControl_v3a0ext_sequence[] = {
  { &hf_rrc_sfn_Offset_Validity, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_SFN_Offset_Validity },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_MeasurementControl_v3a0ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_MeasurementControl_v3a0ext, MeasurementControl_v3a0ext_sequence);

  return offset;
}



static int
dissect_rrc_DeltaRSCP(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            -5, -1, NULL, FALSE);

  return offset;
}


static const per_sequence_t Intra_FreqEventCriteriaList_v590ext_sequence_of[1] = {
  { &hf_rrc_Intra_FreqEventCriteriaList_v590ext_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DeltaRSCP },
};

static int
dissect_rrc_Intra_FreqEventCriteriaList_v590ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_Intra_FreqEventCriteriaList_v590ext, Intra_FreqEventCriteriaList_v590ext_sequence_of,
                                                  1, maxMeasEvent, FALSE);

  return offset;
}


static const per_sequence_t DeltaRSCPPerCell_sequence[] = {
  { &hf_rrc_deltaRSCP       , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DeltaRSCP },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_DeltaRSCPPerCell(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_DeltaRSCPPerCell, DeltaRSCPPerCell_sequence);

  return offset;
}


static const per_sequence_t ThreholdNonUsedFrequency_deltaList_sequence_of[1] = {
  { &hf_rrc_ThreholdNonUsedFrequency_deltaList_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DeltaRSCPPerCell },
};

static int
dissect_rrc_ThreholdNonUsedFrequency_deltaList(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_ThreholdNonUsedFrequency_deltaList, ThreholdNonUsedFrequency_deltaList_sequence_of,
                                                  1, maxFreq, FALSE);

  return offset;
}


static const per_sequence_t Inter_FreqEventCriteria_v590ext_sequence[] = {
  { &hf_rrc_threholdUsedFrequency_delta, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DeltaRSCP },
  { &hf_rrc_threholdNonUsedFrequency_deltaList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_ThreholdNonUsedFrequency_deltaList },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_Inter_FreqEventCriteria_v590ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_Inter_FreqEventCriteria_v590ext, Inter_FreqEventCriteria_v590ext_sequence);

  return offset;
}


static const per_sequence_t Inter_FreqEventCriteriaList_v590ext_sequence_of[1] = {
  { &hf_rrc_Inter_FreqEventCriteriaList_v590ext_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_Inter_FreqEventCriteria_v590ext },
};

static int
dissect_rrc_Inter_FreqEventCriteriaList_v590ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_Inter_FreqEventCriteriaList_v590ext, Inter_FreqEventCriteriaList_v590ext_sequence_of,
                                                  1, maxMeasEvent, FALSE);

  return offset;
}


static const value_string rrc_T_measurementCommand_v590ext_vals[] = {
  {   0, "intra-frequency" },
  {   1, "inter-frequency" },
  { 0, NULL }
};

static const per_choice_t T_measurementCommand_v590ext_choice[] = {
  {   0, &hf_rrc_intra_frequency , ASN1_NO_EXTENSIONS     , dissect_rrc_Intra_FreqEventCriteriaList_v590ext },
  {   1, &hf_rrc_inter_frequency , ASN1_NO_EXTENSIONS     , dissect_rrc_Inter_FreqEventCriteriaList_v590ext },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_measurementCommand_v590ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_measurementCommand_v590ext, T_measurementCommand_v590ext_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t PeriodicReportingInfo_1b_sequence[] = {
  { &hf_rrc_reportingAmount , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_ReportingAmount },
  { &hf_rrc_reportingInterval, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_ReportingInterval },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_PeriodicReportingInfo_1b(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_PeriodicReportingInfo_1b, PeriodicReportingInfo_1b_sequence);

  return offset;
}


static const per_sequence_t IntraFreqReportingCriteria_1b_r5_sequence[] = {
  { &hf_rrc_periodicReportingInfo_1b, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PeriodicReportingInfo_1b },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_IntraFreqReportingCriteria_1b_r5(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_IntraFreqReportingCriteria_1b_r5, IntraFreqReportingCriteria_1b_r5_sequence);

  return offset;
}


static const per_sequence_t IntraFreqEvent_1d_r5_sequence[] = {
  { &hf_rrc_triggeringCondition, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_TriggeringCondition2 },
  { &hf_rrc_useCIO          , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_BOOLEAN },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_IntraFreqEvent_1d_r5(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_IntraFreqEvent_1d_r5, IntraFreqEvent_1d_r5_sequence);

  return offset;
}


static const per_sequence_t MeasurementControl_v590ext_IEs_sequence[] = {
  { &hf_rrc_measurementCommand_v590ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_measurementCommand_v590ext },
  { &hf_rrc_intraFreqReportingCriteria_1b_r5, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_IntraFreqReportingCriteria_1b_r5 },
  { &hf_rrc_intraFreqEvent_1d_r5, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_IntraFreqEvent_1d_r5 },
  { &hf_rrc_rrc_TransactionIdentifier_MSP_v590ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RRC_TransactionIdentifier },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_MeasurementControl_v590ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_MeasurementControl_v590ext_IEs, MeasurementControl_v590ext_IEs_sequence);

  return offset;
}



static int
dissect_rrc_InterRATCellInfoIndication(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            0U, 3U, NULL, FALSE);

  return offset;
}


static const per_sequence_t MeasurementControl_v5b0ext_IEs_sequence[] = {
  { &hf_rrc_interRATCellInfoIndication, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_InterRATCellInfoIndication },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_MeasurementControl_v5b0ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_MeasurementControl_v5b0ext_IEs, MeasurementControl_v5b0ext_IEs_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions_58_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_nonCriticalExtensions_58(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_nonCriticalExtensions_58, T_nonCriticalExtensions_58_sequence);

  return offset;
}


static const per_sequence_t T_v5b0NonCriticalExtensions_sequence[] = {
  { &hf_rrc_measurementControl_v5b0ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MeasurementControl_v5b0ext_IEs },
  { &hf_rrc_nonCriticalExtensions_58, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_nonCriticalExtensions_58 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v5b0NonCriticalExtensions(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v5b0NonCriticalExtensions, T_v5b0NonCriticalExtensions_sequence);

  return offset;
}


static const per_sequence_t T_v590NonCriticalExtensions_08_sequence[] = {
  { &hf_rrc_measurementControl_v590ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MeasurementControl_v590ext_IEs },
  { &hf_rrc_v5b0NonCriticalExtensions, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v5b0NonCriticalExtensions },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v590NonCriticalExtensions_08(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v590NonCriticalExtensions_08, T_v590NonCriticalExtensions_08_sequence);

  return offset;
}


static const per_sequence_t T_v4b0NonCriticalExtensions_05_sequence[] = {
  { &hf_rrc_v590NonCriticalExtensions_08, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v590NonCriticalExtensions_08 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v4b0NonCriticalExtensions_05(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v4b0NonCriticalExtensions_05, T_v4b0NonCriticalExtensions_05_sequence);

  return offset;
}


static const per_sequence_t T_laterNonCriticalExtensions_18_sequence[] = {
  { &hf_rrc_measurementControl_r3_add_ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_BIT_STRING },
  { &hf_rrc_v4b0NonCriticalExtensions_05, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v4b0NonCriticalExtensions_05 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_laterNonCriticalExtensions_18(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_laterNonCriticalExtensions_18, T_laterNonCriticalExtensions_18_sequence);

  return offset;
}


static const per_sequence_t T_v3a0NonCriticalExtensions_04_sequence[] = {
  { &hf_rrc_measurementControl_v3a0ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MeasurementControl_v3a0ext },
  { &hf_rrc_laterNonCriticalExtensions_18, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_laterNonCriticalExtensions_18 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v3a0NonCriticalExtensions_04(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v3a0NonCriticalExtensions_04, T_v3a0NonCriticalExtensions_04_sequence);

  return offset;
}


static const per_sequence_t T_v390nonCriticalExtensions_sequence[] = {
  { &hf_rrc_measurementControl_v390ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MeasurementControl_v390ext },
  { &hf_rrc_v3a0NonCriticalExtensions_04, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v3a0NonCriticalExtensions_04 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v390nonCriticalExtensions(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v390nonCriticalExtensions, T_v390nonCriticalExtensions_sequence);

  return offset;
}


static const per_sequence_t T_r3_11_sequence[] = {
  { &hf_rrc_measurementControl_r3, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MeasurementControl_r3_IEs },
  { &hf_rrc_v390nonCriticalExtensions, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v390nonCriticalExtensions },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_r3_11(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_r3_11, T_r3_11_sequence);

  return offset;
}


static const per_sequence_t T_fdd_133_sequence[] = {
  { &hf_rrc_primaryCPICH_Info, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_PrimaryCPICH_Info },
  { &hf_rrc_primaryCPICH_TX_Power, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_PrimaryCPICH_TX_Power },
  { &hf_rrc_readSFN_Indicator, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { &hf_rrc_tx_DiversityIndicator, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_fdd_133(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_fdd_133, T_fdd_133_sequence);

  return offset;
}


static const per_sequence_t SEQUENCE_SIZE_1_maxTS_OF_TimeslotInfo_sequence_of[1] = {
  { &hf_rrc_tdd384_item_01  , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TimeslotInfo },
};

static int
dissect_rrc_SEQUENCE_SIZE_1_maxTS_OF_TimeslotInfo(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_SEQUENCE_SIZE_1_maxTS_OF_TimeslotInfo, SEQUENCE_SIZE_1_maxTS_OF_TimeslotInfo_sequence_of,
                                                  1, maxTS, FALSE);

  return offset;
}


static const per_sequence_t TimeslotInfo_LCR_r4_sequence[] = {
  { &hf_rrc_timeslotNumber_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TimeslotNumber_LCR_r4 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_TimeslotInfo_LCR_r4(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_TimeslotInfo_LCR_r4, TimeslotInfo_LCR_r4_sequence);

  return offset;
}


static const per_sequence_t SEQUENCE_SIZE_1_maxTS_LCR_OF_TimeslotInfo_LCR_r4_sequence_of[1] = {
  { &hf_rrc_tdd128_item_02  , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TimeslotInfo_LCR_r4 },
};

static int
dissect_rrc_SEQUENCE_SIZE_1_maxTS_LCR_OF_TimeslotInfo_LCR_r4(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_SEQUENCE_SIZE_1_maxTS_LCR_OF_TimeslotInfo_LCR_r4, SEQUENCE_SIZE_1_maxTS_LCR_OF_TimeslotInfo_LCR_r4_sequence_of,
                                                  1, maxTS_LCR, FALSE);

  return offset;
}


static const value_string rrc_TimeslotInfoList_r4_vals[] = {
  {   0, "tdd384" },
  {   1, "tdd128" },
  { 0, NULL }
};

static const per_choice_t TimeslotInfoList_r4_choice[] = {
  {   0, &hf_rrc_tdd384_47       , ASN1_NO_EXTENSIONS     , dissect_rrc_SEQUENCE_SIZE_1_maxTS_OF_TimeslotInfo },
  {   1, &hf_rrc_tdd128_68       , ASN1_NO_EXTENSIONS     , dissect_rrc_SEQUENCE_SIZE_1_maxTS_LCR_OF_TimeslotInfo_LCR_r4 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_TimeslotInfoList_r4(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_TimeslotInfoList_r4, TimeslotInfoList_r4_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t T_tdd_84_sequence[] = {
  { &hf_rrc_primaryCCPCH_Info_03, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PrimaryCCPCH_Info_r4 },
  { &hf_rrc_primaryCCPCH_TX_Power, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_PrimaryCCPCH_TX_Power },
  { &hf_rrc_timeslotInfoList_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_TimeslotInfoList_r4 },
  { &hf_rrc_readSFN_Indicator, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_tdd_84(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_tdd_84, T_tdd_84_sequence);

  return offset;
}


static const value_string rrc_T_modeSpecificInfo_94_vals[] = {
  {   0, "fdd" },
  {   1, "tdd" },
  { 0, NULL }
};

static const per_choice_t T_modeSpecificInfo_94_choice[] = {
  {   0, &hf_rrc_fdd_136         , ASN1_NO_EXTENSIONS     , dissect_rrc_T_fdd_133 },
  {   1, &hf_rrc_tdd_90          , ASN1_NO_EXTENSIONS     , dissect_rrc_T_tdd_84 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_modeSpecificInfo_94(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_modeSpecificInfo_94, T_modeSpecificInfo_94_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t CellInfo_r4_sequence[] = {
  { &hf_rrc_cellIndividualOffset, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CellIndividualOffset },
  { &hf_rrc_referenceTimeDifferenceToCell, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_ReferenceTimeDifferenceToCell },
  { &hf_rrc_modeSpecificInfo_94, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_modeSpecificInfo_94 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_CellInfo_r4(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_CellInfo_r4, CellInfo_r4_sequence);

  return offset;
}


static const per_sequence_t NewIntraFreqCell_r4_sequence[] = {
  { &hf_rrc_intraFreqCellID , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_IntraFreqCellID },
  { &hf_rrc_cellInfo_01     , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CellInfo_r4 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_NewIntraFreqCell_r4(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_NewIntraFreqCell_r4, NewIntraFreqCell_r4_sequence);

  return offset;
}


static const per_sequence_t NewIntraFreqCellList_r4_sequence_of[1] = {
  { &hf_rrc_NewIntraFreqCellList_r4_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_NewIntraFreqCell_r4 },
};

static int
dissect_rrc_NewIntraFreqCellList_r4(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_NewIntraFreqCellList_r4, NewIntraFreqCellList_r4_sequence_of,
                                                  1, maxCellMeas, FALSE);

  return offset;
}


static const per_sequence_t IntraFreqCellInfoList_r4_sequence[] = {
  { &hf_rrc_removedIntraFreqCellList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RemovedIntraFreqCellList },
  { &hf_rrc_newIntraFreqCellList_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_NewIntraFreqCellList_r4 },
  { &hf_rrc_cellsForIntraFreqMeasList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CellsForIntraFreqMeasList },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_IntraFreqCellInfoList_r4(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_IntraFreqCellInfoList_r4, IntraFreqCellInfoList_r4_sequence);

  return offset;
}


static const value_string rrc_ForbiddenAffectCell_r4_vals[] = {
  {   0, "fdd" },
  {   1, "tdd" },
  { 0, NULL }
};

static const per_choice_t ForbiddenAffectCell_r4_choice[] = {
  {   0, &hf_rrc_fdd_155         , ASN1_NO_EXTENSIONS     , dissect_rrc_PrimaryCPICH_Info },
  {   1, &hf_rrc_tdd_47          , ASN1_NO_EXTENSIONS     , dissect_rrc_PrimaryCCPCH_Info_r4 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_ForbiddenAffectCell_r4(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_ForbiddenAffectCell_r4, ForbiddenAffectCell_r4_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t ForbiddenAffectCellList_r4_sequence_of[1] = {
  { &hf_rrc_ForbiddenAffectCellList_r4_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_ForbiddenAffectCell_r4 },
};

static int
dissect_rrc_ForbiddenAffectCellList_r4(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_ForbiddenAffectCellList_r4, ForbiddenAffectCellList_r4_sequence_of,
                                                  1, maxCellMeas, FALSE);

  return offset;
}


static const per_sequence_t Event1a_r4_sequence[] = {
  { &hf_rrc_triggeringCondition, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TriggeringCondition2 },
  { &hf_rrc_reportingRange  , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_ReportingRange },
  { &hf_rrc_forbiddenAffectCellList_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_ForbiddenAffectCellList_r4 },
  { &hf_rrc_w               , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_W },
  { &hf_rrc_reportDeactivationThreshold, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_ReportDeactivationThreshold },
  { &hf_rrc_reportingAmount , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_ReportingAmount },
  { &hf_rrc_reportingInterval, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_ReportingInterval },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_Event1a_r4(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_Event1a_r4, Event1a_r4_sequence);

  return offset;
}


static const per_sequence_t Event1b_r4_sequence[] = {
  { &hf_rrc_triggeringCondition_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TriggeringCondition1 },
  { &hf_rrc_reportingRange  , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_ReportingRange },
  { &hf_rrc_forbiddenAffectCellList_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_ForbiddenAffectCellList_r4 },
  { &hf_rrc_w               , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_W },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_Event1b_r4(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_Event1b_r4, Event1b_r4_sequence);

  return offset;
}


static const value_string rrc_IntraFreqEvent_r4_vals[] = {
  {   0, "e1a" },
  {   1, "e1b" },
  {   2, "e1c" },
  {   3, "e1d" },
  {   4, "e1e" },
  {   5, "e1f" },
  {   6, "e1g" },
  {   7, "e1h" },
  {   8, "e1i" },
  { 0, NULL }
};

static const per_choice_t IntraFreqEvent_r4_choice[] = {
  {   0, &hf_rrc_e1a_01          , ASN1_NO_EXTENSIONS     , dissect_rrc_Event1a_r4 },
  {   1, &hf_rrc_e1b_01          , ASN1_NO_EXTENSIONS     , dissect_rrc_Event1b_r4 },
  {   2, &hf_rrc_e1c             , ASN1_NO_EXTENSIONS     , dissect_rrc_Event1c },
  {   3, &hf_rrc_e1d             , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   4, &hf_rrc_e1e             , ASN1_NO_EXTENSIONS     , dissect_rrc_Event1e },
  {   5, &hf_rrc_e1f             , ASN1_NO_EXTENSIONS     , dissect_rrc_Event1f },
  {   6, &hf_rrc_e1g             , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   7, &hf_rrc_e1h             , ASN1_NO_EXTENSIONS     , dissect_rrc_ThresholdUsedFrequency },
  {   8, &hf_rrc_e1i             , ASN1_NO_EXTENSIONS     , dissect_rrc_ThresholdUsedFrequency },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_IntraFreqEvent_r4(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_IntraFreqEvent_r4, IntraFreqEvent_r4_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t IntraFreqEventCriteria_r4_sequence[] = {
  { &hf_rrc_event_01        , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_IntraFreqEvent_r4 },
  { &hf_rrc_hysteresis_01   , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_Hysteresis },
  { &hf_rrc_timeToTrigger   , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TimeToTrigger },
  { &hf_rrc_reportingCellStatus, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_ReportingCellStatus },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_IntraFreqEventCriteria_r4(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_IntraFreqEventCriteria_r4, IntraFreqEventCriteria_r4_sequence);

  return offset;
}


static const per_sequence_t IntraFreqEventCriteriaList_r4_sequence_of[1] = {
  { &hf_rrc_IntraFreqEventCriteriaList_r4_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_IntraFreqEventCriteria_r4 },
};

static int
dissect_rrc_IntraFreqEventCriteriaList_r4(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_IntraFreqEventCriteriaList_r4, IntraFreqEventCriteriaList_r4_sequence_of,
                                                  1, maxMeasEvent, FALSE);

  return offset;
}


static const per_sequence_t IntraFreqReportingCriteria_r4_sequence[] = {
  { &hf_rrc_eventCriteriaList_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_IntraFreqEventCriteriaList_r4 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_IntraFreqReportingCriteria_r4(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_IntraFreqReportingCriteria_r4, IntraFreqReportingCriteria_r4_sequence);

  return offset;
}


static const value_string rrc_IntraFreqReportCriteria_r4_vals[] = {
  {   0, "intraFreqReportingCriteria" },
  {   1, "periodicalReportingCriteria" },
  {   2, "noReporting" },
  { 0, NULL }
};

static const per_choice_t IntraFreqReportCriteria_r4_choice[] = {
  {   0, &hf_rrc_intraFreqReportingCriteria_03, ASN1_NO_EXTENSIONS     , dissect_rrc_IntraFreqReportingCriteria_r4 },
  {   1, &hf_rrc_periodicalReportingCriteria_01, ASN1_NO_EXTENSIONS     , dissect_rrc_PeriodicalWithReportingCellStatus },
  {   2, &hf_rrc_noReporting     , ASN1_NO_EXTENSIONS     , dissect_rrc_ReportingCellStatusOpt },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_IntraFreqReportCriteria_r4(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_IntraFreqReportCriteria_r4, IntraFreqReportCriteria_r4_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t IntraFrequencyMeasurement_r4_sequence[] = {
  { &hf_rrc_intraFreqCellInfoList_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_IntraFreqCellInfoList_r4 },
  { &hf_rrc_intraFreqMeasQuantity, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_IntraFreqMeasQuantity },
  { &hf_rrc_intraFreqReportingQuantity, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_IntraFreqReportingQuantity },
  { &hf_rrc_measurementValidity, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MeasurementValidity },
  { &hf_rrc_reportCriteria_10, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_IntraFreqReportCriteria_r4 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_IntraFrequencyMeasurement_r4(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_IntraFrequencyMeasurement_r4, IntraFrequencyMeasurement_r4_sequence);

  return offset;
}


static const per_sequence_t NewInterFreqCell_r4_sequence[] = {
  { &hf_rrc_interFreqCellID , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_InterFreqCellID },
  { &hf_rrc_frequencyInfo   , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_FrequencyInfo },
  { &hf_rrc_cellInfo_01     , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CellInfo_r4 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_NewInterFreqCell_r4(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_NewInterFreqCell_r4, NewInterFreqCell_r4_sequence);

  return offset;
}


static const per_sequence_t NewInterFreqCellList_r4_sequence_of[1] = {
  { &hf_rrc_NewInterFreqCellList_r4_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_NewInterFreqCell_r4 },
};

static int
dissect_rrc_NewInterFreqCellList_r4(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_NewInterFreqCellList_r4, NewInterFreqCellList_r4_sequence_of,
                                                  1, maxCellMeas, FALSE);

  return offset;
}


static const per_sequence_t InterFreqCellInfoList_r4_sequence[] = {
  { &hf_rrc_removedInterFreqCellList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RemovedInterFreqCellList },
  { &hf_rrc_newInterFreqCellList_03, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_NewInterFreqCellList_r4 },
  { &hf_rrc_cellsForInterFreqMeasList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CellsForInterFreqMeasList },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_InterFreqCellInfoList_r4(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_InterFreqCellInfoList_r4, InterFreqCellInfoList_r4_sequence);

  return offset;
}


static const value_string rrc_InterFreqReportCriteria_r4_vals[] = {
  {   0, "intraFreqReportingCriteria" },
  {   1, "interFreqReportingCriteria" },
  {   2, "periodicalReportingCriteria" },
  {   3, "noReporting" },
  { 0, NULL }
};

static const per_choice_t InterFreqReportCriteria_r4_choice[] = {
  {   0, &hf_rrc_intraFreqReportingCriteria_03, ASN1_NO_EXTENSIONS     , dissect_rrc_IntraFreqReportingCriteria_r4 },
  {   1, &hf_rrc_interFreqReportingCriteria_01, ASN1_NO_EXTENSIONS     , dissect_rrc_InterFreqReportingCriteria },
  {   2, &hf_rrc_periodicalReportingCriteria_01, ASN1_NO_EXTENSIONS     , dissect_rrc_PeriodicalWithReportingCellStatus },
  {   3, &hf_rrc_noReporting     , ASN1_NO_EXTENSIONS     , dissect_rrc_ReportingCellStatusOpt },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_InterFreqReportCriteria_r4(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_InterFreqReportCriteria_r4, InterFreqReportCriteria_r4_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t InterFrequencyMeasurement_r4_sequence[] = {
  { &hf_rrc_interFreqCellInfoList_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_InterFreqCellInfoList_r4 },
  { &hf_rrc_interFreqMeasQuantity, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_InterFreqMeasQuantity },
  { &hf_rrc_interFreqReportingQuantity, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_InterFreqReportingQuantity },
  { &hf_rrc_measurementValidity, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MeasurementValidity },
  { &hf_rrc_interFreqSetUpdate, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_AutonomousUpdateMode },
  { &hf_rrc_reportCriteria_02, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_InterFreqReportCriteria_r4 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_InterFrequencyMeasurement_r4(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_InterFrequencyMeasurement_r4, InterFrequencyMeasurement_r4_sequence);

  return offset;
}


static const per_sequence_t InterRATCellInfoList_r4_sequence[] = {
  { &hf_rrc_removedInterRATCellList, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RemovedInterRATCellList },
  { &hf_rrc_newInterRATCellList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_NewInterRATCellList },
  { &hf_rrc_cellsForInterRATMeasList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CellsForInterRATMeasList },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_InterRATCellInfoList_r4(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_InterRATCellInfoList_r4, InterRATCellInfoList_r4_sequence);

  return offset;
}


static const per_sequence_t InterRATMeasurement_r4_sequence[] = {
  { &hf_rrc_interRATCellInfoList_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_InterRATCellInfoList_r4 },
  { &hf_rrc_interRATMeasQuantity, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_InterRATMeasQuantity },
  { &hf_rrc_interRATReportingQuantity, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_InterRATReportingQuantity },
  { &hf_rrc_reportCriteria_08, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_InterRATReportCriteria },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_InterRATMeasurement_r4(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_InterRATMeasurement_r4, InterRATMeasurement_r4_sequence);

  return offset;
}


static const per_sequence_t UE_Positioning_ReportingQuantity_r4_sequence[] = {
  { &hf_rrc_methodType      , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UE_Positioning_MethodType },
  { &hf_rrc_positioningMethod, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PositioningMethod },
  { &hf_rrc_horizontalAccuracy, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_Positioning_Accuracy },
  { &hf_rrc_verticalAccuracy, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_Positioning_Accuracy },
  { &hf_rrc_gps_TimingOfCellWanted, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { &hf_rrc_additionalAssistanceDataReq, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { &hf_rrc_environmentCharacterisation, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_EnvironmentCharacterisation },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UE_Positioning_ReportingQuantity_r4(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UE_Positioning_ReportingQuantity_r4, UE_Positioning_ReportingQuantity_r4_sequence);

  return offset;
}


static const per_sequence_t T_fdd_177_sequence[] = {
  { &hf_rrc_primaryCPICH_Info, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PrimaryCPICH_Info },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_fdd_177(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_fdd_177, T_fdd_177_sequence);

  return offset;
}


static const per_sequence_t T_tdd_126_sequence[] = {
  { &hf_rrc_cellAndChannelIdentity, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CellAndChannelIdentity },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_tdd_126(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_tdd_126, T_tdd_126_sequence);

  return offset;
}


static const value_string rrc_T_modeSpecificInfo_143_vals[] = {
  {   0, "fdd" },
  {   1, "tdd" },
  { 0, NULL }
};

static const per_choice_t T_modeSpecificInfo_143_choice[] = {
  {   0, &hf_rrc_fdd_183         , ASN1_NO_EXTENSIONS     , dissect_rrc_T_fdd_177 },
  {   1, &hf_rrc_tdd_134         , ASN1_NO_EXTENSIONS     , dissect_rrc_T_tdd_126 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_modeSpecificInfo_143(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_modeSpecificInfo_143, T_modeSpecificInfo_143_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t T_ueBased_04_sequence[] = {
  { &hf_rrc_cellPosition    , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_ReferenceCellPosition },
  { &hf_rrc_roundTripTime   , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_INTEGER_0_32766 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_ueBased_04(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_ueBased_04, T_ueBased_04_sequence);

  return offset;
}


static const per_sequence_t T_ueAssisted_04_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_ueAssisted_04(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_ueAssisted_04, T_ueAssisted_04_sequence);

  return offset;
}


static const value_string rrc_T_positioningMode_04_vals[] = {
  {   0, "ueBased" },
  {   1, "ueAssisted" },
  { 0, NULL }
};

static const per_choice_t T_positioningMode_04_choice[] = {
  {   0, &hf_rrc_ueBased_04      , ASN1_NO_EXTENSIONS     , dissect_rrc_T_ueBased_04 },
  {   1, &hf_rrc_ueAssisted_04   , ASN1_NO_EXTENSIONS     , dissect_rrc_T_ueAssisted_04 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_positioningMode_04(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_positioningMode_04, T_positioningMode_04_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t T_fdd_170_sequence[] = {
  { &hf_rrc_ip_Spacing      , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_IP_Spacing },
  { &hf_rrc_ip_Length       , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_IP_Length },
  { &hf_rrc_ip_Offset       , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_0_9 },
  { &hf_rrc_seed            , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_0_63 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_fdd_170(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_fdd_170, T_fdd_170_sequence);

  return offset;
}


static const per_sequence_t T_tdd_119_sequence[] = {
  { &hf_rrc_ip_Spacing_TDD  , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_IP_Spacing_TDD },
  { &hf_rrc_ip_slot         , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_0_14 },
  { &hf_rrc_ip_Start        , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_0_4095 },
  { &hf_rrc_ip_PCCPCG       , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_IP_PCCPCH_r4 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_tdd_119(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_tdd_119, T_tdd_119_sequence);

  return offset;
}


static const value_string rrc_T_modeSpecificInfo_136_vals[] = {
  {   0, "fdd" },
  {   1, "tdd" },
  { 0, NULL }
};

static const per_choice_t T_modeSpecificInfo_136_choice[] = {
  {   0, &hf_rrc_fdd_176         , ASN1_NO_EXTENSIONS     , dissect_rrc_T_fdd_170 },
  {   1, &hf_rrc_tdd_127         , ASN1_NO_EXTENSIONS     , dissect_rrc_T_tdd_119 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_modeSpecificInfo_136(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_modeSpecificInfo_136, T_modeSpecificInfo_136_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t UE_Positioning_IPDL_Parameters_r4_sequence[] = {
  { &hf_rrc_modeSpecificInfo_136, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_modeSpecificInfo_136 },
  { &hf_rrc_burstModeParameters, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_BurstModeParameters },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UE_Positioning_IPDL_Parameters_r4(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UE_Positioning_IPDL_Parameters_r4, UE_Positioning_IPDL_Parameters_r4_sequence);

  return offset;
}


static const per_sequence_t UE_Positioning_OTDOA_ReferenceCellInfo_r4_sequence[] = {
  { &hf_rrc_sfn             , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_INTEGER_0_4095 },
  { &hf_rrc_modeSpecificInfo_143, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_modeSpecificInfo_143 },
  { &hf_rrc_frequencyInfo   , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_FrequencyInfo },
  { &hf_rrc_positioningMode_04, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_positioningMode_04 },
  { &hf_rrc_ue_positioning_IPDL_Paremeters_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_Positioning_IPDL_Parameters_r4 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UE_Positioning_OTDOA_ReferenceCellInfo_r4(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UE_Positioning_OTDOA_ReferenceCellInfo_r4, UE_Positioning_OTDOA_ReferenceCellInfo_r4_sequence);

  return offset;
}


static const per_sequence_t T_fdd_173_sequence[] = {
  { &hf_rrc_primaryCPICH_Info, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PrimaryCPICH_Info },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_fdd_173(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_fdd_173, T_fdd_173_sequence);

  return offset;
}


static const per_sequence_t T_tdd_122_sequence[] = {
  { &hf_rrc_cellAndChannelIdentity, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CellAndChannelIdentity },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_tdd_122(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_tdd_122, T_tdd_122_sequence);

  return offset;
}


static const value_string rrc_T_modeSpecificInfo_139_vals[] = {
  {   0, "fdd" },
  {   1, "tdd" },
  { 0, NULL }
};

static const per_choice_t T_modeSpecificInfo_139_choice[] = {
  {   0, &hf_rrc_fdd_179         , ASN1_NO_EXTENSIONS     , dissect_rrc_T_fdd_173 },
  {   1, &hf_rrc_tdd_130         , ASN1_NO_EXTENSIONS     , dissect_rrc_T_tdd_122 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_modeSpecificInfo_139(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_modeSpecificInfo_139, T_modeSpecificInfo_139_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t T_ueBased_01_sequence[] = {
  { &hf_rrc_relativeNorth   , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_INTEGER_M20000_20000 },
  { &hf_rrc_relativeEast    , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_INTEGER_M20000_20000 },
  { &hf_rrc_relativeAltitude, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_INTEGER_M4000_4000 },
  { &hf_rrc_fineSFN_SFN     , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_FineSFN_SFN },
  { &hf_rrc_roundTripTime   , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_INTEGER_0_32766 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_ueBased_01(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_ueBased_01, T_ueBased_01_sequence);

  return offset;
}


static const per_sequence_t T_ueAssisted_01_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_ueAssisted_01(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_ueAssisted_01, T_ueAssisted_01_sequence);

  return offset;
}


static const value_string rrc_T_positioningMode_01_vals[] = {
  {   0, "ueBased" },
  {   1, "ueAssisted" },
  { 0, NULL }
};

static const per_choice_t T_positioningMode_01_choice[] = {
  {   0, &hf_rrc_ueBased_01      , ASN1_NO_EXTENSIONS     , dissect_rrc_T_ueBased_01 },
  {   1, &hf_rrc_ueAssisted_01   , ASN1_NO_EXTENSIONS     , dissect_rrc_T_ueAssisted_01 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_positioningMode_01(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_positioningMode_01, T_positioningMode_01_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t UE_Positioning_OTDOA_NeighbourCellInfo_r4_sequence[] = {
  { &hf_rrc_modeSpecificInfo_139, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_modeSpecificInfo_139 },
  { &hf_rrc_frequencyInfo   , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_FrequencyInfo },
  { &hf_rrc_ue_positioning_IPDL_Paremeters_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_Positioning_IPDL_Parameters_r4 },
  { &hf_rrc_sfn_SFN_RelTimeDifference, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SFN_SFN_RelTimeDifference1 },
  { &hf_rrc_sfn_Offset_Validity, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_SFN_Offset_Validity },
  { &hf_rrc_sfn_SFN_Drift   , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_SFN_SFN_Drift },
  { &hf_rrc_searchWindowSize, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_OTDOA_SearchWindowSize },
  { &hf_rrc_positioningMode_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_positioningMode_01 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UE_Positioning_OTDOA_NeighbourCellInfo_r4(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UE_Positioning_OTDOA_NeighbourCellInfo_r4, UE_Positioning_OTDOA_NeighbourCellInfo_r4_sequence);

  return offset;
}


static const per_sequence_t UE_Positioning_OTDOA_NeighbourCellList_r4_sequence_of[1] = {
  { &hf_rrc_UE_Positioning_OTDOA_NeighbourCellList_r4_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UE_Positioning_OTDOA_NeighbourCellInfo_r4 },
};

static int
dissect_rrc_UE_Positioning_OTDOA_NeighbourCellList_r4(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_UE_Positioning_OTDOA_NeighbourCellList_r4, UE_Positioning_OTDOA_NeighbourCellList_r4_sequence_of,
                                                  1, maxCellMeas, FALSE);

  return offset;
}


static const per_sequence_t UE_Positioning_OTDOA_AssistanceData_r4_sequence[] = {
  { &hf_rrc_ue_positioning_OTDOA_ReferenceCellInfo_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_Positioning_OTDOA_ReferenceCellInfo_r4 },
  { &hf_rrc_ue_positioning_OTDOA_NeighbourCellList_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_Positioning_OTDOA_NeighbourCellList_r4 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UE_Positioning_OTDOA_AssistanceData_r4(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UE_Positioning_OTDOA_AssistanceData_r4, UE_Positioning_OTDOA_AssistanceData_r4_sequence);

  return offset;
}


static const per_sequence_t UE_Positioning_Measurement_r4_sequence[] = {
  { &hf_rrc_ue_positioning_ReportingQuantity_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UE_Positioning_ReportingQuantity_r4 },
  { &hf_rrc_measurementValidity, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MeasurementValidity },
  { &hf_rrc_reportCriteria_20, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UE_Positioning_ReportCriteria },
  { &hf_rrc_ue_positioning_OTDOA_AssistanceData_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_Positioning_OTDOA_AssistanceData_r4 },
  { &hf_rrc_ue_positioning_GPS_AssistanceData, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_Positioning_GPS_AssistanceData },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UE_Positioning_Measurement_r4(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UE_Positioning_Measurement_r4, UE_Positioning_Measurement_r4_sequence);

  return offset;
}


static const per_sequence_t T_fdd_166_sequence[] = {
  { &hf_rrc_ue_RX_TX_TimeDifference, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_fdd_166(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_fdd_166, T_fdd_166_sequence);

  return offset;
}


static const per_sequence_t T_tdd384_43_sequence[] = {
  { &hf_rrc_appliedTA_02    , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_tdd384_43(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_tdd384_43, T_tdd384_43_sequence);

  return offset;
}


static const per_sequence_t T_tdd128_67_sequence[] = {
  { &hf_rrc_t_ADVinfo_02    , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_tdd128_67(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_tdd128_67, T_tdd128_67_sequence);

  return offset;
}


static const value_string rrc_T_tddOption_28_vals[] = {
  {   0, "tdd384" },
  {   1, "tdd128" },
  { 0, NULL }
};

static const per_choice_t T_tddOption_28_choice[] = {
  {   0, &hf_rrc_tdd384_48       , ASN1_NO_EXTENSIONS     , dissect_rrc_T_tdd384_43 },
  {   1, &hf_rrc_tdd128_70       , ASN1_NO_EXTENSIONS     , dissect_rrc_T_tdd128_67 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_tddOption_28(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_tddOption_28, T_tddOption_28_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t T_tdd_115_sequence[] = {
  { &hf_rrc_tddOption_28    , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_tddOption_28 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_tdd_115(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_tdd_115, T_tdd_115_sequence);

  return offset;
}


static const value_string rrc_T_modeSpecificInfo_134_vals[] = {
  {   0, "fdd" },
  {   1, "tdd" },
  { 0, NULL }
};

static const per_choice_t T_modeSpecificInfo_134_choice[] = {
  {   0, &hf_rrc_fdd_172         , ASN1_NO_EXTENSIONS     , dissect_rrc_T_fdd_166 },
  {   1, &hf_rrc_tdd_123         , ASN1_NO_EXTENSIONS     , dissect_rrc_T_tdd_115 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_modeSpecificInfo_134(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_modeSpecificInfo_134, T_modeSpecificInfo_134_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t UE_InternalReportingQuantity_r4_sequence[] = {
  { &hf_rrc_ue_TransmittedPower, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { &hf_rrc_modeSpecificInfo_134, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_modeSpecificInfo_134 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UE_InternalReportingQuantity_r4(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UE_InternalReportingQuantity_r4, UE_InternalReportingQuantity_r4_sequence);

  return offset;
}


static const per_sequence_t UE_InternalMeasurement_r4_sequence[] = {
  { &hf_rrc_ue_InternalMeasQuantity, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_InternalMeasQuantity },
  { &hf_rrc_ue_InternalReportingQuantity_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_InternalReportingQuantity_r4 },
  { &hf_rrc_reportCriteria_19, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UE_InternalReportCriteria },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UE_InternalMeasurement_r4(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UE_InternalMeasurement_r4, UE_InternalMeasurement_r4_sequence);

  return offset;
}


static const value_string rrc_MeasurementType_r4_vals[] = {
  {   0, "intraFrequencyMeasurement" },
  {   1, "interFrequencyMeasurement" },
  {   2, "interRATMeasurement" },
  {   3, "up-Measurement" },
  {   4, "trafficVolumeMeasurement" },
  {   5, "qualityMeasurement" },
  {   6, "ue-InternalMeasurement" },
  { 0, NULL }
};

static const per_choice_t MeasurementType_r4_choice[] = {
  {   0, &hf_rrc_intraFrequencyMeasurement_01, ASN1_NO_EXTENSIONS     , dissect_rrc_IntraFrequencyMeasurement_r4 },
  {   1, &hf_rrc_interFrequencyMeasurement_01, ASN1_NO_EXTENSIONS     , dissect_rrc_InterFrequencyMeasurement_r4 },
  {   2, &hf_rrc_interRATMeasurement_01, ASN1_NO_EXTENSIONS     , dissect_rrc_InterRATMeasurement_r4 },
  {   3, &hf_rrc_up_Measurement  , ASN1_NO_EXTENSIONS     , dissect_rrc_UE_Positioning_Measurement_r4 },
  {   4, &hf_rrc_trafficVolumeMeasurement, ASN1_NO_EXTENSIONS     , dissect_rrc_TrafficVolumeMeasurement },
  {   5, &hf_rrc_qualityMeasurement, ASN1_NO_EXTENSIONS     , dissect_rrc_QualityMeasurement },
  {   6, &hf_rrc_ue_InternalMeasurement_01, ASN1_NO_EXTENSIONS     , dissect_rrc_UE_InternalMeasurement_r4 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_MeasurementType_r4(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_MeasurementType_r4, MeasurementType_r4_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t T_modify_02_sequence[] = {
  { &hf_rrc_measurementType_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MeasurementType_r4 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_modify_02(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_modify_02, T_modify_02_sequence);

  return offset;
}


static const value_string rrc_MeasurementCommand_r4_vals[] = {
  {   0, "setup" },
  {   1, "modify" },
  {   2, "release" },
  { 0, NULL }
};

static const per_choice_t MeasurementCommand_r4_choice[] = {
  {   0, &hf_rrc_setup_01        , ASN1_NO_EXTENSIONS     , dissect_rrc_MeasurementType_r4 },
  {   1, &hf_rrc_modify_02       , ASN1_NO_EXTENSIONS     , dissect_rrc_T_modify_02 },
  {   2, &hf_rrc_release_01      , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_MeasurementCommand_r4(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_MeasurementCommand_r4, MeasurementCommand_r4_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t MeasurementControl_r4_IEs_sequence[] = {
  { &hf_rrc_measurementIdentity, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MeasurementIdentity },
  { &hf_rrc_measurementCommand_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MeasurementCommand_r4 },
  { &hf_rrc_measurementReportingMode, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MeasurementReportingMode },
  { &hf_rrc_additionalMeasurementList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_AdditionalMeasurementID_List },
  { &hf_rrc_dpch_CompressedModeStatusInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DPCH_CompressedModeStatusInfo },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_MeasurementControl_r4_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_MeasurementControl_r4_IEs, MeasurementControl_r4_IEs_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions_59_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_nonCriticalExtensions_59(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_nonCriticalExtensions_59, T_nonCriticalExtensions_59_sequence);

  return offset;
}


static const per_sequence_t T_v5b0NonCriticalExtensions_01_sequence[] = {
  { &hf_rrc_measurementControl_v5b0ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MeasurementControl_v5b0ext_IEs },
  { &hf_rrc_nonCriticalExtensions_59, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_nonCriticalExtensions_59 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v5b0NonCriticalExtensions_01(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v5b0NonCriticalExtensions_01, T_v5b0NonCriticalExtensions_01_sequence);

  return offset;
}


static const per_sequence_t T_v590NonCriticalExtensions_09_sequence[] = {
  { &hf_rrc_measurementControl_v590ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MeasurementControl_v590ext_IEs },
  { &hf_rrc_v5b0NonCriticalExtensions_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v5b0NonCriticalExtensions_01 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v590NonCriticalExtensions_09(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v590NonCriticalExtensions_09, T_v590NonCriticalExtensions_09_sequence);

  return offset;
}


static const per_sequence_t T_v4d0NonCriticalExtensions_03_sequence[] = {
  { &hf_rrc_measurementControl_r4_add_ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_BIT_STRING },
  { &hf_rrc_v590NonCriticalExtensions_09, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v590NonCriticalExtensions_09 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v4d0NonCriticalExtensions_03(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v4d0NonCriticalExtensions_03, T_v4d0NonCriticalExtensions_03_sequence);

  return offset;
}


static const per_sequence_t T_r4_03_sequence[] = {
  { &hf_rrc_measurementControl_r4, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MeasurementControl_r4_IEs },
  { &hf_rrc_v4d0NonCriticalExtensions_03, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v4d0NonCriticalExtensions_03 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_r4_03(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_r4_03, T_r4_03_sequence);

  return offset;
}


static const per_sequence_t Event1d_sequence[] = {
  { &hf_rrc_triggeringCondition, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_TriggeringCondition2 },
  { &hf_rrc_useCIO          , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_BOOLEAN },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_Event1d(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_Event1d, Event1d_sequence);

  return offset;
}



static int
dissect_rrc_ThresholdUsedFrequency_r6(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            -120, 165U, NULL, FALSE);

  return offset;
}


static const per_sequence_t Event1e_r6_sequence[] = {
  { &hf_rrc_triggeringCondition, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TriggeringCondition2 },
  { &hf_rrc_thresholdUsedFrequency_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_ThresholdUsedFrequency_r6 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_Event1e_r6(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_Event1e_r6, Event1e_r6_sequence);

  return offset;
}


static const per_sequence_t Event1f_r6_sequence[] = {
  { &hf_rrc_triggeringCondition_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TriggeringCondition1 },
  { &hf_rrc_thresholdUsedFrequency_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_ThresholdUsedFrequency_r6 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_Event1f_r6(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_Event1f_r6, Event1f_r6_sequence);

  return offset;
}


static const per_sequence_t Event1j_r6_sequence[] = {
  { &hf_rrc_replacementActivationThreshold, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_ReplacementActivationThreshold },
  { &hf_rrc_reportingAmount , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_ReportingAmount },
  { &hf_rrc_reportingInterval, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_ReportingInterval },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_Event1j_r6(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_Event1j_r6, Event1j_r6_sequence);

  return offset;
}


static const value_string rrc_IntraFreqEvent_r6_vals[] = {
  {   0, "e1a" },
  {   1, "e1b" },
  {   2, "e1c" },
  {   3, "e1d" },
  {   4, "e1e" },
  {   5, "e1f" },
  {   6, "e1g" },
  {   7, "e1h" },
  {   8, "e1i" },
  {   9, "e1j" },
  { 0, NULL }
};

static const per_choice_t IntraFreqEvent_r6_choice[] = {
  {   0, &hf_rrc_e1a_01          , ASN1_NO_EXTENSIONS     , dissect_rrc_Event1a_r4 },
  {   1, &hf_rrc_e1b_01          , ASN1_NO_EXTENSIONS     , dissect_rrc_Event1b_r4 },
  {   2, &hf_rrc_e1c             , ASN1_NO_EXTENSIONS     , dissect_rrc_Event1c },
  {   3, &hf_rrc_e1d_01          , ASN1_NO_EXTENSIONS     , dissect_rrc_Event1d },
  {   4, &hf_rrc_e1e_01          , ASN1_NO_EXTENSIONS     , dissect_rrc_Event1e_r6 },
  {   5, &hf_rrc_e1f_01          , ASN1_NO_EXTENSIONS     , dissect_rrc_Event1f_r6 },
  {   6, &hf_rrc_e1g             , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   7, &hf_rrc_e1h_01          , ASN1_NO_EXTENSIONS     , dissect_rrc_ThresholdUsedFrequency_r6 },
  {   8, &hf_rrc_e1i_01          , ASN1_NO_EXTENSIONS     , dissect_rrc_ThresholdUsedFrequency_r6 },
  {   9, &hf_rrc_e1j             , ASN1_NO_EXTENSIONS     , dissect_rrc_Event1j_r6 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_IntraFreqEvent_r6(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_IntraFreqEvent_r6, IntraFreqEvent_r6_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t IntraFreqEventCriteria_r6_sequence[] = {
  { &hf_rrc_event_03        , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_IntraFreqEvent_r6 },
  { &hf_rrc_hysteresis_01   , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_Hysteresis },
  { &hf_rrc_timeToTrigger   , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TimeToTrigger },
  { &hf_rrc_reportingCellStatus, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_ReportingCellStatus },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_IntraFreqEventCriteria_r6(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_IntraFreqEventCriteria_r6, IntraFreqEventCriteria_r6_sequence);

  return offset;
}


static const per_sequence_t IntraFreqEventCriteriaList_r6_sequence_of[1] = {
  { &hf_rrc_IntraFreqEventCriteriaList_r6_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_IntraFreqEventCriteria_r6 },
};

static int
dissect_rrc_IntraFreqEventCriteriaList_r6(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_IntraFreqEventCriteriaList_r6, IntraFreqEventCriteriaList_r6_sequence_of,
                                                  1, maxMeasEvent, FALSE);

  return offset;
}


static const per_sequence_t IntraFreqReportingCriteria_r6_sequence[] = {
  { &hf_rrc_eventCriteriaList_03, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_IntraFreqEventCriteriaList_r6 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_IntraFreqReportingCriteria_r6(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_IntraFreqReportingCriteria_r6, IntraFreqReportingCriteria_r6_sequence);

  return offset;
}


static const value_string rrc_IntraFreqReportCriteria_r6_vals[] = {
  {   0, "intraFreqReportingCriteria" },
  {   1, "periodicalReportingCriteria" },
  {   2, "noReporting" },
  { 0, NULL }
};

static const per_choice_t IntraFreqReportCriteria_r6_choice[] = {
  {   0, &hf_rrc_intraFreqReportingCriteria_04, ASN1_NO_EXTENSIONS     , dissect_rrc_IntraFreqReportingCriteria_r6 },
  {   1, &hf_rrc_periodicalReportingCriteria_01, ASN1_NO_EXTENSIONS     , dissect_rrc_PeriodicalWithReportingCellStatus },
  {   2, &hf_rrc_noReporting     , ASN1_NO_EXTENSIONS     , dissect_rrc_ReportingCellStatusOpt },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_IntraFreqReportCriteria_r6(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_IntraFreqReportCriteria_r6, IntraFreqReportCriteria_r6_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t IntraFrequencyMeasurement_r6_sequence[] = {
  { &hf_rrc_intraFreqCellInfoList_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_IntraFreqCellInfoList_r4 },
  { &hf_rrc_intraFreqMeasQuantity, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_IntraFreqMeasQuantity },
  { &hf_rrc_intraFreqReportingQuantity, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_IntraFreqReportingQuantity },
  { &hf_rrc_measurementValidity, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MeasurementValidity },
  { &hf_rrc_reportCriteria_11, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_IntraFreqReportCriteria_r6 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_IntraFrequencyMeasurement_r6(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_IntraFrequencyMeasurement_r6, IntraFrequencyMeasurement_r6_sequence);

  return offset;
}


static const per_sequence_t NonUsedFreqWList_r6_sequence_of[1] = {
  { &hf_rrc_NonUsedFreqWList_r6_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_W },
};

static int
dissect_rrc_NonUsedFreqWList_r6(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_NonUsedFreqWList_r6, NonUsedFreqWList_r6_sequence_of,
                                                  1, maxFreq, FALSE);

  return offset;
}


static const per_sequence_t Event2a_r6_sequence[] = {
  { &hf_rrc_usedFreqW       , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_W },
  { &hf_rrc_hysteresis      , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_HysteresisInterFreq },
  { &hf_rrc_timeToTrigger   , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TimeToTrigger },
  { &hf_rrc_reportingCellStatus, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_ReportingCellStatus },
  { &hf_rrc_nonUsedFreqParameterList_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_NonUsedFreqWList_r6 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_Event2a_r6(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_Event2a_r6, Event2a_r6_sequence);

  return offset;
}



static int
dissect_rrc_Threshold_r6(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            -120, 0U, NULL, FALSE);

  return offset;
}


static const per_sequence_t NonUsedFreqParameter_r6_sequence[] = {
  { &hf_rrc_nonUsedFreqThreshold_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_Threshold_r6 },
  { &hf_rrc_nonUsedFreqW    , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_W },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_NonUsedFreqParameter_r6(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_NonUsedFreqParameter_r6, NonUsedFreqParameter_r6_sequence);

  return offset;
}


static const per_sequence_t NonUsedFreqParameterList_r6_sequence_of[1] = {
  { &hf_rrc_NonUsedFreqParameterList_r6_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_NonUsedFreqParameter_r6 },
};

static int
dissect_rrc_NonUsedFreqParameterList_r6(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_NonUsedFreqParameterList_r6, NonUsedFreqParameterList_r6_sequence_of,
                                                  1, maxFreq, FALSE);

  return offset;
}


static const per_sequence_t Event2b_r6_sequence[] = {
  { &hf_rrc_usedFreqThreshold_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_Threshold_r6 },
  { &hf_rrc_usedFreqW       , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_W },
  { &hf_rrc_hysteresis      , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_HysteresisInterFreq },
  { &hf_rrc_timeToTrigger   , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TimeToTrigger },
  { &hf_rrc_reportingCellStatus, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_ReportingCellStatus },
  { &hf_rrc_nonUsedFreqParameterList_04, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_NonUsedFreqParameterList_r6 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_Event2b_r6(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_Event2b_r6, Event2b_r6_sequence);

  return offset;
}


static const per_sequence_t Event2c_r6_sequence[] = {
  { &hf_rrc_hysteresis      , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_HysteresisInterFreq },
  { &hf_rrc_timeToTrigger   , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TimeToTrigger },
  { &hf_rrc_reportingCellStatus, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_ReportingCellStatus },
  { &hf_rrc_nonUsedFreqParameterList_04, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_NonUsedFreqParameterList_r6 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_Event2c_r6(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_Event2c_r6, Event2c_r6_sequence);

  return offset;
}


static const per_sequence_t Event2d_r6_sequence[] = {
  { &hf_rrc_usedFreqThreshold_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_Threshold_r6 },
  { &hf_rrc_usedFreqW       , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_W },
  { &hf_rrc_hysteresis      , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_HysteresisInterFreq },
  { &hf_rrc_timeToTrigger   , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TimeToTrigger },
  { &hf_rrc_reportingCellStatus, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_ReportingCellStatus },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_Event2d_r6(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_Event2d_r6, Event2d_r6_sequence);

  return offset;
}


static const per_sequence_t Event2e_r6_sequence[] = {
  { &hf_rrc_hysteresis      , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_HysteresisInterFreq },
  { &hf_rrc_timeToTrigger   , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TimeToTrigger },
  { &hf_rrc_reportingCellStatus, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_ReportingCellStatus },
  { &hf_rrc_nonUsedFreqParameterList_04, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_NonUsedFreqParameterList_r6 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_Event2e_r6(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_Event2e_r6, Event2e_r6_sequence);

  return offset;
}


static const per_sequence_t Event2f_r6_sequence[] = {
  { &hf_rrc_usedFreqThreshold_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_Threshold_r6 },
  { &hf_rrc_usedFreqW       , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_W },
  { &hf_rrc_hysteresis      , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_HysteresisInterFreq },
  { &hf_rrc_timeToTrigger   , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TimeToTrigger },
  { &hf_rrc_reportingCellStatus, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_ReportingCellStatus },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_Event2f_r6(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_Event2f_r6, Event2f_r6_sequence);

  return offset;
}


static const value_string rrc_InterFreqEvent_r6_vals[] = {
  {   0, "event2a" },
  {   1, "event2b" },
  {   2, "event2c" },
  {   3, "event2d" },
  {   4, "event2e" },
  {   5, "event2f" },
  { 0, NULL }
};

static const per_choice_t InterFreqEvent_r6_choice[] = {
  {   0, &hf_rrc_event2a_01      , ASN1_NO_EXTENSIONS     , dissect_rrc_Event2a_r6 },
  {   1, &hf_rrc_event2b_01      , ASN1_NO_EXTENSIONS     , dissect_rrc_Event2b_r6 },
  {   2, &hf_rrc_event2c_01      , ASN1_NO_EXTENSIONS     , dissect_rrc_Event2c_r6 },
  {   3, &hf_rrc_event2d_01      , ASN1_NO_EXTENSIONS     , dissect_rrc_Event2d_r6 },
  {   4, &hf_rrc_event2e_01      , ASN1_NO_EXTENSIONS     , dissect_rrc_Event2e_r6 },
  {   5, &hf_rrc_event2f_01      , ASN1_NO_EXTENSIONS     , dissect_rrc_Event2f_r6 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_InterFreqEvent_r6(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_InterFreqEvent_r6, InterFreqEvent_r6_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t InterFreqEventList_r6_sequence_of[1] = {
  { &hf_rrc_InterFreqEventList_r6_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_InterFreqEvent_r6 },
};

static int
dissect_rrc_InterFreqEventList_r6(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_InterFreqEventList_r6, InterFreqEventList_r6_sequence_of,
                                                  1, maxMeasEvent, FALSE);

  return offset;
}


static const per_sequence_t InterFreqReportingCriteria_r6_sequence[] = {
  { &hf_rrc_interFreqEventList_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_InterFreqEventList_r6 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_InterFreqReportingCriteria_r6(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_InterFreqReportingCriteria_r6, InterFreqReportingCriteria_r6_sequence);

  return offset;
}


static const value_string rrc_InterFreqReportCriteria_r6_vals[] = {
  {   0, "intraFreqReportingCriteria" },
  {   1, "interFreqReportingCriteria" },
  {   2, "periodicalReportingCriteria" },
  {   3, "noReporting" },
  { 0, NULL }
};

static const per_choice_t InterFreqReportCriteria_r6_choice[] = {
  {   0, &hf_rrc_intraFreqReportingCriteria_04, ASN1_NO_EXTENSIONS     , dissect_rrc_IntraFreqReportingCriteria_r6 },
  {   1, &hf_rrc_interFreqReportingCriteria_02, ASN1_NO_EXTENSIONS     , dissect_rrc_InterFreqReportingCriteria_r6 },
  {   2, &hf_rrc_periodicalReportingCriteria_01, ASN1_NO_EXTENSIONS     , dissect_rrc_PeriodicalWithReportingCellStatus },
  {   3, &hf_rrc_noReporting     , ASN1_NO_EXTENSIONS     , dissect_rrc_ReportingCellStatusOpt },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_InterFreqReportCriteria_r6(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_InterFreqReportCriteria_r6, InterFreqReportCriteria_r6_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t InterFrequencyMeasurement_r6_sequence[] = {
  { &hf_rrc_interFreqCellInfoList_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_InterFreqCellInfoList_r4 },
  { &hf_rrc_interFreqMeasQuantity, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_InterFreqMeasQuantity },
  { &hf_rrc_interFreqReportingQuantity, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_InterFreqReportingQuantity },
  { &hf_rrc_measurementValidity, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MeasurementValidity },
  { &hf_rrc_interFreqSetUpdate, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_AutonomousUpdateMode },
  { &hf_rrc_reportCriteria_03, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_InterFreqReportCriteria_r6 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_InterFrequencyMeasurement_r6(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_InterFrequencyMeasurement_r6, InterFrequencyMeasurement_r6_sequence);

  return offset;
}


static const per_sequence_t InterRATCellInfoList_r6_sequence[] = {
  { &hf_rrc_removedInterRATCellList, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RemovedInterRATCellList },
  { &hf_rrc_newInterRATCellList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_NewInterRATCellList },
  { &hf_rrc_cellsForInterRATMeasList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CellsForInterRATMeasList },
  { &hf_rrc_interRATCellInfoIndication_r6, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_InterRATCellInfoIndication },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_InterRATCellInfoList_r6(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_InterRATCellInfoList_r6, InterRATCellInfoList_r6_sequence);

  return offset;
}


static const per_sequence_t InterRATMeasurement_r6_sequence[] = {
  { &hf_rrc_interRATCellInfoList_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_InterRATCellInfoList_r6 },
  { &hf_rrc_interRATMeasQuantity, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_InterRATMeasQuantity },
  { &hf_rrc_interRATReportingQuantity, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_InterRATReportingQuantity },
  { &hf_rrc_reportCriteria_08, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_InterRATReportCriteria },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_InterRATMeasurement_r6(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_InterRATMeasurement_r6, InterRATMeasurement_r6_sequence);

  return offset;
}


static const value_string rrc_MeasurementType_r6_vals[] = {
  {   0, "intraFrequencyMeasurement" },
  {   1, "interFrequencyMeasurement" },
  {   2, "interRATMeasurement" },
  {   3, "up-Measurement" },
  {   4, "trafficVolumeMeasurement" },
  {   5, "qualityMeasurement" },
  {   6, "ue-InternalMeasurement" },
  { 0, NULL }
};

static const per_choice_t MeasurementType_r6_choice[] = {
  {   0, &hf_rrc_intraFrequencyMeasurement_02, ASN1_NO_EXTENSIONS     , dissect_rrc_IntraFrequencyMeasurement_r6 },
  {   1, &hf_rrc_interFrequencyMeasurement_02, ASN1_NO_EXTENSIONS     , dissect_rrc_InterFrequencyMeasurement_r6 },
  {   2, &hf_rrc_interRATMeasurement_02, ASN1_NO_EXTENSIONS     , dissect_rrc_InterRATMeasurement_r6 },
  {   3, &hf_rrc_up_Measurement  , ASN1_NO_EXTENSIONS     , dissect_rrc_UE_Positioning_Measurement_r4 },
  {   4, &hf_rrc_trafficVolumeMeasurement, ASN1_NO_EXTENSIONS     , dissect_rrc_TrafficVolumeMeasurement },
  {   5, &hf_rrc_qualityMeasurement, ASN1_NO_EXTENSIONS     , dissect_rrc_QualityMeasurement },
  {   6, &hf_rrc_ue_InternalMeasurement_01, ASN1_NO_EXTENSIONS     , dissect_rrc_UE_InternalMeasurement_r4 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_MeasurementType_r6(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_MeasurementType_r6, MeasurementType_r6_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t T_modify_03_sequence[] = {
  { &hf_rrc_measurementType_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MeasurementType_r6 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_modify_03(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_modify_03, T_modify_03_sequence);

  return offset;
}


static const value_string rrc_MeasurementCommand_r6_vals[] = {
  {   0, "setup" },
  {   1, "modify" },
  {   2, "release" },
  { 0, NULL }
};

static const per_choice_t MeasurementCommand_r6_choice[] = {
  {   0, &hf_rrc_setup_02        , ASN1_NO_EXTENSIONS     , dissect_rrc_MeasurementType_r6 },
  {   1, &hf_rrc_modify_03       , ASN1_NO_EXTENSIONS     , dissect_rrc_T_modify_03 },
  {   2, &hf_rrc_release_01      , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_MeasurementCommand_r6(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_MeasurementCommand_r6, MeasurementCommand_r6_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t MeasurementControl_r6_IEs_sequence[] = {
  { &hf_rrc_measurementIdentity, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MeasurementIdentity },
  { &hf_rrc_measurementCommand_02, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MeasurementCommand_r6 },
  { &hf_rrc_measurementReportingMode, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MeasurementReportingMode },
  { &hf_rrc_additionalMeasurementList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_AdditionalMeasurementID_List },
  { &hf_rrc_dpch_CompressedModeStatusInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DPCH_CompressedModeStatusInfo },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_MeasurementControl_r6_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_MeasurementControl_r6_IEs, MeasurementControl_r6_IEs_sequence);

  return offset;
}


static const per_sequence_t MeasurementControl_v6a0ext_IEs_sequence[] = {
  { &hf_rrc_intraFreqReportingCriteria_1b_r5, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_IntraFreqReportingCriteria_1b_r5 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_MeasurementControl_v6a0ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_MeasurementControl_v6a0ext_IEs, MeasurementControl_v6a0ext_IEs_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions_60_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_nonCriticalExtensions_60(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_nonCriticalExtensions_60, T_nonCriticalExtensions_60_sequence);

  return offset;
}


static const per_sequence_t T_v6a0NonCriticalExtensions_sequence[] = {
  { &hf_rrc_measurementControl_v6a0ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MeasurementControl_v6a0ext_IEs },
  { &hf_rrc_nonCriticalExtensions_60, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_nonCriticalExtensions_60 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v6a0NonCriticalExtensions(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v6a0NonCriticalExtensions, T_v6a0NonCriticalExtensions_sequence);

  return offset;
}


static const per_sequence_t T_r6_05_sequence[] = {
  { &hf_rrc_measurementControl_r6, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MeasurementControl_r6_IEs },
  { &hf_rrc_v6a0NonCriticalExtensions, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v6a0NonCriticalExtensions },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_r6_05(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_r6_05, T_r6_05_sequence);

  return offset;
}


static const per_sequence_t Event1b_r7_sequence[] = {
  { &hf_rrc_triggeringCondition_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TriggeringCondition1 },
  { &hf_rrc_reportingRange  , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_ReportingRange },
  { &hf_rrc_forbiddenAffectCellList_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_ForbiddenAffectCellList_r4 },
  { &hf_rrc_w               , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_W },
  { &hf_rrc_periodicReportingInfo_1b, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_PeriodicReportingInfo_1b },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_Event1b_r7(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_Event1b_r7, Event1b_r7_sequence);

  return offset;
}


static const value_string rrc_IntraFreqEvent_r7_vals[] = {
  {   0, "e1a" },
  {   1, "e1b" },
  {   2, "e1c" },
  {   3, "e1d" },
  {   4, "e1e" },
  {   5, "e1f" },
  {   6, "e1g" },
  {   7, "e1h" },
  {   8, "e1i" },
  {   9, "e1j" },
  { 0, NULL }
};

static const per_choice_t IntraFreqEvent_r7_choice[] = {
  {   0, &hf_rrc_e1a_01          , ASN1_NO_EXTENSIONS     , dissect_rrc_Event1a_r4 },
  {   1, &hf_rrc_e1b_03          , ASN1_NO_EXTENSIONS     , dissect_rrc_Event1b_r7 },
  {   2, &hf_rrc_e1c             , ASN1_NO_EXTENSIONS     , dissect_rrc_Event1c },
  {   3, &hf_rrc_e1d_01          , ASN1_NO_EXTENSIONS     , dissect_rrc_Event1d },
  {   4, &hf_rrc_e1e_01          , ASN1_NO_EXTENSIONS     , dissect_rrc_Event1e_r6 },
  {   5, &hf_rrc_e1f_01          , ASN1_NO_EXTENSIONS     , dissect_rrc_Event1f_r6 },
  {   6, &hf_rrc_e1g             , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   7, &hf_rrc_e1h_01          , ASN1_NO_EXTENSIONS     , dissect_rrc_ThresholdUsedFrequency_r6 },
  {   8, &hf_rrc_e1i_01          , ASN1_NO_EXTENSIONS     , dissect_rrc_ThresholdUsedFrequency_r6 },
  {   9, &hf_rrc_e1j             , ASN1_NO_EXTENSIONS     , dissect_rrc_Event1j_r6 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_IntraFreqEvent_r7(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_IntraFreqEvent_r7, IntraFreqEvent_r7_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t IntraFreqEventCriteria_r7_sequence[] = {
  { &hf_rrc_event_04        , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_IntraFreqEvent_r7 },
  { &hf_rrc_hysteresis_01   , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_Hysteresis },
  { &hf_rrc_timeToTrigger   , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TimeToTrigger },
  { &hf_rrc_reportingCellStatus, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_ReportingCellStatus },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_IntraFreqEventCriteria_r7(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_IntraFreqEventCriteria_r7, IntraFreqEventCriteria_r7_sequence);

  return offset;
}


static const per_sequence_t IntraFreqEventCriteriaList_r7_sequence_of[1] = {
  { &hf_rrc_IntraFreqEventCriteriaList_r7_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_IntraFreqEventCriteria_r7 },
};

static int
dissect_rrc_IntraFreqEventCriteriaList_r7(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_IntraFreqEventCriteriaList_r7, IntraFreqEventCriteriaList_r7_sequence_of,
                                                  1, maxMeasEvent, FALSE);

  return offset;
}


static const per_sequence_t IntraFreqReportingCriteria_r7_sequence[] = {
  { &hf_rrc_eventCriteriaList_04, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_IntraFreqEventCriteriaList_r7 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_IntraFreqReportingCriteria_r7(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_IntraFreqReportingCriteria_r7, IntraFreqReportingCriteria_r7_sequence);

  return offset;
}


static const value_string rrc_IntraFreqReportCriteria_r7_vals[] = {
  {   0, "intraFreqReportingCriteria" },
  {   1, "periodicalReportingCriteria" },
  {   2, "noReporting" },
  { 0, NULL }
};

static const per_choice_t IntraFreqReportCriteria_r7_choice[] = {
  {   0, &hf_rrc_intraFreqReportingCriteria_05, ASN1_NO_EXTENSIONS     , dissect_rrc_IntraFreqReportingCriteria_r7 },
  {   1, &hf_rrc_periodicalReportingCriteria_01, ASN1_NO_EXTENSIONS     , dissect_rrc_PeriodicalWithReportingCellStatus },
  {   2, &hf_rrc_noReporting     , ASN1_NO_EXTENSIONS     , dissect_rrc_ReportingCellStatusOpt },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_IntraFreqReportCriteria_r7(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_IntraFreqReportCriteria_r7, IntraFreqReportCriteria_r7_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t IntraFrequencyMeasurement_r7_sequence[] = {
  { &hf_rrc_intraFreqCellInfoList_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_IntraFreqCellInfoList_r4 },
  { &hf_rrc_intraFreqMeasQuantity, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_IntraFreqMeasQuantity },
  { &hf_rrc_intraFreqReportingQuantity, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_IntraFreqReportingQuantity },
  { &hf_rrc_measurementValidity, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MeasurementValidity },
  { &hf_rrc_reportCriteria_12, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_IntraFreqReportCriteria_r7 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_IntraFrequencyMeasurement_r7(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_IntraFrequencyMeasurement_r7, IntraFrequencyMeasurement_r7_sequence);

  return offset;
}


static const value_string rrc_InterFreqReportCriteria_r7_vals[] = {
  {   0, "intraFreqReportingCriteria" },
  {   1, "interFreqReportingCriteria" },
  {   2, "periodicalReportingCriteria" },
  {   3, "noReporting" },
  { 0, NULL }
};

static const per_choice_t InterFreqReportCriteria_r7_choice[] = {
  {   0, &hf_rrc_intraFreqReportingCriteria_05, ASN1_NO_EXTENSIONS     , dissect_rrc_IntraFreqReportingCriteria_r7 },
  {   1, &hf_rrc_interFreqReportingCriteria_02, ASN1_NO_EXTENSIONS     , dissect_rrc_InterFreqReportingCriteria_r6 },
  {   2, &hf_rrc_periodicalReportingCriteria_01, ASN1_NO_EXTENSIONS     , dissect_rrc_PeriodicalWithReportingCellStatus },
  {   3, &hf_rrc_noReporting     , ASN1_NO_EXTENSIONS     , dissect_rrc_ReportingCellStatusOpt },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_InterFreqReportCriteria_r7(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_InterFreqReportCriteria_r7, InterFreqReportCriteria_r7_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t InterFrequencyMeasurement_r7_sequence[] = {
  { &hf_rrc_interFreqCellInfoList_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_InterFreqCellInfoList_r4 },
  { &hf_rrc_interFreqMeasQuantity, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_InterFreqMeasQuantity },
  { &hf_rrc_interFreqReportingQuantity, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_InterFreqReportingQuantity },
  { &hf_rrc_measurementValidity, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MeasurementValidity },
  { &hf_rrc_interFreqSetUpdate, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_AutonomousUpdateMode },
  { &hf_rrc_reportCriteria_04, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_InterFreqReportCriteria_r7 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_InterFrequencyMeasurement_r7(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_InterFrequencyMeasurement_r7, InterFrequencyMeasurement_r7_sequence);

  return offset;
}


static const value_string rrc_T_velocityRequested_vals[] = {
  {   0, "true" },
  { 0, NULL }
};


static int
dissect_rrc_T_velocityRequested(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t UE_Positioning_ReportingQuantity_r7_sequence[] = {
  { &hf_rrc_methodType      , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UE_Positioning_MethodType },
  { &hf_rrc_positioningMethod, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PositioningMethod },
  { &hf_rrc_horizontalAccuracy, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_Positioning_Accuracy },
  { &hf_rrc_verticalAccuracy, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_Positioning_Accuracy },
  { &hf_rrc_gps_TimingOfCellWanted, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { &hf_rrc_additionalAssistanceDataReq, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { &hf_rrc_environmentCharacterisation, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_EnvironmentCharacterisation },
  { &hf_rrc_velocityRequested, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_velocityRequested },
  { &hf_rrc_gANSSPositioningMethods, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_BIT_STRING_SIZE_16 },
  { &hf_rrc_gANSSTimingOfCellWanted, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_BIT_STRING_SIZE_8 },
  { &hf_rrc_gANSSCarrierPhaseMeasurementRequested, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_BIT_STRING_SIZE_8 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UE_Positioning_ReportingQuantity_r7(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UE_Positioning_ReportingQuantity_r7, UE_Positioning_ReportingQuantity_r7_sequence);

  return offset;
}


static const value_string rrc_ThresholdSFN_GPS_TOW_us_vals[] = {
  {   0, "us1" },
  {   1, "us2" },
  {   2, "us3" },
  {   3, "us5" },
  {   4, "us10" },
  {   5, "us20" },
  {   6, "us50" },
  {   7, "us100" },
  { 0, NULL }
};


static int
dissect_rrc_ThresholdSFN_GPS_TOW_us(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     8, NULL, FALSE, 0, NULL);

  return offset;
}


static const value_string rrc_ThresholdSFN_GANSS_TOW_vals[] = {
  {   0, "us1" },
  {   1, "us2" },
  {   2, "us3" },
  {   3, "us5" },
  {   4, "us10" },
  {   5, "us20" },
  {   6, "us50" },
  {   7, "us100" },
  { 0, NULL }
};


static int
dissect_rrc_ThresholdSFN_GANSS_TOW(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     8, NULL, FALSE, 0, NULL);

  return offset;
}


static const value_string rrc_UE_Positioning_EventSpecificInfo_r7_vals[] = {
  {   0, "e7a" },
  {   1, "e7b" },
  {   2, "e7c" },
  {   3, "e7d" },
  { 0, NULL }
};

static const per_choice_t UE_Positioning_EventSpecificInfo_r7_choice[] = {
  {   0, &hf_rrc_e7a             , ASN1_NO_EXTENSIONS     , dissect_rrc_ThresholdPositionChange },
  {   1, &hf_rrc_e7b             , ASN1_NO_EXTENSIONS     , dissect_rrc_ThresholdSFN_SFN_Change },
  {   2, &hf_rrc_e7c_01          , ASN1_NO_EXTENSIONS     , dissect_rrc_ThresholdSFN_GPS_TOW_us },
  {   3, &hf_rrc_e7d             , ASN1_NO_EXTENSIONS     , dissect_rrc_ThresholdSFN_GANSS_TOW },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_UE_Positioning_EventSpecificInfo_r7(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_UE_Positioning_EventSpecificInfo_r7, UE_Positioning_EventSpecificInfo_r7_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t UE_Positioning_EventParam_r7_sequence[] = {
  { &hf_rrc_reportingAmount , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_ReportingAmount },
  { &hf_rrc_reportFirstFix  , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { &hf_rrc_measurementInterval, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UE_Positioning_MeasurementInterval },
  { &hf_rrc_eventSpecificInfo_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UE_Positioning_EventSpecificInfo_r7 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UE_Positioning_EventParam_r7(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UE_Positioning_EventParam_r7, UE_Positioning_EventParam_r7_sequence);

  return offset;
}


static const per_sequence_t UE_Positioning_EventParamList_r7_sequence_of[1] = {
  { &hf_rrc_UE_Positioning_EventParamList_r7_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UE_Positioning_EventParam_r7 },
};

static int
dissect_rrc_UE_Positioning_EventParamList_r7(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_UE_Positioning_EventParamList_r7, UE_Positioning_EventParamList_r7_sequence_of,
                                                  1, maxMeasEvent, FALSE);

  return offset;
}


static const value_string rrc_UE_Positioning_ReportCriteria_r7_vals[] = {
  {   0, "ue-positioning-ReportingCriteria" },
  {   1, "periodicalReportingCriteria" },
  {   2, "noReporting" },
  { 0, NULL }
};

static const per_choice_t UE_Positioning_ReportCriteria_r7_choice[] = {
  {   0, &hf_rrc_ue_positioning_ReportingCriteria_01, ASN1_NO_EXTENSIONS     , dissect_rrc_UE_Positioning_EventParamList_r7 },
  {   1, &hf_rrc_periodicalReportingCriteria, ASN1_NO_EXTENSIONS     , dissect_rrc_PeriodicalReportingCriteria },
  {   2, &hf_rrc_noReporting_02  , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_UE_Positioning_ReportCriteria_r7(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_UE_Positioning_ReportCriteria_r7, UE_Positioning_ReportCriteria_r7_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t T_fdd_179_sequence[] = {
  { &hf_rrc_primaryCPICH_Info, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PrimaryCPICH_Info },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_fdd_179(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_fdd_179, T_fdd_179_sequence);

  return offset;
}


static const per_sequence_t T_tdd_128_sequence[] = {
  { &hf_rrc_cellAndChannelIdentity, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CellAndChannelIdentity },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_tdd_128(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_tdd_128, T_tdd_128_sequence);

  return offset;
}


static const value_string rrc_T_modeSpecificInfo_145_vals[] = {
  {   0, "fdd" },
  {   1, "tdd" },
  { 0, NULL }
};

static const per_choice_t T_modeSpecificInfo_145_choice[] = {
  {   0, &hf_rrc_fdd_185         , ASN1_NO_EXTENSIONS     , dissect_rrc_T_fdd_179 },
  {   1, &hf_rrc_tdd_136         , ASN1_NO_EXTENSIONS     , dissect_rrc_T_tdd_128 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_modeSpecificInfo_145(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_modeSpecificInfo_145, T_modeSpecificInfo_145_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t T_ueBased_05_sequence[] = {
  { &hf_rrc_cellPosition    , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_ReferenceCellPosition },
  { &hf_rrc_roundTripTime   , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_INTEGER_0_32766 },
  { &hf_rrc_roundTripTimeExtension, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_INTEGER_0_70274 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_ueBased_05(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_ueBased_05, T_ueBased_05_sequence);

  return offset;
}


static const per_sequence_t T_ueAssisted_05_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_ueAssisted_05(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_ueAssisted_05, T_ueAssisted_05_sequence);

  return offset;
}


static const value_string rrc_T_positioningMode_05_vals[] = {
  {   0, "ueBased" },
  {   1, "ueAssisted" },
  { 0, NULL }
};

static const per_choice_t T_positioningMode_05_choice[] = {
  {   0, &hf_rrc_ueBased_05      , ASN1_NO_EXTENSIONS     , dissect_rrc_T_ueBased_05 },
  {   1, &hf_rrc_ueAssisted_05   , ASN1_NO_EXTENSIONS     , dissect_rrc_T_ueAssisted_05 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_positioningMode_05(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_positioningMode_05, T_positioningMode_05_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t UE_Positioning_OTDOA_ReferenceCellInfo_r7_sequence[] = {
  { &hf_rrc_sfn             , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_INTEGER_0_4095 },
  { &hf_rrc_modeSpecificInfo_145, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_modeSpecificInfo_145 },
  { &hf_rrc_frequencyInfo   , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_FrequencyInfo },
  { &hf_rrc_positioningMode_05, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_positioningMode_05 },
  { &hf_rrc_ue_positioning_IPDL_Paremeters_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_Positioning_IPDL_Parameters_r4 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UE_Positioning_OTDOA_ReferenceCellInfo_r7(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UE_Positioning_OTDOA_ReferenceCellInfo_r7, UE_Positioning_OTDOA_ReferenceCellInfo_r7_sequence);

  return offset;
}


static const per_sequence_t T_fdd_175_sequence[] = {
  { &hf_rrc_primaryCPICH_Info, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PrimaryCPICH_Info },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_fdd_175(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_fdd_175, T_fdd_175_sequence);

  return offset;
}


static const per_sequence_t T_tdd_124_sequence[] = {
  { &hf_rrc_cellAndChannelIdentity, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CellAndChannelIdentity },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_tdd_124(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_tdd_124, T_tdd_124_sequence);

  return offset;
}


static const value_string rrc_T_modeSpecificInfo_141_vals[] = {
  {   0, "fdd" },
  {   1, "tdd" },
  { 0, NULL }
};

static const per_choice_t T_modeSpecificInfo_141_choice[] = {
  {   0, &hf_rrc_fdd_181         , ASN1_NO_EXTENSIONS     , dissect_rrc_T_fdd_175 },
  {   1, &hf_rrc_tdd_132         , ASN1_NO_EXTENSIONS     , dissect_rrc_T_tdd_124 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_modeSpecificInfo_141(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_modeSpecificInfo_141, T_modeSpecificInfo_141_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t T_ueBased_02_sequence[] = {
  { &hf_rrc_relativeNorth   , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_INTEGER_M20000_20000 },
  { &hf_rrc_relativeEast    , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_INTEGER_M20000_20000 },
  { &hf_rrc_relativeAltitude, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_INTEGER_M4000_4000 },
  { &hf_rrc_fineSFN_SFN     , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_FineSFN_SFN },
  { &hf_rrc_roundTripTime   , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_INTEGER_0_32766 },
  { &hf_rrc_roundTripTimeExtension, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_INTEGER_0_70274 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_ueBased_02(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_ueBased_02, T_ueBased_02_sequence);

  return offset;
}


static const per_sequence_t T_ueAssisted_02_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_ueAssisted_02(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_ueAssisted_02, T_ueAssisted_02_sequence);

  return offset;
}


static const value_string rrc_T_positioningMode_02_vals[] = {
  {   0, "ueBased" },
  {   1, "ueAssisted" },
  { 0, NULL }
};

static const per_choice_t T_positioningMode_02_choice[] = {
  {   0, &hf_rrc_ueBased_02      , ASN1_NO_EXTENSIONS     , dissect_rrc_T_ueBased_02 },
  {   1, &hf_rrc_ueAssisted_02   , ASN1_NO_EXTENSIONS     , dissect_rrc_T_ueAssisted_02 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_positioningMode_02(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_positioningMode_02, T_positioningMode_02_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t UE_Positioning_OTDOA_NeighbourCellInfo_r7_sequence[] = {
  { &hf_rrc_modeSpecificInfo_141, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_modeSpecificInfo_141 },
  { &hf_rrc_frequencyInfo   , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_FrequencyInfo },
  { &hf_rrc_ue_positioning_IPDL_Paremeters_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_Positioning_IPDL_Parameters_r4 },
  { &hf_rrc_sfn_SFN_RelTimeDifference, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SFN_SFN_RelTimeDifference1 },
  { &hf_rrc_sfn_Offset_Validity, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_SFN_Offset_Validity },
  { &hf_rrc_sfn_SFN_Drift   , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_SFN_SFN_Drift },
  { &hf_rrc_searchWindowSize, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_OTDOA_SearchWindowSize },
  { &hf_rrc_positioningMode_02, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_positioningMode_02 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UE_Positioning_OTDOA_NeighbourCellInfo_r7(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UE_Positioning_OTDOA_NeighbourCellInfo_r7, UE_Positioning_OTDOA_NeighbourCellInfo_r7_sequence);

  return offset;
}


static const per_sequence_t UE_Positioning_OTDOA_NeighbourCellList_r7_sequence_of[1] = {
  { &hf_rrc_UE_Positioning_OTDOA_NeighbourCellList_r7_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UE_Positioning_OTDOA_NeighbourCellInfo_r7 },
};

static int
dissect_rrc_UE_Positioning_OTDOA_NeighbourCellList_r7(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_UE_Positioning_OTDOA_NeighbourCellList_r7, UE_Positioning_OTDOA_NeighbourCellList_r7_sequence_of,
                                                  1, maxCellMeas, FALSE);

  return offset;
}


static const per_sequence_t UE_Positioning_OTDOA_AssistanceData_r7_sequence[] = {
  { &hf_rrc_ue_positioning_OTDOA_ReferenceCellInfo_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_Positioning_OTDOA_ReferenceCellInfo_r7 },
  { &hf_rrc_ue_positioning_OTDOA_NeighbourCellList_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_Positioning_OTDOA_NeighbourCellList_r7 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UE_Positioning_OTDOA_AssistanceData_r7(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UE_Positioning_OTDOA_AssistanceData_r7, UE_Positioning_OTDOA_AssistanceData_r7_sequence);

  return offset;
}


static const per_sequence_t UE_Positioning_GPS_ReferenceTime_r7_sequence[] = {
  { &hf_rrc_gps_Week        , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_0_1023 },
  { &hf_rrc_gps_tow_1msec   , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_GPS_TOW_1msec },
  { &hf_rrc_utran_GPSReferenceTime, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UTRAN_GPSReferenceTime },
  { &hf_rrc_ue_Positioning_GPS_ReferenceTimeUncertainty, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_Positioning_GPS_ReferenceTimeUncertainty },
  { &hf_rrc_sfn_tow_Uncertainty, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_SFN_TOW_Uncertainty },
  { &hf_rrc_utran_GPS_DriftRate, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UTRAN_GPS_DriftRate },
  { &hf_rrc_gps_TOW_AssistList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_GPS_TOW_AssistList },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UE_Positioning_GPS_ReferenceTime_r7(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UE_Positioning_GPS_ReferenceTime_r7, UE_Positioning_GPS_ReferenceTime_r7_sequence);

  return offset;
}


static const per_sequence_t UE_Positioning_GPS_AcquisitionAssistance_r7_sequence[] = {
  { &hf_rrc_gps_ReferenceTime, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_GPS_TOW_1msec },
  { &hf_rrc_utran_GPSReferenceTime, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UTRAN_GPSReferenceTime },
  { &hf_rrc_ue_Positioning_GPS_ReferenceTimeUncertainty, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_Positioning_GPS_ReferenceTimeUncertainty },
  { &hf_rrc_satelliteInformationList_03, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_AcquisitionSatInfoList },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UE_Positioning_GPS_AcquisitionAssistance_r7(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UE_Positioning_GPS_AcquisitionAssistance_r7, UE_Positioning_GPS_AcquisitionAssistance_r7_sequence);

  return offset;
}


static const per_sequence_t UE_Positioning_GPS_AssistanceData_r7_sequence[] = {
  { &hf_rrc_ue_positioning_GPS_ReferenceTime_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_Positioning_GPS_ReferenceTime_r7 },
  { &hf_rrc_ue_positioning_GPS_ReferenceLocation, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_ReferenceLocation },
  { &hf_rrc_ue_positioning_GPS_DGPS_Corrections, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_Positioning_GPS_DGPS_Corrections },
  { &hf_rrc_ue_positioning_GPS_NavigationModel, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_Positioning_GPS_NavigationModel },
  { &hf_rrc_ue_positioning_GPS_IonosphericModel, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_Positioning_GPS_IonosphericModel },
  { &hf_rrc_ue_positioning_GPS_UTC_Model, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_Positioning_GPS_UTC_Model },
  { &hf_rrc_ue_positioning_GPS_Almanac, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_Positioning_GPS_Almanac },
  { &hf_rrc_ue_positioning_GPS_AcquisitionAssistance_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_Positioning_GPS_AcquisitionAssistance_r7 },
  { &hf_rrc_ue_positioning_GPS_Real_timeIntegrity, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_BadSatList },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UE_Positioning_GPS_AssistanceData_r7(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UE_Positioning_GPS_AssistanceData_r7, UE_Positioning_GPS_AssistanceData_r7_sequence);

  return offset;
}


static const per_sequence_t UE_Positioning_Measurement_r7_sequence[] = {
  { &hf_rrc_ue_positioning_ReportingQuantity_02, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UE_Positioning_ReportingQuantity_r7 },
  { &hf_rrc_measurementValidity, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MeasurementValidity },
  { &hf_rrc_reportCriteria_21, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UE_Positioning_ReportCriteria_r7 },
  { &hf_rrc_ue_positioning_OTDOA_AssistanceData_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_Positioning_OTDOA_AssistanceData_r7 },
  { &hf_rrc_ue_positioning_GPS_AssistanceData_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_Positioning_GPS_AssistanceData_r7 },
  { &hf_rrc_ue_positioning_GANSS_AssistanceData, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_Positioning_GANSS_AssistanceData },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UE_Positioning_Measurement_r7(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UE_Positioning_Measurement_r7, UE_Positioning_Measurement_r7_sequence);

  return offset;
}


static const value_string rrc_MeasurementType_r7_vals[] = {
  {   0, "intraFrequencyMeasurement" },
  {   1, "interFrequencyMeasurement" },
  {   2, "interRATMeasurement" },
  {   3, "up-Measurement" },
  {   4, "trafficVolumeMeasurement" },
  {   5, "qualityMeasurement" },
  {   6, "ue-InternalMeasurement" },
  { 0, NULL }
};

static const per_choice_t MeasurementType_r7_choice[] = {
  {   0, &hf_rrc_intraFrequencyMeasurement_03, ASN1_NO_EXTENSIONS     , dissect_rrc_IntraFrequencyMeasurement_r7 },
  {   1, &hf_rrc_interFrequencyMeasurement_03, ASN1_NO_EXTENSIONS     , dissect_rrc_InterFrequencyMeasurement_r7 },
  {   2, &hf_rrc_interRATMeasurement_02, ASN1_NO_EXTENSIONS     , dissect_rrc_InterRATMeasurement_r6 },
  {   3, &hf_rrc_up_Measurement_01, ASN1_NO_EXTENSIONS     , dissect_rrc_UE_Positioning_Measurement_r7 },
  {   4, &hf_rrc_trafficVolumeMeasurement, ASN1_NO_EXTENSIONS     , dissect_rrc_TrafficVolumeMeasurement },
  {   5, &hf_rrc_qualityMeasurement, ASN1_NO_EXTENSIONS     , dissect_rrc_QualityMeasurement },
  {   6, &hf_rrc_ue_InternalMeasurement_01, ASN1_NO_EXTENSIONS     , dissect_rrc_UE_InternalMeasurement_r4 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_MeasurementType_r7(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_MeasurementType_r7, MeasurementType_r7_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t T_modify_04_sequence[] = {
  { &hf_rrc_measurementType_03, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MeasurementType_r7 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_modify_04(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_modify_04, T_modify_04_sequence);

  return offset;
}


static const value_string rrc_MeasurementCommand_r7_vals[] = {
  {   0, "setup" },
  {   1, "modify" },
  {   2, "release" },
  { 0, NULL }
};

static const per_choice_t MeasurementCommand_r7_choice[] = {
  {   0, &hf_rrc_setup_03        , ASN1_NO_EXTENSIONS     , dissect_rrc_MeasurementType_r7 },
  {   1, &hf_rrc_modify_04       , ASN1_NO_EXTENSIONS     , dissect_rrc_T_modify_04 },
  {   2, &hf_rrc_release_01      , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_MeasurementCommand_r7(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_MeasurementCommand_r7, MeasurementCommand_r7_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t MeasurementControl_r7_IEs_sequence[] = {
  { &hf_rrc_measurementIdentity, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MeasurementIdentity },
  { &hf_rrc_measurementCommand_03, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MeasurementCommand_r7 },
  { &hf_rrc_measurementReportingMode, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MeasurementReportingMode },
  { &hf_rrc_additionalMeasurementList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_AdditionalMeasurementID_List },
  { &hf_rrc_dpch_CompressedModeStatusInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DPCH_CompressedModeStatusInfo },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_MeasurementControl_r7_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_MeasurementControl_r7_IEs, MeasurementControl_r7_IEs_sequence);

  return offset;
}


static const per_sequence_t NewInterFreqCell_v7b0ext_sequence[] = {
  { &hf_rrc_intraSecondaryFreqIndicator, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_NewInterFreqCell_v7b0ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_NewInterFreqCell_v7b0ext, NewInterFreqCell_v7b0ext_sequence);

  return offset;
}


static const per_sequence_t NewInterFreqCellList_v7b0ext_sequence_of[1] = {
  { &hf_rrc_NewInterFreqCellList_v7b0ext_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_NewInterFreqCell_v7b0ext },
};

static int
dissect_rrc_NewInterFreqCellList_v7b0ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_NewInterFreqCellList_v7b0ext, NewInterFreqCellList_v7b0ext_sequence_of,
                                                  1, maxCellMeas, FALSE);

  return offset;
}


static const per_sequence_t MeasurementControl_v7b0ext_IEs_sequence[] = {
  { &hf_rrc_newInterFreqCellList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_NewInterFreqCellList_v7b0ext },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_MeasurementControl_v7b0ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_MeasurementControl_v7b0ext_IEs, MeasurementControl_v7b0ext_IEs_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions_61_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_nonCriticalExtensions_61(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_nonCriticalExtensions_61, T_nonCriticalExtensions_61_sequence);

  return offset;
}


static const per_sequence_t T_v7b0NonCriticalExtensions_sequence[] = {
  { &hf_rrc_measurementControl_v7b0ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MeasurementControl_v7b0ext_IEs },
  { &hf_rrc_nonCriticalExtensions_61, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_nonCriticalExtensions_61 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v7b0NonCriticalExtensions(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v7b0NonCriticalExtensions, T_v7b0NonCriticalExtensions_sequence);

  return offset;
}


static const per_sequence_t T_r7_04_sequence[] = {
  { &hf_rrc_measurementControl_r7, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MeasurementControl_r7_IEs },
  { &hf_rrc_measurementControl_r7_add_ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_BIT_STRING },
  { &hf_rrc_v7b0NonCriticalExtensions, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v7b0NonCriticalExtensions },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_r7_04(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_r7_04, T_r7_04_sequence);

  return offset;
}


static const per_sequence_t T_tdd128_62_sequence[] = {
  { &hf_rrc_intraSecondaryFreqIndicator, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_tdd128_62(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_tdd128_62, T_tdd128_62_sequence);

  return offset;
}


static const value_string rrc_T_modeSpecificInfo_125_vals[] = {
  {   0, "noInfo" },
  {   1, "tdd128" },
  { 0, NULL }
};

static const per_choice_t T_modeSpecificInfo_125_choice[] = {
  {   0, &hf_rrc_noInfo          , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   1, &hf_rrc_tdd128_64       , ASN1_NO_EXTENSIONS     , dissect_rrc_T_tdd128_62 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_modeSpecificInfo_125(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_modeSpecificInfo_125, T_modeSpecificInfo_125_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t NewInterFreqCell_r8_sequence[] = {
  { &hf_rrc_interFreqCellID , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_InterFreqCellID },
  { &hf_rrc_frequencyInfo   , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_FrequencyInfo },
  { &hf_rrc_cellInfo_01     , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CellInfo_r4 },
  { &hf_rrc_modeSpecificInfo_125, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_modeSpecificInfo_125 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_NewInterFreqCell_r8(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_NewInterFreqCell_r8, NewInterFreqCell_r8_sequence);

  return offset;
}


static const per_sequence_t NewInterFreqCellList_r8_sequence_of[1] = {
  { &hf_rrc_NewInterFreqCellList_r8_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_NewInterFreqCell_r8 },
};

static int
dissect_rrc_NewInterFreqCellList_r8(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_NewInterFreqCellList_r8, NewInterFreqCellList_r8_sequence_of,
                                                  1, maxCellMeas, FALSE);

  return offset;
}


static const per_sequence_t InterFreqCellInfoList_r8_sequence[] = {
  { &hf_rrc_removedInterFreqCellList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RemovedInterFreqCellList },
  { &hf_rrc_newInterFreqCellList_04, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_NewInterFreqCellList_r8 },
  { &hf_rrc_cellsForInterFreqMeasList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CellsForInterFreqMeasList },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_InterFreqCellInfoList_r8(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_InterFreqCellInfoList_r8, InterFreqCellInfoList_r8_sequence);

  return offset;
}


static const per_sequence_t InterFrequencyMeasurement_r8_sequence[] = {
  { &hf_rrc_interFreqCellInfoList_02, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_InterFreqCellInfoList_r8 },
  { &hf_rrc_interFreqMeasQuantity, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_InterFreqMeasQuantity },
  { &hf_rrc_interFreqReportingQuantity, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_InterFreqReportingQuantity },
  { &hf_rrc_measurementValidity, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MeasurementValidity },
  { &hf_rrc_interFreqSetUpdate, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_AutonomousUpdateMode },
  { &hf_rrc_adjacentFrequencyIndex, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_INTEGER_0_31 },
  { &hf_rrc_reportCriteria_04, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_InterFreqReportCriteria_r7 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_InterFrequencyMeasurement_r8(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_InterFrequencyMeasurement_r8, InterFrequencyMeasurement_r8_sequence);

  return offset;
}



static int
dissect_rrc_EARFCN(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            0U, 65535U, NULL, FALSE);

  return offset;
}


static const per_sequence_t SEQUENCE_SIZE_1_maxNumEUTRAFreqs_OF_EARFCN_sequence_of[1] = {
  { &hf_rrc_removeSomeFrequencies_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_EARFCN },
};

static int
dissect_rrc_SEQUENCE_SIZE_1_maxNumEUTRAFreqs_OF_EARFCN(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_SEQUENCE_SIZE_1_maxNumEUTRAFreqs_OF_EARFCN, SEQUENCE_SIZE_1_maxNumEUTRAFreqs_OF_EARFCN_sequence_of,
                                                  1, maxNumEUTRAFreqs, FALSE);

  return offset;
}


static const value_string rrc_EUTRA_FrequencyRemoval_vals[] = {
  {   0, "removeAllFrequencies" },
  {   1, "removeSomeFrequencies" },
  {   2, "removeNoFrequencies" },
  { 0, NULL }
};

static const per_choice_t EUTRA_FrequencyRemoval_choice[] = {
  {   0, &hf_rrc_removeAllFrequencies, ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   1, &hf_rrc_removeSomeFrequencies, ASN1_NO_EXTENSIONS     , dissect_rrc_SEQUENCE_SIZE_1_maxNumEUTRAFreqs_OF_EARFCN },
  {   2, &hf_rrc_removeNoFrequencies, ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_EUTRA_FrequencyRemoval(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_EUTRA_FrequencyRemoval, EUTRA_FrequencyRemoval_choice,
                                 NULL);

  return offset;
}


static const value_string rrc_EUTRA_MeasurementBandwidth_vals[] = {
  {   0, "mbw6" },
  {   1, "mbw15" },
  {   2, "mbw25" },
  {   3, "mbw50" },
  {   4, "mbw75" },
  {   5, "mbw100" },
  { 0, NULL }
};


static int
dissect_rrc_EUTRA_MeasurementBandwidth(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     6, NULL, FALSE, 0, NULL);

  return offset;
}



static int
dissect_rrc_EUTRA_PhysicalCellIdentity(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            0U, 503U, NULL, FALSE);

  return offset;
}


static const per_sequence_t EUTRA_BlacklistedCell_sequence[] = {
  { &hf_rrc_physicalCellIdentity, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_EUTRA_PhysicalCellIdentity },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_EUTRA_BlacklistedCell(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_EUTRA_BlacklistedCell, EUTRA_BlacklistedCell_sequence);

  return offset;
}


static const per_sequence_t EUTRA_BlacklistedCellPerFreqList_sequence_of[1] = {
  { &hf_rrc_EUTRA_BlacklistedCellPerFreqList_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_EUTRA_BlacklistedCell },
};

static int
dissect_rrc_EUTRA_BlacklistedCellPerFreqList(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_EUTRA_BlacklistedCellPerFreqList, EUTRA_BlacklistedCellPerFreqList_sequence_of,
                                                  1, maxEUTRACellPerFreq, FALSE);

  return offset;
}


static const per_sequence_t EUTRA_FrequencyInfo_sequence[] = {
  { &hf_rrc_earfcn          , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_EARFCN },
  { &hf_rrc_measurementBandwidth, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_EUTRA_MeasurementBandwidth },
  { &hf_rrc_eutra_blackListedCellList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_EUTRA_BlacklistedCellPerFreqList },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_EUTRA_FrequencyInfo(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_EUTRA_FrequencyInfo, EUTRA_FrequencyInfo_sequence);

  return offset;
}


static const per_sequence_t EUTRA_FrequencyInfoList_sequence_of[1] = {
  { &hf_rrc_EUTRA_FrequencyInfoList_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_EUTRA_FrequencyInfo },
};

static int
dissect_rrc_EUTRA_FrequencyInfoList(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_EUTRA_FrequencyInfoList, EUTRA_FrequencyInfoList_sequence_of,
                                                  1, maxNumEUTRAFreqs, FALSE);

  return offset;
}


static const per_sequence_t EUTRA_FrequencyList_sequence[] = {
  { &hf_rrc_eutraFrequencyRemoval, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_EUTRA_FrequencyRemoval },
  { &hf_rrc_eutraNewFrequencies, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_EUTRA_FrequencyInfoList },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_EUTRA_FrequencyList(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_EUTRA_FrequencyList, EUTRA_FrequencyList_sequence);

  return offset;
}


static const value_string rrc_T_interRATMeasurementObjects_vals[] = {
  {   0, "interRATCellInfoList" },
  {   1, "eutra-FrequencyList" },
  { 0, NULL }
};

static const per_choice_t T_interRATMeasurementObjects_choice[] = {
  {   0, &hf_rrc_interRATCellInfoList_02, ASN1_NO_EXTENSIONS     , dissect_rrc_InterRATCellInfoList_r6 },
  {   1, &hf_rrc_eutra_FrequencyList, ASN1_NO_EXTENSIONS     , dissect_rrc_EUTRA_FrequencyList },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_interRATMeasurementObjects(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_interRATMeasurementObjects, T_interRATMeasurementObjects_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t T_gsm_10_sequence[] = {
  { &hf_rrc_measurementQuantity_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MeasurementQuantityGSM },
  { &hf_rrc_filterCoefficient, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_FilterCoefficient },
  { &hf_rrc_bsic_VerificationRequired, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BSIC_VerificationRequired },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_gsm_10(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_gsm_10, T_gsm_10_sequence);

  return offset;
}


static const per_sequence_t T_is_2000_01_sequence[] = {
  { &hf_rrc_tadd_EcIo       , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_0_63 },
  { &hf_rrc_tcomp_EcIo      , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_0_15 },
  { &hf_rrc_softSlope       , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_INTEGER_0_63 },
  { &hf_rrc_addIntercept    , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_INTEGER_0_63 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_is_2000_01(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_is_2000_01, T_is_2000_01_sequence);

  return offset;
}


static const value_string rrc_MeasurementQuantityEUTRA_vals[] = {
  {   0, "rsrp" },
  {   1, "rsrq" },
  { 0, NULL }
};


static int
dissect_rrc_MeasurementQuantityEUTRA(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     2, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t T_e_UTRA_01_sequence[] = {
  { &hf_rrc_measurementQuantity_02, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MeasurementQuantityEUTRA },
  { &hf_rrc_filterCoefficient, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_FilterCoefficient },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_e_UTRA_01(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_e_UTRA_01, T_e_UTRA_01_sequence);

  return offset;
}


static const value_string rrc_T_ratSpecificInfo_01_vals[] = {
  {   0, "gsm" },
  {   1, "is-2000" },
  {   2, "e-UTRA" },
  { 0, NULL }
};

static const per_choice_t T_ratSpecificInfo_01_choice[] = {
  {   0, &hf_rrc_gsm_11          , ASN1_NO_EXTENSIONS     , dissect_rrc_T_gsm_10 },
  {   1, &hf_rrc_is_2000_02      , ASN1_NO_EXTENSIONS     , dissect_rrc_T_is_2000_01 },
  {   2, &hf_rrc_e_UTRA_01       , ASN1_NO_EXTENSIONS     , dissect_rrc_T_e_UTRA_01 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_ratSpecificInfo_01(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_ratSpecificInfo_01, T_ratSpecificInfo_01_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t InterRATMeasQuantity_r8_sequence[] = {
  { &hf_rrc_measQuantityUTRAN_QualityEstimate, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_IntraFreqMeasQuantity },
  { &hf_rrc_ratSpecificInfo_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_ratSpecificInfo_01 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_InterRATMeasQuantity_r8(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_InterRATMeasQuantity_r8, InterRATMeasQuantity_r8_sequence);

  return offset;
}


static const per_sequence_t T_gsm_12_sequence[] = {
  { &hf_rrc_gsm_Carrier_RSSI, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_gsm_12(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_gsm_12, T_gsm_12_sequence);

  return offset;
}


static const value_string rrc_T_reportingQuantity_vals[] = {
  {   0, "measurementQuantity" },
  {   1, "both" },
  { 0, NULL }
};


static int
dissect_rrc_T_reportingQuantity(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     2, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t T_eutra_02_sequence[] = {
  { &hf_rrc_reportingQuantity, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_reportingQuantity },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_eutra_02(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_eutra_02, T_eutra_02_sequence);

  return offset;
}


static const value_string rrc_T_ratSpecificInfo_03_vals[] = {
  {   0, "gsm" },
  {   1, "eutra" },
  { 0, NULL }
};

static const per_choice_t T_ratSpecificInfo_03_choice[] = {
  {   0, &hf_rrc_gsm_14          , ASN1_NO_EXTENSIONS     , dissect_rrc_T_gsm_12 },
  {   1, &hf_rrc_eutra_02        , ASN1_NO_EXTENSIONS     , dissect_rrc_T_eutra_02 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_ratSpecificInfo_03(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_ratSpecificInfo_03, T_ratSpecificInfo_03_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t InterRATReportingQuantity_r8_sequence[] = {
  { &hf_rrc_utran_EstimatedQuality, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { &hf_rrc_ratSpecificInfo_03, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_ratSpecificInfo_03 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_InterRATReportingQuantity_r8(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_InterRATReportingQuantity_r8, InterRATReportingQuantity_r8_sequence);

  return offset;
}



static int
dissect_rrc_INTEGER_2_3(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            2U, 3U, NULL, FALSE);

  return offset;
}


static const per_sequence_t IdleIntervalInfo_sequence[] = {
  { &hf_rrc_k               , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_INTEGER_2_3 },
  { &hf_rrc_offset_02       , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_INTEGER_0_7 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_IdleIntervalInfo(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_IdleIntervalInfo, IdleIntervalInfo_sequence);

  return offset;
}


static const per_sequence_t InterRATMeasurement_r8_sequence[] = {
  { &hf_rrc_interRATMeasurementObjects, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_interRATMeasurementObjects },
  { &hf_rrc_interRATMeasQuantity_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_InterRATMeasQuantity_r8 },
  { &hf_rrc_interRATReportingQuantity_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_InterRATReportingQuantity_r8 },
  { &hf_rrc_reportCriteria_08, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_InterRATReportCriteria },
  { &hf_rrc_idleIntervalInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_IdleIntervalInfo },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_InterRATMeasurement_r8(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_InterRATMeasurement_r8, InterRATMeasurement_r8_sequence);

  return offset;
}


static const value_string rrc_T_velocityRequested_01_vals[] = {
  {   0, "true" },
  { 0, NULL }
};


static int
dissect_rrc_T_velocityRequested_01(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t UE_Positioning_ReportingQuantity_r8_sequence[] = {
  { &hf_rrc_methodType      , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UE_Positioning_MethodType },
  { &hf_rrc_positioningMethod, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PositioningMethod },
  { &hf_rrc_horizontalAccuracy, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_Positioning_Accuracy },
  { &hf_rrc_verticalAccuracy, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_Positioning_Accuracy },
  { &hf_rrc_gps_TimingOfCellWanted, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { &hf_rrc_additionalAssistanceDataReq, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { &hf_rrc_environmentCharacterisation, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_EnvironmentCharacterisation },
  { &hf_rrc_velocityRequested_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_velocityRequested_01 },
  { &hf_rrc_gANSSPositioningMethods, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_BIT_STRING_SIZE_16 },
  { &hf_rrc_gANSSTimingOfCellWanted, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_BIT_STRING_SIZE_8 },
  { &hf_rrc_gANSSCarrierPhaseMeasurementRequested, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_BIT_STRING_SIZE_8 },
  { &hf_rrc_gANSSMultiFreqMeasurementRequested, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_BIT_STRING_SIZE_8 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UE_Positioning_ReportingQuantity_r8(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UE_Positioning_ReportingQuantity_r8, UE_Positioning_ReportingQuantity_r8_sequence);

  return offset;
}


static const per_sequence_t UE_Positioning_GANSS_Almanac_r8_sequence[] = {
  { &hf_rrc_ganss_wk_number , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_0_255 },
  { &hf_rrc_alm_keplerianParameters, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_ALM_keplerianParameters },
  { &hf_rrc_alm_keplerianNAVAlmanac, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_ALM_NAVKeplerianSet },
  { &hf_rrc_alm_keplerianReducedAlmanac, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_ALM_ReducedKeplerianSet },
  { &hf_rrc_alm_keplerianMidiAlmanac, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_ALM_MidiAlmanacSet },
  { &hf_rrc_alm_keplerianGLONASS, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_ALM_GlonassAlmanacSet },
  { &hf_rrc_alm_ecefSBASAlmanac, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_ALM_ECEFsbasAlmanacSet },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UE_Positioning_GANSS_Almanac_r8(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UE_Positioning_GANSS_Almanac_r8, UE_Positioning_GANSS_Almanac_r8_sequence);

  return offset;
}


static const per_sequence_t GANSSGenericData_r8_sequence[] = {
  { &hf_rrc_ganssId         , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_INTEGER_0_7 },
  { &hf_rrc_uePositiningGANSSsbasID, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_Positioning_GANSS_SBAS_ID },
  { &hf_rrc_ganssTimeModelsList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_GANSSTimeModelsList },
  { &hf_rrc_uePositioningDGANSSCorrections, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_Positioning_DGANSSCorrections },
  { &hf_rrc_uePositioningGANSSNavigationModel, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_Positioning_GANSS_NavigationModel },
  { &hf_rrc_uePositioningGANSSAddNavigationModels, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_Positioning_GANSS_AddNavigationModels },
  { &hf_rrc_uePositioningGANSSRealTimeIntegrity, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_Positioning_GANSS_RealTimeIntegrity },
  { &hf_rrc_uePositioningGANSSDataBitAssistance, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_Positioning_GANSS_Data_Bit_Assistance },
  { &hf_rrc_uePositioningGANSSReferenceMeasurementInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_Positioning_GANSS_ReferenceMeasurementInfo },
  { &hf_rrc_uePositioningGANSSAlmanac_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_Positioning_GANSS_Almanac_r8 },
  { &hf_rrc_uePositioningGANSSUTCModel, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_Positioning_GANSS_UTCModel },
  { &hf_rrc_uePositioningGANSSAddUTCModels, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_Positioning_GANSS_AddUTCModels },
  { &hf_rrc_uePositioningGANSSAuxiliaryInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_Positioning_GANSS_AuxiliaryInfo },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_GANSSGenericData_r8(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_GANSSGenericData_r8, GANSSGenericData_r8_sequence);

  return offset;
}


static const per_sequence_t GANSSGenericDataList_r8_sequence_of[1] = {
  { &hf_rrc_GANSSGenericDataList_r8_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_GANSSGenericData_r8 },
};

static int
dissect_rrc_GANSSGenericDataList_r8(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_GANSSGenericDataList_r8, GANSSGenericDataList_r8_sequence_of,
                                                  1, maxGANSS, FALSE);

  return offset;
}


static const per_sequence_t UE_Positioning_GANSS_AssistanceData_r8_sequence[] = {
  { &hf_rrc_ue_positioning_GANSS_ReferenceTime, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_Positioning_GANSS_ReferenceTime },
  { &hf_rrc_uePositioningGanssReferencePosition, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_ReferenceLocationGANSS },
  { &hf_rrc_uePositioningGanssIonosphericModel, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_Positioning_GANSS_IonosphericModel },
  { &hf_rrc_uePositioningGanssAddIonoModel, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_Positioning_GANSS_AddIonoModel },
  { &hf_rrc_uePositioningGanssEarthOrientationPara, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_Positioning_GANSS_EarthOrientPara },
  { &hf_rrc_ganssGenericDataList_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_GANSSGenericDataList_r8 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UE_Positioning_GANSS_AssistanceData_r8(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UE_Positioning_GANSS_AssistanceData_r8, UE_Positioning_GANSS_AssistanceData_r8_sequence);

  return offset;
}


static const per_sequence_t UE_Positioning_Measurement_r8_sequence[] = {
  { &hf_rrc_ue_positioning_ReportingQuantity_03, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UE_Positioning_ReportingQuantity_r8 },
  { &hf_rrc_measurementValidity, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MeasurementValidity },
  { &hf_rrc_reportCriteria_21, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UE_Positioning_ReportCriteria_r7 },
  { &hf_rrc_ue_positioning_OTDOA_AssistanceData_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_Positioning_OTDOA_AssistanceData_r7 },
  { &hf_rrc_ue_positioning_GPS_AssistanceData_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_Positioning_GPS_AssistanceData_r7 },
  { &hf_rrc_ue_positioning_GANSS_AssistanceData_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_Positioning_GANSS_AssistanceData_r8 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UE_Positioning_Measurement_r8(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UE_Positioning_Measurement_r8, UE_Positioning_Measurement_r8_sequence);

  return offset;
}


static const value_string rrc_MeasurementType_r8_vals[] = {
  {   0, "intraFrequencyMeasurement" },
  {   1, "interFrequencyMeasurement" },
  {   2, "interRATMeasurement" },
  {   3, "up-Measurement" },
  {   4, "trafficVolumeMeasurement" },
  {   5, "qualityMeasurement" },
  {   6, "ue-InternalMeasurement" },
  { 0, NULL }
};

static const per_choice_t MeasurementType_r8_choice[] = {
  {   0, &hf_rrc_intraFrequencyMeasurement_03, ASN1_NO_EXTENSIONS     , dissect_rrc_IntraFrequencyMeasurement_r7 },
  {   1, &hf_rrc_interFrequencyMeasurement_04, ASN1_NO_EXTENSIONS     , dissect_rrc_InterFrequencyMeasurement_r8 },
  {   2, &hf_rrc_interRATMeasurement_03, ASN1_NO_EXTENSIONS     , dissect_rrc_InterRATMeasurement_r8 },
  {   3, &hf_rrc_up_Measurement_02, ASN1_NO_EXTENSIONS     , dissect_rrc_UE_Positioning_Measurement_r8 },
  {   4, &hf_rrc_trafficVolumeMeasurement, ASN1_NO_EXTENSIONS     , dissect_rrc_TrafficVolumeMeasurement },
  {   5, &hf_rrc_qualityMeasurement, ASN1_NO_EXTENSIONS     , dissect_rrc_QualityMeasurement },
  {   6, &hf_rrc_ue_InternalMeasurement_01, ASN1_NO_EXTENSIONS     , dissect_rrc_UE_InternalMeasurement_r4 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_MeasurementType_r8(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_MeasurementType_r8, MeasurementType_r8_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t T_modify_05_sequence[] = {
  { &hf_rrc_measurementType_04, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MeasurementType_r8 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_modify_05(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_modify_05, T_modify_05_sequence);

  return offset;
}


static const value_string rrc_MeasurementCommand_r8_vals[] = {
  {   0, "setup" },
  {   1, "modify" },
  {   2, "release" },
  { 0, NULL }
};

static const per_choice_t MeasurementCommand_r8_choice[] = {
  {   0, &hf_rrc_setup_04        , ASN1_NO_EXTENSIONS     , dissect_rrc_MeasurementType_r8 },
  {   1, &hf_rrc_modify_05       , ASN1_NO_EXTENSIONS     , dissect_rrc_T_modify_05 },
  {   2, &hf_rrc_release_01      , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_MeasurementCommand_r8(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_MeasurementCommand_r8, MeasurementCommand_r8_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t MeasurementControl_r8_IEs_sequence[] = {
  { &hf_rrc_measurementIdentity, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MeasurementIdentity },
  { &hf_rrc_measurementCommand_04, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MeasurementCommand_r8 },
  { &hf_rrc_measurementReportingMode, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MeasurementReportingMode },
  { &hf_rrc_additionalMeasurementList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_AdditionalMeasurementID_List },
  { &hf_rrc_dpch_CompressedModeStatusInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DPCH_CompressedModeStatusInfo },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_MeasurementControl_r8_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_MeasurementControl_r8_IEs, MeasurementControl_r8_IEs_sequence);

  return offset;
}


static const per_sequence_t T_fdd_142_sequence[] = {
  { &hf_rrc_q_QualMin       , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_Q_QualMin },
  { &hf_rrc_q_RxlevMin      , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_Q_RxlevMin },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_fdd_142(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_fdd_142, T_fdd_142_sequence);

  return offset;
}


static const per_sequence_t T_tdd_93_sequence[] = {
  { &hf_rrc_q_RxlevMin      , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_Q_RxlevMin },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_tdd_93(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_tdd_93, T_tdd_93_sequence);

  return offset;
}


static const per_sequence_t T_gsm_02_sequence[] = {
  { &hf_rrc_q_RxlevMin      , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_Q_RxlevMin },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_gsm_02(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_gsm_02, T_gsm_02_sequence);

  return offset;
}


static const value_string rrc_T_modeSpecificInfo_103_vals[] = {
  {   0, "fdd" },
  {   1, "tdd" },
  {   2, "gsm" },
  { 0, NULL }
};

static const per_choice_t T_modeSpecificInfo_103_choice[] = {
  {   0, &hf_rrc_fdd_147         , ASN1_NO_EXTENSIONS     , dissect_rrc_T_fdd_142 },
  {   1, &hf_rrc_tdd_100         , ASN1_NO_EXTENSIONS     , dissect_rrc_T_tdd_93 },
  {   2, &hf_rrc_gsm_02          , ASN1_NO_EXTENSIONS     , dissect_rrc_T_gsm_02 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_modeSpecificInfo_103(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_modeSpecificInfo_103, T_modeSpecificInfo_103_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t CellSelectReselectInfoMC_RSCP_sequence[] = {
  { &hf_rrc_q_OffsetS_N     , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_Q_OffsetS_N },
  { &hf_rrc_maxAllowedUL_TX_Power, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MaxAllowedUL_TX_Power },
  { &hf_rrc_hcs_NeighbouringCellInformation_RSCP, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_HCS_NeighbouringCellInformation_RSCP },
  { &hf_rrc_modeSpecificInfo_103, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_modeSpecificInfo_103 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_CellSelectReselectInfoMC_RSCP(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_CellSelectReselectInfoMC_RSCP, CellSelectReselectInfoMC_RSCP_sequence);

  return offset;
}


static const per_sequence_t CellInfo_LCR_r8_ext_sequence[] = {
  { &hf_rrc_cellSelectionReselectionInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CellSelectReselectInfoMC_RSCP },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_CellInfo_LCR_r8_ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_CellInfo_LCR_r8_ext, CellInfo_LCR_r8_ext_sequence);

  return offset;
}


static const per_sequence_t NewIntraFreqCell_LCR_v8a0ext_sequence[] = {
  { &hf_rrc_cellInfo_LCR_r8 , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CellInfo_LCR_r8_ext },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_NewIntraFreqCell_LCR_v8a0ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_NewIntraFreqCell_LCR_v8a0ext, NewIntraFreqCell_LCR_v8a0ext_sequence);

  return offset;
}


static const per_sequence_t NewIntraFreqCellList_LCR_v8a0ext_sequence_of[1] = {
  { &hf_rrc_NewIntraFreqCellList_LCR_v8a0ext_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_NewIntraFreqCell_LCR_v8a0ext },
};

static int
dissect_rrc_NewIntraFreqCellList_LCR_v8a0ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_NewIntraFreqCellList_LCR_v8a0ext, NewIntraFreqCellList_LCR_v8a0ext_sequence_of,
                                                  1, maxCellMeas, FALSE);

  return offset;
}


static const per_sequence_t NewInterFreqCell_LCR_v8a0ext_sequence[] = {
  { &hf_rrc_cellInfo_LCR_r8 , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CellInfo_LCR_r8_ext },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_NewInterFreqCell_LCR_v8a0ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_NewInterFreqCell_LCR_v8a0ext, NewInterFreqCell_LCR_v8a0ext_sequence);

  return offset;
}


static const per_sequence_t NewInterFreqCellList_LCR_v8a0ext_sequence_of[1] = {
  { &hf_rrc_NewInterFreqCellList_LCR_v8a0ext_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_NewInterFreqCell_LCR_v8a0ext },
};

static int
dissect_rrc_NewInterFreqCellList_LCR_v8a0ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_NewInterFreqCellList_LCR_v8a0ext, NewInterFreqCellList_LCR_v8a0ext_sequence_of,
                                                  1, maxCellMeas, FALSE);

  return offset;
}


static const per_sequence_t MeasurementControl_v8a0ext_IEs_sequence[] = {
  { &hf_rrc_newIntraFreqCellList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_NewIntraFreqCellList_LCR_v8a0ext },
  { &hf_rrc_newInterFreqCellList_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_NewInterFreqCellList_LCR_v8a0ext },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_MeasurementControl_v8a0ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_MeasurementControl_v8a0ext_IEs, MeasurementControl_v8a0ext_IEs_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions_62_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_nonCriticalExtensions_62(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_nonCriticalExtensions_62, T_nonCriticalExtensions_62_sequence);

  return offset;
}


static const per_sequence_t T_v8a0NonCriticalExtensions_03_sequence[] = {
  { &hf_rrc_measurementControl_v8a0ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MeasurementControl_v8a0ext_IEs },
  { &hf_rrc_nonCriticalExtensions_62, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_nonCriticalExtensions_62 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v8a0NonCriticalExtensions_03(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v8a0NonCriticalExtensions_03, T_v8a0NonCriticalExtensions_03_sequence);

  return offset;
}


static const per_sequence_t T_r8_05_sequence[] = {
  { &hf_rrc_measurementControl_r8, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MeasurementControl_r8_IEs },
  { &hf_rrc_measurementControl_r8_add_ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_BIT_STRING },
  { &hf_rrc_v8a0NonCriticalExtensions_03, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v8a0NonCriticalExtensions_03 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_r8_05(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_r8_05, T_r8_05_sequence);

  return offset;
}


static const per_sequence_t T_fdd_134_sequence[] = {
  { &hf_rrc_primaryCPICH_Info, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_PrimaryCPICH_Info },
  { &hf_rrc_primaryCPICH_TX_Power, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_PrimaryCPICH_TX_Power },
  { &hf_rrc_readSFN_Indicator, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { &hf_rrc_tx_DiversityIndicator, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_fdd_134(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_fdd_134, T_fdd_134_sequence);

  return offset;
}


static const per_sequence_t T_tdd_85_sequence[] = {
  { &hf_rrc_primaryCCPCH_Info_03, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PrimaryCCPCH_Info_r4 },
  { &hf_rrc_primaryCCPCH_TX_Power, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_PrimaryCCPCH_TX_Power },
  { &hf_rrc_timeslotInfoList_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_TimeslotInfoList_r4 },
  { &hf_rrc_readSFN_Indicator, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_tdd_85(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_tdd_85, T_tdd_85_sequence);

  return offset;
}


static const value_string rrc_T_modeSpecificInfo_95_vals[] = {
  {   0, "fdd" },
  {   1, "tdd" },
  { 0, NULL }
};

static const per_choice_t T_modeSpecificInfo_95_choice[] = {
  {   0, &hf_rrc_fdd_137         , ASN1_NO_EXTENSIONS     , dissect_rrc_T_fdd_134 },
  {   1, &hf_rrc_tdd_91          , ASN1_NO_EXTENSIONS     , dissect_rrc_T_tdd_85 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_modeSpecificInfo_95(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_modeSpecificInfo_95, T_modeSpecificInfo_95_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t CellInfo_r9_sequence[] = {
  { &hf_rrc_cellIndividualOffset, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CellIndividualOffset },
  { &hf_rrc_referenceTimeDifferenceToCell, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_ReferenceTimeDifferenceToCell },
  { &hf_rrc_modeSpecificInfo_95, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_modeSpecificInfo_95 },
  { &hf_rrc_cellSelectionReselectionInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CellSelectReselectInfoMC_RSCP },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_CellInfo_r9(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_CellInfo_r9, CellInfo_r9_sequence);

  return offset;
}


static const per_sequence_t NewIntraFreqCell_r9_sequence[] = {
  { &hf_rrc_intraFreqCellID , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_IntraFreqCellID },
  { &hf_rrc_cellInfo_02     , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CellInfo_r9 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_NewIntraFreqCell_r9(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_NewIntraFreqCell_r9, NewIntraFreqCell_r9_sequence);

  return offset;
}


static const per_sequence_t NewIntraFreqCellList_r9_sequence_of[1] = {
  { &hf_rrc_NewIntraFreqCellList_r9_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_NewIntraFreqCell_r9 },
};

static int
dissect_rrc_NewIntraFreqCellList_r9(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_NewIntraFreqCellList_r9, NewIntraFreqCellList_r9_sequence_of,
                                                  1, maxCellMeas, FALSE);

  return offset;
}


static const per_sequence_t T_fdd_149_sequence[] = {
  { &hf_rrc_startPSC_01     , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PrimaryCPICH_Info },
  { &hf_rrc_numberOfPSCs_01 , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_1_512 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_fdd_149(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_fdd_149, T_fdd_149_sequence);

  return offset;
}


static const value_string rrc_T_modeSpecificInfo_110_vals[] = {
  {   0, "fdd" },
  {   1, "tdd" },
  { 0, NULL }
};

static const per_choice_t T_modeSpecificInfo_110_choice[] = {
  {   0, &hf_rrc_fdd_154         , ASN1_NO_EXTENSIONS     , dissect_rrc_T_fdd_149 },
  {   1, &hf_rrc_tdd             , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_modeSpecificInfo_110(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_modeSpecificInfo_110, T_modeSpecificInfo_110_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t CSGCellInfo_sequence[] = {
  { &hf_rrc_modeSpecificInfo_110, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_modeSpecificInfo_110 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_CSGCellInfo(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_CSGCellInfo, CSGCellInfo_sequence);

  return offset;
}


static const per_sequence_t CSGCellInfoList_sequence_of[1] = {
  { &hf_rrc_CSGCellInfoList_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CSGCellInfo },
};

static int
dissect_rrc_CSGCellInfoList(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_CSGCellInfoList, CSGCellInfoList_sequence_of,
                                                  1, maxMeasCSGRange, FALSE);

  return offset;
}



static int
dissect_rrc_CSGIntraFreqCellInfoList(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_rrc_CSGCellInfoList(tvb, offset, actx, tree, hf_index);

  return offset;
}


static const per_sequence_t IntraFreqSIAcquisitionInfo_sequence_of[1] = {
  { &hf_rrc_IntraFreqSIAcquisitionInfo_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PrimaryCPICH_Info },
};

static int
dissect_rrc_IntraFreqSIAcquisitionInfo(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_IntraFreqSIAcquisitionInfo, IntraFreqSIAcquisitionInfo_sequence_of,
                                                  1, maxSIrequest, FALSE);

  return offset;
}


static const per_sequence_t T_fdd_154_sequence[] = {
  { &hf_rrc_intraFreqSIAcquisitionInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_IntraFreqSIAcquisitionInfo },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_fdd_154(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_fdd_154, T_fdd_154_sequence);

  return offset;
}


static const value_string rrc_T_modeSpecificInfo_115_vals[] = {
  {   0, "fdd" },
  {   1, "tdd" },
  { 0, NULL }
};

static const per_choice_t T_modeSpecificInfo_115_choice[] = {
  {   0, &hf_rrc_fdd_160         , ASN1_NO_EXTENSIONS     , dissect_rrc_T_fdd_154 },
  {   1, &hf_rrc_tdd             , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_modeSpecificInfo_115(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_modeSpecificInfo_115, T_modeSpecificInfo_115_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t IntraFreqSIAcquisition_sequence[] = {
  { &hf_rrc_modeSpecificInfo_115, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_modeSpecificInfo_115 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_IntraFreqSIAcquisition(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_IntraFreqSIAcquisition, IntraFreqSIAcquisition_sequence);

  return offset;
}


static const per_sequence_t IntraFreqCellInfoList_r9_sequence[] = {
  { &hf_rrc_removedIntraFreqCellList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RemovedIntraFreqCellList },
  { &hf_rrc_newIntraFreqCellList_04, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_NewIntraFreqCellList_r9 },
  { &hf_rrc_cellsForIntraFreqMeasList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CellsForIntraFreqMeasList },
  { &hf_rrc_cSGIntraFreqCellInfoList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CSGIntraFreqCellInfoList },
  { &hf_rrc_intraFreqSIAcquisition, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_IntraFreqSIAcquisition },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_IntraFreqCellInfoList_r9(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_IntraFreqCellInfoList_r9, IntraFreqCellInfoList_r9_sequence);

  return offset;
}



static int
dissect_rrc_IntraFreqCellIDOnSecULFreq(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            0U, maxCellMeasOnSecULFreq_1, NULL, FALSE);

  return offset;
}


static const per_sequence_t SEQUENCE_SIZE_1_maxCellMeasOnSecULFreq_OF_IntraFreqCellIDOnSecULFreq_sequence_of[1] = {
  { &hf_rrc_removeSomeIntraFreqCells_item_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_IntraFreqCellIDOnSecULFreq },
};

static int
dissect_rrc_SEQUENCE_SIZE_1_maxCellMeasOnSecULFreq_OF_IntraFreqCellIDOnSecULFreq(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_SEQUENCE_SIZE_1_maxCellMeasOnSecULFreq_OF_IntraFreqCellIDOnSecULFreq, SEQUENCE_SIZE_1_maxCellMeasOnSecULFreq_OF_IntraFreqCellIDOnSecULFreq_sequence_of,
                                                  1, maxCellMeasOnSecULFreq, FALSE);

  return offset;
}


static const value_string rrc_RemovedIntraFreqCellListOnSecULFreq_vals[] = {
  {   0, "removeAllIntraFreqCells" },
  {   1, "removeSomeIntraFreqCells" },
  {   2, "removeNoIntraFreqCells" },
  { 0, NULL }
};

static const per_choice_t RemovedIntraFreqCellListOnSecULFreq_choice[] = {
  {   0, &hf_rrc_removeAllIntraFreqCells, ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   1, &hf_rrc_removeSomeIntraFreqCells_01, ASN1_NO_EXTENSIONS     , dissect_rrc_SEQUENCE_SIZE_1_maxCellMeasOnSecULFreq_OF_IntraFreqCellIDOnSecULFreq },
  {   2, &hf_rrc_removeNoIntraFreqCells, ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_RemovedIntraFreqCellListOnSecULFreq(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_RemovedIntraFreqCellListOnSecULFreq, RemovedIntraFreqCellListOnSecULFreq_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t NewIntraFreqCellOnSecULFreq_sequence[] = {
  { &hf_rrc_intraFreqCellIDOnSecULFreq, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_IntraFreqCellIDOnSecULFreq },
  { &hf_rrc_cellInfo_01     , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CellInfo_r4 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_NewIntraFreqCellOnSecULFreq(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_NewIntraFreqCellOnSecULFreq, NewIntraFreqCellOnSecULFreq_sequence);

  return offset;
}


static const per_sequence_t NewIntraFreqCellListOnSecULFreq_sequence_of[1] = {
  { &hf_rrc_NewIntraFreqCellListOnSecULFreq_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_NewIntraFreqCellOnSecULFreq },
};

static int
dissect_rrc_NewIntraFreqCellListOnSecULFreq(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_NewIntraFreqCellListOnSecULFreq, NewIntraFreqCellListOnSecULFreq_sequence_of,
                                                  1, maxCellMeasOnSecULFreq, FALSE);

  return offset;
}


static const per_sequence_t CellsForIntraFreqMeasListOnSecULFreq_sequence_of[1] = {
  { &hf_rrc_CellsForIntraFreqMeasListOnSecULFreq_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_IntraFreqCellIDOnSecULFreq },
};

static int
dissect_rrc_CellsForIntraFreqMeasListOnSecULFreq(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_CellsForIntraFreqMeasListOnSecULFreq, CellsForIntraFreqMeasListOnSecULFreq_sequence_of,
                                                  1, maxCellMeasOnSecULFreq, FALSE);

  return offset;
}


static const per_sequence_t IntraFreqCellInfoListInfoOnSecULFreq_sequence[] = {
  { &hf_rrc_removedIntraFreqCellList_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RemovedIntraFreqCellListOnSecULFreq },
  { &hf_rrc_newIntraFreqCellList_03, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_NewIntraFreqCellListOnSecULFreq },
  { &hf_rrc_cellsForIntraFreqMeasList_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CellsForIntraFreqMeasListOnSecULFreq },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_IntraFreqCellInfoListInfoOnSecULFreq(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_IntraFreqCellInfoListInfoOnSecULFreq, IntraFreqCellInfoListInfoOnSecULFreq_sequence);

  return offset;
}


static const per_sequence_t IntraFreqCellInfoListOnSecULFreq_sequence[] = {
  { &hf_rrc_intraFreqCellInfoListOnSecULFreq, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_IntraFreqCellInfoListInfoOnSecULFreq },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_IntraFreqCellInfoListOnSecULFreq(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_IntraFreqCellInfoListOnSecULFreq, IntraFreqCellInfoListOnSecULFreq_sequence);

  return offset;
}


static const per_sequence_t ForbiddenAffectCellOnSecULFreq_sequence[] = {
  { &hf_rrc_primaryCPICH    , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PrimaryCPICH_Info },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_ForbiddenAffectCellOnSecULFreq(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_ForbiddenAffectCellOnSecULFreq, ForbiddenAffectCellOnSecULFreq_sequence);

  return offset;
}


static const per_sequence_t ForbiddenAffectCellListOnSecULFreq_sequence_of[1] = {
  { &hf_rrc_ForbiddenAffectCellListOnSecULFreq_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_ForbiddenAffectCellOnSecULFreq },
};

static int
dissect_rrc_ForbiddenAffectCellListOnSecULFreq(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_ForbiddenAffectCellListOnSecULFreq, ForbiddenAffectCellListOnSecULFreq_sequence_of,
                                                  1, maxCellMeasOnSecULFreq, FALSE);

  return offset;
}


static const per_sequence_t Event1a_OnSecUlFreq_r9_sequence[] = {
  { &hf_rrc_triggeringCondition, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TriggeringCondition2 },
  { &hf_rrc_reportingRange  , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_ReportingRange },
  { &hf_rrc_forbiddenAffectCellListOnSecULFreq, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_ForbiddenAffectCellListOnSecULFreq },
  { &hf_rrc_w               , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_W },
  { &hf_rrc_reportDeactivationThreshold, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_ReportDeactivationThreshold },
  { &hf_rrc_reportingAmount , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_ReportingAmount },
  { &hf_rrc_reportingInterval, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_ReportingInterval },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_Event1a_OnSecUlFreq_r9(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_Event1a_OnSecUlFreq_r9, Event1a_OnSecUlFreq_r9_sequence);

  return offset;
}


static const per_sequence_t Event1b_OnSecUlFreq_r9_sequence[] = {
  { &hf_rrc_triggeringCondition_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TriggeringCondition1 },
  { &hf_rrc_reportingRange  , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_ReportingRange },
  { &hf_rrc_forbiddenAffectCellListOnSecULFreq, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_ForbiddenAffectCellListOnSecULFreq },
  { &hf_rrc_w               , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_W },
  { &hf_rrc_periodicReportingInfo_1b, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_PeriodicReportingInfo_1b },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_Event1b_OnSecUlFreq_r9(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_Event1b_OnSecUlFreq_r9, Event1b_OnSecUlFreq_r9_sequence);

  return offset;
}


static const value_string rrc_IntraFreqEventOnSecULFreq_vals[] = {
  {   0, "e1a" },
  {   1, "e1b" },
  {   2, "e1c" },
  {   3, "e1e" },
  {   4, "e1f" },
  { 0, NULL }
};

static const per_choice_t IntraFreqEventOnSecULFreq_choice[] = {
  {   0, &hf_rrc_e1a_03          , ASN1_NO_EXTENSIONS     , dissect_rrc_Event1a_OnSecUlFreq_r9 },
  {   1, &hf_rrc_e1b_04          , ASN1_NO_EXTENSIONS     , dissect_rrc_Event1b_OnSecUlFreq_r9 },
  {   2, &hf_rrc_e1c             , ASN1_NO_EXTENSIONS     , dissect_rrc_Event1c },
  {   3, &hf_rrc_e1e_01          , ASN1_NO_EXTENSIONS     , dissect_rrc_Event1e_r6 },
  {   4, &hf_rrc_e1f_01          , ASN1_NO_EXTENSIONS     , dissect_rrc_Event1f_r6 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_IntraFreqEventOnSecULFreq(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_IntraFreqEventOnSecULFreq, IntraFreqEventOnSecULFreq_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t IntraFreqEventCriteriaOnSecULFreq_sequence[] = {
  { &hf_rrc_event_05        , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_IntraFreqEventOnSecULFreq },
  { &hf_rrc_hysteresis_01   , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_Hysteresis },
  { &hf_rrc_timeToTrigger   , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TimeToTrigger },
  { &hf_rrc_reportingCellStatus, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_ReportingCellStatus },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_IntraFreqEventCriteriaOnSecULFreq(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_IntraFreqEventCriteriaOnSecULFreq, IntraFreqEventCriteriaOnSecULFreq_sequence);

  return offset;
}


static const per_sequence_t SEQUENCE_SIZE_1_maxMeasEventOnSecULFreq_OF_IntraFreqEventCriteriaOnSecULFreq_sequence_of[1] = {
  { &hf_rrc_intraFreqEventCriteria_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_IntraFreqEventCriteriaOnSecULFreq },
};

static int
dissect_rrc_SEQUENCE_SIZE_1_maxMeasEventOnSecULFreq_OF_IntraFreqEventCriteriaOnSecULFreq(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_SEQUENCE_SIZE_1_maxMeasEventOnSecULFreq_OF_IntraFreqEventCriteriaOnSecULFreq, SEQUENCE_SIZE_1_maxMeasEventOnSecULFreq_OF_IntraFreqEventCriteriaOnSecULFreq_sequence_of,
                                                  1, maxMeasEventOnSecULFreq, FALSE);

  return offset;
}


static const per_sequence_t IntraFreqEventCriteriaListOnSecULFreq_sequence[] = {
  { &hf_rrc_frequencyInfo   , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_FrequencyInfo },
  { &hf_rrc_intraFreqEventCriteria, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SEQUENCE_SIZE_1_maxMeasEventOnSecULFreq_OF_IntraFreqEventCriteriaOnSecULFreq },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_IntraFreqEventCriteriaListOnSecULFreq(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_IntraFreqEventCriteriaListOnSecULFreq, IntraFreqEventCriteriaListOnSecULFreq_sequence);

  return offset;
}


static const per_sequence_t IntraFreqReportingCriteria_r9_sequence[] = {
  { &hf_rrc_eventCriteriaList_04, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_IntraFreqEventCriteriaList_r7 },
  { &hf_rrc_eventCriteriaListOnSecULFreq, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_IntraFreqEventCriteriaListOnSecULFreq },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_IntraFreqReportingCriteria_r9(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_IntraFreqReportingCriteria_r9, IntraFreqReportingCriteria_r9_sequence);

  return offset;
}


static const value_string rrc_IntraFreqReportCriteria_r9_vals[] = {
  {   0, "intraFreqReportingCriteria" },
  {   1, "periodicalReportingCriteria" },
  {   2, "noReporting" },
  { 0, NULL }
};

static const per_choice_t IntraFreqReportCriteria_r9_choice[] = {
  {   0, &hf_rrc_intraFreqReportingCriteria_06, ASN1_NO_EXTENSIONS     , dissect_rrc_IntraFreqReportingCriteria_r9 },
  {   1, &hf_rrc_periodicalReportingCriteria_01, ASN1_NO_EXTENSIONS     , dissect_rrc_PeriodicalWithReportingCellStatus },
  {   2, &hf_rrc_noReporting     , ASN1_NO_EXTENSIONS     , dissect_rrc_ReportingCellStatusOpt },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_IntraFreqReportCriteria_r9(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_IntraFreqReportCriteria_r9, IntraFreqReportCriteria_r9_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t IntraFrequencyMeasurement_r9_sequence[] = {
  { &hf_rrc_intraFreqCellInfoList_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_IntraFreqCellInfoList_r9 },
  { &hf_rrc_intraFreqCellInfoListOnSecULFreq_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_IntraFreqCellInfoListOnSecULFreq },
  { &hf_rrc_intraFreqMeasQuantity, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_IntraFreqMeasQuantity },
  { &hf_rrc_intraFreqReportingQuantity, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_IntraFreqReportingQuantity },
  { &hf_rrc_measurementValidity, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MeasurementValidity },
  { &hf_rrc_reportCriteria_13, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_IntraFreqReportCriteria_r9 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_IntraFrequencyMeasurement_r9(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_IntraFrequencyMeasurement_r9, IntraFrequencyMeasurement_r9_sequence);

  return offset;
}


static const per_sequence_t T_tdd128_63_sequence[] = {
  { &hf_rrc_intraSecondaryFreqIndicator, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_tdd128_63(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_tdd128_63, T_tdd128_63_sequence);

  return offset;
}


static const value_string rrc_T_modeSpecificInfo_126_vals[] = {
  {   0, "noInfo" },
  {   1, "tdd128" },
  { 0, NULL }
};

static const per_choice_t T_modeSpecificInfo_126_choice[] = {
  {   0, &hf_rrc_noInfo          , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   1, &hf_rrc_tdd128_65       , ASN1_NO_EXTENSIONS     , dissect_rrc_T_tdd128_63 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_modeSpecificInfo_126(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_modeSpecificInfo_126, T_modeSpecificInfo_126_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t NewInterFreqCell_r9_sequence[] = {
  { &hf_rrc_interFreqCellID , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_InterFreqCellID },
  { &hf_rrc_frequencyInfo   , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_FrequencyInfo },
  { &hf_rrc_cellInfo_02     , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CellInfo_r9 },
  { &hf_rrc_modeSpecificInfo_126, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_modeSpecificInfo_126 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_NewInterFreqCell_r9(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_NewInterFreqCell_r9, NewInterFreqCell_r9_sequence);

  return offset;
}


static const per_sequence_t NewInterFreqCellList_r9_sequence_of[1] = {
  { &hf_rrc_NewInterFreqCellList_r9_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_NewInterFreqCell_r9 },
};

static int
dissect_rrc_NewInterFreqCellList_r9(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_NewInterFreqCellList_r9, NewInterFreqCellList_r9_sequence_of,
                                                  1, maxCellMeas, FALSE);

  return offset;
}


static const per_sequence_t CSGInterFreqCellInfo_sequence[] = {
  { &hf_rrc_frequencyInfo   , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_FrequencyInfo },
  { &hf_rrc_cSGInterFreqCellInfoListperFreq, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CSGCellInfoList },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_CSGInterFreqCellInfo(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_CSGInterFreqCellInfo, CSGInterFreqCellInfo_sequence);

  return offset;
}


static const per_sequence_t CSGInterFreqCellInfoList_sequence_of[1] = {
  { &hf_rrc_CSGInterFreqCellInfoList_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CSGInterFreqCellInfo },
};

static int
dissect_rrc_CSGInterFreqCellInfoList(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_CSGInterFreqCellInfoList, CSGInterFreqCellInfoList_sequence_of,
                                                  1, maxFreq, FALSE);

  return offset;
}


static const per_sequence_t T_fdd_150_sequence[] = {
  { &hf_rrc_primaryScramblingCode_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PrimaryCPICH_Info },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_fdd_150(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_fdd_150, T_fdd_150_sequence);

  return offset;
}


static const value_string rrc_T_modeSpecificInfo_111_vals[] = {
  {   0, "fdd" },
  {   1, "tdd" },
  { 0, NULL }
};

static const per_choice_t T_modeSpecificInfo_111_choice[] = {
  {   0, &hf_rrc_fdd_156         , ASN1_NO_EXTENSIONS     , dissect_rrc_T_fdd_150 },
  {   1, &hf_rrc_tdd             , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_modeSpecificInfo_111(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_modeSpecificInfo_111, T_modeSpecificInfo_111_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t InterFreqSIAcquisition_sequence[] = {
  { &hf_rrc_frequencyInfo   , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_FrequencyInfo },
  { &hf_rrc_modeSpecificInfo_111, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_modeSpecificInfo_111 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_InterFreqSIAcquisition(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_InterFreqSIAcquisition, InterFreqSIAcquisition_sequence);

  return offset;
}


static const per_sequence_t InterFreqCellInfoList_r9_sequence[] = {
  { &hf_rrc_removedInterFreqCellList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RemovedInterFreqCellList },
  { &hf_rrc_newInterFreqCellList_05, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_NewInterFreqCellList_r9 },
  { &hf_rrc_cellsForInterFreqMeasList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CellsForInterFreqMeasList },
  { &hf_rrc_cSGInterFreqCellInfoList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CSGInterFreqCellInfoList },
  { &hf_rrc_interFreqSIAcquisition, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_InterFreqSIAcquisition },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_InterFreqCellInfoList_r9(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_InterFreqCellInfoList_r9, InterFreqCellInfoList_r9_sequence);

  return offset;
}


static const value_string rrc_InterFreqReportCriteria_r9_vals[] = {
  {   0, "intraFreqReportingCriteria" },
  {   1, "interFreqReportingCriteria" },
  {   2, "periodicalReportingCriteria" },
  {   3, "noReporting" },
  { 0, NULL }
};

static const per_choice_t InterFreqReportCriteria_r9_choice[] = {
  {   0, &hf_rrc_intraFreqReportingCriteria_06, ASN1_NO_EXTENSIONS     , dissect_rrc_IntraFreqReportingCriteria_r9 },
  {   1, &hf_rrc_interFreqReportingCriteria_02, ASN1_NO_EXTENSIONS     , dissect_rrc_InterFreqReportingCriteria_r6 },
  {   2, &hf_rrc_periodicalReportingCriteria_01, ASN1_NO_EXTENSIONS     , dissect_rrc_PeriodicalWithReportingCellStatus },
  {   3, &hf_rrc_noReporting     , ASN1_NO_EXTENSIONS     , dissect_rrc_ReportingCellStatusOpt },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_InterFreqReportCriteria_r9(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_InterFreqReportCriteria_r9, InterFreqReportCriteria_r9_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t InterFrequencyMeasurement_r9_sequence[] = {
  { &hf_rrc_interFreqCellInfoList_03, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_InterFreqCellInfoList_r9 },
  { &hf_rrc_interFreqMeasQuantity, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_InterFreqMeasQuantity },
  { &hf_rrc_interFreqReportingQuantity, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_InterFreqReportingQuantity },
  { &hf_rrc_measurementValidity, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MeasurementValidity },
  { &hf_rrc_interFreqSetUpdate, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_AutonomousUpdateMode },
  { &hf_rrc_adjacentFrequencyIndex, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_INTEGER_0_31 },
  { &hf_rrc_interBandFrequencyIndex, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_INTEGER_0_31 },
  { &hf_rrc_reportCriteria_05, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_InterFreqReportCriteria_r9 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_InterFrequencyMeasurement_r9(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_InterFrequencyMeasurement_r9, InterFrequencyMeasurement_r9_sequence);

  return offset;
}


static const per_sequence_t EUTRA_SIAcquisition_sequence[] = {
  { &hf_rrc_earfcn          , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_EARFCN },
  { &hf_rrc_physicalCellIdentity, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_EUTRA_PhysicalCellIdentity },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_EUTRA_SIAcquisition(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_EUTRA_SIAcquisition, EUTRA_SIAcquisition_sequence);

  return offset;
}


static const per_sequence_t EUTRA_FrequencyList_r9_sequence[] = {
  { &hf_rrc_eutraFrequencyRemoval, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_EUTRA_FrequencyRemoval },
  { &hf_rrc_eutraNewFrequencies, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_EUTRA_FrequencyInfoList },
  { &hf_rrc_eutraSIAcquisition, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_EUTRA_SIAcquisition },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_EUTRA_FrequencyList_r9(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_EUTRA_FrequencyList_r9, EUTRA_FrequencyList_r9_sequence);

  return offset;
}


static const value_string rrc_T_interRATMeasurementObjects_01_vals[] = {
  {   0, "interRATCellInfoList" },
  {   1, "eutra-FrequencyList" },
  { 0, NULL }
};

static const per_choice_t T_interRATMeasurementObjects_01_choice[] = {
  {   0, &hf_rrc_interRATCellInfoList_02, ASN1_NO_EXTENSIONS     , dissect_rrc_InterRATCellInfoList_r6 },
  {   1, &hf_rrc_eutra_FrequencyList_01, ASN1_NO_EXTENSIONS     , dissect_rrc_EUTRA_FrequencyList_r9 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_interRATMeasurementObjects_01(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_interRATMeasurementObjects_01, T_interRATMeasurementObjects_01_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t InterRATMeasurement_r9_sequence[] = {
  { &hf_rrc_interRATMeasurementObjects_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_interRATMeasurementObjects_01 },
  { &hf_rrc_interRATMeasQuantity_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_InterRATMeasQuantity_r8 },
  { &hf_rrc_interRATReportingQuantity_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_InterRATReportingQuantity_r8 },
  { &hf_rrc_reportCriteria_08, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_InterRATReportCriteria },
  { &hf_rrc_idleIntervalInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_IdleIntervalInfo },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_InterRATMeasurement_r9(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_InterRATMeasurement_r9, InterRATMeasurement_r9_sequence);

  return offset;
}


static const per_sequence_t DGPS_CorrectionSatInfo_r9_sequence[] = {
  { &hf_rrc_satID           , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SatID },
  { &hf_rrc_iode            , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_IODE },
  { &hf_rrc_udre            , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UDRE },
  { &hf_rrc_prc             , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PRC },
  { &hf_rrc_rrc             , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RRC },
  { &hf_rrc_udreGrowthRate  , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UDREGrowthRate },
  { &hf_rrc_udreValidityTime, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UDREValidityTime },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_DGPS_CorrectionSatInfo_r9(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_DGPS_CorrectionSatInfo_r9, DGPS_CorrectionSatInfo_r9_sequence);

  return offset;
}


static const per_sequence_t DGPS_CorrectionSatInfoList_r9_sequence_of[1] = {
  { &hf_rrc_DGPS_CorrectionSatInfoList_r9_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DGPS_CorrectionSatInfo_r9 },
};

static int
dissect_rrc_DGPS_CorrectionSatInfoList_r9(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_DGPS_CorrectionSatInfoList_r9, DGPS_CorrectionSatInfoList_r9_sequence_of,
                                                  1, maxSat, FALSE);

  return offset;
}


static const per_sequence_t UE_Positioning_GPS_DGPS_Corrections_r9_sequence[] = {
  { &hf_rrc_gps_TOW         , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_GPS_TOW_1sec },
  { &hf_rrc_statusHealth    , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DiffCorrectionStatus },
  { &hf_rrc_dgps_CorrectionSatInfoList_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DGPS_CorrectionSatInfoList_r9 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UE_Positioning_GPS_DGPS_Corrections_r9(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UE_Positioning_GPS_DGPS_Corrections_r9, UE_Positioning_GPS_DGPS_Corrections_r9_sequence);

  return offset;
}


static const per_sequence_t UE_Positioning_GPS_AssistanceData_r9_sequence[] = {
  { &hf_rrc_ue_positioning_GPS_ReferenceTime_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_Positioning_GPS_ReferenceTime_r7 },
  { &hf_rrc_ue_positioning_GPS_ReferenceLocation, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_ReferenceLocation },
  { &hf_rrc_ue_positioning_GPS_DGPS_Corrections_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_Positioning_GPS_DGPS_Corrections_r9 },
  { &hf_rrc_ue_positioning_GPS_NavigationModel, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_Positioning_GPS_NavigationModel },
  { &hf_rrc_ue_positioning_GPS_IonosphericModel, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_Positioning_GPS_IonosphericModel },
  { &hf_rrc_ue_positioning_GPS_UTC_Model, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_Positioning_GPS_UTC_Model },
  { &hf_rrc_ue_positioning_GPS_Almanac, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_Positioning_GPS_Almanac },
  { &hf_rrc_ue_positioning_GPS_AcquisitionAssistance_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_Positioning_GPS_AcquisitionAssistance_r7 },
  { &hf_rrc_ue_positioning_GPS_Real_timeIntegrity, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_BadSatList },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UE_Positioning_GPS_AssistanceData_r9(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UE_Positioning_GPS_AssistanceData_r9, UE_Positioning_GPS_AssistanceData_r9_sequence);

  return offset;
}


static const per_sequence_t DGANSSSignalInformation_r9_sequence[] = {
  { &hf_rrc_satId           , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_0_63 },
  { &hf_rrc_iode_dganss     , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_10 },
  { &hf_rrc_udre            , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UDRE },
  { &hf_rrc_ganss_prc       , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_M2047_2047 },
  { &hf_rrc_ganss_rrc       , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_M127_127 },
  { &hf_rrc_udreGrowthRate  , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UDREGrowthRate },
  { &hf_rrc_udreValidityTime, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UDREValidityTime },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_DGANSSSignalInformation_r9(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_DGANSSSignalInformation_r9, DGANSSSignalInformation_r9_sequence);

  return offset;
}


static const per_sequence_t DGANSSSignalInformationList_r9_sequence_of[1] = {
  { &hf_rrc_DGANSSSignalInformationList_r9_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DGANSSSignalInformation_r9 },
};

static int
dissect_rrc_DGANSSSignalInformationList_r9(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_DGANSSSignalInformationList_r9, DGANSSSignalInformationList_r9_sequence_of,
                                                  1, maxGANSSSat, FALSE);

  return offset;
}


static const per_sequence_t DGANSSInfo_r9_sequence[] = {
  { &hf_rrc_ganssSignalId   , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_GANSS_Signal_Id },
  { &hf_rrc_ganssStatusHealth, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_GANSS_Status_Health },
  { &hf_rrc_dgansssignalInformationList_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DGANSSSignalInformationList_r9 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_DGANSSInfo_r9(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_DGANSSInfo_r9, DGANSSInfo_r9_sequence);

  return offset;
}


static const per_sequence_t DGANSSInfoList_r9_sequence_of[1] = {
  { &hf_rrc_DGANSSInfoList_r9_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DGANSSInfo_r9 },
};

static int
dissect_rrc_DGANSSInfoList_r9(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_DGANSSInfoList_r9, DGANSSInfoList_r9_sequence_of,
                                                  1, maxSgnType, FALSE);

  return offset;
}


static const per_sequence_t UE_Positioning_DGANSSCorrections_r9_sequence[] = {
  { &hf_rrc_dganssreferencetime, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_0_119 },
  { &hf_rrc_dganssInfoList_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DGANSSInfoList_r9 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UE_Positioning_DGANSSCorrections_r9(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UE_Positioning_DGANSSCorrections_r9, UE_Positioning_DGANSSCorrections_r9_sequence);

  return offset;
}


static const per_sequence_t GANSSGenericData_r9_sequence[] = {
  { &hf_rrc_ganssId         , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_INTEGER_0_7 },
  { &hf_rrc_uePositiningGANSSsbasID, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_Positioning_GANSS_SBAS_ID },
  { &hf_rrc_ganssTimeModelsList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_GANSSTimeModelsList },
  { &hf_rrc_uePositioningDGANSSCorrections_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_Positioning_DGANSSCorrections_r9 },
  { &hf_rrc_uePositioningGANSSNavigationModel, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_Positioning_GANSS_NavigationModel },
  { &hf_rrc_uePositioningGANSSAddNavigationModels, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_Positioning_GANSS_AddNavigationModels },
  { &hf_rrc_uePositioningGANSSRealTimeIntegrity, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_Positioning_GANSS_RealTimeIntegrity },
  { &hf_rrc_uePositioningGANSSDataBitAssistance, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_Positioning_GANSS_Data_Bit_Assistance },
  { &hf_rrc_uePositioningGANSSReferenceMeasurementInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_Positioning_GANSS_ReferenceMeasurementInfo },
  { &hf_rrc_uePositioningGANSSAlmanac_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_Positioning_GANSS_Almanac_r8 },
  { &hf_rrc_uePositioningGANSSUTCModel, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_Positioning_GANSS_UTCModel },
  { &hf_rrc_uePositioningGANSSAddUTCModels, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_Positioning_GANSS_AddUTCModels },
  { &hf_rrc_uePositioningGANSSAuxiliaryInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_Positioning_GANSS_AuxiliaryInfo },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_GANSSGenericData_r9(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_GANSSGenericData_r9, GANSSGenericData_r9_sequence);

  return offset;
}


static const per_sequence_t GANSSGenericDataList_r9_sequence_of[1] = {
  { &hf_rrc_GANSSGenericDataList_r9_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_GANSSGenericData_r9 },
};

static int
dissect_rrc_GANSSGenericDataList_r9(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_GANSSGenericDataList_r9, GANSSGenericDataList_r9_sequence_of,
                                                  1, maxGANSS, FALSE);

  return offset;
}


static const per_sequence_t UE_Positioning_GANSS_AssistanceData_r9_sequence[] = {
  { &hf_rrc_ue_positioning_GANSS_ReferenceTime, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_Positioning_GANSS_ReferenceTime },
  { &hf_rrc_uePositioningGanssReferencePosition, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_ReferenceLocationGANSS },
  { &hf_rrc_uePositioningGanssIonosphericModel, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_Positioning_GANSS_IonosphericModel },
  { &hf_rrc_uePositioningGanssAddIonoModel, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_Positioning_GANSS_AddIonoModel },
  { &hf_rrc_uePositioningGanssEarthOrientationPara, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_Positioning_GANSS_EarthOrientPara },
  { &hf_rrc_ganssGenericDataList_03, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_GANSSGenericDataList_r9 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UE_Positioning_GANSS_AssistanceData_r9(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UE_Positioning_GANSS_AssistanceData_r9, UE_Positioning_GANSS_AssistanceData_r9_sequence);

  return offset;
}


static const per_sequence_t UE_Positioning_Measurement_r9_sequence[] = {
  { &hf_rrc_ue_positioning_ReportingQuantity_03, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UE_Positioning_ReportingQuantity_r8 },
  { &hf_rrc_measurementValidity, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MeasurementValidity },
  { &hf_rrc_reportCriteria_21, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UE_Positioning_ReportCriteria_r7 },
  { &hf_rrc_ue_positioning_OTDOA_AssistanceData_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_Positioning_OTDOA_AssistanceData_r7 },
  { &hf_rrc_ue_positioning_GPS_AssistanceData_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_Positioning_GPS_AssistanceData_r9 },
  { &hf_rrc_ue_positioning_GANSS_AssistanceData_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_Positioning_GANSS_AssistanceData_r9 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UE_Positioning_Measurement_r9(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UE_Positioning_Measurement_r9, UE_Positioning_Measurement_r9_sequence);

  return offset;
}


static const value_string rrc_T_uTRACSGProximityDetec_vals[] = {
  {   0, "enable" },
  { 0, NULL }
};


static int
dissect_rrc_T_uTRACSGProximityDetec(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}


static const value_string rrc_T_e_UTRACSGProximityDetec_vals[] = {
  {   0, "enable" },
  { 0, NULL }
};


static int
dissect_rrc_T_e_UTRACSGProximityDetec(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t CSGProximityDetection_sequence[] = {
  { &hf_rrc_uTRACSGProximityDetec, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_uTRACSGProximityDetec },
  { &hf_rrc_e_UTRACSGProximityDetec, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_e_UTRACSGProximityDetec },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_CSGProximityDetection(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_CSGProximityDetection, CSGProximityDetection_sequence);

  return offset;
}


static const value_string rrc_MeasurementType_r9_vals[] = {
  {   0, "intraFrequencyMeasurement" },
  {   1, "interFrequencyMeasurement" },
  {   2, "interRATMeasurement" },
  {   3, "up-Measurement" },
  {   4, "trafficVolumeMeasurement" },
  {   5, "qualityMeasurement" },
  {   6, "ue-InternalMeasurement" },
  {   7, "csgProximityDetection" },
  { 0, NULL }
};

static const per_choice_t MeasurementType_r9_choice[] = {
  {   0, &hf_rrc_intraFrequencyMeasurement_04, ASN1_NO_EXTENSIONS     , dissect_rrc_IntraFrequencyMeasurement_r9 },
  {   1, &hf_rrc_interFrequencyMeasurement_05, ASN1_NO_EXTENSIONS     , dissect_rrc_InterFrequencyMeasurement_r9 },
  {   2, &hf_rrc_interRATMeasurement_04, ASN1_NO_EXTENSIONS     , dissect_rrc_InterRATMeasurement_r9 },
  {   3, &hf_rrc_up_Measurement_03, ASN1_NO_EXTENSIONS     , dissect_rrc_UE_Positioning_Measurement_r9 },
  {   4, &hf_rrc_trafficVolumeMeasurement, ASN1_NO_EXTENSIONS     , dissect_rrc_TrafficVolumeMeasurement },
  {   5, &hf_rrc_qualityMeasurement, ASN1_NO_EXTENSIONS     , dissect_rrc_QualityMeasurement },
  {   6, &hf_rrc_ue_InternalMeasurement_01, ASN1_NO_EXTENSIONS     , dissect_rrc_UE_InternalMeasurement_r4 },
  {   7, &hf_rrc_csgProximityDetection, ASN1_NO_EXTENSIONS     , dissect_rrc_CSGProximityDetection },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_MeasurementType_r9(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_MeasurementType_r9, MeasurementType_r9_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t T_modify_06_sequence[] = {
  { &hf_rrc_measurementType_05, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MeasurementType_r9 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_modify_06(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_modify_06, T_modify_06_sequence);

  return offset;
}


static const value_string rrc_MeasurementCommand_r9_vals[] = {
  {   0, "setup" },
  {   1, "modify" },
  {   2, "release" },
  { 0, NULL }
};

static const per_choice_t MeasurementCommand_r9_choice[] = {
  {   0, &hf_rrc_setup_05        , ASN1_NO_EXTENSIONS     , dissect_rrc_MeasurementType_r9 },
  {   1, &hf_rrc_modify_06       , ASN1_NO_EXTENSIONS     , dissect_rrc_T_modify_06 },
  {   2, &hf_rrc_release_01      , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_MeasurementCommand_r9(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_MeasurementCommand_r9, MeasurementCommand_r9_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t MeasurementControl_r9_IEs_sequence[] = {
  { &hf_rrc_measurementIdentity, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MeasurementIdentity },
  { &hf_rrc_measurementCommand_05, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MeasurementCommand_r9 },
  { &hf_rrc_measurementReportingMode, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MeasurementReportingMode },
  { &hf_rrc_additionalMeasurementList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_AdditionalMeasurementID_List },
  { &hf_rrc_cellDCHMeasOccasionInfo_TDD128, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CellDCHMeasOccasionInfo_TDD128_r9 },
  { &hf_rrc_dpch_CompressedModeStatusInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DPCH_CompressedModeStatusInfo },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_MeasurementControl_r9_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_MeasurementControl_r9_IEs, MeasurementControl_r9_IEs_sequence);

  return offset;
}



static int
dissect_rrc_MeasurementIdentity_r9(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            1U, 32U, NULL, FALSE);

  return offset;
}


static const per_sequence_t AdditionalMeasurementID_List_r9_sequence_of[1] = {
  { &hf_rrc_AdditionalMeasurementID_List_r9_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MeasurementIdentity_r9 },
};

static int
dissect_rrc_AdditionalMeasurementID_List_r9(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_AdditionalMeasurementID_List_r9, AdditionalMeasurementID_List_r9_sequence_of,
                                                  1, maxAdditionalMeas, FALSE);

  return offset;
}


static const per_sequence_t MeasurementControl_v970ext_IEs_sequence[] = {
  { &hf_rrc_measurementIdentity_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MeasurementIdentity_r9 },
  { &hf_rrc_additionalMeasurementList_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_AdditionalMeasurementID_List_r9 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_MeasurementControl_v970ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_MeasurementControl_v970ext_IEs, MeasurementControl_v970ext_IEs_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions_63_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_nonCriticalExtensions_63(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_nonCriticalExtensions_63, T_nonCriticalExtensions_63_sequence);

  return offset;
}


static const per_sequence_t T_v970NonCriticalExtensions_sequence[] = {
  { &hf_rrc_measurementControl_v970ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MeasurementControl_v970ext_IEs },
  { &hf_rrc_nonCriticalExtensions_63, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_nonCriticalExtensions_63 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v970NonCriticalExtensions(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v970NonCriticalExtensions, T_v970NonCriticalExtensions_sequence);

  return offset;
}


static const per_sequence_t T_r9_04_sequence[] = {
  { &hf_rrc_measurementControl_r9, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MeasurementControl_r9_IEs },
  { &hf_rrc_measurementControl_r9_add_ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_BIT_STRING },
  { &hf_rrc_v970NonCriticalExtensions, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v970NonCriticalExtensions },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_r9_04(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_r9_04, T_r9_04_sequence);

  return offset;
}


static const per_sequence_t T_tdd128_65_sequence[] = {
  { &hf_rrc_sNPLMonitorSetIndicator_TDD128, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_BIT_STRING_SIZE_5 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_tdd128_65(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_tdd128_65, T_tdd128_65_sequence);

  return offset;
}


static const value_string rrc_T_modeSpecificInfo_128_vals[] = {
  {   0, "noInfo" },
  {   1, "tdd128" },
  { 0, NULL }
};

static const per_choice_t T_modeSpecificInfo_128_choice[] = {
  {   0, &hf_rrc_noInfo          , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   1, &hf_rrc_tdd128_67       , ASN1_NO_EXTENSIONS     , dissect_rrc_T_tdd128_65 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_modeSpecificInfo_128(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_modeSpecificInfo_128, T_modeSpecificInfo_128_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t NewIntraFreqCell_r10_sequence[] = {
  { &hf_rrc_intraFreqCellID , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_IntraFreqCellID },
  { &hf_rrc_cellInfo_02     , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CellInfo_r9 },
  { &hf_rrc_modeSpecificInfo_128, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_modeSpecificInfo_128 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_NewIntraFreqCell_r10(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_NewIntraFreqCell_r10, NewIntraFreqCell_r10_sequence);

  return offset;
}


static const per_sequence_t NewIntraFreqCellList_r10_sequence_of[1] = {
  { &hf_rrc_NewIntraFreqCellList_r10_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_NewIntraFreqCell_r10 },
};

static int
dissect_rrc_NewIntraFreqCellList_r10(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_NewIntraFreqCellList_r10, NewIntraFreqCellList_r10_sequence_of,
                                                  1, maxCellMeas, FALSE);

  return offset;
}


static const per_sequence_t IntraFreqCellInfoList_r10_sequence[] = {
  { &hf_rrc_removedIntraFreqCellList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RemovedIntraFreqCellList },
  { &hf_rrc_newIntraFreqCellList_05, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_NewIntraFreqCellList_r10 },
  { &hf_rrc_cellsForIntraFreqMeasList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CellsForIntraFreqMeasList },
  { &hf_rrc_cSGIntraFreqCellInfoList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CSGIntraFreqCellInfoList },
  { &hf_rrc_intraFreqSIAcquisition, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_IntraFreqSIAcquisition },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_IntraFreqCellInfoList_r10(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_IntraFreqCellInfoList_r10, IntraFreqCellInfoList_r10_sequence);

  return offset;
}


static const per_sequence_t IntraFrequencyMeasurement_r10_sequence[] = {
  { &hf_rrc_intraFreqCellInfoList_03, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_IntraFreqCellInfoList_r10 },
  { &hf_rrc_intraFreqCellInfoListOnSecULFreq_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_IntraFreqCellInfoListOnSecULFreq },
  { &hf_rrc_intraFreqMeasQuantity, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_IntraFreqMeasQuantity },
  { &hf_rrc_intraFreqReportingQuantity, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_IntraFreqReportingQuantity },
  { &hf_rrc_measurementValidity, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MeasurementValidity },
  { &hf_rrc_reportCriteria_13, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_IntraFreqReportCriteria_r9 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_IntraFrequencyMeasurement_r10(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_IntraFrequencyMeasurement_r10, IntraFrequencyMeasurement_r10_sequence);

  return offset;
}


static const per_sequence_t T_tdd128_64_sequence[] = {
  { &hf_rrc_intraSecondaryFreqIndicator, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { &hf_rrc_sNPLMonitorSetIndicator_TDD128, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_BIT_STRING_SIZE_5 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_tdd128_64(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_tdd128_64, T_tdd128_64_sequence);

  return offset;
}


static const value_string rrc_T_modeSpecificInfo_127_vals[] = {
  {   0, "noInfo" },
  {   1, "tdd128" },
  { 0, NULL }
};

static const per_choice_t T_modeSpecificInfo_127_choice[] = {
  {   0, &hf_rrc_noInfo          , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   1, &hf_rrc_tdd128_66       , ASN1_NO_EXTENSIONS     , dissect_rrc_T_tdd128_64 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_modeSpecificInfo_127(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_modeSpecificInfo_127, T_modeSpecificInfo_127_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t NewInterFreqCell_r10_sequence[] = {
  { &hf_rrc_interFreqCellID , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_InterFreqCellID },
  { &hf_rrc_frequencyInfo   , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_FrequencyInfo },
  { &hf_rrc_cellInfo_02     , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CellInfo_r9 },
  { &hf_rrc_modeSpecificInfo_127, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_modeSpecificInfo_127 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_NewInterFreqCell_r10(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_NewInterFreqCell_r10, NewInterFreqCell_r10_sequence);

  return offset;
}


static const per_sequence_t NewInterFreqCellList_r10_sequence_of[1] = {
  { &hf_rrc_NewInterFreqCellList_r10_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_NewInterFreqCell_r10 },
};

static int
dissect_rrc_NewInterFreqCellList_r10(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_NewInterFreqCellList_r10, NewInterFreqCellList_r10_sequence_of,
                                                  1, maxCellMeas, FALSE);

  return offset;
}


static const per_sequence_t InterFreqCellInfoList_r10_sequence[] = {
  { &hf_rrc_removedInterFreqCellList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RemovedInterFreqCellList },
  { &hf_rrc_newInterFreqCellList_06, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_NewInterFreqCellList_r10 },
  { &hf_rrc_cellsForInterFreqMeasList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CellsForInterFreqMeasList },
  { &hf_rrc_cSGInterFreqCellInfoList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CSGInterFreqCellInfoList },
  { &hf_rrc_interFreqSIAcquisition, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_InterFreqSIAcquisition },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_InterFreqCellInfoList_r10(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_InterFreqCellInfoList_r10, InterFreqCellInfoList_r10_sequence);

  return offset;
}



static int
dissect_rrc_FrequencyIndexForEnhancedMeas(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            0U, maxCellMeas_1, NULL, FALSE);

  return offset;
}


static const per_sequence_t FreqIndexListForEnhancedMeas_sequence_of[1] = {
  { &hf_rrc_FreqIndexListForEnhancedMeas_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_FrequencyIndexForEnhancedMeas },
};

static int
dissect_rrc_FreqIndexListForEnhancedMeas(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_FreqIndexListForEnhancedMeas, FreqIndexListForEnhancedMeas_sequence_of,
                                                  1, maxFreqMeasWithoutCM, FALSE);

  return offset;
}


static const value_string rrc_ReportingCellStatus_r10_vals[] = {
  {   0, "withinActiveSet" },
  {   1, "withinMonitoredSetUsedFreq" },
  {   2, "withinActiveAndOrMonitoredUsedFreq" },
  {   3, "withinDetectedSetUsedFreq" },
  {   4, "withinMonitoredAndOrDetectedUsedFreq" },
  {   5, "allActiveplusMonitoredSet" },
  {   6, "allActivePlusDetectedSet" },
  {   7, "allActivePlusMonitoredAndOrDetectedSet" },
  {   8, "withinVirtualActSet" },
  {   9, "withinMonitoredSetNonUsedFreq" },
  {  10, "withinMonitoredAndOrVirtualActiveSetNonUsedFreq" },
  {  11, "allVirtualActSetplusMonitoredSetNonUsedFreq" },
  {  12, "withinActSetOrVirtualActSet-InterRATcells" },
  {  13, "withinActSetAndOrMonitoredUsedFreqOrVirtualActSetAndOrMonitoredNonUsedFreq" },
  {  14, "allVirtualActSetplusMonitoredSetplusDetectedSetNonUsedFreq" },
  { 0, NULL }
};

static const per_choice_t ReportingCellStatus_r10_choice[] = {
  {   0, &hf_rrc_withinActiveSet , ASN1_NO_EXTENSIONS     , dissect_rrc_MaxNumberOfReportingCellsType1 },
  {   1, &hf_rrc_withinMonitoredSetUsedFreq, ASN1_NO_EXTENSIONS     , dissect_rrc_MaxNumberOfReportingCellsType1 },
  {   2, &hf_rrc_withinActiveAndOrMonitoredUsedFreq, ASN1_NO_EXTENSIONS     , dissect_rrc_MaxNumberOfReportingCellsType1 },
  {   3, &hf_rrc_withinDetectedSetUsedFreq, ASN1_NO_EXTENSIONS     , dissect_rrc_MaxNumberOfReportingCellsType1 },
  {   4, &hf_rrc_withinMonitoredAndOrDetectedUsedFreq, ASN1_NO_EXTENSIONS     , dissect_rrc_MaxNumberOfReportingCellsType1 },
  {   5, &hf_rrc_allActiveplusMonitoredSet, ASN1_NO_EXTENSIONS     , dissect_rrc_MaxNumberOfReportingCellsType3 },
  {   6, &hf_rrc_allActivePlusDetectedSet, ASN1_NO_EXTENSIONS     , dissect_rrc_MaxNumberOfReportingCellsType3 },
  {   7, &hf_rrc_allActivePlusMonitoredAndOrDetectedSet, ASN1_NO_EXTENSIONS     , dissect_rrc_MaxNumberOfReportingCellsType3 },
  {   8, &hf_rrc_withinVirtualActSet, ASN1_NO_EXTENSIONS     , dissect_rrc_MaxNumberOfReportingCellsType1 },
  {   9, &hf_rrc_withinMonitoredSetNonUsedFreq, ASN1_NO_EXTENSIONS     , dissect_rrc_MaxNumberOfReportingCellsType1 },
  {  10, &hf_rrc_withinMonitoredAndOrVirtualActiveSetNonUsedFreq, ASN1_NO_EXTENSIONS     , dissect_rrc_MaxNumberOfReportingCellsType1 },
  {  11, &hf_rrc_allVirtualActSetplusMonitoredSetNonUsedFreq, ASN1_NO_EXTENSIONS     , dissect_rrc_MaxNumberOfReportingCellsType3 },
  {  12, &hf_rrc_withinActSetOrVirtualActSet_InterRATcells, ASN1_NO_EXTENSIONS     , dissect_rrc_MaxNumberOfReportingCellsType2 },
  {  13, &hf_rrc_withinActSetAndOrMonitoredUsedFreqOrVirtualActSetAndOrMonitoredNonUsedFreq, ASN1_NO_EXTENSIONS     , dissect_rrc_MaxNumberOfReportingCellsType2 },
  {  14, &hf_rrc_allVirtualActSetplusMonitoredSetplusDetectedSetNonUsedFreq, ASN1_NO_EXTENSIONS     , dissect_rrc_MaxNumberOfReportingCellsType3 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_ReportingCellStatus_r10(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_ReportingCellStatus_r10, ReportingCellStatus_r10_choice,
                                 NULL);

  return offset;
}


static const value_string rrc_T_nonUsedFreqTriggeringConditionDetectedCells_02_vals[] = {
  {   0, "true" },
  { 0, NULL }
};


static int
dissect_rrc_T_nonUsedFreqTriggeringConditionDetectedCells_02(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t NonUsedFreq2aParameter_r10_sequence[] = {
  { &hf_rrc_nonUsedFreqW    , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_W },
  { &hf_rrc_nonUsedFreqTriggeringConditionDetectedCells_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_nonUsedFreqTriggeringConditionDetectedCells_02 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_NonUsedFreq2aParameter_r10(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_NonUsedFreq2aParameter_r10, NonUsedFreq2aParameter_r10_sequence);

  return offset;
}


static const per_sequence_t NonUsedFreq2aParameterList_r10_sequence_of[1] = {
  { &hf_rrc_NonUsedFreq2aParameterList_r10_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_NonUsedFreq2aParameter_r10 },
};

static int
dissect_rrc_NonUsedFreq2aParameterList_r10(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_NonUsedFreq2aParameterList_r10, NonUsedFreq2aParameterList_r10_sequence_of,
                                                  1, maxFreq, FALSE);

  return offset;
}


static const per_sequence_t Event2a_r10_sequence[] = {
  { &hf_rrc_usedFreqW       , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_W },
  { &hf_rrc_hysteresis      , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_HysteresisInterFreq },
  { &hf_rrc_timeToTrigger   , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TimeToTrigger },
  { &hf_rrc_reportingCellStatus_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_ReportingCellStatus_r10 },
  { &hf_rrc_nonUsedFreqParameterList_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_NonUsedFreq2aParameterList_r10 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_Event2a_r10(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_Event2a_r10, Event2a_r10_sequence);

  return offset;
}


static const value_string rrc_T_nonUsedFreqTriggeringConditionDetectedCells_vals[] = {
  {   0, "true" },
  { 0, NULL }
};


static int
dissect_rrc_T_nonUsedFreqTriggeringConditionDetectedCells(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t NonUsedFreqParameter_r10_sequence[] = {
  { &hf_rrc_nonUsedFreqThreshold_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_Threshold_r6 },
  { &hf_rrc_nonUsedFreqW    , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_W },
  { &hf_rrc_nonUsedFreqTriggeringConditionDetectedCells, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_nonUsedFreqTriggeringConditionDetectedCells },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_NonUsedFreqParameter_r10(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_NonUsedFreqParameter_r10, NonUsedFreqParameter_r10_sequence);

  return offset;
}


static const per_sequence_t NonUsedFreqParameterList_r10_sequence_of[1] = {
  { &hf_rrc_NonUsedFreqParameterList_r10_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_NonUsedFreqParameter_r10 },
};

static int
dissect_rrc_NonUsedFreqParameterList_r10(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_NonUsedFreqParameterList_r10, NonUsedFreqParameterList_r10_sequence_of,
                                                  1, maxFreq, FALSE);

  return offset;
}


static const per_sequence_t Event2b_r10_sequence[] = {
  { &hf_rrc_usedFreqThreshold_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_Threshold_r6 },
  { &hf_rrc_usedFreqW       , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_W },
  { &hf_rrc_hysteresis      , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_HysteresisInterFreq },
  { &hf_rrc_timeToTrigger   , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TimeToTrigger },
  { &hf_rrc_reportingCellStatus_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_ReportingCellStatus_r10 },
  { &hf_rrc_nonUsedFreqParameterList_05, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_NonUsedFreqParameterList_r10 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_Event2b_r10(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_Event2b_r10, Event2b_r10_sequence);

  return offset;
}


static const per_sequence_t Event2c_r10_sequence[] = {
  { &hf_rrc_hysteresis      , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_HysteresisInterFreq },
  { &hf_rrc_timeToTrigger   , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TimeToTrigger },
  { &hf_rrc_reportingCellStatus_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_ReportingCellStatus_r10 },
  { &hf_rrc_nonUsedFreqParameterList_05, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_NonUsedFreqParameterList_r10 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_Event2c_r10(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_Event2c_r10, Event2c_r10_sequence);

  return offset;
}


static const per_sequence_t Event2d_r10_sequence[] = {
  { &hf_rrc_usedFreqThreshold_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_Threshold_r6 },
  { &hf_rrc_usedFreqW       , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_W },
  { &hf_rrc_hysteresis      , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_HysteresisInterFreq },
  { &hf_rrc_timeToTrigger   , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TimeToTrigger },
  { &hf_rrc_reportingCellStatus_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_ReportingCellStatus_r10 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_Event2d_r10(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_Event2d_r10, Event2d_r10_sequence);

  return offset;
}


static const per_sequence_t Event2e_r10_sequence[] = {
  { &hf_rrc_hysteresis      , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_HysteresisInterFreq },
  { &hf_rrc_timeToTrigger   , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TimeToTrigger },
  { &hf_rrc_reportingCellStatus_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_ReportingCellStatus_r10 },
  { &hf_rrc_nonUsedFreqParameterList_05, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_NonUsedFreqParameterList_r10 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_Event2e_r10(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_Event2e_r10, Event2e_r10_sequence);

  return offset;
}


static const per_sequence_t Event2f_r10_sequence[] = {
  { &hf_rrc_usedFreqThreshold_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_Threshold_r6 },
  { &hf_rrc_usedFreqW       , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_W },
  { &hf_rrc_hysteresis      , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_HysteresisInterFreq },
  { &hf_rrc_timeToTrigger   , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TimeToTrigger },
  { &hf_rrc_reportingCellStatus_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_ReportingCellStatus_r10 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_Event2f_r10(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_Event2f_r10, Event2f_r10_sequence);

  return offset;
}


static const value_string rrc_InterFreqEvent_r10_vals[] = {
  {   0, "event2a" },
  {   1, "event2b" },
  {   2, "event2c" },
  {   3, "event2d" },
  {   4, "event2e" },
  {   5, "event2f" },
  { 0, NULL }
};

static const per_choice_t InterFreqEvent_r10_choice[] = {
  {   0, &hf_rrc_event2a_02      , ASN1_NO_EXTENSIONS     , dissect_rrc_Event2a_r10 },
  {   1, &hf_rrc_event2b_02      , ASN1_NO_EXTENSIONS     , dissect_rrc_Event2b_r10 },
  {   2, &hf_rrc_event2c_02      , ASN1_NO_EXTENSIONS     , dissect_rrc_Event2c_r10 },
  {   3, &hf_rrc_event2d_02      , ASN1_NO_EXTENSIONS     , dissect_rrc_Event2d_r10 },
  {   4, &hf_rrc_event2e_02      , ASN1_NO_EXTENSIONS     , dissect_rrc_Event2e_r10 },
  {   5, &hf_rrc_event2f_02      , ASN1_NO_EXTENSIONS     , dissect_rrc_Event2f_r10 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_InterFreqEvent_r10(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_InterFreqEvent_r10, InterFreqEvent_r10_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t InterFreqEventList_r10_sequence_of[1] = {
  { &hf_rrc_InterFreqEventList_r10_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_InterFreqEvent_r10 },
};

static int
dissect_rrc_InterFreqEventList_r10(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_InterFreqEventList_r10, InterFreqEventList_r10_sequence_of,
                                                  1, maxMeasEvent, FALSE);

  return offset;
}


static const per_sequence_t InterFreqReportingCriteria_r10_sequence[] = {
  { &hf_rrc_interFreqEventList_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_InterFreqEventList_r10 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_InterFreqReportingCriteria_r10(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_InterFreqReportingCriteria_r10, InterFreqReportingCriteria_r10_sequence);

  return offset;
}


static const per_sequence_t PeriodicalWithReportingCellStatus_r10_sequence[] = {
  { &hf_rrc_periodicalReportingCriteria, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PeriodicalReportingCriteria },
  { &hf_rrc_reportingCellStatus_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_ReportingCellStatus_r10 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_PeriodicalWithReportingCellStatus_r10(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_PeriodicalWithReportingCellStatus_r10, PeriodicalWithReportingCellStatus_r10_sequence);

  return offset;
}


static const per_sequence_t ReportingCellStatusOpt_r10_sequence[] = {
  { &hf_rrc_reportingCellStatus_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_ReportingCellStatus_r10 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_ReportingCellStatusOpt_r10(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_ReportingCellStatusOpt_r10, ReportingCellStatusOpt_r10_sequence);

  return offset;
}


static const value_string rrc_InterFreqReportCriteria_r10_vals[] = {
  {   0, "intraFreqReportingCriteria" },
  {   1, "interFreqReportingCriteria" },
  {   2, "periodicalReportingCriteria" },
  {   3, "noReporting" },
  { 0, NULL }
};

static const per_choice_t InterFreqReportCriteria_r10_choice[] = {
  {   0, &hf_rrc_intraFreqReportingCriteria_06, ASN1_NO_EXTENSIONS     , dissect_rrc_IntraFreqReportingCriteria_r9 },
  {   1, &hf_rrc_interFreqReportingCriteria_03, ASN1_NO_EXTENSIONS     , dissect_rrc_InterFreqReportingCriteria_r10 },
  {   2, &hf_rrc_periodicalReportingCriteria_02, ASN1_NO_EXTENSIONS     , dissect_rrc_PeriodicalWithReportingCellStatus_r10 },
  {   3, &hf_rrc_noReporting_01  , ASN1_NO_EXTENSIONS     , dissect_rrc_ReportingCellStatusOpt_r10 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_InterFreqReportCriteria_r10(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_InterFreqReportCriteria_r10, InterFreqReportCriteria_r10_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t InterFrequencyMeasurement_r10_sequence[] = {
  { &hf_rrc_interFreqCellInfoList_04, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_InterFreqCellInfoList_r10 },
  { &hf_rrc_interFreqMeasQuantity, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_InterFreqMeasQuantity },
  { &hf_rrc_interFreqReportingQuantity, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_InterFreqReportingQuantity },
  { &hf_rrc_measurementValidity, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MeasurementValidity },
  { &hf_rrc_interFreqSetUpdate, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_AutonomousUpdateMode },
  { &hf_rrc_adjacentFrequencyIndex, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_INTEGER_0_31 },
  { &hf_rrc_interBandFrequencyIndex, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_INTEGER_0_31 },
  { &hf_rrc_freqIndexListForEnhancedMeas, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_FreqIndexListForEnhancedMeas },
  { &hf_rrc_reportCriteria_06, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_InterFreqReportCriteria_r10 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_InterFrequencyMeasurement_r10(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_InterFrequencyMeasurement_r10, InterFrequencyMeasurement_r10_sequence);

  return offset;
}


static const per_sequence_t UE_Positioning_GPS_ReferenceTime_r10_sequence[] = {
  { &hf_rrc_gps_Week        , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_0_1023 },
  { &hf_rrc_gps_Week_Cycle_Number, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_INTEGER_0_7 },
  { &hf_rrc_gps_tow_1msec   , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_GPS_TOW_1msec },
  { &hf_rrc_utran_GPSReferenceTime, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UTRAN_GPSReferenceTime },
  { &hf_rrc_ue_Positioning_GPS_ReferenceTimeUncertainty, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_Positioning_GPS_ReferenceTimeUncertainty },
  { &hf_rrc_sfn_tow_Uncertainty, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_SFN_TOW_Uncertainty },
  { &hf_rrc_utran_GPS_DriftRate, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UTRAN_GPS_DriftRate },
  { &hf_rrc_gps_TOW_AssistList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_GPS_TOW_AssistList },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UE_Positioning_GPS_ReferenceTime_r10(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UE_Positioning_GPS_ReferenceTime_r10, UE_Positioning_GPS_ReferenceTime_r10_sequence);

  return offset;
}


static const per_sequence_t UE_Positioning_GPS_Almanac_r10_sequence[] = {
  { &hf_rrc_wn_a            , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_8 },
  { &hf_rrc_complete_Almanac_Provided, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_BOOLEAN },
  { &hf_rrc_almanacSatInfoList, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_AlmanacSatInfoList },
  { &hf_rrc_sv_GlobalHealth , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_BIT_STRING_SIZE_364 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UE_Positioning_GPS_Almanac_r10(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UE_Positioning_GPS_Almanac_r10, UE_Positioning_GPS_Almanac_r10_sequence);

  return offset;
}


static const per_sequence_t AzimuthAndElevation_r10_sequence[] = {
  { &hf_rrc_azimuth         , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_0_31 },
  { &hf_rrc_elevation       , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_0_7 },
  { &hf_rrc_azimuthLSB      , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_0_15 },
  { &hf_rrc_elevationLSB    , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_0_15 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_AzimuthAndElevation_r10(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_AzimuthAndElevation_r10, AzimuthAndElevation_r10_sequence);

  return offset;
}


static const per_sequence_t AcquisitionSatInfo_r10_sequence[] = {
  { &hf_rrc_satID           , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SatID },
  { &hf_rrc_doppler0thOrder , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_M2048_2047 },
  { &hf_rrc_extraDopplerInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_ExtraDopplerInfo },
  { &hf_rrc_codePhase       , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_0_1022 },
  { &hf_rrc_integerCodePhase, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_0_19 },
  { &hf_rrc_gps_BitNumber   , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_0_3 },
  { &hf_rrc_codePhaseSearchWindow, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CodePhaseSearchWindow },
  { &hf_rrc_azimuthAndElevation_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_AzimuthAndElevation_r10 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_AcquisitionSatInfo_r10(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_AcquisitionSatInfo_r10, AcquisitionSatInfo_r10_sequence);

  return offset;
}


static const per_sequence_t AcquisitionSatInfoList_r10_sequence_of[1] = {
  { &hf_rrc_AcquisitionSatInfoList_r10_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_AcquisitionSatInfo_r10 },
};

static int
dissect_rrc_AcquisitionSatInfoList_r10(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_AcquisitionSatInfoList_r10, AcquisitionSatInfoList_r10_sequence_of,
                                                  1, maxSat, FALSE);

  return offset;
}


static const per_sequence_t UE_Positioning_GPS_AcquisitionAssistance_r10_sequence[] = {
  { &hf_rrc_gps_ReferenceTime, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_GPS_TOW_1msec },
  { &hf_rrc_utran_GPSReferenceTime, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UTRAN_GPSReferenceTime },
  { &hf_rrc_ue_Positioning_GPS_ReferenceTimeUncertainty, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_Positioning_GPS_ReferenceTimeUncertainty },
  { &hf_rrc_satelliteInformationList_05, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_AcquisitionSatInfoList_r10 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UE_Positioning_GPS_AcquisitionAssistance_r10(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UE_Positioning_GPS_AcquisitionAssistance_r10, UE_Positioning_GPS_AcquisitionAssistance_r10_sequence);

  return offset;
}


static const per_sequence_t UE_Positioning_GPS_AssistanceData_r10_sequence[] = {
  { &hf_rrc_ue_positioning_GPS_ReferenceTime_04, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_Positioning_GPS_ReferenceTime_r10 },
  { &hf_rrc_ue_positioning_GPS_ReferenceLocation, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_ReferenceLocation },
  { &hf_rrc_ue_positioning_GPS_DGPS_Corrections_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_Positioning_GPS_DGPS_Corrections_r9 },
  { &hf_rrc_ue_positioning_GPS_NavigationModel, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_Positioning_GPS_NavigationModel },
  { &hf_rrc_ue_positioning_GPS_IonosphericModel, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_Positioning_GPS_IonosphericModel },
  { &hf_rrc_ue_positioning_GPS_UTC_Model, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_Positioning_GPS_UTC_Model },
  { &hf_rrc_ue_positioning_GPS_Almanac_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_Positioning_GPS_Almanac_r10 },
  { &hf_rrc_ue_positioning_GPS_AcquisitionAssistance_04, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_Positioning_GPS_AcquisitionAssistance_r10 },
  { &hf_rrc_ue_positioning_GPS_Real_timeIntegrity, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_BadSatList },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UE_Positioning_GPS_AssistanceData_r10(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UE_Positioning_GPS_AssistanceData_r10, UE_Positioning_GPS_AssistanceData_r10_sequence);

  return offset;
}


static const per_sequence_t T_fdd_168_sequence[] = {
  { &hf_rrc_primary_CPICH_Info, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PrimaryCPICH_Info },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_fdd_168(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_fdd_168, T_fdd_168_sequence);

  return offset;
}


static const per_sequence_t T_tdd_117_sequence[] = {
  { &hf_rrc_cellParameters  , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CellParametersID },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_tdd_117(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_tdd_117, T_tdd_117_sequence);

  return offset;
}


static const value_string rrc_T_mode_01_vals[] = {
  {   0, "fdd" },
  {   1, "tdd" },
  { 0, NULL }
};

static const per_choice_t T_mode_01_choice[] = {
  {   0, &hf_rrc_fdd_174         , ASN1_NO_EXTENSIONS     , dissect_rrc_T_fdd_168 },
  {   1, &hf_rrc_tdd_125         , ASN1_NO_EXTENSIONS     , dissect_rrc_T_tdd_117 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_mode_01(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_mode_01, T_mode_01_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t T_utran_ganssreferenceTime_01_sequence[] = {
  { &hf_rrc_timingOfCellFrames, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_0_3999999 },
  { &hf_rrc_mode_01         , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_mode_01 },
  { &hf_rrc_referenceSfn    , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_0_4095 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_utran_ganssreferenceTime_01(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_utran_ganssreferenceTime_01, T_utran_ganssreferenceTime_01_sequence);

  return offset;
}


static const per_sequence_t UE_Positioning_GANSS_ReferenceTime_r10_sequence[] = {
  { &hf_rrc_ganssDay        , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_INTEGER_0_8191 },
  { &hf_rrc_ganss_Day_Cycle_Number, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_INTEGER_0_7 },
  { &hf_rrc_ganssTod        , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_0_86399 },
  { &hf_rrc_ganssTodUncertainty, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_INTEGER_0_127 },
  { &hf_rrc_ganssTimeId     , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_INTEGER_0_7 },
  { &hf_rrc_utran_ganssreferenceTime_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_utran_ganssreferenceTime_01 },
  { &hf_rrc_tutran_ganss_driftRate, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_Tutran_Ganss_DriftRate },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UE_Positioning_GANSS_ReferenceTime_r10(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UE_Positioning_GANSS_ReferenceTime_r10, UE_Positioning_GANSS_ReferenceTime_r10_sequence);

  return offset;
}


static const value_string rrc_T_gnss_to_id_01_vals[] = {
  {   0, "gps" },
  {   1, "galileo" },
  {   2, "qzss" },
  {   3, "glonass" },
  {   4, "spare4" },
  {   5, "spare3" },
  {   6, "spare2" },
  {   7, "spare1" },
  { 0, NULL }
};


static int
dissect_rrc_T_gnss_to_id_01(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     8, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t UE_Positioning_GANSS_TimeModel_r10_sequence[] = {
  { &hf_rrc_ganss_timeModelreferenceTime, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_0_37799 },
  { &hf_rrc_ganss_t_a0      , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_M2147483648_2147483647 },
  { &hf_rrc_ganss_t_a1      , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_INTEGER_M8388608_8388607 },
  { &hf_rrc_ganss_t_a2      , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_INTEGER_M64_63 },
  { &hf_rrc_gnss_to_id_01   , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_gnss_to_id_01 },
  { &hf_rrc_ganss_wk_number_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_INTEGER_0_8191 },
  { &hf_rrc_deltaT          , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_INTEGER_M128_127 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UE_Positioning_GANSS_TimeModel_r10(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UE_Positioning_GANSS_TimeModel_r10, UE_Positioning_GANSS_TimeModel_r10_sequence);

  return offset;
}


static const per_sequence_t GANSSTimeModelsList_r10_sequence_of[1] = {
  { &hf_rrc_GANSSTimeModelsList_r10_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UE_Positioning_GANSS_TimeModel_r10 },
};

static int
dissect_rrc_GANSSTimeModelsList_r10(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_GANSSTimeModelsList_r10, GANSSTimeModelsList_r10_sequence_of,
                                                  1, maxGANSS_1, FALSE);

  return offset;
}


static const per_sequence_t GANSSSatelliteInformation_r10_sequence[] = {
  { &hf_rrc_ganssSatId      , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_0_63 },
  { &hf_rrc_dopplerZeroOrder, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_M2048_2047 },
  { &hf_rrc_extraDoppler    , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_ExtraDoppler },
  { &hf_rrc_codePhase_01    , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_0_1023 },
  { &hf_rrc_integerCodePhase_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_0_127 },
  { &hf_rrc_codePhaseSearchWindow_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_5 },
  { &hf_rrc_azimuthandElevation_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_AzimuthAndElevation_r10 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_GANSSSatelliteInformation_r10(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_GANSSSatelliteInformation_r10, GANSSSatelliteInformation_r10_sequence);

  return offset;
}


static const per_sequence_t GANSSSatelliteInformationList_r10_sequence_of[1] = {
  { &hf_rrc_GANSSSatelliteInformationList_r10_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_GANSSSatelliteInformation_r10 },
};

static int
dissect_rrc_GANSSSatelliteInformationList_r10(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_GANSSSatelliteInformationList_r10, GANSSSatelliteInformationList_r10_sequence_of,
                                                  1, maxGANSSSat, FALSE);

  return offset;
}


static const per_sequence_t UE_Positioning_GANSS_ReferenceMeasurementInfo_r10_sequence[] = {
  { &hf_rrc_ganssSignalId   , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_GANSS_Signal_Id },
  { &hf_rrc_satelliteInformationList_02, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_GANSSSatelliteInformationList_r10 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UE_Positioning_GANSS_ReferenceMeasurementInfo_r10(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UE_Positioning_GANSS_ReferenceMeasurementInfo_r10, UE_Positioning_GANSS_ReferenceMeasurementInfo_r10_sequence);

  return offset;
}


static const per_sequence_t UE_Positioning_GANSS_Almanac_r10_sequence[] = {
  { &hf_rrc_ganss_wk_number , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_0_255 },
  { &hf_rrc_complete_Almanac_Provided, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_BOOLEAN },
  { &hf_rrc_alm_keplerianParameters, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_ALM_keplerianParameters },
  { &hf_rrc_alm_keplerianNAVAlmanac, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_ALM_NAVKeplerianSet },
  { &hf_rrc_alm_keplerianReducedAlmanac, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_ALM_ReducedKeplerianSet },
  { &hf_rrc_alm_keplerianMidiAlmanac, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_ALM_MidiAlmanacSet },
  { &hf_rrc_alm_keplerianGLONASS, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_ALM_GlonassAlmanacSet },
  { &hf_rrc_alm_ecefSBASAlmanac, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_ALM_ECEFsbasAlmanacSet },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UE_Positioning_GANSS_Almanac_r10(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UE_Positioning_GANSS_Almanac_r10, UE_Positioning_GANSS_Almanac_r10_sequence);

  return offset;
}


static const per_sequence_t GANSSGenericData_r10_sequence[] = {
  { &hf_rrc_ganssId         , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_INTEGER_0_7 },
  { &hf_rrc_uePositiningGANSSsbasID, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_Positioning_GANSS_SBAS_ID },
  { &hf_rrc_ganssTimeModelsList_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_GANSSTimeModelsList_r10 },
  { &hf_rrc_uePositioningDGANSSCorrections_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_Positioning_DGANSSCorrections_r9 },
  { &hf_rrc_uePositioningGANSSNavigationModel, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_Positioning_GANSS_NavigationModel },
  { &hf_rrc_uePositioningGANSSAddNavigationModels, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_Positioning_GANSS_AddNavigationModels },
  { &hf_rrc_uePositioningGANSSRealTimeIntegrity, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_Positioning_GANSS_RealTimeIntegrity },
  { &hf_rrc_uePositioningGANSSDataBitAssistance, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_Positioning_GANSS_Data_Bit_Assistance },
  { &hf_rrc_uePositioningGANSSReferenceMeasurementInfo_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_Positioning_GANSS_ReferenceMeasurementInfo_r10 },
  { &hf_rrc_uePositioningGANSSAlmanac_04, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_Positioning_GANSS_Almanac_r10 },
  { &hf_rrc_uePositioningGANSSUTCModel, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_Positioning_GANSS_UTCModel },
  { &hf_rrc_uePositioningGANSSAddUTCModels, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_Positioning_GANSS_AddUTCModels },
  { &hf_rrc_uePositioningGANSSAuxiliaryInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_Positioning_GANSS_AuxiliaryInfo },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_GANSSGenericData_r10(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_GANSSGenericData_r10, GANSSGenericData_r10_sequence);

  return offset;
}


static const per_sequence_t GANSSGenericDataList_r10_sequence_of[1] = {
  { &hf_rrc_GANSSGenericDataList_r10_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_GANSSGenericData_r10 },
};

static int
dissect_rrc_GANSSGenericDataList_r10(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_GANSSGenericDataList_r10, GANSSGenericDataList_r10_sequence_of,
                                                  1, maxGANSS, FALSE);

  return offset;
}


static const per_sequence_t UE_Positioning_GANSS_AssistanceData_r10_sequence[] = {
  { &hf_rrc_ue_positioning_GANSS_ReferenceTime_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_Positioning_GANSS_ReferenceTime_r10 },
  { &hf_rrc_uePositioningGanssReferencePosition, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_ReferenceLocationGANSS },
  { &hf_rrc_uePositioningGanssIonosphericModel, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_Positioning_GANSS_IonosphericModel },
  { &hf_rrc_uePositioningGanssAddIonoModel, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_Positioning_GANSS_AddIonoModel },
  { &hf_rrc_uePositioningGanssEarthOrientationPara, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_Positioning_GANSS_EarthOrientPara },
  { &hf_rrc_ganssGenericDataList_06, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_GANSSGenericDataList_r10 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UE_Positioning_GANSS_AssistanceData_r10(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UE_Positioning_GANSS_AssistanceData_r10, UE_Positioning_GANSS_AssistanceData_r10_sequence);

  return offset;
}


static const per_sequence_t UE_Positioning_Measurement_r10_sequence[] = {
  { &hf_rrc_ue_positioning_ReportingQuantity_03, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UE_Positioning_ReportingQuantity_r8 },
  { &hf_rrc_measurementValidity, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MeasurementValidity },
  { &hf_rrc_reportCriteria_21, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UE_Positioning_ReportCriteria_r7 },
  { &hf_rrc_ue_positioning_OTDOA_AssistanceData_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_Positioning_OTDOA_AssistanceData_r7 },
  { &hf_rrc_ue_positioning_GPS_AssistanceData_03, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_Positioning_GPS_AssistanceData_r10 },
  { &hf_rrc_ue_positioning_GANSS_AssistanceData_03, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_Positioning_GANSS_AssistanceData_r10 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UE_Positioning_Measurement_r10(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UE_Positioning_Measurement_r10, UE_Positioning_Measurement_r10_sequence);

  return offset;
}


static const value_string rrc_MeasurementType_r10_vals[] = {
  {   0, "intraFrequencyMeasurement" },
  {   1, "interFrequencyMeasurement" },
  {   2, "interRATMeasurement" },
  {   3, "up-Measurement" },
  {   4, "trafficVolumeMeasurement" },
  {   5, "qualityMeasurement" },
  {   6, "ue-InternalMeasurement" },
  {   7, "csgProximityDetection" },
  { 0, NULL }
};

static const per_choice_t MeasurementType_r10_choice[] = {
  {   0, &hf_rrc_intraFrequencyMeasurement_05, ASN1_NO_EXTENSIONS     , dissect_rrc_IntraFrequencyMeasurement_r10 },
  {   1, &hf_rrc_interFrequencyMeasurement_06, ASN1_NO_EXTENSIONS     , dissect_rrc_InterFrequencyMeasurement_r10 },
  {   2, &hf_rrc_interRATMeasurement_04, ASN1_NO_EXTENSIONS     , dissect_rrc_InterRATMeasurement_r9 },
  {   3, &hf_rrc_up_Measurement_04, ASN1_NO_EXTENSIONS     , dissect_rrc_UE_Positioning_Measurement_r10 },
  {   4, &hf_rrc_trafficVolumeMeasurement, ASN1_NO_EXTENSIONS     , dissect_rrc_TrafficVolumeMeasurement },
  {   5, &hf_rrc_qualityMeasurement, ASN1_NO_EXTENSIONS     , dissect_rrc_QualityMeasurement },
  {   6, &hf_rrc_ue_InternalMeasurement_01, ASN1_NO_EXTENSIONS     , dissect_rrc_UE_InternalMeasurement_r4 },
  {   7, &hf_rrc_csgProximityDetection, ASN1_NO_EXTENSIONS     , dissect_rrc_CSGProximityDetection },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_MeasurementType_r10(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_MeasurementType_r10, MeasurementType_r10_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t T_modify_07_sequence[] = {
  { &hf_rrc_measurementType_06, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MeasurementType_r10 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_modify_07(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_modify_07, T_modify_07_sequence);

  return offset;
}


static const value_string rrc_MeasurementCommand_r10_vals[] = {
  {   0, "setup" },
  {   1, "modify" },
  {   2, "release" },
  { 0, NULL }
};

static const per_choice_t MeasurementCommand_r10_choice[] = {
  {   0, &hf_rrc_setup_06        , ASN1_NO_EXTENSIONS     , dissect_rrc_MeasurementType_r10 },
  {   1, &hf_rrc_modify_07       , ASN1_NO_EXTENSIONS     , dissect_rrc_T_modify_07 },
  {   2, &hf_rrc_release_01      , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_MeasurementCommand_r10(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_MeasurementCommand_r10, MeasurementCommand_r10_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t T_activate_04_sequence[] = {
  { &hf_rrc_tgcfn           , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TGCFN },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_activate_04(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_activate_04, T_activate_04_sequence);

  return offset;
}


static const value_string rrc_T_tgps_Status_04_vals[] = {
  {   0, "activate" },
  {   1, "deactivate" },
  { 0, NULL }
};

static const per_choice_t T_tgps_Status_04_choice[] = {
  {   0, &hf_rrc_activate_04     , ASN1_NO_EXTENSIONS     , dissect_rrc_T_activate_04 },
  {   1, &hf_rrc_deactivate      , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_tgps_Status_04(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_tgps_Status_04, T_tgps_Status_04_choice,
                                 NULL);

  return offset;
}


static const value_string rrc_T_freqSpecificCompressedMode_01_vals[] = {
  {   0, "true" },
  { 0, NULL }
};


static int
dissect_rrc_T_freqSpecificCompressedMode_01(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t TGP_SequenceShort_r10_sequence[] = {
  { &hf_rrc_tgpsi           , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TGPSI },
  { &hf_rrc_tgps_Status_04  , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_tgps_Status_04 },
  { &hf_rrc_freqSpecificCompressedMode_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_freqSpecificCompressedMode_01 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_TGP_SequenceShort_r10(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_TGP_SequenceShort_r10, TGP_SequenceShort_r10_sequence);

  return offset;
}


static const per_sequence_t SEQUENCE_SIZE_1_maxTGPS_OF_TGP_SequenceShort_r10_sequence_of[1] = {
  { &hf_rrc_tgp_SequenceShortList_item_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TGP_SequenceShort_r10 },
};

static int
dissect_rrc_SEQUENCE_SIZE_1_maxTGPS_OF_TGP_SequenceShort_r10(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_SEQUENCE_SIZE_1_maxTGPS_OF_TGP_SequenceShort_r10, SEQUENCE_SIZE_1_maxTGPS_OF_TGP_SequenceShort_r10_sequence_of,
                                                  1, maxTGPS, FALSE);

  return offset;
}


static const per_sequence_t DPCH_CompressedModeStatusInfo_r10_sequence[] = {
  { &hf_rrc_tgps_Reconfiguration_CFN, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TGPS_Reconfiguration_CFN },
  { &hf_rrc_tgp_SequenceShortList_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SEQUENCE_SIZE_1_maxTGPS_OF_TGP_SequenceShort_r10 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_DPCH_CompressedModeStatusInfo_r10(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_DPCH_CompressedModeStatusInfo_r10, DPCH_CompressedModeStatusInfo_r10_sequence);

  return offset;
}


static const per_sequence_t MeasurementControl_r10_IEs_sequence[] = {
  { &hf_rrc_measurementIdentity, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MeasurementIdentity },
  { &hf_rrc_measurementCommand_06, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MeasurementCommand_r10 },
  { &hf_rrc_measurementReportingMode, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MeasurementReportingMode },
  { &hf_rrc_additionalMeasurementList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_AdditionalMeasurementID_List },
  { &hf_rrc_cellDCHMeasOccasionInfo_TDD128, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CellDCHMeasOccasionInfo_TDD128_r9 },
  { &hf_rrc_dpch_CompressedModeStatusInfo_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DPCH_CompressedModeStatusInfo_r10 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_MeasurementControl_r10_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_MeasurementControl_r10_IEs, MeasurementControl_r10_IEs_sequence);

  return offset;
}


static const per_sequence_t PeriodicalWithReportingCellStatusOnSecULFreq_sequence[] = {
  { &hf_rrc_frequencyInfo   , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_FrequencyInfo },
  { &hf_rrc_periodicalWithReportingCellStatus, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PeriodicalWithReportingCellStatus_r10 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_PeriodicalWithReportingCellStatusOnSecULFreq(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_PeriodicalWithReportingCellStatusOnSecULFreq, PeriodicalWithReportingCellStatusOnSecULFreq_sequence);

  return offset;
}


static const per_sequence_t MeasurementControl_vaa0ext_IEs_sequence[] = {
  { &hf_rrc_periodicalReportingCriteriaOnSecULFreq, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_PeriodicalWithReportingCellStatusOnSecULFreq },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_MeasurementControl_vaa0ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_MeasurementControl_vaa0ext_IEs, MeasurementControl_vaa0ext_IEs_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions_64_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_nonCriticalExtensions_64(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_nonCriticalExtensions_64, T_nonCriticalExtensions_64_sequence);

  return offset;
}


static const per_sequence_t T_vaa0NonCriticalExtensions_sequence[] = {
  { &hf_rrc_measurementControl_vaa0ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MeasurementControl_vaa0ext_IEs },
  { &hf_rrc_nonCriticalExtensions_64, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_nonCriticalExtensions_64 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_vaa0NonCriticalExtensions(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_vaa0NonCriticalExtensions, T_vaa0NonCriticalExtensions_sequence);

  return offset;
}


static const per_sequence_t T_r10_04_sequence[] = {
  { &hf_rrc_measurementControl_r10, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MeasurementControl_r10_IEs },
  { &hf_rrc_measurementControl_r10_add_ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_BIT_STRING },
  { &hf_rrc_vaa0NonCriticalExtensions, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_vaa0NonCriticalExtensions },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_r10_04(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_r10_04, T_r10_04_sequence);

  return offset;
}


static const per_sequence_t CellInfoListToBeExcluded_r11_sequence_of[1] = {
  { &hf_rrc_CellInfoListToBeExcluded_r11_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PrimaryCPICH_Info },
};

static int
dissect_rrc_CellInfoListToBeExcluded_r11(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_CellInfoListToBeExcluded_r11, CellInfoListToBeExcluded_r11_sequence_of,
                                                  1, 64, FALSE);

  return offset;
}


static const per_sequence_t IntraFreqReportingCriteria_r11_sequence[] = {
  { &hf_rrc_eventCriteriaList_04, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_IntraFreqEventCriteriaList_r7 },
  { &hf_rrc_eventCriteriaListOnSecULFreq, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_IntraFreqEventCriteriaListOnSecULFreq },
  { &hf_rrc_excludedCellInfoList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CellInfoListToBeExcluded_r11 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_IntraFreqReportingCriteria_r11(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_IntraFreqReportingCriteria_r11, IntraFreqReportingCriteria_r11_sequence);

  return offset;
}


static const value_string rrc_IntraFreqReportCriteria_r11_vals[] = {
  {   0, "intraFreqReportingCriteria" },
  {   1, "periodicalReportingCriteria" },
  {   2, "noReporting" },
  { 0, NULL }
};

static const per_choice_t IntraFreqReportCriteria_r11_choice[] = {
  {   0, &hf_rrc_intraFreqReportingCriteria_07, ASN1_NO_EXTENSIONS     , dissect_rrc_IntraFreqReportingCriteria_r11 },
  {   1, &hf_rrc_periodicalReportingCriteria_01, ASN1_NO_EXTENSIONS     , dissect_rrc_PeriodicalWithReportingCellStatus },
  {   2, &hf_rrc_noReporting     , ASN1_NO_EXTENSIONS     , dissect_rrc_ReportingCellStatusOpt },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_IntraFreqReportCriteria_r11(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_IntraFreqReportCriteria_r11, IntraFreqReportCriteria_r11_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t IntraFrequencyMeasurement_r11_sequence[] = {
  { &hf_rrc_intraFreqCellInfoList_03, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_IntraFreqCellInfoList_r10 },
  { &hf_rrc_intraFreqCellInfoListOnSecULFreq_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_IntraFreqCellInfoListOnSecULFreq },
  { &hf_rrc_intraFreqMeasQuantity, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_IntraFreqMeasQuantity },
  { &hf_rrc_intraFreqReportingQuantity, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_IntraFreqReportingQuantity },
  { &hf_rrc_measurementValidity, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MeasurementValidity },
  { &hf_rrc_reportCriteria_14, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_IntraFreqReportCriteria_r11 },
  { &hf_rrc_periodicalReportingCriteriaOnSecULFreq, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_PeriodicalWithReportingCellStatusOnSecULFreq },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_IntraFrequencyMeasurement_r11(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_IntraFrequencyMeasurement_r11, IntraFrequencyMeasurement_r11_sequence);

  return offset;
}


static const value_string rrc_T_nonUsedFreqTriggeringConditionDetectedCells_03_vals[] = {
  {   0, "true" },
  { 0, NULL }
};


static int
dissect_rrc_T_nonUsedFreqTriggeringConditionDetectedCells_03(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t NonUsedFreq2aParameter_r11_sequence[] = {
  { &hf_rrc_nonUsedFreqW    , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_W },
  { &hf_rrc_nonUsedFreqTriggeringConditionDetectedCells_03, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_nonUsedFreqTriggeringConditionDetectedCells_03 },
  { &hf_rrc_excludedCellInfoList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CellInfoListToBeExcluded_r11 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_NonUsedFreq2aParameter_r11(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_NonUsedFreq2aParameter_r11, NonUsedFreq2aParameter_r11_sequence);

  return offset;
}


static const per_sequence_t NonUsedFreq2aParameterList_r11_sequence_of[1] = {
  { &hf_rrc_NonUsedFreq2aParameterList_r11_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_NonUsedFreq2aParameter_r11 },
};

static int
dissect_rrc_NonUsedFreq2aParameterList_r11(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_NonUsedFreq2aParameterList_r11, NonUsedFreq2aParameterList_r11_sequence_of,
                                                  1, maxFreq, FALSE);

  return offset;
}


static const per_sequence_t Event2a_r11_sequence[] = {
  { &hf_rrc_usedFreqW       , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_W },
  { &hf_rrc_hysteresis      , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_HysteresisInterFreq },
  { &hf_rrc_timeToTrigger   , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TimeToTrigger },
  { &hf_rrc_reportingCellStatus_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_ReportingCellStatus_r10 },
  { &hf_rrc_nonUsedFreqParameterList_03, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_NonUsedFreq2aParameterList_r11 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_Event2a_r11(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_Event2a_r11, Event2a_r11_sequence);

  return offset;
}


static const value_string rrc_T_nonUsedFreqTriggeringConditionDetectedCells_01_vals[] = {
  {   0, "true" },
  { 0, NULL }
};


static int
dissect_rrc_T_nonUsedFreqTriggeringConditionDetectedCells_01(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t NonUsedFreqParameter_r11_sequence[] = {
  { &hf_rrc_nonUsedFreqThreshold_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_Threshold_r6 },
  { &hf_rrc_nonUsedFreqW    , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_W },
  { &hf_rrc_nonUsedFreqTriggeringConditionDetectedCells_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_nonUsedFreqTriggeringConditionDetectedCells_01 },
  { &hf_rrc_excludedCellInfoList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CellInfoListToBeExcluded_r11 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_NonUsedFreqParameter_r11(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_NonUsedFreqParameter_r11, NonUsedFreqParameter_r11_sequence);

  return offset;
}


static const per_sequence_t NonUsedFreqParameterList_r11_sequence_of[1] = {
  { &hf_rrc_NonUsedFreqParameterList_r11_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_NonUsedFreqParameter_r11 },
};

static int
dissect_rrc_NonUsedFreqParameterList_r11(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_NonUsedFreqParameterList_r11, NonUsedFreqParameterList_r11_sequence_of,
                                                  1, maxFreq, FALSE);

  return offset;
}


static const per_sequence_t Event2b_r11_sequence[] = {
  { &hf_rrc_usedFreqThreshold_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_Threshold_r6 },
  { &hf_rrc_usedFreqW       , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_W },
  { &hf_rrc_hysteresis      , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_HysteresisInterFreq },
  { &hf_rrc_timeToTrigger   , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TimeToTrigger },
  { &hf_rrc_reportingCellStatus_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_ReportingCellStatus_r10 },
  { &hf_rrc_nonUsedFreqParameterList_06, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_NonUsedFreqParameterList_r11 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_Event2b_r11(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_Event2b_r11, Event2b_r11_sequence);

  return offset;
}


static const per_sequence_t Event2c_r11_sequence[] = {
  { &hf_rrc_hysteresis      , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_HysteresisInterFreq },
  { &hf_rrc_timeToTrigger   , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TimeToTrigger },
  { &hf_rrc_reportingCellStatus_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_ReportingCellStatus_r10 },
  { &hf_rrc_nonUsedFreqParameterList_06, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_NonUsedFreqParameterList_r11 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_Event2c_r11(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_Event2c_r11, Event2c_r11_sequence);

  return offset;
}


static const per_sequence_t Event2e_r11_sequence[] = {
  { &hf_rrc_hysteresis      , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_HysteresisInterFreq },
  { &hf_rrc_timeToTrigger   , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TimeToTrigger },
  { &hf_rrc_reportingCellStatus_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_ReportingCellStatus_r10 },
  { &hf_rrc_nonUsedFreqParameterList_06, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_NonUsedFreqParameterList_r11 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_Event2e_r11(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_Event2e_r11, Event2e_r11_sequence);

  return offset;
}


static const value_string rrc_InterFreqEvent_r11_vals[] = {
  {   0, "event2a" },
  {   1, "event2b" },
  {   2, "event2c" },
  {   3, "event2d" },
  {   4, "event2e" },
  {   5, "event2f" },
  { 0, NULL }
};

static const per_choice_t InterFreqEvent_r11_choice[] = {
  {   0, &hf_rrc_event2a_03      , ASN1_NO_EXTENSIONS     , dissect_rrc_Event2a_r11 },
  {   1, &hf_rrc_event2b_03      , ASN1_NO_EXTENSIONS     , dissect_rrc_Event2b_r11 },
  {   2, &hf_rrc_event2c_03      , ASN1_NO_EXTENSIONS     , dissect_rrc_Event2c_r11 },
  {   3, &hf_rrc_event2d_02      , ASN1_NO_EXTENSIONS     , dissect_rrc_Event2d_r10 },
  {   4, &hf_rrc_event2e_03      , ASN1_NO_EXTENSIONS     , dissect_rrc_Event2e_r11 },
  {   5, &hf_rrc_event2f_02      , ASN1_NO_EXTENSIONS     , dissect_rrc_Event2f_r10 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_InterFreqEvent_r11(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_InterFreqEvent_r11, InterFreqEvent_r11_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t InterFreqEventList_r11_sequence_of[1] = {
  { &hf_rrc_InterFreqEventList_r11_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_InterFreqEvent_r11 },
};

static int
dissect_rrc_InterFreqEventList_r11(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_InterFreqEventList_r11, InterFreqEventList_r11_sequence_of,
                                                  1, maxMeasEvent, FALSE);

  return offset;
}


static const per_sequence_t InterFreqReportingCriteria_r11_sequence[] = {
  { &hf_rrc_interFreqEventList_03, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_InterFreqEventList_r11 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_InterFreqReportingCriteria_r11(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_InterFreqReportingCriteria_r11, InterFreqReportingCriteria_r11_sequence);

  return offset;
}


static const value_string rrc_InterFreqReportCriteria_r11_vals[] = {
  {   0, "intraFreqReportingCriteria" },
  {   1, "interFreqReportingCriteria" },
  {   2, "periodicalReportingCriteria" },
  {   3, "noReporting" },
  { 0, NULL }
};

static const per_choice_t InterFreqReportCriteria_r11_choice[] = {
  {   0, &hf_rrc_intraFreqReportingCriteria_07, ASN1_NO_EXTENSIONS     , dissect_rrc_IntraFreqReportingCriteria_r11 },
  {   1, &hf_rrc_interFreqReportingCriteria_04, ASN1_NO_EXTENSIONS     , dissect_rrc_InterFreqReportingCriteria_r11 },
  {   2, &hf_rrc_periodicalReportingCriteria_02, ASN1_NO_EXTENSIONS     , dissect_rrc_PeriodicalWithReportingCellStatus_r10 },
  {   3, &hf_rrc_noReporting_01  , ASN1_NO_EXTENSIONS     , dissect_rrc_ReportingCellStatusOpt_r10 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_InterFreqReportCriteria_r11(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_InterFreqReportCriteria_r11, InterFreqReportCriteria_r11_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t InterFrequencyMeasurement_r11_sequence[] = {
  { &hf_rrc_interFreqCellInfoList_04, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_InterFreqCellInfoList_r10 },
  { &hf_rrc_interFreqMeasQuantity, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_InterFreqMeasQuantity },
  { &hf_rrc_interFreqReportingQuantity, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_InterFreqReportingQuantity },
  { &hf_rrc_measurementValidity, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MeasurementValidity },
  { &hf_rrc_interFreqSetUpdate, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_AutonomousUpdateMode },
  { &hf_rrc_adjacentFrequencyIndex, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_INTEGER_0_31 },
  { &hf_rrc_interBandFrequencyIndex, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_INTEGER_0_31 },
  { &hf_rrc_freqIndexListForEnhancedMeas, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_FreqIndexListForEnhancedMeas },
  { &hf_rrc_reportCriteria_07, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_InterFreqReportCriteria_r11 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_InterFrequencyMeasurement_r11(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_InterFrequencyMeasurement_r11, InterFrequencyMeasurement_r11_sequence);

  return offset;
}


static const value_string rrc_T_widebandRSRQMeasurement_vals[] = {
  {   0, "true" },
  { 0, NULL }
};


static int
dissect_rrc_T_widebandRSRQMeasurement(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t EUTRA_FrequencyInfo_r11_sequence[] = {
  { &hf_rrc_earfcn          , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_EARFCN },
  { &hf_rrc_measurementBandwidth, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_EUTRA_MeasurementBandwidth },
  { &hf_rrc_eutra_blackListedCellList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_EUTRA_BlacklistedCellPerFreqList },
  { &hf_rrc_widebandRSRQMeasurement, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_widebandRSRQMeasurement },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_EUTRA_FrequencyInfo_r11(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_EUTRA_FrequencyInfo_r11, EUTRA_FrequencyInfo_r11_sequence);

  return offset;
}


static const per_sequence_t EUTRA_FrequencyInfoList_r11_sequence_of[1] = {
  { &hf_rrc_EUTRA_FrequencyInfoList_r11_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_EUTRA_FrequencyInfo_r11 },
};

static int
dissect_rrc_EUTRA_FrequencyInfoList_r11(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_EUTRA_FrequencyInfoList_r11, EUTRA_FrequencyInfoList_r11_sequence_of,
                                                  1, maxNumEUTRAFreqs, FALSE);

  return offset;
}


static const per_sequence_t EUTRA_FrequencyList_r11_sequence[] = {
  { &hf_rrc_eutraFrequencyRemoval, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_EUTRA_FrequencyRemoval },
  { &hf_rrc_eutraNewFrequencies_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_EUTRA_FrequencyInfoList_r11 },
  { &hf_rrc_eutraSIAcquisition, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_EUTRA_SIAcquisition },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_EUTRA_FrequencyList_r11(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_EUTRA_FrequencyList_r11, EUTRA_FrequencyList_r11_sequence);

  return offset;
}


static const value_string rrc_T_interRATMeasurementObjects_02_vals[] = {
  {   0, "interRATCellInfoList" },
  {   1, "eutra-FrequencyList" },
  { 0, NULL }
};

static const per_choice_t T_interRATMeasurementObjects_02_choice[] = {
  {   0, &hf_rrc_interRATCellInfoList_02, ASN1_NO_EXTENSIONS     , dissect_rrc_InterRATCellInfoList_r6 },
  {   1, &hf_rrc_eutra_FrequencyList_02, ASN1_NO_EXTENSIONS     , dissect_rrc_EUTRA_FrequencyList_r11 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_interRATMeasurementObjects_02(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_interRATMeasurementObjects_02, T_interRATMeasurementObjects_02_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t InterRATMeasurement_r11_sequence[] = {
  { &hf_rrc_interRATMeasurementObjects_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_interRATMeasurementObjects_02 },
  { &hf_rrc_interRATMeasQuantity_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_InterRATMeasQuantity_r8 },
  { &hf_rrc_interRATReportingQuantity_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_InterRATReportingQuantity_r8 },
  { &hf_rrc_reportCriteria_08, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_InterRATReportCriteria },
  { &hf_rrc_idleIntervalInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_IdleIntervalInfo },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_InterRATMeasurement_r11(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_InterRATMeasurement_r11, InterRATMeasurement_r11_sequence);

  return offset;
}


static const per_sequence_t SEQUENCE_SIZE_1_maxNumEUTRAFreqs_FACH_OF_EARFCN_sequence_of[1] = {
  { &hf_rrc_eutraFrequenciesForMeasurement_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_EARFCN },
};

static int
dissect_rrc_SEQUENCE_SIZE_1_maxNumEUTRAFreqs_FACH_OF_EARFCN(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_SEQUENCE_SIZE_1_maxNumEUTRAFreqs_FACH_OF_EARFCN, SEQUENCE_SIZE_1_maxNumEUTRAFreqs_FACH_OF_EARFCN_sequence_of,
                                                  1, maxNumEUTRAFreqs_FACH, FALSE);

  return offset;
}


static const value_string rrc_T_reportCriteria_vals[] = {
  {   0, "event-triggeredreporting" },
  {   1, "periodicalReporting" },
  { 0, NULL }
};


static int
dissect_rrc_T_reportCriteria(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     2, NULL, FALSE, 0, NULL);

  return offset;
}


static const value_string rrc_T_measurementQuantity_vals[] = {
  {   0, "rsrp" },
  {   1, "rsrq" },
  { 0, NULL }
};


static int
dissect_rrc_T_measurementQuantity(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     2, NULL, FALSE, 0, NULL);

  return offset;
}



static int
dissect_rrc_INTEGER_0_97(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            0U, 97U, NULL, FALSE);

  return offset;
}


static const per_sequence_t EutraMeasurementForCELLFACH_sequence[] = {
  { &hf_rrc_eutraFrequenciesForMeasurement, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SEQUENCE_SIZE_1_maxNumEUTRAFreqs_FACH_OF_EARFCN },
  { &hf_rrc_reportCriteria  , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_reportCriteria },
  { &hf_rrc_measurementQuantity, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_measurementQuantity },
  { &hf_rrc_reportingThreshold, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_0_97 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_EutraMeasurementForCELLFACH(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_EutraMeasurementForCELLFACH, EutraMeasurementForCELLFACH_sequence);

  return offset;
}


static const value_string rrc_MeasurementType_r11_vals[] = {
  {   0, "intraFrequencyMeasurement" },
  {   1, "interFrequencyMeasurement" },
  {   2, "interRATMeasurement" },
  {   3, "up-Measurement" },
  {   4, "trafficVolumeMeasurement" },
  {   5, "qualityMeasurement" },
  {   6, "ue-InternalMeasurement" },
  {   7, "csgProximityDetection" },
  {   8, "eutraMeasurementForCELLFACH" },
  { 0, NULL }
};

static const per_choice_t MeasurementType_r11_choice[] = {
  {   0, &hf_rrc_intraFrequencyMeasurement_06, ASN1_NO_EXTENSIONS     , dissect_rrc_IntraFrequencyMeasurement_r11 },
  {   1, &hf_rrc_interFrequencyMeasurement_07, ASN1_NO_EXTENSIONS     , dissect_rrc_InterFrequencyMeasurement_r11 },
  {   2, &hf_rrc_interRATMeasurement_05, ASN1_NO_EXTENSIONS     , dissect_rrc_InterRATMeasurement_r11 },
  {   3, &hf_rrc_up_Measurement_04, ASN1_NO_EXTENSIONS     , dissect_rrc_UE_Positioning_Measurement_r10 },
  {   4, &hf_rrc_trafficVolumeMeasurement, ASN1_NO_EXTENSIONS     , dissect_rrc_TrafficVolumeMeasurement },
  {   5, &hf_rrc_qualityMeasurement, ASN1_NO_EXTENSIONS     , dissect_rrc_QualityMeasurement },
  {   6, &hf_rrc_ue_InternalMeasurement_01, ASN1_NO_EXTENSIONS     , dissect_rrc_UE_InternalMeasurement_r4 },
  {   7, &hf_rrc_csgProximityDetection, ASN1_NO_EXTENSIONS     , dissect_rrc_CSGProximityDetection },
  {   8, &hf_rrc_eutraMeasurementForCELLFACH, ASN1_NO_EXTENSIONS     , dissect_rrc_EutraMeasurementForCELLFACH },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_MeasurementType_r11(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_MeasurementType_r11, MeasurementType_r11_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t T_modify_08_sequence[] = {
  { &hf_rrc_measurementType_07, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MeasurementType_r11 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_modify_08(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_modify_08, T_modify_08_sequence);

  return offset;
}


static const value_string rrc_MeasurementCommand_r11_vals[] = {
  {   0, "setup" },
  {   1, "modify" },
  {   2, "release" },
  { 0, NULL }
};

static const per_choice_t MeasurementCommand_r11_choice[] = {
  {   0, &hf_rrc_setup_07        , ASN1_NO_EXTENSIONS     , dissect_rrc_MeasurementType_r11 },
  {   1, &hf_rrc_modify_08       , ASN1_NO_EXTENSIONS     , dissect_rrc_T_modify_08 },
  {   2, &hf_rrc_release_01      , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_MeasurementCommand_r11(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_MeasurementCommand_r11, MeasurementCommand_r11_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t MeasurementControl_r11_IEs_sequence[] = {
  { &hf_rrc_measurementIdentity_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MeasurementIdentity_r9 },
  { &hf_rrc_measurementCommand_07, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MeasurementCommand_r11 },
  { &hf_rrc_measurementReportingMode, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MeasurementReportingMode },
  { &hf_rrc_additionalMeasurementList_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_AdditionalMeasurementID_List_r9 },
  { &hf_rrc_cellDCHMeasOccasionInfo_TDD128, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CellDCHMeasOccasionInfo_TDD128_r9 },
  { &hf_rrc_dpch_CompressedModeStatusInfo_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DPCH_CompressedModeStatusInfo_r10 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_MeasurementControl_r11_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_MeasurementControl_r11_IEs, MeasurementControl_r11_IEs_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions_65_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_nonCriticalExtensions_65(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_nonCriticalExtensions_65, T_nonCriticalExtensions_65_sequence);

  return offset;
}


static const per_sequence_t T_r11_05_sequence[] = {
  { &hf_rrc_measurementControl_r11, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MeasurementControl_r11_IEs },
  { &hf_rrc_measurementControl_r11_add_ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_BIT_STRING },
  { &hf_rrc_nonCriticalExtensions_65, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_nonCriticalExtensions_65 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_r11_05(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_r11_05, T_r11_05_sequence);

  return offset;
}


static const per_sequence_t T_criticalExtensions_52_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_criticalExtensions_52(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_criticalExtensions_52, T_criticalExtensions_52_sequence);

  return offset;
}


static const value_string rrc_T_criticalExtensions_51_vals[] = {
  {   0, "r11" },
  {   1, "criticalExtensions" },
  { 0, NULL }
};

static const per_choice_t T_criticalExtensions_51_choice[] = {
  {   0, &hf_rrc_r11_05          , ASN1_NO_EXTENSIONS     , dissect_rrc_T_r11_05 },
  {   1, &hf_rrc_criticalExtensions_53, ASN1_NO_EXTENSIONS     , dissect_rrc_T_criticalExtensions_52 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_criticalExtensions_51(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_criticalExtensions_51, T_criticalExtensions_51_choice,
                                 NULL);

  return offset;
}


static const value_string rrc_T_criticalExtensions_50_vals[] = {
  {   0, "r10" },
  {   1, "criticalExtensions" },
  { 0, NULL }
};

static const per_choice_t T_criticalExtensions_50_choice[] = {
  {   0, &hf_rrc_r10_04          , ASN1_NO_EXTENSIONS     , dissect_rrc_T_r10_04 },
  {   1, &hf_rrc_criticalExtensions_52, ASN1_NO_EXTENSIONS     , dissect_rrc_T_criticalExtensions_51 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_criticalExtensions_50(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_criticalExtensions_50, T_criticalExtensions_50_choice,
                                 NULL);

  return offset;
}


static const value_string rrc_T_criticalExtensions_49_vals[] = {
  {   0, "r9" },
  {   1, "criticalExtensions" },
  { 0, NULL }
};

static const per_choice_t T_criticalExtensions_49_choice[] = {
  {   0, &hf_rrc_r9_04           , ASN1_NO_EXTENSIONS     , dissect_rrc_T_r9_04 },
  {   1, &hf_rrc_criticalExtensions_51, ASN1_NO_EXTENSIONS     , dissect_rrc_T_criticalExtensions_50 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_criticalExtensions_49(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_criticalExtensions_49, T_criticalExtensions_49_choice,
                                 NULL);

  return offset;
}


static const value_string rrc_T_criticalExtensions_48_vals[] = {
  {   0, "r8" },
  {   1, "criticalExtensions" },
  { 0, NULL }
};

static const per_choice_t T_criticalExtensions_48_choice[] = {
  {   0, &hf_rrc_r8_05           , ASN1_NO_EXTENSIONS     , dissect_rrc_T_r8_05 },
  {   1, &hf_rrc_criticalExtensions_50, ASN1_NO_EXTENSIONS     , dissect_rrc_T_criticalExtensions_49 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_criticalExtensions_48(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_criticalExtensions_48, T_criticalExtensions_48_choice,
                                 NULL);

  return offset;
}


static const value_string rrc_T_criticalExtensions_47_vals[] = {
  {   0, "r7" },
  {   1, "criticalExtensions" },
  { 0, NULL }
};

static const per_choice_t T_criticalExtensions_47_choice[] = {
  {   0, &hf_rrc_r7_04           , ASN1_NO_EXTENSIONS     , dissect_rrc_T_r7_04 },
  {   1, &hf_rrc_criticalExtensions_49, ASN1_NO_EXTENSIONS     , dissect_rrc_T_criticalExtensions_48 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_criticalExtensions_47(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_criticalExtensions_47, T_criticalExtensions_47_choice,
                                 NULL);

  return offset;
}


static const value_string rrc_T_criticalExtensions_46_vals[] = {
  {   0, "r6" },
  {   1, "criticalExtensions" },
  { 0, NULL }
};

static const per_choice_t T_criticalExtensions_46_choice[] = {
  {   0, &hf_rrc_r6_05           , ASN1_NO_EXTENSIONS     , dissect_rrc_T_r6_05 },
  {   1, &hf_rrc_criticalExtensions_48, ASN1_NO_EXTENSIONS     , dissect_rrc_T_criticalExtensions_47 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_criticalExtensions_46(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_criticalExtensions_46, T_criticalExtensions_46_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t T_later_than_r4_sequence[] = {
  { &hf_rrc_rrc_TransactionIdentifier_MSP, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RRC_TransactionIdentifier },
  { &hf_rrc_criticalExtensions_47, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_criticalExtensions_46 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_later_than_r4(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_later_than_r4, T_later_than_r4_sequence);

  return offset;
}


static const value_string rrc_T_criticalExtensions_45_vals[] = {
  {   0, "r4" },
  {   1, "later-than-r4" },
  { 0, NULL }
};

static const per_choice_t T_criticalExtensions_45_choice[] = {
  {   0, &hf_rrc_r4_03           , ASN1_NO_EXTENSIONS     , dissect_rrc_T_r4_03 },
  {   1, &hf_rrc_later_than_r4   , ASN1_NO_EXTENSIONS     , dissect_rrc_T_later_than_r4 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_criticalExtensions_45(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_criticalExtensions_45, T_criticalExtensions_45_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t T_later_than_r3_09_sequence[] = {
  { &hf_rrc_rrc_TransactionIdentifier, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RRC_TransactionIdentifier },
  { &hf_rrc_criticalExtensions_46, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_criticalExtensions_45 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_later_than_r3_09(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_later_than_r3_09, T_later_than_r3_09_sequence);

  return offset;
}


static const value_string rrc_MeasurementControl_vals[] = {
  {   0, "r3" },
  {   1, "later-than-r3" },
  { 0, NULL }
};

static const per_choice_t MeasurementControl_choice[] = {
  {   0, &hf_rrc_r3_11           , ASN1_NO_EXTENSIONS     , dissect_rrc_T_r3_11 },
  {   1, &hf_rrc_later_than_r3_09, ASN1_NO_EXTENSIONS     , dissect_rrc_T_later_than_r3_09 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_MeasurementControl(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
#line 285 "../../asn1/rrc/rrc.cnf"
  col_append_str(actx->pinfo->cinfo, COL_INFO, "MeasurementControl");
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_MeasurementControl, MeasurementControl_choice,
                                 NULL);




  return offset;
}


static const value_string rrc_PagingCause_vals[] = {
  {   0, "terminatingConversationalCall" },
  {   1, "terminatingStreamingCall" },
  {   2, "terminatingInteractiveCall" },
  {   3, "terminatingBackgroundCall" },
  {   4, "terminatingHighPrioritySignalling" },
  {   5, "terminatingLowPrioritySignalling" },
  {   6, "terminatingCauseUnknown" },
  {   7, "spare" },
  { 0, NULL }
};


static int
dissect_rrc_PagingCause(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     8, NULL, FALSE, 0, NULL);

  return offset;
}


static const value_string rrc_PagingRecordTypeID_vals[] = {
  {   0, "imsi-GSM-MAP" },
  {   1, "tmsi-GSM-MAP-P-TMSI" },
  {   2, "imsi-DS-41" },
  {   3, "tmsi-DS-41" },
  { 0, NULL }
};


static int
dissect_rrc_PagingRecordTypeID(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     4, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions_69_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_nonCriticalExtensions_69(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_nonCriticalExtensions_69, T_nonCriticalExtensions_69_sequence);

  return offset;
}


static const per_sequence_t T_laterNonCriticalExtensions_22_sequence[] = {
  { &hf_rrc_pagingType2_r3_add_ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_BIT_STRING },
  { &hf_rrc_nonCriticalExtensions_69, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_nonCriticalExtensions_69 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_laterNonCriticalExtensions_22(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_laterNonCriticalExtensions_22, T_laterNonCriticalExtensions_22_sequence);

  return offset;
}


static const per_sequence_t PagingType2_sequence[] = {
  { &hf_rrc_rrc_TransactionIdentifier, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RRC_TransactionIdentifier },
  { &hf_rrc_pagingCause     , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PagingCause },
  { &hf_rrc_cn_DomainIdentity, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CN_DomainIdentity },
  { &hf_rrc_pagingRecordTypeID, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PagingRecordTypeID },
  { &hf_rrc_laterNonCriticalExtensions_22, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_laterNonCriticalExtensions_22 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_PagingType2(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
#line 301 "../../asn1/rrc/rrc.cnf"
  col_append_str(actx->pinfo->cinfo, COL_INFO, "PagingType2");
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_PagingType2, PagingType2_sequence);




  return offset;
}


static const value_string rrc_UL_ChannelRequirementWithCPCH_SetID_vals[] = {
  {   0, "ul-DPCH-Info" },
  {   1, "dummy1" },
  {   2, "dummy2" },
  { 0, NULL }
};

static const per_choice_t UL_ChannelRequirementWithCPCH_SetID_choice[] = {
  {   0, &hf_rrc_ul_DPCH_Info_03 , ASN1_NO_EXTENSIONS     , dissect_rrc_UL_DPCH_Info },
  {   1, &hf_rrc_dummy1_04       , ASN1_NO_EXTENSIONS     , dissect_rrc_CPCH_SetInfo },
  {   2, &hf_rrc_dummy2_10       , ASN1_NO_EXTENSIONS     , dissect_rrc_CPCH_SetID },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_UL_ChannelRequirementWithCPCH_SetID(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_UL_ChannelRequirementWithCPCH_SetID, UL_ChannelRequirementWithCPCH_SetID_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t T_fdd_18_sequence[] = {
  { &hf_rrc_dummy_06        , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_PDSCH_Information },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_fdd_18(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_fdd_18, T_fdd_18_sequence);

  return offset;
}


static const value_string rrc_T_modeSpecificInfo_12_vals[] = {
  {   0, "fdd" },
  {   1, "tdd" },
  { 0, NULL }
};

static const per_choice_t T_modeSpecificInfo_12_choice[] = {
  {   0, &hf_rrc_fdd_18          , ASN1_NO_EXTENSIONS     , dissect_rrc_T_fdd_18 },
  {   1, &hf_rrc_tdd             , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_modeSpecificInfo_12(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_modeSpecificInfo_12, T_modeSpecificInfo_12_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t PhysicalChannelReconfiguration_r3_IEs_sequence[] = {
  { &hf_rrc_rrc_TransactionIdentifier, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RRC_TransactionIdentifier },
  { &hf_rrc_integrityProtectionModeInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_IntegrityProtectionModeInfo },
  { &hf_rrc_cipheringModeInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CipheringModeInfo },
  { &hf_rrc_activationTime  , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_ActivationTime },
  { &hf_rrc_new_U_RNTI      , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_U_RNTI },
  { &hf_rrc_new_C_RNTI      , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_C_RNTI },
  { &hf_rrc_rrc_StateIndicator, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RRC_StateIndicator },
  { &hf_rrc_utran_DRX_CycleLengthCoeff, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UTRAN_DRX_CycleLengthCoefficient },
  { &hf_rrc_cn_InformationInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CN_InformationInfo },
  { &hf_rrc_ura_Identity    , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_URA_Identity },
  { &hf_rrc_dl_CounterSynchronisationInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_CounterSynchronisationInfo },
  { &hf_rrc_frequencyInfo   , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_FrequencyInfo },
  { &hf_rrc_maxAllowedUL_TX_Power, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MaxAllowedUL_TX_Power },
  { &hf_rrc_ul_ChannelRequirement_03, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_ChannelRequirementWithCPCH_SetID },
  { &hf_rrc_modeSpecificInfo_12, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_modeSpecificInfo_12 },
  { &hf_rrc_dl_CommonInformation, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_CommonInformation },
  { &hf_rrc_dl_InformationPerRL_List, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_InformationPerRL_List },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_PhysicalChannelReconfiguration_r3_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_PhysicalChannelReconfiguration_r3_IEs, PhysicalChannelReconfiguration_r3_IEs_sequence);

  return offset;
}


static const per_sequence_t PhysicalChannelReconfiguration_v3a0ext_sequence[] = {
  { &hf_rrc_new_DSCH_RNTI   , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DSCH_RNTI },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_PhysicalChannelReconfiguration_v3a0ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_PhysicalChannelReconfiguration_v3a0ext, PhysicalChannelReconfiguration_v3a0ext_sequence);

  return offset;
}


static const per_sequence_t PhysicalChannelReconfiguration_v4b0ext_IEs_sequence[] = {
  { &hf_rrc_dummy_02        , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_SSDT_UL },
  { &hf_rrc_cell_id_PerRL_List, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CellIdentity_PerRL_List },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_PhysicalChannelReconfiguration_v4b0ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_PhysicalChannelReconfiguration_v4b0ext_IEs, PhysicalChannelReconfiguration_v4b0ext_IEs_sequence);

  return offset;
}


static const per_sequence_t PhysicalChannelReconfiguration_v590ext_IEs_sequence[] = {
  { &hf_rrc_dl_TPC_PowerOffsetPerRL_List, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_TPC_PowerOffsetPerRL_List },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_PhysicalChannelReconfiguration_v590ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_PhysicalChannelReconfiguration_v590ext_IEs, PhysicalChannelReconfiguration_v590ext_IEs_sequence);

  return offset;
}


static const value_string rrc_DelayRestrictionFlag_vals[] = {
  {   0, "true" },
  { 0, NULL }
};


static int
dissect_rrc_DelayRestrictionFlag(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}


static const value_string rrc_T_postVerificationPeriod_01_vals[] = {
  {   0, "true" },
  { 0, NULL }
};


static int
dissect_rrc_T_postVerificationPeriod_01(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t PhysicalChannelReconfiguration_v690ext_IEs_sequence[] = {
  { &hf_rrc_delayRestrictionFlag, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DelayRestrictionFlag },
  { &hf_rrc_primary_plmn_Identity, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_PLMN_Identity },
  { &hf_rrc_harq_Preamble_Mode, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_HARQ_Preamble_Mode },
  { &hf_rrc_beaconPLEst     , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_BEACON_PL_Est },
  { &hf_rrc_postVerificationPeriod_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_postVerificationPeriod_01 },
  { &hf_rrc_dhs_sync        , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DHS_Sync },
  { &hf_rrc_timingMaintainedSynchInd, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_TimingMaintainedSynchInd },
  { &hf_rrc_mbms_PL_ServiceRestrictInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MBMS_PL_ServiceRestrictInfo_r6 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_PhysicalChannelReconfiguration_v690ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_PhysicalChannelReconfiguration_v690ext_IEs, PhysicalChannelReconfiguration_v690ext_IEs_sequence);

  return offset;
}


static const value_string rrc_High_MobilityDetected_vals[] = {
  {   0, "high-MobilityDetected" },
  { 0, NULL }
};


static int
dissect_rrc_High_MobilityDetected(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t PhysicalChannelReconfiguration_v770ext_IEs_sequence[] = {
  { &hf_rrc_ueMobilityStateIndicator, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_High_MobilityDetected },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_PhysicalChannelReconfiguration_v770ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_PhysicalChannelReconfiguration_v770ext_IEs, PhysicalChannelReconfiguration_v770ext_IEs_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions_70_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_nonCriticalExtensions_70(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_nonCriticalExtensions_70, T_nonCriticalExtensions_70_sequence);

  return offset;
}


static const per_sequence_t T_v770NonCriticalExtensions_04_sequence[] = {
  { &hf_rrc_physicalChannelReconfiguration_v770ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PhysicalChannelReconfiguration_v770ext_IEs },
  { &hf_rrc_nonCriticalExtensions_70, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_nonCriticalExtensions_70 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v770NonCriticalExtensions_04(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v770NonCriticalExtensions_04, T_v770NonCriticalExtensions_04_sequence);

  return offset;
}


static const per_sequence_t T_v690NonCriticalExtensions_12_sequence[] = {
  { &hf_rrc_physicalChannelReconfiguration_v690ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PhysicalChannelReconfiguration_v690ext_IEs },
  { &hf_rrc_v770NonCriticalExtensions_04, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v770NonCriticalExtensions_04 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v690NonCriticalExtensions_12(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v690NonCriticalExtensions_12, T_v690NonCriticalExtensions_12_sequence);

  return offset;
}


static const per_sequence_t T_v590NonCriticalExtenstions_02_sequence[] = {
  { &hf_rrc_physicalChannelReconfiguration_v590ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PhysicalChannelReconfiguration_v590ext_IEs },
  { &hf_rrc_v690NonCriticalExtensions_12, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v690NonCriticalExtensions_12 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v590NonCriticalExtenstions_02(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v590NonCriticalExtenstions_02, T_v590NonCriticalExtenstions_02_sequence);

  return offset;
}


static const per_sequence_t T_v4b0NonCriticalExtenstions_sequence[] = {
  { &hf_rrc_physicalChannelReconfiguration_v4b0ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PhysicalChannelReconfiguration_v4b0ext_IEs },
  { &hf_rrc_v590NonCriticalExtenstions_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v590NonCriticalExtenstions_02 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v4b0NonCriticalExtenstions(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v4b0NonCriticalExtenstions, T_v4b0NonCriticalExtenstions_sequence);

  return offset;
}


static const per_sequence_t T_laterNonCriticalExtensions_23_sequence[] = {
  { &hf_rrc_physicalChannelReconfiguration_r3_add_ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_BIT_STRING },
  { &hf_rrc_v4b0NonCriticalExtenstions, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v4b0NonCriticalExtenstions },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_laterNonCriticalExtensions_23(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_laterNonCriticalExtensions_23, T_laterNonCriticalExtensions_23_sequence);

  return offset;
}


static const per_sequence_t T_v3a0NonCriticalExtensions_05_sequence[] = {
  { &hf_rrc_physicalChannelReconfiguration_v3a0ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PhysicalChannelReconfiguration_v3a0ext },
  { &hf_rrc_laterNonCriticalExtensions_23, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_laterNonCriticalExtensions_23 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v3a0NonCriticalExtensions_05(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v3a0NonCriticalExtensions_05, T_v3a0NonCriticalExtensions_05_sequence);

  return offset;
}


static const per_sequence_t T_r3_12_sequence[] = {
  { &hf_rrc_physicalChannelReconfiguration_r3, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PhysicalChannelReconfiguration_r3_IEs },
  { &hf_rrc_v3a0NonCriticalExtensions_05, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v3a0NonCriticalExtensions_05 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_r3_12(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_r3_12, T_r3_12_sequence);

  return offset;
}


static const value_string rrc_UL_ChannelRequirementWithCPCH_SetID_r4_vals[] = {
  {   0, "ul-DPCH-Info" },
  {   1, "dummy1" },
  {   2, "dummy2" },
  { 0, NULL }
};

static const per_choice_t UL_ChannelRequirementWithCPCH_SetID_r4_choice[] = {
  {   0, &hf_rrc_ul_DPCH_Info_06 , ASN1_NO_EXTENSIONS     , dissect_rrc_UL_DPCH_Info_r4 },
  {   1, &hf_rrc_dummy1_04       , ASN1_NO_EXTENSIONS     , dissect_rrc_CPCH_SetInfo },
  {   2, &hf_rrc_dummy2_10       , ASN1_NO_EXTENSIONS     , dissect_rrc_CPCH_SetID },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_UL_ChannelRequirementWithCPCH_SetID_r4(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_UL_ChannelRequirementWithCPCH_SetID_r4, UL_ChannelRequirementWithCPCH_SetID_r4_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t T_fdd_19_sequence[] = {
  { &hf_rrc_dummy_06        , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_PDSCH_Information },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_fdd_19(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_fdd_19, T_fdd_19_sequence);

  return offset;
}


static const value_string rrc_T_modeSpecificInfo_13_vals[] = {
  {   0, "fdd" },
  {   1, "tdd" },
  { 0, NULL }
};

static const per_choice_t T_modeSpecificInfo_13_choice[] = {
  {   0, &hf_rrc_fdd_19          , ASN1_NO_EXTENSIONS     , dissect_rrc_T_fdd_19 },
  {   1, &hf_rrc_tdd             , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_modeSpecificInfo_13(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_modeSpecificInfo_13, T_modeSpecificInfo_13_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t PhysicalChannelReconfiguration_r4_IEs_sequence[] = {
  { &hf_rrc_integrityProtectionModeInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_IntegrityProtectionModeInfo },
  { &hf_rrc_cipheringModeInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CipheringModeInfo },
  { &hf_rrc_activationTime  , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_ActivationTime },
  { &hf_rrc_new_U_RNTI      , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_U_RNTI },
  { &hf_rrc_new_C_RNTI      , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_C_RNTI },
  { &hf_rrc_new_DSCH_RNTI   , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DSCH_RNTI },
  { &hf_rrc_rrc_StateIndicator, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RRC_StateIndicator },
  { &hf_rrc_utran_DRX_CycleLengthCoeff, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UTRAN_DRX_CycleLengthCoefficient },
  { &hf_rrc_cn_InformationInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CN_InformationInfo },
  { &hf_rrc_ura_Identity    , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_URA_Identity },
  { &hf_rrc_dl_CounterSynchronisationInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_CounterSynchronisationInfo },
  { &hf_rrc_frequencyInfo   , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_FrequencyInfo },
  { &hf_rrc_maxAllowedUL_TX_Power, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MaxAllowedUL_TX_Power },
  { &hf_rrc_ul_ChannelRequirement_04, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_ChannelRequirementWithCPCH_SetID_r4 },
  { &hf_rrc_modeSpecificInfo_13, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_modeSpecificInfo_13 },
  { &hf_rrc_dl_CommonInformation_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_CommonInformation_r4 },
  { &hf_rrc_dl_InformationPerRL_List_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_InformationPerRL_List_r4 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_PhysicalChannelReconfiguration_r4_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_PhysicalChannelReconfiguration_r4_IEs, PhysicalChannelReconfiguration_r4_IEs_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions_71_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_nonCriticalExtensions_71(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_nonCriticalExtensions_71, T_nonCriticalExtensions_71_sequence);

  return offset;
}


static const per_sequence_t T_v770NonCriticalExtensions_05_sequence[] = {
  { &hf_rrc_physicalChannelReconfiguration_v770ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PhysicalChannelReconfiguration_v770ext_IEs },
  { &hf_rrc_nonCriticalExtensions_71, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_nonCriticalExtensions_71 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v770NonCriticalExtensions_05(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v770NonCriticalExtensions_05, T_v770NonCriticalExtensions_05_sequence);

  return offset;
}


static const per_sequence_t T_v690NonCriticalExtensions_13_sequence[] = {
  { &hf_rrc_physicalChannelReconfiguration_v690ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PhysicalChannelReconfiguration_v690ext_IEs },
  { &hf_rrc_v770NonCriticalExtensions_05, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v770NonCriticalExtensions_05 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v690NonCriticalExtensions_13(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v690NonCriticalExtensions_13, T_v690NonCriticalExtensions_13_sequence);

  return offset;
}


static const per_sequence_t T_v590NonCriticalExtenstions_03_sequence[] = {
  { &hf_rrc_physicalChannelReconfiguration_v590ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PhysicalChannelReconfiguration_v590ext_IEs },
  { &hf_rrc_v690NonCriticalExtensions_13, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v690NonCriticalExtensions_13 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v590NonCriticalExtenstions_03(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v590NonCriticalExtenstions_03, T_v590NonCriticalExtenstions_03_sequence);

  return offset;
}


static const per_sequence_t T_v4d0NonCriticalExtensions_04_sequence[] = {
  { &hf_rrc_physicalChannelReconfiguration_r4_add_ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_BIT_STRING },
  { &hf_rrc_v590NonCriticalExtenstions_03, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v590NonCriticalExtenstions_03 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v4d0NonCriticalExtensions_04(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v4d0NonCriticalExtensions_04, T_v4d0NonCriticalExtensions_04_sequence);

  return offset;
}


static const per_sequence_t T_r4_04_sequence[] = {
  { &hf_rrc_physicalChannelReconfiguration_r4, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PhysicalChannelReconfiguration_r4_IEs },
  { &hf_rrc_v4d0NonCriticalExtensions_04, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v4d0NonCriticalExtensions_04 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_r4_04(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_r4_04, T_r4_04_sequence);

  return offset;
}


static const value_string rrc_UL_ChannelRequirementWithCPCH_SetID_r5_vals[] = {
  {   0, "ul-DPCH-Info" },
  {   1, "dummy1" },
  {   2, "dummy2" },
  { 0, NULL }
};

static const per_choice_t UL_ChannelRequirementWithCPCH_SetID_r5_choice[] = {
  {   0, &hf_rrc_ul_DPCH_Info_08 , ASN1_NO_EXTENSIONS     , dissect_rrc_UL_DPCH_Info_r5 },
  {   1, &hf_rrc_dummy1_04       , ASN1_NO_EXTENSIONS     , dissect_rrc_CPCH_SetInfo },
  {   2, &hf_rrc_dummy2_10       , ASN1_NO_EXTENSIONS     , dissect_rrc_CPCH_SetID },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_UL_ChannelRequirementWithCPCH_SetID_r5(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_UL_ChannelRequirementWithCPCH_SetID_r5, UL_ChannelRequirementWithCPCH_SetID_r5_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t T_fdd_20_sequence[] = {
  { &hf_rrc_dummy_06        , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_PDSCH_Information },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_fdd_20(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_fdd_20, T_fdd_20_sequence);

  return offset;
}


static const value_string rrc_T_modeSpecificInfo_14_vals[] = {
  {   0, "fdd" },
  {   1, "tdd" },
  { 0, NULL }
};

static const per_choice_t T_modeSpecificInfo_14_choice[] = {
  {   0, &hf_rrc_fdd_20          , ASN1_NO_EXTENSIONS     , dissect_rrc_T_fdd_20 },
  {   1, &hf_rrc_tdd             , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_modeSpecificInfo_14(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_modeSpecificInfo_14, T_modeSpecificInfo_14_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t PhysicalChannelReconfiguration_r5_IEs_sequence[] = {
  { &hf_rrc_integrityProtectionModeInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_IntegrityProtectionModeInfo },
  { &hf_rrc_cipheringModeInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CipheringModeInfo },
  { &hf_rrc_activationTime  , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_ActivationTime },
  { &hf_rrc_new_U_RNTI      , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_U_RNTI },
  { &hf_rrc_new_C_RNTI      , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_C_RNTI },
  { &hf_rrc_new_DSCH_RNTI   , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DSCH_RNTI },
  { &hf_rrc_new_H_RNTI      , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_H_RNTI },
  { &hf_rrc_rrc_StateIndicator, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RRC_StateIndicator },
  { &hf_rrc_utran_DRX_CycleLengthCoeff, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UTRAN_DRX_CycleLengthCoefficient },
  { &hf_rrc_cn_InformationInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CN_InformationInfo },
  { &hf_rrc_ura_Identity    , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_URA_Identity },
  { &hf_rrc_dl_CounterSynchronisationInfo_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_CounterSynchronisationInfo_r5 },
  { &hf_rrc_frequencyInfo   , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_FrequencyInfo },
  { &hf_rrc_maxAllowedUL_TX_Power, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MaxAllowedUL_TX_Power },
  { &hf_rrc_ul_ChannelRequirement_05, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_ChannelRequirementWithCPCH_SetID_r5 },
  { &hf_rrc_modeSpecificInfo_14, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_modeSpecificInfo_14 },
  { &hf_rrc_dl_HSPDSCH_Information, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_HSPDSCH_Information },
  { &hf_rrc_dl_CommonInformation_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_CommonInformation_r5 },
  { &hf_rrc_dl_InformationPerRL_List_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_InformationPerRL_List_r5 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_PhysicalChannelReconfiguration_r5_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_PhysicalChannelReconfiguration_r5_IEs, PhysicalChannelReconfiguration_r5_IEs_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions_72_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_nonCriticalExtensions_72(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_nonCriticalExtensions_72, T_nonCriticalExtensions_72_sequence);

  return offset;
}


static const per_sequence_t T_v770NonCriticalExtensions_06_sequence[] = {
  { &hf_rrc_physicalChannelReconfiguration_v770ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PhysicalChannelReconfiguration_v770ext_IEs },
  { &hf_rrc_nonCriticalExtensions_72, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_nonCriticalExtensions_72 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v770NonCriticalExtensions_06(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v770NonCriticalExtensions_06, T_v770NonCriticalExtensions_06_sequence);

  return offset;
}


static const per_sequence_t T_v690NonCriticalExtensions_14_sequence[] = {
  { &hf_rrc_physicalChannelReconfiguration_v690ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PhysicalChannelReconfiguration_v690ext_IEs },
  { &hf_rrc_v770NonCriticalExtensions_06, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v770NonCriticalExtensions_06 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v690NonCriticalExtensions_14(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v690NonCriticalExtensions_14, T_v690NonCriticalExtensions_14_sequence);

  return offset;
}


static const per_sequence_t T_r5_04_sequence[] = {
  { &hf_rrc_physicalChannelReconfiguration_r5, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PhysicalChannelReconfiguration_r5_IEs },
  { &hf_rrc_physicalChannelReconfiguration_r5_add_ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_BIT_STRING },
  { &hf_rrc_v690NonCriticalExtensions_14, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v690NonCriticalExtensions_14 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_r5_04(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_r5_04, T_r5_04_sequence);

  return offset;
}


static const per_sequence_t PhysicalChannelReconfiguration_r6_IEs_sequence[] = {
  { &hf_rrc_integrityProtectionModeInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_IntegrityProtectionModeInfo },
  { &hf_rrc_cipheringModeInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CipheringModeInfo },
  { &hf_rrc_activationTime  , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_ActivationTime },
  { &hf_rrc_delayRestrictionFlag, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DelayRestrictionFlag },
  { &hf_rrc_new_U_RNTI      , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_U_RNTI },
  { &hf_rrc_new_C_RNTI      , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_C_RNTI },
  { &hf_rrc_new_DSCH_RNTI   , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DSCH_RNTI },
  { &hf_rrc_new_H_RNTI      , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_H_RNTI },
  { &hf_rrc_newPrimary_E_RNTI, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_E_RNTI },
  { &hf_rrc_newSecondary_E_RNTI, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_E_RNTI },
  { &hf_rrc_rrc_StateIndicator, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RRC_StateIndicator },
  { &hf_rrc_utran_DRX_CycleLengthCoeff, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UTRAN_DRX_CycleLengthCoefficient },
  { &hf_rrc_cn_InformationInfo_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CN_InformationInfo_r6 },
  { &hf_rrc_ura_Identity    , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_URA_Identity },
  { &hf_rrc_dl_CounterSynchronisationInfo_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_CounterSynchronisationInfo_r5 },
  { &hf_rrc_frequencyInfo   , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_FrequencyInfo },
  { &hf_rrc_maxAllowedUL_TX_Power, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MaxAllowedUL_TX_Power },
  { &hf_rrc_ul_DPCH_Info    , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_DPCH_Info_r6 },
  { &hf_rrc_ul_EDCH_Information, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_EDCH_Information_r6 },
  { &hf_rrc_dl_HSPDSCH_Information_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_HSPDSCH_Information_r6 },
  { &hf_rrc_dl_CommonInformation_03, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_CommonInformation_r6 },
  { &hf_rrc_dl_InformationPerRL_List_03, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_InformationPerRL_List_r6 },
  { &hf_rrc_mbms_PL_ServiceRestrictInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MBMS_PL_ServiceRestrictInfo_r6 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_PhysicalChannelReconfiguration_r6_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_PhysicalChannelReconfiguration_r6_IEs, PhysicalChannelReconfiguration_r6_IEs_sequence);

  return offset;
}


static const per_sequence_t PhysicalChannelReconfiguration_v6b0ext_IEs_sequence[] = {
  { &hf_rrc_dl_InformationPerRL_List_v6b0ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_InformationPerRL_List_v6b0ext },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_PhysicalChannelReconfiguration_v6b0ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_PhysicalChannelReconfiguration_v6b0ext_IEs, PhysicalChannelReconfiguration_v6b0ext_IEs_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions_73_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_nonCriticalExtensions_73(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_nonCriticalExtensions_73, T_nonCriticalExtensions_73_sequence);

  return offset;
}


static const per_sequence_t T_v770NonCriticalExtensions_07_sequence[] = {
  { &hf_rrc_physicalChannelReconfiguration_v770ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PhysicalChannelReconfiguration_v770ext_IEs },
  { &hf_rrc_nonCriticalExtensions_73, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_nonCriticalExtensions_73 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v770NonCriticalExtensions_07(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v770NonCriticalExtensions_07, T_v770NonCriticalExtensions_07_sequence);

  return offset;
}


static const per_sequence_t T_v6b0NonCriticalExtensions_05_sequence[] = {
  { &hf_rrc_physicalChannelReconfiguration_v6b0ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PhysicalChannelReconfiguration_v6b0ext_IEs },
  { &hf_rrc_v770NonCriticalExtensions_07, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v770NonCriticalExtensions_07 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v6b0NonCriticalExtensions_05(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v6b0NonCriticalExtensions_05, T_v6b0NonCriticalExtensions_05_sequence);

  return offset;
}


static const per_sequence_t T_r6_06_sequence[] = {
  { &hf_rrc_physicalChannelReconfiguration_r6, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PhysicalChannelReconfiguration_r6_IEs },
  { &hf_rrc_physicalChannelReconfiguration_r6_add_ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_BIT_STRING },
  { &hf_rrc_v6b0NonCriticalExtensions_06, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v6b0NonCriticalExtensions_05 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_r6_06(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_r6_06, T_r6_06_sequence);

  return offset;
}


static const value_string rrc_T_responseToChangeOfUE_Capability_vals[] = {
  {   0, "true" },
  { 0, NULL }
};


static int
dissect_rrc_T_responseToChangeOfUE_Capability(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t PhysicalChannelReconfiguration_r7_IEs_sequence[] = {
  { &hf_rrc_integrityProtectionModeInfo_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_IntegrityProtectionModeInfo_r7 },
  { &hf_rrc_cipheringModeInfo_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CipheringModeInfo_r7 },
  { &hf_rrc_activationTime  , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_ActivationTime },
  { &hf_rrc_delayRestrictionFlag, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DelayRestrictionFlag },
  { &hf_rrc_new_U_RNTI      , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_U_RNTI },
  { &hf_rrc_new_C_RNTI      , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_C_RNTI },
  { &hf_rrc_new_DSCH_RNTI   , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DSCH_RNTI },
  { &hf_rrc_new_H_RNTI      , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_H_RNTI },
  { &hf_rrc_newPrimary_E_RNTI, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_E_RNTI },
  { &hf_rrc_newSecondary_E_RNTI, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_E_RNTI },
  { &hf_rrc_rrc_StateIndicator, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RRC_StateIndicator },
  { &hf_rrc_ueMobilityStateIndicator, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_High_MobilityDetected },
  { &hf_rrc_utran_DRX_CycleLengthCoeff_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UTRAN_DRX_CycleLengthCoefficient_r7 },
  { &hf_rrc_cn_InformationInfo_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CN_InformationInfo_r6 },
  { &hf_rrc_ura_Identity    , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_URA_Identity },
  { &hf_rrc_supportForChangeOfUE_Capability, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_BOOLEAN },
  { &hf_rrc_responseToChangeOfUE_Capability, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_responseToChangeOfUE_Capability },
  { &hf_rrc_dl_CounterSynchronisationInfo_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_CounterSynchronisationInfo_r5 },
  { &hf_rrc_frequencyInfo   , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_FrequencyInfo },
  { &hf_rrc_multi_frequencyInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_Multi_frequencyInfo_LCR_r7 },
  { &hf_rrc_dtx_drx_TimingInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DTX_DRX_TimingInfo_r7 },
  { &hf_rrc_dtx_drx_Info    , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DTX_DRX_Info_r7 },
  { &hf_rrc_hs_scch_LessInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_HS_SCCH_LessInfo_r7 },
  { &hf_rrc_mimoParameters  , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MIMO_Parameters_r7 },
  { &hf_rrc_maxAllowedUL_TX_Power, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MaxAllowedUL_TX_Power },
  { &hf_rrc_ul_DPCH_Info_01 , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_DPCH_Info_r7 },
  { &hf_rrc_ul_EDCH_Information_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_EDCH_Information_r7 },
  { &hf_rrc_dl_HSPDSCH_Information_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_HSPDSCH_Information_r7 },
  { &hf_rrc_dl_CommonInformation_04, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_CommonInformation_r7 },
  { &hf_rrc_dl_InformationPerRL_List_04, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_InformationPerRL_List_r7 },
  { &hf_rrc_mbms_PL_ServiceRestrictInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MBMS_PL_ServiceRestrictInfo_r6 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_PhysicalChannelReconfiguration_r7_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_PhysicalChannelReconfiguration_r7_IEs, PhysicalChannelReconfiguration_r7_IEs_sequence);

  return offset;
}


static const per_sequence_t PhysicalChannelReconfiguration_v780ext_IEs_sequence[] = {
  { &hf_rrc_harqInfo        , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_HARQ_Info_r7 },
  { &hf_rrc_hs_DSCH_TBSizeTable, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_HS_DSCH_TBSizeTable },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_PhysicalChannelReconfiguration_v780ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_PhysicalChannelReconfiguration_v780ext_IEs, PhysicalChannelReconfiguration_v780ext_IEs_sequence);

  return offset;
}


static const per_sequence_t PhysicalChannelReconfiguration_v7d0ext_IEs_sequence[] = {
  { &hf_rrc_ul_EDCH_Information_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_EDCH_Information_ext },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_PhysicalChannelReconfiguration_v7d0ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_PhysicalChannelReconfiguration_v7d0ext_IEs, PhysicalChannelReconfiguration_v7d0ext_IEs_sequence);

  return offset;
}


static const per_sequence_t PhysicalChannelReconfiguration_v7f0ext_IEs_sequence[] = {
  { &hf_rrc_mimoParameters_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MIMO_Parameters_v7f0ext },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_PhysicalChannelReconfiguration_v7f0ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_PhysicalChannelReconfiguration_v7f0ext_IEs, PhysicalChannelReconfiguration_v7f0ext_IEs_sequence);

  return offset;
}


static const per_sequence_t PhysicalChannelReconfiguration_v7g0ext_IEs_sequence[] = {
  { &hf_rrc_mimoParameters_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MIMO_Parameters_v7g0ext },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_PhysicalChannelReconfiguration_v7g0ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_PhysicalChannelReconfiguration_v7g0ext_IEs, PhysicalChannelReconfiguration_v7g0ext_IEs_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions_74_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_nonCriticalExtensions_74(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_nonCriticalExtensions_74, T_nonCriticalExtensions_74_sequence);

  return offset;
}


static const per_sequence_t T_v7g0NonCriticalExtensions_06_sequence[] = {
  { &hf_rrc_physicalChannelReconfiguration_v7g0ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PhysicalChannelReconfiguration_v7g0ext_IEs },
  { &hf_rrc_nonCriticalExtensions_74, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_nonCriticalExtensions_74 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v7g0NonCriticalExtensions_06(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v7g0NonCriticalExtensions_06, T_v7g0NonCriticalExtensions_06_sequence);

  return offset;
}


static const per_sequence_t T_v7f0NonCriticalExtensions_05_sequence[] = {
  { &hf_rrc_physicalChannelReconfiguration_v7f0ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PhysicalChannelReconfiguration_v7f0ext_IEs },
  { &hf_rrc_v7g0NonCriticalExtensions_06, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v7g0NonCriticalExtensions_06 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v7f0NonCriticalExtensions_05(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v7f0NonCriticalExtensions_05, T_v7f0NonCriticalExtensions_05_sequence);

  return offset;
}


static const per_sequence_t T_v7d0NonCriticalExtensions_04_sequence[] = {
  { &hf_rrc_physicalChannelReconfiguration_v7d0ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PhysicalChannelReconfiguration_v7d0ext_IEs },
  { &hf_rrc_v7f0NonCriticalExtensions_05, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v7f0NonCriticalExtensions_05 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v7d0NonCriticalExtensions_04(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v7d0NonCriticalExtensions_04, T_v7d0NonCriticalExtensions_04_sequence);

  return offset;
}


static const per_sequence_t T_v780NonCriticalExtensions_04_sequence[] = {
  { &hf_rrc_physicalChannelReconfiguration_v780ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PhysicalChannelReconfiguration_v780ext_IEs },
  { &hf_rrc_v7d0NonCriticalExtensions_04, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v7d0NonCriticalExtensions_04 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v780NonCriticalExtensions_04(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v780NonCriticalExtensions_04, T_v780NonCriticalExtensions_04_sequence);

  return offset;
}


static const per_sequence_t T_r7_05_sequence[] = {
  { &hf_rrc_physicalChannelReconfiguration_r7, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PhysicalChannelReconfiguration_r7_IEs },
  { &hf_rrc_physicalChannelReconfiguration_r7_add_ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_BIT_STRING },
  { &hf_rrc_v780NonCriticalExtensions_04, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v780NonCriticalExtensions_04 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_r7_05(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_r7_05, T_r7_05_sequence);

  return offset;
}


static const value_string rrc_T_responseToChangeOfUE_Capability_01_vals[] = {
  {   0, "true" },
  { 0, NULL }
};


static int
dissect_rrc_T_responseToChangeOfUE_Capability_01(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t PhysicalChannelReconfiguration_r8_IEs_sequence[] = {
  { &hf_rrc_integrityProtectionModeInfo_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_IntegrityProtectionModeInfo_r7 },
  { &hf_rrc_cipheringModeInfo_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CipheringModeInfo_r7 },
  { &hf_rrc_activationTime  , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_ActivationTime },
  { &hf_rrc_delayRestrictionFlag, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DelayRestrictionFlag },
  { &hf_rrc_new_U_RNTI      , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_U_RNTI },
  { &hf_rrc_new_C_RNTI      , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_C_RNTI },
  { &hf_rrc_new_DSCH_RNTI   , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DSCH_RNTI },
  { &hf_rrc_new_H_RNTI      , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_H_RNTI },
  { &hf_rrc_newPrimary_E_RNTI, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_E_RNTI },
  { &hf_rrc_newSecondary_E_RNTI, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_E_RNTI },
  { &hf_rrc_rrc_StateIndicator, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RRC_StateIndicator },
  { &hf_rrc_ueMobilityStateIndicator, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_High_MobilityDetected },
  { &hf_rrc_utran_DRX_CycleLengthCoeff_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UTRAN_DRX_CycleLengthCoefficient_r7 },
  { &hf_rrc_cn_InformationInfo_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CN_InformationInfo_r6 },
  { &hf_rrc_ura_Identity    , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_URA_Identity },
  { &hf_rrc_supportForChangeOfUE_Capability, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_BOOLEAN },
  { &hf_rrc_responseToChangeOfUE_Capability_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_responseToChangeOfUE_Capability_01 },
  { &hf_rrc_dl_CounterSynchronisationInfo_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_CounterSynchronisationInfo_r5 },
  { &hf_rrc_harqInfo        , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_HARQ_Info_r7 },
  { &hf_rrc_frequencyInfo   , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_FrequencyInfo },
  { &hf_rrc_multi_frequencyInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_Multi_frequencyInfo_LCR_r7 },
  { &hf_rrc_dtx_drx_TimingInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DTX_DRX_TimingInfo_r7 },
  { &hf_rrc_dtx_drx_Info    , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DTX_DRX_Info_r7 },
  { &hf_rrc_hs_scch_LessInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_HS_SCCH_LessInfo_r7 },
  { &hf_rrc_mimoParameters_03, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MIMO_Parameters_r8 },
  { &hf_rrc_maxAllowedUL_TX_Power, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MaxAllowedUL_TX_Power },
  { &hf_rrc_ul_DPCH_Info_01 , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_DPCH_Info_r7 },
  { &hf_rrc_ul_EDCH_Information_03, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_EDCH_Information_r8 },
  { &hf_rrc_dl_HSPDSCH_Information_03, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_HSPDSCH_Information_r8 },
  { &hf_rrc_dl_CommonInformation_05, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_CommonInformation_r8 },
  { &hf_rrc_dl_InformationPerRL_List_05, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_InformationPerRL_List_r8 },
  { &hf_rrc_dl_SecondaryCellInfoFDD, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_SecondaryCellInfoFDD },
  { &hf_rrc_controlChannelDRXInfo_TDD128, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_ControlChannelDRXInfo_TDD128_r8 },
  { &hf_rrc_sps_Information_TDD128, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_SPS_Information_TDD128_r8 },
  { &hf_rrc_mbms_PL_ServiceRestrictInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MBMS_PL_ServiceRestrictInfo_r6 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_PhysicalChannelReconfiguration_r8_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_PhysicalChannelReconfiguration_r8_IEs, PhysicalChannelReconfiguration_r8_IEs_sequence);

  return offset;
}


static const per_sequence_t PhysicalChannelReconfiguration_v890ext_IEs_sequence[] = {
  { &hf_rrc_dl_SecondaryCellInfoFDD_v890ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_SecondaryCellInfoFDD_v890ext },
  { &hf_rrc_dl_HSPDSCH_Information_04, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_HSPDSCH_Information_r8_ext },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_PhysicalChannelReconfiguration_v890ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_PhysicalChannelReconfiguration_v890ext_IEs, PhysicalChannelReconfiguration_v890ext_IEs_sequence);

  return offset;
}


static const per_sequence_t PhysicalChannelReconfiguration_v8a0ext_IEs_sequence[] = {
  { &hf_rrc_dl_HSPDSCH_Information_05, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_HSPDSCH_Information_r8_ext2 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_PhysicalChannelReconfiguration_v8a0ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_PhysicalChannelReconfiguration_v8a0ext_IEs, PhysicalChannelReconfiguration_v8a0ext_IEs_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions_75_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_nonCriticalExtensions_75(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_nonCriticalExtensions_75, T_nonCriticalExtensions_75_sequence);

  return offset;
}


static const per_sequence_t T_v8a0NonCriticalExtensions_04_sequence[] = {
  { &hf_rrc_physicalChannelReconfiguration_v8a0ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PhysicalChannelReconfiguration_v8a0ext_IEs },
  { &hf_rrc_nonCriticalExtensions_75, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_nonCriticalExtensions_75 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v8a0NonCriticalExtensions_04(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v8a0NonCriticalExtensions_04, T_v8a0NonCriticalExtensions_04_sequence);

  return offset;
}


static const per_sequence_t T_v7g0NonCriticalExtensions_07_sequence[] = {
  { &hf_rrc_physicalChannelReconfiguration_v7g0ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PhysicalChannelReconfiguration_v7g0ext_IEs },
  { &hf_rrc_v8a0NonCriticalExtensions_04, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v8a0NonCriticalExtensions_04 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v7g0NonCriticalExtensions_07(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v7g0NonCriticalExtensions_07, T_v7g0NonCriticalExtensions_07_sequence);

  return offset;
}


static const per_sequence_t T_v890NonCriticalExtensions_04_sequence[] = {
  { &hf_rrc_physicalChannelReconfiguration_v890ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PhysicalChannelReconfiguration_v890ext_IEs },
  { &hf_rrc_v7g0NonCriticalExtensions_07, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v7g0NonCriticalExtensions_07 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v890NonCriticalExtensions_04(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v890NonCriticalExtensions_04, T_v890NonCriticalExtensions_04_sequence);

  return offset;
}


static const per_sequence_t T_v7f0NonCriticalExtensions_06_sequence[] = {
  { &hf_rrc_physicalChannelReconfiguration_v7f0ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PhysicalChannelReconfiguration_v7f0ext_IEs },
  { &hf_rrc_v890NonCriticalExtensions_04, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v890NonCriticalExtensions_04 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v7f0NonCriticalExtensions_06(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v7f0NonCriticalExtensions_06, T_v7f0NonCriticalExtensions_06_sequence);

  return offset;
}


static const per_sequence_t T_v7d0NonCriticalExtensions_05_sequence[] = {
  { &hf_rrc_physicalChannelReconfiguration_v7d0ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PhysicalChannelReconfiguration_v7d0ext_IEs },
  { &hf_rrc_v7f0NonCriticalExtensions_06, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v7f0NonCriticalExtensions_06 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v7d0NonCriticalExtensions_05(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v7d0NonCriticalExtensions_05, T_v7d0NonCriticalExtensions_05_sequence);

  return offset;
}


static const per_sequence_t T_r8_06_sequence[] = {
  { &hf_rrc_physicalChannelReconfiguration_r8, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PhysicalChannelReconfiguration_r8_IEs },
  { &hf_rrc_physicalChannelReconfiguration_r8_add_ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_BIT_STRING },
  { &hf_rrc_v7d0NonCriticalExtensions_05, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v7d0NonCriticalExtensions_05 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_r8_06(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_r8_06, T_r8_06_sequence);

  return offset;
}


static const value_string rrc_T_responseToChangeOfUE_Capability_02_vals[] = {
  {   0, "true" },
  { 0, NULL }
};


static int
dissect_rrc_T_responseToChangeOfUE_Capability_02(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t PhysicalChannelReconfiguration_r9_IEs_sequence[] = {
  { &hf_rrc_integrityProtectionModeInfo_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_IntegrityProtectionModeInfo_r7 },
  { &hf_rrc_cipheringModeInfo_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CipheringModeInfo_r7 },
  { &hf_rrc_activationTime  , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_ActivationTime },
  { &hf_rrc_delayRestrictionFlag, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DelayRestrictionFlag },
  { &hf_rrc_new_U_RNTI      , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_U_RNTI },
  { &hf_rrc_new_C_RNTI      , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_C_RNTI },
  { &hf_rrc_new_DSCH_RNTI   , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DSCH_RNTI },
  { &hf_rrc_new_H_RNTI      , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_H_RNTI },
  { &hf_rrc_newPrimary_E_RNTI, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_E_RNTI },
  { &hf_rrc_newSecondary_E_RNTI, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_E_RNTI },
  { &hf_rrc_rrc_StateIndicator, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RRC_StateIndicator },
  { &hf_rrc_ueMobilityStateIndicator, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_High_MobilityDetected },
  { &hf_rrc_utran_DRX_CycleLengthCoeff_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UTRAN_DRX_CycleLengthCoefficient_r7 },
  { &hf_rrc_cn_InformationInfo_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CN_InformationInfo_r6 },
  { &hf_rrc_ura_Identity    , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_URA_Identity },
  { &hf_rrc_supportForChangeOfUE_Capability, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_BOOLEAN },
  { &hf_rrc_responseToChangeOfUE_Capability_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_responseToChangeOfUE_Capability_02 },
  { &hf_rrc_dl_CounterSynchronisationInfo_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_CounterSynchronisationInfo_r5 },
  { &hf_rrc_harqInfo        , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_HARQ_Info_r7 },
  { &hf_rrc_frequencyInfo   , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_FrequencyInfo },
  { &hf_rrc_multi_frequencyInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_Multi_frequencyInfo_LCR_r7 },
  { &hf_rrc_dtx_drx_TimingInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DTX_DRX_TimingInfo_r7 },
  { &hf_rrc_dtx_drx_Info    , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DTX_DRX_Info_r7 },
  { &hf_rrc_hs_scch_LessInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_HS_SCCH_LessInfo_r7 },
  { &hf_rrc_mimoParameters_04, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MIMO_Parameters_r9 },
  { &hf_rrc_controlChannelDRXInfo_TDD128, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_ControlChannelDRXInfo_TDD128_r8 },
  { &hf_rrc_sps_Information_TDD128, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_SPS_Information_TDD128_r8 },
  { &hf_rrc_maxAllowedUL_TX_Power, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MaxAllowedUL_TX_Power },
  { &hf_rrc_ul_DPCH_Info_01 , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_DPCH_Info_r7 },
  { &hf_rrc_ul_EDCH_Information_04, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_EDCH_Information_r9 },
  { &hf_rrc_ul_SecondaryCellInfoFDD, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_SecondaryCellInfoFDD },
  { &hf_rrc_dl_HSPDSCH_Information_06, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_HSPDSCH_Information_r9 },
  { &hf_rrc_dl_CommonInformation_05, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_CommonInformation_r8 },
  { &hf_rrc_dl_InformationPerRL_List_05, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_InformationPerRL_List_r8 },
  { &hf_rrc_dl_SecondaryCellInfoFDD_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_SecondaryCellInfoFDD_r9 },
  { &hf_rrc_mbms_PL_ServiceRestrictInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MBMS_PL_ServiceRestrictInfo_r6 },
  { &hf_rrc_cellDCHMeasOccasionInfo_TDD128, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CellDCHMeasOccasionInfo_TDD128_r9 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_PhysicalChannelReconfiguration_r9_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_PhysicalChannelReconfiguration_r9_IEs, PhysicalChannelReconfiguration_r9_IEs_sequence);

  return offset;
}


static const per_sequence_t PhysicalChannelReconfiguration_v950ext_IEs_sequence[] = {
  { &hf_rrc_secondaryCellMimoParameters, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_SecondaryCellMIMOparametersFDD_v950ext },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_PhysicalChannelReconfiguration_v950ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_PhysicalChannelReconfiguration_v950ext_IEs, PhysicalChannelReconfiguration_v950ext_IEs_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions_76_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_nonCriticalExtensions_76(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_nonCriticalExtensions_76, T_nonCriticalExtensions_76_sequence);

  return offset;
}


static const per_sequence_t T_v950NonCriticalExtensions_03_sequence[] = {
  { &hf_rrc_physicalChannelReconfiguration_v950ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PhysicalChannelReconfiguration_v950ext_IEs },
  { &hf_rrc_nonCriticalExtensions_76, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_nonCriticalExtensions_76 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v950NonCriticalExtensions_03(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v950NonCriticalExtensions_03, T_v950NonCriticalExtensions_03_sequence);

  return offset;
}


static const per_sequence_t T_r9_05_sequence[] = {
  { &hf_rrc_physicalChannelReconfiguration_r9, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PhysicalChannelReconfiguration_r9_IEs },
  { &hf_rrc_physicalChannelReconfiguration_r9_add_ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_BIT_STRING },
  { &hf_rrc_v950NonCriticalExtensions_03, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v950NonCriticalExtensions_03 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_r9_05(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_r9_05, T_r9_05_sequence);

  return offset;
}


static const value_string rrc_T_responseToChangeOfUE_Capability_03_vals[] = {
  {   0, "true" },
  { 0, NULL }
};


static int
dissect_rrc_T_responseToChangeOfUE_Capability_03(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t PhysicalChannelReconfiguration_r10_IEs_sequence[] = {
  { &hf_rrc_integrityProtectionModeInfo_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_IntegrityProtectionModeInfo_r7 },
  { &hf_rrc_cipheringModeInfo_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CipheringModeInfo_r7 },
  { &hf_rrc_activationTime  , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_ActivationTime },
  { &hf_rrc_delayRestrictionFlag, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DelayRestrictionFlag },
  { &hf_rrc_new_U_RNTI      , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_U_RNTI },
  { &hf_rrc_new_C_RNTI      , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_C_RNTI },
  { &hf_rrc_new_DSCH_RNTI   , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DSCH_RNTI },
  { &hf_rrc_new_H_RNTI      , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_H_RNTI },
  { &hf_rrc_newPrimary_E_RNTI, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_E_RNTI },
  { &hf_rrc_newSecondary_E_RNTI, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_E_RNTI },
  { &hf_rrc_rrc_StateIndicator, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RRC_StateIndicator },
  { &hf_rrc_ueMobilityStateIndicator, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_High_MobilityDetected },
  { &hf_rrc_utran_DRX_CycleLengthCoeff_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UTRAN_DRX_CycleLengthCoefficient_r7 },
  { &hf_rrc_cn_InformationInfo_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CN_InformationInfo_r6 },
  { &hf_rrc_ura_Identity    , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_URA_Identity },
  { &hf_rrc_supportForChangeOfUE_Capability, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_BOOLEAN },
  { &hf_rrc_responseToChangeOfUE_Capability_03, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_responseToChangeOfUE_Capability_03 },
  { &hf_rrc_dl_CounterSynchronisationInfo_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_CounterSynchronisationInfo_r5 },
  { &hf_rrc_harqInfo        , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_HARQ_Info_r7 },
  { &hf_rrc_frequencyInfo   , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_FrequencyInfo },
  { &hf_rrc_multi_frequencyInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_Multi_frequencyInfo_LCR_r7 },
  { &hf_rrc_dtx_drx_TimingInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DTX_DRX_TimingInfo_r7 },
  { &hf_rrc_dtx_drx_Info    , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DTX_DRX_Info_r7 },
  { &hf_rrc_hs_scch_LessInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_HS_SCCH_LessInfo_r7 },
  { &hf_rrc_mimoParameters_04, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MIMO_Parameters_r9 },
  { &hf_rrc_controlChannelDRXInfo_TDD128, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_ControlChannelDRXInfo_TDD128_r8 },
  { &hf_rrc_sps_Information_TDD128, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_SPS_Information_TDD128_r8 },
  { &hf_rrc_maxAllowedUL_TX_Power, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MaxAllowedUL_TX_Power },
  { &hf_rrc_ul_DPCH_Info_01 , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_DPCH_Info_r7 },
  { &hf_rrc_ul_EDCH_Information_04, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_EDCH_Information_r9 },
  { &hf_rrc_ul_SecondaryCellInfoFDD, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_SecondaryCellInfoFDD },
  { &hf_rrc_ul_MulticarrierEDCHInfo_TDD128, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_MulticarrierEDCHInfo_TDD128 },
  { &hf_rrc_dl_HSPDSCH_Information_06, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_HSPDSCH_Information_r9 },
  { &hf_rrc_dl_CommonInformation_06, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_CommonInformation_r10 },
  { &hf_rrc_dl_InformationPerRL_List_05, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_InformationPerRL_List_r8 },
  { &hf_rrc_dl_SecondaryCellInfoFDD_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_SecondaryCellInfoFDD_r10 },
  { &hf_rrc_additionalDLSecCellInfoListFDD, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_AdditionalDLSecCellInfoListFDD },
  { &hf_rrc_mu_MIMO_Info_TDD128, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MU_MIMO_Info_TDD128 },
  { &hf_rrc_mbms_PL_ServiceRestrictInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MBMS_PL_ServiceRestrictInfo_r6 },
  { &hf_rrc_cellDCHMeasOccasionInfo_TDD128, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CellDCHMeasOccasionInfo_TDD128_r9 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_PhysicalChannelReconfiguration_r10_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_PhysicalChannelReconfiguration_r10_IEs, PhysicalChannelReconfiguration_r10_IEs_sequence);

  return offset;
}


static const per_sequence_t PhysicalChannelReconfiguration_vbxyext_IEs_sequence[] = {
  { &hf_rrc_ulOLTDInfoFDD   , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_OLTD_InfoFDD },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_PhysicalChannelReconfiguration_vbxyext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_PhysicalChannelReconfiguration_vbxyext_IEs, PhysicalChannelReconfiguration_vbxyext_IEs_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions_77_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_nonCriticalExtensions_77(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_nonCriticalExtensions_77, T_nonCriticalExtensions_77_sequence);

  return offset;
}


static const per_sequence_t T_vbxyNonCriticalExtensions_08_sequence[] = {
  { &hf_rrc_physicalChannelReconfiguration_vbxyext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PhysicalChannelReconfiguration_vbxyext_IEs },
  { &hf_rrc_nonCriticalExtensions_77, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_nonCriticalExtensions_77 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_vbxyNonCriticalExtensions_08(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_vbxyNonCriticalExtensions_08, T_vbxyNonCriticalExtensions_08_sequence);

  return offset;
}


static const per_sequence_t T_r10_05_sequence[] = {
  { &hf_rrc_physicalChannelReconfiguration_r10, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PhysicalChannelReconfiguration_r10_IEs },
  { &hf_rrc_physicalChannelReconfiguration_r10_add_ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_BIT_STRING },
  { &hf_rrc_vbxyNonCriticalExtensions_08, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_vbxyNonCriticalExtensions_08 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_r10_05(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_r10_05, T_r10_05_sequence);

  return offset;
}


static const value_string rrc_T_responseToChangeOfUE_Capability_04_vals[] = {
  {   0, "true" },
  { 0, NULL }
};


static int
dissect_rrc_T_responseToChangeOfUE_Capability_04(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t PhysicalChannelReconfiguration_r11_IEs_sequence[] = {
  { &hf_rrc_integrityProtectionModeInfo_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_IntegrityProtectionModeInfo_r7 },
  { &hf_rrc_cipheringModeInfo_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CipheringModeInfo_r7 },
  { &hf_rrc_activationTime  , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_ActivationTime },
  { &hf_rrc_delayRestrictionFlag, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DelayRestrictionFlag },
  { &hf_rrc_new_U_RNTI      , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_U_RNTI },
  { &hf_rrc_new_C_RNTI      , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_C_RNTI },
  { &hf_rrc_new_DSCH_RNTI   , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DSCH_RNTI },
  { &hf_rrc_new_H_RNTI      , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_H_RNTI },
  { &hf_rrc_newPrimary_E_RNTI, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_E_RNTI },
  { &hf_rrc_newSecondary_E_RNTI, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_E_RNTI },
  { &hf_rrc_rrc_StateIndicator, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RRC_StateIndicator },
  { &hf_rrc_ueMobilityStateIndicator, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_High_MobilityDetected },
  { &hf_rrc_utran_DRX_CycleLengthCoeff_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UTRAN_DRX_CycleLengthCoefficient_r7 },
  { &hf_rrc_cn_InformationInfo_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CN_InformationInfo_r6 },
  { &hf_rrc_ura_Identity    , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_URA_Identity },
  { &hf_rrc_supportForChangeOfUE_Capability, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_BOOLEAN },
  { &hf_rrc_responseToChangeOfUE_Capability_04, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_responseToChangeOfUE_Capability_04 },
  { &hf_rrc_dl_CounterSynchronisationInfo_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_CounterSynchronisationInfo_r5 },
  { &hf_rrc_harqInfo_01     , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_HARQ_Info_r11 },
  { &hf_rrc_frequencyInfo   , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_FrequencyInfo },
  { &hf_rrc_multi_frequencyInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_Multi_frequencyInfo_LCR_r7 },
  { &hf_rrc_dtx_drx_TimingInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DTX_DRX_TimingInfo_r7 },
  { &hf_rrc_dtx_drx_Info    , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DTX_DRX_Info_r7 },
  { &hf_rrc_hs_scch_LessInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_HS_SCCH_LessInfo_r7 },
  { &hf_rrc_mimoParameters_04, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MIMO_Parameters_r9 },
  { &hf_rrc_mimo4x4Parameters, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MIMO4x4_Parameters },
  { &hf_rrc_controlChannelDRXInfo_TDD128, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_ControlChannelDRXInfo_TDD128_r8 },
  { &hf_rrc_sps_Information_TDD128, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_SPS_Information_TDD128_r8 },
  { &hf_rrc_maxAllowedUL_TX_Power, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MaxAllowedUL_TX_Power },
  { &hf_rrc_ul_DPCH_Info_02 , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_DPCH_Info_r11 },
  { &hf_rrc_ul_EDCH_Information_05, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_EDCH_Information_r11 },
  { &hf_rrc_ul_SecondaryCellInfoFDD, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_SecondaryCellInfoFDD },
  { &hf_rrc_ul_MulticarrierEDCHInfo_TDD128, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_MulticarrierEDCHInfo_TDD128 },
  { &hf_rrc_ul_CLTD_InfoFDD , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_CLTD_InfoFDD },
  { &hf_rrc_ul_OLTD_InfoFDD , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_OLTD_InfoFDD },
  { &hf_rrc_dl_HSPDSCH_Information_07, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_HSPDSCH_Information_r11 },
  { &hf_rrc_dl_CommonInformation_07, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_CommonInformation_r11 },
  { &hf_rrc_dl_InformationPerRL_List_06, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_InformationPerRL_List_r11 },
  { &hf_rrc_dl_SecondaryCellInfoFDD_03, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_SecondaryCellInfoFDD_r11 },
  { &hf_rrc_additionalDLSecCellInfoListFDD_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_AdditionalDLSecCellInfoListFDD_r11 },
  { &hf_rrc_additionalDLSecCellInfoListFDD2, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_AdditionalDLSecCellInfoListFDD2_r11 },
  { &hf_rrc_commonERGCHInfoFDD, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CommonERGCHInfoFDD },
  { &hf_rrc_mu_MIMO_Info_TDD128, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MU_MIMO_Info_TDD128 },
  { &hf_rrc_mbms_PL_ServiceRestrictInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MBMS_PL_ServiceRestrictInfo_r6 },
  { &hf_rrc_cellDCHMeasOccasionInfo_TDD128, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CellDCHMeasOccasionInfo_TDD128_r9 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_PhysicalChannelReconfiguration_r11_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_PhysicalChannelReconfiguration_r11_IEs, PhysicalChannelReconfiguration_r11_IEs_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions_78_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_nonCriticalExtensions_78(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_nonCriticalExtensions_78, T_nonCriticalExtensions_78_sequence);

  return offset;
}


static const per_sequence_t T_r11_06_sequence[] = {
  { &hf_rrc_physicalChannelReconfiguration_r11, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PhysicalChannelReconfiguration_r11_IEs },
  { &hf_rrc_physicalChannelReconfiguration_r11_add_ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_BIT_STRING },
  { &hf_rrc_nonCriticalExtensions_78, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_nonCriticalExtensions_78 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_r11_06(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_r11_06, T_r11_06_sequence);

  return offset;
}


static const per_sequence_t T_criticalExtensions_61_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_criticalExtensions_61(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_criticalExtensions_61, T_criticalExtensions_61_sequence);

  return offset;
}


static const value_string rrc_T_criticalExtensions_60_vals[] = {
  {   0, "r11" },
  {   1, "criticalExtensions" },
  { 0, NULL }
};

static const per_choice_t T_criticalExtensions_60_choice[] = {
  {   0, &hf_rrc_r11_06          , ASN1_NO_EXTENSIONS     , dissect_rrc_T_r11_06 },
  {   1, &hf_rrc_criticalExtensions_62, ASN1_NO_EXTENSIONS     , dissect_rrc_T_criticalExtensions_61 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_criticalExtensions_60(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_criticalExtensions_60, T_criticalExtensions_60_choice,
                                 NULL);

  return offset;
}


static const value_string rrc_T_criticalExtensions_59_vals[] = {
  {   0, "r10" },
  {   1, "criticalExtensions" },
  { 0, NULL }
};

static const per_choice_t T_criticalExtensions_59_choice[] = {
  {   0, &hf_rrc_r10_05          , ASN1_NO_EXTENSIONS     , dissect_rrc_T_r10_05 },
  {   1, &hf_rrc_criticalExtensions_61, ASN1_NO_EXTENSIONS     , dissect_rrc_T_criticalExtensions_60 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_criticalExtensions_59(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_criticalExtensions_59, T_criticalExtensions_59_choice,
                                 NULL);

  return offset;
}


static const value_string rrc_T_criticalExtensions_58_vals[] = {
  {   0, "r9" },
  {   1, "criticalExtensions" },
  { 0, NULL }
};

static const per_choice_t T_criticalExtensions_58_choice[] = {
  {   0, &hf_rrc_r9_05           , ASN1_NO_EXTENSIONS     , dissect_rrc_T_r9_05 },
  {   1, &hf_rrc_criticalExtensions_60, ASN1_NO_EXTENSIONS     , dissect_rrc_T_criticalExtensions_59 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_criticalExtensions_58(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_criticalExtensions_58, T_criticalExtensions_58_choice,
                                 NULL);

  return offset;
}


static const value_string rrc_T_criticalExtensions_57_vals[] = {
  {   0, "r8" },
  {   1, "criticalExtensions" },
  { 0, NULL }
};

static const per_choice_t T_criticalExtensions_57_choice[] = {
  {   0, &hf_rrc_r8_06           , ASN1_NO_EXTENSIONS     , dissect_rrc_T_r8_06 },
  {   1, &hf_rrc_criticalExtensions_59, ASN1_NO_EXTENSIONS     , dissect_rrc_T_criticalExtensions_58 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_criticalExtensions_57(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_criticalExtensions_57, T_criticalExtensions_57_choice,
                                 NULL);

  return offset;
}


static const value_string rrc_T_criticalExtensions_56_vals[] = {
  {   0, "r7" },
  {   1, "criticalExtensions" },
  { 0, NULL }
};

static const per_choice_t T_criticalExtensions_56_choice[] = {
  {   0, &hf_rrc_r7_05           , ASN1_NO_EXTENSIONS     , dissect_rrc_T_r7_05 },
  {   1, &hf_rrc_criticalExtensions_58, ASN1_NO_EXTENSIONS     , dissect_rrc_T_criticalExtensions_57 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_criticalExtensions_56(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_criticalExtensions_56, T_criticalExtensions_56_choice,
                                 NULL);

  return offset;
}


static const value_string rrc_T_criticalExtensions_55_vals[] = {
  {   0, "r6" },
  {   1, "criticalExtensions" },
  { 0, NULL }
};

static const per_choice_t T_criticalExtensions_55_choice[] = {
  {   0, &hf_rrc_r6_06           , ASN1_NO_EXTENSIONS     , dissect_rrc_T_r6_06 },
  {   1, &hf_rrc_criticalExtensions_57, ASN1_NO_EXTENSIONS     , dissect_rrc_T_criticalExtensions_56 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_criticalExtensions_55(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_criticalExtensions_55, T_criticalExtensions_55_choice,
                                 NULL);

  return offset;
}


static const value_string rrc_T_criticalExtensions_54_vals[] = {
  {   0, "r5" },
  {   1, "criticalExtensions" },
  { 0, NULL }
};

static const per_choice_t T_criticalExtensions_54_choice[] = {
  {   0, &hf_rrc_r5_04           , ASN1_NO_EXTENSIONS     , dissect_rrc_T_r5_04 },
  {   1, &hf_rrc_criticalExtensions_56, ASN1_NO_EXTENSIONS     , dissect_rrc_T_criticalExtensions_55 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_criticalExtensions_54(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_criticalExtensions_54, T_criticalExtensions_54_choice,
                                 NULL);

  return offset;
}


static const value_string rrc_T_criticalExtensions_53_vals[] = {
  {   0, "r4" },
  {   1, "criticalExtensions" },
  { 0, NULL }
};

static const per_choice_t T_criticalExtensions_53_choice[] = {
  {   0, &hf_rrc_r4_04           , ASN1_NO_EXTENSIONS     , dissect_rrc_T_r4_04 },
  {   1, &hf_rrc_criticalExtensions_55, ASN1_NO_EXTENSIONS     , dissect_rrc_T_criticalExtensions_54 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_criticalExtensions_53(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_criticalExtensions_53, T_criticalExtensions_53_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t T_later_than_r3_10_sequence[] = {
  { &hf_rrc_rrc_TransactionIdentifier, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RRC_TransactionIdentifier },
  { &hf_rrc_criticalExtensions_54, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_criticalExtensions_53 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_later_than_r3_10(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_later_than_r3_10, T_later_than_r3_10_sequence);

  return offset;
}


static const value_string rrc_PhysicalChannelReconfiguration_vals[] = {
  {   0, "r3" },
  {   1, "later-than-r3" },
  { 0, NULL }
};

static const per_choice_t PhysicalChannelReconfiguration_choice[] = {
  {   0, &hf_rrc_r3_12           , ASN1_NO_EXTENSIONS     , dissect_rrc_T_r3_12 },
  {   1, &hf_rrc_later_than_r3_10, ASN1_NO_EXTENSIONS     , dissect_rrc_T_later_than_r3_10 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_PhysicalChannelReconfiguration(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
#line 305 "../../asn1/rrc/rrc.cnf"
  col_append_str(actx->pinfo->cinfo, COL_INFO, "PhysicalChannelReconfiguration");
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_PhysicalChannelReconfiguration, PhysicalChannelReconfiguration_choice,
                                 NULL);




  return offset;
}



static int
dissect_rrc_INTEGER_1_256(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            1U, 256U, NULL, FALSE);

  return offset;
}


static const per_sequence_t AllocationPeriodInfo_sequence[] = {
  { &hf_rrc_allocationActivationTime, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_0_255 },
  { &hf_rrc_allocationDuration, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_1_256 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_AllocationPeriodInfo(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_AllocationPeriodInfo, AllocationPeriodInfo_sequence);

  return offset;
}



static int
dissect_rrc_PUSCH_Identity(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            1U, hiPUSCHidentities, NULL, FALSE);

  return offset;
}


static const per_sequence_t T_old_Configuration_03_sequence[] = {
  { &hf_rrc_tfcs_ID_01      , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_TFCS_IdentityPlain },
  { &hf_rrc_pusch_Identity  , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PUSCH_Identity },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_old_Configuration_03(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_old_Configuration_03, T_old_Configuration_03_sequence);

  return offset;
}


static const per_sequence_t PUSCH_Info_sequence[] = {
  { &hf_rrc_tfcs_ID_01      , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_TFCS_IdentityPlain },
  { &hf_rrc_commonTimeslotInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CommonTimeslotInfo },
  { &hf_rrc_pusch_TimeslotsCodes, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UplinkTimeslotsCodes },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_PUSCH_Info(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_PUSCH_Info, PUSCH_Info_sequence);

  return offset;
}


static const per_sequence_t T_new_Configuration_03_sequence[] = {
  { &hf_rrc_pusch_Info      , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PUSCH_Info },
  { &hf_rrc_pusch_Identity  , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_PUSCH_Identity },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_new_Configuration_03(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_new_Configuration_03, T_new_Configuration_03_sequence);

  return offset;
}


static const value_string rrc_T_configuration_03_vals[] = {
  {   0, "old-Configuration" },
  {   1, "new-Configuration" },
  { 0, NULL }
};

static const per_choice_t T_configuration_03_choice[] = {
  {   0, &hf_rrc_old_Configuration_03, ASN1_NO_EXTENSIONS     , dissect_rrc_T_old_Configuration_03 },
  {   1, &hf_rrc_new_Configuration_03, ASN1_NO_EXTENSIONS     , dissect_rrc_T_new_Configuration_03 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_configuration_03(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_configuration_03, T_configuration_03_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t T_pusch_AllocationAssignment_sequence[] = {
  { &hf_rrc_pusch_AllocationPeriodInfo, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_AllocationPeriodInfo },
  { &hf_rrc_pusch_PowerControlInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_TargetSIR },
  { &hf_rrc_configuration_03, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_configuration_03 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_pusch_AllocationAssignment(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_pusch_AllocationAssignment, T_pusch_AllocationAssignment_sequence);

  return offset;
}


static const value_string rrc_T_pusch_Allocation_vals[] = {
  {   0, "pusch-AllocationPending" },
  {   1, "pusch-AllocationAssignment" },
  { 0, NULL }
};

static const per_choice_t T_pusch_Allocation_choice[] = {
  {   0, &hf_rrc_pusch_AllocationPending, ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   1, &hf_rrc_pusch_AllocationAssignment, ASN1_NO_EXTENSIONS     , dissect_rrc_T_pusch_AllocationAssignment },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_pusch_Allocation(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_pusch_Allocation, T_pusch_Allocation_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t PUSCH_CapacityAllocationInfo_sequence[] = {
  { &hf_rrc_pusch_Allocation, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_pusch_Allocation },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_PUSCH_CapacityAllocationInfo(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_PUSCH_CapacityAllocationInfo, PUSCH_CapacityAllocationInfo_sequence);

  return offset;
}


static const per_sequence_t PDSCH_PowerControlInfo_sequence[] = {
  { &hf_rrc_tpc_StepSizeTDD , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_TPC_StepSizeTDD },
  { &hf_rrc_ul_CCTrChTPCList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_CCTrChTPCList },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_PDSCH_PowerControlInfo(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_PDSCH_PowerControlInfo, PDSCH_PowerControlInfo_sequence);

  return offset;
}



static int
dissect_rrc_PDSCH_Identity(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            1U, hiPDSCHidentities, NULL, FALSE);

  return offset;
}


static const per_sequence_t T_old_Configuration_sequence[] = {
  { &hf_rrc_tfcs_ID_01      , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_TFCS_IdentityPlain },
  { &hf_rrc_pdsch_Identity  , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PDSCH_Identity },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_old_Configuration(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_old_Configuration, T_old_Configuration_sequence);

  return offset;
}


static const per_sequence_t PDSCH_Info_sequence[] = {
  { &hf_rrc_tfcs_ID_01      , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_TFCS_IdentityPlain },
  { &hf_rrc_commonTimeslotInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CommonTimeslotInfo },
  { &hf_rrc_pdsch_TimeslotsCodes, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DownlinkTimeslotsCodes },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_PDSCH_Info(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_PDSCH_Info, PDSCH_Info_sequence);

  return offset;
}


static const per_sequence_t T_new_Configuration_sequence[] = {
  { &hf_rrc_pdsch_Info      , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PDSCH_Info },
  { &hf_rrc_pdsch_Identity  , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_PDSCH_Identity },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_new_Configuration(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_new_Configuration, T_new_Configuration_sequence);

  return offset;
}


static const value_string rrc_T_configuration_vals[] = {
  {   0, "old-Configuration" },
  {   1, "new-Configuration" },
  { 0, NULL }
};

static const per_choice_t T_configuration_choice[] = {
  {   0, &hf_rrc_old_Configuration, ASN1_NO_EXTENSIONS     , dissect_rrc_T_old_Configuration },
  {   1, &hf_rrc_new_Configuration, ASN1_NO_EXTENSIONS     , dissect_rrc_T_new_Configuration },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_configuration(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_configuration, T_configuration_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t PDSCH_CapacityAllocationInfo_sequence[] = {
  { &hf_rrc_pdsch_PowerControlInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_PDSCH_PowerControlInfo },
  { &hf_rrc_pdsch_AllocationPeriodInfo, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_AllocationPeriodInfo },
  { &hf_rrc_configuration   , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_configuration },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_PDSCH_CapacityAllocationInfo(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_PDSCH_CapacityAllocationInfo, PDSCH_CapacityAllocationInfo_sequence);

  return offset;
}


static const value_string rrc_T_confirmRequest_vals[] = {
  {   0, "confirmPDSCH" },
  {   1, "confirmPUSCH" },
  { 0, NULL }
};


static int
dissect_rrc_T_confirmRequest(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     2, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t TimeslotList_sequence_of[1] = {
  { &hf_rrc_TimeslotList_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TimeslotNumber },
};

static int
dissect_rrc_TimeslotList(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_TimeslotList, TimeslotList_sequence_of,
                                                  1, maxTS, FALSE);

  return offset;
}


static const per_sequence_t PhysicalSharedChannelAllocation_r3_IEs_sequence[] = {
  { &hf_rrc_dsch_RNTI       , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DSCH_RNTI },
  { &hf_rrc_rrc_TransactionIdentifier, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RRC_TransactionIdentifier },
  { &hf_rrc_ul_TimingAdvance_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_TimingAdvanceControl },
  { &hf_rrc_pusch_CapacityAllocationInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_PUSCH_CapacityAllocationInfo },
  { &hf_rrc_pdsch_CapacityAllocationInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_PDSCH_CapacityAllocationInfo },
  { &hf_rrc_confirmRequest  , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_confirmRequest },
  { &hf_rrc_trafficVolumeReportRequest, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_INTEGER_0_255 },
  { &hf_rrc_iscpTimeslotList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_TimeslotList },
  { &hf_rrc_requestPCCPCHRSCP, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_PhysicalSharedChannelAllocation_r3_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_PhysicalSharedChannelAllocation_r3_IEs, PhysicalSharedChannelAllocation_r3_IEs_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions_81_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_nonCriticalExtensions_81(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_nonCriticalExtensions_81, T_nonCriticalExtensions_81_sequence);

  return offset;
}


static const per_sequence_t T_laterNonCriticalExtensions_26_sequence[] = {
  { &hf_rrc_physicalSharedChannelAllocation_r3_add_ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_BIT_STRING },
  { &hf_rrc_nonCriticalExtensions_81, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_nonCriticalExtensions_81 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_laterNonCriticalExtensions_26(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_laterNonCriticalExtensions_26, T_laterNonCriticalExtensions_26_sequence);

  return offset;
}


static const per_sequence_t T_r3_13_sequence[] = {
  { &hf_rrc_physicalSharedChannelAllocation_r3, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PhysicalSharedChannelAllocation_r3_IEs },
  { &hf_rrc_laterNonCriticalExtensions_26, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_laterNonCriticalExtensions_26 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_r3_13(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_r3_13, T_r3_13_sequence);

  return offset;
}


static const per_sequence_t T_tdd128_48_sequence[] = {
  { &hf_rrc_tpc_StepSize    , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_TPC_StepSizeTDD },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_tdd128_48(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_tdd128_48, T_tdd128_48_sequence);

  return offset;
}


static const value_string rrc_T_tddOption_16_vals[] = {
  {   0, "tdd384" },
  {   1, "tdd128" },
  { 0, NULL }
};

static const per_choice_t T_tddOption_16_choice[] = {
  {   0, &hf_rrc_tdd384_11       , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   1, &hf_rrc_tdd128_49       , ASN1_NO_EXTENSIONS     , dissect_rrc_T_tdd128_48 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_tddOption_16(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_tddOption_16, T_tddOption_16_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t PUSCH_PowerControlInfo_r4_sequence[] = {
  { &hf_rrc_ul_TargetSIR    , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UL_TargetSIR },
  { &hf_rrc_tddOption_16    , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_tddOption_16 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_PUSCH_PowerControlInfo_r4(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_PUSCH_PowerControlInfo_r4, PUSCH_PowerControlInfo_r4_sequence);

  return offset;
}


static const per_sequence_t T_old_Configuration_04_sequence[] = {
  { &hf_rrc_tfcs_ID_01      , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_TFCS_IdentityPlain },
  { &hf_rrc_pusch_Identity  , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PUSCH_Identity },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_old_Configuration_04(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_old_Configuration_04, T_old_Configuration_04_sequence);

  return offset;
}


static const per_sequence_t T_tdd384_29_sequence[] = {
  { &hf_rrc_pusch_TimeslotsCodes, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UplinkTimeslotsCodes },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_tdd384_29(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_tdd384_29, T_tdd384_29_sequence);

  return offset;
}


static const per_sequence_t T_tdd128_47_sequence[] = {
  { &hf_rrc_pusch_TimeslotsCodes_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UplinkTimeslotsCodes_LCR_r4 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_tdd128_47(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_tdd128_47, T_tdd128_47_sequence);

  return offset;
}


static const value_string rrc_T_tddOption_15_vals[] = {
  {   0, "tdd384" },
  {   1, "tdd128" },
  { 0, NULL }
};

static const per_choice_t T_tddOption_15_choice[] = {
  {   0, &hf_rrc_tdd384_32       , ASN1_NO_EXTENSIONS     , dissect_rrc_T_tdd384_29 },
  {   1, &hf_rrc_tdd128_48       , ASN1_NO_EXTENSIONS     , dissect_rrc_T_tdd128_47 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_tddOption_15(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_tddOption_15, T_tddOption_15_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t PUSCH_Info_r4_sequence[] = {
  { &hf_rrc_tfcs_ID_01      , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_TFCS_IdentityPlain },
  { &hf_rrc_commonTimeslotInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CommonTimeslotInfo },
  { &hf_rrc_tddOption_15    , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_tddOption_15 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_PUSCH_Info_r4(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_PUSCH_Info_r4, PUSCH_Info_r4_sequence);

  return offset;
}


static const per_sequence_t T_new_Configuration_04_sequence[] = {
  { &hf_rrc_pusch_Info_01   , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PUSCH_Info_r4 },
  { &hf_rrc_pusch_Identity  , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_PUSCH_Identity },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_new_Configuration_04(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_new_Configuration_04, T_new_Configuration_04_sequence);

  return offset;
}


static const value_string rrc_T_configuration_04_vals[] = {
  {   0, "old-Configuration" },
  {   1, "new-Configuration" },
  { 0, NULL }
};

static const per_choice_t T_configuration_04_choice[] = {
  {   0, &hf_rrc_old_Configuration_04, ASN1_NO_EXTENSIONS     , dissect_rrc_T_old_Configuration_04 },
  {   1, &hf_rrc_new_Configuration_04, ASN1_NO_EXTENSIONS     , dissect_rrc_T_new_Configuration_04 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_configuration_04(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_configuration_04, T_configuration_04_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t T_pusch_AllocationAssignment_01_sequence[] = {
  { &hf_rrc_pusch_AllocationPeriodInfo, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_AllocationPeriodInfo },
  { &hf_rrc_pusch_PowerControlInfo_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_PUSCH_PowerControlInfo_r4 },
  { &hf_rrc_configuration_04, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_configuration_04 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_pusch_AllocationAssignment_01(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_pusch_AllocationAssignment_01, T_pusch_AllocationAssignment_01_sequence);

  return offset;
}


static const value_string rrc_T_pusch_Allocation_01_vals[] = {
  {   0, "pusch-AllocationPending" },
  {   1, "pusch-AllocationAssignment" },
  { 0, NULL }
};

static const per_choice_t T_pusch_Allocation_01_choice[] = {
  {   0, &hf_rrc_pusch_AllocationPending, ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   1, &hf_rrc_pusch_AllocationAssignment_01, ASN1_NO_EXTENSIONS     , dissect_rrc_T_pusch_AllocationAssignment_01 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_pusch_Allocation_01(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_pusch_Allocation_01, T_pusch_Allocation_01_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t PUSCH_CapacityAllocationInfo_r4_sequence[] = {
  { &hf_rrc_pusch_Allocation_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_pusch_Allocation_01 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_PUSCH_CapacityAllocationInfo_r4(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_PUSCH_CapacityAllocationInfo_r4, PUSCH_CapacityAllocationInfo_r4_sequence);

  return offset;
}


static const per_sequence_t T_old_Configuration_01_sequence[] = {
  { &hf_rrc_tfcs_ID_01      , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_TFCS_IdentityPlain },
  { &hf_rrc_pdsch_Identity  , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PDSCH_Identity },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_old_Configuration_01(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_old_Configuration_01, T_old_Configuration_01_sequence);

  return offset;
}


static const per_sequence_t T_tdd384_27_sequence[] = {
  { &hf_rrc_pdsch_TimeslotsCodes, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DownlinkTimeslotsCodes },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_tdd384_27(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_tdd384_27, T_tdd384_27_sequence);

  return offset;
}


static const per_sequence_t T_tdd128_44_sequence[] = {
  { &hf_rrc_pdsch_TimeslotsCodes_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DownlinkTimeslotsCodes_LCR_r4 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_tdd128_44(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_tdd128_44, T_tdd128_44_sequence);

  return offset;
}


static const value_string rrc_T_tddOption_12_vals[] = {
  {   0, "tdd384" },
  {   1, "tdd128" },
  { 0, NULL }
};

static const per_choice_t T_tddOption_12_choice[] = {
  {   0, &hf_rrc_tdd384_29       , ASN1_NO_EXTENSIONS     , dissect_rrc_T_tdd384_27 },
  {   1, &hf_rrc_tdd128_45       , ASN1_NO_EXTENSIONS     , dissect_rrc_T_tdd128_44 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_tddOption_12(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_tddOption_12, T_tddOption_12_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t PDSCH_Info_r4_sequence[] = {
  { &hf_rrc_tfcs_ID_01      , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_TFCS_IdentityPlain },
  { &hf_rrc_commonTimeslotInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CommonTimeslotInfo },
  { &hf_rrc_tddOption_12    , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_tddOption_12 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_PDSCH_Info_r4(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_PDSCH_Info_r4, PDSCH_Info_r4_sequence);

  return offset;
}


static const per_sequence_t T_new_Configuration_01_sequence[] = {
  { &hf_rrc_pdsch_Info_01   , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PDSCH_Info_r4 },
  { &hf_rrc_pdsch_Identity  , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_PDSCH_Identity },
  { &hf_rrc_pdsch_PowerControlInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_PDSCH_PowerControlInfo },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_new_Configuration_01(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_new_Configuration_01, T_new_Configuration_01_sequence);

  return offset;
}


static const value_string rrc_T_configuration_01_vals[] = {
  {   0, "old-Configuration" },
  {   1, "new-Configuration" },
  { 0, NULL }
};

static const per_choice_t T_configuration_01_choice[] = {
  {   0, &hf_rrc_old_Configuration_01, ASN1_NO_EXTENSIONS     , dissect_rrc_T_old_Configuration_01 },
  {   1, &hf_rrc_new_Configuration_01, ASN1_NO_EXTENSIONS     , dissect_rrc_T_new_Configuration_01 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_configuration_01(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_configuration_01, T_configuration_01_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t PDSCH_CapacityAllocationInfo_r4_sequence[] = {
  { &hf_rrc_pdsch_AllocationPeriodInfo, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_AllocationPeriodInfo },
  { &hf_rrc_configuration_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_configuration_01 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_PDSCH_CapacityAllocationInfo_r4(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_PDSCH_CapacityAllocationInfo_r4, PDSCH_CapacityAllocationInfo_r4_sequence);

  return offset;
}


static const value_string rrc_T_confirmRequest_01_vals[] = {
  {   0, "confirmPDSCH" },
  {   1, "confirmPUSCH" },
  { 0, NULL }
};


static int
dissect_rrc_T_confirmRequest_01(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     2, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t SEQUENCE_SIZE_1_maxTS_OF_TimeslotNumber_sequence_of[1] = {
  { &hf_rrc_tdd384_item     , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TimeslotNumber },
};

static int
dissect_rrc_SEQUENCE_SIZE_1_maxTS_OF_TimeslotNumber(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_SEQUENCE_SIZE_1_maxTS_OF_TimeslotNumber, SEQUENCE_SIZE_1_maxTS_OF_TimeslotNumber_sequence_of,
                                                  1, maxTS, FALSE);

  return offset;
}


static const per_sequence_t SEQUENCE_SIZE_1_maxTS_LCR_OF_TimeslotNumber_LCR_r4_sequence_of[1] = {
  { &hf_rrc_tdd128_item_01  , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TimeslotNumber_LCR_r4 },
};

static int
dissect_rrc_SEQUENCE_SIZE_1_maxTS_LCR_OF_TimeslotNumber_LCR_r4(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_SEQUENCE_SIZE_1_maxTS_LCR_OF_TimeslotNumber_LCR_r4, SEQUENCE_SIZE_1_maxTS_LCR_OF_TimeslotNumber_LCR_r4_sequence_of,
                                                  1, maxTS_LCR, FALSE);

  return offset;
}


static const value_string rrc_TimeslotList_r4_vals[] = {
  {   0, "tdd384" },
  {   1, "tdd128" },
  { 0, NULL }
};

static const per_choice_t TimeslotList_r4_choice[] = {
  {   0, &hf_rrc_tdd384_37       , ASN1_NO_EXTENSIONS     , dissect_rrc_SEQUENCE_SIZE_1_maxTS_OF_TimeslotNumber },
  {   1, &hf_rrc_tdd128_54       , ASN1_NO_EXTENSIONS     , dissect_rrc_SEQUENCE_SIZE_1_maxTS_LCR_OF_TimeslotNumber_LCR_r4 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_TimeslotList_r4(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_TimeslotList_r4, TimeslotList_r4_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t PhysicalSharedChannelAllocation_r4_IEs_sequence[] = {
  { &hf_rrc_ul_TimingAdvance_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_TimingAdvanceControl_r4 },
  { &hf_rrc_pusch_CapacityAllocationInfo_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_PUSCH_CapacityAllocationInfo_r4 },
  { &hf_rrc_pdsch_CapacityAllocationInfo_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_PDSCH_CapacityAllocationInfo_r4 },
  { &hf_rrc_confirmRequest_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_confirmRequest_01 },
  { &hf_rrc_trafficVolumeReportRequest, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_INTEGER_0_255 },
  { &hf_rrc_iscpTimeslotList_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_TimeslotList_r4 },
  { &hf_rrc_requestPCCPCHRSCP, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_PhysicalSharedChannelAllocation_r4_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_PhysicalSharedChannelAllocation_r4_IEs, PhysicalSharedChannelAllocation_r4_IEs_sequence);

  return offset;
}


static const per_sequence_t PhysicalSharedChannelAllocation_v690ext_IEs_sequence[] = {
  { &hf_rrc_beaconPLEst     , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_BEACON_PL_Est },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_PhysicalSharedChannelAllocation_v690ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_PhysicalSharedChannelAllocation_v690ext_IEs, PhysicalSharedChannelAllocation_v690ext_IEs_sequence);

  return offset;
}


static const per_sequence_t T_tdd128_49_sequence[] = {
  { &hf_rrc_tpc_StepSize    , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_TPC_StepSizeTDD },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_tdd128_49(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_tdd128_49, T_tdd128_49_sequence);

  return offset;
}


static const value_string rrc_T_tddOption_17_vals[] = {
  {   0, "tdd384" },
  {   1, "tdd768" },
  {   2, "tdd128" },
  { 0, NULL }
};

static const per_choice_t T_tddOption_17_choice[] = {
  {   0, &hf_rrc_tdd384_11       , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   1, &hf_rrc_tdd768_06       , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   2, &hf_rrc_tdd128_50       , ASN1_NO_EXTENSIONS     , dissect_rrc_T_tdd128_49 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_tddOption_17(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_tddOption_17, T_tddOption_17_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t PUSCH_PowerControlInfo_r7_sequence[] = {
  { &hf_rrc_ul_TargetSIR    , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UL_TargetSIR },
  { &hf_rrc_tddOption_17    , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_tddOption_17 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_PUSCH_PowerControlInfo_r7(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_PUSCH_PowerControlInfo_r7, PUSCH_PowerControlInfo_r7_sequence);

  return offset;
}


static const per_sequence_t T_old_Configuration_05_sequence[] = {
  { &hf_rrc_tfcs_ID_01      , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_TFCS_IdentityPlain },
  { &hf_rrc_pusch_Identity  , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PUSCH_Identity },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_old_Configuration_05(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_old_Configuration_05, T_old_Configuration_05_sequence);

  return offset;
}


static const per_sequence_t PUSCH_Info_VHCR_sequence[] = {
  { &hf_rrc_tfcs_ID_01      , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_TFCS_IdentityPlain },
  { &hf_rrc_commonTimeslotInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CommonTimeslotInfo },
  { &hf_rrc_pusch_TimeslotsCodes_VHCR, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UplinkTimeslotsCodes_VHCR },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_PUSCH_Info_VHCR(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_PUSCH_Info_VHCR, PUSCH_Info_VHCR_sequence);

  return offset;
}


static const per_sequence_t T_new_Configuration_05_sequence[] = {
  { &hf_rrc_pusch_Info_02   , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PUSCH_Info_VHCR },
  { &hf_rrc_pusch_Identity  , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_PUSCH_Identity },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_new_Configuration_05(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_new_Configuration_05, T_new_Configuration_05_sequence);

  return offset;
}


static const value_string rrc_T_configuration_05_vals[] = {
  {   0, "old-Configuration" },
  {   1, "new-Configuration" },
  { 0, NULL }
};

static const per_choice_t T_configuration_05_choice[] = {
  {   0, &hf_rrc_old_Configuration_05, ASN1_NO_EXTENSIONS     , dissect_rrc_T_old_Configuration_05 },
  {   1, &hf_rrc_new_Configuration_05, ASN1_NO_EXTENSIONS     , dissect_rrc_T_new_Configuration_05 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_configuration_05(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_configuration_05, T_configuration_05_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t T_pusch_AllocationAssignment_02_sequence[] = {
  { &hf_rrc_pusch_AllocationPeriodInfo, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_AllocationPeriodInfo },
  { &hf_rrc_pusch_PowerControlInfo_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_PUSCH_PowerControlInfo_r7 },
  { &hf_rrc_configuration_05, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_configuration_05 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_pusch_AllocationAssignment_02(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_pusch_AllocationAssignment_02, T_pusch_AllocationAssignment_02_sequence);

  return offset;
}


static const value_string rrc_T_pusch_Allocation_02_vals[] = {
  {   0, "pusch-AllocationPending" },
  {   1, "pusch-AllocationAssignment" },
  { 0, NULL }
};

static const per_choice_t T_pusch_Allocation_02_choice[] = {
  {   0, &hf_rrc_pusch_AllocationPending, ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   1, &hf_rrc_pusch_AllocationAssignment_02, ASN1_NO_EXTENSIONS     , dissect_rrc_T_pusch_AllocationAssignment_02 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_pusch_Allocation_02(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_pusch_Allocation_02, T_pusch_Allocation_02_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t PUSCH_CapacityAllocationInfo_r7_sequence[] = {
  { &hf_rrc_pusch_Allocation_02, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_pusch_Allocation_02 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_PUSCH_CapacityAllocationInfo_r7(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_PUSCH_CapacityAllocationInfo_r7, PUSCH_CapacityAllocationInfo_r7_sequence);

  return offset;
}


static const per_sequence_t T_old_Configuration_02_sequence[] = {
  { &hf_rrc_tfcs_ID_01      , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_TFCS_IdentityPlain },
  { &hf_rrc_pdsch_Identity  , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PDSCH_Identity },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_old_Configuration_02(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_old_Configuration_02, T_old_Configuration_02_sequence);

  return offset;
}


static const per_sequence_t T_tdd384_28_sequence[] = {
  { &hf_rrc_pdsch_TimeslotsCodes, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DownlinkTimeslotsCodes },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_tdd384_28(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_tdd384_28, T_tdd384_28_sequence);

  return offset;
}


static const per_sequence_t T_tdd768_17_sequence[] = {
  { &hf_rrc_pdsch_TimeslotsCodes_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DownlinkTimeslotsCodes_VHCR },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_tdd768_17(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_tdd768_17, T_tdd768_17_sequence);

  return offset;
}


static const per_sequence_t T_tdd128_45_sequence[] = {
  { &hf_rrc_pdsch_TimeslotsCodes_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DownlinkTimeslotsCodes_LCR_r4 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_tdd128_45(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_tdd128_45, T_tdd128_45_sequence);

  return offset;
}


static const value_string rrc_T_tddOption_13_vals[] = {
  {   0, "tdd384" },
  {   1, "tdd768" },
  {   2, "tdd128" },
  { 0, NULL }
};

static const per_choice_t T_tddOption_13_choice[] = {
  {   0, &hf_rrc_tdd384_30       , ASN1_NO_EXTENSIONS     , dissect_rrc_T_tdd384_28 },
  {   1, &hf_rrc_tdd768_19       , ASN1_NO_EXTENSIONS     , dissect_rrc_T_tdd768_17 },
  {   2, &hf_rrc_tdd128_46       , ASN1_NO_EXTENSIONS     , dissect_rrc_T_tdd128_45 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_tddOption_13(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_tddOption_13, T_tddOption_13_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t PDSCH_Info_r7_sequence[] = {
  { &hf_rrc_tfcs_ID_01      , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_TFCS_IdentityPlain },
  { &hf_rrc_commonTimeslotInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CommonTimeslotInfo },
  { &hf_rrc_tddOption_13    , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_tddOption_13 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_PDSCH_Info_r7(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_PDSCH_Info_r7, PDSCH_Info_r7_sequence);

  return offset;
}


static const per_sequence_t T_new_Configuration_02_sequence[] = {
  { &hf_rrc_pdsch_Info_02   , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PDSCH_Info_r7 },
  { &hf_rrc_pdsch_Identity  , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_PDSCH_Identity },
  { &hf_rrc_pdsch_PowerControlInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_PDSCH_PowerControlInfo },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_new_Configuration_02(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_new_Configuration_02, T_new_Configuration_02_sequence);

  return offset;
}


static const value_string rrc_T_configuration_02_vals[] = {
  {   0, "old-Configuration" },
  {   1, "new-Configuration" },
  { 0, NULL }
};

static const per_choice_t T_configuration_02_choice[] = {
  {   0, &hf_rrc_old_Configuration_02, ASN1_NO_EXTENSIONS     , dissect_rrc_T_old_Configuration_02 },
  {   1, &hf_rrc_new_Configuration_02, ASN1_NO_EXTENSIONS     , dissect_rrc_T_new_Configuration_02 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_configuration_02(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_configuration_02, T_configuration_02_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t PDSCH_CapacityAllocationInfo_r7_sequence[] = {
  { &hf_rrc_pdsch_AllocationPeriodInfo, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_AllocationPeriodInfo },
  { &hf_rrc_configuration_02, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_configuration_02 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_PDSCH_CapacityAllocationInfo_r7(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_PDSCH_CapacityAllocationInfo_r7, PDSCH_CapacityAllocationInfo_r7_sequence);

  return offset;
}


static const per_sequence_t PhysicalSharedChannelAllocation_v770ext_IEs_sequence[] = {
  { &hf_rrc_ul_TimingAdvance_03, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_TimingAdvanceControl_r7 },
  { &hf_rrc_pusch_CapacityAllocationInfo_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_PUSCH_CapacityAllocationInfo_r7 },
  { &hf_rrc_pdsch_CapacityAllocationInfo_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_PDSCH_CapacityAllocationInfo_r7 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_PhysicalSharedChannelAllocation_v770ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_PhysicalSharedChannelAllocation_v770ext_IEs, PhysicalSharedChannelAllocation_v770ext_IEs_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions_82_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_nonCriticalExtensions_82(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_nonCriticalExtensions_82, T_nonCriticalExtensions_82_sequence);

  return offset;
}


static const per_sequence_t T_v770NonCriticalExtensions_09_sequence[] = {
  { &hf_rrc_physicalSharedChannelAllocation_v770ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PhysicalSharedChannelAllocation_v770ext_IEs },
  { &hf_rrc_nonCriticalExtensions_82, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_nonCriticalExtensions_82 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v770NonCriticalExtensions_09(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v770NonCriticalExtensions_09, T_v770NonCriticalExtensions_09_sequence);

  return offset;
}


static const per_sequence_t T_v690NonCriticalExtensions_15_sequence[] = {
  { &hf_rrc_physicalSharedChannelAllocation_v690ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PhysicalSharedChannelAllocation_v690ext_IEs },
  { &hf_rrc_v770NonCriticalExtensions_09, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v770NonCriticalExtensions_09 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v690NonCriticalExtensions_15(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v690NonCriticalExtensions_15, T_v690NonCriticalExtensions_15_sequence);

  return offset;
}


static const per_sequence_t T_v4d0NonCriticalExtensions_05_sequence[] = {
  { &hf_rrc_physicalSharedChannelAllocation_r4_add_ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_BIT_STRING },
  { &hf_rrc_v690NonCriticalExtensions_15, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v690NonCriticalExtensions_15 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v4d0NonCriticalExtensions_05(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v4d0NonCriticalExtensions_05, T_v4d0NonCriticalExtensions_05_sequence);

  return offset;
}


static const per_sequence_t T_r4_05_sequence[] = {
  { &hf_rrc_physicalSharedChannelAllocation_r4, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PhysicalSharedChannelAllocation_r4_IEs },
  { &hf_rrc_v4d0NonCriticalExtensions_05, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v4d0NonCriticalExtensions_05 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_r4_05(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_r4_05, T_r4_05_sequence);

  return offset;
}


static const per_sequence_t T_criticalExtensions_63_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_criticalExtensions_63(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_criticalExtensions_63, T_criticalExtensions_63_sequence);

  return offset;
}


static const value_string rrc_T_criticalExtensions_62_vals[] = {
  {   0, "r4" },
  {   1, "criticalExtensions" },
  { 0, NULL }
};

static const per_choice_t T_criticalExtensions_62_choice[] = {
  {   0, &hf_rrc_r4_05           , ASN1_NO_EXTENSIONS     , dissect_rrc_T_r4_05 },
  {   1, &hf_rrc_criticalExtensions_64, ASN1_NO_EXTENSIONS     , dissect_rrc_T_criticalExtensions_63 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_criticalExtensions_62(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_criticalExtensions_62, T_criticalExtensions_62_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t T_later_than_r3_11_sequence[] = {
  { &hf_rrc_dsch_RNTI       , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DSCH_RNTI },
  { &hf_rrc_rrc_TransactionIdentifier, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RRC_TransactionIdentifier },
  { &hf_rrc_criticalExtensions_63, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_criticalExtensions_62 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_later_than_r3_11(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_later_than_r3_11, T_later_than_r3_11_sequence);

  return offset;
}


static const value_string rrc_PhysicalSharedChannelAllocation_vals[] = {
  {   0, "r3" },
  {   1, "later-than-r3" },
  { 0, NULL }
};

static const per_choice_t PhysicalSharedChannelAllocation_choice[] = {
  {   0, &hf_rrc_r3_13           , ASN1_NO_EXTENSIONS     , dissect_rrc_T_r3_13 },
  {   1, &hf_rrc_later_than_r3_11, ASN1_NO_EXTENSIONS     , dissect_rrc_T_later_than_r3_11 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_PhysicalSharedChannelAllocation(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
#line 317 "../../asn1/rrc/rrc.cnf"
  col_append_str(actx->pinfo->cinfo, COL_INFO, "PhysicalSharedChannelAllocation");
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_PhysicalSharedChannelAllocation, PhysicalSharedChannelAllocation_choice,
                                 NULL);




  return offset;
}


static const per_sequence_t RAB_InformationReconfig_sequence[] = {
  { &hf_rrc_rab_Identity    , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RAB_Identity },
  { &hf_rrc_cn_DomainIdentity, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CN_DomainIdentity },
  { &hf_rrc_nas_Synchronisation_Indicator, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_NAS_Synchronisation_Indicator },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_RAB_InformationReconfig(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_RAB_InformationReconfig, RAB_InformationReconfig_sequence);

  return offset;
}


static const per_sequence_t RAB_InformationReconfigList_sequence_of[1] = {
  { &hf_rrc_RAB_InformationReconfigList_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RAB_InformationReconfig },
};

static int
dissect_rrc_RAB_InformationReconfigList(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_RAB_InformationReconfigList, RAB_InformationReconfigList_sequence_of,
                                                  1, maxRABsetup, FALSE);

  return offset;
}


static const per_sequence_t T_fdd_21_sequence[] = {
  { &hf_rrc_dummy1          , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CPCH_SetID },
  { &hf_rrc_dummy2_02       , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DRAC_StaticInformationList },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_fdd_21(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_fdd_21, T_fdd_21_sequence);

  return offset;
}


static const value_string rrc_T_dummy_01_vals[] = {
  {   0, "fdd" },
  {   1, "tdd" },
  { 0, NULL }
};

static const per_choice_t T_dummy_01_choice[] = {
  {   0, &hf_rrc_fdd_21          , ASN1_NO_EXTENSIONS     , dissect_rrc_T_fdd_21 },
  {   1, &hf_rrc_tdd             , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_dummy_01(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_dummy_01, T_dummy_01_choice,
                                 NULL);

  return offset;
}


static const value_string rrc_T_tfs_SignallingMode_06_vals[] = {
  {   0, "explicit-config" },
  {   1, "sameAsULTrCH" },
  { 0, NULL }
};

static const per_choice_t T_tfs_SignallingMode_06_choice[] = {
  {   0, &hf_rrc_explicit_config , ASN1_NO_EXTENSIONS     , dissect_rrc_TransportFormatSet },
  {   1, &hf_rrc_sameAsULTrCH    , ASN1_NO_EXTENSIONS     , dissect_rrc_UL_TransportChannelIdentity },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_tfs_SignallingMode_06(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_tfs_SignallingMode_06, T_tfs_SignallingMode_06_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t DL_AddReconfTransChInformation2_sequence[] = {
  { &hf_rrc_dl_TransportChannelType_03, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DL_TrCH_Type },
  { &hf_rrc_transportChannelIdentity, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TransportChannelIdentity },
  { &hf_rrc_tfs_SignallingMode_06, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_tfs_SignallingMode_06 },
  { &hf_rrc_qualityTarget   , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_QualityTarget },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_DL_AddReconfTransChInformation2(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_DL_AddReconfTransChInformation2, DL_AddReconfTransChInformation2_sequence);

  return offset;
}


static const per_sequence_t DL_AddReconfTransChInfo2List_sequence_of[1] = {
  { &hf_rrc_DL_AddReconfTransChInfo2List_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DL_AddReconfTransChInformation2 },
};

static int
dissect_rrc_DL_AddReconfTransChInfo2List(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_DL_AddReconfTransChInfo2List, DL_AddReconfTransChInfo2List_sequence_of,
                                                  1, maxTrCHpreconf, FALSE);

  return offset;
}


static const per_sequence_t T_fdd_22_sequence[] = {
  { &hf_rrc_dummy_06        , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_PDSCH_Information },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_fdd_22(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_fdd_22, T_fdd_22_sequence);

  return offset;
}


static const value_string rrc_T_modeSpecificPhysChInfo_03_vals[] = {
  {   0, "fdd" },
  {   1, "tdd" },
  { 0, NULL }
};

static const per_choice_t T_modeSpecificPhysChInfo_03_choice[] = {
  {   0, &hf_rrc_fdd_22          , ASN1_NO_EXTENSIONS     , dissect_rrc_T_fdd_22 },
  {   1, &hf_rrc_tdd             , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_modeSpecificPhysChInfo_03(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_modeSpecificPhysChInfo_03, T_modeSpecificPhysChInfo_03_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t RadioBearerReconfiguration_r3_IEs_sequence[] = {
  { &hf_rrc_rrc_TransactionIdentifier, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RRC_TransactionIdentifier },
  { &hf_rrc_integrityProtectionModeInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_IntegrityProtectionModeInfo },
  { &hf_rrc_cipheringModeInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CipheringModeInfo },
  { &hf_rrc_activationTime  , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_ActivationTime },
  { &hf_rrc_new_U_RNTI      , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_U_RNTI },
  { &hf_rrc_new_C_RNTI      , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_C_RNTI },
  { &hf_rrc_rrc_StateIndicator, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RRC_StateIndicator },
  { &hf_rrc_utran_DRX_CycleLengthCoeff, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UTRAN_DRX_CycleLengthCoefficient },
  { &hf_rrc_cn_InformationInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CN_InformationInfo },
  { &hf_rrc_ura_Identity    , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_URA_Identity },
  { &hf_rrc_rab_InformationReconfigList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RAB_InformationReconfigList },
  { &hf_rrc_rb_InformationReconfigList, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RB_InformationReconfigList },
  { &hf_rrc_rb_InformationAffectedList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RB_InformationAffectedList },
  { &hf_rrc_ul_CommonTransChInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_CommonTransChInfo },
  { &hf_rrc_ul_deletedTransChInfoList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_DeletedTransChInfoList },
  { &hf_rrc_ul_AddReconfTransChInfoList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_AddReconfTransChInfoList },
  { &hf_rrc_dummy_14        , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_dummy_01 },
  { &hf_rrc_dl_CommonTransChInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_CommonTransChInfo },
  { &hf_rrc_dl_DeletedTransChInfoList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_DeletedTransChInfoList },
  { &hf_rrc_dl_AddReconfTransChInfoList_07, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_AddReconfTransChInfo2List },
  { &hf_rrc_frequencyInfo   , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_FrequencyInfo },
  { &hf_rrc_maxAllowedUL_TX_Power, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MaxAllowedUL_TX_Power },
  { &hf_rrc_ul_ChannelRequirement, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_ChannelRequirement },
  { &hf_rrc_modeSpecificPhysChInfo_03, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_modeSpecificPhysChInfo_03 },
  { &hf_rrc_dl_CommonInformation, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_CommonInformation },
  { &hf_rrc_dl_InformationPerRL_List, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DL_InformationPerRL_List },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_RadioBearerReconfiguration_r3_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_RadioBearerReconfiguration_r3_IEs, RadioBearerReconfiguration_r3_IEs_sequence);

  return offset;
}


static const per_sequence_t RadioBearerReconfiguration_v3a0ext_sequence[] = {
  { &hf_rrc_new_DSCH_RNTI   , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DSCH_RNTI },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_RadioBearerReconfiguration_v3a0ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_RadioBearerReconfiguration_v3a0ext, RadioBearerReconfiguration_v3a0ext_sequence);

  return offset;
}


static const per_sequence_t RadioBearerReconfiguration_v4b0ext_IEs_sequence[] = {
  { &hf_rrc_dummy_02        , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_SSDT_UL },
  { &hf_rrc_cell_id_PerRL_List, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CellIdentity_PerRL_List },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_RadioBearerReconfiguration_v4b0ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_RadioBearerReconfiguration_v4b0ext_IEs, RadioBearerReconfiguration_v4b0ext_IEs_sequence);

  return offset;
}


static const per_sequence_t RadioBearerReconfiguration_v590ext_IEs_sequence[] = {
  { &hf_rrc_dl_TPC_PowerOffsetPerRL_List, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_TPC_PowerOffsetPerRL_List },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_RadioBearerReconfiguration_v590ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_RadioBearerReconfiguration_v590ext_IEs, RadioBearerReconfiguration_v590ext_IEs_sequence);

  return offset;
}


static const per_sequence_t RadioBearerReconfiguration_v5d0ext_IEs_sequence[] = {
  { &hf_rrc_pdcp_ROHC_TargetMode, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_PDCP_ROHC_TargetMode },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_RadioBearerReconfiguration_v5d0ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_RadioBearerReconfiguration_v5d0ext_IEs, RadioBearerReconfiguration_v5d0ext_IEs_sequence);

  return offset;
}


static const value_string rrc_T_postVerificationPeriod_02_vals[] = {
  {   0, "true" },
  { 0, NULL }
};


static int
dissect_rrc_T_postVerificationPeriod_02(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t RadioBearerReconfiguration_v690ext_IEs_sequence[] = {
  { &hf_rrc_delayRestrictionFlag, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DelayRestrictionFlag },
  { &hf_rrc_primary_plmn_Identity, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_PLMN_Identity },
  { &hf_rrc_harq_Preamble_Mode, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_HARQ_Preamble_Mode },
  { &hf_rrc_beaconPLEst     , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_BEACON_PL_Est },
  { &hf_rrc_postVerificationPeriod_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_postVerificationPeriod_02 },
  { &hf_rrc_dhs_sync        , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DHS_Sync },
  { &hf_rrc_timingMaintainedSynchInd, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_TimingMaintainedSynchInd },
  { &hf_rrc_mbms_PL_ServiceRestrictInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MBMS_PL_ServiceRestrictInfo_r6 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_RadioBearerReconfiguration_v690ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_RadioBearerReconfiguration_v690ext_IEs, RadioBearerReconfiguration_v690ext_IEs_sequence);

  return offset;
}


static const per_sequence_t RadioBearerReconfiguration_v770ext_IEs_sequence[] = {
  { &hf_rrc_ueMobilityStateIndicator, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_High_MobilityDetected },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_RadioBearerReconfiguration_v770ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_RadioBearerReconfiguration_v770ext_IEs, RadioBearerReconfiguration_v770ext_IEs_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions_84_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_nonCriticalExtensions_84(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_nonCriticalExtensions_84, T_nonCriticalExtensions_84_sequence);

  return offset;
}


static const per_sequence_t T_v770NonCriticalExtensions_10_sequence[] = {
  { &hf_rrc_radioBearerReconfiguration_v770ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RadioBearerReconfiguration_v770ext_IEs },
  { &hf_rrc_nonCriticalExtensions_84, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_nonCriticalExtensions_84 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v770NonCriticalExtensions_10(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v770NonCriticalExtensions_10, T_v770NonCriticalExtensions_10_sequence);

  return offset;
}


static const per_sequence_t T_v690NonCriticalExtensions_16_sequence[] = {
  { &hf_rrc_radioBearerReconfiguration_v690ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RadioBearerReconfiguration_v690ext_IEs },
  { &hf_rrc_v770NonCriticalExtensions_10, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v770NonCriticalExtensions_10 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v690NonCriticalExtensions_16(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v690NonCriticalExtensions_16, T_v690NonCriticalExtensions_16_sequence);

  return offset;
}


static const per_sequence_t T_v5d0NonCriticalExtenstions_06_sequence[] = {
  { &hf_rrc_radioBearerReconfiguration_v5d0ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RadioBearerReconfiguration_v5d0ext_IEs },
  { &hf_rrc_v690NonCriticalExtensions_16, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v690NonCriticalExtensions_16 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v5d0NonCriticalExtenstions_06(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v5d0NonCriticalExtenstions_06, T_v5d0NonCriticalExtenstions_06_sequence);

  return offset;
}


static const per_sequence_t T_v590NonCriticalExtensions_14_sequence[] = {
  { &hf_rrc_radioBearerReconfiguration_v590ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RadioBearerReconfiguration_v590ext_IEs },
  { &hf_rrc_v5d0NonCriticalExtenstions_06, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v5d0NonCriticalExtenstions_06 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v590NonCriticalExtensions_14(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v590NonCriticalExtensions_14, T_v590NonCriticalExtensions_14_sequence);

  return offset;
}


static const per_sequence_t T_v4b0NonCriticalExtensions_07_sequence[] = {
  { &hf_rrc_radioBearerReconfiguration_v4b0ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RadioBearerReconfiguration_v4b0ext_IEs },
  { &hf_rrc_v590NonCriticalExtensions_14, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v590NonCriticalExtensions_14 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v4b0NonCriticalExtensions_07(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v4b0NonCriticalExtensions_07, T_v4b0NonCriticalExtensions_07_sequence);

  return offset;
}


static const per_sequence_t T_laterNonCriticalExtensions_28_sequence[] = {
  { &hf_rrc_radioBearerReconfiguration_r3_add_ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_BIT_STRING },
  { &hf_rrc_v4b0NonCriticalExtensions_07, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v4b0NonCriticalExtensions_07 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_laterNonCriticalExtensions_28(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_laterNonCriticalExtensions_28, T_laterNonCriticalExtensions_28_sequence);

  return offset;
}


static const per_sequence_t T_v3aoNonCriticalExtensions_sequence[] = {
  { &hf_rrc_radioBearerReconfiguration_v3a0ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RadioBearerReconfiguration_v3a0ext },
  { &hf_rrc_laterNonCriticalExtensions_28, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_laterNonCriticalExtensions_28 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v3aoNonCriticalExtensions(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v3aoNonCriticalExtensions, T_v3aoNonCriticalExtensions_sequence);

  return offset;
}


static const per_sequence_t T_r3_14_sequence[] = {
  { &hf_rrc_radioBearerReconfiguration_r3, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RadioBearerReconfiguration_r3_IEs },
  { &hf_rrc_v3aoNonCriticalExtensions, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v3aoNonCriticalExtensions },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_r3_14(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_r3_14, T_r3_14_sequence);

  return offset;
}


static const per_sequence_t T_fdd_23_sequence[] = {
  { &hf_rrc_dummy1          , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CPCH_SetID },
  { &hf_rrc_dummy2_02       , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DRAC_StaticInformationList },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_fdd_23(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_fdd_23, T_fdd_23_sequence);

  return offset;
}


static const value_string rrc_T_dummy_02_vals[] = {
  {   0, "fdd" },
  {   1, "tdd" },
  { 0, NULL }
};

static const per_choice_t T_dummy_02_choice[] = {
  {   0, &hf_rrc_fdd_23          , ASN1_NO_EXTENSIONS     , dissect_rrc_T_fdd_23 },
  {   1, &hf_rrc_tdd             , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_dummy_02(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_dummy_02, T_dummy_02_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t T_fdd_24_sequence[] = {
  { &hf_rrc_dummy_06        , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_PDSCH_Information },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_fdd_24(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_fdd_24, T_fdd_24_sequence);

  return offset;
}


static const value_string rrc_T_modeSpecificPhysChInfo_04_vals[] = {
  {   0, "fdd" },
  {   1, "tdd" },
  { 0, NULL }
};

static const per_choice_t T_modeSpecificPhysChInfo_04_choice[] = {
  {   0, &hf_rrc_fdd_24          , ASN1_NO_EXTENSIONS     , dissect_rrc_T_fdd_24 },
  {   1, &hf_rrc_tdd             , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_modeSpecificPhysChInfo_04(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_modeSpecificPhysChInfo_04, T_modeSpecificPhysChInfo_04_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t RadioBearerReconfiguration_r4_IEs_sequence[] = {
  { &hf_rrc_integrityProtectionModeInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_IntegrityProtectionModeInfo },
  { &hf_rrc_cipheringModeInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CipheringModeInfo },
  { &hf_rrc_activationTime  , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_ActivationTime },
  { &hf_rrc_new_U_RNTI      , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_U_RNTI },
  { &hf_rrc_new_C_RNTI      , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_C_RNTI },
  { &hf_rrc_new_DSCH_RNTI   , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DSCH_RNTI },
  { &hf_rrc_rrc_StateIndicator, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RRC_StateIndicator },
  { &hf_rrc_utran_DRX_CycleLengthCoeff, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UTRAN_DRX_CycleLengthCoefficient },
  { &hf_rrc_cn_InformationInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CN_InformationInfo },
  { &hf_rrc_ura_Identity    , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_URA_Identity },
  { &hf_rrc_rab_InformationReconfigList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RAB_InformationReconfigList },
  { &hf_rrc_rb_InformationReconfigList_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RB_InformationReconfigList_r4 },
  { &hf_rrc_rb_InformationAffectedList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RB_InformationAffectedList },
  { &hf_rrc_ul_CommonTransChInfo_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_CommonTransChInfo_r4 },
  { &hf_rrc_ul_deletedTransChInfoList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_DeletedTransChInfoList },
  { &hf_rrc_ul_AddReconfTransChInfoList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_AddReconfTransChInfoList },
  { &hf_rrc_dummy_15        , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_dummy_02 },
  { &hf_rrc_dl_CommonTransChInfo_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_CommonTransChInfo_r4 },
  { &hf_rrc_dl_DeletedTransChInfoList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_DeletedTransChInfoList },
  { &hf_rrc_dl_AddReconfTransChInfoList_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_AddReconfTransChInfoList_r4 },
  { &hf_rrc_frequencyInfo   , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_FrequencyInfo },
  { &hf_rrc_maxAllowedUL_TX_Power, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MaxAllowedUL_TX_Power },
  { &hf_rrc_ul_ChannelRequirement_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_ChannelRequirement_r4 },
  { &hf_rrc_modeSpecificPhysChInfo_04, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_modeSpecificPhysChInfo_04 },
  { &hf_rrc_dl_CommonInformation_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_CommonInformation_r4 },
  { &hf_rrc_dl_InformationPerRL_List_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_InformationPerRL_List_r4 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_RadioBearerReconfiguration_r4_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_RadioBearerReconfiguration_r4_IEs, RadioBearerReconfiguration_r4_IEs_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions_85_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_nonCriticalExtensions_85(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_nonCriticalExtensions_85, T_nonCriticalExtensions_85_sequence);

  return offset;
}


static const per_sequence_t T_v770NonCriticalExtensions_11_sequence[] = {
  { &hf_rrc_radioBearerReconfiguration_v770ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RadioBearerReconfiguration_v770ext_IEs },
  { &hf_rrc_nonCriticalExtensions_85, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_nonCriticalExtensions_85 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v770NonCriticalExtensions_11(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v770NonCriticalExtensions_11, T_v770NonCriticalExtensions_11_sequence);

  return offset;
}


static const per_sequence_t T_v690NonCriticalExtensions_17_sequence[] = {
  { &hf_rrc_radioBearerReconfiguration_v690ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RadioBearerReconfiguration_v690ext_IEs },
  { &hf_rrc_v770NonCriticalExtensions_11, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v770NonCriticalExtensions_11 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v690NonCriticalExtensions_17(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v690NonCriticalExtensions_17, T_v690NonCriticalExtensions_17_sequence);

  return offset;
}


static const per_sequence_t T_v5d0NonCriticalExtenstions_07_sequence[] = {
  { &hf_rrc_radioBearerReconfiguration_v5d0ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RadioBearerReconfiguration_v5d0ext_IEs },
  { &hf_rrc_v690NonCriticalExtensions_17, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v690NonCriticalExtensions_17 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v5d0NonCriticalExtenstions_07(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v5d0NonCriticalExtenstions_07, T_v5d0NonCriticalExtenstions_07_sequence);

  return offset;
}


static const per_sequence_t T_v590NonCriticalExtensions_15_sequence[] = {
  { &hf_rrc_radioBearerReconfiguration_v590ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RadioBearerReconfiguration_v590ext_IEs },
  { &hf_rrc_v5d0NonCriticalExtenstions_07, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v5d0NonCriticalExtenstions_07 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v590NonCriticalExtensions_15(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v590NonCriticalExtensions_15, T_v590NonCriticalExtensions_15_sequence);

  return offset;
}


static const per_sequence_t T_v4d0NonCriticalExtensions_06_sequence[] = {
  { &hf_rrc_radioBearerReconfiguration_r4_add_ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_BIT_STRING },
  { &hf_rrc_v590NonCriticalExtensions_15, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v590NonCriticalExtensions_15 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v4d0NonCriticalExtensions_06(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v4d0NonCriticalExtensions_06, T_v4d0NonCriticalExtensions_06_sequence);

  return offset;
}


static const per_sequence_t T_r4_06_sequence[] = {
  { &hf_rrc_radioBearerReconfiguration_r4, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RadioBearerReconfiguration_r4_IEs },
  { &hf_rrc_v4d0NonCriticalExtensions_06, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v4d0NonCriticalExtensions_06 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_r4_06(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_r4_06, T_r4_06_sequence);

  return offset;
}


static const per_sequence_t T_fdd_25_sequence[] = {
  { &hf_rrc_dummy1          , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CPCH_SetID },
  { &hf_rrc_dummy2_02       , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DRAC_StaticInformationList },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_fdd_25(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_fdd_25, T_fdd_25_sequence);

  return offset;
}


static const value_string rrc_T_dummy_03_vals[] = {
  {   0, "fdd" },
  {   1, "tdd" },
  { 0, NULL }
};

static const per_choice_t T_dummy_03_choice[] = {
  {   0, &hf_rrc_fdd_25          , ASN1_NO_EXTENSIONS     , dissect_rrc_T_fdd_25 },
  {   1, &hf_rrc_tdd             , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_dummy_03(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_dummy_03, T_dummy_03_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t T_complete_09_sequence[] = {
  { &hf_rrc_rab_InformationReconfigList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RAB_InformationReconfigList },
  { &hf_rrc_rb_InformationReconfigList_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RB_InformationReconfigList_r5 },
  { &hf_rrc_rb_InformationAffectedList_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RB_InformationAffectedList_r5 },
  { &hf_rrc_rb_PDCPContextRelocationList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RB_PDCPContextRelocationList },
  { &hf_rrc_ul_CommonTransChInfo_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_CommonTransChInfo_r4 },
  { &hf_rrc_ul_deletedTransChInfoList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_DeletedTransChInfoList },
  { &hf_rrc_ul_AddReconfTransChInfoList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_AddReconfTransChInfoList },
  { &hf_rrc_dummy_16        , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_dummy_03 },
  { &hf_rrc_dl_CommonTransChInfo_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_CommonTransChInfo_r4 },
  { &hf_rrc_dl_DeletedTransChInfoList_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_DeletedTransChInfoList_r5 },
  { &hf_rrc_dl_AddReconfTransChInfoList_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_AddReconfTransChInfoList_r5 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_complete_09(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_complete_09, T_complete_09_sequence);

  return offset;
}



static int
dissect_rrc_DefaultConfigIdentity_r5(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            0U, 13U, NULL, FALSE);

  return offset;
}


static const per_sequence_t T_defaultConfig_09_sequence[] = {
  { &hf_rrc_defaultConfigMode, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DefaultConfigMode },
  { &hf_rrc_defaultConfigIdentity_02, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DefaultConfigIdentity_r5 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_defaultConfig_09(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_defaultConfig_09, T_defaultConfig_09_sequence);

  return offset;
}


static const value_string rrc_T_preConfigMode_09_vals[] = {
  {   0, "predefinedConfigIdentity" },
  {   1, "defaultConfig" },
  { 0, NULL }
};

static const per_choice_t T_preConfigMode_09_choice[] = {
  {   0, &hf_rrc_predefinedConfigIdentity, ASN1_NO_EXTENSIONS     , dissect_rrc_PredefinedConfigIdentity },
  {   1, &hf_rrc_defaultConfig_09, ASN1_NO_EXTENSIONS     , dissect_rrc_T_defaultConfig_09 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_preConfigMode_09(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_preConfigMode_09, T_preConfigMode_09_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t T_preconfiguration_09_sequence[] = {
  { &hf_rrc_preConfigMode_09, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_preConfigMode_09 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_preconfiguration_09(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_preconfiguration_09, T_preconfiguration_09_sequence);

  return offset;
}


static const value_string rrc_T_specificationMode_09_vals[] = {
  {   0, "complete" },
  {   1, "preconfiguration" },
  { 0, NULL }
};

static const per_choice_t T_specificationMode_09_choice[] = {
  {   0, &hf_rrc_complete_09     , ASN1_NO_EXTENSIONS     , dissect_rrc_T_complete_09 },
  {   1, &hf_rrc_preconfiguration_09, ASN1_NO_EXTENSIONS     , dissect_rrc_T_preconfiguration_09 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_specificationMode_09(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_specificationMode_09, T_specificationMode_09_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t T_fdd_26_sequence[] = {
  { &hf_rrc_dummy_06        , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_PDSCH_Information },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_fdd_26(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_fdd_26, T_fdd_26_sequence);

  return offset;
}


static const value_string rrc_T_modeSpecificPhysChInfo_05_vals[] = {
  {   0, "fdd" },
  {   1, "tdd" },
  { 0, NULL }
};

static const per_choice_t T_modeSpecificPhysChInfo_05_choice[] = {
  {   0, &hf_rrc_fdd_26          , ASN1_NO_EXTENSIONS     , dissect_rrc_T_fdd_26 },
  {   1, &hf_rrc_tdd             , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_modeSpecificPhysChInfo_05(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_modeSpecificPhysChInfo_05, T_modeSpecificPhysChInfo_05_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t RadioBearerReconfiguration_r5_IEs_sequence[] = {
  { &hf_rrc_integrityProtectionModeInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_IntegrityProtectionModeInfo },
  { &hf_rrc_cipheringModeInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CipheringModeInfo },
  { &hf_rrc_activationTime  , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_ActivationTime },
  { &hf_rrc_new_U_RNTI      , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_U_RNTI },
  { &hf_rrc_new_C_RNTI      , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_C_RNTI },
  { &hf_rrc_new_DSCH_RNTI   , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DSCH_RNTI },
  { &hf_rrc_new_H_RNTI      , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_H_RNTI },
  { &hf_rrc_rrc_StateIndicator, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RRC_StateIndicator },
  { &hf_rrc_utran_DRX_CycleLengthCoeff, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UTRAN_DRX_CycleLengthCoefficient },
  { &hf_rrc_cn_InformationInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CN_InformationInfo },
  { &hf_rrc_ura_Identity    , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_URA_Identity },
  { &hf_rrc_specificationMode_09, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_specificationMode_09 },
  { &hf_rrc_frequencyInfo   , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_FrequencyInfo },
  { &hf_rrc_maxAllowedUL_TX_Power, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MaxAllowedUL_TX_Power },
  { &hf_rrc_ul_ChannelRequirement_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_ChannelRequirement_r5 },
  { &hf_rrc_modeSpecificPhysChInfo_05, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_modeSpecificPhysChInfo_05 },
  { &hf_rrc_dl_HSPDSCH_Information, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_HSPDSCH_Information },
  { &hf_rrc_dl_CommonInformation_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_CommonInformation_r5 },
  { &hf_rrc_dl_InformationPerRL_List_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_InformationPerRL_List_r5 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_RadioBearerReconfiguration_r5_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_RadioBearerReconfiguration_r5_IEs, RadioBearerReconfiguration_r5_IEs_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions_86_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_nonCriticalExtensions_86(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_nonCriticalExtensions_86, T_nonCriticalExtensions_86_sequence);

  return offset;
}


static const per_sequence_t T_v770NonCriticalExtensions_12_sequence[] = {
  { &hf_rrc_radioBearerReconfiguration_v770ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RadioBearerReconfiguration_v770ext_IEs },
  { &hf_rrc_nonCriticalExtensions_86, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_nonCriticalExtensions_86 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v770NonCriticalExtensions_12(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v770NonCriticalExtensions_12, T_v770NonCriticalExtensions_12_sequence);

  return offset;
}


static const per_sequence_t T_v690NonCriticalExtensions_18_sequence[] = {
  { &hf_rrc_radioBearerReconfiguration_v690ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RadioBearerReconfiguration_v690ext_IEs },
  { &hf_rrc_v770NonCriticalExtensions_12, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v770NonCriticalExtensions_12 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v690NonCriticalExtensions_18(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v690NonCriticalExtensions_18, T_v690NonCriticalExtensions_18_sequence);

  return offset;
}


static const per_sequence_t T_v5d0NonCriticalExtenstions_08_sequence[] = {
  { &hf_rrc_radioBearerReconfiguration_v5d0ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RadioBearerReconfiguration_v5d0ext_IEs },
  { &hf_rrc_v690NonCriticalExtensions_18, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v690NonCriticalExtensions_18 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v5d0NonCriticalExtenstions_08(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v5d0NonCriticalExtenstions_08, T_v5d0NonCriticalExtenstions_08_sequence);

  return offset;
}


static const per_sequence_t T_r5_05_sequence[] = {
  { &hf_rrc_radioBearerReconfiguration_r5, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RadioBearerReconfiguration_r5_IEs },
  { &hf_rrc_radioBearerReconfiguration_r5_add_ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_BIT_STRING },
  { &hf_rrc_v5d0NonCriticalExtenstions_08, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v5d0NonCriticalExtenstions_08 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_r5_05(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_r5_05, T_r5_05_sequence);

  return offset;
}


static const per_sequence_t T_complete_10_sequence[] = {
  { &hf_rrc_rab_InformationReconfigList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RAB_InformationReconfigList },
  { &hf_rrc_rb_InformationReconfigList_03, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RB_InformationReconfigList_r6 },
  { &hf_rrc_rb_InformationAffectedList_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RB_InformationAffectedList_r6 },
  { &hf_rrc_rb_PDCPContextRelocationList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RB_PDCPContextRelocationList },
  { &hf_rrc_pdcp_ROHC_TargetMode, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_PDCP_ROHC_TargetMode },
  { &hf_rrc_ul_CommonTransChInfo_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_CommonTransChInfo_r4 },
  { &hf_rrc_ul_deletedTransChInfoList_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_DeletedTransChInfoList_r6 },
  { &hf_rrc_ul_AddReconfTransChInfoList_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_AddReconfTransChInfoList_r6 },
  { &hf_rrc_dl_CommonTransChInfo_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_CommonTransChInfo_r4 },
  { &hf_rrc_dl_DeletedTransChInfoList_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_DeletedTransChInfoList_r5 },
  { &hf_rrc_dl_AddReconfTransChInfoList_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_AddReconfTransChInfoList_r5 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_complete_10(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_complete_10, T_complete_10_sequence);

  return offset;
}



static int
dissect_rrc_DefaultConfigIdentity_r6(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            0U, 31U, NULL, FALSE);

  return offset;
}


static const per_sequence_t T_defaultConfig_10_sequence[] = {
  { &hf_rrc_defaultConfigMode, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DefaultConfigMode },
  { &hf_rrc_defaultConfigIdentity_03, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DefaultConfigIdentity_r6 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_defaultConfig_10(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_defaultConfig_10, T_defaultConfig_10_sequence);

  return offset;
}


static const value_string rrc_T_preConfigMode_10_vals[] = {
  {   0, "predefinedConfigIdentity" },
  {   1, "defaultConfig" },
  { 0, NULL }
};

static const per_choice_t T_preConfigMode_10_choice[] = {
  {   0, &hf_rrc_predefinedConfigIdentity, ASN1_NO_EXTENSIONS     , dissect_rrc_PredefinedConfigIdentity },
  {   1, &hf_rrc_defaultConfig_10, ASN1_NO_EXTENSIONS     , dissect_rrc_T_defaultConfig_10 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_preConfigMode_10(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_preConfigMode_10, T_preConfigMode_10_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t T_preconfiguration_10_sequence[] = {
  { &hf_rrc_preConfigMode_10, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_preConfigMode_10 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_preconfiguration_10(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_preconfiguration_10, T_preconfiguration_10_sequence);

  return offset;
}


static const value_string rrc_T_specificationMode_10_vals[] = {
  {   0, "complete" },
  {   1, "preconfiguration" },
  { 0, NULL }
};

static const per_choice_t T_specificationMode_10_choice[] = {
  {   0, &hf_rrc_complete_10     , ASN1_NO_EXTENSIONS     , dissect_rrc_T_complete_10 },
  {   1, &hf_rrc_preconfiguration_10, ASN1_NO_EXTENSIONS     , dissect_rrc_T_preconfiguration_10 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_specificationMode_10(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_specificationMode_10, T_specificationMode_10_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t RadioBearerReconfiguration_r6_IEs_sequence[] = {
  { &hf_rrc_integrityProtectionModeInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_IntegrityProtectionModeInfo },
  { &hf_rrc_cipheringModeInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CipheringModeInfo },
  { &hf_rrc_activationTime  , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_ActivationTime },
  { &hf_rrc_delayRestrictionFlag, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DelayRestrictionFlag },
  { &hf_rrc_new_U_RNTI      , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_U_RNTI },
  { &hf_rrc_new_C_RNTI      , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_C_RNTI },
  { &hf_rrc_new_DSCH_RNTI   , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DSCH_RNTI },
  { &hf_rrc_new_H_RNTI      , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_H_RNTI },
  { &hf_rrc_newPrimary_E_RNTI, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_E_RNTI },
  { &hf_rrc_newSecondary_E_RNTI, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_E_RNTI },
  { &hf_rrc_rrc_StateIndicator, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RRC_StateIndicator },
  { &hf_rrc_utran_DRX_CycleLengthCoeff, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UTRAN_DRX_CycleLengthCoefficient },
  { &hf_rrc_cn_InformationInfo_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CN_InformationInfo_r6 },
  { &hf_rrc_ura_Identity    , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_URA_Identity },
  { &hf_rrc_specificationMode_10, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_specificationMode_10 },
  { &hf_rrc_frequencyInfo   , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_FrequencyInfo },
  { &hf_rrc_maxAllowedUL_TX_Power, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MaxAllowedUL_TX_Power },
  { &hf_rrc_ul_DPCH_Info    , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_DPCH_Info_r6 },
  { &hf_rrc_ul_EDCH_Information, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_EDCH_Information_r6 },
  { &hf_rrc_dl_HSPDSCH_Information_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_HSPDSCH_Information_r6 },
  { &hf_rrc_dl_CommonInformation_03, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_CommonInformation_r6 },
  { &hf_rrc_dl_InformationPerRL_List_03, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_InformationPerRL_List_r6 },
  { &hf_rrc_mbms_PL_ServiceRestrictInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MBMS_PL_ServiceRestrictInfo_r6 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_RadioBearerReconfiguration_r6_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_RadioBearerReconfiguration_r6_IEs, RadioBearerReconfiguration_r6_IEs_sequence);

  return offset;
}


static const per_sequence_t RadioBearerReconfiguration_v6b0ext_IEs_sequence[] = {
  { &hf_rrc_dl_InformationPerRL_List_v6b0ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_InformationPerRL_List_v6b0ext },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_RadioBearerReconfiguration_v6b0ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_RadioBearerReconfiguration_v6b0ext_IEs, RadioBearerReconfiguration_v6b0ext_IEs_sequence);

  return offset;
}


static const per_sequence_t RAB_InformationMBMSPtp_sequence[] = {
  { &hf_rrc_rb_Identity     , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RB_Identity },
  { &hf_rrc_mbms_ServiceIdentity, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_OCTET_STRING_SIZE_3 },
  { &hf_rrc_mbms_SessionIdentity, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MBMS_SessionIdentity },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_RAB_InformationMBMSPtp(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_RAB_InformationMBMSPtp, RAB_InformationMBMSPtp_sequence);

  return offset;
}


static const per_sequence_t RAB_InformationMBMSPtpList_sequence_of[1] = {
  { &hf_rrc_RAB_InformationMBMSPtpList_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RAB_InformationMBMSPtp },
};

static int
dissect_rrc_RAB_InformationMBMSPtpList(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_RAB_InformationMBMSPtpList, RAB_InformationMBMSPtpList_sequence_of,
                                                  1, maxMBMSservSelect, FALSE);

  return offset;
}


static const per_sequence_t T_complete_11_sequence[] = {
  { &hf_rrc_rab_InformationMBMSPtpList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RAB_InformationMBMSPtpList },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_complete_11(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_complete_11, T_complete_11_sequence);

  return offset;
}


static const value_string rrc_T_specificationMode_11_vals[] = {
  {   0, "complete" },
  { 0, NULL }
};

static const per_choice_t T_specificationMode_11_choice[] = {
  {   0, &hf_rrc_complete_11     , ASN1_NO_EXTENSIONS     , dissect_rrc_T_complete_11 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_specificationMode_11(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_specificationMode_11, T_specificationMode_11_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t RadioBearerReconfiguration_v6f0ext_IEs_sequence[] = {
  { &hf_rrc_specificationMode_11, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_specificationMode_11 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_RadioBearerReconfiguration_v6f0ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_RadioBearerReconfiguration_v6f0ext_IEs, RadioBearerReconfiguration_v6f0ext_IEs_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions_87_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_nonCriticalExtensions_87(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_nonCriticalExtensions_87, T_nonCriticalExtensions_87_sequence);

  return offset;
}


static const per_sequence_t T_v770NonCriticalExtensions_13_sequence[] = {
  { &hf_rrc_radioBearerReconfiguration_v770ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RadioBearerReconfiguration_v770ext_IEs },
  { &hf_rrc_nonCriticalExtensions_87, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_nonCriticalExtensions_87 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v770NonCriticalExtensions_13(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v770NonCriticalExtensions_13, T_v770NonCriticalExtensions_13_sequence);

  return offset;
}


static const per_sequence_t T_v6f0NonCriticalExtensions_sequence[] = {
  { &hf_rrc_radioBearerRconfiguration_v6f0ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RadioBearerReconfiguration_v6f0ext_IEs },
  { &hf_rrc_v770NonCriticalExtensions_13, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v770NonCriticalExtensions_13 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v6f0NonCriticalExtensions(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v6f0NonCriticalExtensions, T_v6f0NonCriticalExtensions_sequence);

  return offset;
}


static const per_sequence_t T_v6b0NonCriticalExtensions_06_sequence[] = {
  { &hf_rrc_radioBearerReconfiguration_v6b0ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RadioBearerReconfiguration_v6b0ext_IEs },
  { &hf_rrc_v6f0NonCriticalExtensions, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v6f0NonCriticalExtensions },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v6b0NonCriticalExtensions_06(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v6b0NonCriticalExtensions_06, T_v6b0NonCriticalExtensions_06_sequence);

  return offset;
}


static const per_sequence_t T_r6_07_sequence[] = {
  { &hf_rrc_radioBearerReconfiguration_r6, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RadioBearerReconfiguration_r6_IEs },
  { &hf_rrc_radioBearerReconfiguration_r6_add_ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_BIT_STRING },
  { &hf_rrc_v6b0NonCriticalExtensions_07, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v6b0NonCriticalExtensions_06 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_r6_07(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_r6_07, T_r6_07_sequence);

  return offset;
}


static const value_string rrc_T_responseToChangeOfUE_Capability_05_vals[] = {
  {   0, "true" },
  { 0, NULL }
};


static int
dissect_rrc_T_responseToChangeOfUE_Capability_05(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t T_complete_12_sequence[] = {
  { &hf_rrc_rab_InformationReconfigList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RAB_InformationReconfigList },
  { &hf_rrc_rab_InformationMBMSPtpList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RAB_InformationMBMSPtpList },
  { &hf_rrc_rb_InformationReconfigList_04, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RB_InformationReconfigList_r7 },
  { &hf_rrc_rb_InformationAffectedList_03, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RB_InformationAffectedList_r7 },
  { &hf_rrc_rb_PDCPContextRelocationList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RB_PDCPContextRelocationList },
  { &hf_rrc_pdcp_ROHC_TargetMode, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_PDCP_ROHC_TargetMode },
  { &hf_rrc_ul_CommonTransChInfo_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_CommonTransChInfo_r4 },
  { &hf_rrc_ul_deletedTransChInfoList_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_DeletedTransChInfoList_r6 },
  { &hf_rrc_ul_AddReconfTransChInfoList_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_AddReconfTransChInfoList_r7 },
  { &hf_rrc_dl_CommonTransChInfo_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_CommonTransChInfo_r4 },
  { &hf_rrc_dl_DeletedTransChInfoList_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_DeletedTransChInfoList_r7 },
  { &hf_rrc_dl_AddReconfTransChInfoList_03, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_AddReconfTransChInfoList_r7 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_complete_12(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_complete_12, T_complete_12_sequence);

  return offset;
}


static const per_sequence_t T_defaultConfig_11_sequence[] = {
  { &hf_rrc_defaultConfigMode, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DefaultConfigMode },
  { &hf_rrc_defaultConfigIdentity_03, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DefaultConfigIdentity_r6 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_defaultConfig_11(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_defaultConfig_11, T_defaultConfig_11_sequence);

  return offset;
}


static const value_string rrc_T_preConfigMode_11_vals[] = {
  {   0, "predefinedConfigIdentity" },
  {   1, "defaultConfig" },
  { 0, NULL }
};

static const per_choice_t T_preConfigMode_11_choice[] = {
  {   0, &hf_rrc_predefinedConfigIdentity, ASN1_NO_EXTENSIONS     , dissect_rrc_PredefinedConfigIdentity },
  {   1, &hf_rrc_defaultConfig_11, ASN1_NO_EXTENSIONS     , dissect_rrc_T_defaultConfig_11 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_preConfigMode_11(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_preConfigMode_11, T_preConfigMode_11_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t T_preconfiguration_11_sequence[] = {
  { &hf_rrc_preConfigMode_11, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_preConfigMode_11 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_preconfiguration_11(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_preconfiguration_11, T_preconfiguration_11_sequence);

  return offset;
}


static const value_string rrc_T_specificationMode_12_vals[] = {
  {   0, "complete" },
  {   1, "preconfiguration" },
  { 0, NULL }
};

static const per_choice_t T_specificationMode_12_choice[] = {
  {   0, &hf_rrc_complete_12     , ASN1_NO_EXTENSIONS     , dissect_rrc_T_complete_12 },
  {   1, &hf_rrc_preconfiguration_11, ASN1_NO_EXTENSIONS     , dissect_rrc_T_preconfiguration_11 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_specificationMode_12(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_specificationMode_12, T_specificationMode_12_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t RadioBearerReconfiguration_r7_IEs_sequence[] = {
  { &hf_rrc_integrityProtectionModeInfo_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_IntegrityProtectionModeInfo_r7 },
  { &hf_rrc_cipheringModeInfo_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CipheringModeInfo_r7 },
  { &hf_rrc_activationTime  , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_ActivationTime },
  { &hf_rrc_delayRestrictionFlag, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DelayRestrictionFlag },
  { &hf_rrc_new_U_RNTI      , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_U_RNTI },
  { &hf_rrc_new_C_RNTI      , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_C_RNTI },
  { &hf_rrc_new_DSCH_RNTI   , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DSCH_RNTI },
  { &hf_rrc_new_H_RNTI      , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_H_RNTI },
  { &hf_rrc_newPrimary_E_RNTI, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_E_RNTI },
  { &hf_rrc_newSecondary_E_RNTI, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_E_RNTI },
  { &hf_rrc_rrc_StateIndicator, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RRC_StateIndicator },
  { &hf_rrc_ueMobilityStateIndicator, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_High_MobilityDetected },
  { &hf_rrc_utran_DRX_CycleLengthCoeff_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UTRAN_DRX_CycleLengthCoefficient_r7 },
  { &hf_rrc_cn_InformationInfo_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CN_InformationInfo_r6 },
  { &hf_rrc_ura_Identity    , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_URA_Identity },
  { &hf_rrc_supportForChangeOfUE_Capability, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_BOOLEAN },
  { &hf_rrc_responseToChangeOfUE_Capability_05, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_responseToChangeOfUE_Capability_05 },
  { &hf_rrc_specificationMode_12, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_specificationMode_12 },
  { &hf_rrc_frequencyInfo   , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_FrequencyInfo },
  { &hf_rrc_multi_frequencyInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_Multi_frequencyInfo_LCR_r7 },
  { &hf_rrc_dtx_drx_TimingInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DTX_DRX_TimingInfo_r7 },
  { &hf_rrc_dtx_drx_Info    , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DTX_DRX_Info_r7 },
  { &hf_rrc_hs_scch_LessInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_HS_SCCH_LessInfo_r7 },
  { &hf_rrc_mimoParameters  , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MIMO_Parameters_r7 },
  { &hf_rrc_maxAllowedUL_TX_Power, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MaxAllowedUL_TX_Power },
  { &hf_rrc_ul_DPCH_Info_01 , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_DPCH_Info_r7 },
  { &hf_rrc_ul_EDCH_Information_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_EDCH_Information_r7 },
  { &hf_rrc_dl_HSPDSCH_Information_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_HSPDSCH_Information_r7 },
  { &hf_rrc_dl_CommonInformation_04, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_CommonInformation_r7 },
  { &hf_rrc_dl_InformationPerRL_List_04, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_InformationPerRL_List_r7 },
  { &hf_rrc_mbms_PL_ServiceRestrictInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MBMS_PL_ServiceRestrictInfo_r6 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_RadioBearerReconfiguration_r7_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_RadioBearerReconfiguration_r7_IEs, RadioBearerReconfiguration_r7_IEs_sequence);

  return offset;
}


static const per_sequence_t RadioBearerReconfiguration_v780ext_IEs_sequence[] = {
  { &hf_rrc_hs_DSCH_TBSizeTable, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_HS_DSCH_TBSizeTable },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_RadioBearerReconfiguration_v780ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_RadioBearerReconfiguration_v780ext_IEs, RadioBearerReconfiguration_v780ext_IEs_sequence);

  return offset;
}


static const per_sequence_t RadioBearerReconfiguration_v790ext_IEs_sequence[] = {
  { &hf_rrc_radioBearerReconfiguration_r7_add_ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_BIT_STRING },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_RadioBearerReconfiguration_v790ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_RadioBearerReconfiguration_v790ext_IEs, RadioBearerReconfiguration_v790ext_IEs_sequence);

  return offset;
}


static const per_sequence_t RadioBearerReconfiguration_v7d0ext_IEs_sequence[] = {
  { &hf_rrc_ul_EDCH_Information_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_EDCH_Information_ext },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_RadioBearerReconfiguration_v7d0ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_RadioBearerReconfiguration_v7d0ext_IEs, RadioBearerReconfiguration_v7d0ext_IEs_sequence);

  return offset;
}


static const per_sequence_t RadioBearerReconfiguration_v7f0ext_IEs_sequence[] = {
  { &hf_rrc_mimoParameters_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MIMO_Parameters_v7f0ext },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_RadioBearerReconfiguration_v7f0ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_RadioBearerReconfiguration_v7f0ext_IEs, RadioBearerReconfiguration_v7f0ext_IEs_sequence);

  return offset;
}


static const per_sequence_t RadioBearerReconfiguration_v7g0ext_IEs_sequence[] = {
  { &hf_rrc_mimoParameters_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MIMO_Parameters_v7g0ext },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_RadioBearerReconfiguration_v7g0ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_RadioBearerReconfiguration_v7g0ext_IEs, RadioBearerReconfiguration_v7g0ext_IEs_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions_88_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_nonCriticalExtensions_88(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_nonCriticalExtensions_88, T_nonCriticalExtensions_88_sequence);

  return offset;
}


static const per_sequence_t T_v7g0NonCriticalExtensions_08_sequence[] = {
  { &hf_rrc_radioBearerReconfiguration_v7g0ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RadioBearerReconfiguration_v7g0ext_IEs },
  { &hf_rrc_nonCriticalExtensions_88, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_nonCriticalExtensions_88 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v7g0NonCriticalExtensions_08(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v7g0NonCriticalExtensions_08, T_v7g0NonCriticalExtensions_08_sequence);

  return offset;
}


static const per_sequence_t T_v7f0NonCriticalExtensions_07_sequence[] = {
  { &hf_rrc_radioBearerReconfiguration_v7f0ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RadioBearerReconfiguration_v7f0ext_IEs },
  { &hf_rrc_v7g0NonCriticalExtensions_08, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v7g0NonCriticalExtensions_08 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v7f0NonCriticalExtensions_07(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v7f0NonCriticalExtensions_07, T_v7f0NonCriticalExtensions_07_sequence);

  return offset;
}


static const per_sequence_t T_v7d0NonCriticalExtensions_06_sequence[] = {
  { &hf_rrc_radioBearerReconfiguration_v7d0ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RadioBearerReconfiguration_v7d0ext_IEs },
  { &hf_rrc_v7f0NonCriticalExtensions_07, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v7f0NonCriticalExtensions_07 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v7d0NonCriticalExtensions_06(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v7d0NonCriticalExtensions_06, T_v7d0NonCriticalExtensions_06_sequence);

  return offset;
}


static const per_sequence_t T_v790NonCriticalExtensions_sequence[] = {
  { &hf_rrc_radioBearerReconfiguration_v790ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RadioBearerReconfiguration_v790ext_IEs },
  { &hf_rrc_v7d0NonCriticalExtensions_06, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v7d0NonCriticalExtensions_06 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v790NonCriticalExtensions(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v790NonCriticalExtensions, T_v790NonCriticalExtensions_sequence);

  return offset;
}


static const per_sequence_t T_v780NonCriticalExtensions_05_sequence[] = {
  { &hf_rrc_radioBearerReconfiguration_v780ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RadioBearerReconfiguration_v780ext_IEs },
  { &hf_rrc_v790NonCriticalExtensions, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v790NonCriticalExtensions },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v780NonCriticalExtensions_05(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v780NonCriticalExtensions_05, T_v780NonCriticalExtensions_05_sequence);

  return offset;
}


static const per_sequence_t T_r7_06_sequence[] = {
  { &hf_rrc_radioBearerReconfiguration_r7, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RadioBearerReconfiguration_r7_IEs },
  { &hf_rrc_v780NonCriticalExtensions_05, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v780NonCriticalExtensions_05 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_r7_06(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_r7_06, T_r7_06_sequence);

  return offset;
}


static const value_string rrc_T_responseToChangeOfUE_Capability_06_vals[] = {
  {   0, "true" },
  { 0, NULL }
};


static int
dissect_rrc_T_responseToChangeOfUE_Capability_06(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t RAB_InformationReconfig_r8_sequence[] = {
  { &hf_rrc_rab_Identity    , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RAB_Identity },
  { &hf_rrc_cn_DomainIdentity, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CN_DomainIdentity },
  { &hf_rrc_nas_Synchronisation_Indicator, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_NAS_Synchronisation_Indicator },
  { &hf_rrc_cs_HSPA_Information, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CS_HSPA_Information },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_RAB_InformationReconfig_r8(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_RAB_InformationReconfig_r8, RAB_InformationReconfig_r8_sequence);

  return offset;
}


static const per_sequence_t RAB_InformationReconfigList_r8_sequence_of[1] = {
  { &hf_rrc_RAB_InformationReconfigList_r8_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RAB_InformationReconfig_r8 },
};

static int
dissect_rrc_RAB_InformationReconfigList_r8(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_RAB_InformationReconfigList_r8, RAB_InformationReconfigList_r8_sequence_of,
                                                  1, maxRABsetup, FALSE);

  return offset;
}


static const per_sequence_t T_complete_13_sequence[] = {
  { &hf_rrc_rab_InformationReconfigList_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RAB_InformationReconfigList_r8 },
  { &hf_rrc_rab_InformationMBMSPtpList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RAB_InformationMBMSPtpList },
  { &hf_rrc_rb_InformationReconfigList_05, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RB_InformationReconfigList_r8 },
  { &hf_rrc_rb_InformationAffectedList_04, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RB_InformationAffectedList_r8 },
  { &hf_rrc_rb_PDCPContextRelocationList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RB_PDCPContextRelocationList },
  { &hf_rrc_pdcp_ROHC_TargetMode, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_PDCP_ROHC_TargetMode },
  { &hf_rrc_ul_CommonTransChInfo_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_CommonTransChInfo_r4 },
  { &hf_rrc_ul_deletedTransChInfoList_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_DeletedTransChInfoList_r6 },
  { &hf_rrc_ul_AddReconfTransChInfoList_03, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_AddReconfTransChInfoList_r8 },
  { &hf_rrc_dl_CommonTransChInfo_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_CommonTransChInfo_r4 },
  { &hf_rrc_dl_DeletedTransChInfoList_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_DeletedTransChInfoList_r7 },
  { &hf_rrc_dl_AddReconfTransChInfoList_03, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_AddReconfTransChInfoList_r7 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_complete_13(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_complete_13, T_complete_13_sequence);

  return offset;
}


static const per_sequence_t T_defaultConfig_12_sequence[] = {
  { &hf_rrc_defaultConfigMode, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DefaultConfigMode },
  { &hf_rrc_defaultConfigIdentity_03, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DefaultConfigIdentity_r6 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_defaultConfig_12(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_defaultConfig_12, T_defaultConfig_12_sequence);

  return offset;
}


static const value_string rrc_T_preConfigMode_12_vals[] = {
  {   0, "predefinedConfigIdentity" },
  {   1, "defaultConfig" },
  { 0, NULL }
};

static const per_choice_t T_preConfigMode_12_choice[] = {
  {   0, &hf_rrc_predefinedConfigIdentity, ASN1_NO_EXTENSIONS     , dissect_rrc_PredefinedConfigIdentity },
  {   1, &hf_rrc_defaultConfig_12, ASN1_NO_EXTENSIONS     , dissect_rrc_T_defaultConfig_12 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_preConfigMode_12(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_preConfigMode_12, T_preConfigMode_12_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t T_preconfiguration_12_sequence[] = {
  { &hf_rrc_preConfigMode_12, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_preConfigMode_12 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_preconfiguration_12(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_preconfiguration_12, T_preconfiguration_12_sequence);

  return offset;
}


static const value_string rrc_T_specificationMode_13_vals[] = {
  {   0, "complete" },
  {   1, "preconfiguration" },
  { 0, NULL }
};

static const per_choice_t T_specificationMode_13_choice[] = {
  {   0, &hf_rrc_complete_13     , ASN1_NO_EXTENSIONS     , dissect_rrc_T_complete_13 },
  {   1, &hf_rrc_preconfiguration_12, ASN1_NO_EXTENSIONS     , dissect_rrc_T_preconfiguration_12 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_specificationMode_13(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_specificationMode_13, T_specificationMode_13_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t RadioBearerReconfiguration_r8_IEs_sequence[] = {
  { &hf_rrc_integrityProtectionModeInfo_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_IntegrityProtectionModeInfo_r7 },
  { &hf_rrc_cipheringModeInfo_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CipheringModeInfo_r7 },
  { &hf_rrc_activationTime  , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_ActivationTime },
  { &hf_rrc_delayRestrictionFlag, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DelayRestrictionFlag },
  { &hf_rrc_new_U_RNTI      , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_U_RNTI },
  { &hf_rrc_new_C_RNTI      , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_C_RNTI },
  { &hf_rrc_new_DSCH_RNTI   , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DSCH_RNTI },
  { &hf_rrc_new_H_RNTI      , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_H_RNTI },
  { &hf_rrc_newPrimary_E_RNTI, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_E_RNTI },
  { &hf_rrc_newSecondary_E_RNTI, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_E_RNTI },
  { &hf_rrc_rrc_StateIndicator, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RRC_StateIndicator },
  { &hf_rrc_ueMobilityStateIndicator, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_High_MobilityDetected },
  { &hf_rrc_utran_DRX_CycleLengthCoeff_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UTRAN_DRX_CycleLengthCoefficient_r7 },
  { &hf_rrc_cn_InformationInfo_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CN_InformationInfo_r6 },
  { &hf_rrc_ura_Identity    , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_URA_Identity },
  { &hf_rrc_supportForChangeOfUE_Capability, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_BOOLEAN },
  { &hf_rrc_responseToChangeOfUE_Capability_06, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_responseToChangeOfUE_Capability_06 },
  { &hf_rrc_dummy_07        , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DefaultConfigForCellFACH },
  { &hf_rrc_specificationMode_13, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_specificationMode_13 },
  { &hf_rrc_frequencyInfo   , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_FrequencyInfo },
  { &hf_rrc_multi_frequencyInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_Multi_frequencyInfo_LCR_r7 },
  { &hf_rrc_dtx_drx_TimingInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DTX_DRX_TimingInfo_r7 },
  { &hf_rrc_dtx_drx_Info    , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DTX_DRX_Info_r7 },
  { &hf_rrc_hs_scch_LessInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_HS_SCCH_LessInfo_r7 },
  { &hf_rrc_mimoParameters_03, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MIMO_Parameters_r8 },
  { &hf_rrc_maxAllowedUL_TX_Power, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MaxAllowedUL_TX_Power },
  { &hf_rrc_ul_DPCH_Info_01 , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_DPCH_Info_r7 },
  { &hf_rrc_ul_EDCH_Information_03, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_EDCH_Information_r8 },
  { &hf_rrc_dl_HSPDSCH_Information_03, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_HSPDSCH_Information_r8 },
  { &hf_rrc_dl_CommonInformation_05, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_CommonInformation_r8 },
  { &hf_rrc_dl_InformationPerRL_List_05, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_InformationPerRL_List_r8 },
  { &hf_rrc_dl_SecondaryCellInfoFDD, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_SecondaryCellInfoFDD },
  { &hf_rrc_controlChannelDRXInfo_TDD128, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_ControlChannelDRXInfo_TDD128_r8 },
  { &hf_rrc_sps_Information_TDD128, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_SPS_Information_TDD128_r8 },
  { &hf_rrc_mbms_PL_ServiceRestrictInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MBMS_PL_ServiceRestrictInfo_r6 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_RadioBearerReconfiguration_r8_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_RadioBearerReconfiguration_r8_IEs, RadioBearerReconfiguration_r8_IEs_sequence);

  return offset;
}


static const per_sequence_t RadioBearerReconfiguration_v890ext_IEs_sequence[] = {
  { &hf_rrc_dl_SecondaryCellInfoFDD_v890ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_SecondaryCellInfoFDD_v890ext },
  { &hf_rrc_dl_HSPDSCH_Information_04, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_HSPDSCH_Information_r8_ext },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_RadioBearerReconfiguration_v890ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_RadioBearerReconfiguration_v890ext_IEs, RadioBearerReconfiguration_v890ext_IEs_sequence);

  return offset;
}


static const per_sequence_t RadioBearerReconfiguration_v8a0ext_IEs_sequence[] = {
  { &hf_rrc_dl_HSPDSCH_Information_05, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_HSPDSCH_Information_r8_ext2 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_RadioBearerReconfiguration_v8a0ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_RadioBearerReconfiguration_v8a0ext_IEs, RadioBearerReconfiguration_v8a0ext_IEs_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions_89_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_nonCriticalExtensions_89(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_nonCriticalExtensions_89, T_nonCriticalExtensions_89_sequence);

  return offset;
}


static const per_sequence_t T_v8a0NonCriticalExtensions_05_sequence[] = {
  { &hf_rrc_radioBearerReconfiguration_v8a0ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RadioBearerReconfiguration_v8a0ext_IEs },
  { &hf_rrc_nonCriticalExtensions_89, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_nonCriticalExtensions_89 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v8a0NonCriticalExtensions_05(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v8a0NonCriticalExtensions_05, T_v8a0NonCriticalExtensions_05_sequence);

  return offset;
}


static const per_sequence_t T_v7g0NonCriticalExtensions_09_sequence[] = {
  { &hf_rrc_radioBearerReconfiguration_v7g0ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RadioBearerReconfiguration_v7g0ext_IEs },
  { &hf_rrc_v8a0NonCriticalExtensions_05, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v8a0NonCriticalExtensions_05 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v7g0NonCriticalExtensions_09(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v7g0NonCriticalExtensions_09, T_v7g0NonCriticalExtensions_09_sequence);

  return offset;
}


static const per_sequence_t T_v890NonCriticalExtensions_05_sequence[] = {
  { &hf_rrc_radioBearerReconfiguration_v890ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RadioBearerReconfiguration_v890ext_IEs },
  { &hf_rrc_v7g0NonCriticalExtensions_09, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v7g0NonCriticalExtensions_09 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v890NonCriticalExtensions_05(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v890NonCriticalExtensions_05, T_v890NonCriticalExtensions_05_sequence);

  return offset;
}


static const per_sequence_t T_v7f0NonCriticalExtensions_08_sequence[] = {
  { &hf_rrc_radioBearerReconfiguration_v7f0ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RadioBearerReconfiguration_v7f0ext_IEs },
  { &hf_rrc_v890NonCriticalExtensions_05, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v890NonCriticalExtensions_05 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v7f0NonCriticalExtensions_08(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v7f0NonCriticalExtensions_08, T_v7f0NonCriticalExtensions_08_sequence);

  return offset;
}


static const per_sequence_t T_v7d0NonCriticalExtensions_07_sequence[] = {
  { &hf_rrc_radioBearerReconfiguration_v7d0ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RadioBearerReconfiguration_v7d0ext_IEs },
  { &hf_rrc_v7f0NonCriticalExtensions_08, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v7f0NonCriticalExtensions_08 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v7d0NonCriticalExtensions_07(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v7d0NonCriticalExtensions_07, T_v7d0NonCriticalExtensions_07_sequence);

  return offset;
}


static const per_sequence_t T_r8_07_sequence[] = {
  { &hf_rrc_radioBearerReconfiguration_r8, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RadioBearerReconfiguration_r8_IEs },
  { &hf_rrc_radioBearerReconfiguration_r8_add_ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_BIT_STRING },
  { &hf_rrc_v7d0NonCriticalExtensions_07, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v7d0NonCriticalExtensions_07 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_r8_07(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_r8_07, T_r8_07_sequence);

  return offset;
}


static const value_string rrc_T_responseToChangeOfUE_Capability_07_vals[] = {
  {   0, "true" },
  { 0, NULL }
};


static int
dissect_rrc_T_responseToChangeOfUE_Capability_07(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t T_complete_14_sequence[] = {
  { &hf_rrc_rab_InformationReconfigList_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RAB_InformationReconfigList_r8 },
  { &hf_rrc_rab_InformationMBMSPtpList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RAB_InformationMBMSPtpList },
  { &hf_rrc_rb_InformationReconfigList_05, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RB_InformationReconfigList_r8 },
  { &hf_rrc_rb_InformationAffectedList_04, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RB_InformationAffectedList_r8 },
  { &hf_rrc_rb_PDCPContextRelocationList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RB_PDCPContextRelocationList },
  { &hf_rrc_pdcp_ROHC_TargetMode, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_PDCP_ROHC_TargetMode },
  { &hf_rrc_ul_CommonTransChInfo_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_CommonTransChInfo_r4 },
  { &hf_rrc_ul_deletedTransChInfoList_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_DeletedTransChInfoList_r6 },
  { &hf_rrc_ul_AddReconfTransChInfoList_03, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_AddReconfTransChInfoList_r8 },
  { &hf_rrc_dl_CommonTransChInfo_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_CommonTransChInfo_r4 },
  { &hf_rrc_dl_DeletedTransChInfoList_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_DeletedTransChInfoList_r7 },
  { &hf_rrc_dl_AddReconfTransChInfoList_04, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_AddReconfTransChInfoList_r9 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_complete_14(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_complete_14, T_complete_14_sequence);

  return offset;
}


static const per_sequence_t T_defaultConfig_13_sequence[] = {
  { &hf_rrc_defaultConfigMode, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DefaultConfigMode },
  { &hf_rrc_defaultConfigIdentity_03, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DefaultConfigIdentity_r6 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_defaultConfig_13(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_defaultConfig_13, T_defaultConfig_13_sequence);

  return offset;
}


static const value_string rrc_T_preConfigMode_13_vals[] = {
  {   0, "predefinedConfigIdentity" },
  {   1, "defaultConfig" },
  { 0, NULL }
};

static const per_choice_t T_preConfigMode_13_choice[] = {
  {   0, &hf_rrc_predefinedConfigIdentity, ASN1_NO_EXTENSIONS     , dissect_rrc_PredefinedConfigIdentity },
  {   1, &hf_rrc_defaultConfig_13, ASN1_NO_EXTENSIONS     , dissect_rrc_T_defaultConfig_13 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_preConfigMode_13(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_preConfigMode_13, T_preConfigMode_13_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t T_preconfiguration_13_sequence[] = {
  { &hf_rrc_preConfigMode_13, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_preConfigMode_13 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_preconfiguration_13(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_preconfiguration_13, T_preconfiguration_13_sequence);

  return offset;
}


static const value_string rrc_T_specificationMode_14_vals[] = {
  {   0, "complete" },
  {   1, "preconfiguration" },
  { 0, NULL }
};

static const per_choice_t T_specificationMode_14_choice[] = {
  {   0, &hf_rrc_complete_14     , ASN1_NO_EXTENSIONS     , dissect_rrc_T_complete_14 },
  {   1, &hf_rrc_preconfiguration_13, ASN1_NO_EXTENSIONS     , dissect_rrc_T_preconfiguration_13 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_specificationMode_14(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_specificationMode_14, T_specificationMode_14_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t RadioBearerReconfiguration_r9_IEs_sequence[] = {
  { &hf_rrc_integrityProtectionModeInfo_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_IntegrityProtectionModeInfo_r7 },
  { &hf_rrc_cipheringModeInfo_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CipheringModeInfo_r7 },
  { &hf_rrc_activationTime  , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_ActivationTime },
  { &hf_rrc_delayRestrictionFlag, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DelayRestrictionFlag },
  { &hf_rrc_new_U_RNTI      , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_U_RNTI },
  { &hf_rrc_new_C_RNTI      , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_C_RNTI },
  { &hf_rrc_new_DSCH_RNTI   , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DSCH_RNTI },
  { &hf_rrc_new_H_RNTI      , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_H_RNTI },
  { &hf_rrc_newPrimary_E_RNTI, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_E_RNTI },
  { &hf_rrc_newSecondary_E_RNTI, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_E_RNTI },
  { &hf_rrc_rrc_StateIndicator, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RRC_StateIndicator },
  { &hf_rrc_ueMobilityStateIndicator, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_High_MobilityDetected },
  { &hf_rrc_utran_DRX_CycleLengthCoeff_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UTRAN_DRX_CycleLengthCoefficient_r7 },
  { &hf_rrc_cn_InformationInfo_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CN_InformationInfo_r6 },
  { &hf_rrc_ura_Identity    , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_URA_Identity },
  { &hf_rrc_supportForChangeOfUE_Capability, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_BOOLEAN },
  { &hf_rrc_responseToChangeOfUE_Capability_07, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_responseToChangeOfUE_Capability_07 },
  { &hf_rrc_defaultConfigForCellFACH, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DefaultConfigForCellFACH },
  { &hf_rrc_specificationMode_14, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_specificationMode_14 },
  { &hf_rrc_frequencyInfo   , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_FrequencyInfo },
  { &hf_rrc_multi_frequencyInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_Multi_frequencyInfo_LCR_r7 },
  { &hf_rrc_dtx_drx_TimingInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DTX_DRX_TimingInfo_r7 },
  { &hf_rrc_dtx_drx_Info    , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DTX_DRX_Info_r7 },
  { &hf_rrc_hs_scch_LessInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_HS_SCCH_LessInfo_r7 },
  { &hf_rrc_mimoParameters_04, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MIMO_Parameters_r9 },
  { &hf_rrc_maxAllowedUL_TX_Power, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MaxAllowedUL_TX_Power },
  { &hf_rrc_ul_DPCH_Info_01 , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_DPCH_Info_r7 },
  { &hf_rrc_ul_EDCH_Information_04, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_EDCH_Information_r9 },
  { &hf_rrc_ul_SecondaryCellInfoFDD, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_SecondaryCellInfoFDD },
  { &hf_rrc_dl_HSPDSCH_Information_06, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_HSPDSCH_Information_r9 },
  { &hf_rrc_dl_CommonInformation_05, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_CommonInformation_r8 },
  { &hf_rrc_dl_InformationPerRL_List_05, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_InformationPerRL_List_r8 },
  { &hf_rrc_dl_SecondaryCellInfoFDD_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_SecondaryCellInfoFDD_r9 },
  { &hf_rrc_controlChannelDRXInfo_TDD128, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_ControlChannelDRXInfo_TDD128_r8 },
  { &hf_rrc_sps_Information_TDD128, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_SPS_Information_TDD128_r8 },
  { &hf_rrc_mbms_PL_ServiceRestrictInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MBMS_PL_ServiceRestrictInfo_r6 },
  { &hf_rrc_cellDCHMeasOccasionInfo_TDD128, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CellDCHMeasOccasionInfo_TDD128_r9 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_RadioBearerReconfiguration_r9_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_RadioBearerReconfiguration_r9_IEs, RadioBearerReconfiguration_r9_IEs_sequence);

  return offset;
}


static const per_sequence_t RadioBearerReconfiguration_v950ext_IEs_sequence[] = {
  { &hf_rrc_secondaryCellMimoParameters, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_SecondaryCellMIMOparametersFDD_v950ext },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_RadioBearerReconfiguration_v950ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_RadioBearerReconfiguration_v950ext_IEs, RadioBearerReconfiguration_v950ext_IEs_sequence);

  return offset;
}


static const per_sequence_t RadioBearerReconfiguration_v9c0ext_IEs_sequence[] = {
  { &hf_rrc_dl_AddReconfTransChInfoList_05, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_AddReconfTransChInfoList_TDD128_v9c0ext },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_RadioBearerReconfiguration_v9c0ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_RadioBearerReconfiguration_v9c0ext_IEs, RadioBearerReconfiguration_v9c0ext_IEs_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions_90_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_nonCriticalExtensions_90(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_nonCriticalExtensions_90, T_nonCriticalExtensions_90_sequence);

  return offset;
}


static const per_sequence_t T_v9c0NonCriticalExtensions_04_sequence[] = {
  { &hf_rrc_radioBearerReconfiguration_v9c0ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RadioBearerReconfiguration_v9c0ext_IEs },
  { &hf_rrc_nonCriticalExtensions_90, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_nonCriticalExtensions_90 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v9c0NonCriticalExtensions_04(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v9c0NonCriticalExtensions_04, T_v9c0NonCriticalExtensions_04_sequence);

  return offset;
}


static const per_sequence_t T_v950NonCriticalExtensions_04_sequence[] = {
  { &hf_rrc_radioBearerReconfiguration_v950ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RadioBearerReconfiguration_v950ext_IEs },
  { &hf_rrc_v9c0NonCriticalExtensions_04, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v9c0NonCriticalExtensions_04 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v950NonCriticalExtensions_04(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v950NonCriticalExtensions_04, T_v950NonCriticalExtensions_04_sequence);

  return offset;
}


static const per_sequence_t T_r9_06_sequence[] = {
  { &hf_rrc_radioBearerReconfiguration_r9, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RadioBearerReconfiguration_r9_IEs },
  { &hf_rrc_radioBearerReconfiguration_r9_add_ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_BIT_STRING },
  { &hf_rrc_v950NonCriticalExtensions_04, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v950NonCriticalExtensions_04 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_r9_06(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_r9_06, T_r9_06_sequence);

  return offset;
}


static const value_string rrc_T_responseToChangeOfUE_Capability_08_vals[] = {
  {   0, "true" },
  { 0, NULL }
};


static int
dissect_rrc_T_responseToChangeOfUE_Capability_08(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t T_complete_15_sequence[] = {
  { &hf_rrc_rab_InformationReconfigList_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RAB_InformationReconfigList_r8 },
  { &hf_rrc_rab_InformationMBMSPtpList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RAB_InformationMBMSPtpList },
  { &hf_rrc_rb_InformationReconfigList_05, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RB_InformationReconfigList_r8 },
  { &hf_rrc_rb_InformationAffectedList_04, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RB_InformationAffectedList_r8 },
  { &hf_rrc_rb_PDCPContextRelocationList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RB_PDCPContextRelocationList },
  { &hf_rrc_pdcp_ROHC_TargetMode, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_PDCP_ROHC_TargetMode },
  { &hf_rrc_ul_CommonTransChInfo_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_CommonTransChInfo_r4 },
  { &hf_rrc_ul_deletedTransChInfoList_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_DeletedTransChInfoList_r6 },
  { &hf_rrc_ul_AddReconfTransChInfoList_03, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_AddReconfTransChInfoList_r8 },
  { &hf_rrc_dl_CommonTransChInfo_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_CommonTransChInfo_r4 },
  { &hf_rrc_dl_DeletedTransChInfoList_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_DeletedTransChInfoList_r7 },
  { &hf_rrc_dl_AddReconfTransChInfoList_04, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_AddReconfTransChInfoList_r9 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_complete_15(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_complete_15, T_complete_15_sequence);

  return offset;
}


static const per_sequence_t T_defaultConfig_14_sequence[] = {
  { &hf_rrc_defaultConfigMode, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DefaultConfigMode },
  { &hf_rrc_defaultConfigIdentity_03, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DefaultConfigIdentity_r6 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_defaultConfig_14(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_defaultConfig_14, T_defaultConfig_14_sequence);

  return offset;
}


static const value_string rrc_T_preConfigMode_14_vals[] = {
  {   0, "predefinedConfigIdentity" },
  {   1, "defaultConfig" },
  { 0, NULL }
};

static const per_choice_t T_preConfigMode_14_choice[] = {
  {   0, &hf_rrc_predefinedConfigIdentity, ASN1_NO_EXTENSIONS     , dissect_rrc_PredefinedConfigIdentity },
  {   1, &hf_rrc_defaultConfig_14, ASN1_NO_EXTENSIONS     , dissect_rrc_T_defaultConfig_14 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_preConfigMode_14(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_preConfigMode_14, T_preConfigMode_14_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t T_preconfiguration_14_sequence[] = {
  { &hf_rrc_preConfigMode_14, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_preConfigMode_14 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_preconfiguration_14(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_preconfiguration_14, T_preconfiguration_14_sequence);

  return offset;
}


static const value_string rrc_T_specificationMode_15_vals[] = {
  {   0, "complete" },
  {   1, "preconfiguration" },
  { 0, NULL }
};

static const per_choice_t T_specificationMode_15_choice[] = {
  {   0, &hf_rrc_complete_15     , ASN1_NO_EXTENSIONS     , dissect_rrc_T_complete_15 },
  {   1, &hf_rrc_preconfiguration_14, ASN1_NO_EXTENSIONS     , dissect_rrc_T_preconfiguration_14 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_specificationMode_15(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_specificationMode_15, T_specificationMode_15_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t RadioBearerReconfiguration_r10_IEs_sequence[] = {
  { &hf_rrc_integrityProtectionModeInfo_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_IntegrityProtectionModeInfo_r7 },
  { &hf_rrc_cipheringModeInfo_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CipheringModeInfo_r7 },
  { &hf_rrc_activationTime  , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_ActivationTime },
  { &hf_rrc_delayRestrictionFlag, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DelayRestrictionFlag },
  { &hf_rrc_new_U_RNTI      , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_U_RNTI },
  { &hf_rrc_new_C_RNTI      , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_C_RNTI },
  { &hf_rrc_new_DSCH_RNTI   , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DSCH_RNTI },
  { &hf_rrc_new_H_RNTI      , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_H_RNTI },
  { &hf_rrc_newPrimary_E_RNTI, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_E_RNTI },
  { &hf_rrc_newSecondary_E_RNTI, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_E_RNTI },
  { &hf_rrc_rrc_StateIndicator, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RRC_StateIndicator },
  { &hf_rrc_ueMobilityStateIndicator, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_High_MobilityDetected },
  { &hf_rrc_utran_DRX_CycleLengthCoeff_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UTRAN_DRX_CycleLengthCoefficient_r7 },
  { &hf_rrc_cn_InformationInfo_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CN_InformationInfo_r6 },
  { &hf_rrc_ura_Identity    , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_URA_Identity },
  { &hf_rrc_supportForChangeOfUE_Capability, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_BOOLEAN },
  { &hf_rrc_responseToChangeOfUE_Capability_08, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_responseToChangeOfUE_Capability_08 },
  { &hf_rrc_defaultConfigForCellFACH, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DefaultConfigForCellFACH },
  { &hf_rrc_specificationMode_15, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_specificationMode_15 },
  { &hf_rrc_frequencyInfo   , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_FrequencyInfo },
  { &hf_rrc_multi_frequencyInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_Multi_frequencyInfo_LCR_r7 },
  { &hf_rrc_dtx_drx_TimingInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DTX_DRX_TimingInfo_r7 },
  { &hf_rrc_dtx_drx_Info    , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DTX_DRX_Info_r7 },
  { &hf_rrc_hs_scch_LessInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_HS_SCCH_LessInfo_r7 },
  { &hf_rrc_mimoParameters_04, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MIMO_Parameters_r9 },
  { &hf_rrc_maxAllowedUL_TX_Power, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MaxAllowedUL_TX_Power },
  { &hf_rrc_ul_DPCH_Info_01 , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_DPCH_Info_r7 },
  { &hf_rrc_ul_EDCH_Information_04, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_EDCH_Information_r9 },
  { &hf_rrc_ul_SecondaryCellInfoFDD, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_SecondaryCellInfoFDD },
  { &hf_rrc_ul_MulticarrierEDCHInfo_TDD128, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_MulticarrierEDCHInfo_TDD128 },
  { &hf_rrc_dl_HSPDSCH_Information_06, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_HSPDSCH_Information_r9 },
  { &hf_rrc_dl_CommonInformation_06, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_CommonInformation_r10 },
  { &hf_rrc_dl_InformationPerRL_List_05, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_InformationPerRL_List_r8 },
  { &hf_rrc_dl_SecondaryCellInfoFDD_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_SecondaryCellInfoFDD_r10 },
  { &hf_rrc_additionalDLSecCellInfoListFDD, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_AdditionalDLSecCellInfoListFDD },
  { &hf_rrc_controlChannelDRXInfo_TDD128, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_ControlChannelDRXInfo_TDD128_r8 },
  { &hf_rrc_sps_Information_TDD128, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_SPS_Information_TDD128_r8 },
  { &hf_rrc_mu_MIMO_Info_TDD128, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MU_MIMO_Info_TDD128 },
  { &hf_rrc_mbms_PL_ServiceRestrictInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MBMS_PL_ServiceRestrictInfo_r6 },
  { &hf_rrc_cellDCHMeasOccasionInfo_TDD128, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CellDCHMeasOccasionInfo_TDD128_r9 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_RadioBearerReconfiguration_r10_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_RadioBearerReconfiguration_r10_IEs, RadioBearerReconfiguration_r10_IEs_sequence);

  return offset;
}


static const per_sequence_t RB_IdentityList_sequence_of[1] = {
  { &hf_rrc_RB_IdentityList_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RB_Identity },
};

static int
dissect_rrc_RB_IdentityList(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_RB_IdentityList, RB_IdentityList_sequence_of,
                                                  1, maxRB, FALSE);

  return offset;
}


static const per_sequence_t RadioBearerReconfiguration_vaa0ext_IEs_sequence[] = {
  { &hf_rrc_umRlc_ReEstablishmentRBList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RB_IdentityList },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_RadioBearerReconfiguration_vaa0ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_RadioBearerReconfiguration_vaa0ext_IEs, RadioBearerReconfiguration_vaa0ext_IEs_sequence);

  return offset;
}


static const per_sequence_t RadioBearerReconfiguration_vbxyext_IEs_sequence[] = {
  { &hf_rrc_ulOLTDInfoFDD   , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_OLTD_InfoFDD },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_RadioBearerReconfiguration_vbxyext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_RadioBearerReconfiguration_vbxyext_IEs, RadioBearerReconfiguration_vbxyext_IEs_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions_91_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_nonCriticalExtensions_91(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_nonCriticalExtensions_91, T_nonCriticalExtensions_91_sequence);

  return offset;
}


static const per_sequence_t T_vbxyNonCriticalExtensions_09_sequence[] = {
  { &hf_rrc_radioBearerReconfiguration_vbxyext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RadioBearerReconfiguration_vbxyext_IEs },
  { &hf_rrc_nonCriticalExtensions_91, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_nonCriticalExtensions_91 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_vbxyNonCriticalExtensions_09(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_vbxyNonCriticalExtensions_09, T_vbxyNonCriticalExtensions_09_sequence);

  return offset;
}


static const per_sequence_t T_vaa0NonCriticalExtensions_01_sequence[] = {
  { &hf_rrc_radioBearerReconfiguration_vaa0ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RadioBearerReconfiguration_vaa0ext_IEs },
  { &hf_rrc_vbxyNonCriticalExtensions_09, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_vbxyNonCriticalExtensions_09 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_vaa0NonCriticalExtensions_01(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_vaa0NonCriticalExtensions_01, T_vaa0NonCriticalExtensions_01_sequence);

  return offset;
}


static const per_sequence_t T_v9c0NonCriticalExtensions_05_sequence[] = {
  { &hf_rrc_radioBearerReconfiguration_v9c0ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RadioBearerReconfiguration_v9c0ext_IEs },
  { &hf_rrc_vaa0NonCriticalExtensions_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_vaa0NonCriticalExtensions_01 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v9c0NonCriticalExtensions_05(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v9c0NonCriticalExtensions_05, T_v9c0NonCriticalExtensions_05_sequence);

  return offset;
}


static const per_sequence_t T_r10_06_sequence[] = {
  { &hf_rrc_radioBearerReconfiguration_r10, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RadioBearerReconfiguration_r10_IEs },
  { &hf_rrc_radioBearerReconfiguration_r10_add_ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_BIT_STRING },
  { &hf_rrc_v9c0NonCriticalExtensions_05, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v9c0NonCriticalExtensions_05 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_r10_06(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_r10_06, T_r10_06_sequence);

  return offset;
}


static const value_string rrc_T_responseToChangeOfUE_Capability_09_vals[] = {
  {   0, "true" },
  { 0, NULL }
};


static int
dissect_rrc_T_responseToChangeOfUE_Capability_09(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t T_complete_16_sequence[] = {
  { &hf_rrc_rab_InformationReconfigList_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RAB_InformationReconfigList_r8 },
  { &hf_rrc_rab_InformationMBMSPtpList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RAB_InformationMBMSPtpList },
  { &hf_rrc_rb_InformationReconfigList_06, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RB_InformationReconfigList_r11 },
  { &hf_rrc_rb_InformationAffectedList_04, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RB_InformationAffectedList_r8 },
  { &hf_rrc_rb_PDCPContextRelocationList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RB_PDCPContextRelocationList },
  { &hf_rrc_pdcp_ROHC_TargetMode, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_PDCP_ROHC_TargetMode },
  { &hf_rrc_umRlc_ReEstablishmentRBList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RB_IdentityList },
  { &hf_rrc_ul_CommonTransChInfo_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_CommonTransChInfo_r4 },
  { &hf_rrc_ul_deletedTransChInfoList_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_DeletedTransChInfoList_r6 },
  { &hf_rrc_ul_AddReconfTransChInfoList_03, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_AddReconfTransChInfoList_r8 },
  { &hf_rrc_dl_CommonTransChInfo_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_CommonTransChInfo_r4 },
  { &hf_rrc_dl_DeletedTransChInfoList_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_DeletedTransChInfoList_r7 },
  { &hf_rrc_dl_AddReconfTransChInfoList_06, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_AddReconfTransChInfoList_r11 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_complete_16(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_complete_16, T_complete_16_sequence);

  return offset;
}


static const per_sequence_t T_defaultConfig_15_sequence[] = {
  { &hf_rrc_defaultConfigMode, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DefaultConfigMode },
  { &hf_rrc_defaultConfigIdentity_03, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DefaultConfigIdentity_r6 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_defaultConfig_15(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_defaultConfig_15, T_defaultConfig_15_sequence);

  return offset;
}


static const value_string rrc_T_preConfigMode_15_vals[] = {
  {   0, "predefinedConfigIdentity" },
  {   1, "defaultConfig" },
  { 0, NULL }
};

static const per_choice_t T_preConfigMode_15_choice[] = {
  {   0, &hf_rrc_predefinedConfigIdentity, ASN1_NO_EXTENSIONS     , dissect_rrc_PredefinedConfigIdentity },
  {   1, &hf_rrc_defaultConfig_15, ASN1_NO_EXTENSIONS     , dissect_rrc_T_defaultConfig_15 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_preConfigMode_15(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_preConfigMode_15, T_preConfigMode_15_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t T_preconfiguration_15_sequence[] = {
  { &hf_rrc_preConfigMode_15, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_preConfigMode_15 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_preconfiguration_15(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_preconfiguration_15, T_preconfiguration_15_sequence);

  return offset;
}


static const value_string rrc_T_specificationMode_16_vals[] = {
  {   0, "complete" },
  {   1, "preconfiguration" },
  { 0, NULL }
};

static const per_choice_t T_specificationMode_16_choice[] = {
  {   0, &hf_rrc_complete_16     , ASN1_NO_EXTENSIONS     , dissect_rrc_T_complete_16 },
  {   1, &hf_rrc_preconfiguration_15, ASN1_NO_EXTENSIONS     , dissect_rrc_T_preconfiguration_15 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_specificationMode_16(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_specificationMode_16, T_specificationMode_16_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t RadioBearerReconfiguration_r11_IEs_sequence[] = {
  { &hf_rrc_integrityProtectionModeInfo_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_IntegrityProtectionModeInfo_r7 },
  { &hf_rrc_cipheringModeInfo_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CipheringModeInfo_r7 },
  { &hf_rrc_activationTime  , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_ActivationTime },
  { &hf_rrc_delayRestrictionFlag, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DelayRestrictionFlag },
  { &hf_rrc_new_U_RNTI      , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_U_RNTI },
  { &hf_rrc_new_C_RNTI      , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_C_RNTI },
  { &hf_rrc_new_DSCH_RNTI   , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DSCH_RNTI },
  { &hf_rrc_new_H_RNTI      , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_H_RNTI },
  { &hf_rrc_newPrimary_E_RNTI, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_E_RNTI },
  { &hf_rrc_newSecondary_E_RNTI, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_E_RNTI },
  { &hf_rrc_rrc_StateIndicator, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RRC_StateIndicator },
  { &hf_rrc_ueMobilityStateIndicator, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_High_MobilityDetected },
  { &hf_rrc_utran_DRX_CycleLengthCoeff_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UTRAN_DRX_CycleLengthCoefficient_r7 },
  { &hf_rrc_cn_InformationInfo_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CN_InformationInfo_r6 },
  { &hf_rrc_ura_Identity    , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_URA_Identity },
  { &hf_rrc_supportForChangeOfUE_Capability, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_BOOLEAN },
  { &hf_rrc_responseToChangeOfUE_Capability_09, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_responseToChangeOfUE_Capability_09 },
  { &hf_rrc_defaultConfigForCellFACH, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DefaultConfigForCellFACH },
  { &hf_rrc_specificationMode_16, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_specificationMode_16 },
  { &hf_rrc_frequencyInfo   , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_FrequencyInfo },
  { &hf_rrc_multi_frequencyInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_Multi_frequencyInfo_LCR_r7 },
  { &hf_rrc_dtx_drx_TimingInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DTX_DRX_TimingInfo_r7 },
  { &hf_rrc_dtx_drx_Info    , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DTX_DRX_Info_r7 },
  { &hf_rrc_hs_scch_LessInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_HS_SCCH_LessInfo_r7 },
  { &hf_rrc_mimoParameters_04, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MIMO_Parameters_r9 },
  { &hf_rrc_mimo4x4Parameters, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MIMO4x4_Parameters },
  { &hf_rrc_maxAllowedUL_TX_Power, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MaxAllowedUL_TX_Power },
  { &hf_rrc_ul_DPCH_Info_02 , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_DPCH_Info_r11 },
  { &hf_rrc_ul_EDCH_Information_05, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_EDCH_Information_r11 },
  { &hf_rrc_ul_SecondaryCellInfoFDD, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_SecondaryCellInfoFDD },
  { &hf_rrc_ul_MulticarrierEDCHInfo_TDD128, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_MulticarrierEDCHInfo_TDD128 },
  { &hf_rrc_ul_CLTD_InfoFDD , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_CLTD_InfoFDD },
  { &hf_rrc_ul_OLTD_InfoFDD , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_OLTD_InfoFDD },
  { &hf_rrc_dl_HSPDSCH_Information_07, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_HSPDSCH_Information_r11 },
  { &hf_rrc_dl_CommonInformation_07, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_CommonInformation_r11 },
  { &hf_rrc_dl_InformationPerRL_List_06, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_InformationPerRL_List_r11 },
  { &hf_rrc_dl_SecondaryCellInfoFDD_03, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_SecondaryCellInfoFDD_r11 },
  { &hf_rrc_additionalDLSecCellInfoListFDD_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_AdditionalDLSecCellInfoListFDD_r11 },
  { &hf_rrc_additionalDLSecCellInfoListFDD2, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_AdditionalDLSecCellInfoListFDD2_r11 },
  { &hf_rrc_commonERGCHInfoFDD, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CommonERGCHInfoFDD },
  { &hf_rrc_controlChannelDRXInfo_TDD128, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_ControlChannelDRXInfo_TDD128_r8 },
  { &hf_rrc_sps_Information_TDD128, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_SPS_Information_TDD128_r8 },
  { &hf_rrc_mu_MIMO_Info_TDD128, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MU_MIMO_Info_TDD128 },
  { &hf_rrc_mbms_PL_ServiceRestrictInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MBMS_PL_ServiceRestrictInfo_r6 },
  { &hf_rrc_cellDCHMeasOccasionInfo_TDD128, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CellDCHMeasOccasionInfo_TDD128_r9 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_RadioBearerReconfiguration_r11_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_RadioBearerReconfiguration_r11_IEs, RadioBearerReconfiguration_r11_IEs_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions_92_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_nonCriticalExtensions_92(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_nonCriticalExtensions_92, T_nonCriticalExtensions_92_sequence);

  return offset;
}


static const per_sequence_t T_r11_07_sequence[] = {
  { &hf_rrc_radioBearerReconfiguration_r11, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RadioBearerReconfiguration_r11_IEs },
  { &hf_rrc_radioBearerReconfiguration_r11_add_ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_BIT_STRING },
  { &hf_rrc_nonCriticalExtensions_92, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_nonCriticalExtensions_92 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_r11_07(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_r11_07, T_r11_07_sequence);

  return offset;
}


static const per_sequence_t T_criticalExtensions_72_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_criticalExtensions_72(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_criticalExtensions_72, T_criticalExtensions_72_sequence);

  return offset;
}


static const value_string rrc_T_criticalExtensions_71_vals[] = {
  {   0, "r11" },
  {   1, "criticalExtensions" },
  { 0, NULL }
};

static const per_choice_t T_criticalExtensions_71_choice[] = {
  {   0, &hf_rrc_r11_07          , ASN1_NO_EXTENSIONS     , dissect_rrc_T_r11_07 },
  {   1, &hf_rrc_criticalExtensions_73, ASN1_NO_EXTENSIONS     , dissect_rrc_T_criticalExtensions_72 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_criticalExtensions_71(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_criticalExtensions_71, T_criticalExtensions_71_choice,
                                 NULL);

  return offset;
}


static const value_string rrc_T_criticalExtensions_70_vals[] = {
  {   0, "r10" },
  {   1, "criticalExtensions" },
  { 0, NULL }
};

static const per_choice_t T_criticalExtensions_70_choice[] = {
  {   0, &hf_rrc_r10_06          , ASN1_NO_EXTENSIONS     , dissect_rrc_T_r10_06 },
  {   1, &hf_rrc_criticalExtensions_72, ASN1_NO_EXTENSIONS     , dissect_rrc_T_criticalExtensions_71 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_criticalExtensions_70(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_criticalExtensions_70, T_criticalExtensions_70_choice,
                                 NULL);

  return offset;
}


static const value_string rrc_T_criticalExtensions_69_vals[] = {
  {   0, "r9" },
  {   1, "criticalExtensions" },
  { 0, NULL }
};

static const per_choice_t T_criticalExtensions_69_choice[] = {
  {   0, &hf_rrc_r9_06           , ASN1_NO_EXTENSIONS     , dissect_rrc_T_r9_06 },
  {   1, &hf_rrc_criticalExtensions_71, ASN1_NO_EXTENSIONS     , dissect_rrc_T_criticalExtensions_70 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_criticalExtensions_69(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_criticalExtensions_69, T_criticalExtensions_69_choice,
                                 NULL);

  return offset;
}


static const value_string rrc_T_criticalExtensions_68_vals[] = {
  {   0, "r8" },
  {   1, "criticalExtensions" },
  { 0, NULL }
};

static const per_choice_t T_criticalExtensions_68_choice[] = {
  {   0, &hf_rrc_r8_07           , ASN1_NO_EXTENSIONS     , dissect_rrc_T_r8_07 },
  {   1, &hf_rrc_criticalExtensions_70, ASN1_NO_EXTENSIONS     , dissect_rrc_T_criticalExtensions_69 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_criticalExtensions_68(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_criticalExtensions_68, T_criticalExtensions_68_choice,
                                 NULL);

  return offset;
}


static const value_string rrc_T_criticalExtensions_67_vals[] = {
  {   0, "r7" },
  {   1, "criticalExtensions" },
  { 0, NULL }
};

static const per_choice_t T_criticalExtensions_67_choice[] = {
  {   0, &hf_rrc_r7_06           , ASN1_NO_EXTENSIONS     , dissect_rrc_T_r7_06 },
  {   1, &hf_rrc_criticalExtensions_69, ASN1_NO_EXTENSIONS     , dissect_rrc_T_criticalExtensions_68 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_criticalExtensions_67(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_criticalExtensions_67, T_criticalExtensions_67_choice,
                                 NULL);

  return offset;
}


static const value_string rrc_T_criticalExtensions_66_vals[] = {
  {   0, "r6" },
  {   1, "criticalExtensions" },
  { 0, NULL }
};

static const per_choice_t T_criticalExtensions_66_choice[] = {
  {   0, &hf_rrc_r6_07           , ASN1_NO_EXTENSIONS     , dissect_rrc_T_r6_07 },
  {   1, &hf_rrc_criticalExtensions_68, ASN1_NO_EXTENSIONS     , dissect_rrc_T_criticalExtensions_67 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_criticalExtensions_66(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_criticalExtensions_66, T_criticalExtensions_66_choice,
                                 NULL);

  return offset;
}


static const value_string rrc_T_criticalExtensions_65_vals[] = {
  {   0, "r5" },
  {   1, "criticalExtensions" },
  { 0, NULL }
};

static const per_choice_t T_criticalExtensions_65_choice[] = {
  {   0, &hf_rrc_r5_05           , ASN1_NO_EXTENSIONS     , dissect_rrc_T_r5_05 },
  {   1, &hf_rrc_criticalExtensions_67, ASN1_NO_EXTENSIONS     , dissect_rrc_T_criticalExtensions_66 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_criticalExtensions_65(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_criticalExtensions_65, T_criticalExtensions_65_choice,
                                 NULL);

  return offset;
}


static const value_string rrc_T_criticalExtensions_64_vals[] = {
  {   0, "r4" },
  {   1, "criticalExtensions" },
  { 0, NULL }
};

static const per_choice_t T_criticalExtensions_64_choice[] = {
  {   0, &hf_rrc_r4_06           , ASN1_NO_EXTENSIONS     , dissect_rrc_T_r4_06 },
  {   1, &hf_rrc_criticalExtensions_66, ASN1_NO_EXTENSIONS     , dissect_rrc_T_criticalExtensions_65 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_criticalExtensions_64(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_criticalExtensions_64, T_criticalExtensions_64_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t T_later_than_r3_12_sequence[] = {
  { &hf_rrc_rrc_TransactionIdentifier, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RRC_TransactionIdentifier },
  { &hf_rrc_criticalExtensions_65, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_criticalExtensions_64 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_later_than_r3_12(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_later_than_r3_12, T_later_than_r3_12_sequence);

  return offset;
}


static const value_string rrc_RadioBearerReconfiguration_vals[] = {
  {   0, "r3" },
  {   1, "later-than-r3" },
  { 0, NULL }
};

static const per_choice_t RadioBearerReconfiguration_choice[] = {
  {   0, &hf_rrc_r3_14           , ASN1_NO_EXTENSIONS     , dissect_rrc_T_r3_14 },
  {   1, &hf_rrc_later_than_r3_12, ASN1_NO_EXTENSIONS     , dissect_rrc_T_later_than_r3_12 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_RadioBearerReconfiguration(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
#line 325 "../../asn1/rrc/rrc.cnf"
  col_append_str(actx->pinfo->cinfo, COL_INFO, "RadioBearerReconfiguration");
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_RadioBearerReconfiguration, RadioBearerReconfiguration_choice,
                                 NULL);




  return offset;
}


static const per_sequence_t T_fdd_27_sequence[] = {
  { &hf_rrc_dummy1          , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CPCH_SetID },
  { &hf_rrc_dummy2_02       , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DRAC_StaticInformationList },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_fdd_27(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_fdd_27, T_fdd_27_sequence);

  return offset;
}


static const value_string rrc_T_dummy_04_vals[] = {
  {   0, "fdd" },
  {   1, "tdd" },
  { 0, NULL }
};

static const per_choice_t T_dummy_04_choice[] = {
  {   0, &hf_rrc_fdd_27          , ASN1_NO_EXTENSIONS     , dissect_rrc_T_fdd_27 },
  {   1, &hf_rrc_tdd             , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_dummy_04(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_dummy_04, T_dummy_04_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t T_fdd_28_sequence[] = {
  { &hf_rrc_dummy_06        , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_PDSCH_Information },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_fdd_28(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_fdd_28, T_fdd_28_sequence);

  return offset;
}


static const value_string rrc_T_modeSpecificPhysChInfo_06_vals[] = {
  {   0, "fdd" },
  {   1, "tdd" },
  { 0, NULL }
};

static const per_choice_t T_modeSpecificPhysChInfo_06_choice[] = {
  {   0, &hf_rrc_fdd_28          , ASN1_NO_EXTENSIONS     , dissect_rrc_T_fdd_28 },
  {   1, &hf_rrc_tdd             , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_modeSpecificPhysChInfo_06(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_modeSpecificPhysChInfo_06, T_modeSpecificPhysChInfo_06_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t RadioBearerRelease_r3_IEs_sequence[] = {
  { &hf_rrc_rrc_TransactionIdentifier, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RRC_TransactionIdentifier },
  { &hf_rrc_integrityProtectionModeInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_IntegrityProtectionModeInfo },
  { &hf_rrc_cipheringModeInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CipheringModeInfo },
  { &hf_rrc_activationTime  , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_ActivationTime },
  { &hf_rrc_new_U_RNTI      , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_U_RNTI },
  { &hf_rrc_new_C_RNTI      , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_C_RNTI },
  { &hf_rrc_rrc_StateIndicator, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RRC_StateIndicator },
  { &hf_rrc_utran_DRX_CycleLengthCoeff, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UTRAN_DRX_CycleLengthCoefficient },
  { &hf_rrc_cn_InformationInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CN_InformationInfo },
  { &hf_rrc_signallingConnectionRelIndication, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CN_DomainIdentity },
  { &hf_rrc_ura_Identity    , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_URA_Identity },
  { &hf_rrc_rab_InformationReconfigList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RAB_InformationReconfigList },
  { &hf_rrc_rb_InformationReleaseList, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RB_InformationReleaseList },
  { &hf_rrc_rb_InformationAffectedList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RB_InformationAffectedList },
  { &hf_rrc_dl_CounterSynchronisationInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_CounterSynchronisationInfo },
  { &hf_rrc_ul_CommonTransChInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_CommonTransChInfo },
  { &hf_rrc_ul_deletedTransChInfoList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_DeletedTransChInfoList },
  { &hf_rrc_ul_AddReconfTransChInfoList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_AddReconfTransChInfoList },
  { &hf_rrc_dummy_17        , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_dummy_04 },
  { &hf_rrc_dl_CommonTransChInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_CommonTransChInfo },
  { &hf_rrc_dl_DeletedTransChInfoList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_DeletedTransChInfoList },
  { &hf_rrc_dl_AddReconfTransChInfoList_07, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_AddReconfTransChInfo2List },
  { &hf_rrc_frequencyInfo   , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_FrequencyInfo },
  { &hf_rrc_maxAllowedUL_TX_Power, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MaxAllowedUL_TX_Power },
  { &hf_rrc_ul_ChannelRequirement, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_ChannelRequirement },
  { &hf_rrc_modeSpecificPhysChInfo_06, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_modeSpecificPhysChInfo_06 },
  { &hf_rrc_dl_CommonInformation, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_CommonInformation },
  { &hf_rrc_dl_InformationPerRL_List, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_InformationPerRL_List },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_RadioBearerRelease_r3_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_RadioBearerRelease_r3_IEs, RadioBearerRelease_r3_IEs_sequence);

  return offset;
}


static const per_sequence_t RadioBearerRelease_v3a0ext_sequence[] = {
  { &hf_rrc_new_DSCH_RNTI   , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DSCH_RNTI },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_RadioBearerRelease_v3a0ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_RadioBearerRelease_v3a0ext, RadioBearerRelease_v3a0ext_sequence);

  return offset;
}


static const per_sequence_t RadioBearerRelease_v4b0ext_IEs_sequence[] = {
  { &hf_rrc_dummy_02        , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_SSDT_UL },
  { &hf_rrc_cell_id_PerRL_List, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CellIdentity_PerRL_List },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_RadioBearerRelease_v4b0ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_RadioBearerRelease_v4b0ext_IEs, RadioBearerRelease_v4b0ext_IEs_sequence);

  return offset;
}


static const per_sequence_t RadioBearerRelease_v590ext_IEs_sequence[] = {
  { &hf_rrc_dl_TPC_PowerOffsetPerRL_List, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_TPC_PowerOffsetPerRL_List },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_RadioBearerRelease_v590ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_RadioBearerRelease_v590ext_IEs, RadioBearerRelease_v590ext_IEs_sequence);

  return offset;
}


static const value_string rrc_T_postVerificationPeriod_03_vals[] = {
  {   0, "true" },
  { 0, NULL }
};


static int
dissect_rrc_T_postVerificationPeriod_03(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t RadioBearerRelease_v690ext_IEs_sequence[] = {
  { &hf_rrc_primary_plmn_Identity, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_PLMN_Identity },
  { &hf_rrc_harq_Preamble_Mode, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_HARQ_Preamble_Mode },
  { &hf_rrc_beaconPLEst     , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_BEACON_PL_Est },
  { &hf_rrc_postVerificationPeriod_03, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_postVerificationPeriod_03 },
  { &hf_rrc_dhs_sync        , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DHS_Sync },
  { &hf_rrc_timingMaintainedSynchInd, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_TimingMaintainedSynchInd },
  { &hf_rrc_mbms_PL_ServiceRestrictInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MBMS_PL_ServiceRestrictInfo_r6 },
  { &hf_rrc_mbms_RB_ListReleasedToChangeTransferMode, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RB_InformationReleaseList },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_RadioBearerRelease_v690ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_RadioBearerRelease_v690ext_IEs, RadioBearerRelease_v690ext_IEs_sequence);

  return offset;
}


static const per_sequence_t RadioBearerRelease_v770ext_IEs_sequence[] = {
  { &hf_rrc_ueMobilityStateIndicator, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_High_MobilityDetected },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_RadioBearerRelease_v770ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_RadioBearerRelease_v770ext_IEs, RadioBearerRelease_v770ext_IEs_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions_95_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_nonCriticalExtensions_95(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_nonCriticalExtensions_95, T_nonCriticalExtensions_95_sequence);

  return offset;
}


static const per_sequence_t T_v770NonCriticalExtensions_15_sequence[] = {
  { &hf_rrc_radioBearerRelease_v770ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RadioBearerRelease_v770ext_IEs },
  { &hf_rrc_nonCriticalExtensions_95, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_nonCriticalExtensions_95 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v770NonCriticalExtensions_15(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v770NonCriticalExtensions_15, T_v770NonCriticalExtensions_15_sequence);

  return offset;
}


static const per_sequence_t T_v690NonCriticalExtensions_19_sequence[] = {
  { &hf_rrc_radioBearerRelease_v690ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RadioBearerRelease_v690ext_IEs },
  { &hf_rrc_v770NonCriticalExtensions_15, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v770NonCriticalExtensions_15 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v690NonCriticalExtensions_19(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v690NonCriticalExtensions_19, T_v690NonCriticalExtensions_19_sequence);

  return offset;
}


static const per_sequence_t T_v590NonCriticalExtensions_16_sequence[] = {
  { &hf_rrc_radioBearerRelease_v590ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RadioBearerRelease_v590ext_IEs },
  { &hf_rrc_v690NonCriticalExtensions_19, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v690NonCriticalExtensions_19 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v590NonCriticalExtensions_16(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v590NonCriticalExtensions_16, T_v590NonCriticalExtensions_16_sequence);

  return offset;
}


static const per_sequence_t T_v4b0NonCriticalExtensions_08_sequence[] = {
  { &hf_rrc_radioBearerRelease_v4b0ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RadioBearerRelease_v4b0ext_IEs },
  { &hf_rrc_v590NonCriticalExtensions_16, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v590NonCriticalExtensions_16 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v4b0NonCriticalExtensions_08(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v4b0NonCriticalExtensions_08, T_v4b0NonCriticalExtensions_08_sequence);

  return offset;
}


static const per_sequence_t T_laterNonCriticalExtensions_31_sequence[] = {
  { &hf_rrc_radioBearerRelease_r3_add_ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_BIT_STRING },
  { &hf_rrc_v4b0NonCriticalExtensions_08, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v4b0NonCriticalExtensions_08 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_laterNonCriticalExtensions_31(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_laterNonCriticalExtensions_31, T_laterNonCriticalExtensions_31_sequence);

  return offset;
}


static const per_sequence_t T_v3a0NonCriticalExtensions_06_sequence[] = {
  { &hf_rrc_radioBearerRelease_v3a0ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RadioBearerRelease_v3a0ext },
  { &hf_rrc_laterNonCriticalExtensions_31, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_laterNonCriticalExtensions_31 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v3a0NonCriticalExtensions_06(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v3a0NonCriticalExtensions_06, T_v3a0NonCriticalExtensions_06_sequence);

  return offset;
}


static const per_sequence_t T_r3_15_sequence[] = {
  { &hf_rrc_radioBearerRelease_r3, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RadioBearerRelease_r3_IEs },
  { &hf_rrc_v3a0NonCriticalExtensions_06, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v3a0NonCriticalExtensions_06 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_r3_15(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_r3_15, T_r3_15_sequence);

  return offset;
}


static const per_sequence_t T_fdd_29_sequence[] = {
  { &hf_rrc_dummy1          , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CPCH_SetID },
  { &hf_rrc_dummy2_02       , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DRAC_StaticInformationList },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_fdd_29(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_fdd_29, T_fdd_29_sequence);

  return offset;
}


static const value_string rrc_T_dummy_05_vals[] = {
  {   0, "fdd" },
  {   1, "tdd" },
  { 0, NULL }
};

static const per_choice_t T_dummy_05_choice[] = {
  {   0, &hf_rrc_fdd_29          , ASN1_NO_EXTENSIONS     , dissect_rrc_T_fdd_29 },
  {   1, &hf_rrc_tdd             , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_dummy_05(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_dummy_05, T_dummy_05_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t T_fdd_30_sequence[] = {
  { &hf_rrc_dummy_06        , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_PDSCH_Information },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_fdd_30(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_fdd_30, T_fdd_30_sequence);

  return offset;
}


static const value_string rrc_T_modeSpecificPhysChInfo_07_vals[] = {
  {   0, "fdd" },
  {   1, "tdd" },
  { 0, NULL }
};

static const per_choice_t T_modeSpecificPhysChInfo_07_choice[] = {
  {   0, &hf_rrc_fdd_30          , ASN1_NO_EXTENSIONS     , dissect_rrc_T_fdd_30 },
  {   1, &hf_rrc_tdd             , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_modeSpecificPhysChInfo_07(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_modeSpecificPhysChInfo_07, T_modeSpecificPhysChInfo_07_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t RadioBearerRelease_r4_IEs_sequence[] = {
  { &hf_rrc_integrityProtectionModeInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_IntegrityProtectionModeInfo },
  { &hf_rrc_cipheringModeInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CipheringModeInfo },
  { &hf_rrc_activationTime  , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_ActivationTime },
  { &hf_rrc_new_U_RNTI      , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_U_RNTI },
  { &hf_rrc_new_C_RNTI      , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_C_RNTI },
  { &hf_rrc_new_DSCH_RNTI   , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DSCH_RNTI },
  { &hf_rrc_rrc_StateIndicator, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RRC_StateIndicator },
  { &hf_rrc_utran_DRX_CycleLengthCoeff, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UTRAN_DRX_CycleLengthCoefficient },
  { &hf_rrc_cn_InformationInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CN_InformationInfo },
  { &hf_rrc_signallingConnectionRelIndication, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CN_DomainIdentity },
  { &hf_rrc_ura_Identity    , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_URA_Identity },
  { &hf_rrc_rab_InformationReconfigList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RAB_InformationReconfigList },
  { &hf_rrc_rb_InformationReleaseList, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RB_InformationReleaseList },
  { &hf_rrc_rb_InformationAffectedList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RB_InformationAffectedList },
  { &hf_rrc_dl_CounterSynchronisationInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_CounterSynchronisationInfo },
  { &hf_rrc_ul_CommonTransChInfo_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_CommonTransChInfo_r4 },
  { &hf_rrc_ul_deletedTransChInfoList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_DeletedTransChInfoList },
  { &hf_rrc_ul_AddReconfTransChInfoList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_AddReconfTransChInfoList },
  { &hf_rrc_dummy_18        , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_dummy_05 },
  { &hf_rrc_dl_CommonTransChInfo_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_CommonTransChInfo_r4 },
  { &hf_rrc_dl_DeletedTransChInfoList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_DeletedTransChInfoList },
  { &hf_rrc_dl_AddReconfTransChInfoList_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_AddReconfTransChInfoList_r4 },
  { &hf_rrc_frequencyInfo   , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_FrequencyInfo },
  { &hf_rrc_maxAllowedUL_TX_Power, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MaxAllowedUL_TX_Power },
  { &hf_rrc_ul_ChannelRequirement_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_ChannelRequirement_r4 },
  { &hf_rrc_modeSpecificPhysChInfo_07, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_modeSpecificPhysChInfo_07 },
  { &hf_rrc_dl_CommonInformation_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_CommonInformation_r4 },
  { &hf_rrc_dl_InformationPerRL_List_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_InformationPerRL_List_r4 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_RadioBearerRelease_r4_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_RadioBearerRelease_r4_IEs, RadioBearerRelease_r4_IEs_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions_96_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_nonCriticalExtensions_96(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_nonCriticalExtensions_96, T_nonCriticalExtensions_96_sequence);

  return offset;
}


static const per_sequence_t T_v770NonCriticalExtensions_16_sequence[] = {
  { &hf_rrc_radioBearerRelease_v770ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RadioBearerRelease_v770ext_IEs },
  { &hf_rrc_nonCriticalExtensions_96, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_nonCriticalExtensions_96 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v770NonCriticalExtensions_16(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v770NonCriticalExtensions_16, T_v770NonCriticalExtensions_16_sequence);

  return offset;
}


static const per_sequence_t T_v690NonCriticalExtensions_20_sequence[] = {
  { &hf_rrc_radioBearerRelease_v690ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RadioBearerRelease_v690ext_IEs },
  { &hf_rrc_v770NonCriticalExtensions_16, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v770NonCriticalExtensions_16 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v690NonCriticalExtensions_20(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v690NonCriticalExtensions_20, T_v690NonCriticalExtensions_20_sequence);

  return offset;
}


static const per_sequence_t T_v590NonCriticalExtensions_17_sequence[] = {
  { &hf_rrc_radioBearerRelease_v590ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RadioBearerRelease_v590ext_IEs },
  { &hf_rrc_v690NonCriticalExtensions_20, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v690NonCriticalExtensions_20 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v590NonCriticalExtensions_17(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v590NonCriticalExtensions_17, T_v590NonCriticalExtensions_17_sequence);

  return offset;
}


static const per_sequence_t T_v4d0NonCriticalExtensions_07_sequence[] = {
  { &hf_rrc_radioBearerRelease_r4_add_ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_BIT_STRING },
  { &hf_rrc_v590NonCriticalExtensions_17, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v590NonCriticalExtensions_17 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v4d0NonCriticalExtensions_07(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v4d0NonCriticalExtensions_07, T_v4d0NonCriticalExtensions_07_sequence);

  return offset;
}


static const per_sequence_t T_r4_07_sequence[] = {
  { &hf_rrc_radioBearerRelease_r4, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RadioBearerRelease_r4_IEs },
  { &hf_rrc_v4d0NonCriticalExtensions_07, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v4d0NonCriticalExtensions_07 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_r4_07(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_r4_07, T_r4_07_sequence);

  return offset;
}


static const per_sequence_t T_fdd_31_sequence[] = {
  { &hf_rrc_dummy1          , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CPCH_SetID },
  { &hf_rrc_dummy2_02       , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DRAC_StaticInformationList },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_fdd_31(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_fdd_31, T_fdd_31_sequence);

  return offset;
}


static const value_string rrc_T_dummy_06_vals[] = {
  {   0, "fdd" },
  {   1, "tdd" },
  { 0, NULL }
};

static const per_choice_t T_dummy_06_choice[] = {
  {   0, &hf_rrc_fdd_31          , ASN1_NO_EXTENSIONS     , dissect_rrc_T_fdd_31 },
  {   1, &hf_rrc_tdd             , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_dummy_06(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_dummy_06, T_dummy_06_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t T_fdd_32_sequence[] = {
  { &hf_rrc_dummy_06        , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_PDSCH_Information },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_fdd_32(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_fdd_32, T_fdd_32_sequence);

  return offset;
}


static const value_string rrc_T_modeSpecificPhysChInfo_08_vals[] = {
  {   0, "fdd" },
  {   1, "tdd" },
  { 0, NULL }
};

static const per_choice_t T_modeSpecificPhysChInfo_08_choice[] = {
  {   0, &hf_rrc_fdd_32          , ASN1_NO_EXTENSIONS     , dissect_rrc_T_fdd_32 },
  {   1, &hf_rrc_tdd             , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_modeSpecificPhysChInfo_08(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_modeSpecificPhysChInfo_08, T_modeSpecificPhysChInfo_08_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t RadioBearerRelease_r5_IEs_sequence[] = {
  { &hf_rrc_integrityProtectionModeInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_IntegrityProtectionModeInfo },
  { &hf_rrc_cipheringModeInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CipheringModeInfo },
  { &hf_rrc_activationTime  , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_ActivationTime },
  { &hf_rrc_new_U_RNTI      , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_U_RNTI },
  { &hf_rrc_new_C_RNTI      , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_C_RNTI },
  { &hf_rrc_new_DSCH_RNTI   , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DSCH_RNTI },
  { &hf_rrc_new_H_RNTI      , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_H_RNTI },
  { &hf_rrc_rrc_StateIndicator, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RRC_StateIndicator },
  { &hf_rrc_utran_DRX_CycleLengthCoeff, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UTRAN_DRX_CycleLengthCoefficient },
  { &hf_rrc_cn_InformationInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CN_InformationInfo },
  { &hf_rrc_signallingConnectionRelIndication, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CN_DomainIdentity },
  { &hf_rrc_ura_Identity    , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_URA_Identity },
  { &hf_rrc_rab_InformationReconfigList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RAB_InformationReconfigList },
  { &hf_rrc_rb_InformationReleaseList, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RB_InformationReleaseList },
  { &hf_rrc_rb_InformationAffectedList_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RB_InformationAffectedList_r5 },
  { &hf_rrc_dl_CounterSynchronisationInfo_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_CounterSynchronisationInfo_r5 },
  { &hf_rrc_ul_CommonTransChInfo_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_CommonTransChInfo_r4 },
  { &hf_rrc_ul_deletedTransChInfoList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_DeletedTransChInfoList },
  { &hf_rrc_ul_AddReconfTransChInfoList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_AddReconfTransChInfoList },
  { &hf_rrc_dummy_19        , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_dummy_06 },
  { &hf_rrc_dl_CommonTransChInfo_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_CommonTransChInfo_r4 },
  { &hf_rrc_dl_DeletedTransChInfoList_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_DeletedTransChInfoList_r5 },
  { &hf_rrc_dl_AddReconfTransChInfoList_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_AddReconfTransChInfoList_r5 },
  { &hf_rrc_frequencyInfo   , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_FrequencyInfo },
  { &hf_rrc_maxAllowedUL_TX_Power, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MaxAllowedUL_TX_Power },
  { &hf_rrc_ul_ChannelRequirement_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_ChannelRequirement_r5 },
  { &hf_rrc_modeSpecificPhysChInfo_08, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_modeSpecificPhysChInfo_08 },
  { &hf_rrc_dl_HSPDSCH_Information, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_HSPDSCH_Information },
  { &hf_rrc_dl_CommonInformation_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_CommonInformation_r5 },
  { &hf_rrc_dl_InformationPerRL_List_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_InformationPerRL_List_r5 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_RadioBearerRelease_r5_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_RadioBearerRelease_r5_IEs, RadioBearerRelease_r5_IEs_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions_97_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_nonCriticalExtensions_97(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_nonCriticalExtensions_97, T_nonCriticalExtensions_97_sequence);

  return offset;
}


static const per_sequence_t T_v770NonCriticalExtensions_17_sequence[] = {
  { &hf_rrc_radioBearerRelease_v770ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RadioBearerRelease_v770ext_IEs },
  { &hf_rrc_nonCriticalExtensions_97, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_nonCriticalExtensions_97 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v770NonCriticalExtensions_17(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v770NonCriticalExtensions_17, T_v770NonCriticalExtensions_17_sequence);

  return offset;
}


static const per_sequence_t T_v690NonCriticalExtensions_21_sequence[] = {
  { &hf_rrc_radioBearerRelease_v690ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RadioBearerRelease_v690ext_IEs },
  { &hf_rrc_v770NonCriticalExtensions_17, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v770NonCriticalExtensions_17 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v690NonCriticalExtensions_21(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v690NonCriticalExtensions_21, T_v690NonCriticalExtensions_21_sequence);

  return offset;
}


static const per_sequence_t T_r5_06_sequence[] = {
  { &hf_rrc_radioBearerRelease_r5, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RadioBearerRelease_r5_IEs },
  { &hf_rrc_radioBearerRelease_r5_add_ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_BIT_STRING },
  { &hf_rrc_v690NonCriticalExtensions_21, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v690NonCriticalExtensions_21 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_r5_06(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_r5_06, T_r5_06_sequence);

  return offset;
}


static const per_sequence_t RadioBearerRelease_r6_IEs_sequence[] = {
  { &hf_rrc_integrityProtectionModeInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_IntegrityProtectionModeInfo },
  { &hf_rrc_cipheringModeInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CipheringModeInfo },
  { &hf_rrc_activationTime  , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_ActivationTime },
  { &hf_rrc_new_U_RNTI      , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_U_RNTI },
  { &hf_rrc_new_C_RNTI      , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_C_RNTI },
  { &hf_rrc_new_DSCH_RNTI   , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DSCH_RNTI },
  { &hf_rrc_new_H_RNTI      , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_H_RNTI },
  { &hf_rrc_newPrimary_E_RNTI, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_E_RNTI },
  { &hf_rrc_newSecondary_E_RNTI, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_E_RNTI },
  { &hf_rrc_rrc_StateIndicator, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RRC_StateIndicator },
  { &hf_rrc_utran_DRX_CycleLengthCoeff, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UTRAN_DRX_CycleLengthCoefficient },
  { &hf_rrc_cn_InformationInfo_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CN_InformationInfo_r6 },
  { &hf_rrc_signallingConnectionRelIndication, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CN_DomainIdentity },
  { &hf_rrc_ura_Identity    , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_URA_Identity },
  { &hf_rrc_rab_InformationReconfigList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RAB_InformationReconfigList },
  { &hf_rrc_rb_InformationReleaseList, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RB_InformationReleaseList },
  { &hf_rrc_rb_InformationReconfigList_03, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RB_InformationReconfigList_r6 },
  { &hf_rrc_rb_InformationAffectedList_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RB_InformationAffectedList_r6 },
  { &hf_rrc_dl_CounterSynchronisationInfo_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_CounterSynchronisationInfo_r5 },
  { &hf_rrc_ul_CommonTransChInfo_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_CommonTransChInfo_r4 },
  { &hf_rrc_ul_deletedTransChInfoList_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_DeletedTransChInfoList_r6 },
  { &hf_rrc_ul_AddReconfTransChInfoList_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_AddReconfTransChInfoList_r6 },
  { &hf_rrc_dl_CommonTransChInfo_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_CommonTransChInfo_r4 },
  { &hf_rrc_dl_DeletedTransChInfoList_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_DeletedTransChInfoList_r5 },
  { &hf_rrc_dl_AddReconfTransChInfoList_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_AddReconfTransChInfoList_r5 },
  { &hf_rrc_frequencyInfo   , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_FrequencyInfo },
  { &hf_rrc_maxAllowedUL_TX_Power, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MaxAllowedUL_TX_Power },
  { &hf_rrc_ul_DPCH_Info    , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_DPCH_Info_r6 },
  { &hf_rrc_ul_EDCH_Information, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_EDCH_Information_r6 },
  { &hf_rrc_dl_HSPDSCH_Information_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_HSPDSCH_Information_r6 },
  { &hf_rrc_dl_CommonInformation_03, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_CommonInformation_r6 },
  { &hf_rrc_dl_InformationPerRL_List_03, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_InformationPerRL_List_r6 },
  { &hf_rrc_mbms_PL_ServiceRestrictInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MBMS_PL_ServiceRestrictInfo_r6 },
  { &hf_rrc_mbms_RB_ListReleasedToChangeTransferMode, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RB_InformationReleaseList },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_RadioBearerRelease_r6_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_RadioBearerRelease_r6_IEs, RadioBearerRelease_r6_IEs_sequence);

  return offset;
}


static const per_sequence_t RadioBearerRelease_v6b0ext_IEs_sequence[] = {
  { &hf_rrc_dl_InformationPerRL_List_v6b0ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_InformationPerRL_List_v6b0ext },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_RadioBearerRelease_v6b0ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_RadioBearerRelease_v6b0ext_IEs, RadioBearerRelease_v6b0ext_IEs_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions_98_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_nonCriticalExtensions_98(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_nonCriticalExtensions_98, T_nonCriticalExtensions_98_sequence);

  return offset;
}


static const per_sequence_t T_v770NonCriticalExtensions_18_sequence[] = {
  { &hf_rrc_radioBearerRelease_v770ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RadioBearerRelease_v770ext_IEs },
  { &hf_rrc_nonCriticalExtensions_98, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_nonCriticalExtensions_98 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v770NonCriticalExtensions_18(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v770NonCriticalExtensions_18, T_v770NonCriticalExtensions_18_sequence);

  return offset;
}


static const per_sequence_t T_v6b0NonCriticalExtensions_07_sequence[] = {
  { &hf_rrc_radioBearerRelease_v6b0ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RadioBearerRelease_v6b0ext_IEs },
  { &hf_rrc_v770NonCriticalExtensions_18, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v770NonCriticalExtensions_18 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v6b0NonCriticalExtensions_07(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v6b0NonCriticalExtensions_07, T_v6b0NonCriticalExtensions_07_sequence);

  return offset;
}


static const per_sequence_t T_r6_08_sequence[] = {
  { &hf_rrc_radioBearerRelease_r6, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RadioBearerRelease_r6_IEs },
  { &hf_rrc_radioBearerRelease_r6_add_ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_BIT_STRING },
  { &hf_rrc_v6b0NonCriticalExtensions_08, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v6b0NonCriticalExtensions_07 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_r6_08(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_r6_08, T_r6_08_sequence);

  return offset;
}


static const per_sequence_t RadioBearerRelease_r7_IEs_sequence[] = {
  { &hf_rrc_integrityProtectionModeInfo_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_IntegrityProtectionModeInfo_r7 },
  { &hf_rrc_cipheringModeInfo_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CipheringModeInfo_r7 },
  { &hf_rrc_activationTime  , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_ActivationTime },
  { &hf_rrc_new_U_RNTI      , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_U_RNTI },
  { &hf_rrc_new_C_RNTI      , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_C_RNTI },
  { &hf_rrc_new_DSCH_RNTI   , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DSCH_RNTI },
  { &hf_rrc_new_H_RNTI      , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_H_RNTI },
  { &hf_rrc_newPrimary_E_RNTI, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_E_RNTI },
  { &hf_rrc_newSecondary_E_RNTI, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_E_RNTI },
  { &hf_rrc_rrc_StateIndicator, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RRC_StateIndicator },
  { &hf_rrc_ueMobilityStateIndicator, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_High_MobilityDetected },
  { &hf_rrc_utran_DRX_CycleLengthCoeff_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UTRAN_DRX_CycleLengthCoefficient_r7 },
  { &hf_rrc_cn_InformationInfo_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CN_InformationInfo_r6 },
  { &hf_rrc_signallingConnectionRelIndication, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CN_DomainIdentity },
  { &hf_rrc_ura_Identity    , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_URA_Identity },
  { &hf_rrc_supportForChangeOfUE_Capability, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_BOOLEAN },
  { &hf_rrc_rab_InformationReconfigList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RAB_InformationReconfigList },
  { &hf_rrc_rb_InformationReleaseList, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RB_InformationReleaseList },
  { &hf_rrc_rb_InformationReconfigList_04, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RB_InformationReconfigList_r7 },
  { &hf_rrc_rb_InformationAffectedList_03, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RB_InformationAffectedList_r7 },
  { &hf_rrc_dl_CounterSynchronisationInfo_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_CounterSynchronisationInfo_r5 },
  { &hf_rrc_ul_CommonTransChInfo_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_CommonTransChInfo_r4 },
  { &hf_rrc_ul_deletedTransChInfoList_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_DeletedTransChInfoList_r6 },
  { &hf_rrc_ul_AddReconfTransChInfoList_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_AddReconfTransChInfoList_r7 },
  { &hf_rrc_dl_CommonTransChInfo_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_CommonTransChInfo_r4 },
  { &hf_rrc_dl_DeletedTransChInfoList_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_DeletedTransChInfoList_r7 },
  { &hf_rrc_dl_AddReconfTransChInfoList_03, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_AddReconfTransChInfoList_r7 },
  { &hf_rrc_frequencyInfo   , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_FrequencyInfo },
  { &hf_rrc_multi_frequencyInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_Multi_frequencyInfo_LCR_r7 },
  { &hf_rrc_dtx_drx_TimingInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DTX_DRX_TimingInfo_r7 },
  { &hf_rrc_dtx_drx_Info    , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DTX_DRX_Info_r7 },
  { &hf_rrc_hs_scch_LessInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_HS_SCCH_LessInfo_r7 },
  { &hf_rrc_mimoParameters  , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MIMO_Parameters_r7 },
  { &hf_rrc_maxAllowedUL_TX_Power, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MaxAllowedUL_TX_Power },
  { &hf_rrc_ul_DPCH_Info_01 , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_DPCH_Info_r7 },
  { &hf_rrc_ul_EDCH_Information_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_EDCH_Information_r7 },
  { &hf_rrc_dl_HSPDSCH_Information_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_HSPDSCH_Information_r7 },
  { &hf_rrc_dl_CommonInformation_04, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_CommonInformation_r7 },
  { &hf_rrc_dl_InformationPerRL_List_04, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_InformationPerRL_List_r7 },
  { &hf_rrc_mbms_PL_ServiceRestrictInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MBMS_PL_ServiceRestrictInfo_r6 },
  { &hf_rrc_mbms_RB_ListReleasedToChangeTransferMode, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RB_InformationReleaseList },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_RadioBearerRelease_r7_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_RadioBearerRelease_r7_IEs, RadioBearerRelease_r7_IEs_sequence);

  return offset;
}


static const per_sequence_t RadioBearerRelease_v780ext_IEs_sequence[] = {
  { &hf_rrc_hs_DSCH_TBSizeTable, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_HS_DSCH_TBSizeTable },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_RadioBearerRelease_v780ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_RadioBearerRelease_v780ext_IEs, RadioBearerRelease_v780ext_IEs_sequence);

  return offset;
}


static const per_sequence_t RadioBearerRelease_v7d0ext_IEs_sequence[] = {
  { &hf_rrc_ul_EDCH_Information_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_EDCH_Information_ext },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_RadioBearerRelease_v7d0ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_RadioBearerRelease_v7d0ext_IEs, RadioBearerRelease_v7d0ext_IEs_sequence);

  return offset;
}


static const per_sequence_t RadioBearerRelease_v7f0ext_IEs_sequence[] = {
  { &hf_rrc_mimoParameters_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MIMO_Parameters_v7f0ext },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_RadioBearerRelease_v7f0ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_RadioBearerRelease_v7f0ext_IEs, RadioBearerRelease_v7f0ext_IEs_sequence);

  return offset;
}


static const per_sequence_t RadioBearerRelease_v7g0ext_IEs_sequence[] = {
  { &hf_rrc_mimoParameters_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MIMO_Parameters_v7g0ext },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_RadioBearerRelease_v7g0ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_RadioBearerRelease_v7g0ext_IEs, RadioBearerRelease_v7g0ext_IEs_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions_99_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_nonCriticalExtensions_99(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_nonCriticalExtensions_99, T_nonCriticalExtensions_99_sequence);

  return offset;
}


static const per_sequence_t T_v7g0NonCriticalExtensions_10_sequence[] = {
  { &hf_rrc_radioBearerRelease_v7g0ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RadioBearerRelease_v7g0ext_IEs },
  { &hf_rrc_nonCriticalExtensions_99, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_nonCriticalExtensions_99 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v7g0NonCriticalExtensions_10(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v7g0NonCriticalExtensions_10, T_v7g0NonCriticalExtensions_10_sequence);

  return offset;
}


static const per_sequence_t T_v7f0NonCriticalExtensions_09_sequence[] = {
  { &hf_rrc_radioBearerRelease_v7f0ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RadioBearerRelease_v7f0ext_IEs },
  { &hf_rrc_v7g0NonCriticalExtensions_10, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v7g0NonCriticalExtensions_10 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v7f0NonCriticalExtensions_09(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v7f0NonCriticalExtensions_09, T_v7f0NonCriticalExtensions_09_sequence);

  return offset;
}


static const per_sequence_t T_v7d0NonCriticalExtensions_08_sequence[] = {
  { &hf_rrc_radioBearerRelease_v7d0ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RadioBearerRelease_v7d0ext_IEs },
  { &hf_rrc_v7f0NonCriticalExtensions_09, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v7f0NonCriticalExtensions_09 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v7d0NonCriticalExtensions_08(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v7d0NonCriticalExtensions_08, T_v7d0NonCriticalExtensions_08_sequence);

  return offset;
}


static const per_sequence_t T_v780NonCriticalExtensions_06_sequence[] = {
  { &hf_rrc_radioBearerRelease_v780ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RadioBearerRelease_v780ext_IEs },
  { &hf_rrc_v7d0NonCriticalExtensions_08, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v7d0NonCriticalExtensions_08 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v780NonCriticalExtensions_06(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v780NonCriticalExtensions_06, T_v780NonCriticalExtensions_06_sequence);

  return offset;
}


static const per_sequence_t T_r7_07_sequence[] = {
  { &hf_rrc_radioBearerRelease_r7, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RadioBearerRelease_r7_IEs },
  { &hf_rrc_radioBearerRelease_r7_add_ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_BIT_STRING },
  { &hf_rrc_v780NonCriticalExtensions_06, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v780NonCriticalExtensions_06 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_r7_07(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_r7_07, T_r7_07_sequence);

  return offset;
}


static const per_sequence_t RadioBearerRelease_r8_IEs_sequence[] = {
  { &hf_rrc_integrityProtectionModeInfo_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_IntegrityProtectionModeInfo_r7 },
  { &hf_rrc_cipheringModeInfo_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CipheringModeInfo_r7 },
  { &hf_rrc_activationTime  , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_ActivationTime },
  { &hf_rrc_new_U_RNTI      , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_U_RNTI },
  { &hf_rrc_new_C_RNTI      , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_C_RNTI },
  { &hf_rrc_new_DSCH_RNTI   , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DSCH_RNTI },
  { &hf_rrc_new_H_RNTI      , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_H_RNTI },
  { &hf_rrc_newPrimary_E_RNTI, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_E_RNTI },
  { &hf_rrc_newSecondary_E_RNTI, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_E_RNTI },
  { &hf_rrc_rrc_StateIndicator, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RRC_StateIndicator },
  { &hf_rrc_ueMobilityStateIndicator, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_High_MobilityDetected },
  { &hf_rrc_utran_DRX_CycleLengthCoeff_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UTRAN_DRX_CycleLengthCoefficient_r7 },
  { &hf_rrc_cn_InformationInfo_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CN_InformationInfo_r6 },
  { &hf_rrc_signallingConnectionRelIndication, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CN_DomainIdentity },
  { &hf_rrc_ura_Identity    , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_URA_Identity },
  { &hf_rrc_supportForChangeOfUE_Capability, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_BOOLEAN },
  { &hf_rrc_rab_InformationReconfigList_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RAB_InformationReconfigList_r8 },
  { &hf_rrc_rb_InformationReleaseList, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RB_InformationReleaseList },
  { &hf_rrc_rb_InformationReconfigList_05, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RB_InformationReconfigList_r8 },
  { &hf_rrc_rb_InformationAffectedList_04, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RB_InformationAffectedList_r8 },
  { &hf_rrc_dl_CounterSynchronisationInfo_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_CounterSynchronisationInfo_r5 },
  { &hf_rrc_ul_CommonTransChInfo_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_CommonTransChInfo_r4 },
  { &hf_rrc_ul_deletedTransChInfoList_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_DeletedTransChInfoList_r6 },
  { &hf_rrc_ul_AddReconfTransChInfoList_03, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_AddReconfTransChInfoList_r8 },
  { &hf_rrc_dl_CommonTransChInfo_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_CommonTransChInfo_r4 },
  { &hf_rrc_dl_DeletedTransChInfoList_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_DeletedTransChInfoList_r7 },
  { &hf_rrc_dl_AddReconfTransChInfoList_03, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_AddReconfTransChInfoList_r7 },
  { &hf_rrc_frequencyInfo   , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_FrequencyInfo },
  { &hf_rrc_multi_frequencyInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_Multi_frequencyInfo_LCR_r7 },
  { &hf_rrc_dtx_drx_TimingInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DTX_DRX_TimingInfo_r7 },
  { &hf_rrc_dtx_drx_Info    , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DTX_DRX_Info_r7 },
  { &hf_rrc_hs_scch_LessInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_HS_SCCH_LessInfo_r7 },
  { &hf_rrc_mimoParameters_03, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MIMO_Parameters_r8 },
  { &hf_rrc_maxAllowedUL_TX_Power, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MaxAllowedUL_TX_Power },
  { &hf_rrc_ul_DPCH_Info_01 , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_DPCH_Info_r7 },
  { &hf_rrc_ul_EDCH_Information_03, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_EDCH_Information_r8 },
  { &hf_rrc_dl_HSPDSCH_Information_03, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_HSPDSCH_Information_r8 },
  { &hf_rrc_dl_CommonInformation_05, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_CommonInformation_r8 },
  { &hf_rrc_dl_InformationPerRL_List_05, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_InformationPerRL_List_r8 },
  { &hf_rrc_dl_SecondaryCellInfoFDD, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_SecondaryCellInfoFDD },
  { &hf_rrc_controlChannelDRXInfo_TDD128, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_ControlChannelDRXInfo_TDD128_r8 },
  { &hf_rrc_sps_Information_TDD128, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_SPS_Information_TDD128_r8 },
  { &hf_rrc_mbms_PL_ServiceRestrictInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MBMS_PL_ServiceRestrictInfo_r6 },
  { &hf_rrc_mbms_RB_ListReleasedToChangeTransferMode, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RB_InformationReleaseList },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_RadioBearerRelease_r8_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_RadioBearerRelease_r8_IEs, RadioBearerRelease_r8_IEs_sequence);

  return offset;
}


static const per_sequence_t RadioBearerRelease_v890ext_IEs_sequence[] = {
  { &hf_rrc_dl_SecondaryCellInfoFDD_v890ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_SecondaryCellInfoFDD_v890ext },
  { &hf_rrc_dl_HSPDSCH_Information_04, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_HSPDSCH_Information_r8_ext },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_RadioBearerRelease_v890ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_RadioBearerRelease_v890ext_IEs, RadioBearerRelease_v890ext_IEs_sequence);

  return offset;
}


static const per_sequence_t RadioBearerRelease_v8a0ext_IEs_sequence[] = {
  { &hf_rrc_dl_HSPDSCH_Information_05, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_HSPDSCH_Information_r8_ext2 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_RadioBearerRelease_v8a0ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_RadioBearerRelease_v8a0ext_IEs, RadioBearerRelease_v8a0ext_IEs_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions_100_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_nonCriticalExtensions_100(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_nonCriticalExtensions_100, T_nonCriticalExtensions_100_sequence);

  return offset;
}


static const per_sequence_t T_v8a0NonCriticalExtensions_06_sequence[] = {
  { &hf_rrc_radioBearerRelease_v8a0ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RadioBearerRelease_v8a0ext_IEs },
  { &hf_rrc_nonCriticalExtensions_100, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_nonCriticalExtensions_100 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v8a0NonCriticalExtensions_06(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v8a0NonCriticalExtensions_06, T_v8a0NonCriticalExtensions_06_sequence);

  return offset;
}


static const per_sequence_t T_v7g0NonCriticalExtensions_11_sequence[] = {
  { &hf_rrc_radioBearerRelease_v7g0ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RadioBearerRelease_v7g0ext_IEs },
  { &hf_rrc_v8a0NonCriticalExtensions_06, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v8a0NonCriticalExtensions_06 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v7g0NonCriticalExtensions_11(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v7g0NonCriticalExtensions_11, T_v7g0NonCriticalExtensions_11_sequence);

  return offset;
}


static const per_sequence_t T_v890NonCriticalExtensions_06_sequence[] = {
  { &hf_rrc_radioBearerRelease_v890ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RadioBearerRelease_v890ext_IEs },
  { &hf_rrc_v7g0NonCriticalExtensions_11, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v7g0NonCriticalExtensions_11 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v890NonCriticalExtensions_06(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v890NonCriticalExtensions_06, T_v890NonCriticalExtensions_06_sequence);

  return offset;
}


static const per_sequence_t T_v7f0NonCriticalExtensions_10_sequence[] = {
  { &hf_rrc_radioBearerRelease_v7f0ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RadioBearerRelease_v7f0ext_IEs },
  { &hf_rrc_v890NonCriticalExtensions_06, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v890NonCriticalExtensions_06 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v7f0NonCriticalExtensions_10(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v7f0NonCriticalExtensions_10, T_v7f0NonCriticalExtensions_10_sequence);

  return offset;
}


static const per_sequence_t T_v7d0NonCriticalExtensions_09_sequence[] = {
  { &hf_rrc_radioBearerRelease_v7d0ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RadioBearerRelease_v7d0ext_IEs },
  { &hf_rrc_v7f0NonCriticalExtensions_10, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v7f0NonCriticalExtensions_10 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v7d0NonCriticalExtensions_09(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v7d0NonCriticalExtensions_09, T_v7d0NonCriticalExtensions_09_sequence);

  return offset;
}


static const per_sequence_t T_r8_08_sequence[] = {
  { &hf_rrc_radioBearerRelease_r8, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RadioBearerRelease_r8_IEs },
  { &hf_rrc_radioBearerRelease_r8_add_ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_BIT_STRING },
  { &hf_rrc_v7d0NonCriticalExtensions_09, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v7d0NonCriticalExtensions_09 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_r8_08(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_r8_08, T_r8_08_sequence);

  return offset;
}


static const per_sequence_t RadioBearerRelease_r9_IEs_sequence[] = {
  { &hf_rrc_integrityProtectionModeInfo_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_IntegrityProtectionModeInfo_r7 },
  { &hf_rrc_cipheringModeInfo_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CipheringModeInfo_r7 },
  { &hf_rrc_activationTime  , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_ActivationTime },
  { &hf_rrc_new_U_RNTI      , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_U_RNTI },
  { &hf_rrc_new_C_RNTI      , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_C_RNTI },
  { &hf_rrc_new_DSCH_RNTI   , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DSCH_RNTI },
  { &hf_rrc_new_H_RNTI      , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_H_RNTI },
  { &hf_rrc_newPrimary_E_RNTI, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_E_RNTI },
  { &hf_rrc_newSecondary_E_RNTI, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_E_RNTI },
  { &hf_rrc_rrc_StateIndicator, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RRC_StateIndicator },
  { &hf_rrc_ueMobilityStateIndicator, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_High_MobilityDetected },
  { &hf_rrc_utran_DRX_CycleLengthCoeff_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UTRAN_DRX_CycleLengthCoefficient_r7 },
  { &hf_rrc_cn_InformationInfo_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CN_InformationInfo_r6 },
  { &hf_rrc_signallingConnectionRelIndication, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CN_DomainIdentity },
  { &hf_rrc_ura_Identity    , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_URA_Identity },
  { &hf_rrc_supportForChangeOfUE_Capability, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_BOOLEAN },
  { &hf_rrc_rab_InformationReconfigList_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RAB_InformationReconfigList_r8 },
  { &hf_rrc_rb_InformationReleaseList, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RB_InformationReleaseList },
  { &hf_rrc_rb_InformationReconfigList_05, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RB_InformationReconfigList_r8 },
  { &hf_rrc_rb_InformationAffectedList_04, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RB_InformationAffectedList_r8 },
  { &hf_rrc_dl_CounterSynchronisationInfo_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_CounterSynchronisationInfo_r5 },
  { &hf_rrc_ul_CommonTransChInfo_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_CommonTransChInfo_r4 },
  { &hf_rrc_ul_deletedTransChInfoList_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_DeletedTransChInfoList_r6 },
  { &hf_rrc_ul_AddReconfTransChInfoList_03, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_AddReconfTransChInfoList_r8 },
  { &hf_rrc_dl_CommonTransChInfo_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_CommonTransChInfo_r4 },
  { &hf_rrc_dl_DeletedTransChInfoList_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_DeletedTransChInfoList_r7 },
  { &hf_rrc_dl_AddReconfTransChInfoList_04, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_AddReconfTransChInfoList_r9 },
  { &hf_rrc_frequencyInfo   , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_FrequencyInfo },
  { &hf_rrc_multi_frequencyInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_Multi_frequencyInfo_LCR_r7 },
  { &hf_rrc_dtx_drx_TimingInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DTX_DRX_TimingInfo_r7 },
  { &hf_rrc_dtx_drx_Info    , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DTX_DRX_Info_r7 },
  { &hf_rrc_hs_scch_LessInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_HS_SCCH_LessInfo_r7 },
  { &hf_rrc_mimoParameters_04, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MIMO_Parameters_r9 },
  { &hf_rrc_maxAllowedUL_TX_Power, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MaxAllowedUL_TX_Power },
  { &hf_rrc_ul_DPCH_Info_01 , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_DPCH_Info_r7 },
  { &hf_rrc_ul_EDCH_Information_04, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_EDCH_Information_r9 },
  { &hf_rrc_ul_SecondaryCellInfoFDD, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_SecondaryCellInfoFDD },
  { &hf_rrc_dl_HSPDSCH_Information_06, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_HSPDSCH_Information_r9 },
  { &hf_rrc_dl_CommonInformation_05, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_CommonInformation_r8 },
  { &hf_rrc_dl_InformationPerRL_List_05, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_InformationPerRL_List_r8 },
  { &hf_rrc_dl_SecondaryCellInfoFDD_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_SecondaryCellInfoFDD_r9 },
  { &hf_rrc_controlChannelDRXInfo_TDD128, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_ControlChannelDRXInfo_TDD128_r8 },
  { &hf_rrc_sps_Information_TDD128, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_SPS_Information_TDD128_r8 },
  { &hf_rrc_mbms_PL_ServiceRestrictInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MBMS_PL_ServiceRestrictInfo_r6 },
  { &hf_rrc_mbms_RB_ListReleasedToChangeTransferMode, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RB_InformationReleaseList },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_RadioBearerRelease_r9_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_RadioBearerRelease_r9_IEs, RadioBearerRelease_r9_IEs_sequence);

  return offset;
}


static const per_sequence_t RadioBearerRelease_v950ext_IEs_sequence[] = {
  { &hf_rrc_secondaryCellMimoParameters, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_SecondaryCellMIMOparametersFDD_v950ext },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_RadioBearerRelease_v950ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_RadioBearerRelease_v950ext_IEs, RadioBearerRelease_v950ext_IEs_sequence);

  return offset;
}


static const per_sequence_t RadioBearerRelease_v970ext_IEs_sequence[] = {
  { &hf_rrc_cellDCHMeasOccasionInfo_TDD128, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CellDCHMeasOccasionInfo_TDD128_r9 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_RadioBearerRelease_v970ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_RadioBearerRelease_v970ext_IEs, RadioBearerRelease_v970ext_IEs_sequence);

  return offset;
}


static const per_sequence_t RadioBearerRelease_v9c0ext_IEs_sequence[] = {
  { &hf_rrc_dl_AddReconfTransChInfoList_05, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_AddReconfTransChInfoList_TDD128_v9c0ext },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_RadioBearerRelease_v9c0ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_RadioBearerRelease_v9c0ext_IEs, RadioBearerRelease_v9c0ext_IEs_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions_101_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_nonCriticalExtensions_101(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_nonCriticalExtensions_101, T_nonCriticalExtensions_101_sequence);

  return offset;
}


static const per_sequence_t T_v9c0NonCriticalExtensions_06_sequence[] = {
  { &hf_rrc_radioBearerRelease_v9c0ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RadioBearerRelease_v9c0ext_IEs },
  { &hf_rrc_nonCriticalExtensions_101, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_nonCriticalExtensions_101 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v9c0NonCriticalExtensions_06(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v9c0NonCriticalExtensions_06, T_v9c0NonCriticalExtensions_06_sequence);

  return offset;
}


static const per_sequence_t T_v970NonCriticalExtensions_02_sequence[] = {
  { &hf_rrc_radioBearerRelease_v970ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RadioBearerRelease_v970ext_IEs },
  { &hf_rrc_v9c0NonCriticalExtensions_06, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v9c0NonCriticalExtensions_06 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v970NonCriticalExtensions_02(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v970NonCriticalExtensions_02, T_v970NonCriticalExtensions_02_sequence);

  return offset;
}


static const per_sequence_t T_v950NonCriticalExtensions_05_sequence[] = {
  { &hf_rrc_radioBearerRelease_v950ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RadioBearerRelease_v950ext_IEs },
  { &hf_rrc_v970NonCriticalExtensions_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v970NonCriticalExtensions_02 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v950NonCriticalExtensions_05(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v950NonCriticalExtensions_05, T_v950NonCriticalExtensions_05_sequence);

  return offset;
}


static const per_sequence_t T_r9_07_sequence[] = {
  { &hf_rrc_radioBearerRelease_r9, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RadioBearerRelease_r9_IEs },
  { &hf_rrc_radioBearerRelease_r9_add_ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_BIT_STRING },
  { &hf_rrc_v950NonCriticalExtensions_05, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v950NonCriticalExtensions_05 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_r9_07(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_r9_07, T_r9_07_sequence);

  return offset;
}


static const per_sequence_t RadioBearerRelease_r10_IEs_sequence[] = {
  { &hf_rrc_integrityProtectionModeInfo_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_IntegrityProtectionModeInfo_r7 },
  { &hf_rrc_cipheringModeInfo_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CipheringModeInfo_r7 },
  { &hf_rrc_activationTime  , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_ActivationTime },
  { &hf_rrc_new_U_RNTI      , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_U_RNTI },
  { &hf_rrc_new_C_RNTI      , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_C_RNTI },
  { &hf_rrc_new_DSCH_RNTI   , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DSCH_RNTI },
  { &hf_rrc_new_H_RNTI      , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_H_RNTI },
  { &hf_rrc_newPrimary_E_RNTI, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_E_RNTI },
  { &hf_rrc_newSecondary_E_RNTI, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_E_RNTI },
  { &hf_rrc_rrc_StateIndicator, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RRC_StateIndicator },
  { &hf_rrc_ueMobilityStateIndicator, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_High_MobilityDetected },
  { &hf_rrc_utran_DRX_CycleLengthCoeff_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UTRAN_DRX_CycleLengthCoefficient_r7 },
  { &hf_rrc_cn_InformationInfo_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CN_InformationInfo_r6 },
  { &hf_rrc_signallingConnectionRelIndication, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CN_DomainIdentity },
  { &hf_rrc_ura_Identity    , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_URA_Identity },
  { &hf_rrc_supportForChangeOfUE_Capability, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_BOOLEAN },
  { &hf_rrc_rab_InformationReconfigList_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RAB_InformationReconfigList_r8 },
  { &hf_rrc_rb_InformationReleaseList, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RB_InformationReleaseList },
  { &hf_rrc_rb_InformationReconfigList_05, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RB_InformationReconfigList_r8 },
  { &hf_rrc_rb_InformationAffectedList_04, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RB_InformationAffectedList_r8 },
  { &hf_rrc_dl_CounterSynchronisationInfo_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_CounterSynchronisationInfo_r5 },
  { &hf_rrc_ul_CommonTransChInfo_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_CommonTransChInfo_r4 },
  { &hf_rrc_ul_deletedTransChInfoList_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_DeletedTransChInfoList_r6 },
  { &hf_rrc_ul_AddReconfTransChInfoList_03, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_AddReconfTransChInfoList_r8 },
  { &hf_rrc_dl_CommonTransChInfo_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_CommonTransChInfo_r4 },
  { &hf_rrc_dl_DeletedTransChInfoList_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_DeletedTransChInfoList_r7 },
  { &hf_rrc_dl_AddReconfTransChInfoList_04, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_AddReconfTransChInfoList_r9 },
  { &hf_rrc_frequencyInfo   , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_FrequencyInfo },
  { &hf_rrc_multi_frequencyInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_Multi_frequencyInfo_LCR_r7 },
  { &hf_rrc_dtx_drx_TimingInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DTX_DRX_TimingInfo_r7 },
  { &hf_rrc_dtx_drx_Info    , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DTX_DRX_Info_r7 },
  { &hf_rrc_hs_scch_LessInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_HS_SCCH_LessInfo_r7 },
  { &hf_rrc_mimoParameters_04, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MIMO_Parameters_r9 },
  { &hf_rrc_maxAllowedUL_TX_Power, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MaxAllowedUL_TX_Power },
  { &hf_rrc_ul_DPCH_Info_01 , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_DPCH_Info_r7 },
  { &hf_rrc_ul_EDCH_Information_04, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_EDCH_Information_r9 },
  { &hf_rrc_ul_SecondaryCellInfoFDD, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_SecondaryCellInfoFDD },
  { &hf_rrc_ul_MulticarrierEDCHInfo_TDD128, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_MulticarrierEDCHInfo_TDD128 },
  { &hf_rrc_dl_HSPDSCH_Information_06, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_HSPDSCH_Information_r9 },
  { &hf_rrc_dl_CommonInformation_06, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_CommonInformation_r10 },
  { &hf_rrc_dl_InformationPerRL_List_05, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_InformationPerRL_List_r8 },
  { &hf_rrc_dl_SecondaryCellInfoFDD_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_SecondaryCellInfoFDD_r10 },
  { &hf_rrc_additionalDLSecCellInfoListFDD, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_AdditionalDLSecCellInfoListFDD },
  { &hf_rrc_controlChannelDRXInfo_TDD128, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_ControlChannelDRXInfo_TDD128_r8 },
  { &hf_rrc_sps_Information_TDD128, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_SPS_Information_TDD128_r8 },
  { &hf_rrc_mu_MIMO_Info_TDD128, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MU_MIMO_Info_TDD128 },
  { &hf_rrc_mbms_PL_ServiceRestrictInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MBMS_PL_ServiceRestrictInfo_r6 },
  { &hf_rrc_mbms_RB_ListReleasedToChangeTransferMode, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RB_InformationReleaseList },
  { &hf_rrc_cellDCHMeasOccasionInfo_TDD128, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CellDCHMeasOccasionInfo_TDD128_r9 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_RadioBearerRelease_r10_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_RadioBearerRelease_r10_IEs, RadioBearerRelease_r10_IEs_sequence);

  return offset;
}


static const per_sequence_t RadioBearerRelease_vbxyext_IEs_sequence[] = {
  { &hf_rrc_ulOLTDInfoFDD   , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_OLTD_InfoFDD },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_RadioBearerRelease_vbxyext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_RadioBearerRelease_vbxyext_IEs, RadioBearerRelease_vbxyext_IEs_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions_102_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_nonCriticalExtensions_102(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_nonCriticalExtensions_102, T_nonCriticalExtensions_102_sequence);

  return offset;
}


static const per_sequence_t T_vbxyNonCriticalExtensions_10_sequence[] = {
  { &hf_rrc_radioBearerRelease_vbxyext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RadioBearerRelease_vbxyext_IEs },
  { &hf_rrc_nonCriticalExtensions_102, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_nonCriticalExtensions_102 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_vbxyNonCriticalExtensions_10(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_vbxyNonCriticalExtensions_10, T_vbxyNonCriticalExtensions_10_sequence);

  return offset;
}


static const per_sequence_t T_v9c0NonCriticalExtensions_07_sequence[] = {
  { &hf_rrc_radioBearerRelease_v9c0ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RadioBearerRelease_v9c0ext_IEs },
  { &hf_rrc_vbxyNonCriticalExtensions_10, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_vbxyNonCriticalExtensions_10 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v9c0NonCriticalExtensions_07(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v9c0NonCriticalExtensions_07, T_v9c0NonCriticalExtensions_07_sequence);

  return offset;
}


static const per_sequence_t T_r10_07_sequence[] = {
  { &hf_rrc_radioBearerRelease_r10, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RadioBearerRelease_r10_IEs },
  { &hf_rrc_radioBearerRelease_r10_add_ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_BIT_STRING },
  { &hf_rrc_v9c0NonCriticalExtensions_07, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v9c0NonCriticalExtensions_07 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_r10_07(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_r10_07, T_r10_07_sequence);

  return offset;
}


static const per_sequence_t RadioBearerRelease_r11_IEs_sequence[] = {
  { &hf_rrc_integrityProtectionModeInfo_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_IntegrityProtectionModeInfo_r7 },
  { &hf_rrc_cipheringModeInfo_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CipheringModeInfo_r7 },
  { &hf_rrc_activationTime  , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_ActivationTime },
  { &hf_rrc_new_U_RNTI      , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_U_RNTI },
  { &hf_rrc_new_C_RNTI      , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_C_RNTI },
  { &hf_rrc_new_DSCH_RNTI   , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DSCH_RNTI },
  { &hf_rrc_new_H_RNTI      , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_H_RNTI },
  { &hf_rrc_newPrimary_E_RNTI, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_E_RNTI },
  { &hf_rrc_newSecondary_E_RNTI, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_E_RNTI },
  { &hf_rrc_rrc_StateIndicator, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RRC_StateIndicator },
  { &hf_rrc_ueMobilityStateIndicator, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_High_MobilityDetected },
  { &hf_rrc_utran_DRX_CycleLengthCoeff_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UTRAN_DRX_CycleLengthCoefficient_r7 },
  { &hf_rrc_cn_InformationInfo_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CN_InformationInfo_r6 },
  { &hf_rrc_signallingConnectionRelIndication, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CN_DomainIdentity },
  { &hf_rrc_ura_Identity    , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_URA_Identity },
  { &hf_rrc_supportForChangeOfUE_Capability, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_BOOLEAN },
  { &hf_rrc_rab_InformationReconfigList_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RAB_InformationReconfigList_r8 },
  { &hf_rrc_rb_InformationReleaseList, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RB_InformationReleaseList },
  { &hf_rrc_rb_InformationReconfigList_06, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RB_InformationReconfigList_r11 },
  { &hf_rrc_rb_InformationAffectedList_04, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RB_InformationAffectedList_r8 },
  { &hf_rrc_dl_CounterSynchronisationInfo_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_CounterSynchronisationInfo_r5 },
  { &hf_rrc_ul_CommonTransChInfo_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_CommonTransChInfo_r4 },
  { &hf_rrc_ul_deletedTransChInfoList_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_DeletedTransChInfoList_r6 },
  { &hf_rrc_ul_AddReconfTransChInfoList_03, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_AddReconfTransChInfoList_r8 },
  { &hf_rrc_dl_CommonTransChInfo_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_CommonTransChInfo_r4 },
  { &hf_rrc_dl_DeletedTransChInfoList_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_DeletedTransChInfoList_r7 },
  { &hf_rrc_dl_AddReconfTransChInfoList_06, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_AddReconfTransChInfoList_r11 },
  { &hf_rrc_frequencyInfo   , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_FrequencyInfo },
  { &hf_rrc_multi_frequencyInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_Multi_frequencyInfo_LCR_r7 },
  { &hf_rrc_dtx_drx_TimingInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DTX_DRX_TimingInfo_r7 },
  { &hf_rrc_dtx_drx_Info    , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DTX_DRX_Info_r7 },
  { &hf_rrc_hs_scch_LessInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_HS_SCCH_LessInfo_r7 },
  { &hf_rrc_mimoParameters_04, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MIMO_Parameters_r9 },
  { &hf_rrc_mimo4x4Parameters, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MIMO4x4_Parameters },
  { &hf_rrc_maxAllowedUL_TX_Power, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MaxAllowedUL_TX_Power },
  { &hf_rrc_ul_DPCH_Info_02 , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_DPCH_Info_r11 },
  { &hf_rrc_ul_EDCH_Information_05, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_EDCH_Information_r11 },
  { &hf_rrc_ul_SecondaryCellInfoFDD, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_SecondaryCellInfoFDD },
  { &hf_rrc_ul_MulticarrierEDCHInfo_TDD128, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_MulticarrierEDCHInfo_TDD128 },
  { &hf_rrc_ul_CLTD_InfoFDD , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_CLTD_InfoFDD },
  { &hf_rrc_ul_OLTD_InfoFDD , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_OLTD_InfoFDD },
  { &hf_rrc_dl_HSPDSCH_Information_07, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_HSPDSCH_Information_r11 },
  { &hf_rrc_dl_CommonInformation_07, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_CommonInformation_r11 },
  { &hf_rrc_dl_InformationPerRL_List_06, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_InformationPerRL_List_r11 },
  { &hf_rrc_dl_SecondaryCellInfoFDD_03, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_SecondaryCellInfoFDD_r11 },
  { &hf_rrc_additionalDLSecCellInfoListFDD_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_AdditionalDLSecCellInfoListFDD_r11 },
  { &hf_rrc_additionalDLSecCellInfoListFDD2, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_AdditionalDLSecCellInfoListFDD2_r11 },
  { &hf_rrc_commonERGCHInfoFDD, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CommonERGCHInfoFDD },
  { &hf_rrc_controlChannelDRXInfo_TDD128, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_ControlChannelDRXInfo_TDD128_r8 },
  { &hf_rrc_sps_Information_TDD128, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_SPS_Information_TDD128_r8 },
  { &hf_rrc_mu_MIMO_Info_TDD128, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MU_MIMO_Info_TDD128 },
  { &hf_rrc_mbms_PL_ServiceRestrictInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MBMS_PL_ServiceRestrictInfo_r6 },
  { &hf_rrc_mbms_RB_ListReleasedToChangeTransferMode, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RB_InformationReleaseList },
  { &hf_rrc_cellDCHMeasOccasionInfo_TDD128, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CellDCHMeasOccasionInfo_TDD128_r9 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_RadioBearerRelease_r11_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_RadioBearerRelease_r11_IEs, RadioBearerRelease_r11_IEs_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions_103_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_nonCriticalExtensions_103(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_nonCriticalExtensions_103, T_nonCriticalExtensions_103_sequence);

  return offset;
}


static const per_sequence_t T_r11_08_sequence[] = {
  { &hf_rrc_radioBearerRelease_r11, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RadioBearerRelease_r11_IEs },
  { &hf_rrc_radioBearerRelease_r11_add_ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_BIT_STRING },
  { &hf_rrc_nonCriticalExtensions_103, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_nonCriticalExtensions_103 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_r11_08(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_r11_08, T_r11_08_sequence);

  return offset;
}


static const per_sequence_t T_criticalExtensions_81_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_criticalExtensions_81(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_criticalExtensions_81, T_criticalExtensions_81_sequence);

  return offset;
}


static const value_string rrc_T_criticalExtensions_80_vals[] = {
  {   0, "r11" },
  {   1, "criticalExtensions" },
  { 0, NULL }
};

static const per_choice_t T_criticalExtensions_80_choice[] = {
  {   0, &hf_rrc_r11_08          , ASN1_NO_EXTENSIONS     , dissect_rrc_T_r11_08 },
  {   1, &hf_rrc_criticalExtensions_82, ASN1_NO_EXTENSIONS     , dissect_rrc_T_criticalExtensions_81 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_criticalExtensions_80(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_criticalExtensions_80, T_criticalExtensions_80_choice,
                                 NULL);

  return offset;
}


static const value_string rrc_T_criticalExtensions_79_vals[] = {
  {   0, "r10" },
  {   1, "criticalExtensions" },
  { 0, NULL }
};

static const per_choice_t T_criticalExtensions_79_choice[] = {
  {   0, &hf_rrc_r10_07          , ASN1_NO_EXTENSIONS     , dissect_rrc_T_r10_07 },
  {   1, &hf_rrc_criticalExtensions_81, ASN1_NO_EXTENSIONS     , dissect_rrc_T_criticalExtensions_80 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_criticalExtensions_79(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_criticalExtensions_79, T_criticalExtensions_79_choice,
                                 NULL);

  return offset;
}


static const value_string rrc_T_criticalExtensions_78_vals[] = {
  {   0, "r9" },
  {   1, "criticalExtensions" },
  { 0, NULL }
};

static const per_choice_t T_criticalExtensions_78_choice[] = {
  {   0, &hf_rrc_r9_07           , ASN1_NO_EXTENSIONS     , dissect_rrc_T_r9_07 },
  {   1, &hf_rrc_criticalExtensions_80, ASN1_NO_EXTENSIONS     , dissect_rrc_T_criticalExtensions_79 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_criticalExtensions_78(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_criticalExtensions_78, T_criticalExtensions_78_choice,
                                 NULL);

  return offset;
}


static const value_string rrc_T_criticalExtensions_77_vals[] = {
  {   0, "r8" },
  {   1, "criticalExtensions" },
  { 0, NULL }
};

static const per_choice_t T_criticalExtensions_77_choice[] = {
  {   0, &hf_rrc_r8_08           , ASN1_NO_EXTENSIONS     , dissect_rrc_T_r8_08 },
  {   1, &hf_rrc_criticalExtensions_79, ASN1_NO_EXTENSIONS     , dissect_rrc_T_criticalExtensions_78 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_criticalExtensions_77(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_criticalExtensions_77, T_criticalExtensions_77_choice,
                                 NULL);

  return offset;
}


static const value_string rrc_T_criticalExtensions_76_vals[] = {
  {   0, "r7" },
  {   1, "criticalExtensions" },
  { 0, NULL }
};

static const per_choice_t T_criticalExtensions_76_choice[] = {
  {   0, &hf_rrc_r7_07           , ASN1_NO_EXTENSIONS     , dissect_rrc_T_r7_07 },
  {   1, &hf_rrc_criticalExtensions_78, ASN1_NO_EXTENSIONS     , dissect_rrc_T_criticalExtensions_77 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_criticalExtensions_76(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_criticalExtensions_76, T_criticalExtensions_76_choice,
                                 NULL);

  return offset;
}


static const value_string rrc_T_criticalExtensions_75_vals[] = {
  {   0, "r6" },
  {   1, "criticalExtensions" },
  { 0, NULL }
};

static const per_choice_t T_criticalExtensions_75_choice[] = {
  {   0, &hf_rrc_r6_08           , ASN1_NO_EXTENSIONS     , dissect_rrc_T_r6_08 },
  {   1, &hf_rrc_criticalExtensions_77, ASN1_NO_EXTENSIONS     , dissect_rrc_T_criticalExtensions_76 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_criticalExtensions_75(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_criticalExtensions_75, T_criticalExtensions_75_choice,
                                 NULL);

  return offset;
}


static const value_string rrc_T_criticalExtensions_74_vals[] = {
  {   0, "r5" },
  {   1, "criticalExtensions" },
  { 0, NULL }
};

static const per_choice_t T_criticalExtensions_74_choice[] = {
  {   0, &hf_rrc_r5_06           , ASN1_NO_EXTENSIONS     , dissect_rrc_T_r5_06 },
  {   1, &hf_rrc_criticalExtensions_76, ASN1_NO_EXTENSIONS     , dissect_rrc_T_criticalExtensions_75 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_criticalExtensions_74(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_criticalExtensions_74, T_criticalExtensions_74_choice,
                                 NULL);

  return offset;
}


static const value_string rrc_T_criticalExtensions_73_vals[] = {
  {   0, "r4" },
  {   1, "criticalExtensions" },
  { 0, NULL }
};

static const per_choice_t T_criticalExtensions_73_choice[] = {
  {   0, &hf_rrc_r4_07           , ASN1_NO_EXTENSIONS     , dissect_rrc_T_r4_07 },
  {   1, &hf_rrc_criticalExtensions_75, ASN1_NO_EXTENSIONS     , dissect_rrc_T_criticalExtensions_74 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_criticalExtensions_73(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_criticalExtensions_73, T_criticalExtensions_73_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t T_later_than_r3_13_sequence[] = {
  { &hf_rrc_rrc_TransactionIdentifier, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RRC_TransactionIdentifier },
  { &hf_rrc_criticalExtensions_74, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_criticalExtensions_73 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_later_than_r3_13(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_later_than_r3_13, T_later_than_r3_13_sequence);

  return offset;
}


static const value_string rrc_RadioBearerRelease_vals[] = {
  {   0, "r3" },
  {   1, "later-than-r3" },
  { 0, NULL }
};

static const per_choice_t RadioBearerRelease_choice[] = {
  {   0, &hf_rrc_r3_15           , ASN1_NO_EXTENSIONS     , dissect_rrc_T_r3_15 },
  {   1, &hf_rrc_later_than_r3_13, ASN1_NO_EXTENSIONS     , dissect_rrc_T_later_than_r3_13 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_RadioBearerRelease(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
#line 337 "../../asn1/rrc/rrc.cnf"
  col_append_str(actx->pinfo->cinfo, COL_INFO, "RadioBearerRelease");
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_RadioBearerRelease, RadioBearerRelease_choice,
                                 NULL);




  return offset;
}


static const value_string rrc_RLC_InfoChoice_vals[] = {
  {   0, "rlc-Info" },
  {   1, "same-as-RB" },
  { 0, NULL }
};

static const per_choice_t RLC_InfoChoice_choice[] = {
  {   0, &hf_rrc_rlc_Info        , ASN1_NO_EXTENSIONS     , dissect_rrc_RLC_Info },
  {   1, &hf_rrc_same_as_RB      , ASN1_NO_EXTENSIONS     , dissect_rrc_RB_Identity },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_RLC_InfoChoice(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_RLC_InfoChoice, RLC_InfoChoice_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t SRB_InformationSetup_sequence[] = {
  { &hf_rrc_rb_Identity     , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RB_Identity },
  { &hf_rrc_rlc_InfoChoice  , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RLC_InfoChoice },
  { &hf_rrc_rb_MappingInfo  , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RB_MappingInfo },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_SRB_InformationSetup(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_SRB_InformationSetup, SRB_InformationSetup_sequence);

  return offset;
}


static const per_sequence_t SRB_InformationSetupList_sequence_of[1] = {
  { &hf_rrc_SRB_InformationSetupList_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SRB_InformationSetup },
};

static int
dissect_rrc_SRB_InformationSetupList(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_SRB_InformationSetupList, SRB_InformationSetupList_sequence_of,
                                                  1, maxSRBsetup, FALSE);

  return offset;
}


static const per_sequence_t RB_InformationSetup_sequence[] = {
  { &hf_rrc_rb_Identity     , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RB_Identity },
  { &hf_rrc_pdcp_Info       , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_PDCP_Info },
  { &hf_rrc_rlc_InfoChoice  , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RLC_InfoChoice },
  { &hf_rrc_rb_MappingInfo  , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RB_MappingInfo },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_RB_InformationSetup(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_RB_InformationSetup, RB_InformationSetup_sequence);

  return offset;
}


static const per_sequence_t RB_InformationSetupList_sequence_of[1] = {
  { &hf_rrc_RB_InformationSetupList_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RB_InformationSetup },
};

static int
dissect_rrc_RB_InformationSetupList(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_RB_InformationSetupList, RB_InformationSetupList_sequence_of,
                                                  1, maxRBperRAB, FALSE);

  return offset;
}


static const per_sequence_t RAB_InformationSetup_sequence[] = {
  { &hf_rrc_rab_Info_02     , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RAB_Info },
  { &hf_rrc_rb_InformationSetupList, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RB_InformationSetupList },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_RAB_InformationSetup(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_RAB_InformationSetup, RAB_InformationSetup_sequence);

  return offset;
}


static const per_sequence_t RAB_InformationSetupList_sequence_of[1] = {
  { &hf_rrc_RAB_InformationSetupList_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RAB_InformationSetup },
};

static int
dissect_rrc_RAB_InformationSetupList(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_RAB_InformationSetupList, RAB_InformationSetupList_sequence_of,
                                                  1, maxRABsetup, FALSE);

  return offset;
}


static const per_sequence_t T_fdd_33_sequence[] = {
  { &hf_rrc_dummy1          , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CPCH_SetID },
  { &hf_rrc_dummy2_02       , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DRAC_StaticInformationList },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_fdd_33(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_fdd_33, T_fdd_33_sequence);

  return offset;
}


static const value_string rrc_T_dummy_07_vals[] = {
  {   0, "fdd" },
  {   1, "tdd" },
  { 0, NULL }
};

static const per_choice_t T_dummy_07_choice[] = {
  {   0, &hf_rrc_fdd_33          , ASN1_NO_EXTENSIONS     , dissect_rrc_T_fdd_33 },
  {   1, &hf_rrc_tdd             , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_dummy_07(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_dummy_07, T_dummy_07_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t T_fdd_34_sequence[] = {
  { &hf_rrc_dummy_06        , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_PDSCH_Information },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_fdd_34(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_fdd_34, T_fdd_34_sequence);

  return offset;
}


static const value_string rrc_T_modeSpecificPhysChInfo_09_vals[] = {
  {   0, "fdd" },
  {   1, "tdd" },
  { 0, NULL }
};

static const per_choice_t T_modeSpecificPhysChInfo_09_choice[] = {
  {   0, &hf_rrc_fdd_34          , ASN1_NO_EXTENSIONS     , dissect_rrc_T_fdd_34 },
  {   1, &hf_rrc_tdd             , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_modeSpecificPhysChInfo_09(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_modeSpecificPhysChInfo_09, T_modeSpecificPhysChInfo_09_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t RadioBearerSetup_r3_IEs_sequence[] = {
  { &hf_rrc_rrc_TransactionIdentifier, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RRC_TransactionIdentifier },
  { &hf_rrc_integrityProtectionModeInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_IntegrityProtectionModeInfo },
  { &hf_rrc_cipheringModeInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CipheringModeInfo },
  { &hf_rrc_activationTime  , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_ActivationTime },
  { &hf_rrc_new_U_RNTI      , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_U_RNTI },
  { &hf_rrc_new_C_RNTI      , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_C_RNTI },
  { &hf_rrc_rrc_StateIndicator, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RRC_StateIndicator },
  { &hf_rrc_utran_DRX_CycleLengthCoeff, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UTRAN_DRX_CycleLengthCoefficient },
  { &hf_rrc_ura_Identity    , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_URA_Identity },
  { &hf_rrc_cn_InformationInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CN_InformationInfo },
  { &hf_rrc_srb_InformationSetupList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_SRB_InformationSetupList },
  { &hf_rrc_rab_InformationSetupList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RAB_InformationSetupList },
  { &hf_rrc_rb_InformationAffectedList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RB_InformationAffectedList },
  { &hf_rrc_dl_CounterSynchronisationInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_CounterSynchronisationInfo },
  { &hf_rrc_ul_CommonTransChInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_CommonTransChInfo },
  { &hf_rrc_ul_deletedTransChInfoList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_DeletedTransChInfoList },
  { &hf_rrc_ul_AddReconfTransChInfoList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_AddReconfTransChInfoList },
  { &hf_rrc_dummy_20        , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_dummy_07 },
  { &hf_rrc_dl_CommonTransChInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_CommonTransChInfo },
  { &hf_rrc_dl_DeletedTransChInfoList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_DeletedTransChInfoList },
  { &hf_rrc_dl_AddReconfTransChInfoList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_AddReconfTransChInfoList },
  { &hf_rrc_frequencyInfo   , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_FrequencyInfo },
  { &hf_rrc_maxAllowedUL_TX_Power, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MaxAllowedUL_TX_Power },
  { &hf_rrc_ul_ChannelRequirement, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_ChannelRequirement },
  { &hf_rrc_modeSpecificPhysChInfo_09, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_modeSpecificPhysChInfo_09 },
  { &hf_rrc_dl_CommonInformation, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_CommonInformation },
  { &hf_rrc_dl_InformationPerRL_List, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_InformationPerRL_List },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_RadioBearerSetup_r3_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_RadioBearerSetup_r3_IEs, RadioBearerSetup_r3_IEs_sequence);

  return offset;
}


static const per_sequence_t RadioBearerSetup_v3a0ext_sequence[] = {
  { &hf_rrc_new_DSCH_RNTI   , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DSCH_RNTI },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_RadioBearerSetup_v3a0ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_RadioBearerSetup_v3a0ext, RadioBearerSetup_v3a0ext_sequence);

  return offset;
}


static const per_sequence_t RadioBearerSetup_v4b0ext_IEs_sequence[] = {
  { &hf_rrc_dummy_02        , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_SSDT_UL },
  { &hf_rrc_cell_id_PerRL_List, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CellIdentity_PerRL_List },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_RadioBearerSetup_v4b0ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_RadioBearerSetup_v4b0ext_IEs, RadioBearerSetup_v4b0ext_IEs_sequence);

  return offset;
}


static const per_sequence_t RadioBearerSetup_v590ext_IEs_sequence[] = {
  { &hf_rrc_dl_TPC_PowerOffsetPerRL_List, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_TPC_PowerOffsetPerRL_List },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_RadioBearerSetup_v590ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_RadioBearerSetup_v590ext_IEs, RadioBearerSetup_v590ext_IEs_sequence);

  return offset;
}


static const per_sequence_t RadioBearerSetup_v5d0ext_IEs_sequence[] = {
  { &hf_rrc_pdcp_ROHC_TargetMode, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_PDCP_ROHC_TargetMode },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_RadioBearerSetup_v5d0ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_RadioBearerSetup_v5d0ext_IEs, RadioBearerSetup_v5d0ext_IEs_sequence);

  return offset;
}


static const value_string rrc_T_postVerificationPeriod_04_vals[] = {
  {   0, "true" },
  { 0, NULL }
};


static int
dissect_rrc_T_postVerificationPeriod_04(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t RAB_Info_r6_ext_sequence[] = {
  { &hf_rrc_mbms_SessionIdentity, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MBMS_SessionIdentity },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_RAB_Info_r6_ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_RAB_Info_r6_ext, RAB_Info_r6_ext_sequence);

  return offset;
}


static const per_sequence_t RAB_InformationSetup_r6_ext_sequence[] = {
  { &hf_rrc_rab_Info_r6_ext , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RAB_Info_r6_ext },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_RAB_InformationSetup_r6_ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_RAB_InformationSetup_r6_ext, RAB_InformationSetup_r6_ext_sequence);

  return offset;
}


static const per_sequence_t RAB_InformationSetupList_r6_ext_sequence_of[1] = {
  { &hf_rrc_RAB_InformationSetupList_r6_ext_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RAB_InformationSetup_r6_ext },
};

static int
dissect_rrc_RAB_InformationSetupList_r6_ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_RAB_InformationSetupList_r6_ext, RAB_InformationSetupList_r6_ext_sequence_of,
                                                  1, maxRABsetup, FALSE);

  return offset;
}


static const per_sequence_t RadioBearerSetup_v690ext_IEs_sequence[] = {
  { &hf_rrc_primary_plmn_Identity, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_PLMN_Identity },
  { &hf_rrc_harq_Preamble_Mode, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_HARQ_Preamble_Mode },
  { &hf_rrc_beaconPLEst     , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_BEACON_PL_Est },
  { &hf_rrc_postVerificationPeriod_04, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_postVerificationPeriod_04 },
  { &hf_rrc_dhs_sync        , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DHS_Sync },
  { &hf_rrc_timingMaintainedSynchInd, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_TimingMaintainedSynchInd },
  { &hf_rrc_rab_InformationSetupList_08, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RAB_InformationSetupList_r6_ext },
  { &hf_rrc_mbms_PL_ServiceRestrictInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MBMS_PL_ServiceRestrictInfo_r6 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_RadioBearerSetup_v690ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_RadioBearerSetup_v690ext_IEs, RadioBearerSetup_v690ext_IEs_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions_106_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_nonCriticalExtensions_106(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_nonCriticalExtensions_106, T_nonCriticalExtensions_106_sequence);

  return offset;
}


static const per_sequence_t T_v690NonCriticalExtensions_22_sequence[] = {
  { &hf_rrc_radioBearerSetup_v690ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RadioBearerSetup_v690ext_IEs },
  { &hf_rrc_nonCriticalExtensions_106, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_nonCriticalExtensions_106 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v690NonCriticalExtensions_22(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v690NonCriticalExtensions_22, T_v690NonCriticalExtensions_22_sequence);

  return offset;
}


static const per_sequence_t T_v5d0NonCriticalExtenstions_09_sequence[] = {
  { &hf_rrc_radioBearerSetup_v5d0ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RadioBearerSetup_v5d0ext_IEs },
  { &hf_rrc_v690NonCriticalExtensions_22, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v690NonCriticalExtensions_22 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v5d0NonCriticalExtenstions_09(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v5d0NonCriticalExtenstions_09, T_v5d0NonCriticalExtenstions_09_sequence);

  return offset;
}


static const per_sequence_t T_v590NonCriticalExtensions_18_sequence[] = {
  { &hf_rrc_radioBearerSetup_v590ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RadioBearerSetup_v590ext_IEs },
  { &hf_rrc_v5d0NonCriticalExtenstions_09, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v5d0NonCriticalExtenstions_09 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v590NonCriticalExtensions_18(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v590NonCriticalExtensions_18, T_v590NonCriticalExtensions_18_sequence);

  return offset;
}


static const per_sequence_t T_v4b0NonCriticalExtensions_09_sequence[] = {
  { &hf_rrc_radioBearerSetup_v4b0ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RadioBearerSetup_v4b0ext_IEs },
  { &hf_rrc_v590NonCriticalExtensions_18, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v590NonCriticalExtensions_18 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v4b0NonCriticalExtensions_09(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v4b0NonCriticalExtensions_09, T_v4b0NonCriticalExtensions_09_sequence);

  return offset;
}


static const per_sequence_t T_laterNonCriticalExtensions_34_sequence[] = {
  { &hf_rrc_radioBearerSetup_r3_add_ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_BIT_STRING },
  { &hf_rrc_v4b0NonCriticalExtensions_09, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v4b0NonCriticalExtensions_09 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_laterNonCriticalExtensions_34(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_laterNonCriticalExtensions_34, T_laterNonCriticalExtensions_34_sequence);

  return offset;
}


static const per_sequence_t T_v3a0NonCriticalExtensions_07_sequence[] = {
  { &hf_rrc_radioBearerSetup_v3a0ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RadioBearerSetup_v3a0ext },
  { &hf_rrc_laterNonCriticalExtensions_34, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_laterNonCriticalExtensions_34 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v3a0NonCriticalExtensions_07(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v3a0NonCriticalExtensions_07, T_v3a0NonCriticalExtensions_07_sequence);

  return offset;
}


static const per_sequence_t T_r3_16_sequence[] = {
  { &hf_rrc_radioBearerSetup_r3, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RadioBearerSetup_r3_IEs },
  { &hf_rrc_v3a0NonCriticalExtensions_07, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v3a0NonCriticalExtensions_07 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_r3_16(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_r3_16, T_r3_16_sequence);

  return offset;
}


static const per_sequence_t RB_InformationSetup_r4_sequence[] = {
  { &hf_rrc_rb_Identity     , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RB_Identity },
  { &hf_rrc_pdcp_Info_01    , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_PDCP_Info_r4 },
  { &hf_rrc_rlc_InfoChoice  , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RLC_InfoChoice },
  { &hf_rrc_rb_MappingInfo  , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RB_MappingInfo },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_RB_InformationSetup_r4(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_RB_InformationSetup_r4, RB_InformationSetup_r4_sequence);

  return offset;
}


static const per_sequence_t RB_InformationSetupList_r4_sequence_of[1] = {
  { &hf_rrc_RB_InformationSetupList_r4_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RB_InformationSetup_r4 },
};

static int
dissect_rrc_RB_InformationSetupList_r4(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_RB_InformationSetupList_r4, RB_InformationSetupList_r4_sequence_of,
                                                  1, maxRBperRAB, FALSE);

  return offset;
}


static const per_sequence_t RAB_InformationSetup_r4_sequence[] = {
  { &hf_rrc_rab_Info_02     , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RAB_Info },
  { &hf_rrc_rb_InformationSetupList_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RB_InformationSetupList_r4 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_RAB_InformationSetup_r4(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_RAB_InformationSetup_r4, RAB_InformationSetup_r4_sequence);

  return offset;
}


static const per_sequence_t RAB_InformationSetupList_r4_sequence_of[1] = {
  { &hf_rrc_RAB_InformationSetupList_r4_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RAB_InformationSetup_r4 },
};

static int
dissect_rrc_RAB_InformationSetupList_r4(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_RAB_InformationSetupList_r4, RAB_InformationSetupList_r4_sequence_of,
                                                  1, maxRABsetup, FALSE);

  return offset;
}


static const per_sequence_t T_fdd_35_sequence[] = {
  { &hf_rrc_dummy1          , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CPCH_SetID },
  { &hf_rrc_dummy2_02       , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DRAC_StaticInformationList },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_fdd_35(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_fdd_35, T_fdd_35_sequence);

  return offset;
}


static const value_string rrc_T_dummy_08_vals[] = {
  {   0, "fdd" },
  {   1, "tdd" },
  { 0, NULL }
};

static const per_choice_t T_dummy_08_choice[] = {
  {   0, &hf_rrc_fdd_35          , ASN1_NO_EXTENSIONS     , dissect_rrc_T_fdd_35 },
  {   1, &hf_rrc_tdd             , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_dummy_08(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_dummy_08, T_dummy_08_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t T_fdd_36_sequence[] = {
  { &hf_rrc_dummy_06        , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_PDSCH_Information },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_fdd_36(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_fdd_36, T_fdd_36_sequence);

  return offset;
}


static const value_string rrc_T_modeSpecificPhysChInfo_10_vals[] = {
  {   0, "fdd" },
  {   1, "tdd" },
  { 0, NULL }
};

static const per_choice_t T_modeSpecificPhysChInfo_10_choice[] = {
  {   0, &hf_rrc_fdd_36          , ASN1_NO_EXTENSIONS     , dissect_rrc_T_fdd_36 },
  {   1, &hf_rrc_tdd             , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_modeSpecificPhysChInfo_10(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_modeSpecificPhysChInfo_10, T_modeSpecificPhysChInfo_10_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t RadioBearerSetup_r4_IEs_sequence[] = {
  { &hf_rrc_integrityProtectionModeInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_IntegrityProtectionModeInfo },
  { &hf_rrc_cipheringModeInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CipheringModeInfo },
  { &hf_rrc_activationTime  , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_ActivationTime },
  { &hf_rrc_new_U_RNTI      , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_U_RNTI },
  { &hf_rrc_new_C_RNTI      , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_C_RNTI },
  { &hf_rrc_new_DSCH_RNTI   , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DSCH_RNTI },
  { &hf_rrc_rrc_StateIndicator, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RRC_StateIndicator },
  { &hf_rrc_utran_DRX_CycleLengthCoeff, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UTRAN_DRX_CycleLengthCoefficient },
  { &hf_rrc_ura_Identity    , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_URA_Identity },
  { &hf_rrc_cn_InformationInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CN_InformationInfo },
  { &hf_rrc_srb_InformationSetupList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_SRB_InformationSetupList },
  { &hf_rrc_rab_InformationSetupList_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RAB_InformationSetupList_r4 },
  { &hf_rrc_rb_InformationAffectedList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RB_InformationAffectedList },
  { &hf_rrc_dl_CounterSynchronisationInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_CounterSynchronisationInfo },
  { &hf_rrc_ul_CommonTransChInfo_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_CommonTransChInfo_r4 },
  { &hf_rrc_ul_deletedTransChInfoList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_DeletedTransChInfoList },
  { &hf_rrc_ul_AddReconfTransChInfoList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_AddReconfTransChInfoList },
  { &hf_rrc_dummy_21        , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_dummy_08 },
  { &hf_rrc_dl_CommonTransChInfo_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_CommonTransChInfo_r4 },
  { &hf_rrc_dl_DeletedTransChInfoList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_DeletedTransChInfoList },
  { &hf_rrc_dl_AddReconfTransChInfoList_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_AddReconfTransChInfoList_r4 },
  { &hf_rrc_frequencyInfo   , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_FrequencyInfo },
  { &hf_rrc_maxAllowedUL_TX_Power, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MaxAllowedUL_TX_Power },
  { &hf_rrc_ul_ChannelRequirement_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_ChannelRequirement_r4 },
  { &hf_rrc_modeSpecificPhysChInfo_10, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_modeSpecificPhysChInfo_10 },
  { &hf_rrc_dl_CommonInformation_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_CommonInformation_r4 },
  { &hf_rrc_dl_InformationPerRL_List_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_InformationPerRL_List_r4 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_RadioBearerSetup_r4_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_RadioBearerSetup_r4_IEs, RadioBearerSetup_r4_IEs_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions_107_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_nonCriticalExtensions_107(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_nonCriticalExtensions_107, T_nonCriticalExtensions_107_sequence);

  return offset;
}


static const per_sequence_t T_v690NonCriticalExtensions_23_sequence[] = {
  { &hf_rrc_radioBearerSetup_v690ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RadioBearerSetup_v690ext_IEs },
  { &hf_rrc_nonCriticalExtensions_107, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_nonCriticalExtensions_107 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v690NonCriticalExtensions_23(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v690NonCriticalExtensions_23, T_v690NonCriticalExtensions_23_sequence);

  return offset;
}


static const per_sequence_t T_v5d0NonCriticalExtenstions_10_sequence[] = {
  { &hf_rrc_radioBearerSetup_v5d0ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RadioBearerSetup_v5d0ext_IEs },
  { &hf_rrc_v690NonCriticalExtensions_23, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v690NonCriticalExtensions_23 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v5d0NonCriticalExtenstions_10(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v5d0NonCriticalExtenstions_10, T_v5d0NonCriticalExtenstions_10_sequence);

  return offset;
}


static const per_sequence_t T_v590NonCriticalExtensions_19_sequence[] = {
  { &hf_rrc_radioBearerSetup_v590ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RadioBearerSetup_v590ext_IEs },
  { &hf_rrc_v5d0NonCriticalExtenstions_10, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v5d0NonCriticalExtenstions_10 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v590NonCriticalExtensions_19(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v590NonCriticalExtensions_19, T_v590NonCriticalExtensions_19_sequence);

  return offset;
}


static const per_sequence_t T_v4d0NonCriticalExtensions_08_sequence[] = {
  { &hf_rrc_radioBearerSetup_r4_add_ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_BIT_STRING },
  { &hf_rrc_v590NonCriticalExtensions_19, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v590NonCriticalExtensions_19 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v4d0NonCriticalExtensions_08(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v4d0NonCriticalExtensions_08, T_v4d0NonCriticalExtensions_08_sequence);

  return offset;
}


static const per_sequence_t T_r4_08_sequence[] = {
  { &hf_rrc_radioBearerSetup_r4, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RadioBearerSetup_r4_IEs },
  { &hf_rrc_v4d0NonCriticalExtensions_08, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v4d0NonCriticalExtensions_08 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_r4_08(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_r4_08, T_r4_08_sequence);

  return offset;
}


static const value_string rrc_RLC_InfoChoice_r5_vals[] = {
  {   0, "rlc-Info" },
  {   1, "same-as-RB" },
  { 0, NULL }
};

static const per_choice_t RLC_InfoChoice_r5_choice[] = {
  {   0, &hf_rrc_rlc_Info_01     , ASN1_NO_EXTENSIONS     , dissect_rrc_RLC_Info_r5 },
  {   1, &hf_rrc_same_as_RB      , ASN1_NO_EXTENSIONS     , dissect_rrc_RB_Identity },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_RLC_InfoChoice_r5(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_RLC_InfoChoice_r5, RLC_InfoChoice_r5_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t SRB_InformationSetup_r5_sequence[] = {
  { &hf_rrc_rb_Identity     , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RB_Identity },
  { &hf_rrc_rlc_InfoChoice_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RLC_InfoChoice_r5 },
  { &hf_rrc_rb_MappingInfo_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RB_MappingInfo_r5 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_SRB_InformationSetup_r5(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_SRB_InformationSetup_r5, SRB_InformationSetup_r5_sequence);

  return offset;
}


static const per_sequence_t SRB_InformationSetupList_r5_sequence_of[1] = {
  { &hf_rrc_SRB_InformationSetupList_r5_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SRB_InformationSetup_r5 },
};

static int
dissect_rrc_SRB_InformationSetupList_r5(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_SRB_InformationSetupList_r5, SRB_InformationSetupList_r5_sequence_of,
                                                  1, maxSRBsetup, FALSE);

  return offset;
}


static const per_sequence_t RB_InformationSetup_r5_sequence[] = {
  { &hf_rrc_rb_Identity     , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RB_Identity },
  { &hf_rrc_pdcp_Info_01    , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_PDCP_Info_r4 },
  { &hf_rrc_rlc_InfoChoice_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RLC_InfoChoice_r5 },
  { &hf_rrc_rb_MappingInfo_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RB_MappingInfo_r5 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_RB_InformationSetup_r5(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_RB_InformationSetup_r5, RB_InformationSetup_r5_sequence);

  return offset;
}


static const per_sequence_t RB_InformationSetupList_r5_sequence_of[1] = {
  { &hf_rrc_RB_InformationSetupList_r5_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RB_InformationSetup_r5 },
};

static int
dissect_rrc_RB_InformationSetupList_r5(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_RB_InformationSetupList_r5, RB_InformationSetupList_r5_sequence_of,
                                                  1, maxRBperRAB, FALSE);

  return offset;
}


static const per_sequence_t RAB_InformationSetup_r5_sequence[] = {
  { &hf_rrc_rab_Info_02     , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RAB_Info },
  { &hf_rrc_rb_InformationSetupList_02, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RB_InformationSetupList_r5 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_RAB_InformationSetup_r5(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_RAB_InformationSetup_r5, RAB_InformationSetup_r5_sequence);

  return offset;
}


static const per_sequence_t RAB_InformationSetupList_r5_sequence_of[1] = {
  { &hf_rrc_RAB_InformationSetupList_r5_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RAB_InformationSetup_r5 },
};

static int
dissect_rrc_RAB_InformationSetupList_r5(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_RAB_InformationSetupList_r5, RAB_InformationSetupList_r5_sequence_of,
                                                  1, maxRABsetup, FALSE);

  return offset;
}


static const per_sequence_t T_fdd_37_sequence[] = {
  { &hf_rrc_dummy1          , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CPCH_SetID },
  { &hf_rrc_dummy2_02       , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DRAC_StaticInformationList },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_fdd_37(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_fdd_37, T_fdd_37_sequence);

  return offset;
}


static const value_string rrc_T_dummy_09_vals[] = {
  {   0, "fdd" },
  {   1, "tdd" },
  { 0, NULL }
};

static const per_choice_t T_dummy_09_choice[] = {
  {   0, &hf_rrc_fdd_37          , ASN1_NO_EXTENSIONS     , dissect_rrc_T_fdd_37 },
  {   1, &hf_rrc_tdd             , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_dummy_09(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_dummy_09, T_dummy_09_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t T_fdd_38_sequence[] = {
  { &hf_rrc_dummy_06        , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_PDSCH_Information },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_fdd_38(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_fdd_38, T_fdd_38_sequence);

  return offset;
}


static const value_string rrc_T_modeSpecificPhysChInfo_11_vals[] = {
  {   0, "fdd" },
  {   1, "tdd" },
  { 0, NULL }
};

static const per_choice_t T_modeSpecificPhysChInfo_11_choice[] = {
  {   0, &hf_rrc_fdd_38          , ASN1_NO_EXTENSIONS     , dissect_rrc_T_fdd_38 },
  {   1, &hf_rrc_tdd             , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_modeSpecificPhysChInfo_11(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_modeSpecificPhysChInfo_11, T_modeSpecificPhysChInfo_11_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t RadioBearerSetup_r5_IEs_sequence[] = {
  { &hf_rrc_integrityProtectionModeInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_IntegrityProtectionModeInfo },
  { &hf_rrc_cipheringModeInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CipheringModeInfo },
  { &hf_rrc_activationTime  , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_ActivationTime },
  { &hf_rrc_new_U_RNTI      , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_U_RNTI },
  { &hf_rrc_new_C_RNTI      , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_C_RNTI },
  { &hf_rrc_new_DSCH_RNTI   , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DSCH_RNTI },
  { &hf_rrc_new_H_RNTI      , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_H_RNTI },
  { &hf_rrc_rrc_StateIndicator, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RRC_StateIndicator },
  { &hf_rrc_utran_DRX_CycleLengthCoeff, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UTRAN_DRX_CycleLengthCoefficient },
  { &hf_rrc_ura_Identity    , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_URA_Identity },
  { &hf_rrc_cn_InformationInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CN_InformationInfo },
  { &hf_rrc_srb_InformationSetupList_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_SRB_InformationSetupList_r5 },
  { &hf_rrc_rab_InformationSetupList_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RAB_InformationSetupList_r5 },
  { &hf_rrc_rb_InformationAffectedList_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RB_InformationAffectedList_r5 },
  { &hf_rrc_dl_CounterSynchronisationInfo_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_CounterSynchronisationInfo_r5 },
  { &hf_rrc_ul_CommonTransChInfo_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_CommonTransChInfo_r4 },
  { &hf_rrc_ul_deletedTransChInfoList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_DeletedTransChInfoList },
  { &hf_rrc_ul_AddReconfTransChInfoList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_AddReconfTransChInfoList },
  { &hf_rrc_dummy_22        , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_dummy_09 },
  { &hf_rrc_dl_CommonTransChInfo_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_CommonTransChInfo_r4 },
  { &hf_rrc_dl_DeletedTransChInfoList_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_DeletedTransChInfoList_r5 },
  { &hf_rrc_dl_AddReconfTransChInfoList_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_AddReconfTransChInfoList_r5 },
  { &hf_rrc_frequencyInfo   , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_FrequencyInfo },
  { &hf_rrc_maxAllowedUL_TX_Power, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MaxAllowedUL_TX_Power },
  { &hf_rrc_ul_ChannelRequirement_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_ChannelRequirement_r5 },
  { &hf_rrc_modeSpecificPhysChInfo_11, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_modeSpecificPhysChInfo_11 },
  { &hf_rrc_dl_HSPDSCH_Information, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_HSPDSCH_Information },
  { &hf_rrc_dl_CommonInformation_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_CommonInformation_r5 },
  { &hf_rrc_dl_InformationPerRL_List_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_InformationPerRL_List_r5 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_RadioBearerSetup_r5_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_RadioBearerSetup_r5_IEs, RadioBearerSetup_r5_IEs_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions_108_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_nonCriticalExtensions_108(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_nonCriticalExtensions_108, T_nonCriticalExtensions_108_sequence);

  return offset;
}


static const per_sequence_t T_v690NonCriticalExtensions_24_sequence[] = {
  { &hf_rrc_radioBearerSetup_v690ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RadioBearerSetup_v690ext_IEs },
  { &hf_rrc_nonCriticalExtensions_108, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_nonCriticalExtensions_108 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v690NonCriticalExtensions_24(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v690NonCriticalExtensions_24, T_v690NonCriticalExtensions_24_sequence);

  return offset;
}


static const per_sequence_t T_v5d0NonCriticalExtenstions_11_sequence[] = {
  { &hf_rrc_radioBearerSetup_v5d0ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RadioBearerSetup_v5d0ext_IEs },
  { &hf_rrc_v690NonCriticalExtensions_24, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v690NonCriticalExtensions_24 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v5d0NonCriticalExtenstions_11(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v5d0NonCriticalExtenstions_11, T_v5d0NonCriticalExtenstions_11_sequence);

  return offset;
}


static const per_sequence_t T_r5_07_sequence[] = {
  { &hf_rrc_radioBearerSetup_r5, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RadioBearerSetup_r5_IEs },
  { &hf_rrc_radioBearerSetup_r5_add_ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_BIT_STRING },
  { &hf_rrc_v5d0NonCriticalExtenstions_11, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v5d0NonCriticalExtenstions_11 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_r5_07(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_r5_07, T_r5_07_sequence);

  return offset;
}


static const value_string rrc_RLC_InfoChoice_r6_vals[] = {
  {   0, "rlc-Info" },
  {   1, "same-as-RB" },
  { 0, NULL }
};

static const per_choice_t RLC_InfoChoice_r6_choice[] = {
  {   0, &hf_rrc_rlc_Info_02     , ASN1_NO_EXTENSIONS     , dissect_rrc_RLC_Info_r6 },
  {   1, &hf_rrc_same_as_RB      , ASN1_NO_EXTENSIONS     , dissect_rrc_RB_Identity },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_RLC_InfoChoice_r6(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_RLC_InfoChoice_r6, RLC_InfoChoice_r6_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t SRB_InformationSetup_r6_sequence[] = {
  { &hf_rrc_rb_Identity     , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RB_Identity },
  { &hf_rrc_rlc_InfoChoice_02, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RLC_InfoChoice_r6 },
  { &hf_rrc_rb_MappingInfo_02, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RB_MappingInfo_r6 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_SRB_InformationSetup_r6(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_SRB_InformationSetup_r6, SRB_InformationSetup_r6_sequence);

  return offset;
}


static const per_sequence_t SRB_InformationSetupList_r6_sequence_of[1] = {
  { &hf_rrc_SRB_InformationSetupList_r6_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SRB_InformationSetup_r6 },
};

static int
dissect_rrc_SRB_InformationSetupList_r6(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_SRB_InformationSetupList_r6, SRB_InformationSetupList_r6_sequence_of,
                                                  1, maxSRBsetup, FALSE);

  return offset;
}


static const per_sequence_t RB_InformationSetup_r6_sequence[] = {
  { &hf_rrc_rb_Identity     , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RB_Identity },
  { &hf_rrc_pdcp_Info_01    , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_PDCP_Info_r4 },
  { &hf_rrc_rlc_InfoChoice_02, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RLC_InfoChoice_r6 },
  { &hf_rrc_rb_MappingInfo_02, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RB_MappingInfo_r6 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_RB_InformationSetup_r6(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_RB_InformationSetup_r6, RB_InformationSetup_r6_sequence);

  return offset;
}


static const per_sequence_t RB_InformationSetupList_r6_sequence_of[1] = {
  { &hf_rrc_RB_InformationSetupList_r6_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RB_InformationSetup_r6 },
};

static int
dissect_rrc_RB_InformationSetupList_r6(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_RB_InformationSetupList_r6, RB_InformationSetupList_r6_sequence_of,
                                                  1, maxRBperRAB, FALSE);

  return offset;
}


static const per_sequence_t RAB_InformationSetup_r6_sequence[] = {
  { &hf_rrc_rab_Info_01     , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RAB_Info_r6 },
  { &hf_rrc_rb_InformationSetupList_03, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RB_InformationSetupList_r6 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_RAB_InformationSetup_r6(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_RAB_InformationSetup_r6, RAB_InformationSetup_r6_sequence);

  return offset;
}


static const per_sequence_t RAB_InformationSetupList_r6_sequence_of[1] = {
  { &hf_rrc_RAB_InformationSetupList_r6_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RAB_InformationSetup_r6 },
};

static int
dissect_rrc_RAB_InformationSetupList_r6(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_RAB_InformationSetupList_r6, RAB_InformationSetupList_r6_sequence_of,
                                                  1, maxRABsetup, FALSE);

  return offset;
}


static const per_sequence_t T_complete_17_sequence[] = {
  { &hf_rrc_srb_InformationSetupList_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_SRB_InformationSetupList_r6 },
  { &hf_rrc_rab_InformationSetupList_03, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RAB_InformationSetupList_r6 },
  { &hf_rrc_rab_InformationReconfigList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RAB_InformationReconfigList },
  { &hf_rrc_rb_InformationReconfigList_03, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RB_InformationReconfigList_r6 },
  { &hf_rrc_rb_InformationAffectedList_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RB_InformationAffectedList_r6 },
  { &hf_rrc_dl_CounterSynchronisationInfo_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_CounterSynchronisationInfo_r5 },
  { &hf_rrc_pdcp_ROHC_TargetMode, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_PDCP_ROHC_TargetMode },
  { &hf_rrc_ul_CommonTransChInfo_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_CommonTransChInfo_r4 },
  { &hf_rrc_ul_deletedTransChInfoList_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_DeletedTransChInfoList_r6 },
  { &hf_rrc_ul_AddReconfTransChInfoList_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_AddReconfTransChInfoList_r6 },
  { &hf_rrc_dl_CommonTransChInfo_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_CommonTransChInfo_r4 },
  { &hf_rrc_dl_DeletedTransChInfoList_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_DeletedTransChInfoList_r5 },
  { &hf_rrc_dl_AddReconfTransChInfoList_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_AddReconfTransChInfoList_r5 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_complete_17(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_complete_17, T_complete_17_sequence);

  return offset;
}


static const value_string rrc_T_rb_Change_vals[] = {
  {   0, "release" },
  {   1, "re-mapToDefaultRb" },
  { 0, NULL }
};

static const per_choice_t T_rb_Change_choice[] = {
  {   0, &hf_rrc_release_01      , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   1, &hf_rrc_re_mapToDefaultRb, ASN1_NO_EXTENSIONS     , dissect_rrc_RB_Identity },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_rb_Change(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_rb_Change, T_rb_Change_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t RB_InformationChanged_r6_sequence[] = {
  { &hf_rrc_rb_Identity     , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RB_Identity },
  { &hf_rrc_rb_Change       , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_rb_Change },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_RB_InformationChanged_r6(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_RB_InformationChanged_r6, RB_InformationChanged_r6_sequence);

  return offset;
}


static const per_sequence_t RB_InformationChangedList_r6_sequence_of[1] = {
  { &hf_rrc_RB_InformationChangedList_r6_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RB_InformationChanged_r6 },
};

static int
dissect_rrc_RB_InformationChangedList_r6(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_RB_InformationChangedList_r6, RB_InformationChangedList_r6_sequence_of,
                                                  1, maxRB, FALSE);

  return offset;
}


static const per_sequence_t T_fdd_52_sequence[] = {
  { &hf_rrc_gainFactorBetaC , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_GainFactor },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_fdd_52(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_fdd_52, T_fdd_52_sequence);

  return offset;
}


static const value_string rrc_T_modeSpecificInfo_22_vals[] = {
  {   0, "fdd" },
  {   1, "tdd" },
  { 0, NULL }
};

static const per_choice_t T_modeSpecificInfo_22_choice[] = {
  {   0, &hf_rrc_fdd_53          , ASN1_NO_EXTENSIONS     , dissect_rrc_T_fdd_52 },
  {   1, &hf_rrc_tdd             , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_modeSpecificInfo_22(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_modeSpecificInfo_22, T_modeSpecificInfo_22_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t PowerOffsetInfoShort_sequence[] = {
  { &hf_rrc_referenceTFC    , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TFC_Value },
  { &hf_rrc_modeSpecificInfo_22, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_modeSpecificInfo_22 },
  { &hf_rrc_gainFactorBetaD , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_GainFactor },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_PowerOffsetInfoShort(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_PowerOffsetInfoShort, PowerOffsetInfoShort_sequence);

  return offset;
}


static const per_sequence_t T_dummy_10_sequence[] = {
  { &hf_rrc_rab_Info_01     , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RAB_Info_r6 },
  { &hf_rrc_defaultConfigMode, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DefaultConfigMode },
  { &hf_rrc_defaultConfigIdentity_03, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DefaultConfigIdentity_r6 },
  { &hf_rrc_rb_InformationChangedList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RB_InformationChangedList_r6 },
  { &hf_rrc_powerOffsetInfoShort, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PowerOffsetInfoShort },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_dummy_10(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_dummy_10, T_dummy_10_sequence);

  return offset;
}


static const value_string rrc_T_specificationMode_17_vals[] = {
  {   0, "complete" },
  {   1, "dummy" },
  { 0, NULL }
};

static const per_choice_t T_specificationMode_17_choice[] = {
  {   0, &hf_rrc_complete_17     , ASN1_NO_EXTENSIONS     , dissect_rrc_T_complete_17 },
  {   1, &hf_rrc_dummy_23        , ASN1_NO_EXTENSIONS     , dissect_rrc_T_dummy_10 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_specificationMode_17(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_specificationMode_17, T_specificationMode_17_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t RadioBearerSetup_r6_IEs_sequence[] = {
  { &hf_rrc_integrityProtectionModeInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_IntegrityProtectionModeInfo },
  { &hf_rrc_cipheringModeInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CipheringModeInfo },
  { &hf_rrc_activationTime  , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_ActivationTime },
  { &hf_rrc_new_U_RNTI      , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_U_RNTI },
  { &hf_rrc_new_C_RNTI      , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_C_RNTI },
  { &hf_rrc_new_DSCH_RNTI   , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DSCH_RNTI },
  { &hf_rrc_new_H_RNTI      , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_H_RNTI },
  { &hf_rrc_newPrimary_E_RNTI, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_E_RNTI },
  { &hf_rrc_newSecondary_E_RNTI, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_E_RNTI },
  { &hf_rrc_rrc_StateIndicator, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RRC_StateIndicator },
  { &hf_rrc_utran_DRX_CycleLengthCoeff, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UTRAN_DRX_CycleLengthCoefficient },
  { &hf_rrc_ura_Identity    , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_URA_Identity },
  { &hf_rrc_cn_InformationInfo_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CN_InformationInfo_r6 },
  { &hf_rrc_specificationMode_17, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_specificationMode_17 },
  { &hf_rrc_frequencyInfo   , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_FrequencyInfo },
  { &hf_rrc_maxAllowedUL_TX_Power, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MaxAllowedUL_TX_Power },
  { &hf_rrc_ul_DPCH_Info    , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_DPCH_Info_r6 },
  { &hf_rrc_ul_EDCH_Information, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_EDCH_Information_r6 },
  { &hf_rrc_dl_HSPDSCH_Information_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_HSPDSCH_Information_r6 },
  { &hf_rrc_dl_CommonInformation_03, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_CommonInformation_r6 },
  { &hf_rrc_dl_InformationPerRL_List_03, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_InformationPerRL_List_r6 },
  { &hf_rrc_mbms_PL_ServiceRestrictInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MBMS_PL_ServiceRestrictInfo_r6 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_RadioBearerSetup_r6_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_RadioBearerSetup_r6_IEs, RadioBearerSetup_r6_IEs_sequence);

  return offset;
}


static const per_sequence_t RAB_Info_v6b0ext_sequence[] = {
  { &hf_rrc_mbms_ServiceIdentity, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_OCTET_STRING_SIZE_3 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_RAB_Info_v6b0ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_RAB_Info_v6b0ext, RAB_Info_v6b0ext_sequence);

  return offset;
}


static const per_sequence_t RAB_InformationSetup_v6b0ext_sequence[] = {
  { &hf_rrc_rab_Info_v6b0ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RAB_Info_v6b0ext },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_RAB_InformationSetup_v6b0ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_RAB_InformationSetup_v6b0ext, RAB_InformationSetup_v6b0ext_sequence);

  return offset;
}


static const per_sequence_t RAB_InformationSetupList_v6b0ext_sequence_of[1] = {
  { &hf_rrc_RAB_InformationSetupList_v6b0ext_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RAB_InformationSetup_v6b0ext },
};

static int
dissect_rrc_RAB_InformationSetupList_v6b0ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_RAB_InformationSetupList_v6b0ext, RAB_InformationSetupList_v6b0ext_sequence_of,
                                                  1, maxRABsetup, FALSE);

  return offset;
}


static const per_sequence_t RadioBearerSetup_v6b0ext_IEs_sequence[] = {
  { &hf_rrc_dl_InformationPerRL_List_v6b0ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_InformationPerRL_List_v6b0ext },
  { &hf_rrc_rab_InformationSetupListExt, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RAB_InformationSetupList_v6b0ext },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_RadioBearerSetup_v6b0ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_RadioBearerSetup_v6b0ext_IEs, RadioBearerSetup_v6b0ext_IEs_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions_109_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_nonCriticalExtensions_109(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_nonCriticalExtensions_109, T_nonCriticalExtensions_109_sequence);

  return offset;
}


static const per_sequence_t T_v6b0NonCriticalExtensions_08_sequence[] = {
  { &hf_rrc_radioBearerSetup_v6b0ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RadioBearerSetup_v6b0ext_IEs },
  { &hf_rrc_nonCriticalExtensions_109, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_nonCriticalExtensions_109 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v6b0NonCriticalExtensions_08(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v6b0NonCriticalExtensions_08, T_v6b0NonCriticalExtensions_08_sequence);

  return offset;
}


static const per_sequence_t T_r6_09_sequence[] = {
  { &hf_rrc_radioBearerSetup_r6, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RadioBearerSetup_r6_IEs },
  { &hf_rrc_radioBearerSetup_r6_add_ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_BIT_STRING },
  { &hf_rrc_v6b0NonCriticalExtensions_09, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v6b0NonCriticalExtensions_08 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_r6_09(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_r6_09, T_r6_09_sequence);

  return offset;
}


static const per_sequence_t SRB_InformationSetup_r7_sequence[] = {
  { &hf_rrc_rb_Identity     , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RB_Identity },
  { &hf_rrc_rlc_InfoChoice_03, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RLC_InfoChoice_r7 },
  { &hf_rrc_rb_MappingInfo_03, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RB_MappingInfo_r7 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_SRB_InformationSetup_r7(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_SRB_InformationSetup_r7, SRB_InformationSetup_r7_sequence);

  return offset;
}


static const per_sequence_t SRB_InformationSetupList_r7_sequence_of[1] = {
  { &hf_rrc_SRB_InformationSetupList_r7_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SRB_InformationSetup_r7 },
};

static int
dissect_rrc_SRB_InformationSetupList_r7(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_SRB_InformationSetupList_r7, SRB_InformationSetupList_r7_sequence_of,
                                                  1, maxSRBsetup, FALSE);

  return offset;
}


static const per_sequence_t RAB_InformationSetupList_r7_sequence_of[1] = {
  { &hf_rrc_RAB_InformationSetupList_r7_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RAB_InformationSetup_r7 },
};

static int
dissect_rrc_RAB_InformationSetupList_r7(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_RAB_InformationSetupList_r7, RAB_InformationSetupList_r7_sequence_of,
                                                  1, maxRABsetup, FALSE);

  return offset;
}


static const per_sequence_t T_complete_18_sequence[] = {
  { &hf_rrc_srb_InformationSetupList_03, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_SRB_InformationSetupList_r7 },
  { &hf_rrc_rab_InformationSetupList_04, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RAB_InformationSetupList_r7 },
  { &hf_rrc_rab_InformationReconfigList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RAB_InformationReconfigList },
  { &hf_rrc_rb_InformationReconfigList_04, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RB_InformationReconfigList_r7 },
  { &hf_rrc_rb_InformationAffectedList_03, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RB_InformationAffectedList_r7 },
  { &hf_rrc_dl_CounterSynchronisationInfo_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_CounterSynchronisationInfo_r5 },
  { &hf_rrc_pdcp_ROHC_TargetMode, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_PDCP_ROHC_TargetMode },
  { &hf_rrc_ul_CommonTransChInfo_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_CommonTransChInfo_r4 },
  { &hf_rrc_ul_deletedTransChInfoList_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_DeletedTransChInfoList_r6 },
  { &hf_rrc_ul_AddReconfTransChInfoList_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_AddReconfTransChInfoList_r7 },
  { &hf_rrc_dl_CommonTransChInfo_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_CommonTransChInfo_r4 },
  { &hf_rrc_dl_DeletedTransChInfoList_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_DeletedTransChInfoList_r7 },
  { &hf_rrc_dl_AddReconfTransChInfoList_03, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_AddReconfTransChInfoList_r7 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_complete_18(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_complete_18, T_complete_18_sequence);

  return offset;
}


static const value_string rrc_T_specificationMode_18_vals[] = {
  {   0, "complete" },
  {   1, "dummy" },
  { 0, NULL }
};

static const per_choice_t T_specificationMode_18_choice[] = {
  {   0, &hf_rrc_complete_18     , ASN1_NO_EXTENSIONS     , dissect_rrc_T_complete_18 },
  {   1, &hf_rrc_dummy_24        , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_specificationMode_18(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_specificationMode_18, T_specificationMode_18_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t RadioBearerSetup_r7_IEs_sequence[] = {
  { &hf_rrc_integrityProtectionModeInfo_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_IntegrityProtectionModeInfo_r7 },
  { &hf_rrc_cipheringModeInfo_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CipheringModeInfo_r7 },
  { &hf_rrc_activationTime  , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_ActivationTime },
  { &hf_rrc_new_U_RNTI      , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_U_RNTI },
  { &hf_rrc_new_C_RNTI      , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_C_RNTI },
  { &hf_rrc_new_DSCH_RNTI   , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DSCH_RNTI },
  { &hf_rrc_new_H_RNTI      , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_H_RNTI },
  { &hf_rrc_newPrimary_E_RNTI, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_E_RNTI },
  { &hf_rrc_newSecondary_E_RNTI, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_E_RNTI },
  { &hf_rrc_rrc_StateIndicator, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RRC_StateIndicator },
  { &hf_rrc_utran_DRX_CycleLengthCoeff_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UTRAN_DRX_CycleLengthCoefficient_r7 },
  { &hf_rrc_ura_Identity    , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_URA_Identity },
  { &hf_rrc_supportForChangeOfUE_Capability, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_BOOLEAN },
  { &hf_rrc_cn_InformationInfo_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CN_InformationInfo_r6 },
  { &hf_rrc_specificationMode_18, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_specificationMode_18 },
  { &hf_rrc_frequencyInfo   , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_FrequencyInfo },
  { &hf_rrc_multi_frequencyInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_Multi_frequencyInfo_LCR_r7 },
  { &hf_rrc_dtx_drx_TimingInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DTX_DRX_TimingInfo_r7 },
  { &hf_rrc_dtx_drx_Info    , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DTX_DRX_Info_r7 },
  { &hf_rrc_hs_scch_LessInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_HS_SCCH_LessInfo_r7 },
  { &hf_rrc_mimoParameters  , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MIMO_Parameters_r7 },
  { &hf_rrc_maxAllowedUL_TX_Power, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MaxAllowedUL_TX_Power },
  { &hf_rrc_ul_DPCH_Info_01 , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_DPCH_Info_r7 },
  { &hf_rrc_ul_EDCH_Information_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_EDCH_Information_r7 },
  { &hf_rrc_dl_HSPDSCH_Information_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_HSPDSCH_Information_r7 },
  { &hf_rrc_dl_CommonInformation_04, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_CommonInformation_r7 },
  { &hf_rrc_dl_InformationPerRL_List_04, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_InformationPerRL_List_r7 },
  { &hf_rrc_mbms_PL_ServiceRestrictInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MBMS_PL_ServiceRestrictInfo_r6 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_RadioBearerSetup_r7_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_RadioBearerSetup_r7_IEs, RadioBearerSetup_r7_IEs_sequence);

  return offset;
}



static int
dissect_rrc_T_radioBearerSetup_r7_add_ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_bit_string_containing_pdu_new(tvb, offset, actx, tree, hf_index,
                                                              NO_BOUND, NO_BOUND, FALSE, dissect_RadioBearerSetup_r7_add_ext_IEs_PDU);

  return offset;
}


static const per_sequence_t RadioBearerSetup_v780ext_IEs_sequence[] = {
  { &hf_rrc_hs_DSCH_TBSizeTable, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_HS_DSCH_TBSizeTable },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_RadioBearerSetup_v780ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_RadioBearerSetup_v780ext_IEs, RadioBearerSetup_v780ext_IEs_sequence);

  return offset;
}


static const per_sequence_t RAB_InformationSetupList_v820ext_sequence_of[1] = {
  { &hf_rrc_RAB_InformationSetupList_v820ext_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RAB_InformationSetup_v820ext },
};

static int
dissect_rrc_RAB_InformationSetupList_v820ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_RAB_InformationSetupList_v820ext, RAB_InformationSetupList_v820ext_sequence_of,
                                                  1, maxRABsetup, FALSE);

  return offset;
}


static const per_sequence_t RadioBearerSetup_v820ext_IEs_sequence[] = {
  { &hf_rrc_rab_InformationSetupList_06, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RAB_InformationSetupList_v820ext },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_RadioBearerSetup_v820ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_RadioBearerSetup_v820ext_IEs, RadioBearerSetup_v820ext_IEs_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions_110_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_nonCriticalExtensions_110(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_nonCriticalExtensions_110, T_nonCriticalExtensions_110_sequence);

  return offset;
}


static const per_sequence_t T_v820NonCriticalExtensions_01_sequence[] = {
  { &hf_rrc_radioBearerSetup_v820ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RadioBearerSetup_v820ext_IEs },
  { &hf_rrc_nonCriticalExtensions_110, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_nonCriticalExtensions_110 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v820NonCriticalExtensions_01(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v820NonCriticalExtensions_01, T_v820NonCriticalExtensions_01_sequence);

  return offset;
}


static const per_sequence_t T_v780NonCriticalExtensions_07_sequence[] = {
  { &hf_rrc_radioBearerSetup_v780ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RadioBearerSetup_v780ext_IEs },
  { &hf_rrc_v820NonCriticalExtensions_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v820NonCriticalExtensions_01 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v780NonCriticalExtensions_07(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v780NonCriticalExtensions_07, T_v780NonCriticalExtensions_07_sequence);

  return offset;
}


static const per_sequence_t T_r7_08_sequence[] = {
  { &hf_rrc_radioBearerSetup_r7, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RadioBearerSetup_r7_IEs },
  { &hf_rrc_radioBearerSetup_r7_add_ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_radioBearerSetup_r7_add_ext },
  { &hf_rrc_v780NonCriticalExtensions_07, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v780NonCriticalExtensions_07 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_r7_08(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_r7_08, T_r7_08_sequence);

  return offset;
}


static const per_sequence_t SRB_InformationSetup_r8_sequence[] = {
  { &hf_rrc_rb_Identity     , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RB_Identity },
  { &hf_rrc_rlc_InfoChoice_03, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RLC_InfoChoice_r7 },
  { &hf_rrc_rb_MappingInfo_04, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RB_MappingInfo_r8 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_SRB_InformationSetup_r8(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_SRB_InformationSetup_r8, SRB_InformationSetup_r8_sequence);

  return offset;
}


static const per_sequence_t SRB_InformationSetupList_r8_sequence_of[1] = {
  { &hf_rrc_SRB_InformationSetupList_r8_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SRB_InformationSetup_r8 },
};

static int
dissect_rrc_SRB_InformationSetupList_r8(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_SRB_InformationSetupList_r8, SRB_InformationSetupList_r8_sequence_of,
                                                  1, maxSRBsetup, FALSE);

  return offset;
}


static const per_sequence_t RAB_InformationSetupList_r8_sequence_of[1] = {
  { &hf_rrc_RAB_InformationSetupList_r8_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RAB_InformationSetup_r8 },
};

static int
dissect_rrc_RAB_InformationSetupList_r8(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_RAB_InformationSetupList_r8, RAB_InformationSetupList_r8_sequence_of,
                                                  1, maxRABsetup, FALSE);

  return offset;
}


static const per_sequence_t RadioBearerSetup_r8_IEs_sequence[] = {
  { &hf_rrc_integrityProtectionModeInfo_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_IntegrityProtectionModeInfo_r7 },
  { &hf_rrc_cipheringModeInfo_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CipheringModeInfo_r7 },
  { &hf_rrc_sr_vcc_Info     , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_SR_VCC_Info },
  { &hf_rrc_activationTime  , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_ActivationTime },
  { &hf_rrc_new_U_RNTI      , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_U_RNTI },
  { &hf_rrc_new_C_RNTI      , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_C_RNTI },
  { &hf_rrc_new_DSCH_RNTI   , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DSCH_RNTI },
  { &hf_rrc_new_H_RNTI      , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_H_RNTI },
  { &hf_rrc_newPrimary_E_RNTI, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_E_RNTI },
  { &hf_rrc_newSecondary_E_RNTI, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_E_RNTI },
  { &hf_rrc_rrc_StateIndicator, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RRC_StateIndicator },
  { &hf_rrc_utran_DRX_CycleLengthCoeff_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UTRAN_DRX_CycleLengthCoefficient_r7 },
  { &hf_rrc_ura_Identity    , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_URA_Identity },
  { &hf_rrc_supportForChangeOfUE_Capability, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_BOOLEAN },
  { &hf_rrc_cn_InformationInfo_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CN_InformationInfo_r6 },
  { &hf_rrc_srb_InformationSetupList_04, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_SRB_InformationSetupList_r8 },
  { &hf_rrc_rab_InformationSetupList_05, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RAB_InformationSetupList_r8 },
  { &hf_rrc_rab_InformationReconfigList_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RAB_InformationReconfigList_r8 },
  { &hf_rrc_rb_InformationReconfigList_05, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RB_InformationReconfigList_r8 },
  { &hf_rrc_rb_InformationAffectedList_04, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RB_InformationAffectedList_r8 },
  { &hf_rrc_dl_CounterSynchronisationInfo_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_CounterSynchronisationInfo_r5 },
  { &hf_rrc_pdcp_ROHC_TargetMode, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_PDCP_ROHC_TargetMode },
  { &hf_rrc_ul_CommonTransChInfo_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_CommonTransChInfo_r4 },
  { &hf_rrc_ul_deletedTransChInfoList_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_DeletedTransChInfoList_r6 },
  { &hf_rrc_ul_AddReconfTransChInfoList_03, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_AddReconfTransChInfoList_r8 },
  { &hf_rrc_dl_CommonTransChInfo_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_CommonTransChInfo_r4 },
  { &hf_rrc_dl_DeletedTransChInfoList_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_DeletedTransChInfoList_r7 },
  { &hf_rrc_dl_AddReconfTransChInfoList_03, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_AddReconfTransChInfoList_r7 },
  { &hf_rrc_frequencyInfo   , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_FrequencyInfo },
  { &hf_rrc_multi_frequencyInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_Multi_frequencyInfo_LCR_r7 },
  { &hf_rrc_dtx_drx_TimingInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DTX_DRX_TimingInfo_r7 },
  { &hf_rrc_dtx_drx_Info    , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DTX_DRX_Info_r7 },
  { &hf_rrc_hs_scch_LessInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_HS_SCCH_LessInfo_r7 },
  { &hf_rrc_mimoParameters_03, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MIMO_Parameters_r8 },
  { &hf_rrc_maxAllowedUL_TX_Power, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MaxAllowedUL_TX_Power },
  { &hf_rrc_ul_DPCH_Info_01 , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_DPCH_Info_r7 },
  { &hf_rrc_ul_EDCH_Information_03, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_EDCH_Information_r8 },
  { &hf_rrc_dl_HSPDSCH_Information_03, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_HSPDSCH_Information_r8 },
  { &hf_rrc_dl_CommonInformation_05, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_CommonInformation_r8 },
  { &hf_rrc_dl_InformationPerRL_List_05, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_InformationPerRL_List_r8 },
  { &hf_rrc_dl_SecondaryCellInfoFDD, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_SecondaryCellInfoFDD },
  { &hf_rrc_controlChannelDRXInfo_TDD128, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_ControlChannelDRXInfo_TDD128_r8 },
  { &hf_rrc_sps_Information_TDD128, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_SPS_Information_TDD128_r8 },
  { &hf_rrc_mbms_PL_ServiceRestrictInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MBMS_PL_ServiceRestrictInfo_r6 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_RadioBearerSetup_r8_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_RadioBearerSetup_r8_IEs, RadioBearerSetup_r8_IEs_sequence);

  return offset;
}


static const per_sequence_t RadioBearerSetup_v7d0ext_IEs_sequence[] = {
  { &hf_rrc_ul_EDCH_Information_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_EDCH_Information_ext },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_RadioBearerSetup_v7d0ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_RadioBearerSetup_v7d0ext_IEs, RadioBearerSetup_v7d0ext_IEs_sequence);

  return offset;
}


static const per_sequence_t RadioBearerSetup_v7f0ext_IEs_sequence[] = {
  { &hf_rrc_mimoParameters_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MIMO_Parameters_v7f0ext },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_RadioBearerSetup_v7f0ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_RadioBearerSetup_v7f0ext_IEs, RadioBearerSetup_v7f0ext_IEs_sequence);

  return offset;
}


static const per_sequence_t RadioBearerSetup_v890ext_IEs_sequence[] = {
  { &hf_rrc_dl_SecondaryCellInfoFDD_v890ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_SecondaryCellInfoFDD_v890ext },
  { &hf_rrc_dl_HSPDSCH_Information_04, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_HSPDSCH_Information_r8_ext },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_RadioBearerSetup_v890ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_RadioBearerSetup_v890ext_IEs, RadioBearerSetup_v890ext_IEs_sequence);

  return offset;
}


static const per_sequence_t RadioBearerSetup_v7g0ext_IEs_sequence[] = {
  { &hf_rrc_mimoParameters_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MIMO_Parameters_v7g0ext },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_RadioBearerSetup_v7g0ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_RadioBearerSetup_v7g0ext_IEs, RadioBearerSetup_v7g0ext_IEs_sequence);

  return offset;
}


static const per_sequence_t RadioBearerSetup_v8a0ext_IEs_sequence[] = {
  { &hf_rrc_dl_HSPDSCH_Information_05, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_HSPDSCH_Information_r8_ext2 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_RadioBearerSetup_v8a0ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_RadioBearerSetup_v8a0ext_IEs, RadioBearerSetup_v8a0ext_IEs_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions_111_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_nonCriticalExtensions_111(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_nonCriticalExtensions_111, T_nonCriticalExtensions_111_sequence);

  return offset;
}


static const per_sequence_t T_v8a0NonCriticalExtensions_07_sequence[] = {
  { &hf_rrc_radioBearerSetup_v8a0ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RadioBearerSetup_v8a0ext_IEs },
  { &hf_rrc_nonCriticalExtensions_111, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_nonCriticalExtensions_111 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v8a0NonCriticalExtensions_07(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v8a0NonCriticalExtensions_07, T_v8a0NonCriticalExtensions_07_sequence);

  return offset;
}


static const per_sequence_t T_v7g0NonCriticalExtensions_12_sequence[] = {
  { &hf_rrc_radioBearerSetup_v7g0ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RadioBearerSetup_v7g0ext_IEs },
  { &hf_rrc_v8a0NonCriticalExtensions_07, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v8a0NonCriticalExtensions_07 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v7g0NonCriticalExtensions_12(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v7g0NonCriticalExtensions_12, T_v7g0NonCriticalExtensions_12_sequence);

  return offset;
}


static const per_sequence_t T_v890NonCriticalExtensions_07_sequence[] = {
  { &hf_rrc_radioBearerSetup_v890ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RadioBearerSetup_v890ext_IEs },
  { &hf_rrc_v7g0NonCriticalExtensions_12, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v7g0NonCriticalExtensions_12 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v890NonCriticalExtensions_07(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v890NonCriticalExtensions_07, T_v890NonCriticalExtensions_07_sequence);

  return offset;
}


static const per_sequence_t T_v7f0NonCriticalExtensions_11_sequence[] = {
  { &hf_rrc_radioBearerSetup_v7f0ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RadioBearerSetup_v7f0ext_IEs },
  { &hf_rrc_v890NonCriticalExtensions_07, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v890NonCriticalExtensions_07 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v7f0NonCriticalExtensions_11(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v7f0NonCriticalExtensions_11, T_v7f0NonCriticalExtensions_11_sequence);

  return offset;
}


static const per_sequence_t T_v7d0NonCriticalExtensions_10_sequence[] = {
  { &hf_rrc_radioBearerSetup_v7d0ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RadioBearerSetup_v7d0ext_IEs },
  { &hf_rrc_v7f0NonCriticalExtensions_11, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v7f0NonCriticalExtensions_11 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v7d0NonCriticalExtensions_10(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v7d0NonCriticalExtensions_10, T_v7d0NonCriticalExtensions_10_sequence);

  return offset;
}


static const per_sequence_t T_r8_09_sequence[] = {
  { &hf_rrc_radioBearerSetup_r8, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RadioBearerSetup_r8_IEs },
  { &hf_rrc_radioBearerSetup_r8_add_ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_BIT_STRING },
  { &hf_rrc_v7d0NonCriticalExtensions_10, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v7d0NonCriticalExtensions_10 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_r8_09(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_r8_09, T_r8_09_sequence);

  return offset;
}


static const per_sequence_t RadioBearerSetup_r9_IEs_sequence[] = {
  { &hf_rrc_integrityProtectionModeInfo_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_IntegrityProtectionModeInfo_r7 },
  { &hf_rrc_cipheringModeInfo_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CipheringModeInfo_r7 },
  { &hf_rrc_sr_vcc_Info     , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_SR_VCC_Info },
  { &hf_rrc_activationTime  , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_ActivationTime },
  { &hf_rrc_new_U_RNTI      , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_U_RNTI },
  { &hf_rrc_new_C_RNTI      , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_C_RNTI },
  { &hf_rrc_new_DSCH_RNTI   , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DSCH_RNTI },
  { &hf_rrc_new_H_RNTI      , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_H_RNTI },
  { &hf_rrc_newPrimary_E_RNTI, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_E_RNTI },
  { &hf_rrc_newSecondary_E_RNTI, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_E_RNTI },
  { &hf_rrc_rrc_StateIndicator, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RRC_StateIndicator },
  { &hf_rrc_utran_DRX_CycleLengthCoeff_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UTRAN_DRX_CycleLengthCoefficient_r7 },
  { &hf_rrc_ura_Identity    , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_URA_Identity },
  { &hf_rrc_supportForChangeOfUE_Capability, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_BOOLEAN },
  { &hf_rrc_cn_InformationInfo_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CN_InformationInfo_r6 },
  { &hf_rrc_srb_InformationSetupList_04, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_SRB_InformationSetupList_r8 },
  { &hf_rrc_rab_InformationSetupList_05, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RAB_InformationSetupList_r8 },
  { &hf_rrc_rab_InformationReconfigList_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RAB_InformationReconfigList_r8 },
  { &hf_rrc_rb_InformationReconfigList_05, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RB_InformationReconfigList_r8 },
  { &hf_rrc_rb_InformationAffectedList_04, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RB_InformationAffectedList_r8 },
  { &hf_rrc_dl_CounterSynchronisationInfo_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_CounterSynchronisationInfo_r5 },
  { &hf_rrc_pdcp_ROHC_TargetMode, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_PDCP_ROHC_TargetMode },
  { &hf_rrc_ul_CommonTransChInfo_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_CommonTransChInfo_r4 },
  { &hf_rrc_ul_deletedTransChInfoList_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_DeletedTransChInfoList_r6 },
  { &hf_rrc_ul_AddReconfTransChInfoList_03, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_AddReconfTransChInfoList_r8 },
  { &hf_rrc_dl_CommonTransChInfo_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_CommonTransChInfo_r4 },
  { &hf_rrc_dl_DeletedTransChInfoList_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_DeletedTransChInfoList_r7 },
  { &hf_rrc_dl_AddReconfTransChInfoList_04, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_AddReconfTransChInfoList_r9 },
  { &hf_rrc_frequencyInfo   , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_FrequencyInfo },
  { &hf_rrc_multi_frequencyInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_Multi_frequencyInfo_LCR_r7 },
  { &hf_rrc_dtx_drx_TimingInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DTX_DRX_TimingInfo_r7 },
  { &hf_rrc_dtx_drx_Info    , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DTX_DRX_Info_r7 },
  { &hf_rrc_hs_scch_LessInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_HS_SCCH_LessInfo_r7 },
  { &hf_rrc_mimoParameters_04, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MIMO_Parameters_r9 },
  { &hf_rrc_maxAllowedUL_TX_Power, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MaxAllowedUL_TX_Power },
  { &hf_rrc_ul_DPCH_Info_01 , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_DPCH_Info_r7 },
  { &hf_rrc_ul_EDCH_Information_04, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_EDCH_Information_r9 },
  { &hf_rrc_ul_SecondaryCellInfoFDD, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_SecondaryCellInfoFDD },
  { &hf_rrc_dl_HSPDSCH_Information_06, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_HSPDSCH_Information_r9 },
  { &hf_rrc_dl_CommonInformation_05, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_CommonInformation_r8 },
  { &hf_rrc_dl_InformationPerRL_List_05, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_InformationPerRL_List_r8 },
  { &hf_rrc_dl_SecondaryCellInfoFDD_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_SecondaryCellInfoFDD_r9 },
  { &hf_rrc_controlChannelDRXInfo_TDD128, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_ControlChannelDRXInfo_TDD128_r8 },
  { &hf_rrc_sps_Information_TDD128, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_SPS_Information_TDD128_r8 },
  { &hf_rrc_mbms_PL_ServiceRestrictInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MBMS_PL_ServiceRestrictInfo_r6 },
  { &hf_rrc_cellDCHMeasOccasionInfo_TDD128, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CellDCHMeasOccasionInfo_TDD128_r9 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_RadioBearerSetup_r9_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_RadioBearerSetup_r9_IEs, RadioBearerSetup_r9_IEs_sequence);

  return offset;
}


static const per_sequence_t RadioBearerSetup_v950ext_IEs_sequence[] = {
  { &hf_rrc_secondaryCellMimoParameters, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_SecondaryCellMIMOparametersFDD_v950ext },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_RadioBearerSetup_v950ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_RadioBearerSetup_v950ext_IEs, RadioBearerSetup_v950ext_IEs_sequence);

  return offset;
}


static const per_sequence_t RadioBearerSetup_v9c0ext_IEs_sequence[] = {
  { &hf_rrc_dl_AddReconfTransChInfoList_05, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_AddReconfTransChInfoList_TDD128_v9c0ext },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_RadioBearerSetup_v9c0ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_RadioBearerSetup_v9c0ext_IEs, RadioBearerSetup_v9c0ext_IEs_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions_112_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_nonCriticalExtensions_112(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_nonCriticalExtensions_112, T_nonCriticalExtensions_112_sequence);

  return offset;
}


static const per_sequence_t T_v9c0NonCriticalExtensions_08_sequence[] = {
  { &hf_rrc_radioBearerSetup_v9c0ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RadioBearerSetup_v9c0ext_IEs },
  { &hf_rrc_nonCriticalExtensions_112, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_nonCriticalExtensions_112 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v9c0NonCriticalExtensions_08(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v9c0NonCriticalExtensions_08, T_v9c0NonCriticalExtensions_08_sequence);

  return offset;
}


static const per_sequence_t T_v950NonCriticalExtensions_06_sequence[] = {
  { &hf_rrc_radioBearerSetup_v950ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RadioBearerSetup_v950ext_IEs },
  { &hf_rrc_v9c0NonCriticalExtensions_08, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v9c0NonCriticalExtensions_08 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v950NonCriticalExtensions_06(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v950NonCriticalExtensions_06, T_v950NonCriticalExtensions_06_sequence);

  return offset;
}


static const per_sequence_t T_r9_08_sequence[] = {
  { &hf_rrc_radioBearerSetup_r9, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RadioBearerSetup_r9_IEs },
  { &hf_rrc_radioBearerSetup_r9_add_ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_BIT_STRING },
  { &hf_rrc_v950NonCriticalExtensions_06, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v950NonCriticalExtensions_06 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_r9_08(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_r9_08, T_r9_08_sequence);

  return offset;
}


static const per_sequence_t RadioBearerSetup_r10_IEs_sequence[] = {
  { &hf_rrc_integrityProtectionModeInfo_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_IntegrityProtectionModeInfo_r7 },
  { &hf_rrc_cipheringModeInfo_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CipheringModeInfo_r7 },
  { &hf_rrc_sr_vcc_Info     , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_SR_VCC_Info },
  { &hf_rrc_activationTime  , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_ActivationTime },
  { &hf_rrc_new_U_RNTI      , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_U_RNTI },
  { &hf_rrc_new_C_RNTI      , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_C_RNTI },
  { &hf_rrc_new_DSCH_RNTI   , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DSCH_RNTI },
  { &hf_rrc_new_H_RNTI      , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_H_RNTI },
  { &hf_rrc_newPrimary_E_RNTI, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_E_RNTI },
  { &hf_rrc_newSecondary_E_RNTI, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_E_RNTI },
  { &hf_rrc_rrc_StateIndicator, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RRC_StateIndicator },
  { &hf_rrc_utran_DRX_CycleLengthCoeff_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UTRAN_DRX_CycleLengthCoefficient_r7 },
  { &hf_rrc_ura_Identity    , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_URA_Identity },
  { &hf_rrc_supportForChangeOfUE_Capability, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_BOOLEAN },
  { &hf_rrc_cn_InformationInfo_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CN_InformationInfo_r6 },
  { &hf_rrc_srb_InformationSetupList_04, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_SRB_InformationSetupList_r8 },
  { &hf_rrc_rab_InformationSetupList_05, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RAB_InformationSetupList_r8 },
  { &hf_rrc_rab_InformationReconfigList_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RAB_InformationReconfigList_r8 },
  { &hf_rrc_rb_InformationReconfigList_05, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RB_InformationReconfigList_r8 },
  { &hf_rrc_rb_InformationAffectedList_04, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RB_InformationAffectedList_r8 },
  { &hf_rrc_dl_CounterSynchronisationInfo_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_CounterSynchronisationInfo_r5 },
  { &hf_rrc_pdcp_ROHC_TargetMode, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_PDCP_ROHC_TargetMode },
  { &hf_rrc_ul_CommonTransChInfo_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_CommonTransChInfo_r4 },
  { &hf_rrc_ul_deletedTransChInfoList_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_DeletedTransChInfoList_r6 },
  { &hf_rrc_ul_AddReconfTransChInfoList_03, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_AddReconfTransChInfoList_r8 },
  { &hf_rrc_dl_CommonTransChInfo_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_CommonTransChInfo_r4 },
  { &hf_rrc_dl_DeletedTransChInfoList_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_DeletedTransChInfoList_r7 },
  { &hf_rrc_dl_AddReconfTransChInfoList_04, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_AddReconfTransChInfoList_r9 },
  { &hf_rrc_frequencyInfo   , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_FrequencyInfo },
  { &hf_rrc_multi_frequencyInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_Multi_frequencyInfo_LCR_r7 },
  { &hf_rrc_dtx_drx_TimingInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DTX_DRX_TimingInfo_r7 },
  { &hf_rrc_dtx_drx_Info    , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DTX_DRX_Info_r7 },
  { &hf_rrc_hs_scch_LessInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_HS_SCCH_LessInfo_r7 },
  { &hf_rrc_mimoParameters_04, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MIMO_Parameters_r9 },
  { &hf_rrc_maxAllowedUL_TX_Power, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MaxAllowedUL_TX_Power },
  { &hf_rrc_ul_DPCH_Info_01 , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_DPCH_Info_r7 },
  { &hf_rrc_ul_EDCH_Information_04, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_EDCH_Information_r9 },
  { &hf_rrc_ul_SecondaryCellInfoFDD, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_SecondaryCellInfoFDD },
  { &hf_rrc_ul_MulticarrierEDCHInfo_TDD128, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_MulticarrierEDCHInfo_TDD128 },
  { &hf_rrc_dl_HSPDSCH_Information_06, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_HSPDSCH_Information_r9 },
  { &hf_rrc_dl_CommonInformation_06, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_CommonInformation_r10 },
  { &hf_rrc_dl_InformationPerRL_List_05, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_InformationPerRL_List_r8 },
  { &hf_rrc_dl_SecondaryCellInfoFDD_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_SecondaryCellInfoFDD_r10 },
  { &hf_rrc_additionalDLSecCellInfoListFDD, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_AdditionalDLSecCellInfoListFDD },
  { &hf_rrc_controlChannelDRXInfo_TDD128, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_ControlChannelDRXInfo_TDD128_r8 },
  { &hf_rrc_sps_Information_TDD128, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_SPS_Information_TDD128_r8 },
  { &hf_rrc_mu_MIMO_Info_TDD128, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MU_MIMO_Info_TDD128 },
  { &hf_rrc_mbms_PL_ServiceRestrictInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MBMS_PL_ServiceRestrictInfo_r6 },
  { &hf_rrc_cellDCHMeasOccasionInfo_TDD128, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CellDCHMeasOccasionInfo_TDD128_r9 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_RadioBearerSetup_r10_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_RadioBearerSetup_r10_IEs, RadioBearerSetup_r10_IEs_sequence);

  return offset;
}


static const per_sequence_t RadioBearerSetup_vbxyext_IEs_sequence[] = {
  { &hf_rrc_ulOLTDInfoFDD   , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_OLTD_InfoFDD },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_RadioBearerSetup_vbxyext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_RadioBearerSetup_vbxyext_IEs, RadioBearerSetup_vbxyext_IEs_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions_113_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_nonCriticalExtensions_113(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_nonCriticalExtensions_113, T_nonCriticalExtensions_113_sequence);

  return offset;
}


static const per_sequence_t T_vbxyNonCriticalExtensions_11_sequence[] = {
  { &hf_rrc_radioBearerSetup_vbxyext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RadioBearerSetup_vbxyext_IEs },
  { &hf_rrc_nonCriticalExtensions_113, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_nonCriticalExtensions_113 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_vbxyNonCriticalExtensions_11(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_vbxyNonCriticalExtensions_11, T_vbxyNonCriticalExtensions_11_sequence);

  return offset;
}


static const per_sequence_t T_v9c0NonCriticalExtensions_09_sequence[] = {
  { &hf_rrc_radioBearerSetup_v9c0ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RadioBearerSetup_v9c0ext_IEs },
  { &hf_rrc_vbxyNonCriticalExtensions_11, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_vbxyNonCriticalExtensions_11 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v9c0NonCriticalExtensions_09(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v9c0NonCriticalExtensions_09, T_v9c0NonCriticalExtensions_09_sequence);

  return offset;
}


static const per_sequence_t T_r10_08_sequence[] = {
  { &hf_rrc_radioBearerSetup_r10, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RadioBearerSetup_r10_IEs },
  { &hf_rrc_radioBearerSetup_r10_add_ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_BIT_STRING },
  { &hf_rrc_v9c0NonCriticalExtensions_09, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v9c0NonCriticalExtensions_09 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_r10_08(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_r10_08, T_r10_08_sequence);

  return offset;
}


static const value_string rrc_RLC_InfoChoice_r11_vals[] = {
  {   0, "rlc-Info" },
  {   1, "same-as-RB" },
  { 0, NULL }
};

static const per_choice_t RLC_InfoChoice_r11_choice[] = {
  {   0, &hf_rrc_rlc_Info_04     , ASN1_NO_EXTENSIONS     , dissect_rrc_RLC_Info_r11 },
  {   1, &hf_rrc_same_as_RB      , ASN1_NO_EXTENSIONS     , dissect_rrc_RB_Identity },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_RLC_InfoChoice_r11(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_RLC_InfoChoice_r11, RLC_InfoChoice_r11_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t RB_InformationSetup_r11_sequence[] = {
  { &hf_rrc_rb_Identity     , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RB_Identity },
  { &hf_rrc_pdcp_Info_01    , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_PDCP_Info_r4 },
  { &hf_rrc_rlc_InfoChoice_04, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RLC_InfoChoice_r11 },
  { &hf_rrc_rb_MappingInfo_04, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RB_MappingInfo_r8 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_RB_InformationSetup_r11(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_RB_InformationSetup_r11, RB_InformationSetup_r11_sequence);

  return offset;
}


static const per_sequence_t RB_InformationSetupList_r11_sequence_of[1] = {
  { &hf_rrc_RB_InformationSetupList_r11_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RB_InformationSetup_r11 },
};

static int
dissect_rrc_RB_InformationSetupList_r11(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_RB_InformationSetupList_r11, RB_InformationSetupList_r11_sequence_of,
                                                  1, maxRBperRAB, FALSE);

  return offset;
}


static const per_sequence_t RAB_InformationSetup_r11_sequence[] = {
  { &hf_rrc_rab_Info_03     , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RAB_Info_r7 },
  { &hf_rrc_cs_HSPA_Information, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CS_HSPA_Information },
  { &hf_rrc_rab_InfoReplace , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RAB_InfoReplace },
  { &hf_rrc_rb_InformationSetupList_06, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RB_InformationSetupList_r11 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_RAB_InformationSetup_r11(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_RAB_InformationSetup_r11, RAB_InformationSetup_r11_sequence);

  return offset;
}


static const per_sequence_t RAB_InformationSetupList_r11_sequence_of[1] = {
  { &hf_rrc_RAB_InformationSetupList_r11_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RAB_InformationSetup_r11 },
};

static int
dissect_rrc_RAB_InformationSetupList_r11(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_RAB_InformationSetupList_r11, RAB_InformationSetupList_r11_sequence_of,
                                                  1, maxRABsetup, FALSE);

  return offset;
}


static const per_sequence_t RadioBearerSetup_r11_IEs_sequence[] = {
  { &hf_rrc_integrityProtectionModeInfo_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_IntegrityProtectionModeInfo_r7 },
  { &hf_rrc_cipheringModeInfo_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CipheringModeInfo_r7 },
  { &hf_rrc_sr_vcc_Info     , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_SR_VCC_Info },
  { &hf_rrc_activationTime  , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_ActivationTime },
  { &hf_rrc_new_U_RNTI      , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_U_RNTI },
  { &hf_rrc_new_C_RNTI      , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_C_RNTI },
  { &hf_rrc_new_DSCH_RNTI   , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DSCH_RNTI },
  { &hf_rrc_new_H_RNTI      , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_H_RNTI },
  { &hf_rrc_newPrimary_E_RNTI, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_E_RNTI },
  { &hf_rrc_newSecondary_E_RNTI, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_E_RNTI },
  { &hf_rrc_rrc_StateIndicator, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RRC_StateIndicator },
  { &hf_rrc_utran_DRX_CycleLengthCoeff_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UTRAN_DRX_CycleLengthCoefficient_r7 },
  { &hf_rrc_ura_Identity    , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_URA_Identity },
  { &hf_rrc_supportForChangeOfUE_Capability, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_BOOLEAN },
  { &hf_rrc_cn_InformationInfo_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CN_InformationInfo_r6 },
  { &hf_rrc_srb_InformationSetupList_04, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_SRB_InformationSetupList_r8 },
  { &hf_rrc_rab_InformationSetupList_07, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RAB_InformationSetupList_r11 },
  { &hf_rrc_rab_InformationReconfigList_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RAB_InformationReconfigList_r8 },
  { &hf_rrc_rb_InformationReconfigList_06, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RB_InformationReconfigList_r11 },
  { &hf_rrc_rb_InformationAffectedList_04, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RB_InformationAffectedList_r8 },
  { &hf_rrc_dl_CounterSynchronisationInfo_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_CounterSynchronisationInfo_r5 },
  { &hf_rrc_pdcp_ROHC_TargetMode, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_PDCP_ROHC_TargetMode },
  { &hf_rrc_ul_CommonTransChInfo_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_CommonTransChInfo_r4 },
  { &hf_rrc_ul_deletedTransChInfoList_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_DeletedTransChInfoList_r6 },
  { &hf_rrc_ul_AddReconfTransChInfoList_03, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_AddReconfTransChInfoList_r8 },
  { &hf_rrc_dl_CommonTransChInfo_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_CommonTransChInfo_r4 },
  { &hf_rrc_dl_DeletedTransChInfoList_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_DeletedTransChInfoList_r7 },
  { &hf_rrc_dl_AddReconfTransChInfoList_06, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_AddReconfTransChInfoList_r11 },
  { &hf_rrc_frequencyInfo   , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_FrequencyInfo },
  { &hf_rrc_multi_frequencyInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_Multi_frequencyInfo_LCR_r7 },
  { &hf_rrc_dtx_drx_TimingInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DTX_DRX_TimingInfo_r7 },
  { &hf_rrc_dtx_drx_Info    , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DTX_DRX_Info_r7 },
  { &hf_rrc_hs_scch_LessInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_HS_SCCH_LessInfo_r7 },
  { &hf_rrc_mimoParameters_04, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MIMO_Parameters_r9 },
  { &hf_rrc_mimo4x4Parameters, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MIMO4x4_Parameters },
  { &hf_rrc_maxAllowedUL_TX_Power, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MaxAllowedUL_TX_Power },
  { &hf_rrc_ul_DPCH_Info_02 , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_DPCH_Info_r11 },
  { &hf_rrc_ul_EDCH_Information_05, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_EDCH_Information_r11 },
  { &hf_rrc_ul_SecondaryCellInfoFDD, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_SecondaryCellInfoFDD },
  { &hf_rrc_ul_MulticarrierEDCHInfo_TDD128, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_MulticarrierEDCHInfo_TDD128 },
  { &hf_rrc_ul_CLTD_InfoFDD , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_CLTD_InfoFDD },
  { &hf_rrc_ul_OLTD_InfoFDD , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_OLTD_InfoFDD },
  { &hf_rrc_dl_HSPDSCH_Information_07, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_HSPDSCH_Information_r11 },
  { &hf_rrc_dl_CommonInformation_07, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_CommonInformation_r11 },
  { &hf_rrc_dl_InformationPerRL_List_06, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_InformationPerRL_List_r11 },
  { &hf_rrc_dl_SecondaryCellInfoFDD_03, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_SecondaryCellInfoFDD_r11 },
  { &hf_rrc_additionalDLSecCellInfoListFDD_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_AdditionalDLSecCellInfoListFDD_r11 },
  { &hf_rrc_additionalDLSecCellInfoListFDD2, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_AdditionalDLSecCellInfoListFDD2_r11 },
  { &hf_rrc_commonERGCHInfoFDD, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CommonERGCHInfoFDD },
  { &hf_rrc_controlChannelDRXInfo_TDD128, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_ControlChannelDRXInfo_TDD128_r8 },
  { &hf_rrc_sps_Information_TDD128, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_SPS_Information_TDD128_r8 },
  { &hf_rrc_mu_MIMO_Info_TDD128, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MU_MIMO_Info_TDD128 },
  { &hf_rrc_mbms_PL_ServiceRestrictInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MBMS_PL_ServiceRestrictInfo_r6 },
  { &hf_rrc_cellDCHMeasOccasionInfo_TDD128, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CellDCHMeasOccasionInfo_TDD128_r9 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_RadioBearerSetup_r11_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_RadioBearerSetup_r11_IEs, RadioBearerSetup_r11_IEs_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions_114_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_nonCriticalExtensions_114(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_nonCriticalExtensions_114, T_nonCriticalExtensions_114_sequence);

  return offset;
}


static const per_sequence_t T_r11_09_sequence[] = {
  { &hf_rrc_radioBearerSetup_r11, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RadioBearerSetup_r11_IEs },
  { &hf_rrc_radioBearerSetup_r11_add_ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_BIT_STRING },
  { &hf_rrc_nonCriticalExtensions_114, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_nonCriticalExtensions_114 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_r11_09(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_r11_09, T_r11_09_sequence);

  return offset;
}


static const per_sequence_t T_criticalExtensions_90_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_criticalExtensions_90(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_criticalExtensions_90, T_criticalExtensions_90_sequence);

  return offset;
}


static const value_string rrc_T_criticalExtensions_89_vals[] = {
  {   0, "r11" },
  {   1, "criticalExtensions" },
  { 0, NULL }
};

static const per_choice_t T_criticalExtensions_89_choice[] = {
  {   0, &hf_rrc_r11_09          , ASN1_NO_EXTENSIONS     , dissect_rrc_T_r11_09 },
  {   1, &hf_rrc_criticalExtensions_91, ASN1_NO_EXTENSIONS     , dissect_rrc_T_criticalExtensions_90 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_criticalExtensions_89(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_criticalExtensions_89, T_criticalExtensions_89_choice,
                                 NULL);

  return offset;
}


static const value_string rrc_T_criticalExtensions_88_vals[] = {
  {   0, "r10" },
  {   1, "criticalExtensions" },
  { 0, NULL }
};

static const per_choice_t T_criticalExtensions_88_choice[] = {
  {   0, &hf_rrc_r10_08          , ASN1_NO_EXTENSIONS     , dissect_rrc_T_r10_08 },
  {   1, &hf_rrc_criticalExtensions_90, ASN1_NO_EXTENSIONS     , dissect_rrc_T_criticalExtensions_89 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_criticalExtensions_88(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_criticalExtensions_88, T_criticalExtensions_88_choice,
                                 NULL);

  return offset;
}


static const value_string rrc_T_criticalExtensions_87_vals[] = {
  {   0, "r9" },
  {   1, "criticalExtensions" },
  { 0, NULL }
};

static const per_choice_t T_criticalExtensions_87_choice[] = {
  {   0, &hf_rrc_r9_08           , ASN1_NO_EXTENSIONS     , dissect_rrc_T_r9_08 },
  {   1, &hf_rrc_criticalExtensions_89, ASN1_NO_EXTENSIONS     , dissect_rrc_T_criticalExtensions_88 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_criticalExtensions_87(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_criticalExtensions_87, T_criticalExtensions_87_choice,
                                 NULL);

  return offset;
}


static const value_string rrc_T_criticalExtensions_86_vals[] = {
  {   0, "r8" },
  {   1, "criticalExtensions" },
  { 0, NULL }
};

static const per_choice_t T_criticalExtensions_86_choice[] = {
  {   0, &hf_rrc_r8_09           , ASN1_NO_EXTENSIONS     , dissect_rrc_T_r8_09 },
  {   1, &hf_rrc_criticalExtensions_88, ASN1_NO_EXTENSIONS     , dissect_rrc_T_criticalExtensions_87 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_criticalExtensions_86(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_criticalExtensions_86, T_criticalExtensions_86_choice,
                                 NULL);

  return offset;
}


static const value_string rrc_T_criticalExtensions_85_vals[] = {
  {   0, "r7" },
  {   1, "criticalExtensions" },
  { 0, NULL }
};

static const per_choice_t T_criticalExtensions_85_choice[] = {
  {   0, &hf_rrc_r7_08           , ASN1_NO_EXTENSIONS     , dissect_rrc_T_r7_08 },
  {   1, &hf_rrc_criticalExtensions_87, ASN1_NO_EXTENSIONS     , dissect_rrc_T_criticalExtensions_86 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_criticalExtensions_85(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_criticalExtensions_85, T_criticalExtensions_85_choice,
                                 NULL);

  return offset;
}


static const value_string rrc_T_criticalExtensions_84_vals[] = {
  {   0, "r6" },
  {   1, "criticalExtensions" },
  { 0, NULL }
};

static const per_choice_t T_criticalExtensions_84_choice[] = {
  {   0, &hf_rrc_r6_09           , ASN1_NO_EXTENSIONS     , dissect_rrc_T_r6_09 },
  {   1, &hf_rrc_criticalExtensions_86, ASN1_NO_EXTENSIONS     , dissect_rrc_T_criticalExtensions_85 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_criticalExtensions_84(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_criticalExtensions_84, T_criticalExtensions_84_choice,
                                 NULL);

  return offset;
}


static const value_string rrc_T_criticalExtensions_83_vals[] = {
  {   0, "r5" },
  {   1, "criticalExtensions" },
  { 0, NULL }
};

static const per_choice_t T_criticalExtensions_83_choice[] = {
  {   0, &hf_rrc_r5_07           , ASN1_NO_EXTENSIONS     , dissect_rrc_T_r5_07 },
  {   1, &hf_rrc_criticalExtensions_85, ASN1_NO_EXTENSIONS     , dissect_rrc_T_criticalExtensions_84 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_criticalExtensions_83(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_criticalExtensions_83, T_criticalExtensions_83_choice,
                                 NULL);

  return offset;
}


static const value_string rrc_T_criticalExtensions_82_vals[] = {
  {   0, "r4" },
  {   1, "criticalExtensions" },
  { 0, NULL }
};

static const per_choice_t T_criticalExtensions_82_choice[] = {
  {   0, &hf_rrc_r4_08           , ASN1_NO_EXTENSIONS     , dissect_rrc_T_r4_08 },
  {   1, &hf_rrc_criticalExtensions_84, ASN1_NO_EXTENSIONS     , dissect_rrc_T_criticalExtensions_83 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_criticalExtensions_82(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_criticalExtensions_82, T_criticalExtensions_82_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t T_later_than_r3_14_sequence[] = {
  { &hf_rrc_rrc_TransactionIdentifier, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RRC_TransactionIdentifier },
  { &hf_rrc_criticalExtensions_83, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_criticalExtensions_82 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_later_than_r3_14(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_later_than_r3_14, T_later_than_r3_14_sequence);

  return offset;
}


static const value_string rrc_RadioBearerSetup_vals[] = {
  {   0, "r3" },
  {   1, "later-than-r3" },
  { 0, NULL }
};

static const per_choice_t RadioBearerSetup_choice[] = {
  {   0, &hf_rrc_r3_16           , ASN1_NO_EXTENSIONS     , dissect_rrc_T_r3_16 },
  {   1, &hf_rrc_later_than_r3_14, ASN1_NO_EXTENSIONS     , dissect_rrc_T_later_than_r3_14 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_RadioBearerSetup(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
#line 349 "../../asn1/rrc/rrc.cnf"
  col_append_str(actx->pinfo->cinfo, COL_INFO, "RadioBearerSetup");
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_RadioBearerSetup, RadioBearerSetup_choice,
                                 NULL);




  return offset;
}



static int
dissect_rrc_N_308(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            1U, 8U, NULL, FALSE);

  return offset;
}


static const value_string rrc_ReleaseCause_vals[] = {
  {   0, "normalEvent" },
  {   1, "unspecified" },
  {   2, "pre-emptiveRelease" },
  {   3, "congestion" },
  {   4, "re-establishmentReject" },
  {   5, "directedsignallingconnectionre-establishment" },
  {   6, "userInactivity" },
  {   7, "spare" },
  { 0, NULL }
};


static int
dissect_rrc_ReleaseCause(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     8, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t GSM_BA_Range_sequence[] = {
  { &hf_rrc_gsmLowRangeUARFCN, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UARFCN },
  { &hf_rrc_gsmUpRangeUARFCN, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UARFCN },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_GSM_BA_Range(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_GSM_BA_Range, GSM_BA_Range_sequence);

  return offset;
}


static const per_sequence_t GSM_BA_Range_List_sequence_of[1] = {
  { &hf_rrc_GSM_BA_Range_List_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_GSM_BA_Range },
};

static int
dissect_rrc_GSM_BA_Range_List(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_GSM_BA_Range_List, GSM_BA_Range_List_sequence_of,
                                                  1, maxNumGSMFreqRanges, FALSE);

  return offset;
}


static const per_sequence_t FDD_UMTS_Frequency_List_sequence_of[1] = {
  { &hf_rrc_FDD_UMTS_Frequency_List_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_FrequencyInfoFDD },
};

static int
dissect_rrc_FDD_UMTS_Frequency_List(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_FDD_UMTS_Frequency_List, FDD_UMTS_Frequency_List_sequence_of,
                                                  1, maxNumFDDFreqs, FALSE);

  return offset;
}


static const per_sequence_t TDD_UMTS_Frequency_List_sequence_of[1] = {
  { &hf_rrc_TDD_UMTS_Frequency_List_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_FrequencyInfoTDD },
};

static int
dissect_rrc_TDD_UMTS_Frequency_List(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_TDD_UMTS_Frequency_List, TDD_UMTS_Frequency_List_sequence_of,
                                                  1, maxNumTDDFreqs, FALSE);

  return offset;
}


static const per_sequence_t FrequencyInfoCDMA2000_sequence[] = {
  { &hf_rrc_band_Class      , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_5 },
  { &hf_rrc_cdma_Freq       , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_11 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_FrequencyInfoCDMA2000(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_FrequencyInfoCDMA2000, FrequencyInfoCDMA2000_sequence);

  return offset;
}


static const per_sequence_t CDMA2000_UMTS_Frequency_List_sequence_of[1] = {
  { &hf_rrc_CDMA2000_UMTS_Frequency_List_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_FrequencyInfoCDMA2000 },
};

static int
dissect_rrc_CDMA2000_UMTS_Frequency_List(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_CDMA2000_UMTS_Frequency_List, CDMA2000_UMTS_Frequency_List_sequence_of,
                                                  1, maxNumCDMA2000Freqs, FALSE);

  return offset;
}


static const per_sequence_t Rplmn_Information_sequence[] = {
  { &hf_rrc_gsm_BA_Range_List, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_GSM_BA_Range_List },
  { &hf_rrc_fdd_UMTS_Frequency_List, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_FDD_UMTS_Frequency_List },
  { &hf_rrc_tdd_UMTS_Frequency_List, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_TDD_UMTS_Frequency_List },
  { &hf_rrc_cdma2000_UMTS_Frequency_List, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CDMA2000_UMTS_Frequency_List },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_Rplmn_Information(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_Rplmn_Information, Rplmn_Information_sequence);

  return offset;
}


static const per_sequence_t RRCConnectionRelease_r3_IEs_sequence[] = {
  { &hf_rrc_rrc_TransactionIdentifier, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RRC_TransactionIdentifier },
  { &hf_rrc_n_308           , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_N_308 },
  { &hf_rrc_releaseCause    , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_ReleaseCause },
  { &hf_rrc_rplmn_information, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_Rplmn_Information },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_RRCConnectionRelease_r3_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_RRCConnectionRelease_r3_IEs, RRCConnectionRelease_r3_IEs_sequence);

  return offset;
}


static const value_string rrc_InterRATInfo_vals[] = {
  {   0, "gsm" },
  { 0, NULL }
};


static int
dissect_rrc_InterRATInfo(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t GSM_TargetCellInfo_sequence[] = {
  { &hf_rrc_bcch_ARFCN      , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BCCH_ARFCN },
  { &hf_rrc_frequency_band  , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_Frequency_Band },
  { &hf_rrc_bsic            , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_BSIC },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_GSM_TargetCellInfo(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_GSM_TargetCellInfo, GSM_TargetCellInfo_sequence);

  return offset;
}


static const per_sequence_t GSM_TargetCellInfoList_sequence_of[1] = {
  { &hf_rrc_GSM_TargetCellInfoList_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_GSM_TargetCellInfo },
};

static int
dissect_rrc_GSM_TargetCellInfoList(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_GSM_TargetCellInfoList, GSM_TargetCellInfoList_sequence_of,
                                                  1, maxGSMTargetCells, FALSE);

  return offset;
}


static const per_sequence_t InterRATInfo_r6_sequence[] = {
  { &hf_rrc_rat_01          , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_InterRATInfo },
  { &hf_rrc_gsm_TargetCellInfoList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_GSM_TargetCellInfoList },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_InterRATInfo_r6(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_InterRATInfo_r6, InterRATInfo_r6_sequence);

  return offset;
}


static const value_string rrc_RedirectionInfo_r6_vals[] = {
  {   0, "frequencyInfo" },
  {   1, "interRATInfo" },
  { 0, NULL }
};

static const per_choice_t RedirectionInfo_r6_choice[] = {
  {   0, &hf_rrc_frequencyInfo   , ASN1_NO_EXTENSIONS     , dissect_rrc_FrequencyInfo },
  {   1, &hf_rrc_interRATInfo_01 , ASN1_NO_EXTENSIONS     , dissect_rrc_InterRATInfo_r6 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_RedirectionInfo_r6(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_RedirectionInfo_r6, RedirectionInfo_r6_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t RRCConnectionRelease_v690ext_IEs_sequence[] = {
  { &hf_rrc_redirectionInfo_v690ext_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RedirectionInfo_r6 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_RRCConnectionRelease_v690ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_RRCConnectionRelease_v690ext_IEs, RRCConnectionRelease_v690ext_IEs_sequence);

  return offset;
}


static const per_sequence_t RRCConnectionRelease_v770ext_IEs_sequence[] = {
  { &hf_rrc_ueMobilityStateIndicator, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_High_MobilityDetected },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_RRCConnectionRelease_v770ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_RRCConnectionRelease_v770ext_IEs, RRCConnectionRelease_v770ext_IEs_sequence);

  return offset;
}


static const per_sequence_t EUTRA_TargetFreqInfo_sequence[] = {
  { &hf_rrc_dlEUTRACarrierFreq, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_EARFCN },
  { &hf_rrc_eutraBlacklistedCellPerFreqList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_EUTRA_BlacklistedCellPerFreqList },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_EUTRA_TargetFreqInfo(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_EUTRA_TargetFreqInfo, EUTRA_TargetFreqInfo_sequence);

  return offset;
}


static const per_sequence_t EUTRA_TargetFreqInfoList_sequence_of[1] = {
  { &hf_rrc_EUTRA_TargetFreqInfoList_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_EUTRA_TargetFreqInfo },
};

static int
dissect_rrc_EUTRA_TargetFreqInfoList(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_EUTRA_TargetFreqInfoList, EUTRA_TargetFreqInfoList_sequence_of,
                                                  1, maxEUTRATargetFreqs, FALSE);

  return offset;
}


static const per_sequence_t T_eutra_01_sequence[] = {
  { &hf_rrc_eutra_TargetFreqInfoList, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_EUTRA_TargetFreqInfoList },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_eutra_01(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_eutra_01, T_eutra_01_sequence);

  return offset;
}


static const value_string rrc_InterRATInfo_v860ext_vals[] = {
  {   0, "gsm" },
  {   1, "eutra" },
  { 0, NULL }
};

static const per_choice_t InterRATInfo_v860ext_choice[] = {
  {   0, &hf_rrc_gsm_09          , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   1, &hf_rrc_eutra_01        , ASN1_NO_EXTENSIONS     , dissect_rrc_T_eutra_01 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_InterRATInfo_v860ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_InterRATInfo_v860ext, InterRATInfo_v860ext_choice,
                                 NULL);

  return offset;
}


static const value_string rrc_RedirectionInfo_v860ext_vals[] = {
  {   0, "frequencyInfo" },
  {   1, "interRATInfo" },
  { 0, NULL }
};

static const per_choice_t RedirectionInfo_v860ext_choice[] = {
  {   0, &hf_rrc_frequencyInfo_03, ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   1, &hf_rrc_interRATInfo_02 , ASN1_NO_EXTENSIONS     , dissect_rrc_InterRATInfo_v860ext },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_RedirectionInfo_v860ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_RedirectionInfo_v860ext, RedirectionInfo_v860ext_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t RRCConnectionRelease_v860ext_IEs_sequence[] = {
  { &hf_rrc_redirectionInfo_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RedirectionInfo_v860ext },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_RRCConnectionRelease_v860ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_RRCConnectionRelease_v860ext_IEs, RRCConnectionRelease_v860ext_IEs_sequence);

  return offset;
}



static int
dissect_rrc_ExtendedWaitTime(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            1U, 1800U, NULL, FALSE);

  return offset;
}


static const per_sequence_t RRCConnectionRelease_va40ext_IEs_sequence[] = {
  { &hf_rrc_extendedWaitTime, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_ExtendedWaitTime },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_RRCConnectionRelease_va40ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_RRCConnectionRelease_va40ext_IEs, RRCConnectionRelease_va40ext_IEs_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions_119_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_nonCriticalExtensions_119(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_nonCriticalExtensions_119, T_nonCriticalExtensions_119_sequence);

  return offset;
}


static const per_sequence_t T_va40NonCriticalExtensions_06_sequence[] = {
  { &hf_rrc_rrcConnectionRelease_va40ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RRCConnectionRelease_va40ext_IEs },
  { &hf_rrc_nonCriticalExtensions_119, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_nonCriticalExtensions_119 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_va40NonCriticalExtensions_06(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_va40NonCriticalExtensions_06, T_va40NonCriticalExtensions_06_sequence);

  return offset;
}


static const per_sequence_t T_v860NonCriticalExtensions_11_sequence[] = {
  { &hf_rrc_rrcConnectionRelease_v860ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RRCConnectionRelease_v860ext_IEs },
  { &hf_rrc_va40NonCriticalExtensions_06, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_va40NonCriticalExtensions_06 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v860NonCriticalExtensions_11(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v860NonCriticalExtensions_11, T_v860NonCriticalExtensions_11_sequence);

  return offset;
}


static const per_sequence_t T_v770NonCriticalExtensions_21_sequence[] = {
  { &hf_rrc_rrcConnectionRelease_v770ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RRCConnectionRelease_v770ext_IEs },
  { &hf_rrc_v860NonCriticalExtensions_11, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v860NonCriticalExtensions_11 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v770NonCriticalExtensions_21(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v770NonCriticalExtensions_21, T_v770NonCriticalExtensions_21_sequence);

  return offset;
}


static const per_sequence_t T_v690NonCriticalExtensions_26_sequence[] = {
  { &hf_rrc_rrcConnectionRelease_v690ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RRCConnectionRelease_v690ext_IEs },
  { &hf_rrc_v770NonCriticalExtensions_21, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v770NonCriticalExtensions_21 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v690NonCriticalExtensions_26(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v690NonCriticalExtensions_26, T_v690NonCriticalExtensions_26_sequence);

  return offset;
}


static const per_sequence_t T_laterNonCriticalExtensions_38_sequence[] = {
  { &hf_rrc_rrcConnectionRelease_r3_add_ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_BIT_STRING },
  { &hf_rrc_v690NonCriticalExtensions_26, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v690NonCriticalExtensions_26 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_laterNonCriticalExtensions_38(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_laterNonCriticalExtensions_38, T_laterNonCriticalExtensions_38_sequence);

  return offset;
}


static const per_sequence_t T_r3_18_sequence[] = {
  { &hf_rrc_rrcConnectionRelease_r3, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RRCConnectionRelease_r3_IEs },
  { &hf_rrc_laterNonCriticalExtensions_38, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_laterNonCriticalExtensions_38 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_r3_18(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_r3_18, T_r3_18_sequence);

  return offset;
}


static const per_sequence_t Rplmn_Information_r4_sequence[] = {
  { &hf_rrc_gsm_BA_Range_List, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_GSM_BA_Range_List },
  { &hf_rrc_fdd_UMTS_Frequency_List, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_FDD_UMTS_Frequency_List },
  { &hf_rrc_tdd384_UMTS_Frequency_List, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_TDD_UMTS_Frequency_List },
  { &hf_rrc_tdd128_UMTS_Frequency_List, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_TDD_UMTS_Frequency_List },
  { &hf_rrc_cdma2000_UMTS_Frequency_List, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CDMA2000_UMTS_Frequency_List },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_Rplmn_Information_r4(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_Rplmn_Information_r4, Rplmn_Information_r4_sequence);

  return offset;
}


static const per_sequence_t RRCConnectionRelease_r4_IEs_sequence[] = {
  { &hf_rrc_n_308           , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_N_308 },
  { &hf_rrc_releaseCause    , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_ReleaseCause },
  { &hf_rrc_rplmn_information_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_Rplmn_Information_r4 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_RRCConnectionRelease_r4_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_RRCConnectionRelease_r4_IEs, RRCConnectionRelease_r4_IEs_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions_120_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_nonCriticalExtensions_120(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_nonCriticalExtensions_120, T_nonCriticalExtensions_120_sequence);

  return offset;
}


static const per_sequence_t T_va40NonCriticalExtensions_07_sequence[] = {
  { &hf_rrc_rrcConnectionRelease_va40ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RRCConnectionRelease_va40ext_IEs },
  { &hf_rrc_nonCriticalExtensions_120, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_nonCriticalExtensions_120 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_va40NonCriticalExtensions_07(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_va40NonCriticalExtensions_07, T_va40NonCriticalExtensions_07_sequence);

  return offset;
}


static const per_sequence_t T_v860NonCriticalExtensions_12_sequence[] = {
  { &hf_rrc_rrcConnectionRelease_v860ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RRCConnectionRelease_v860ext_IEs },
  { &hf_rrc_va40NonCriticalExtensions_07, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_va40NonCriticalExtensions_07 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v860NonCriticalExtensions_12(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v860NonCriticalExtensions_12, T_v860NonCriticalExtensions_12_sequence);

  return offset;
}


static const per_sequence_t T_v770NonCriticalExtensions_22_sequence[] = {
  { &hf_rrc_rrcConnectionRelease_v770ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RRCConnectionRelease_v770ext_IEs },
  { &hf_rrc_v860NonCriticalExtensions_12, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v860NonCriticalExtensions_12 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v770NonCriticalExtensions_22(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v770NonCriticalExtensions_22, T_v770NonCriticalExtensions_22_sequence);

  return offset;
}


static const per_sequence_t T_v690NonCriticalExtensions_27_sequence[] = {
  { &hf_rrc_rrcConnectionRelease_v690ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RRCConnectionRelease_v690ext_IEs },
  { &hf_rrc_v770NonCriticalExtensions_22, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v770NonCriticalExtensions_22 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v690NonCriticalExtensions_27(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v690NonCriticalExtensions_27, T_v690NonCriticalExtensions_27_sequence);

  return offset;
}


static const per_sequence_t T_v4d0NonCriticalExtensions_09_sequence[] = {
  { &hf_rrc_rrcConnectionRelease_r4_add_ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_BIT_STRING },
  { &hf_rrc_v690NonCriticalExtensions_27, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v690NonCriticalExtensions_27 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v4d0NonCriticalExtensions_09(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v4d0NonCriticalExtensions_09, T_v4d0NonCriticalExtensions_09_sequence);

  return offset;
}


static const per_sequence_t T_r4_09_sequence[] = {
  { &hf_rrc_rrcConnectionRelease_r4, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RRCConnectionRelease_r4_IEs },
  { &hf_rrc_v4d0NonCriticalExtensions_09, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v4d0NonCriticalExtensions_09 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_r4_09(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_r4_09, T_r4_09_sequence);

  return offset;
}


static const per_sequence_t T_criticalExtensions_93_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_criticalExtensions_93(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_criticalExtensions_93, T_criticalExtensions_93_sequence);

  return offset;
}


static const value_string rrc_T_criticalExtensions_92_vals[] = {
  {   0, "r4" },
  {   1, "criticalExtensions" },
  { 0, NULL }
};

static const per_choice_t T_criticalExtensions_92_choice[] = {
  {   0, &hf_rrc_r4_09           , ASN1_NO_EXTENSIONS     , dissect_rrc_T_r4_09 },
  {   1, &hf_rrc_criticalExtensions_94, ASN1_NO_EXTENSIONS     , dissect_rrc_T_criticalExtensions_93 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_criticalExtensions_92(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_criticalExtensions_92, T_criticalExtensions_92_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t T_later_than_r3_16_sequence[] = {
  { &hf_rrc_rrc_TransactionIdentifier, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RRC_TransactionIdentifier },
  { &hf_rrc_criticalExtensions_93, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_criticalExtensions_92 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_later_than_r3_16(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_later_than_r3_16, T_later_than_r3_16_sequence);

  return offset;
}


static const value_string rrc_RRCConnectionRelease_vals[] = {
  {   0, "r3" },
  {   1, "later-than-r3" },
  { 0, NULL }
};

static const per_choice_t RRCConnectionRelease_choice[] = {
  {   0, &hf_rrc_r3_18           , ASN1_NO_EXTENSIONS     , dissect_rrc_T_r3_18 },
  {   1, &hf_rrc_later_than_r3_16, ASN1_NO_EXTENSIONS     , dissect_rrc_T_later_than_r3_16 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_RRCConnectionRelease(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
#line 365 "../../asn1/rrc/rrc.cnf"
  col_append_str(actx->pinfo->cinfo, COL_INFO, "RRCConnectionRelease");
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_RRCConnectionRelease, RRCConnectionRelease_choice,
                                 NULL);




  return offset;
}



static int
dissect_rrc_T_cipheringAlgorithmCap(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_bit_string(tvb, offset, actx, tree, hf_index,
                                     16, 16, FALSE, NULL);

  return offset;
}



static int
dissect_rrc_T_integrityProtectionAlgorithmCap(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_bit_string(tvb, offset, actx, tree, hf_index,
                                     16, 16, FALSE, NULL);

  return offset;
}


static const per_sequence_t SecurityCapability_sequence[] = {
  { &hf_rrc_cipheringAlgorithmCap, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_cipheringAlgorithmCap },
  { &hf_rrc_integrityProtectionAlgorithmCap, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_integrityProtectionAlgorithmCap },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_SecurityCapability(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_SecurityCapability, SecurityCapability_sequence);

  return offset;
}



static int
dissect_rrc_GsmSecurityCapability(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_bit_string(tvb, offset, actx, tree, hf_index,
                                     7, 7, FALSE, NULL);

  return offset;
}


static const per_sequence_t T_gsm_16_sequence[] = {
  { &hf_rrc_gsmSecurityCapability, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_GsmSecurityCapability },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_gsm_16(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_gsm_16, T_gsm_16_sequence);

  return offset;
}


static const value_string rrc_InterRAT_UE_SecurityCapability_vals[] = {
  {   0, "gsm" },
  { 0, NULL }
};

static const per_choice_t InterRAT_UE_SecurityCapability_choice[] = {
  {   0, &hf_rrc_gsm_18          , ASN1_NO_EXTENSIONS     , dissect_rrc_T_gsm_16 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_InterRAT_UE_SecurityCapability(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_InterRAT_UE_SecurityCapability, InterRAT_UE_SecurityCapability_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t InterRAT_UE_SecurityCapList_sequence_of[1] = {
  { &hf_rrc_InterRAT_UE_SecurityCapList_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_InterRAT_UE_SecurityCapability },
};

static int
dissect_rrc_InterRAT_UE_SecurityCapList(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_InterRAT_UE_SecurityCapList, InterRAT_UE_SecurityCapList_sequence_of,
                                                  1, maxInterSysMessages, FALSE);

  return offset;
}


static const per_sequence_t SecurityModeCommand_r3_IEs_sequence[] = {
  { &hf_rrc_rrc_TransactionIdentifier, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RRC_TransactionIdentifier },
  { &hf_rrc_securityCapability, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SecurityCapability },
  { &hf_rrc_cipheringModeInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CipheringModeInfo },
  { &hf_rrc_integrityProtectionModeInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_IntegrityProtectionModeInfo },
  { &hf_rrc_cn_DomainIdentity, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CN_DomainIdentity },
  { &hf_rrc_ue_SystemSpecificSecurityCap, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_InterRAT_UE_SecurityCapList },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_SecurityModeCommand_r3_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_SecurityModeCommand_r3_IEs, SecurityModeCommand_r3_IEs_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions_139_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_nonCriticalExtensions_139(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_nonCriticalExtensions_139, T_nonCriticalExtensions_139_sequence);

  return offset;
}


static const per_sequence_t T_laterNonCriticalExtensions_45_sequence[] = {
  { &hf_rrc_securityModeCommand_r3_add_ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_BIT_STRING },
  { &hf_rrc_nonCriticalExtensions_139, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_nonCriticalExtensions_139 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_laterNonCriticalExtensions_45(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_laterNonCriticalExtensions_45, T_laterNonCriticalExtensions_45_sequence);

  return offset;
}


static const per_sequence_t T_r3_22_sequence[] = {
  { &hf_rrc_securityModeCommand_r3, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SecurityModeCommand_r3_IEs },
  { &hf_rrc_laterNonCriticalExtensions_45, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_laterNonCriticalExtensions_45 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_r3_22(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_r3_22, T_r3_22_sequence);

  return offset;
}


static const per_sequence_t SecurityModeCommand_r7_IEs_sequence[] = {
  { &hf_rrc_securityCapability, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SecurityCapability },
  { &hf_rrc_cipheringModeInfo_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CipheringModeInfo_r7 },
  { &hf_rrc_integrityProtectionModeInfo_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_IntegrityProtectionModeInfo_r7 },
  { &hf_rrc_cn_DomainIdentity, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CN_DomainIdentity },
  { &hf_rrc_ue_SystemSpecificSecurityCap, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_InterRAT_UE_SecurityCapList },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_SecurityModeCommand_r7_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_SecurityModeCommand_r7_IEs, SecurityModeCommand_r7_IEs_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions_140_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_nonCriticalExtensions_140(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_nonCriticalExtensions_140, T_nonCriticalExtensions_140_sequence);

  return offset;
}


static const per_sequence_t T_r7_10_sequence[] = {
  { &hf_rrc_securityModeCommand_r7, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SecurityModeCommand_r7_IEs },
  { &hf_rrc_securityModeCommand_r7_add_ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_BIT_STRING },
  { &hf_rrc_nonCriticalExtensions_140, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_nonCriticalExtensions_140 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_r7_10(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_r7_10, T_r7_10_sequence);

  return offset;
}


static const per_sequence_t T_criticalExtensions_109_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_criticalExtensions_109(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_criticalExtensions_109, T_criticalExtensions_109_sequence);

  return offset;
}


static const value_string rrc_T_criticalExtensions_108_vals[] = {
  {   0, "r7" },
  {   1, "criticalExtensions" },
  { 0, NULL }
};

static const per_choice_t T_criticalExtensions_108_choice[] = {
  {   0, &hf_rrc_r7_10           , ASN1_NO_EXTENSIONS     , dissect_rrc_T_r7_10 },
  {   1, &hf_rrc_criticalExtensions_110, ASN1_NO_EXTENSIONS     , dissect_rrc_T_criticalExtensions_109 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_criticalExtensions_108(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_criticalExtensions_108, T_criticalExtensions_108_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t T_later_than_r3_19_sequence[] = {
  { &hf_rrc_rrc_TransactionIdentifier, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RRC_TransactionIdentifier },
  { &hf_rrc_criticalExtensions_109, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_criticalExtensions_108 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_later_than_r3_19(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_later_than_r3_19, T_later_than_r3_19_sequence);

  return offset;
}


static const value_string rrc_SecurityModeCommand_vals[] = {
  {   0, "r3" },
  {   1, "later-than-r3" },
  { 0, NULL }
};

static const per_choice_t SecurityModeCommand_choice[] = {
  {   0, &hf_rrc_r3_22           , ASN1_NO_EXTENSIONS     , dissect_rrc_T_r3_22 },
  {   1, &hf_rrc_later_than_r3_19, ASN1_NO_EXTENSIONS     , dissect_rrc_T_later_than_r3_19 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_SecurityModeCommand(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
#line 393 "../../asn1/rrc/rrc.cnf"
  col_append_str(actx->pinfo->cinfo, COL_INFO, "SecurityModeCommand");
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_SecurityModeCommand, SecurityModeCommand_choice,
                                 NULL);




  return offset;
}


static const per_sequence_t SignallingConnectionRelease_r3_IEs_sequence[] = {
  { &hf_rrc_rrc_TransactionIdentifier, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RRC_TransactionIdentifier },
  { &hf_rrc_cn_DomainIdentity, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CN_DomainIdentity },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_SignallingConnectionRelease_r3_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_SignallingConnectionRelease_r3_IEs, SignallingConnectionRelease_r3_IEs_sequence);

  return offset;
}


static const per_sequence_t SignallingConnectionRelease_va40ext_IEs_sequence[] = {
  { &hf_rrc_extendedWaitTime, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_ExtendedWaitTime },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_SignallingConnectionRelease_va40ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_SignallingConnectionRelease_va40ext_IEs, SignallingConnectionRelease_va40ext_IEs_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions_143_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_nonCriticalExtensions_143(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_nonCriticalExtensions_143, T_nonCriticalExtensions_143_sequence);

  return offset;
}


static const per_sequence_t T_va40NonCriticalExtensions_13_sequence[] = {
  { &hf_rrc_signallingConnectionRelease_va40ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SignallingConnectionRelease_va40ext_IEs },
  { &hf_rrc_nonCriticalExtensions_143, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_nonCriticalExtensions_143 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_va40NonCriticalExtensions_13(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_va40NonCriticalExtensions_13, T_va40NonCriticalExtensions_13_sequence);

  return offset;
}


static const per_sequence_t T_laterNonCriticalExtensions_48_sequence[] = {
  { &hf_rrc_signallingConnectionRelease_r3_add_ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_BIT_STRING },
  { &hf_rrc_va40NonCriticalExtensions_14, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_va40NonCriticalExtensions_13 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_laterNonCriticalExtensions_48(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_laterNonCriticalExtensions_48, T_laterNonCriticalExtensions_48_sequence);

  return offset;
}


static const per_sequence_t T_r3_23_sequence[] = {
  { &hf_rrc_signallingConnectionRelease_r3, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SignallingConnectionRelease_r3_IEs },
  { &hf_rrc_laterNonCriticalExtensions_48, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_laterNonCriticalExtensions_48 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_r3_23(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_r3_23, T_r3_23_sequence);

  return offset;
}


static const per_sequence_t T_criticalExtensions_110_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_criticalExtensions_110(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_criticalExtensions_110, T_criticalExtensions_110_sequence);

  return offset;
}


static const per_sequence_t T_later_than_r3_20_sequence[] = {
  { &hf_rrc_rrc_TransactionIdentifier, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RRC_TransactionIdentifier },
  { &hf_rrc_criticalExtensions_111, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_criticalExtensions_110 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_later_than_r3_20(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_later_than_r3_20, T_later_than_r3_20_sequence);

  return offset;
}


static const value_string rrc_SignallingConnectionRelease_vals[] = {
  {   0, "r3" },
  {   1, "later-than-r3" },
  { 0, NULL }
};

static const per_choice_t SignallingConnectionRelease_choice[] = {
  {   0, &hf_rrc_r3_23           , ASN1_NO_EXTENSIONS     , dissect_rrc_T_r3_23 },
  {   1, &hf_rrc_later_than_r3_20, ASN1_NO_EXTENSIONS     , dissect_rrc_T_later_than_r3_20 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_SignallingConnectionRelease(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
#line 415 "../../asn1/rrc/rrc.cnf"
  col_append_str(actx->pinfo->cinfo, COL_INFO, "SignallingConnectionRelease");
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_SignallingConnectionRelease, SignallingConnectionRelease_choice,
                                 NULL);




  return offset;
}


static const per_sequence_t T_fdd_39_sequence[] = {
  { &hf_rrc_dummy1          , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CPCH_SetID },
  { &hf_rrc_dummy2_02       , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DRAC_StaticInformationList },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_fdd_39(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_fdd_39, T_fdd_39_sequence);

  return offset;
}


static const value_string rrc_T_dummy_11_vals[] = {
  {   0, "fdd" },
  {   1, "tdd" },
  { 0, NULL }
};

static const per_choice_t T_dummy_11_choice[] = {
  {   0, &hf_rrc_fdd_39          , ASN1_NO_EXTENSIONS     , dissect_rrc_T_fdd_39 },
  {   1, &hf_rrc_tdd             , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_dummy_11(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_dummy_11, T_dummy_11_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t T_fdd_40_sequence[] = {
  { &hf_rrc_dummy_06        , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_PDSCH_Information },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_fdd_40(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_fdd_40, T_fdd_40_sequence);

  return offset;
}


static const value_string rrc_T_modeSpecificPhysChInfo_12_vals[] = {
  {   0, "fdd" },
  {   1, "tdd" },
  { 0, NULL }
};

static const per_choice_t T_modeSpecificPhysChInfo_12_choice[] = {
  {   0, &hf_rrc_fdd_40          , ASN1_NO_EXTENSIONS     , dissect_rrc_T_fdd_40 },
  {   1, &hf_rrc_tdd             , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_modeSpecificPhysChInfo_12(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_modeSpecificPhysChInfo_12, T_modeSpecificPhysChInfo_12_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t TransportChannelReconfiguration_r3_IEs_sequence[] = {
  { &hf_rrc_rrc_TransactionIdentifier, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RRC_TransactionIdentifier },
  { &hf_rrc_integrityProtectionModeInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_IntegrityProtectionModeInfo },
  { &hf_rrc_cipheringModeInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CipheringModeInfo },
  { &hf_rrc_activationTime  , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_ActivationTime },
  { &hf_rrc_new_U_RNTI      , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_U_RNTI },
  { &hf_rrc_new_C_RNTI      , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_C_RNTI },
  { &hf_rrc_rrc_StateIndicator, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RRC_StateIndicator },
  { &hf_rrc_utran_DRX_CycleLengthCoeff, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UTRAN_DRX_CycleLengthCoefficient },
  { &hf_rrc_cn_InformationInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CN_InformationInfo },
  { &hf_rrc_ura_Identity    , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_URA_Identity },
  { &hf_rrc_dl_CounterSynchronisationInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_CounterSynchronisationInfo },
  { &hf_rrc_ul_CommonTransChInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_CommonTransChInfo },
  { &hf_rrc_ul_AddReconfTransChInfoList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_AddReconfTransChInfoList },
  { &hf_rrc_dummy_25        , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_dummy_11 },
  { &hf_rrc_dl_CommonTransChInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_CommonTransChInfo },
  { &hf_rrc_dl_AddReconfTransChInfoList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_AddReconfTransChInfoList },
  { &hf_rrc_frequencyInfo   , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_FrequencyInfo },
  { &hf_rrc_maxAllowedUL_TX_Power, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MaxAllowedUL_TX_Power },
  { &hf_rrc_ul_ChannelRequirement, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_ChannelRequirement },
  { &hf_rrc_modeSpecificPhysChInfo_12, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_modeSpecificPhysChInfo_12 },
  { &hf_rrc_dl_CommonInformation, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_CommonInformation },
  { &hf_rrc_dl_InformationPerRL_List, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_InformationPerRL_List },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_TransportChannelReconfiguration_r3_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_TransportChannelReconfiguration_r3_IEs, TransportChannelReconfiguration_r3_IEs_sequence);

  return offset;
}


static const per_sequence_t TransportChannelReconfiguration_v3a0ext_sequence[] = {
  { &hf_rrc_new_DSCH_RNTI   , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DSCH_RNTI },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_TransportChannelReconfiguration_v3a0ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_TransportChannelReconfiguration_v3a0ext, TransportChannelReconfiguration_v3a0ext_sequence);

  return offset;
}


static const per_sequence_t TransportChannelReconfiguration_v4b0ext_IEs_sequence[] = {
  { &hf_rrc_dummy_02        , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_SSDT_UL },
  { &hf_rrc_cell_id_PerRL_List, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CellIdentity_PerRL_List },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_TransportChannelReconfiguration_v4b0ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_TransportChannelReconfiguration_v4b0ext_IEs, TransportChannelReconfiguration_v4b0ext_IEs_sequence);

  return offset;
}


static const per_sequence_t TransportChannelReconfiguration_v590ext_IEs_sequence[] = {
  { &hf_rrc_dl_TPC_PowerOffsetPerRL_List, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_TPC_PowerOffsetPerRL_List },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_TransportChannelReconfiguration_v590ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_TransportChannelReconfiguration_v590ext_IEs, TransportChannelReconfiguration_v590ext_IEs_sequence);

  return offset;
}


static const value_string rrc_T_postVerificationPeriod_06_vals[] = {
  {   0, "true" },
  { 0, NULL }
};


static int
dissect_rrc_T_postVerificationPeriod_06(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t TransportChannelReconfiguration_v690ext_IEs_sequence[] = {
  { &hf_rrc_delayRestrictionFlag, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DelayRestrictionFlag },
  { &hf_rrc_primary_plmn_Identity, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_PLMN_Identity },
  { &hf_rrc_harq_Preamble_Mode, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_HARQ_Preamble_Mode },
  { &hf_rrc_beaconPLEst     , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_BEACON_PL_Est },
  { &hf_rrc_postVerificationPeriod_06, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_postVerificationPeriod_06 },
  { &hf_rrc_dhs_sync        , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DHS_Sync },
  { &hf_rrc_timingMaintainedSynchInd, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_TimingMaintainedSynchInd },
  { &hf_rrc_mbms_PL_ServiceRestrictInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MBMS_PL_ServiceRestrictInfo_r6 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_TransportChannelReconfiguration_v690ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_TransportChannelReconfiguration_v690ext_IEs, TransportChannelReconfiguration_v690ext_IEs_sequence);

  return offset;
}


static const per_sequence_t TransportChannelReconfiguration_v770ext_IEs_sequence[] = {
  { &hf_rrc_ueMobilityStateIndicator, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_High_MobilityDetected },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_TransportChannelReconfiguration_v770ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_TransportChannelReconfiguration_v770ext_IEs, TransportChannelReconfiguration_v770ext_IEs_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions_146_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_nonCriticalExtensions_146(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_nonCriticalExtensions_146, T_nonCriticalExtensions_146_sequence);

  return offset;
}


static const per_sequence_t T_v770NonCriticalExtensions_25_sequence[] = {
  { &hf_rrc_transportChannelReconfiguration_v770ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TransportChannelReconfiguration_v770ext_IEs },
  { &hf_rrc_nonCriticalExtensions_146, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_nonCriticalExtensions_146 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v770NonCriticalExtensions_25(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v770NonCriticalExtensions_25, T_v770NonCriticalExtensions_25_sequence);

  return offset;
}


static const per_sequence_t T_v690NonCriticalExtensions_36_sequence[] = {
  { &hf_rrc_transportChannelReconfiguration_v690ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TransportChannelReconfiguration_v690ext_IEs },
  { &hf_rrc_v770NonCriticalExtensions_25, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v770NonCriticalExtensions_25 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v690NonCriticalExtensions_36(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v690NonCriticalExtensions_36, T_v690NonCriticalExtensions_36_sequence);

  return offset;
}


static const per_sequence_t T_v590NonCriticalExtensions_24_sequence[] = {
  { &hf_rrc_transportChannelReconfiguration_v590ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TransportChannelReconfiguration_v590ext_IEs },
  { &hf_rrc_v690NonCriticalExtensions_36, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v690NonCriticalExtensions_36 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v590NonCriticalExtensions_24(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v590NonCriticalExtensions_24, T_v590NonCriticalExtensions_24_sequence);

  return offset;
}


static const per_sequence_t T_v4b0NonCriticalExtensions_13_sequence[] = {
  { &hf_rrc_transportChannelReconfiguration_v4b0ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TransportChannelReconfiguration_v4b0ext_IEs },
  { &hf_rrc_v590NonCriticalExtensions_24, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v590NonCriticalExtensions_24 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v4b0NonCriticalExtensions_13(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v4b0NonCriticalExtensions_13, T_v4b0NonCriticalExtensions_13_sequence);

  return offset;
}


static const per_sequence_t T_laterNonCriticalExtensions_51_sequence[] = {
  { &hf_rrc_transportChannelReconfiguration_r3_add_ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_BIT_STRING },
  { &hf_rrc_v4b0NonCriticalExtensions_13, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v4b0NonCriticalExtensions_13 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_laterNonCriticalExtensions_51(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_laterNonCriticalExtensions_51, T_laterNonCriticalExtensions_51_sequence);

  return offset;
}


static const per_sequence_t T_v3a0NonCriticalExtensions_09_sequence[] = {
  { &hf_rrc_transportChannelReconfiguration_v3a0ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TransportChannelReconfiguration_v3a0ext },
  { &hf_rrc_laterNonCriticalExtensions_51, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_laterNonCriticalExtensions_51 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v3a0NonCriticalExtensions_09(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v3a0NonCriticalExtensions_09, T_v3a0NonCriticalExtensions_09_sequence);

  return offset;
}


static const per_sequence_t T_r3_24_sequence[] = {
  { &hf_rrc_transportChannelReconfiguration_r3, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TransportChannelReconfiguration_r3_IEs },
  { &hf_rrc_v3a0NonCriticalExtensions_09, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v3a0NonCriticalExtensions_09 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_r3_24(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_r3_24, T_r3_24_sequence);

  return offset;
}


static const per_sequence_t T_fdd_41_sequence[] = {
  { &hf_rrc_dummy1          , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CPCH_SetID },
  { &hf_rrc_dummy2_02       , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DRAC_StaticInformationList },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_fdd_41(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_fdd_41, T_fdd_41_sequence);

  return offset;
}


static const value_string rrc_T_dummy_12_vals[] = {
  {   0, "fdd" },
  {   1, "tdd" },
  { 0, NULL }
};

static const per_choice_t T_dummy_12_choice[] = {
  {   0, &hf_rrc_fdd_41          , ASN1_NO_EXTENSIONS     , dissect_rrc_T_fdd_41 },
  {   1, &hf_rrc_tdd             , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_dummy_12(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_dummy_12, T_dummy_12_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t T_fdd_42_sequence[] = {
  { &hf_rrc_dummy_06        , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_PDSCH_Information },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_fdd_42(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_fdd_42, T_fdd_42_sequence);

  return offset;
}


static const value_string rrc_T_modeSpecificPhysChInfo_13_vals[] = {
  {   0, "fdd" },
  {   1, "tdd" },
  { 0, NULL }
};

static const per_choice_t T_modeSpecificPhysChInfo_13_choice[] = {
  {   0, &hf_rrc_fdd_42          , ASN1_NO_EXTENSIONS     , dissect_rrc_T_fdd_42 },
  {   1, &hf_rrc_tdd             , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_modeSpecificPhysChInfo_13(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_modeSpecificPhysChInfo_13, T_modeSpecificPhysChInfo_13_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t TransportChannelReconfiguration_r4_IEs_sequence[] = {
  { &hf_rrc_integrityProtectionModeInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_IntegrityProtectionModeInfo },
  { &hf_rrc_cipheringModeInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CipheringModeInfo },
  { &hf_rrc_activationTime  , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_ActivationTime },
  { &hf_rrc_new_U_RNTI      , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_U_RNTI },
  { &hf_rrc_new_C_RNTI      , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_C_RNTI },
  { &hf_rrc_new_DSCH_RNTI   , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DSCH_RNTI },
  { &hf_rrc_rrc_StateIndicator, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RRC_StateIndicator },
  { &hf_rrc_utran_DRX_CycleLengthCoeff, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UTRAN_DRX_CycleLengthCoefficient },
  { &hf_rrc_cn_InformationInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CN_InformationInfo },
  { &hf_rrc_ura_Identity    , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_URA_Identity },
  { &hf_rrc_dl_CounterSynchronisationInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_CounterSynchronisationInfo },
  { &hf_rrc_ul_CommonTransChInfo_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_CommonTransChInfo_r4 },
  { &hf_rrc_ul_AddReconfTransChInfoList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_AddReconfTransChInfoList },
  { &hf_rrc_dummy_26        , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_dummy_12 },
  { &hf_rrc_dl_CommonTransChInfo_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_CommonTransChInfo_r4 },
  { &hf_rrc_dl_AddReconfTransChInfoList_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_AddReconfTransChInfoList_r4 },
  { &hf_rrc_frequencyInfo   , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_FrequencyInfo },
  { &hf_rrc_maxAllowedUL_TX_Power, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MaxAllowedUL_TX_Power },
  { &hf_rrc_ul_ChannelRequirement_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_ChannelRequirement_r4 },
  { &hf_rrc_modeSpecificPhysChInfo_13, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_modeSpecificPhysChInfo_13 },
  { &hf_rrc_dl_CommonInformation_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_CommonInformation_r4 },
  { &hf_rrc_dl_InformationPerRL_List_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_InformationPerRL_List_r4 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_TransportChannelReconfiguration_r4_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_TransportChannelReconfiguration_r4_IEs, TransportChannelReconfiguration_r4_IEs_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions_147_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_nonCriticalExtensions_147(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_nonCriticalExtensions_147, T_nonCriticalExtensions_147_sequence);

  return offset;
}


static const per_sequence_t T_v770NonCriticalExtensions_26_sequence[] = {
  { &hf_rrc_transportChannelReconfiguration_v770ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TransportChannelReconfiguration_v770ext_IEs },
  { &hf_rrc_nonCriticalExtensions_147, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_nonCriticalExtensions_147 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v770NonCriticalExtensions_26(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v770NonCriticalExtensions_26, T_v770NonCriticalExtensions_26_sequence);

  return offset;
}


static const per_sequence_t T_v690NonCriticalExtensions_37_sequence[] = {
  { &hf_rrc_transportChannelReconfiguration_v690ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TransportChannelReconfiguration_v690ext_IEs },
  { &hf_rrc_v770NonCriticalExtensions_26, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v770NonCriticalExtensions_26 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v690NonCriticalExtensions_37(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v690NonCriticalExtensions_37, T_v690NonCriticalExtensions_37_sequence);

  return offset;
}


static const per_sequence_t T_v590NonCriticalExtensions_25_sequence[] = {
  { &hf_rrc_transportChannelReconfiguration_v590ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TransportChannelReconfiguration_v590ext_IEs },
  { &hf_rrc_v690NonCriticalExtensions_37, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v690NonCriticalExtensions_37 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v590NonCriticalExtensions_25(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v590NonCriticalExtensions_25, T_v590NonCriticalExtensions_25_sequence);

  return offset;
}


static const per_sequence_t T_v4d0NonCriticalExtensions_12_sequence[] = {
  { &hf_rrc_transportChannelReconfiguration_r4_add_ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_BIT_STRING },
  { &hf_rrc_v590NonCriticalExtensions_25, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v590NonCriticalExtensions_25 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v4d0NonCriticalExtensions_12(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v4d0NonCriticalExtensions_12, T_v4d0NonCriticalExtensions_12_sequence);

  return offset;
}


static const per_sequence_t T_r4_12_sequence[] = {
  { &hf_rrc_transportChannelReconfiguration_r4, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TransportChannelReconfiguration_r4_IEs },
  { &hf_rrc_v4d0NonCriticalExtensions_12, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v4d0NonCriticalExtensions_12 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_r4_12(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_r4_12, T_r4_12_sequence);

  return offset;
}


static const per_sequence_t T_fdd_43_sequence[] = {
  { &hf_rrc_dummy1          , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CPCH_SetID },
  { &hf_rrc_dummy2_02       , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DRAC_StaticInformationList },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_fdd_43(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_fdd_43, T_fdd_43_sequence);

  return offset;
}


static const value_string rrc_T_dummy_13_vals[] = {
  {   0, "fdd" },
  {   1, "tdd" },
  { 0, NULL }
};

static const per_choice_t T_dummy_13_choice[] = {
  {   0, &hf_rrc_fdd_43          , ASN1_NO_EXTENSIONS     , dissect_rrc_T_fdd_43 },
  {   1, &hf_rrc_tdd             , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_dummy_13(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_dummy_13, T_dummy_13_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t T_fdd_44_sequence[] = {
  { &hf_rrc_dummy_06        , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_PDSCH_Information },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_fdd_44(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_fdd_44, T_fdd_44_sequence);

  return offset;
}


static const value_string rrc_T_modeSpecificPhysChInfo_14_vals[] = {
  {   0, "fdd" },
  {   1, "tdd" },
  { 0, NULL }
};

static const per_choice_t T_modeSpecificPhysChInfo_14_choice[] = {
  {   0, &hf_rrc_fdd_44          , ASN1_NO_EXTENSIONS     , dissect_rrc_T_fdd_44 },
  {   1, &hf_rrc_tdd             , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_modeSpecificPhysChInfo_14(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_modeSpecificPhysChInfo_14, T_modeSpecificPhysChInfo_14_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t TransportChannelReconfiguration_r5_IEs_sequence[] = {
  { &hf_rrc_integrityProtectionModeInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_IntegrityProtectionModeInfo },
  { &hf_rrc_cipheringModeInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CipheringModeInfo },
  { &hf_rrc_activationTime  , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_ActivationTime },
  { &hf_rrc_new_U_RNTI      , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_U_RNTI },
  { &hf_rrc_new_C_RNTI      , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_C_RNTI },
  { &hf_rrc_new_DSCH_RNTI   , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DSCH_RNTI },
  { &hf_rrc_new_H_RNTI      , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_H_RNTI },
  { &hf_rrc_rrc_StateIndicator, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RRC_StateIndicator },
  { &hf_rrc_utran_DRX_CycleLengthCoeff, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UTRAN_DRX_CycleLengthCoefficient },
  { &hf_rrc_cn_InformationInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CN_InformationInfo },
  { &hf_rrc_ura_Identity    , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_URA_Identity },
  { &hf_rrc_dl_CounterSynchronisationInfo_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_CounterSynchronisationInfo_r5 },
  { &hf_rrc_ul_CommonTransChInfo_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_CommonTransChInfo_r4 },
  { &hf_rrc_ul_AddReconfTransChInfoList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_AddReconfTransChInfoList },
  { &hf_rrc_dummy_27        , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_dummy_13 },
  { &hf_rrc_dl_CommonTransChInfo_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_CommonTransChInfo_r4 },
  { &hf_rrc_dl_AddReconfTransChInfoList_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_AddReconfTransChInfoList_r5 },
  { &hf_rrc_frequencyInfo   , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_FrequencyInfo },
  { &hf_rrc_maxAllowedUL_TX_Power, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MaxAllowedUL_TX_Power },
  { &hf_rrc_ul_ChannelRequirement_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_ChannelRequirement_r5 },
  { &hf_rrc_modeSpecificPhysChInfo_14, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_modeSpecificPhysChInfo_14 },
  { &hf_rrc_dl_HSPDSCH_Information, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_HSPDSCH_Information },
  { &hf_rrc_dl_CommonInformation_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_CommonInformation_r5 },
  { &hf_rrc_dl_InformationPerRL_List_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_InformationPerRL_List_r5 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_TransportChannelReconfiguration_r5_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_TransportChannelReconfiguration_r5_IEs, TransportChannelReconfiguration_r5_IEs_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions_148_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_nonCriticalExtensions_148(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_nonCriticalExtensions_148, T_nonCriticalExtensions_148_sequence);

  return offset;
}


static const per_sequence_t T_v770NonCriticalExtensions_27_sequence[] = {
  { &hf_rrc_transportChannelReconfiguration_v770ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TransportChannelReconfiguration_v770ext_IEs },
  { &hf_rrc_nonCriticalExtensions_148, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_nonCriticalExtensions_148 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v770NonCriticalExtensions_27(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v770NonCriticalExtensions_27, T_v770NonCriticalExtensions_27_sequence);

  return offset;
}


static const per_sequence_t T_v690NonCriticalExtensions_38_sequence[] = {
  { &hf_rrc_transportChannelReconfiguration_v690ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TransportChannelReconfiguration_v690ext_IEs },
  { &hf_rrc_v770NonCriticalExtensions_27, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v770NonCriticalExtensions_27 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v690NonCriticalExtensions_38(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v690NonCriticalExtensions_38, T_v690NonCriticalExtensions_38_sequence);

  return offset;
}


static const per_sequence_t T_r5_10_sequence[] = {
  { &hf_rrc_transportChannelReconfiguration_r5, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TransportChannelReconfiguration_r5_IEs },
  { &hf_rrc_transportChannelReconfiguration_r5_add_ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_BIT_STRING },
  { &hf_rrc_v690NonCriticalExtensions_38, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v690NonCriticalExtensions_38 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_r5_10(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_r5_10, T_r5_10_sequence);

  return offset;
}


static const per_sequence_t TransportChannelReconfiguration_r6_IEs_sequence[] = {
  { &hf_rrc_integrityProtectionModeInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_IntegrityProtectionModeInfo },
  { &hf_rrc_cipheringModeInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CipheringModeInfo },
  { &hf_rrc_activationTime  , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_ActivationTime },
  { &hf_rrc_delayRestrictionFlag, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DelayRestrictionFlag },
  { &hf_rrc_new_U_RNTI      , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_U_RNTI },
  { &hf_rrc_new_C_RNTI      , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_C_RNTI },
  { &hf_rrc_new_DSCH_RNTI   , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DSCH_RNTI },
  { &hf_rrc_new_H_RNTI      , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_H_RNTI },
  { &hf_rrc_newPrimary_E_RNTI, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_E_RNTI },
  { &hf_rrc_newSecondary_E_RNTI, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_E_RNTI },
  { &hf_rrc_rrc_StateIndicator, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RRC_StateIndicator },
  { &hf_rrc_utran_DRX_CycleLengthCoeff, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UTRAN_DRX_CycleLengthCoefficient },
  { &hf_rrc_cn_InformationInfo_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CN_InformationInfo_r6 },
  { &hf_rrc_ura_Identity    , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_URA_Identity },
  { &hf_rrc_dl_CounterSynchronisationInfo_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_CounterSynchronisationInfo_r5 },
  { &hf_rrc_ul_CommonTransChInfo_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_CommonTransChInfo_r4 },
  { &hf_rrc_ul_AddReconfTransChInfoList_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_AddReconfTransChInfoList_r6 },
  { &hf_rrc_dl_CommonTransChInfo_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_CommonTransChInfo_r4 },
  { &hf_rrc_dl_AddReconfTransChInfoList_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_AddReconfTransChInfoList_r5 },
  { &hf_rrc_frequencyInfo   , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_FrequencyInfo },
  { &hf_rrc_maxAllowedUL_TX_Power, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MaxAllowedUL_TX_Power },
  { &hf_rrc_ul_DPCH_Info    , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_DPCH_Info_r6 },
  { &hf_rrc_ul_EDCH_Information, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_EDCH_Information_r6 },
  { &hf_rrc_dl_HSPDSCH_Information_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_HSPDSCH_Information_r6 },
  { &hf_rrc_dl_CommonInformation_03, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_CommonInformation_r6 },
  { &hf_rrc_dl_InformationPerRL_List_03, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_InformationPerRL_List_r6 },
  { &hf_rrc_mbms_PL_ServiceRestrictInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MBMS_PL_ServiceRestrictInfo_r6 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_TransportChannelReconfiguration_r6_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_TransportChannelReconfiguration_r6_IEs, TransportChannelReconfiguration_r6_IEs_sequence);

  return offset;
}


static const per_sequence_t TransportChannelReconfiguration_v6b0ext_IEs_sequence[] = {
  { &hf_rrc_dl_InformationPerRL_List_v6b0ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_InformationPerRL_List_v6b0ext },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_TransportChannelReconfiguration_v6b0ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_TransportChannelReconfiguration_v6b0ext_IEs, TransportChannelReconfiguration_v6b0ext_IEs_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions_149_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_nonCriticalExtensions_149(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_nonCriticalExtensions_149, T_nonCriticalExtensions_149_sequence);

  return offset;
}


static const per_sequence_t T_v770NonCriticalExtensions_28_sequence[] = {
  { &hf_rrc_transportChannelReconfiguration_v770ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TransportChannelReconfiguration_v770ext_IEs },
  { &hf_rrc_nonCriticalExtensions_149, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_nonCriticalExtensions_149 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v770NonCriticalExtensions_28(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v770NonCriticalExtensions_28, T_v770NonCriticalExtensions_28_sequence);

  return offset;
}


static const per_sequence_t T_v6b0NonCriticalExtensions_11_sequence[] = {
  { &hf_rrc_transportChannelReconfiguration_v6b0ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TransportChannelReconfiguration_v6b0ext_IEs },
  { &hf_rrc_v770NonCriticalExtensions_28, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v770NonCriticalExtensions_28 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v6b0NonCriticalExtensions_11(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v6b0NonCriticalExtensions_11, T_v6b0NonCriticalExtensions_11_sequence);

  return offset;
}


static const per_sequence_t T_r6_11_sequence[] = {
  { &hf_rrc_transportChannelReconfiguration_r6, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TransportChannelReconfiguration_r6_IEs },
  { &hf_rrc_transportChannelReconfiguration_r6_add_ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_BIT_STRING },
  { &hf_rrc_v6b0NonCriticalExtensions_12, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v6b0NonCriticalExtensions_11 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_r6_11(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_r6_11, T_r6_11_sequence);

  return offset;
}


static const value_string rrc_T_responseToChangeOfUE_Capability_10_vals[] = {
  {   0, "true" },
  { 0, NULL }
};


static int
dissect_rrc_T_responseToChangeOfUE_Capability_10(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t TransportChannelReconfiguration_r7_IEs_sequence[] = {
  { &hf_rrc_integrityProtectionModeInfo_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_IntegrityProtectionModeInfo_r7 },
  { &hf_rrc_cipheringModeInfo_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CipheringModeInfo_r7 },
  { &hf_rrc_activationTime  , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_ActivationTime },
  { &hf_rrc_delayRestrictionFlag, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DelayRestrictionFlag },
  { &hf_rrc_new_U_RNTI      , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_U_RNTI },
  { &hf_rrc_new_C_RNTI      , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_C_RNTI },
  { &hf_rrc_new_DSCH_RNTI   , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DSCH_RNTI },
  { &hf_rrc_new_H_RNTI      , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_H_RNTI },
  { &hf_rrc_newPrimary_E_RNTI, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_E_RNTI },
  { &hf_rrc_newSecondary_E_RNTI, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_E_RNTI },
  { &hf_rrc_rrc_StateIndicator, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RRC_StateIndicator },
  { &hf_rrc_ueMobilityStateIndicator, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_High_MobilityDetected },
  { &hf_rrc_utran_DRX_CycleLengthCoeff_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UTRAN_DRX_CycleLengthCoefficient_r7 },
  { &hf_rrc_cn_InformationInfo_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CN_InformationInfo_r6 },
  { &hf_rrc_ura_Identity    , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_URA_Identity },
  { &hf_rrc_supportForChangeOfUE_Capability, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_BOOLEAN },
  { &hf_rrc_responseToChangeOfUE_Capability_10, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_responseToChangeOfUE_Capability_10 },
  { &hf_rrc_dl_CounterSynchronisationInfo_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_CounterSynchronisationInfo_r5 },
  { &hf_rrc_ul_CommonTransChInfo_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_CommonTransChInfo_r4 },
  { &hf_rrc_ul_AddReconfTransChInfoList_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_AddReconfTransChInfoList_r7 },
  { &hf_rrc_dl_CommonTransChInfo_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_CommonTransChInfo_r4 },
  { &hf_rrc_dl_AddReconfTransChInfoList_03, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_AddReconfTransChInfoList_r7 },
  { &hf_rrc_frequencyInfo   , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_FrequencyInfo },
  { &hf_rrc_multi_frequencyInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_Multi_frequencyInfo_LCR_r7 },
  { &hf_rrc_dtx_drx_TimingInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DTX_DRX_TimingInfo_r7 },
  { &hf_rrc_dtx_drx_Info    , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DTX_DRX_Info_r7 },
  { &hf_rrc_hs_scch_LessInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_HS_SCCH_LessInfo_r7 },
  { &hf_rrc_mimoParameters  , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MIMO_Parameters_r7 },
  { &hf_rrc_maxAllowedUL_TX_Power, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MaxAllowedUL_TX_Power },
  { &hf_rrc_ul_DPCH_Info_01 , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_DPCH_Info_r7 },
  { &hf_rrc_ul_EDCH_Information_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_EDCH_Information_r7 },
  { &hf_rrc_dl_HSPDSCH_Information_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_HSPDSCH_Information_r7 },
  { &hf_rrc_dl_CommonInformation_04, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_CommonInformation_r7 },
  { &hf_rrc_dl_InformationPerRL_List_04, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_InformationPerRL_List_r7 },
  { &hf_rrc_mbms_PL_ServiceRestrictInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MBMS_PL_ServiceRestrictInfo_r6 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_TransportChannelReconfiguration_r7_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_TransportChannelReconfiguration_r7_IEs, TransportChannelReconfiguration_r7_IEs_sequence);

  return offset;
}


static const per_sequence_t TransportChannelReconfiguration_v780ext_IEs_sequence[] = {
  { &hf_rrc_hs_DSCH_TBSizeTable, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_HS_DSCH_TBSizeTable },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_TransportChannelReconfiguration_v780ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_TransportChannelReconfiguration_v780ext_IEs, TransportChannelReconfiguration_v780ext_IEs_sequence);

  return offset;
}


static const per_sequence_t TransportChannelReconfiguration_v7d0ext_IEs_sequence[] = {
  { &hf_rrc_ul_EDCH_Information_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_EDCH_Information_ext },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_TransportChannelReconfiguration_v7d0ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_TransportChannelReconfiguration_v7d0ext_IEs, TransportChannelReconfiguration_v7d0ext_IEs_sequence);

  return offset;
}


static const per_sequence_t TransportChannelReconfiguration_v7f0ext_IEs_sequence[] = {
  { &hf_rrc_mimoParameters_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MIMO_Parameters_v7f0ext },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_TransportChannelReconfiguration_v7f0ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_TransportChannelReconfiguration_v7f0ext_IEs, TransportChannelReconfiguration_v7f0ext_IEs_sequence);

  return offset;
}


static const per_sequence_t TransportChannelReconfiguration_v7g0ext_IEs_sequence[] = {
  { &hf_rrc_mimoParameters_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MIMO_Parameters_v7g0ext },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_TransportChannelReconfiguration_v7g0ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_TransportChannelReconfiguration_v7g0ext_IEs, TransportChannelReconfiguration_v7g0ext_IEs_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions_150_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_nonCriticalExtensions_150(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_nonCriticalExtensions_150, T_nonCriticalExtensions_150_sequence);

  return offset;
}


static const per_sequence_t T_v7g0NonCriticalExtensions_15_sequence[] = {
  { &hf_rrc_transportChannelReconfiguration_v7g0ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TransportChannelReconfiguration_v7g0ext_IEs },
  { &hf_rrc_nonCriticalExtensions_150, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_nonCriticalExtensions_150 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v7g0NonCriticalExtensions_15(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v7g0NonCriticalExtensions_15, T_v7g0NonCriticalExtensions_15_sequence);

  return offset;
}


static const per_sequence_t T_v7f0NonCriticalExtensions_14_sequence[] = {
  { &hf_rrc_transportChannelReconfiguration_v7f0ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TransportChannelReconfiguration_v7f0ext_IEs },
  { &hf_rrc_v7g0NonCriticalExtensions_15, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v7g0NonCriticalExtensions_15 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v7f0NonCriticalExtensions_14(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v7f0NonCriticalExtensions_14, T_v7f0NonCriticalExtensions_14_sequence);

  return offset;
}


static const per_sequence_t T_v7d0NonCriticalExtensions_13_sequence[] = {
  { &hf_rrc_transportChannelReconfiguration_v7d0ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TransportChannelReconfiguration_v7d0ext_IEs },
  { &hf_rrc_v7f0NonCriticalExtensions_14, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v7f0NonCriticalExtensions_14 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v7d0NonCriticalExtensions_13(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v7d0NonCriticalExtensions_13, T_v7d0NonCriticalExtensions_13_sequence);

  return offset;
}


static const per_sequence_t T_v780NonCriticalExtensions_09_sequence[] = {
  { &hf_rrc_transportChannelReconfiguration_v780ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TransportChannelReconfiguration_v780ext_IEs },
  { &hf_rrc_v7d0NonCriticalExtensions_13, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v7d0NonCriticalExtensions_13 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v780NonCriticalExtensions_09(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v780NonCriticalExtensions_09, T_v780NonCriticalExtensions_09_sequence);

  return offset;
}


static const per_sequence_t T_r7_11_sequence[] = {
  { &hf_rrc_transportChannelReconfiguration_r7, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TransportChannelReconfiguration_r7_IEs },
  { &hf_rrc_transportChannelReconfiguration_r7_add_ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_BIT_STRING },
  { &hf_rrc_v780NonCriticalExtensions_09, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v780NonCriticalExtensions_09 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_r7_11(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_r7_11, T_r7_11_sequence);

  return offset;
}


static const value_string rrc_T_responseToChangeOfUE_Capability_11_vals[] = {
  {   0, "true" },
  { 0, NULL }
};


static int
dissect_rrc_T_responseToChangeOfUE_Capability_11(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t TransportChannelReconfiguration_r8_IEs_sequence[] = {
  { &hf_rrc_integrityProtectionModeInfo_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_IntegrityProtectionModeInfo_r7 },
  { &hf_rrc_cipheringModeInfo_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CipheringModeInfo_r7 },
  { &hf_rrc_activationTime  , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_ActivationTime },
  { &hf_rrc_delayRestrictionFlag, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DelayRestrictionFlag },
  { &hf_rrc_new_U_RNTI      , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_U_RNTI },
  { &hf_rrc_new_C_RNTI      , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_C_RNTI },
  { &hf_rrc_new_DSCH_RNTI   , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DSCH_RNTI },
  { &hf_rrc_new_H_RNTI      , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_H_RNTI },
  { &hf_rrc_newPrimary_E_RNTI, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_E_RNTI },
  { &hf_rrc_newSecondary_E_RNTI, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_E_RNTI },
  { &hf_rrc_rrc_StateIndicator, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RRC_StateIndicator },
  { &hf_rrc_ueMobilityStateIndicator, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_High_MobilityDetected },
  { &hf_rrc_utran_DRX_CycleLengthCoeff_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UTRAN_DRX_CycleLengthCoefficient_r7 },
  { &hf_rrc_cn_InformationInfo_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CN_InformationInfo_r6 },
  { &hf_rrc_ura_Identity    , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_URA_Identity },
  { &hf_rrc_supportForChangeOfUE_Capability, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_BOOLEAN },
  { &hf_rrc_responseToChangeOfUE_Capability_11, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_responseToChangeOfUE_Capability_11 },
  { &hf_rrc_dl_CounterSynchronisationInfo_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_CounterSynchronisationInfo_r5 },
  { &hf_rrc_ul_CommonTransChInfo_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_CommonTransChInfo_r4 },
  { &hf_rrc_ul_AddReconfTransChInfoList_03, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_AddReconfTransChInfoList_r8 },
  { &hf_rrc_dl_CommonTransChInfo_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_CommonTransChInfo_r4 },
  { &hf_rrc_dl_AddReconfTransChInfoList_03, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_AddReconfTransChInfoList_r7 },
  { &hf_rrc_frequencyInfo   , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_FrequencyInfo },
  { &hf_rrc_multi_frequencyInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_Multi_frequencyInfo_LCR_r7 },
  { &hf_rrc_dtx_drx_TimingInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DTX_DRX_TimingInfo_r7 },
  { &hf_rrc_dtx_drx_Info    , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DTX_DRX_Info_r7 },
  { &hf_rrc_hs_scch_LessInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_HS_SCCH_LessInfo_r7 },
  { &hf_rrc_mimoParameters_03, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MIMO_Parameters_r8 },
  { &hf_rrc_maxAllowedUL_TX_Power, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MaxAllowedUL_TX_Power },
  { &hf_rrc_ul_DPCH_Info_01 , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_DPCH_Info_r7 },
  { &hf_rrc_ul_EDCH_Information_03, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_EDCH_Information_r8 },
  { &hf_rrc_dl_HSPDSCH_Information_03, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_HSPDSCH_Information_r8 },
  { &hf_rrc_dl_CommonInformation_05, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_CommonInformation_r8 },
  { &hf_rrc_dl_InformationPerRL_List_05, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_InformationPerRL_List_r8 },
  { &hf_rrc_dl_SecondaryCellInfoFDD, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_SecondaryCellInfoFDD },
  { &hf_rrc_controlChannelDRXInfo_TDD128, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_ControlChannelDRXInfo_TDD128_r8 },
  { &hf_rrc_sps_Information_TDD128, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_SPS_Information_TDD128_r8 },
  { &hf_rrc_mbms_PL_ServiceRestrictInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MBMS_PL_ServiceRestrictInfo_r6 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_TransportChannelReconfiguration_r8_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_TransportChannelReconfiguration_r8_IEs, TransportChannelReconfiguration_r8_IEs_sequence);

  return offset;
}


static const per_sequence_t TransportChannelReconfiguration_v890ext_IEs_sequence[] = {
  { &hf_rrc_dl_SecondaryCellInfoFDD_v890ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_SecondaryCellInfoFDD_v890ext },
  { &hf_rrc_dl_HSPDSCH_Information_04, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_HSPDSCH_Information_r8_ext },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_TransportChannelReconfiguration_v890ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_TransportChannelReconfiguration_v890ext_IEs, TransportChannelReconfiguration_v890ext_IEs_sequence);

  return offset;
}


static const per_sequence_t TransportChannelReconfiguration_v8a0ext_IEs_sequence[] = {
  { &hf_rrc_dl_HSPDSCH_Information_05, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_HSPDSCH_Information_r8_ext2 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_TransportChannelReconfiguration_v8a0ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_TransportChannelReconfiguration_v8a0ext_IEs, TransportChannelReconfiguration_v8a0ext_IEs_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions_151_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_nonCriticalExtensions_151(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_nonCriticalExtensions_151, T_nonCriticalExtensions_151_sequence);

  return offset;
}


static const per_sequence_t T_v8a0NonCriticalExtensions_09_sequence[] = {
  { &hf_rrc_transportChannelReconfiguration_v8a0ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TransportChannelReconfiguration_v8a0ext_IEs },
  { &hf_rrc_nonCriticalExtensions_151, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_nonCriticalExtensions_151 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v8a0NonCriticalExtensions_09(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v8a0NonCriticalExtensions_09, T_v8a0NonCriticalExtensions_09_sequence);

  return offset;
}


static const per_sequence_t T_v7g0NonCriticalExtensions_16_sequence[] = {
  { &hf_rrc_transportChannelReconfiguration_v7g0ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TransportChannelReconfiguration_v7g0ext_IEs },
  { &hf_rrc_v8a0NonCriticalExtensions_09, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v8a0NonCriticalExtensions_09 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v7g0NonCriticalExtensions_16(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v7g0NonCriticalExtensions_16, T_v7g0NonCriticalExtensions_16_sequence);

  return offset;
}


static const per_sequence_t T_v890NonCriticalExtensions_09_sequence[] = {
  { &hf_rrc_transportChannelReconfiguration_v890ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TransportChannelReconfiguration_v890ext_IEs },
  { &hf_rrc_v7g0NonCriticalExtensions_16, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v7g0NonCriticalExtensions_16 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v890NonCriticalExtensions_09(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v890NonCriticalExtensions_09, T_v890NonCriticalExtensions_09_sequence);

  return offset;
}


static const per_sequence_t T_v7f0NonCriticalExtensions_15_sequence[] = {
  { &hf_rrc_transportChannelReconfiguration_v7f0ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TransportChannelReconfiguration_v7f0ext_IEs },
  { &hf_rrc_v890NonCriticalExtensions_09, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v890NonCriticalExtensions_09 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v7f0NonCriticalExtensions_15(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v7f0NonCriticalExtensions_15, T_v7f0NonCriticalExtensions_15_sequence);

  return offset;
}


static const per_sequence_t T_v7d0NonCriticalExtensions_14_sequence[] = {
  { &hf_rrc_transportChannelReconfiguration_v7d0ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TransportChannelReconfiguration_v7d0ext_IEs },
  { &hf_rrc_v7f0NonCriticalExtensions_15, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v7f0NonCriticalExtensions_15 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v7d0NonCriticalExtensions_14(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v7d0NonCriticalExtensions_14, T_v7d0NonCriticalExtensions_14_sequence);

  return offset;
}


static const per_sequence_t T_r8_11_sequence[] = {
  { &hf_rrc_transportChannelReconfiguration_r8, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TransportChannelReconfiguration_r8_IEs },
  { &hf_rrc_transportChannelReconfiguration_r8_add_ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_BIT_STRING },
  { &hf_rrc_v7d0NonCriticalExtensions_14, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v7d0NonCriticalExtensions_14 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_r8_11(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_r8_11, T_r8_11_sequence);

  return offset;
}


static const value_string rrc_T_responseToChangeOfUE_Capability_12_vals[] = {
  {   0, "true" },
  { 0, NULL }
};


static int
dissect_rrc_T_responseToChangeOfUE_Capability_12(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t TransportChannelReconfiguration_r9_IEs_sequence[] = {
  { &hf_rrc_integrityProtectionModeInfo_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_IntegrityProtectionModeInfo_r7 },
  { &hf_rrc_cipheringModeInfo_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CipheringModeInfo_r7 },
  { &hf_rrc_activationTime  , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_ActivationTime },
  { &hf_rrc_delayRestrictionFlag, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DelayRestrictionFlag },
  { &hf_rrc_new_U_RNTI      , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_U_RNTI },
  { &hf_rrc_new_C_RNTI      , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_C_RNTI },
  { &hf_rrc_new_DSCH_RNTI   , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DSCH_RNTI },
  { &hf_rrc_new_H_RNTI      , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_H_RNTI },
  { &hf_rrc_newPrimary_E_RNTI, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_E_RNTI },
  { &hf_rrc_newSecondary_E_RNTI, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_E_RNTI },
  { &hf_rrc_rrc_StateIndicator, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RRC_StateIndicator },
  { &hf_rrc_ueMobilityStateIndicator, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_High_MobilityDetected },
  { &hf_rrc_utran_DRX_CycleLengthCoeff_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UTRAN_DRX_CycleLengthCoefficient_r7 },
  { &hf_rrc_cn_InformationInfo_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CN_InformationInfo_r6 },
  { &hf_rrc_ura_Identity    , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_URA_Identity },
  { &hf_rrc_supportForChangeOfUE_Capability, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_BOOLEAN },
  { &hf_rrc_responseToChangeOfUE_Capability_12, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_responseToChangeOfUE_Capability_12 },
  { &hf_rrc_dl_CounterSynchronisationInfo_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_CounterSynchronisationInfo_r5 },
  { &hf_rrc_ul_CommonTransChInfo_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_CommonTransChInfo_r4 },
  { &hf_rrc_ul_AddReconfTransChInfoList_03, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_AddReconfTransChInfoList_r8 },
  { &hf_rrc_dl_CommonTransChInfo_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_CommonTransChInfo_r4 },
  { &hf_rrc_dl_AddReconfTransChInfoList_04, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_AddReconfTransChInfoList_r9 },
  { &hf_rrc_frequencyInfo   , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_FrequencyInfo },
  { &hf_rrc_multi_frequencyInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_Multi_frequencyInfo_LCR_r7 },
  { &hf_rrc_dtx_drx_TimingInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DTX_DRX_TimingInfo_r7 },
  { &hf_rrc_dtx_drx_Info    , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DTX_DRX_Info_r7 },
  { &hf_rrc_hs_scch_LessInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_HS_SCCH_LessInfo_r7 },
  { &hf_rrc_mimoParameters_04, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MIMO_Parameters_r9 },
  { &hf_rrc_maxAllowedUL_TX_Power, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MaxAllowedUL_TX_Power },
  { &hf_rrc_ul_DPCH_Info_01 , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_DPCH_Info_r7 },
  { &hf_rrc_ul_EDCH_Information_04, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_EDCH_Information_r9 },
  { &hf_rrc_ul_SecondaryCellInfoFDD, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_SecondaryCellInfoFDD },
  { &hf_rrc_dl_HSPDSCH_Information_06, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_HSPDSCH_Information_r9 },
  { &hf_rrc_dl_CommonInformation_05, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_CommonInformation_r8 },
  { &hf_rrc_dl_InformationPerRL_List_05, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_InformationPerRL_List_r8 },
  { &hf_rrc_dl_SecondaryCellInfoFDD_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_SecondaryCellInfoFDD_r9 },
  { &hf_rrc_controlChannelDRXInfo_TDD128, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_ControlChannelDRXInfo_TDD128_r8 },
  { &hf_rrc_sps_Information_TDD128, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_SPS_Information_TDD128_r8 },
  { &hf_rrc_mbms_PL_ServiceRestrictInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MBMS_PL_ServiceRestrictInfo_r6 },
  { &hf_rrc_cellDCHMeasOccasionInfo_TDD128, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CellDCHMeasOccasionInfo_TDD128_r9 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_TransportChannelReconfiguration_r9_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_TransportChannelReconfiguration_r9_IEs, TransportChannelReconfiguration_r9_IEs_sequence);

  return offset;
}


static const per_sequence_t TransportChannelReconfiguration_v950ext_IEs_sequence[] = {
  { &hf_rrc_secondaryCellMimoParameters, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_SecondaryCellMIMOparametersFDD_v950ext },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_TransportChannelReconfiguration_v950ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_TransportChannelReconfiguration_v950ext_IEs, TransportChannelReconfiguration_v950ext_IEs_sequence);

  return offset;
}


static const per_sequence_t TransportChannelReconfiguration_v9c0ext_IEs_sequence[] = {
  { &hf_rrc_dl_AddReconfTransChInfoList_05, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_AddReconfTransChInfoList_TDD128_v9c0ext },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_TransportChannelReconfiguration_v9c0ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_TransportChannelReconfiguration_v9c0ext_IEs, TransportChannelReconfiguration_v9c0ext_IEs_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions_152_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_nonCriticalExtensions_152(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_nonCriticalExtensions_152, T_nonCriticalExtensions_152_sequence);

  return offset;
}


static const per_sequence_t T_v9c0NonCriticalExtensions_10_sequence[] = {
  { &hf_rrc_transportChannelReconfiguration_v9c0ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TransportChannelReconfiguration_v9c0ext_IEs },
  { &hf_rrc_nonCriticalExtensions_152, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_nonCriticalExtensions_152 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v9c0NonCriticalExtensions_10(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v9c0NonCriticalExtensions_10, T_v9c0NonCriticalExtensions_10_sequence);

  return offset;
}


static const per_sequence_t T_v950NonCriticalExtensions_08_sequence[] = {
  { &hf_rrc_transportChannelReconfiguration_v950ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TransportChannelReconfiguration_v950ext_IEs },
  { &hf_rrc_v9c0NonCriticalExtensions_10, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v9c0NonCriticalExtensions_10 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v950NonCriticalExtensions_08(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v950NonCriticalExtensions_08, T_v950NonCriticalExtensions_08_sequence);

  return offset;
}


static const per_sequence_t T_r9_10_sequence[] = {
  { &hf_rrc_transportChannelReconfiguration_r9, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TransportChannelReconfiguration_r9_IEs },
  { &hf_rrc_transportChannelReconfiguration_r9_add_ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_BIT_STRING },
  { &hf_rrc_v950NonCriticalExtensions_08, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v950NonCriticalExtensions_08 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_r9_10(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_r9_10, T_r9_10_sequence);

  return offset;
}


static const value_string rrc_T_responseToChangeOfUE_Capability_13_vals[] = {
  {   0, "true" },
  { 0, NULL }
};


static int
dissect_rrc_T_responseToChangeOfUE_Capability_13(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t TransportChannelReconfiguration_r10_IEs_sequence[] = {
  { &hf_rrc_integrityProtectionModeInfo_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_IntegrityProtectionModeInfo_r7 },
  { &hf_rrc_cipheringModeInfo_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CipheringModeInfo_r7 },
  { &hf_rrc_activationTime  , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_ActivationTime },
  { &hf_rrc_delayRestrictionFlag, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DelayRestrictionFlag },
  { &hf_rrc_new_U_RNTI      , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_U_RNTI },
  { &hf_rrc_new_C_RNTI      , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_C_RNTI },
  { &hf_rrc_new_DSCH_RNTI   , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DSCH_RNTI },
  { &hf_rrc_new_H_RNTI      , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_H_RNTI },
  { &hf_rrc_newPrimary_E_RNTI, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_E_RNTI },
  { &hf_rrc_newSecondary_E_RNTI, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_E_RNTI },
  { &hf_rrc_rrc_StateIndicator, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RRC_StateIndicator },
  { &hf_rrc_ueMobilityStateIndicator, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_High_MobilityDetected },
  { &hf_rrc_utran_DRX_CycleLengthCoeff_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UTRAN_DRX_CycleLengthCoefficient_r7 },
  { &hf_rrc_cn_InformationInfo_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CN_InformationInfo_r6 },
  { &hf_rrc_ura_Identity    , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_URA_Identity },
  { &hf_rrc_supportForChangeOfUE_Capability, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_BOOLEAN },
  { &hf_rrc_responseToChangeOfUE_Capability_13, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_responseToChangeOfUE_Capability_13 },
  { &hf_rrc_dl_CounterSynchronisationInfo_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_CounterSynchronisationInfo_r5 },
  { &hf_rrc_ul_CommonTransChInfo_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_CommonTransChInfo_r4 },
  { &hf_rrc_ul_AddReconfTransChInfoList_03, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_AddReconfTransChInfoList_r8 },
  { &hf_rrc_dl_CommonTransChInfo_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_CommonTransChInfo_r4 },
  { &hf_rrc_dl_AddReconfTransChInfoList_04, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_AddReconfTransChInfoList_r9 },
  { &hf_rrc_frequencyInfo   , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_FrequencyInfo },
  { &hf_rrc_multi_frequencyInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_Multi_frequencyInfo_LCR_r7 },
  { &hf_rrc_dtx_drx_TimingInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DTX_DRX_TimingInfo_r7 },
  { &hf_rrc_dtx_drx_Info    , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DTX_DRX_Info_r7 },
  { &hf_rrc_hs_scch_LessInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_HS_SCCH_LessInfo_r7 },
  { &hf_rrc_mimoParameters_04, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MIMO_Parameters_r9 },
  { &hf_rrc_maxAllowedUL_TX_Power, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MaxAllowedUL_TX_Power },
  { &hf_rrc_ul_DPCH_Info_01 , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_DPCH_Info_r7 },
  { &hf_rrc_ul_EDCH_Information_04, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_EDCH_Information_r9 },
  { &hf_rrc_ul_SecondaryCellInfoFDD, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_SecondaryCellInfoFDD },
  { &hf_rrc_ul_MulticarrierEDCHInfo_TDD128, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_MulticarrierEDCHInfo_TDD128 },
  { &hf_rrc_dl_HSPDSCH_Information_06, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_HSPDSCH_Information_r9 },
  { &hf_rrc_dl_CommonInformation_06, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_CommonInformation_r10 },
  { &hf_rrc_dl_InformationPerRL_List_05, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_InformationPerRL_List_r8 },
  { &hf_rrc_dl_SecondaryCellInfoFDD_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_SecondaryCellInfoFDD_r10 },
  { &hf_rrc_additionalDLSecCellInfoListFDD, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_AdditionalDLSecCellInfoListFDD },
  { &hf_rrc_controlChannelDRXInfo_TDD128, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_ControlChannelDRXInfo_TDD128_r8 },
  { &hf_rrc_sps_Information_TDD128, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_SPS_Information_TDD128_r8 },
  { &hf_rrc_mu_MIMO_Info_TDD128, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MU_MIMO_Info_TDD128 },
  { &hf_rrc_mbms_PL_ServiceRestrictInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MBMS_PL_ServiceRestrictInfo_r6 },
  { &hf_rrc_cellDCHMeasOccasionInfo_TDD128, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CellDCHMeasOccasionInfo_TDD128_r9 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_TransportChannelReconfiguration_r10_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_TransportChannelReconfiguration_r10_IEs, TransportChannelReconfiguration_r10_IEs_sequence);

  return offset;
}


static const per_sequence_t TransportChannelReconfiguration_vbxyext_IEs_sequence[] = {
  { &hf_rrc_ulOLTDInfoFDD   , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_OLTD_InfoFDD },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_TransportChannelReconfiguration_vbxyext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_TransportChannelReconfiguration_vbxyext_IEs, TransportChannelReconfiguration_vbxyext_IEs_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions_153_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_nonCriticalExtensions_153(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_nonCriticalExtensions_153, T_nonCriticalExtensions_153_sequence);

  return offset;
}


static const per_sequence_t T_vbxyNonCriticalExtensions_14_sequence[] = {
  { &hf_rrc_transportChannelReconfiguration_vbxyext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TransportChannelReconfiguration_vbxyext_IEs },
  { &hf_rrc_nonCriticalExtensions_153, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_nonCriticalExtensions_153 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_vbxyNonCriticalExtensions_14(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_vbxyNonCriticalExtensions_14, T_vbxyNonCriticalExtensions_14_sequence);

  return offset;
}


static const per_sequence_t T_v9c0NonCriticalExtensions_11_sequence[] = {
  { &hf_rrc_transportChannelReconfiguration_v9c0ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TransportChannelReconfiguration_v9c0ext_IEs },
  { &hf_rrc_vbxyNonCriticalExtensions_14, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_vbxyNonCriticalExtensions_14 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v9c0NonCriticalExtensions_11(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v9c0NonCriticalExtensions_11, T_v9c0NonCriticalExtensions_11_sequence);

  return offset;
}


static const per_sequence_t T_r10_10_sequence[] = {
  { &hf_rrc_transportChannelReconfiguration_r10, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TransportChannelReconfiguration_r10_IEs },
  { &hf_rrc_transportChannelReconfiguration_r10_add_ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_BIT_STRING },
  { &hf_rrc_v9c0NonCriticalExtensions_11, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v9c0NonCriticalExtensions_11 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_r10_10(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_r10_10, T_r10_10_sequence);

  return offset;
}


static const value_string rrc_T_responseToChangeOfUE_Capability_14_vals[] = {
  {   0, "true" },
  { 0, NULL }
};


static int
dissect_rrc_T_responseToChangeOfUE_Capability_14(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t TransportChannelReconfiguration_r11_IEs_sequence[] = {
  { &hf_rrc_integrityProtectionModeInfo_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_IntegrityProtectionModeInfo_r7 },
  { &hf_rrc_cipheringModeInfo_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CipheringModeInfo_r7 },
  { &hf_rrc_activationTime  , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_ActivationTime },
  { &hf_rrc_delayRestrictionFlag, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DelayRestrictionFlag },
  { &hf_rrc_new_U_RNTI      , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_U_RNTI },
  { &hf_rrc_new_C_RNTI      , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_C_RNTI },
  { &hf_rrc_new_DSCH_RNTI   , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DSCH_RNTI },
  { &hf_rrc_new_H_RNTI      , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_H_RNTI },
  { &hf_rrc_newPrimary_E_RNTI, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_E_RNTI },
  { &hf_rrc_newSecondary_E_RNTI, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_E_RNTI },
  { &hf_rrc_rrc_StateIndicator, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RRC_StateIndicator },
  { &hf_rrc_ueMobilityStateIndicator, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_High_MobilityDetected },
  { &hf_rrc_utran_DRX_CycleLengthCoeff_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UTRAN_DRX_CycleLengthCoefficient_r7 },
  { &hf_rrc_cn_InformationInfo_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CN_InformationInfo_r6 },
  { &hf_rrc_ura_Identity    , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_URA_Identity },
  { &hf_rrc_supportForChangeOfUE_Capability, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_BOOLEAN },
  { &hf_rrc_responseToChangeOfUE_Capability_14, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_responseToChangeOfUE_Capability_14 },
  { &hf_rrc_dl_CounterSynchronisationInfo_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_CounterSynchronisationInfo_r5 },
  { &hf_rrc_ul_CommonTransChInfo_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_CommonTransChInfo_r4 },
  { &hf_rrc_ul_AddReconfTransChInfoList_03, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_AddReconfTransChInfoList_r8 },
  { &hf_rrc_dl_CommonTransChInfo_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_CommonTransChInfo_r4 },
  { &hf_rrc_dl_AddReconfTransChInfoList_06, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_AddReconfTransChInfoList_r11 },
  { &hf_rrc_frequencyInfo   , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_FrequencyInfo },
  { &hf_rrc_multi_frequencyInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_Multi_frequencyInfo_LCR_r7 },
  { &hf_rrc_dtx_drx_TimingInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DTX_DRX_TimingInfo_r7 },
  { &hf_rrc_dtx_drx_Info    , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DTX_DRX_Info_r7 },
  { &hf_rrc_hs_scch_LessInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_HS_SCCH_LessInfo_r7 },
  { &hf_rrc_mimoParameters_04, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MIMO_Parameters_r9 },
  { &hf_rrc_mimo4x4Parameters, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MIMO4x4_Parameters },
  { &hf_rrc_maxAllowedUL_TX_Power, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MaxAllowedUL_TX_Power },
  { &hf_rrc_ul_DPCH_Info_02 , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_DPCH_Info_r11 },
  { &hf_rrc_ul_EDCH_Information_05, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_EDCH_Information_r11 },
  { &hf_rrc_ul_SecondaryCellInfoFDD, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_SecondaryCellInfoFDD },
  { &hf_rrc_ul_MulticarrierEDCHInfo_TDD128, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_MulticarrierEDCHInfo_TDD128 },
  { &hf_rrc_ul_CLTD_InfoFDD , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_CLTD_InfoFDD },
  { &hf_rrc_ul_OLTD_InfoFDD , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_OLTD_InfoFDD },
  { &hf_rrc_dl_HSPDSCH_Information_07, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_HSPDSCH_Information_r11 },
  { &hf_rrc_dl_CommonInformation_07, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_CommonInformation_r11 },
  { &hf_rrc_dl_InformationPerRL_List_06, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_InformationPerRL_List_r11 },
  { &hf_rrc_dl_SecondaryCellInfoFDD_03, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_SecondaryCellInfoFDD_r11 },
  { &hf_rrc_additionalDLSecCellInfoListFDD_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_AdditionalDLSecCellInfoListFDD_r11 },
  { &hf_rrc_additionalDLSecCellInfoListFDD2, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_AdditionalDLSecCellInfoListFDD2_r11 },
  { &hf_rrc_commonERGCHInfoFDD, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CommonERGCHInfoFDD },
  { &hf_rrc_controlChannelDRXInfo_TDD128, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_ControlChannelDRXInfo_TDD128_r8 },
  { &hf_rrc_sps_Information_TDD128, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_SPS_Information_TDD128_r8 },
  { &hf_rrc_mu_MIMO_Info_TDD128, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MU_MIMO_Info_TDD128 },
  { &hf_rrc_mbms_PL_ServiceRestrictInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MBMS_PL_ServiceRestrictInfo_r6 },
  { &hf_rrc_cellDCHMeasOccasionInfo_TDD128, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CellDCHMeasOccasionInfo_TDD128_r9 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_TransportChannelReconfiguration_r11_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_TransportChannelReconfiguration_r11_IEs, TransportChannelReconfiguration_r11_IEs_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions_154_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_nonCriticalExtensions_154(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_nonCriticalExtensions_154, T_nonCriticalExtensions_154_sequence);

  return offset;
}


static const per_sequence_t T_r11_11_sequence[] = {
  { &hf_rrc_transportChannelReconfiguration_r11, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TransportChannelReconfiguration_r11_IEs },
  { &hf_rrc_transportChannelReconfiguration_r11_add_ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_BIT_STRING },
  { &hf_rrc_nonCriticalExtensions_154, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_nonCriticalExtensions_154 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_r11_11(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_r11_11, T_r11_11_sequence);

  return offset;
}


static const per_sequence_t T_criticalExtensions_119_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_criticalExtensions_119(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_criticalExtensions_119, T_criticalExtensions_119_sequence);

  return offset;
}


static const value_string rrc_T_criticalExtensions_118_vals[] = {
  {   0, "r11" },
  {   1, "criticalExtensions" },
  { 0, NULL }
};

static const per_choice_t T_criticalExtensions_118_choice[] = {
  {   0, &hf_rrc_r11_11          , ASN1_NO_EXTENSIONS     , dissect_rrc_T_r11_11 },
  {   1, &hf_rrc_criticalExtensions_120, ASN1_NO_EXTENSIONS     , dissect_rrc_T_criticalExtensions_119 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_criticalExtensions_118(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_criticalExtensions_118, T_criticalExtensions_118_choice,
                                 NULL);

  return offset;
}


static const value_string rrc_T_criticalExtensions_117_vals[] = {
  {   0, "r10" },
  {   1, "criticalExtensions" },
  { 0, NULL }
};

static const per_choice_t T_criticalExtensions_117_choice[] = {
  {   0, &hf_rrc_r10_10          , ASN1_NO_EXTENSIONS     , dissect_rrc_T_r10_10 },
  {   1, &hf_rrc_criticalExtensions_119, ASN1_NO_EXTENSIONS     , dissect_rrc_T_criticalExtensions_118 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_criticalExtensions_117(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_criticalExtensions_117, T_criticalExtensions_117_choice,
                                 NULL);

  return offset;
}


static const value_string rrc_T_criticalExtensions_116_vals[] = {
  {   0, "r9" },
  {   1, "criticalExtensions" },
  { 0, NULL }
};

static const per_choice_t T_criticalExtensions_116_choice[] = {
  {   0, &hf_rrc_r9_10           , ASN1_NO_EXTENSIONS     , dissect_rrc_T_r9_10 },
  {   1, &hf_rrc_criticalExtensions_118, ASN1_NO_EXTENSIONS     , dissect_rrc_T_criticalExtensions_117 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_criticalExtensions_116(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_criticalExtensions_116, T_criticalExtensions_116_choice,
                                 NULL);

  return offset;
}


static const value_string rrc_T_criticalExtensions_115_vals[] = {
  {   0, "r8" },
  {   1, "criticalExtensions" },
  { 0, NULL }
};

static const per_choice_t T_criticalExtensions_115_choice[] = {
  {   0, &hf_rrc_r8_11           , ASN1_NO_EXTENSIONS     , dissect_rrc_T_r8_11 },
  {   1, &hf_rrc_criticalExtensions_117, ASN1_NO_EXTENSIONS     , dissect_rrc_T_criticalExtensions_116 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_criticalExtensions_115(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_criticalExtensions_115, T_criticalExtensions_115_choice,
                                 NULL);

  return offset;
}


static const value_string rrc_T_criticalExtensions_114_vals[] = {
  {   0, "r7" },
  {   1, "criticalExtensions" },
  { 0, NULL }
};

static const per_choice_t T_criticalExtensions_114_choice[] = {
  {   0, &hf_rrc_r7_11           , ASN1_NO_EXTENSIONS     , dissect_rrc_T_r7_11 },
  {   1, &hf_rrc_criticalExtensions_116, ASN1_NO_EXTENSIONS     , dissect_rrc_T_criticalExtensions_115 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_criticalExtensions_114(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_criticalExtensions_114, T_criticalExtensions_114_choice,
                                 NULL);

  return offset;
}


static const value_string rrc_T_criticalExtensions_113_vals[] = {
  {   0, "r6" },
  {   1, "criticalExtensions" },
  { 0, NULL }
};

static const per_choice_t T_criticalExtensions_113_choice[] = {
  {   0, &hf_rrc_r6_11           , ASN1_NO_EXTENSIONS     , dissect_rrc_T_r6_11 },
  {   1, &hf_rrc_criticalExtensions_115, ASN1_NO_EXTENSIONS     , dissect_rrc_T_criticalExtensions_114 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_criticalExtensions_113(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_criticalExtensions_113, T_criticalExtensions_113_choice,
                                 NULL);

  return offset;
}


static const value_string rrc_T_criticalExtensions_112_vals[] = {
  {   0, "r5" },
  {   1, "criticalExtensions" },
  { 0, NULL }
};

static const per_choice_t T_criticalExtensions_112_choice[] = {
  {   0, &hf_rrc_r5_10           , ASN1_NO_EXTENSIONS     , dissect_rrc_T_r5_10 },
  {   1, &hf_rrc_criticalExtensions_114, ASN1_NO_EXTENSIONS     , dissect_rrc_T_criticalExtensions_113 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_criticalExtensions_112(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_criticalExtensions_112, T_criticalExtensions_112_choice,
                                 NULL);

  return offset;
}


static const value_string rrc_T_criticalExtensions_111_vals[] = {
  {   0, "r4" },
  {   1, "criticalExtensions" },
  { 0, NULL }
};

static const per_choice_t T_criticalExtensions_111_choice[] = {
  {   0, &hf_rrc_r4_12           , ASN1_NO_EXTENSIONS     , dissect_rrc_T_r4_12 },
  {   1, &hf_rrc_criticalExtensions_113, ASN1_NO_EXTENSIONS     , dissect_rrc_T_criticalExtensions_112 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_criticalExtensions_111(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_criticalExtensions_111, T_criticalExtensions_111_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t T_later_than_r3_21_sequence[] = {
  { &hf_rrc_rrc_TransactionIdentifier, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RRC_TransactionIdentifier },
  { &hf_rrc_criticalExtensions_112, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_criticalExtensions_111 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_later_than_r3_21(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_later_than_r3_21, T_later_than_r3_21_sequence);

  return offset;
}


static const value_string rrc_TransportChannelReconfiguration_vals[] = {
  {   0, "r3" },
  {   1, "later-than-r3" },
  { 0, NULL }
};

static const per_choice_t TransportChannelReconfiguration_choice[] = {
  {   0, &hf_rrc_r3_24           , ASN1_NO_EXTENSIONS     , dissect_rrc_T_r3_24 },
  {   1, &hf_rrc_later_than_r3_21, ASN1_NO_EXTENSIONS     , dissect_rrc_T_later_than_r3_21 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_TransportChannelReconfiguration(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
#line 435 "../../asn1/rrc/rrc.cnf"
  col_append_str(actx->pinfo->cinfo, COL_INFO, "TransportChannelReconfiguration");
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_TransportChannelReconfiguration, TransportChannelReconfiguration_choice,
                                 NULL);




  return offset;
}


static const per_sequence_t T_tdd_09_sequence[] = {
  { &hf_rrc_tfcs_ID         , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_TFCS_Identity },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_tdd_09(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_tdd_09, T_tdd_09_sequence);

  return offset;
}


static const value_string rrc_T_modeSpecificInfo_15_vals[] = {
  {   0, "fdd" },
  {   1, "tdd" },
  { 0, NULL }
};

static const per_choice_t T_modeSpecificInfo_15_choice[] = {
  {   0, &hf_rrc_fdd_45          , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   1, &hf_rrc_tdd_10          , ASN1_NO_EXTENSIONS     , dissect_rrc_T_tdd_09 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_modeSpecificInfo_15(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_modeSpecificInfo_15, T_modeSpecificInfo_15_choice,
                                 NULL);

  return offset;
}


static const value_string rrc_TFC_ControlDuration_vals[] = {
  {   0, "tfc-cd1" },
  {   1, "tfc-cd2" },
  {   2, "tfc-cd4" },
  {   3, "tfc-cd8" },
  {   4, "tfc-cd16" },
  {   5, "tfc-cd24" },
  {   6, "tfc-cd32" },
  {   7, "tfc-cd48" },
  {   8, "tfc-cd64" },
  {   9, "tfc-cd128" },
  {  10, "tfc-cd192" },
  {  11, "tfc-cd256" },
  {  12, "tfc-cd512" },
  { 0, NULL }
};


static int
dissect_rrc_TFC_ControlDuration(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     13, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t TransportFormatCombinationControl_v820ext_IEs_sequence[] = {
  { &hf_rrc_ul_AMR_Rate     , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_AMR_Rate },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_TransportFormatCombinationControl_v820ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_TransportFormatCombinationControl_v820ext_IEs, TransportFormatCombinationControl_v820ext_IEs_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions_157_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_nonCriticalExtensions_157(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_nonCriticalExtensions_157, T_nonCriticalExtensions_157_sequence);

  return offset;
}


static const per_sequence_t T_v820NonCriticalExtensions_02_sequence[] = {
  { &hf_rrc_transportformatcombinationcontrol_v820ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TransportFormatCombinationControl_v820ext_IEs },
  { &hf_rrc_nonCriticalExtensions_157, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_nonCriticalExtensions_157 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v820NonCriticalExtensions_02(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v820NonCriticalExtensions_02, T_v820NonCriticalExtensions_02_sequence);

  return offset;
}


static const per_sequence_t T_laterNonCriticalExtensions_54_sequence[] = {
  { &hf_rrc_transportFormatCombinationControl_r3_add_ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_BIT_STRING },
  { &hf_rrc_v820NonCriticalExtensions_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v820NonCriticalExtensions_02 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_laterNonCriticalExtensions_54(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_laterNonCriticalExtensions_54, T_laterNonCriticalExtensions_54_sequence);

  return offset;
}


static const per_sequence_t TransportFormatCombinationControl_sequence[] = {
  { &hf_rrc_rrc_TransactionIdentifier, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RRC_TransactionIdentifier },
  { &hf_rrc_modeSpecificInfo_15, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_modeSpecificInfo_15 },
  { &hf_rrc_dpch_TFCS_InUplink, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TFC_Subset },
  { &hf_rrc_activationTimeForTFCSubset, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_ActivationTime },
  { &hf_rrc_tfc_ControlDuration, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_TFC_ControlDuration },
  { &hf_rrc_laterNonCriticalExtensions_54, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_laterNonCriticalExtensions_54 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_TransportFormatCombinationControl(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
#line 447 "../../asn1/rrc/rrc.cnf"
  col_append_str(actx->pinfo->cinfo, COL_INFO, "TransportFormatCombinationControl");
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_TransportFormatCombinationControl, TransportFormatCombinationControl_sequence);




  return offset;
}


static const value_string rrc_SystemSpecificCapUpdateReq_vals[] = {
  {   0, "gsm" },
  { 0, NULL }
};


static int
dissect_rrc_SystemSpecificCapUpdateReq(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t SystemSpecificCapUpdateReqList_sequence_of[1] = {
  { &hf_rrc_SystemSpecificCapUpdateReqList_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SystemSpecificCapUpdateReq },
};

static int
dissect_rrc_SystemSpecificCapUpdateReqList(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_SystemSpecificCapUpdateReqList, SystemSpecificCapUpdateReqList_sequence_of,
                                                  1, maxSystemCapability, FALSE);

  return offset;
}


static const per_sequence_t CapabilityUpdateRequirement_sequence[] = {
  { &hf_rrc_ue_RadioCapabilityFDDUpdateRequirement, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { &hf_rrc_ue_RadioCapabilityTDDUpdateRequirement, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { &hf_rrc_systemSpecificCapUpdateReqList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_SystemSpecificCapUpdateReqList },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_CapabilityUpdateRequirement(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_CapabilityUpdateRequirement, CapabilityUpdateRequirement_sequence);

  return offset;
}


static const per_sequence_t UECapabilityEnquiry_r3_IEs_sequence[] = {
  { &hf_rrc_rrc_TransactionIdentifier, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RRC_TransactionIdentifier },
  { &hf_rrc_capabilityUpdateRequirement, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CapabilityUpdateRequirement },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UECapabilityEnquiry_r3_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UECapabilityEnquiry_r3_IEs, UECapabilityEnquiry_r3_IEs_sequence);

  return offset;
}


static const per_sequence_t CapabilityUpdateRequirement_r4_ext_sequence[] = {
  { &hf_rrc_ue_RadioCapabilityUpdateRequirement_TDD128, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_CapabilityUpdateRequirement_r4_ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_CapabilityUpdateRequirement_r4_ext, CapabilityUpdateRequirement_r4_ext_sequence);

  return offset;
}


static const per_sequence_t UECapabilityEnquiry_v4b0ext_IEs_sequence[] = {
  { &hf_rrc_capabilityUpdateRequirement_r4_ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CapabilityUpdateRequirement_r4_ext },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UECapabilityEnquiry_v4b0ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UECapabilityEnquiry_v4b0ext_IEs, UECapabilityEnquiry_v4b0ext_IEs_sequence);

  return offset;
}


static const value_string rrc_SystemSpecificCapUpdateReq_v590ext_vals[] = {
  {   0, "geranIu" },
  { 0, NULL }
};


static int
dissect_rrc_SystemSpecificCapUpdateReq_v590ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t UECapabilityEnquiry_v590ext_IEs_sequence[] = {
  { &hf_rrc_systemSpecificCapUpdateReq, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SystemSpecificCapUpdateReq_v590ext },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UECapabilityEnquiry_v590ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UECapabilityEnquiry_v590ext_IEs, UECapabilityEnquiry_v590ext_IEs_sequence);

  return offset;
}


static const per_sequence_t CapabilityUpdateRequirement_v770ext_sequence[] = {
  { &hf_rrc_ue_RadioCapabilityTDDUpdateRequirement_TDD768, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_CapabilityUpdateRequirement_v770ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_CapabilityUpdateRequirement_v770ext, CapabilityUpdateRequirement_v770ext_sequence);

  return offset;
}


static const per_sequence_t UECapabilityEnquiry_v770ext_IEs_sequence[] = {
  { &hf_rrc_capabilityUpdateRequirement_05, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CapabilityUpdateRequirement_v770ext },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UECapabilityEnquiry_v770ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UECapabilityEnquiry_v770ext_IEs, UECapabilityEnquiry_v770ext_IEs_sequence);

  return offset;
}


static const value_string rrc_SystemSpecificCapUpdateReq_r8_vals[] = {
  {   0, "gsm" },
  {   1, "geranIu" },
  {   2, "eutra" },
  {   3, "spare5" },
  {   4, "spare4" },
  {   5, "spare3" },
  {   6, "spare2" },
  {   7, "spare1" },
  { 0, NULL }
};


static int
dissect_rrc_SystemSpecificCapUpdateReq_r8(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     8, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t SystemSpecificCapUpdateReqList_r8_sequence_of[1] = {
  { &hf_rrc_SystemSpecificCapUpdateReqList_r8_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SystemSpecificCapUpdateReq_r8 },
};

static int
dissect_rrc_SystemSpecificCapUpdateReqList_r8(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_SystemSpecificCapUpdateReqList_r8, SystemSpecificCapUpdateReqList_r8_sequence_of,
                                                  1, maxSystemCapability, FALSE);

  return offset;
}


static const per_sequence_t CapabilityUpdateRequirement_v860ext_sequence[] = {
  { &hf_rrc_systemSpecificCapUpdateReqList_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_SystemSpecificCapUpdateReqList_r8 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_CapabilityUpdateRequirement_v860ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_CapabilityUpdateRequirement_v860ext, CapabilityUpdateRequirement_v860ext_sequence);

  return offset;
}


static const per_sequence_t UECapabilityEnquiry_v860ext_IEs_sequence[] = {
  { &hf_rrc_capabilityUpdateRequirement_06, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CapabilityUpdateRequirement_v860ext },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UECapabilityEnquiry_v860ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UECapabilityEnquiry_v860ext_IEs, UECapabilityEnquiry_v860ext_IEs_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions_159_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_nonCriticalExtensions_159(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_nonCriticalExtensions_159, T_nonCriticalExtensions_159_sequence);

  return offset;
}


static const per_sequence_t T_v860NonCriticalExtensions_18_sequence[] = {
  { &hf_rrc_ueCapabilityEnquiry_v860ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UECapabilityEnquiry_v860ext_IEs },
  { &hf_rrc_nonCriticalExtensions_159, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_nonCriticalExtensions_159 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v860NonCriticalExtensions_18(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v860NonCriticalExtensions_18, T_v860NonCriticalExtensions_18_sequence);

  return offset;
}


static const per_sequence_t T_v770NonCriticalExtensions_30_sequence[] = {
  { &hf_rrc_ueCapabilityEnquiry_v770ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UECapabilityEnquiry_v770ext_IEs },
  { &hf_rrc_v860NonCriticalExtensions_18, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v860NonCriticalExtensions_18 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v770NonCriticalExtensions_30(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v770NonCriticalExtensions_30, T_v770NonCriticalExtensions_30_sequence);

  return offset;
}


static const per_sequence_t T_v590NonCriticalExtensions_26_sequence[] = {
  { &hf_rrc_ueCapabilityEnquiry_v590ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UECapabilityEnquiry_v590ext_IEs },
  { &hf_rrc_v770NonCriticalExtensions_30, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v770NonCriticalExtensions_30 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v590NonCriticalExtensions_26(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v590NonCriticalExtensions_26, T_v590NonCriticalExtensions_26_sequence);

  return offset;
}


static const per_sequence_t T_v4b0NonCriticalExtensions_14_sequence[] = {
  { &hf_rrc_ueCapabilityEnquiry_v4b0ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UECapabilityEnquiry_v4b0ext_IEs },
  { &hf_rrc_v590NonCriticalExtensions_26, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v590NonCriticalExtensions_26 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v4b0NonCriticalExtensions_14(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v4b0NonCriticalExtensions_14, T_v4b0NonCriticalExtensions_14_sequence);

  return offset;
}


static const per_sequence_t T_laterNonCriticalExtensions_56_sequence[] = {
  { &hf_rrc_ueCapabilityEnquiry_r3_add_ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_BIT_STRING },
  { &hf_rrc_v4b0NonCriticalExtensions_14, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v4b0NonCriticalExtensions_14 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_laterNonCriticalExtensions_56(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_laterNonCriticalExtensions_56, T_laterNonCriticalExtensions_56_sequence);

  return offset;
}


static const per_sequence_t T_r3_25_sequence[] = {
  { &hf_rrc_ueCapabilityEnquiry_r3, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UECapabilityEnquiry_r3_IEs },
  { &hf_rrc_laterNonCriticalExtensions_56, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_laterNonCriticalExtensions_56 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_r3_25(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_r3_25, T_r3_25_sequence);

  return offset;
}


static const per_sequence_t T_criticalExtensions_120_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_criticalExtensions_120(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_criticalExtensions_120, T_criticalExtensions_120_sequence);

  return offset;
}


static const per_sequence_t T_later_than_r3_22_sequence[] = {
  { &hf_rrc_rrc_TransactionIdentifier, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RRC_TransactionIdentifier },
  { &hf_rrc_criticalExtensions_121, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_criticalExtensions_120 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_later_than_r3_22(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_later_than_r3_22, T_later_than_r3_22_sequence);

  return offset;
}


static const value_string rrc_UECapabilityEnquiry_vals[] = {
  {   0, "r3" },
  {   1, "later-than-r3" },
  { 0, NULL }
};

static const per_choice_t UECapabilityEnquiry_choice[] = {
  {   0, &hf_rrc_r3_25           , ASN1_NO_EXTENSIONS     , dissect_rrc_T_r3_25 },
  {   1, &hf_rrc_later_than_r3_22, ASN1_NO_EXTENSIONS     , dissect_rrc_T_later_than_r3_22 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_UECapabilityEnquiry(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
#line 455 "../../asn1/rrc/rrc.cnf"
  col_append_str(actx->pinfo->cinfo, COL_INFO, "UECapabilityEnquiry");
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_UECapabilityEnquiry, UECapabilityEnquiry_choice,
                                 NULL);




  return offset;
}


static const per_sequence_t UECapabilityInformationConfirm_r3_IEs_sequence[] = {
  { &hf_rrc_rrc_TransactionIdentifier, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RRC_TransactionIdentifier },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UECapabilityInformationConfirm_r3_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UECapabilityInformationConfirm_r3_IEs, UECapabilityInformationConfirm_r3_IEs_sequence);

  return offset;
}


static const value_string rrc_T_acceptanceOfChangeOfCapability_vals[] = {
  {   0, "refused" },
  {   1, "accepted" },
  {   2, "acceptedWithReconfigurationToFollow" },
  { 0, NULL }
};


static int
dissect_rrc_T_acceptanceOfChangeOfCapability(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     3, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t UECapabilityInformationConfirm_v770ext_IEs_sequence[] = {
  { &hf_rrc_acceptanceOfChangeOfCapability, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_acceptanceOfChangeOfCapability },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UECapabilityInformationConfirm_v770ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UECapabilityInformationConfirm_v770ext_IEs, UECapabilityInformationConfirm_v770ext_IEs_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions_162_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_nonCriticalExtensions_162(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_nonCriticalExtensions_162, T_nonCriticalExtensions_162_sequence);

  return offset;
}


static const per_sequence_t T_v770NonCriticalExtensions_31_sequence[] = {
  { &hf_rrc_ueCapabilityInformationConfirm_v770ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UECapabilityInformationConfirm_v770ext_IEs },
  { &hf_rrc_nonCriticalExtensions_162, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_nonCriticalExtensions_162 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v770NonCriticalExtensions_31(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v770NonCriticalExtensions_31, T_v770NonCriticalExtensions_31_sequence);

  return offset;
}


static const per_sequence_t T_laterNonCriticalExtensions_58_sequence[] = {
  { &hf_rrc_ueCapabilityInformationConfirm_r3_add_ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_BIT_STRING },
  { &hf_rrc_v770NonCriticalExtensions_31, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v770NonCriticalExtensions_31 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_laterNonCriticalExtensions_58(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_laterNonCriticalExtensions_58, T_laterNonCriticalExtensions_58_sequence);

  return offset;
}


static const per_sequence_t T_r3_26_sequence[] = {
  { &hf_rrc_ueCapabilityInformationConfirm_r3, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UECapabilityInformationConfirm_r3_IEs },
  { &hf_rrc_laterNonCriticalExtensions_58, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_laterNonCriticalExtensions_58 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_r3_26(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_r3_26, T_r3_26_sequence);

  return offset;
}


static const per_sequence_t T_criticalExtensions_121_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_criticalExtensions_121(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_criticalExtensions_121, T_criticalExtensions_121_sequence);

  return offset;
}


static const per_sequence_t T_later_than_r3_23_sequence[] = {
  { &hf_rrc_rrc_TransactionIdentifier, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RRC_TransactionIdentifier },
  { &hf_rrc_criticalExtensions_122, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_criticalExtensions_121 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_later_than_r3_23(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_later_than_r3_23, T_later_than_r3_23_sequence);

  return offset;
}


static const value_string rrc_UECapabilityInformationConfirm_vals[] = {
  {   0, "r3" },
  {   1, "later-than-r3" },
  { 0, NULL }
};

static const per_choice_t UECapabilityInformationConfirm_choice[] = {
  {   0, &hf_rrc_r3_26           , ASN1_NO_EXTENSIONS     , dissect_rrc_T_r3_26 },
  {   1, &hf_rrc_later_than_r3_23, ASN1_NO_EXTENSIONS     , dissect_rrc_T_later_than_r3_23 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_UECapabilityInformationConfirm(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
#line 463 "../../asn1/rrc/rrc.cnf"
  col_append_str(actx->pinfo->cinfo, COL_INFO, "UECapabilityInformationConfirm");
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_UECapabilityInformationConfirm, UECapabilityInformationConfirm_choice,
                                 NULL);




  return offset;
}


static const per_sequence_t CCTrCH_PowerControlInfo_sequence[] = {
  { &hf_rrc_tfcs_Identity   , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_TFCS_Identity },
  { &hf_rrc_ul_DPCH_PowerControlInfo, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UL_DPCH_PowerControlInfo },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_CCTrCH_PowerControlInfo(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_CCTrCH_PowerControlInfo, CCTrCH_PowerControlInfo_sequence);

  return offset;
}



static int
dissect_rrc_Alpha(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            0U, 8U, NULL, FALSE);

  return offset;
}



static int
dissect_rrc_SpecialBurstScheduling(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            0U, 7U, NULL, FALSE);

  return offset;
}


static const per_sequence_t UplinkPhysicalChannelControl_r3_IEs_sequence[] = {
  { &hf_rrc_rrc_TransactionIdentifier, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RRC_TransactionIdentifier },
  { &hf_rrc_ccTrCH_PowerControlInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CCTrCH_PowerControlInfo },
  { &hf_rrc_timingAdvance   , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_TimingAdvanceControl },
  { &hf_rrc_alpha           , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_Alpha },
  { &hf_rrc_specialBurstScheduling, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_SpecialBurstScheduling },
  { &hf_rrc_prach_ConstantValue, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_ConstantValueTdd },
  { &hf_rrc_pusch_ConstantValue, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_ConstantValueTdd },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UplinkPhysicalChannelControl_r3_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UplinkPhysicalChannelControl_r3_IEs, UplinkPhysicalChannelControl_r3_IEs_sequence);

  return offset;
}



static int
dissect_rrc_MaxPowerIncrease_r4(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            0U, 3U, NULL, FALSE);

  return offset;
}


static const per_sequence_t OpenLoopPowerControl_IPDL_TDD_r4_sequence[] = {
  { &hf_rrc_ipdl_alpha      , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_Alpha },
  { &hf_rrc_maxPowerIncrease, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MaxPowerIncrease_r4 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_OpenLoopPowerControl_IPDL_TDD_r4(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_OpenLoopPowerControl_IPDL_TDD_r4, OpenLoopPowerControl_IPDL_TDD_r4_sequence);

  return offset;
}


static const per_sequence_t UplinkPhysicalChannelControl_v4b0ext_IEs_sequence[] = {
  { &hf_rrc_openLoopPowerControl_IPDL_TDD, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_OpenLoopPowerControl_IPDL_TDD_r4 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UplinkPhysicalChannelControl_v4b0ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UplinkPhysicalChannelControl_v4b0ext_IEs, UplinkPhysicalChannelControl_v4b0ext_IEs_sequence);

  return offset;
}


static const per_sequence_t T_noncriticalExtensions_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_noncriticalExtensions(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_noncriticalExtensions, T_noncriticalExtensions_sequence);

  return offset;
}


static const per_sequence_t T_v4b0NonCriticalExtensions_16_sequence[] = {
  { &hf_rrc_uplinkPysicalChannelControl_v4b0ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UplinkPhysicalChannelControl_v4b0ext_IEs },
  { &hf_rrc_noncriticalExtensions, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_noncriticalExtensions },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v4b0NonCriticalExtensions_16(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v4b0NonCriticalExtensions_16, T_v4b0NonCriticalExtensions_16_sequence);

  return offset;
}


static const per_sequence_t T_laterNonCriticalExtensions_60_sequence[] = {
  { &hf_rrc_uplinkPhysicalChannelControl_r3_add_ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_BIT_STRING },
  { &hf_rrc_v4b0NonCriticalExtensions_16, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v4b0NonCriticalExtensions_16 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_laterNonCriticalExtensions_60(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_laterNonCriticalExtensions_60, T_laterNonCriticalExtensions_60_sequence);

  return offset;
}


static const per_sequence_t T_r3_27_sequence[] = {
  { &hf_rrc_uplinkPhysicalChannelControl_r3, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UplinkPhysicalChannelControl_r3_IEs },
  { &hf_rrc_laterNonCriticalExtensions_60, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_laterNonCriticalExtensions_60 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_r3_27(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_r3_27, T_r3_27_sequence);

  return offset;
}


static const per_sequence_t CCTrCH_PowerControlInfo_r4_sequence[] = {
  { &hf_rrc_tfcs_Identity   , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_TFCS_Identity },
  { &hf_rrc_ul_DPCH_PowerControlInfo_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UL_DPCH_PowerControlInfo_r4 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_CCTrCH_PowerControlInfo_r4(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_CCTrCH_PowerControlInfo_r4, CCTrCH_PowerControlInfo_r4_sequence);

  return offset;
}


static const per_sequence_t T_tdd384_08_sequence[] = {
  { &hf_rrc_timingAdvance_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_TimingAdvanceControl_r4 },
  { &hf_rrc_alpha           , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_Alpha },
  { &hf_rrc_prach_ConstantValue, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_ConstantValueTdd },
  { &hf_rrc_pusch_ConstantValue, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_ConstantValueTdd },
  { &hf_rrc_openLoopPowerControl_IPDL_TDD, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_OpenLoopPowerControl_IPDL_TDD_r4 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_tdd384_08(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_tdd384_08, T_tdd384_08_sequence);

  return offset;
}


static const per_sequence_t T_tdd128_08_sequence[] = {
  { &hf_rrc_ul_SynchronisationParameters, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_SynchronisationParameters_r4 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_tdd128_08(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_tdd128_08, T_tdd128_08_sequence);

  return offset;
}


static const value_string rrc_T_tddOption_vals[] = {
  {   0, "tdd384" },
  {   1, "tdd128" },
  { 0, NULL }
};

static const per_choice_t T_tddOption_choice[] = {
  {   0, &hf_rrc_tdd384_08       , ASN1_NO_EXTENSIONS     , dissect_rrc_T_tdd384_08 },
  {   1, &hf_rrc_tdd128_08       , ASN1_NO_EXTENSIONS     , dissect_rrc_T_tdd128_08 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_tddOption(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_tddOption, T_tddOption_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t UplinkPhysicalChannelControl_r4_IEs_sequence[] = {
  { &hf_rrc_ccTrCH_PowerControlInfo_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CCTrCH_PowerControlInfo_r4 },
  { &hf_rrc_specialBurstScheduling, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_SpecialBurstScheduling },
  { &hf_rrc_tddOption       , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_tddOption },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UplinkPhysicalChannelControl_r4_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UplinkPhysicalChannelControl_r4_IEs, UplinkPhysicalChannelControl_r4_IEs_sequence);

  return offset;
}


static const per_sequence_t UplinkPhysicalChannelControl_v690ext_IEs_sequence[] = {
  { &hf_rrc_beaconPLEst     , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_BEACON_PL_Est },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UplinkPhysicalChannelControl_v690ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UplinkPhysicalChannelControl_v690ext_IEs, UplinkPhysicalChannelControl_v690ext_IEs_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions_164_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_nonCriticalExtensions_164(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_nonCriticalExtensions_164, T_nonCriticalExtensions_164_sequence);

  return offset;
}


static const per_sequence_t T_v690NonCriticalExtensions_41_sequence[] = {
  { &hf_rrc_uplinkPhysicalChannelControl_v690ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UplinkPhysicalChannelControl_v690ext_IEs },
  { &hf_rrc_nonCriticalExtensions_164, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_nonCriticalExtensions_164 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v690NonCriticalExtensions_41(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v690NonCriticalExtensions_41, T_v690NonCriticalExtensions_41_sequence);

  return offset;
}


static const per_sequence_t T_v4d0NonCriticalExtensions_13_sequence[] = {
  { &hf_rrc_uplinkPhysicalChannelControl_r4_add_ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_BIT_STRING },
  { &hf_rrc_v690NonCriticalExtensions_41, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v690NonCriticalExtensions_41 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v4d0NonCriticalExtensions_13(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v4d0NonCriticalExtensions_13, T_v4d0NonCriticalExtensions_13_sequence);

  return offset;
}


static const per_sequence_t T_r4_13_sequence[] = {
  { &hf_rrc_uplinkPhysicalChannelControl_r4, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UplinkPhysicalChannelControl_r4_IEs },
  { &hf_rrc_v4d0NonCriticalExtensions_13, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v4d0NonCriticalExtensions_13 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_r4_13(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_r4_13, T_r4_13_sequence);

  return offset;
}


static const per_sequence_t CCTrCH_PowerControlInfo_r5_sequence[] = {
  { &hf_rrc_tfcs_Identity   , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_TFCS_Identity },
  { &hf_rrc_ul_DPCH_PowerControlInfo_02, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UL_DPCH_PowerControlInfo_r5 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_CCTrCH_PowerControlInfo_r5(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_CCTrCH_PowerControlInfo_r5, CCTrCH_PowerControlInfo_r5_sequence);

  return offset;
}


static const per_sequence_t T_tdd384_09_sequence[] = {
  { &hf_rrc_timingAdvance_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_TimingAdvanceControl_r4 },
  { &hf_rrc_alpha           , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_Alpha },
  { &hf_rrc_prach_ConstantValue, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_ConstantValueTdd },
  { &hf_rrc_pusch_ConstantValue, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_ConstantValueTdd },
  { &hf_rrc_openLoopPowerControl_IPDL_TDD, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_OpenLoopPowerControl_IPDL_TDD_r4 },
  { &hf_rrc_hs_SICH_PowerControl, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_HS_SICH_Power_Control_Info_TDD384 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_tdd384_09(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_tdd384_09, T_tdd384_09_sequence);

  return offset;
}


static const per_sequence_t T_tdd128_09_sequence[] = {
  { &hf_rrc_ul_SynchronisationParameters, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_SynchronisationParameters_r4 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_tdd128_09(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_tdd128_09, T_tdd128_09_sequence);

  return offset;
}


static const value_string rrc_T_tddOption_01_vals[] = {
  {   0, "tdd384" },
  {   1, "tdd128" },
  { 0, NULL }
};

static const per_choice_t T_tddOption_01_choice[] = {
  {   0, &hf_rrc_tdd384_09       , ASN1_NO_EXTENSIONS     , dissect_rrc_T_tdd384_09 },
  {   1, &hf_rrc_tdd128_09       , ASN1_NO_EXTENSIONS     , dissect_rrc_T_tdd128_09 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_tddOption_01(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_tddOption_01, T_tddOption_01_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t UplinkPhysicalChannelControl_r5_IEs_sequence[] = {
  { &hf_rrc_ccTrCH_PowerControlInfo_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CCTrCH_PowerControlInfo_r5 },
  { &hf_rrc_specialBurstScheduling, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_SpecialBurstScheduling },
  { &hf_rrc_tddOption_01    , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_tddOption_01 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UplinkPhysicalChannelControl_r5_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UplinkPhysicalChannelControl_r5_IEs, UplinkPhysicalChannelControl_r5_IEs_sequence);

  return offset;
}


static const value_string rrc_T_tpc_Step_Size_vals[] = {
  {   0, "s1" },
  {   1, "s2" },
  {   2, "s3" },
  {   3, "spare1" },
  { 0, NULL }
};


static int
dissect_rrc_T_tpc_Step_Size(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     4, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t UplinkPhysicalChannelControl_v6a0ext_IEs_sequence[] = {
  { &hf_rrc_desired_HS_SICH_PowerLevel, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_INTEGER_M120_M58 },
  { &hf_rrc_tpc_Step_Size   , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_tpc_Step_Size },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UplinkPhysicalChannelControl_v6a0ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UplinkPhysicalChannelControl_v6a0ext_IEs, UplinkPhysicalChannelControl_v6a0ext_IEs_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions_165_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_nonCriticalExtensions_165(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_nonCriticalExtensions_165, T_nonCriticalExtensions_165_sequence);

  return offset;
}


static const per_sequence_t T_v6a0NonCriticalExtensions_01_sequence[] = {
  { &hf_rrc_uplinkPhysicalChannelControl_v6a0ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UplinkPhysicalChannelControl_v6a0ext_IEs },
  { &hf_rrc_nonCriticalExtensions_165, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_nonCriticalExtensions_165 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v6a0NonCriticalExtensions_01(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v6a0NonCriticalExtensions_01, T_v6a0NonCriticalExtensions_01_sequence);

  return offset;
}


static const per_sequence_t T_v690NonCriticalExtensions_42_sequence[] = {
  { &hf_rrc_uplinkPhysicalChannelControl_v690ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UplinkPhysicalChannelControl_v690ext_IEs },
  { &hf_rrc_v6a0NonCriticalExtensions_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v6a0NonCriticalExtensions_01 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v690NonCriticalExtensions_42(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v690NonCriticalExtensions_42, T_v690NonCriticalExtensions_42_sequence);

  return offset;
}


static const per_sequence_t T_r5_11_sequence[] = {
  { &hf_rrc_uplinkPhysicalChannelControl_r5, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UplinkPhysicalChannelControl_r5_IEs },
  { &hf_rrc_uplinkPhysicalChannelControl_r5_add_ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_BIT_STRING },
  { &hf_rrc_v690NonCriticalExtensions_42, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v690NonCriticalExtensions_42 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_r5_11(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_r5_11, T_r5_11_sequence);

  return offset;
}


static const per_sequence_t CCTrCH_PowerControlInfo_r7_sequence[] = {
  { &hf_rrc_tfcs_Identity   , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_TFCS_Identity },
  { &hf_rrc_ul_DPCH_PowerControlInfo_03, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UL_DPCH_PowerControlInfo_r7 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_CCTrCH_PowerControlInfo_r7(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_CCTrCH_PowerControlInfo_r7, CCTrCH_PowerControlInfo_r7_sequence);

  return offset;
}


static const per_sequence_t T_tdd384_10_sequence[] = {
  { &hf_rrc_timingAdvance_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_TimingAdvanceControl_r4 },
  { &hf_rrc_alpha           , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_Alpha },
  { &hf_rrc_prach_ConstantValue, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_ConstantValueTdd },
  { &hf_rrc_pusch_ConstantValue, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_ConstantValueTdd },
  { &hf_rrc_openLoopPowerControl_IPDL_TDD, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_OpenLoopPowerControl_IPDL_TDD_r4 },
  { &hf_rrc_hs_SICH_PowerControl, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_HS_SICH_Power_Control_Info_TDD384 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_tdd384_10(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_tdd384_10, T_tdd384_10_sequence);

  return offset;
}


static const per_sequence_t T_tdd768_05_sequence[] = {
  { &hf_rrc_timingAdvance_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_TimingAdvanceControl_r7 },
  { &hf_rrc_alpha           , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_Alpha },
  { &hf_rrc_prach_ConstantValue, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_ConstantValueTdd },
  { &hf_rrc_pusch_ConstantValue, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_ConstantValueTdd },
  { &hf_rrc_openLoopPowerControl_IPDL_TDD, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_OpenLoopPowerControl_IPDL_TDD_r4 },
  { &hf_rrc_hs_SICH_PowerControl_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_HS_SICH_Power_Control_Info_TDD768 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_tdd768_05(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_tdd768_05, T_tdd768_05_sequence);

  return offset;
}


static const value_string rrc_T_tpc_Step_Size_01_vals[] = {
  {   0, "s1" },
  {   1, "s2" },
  {   2, "s3" },
  {   3, "spare1" },
  { 0, NULL }
};


static int
dissect_rrc_T_tpc_Step_Size_01(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     4, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t T_tdd128_10_sequence[] = {
  { &hf_rrc_ul_SynchronisationParameters, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_SynchronisationParameters_r4 },
  { &hf_rrc_desired_HS_SICH_PowerLevel, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_INTEGER_M120_M58 },
  { &hf_rrc_tpc_Step_Size_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_tpc_Step_Size_01 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_tdd128_10(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_tdd128_10, T_tdd128_10_sequence);

  return offset;
}


static const value_string rrc_T_tddOption_02_vals[] = {
  {   0, "tdd384" },
  {   1, "tdd768" },
  {   2, "tdd128" },
  { 0, NULL }
};

static const per_choice_t T_tddOption_02_choice[] = {
  {   0, &hf_rrc_tdd384_10       , ASN1_NO_EXTENSIONS     , dissect_rrc_T_tdd384_10 },
  {   1, &hf_rrc_tdd768_05       , ASN1_NO_EXTENSIONS     , dissect_rrc_T_tdd768_05 },
  {   2, &hf_rrc_tdd128_10       , ASN1_NO_EXTENSIONS     , dissect_rrc_T_tdd128_10 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_tddOption_02(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_tddOption_02, T_tddOption_02_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t UplinkPhysicalChannelControl_r7_IEs_sequence[] = {
  { &hf_rrc_ccTrCH_PowerControlInfo_03, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CCTrCH_PowerControlInfo_r7 },
  { &hf_rrc_specialBurstScheduling, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_SpecialBurstScheduling },
  { &hf_rrc_tddOption_02    , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_tddOption_02 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UplinkPhysicalChannelControl_r7_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UplinkPhysicalChannelControl_r7_IEs, UplinkPhysicalChannelControl_r7_IEs_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions_166_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_nonCriticalExtensions_166(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_nonCriticalExtensions_166, T_nonCriticalExtensions_166_sequence);

  return offset;
}


static const per_sequence_t T_r7_12_sequence[] = {
  { &hf_rrc_uplinkPhysicalChannelControl_r7, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UplinkPhysicalChannelControl_r7_IEs },
  { &hf_rrc_uplinkPhysicalChannelControl_r7_add_ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_BIT_STRING },
  { &hf_rrc_nonCriticalExtensions_166, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_nonCriticalExtensions_166 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_r7_12(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_r7_12, T_r7_12_sequence);

  return offset;
}


static const per_sequence_t T_criticalExtensions_125_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_criticalExtensions_125(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_criticalExtensions_125, T_criticalExtensions_125_sequence);

  return offset;
}


static const value_string rrc_T_criticalExtensions_124_vals[] = {
  {   0, "r7" },
  {   1, "criticalExtensions" },
  { 0, NULL }
};

static const per_choice_t T_criticalExtensions_124_choice[] = {
  {   0, &hf_rrc_r7_12           , ASN1_NO_EXTENSIONS     , dissect_rrc_T_r7_12 },
  {   1, &hf_rrc_criticalExtensions_126, ASN1_NO_EXTENSIONS     , dissect_rrc_T_criticalExtensions_125 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_criticalExtensions_124(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_criticalExtensions_124, T_criticalExtensions_124_choice,
                                 NULL);

  return offset;
}


static const value_string rrc_T_criticalExtensions_123_vals[] = {
  {   0, "r5" },
  {   1, "criticalExtensions" },
  { 0, NULL }
};

static const per_choice_t T_criticalExtensions_123_choice[] = {
  {   0, &hf_rrc_r5_11           , ASN1_NO_EXTENSIONS     , dissect_rrc_T_r5_11 },
  {   1, &hf_rrc_criticalExtensions_125, ASN1_NO_EXTENSIONS     , dissect_rrc_T_criticalExtensions_124 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_criticalExtensions_123(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_criticalExtensions_123, T_criticalExtensions_123_choice,
                                 NULL);

  return offset;
}


static const value_string rrc_T_criticalExtensions_122_vals[] = {
  {   0, "r4" },
  {   1, "criticalExtensions" },
  { 0, NULL }
};

static const per_choice_t T_criticalExtensions_122_choice[] = {
  {   0, &hf_rrc_r4_13           , ASN1_NO_EXTENSIONS     , dissect_rrc_T_r4_13 },
  {   1, &hf_rrc_criticalExtensions_124, ASN1_NO_EXTENSIONS     , dissect_rrc_T_criticalExtensions_123 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_criticalExtensions_122(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_criticalExtensions_122, T_criticalExtensions_122_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t T_later_than_r3_24_sequence[] = {
  { &hf_rrc_rrc_TransactionIdentifier, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RRC_TransactionIdentifier },
  { &hf_rrc_criticalExtensions_123, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_criticalExtensions_122 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_later_than_r3_24(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_later_than_r3_24, T_later_than_r3_24_sequence);

  return offset;
}


static const value_string rrc_UplinkPhysicalChannelControl_vals[] = {
  {   0, "r3" },
  {   1, "later-than-r3" },
  { 0, NULL }
};

static const per_choice_t UplinkPhysicalChannelControl_choice[] = {
  {   0, &hf_rrc_r3_27           , ASN1_NO_EXTENSIONS     , dissect_rrc_T_r3_27 },
  {   1, &hf_rrc_later_than_r3_24, ASN1_NO_EXTENSIONS     , dissect_rrc_T_later_than_r3_24 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_UplinkPhysicalChannelControl(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
#line 479 "../../asn1/rrc/rrc.cnf"
  col_append_str(actx->pinfo->cinfo, COL_INFO, "UplinkPhysicalChannelControl");
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_UplinkPhysicalChannelControl, UplinkPhysicalChannelControl_choice,
                                 NULL);




  return offset;
}


static const per_sequence_t URAUpdateConfirm_r3_IEs_sequence[] = {
  { &hf_rrc_rrc_TransactionIdentifier, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RRC_TransactionIdentifier },
  { &hf_rrc_integrityProtectionModeInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_IntegrityProtectionModeInfo },
  { &hf_rrc_cipheringModeInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CipheringModeInfo },
  { &hf_rrc_new_U_RNTI      , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_U_RNTI },
  { &hf_rrc_new_C_RNTI      , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_C_RNTI },
  { &hf_rrc_rrc_StateIndicator, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RRC_StateIndicator },
  { &hf_rrc_utran_DRX_CycleLengthCoeff, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UTRAN_DRX_CycleLengthCoefficient },
  { &hf_rrc_cn_InformationInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CN_InformationInfo },
  { &hf_rrc_ura_Identity    , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_URA_Identity },
  { &hf_rrc_dl_CounterSynchronisationInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_CounterSynchronisationInfo },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_URAUpdateConfirm_r3_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_URAUpdateConfirm_r3_IEs, URAUpdateConfirm_r3_IEs_sequence);

  return offset;
}


static const per_sequence_t URAUpdateConfirm_v690ext_IEs_sequence[] = {
  { &hf_rrc_primary_plmn_Identity, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_PLMN_Identity },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_URAUpdateConfirm_v690ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_URAUpdateConfirm_v690ext_IEs, URAUpdateConfirm_v690ext_IEs_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions_169_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_nonCriticalExtensions_169(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_nonCriticalExtensions_169, T_nonCriticalExtensions_169_sequence);

  return offset;
}


static const per_sequence_t T_v690NonCriticalExtensions_43_sequence[] = {
  { &hf_rrc_uraUpdateConfirm_v690ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_URAUpdateConfirm_v690ext_IEs },
  { &hf_rrc_nonCriticalExtensions_169, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_nonCriticalExtensions_169 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v690NonCriticalExtensions_43(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v690NonCriticalExtensions_43, T_v690NonCriticalExtensions_43_sequence);

  return offset;
}


static const per_sequence_t T_laterNonCriticalExtensions_62_sequence[] = {
  { &hf_rrc_uraUpdateConfirm_r3_add_ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_BIT_STRING },
  { &hf_rrc_v690NonCriticalExtensions_43, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v690NonCriticalExtensions_43 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_laterNonCriticalExtensions_62(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_laterNonCriticalExtensions_62, T_laterNonCriticalExtensions_62_sequence);

  return offset;
}


static const per_sequence_t T_r3_28_sequence[] = {
  { &hf_rrc_uraUpdateConfirm_r3, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_URAUpdateConfirm_r3_IEs },
  { &hf_rrc_laterNonCriticalExtensions_62, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_laterNonCriticalExtensions_62 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_r3_28(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_r3_28, T_r3_28_sequence);

  return offset;
}


static const per_sequence_t URAUpdateConfirm_r5_IEs_sequence[] = {
  { &hf_rrc_integrityProtectionModeInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_IntegrityProtectionModeInfo },
  { &hf_rrc_cipheringModeInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CipheringModeInfo },
  { &hf_rrc_new_U_RNTI      , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_U_RNTI },
  { &hf_rrc_new_C_RNTI      , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_C_RNTI },
  { &hf_rrc_rrc_StateIndicator, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RRC_StateIndicator },
  { &hf_rrc_utran_DRX_CycleLengthCoeff, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UTRAN_DRX_CycleLengthCoefficient },
  { &hf_rrc_cn_InformationInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CN_InformationInfo },
  { &hf_rrc_ura_Identity    , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_URA_Identity },
  { &hf_rrc_dl_CounterSynchronisationInfo_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_CounterSynchronisationInfo_r5 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_URAUpdateConfirm_r5_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_URAUpdateConfirm_r5_IEs, URAUpdateConfirm_r5_IEs_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions_170_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_nonCriticalExtensions_170(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_nonCriticalExtensions_170, T_nonCriticalExtensions_170_sequence);

  return offset;
}


static const per_sequence_t T_v690NonCriticalExtensions_44_sequence[] = {
  { &hf_rrc_uraUpdateConfirm_v690ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_URAUpdateConfirm_v690ext_IEs },
  { &hf_rrc_nonCriticalExtensions_170, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_nonCriticalExtensions_170 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v690NonCriticalExtensions_44(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v690NonCriticalExtensions_44, T_v690NonCriticalExtensions_44_sequence);

  return offset;
}


static const per_sequence_t T_r5_12_sequence[] = {
  { &hf_rrc_uraUpdateConfirm_r5, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_URAUpdateConfirm_r5_IEs },
  { &hf_rrc_v690NonCriticalExtensions_44, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v690NonCriticalExtensions_44 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_r5_12(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_r5_12, T_r5_12_sequence);

  return offset;
}


static const per_sequence_t URAUpdateConfirm_r7_IEs_sequence[] = {
  { &hf_rrc_integrityProtectionModeInfo_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_IntegrityProtectionModeInfo_r7 },
  { &hf_rrc_cipheringModeInfo_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CipheringModeInfo_r7 },
  { &hf_rrc_new_U_RNTI      , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_U_RNTI },
  { &hf_rrc_new_C_RNTI      , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_C_RNTI },
  { &hf_rrc_rrc_StateIndicator, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RRC_StateIndicator },
  { &hf_rrc_utran_DRX_CycleLengthCoeff_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UTRAN_DRX_CycleLengthCoefficient_r7 },
  { &hf_rrc_cn_InformationInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CN_InformationInfo },
  { &hf_rrc_primary_plmn_Identity, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_PLMN_Identity },
  { &hf_rrc_ura_Identity    , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_URA_Identity },
  { &hf_rrc_supportForChangeOfUE_Capability, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_BOOLEAN },
  { &hf_rrc_dl_CounterSynchronisationInfo_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_CounterSynchronisationInfo_r5 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_URAUpdateConfirm_r7_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_URAUpdateConfirm_r7_IEs, URAUpdateConfirm_r7_IEs_sequence);

  return offset;
}


static const per_sequence_t URAUpdateConfirm_v860ext_IEs_sequence[] = {
  { &hf_rrc_new_H_RNTI      , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_H_RNTI },
  { &hf_rrc_newPrimary_E_RNTI, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_E_RNTI },
  { &hf_rrc_dummy_07        , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DefaultConfigForCellFACH },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_URAUpdateConfirm_v860ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_URAUpdateConfirm_v860ext_IEs, URAUpdateConfirm_v860ext_IEs_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions_171_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_nonCriticalExtensions_171(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_nonCriticalExtensions_171, T_nonCriticalExtensions_171_sequence);

  return offset;
}


static const per_sequence_t T_v860NonCriticalExtensions_20_sequence[] = {
  { &hf_rrc_uraUpdateConfirm_v860ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_URAUpdateConfirm_v860ext_IEs },
  { &hf_rrc_nonCriticalExtensions_171, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_nonCriticalExtensions_171 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v860NonCriticalExtensions_20(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v860NonCriticalExtensions_20, T_v860NonCriticalExtensions_20_sequence);

  return offset;
}


static const per_sequence_t T_r7_13_sequence[] = {
  { &hf_rrc_uraUpdateConfirm_r7, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_URAUpdateConfirm_r7_IEs },
  { &hf_rrc_uraUpdateConfirm_r7_add_ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_BIT_STRING },
  { &hf_rrc_v860NonCriticalExtensions_20, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v860NonCriticalExtensions_20 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_r7_13(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_r7_13, T_r7_13_sequence);

  return offset;
}


static const per_sequence_t T_criticalExtensions_128_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_criticalExtensions_128(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_criticalExtensions_128, T_criticalExtensions_128_sequence);

  return offset;
}


static const value_string rrc_T_criticalExtensions_127_vals[] = {
  {   0, "r7" },
  {   1, "criticalExtensions" },
  { 0, NULL }
};

static const per_choice_t T_criticalExtensions_127_choice[] = {
  {   0, &hf_rrc_r7_13           , ASN1_NO_EXTENSIONS     , dissect_rrc_T_r7_13 },
  {   1, &hf_rrc_criticalExtensions_129, ASN1_NO_EXTENSIONS     , dissect_rrc_T_criticalExtensions_128 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_criticalExtensions_127(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_criticalExtensions_127, T_criticalExtensions_127_choice,
                                 NULL);

  return offset;
}


static const value_string rrc_T_criticalExtensions_126_vals[] = {
  {   0, "r5" },
  {   1, "criticalExtensions" },
  { 0, NULL }
};

static const per_choice_t T_criticalExtensions_126_choice[] = {
  {   0, &hf_rrc_r5_12           , ASN1_NO_EXTENSIONS     , dissect_rrc_T_r5_12 },
  {   1, &hf_rrc_criticalExtensions_128, ASN1_NO_EXTENSIONS     , dissect_rrc_T_criticalExtensions_127 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_criticalExtensions_126(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_criticalExtensions_126, T_criticalExtensions_126_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t T_later_than_r3_25_sequence[] = {
  { &hf_rrc_rrc_TransactionIdentifier, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RRC_TransactionIdentifier },
  { &hf_rrc_criticalExtensions_127, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_criticalExtensions_126 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_later_than_r3_25(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_later_than_r3_25, T_later_than_r3_25_sequence);

  return offset;
}


static const value_string rrc_URAUpdateConfirm_vals[] = {
  {   0, "r3" },
  {   1, "later-than-r3" },
  { 0, NULL }
};

static const per_choice_t URAUpdateConfirm_choice[] = {
  {   0, &hf_rrc_r3_28           , ASN1_NO_EXTENSIONS     , dissect_rrc_T_r3_28 },
  {   1, &hf_rrc_later_than_r3_25, ASN1_NO_EXTENSIONS     , dissect_rrc_T_later_than_r3_25 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_URAUpdateConfirm(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
#line 487 "../../asn1/rrc/rrc.cnf"
  col_append_str(actx->pinfo->cinfo, COL_INFO, "URAUpdateConfirm");
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_URAUpdateConfirm, URAUpdateConfirm_choice,
                                 NULL);




  return offset;
}


static const value_string rrc_T_301_vals[] = {
  {   0, "ms100" },
  {   1, "ms200" },
  {   2, "ms400" },
  {   3, "ms600" },
  {   4, "ms800" },
  {   5, "ms1000" },
  {   6, "ms1200" },
  {   7, "ms1400" },
  {   8, "ms1600" },
  {   9, "ms1800" },
  {  10, "ms2000" },
  {  11, "ms3000" },
  {  12, "ms4000" },
  {  13, "ms6000" },
  {  14, "ms8000" },
  {  15, "spare" },
  { 0, NULL }
};


static int
dissect_rrc_T_301(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     16, NULL, FALSE, 0, NULL);

  return offset;
}



static int
dissect_rrc_N_301(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            0U, 7U, NULL, FALSE);

  return offset;
}


static const value_string rrc_T_302_vals[] = {
  {   0, "ms100" },
  {   1, "ms200" },
  {   2, "ms400" },
  {   3, "ms600" },
  {   4, "ms800" },
  {   5, "ms1000" },
  {   6, "ms1200" },
  {   7, "ms1400" },
  {   8, "ms1600" },
  {   9, "ms1800" },
  {  10, "ms2000" },
  {  11, "ms3000" },
  {  12, "ms4000" },
  {  13, "ms6000" },
  {  14, "ms8000" },
  {  15, "spare" },
  { 0, NULL }
};


static int
dissect_rrc_T_302(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     16, NULL, FALSE, 0, NULL);

  return offset;
}



static int
dissect_rrc_N_302(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            0U, 7U, NULL, FALSE);

  return offset;
}


static const value_string rrc_T_304_vals[] = {
  {   0, "ms100" },
  {   1, "ms200" },
  {   2, "ms400" },
  {   3, "ms1000" },
  {   4, "ms2000" },
  {   5, "spare3" },
  {   6, "spare2" },
  {   7, "spare1" },
  { 0, NULL }
};


static int
dissect_rrc_T_304(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     8, NULL, FALSE, 0, NULL);

  return offset;
}



static int
dissect_rrc_N_304(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            0U, 7U, NULL, FALSE);

  return offset;
}


static const value_string rrc_T_305_vals[] = {
  {   0, "noUpdate" },
  {   1, "m5" },
  {   2, "m10" },
  {   3, "m30" },
  {   4, "m60" },
  {   5, "m120" },
  {   6, "m360" },
  {   7, "m720" },
  { 0, NULL }
};


static int
dissect_rrc_T_305(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     8, NULL, FALSE, 0, NULL);

  return offset;
}


static const value_string rrc_T_307_vals[] = {
  {   0, "s5" },
  {   1, "s10" },
  {   2, "s15" },
  {   3, "s20" },
  {   4, "s30" },
  {   5, "s40" },
  {   6, "s50" },
  {   7, "spare" },
  { 0, NULL }
};


static int
dissect_rrc_T_307(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     8, NULL, FALSE, 0, NULL);

  return offset;
}


static const value_string rrc_T_308_vals[] = {
  {   0, "ms40" },
  {   1, "ms80" },
  {   2, "ms160" },
  {   3, "ms320" },
  { 0, NULL }
};


static int
dissect_rrc_T_308(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     4, NULL, FALSE, 0, NULL);

  return offset;
}



static int
dissect_rrc_T_309(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            1U, 8U, NULL, FALSE);

  return offset;
}


static const value_string rrc_T_310_vals[] = {
  {   0, "ms40" },
  {   1, "ms80" },
  {   2, "ms120" },
  {   3, "ms160" },
  {   4, "ms200" },
  {   5, "ms240" },
  {   6, "ms280" },
  {   7, "ms320" },
  { 0, NULL }
};


static int
dissect_rrc_T_310(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     8, NULL, FALSE, 0, NULL);

  return offset;
}



static int
dissect_rrc_N_310(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            0U, 7U, NULL, FALSE);

  return offset;
}


static const value_string rrc_T_311_vals[] = {
  {   0, "ms250" },
  {   1, "ms500" },
  {   2, "ms750" },
  {   3, "ms1000" },
  {   4, "ms1250" },
  {   5, "ms1500" },
  {   6, "ms1750" },
  {   7, "ms2000" },
  { 0, NULL }
};


static int
dissect_rrc_T_311(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     8, NULL, FALSE, 0, NULL);

  return offset;
}



static int
dissect_rrc_T_312(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            0U, 15U, NULL, FALSE);

  return offset;
}


static const value_string rrc_N_312_vals[] = {
  {   0, "s1" },
  {   1, "s50" },
  {   2, "s100" },
  {   3, "s200" },
  {   4, "s400" },
  {   5, "s600" },
  {   6, "s800" },
  {   7, "s1000" },
  { 0, NULL }
};


static int
dissect_rrc_N_312(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     8, NULL, FALSE, 0, NULL);

  return offset;
}



static int
dissect_rrc_T_313(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            0U, 15U, NULL, FALSE);

  return offset;
}


static const value_string rrc_N_313_vals[] = {
  {   0, "s1" },
  {   1, "s2" },
  {   2, "s4" },
  {   3, "s10" },
  {   4, "s20" },
  {   5, "s50" },
  {   6, "s100" },
  {   7, "s200" },
  { 0, NULL }
};


static int
dissect_rrc_N_313(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     8, NULL, FALSE, 0, NULL);

  return offset;
}


static const value_string rrc_T_314_vals[] = {
  {   0, "s0" },
  {   1, "s2" },
  {   2, "s4" },
  {   3, "s6" },
  {   4, "s8" },
  {   5, "s12" },
  {   6, "s16" },
  {   7, "s20" },
  { 0, NULL }
};


static int
dissect_rrc_T_314(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     8, NULL, FALSE, 0, NULL);

  return offset;
}


static const value_string rrc_T_315_vals[] = {
  {   0, "s0" },
  {   1, "s10" },
  {   2, "s30" },
  {   3, "s60" },
  {   4, "s180" },
  {   5, "s600" },
  {   6, "s1200" },
  {   7, "s1800" },
  { 0, NULL }
};


static int
dissect_rrc_T_315(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     8, NULL, FALSE, 0, NULL);

  return offset;
}


static const value_string rrc_N_315_vals[] = {
  {   0, "s1" },
  {   1, "s50" },
  {   2, "s100" },
  {   3, "s200" },
  {   4, "s400" },
  {   5, "s600" },
  {   6, "s800" },
  {   7, "s1000" },
  { 0, NULL }
};


static int
dissect_rrc_N_315(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     8, NULL, FALSE, 0, NULL);

  return offset;
}


static const value_string rrc_T_316_vals[] = {
  {   0, "s0" },
  {   1, "s10" },
  {   2, "s20" },
  {   3, "s30" },
  {   4, "s40" },
  {   5, "s50" },
  {   6, "s-inf" },
  {   7, "spare" },
  { 0, NULL }
};


static int
dissect_rrc_T_316(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     8, NULL, FALSE, 0, NULL);

  return offset;
}


static const value_string rrc_T_317_vals[] = {
  {   0, "infinity0" },
  {   1, "infinity1" },
  {   2, "infinity2" },
  {   3, "infinity3" },
  {   4, "infinity4" },
  {   5, "infinity5" },
  {   6, "infinity6" },
  {   7, "infinity7" },
  { 0, NULL }
};


static int
dissect_rrc_T_317(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     8, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t UE_ConnTimersAndConstants_sequence[] = {
  { &hf_rrc_t_301           , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_301 },
  { &hf_rrc_n_301           , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_N_301 },
  { &hf_rrc_t_302           , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_302 },
  { &hf_rrc_n_302           , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_N_302 },
  { &hf_rrc_t_304           , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_304 },
  { &hf_rrc_n_304           , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_N_304 },
  { &hf_rrc_t_305           , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_305 },
  { &hf_rrc_t_307           , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_307 },
  { &hf_rrc_t_308           , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_308 },
  { &hf_rrc_t_309           , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_309 },
  { &hf_rrc_t_310           , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_310 },
  { &hf_rrc_n_310           , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_N_310 },
  { &hf_rrc_t_311           , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_311 },
  { &hf_rrc_t_312           , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_312 },
  { &hf_rrc_n_312           , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_N_312 },
  { &hf_rrc_t_313           , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_313 },
  { &hf_rrc_n_313           , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_N_313 },
  { &hf_rrc_t_314           , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_314 },
  { &hf_rrc_t_315           , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_315 },
  { &hf_rrc_n_315           , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_N_315 },
  { &hf_rrc_t_316           , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_316 },
  { &hf_rrc_t_317           , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_317 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UE_ConnTimersAndConstants(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UE_ConnTimersAndConstants, UE_ConnTimersAndConstants_sequence);

  return offset;
}



static int
dissect_rrc_T_cn_CommonGSM_MAP_NAS_SysInfo_02(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
#line 666 "../../asn1/rrc/rrc.cnf"
  rrc_nas_sys_info_gsm_map_type = RRC_NAS_SYS_INFO_CN_COMMON;
  offset = dissect_rrc_NAS_SystemInformationGSM_MAP(tvb, offset, actx, tree, hf_index);




  return offset;
}



static int
dissect_rrc_CN_DRX_CycleLengthCoefficient(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            6U, 9U, NULL, FALSE);

  return offset;
}


static const per_sequence_t CN_DomainInformationFull_sequence[] = {
  { &hf_rrc_cn_DomainIdentity, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CN_DomainIdentity },
  { &hf_rrc_cn_DomainSpecificNAS_Info, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_NAS_SystemInformationGSM_MAP },
  { &hf_rrc_cn_DRX_CycleLengthCoeff, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CN_DRX_CycleLengthCoefficient },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_CN_DomainInformationFull(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_CN_DomainInformationFull, CN_DomainInformationFull_sequence);

  return offset;
}


static const per_sequence_t CN_DomainInformationListFull_sequence_of[1] = {
  { &hf_rrc_CN_DomainInformationListFull_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CN_DomainInformationFull },
};

static int
dissect_rrc_CN_DomainInformationListFull(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_CN_DomainInformationListFull, CN_DomainInformationListFull_sequence_of,
                                                  1, maxCNdomains, FALSE);

  return offset;
}


static const per_sequence_t CN_InformationInfoFull_sequence[] = {
  { &hf_rrc_plmn_Identity   , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_PLMN_Identity },
  { &hf_rrc_cn_CommonGSM_MAP_NAS_SysInfo_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_cn_CommonGSM_MAP_NAS_SysInfo_02 },
  { &hf_rrc_cn_DomainInformationListFull, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CN_DomainInformationListFull },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_CN_InformationInfoFull(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_CN_InformationInfoFull, CN_InformationInfoFull_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions_176_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_nonCriticalExtensions_176(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_nonCriticalExtensions_176, T_nonCriticalExtensions_176_sequence);

  return offset;
}


static const per_sequence_t UTRANMobilityInformation_r3_IEs_sequence[] = {
  { &hf_rrc_rrc_TransactionIdentifier, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RRC_TransactionIdentifier },
  { &hf_rrc_integrityProtectionModeInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_IntegrityProtectionModeInfo },
  { &hf_rrc_cipheringModeInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CipheringModeInfo },
  { &hf_rrc_new_U_RNTI      , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_U_RNTI },
  { &hf_rrc_new_C_RNTI      , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_C_RNTI },
  { &hf_rrc_ue_ConnTimersAndConstants, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_ConnTimersAndConstants },
  { &hf_rrc_cn_InformationInfo_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CN_InformationInfoFull },
  { &hf_rrc_ura_Identity    , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_URA_Identity },
  { &hf_rrc_dl_CounterSynchronisationInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_CounterSynchronisationInfo },
  { &hf_rrc_nonCriticalExtensions_176, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_nonCriticalExtensions_176 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UTRANMobilityInformation_r3_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UTRANMobilityInformation_r3_IEs, UTRANMobilityInformation_r3_IEs_sequence);

  return offset;
}


static const value_string rrc_N_312ext_vals[] = {
  {   0, "s2" },
  {   1, "s4" },
  {   2, "s10" },
  {   3, "s20" },
  { 0, NULL }
};


static int
dissect_rrc_N_312ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     4, NULL, FALSE, 0, NULL);

  return offset;
}


static const value_string rrc_N_315ext_vals[] = {
  {   0, "s2" },
  {   1, "s4" },
  {   2, "s10" },
  {   3, "s20" },
  { 0, NULL }
};


static int
dissect_rrc_N_315ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     4, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t UE_ConnTimersAndConstants_v3a0ext_sequence[] = {
  { &hf_rrc_n_312_01        , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_N_312ext },
  { &hf_rrc_n_315_01        , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_N_315ext },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UE_ConnTimersAndConstants_v3a0ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UE_ConnTimersAndConstants_v3a0ext, UE_ConnTimersAndConstants_v3a0ext_sequence);

  return offset;
}


static const per_sequence_t UTRANMobilityInformation_v3a0ext_IEs_sequence[] = {
  { &hf_rrc_ue_ConnTimersAndConstants_v3a0ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UE_ConnTimersAndConstants_v3a0ext },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UTRANMobilityInformation_v3a0ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UTRANMobilityInformation_v3a0ext_IEs, UTRANMobilityInformation_v3a0ext_IEs_sequence);

  return offset;
}


static const per_sequence_t UtranMobilityInformation_v690ext_IEs_sequence[] = {
  { &hf_rrc_primary_plmn_Identity, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_PLMN_Identity },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UtranMobilityInformation_v690ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UtranMobilityInformation_v690ext_IEs, UtranMobilityInformation_v690ext_IEs_sequence);

  return offset;
}


static const value_string rrc_T_323_vals[] = {
  {   0, "s0" },
  {   1, "s5" },
  {   2, "s10" },
  {   3, "s20" },
  {   4, "s30" },
  {   5, "s60" },
  {   6, "s90" },
  {   7, "s120" },
  { 0, NULL }
};


static int
dissect_rrc_T_323(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     8, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t UE_ConnTimersAndConstants_v860ext_sequence[] = {
  { &hf_rrc_t_323           , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_323 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UE_ConnTimersAndConstants_v860ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UE_ConnTimersAndConstants_v860ext, UE_ConnTimersAndConstants_v860ext_sequence);

  return offset;
}


static const per_sequence_t UTRANMobilityInformation_v860ext1_IEs_sequence[] = {
  { &hf_rrc_ue_ConnTimersAndConstants_02, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UE_ConnTimersAndConstants_v860ext },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UTRANMobilityInformation_v860ext1_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UTRANMobilityInformation_v860ext1_IEs, UTRANMobilityInformation_v860ext1_IEs_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions_173_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_nonCriticalExtensions_173(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_nonCriticalExtensions_173, T_nonCriticalExtensions_173_sequence);

  return offset;
}


static const per_sequence_t T_v860NonCriticalExtentions_sequence[] = {
  { &hf_rrc_utranMobilityInformation_v860ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UTRANMobilityInformation_v860ext1_IEs },
  { &hf_rrc_nonCriticalExtensions_173, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_nonCriticalExtensions_173 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v860NonCriticalExtentions(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v860NonCriticalExtentions, T_v860NonCriticalExtentions_sequence);

  return offset;
}


static const per_sequence_t T_v690NonCriticalExtensions_46_sequence[] = {
  { &hf_rrc_utranMobilityInformation_v690ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UtranMobilityInformation_v690ext_IEs },
  { &hf_rrc_v860NonCriticalExtentions, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v860NonCriticalExtentions },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v690NonCriticalExtensions_46(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v690NonCriticalExtensions_46, T_v690NonCriticalExtensions_46_sequence);

  return offset;
}


static const per_sequence_t T_laterNonCriticalExtensions_64_sequence[] = {
  { &hf_rrc_utranMobilityInformation_r3_add_ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_BIT_STRING },
  { &hf_rrc_v690NonCriticalExtensions_46, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v690NonCriticalExtensions_46 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_laterNonCriticalExtensions_64(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_laterNonCriticalExtensions_64, T_laterNonCriticalExtensions_64_sequence);

  return offset;
}


static const per_sequence_t T_v3a0NonCriticalExtensions_11_sequence[] = {
  { &hf_rrc_utranMobilityInformation_v3a0ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UTRANMobilityInformation_v3a0ext_IEs },
  { &hf_rrc_laterNonCriticalExtensions_64, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_laterNonCriticalExtensions_64 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v3a0NonCriticalExtensions_11(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v3a0NonCriticalExtensions_11, T_v3a0NonCriticalExtensions_11_sequence);

  return offset;
}


static const per_sequence_t T_r3_30_sequence[] = {
  { &hf_rrc_utranMobilityInformation_r3, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UTRANMobilityInformation_r3_IEs },
  { &hf_rrc_v3a0NonCriticalExtensions_11, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v3a0NonCriticalExtensions_11 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_r3_30(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_r3_30, T_r3_30_sequence);

  return offset;
}


static const value_string rrc_N_312_r5_vals[] = {
  {   0, "s1" },
  {   1, "s2" },
  {   2, "s4" },
  {   3, "s10" },
  {   4, "s20" },
  {   5, "s50" },
  {   6, "s100" },
  {   7, "s200" },
  {   8, "s400" },
  {   9, "s600" },
  {  10, "s800" },
  {  11, "s1000" },
  { 0, NULL }
};


static int
dissect_rrc_N_312_r5(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     12, NULL, FALSE, 0, NULL);

  return offset;
}


static const value_string rrc_N_315_r5_vals[] = {
  {   0, "s1" },
  {   1, "s2" },
  {   2, "s4" },
  {   3, "s10" },
  {   4, "s20" },
  {   5, "s50" },
  {   6, "s100" },
  {   7, "s200" },
  {   8, "s400" },
  {   9, "s600" },
  {  10, "s800" },
  {  11, "s1000" },
  { 0, NULL }
};


static int
dissect_rrc_N_315_r5(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     12, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t UE_ConnTimersAndConstants_r5_sequence[] = {
  { &hf_rrc_t_301           , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_301 },
  { &hf_rrc_n_301           , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_N_301 },
  { &hf_rrc_t_302           , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_302 },
  { &hf_rrc_n_302           , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_N_302 },
  { &hf_rrc_t_304           , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_304 },
  { &hf_rrc_n_304           , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_N_304 },
  { &hf_rrc_t_305           , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_305 },
  { &hf_rrc_t_307           , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_307 },
  { &hf_rrc_t_308           , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_308 },
  { &hf_rrc_t_309           , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_309 },
  { &hf_rrc_t_310           , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_310 },
  { &hf_rrc_n_310           , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_N_310 },
  { &hf_rrc_t_311           , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_311 },
  { &hf_rrc_t_312           , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_312 },
  { &hf_rrc_n_312_02        , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_N_312_r5 },
  { &hf_rrc_t_313           , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_313 },
  { &hf_rrc_n_313           , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_N_313 },
  { &hf_rrc_t_314           , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_314 },
  { &hf_rrc_t_315           , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_315 },
  { &hf_rrc_n_315_02        , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_N_315_r5 },
  { &hf_rrc_t_316           , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_316 },
  { &hf_rrc_t_317           , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_317 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UE_ConnTimersAndConstants_r5(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UE_ConnTimersAndConstants_r5, UE_ConnTimersAndConstants_r5_sequence);

  return offset;
}


static const per_sequence_t UTRANMobilityInformation_r5_IEs_sequence[] = {
  { &hf_rrc_integrityProtectionModeInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_IntegrityProtectionModeInfo },
  { &hf_rrc_cipheringModeInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CipheringModeInfo },
  { &hf_rrc_new_U_RNTI      , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_U_RNTI },
  { &hf_rrc_new_C_RNTI      , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_C_RNTI },
  { &hf_rrc_ue_ConnTimersAndConstants_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_ConnTimersAndConstants_r5 },
  { &hf_rrc_cn_InformationInfo_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CN_InformationInfoFull },
  { &hf_rrc_ura_Identity    , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_URA_Identity },
  { &hf_rrc_dl_CounterSynchronisationInfo_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_CounterSynchronisationInfo_r5 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UTRANMobilityInformation_r5_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UTRANMobilityInformation_r5_IEs, UTRANMobilityInformation_r5_IEs_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions_174_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_nonCriticalExtensions_174(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_nonCriticalExtensions_174, T_nonCriticalExtensions_174_sequence);

  return offset;
}


static const per_sequence_t T_v860NonCriticalExtentions_01_sequence[] = {
  { &hf_rrc_utranMobilityInformation_v860ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UTRANMobilityInformation_v860ext1_IEs },
  { &hf_rrc_nonCriticalExtensions_174, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_nonCriticalExtensions_174 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v860NonCriticalExtentions_01(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v860NonCriticalExtentions_01, T_v860NonCriticalExtentions_01_sequence);

  return offset;
}


static const per_sequence_t T_v690NonCriticalExtensions_47_sequence[] = {
  { &hf_rrc_utranMobilityInformation_v690ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UtranMobilityInformation_v690ext_IEs },
  { &hf_rrc_v860NonCriticalExtentions_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v860NonCriticalExtentions_01 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v690NonCriticalExtensions_47(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v690NonCriticalExtensions_47, T_v690NonCriticalExtensions_47_sequence);

  return offset;
}


static const per_sequence_t T_r5_13_sequence[] = {
  { &hf_rrc_utranMobilityInformation_r5, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UTRANMobilityInformation_r5_IEs },
  { &hf_rrc_v690NonCriticalExtensions_47, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v690NonCriticalExtensions_47 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_r5_13(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_r5_13, T_r5_13_sequence);

  return offset;
}


static const per_sequence_t UTRANMobilityInformation_r7_IEs_sequence[] = {
  { &hf_rrc_integrityProtectionModeInfo_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_IntegrityProtectionModeInfo_r7 },
  { &hf_rrc_cipheringModeInfo_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CipheringModeInfo_r7 },
  { &hf_rrc_new_U_RNTI      , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_U_RNTI },
  { &hf_rrc_new_C_RNTI      , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_C_RNTI },
  { &hf_rrc_new_H_RNTI      , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_H_RNTI },
  { &hf_rrc_ue_ConnTimersAndConstants_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_ConnTimersAndConstants_r5 },
  { &hf_rrc_cn_InformationInfo_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CN_InformationInfoFull },
  { &hf_rrc_primary_plmn_Identity, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_PLMN_Identity },
  { &hf_rrc_ura_Identity    , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_URA_Identity },
  { &hf_rrc_supportForChangeOfUE_Capability, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_BOOLEAN },
  { &hf_rrc_dl_CounterSynchronisationInfo_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_CounterSynchronisationInfo_r5 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UTRANMobilityInformation_r7_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UTRANMobilityInformation_r7_IEs, UTRANMobilityInformation_r7_IEs_sequence);

  return offset;
}


static const value_string rrc_T_322_vals[] = {
  {   0, "m5" },
  {   1, "m10" },
  {   2, "m20" },
  {   3, "m30" },
  {   4, "m60" },
  {   5, "m120" },
  {   6, "m180" },
  {   7, "spare1" },
  { 0, NULL }
};


static int
dissect_rrc_T_322(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     8, NULL, FALSE, 0, NULL);

  return offset;
}



static int
dissect_rrc_INTEGER_0_maxPrio_1(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            0U, maxPrio_1, NULL, FALSE);

  return offset;
}


static const per_sequence_t T_utraFDD_item_sequence[] = {
  { &hf_rrc_uarfcn          , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UARFCN },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_utraFDD_item(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_utraFDD_item, T_utraFDD_item_sequence);

  return offset;
}


static const per_sequence_t T_utraFDD_sequence_of[1] = {
  { &hf_rrc_utraFDD_item    , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_utraFDD_item },
};

static int
dissect_rrc_T_utraFDD(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_T_utraFDD, T_utraFDD_sequence_of,
                                                  1, maxNumFDDFreqs, FALSE);

  return offset;
}


static const per_sequence_t T_utraTDD_item_sequence[] = {
  { &hf_rrc_uarfcn          , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UARFCN },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_utraTDD_item(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_utraTDD_item, T_utraTDD_item_sequence);

  return offset;
}


static const per_sequence_t T_utraTDD_sequence_of[1] = {
  { &hf_rrc_utraTDD_item    , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_utraTDD_item },
};

static int
dissect_rrc_T_utraTDD(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_T_utraTDD, T_utraTDD_sequence_of,
                                                  1, maxNumTDDFreqs, FALSE);

  return offset;
}


static const per_sequence_t T_eutra_item_sequence[] = {
  { &hf_rrc_earfcn          , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_EARFCN },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_eutra_item(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_eutra_item, T_eutra_item_sequence);

  return offset;
}


static const per_sequence_t T_eutra_sequence_of[1] = {
  { &hf_rrc_eutra_item      , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_eutra_item },
};

static int
dissect_rrc_T_eutra(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_T_eutra, T_eutra_sequence_of,
                                                  1, maxNumEUTRAFreqs, FALSE);

  return offset;
}


static const value_string rrc_T_bandIndicator_vals[] = {
  {   0, "dcs1800" },
  {   1, "pcs1900" },
  { 0, NULL }
};


static int
dissect_rrc_T_bandIndicator(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     2, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t SEQUENCE_SIZE_0_31_OF_BCCH_ARFCN_sequence_of[1] = {
  { &hf_rrc_explicitListOfARFCNs_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BCCH_ARFCN },
};

static int
dissect_rrc_SEQUENCE_SIZE_0_31_OF_BCCH_ARFCN(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_SEQUENCE_SIZE_0_31_OF_BCCH_ARFCN, SEQUENCE_SIZE_0_31_OF_BCCH_ARFCN_sequence_of,
                                                  0, 31, FALSE);

  return offset;
}


static const per_sequence_t T_equallySpacedARFCNs_sequence[] = {
  { &hf_rrc_arfcn_Spacing   , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_1_8 },
  { &hf_rrc_numberOfFollowingARFCNs, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_0_31 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_equallySpacedARFCNs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_equallySpacedARFCNs, T_equallySpacedARFCNs_sequence);

  return offset;
}



static int
dissect_rrc_OCTET_STRING_SIZE_1_16(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_octet_string(tvb, offset, actx, tree, hf_index,
                                       1, 16, FALSE, NULL);

  return offset;
}


static const per_sequence_t T_continuousRangeOfARFCNs_sequence[] = {
  { &hf_rrc_endingARFCN     , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BCCH_ARFCN },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_continuousRangeOfARFCNs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_continuousRangeOfARFCNs, T_continuousRangeOfARFCNs_sequence);

  return offset;
}


static const value_string rrc_T_followingARFCNs_vals[] = {
  {   0, "explicitListOfARFCNs" },
  {   1, "equallySpacedARFCNs" },
  {   2, "variableBitMapOfARFCNs" },
  {   3, "continuousRangeOfARFCNs" },
  { 0, NULL }
};

static const per_choice_t T_followingARFCNs_choice[] = {
  {   0, &hf_rrc_explicitListOfARFCNs, ASN1_NO_EXTENSIONS     , dissect_rrc_SEQUENCE_SIZE_0_31_OF_BCCH_ARFCN },
  {   1, &hf_rrc_equallySpacedARFCNs, ASN1_NO_EXTENSIONS     , dissect_rrc_T_equallySpacedARFCNs },
  {   2, &hf_rrc_variableBitMapOfARFCNs, ASN1_NO_EXTENSIONS     , dissect_rrc_OCTET_STRING_SIZE_1_16 },
  {   3, &hf_rrc_continuousRangeOfARFCNs, ASN1_NO_EXTENSIONS     , dissect_rrc_T_continuousRangeOfARFCNs },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_followingARFCNs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_followingARFCNs, T_followingARFCNs_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t GSM_CellGroup_sequence[] = {
  { &hf_rrc_startingARFCN   , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BCCH_ARFCN },
  { &hf_rrc_bandIndicator   , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_bandIndicator },
  { &hf_rrc_followingARFCNs , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_followingARFCNs },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_GSM_CellGroup(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_GSM_CellGroup, GSM_CellGroup_sequence);

  return offset;
}


static const per_sequence_t T_gsm_01_sequence[] = {
  { &hf_rrc_gsm_CellGroup   , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_GSM_CellGroup },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_gsm_01(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_gsm_01, T_gsm_01_sequence);

  return offset;
}


static const value_string rrc_T_radioAccessTechnology_vals[] = {
  {   0, "utraFDD" },
  {   1, "utraTDD" },
  {   2, "eutra" },
  {   3, "gsm" },
  { 0, NULL }
};

static const per_choice_t T_radioAccessTechnology_choice[] = {
  {   0, &hf_rrc_utraFDD         , ASN1_NO_EXTENSIONS     , dissect_rrc_T_utraFDD },
  {   1, &hf_rrc_utraTDD         , ASN1_NO_EXTENSIONS     , dissect_rrc_T_utraTDD },
  {   2, &hf_rrc_eutra           , ASN1_NO_EXTENSIONS     , dissect_rrc_T_eutra },
  {   3, &hf_rrc_gsm_01          , ASN1_NO_EXTENSIONS     , dissect_rrc_T_gsm_01 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_radioAccessTechnology(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_radioAccessTechnology, T_radioAccessTechnology_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t PriorityLevel_sequence[] = {
  { &hf_rrc_priority        , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_INTEGER_0_maxPrio_1 },
  { &hf_rrc_radioAccessTechnology, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_radioAccessTechnology },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_PriorityLevel(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_PriorityLevel, PriorityLevel_sequence);

  return offset;
}


static const per_sequence_t PriorityLevelList_sequence_of[1] = {
  { &hf_rrc_PriorityLevelList_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PriorityLevel },
};

static int
dissect_rrc_PriorityLevelList(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_PriorityLevelList, PriorityLevelList_sequence_of,
                                                  1, maxPrio, FALSE);

  return offset;
}


static const per_sequence_t T_configureDedicatedPriorities_sequence[] = {
  { &hf_rrc_t_322           , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_322 },
  { &hf_rrc_priorityLevelList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_PriorityLevelList },
  { &hf_rrc_eutraDetection  , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_configureDedicatedPriorities(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_configureDedicatedPriorities, T_configureDedicatedPriorities_sequence);

  return offset;
}


static const value_string rrc_T_action_vals[] = {
  {   0, "clearDedicatedPriorities" },
  {   1, "configureDedicatedPriorities" },
  { 0, NULL }
};

static const per_choice_t T_action_choice[] = {
  {   0, &hf_rrc_clearDedicatedPriorities, ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   1, &hf_rrc_configureDedicatedPriorities, ASN1_NO_EXTENSIONS     , dissect_rrc_T_configureDedicatedPriorities },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_action(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_action, T_action_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t DedicatedPriorityInformation_sequence[] = {
  { &hf_rrc_action          , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_action },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_DedicatedPriorityInformation(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_DedicatedPriorityInformation, DedicatedPriorityInformation_sequence);

  return offset;
}


static const per_sequence_t UTRANMobilityInformation_v860ext2_IEs_sequence[] = {
  { &hf_rrc_newPrimary_E_RNTI, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_E_RNTI },
  { &hf_rrc_ue_ConnTimersAndConstants_02, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UE_ConnTimersAndConstants_v860ext },
  { &hf_rrc_dedicatedPriorityInformation, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DedicatedPriorityInformation },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UTRANMobilityInformation_v860ext2_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UTRANMobilityInformation_v860ext2_IEs, UTRANMobilityInformation_v860ext2_IEs_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions_175_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_nonCriticalExtensions_175(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_nonCriticalExtensions_175, T_nonCriticalExtensions_175_sequence);

  return offset;
}


static const per_sequence_t T_v860NonCriticalExtensions_22_sequence[] = {
  { &hf_rrc_utranMobilityInformation_v860ext_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UTRANMobilityInformation_v860ext2_IEs },
  { &hf_rrc_nonCriticalExtensions_175, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_nonCriticalExtensions_175 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v860NonCriticalExtensions_22(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v860NonCriticalExtensions_22, T_v860NonCriticalExtensions_22_sequence);

  return offset;
}


static const per_sequence_t T_r7_14_sequence[] = {
  { &hf_rrc_utranMobilityInformation_r7, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UTRANMobilityInformation_r7_IEs },
  { &hf_rrc_utranMobilityInformation_r7_add_ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_BIT_STRING },
  { &hf_rrc_v860NonCriticalExtensions_22, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v860NonCriticalExtensions_22 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_r7_14(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_r7_14, T_r7_14_sequence);

  return offset;
}


static const per_sequence_t T_criticalExtensions_132_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_criticalExtensions_132(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_criticalExtensions_132, T_criticalExtensions_132_sequence);

  return offset;
}


static const value_string rrc_T_criticalExtensions_131_vals[] = {
  {   0, "r7" },
  {   1, "criticalExtensions" },
  { 0, NULL }
};

static const per_choice_t T_criticalExtensions_131_choice[] = {
  {   0, &hf_rrc_r7_14           , ASN1_NO_EXTENSIONS     , dissect_rrc_T_r7_14 },
  {   1, &hf_rrc_criticalExtensions_133, ASN1_NO_EXTENSIONS     , dissect_rrc_T_criticalExtensions_132 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_criticalExtensions_131(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_criticalExtensions_131, T_criticalExtensions_131_choice,
                                 NULL);

  return offset;
}


static const value_string rrc_T_criticalExtensions_130_vals[] = {
  {   0, "r5" },
  {   1, "criticalExtensions" },
  { 0, NULL }
};

static const per_choice_t T_criticalExtensions_130_choice[] = {
  {   0, &hf_rrc_r5_13           , ASN1_NO_EXTENSIONS     , dissect_rrc_T_r5_13 },
  {   1, &hf_rrc_criticalExtensions_132, ASN1_NO_EXTENSIONS     , dissect_rrc_T_criticalExtensions_131 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_criticalExtensions_130(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_criticalExtensions_130, T_criticalExtensions_130_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t T_later_than_r3_27_sequence[] = {
  { &hf_rrc_rrc_TransactionIdentifier, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RRC_TransactionIdentifier },
  { &hf_rrc_criticalExtensions_131, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_criticalExtensions_130 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_later_than_r3_27(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_later_than_r3_27, T_later_than_r3_27_sequence);

  return offset;
}


static const value_string rrc_UTRANMobilityInformation_vals[] = {
  {   0, "r3" },
  {   1, "later-than-r3" },
  { 0, NULL }
};

static const per_choice_t UTRANMobilityInformation_choice[] = {
  {   0, &hf_rrc_r3_30           , ASN1_NO_EXTENSIONS     , dissect_rrc_T_r3_30 },
  {   1, &hf_rrc_later_than_r3_27, ASN1_NO_EXTENSIONS     , dissect_rrc_T_later_than_r3_27 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_UTRANMobilityInformation(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
#line 495 "../../asn1/rrc/rrc.cnf"
  col_append_str(actx->pinfo->cinfo, COL_INFO, "UTRANMobilityInformation");
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_UTRANMobilityInformation, UTRANMobilityInformation_choice,
                                 NULL);




  return offset;
}


static const per_sequence_t T_single_GERANIu_Message_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_single_GERANIu_Message(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_single_GERANIu_Message, T_single_GERANIu_Message_sequence);

  return offset;
}



static int
dissect_rrc_BIT_STRING_SIZE_1_32768(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_bit_string(tvb, offset, actx, tree, hf_index,
                                     1, 32768, FALSE, NULL);

  return offset;
}


static const per_sequence_t GERANIu_MessageList_sequence_of[1] = {
  { &hf_rrc_GERANIu_MessageList_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_1_32768 },
};

static int
dissect_rrc_GERANIu_MessageList(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_GERANIu_MessageList, GERANIu_MessageList_sequence_of,
                                                  1, maxInterSysMessages, FALSE);

  return offset;
}


static const per_sequence_t T_geranIu_MessageList_sequence[] = {
  { &hf_rrc_geranIu_Messages, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_GERANIu_MessageList },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_geranIu_MessageList(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_geranIu_MessageList, T_geranIu_MessageList_sequence);

  return offset;
}


static const value_string rrc_T_geranIu_Message_vals[] = {
  {   0, "single-GERANIu-Message" },
  {   1, "geranIu-MessageList" },
  { 0, NULL }
};

static const per_choice_t T_geranIu_Message_choice[] = {
  {   0, &hf_rrc_single_GERANIu_Message, ASN1_NO_EXTENSIONS     , dissect_rrc_T_single_GERANIu_Message },
  {   1, &hf_rrc_geranIu_MessageList, ASN1_NO_EXTENSIONS     , dissect_rrc_T_geranIu_MessageList },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_geranIu_Message(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_geranIu_Message, T_geranIu_Message_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t HandoverFromUTRANCommand_GERANIu_r5_IEs_sequence[] = {
  { &hf_rrc_activationTime  , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_ActivationTime },
  { &hf_rrc_frequency_Band  , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_Frequency_Band },
  { &hf_rrc_geranIu_Message , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_geranIu_Message },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_HandoverFromUTRANCommand_GERANIu_r5_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_HandoverFromUTRANCommand_GERANIu_r5_IEs, HandoverFromUTRANCommand_GERANIu_r5_IEs_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions_51_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_nonCriticalExtensions_51(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_nonCriticalExtensions_51, T_nonCriticalExtensions_51_sequence);

  return offset;
}


static const per_sequence_t T_r5_03_sequence[] = {
  { &hf_rrc_handoverFromUTRANCommand_GERANIu_r5, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_HandoverFromUTRANCommand_GERANIu_r5_IEs },
  { &hf_rrc_nonCriticalExtensions_51, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_nonCriticalExtensions_51 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_r5_03(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_r5_03, T_r5_03_sequence);

  return offset;
}


static const per_sequence_t T_criticalExtensions_40_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_criticalExtensions_40(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_criticalExtensions_40, T_criticalExtensions_40_sequence);

  return offset;
}


static const per_sequence_t T_later_than_r5_sequence[] = {
  { &hf_rrc_criticalExtensions_41, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_criticalExtensions_40 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_later_than_r5(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_later_than_r5, T_later_than_r5_sequence);

  return offset;
}


static const value_string rrc_T_handoverFromUTRANCommand_GERANIu_vals[] = {
  {   0, "r5" },
  {   1, "later-than-r5" },
  { 0, NULL }
};

static const per_choice_t T_handoverFromUTRANCommand_GERANIu_choice[] = {
  {   0, &hf_rrc_r5_03           , ASN1_NO_EXTENSIONS     , dissect_rrc_T_r5_03 },
  {   1, &hf_rrc_later_than_r5   , ASN1_NO_EXTENSIONS     , dissect_rrc_T_later_than_r5 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_handoverFromUTRANCommand_GERANIu(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_handoverFromUTRANCommand_GERANIu, T_handoverFromUTRANCommand_GERANIu_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t HandoverFromUTRANCommand_GERANIu_sequence[] = {
  { &hf_rrc_rrc_TransactionIdentifier, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RRC_TransactionIdentifier },
  { &hf_rrc_handoverFromUTRANCommand_GERANIu_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_handoverFromUTRANCommand_GERANIu },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_HandoverFromUTRANCommand_GERANIu(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
#line 229 "../../asn1/rrc/rrc.cnf"
  col_append_str(actx->pinfo->cinfo, COL_INFO, "HandoverFromUTRANCommand-GERANIu");
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_HandoverFromUTRANCommand_GERANIu, HandoverFromUTRANCommand_GERANIu_sequence);




  return offset;
}


static const value_string rrc_T_other_01_vals[] = {
  {   0, "sameAsMIB-MultiPLMN-Id" },
  {   1, "explicitPLMN-Id" },
  { 0, NULL }
};

static const per_choice_t T_other_01_choice[] = {
  {   0, &hf_rrc_sameAsMIB_MultiPLMN_Id, ASN1_NO_EXTENSIONS     , dissect_rrc_INTEGER_1_5 },
  {   1, &hf_rrc_explicitPLMN_Id , ASN1_NO_EXTENSIONS     , dissect_rrc_PLMN_Identity },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_other_01(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_other_01, T_other_01_choice,
                                 NULL);

  return offset;
}


static const value_string rrc_T_plmn_Identity_vals[] = {
  {   0, "sameAsMIB-PLMN-Id" },
  {   1, "other" },
  { 0, NULL }
};

static const per_choice_t T_plmn_Identity_choice[] = {
  {   0, &hf_rrc_sameAsMIB_PLMN_Id, ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   1, &hf_rrc_other_01        , ASN1_NO_EXTENSIONS     , dissect_rrc_T_other_01 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_plmn_Identity(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_plmn_Identity, T_plmn_Identity_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t MBMS_ServiceIdentity_r6_sequence[] = {
  { &hf_rrc_serviceIdentity , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_OCTET_STRING_SIZE_3 },
  { &hf_rrc_plmn_Identity_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_plmn_Identity },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_MBMS_ServiceIdentity_r6(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_MBMS_ServiceIdentity_r6, MBMS_ServiceIdentity_r6_sequence);

  return offset;
}


static const per_sequence_t MBMS_TransmissionIdentity_sequence[] = {
  { &hf_rrc_mbms_ServiceIdentity_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MBMS_ServiceIdentity_r6 },
  { &hf_rrc_mbms_SessionIdentity, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MBMS_SessionIdentity },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_MBMS_TransmissionIdentity(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_MBMS_TransmissionIdentity, MBMS_TransmissionIdentity_sequence);

  return offset;
}


static const value_string rrc_MBMS_RequiredUEAction_Mod_vals[] = {
  {   0, "none" },
  {   1, "acquireCountingInfo" },
  {   2, "acquireCountingInfoPTM-RBsUnmodified" },
  {   3, "acquirePTM-RBInfo" },
  {   4, "requestPTPRB" },
  {   5, "releasePTM-RB" },
  { 0, NULL }
};


static int
dissect_rrc_MBMS_RequiredUEAction_Mod(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     6, NULL, FALSE, 0, NULL);

  return offset;
}



static int
dissect_rrc_MBMS_PFLIndex(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            1U, maxMBMS_Freq, NULL, FALSE);

  return offset;
}



static int
dissect_rrc_MBMS_PFLInfo(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_rrc_FrequencyInfo(tvb, offset, actx, tree, hf_index);

  return offset;
}


static const value_string rrc_T_mbms_PreferredFrequency_vals[] = {
  {   0, "mcch" },
  {   1, "dcch" },
  { 0, NULL }
};

static const per_choice_t T_mbms_PreferredFrequency_choice[] = {
  {   0, &hf_rrc_mcch            , ASN1_NO_EXTENSIONS     , dissect_rrc_MBMS_PFLIndex },
  {   1, &hf_rrc_dcch            , ASN1_NO_EXTENSIONS     , dissect_rrc_MBMS_PFLInfo },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_mbms_PreferredFrequency(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_mbms_PreferredFrequency, T_mbms_PreferredFrequency_choice,
                                 NULL);

  return offset;
}


static const value_string rrc_T_dummy_16_vals[] = {
  {   0, "true" },
  { 0, NULL }
};


static int
dissect_rrc_T_dummy_16(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t MBMS_ModifedService_r6_sequence[] = {
  { &hf_rrc_mbms_TransmissionIdentity, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MBMS_TransmissionIdentity },
  { &hf_rrc_mbms_RequiredUEAction, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MBMS_RequiredUEAction_Mod },
  { &hf_rrc_mbms_PreferredFrequency, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_mbms_PreferredFrequency },
  { &hf_rrc_dummy_61        , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_dummy_16 },
  { &hf_rrc_continueMCCHReading, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_MBMS_ModifedService_r6(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_MBMS_ModifedService_r6, MBMS_ModifedService_r6_sequence);

  return offset;
}


static const per_sequence_t MBMS_ModifedServiceList_r6_sequence_of[1] = {
  { &hf_rrc_MBMS_ModifedServiceList_r6_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MBMS_ModifedService_r6 },
};

static int
dissect_rrc_MBMS_ModifedServiceList_r6(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_MBMS_ModifedServiceList_r6, MBMS_ModifedServiceList_r6_sequence_of,
                                                  1, maxMBMSservModif, FALSE);

  return offset;
}


static const value_string rrc_T_mbms_ReacquireMCCH_vals[] = {
  {   0, "true" },
  { 0, NULL }
};


static int
dissect_rrc_T_mbms_ReacquireMCCH(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}



static int
dissect_rrc_DynamicPersistenceLevel(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            1U, 8U, NULL, FALSE);

  return offset;
}



static int
dissect_rrc_MBMS_NumberOfNeighbourCells_r6(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            0U, 32U, NULL, FALSE);

  return offset;
}


static const value_string rrc_T_mbms_AllUnmodifiedPTMServices_vals[] = {
  {   0, "true" },
  { 0, NULL }
};


static int
dissect_rrc_T_mbms_AllUnmodifiedPTMServices(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}



static int
dissect_rrc_MBMS_PTMActivationTime_r6(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            0U, 2047U, NULL, FALSE);

  return offset;
}



static int
dissect_rrc_MBSFN_ClusterFrequency_r7(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            1U, maxMBSFNClusters, NULL, FALSE);

  return offset;
}


static const per_sequence_t MBMS_ModifiedService_v770ext_sequence[] = {
  { &hf_rrc_mbsfnClusterFrequency, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MBSFN_ClusterFrequency_r7 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_MBMS_ModifiedService_v770ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_MBMS_ModifiedService_v770ext, MBMS_ModifiedService_v770ext_sequence);

  return offset;
}


static const per_sequence_t MBMS_ModifiedServiceList_v770ext_sequence_of[1] = {
  { &hf_rrc_MBMS_ModifiedServiceList_v770ext_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MBMS_ModifiedService_v770ext },
};

static int
dissect_rrc_MBMS_ModifiedServiceList_v770ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_MBMS_ModifiedServiceList_v770ext, MBMS_ModifiedServiceList_v770ext_sequence_of,
                                                  1, maxMBMSservModif, FALSE);

  return offset;
}



static int
dissect_rrc_MIB_ValueTag(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            1U, 8U, NULL, FALSE);

  return offset;
}


static const per_sequence_t MBMSModifiedServicesInformation_v770ext_IEs_sequence[] = {
  { &hf_rrc_modifiedServiceList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MBMS_ModifiedServiceList_v770ext },
  { &hf_rrc_mib_ValueTag    , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MIB_ValueTag },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_MBMSModifiedServicesInformation_v770ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_MBMSModifiedServicesInformation_v770ext_IEs, MBMSModifiedServicesInformation_v770ext_IEs_sequence);

  return offset;
}


static const value_string rrc_MBMS_PTM_RBReleaseCause_LCR_r7_vals[] = {
  {   0, "normalRelease" },
  {   1, "outOfMBMSServiceCoverageInRAN" },
  {   2, "networkAbnormalRelease" },
  {   3, "spare5" },
  {   4, "spare4" },
  {   5, "spare3" },
  {   6, "spare2" },
  {   7, "spare1" },
  { 0, NULL }
};


static int
dissect_rrc_MBMS_PTM_RBReleaseCause_LCR_r7(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     8, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t MBMS_ModifiedService_LCR_v7c0ext_sequence[] = {
  { &hf_rrc_rbReleaseCause  , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MBMS_PTM_RBReleaseCause_LCR_r7 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_MBMS_ModifiedService_LCR_v7c0ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_MBMS_ModifiedService_LCR_v7c0ext, MBMS_ModifiedService_LCR_v7c0ext_sequence);

  return offset;
}


static const per_sequence_t MBMS_ModifiedServiceList_LCR_v7c0ext_sequence_of[1] = {
  { &hf_rrc_MBMS_ModifiedServiceList_LCR_v7c0ext_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MBMS_ModifiedService_LCR_v7c0ext },
};

static int
dissect_rrc_MBMS_ModifiedServiceList_LCR_v7c0ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_MBMS_ModifiedServiceList_LCR_v7c0ext, MBMS_ModifiedServiceList_LCR_v7c0ext_sequence_of,
                                                  1, maxMBMSservModif, FALSE);

  return offset;
}


static const per_sequence_t MBMSModifiedServicesInformation_v7c0ext_IEs_sequence[] = {
  { &hf_rrc_modifiedServiceList_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MBMS_ModifiedServiceList_LCR_v7c0ext },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_MBMSModifiedServicesInformation_v7c0ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_MBMSModifiedServicesInformation_v7c0ext_IEs, MBMSModifiedServicesInformation_v7c0ext_IEs_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions_184_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_nonCriticalExtensions_184(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_nonCriticalExtensions_184, T_nonCriticalExtensions_184_sequence);

  return offset;
}


static const per_sequence_t T_v7c0NonCriticalExtensions_sequence[] = {
  { &hf_rrc_mbmsModifiedServicesInformation_v7c0ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MBMSModifiedServicesInformation_v7c0ext_IEs },
  { &hf_rrc_nonCriticalExtensions_184, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_nonCriticalExtensions_184 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v7c0NonCriticalExtensions(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v7c0NonCriticalExtensions, T_v7c0NonCriticalExtensions_sequence);

  return offset;
}


static const per_sequence_t T_v770NonCriticalExtensions_36_sequence[] = {
  { &hf_rrc_mbmsModifiedServicesInformation_v770ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MBMSModifiedServicesInformation_v770ext_IEs },
  { &hf_rrc_v7c0NonCriticalExtensions, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v7c0NonCriticalExtensions },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v770NonCriticalExtensions_36(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v770NonCriticalExtensions_36, T_v770NonCriticalExtensions_36_sequence);

  return offset;
}


static const per_sequence_t MBMSModifiedServicesInformation_sequence[] = {
  { &hf_rrc_modifedServiceList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MBMS_ModifedServiceList_r6 },
  { &hf_rrc_mbms_ReacquireMCCH, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_mbms_ReacquireMCCH },
  { &hf_rrc_mbms_DynamicPersistenceLevel, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DynamicPersistenceLevel },
  { &hf_rrc_endOfModifiedMCCHInformation, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_INTEGER_1_16 },
  { &hf_rrc_mbmsNumberOfNeighbourCells, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MBMS_NumberOfNeighbourCells_r6 },
  { &hf_rrc_mbms_AllUnmodifiedPTMServices, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_mbms_AllUnmodifiedPTMServices },
  { &hf_rrc_mbms_PTMActivationTime, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MBMS_PTMActivationTime_r6 },
  { &hf_rrc_v770NonCriticalExtensions_36, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v770NonCriticalExtensions_36 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_MBMSModifiedServicesInformation(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
#line 269 "../../asn1/rrc/rrc.cnf"
  col_append_str(actx->pinfo->cinfo, COL_INFO, "MBMSModifiedServicesInformation");
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_MBMSModifiedServicesInformation, MBMSModifiedServicesInformation_sequence);




  return offset;
}



static int
dissect_rrc_OCTET_STRING_SIZE_1_2(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_octet_string(tvb, offset, actx, tree, hf_index,
                                       1, 2, FALSE, NULL);

  return offset;
}



static int
dissect_rrc_OCTET_STRING_SIZE_2(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_octet_string(tvb, offset, actx, tree, hf_index,
                                       2, 2, FALSE, NULL);

  return offset;
}


static const per_sequence_t ETWS_Information_sequence[] = {
  { &hf_rrc_warningType     , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_OCTET_STRING_SIZE_1_2 },
  { &hf_rrc_messageIdentifier, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_OCTET_STRING_SIZE_2 },
  { &hf_rrc_serialNumber    , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_OCTET_STRING_SIZE_2 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_ETWS_Information(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_ETWS_Information, ETWS_Information_sequence);

  return offset;
}



static int
dissect_rrc_ETWS_WarningSecurityInfo(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_octet_string(tvb, offset, actx, tree, hf_index,
                                       NO_BOUND, NO_BOUND, FALSE, NULL);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions_36_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_nonCriticalExtensions_36(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_nonCriticalExtensions_36, T_nonCriticalExtensions_36_sequence);

  return offset;
}


static const per_sequence_t ETWSPrimaryNotificationWithSecurity_sequence[] = {
  { &hf_rrc_etws_Information, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_ETWS_Information },
  { &hf_rrc_etws_WarningSecurityInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_ETWS_WarningSecurityInfo },
  { &hf_rrc_nonCriticalExtensions_36, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_nonCriticalExtensions_36 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_ETWSPrimaryNotificationWithSecurity(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
#line 213 "../../asn1/rrc/rrc.cnf"
  col_append_str(actx->pinfo->cinfo, COL_INFO, "ETWSPrimaryNotificationWithSecurity");
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_ETWSPrimaryNotificationWithSecurity, ETWSPrimaryNotificationWithSecurity_sequence);




  return offset;
}



static int
dissect_rrc_T_eutra_Message(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
#line 609 "../../asn1/rrc/rrc.cnf"
  tvbuff_t *eutra_message_tvb = NULL;
  offset = dissect_per_octet_string(tvb, offset, actx, tree, hf_index,
                                       NO_BOUND, NO_BOUND, FALSE, &eutra_message_tvb);

  if (eutra_message_tvb && lte_rrc_dl_dcch_handle)
    call_dissector(lte_rrc_dl_dcch_handle, eutra_message_tvb, actx->pinfo, tree);



  return offset;
}


static const per_sequence_t HandoverFromUTRANCommand_EUTRA_r8_IEs_sequence[] = {
  { &hf_rrc_activationTime  , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_ActivationTime },
  { &hf_rrc_toHandoverRAB_Info_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RAB_InformationList },
  { &hf_rrc_eutra_Message   , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_eutra_Message },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_HandoverFromUTRANCommand_EUTRA_r8_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_HandoverFromUTRANCommand_EUTRA_r8_IEs, HandoverFromUTRANCommand_EUTRA_r8_IEs_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions_53_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_nonCriticalExtensions_53(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_nonCriticalExtensions_53, T_nonCriticalExtensions_53_sequence);

  return offset;
}


static const per_sequence_t T_r8_04_sequence[] = {
  { &hf_rrc_handoverFromUTRANCommand_EUTRA_r8, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_HandoverFromUTRANCommand_EUTRA_r8_IEs },
  { &hf_rrc_handoverFromUTRANCommand_EUTRA_r8_add_ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_BIT_STRING },
  { &hf_rrc_nonCriticalExtensions_53, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_nonCriticalExtensions_53 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_r8_04(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_r8_04, T_r8_04_sequence);

  return offset;
}



static int
dissect_rrc_OCTET_STRING_SIZE_1_32(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_octet_string(tvb, offset, actx, tree, hf_index,
                                       1, 32, FALSE, NULL);

  return offset;
}


static const per_sequence_t RSR_VCC_Info_sequence[] = {
  { &hf_rrc_nonce           , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_BIT_STRING_SIZE_128 },
  { &hf_rrc_ims_Information , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_OCTET_STRING_SIZE_1_32 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_RSR_VCC_Info(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_RSR_VCC_Info, RSR_VCC_Info_sequence);

  return offset;
}



static int
dissect_rrc_T_eutra_Message_01(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
#line 615 "../../asn1/rrc/rrc.cnf"
  tvbuff_t *eutra_message_tvb = NULL;
  offset = dissect_per_octet_string(tvb, offset, actx, tree, hf_index,
                                       NO_BOUND, NO_BOUND, FALSE, &eutra_message_tvb);

  if (eutra_message_tvb && lte_rrc_dl_dcch_handle)
    call_dissector(lte_rrc_dl_dcch_handle, eutra_message_tvb, actx->pinfo, tree);



  return offset;
}


static const per_sequence_t HandoverFromUTRANCommand_EUTRA_r11_IEs_sequence[] = {
  { &hf_rrc_rsr_vcc_Info    , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RSR_VCC_Info },
  { &hf_rrc_activationTime  , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_ActivationTime },
  { &hf_rrc_toHandoverRAB_Info_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RAB_InformationList },
  { &hf_rrc_eutra_Message_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_eutra_Message_01 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_HandoverFromUTRANCommand_EUTRA_r11_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_HandoverFromUTRANCommand_EUTRA_r11_IEs, HandoverFromUTRANCommand_EUTRA_r11_IEs_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions_54_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_nonCriticalExtensions_54(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_nonCriticalExtensions_54, T_nonCriticalExtensions_54_sequence);

  return offset;
}


static const per_sequence_t T_r11_04_sequence[] = {
  { &hf_rrc_handoverFromUTRANCommand_EUTRA_r11, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_HandoverFromUTRANCommand_EUTRA_r11_IEs },
  { &hf_rrc_handoverFromUTRANCommand_EUTRA_r11_add_ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_BIT_STRING },
  { &hf_rrc_nonCriticalExtensions_54, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_nonCriticalExtensions_54 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_r11_04(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_r11_04, T_r11_04_sequence);

  return offset;
}


static const per_sequence_t T_criticalExtensions_44_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_criticalExtensions_44(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_criticalExtensions_44, T_criticalExtensions_44_sequence);

  return offset;
}


static const value_string rrc_T_criticalExtensions_43_vals[] = {
  {   0, "r11" },
  {   1, "criticalExtensions" },
  { 0, NULL }
};

static const per_choice_t T_criticalExtensions_43_choice[] = {
  {   0, &hf_rrc_r11_04          , ASN1_NO_EXTENSIONS     , dissect_rrc_T_r11_04 },
  {   1, &hf_rrc_criticalExtensions_45, ASN1_NO_EXTENSIONS     , dissect_rrc_T_criticalExtensions_44 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_criticalExtensions_43(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_criticalExtensions_43, T_criticalExtensions_43_choice,
                                 NULL);

  return offset;
}


static const value_string rrc_T_criticalExtensions_42_vals[] = {
  {   0, "r8" },
  {   1, "criticalExtensions" },
  { 0, NULL }
};

static const per_choice_t T_criticalExtensions_42_choice[] = {
  {   0, &hf_rrc_r8_04           , ASN1_NO_EXTENSIONS     , dissect_rrc_T_r8_04 },
  {   1, &hf_rrc_criticalExtensions_44, ASN1_NO_EXTENSIONS     , dissect_rrc_T_criticalExtensions_43 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_criticalExtensions_42(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_criticalExtensions_42, T_criticalExtensions_42_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t HandoverFromUTRANCommand_EUTRA_sequence[] = {
  { &hf_rrc_rrc_TransactionIdentifier, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RRC_TransactionIdentifier },
  { &hf_rrc_criticalExtensions_43, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_criticalExtensions_42 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_HandoverFromUTRANCommand_EUTRA(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
#line 225 "../../asn1/rrc/rrc.cnf"
  col_append_str(actx->pinfo->cinfo, COL_INFO, "HandoverFromUTRANCommand-EUTRA");
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_HandoverFromUTRANCommand_EUTRA, HandoverFromUTRANCommand_EUTRA_sequence);




  return offset;
}


static const value_string rrc_T_loggedMeasReportRequest_vals[] = {
  {   0, "true" },
  { 0, NULL }
};


static int
dissect_rrc_T_loggedMeasReportRequest(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}


static const value_string rrc_T_loggedANRReportRequest_vals[] = {
  {   0, "true" },
  { 0, NULL }
};


static int
dissect_rrc_T_loggedANRReportRequest(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t UEInformationRequest_r10_IEs_sequence[] = {
  { &hf_rrc_loggedMeasReportRequest, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_loggedMeasReportRequest },
  { &hf_rrc_loggedANRReportRequest, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_loggedANRReportRequest },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UEInformationRequest_r10_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UEInformationRequest_r10_IEs, UEInformationRequest_r10_IEs_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions_192_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_nonCriticalExtensions_192(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_nonCriticalExtensions_192, T_nonCriticalExtensions_192_sequence);

  return offset;
}


static const per_sequence_t T_r10_12_sequence[] = {
  { &hf_rrc_ueInformationRequest_r10, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UEInformationRequest_r10_IEs },
  { &hf_rrc_ueInformationRequest_r10_add_ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_BIT_STRING },
  { &hf_rrc_nonCriticalExtensions_192, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_nonCriticalExtensions_192 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_r10_12(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_r10_12, T_r10_12_sequence);

  return offset;
}


static const value_string rrc_T_loggedMeasReportRequest_01_vals[] = {
  {   0, "true" },
  { 0, NULL }
};


static int
dissect_rrc_T_loggedMeasReportRequest_01(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}


static const value_string rrc_T_loggedANRReportRequest_01_vals[] = {
  {   0, "true" },
  { 0, NULL }
};


static int
dissect_rrc_T_loggedANRReportRequest_01(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}


static const value_string rrc_T_loggedConnectionEstablishmentFailureRequest_vals[] = {
  {   0, "true" },
  { 0, NULL }
};


static int
dissect_rrc_T_loggedConnectionEstablishmentFailureRequest(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t UEInformationRequest_r11_IEs_sequence[] = {
  { &hf_rrc_loggedMeasReportRequest_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_loggedMeasReportRequest_01 },
  { &hf_rrc_loggedANRReportRequest_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_loggedANRReportRequest_01 },
  { &hf_rrc_loggedConnectionEstablishmentFailureRequest, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_loggedConnectionEstablishmentFailureRequest },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UEInformationRequest_r11_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UEInformationRequest_r11_IEs, UEInformationRequest_r11_IEs_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions_193_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_nonCriticalExtensions_193(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_nonCriticalExtensions_193, T_nonCriticalExtensions_193_sequence);

  return offset;
}


static const per_sequence_t T_r11_13_sequence[] = {
  { &hf_rrc_ueInformationRequest_r11, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UEInformationRequest_r11_IEs },
  { &hf_rrc_ueInformationRequest_r11_add_ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_BIT_STRING },
  { &hf_rrc_nonCriticalExtensions_193, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_nonCriticalExtensions_193 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_r11_13(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_r11_13, T_r11_13_sequence);

  return offset;
}


static const per_sequence_t T_criticalExtensions_138_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_criticalExtensions_138(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_criticalExtensions_138, T_criticalExtensions_138_sequence);

  return offset;
}


static const value_string rrc_T_criticalExtensions_137_vals[] = {
  {   0, "r11" },
  {   1, "criticalExtensions" },
  { 0, NULL }
};

static const per_choice_t T_criticalExtensions_137_choice[] = {
  {   0, &hf_rrc_r11_13          , ASN1_NO_EXTENSIONS     , dissect_rrc_T_r11_13 },
  {   1, &hf_rrc_criticalExtensions_139, ASN1_NO_EXTENSIONS     , dissect_rrc_T_criticalExtensions_138 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_criticalExtensions_137(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_criticalExtensions_137, T_criticalExtensions_137_choice,
                                 NULL);

  return offset;
}


static const value_string rrc_T_criticalExtensions_136_vals[] = {
  {   0, "r10" },
  {   1, "criticalExtensions" },
  { 0, NULL }
};

static const per_choice_t T_criticalExtensions_136_choice[] = {
  {   0, &hf_rrc_r10_12          , ASN1_NO_EXTENSIONS     , dissect_rrc_T_r10_12 },
  {   1, &hf_rrc_criticalExtensions_138, ASN1_NO_EXTENSIONS     , dissect_rrc_T_criticalExtensions_137 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_criticalExtensions_136(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_criticalExtensions_136, T_criticalExtensions_136_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t UEInformationRequest_sequence[] = {
  { &hf_rrc_rrc_TransactionIdentifier, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RRC_TransactionIdentifier },
  { &hf_rrc_criticalExtensions_137, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_criticalExtensions_136 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UEInformationRequest(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
#line 467 "../../asn1/rrc/rrc.cnf"
  col_append_str(actx->pinfo->cinfo, COL_INFO, "UEInformationRequest");
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UEInformationRequest, UEInformationRequest_sequence);




  return offset;
}



static int
dissect_rrc_BIT_STRING_SIZE_48(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_bit_string(tvb, offset, actx, tree, hf_index,
                                     48, 48, FALSE, NULL);

  return offset;
}


static const value_string rrc_T_loggingDuration_01_vals[] = {
  {   0, "min10" },
  {   1, "min20" },
  {   2, "min40" },
  {   3, "hr1" },
  {   4, "hr1min30" },
  {   5, "hr2" },
  {   6, "spare2" },
  {   7, "spare1" },
  { 0, NULL }
};


static int
dissect_rrc_T_loggingDuration_01(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     8, NULL, FALSE, 0, NULL);

  return offset;
}


static const value_string rrc_T_loggingInterval_vals[] = {
  {   0, "li128" },
  {   1, "li256" },
  {   2, "li512" },
  {   3, "li1024" },
  {   4, "li2048" },
  {   5, "li3072" },
  {   6, "li4096" },
  {   7, "li6144" },
  { 0, NULL }
};


static int
dissect_rrc_T_loggingInterval(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     8, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t TraceReference_sequence[] = {
  { &hf_rrc_plmn_Identity   , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PLMN_Identity },
  { &hf_rrc_traceId         , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_OCTET_STRING_SIZE_3 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_TraceReference(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_TraceReference, TraceReference_sequence);

  return offset;
}


static const per_sequence_t TraceRecordingSession_sequence[] = {
  { &hf_rrc_traceRecordingSession_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_OCTET_STRING_SIZE_2 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_TraceRecordingSession(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_TraceRecordingSession, TraceRecordingSession_sequence);

  return offset;
}



static int
dissect_rrc_OCTET_STRING_SIZE_1(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_octet_string(tvb, offset, actx, tree, hf_index,
                                       1, 1, FALSE, NULL);

  return offset;
}


static const per_sequence_t TCE_Id_sequence[] = {
  { &hf_rrc_tce_Id_01       , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_OCTET_STRING_SIZE_1 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_TCE_Id(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_TCE_Id, TCE_Id_sequence);

  return offset;
}


static const per_sequence_t SEQUENCE_SIZE_1_32_OF_CellIdentity_sequence_of[1] = {
  { &hf_rrc_cellIDList_item , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CellIdentity },
};

static int
dissect_rrc_SEQUENCE_SIZE_1_32_OF_CellIdentity(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_SEQUENCE_SIZE_1_32_OF_CellIdentity, SEQUENCE_SIZE_1_32_OF_CellIdentity_sequence_of,
                                                  1, 32, FALSE);

  return offset;
}


static const per_sequence_t LAI_sequence[] = {
  { &hf_rrc_plmn_Identity   , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PLMN_Identity },
  { &hf_rrc_lac             , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_16 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_LAI(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_LAI, LAI_sequence);

  return offset;
}


static const per_sequence_t SEQUENCE_SIZE_1_8_OF_LAI_sequence_of[1] = {
  { &hf_rrc_locationAreaList_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_LAI },
};

static int
dissect_rrc_SEQUENCE_SIZE_1_8_OF_LAI(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_SEQUENCE_SIZE_1_8_OF_LAI, SEQUENCE_SIZE_1_8_OF_LAI_sequence_of,
                                                  1, 8, FALSE);

  return offset;
}



static int
dissect_rrc_RoutingAreaCode(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_bit_string(tvb, offset, actx, tree, hf_index,
                                     8, 8, FALSE, NULL);

  return offset;
}


static const per_sequence_t RAI_sequence[] = {
  { &hf_rrc_lai             , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_LAI },
  { &hf_rrc_rac             , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RoutingAreaCode },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_RAI(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_RAI, RAI_sequence);

  return offset;
}


static const per_sequence_t SEQUENCE_SIZE_1_8_OF_RAI_sequence_of[1] = {
  { &hf_rrc_routingAreaList_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RAI },
};

static int
dissect_rrc_SEQUENCE_SIZE_1_8_OF_RAI(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_SEQUENCE_SIZE_1_8_OF_RAI, SEQUENCE_SIZE_1_8_OF_RAI_sequence_of,
                                                  1, 8, FALSE);

  return offset;
}


static const value_string rrc_T_areaConfiguration_vals[] = {
  {   0, "cellIDList" },
  {   1, "locationAreaList" },
  {   2, "routingAreaList" },
  { 0, NULL }
};

static const per_choice_t T_areaConfiguration_choice[] = {
  {   0, &hf_rrc_cellIDList      , ASN1_NO_EXTENSIONS     , dissect_rrc_SEQUENCE_SIZE_1_32_OF_CellIdentity },
  {   1, &hf_rrc_locationAreaList, ASN1_NO_EXTENSIONS     , dissect_rrc_SEQUENCE_SIZE_1_8_OF_LAI },
  {   2, &hf_rrc_routingAreaList , ASN1_NO_EXTENSIONS     , dissect_rrc_SEQUENCE_SIZE_1_8_OF_RAI },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_areaConfiguration(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_areaConfiguration, T_areaConfiguration_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t LoggedMeasurementsConfigurationInfo_sequence[] = {
  { &hf_rrc_absoluteTimeInfo, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_48 },
  { &hf_rrc_loggingDuration_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_loggingDuration_01 },
  { &hf_rrc_loggingInterval , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_loggingInterval },
  { &hf_rrc_traceReference  , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TraceReference },
  { &hf_rrc_traceRecordingSession, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TraceRecordingSession },
  { &hf_rrc_tce_Id          , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TCE_Id },
  { &hf_rrc_areaConfiguration, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_areaConfiguration },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_LoggedMeasurementsConfigurationInfo(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_LoggedMeasurementsConfigurationInfo, LoggedMeasurementsConfigurationInfo_sequence);

  return offset;
}


static const value_string rrc_T_loggingDuration_vals[] = {
  {   0, "min10" },
  {   1, "min30" },
  {   2, "hr1" },
  {   3, "spare2" },
  {   4, "spare1" },
  { 0, NULL }
};


static int
dissect_rrc_T_loggingDuration(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     5, NULL, FALSE, 0, NULL);

  return offset;
}



static int
dissect_rrc_INTEGER_M120_M25(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            -120, -25, NULL, FALSE);

  return offset;
}


static const per_sequence_t RSCPforANR_sequence[] = {
  { &hf_rrc_rSCP            , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_INTEGER_M120_M25 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_RSCPforANR(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_RSCPforANR, RSCPforANR_sequence);

  return offset;
}



static int
dissect_rrc_INTEGER_M24_0(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            -24, 0U, NULL, FALSE);

  return offset;
}


static const per_sequence_t Ec_N0forANR_sequence[] = {
  { &hf_rrc_ec_N0           , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_INTEGER_M24_0 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_Ec_N0forANR(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_Ec_N0forANR, Ec_N0forANR_sequence);

  return offset;
}


static const value_string rrc_LoggingAbsoluteThreshold_vals[] = {
  {   0, "rscpforANR" },
  {   1, "ec-N0forANR" },
  { 0, NULL }
};

static const per_choice_t LoggingAbsoluteThreshold_choice[] = {
  {   0, &hf_rrc_rscpforANR      , ASN1_NO_EXTENSIONS     , dissect_rrc_RSCPforANR },
  {   1, &hf_rrc_ec_N0forANR     , ASN1_NO_EXTENSIONS     , dissect_rrc_Ec_N0forANR },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_LoggingAbsoluteThreshold(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_LoggingAbsoluteThreshold, LoggingAbsoluteThreshold_choice,
                                 NULL);

  return offset;
}



static int
dissect_rrc_LoggingRelativeThreshold(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            -4, 16U, NULL, FALSE);

  return offset;
}


static const per_sequence_t IntraUTRAANR_sequence[] = {
  { &hf_rrc_loggingAbsoluteThreshold, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_LoggingAbsoluteThreshold },
  { &hf_rrc_loggingRelativeThreshold, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_LoggingRelativeThreshold },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_IntraUTRAANR(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_IntraUTRAANR, IntraUTRAANR_sequence);

  return offset;
}


static const value_string rrc_T_interRATANRforEUTRAIndicator_vals[] = {
  {   0, "true" },
  { 0, NULL }
};


static int
dissect_rrc_T_interRATANRforEUTRAIndicator(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}


static const value_string rrc_T_interRATANRforGSMIndicator_vals[] = {
  {   0, "true" },
  { 0, NULL }
};


static int
dissect_rrc_T_interRATANRforGSMIndicator(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t LoggedANRConfigurationInfo_sequence[] = {
  { &hf_rrc_loggingDuration , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_loggingDuration },
  { &hf_rrc_intraUTRAANR    , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_IntraUTRAANR },
  { &hf_rrc_interRATANRforEUTRAIndicator, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_interRATANRforEUTRAIndicator },
  { &hf_rrc_interRATANRforGSMIndicator, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_interRATANRforGSMIndicator },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_LoggedANRConfigurationInfo(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_LoggedANRConfigurationInfo, LoggedANRConfigurationInfo_sequence);

  return offset;
}


static const per_sequence_t LoggingMeasurementConfiguration_r10_IEs_sequence[] = {
  { &hf_rrc_loggedMeasurementsConfigurationInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_LoggedMeasurementsConfigurationInfo },
  { &hf_rrc_loggedANRConfigurationInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_LoggedANRConfigurationInfo },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_LoggingMeasurementConfiguration_r10_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_LoggingMeasurementConfiguration_r10_IEs, LoggingMeasurementConfiguration_r10_IEs_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions_189_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_nonCriticalExtensions_189(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_nonCriticalExtensions_189, T_nonCriticalExtensions_189_sequence);

  return offset;
}


static const per_sequence_t T_r10_11_sequence[] = {
  { &hf_rrc_loggingMeasurementConfiguration_r10_IEs, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_LoggingMeasurementConfiguration_r10_IEs },
  { &hf_rrc_loggingMeasurementConfiguration_r10_add_ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_BIT_STRING },
  { &hf_rrc_nonCriticalExtensions_189, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_nonCriticalExtensions_189 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_r10_11(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_r10_11, T_r10_11_sequence);

  return offset;
}


static const value_string rrc_T_loggingDuration_02_vals[] = {
  {   0, "min10" },
  {   1, "min20" },
  {   2, "min40" },
  {   3, "hr1" },
  {   4, "hr1min30" },
  {   5, "hr2" },
  {   6, "spare2" },
  {   7, "spare1" },
  { 0, NULL }
};


static int
dissect_rrc_T_loggingDuration_02(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     8, NULL, FALSE, 0, NULL);

  return offset;
}


static const value_string rrc_T_loggingInterval_01_vals[] = {
  {   0, "li128" },
  {   1, "li256" },
  {   2, "li512" },
  {   3, "li1024" },
  {   4, "li2048" },
  {   5, "li3072" },
  {   6, "li4096" },
  {   7, "li6144" },
  { 0, NULL }
};


static int
dissect_rrc_T_loggingInterval_01(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     8, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t SEQUENCE_SIZE_1_15_OF_PLMN_Identity_sequence_of[1] = {
  { &hf_rrc_plmnList_item   , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PLMN_Identity },
};

static int
dissect_rrc_SEQUENCE_SIZE_1_15_OF_PLMN_Identity(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_SEQUENCE_SIZE_1_15_OF_PLMN_Identity, SEQUENCE_SIZE_1_15_OF_PLMN_Identity_sequence_of,
                                                  1, 15, FALSE);

  return offset;
}


static const per_sequence_t CellIDListItem_sequence[] = {
  { &hf_rrc_cell_Identity   , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CellIdentity },
  { &hf_rrc_plmn_Identity   , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_PLMN_Identity },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_CellIDListItem(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_CellIDListItem, CellIDListItem_sequence);

  return offset;
}


static const per_sequence_t SEQUENCE_SIZE_1_32_OF_CellIDListItem_sequence_of[1] = {
  { &hf_rrc_cellIDList_item_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CellIDListItem },
};

static int
dissect_rrc_SEQUENCE_SIZE_1_32_OF_CellIDListItem(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_SEQUENCE_SIZE_1_32_OF_CellIDListItem, SEQUENCE_SIZE_1_32_OF_CellIDListItem_sequence_of,
                                                  1, 32, FALSE);

  return offset;
}


static const value_string rrc_T_areaConfiguration_01_vals[] = {
  {   0, "cellIDList" },
  {   1, "locationAreaList" },
  {   2, "routingAreaList" },
  { 0, NULL }
};

static const per_choice_t T_areaConfiguration_01_choice[] = {
  {   0, &hf_rrc_cellIDList_01   , ASN1_NO_EXTENSIONS     , dissect_rrc_SEQUENCE_SIZE_1_32_OF_CellIDListItem },
  {   1, &hf_rrc_locationAreaList, ASN1_NO_EXTENSIONS     , dissect_rrc_SEQUENCE_SIZE_1_8_OF_LAI },
  {   2, &hf_rrc_routingAreaList , ASN1_NO_EXTENSIONS     , dissect_rrc_SEQUENCE_SIZE_1_8_OF_RAI },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_areaConfiguration_01(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_areaConfiguration_01, T_areaConfiguration_01_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t LoggedMeasurementsConfigurationInfo_r11_sequence[] = {
  { &hf_rrc_absoluteTimeInfo, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_48 },
  { &hf_rrc_loggingDuration_02, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_loggingDuration_02 },
  { &hf_rrc_loggingInterval_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_loggingInterval_01 },
  { &hf_rrc_traceReference  , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TraceReference },
  { &hf_rrc_traceRecordingSession, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TraceRecordingSession },
  { &hf_rrc_tce_Id          , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TCE_Id },
  { &hf_rrc_plmnList        , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_SEQUENCE_SIZE_1_15_OF_PLMN_Identity },
  { &hf_rrc_areaConfiguration_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_areaConfiguration_01 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_LoggedMeasurementsConfigurationInfo_r11(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_LoggedMeasurementsConfigurationInfo_r11, LoggedMeasurementsConfigurationInfo_r11_sequence);

  return offset;
}


static const per_sequence_t LoggingMeasurementConfiguration_r11_IEs_sequence[] = {
  { &hf_rrc_loggedMeasurementsConfigurationInfo_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_LoggedMeasurementsConfigurationInfo_r11 },
  { &hf_rrc_loggedANRConfigurationInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_LoggedANRConfigurationInfo },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_LoggingMeasurementConfiguration_r11_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_LoggingMeasurementConfiguration_r11_IEs, LoggingMeasurementConfiguration_r11_IEs_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions_190_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_nonCriticalExtensions_190(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_nonCriticalExtensions_190, T_nonCriticalExtensions_190_sequence);

  return offset;
}


static const per_sequence_t T_r11_12_sequence[] = {
  { &hf_rrc_loggingMeasurementConfiguration_r11_IEs, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_LoggingMeasurementConfiguration_r11_IEs },
  { &hf_rrc_loggingMeasurementConfiguration_r11_add_ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_BIT_STRING },
  { &hf_rrc_nonCriticalExtensions_190, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_nonCriticalExtensions_190 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_r11_12(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_r11_12, T_r11_12_sequence);

  return offset;
}


static const per_sequence_t T_criticalExtensions_135_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_criticalExtensions_135(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_criticalExtensions_135, T_criticalExtensions_135_sequence);

  return offset;
}


static const value_string rrc_T_criticalExtensions_134_vals[] = {
  {   0, "r11" },
  {   1, "criticalExtensions" },
  { 0, NULL }
};

static const per_choice_t T_criticalExtensions_134_choice[] = {
  {   0, &hf_rrc_r11_12          , ASN1_NO_EXTENSIONS     , dissect_rrc_T_r11_12 },
  {   1, &hf_rrc_criticalExtensions_136, ASN1_NO_EXTENSIONS     , dissect_rrc_T_criticalExtensions_135 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_criticalExtensions_134(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_criticalExtensions_134, T_criticalExtensions_134_choice,
                                 NULL);

  return offset;
}


static const value_string rrc_T_criticalExtensions_133_vals[] = {
  {   0, "r10" },
  {   1, "criticalExtensions" },
  { 0, NULL }
};

static const per_choice_t T_criticalExtensions_133_choice[] = {
  {   0, &hf_rrc_r10_11          , ASN1_NO_EXTENSIONS     , dissect_rrc_T_r10_11 },
  {   1, &hf_rrc_criticalExtensions_135, ASN1_NO_EXTENSIONS     , dissect_rrc_T_criticalExtensions_134 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_criticalExtensions_133(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_criticalExtensions_133, T_criticalExtensions_133_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t LoggingMeasurementConfiguration_sequence[] = {
  { &hf_rrc_rrc_TransactionIdentifier, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RRC_TransactionIdentifier },
  { &hf_rrc_criticalExtensions_134, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_criticalExtensions_133 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_LoggingMeasurementConfiguration(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
#line 245 "../../asn1/rrc/rrc.cnf"
  col_append_str(actx->pinfo->cinfo, COL_INFO, "LoggingMeasurementConfiguration");
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_LoggingMeasurementConfiguration, LoggingMeasurementConfiguration_sequence);




  return offset;
}


static const value_string rrc_DL_DCCH_MessageType_vals[] = {
  {   0, "activeSetUpdate" },
  {   1, "assistanceDataDelivery" },
  {   2, "cellChangeOrderFromUTRAN" },
  {   3, "cellUpdateConfirm" },
  {   4, "counterCheck" },
  {   5, "downlinkDirectTransfer" },
  {   6, "handoverFromUTRANCommand-GSM" },
  {   7, "handoverFromUTRANCommand-CDMA2000" },
  {   8, "measurementControl" },
  {   9, "pagingType2" },
  {  10, "physicalChannelReconfiguration" },
  {  11, "physicalSharedChannelAllocation" },
  {  12, "radioBearerReconfiguration" },
  {  13, "radioBearerRelease" },
  {  14, "radioBearerSetup" },
  {  15, "rrcConnectionRelease" },
  {  16, "securityModeCommand" },
  {  17, "signallingConnectionRelease" },
  {  18, "transportChannelReconfiguration" },
  {  19, "transportFormatCombinationControl" },
  {  20, "ueCapabilityEnquiry" },
  {  21, "ueCapabilityInformationConfirm" },
  {  22, "uplinkPhysicalChannelControl" },
  {  23, "uraUpdateConfirm" },
  {  24, "utranMobilityInformation" },
  {  25, "handoverFromUTRANCommand-GERANIu" },
  {  26, "mbmsModifiedServicesInformation" },
  {  27, "etwsPrimaryNotificationWithSecurity" },
  {  28, "handoverFromUTRANCommand-EUTRA" },
  {  29, "ueInformationRequest" },
  {  30, "loggingMeasurementConfiguration" },
  {  31, "spare1" },
  { 0, NULL }
};

static value_string_ext rrc_DL_DCCH_MessageType_vals_ext = VALUE_STRING_EXT_INIT(rrc_DL_DCCH_MessageType_vals);

static const per_choice_t DL_DCCH_MessageType_choice[] = {
  {   0, &hf_rrc_activeSetUpdate , ASN1_NO_EXTENSIONS     , dissect_rrc_ActiveSetUpdate },
  {   1, &hf_rrc_assistanceDataDelivery, ASN1_NO_EXTENSIONS     , dissect_rrc_AssistanceDataDelivery },
  {   2, &hf_rrc_cellChangeOrderFromUTRAN, ASN1_NO_EXTENSIONS     , dissect_rrc_CellChangeOrderFromUTRAN },
  {   3, &hf_rrc_cellUpdateConfirm, ASN1_NO_EXTENSIONS     , dissect_rrc_CellUpdateConfirm },
  {   4, &hf_rrc_counterCheck    , ASN1_NO_EXTENSIONS     , dissect_rrc_CounterCheck },
  {   5, &hf_rrc_downlinkDirectTransfer, ASN1_NO_EXTENSIONS     , dissect_rrc_DownlinkDirectTransfer },
  {   6, &hf_rrc_handoverFromUTRANCommand_GSM, ASN1_NO_EXTENSIONS     , dissect_rrc_HandoverFromUTRANCommand_GSM },
  {   7, &hf_rrc_handoverFromUTRANCommand_CDMA2000, ASN1_NO_EXTENSIONS     , dissect_rrc_HandoverFromUTRANCommand_CDMA2000 },
  {   8, &hf_rrc_measurementControl, ASN1_NO_EXTENSIONS     , dissect_rrc_MeasurementControl },
  {   9, &hf_rrc_pagingType2     , ASN1_NO_EXTENSIONS     , dissect_rrc_PagingType2 },
  {  10, &hf_rrc_physicalChannelReconfiguration, ASN1_NO_EXTENSIONS     , dissect_rrc_PhysicalChannelReconfiguration },
  {  11, &hf_rrc_physicalSharedChannelAllocation, ASN1_NO_EXTENSIONS     , dissect_rrc_PhysicalSharedChannelAllocation },
  {  12, &hf_rrc_radioBearerReconfiguration, ASN1_NO_EXTENSIONS     , dissect_rrc_RadioBearerReconfiguration },
  {  13, &hf_rrc_radioBearerRelease, ASN1_NO_EXTENSIONS     , dissect_rrc_RadioBearerRelease },
  {  14, &hf_rrc_radioBearerSetup, ASN1_NO_EXTENSIONS     , dissect_rrc_RadioBearerSetup },
  {  15, &hf_rrc_rrcConnectionRelease, ASN1_NO_EXTENSIONS     , dissect_rrc_RRCConnectionRelease },
  {  16, &hf_rrc_securityModeCommand, ASN1_NO_EXTENSIONS     , dissect_rrc_SecurityModeCommand },
  {  17, &hf_rrc_signallingConnectionRelease, ASN1_NO_EXTENSIONS     , dissect_rrc_SignallingConnectionRelease },
  {  18, &hf_rrc_transportChannelReconfiguration, ASN1_NO_EXTENSIONS     , dissect_rrc_TransportChannelReconfiguration },
  {  19, &hf_rrc_transportFormatCombinationControl, ASN1_NO_EXTENSIONS     , dissect_rrc_TransportFormatCombinationControl },
  {  20, &hf_rrc_ueCapabilityEnquiry, ASN1_NO_EXTENSIONS     , dissect_rrc_UECapabilityEnquiry },
  {  21, &hf_rrc_ueCapabilityInformationConfirm, ASN1_NO_EXTENSIONS     , dissect_rrc_UECapabilityInformationConfirm },
  {  22, &hf_rrc_uplinkPhysicalChannelControl, ASN1_NO_EXTENSIONS     , dissect_rrc_UplinkPhysicalChannelControl },
  {  23, &hf_rrc_uraUpdateConfirm, ASN1_NO_EXTENSIONS     , dissect_rrc_URAUpdateConfirm },
  {  24, &hf_rrc_utranMobilityInformation, ASN1_NO_EXTENSIONS     , dissect_rrc_UTRANMobilityInformation },
  {  25, &hf_rrc_handoverFromUTRANCommand_GERANIu, ASN1_NO_EXTENSIONS     , dissect_rrc_HandoverFromUTRANCommand_GERANIu },
  {  26, &hf_rrc_mbmsModifiedServicesInformation, ASN1_NO_EXTENSIONS     , dissect_rrc_MBMSModifiedServicesInformation },
  {  27, &hf_rrc_etwsPrimaryNotificationWithSecurity, ASN1_NO_EXTENSIONS     , dissect_rrc_ETWSPrimaryNotificationWithSecurity },
  {  28, &hf_rrc_handoverFromUTRANCommand_EUTRA, ASN1_NO_EXTENSIONS     , dissect_rrc_HandoverFromUTRANCommand_EUTRA },
  {  29, &hf_rrc_ueInformationRequest, ASN1_NO_EXTENSIONS     , dissect_rrc_UEInformationRequest },
  {  30, &hf_rrc_loggingMeasurementConfiguration, ASN1_NO_EXTENSIONS     , dissect_rrc_LoggingMeasurementConfiguration },
  {  31, &hf_rrc_spare1          , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_DL_DCCH_MessageType(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
#line 880 "../../asn1/rrc/rrc.cnf"
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_DL_DCCH_MessageType, DL_DCCH_MessageType_choice,
                                 &msg_type);




  return offset;
}


static const per_sequence_t DL_DCCH_Message_sequence[] = {
  { &hf_rrc_integrityCheckInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_IntegrityCheckInfo },
  { &hf_rrc_message         , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DL_DCCH_MessageType },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_DL_DCCH_Message(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
#line 883 "../../asn1/rrc/rrc.cnf"
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_DL_DCCH_Message, DL_DCCH_Message_sequence);




  return offset;
}



static int
dissect_rrc_START_Value(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
#line 793 "../../asn1/rrc/rrc.cnf"
  tvbuff_t * start_val;
  fp_info       *fpinf;
  rrc_ciphering_info * c_inf;
  int i;
  guint32 * start;

  offset = dissect_per_bit_string(tvb, offset, actx, tree, hf_index,
                                     20, 20, FALSE, &start_val);


  /*We base this map on communication context from fp*/
    fpinf = p_get_proto_data(actx->pinfo->fd, proto_fp);

  /*If no info found, skip all this*/
  if(fpinf == NULL){
      return offset;
  }
  /*Retrieve the start value for the two ciphering domains*/
  switch(rrc_nas_sys_info_gsm_map_type){
    case RRC_NAS_SYS_INFO_CS:
      /*
      g_warning("Not implemented");
      */
      break;
    case RRC_NAS_SYS_INFO_PS:

      /*Find the entry for the communication context (taken from FP)*/
      if( (c_inf = g_tree_lookup(rrc_ciph_inf, GINT_TO_POINTER(fpinf->com_context_id))) == NULL ){
        c_inf = g_malloc0(sizeof(rrc_ciphering_info));

        /*Initiate tree with START_PS values.*/
        if(!c_inf->start_ps)
          c_inf->start_ps = g_tree_new_full(rrc_key_cmp,
                                            NULL,rrc_free_key,rrc_free_value);

          /*Clear and initialize seq_no matrix*/
          for(i = 0; i< 31; i++){
            c_inf->seq_no[i][0] = -1;
            c_inf->seq_no[i][1] = -1;
          }
          g_tree_insert(rrc_ciph_inf, GINT_TO_POINTER(fpinf->com_context_id), c_inf);
        }

        /*Retrieve and store the value*/
        start = g_malloc(sizeof(guint32));
                *start = tvb_get_bits32(start_val,0,20,ENC_BIG_ENDIAN);
                if(c_inf && c_inf->start_ps)
                /*Insert the value based on current frame num since this might vary over time*/
                g_tree_insert(c_inf->start_ps, GUINT_TO_POINTER(actx->pinfo->fd->num), start);

            break;
            default:
            break;
        }
        /*is this dangerous?*/
        rrc_nas_sys_info_gsm_map_type = RRC_NAS_SYS_INFO_CN_COMMON;



  return offset;
}


static const per_sequence_t STARTSingle_sequence[] = {
  { &hf_rrc_cn_DomainIdentity, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CN_DomainIdentity },
  { &hf_rrc_start_Value     , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_START_Value },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_STARTSingle(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_STARTSingle, STARTSingle_sequence);

  return offset;
}


static const per_sequence_t STARTList_sequence_of[1] = {
  { &hf_rrc_STARTList_item  , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_STARTSingle },
};

static int
dissect_rrc_STARTList(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_STARTList, STARTList_sequence_of,
                                                  1, maxCNdomains, FALSE);

  return offset;
}


static const per_sequence_t UL_CounterSynchronisationInfo_sequence[] = {
  { &hf_rrc_rB_WithPDCP_InfoList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RB_WithPDCP_InfoList },
  { &hf_rrc_startList       , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_STARTList },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UL_CounterSynchronisationInfo(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UL_CounterSynchronisationInfo, UL_CounterSynchronisationInfo_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions_07_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_nonCriticalExtensions_07(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_nonCriticalExtensions_07, T_nonCriticalExtensions_07_sequence);

  return offset;
}


static const per_sequence_t T_laterNonCriticalExtensions_01_sequence[] = {
  { &hf_rrc_activeSetUpdateComplete_r3_add_ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_BIT_STRING },
  { &hf_rrc_nonCriticalExtensions_07, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_nonCriticalExtensions_07 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_laterNonCriticalExtensions_01(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_laterNonCriticalExtensions_01, T_laterNonCriticalExtensions_01_sequence);

  return offset;
}


static const per_sequence_t ActiveSetUpdateComplete_sequence[] = {
  { &hf_rrc_rrc_TransactionIdentifier, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RRC_TransactionIdentifier },
  { &hf_rrc_dummy_03        , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_IntegrityProtActivationInfo },
  { &hf_rrc_dummy2_01       , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RB_ActivationTimeInfoList },
  { &hf_rrc_dummy3_01       , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_CounterSynchronisationInfo },
  { &hf_rrc_laterNonCriticalExtensions_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_laterNonCriticalExtensions_01 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_ActiveSetUpdateComplete(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
#line 169 "../../asn1/rrc/rrc.cnf"
  col_append_str(actx->pinfo->cinfo, COL_INFO, "ActiveSetUpdateComplete");
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_ActiveSetUpdateComplete, ActiveSetUpdateComplete_sequence);




  return offset;
}


static const value_string rrc_ProtocolErrorCause_vals[] = {
  {   0, "asn1-ViolationOrEncodingError" },
  {   1, "messageTypeNonexistent" },
  {   2, "messageNotCompatibleWithReceiverState" },
  {   3, "ie-ValueNotComprehended" },
  {   4, "informationElementMissing" },
  {   5, "messageExtensionNotComprehended" },
  {   6, "spare2" },
  {   7, "spare1" },
  { 0, NULL }
};


static int
dissect_rrc_ProtocolErrorCause(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     8, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t T_type1_07_sequence[] = {
  { &hf_rrc_protocolErrorCause, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_ProtocolErrorCause },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_type1_07(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_type1_07, T_type1_07_sequence);

  return offset;
}


static const value_string rrc_T_diagnosticsType_01_vals[] = {
  {   0, "type1" },
  {   1, "spare" },
  { 0, NULL }
};

static const per_choice_t T_diagnosticsType_01_choice[] = {
  {   0, &hf_rrc_type1_08        , ASN1_NO_EXTENSIONS     , dissect_rrc_T_type1_07 },
  {   1, &hf_rrc_spare           , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_diagnosticsType_01(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_diagnosticsType_01, T_diagnosticsType_01_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t ProtocolErrorInformation_sequence[] = {
  { &hf_rrc_diagnosticsType_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_diagnosticsType_01 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_ProtocolErrorInformation(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_ProtocolErrorInformation, ProtocolErrorInformation_sequence);

  return offset;
}


static const value_string rrc_FailureCauseWithProtErr_vals[] = {
  {   0, "configurationUnsupported" },
  {   1, "physicalChannelFailure" },
  {   2, "incompatibleSimultaneousReconfiguration" },
  {   3, "compressedModeRuntimeError" },
  {   4, "protocolError" },
  {   5, "cellUpdateOccurred" },
  {   6, "invalidConfiguration" },
  {   7, "configurationIncomplete" },
  {   8, "unsupportedMeasurement" },
  {   9, "mbmsSessionAlreadyReceivedCorrectly" },
  {  10, "lowerPriorityMBMSService" },
  {  11, "spare5" },
  {  12, "spare4" },
  {  13, "spare3" },
  {  14, "spare2" },
  {  15, "spare1" },
  { 0, NULL }
};

static const per_choice_t FailureCauseWithProtErr_choice[] = {
  {   0, &hf_rrc_configurationUnsupported, ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   1, &hf_rrc_physicalChannelFailure, ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   2, &hf_rrc_incompatibleSimultaneousReconfiguration, ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   3, &hf_rrc_compressedModeRuntimeError, ASN1_NO_EXTENSIONS     , dissect_rrc_TGPSI },
  {   4, &hf_rrc_protocolError   , ASN1_NO_EXTENSIONS     , dissect_rrc_ProtocolErrorInformation },
  {   5, &hf_rrc_cellUpdateOccurred, ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   6, &hf_rrc_invalidConfiguration, ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   7, &hf_rrc_configurationIncomplete, ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   8, &hf_rrc_unsupportedMeasurement, ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   9, &hf_rrc_mbmsSessionAlreadyReceivedCorrectly, ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {  10, &hf_rrc_lowerPriorityMBMSService, ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {  11, &hf_rrc_spare5          , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {  12, &hf_rrc_spare4          , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {  13, &hf_rrc_spare3          , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {  14, &hf_rrc_spare2          , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {  15, &hf_rrc_spare1          , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_FailureCauseWithProtErr(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_FailureCauseWithProtErr, FailureCauseWithProtErr_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions_08_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_nonCriticalExtensions_08(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_nonCriticalExtensions_08, T_nonCriticalExtensions_08_sequence);

  return offset;
}


static const per_sequence_t T_laterNonCriticalExtensions_02_sequence[] = {
  { &hf_rrc_activeSetUpdateFailure_r3_add_ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_BIT_STRING },
  { &hf_rrc_nonCriticalExtensions_08, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_nonCriticalExtensions_08 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_laterNonCriticalExtensions_02(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_laterNonCriticalExtensions_02, T_laterNonCriticalExtensions_02_sequence);

  return offset;
}


static const per_sequence_t ActiveSetUpdateFailure_sequence[] = {
  { &hf_rrc_rrc_TransactionIdentifier, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RRC_TransactionIdentifier },
  { &hf_rrc_failureCause    , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_FailureCauseWithProtErr },
  { &hf_rrc_laterNonCriticalExtensions_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_laterNonCriticalExtensions_02 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_ActiveSetUpdateFailure(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
#line 173 "../../asn1/rrc/rrc.cnf"
  col_append_str(actx->pinfo->cinfo, COL_INFO, "ActiveSetUpdateFailure");
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_ActiveSetUpdateFailure, ActiveSetUpdateFailure_sequence);




  return offset;
}


static const value_string rrc_InterRAT_ChangeFailureCause_vals[] = {
  {   0, "configurationUnacceptable" },
  {   1, "physicalChannelFailure" },
  {   2, "protocolError" },
  {   3, "unspecified" },
  {   4, "spare4" },
  {   5, "spare3" },
  {   6, "spare2" },
  {   7, "spare1" },
  { 0, NULL }
};

static const per_choice_t InterRAT_ChangeFailureCause_choice[] = {
  {   0, &hf_rrc_configurationUnacceptable, ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   1, &hf_rrc_physicalChannelFailure, ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   2, &hf_rrc_protocolError   , ASN1_NO_EXTENSIONS     , dissect_rrc_ProtocolErrorInformation },
  {   3, &hf_rrc_unspecified     , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   4, &hf_rrc_spare4          , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   5, &hf_rrc_spare3          , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   6, &hf_rrc_spare2          , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   7, &hf_rrc_spare1          , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_InterRAT_ChangeFailureCause(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_InterRAT_ChangeFailureCause, InterRAT_ChangeFailureCause_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t CellChangeOrderFromUTRANFailure_r3_IEs_sequence[] = {
  { &hf_rrc_rrc_TransactionIdentifier, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RRC_TransactionIdentifier },
  { &hf_rrc_dummy_01        , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_IntegrityProtectionModeInfo },
  { &hf_rrc_interRAT_ChangeFailureCause, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_InterRAT_ChangeFailureCause },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_CellChangeOrderFromUTRANFailure_r3_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_CellChangeOrderFromUTRANFailure_r3_IEs, CellChangeOrderFromUTRANFailure_r3_IEs_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions_11_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_nonCriticalExtensions_11(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_nonCriticalExtensions_11, T_nonCriticalExtensions_11_sequence);

  return offset;
}


static const per_sequence_t T_laterNonCriticalExtensions_05_sequence[] = {
  { &hf_rrc_cellChangeOrderFromUTRANFailure_r3_add_ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_BIT_STRING },
  { &hf_rrc_nonCriticalExtensions_11, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_nonCriticalExtensions_11 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_laterNonCriticalExtensions_05(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_laterNonCriticalExtensions_05, T_laterNonCriticalExtensions_05_sequence);

  return offset;
}


static const per_sequence_t T_r3_03_sequence[] = {
  { &hf_rrc_cellChangeOrderFromUTRANFailure_r3, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CellChangeOrderFromUTRANFailure_r3_IEs },
  { &hf_rrc_laterNonCriticalExtensions_05, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_laterNonCriticalExtensions_05 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_r3_03(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_r3_03, T_r3_03_sequence);

  return offset;
}


static const per_sequence_t T_criticalExtensions_09_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_criticalExtensions_09(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_criticalExtensions_09, T_criticalExtensions_09_sequence);

  return offset;
}


static const per_sequence_t T_dummy_sequence[] = {
  { &hf_rrc_rrc_TransactionIdentifier, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RRC_TransactionIdentifier },
  { &hf_rrc_criticalExtensions_09, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_criticalExtensions_09 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_dummy(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_dummy, T_dummy_sequence);

  return offset;
}


static const value_string rrc_CellChangeOrderFromUTRANFailure_vals[] = {
  {   0, "r3" },
  {   1, "dummy" },
  { 0, NULL }
};

static const per_choice_t CellChangeOrderFromUTRANFailure_choice[] = {
  {   0, &hf_rrc_r3_03           , ASN1_NO_EXTENSIONS     , dissect_rrc_T_r3_03 },
  {   1, &hf_rrc_dummy_04        , ASN1_NO_EXTENSIONS     , dissect_rrc_T_dummy },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_CellChangeOrderFromUTRANFailure(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
#line 185 "../../asn1/rrc/rrc.cnf"
  col_append_str(actx->pinfo->cinfo, COL_INFO, "CellChangeOrderFromUTRANFailure");
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_CellChangeOrderFromUTRANFailure, CellChangeOrderFromUTRANFailure_choice,
                                 NULL);




  return offset;
}



static int
dissect_rrc_COUNT_C(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            0U, 4294967295U, NULL, FALSE);

  return offset;
}


static const per_sequence_t RB_COUNT_C_Information_sequence[] = {
  { &hf_rrc_rb_Identity     , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RB_Identity },
  { &hf_rrc_count_C_UL      , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_COUNT_C },
  { &hf_rrc_count_C_DL      , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_COUNT_C },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_RB_COUNT_C_Information(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_RB_COUNT_C_Information, RB_COUNT_C_Information_sequence);

  return offset;
}


static const per_sequence_t RB_COUNT_C_InformationList_sequence_of[1] = {
  { &hf_rrc_RB_COUNT_C_InformationList_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RB_COUNT_C_Information },
};

static int
dissect_rrc_RB_COUNT_C_InformationList(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_RB_COUNT_C_InformationList, RB_COUNT_C_InformationList_sequence_of,
                                                  1, maxRBallRABs, FALSE);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions_34_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_nonCriticalExtensions_34(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_nonCriticalExtensions_34, T_nonCriticalExtensions_34_sequence);

  return offset;
}


static const per_sequence_t T_laterNonCriticalExtensions_10_sequence[] = {
  { &hf_rrc_counterCheckResponse_r3_add_ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_BIT_STRING },
  { &hf_rrc_nonCriticalExtensions_34, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_nonCriticalExtensions_34 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_laterNonCriticalExtensions_10(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_laterNonCriticalExtensions_10, T_laterNonCriticalExtensions_10_sequence);

  return offset;
}


static const per_sequence_t CounterCheckResponse_sequence[] = {
  { &hf_rrc_rrc_TransactionIdentifier, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RRC_TransactionIdentifier },
  { &hf_rrc_rb_COUNT_C_InformationList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RB_COUNT_C_InformationList },
  { &hf_rrc_laterNonCriticalExtensions_10, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_laterNonCriticalExtensions_10 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_CounterCheckResponse(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
#line 205 "../../asn1/rrc/rrc.cnf"
  col_append_str(actx->pinfo->cinfo, COL_INFO, "CounterCheckResponse");
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_CounterCheckResponse, CounterCheckResponse_sequence);




  return offset;
}


static const value_string rrc_T_loggedMeasAvailable_01_vals[] = {
  {   0, "true" },
  { 0, NULL }
};


static int
dissect_rrc_T_loggedMeasAvailable_01(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t HandoverToUTRANComplete_va40ext_sequence[] = {
  { &hf_rrc_loggedMeasAvailable_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_loggedMeasAvailable_01 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_HandoverToUTRANComplete_va40ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_HandoverToUTRANComplete_va40ext, HandoverToUTRANComplete_va40ext_sequence);

  return offset;
}


static const value_string rrc_T_loggedConnectionEstablishmentFailureInfoAvailable_vals[] = {
  {   0, "true" },
  { 0, NULL }
};


static int
dissect_rrc_T_loggedConnectionEstablishmentFailureInfoAvailable(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t HandoverToUTRANComplete_vbxyext_IEs_sequence[] = {
  { &hf_rrc_loggedConnectionEstablishmentFailureInfoAvailable, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_loggedConnectionEstablishmentFailureInfoAvailable },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_HandoverToUTRANComplete_vbxyext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_HandoverToUTRANComplete_vbxyext_IEs, HandoverToUTRANComplete_vbxyext_IEs_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions_46_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_nonCriticalExtensions_46(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_nonCriticalExtensions_46, T_nonCriticalExtensions_46_sequence);

  return offset;
}


static const per_sequence_t T_vbxyNonCriticalExtensions_04_sequence[] = {
  { &hf_rrc_handoverToUTRANComplete_vbxyext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_HandoverToUTRANComplete_vbxyext_IEs },
  { &hf_rrc_nonCriticalExtensions_46, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_nonCriticalExtensions_46 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_vbxyNonCriticalExtensions_04(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_vbxyNonCriticalExtensions_04, T_vbxyNonCriticalExtensions_04_sequence);

  return offset;
}


static const per_sequence_t T_va40NonCriticalExtensions_01_sequence[] = {
  { &hf_rrc_handoverToUTRANComplete_va40ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_HandoverToUTRANComplete_va40ext },
  { &hf_rrc_vbxyNonCriticalExtensions_04, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_vbxyNonCriticalExtensions_04 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_va40NonCriticalExtensions_01(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_va40NonCriticalExtensions_01, T_va40NonCriticalExtensions_01_sequence);

  return offset;
}


static const per_sequence_t T_laterNonCriticalExtensions_12_sequence[] = {
  { &hf_rrc_handoverToUTRANComplete_r3_add_ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_BIT_STRING },
  { &hf_rrc_va40NonCriticalExtensions_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_va40NonCriticalExtensions_01 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_laterNonCriticalExtensions_12(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_laterNonCriticalExtensions_12, T_laterNonCriticalExtensions_12_sequence);

  return offset;
}


static const per_sequence_t HandoverToUTRANComplete_sequence[] = {
  { &hf_rrc_startList       , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_STARTList },
  { &hf_rrc_count_C_ActivationTime, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_ActivationTime },
  { &hf_rrc_laterNonCriticalExtensions_12, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_laterNonCriticalExtensions_12 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_HandoverToUTRANComplete(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
#line 217 "../../asn1/rrc/rrc.cnf"
  col_append_str(actx->pinfo->cinfo, COL_INFO, "HandoverToUTRANComplete");
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_HandoverToUTRANComplete, HandoverToUTRANComplete_sequence);




  return offset;
}



static int
dissect_rrc_RoutingParameter(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_bit_string(tvb, offset, actx, tree, hf_index,
                                     10, 10, FALSE, NULL);

  return offset;
}


static const per_sequence_t T_localPTMSI_sequence[] = {
  { &hf_rrc_routingparameter, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RoutingParameter },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_localPTMSI(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_localPTMSI, T_localPTMSI_sequence);

  return offset;
}


static const per_sequence_t T_tMSIofsamePLMN_sequence[] = {
  { &hf_rrc_routingparameter, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RoutingParameter },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_tMSIofsamePLMN(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_tMSIofsamePLMN, T_tMSIofsamePLMN_sequence);

  return offset;
}


static const per_sequence_t T_tMSIofdifferentPLMN_sequence[] = {
  { &hf_rrc_routingparameter, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RoutingParameter },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_tMSIofdifferentPLMN(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_tMSIofdifferentPLMN, T_tMSIofdifferentPLMN_sequence);

  return offset;
}


static const per_sequence_t T_iMSIresponsetopaging_sequence[] = {
  { &hf_rrc_routingparameter, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RoutingParameter },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_iMSIresponsetopaging(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_iMSIresponsetopaging, T_iMSIresponsetopaging_sequence);

  return offset;
}


static const per_sequence_t T_iMSIcauseUEinitiatedEvent_sequence[] = {
  { &hf_rrc_routingparameter, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RoutingParameter },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_iMSIcauseUEinitiatedEvent(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_iMSIcauseUEinitiatedEvent, T_iMSIcauseUEinitiatedEvent_sequence);

  return offset;
}


static const per_sequence_t T_iMEI_sequence[] = {
  { &hf_rrc_routingparameter, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RoutingParameter },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_iMEI(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_iMEI, T_iMEI_sequence);

  return offset;
}


static const per_sequence_t T_spare2_sequence[] = {
  { &hf_rrc_routingparameter, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RoutingParameter },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_spare2(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_spare2, T_spare2_sequence);

  return offset;
}


static const per_sequence_t T_spare1_sequence[] = {
  { &hf_rrc_routingparameter, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RoutingParameter },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_spare1(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_spare1, T_spare1_sequence);

  return offset;
}


static const value_string rrc_T_routingbasis_vals[] = {
  {   0, "localPTMSI" },
  {   1, "tMSIofsamePLMN" },
  {   2, "tMSIofdifferentPLMN" },
  {   3, "iMSIresponsetopaging" },
  {   4, "iMSIcauseUEinitiatedEvent" },
  {   5, "iMEI" },
  {   6, "spare2" },
  {   7, "spare1" },
  { 0, NULL }
};

static const per_choice_t T_routingbasis_choice[] = {
  {   0, &hf_rrc_localPTMSI      , ASN1_NO_EXTENSIONS     , dissect_rrc_T_localPTMSI },
  {   1, &hf_rrc_tMSIofsamePLMN  , ASN1_NO_EXTENSIONS     , dissect_rrc_T_tMSIofsamePLMN },
  {   2, &hf_rrc_tMSIofdifferentPLMN, ASN1_NO_EXTENSIONS     , dissect_rrc_T_tMSIofdifferentPLMN },
  {   3, &hf_rrc_iMSIresponsetopaging, ASN1_NO_EXTENSIONS     , dissect_rrc_T_iMSIresponsetopaging },
  {   4, &hf_rrc_iMSIcauseUEinitiatedEvent, ASN1_NO_EXTENSIONS     , dissect_rrc_T_iMSIcauseUEinitiatedEvent },
  {   5, &hf_rrc_iMEI            , ASN1_NO_EXTENSIONS     , dissect_rrc_T_iMEI },
  {   6, &hf_rrc_spare2_01       , ASN1_NO_EXTENSIONS     , dissect_rrc_T_spare2 },
  {   7, &hf_rrc_spare1_01       , ASN1_NO_EXTENSIONS     , dissect_rrc_T_spare1 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_routingbasis(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_routingbasis, T_routingbasis_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t Gsm_map_IDNNS_sequence[] = {
  { &hf_rrc_routingbasis    , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_routingbasis },
  { &hf_rrc_dummy_28        , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_Gsm_map_IDNNS(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_Gsm_map_IDNNS, Gsm_map_IDNNS_sequence);

  return offset;
}



static int
dissect_rrc_Ansi_41_IDNNS(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_bit_string(tvb, offset, actx, tree, hf_index,
                                     14, 14, FALSE, NULL);

  return offset;
}


static const value_string rrc_T_cn_Type_01_vals[] = {
  {   0, "gsm-Map-IDNNS" },
  {   1, "ansi-41-IDNNS" },
  { 0, NULL }
};

static const per_choice_t T_cn_Type_01_choice[] = {
  {   0, &hf_rrc_gsm_Map_IDNNS   , ASN1_NO_EXTENSIONS     , dissect_rrc_Gsm_map_IDNNS },
  {   1, &hf_rrc_ansi_41_IDNNS   , ASN1_NO_EXTENSIONS     , dissect_rrc_Ansi_41_IDNNS },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_cn_Type_01(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_cn_Type_01, T_cn_Type_01_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t T_release99_sequence[] = {
  { &hf_rrc_cn_Type_01      , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_cn_Type_01 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_release99(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_release99, T_release99_sequence);

  return offset;
}


static const per_sequence_t T_later_sequence[] = {
  { &hf_rrc_futurecoding    , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_15 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_later(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_later, T_later_sequence);

  return offset;
}


static const value_string rrc_T_version_vals[] = {
  {   0, "release99" },
  {   1, "later" },
  { 0, NULL }
};

static const per_choice_t T_version_choice[] = {
  {   0, &hf_rrc_release99       , ASN1_NO_EXTENSIONS     , dissect_rrc_T_release99 },
  {   1, &hf_rrc_later           , ASN1_NO_EXTENSIONS     , dissect_rrc_T_later },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_version(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_version, T_version_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t IntraDomainNasNodeSelector_sequence[] = {
  { &hf_rrc_version         , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_version },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_IntraDomainNasNodeSelector(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_IntraDomainNasNodeSelector, IntraDomainNasNodeSelector_sequence);

  return offset;
}



static int
dissect_rrc_CPICH_Ec_N0(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            0U, 63U, NULL, FALSE);

  return offset;
}



static int
dissect_rrc_CPICH_RSCP(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            0U, 127U, NULL, FALSE);

  return offset;
}



static int
dissect_rrc_Pathloss(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            46U, 173U, NULL, FALSE);

  return offset;
}


static const value_string rrc_T_measurementQuantity_01_vals[] = {
  {   0, "cpich-Ec-N0" },
  {   1, "cpich-RSCP" },
  {   2, "pathloss" },
  {   3, "spare" },
  { 0, NULL }
};

static const per_choice_t T_measurementQuantity_01_choice[] = {
  {   0, &hf_rrc_cpich_Ec_N0_01  , ASN1_NO_EXTENSIONS     , dissect_rrc_CPICH_Ec_N0 },
  {   1, &hf_rrc_cpich_RSCP_01   , ASN1_NO_EXTENSIONS     , dissect_rrc_CPICH_RSCP },
  {   2, &hf_rrc_pathloss        , ASN1_NO_EXTENSIONS     , dissect_rrc_Pathloss },
  {   3, &hf_rrc_spare           , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_measurementQuantity_01(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_measurementQuantity_01, T_measurementQuantity_01_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t T_fdd_160_sequence[] = {
  { &hf_rrc_measurementQuantity_03, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_measurementQuantity_01 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_fdd_160(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_fdd_160, T_fdd_160_sequence);

  return offset;
}



static int
dissect_rrc_TimeslotISCP(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            0U, 127U, NULL, FALSE);

  return offset;
}


static const per_sequence_t TimeslotISCP_List_sequence_of[1] = {
  { &hf_rrc_TimeslotISCP_List_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TimeslotISCP },
};

static int
dissect_rrc_TimeslotISCP_List(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_TimeslotISCP_List, TimeslotISCP_List_sequence_of,
                                                  1, maxTS, FALSE);

  return offset;
}



static int
dissect_rrc_PrimaryCCPCH_RSCP(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            0U, 127U, NULL, FALSE);

  return offset;
}


static const per_sequence_t T_tdd_108_sequence[] = {
  { &hf_rrc_timeslotISCP    , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_TimeslotISCP_List },
  { &hf_rrc_primaryCCPCH_RSCP, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_PrimaryCCPCH_RSCP },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_tdd_108(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_tdd_108, T_tdd_108_sequence);

  return offset;
}


static const value_string rrc_T_modeSpecificInfo_121_vals[] = {
  {   0, "fdd" },
  {   1, "tdd" },
  { 0, NULL }
};

static const per_choice_t T_modeSpecificInfo_121_choice[] = {
  {   0, &hf_rrc_fdd_166         , ASN1_NO_EXTENSIONS     , dissect_rrc_T_fdd_160 },
  {   1, &hf_rrc_tdd_116         , ASN1_NO_EXTENSIONS     , dissect_rrc_T_tdd_108 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_modeSpecificInfo_121(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_modeSpecificInfo_121, T_modeSpecificInfo_121_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t T_currentCell_sequence[] = {
  { &hf_rrc_modeSpecificInfo_121, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_modeSpecificInfo_121 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_currentCell(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_currentCell, T_currentCell_sequence);

  return offset;
}



static int
dissect_rrc_SFN_SFN_ObsTimeDifference1(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            0U, 16777215U, NULL, FALSE);

  return offset;
}



static int
dissect_rrc_SFN_SFN_ObsTimeDifference2(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            0U, 65535U, NULL, FALSE);

  return offset;
}


static const value_string rrc_SFN_SFN_ObsTimeDifference_vals[] = {
  {   0, "type1" },
  {   1, "type2" },
  { 0, NULL }
};

static const per_choice_t SFN_SFN_ObsTimeDifference_choice[] = {
  {   0, &hf_rrc_type1_07        , ASN1_NO_EXTENSIONS     , dissect_rrc_SFN_SFN_ObsTimeDifference1 },
  {   1, &hf_rrc_type2_06        , ASN1_NO_EXTENSIONS     , dissect_rrc_SFN_SFN_ObsTimeDifference2 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_SFN_SFN_ObsTimeDifference(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_SFN_SFN_ObsTimeDifference, SFN_SFN_ObsTimeDifference_choice,
                                 NULL);

  return offset;
}


static const value_string rrc_T_measurementQuantity_02_vals[] = {
  {   0, "cpich-Ec-N0" },
  {   1, "cpich-RSCP" },
  {   2, "pathloss" },
  {   3, "spare" },
  { 0, NULL }
};

static const per_choice_t T_measurementQuantity_02_choice[] = {
  {   0, &hf_rrc_cpich_Ec_N0_01  , ASN1_NO_EXTENSIONS     , dissect_rrc_CPICH_Ec_N0 },
  {   1, &hf_rrc_cpich_RSCP_01   , ASN1_NO_EXTENSIONS     , dissect_rrc_CPICH_RSCP },
  {   2, &hf_rrc_pathloss        , ASN1_NO_EXTENSIONS     , dissect_rrc_Pathloss },
  {   3, &hf_rrc_spare           , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_measurementQuantity_02(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_measurementQuantity_02, T_measurementQuantity_02_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t T_fdd_161_sequence[] = {
  { &hf_rrc_primaryCPICH_Info, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PrimaryCPICH_Info },
  { &hf_rrc_measurementQuantity_04, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_measurementQuantity_02 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_fdd_161(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_fdd_161, T_fdd_161_sequence);

  return offset;
}


static const per_sequence_t T_tdd_109_sequence[] = {
  { &hf_rrc_cellParametersID, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CellParametersID },
  { &hf_rrc_primaryCCPCH_RSCP, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PrimaryCCPCH_RSCP },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_tdd_109(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_tdd_109, T_tdd_109_sequence);

  return offset;
}


static const value_string rrc_T_modeSpecificInfo_122_vals[] = {
  {   0, "fdd" },
  {   1, "tdd" },
  { 0, NULL }
};

static const per_choice_t T_modeSpecificInfo_122_choice[] = {
  {   0, &hf_rrc_fdd_167         , ASN1_NO_EXTENSIONS     , dissect_rrc_T_fdd_161 },
  {   1, &hf_rrc_tdd_117         , ASN1_NO_EXTENSIONS     , dissect_rrc_T_tdd_109 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_modeSpecificInfo_122(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_modeSpecificInfo_122, T_modeSpecificInfo_122_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t MonitoredCellRACH_Result_sequence[] = {
  { &hf_rrc_sfn_SFN_ObsTimeDifference, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_SFN_SFN_ObsTimeDifference },
  { &hf_rrc_modeSpecificInfo_122, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_modeSpecificInfo_122 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_MonitoredCellRACH_Result(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_MonitoredCellRACH_Result, MonitoredCellRACH_Result_sequence);

  return offset;
}


static const per_sequence_t MonitoredCellRACH_List_sequence_of[1] = {
  { &hf_rrc_MonitoredCellRACH_List_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MonitoredCellRACH_Result },
};

static int
dissect_rrc_MonitoredCellRACH_List(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_MonitoredCellRACH_List, MonitoredCellRACH_List_sequence_of,
                                                  1, 8, FALSE);

  return offset;
}


static const per_sequence_t MeasuredResultsOnRACH_sequence[] = {
  { &hf_rrc_currentCell     , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_currentCell },
  { &hf_rrc_monitoredCells  , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MonitoredCellRACH_List },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_MeasuredResultsOnRACH(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_MeasuredResultsOnRACH, MeasuredResultsOnRACH_sequence);

  return offset;
}


static const per_sequence_t InitialDirectTransfer_v3a0ext_sequence[] = {
  { &hf_rrc_start_Value     , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_START_Value },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_InitialDirectTransfer_v3a0ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_InitialDirectTransfer_v3a0ext, InitialDirectTransfer_v3a0ext_sequence);

  return offset;
}



static int
dissect_rrc_T_initialDirectTransfer_r3_add_ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_bit_string_containing_pdu_new(tvb, offset, actx, tree, hf_index,
                                                              NO_BOUND, NO_BOUND, FALSE, dissect_InitialDirectTransfer_r3_add_ext_IEs_PDU);

  return offset;
}


static const value_string rrc_EstablishmentCause_vals[] = {
  {   0, "originatingConversationalCall" },
  {   1, "originatingStreamingCall" },
  {   2, "originatingInteractiveCall" },
  {   3, "originatingBackgroundCall" },
  {   4, "originatingSubscribedTrafficCall" },
  {   5, "terminatingConversationalCall" },
  {   6, "terminatingStreamingCall" },
  {   7, "terminatingInteractiveCall" },
  {   8, "terminatingBackgroundCall" },
  {   9, "emergencyCall" },
  {  10, "interRAT-CellReselection" },
  {  11, "interRAT-CellChangeOrder" },
  {  12, "registration" },
  {  13, "detach" },
  {  14, "originatingHighPrioritySignalling" },
  {  15, "originatingLowPrioritySignalling" },
  {  16, "callRe-establishment" },
  {  17, "terminatingHighPrioritySignalling" },
  {  18, "terminatingLowPrioritySignalling" },
  {  19, "terminatingCauseUnknown" },
  {  20, "mbms-Reception" },
  {  21, "mbms-PTP-RB-Request" },
  {  22, "delayTolerantAccess" },
  {  23, "spare9" },
  {  24, "spare8" },
  {  25, "spare7" },
  {  26, "spare6" },
  {  27, "spare5" },
  {  28, "spare4" },
  {  29, "spare3" },
  {  30, "spare2" },
  {  31, "spare1" },
  { 0, NULL }
};

static value_string_ext rrc_EstablishmentCause_vals_ext = VALUE_STRING_EXT_INIT(rrc_EstablishmentCause_vals);


static int
dissect_rrc_EstablishmentCause(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     32, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t InitialDirectTransfer_v590ext_sequence[] = {
  { &hf_rrc_establishmentCause, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_EstablishmentCause },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_InitialDirectTransfer_v590ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_InitialDirectTransfer_v590ext, InitialDirectTransfer_v590ext_sequence);

  return offset;
}


static const per_sequence_t InterFreqRACHRepCellsList_sequence_of[1] = {
  { &hf_rrc_InterFreqRACHRepCellsList_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_InterFreqCellID },
};

static int
dissect_rrc_InterFreqRACHRepCellsList(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_InterFreqRACHRepCellsList, InterFreqRACHRepCellsList_sequence_of,
                                                  1, maxFreq, FALSE);

  return offset;
}


static const per_sequence_t MeasuredResultsOnRACHinterFreq_sequence[] = {
  { &hf_rrc_interFreqCellIndication_SIB11, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_0_1 },
  { &hf_rrc_interFreqCellIndication_SIB12, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_0_1 },
  { &hf_rrc_interFreqRACHRepCellsList, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_InterFreqRACHRepCellsList },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_MeasuredResultsOnRACHinterFreq(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_MeasuredResultsOnRACHinterFreq, MeasuredResultsOnRACHinterFreq_sequence);

  return offset;
}



static int
dissect_rrc_P_TMSI_GSM_MAP(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_bit_string(tvb, offset, actx, tree, hf_index,
                                     32, 32, FALSE, NULL);

  return offset;
}


static const per_sequence_t MBMS_JoinedInformation_r6_sequence[] = {
  { &hf_rrc_p_TMSI          , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_P_TMSI_GSM_MAP },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_MBMS_JoinedInformation_r6(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_MBMS_JoinedInformation_r6, MBMS_JoinedInformation_r6_sequence);

  return offset;
}


static const per_sequence_t InitialDirectTransfer_v690ext_IEs_sequence[] = {
  { &hf_rrc_plmn_Identity   , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_PLMN_Identity },
  { &hf_rrc_measuredResultsOnRACHinterFreq, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MeasuredResultsOnRACHinterFreq },
  { &hf_rrc_mbms_JoinedInformation, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MBMS_JoinedInformation_r6 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_InitialDirectTransfer_v690ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_InitialDirectTransfer_v690ext_IEs, InitialDirectTransfer_v690ext_IEs_sequence);

  return offset;
}


static const value_string rrc_T_csCallType_01_vals[] = {
  {   0, "speech" },
  {   1, "video" },
  {   2, "other" },
  {   3, "spare" },
  { 0, NULL }
};


static int
dissect_rrc_T_csCallType_01(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     4, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t InitialDirectTransfer_v770ext_IEs_sequence[] = {
  { &hf_rrc_csCallType_01   , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_csCallType_01 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_InitialDirectTransfer_v770ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_InitialDirectTransfer_v770ext_IEs, InitialDirectTransfer_v770ext_IEs_sequence);

  return offset;
}


static const value_string rrc_T_supportOfCSG_vals[] = {
  {   0, "true" },
  { 0, NULL }
};


static int
dissect_rrc_T_supportOfCSG(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t InitialDirectTransfer_v860ext_IEs_sequence[] = {
  { &hf_rrc_supportOfCSG    , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_supportOfCSG },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_InitialDirectTransfer_v860ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_InitialDirectTransfer_v860ext_IEs, InitialDirectTransfer_v860ext_IEs_sequence);

  return offset;
}



static int
dissect_rrc_BIT_STRING_SIZE_maxNumEUTRAFreqs_FACH(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_bit_string(tvb, offset, actx, tree, hf_index,
                                     maxNumEUTRAFreqs_FACH, maxNumEUTRAFreqs_FACH, FALSE, NULL);

  return offset;
}


static const per_sequence_t MeasuredResultsOnRACHeutraFreq_sequence[] = {
  { &hf_rrc_eutraFrequencyIndication_SIB19, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_0_1 },
  { &hf_rrc_eutraFrequencyIndicator, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_maxNumEUTRAFreqs_FACH },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_MeasuredResultsOnRACHeutraFreq(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_MeasuredResultsOnRACHeutraFreq, MeasuredResultsOnRACHeutraFreq_sequence);

  return offset;
}


static const per_sequence_t InitialDirectTransfer_vbxyext_IEs_sequence[] = {
  { &hf_rrc_measuredResultsOnRACHeutraFreq, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MeasuredResultsOnRACHeutraFreq },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_InitialDirectTransfer_vbxyext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_InitialDirectTransfer_vbxyext_IEs, InitialDirectTransfer_vbxyext_IEs_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions_47_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_nonCriticalExtensions_47(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_nonCriticalExtensions_47, T_nonCriticalExtensions_47_sequence);

  return offset;
}


static const per_sequence_t T_vbxyNonCriticalExtensions_05_sequence[] = {
  { &hf_rrc_initialDirectTransfer_vbxyext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_InitialDirectTransfer_vbxyext_IEs },
  { &hf_rrc_nonCriticalExtensions_47, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_nonCriticalExtensions_47 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_vbxyNonCriticalExtensions_05(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_vbxyNonCriticalExtensions_05, T_vbxyNonCriticalExtensions_05_sequence);

  return offset;
}


static const per_sequence_t T_v860NonCriticalExtensions_03_sequence[] = {
  { &hf_rrc_initialDirectTransfer_v860ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_InitialDirectTransfer_v860ext_IEs },
  { &hf_rrc_vbxyNonCriticalExtensions_05, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_vbxyNonCriticalExtensions_05 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v860NonCriticalExtensions_03(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v860NonCriticalExtensions_03, T_v860NonCriticalExtensions_03_sequence);

  return offset;
}


static const per_sequence_t T_v770NonCriticalExtensions_01_sequence[] = {
  { &hf_rrc_initialDirectTransfer_v770ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_InitialDirectTransfer_v770ext_IEs },
  { &hf_rrc_v860NonCriticalExtensions_03, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v860NonCriticalExtensions_03 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v770NonCriticalExtensions_01(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v770NonCriticalExtensions_01, T_v770NonCriticalExtensions_01_sequence);

  return offset;
}


static const per_sequence_t T_v690NonCriticalExtensions_08_sequence[] = {
  { &hf_rrc_initialDirectTransfer_v690ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_InitialDirectTransfer_v690ext_IEs },
  { &hf_rrc_v770NonCriticalExtensions_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v770NonCriticalExtensions_01 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v690NonCriticalExtensions_08(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v690NonCriticalExtensions_08, T_v690NonCriticalExtensions_08_sequence);

  return offset;
}


static const per_sequence_t T_v590NonCriticalExtensions_05_sequence[] = {
  { &hf_rrc_initialDirectTransfer_v590ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_InitialDirectTransfer_v590ext },
  { &hf_rrc_v690NonCriticalExtensions_08, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v690NonCriticalExtensions_08 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v590NonCriticalExtensions_05(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v590NonCriticalExtensions_05, T_v590NonCriticalExtensions_05_sequence);

  return offset;
}


static const per_sequence_t T_laterNonCriticalExtensions_13_sequence[] = {
  { &hf_rrc_initialDirectTransfer_r3_add_ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_initialDirectTransfer_r3_add_ext },
  { &hf_rrc_v590NonCriticalExtensions_05, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v590NonCriticalExtensions_05 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_laterNonCriticalExtensions_13(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_laterNonCriticalExtensions_13, T_laterNonCriticalExtensions_13_sequence);

  return offset;
}


static const per_sequence_t T_v3a0NonCriticalExtensions_02_sequence[] = {
  { &hf_rrc_initialDirectTransfer_v3a0ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_InitialDirectTransfer_v3a0ext },
  { &hf_rrc_laterNonCriticalExtensions_13, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_laterNonCriticalExtensions_13 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v3a0NonCriticalExtensions_02(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v3a0NonCriticalExtensions_02, T_v3a0NonCriticalExtensions_02_sequence);

  return offset;
}


static const per_sequence_t InitialDirectTransfer_sequence[] = {
  { &hf_rrc_cn_DomainIdentity, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CN_DomainIdentity },
  { &hf_rrc_intraDomainNasNodeSelector, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_IntraDomainNasNodeSelector },
  { &hf_rrc_nas_Message     , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_NAS_Message },
  { &hf_rrc_measuredResultsOnRACH, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MeasuredResultsOnRACH },
  { &hf_rrc_v3a0NonCriticalExtensions_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v3a0NonCriticalExtensions_02 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_InitialDirectTransfer(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
#line 221 "../../asn1/rrc/rrc.cnf"
  col_append_str(actx->pinfo->cinfo, COL_INFO, "InitialDirectTransfer");
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_InitialDirectTransfer, InitialDirectTransfer_sequence);




  return offset;
}


static const value_string rrc_InterRAT_HO_FailureCause_vals[] = {
  {   0, "configurationUnacceptable" },
  {   1, "physicalChannelFailure" },
  {   2, "protocolError" },
  {   3, "interRAT-ProtocolError" },
  {   4, "unspecified" },
  {   5, "spare11" },
  {   6, "spare10" },
  {   7, "spare9" },
  {   8, "spare8" },
  {   9, "spare7" },
  {  10, "spare6" },
  {  11, "spare5" },
  {  12, "spare4" },
  {  13, "spare3" },
  {  14, "spare2" },
  {  15, "spare1" },
  { 0, NULL }
};

static const per_choice_t InterRAT_HO_FailureCause_choice[] = {
  {   0, &hf_rrc_configurationUnacceptable, ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   1, &hf_rrc_physicalChannelFailure, ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   2, &hf_rrc_protocolError   , ASN1_NO_EXTENSIONS     , dissect_rrc_ProtocolErrorInformation },
  {   3, &hf_rrc_interRAT_ProtocolError, ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   4, &hf_rrc_unspecified     , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   5, &hf_rrc_spare11         , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   6, &hf_rrc_spare10         , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   7, &hf_rrc_spare9          , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   8, &hf_rrc_spare8          , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   9, &hf_rrc_spare7          , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {  10, &hf_rrc_spare6          , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {  11, &hf_rrc_spare5          , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {  12, &hf_rrc_spare4          , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {  13, &hf_rrc_spare3          , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {  14, &hf_rrc_spare2          , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {  15, &hf_rrc_spare1          , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_InterRAT_HO_FailureCause(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_InterRAT_HO_FailureCause, InterRAT_HO_FailureCause_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t T_gsm_sequence[] = {
  { &hf_rrc_gsm_MessageList_gsm, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_GSM_MessageList },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_gsm(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_gsm, T_gsm_sequence);

  return offset;
}


static const per_sequence_t T_cdma2000_sequence[] = {
  { &hf_rrc_cdma2000_MessageList, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CDMA2000_MessageList },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_cdma2000(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_cdma2000, T_cdma2000_sequence);

  return offset;
}


static const value_string rrc_T_interRATMessage_vals[] = {
  {   0, "gsm" },
  {   1, "cdma2000" },
  { 0, NULL }
};

static const per_choice_t T_interRATMessage_choice[] = {
  {   0, &hf_rrc_gsm             , ASN1_NO_EXTENSIONS     , dissect_rrc_T_gsm },
  {   1, &hf_rrc_cdma2000        , ASN1_NO_EXTENSIONS     , dissect_rrc_T_cdma2000 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_interRATMessage(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_interRATMessage, T_interRATMessage_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t HandoverFromUtranFailure_v590ext_IEs_sequence[] = {
  { &hf_rrc_geranIu_MessageList_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_GERANIu_MessageList },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_HandoverFromUtranFailure_v590ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_HandoverFromUtranFailure_v590ext_IEs, HandoverFromUtranFailure_v590ext_IEs_sequence);

  return offset;
}



static int
dissect_rrc_T_eutra_Message_02(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
#line 621 "../../asn1/rrc/rrc.cnf"
  tvbuff_t *eutra_message_tvb = NULL;
  offset = dissect_per_octet_string(tvb, offset, actx, tree, hf_index,
                                       NO_BOUND, NO_BOUND, FALSE, &eutra_message_tvb);

  if (eutra_message_tvb && lte_rrc_dl_dcch_handle)
    call_dissector(lte_rrc_dl_dcch_handle, eutra_message_tvb, actx->pinfo, tree);



  return offset;
}


static const per_sequence_t HandoverFromUtranFailure_v860ext_IEs_sequence[] = {
  { &hf_rrc_eutra_Message_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_eutra_Message_02 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_HandoverFromUtranFailure_v860ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_HandoverFromUtranFailure_v860ext_IEs, HandoverFromUtranFailure_v860ext_IEs_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions_55_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_nonCriticalExtensions_55(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_nonCriticalExtensions_55, T_nonCriticalExtensions_55_sequence);

  return offset;
}


static const per_sequence_t T_v860NonCriticalExtensions_06_sequence[] = {
  { &hf_rrc_handoverFromUTRANFailure_v860ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_HandoverFromUtranFailure_v860ext_IEs },
  { &hf_rrc_nonCriticalExtensions_55, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_nonCriticalExtensions_55 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v860NonCriticalExtensions_06(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v860NonCriticalExtensions_06, T_v860NonCriticalExtensions_06_sequence);

  return offset;
}


static const per_sequence_t T_v590NonCriticalExtensions_06_sequence[] = {
  { &hf_rrc_handoverFromUTRANFailure_v590ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_HandoverFromUtranFailure_v590ext_IEs },
  { &hf_rrc_v860NonCriticalExtensions_06, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v860NonCriticalExtensions_06 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v590NonCriticalExtensions_06(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v590NonCriticalExtensions_06, T_v590NonCriticalExtensions_06_sequence);

  return offset;
}


static const per_sequence_t T_laterNonCriticalExtensions_16_sequence[] = {
  { &hf_rrc_handoverFromUTRANFailure_r3_add_ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_BIT_STRING },
  { &hf_rrc_v590NonCriticalExtensions_06, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v590NonCriticalExtensions_06 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_laterNonCriticalExtensions_16(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_laterNonCriticalExtensions_16, T_laterNonCriticalExtensions_16_sequence);

  return offset;
}


static const per_sequence_t HandoverFromUTRANFailure_sequence[] = {
  { &hf_rrc_rrc_TransactionIdentifier, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RRC_TransactionIdentifier },
  { &hf_rrc_interRAT_HO_FailureCause, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_InterRAT_HO_FailureCause },
  { &hf_rrc_interRATMessage , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_interRATMessage },
  { &hf_rrc_laterNonCriticalExtensions_16, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_laterNonCriticalExtensions_16 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_HandoverFromUTRANFailure(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
#line 241 "../../asn1/rrc/rrc.cnf"
  col_append_str(actx->pinfo->cinfo, COL_INFO, "HandoverFromUTRANFailure");
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_HandoverFromUTRANFailure, HandoverFromUTRANFailure_sequence);




  return offset;
}


static const per_sequence_t MeasurementControlFailure_v590ext_IEs_sequence[] = {
  { &hf_rrc_rrc_TransactionIdentifier_MSP_v590ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RRC_TransactionIdentifier },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_MeasurementControlFailure_v590ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_MeasurementControlFailure_v590ext_IEs, MeasurementControlFailure_v590ext_IEs_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions_66_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_nonCriticalExtensions_66(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_nonCriticalExtensions_66, T_nonCriticalExtensions_66_sequence);

  return offset;
}


static const per_sequence_t T_v590NonCriticalExtensions_10_sequence[] = {
  { &hf_rrc_measurementControlFailure_v590ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MeasurementControlFailure_v590ext_IEs },
  { &hf_rrc_nonCriticalExtensions_66, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_nonCriticalExtensions_66 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v590NonCriticalExtensions_10(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v590NonCriticalExtensions_10, T_v590NonCriticalExtensions_10_sequence);

  return offset;
}


static const per_sequence_t T_laterNonCriticalExtensions_19_sequence[] = {
  { &hf_rrc_measurementControlFailure_r3_add_ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_BIT_STRING },
  { &hf_rrc_v590NonCriticalExtensions_10, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v590NonCriticalExtensions_10 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_laterNonCriticalExtensions_19(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_laterNonCriticalExtensions_19, T_laterNonCriticalExtensions_19_sequence);

  return offset;
}


static const per_sequence_t MeasurementControlFailure_sequence[] = {
  { &hf_rrc_rrc_TransactionIdentifier, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RRC_TransactionIdentifier },
  { &hf_rrc_failureCause    , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_FailureCauseWithProtErr },
  { &hf_rrc_laterNonCriticalExtensions_19, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_laterNonCriticalExtensions_19 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_MeasurementControlFailure(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
#line 289 "../../asn1/rrc/rrc.cnf"
  col_append_str(actx->pinfo->cinfo, COL_INFO, "MeasurementControlFailure");
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_MeasurementControlFailure, MeasurementControlFailure_sequence);




  return offset;
}


static const per_sequence_t CountC_SFN_Frame_difference_sequence[] = {
  { &hf_rrc_countC_SFN_High , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_0_15 },
  { &hf_rrc_off             , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_0_255 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_CountC_SFN_Frame_difference(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_CountC_SFN_Frame_difference, CountC_SFN_Frame_difference_sequence);

  return offset;
}


static const per_sequence_t T_fdd_148_sequence[] = {
  { &hf_rrc_countC_SFN_Frame_difference, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CountC_SFN_Frame_difference },
  { &hf_rrc_tm              , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_0_38399 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_fdd_148(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_fdd_148, T_fdd_148_sequence);

  return offset;
}


static const per_sequence_t T_tdd_99_sequence[] = {
  { &hf_rrc_countC_SFN_Frame_difference, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CountC_SFN_Frame_difference },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_tdd_99(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_tdd_99, T_tdd_99_sequence);

  return offset;
}


static const value_string rrc_T_modeSpecificInfo_109_vals[] = {
  {   0, "fdd" },
  {   1, "tdd" },
  { 0, NULL }
};

static const per_choice_t T_modeSpecificInfo_109_choice[] = {
  {   0, &hf_rrc_fdd_153         , ASN1_NO_EXTENSIONS     , dissect_rrc_T_fdd_148 },
  {   1, &hf_rrc_tdd_106         , ASN1_NO_EXTENSIONS     , dissect_rrc_T_tdd_99 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_modeSpecificInfo_109(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_modeSpecificInfo_109, T_modeSpecificInfo_109_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t CellSynchronisationInfo_sequence[] = {
  { &hf_rrc_modeSpecificInfo_109, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_modeSpecificInfo_109 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_CellSynchronisationInfo(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_CellSynchronisationInfo, CellSynchronisationInfo_sequence);

  return offset;
}


static const per_sequence_t T_fdd_139_sequence[] = {
  { &hf_rrc_primaryCPICH_Info, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PrimaryCPICH_Info },
  { &hf_rrc_cpich_Ec_N0_01  , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CPICH_Ec_N0 },
  { &hf_rrc_cpich_RSCP_01   , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CPICH_RSCP },
  { &hf_rrc_pathloss        , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_Pathloss },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_fdd_139(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_fdd_139, T_fdd_139_sequence);

  return offset;
}


static const per_sequence_t T_tdd_90_sequence[] = {
  { &hf_rrc_cellParametersID, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CellParametersID },
  { &hf_rrc_proposedTGSN    , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_TGSN },
  { &hf_rrc_primaryCCPCH_RSCP, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_PrimaryCCPCH_RSCP },
  { &hf_rrc_pathloss        , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_Pathloss },
  { &hf_rrc_timeslotISCP_List, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_TimeslotISCP_List },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_tdd_90(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_tdd_90, T_tdd_90_sequence);

  return offset;
}


static const value_string rrc_T_modeSpecificInfo_100_vals[] = {
  {   0, "fdd" },
  {   1, "tdd" },
  { 0, NULL }
};

static const per_choice_t T_modeSpecificInfo_100_choice[] = {
  {   0, &hf_rrc_fdd_142         , ASN1_NO_EXTENSIONS     , dissect_rrc_T_fdd_139 },
  {   1, &hf_rrc_tdd_96          , ASN1_NO_EXTENSIONS     , dissect_rrc_T_tdd_90 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_modeSpecificInfo_100(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_modeSpecificInfo_100, T_modeSpecificInfo_100_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t CellMeasuredResults_sequence[] = {
  { &hf_rrc_cellIdentity    , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CellIdentity },
  { &hf_rrc_dummy_48        , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_SFN_SFN_ObsTimeDifference },
  { &hf_rrc_cellSynchronisationInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CellSynchronisationInfo },
  { &hf_rrc_modeSpecificInfo_100, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_modeSpecificInfo_100 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_CellMeasuredResults(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_CellMeasuredResults, CellMeasuredResults_sequence);

  return offset;
}


static const per_sequence_t IntraFreqMeasuredResultsList_sequence_of[1] = {
  { &hf_rrc_IntraFreqMeasuredResultsList_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CellMeasuredResults },
};

static int
dissect_rrc_IntraFreqMeasuredResultsList(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_IntraFreqMeasuredResultsList, IntraFreqMeasuredResultsList_sequence_of,
                                                  1, maxCellMeas, FALSE);

  return offset;
}



static int
dissect_rrc_UTRA_CarrierRSSI(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            0U, 127U, NULL, FALSE);

  return offset;
}


static const per_sequence_t InterFreqCellMeasuredResultsList_sequence_of[1] = {
  { &hf_rrc_InterFreqCellMeasuredResultsList_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CellMeasuredResults },
};

static int
dissect_rrc_InterFreqCellMeasuredResultsList(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_InterFreqCellMeasuredResultsList, InterFreqCellMeasuredResultsList_sequence_of,
                                                  1, maxCellMeas, FALSE);

  return offset;
}


static const per_sequence_t InterFreqMeasuredResults_sequence[] = {
  { &hf_rrc_frequencyInfo   , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_FrequencyInfo },
  { &hf_rrc_utra_CarrierRSSI, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UTRA_CarrierRSSI },
  { &hf_rrc_interFreqCellMeasuredResultsList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_InterFreqCellMeasuredResultsList },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_InterFreqMeasuredResults(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_InterFreqMeasuredResults, InterFreqMeasuredResults_sequence);

  return offset;
}


static const per_sequence_t InterFreqMeasuredResultsList_sequence_of[1] = {
  { &hf_rrc_InterFreqMeasuredResultsList_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_InterFreqMeasuredResults },
};

static int
dissect_rrc_InterFreqMeasuredResultsList(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_InterFreqMeasuredResultsList, InterFreqMeasuredResultsList_sequence_of,
                                                  1, maxFreq, FALSE);

  return offset;
}



static int
dissect_rrc_GSM_CarrierRSSI(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_bit_string(tvb, offset, actx, tree, hf_index,
                                     6, 6, FALSE, NULL);

  return offset;
}



static int
dissect_rrc_INTEGER_46_173(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            46U, 173U, NULL, FALSE);

  return offset;
}



static int
dissect_rrc_INTEGER_0_maxCellMeas(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            0U, maxCellMeas, NULL, FALSE);

  return offset;
}


static const value_string rrc_BSICReported_vals[] = {
  {   0, "verifiedBSIC" },
  {   1, "nonVerifiedBSIC" },
  { 0, NULL }
};

static const per_choice_t BSICReported_choice[] = {
  {   0, &hf_rrc_verifiedBSIC    , ASN1_NO_EXTENSIONS     , dissect_rrc_INTEGER_0_maxCellMeas },
  {   1, &hf_rrc_nonVerifiedBSIC , ASN1_NO_EXTENSIONS     , dissect_rrc_BCCH_ARFCN },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_BSICReported(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_BSICReported, BSICReported_choice,
                                 NULL);

  return offset;
}



static int
dissect_rrc_ObservedTimeDifferenceToGSM(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            0U, 4095U, NULL, FALSE);

  return offset;
}


static const per_sequence_t GSM_MeasuredResults_sequence[] = {
  { &hf_rrc_gsm_CarrierRSSI , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_GSM_CarrierRSSI },
  { &hf_rrc_dummy_51        , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_INTEGER_46_173 },
  { &hf_rrc_bsicReported    , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BSICReported },
  { &hf_rrc_dummy2_12       , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_ObservedTimeDifferenceToGSM },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_GSM_MeasuredResults(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_GSM_MeasuredResults, GSM_MeasuredResults_sequence);

  return offset;
}


static const per_sequence_t GSM_MeasuredResultsList_sequence_of[1] = {
  { &hf_rrc_GSM_MeasuredResultsList_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_GSM_MeasuredResults },
};

static int
dissect_rrc_GSM_MeasuredResultsList(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_GSM_MeasuredResultsList, GSM_MeasuredResultsList_sequence_of,
                                                  1, maxReportedGSMCells, FALSE);

  return offset;
}


static const value_string rrc_InterRATMeasuredResults_vals[] = {
  {   0, "gsm" },
  {   1, "spare" },
  { 0, NULL }
};

static const per_choice_t InterRATMeasuredResults_choice[] = {
  {   0, &hf_rrc_gsm_12          , ASN1_NO_EXTENSIONS     , dissect_rrc_GSM_MeasuredResultsList },
  {   1, &hf_rrc_spare           , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_InterRATMeasuredResults(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_InterRATMeasuredResults, InterRATMeasuredResults_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t InterRATMeasuredResultsList_sequence_of[1] = {
  { &hf_rrc_InterRATMeasuredResultsList_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_InterRATMeasuredResults },
};

static int
dissect_rrc_InterRATMeasuredResultsList(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_InterRATMeasuredResultsList, InterRATMeasuredResultsList_sequence_of,
                                                  1, maxOtherRAT_16, FALSE);

  return offset;
}


static const value_string rrc_RLC_BuffersPayload_vals[] = {
  {   0, "pl0" },
  {   1, "pl4" },
  {   2, "pl8" },
  {   3, "pl16" },
  {   4, "pl32" },
  {   5, "pl64" },
  {   6, "pl128" },
  {   7, "pl256" },
  {   8, "pl512" },
  {   9, "pl1024" },
  {  10, "pl2k" },
  {  11, "pl4k" },
  {  12, "pl8k" },
  {  13, "pl16k" },
  {  14, "pl32k" },
  {  15, "pl64k" },
  {  16, "pl128k" },
  {  17, "pl256k" },
  {  18, "pl512k" },
  {  19, "pl1024k" },
  {  20, "spare12" },
  {  21, "spare11" },
  {  22, "spare10" },
  {  23, "spare9" },
  {  24, "spare8" },
  {  25, "spare7" },
  {  26, "spare6" },
  {  27, "spare5" },
  {  28, "spare4" },
  {  29, "spare3" },
  {  30, "spare2" },
  {  31, "spare1" },
  { 0, NULL }
};

static value_string_ext rrc_RLC_BuffersPayload_vals_ext = VALUE_STRING_EXT_INIT(rrc_RLC_BuffersPayload_vals);


static int
dissect_rrc_RLC_BuffersPayload(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     32, NULL, FALSE, 0, NULL);

  return offset;
}


static const value_string rrc_AverageRLC_BufferPayload_vals[] = {
  {   0, "pla0" },
  {   1, "pla4" },
  {   2, "pla8" },
  {   3, "pla16" },
  {   4, "pla32" },
  {   5, "pla64" },
  {   6, "pla128" },
  {   7, "pla256" },
  {   8, "pla512" },
  {   9, "pla1024" },
  {  10, "pla2k" },
  {  11, "pla4k" },
  {  12, "pla8k" },
  {  13, "pla16k" },
  {  14, "pla32k" },
  {  15, "pla64k" },
  {  16, "pla128k" },
  {  17, "pla256k" },
  {  18, "pla512k" },
  {  19, "pla1024k" },
  {  20, "spare12" },
  {  21, "spare11" },
  {  22, "spare10" },
  {  23, "spare9" },
  {  24, "spare8" },
  {  25, "spare7" },
  {  26, "spare6" },
  {  27, "spare5" },
  {  28, "spare4" },
  {  29, "spare3" },
  {  30, "spare2" },
  {  31, "spare1" },
  { 0, NULL }
};

static value_string_ext rrc_AverageRLC_BufferPayload_vals_ext = VALUE_STRING_EXT_INIT(rrc_AverageRLC_BufferPayload_vals);


static int
dissect_rrc_AverageRLC_BufferPayload(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     32, NULL, FALSE, 0, NULL);

  return offset;
}


static const value_string rrc_VarianceOfRLC_BufferPayload_vals[] = {
  {   0, "plv0" },
  {   1, "plv4" },
  {   2, "plv8" },
  {   3, "plv16" },
  {   4, "plv32" },
  {   5, "plv64" },
  {   6, "plv128" },
  {   7, "plv256" },
  {   8, "plv512" },
  {   9, "plv1024" },
  {  10, "plv2k" },
  {  11, "plv4k" },
  {  12, "plv8k" },
  {  13, "plv16k" },
  {  14, "spare2" },
  {  15, "spare1" },
  { 0, NULL }
};


static int
dissect_rrc_VarianceOfRLC_BufferPayload(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     16, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t TrafficVolumeMeasuredResults_sequence[] = {
  { &hf_rrc_rb_Identity     , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RB_Identity },
  { &hf_rrc_rlc_BuffersPayload, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RLC_BuffersPayload },
  { &hf_rrc_averageRLC_BufferPayload_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_AverageRLC_BufferPayload },
  { &hf_rrc_varianceOfRLC_BufferPayload_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_VarianceOfRLC_BufferPayload },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_TrafficVolumeMeasuredResults(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_TrafficVolumeMeasuredResults, TrafficVolumeMeasuredResults_sequence);

  return offset;
}


static const per_sequence_t TrafficVolumeMeasuredResultsList_sequence_of[1] = {
  { &hf_rrc_TrafficVolumeMeasuredResultsList_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TrafficVolumeMeasuredResults },
};

static int
dissect_rrc_TrafficVolumeMeasuredResultsList(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_TrafficVolumeMeasuredResultsList, TrafficVolumeMeasuredResultsList_sequence_of,
                                                  1, maxRB, FALSE);

  return offset;
}



static int
dissect_rrc_DL_TransportChannelBLER(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            0U, 63U, NULL, FALSE);

  return offset;
}


static const per_sequence_t BLER_MeasurementResults_sequence[] = {
  { &hf_rrc_transportChannelIdentity, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TransportChannelIdentity },
  { &hf_rrc_dl_TransportChannelBLER, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_TransportChannelBLER },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_BLER_MeasurementResults(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_BLER_MeasurementResults, BLER_MeasurementResults_sequence);

  return offset;
}


static const per_sequence_t BLER_MeasurementResultsList_sequence_of[1] = {
  { &hf_rrc_BLER_MeasurementResultsList_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BLER_MeasurementResults },
};

static int
dissect_rrc_BLER_MeasurementResultsList(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_BLER_MeasurementResultsList, BLER_MeasurementResultsList_sequence_of,
                                                  1, maxTrCH, FALSE);

  return offset;
}



static int
dissect_rrc_SIR(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            0U, 63U, NULL, FALSE);

  return offset;
}


static const per_sequence_t SIR_TimeslotList_sequence_of[1] = {
  { &hf_rrc_SIR_TimeslotList_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SIR },
};

static int
dissect_rrc_SIR_TimeslotList(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_SIR_TimeslotList, SIR_TimeslotList_sequence_of,
                                                  1, maxTS, FALSE);

  return offset;
}


static const per_sequence_t SIR_MeasurementResults_sequence[] = {
  { &hf_rrc_tfcs_ID_01      , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TFCS_IdentityPlain },
  { &hf_rrc_sir_TimeslotList, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SIR_TimeslotList },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_SIR_MeasurementResults(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_SIR_MeasurementResults, SIR_MeasurementResults_sequence);

  return offset;
}


static const per_sequence_t SIR_MeasurementList_sequence_of[1] = {
  { &hf_rrc_SIR_MeasurementList_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SIR_MeasurementResults },
};

static int
dissect_rrc_SIR_MeasurementList(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_SIR_MeasurementList, SIR_MeasurementList_sequence_of,
                                                  1, maxCCTrCH, FALSE);

  return offset;
}


static const per_sequence_t T_tdd_111_sequence[] = {
  { &hf_rrc_sir_MeasurementResults, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_SIR_MeasurementList },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_tdd_111(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_tdd_111, T_tdd_111_sequence);

  return offset;
}


static const value_string rrc_T_modeSpecificInfo_129_vals[] = {
  {   0, "fdd" },
  {   1, "tdd" },
  { 0, NULL }
};

static const per_choice_t T_modeSpecificInfo_129_choice[] = {
  {   0, &hf_rrc_fdd_45          , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   1, &hf_rrc_tdd_119         , ASN1_NO_EXTENSIONS     , dissect_rrc_T_tdd_111 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_modeSpecificInfo_129(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_modeSpecificInfo_129, T_modeSpecificInfo_129_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t QualityMeasuredResults_sequence[] = {
  { &hf_rrc_blerMeasurementResultsList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_BLER_MeasurementResultsList },
  { &hf_rrc_modeSpecificInfo_129, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_modeSpecificInfo_129 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_QualityMeasuredResults(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_QualityMeasuredResults, QualityMeasuredResults_sequence);

  return offset;
}



static int
dissect_rrc_UE_TransmittedPower(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            0U, 104U, NULL, FALSE);

  return offset;
}



static int
dissect_rrc_UE_RX_TX_TimeDifferenceType1(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            768U, 1791U, NULL, FALSE);

  return offset;
}


static const per_sequence_t UE_RX_TX_ReportEntry_sequence[] = {
  { &hf_rrc_primaryCPICH_Info, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PrimaryCPICH_Info },
  { &hf_rrc_ue_RX_TX_TimeDifferenceType1, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UE_RX_TX_TimeDifferenceType1 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UE_RX_TX_ReportEntry(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UE_RX_TX_ReportEntry, UE_RX_TX_ReportEntry_sequence);

  return offset;
}


static const per_sequence_t UE_RX_TX_ReportEntryList_sequence_of[1] = {
  { &hf_rrc_UE_RX_TX_ReportEntryList_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UE_RX_TX_ReportEntry },
};

static int
dissect_rrc_UE_RX_TX_ReportEntryList(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_UE_RX_TX_ReportEntryList, UE_RX_TX_ReportEntryList_sequence_of,
                                                  1, maxRL, FALSE);

  return offset;
}


static const per_sequence_t T_fdd_164_sequence[] = {
  { &hf_rrc_ue_TransmittedPowerFDD, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_TransmittedPower },
  { &hf_rrc_ue_RX_TX_ReportEntryList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_RX_TX_ReportEntryList },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_fdd_164(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_fdd_164, T_fdd_164_sequence);

  return offset;
}


static const per_sequence_t UE_TransmittedPowerTDD_List_sequence_of[1] = {
  { &hf_rrc_UE_TransmittedPowerTDD_List_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UE_TransmittedPower },
};

static int
dissect_rrc_UE_TransmittedPowerTDD_List(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_UE_TransmittedPowerTDD_List, UE_TransmittedPowerTDD_List_sequence_of,
                                                  1, maxTS, FALSE);

  return offset;
}


static const per_sequence_t T_tdd_113_sequence[] = {
  { &hf_rrc_ue_TransmittedPowerTDD_List, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_TransmittedPowerTDD_List },
  { &hf_rrc_appliedTA       , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_TimingAdvance },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_tdd_113(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_tdd_113, T_tdd_113_sequence);

  return offset;
}


static const value_string rrc_T_modeSpecificInfo_131_vals[] = {
  {   0, "fdd" },
  {   1, "tdd" },
  { 0, NULL }
};

static const per_choice_t T_modeSpecificInfo_131_choice[] = {
  {   0, &hf_rrc_fdd_170         , ASN1_NO_EXTENSIONS     , dissect_rrc_T_fdd_164 },
  {   1, &hf_rrc_tdd_121         , ASN1_NO_EXTENSIONS     , dissect_rrc_T_tdd_113 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_modeSpecificInfo_131(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_modeSpecificInfo_131, T_modeSpecificInfo_131_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t UE_InternalMeasuredResults_sequence[] = {
  { &hf_rrc_modeSpecificInfo_131, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_modeSpecificInfo_131 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UE_InternalMeasuredResults(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UE_InternalMeasuredResults, UE_InternalMeasuredResults_sequence);

  return offset;
}



static int
dissect_rrc_UE_RX_TX_TimeDifferenceType2(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            0U, 8191U, NULL, FALSE);

  return offset;
}


static const per_sequence_t UE_Positioning_OTDOA_Quality_sequence[] = {
  { &hf_rrc_stdResolution   , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_2 },
  { &hf_rrc_numberOfOTDOA_Measurements, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_3 },
  { &hf_rrc_stdOfOTDOA_Measurements, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_5 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UE_Positioning_OTDOA_Quality(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UE_Positioning_OTDOA_Quality, UE_Positioning_OTDOA_Quality_sequence);

  return offset;
}


static const per_sequence_t NeighbourQuality_sequence[] = {
  { &hf_rrc_ue_Positioning_OTDOA_Quality, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UE_Positioning_OTDOA_Quality },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_NeighbourQuality(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_NeighbourQuality, NeighbourQuality_sequence);

  return offset;
}


static const per_sequence_t UE_RX_TX_TimeDifferenceType2Info_sequence[] = {
  { &hf_rrc_ue_RX_TX_TimeDifferenceType2, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UE_RX_TX_TimeDifferenceType2 },
  { &hf_rrc_neighbourQuality, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_NeighbourQuality },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UE_RX_TX_TimeDifferenceType2Info(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UE_RX_TX_TimeDifferenceType2Info, UE_RX_TX_TimeDifferenceType2Info_sequence);

  return offset;
}


static const per_sequence_t T_fdd_171_sequence[] = {
  { &hf_rrc_referenceCellIDentity, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PrimaryCPICH_Info },
  { &hf_rrc_ue_RX_TX_TimeDifferenceType2Info, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UE_RX_TX_TimeDifferenceType2Info },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_fdd_171(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_fdd_171, T_fdd_171_sequence);

  return offset;
}


static const per_sequence_t T_tdd_120_sequence[] = {
  { &hf_rrc_referenceCellIdentity, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CellParametersID },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_tdd_120(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_tdd_120, T_tdd_120_sequence);

  return offset;
}


static const value_string rrc_T_modeSpecificInfo_137_vals[] = {
  {   0, "fdd" },
  {   1, "tdd" },
  { 0, NULL }
};

static const per_choice_t T_modeSpecificInfo_137_choice[] = {
  {   0, &hf_rrc_fdd_177         , ASN1_NO_EXTENSIONS     , dissect_rrc_T_fdd_171 },
  {   1, &hf_rrc_tdd_128         , ASN1_NO_EXTENSIONS     , dissect_rrc_T_tdd_120 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_modeSpecificInfo_137(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_modeSpecificInfo_137, T_modeSpecificInfo_137_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t T_fdd_162_sequence[] = {
  { &hf_rrc_neighbourIdentity, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_PrimaryCPICH_Info },
  { &hf_rrc_uE_RX_TX_TimeDifferenceType2Info, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_RX_TX_TimeDifferenceType2Info },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_fdd_162(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_fdd_162, T_fdd_162_sequence);

  return offset;
}


static const per_sequence_t T_tdd_110_sequence[] = {
  { &hf_rrc_neighbourAndChannelIdentity, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CellAndChannelIdentity },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_tdd_110(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_tdd_110, T_tdd_110_sequence);

  return offset;
}


static const value_string rrc_T_modeSpecificInfo_123_vals[] = {
  {   0, "fdd" },
  {   1, "tdd" },
  { 0, NULL }
};

static const per_choice_t T_modeSpecificInfo_123_choice[] = {
  {   0, &hf_rrc_fdd_168         , ASN1_NO_EXTENSIONS     , dissect_rrc_T_fdd_162 },
  {   1, &hf_rrc_tdd_118         , ASN1_NO_EXTENSIONS     , dissect_rrc_T_tdd_110 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_modeSpecificInfo_123(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_modeSpecificInfo_123, T_modeSpecificInfo_123_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t Neighbour_sequence[] = {
  { &hf_rrc_modeSpecificInfo_123, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_modeSpecificInfo_123 },
  { &hf_rrc_neighbourQuality, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_NeighbourQuality },
  { &hf_rrc_sfn_SFN_ObsTimeDifference2, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SFN_SFN_ObsTimeDifference2 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_Neighbour(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_Neighbour, Neighbour_sequence);

  return offset;
}


static const per_sequence_t NeighbourList_sequence_of[1] = {
  { &hf_rrc_NeighbourList_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_Neighbour },
};

static int
dissect_rrc_NeighbourList(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_NeighbourList, NeighbourList_sequence_of,
                                                  1, maxCellMeas, FALSE);

  return offset;
}


static const per_sequence_t UE_Positioning_OTDOA_Measurement_sequence[] = {
  { &hf_rrc_sfn             , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_0_4095 },
  { &hf_rrc_modeSpecificInfo_137, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_modeSpecificInfo_137 },
  { &hf_rrc_neighbourList_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_NeighbourList },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UE_Positioning_OTDOA_Measurement(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UE_Positioning_OTDOA_Measurement, UE_Positioning_OTDOA_Measurement_sequence);

  return offset;
}



static int
dissect_rrc_INTEGER_0_16383(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            0U, 16383U, NULL, FALSE);

  return offset;
}


static const per_sequence_t T_ue_GPSTimingOfCell_sequence[] = {
  { &hf_rrc_ms_part_01      , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_0_16383 },
  { &hf_rrc_ls_part         , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_0_4294967295 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_ue_GPSTimingOfCell(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_ue_GPSTimingOfCell, T_ue_GPSTimingOfCell_sequence);

  return offset;
}


static const per_sequence_t T_fdd_183_sequence[] = {
  { &hf_rrc_referenceIdentity, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PrimaryCPICH_Info },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_fdd_183(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_fdd_183, T_fdd_183_sequence);

  return offset;
}


static const per_sequence_t T_tdd_132_sequence[] = {
  { &hf_rrc_referenceIdentity_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CellParametersID },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_tdd_132(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_tdd_132, T_tdd_132_sequence);

  return offset;
}


static const value_string rrc_T_modeSpecificInfo_148_vals[] = {
  {   0, "fdd" },
  {   1, "tdd" },
  { 0, NULL }
};

static const per_choice_t T_modeSpecificInfo_148_choice[] = {
  {   0, &hf_rrc_fdd_189         , ASN1_NO_EXTENSIONS     , dissect_rrc_T_fdd_183 },
  {   1, &hf_rrc_tdd_140         , ASN1_NO_EXTENSIONS     , dissect_rrc_T_tdd_132 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_modeSpecificInfo_148(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_modeSpecificInfo_148, T_modeSpecificInfo_148_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t UTRAN_GPSReferenceTimeResult_sequence[] = {
  { &hf_rrc_ue_GPSTimingOfCell, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_ue_GPSTimingOfCell },
  { &hf_rrc_modeSpecificInfo_148, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_modeSpecificInfo_148 },
  { &hf_rrc_sfn             , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_0_4095 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UTRAN_GPSReferenceTimeResult(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UTRAN_GPSReferenceTimeResult, UTRAN_GPSReferenceTimeResult_sequence);

  return offset;
}


static const per_sequence_t T_fdd_180_sequence[] = {
  { &hf_rrc_primaryCPICH_Info, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PrimaryCPICH_Info },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_fdd_180(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_fdd_180, T_fdd_180_sequence);

  return offset;
}


static const per_sequence_t T_tdd_129_sequence[] = {
  { &hf_rrc_cellAndChannelIdentity, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CellAndChannelIdentity },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_tdd_129(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_tdd_129, T_tdd_129_sequence);

  return offset;
}


static const value_string rrc_T_modeSpecificInfo_146_vals[] = {
  {   0, "fdd" },
  {   1, "tdd" },
  { 0, NULL }
};

static const per_choice_t T_modeSpecificInfo_146_choice[] = {
  {   0, &hf_rrc_fdd_186         , ASN1_NO_EXTENSIONS     , dissect_rrc_T_fdd_180 },
  {   1, &hf_rrc_tdd_137         , ASN1_NO_EXTENSIONS     , dissect_rrc_T_tdd_129 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_modeSpecificInfo_146(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_modeSpecificInfo_146, T_modeSpecificInfo_146_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t T_cell_Timing_sequence[] = {
  { &hf_rrc_sfn             , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_0_4095 },
  { &hf_rrc_modeSpecificInfo_146, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_modeSpecificInfo_146 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_cell_Timing(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_cell_Timing, T_cell_Timing_sequence);

  return offset;
}


static const value_string rrc_T_referenceTime_02_vals[] = {
  {   0, "utran-GPSReferenceTimeResult" },
  {   1, "gps-ReferenceTimeOnly" },
  {   2, "cell-Timing" },
  { 0, NULL }
};

static const per_choice_t T_referenceTime_02_choice[] = {
  {   0, &hf_rrc_utran_GPSReferenceTimeResult, ASN1_NO_EXTENSIONS     , dissect_rrc_UTRAN_GPSReferenceTimeResult },
  {   1, &hf_rrc_gps_ReferenceTimeOnly, ASN1_NO_EXTENSIONS     , dissect_rrc_GPS_TOW_1msec },
  {   2, &hf_rrc_cell_Timing     , ASN1_NO_EXTENSIONS     , dissect_rrc_T_cell_Timing },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_referenceTime_02(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_referenceTime_02, T_referenceTime_02_choice,
                                 NULL);

  return offset;
}


static const value_string rrc_T_latitudeSign_03_vals[] = {
  {   0, "north" },
  {   1, "south" },
  { 0, NULL }
};


static int
dissect_rrc_T_latitudeSign_03(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     2, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t EllipsoidPointUncertCircle_sequence[] = {
  { &hf_rrc_latitudeSign_03 , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_latitudeSign_03 },
  { &hf_rrc_latitude        , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_0_8388607 },
  { &hf_rrc_longitude       , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_M8388608_8388607 },
  { &hf_rrc_uncertaintyCode , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_0_127 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_EllipsoidPointUncertCircle(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_EllipsoidPointUncertCircle, EllipsoidPointUncertCircle_sequence);

  return offset;
}


static const value_string rrc_T_latitudeSign_04_vals[] = {
  {   0, "north" },
  {   1, "south" },
  { 0, NULL }
};


static int
dissect_rrc_T_latitudeSign_04(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     2, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t EllipsoidPointUncertEllipse_sequence[] = {
  { &hf_rrc_latitudeSign_04 , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_latitudeSign_04 },
  { &hf_rrc_latitude        , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_0_8388607 },
  { &hf_rrc_longitude       , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_M8388608_8388607 },
  { &hf_rrc_uncertaintySemiMajor, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_0_127 },
  { &hf_rrc_uncertaintySemiMinor, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_0_127 },
  { &hf_rrc_orientationMajorAxis, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_0_89 },
  { &hf_rrc_confidence      , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_0_100 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_EllipsoidPointUncertEllipse(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_EllipsoidPointUncertEllipse, EllipsoidPointUncertEllipse_sequence);

  return offset;
}


static const value_string rrc_PositionEstimate_vals[] = {
  {   0, "ellipsoidPoint" },
  {   1, "ellipsoidPointUncertCircle" },
  {   2, "ellipsoidPointUncertEllipse" },
  {   3, "ellipsoidPointAltitude" },
  {   4, "ellipsoidPointAltitudeEllipse" },
  { 0, NULL }
};

static const per_choice_t PositionEstimate_choice[] = {
  {   0, &hf_rrc_ellipsoidPoint  , ASN1_NO_EXTENSIONS     , dissect_rrc_EllipsoidPoint },
  {   1, &hf_rrc_ellipsoidPointUncertCircle, ASN1_NO_EXTENSIONS     , dissect_rrc_EllipsoidPointUncertCircle },
  {   2, &hf_rrc_ellipsoidPointUncertEllipse, ASN1_NO_EXTENSIONS     , dissect_rrc_EllipsoidPointUncertEllipse },
  {   3, &hf_rrc_ellipsoidPointAltitude, ASN1_NO_EXTENSIONS     , dissect_rrc_EllipsoidPointAltitude },
  {   4, &hf_rrc_ellipsoidPointAltitudeEllipse, ASN1_NO_EXTENSIONS     , dissect_rrc_EllipsoidPointAltitudeEllipsoide },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_PositionEstimate(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_PositionEstimate, PositionEstimate_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t UE_Positioning_PositionEstimateInfo_sequence[] = {
  { &hf_rrc_referenceTime_02, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_referenceTime_02 },
  { &hf_rrc_positionEstimate, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PositionEstimate },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UE_Positioning_PositionEstimateInfo(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UE_Positioning_PositionEstimateInfo, UE_Positioning_PositionEstimateInfo_sequence);

  return offset;
}


static const value_string rrc_T_referenceTime_01_vals[] = {
  {   0, "utran-GPSReferenceTimeResult" },
  {   1, "gps-ReferenceTimeOnly" },
  { 0, NULL }
};

static const per_choice_t T_referenceTime_01_choice[] = {
  {   0, &hf_rrc_utran_GPSReferenceTimeResult, ASN1_NO_EXTENSIONS     , dissect_rrc_UTRAN_GPSReferenceTimeResult },
  {   1, &hf_rrc_gps_ReferenceTimeOnly, ASN1_NO_EXTENSIONS     , dissect_rrc_GPS_TOW_1msec },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_referenceTime_01(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_referenceTime_01, T_referenceTime_01_choice,
                                 NULL);

  return offset;
}



static int
dissect_rrc_INTEGER_M32768_32768(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            -32768, 32768U, NULL, FALSE);

  return offset;
}


static const value_string rrc_MultipathIndicator_vals[] = {
  {   0, "nm" },
  {   1, "low" },
  {   2, "medium" },
  {   3, "high" },
  { 0, NULL }
};


static int
dissect_rrc_MultipathIndicator(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     4, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t GPS_MeasurementParam_sequence[] = {
  { &hf_rrc_satelliteID     , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_0_63 },
  { &hf_rrc_c_N0            , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_0_63 },
  { &hf_rrc_doppler_01      , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_M32768_32768 },
  { &hf_rrc_wholeGPS_Chips  , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_0_1022 },
  { &hf_rrc_fractionalGPS_Chips, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_0_1023 },
  { &hf_rrc_multipathIndicator_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MultipathIndicator },
  { &hf_rrc_pseudorangeRMS_Error, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_0_63 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_GPS_MeasurementParam(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_GPS_MeasurementParam, GPS_MeasurementParam_sequence);

  return offset;
}


static const per_sequence_t GPS_MeasurementParamList_sequence_of[1] = {
  { &hf_rrc_GPS_MeasurementParamList_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_GPS_MeasurementParam },
};

static int
dissect_rrc_GPS_MeasurementParamList(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_GPS_MeasurementParamList, GPS_MeasurementParamList_sequence_of,
                                                  1, maxSat, FALSE);

  return offset;
}


static const per_sequence_t UE_Positioning_GPS_MeasurementResults_sequence[] = {
  { &hf_rrc_referenceTime_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_referenceTime_01 },
  { &hf_rrc_gps_MeasurementParamList, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_GPS_MeasurementParamList },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UE_Positioning_GPS_MeasurementResults(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UE_Positioning_GPS_MeasurementResults, UE_Positioning_GPS_MeasurementResults_sequence);

  return offset;
}


static const value_string rrc_UE_Positioning_ErrorCause_vals[] = {
  {   0, "notEnoughOTDOA-Cells" },
  {   1, "notEnoughGPS-Satellites" },
  {   2, "assistanceDataMissing" },
  {   3, "notAccomplishedGPS-TimingOfCellFrames" },
  {   4, "undefinedError" },
  {   5, "requestDeniedByUser" },
  {   6, "notProcessedAndTimeout" },
  {   7, "referenceCellNotServingCell" },
  { 0, NULL }
};


static int
dissect_rrc_UE_Positioning_ErrorCause(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     8, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t SatData_sequence[] = {
  { &hf_rrc_satID           , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SatID },
  { &hf_rrc_iode            , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_IODE },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_SatData(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_SatData, SatData_sequence);

  return offset;
}


static const per_sequence_t SatDataList_sequence_of[1] = {
  { &hf_rrc_SatDataList_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SatData },
};

static int
dissect_rrc_SatDataList(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_SatDataList, SatDataList_sequence_of,
                                                  0, maxSat, FALSE);

  return offset;
}


static const per_sequence_t UE_Positioning_GPS_NavModelAddDataReq_sequence[] = {
  { &hf_rrc_gps_Week        , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_0_1023 },
  { &hf_rrc_gps_Toe         , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_0_255 },
  { &hf_rrc_tToeLimit       , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_0_15 },
  { &hf_rrc_satDataList     , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SatDataList },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UE_Positioning_GPS_NavModelAddDataReq(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UE_Positioning_GPS_NavModelAddDataReq, UE_Positioning_GPS_NavModelAddDataReq_sequence);

  return offset;
}


static const per_sequence_t UE_Positioning_GPS_AdditionalAssistanceDataRequest_sequence[] = {
  { &hf_rrc_almanacRequest  , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { &hf_rrc_utcModelRequest , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { &hf_rrc_ionosphericModelRequest, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { &hf_rrc_navigationModelRequest, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { &hf_rrc_dgpsCorrectionsRequest, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { &hf_rrc_referenceLocationRequest, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { &hf_rrc_referenceTimeRequest, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { &hf_rrc_aquisitionAssistanceRequest, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { &hf_rrc_realTimeIntegrityRequest, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { &hf_rrc_navModelAddDataRequest, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_Positioning_GPS_NavModelAddDataReq },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UE_Positioning_GPS_AdditionalAssistanceDataRequest(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UE_Positioning_GPS_AdditionalAssistanceDataRequest, UE_Positioning_GPS_AdditionalAssistanceDataRequest_sequence);

  return offset;
}


static const per_sequence_t UE_Positioning_Error_sequence[] = {
  { &hf_rrc_errorReason     , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UE_Positioning_ErrorCause },
  { &hf_rrc_ue_positioning_GPS_additionalAssistanceDataRequest, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_Positioning_GPS_AdditionalAssistanceDataRequest },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UE_Positioning_Error(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UE_Positioning_Error, UE_Positioning_Error_sequence);

  return offset;
}


static const per_sequence_t UE_Positioning_MeasuredResults_sequence[] = {
  { &hf_rrc_ue_positioning_OTDOA_Measurement, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_Positioning_OTDOA_Measurement },
  { &hf_rrc_ue_positioning_PositionEstimateInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_Positioning_PositionEstimateInfo },
  { &hf_rrc_ue_positioning_GPS_Measurement, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_Positioning_GPS_MeasurementResults },
  { &hf_rrc_ue_positioning_Error, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_Positioning_Error },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UE_Positioning_MeasuredResults(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UE_Positioning_MeasuredResults, UE_Positioning_MeasuredResults_sequence);

  return offset;
}


static const value_string rrc_MeasuredResults_vals[] = {
  {   0, "intraFreqMeasuredResultsList" },
  {   1, "interFreqMeasuredResultsList" },
  {   2, "interRATMeasuredResultsList" },
  {   3, "trafficVolumeMeasuredResultsList" },
  {   4, "qualityMeasuredResults" },
  {   5, "ue-InternalMeasuredResults" },
  {   6, "ue-positioning-MeasuredResults" },
  {   7, "spare" },
  { 0, NULL }
};

static const per_choice_t MeasuredResults_choice[] = {
  {   0, &hf_rrc_intraFreqMeasuredResultsList, ASN1_NO_EXTENSIONS     , dissect_rrc_IntraFreqMeasuredResultsList },
  {   1, &hf_rrc_interFreqMeasuredResultsList, ASN1_NO_EXTENSIONS     , dissect_rrc_InterFreqMeasuredResultsList },
  {   2, &hf_rrc_interRATMeasuredResultsList, ASN1_NO_EXTENSIONS     , dissect_rrc_InterRATMeasuredResultsList },
  {   3, &hf_rrc_trafficVolumeMeasuredResultsList, ASN1_NO_EXTENSIONS     , dissect_rrc_TrafficVolumeMeasuredResultsList },
  {   4, &hf_rrc_qualityMeasuredResults, ASN1_NO_EXTENSIONS     , dissect_rrc_QualityMeasuredResults },
  {   5, &hf_rrc_ue_InternalMeasuredResults, ASN1_NO_EXTENSIONS     , dissect_rrc_UE_InternalMeasuredResults },
  {   6, &hf_rrc_ue_positioning_MeasuredResults, ASN1_NO_EXTENSIONS     , dissect_rrc_UE_Positioning_MeasuredResults },
  {   7, &hf_rrc_spare           , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_MeasuredResults(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_MeasuredResults, MeasuredResults_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t MeasuredResultsList_sequence_of[1] = {
  { &hf_rrc_MeasuredResultsList_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MeasuredResults },
};

static int
dissect_rrc_MeasuredResultsList(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_MeasuredResultsList, MeasuredResultsList_sequence_of,
                                                  1, maxAdditionalMeas, FALSE);

  return offset;
}


static const value_string rrc_EventIDIntraFreq_vals[] = {
  {   0, "e1a" },
  {   1, "e1b" },
  {   2, "e1c" },
  {   3, "e1d" },
  {   4, "e1e" },
  {   5, "e1f" },
  {   6, "e1g" },
  {   7, "e1h" },
  {   8, "e1i" },
  {   9, "e1j" },
  {  10, "spare6" },
  {  11, "spare5" },
  {  12, "spare4" },
  {  13, "spare3" },
  {  14, "spare2" },
  {  15, "spare1" },
  { 0, NULL }
};


static int
dissect_rrc_EventIDIntraFreq(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     16, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t SEQUENCE_SIZE_1_maxCellMeas_OF_PrimaryCPICH_Info_sequence_of[1] = {
  { &hf_rrc_fdd_item_01     , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PrimaryCPICH_Info },
};

static int
dissect_rrc_SEQUENCE_SIZE_1_maxCellMeas_OF_PrimaryCPICH_Info(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_SEQUENCE_SIZE_1_maxCellMeas_OF_PrimaryCPICH_Info, SEQUENCE_SIZE_1_maxCellMeas_OF_PrimaryCPICH_Info_sequence_of,
                                                  1, maxCellMeas, FALSE);

  return offset;
}


static const per_sequence_t SEQUENCE_SIZE_1_maxCellMeas_OF_PrimaryCCPCH_Info_sequence_of[1] = {
  { &hf_rrc_tdd_item_02     , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PrimaryCCPCH_Info },
};

static int
dissect_rrc_SEQUENCE_SIZE_1_maxCellMeas_OF_PrimaryCCPCH_Info(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_SEQUENCE_SIZE_1_maxCellMeas_OF_PrimaryCCPCH_Info, SEQUENCE_SIZE_1_maxCellMeas_OF_PrimaryCCPCH_Info_sequence_of,
                                                  1, maxCellMeas, FALSE);

  return offset;
}


static const value_string rrc_CellMeasurementEventResults_vals[] = {
  {   0, "fdd" },
  {   1, "tdd" },
  { 0, NULL }
};

static const per_choice_t CellMeasurementEventResults_choice[] = {
  {   0, &hf_rrc_fdd_144         , ASN1_NO_EXTENSIONS     , dissect_rrc_SEQUENCE_SIZE_1_maxCellMeas_OF_PrimaryCPICH_Info },
  {   1, &hf_rrc_tdd_98          , ASN1_NO_EXTENSIONS     , dissect_rrc_SEQUENCE_SIZE_1_maxCellMeas_OF_PrimaryCCPCH_Info },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_CellMeasurementEventResults(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_CellMeasurementEventResults, CellMeasurementEventResults_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t IntraFreqEventResults_sequence[] = {
  { &hf_rrc_eventID_02      , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_EventIDIntraFreq },
  { &hf_rrc_cellMeasurementEventResults, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CellMeasurementEventResults },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_IntraFreqEventResults(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_IntraFreqEventResults, IntraFreqEventResults_sequence);

  return offset;
}


static const value_string rrc_EventIDInterFreq_vals[] = {
  {   0, "e2a" },
  {   1, "e2b" },
  {   2, "e2c" },
  {   3, "e2d" },
  {   4, "e2e" },
  {   5, "e2f" },
  {   6, "spare2" },
  {   7, "spare1" },
  { 0, NULL }
};


static int
dissect_rrc_EventIDInterFreq(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     8, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t InterFreqCell_sequence[] = {
  { &hf_rrc_frequencyInfo   , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_FrequencyInfo },
  { &hf_rrc_nonFreqRelatedEventResults, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CellMeasurementEventResults },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_InterFreqCell(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_InterFreqCell, InterFreqCell_sequence);

  return offset;
}


static const per_sequence_t InterFreqCellList_sequence_of[1] = {
  { &hf_rrc_InterFreqCellList_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_InterFreqCell },
};

static int
dissect_rrc_InterFreqCellList(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_InterFreqCellList, InterFreqCellList_sequence_of,
                                                  1, maxFreq, FALSE);

  return offset;
}


static const per_sequence_t InterFreqEventResults_sequence[] = {
  { &hf_rrc_eventID_01      , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_EventIDInterFreq },
  { &hf_rrc_interFreqCellList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_InterFreqCellList },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_InterFreqEventResults(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_InterFreqEventResults, InterFreqEventResults_sequence);

  return offset;
}


static const value_string rrc_EventIDInterRAT_vals[] = {
  {   0, "e3a" },
  {   1, "e3b" },
  {   2, "e3c" },
  {   3, "e3d" },
  { 0, NULL }
};


static int
dissect_rrc_EventIDInterRAT(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     4, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t CellToReport_sequence[] = {
  { &hf_rrc_bsicReported    , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BSICReported },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_CellToReport(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_CellToReport, CellToReport_sequence);

  return offset;
}


static const per_sequence_t CellToReportList_sequence_of[1] = {
  { &hf_rrc_CellToReportList_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CellToReport },
};

static int
dissect_rrc_CellToReportList(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_CellToReportList, CellToReportList_sequence_of,
                                                  1, maxCellMeas, FALSE);

  return offset;
}


static const per_sequence_t InterRATEventResults_sequence[] = {
  { &hf_rrc_eventID         , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_EventIDInterRAT },
  { &hf_rrc_cellToReportList, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CellToReportList },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_InterRATEventResults(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_InterRATEventResults, InterRATEventResults_sequence);

  return offset;
}


static const per_sequence_t TrafficVolumeEventResults_sequence[] = {
  { &hf_rrc_ul_transportChannelCausingEvent, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UL_TrCH_Identity },
  { &hf_rrc_trafficVolumeEventIdentity, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TrafficVolumeEventType },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_TrafficVolumeEventResults(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_TrafficVolumeEventResults, TrafficVolumeEventResults_sequence);

  return offset;
}


static const per_sequence_t QualityEventResults_sequence_of[1] = {
  { &hf_rrc_QualityEventResults_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TransportChannelIdentity },
};

static int
dissect_rrc_QualityEventResults(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_QualityEventResults, QualityEventResults_sequence_of,
                                                  1, maxTrCH, FALSE);

  return offset;
}


static const value_string rrc_UE_InternalEventResults_vals[] = {
  {   0, "event6a" },
  {   1, "event6b" },
  {   2, "event6c" },
  {   3, "event6d" },
  {   4, "event6e" },
  {   5, "event6f" },
  {   6, "event6g" },
  {   7, "spare" },
  { 0, NULL }
};

static const per_choice_t UE_InternalEventResults_choice[] = {
  {   0, &hf_rrc_event6a_01      , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   1, &hf_rrc_event6b_01      , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   2, &hf_rrc_event6c_01      , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   3, &hf_rrc_event6d_01      , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   4, &hf_rrc_event6e_01      , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   5, &hf_rrc_event6f_01      , ASN1_NO_EXTENSIONS     , dissect_rrc_PrimaryCPICH_Info },
  {   6, &hf_rrc_event6g_01      , ASN1_NO_EXTENSIONS     , dissect_rrc_PrimaryCPICH_Info },
  {   7, &hf_rrc_spare           , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_UE_InternalEventResults(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_UE_InternalEventResults, UE_InternalEventResults_choice,
                                 NULL);

  return offset;
}


static const value_string rrc_UE_Positioning_MeasurementEventResults_vals[] = {
  {   0, "event7a" },
  {   1, "event7b" },
  {   2, "event7c" },
  {   3, "additionalOrReplacedPosMeasEvent" },
  { 0, NULL }
};

static const per_choice_t UE_Positioning_MeasurementEventResults_choice[] = {
  {   0, &hf_rrc_event7a         , ASN1_NO_EXTENSIONS     , dissect_rrc_UE_Positioning_PositionEstimateInfo },
  {   1, &hf_rrc_event7b         , ASN1_NO_EXTENSIONS     , dissect_rrc_UE_Positioning_OTDOA_Measurement },
  {   2, &hf_rrc_event7c         , ASN1_NO_EXTENSIONS     , dissect_rrc_UE_Positioning_GPS_MeasurementResults },
  {   3, &hf_rrc_additionalOrReplacedPosMeasEvent, ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_UE_Positioning_MeasurementEventResults(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_UE_Positioning_MeasurementEventResults, UE_Positioning_MeasurementEventResults_choice,
                                 NULL);

  return offset;
}


static const value_string rrc_EventResults_vals[] = {
  {   0, "intraFreqEventResults" },
  {   1, "interFreqEventResults" },
  {   2, "interRATEventResults" },
  {   3, "trafficVolumeEventResults" },
  {   4, "qualityEventResults" },
  {   5, "ue-InternalEventResults" },
  {   6, "ue-positioning-MeasurementEventResults" },
  {   7, "spare" },
  { 0, NULL }
};

static const per_choice_t EventResults_choice[] = {
  {   0, &hf_rrc_intraFreqEventResults, ASN1_NO_EXTENSIONS     , dissect_rrc_IntraFreqEventResults },
  {   1, &hf_rrc_interFreqEventResults, ASN1_NO_EXTENSIONS     , dissect_rrc_InterFreqEventResults },
  {   2, &hf_rrc_interRATEventResults, ASN1_NO_EXTENSIONS     , dissect_rrc_InterRATEventResults },
  {   3, &hf_rrc_trafficVolumeEventResults, ASN1_NO_EXTENSIONS     , dissect_rrc_TrafficVolumeEventResults },
  {   4, &hf_rrc_qualityEventResults, ASN1_NO_EXTENSIONS     , dissect_rrc_QualityEventResults },
  {   5, &hf_rrc_ue_InternalEventResults, ASN1_NO_EXTENSIONS     , dissect_rrc_UE_InternalEventResults },
  {   6, &hf_rrc_ue_positioning_MeasurementEventResults, ASN1_NO_EXTENSIONS     , dissect_rrc_UE_Positioning_MeasurementEventResults },
  {   7, &hf_rrc_spare           , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_EventResults(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_EventResults, EventResults_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t T_fdd_163_sequence[] = {
  { &hf_rrc_frequencyInfo   , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_FrequencyInfo },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_fdd_163(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_fdd_163, T_fdd_163_sequence);

  return offset;
}


static const value_string rrc_T_modeSpecificInfo_124_vals[] = {
  {   0, "fdd" },
  {   1, "tdd" },
  { 0, NULL }
};

static const per_choice_t T_modeSpecificInfo_124_choice[] = {
  {   0, &hf_rrc_fdd_169         , ASN1_NO_EXTENSIONS     , dissect_rrc_T_fdd_163 },
  {   1, &hf_rrc_tdd             , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_modeSpecificInfo_124(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_modeSpecificInfo_124, T_modeSpecificInfo_124_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t Neighbour_v390ext_sequence[] = {
  { &hf_rrc_modeSpecificInfo_124, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_modeSpecificInfo_124 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_Neighbour_v390ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_Neighbour_v390ext, Neighbour_v390ext_sequence);

  return offset;
}


static const per_sequence_t NeighbourList_v390ext_sequence_of[1] = {
  { &hf_rrc_NeighbourList_v390ext_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_Neighbour_v390ext },
};

static int
dissect_rrc_NeighbourList_v390ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_NeighbourList_v390ext, NeighbourList_v390ext_sequence_of,
                                                  1, maxCellMeas, FALSE);

  return offset;
}


static const per_sequence_t UE_Positioning_OTDOA_Measurement_v390ext_sequence[] = {
  { &hf_rrc_neighbourList_v390ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_NeighbourList_v390ext },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UE_Positioning_OTDOA_Measurement_v390ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UE_Positioning_OTDOA_Measurement_v390ext, UE_Positioning_OTDOA_Measurement_v390ext_sequence);

  return offset;
}


static const per_sequence_t UE_Positioning_MeasuredResults_v390ext_sequence[] = {
  { &hf_rrc_ue_Positioning_OTDOA_Measurement_v390ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UE_Positioning_OTDOA_Measurement_v390ext },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UE_Positioning_MeasuredResults_v390ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UE_Positioning_MeasuredResults_v390ext, UE_Positioning_MeasuredResults_v390ext_sequence);

  return offset;
}


static const per_sequence_t MeasuredResults_v390ext_sequence[] = {
  { &hf_rrc_ue_positioning_MeasuredResults_v390ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UE_Positioning_MeasuredResults_v390ext },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_MeasuredResults_v390ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_MeasuredResults_v390ext, MeasuredResults_v390ext_sequence);

  return offset;
}


static const per_sequence_t MeasurementReport_v390ext_sequence[] = {
  { &hf_rrc_measuredResults_v390ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MeasuredResults_v390ext },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_MeasurementReport_v390ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_MeasurementReport_v390ext, MeasurementReport_v390ext_sequence);

  return offset;
}


static const per_sequence_t PrimaryCCPCH_Info_LCR_r4_sequence[] = {
  { &hf_rrc_tstd_Indicator  , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { &hf_rrc_cellParametersID, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CellParametersID },
  { &hf_rrc_sctd_Indicator  , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_PrimaryCCPCH_Info_LCR_r4(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_PrimaryCCPCH_Info_LCR_r4, PrimaryCCPCH_Info_LCR_r4_sequence);

  return offset;
}


static const per_sequence_t CellMeasurementEventResults_LCR_r4_sequence_of[1] = {
  { &hf_rrc_CellMeasurementEventResults_LCR_r4_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PrimaryCCPCH_Info_LCR_r4 },
};

static int
dissect_rrc_CellMeasurementEventResults_LCR_r4(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_CellMeasurementEventResults_LCR_r4, CellMeasurementEventResults_LCR_r4_sequence_of,
                                                  1, maxCellMeas, FALSE);

  return offset;
}


static const per_sequence_t InterFreqCell_LCR_r4_sequence[] = {
  { &hf_rrc_frequencyInfo   , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_FrequencyInfo },
  { &hf_rrc_nonFreqRelatedEventResults_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CellMeasurementEventResults_LCR_r4 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_InterFreqCell_LCR_r4(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_InterFreqCell_LCR_r4, InterFreqCell_LCR_r4_sequence);

  return offset;
}


static const per_sequence_t InterFreqCellList_LCR_r4_ext_sequence_of[1] = {
  { &hf_rrc_InterFreqCellList_LCR_r4_ext_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_InterFreqCell_LCR_r4 },
};

static int
dissect_rrc_InterFreqCellList_LCR_r4_ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_InterFreqCellList_LCR_r4_ext, InterFreqCellList_LCR_r4_ext_sequence_of,
                                                  1, maxFreq, FALSE);

  return offset;
}


static const per_sequence_t InterFreqEventResults_LCR_r4_ext_sequence[] = {
  { &hf_rrc_eventID_01      , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_EventIDInterFreq },
  { &hf_rrc_interFreqCellList_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_InterFreqCellList_LCR_r4_ext },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_InterFreqEventResults_LCR_r4_ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_InterFreqEventResults_LCR_r4_ext, InterFreqEventResults_LCR_r4_ext_sequence);

  return offset;
}


static const per_sequence_t T_ADVinfo_sequence[] = {
  { &hf_rrc_t_ADV           , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_0_2047 },
  { &hf_rrc_sfn             , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_0_4095 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_ADVinfo(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_ADVinfo, T_ADVinfo_sequence);

  return offset;
}


static const per_sequence_t UE_InternalMeasuredResults_LCR_r4_sequence[] = {
  { &hf_rrc_ue_TransmittedPowerTDD_List, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_TransmittedPowerTDD_List },
  { &hf_rrc_t_ADVinfo       , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_ADVinfo },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UE_InternalMeasuredResults_LCR_r4(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UE_InternalMeasuredResults_LCR_r4, UE_InternalMeasuredResults_LCR_r4_sequence);

  return offset;
}


static const value_string rrc_MeasuredResults_LCR_r4_vals[] = {
  {   0, "intraFreqMeasuredResultsList" },
  {   1, "interFreqMeasuredResultsList" },
  {   2, "interRATMeasuredResultsList" },
  {   3, "trafficVolumeMeasuredResultsList" },
  {   4, "qualityMeasuredResults" },
  {   5, "ue-InternalMeasuredResults" },
  {   6, "ue-positioniing-MeasuredResults" },
  {   7, "spare" },
  { 0, NULL }
};

static const per_choice_t MeasuredResults_LCR_r4_choice[] = {
  {   0, &hf_rrc_intraFreqMeasuredResultsList, ASN1_NO_EXTENSIONS     , dissect_rrc_IntraFreqMeasuredResultsList },
  {   1, &hf_rrc_interFreqMeasuredResultsList, ASN1_NO_EXTENSIONS     , dissect_rrc_InterFreqMeasuredResultsList },
  {   2, &hf_rrc_interRATMeasuredResultsList, ASN1_NO_EXTENSIONS     , dissect_rrc_InterRATMeasuredResultsList },
  {   3, &hf_rrc_trafficVolumeMeasuredResultsList, ASN1_NO_EXTENSIONS     , dissect_rrc_TrafficVolumeMeasuredResultsList },
  {   4, &hf_rrc_qualityMeasuredResults, ASN1_NO_EXTENSIONS     , dissect_rrc_QualityMeasuredResults },
  {   5, &hf_rrc_ue_InternalMeasuredResults_01, ASN1_NO_EXTENSIONS     , dissect_rrc_UE_InternalMeasuredResults_LCR_r4 },
  {   6, &hf_rrc_ue_positioniing_MeasuredResults, ASN1_NO_EXTENSIONS     , dissect_rrc_UE_Positioning_MeasuredResults },
  {   7, &hf_rrc_spare           , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_MeasuredResults_LCR_r4(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_MeasuredResults_LCR_r4, MeasuredResults_LCR_r4_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t MeasuredResultsList_LCR_r4_ext_sequence_of[1] = {
  { &hf_rrc_MeasuredResultsList_LCR_r4_ext_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MeasuredResults_LCR_r4 },
};

static int
dissect_rrc_MeasuredResultsList_LCR_r4_ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_MeasuredResultsList_LCR_r4_ext, MeasuredResultsList_LCR_r4_ext_sequence_of,
                                                  1, maxAdditionalMeas, FALSE);

  return offset;
}


static const per_sequence_t MeasurementReport_v4b0ext_IEs_sequence[] = {
  { &hf_rrc_interFreqEventResults_LCR, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_InterFreqEventResults_LCR_r4_ext },
  { &hf_rrc_additionalMeasuredResults_LCR, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MeasuredResultsList_LCR_r4_ext },
  { &hf_rrc_dummy_12        , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_PrimaryCPICH_Info },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_MeasurementReport_v4b0ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_MeasurementReport_v4b0ext_IEs, MeasurementReport_v4b0ext_IEs_sequence);

  return offset;
}


static const per_sequence_t IntraFrequencyMeasuredResultsList_v590ext_sequence_of[1] = {
  { &hf_rrc_IntraFrequencyMeasuredResultsList_v590ext_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DeltaRSCPPerCell },
};

static int
dissect_rrc_IntraFrequencyMeasuredResultsList_v590ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_IntraFrequencyMeasuredResultsList_v590ext, IntraFrequencyMeasuredResultsList_v590ext_sequence_of,
                                                  1, maxCellMeas, FALSE);

  return offset;
}


static const per_sequence_t InterFrequencyMeasuredResultsList_v590ext_sequence_of[1] = {
  { &hf_rrc_InterFrequencyMeasuredResultsList_v590ext_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DeltaRSCPPerCell },
};

static int
dissect_rrc_InterFrequencyMeasuredResultsList_v590ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_InterFrequencyMeasuredResultsList_v590ext, InterFrequencyMeasuredResultsList_v590ext_sequence_of,
                                                  1, maxCellMeas, FALSE);

  return offset;
}


static const value_string rrc_MeasuredResults_v590ext_vals[] = {
  {   0, "intraFrequencyMeasuredResultsList" },
  {   1, "interFrequencyMeasuredResultsList" },
  { 0, NULL }
};

static const per_choice_t MeasuredResults_v590ext_choice[] = {
  {   0, &hf_rrc_intraFrequencyMeasuredResultsList, ASN1_NO_EXTENSIONS     , dissect_rrc_IntraFrequencyMeasuredResultsList_v590ext },
  {   1, &hf_rrc_interFrequencyMeasuredResultsList, ASN1_NO_EXTENSIONS     , dissect_rrc_InterFrequencyMeasuredResultsList_v590ext },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_MeasuredResults_v590ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_MeasuredResults_v590ext, MeasuredResults_v590ext_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t MeasurementReport_v590ext_IEs_sequence[] = {
  { &hf_rrc_measuredResults_v590ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MeasuredResults_v590ext },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_MeasurementReport_v590ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_MeasurementReport_v590ext_IEs, MeasurementReport_v590ext_IEs_sequence);

  return offset;
}


static const per_sequence_t MeasurementReport_v5b0ext_IEs_sequence[] = {
  { &hf_rrc_interRATCellInfoIndication, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_InterRATCellInfoIndication },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_MeasurementReport_v5b0ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_MeasurementReport_v5b0ext_IEs, MeasurementReport_v5b0ext_IEs_sequence);

  return offset;
}


static const per_sequence_t MeasurementReport_v690ext_IEs_sequence[] = {
  { &hf_rrc_measuredResultsOnRACHinterFreq, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MeasuredResultsOnRACHinterFreq },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_MeasurementReport_v690ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_MeasurementReport_v690ext_IEs, MeasurementReport_v690ext_IEs_sequence);

  return offset;
}


static const per_sequence_t T_tdd384_768_02_sequence[] = {
  { &hf_rrc_appliedTA_01    , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_EXT_UL_TimingAdvance },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_tdd384_768_02(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_tdd384_768_02, T_tdd384_768_02_sequence);

  return offset;
}


static const per_sequence_t T_ADVinfo_ext_sequence[] = {
  { &hf_rrc_t_ADV_01        , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_0_8191 },
  { &hf_rrc_sfn             , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_0_4095 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_ADVinfo_ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_ADVinfo_ext, T_ADVinfo_ext_sequence);

  return offset;
}


static const per_sequence_t T_tdd128_66_sequence[] = {
  { &hf_rrc_t_ADVinfo_01    , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_ADVinfo_ext },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_tdd128_66(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_tdd128_66, T_tdd128_66_sequence);

  return offset;
}


static const value_string rrc_T_modeSpecificInfo_132_vals[] = {
  {   0, "tdd384-768" },
  {   1, "tdd128" },
  { 0, NULL }
};

static const per_choice_t T_modeSpecificInfo_132_choice[] = {
  {   0, &hf_rrc_tdd384_768_03   , ASN1_NO_EXTENSIONS     , dissect_rrc_T_tdd384_768_02 },
  {   1, &hf_rrc_tdd128_69       , ASN1_NO_EXTENSIONS     , dissect_rrc_T_tdd128_66 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_modeSpecificInfo_132(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_modeSpecificInfo_132, T_modeSpecificInfo_132_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t UE_InternalMeasuredResults_v770ext_sequence[] = {
  { &hf_rrc_modeSpecificInfo_132, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_modeSpecificInfo_132 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UE_InternalMeasuredResults_v770ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UE_InternalMeasuredResults_v770ext, UE_InternalMeasuredResults_v770ext_sequence);

  return offset;
}



static int
dissect_rrc_INTEGER_0_80(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            0U, 80U, NULL, FALSE);

  return offset;
}


static const per_sequence_t T_ue_GANSSTimingOfCellFrames_sequence[] = {
  { &hf_rrc_ms_Part         , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_0_80 },
  { &hf_rrc_ls_Part         , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_0_4294967295 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_ue_GANSSTimingOfCellFrames(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_ue_GANSSTimingOfCellFrames, T_ue_GANSSTimingOfCellFrames_sequence);

  return offset;
}


static const per_sequence_t T_fdd_181_sequence[] = {
  { &hf_rrc_primary_CPICH_Info, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PrimaryCPICH_Info },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_fdd_181(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_fdd_181, T_fdd_181_sequence);

  return offset;
}


static const per_sequence_t T_tdd_130_sequence[] = {
  { &hf_rrc_cellParameters  , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CellParametersID },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_tdd_130(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_tdd_130, T_tdd_130_sequence);

  return offset;
}


static const value_string rrc_T_mode_02_vals[] = {
  {   0, "fdd" },
  {   1, "tdd" },
  { 0, NULL }
};

static const per_choice_t T_mode_02_choice[] = {
  {   0, &hf_rrc_fdd_187         , ASN1_NO_EXTENSIONS     , dissect_rrc_T_fdd_181 },
  {   1, &hf_rrc_tdd_138         , ASN1_NO_EXTENSIONS     , dissect_rrc_T_tdd_130 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_mode_02(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_mode_02, T_mode_02_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t UTRAN_GANSSReferenceTime_sequence[] = {
  { &hf_rrc_ue_GANSSTimingOfCellFrames, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_ue_GANSSTimingOfCellFrames },
  { &hf_rrc_gANSS_TimeId    , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_INTEGER_0_7 },
  { &hf_rrc_gANSS_TimeUncertainty, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_INTEGER_0_127 },
  { &hf_rrc_mode_02         , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_mode_02 },
  { &hf_rrc_referenceSfn    , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_0_4095 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UTRAN_GANSSReferenceTime(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UTRAN_GANSSReferenceTime, UTRAN_GANSSReferenceTime_sequence);

  return offset;
}



static int
dissect_rrc_INTEGER_0_3599999(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            0U, 3599999U, NULL, FALSE);

  return offset;
}


static const per_sequence_t GANSSReferenceTimeOnly_sequence[] = {
  { &hf_rrc_gANSS_tod       , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_0_3599999 },
  { &hf_rrc_gANSS_timeId    , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_INTEGER_0_7 },
  { &hf_rrc_gANSS_tod_uncertainty, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_INTEGER_0_127 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_GANSSReferenceTimeOnly(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_GANSSReferenceTimeOnly, GANSSReferenceTimeOnly_sequence);

  return offset;
}


static const value_string rrc_T_referenceTime_03_vals[] = {
  {   0, "utran-GANSSReferenceTimeResult" },
  {   1, "ganssReferenceTimeOnly" },
  { 0, NULL }
};

static const per_choice_t T_referenceTime_03_choice[] = {
  {   0, &hf_rrc_utran_GANSSReferenceTimeResult, ASN1_NO_EXTENSIONS     , dissect_rrc_UTRAN_GANSSReferenceTime },
  {   1, &hf_rrc_ganssReferenceTimeOnly, ASN1_NO_EXTENSIONS     , dissect_rrc_GANSSReferenceTimeOnly },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_referenceTime_03(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_referenceTime_03, T_referenceTime_03_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t T_r7_15_sequence[] = {
  { &hf_rrc_referenceTime_03, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_referenceTime_03 },
  { &hf_rrc_positionEstimate, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PositionEstimate },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_r7_15(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_r7_15, T_r7_15_sequence);

  return offset;
}


static const value_string rrc_T_referenceTimeOptions_vals[] = {
  {   0, "earlier-than-r7" },
  {   1, "r7" },
  { 0, NULL }
};

static const per_choice_t T_referenceTimeOptions_choice[] = {
  {   0, &hf_rrc_earlier_than_r7 , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   1, &hf_rrc_r7_15           , ASN1_NO_EXTENSIONS     , dissect_rrc_T_r7_15 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_referenceTimeOptions(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_referenceTimeOptions, T_referenceTimeOptions_choice,
                                 NULL);

  return offset;
}



static int
dissect_rrc_INTEGER_0_359(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            0U, 359U, NULL, FALSE);

  return offset;
}


static const per_sequence_t HorizontalVelocity_sequence[] = {
  { &hf_rrc_bearing         , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_0_359 },
  { &hf_rrc_horizontalSpeed , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_0_2047 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_HorizontalVelocity(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_HorizontalVelocity, HorizontalVelocity_sequence);

  return offset;
}


static const value_string rrc_T_verticalSpeedDirection_vals[] = {
  {   0, "upward" },
  {   1, "downward" },
  { 0, NULL }
};


static int
dissect_rrc_T_verticalSpeedDirection(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     2, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t HorizontalWithVerticalVelocity_sequence[] = {
  { &hf_rrc_verticalSpeedDirection, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_verticalSpeedDirection },
  { &hf_rrc_bearing         , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_0_359 },
  { &hf_rrc_horizontalSpeed , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_0_2047 },
  { &hf_rrc_verticalSpeed   , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_0_255 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_HorizontalWithVerticalVelocity(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_HorizontalWithVerticalVelocity, HorizontalWithVerticalVelocity_sequence);

  return offset;
}


static const per_sequence_t HorizontalVelocityWithUncertainty_sequence[] = {
  { &hf_rrc_bearing         , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_0_359 },
  { &hf_rrc_horizontalSpeed , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_0_2047 },
  { &hf_rrc_horizontalSpeedUncertainty, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_0_255 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_HorizontalVelocityWithUncertainty(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_HorizontalVelocityWithUncertainty, HorizontalVelocityWithUncertainty_sequence);

  return offset;
}


static const value_string rrc_T_verticalSpeedDirection_01_vals[] = {
  {   0, "upward" },
  {   1, "downward" },
  { 0, NULL }
};


static int
dissect_rrc_T_verticalSpeedDirection_01(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     2, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t HorizontalWithVerticalVelocityAndUncertainty_sequence[] = {
  { &hf_rrc_verticalSpeedDirection_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_verticalSpeedDirection_01 },
  { &hf_rrc_bearing         , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_0_359 },
  { &hf_rrc_horizontalSpeed , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_0_2047 },
  { &hf_rrc_verticalSpeed   , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_0_255 },
  { &hf_rrc_horizontalUncertaintySpeed, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_0_255 },
  { &hf_rrc_verticalUncertaintySpeed, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_0_255 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_HorizontalWithVerticalVelocityAndUncertainty(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_HorizontalWithVerticalVelocityAndUncertainty, HorizontalWithVerticalVelocityAndUncertainty_sequence);

  return offset;
}


static const value_string rrc_VelocityEstimate_vals[] = {
  {   0, "horizontalVelocity" },
  {   1, "horizontalWithVerticalVelocity" },
  {   2, "horizontalVelocityWithUncertainty" },
  {   3, "horizontalWithVerticalVelocityAndUncertainty" },
  { 0, NULL }
};

static const per_choice_t VelocityEstimate_choice[] = {
  {   0, &hf_rrc_horizontalVelocity, ASN1_NO_EXTENSIONS     , dissect_rrc_HorizontalVelocity },
  {   1, &hf_rrc_horizontalWithVerticalVelocity, ASN1_NO_EXTENSIONS     , dissect_rrc_HorizontalWithVerticalVelocity },
  {   2, &hf_rrc_horizontalVelocityWithUncertainty, ASN1_NO_EXTENSIONS     , dissect_rrc_HorizontalVelocityWithUncertainty },
  {   3, &hf_rrc_horizontalWithVerticalVelocityAndUncertainty, ASN1_NO_EXTENSIONS     , dissect_rrc_HorizontalWithVerticalVelocityAndUncertainty },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_VelocityEstimate(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_VelocityEstimate, VelocityEstimate_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t UE_Positioning_PositionEstimateInfo_v770ext_sequence[] = {
  { &hf_rrc_referenceTimeOptions, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_referenceTimeOptions },
  { &hf_rrc_positionData    , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_16 },
  { &hf_rrc_velocityEstimate, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_VelocityEstimate },
  { &hf_rrc_ue_Positioning_GPS_ReferenceTimeUncertainty, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_Positioning_GPS_ReferenceTimeUncertainty },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UE_Positioning_PositionEstimateInfo_v770ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UE_Positioning_PositionEstimateInfo_v770ext, UE_Positioning_PositionEstimateInfo_v770ext_sequence);

  return offset;
}


static const per_sequence_t UE_Positioning_GPS_MeasurementResults_v770ext_sequence[] = {
  { &hf_rrc_ue_Positioning_GPS_ReferenceTimeUncertainty, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_Positioning_GPS_ReferenceTimeUncertainty },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UE_Positioning_GPS_MeasurementResults_v770ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UE_Positioning_GPS_MeasurementResults_v770ext, UE_Positioning_GPS_MeasurementResults_v770ext_sequence);

  return offset;
}


static const value_string rrc_UE_Positioning_ErrorCause_r7_vals[] = {
  {   0, "notEnoughOTDOA-Cells" },
  {   1, "notEnoughGPS-Satellites" },
  {   2, "assistanceDataMissing" },
  {   3, "notAccomplishedGPS-TimingOfCellFrames" },
  {   4, "undefinedError" },
  {   5, "requestDeniedByUser" },
  {   6, "notProcessedAndTimeout" },
  {   7, "referenceCellNotServingCell" },
  {   8, "notEnoughGANSS-Satellites" },
  {   9, "notAccomplishedGANSS-TimingOfCellFrames" },
  {  10, "spare6" },
  {  11, "spare5" },
  {  12, "spare4" },
  {  13, "spare3" },
  {  14, "spare2" },
  {  15, "spare1" },
  { 0, NULL }
};


static int
dissect_rrc_UE_Positioning_ErrorCause_r7(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     16, NULL, FALSE, 0, NULL);

  return offset;
}



static int
dissect_rrc_DGANSS_Sig_Id_Req(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_bit_string(tvb, offset, actx, tree, hf_index,
                                     8, 8, FALSE, NULL);

  return offset;
}


static const per_sequence_t T_ganssSatelliteInfo_sequence_of[1] = {
  { &hf_rrc_ganssSatelliteInfo_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_0_63 },
};

static int
dissect_rrc_T_ganssSatelliteInfo(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_T_ganssSatelliteInfo, T_ganssSatelliteInfo_sequence_of,
                                                  1, maxGANSSSat, FALSE);

  return offset;
}


static const per_sequence_t ReqDataBitAssistance_sequence[] = {
  { &hf_rrc_ganssSignalID   , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DGANSS_Sig_Id_Req },
  { &hf_rrc_ganssDataBitInterval, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_0_15 },
  { &hf_rrc_ganssSatelliteInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_ganssSatelliteInfo },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_ReqDataBitAssistance(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_ReqDataBitAssistance, ReqDataBitAssistance_sequence);

  return offset;
}


static const per_sequence_t GanssDataBits_sequence[] = {
  { &hf_rrc_ganssTod        , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_0_86399 },
  { &hf_rrc_dataBitAssistance, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_ReqDataBitAssistance },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_GanssDataBits(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_GanssDataBits, GanssDataBits_sequence);

  return offset;
}



static int
dissect_rrc_INTEGER_0_167(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            0U, 167U, NULL, FALSE);

  return offset;
}


static const per_sequence_t SatellitesListRelatedData_sequence[] = {
  { &hf_rrc_satId           , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_0_63 },
  { &hf_rrc_iod_02          , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_0_1023 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_SatellitesListRelatedData(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_SatellitesListRelatedData, SatellitesListRelatedData_sequence);

  return offset;
}


static const per_sequence_t SatellitesListRelatedDataList_sequence_of[1] = {
  { &hf_rrc_SatellitesListRelatedDataList_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SatellitesListRelatedData },
};

static int
dissect_rrc_SatellitesListRelatedDataList(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_SatellitesListRelatedDataList, SatellitesListRelatedDataList_sequence_of,
                                                  1, maxGANSSSat, FALSE);

  return offset;
}


static const per_sequence_t GanssNavigationModelAdditionalData_sequence[] = {
  { &hf_rrc_ganssWeek       , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_0_4095 },
  { &hf_rrc_ganssToe        , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_0_167 },
  { &hf_rrc_t_toeLimit      , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_0_10 },
  { &hf_rrc_satellitesListRelatedDataList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_SatellitesListRelatedDataList },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_GanssNavigationModelAdditionalData(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_GanssNavigationModelAdditionalData, GanssNavigationModelAdditionalData_sequence);

  return offset;
}


static const per_sequence_t GanssReqGenericData_sequence[] = {
  { &hf_rrc_ganssId         , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_INTEGER_0_7 },
  { &hf_rrc_ganssRealTimeIntegrity, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { &hf_rrc_ganssDifferentialCorrection, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DGANSS_Sig_Id_Req },
  { &hf_rrc_ganssAlmanac    , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { &hf_rrc_ganssNavigationModel, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { &hf_rrc_ganssTimeModelGNSS_GNSS, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_BIT_STRING_SIZE_8 },
  { &hf_rrc_ganssReferenceMeasurementInfo, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { &hf_rrc_ganssDataBits   , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_GanssDataBits },
  { &hf_rrc_ganssUTCModel   , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { &hf_rrc_ganssNavigationModelAdditionalData, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_GanssNavigationModelAdditionalData },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_GanssReqGenericData(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_GanssReqGenericData, GanssReqGenericData_sequence);

  return offset;
}


static const per_sequence_t GanssRequestedGenericAssistanceDataList_sequence_of[1] = {
  { &hf_rrc_GanssRequestedGenericAssistanceDataList_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_GanssReqGenericData },
};

static int
dissect_rrc_GanssRequestedGenericAssistanceDataList(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_GanssRequestedGenericAssistanceDataList, GanssRequestedGenericAssistanceDataList_sequence_of,
                                                  1, maxGANSS, FALSE);

  return offset;
}


static const per_sequence_t UE_Positioning_GANSS_AdditionalAssistanceDataRequest_sequence[] = {
  { &hf_rrc_ganssReferenceTime, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { &hf_rrc_ganssreferenceLocation, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { &hf_rrc_ganssIonosphericModel, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { &hf_rrc_ganssRequestedGenericAssistanceDataList, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_GanssRequestedGenericAssistanceDataList },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UE_Positioning_GANSS_AdditionalAssistanceDataRequest(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UE_Positioning_GANSS_AdditionalAssistanceDataRequest, UE_Positioning_GANSS_AdditionalAssistanceDataRequest_sequence);

  return offset;
}


static const per_sequence_t UE_Positioning_Error_r7_sequence[] = {
  { &hf_rrc_errorReason_01  , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UE_Positioning_ErrorCause_r7 },
  { &hf_rrc_ue_positioning_GPS_additionalAssistanceDataRequest, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_Positioning_GPS_AdditionalAssistanceDataRequest },
  { &hf_rrc_ue_positioning_GANSS_additionalAssistanceDataRequest, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_Positioning_GANSS_AdditionalAssistanceDataRequest },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UE_Positioning_Error_r7(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UE_Positioning_Error_r7, UE_Positioning_Error_r7_sequence);

  return offset;
}


static const value_string rrc_T_referenceTime_vals[] = {
  {   0, "utran-GANSSReferenceTimeResult" },
  {   1, "ganssReferenceTimeOnly" },
  { 0, NULL }
};

static const per_choice_t T_referenceTime_choice[] = {
  {   0, &hf_rrc_utran_GANSSReferenceTimeResult, ASN1_NO_EXTENSIONS     , dissect_rrc_UTRAN_GANSSReferenceTime },
  {   1, &hf_rrc_ganssReferenceTimeOnly, ASN1_NO_EXTENSIONS     , dissect_rrc_GANSSReferenceTimeOnly },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_referenceTime(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_referenceTime, T_referenceTime_choice,
                                 NULL);

  return offset;
}


static const value_string rrc_T_multipathIndicator_vals[] = {
  {   0, "nm" },
  {   1, "low" },
  {   2, "medium" },
  {   3, "high" },
  { 0, NULL }
};


static int
dissect_rrc_T_multipathIndicator(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     4, NULL, FALSE, 0, NULL);

  return offset;
}



static int
dissect_rrc_INTEGER_0_2097151(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            0U, 2097151U, NULL, FALSE);

  return offset;
}



static int
dissect_rrc_INTEGER_M32768_32767(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            -32768, 32767U, NULL, FALSE);

  return offset;
}



static int
dissect_rrc_INTEGER_0_33554431(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            0U, 33554431U, NULL, FALSE);

  return offset;
}


static const per_sequence_t GANSSMeasurementParameters_item_sequence[] = {
  { &hf_rrc_satId           , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_0_63 },
  { &hf_rrc_cSurNzero       , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_0_63 },
  { &hf_rrc_multipathIndicator, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_multipathIndicator },
  { &hf_rrc_carrierQualityIndication, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_BIT_STRING_SIZE_2 },
  { &hf_rrc_ganssCodePhase  , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_0_2097151 },
  { &hf_rrc_ganssIntegerCodePhase, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_INTEGER_0_63 },
  { &hf_rrc_codePhaseRmsError, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_0_63 },
  { &hf_rrc_doppler         , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_M32768_32767 },
  { &hf_rrc_adr             , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_INTEGER_0_33554431 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_GANSSMeasurementParameters_item(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_GANSSMeasurementParameters_item, GANSSMeasurementParameters_item_sequence);

  return offset;
}


static const per_sequence_t GANSSMeasurementParameters_sequence_of[1] = {
  { &hf_rrc_GANSSMeasurementParameters_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_GANSSMeasurementParameters_item },
};

static int
dissect_rrc_GANSSMeasurementParameters(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_GANSSMeasurementParameters, GANSSMeasurementParameters_sequence_of,
                                                  1, maxGANSSSat, FALSE);

  return offset;
}


static const per_sequence_t GANSSMeasurementSignalList_item_sequence[] = {
  { &hf_rrc_ganssSignalId   , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_GANSS_Signal_Id },
  { &hf_rrc_ganssCodePhaseAmbiguity, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_INTEGER_0_31 },
  { &hf_rrc_ganssMeasurementParameters, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_GANSSMeasurementParameters },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_GANSSMeasurementSignalList_item(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_GANSSMeasurementSignalList_item, GANSSMeasurementSignalList_item_sequence);

  return offset;
}


static const per_sequence_t GANSSMeasurementSignalList_sequence_of[1] = {
  { &hf_rrc_GANSSMeasurementSignalList_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_GANSSMeasurementSignalList_item },
};

static int
dissect_rrc_GANSSMeasurementSignalList(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_GANSSMeasurementSignalList, GANSSMeasurementSignalList_sequence_of,
                                                  1, maxSgnType, FALSE);

  return offset;
}


static const per_sequence_t GANSSGenericMeasurementInfo_item_sequence[] = {
  { &hf_rrc_ganssId         , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_INTEGER_0_7 },
  { &hf_rrc_ganssMeasurementSignalList, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_GANSSMeasurementSignalList },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_GANSSGenericMeasurementInfo_item(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_GANSSGenericMeasurementInfo_item, GANSSGenericMeasurementInfo_item_sequence);

  return offset;
}


static const per_sequence_t GANSSGenericMeasurementInfo_sequence_of[1] = {
  { &hf_rrc_GANSSGenericMeasurementInfo_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_GANSSGenericMeasurementInfo_item },
};

static int
dissect_rrc_GANSSGenericMeasurementInfo(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_GANSSGenericMeasurementInfo, GANSSGenericMeasurementInfo_sequence_of,
                                                  1, maxGANSS, FALSE);

  return offset;
}


static const per_sequence_t UE_Positioning_GANSS_MeasuredResults_sequence[] = {
  { &hf_rrc_referenceTime   , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_referenceTime },
  { &hf_rrc_ganssGenericMeasurementInfo, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_GANSSGenericMeasurementInfo },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UE_Positioning_GANSS_MeasuredResults(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UE_Positioning_GANSS_MeasuredResults, UE_Positioning_GANSS_MeasuredResults_sequence);

  return offset;
}


static const per_sequence_t UE_Positioning_MeasuredResults_v770ext_sequence[] = {
  { &hf_rrc_ue_positioning_PositionEstimateInfo_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_Positioning_PositionEstimateInfo_v770ext },
  { &hf_rrc_ue_positioning_GPS_Measurement_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_Positioning_GPS_MeasurementResults_v770ext },
  { &hf_rrc_ue_positioning_Error_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_Positioning_Error_r7 },
  { &hf_rrc_ue_positioning_Ganss_MeasuredResults, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_Positioning_GANSS_MeasuredResults },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UE_Positioning_MeasuredResults_v770ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UE_Positioning_MeasuredResults_v770ext, UE_Positioning_MeasuredResults_v770ext_sequence);

  return offset;
}


static const value_string rrc_MeasuredResults_v770ext_vals[] = {
  {   0, "ueInternalMeasuredResults" },
  {   1, "ue-positioning-MeasuredResults" },
  { 0, NULL }
};

static const per_choice_t MeasuredResults_v770ext_choice[] = {
  {   0, &hf_rrc_ueInternalMeasuredResults, ASN1_NO_EXTENSIONS     , dissect_rrc_UE_InternalMeasuredResults_v770ext },
  {   1, &hf_rrc_ue_positioning_MeasuredResults_01, ASN1_NO_EXTENSIONS     , dissect_rrc_UE_Positioning_MeasuredResults_v770ext },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_MeasuredResults_v770ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_MeasuredResults_v770ext, MeasuredResults_v770ext_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t MeasuredResultsList_v770xet_item_sequence[] = {
  { &hf_rrc_measuredResults_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MeasuredResults_v770ext },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_MeasuredResultsList_v770xet_item(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_MeasuredResultsList_v770xet_item, MeasuredResultsList_v770xet_item_sequence);

  return offset;
}


static const per_sequence_t MeasuredResultsList_v770xet_sequence_of[1] = {
  { &hf_rrc_MeasuredResultsList_v770xet_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MeasuredResultsList_v770xet_item },
};

static int
dissect_rrc_MeasuredResultsList_v770xet(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_MeasuredResultsList_v770xet, MeasuredResultsList_v770xet_sequence_of,
                                                  1, maxAdditionalMeas, FALSE);

  return offset;
}


static const value_string rrc_UE_Positioning_MeasurementEventResults_v770ext_vals[] = {
  {   0, "event7a" },
  {   1, "event7c" },
  {   2, "event7d" },
  {   3, "spare" },
  { 0, NULL }
};

static const per_choice_t UE_Positioning_MeasurementEventResults_v770ext_choice[] = {
  {   0, &hf_rrc_event7a_01      , ASN1_NO_EXTENSIONS     , dissect_rrc_UE_Positioning_PositionEstimateInfo_v770ext },
  {   1, &hf_rrc_event7c_01      , ASN1_NO_EXTENSIONS     , dissect_rrc_UE_Positioning_GPS_MeasurementResults_v770ext },
  {   2, &hf_rrc_event7d         , ASN1_NO_EXTENSIONS     , dissect_rrc_UE_Positioning_GANSS_MeasuredResults },
  {   3, &hf_rrc_spare           , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_UE_Positioning_MeasurementEventResults_v770ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_UE_Positioning_MeasurementEventResults_v770ext, UE_Positioning_MeasurementEventResults_v770ext_choice,
                                 NULL);

  return offset;
}


static const value_string rrc_EventResults_v770ext_vals[] = {
  {   0, "ue-positioning-MeasurementEventResults" },
  { 0, NULL }
};

static const per_choice_t EventResults_v770ext_choice[] = {
  {   0, &hf_rrc_ue_positioning_MeasurementEventResults_01, ASN1_NO_EXTENSIONS     , dissect_rrc_UE_Positioning_MeasurementEventResults_v770ext },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_EventResults_v770ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_EventResults_v770ext, EventResults_v770ext_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t Neighbour_TDD_r7_sequence[] = {
  { &hf_rrc_uarfcn          , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UARFCN },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_Neighbour_TDD_r7(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_Neighbour_TDD_r7, Neighbour_TDD_r7_sequence);

  return offset;
}


static const per_sequence_t NeighbourList_TDD_r7_sequence_of[1] = {
  { &hf_rrc_NeighbourList_TDD_r7_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_Neighbour_TDD_r7 },
};

static int
dissect_rrc_NeighbourList_TDD_r7(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_NeighbourList_TDD_r7, NeighbourList_TDD_r7_sequence_of,
                                                  1, maxCellMeas, FALSE);

  return offset;
}


static const per_sequence_t UE_Positioning_OTDOA_MeasuredResultsTDD_ext_sequence[] = {
  { &hf_rrc_neighbourList   , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_NeighbourList_TDD_r7 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UE_Positioning_OTDOA_MeasuredResultsTDD_ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UE_Positioning_OTDOA_MeasuredResultsTDD_ext, UE_Positioning_OTDOA_MeasuredResultsTDD_ext_sequence);

  return offset;
}


static const per_sequence_t MeasurementReport_v770ext_IEs_sequence[] = {
  { &hf_rrc_measuredResults_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MeasuredResults_v770ext },
  { &hf_rrc_additionalMeasuredResults_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MeasuredResultsList_v770xet },
  { &hf_rrc_eventResults_01 , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_EventResults_v770ext },
  { &hf_rrc_ue_Positioning_OTDOA_MeasuredResults, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_Positioning_OTDOA_MeasuredResultsTDD_ext },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_MeasurementReport_v770ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_MeasurementReport_v770ext_IEs, MeasurementReport_v770ext_IEs_sequence);

  return offset;
}


static const value_string rrc_T_ganssEOPreq_vals[] = {
  {   0, "true" },
  { 0, NULL }
};


static int
dissect_rrc_T_ganssEOPreq(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}


static const value_string rrc_T_ganssAddNavigationModel_vals[] = {
  {   0, "true" },
  { 0, NULL }
};


static int
dissect_rrc_T_ganssAddNavigationModel(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}


static const value_string rrc_T_ganssAddUTCmodel_vals[] = {
  {   0, "true" },
  { 0, NULL }
};


static int
dissect_rrc_T_ganssAddUTCmodel(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}


static const value_string rrc_T_ganssAuxInfo_vals[] = {
  {   0, "true" },
  { 0, NULL }
};


static int
dissect_rrc_T_ganssAuxInfo(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t T_ganssAddADchoices_sequence[] = {
  { &hf_rrc_orbitModelID    , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_INTEGER_0_7 },
  { &hf_rrc_clockModelID    , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_INTEGER_0_7 },
  { &hf_rrc_utcModelID      , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_INTEGER_0_7 },
  { &hf_rrc_almanacModelID  , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_INTEGER_0_7 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_ganssAddADchoices(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_ganssAddADchoices, T_ganssAddADchoices_sequence);

  return offset;
}


static const per_sequence_t GanssReqGenericData_v860ext_sequence[] = {
  { &hf_rrc_sbasId          , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_Positioning_GANSS_SBAS_ID },
  { &hf_rrc_ganssAddNavigationModel, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_ganssAddNavigationModel },
  { &hf_rrc_ganssAddUTCmodel, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_ganssAddUTCmodel },
  { &hf_rrc_ganssAuxInfo    , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_ganssAuxInfo },
  { &hf_rrc_ganssAddADchoices, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_ganssAddADchoices },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_GanssReqGenericData_v860ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_GanssReqGenericData_v860ext, GanssReqGenericData_v860ext_sequence);

  return offset;
}


static const per_sequence_t GanssRequestedGenericAssistanceDataList_v860ext_sequence_of[1] = {
  { &hf_rrc_GanssRequestedGenericAssistanceDataList_v860ext_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_GanssReqGenericData_v860ext },
};

static int
dissect_rrc_GanssRequestedGenericAssistanceDataList_v860ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_GanssRequestedGenericAssistanceDataList_v860ext, GanssRequestedGenericAssistanceDataList_v860ext_sequence_of,
                                                  1, maxGANSS, FALSE);

  return offset;
}


static const per_sequence_t UE_Positioning_GANSS_AdditionalAssistanceDataRequest_v860ext_sequence[] = {
  { &hf_rrc_ganssAddIonoModelReq, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_BIT_STRING_SIZE_2 },
  { &hf_rrc_ganssEOPreq     , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_ganssEOPreq },
  { &hf_rrc_ganssRequestedGenericAssistanceDataList_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_GanssRequestedGenericAssistanceDataList_v860ext },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UE_Positioning_GANSS_AdditionalAssistanceDataRequest_v860ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UE_Positioning_GANSS_AdditionalAssistanceDataRequest_v860ext, UE_Positioning_GANSS_AdditionalAssistanceDataRequest_v860ext_sequence);

  return offset;
}


static const per_sequence_t UE_Positioning_Error_v860ext_sequence[] = {
  { &hf_rrc_ue_positioning_GANSS_additionalAssistanceDataRequest_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_Positioning_GANSS_AdditionalAssistanceDataRequest_v860ext },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UE_Positioning_Error_v860ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UE_Positioning_Error_v860ext, UE_Positioning_Error_v860ext_sequence);

  return offset;
}



static int
dissect_rrc_INTEGER_32_127(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            32U, 127U, NULL, FALSE);

  return offset;
}



static int
dissect_rrc_INTEGER_64_127(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            64U, 127U, NULL, FALSE);

  return offset;
}


static const per_sequence_t GANSSMeasurementParameters_v860ext_item_sequence[] = {
  { &hf_rrc_ganssIntegerCodePhaseExt, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_INTEGER_64_127 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_GANSSMeasurementParameters_v860ext_item(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_GANSSMeasurementParameters_v860ext_item, GANSSMeasurementParameters_v860ext_item_sequence);

  return offset;
}


static const per_sequence_t GANSSMeasurementParameters_v860ext_sequence_of[1] = {
  { &hf_rrc_GANSSMeasurementParameters_v860ext_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_GANSSMeasurementParameters_v860ext_item },
};

static int
dissect_rrc_GANSSMeasurementParameters_v860ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_GANSSMeasurementParameters_v860ext, GANSSMeasurementParameters_v860ext_sequence_of,
                                                  1, maxGANSSSat, FALSE);

  return offset;
}


static const per_sequence_t GANSSMeasurementSignalList_v860ext_item_sequence[] = {
  { &hf_rrc_ganssCodePhaseAmbiguityExt, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_INTEGER_32_127 },
  { &hf_rrc_ganssMeasurementParameters_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_GANSSMeasurementParameters_v860ext },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_GANSSMeasurementSignalList_v860ext_item(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_GANSSMeasurementSignalList_v860ext_item, GANSSMeasurementSignalList_v860ext_item_sequence);

  return offset;
}


static const per_sequence_t GANSSMeasurementSignalList_v860ext_sequence_of[1] = {
  { &hf_rrc_GANSSMeasurementSignalList_v860ext_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_GANSSMeasurementSignalList_v860ext_item },
};

static int
dissect_rrc_GANSSMeasurementSignalList_v860ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_GANSSMeasurementSignalList_v860ext, GANSSMeasurementSignalList_v860ext_sequence_of,
                                                  1, maxSgnType, FALSE);

  return offset;
}


static const per_sequence_t GANSSGenericMeasurementInfo_v860ext_item_sequence[] = {
  { &hf_rrc_ganssMeasurementSignalList_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_GANSSMeasurementSignalList_v860ext },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_GANSSGenericMeasurementInfo_v860ext_item(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_GANSSGenericMeasurementInfo_v860ext_item, GANSSGenericMeasurementInfo_v860ext_item_sequence);

  return offset;
}


static const per_sequence_t GANSSGenericMeasurementInfo_v860ext_sequence_of[1] = {
  { &hf_rrc_GANSSGenericMeasurementInfo_v860ext_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_GANSSGenericMeasurementInfo_v860ext_item },
};

static int
dissect_rrc_GANSSGenericMeasurementInfo_v860ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_GANSSGenericMeasurementInfo_v860ext, GANSSGenericMeasurementInfo_v860ext_sequence_of,
                                                  1, maxGANSS, FALSE);

  return offset;
}


static const per_sequence_t UE_Positioning_GANSS_MeasuredResults_v860ext_sequence[] = {
  { &hf_rrc_ganssGenericMeasurementInfo_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_GANSSGenericMeasurementInfo_v860ext },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UE_Positioning_GANSS_MeasuredResults_v860ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UE_Positioning_GANSS_MeasuredResults_v860ext, UE_Positioning_GANSS_MeasuredResults_v860ext_sequence);

  return offset;
}


static const per_sequence_t UE_Positioning_MeasuredResults_v860ext_sequence[] = {
  { &hf_rrc_ue_positioning_Error_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_Positioning_Error_v860ext },
  { &hf_rrc_ue_positioning_Ganss_MeasurementResults, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_Positioning_GANSS_MeasuredResults_v860ext },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UE_Positioning_MeasuredResults_v860ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UE_Positioning_MeasuredResults_v860ext, UE_Positioning_MeasuredResults_v860ext_sequence);

  return offset;
}


static const value_string rrc_MeasuredResults_v860ext_vals[] = {
  {   0, "ue-positioning-MeasuredResults" },
  { 0, NULL }
};

static const per_choice_t MeasuredResults_v860ext_choice[] = {
  {   0, &hf_rrc_ue_positioning_MeasuredResults_02, ASN1_NO_EXTENSIONS     , dissect_rrc_UE_Positioning_MeasuredResults_v860ext },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_MeasuredResults_v860ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_MeasuredResults_v860ext, MeasuredResults_v860ext_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t MeasuredResultsList_v860ext_item_sequence[] = {
  { &hf_rrc_measuredResults_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MeasuredResults_v860ext },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_MeasuredResultsList_v860ext_item(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_MeasuredResultsList_v860ext_item, MeasuredResultsList_v860ext_item_sequence);

  return offset;
}


static const per_sequence_t MeasuredResultsList_v860ext_sequence_of[1] = {
  { &hf_rrc_MeasuredResultsList_v860ext_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MeasuredResultsList_v860ext_item },
};

static int
dissect_rrc_MeasuredResultsList_v860ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_MeasuredResultsList_v860ext, MeasuredResultsList_v860ext_sequence_of,
                                                  1, maxAdditionalMeas, FALSE);

  return offset;
}


static const value_string rrc_UE_Positioning_MeasurementEventResults_v860ext_vals[] = {
  {   0, "event7d" },
  { 0, NULL }
};

static const per_choice_t UE_Positioning_MeasurementEventResults_v860ext_choice[] = {
  {   0, &hf_rrc_event7d_01      , ASN1_NO_EXTENSIONS     , dissect_rrc_UE_Positioning_GANSS_MeasuredResults_v860ext },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_UE_Positioning_MeasurementEventResults_v860ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_UE_Positioning_MeasurementEventResults_v860ext, UE_Positioning_MeasurementEventResults_v860ext_choice,
                                 NULL);

  return offset;
}


static const value_string rrc_EventResults_v860ext_vals[] = {
  {   0, "ue-positioning-MeasurementEventResults" },
  { 0, NULL }
};

static const per_choice_t EventResults_v860ext_choice[] = {
  {   0, &hf_rrc_ue_positioning_MeasurementEventResults_02, ASN1_NO_EXTENSIONS     , dissect_rrc_UE_Positioning_MeasurementEventResults_v860ext },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_EventResults_v860ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_EventResults_v860ext, EventResults_v860ext_choice,
                                 NULL);

  return offset;
}



static int
dissect_rrc_INTEGER_0_33(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            0U, 33U, NULL, FALSE);

  return offset;
}


static const per_sequence_t EUTRA_MeasuredCells_sequence[] = {
  { &hf_rrc_physicalCellIdentity, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_EUTRA_PhysicalCellIdentity },
  { &hf_rrc_rSRP            , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_INTEGER_0_97 },
  { &hf_rrc_rSRQ            , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_INTEGER_0_33 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_EUTRA_MeasuredCells(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_EUTRA_MeasuredCells, EUTRA_MeasuredCells_sequence);

  return offset;
}


static const per_sequence_t SEQUENCE_SIZE_1_maxReportedEUTRACellPerFreq_OF_EUTRA_MeasuredCells_sequence_of[1] = {
  { &hf_rrc_measuredEUTRACells_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_EUTRA_MeasuredCells },
};

static int
dissect_rrc_SEQUENCE_SIZE_1_maxReportedEUTRACellPerFreq_OF_EUTRA_MeasuredCells(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_SEQUENCE_SIZE_1_maxReportedEUTRACellPerFreq_OF_EUTRA_MeasuredCells, SEQUENCE_SIZE_1_maxReportedEUTRACellPerFreq_OF_EUTRA_MeasuredCells_sequence_of,
                                                  1, maxReportedEUTRACellPerFreq, FALSE);

  return offset;
}


static const per_sequence_t Eutra_MeasuredResult_sequence[] = {
  { &hf_rrc_earfcn          , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_EARFCN },
  { &hf_rrc_measuredEUTRACells, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SEQUENCE_SIZE_1_maxReportedEUTRACellPerFreq_OF_EUTRA_MeasuredCells },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_Eutra_MeasuredResult(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_Eutra_MeasuredResult, Eutra_MeasuredResult_sequence);

  return offset;
}


static const per_sequence_t Eutra_MeasuredResultList_sequence_of[1] = {
  { &hf_rrc_Eutra_MeasuredResultList_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_Eutra_MeasuredResult },
};

static int
dissect_rrc_Eutra_MeasuredResultList(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_Eutra_MeasuredResultList, Eutra_MeasuredResultList_sequence_of,
                                                  1, maxReportedEUTRAFreqs, FALSE);

  return offset;
}


static const per_sequence_t EUTRA_MeasuredResults_sequence[] = {
  { &hf_rrc_eutraMeasuredResultList, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_Eutra_MeasuredResultList },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_EUTRA_MeasuredResults(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_EUTRA_MeasuredResults, EUTRA_MeasuredResults_sequence);

  return offset;
}


static const per_sequence_t SEQUENCE_SIZE_1_maxReportedEUTRACellPerFreq_OF_EUTRA_PhysicalCellIdentity_sequence_of[1] = {
  { &hf_rrc_reportedCells_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_EUTRA_PhysicalCellIdentity },
};

static int
dissect_rrc_SEQUENCE_SIZE_1_maxReportedEUTRACellPerFreq_OF_EUTRA_PhysicalCellIdentity(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_SEQUENCE_SIZE_1_maxReportedEUTRACellPerFreq_OF_EUTRA_PhysicalCellIdentity, SEQUENCE_SIZE_1_maxReportedEUTRACellPerFreq_OF_EUTRA_PhysicalCellIdentity_sequence_of,
                                                  1, maxReportedEUTRACellPerFreq, FALSE);

  return offset;
}


static const per_sequence_t Eutra_EventResult_sequence[] = {
  { &hf_rrc_earfcn          , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_EARFCN },
  { &hf_rrc_reportedCells   , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SEQUENCE_SIZE_1_maxReportedEUTRACellPerFreq_OF_EUTRA_PhysicalCellIdentity },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_Eutra_EventResult(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_Eutra_EventResult, Eutra_EventResult_sequence);

  return offset;
}


static const per_sequence_t Eutra_EventResultList_sequence_of[1] = {
  { &hf_rrc_Eutra_EventResultList_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_Eutra_EventResult },
};

static int
dissect_rrc_Eutra_EventResultList(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_Eutra_EventResultList, Eutra_EventResultList_sequence_of,
                                                  1, maxReportedEUTRAFreqs, FALSE);

  return offset;
}


static const per_sequence_t EUTRA_EventResults_sequence[] = {
  { &hf_rrc_eventID         , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_EventIDInterRAT },
  { &hf_rrc_eutra_EventResultsList, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_Eutra_EventResultList },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_EUTRA_EventResults(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_EUTRA_EventResults, EUTRA_EventResults_sequence);

  return offset;
}


static const per_sequence_t MeasurementReport_v860ext_IEs_sequence[] = {
  { &hf_rrc_activationTime  , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_ActivationTime },
  { &hf_rrc_measuredResults_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MeasuredResults_v860ext },
  { &hf_rrc_additionalMeasuredResults_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MeasuredResultsList_v860ext },
  { &hf_rrc_eventResults_02 , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_EventResults_v860ext },
  { &hf_rrc_eutra_MeasuredResults, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_EUTRA_MeasuredResults },
  { &hf_rrc_eutra_EventResults, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_EUTRA_EventResults },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_MeasurementReport_v860ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_MeasurementReport_v860ext_IEs, MeasurementReport_v860ext_IEs_sequence);

  return offset;
}


static const value_string rrc_T_cSGproximityInd_vals[] = {
  {   0, "entering" },
  {   1, "leaving" },
  { 0, NULL }
};


static int
dissect_rrc_T_cSGproximityInd(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     2, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t T_uTRA_sequence[] = {
  { &hf_rrc_cSGFrequencyInfoUTRA, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_FrequencyInfo },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_uTRA(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_uTRA, T_uTRA_sequence);

  return offset;
}


static const per_sequence_t T_e_UTRA_sequence[] = {
  { &hf_rrc_cSGFrequencyInfoEUTRA, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_EARFCN },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_e_UTRA(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_e_UTRA, T_e_UTRA_sequence);

  return offset;
}


static const value_string rrc_T_radioAccessTechnology_01_vals[] = {
  {   0, "uTRA" },
  {   1, "e-UTRA" },
  { 0, NULL }
};

static const per_choice_t T_radioAccessTechnology_01_choice[] = {
  {   0, &hf_rrc_uTRA            , ASN1_NO_EXTENSIONS     , dissect_rrc_T_uTRA },
  {   1, &hf_rrc_e_UTRA          , ASN1_NO_EXTENSIONS     , dissect_rrc_T_e_UTRA },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_radioAccessTechnology_01(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_radioAccessTechnology_01, T_radioAccessTechnology_01_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t CSGProximityIndication_sequence[] = {
  { &hf_rrc_cSGproximityInd , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_cSGproximityInd },
  { &hf_rrc_radioAccessTechnology_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_radioAccessTechnology_01 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_CSGProximityIndication(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_CSGProximityIndication, CSGProximityIndication_sequence);

  return offset;
}



static int
dissect_rrc_CSG_Identity(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_bit_string(tvb, offset, actx, tree, hf_index,
                                     27, 27, FALSE, NULL);

  return offset;
}


static const value_string rrc_T_csgMemberIndication_vals[] = {
  {   0, "member" },
  { 0, NULL }
};


static int
dissect_rrc_T_csgMemberIndication(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t CellMeasuredResults_v920ext_sequence[] = {
  { &hf_rrc_csgIdentity     , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CSG_Identity },
  { &hf_rrc_csgMemberIndication, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_csgMemberIndication },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_CellMeasuredResults_v920ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_CellMeasuredResults_v920ext, CellMeasuredResults_v920ext_sequence);

  return offset;
}


static const per_sequence_t IntraFreqMeasuredResultsList_v920ext_sequence_of[1] = {
  { &hf_rrc_IntraFreqMeasuredResultsList_v920ext_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CellMeasuredResults_v920ext },
};

static int
dissect_rrc_IntraFreqMeasuredResultsList_v920ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_IntraFreqMeasuredResultsList_v920ext, IntraFreqMeasuredResultsList_v920ext_sequence_of,
                                                  1, maxCellMeas, FALSE);

  return offset;
}


static const per_sequence_t InterFreqCellMeasuredResultsList_v920ext_sequence_of[1] = {
  { &hf_rrc_InterFreqCellMeasuredResultsList_v920ext_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CellMeasuredResults_v920ext },
};

static int
dissect_rrc_InterFreqCellMeasuredResultsList_v920ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_InterFreqCellMeasuredResultsList_v920ext, InterFreqCellMeasuredResultsList_v920ext_sequence_of,
                                                  1, maxCellMeas, FALSE);

  return offset;
}


static const per_sequence_t InterFreqMeasuredResults_v920ext_sequence[] = {
  { &hf_rrc_interFreqCellMeasuredResultsList_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_InterFreqCellMeasuredResultsList_v920ext },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_InterFreqMeasuredResults_v920ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_InterFreqMeasuredResults_v920ext, InterFreqMeasuredResults_v920ext_sequence);

  return offset;
}


static const per_sequence_t InterFreqMeasuredResultsList_v920ext_sequence_of[1] = {
  { &hf_rrc_InterFreqMeasuredResultsList_v920ext_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_InterFreqMeasuredResults_v920ext },
};

static int
dissect_rrc_InterFreqMeasuredResultsList_v920ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_InterFreqMeasuredResultsList_v920ext, InterFreqMeasuredResultsList_v920ext_sequence_of,
                                                  1, maxFreq, FALSE);

  return offset;
}


static const value_string rrc_MeasuredResults_v920ext_vals[] = {
  {   0, "intraFreqMeasuredResultsList" },
  {   1, "interFreqMeasuredResultsList" },
  { 0, NULL }
};

static const per_choice_t MeasuredResults_v920ext_choice[] = {
  {   0, &hf_rrc_intraFreqMeasuredResultsList_02, ASN1_NO_EXTENSIONS     , dissect_rrc_IntraFreqMeasuredResultsList_v920ext },
  {   1, &hf_rrc_interFreqMeasuredResultsList_01, ASN1_NO_EXTENSIONS     , dissect_rrc_InterFreqMeasuredResultsList_v920ext },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_MeasuredResults_v920ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_MeasuredResults_v920ext, MeasuredResults_v920ext_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t T_fdd_140_sequence[] = {
  { &hf_rrc_primaryCPICH_Info, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PrimaryCPICH_Info },
  { &hf_rrc_cpich_Ec_N0_01  , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CPICH_Ec_N0 },
  { &hf_rrc_cpich_RSCP_01   , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CPICH_RSCP },
  { &hf_rrc_deltaRSCPPerCell, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DeltaRSCPPerCell },
  { &hf_rrc_pathloss        , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_Pathloss },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_fdd_140(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_fdd_140, T_fdd_140_sequence);

  return offset;
}


static const per_sequence_t T_tdd_91_sequence[] = {
  { &hf_rrc_cellParametersID, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CellParametersID },
  { &hf_rrc_proposedTGSN    , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_TGSN },
  { &hf_rrc_primaryCCPCH_RSCP, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_PrimaryCCPCH_RSCP },
  { &hf_rrc_pathloss        , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_Pathloss },
  { &hf_rrc_timeslotISCP_List, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_TimeslotISCP_List },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_tdd_91(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_tdd_91, T_tdd_91_sequence);

  return offset;
}


static const value_string rrc_T_modeSpecificInfo_101_vals[] = {
  {   0, "fdd" },
  {   1, "tdd" },
  { 0, NULL }
};

static const per_choice_t T_modeSpecificInfo_101_choice[] = {
  {   0, &hf_rrc_fdd_143         , ASN1_NO_EXTENSIONS     , dissect_rrc_T_fdd_140 },
  {   1, &hf_rrc_tdd_97          , ASN1_NO_EXTENSIONS     , dissect_rrc_T_tdd_91 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_modeSpecificInfo_101(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_modeSpecificInfo_101, T_modeSpecificInfo_101_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t CellMeasuredResults_r9_sequence[] = {
  { &hf_rrc_cellIdentity    , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CellIdentity },
  { &hf_rrc_cellSynchronisationInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CellSynchronisationInfo },
  { &hf_rrc_modeSpecificInfo_101, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_modeSpecificInfo_101 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_CellMeasuredResults_r9(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_CellMeasuredResults_r9, CellMeasuredResults_r9_sequence);

  return offset;
}


static const per_sequence_t IntraFrequencyMeasuredResultsListOnSecULFreq_sequence_of[1] = {
  { &hf_rrc_IntraFrequencyMeasuredResultsListOnSecULFreq_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CellMeasuredResults_r9 },
};

static int
dissect_rrc_IntraFrequencyMeasuredResultsListOnSecULFreq(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_IntraFrequencyMeasuredResultsListOnSecULFreq, IntraFrequencyMeasuredResultsListOnSecULFreq_sequence_of,
                                                  1, maxCellMeasOnSecULFreq, FALSE);

  return offset;
}


static const per_sequence_t MeasuredResultsOnSecUlFreq_sequence[] = {
  { &hf_rrc_intraFreqMeasuredResultsList_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_IntraFrequencyMeasuredResultsListOnSecULFreq },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_MeasuredResultsOnSecUlFreq(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_MeasuredResultsOnSecUlFreq, MeasuredResultsOnSecUlFreq_sequence);

  return offset;
}


static const per_sequence_t MeasuredResultsList_v920ext_sequence_of[1] = {
  { &hf_rrc_MeasuredResultsList_v920ext_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MeasuredResults_v920ext },
};

static int
dissect_rrc_MeasuredResultsList_v920ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_MeasuredResultsList_v920ext, MeasuredResultsList_v920ext_sequence_of,
                                                  1, maxAdditionalMeas, FALSE);

  return offset;
}


static const per_sequence_t MeasuredResultsListOnSecUlFreq_sequence_of[1] = {
  { &hf_rrc_MeasuredResultsListOnSecUlFreq_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MeasuredResultsOnSecUlFreq },
};

static int
dissect_rrc_MeasuredResultsListOnSecUlFreq(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_MeasuredResultsListOnSecUlFreq, MeasuredResultsListOnSecUlFreq_sequence_of,
                                                  1, maxAdditionalMeas, FALSE);

  return offset;
}


static const per_sequence_t SEQUENCE_SIZE_1_maxCellMeasOnSecULFreq_OF_PrimaryCPICH_Info_sequence_of[1] = {
  { &hf_rrc_fdd_item_01     , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PrimaryCPICH_Info },
};

static int
dissect_rrc_SEQUENCE_SIZE_1_maxCellMeasOnSecULFreq_OF_PrimaryCPICH_Info(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_SEQUENCE_SIZE_1_maxCellMeasOnSecULFreq_OF_PrimaryCPICH_Info, SEQUENCE_SIZE_1_maxCellMeasOnSecULFreq_OF_PrimaryCPICH_Info_sequence_of,
                                                  1, maxCellMeasOnSecULFreq, FALSE);

  return offset;
}


static const value_string rrc_CellMeasurementEventResultsOnSecUlFreq_vals[] = {
  {   0, "fdd" },
  {   1, "spare" },
  { 0, NULL }
};

static const per_choice_t CellMeasurementEventResultsOnSecUlFreq_choice[] = {
  {   0, &hf_rrc_fdd_145         , ASN1_NO_EXTENSIONS     , dissect_rrc_SEQUENCE_SIZE_1_maxCellMeasOnSecULFreq_OF_PrimaryCPICH_Info },
  {   1, &hf_rrc_spare           , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_CellMeasurementEventResultsOnSecUlFreq(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_CellMeasurementEventResultsOnSecUlFreq, CellMeasurementEventResultsOnSecUlFreq_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t IntraFreqEventResultsOnSecUlFreq_sequence[] = {
  { &hf_rrc_eventID_02      , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_EventIDIntraFreq },
  { &hf_rrc_cellMeasurementEventResultsOnSecUlFreq, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CellMeasurementEventResultsOnSecUlFreq },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_IntraFreqEventResultsOnSecUlFreq(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_IntraFreqEventResultsOnSecUlFreq, IntraFreqEventResultsOnSecUlFreq_sequence);

  return offset;
}


static const per_sequence_t EventResultsOnSecUlFreq_sequence[] = {
  { &hf_rrc_intraFreqEventResults_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_IntraFreqEventResultsOnSecUlFreq },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_EventResultsOnSecUlFreq(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_EventResultsOnSecUlFreq, EventResultsOnSecUlFreq_sequence);

  return offset;
}


static const per_sequence_t T_cgiInfo_sequence[] = {
  { &hf_rrc_plmn_Identity   , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PLMN_Identity },
  { &hf_rrc_cellIdentity_01 , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_28 },
  { &hf_rrc_trackingAreaCode, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_16 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_cgiInfo(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_cgiInfo, T_cgiInfo_sequence);

  return offset;
}


static const value_string rrc_T_csgMemberIndication_01_vals[] = {
  {   0, "member" },
  { 0, NULL }
};


static int
dissect_rrc_T_csgMemberIndication_01(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t EUTRA_SIacquisitionResults_sequence[] = {
  { &hf_rrc_cgiInfo         , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_cgiInfo },
  { &hf_rrc_csgIdentity     , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CSG_Identity },
  { &hf_rrc_csgMemberIndication_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_csgMemberIndication_01 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_EUTRA_SIacquisitionResults(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_EUTRA_SIacquisitionResults, EUTRA_SIacquisitionResults_sequence);

  return offset;
}


static const per_sequence_t EUTRA_MeasuredCells_v920ext_sequence[] = {
  { &hf_rrc_eutraSIacquisitionResults, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_EUTRA_SIacquisitionResults },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_EUTRA_MeasuredCells_v920ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_EUTRA_MeasuredCells_v920ext, EUTRA_MeasuredCells_v920ext_sequence);

  return offset;
}


static const per_sequence_t SEQUENCE_SIZE_1_maxReportedEUTRACellPerFreq_OF_EUTRA_MeasuredCells_v920ext_sequence_of[1] = {
  { &hf_rrc_measuredEUTRACells_v920ext_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_EUTRA_MeasuredCells_v920ext },
};

static int
dissect_rrc_SEQUENCE_SIZE_1_maxReportedEUTRACellPerFreq_OF_EUTRA_MeasuredCells_v920ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_SEQUENCE_SIZE_1_maxReportedEUTRACellPerFreq_OF_EUTRA_MeasuredCells_v920ext, SEQUENCE_SIZE_1_maxReportedEUTRACellPerFreq_OF_EUTRA_MeasuredCells_v920ext_sequence_of,
                                                  1, maxReportedEUTRACellPerFreq, FALSE);

  return offset;
}


static const per_sequence_t Eutra_MeasuredResult_v920ext_sequence[] = {
  { &hf_rrc_measuredEUTRACells_v920ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SEQUENCE_SIZE_1_maxReportedEUTRACellPerFreq_OF_EUTRA_MeasuredCells_v920ext },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_Eutra_MeasuredResult_v920ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_Eutra_MeasuredResult_v920ext, Eutra_MeasuredResult_v920ext_sequence);

  return offset;
}


static const per_sequence_t Eutra_MeasuredResultList_v920ext_sequence_of[1] = {
  { &hf_rrc_Eutra_MeasuredResultList_v920ext_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_Eutra_MeasuredResult_v920ext },
};

static int
dissect_rrc_Eutra_MeasuredResultList_v920ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_Eutra_MeasuredResultList_v920ext, Eutra_MeasuredResultList_v920ext_sequence_of,
                                                  1, maxReportedEUTRAFreqs, FALSE);

  return offset;
}


static const per_sequence_t EUTRA_MeasuredResults_v920ext_sequence[] = {
  { &hf_rrc_eutraMeasuredResultList_v920ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_Eutra_MeasuredResultList_v920ext },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_EUTRA_MeasuredResults_v920ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_EUTRA_MeasuredResults_v920ext, EUTRA_MeasuredResults_v920ext_sequence);

  return offset;
}


static const per_sequence_t MeasurementReport_v920ext_IEs_sequence[] = {
  { &hf_rrc_cSGProximityIndication, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CSGProximityIndication },
  { &hf_rrc_measuredResults_03, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MeasuredResults_v920ext },
  { &hf_rrc_measuredResultsOnSecUlFreq, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MeasuredResultsOnSecUlFreq },
  { &hf_rrc_additionalMeasuredResults_03, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MeasuredResultsList_v920ext },
  { &hf_rrc_additionalMeasuredResultsOnSecUlFreq, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MeasuredResultsListOnSecUlFreq },
  { &hf_rrc_eventResultsOnSecUlFreq, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_EventResultsOnSecUlFreq },
  { &hf_rrc_eutra_MeasuredResults_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_EUTRA_MeasuredResults_v920ext },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_MeasurementReport_v920ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_MeasurementReport_v920ext_IEs, MeasurementReport_v920ext_IEs_sequence);

  return offset;
}



static int
dissect_rrc_MeasurementIdentityExt(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            17U, 32U, NULL, FALSE);

  return offset;
}


static const per_sequence_t MeasurementReport_v970ext_IEs_sequence[] = {
  { &hf_rrc_measurementIdentity_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MeasurementIdentityExt },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_MeasurementReport_v970ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_MeasurementReport_v970ext_IEs, MeasurementReport_v970ext_IEs_sequence);

  return offset;
}


static const value_string rrc_T_detectedSetTrigger_vals[] = {
  {   0, "true" },
  { 0, NULL }
};


static int
dissect_rrc_T_detectedSetTrigger(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t InterFreqEventResults_va40ext_sequence[] = {
  { &hf_rrc_detectedSetTrigger, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_detectedSetTrigger },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_InterFreqEventResults_va40ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_InterFreqEventResults_va40ext, InterFreqEventResults_va40ext_sequence);

  return offset;
}


static const value_string rrc_EventResults_va40ext_vals[] = {
  {   0, "interFreqEventResults" },
  { 0, NULL }
};

static const per_choice_t EventResults_va40ext_choice[] = {
  {   0, &hf_rrc_interFreqEventResults_01, ASN1_NO_EXTENSIONS     , dissect_rrc_InterFreqEventResults_va40ext },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_EventResults_va40ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_EventResults_va40ext, EventResults_va40ext_choice,
                                 NULL);

  return offset;
}


static const value_string rrc_T_loggedMeasAvailable_02_vals[] = {
  {   0, "true" },
  { 0, NULL }
};


static int
dissect_rrc_T_loggedMeasAvailable_02(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}


static const value_string rrc_T_loggedANRResultsAvailable_01_vals[] = {
  {   0, "true" },
  { 0, NULL }
};


static int
dissect_rrc_T_loggedANRResultsAvailable_01(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t MeasurementReport_va40ext_IEs_sequence[] = {
  { &hf_rrc_eventResults_03 , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_EventResults_va40ext },
  { &hf_rrc_loggedMeasAvailable_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_loggedMeasAvailable_02 },
  { &hf_rrc_loggedANRResultsAvailable_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_loggedANRResultsAvailable_01 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_MeasurementReport_va40ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_MeasurementReport_va40ext_IEs, MeasurementReport_va40ext_IEs_sequence);

  return offset;
}


static const per_sequence_t EUTRA_EventResultsForCELLFACH_sequence[] = {
  { &hf_rrc_eutraFrequencyIndicator, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_maxNumEUTRAFreqs_FACH },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_EUTRA_EventResultsForCELLFACH(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_EUTRA_EventResultsForCELLFACH, EUTRA_EventResultsForCELLFACH_sequence);

  return offset;
}


static const per_sequence_t MeasurementReport_vbxyext_IEs_sequence[] = {
  { &hf_rrc_measuredResultsOnRACHeutraFreq, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MeasuredResultsOnRACHeutraFreq },
  { &hf_rrc_eutra_EventResultsForCELLFACH, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_EUTRA_EventResultsForCELLFACH },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_MeasurementReport_vbxyext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_MeasurementReport_vbxyext_IEs, MeasurementReport_vbxyext_IEs_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions_67_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_nonCriticalExtensions_67(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_nonCriticalExtensions_67, T_nonCriticalExtensions_67_sequence);

  return offset;
}


static const per_sequence_t T_vbxyNonCriticalExtensions_07_sequence[] = {
  { &hf_rrc_measurementReport_vbxyext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MeasurementReport_vbxyext_IEs },
  { &hf_rrc_nonCriticalExtensions_67, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_nonCriticalExtensions_67 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_vbxyNonCriticalExtensions_07(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_vbxyNonCriticalExtensions_07, T_vbxyNonCriticalExtensions_07_sequence);

  return offset;
}


static const per_sequence_t T_va40NonCriticalExtensions_04_sequence[] = {
  { &hf_rrc_measurementReport_va40ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MeasurementReport_va40ext_IEs },
  { &hf_rrc_vbxyNonCriticalExtensions_07, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_vbxyNonCriticalExtensions_07 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_va40NonCriticalExtensions_04(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_va40NonCriticalExtensions_04, T_va40NonCriticalExtensions_04_sequence);

  return offset;
}


static const per_sequence_t T_v970NonCriticalExtensions_01_sequence[] = {
  { &hf_rrc_measurementReport_v970ext_IE, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MeasurementReport_v970ext_IEs },
  { &hf_rrc_va40NonCriticalExtensions_04, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_va40NonCriticalExtensions_04 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v970NonCriticalExtensions_01(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v970NonCriticalExtensions_01, T_v970NonCriticalExtensions_01_sequence);

  return offset;
}


static const per_sequence_t T_v920NonCriticalExtensions_01_sequence[] = {
  { &hf_rrc_measurementReport_v920ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MeasurementReport_v920ext_IEs },
  { &hf_rrc_v970NonCriticalExtensions_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v970NonCriticalExtensions_01 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v920NonCriticalExtensions_01(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v920NonCriticalExtensions_01, T_v920NonCriticalExtensions_01_sequence);

  return offset;
}


static const per_sequence_t T_v860NonCriticalExtensions_08_sequence[] = {
  { &hf_rrc_measurementReport_v860ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MeasurementReport_v860ext_IEs },
  { &hf_rrc_v920NonCriticalExtensions_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v920NonCriticalExtensions_01 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v860NonCriticalExtensions_08(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v860NonCriticalExtensions_08, T_v860NonCriticalExtensions_08_sequence);

  return offset;
}


static const per_sequence_t T_v770NonCriticalExtensions_03_sequence[] = {
  { &hf_rrc_measurementReport_v770ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MeasurementReport_v770ext_IEs },
  { &hf_rrc_v860NonCriticalExtensions_08, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v860NonCriticalExtensions_08 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v770NonCriticalExtensions_03(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v770NonCriticalExtensions_03, T_v770NonCriticalExtensions_03_sequence);

  return offset;
}


static const per_sequence_t T_v690NonCriticalExtensions_11_sequence[] = {
  { &hf_rrc_measurementReport_v690ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MeasurementReport_v690ext_IEs },
  { &hf_rrc_v770NonCriticalExtensions_03, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v770NonCriticalExtensions_03 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v690NonCriticalExtensions_11(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v690NonCriticalExtensions_11, T_v690NonCriticalExtensions_11_sequence);

  return offset;
}


static const per_sequence_t T_v5b0NonCriticalExtensions_02_sequence[] = {
  { &hf_rrc_measurementReport_v5b0ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MeasurementReport_v5b0ext_IEs },
  { &hf_rrc_v690NonCriticalExtensions_11, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v690NonCriticalExtensions_11 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v5b0NonCriticalExtensions_02(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v5b0NonCriticalExtensions_02, T_v5b0NonCriticalExtensions_02_sequence);

  return offset;
}


static const per_sequence_t T_v590NonCriticalExtensions_11_sequence[] = {
  { &hf_rrc_measurementReport_v590ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MeasurementReport_v590ext_IEs },
  { &hf_rrc_v5b0NonCriticalExtensions_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v5b0NonCriticalExtensions_02 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v590NonCriticalExtensions_11(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v590NonCriticalExtensions_11, T_v590NonCriticalExtensions_11_sequence);

  return offset;
}


static const per_sequence_t T_v4b0NonCriticalExtensions_06_sequence[] = {
  { &hf_rrc_measurementReport_v4b0ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MeasurementReport_v4b0ext_IEs },
  { &hf_rrc_v590NonCriticalExtensions_11, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v590NonCriticalExtensions_11 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v4b0NonCriticalExtensions_06(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v4b0NonCriticalExtensions_06, T_v4b0NonCriticalExtensions_06_sequence);

  return offset;
}


static const per_sequence_t T_laterNonCriticalExtensions_20_sequence[] = {
  { &hf_rrc_measurementReport_r3_add_ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_BIT_STRING },
  { &hf_rrc_v4b0NonCriticalExtensions_06, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v4b0NonCriticalExtensions_06 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_laterNonCriticalExtensions_20(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_laterNonCriticalExtensions_20, T_laterNonCriticalExtensions_20_sequence);

  return offset;
}


static const per_sequence_t T_v390nonCriticalExtensions_01_sequence[] = {
  { &hf_rrc_measurementReport_v390ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MeasurementReport_v390ext },
  { &hf_rrc_laterNonCriticalExtensions_20, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_laterNonCriticalExtensions_20 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v390nonCriticalExtensions_01(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v390nonCriticalExtensions_01, T_v390nonCriticalExtensions_01_sequence);

  return offset;
}


static const per_sequence_t MeasurementReport_sequence[] = {
  { &hf_rrc_measurementIdentity, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MeasurementIdentity },
  { &hf_rrc_measuredResults , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MeasuredResults },
  { &hf_rrc_measuredResultsOnRACH, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MeasuredResultsOnRACH },
  { &hf_rrc_additionalMeasuredResults, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MeasuredResultsList },
  { &hf_rrc_eventResults    , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_EventResults },
  { &hf_rrc_v390nonCriticalExtensions_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v390nonCriticalExtensions_01 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_MeasurementReport(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
#line 293 "../../asn1/rrc/rrc.cnf"
  col_append_str(actx->pinfo->cinfo, COL_INFO, "MeasurementReport");
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_MeasurementReport, MeasurementReport_sequence);




  return offset;
}


static const value_string rrc_T_deferredMeasurementControlReading_vals[] = {
  {   0, "true" },
  { 0, NULL }
};


static int
dissect_rrc_T_deferredMeasurementControlReading(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t PhysicalChannelReconfigurationComplete_v770ext_IEs_sequence[] = {
  { &hf_rrc_ext_UL_TimingAdvance, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_EXT_UL_TimingAdvance },
  { &hf_rrc_deferredMeasurementControlReading, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_deferredMeasurementControlReading },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_PhysicalChannelReconfigurationComplete_v770ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_PhysicalChannelReconfigurationComplete_v770ext_IEs, PhysicalChannelReconfigurationComplete_v770ext_IEs_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions_79_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_nonCriticalExtensions_79(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_nonCriticalExtensions_79, T_nonCriticalExtensions_79_sequence);

  return offset;
}


static const per_sequence_t T_v770NonCriticalExtensions_08_sequence[] = {
  { &hf_rrc_physicalChannelReconfigurationComplete_v770ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PhysicalChannelReconfigurationComplete_v770ext_IEs },
  { &hf_rrc_nonCriticalExtensions_79, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_nonCriticalExtensions_79 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v770NonCriticalExtensions_08(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v770NonCriticalExtensions_08, T_v770NonCriticalExtensions_08_sequence);

  return offset;
}


static const per_sequence_t T_laterNonCriticalExtensions_24_sequence[] = {
  { &hf_rrc_physicalChannelReconfigurationComplete_r3_add_ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_BIT_STRING },
  { &hf_rrc_v770NonCriticalExtensions_08, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v770NonCriticalExtensions_08 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_laterNonCriticalExtensions_24(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_laterNonCriticalExtensions_24, T_laterNonCriticalExtensions_24_sequence);

  return offset;
}


static const per_sequence_t PhysicalChannelReconfigurationComplete_sequence[] = {
  { &hf_rrc_rrc_TransactionIdentifier, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RRC_TransactionIdentifier },
  { &hf_rrc_ul_IntegProtActivationInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_IntegrityProtActivationInfo },
  { &hf_rrc_ul_TimingAdvance, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_TimingAdvance },
  { &hf_rrc_count_C_ActivationTime, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_ActivationTime },
  { &hf_rrc_dummy_13        , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RB_ActivationTimeInfoList },
  { &hf_rrc_ul_CounterSynchronisationInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_CounterSynchronisationInfo },
  { &hf_rrc_laterNonCriticalExtensions_24, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_laterNonCriticalExtensions_24 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_PhysicalChannelReconfigurationComplete(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
#line 309 "../../asn1/rrc/rrc.cnf"
  col_append_str(actx->pinfo->cinfo, COL_INFO, "PhysicalChannelReconfigurationComplete");
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_PhysicalChannelReconfigurationComplete, PhysicalChannelReconfigurationComplete_sequence);




  return offset;
}


static const per_sequence_t T_nonCriticalExtensions_80_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_nonCriticalExtensions_80(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_nonCriticalExtensions_80, T_nonCriticalExtensions_80_sequence);

  return offset;
}


static const per_sequence_t T_laterNonCriticalExtensions_25_sequence[] = {
  { &hf_rrc_physicalChannelReconfigurationFailure_r3_add_ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_BIT_STRING },
  { &hf_rrc_nonCriticalExtensions_80, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_nonCriticalExtensions_80 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_laterNonCriticalExtensions_25(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_laterNonCriticalExtensions_25, T_laterNonCriticalExtensions_25_sequence);

  return offset;
}


static const per_sequence_t PhysicalChannelReconfigurationFailure_sequence[] = {
  { &hf_rrc_rrc_TransactionIdentifier, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RRC_TransactionIdentifier },
  { &hf_rrc_failureCause    , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_FailureCauseWithProtErr },
  { &hf_rrc_laterNonCriticalExtensions_25, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_laterNonCriticalExtensions_25 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_PhysicalChannelReconfigurationFailure(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
#line 313 "../../asn1/rrc/rrc.cnf"
  col_append_str(actx->pinfo->cinfo, COL_INFO, "PhysicalChannelReconfigurationFailure");
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_PhysicalChannelReconfigurationFailure, PhysicalChannelReconfigurationFailure_sequence);




  return offset;
}


static const value_string rrc_T_deferredMeasurementControlReading_01_vals[] = {
  {   0, "true" },
  { 0, NULL }
};


static int
dissect_rrc_T_deferredMeasurementControlReading_01(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t RadioBearerReconfigurationComplete_v770ext_IEs_sequence[] = {
  { &hf_rrc_ext_UL_TimingAdvance, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_EXT_UL_TimingAdvance },
  { &hf_rrc_deferredMeasurementControlReading_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_deferredMeasurementControlReading_01 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_RadioBearerReconfigurationComplete_v770ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_RadioBearerReconfigurationComplete_v770ext_IEs, RadioBearerReconfigurationComplete_v770ext_IEs_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions_93_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_nonCriticalExtensions_93(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_nonCriticalExtensions_93, T_nonCriticalExtensions_93_sequence);

  return offset;
}


static const per_sequence_t T_v770NonCriticalExtensions_14_sequence[] = {
  { &hf_rrc_radioBearerReconfigurationComplete_v770ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RadioBearerReconfigurationComplete_v770ext_IEs },
  { &hf_rrc_nonCriticalExtensions_93, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_nonCriticalExtensions_93 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v770NonCriticalExtensions_14(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v770NonCriticalExtensions_14, T_v770NonCriticalExtensions_14_sequence);

  return offset;
}


static const per_sequence_t T_laterNonCriticalExtensions_29_sequence[] = {
  { &hf_rrc_radioBearerReconfigurationComplete_r3_add_ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_BIT_STRING },
  { &hf_rrc_v770NonCriticalExtensions_14, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v770NonCriticalExtensions_14 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_laterNonCriticalExtensions_29(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_laterNonCriticalExtensions_29, T_laterNonCriticalExtensions_29_sequence);

  return offset;
}


static const per_sequence_t RadioBearerReconfigurationComplete_sequence[] = {
  { &hf_rrc_rrc_TransactionIdentifier, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RRC_TransactionIdentifier },
  { &hf_rrc_ul_IntegProtActivationInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_IntegrityProtActivationInfo },
  { &hf_rrc_ul_TimingAdvance, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_TimingAdvance },
  { &hf_rrc_count_C_ActivationTime, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_ActivationTime },
  { &hf_rrc_dummy_13        , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RB_ActivationTimeInfoList },
  { &hf_rrc_ul_CounterSynchronisationInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_CounterSynchronisationInfo },
  { &hf_rrc_laterNonCriticalExtensions_29, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_laterNonCriticalExtensions_29 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_RadioBearerReconfigurationComplete(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
#line 329 "../../asn1/rrc/rrc.cnf"
  col_append_str(actx->pinfo->cinfo, COL_INFO, "RadioBearerReconfigurationComplete");
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_RadioBearerReconfigurationComplete, RadioBearerReconfigurationComplete_sequence);




  return offset;
}


static const per_sequence_t T_nonCriticalExtensions_94_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_nonCriticalExtensions_94(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_nonCriticalExtensions_94, T_nonCriticalExtensions_94_sequence);

  return offset;
}


static const per_sequence_t T_laterNonCriticalExtensions_30_sequence[] = {
  { &hf_rrc_radioBearerReconfigurationFailure_r3_add_ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_BIT_STRING },
  { &hf_rrc_nonCriticalExtensions_94, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_nonCriticalExtensions_94 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_laterNonCriticalExtensions_30(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_laterNonCriticalExtensions_30, T_laterNonCriticalExtensions_30_sequence);

  return offset;
}


static const per_sequence_t RadioBearerReconfigurationFailure_sequence[] = {
  { &hf_rrc_rrc_TransactionIdentifier, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RRC_TransactionIdentifier },
  { &hf_rrc_failureCause    , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_FailureCauseWithProtErr },
  { &hf_rrc_potentiallySuccesfulBearerList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RB_IdentityList },
  { &hf_rrc_laterNonCriticalExtensions_30, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_laterNonCriticalExtensions_30 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_RadioBearerReconfigurationFailure(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
#line 333 "../../asn1/rrc/rrc.cnf"
  col_append_str(actx->pinfo->cinfo, COL_INFO, "RadioBearerReconfigurationFailure");
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_RadioBearerReconfigurationFailure, RadioBearerReconfigurationFailure_sequence);




  return offset;
}


static const value_string rrc_T_deferredMeasurementControlReading_02_vals[] = {
  {   0, "true" },
  { 0, NULL }
};


static int
dissect_rrc_T_deferredMeasurementControlReading_02(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t RadioBearerReleaseComplete_v770ext_IEs_sequence[] = {
  { &hf_rrc_ext_UL_TimingAdvance, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_EXT_UL_TimingAdvance },
  { &hf_rrc_deferredMeasurementControlReading_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_deferredMeasurementControlReading_02 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_RadioBearerReleaseComplete_v770ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_RadioBearerReleaseComplete_v770ext_IEs, RadioBearerReleaseComplete_v770ext_IEs_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions_104_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_nonCriticalExtensions_104(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_nonCriticalExtensions_104, T_nonCriticalExtensions_104_sequence);

  return offset;
}


static const per_sequence_t T_v770NonCriticalExtensions_19_sequence[] = {
  { &hf_rrc_radioBearerReleaseComplete_v770ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RadioBearerReleaseComplete_v770ext_IEs },
  { &hf_rrc_nonCriticalExtensions_104, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_nonCriticalExtensions_104 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v770NonCriticalExtensions_19(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v770NonCriticalExtensions_19, T_v770NonCriticalExtensions_19_sequence);

  return offset;
}


static const per_sequence_t T_laterNonCriticalExtensions_32_sequence[] = {
  { &hf_rrc_radioBearerReleaseComplete_r3_add_ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_BIT_STRING },
  { &hf_rrc_v770NonCriticalExtensions_19, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v770NonCriticalExtensions_19 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_laterNonCriticalExtensions_32(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_laterNonCriticalExtensions_32, T_laterNonCriticalExtensions_32_sequence);

  return offset;
}


static const per_sequence_t RadioBearerReleaseComplete_sequence[] = {
  { &hf_rrc_rrc_TransactionIdentifier, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RRC_TransactionIdentifier },
  { &hf_rrc_ul_IntegProtActivationInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_IntegrityProtActivationInfo },
  { &hf_rrc_ul_TimingAdvance, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_TimingAdvance },
  { &hf_rrc_count_C_ActivationTime, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_ActivationTime },
  { &hf_rrc_dummy_13        , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RB_ActivationTimeInfoList },
  { &hf_rrc_ul_CounterSynchronisationInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_CounterSynchronisationInfo },
  { &hf_rrc_laterNonCriticalExtensions_32, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_laterNonCriticalExtensions_32 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_RadioBearerReleaseComplete(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
#line 341 "../../asn1/rrc/rrc.cnf"
  col_append_str(actx->pinfo->cinfo, COL_INFO, "RadioBearerReleaseComplete");
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_RadioBearerReleaseComplete, RadioBearerReleaseComplete_sequence);




  return offset;
}


static const per_sequence_t T_nonCriticalExtensions_105_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_nonCriticalExtensions_105(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_nonCriticalExtensions_105, T_nonCriticalExtensions_105_sequence);

  return offset;
}


static const per_sequence_t T_laterNonCriticalExtensions_33_sequence[] = {
  { &hf_rrc_radioBearerReleaseFailure_r3_add_ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_BIT_STRING },
  { &hf_rrc_nonCriticalExtensions_105, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_nonCriticalExtensions_105 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_laterNonCriticalExtensions_33(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_laterNonCriticalExtensions_33, T_laterNonCriticalExtensions_33_sequence);

  return offset;
}


static const per_sequence_t RadioBearerReleaseFailure_sequence[] = {
  { &hf_rrc_rrc_TransactionIdentifier, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RRC_TransactionIdentifier },
  { &hf_rrc_failureCause    , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_FailureCauseWithProtErr },
  { &hf_rrc_potentiallySuccesfulBearerList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RB_IdentityList },
  { &hf_rrc_laterNonCriticalExtensions_33, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_laterNonCriticalExtensions_33 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_RadioBearerReleaseFailure(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
#line 345 "../../asn1/rrc/rrc.cnf"
  col_append_str(actx->pinfo->cinfo, COL_INFO, "RadioBearerReleaseFailure");
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_RadioBearerReleaseFailure, RadioBearerReleaseFailure_sequence);




  return offset;
}


static const value_string rrc_T_deferredMeasurementControlReading_03_vals[] = {
  {   0, "true" },
  { 0, NULL }
};


static int
dissect_rrc_T_deferredMeasurementControlReading_03(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t RadioBearerSetupComplete_v770ext_IEs_sequence[] = {
  { &hf_rrc_ext_UL_TimingAdvance, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_EXT_UL_TimingAdvance },
  { &hf_rrc_deferredMeasurementControlReading_03, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_deferredMeasurementControlReading_03 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_RadioBearerSetupComplete_v770ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_RadioBearerSetupComplete_v770ext_IEs, RadioBearerSetupComplete_v770ext_IEs_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions_116_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_nonCriticalExtensions_116(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_nonCriticalExtensions_116, T_nonCriticalExtensions_116_sequence);

  return offset;
}


static const per_sequence_t T_v770NonCriticalExtensions_20_sequence[] = {
  { &hf_rrc_radioBearerSetupComplete_v770ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RadioBearerSetupComplete_v770ext_IEs },
  { &hf_rrc_nonCriticalExtensions_116, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_nonCriticalExtensions_116 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v770NonCriticalExtensions_20(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v770NonCriticalExtensions_20, T_v770NonCriticalExtensions_20_sequence);

  return offset;
}


static const per_sequence_t T_laterNonCriticalExtensions_35_sequence[] = {
  { &hf_rrc_radioBearerSetupComplete_r3_add_ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_BIT_STRING },
  { &hf_rrc_v770NonCriticalExtensions_20, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v770NonCriticalExtensions_20 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_laterNonCriticalExtensions_35(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_laterNonCriticalExtensions_35, T_laterNonCriticalExtensions_35_sequence);

  return offset;
}


static const per_sequence_t RadioBearerSetupComplete_sequence[] = {
  { &hf_rrc_rrc_TransactionIdentifier, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RRC_TransactionIdentifier },
  { &hf_rrc_ul_IntegProtActivationInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_IntegrityProtActivationInfo },
  { &hf_rrc_ul_TimingAdvance, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_TimingAdvance },
  { &hf_rrc_start_Value     , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_START_Value },
  { &hf_rrc_count_C_ActivationTime, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_ActivationTime },
  { &hf_rrc_dummy_13        , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RB_ActivationTimeInfoList },
  { &hf_rrc_ul_CounterSynchronisationInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_CounterSynchronisationInfo },
  { &hf_rrc_laterNonCriticalExtensions_35, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_laterNonCriticalExtensions_35 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_RadioBearerSetupComplete(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
#line 353 "../../asn1/rrc/rrc.cnf"
  col_append_str(actx->pinfo->cinfo, COL_INFO, "RadioBearerSetupComplete");
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_RadioBearerSetupComplete, RadioBearerSetupComplete_sequence);




  return offset;
}


static const per_sequence_t T_nonCriticalExtensions_117_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_nonCriticalExtensions_117(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_nonCriticalExtensions_117, T_nonCriticalExtensions_117_sequence);

  return offset;
}


static const per_sequence_t T_laterNonCriticalExtensions_36_sequence[] = {
  { &hf_rrc_radioBearerSetupFailure_r3_add_ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_BIT_STRING },
  { &hf_rrc_nonCriticalExtensions_117, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_nonCriticalExtensions_117 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_laterNonCriticalExtensions_36(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_laterNonCriticalExtensions_36, T_laterNonCriticalExtensions_36_sequence);

  return offset;
}


static const per_sequence_t RadioBearerSetupFailure_sequence[] = {
  { &hf_rrc_rrc_TransactionIdentifier, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RRC_TransactionIdentifier },
  { &hf_rrc_failureCause    , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_FailureCauseWithProtErr },
  { &hf_rrc_potentiallySuccesfulBearerList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RB_IdentityList },
  { &hf_rrc_laterNonCriticalExtensions_36, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_laterNonCriticalExtensions_36 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_RadioBearerSetupFailure(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
#line 357 "../../asn1/rrc/rrc.cnf"
  col_append_str(actx->pinfo->cinfo, COL_INFO, "RadioBearerSetupFailure");
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_RadioBearerSetupFailure, RadioBearerSetupFailure_sequence);




  return offset;
}


static const per_sequence_t T_nonCriticalExtensions_124_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_nonCriticalExtensions_124(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_nonCriticalExtensions_124, T_nonCriticalExtensions_124_sequence);

  return offset;
}


static const per_sequence_t T_laterNonCriticalExtensions_40_sequence[] = {
  { &hf_rrc_rrcConnectionReleaseComplete_r3_add_ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_BIT_STRING },
  { &hf_rrc_nonCriticalExtensions_124, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_nonCriticalExtensions_124 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_laterNonCriticalExtensions_40(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_laterNonCriticalExtensions_40, T_laterNonCriticalExtensions_40_sequence);

  return offset;
}


static const per_sequence_t RRCConnectionReleaseComplete_sequence[] = {
  { &hf_rrc_rrc_TransactionIdentifier, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RRC_TransactionIdentifier },
  { &hf_rrc_errorIndication , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_FailureCauseWithProtErr },
  { &hf_rrc_laterNonCriticalExtensions_40, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_laterNonCriticalExtensions_40 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_RRCConnectionReleaseComplete(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
#line 373 "../../asn1/rrc/rrc.cnf"
  col_append_str(actx->pinfo->cinfo, COL_INFO, "RRCConnectionReleaseComplete");
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_RRCConnectionReleaseComplete, RRCConnectionReleaseComplete_sequence);




  return offset;
}


static const value_string rrc_MaxHcContextSpace_vals[] = {
  {   0, "dummy" },
  {   1, "by1024" },
  {   2, "by2048" },
  {   3, "by4096" },
  {   4, "by8192" },
  { 0, NULL }
};


static int
dissect_rrc_MaxHcContextSpace(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     5, NULL, FALSE, 0, NULL);

  return offset;
}


static const value_string rrc_T_supportForRfc2507_vals[] = {
  {   0, "notSupported" },
  {   1, "supported" },
  { 0, NULL }
};

static const per_choice_t T_supportForRfc2507_choice[] = {
  {   0, &hf_rrc_notSupported    , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   1, &hf_rrc_supported_01    , ASN1_NO_EXTENSIONS     , dissect_rrc_MaxHcContextSpace },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_supportForRfc2507(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_supportForRfc2507, T_supportForRfc2507_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t PDCP_Capability_sequence[] = {
  { &hf_rrc_losslessSRNS_RelocationSupport, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { &hf_rrc_supportForRfc2507, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_supportForRfc2507 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_PDCP_Capability(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_PDCP_Capability, PDCP_Capability_sequence);

  return offset;
}


static const value_string rrc_TotalRLC_AM_BufferSize_vals[] = {
  {   0, "dummy" },
  {   1, "kb10" },
  {   2, "kb50" },
  {   3, "kb100" },
  {   4, "kb150" },
  {   5, "kb500" },
  {   6, "kb1000" },
  {   7, "spare" },
  { 0, NULL }
};


static int
dissect_rrc_TotalRLC_AM_BufferSize(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     8, NULL, FALSE, 0, NULL);

  return offset;
}


static const value_string rrc_MaximumRLC_WindowSize_vals[] = {
  {   0, "mws2047" },
  {   1, "mws4095" },
  { 0, NULL }
};


static int
dissect_rrc_MaximumRLC_WindowSize(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     2, NULL, FALSE, 0, NULL);

  return offset;
}


static const value_string rrc_MaximumAM_EntityNumberRLC_Cap_vals[] = {
  {   0, "dummy" },
  {   1, "am4" },
  {   2, "am5" },
  {   3, "am6" },
  {   4, "am8" },
  {   5, "am16" },
  {   6, "am30" },
  { 0, NULL }
};


static int
dissect_rrc_MaximumAM_EntityNumberRLC_Cap(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     7, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t RLC_Capability_sequence[] = {
  { &hf_rrc_totalRLC_AM_BufferSize, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TotalRLC_AM_BufferSize },
  { &hf_rrc_maximumRLC_WindowSize, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MaximumRLC_WindowSize },
  { &hf_rrc_maximumAM_EntityNumber, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MaximumAM_EntityNumberRLC_Cap },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_RLC_Capability(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_RLC_Capability, RLC_Capability_sequence);

  return offset;
}


static const value_string rrc_MaxNoBits_vals[] = {
  {   0, "b640" },
  {   1, "b1280" },
  {   2, "b2560" },
  {   3, "b3840" },
  {   4, "b5120" },
  {   5, "b6400" },
  {   6, "b7680" },
  {   7, "b8960" },
  {   8, "b10240" },
  {   9, "b20480" },
  {  10, "b40960" },
  {  11, "b81920" },
  {  12, "b163840" },
  { 0, NULL }
};


static int
dissect_rrc_MaxNoBits(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     13, NULL, FALSE, 0, NULL);

  return offset;
}


static const value_string rrc_TurboSupport_vals[] = {
  {   0, "notSupported" },
  {   1, "supported" },
  { 0, NULL }
};

static const per_choice_t TurboSupport_choice[] = {
  {   0, &hf_rrc_notSupported    , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   1, &hf_rrc_supported_05    , ASN1_NO_EXTENSIONS     , dissect_rrc_MaxNoBits },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_TurboSupport(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_TurboSupport, TurboSupport_choice,
                                 NULL);

  return offset;
}


static const value_string rrc_MaxSimultaneousTransChsDL_vals[] = {
  {   0, "e4" },
  {   1, "e8" },
  {   2, "e16" },
  {   3, "e32" },
  { 0, NULL }
};


static int
dissect_rrc_MaxSimultaneousTransChsDL(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     4, NULL, FALSE, 0, NULL);

  return offset;
}



static int
dissect_rrc_MaxSimultaneousCCTrCH_Count(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            1U, 8U, NULL, FALSE);

  return offset;
}


static const value_string rrc_MaxTransportBlocksDL_vals[] = {
  {   0, "tb4" },
  {   1, "tb8" },
  {   2, "tb16" },
  {   3, "tb32" },
  {   4, "tb48" },
  {   5, "tb64" },
  {   6, "tb96" },
  {   7, "tb128" },
  {   8, "tb256" },
  {   9, "tb512" },
  { 0, NULL }
};


static int
dissect_rrc_MaxTransportBlocksDL(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     10, NULL, FALSE, 0, NULL);

  return offset;
}


static const value_string rrc_MaxNumberOfTFC_DL_vals[] = {
  {   0, "tfc16" },
  {   1, "tfc32" },
  {   2, "tfc48" },
  {   3, "tfc64" },
  {   4, "tfc96" },
  {   5, "tfc128" },
  {   6, "tfc256" },
  {   7, "tfc512" },
  {   8, "tfc1024" },
  { 0, NULL }
};


static int
dissect_rrc_MaxNumberOfTFC_DL(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     9, NULL, FALSE, 0, NULL);

  return offset;
}


static const value_string rrc_MaxNumberOfTF_vals[] = {
  {   0, "tf32" },
  {   1, "tf64" },
  {   2, "tf128" },
  {   3, "tf256" },
  {   4, "tf512" },
  {   5, "tf1024" },
  { 0, NULL }
};


static int
dissect_rrc_MaxNumberOfTF(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     6, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t DL_TransChCapability_sequence[] = {
  { &hf_rrc_maxNoBitsReceived, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MaxNoBits },
  { &hf_rrc_maxConvCodeBitsReceived, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MaxNoBits },
  { &hf_rrc_turboDecodingSupport, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TurboSupport },
  { &hf_rrc_maxSimultaneousTransChs, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MaxSimultaneousTransChsDL },
  { &hf_rrc_maxSimultaneousCCTrCH_Count, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MaxSimultaneousCCTrCH_Count },
  { &hf_rrc_maxReceivedTransportBlocks, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MaxTransportBlocksDL },
  { &hf_rrc_maxNumberOfTFC  , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MaxNumberOfTFC_DL },
  { &hf_rrc_maxNumberOfTF   , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MaxNumberOfTF },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_DL_TransChCapability(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_DL_TransChCapability, DL_TransChCapability_sequence);

  return offset;
}


static const value_string rrc_MaxSimultaneousTransChsUL_vals[] = {
  {   0, "dummy" },
  {   1, "e4" },
  {   2, "e8" },
  {   3, "e16" },
  {   4, "e32" },
  { 0, NULL }
};


static int
dissect_rrc_MaxSimultaneousTransChsUL(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     5, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t T_tdd_12_sequence[] = {
  { &hf_rrc_maxSimultaneousCCTrCH_Count, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MaxSimultaneousCCTrCH_Count },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_tdd_12(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_tdd_12, T_tdd_12_sequence);

  return offset;
}


static const value_string rrc_T_modeSpecificInfo_18_vals[] = {
  {   0, "fdd" },
  {   1, "tdd" },
  { 0, NULL }
};

static const per_choice_t T_modeSpecificInfo_18_choice[] = {
  {   0, &hf_rrc_fdd_45          , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   1, &hf_rrc_tdd_13          , ASN1_NO_EXTENSIONS     , dissect_rrc_T_tdd_12 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_modeSpecificInfo_18(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_modeSpecificInfo_18, T_modeSpecificInfo_18_choice,
                                 NULL);

  return offset;
}


static const value_string rrc_MaxTransportBlocksUL_vals[] = {
  {   0, "dummy" },
  {   1, "tb4" },
  {   2, "tb8" },
  {   3, "tb16" },
  {   4, "tb32" },
  {   5, "tb48" },
  {   6, "tb64" },
  {   7, "tb96" },
  {   8, "tb128" },
  {   9, "tb256" },
  {  10, "tb512" },
  { 0, NULL }
};


static int
dissect_rrc_MaxTransportBlocksUL(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     11, NULL, FALSE, 0, NULL);

  return offset;
}


static const value_string rrc_MaxNumberOfTFC_UL_vals[] = {
  {   0, "dummy1" },
  {   1, "dummy2" },
  {   2, "tfc16" },
  {   3, "tfc32" },
  {   4, "tfc48" },
  {   5, "tfc64" },
  {   6, "tfc96" },
  {   7, "tfc128" },
  {   8, "tfc256" },
  {   9, "tfc512" },
  {  10, "tfc1024" },
  { 0, NULL }
};


static int
dissect_rrc_MaxNumberOfTFC_UL(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     11, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t UL_TransChCapability_sequence[] = {
  { &hf_rrc_maxNoBitsTransmitted, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MaxNoBits },
  { &hf_rrc_maxConvCodeBitsTransmitted, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MaxNoBits },
  { &hf_rrc_turboEncodingSupport, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TurboSupport },
  { &hf_rrc_maxSimultaneousTransChs_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MaxSimultaneousTransChsUL },
  { &hf_rrc_modeSpecificInfo_18, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_modeSpecificInfo_18 },
  { &hf_rrc_maxTransmittedBlocks, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MaxTransportBlocksUL },
  { &hf_rrc_maxNumberOfTFC_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MaxNumberOfTFC_UL },
  { &hf_rrc_maxNumberOfTF   , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MaxNumberOfTF },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UL_TransChCapability(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UL_TransChCapability, UL_TransChCapability_sequence);

  return offset;
}


static const per_sequence_t TransportChannelCapability_sequence[] = {
  { &hf_rrc_dl_TransChCapability, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DL_TransChCapability },
  { &hf_rrc_ul_TransChCapability, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UL_TransChCapability },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_TransportChannelCapability(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_TransportChannelCapability, TransportChannelCapability_sequence);

  return offset;
}



static int
dissect_rrc_UE_PowerClass(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            1U, 4U, NULL, FALSE);

  return offset;
}


static const value_string rrc_TxRxFrequencySeparation_vals[] = {
  {   0, "default-TxRx-separation" },
  {   1, "spare2" },
  {   2, "spare1" },
  { 0, NULL }
};


static int
dissect_rrc_TxRxFrequencySeparation(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     3, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t T_fddRF_Capability_sequence[] = {
  { &hf_rrc_ue_PowerClass   , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UE_PowerClass },
  { &hf_rrc_txRxFrequencySeparation, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TxRxFrequencySeparation },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_fddRF_Capability(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_fddRF_Capability, T_fddRF_Capability_sequence);

  return offset;
}


static const value_string rrc_RadioFrequencyBandTDDList_vals[] = {
  {   0, "a" },
  {   1, "b" },
  {   2, "c" },
  {   3, "ab" },
  {   4, "ac" },
  {   5, "bc" },
  {   6, "abc" },
  {   7, "spare" },
  { 0, NULL }
};


static int
dissect_rrc_RadioFrequencyBandTDDList(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     8, NULL, FALSE, 0, NULL);

  return offset;
}


static const value_string rrc_ChipRateCapability_vals[] = {
  {   0, "mcps3-84" },
  {   1, "mcps1-28" },
  { 0, NULL }
};


static int
dissect_rrc_ChipRateCapability(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     2, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t T_tddRF_Capability_sequence[] = {
  { &hf_rrc_ue_PowerClass   , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UE_PowerClass },
  { &hf_rrc_radioFrequencyTDDBandList, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RadioFrequencyBandTDDList },
  { &hf_rrc_chipRateCapability, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_ChipRateCapability },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_tddRF_Capability(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_tddRF_Capability, T_tddRF_Capability_sequence);

  return offset;
}


static const per_sequence_t RF_Capability_sequence[] = {
  { &hf_rrc_fddRF_Capability, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_fddRF_Capability },
  { &hf_rrc_tddRF_Capability, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_tddRF_Capability },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_RF_Capability(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_RF_Capability, RF_Capability_sequence);

  return offset;
}


static const value_string rrc_MaxNoPhysChBitsReceived_vals[] = {
  {   0, "dummy" },
  {   1, "b1200" },
  {   2, "b2400" },
  {   3, "b3600" },
  {   4, "b4800" },
  {   5, "b7200" },
  {   6, "b9600" },
  {   7, "b14400" },
  {   8, "b19200" },
  {   9, "b28800" },
  {  10, "b38400" },
  {  11, "b48000" },
  {  12, "b57600" },
  {  13, "b67200" },
  {  14, "b76800" },
  { 0, NULL }
};


static int
dissect_rrc_MaxNoPhysChBitsReceived(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     15, NULL, FALSE, 0, NULL);

  return offset;
}


static const value_string rrc_MaxNoSCCPCH_RL_vals[] = {
  {   0, "rl1" },
  { 0, NULL }
};


static int
dissect_rrc_MaxNoSCCPCH_RL(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t T_supported_02_sequence[] = {
  { &hf_rrc_maxNoSCCPCH_RL  , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MaxNoSCCPCH_RL },
  { &hf_rrc_simultaneousSCCPCH_DPCH_DPDCH_Reception, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_supported_02(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_supported_02, T_supported_02_sequence);

  return offset;
}


static const value_string rrc_SimultaneousSCCPCH_DPCH_Reception_vals[] = {
  {   0, "notSupported" },
  {   1, "supported" },
  { 0, NULL }
};

static const per_choice_t SimultaneousSCCPCH_DPCH_Reception_choice[] = {
  {   0, &hf_rrc_notSupported    , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   1, &hf_rrc_supported_04    , ASN1_NO_EXTENSIONS     , dissect_rrc_T_supported_02 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_SimultaneousSCCPCH_DPCH_Reception(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_SimultaneousSCCPCH_DPCH_Reception, SimultaneousSCCPCH_DPCH_Reception_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t DL_PhysChCapabilityFDD_sequence[] = {
  { &hf_rrc_maxNoDPCH_PDSCH_Codes, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_1_8 },
  { &hf_rrc_maxNoPhysChBitsReceived, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MaxNoPhysChBitsReceived },
  { &hf_rrc_supportForSF_512, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { &hf_rrc_dummy_28        , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { &hf_rrc_dummy2_05       , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SimultaneousSCCPCH_DPCH_Reception },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_DL_PhysChCapabilityFDD(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_DL_PhysChCapabilityFDD, DL_PhysChCapabilityFDD_sequence);

  return offset;
}


static const value_string rrc_MaxNoDPDCH_BitsTransmitted_vals[] = {
  {   0, "b600" },
  {   1, "b1200" },
  {   2, "b2400" },
  {   3, "b4800" },
  {   4, "b9600" },
  {   5, "b19200" },
  {   6, "b28800" },
  {   7, "b38400" },
  {   8, "b48000" },
  {   9, "b57600" },
  { 0, NULL }
};


static int
dissect_rrc_MaxNoDPDCH_BitsTransmitted(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     10, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t UL_PhysChCapabilityFDD_sequence[] = {
  { &hf_rrc_maxNoDPDCH_BitsTransmitted, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MaxNoDPDCH_BitsTransmitted },
  { &hf_rrc_dummy_28        , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UL_PhysChCapabilityFDD(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UL_PhysChCapabilityFDD, UL_PhysChCapabilityFDD_sequence);

  return offset;
}


static const per_sequence_t T_fddPhysChCapability_sequence[] = {
  { &hf_rrc_downlinkPhysChCapability, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DL_PhysChCapabilityFDD },
  { &hf_rrc_uplinkPhysChCapability, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UL_PhysChCapabilityFDD },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_fddPhysChCapability(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_fddPhysChCapability, T_fddPhysChCapability_sequence);

  return offset;
}



static int
dissect_rrc_MaxTS_PerFrame(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            1U, 14U, NULL, FALSE);

  return offset;
}



static int
dissect_rrc_MaxPhysChPerFrame(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            1U, 224U, NULL, FALSE);

  return offset;
}


static const value_string rrc_MinimumSF_DL_vals[] = {
  {   0, "sf1" },
  {   1, "sf16" },
  { 0, NULL }
};


static int
dissect_rrc_MinimumSF_DL(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     2, NULL, FALSE, 0, NULL);

  return offset;
}



static int
dissect_rrc_MaxPhysChPerTS(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            1U, 16U, NULL, FALSE);

  return offset;
}


static const per_sequence_t DL_PhysChCapabilityTDD_sequence[] = {
  { &hf_rrc_maxTS_PerFrame  , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MaxTS_PerFrame },
  { &hf_rrc_maxPhysChPerFrame, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MaxPhysChPerFrame },
  { &hf_rrc_minimumSF       , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MinimumSF_DL },
  { &hf_rrc_supportOfPDSCH  , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { &hf_rrc_maxPhysChPerTS  , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MaxPhysChPerTS },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_DL_PhysChCapabilityTDD(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_DL_PhysChCapabilityTDD, DL_PhysChCapabilityTDD_sequence);

  return offset;
}


static const value_string rrc_MaxPhysChPerTimeslot_vals[] = {
  {   0, "ts1" },
  {   1, "ts2" },
  { 0, NULL }
};


static int
dissect_rrc_MaxPhysChPerTimeslot(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     2, NULL, FALSE, 0, NULL);

  return offset;
}


static const value_string rrc_MinimumSF_UL_vals[] = {
  {   0, "sf1" },
  {   1, "sf2" },
  {   2, "sf4" },
  {   3, "sf8" },
  {   4, "dummy" },
  { 0, NULL }
};


static int
dissect_rrc_MinimumSF_UL(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     5, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t UL_PhysChCapabilityTDD_sequence[] = {
  { &hf_rrc_maxTS_PerFrame  , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MaxTS_PerFrame },
  { &hf_rrc_maxPhysChPerTimeslot, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MaxPhysChPerTimeslot },
  { &hf_rrc_minimumSF_02    , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MinimumSF_UL },
  { &hf_rrc_supportOfPUSCH  , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UL_PhysChCapabilityTDD(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UL_PhysChCapabilityTDD, UL_PhysChCapabilityTDD_sequence);

  return offset;
}


static const per_sequence_t T_tddPhysChCapability_sequence[] = {
  { &hf_rrc_downlinkPhysChCapability_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DL_PhysChCapabilityTDD },
  { &hf_rrc_uplinkPhysChCapability_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UL_PhysChCapabilityTDD },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_tddPhysChCapability(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_tddPhysChCapability, T_tddPhysChCapability_sequence);

  return offset;
}


static const per_sequence_t PhysicalChannelCapability_sequence[] = {
  { &hf_rrc_fddPhysChCapability, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_fddPhysChCapability },
  { &hf_rrc_tddPhysChCapability, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_tddPhysChCapability },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_PhysicalChannelCapability(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_PhysicalChannelCapability, PhysicalChannelCapability_sequence);

  return offset;
}


static const per_sequence_t MultiRAT_Capability_sequence[] = {
  { &hf_rrc_supportOfGSM    , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { &hf_rrc_supportOfMulticarrier, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_MultiRAT_Capability(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_MultiRAT_Capability, MultiRAT_Capability_sequence);

  return offset;
}


static const value_string rrc_MultiModeCapability_vals[] = {
  {   0, "tdd" },
  {   1, "fdd" },
  {   2, "fdd-tdd" },
  { 0, NULL }
};


static int
dissect_rrc_MultiModeCapability(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     3, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t UE_MultiModeRAT_Capability_sequence[] = {
  { &hf_rrc_multiRAT_CapabilityList, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MultiRAT_Capability },
  { &hf_rrc_multiModeCapability, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MultiModeCapability },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UE_MultiModeRAT_Capability(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UE_MultiModeRAT_Capability, UE_MultiModeRAT_Capability_sequence);

  return offset;
}


static const value_string rrc_NetworkAssistedGPS_Supported_vals[] = {
  {   0, "networkBased" },
  {   1, "ue-Based" },
  {   2, "bothNetworkAndUE-Based" },
  {   3, "noNetworkAssistedGPS" },
  { 0, NULL }
};


static int
dissect_rrc_NetworkAssistedGPS_Supported(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     4, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t UE_Positioning_Capability_sequence[] = {
  { &hf_rrc_standaloneLocMethodsSupported, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { &hf_rrc_ue_BasedOTDOA_Supported, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { &hf_rrc_networkAssistedGPS_Supported, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_NetworkAssistedGPS_Supported },
  { &hf_rrc_supportForUE_GPS_TimingOfCellFrames, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { &hf_rrc_supportForIPDL  , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UE_Positioning_Capability(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UE_Positioning_Capability, UE_Positioning_Capability_sequence);

  return offset;
}


static const per_sequence_t GSM_Measurements_sequence[] = {
  { &hf_rrc_gsm900          , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { &hf_rrc_dcs1800         , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { &hf_rrc_gsm1900         , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_GSM_Measurements(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_GSM_Measurements, GSM_Measurements_sequence);

  return offset;
}


static const per_sequence_t CompressedModeMeasCapability_sequence[] = {
  { &hf_rrc_fdd_Measurements, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { &hf_rrc_tdd_Measurements, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_BOOLEAN },
  { &hf_rrc_gsm_Measurements, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_GSM_Measurements },
  { &hf_rrc_multiCarrierMeasurements, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_BOOLEAN },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_CompressedModeMeasCapability(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_CompressedModeMeasCapability, CompressedModeMeasCapability_sequence);

  return offset;
}


static const per_sequence_t MeasurementCapability_sequence[] = {
  { &hf_rrc_downlinkCompressedMode, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CompressedModeMeasCapability },
  { &hf_rrc_uplinkCompressedMode, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CompressedModeMeasCapability },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_MeasurementCapability(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_MeasurementCapability, MeasurementCapability_sequence);

  return offset;
}


static const per_sequence_t UE_RadioAccessCapability_sequence[] = {
  { &hf_rrc_pdcp_Capability_02, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PDCP_Capability },
  { &hf_rrc_rlc_Capability_04, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RLC_Capability },
  { &hf_rrc_transportChannelCapability, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TransportChannelCapability },
  { &hf_rrc_rf_Capability_02, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RF_Capability },
  { &hf_rrc_physicalChannelCapability_05, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PhysicalChannelCapability },
  { &hf_rrc_ue_MultiModeRAT_Capability, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UE_MultiModeRAT_Capability },
  { &hf_rrc_securityCapability, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SecurityCapability },
  { &hf_rrc_ue_positioning_Capability, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UE_Positioning_Capability },
  { &hf_rrc_measurementCapability_04, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MeasurementCapability },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UE_RadioAccessCapability(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UE_RadioAccessCapability, UE_RadioAccessCapability_sequence);

  return offset;
}



static int
dissect_rrc_GSM_Classmark2(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
#line 571 "../../asn1/rrc/rrc.cnf"
  tvbuff_t *parameter_tvb=NULL;

  offset = dissect_per_octet_string(tvb, offset, actx, tree, hf_index,
                                       5, 5, FALSE, &parameter_tvb);

  if (parameter_tvb){
    de_ms_cm_2(parameter_tvb, tree, actx->pinfo, 2, tvb_get_guint8(parameter_tvb,1), NULL, 0);
  }



  return offset;
}



static int
dissect_rrc_GSM_Classmark3(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
#line 579 "../../asn1/rrc/rrc.cnf"
  tvbuff_t *parameter_tvb=NULL;

  offset = dissect_per_octet_string(tvb, offset, actx, tree, hf_index,
                                       1, 32, FALSE, &parameter_tvb);

  if (parameter_tvb){
    de_ms_cm_3(parameter_tvb, tree, actx->pinfo, 0, tvb_length(parameter_tvb), NULL, 0);
  }



  return offset;
}


static const per_sequence_t T_gsm_15_sequence[] = {
  { &hf_rrc_gsm_Classmark2  , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_GSM_Classmark2 },
  { &hf_rrc_gsm_Classmark3  , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_GSM_Classmark3 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_gsm_15(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_gsm_15, T_gsm_15_sequence);

  return offset;
}


static const per_sequence_t T_cdma2000_01_sequence[] = {
  { &hf_rrc_cdma2000_MessageList, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CDMA2000_MessageList },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_cdma2000_01(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_cdma2000_01, T_cdma2000_01_sequence);

  return offset;
}


static const value_string rrc_InterRAT_UE_RadioAccessCapability_vals[] = {
  {   0, "gsm" },
  {   1, "cdma2000" },
  { 0, NULL }
};

static const per_choice_t InterRAT_UE_RadioAccessCapability_choice[] = {
  {   0, &hf_rrc_gsm_17          , ASN1_NO_EXTENSIONS     , dissect_rrc_T_gsm_15 },
  {   1, &hf_rrc_cdma2000_01     , ASN1_NO_EXTENSIONS     , dissect_rrc_T_cdma2000_01 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_InterRAT_UE_RadioAccessCapability(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_InterRAT_UE_RadioAccessCapability, InterRAT_UE_RadioAccessCapability_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t InterRAT_UE_RadioAccessCapabilityList_sequence_of[1] = {
  { &hf_rrc_InterRAT_UE_RadioAccessCapabilityList_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_InterRAT_UE_RadioAccessCapability },
};

static int
dissect_rrc_InterRAT_UE_RadioAccessCapabilityList(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_InterRAT_UE_RadioAccessCapabilityList, InterRAT_UE_RadioAccessCapabilityList_sequence_of,
                                                  1, maxInterSysMessages, FALSE);

  return offset;
}


static const value_string rrc_RadioFrequencyBandFDD_vals[] = {
  {   0, "fdd2100" },
  {   1, "fdd1900" },
  {   2, "fdd1800" },
  {   3, "bandVI" },
  {   4, "bandIV" },
  {   5, "bandV" },
  {   6, "bandVII" },
  {   7, "extension-indicator" },
  { 0, NULL }
};


static int
dissect_rrc_RadioFrequencyBandFDD(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     8, NULL, FALSE, 0, NULL);

  return offset;
}


static const value_string rrc_UE_PowerClassExt_vals[] = {
  {   0, "class1" },
  {   1, "class2" },
  {   2, "class3" },
  {   3, "class4" },
  {   4, "spare4" },
  {   5, "spare3" },
  {   6, "spare2" },
  {   7, "spare1" },
  { 0, NULL }
};


static int
dissect_rrc_UE_PowerClassExt(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     8, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t T_fddRF_Capability_03_sequence[] = {
  { &hf_rrc_ue_PowerClass_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UE_PowerClassExt },
  { &hf_rrc_txRxFrequencySeparation, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TxRxFrequencySeparation },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_fddRF_Capability_03(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_fddRF_Capability_03, T_fddRF_Capability_03_sequence);

  return offset;
}


static const per_sequence_t CompressedModeMeasCapabFDD_sequence[] = {
  { &hf_rrc_radioFrequencyBandFDD, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RadioFrequencyBandFDD },
  { &hf_rrc_dl_MeasurementsFDD, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { &hf_rrc_ul_MeasurementsFDD, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_CompressedModeMeasCapabFDD(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_CompressedModeMeasCapabFDD, CompressedModeMeasCapabFDD_sequence);

  return offset;
}


static const per_sequence_t CompressedModeMeasCapabFDDList_sequence_of[1] = {
  { &hf_rrc_CompressedModeMeasCapabFDDList_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CompressedModeMeasCapabFDD },
};

static int
dissect_rrc_CompressedModeMeasCapabFDDList(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_CompressedModeMeasCapabFDDList, CompressedModeMeasCapabFDDList_sequence_of,
                                                  1, maxFreqBandsFDD, FALSE);

  return offset;
}


static const value_string rrc_RadioFrequencyBandTDD_vals[] = {
  {   0, "a" },
  {   1, "b" },
  {   2, "c" },
  {   3, "d" },
  { 0, NULL }
};


static int
dissect_rrc_RadioFrequencyBandTDD(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     4, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t CompressedModeMeasCapabTDD_sequence[] = {
  { &hf_rrc_radioFrequencyBandTDD, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RadioFrequencyBandTDD },
  { &hf_rrc_dl_MeasurementsTDD, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { &hf_rrc_ul_MeasurementsTDD, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_CompressedModeMeasCapabTDD(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_CompressedModeMeasCapabTDD, CompressedModeMeasCapabTDD_sequence);

  return offset;
}


static const per_sequence_t CompressedModeMeasCapabTDDList_sequence_of[1] = {
  { &hf_rrc_CompressedModeMeasCapabTDDList_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CompressedModeMeasCapabTDD },
};

static int
dissect_rrc_CompressedModeMeasCapabTDDList(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_CompressedModeMeasCapabTDDList, CompressedModeMeasCapabTDDList_sequence_of,
                                                  1, maxFreqBandsTDD, FALSE);

  return offset;
}


static const value_string rrc_RadioFrequencyBandGSM_vals[] = {
  {   0, "gsm450" },
  {   1, "gsm480" },
  {   2, "gsm850" },
  {   3, "gsm900P" },
  {   4, "gsm900E" },
  {   5, "gsm1800" },
  {   6, "gsm1900" },
  {   7, "spare9" },
  {   8, "spare8" },
  {   9, "spare7" },
  {  10, "spare6" },
  {  11, "spare5" },
  {  12, "spare4" },
  {  13, "spare3" },
  {  14, "spare2" },
  {  15, "spare1" },
  { 0, NULL }
};


static int
dissect_rrc_RadioFrequencyBandGSM(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     16, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t CompressedModeMeasCapabGSM_sequence[] = {
  { &hf_rrc_radioFrequencyBandGSM, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RadioFrequencyBandGSM },
  { &hf_rrc_dl_MeasurementsGSM, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { &hf_rrc_ul_MeasurementsGSM, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_CompressedModeMeasCapabGSM(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_CompressedModeMeasCapabGSM, CompressedModeMeasCapabGSM_sequence);

  return offset;
}


static const per_sequence_t CompressedModeMeasCapabGSMList_sequence_of[1] = {
  { &hf_rrc_CompressedModeMeasCapabGSMList_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CompressedModeMeasCapabGSM },
};

static int
dissect_rrc_CompressedModeMeasCapabGSMList(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_CompressedModeMeasCapabGSMList, CompressedModeMeasCapabGSMList_sequence_of,
                                                  1, maxFreqBandsGSM, FALSE);

  return offset;
}


static const per_sequence_t CompressedModeMeasCapabMC_sequence[] = {
  { &hf_rrc_dl_MeasurementsMC, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { &hf_rrc_ul_MeasurementsMC, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_CompressedModeMeasCapabMC(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_CompressedModeMeasCapabMC, CompressedModeMeasCapabMC_sequence);

  return offset;
}


static const per_sequence_t MeasurementCapabilityExt_sequence[] = {
  { &hf_rrc_compressedModeMeasCapabFDDList, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CompressedModeMeasCapabFDDList },
  { &hf_rrc_compressedModeMeasCapabTDDList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CompressedModeMeasCapabTDDList },
  { &hf_rrc_compressedModeMeasCapabGSMList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CompressedModeMeasCapabGSMList },
  { &hf_rrc_compressedModeMeasCapabMC, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CompressedModeMeasCapabMC },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_MeasurementCapabilityExt(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_MeasurementCapabilityExt, MeasurementCapabilityExt_sequence);

  return offset;
}


static const per_sequence_t UE_RadioAccessCapabBandFDD_sequence[] = {
  { &hf_rrc_radioFrequencyBandFDD, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RadioFrequencyBandFDD },
  { &hf_rrc_fddRF_Capability_03, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_fddRF_Capability_03 },
  { &hf_rrc_measurementCapability_05, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MeasurementCapabilityExt },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UE_RadioAccessCapabBandFDD(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UE_RadioAccessCapabBandFDD, UE_RadioAccessCapabBandFDD_sequence);

  return offset;
}


static const per_sequence_t UE_RadioAccessCapabBandFDDList_sequence_of[1] = {
  { &hf_rrc_UE_RadioAccessCapabBandFDDList_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UE_RadioAccessCapabBandFDD },
};

static int
dissect_rrc_UE_RadioAccessCapabBandFDDList(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_UE_RadioAccessCapabBandFDDList, UE_RadioAccessCapabBandFDDList_sequence_of,
                                                  1, maxFreqBandsFDD, FALSE);

  return offset;
}


static const per_sequence_t UE_RadioAccessCapability_v370ext_sequence[] = {
  { &hf_rrc_ue_RadioAccessCapabBandFDDList, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UE_RadioAccessCapabBandFDDList },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UE_RadioAccessCapability_v370ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UE_RadioAccessCapability_v370ext, UE_RadioAccessCapability_v370ext_sequence);

  return offset;
}


static const per_sequence_t RRCConnectionSetupComplete_v370ext_sequence[] = {
  { &hf_rrc_ue_RadioAccessCapability_v370ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_RadioAccessCapability_v370ext },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_RRCConnectionSetupComplete_v370ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_RRCConnectionSetupComplete_v370ext, RRCConnectionSetupComplete_v370ext_sequence);

  return offset;
}


static const per_sequence_t UE_PositioningCapabilityExt_v380_sequence[] = {
  { &hf_rrc_rx_tx_TimeDifferenceType2Capable, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UE_PositioningCapabilityExt_v380(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UE_PositioningCapabilityExt_v380, UE_PositioningCapabilityExt_v380_sequence);

  return offset;
}


static const per_sequence_t UE_RadioAccessCapability_v380ext_sequence[] = {
  { &hf_rrc_ue_PositioningCapabilityExt_v380, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UE_PositioningCapabilityExt_v380 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UE_RadioAccessCapability_v380ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UE_RadioAccessCapability_v380ext, UE_RadioAccessCapability_v380ext_sequence);

  return offset;
}


static const value_string rrc_SupportOfDedicatedPilotsForChEstimation_vals[] = {
  {   0, "true" },
  { 0, NULL }
};


static int
dissect_rrc_SupportOfDedicatedPilotsForChEstimation(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t DL_PhysChCapabilityFDD_v380ext_sequence[] = {
  { &hf_rrc_dummy_29        , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_SupportOfDedicatedPilotsForChEstimation },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_DL_PhysChCapabilityFDD_v380ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_DL_PhysChCapabilityFDD_v380ext, DL_PhysChCapabilityFDD_v380ext_sequence);

  return offset;
}


static const per_sequence_t RRCConnectionSetupComplete_v380ext_IEs_sequence[] = {
  { &hf_rrc_ue_RadioAccessCapability_v380ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_RadioAccessCapability_v380ext },
  { &hf_rrc_dl_PhysChCapabilityFDD_v380ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DL_PhysChCapabilityFDD_v380ext },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_RRCConnectionSetupComplete_v380ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_RRCConnectionSetupComplete_v380ext_IEs, RRCConnectionSetupComplete_v380ext_IEs_sequence);

  return offset;
}


static const value_string rrc_T_validity_CellPCH_UraPCH_vals[] = {
  {   0, "true" },
  { 0, NULL }
};


static int
dissect_rrc_T_validity_CellPCH_UraPCH(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t UE_PositioningCapabilityExt_v3a0_sequence[] = {
  { &hf_rrc_validity_CellPCH_UraPCH, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_validity_CellPCH_UraPCH },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UE_PositioningCapabilityExt_v3a0(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UE_PositioningCapabilityExt_v3a0, UE_PositioningCapabilityExt_v3a0_sequence);

  return offset;
}


static const per_sequence_t UE_RadioAccessCapability_v3a0ext_sequence[] = {
  { &hf_rrc_ue_PositioningCapabilityExt_v3a0, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UE_PositioningCapabilityExt_v3a0 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UE_RadioAccessCapability_v3a0ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UE_RadioAccessCapability_v3a0ext, UE_RadioAccessCapability_v3a0ext_sequence);

  return offset;
}


static const per_sequence_t RRCConnectionSetupComplete_v3a0ext_IEs_sequence[] = {
  { &hf_rrc_ue_RadioAccessCapability_v3a0ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_RadioAccessCapability_v3a0ext },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_RRCConnectionSetupComplete_v3a0ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_RRCConnectionSetupComplete_v3a0ext_IEs, RRCConnectionSetupComplete_v3a0ext_IEs_sequence);

  return offset;
}



static int
dissect_rrc_T_rrcConnectionSetupComplete_r3_add_ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_bit_string_containing_pdu_new(tvb, offset, actx, tree, hf_index,
                                                              NO_BOUND, NO_BOUND, FALSE, dissect_RRCConnectionSetupComplete_r3_add_ext_IEs_PDU);

  return offset;
}


static const value_string rrc_T_sfn_sfnType2Capability_vals[] = {
  {   0, "true" },
  { 0, NULL }
};


static int
dissect_rrc_T_sfn_sfnType2Capability(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t UE_PositioningCapabilityExt_v3g0_sequence[] = {
  { &hf_rrc_sfn_sfnType2Capability, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_sfn_sfnType2Capability },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UE_PositioningCapabilityExt_v3g0(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UE_PositioningCapabilityExt_v3g0, UE_PositioningCapabilityExt_v3g0_sequence);

  return offset;
}


static const per_sequence_t UE_RadioAccessCapability_v3g0ext_sequence[] = {
  { &hf_rrc_ue_PositioningCapabilityExt_v3g0, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UE_PositioningCapabilityExt_v3g0 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UE_RadioAccessCapability_v3g0ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UE_RadioAccessCapability_v3g0ext, UE_RadioAccessCapability_v3g0ext_sequence);

  return offset;
}


static const per_sequence_t RRCConnectionSetupComplete_v3g0ext_IEs_sequence[] = {
  { &hf_rrc_ue_RadioAccessCapability_v3g0ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_RadioAccessCapability_v3g0ext },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_RRCConnectionSetupComplete_v3g0ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_RRCConnectionSetupComplete_v3g0ext_IEs, RRCConnectionSetupComplete_v3g0ext_IEs_sequence);

  return offset;
}


static const value_string rrc_MaxROHC_ContextSessions_r4_vals[] = {
  {   0, "s2" },
  {   1, "s4" },
  {   2, "s8" },
  {   3, "s12" },
  {   4, "s16" },
  {   5, "s24" },
  {   6, "s32" },
  {   7, "s48" },
  {   8, "s64" },
  {   9, "s128" },
  {  10, "s256" },
  {  11, "s512" },
  {  12, "s1024" },
  {  13, "s16384" },
  { 0, NULL }
};


static int
dissect_rrc_MaxROHC_ContextSessions_r4(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     14, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t T_supported_sequence[] = {
  { &hf_rrc_maxROHC_ContextSessions, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MaxROHC_ContextSessions_r4 },
  { &hf_rrc_reverseCompressionDepth, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_INTEGER_0_65535 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_supported(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_supported, T_supported_sequence);

  return offset;
}


static const value_string rrc_T_supportForRfc3095_vals[] = {
  {   0, "notSupported" },
  {   1, "supported" },
  { 0, NULL }
};

static const per_choice_t T_supportForRfc3095_choice[] = {
  {   0, &hf_rrc_notSupported    , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   1, &hf_rrc_supported_02    , ASN1_NO_EXTENSIONS     , dissect_rrc_T_supported },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_supportForRfc3095(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_supportForRfc3095, T_supportForRfc3095_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t PDCP_Capability_r4_ext_sequence[] = {
  { &hf_rrc_supportForRfc3095, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_supportForRfc3095 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_PDCP_Capability_r4_ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_PDCP_Capability_r4_ext, PDCP_Capability_r4_ext_sequence);

  return offset;
}


static const per_sequence_t T_tddRF_Capability_01_sequence[] = {
  { &hf_rrc_ue_PowerClass   , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UE_PowerClass },
  { &hf_rrc_radioFrequencyBandTDDList, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RadioFrequencyBandTDDList },
  { &hf_rrc_chipRateCapability, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_ChipRateCapability },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_tddRF_Capability_01(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_tddRF_Capability_01, T_tddRF_Capability_01_sequence);

  return offset;
}


static const per_sequence_t RF_Capability_r4_ext_sequence[] = {
  { &hf_rrc_tddRF_Capability_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_tddRF_Capability_01 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_RF_Capability_r4_ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_RF_Capability_r4_ext, RF_Capability_r4_ext_sequence);

  return offset;
}



static int
dissect_rrc_MaxTS_PerSubFrame_r4(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            1U, 6U, NULL, FALSE);

  return offset;
}



static int
dissect_rrc_MaxPhysChPerSubFrame_r4(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            1U, 96U, NULL, FALSE);

  return offset;
}


static const per_sequence_t DL_PhysChCapabilityTDD_LCR_r4_sequence[] = {
  { &hf_rrc_maxTS_PerSubFrame, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MaxTS_PerSubFrame_r4 },
  { &hf_rrc_maxPhysChPerFrame_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MaxPhysChPerSubFrame_r4 },
  { &hf_rrc_minimumSF       , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MinimumSF_DL },
  { &hf_rrc_supportOfPDSCH  , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { &hf_rrc_maxPhysChPerTS  , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MaxPhysChPerTS },
  { &hf_rrc_supportOf8PSK   , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_DL_PhysChCapabilityTDD_LCR_r4(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_DL_PhysChCapabilityTDD_LCR_r4, DL_PhysChCapabilityTDD_LCR_r4_sequence);

  return offset;
}


static const per_sequence_t UL_PhysChCapabilityTDD_LCR_r4_sequence[] = {
  { &hf_rrc_maxTS_PerSubFrame, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MaxTS_PerSubFrame_r4 },
  { &hf_rrc_maxPhysChPerTimeslot, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MaxPhysChPerTimeslot },
  { &hf_rrc_minimumSF_02    , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MinimumSF_UL },
  { &hf_rrc_supportOfPUSCH  , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { &hf_rrc_supportOf8PSK   , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UL_PhysChCapabilityTDD_LCR_r4(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UL_PhysChCapabilityTDD_LCR_r4, UL_PhysChCapabilityTDD_LCR_r4_sequence);

  return offset;
}


static const per_sequence_t T_tdd128_PhysChCapability_sequence[] = {
  { &hf_rrc_downlinkPhysChCapability_12, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DL_PhysChCapabilityTDD_LCR_r4 },
  { &hf_rrc_uplinkPhysChCapability_09, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UL_PhysChCapabilityTDD_LCR_r4 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_tdd128_PhysChCapability(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_tdd128_PhysChCapability, T_tdd128_PhysChCapability_sequence);

  return offset;
}


static const per_sequence_t PhysicalChannelCapability_LCR_r4_sequence[] = {
  { &hf_rrc_tdd128_PhysChCapability, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_tdd128_PhysChCapability },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_PhysicalChannelCapability_LCR_r4(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_PhysicalChannelCapability_LCR_r4, PhysicalChannelCapability_LCR_r4_sequence);

  return offset;
}


static const per_sequence_t CompressedModeMeasCapability_LCR_r4_sequence[] = {
  { &hf_rrc_tdd128_Measurements, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_BOOLEAN },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_CompressedModeMeasCapability_LCR_r4(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_CompressedModeMeasCapability_LCR_r4, CompressedModeMeasCapability_LCR_r4_sequence);

  return offset;
}


static const per_sequence_t MeasurementCapability_r4_ext_sequence[] = {
  { &hf_rrc_downlinkCompressedMode_LCR, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CompressedModeMeasCapability_LCR_r4 },
  { &hf_rrc_uplinkCompressedMode_LCR, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CompressedModeMeasCapability_LCR_r4 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_MeasurementCapability_r4_ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_MeasurementCapability_r4_ext, MeasurementCapability_r4_ext_sequence);

  return offset;
}


static const per_sequence_t T_tdd_CapabilityExt_sequence[] = {
  { &hf_rrc_rf_Capability_03, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RF_Capability_r4_ext },
  { &hf_rrc_physicalChannelCapability_LCR, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PhysicalChannelCapability_LCR_r4 },
  { &hf_rrc_measurementCapability_r4_ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MeasurementCapability_r4_ext },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_tdd_CapabilityExt(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_tdd_CapabilityExt, T_tdd_CapabilityExt_sequence);

  return offset;
}


static const value_string rrc_AccessStratumReleaseIndicator_vals[] = {
  {   0, "rel-4" },
  {   1, "rel-5" },
  {   2, "rel-6" },
  {   3, "rel-7" },
  {   4, "rel-8" },
  {   5, "rel-9" },
  {   6, "rel-10" },
  {   7, "rel-11" },
  {   8, "spare8" },
  {   9, "spare7" },
  {  10, "spare6" },
  {  11, "spare5" },
  {  12, "spare4" },
  {  13, "spare3" },
  {  14, "spare2" },
  {  15, "spare1" },
  { 0, NULL }
};


static int
dissect_rrc_AccessStratumReleaseIndicator(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     16, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t UE_RadioAccessCapability_v4b0ext_sequence[] = {
  { &hf_rrc_pdcp_Capability_r4_ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PDCP_Capability_r4_ext },
  { &hf_rrc_tdd_CapabilityExt, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_tdd_CapabilityExt },
  { &hf_rrc_accessStratumReleaseIndicator, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_AccessStratumReleaseIndicator },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UE_RadioAccessCapability_v4b0ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UE_RadioAccessCapability_v4b0ext, UE_RadioAccessCapability_v4b0ext_sequence);

  return offset;
}


static const per_sequence_t RRCConnectionSetupComplete_v4b0ext_IEs_sequence[] = {
  { &hf_rrc_ue_RadioAccessCapability_v4b0ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_RadioAccessCapability_v4b0ext },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_RRCConnectionSetupComplete_v4b0ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_RRCConnectionSetupComplete_v4b0ext_IEs, RRCConnectionSetupComplete_v4b0ext_IEs_sequence);

  return offset;
}


static const value_string rrc_DL_CapabilityWithSimultaneousHS_DSCHConfig_vals[] = {
  {   0, "kbps32" },
  {   1, "kbps64" },
  {   2, "kbps128" },
  {   3, "kbps384" },
  { 0, NULL }
};


static int
dissect_rrc_DL_CapabilityWithSimultaneousHS_DSCHConfig(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     4, NULL, FALSE, 0, NULL);

  return offset;
}


static const value_string rrc_MaxHcContextSpace_r5_ext_vals[] = {
  {   0, "by16384" },
  {   1, "by32768" },
  {   2, "by65536" },
  {   3, "by131072" },
  { 0, NULL }
};


static int
dissect_rrc_MaxHcContextSpace_r5_ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     4, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t PDCP_Capability_r5_ext_sequence[] = {
  { &hf_rrc_supportForRfc3095ContextRelocation, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { &hf_rrc_maxHcContextSpace, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MaxHcContextSpace_r5_ext },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_PDCP_Capability_r5_ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_PDCP_Capability_r5_ext, PDCP_Capability_r5_ext_sequence);

  return offset;
}


static const value_string rrc_TotalRLC_AM_BufferSize_r5_ext_vals[] = {
  {   0, "kb200" },
  {   1, "kb300" },
  {   2, "kb400" },
  {   3, "kb750" },
  { 0, NULL }
};


static int
dissect_rrc_TotalRLC_AM_BufferSize_r5_ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     4, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t RLC_Capability_r5_ext_sequence[] = {
  { &hf_rrc_totalRLC_AM_BufferSize_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_TotalRLC_AM_BufferSize_r5_ext },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_RLC_Capability_r5_ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_RLC_Capability_r5_ext, RLC_Capability_r5_ext_sequence);

  return offset;
}



static int
dissect_rrc_HSDSCH_physical_layer_category(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            1U, 64U, NULL, FALSE);

  return offset;
}


static const per_sequence_t T_supported_01_sequence[] = {
  { &hf_rrc_hsdsch_physical_layer_category, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_HSDSCH_physical_layer_category },
  { &hf_rrc_dummy_28        , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { &hf_rrc_dummy2_06       , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_supported_01(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_supported_01, T_supported_01_sequence);

  return offset;
}


static const value_string rrc_T_fdd_hspdsch_vals[] = {
  {   0, "supported" },
  {   1, "unsupported" },
  { 0, NULL }
};

static const per_choice_t T_fdd_hspdsch_choice[] = {
  {   0, &hf_rrc_supported_03    , ASN1_NO_EXTENSIONS     , dissect_rrc_T_supported_01 },
  {   1, &hf_rrc_unsupported     , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_fdd_hspdsch(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_fdd_hspdsch, T_fdd_hspdsch_choice,
                                 NULL);

  return offset;
}


static const value_string rrc_T_tdd384_hspdsch_vals[] = {
  {   0, "supported" },
  {   1, "unsupported" },
  { 0, NULL }
};

static const per_choice_t T_tdd384_hspdsch_choice[] = {
  {   0, &hf_rrc_supported       , ASN1_NO_EXTENSIONS     , dissect_rrc_HSDSCH_physical_layer_category },
  {   1, &hf_rrc_unsupported     , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_tdd384_hspdsch(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_tdd384_hspdsch, T_tdd384_hspdsch_choice,
                                 NULL);

  return offset;
}


static const value_string rrc_T_tdd128_hspdsch_vals[] = {
  {   0, "supported" },
  {   1, "unsupported" },
  { 0, NULL }
};

static const per_choice_t T_tdd128_hspdsch_choice[] = {
  {   0, &hf_rrc_supported       , ASN1_NO_EXTENSIONS     , dissect_rrc_HSDSCH_physical_layer_category },
  {   1, &hf_rrc_unsupported     , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_tdd128_hspdsch(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_tdd128_hspdsch, T_tdd128_hspdsch_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t PhysicalChannelCapability_hspdsch_r5_sequence[] = {
  { &hf_rrc_fdd_hspdsch     , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_fdd_hspdsch },
  { &hf_rrc_tdd384_hspdsch  , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_tdd384_hspdsch },
  { &hf_rrc_tdd128_hspdsch  , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_tdd128_hspdsch },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_PhysicalChannelCapability_hspdsch_r5(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_PhysicalChannelCapability_hspdsch_r5, PhysicalChannelCapability_hspdsch_r5_sequence);

  return offset;
}


static const per_sequence_t MultiModeRAT_Capability_v590ext_sequence[] = {
  { &hf_rrc_supportOfUTRAN_ToGERAN_NACC, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_MultiModeRAT_Capability_v590ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_MultiModeRAT_Capability_v590ext, MultiModeRAT_Capability_v590ext_sequence);

  return offset;
}


static const per_sequence_t UE_RadioAccessCapability_v590ext_sequence[] = {
  { &hf_rrc_dl_CapabilityWithSimultaneousHS_DSCHConfig, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_CapabilityWithSimultaneousHS_DSCHConfig },
  { &hf_rrc_pdcp_Capability_r5_ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PDCP_Capability_r5_ext },
  { &hf_rrc_rlc_Capability_r5_ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RLC_Capability_r5_ext },
  { &hf_rrc_physicalChannelCapability_07, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PhysicalChannelCapability_hspdsch_r5 },
  { &hf_rrc_multiModeRAT_Capability_v590ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MultiModeRAT_Capability_v590ext },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UE_RadioAccessCapability_v590ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UE_RadioAccessCapability_v590ext, UE_RadioAccessCapability_v590ext_sequence);

  return offset;
}



static int
dissect_rrc_GERANIu_RadioAccessCapability(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_bit_string(tvb, offset, actx, tree, hf_index,
                                     1, 170, FALSE, NULL);

  return offset;
}


static const per_sequence_t InterRAT_UE_RadioAccessCapability_v590ext_sequence[] = {
  { &hf_rrc_geranIu_RadioAccessCapability, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_GERANIu_RadioAccessCapability },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_InterRAT_UE_RadioAccessCapability_v590ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_InterRAT_UE_RadioAccessCapability_v590ext, InterRAT_UE_RadioAccessCapability_v590ext_sequence);

  return offset;
}


static const per_sequence_t RRCConnectionSetupComplete_v590ext_IEs_sequence[] = {
  { &hf_rrc_ue_RadioAccessCapability_v590ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_RadioAccessCapability_v590ext },
  { &hf_rrc_ue_RATSpecificCapability_v590ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_InterRAT_UE_RadioAccessCapability_v590ext },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_RRCConnectionSetupComplete_v590ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_RRCConnectionSetupComplete_v590ext_IEs, RRCConnectionSetupComplete_v590ext_IEs_sequence);

  return offset;
}


static const value_string rrc_T_losslessDLRLC_PDUSizeChange_vals[] = {
  {   0, "true" },
  { 0, NULL }
};


static int
dissect_rrc_T_losslessDLRLC_PDUSizeChange(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t PDCP_Capability_r5_ext2_sequence[] = {
  { &hf_rrc_losslessDLRLC_PDUSizeChange, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_losslessDLRLC_PDUSizeChange },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_PDCP_Capability_r5_ext2(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_PDCP_Capability_r5_ext2, PDCP_Capability_r5_ext2_sequence);

  return offset;
}


static const per_sequence_t UE_RadioAccessCapability_v5c0ext_sequence[] = {
  { &hf_rrc_pdcp_Capability_r5_ext2, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PDCP_Capability_r5_ext2 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UE_RadioAccessCapability_v5c0ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UE_RadioAccessCapability_v5c0ext, UE_RadioAccessCapability_v5c0ext_sequence);

  return offset;
}


static const per_sequence_t RRCConnectionSetupComplete_v5c0ext_IEs_sequence[] = {
  { &hf_rrc_ue_RadioAccessCapability_v5c0ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_RadioAccessCapability_v5c0ext },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_RRCConnectionSetupComplete_v5c0ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_RRCConnectionSetupComplete_v5c0ext_IEs, RRCConnectionSetupComplete_v5c0ext_IEs_sequence);

  return offset;
}



static int
dissect_rrc_T_ueCapabilityContainer(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_bit_string_containing_pdu_new(tvb, offset, actx, tree, hf_index,
                                                              NO_BOUND, NO_BOUND, FALSE, dissect_UE_CapabilityContainer_IEs_PDU);

  return offset;
}


static const per_sequence_t RRCConnectionSetupComplete_v690ext_IEs_sequence[] = {
  { &hf_rrc_ueCapabilityContainer, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_ueCapabilityContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_RRCConnectionSetupComplete_v690ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_RRCConnectionSetupComplete_v690ext_IEs, RRCConnectionSetupComplete_v690ext_IEs_sequence);

  return offset;
}


static const value_string rrc_T_deferredMeasurementControlReading_04_vals[] = {
  {   0, "true" },
  { 0, NULL }
};


static int
dissect_rrc_T_deferredMeasurementControlReading_04(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t RRCConnectionSetupComplete_v770ext_IEs_sequence[] = {
  { &hf_rrc_deferredMeasurementControlReading_04, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_deferredMeasurementControlReading_04 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_RRCConnectionSetupComplete_v770ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_RRCConnectionSetupComplete_v770ext_IEs, RRCConnectionSetupComplete_v770ext_IEs_sequence);

  return offset;
}


static const value_string rrc_T_loggedMeasAvailable_03_vals[] = {
  {   0, "true" },
  { 0, NULL }
};


static int
dissect_rrc_T_loggedMeasAvailable_03(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}


static const value_string rrc_T_loggedANRResultsAvailable_02_vals[] = {
  {   0, "true" },
  { 0, NULL }
};


static int
dissect_rrc_T_loggedANRResultsAvailable_02(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t RRCConnectionSetupComplete_va40ext_IEs_sequence[] = {
  { &hf_rrc_loggedMeasAvailable_03, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_loggedMeasAvailable_03 },
  { &hf_rrc_loggedANRResultsAvailable_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_loggedANRResultsAvailable_02 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_RRCConnectionSetupComplete_va40ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_RRCConnectionSetupComplete_va40ext_IEs, RRCConnectionSetupComplete_va40ext_IEs_sequence);

  return offset;
}


static const value_string rrc_T_loggedConnectionEstablishmentFailureInfoAvailable_01_vals[] = {
  {   0, "true" },
  { 0, NULL }
};


static int
dissect_rrc_T_loggedConnectionEstablishmentFailureInfoAvailable_01(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t RRCConnectionSetupComplete_vbxyext_IEs_sequence[] = {
  { &hf_rrc_loggedConnectionEstablishmentFailureInfoAvailable_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_loggedConnectionEstablishmentFailureInfoAvailable_01 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_RRCConnectionSetupComplete_vbxyext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_RRCConnectionSetupComplete_vbxyext_IEs, RRCConnectionSetupComplete_vbxyext_IEs_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions_135_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_nonCriticalExtensions_135(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_nonCriticalExtensions_135, T_nonCriticalExtensions_135_sequence);

  return offset;
}


static const per_sequence_t T_vbxyNonCriticalExtensions_13_sequence[] = {
  { &hf_rrc_rrcConnectionSetupComplete_vbxyext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RRCConnectionSetupComplete_vbxyext_IEs },
  { &hf_rrc_nonCriticalExtensions_135, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_nonCriticalExtensions_135 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_vbxyNonCriticalExtensions_13(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_vbxyNonCriticalExtensions_13, T_vbxyNonCriticalExtensions_13_sequence);

  return offset;
}


static const per_sequence_t T_va40NonCriticalExtensions_11_sequence[] = {
  { &hf_rrc_rrcConectionSetupComplete_va40ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RRCConnectionSetupComplete_va40ext_IEs },
  { &hf_rrc_vbxyNonCriticalExtensions_13, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_vbxyNonCriticalExtensions_13 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_va40NonCriticalExtensions_11(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_va40NonCriticalExtensions_11, T_va40NonCriticalExtensions_11_sequence);

  return offset;
}


static const per_sequence_t T_v770NonCriticalExtensions_24_sequence[] = {
  { &hf_rrc_rrcConectionSetupComplete_v770ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RRCConnectionSetupComplete_v770ext_IEs },
  { &hf_rrc_va40NonCriticalExtensions_12, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_va40NonCriticalExtensions_11 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v770NonCriticalExtensions_24(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v770NonCriticalExtensions_24, T_v770NonCriticalExtensions_24_sequence);

  return offset;
}


static const per_sequence_t T_v690NonCriticalExtensions_35_sequence[] = {
  { &hf_rrc_rrcConnectionSetupComplete_v690ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RRCConnectionSetupComplete_v690ext_IEs },
  { &hf_rrc_v770NonCriticalExtensions_24, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v770NonCriticalExtensions_24 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v690NonCriticalExtensions_35(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v690NonCriticalExtensions_35, T_v690NonCriticalExtensions_35_sequence);

  return offset;
}


static const per_sequence_t T_v5c0NonCriticalExtensions_sequence[] = {
  { &hf_rrc_rrcConnectionSetupComplete_v5c0ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RRCConnectionSetupComplete_v5c0ext_IEs },
  { &hf_rrc_v690NonCriticalExtensions_35, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v690NonCriticalExtensions_35 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v5c0NonCriticalExtensions(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v5c0NonCriticalExtensions, T_v5c0NonCriticalExtensions_sequence);

  return offset;
}


static const per_sequence_t T_v590NonCriticalExtensions_23_sequence[] = {
  { &hf_rrc_rrcConnectionSetupComplete_v590ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RRCConnectionSetupComplete_v590ext_IEs },
  { &hf_rrc_v5c0NonCriticalExtensions, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v5c0NonCriticalExtensions },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v590NonCriticalExtensions_23(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v590NonCriticalExtensions_23, T_v590NonCriticalExtensions_23_sequence);

  return offset;
}


static const per_sequence_t T_v4b0NonCriticalExtensions_12_sequence[] = {
  { &hf_rrc_rrcConnectionSetupComplete_v4b0ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RRCConnectionSetupComplete_v4b0ext_IEs },
  { &hf_rrc_v590NonCriticalExtensions_23, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v590NonCriticalExtensions_23 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v4b0NonCriticalExtensions_12(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v4b0NonCriticalExtensions_12, T_v4b0NonCriticalExtensions_12_sequence);

  return offset;
}


static const per_sequence_t T_v3g0NonCriticalExtensions_01_sequence[] = {
  { &hf_rrc_rrcConnectionSetupComplete_v3g0ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RRCConnectionSetupComplete_v3g0ext_IEs },
  { &hf_rrc_v4b0NonCriticalExtensions_12, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v4b0NonCriticalExtensions_12 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v3g0NonCriticalExtensions_01(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v3g0NonCriticalExtensions_01, T_v3g0NonCriticalExtensions_01_sequence);

  return offset;
}


static const per_sequence_t T_laterNonCriticalExtensions_42_sequence[] = {
  { &hf_rrc_rrcConnectionSetupComplete_r3_add_ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_rrcConnectionSetupComplete_r3_add_ext },
  { &hf_rrc_v3g0NonCriticalExtensions_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v3g0NonCriticalExtensions_01 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_laterNonCriticalExtensions_42(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_laterNonCriticalExtensions_42, T_laterNonCriticalExtensions_42_sequence);

  return offset;
}


static const per_sequence_t T_v3a0NonCriticalExtensions_08_sequence[] = {
  { &hf_rrc_rrcConnectionSetupComplete_v3a0ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RRCConnectionSetupComplete_v3a0ext_IEs },
  { &hf_rrc_laterNonCriticalExtensions_42, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_laterNonCriticalExtensions_42 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v3a0NonCriticalExtensions_08(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v3a0NonCriticalExtensions_08, T_v3a0NonCriticalExtensions_08_sequence);

  return offset;
}


static const per_sequence_t T_v380NonCriticalExtensions_sequence[] = {
  { &hf_rrc_rrcConnectionSetupComplete_v380ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RRCConnectionSetupComplete_v380ext_IEs },
  { &hf_rrc_v3a0NonCriticalExtensions_08, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v3a0NonCriticalExtensions_08 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v380NonCriticalExtensions(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v380NonCriticalExtensions, T_v380NonCriticalExtensions_sequence);

  return offset;
}


static const per_sequence_t T_v370NonCriticalExtensions_sequence[] = {
  { &hf_rrc_rrcConnectionSetupComplete_v370ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RRCConnectionSetupComplete_v370ext },
  { &hf_rrc_v380NonCriticalExtensions, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v380NonCriticalExtensions },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v370NonCriticalExtensions(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v370NonCriticalExtensions, T_v370NonCriticalExtensions_sequence);

  return offset;
}


static const per_sequence_t RRCConnectionSetupComplete_sequence[] = {
  { &hf_rrc_rrc_TransactionIdentifier, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RRC_TransactionIdentifier },
  { &hf_rrc_startList       , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_STARTList },
  { &hf_rrc_ue_RadioAccessCapability_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_RadioAccessCapability },
  { &hf_rrc_ue_RATSpecificCapability, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_InterRAT_UE_RadioAccessCapabilityList },
  { &hf_rrc_v370NonCriticalExtensions, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v370NonCriticalExtensions },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_RRCConnectionSetupComplete(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
#line 385 "../../asn1/rrc/rrc.cnf"
  col_append_str(actx->pinfo->cinfo, COL_INFO, "RRCConnectionSetupComplete");
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_RRCConnectionSetupComplete, RRCConnectionSetupComplete_sequence);




  return offset;
}


static const value_string rrc_ReceivedMessageType_vals[] = {
  {   0, "activeSetUpdate" },
  {   1, "cellChangeOrderFromUTRAN" },
  {   2, "cellUpdateConfirm" },
  {   3, "counterCheck" },
  {   4, "downlinkDirectTransfer" },
  {   5, "interRATHandoverCommand" },
  {   6, "measurementControl" },
  {   7, "pagingType2" },
  {   8, "physicalChannelReconfiguration" },
  {   9, "physicalSharedChannelAllocation" },
  {  10, "radioBearerReconfiguration" },
  {  11, "radioBearerRelease" },
  {  12, "radioBearerSetup" },
  {  13, "rrcConnectionRelease" },
  {  14, "rrcConnectionReject" },
  {  15, "rrcConnectionSetup" },
  {  16, "securityModeCommand" },
  {  17, "signallingConnectionRelease" },
  {  18, "transportChannelReconfiguration" },
  {  19, "transportFormatCombinationControl" },
  {  20, "ueCapabilityEnquiry" },
  {  21, "ueCapabilityInformationConfirm" },
  {  22, "uplinkPhysicalChannelControl" },
  {  23, "uraUpdateConfirm" },
  {  24, "utranMobilityInformation" },
  {  25, "assistanceDataDelivery" },
  {  26, "spare6" },
  {  27, "spare5" },
  {  28, "spare4" },
  {  29, "spare3" },
  {  30, "spare2" },
  {  31, "spare1" },
  { 0, NULL }
};

static value_string_ext rrc_ReceivedMessageType_vals_ext = VALUE_STRING_EXT_INIT(rrc_ReceivedMessageType_vals);


static int
dissect_rrc_ReceivedMessageType(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     32, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t IdentificationOfReceivedMessage_sequence[] = {
  { &hf_rrc_rrc_TransactionIdentifier, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RRC_TransactionIdentifier },
  { &hf_rrc_receivedMessageType, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_ReceivedMessageType },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_IdentificationOfReceivedMessage(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_IdentificationOfReceivedMessage, IdentificationOfReceivedMessage_sequence);

  return offset;
}


static const value_string rrc_T_type1_vals[] = {
  {   0, "asn1-ViolationOrEncodingError" },
  {   1, "messageTypeNonexistent" },
  {   2, "messageNotCompatibleWithReceiverState" },
  {   3, "ie-ValueNotComprehended" },
  {   4, "conditionalInformationElementError" },
  {   5, "messageExtensionNotComprehended" },
  {   6, "spare1" },
  {   7, "spare2" },
  { 0, NULL }
};

static const per_choice_t T_type1_choice[] = {
  {   0, &hf_rrc_asn1_ViolationOrEncodingError, ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   1, &hf_rrc_messageTypeNonexistent, ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   2, &hf_rrc_messageNotCompatibleWithReceiverState, ASN1_NO_EXTENSIONS     , dissect_rrc_IdentificationOfReceivedMessage },
  {   3, &hf_rrc_ie_ValueNotComprehended, ASN1_NO_EXTENSIONS     , dissect_rrc_IdentificationOfReceivedMessage },
  {   4, &hf_rrc_conditionalInformationElementError, ASN1_NO_EXTENSIONS     , dissect_rrc_IdentificationOfReceivedMessage },
  {   5, &hf_rrc_messageExtensionNotComprehended, ASN1_NO_EXTENSIONS     , dissect_rrc_IdentificationOfReceivedMessage },
  {   6, &hf_rrc_spare1          , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   7, &hf_rrc_spare2          , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_type1(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_type1, T_type1_choice,
                                 NULL);

  return offset;
}


static const value_string rrc_T_diagnosticsType_vals[] = {
  {   0, "type1" },
  {   1, "spare" },
  { 0, NULL }
};

static const per_choice_t T_diagnosticsType_choice[] = {
  {   0, &hf_rrc_type1           , ASN1_NO_EXTENSIONS     , dissect_rrc_T_type1 },
  {   1, &hf_rrc_spare           , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_diagnosticsType(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_diagnosticsType, T_diagnosticsType_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t ProtocolErrorMoreInformation_sequence[] = {
  { &hf_rrc_diagnosticsType , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_diagnosticsType },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_ProtocolErrorMoreInformation(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_ProtocolErrorMoreInformation, ProtocolErrorMoreInformation_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions_138_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_nonCriticalExtensions_138(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_nonCriticalExtensions_138, T_nonCriticalExtensions_138_sequence);

  return offset;
}


static const per_sequence_t T_laterNonCriticalExtensions_44_sequence[] = {
  { &hf_rrc_rrcStatus_r3_add_ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_BIT_STRING },
  { &hf_rrc_nonCriticalExtensions_138, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_nonCriticalExtensions_138 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_laterNonCriticalExtensions_44(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_laterNonCriticalExtensions_44, T_laterNonCriticalExtensions_44_sequence);

  return offset;
}


static const per_sequence_t RRCStatus_sequence[] = {
  { &hf_rrc_protocolErrorInformation, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_ProtocolErrorMoreInformation },
  { &hf_rrc_laterNonCriticalExtensions_44, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_laterNonCriticalExtensions_44 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_RRCStatus(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
#line 389 "../../asn1/rrc/rrc.cnf"
  col_append_str(actx->pinfo->cinfo, COL_INFO, "RRCStatus");
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_RRCStatus, RRCStatus_sequence);




  return offset;
}


static const per_sequence_t T_nonCriticalExtensions_141_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_nonCriticalExtensions_141(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_nonCriticalExtensions_141, T_nonCriticalExtensions_141_sequence);

  return offset;
}


static const per_sequence_t T_laterNonCriticalExtensions_46_sequence[] = {
  { &hf_rrc_securityModeComplete_r3_add_ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_BIT_STRING },
  { &hf_rrc_nonCriticalExtensions_141, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_nonCriticalExtensions_141 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_laterNonCriticalExtensions_46(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_laterNonCriticalExtensions_46, T_laterNonCriticalExtensions_46_sequence);

  return offset;
}


static const per_sequence_t SecurityModeComplete_sequence[] = {
  { &hf_rrc_rrc_TransactionIdentifier, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RRC_TransactionIdentifier },
  { &hf_rrc_ul_IntegProtActivationInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_IntegrityProtActivationInfo },
  { &hf_rrc_rb_UL_CiphActivationTimeInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RB_ActivationTimeInfoList },
  { &hf_rrc_laterNonCriticalExtensions_46, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_laterNonCriticalExtensions_46 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_SecurityModeComplete(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
#line 397 "../../asn1/rrc/rrc.cnf"
rrc_ciphering_info * c_inf ;
fp_info       *fpinf ;

  col_append_str(actx->pinfo->cinfo, COL_INFO, "SecurityModeComplete");

  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_SecurityModeComplete, SecurityModeComplete_sequence);


  fpinf = p_get_proto_data(actx->pinfo->fd, proto_fp);
  if(fpinf && ((c_inf = g_tree_lookup(rrc_ciph_inf, GINT_TO_POINTER(fpinf->com_context_id))) != NULL) ){
    c_inf->setup_frame = actx->pinfo->fd->num;
  }




  return offset;
}


static const per_sequence_t T_nonCriticalExtensions_142_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_nonCriticalExtensions_142(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_nonCriticalExtensions_142, T_nonCriticalExtensions_142_sequence);

  return offset;
}


static const per_sequence_t T_laterNonCriticalExtensions_47_sequence[] = {
  { &hf_rrc_securityModeFailure_r3_add_ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_BIT_STRING },
  { &hf_rrc_nonCriticalExtensions_142, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_nonCriticalExtensions_142 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_laterNonCriticalExtensions_47(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_laterNonCriticalExtensions_47, T_laterNonCriticalExtensions_47_sequence);

  return offset;
}


static const per_sequence_t SecurityModeFailure_sequence[] = {
  { &hf_rrc_rrc_TransactionIdentifier, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RRC_TransactionIdentifier },
  { &hf_rrc_failureCause    , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_FailureCauseWithProtErr },
  { &hf_rrc_laterNonCriticalExtensions_47, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_laterNonCriticalExtensions_47 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_SecurityModeFailure(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
#line 411 "../../asn1/rrc/rrc.cnf"
  col_append_str(actx->pinfo->cinfo, COL_INFO, "SecurityModeFailure");
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_SecurityModeFailure, SecurityModeFailure_sequence);




  return offset;
}


static const value_string rrc_T_signallingConnectionReleaseIndicationCause_vals[] = {
  {   0, "uERequestedPSDataSessionEnd" },
  {   1, "anyOtherCause" },
  { 0, NULL }
};


static int
dissect_rrc_T_signallingConnectionReleaseIndicationCause(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     2, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t SignallingConnectionReleaseIndication_v860ext_sequence[] = {
  { &hf_rrc_signallingConnectionReleaseIndicationCause, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_signallingConnectionReleaseIndicationCause },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_SignallingConnectionReleaseIndication_v860ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_SignallingConnectionReleaseIndication_v860ext, SignallingConnectionReleaseIndication_v860ext_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions_144_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_nonCriticalExtensions_144(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_nonCriticalExtensions_144, T_nonCriticalExtensions_144_sequence);

  return offset;
}


static const per_sequence_t T_v860nonCriticalExtentions_sequence[] = {
  { &hf_rrc_signallingConnectionReleaseIndication_v860ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SignallingConnectionReleaseIndication_v860ext },
  { &hf_rrc_nonCriticalExtensions_144, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_nonCriticalExtensions_144 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v860nonCriticalExtentions(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v860nonCriticalExtentions, T_v860nonCriticalExtentions_sequence);

  return offset;
}


static const per_sequence_t T_laterNonCriticalExtensions_49_sequence[] = {
  { &hf_rrc_signallingConnectionReleaseIndication_r3_add_ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_BIT_STRING },
  { &hf_rrc_v860nonCriticalExtentions, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v860nonCriticalExtentions },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_laterNonCriticalExtensions_49(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_laterNonCriticalExtensions_49, T_laterNonCriticalExtensions_49_sequence);

  return offset;
}


static const per_sequence_t SignallingConnectionReleaseIndication_sequence[] = {
  { &hf_rrc_cn_DomainIdentity, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CN_DomainIdentity },
  { &hf_rrc_laterNonCriticalExtensions_49, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_laterNonCriticalExtensions_49 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_SignallingConnectionReleaseIndication(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
#line 419 "../../asn1/rrc/rrc.cnf"
  col_append_str(actx->pinfo->cinfo, COL_INFO, "SignallingConnectionReleaseIndication");
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_SignallingConnectionReleaseIndication, SignallingConnectionReleaseIndication_sequence);




  return offset;
}


static const value_string rrc_T_deferredMeasurementControlReading_05_vals[] = {
  {   0, "true" },
  { 0, NULL }
};


static int
dissect_rrc_T_deferredMeasurementControlReading_05(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t TransportChannelReconfigurationComplete_v770ext_IEs_sequence[] = {
  { &hf_rrc_ext_UL_TimingAdvance, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_EXT_UL_TimingAdvance },
  { &hf_rrc_deferredMeasurementControlReading_05, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_deferredMeasurementControlReading_05 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_TransportChannelReconfigurationComplete_v770ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_TransportChannelReconfigurationComplete_v770ext_IEs, TransportChannelReconfigurationComplete_v770ext_IEs_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions_155_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_nonCriticalExtensions_155(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_nonCriticalExtensions_155, T_nonCriticalExtensions_155_sequence);

  return offset;
}


static const per_sequence_t T_v770NonCriticalExtensions_29_sequence[] = {
  { &hf_rrc_transportChannelReconfigurationComplete_v770ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TransportChannelReconfigurationComplete_v770ext_IEs },
  { &hf_rrc_nonCriticalExtensions_155, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_nonCriticalExtensions_155 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v770NonCriticalExtensions_29(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v770NonCriticalExtensions_29, T_v770NonCriticalExtensions_29_sequence);

  return offset;
}


static const per_sequence_t T_laterNonCriticalExtensions_52_sequence[] = {
  { &hf_rrc_transportChannelReconfigurationComplete_r3_add_ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_BIT_STRING },
  { &hf_rrc_v770NonCriticalExtensions_29, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v770NonCriticalExtensions_29 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_laterNonCriticalExtensions_52(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_laterNonCriticalExtensions_52, T_laterNonCriticalExtensions_52_sequence);

  return offset;
}


static const per_sequence_t TransportChannelReconfigurationComplete_sequence[] = {
  { &hf_rrc_rrc_TransactionIdentifier, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RRC_TransactionIdentifier },
  { &hf_rrc_ul_IntegProtActivationInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_IntegrityProtActivationInfo },
  { &hf_rrc_ul_TimingAdvance, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_TimingAdvance },
  { &hf_rrc_count_C_ActivationTime, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_ActivationTime },
  { &hf_rrc_dummy_13        , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RB_ActivationTimeInfoList },
  { &hf_rrc_ul_CounterSynchronisationInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_CounterSynchronisationInfo },
  { &hf_rrc_laterNonCriticalExtensions_52, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_laterNonCriticalExtensions_52 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_TransportChannelReconfigurationComplete(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
#line 439 "../../asn1/rrc/rrc.cnf"
  col_append_str(actx->pinfo->cinfo, COL_INFO, "TransportChannelReconfigurationComplete");
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_TransportChannelReconfigurationComplete, TransportChannelReconfigurationComplete_sequence);




  return offset;
}


static const per_sequence_t T_nonCriticalExtensions_156_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_nonCriticalExtensions_156(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_nonCriticalExtensions_156, T_nonCriticalExtensions_156_sequence);

  return offset;
}


static const per_sequence_t T_laterNonCriticalExtensions_53_sequence[] = {
  { &hf_rrc_transportChannelReconfigurationFailure_r3_add_ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_BIT_STRING },
  { &hf_rrc_nonCriticalExtensions_156, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_nonCriticalExtensions_156 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_laterNonCriticalExtensions_53(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_laterNonCriticalExtensions_53, T_laterNonCriticalExtensions_53_sequence);

  return offset;
}


static const per_sequence_t TransportChannelReconfigurationFailure_sequence[] = {
  { &hf_rrc_rrc_TransactionIdentifier, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RRC_TransactionIdentifier },
  { &hf_rrc_failureCause    , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_FailureCauseWithProtErr },
  { &hf_rrc_laterNonCriticalExtensions_53, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_laterNonCriticalExtensions_53 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_TransportChannelReconfigurationFailure(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
#line 443 "../../asn1/rrc/rrc.cnf"
  col_append_str(actx->pinfo->cinfo, COL_INFO, "TransportChannelReconfigurationFailure");
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_TransportChannelReconfigurationFailure, TransportChannelReconfigurationFailure_sequence);




  return offset;
}


static const per_sequence_t T_nonCriticalExtensions_158_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_nonCriticalExtensions_158(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_nonCriticalExtensions_158, T_nonCriticalExtensions_158_sequence);

  return offset;
}


static const per_sequence_t T_laterNonCriticalExtensions_55_sequence[] = {
  { &hf_rrc_transportFormatCombinationControlFailure_r3_add_ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_BIT_STRING },
  { &hf_rrc_nonCriticalExtensions_158, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_nonCriticalExtensions_158 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_laterNonCriticalExtensions_55(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_laterNonCriticalExtensions_55, T_laterNonCriticalExtensions_55_sequence);

  return offset;
}


static const per_sequence_t TransportFormatCombinationControlFailure_sequence[] = {
  { &hf_rrc_rrc_TransactionIdentifier, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RRC_TransactionIdentifier },
  { &hf_rrc_failureCause    , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_FailureCauseWithProtErr },
  { &hf_rrc_laterNonCriticalExtensions_55, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_laterNonCriticalExtensions_55 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_TransportFormatCombinationControlFailure(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
#line 451 "../../asn1/rrc/rrc.cnf"
  col_append_str(actx->pinfo->cinfo, COL_INFO, "TransportFormatCombinationControlFailure");
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_TransportFormatCombinationControlFailure, TransportFormatCombinationControlFailure_sequence);




  return offset;
}


static const per_sequence_t UECapabilityInformation_v370ext_sequence[] = {
  { &hf_rrc_ue_RadioAccessCapability_v370ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_RadioAccessCapability_v370ext },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UECapabilityInformation_v370ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UECapabilityInformation_v370ext, UECapabilityInformation_v370ext_sequence);

  return offset;
}


static const per_sequence_t UECapabilityInformation_v380ext_IEs_sequence[] = {
  { &hf_rrc_ue_RadioAccessCapability_v380ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_RadioAccessCapability_v380ext },
  { &hf_rrc_dl_PhysChCapabilityFDD_v380ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DL_PhysChCapabilityFDD_v380ext },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UECapabilityInformation_v380ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UECapabilityInformation_v380ext_IEs, UECapabilityInformation_v380ext_IEs_sequence);

  return offset;
}


static const per_sequence_t UECapabilityInformation_v3a0ext_IEs_sequence[] = {
  { &hf_rrc_ue_RadioAccessCapability_v3a0ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_RadioAccessCapability_v3a0ext },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UECapabilityInformation_v3a0ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UECapabilityInformation_v3a0ext_IEs, UECapabilityInformation_v3a0ext_IEs_sequence);

  return offset;
}



static int
dissect_rrc_T_ueCapabilityInformation_r3_add_ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_bit_string_containing_pdu_new(tvb, offset, actx, tree, hf_index,
                                                              NO_BOUND, NO_BOUND, FALSE, dissect_UECapabilityInformation_r3_add_ext_IEs_PDU);

  return offset;
}


static const per_sequence_t UECapabilityInformation_v4b0ext_sequence[] = {
  { &hf_rrc_ue_RadioAccessCapability_v4b0ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_RadioAccessCapability_v4b0ext },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UECapabilityInformation_v4b0ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UECapabilityInformation_v4b0ext, UECapabilityInformation_v4b0ext_sequence);

  return offset;
}


static const per_sequence_t UECapabilityInformation_v590ext_sequence[] = {
  { &hf_rrc_ue_RadioAccessCapability_v3g0ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_RadioAccessCapability_v3g0ext },
  { &hf_rrc_ue_RadioAccessCapability_v590ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_RadioAccessCapability_v590ext },
  { &hf_rrc_ue_RATSpecificCapability_v590ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_InterRAT_UE_RadioAccessCapability_v590ext },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UECapabilityInformation_v590ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UECapabilityInformation_v590ext, UECapabilityInformation_v590ext_sequence);

  return offset;
}


static const per_sequence_t UECapabilityInformation_v5c0ext_sequence[] = {
  { &hf_rrc_ue_RadioAccessCapability_v5c0ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_RadioAccessCapability_v5c0ext },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UECapabilityInformation_v5c0ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UECapabilityInformation_v5c0ext, UECapabilityInformation_v5c0ext_sequence);

  return offset;
}



static int
dissect_rrc_T_ueCapabilityContainer_01(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_bit_string_containing_pdu_new(tvb, offset, actx, tree, hf_index,
                                                              NO_BOUND, NO_BOUND, FALSE, dissect_UE_CapabilityContainer_IEs_PDU);

  return offset;
}


static const per_sequence_t UECapabilityInformation_v690ext_IEs_sequence[] = {
  { &hf_rrc_ueCapabilityContainer_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_ueCapabilityContainer_01 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UECapabilityInformation_v690ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UECapabilityInformation_v690ext_IEs, UECapabilityInformation_v690ext_IEs_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions_160_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_nonCriticalExtensions_160(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_nonCriticalExtensions_160, T_nonCriticalExtensions_160_sequence);

  return offset;
}


static const per_sequence_t T_v690NonCriticalExtensions_39_sequence[] = {
  { &hf_rrc_ueCapabilityInformation_v690ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UECapabilityInformation_v690ext_IEs },
  { &hf_rrc_nonCriticalExtensions_160, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_nonCriticalExtensions_160 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v690NonCriticalExtensions_39(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v690NonCriticalExtensions_39, T_v690NonCriticalExtensions_39_sequence);

  return offset;
}


static const per_sequence_t T_v5c0NonCriticalExtensions_01_sequence[] = {
  { &hf_rrc_ueCapabilityInformation_v5c0ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UECapabilityInformation_v5c0ext },
  { &hf_rrc_v690NonCriticalExtensions_39, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v690NonCriticalExtensions_39 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v5c0NonCriticalExtensions_01(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v5c0NonCriticalExtensions_01, T_v5c0NonCriticalExtensions_01_sequence);

  return offset;
}


static const per_sequence_t T_v590NonCriticalExtensions_27_sequence[] = {
  { &hf_rrc_ueCapabilityInformation_v590ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UECapabilityInformation_v590ext },
  { &hf_rrc_v5c0NonCriticalExtensions_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v5c0NonCriticalExtensions_01 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v590NonCriticalExtensions_27(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v590NonCriticalExtensions_27, T_v590NonCriticalExtensions_27_sequence);

  return offset;
}


static const per_sequence_t T_v4b0NonCriticalExtensions_15_sequence[] = {
  { &hf_rrc_ueCapabilityInformation_v4b0ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UECapabilityInformation_v4b0ext },
  { &hf_rrc_v590NonCriticalExtensions_27, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v590NonCriticalExtensions_27 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v4b0NonCriticalExtensions_15(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v4b0NonCriticalExtensions_15, T_v4b0NonCriticalExtensions_15_sequence);

  return offset;
}


static const per_sequence_t T_laterNonCriticalExtensions_57_sequence[] = {
  { &hf_rrc_ueCapabilityInformation_r3_add_ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_ueCapabilityInformation_r3_add_ext },
  { &hf_rrc_v4b0NonCriticalExtensions_15, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v4b0NonCriticalExtensions_15 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_laterNonCriticalExtensions_57(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_laterNonCriticalExtensions_57, T_laterNonCriticalExtensions_57_sequence);

  return offset;
}


static const per_sequence_t T_v3a0NonCriticalExtensions_10_sequence[] = {
  { &hf_rrc_ueCapabilityInformation_v3a0ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UECapabilityInformation_v3a0ext_IEs },
  { &hf_rrc_laterNonCriticalExtensions_57, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_laterNonCriticalExtensions_57 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v3a0NonCriticalExtensions_10(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v3a0NonCriticalExtensions_10, T_v3a0NonCriticalExtensions_10_sequence);

  return offset;
}


static const per_sequence_t T_v380NonCriticalExtensions_01_sequence[] = {
  { &hf_rrc_ueCapabilityInformation_v380ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UECapabilityInformation_v380ext_IEs },
  { &hf_rrc_v3a0NonCriticalExtensions_10, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v3a0NonCriticalExtensions_10 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v380NonCriticalExtensions_01(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v380NonCriticalExtensions_01, T_v380NonCriticalExtensions_01_sequence);

  return offset;
}


static const per_sequence_t T_v370NonCriticalExtensions_01_sequence[] = {
  { &hf_rrc_ueCapabilityInformation_v370ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UECapabilityInformation_v370ext },
  { &hf_rrc_v380NonCriticalExtensions_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v380NonCriticalExtensions_01 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v370NonCriticalExtensions_01(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v370NonCriticalExtensions_01, T_v370NonCriticalExtensions_01_sequence);

  return offset;
}


static const per_sequence_t UECapabilityInformation_sequence[] = {
  { &hf_rrc_rrc_TransactionIdentifier, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RRC_TransactionIdentifier },
  { &hf_rrc_ue_RadioAccessCapability_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_RadioAccessCapability },
  { &hf_rrc_ue_RATSpecificCapability, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_InterRAT_UE_RadioAccessCapabilityList },
  { &hf_rrc_v370NonCriticalExtensions_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v370NonCriticalExtensions_01 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UECapabilityInformation(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
#line 459 "../../asn1/rrc/rrc.cnf"
  col_append_str(actx->pinfo->cinfo, COL_INFO, "UECapabilityInformation");
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UECapabilityInformation, UECapabilityInformation_sequence);




  return offset;
}


static const per_sequence_t UplinkDirectTransfer_v690ext_IEs_sequence[] = {
  { &hf_rrc_measuredResultsOnRACHinterFreq, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MeasuredResultsOnRACHinterFreq },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UplinkDirectTransfer_v690ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UplinkDirectTransfer_v690ext_IEs, UplinkDirectTransfer_v690ext_IEs_sequence);

  return offset;
}


static const per_sequence_t MonitoredCellRACH_List_v7g0ext_sequence_of[1] = {
  { &hf_rrc_MonitoredCellRACH_List_v7g0ext_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DeltaRSCPPerCell },
};

static int
dissect_rrc_MonitoredCellRACH_List_v7g0ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_MonitoredCellRACH_List_v7g0ext, MonitoredCellRACH_List_v7g0ext_sequence_of,
                                                  1, 8, FALSE);

  return offset;
}


static const per_sequence_t MeasuredResultsOnRACH_v7g0ext_sequence[] = {
  { &hf_rrc_currentCell_DeltaRSCP, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DeltaRSCPPerCell },
  { &hf_rrc_monitoredCellRACH_List_v7g0ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MonitoredCellRACH_List_v7g0ext },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_MeasuredResultsOnRACH_v7g0ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_MeasuredResultsOnRACH_v7g0ext, MeasuredResultsOnRACH_v7g0ext_sequence);

  return offset;
}


static const per_sequence_t UplinkDirectTransfer_v7g0ext_IEs_sequence[] = {
  { &hf_rrc_measuredResultsOnRACH_v7g0ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MeasuredResultsOnRACH_v7g0ext },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UplinkDirectTransfer_v7g0ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UplinkDirectTransfer_v7g0ext_IEs, UplinkDirectTransfer_v7g0ext_IEs_sequence);

  return offset;
}


static const per_sequence_t UplinkDirectTransfer_vbxyext_IEs_sequence[] = {
  { &hf_rrc_measuredResultsOnRACHeutraFreq, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MeasuredResultsOnRACHeutraFreq },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UplinkDirectTransfer_vbxyext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UplinkDirectTransfer_vbxyext_IEs, UplinkDirectTransfer_vbxyext_IEs_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions_163_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_nonCriticalExtensions_163(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_nonCriticalExtensions_163, T_nonCriticalExtensions_163_sequence);

  return offset;
}


static const per_sequence_t T_vbxyNonCriticalExtensions_15_sequence[] = {
  { &hf_rrc_uplinkDirectTransfer_vbxyext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UplinkDirectTransfer_vbxyext_IEs },
  { &hf_rrc_nonCriticalExtensions_163, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_nonCriticalExtensions_163 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_vbxyNonCriticalExtensions_15(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_vbxyNonCriticalExtensions_15, T_vbxyNonCriticalExtensions_15_sequence);

  return offset;
}


static const per_sequence_t T_v7g0NonCriticalExtensions_17_sequence[] = {
  { &hf_rrc_uplinkDirectTransfer_v7g0ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UplinkDirectTransfer_v7g0ext_IEs },
  { &hf_rrc_vbxyNonCriticalExtensions_15, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_vbxyNonCriticalExtensions_15 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v7g0NonCriticalExtensions_17(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v7g0NonCriticalExtensions_17, T_v7g0NonCriticalExtensions_17_sequence);

  return offset;
}


static const per_sequence_t T_v690NonCriticalExtensions_40_sequence[] = {
  { &hf_rrc_uplinkDirectTransfer_v690ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UplinkDirectTransfer_v690ext_IEs },
  { &hf_rrc_v7g0NonCriticalExtensions_17, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v7g0NonCriticalExtensions_17 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v690NonCriticalExtensions_40(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v690NonCriticalExtensions_40, T_v690NonCriticalExtensions_40_sequence);

  return offset;
}


static const per_sequence_t T_laterNonCriticalExtensions_59_sequence[] = {
  { &hf_rrc_uplinkDirectTransfer_r3_add_ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_BIT_STRING },
  { &hf_rrc_v690NonCriticalExtensions_40, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v690NonCriticalExtensions_40 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_laterNonCriticalExtensions_59(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_laterNonCriticalExtensions_59, T_laterNonCriticalExtensions_59_sequence);

  return offset;
}


static const per_sequence_t UplinkDirectTransfer_sequence[] = {
  { &hf_rrc_cn_DomainIdentity, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CN_DomainIdentity },
  { &hf_rrc_nas_Message     , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_NAS_Message },
  { &hf_rrc_measuredResultsOnRACH, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MeasuredResultsOnRACH },
  { &hf_rrc_laterNonCriticalExtensions_59, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_laterNonCriticalExtensions_59 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UplinkDirectTransfer(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
#line 475 "../../asn1/rrc/rrc.cnf"
  col_append_str(actx->pinfo->cinfo, COL_INFO, "UplinkDirectTransfer");
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UplinkDirectTransfer, UplinkDirectTransfer_sequence);




  return offset;
}


static const value_string rrc_T_deferredMeasurementControlReading_06_vals[] = {
  {   0, "true" },
  { 0, NULL }
};


static int
dissect_rrc_T_deferredMeasurementControlReading_06(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t UTRANMobilityInformationConfirm_v770ext_IEs_sequence[] = {
  { &hf_rrc_deferredMeasurementControlReading_06, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_deferredMeasurementControlReading_06 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UTRANMobilityInformationConfirm_v770ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UTRANMobilityInformationConfirm_v770ext_IEs, UTRANMobilityInformationConfirm_v770ext_IEs_sequence);

  return offset;
}


static const value_string rrc_T_loggedMeasAvailable_05_vals[] = {
  {   0, "true" },
  { 0, NULL }
};


static int
dissect_rrc_T_loggedMeasAvailable_05(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t UTRANMobilityInformationConfirm_va40ext_IEs_sequence[] = {
  { &hf_rrc_loggedMeasAvailable_05, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_loggedMeasAvailable_05 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UTRANMobilityInformationConfirm_va40ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UTRANMobilityInformationConfirm_va40ext_IEs, UTRANMobilityInformationConfirm_va40ext_IEs_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions_177_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_nonCriticalExtensions_177(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_nonCriticalExtensions_177, T_nonCriticalExtensions_177_sequence);

  return offset;
}


static const per_sequence_t T_va40NonCriticalExtensions_16_sequence[] = {
  { &hf_rrc_utranMobilityInformationConfirm_va40ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UTRANMobilityInformationConfirm_va40ext_IEs },
  { &hf_rrc_nonCriticalExtensions_177, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_nonCriticalExtensions_177 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_va40NonCriticalExtensions_16(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_va40NonCriticalExtensions_16, T_va40NonCriticalExtensions_16_sequence);

  return offset;
}


static const per_sequence_t T_v770NonCriticalExtension_01_sequence[] = {
  { &hf_rrc_utranMobilityInformationConfirm_v770ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UTRANMobilityInformationConfirm_v770ext_IEs },
  { &hf_rrc_va40NonCriticalExtensions_17, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_va40NonCriticalExtensions_16 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v770NonCriticalExtension_01(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v770NonCriticalExtension_01, T_v770NonCriticalExtension_01_sequence);

  return offset;
}


static const per_sequence_t T_laterNonCriticalExtensions_65_sequence[] = {
  { &hf_rrc_utranMobilityInformationConfirm_r3_add_ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_BIT_STRING },
  { &hf_rrc_v770NonCriticalExtension_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v770NonCriticalExtension_01 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_laterNonCriticalExtensions_65(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_laterNonCriticalExtensions_65, T_laterNonCriticalExtensions_65_sequence);

  return offset;
}


static const per_sequence_t UTRANMobilityInformationConfirm_sequence[] = {
  { &hf_rrc_rrc_TransactionIdentifier, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RRC_TransactionIdentifier },
  { &hf_rrc_ul_IntegProtActivationInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_IntegrityProtActivationInfo },
  { &hf_rrc_count_C_ActivationTime, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_ActivationTime },
  { &hf_rrc_dummy_13        , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RB_ActivationTimeInfoList },
  { &hf_rrc_ul_CounterSynchronisationInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_CounterSynchronisationInfo },
  { &hf_rrc_laterNonCriticalExtensions_65, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_laterNonCriticalExtensions_65 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UTRANMobilityInformationConfirm(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
#line 499 "../../asn1/rrc/rrc.cnf"
  col_append_str(actx->pinfo->cinfo, COL_INFO, "UTRANMobilityInformationConfirm");
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UTRANMobilityInformationConfirm, UTRANMobilityInformationConfirm_sequence);




  return offset;
}


static const per_sequence_t T_nonCriticalExtensions_178_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_nonCriticalExtensions_178(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_nonCriticalExtensions_178, T_nonCriticalExtensions_178_sequence);

  return offset;
}


static const per_sequence_t T_laterNonCriticalExtensions_66_sequence[] = {
  { &hf_rrc_utranMobilityInformationFailure_r3_add_ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_BIT_STRING },
  { &hf_rrc_nonCriticalExtensions_178, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_nonCriticalExtensions_178 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_laterNonCriticalExtensions_66(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_laterNonCriticalExtensions_66, T_laterNonCriticalExtensions_66_sequence);

  return offset;
}


static const per_sequence_t UTRANMobilityInformationFailure_sequence[] = {
  { &hf_rrc_rrc_TransactionIdentifier, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RRC_TransactionIdentifier },
  { &hf_rrc_failureCause    , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_FailureCauseWithProtErr },
  { &hf_rrc_laterNonCriticalExtensions_66, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_laterNonCriticalExtensions_66 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UTRANMobilityInformationFailure(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
#line 503 "../../asn1/rrc/rrc.cnf"
  col_append_str(actx->pinfo->cinfo, COL_INFO, "UTRANMobilityInformationFailure");
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UTRANMobilityInformationFailure, UTRANMobilityInformationFailure_sequence);




  return offset;
}


static const per_sequence_t MBMS_SelectedServicesListFull_sequence_of[1] = {
  { &hf_rrc_MBMS_SelectedServicesListFull_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MBMS_ServiceIdentity_r6 },
};

static int
dissect_rrc_MBMS_SelectedServicesListFull(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_MBMS_SelectedServicesListFull, MBMS_SelectedServicesListFull_sequence_of,
                                                  1, maxMBMSservSelect, FALSE);

  return offset;
}


static const value_string rrc_T_status_vals[] = {
  {   0, "none" },
  {   1, "some" },
  { 0, NULL }
};

static const per_choice_t T_status_choice[] = {
  {   0, &hf_rrc_none            , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   1, &hf_rrc_some            , ASN1_NO_EXTENSIONS     , dissect_rrc_MBMS_SelectedServicesListFull },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_status(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_status, T_status_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t MBMS_SelectedServiceInfo_sequence[] = {
  { &hf_rrc_status          , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_status },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_MBMS_SelectedServiceInfo(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_MBMS_SelectedServiceInfo, MBMS_SelectedServiceInfo_sequence);

  return offset;
}


static const per_sequence_t MBMSModificationRequest_v6b0ext_IEs_sequence[] = {
  { &hf_rrc_mbmsSelectedServiceInfo, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MBMS_SelectedServiceInfo },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_MBMSModificationRequest_v6b0ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_MBMSModificationRequest_v6b0ext_IEs, MBMSModificationRequest_v6b0ext_IEs_sequence);

  return offset;
}


static const value_string rrc_T_mbmsSupportOfServiceChangeForAPtpRB_vals[] = {
  {   0, "true" },
  { 0, NULL }
};


static int
dissect_rrc_T_mbmsSupportOfServiceChangeForAPtpRB(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t MBMSModificationRequest_v6f0ext_IEs_sequence[] = {
  { &hf_rrc_mbmsSupportOfServiceChangeForAPtpRB, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_mbmsSupportOfServiceChangeForAPtpRB },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_MBMSModificationRequest_v6f0ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_MBMSModificationRequest_v6f0ext_IEs, MBMSModificationRequest_v6f0ext_IEs_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions_183_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_nonCriticalExtensions_183(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_nonCriticalExtensions_183, T_nonCriticalExtensions_183_sequence);

  return offset;
}


static const per_sequence_t T_v6f0NonCriticalExtensions_02_sequence[] = {
  { &hf_rrc_mbmsModificationRequest_v6f0ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MBMSModificationRequest_v6f0ext_IEs },
  { &hf_rrc_nonCriticalExtensions_183, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_nonCriticalExtensions_183 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v6f0NonCriticalExtensions_02(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v6f0NonCriticalExtensions_02, T_v6f0NonCriticalExtensions_02_sequence);

  return offset;
}


static const per_sequence_t T_v6b0NonCriticalExtensions_13_sequence[] = {
  { &hf_rrc_mbmsModificationRequest_v6b0ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MBMSModificationRequest_v6b0ext_IEs },
  { &hf_rrc_v6f0NonCriticalExtensions_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v6f0NonCriticalExtensions_02 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v6b0NonCriticalExtensions_13(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v6b0NonCriticalExtensions_13, T_v6b0NonCriticalExtensions_13_sequence);

  return offset;
}


static const per_sequence_t MBMSModificationRequest_sequence[] = {
  { &hf_rrc_mbms_PreferredFreqRequest, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MBMS_ServiceIdentity_r6 },
  { &hf_rrc_rb_InformationReleaseList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RB_InformationReleaseList },
  { &hf_rrc_v6b0NonCriticalExtensions_14, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v6b0NonCriticalExtensions_13 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_MBMSModificationRequest(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
#line 265 "../../asn1/rrc/rrc.cnf"
  col_append_str(actx->pinfo->cinfo, COL_INFO, "MBMSModificationRequest");
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_MBMSModificationRequest, MBMSModificationRequest_sequence);




  return offset;
}



static int
dissect_rrc_INTEGER_0_7200(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            0U, 7200U, NULL, FALSE);

  return offset;
}


static const per_sequence_t LoggedMeasServingCellMeas_FDD_sequence[] = {
  { &hf_rrc_cellIdentity    , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CellIdentity },
  { &hf_rrc_cpich_RSCP_01   , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CPICH_RSCP },
  { &hf_rrc_cpich_Ec_N0_01  , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CPICH_Ec_N0 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_LoggedMeasServingCellMeas_FDD(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_LoggedMeasServingCellMeas_FDD, LoggedMeasServingCellMeas_FDD_sequence);

  return offset;
}


static const per_sequence_t LoggedMeasNeighbourInfo_FDD_sequence[] = {
  { &hf_rrc_primaryCPICH_Info, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PrimaryCPICH_Info },
  { &hf_rrc_cpich_RSCP_01   , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CPICH_RSCP },
  { &hf_rrc_cpich_Ec_N0_01  , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CPICH_Ec_N0 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_LoggedMeasNeighbourInfo_FDD(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_LoggedMeasNeighbourInfo_FDD, LoggedMeasNeighbourInfo_FDD_sequence);

  return offset;
}


static const per_sequence_t LoggedMeasIntrafreqNeighbourList_FDD_sequence_of[1] = {
  { &hf_rrc_LoggedMeasIntrafreqNeighbourList_FDD_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_LoggedMeasNeighbourInfo_FDD },
};

static int
dissect_rrc_LoggedMeasIntrafreqNeighbourList_FDD(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_LoggedMeasIntrafreqNeighbourList_FDD, LoggedMeasIntrafreqNeighbourList_FDD_sequence_of,
                                                  1, maxnumLoggedMeas, FALSE);

  return offset;
}


static const per_sequence_t SEQUENCE_SIZE_1_maxnumLoggedMeas_OF_LoggedMeasNeighbourInfo_FDD_sequence_of[1] = {
  { &hf_rrc_loggedMeasInterfreqNeighbourList_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_LoggedMeasNeighbourInfo_FDD },
};

static int
dissect_rrc_SEQUENCE_SIZE_1_maxnumLoggedMeas_OF_LoggedMeasNeighbourInfo_FDD(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_SEQUENCE_SIZE_1_maxnumLoggedMeas_OF_LoggedMeasNeighbourInfo_FDD, SEQUENCE_SIZE_1_maxnumLoggedMeas_OF_LoggedMeasNeighbourInfo_FDD_sequence_of,
                                                  1, maxnumLoggedMeas, FALSE);

  return offset;
}


static const per_sequence_t LoggedMeasInterfreqInfo_FDD_sequence[] = {
  { &hf_rrc_frequencyInfo   , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_FrequencyInfo },
  { &hf_rrc_loggedMeasInterfreqNeighbourList, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SEQUENCE_SIZE_1_maxnumLoggedMeas_OF_LoggedMeasNeighbourInfo_FDD },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_LoggedMeasInterfreqInfo_FDD(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_LoggedMeasInterfreqInfo_FDD, LoggedMeasInterfreqInfo_FDD_sequence);

  return offset;
}


static const per_sequence_t LoggedMeasInterfreqList_FDD_sequence_of[1] = {
  { &hf_rrc_LoggedMeasInterfreqList_FDD_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_LoggedMeasInterfreqInfo_FDD },
};

static int
dissect_rrc_LoggedMeasInterfreqList_FDD(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_LoggedMeasInterfreqList_FDD, LoggedMeasInterfreqList_FDD_sequence_of,
                                                  1, maxNumFDDFreqs, FALSE);

  return offset;
}


static const per_sequence_t LoggedMeasEUTRAFreqNeighbourInfo_sequence[] = {
  { &hf_rrc_physicalCellIdentity, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_EUTRA_PhysicalCellIdentity },
  { &hf_rrc_rSRP            , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_0_97 },
  { &hf_rrc_rSRQ            , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_0_33 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_LoggedMeasEUTRAFreqNeighbourInfo(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_LoggedMeasEUTRAFreqNeighbourInfo, LoggedMeasEUTRAFreqNeighbourInfo_sequence);

  return offset;
}


static const per_sequence_t SEQUENCE_SIZE_1_maxnumLoggedMeas_OF_LoggedMeasEUTRAFreqNeighbourInfo_sequence_of[1] = {
  { &hf_rrc_loggedMeasEUTRAFreqNeighbourInfoList_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_LoggedMeasEUTRAFreqNeighbourInfo },
};

static int
dissect_rrc_SEQUENCE_SIZE_1_maxnumLoggedMeas_OF_LoggedMeasEUTRAFreqNeighbourInfo(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_SEQUENCE_SIZE_1_maxnumLoggedMeas_OF_LoggedMeasEUTRAFreqNeighbourInfo, SEQUENCE_SIZE_1_maxnumLoggedMeas_OF_LoggedMeasEUTRAFreqNeighbourInfo_sequence_of,
                                                  1, maxnumLoggedMeas, FALSE);

  return offset;
}


static const per_sequence_t LoggedMeasEUTRAFreqInfo_sequence[] = {
  { &hf_rrc_earfcn          , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_EARFCN },
  { &hf_rrc_loggedMeasEUTRAFreqNeighbourInfoList, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SEQUENCE_SIZE_1_maxnumLoggedMeas_OF_LoggedMeasEUTRAFreqNeighbourInfo },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_LoggedMeasEUTRAFreqInfo(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_LoggedMeasEUTRAFreqInfo, LoggedMeasEUTRAFreqInfo_sequence);

  return offset;
}


static const per_sequence_t LoggedMeasEUTRAFreqList_sequence_of[1] = {
  { &hf_rrc_LoggedMeasEUTRAFreqList_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_LoggedMeasEUTRAFreqInfo },
};

static int
dissect_rrc_LoggedMeasEUTRAFreqList(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_LoggedMeasEUTRAFreqList, LoggedMeasEUTRAFreqList_sequence_of,
                                                  1, maxNumEUTRAFreqs, FALSE);

  return offset;
}


static const per_sequence_t LoggedMeasGSMNeighbourCellsinfo_sequence[] = {
  { &hf_rrc_bsic            , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BSIC },
  { &hf_rrc_frequency_band  , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_Frequency_Band },
  { &hf_rrc_bcch_ARFCN      , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BCCH_ARFCN },
  { &hf_rrc_gsm_CarrierRSSI , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_GSM_CarrierRSSI },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_LoggedMeasGSMNeighbourCellsinfo(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_LoggedMeasGSMNeighbourCellsinfo, LoggedMeasGSMNeighbourCellsinfo_sequence);

  return offset;
}


static const per_sequence_t LoggedMeasGSMNeighbourCellsList_sequence_of[1] = {
  { &hf_rrc_LoggedMeasGSMNeighbourCellsList_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_LoggedMeasGSMNeighbourCellsinfo },
};

static int
dissect_rrc_LoggedMeasGSMNeighbourCellsList(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_LoggedMeasGSMNeighbourCellsList, LoggedMeasGSMNeighbourCellsList_sequence_of,
                                                  1, maxnumLoggedMeas, FALSE);

  return offset;
}


static const per_sequence_t LoggedMeasInterRATNeighbourMeas_sequence[] = {
  { &hf_rrc_loggedMeasEUTRAFreqList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_LoggedMeasEUTRAFreqList },
  { &hf_rrc_loggedMeasGSMNeighbourCellsList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_LoggedMeasGSMNeighbourCellsList },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_LoggedMeasInterRATNeighbourMeas(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_LoggedMeasInterRATNeighbourMeas, LoggedMeasInterRATNeighbourMeas_sequence);

  return offset;
}


static const value_string rrc_LoggedMeasLocationInfo_vals[] = {
  {   0, "ellipsoidPoint" },
  {   1, "ellipsoidPointAltitude" },
  { 0, NULL }
};

static const per_choice_t LoggedMeasLocationInfo_choice[] = {
  {   0, &hf_rrc_ellipsoidPoint  , ASN1_NO_EXTENSIONS     , dissect_rrc_EllipsoidPoint },
  {   1, &hf_rrc_ellipsoidPointAltitude, ASN1_NO_EXTENSIONS     , dissect_rrc_EllipsoidPointAltitude },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_LoggedMeasLocationInfo(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_LoggedMeasLocationInfo, LoggedMeasLocationInfo_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t LoggedMeasInfo_FDD_sequence[] = {
  { &hf_rrc_relativeTimeStamp, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_0_7200 },
  { &hf_rrc_loggedMeasServingCellMeas, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_LoggedMeasServingCellMeas_FDD },
  { &hf_rrc_loggedMeasIntrafreqNeighbourList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_LoggedMeasIntrafreqNeighbourList_FDD },
  { &hf_rrc_loggedMeasInterfreqList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_LoggedMeasInterfreqList_FDD },
  { &hf_rrc_loggedMeasInterRATNeighbourMeas, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_LoggedMeasInterRATNeighbourMeas },
  { &hf_rrc_loggedMeasLocationInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_LoggedMeasLocationInfo },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_LoggedMeasInfo_FDD(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_LoggedMeasInfo_FDD, LoggedMeasInfo_FDD_sequence);

  return offset;
}


static const per_sequence_t LoggedMeasInfoList_FDD_sequence_of[1] = {
  { &hf_rrc_LoggedMeasInfoList_FDD_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_LoggedMeasInfo_FDD },
};

static int
dissect_rrc_LoggedMeasInfoList_FDD(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_LoggedMeasInfoList_FDD, LoggedMeasInfoList_FDD_sequence_of,
                                                  1, maxLoggedMeasReport, FALSE);

  return offset;
}


static const per_sequence_t T_fdd_158_sequence[] = {
  { &hf_rrc_loggedMeasInfoList_02, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_LoggedMeasInfoList_FDD },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_fdd_158(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_fdd_158, T_fdd_158_sequence);

  return offset;
}


static const per_sequence_t LoggedMeasServingCellMeas_TDD128_sequence[] = {
  { &hf_rrc_cellIdentity    , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CellIdentity },
  { &hf_rrc_primaryCCPCH_RSCP, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PrimaryCCPCH_RSCP },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_LoggedMeasServingCellMeas_TDD128(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_LoggedMeasServingCellMeas_TDD128, LoggedMeasServingCellMeas_TDD128_sequence);

  return offset;
}


static const per_sequence_t LoggedMeasNeighbourMeas_TDD128_sequence[] = {
  { &hf_rrc_cellParametersID, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CellParametersID },
  { &hf_rrc_primaryCCPCH_RSCP, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PrimaryCCPCH_RSCP },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_LoggedMeasNeighbourMeas_TDD128(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_LoggedMeasNeighbourMeas_TDD128, LoggedMeasNeighbourMeas_TDD128_sequence);

  return offset;
}


static const per_sequence_t LoggedMeasIntrafreqNeighbourMeasList_TDD128_sequence_of[1] = {
  { &hf_rrc_LoggedMeasIntrafreqNeighbourMeasList_TDD128_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_LoggedMeasNeighbourMeas_TDD128 },
};

static int
dissect_rrc_LoggedMeasIntrafreqNeighbourMeasList_TDD128(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_LoggedMeasIntrafreqNeighbourMeasList_TDD128, LoggedMeasIntrafreqNeighbourMeasList_TDD128_sequence_of,
                                                  1, maxnumLoggedMeas, FALSE);

  return offset;
}


static const per_sequence_t LoggedMeasInterfreqNeighbourList_TDD128_sequence_of[1] = {
  { &hf_rrc_LoggedMeasInterfreqNeighbourList_TDD128_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_LoggedMeasNeighbourMeas_TDD128 },
};

static int
dissect_rrc_LoggedMeasInterfreqNeighbourList_TDD128(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_LoggedMeasInterfreqNeighbourList_TDD128, LoggedMeasInterfreqNeighbourList_TDD128_sequence_of,
                                                  1, maxnumLoggedMeas, FALSE);

  return offset;
}


static const per_sequence_t LoggedMeasInterfreqInfo_TDD128_sequence[] = {
  { &hf_rrc_frequencyInfo   , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_FrequencyInfo },
  { &hf_rrc_loggedMeasInterfreqNeighbourList_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_LoggedMeasInterfreqNeighbourList_TDD128 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_LoggedMeasInterfreqInfo_TDD128(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_LoggedMeasInterfreqInfo_TDD128, LoggedMeasInterfreqInfo_TDD128_sequence);

  return offset;
}


static const per_sequence_t LoggedMeasInterfreqList_TDD128_sequence_of[1] = {
  { &hf_rrc_LoggedMeasInterfreqList_TDD128_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_LoggedMeasInterfreqInfo_TDD128 },
};

static int
dissect_rrc_LoggedMeasInterfreqList_TDD128(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_LoggedMeasInterfreqList_TDD128, LoggedMeasInterfreqList_TDD128_sequence_of,
                                                  1, maxNumTDDFreqs, FALSE);

  return offset;
}


static const per_sequence_t LoggedMeasInfo_TDD128_sequence[] = {
  { &hf_rrc_relativeTimeStamp, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_0_7200 },
  { &hf_rrc_loggedMeasServingCellMeas_02, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_LoggedMeasServingCellMeas_TDD128 },
  { &hf_rrc_loggedMeasIntrafreqNeighbourList_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_LoggedMeasIntrafreqNeighbourMeasList_TDD128 },
  { &hf_rrc_loggedMeasInterfreqList_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_LoggedMeasInterfreqList_TDD128 },
  { &hf_rrc_loggedMeasInterRATNeighbourMeas, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_LoggedMeasInterRATNeighbourMeas },
  { &hf_rrc_loggedMeasLocationInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_LoggedMeasLocationInfo },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_LoggedMeasInfo_TDD128(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_LoggedMeasInfo_TDD128, LoggedMeasInfo_TDD128_sequence);

  return offset;
}


static const per_sequence_t LoggedMeasInfoList_TDD128_sequence_of[1] = {
  { &hf_rrc_LoggedMeasInfoList_TDD128_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_LoggedMeasInfo_TDD128 },
};

static int
dissect_rrc_LoggedMeasInfoList_TDD128(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_LoggedMeasInfoList_TDD128, LoggedMeasInfoList_TDD128_sequence_of,
                                                  1, maxLoggedMeasReport, FALSE);

  return offset;
}


static const per_sequence_t T_tdd_106_sequence[] = {
  { &hf_rrc_loggedMeasInfoList_03, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_LoggedMeasInfoList_TDD128 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_tdd_106(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_tdd_106, T_tdd_106_sequence);

  return offset;
}


static const value_string rrc_T_modeSpecificInfo_119_vals[] = {
  {   0, "fdd" },
  {   1, "tdd" },
  { 0, NULL }
};

static const per_choice_t T_modeSpecificInfo_119_choice[] = {
  {   0, &hf_rrc_fdd_164         , ASN1_NO_EXTENSIONS     , dissect_rrc_T_fdd_158 },
  {   1, &hf_rrc_tdd_114         , ASN1_NO_EXTENSIONS     , dissect_rrc_T_tdd_106 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_modeSpecificInfo_119(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_modeSpecificInfo_119, T_modeSpecificInfo_119_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t LoggedMeasInfoList_sequence[] = {
  { &hf_rrc_modeSpecificInfo_119, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_modeSpecificInfo_119 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_LoggedMeasInfoList(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_LoggedMeasInfoList, LoggedMeasInfoList_sequence);

  return offset;
}


static const value_string rrc_T_loggedMeasAvailable_06_vals[] = {
  {   0, "true" },
  { 0, NULL }
};


static int
dissect_rrc_T_loggedMeasAvailable_06(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t LoggedMeasReport_sequence[] = {
  { &hf_rrc_absoluteTimeInfo, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_48 },
  { &hf_rrc_traceReference  , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TraceReference },
  { &hf_rrc_traceRecordingSession, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TraceRecordingSession },
  { &hf_rrc_tce_Id          , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TCE_Id },
  { &hf_rrc_loggedMeasInfoList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_LoggedMeasInfoList },
  { &hf_rrc_loggedMeasAvailable_06, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_loggedMeasAvailable_06 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_LoggedMeasReport(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_LoggedMeasReport, LoggedMeasReport_sequence);

  return offset;
}


static const per_sequence_t LoggedUTRACellInfo_sequence[] = {
  { &hf_rrc_plmn_Identity   , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PLMN_Identity },
  { &hf_rrc_cellIdentity    , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CellIdentity },
  { &hf_rrc_uarfcn          , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UARFCN },
  { &hf_rrc_cellParametersID, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CellParametersID },
  { &hf_rrc_primaryScramblingCode, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_PrimaryScramblingCode },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_LoggedUTRACellInfo(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_LoggedUTRACellInfo, LoggedUTRACellInfo_sequence);

  return offset;
}


static const per_sequence_t LoggedEUTRACellInfo_sequence[] = {
  { &hf_rrc_plmn_Identity   , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PLMN_Identity },
  { &hf_rrc_trackingAreaCode, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_16 },
  { &hf_rrc_eutraCellIdentity, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_28 },
  { &hf_rrc_earfcn          , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_EARFCN },
  { &hf_rrc_physicalCellIdentity, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_EUTRA_PhysicalCellIdentity },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_LoggedEUTRACellInfo(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_LoggedEUTRACellInfo, LoggedEUTRACellInfo_sequence);

  return offset;
}


static const per_sequence_t LoggedGSMCellInfo_sequence[] = {
  { &hf_rrc_plmn_Identity   , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PLMN_Identity },
  { &hf_rrc_lac             , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_16 },
  { &hf_rrc_gsmCellIdentity , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_16 },
  { &hf_rrc_bcch_ARFCN      , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BCCH_ARFCN },
  { &hf_rrc_bandIndicator_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_Frequency_Band },
  { &hf_rrc_bsic            , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BSIC },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_LoggedGSMCellInfo(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_LoggedGSMCellInfo, LoggedGSMCellInfo_sequence);

  return offset;
}


static const value_string rrc_LoggedCellInfo_vals[] = {
  {   0, "loggedUTRACellInfo" },
  {   1, "loggedEUTRACellInfo" },
  {   2, "loggedGSMCellInfo" },
  { 0, NULL }
};

static const per_choice_t LoggedCellInfo_choice[] = {
  {   0, &hf_rrc_loggedUTRACellInfo, ASN1_NO_EXTENSIONS     , dissect_rrc_LoggedUTRACellInfo },
  {   1, &hf_rrc_loggedEUTRACellInfo, ASN1_NO_EXTENSIONS     , dissect_rrc_LoggedEUTRACellInfo },
  {   2, &hf_rrc_loggedGSMCellInfo, ASN1_NO_EXTENSIONS     , dissect_rrc_LoggedGSMCellInfo },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_LoggedCellInfo(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_LoggedCellInfo, LoggedCellInfo_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t LoggedANRReportInfo_sequence[] = {
  { &hf_rrc_plmn_Identity   , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PLMN_Identity },
  { &hf_rrc_servingCell     , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CellIdentity },
  { &hf_rrc_loggedCellInfo  , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_LoggedCellInfo },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_LoggedANRReportInfo(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_LoggedANRReportInfo, LoggedANRReportInfo_sequence);

  return offset;
}


static const per_sequence_t LoggedANRReportInfoList_sequence_of[1] = {
  { &hf_rrc_LoggedANRReportInfoList_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_LoggedANRReportInfo },
};

static int
dissect_rrc_LoggedANRReportInfoList(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_LoggedANRReportInfoList, LoggedANRReportInfoList_sequence_of,
                                                  1, maxNumANRLoggedItems, FALSE);

  return offset;
}


static const per_sequence_t LoggedMeasServingCellMeas_FDD_vbxyext_sequence[] = {
  { &hf_rrc_plmnIdentity    , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_PLMN_Identity },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_LoggedMeasServingCellMeas_FDD_vbxyext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_LoggedMeasServingCellMeas_FDD_vbxyext, LoggedMeasServingCellMeas_FDD_vbxyext_sequence);

  return offset;
}


static const value_string rrc_LoggedMeasLocationInfo_vbxyext_vals[] = {
  {   0, "ellipsoidPoint" },
  {   1, "ellipsoidPointUncertCircle" },
  {   2, "ellipsoidPointUncertEllipse" },
  {   3, "ellipsoidPointAltitude" },
  {   4, "ellipsoidPointAltitudeEllipse" },
  { 0, NULL }
};

static const per_choice_t LoggedMeasLocationInfo_vbxyext_choice[] = {
  {   0, &hf_rrc_ellipsoidPoint  , ASN1_NO_EXTENSIONS     , dissect_rrc_EllipsoidPoint },
  {   1, &hf_rrc_ellipsoidPointUncertCircle, ASN1_NO_EXTENSIONS     , dissect_rrc_EllipsoidPointUncertCircle },
  {   2, &hf_rrc_ellipsoidPointUncertEllipse, ASN1_NO_EXTENSIONS     , dissect_rrc_EllipsoidPointUncertEllipse },
  {   3, &hf_rrc_ellipsoidPointAltitude, ASN1_NO_EXTENSIONS     , dissect_rrc_EllipsoidPointAltitude },
  {   4, &hf_rrc_ellipsoidPointAltitudeEllipse, ASN1_NO_EXTENSIONS     , dissect_rrc_EllipsoidPointAltitudeEllipsoide },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_LoggedMeasLocationInfo_vbxyext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_LoggedMeasLocationInfo_vbxyext, LoggedMeasLocationInfo_vbxyext_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t LoggedMeasInfo_FDD_vbxyext_sequence[] = {
  { &hf_rrc_loggedMeasServingCellMeas_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_LoggedMeasServingCellMeas_FDD_vbxyext },
  { &hf_rrc_loggedMeasLocationInfo_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_LoggedMeasLocationInfo_vbxyext },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_LoggedMeasInfo_FDD_vbxyext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_LoggedMeasInfo_FDD_vbxyext, LoggedMeasInfo_FDD_vbxyext_sequence);

  return offset;
}


static const per_sequence_t LoggedMeasInfoList_FDD_vbxyext_sequence_of[1] = {
  { &hf_rrc_LoggedMeasInfoList_FDD_vbxyext_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_LoggedMeasInfo_FDD_vbxyext },
};

static int
dissect_rrc_LoggedMeasInfoList_FDD_vbxyext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_LoggedMeasInfoList_FDD_vbxyext, LoggedMeasInfoList_FDD_vbxyext_sequence_of,
                                                  1, maxLoggedMeasReport, FALSE);

  return offset;
}


static const per_sequence_t T_fdd_159_sequence[] = {
  { &hf_rrc_loggedMeasInfoList_04, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_LoggedMeasInfoList_FDD_vbxyext },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_fdd_159(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_fdd_159, T_fdd_159_sequence);

  return offset;
}


static const per_sequence_t LoggedMeasServingCellMeas_TDD128_vbxyext_sequence[] = {
  { &hf_rrc_plmnIdentity    , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_PLMN_Identity },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_LoggedMeasServingCellMeas_TDD128_vbxyext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_LoggedMeasServingCellMeas_TDD128_vbxyext, LoggedMeasServingCellMeas_TDD128_vbxyext_sequence);

  return offset;
}


static const per_sequence_t LoggedMeasInfo_TDD128_vbxyext_sequence[] = {
  { &hf_rrc_loggedMeasServingCellMeas_03, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_LoggedMeasServingCellMeas_TDD128_vbxyext },
  { &hf_rrc_loggedMeasLocationInfo_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_LoggedMeasLocationInfo_vbxyext },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_LoggedMeasInfo_TDD128_vbxyext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_LoggedMeasInfo_TDD128_vbxyext, LoggedMeasInfo_TDD128_vbxyext_sequence);

  return offset;
}


static const per_sequence_t LoggedMeasInfoList_TDD128_vbxyext_sequence_of[1] = {
  { &hf_rrc_LoggedMeasInfoList_TDD128_vbxyext_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_LoggedMeasInfo_TDD128_vbxyext },
};

static int
dissect_rrc_LoggedMeasInfoList_TDD128_vbxyext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_LoggedMeasInfoList_TDD128_vbxyext, LoggedMeasInfoList_TDD128_vbxyext_sequence_of,
                                                  1, maxLoggedMeasReport, FALSE);

  return offset;
}


static const per_sequence_t T_tdd_107_sequence[] = {
  { &hf_rrc_loggedMeasInfoList_05, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_LoggedMeasInfoList_TDD128_vbxyext },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_tdd_107(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_tdd_107, T_tdd_107_sequence);

  return offset;
}


static const value_string rrc_T_modeSpecificInfo_120_vals[] = {
  {   0, "fdd" },
  {   1, "tdd" },
  { 0, NULL }
};

static const per_choice_t T_modeSpecificInfo_120_choice[] = {
  {   0, &hf_rrc_fdd_165         , ASN1_NO_EXTENSIONS     , dissect_rrc_T_fdd_159 },
  {   1, &hf_rrc_tdd_115         , ASN1_NO_EXTENSIONS     , dissect_rrc_T_tdd_107 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_modeSpecificInfo_120(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_modeSpecificInfo_120, T_modeSpecificInfo_120_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t LoggedMeasInfoList_vbxyext_sequence[] = {
  { &hf_rrc_modeSpecificInfo_120, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_modeSpecificInfo_120 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_LoggedMeasInfoList_vbxyext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_LoggedMeasInfoList_vbxyext, LoggedMeasInfoList_vbxyext_sequence);

  return offset;
}


static const per_sequence_t LoggedMeasReport_vbxyext_sequence[] = {
  { &hf_rrc_loggedMeasInfoList_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_LoggedMeasInfoList_vbxyext },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_LoggedMeasReport_vbxyext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_LoggedMeasReport_vbxyext, LoggedMeasReport_vbxyext_sequence);

  return offset;
}



static int
dissect_rrc_INTEGER_1_172800(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            1U, 172800U, NULL, FALSE);

  return offset;
}



static int
dissect_rrc_INTEGER_0_8(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            0U, 8U, NULL, FALSE);

  return offset;
}


static const per_sequence_t LoggedConnectionEstablishmentFailureAccessInfo_FDD_sequence[] = {
  { &hf_rrc_numberOfRRCMsgTransmitted, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_0_8 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_LoggedConnectionEstablishmentFailureAccessInfo_FDD(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_LoggedConnectionEstablishmentFailureAccessInfo_FDD, LoggedConnectionEstablishmentFailureAccessInfo_FDD_sequence);

  return offset;
}


static const value_string rrc_T_location_vals[] = {
  {   0, "ellipsoidPoint" },
  {   1, "ellipsoidPointUncertCircle" },
  {   2, "ellipsoidPointUncertEllipse" },
  {   3, "ellipsoidPointAltitude" },
  {   4, "ellipsoidPointAltitudeEllipse" },
  { 0, NULL }
};

static const per_choice_t T_location_choice[] = {
  {   0, &hf_rrc_ellipsoidPoint  , ASN1_NO_EXTENSIONS     , dissect_rrc_EllipsoidPoint },
  {   1, &hf_rrc_ellipsoidPointUncertCircle, ASN1_NO_EXTENSIONS     , dissect_rrc_EllipsoidPointUncertCircle },
  {   2, &hf_rrc_ellipsoidPointUncertEllipse, ASN1_NO_EXTENSIONS     , dissect_rrc_EllipsoidPointUncertEllipse },
  {   3, &hf_rrc_ellipsoidPointAltitude, ASN1_NO_EXTENSIONS     , dissect_rrc_EllipsoidPointAltitude },
  {   4, &hf_rrc_ellipsoidPointAltitudeEllipse, ASN1_NO_EXTENSIONS     , dissect_rrc_EllipsoidPointAltitudeEllipsoide },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_location(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_location, T_location_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t LoggedConnectionEstablishmentFailureLocationInfo_sequence[] = {
  { &hf_rrc_location        , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_location },
  { &hf_rrc_horizontalVelocity, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_HorizontalVelocity },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_LoggedConnectionEstablishmentFailureLocationInfo(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_LoggedConnectionEstablishmentFailureLocationInfo, LoggedConnectionEstablishmentFailureLocationInfo_sequence);

  return offset;
}


static const per_sequence_t LoggedConnectionEstablishmentFailureInfo_FDD_sequence[] = {
  { &hf_rrc_loggedConnectionEstablishmentFailureAccessInfo_FDD, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_LoggedConnectionEstablishmentFailureAccessInfo_FDD },
  { &hf_rrc_loggedMeasFailedCellMeas, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_LoggedMeasServingCellMeas_FDD },
  { &hf_rrc_loggedMeasIntrafreqNeighbourList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_LoggedMeasIntrafreqNeighbourList_FDD },
  { &hf_rrc_loggedMeasInterfreqList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_LoggedMeasInterfreqList_FDD },
  { &hf_rrc_loggedMeasInterRATNeighbourMeas, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_LoggedMeasInterRATNeighbourMeas },
  { &hf_rrc_loggedConnectionEstablishmentFailureLocationInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_LoggedConnectionEstablishmentFailureLocationInfo },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_LoggedConnectionEstablishmentFailureInfo_FDD(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_LoggedConnectionEstablishmentFailureInfo_FDD, LoggedConnectionEstablishmentFailureInfo_FDD_sequence);

  return offset;
}


static const per_sequence_t T_fdd_157_sequence[] = {
  { &hf_rrc_loggedConnectionEstablishmentFailureInfo, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_LoggedConnectionEstablishmentFailureInfo_FDD },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_fdd_157(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_fdd_157, T_fdd_157_sequence);

  return offset;
}


static const per_sequence_t LoggedConnectionEstablishmentFailureAccessInfo_TDD_sequence[] = {
  { &hf_rrc_numberOfRRCMsgTransmitted, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_0_8 },
  { &hf_rrc_fpachReceived   , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { &hf_rrc_e_RUCCHFailure  , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_BOOLEAN },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_LoggedConnectionEstablishmentFailureAccessInfo_TDD(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_LoggedConnectionEstablishmentFailureAccessInfo_TDD, LoggedConnectionEstablishmentFailureAccessInfo_TDD_sequence);

  return offset;
}


static const per_sequence_t LoggedConnectionEstablishmentFailureInfo_TDD128_sequence[] = {
  { &hf_rrc_loggedConnectionEstablishmentFailureAccessInfo_TDD, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_LoggedConnectionEstablishmentFailureAccessInfo_TDD },
  { &hf_rrc_loggedMeasFailedCellMeas_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_LoggedMeasServingCellMeas_TDD128 },
  { &hf_rrc_loggedMeasIntrafreqNeighbourList_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_LoggedMeasIntrafreqNeighbourMeasList_TDD128 },
  { &hf_rrc_loggedMeasInterfreqList_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_LoggedMeasInterfreqList_TDD128 },
  { &hf_rrc_loggedMeasInterRATNeighbourMeas, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_LoggedMeasInterRATNeighbourMeas },
  { &hf_rrc_loggedConnectionEstablishmentFailureLocationInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_LoggedConnectionEstablishmentFailureLocationInfo },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_LoggedConnectionEstablishmentFailureInfo_TDD128(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_LoggedConnectionEstablishmentFailureInfo_TDD128, LoggedConnectionEstablishmentFailureInfo_TDD128_sequence);

  return offset;
}


static const per_sequence_t T_tdd_105_sequence[] = {
  { &hf_rrc_loggedConnectionEstablishmentFailureInfo_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_LoggedConnectionEstablishmentFailureInfo_TDD128 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_tdd_105(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_tdd_105, T_tdd_105_sequence);

  return offset;
}


static const value_string rrc_T_modeSpecificInfo_118_vals[] = {
  {   0, "fdd" },
  {   1, "tdd" },
  { 0, NULL }
};

static const per_choice_t T_modeSpecificInfo_118_choice[] = {
  {   0, &hf_rrc_fdd_163         , ASN1_NO_EXTENSIONS     , dissect_rrc_T_fdd_157 },
  {   1, &hf_rrc_tdd_113         , ASN1_NO_EXTENSIONS     , dissect_rrc_T_tdd_105 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_modeSpecificInfo_118(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_modeSpecificInfo_118, T_modeSpecificInfo_118_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t LoggedConnectionEstablishmentFailureReport_sequence[] = {
  { &hf_rrc_timeSinceFailure, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_1_172800 },
  { &hf_rrc_modeSpecificInfo_118, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_modeSpecificInfo_118 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_LoggedConnectionEstablishmentFailureReport(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_LoggedConnectionEstablishmentFailureReport, LoggedConnectionEstablishmentFailureReport_sequence);

  return offset;
}


static const per_sequence_t UEInformationResponse_vbxyext_IEs_sequence[] = {
  { &hf_rrc_loggedMeasReport_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_LoggedMeasReport_vbxyext },
  { &hf_rrc_loggedConnectionEstablishmentFailureReport, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_LoggedConnectionEstablishmentFailureReport },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UEInformationResponse_vbxyext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UEInformationResponse_vbxyext_IEs, UEInformationResponse_vbxyext_IEs_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions_191_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_nonCriticalExtensions_191(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_nonCriticalExtensions_191, T_nonCriticalExtensions_191_sequence);

  return offset;
}


static const per_sequence_t T_vbxyNonCriticalExtensions_17_sequence[] = {
  { &hf_rrc_ueInformationResponse_vbxyext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UEInformationResponse_vbxyext_IEs },
  { &hf_rrc_nonCriticalExtensions_191, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_nonCriticalExtensions_191 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_vbxyNonCriticalExtensions_17(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_vbxyNonCriticalExtensions_17, T_vbxyNonCriticalExtensions_17_sequence);

  return offset;
}


static const per_sequence_t UEInformationResponse_sequence[] = {
  { &hf_rrc_rrc_TransactionIdentifier, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RRC_TransactionIdentifier },
  { &hf_rrc_loggedMeasReport, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_LoggedMeasReport },
  { &hf_rrc_loggedANRReportInfoList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_LoggedANRReportInfoList },
  { &hf_rrc_vbxyNonCriticalExtensions_17, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_vbxyNonCriticalExtensions_17 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UEInformationResponse(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
#line 471 "../../asn1/rrc/rrc.cnf"
  col_append_str(actx->pinfo->cinfo, COL_INFO, "UEInformationResponse");
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UEInformationResponse, UEInformationResponse_sequence);




  return offset;
}


static const value_string rrc_UL_DCCH_MessageType_ext_vals[] = {
  {   0, "ueInformationResponse" },
  {   1, "spare15" },
  {   2, "spare14" },
  {   3, "spare13" },
  {   4, "spare12" },
  {   5, "spare11" },
  {   6, "spare10" },
  {   7, "spare9" },
  {   8, "spare8" },
  {   9, "spare7" },
  {  10, "spare6" },
  {  11, "spare5" },
  {  12, "spare4" },
  {  13, "spare3" },
  {  14, "spare2" },
  {  15, "spare1" },
  { 0, NULL }
};

static const per_choice_t UL_DCCH_MessageType_ext_choice[] = {
  {   0, &hf_rrc_ueInformationResponse, ASN1_NO_EXTENSIONS     , dissect_rrc_UEInformationResponse },
  {   1, &hf_rrc_spare15         , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   2, &hf_rrc_spare14         , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   3, &hf_rrc_spare13         , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   4, &hf_rrc_spare12         , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   5, &hf_rrc_spare11         , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   6, &hf_rrc_spare10         , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   7, &hf_rrc_spare9          , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   8, &hf_rrc_spare8          , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   9, &hf_rrc_spare7          , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {  10, &hf_rrc_spare6          , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {  11, &hf_rrc_spare5          , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {  12, &hf_rrc_spare4          , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {  13, &hf_rrc_spare3          , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {  14, &hf_rrc_spare2          , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {  15, &hf_rrc_spare1          , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_UL_DCCH_MessageType_ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_UL_DCCH_MessageType_ext, UL_DCCH_MessageType_ext_choice,
                                 NULL);

  return offset;
}


static const value_string rrc_UL_DCCH_MessageType_vals[] = {
  {   0, "activeSetUpdateComplete" },
  {   1, "activeSetUpdateFailure" },
  {   2, "cellChangeOrderFromUTRANFailure" },
  {   3, "counterCheckResponse" },
  {   4, "handoverToUTRANComplete" },
  {   5, "initialDirectTransfer" },
  {   6, "handoverFromUTRANFailure" },
  {   7, "measurementControlFailure" },
  {   8, "measurementReport" },
  {   9, "physicalChannelReconfigurationComplete" },
  {  10, "physicalChannelReconfigurationFailure" },
  {  11, "radioBearerReconfigurationComplete" },
  {  12, "radioBearerReconfigurationFailure" },
  {  13, "radioBearerReleaseComplete" },
  {  14, "radioBearerReleaseFailure" },
  {  15, "radioBearerSetupComplete" },
  {  16, "radioBearerSetupFailure" },
  {  17, "rrcConnectionReleaseComplete" },
  {  18, "rrcConnectionSetupComplete" },
  {  19, "rrcStatus" },
  {  20, "securityModeComplete" },
  {  21, "securityModeFailure" },
  {  22, "signallingConnectionReleaseIndication" },
  {  23, "transportChannelReconfigurationComplete" },
  {  24, "transportChannelReconfigurationFailure" },
  {  25, "transportFormatCombinationControlFailure" },
  {  26, "ueCapabilityInformation" },
  {  27, "uplinkDirectTransfer" },
  {  28, "utranMobilityInformationConfirm" },
  {  29, "utranMobilityInformationFailure" },
  {  30, "mbmsModificationRequest" },
  {  31, "ul-DCCH-MessageType-ext" },
  { 0, NULL }
};

static value_string_ext rrc_UL_DCCH_MessageType_vals_ext = VALUE_STRING_EXT_INIT(rrc_UL_DCCH_MessageType_vals);

static const per_choice_t UL_DCCH_MessageType_choice[] = {
  {   0, &hf_rrc_activeSetUpdateComplete, ASN1_NO_EXTENSIONS     , dissect_rrc_ActiveSetUpdateComplete },
  {   1, &hf_rrc_activeSetUpdateFailure, ASN1_NO_EXTENSIONS     , dissect_rrc_ActiveSetUpdateFailure },
  {   2, &hf_rrc_cellChangeOrderFromUTRANFailure, ASN1_NO_EXTENSIONS     , dissect_rrc_CellChangeOrderFromUTRANFailure },
  {   3, &hf_rrc_counterCheckResponse, ASN1_NO_EXTENSIONS     , dissect_rrc_CounterCheckResponse },
  {   4, &hf_rrc_handoverToUTRANComplete, ASN1_NO_EXTENSIONS     , dissect_rrc_HandoverToUTRANComplete },
  {   5, &hf_rrc_initialDirectTransfer, ASN1_NO_EXTENSIONS     , dissect_rrc_InitialDirectTransfer },
  {   6, &hf_rrc_handoverFromUTRANFailure, ASN1_NO_EXTENSIONS     , dissect_rrc_HandoverFromUTRANFailure },
  {   7, &hf_rrc_measurementControlFailure, ASN1_NO_EXTENSIONS     , dissect_rrc_MeasurementControlFailure },
  {   8, &hf_rrc_measurementReport, ASN1_NO_EXTENSIONS     , dissect_rrc_MeasurementReport },
  {   9, &hf_rrc_physicalChannelReconfigurationComplete, ASN1_NO_EXTENSIONS     , dissect_rrc_PhysicalChannelReconfigurationComplete },
  {  10, &hf_rrc_physicalChannelReconfigurationFailure, ASN1_NO_EXTENSIONS     , dissect_rrc_PhysicalChannelReconfigurationFailure },
  {  11, &hf_rrc_radioBearerReconfigurationComplete, ASN1_NO_EXTENSIONS     , dissect_rrc_RadioBearerReconfigurationComplete },
  {  12, &hf_rrc_radioBearerReconfigurationFailure, ASN1_NO_EXTENSIONS     , dissect_rrc_RadioBearerReconfigurationFailure },
  {  13, &hf_rrc_radioBearerReleaseComplete, ASN1_NO_EXTENSIONS     , dissect_rrc_RadioBearerReleaseComplete },
  {  14, &hf_rrc_radioBearerReleaseFailure, ASN1_NO_EXTENSIONS     , dissect_rrc_RadioBearerReleaseFailure },
  {  15, &hf_rrc_radioBearerSetupComplete, ASN1_NO_EXTENSIONS     , dissect_rrc_RadioBearerSetupComplete },
  {  16, &hf_rrc_radioBearerSetupFailure, ASN1_NO_EXTENSIONS     , dissect_rrc_RadioBearerSetupFailure },
  {  17, &hf_rrc_rrcConnectionReleaseComplete, ASN1_NO_EXTENSIONS     , dissect_rrc_RRCConnectionReleaseComplete },
  {  18, &hf_rrc_rrcConnectionSetupComplete, ASN1_NO_EXTENSIONS     , dissect_rrc_RRCConnectionSetupComplete },
  {  19, &hf_rrc_rrcStatus       , ASN1_NO_EXTENSIONS     , dissect_rrc_RRCStatus },
  {  20, &hf_rrc_securityModeComplete, ASN1_NO_EXTENSIONS     , dissect_rrc_SecurityModeComplete },
  {  21, &hf_rrc_securityModeFailure, ASN1_NO_EXTENSIONS     , dissect_rrc_SecurityModeFailure },
  {  22, &hf_rrc_signallingConnectionReleaseIndication, ASN1_NO_EXTENSIONS     , dissect_rrc_SignallingConnectionReleaseIndication },
  {  23, &hf_rrc_transportChannelReconfigurationComplete, ASN1_NO_EXTENSIONS     , dissect_rrc_TransportChannelReconfigurationComplete },
  {  24, &hf_rrc_transportChannelReconfigurationFailure, ASN1_NO_EXTENSIONS     , dissect_rrc_TransportChannelReconfigurationFailure },
  {  25, &hf_rrc_transportFormatCombinationControlFailure, ASN1_NO_EXTENSIONS     , dissect_rrc_TransportFormatCombinationControlFailure },
  {  26, &hf_rrc_ueCapabilityInformation, ASN1_NO_EXTENSIONS     , dissect_rrc_UECapabilityInformation },
  {  27, &hf_rrc_uplinkDirectTransfer, ASN1_NO_EXTENSIONS     , dissect_rrc_UplinkDirectTransfer },
  {  28, &hf_rrc_utranMobilityInformationConfirm, ASN1_NO_EXTENSIONS     , dissect_rrc_UTRANMobilityInformationConfirm },
  {  29, &hf_rrc_utranMobilityInformationFailure, ASN1_NO_EXTENSIONS     , dissect_rrc_UTRANMobilityInformationFailure },
  {  30, &hf_rrc_mbmsModificationRequest, ASN1_NO_EXTENSIONS     , dissect_rrc_MBMSModificationRequest },
  {  31, &hf_rrc_ul_DCCH_MessageType_ext, ASN1_NO_EXTENSIONS     , dissect_rrc_UL_DCCH_MessageType_ext },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_UL_DCCH_MessageType(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_UL_DCCH_MessageType, UL_DCCH_MessageType_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t UL_DCCH_Message_sequence[] = {
  { &hf_rrc_integrityCheckInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_IntegrityCheckInfo },
  { &hf_rrc_message_01      , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UL_DCCH_MessageType },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UL_DCCH_Message(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UL_DCCH_Message, UL_DCCH_Message_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions_24_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_nonCriticalExtensions_24(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_nonCriticalExtensions_24, T_nonCriticalExtensions_24_sequence);

  return offset;
}


static const per_sequence_t T_v690NonCriticalExtensions_05_sequence[] = {
  { &hf_rrc_cellUpdateConfirm_v690ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CellUpdateConfirm_v690ext_IEs },
  { &hf_rrc_nonCriticalExtensions_24, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_nonCriticalExtensions_24 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v690NonCriticalExtensions_05(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v690NonCriticalExtensions_05, T_v690NonCriticalExtensions_05_sequence);

  return offset;
}


static const per_sequence_t T_v5d0NonCriticalExtenstions_03_sequence[] = {
  { &hf_rrc_cellUpdateConfirm_v5d0ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CellUpdateConfirm_v5d0ext_IEs },
  { &hf_rrc_v690NonCriticalExtensions_05, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v690NonCriticalExtensions_05 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v5d0NonCriticalExtenstions_03(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v5d0NonCriticalExtenstions_03, T_v5d0NonCriticalExtenstions_03_sequence);

  return offset;
}


static const per_sequence_t T_v590NonCriticalExtensions_03_sequence[] = {
  { &hf_rrc_cellUpdateConfirm_v590ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CellUpdateConfirm_v590ext_IEs },
  { &hf_rrc_v5d0NonCriticalExtenstions_03, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v5d0NonCriticalExtenstions_03 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v590NonCriticalExtensions_03(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v590NonCriticalExtensions_03, T_v590NonCriticalExtensions_03_sequence);

  return offset;
}


static const per_sequence_t T_v4b0NonCriticalExtensions_03_sequence[] = {
  { &hf_rrc_cellUpdateConfirm_v4b0ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CellUpdateConfirm_v4b0ext_IEs },
  { &hf_rrc_v590NonCriticalExtensions_03, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v590NonCriticalExtensions_03 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v4b0NonCriticalExtensions_03(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v4b0NonCriticalExtensions_03, T_v4b0NonCriticalExtensions_03_sequence);

  return offset;
}


static const per_sequence_t T_laterNonCriticalExtensions_08_sequence[] = {
  { &hf_rrc_cellUpdateConfirm_CCCH_r3_add_ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_BIT_STRING },
  { &hf_rrc_v4b0NonCriticalExtensions_03, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v4b0NonCriticalExtensions_03 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_laterNonCriticalExtensions_08(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_laterNonCriticalExtensions_08, T_laterNonCriticalExtensions_08_sequence);

  return offset;
}


static const per_sequence_t T_r3_05_sequence[] = {
  { &hf_rrc_u_RNTI          , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_U_RNTI },
  { &hf_rrc_cellUpdateConfirm_r3, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CellUpdateConfirm_r3_IEs },
  { &hf_rrc_laterNonCriticalExtensions_08, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_laterNonCriticalExtensions_08 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_r3_05(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_r3_05, T_r3_05_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions_25_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_nonCriticalExtensions_25(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_nonCriticalExtensions_25, T_nonCriticalExtensions_25_sequence);

  return offset;
}


static const per_sequence_t T_v690NonCriticalExtensions_06_sequence[] = {
  { &hf_rrc_cellUpdateConfirm_v690ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CellUpdateConfirm_v690ext_IEs },
  { &hf_rrc_nonCriticalExtensions_25, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_nonCriticalExtensions_25 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v690NonCriticalExtensions_06(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v690NonCriticalExtensions_06, T_v690NonCriticalExtensions_06_sequence);

  return offset;
}


static const per_sequence_t T_v5d0NonCriticalExtenstions_04_sequence[] = {
  { &hf_rrc_cellUpdateConfirm_v5d0ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CellUpdateConfirm_v5d0ext_IEs },
  { &hf_rrc_v690NonCriticalExtensions_06, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v690NonCriticalExtensions_06 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v5d0NonCriticalExtenstions_04(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v5d0NonCriticalExtenstions_04, T_v5d0NonCriticalExtenstions_04_sequence);

  return offset;
}


static const per_sequence_t T_v590NonCriticalExtensions_04_sequence[] = {
  { &hf_rrc_cellUpdateConfirm_v590ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CellUpdateConfirm_v590ext_IEs },
  { &hf_rrc_v5d0NonCriticalExtenstions_04, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v5d0NonCriticalExtenstions_04 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v590NonCriticalExtensions_04(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v590NonCriticalExtensions_04, T_v590NonCriticalExtensions_04_sequence);

  return offset;
}


static const per_sequence_t T_v4d0NonCriticalExtensions_01_sequence[] = {
  { &hf_rrc_cellUpdateConfirm_CCCH_r4_add_ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_BIT_STRING },
  { &hf_rrc_v590NonCriticalExtensions_04, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v590NonCriticalExtensions_04 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v4d0NonCriticalExtensions_01(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v4d0NonCriticalExtensions_01, T_v4d0NonCriticalExtensions_01_sequence);

  return offset;
}


static const per_sequence_t T_r4_01_sequence[] = {
  { &hf_rrc_cellUpdateConfirm_r4, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CellUpdateConfirm_r4_IEs },
  { &hf_rrc_v4d0NonCriticalExtensions_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v4d0NonCriticalExtensions_01 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_r4_01(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_r4_01, T_r4_01_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions_26_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_nonCriticalExtensions_26(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_nonCriticalExtensions_26, T_nonCriticalExtensions_26_sequence);

  return offset;
}


static const per_sequence_t T_v690NonCriticalExtensions_07_sequence[] = {
  { &hf_rrc_cellUpdateConfirm_v690ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CellUpdateConfirm_v690ext_IEs },
  { &hf_rrc_nonCriticalExtensions_26, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_nonCriticalExtensions_26 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v690NonCriticalExtensions_07(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v690NonCriticalExtensions_07, T_v690NonCriticalExtensions_07_sequence);

  return offset;
}


static const per_sequence_t T_v5d0NonCriticalExtenstions_05_sequence[] = {
  { &hf_rrc_cellUpdateConfirm_v5d0ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CellUpdateConfirm_v5d0ext_IEs },
  { &hf_rrc_v690NonCriticalExtensions_07, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v690NonCriticalExtensions_07 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v5d0NonCriticalExtenstions_05(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v5d0NonCriticalExtenstions_05, T_v5d0NonCriticalExtenstions_05_sequence);

  return offset;
}


static const per_sequence_t T_r5_01_sequence[] = {
  { &hf_rrc_cellUpdateConfirm_r5, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CellUpdateConfirm_r5_IEs },
  { &hf_rrc_cellUpdateConfirm_CCCH_r5_add_ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_BIT_STRING },
  { &hf_rrc_v5d0NonCriticalExtenstions_05, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v5d0NonCriticalExtenstions_05 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_r5_01(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_r5_01, T_r5_01_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions_27_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_nonCriticalExtensions_27(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_nonCriticalExtensions_27, T_nonCriticalExtensions_27_sequence);

  return offset;
}


static const per_sequence_t T_v6b0NonCriticalExtensions_03_sequence[] = {
  { &hf_rrc_cellUpdateConfirm_v6b0ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CellUpdateConfirm_v6b0ext_IEs },
  { &hf_rrc_nonCriticalExtensions_27, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_nonCriticalExtensions_27 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v6b0NonCriticalExtensions_03(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v6b0NonCriticalExtensions_03, T_v6b0NonCriticalExtensions_03_sequence);

  return offset;
}


static const per_sequence_t T_r6_02_sequence[] = {
  { &hf_rrc_cellUpdateConfirm_r6, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CellUpdateConfirm_r6_IEs },
  { &hf_rrc_cellUpdateConfirm_r6_add_ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_BIT_STRING },
  { &hf_rrc_v6b0NonCriticalExtensions_03, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v6b0NonCriticalExtensions_03 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_r6_02(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_r6_02, T_r6_02_sequence);

  return offset;
}



static int
dissect_rrc_T_cellUpdateConfirm_r7_add_ext_01(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_bit_string_containing_pdu_new(tvb, offset, actx, tree, hf_index,
                                                              NO_BOUND, NO_BOUND, FALSE, dissect_CellUpdateConfirm_r7_add_ext_IEs_PDU);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions_28_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_nonCriticalExtensions_28(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_nonCriticalExtensions_28, T_nonCriticalExtensions_28_sequence);

  return offset;
}


static const per_sequence_t T_v860NonCriticalExtensions_02_sequence[] = {
  { &hf_rrc_cellUpdateConfirm_v860ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CellUpdateConfirm_v860ext_IEs },
  { &hf_rrc_nonCriticalExtensions_28, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_nonCriticalExtensions_28 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v860NonCriticalExtensions_02(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v860NonCriticalExtensions_02, T_v860NonCriticalExtensions_02_sequence);

  return offset;
}


static const per_sequence_t T_v780NonCriticalExtensions_02_sequence[] = {
  { &hf_rrc_cellUpdateConfirm_v780ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CellUpdateConfirm_v780ext_IEs },
  { &hf_rrc_v860NonCriticalExtensions_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v860NonCriticalExtensions_02 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v780NonCriticalExtensions_02(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v780NonCriticalExtensions_02, T_v780NonCriticalExtensions_02_sequence);

  return offset;
}


static const per_sequence_t T_r7_02_sequence[] = {
  { &hf_rrc_cellUpdateConfirm_r7, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CellUpdateConfirm_r7_IEs },
  { &hf_rrc_cellUpdateConfirm_r7_add_ext_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_cellUpdateConfirm_r7_add_ext_01 },
  { &hf_rrc_v780NonCriticalExtensions_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v780NonCriticalExtensions_02 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_r7_02(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_r7_02, T_r7_02_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions_29_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_nonCriticalExtensions_29(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_nonCriticalExtensions_29, T_nonCriticalExtensions_29_sequence);

  return offset;
}


static const per_sequence_t T_v7g0NonCriticalExtensions_05_sequence[] = {
  { &hf_rrc_cellUpdateConfirm_v7g0ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CellUpdateConfirm_v7g0ext_IEs },
  { &hf_rrc_nonCriticalExtensions_29, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_nonCriticalExtensions_29 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v7g0NonCriticalExtensions_05(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v7g0NonCriticalExtensions_05, T_v7g0NonCriticalExtensions_05_sequence);

  return offset;
}


static const per_sequence_t T_v7f0NonCriticalExtensions_03_sequence[] = {
  { &hf_rrc_cellUpdateConfirm_v7f0ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CellUpdateConfirm_v7f0ext_IEs },
  { &hf_rrc_v7g0NonCriticalExtensions_05, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v7g0NonCriticalExtensions_05 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v7f0NonCriticalExtensions_03(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v7f0NonCriticalExtensions_03, T_v7f0NonCriticalExtensions_03_sequence);

  return offset;
}


static const per_sequence_t T_v8a0NonCriticalExtensions_01_sequence[] = {
  { &hf_rrc_cellUpdateConfirm_v8a0ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CellUpdateConfirm_v8a0ext_IEs },
  { &hf_rrc_v7f0NonCriticalExtensions_03, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v7f0NonCriticalExtensions_03 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v8a0NonCriticalExtensions_01(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v8a0NonCriticalExtensions_01, T_v8a0NonCriticalExtensions_01_sequence);

  return offset;
}


static const per_sequence_t T_v890NonCriticalExtensions_02_sequence[] = {
  { &hf_rrc_cellUpdateConfirm_v890ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CellUpdateConfirm_v890ext_IEs },
  { &hf_rrc_v8a0NonCriticalExtensions_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v8a0NonCriticalExtensions_01 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v890NonCriticalExtensions_02(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v890NonCriticalExtensions_02, T_v890NonCriticalExtensions_02_sequence);

  return offset;
}


static const per_sequence_t T_v7d0NonCriticalExtensions_01_sequence[] = {
  { &hf_rrc_cellUpdateConfirm_v7d0ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CellUpdateConfirm_v7d0ext_IEs },
  { &hf_rrc_v890NonCriticalExtensions_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v890NonCriticalExtensions_02 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v7d0NonCriticalExtensions_01(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v7d0NonCriticalExtensions_01, T_v7d0NonCriticalExtensions_01_sequence);

  return offset;
}


static const per_sequence_t T_r8_02_sequence[] = {
  { &hf_rrc_cellUpdateConfirm_r8, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CellUpdateConfirm_r8_IEs },
  { &hf_rrc_cellUpdateConfirm_r8_add_ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_BIT_STRING },
  { &hf_rrc_v7d0NonCriticalExtensions_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v7d0NonCriticalExtensions_01 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_r8_02(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_r8_02, T_r8_02_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions_30_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_nonCriticalExtensions_30(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_nonCriticalExtensions_30, T_nonCriticalExtensions_30_sequence);

  return offset;
}


static const per_sequence_t T_r9_02_sequence[] = {
  { &hf_rrc_cellUpdateConfirm_r9, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CellUpdateConfirm_r9_IEs },
  { &hf_rrc_cellUpdateConfirm_r9_add_ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_BIT_STRING },
  { &hf_rrc_nonCriticalExtensions_30, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_nonCriticalExtensions_30 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_r9_02(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_r9_02, T_r9_02_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions_31_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_nonCriticalExtensions_31(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_nonCriticalExtensions_31, T_nonCriticalExtensions_31_sequence);

  return offset;
}


static const per_sequence_t T_vbxyNonCriticalExtensions_03_sequence[] = {
  { &hf_rrc_cellUpdateConfirm_vbxyext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CellUpdateConfirm_vbxyext_IEs },
  { &hf_rrc_nonCriticalExtensions_31, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_nonCriticalExtensions_31 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_vbxyNonCriticalExtensions_03(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_vbxyNonCriticalExtensions_03, T_vbxyNonCriticalExtensions_03_sequence);

  return offset;
}


static const per_sequence_t T_r10_02_sequence[] = {
  { &hf_rrc_cellUpdateConfirm_r10, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CellUpdateConfirm_r10_IEs },
  { &hf_rrc_cellUpdateConfirm_r10_add_ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_BIT_STRING },
  { &hf_rrc_vbxyNonCriticalExtensions_03, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_vbxyNonCriticalExtensions_03 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_r10_02(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_r10_02, T_r10_02_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions_32_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_nonCriticalExtensions_32(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_nonCriticalExtensions_32, T_nonCriticalExtensions_32_sequence);

  return offset;
}


static const per_sequence_t T_r11_02_sequence[] = {
  { &hf_rrc_cellUpdateConfirm_r11, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CellUpdateConfirm_r11_IEs },
  { &hf_rrc_cellUpdateConfirm_r11_add_ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_BIT_STRING },
  { &hf_rrc_nonCriticalExtensions_32, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_nonCriticalExtensions_32 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_r11_02(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_r11_02, T_r11_02_sequence);

  return offset;
}


static const per_sequence_t T_criticalExtensions_27_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_criticalExtensions_27(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_criticalExtensions_27, T_criticalExtensions_27_sequence);

  return offset;
}


static const value_string rrc_T_criticalExtensions_26_vals[] = {
  {   0, "r11" },
  {   1, "criticalExtensions" },
  { 0, NULL }
};

static const per_choice_t T_criticalExtensions_26_choice[] = {
  {   0, &hf_rrc_r11_02          , ASN1_NO_EXTENSIONS     , dissect_rrc_T_r11_02 },
  {   1, &hf_rrc_criticalExtensions_27, ASN1_NO_EXTENSIONS     , dissect_rrc_T_criticalExtensions_27 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_criticalExtensions_26(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_criticalExtensions_26, T_criticalExtensions_26_choice,
                                 NULL);

  return offset;
}


static const value_string rrc_T_criticalExtensions_25_vals[] = {
  {   0, "r10" },
  {   1, "criticalExtensions" },
  { 0, NULL }
};

static const per_choice_t T_criticalExtensions_25_choice[] = {
  {   0, &hf_rrc_r10_02          , ASN1_NO_EXTENSIONS     , dissect_rrc_T_r10_02 },
  {   1, &hf_rrc_criticalExtensions_26, ASN1_NO_EXTENSIONS     , dissect_rrc_T_criticalExtensions_26 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_criticalExtensions_25(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_criticalExtensions_25, T_criticalExtensions_25_choice,
                                 NULL);

  return offset;
}


static const value_string rrc_T_criticalExtensions_24_vals[] = {
  {   0, "r9" },
  {   1, "criticalExtensions" },
  { 0, NULL }
};

static const per_choice_t T_criticalExtensions_24_choice[] = {
  {   0, &hf_rrc_r9_02           , ASN1_NO_EXTENSIONS     , dissect_rrc_T_r9_02 },
  {   1, &hf_rrc_criticalExtensions_25, ASN1_NO_EXTENSIONS     , dissect_rrc_T_criticalExtensions_25 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_criticalExtensions_24(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_criticalExtensions_24, T_criticalExtensions_24_choice,
                                 NULL);

  return offset;
}


static const value_string rrc_T_criticalExtensions_23_vals[] = {
  {   0, "r8" },
  {   1, "criticalExtensions" },
  { 0, NULL }
};

static const per_choice_t T_criticalExtensions_23_choice[] = {
  {   0, &hf_rrc_r8_02           , ASN1_NO_EXTENSIONS     , dissect_rrc_T_r8_02 },
  {   1, &hf_rrc_criticalExtensions_24, ASN1_NO_EXTENSIONS     , dissect_rrc_T_criticalExtensions_24 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_criticalExtensions_23(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_criticalExtensions_23, T_criticalExtensions_23_choice,
                                 NULL);

  return offset;
}


static const value_string rrc_T_criticalExtensions_22_vals[] = {
  {   0, "r7" },
  {   1, "criticalExtensions" },
  { 0, NULL }
};

static const per_choice_t T_criticalExtensions_22_choice[] = {
  {   0, &hf_rrc_r7_02           , ASN1_NO_EXTENSIONS     , dissect_rrc_T_r7_02 },
  {   1, &hf_rrc_criticalExtensions_23, ASN1_NO_EXTENSIONS     , dissect_rrc_T_criticalExtensions_23 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_criticalExtensions_22(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_criticalExtensions_22, T_criticalExtensions_22_choice,
                                 NULL);

  return offset;
}


static const value_string rrc_T_criticalExtensions_21_vals[] = {
  {   0, "r6" },
  {   1, "criticalExtensions" },
  { 0, NULL }
};

static const per_choice_t T_criticalExtensions_21_choice[] = {
  {   0, &hf_rrc_r6_02           , ASN1_NO_EXTENSIONS     , dissect_rrc_T_r6_02 },
  {   1, &hf_rrc_criticalExtensions_22, ASN1_NO_EXTENSIONS     , dissect_rrc_T_criticalExtensions_22 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_criticalExtensions_21(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_criticalExtensions_21, T_criticalExtensions_21_choice,
                                 NULL);

  return offset;
}


static const value_string rrc_T_criticalExtensions_20_vals[] = {
  {   0, "r5" },
  {   1, "criticalExtensions" },
  { 0, NULL }
};

static const per_choice_t T_criticalExtensions_20_choice[] = {
  {   0, &hf_rrc_r5_01           , ASN1_NO_EXTENSIONS     , dissect_rrc_T_r5_01 },
  {   1, &hf_rrc_criticalExtensions_21, ASN1_NO_EXTENSIONS     , dissect_rrc_T_criticalExtensions_21 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_criticalExtensions_20(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_criticalExtensions_20, T_criticalExtensions_20_choice,
                                 NULL);

  return offset;
}


static const value_string rrc_T_criticalExtensions_19_vals[] = {
  {   0, "r4" },
  {   1, "criticalExtensions" },
  { 0, NULL }
};

static const per_choice_t T_criticalExtensions_19_choice[] = {
  {   0, &hf_rrc_r4_01           , ASN1_NO_EXTENSIONS     , dissect_rrc_T_r4_01 },
  {   1, &hf_rrc_criticalExtensions_20, ASN1_NO_EXTENSIONS     , dissect_rrc_T_criticalExtensions_20 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_criticalExtensions_19(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_criticalExtensions_19, T_criticalExtensions_19_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t T_later_than_r3_04_sequence[] = {
  { &hf_rrc_u_RNTI          , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_U_RNTI },
  { &hf_rrc_rrc_TransactionIdentifier, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RRC_TransactionIdentifier },
  { &hf_rrc_criticalExtensions_19, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_criticalExtensions_19 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_later_than_r3_04(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_later_than_r3_04, T_later_than_r3_04_sequence);

  return offset;
}


static const value_string rrc_CellUpdateConfirm_CCCH_vals[] = {
  {   0, "r3" },
  {   1, "later-than-r3" },
  { 0, NULL }
};

static const per_choice_t CellUpdateConfirm_CCCH_choice[] = {
  {   0, &hf_rrc_r3_05           , ASN1_NO_EXTENSIONS     , dissect_rrc_T_r3_05 },
  {   1, &hf_rrc_later_than_r3_04, ASN1_NO_EXTENSIONS     , dissect_rrc_T_later_than_r3_04 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_CellUpdateConfirm_CCCH(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
#line 193 "../../asn1/rrc/rrc.cnf"
  col_append_str(actx->pinfo->cinfo, COL_INFO, "CellUpdateConfirm-CCCH");
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_CellUpdateConfirm_CCCH, CellUpdateConfirm_CCCH_choice,
                                 NULL);




  return offset;
}


static const per_sequence_t IMSI_GSM_MAP_sequence_of[1] = {
  { &hf_rrc_IMSI_GSM_MAP_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_Digit },
};

static int
dissect_rrc_IMSI_GSM_MAP(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_IMSI_GSM_MAP, IMSI_GSM_MAP_sequence_of,
                                                  6, 21, FALSE);

  return offset;
}



static int
dissect_rrc_TMSI_GSM_MAP(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_bit_string(tvb, offset, actx, tree, hf_index,
                                     32, 32, FALSE, NULL);

  return offset;
}


static const per_sequence_t TMSI_and_LAI_GSM_MAP_sequence[] = {
  { &hf_rrc_tmsi            , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TMSI_GSM_MAP },
  { &hf_rrc_lai             , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_LAI },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_TMSI_and_LAI_GSM_MAP(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_TMSI_and_LAI_GSM_MAP, TMSI_and_LAI_GSM_MAP_sequence);

  return offset;
}


static const per_sequence_t P_TMSI_and_RAI_GSM_MAP_sequence[] = {
  { &hf_rrc_p_TMSI          , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_P_TMSI_GSM_MAP },
  { &hf_rrc_rai             , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RAI },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_P_TMSI_and_RAI_GSM_MAP(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_P_TMSI_and_RAI_GSM_MAP, P_TMSI_and_RAI_GSM_MAP_sequence);

  return offset;
}



static int
dissect_rrc_IMEI_Digit(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            0U, 15U, NULL, FALSE);

  return offset;
}


static const per_sequence_t IMEI_sequence_of[1] = {
  { &hf_rrc_IMEI_item       , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_IMEI_Digit },
};

static int
dissect_rrc_IMEI(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_IMEI, IMEI_sequence_of,
                                                  15, 15, FALSE);

  return offset;
}



static int
dissect_rrc_ESN_DS_41(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_bit_string(tvb, offset, actx, tree, hf_index,
                                     32, 32, FALSE, NULL);

  return offset;
}



static int
dissect_rrc_IMSI_DS_41(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_octet_string(tvb, offset, actx, tree, hf_index,
                                       5, 7, FALSE, NULL);

  return offset;
}


static const per_sequence_t IMSI_and_ESN_DS_41_sequence[] = {
  { &hf_rrc_imsi_DS_41      , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_IMSI_DS_41 },
  { &hf_rrc_esn_DS_41       , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_ESN_DS_41 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_IMSI_and_ESN_DS_41(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_IMSI_and_ESN_DS_41, IMSI_and_ESN_DS_41_sequence);

  return offset;
}



static int
dissect_rrc_TMSI_DS_41(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_octet_string(tvb, offset, actx, tree, hf_index,
                                       2, 17, FALSE, NULL);

  return offset;
}


static const value_string rrc_InitialUE_Identity_vals[] = {
  {   0, "imsi" },
  {   1, "tmsi-and-LAI" },
  {   2, "p-TMSI-and-RAI" },
  {   3, "imei" },
  {   4, "esn-DS-41" },
  {   5, "imsi-DS-41" },
  {   6, "imsi-and-ESN-DS-41" },
  {   7, "tmsi-DS-41" },
  { 0, NULL }
};

static const per_choice_t InitialUE_Identity_choice[] = {
  {   0, &hf_rrc_imsi            , ASN1_NO_EXTENSIONS     , dissect_rrc_IMSI_GSM_MAP },
  {   1, &hf_rrc_tmsi_and_LAI    , ASN1_NO_EXTENSIONS     , dissect_rrc_TMSI_and_LAI_GSM_MAP },
  {   2, &hf_rrc_p_TMSI_and_RAI  , ASN1_NO_EXTENSIONS     , dissect_rrc_P_TMSI_and_RAI_GSM_MAP },
  {   3, &hf_rrc_imei            , ASN1_NO_EXTENSIONS     , dissect_rrc_IMEI },
  {   4, &hf_rrc_esn_DS_41       , ASN1_NO_EXTENSIONS     , dissect_rrc_ESN_DS_41 },
  {   5, &hf_rrc_imsi_DS_41      , ASN1_NO_EXTENSIONS     , dissect_rrc_IMSI_DS_41 },
  {   6, &hf_rrc_imsi_and_ESN_DS_41, ASN1_NO_EXTENSIONS     , dissect_rrc_IMSI_and_ESN_DS_41 },
  {   7, &hf_rrc_tmsi_DS_41      , ASN1_NO_EXTENSIONS     , dissect_rrc_TMSI_DS_41 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_InitialUE_Identity(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_InitialUE_Identity, InitialUE_Identity_choice,
                                 NULL);

  return offset;
}


static const value_string rrc_RejectionCause_vals[] = {
  {   0, "congestion" },
  {   1, "unspecified" },
  { 0, NULL }
};


static int
dissect_rrc_RejectionCause(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     2, NULL, FALSE, 0, NULL);

  return offset;
}


static const value_string rrc_RedirectionInfo_vals[] = {
  {   0, "frequencyInfo" },
  {   1, "interRATInfo" },
  { 0, NULL }
};

static const per_choice_t RedirectionInfo_choice[] = {
  {   0, &hf_rrc_frequencyInfo   , ASN1_NO_EXTENSIONS     , dissect_rrc_FrequencyInfo },
  {   1, &hf_rrc_interRATInfo    , ASN1_NO_EXTENSIONS     , dissect_rrc_InterRATInfo },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_RedirectionInfo(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_RedirectionInfo, RedirectionInfo_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t RRCConnectionReject_r3_IEs_sequence[] = {
  { &hf_rrc_initialUE_Identity, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_InitialUE_Identity },
  { &hf_rrc_rrc_TransactionIdentifier, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RRC_TransactionIdentifier },
  { &hf_rrc_rejectionCause  , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RejectionCause },
  { &hf_rrc_waitTime        , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_WaitTime },
  { &hf_rrc_redirectionInfo , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RedirectionInfo },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_RRCConnectionReject_r3_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_RRCConnectionReject_r3_IEs, RRCConnectionReject_r3_IEs_sequence);

  return offset;
}


static const per_sequence_t RRCConnectionReject_v690ext_IEs_sequence[] = {
  { &hf_rrc_redirectionInfo_v690ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_GSM_TargetCellInfoList },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_RRCConnectionReject_v690ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_RRCConnectionReject_v690ext_IEs, RRCConnectionReject_v690ext_IEs_sequence);

  return offset;
}


static const value_string rrc_T_countingCompletion_vals[] = {
  {   0, "true" },
  { 0, NULL }
};


static int
dissect_rrc_T_countingCompletion(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t RRCConnectionReject_v6f0ext_IEs_sequence[] = {
  { &hf_rrc_countingCompletion, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_countingCompletion },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_RRCConnectionReject_v6f0ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_RRCConnectionReject_v6f0ext_IEs, RRCConnectionReject_v6f0ext_IEs_sequence);

  return offset;
}


static const per_sequence_t RRCConnectionReject_v860ext_IEs_sequence[] = {
  { &hf_rrc_redirectionInfo_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RedirectionInfo_v860ext },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_RRCConnectionReject_v860ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_RRCConnectionReject_v860ext_IEs, RRCConnectionReject_v860ext_IEs_sequence);

  return offset;
}


static const per_sequence_t RRCConnectionReject_va40ext_IEs_sequence[] = {
  { &hf_rrc_extendedWaitTime, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_ExtendedWaitTime },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_RRCConnectionReject_va40ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_RRCConnectionReject_va40ext_IEs, RRCConnectionReject_va40ext_IEs_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions_118_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_nonCriticalExtensions_118(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_nonCriticalExtensions_118, T_nonCriticalExtensions_118_sequence);

  return offset;
}


static const per_sequence_t T_va40NonCriticalExtensions_05_sequence[] = {
  { &hf_rrc_rrcConnectionReject_va40ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RRCConnectionReject_va40ext_IEs },
  { &hf_rrc_nonCriticalExtensions_118, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_nonCriticalExtensions_118 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_va40NonCriticalExtensions_05(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_va40NonCriticalExtensions_05, T_va40NonCriticalExtensions_05_sequence);

  return offset;
}


static const per_sequence_t T_v860NonCriticalExtensions_10_sequence[] = {
  { &hf_rrc_rrcConnectionReject_v860ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RRCConnectionReject_v860ext_IEs },
  { &hf_rrc_va40NonCriticalExtensions_05, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_va40NonCriticalExtensions_05 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v860NonCriticalExtensions_10(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v860NonCriticalExtensions_10, T_v860NonCriticalExtensions_10_sequence);

  return offset;
}


static const per_sequence_t T_v6f0NonCriticalExtensions_01_sequence[] = {
  { &hf_rrc_rrcConnectionReject_v6f0ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RRCConnectionReject_v6f0ext_IEs },
  { &hf_rrc_v860NonCriticalExtensions_10, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v860NonCriticalExtensions_10 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v6f0NonCriticalExtensions_01(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v6f0NonCriticalExtensions_01, T_v6f0NonCriticalExtensions_01_sequence);

  return offset;
}


static const per_sequence_t T_v690NonCriticalExtensions_25_sequence[] = {
  { &hf_rrc_rrcConnectionReject_v690ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RRCConnectionReject_v690ext_IEs },
  { &hf_rrc_v6f0NonCriticalExtensions_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v6f0NonCriticalExtensions_01 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v690NonCriticalExtensions_25(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v690NonCriticalExtensions_25, T_v690NonCriticalExtensions_25_sequence);

  return offset;
}


static const per_sequence_t T_laterNonCriticalExtensions_37_sequence[] = {
  { &hf_rrc_rrcConnectionReject_r3_add_ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_BIT_STRING },
  { &hf_rrc_v690NonCriticalExtensions_25, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v690NonCriticalExtensions_25 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_laterNonCriticalExtensions_37(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_laterNonCriticalExtensions_37, T_laterNonCriticalExtensions_37_sequence);

  return offset;
}


static const per_sequence_t T_r3_17_sequence[] = {
  { &hf_rrc_rrcConnectionReject_r3, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RRCConnectionReject_r3_IEs },
  { &hf_rrc_laterNonCriticalExtensions_37, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_laterNonCriticalExtensions_37 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_r3_17(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_r3_17, T_r3_17_sequence);

  return offset;
}


static const per_sequence_t T_criticalExtensions_91_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_criticalExtensions_91(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_criticalExtensions_91, T_criticalExtensions_91_sequence);

  return offset;
}


static const per_sequence_t T_later_than_r3_15_sequence[] = {
  { &hf_rrc_initialUE_Identity, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_InitialUE_Identity },
  { &hf_rrc_rrc_TransactionIdentifier, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RRC_TransactionIdentifier },
  { &hf_rrc_criticalExtensions_92, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_criticalExtensions_91 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_later_than_r3_15(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_later_than_r3_15, T_later_than_r3_15_sequence);

  return offset;
}


static const value_string rrc_RRCConnectionReject_vals[] = {
  {   0, "r3" },
  {   1, "later-than-r3" },
  { 0, NULL }
};

static const per_choice_t RRCConnectionReject_choice[] = {
  {   0, &hf_rrc_r3_17           , ASN1_NO_EXTENSIONS     , dissect_rrc_T_r3_17 },
  {   1, &hf_rrc_later_than_r3_15, ASN1_NO_EXTENSIONS     , dissect_rrc_T_later_than_r3_15 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_RRCConnectionReject(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
#line 361 "../../asn1/rrc/rrc.cnf"
  col_append_str(actx->pinfo->cinfo, COL_INFO, "RRCConnectionReject");
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_RRCConnectionReject, RRCConnectionReject_choice,
                                 NULL);




  return offset;
}


static const per_sequence_t RRCConnectionRelease_CCCH_r3_IEs_sequence[] = {
  { &hf_rrc_u_RNTI          , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_U_RNTI },
  { &hf_rrc_rrcConnectionRelease_02, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RRCConnectionRelease_r3_IEs },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_RRCConnectionRelease_CCCH_r3_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_RRCConnectionRelease_CCCH_r3_IEs, RRCConnectionRelease_CCCH_r3_IEs_sequence);

  return offset;
}



static int
dissect_rrc_RRCConnectionRelease_CCCH_v690ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_rrc_RRCConnectionRelease_v690ext_IEs(tvb, offset, actx, tree, hf_index);

  return offset;
}



static int
dissect_rrc_RRCConnectionRelease_CCCH_v860ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_rrc_RRCConnectionRelease_v860ext_IEs(tvb, offset, actx, tree, hf_index);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions_121_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_nonCriticalExtensions_121(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_nonCriticalExtensions_121, T_nonCriticalExtensions_121_sequence);

  return offset;
}


static const per_sequence_t T_va40NonCriticalExtensions_08_sequence[] = {
  { &hf_rrc_rrcConnectionRelease_va40ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RRCConnectionRelease_va40ext_IEs },
  { &hf_rrc_nonCriticalExtensions_121, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_nonCriticalExtensions_121 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_va40NonCriticalExtensions_08(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_va40NonCriticalExtensions_08, T_va40NonCriticalExtensions_08_sequence);

  return offset;
}


static const per_sequence_t T_v860NonCriticalExtensions_13_sequence[] = {
  { &hf_rrc_rrcConnectionRelease_v860ext_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RRCConnectionRelease_CCCH_v860ext_IEs },
  { &hf_rrc_va40NonCriticalExtensions_08, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_va40NonCriticalExtensions_08 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v860NonCriticalExtensions_13(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v860NonCriticalExtensions_13, T_v860NonCriticalExtensions_13_sequence);

  return offset;
}


static const per_sequence_t T_v690NonCriticalExtensions_28_sequence[] = {
  { &hf_rrc_rrcConnectionRelease_v690ext_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RRCConnectionRelease_CCCH_v690ext_IEs },
  { &hf_rrc_v860NonCriticalExtensions_13, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v860NonCriticalExtensions_13 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v690NonCriticalExtensions_28(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v690NonCriticalExtensions_28, T_v690NonCriticalExtensions_28_sequence);

  return offset;
}


static const per_sequence_t T_laterNonCriticalExtensions_39_sequence[] = {
  { &hf_rrc_rrcConnectionRelease_CCCH_r3_add_ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_BIT_STRING },
  { &hf_rrc_v690NonCriticalExtensions_28, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v690NonCriticalExtensions_28 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_laterNonCriticalExtensions_39(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_laterNonCriticalExtensions_39, T_laterNonCriticalExtensions_39_sequence);

  return offset;
}


static const per_sequence_t T_r3_19_sequence[] = {
  { &hf_rrc_rrcConnectionRelease_CCCH_r3, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RRCConnectionRelease_CCCH_r3_IEs },
  { &hf_rrc_laterNonCriticalExtensions_39, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_laterNonCriticalExtensions_39 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_r3_19(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_r3_19, T_r3_19_sequence);

  return offset;
}


static const per_sequence_t RRCConnectionRelease_CCCH_r4_IEs_sequence[] = {
  { &hf_rrc_rrcConnectionRelease_03, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RRCConnectionRelease_r4_IEs },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_RRCConnectionRelease_CCCH_r4_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_RRCConnectionRelease_CCCH_r4_IEs, RRCConnectionRelease_CCCH_r4_IEs_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions_122_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_nonCriticalExtensions_122(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_nonCriticalExtensions_122, T_nonCriticalExtensions_122_sequence);

  return offset;
}


static const per_sequence_t T_va40NonCriticalExtensions_09_sequence[] = {
  { &hf_rrc_rrcConnectionRelease_va40ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RRCConnectionRelease_va40ext_IEs },
  { &hf_rrc_nonCriticalExtensions_122, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_nonCriticalExtensions_122 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_va40NonCriticalExtensions_09(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_va40NonCriticalExtensions_09, T_va40NonCriticalExtensions_09_sequence);

  return offset;
}


static const per_sequence_t T_v860NonCriticalExtensions_14_sequence[] = {
  { &hf_rrc_rrcConnectionRelease_v860ext_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RRCConnectionRelease_CCCH_v860ext_IEs },
  { &hf_rrc_va40NonCriticalExtensions_09, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_va40NonCriticalExtensions_09 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v860NonCriticalExtensions_14(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v860NonCriticalExtensions_14, T_v860NonCriticalExtensions_14_sequence);

  return offset;
}


static const per_sequence_t T_v690NonCriticalExtensions_29_sequence[] = {
  { &hf_rrc_rrcConnectionRelease_v690ext_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RRCConnectionRelease_CCCH_v690ext_IEs },
  { &hf_rrc_v860NonCriticalExtensions_14, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v860NonCriticalExtensions_14 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v690NonCriticalExtensions_29(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v690NonCriticalExtensions_29, T_v690NonCriticalExtensions_29_sequence);

  return offset;
}


static const per_sequence_t T_v4d0NonCriticalExtensions_10_sequence[] = {
  { &hf_rrc_rrcConnectionRelease_CCCH_r4_add_ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_BIT_STRING },
  { &hf_rrc_v690NonCriticalExtensions_29, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v690NonCriticalExtensions_29 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v4d0NonCriticalExtensions_10(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v4d0NonCriticalExtensions_10, T_v4d0NonCriticalExtensions_10_sequence);

  return offset;
}


static const per_sequence_t T_r4_10_sequence[] = {
  { &hf_rrc_rrcConnectionRelease_CCCH_r4, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RRCConnectionRelease_CCCH_r4_IEs },
  { &hf_rrc_v4d0NonCriticalExtensions_10, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v4d0NonCriticalExtensions_10 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_r4_10(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_r4_10, T_r4_10_sequence);

  return offset;
}



static int
dissect_rrc_BIT_STRING_SIZE_29(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_bit_string(tvb, offset, actx, tree, hf_index,
                                     29, 29, FALSE, NULL);

  return offset;
}


static const value_string rrc_U_RNTI_Group_vals[] = {
  {   0, "all" },
  {   1, "u-RNTI-BitMaskIndex-b1" },
  {   2, "u-RNTI-BitMaskIndex-b2" },
  {   3, "u-RNTI-BitMaskIndex-b3" },
  {   4, "u-RNTI-BitMaskIndex-b4" },
  {   5, "u-RNTI-BitMaskIndex-b5" },
  {   6, "u-RNTI-BitMaskIndex-b6" },
  {   7, "u-RNTI-BitMaskIndex-b7" },
  {   8, "u-RNTI-BitMaskIndex-b8" },
  {   9, "u-RNTI-BitMaskIndex-b9" },
  {  10, "u-RNTI-BitMaskIndex-b10" },
  {  11, "u-RNTI-BitMaskIndex-b11" },
  {  12, "u-RNTI-BitMaskIndex-b12" },
  {  13, "u-RNTI-BitMaskIndex-b13" },
  {  14, "u-RNTI-BitMaskIndex-b14" },
  {  15, "u-RNTI-BitMaskIndex-b15" },
  {  16, "u-RNTI-BitMaskIndex-b16" },
  {  17, "u-RNTI-BitMaskIndex-b17" },
  {  18, "u-RNTI-BitMaskIndex-b18" },
  {  19, "u-RNTI-BitMaskIndex-b19" },
  {  20, "u-RNTI-BitMaskIndex-b20" },
  {  21, "u-RNTI-BitMaskIndex-b21" },
  {  22, "u-RNTI-BitMaskIndex-b22" },
  {  23, "u-RNTI-BitMaskIndex-b23" },
  {  24, "u-RNTI-BitMaskIndex-b24" },
  {  25, "u-RNTI-BitMaskIndex-b25" },
  {  26, "u-RNTI-BitMaskIndex-b26" },
  {  27, "u-RNTI-BitMaskIndex-b27" },
  {  28, "u-RNTI-BitMaskIndex-b28" },
  {  29, "u-RNTI-BitMaskIndex-b29" },
  {  30, "u-RNTI-BitMaskIndex-b30" },
  {  31, "u-RNTI-BitMaskIndex-b31" },
  { 0, NULL }
};

static value_string_ext rrc_U_RNTI_Group_vals_ext = VALUE_STRING_EXT_INIT(rrc_U_RNTI_Group_vals);

static const per_choice_t U_RNTI_Group_choice[] = {
  {   0, &hf_rrc_all             , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   1, &hf_rrc_u_RNTI_BitMaskIndex_b1, ASN1_NO_EXTENSIONS     , dissect_rrc_BIT_STRING_SIZE_31 },
  {   2, &hf_rrc_u_RNTI_BitMaskIndex_b2, ASN1_NO_EXTENSIONS     , dissect_rrc_BIT_STRING_SIZE_30 },
  {   3, &hf_rrc_u_RNTI_BitMaskIndex_b3, ASN1_NO_EXTENSIONS     , dissect_rrc_BIT_STRING_SIZE_29 },
  {   4, &hf_rrc_u_RNTI_BitMaskIndex_b4, ASN1_NO_EXTENSIONS     , dissect_rrc_BIT_STRING_SIZE_28 },
  {   5, &hf_rrc_u_RNTI_BitMaskIndex_b5, ASN1_NO_EXTENSIONS     , dissect_rrc_BIT_STRING_SIZE_27 },
  {   6, &hf_rrc_u_RNTI_BitMaskIndex_b6, ASN1_NO_EXTENSIONS     , dissect_rrc_BIT_STRING_SIZE_26 },
  {   7, &hf_rrc_u_RNTI_BitMaskIndex_b7, ASN1_NO_EXTENSIONS     , dissect_rrc_BIT_STRING_SIZE_25 },
  {   8, &hf_rrc_u_RNTI_BitMaskIndex_b8, ASN1_NO_EXTENSIONS     , dissect_rrc_BIT_STRING_SIZE_24 },
  {   9, &hf_rrc_u_RNTI_BitMaskIndex_b9, ASN1_NO_EXTENSIONS     , dissect_rrc_BIT_STRING_SIZE_23 },
  {  10, &hf_rrc_u_RNTI_BitMaskIndex_b10, ASN1_NO_EXTENSIONS     , dissect_rrc_BIT_STRING_SIZE_22 },
  {  11, &hf_rrc_u_RNTI_BitMaskIndex_b11, ASN1_NO_EXTENSIONS     , dissect_rrc_BIT_STRING_SIZE_21 },
  {  12, &hf_rrc_u_RNTI_BitMaskIndex_b12, ASN1_NO_EXTENSIONS     , dissect_rrc_BIT_STRING_SIZE_20 },
  {  13, &hf_rrc_u_RNTI_BitMaskIndex_b13, ASN1_NO_EXTENSIONS     , dissect_rrc_BIT_STRING_SIZE_19 },
  {  14, &hf_rrc_u_RNTI_BitMaskIndex_b14, ASN1_NO_EXTENSIONS     , dissect_rrc_BIT_STRING_SIZE_18 },
  {  15, &hf_rrc_u_RNTI_BitMaskIndex_b15, ASN1_NO_EXTENSIONS     , dissect_rrc_BIT_STRING_SIZE_17 },
  {  16, &hf_rrc_u_RNTI_BitMaskIndex_b16, ASN1_NO_EXTENSIONS     , dissect_rrc_BIT_STRING_SIZE_16 },
  {  17, &hf_rrc_u_RNTI_BitMaskIndex_b17, ASN1_NO_EXTENSIONS     , dissect_rrc_BIT_STRING_SIZE_15 },
  {  18, &hf_rrc_u_RNTI_BitMaskIndex_b18, ASN1_NO_EXTENSIONS     , dissect_rrc_BIT_STRING_SIZE_14 },
  {  19, &hf_rrc_u_RNTI_BitMaskIndex_b19, ASN1_NO_EXTENSIONS     , dissect_rrc_BIT_STRING_SIZE_13 },
  {  20, &hf_rrc_u_RNTI_BitMaskIndex_b20, ASN1_NO_EXTENSIONS     , dissect_rrc_BIT_STRING_SIZE_12 },
  {  21, &hf_rrc_u_RNTI_BitMaskIndex_b21, ASN1_NO_EXTENSIONS     , dissect_rrc_BIT_STRING_SIZE_11 },
  {  22, &hf_rrc_u_RNTI_BitMaskIndex_b22, ASN1_NO_EXTENSIONS     , dissect_rrc_BIT_STRING_SIZE_10 },
  {  23, &hf_rrc_u_RNTI_BitMaskIndex_b23, ASN1_NO_EXTENSIONS     , dissect_rrc_BIT_STRING_SIZE_9 },
  {  24, &hf_rrc_u_RNTI_BitMaskIndex_b24, ASN1_NO_EXTENSIONS     , dissect_rrc_BIT_STRING_SIZE_8 },
  {  25, &hf_rrc_u_RNTI_BitMaskIndex_b25, ASN1_NO_EXTENSIONS     , dissect_rrc_BIT_STRING_SIZE_7 },
  {  26, &hf_rrc_u_RNTI_BitMaskIndex_b26, ASN1_NO_EXTENSIONS     , dissect_rrc_BIT_STRING_SIZE_6 },
  {  27, &hf_rrc_u_RNTI_BitMaskIndex_b27, ASN1_NO_EXTENSIONS     , dissect_rrc_BIT_STRING_SIZE_5 },
  {  28, &hf_rrc_u_RNTI_BitMaskIndex_b28, ASN1_NO_EXTENSIONS     , dissect_rrc_BIT_STRING_SIZE_4 },
  {  29, &hf_rrc_u_RNTI_BitMaskIndex_b29, ASN1_NO_EXTENSIONS     , dissect_rrc_BIT_STRING_SIZE_3 },
  {  30, &hf_rrc_u_RNTI_BitMaskIndex_b30, ASN1_NO_EXTENSIONS     , dissect_rrc_BIT_STRING_SIZE_2 },
  {  31, &hf_rrc_u_RNTI_BitMaskIndex_b31, ASN1_NO_EXTENSIONS     , dissect_rrc_BIT_STRING_SIZE_1 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_U_RNTI_Group(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_U_RNTI_Group, U_RNTI_Group_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t GroupReleaseInformation_sequence[] = {
  { &hf_rrc_uRNTI_Group     , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_U_RNTI_Group },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_GroupReleaseInformation(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_GroupReleaseInformation, GroupReleaseInformation_sequence);

  return offset;
}


static const per_sequence_t SEQUENCE_SIZE_1_maxURNTI_Group_OF_GroupReleaseInformation_sequence_of[1] = {
  { &hf_rrc_groupIdentity_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_GroupReleaseInformation },
};

static int
dissect_rrc_SEQUENCE_SIZE_1_maxURNTI_Group_OF_GroupReleaseInformation(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_SEQUENCE_SIZE_1_maxURNTI_Group_OF_GroupReleaseInformation, SEQUENCE_SIZE_1_maxURNTI_Group_OF_GroupReleaseInformation_sequence_of,
                                                  1, maxURNTI_Group, FALSE);

  return offset;
}



static int
dissect_rrc_RRCConnectionRelease_CCCH_r5_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_rrc_RRCConnectionRelease_CCCH_r4_IEs(tvb, offset, actx, tree, hf_index);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions_123_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_nonCriticalExtensions_123(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_nonCriticalExtensions_123, T_nonCriticalExtensions_123_sequence);

  return offset;
}


static const per_sequence_t T_va40NonCriticalExtensions_10_sequence[] = {
  { &hf_rrc_rrcConnectionRelease_va40ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RRCConnectionRelease_va40ext_IEs },
  { &hf_rrc_nonCriticalExtensions_123, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_nonCriticalExtensions_123 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_va40NonCriticalExtensions_10(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_va40NonCriticalExtensions_10, T_va40NonCriticalExtensions_10_sequence);

  return offset;
}


static const per_sequence_t T_v860NonCriticalExtensions_15_sequence[] = {
  { &hf_rrc_rrcConnectionRelease_v860ext_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RRCConnectionRelease_CCCH_v860ext_IEs },
  { &hf_rrc_va40NonCriticalExtensions_10, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_va40NonCriticalExtensions_10 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v860NonCriticalExtensions_15(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v860NonCriticalExtensions_15, T_v860NonCriticalExtensions_15_sequence);

  return offset;
}


static const per_sequence_t T_v690NonCriticalExtensions_30_sequence[] = {
  { &hf_rrc_rrcConnectionRelease_v690ext_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RRCConnectionRelease_CCCH_v690ext_IEs },
  { &hf_rrc_v860NonCriticalExtensions_15, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v860NonCriticalExtensions_15 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v690NonCriticalExtensions_30(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v690NonCriticalExtensions_30, T_v690NonCriticalExtensions_30_sequence);

  return offset;
}


static const per_sequence_t T_r5_08_sequence[] = {
  { &hf_rrc_rrcConnectionRelease_CCCH_r5, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RRCConnectionRelease_CCCH_r5_IEs },
  { &hf_rrc_rrcConnectionRelease_CCCH_r5_add_ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_BIT_STRING },
  { &hf_rrc_v690NonCriticalExtensions_30, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v690NonCriticalExtensions_30 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_r5_08(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_r5_08, T_r5_08_sequence);

  return offset;
}


static const per_sequence_t T_criticalExtensions_97_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_criticalExtensions_97(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_criticalExtensions_97, T_criticalExtensions_97_sequence);

  return offset;
}


static const value_string rrc_T_criticalExtensions_96_vals[] = {
  {   0, "r5" },
  {   1, "criticalExtensions" },
  { 0, NULL }
};

static const per_choice_t T_criticalExtensions_96_choice[] = {
  {   0, &hf_rrc_r5_08           , ASN1_NO_EXTENSIONS     , dissect_rrc_T_r5_08 },
  {   1, &hf_rrc_criticalExtensions_98, ASN1_NO_EXTENSIONS     , dissect_rrc_T_criticalExtensions_97 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_criticalExtensions_96(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_criticalExtensions_96, T_criticalExtensions_96_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t T_criticalExtensions_95_sequence[] = {
  { &hf_rrc_groupIdentity   , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_SEQUENCE_SIZE_1_maxURNTI_Group_OF_GroupReleaseInformation },
  { &hf_rrc_criticalExtensions_97, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_criticalExtensions_96 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_criticalExtensions_95(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_criticalExtensions_95, T_criticalExtensions_95_sequence);

  return offset;
}


static const value_string rrc_T_criticalExtensions_94_vals[] = {
  {   0, "r4" },
  {   1, "criticalExtensions" },
  { 0, NULL }
};

static const per_choice_t T_criticalExtensions_94_choice[] = {
  {   0, &hf_rrc_r4_10           , ASN1_NO_EXTENSIONS     , dissect_rrc_T_r4_10 },
  {   1, &hf_rrc_criticalExtensions_96, ASN1_NO_EXTENSIONS     , dissect_rrc_T_criticalExtensions_95 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_criticalExtensions_94(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_criticalExtensions_94, T_criticalExtensions_94_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t T_later_than_r3_17_sequence[] = {
  { &hf_rrc_u_RNTI          , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_U_RNTI },
  { &hf_rrc_rrc_TransactionIdentifier, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RRC_TransactionIdentifier },
  { &hf_rrc_criticalExtensions_95, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_criticalExtensions_94 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_later_than_r3_17(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_later_than_r3_17, T_later_than_r3_17_sequence);

  return offset;
}


static const value_string rrc_RRCConnectionRelease_CCCH_vals[] = {
  {   0, "r3" },
  {   1, "later-than-r3" },
  { 0, NULL }
};

static const per_choice_t RRCConnectionRelease_CCCH_choice[] = {
  {   0, &hf_rrc_r3_19           , ASN1_NO_EXTENSIONS     , dissect_rrc_T_r3_19 },
  {   1, &hf_rrc_later_than_r3_17, ASN1_NO_EXTENSIONS     , dissect_rrc_T_later_than_r3_17 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_RRCConnectionRelease_CCCH(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
#line 369 "../../asn1/rrc/rrc.cnf"
  col_append_str(actx->pinfo->cinfo, COL_INFO, "RRCConnectionRelease-CCCH");
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_RRCConnectionRelease_CCCH, RRCConnectionRelease_CCCH_choice,
                                 NULL);




  return offset;
}


static const per_sequence_t SRB_InformationSetupList2_sequence_of[1] = {
  { &hf_rrc_SRB_InformationSetupList2_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SRB_InformationSetup },
};

static int
dissect_rrc_SRB_InformationSetupList2(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_SRB_InformationSetupList2, SRB_InformationSetupList2_sequence_of,
                                                  3, 4, FALSE);

  return offset;
}


static const per_sequence_t RRCConnectionSetup_r3_IEs_sequence[] = {
  { &hf_rrc_initialUE_Identity, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_InitialUE_Identity },
  { &hf_rrc_rrc_TransactionIdentifier, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RRC_TransactionIdentifier },
  { &hf_rrc_activationTime  , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_ActivationTime },
  { &hf_rrc_new_U_RNTI      , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_U_RNTI },
  { &hf_rrc_new_c_RNTI      , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_C_RNTI },
  { &hf_rrc_rrc_StateIndicator, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RRC_StateIndicator },
  { &hf_rrc_utran_DRX_CycleLengthCoeff, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UTRAN_DRX_CycleLengthCoefficient },
  { &hf_rrc_capabilityUpdateRequirement, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CapabilityUpdateRequirement },
  { &hf_rrc_srb_InformationSetupList_05, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SRB_InformationSetupList2 },
  { &hf_rrc_ul_CommonTransChInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_CommonTransChInfo },
  { &hf_rrc_ul_AddReconfTransChInfoList, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UL_AddReconfTransChInfoList },
  { &hf_rrc_dl_CommonTransChInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_CommonTransChInfo },
  { &hf_rrc_dl_AddReconfTransChInfoList, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DL_AddReconfTransChInfoList },
  { &hf_rrc_frequencyInfo   , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_FrequencyInfo },
  { &hf_rrc_maxAllowedUL_TX_Power, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MaxAllowedUL_TX_Power },
  { &hf_rrc_ul_ChannelRequirement, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_ChannelRequirement },
  { &hf_rrc_dl_CommonInformation, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_CommonInformation },
  { &hf_rrc_dl_InformationPerRL_List, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_InformationPerRL_List },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_RRCConnectionSetup_r3_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_RRCConnectionSetup_r3_IEs, RRCConnectionSetup_r3_IEs_sequence);

  return offset;
}


static const per_sequence_t RRCConnectionSetup_v4b0ext_IEs_sequence[] = {
  { &hf_rrc_capabilityUpdateRequirement_r4_ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CapabilityUpdateRequirement_r4_ext },
  { &hf_rrc_dummy_02        , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_SSDT_UL },
  { &hf_rrc_cell_id_PerRL_List, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CellIdentity_PerRL_List },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_RRCConnectionSetup_v4b0ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_RRCConnectionSetup_v4b0ext_IEs, RRCConnectionSetup_v4b0ext_IEs_sequence);

  return offset;
}


static const per_sequence_t RRCConnectionSetup_v590ext_IEs_sequence[] = {
  { &hf_rrc_systemSpecificCapUpdateReq, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_SystemSpecificCapUpdateReq_v590ext },
  { &hf_rrc_dl_TPC_PowerOffsetPerRL_List, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_TPC_PowerOffsetPerRL_List },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_RRCConnectionSetup_v590ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_RRCConnectionSetup_v590ext_IEs, RRCConnectionSetup_v590ext_IEs_sequence);

  return offset;
}


static const value_string rrc_T_postVerificationPeriod_05_vals[] = {
  {   0, "true" },
  { 0, NULL }
};


static int
dissect_rrc_T_postVerificationPeriod_05(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t RRCConnectionSetup_v690ext_IEs_sequence[] = {
  { &hf_rrc_beaconPLEst     , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_BEACON_PL_Est },
  { &hf_rrc_postVerificationPeriod_05, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_postVerificationPeriod_05 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_RRCConnectionSetup_v690ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_RRCConnectionSetup_v690ext_IEs, RRCConnectionSetup_v690ext_IEs_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions_126_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_nonCriticalExtensions_126(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_nonCriticalExtensions_126, T_nonCriticalExtensions_126_sequence);

  return offset;
}


static const per_sequence_t T_v690NonCriticalExtensions_32_sequence[] = {
  { &hf_rrc_rrcConnectionSetup_v690ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RRCConnectionSetup_v690ext_IEs },
  { &hf_rrc_nonCriticalExtensions_126, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_nonCriticalExtensions_126 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v690NonCriticalExtensions_32(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v690NonCriticalExtensions_32, T_v690NonCriticalExtensions_32_sequence);

  return offset;
}


static const per_sequence_t T_v590NonCriticalExtensions_21_sequence[] = {
  { &hf_rrc_rrcConnectionSetup_v590ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RRCConnectionSetup_v590ext_IEs },
  { &hf_rrc_v690NonCriticalExtensions_32, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v690NonCriticalExtensions_32 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v590NonCriticalExtensions_21(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v590NonCriticalExtensions_21, T_v590NonCriticalExtensions_21_sequence);

  return offset;
}


static const per_sequence_t T_v4b0NonCriticalExtensions_11_sequence[] = {
  { &hf_rrc_rrcConnectionSetup_v4b0ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RRCConnectionSetup_v4b0ext_IEs },
  { &hf_rrc_v590NonCriticalExtensions_21, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v590NonCriticalExtensions_21 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v4b0NonCriticalExtensions_11(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v4b0NonCriticalExtensions_11, T_v4b0NonCriticalExtensions_11_sequence);

  return offset;
}


static const per_sequence_t T_laterNonCriticalExtensions_41_sequence[] = {
  { &hf_rrc_rrcConnectionSetup_r3_add_ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_BIT_STRING },
  { &hf_rrc_v4b0NonCriticalExtensions_11, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v4b0NonCriticalExtensions_11 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_laterNonCriticalExtensions_41(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_laterNonCriticalExtensions_41, T_laterNonCriticalExtensions_41_sequence);

  return offset;
}


static const per_sequence_t T_r3_20_sequence[] = {
  { &hf_rrc_rrcConnectionSetup_r3, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RRCConnectionSetup_r3_IEs },
  { &hf_rrc_laterNonCriticalExtensions_41, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_laterNonCriticalExtensions_41 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_r3_20(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_r3_20, T_r3_20_sequence);

  return offset;
}


static const per_sequence_t CapabilityUpdateRequirement_r4_sequence[] = {
  { &hf_rrc_ue_RadioCapabilityFDDUpdateRequirement_FDD, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { &hf_rrc_ue_RadioCapabilityTDDUpdateRequirement_TDD384, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { &hf_rrc_ue_RadioCapabilityTDDUpdateRequirement_TDD128, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { &hf_rrc_systemSpecificCapUpdateReqList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_SystemSpecificCapUpdateReqList },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_CapabilityUpdateRequirement_r4(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_CapabilityUpdateRequirement_r4, CapabilityUpdateRequirement_r4_sequence);

  return offset;
}


static const per_sequence_t RRCConnectionSetup_r4_IEs_sequence[] = {
  { &hf_rrc_activationTime  , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_ActivationTime },
  { &hf_rrc_new_U_RNTI      , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_U_RNTI },
  { &hf_rrc_new_c_RNTI      , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_C_RNTI },
  { &hf_rrc_rrc_StateIndicator, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RRC_StateIndicator },
  { &hf_rrc_utran_DRX_CycleLengthCoeff, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UTRAN_DRX_CycleLengthCoefficient },
  { &hf_rrc_capabilityUpdateRequirement_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CapabilityUpdateRequirement_r4 },
  { &hf_rrc_srb_InformationSetupList_05, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SRB_InformationSetupList2 },
  { &hf_rrc_ul_CommonTransChInfo_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_CommonTransChInfo_r4 },
  { &hf_rrc_ul_AddReconfTransChInfoList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_AddReconfTransChInfoList },
  { &hf_rrc_dl_CommonTransChInfo_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_CommonTransChInfo_r4 },
  { &hf_rrc_dl_AddReconfTransChInfoList_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_AddReconfTransChInfoList_r4 },
  { &hf_rrc_frequencyInfo   , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_FrequencyInfo },
  { &hf_rrc_maxAllowedUL_TX_Power, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MaxAllowedUL_TX_Power },
  { &hf_rrc_ul_ChannelRequirement_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_ChannelRequirement_r4 },
  { &hf_rrc_dl_CommonInformation_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_CommonInformation_r4 },
  { &hf_rrc_dl_InformationPerRL_List_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_InformationPerRL_List_r4 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_RRCConnectionSetup_r4_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_RRCConnectionSetup_r4_IEs, RRCConnectionSetup_r4_IEs_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions_127_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_nonCriticalExtensions_127(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_nonCriticalExtensions_127, T_nonCriticalExtensions_127_sequence);

  return offset;
}


static const per_sequence_t T_v690NonCriticalExtensions_33_sequence[] = {
  { &hf_rrc_rrcConnectionSetup_v690ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RRCConnectionSetup_v690ext_IEs },
  { &hf_rrc_nonCriticalExtensions_127, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_nonCriticalExtensions_127 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v690NonCriticalExtensions_33(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v690NonCriticalExtensions_33, T_v690NonCriticalExtensions_33_sequence);

  return offset;
}


static const per_sequence_t T_v590NonCriticalExtensions_22_sequence[] = {
  { &hf_rrc_rrcConnectionSetup_v590ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RRCConnectionSetup_v590ext_IEs },
  { &hf_rrc_v690NonCriticalExtensions_33, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v690NonCriticalExtensions_33 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v590NonCriticalExtensions_22(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v590NonCriticalExtensions_22, T_v590NonCriticalExtensions_22_sequence);

  return offset;
}


static const per_sequence_t T_v4d0NonCriticalExtensions_11_sequence[] = {
  { &hf_rrc_rrcConnectionSetup_r4_add_ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_BIT_STRING },
  { &hf_rrc_v590NonCriticalExtensions_22, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v590NonCriticalExtensions_22 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v4d0NonCriticalExtensions_11(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v4d0NonCriticalExtensions_11, T_v4d0NonCriticalExtensions_11_sequence);

  return offset;
}


static const per_sequence_t T_r4_11_sequence[] = {
  { &hf_rrc_rrcConnectionSetup_r4, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RRCConnectionSetup_r4_IEs },
  { &hf_rrc_v4d0NonCriticalExtensions_11, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v4d0NonCriticalExtensions_11 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_r4_11(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_r4_11, T_r4_11_sequence);

  return offset;
}


static const value_string rrc_SystemSpecificCapUpdateReq_r5_vals[] = {
  {   0, "gsm" },
  {   1, "geranIu" },
  { 0, NULL }
};


static int
dissect_rrc_SystemSpecificCapUpdateReq_r5(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     2, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t SystemSpecificCapUpdateReqList_r5_sequence_of[1] = {
  { &hf_rrc_SystemSpecificCapUpdateReqList_r5_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SystemSpecificCapUpdateReq_r5 },
};

static int
dissect_rrc_SystemSpecificCapUpdateReqList_r5(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_SystemSpecificCapUpdateReqList_r5, SystemSpecificCapUpdateReqList_r5_sequence_of,
                                                  1, maxSystemCapability, FALSE);

  return offset;
}


static const per_sequence_t CapabilityUpdateRequirement_r5_sequence[] = {
  { &hf_rrc_ue_RadioCapabilityFDDUpdateRequirement_FDD, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { &hf_rrc_ue_RadioCapabilityTDDUpdateRequirement_TDD384, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { &hf_rrc_ue_RadioCapabilityTDDUpdateRequirement_TDD128, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { &hf_rrc_systemSpecificCapUpdateReqList_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_SystemSpecificCapUpdateReqList_r5 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_CapabilityUpdateRequirement_r5(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_CapabilityUpdateRequirement_r5, CapabilityUpdateRequirement_r5_sequence);

  return offset;
}


static const per_sequence_t T_complete_19_sequence[] = {
  { &hf_rrc_srb_InformationSetupList_05, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SRB_InformationSetupList2 },
  { &hf_rrc_ul_CommonTransChInfo_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_CommonTransChInfo_r4 },
  { &hf_rrc_ul_AddReconfTransChInfoList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_AddReconfTransChInfoList },
  { &hf_rrc_dl_CommonTransChInfo_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_CommonTransChInfo_r4 },
  { &hf_rrc_dl_AddReconfTransChInfoList_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_AddReconfTransChInfoList_r4 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_complete_19(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_complete_19, T_complete_19_sequence);

  return offset;
}


static const per_sequence_t T_defaultConfig_16_sequence[] = {
  { &hf_rrc_defaultConfigMode, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DefaultConfigMode },
  { &hf_rrc_defaultConfigIdentity_02, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DefaultConfigIdentity_r5 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_defaultConfig_16(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_defaultConfig_16, T_defaultConfig_16_sequence);

  return offset;
}


static const value_string rrc_T_preConfigMode_16_vals[] = {
  {   0, "predefinedConfigIdentity" },
  {   1, "defaultConfig" },
  { 0, NULL }
};

static const per_choice_t T_preConfigMode_16_choice[] = {
  {   0, &hf_rrc_predefinedConfigIdentity, ASN1_NO_EXTENSIONS     , dissect_rrc_PredefinedConfigIdentity },
  {   1, &hf_rrc_defaultConfig_16, ASN1_NO_EXTENSIONS     , dissect_rrc_T_defaultConfig_16 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_preConfigMode_16(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_preConfigMode_16, T_preConfigMode_16_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t T_preconfiguration_16_sequence[] = {
  { &hf_rrc_preConfigMode_16, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_preConfigMode_16 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_preconfiguration_16(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_preconfiguration_16, T_preconfiguration_16_sequence);

  return offset;
}


static const value_string rrc_T_specificationMode_19_vals[] = {
  {   0, "complete" },
  {   1, "preconfiguration" },
  { 0, NULL }
};

static const per_choice_t T_specificationMode_19_choice[] = {
  {   0, &hf_rrc_complete_19     , ASN1_NO_EXTENSIONS     , dissect_rrc_T_complete_19 },
  {   1, &hf_rrc_preconfiguration_16, ASN1_NO_EXTENSIONS     , dissect_rrc_T_preconfiguration_16 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_specificationMode_19(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_specificationMode_19, T_specificationMode_19_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t T_fdd_83_sequence[] = {
  { &hf_rrc_primaryCPICH_Info, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PrimaryCPICH_Info },
  { &hf_rrc_dummy1_02       , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_PDSCH_SHO_DCH_Info },
  { &hf_rrc_dummy2_07       , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_PDSCH_CodeMapping },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_fdd_83(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_fdd_83, T_fdd_83_sequence);

  return offset;
}


static const value_string rrc_T_modeSpecificInfo_51_vals[] = {
  {   0, "fdd" },
  {   1, "tdd" },
  { 0, NULL }
};

static const per_choice_t T_modeSpecificInfo_51_choice[] = {
  {   0, &hf_rrc_fdd_84          , ASN1_NO_EXTENSIONS     , dissect_rrc_T_fdd_83 },
  {   1, &hf_rrc_tdd_47          , ASN1_NO_EXTENSIONS     , dissect_rrc_PrimaryCCPCH_Info_r4 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_modeSpecificInfo_51(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_modeSpecificInfo_51, T_modeSpecificInfo_51_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t DL_InformationPerRL_r5bis_sequence[] = {
  { &hf_rrc_modeSpecificInfo_51, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_modeSpecificInfo_51 },
  { &hf_rrc_dl_DPCH_InfoPerRL_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_DPCH_InfoPerRL_r5 },
  { &hf_rrc_dummy_43        , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_SCCPCH_InfoForFACH_r4 },
  { &hf_rrc_cell_id         , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CellIdentity },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_DL_InformationPerRL_r5bis(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_DL_InformationPerRL_r5bis, DL_InformationPerRL_r5bis_sequence);

  return offset;
}


static const per_sequence_t DL_InformationPerRL_List_r5bis_sequence_of[1] = {
  { &hf_rrc_DL_InformationPerRL_List_r5bis_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DL_InformationPerRL_r5bis },
};

static int
dissect_rrc_DL_InformationPerRL_List_r5bis(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_DL_InformationPerRL_List_r5bis, DL_InformationPerRL_List_r5bis_sequence_of,
                                                  1, maxRL, FALSE);

  return offset;
}


static const per_sequence_t RRCConnectionSetup_r5_IEs_sequence[] = {
  { &hf_rrc_activationTime  , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_ActivationTime },
  { &hf_rrc_new_U_RNTI      , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_U_RNTI },
  { &hf_rrc_new_c_RNTI      , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_C_RNTI },
  { &hf_rrc_rrc_StateIndicator, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RRC_StateIndicator },
  { &hf_rrc_utran_DRX_CycleLengthCoeff, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UTRAN_DRX_CycleLengthCoefficient },
  { &hf_rrc_capabilityUpdateRequirement_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CapabilityUpdateRequirement_r5 },
  { &hf_rrc_specificationMode_19, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_specificationMode_19 },
  { &hf_rrc_frequencyInfo   , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_FrequencyInfo },
  { &hf_rrc_maxAllowedUL_TX_Power, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MaxAllowedUL_TX_Power },
  { &hf_rrc_ul_ChannelRequirement_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_ChannelRequirement_r4 },
  { &hf_rrc_dl_CommonInformation_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_CommonInformation_r4 },
  { &hf_rrc_dl_InformationPerRL_List_08, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_InformationPerRL_List_r5bis },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_RRCConnectionSetup_r5_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_RRCConnectionSetup_r5_IEs, RRCConnectionSetup_r5_IEs_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions_128_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_nonCriticalExtensions_128(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_nonCriticalExtensions_128, T_nonCriticalExtensions_128_sequence);

  return offset;
}


static const per_sequence_t T_v690NonCriticalExtensions_34_sequence[] = {
  { &hf_rrc_rrcConnectionSetup_v690ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RRCConnectionSetup_v690ext_IEs },
  { &hf_rrc_nonCriticalExtensions_128, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_nonCriticalExtensions_128 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v690NonCriticalExtensions_34(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v690NonCriticalExtensions_34, T_v690NonCriticalExtensions_34_sequence);

  return offset;
}


static const per_sequence_t T_r5_09_sequence[] = {
  { &hf_rrc_rrcConnectionSetup_r5, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RRCConnectionSetup_r5_IEs },
  { &hf_rrc_rrcConnectionSetup_r5_add_ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_BIT_STRING },
  { &hf_rrc_v690NonCriticalExtensions_34, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v690NonCriticalExtensions_34 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_r5_09(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_r5_09, T_r5_09_sequence);

  return offset;
}


static const per_sequence_t SRB_InformationSetupList2_r6_sequence_of[1] = {
  { &hf_rrc_SRB_InformationSetupList2_r6_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SRB_InformationSetup_r6 },
};

static int
dissect_rrc_SRB_InformationSetupList2_r6(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_SRB_InformationSetupList2_r6, SRB_InformationSetupList2_r6_sequence_of,
                                                  3, 4, FALSE);

  return offset;
}


static const per_sequence_t T_complete_20_sequence[] = {
  { &hf_rrc_srb_InformationSetupList_06, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SRB_InformationSetupList2_r6 },
  { &hf_rrc_ul_CommonTransChInfo_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_CommonTransChInfo_r4 },
  { &hf_rrc_ul_AddReconfTransChInfoList_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_AddReconfTransChInfoList_r6 },
  { &hf_rrc_dl_CommonTransChInfo_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_CommonTransChInfo_r4 },
  { &hf_rrc_dl_AddReconfTransChInfoList_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_AddReconfTransChInfoList_r5 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_complete_20(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_complete_20, T_complete_20_sequence);

  return offset;
}


static const per_sequence_t T_defaultConfig_17_sequence[] = {
  { &hf_rrc_defaultConfigMode, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DefaultConfigMode },
  { &hf_rrc_defaultConfigIdentity_03, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DefaultConfigIdentity_r6 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_defaultConfig_17(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_defaultConfig_17, T_defaultConfig_17_sequence);

  return offset;
}


static const value_string rrc_T_preConfigMode_17_vals[] = {
  {   0, "predefinedConfigIdentity" },
  {   1, "defaultConfig" },
  { 0, NULL }
};

static const per_choice_t T_preConfigMode_17_choice[] = {
  {   0, &hf_rrc_predefinedConfigIdentity, ASN1_NO_EXTENSIONS     , dissect_rrc_PredefinedConfigIdentity },
  {   1, &hf_rrc_defaultConfig_17, ASN1_NO_EXTENSIONS     , dissect_rrc_T_defaultConfig_17 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_preConfigMode_17(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_preConfigMode_17, T_preConfigMode_17_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t T_preconfiguration_17_sequence[] = {
  { &hf_rrc_preConfigMode_17, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_preConfigMode_17 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_preconfiguration_17(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_preconfiguration_17, T_preconfiguration_17_sequence);

  return offset;
}


static const value_string rrc_T_specificationMode_20_vals[] = {
  {   0, "complete" },
  {   1, "preconfiguration" },
  { 0, NULL }
};

static const per_choice_t T_specificationMode_20_choice[] = {
  {   0, &hf_rrc_complete_20     , ASN1_NO_EXTENSIONS     , dissect_rrc_T_complete_20 },
  {   1, &hf_rrc_preconfiguration_17, ASN1_NO_EXTENSIONS     , dissect_rrc_T_preconfiguration_17 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_specificationMode_20(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_specificationMode_20, T_specificationMode_20_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t RRCConnectionSetup_r6_IEs_sequence[] = {
  { &hf_rrc_activationTime  , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_ActivationTime },
  { &hf_rrc_new_U_RNTI      , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_U_RNTI },
  { &hf_rrc_new_c_RNTI      , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_C_RNTI },
  { &hf_rrc_new_H_RNTI      , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_H_RNTI },
  { &hf_rrc_newPrimary_E_RNTI, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_E_RNTI },
  { &hf_rrc_newSecondary_E_RNTI, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_E_RNTI },
  { &hf_rrc_rrc_StateIndicator, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RRC_StateIndicator },
  { &hf_rrc_utran_DRX_CycleLengthCoeff, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UTRAN_DRX_CycleLengthCoefficient },
  { &hf_rrc_capabilityUpdateRequirement_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CapabilityUpdateRequirement_r5 },
  { &hf_rrc_specificationMode_20, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_specificationMode_20 },
  { &hf_rrc_frequencyInfo   , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_FrequencyInfo },
  { &hf_rrc_maxAllowedUL_TX_Power, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MaxAllowedUL_TX_Power },
  { &hf_rrc_ul_DPCH_Info    , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_DPCH_Info_r6 },
  { &hf_rrc_ul_EDCH_Information, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_EDCH_Information_r6 },
  { &hf_rrc_dl_HSPDSCH_Information_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_HSPDSCH_Information_r6 },
  { &hf_rrc_dl_CommonInformation_03, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_CommonInformation_r6 },
  { &hf_rrc_dl_InformationPerRL_List_03, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_InformationPerRL_List_r6 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_RRCConnectionSetup_r6_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_RRCConnectionSetup_r6_IEs, RRCConnectionSetup_r6_IEs_sequence);

  return offset;
}


static const per_sequence_t RRCConnectionSetup_v6b0ext_IEs_sequence[] = {
  { &hf_rrc_dl_InformationPerRL_List_v6b0ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_InformationPerRL_List_v6b0ext },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_RRCConnectionSetup_v6b0ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_RRCConnectionSetup_v6b0ext_IEs, RRCConnectionSetup_v6b0ext_IEs_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions_129_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_nonCriticalExtensions_129(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_nonCriticalExtensions_129, T_nonCriticalExtensions_129_sequence);

  return offset;
}


static const per_sequence_t T_v6b0NonCriticalExtensions_10_sequence[] = {
  { &hf_rrc_rrcConnectionSetup_v6b0ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RRCConnectionSetup_v6b0ext_IEs },
  { &hf_rrc_nonCriticalExtensions_129, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_nonCriticalExtensions_129 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v6b0NonCriticalExtensions_10(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v6b0NonCriticalExtensions_10, T_v6b0NonCriticalExtensions_10_sequence);

  return offset;
}


static const per_sequence_t T_r6_10_sequence[] = {
  { &hf_rrc_rrcConnectionSetup_r6, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RRCConnectionSetup_r6_IEs },
  { &hf_rrc_rrcConnectionSetup_r6_add_ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_BIT_STRING },
  { &hf_rrc_v6b0NonCriticalExtensions_11, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v6b0NonCriticalExtensions_10 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_r6_10(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_r6_10, T_r6_10_sequence);

  return offset;
}


static const per_sequence_t CapabilityUpdateRequirement_r7_sequence[] = {
  { &hf_rrc_ue_RadioCapabilityFDDUpdateRequirement_FDD, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { &hf_rrc_ue_RadioCapabilityTDDUpdateRequirement_TDD384, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { &hf_rrc_ue_RadioCapabilityTDDUpdateRequirement_TDD768, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { &hf_rrc_ue_RadioCapabilityTDDUpdateRequirement_TDD128, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { &hf_rrc_systemSpecificCapUpdateReqList_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_SystemSpecificCapUpdateReqList_r5 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_CapabilityUpdateRequirement_r7(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_CapabilityUpdateRequirement_r7, CapabilityUpdateRequirement_r7_sequence);

  return offset;
}


static const per_sequence_t SRB_InformationSetupList2_r7_sequence_of[1] = {
  { &hf_rrc_SRB_InformationSetupList2_r7_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SRB_InformationSetup_r7 },
};

static int
dissect_rrc_SRB_InformationSetupList2_r7(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_SRB_InformationSetupList2_r7, SRB_InformationSetupList2_r7_sequence_of,
                                                  3, 4, FALSE);

  return offset;
}


static const per_sequence_t T_complete_21_sequence[] = {
  { &hf_rrc_srb_InformationSetupList_07, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SRB_InformationSetupList2_r7 },
  { &hf_rrc_ul_CommonTransChInfo_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_CommonTransChInfo_r4 },
  { &hf_rrc_ul_AddReconfTransChInfoList_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_AddReconfTransChInfoList_r7 },
  { &hf_rrc_dl_CommonTransChInfo_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_CommonTransChInfo_r4 },
  { &hf_rrc_dl_AddReconfTransChInfoList_03, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_AddReconfTransChInfoList_r7 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_complete_21(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_complete_21, T_complete_21_sequence);

  return offset;
}


static const per_sequence_t T_defaultConfig_18_sequence[] = {
  { &hf_rrc_defaultConfigMode, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DefaultConfigMode },
  { &hf_rrc_defaultConfigIdentity_03, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DefaultConfigIdentity_r6 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_defaultConfig_18(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_defaultConfig_18, T_defaultConfig_18_sequence);

  return offset;
}


static const value_string rrc_T_preConfigMode_18_vals[] = {
  {   0, "predefinedConfigIdentity" },
  {   1, "defaultConfig" },
  { 0, NULL }
};

static const per_choice_t T_preConfigMode_18_choice[] = {
  {   0, &hf_rrc_predefinedConfigIdentity, ASN1_NO_EXTENSIONS     , dissect_rrc_PredefinedConfigIdentity },
  {   1, &hf_rrc_defaultConfig_18, ASN1_NO_EXTENSIONS     , dissect_rrc_T_defaultConfig_18 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_preConfigMode_18(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_preConfigMode_18, T_preConfigMode_18_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t T_preconfiguration_18_sequence[] = {
  { &hf_rrc_preConfigMode_18, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_preConfigMode_18 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_preconfiguration_18(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_preconfiguration_18, T_preconfiguration_18_sequence);

  return offset;
}


static const value_string rrc_T_specificationMode_21_vals[] = {
  {   0, "complete" },
  {   1, "preconfiguration" },
  { 0, NULL }
};

static const per_choice_t T_specificationMode_21_choice[] = {
  {   0, &hf_rrc_complete_21     , ASN1_NO_EXTENSIONS     , dissect_rrc_T_complete_21 },
  {   1, &hf_rrc_preconfiguration_18, ASN1_NO_EXTENSIONS     , dissect_rrc_T_preconfiguration_18 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_specificationMode_21(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_specificationMode_21, T_specificationMode_21_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t RRCConnectionSetup_r7_IEs_sequence[] = {
  { &hf_rrc_activationTime  , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_ActivationTime },
  { &hf_rrc_new_U_RNTI      , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_U_RNTI },
  { &hf_rrc_new_c_RNTI      , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_C_RNTI },
  { &hf_rrc_new_H_RNTI      , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_H_RNTI },
  { &hf_rrc_newPrimary_E_RNTI, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_E_RNTI },
  { &hf_rrc_newSecondary_E_RNTI, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_E_RNTI },
  { &hf_rrc_rrc_StateIndicator, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RRC_StateIndicator },
  { &hf_rrc_utran_DRX_CycleLengthCoeff_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UTRAN_DRX_CycleLengthCoefficient_r7 },
  { &hf_rrc_capabilityUpdateRequirement_03, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CapabilityUpdateRequirement_r7 },
  { &hf_rrc_supportForChangeOfUE_Capability, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { &hf_rrc_specificationMode_21, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_specificationMode_21 },
  { &hf_rrc_frequencyInfo   , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_FrequencyInfo },
  { &hf_rrc_multi_frequencyInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_Multi_frequencyInfo_LCR_r7 },
  { &hf_rrc_dtx_drx_TimingInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DTX_DRX_TimingInfo_r7 },
  { &hf_rrc_dtx_drx_Info    , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DTX_DRX_Info_r7 },
  { &hf_rrc_hs_scch_LessInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_HS_SCCH_LessInfo_r7 },
  { &hf_rrc_maxAllowedUL_TX_Power, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MaxAllowedUL_TX_Power },
  { &hf_rrc_ul_DPCH_Info_01 , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_DPCH_Info_r7 },
  { &hf_rrc_ul_EDCH_Information_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_EDCH_Information_r7 },
  { &hf_rrc_dl_HSPDSCH_Information_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_HSPDSCH_Information_r7 },
  { &hf_rrc_dl_CommonInformation_04, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_CommonInformation_r7 },
  { &hf_rrc_dl_InformationPerRL_List_04, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_InformationPerRL_List_r7 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_RRCConnectionSetup_r7_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_RRCConnectionSetup_r7_IEs, RRCConnectionSetup_r7_IEs_sequence);

  return offset;
}


static const per_sequence_t RRCConnectionSetup_v780ext_IEs_sequence[] = {
  { &hf_rrc_hs_DSCH_TBSizeTable, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_HS_DSCH_TBSizeTable },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_RRCConnectionSetup_v780ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_RRCConnectionSetup_v780ext_IEs, RRCConnectionSetup_v780ext_IEs_sequence);

  return offset;
}


static const per_sequence_t RRCConnectionSetup_v7d0ext_IEs_sequence[] = {
  { &hf_rrc_ul_EDCH_Information_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_EDCH_Information_ext },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_RRCConnectionSetup_v7d0ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_RRCConnectionSetup_v7d0ext_IEs, RRCConnectionSetup_v7d0ext_IEs_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions_130_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_nonCriticalExtensions_130(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_nonCriticalExtensions_130, T_nonCriticalExtensions_130_sequence);

  return offset;
}


static const per_sequence_t T_v7d0NonCriticalExtensions_11_sequence[] = {
  { &hf_rrc_rrcConnectionSetup_v7d0ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RRCConnectionSetup_v7d0ext_IEs },
  { &hf_rrc_nonCriticalExtensions_130, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_nonCriticalExtensions_130 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v7d0NonCriticalExtensions_11(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v7d0NonCriticalExtensions_11, T_v7d0NonCriticalExtensions_11_sequence);

  return offset;
}


static const per_sequence_t T_v780NonCriticalExtensions_08_sequence[] = {
  { &hf_rrc_rrcConnectionSetup_v780ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RRCConnectionSetup_v780ext_IEs },
  { &hf_rrc_v7d0NonCriticalExtensions_11, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v7d0NonCriticalExtensions_11 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v780NonCriticalExtensions_08(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v780NonCriticalExtensions_08, T_v780NonCriticalExtensions_08_sequence);

  return offset;
}


static const per_sequence_t T_r7_09_sequence[] = {
  { &hf_rrc_rrcConnectionSetup_r7, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RRCConnectionSetup_r7_IEs },
  { &hf_rrc_rrcConnectionSetup_r7_add_ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_BIT_STRING },
  { &hf_rrc_v780NonCriticalExtensions_08, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v780NonCriticalExtensions_08 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_r7_09(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_r7_09, T_r7_09_sequence);

  return offset;
}


static const per_sequence_t CapabilityUpdateRequirement_r8_sequence[] = {
  { &hf_rrc_ue_RadioCapabilityFDDUpdateRequirement_FDD, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { &hf_rrc_ue_RadioCapabilityTDDUpdateRequirement_TDD384, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { &hf_rrc_ue_RadioCapabilityTDDUpdateRequirement_TDD768, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { &hf_rrc_ue_RadioCapabilityTDDUpdateRequirement_TDD128, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { &hf_rrc_systemSpecificCapUpdateReqList_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_SystemSpecificCapUpdateReqList_r8 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_CapabilityUpdateRequirement_r8(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_CapabilityUpdateRequirement_r8, CapabilityUpdateRequirement_r8_sequence);

  return offset;
}


static const per_sequence_t SRB_InformationSetupList2_r8_sequence_of[1] = {
  { &hf_rrc_SRB_InformationSetupList2_r8_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SRB_InformationSetup_r8 },
};

static int
dissect_rrc_SRB_InformationSetupList2_r8(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_SRB_InformationSetupList2_r8, SRB_InformationSetupList2_r8_sequence_of,
                                                  3, 4, FALSE);

  return offset;
}


static const per_sequence_t T_complete_22_sequence[] = {
  { &hf_rrc_srb_InformationSetupList_08, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SRB_InformationSetupList2_r8 },
  { &hf_rrc_ul_CommonTransChInfo_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_CommonTransChInfo_r4 },
  { &hf_rrc_ul_AddReconfTransChInfoList_03, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_AddReconfTransChInfoList_r8 },
  { &hf_rrc_dl_CommonTransChInfo_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_CommonTransChInfo_r4 },
  { &hf_rrc_dl_AddReconfTransChInfoList_03, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_AddReconfTransChInfoList_r7 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_complete_22(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_complete_22, T_complete_22_sequence);

  return offset;
}


static const per_sequence_t T_defaultConfig_19_sequence[] = {
  { &hf_rrc_defaultConfigMode, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DefaultConfigMode },
  { &hf_rrc_defaultConfigIdentity_03, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DefaultConfigIdentity_r6 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_defaultConfig_19(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_defaultConfig_19, T_defaultConfig_19_sequence);

  return offset;
}


static const value_string rrc_T_preConfigMode_19_vals[] = {
  {   0, "predefinedConfigIdentity" },
  {   1, "defaultConfig" },
  { 0, NULL }
};

static const per_choice_t T_preConfigMode_19_choice[] = {
  {   0, &hf_rrc_predefinedConfigIdentity, ASN1_NO_EXTENSIONS     , dissect_rrc_PredefinedConfigIdentity },
  {   1, &hf_rrc_defaultConfig_19, ASN1_NO_EXTENSIONS     , dissect_rrc_T_defaultConfig_19 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_preConfigMode_19(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_preConfigMode_19, T_preConfigMode_19_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t T_preconfiguration_19_sequence[] = {
  { &hf_rrc_preConfigMode_19, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_preConfigMode_19 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_preconfiguration_19(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_preconfiguration_19, T_preconfiguration_19_sequence);

  return offset;
}


static const value_string rrc_T_specificationMode_22_vals[] = {
  {   0, "complete" },
  {   1, "preconfiguration" },
  { 0, NULL }
};

static const per_choice_t T_specificationMode_22_choice[] = {
  {   0, &hf_rrc_complete_22     , ASN1_NO_EXTENSIONS     , dissect_rrc_T_complete_22 },
  {   1, &hf_rrc_preconfiguration_19, ASN1_NO_EXTENSIONS     , dissect_rrc_T_preconfiguration_19 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_specificationMode_22(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_specificationMode_22, T_specificationMode_22_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t RRCConnectionSetup_r8_IEs_sequence[] = {
  { &hf_rrc_activationTime  , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_ActivationTime },
  { &hf_rrc_new_U_RNTI      , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_U_RNTI },
  { &hf_rrc_new_c_RNTI      , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_C_RNTI },
  { &hf_rrc_new_H_RNTI      , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_H_RNTI },
  { &hf_rrc_newPrimary_E_RNTI, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_E_RNTI },
  { &hf_rrc_newSecondary_E_RNTI, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_E_RNTI },
  { &hf_rrc_rrc_StateIndicator, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RRC_StateIndicator },
  { &hf_rrc_utran_DRX_CycleLengthCoeff_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UTRAN_DRX_CycleLengthCoefficient_r7 },
  { &hf_rrc_capabilityUpdateRequirement_04, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CapabilityUpdateRequirement_r8 },
  { &hf_rrc_supportForChangeOfUE_Capability, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { &hf_rrc_dummy_07        , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DefaultConfigForCellFACH },
  { &hf_rrc_specificationMode_22, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_specificationMode_22 },
  { &hf_rrc_frequencyInfo   , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_FrequencyInfo },
  { &hf_rrc_multi_frequencyInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_Multi_frequencyInfo_LCR_r7 },
  { &hf_rrc_dtx_drx_TimingInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DTX_DRX_TimingInfo_r7 },
  { &hf_rrc_dtx_drx_Info    , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DTX_DRX_Info_r7 },
  { &hf_rrc_hs_scch_LessInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_HS_SCCH_LessInfo_r7 },
  { &hf_rrc_maxAllowedUL_TX_Power, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MaxAllowedUL_TX_Power },
  { &hf_rrc_ul_DPCH_Info_01 , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_DPCH_Info_r7 },
  { &hf_rrc_ul_EDCH_Information_03, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_EDCH_Information_r8 },
  { &hf_rrc_dl_HSPDSCH_Information_03, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_HSPDSCH_Information_r8 },
  { &hf_rrc_dl_CommonInformation_05, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_CommonInformation_r8 },
  { &hf_rrc_dl_InformationPerRL_List_05, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_InformationPerRL_List_r8 },
  { &hf_rrc_dl_SecondaryCellInfoFDD, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_SecondaryCellInfoFDD },
  { &hf_rrc_sps_Information_TDD128, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_SPS_Information_TDD128_r8 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_RRCConnectionSetup_r8_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_RRCConnectionSetup_r8_IEs, RRCConnectionSetup_r8_IEs_sequence);

  return offset;
}


static const per_sequence_t RRCConnectionSetup_v890ext_IEs_sequence[] = {
  { &hf_rrc_dl_SecondaryCellInfoFDD_v890ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_SecondaryCellInfoFDD_v890ext },
  { &hf_rrc_dl_HSPDSCH_Information_04, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_HSPDSCH_Information_r8_ext },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_RRCConnectionSetup_v890ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_RRCConnectionSetup_v890ext_IEs, RRCConnectionSetup_v890ext_IEs_sequence);

  return offset;
}


static const per_sequence_t RRCConnectionSetup_v8a0ext_IEs_sequence[] = {
  { &hf_rrc_dl_HSPDSCH_Information_05, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_HSPDSCH_Information_r8_ext2 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_RRCConnectionSetup_v8a0ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_RRCConnectionSetup_v8a0ext_IEs, RRCConnectionSetup_v8a0ext_IEs_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions_131_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_nonCriticalExtensions_131(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_nonCriticalExtensions_131, T_nonCriticalExtensions_131_sequence);

  return offset;
}


static const per_sequence_t T_v8a0NonCriticalExtensions_08_sequence[] = {
  { &hf_rrc_rrcConnectionSetup_v8a0ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RRCConnectionSetup_v8a0ext_IEs },
  { &hf_rrc_nonCriticalExtensions_131, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_nonCriticalExtensions_131 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v8a0NonCriticalExtensions_08(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v8a0NonCriticalExtensions_08, T_v8a0NonCriticalExtensions_08_sequence);

  return offset;
}


static const per_sequence_t T_v890NonCriticalExtensions_08_sequence[] = {
  { &hf_rrc_rrcConnectionSetup_v890ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RRCConnectionSetup_v890ext_IEs },
  { &hf_rrc_v8a0NonCriticalExtensions_08, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v8a0NonCriticalExtensions_08 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v890NonCriticalExtensions_08(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v890NonCriticalExtensions_08, T_v890NonCriticalExtensions_08_sequence);

  return offset;
}


static const per_sequence_t T_v7d0NonCriticalExtensions_12_sequence[] = {
  { &hf_rrc_rrcConnectionSetup_v7d0ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RRCConnectionSetup_v7d0ext_IEs },
  { &hf_rrc_v890NonCriticalExtensions_08, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v890NonCriticalExtensions_08 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v7d0NonCriticalExtensions_12(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v7d0NonCriticalExtensions_12, T_v7d0NonCriticalExtensions_12_sequence);

  return offset;
}


static const per_sequence_t T_r8_10_sequence[] = {
  { &hf_rrc_rrcConnectionSetup_r8, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RRCConnectionSetup_r8_IEs },
  { &hf_rrc_rrcConnectionSetup_r8_add_ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_BIT_STRING },
  { &hf_rrc_v7d0NonCriticalExtensions_12, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v7d0NonCriticalExtensions_12 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_r8_10(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_r8_10, T_r8_10_sequence);

  return offset;
}


static const per_sequence_t T_complete_23_sequence[] = {
  { &hf_rrc_srb_InformationSetupList_08, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SRB_InformationSetupList2_r8 },
  { &hf_rrc_ul_CommonTransChInfo_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_CommonTransChInfo_r4 },
  { &hf_rrc_ul_AddReconfTransChInfoList_03, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_AddReconfTransChInfoList_r8 },
  { &hf_rrc_dl_CommonTransChInfo_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_CommonTransChInfo_r4 },
  { &hf_rrc_dl_AddReconfTransChInfoList_04, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_AddReconfTransChInfoList_r9 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_complete_23(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_complete_23, T_complete_23_sequence);

  return offset;
}


static const per_sequence_t T_defaultConfig_20_sequence[] = {
  { &hf_rrc_defaultConfigMode, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DefaultConfigMode },
  { &hf_rrc_defaultConfigIdentity_03, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DefaultConfigIdentity_r6 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_defaultConfig_20(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_defaultConfig_20, T_defaultConfig_20_sequence);

  return offset;
}


static const value_string rrc_T_preConfigMode_20_vals[] = {
  {   0, "predefinedConfigIdentity" },
  {   1, "defaultConfig" },
  { 0, NULL }
};

static const per_choice_t T_preConfigMode_20_choice[] = {
  {   0, &hf_rrc_predefinedConfigIdentity, ASN1_NO_EXTENSIONS     , dissect_rrc_PredefinedConfigIdentity },
  {   1, &hf_rrc_defaultConfig_20, ASN1_NO_EXTENSIONS     , dissect_rrc_T_defaultConfig_20 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_preConfigMode_20(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_preConfigMode_20, T_preConfigMode_20_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t T_preconfiguration_20_sequence[] = {
  { &hf_rrc_preConfigMode_20, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_preConfigMode_20 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_preconfiguration_20(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_preconfiguration_20, T_preconfiguration_20_sequence);

  return offset;
}


static const value_string rrc_T_specificationMode_23_vals[] = {
  {   0, "complete" },
  {   1, "preconfiguration" },
  { 0, NULL }
};

static const per_choice_t T_specificationMode_23_choice[] = {
  {   0, &hf_rrc_complete_23     , ASN1_NO_EXTENSIONS     , dissect_rrc_T_complete_23 },
  {   1, &hf_rrc_preconfiguration_20, ASN1_NO_EXTENSIONS     , dissect_rrc_T_preconfiguration_20 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_specificationMode_23(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_specificationMode_23, T_specificationMode_23_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t RRCConnectionSetup_r9_IEs_sequence[] = {
  { &hf_rrc_activationTime  , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_ActivationTime },
  { &hf_rrc_new_U_RNTI      , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_U_RNTI },
  { &hf_rrc_new_c_RNTI      , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_C_RNTI },
  { &hf_rrc_new_H_RNTI      , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_H_RNTI },
  { &hf_rrc_newPrimary_E_RNTI, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_E_RNTI },
  { &hf_rrc_newSecondary_E_RNTI, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_E_RNTI },
  { &hf_rrc_rrc_StateIndicator, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RRC_StateIndicator },
  { &hf_rrc_utran_DRX_CycleLengthCoeff_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UTRAN_DRX_CycleLengthCoefficient_r7 },
  { &hf_rrc_capabilityUpdateRequirement_04, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CapabilityUpdateRequirement_r8 },
  { &hf_rrc_supportForChangeOfUE_Capability, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { &hf_rrc_defaultConfigForCellFACH, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DefaultConfigForCellFACH },
  { &hf_rrc_specificationMode_23, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_specificationMode_23 },
  { &hf_rrc_frequencyInfo   , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_FrequencyInfo },
  { &hf_rrc_multi_frequencyInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_Multi_frequencyInfo_LCR_r7 },
  { &hf_rrc_dtx_drx_TimingInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DTX_DRX_TimingInfo_r7 },
  { &hf_rrc_dtx_drx_Info    , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DTX_DRX_Info_r7 },
  { &hf_rrc_hs_scch_LessInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_HS_SCCH_LessInfo_r7 },
  { &hf_rrc_maxAllowedUL_TX_Power, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MaxAllowedUL_TX_Power },
  { &hf_rrc_ul_DPCH_Info_01 , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_DPCH_Info_r7 },
  { &hf_rrc_ul_EDCH_Information_04, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_EDCH_Information_r9 },
  { &hf_rrc_dl_HSPDSCH_Information_06, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_HSPDSCH_Information_r9 },
  { &hf_rrc_dl_CommonInformation_05, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_CommonInformation_r8 },
  { &hf_rrc_dl_InformationPerRL_List_05, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_InformationPerRL_List_r8 },
  { &hf_rrc_dl_SecondaryCellInfoFDD_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_SecondaryCellInfoFDD_r9 },
  { &hf_rrc_sps_Information_TDD128, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_SPS_Information_TDD128_r8 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_RRCConnectionSetup_r9_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_RRCConnectionSetup_r9_IEs, RRCConnectionSetup_r9_IEs_sequence);

  return offset;
}


static const per_sequence_t RRCConnectionSetup_v950ext_IEs_sequence[] = {
  { &hf_rrc_secondaryCellMimoParameters, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_SecondaryCellMIMOparametersFDD_v950ext },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_RRCConnectionSetup_v950ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_RRCConnectionSetup_v950ext_IEs, RRCConnectionSetup_v950ext_IEs_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions_132_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_nonCriticalExtensions_132(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_nonCriticalExtensions_132, T_nonCriticalExtensions_132_sequence);

  return offset;
}


static const per_sequence_t T_v950NonCriticalExtensions_07_sequence[] = {
  { &hf_rrc_rrcConnectionSetup_v950ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RRCConnectionSetup_v950ext_IEs },
  { &hf_rrc_nonCriticalExtensions_132, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_nonCriticalExtensions_132 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v950NonCriticalExtensions_07(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v950NonCriticalExtensions_07, T_v950NonCriticalExtensions_07_sequence);

  return offset;
}


static const per_sequence_t T_r9_09_sequence[] = {
  { &hf_rrc_rrcConnectionSetup_r9, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RRCConnectionSetup_r9_IEs },
  { &hf_rrc_rrcConnectionSetup_r9_add_ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_BIT_STRING },
  { &hf_rrc_v950NonCriticalExtensions_07, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v950NonCriticalExtensions_07 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_r9_09(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_r9_09, T_r9_09_sequence);

  return offset;
}


static const per_sequence_t T_complete_24_sequence[] = {
  { &hf_rrc_srb_InformationSetupList_08, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SRB_InformationSetupList2_r8 },
  { &hf_rrc_ul_CommonTransChInfo_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_CommonTransChInfo_r4 },
  { &hf_rrc_ul_AddReconfTransChInfoList_03, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_AddReconfTransChInfoList_r8 },
  { &hf_rrc_dl_CommonTransChInfo_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_CommonTransChInfo_r4 },
  { &hf_rrc_dl_AddReconfTransChInfoList_04, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_AddReconfTransChInfoList_r9 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_complete_24(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_complete_24, T_complete_24_sequence);

  return offset;
}


static const per_sequence_t T_defaultConfig_21_sequence[] = {
  { &hf_rrc_defaultConfigMode, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DefaultConfigMode },
  { &hf_rrc_defaultConfigIdentity_03, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DefaultConfigIdentity_r6 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_defaultConfig_21(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_defaultConfig_21, T_defaultConfig_21_sequence);

  return offset;
}


static const value_string rrc_T_preConfigMode_21_vals[] = {
  {   0, "predefinedConfigIdentity" },
  {   1, "defaultConfig" },
  { 0, NULL }
};

static const per_choice_t T_preConfigMode_21_choice[] = {
  {   0, &hf_rrc_predefinedConfigIdentity, ASN1_NO_EXTENSIONS     , dissect_rrc_PredefinedConfigIdentity },
  {   1, &hf_rrc_defaultConfig_21, ASN1_NO_EXTENSIONS     , dissect_rrc_T_defaultConfig_21 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_preConfigMode_21(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_preConfigMode_21, T_preConfigMode_21_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t T_preconfiguration_21_sequence[] = {
  { &hf_rrc_preConfigMode_21, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_preConfigMode_21 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_preconfiguration_21(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_preconfiguration_21, T_preconfiguration_21_sequence);

  return offset;
}


static const value_string rrc_T_specificationMode_24_vals[] = {
  {   0, "complete" },
  {   1, "preconfiguration" },
  { 0, NULL }
};

static const per_choice_t T_specificationMode_24_choice[] = {
  {   0, &hf_rrc_complete_24     , ASN1_NO_EXTENSIONS     , dissect_rrc_T_complete_24 },
  {   1, &hf_rrc_preconfiguration_21, ASN1_NO_EXTENSIONS     , dissect_rrc_T_preconfiguration_21 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_specificationMode_24(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_specificationMode_24, T_specificationMode_24_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t RRCConnectionSetup_r10_IEs_sequence[] = {
  { &hf_rrc_activationTime  , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_ActivationTime },
  { &hf_rrc_new_U_RNTI      , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_U_RNTI },
  { &hf_rrc_new_c_RNTI      , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_C_RNTI },
  { &hf_rrc_new_H_RNTI      , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_H_RNTI },
  { &hf_rrc_newPrimary_E_RNTI, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_E_RNTI },
  { &hf_rrc_newSecondary_E_RNTI, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_E_RNTI },
  { &hf_rrc_rrc_StateIndicator, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RRC_StateIndicator },
  { &hf_rrc_utran_DRX_CycleLengthCoeff_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UTRAN_DRX_CycleLengthCoefficient_r7 },
  { &hf_rrc_capabilityUpdateRequirement_04, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CapabilityUpdateRequirement_r8 },
  { &hf_rrc_supportForChangeOfUE_Capability, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { &hf_rrc_defaultConfigForCellFACH, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DefaultConfigForCellFACH },
  { &hf_rrc_specificationMode_24, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_specificationMode_24 },
  { &hf_rrc_frequencyInfo   , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_FrequencyInfo },
  { &hf_rrc_multi_frequencyInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_Multi_frequencyInfo_LCR_r7 },
  { &hf_rrc_dtx_drx_TimingInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DTX_DRX_TimingInfo_r7 },
  { &hf_rrc_dtx_drx_Info    , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DTX_DRX_Info_r7 },
  { &hf_rrc_hs_scch_LessInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_HS_SCCH_LessInfo_r7 },
  { &hf_rrc_maxAllowedUL_TX_Power, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MaxAllowedUL_TX_Power },
  { &hf_rrc_ul_DPCH_Info_01 , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_DPCH_Info_r7 },
  { &hf_rrc_ul_EDCH_Information_04, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_EDCH_Information_r9 },
  { &hf_rrc_dl_HSPDSCH_Information_06, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_HSPDSCH_Information_r9 },
  { &hf_rrc_dl_CommonInformation_06, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_CommonInformation_r10 },
  { &hf_rrc_dl_InformationPerRL_List_05, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_InformationPerRL_List_r8 },
  { &hf_rrc_dl_SecondaryCellInfoFDD_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_SecondaryCellInfoFDD_r10 },
  { &hf_rrc_additionalDLSecCellInfoListFDD, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_AdditionalDLSecCellInfoListFDD },
  { &hf_rrc_sps_Information_TDD128, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_SPS_Information_TDD128_r8 },
  { &hf_rrc_mu_MIMO_Info_TDD128, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MU_MIMO_Info_TDD128 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_RRCConnectionSetup_r10_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_RRCConnectionSetup_r10_IEs, RRCConnectionSetup_r10_IEs_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions_133_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_nonCriticalExtensions_133(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_nonCriticalExtensions_133, T_nonCriticalExtensions_133_sequence);

  return offset;
}


static const per_sequence_t T_r10_09_sequence[] = {
  { &hf_rrc_rrcConnectionSetup_r10, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RRCConnectionSetup_r10_IEs },
  { &hf_rrc_rrcConnectionSetup_r10_add_ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_BIT_STRING },
  { &hf_rrc_nonCriticalExtensions_133, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_nonCriticalExtensions_133 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_r10_09(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_r10_09, T_r10_09_sequence);

  return offset;
}


static const per_sequence_t T_complete_25_sequence[] = {
  { &hf_rrc_srb_InformationSetupList_08, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SRB_InformationSetupList2_r8 },
  { &hf_rrc_ul_CommonTransChInfo_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_CommonTransChInfo_r4 },
  { &hf_rrc_ul_AddReconfTransChInfoList_03, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_AddReconfTransChInfoList_r8 },
  { &hf_rrc_dl_CommonTransChInfo_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_CommonTransChInfo_r4 },
  { &hf_rrc_dl_AddReconfTransChInfoList_06, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_AddReconfTransChInfoList_r11 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_complete_25(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_complete_25, T_complete_25_sequence);

  return offset;
}


static const per_sequence_t T_defaultConfig_22_sequence[] = {
  { &hf_rrc_defaultConfigMode, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DefaultConfigMode },
  { &hf_rrc_defaultConfigIdentity_03, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DefaultConfigIdentity_r6 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_defaultConfig_22(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_defaultConfig_22, T_defaultConfig_22_sequence);

  return offset;
}


static const value_string rrc_T_preConfigMode_22_vals[] = {
  {   0, "predefinedConfigIdentity" },
  {   1, "defaultConfig" },
  { 0, NULL }
};

static const per_choice_t T_preConfigMode_22_choice[] = {
  {   0, &hf_rrc_predefinedConfigIdentity, ASN1_NO_EXTENSIONS     , dissect_rrc_PredefinedConfigIdentity },
  {   1, &hf_rrc_defaultConfig_22, ASN1_NO_EXTENSIONS     , dissect_rrc_T_defaultConfig_22 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_preConfigMode_22(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_preConfigMode_22, T_preConfigMode_22_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t T_preconfiguration_22_sequence[] = {
  { &hf_rrc_preConfigMode_22, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_preConfigMode_22 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_preconfiguration_22(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_preconfiguration_22, T_preconfiguration_22_sequence);

  return offset;
}


static const value_string rrc_T_specificationMode_25_vals[] = {
  {   0, "complete" },
  {   1, "preconfiguration" },
  { 0, NULL }
};

static const per_choice_t T_specificationMode_25_choice[] = {
  {   0, &hf_rrc_complete_25     , ASN1_NO_EXTENSIONS     , dissect_rrc_T_complete_25 },
  {   1, &hf_rrc_preconfiguration_22, ASN1_NO_EXTENSIONS     , dissect_rrc_T_preconfiguration_22 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_specificationMode_25(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_specificationMode_25, T_specificationMode_25_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t RRCConnectionSetup_r11_IEs_sequence[] = {
  { &hf_rrc_activationTime  , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_ActivationTime },
  { &hf_rrc_new_U_RNTI      , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_U_RNTI },
  { &hf_rrc_new_c_RNTI      , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_C_RNTI },
  { &hf_rrc_new_H_RNTI      , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_H_RNTI },
  { &hf_rrc_newPrimary_E_RNTI, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_E_RNTI },
  { &hf_rrc_newSecondary_E_RNTI, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_E_RNTI },
  { &hf_rrc_rrc_StateIndicator, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RRC_StateIndicator },
  { &hf_rrc_utran_DRX_CycleLengthCoeff_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UTRAN_DRX_CycleLengthCoefficient_r7 },
  { &hf_rrc_capabilityUpdateRequirement_04, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CapabilityUpdateRequirement_r8 },
  { &hf_rrc_supportForChangeOfUE_Capability, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { &hf_rrc_defaultConfigForCellFACH, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DefaultConfigForCellFACH },
  { &hf_rrc_specificationMode_25, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_specificationMode_25 },
  { &hf_rrc_frequencyInfo   , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_FrequencyInfo },
  { &hf_rrc_multi_frequencyInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_Multi_frequencyInfo_LCR_r7 },
  { &hf_rrc_dtx_drx_TimingInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DTX_DRX_TimingInfo_r7 },
  { &hf_rrc_dtx_drx_Info    , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DTX_DRX_Info_r7 },
  { &hf_rrc_hs_scch_LessInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_HS_SCCH_LessInfo_r7 },
  { &hf_rrc_maxAllowedUL_TX_Power, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MaxAllowedUL_TX_Power },
  { &hf_rrc_ul_DPCH_Info_02 , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_DPCH_Info_r11 },
  { &hf_rrc_ul_EDCH_Information_05, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_EDCH_Information_r11 },
  { &hf_rrc_dl_HSPDSCH_Information_07, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_HSPDSCH_Information_r11 },
  { &hf_rrc_dl_CommonInformation_06, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_CommonInformation_r10 },
  { &hf_rrc_dl_InformationPerRL_List_05, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_InformationPerRL_List_r8 },
  { &hf_rrc_dl_SecondaryCellInfoFDD_03, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_SecondaryCellInfoFDD_r11 },
  { &hf_rrc_additionalDLSecCellInfoListFDD_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_AdditionalDLSecCellInfoListFDD_r11 },
  { &hf_rrc_additionalDLSecCellInfoListFDD2, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_AdditionalDLSecCellInfoListFDD2_r11 },
  { &hf_rrc_commonERGCHInfoFDD, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CommonERGCHInfoFDD },
  { &hf_rrc_sps_Information_TDD128, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_SPS_Information_TDD128_r8 },
  { &hf_rrc_mu_MIMO_Info_TDD128, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MU_MIMO_Info_TDD128 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_RRCConnectionSetup_r11_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_RRCConnectionSetup_r11_IEs, RRCConnectionSetup_r11_IEs_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions_134_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_nonCriticalExtensions_134(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_nonCriticalExtensions_134, T_nonCriticalExtensions_134_sequence);

  return offset;
}


static const per_sequence_t T_r11_10_sequence[] = {
  { &hf_rrc_rrcConnectionSetup_r11, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RRCConnectionSetup_r11_IEs },
  { &hf_rrc_rrcConnectionSetup_r11_add_ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_BIT_STRING },
  { &hf_rrc_nonCriticalExtensions_134, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_nonCriticalExtensions_134 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_r11_10(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_r11_10, T_r11_10_sequence);

  return offset;
}


static const per_sequence_t T_criticalExtensions_106_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_criticalExtensions_106(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_criticalExtensions_106, T_criticalExtensions_106_sequence);

  return offset;
}


static const value_string rrc_T_criticalExtensions_105_vals[] = {
  {   0, "r11" },
  {   1, "criticalExtensions" },
  { 0, NULL }
};

static const per_choice_t T_criticalExtensions_105_choice[] = {
  {   0, &hf_rrc_r11_10          , ASN1_NO_EXTENSIONS     , dissect_rrc_T_r11_10 },
  {   1, &hf_rrc_criticalExtensions_107, ASN1_NO_EXTENSIONS     , dissect_rrc_T_criticalExtensions_106 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_criticalExtensions_105(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_criticalExtensions_105, T_criticalExtensions_105_choice,
                                 NULL);

  return offset;
}


static const value_string rrc_T_criticalExtensions_104_vals[] = {
  {   0, "r10" },
  {   1, "criticalExtensions" },
  { 0, NULL }
};

static const per_choice_t T_criticalExtensions_104_choice[] = {
  {   0, &hf_rrc_r10_09          , ASN1_NO_EXTENSIONS     , dissect_rrc_T_r10_09 },
  {   1, &hf_rrc_criticalExtensions_106, ASN1_NO_EXTENSIONS     , dissect_rrc_T_criticalExtensions_105 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_criticalExtensions_104(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_criticalExtensions_104, T_criticalExtensions_104_choice,
                                 NULL);

  return offset;
}


static const value_string rrc_T_criticalExtensions_103_vals[] = {
  {   0, "r9" },
  {   1, "criticalExtensions" },
  { 0, NULL }
};

static const per_choice_t T_criticalExtensions_103_choice[] = {
  {   0, &hf_rrc_r9_09           , ASN1_NO_EXTENSIONS     , dissect_rrc_T_r9_09 },
  {   1, &hf_rrc_criticalExtensions_105, ASN1_NO_EXTENSIONS     , dissect_rrc_T_criticalExtensions_104 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_criticalExtensions_103(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_criticalExtensions_103, T_criticalExtensions_103_choice,
                                 NULL);

  return offset;
}


static const value_string rrc_T_criticalExtensions_102_vals[] = {
  {   0, "r8" },
  {   1, "criticalExtensions" },
  { 0, NULL }
};

static const per_choice_t T_criticalExtensions_102_choice[] = {
  {   0, &hf_rrc_r8_10           , ASN1_NO_EXTENSIONS     , dissect_rrc_T_r8_10 },
  {   1, &hf_rrc_criticalExtensions_104, ASN1_NO_EXTENSIONS     , dissect_rrc_T_criticalExtensions_103 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_criticalExtensions_102(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_criticalExtensions_102, T_criticalExtensions_102_choice,
                                 NULL);

  return offset;
}


static const value_string rrc_T_criticalExtensions_101_vals[] = {
  {   0, "r7" },
  {   1, "criticalExtensions" },
  { 0, NULL }
};

static const per_choice_t T_criticalExtensions_101_choice[] = {
  {   0, &hf_rrc_r7_09           , ASN1_NO_EXTENSIONS     , dissect_rrc_T_r7_09 },
  {   1, &hf_rrc_criticalExtensions_103, ASN1_NO_EXTENSIONS     , dissect_rrc_T_criticalExtensions_102 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_criticalExtensions_101(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_criticalExtensions_101, T_criticalExtensions_101_choice,
                                 NULL);

  return offset;
}


static const value_string rrc_T_criticalExtensions_100_vals[] = {
  {   0, "r6" },
  {   1, "criticalExtensions" },
  { 0, NULL }
};

static const per_choice_t T_criticalExtensions_100_choice[] = {
  {   0, &hf_rrc_r6_10           , ASN1_NO_EXTENSIONS     , dissect_rrc_T_r6_10 },
  {   1, &hf_rrc_criticalExtensions_102, ASN1_NO_EXTENSIONS     , dissect_rrc_T_criticalExtensions_101 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_criticalExtensions_100(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_criticalExtensions_100, T_criticalExtensions_100_choice,
                                 NULL);

  return offset;
}


static const value_string rrc_T_criticalExtensions_99_vals[] = {
  {   0, "r5" },
  {   1, "criticalExtensions" },
  { 0, NULL }
};

static const per_choice_t T_criticalExtensions_99_choice[] = {
  {   0, &hf_rrc_r5_09           , ASN1_NO_EXTENSIONS     , dissect_rrc_T_r5_09 },
  {   1, &hf_rrc_criticalExtensions_101, ASN1_NO_EXTENSIONS     , dissect_rrc_T_criticalExtensions_100 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_criticalExtensions_99(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_criticalExtensions_99, T_criticalExtensions_99_choice,
                                 NULL);

  return offset;
}


static const value_string rrc_T_criticalExtensions_98_vals[] = {
  {   0, "r4" },
  {   1, "criticalExtensions" },
  { 0, NULL }
};

static const per_choice_t T_criticalExtensions_98_choice[] = {
  {   0, &hf_rrc_r4_11           , ASN1_NO_EXTENSIONS     , dissect_rrc_T_r4_11 },
  {   1, &hf_rrc_criticalExtensions_100, ASN1_NO_EXTENSIONS     , dissect_rrc_T_criticalExtensions_99 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_criticalExtensions_98(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_criticalExtensions_98, T_criticalExtensions_98_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t T_later_than_r3_18_sequence[] = {
  { &hf_rrc_initialUE_Identity, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_InitialUE_Identity },
  { &hf_rrc_rrc_TransactionIdentifier, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RRC_TransactionIdentifier },
  { &hf_rrc_criticalExtensions_99, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_criticalExtensions_98 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_later_than_r3_18(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_later_than_r3_18, T_later_than_r3_18_sequence);

  return offset;
}


static const value_string rrc_RRCConnectionSetup_vals[] = {
  {   0, "r3" },
  {   1, "later-than-r3" },
  { 0, NULL }
};

static const per_choice_t RRCConnectionSetup_choice[] = {
  {   0, &hf_rrc_r3_20           , ASN1_NO_EXTENSIONS     , dissect_rrc_T_r3_20 },
  {   1, &hf_rrc_later_than_r3_18, ASN1_NO_EXTENSIONS     , dissect_rrc_T_later_than_r3_18 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_RRCConnectionSetup(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
#line 381 "../../asn1/rrc/rrc.cnf"
  col_append_str(actx->pinfo->cinfo, COL_INFO, "RRCConnectionSetup");
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_RRCConnectionSetup, RRCConnectionSetup_choice,
                                 NULL);




  return offset;
}


static const per_sequence_t URAUpdateConfirm_CCCH_r3_IEs_sequence[] = {
  { &hf_rrc_u_RNTI          , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_U_RNTI },
  { &hf_rrc_uraUpdateConfirm_02, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_URAUpdateConfirm_r3_IEs },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_URAUpdateConfirm_CCCH_r3_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_URAUpdateConfirm_CCCH_r3_IEs, URAUpdateConfirm_CCCH_r3_IEs_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions_172_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_nonCriticalExtensions_172(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_nonCriticalExtensions_172, T_nonCriticalExtensions_172_sequence);

  return offset;
}


static const per_sequence_t T_v860NonCriticalExtensions_21_sequence[] = {
  { &hf_rrc_uraUpdateConfirm_v860ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_URAUpdateConfirm_v860ext_IEs },
  { &hf_rrc_nonCriticalExtensions_172, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_nonCriticalExtensions_172 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v860NonCriticalExtensions_21(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v860NonCriticalExtensions_21, T_v860NonCriticalExtensions_21_sequence);

  return offset;
}


static const per_sequence_t T_v690NonCriticalExtensions_45_sequence[] = {
  { &hf_rrc_uraUpdateConfirm_v690ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_URAUpdateConfirm_v690ext_IEs },
  { &hf_rrc_v860NonCriticalExtensions_21, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v860NonCriticalExtensions_21 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v690NonCriticalExtensions_45(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v690NonCriticalExtensions_45, T_v690NonCriticalExtensions_45_sequence);

  return offset;
}


static const per_sequence_t T_laterNonCriticalExtensions_63_sequence[] = {
  { &hf_rrc_uraUpdateConfirm_CCCH_r3_add_ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_BIT_STRING },
  { &hf_rrc_v690NonCriticalExtensions_45, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v690NonCriticalExtensions_45 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_laterNonCriticalExtensions_63(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_laterNonCriticalExtensions_63, T_laterNonCriticalExtensions_63_sequence);

  return offset;
}


static const per_sequence_t T_r3_29_sequence[] = {
  { &hf_rrc_uraUpdateConfirm_CCCH_r3, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_URAUpdateConfirm_CCCH_r3_IEs },
  { &hf_rrc_laterNonCriticalExtensions_63, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_laterNonCriticalExtensions_63 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_r3_29(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_r3_29, T_r3_29_sequence);

  return offset;
}


static const per_sequence_t T_criticalExtensions_129_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_criticalExtensions_129(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_criticalExtensions_129, T_criticalExtensions_129_sequence);

  return offset;
}


static const per_sequence_t T_later_than_r3_26_sequence[] = {
  { &hf_rrc_u_RNTI          , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_U_RNTI },
  { &hf_rrc_rrc_TransactionIdentifier, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RRC_TransactionIdentifier },
  { &hf_rrc_criticalExtensions_130, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_criticalExtensions_129 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_later_than_r3_26(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_later_than_r3_26, T_later_than_r3_26_sequence);

  return offset;
}


static const value_string rrc_URAUpdateConfirm_CCCH_vals[] = {
  {   0, "r3" },
  {   1, "later-than-r3" },
  { 0, NULL }
};

static const per_choice_t URAUpdateConfirm_CCCH_choice[] = {
  {   0, &hf_rrc_r3_29           , ASN1_NO_EXTENSIONS     , dissect_rrc_T_r3_29 },
  {   1, &hf_rrc_later_than_r3_26, ASN1_NO_EXTENSIONS     , dissect_rrc_T_later_than_r3_26 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_URAUpdateConfirm_CCCH(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
#line 491 "../../asn1/rrc/rrc.cnf"
  col_append_str(actx->pinfo->cinfo, COL_INFO, "URAUpdateConfirm-CCCH");
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_URAUpdateConfirm_CCCH, URAUpdateConfirm_CCCH_choice,
                                 NULL);




  return offset;
}


static const value_string rrc_DL_CCCH_MessageType_vals[] = {
  {   0, "cellUpdateConfirm" },
  {   1, "rrcConnectionReject" },
  {   2, "rrcConnectionRelease" },
  {   3, "rrcConnectionSetup" },
  {   4, "uraUpdateConfirm" },
  {   5, "etwsPrimaryNotificationWithSecurity" },
  {   6, "spare2" },
  {   7, "spare1" },
  { 0, NULL }
};

static const per_choice_t DL_CCCH_MessageType_choice[] = {
  {   0, &hf_rrc_cellUpdateConfirm_01, ASN1_NO_EXTENSIONS     , dissect_rrc_CellUpdateConfirm_CCCH },
  {   1, &hf_rrc_rrcConnectionReject, ASN1_NO_EXTENSIONS     , dissect_rrc_RRCConnectionReject },
  {   2, &hf_rrc_rrcConnectionRelease_01, ASN1_NO_EXTENSIONS     , dissect_rrc_RRCConnectionRelease_CCCH },
  {   3, &hf_rrc_rrcConnectionSetup, ASN1_NO_EXTENSIONS     , dissect_rrc_RRCConnectionSetup },
  {   4, &hf_rrc_uraUpdateConfirm_01, ASN1_NO_EXTENSIONS     , dissect_rrc_URAUpdateConfirm_CCCH },
  {   5, &hf_rrc_etwsPrimaryNotificationWithSecurity, ASN1_NO_EXTENSIONS     , dissect_rrc_ETWSPrimaryNotificationWithSecurity },
  {   6, &hf_rrc_spare2          , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   7, &hf_rrc_spare1          , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_DL_CCCH_MessageType(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_DL_CCCH_MessageType, DL_CCCH_MessageType_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t DL_CCCH_Message_sequence[] = {
  { &hf_rrc_integrityCheckInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_IntegrityCheckInfo },
  { &hf_rrc_message_02      , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DL_CCCH_MessageType },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_DL_CCCH_Message(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_DL_CCCH_Message, DL_CCCH_Message_sequence);

  return offset;
}


static const value_string rrc_CellUpdateCause_vals[] = {
  {   0, "cellReselection" },
  {   1, "periodicalCellUpdate" },
  {   2, "uplinkDataTransmission" },
  {   3, "utran-pagingResponse" },
  {   4, "re-enteredServiceArea" },
  {   5, "radiolinkFailure" },
  {   6, "rlc-unrecoverableError" },
  {   7, "cellUpdateCause-ext" },
  { 0, NULL }
};


static int
dissect_rrc_CellUpdateCause(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     8, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t FailureCauseWithProtErrTrId_sequence[] = {
  { &hf_rrc_rrc_TransactionIdentifier, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RRC_TransactionIdentifier },
  { &hf_rrc_failureCause    , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_FailureCauseWithProtErr },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_FailureCauseWithProtErrTrId(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_FailureCauseWithProtErrTrId, FailureCauseWithProtErrTrId_sequence);

  return offset;
}


static const per_sequence_t Rb_timer_indicator_sequence[] = {
  { &hf_rrc_t314_expired    , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { &hf_rrc_t315_expired    , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_Rb_timer_indicator(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_Rb_timer_indicator, Rb_timer_indicator_sequence);

  return offset;
}



static int
dissect_rrc_T_cellUpdate_r3_add_ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_bit_string_containing_pdu_new(tvb, offset, actx, tree, hf_index,
                                                              NO_BOUND, NO_BOUND, FALSE, dissect_CellUpdate_r3_add_ext_IEs_PDU);

  return offset;
}


static const per_sequence_t CellUpdate_v590ext_sequence[] = {
  { &hf_rrc_establishmentCause, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_EstablishmentCause },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_CellUpdate_v590ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_CellUpdate_v590ext, CellUpdate_v590ext_sequence);

  return offset;
}


static const value_string rrc_CellUpdateCause_ext_vals[] = {
  {   0, "mbms-Reception" },
  {   1, "mbms-PTP-RB-Request" },
  {   2, "dummy" },
  {   3, "spare1" },
  { 0, NULL }
};


static int
dissect_rrc_CellUpdateCause_ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     4, NULL, FALSE, 0, NULL);

  return offset;
}


static const value_string rrc_T_trafficVolumeIndicator_vals[] = {
  {   0, "true" },
  { 0, NULL }
};


static int
dissect_rrc_T_trafficVolumeIndicator(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}


static const value_string rrc_T_reconfigurationStatusIndicator_vals[] = {
  {   0, "true" },
  { 0, NULL }
};


static int
dissect_rrc_T_reconfigurationStatusIndicator(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t CellUpdate_v690ext_IEs_sequence[] = {
  { &hf_rrc_cellUpdateCause_ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CellUpdateCause_ext },
  { &hf_rrc_trafficVolumeIndicator, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_trafficVolumeIndicator },
  { &hf_rrc_measuredResultsOnRACHinterFreq, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MeasuredResultsOnRACHinterFreq },
  { &hf_rrc_reconfigurationStatusIndicator, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_reconfigurationStatusIndicator },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_CellUpdate_v690ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_CellUpdate_v690ext_IEs, CellUpdate_v690ext_IEs_sequence);

  return offset;
}



static int
dissect_rrc_MBMS_ShortTransmissionID(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            1U, maxMBMSservUnmodif, NULL, FALSE);

  return offset;
}


static const per_sequence_t MBMS_SelectedServicesListShort_sequence_of[1] = {
  { &hf_rrc_MBMS_SelectedServicesListShort_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MBMS_ShortTransmissionID },
};

static int
dissect_rrc_MBMS_SelectedServicesListShort(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_MBMS_SelectedServicesListShort, MBMS_SelectedServicesListShort_sequence_of,
                                                  1, maxMBMSservSelect, FALSE);

  return offset;
}


static const per_sequence_t MBMS_SelectedServicesShort_sequence[] = {
  { &hf_rrc_mbms_SelectedServicesList, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MBMS_SelectedServicesListShort },
  { &hf_rrc_modificationPeriodIdentity, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_0_1 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_MBMS_SelectedServicesShort(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_MBMS_SelectedServicesShort, MBMS_SelectedServicesShort_sequence);

  return offset;
}


static const per_sequence_t CellUpdate_v6b0ext_IEs_sequence[] = {
  { &hf_rrc_mbmsSelectedServices, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MBMS_SelectedServicesShort },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_CellUpdate_v6b0ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_CellUpdate_v6b0ext_IEs, CellUpdate_v6b0ext_IEs_sequence);

  return offset;
}


static const value_string rrc_T_csCallType_vals[] = {
  {   0, "speech" },
  {   1, "video" },
  {   2, "other" },
  {   3, "spare" },
  { 0, NULL }
};


static int
dissect_rrc_T_csCallType(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     4, NULL, FALSE, 0, NULL);

  return offset;
}


static const value_string rrc_T_hspdschReception_CellFach_vals[] = {
  {   0, "true" },
  { 0, NULL }
};


static int
dissect_rrc_T_hspdschReception_CellFach(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}


static const value_string rrc_T_capabilityChangeIndicator_vals[] = {
  {   0, "true" },
  { 0, NULL }
};


static int
dissect_rrc_T_capabilityChangeIndicator(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t CellUpdate_v770ext_IEs_sequence[] = {
  { &hf_rrc_csCallType      , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_csCallType },
  { &hf_rrc_hspdschReception_CellFach, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_hspdschReception_CellFach },
  { &hf_rrc_ueMobilityStateIndicator, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_High_MobilityDetected },
  { &hf_rrc_capabilityChangeIndicator, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_capabilityChangeIndicator },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_CellUpdate_v770ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_CellUpdate_v770ext_IEs, CellUpdate_v770ext_IEs_sequence);

  return offset;
}


static const value_string rrc_T_supportOfCommonEDCH_vals[] = {
  {   0, "true" },
  { 0, NULL }
};


static int
dissect_rrc_T_supportOfCommonEDCH(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}


static const value_string rrc_T_supportOfHS_DSCHDRXOperation_vals[] = {
  {   0, "true" },
  { 0, NULL }
};


static int
dissect_rrc_T_supportOfHS_DSCHDRXOperation(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}


static const value_string rrc_T_supportOfMACiis_vals[] = {
  {   0, "true" },
  { 0, NULL }
};


static int
dissect_rrc_T_supportOfMACiis(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}


static const value_string rrc_T_supportOfSPSOperation_vals[] = {
  {   0, "true" },
  { 0, NULL }
};


static int
dissect_rrc_T_supportOfSPSOperation(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}


static const value_string rrc_T_supportOfControlChannelDRXOperation_vals[] = {
  {   0, "true" },
  { 0, NULL }
};


static int
dissect_rrc_T_supportOfControlChannelDRXOperation(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t CellUpdate_v860ext_IEs_sequence[] = {
  { &hf_rrc_supportOfCommonEDCH, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_supportOfCommonEDCH },
  { &hf_rrc_supportOfHS_DSCHDRXOperation, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_supportOfHS_DSCHDRXOperation },
  { &hf_rrc_supportOfMACiis , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_supportOfMACiis },
  { &hf_rrc_supportOfSPSOperation, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_supportOfSPSOperation },
  { &hf_rrc_supportOfControlChannelDRXOperation, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_supportOfControlChannelDRXOperation },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_CellUpdate_v860ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_CellUpdate_v860ext_IEs, CellUpdate_v860ext_IEs_sequence);

  return offset;
}


static const value_string rrc_T_securityRevertStatusIndicator_vals[] = {
  {   0, "revertedBack" },
  {   1, "normalOperation" },
  { 0, NULL }
};


static int
dissect_rrc_T_securityRevertStatusIndicator(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     2, NULL, FALSE, 0, NULL);

  return offset;
}


static const value_string rrc_T_loggedMeasAvailable_vals[] = {
  {   0, "true" },
  { 0, NULL }
};


static int
dissect_rrc_T_loggedMeasAvailable(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}


static const value_string rrc_T_loggedANRResultsAvailable_vals[] = {
  {   0, "true" },
  { 0, NULL }
};


static int
dissect_rrc_T_loggedANRResultsAvailable(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t CellUpdate_va40ext_IEs_sequence[] = {
  { &hf_rrc_securityRevertStatusIndicator, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_securityRevertStatusIndicator },
  { &hf_rrc_loggedMeasAvailable, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_loggedMeasAvailable },
  { &hf_rrc_loggedANRResultsAvailable, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_loggedANRResultsAvailable },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_CellUpdate_va40ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_CellUpdate_va40ext_IEs, CellUpdate_va40ext_IEs_sequence);

  return offset;
}


static const per_sequence_t CellUpdate_vbxyext_IEs_sequence[] = {
  { &hf_rrc_measuredResultsOnRACHeutraFreq, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MeasuredResultsOnRACHeutraFreq },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_CellUpdate_vbxyext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_CellUpdate_vbxyext_IEs, CellUpdate_vbxyext_IEs_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions_12_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_nonCriticalExtensions_12(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_nonCriticalExtensions_12, T_nonCriticalExtensions_12_sequence);

  return offset;
}


static const per_sequence_t T_vbxyNonCriticalExtensions_01_sequence[] = {
  { &hf_rrc_cellUpdate_vbxyext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CellUpdate_vbxyext_IEs },
  { &hf_rrc_nonCriticalExtensions_12, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_nonCriticalExtensions_12 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_vbxyNonCriticalExtensions_01(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_vbxyNonCriticalExtensions_01, T_vbxyNonCriticalExtensions_01_sequence);

  return offset;
}


static const per_sequence_t T_va40NonCriticalExtensions_sequence[] = {
  { &hf_rrc_cellUpdate_va40ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CellUpdate_va40ext_IEs },
  { &hf_rrc_vbxyNonCriticalExtensions_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_vbxyNonCriticalExtensions_01 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_va40NonCriticalExtensions(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_va40NonCriticalExtensions, T_va40NonCriticalExtensions_sequence);

  return offset;
}


static const per_sequence_t T_v860NonCriticalExtensions_sequence[] = {
  { &hf_rrc_cellUpdate_v860ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CellUpdate_v860ext_IEs },
  { &hf_rrc_va40NonCriticalExtensions, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_va40NonCriticalExtensions },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v860NonCriticalExtensions(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v860NonCriticalExtensions, T_v860NonCriticalExtensions_sequence);

  return offset;
}


static const per_sequence_t T_v770NonCriticalExtensions_sequence[] = {
  { &hf_rrc_cellUpdate_v770ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CellUpdate_v770ext_IEs },
  { &hf_rrc_v860NonCriticalExtensions, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v860NonCriticalExtensions },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v770NonCriticalExtensions(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v770NonCriticalExtensions, T_v770NonCriticalExtensions_sequence);

  return offset;
}


static const per_sequence_t T_v6b0NonCriticalExtensions_01_sequence[] = {
  { &hf_rrc_cellUpdate_v6b0ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CellUpdate_v6b0ext_IEs },
  { &hf_rrc_v770NonCriticalExtensions, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v770NonCriticalExtensions },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v6b0NonCriticalExtensions_01(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v6b0NonCriticalExtensions_01, T_v6b0NonCriticalExtensions_01_sequence);

  return offset;
}


static const per_sequence_t T_v690NonCriticalExtensions_01_sequence[] = {
  { &hf_rrc_cellUpdate_v690ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CellUpdate_v690ext_IEs },
  { &hf_rrc_v6b0NonCriticalExtensions_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v6b0NonCriticalExtensions_01 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v690NonCriticalExtensions_01(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v690NonCriticalExtensions_01, T_v690NonCriticalExtensions_01_sequence);

  return offset;
}


static const per_sequence_t T_v590NonCriticalExtensions_02_sequence[] = {
  { &hf_rrc_cellUpdate_v590ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CellUpdate_v590ext },
  { &hf_rrc_v690NonCriticalExtensions_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v690NonCriticalExtensions_01 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v590NonCriticalExtensions_02(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v590NonCriticalExtensions_02, T_v590NonCriticalExtensions_02_sequence);

  return offset;
}


static const per_sequence_t T_laterNonCriticalExtensions_06_sequence[] = {
  { &hf_rrc_cellUpdate_r3_add_ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_cellUpdate_r3_add_ext },
  { &hf_rrc_v590NonCriticalExtensions_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v590NonCriticalExtensions_02 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_laterNonCriticalExtensions_06(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_laterNonCriticalExtensions_06, T_laterNonCriticalExtensions_06_sequence);

  return offset;
}


static const per_sequence_t CellUpdate_sequence[] = {
  { &hf_rrc_u_RNTI          , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_U_RNTI },
  { &hf_rrc_startList       , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_STARTList },
  { &hf_rrc_am_RLC_ErrorIndicationRb2_3or4, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { &hf_rrc_am_RLC_ErrorIndicationRb5orAbove, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { &hf_rrc_cellUpdateCause , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CellUpdateCause },
  { &hf_rrc_failureCause_01 , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_FailureCauseWithProtErrTrId },
  { &hf_rrc_rb_timer_indicator, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_Rb_timer_indicator },
  { &hf_rrc_measuredResultsOnRACH, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MeasuredResultsOnRACH },
  { &hf_rrc_laterNonCriticalExtensions_06, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_laterNonCriticalExtensions_06 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_CellUpdate(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
#line 189 "../../asn1/rrc/rrc.cnf"
  col_append_str(actx->pinfo->cinfo, COL_INFO, "CellUpdate");
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_CellUpdate, CellUpdate_sequence);




  return offset;
}


static const value_string rrc_ProtocolErrorIndicator_vals[] = {
  {   0, "noError" },
  {   1, "errorOccurred" },
  { 0, NULL }
};


static int
dissect_rrc_ProtocolErrorIndicator(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     2, NULL, FALSE, 0, NULL);

  return offset;
}



static int
dissect_rrc_UESpecificBehaviourInformation1idle(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_bit_string(tvb, offset, actx, tree, hf_index,
                                     4, 4, FALSE, NULL);

  return offset;
}


static const per_sequence_t RRCConnectionRequest_v3d0ext_IEs_sequence[] = {
  { &hf_rrc_uESpecificBehaviourInformation1idle, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UESpecificBehaviourInformation1idle },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_RRCConnectionRequest_v3d0ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_RRCConnectionRequest_v3d0ext_IEs, RRCConnectionRequest_v3d0ext_IEs_sequence);

  return offset;
}


static const per_sequence_t RRCConnectionRequest_v4b0ext_IEs_sequence[] = {
  { &hf_rrc_accessStratumReleaseIndicator, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_AccessStratumReleaseIndicator },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_RRCConnectionRequest_v4b0ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_RRCConnectionRequest_v4b0ext_IEs, RRCConnectionRequest_v4b0ext_IEs_sequence);

  return offset;
}


static const per_sequence_t RRCConnectionRequest_v590ext_IEs_sequence[] = {
  { &hf_rrc_predefinedConfigStatusInfo, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_RRCConnectionRequest_v590ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_RRCConnectionRequest_v590ext_IEs, RRCConnectionRequest_v590ext_IEs_sequence);

  return offset;
}


static const value_string rrc_T_ueCapabilityIndication_vals[] = {
  {   0, "hsdch" },
  {   1, "hsdch-edch" },
  { 0, NULL }
};


static int
dissect_rrc_T_ueCapabilityIndication(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     2, NULL, FALSE, 0, NULL);

  return offset;
}


static const value_string rrc_T_csCallType_02_vals[] = {
  {   0, "speech" },
  {   1, "video" },
  {   2, "other" },
  {   3, "spare" },
  { 0, NULL }
};


static int
dissect_rrc_T_csCallType_02(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     4, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t T_cs_domain_sequence[] = {
  { &hf_rrc_csCallType_02   , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_csCallType_02 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_cs_domain(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_cs_domain, T_cs_domain_sequence);

  return offset;
}


static const value_string rrc_T_domainIndicator_vals[] = {
  {   0, "cs-domain" },
  {   1, "ps-domain" },
  { 0, NULL }
};

static const per_choice_t T_domainIndicator_choice[] = {
  {   0, &hf_rrc_cs_domain       , ASN1_NO_EXTENSIONS     , dissect_rrc_T_cs_domain },
  {   1, &hf_rrc_ps_domain       , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_domainIndicator(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_domainIndicator, T_domainIndicator_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t RRCConnectionRequest_v690ext_IEs_sequence[] = {
  { &hf_rrc_ueCapabilityIndication, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_ueCapabilityIndication },
  { &hf_rrc_measuredResultsOnRACHinterFreq, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MeasuredResultsOnRACHinterFreq },
  { &hf_rrc_domainIndicator , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_domainIndicator },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_RRCConnectionRequest_v690ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_RRCConnectionRequest_v690ext_IEs, RRCConnectionRequest_v690ext_IEs_sequence);

  return offset;
}


static const per_sequence_t RRCConnectionRequest_v6b0ext_IEs_sequence[] = {
  { &hf_rrc_mbmsSelectedServices, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MBMS_SelectedServicesShort },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_RRCConnectionRequest_v6b0ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_RRCConnectionRequest_v6b0ext_IEs, RRCConnectionRequest_v6b0ext_IEs_sequence);

  return offset;
}


static const value_string rrc_T_supportForFDPCH_01_vals[] = {
  {   0, "true" },
  { 0, NULL }
};


static int
dissect_rrc_T_supportForFDPCH_01(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t RRCConnectionRequest_v6e0ext_IEs_sequence[] = {
  { &hf_rrc_supportForFDPCH_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_supportForFDPCH_01 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_RRCConnectionRequest_v6e0ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_RRCConnectionRequest_v6e0ext_IEs, RRCConnectionRequest_v6e0ext_IEs_sequence);

  return offset;
}


static const value_string rrc_T_hspdschReception_CellFach_01_vals[] = {
  {   0, "true" },
  { 0, NULL }
};


static int
dissect_rrc_T_hspdschReception_CellFach_01(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}


static const value_string rrc_T_mac_ehsSupport_vals[] = {
  {   0, "true" },
  { 0, NULL }
};


static int
dissect_rrc_T_mac_ehsSupport(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}


static const value_string rrc_T_discontinuousDpcchTransmission_vals[] = {
  {   0, "true" },
  { 0, NULL }
};


static int
dissect_rrc_T_discontinuousDpcchTransmission(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t RRCConnectionRequest_v770ext_IEs_sequence[] = {
  { &hf_rrc_ueMobilityStateIndicator, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_High_MobilityDetected },
  { &hf_rrc_hspdschReception_CellFach_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_hspdschReception_CellFach_01 },
  { &hf_rrc_mac_ehsSupport  , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_mac_ehsSupport },
  { &hf_rrc_discontinuousDpcchTransmission, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_discontinuousDpcchTransmission },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_RRCConnectionRequest_v770ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_RRCConnectionRequest_v770ext_IEs, RRCConnectionRequest_v770ext_IEs_sequence);

  return offset;
}


static const value_string rrc_T_supportForE_FDPCH_vals[] = {
  {   0, "true" },
  { 0, NULL }
};


static int
dissect_rrc_T_supportForE_FDPCH(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t RRCConnectionRequest_v7b0ext_IEs_sequence[] = {
  { &hf_rrc_supportForE_FDPCH, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_supportForE_FDPCH },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_RRCConnectionRequest_v7b0ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_RRCConnectionRequest_v7b0ext_IEs, RRCConnectionRequest_v7b0ext_IEs_sequence);

  return offset;
}


static const value_string rrc_T_supportOfCommonEDCH_01_vals[] = {
  {   0, "true" },
  { 0, NULL }
};


static int
dissect_rrc_T_supportOfCommonEDCH_01(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}


static const value_string rrc_T_multiCellSupport_vals[] = {
  {   0, "true" },
  { 0, NULL }
};


static int
dissect_rrc_T_multiCellSupport(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t Pre_RedirectionInfo_sequence[] = {
  { &hf_rrc_supportEUTRA_FDD, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { &hf_rrc_supportEUTRA_TDD, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_Pre_RedirectionInfo(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_Pre_RedirectionInfo, Pre_RedirectionInfo_sequence);

  return offset;
}


static const value_string rrc_T_supportOfMACiis_01_vals[] = {
  {   0, "true" },
  { 0, NULL }
};


static int
dissect_rrc_T_supportOfMACiis_01(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}


static const value_string rrc_T_supportOfSPSOperation_01_vals[] = {
  {   0, "true" },
  { 0, NULL }
};


static int
dissect_rrc_T_supportOfSPSOperation_01(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t RRCConnectionRequest_v860ext_IEs_sequence[] = {
  { &hf_rrc_supportOfCommonEDCH_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_supportOfCommonEDCH_01 },
  { &hf_rrc_multiCellSupport, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_multiCellSupport },
  { &hf_rrc_pre_redirectionInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_Pre_RedirectionInfo },
  { &hf_rrc_supportOfMACiis_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_supportOfMACiis_01 },
  { &hf_rrc_supportOfSPSOperation_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_supportOfSPSOperation_01 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_RRCConnectionRequest_v860ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_RRCConnectionRequest_v860ext_IEs, RRCConnectionRequest_v860ext_IEs_sequence);

  return offset;
}


static const value_string rrc_T_supportForCSVoiceoverHSPA_vals[] = {
  {   0, "true" },
  { 0, NULL }
};


static int
dissect_rrc_T_supportForCSVoiceoverHSPA(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t RRCConnectionRequest_v7e0ext_IEs_sequence[] = {
  { &hf_rrc_supportForCSVoiceoverHSPA, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_supportForCSVoiceoverHSPA },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_RRCConnectionRequest_v7e0ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_RRCConnectionRequest_v7e0ext_IEs, RRCConnectionRequest_v7e0ext_IEs_sequence);

  return offset;
}


static const per_sequence_t RRCConnectionRequest_v7g0ext_IEs_sequence[] = {
  { &hf_rrc_measuredResultsOnRACH_v7g0ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MeasuredResultsOnRACH_v7g0ext },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_RRCConnectionRequest_v7g0ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_RRCConnectionRequest_v7g0ext_IEs, RRCConnectionRequest_v7g0ext_IEs_sequence);

  return offset;
}


static const value_string rrc_T_supportOfDualCellMIMO_vals[] = {
  {   0, "true" },
  { 0, NULL }
};


static int
dissect_rrc_T_supportOfDualCellMIMO(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t RRCConnectionRequest_v920ext_IEs_sequence[] = {
  { &hf_rrc_supportOfDualCellMIMO, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_supportOfDualCellMIMO },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_RRCConnectionRequest_v920ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_RRCConnectionRequest_v920ext_IEs, RRCConnectionRequest_v920ext_IEs_sequence);

  return offset;
}


static const value_string rrc_T_systemInformationContainerStoredIndicator_vals[] = {
  {   0, "true" },
  { 0, NULL }
};


static int
dissect_rrc_T_systemInformationContainerStoredIndicator(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t RRCConnectionRequest_v940ext_IEs_sequence[] = {
  { &hf_rrc_systemInformationContainerStoredIndicator, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_systemInformationContainerStoredIndicator },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_RRCConnectionRequest_v940ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_RRCConnectionRequest_v940ext_IEs, RRCConnectionRequest_v940ext_IEs_sequence);

  return offset;
}


static const value_string rrc_T_supportOfMoreThanTwoCells_vals[] = {
  {   0, "higherRate" },
  {   1, "lowerRate" },
  { 0, NULL }
};


static int
dissect_rrc_T_supportOfMoreThanTwoCells(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     2, NULL, FALSE, 0, NULL);

  return offset;
}


static const value_string rrc_T_supportOf1stFrequencyBand_vals[] = {
  {   0, "true" },
  { 0, NULL }
};


static int
dissect_rrc_T_supportOf1stFrequencyBand(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}


static const value_string rrc_T_supportOf2ndFrequencyBand_vals[] = {
  {   0, "true" },
  { 0, NULL }
};


static int
dissect_rrc_T_supportOf2ndFrequencyBand(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t RRCConnectionRequest_va40ext_IEs_sequence[] = {
  { &hf_rrc_supportOfMoreThanTwoCells, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_supportOfMoreThanTwoCells },
  { &hf_rrc_supportOf1stFrequencyBand, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_supportOf1stFrequencyBand },
  { &hf_rrc_supportOf2ndFrequencyBand, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_supportOf2ndFrequencyBand },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_RRCConnectionRequest_va40ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_RRCConnectionRequest_va40ext_IEs, RRCConnectionRequest_va40ext_IEs_sequence);

  return offset;
}


static const value_string rrc_T_cSFBIndication_vals[] = {
  {   0, "true" },
  { 0, NULL }
};


static int
dissect_rrc_T_cSFBIndication(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t RRCConnectionRequest_v9b0ext_IEs_sequence[] = {
  { &hf_rrc_cSFBIndication  , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_cSFBIndication },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_RRCConnectionRequest_v9b0ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_RRCConnectionRequest_v9b0ext_IEs, RRCConnectionRequest_v9b0ext_IEs_sequence);

  return offset;
}


static const per_sequence_t RRCConnectionRequest_vbxyext_IEs_sequence[] = {
  { &hf_rrc_measuredResultsOnRACHeutraFreq, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MeasuredResultsOnRACHeutraFreq },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_RRCConnectionRequest_vbxyext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_RRCConnectionRequest_vbxyext_IEs, RRCConnectionRequest_vbxyext_IEs_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions_125_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_nonCriticalExtensions_125(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_nonCriticalExtensions_125, T_nonCriticalExtensions_125_sequence);

  return offset;
}


static const per_sequence_t T_vbxyNonCriticalExtensions_12_sequence[] = {
  { &hf_rrc_rrcConnectionRequest_vbxyext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RRCConnectionRequest_vbxyext_IEs },
  { &hf_rrc_nonCriticalExtensions_125, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_nonCriticalExtensions_125 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_vbxyNonCriticalExtensions_12(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_vbxyNonCriticalExtensions_12, T_vbxyNonCriticalExtensions_12_sequence);

  return offset;
}


static const per_sequence_t T_v9b0NonCriticalExtensions_sequence[] = {
  { &hf_rrc_rrcConnectionRequest_v9b0ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RRCConnectionRequest_v9b0ext_IEs },
  { &hf_rrc_vbxyNonCriticalExtensions_12, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_vbxyNonCriticalExtensions_12 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v9b0NonCriticalExtensions(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v9b0NonCriticalExtensions, T_v9b0NonCriticalExtensions_sequence);

  return offset;
}


static const per_sequence_t RRCConnectionRequest_NonCriticalExts_va40_IEs_sequence[] = {
  { &hf_rrc_rrcConnectionRequest_va40ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RRCConnectionRequest_va40ext_IEs },
  { &hf_rrc_v9b0NonCriticalExtensions, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v9b0NonCriticalExtensions },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_RRCConnectionRequest_NonCriticalExts_va40_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_RRCConnectionRequest_NonCriticalExts_va40_IEs, RRCConnectionRequest_NonCriticalExts_va40_IEs_sequence);

  return offset;
}


static const per_sequence_t T_v940NonCriticalExtensions_sequence[] = {
  { &hf_rrc_rrcConnectionRequest_v940ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RRCConnectionRequest_v940ext_IEs },
  { &hf_rrc_va40NonCriticalExtensions_11, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RRCConnectionRequest_NonCriticalExts_va40_IEs },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v940NonCriticalExtensions(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v940NonCriticalExtensions, T_v940NonCriticalExtensions_sequence);

  return offset;
}


static const per_sequence_t T_v920NonCriticalExtensions_02_sequence[] = {
  { &hf_rrc_rrcConnectionRequest_v920ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RRCConnectionRequest_v920ext_IEs },
  { &hf_rrc_v940NonCriticalExtensions, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v940NonCriticalExtensions },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v920NonCriticalExtensions_02(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v920NonCriticalExtensions_02, T_v920NonCriticalExtensions_02_sequence);

  return offset;
}


static const per_sequence_t T_v7g0NonCriticalExtensions_14_sequence[] = {
  { &hf_rrc_rrcConnectionRequest_v7g0ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RRCConnectionRequest_v7g0ext_IEs },
  { &hf_rrc_v920NonCriticalExtensions_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v920NonCriticalExtensions_02 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v7g0NonCriticalExtensions_14(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v7g0NonCriticalExtensions_14, T_v7g0NonCriticalExtensions_14_sequence);

  return offset;
}


static const per_sequence_t T_v7e0NonCriticalExtensions_01_sequence[] = {
  { &hf_rrc_rrcConnectionRequest_v7e0ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RRCConnectionRequest_v7e0ext_IEs },
  { &hf_rrc_v7g0NonCriticalExtensions_14, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v7g0NonCriticalExtensions_14 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v7e0NonCriticalExtensions_01(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v7e0NonCriticalExtensions_01, T_v7e0NonCriticalExtensions_01_sequence);

  return offset;
}


static const per_sequence_t T_v860NonCriticalExtensions_16_sequence[] = {
  { &hf_rrc_rrcConnectionRequest_v860ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RRCConnectionRequest_v860ext_IEs },
  { &hf_rrc_v7e0NonCriticalExtensions_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v7e0NonCriticalExtensions_01 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v860NonCriticalExtensions_16(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v860NonCriticalExtensions_16, T_v860NonCriticalExtensions_16_sequence);

  return offset;
}


static const per_sequence_t T_v7b0NonCriticalExtensions_01_sequence[] = {
  { &hf_rrc_rrcConnectionRequest_v7b0ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RRCConnectionRequest_v7b0ext_IEs },
  { &hf_rrc_v860NonCriticalExtensions_16, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v860NonCriticalExtensions_16 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v7b0NonCriticalExtensions_01(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v7b0NonCriticalExtensions_01, T_v7b0NonCriticalExtensions_01_sequence);

  return offset;
}


static const per_sequence_t T_v770NonCriticalExtensions_23_sequence[] = {
  { &hf_rrc_rrcConnectionRequest_v770ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RRCConnectionRequest_v770ext_IEs },
  { &hf_rrc_v7b0NonCriticalExtensions_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v7b0NonCriticalExtensions_01 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v770NonCriticalExtensions_23(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v770NonCriticalExtensions_23, T_v770NonCriticalExtensions_23_sequence);

  return offset;
}


static const per_sequence_t T_v6e0NonCriticalExtensions_01_sequence[] = {
  { &hf_rrc_rrcConnectionRequest_v6e0ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RRCConnectionRequest_v6e0ext_IEs },
  { &hf_rrc_v770NonCriticalExtensions_23, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v770NonCriticalExtensions_23 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v6e0NonCriticalExtensions_01(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v6e0NonCriticalExtensions_01, T_v6e0NonCriticalExtensions_01_sequence);

  return offset;
}


static const per_sequence_t T_v6b0NonCriticalExtensions_09_sequence[] = {
  { &hf_rrc_rrcConnectionRequest_v6b0ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RRCConnectionRequest_v6b0ext_IEs },
  { &hf_rrc_v6e0NonCriticalExtensions_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v6e0NonCriticalExtensions_01 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v6b0NonCriticalExtensions_09(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v6b0NonCriticalExtensions_09, T_v6b0NonCriticalExtensions_09_sequence);

  return offset;
}


static const per_sequence_t T_v690NonCriticalExtensions_31_sequence[] = {
  { &hf_rrc_rrcConnectionRequest_v690ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RRCConnectionRequest_v690ext_IEs },
  { &hf_rrc_v6b0NonCriticalExtensions_10, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v6b0NonCriticalExtensions_09 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v690NonCriticalExtensions_31(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v690NonCriticalExtensions_31, T_v690NonCriticalExtensions_31_sequence);

  return offset;
}


static const per_sequence_t T_v590NonCriticalExtensions_20_sequence[] = {
  { &hf_rrc_rrcConnectionRequest_v590ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RRCConnectionRequest_v590ext_IEs },
  { &hf_rrc_v690NonCriticalExtensions_31, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v690NonCriticalExtensions_31 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v590NonCriticalExtensions_20(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v590NonCriticalExtensions_20, T_v590NonCriticalExtensions_20_sequence);

  return offset;
}


static const per_sequence_t T_v4b0NonCriticalExtensions_10_sequence[] = {
  { &hf_rrc_rrcConnectionRequest_v4b0ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RRCConnectionRequest_v4b0ext_IEs },
  { &hf_rrc_v590NonCriticalExtensions_20, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v590NonCriticalExtensions_20 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v4b0NonCriticalExtensions_10(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v4b0NonCriticalExtensions_10, T_v4b0NonCriticalExtensions_10_sequence);

  return offset;
}


static const per_sequence_t T_v3d0NonCriticalExtensions_sequence[] = {
  { &hf_rrc_rRCConnectionRequest_v3d0ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RRCConnectionRequest_v3d0ext_IEs },
  { &hf_rrc_v4b0NonCriticalExtensions_10, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v4b0NonCriticalExtensions_10 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v3d0NonCriticalExtensions(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v3d0NonCriticalExtensions, T_v3d0NonCriticalExtensions_sequence);

  return offset;
}


static const per_sequence_t RRCConnectionRequest_sequence[] = {
  { &hf_rrc_initialUE_Identity, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_InitialUE_Identity },
  { &hf_rrc_establishmentCause, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_EstablishmentCause },
  { &hf_rrc_protocolErrorIndicator_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_ProtocolErrorIndicator },
  { &hf_rrc_measuredResultsOnRACH, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MeasuredResultsOnRACH },
  { &hf_rrc_v3d0NonCriticalExtensions, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v3d0NonCriticalExtensions },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_RRCConnectionRequest(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
#line 377 "../../asn1/rrc/rrc.cnf"
  col_append_str(actx->pinfo->cinfo, COL_INFO, "RRCConnectionRequest");
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_RRCConnectionRequest, RRCConnectionRequest_sequence);




  return offset;
}


static const value_string rrc_URA_UpdateCause_vals[] = {
  {   0, "changeOfURA" },
  {   1, "periodicURAUpdate" },
  {   2, "dummy" },
  {   3, "spare1" },
  { 0, NULL }
};


static int
dissect_rrc_URA_UpdateCause(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     4, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t T_errorOccurred_sequence[] = {
  { &hf_rrc_rrc_TransactionIdentifier, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RRC_TransactionIdentifier },
  { &hf_rrc_protocolErrorInformation_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_ProtocolErrorInformation },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_errorOccurred(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_errorOccurred, T_errorOccurred_sequence);

  return offset;
}


static const value_string rrc_ProtocolErrorIndicatorWithMoreInfo_vals[] = {
  {   0, "noError" },
  {   1, "errorOccurred" },
  { 0, NULL }
};

static const per_choice_t ProtocolErrorIndicatorWithMoreInfo_choice[] = {
  {   0, &hf_rrc_noError         , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   1, &hf_rrc_errorOccurred   , ASN1_NO_EXTENSIONS     , dissect_rrc_T_errorOccurred },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_ProtocolErrorIndicatorWithMoreInfo(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_ProtocolErrorIndicatorWithMoreInfo, ProtocolErrorIndicatorWithMoreInfo_choice,
                                 NULL);

  return offset;
}



static int
dissect_rrc_T_uraUpdate_r3_add_ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_bit_string_containing_pdu_new(tvb, offset, actx, tree, hf_index,
                                                              NO_BOUND, NO_BOUND, FALSE, dissect_URAUpdate_r3_add_ext_IEs_PDU);

  return offset;
}


static const value_string rrc_T_support_hsdschReception_CellUraPch_vals[] = {
  {   0, "true" },
  { 0, NULL }
};


static int
dissect_rrc_T_support_hsdschReception_CellUraPch(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}


static const value_string rrc_T_support_hsdschReception_CellFach_vals[] = {
  {   0, "true" },
  { 0, NULL }
};


static int
dissect_rrc_T_support_hsdschReception_CellFach(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t UraUpdate_v770ext_IEs_sequence[] = {
  { &hf_rrc_support_hsdschReception_CellUraPch, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_support_hsdschReception_CellUraPch },
  { &hf_rrc_support_hsdschReception_CellFach, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_support_hsdschReception_CellFach },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UraUpdate_v770ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UraUpdate_v770ext_IEs, UraUpdate_v770ext_IEs_sequence);

  return offset;
}


static const value_string rrc_T_supportOfHS_DSCHDRXOperation_01_vals[] = {
  {   0, "true" },
  { 0, NULL }
};


static int
dissect_rrc_T_supportOfHS_DSCHDRXOperation_01(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}


static const value_string rrc_T_supportOfCommonEDCH_02_vals[] = {
  {   0, "true" },
  { 0, NULL }
};


static int
dissect_rrc_T_supportOfCommonEDCH_02(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}


static const value_string rrc_T_supportOfMACiis_02_vals[] = {
  {   0, "true" },
  { 0, NULL }
};


static int
dissect_rrc_T_supportOfMACiis_02(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t URAUpdate_v860ext_IEs_sequence[] = {
  { &hf_rrc_supportOfHS_DSCHDRXOperation_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_supportOfHS_DSCHDRXOperation_01 },
  { &hf_rrc_supportOfCommonEDCH_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_supportOfCommonEDCH_02 },
  { &hf_rrc_supportOfMACiis_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_supportOfMACiis_02 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_URAUpdate_v860ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_URAUpdate_v860ext_IEs, URAUpdate_v860ext_IEs_sequence);

  return offset;
}


static const value_string rrc_T_loggedMeasAvailable_04_vals[] = {
  {   0, "true" },
  { 0, NULL }
};


static int
dissect_rrc_T_loggedMeasAvailable_04(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}


static const value_string rrc_T_loggedANRResultsAvailable_03_vals[] = {
  {   0, "true" },
  { 0, NULL }
};


static int
dissect_rrc_T_loggedANRResultsAvailable_03(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t URAUpdate_va40ext_IEs_sequence[] = {
  { &hf_rrc_loggedMeasAvailable_04, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_loggedMeasAvailable_04 },
  { &hf_rrc_loggedANRResultsAvailable_03, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_loggedANRResultsAvailable_03 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_URAUpdate_va40ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_URAUpdate_va40ext_IEs, URAUpdate_va40ext_IEs_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions_167_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_nonCriticalExtensions_167(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_nonCriticalExtensions_167, T_nonCriticalExtensions_167_sequence);

  return offset;
}


static const per_sequence_t T_va40NonCriticalExtensions_15_sequence[] = {
  { &hf_rrc_uraUpdate_va40ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_URAUpdate_va40ext_IEs },
  { &hf_rrc_nonCriticalExtensions_167, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_nonCriticalExtensions_167 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_va40NonCriticalExtensions_15(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_va40NonCriticalExtensions_15, T_va40NonCriticalExtensions_15_sequence);

  return offset;
}


static const per_sequence_t T_v860NonCriticalExtensions_19_sequence[] = {
  { &hf_rrc_uraUpdate_v860ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_URAUpdate_v860ext_IEs },
  { &hf_rrc_va40NonCriticalExtensions_16, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_va40NonCriticalExtensions_15 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v860NonCriticalExtensions_19(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v860NonCriticalExtensions_19, T_v860NonCriticalExtensions_19_sequence);

  return offset;
}


static const per_sequence_t T_v770NonCriticalExtensions_32_sequence[] = {
  { &hf_rrc_uraUpdate_v770ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UraUpdate_v770ext_IEs },
  { &hf_rrc_v860NonCriticalExtensions_19, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v860NonCriticalExtensions_19 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v770NonCriticalExtensions_32(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v770NonCriticalExtensions_32, T_v770NonCriticalExtensions_32_sequence);

  return offset;
}


static const per_sequence_t T_laterNonCriticalExtensions_61_sequence[] = {
  { &hf_rrc_uraUpdate_r3_add_ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_uraUpdate_r3_add_ext },
  { &hf_rrc_v770NonCriticalExtensions_32, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v770NonCriticalExtensions_32 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_laterNonCriticalExtensions_61(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_laterNonCriticalExtensions_61, T_laterNonCriticalExtensions_61_sequence);

  return offset;
}


static const per_sequence_t URAUpdate_sequence[] = {
  { &hf_rrc_u_RNTI          , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_U_RNTI },
  { &hf_rrc_ura_UpdateCause , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_URA_UpdateCause },
  { &hf_rrc_protocolErrorIndicator, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_ProtocolErrorIndicatorWithMoreInfo },
  { &hf_rrc_laterNonCriticalExtensions_61, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_laterNonCriticalExtensions_61 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_URAUpdate(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
#line 483 "../../asn1/rrc/rrc.cnf"
  col_append_str(actx->pinfo->cinfo, COL_INFO, "URAUpdate");
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_URAUpdate, URAUpdate_sequence);




  return offset;
}


static const value_string rrc_UL_CCCH_MessageType_vals[] = {
  {   0, "cellUpdate" },
  {   1, "rrcConnectionRequest" },
  {   2, "uraUpdate" },
  {   3, "spare" },
  { 0, NULL }
};

static const per_choice_t UL_CCCH_MessageType_choice[] = {
  {   0, &hf_rrc_cellUpdate      , ASN1_NO_EXTENSIONS     , dissect_rrc_CellUpdate },
  {   1, &hf_rrc_rrcConnectionRequest, ASN1_NO_EXTENSIONS     , dissect_rrc_RRCConnectionRequest },
  {   2, &hf_rrc_uraUpdate       , ASN1_NO_EXTENSIONS     , dissect_rrc_URAUpdate },
  {   3, &hf_rrc_spare           , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_UL_CCCH_MessageType(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_UL_CCCH_MessageType, UL_CCCH_MessageType_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t UL_CCCH_Message_sequence[] = {
  { &hf_rrc_integrityCheckInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_IntegrityCheckInfo },
  { &hf_rrc_message_03      , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UL_CCCH_MessageType },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UL_CCCH_Message(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UL_CCCH_Message, UL_CCCH_Message_sequence);

  return offset;
}


static const value_string rrc_CN_PagedUE_Identity_vals[] = {
  {   0, "imsi-GSM-MAP" },
  {   1, "tmsi-GSM-MAP" },
  {   2, "p-TMSI-GSM-MAP" },
  {   3, "imsi-DS-41" },
  {   4, "tmsi-DS-41" },
  {   5, "spare3" },
  {   6, "spare2" },
  {   7, "spare1" },
  { 0, NULL }
};

static const per_choice_t CN_PagedUE_Identity_choice[] = {
  {   0, &hf_rrc_imsi_GSM_MAP    , ASN1_NO_EXTENSIONS     , dissect_rrc_IMSI_GSM_MAP },
  {   1, &hf_rrc_tmsi_GSM_MAP    , ASN1_NO_EXTENSIONS     , dissect_rrc_TMSI_GSM_MAP },
  {   2, &hf_rrc_p_TMSI_GSM_MAP  , ASN1_NO_EXTENSIONS     , dissect_rrc_P_TMSI_GSM_MAP },
  {   3, &hf_rrc_imsi_DS_41      , ASN1_NO_EXTENSIONS     , dissect_rrc_IMSI_DS_41 },
  {   4, &hf_rrc_tmsi_DS_41      , ASN1_NO_EXTENSIONS     , dissect_rrc_TMSI_DS_41 },
  {   5, &hf_rrc_spare3          , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   6, &hf_rrc_spare2          , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   7, &hf_rrc_spare1          , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_CN_PagedUE_Identity(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_CN_PagedUE_Identity, CN_PagedUE_Identity_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t T_cn_Identity_sequence[] = {
  { &hf_rrc_pagingCause     , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PagingCause },
  { &hf_rrc_cn_DomainIdentity, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CN_DomainIdentity },
  { &hf_rrc_cn_pagedUE_Identity, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CN_PagedUE_Identity },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_cn_Identity(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_cn_Identity, T_cn_Identity_sequence);

  return offset;
}


static const per_sequence_t T_cn_OriginatedPage_connectedMode_UE_sequence[] = {
  { &hf_rrc_pagingCause     , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PagingCause },
  { &hf_rrc_cn_DomainIdentity, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CN_DomainIdentity },
  { &hf_rrc_pagingRecordTypeID, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PagingRecordTypeID },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_cn_OriginatedPage_connectedMode_UE(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_cn_OriginatedPage_connectedMode_UE, T_cn_OriginatedPage_connectedMode_UE_sequence);

  return offset;
}


static const per_sequence_t T_utran_Identity_sequence[] = {
  { &hf_rrc_u_RNTI          , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_U_RNTI },
  { &hf_rrc_cn_OriginatedPage_connectedMode_UE, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_cn_OriginatedPage_connectedMode_UE },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_utran_Identity(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_utran_Identity, T_utran_Identity_sequence);

  return offset;
}


static const value_string rrc_PagingRecord_vals[] = {
  {   0, "cn-Identity" },
  {   1, "utran-Identity" },
  { 0, NULL }
};

static const per_choice_t PagingRecord_choice[] = {
  {   0, &hf_rrc_cn_Identity     , ASN1_NO_EXTENSIONS     , dissect_rrc_T_cn_Identity },
  {   1, &hf_rrc_utran_Identity  , ASN1_NO_EXTENSIONS     , dissect_rrc_T_utran_Identity },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_PagingRecord(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_PagingRecord, PagingRecord_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t PagingRecordList_sequence_of[1] = {
  { &hf_rrc_PagingRecordList_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PagingRecord },
};

static int
dissect_rrc_PagingRecordList(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_PagingRecordList, PagingRecordList_sequence_of,
                                                  1, maxPage1, FALSE);

  return offset;
}



static int
dissect_rrc_BCCH_ModificationTime(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            0U, 511U, NULL, FALSE);

  return offset;
}


static const per_sequence_t BCCH_ModificationInfo_sequence[] = {
  { &hf_rrc_mib_ValueTag    , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MIB_ValueTag },
  { &hf_rrc_bcch_ModificationTime, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_BCCH_ModificationTime },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_BCCH_ModificationInfo(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_BCCH_ModificationInfo, BCCH_ModificationInfo_sequence);

  return offset;
}


static const per_sequence_t T_cn_OriginatedPage_connectedMode_UE_01_sequence[] = {
  { &hf_rrc_pagingCause     , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PagingCause },
  { &hf_rrc_cn_DomainIdentity, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CN_DomainIdentity },
  { &hf_rrc_pagingRecordTypeID, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PagingRecordTypeID },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_cn_OriginatedPage_connectedMode_UE_01(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_cn_OriginatedPage_connectedMode_UE_01, T_cn_OriginatedPage_connectedMode_UE_01_sequence);

  return offset;
}


static const per_sequence_t T_release_sequence[] = {
  { &hf_rrc_releaseCause    , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_ReleaseCause },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_release(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_release, T_release_sequence);

  return offset;
}


static const value_string rrc_RRC_ConnectionReleaseInformation_vals[] = {
  {   0, "noRelease" },
  {   1, "release" },
  { 0, NULL }
};

static const per_choice_t RRC_ConnectionReleaseInformation_choice[] = {
  {   0, &hf_rrc_noRelease       , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   1, &hf_rrc_release         , ASN1_NO_EXTENSIONS     , dissect_rrc_T_release },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_RRC_ConnectionReleaseInformation(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_RRC_ConnectionReleaseInformation, RRC_ConnectionReleaseInformation_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t T_utran_SingleUE_Identity_sequence[] = {
  { &hf_rrc_u_RNTI          , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_U_RNTI },
  { &hf_rrc_cn_OriginatedPage_connectedMode_UE_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_cn_OriginatedPage_connectedMode_UE_01 },
  { &hf_rrc_rrc_ConnectionReleaseInformation, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RRC_ConnectionReleaseInformation },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_utran_SingleUE_Identity(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_utran_SingleUE_Identity, T_utran_SingleUE_Identity_sequence);

  return offset;
}


static const per_sequence_t GroupIdentityWithReleaseInformation_sequence[] = {
  { &hf_rrc_rrc_ConnectionReleaseInformation, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RRC_ConnectionReleaseInformation },
  { &hf_rrc_groupReleaseInformation, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_GroupReleaseInformation },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_GroupIdentityWithReleaseInformation(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_GroupIdentityWithReleaseInformation, GroupIdentityWithReleaseInformation_sequence);

  return offset;
}


static const per_sequence_t SEQUENCE_SIZE_1_maxURNTI_Group_OF_GroupIdentityWithReleaseInformation_sequence_of[1] = {
  { &hf_rrc_utran_GroupIdentity_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_GroupIdentityWithReleaseInformation },
};

static int
dissect_rrc_SEQUENCE_SIZE_1_maxURNTI_Group_OF_GroupIdentityWithReleaseInformation(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_SEQUENCE_SIZE_1_maxURNTI_Group_OF_GroupIdentityWithReleaseInformation, SEQUENCE_SIZE_1_maxURNTI_Group_OF_GroupIdentityWithReleaseInformation_sequence_of,
                                                  1, maxURNTI_Group, FALSE);

  return offset;
}


static const value_string rrc_PagingRecord2_r5_vals[] = {
  {   0, "utran-SingleUE-Identity" },
  {   1, "utran-GroupIdentity" },
  { 0, NULL }
};

static const per_choice_t PagingRecord2_r5_choice[] = {
  {   0, &hf_rrc_utran_SingleUE_Identity, ASN1_NO_EXTENSIONS     , dissect_rrc_T_utran_SingleUE_Identity },
  {   1, &hf_rrc_utran_GroupIdentity, ASN1_NO_EXTENSIONS     , dissect_rrc_SEQUENCE_SIZE_1_maxURNTI_Group_OF_GroupIdentityWithReleaseInformation },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_PagingRecord2_r5(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_PagingRecord2_r5, PagingRecord2_r5_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t PagingRecord2List_r5_sequence_of[1] = {
  { &hf_rrc_PagingRecord2List_r5_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PagingRecord2_r5 },
};

static int
dissect_rrc_PagingRecord2List_r5(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_PagingRecord2List_r5, PagingRecord2List_r5_sequence_of,
                                                  1, maxPage1, FALSE);

  return offset;
}


static const per_sequence_t PagingType1_v590ext_IEs_sequence[] = {
  { &hf_rrc_pagingRecord2List, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_PagingRecord2List_r5 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_PagingType1_v590ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_PagingType1_v590ext_IEs, PagingType1_v590ext_IEs_sequence);

  return offset;
}


static const per_sequence_t PagingType1_v860ext_IEs_sequence[] = {
  { &hf_rrc_etws_Information, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_ETWS_Information },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_PagingType1_v860ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_PagingType1_v860ext_IEs, PagingType1_v860ext_IEs_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions_68_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_nonCriticalExtensions_68(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_nonCriticalExtensions_68, T_nonCriticalExtensions_68_sequence);

  return offset;
}


static const per_sequence_t T_v860NonCriticalExtensions_09_sequence[] = {
  { &hf_rrc_pagingType1_v860ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PagingType1_v860ext_IEs },
  { &hf_rrc_nonCriticalExtensions_68, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_nonCriticalExtensions_68 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v860NonCriticalExtensions_09(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v860NonCriticalExtensions_09, T_v860NonCriticalExtensions_09_sequence);

  return offset;
}


static const per_sequence_t T_v590NonCriticalExtensions_12_sequence[] = {
  { &hf_rrc_pagingType1_v590ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PagingType1_v590ext_IEs },
  { &hf_rrc_v860NonCriticalExtensions_09, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v860NonCriticalExtensions_09 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v590NonCriticalExtensions_12(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v590NonCriticalExtensions_12, T_v590NonCriticalExtensions_12_sequence);

  return offset;
}


static const per_sequence_t T_laterNonCriticalExtensions_21_sequence[] = {
  { &hf_rrc_pagingType1_r3_add_ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_BIT_STRING },
  { &hf_rrc_v590NonCriticalExtensions_12, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v590NonCriticalExtensions_12 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_laterNonCriticalExtensions_21(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_laterNonCriticalExtensions_21, T_laterNonCriticalExtensions_21_sequence);

  return offset;
}


static const per_sequence_t PagingType1_sequence[] = {
  { &hf_rrc_pagingRecordList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_PagingRecordList },
  { &hf_rrc_bcch_ModificationInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_BCCH_ModificationInfo },
  { &hf_rrc_laterNonCriticalExtensions_21, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_laterNonCriticalExtensions_21 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_PagingType1(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
#line 297 "../../asn1/rrc/rrc.cnf"
  col_append_str(actx->pinfo->cinfo, COL_INFO, "PagingType1");
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_PagingType1, PagingType1_sequence);




  return offset;
}


static const value_string rrc_PCCH_MessageType_vals[] = {
  {   0, "pagingType1" },
  {   1, "spare" },
  { 0, NULL }
};

static const per_choice_t PCCH_MessageType_choice[] = {
  {   0, &hf_rrc_pagingType1     , ASN1_NO_EXTENSIONS     , dissect_rrc_PagingType1 },
  {   1, &hf_rrc_spare           , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_PCCH_MessageType(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_PCCH_MessageType, PCCH_MessageType_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t PCCH_Message_sequence[] = {
  { &hf_rrc_message_04      , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PCCH_MessageType },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_PCCH_Message(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_PCCH_Message, PCCH_Message_sequence);

  return offset;
}


static const value_string rrc_DL_SHCCH_MessageType_vals[] = {
  {   0, "physicalSharedChannelAllocation" },
  {   1, "spare" },
  { 0, NULL }
};

static const per_choice_t DL_SHCCH_MessageType_choice[] = {
  {   0, &hf_rrc_physicalSharedChannelAllocation, ASN1_NO_EXTENSIONS     , dissect_rrc_PhysicalSharedChannelAllocation },
  {   1, &hf_rrc_spare           , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_DL_SHCCH_MessageType(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_DL_SHCCH_MessageType, DL_SHCCH_MessageType_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t DL_SHCCH_Message_sequence[] = {
  { &hf_rrc_message_05      , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DL_SHCCH_MessageType },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_DL_SHCCH_Message(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_DL_SHCCH_Message, DL_SHCCH_Message_sequence);

  return offset;
}


static const per_sequence_t TimeslotWithISCP_sequence[] = {
  { &hf_rrc_timeslot        , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TimeslotNumber },
  { &hf_rrc_timeslotISCP_01 , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TimeslotISCP },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_TimeslotWithISCP(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_TimeslotWithISCP, TimeslotWithISCP_sequence);

  return offset;
}


static const per_sequence_t TimeslotListWithISCP_sequence_of[1] = {
  { &hf_rrc_TimeslotListWithISCP_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TimeslotWithISCP },
};

static int
dissect_rrc_TimeslotListWithISCP(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_TimeslotListWithISCP, TimeslotListWithISCP_sequence_of,
                                                  1, maxTS, FALSE);

  return offset;
}


static const value_string rrc_T_allocationConfirmation_vals[] = {
  {   0, "pdschConfirmation" },
  {   1, "puschConfirmation" },
  { 0, NULL }
};

static const per_choice_t T_allocationConfirmation_choice[] = {
  {   0, &hf_rrc_pdschConfirmation, ASN1_NO_EXTENSIONS     , dissect_rrc_PDSCH_Identity },
  {   1, &hf_rrc_puschConfirmation, ASN1_NO_EXTENSIONS     , dissect_rrc_PUSCH_Identity },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_allocationConfirmation(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_allocationConfirmation, T_allocationConfirmation_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t PUSCHCapacityRequest_v590ext_sequence[] = {
  { &hf_rrc_primaryCCPCH_RSCP_delta, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DeltaRSCP },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_PUSCHCapacityRequest_v590ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_PUSCHCapacityRequest_v590ext, PUSCHCapacityRequest_v590ext_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions_83_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_nonCriticalExtensions_83(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_nonCriticalExtensions_83, T_nonCriticalExtensions_83_sequence);

  return offset;
}


static const per_sequence_t T_v590NonCriticalExtensions_13_sequence[] = {
  { &hf_rrc_puschCapacityRequest_v590ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PUSCHCapacityRequest_v590ext },
  { &hf_rrc_nonCriticalExtensions_83, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_nonCriticalExtensions_83 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v590NonCriticalExtensions_13(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v590NonCriticalExtensions_13, T_v590NonCriticalExtensions_13_sequence);

  return offset;
}


static const per_sequence_t T_laterNonCriticalExtensions_27_sequence[] = {
  { &hf_rrc_puschCapacityRequest_r3_add_ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_BIT_STRING },
  { &hf_rrc_v590NonCriticalExtensions_13, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v590NonCriticalExtensions_13 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_laterNonCriticalExtensions_27(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_laterNonCriticalExtensions_27, T_laterNonCriticalExtensions_27_sequence);

  return offset;
}


static const per_sequence_t PUSCHCapacityRequest_sequence[] = {
  { &hf_rrc_dsch_RNTI       , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DSCH_RNTI },
  { &hf_rrc_trafficVolume   , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_TrafficVolumeMeasuredResultsList },
  { &hf_rrc_timeslotListWithISCP, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_TimeslotListWithISCP },
  { &hf_rrc_primaryCCPCH_RSCP, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_PrimaryCCPCH_RSCP },
  { &hf_rrc_allocationConfirmation, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_allocationConfirmation },
  { &hf_rrc_protocolErrorIndicator, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_ProtocolErrorIndicatorWithMoreInfo },
  { &hf_rrc_laterNonCriticalExtensions_27, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_laterNonCriticalExtensions_27 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_PUSCHCapacityRequest(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
#line 321 "../../asn1/rrc/rrc.cnf"
  col_append_str(actx->pinfo->cinfo, COL_INFO, "PUSCHCapacityRequest");
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_PUSCHCapacityRequest, PUSCHCapacityRequest_sequence);




  return offset;
}


static const value_string rrc_UL_SHCCH_MessageType_vals[] = {
  {   0, "puschCapacityRequest" },
  {   1, "spare" },
  { 0, NULL }
};

static const per_choice_t UL_SHCCH_MessageType_choice[] = {
  {   0, &hf_rrc_puschCapacityRequest, ASN1_NO_EXTENSIONS     , dissect_rrc_PUSCHCapacityRequest },
  {   1, &hf_rrc_spare           , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_UL_SHCCH_MessageType(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_UL_SHCCH_MessageType, UL_SHCCH_MessageType_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t UL_SHCCH_Message_sequence[] = {
  { &hf_rrc_message_06      , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UL_SHCCH_MessageType },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UL_SHCCH_Message(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UL_SHCCH_Message, UL_SHCCH_Message_sequence);

  return offset;
}


static const value_string rrc_SIB_Type_vals[] = {
  {   0, "masterInformationBlock" },
  {   1, "systemInformationBlockType1" },
  {   2, "systemInformationBlockType2" },
  {   3, "systemInformationBlockType3" },
  {   4, "systemInformationBlockType4" },
  {   5, "systemInformationBlockType5" },
  {   6, "systemInformationBlockType6" },
  {   7, "systemInformationBlockType7" },
  {   8, "dummy" },
  {   9, "dummy2" },
  {  10, "dummy3" },
  {  11, "systemInformationBlockType11" },
  {  12, "systemInformationBlockType12" },
  {  13, "systemInformationBlockType13" },
  {  14, "systemInformationBlockType13-1" },
  {  15, "systemInformationBlockType13-2" },
  {  16, "systemInformationBlockType13-3" },
  {  17, "systemInformationBlockType13-4" },
  {  18, "systemInformationBlockType14" },
  {  19, "systemInformationBlockType15" },
  {  20, "systemInformationBlockType15-1" },
  {  21, "systemInformationBlockType15-2" },
  {  22, "systemInformationBlockType15-3" },
  {  23, "systemInformationBlockType16" },
  {  24, "systemInformationBlockType17" },
  {  25, "systemInformationBlockType15-4" },
  {  26, "systemInformationBlockType18" },
  {  27, "schedulingBlock1" },
  {  28, "schedulingBlock2" },
  {  29, "systemInformationBlockType15-5" },
  {  30, "systemInformationBlockType5bis" },
  {  31, "extensionType" },
  { 0, NULL }
};

static value_string_ext rrc_SIB_Type_vals_ext = VALUE_STRING_EXT_INIT(rrc_SIB_Type_vals);


static int
dissect_rrc_SIB_Type(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     32, NULL, FALSE, 0, NULL);

  return offset;
}



static int
dissect_rrc_SIB_Data_fixed(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_bit_string(tvb, offset, actx, tree, hf_index,
                                     222, 222, FALSE, NULL);

  return offset;
}


static const per_sequence_t FirstSegment_sequence[] = {
  { &hf_rrc_sib_Type        , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SIB_Type },
  { &hf_rrc_seg_Count       , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SegCount },
  { &hf_rrc_sib_Data_fixed  , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SIB_Data_fixed },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_FirstSegment(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_FirstSegment, FirstSegment_sequence);

  return offset;
}



static int
dissect_rrc_SegmentIndex(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            1U, 15U, NULL, FALSE);

  return offset;
}


static const per_sequence_t SubsequentSegment_sequence[] = {
  { &hf_rrc_sib_Type        , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SIB_Type },
  { &hf_rrc_segmentIndex    , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SegmentIndex },
  { &hf_rrc_sib_Data_fixed  , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SIB_Data_fixed },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_SubsequentSegment(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_SubsequentSegment, SubsequentSegment_sequence);

  return offset;
}



static int
dissect_rrc_SIB_Data_variable(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_bit_string(tvb, offset, actx, tree, hf_index,
                                     1, 214, FALSE, NULL);

  return offset;
}


static const per_sequence_t LastSegmentShort_sequence[] = {
  { &hf_rrc_sib_Type        , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SIB_Type },
  { &hf_rrc_segmentIndex    , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SegmentIndex },
  { &hf_rrc_sib_Data_variable, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SIB_Data_variable },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_LastSegmentShort(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_LastSegmentShort, LastSegmentShort_sequence);

  return offset;
}


static const per_sequence_t FirstSegmentShort_sequence[] = {
  { &hf_rrc_sib_Type        , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SIB_Type },
  { &hf_rrc_seg_Count       , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SegCount },
  { &hf_rrc_sib_Data_variable, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SIB_Data_variable },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_FirstSegmentShort(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_FirstSegmentShort, FirstSegmentShort_sequence);

  return offset;
}


static const per_sequence_t T_lastAndFirst_01_sequence[] = {
  { &hf_rrc_lastSegmentShort, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_LastSegmentShort },
  { &hf_rrc_firstSegment_01 , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_FirstSegmentShort },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_lastAndFirst_01(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_lastAndFirst_01, T_lastAndFirst_01_sequence);

  return offset;
}


static const per_sequence_t CompleteSIBshort_sequence[] = {
  { &hf_rrc_sib_Type        , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SIB_Type },
  { &hf_rrc_sib_Data_variable, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SIB_Data_variable },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_CompleteSIBshort(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_CompleteSIBshort, CompleteSIBshort_sequence);

  return offset;
}


static const per_sequence_t CompleteSIB_List_sequence_of[1] = {
  { &hf_rrc_CompleteSIB_List_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CompleteSIBshort },
};

static int
dissect_rrc_CompleteSIB_List(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_CompleteSIB_List, CompleteSIB_List_sequence_of,
                                                  1, maxSIBperMsg, FALSE);

  return offset;
}


static const per_sequence_t T_lastAndComplete_01_sequence[] = {
  { &hf_rrc_lastSegmentShort, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_LastSegmentShort },
  { &hf_rrc_completeSIB_List, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CompleteSIB_List },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_lastAndComplete_01(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_lastAndComplete_01, T_lastAndComplete_01_sequence);

  return offset;
}


static const per_sequence_t T_lastAndCompleteAndFirst_01_sequence[] = {
  { &hf_rrc_lastSegmentShort, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_LastSegmentShort },
  { &hf_rrc_completeSIB_List, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CompleteSIB_List },
  { &hf_rrc_firstSegment_01 , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_FirstSegmentShort },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_lastAndCompleteAndFirst_01(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_lastAndCompleteAndFirst_01, T_lastAndCompleteAndFirst_01_sequence);

  return offset;
}


static const per_sequence_t T_completeAndFirst_01_sequence[] = {
  { &hf_rrc_completeSIB_List, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CompleteSIB_List },
  { &hf_rrc_firstSegment_01 , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_FirstSegmentShort },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_completeAndFirst_01(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_completeAndFirst_01, T_completeAndFirst_01_sequence);

  return offset;
}



static int
dissect_rrc_BIT_STRING_SIZE_226(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_bit_string(tvb, offset, actx, tree, hf_index,
                                     226, 226, FALSE, NULL);

  return offset;
}


static const per_sequence_t CompleteSIB_sequence[] = {
  { &hf_rrc_sib_Type        , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SIB_Type },
  { &hf_rrc_sib_Data_fixed_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_226 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_CompleteSIB(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_CompleteSIB, CompleteSIB_sequence);

  return offset;
}


static const per_sequence_t LastSegment_sequence[] = {
  { &hf_rrc_sib_Type        , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SIB_Type },
  { &hf_rrc_segmentIndex    , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SegmentIndex },
  { &hf_rrc_sib_Data_fixed  , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SIB_Data_fixed },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_LastSegment(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_LastSegment, LastSegment_sequence);

  return offset;
}


static const value_string rrc_T_payload_01_vals[] = {
  {   0, "noSegment" },
  {   1, "firstSegment" },
  {   2, "subsequentSegment" },
  {   3, "lastSegmentShort" },
  {   4, "lastAndFirst" },
  {   5, "lastAndComplete" },
  {   6, "lastAndCompleteAndFirst" },
  {   7, "completeSIB-List" },
  {   8, "completeAndFirst" },
  {   9, "completeSIB" },
  {  10, "lastSegment" },
  {  11, "spare5" },
  {  12, "spare4" },
  {  13, "spare3" },
  {  14, "spare2" },
  {  15, "spare1" },
  { 0, NULL }
};

static const per_choice_t T_payload_01_choice[] = {
  {   0, &hf_rrc_noSegment       , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   1, &hf_rrc_firstSegment    , ASN1_NO_EXTENSIONS     , dissect_rrc_FirstSegment },
  {   2, &hf_rrc_subsequentSegment, ASN1_NO_EXTENSIONS     , dissect_rrc_SubsequentSegment },
  {   3, &hf_rrc_lastSegmentShort, ASN1_NO_EXTENSIONS     , dissect_rrc_LastSegmentShort },
  {   4, &hf_rrc_lastAndFirst_01 , ASN1_NO_EXTENSIONS     , dissect_rrc_T_lastAndFirst_01 },
  {   5, &hf_rrc_lastAndComplete_01, ASN1_NO_EXTENSIONS     , dissect_rrc_T_lastAndComplete_01 },
  {   6, &hf_rrc_lastAndCompleteAndFirst_01, ASN1_NO_EXTENSIONS     , dissect_rrc_T_lastAndCompleteAndFirst_01 },
  {   7, &hf_rrc_completeSIB_List, ASN1_NO_EXTENSIONS     , dissect_rrc_CompleteSIB_List },
  {   8, &hf_rrc_completeAndFirst_01, ASN1_NO_EXTENSIONS     , dissect_rrc_T_completeAndFirst_01 },
  {   9, &hf_rrc_completeSIB     , ASN1_NO_EXTENSIONS     , dissect_rrc_CompleteSIB },
  {  10, &hf_rrc_lastSegment     , ASN1_NO_EXTENSIONS     , dissect_rrc_LastSegment },
  {  11, &hf_rrc_spare5          , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {  12, &hf_rrc_spare4          , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {  13, &hf_rrc_spare3          , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {  14, &hf_rrc_spare2          , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {  15, &hf_rrc_spare1          , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_payload_01(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_payload_01, T_payload_01_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t SystemInformation_FACH_sequence[] = {
  { &hf_rrc_payload_01      , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_payload_01 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_SystemInformation_FACH(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
#line 427 "../../asn1/rrc/rrc.cnf"
  col_append_str(actx->pinfo->cinfo, COL_INFO, "SystemInformation-FACH");
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_SystemInformation_FACH, SystemInformation_FACH_sequence);




  return offset;
}


static const per_sequence_t SystemInformationChangeIndication_v860ext_IEs_sequence[] = {
  { &hf_rrc_etws_Information, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_ETWS_Information },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_SystemInformationChangeIndication_v860ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_SystemInformationChangeIndication_v860ext_IEs, SystemInformationChangeIndication_v860ext_IEs_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions_145_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_nonCriticalExtensions_145(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_nonCriticalExtensions_145, T_nonCriticalExtensions_145_sequence);

  return offset;
}


static const per_sequence_t T_v860NonCriticalExtensions_17_sequence[] = {
  { &hf_rrc_systemInformationChangeIndication_v860ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SystemInformationChangeIndication_v860ext_IEs },
  { &hf_rrc_nonCriticalExtensions_145, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_nonCriticalExtensions_145 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v860NonCriticalExtensions_17(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v860NonCriticalExtensions_17, T_v860NonCriticalExtensions_17_sequence);

  return offset;
}


static const per_sequence_t T_laterNonCriticalExtensions_50_sequence[] = {
  { &hf_rrc_systemInformationChangeIndication_r3_add_ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_BIT_STRING },
  { &hf_rrc_v860NonCriticalExtensions_17, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v860NonCriticalExtensions_17 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_laterNonCriticalExtensions_50(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_laterNonCriticalExtensions_50, T_laterNonCriticalExtensions_50_sequence);

  return offset;
}


static const per_sequence_t SystemInformationChangeIndication_sequence[] = {
  { &hf_rrc_bcch_ModificationInfo, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BCCH_ModificationInfo },
  { &hf_rrc_laterNonCriticalExtensions_50, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_laterNonCriticalExtensions_50 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_SystemInformationChangeIndication(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
#line 431 "../../asn1/rrc/rrc.cnf"
  col_append_str(actx->pinfo->cinfo, COL_INFO, "SystemInformationChangeIndication");
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_SystemInformationChangeIndication, SystemInformationChangeIndication_sequence);




  return offset;
}


static const value_string rrc_BCCH_FACH_MessageType_vals[] = {
  {   0, "dummy" },
  {   1, "systemInformationChangeIndication" },
  {   2, "spare2" },
  {   3, "spare1" },
  { 0, NULL }
};

static const per_choice_t BCCH_FACH_MessageType_choice[] = {
  {   0, &hf_rrc_dummy           , ASN1_NO_EXTENSIONS     , dissect_rrc_SystemInformation_FACH },
  {   1, &hf_rrc_systemInformationChangeIndication, ASN1_NO_EXTENSIONS     , dissect_rrc_SystemInformationChangeIndication },
  {   2, &hf_rrc_spare2          , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   3, &hf_rrc_spare1          , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_BCCH_FACH_MessageType(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_BCCH_FACH_MessageType, BCCH_FACH_MessageType_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t BCCH_FACH_Message_sequence[] = {
  { &hf_rrc_message_07      , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BCCH_FACH_MessageType },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_BCCH_FACH_Message(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_BCCH_FACH_Message, BCCH_FACH_Message_sequence);

  return offset;
}



static int
dissect_rrc_SFN_Prime(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            0U, 2047U, NULL, FALSE);

  return offset;
}


static const per_sequence_t T_lastAndFirst_sequence[] = {
  { &hf_rrc_lastSegmentShort, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_LastSegmentShort },
  { &hf_rrc_firstSegment_01 , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_FirstSegmentShort },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_lastAndFirst(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_lastAndFirst, T_lastAndFirst_sequence);

  return offset;
}


static const per_sequence_t T_lastAndComplete_sequence[] = {
  { &hf_rrc_lastSegmentShort, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_LastSegmentShort },
  { &hf_rrc_completeSIB_List, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CompleteSIB_List },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_lastAndComplete(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_lastAndComplete, T_lastAndComplete_sequence);

  return offset;
}


static const per_sequence_t T_lastAndCompleteAndFirst_sequence[] = {
  { &hf_rrc_lastSegmentShort, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_LastSegmentShort },
  { &hf_rrc_completeSIB_List, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CompleteSIB_List },
  { &hf_rrc_firstSegment_01 , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_FirstSegmentShort },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_lastAndCompleteAndFirst(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_lastAndCompleteAndFirst, T_lastAndCompleteAndFirst_sequence);

  return offset;
}


static const per_sequence_t T_completeAndFirst_sequence[] = {
  { &hf_rrc_completeSIB_List, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CompleteSIB_List },
  { &hf_rrc_firstSegment_01 , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_FirstSegmentShort },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_completeAndFirst(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_completeAndFirst, T_completeAndFirst_sequence);

  return offset;
}


static const value_string rrc_T_payload_vals[] = {
  {   0, "noSegment" },
  {   1, "firstSegment" },
  {   2, "subsequentSegment" },
  {   3, "lastSegmentShort" },
  {   4, "lastAndFirst" },
  {   5, "lastAndComplete" },
  {   6, "lastAndCompleteAndFirst" },
  {   7, "completeSIB-List" },
  {   8, "completeAndFirst" },
  {   9, "completeSIB" },
  {  10, "lastSegment" },
  {  11, "spare5" },
  {  12, "spare4" },
  {  13, "spare3" },
  {  14, "spare2" },
  {  15, "spare1" },
  { 0, NULL }
};

static const per_choice_t T_payload_choice[] = {
  {   0, &hf_rrc_noSegment       , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   1, &hf_rrc_firstSegment    , ASN1_NO_EXTENSIONS     , dissect_rrc_FirstSegment },
  {   2, &hf_rrc_subsequentSegment, ASN1_NO_EXTENSIONS     , dissect_rrc_SubsequentSegment },
  {   3, &hf_rrc_lastSegmentShort, ASN1_NO_EXTENSIONS     , dissect_rrc_LastSegmentShort },
  {   4, &hf_rrc_lastAndFirst    , ASN1_NO_EXTENSIONS     , dissect_rrc_T_lastAndFirst },
  {   5, &hf_rrc_lastAndComplete , ASN1_NO_EXTENSIONS     , dissect_rrc_T_lastAndComplete },
  {   6, &hf_rrc_lastAndCompleteAndFirst, ASN1_NO_EXTENSIONS     , dissect_rrc_T_lastAndCompleteAndFirst },
  {   7, &hf_rrc_completeSIB_List, ASN1_NO_EXTENSIONS     , dissect_rrc_CompleteSIB_List },
  {   8, &hf_rrc_completeAndFirst, ASN1_NO_EXTENSIONS     , dissect_rrc_T_completeAndFirst },
  {   9, &hf_rrc_completeSIB     , ASN1_NO_EXTENSIONS     , dissect_rrc_CompleteSIB },
  {  10, &hf_rrc_lastSegment     , ASN1_NO_EXTENSIONS     , dissect_rrc_LastSegment },
  {  11, &hf_rrc_spare5          , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {  12, &hf_rrc_spare4          , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {  13, &hf_rrc_spare3          , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {  14, &hf_rrc_spare2          , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {  15, &hf_rrc_spare1          , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_payload(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_payload, T_payload_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t SystemInformation_BCH_sequence[] = {
  { &hf_rrc_sfn_Prime       , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SFN_Prime },
  { &hf_rrc_payload         , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_payload },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_SystemInformation_BCH(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
#line 423 "../../asn1/rrc/rrc.cnf"
  col_append_str(actx->pinfo->cinfo, COL_INFO, "SystemInformation-BCH");
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_SystemInformation_BCH, SystemInformation_BCH_sequence);




  return offset;
}


static const per_sequence_t BCCH_BCH_Message_sequence[] = {
  { &hf_rrc_message_08      , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SystemInformation_BCH },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_BCCH_BCH_Message(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_BCCH_BCH_Message, BCCH_BCH_Message_sequence);

  return offset;
}


static const value_string rrc_MBMS_AccessProbabilityFactor_vals[] = {
  {   0, "apf0" },
  {   1, "apf32" },
  {   2, "apf64" },
  {   3, "apf96" },
  {   4, "apf128" },
  {   5, "apf160" },
  {   6, "apf192" },
  {   7, "apf224" },
  {   8, "apf256" },
  {   9, "apf288" },
  {  10, "apf320" },
  {  11, "apf352" },
  {  12, "apf384" },
  {  13, "apf416" },
  {  14, "apf448" },
  {  15, "apf480" },
  {  16, "apf512" },
  {  17, "apf544" },
  {  18, "apf576" },
  {  19, "apf608" },
  {  20, "apf640" },
  {  21, "apf672" },
  {  22, "apf704" },
  {  23, "apf736" },
  {  24, "apf768" },
  {  25, "apf800" },
  {  26, "apf832" },
  {  27, "apf864" },
  {  28, "apf896" },
  {  29, "apf928" },
  {  30, "apf960" },
  {  31, "apf1000" },
  { 0, NULL }
};

static value_string_ext rrc_MBMS_AccessProbabilityFactor_vals_ext = VALUE_STRING_EXT_INIT(rrc_MBMS_AccessProbabilityFactor_vals);


static int
dissect_rrc_MBMS_AccessProbabilityFactor(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     32, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t MBMS_ConnectedModeCountingScope_sequence[] = {
  { &hf_rrc_countingForUraPCH, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { &hf_rrc_countingForCellPCH, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { &hf_rrc_countingForCellFACH, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_MBMS_ConnectedModeCountingScope(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_MBMS_ConnectedModeCountingScope, MBMS_ConnectedModeCountingScope_sequence);

  return offset;
}


static const per_sequence_t MBMS_ServiceAccessInfo_r6_sequence[] = {
  { &hf_rrc_shortTransmissionID, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MBMS_ShortTransmissionID },
  { &hf_rrc_accessprobabilityFactor_Idle, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MBMS_AccessProbabilityFactor },
  { &hf_rrc_accessprobabilityFactor_Connected, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MBMS_AccessProbabilityFactor },
  { &hf_rrc_mbms_ConnectedModeCountingScope, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MBMS_ConnectedModeCountingScope },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_MBMS_ServiceAccessInfo_r6(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_MBMS_ServiceAccessInfo_r6, MBMS_ServiceAccessInfo_r6_sequence);

  return offset;
}


static const per_sequence_t MBMS_ServiceAccessInfoList_r6_sequence_of[1] = {
  { &hf_rrc_MBMS_ServiceAccessInfoList_r6_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MBMS_ServiceAccessInfo_r6 },
};

static int
dissect_rrc_MBMS_ServiceAccessInfoList_r6(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_MBMS_ServiceAccessInfoList_r6, MBMS_ServiceAccessInfoList_r6_sequence_of,
                                                  1, maxMBMSservCount, FALSE);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions_179_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_nonCriticalExtensions_179(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_nonCriticalExtensions_179, T_nonCriticalExtensions_179_sequence);

  return offset;
}


static const per_sequence_t MBMSAccessInformation_sequence[] = {
  { &hf_rrc_mbms_ServiceAccessInfoList, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MBMS_ServiceAccessInfoList_r6 },
  { &hf_rrc_nonCriticalExtensions_179, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_nonCriticalExtensions_179 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_MBMSAccessInformation(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
#line 249 "../../asn1/rrc/rrc.cnf"
  col_append_str(actx->pinfo->cinfo, COL_INFO, "MBMSAccessInformation");
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_MBMSAccessInformation, MBMSAccessInformation_sequence);




  return offset;
}



static int
dissect_rrc_MBMS_CommonRBIdentity(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            1U, 32U, NULL, FALSE);

  return offset;
}


static const value_string rrc_TimerDAR_r6_vals[] = {
  {   0, "ms40" },
  {   1, "ms80" },
  {   2, "ms120" },
  {   3, "ms160" },
  {   4, "ms240" },
  {   5, "ms320" },
  {   6, "ms480" },
  {   7, "ms640" },
  {   8, "ms960" },
  {   9, "ms1280" },
  {  10, "ms1920" },
  {  11, "ms2560" },
  {  12, "ms3840" },
  {  13, "ms5120" },
  { 0, NULL }
};


static int
dissect_rrc_TimerDAR_r6(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     14, NULL, FALSE, 0, NULL);

  return offset;
}


static const value_string rrc_WindowSizeDAR_r6_vals[] = {
  {   0, "ws4" },
  {   1, "ws8" },
  {   2, "ws16" },
  {   3, "ws32" },
  {   4, "ws40" },
  {   5, "ws48" },
  {   6, "ws56" },
  {   7, "ws64" },
  { 0, NULL }
};


static int
dissect_rrc_WindowSizeDAR_r6(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     8, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t UM_RLC_DuplAvoid_Reord_Info_r6_sequence[] = {
  { &hf_rrc_timer_DAR       , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TimerDAR_r6 },
  { &hf_rrc_widowSize_DAR   , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_WindowSizeDAR_r6 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UM_RLC_DuplAvoid_Reord_Info_r6(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UM_RLC_DuplAvoid_Reord_Info_r6, UM_RLC_DuplAvoid_Reord_Info_r6_sequence);

  return offset;
}


static const per_sequence_t RLC_Info_MTCH_r6_sequence[] = {
  { &hf_rrc_dl_UM_RLC_LI_size, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DL_UM_RLC_LI_size },
  { &hf_rrc_dl_UM_RLC_DuplAvoid_Reord_Info, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UM_RLC_DuplAvoid_Reord_Info_r6 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_RLC_Info_MTCH_r6(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_RLC_Info_MTCH_r6, RLC_Info_MTCH_r6_sequence);

  return offset;
}


static const per_sequence_t MBMS_CommonRBInformation_r6_sequence[] = {
  { &hf_rrc_commonRBIdentity, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MBMS_CommonRBIdentity },
  { &hf_rrc_pdcp_Info_01    , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PDCP_Info_r4 },
  { &hf_rrc_rlc_Info_05     , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RLC_Info_MTCH_r6 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_MBMS_CommonRBInformation_r6(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_MBMS_CommonRBInformation_r6, MBMS_CommonRBInformation_r6_sequence);

  return offset;
}


static const per_sequence_t MBMS_CommonRBInformationList_r6_sequence_of[1] = {
  { &hf_rrc_MBMS_CommonRBInformationList_r6_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MBMS_CommonRBInformation_r6 },
};

static int
dissect_rrc_MBMS_CommonRBInformationList_r6(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_MBMS_CommonRBInformationList_r6, MBMS_CommonRBInformationList_r6_sequence_of,
                                                  1, maxMBMS_CommonRB, FALSE);

  return offset;
}



static int
dissect_rrc_MBMS_CommonTrChIdentity(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            1U, 32U, NULL, FALSE);

  return offset;
}


static const per_sequence_t MBMS_TranspChInfoForTrCh_r6_sequence[] = {
  { &hf_rrc_commonTrChIdentity, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MBMS_CommonTrChIdentity },
  { &hf_rrc_transportFormatSet, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TransportFormatSet },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_MBMS_TranspChInfoForTrCh_r6(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_MBMS_TranspChInfoForTrCh_r6, MBMS_TranspChInfoForTrCh_r6_sequence);

  return offset;
}


static const per_sequence_t MBMS_TranspChInfoForEachTrCh_r6_sequence_of[1] = {
  { &hf_rrc_MBMS_TranspChInfoForEachTrCh_r6_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MBMS_TranspChInfoForTrCh_r6 },
};

static int
dissect_rrc_MBMS_TranspChInfoForEachTrCh_r6(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_MBMS_TranspChInfoForEachTrCh_r6, MBMS_TranspChInfoForEachTrCh_r6_sequence_of,
                                                  1, maxMBMS_CommonTrCh, FALSE);

  return offset;
}



static int
dissect_rrc_MBMS_CommonCCTrChIdentity(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            1U, 32U, NULL, FALSE);

  return offset;
}


static const per_sequence_t MBMS_TranspChInfoForCCTrCh_r6_sequence[] = {
  { &hf_rrc_commonCCTrChIdentity, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MBMS_CommonCCTrChIdentity },
  { &hf_rrc_transportFormatCombinationSet, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TFCS },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_MBMS_TranspChInfoForCCTrCh_r6(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_MBMS_TranspChInfoForCCTrCh_r6, MBMS_TranspChInfoForCCTrCh_r6_sequence);

  return offset;
}


static const per_sequence_t MBMS_TranspChInfoForEachCCTrCh_r6_sequence_of[1] = {
  { &hf_rrc_MBMS_TranspChInfoForEachCCTrCh_r6_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MBMS_TranspChInfoForCCTrCh_r6 },
};

static int
dissect_rrc_MBMS_TranspChInfoForEachCCTrCh_r6(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_MBMS_TranspChInfoForEachCCTrCh_r6, MBMS_TranspChInfoForEachCCTrCh_r6_sequence_of,
                                                  1, maxMBMS_CommonCCTrCh, FALSE);

  return offset;
}



static int
dissect_rrc_MBMS_CommonPhyChIdentity(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            1U, 32U, NULL, FALSE);

  return offset;
}


static const per_sequence_t T_fdd_111_sequence[] = {
  { &hf_rrc_secondaryScramblingCode, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_SecondaryScramblingCode },
  { &hf_rrc_sttd_Indicator  , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { &hf_rrc_sf_AndCodeNumber_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SF256_AndCodeNumber },
  { &hf_rrc_timingOffset    , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_TimingOffset },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_fdd_111(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_fdd_111, T_fdd_111_sequence);

  return offset;
}


static const per_sequence_t CommonTimeslotInfoMBMS_sequence[] = {
  { &hf_rrc_secondInterleavingMode, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SecondInterleavingMode },
  { &hf_rrc_tfci_Coding     , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_TFCI_Coding },
  { &hf_rrc_puncturingLimit , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PuncturingLimit },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_CommonTimeslotInfoMBMS(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_CommonTimeslotInfoMBMS, CommonTimeslotInfoMBMS_sequence);

  return offset;
}


static const per_sequence_t T_tdd384_32_sequence[] = {
  { &hf_rrc_commonTimeslotInfoMBMS, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CommonTimeslotInfoMBMS },
  { &hf_rrc_downlinkTimeslotsCodes, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DownlinkTimeslotsCodes },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_tdd384_32(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_tdd384_32, T_tdd384_32_sequence);

  return offset;
}


static const per_sequence_t T_tdd128_51_sequence[] = {
  { &hf_rrc_commonTimeslotInfoMBMS, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CommonTimeslotInfoMBMS },
  { &hf_rrc_downlinkTimeslotsCodes_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DownlinkTimeslotsCodes_LCR_r4 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_tdd128_51(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_tdd128_51, T_tdd128_51_sequence);

  return offset;
}


static const value_string rrc_T_modeSpecificInfo_79_vals[] = {
  {   0, "fdd" },
  {   1, "tdd384" },
  {   2, "tdd128" },
  { 0, NULL }
};

static const per_choice_t T_modeSpecificInfo_79_choice[] = {
  {   0, &hf_rrc_fdd_114         , ASN1_NO_EXTENSIONS     , dissect_rrc_T_fdd_111 },
  {   1, &hf_rrc_tdd384_35       , ASN1_NO_EXTENSIONS     , dissect_rrc_T_tdd384_32 },
  {   2, &hf_rrc_tdd128_52       , ASN1_NO_EXTENSIONS     , dissect_rrc_T_tdd128_51 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_modeSpecificInfo_79(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_modeSpecificInfo_79, T_modeSpecificInfo_79_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t SecondaryCCPCHInfo_MBMS_r6_sequence[] = {
  { &hf_rrc_modeSpecificInfo_79, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_modeSpecificInfo_79 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_SecondaryCCPCHInfo_MBMS_r6(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_SecondaryCCPCHInfo_MBMS_r6, SecondaryCCPCHInfo_MBMS_r6_sequence);

  return offset;
}


static const per_sequence_t MBMS_PhyChInformation_r6_sequence[] = {
  { &hf_rrc_mbms_CommonPhyChIdentity, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MBMS_CommonPhyChIdentity },
  { &hf_rrc_secondaryCCPCHInfo_MBMS, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SecondaryCCPCHInfo_MBMS_r6 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_MBMS_PhyChInformation_r6(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_MBMS_PhyChInformation_r6, MBMS_PhyChInformation_r6_sequence);

  return offset;
}


static const per_sequence_t MBMS_PhyChInformationList_r6_sequence_of[1] = {
  { &hf_rrc_MBMS_PhyChInformationList_r6_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MBMS_PhyChInformation_r6 },
};

static int
dissect_rrc_MBMS_PhyChInformationList_r6(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_MBMS_PhyChInformationList_r6, MBMS_PhyChInformationList_r6_sequence_of,
                                                  1, maxMBMS_CommonPhyCh, FALSE);

  return offset;
}



static int
dissect_rrc_INTEGER_M11_4(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            -11, 4U, NULL, FALSE);

  return offset;
}


static const value_string rrc_T_modulation_04_vals[] = {
  {   0, "modQPSK" },
  {   1, "mod16QAM" },
  { 0, NULL }
};

static const per_choice_t T_modulation_04_choice[] = {
  {   0, &hf_rrc_modQPSK         , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   1, &hf_rrc_mod16QAM_01     , ASN1_NO_EXTENSIONS     , dissect_rrc_INTEGER_M11_4 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_modulation_04(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_modulation_04, T_modulation_04_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t T_fdd_112_sequence[] = {
  { &hf_rrc_secondaryScramblingCode, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_SecondaryScramblingCode },
  { &hf_rrc_sttd_Indicator  , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { &hf_rrc_sf_AndCodeNumber_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SF256_AndCodeNumber },
  { &hf_rrc_timingOffset    , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_TimingOffset },
  { &hf_rrc_modulation_04   , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_modulation_04 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_fdd_112(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_fdd_112, T_fdd_112_sequence);

  return offset;
}


static const value_string rrc_T_modulation_05_vals[] = {
  {   0, "modQPSK" },
  {   1, "mod16QAM" },
  { 0, NULL }
};


static int
dissect_rrc_T_modulation_05(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     2, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t T_tdd384_33_sequence[] = {
  { &hf_rrc_commonTimeslotInfoMBMS, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CommonTimeslotInfoMBMS },
  { &hf_rrc_downlinkTimeslotsCodes_02, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DownlinkTimeslotsCodes_r7 },
  { &hf_rrc_modulation_05   , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_modulation_05 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_tdd384_33(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_tdd384_33, T_tdd384_33_sequence);

  return offset;
}


static const value_string rrc_T_modulation_06_vals[] = {
  {   0, "modQPSK" },
  {   1, "mod16QAM" },
  { 0, NULL }
};


static int
dissect_rrc_T_modulation_06(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     2, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t T_tdd768_19_sequence[] = {
  { &hf_rrc_commonTimeslotInfoMBMS, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CommonTimeslotInfoMBMS },
  { &hf_rrc_downlinkTimeslotsCodes_03, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DownlinkTimeslotsCodes_VHCR },
  { &hf_rrc_modulation_06   , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_modulation_06 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_tdd768_19(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_tdd768_19, T_tdd768_19_sequence);

  return offset;
}


static const value_string rrc_TimeSlotLCR_ext_vals[] = {
  {   0, "ts7" },
  {   1, "spare3" },
  {   2, "spare2" },
  {   3, "spare1" },
  { 0, NULL }
};


static int
dissect_rrc_TimeSlotLCR_ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     4, NULL, FALSE, 0, NULL);

  return offset;
}


static const value_string rrc_T_modulation_07_vals[] = {
  {   0, "modQPSK" },
  {   1, "mod16QAM" },
  { 0, NULL }
};


static int
dissect_rrc_T_modulation_07(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     2, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t T_tdd128_52_sequence[] = {
  { &hf_rrc_commonTimeslotInfoMBMS, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CommonTimeslotInfoMBMS },
  { &hf_rrc_downlinkTimeslotsCodes_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DownlinkTimeslotsCodes_LCR_r4 },
  { &hf_rrc_mbsfnSpecialTimeSlot, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_TimeSlotLCR_ext },
  { &hf_rrc_modulation_07   , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_modulation_07 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_tdd128_52(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_tdd128_52, T_tdd128_52_sequence);

  return offset;
}


static const value_string rrc_T_modeSpecificInfo_80_vals[] = {
  {   0, "fdd" },
  {   1, "tdd384" },
  {   2, "tdd768" },
  {   3, "tdd128" },
  { 0, NULL }
};

static const per_choice_t T_modeSpecificInfo_80_choice[] = {
  {   0, &hf_rrc_fdd_115         , ASN1_NO_EXTENSIONS     , dissect_rrc_T_fdd_112 },
  {   1, &hf_rrc_tdd384_36       , ASN1_NO_EXTENSIONS     , dissect_rrc_T_tdd384_33 },
  {   2, &hf_rrc_tdd768_22       , ASN1_NO_EXTENSIONS     , dissect_rrc_T_tdd768_19 },
  {   3, &hf_rrc_tdd128_53       , ASN1_NO_EXTENSIONS     , dissect_rrc_T_tdd128_52 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_modeSpecificInfo_80(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_modeSpecificInfo_80, T_modeSpecificInfo_80_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t SecondaryCCPCHInfo_MBMS_r7_sequence[] = {
  { &hf_rrc_modeSpecificInfo_80, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_modeSpecificInfo_80 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_SecondaryCCPCHInfo_MBMS_r7(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_SecondaryCCPCHInfo_MBMS_r7, SecondaryCCPCHInfo_MBMS_r7_sequence);

  return offset;
}


static const per_sequence_t MBMS_PhyChInformation_r7_sequence[] = {
  { &hf_rrc_mbms_CommonPhyChIdentity, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MBMS_CommonPhyChIdentity },
  { &hf_rrc_secondaryCCPCHInfo_MBMS_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SecondaryCCPCHInfo_MBMS_r7 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_MBMS_PhyChInformation_r7(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_MBMS_PhyChInformation_r7, MBMS_PhyChInformation_r7_sequence);

  return offset;
}


static const per_sequence_t MBMS_PhyChInformationList_r7_sequence_of[1] = {
  { &hf_rrc_MBMS_PhyChInformationList_r7_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MBMS_PhyChInformation_r7 },
};

static int
dissect_rrc_MBMS_PhyChInformationList_r7(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_MBMS_PhyChInformationList_r7, MBMS_PhyChInformationList_r7_sequence_of,
                                                  1, maxMBMS_CommonPhyCh, FALSE);

  return offset;
}


static const per_sequence_t MBMSCommonPTMRBInformation_v770ext_IEs_sequence[] = {
  { &hf_rrc_mbms_PhyChInformationList_r7, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MBMS_PhyChInformationList_r7 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_MBMSCommonPTMRBInformation_v770ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_MBMSCommonPTMRBInformation_v770ext_IEs, MBMSCommonPTMRBInformation_v770ext_IEs_sequence);

  return offset;
}



static int
dissect_rrc_INTEGER_1_maxMBSFNClusters(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            1U, maxMBSFNClusters, NULL, FALSE);

  return offset;
}


static const per_sequence_t MBSFN_TDDTimeSlotInfo_sequence[] = {
  { &hf_rrc_timeSlotNumber_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TimeslotNumber_LCR_r4 },
  { &hf_rrc_cellParametersID, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CellParametersID },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_MBSFN_TDDTimeSlotInfo(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_MBSFN_TDDTimeSlotInfo, MBSFN_TDDTimeSlotInfo_sequence);

  return offset;
}


static const per_sequence_t MBSFN_TDDInformation_sequence_of[1] = {
  { &hf_rrc_MBSFN_TDDInformation_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MBSFN_TDDTimeSlotInfo },
};

static int
dissect_rrc_MBSFN_TDDInformation(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_MBSFN_TDDInformation, MBSFN_TDDInformation_sequence_of,
                                                  1, maxTS, FALSE);

  return offset;
}


static const per_sequence_t MBSFN_TDDTimeSlotInfo_LCR_sequence[] = {
  { &hf_rrc_frequencyIndex  , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_1_maxMBSFNClusters },
  { &hf_rrc_timeSlotList    , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MBSFN_TDDInformation },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_MBSFN_TDDTimeSlotInfo_LCR(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_MBSFN_TDDTimeSlotInfo_LCR, MBSFN_TDDTimeSlotInfo_LCR_sequence);

  return offset;
}


static const per_sequence_t MBSFN_TDDInformation_LCR_sequence_of[1] = {
  { &hf_rrc_MBSFN_TDDInformation_LCR_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MBSFN_TDDTimeSlotInfo_LCR },
};

static int
dissect_rrc_MBSFN_TDDInformation_LCR(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_MBSFN_TDDInformation_LCR, MBSFN_TDDInformation_LCR_sequence_of,
                                                  1, maxMBSFNClusters, FALSE);

  return offset;
}


static const per_sequence_t MBMSCommonPTMRBInformation_v780ext_IEs_sequence[] = {
  { &hf_rrc_mbsfn_TDDInformation_LCR, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MBSFN_TDDInformation_LCR },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_MBMSCommonPTMRBInformation_v780ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_MBMSCommonPTMRBInformation_v780ext_IEs, MBMSCommonPTMRBInformation_v780ext_IEs_sequence);

  return offset;
}



static int
dissect_rrc_INTEGER_0_4(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            0U, 4U, NULL, FALSE);

  return offset;
}


static const per_sequence_t DL_ChannelCodes_MBSFN_IMB384_sequence[] = {
  { &hf_rrc_firstChannelisationCode_02, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_1_15 },
  { &hf_rrc_lastChannelisationCode_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_INTEGER_1_15 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_DL_ChannelCodes_MBSFN_IMB384(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_DL_ChannelCodes_MBSFN_IMB384, DL_ChannelCodes_MBSFN_IMB384_sequence);

  return offset;
}


static const per_sequence_t T_mod16QAM_sequence[] = {
  { &hf_rrc_cpich_SecCCPCH_PowerOffset, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_M11_4 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_mod16QAM(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_mod16QAM, T_mod16QAM_sequence);

  return offset;
}


static const value_string rrc_T_modulation_03_vals[] = {
  {   0, "modQPSK" },
  {   1, "mod16QAM" },
  { 0, NULL }
};

static const per_choice_t T_modulation_03_choice[] = {
  {   0, &hf_rrc_modQPSK         , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   1, &hf_rrc_mod16QAM        , ASN1_NO_EXTENSIONS     , dissect_rrc_T_mod16QAM },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_modulation_03(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_modulation_03, T_modulation_03_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t SecondaryCCPCHFrameType2Info_sequence[] = {
  { &hf_rrc_subFrameNumber  , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_0_4 },
  { &hf_rrc_dl_ChannelisationCodes, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DL_ChannelCodes_MBSFN_IMB384 },
  { &hf_rrc_modulation_03   , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_modulation_03 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_SecondaryCCPCHFrameType2Info(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_SecondaryCCPCHFrameType2Info, SecondaryCCPCHFrameType2Info_sequence);

  return offset;
}


static const per_sequence_t MBMS_PhyChInformation_IMB384_sequence[] = {
  { &hf_rrc_mbms_CommonPhyChIdentity, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MBMS_CommonPhyChIdentity },
  { &hf_rrc_secondaryCCPCHInfo_MBMS_02, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SecondaryCCPCHFrameType2Info },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_MBMS_PhyChInformation_IMB384(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_MBMS_PhyChInformation_IMB384, MBMS_PhyChInformation_IMB384_sequence);

  return offset;
}


static const per_sequence_t MBMS_PhyChInformationList_IMB384_sequence_of[1] = {
  { &hf_rrc_MBMS_PhyChInformationList_IMB384_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MBMS_PhyChInformation_IMB384 },
};

static int
dissect_rrc_MBMS_PhyChInformationList_IMB384(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_MBMS_PhyChInformationList_IMB384, MBMS_PhyChInformationList_IMB384_sequence_of,
                                                  1, maxMBMS_CommonPhyCh, FALSE);

  return offset;
}


static const per_sequence_t MBMSCommonPTMRBInformation_v860ext_IEs_sequence[] = {
  { &hf_rrc_mbms_PhyChInformationList_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MBMS_PhyChInformationList_IMB384 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_MBMSCommonPTMRBInformation_v860ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_MBMSCommonPTMRBInformation_v860ext_IEs, MBMSCommonPTMRBInformation_v860ext_IEs_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions_180_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_nonCriticalExtensions_180(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_nonCriticalExtensions_180, T_nonCriticalExtensions_180_sequence);

  return offset;
}


static const per_sequence_t T_v860NonCriticalExtensions_23_sequence[] = {
  { &hf_rrc_mbmsCommonPTMRBInformation_v860ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MBMSCommonPTMRBInformation_v860ext_IEs },
  { &hf_rrc_nonCriticalExtensions_180, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_nonCriticalExtensions_180 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v860NonCriticalExtensions_23(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v860NonCriticalExtensions_23, T_v860NonCriticalExtensions_23_sequence);

  return offset;
}


static const per_sequence_t T_v780NonCriticalExtensions_10_sequence[] = {
  { &hf_rrc_mbmsCommonPTMRBInformation_v780ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MBMSCommonPTMRBInformation_v780ext_IEs },
  { &hf_rrc_v860NonCriticalExtensions_23, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v860NonCriticalExtensions_23 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v780NonCriticalExtensions_10(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v780NonCriticalExtensions_10, T_v780NonCriticalExtensions_10_sequence);

  return offset;
}


static const per_sequence_t T_v770NonCriticalExtensions_33_sequence[] = {
  { &hf_rrc_mbmsCommonPTMRBInformation_v770ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MBMSCommonPTMRBInformation_v770ext_IEs },
  { &hf_rrc_v780NonCriticalExtensions_10, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v780NonCriticalExtensions_10 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v770NonCriticalExtensions_33(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v770NonCriticalExtensions_33, T_v770NonCriticalExtensions_33_sequence);

  return offset;
}


static const per_sequence_t MBMSCommonPTMRBInformation_sequence[] = {
  { &hf_rrc_mbms_CommonRBInformationList, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MBMS_CommonRBInformationList_r6 },
  { &hf_rrc_mbms_TranspChInfoForEachTrCh, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MBMS_TranspChInfoForEachTrCh_r6 },
  { &hf_rrc_mbms_TranspChInfoForEachCCTrCh, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MBMS_TranspChInfoForEachCCTrCh_r6 },
  { &hf_rrc_mbms_PhyChInformationList, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MBMS_PhyChInformationList_r6 },
  { &hf_rrc_v770NonCriticalExtensions_33, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v770NonCriticalExtensions_33 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_MBMSCommonPTMRBInformation(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
#line 253 "../../asn1/rrc/rrc.cnf"
  col_append_str(actx->pinfo->cinfo, COL_INFO, "MBMSCommonPTMRBInformation");
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_MBMSCommonPTMRBInformation, MBMSCommonPTMRBInformation_sequence);




  return offset;
}



static int
dissect_rrc_MBMS_SCCPCHIdentity(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            1U, maxSCCPCH, NULL, FALSE);

  return offset;
}


static const value_string rrc_MBMS_SoftComb_TimingOffset_vals[] = {
  {   0, "ms0" },
  {   1, "ms10" },
  {   2, "ms20" },
  {   3, "ms40" },
  { 0, NULL }
};


static int
dissect_rrc_MBMS_SoftComb_TimingOffset(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     4, NULL, FALSE, 0, NULL);

  return offset;
}



static int
dissect_rrc_MBMS_LogicalChIdentity(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            1U, 15U, NULL, FALSE);

  return offset;
}


static const per_sequence_t MBMS_PTM_RBInformation_C_sequence[] = {
  { &hf_rrc_rbInformation   , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MBMS_CommonRBIdentity },
  { &hf_rrc_shortTransmissionID, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MBMS_ShortTransmissionID },
  { &hf_rrc_logicalChIdentity, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MBMS_LogicalChIdentity },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_MBMS_PTM_RBInformation_C(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_MBMS_PTM_RBInformation_C, MBMS_PTM_RBInformation_C_sequence);

  return offset;
}


static const per_sequence_t MBMS_PTM_RBInformation_CList_sequence_of[1] = {
  { &hf_rrc_MBMS_PTM_RBInformation_CList_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MBMS_PTM_RBInformation_C },
};

static int
dissect_rrc_MBMS_PTM_RBInformation_CList(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_MBMS_PTM_RBInformation_CList, MBMS_PTM_RBInformation_CList_sequence_of,
                                                  1, maxRBperTrCh, FALSE);

  return offset;
}


static const value_string rrc_MBMS_MSCHSchedulingInfo_vals[] = {
  {   0, "schedulingPeriod-32-Offset" },
  {   1, "schedulingPeriod-64-Offset" },
  {   2, "schedulingPeriod-128-Offset" },
  {   3, "schedulingPeriod-256-Offset" },
  {   4, "schedulingPeriod-512-Offset" },
  {   5, "schedulingPeriod-1024-Offset" },
  { 0, NULL }
};

static const per_choice_t MBMS_MSCHSchedulingInfo_choice[] = {
  {   0, &hf_rrc_schedulingPeriod_32_Offset, ASN1_NO_EXTENSIONS     , dissect_rrc_INTEGER_0_31 },
  {   1, &hf_rrc_schedulingPeriod_64_Offset, ASN1_NO_EXTENSIONS     , dissect_rrc_INTEGER_0_63 },
  {   2, &hf_rrc_schedulingPeriod_128_Offset, ASN1_NO_EXTENSIONS     , dissect_rrc_INTEGER_0_127 },
  {   3, &hf_rrc_schedulingPeriod_256_Offset, ASN1_NO_EXTENSIONS     , dissect_rrc_INTEGER_0_255 },
  {   4, &hf_rrc_schedulingPeriod_512_Offset, ASN1_NO_EXTENSIONS     , dissect_rrc_INTEGER_0_511 },
  {   5, &hf_rrc_schedulingPeriod_1024_Offset, ASN1_NO_EXTENSIONS     , dissect_rrc_INTEGER_0_1023 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_MBMS_MSCHSchedulingInfo(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_MBMS_MSCHSchedulingInfo, MBMS_MSCHSchedulingInfo_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t RLC_Info_MSCH_r6_sequence[] = {
  { &hf_rrc_dl_UM_RLC_LI_size, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DL_UM_RLC_LI_size },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_RLC_Info_MSCH_r6(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_RLC_Info_MSCH_r6, RLC_Info_MSCH_r6_sequence);

  return offset;
}


static const value_string rrc_MBMS_TCTF_Presence_vals[] = {
  {   0, "false" },
  { 0, NULL }
};


static int
dissect_rrc_MBMS_TCTF_Presence(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t MBMS_MSCH_ConfigurationInfo_r6_sequence[] = {
  { &hf_rrc_mschShedulingInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MBMS_MSCHSchedulingInfo },
  { &hf_rrc_rlc_Info_07     , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RLC_Info_MSCH_r6 },
  { &hf_rrc_tctf_Presence   , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MBMS_TCTF_Presence },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_MBMS_MSCH_ConfigurationInfo_r6(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_MBMS_MSCH_ConfigurationInfo_r6, MBMS_MSCH_ConfigurationInfo_r6_sequence);

  return offset;
}


static const per_sequence_t MBMS_TrCHInformation_Curr_sequence[] = {
  { &hf_rrc_transpCh_Info   , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MBMS_CommonTrChIdentity },
  { &hf_rrc_rbInformation_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MBMS_PTM_RBInformation_CList },
  { &hf_rrc_msch_ConfigurationInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MBMS_MSCH_ConfigurationInfo_r6 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_MBMS_TrCHInformation_Curr(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_MBMS_TrCHInformation_Curr, MBMS_TrCHInformation_Curr_sequence);

  return offset;
}


static const per_sequence_t MBMS_TrCHInformation_CurrList_sequence_of[1] = {
  { &hf_rrc_MBMS_TrCHInformation_CurrList_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MBMS_TrCHInformation_Curr },
};

static int
dissect_rrc_MBMS_TrCHInformation_CurrList(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_MBMS_TrCHInformation_CurrList, MBMS_TrCHInformation_CurrList_sequence_of,
                                                  1, maxFACHPCH, FALSE);

  return offset;
}


static const per_sequence_t MBMS_CurrentCell_SCCPCH_r6_sequence[] = {
  { &hf_rrc_sccpchIdentity  , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MBMS_SCCPCHIdentity },
  { &hf_rrc_secondaryCCPCH_Info_03, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MBMS_CommonPhyChIdentity },
  { &hf_rrc_softComb_TimingOffset, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MBMS_SoftComb_TimingOffset },
  { &hf_rrc_transpCh_InfoCommonForAllTrCh, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MBMS_CommonCCTrChIdentity },
  { &hf_rrc_transpCHInformation, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MBMS_TrCHInformation_CurrList },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_MBMS_CurrentCell_SCCPCH_r6(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_MBMS_CurrentCell_SCCPCH_r6, MBMS_CurrentCell_SCCPCH_r6_sequence);

  return offset;
}


static const per_sequence_t MBMS_CurrentCell_SCCPCHList_r6_sequence_of[1] = {
  { &hf_rrc_MBMS_CurrentCell_SCCPCHList_r6_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MBMS_CurrentCell_SCCPCH_r6 },
};

static int
dissect_rrc_MBMS_CurrentCell_SCCPCHList_r6(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_MBMS_CurrentCell_SCCPCHList_r6, MBMS_CurrentCell_SCCPCHList_r6_sequence_of,
                                                  1, maxSCCPCH, FALSE);

  return offset;
}



static int
dissect_rrc_INTEGER_1_maxFACHPCH(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            1U, maxFACHPCH, NULL, FALSE);

  return offset;
}


static const per_sequence_t MBMS_TrCHInformation_SIB5_sequence[] = {
  { &hf_rrc_transpCh_Identity, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_1_maxFACHPCH },
  { &hf_rrc_rbInformation_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MBMS_PTM_RBInformation_CList },
  { &hf_rrc_msch_ConfigurationInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MBMS_MSCH_ConfigurationInfo_r6 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_MBMS_TrCHInformation_SIB5(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_MBMS_TrCHInformation_SIB5, MBMS_TrCHInformation_SIB5_sequence);

  return offset;
}


static const per_sequence_t MBMS_TrCHInformation_SIB5List_sequence_of[1] = {
  { &hf_rrc_MBMS_TrCHInformation_SIB5List_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MBMS_TrCHInformation_SIB5 },
};

static int
dissect_rrc_MBMS_TrCHInformation_SIB5List(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_MBMS_TrCHInformation_SIB5List, MBMS_TrCHInformation_SIB5List_sequence_of,
                                                  1, maxFACHPCH, FALSE);

  return offset;
}


static const per_sequence_t MBMS_SIBType5_SCCPCH_r6_sequence[] = {
  { &hf_rrc_sccpchIdentity  , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MBMS_SCCPCHIdentity },
  { &hf_rrc_transpCHInformation_02, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MBMS_TrCHInformation_SIB5List },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_MBMS_SIBType5_SCCPCH_r6(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_MBMS_SIBType5_SCCPCH_r6, MBMS_SIBType5_SCCPCH_r6_sequence);

  return offset;
}


static const per_sequence_t MBMS_SIBType5_SCCPCHList_r6_sequence_of[1] = {
  { &hf_rrc_MBMS_SIBType5_SCCPCHList_r6_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MBMS_SIBType5_SCCPCH_r6 },
};

static int
dissect_rrc_MBMS_SIBType5_SCCPCHList_r6(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_MBMS_SIBType5_SCCPCHList_r6, MBMS_SIBType5_SCCPCHList_r6_sequence_of,
                                                  1, maxSCCPCH, FALSE);

  return offset;
}



static int
dissect_rrc_INTEGER_2_9(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            2U, 9U, NULL, FALSE);

  return offset;
}


static const per_sequence_t MBSFN_TDM_Info_sequence[] = {
  { &hf_rrc_shortTransmissionID, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MBMS_ShortTransmissionID },
  { &hf_rrc_tDMPeriod       , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_2_9 },
  { &hf_rrc_tDMOffset       , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_0_8 },
  { &hf_rrc_tDMLength       , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_1_8 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_MBSFN_TDM_Info(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_MBSFN_TDM_Info, MBSFN_TDM_Info_sequence);

  return offset;
}


static const per_sequence_t MBSFN_TDM_Info_List_sequence_of[1] = {
  { &hf_rrc_MBSFN_TDM_Info_List_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MBSFN_TDM_Info },
};

static int
dissect_rrc_MBSFN_TDM_Info_List(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_MBSFN_TDM_Info_List, MBSFN_TDM_Info_List_sequence_of,
                                                  1, maxMBMSservUnmodif, FALSE);

  return offset;
}


static const per_sequence_t MBMSCurrentCellPTMRBInfo_v770ext_IEs_sequence[] = {
  { &hf_rrc_mbsfn_TDM_Info_List, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MBSFN_TDM_Info_List },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_MBMSCurrentCellPTMRBInfo_v770ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_MBMSCurrentCellPTMRBInfo_v770ext_IEs, MBMSCurrentCellPTMRBInfo_v770ext_IEs_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions_181_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_nonCriticalExtensions_181(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_nonCriticalExtensions_181, T_nonCriticalExtensions_181_sequence);

  return offset;
}


static const per_sequence_t T_v770NonCriticalExtensions_34_sequence[] = {
  { &hf_rrc_mbmsCurrentCellPTMRBInfo_v770ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MBMSCurrentCellPTMRBInfo_v770ext_IEs },
  { &hf_rrc_nonCriticalExtensions_181, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_nonCriticalExtensions_181 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v770NonCriticalExtensions_34(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v770NonCriticalExtensions_34, T_v770NonCriticalExtensions_34_sequence);

  return offset;
}


static const per_sequence_t MBMSCurrentCellPTMRBInformation_sequence[] = {
  { &hf_rrc_mbms_CurrentCell_SCCPCHList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MBMS_CurrentCell_SCCPCHList_r6 },
  { &hf_rrc_mbms_SIBType5_SCCPCHList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MBMS_SIBType5_SCCPCHList_r6 },
  { &hf_rrc_v770NonCriticalExtensions_34, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v770NonCriticalExtensions_34 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_MBMSCurrentCellPTMRBInformation(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
#line 257 "../../asn1/rrc/rrc.cnf"
  col_append_str(actx->pinfo->cinfo, COL_INFO, "MBMSCurrentCellPTMRBInformation");
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_MBMSCurrentCellPTMRBInformation, MBMSCurrentCellPTMRBInformation_sequence);




  return offset;
}



static int
dissect_rrc_INTEGER_1_maxMBMS_Freq(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            1U, maxMBMS_Freq, NULL, FALSE);

  return offset;
}


static const value_string rrc_MBMS_Qoffset_vals[] = {
  {   0, "q4" },
  {   1, "q8" },
  {   2, "q12" },
  {   3, "q16" },
  {   4, "q20" },
  {   5, "q30" },
  {   6, "q40" },
  {   7, "qInfinity" },
  { 0, NULL }
};


static int
dissect_rrc_MBMS_Qoffset(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     8, NULL, FALSE, 0, NULL);

  return offset;
}


static const value_string rrc_T_layerConvergenceInformation_vals[] = {
  {   0, "mbms-Qoffset" },
  {   1, "mbms-HCSoffset" },
  { 0, NULL }
};

static const per_choice_t T_layerConvergenceInformation_choice[] = {
  {   0, &hf_rrc_mbms_Qoffset    , ASN1_NO_EXTENSIONS     , dissect_rrc_MBMS_Qoffset },
  {   1, &hf_rrc_mbms_HCSoffset  , ASN1_NO_EXTENSIONS     , dissect_rrc_INTEGER_0_7 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_layerConvergenceInformation(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_layerConvergenceInformation, T_layerConvergenceInformation_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t MBMS_PreferredFrequencyInfo_r6_sequence[] = {
  { &hf_rrc_mbmsPreferredFrequency, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_INTEGER_1_maxMBMS_Freq },
  { &hf_rrc_layerConvergenceInformation, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_layerConvergenceInformation },
  { &hf_rrc_mbms_PL_ServiceRestrictInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MBMS_PL_ServiceRestrictInfo_r6 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_MBMS_PreferredFrequencyInfo_r6(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_MBMS_PreferredFrequencyInfo_r6, MBMS_PreferredFrequencyInfo_r6_sequence);

  return offset;
}


static const per_sequence_t MBMS_PreferredFrequencyList_r6_sequence_of[1] = {
  { &hf_rrc_MBMS_PreferredFrequencyList_r6_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MBMS_PreferredFrequencyInfo_r6 },
};

static int
dissect_rrc_MBMS_PreferredFrequencyList_r6(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_MBMS_PreferredFrequencyList_r6, MBMS_PreferredFrequencyList_r6_sequence_of,
                                                  1, maxMBMS_Freq, FALSE);

  return offset;
}


static const value_string rrc_T_318_vals[] = {
  {   0, "ms250" },
  {   1, "ms500" },
  {   2, "ms750" },
  {   3, "ms1000" },
  {   4, "ms1250" },
  {   5, "ms1500" },
  {   6, "ms1750" },
  {   7, "ms2000" },
  {   8, "ms3000" },
  {   9, "ms4000" },
  {  10, "ms6000" },
  {  11, "ms8000" },
  {  12, "ms10000" },
  {  13, "ms12000" },
  {  14, "ms16000" },
  { 0, NULL }
};


static int
dissect_rrc_T_318(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     15, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t MBMS_TimersAndCounters_r6_sequence[] = {
  { &hf_rrc_t_318           , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_318 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_MBMS_TimersAndCounters_r6(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_MBMS_TimersAndCounters_r6, MBMS_TimersAndCounters_r6_sequence);

  return offset;
}



static int
dissect_rrc_MBMS_MICHPowerOffset(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            -10, 5U, NULL, FALSE);

  return offset;
}


static const value_string rrc_MBMS_NI_CountPerFrame_vals[] = {
  {   0, "ni18" },
  {   1, "ni36" },
  {   2, "ni72" },
  {   3, "ni144" },
  { 0, NULL }
};


static int
dissect_rrc_MBMS_NI_CountPerFrame(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     4, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t T_fdd_191_sequence[] = {
  { &hf_rrc_channelisationCode256, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_ChannelisationCode256 },
  { &hf_rrc_ni_CountPerFrame, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MBMS_NI_CountPerFrame },
  { &hf_rrc_sttd_Indicator  , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_fdd_191(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_fdd_191, T_fdd_191_sequence);

  return offset;
}


static const value_string rrc_RepPerLengthOffset_MICH_vals[] = {
  {   0, "rpp4-2" },
  {   1, "rpp8-2" },
  {   2, "rpp8-4" },
  {   3, "rpp16-2" },
  {   4, "rpp16-4" },
  {   5, "rpp32-2" },
  {   6, "rpp32-4" },
  {   7, "rpp64-2" },
  {   8, "rpp64-4" },
  { 0, NULL }
};

static const per_choice_t RepPerLengthOffset_MICH_choice[] = {
  {   0, &hf_rrc_rpp4_2          , ASN1_NO_EXTENSIONS     , dissect_rrc_INTEGER_0_3 },
  {   1, &hf_rrc_rpp8_2          , ASN1_NO_EXTENSIONS     , dissect_rrc_INTEGER_0_7 },
  {   2, &hf_rrc_rpp8_4          , ASN1_NO_EXTENSIONS     , dissect_rrc_INTEGER_0_7 },
  {   3, &hf_rrc_rpp16_2         , ASN1_NO_EXTENSIONS     , dissect_rrc_INTEGER_0_15 },
  {   4, &hf_rrc_rpp16_4         , ASN1_NO_EXTENSIONS     , dissect_rrc_INTEGER_0_15 },
  {   5, &hf_rrc_rpp32_2         , ASN1_NO_EXTENSIONS     , dissect_rrc_INTEGER_0_31 },
  {   6, &hf_rrc_rpp32_4         , ASN1_NO_EXTENSIONS     , dissect_rrc_INTEGER_0_31 },
  {   7, &hf_rrc_rpp64_2         , ASN1_NO_EXTENSIONS     , dissect_rrc_INTEGER_0_63 },
  {   8, &hf_rrc_rpp64_4         , ASN1_NO_EXTENSIONS     , dissect_rrc_INTEGER_0_63 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_RepPerLengthOffset_MICH(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_RepPerLengthOffset_MICH, RepPerLengthOffset_MICH_choice,
                                 NULL);

  return offset;
}


static const value_string rrc_MBMS_MICHNotificationIndLength_vals[] = {
  {   0, "mn4" },
  {   1, "mn8" },
  {   2, "mn16" },
  { 0, NULL }
};


static int
dissect_rrc_MBMS_MICHNotificationIndLength(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     3, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t T_tdd384_44_sequence[] = {
  { &hf_rrc_timeslot        , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TimeslotNumber },
  { &hf_rrc_midambleShiftAndBurstType_04, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MidambleShiftAndBurstType },
  { &hf_rrc_channelisationCode_05, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DL_TS_ChannelisationCode },
  { &hf_rrc_repetitionPeriodLengthOffset_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RepPerLengthOffset_MICH },
  { &hf_rrc_mbmsNotificationIndLength, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MBMS_MICHNotificationIndLength },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_tdd384_44(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_tdd384_44, T_tdd384_44_sequence);

  return offset;
}


static const per_sequence_t SEQUENCE_SIZE_1_2_OF_DL_TS_ChannelisationCode_sequence_of[1] = {
  { &hf_rrc_channelisationCodeList_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DL_TS_ChannelisationCode },
};

static int
dissect_rrc_SEQUENCE_SIZE_1_2_OF_DL_TS_ChannelisationCode(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_SEQUENCE_SIZE_1_2_OF_DL_TS_ChannelisationCode, SEQUENCE_SIZE_1_2_OF_DL_TS_ChannelisationCode_sequence_of,
                                                  1, 2, FALSE);

  return offset;
}


static const per_sequence_t T_tdd128_70_sequence[] = {
  { &hf_rrc_timeslot_01     , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TimeslotNumber_LCR_r4 },
  { &hf_rrc_midambleShiftAndBurstType_03, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MidambleShiftAndBurstType_LCR_r4 },
  { &hf_rrc_channelisationCodeList_02, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SEQUENCE_SIZE_1_2_OF_DL_TS_ChannelisationCode },
  { &hf_rrc_repetitionPeriodLengthOffset_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RepPerLengthOffset_MICH },
  { &hf_rrc_mbmsNotificationIndLength, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MBMS_MICHNotificationIndLength },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_tdd128_70(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_tdd128_70, T_tdd128_70_sequence);

  return offset;
}


static const value_string rrc_T_mode_03_vals[] = {
  {   0, "fdd" },
  {   1, "tdd384" },
  {   2, "tdd128" },
  { 0, NULL }
};

static const per_choice_t T_mode_03_choice[] = {
  {   0, &hf_rrc_fdd_197         , ASN1_NO_EXTENSIONS     , dissect_rrc_T_fdd_191 },
  {   1, &hf_rrc_tdd384_49       , ASN1_NO_EXTENSIONS     , dissect_rrc_T_tdd384_44 },
  {   2, &hf_rrc_tdd128_73       , ASN1_NO_EXTENSIONS     , dissect_rrc_T_tdd128_70 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_mode_03(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_mode_03, T_mode_03_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t MBMS_MICHConfigurationInfo_r6_sequence[] = {
  { &hf_rrc_michPowerOffset , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MBMS_MICHPowerOffset },
  { &hf_rrc_mode_03         , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_mode_03 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_MBMS_MICHConfigurationInfo_r6(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_MBMS_MICHConfigurationInfo_r6, MBMS_MICHConfigurationInfo_r6_sequence);

  return offset;
}



static int
dissect_rrc_MBMS_CellGroupIdentity_r6(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_bit_string(tvb, offset, actx, tree, hf_index,
                                     12, 12, FALSE, NULL);

  return offset;
}


static const per_sequence_t MBMSGeneralInformation_v6b0ext_IEs_sequence[] = {
  { &hf_rrc_indicateChangeInSelectedServices, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_MBMSGeneralInformation_v6b0ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_MBMSGeneralInformation_v6b0ext_IEs, MBMSGeneralInformation_v6b0ext_IEs_sequence);

  return offset;
}


static const per_sequence_t T_tdd384_45_sequence[] = {
  { &hf_rrc_midambleShiftAndBurstType_05, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MidambleShiftAndBurstType_r7 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_tdd384_45(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_tdd384_45, T_tdd384_45_sequence);

  return offset;
}


static const per_sequence_t T_tdd768_25_sequence[] = {
  { &hf_rrc_midambleShiftAndBurstType_06, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MidambleShiftAndBurstType_VHCR },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_tdd768_25(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_tdd768_25, T_tdd768_25_sequence);

  return offset;
}


static const per_sequence_t T_tdd128_71_sequence[] = {
  { &hf_rrc_mbsfnSpecialTimeSlot, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_TimeSlotLCR_ext },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_tdd128_71(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_tdd128_71, T_tdd128_71_sequence);

  return offset;
}


static const value_string rrc_T_mode_04_vals[] = {
  {   0, "tdd384" },
  {   1, "tdd768" },
  {   2, "tdd128" },
  { 0, NULL }
};

static const per_choice_t T_mode_04_choice[] = {
  {   0, &hf_rrc_tdd384_50       , ASN1_NO_EXTENSIONS     , dissect_rrc_T_tdd384_45 },
  {   1, &hf_rrc_tdd768_28       , ASN1_NO_EXTENSIONS     , dissect_rrc_T_tdd768_25 },
  {   2, &hf_rrc_tdd128_74       , ASN1_NO_EXTENSIONS     , dissect_rrc_T_tdd128_71 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_mode_04(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_mode_04, T_mode_04_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t MBMS_MICHConfigurationInfo_v770ext_sequence[] = {
  { &hf_rrc_mode_04         , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_mode_04 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_MBMS_MICHConfigurationInfo_v770ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_MBMS_MICHConfigurationInfo_v770ext, MBMS_MICHConfigurationInfo_v770ext_sequence);

  return offset;
}


static const value_string rrc_T_notificationOfAllMBSFNServicesInTheBand_vals[] = {
  {   0, "true" },
  { 0, NULL }
};


static int
dissect_rrc_T_notificationOfAllMBSFNServicesInTheBand(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t MBSFNservicesNotNotified_r7_sequence[] = {
  { &hf_rrc_notificationOfAllMBSFNServicesInTheBand, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_notificationOfAllMBSFNServicesInTheBand },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_MBSFNservicesNotNotified_r7(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_MBSFNservicesNotNotified_r7, MBSFNservicesNotNotified_r7_sequence);

  return offset;
}


static const value_string rrc_T_mbsfnServicesNotification_vals[] = {
  {   0, "mbsfnServicesNotified" },
  {   1, "mbsfnServicesNotNotified" },
  { 0, NULL }
};

static const per_choice_t T_mbsfnServicesNotification_choice[] = {
  {   0, &hf_rrc_mbsfnServicesNotified, ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   1, &hf_rrc_mbsfnServicesNotNotified, ASN1_NO_EXTENSIONS     , dissect_rrc_MBSFNservicesNotNotified_r7 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_mbsfnServicesNotification(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_mbsfnServicesNotification, T_mbsfnServicesNotification_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t MBSFNInterFrequencyNeighbour_r7_sequence[] = {
  { &hf_rrc_mbsfnFrequency  , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_FrequencyInfo },
  { &hf_rrc_mbsfnServicesNotification, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_mbsfnServicesNotification },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_MBSFNInterFrequencyNeighbour_r7(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_MBSFNInterFrequencyNeighbour_r7, MBSFNInterFrequencyNeighbour_r7_sequence);

  return offset;
}


static const per_sequence_t MBSFN_InterFrequencyNeighbourList_r7_sequence_of[1] = {
  { &hf_rrc_MBSFN_InterFrequencyNeighbourList_r7_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MBSFNInterFrequencyNeighbour_r7 },
};

static int
dissect_rrc_MBSFN_InterFrequencyNeighbourList_r7(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_MBSFN_InterFrequencyNeighbourList_r7, MBSFN_InterFrequencyNeighbourList_r7_sequence_of,
                                                  1, maxMBSFNClusters, FALSE);

  return offset;
}


static const per_sequence_t MBMSGeneralInformation_v770ext_IEs_sequence[] = {
  { &hf_rrc_mbmsMICHConfiguration, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MBMS_MICHConfigurationInfo_v770ext },
  { &hf_rrc_mbsfnInterFrequencyNeighbourList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MBSFN_InterFrequencyNeighbourList_r7 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_MBMSGeneralInformation_v770ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_MBMSGeneralInformation_v770ext_IEs, MBMSGeneralInformation_v770ext_IEs_sequence);

  return offset;
}


static const value_string rrc_T_imb_Indication_01_vals[] = {
  {   0, "true" },
  { 0, NULL }
};


static int
dissect_rrc_T_imb_Indication_01(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t MBSFNInterFrequencyNeighbour_v860ext_sequence[] = {
  { &hf_rrc_imb_Indication_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_imb_Indication_01 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_MBSFNInterFrequencyNeighbour_v860ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_MBSFNInterFrequencyNeighbour_v860ext, MBSFNInterFrequencyNeighbour_v860ext_sequence);

  return offset;
}


static const per_sequence_t MBSFN_InterFrequencyNeighbourList_v860ext_sequence_of[1] = {
  { &hf_rrc_MBSFN_InterFrequencyNeighbourList_v860ext_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MBSFNInterFrequencyNeighbour_v860ext },
};

static int
dissect_rrc_MBSFN_InterFrequencyNeighbourList_v860ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_MBSFN_InterFrequencyNeighbourList_v860ext, MBSFN_InterFrequencyNeighbourList_v860ext_sequence_of,
                                                  1, maxMBSFNClusters, FALSE);

  return offset;
}


static const per_sequence_t MBMSGeneralInformation_v860ext_IEs_sequence[] = {
  { &hf_rrc_mbsfnInterFrequencyNeighbourList_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MBSFN_InterFrequencyNeighbourList_v860ext },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_MBMSGeneralInformation_v860ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_MBMSGeneralInformation_v860ext_IEs, MBMSGeneralInformation_v860ext_IEs_sequence);

  return offset;
}



static int
dissect_rrc_BIT_STRING_SIZE_40(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_bit_string(tvb, offset, actx, tree, hf_index,
                                     40, 40, FALSE, NULL);

  return offset;
}


static const per_sequence_t MBMS_NetworkStandardTimeInformation_LCR_v890ext_sequence[] = {
  { &hf_rrc_networkStandardTime, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_40 },
  { &hf_rrc_correlativeSFN  , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_INTEGER_0_4095 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_MBMS_NetworkStandardTimeInformation_LCR_v890ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_MBMS_NetworkStandardTimeInformation_LCR_v890ext, MBMS_NetworkStandardTimeInformation_LCR_v890ext_sequence);

  return offset;
}


static const value_string rrc_MBMS_NI_CountPerFrame_IMB384_vals[] = {
  {   0, "ni16" },
  {   1, "ni32" },
  {   2, "ni64" },
  {   3, "ni128" },
  { 0, NULL }
};


static int
dissect_rrc_MBMS_NI_CountPerFrame_IMB384(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     4, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t T_imb384_sequence[] = {
  { &hf_rrc_channelisationCode256, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_ChannelisationCode256 },
  { &hf_rrc_ni_CountPerFrame_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MBMS_NI_CountPerFrame_IMB384 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_imb384(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_imb384, T_imb384_sequence);

  return offset;
}


static const value_string rrc_T_mode_05_vals[] = {
  {   0, "imb384" },
  { 0, NULL }
};

static const per_choice_t T_mode_05_choice[] = {
  {   0, &hf_rrc_imb384          , ASN1_NO_EXTENSIONS     , dissect_rrc_T_imb384 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_mode_05(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_mode_05, T_mode_05_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t MBMS_MICHConfigurationInfo_v890ext_sequence[] = {
  { &hf_rrc_mode_05         , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_mode_05 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_MBMS_MICHConfigurationInfo_v890ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_MBMS_MICHConfigurationInfo_v890ext, MBMS_MICHConfigurationInfo_v890ext_sequence);

  return offset;
}


static const per_sequence_t MBMSGeneralInformation_v890ext_IEs_sequence[] = {
  { &hf_rrc_mbmsNetworkStandardTimeInformation_LCR, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MBMS_NetworkStandardTimeInformation_LCR_v890ext },
  { &hf_rrc_mbmsMICHConfiguration_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MBMS_MICHConfigurationInfo_v890ext },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_MBMSGeneralInformation_v890ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_MBMSGeneralInformation_v890ext_IEs, MBMSGeneralInformation_v890ext_IEs_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions_182_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_nonCriticalExtensions_182(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_nonCriticalExtensions_182, T_nonCriticalExtensions_182_sequence);

  return offset;
}


static const per_sequence_t T_v890NoncriticalExtensions_sequence[] = {
  { &hf_rrc_mbmsGeneralInformation_v890ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MBMSGeneralInformation_v890ext_IEs },
  { &hf_rrc_nonCriticalExtensions_182, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_nonCriticalExtensions_182 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v890NoncriticalExtensions(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v890NoncriticalExtensions, T_v890NoncriticalExtensions_sequence);

  return offset;
}


static const per_sequence_t T_v860NonCriticalExtensions_24_sequence[] = {
  { &hf_rrc_mbmsGeneralInformation_v860ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MBMSGeneralInformation_v860ext_IEs },
  { &hf_rrc_v890NoncriticalExtensions, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v890NoncriticalExtensions },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v860NonCriticalExtensions_24(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v860NonCriticalExtensions_24, T_v860NonCriticalExtensions_24_sequence);

  return offset;
}


static const per_sequence_t T_v770NonCriticalExtensions_35_sequence[] = {
  { &hf_rrc_mbmsGeneralInformation_v770ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MBMSGeneralInformation_v770ext_IEs },
  { &hf_rrc_v860NonCriticalExtensions_24, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v860NonCriticalExtensions_24 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v770NonCriticalExtensions_35(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v770NonCriticalExtensions_35, T_v770NonCriticalExtensions_35_sequence);

  return offset;
}


static const per_sequence_t T_v6b0NonCriticalExtensions_12_sequence[] = {
  { &hf_rrc_mbmsGeneralInformation_v6b0ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MBMSGeneralInformation_v6b0ext_IEs },
  { &hf_rrc_v770NonCriticalExtensions_35, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v770NonCriticalExtensions_35 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v6b0NonCriticalExtensions_12(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v6b0NonCriticalExtensions_12, T_v6b0NonCriticalExtensions_12_sequence);

  return offset;
}


static const per_sequence_t MBMSGeneralInformation_sequence[] = {
  { &hf_rrc_mbms_PreferredFrequencyInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MBMS_PreferredFrequencyList_r6 },
  { &hf_rrc_mbms_TimersAndCounters, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MBMS_TimersAndCounters_r6 },
  { &hf_rrc_michConfigurationInfo, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MBMS_MICHConfigurationInfo_r6 },
  { &hf_rrc_cellGroupIdentity, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MBMS_CellGroupIdentity_r6 },
  { &hf_rrc_mschDefaultConfigurationInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MBMS_MSCH_ConfigurationInfo_r6 },
  { &hf_rrc_v6b0NonCriticalExtensions_13, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v6b0NonCriticalExtensions_12 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_MBMSGeneralInformation(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
#line 261 "../../asn1/rrc/rrc.cnf"
  col_append_str(actx->pinfo->cinfo, COL_INFO, "MBMSGeneralInformation");
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_MBMSGeneralInformation, MBMSGeneralInformation_sequence);




  return offset;
}


static const value_string rrc_MBMS_SCCPCHPwrOffsetDiff_vals[] = {
  {   0, "mcpo-minus6" },
  {   1, "mcpo-minus3" },
  {   2, "mcpo-plus3" },
  {   3, "mcpo-plus6" },
  { 0, NULL }
};


static int
dissect_rrc_MBMS_SCCPCHPwrOffsetDiff(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     4, NULL, FALSE, 0, NULL);

  return offset;
}



static int
dissect_rrc_MBMS_L1CombiningTransmTimeDiff(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            0U, 3U, NULL, FALSE);

  return offset;
}


static const per_sequence_t T_mtch_L1CombiningPeriodList_item_sequence[] = {
  { &hf_rrc_periodStart     , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_0_7 },
  { &hf_rrc_periodDuration  , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_1_8 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_mtch_L1CombiningPeriodList_item(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_mtch_L1CombiningPeriodList_item, T_mtch_L1CombiningPeriodList_item_sequence);

  return offset;
}


static const per_sequence_t T_mtch_L1CombiningPeriodList_sequence_of[1] = {
  { &hf_rrc_mtch_L1CombiningPeriodList_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_mtch_L1CombiningPeriodList_item },
};

static int
dissect_rrc_T_mtch_L1CombiningPeriodList(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_T_mtch_L1CombiningPeriodList, T_mtch_L1CombiningPeriodList_sequence_of,
                                                  1, maxMBMS_L1CP, FALSE);

  return offset;
}


static const per_sequence_t MBMS_L1CombiningSchedule_32_sequence[] = {
  { &hf_rrc_cycleOffset     , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_INTEGER_0_7 },
  { &hf_rrc_mtch_L1CombiningPeriodList, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_mtch_L1CombiningPeriodList },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_MBMS_L1CombiningSchedule_32(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_MBMS_L1CombiningSchedule_32, MBMS_L1CombiningSchedule_32_sequence);

  return offset;
}


static const per_sequence_t T_mtch_L1CombiningPeriodList_item_01_sequence[] = {
  { &hf_rrc_periodStart_01  , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_0_15 },
  { &hf_rrc_periodDuration_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_1_16 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_mtch_L1CombiningPeriodList_item_01(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_mtch_L1CombiningPeriodList_item_01, T_mtch_L1CombiningPeriodList_item_01_sequence);

  return offset;
}


static const per_sequence_t T_mtch_L1CombiningPeriodList_01_sequence_of[1] = {
  { &hf_rrc_mtch_L1CombiningPeriodList_item_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_mtch_L1CombiningPeriodList_item_01 },
};

static int
dissect_rrc_T_mtch_L1CombiningPeriodList_01(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_T_mtch_L1CombiningPeriodList_01, T_mtch_L1CombiningPeriodList_01_sequence_of,
                                                  1, maxMBMS_L1CP, FALSE);

  return offset;
}


static const per_sequence_t MBMS_L1CombiningSchedule_64_sequence[] = {
  { &hf_rrc_cycleOffset_01  , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_INTEGER_0_15 },
  { &hf_rrc_mtch_L1CombiningPeriodList_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_mtch_L1CombiningPeriodList_01 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_MBMS_L1CombiningSchedule_64(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_MBMS_L1CombiningSchedule_64, MBMS_L1CombiningSchedule_64_sequence);

  return offset;
}


static const per_sequence_t T_mtch_L1CombiningPeriodList_item_02_sequence[] = {
  { &hf_rrc_periodStart_02  , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_0_31 },
  { &hf_rrc_periodDuration_02, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_1_32 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_mtch_L1CombiningPeriodList_item_02(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_mtch_L1CombiningPeriodList_item_02, T_mtch_L1CombiningPeriodList_item_02_sequence);

  return offset;
}


static const per_sequence_t T_mtch_L1CombiningPeriodList_02_sequence_of[1] = {
  { &hf_rrc_mtch_L1CombiningPeriodList_item_02, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_mtch_L1CombiningPeriodList_item_02 },
};

static int
dissect_rrc_T_mtch_L1CombiningPeriodList_02(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_T_mtch_L1CombiningPeriodList_02, T_mtch_L1CombiningPeriodList_02_sequence_of,
                                                  1, maxMBMS_L1CP, FALSE);

  return offset;
}


static const per_sequence_t MBMS_L1CombiningSchedule_128_sequence[] = {
  { &hf_rrc_cycleOffset_02  , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_INTEGER_0_31 },
  { &hf_rrc_mtch_L1CombiningPeriodList_02, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_mtch_L1CombiningPeriodList_02 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_MBMS_L1CombiningSchedule_128(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_MBMS_L1CombiningSchedule_128, MBMS_L1CombiningSchedule_128_sequence);

  return offset;
}



static int
dissect_rrc_INTEGER_1_64(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            1U, 64U, NULL, FALSE);

  return offset;
}


static const per_sequence_t T_mtch_L1CombiningPeriodList_item_03_sequence[] = {
  { &hf_rrc_periodStart_03  , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_0_63 },
  { &hf_rrc_periodDuration_03, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_1_64 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_mtch_L1CombiningPeriodList_item_03(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_mtch_L1CombiningPeriodList_item_03, T_mtch_L1CombiningPeriodList_item_03_sequence);

  return offset;
}


static const per_sequence_t T_mtch_L1CombiningPeriodList_03_sequence_of[1] = {
  { &hf_rrc_mtch_L1CombiningPeriodList_item_03, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_mtch_L1CombiningPeriodList_item_03 },
};

static int
dissect_rrc_T_mtch_L1CombiningPeriodList_03(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_T_mtch_L1CombiningPeriodList_03, T_mtch_L1CombiningPeriodList_03_sequence_of,
                                                  1, maxMBMS_L1CP, FALSE);

  return offset;
}


static const per_sequence_t MBMS_L1CombiningSchedule_256_sequence[] = {
  { &hf_rrc_cycleOffset_03  , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_INTEGER_0_63 },
  { &hf_rrc_mtch_L1CombiningPeriodList_03, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_mtch_L1CombiningPeriodList_03 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_MBMS_L1CombiningSchedule_256(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_MBMS_L1CombiningSchedule_256, MBMS_L1CombiningSchedule_256_sequence);

  return offset;
}



static int
dissect_rrc_INTEGER_1_128(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            1U, 128U, NULL, FALSE);

  return offset;
}


static const per_sequence_t T_mtch_L1CombiningPeriodList_item_04_sequence[] = {
  { &hf_rrc_periodStart_04  , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_0_127 },
  { &hf_rrc_periodDuration_04, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_1_128 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_mtch_L1CombiningPeriodList_item_04(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_mtch_L1CombiningPeriodList_item_04, T_mtch_L1CombiningPeriodList_item_04_sequence);

  return offset;
}


static const per_sequence_t T_mtch_L1CombiningPeriodList_04_sequence_of[1] = {
  { &hf_rrc_mtch_L1CombiningPeriodList_item_04, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_mtch_L1CombiningPeriodList_item_04 },
};

static int
dissect_rrc_T_mtch_L1CombiningPeriodList_04(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_T_mtch_L1CombiningPeriodList_04, T_mtch_L1CombiningPeriodList_04_sequence_of,
                                                  1, maxMBMS_L1CP, FALSE);

  return offset;
}


static const per_sequence_t MBMS_L1CombiningSchedule_512_sequence[] = {
  { &hf_rrc_cycleOffset_04  , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_INTEGER_0_127 },
  { &hf_rrc_mtch_L1CombiningPeriodList_04, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_mtch_L1CombiningPeriodList_04 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_MBMS_L1CombiningSchedule_512(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_MBMS_L1CombiningSchedule_512, MBMS_L1CombiningSchedule_512_sequence);

  return offset;
}


static const per_sequence_t T_mtch_L1CombiningPeriodList_item_05_sequence[] = {
  { &hf_rrc_periodStart_05  , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_0_255 },
  { &hf_rrc_periodDuration_05, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_1_256 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_mtch_L1CombiningPeriodList_item_05(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_mtch_L1CombiningPeriodList_item_05, T_mtch_L1CombiningPeriodList_item_05_sequence);

  return offset;
}


static const per_sequence_t T_mtch_L1CombiningPeriodList_05_sequence_of[1] = {
  { &hf_rrc_mtch_L1CombiningPeriodList_item_05, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_mtch_L1CombiningPeriodList_item_05 },
};

static int
dissect_rrc_T_mtch_L1CombiningPeriodList_05(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_T_mtch_L1CombiningPeriodList_05, T_mtch_L1CombiningPeriodList_05_sequence_of,
                                                  1, maxMBMS_L1CP, FALSE);

  return offset;
}


static const per_sequence_t MBMS_L1CombiningSchedule_1024_sequence[] = {
  { &hf_rrc_cycleOffset_05  , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_INTEGER_0_255 },
  { &hf_rrc_mtch_L1CombiningPeriodList_05, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_mtch_L1CombiningPeriodList_05 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_MBMS_L1CombiningSchedule_1024(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_MBMS_L1CombiningSchedule_1024, MBMS_L1CombiningSchedule_1024_sequence);

  return offset;
}


static const value_string rrc_MBMS_L1CombiningSchedule_vals[] = {
  {   0, "cycleLength-32" },
  {   1, "cycleLength-64" },
  {   2, "cycleLength-128" },
  {   3, "cycleLength-256" },
  {   4, "cycleLength-512" },
  {   5, "cycleLength-1024" },
  { 0, NULL }
};

static const per_choice_t MBMS_L1CombiningSchedule_choice[] = {
  {   0, &hf_rrc_cycleLength_32  , ASN1_NO_EXTENSIONS     , dissect_rrc_MBMS_L1CombiningSchedule_32 },
  {   1, &hf_rrc_cycleLength_64  , ASN1_NO_EXTENSIONS     , dissect_rrc_MBMS_L1CombiningSchedule_64 },
  {   2, &hf_rrc_cycleLength_128 , ASN1_NO_EXTENSIONS     , dissect_rrc_MBMS_L1CombiningSchedule_128 },
  {   3, &hf_rrc_cycleLength_256 , ASN1_NO_EXTENSIONS     , dissect_rrc_MBMS_L1CombiningSchedule_256 },
  {   4, &hf_rrc_cycleLength_512 , ASN1_NO_EXTENSIONS     , dissect_rrc_MBMS_L1CombiningSchedule_512 },
  {   5, &hf_rrc_cycleLength_1024, ASN1_NO_EXTENSIONS     , dissect_rrc_MBMS_L1CombiningSchedule_1024 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_MBMS_L1CombiningSchedule(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_MBMS_L1CombiningSchedule, MBMS_L1CombiningSchedule_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t T_fdd_192_sequence[] = {
  { &hf_rrc_softComb_TimingOffset, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MBMS_SoftComb_TimingOffset },
  { &hf_rrc_mbms_L1CombiningTransmTimeDiff, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MBMS_L1CombiningTransmTimeDiff },
  { &hf_rrc_mbms_L1CombiningSchedule, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MBMS_L1CombiningSchedule },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_fdd_192(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_fdd_192, T_fdd_192_sequence);

  return offset;
}


static const value_string rrc_T_layer1Combining_vals[] = {
  {   0, "fdd" },
  {   1, "tdd" },
  { 0, NULL }
};

static const per_choice_t T_layer1Combining_choice[] = {
  {   0, &hf_rrc_fdd_198         , ASN1_NO_EXTENSIONS     , dissect_rrc_T_fdd_192 },
  {   1, &hf_rrc_tdd             , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_layer1Combining(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_layer1Combining, T_layer1Combining_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t T_sameAsCurrent_sequence[] = {
  { &hf_rrc_currentCell_SCCPCH, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MBMS_SCCPCHIdentity },
  { &hf_rrc_msch_ConfigurationInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MBMS_MSCH_ConfigurationInfo_r6 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_sameAsCurrent(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_sameAsCurrent, T_sameAsCurrent_sequence);

  return offset;
}


static const per_sequence_t MBMS_PTM_RBInformation_N_sequence[] = {
  { &hf_rrc_shortTransmissionID, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MBMS_ShortTransmissionID },
  { &hf_rrc_logicalChIdentity, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MBMS_LogicalChIdentity },
  { &hf_rrc_layer1_CombiningStatus, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_MBMS_PTM_RBInformation_N(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_MBMS_PTM_RBInformation_N, MBMS_PTM_RBInformation_N_sequence);

  return offset;
}


static const per_sequence_t MBMS_PTM_RBInformation_NList_sequence_of[1] = {
  { &hf_rrc_MBMS_PTM_RBInformation_NList_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MBMS_PTM_RBInformation_N },
};

static int
dissect_rrc_MBMS_PTM_RBInformation_NList(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_MBMS_PTM_RBInformation_NList, MBMS_PTM_RBInformation_NList_sequence_of,
                                                  1, maxRBperTrCh, FALSE);

  return offset;
}


static const per_sequence_t MBMS_TrCHInformation_Neighb_sequence[] = {
  { &hf_rrc_transpCh_Info   , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MBMS_CommonTrChIdentity },
  { &hf_rrc_transpCh_CombiningStatus, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { &hf_rrc_rbInformation_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MBMS_PTM_RBInformation_NList },
  { &hf_rrc_msch_ConfigurationInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MBMS_MSCH_ConfigurationInfo_r6 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_MBMS_TrCHInformation_Neighb(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_MBMS_TrCHInformation_Neighb, MBMS_TrCHInformation_Neighb_sequence);

  return offset;
}


static const per_sequence_t MBMS_TrCHInformation_NeighbList_sequence_of[1] = {
  { &hf_rrc_MBMS_TrCHInformation_NeighbList_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MBMS_TrCHInformation_Neighb },
};

static int
dissect_rrc_MBMS_TrCHInformation_NeighbList(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_MBMS_TrCHInformation_NeighbList, MBMS_TrCHInformation_NeighbList_sequence_of,
                                                  1, maxFACHPCH, FALSE);

  return offset;
}


static const per_sequence_t T_different_sequence[] = {
  { &hf_rrc_transpCh_InfoCommonForAllTrCh, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MBMS_CommonCCTrChIdentity },
  { &hf_rrc_transpCHInformation_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MBMS_TrCHInformation_NeighbList },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_different(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_different, T_different_sequence);

  return offset;
}


static const value_string rrc_MBMS_L23Configuration_vals[] = {
  {   0, "sameAsCurrent" },
  {   1, "different" },
  { 0, NULL }
};

static const per_choice_t MBMS_L23Configuration_choice[] = {
  {   0, &hf_rrc_sameAsCurrent   , ASN1_NO_EXTENSIONS     , dissect_rrc_T_sameAsCurrent },
  {   1, &hf_rrc_different       , ASN1_NO_EXTENSIONS     , dissect_rrc_T_different },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_MBMS_L23Configuration(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_MBMS_L23Configuration, MBMS_L23Configuration_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t MBMS_NeighbouringCellSCCPCH_r6_sequence[] = {
  { &hf_rrc_secondaryCCPCH_Info_03, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MBMS_CommonPhyChIdentity },
  { &hf_rrc_secondaryCCPCHPwrOffsetDiff, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MBMS_SCCPCHPwrOffsetDiff },
  { &hf_rrc_layer1Combining , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_layer1Combining },
  { &hf_rrc_mbms_L23Configuration, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MBMS_L23Configuration },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_MBMS_NeighbouringCellSCCPCH_r6(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_MBMS_NeighbouringCellSCCPCH_r6, MBMS_NeighbouringCellSCCPCH_r6_sequence);

  return offset;
}


static const per_sequence_t MBMS_NeighbouringCellSCCPCHList_r6_sequence_of[1] = {
  { &hf_rrc_MBMS_NeighbouringCellSCCPCHList_r6_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MBMS_NeighbouringCellSCCPCH_r6 },
};

static int
dissect_rrc_MBMS_NeighbouringCellSCCPCHList_r6(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_MBMS_NeighbouringCellSCCPCHList_r6, MBMS_NeighbouringCellSCCPCHList_r6_sequence_of,
                                                  1, maxSCCPCH, FALSE);

  return offset;
}


static const per_sequence_t SecondaryCCPCHInfoDiff_MBMS_sequence[] = {
  { &hf_rrc_secondaryScramblingCode, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_SecondaryScramblingCode },
  { &hf_rrc_sttd_Indicator  , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { &hf_rrc_sf_AndCodeNumber_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_SF256_AndCodeNumber },
  { &hf_rrc_timingOffset    , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_TimingOffset },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_SecondaryCCPCHInfoDiff_MBMS(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_SecondaryCCPCHInfoDiff_MBMS, SecondaryCCPCHInfoDiff_MBMS_sequence);

  return offset;
}


static const per_sequence_t T_fdd_193_sequence[] = {
  { &hf_rrc_softComb_TimingOffset, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MBMS_SoftComb_TimingOffset },
  { &hf_rrc_mbms_L1CombiningTransmTimeDiff, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MBMS_L1CombiningTransmTimeDiff },
  { &hf_rrc_mbms_L1CombiningSchedule, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MBMS_L1CombiningSchedule },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_fdd_193(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_fdd_193, T_fdd_193_sequence);

  return offset;
}


static const value_string rrc_T_layer1Combining_01_vals[] = {
  {   0, "fdd" },
  {   1, "tdd" },
  { 0, NULL }
};

static const per_choice_t T_layer1Combining_01_choice[] = {
  {   0, &hf_rrc_fdd_199         , ASN1_NO_EXTENSIONS     , dissect_rrc_T_fdd_193 },
  {   1, &hf_rrc_tdd             , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_layer1Combining_01(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_layer1Combining_01, T_layer1Combining_01_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t MBMS_NeighbouringCellSCCPCH_v770ext_sequence[] = {
  { &hf_rrc_secondaryCCPCH_Info_03, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MBMS_CommonPhyChIdentity },
  { &hf_rrc_secondaryCCPCH_InfoDiff, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SecondaryCCPCHInfoDiff_MBMS },
  { &hf_rrc_secondaryCCPCHPwrOffsetDiff, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MBMS_SCCPCHPwrOffsetDiff },
  { &hf_rrc_layer1Combining_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_layer1Combining_01 },
  { &hf_rrc_mbms_L23Configuration, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MBMS_L23Configuration },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_MBMS_NeighbouringCellSCCPCH_v770ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_MBMS_NeighbouringCellSCCPCH_v770ext, MBMS_NeighbouringCellSCCPCH_v770ext_sequence);

  return offset;
}


static const per_sequence_t MBMS_NeighbouringCellSCCPCHList_v770ext_sequence_of[1] = {
  { &hf_rrc_MBMS_NeighbouringCellSCCPCHList_v770ext_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MBMS_NeighbouringCellSCCPCH_v770ext },
};

static int
dissect_rrc_MBMS_NeighbouringCellSCCPCHList_v770ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_MBMS_NeighbouringCellSCCPCHList_v770ext, MBMS_NeighbouringCellSCCPCHList_v770ext_sequence_of,
                                                  1, maxSCCPCH, FALSE);

  return offset;
}


static const per_sequence_t MBMSNeighbouringCellPTMRBInformation_v770ext_IEs_sequence[] = {
  { &hf_rrc_neighbouringCellSCCPCHList_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MBMS_NeighbouringCellSCCPCHList_v770ext },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_MBMSNeighbouringCellPTMRBInformation_v770ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_MBMSNeighbouringCellPTMRBInformation_v770ext_IEs, MBMSNeighbouringCellPTMRBInformation_v770ext_IEs_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions_185_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_nonCriticalExtensions_185(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_nonCriticalExtensions_185, T_nonCriticalExtensions_185_sequence);

  return offset;
}


static const per_sequence_t T_v770NonCriticalExtensions_37_sequence[] = {
  { &hf_rrc_mbmsNeighbouringCellPTMRBInformation_v770ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MBMSNeighbouringCellPTMRBInformation_v770ext_IEs },
  { &hf_rrc_nonCriticalExtensions_185, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_nonCriticalExtensions_185 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v770NonCriticalExtensions_37(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v770NonCriticalExtensions_37, T_v770NonCriticalExtensions_37_sequence);

  return offset;
}


static const per_sequence_t MBMSNeighbouringCellPTMRBInformation_sequence[] = {
  { &hf_rrc_neighbouringCellIdentity, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_IntraFreqCellID },
  { &hf_rrc_neighbouringCellSCCPCHList, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MBMS_NeighbouringCellSCCPCHList_r6 },
  { &hf_rrc_v770NonCriticalExtensions_37, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v770NonCriticalExtensions_37 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_MBMSNeighbouringCellPTMRBInformation(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
#line 273 "../../asn1/rrc/rrc.cnf"
  col_append_str(actx->pinfo->cinfo, COL_INFO, "MBMSNeighbouringCellPTMRBInformation");
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_MBMSNeighbouringCellPTMRBInformation, MBMSNeighbouringCellPTMRBInformation_sequence);




  return offset;
}


static const value_string rrc_MBMS_RequiredUEAction_UMod_vals[] = {
  {   0, "none" },
  {   1, "acquirePTM-RBInfo" },
  {   2, "requestPTPRB" },
  { 0, NULL }
};


static int
dissect_rrc_MBMS_RequiredUEAction_UMod(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     3, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t MBMS_UnmodifiedService_r6_sequence[] = {
  { &hf_rrc_mbms_TransmissionIdentity, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MBMS_TransmissionIdentity },
  { &hf_rrc_mbms_RequiredUEAction_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MBMS_RequiredUEAction_UMod },
  { &hf_rrc_mbms_PreferredFrequency_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MBMS_PFLIndex },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_MBMS_UnmodifiedService_r6(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_MBMS_UnmodifiedService_r6, MBMS_UnmodifiedService_r6_sequence);

  return offset;
}


static const per_sequence_t MBMS_UnmodifiedServiceList_r6_sequence_of[1] = {
  { &hf_rrc_MBMS_UnmodifiedServiceList_r6_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MBMS_UnmodifiedService_r6 },
};

static int
dissect_rrc_MBMS_UnmodifiedServiceList_r6(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_MBMS_UnmodifiedServiceList_r6, MBMS_UnmodifiedServiceList_r6_sequence_of,
                                                  1, maxMBMSservUnmodif, FALSE);

  return offset;
}


static const per_sequence_t MBMS_UnmodifiedService_v770ext_sequence[] = {
  { &hf_rrc_mbsfnClusterFrequency, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MBSFN_ClusterFrequency_r7 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_MBMS_UnmodifiedService_v770ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_MBMS_UnmodifiedService_v770ext, MBMS_UnmodifiedService_v770ext_sequence);

  return offset;
}


static const per_sequence_t MBMS_UnmodifiedServiceList_v770ext_sequence_of[1] = {
  { &hf_rrc_MBMS_UnmodifiedServiceList_v770ext_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MBMS_UnmodifiedService_v770ext },
};

static int
dissect_rrc_MBMS_UnmodifiedServiceList_v770ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_MBMS_UnmodifiedServiceList_v770ext, MBMS_UnmodifiedServiceList_v770ext_sequence_of,
                                                  1, maxMBMSservUnmodif, FALSE);

  return offset;
}


static const per_sequence_t MBMSUnmodifiedServicesInformation_v770ext_IEs_sequence[] = {
  { &hf_rrc_unmodifiedServiceList_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MBMS_UnmodifiedServiceList_v770ext },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_MBMSUnmodifiedServicesInformation_v770ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_MBMSUnmodifiedServicesInformation_v770ext_IEs, MBMSUnmodifiedServicesInformation_v770ext_IEs_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions_187_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_nonCriticalExtensions_187(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_nonCriticalExtensions_187, T_nonCriticalExtensions_187_sequence);

  return offset;
}


static const per_sequence_t T_v770NonCriticalExtensions_38_sequence[] = {
  { &hf_rrc_mbmsUnmodifiedServicesInformation_v770ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MBMSUnmodifiedServicesInformation_v770ext_IEs },
  { &hf_rrc_nonCriticalExtensions_187, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_nonCriticalExtensions_187 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v770NonCriticalExtensions_38(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v770NonCriticalExtensions_38, T_v770NonCriticalExtensions_38_sequence);

  return offset;
}


static const per_sequence_t MBMSUnmodifiedServicesInformation_sequence[] = {
  { &hf_rrc_unmodifiedServiceList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MBMS_UnmodifiedServiceList_r6 },
  { &hf_rrc_v770NonCriticalExtensions_38, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v770NonCriticalExtensions_38 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_MBMSUnmodifiedServicesInformation(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
#line 281 "../../asn1/rrc/rrc.cnf"
  col_append_str(actx->pinfo->cinfo, COL_INFO, "MBMSUnmodifiedServicesInformation");
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_MBMSUnmodifiedServicesInformation, MBMSUnmodifiedServicesInformation_sequence);




  return offset;
}


static const value_string rrc_MCCH_MessageType_vals[] = {
  {   0, "mbmsAccessInformation" },
  {   1, "mbmsCommonPTMRBInformation" },
  {   2, "mbmsCurrentCellPTMRBInformation" },
  {   3, "mbmsGeneralInformation" },
  {   4, "mbmsModifiedServicesInformation" },
  {   5, "mbmsNeighbouringCellPTMRBInformation" },
  {   6, "mbmsUnmodifiedServicesInformation" },
  {   7, "spare9" },
  {   8, "spare8" },
  {   9, "spare7" },
  {  10, "spare6" },
  {  11, "spare5" },
  {  12, "spare4" },
  {  13, "spare3" },
  {  14, "spare2" },
  {  15, "spare1" },
  { 0, NULL }
};

static const per_choice_t MCCH_MessageType_choice[] = {
  {   0, &hf_rrc_mbmsAccessInformation, ASN1_NO_EXTENSIONS     , dissect_rrc_MBMSAccessInformation },
  {   1, &hf_rrc_mbmsCommonPTMRBInformation, ASN1_NO_EXTENSIONS     , dissect_rrc_MBMSCommonPTMRBInformation },
  {   2, &hf_rrc_mbmsCurrentCellPTMRBInformation, ASN1_NO_EXTENSIONS     , dissect_rrc_MBMSCurrentCellPTMRBInformation },
  {   3, &hf_rrc_mbmsGeneralInformation, ASN1_NO_EXTENSIONS     , dissect_rrc_MBMSGeneralInformation },
  {   4, &hf_rrc_mbmsModifiedServicesInformation, ASN1_NO_EXTENSIONS     , dissect_rrc_MBMSModifiedServicesInformation },
  {   5, &hf_rrc_mbmsNeighbouringCellPTMRBInformation, ASN1_NO_EXTENSIONS     , dissect_rrc_MBMSNeighbouringCellPTMRBInformation },
  {   6, &hf_rrc_mbmsUnmodifiedServicesInformation, ASN1_NO_EXTENSIONS     , dissect_rrc_MBMSUnmodifiedServicesInformation },
  {   7, &hf_rrc_spare9          , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   8, &hf_rrc_spare8          , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   9, &hf_rrc_spare7          , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {  10, &hf_rrc_spare6          , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {  11, &hf_rrc_spare5          , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {  12, &hf_rrc_spare4          , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {  13, &hf_rrc_spare3          , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {  14, &hf_rrc_spare2          , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {  15, &hf_rrc_spare1          , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_MCCH_MessageType(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_MCCH_MessageType, MCCH_MessageType_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t MCCH_Message_sequence[] = {
  { &hf_rrc_message_09      , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MCCH_MessageType },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_MCCH_Message(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_MCCH_Message, MCCH_Message_sequence);

  return offset;
}


static const per_sequence_t MBMS_ServiceTransmInfo_sequence[] = {
  { &hf_rrc_start           , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_0_255 },
  { &hf_rrc_duration        , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_1_256 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_MBMS_ServiceTransmInfo(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_MBMS_ServiceTransmInfo, MBMS_ServiceTransmInfo_sequence);

  return offset;
}


static const per_sequence_t MBMS_ServiceTransmInfoList_sequence_of[1] = {
  { &hf_rrc_MBMS_ServiceTransmInfoList_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MBMS_ServiceTransmInfo },
};

static int
dissect_rrc_MBMS_ServiceTransmInfoList(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_MBMS_ServiceTransmInfoList, MBMS_ServiceTransmInfoList_sequence_of,
                                                  1, maxMBMSTransmis, FALSE);

  return offset;
}


static const per_sequence_t MBMS_ServiceSchedulingInfo_r6_sequence[] = {
  { &hf_rrc_mbms_TransmissionIdentity, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MBMS_TransmissionIdentity },
  { &hf_rrc_mbms_ServiceTransmInfoList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MBMS_ServiceTransmInfoList },
  { &hf_rrc_nextSchedulingperiod, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_0_31 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_MBMS_ServiceSchedulingInfo_r6(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_MBMS_ServiceSchedulingInfo_r6, MBMS_ServiceSchedulingInfo_r6_sequence);

  return offset;
}


static const per_sequence_t MBMS_ServiceSchedulingInfoList_r6_sequence_of[1] = {
  { &hf_rrc_MBMS_ServiceSchedulingInfoList_r6_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MBMS_ServiceSchedulingInfo_r6 },
};

static int
dissect_rrc_MBMS_ServiceSchedulingInfoList_r6(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_MBMS_ServiceSchedulingInfoList_r6, MBMS_ServiceSchedulingInfoList_r6_sequence_of,
                                                  1, maxMBMSservSched, FALSE);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions_186_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_nonCriticalExtensions_186(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_nonCriticalExtensions_186, T_nonCriticalExtensions_186_sequence);

  return offset;
}


static const per_sequence_t MBMSSchedulingInformation_sequence[] = {
  { &hf_rrc_serviceSchedulingInfoList, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MBMS_ServiceSchedulingInfoList_r6 },
  { &hf_rrc_nonCriticalExtensions_186, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_nonCriticalExtensions_186 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_MBMSSchedulingInformation(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
#line 277 "../../asn1/rrc/rrc.cnf"
  col_append_str(actx->pinfo->cinfo, COL_INFO, "MBMSSchedulingInformation");
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_MBMSSchedulingInformation, MBMSSchedulingInformation_sequence);




  return offset;
}


static const value_string rrc_MSCH_MessageType_vals[] = {
  {   0, "mbmsSchedulingInformation" },
  {   1, "spare3" },
  {   2, "spare2" },
  {   3, "spare1" },
  { 0, NULL }
};

static const per_choice_t MSCH_MessageType_choice[] = {
  {   0, &hf_rrc_mbmsSchedulingInformation, ASN1_NO_EXTENSIONS     , dissect_rrc_MBMSSchedulingInformation },
  {   1, &hf_rrc_spare3          , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   2, &hf_rrc_spare2          , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   3, &hf_rrc_spare1          , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_MSCH_MessageType(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_MSCH_MessageType, MSCH_MessageType_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t MSCH_Message_sequence[] = {
  { &hf_rrc_message_10      , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MSCH_MessageType },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_MSCH_Message(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_MSCH_Message, MSCH_Message_sequence);

  return offset;
}


static const value_string rrc_T_supportForTwoDRXSchemesInPCH_vals[] = {
  {   0, "true" },
  { 0, NULL }
};


static int
dissect_rrc_T_supportForTwoDRXSchemesInPCH(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t CellUpdate_v7e0ext_IEs_sequence[] = {
  { &hf_rrc_supportForTwoDRXSchemesInPCH, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_supportForTwoDRXSchemesInPCH },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_CellUpdate_v7e0ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_CellUpdate_v7e0ext_IEs, CellUpdate_v7e0ext_IEs_sequence);

  return offset;
}


static const per_sequence_t CellUpdate_v7g0ext_IEs_sequence[] = {
  { &hf_rrc_measuredResultsOnRACH_v7g0ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MeasuredResultsOnRACH_v7g0ext },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_CellUpdate_v7g0ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_CellUpdate_v7g0ext_IEs, CellUpdate_v7g0ext_IEs_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions_13_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_nonCriticalExtensions_13(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_nonCriticalExtensions_13, T_nonCriticalExtensions_13_sequence);

  return offset;
}


static const per_sequence_t T_v7g0NonCriticalExtensions_02_sequence[] = {
  { &hf_rrc_cellUpdate_v7g0ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CellUpdate_v7g0ext_IEs },
  { &hf_rrc_nonCriticalExtensions_13, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_nonCriticalExtensions_13 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v7g0NonCriticalExtensions_02(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v7g0NonCriticalExtensions_02, T_v7g0NonCriticalExtensions_02_sequence);

  return offset;
}


static const per_sequence_t CellUpdate_r3_add_ext_IEs_sequence[] = {
  { &hf_rrc_cellUpdate_v7e0ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CellUpdate_v7e0ext_IEs },
  { &hf_rrc_v7g0NonCriticalExtensions_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v7g0NonCriticalExtensions_02 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_CellUpdate_r3_add_ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_CellUpdate_r3_add_ext_IEs, CellUpdate_r3_add_ext_IEs_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions_23_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_nonCriticalExtensions_23(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_nonCriticalExtensions_23, T_nonCriticalExtensions_23_sequence);

  return offset;
}


static const per_sequence_t T_v7g0NonCriticalExtensions_04_sequence[] = {
  { &hf_rrc_cellUpdateConfirm_v7g0ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CellUpdateConfirm_v7g0ext_IEs },
  { &hf_rrc_nonCriticalExtensions_23, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_nonCriticalExtensions_23 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v7g0NonCriticalExtensions_04(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v7g0NonCriticalExtensions_04, T_v7g0NonCriticalExtensions_04_sequence);

  return offset;
}


static const per_sequence_t T_v7f0NonCriticalExtensions_02_sequence[] = {
  { &hf_rrc_cellUpdateConfirm_v7f0ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CellUpdateConfirm_v7f0ext_IEs },
  { &hf_rrc_v7g0NonCriticalExtensions_04, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v7g0NonCriticalExtensions_04 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v7f0NonCriticalExtensions_02(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v7f0NonCriticalExtensions_02, T_v7f0NonCriticalExtensions_02_sequence);

  return offset;
}


static const per_sequence_t CellUpdateConfirm_r7_add_ext_IEs_sequence[] = {
  { &hf_rrc_cellUpdateConfirm_v7d0ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CellUpdateConfirm_v7d0ext_IEs },
  { &hf_rrc_v7f0NonCriticalExtensions_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v7f0NonCriticalExtensions_02 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_CellUpdateConfirm_r7_add_ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_CellUpdateConfirm_r7_add_ext_IEs, CellUpdateConfirm_r7_add_ext_IEs_sequence);

  return offset;
}



static int
dissect_rrc_S_RNTI_2(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_bit_string(tvb, offset, actx, tree, hf_index,
                                     10, 10, FALSE, NULL);

  return offset;
}


static const per_sequence_t U_RNTI_Short_sequence[] = {
  { &hf_rrc_srnc_Identity   , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SRNC_Identity },
  { &hf_rrc_s_RNTI_2        , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_S_RNTI_2 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_U_RNTI_Short(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_U_RNTI_Short, U_RNTI_Short_sequence);

  return offset;
}


static const per_sequence_t T_fdd_06_sequence[] = {
  { &hf_rrc_dummy_06        , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_PDSCH_Information },
  { &hf_rrc_dummy2_04       , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CPCH_SetInfo },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_fdd_06(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_fdd_06, T_fdd_06_sequence);

  return offset;
}


static const value_string rrc_T_modeSpecificInfo_vals[] = {
  {   0, "fdd" },
  {   1, "tdd" },
  { 0, NULL }
};

static const per_choice_t T_modeSpecificInfo_choice[] = {
  {   0, &hf_rrc_fdd_06          , ASN1_NO_EXTENSIONS     , dissect_rrc_T_fdd_06 },
  {   1, &hf_rrc_tdd             , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_modeSpecificInfo(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_modeSpecificInfo, T_modeSpecificInfo_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t T_complete_sequence[] = {
  { &hf_rrc_srb_InformationSetupList, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SRB_InformationSetupList },
  { &hf_rrc_rab_InformationSetupList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RAB_InformationSetupList },
  { &hf_rrc_ul_CommonTransChInfo, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UL_CommonTransChInfo },
  { &hf_rrc_ul_AddReconfTransChInfoList, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UL_AddReconfTransChInfoList },
  { &hf_rrc_dl_CommonTransChInfo, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DL_CommonTransChInfo },
  { &hf_rrc_dl_AddReconfTransChInfoList, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DL_AddReconfTransChInfoList },
  { &hf_rrc_ul_DPCH_Info_03 , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UL_DPCH_Info },
  { &hf_rrc_modeSpecificInfo, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_modeSpecificInfo },
  { &hf_rrc_dl_CommonInformation, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DL_CommonInformation },
  { &hf_rrc_dl_InformationPerRL_List, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DL_InformationPerRL_List },
  { &hf_rrc_frequencyInfo   , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_FrequencyInfo },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_complete(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_complete, T_complete_sequence);

  return offset;
}



static int
dissect_rrc_DefaultConfigIdentity(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            0U, 10U, NULL, FALSE);

  return offset;
}


static const per_sequence_t T_defaultConfig_sequence[] = {
  { &hf_rrc_defaultConfigMode, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DefaultConfigMode },
  { &hf_rrc_defaultConfigIdentity, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DefaultConfigIdentity },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_defaultConfig(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_defaultConfig, T_defaultConfig_sequence);

  return offset;
}


static const value_string rrc_T_preConfigMode_vals[] = {
  {   0, "predefinedConfigIdentity" },
  {   1, "defaultConfig" },
  { 0, NULL }
};

static const per_choice_t T_preConfigMode_choice[] = {
  {   0, &hf_rrc_predefinedConfigIdentity, ASN1_NO_EXTENSIONS     , dissect_rrc_PredefinedConfigIdentity },
  {   1, &hf_rrc_defaultConfig   , ASN1_NO_EXTENSIONS     , dissect_rrc_T_defaultConfig },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_preConfigMode(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_preConfigMode, T_preConfigMode_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t RAB_Info_Post_sequence[] = {
  { &hf_rrc_rab_Identity    , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RAB_Identity },
  { &hf_rrc_cn_DomainIdentity, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CN_DomainIdentity },
  { &hf_rrc_nas_Synchronisation_Indicator, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_NAS_Synchronisation_Indicator },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_RAB_Info_Post(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_RAB_Info_Post, RAB_Info_Post_sequence);

  return offset;
}



static int
dissect_rrc_DPCCH_PowerOffset2(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            -28, -13, NULL, FALSE);

  return offset;
}


static const per_sequence_t UL_DPCH_PowerControlInfoPostFDD_sequence[] = {
  { &hf_rrc_dpcch_PowerOffset_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DPCCH_PowerOffset2 },
  { &hf_rrc_pc_Preamble     , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PC_Preamble },
  { &hf_rrc_sRB_delay       , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SRB_delay },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UL_DPCH_PowerControlInfoPostFDD(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UL_DPCH_PowerControlInfoPostFDD, UL_DPCH_PowerControlInfoPostFDD_sequence);

  return offset;
}



static int
dissect_rrc_ReducedScramblingCodeNumber(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            0U, 8191U, NULL, FALSE);

  return offset;
}


static const per_sequence_t UL_DPCH_InfoPostFDD_sequence[] = {
  { &hf_rrc_ul_DPCH_PowerControlInfo_06, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UL_DPCH_PowerControlInfoPostFDD },
  { &hf_rrc_scramblingCodeType, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_ScramblingCodeType },
  { &hf_rrc_reducedScramblingCodeNumber, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_ReducedScramblingCodeNumber },
  { &hf_rrc_spreadingFactor_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SpreadingFactor },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UL_DPCH_InfoPostFDD(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UL_DPCH_InfoPostFDD, UL_DPCH_InfoPostFDD_sequence);

  return offset;
}


static const per_sequence_t DL_DPCH_InfoCommonPost_sequence[] = {
  { &hf_rrc_dl_DPCH_PowerControlInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_DPCH_PowerControlInfo },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_DL_DPCH_InfoCommonPost(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_DL_DPCH_InfoCommonPost, DL_DPCH_InfoCommonPost_sequence);

  return offset;
}


static const per_sequence_t DL_CommonInformationPost_sequence[] = {
  { &hf_rrc_dl_DPCH_InfoCommon_03, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DL_DPCH_InfoCommonPost },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_DL_CommonInformationPost(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_DL_CommonInformationPost, DL_CommonInformationPost_sequence);

  return offset;
}


static const per_sequence_t DL_DPCH_InfoPerRL_PostFDD_sequence[] = {
  { &hf_rrc_pCPICH_UsageForChannelEst, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PCPICH_UsageForChannelEst },
  { &hf_rrc_dl_ChannelisationCode_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DL_ChannelisationCode },
  { &hf_rrc_tpc_CombinationIndex, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TPC_CombinationIndex },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_DL_DPCH_InfoPerRL_PostFDD(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_DL_DPCH_InfoPerRL_PostFDD, DL_DPCH_InfoPerRL_PostFDD_sequence);

  return offset;
}


static const per_sequence_t DL_InformationPerRL_PostFDD_sequence[] = {
  { &hf_rrc_primaryCPICH_Info, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PrimaryCPICH_Info },
  { &hf_rrc_dl_DPCH_InfoPerRL_05, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DL_DPCH_InfoPerRL_PostFDD },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_DL_InformationPerRL_PostFDD(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_DL_InformationPerRL_PostFDD, DL_InformationPerRL_PostFDD_sequence);

  return offset;
}


static const per_sequence_t DL_InformationPerRL_ListPostFDD_sequence_of[1] = {
  { &hf_rrc_DL_InformationPerRL_ListPostFDD_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DL_InformationPerRL_PostFDD },
};

static int
dissect_rrc_DL_InformationPerRL_ListPostFDD(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_DL_InformationPerRL_ListPostFDD, DL_InformationPerRL_ListPostFDD_sequence_of,
                                                  1, maxRL, FALSE);

  return offset;
}


static const per_sequence_t T_fdd_07_sequence[] = {
  { &hf_rrc_ul_DPCH_Info_04 , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UL_DPCH_InfoPostFDD },
  { &hf_rrc_dl_CommonInformationPost, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DL_CommonInformationPost },
  { &hf_rrc_dl_InformationPerRL_List_07, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DL_InformationPerRL_ListPostFDD },
  { &hf_rrc_frequencyInfo_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_FrequencyInfoFDD },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_fdd_07(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_fdd_07, T_fdd_07_sequence);

  return offset;
}


static const per_sequence_t UL_DPCH_PowerControlInfoPostTDD_sequence[] = {
  { &hf_rrc_ul_TargetSIR    , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UL_TargetSIR },
  { &hf_rrc_ul_TimeslotInterference, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TDD_UL_Interference },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UL_DPCH_PowerControlInfoPostTDD(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UL_DPCH_PowerControlInfoPostTDD, UL_DPCH_PowerControlInfoPostTDD_sequence);

  return offset;
}


static const per_sequence_t UL_DPCH_InfoPostTDD_sequence[] = {
  { &hf_rrc_ul_DPCH_PowerControlInfo_07, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UL_DPCH_PowerControlInfoPostTDD },
  { &hf_rrc_ul_TimingAdvance_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_TimingAdvanceControl },
  { &hf_rrc_ul_CCTrCH_TimeslotsCodes, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UplinkTimeslotsCodes },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UL_DPCH_InfoPostTDD(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UL_DPCH_InfoPostTDD, UL_DPCH_InfoPostTDD_sequence);

  return offset;
}


static const per_sequence_t T_syncCase1_02_sequence[] = {
  { &hf_rrc_timeslot        , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TimeslotNumber },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_syncCase1_02(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_syncCase1_02, T_syncCase1_02_sequence);

  return offset;
}


static const per_sequence_t T_syncCase2_02_sequence[] = {
  { &hf_rrc_timeslotSync2   , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TimeslotSync2 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_syncCase2_02(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_syncCase2_02, T_syncCase2_02_sequence);

  return offset;
}


static const value_string rrc_T_syncCase_02_vals[] = {
  {   0, "syncCase1" },
  {   1, "syncCase2" },
  { 0, NULL }
};

static const per_choice_t T_syncCase_02_choice[] = {
  {   0, &hf_rrc_syncCase1_02    , ASN1_NO_EXTENSIONS     , dissect_rrc_T_syncCase1_02 },
  {   1, &hf_rrc_syncCase2_02    , ASN1_NO_EXTENSIONS     , dissect_rrc_T_syncCase2_02 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_syncCase_02(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_syncCase_02, T_syncCase_02_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t PrimaryCCPCH_InfoPost_sequence[] = {
  { &hf_rrc_syncCase_02     , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_syncCase_02 },
  { &hf_rrc_cellParametersID, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CellParametersID },
  { &hf_rrc_sctd_Indicator  , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_PrimaryCCPCH_InfoPost(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_PrimaryCCPCH_InfoPost, PrimaryCCPCH_InfoPost_sequence);

  return offset;
}


static const per_sequence_t DL_DPCH_InfoPerRL_PostTDD_sequence[] = {
  { &hf_rrc_dl_DPCH_TimeslotsCodes, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DownlinkTimeslotsCodes },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_DL_DPCH_InfoPerRL_PostTDD(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_DL_DPCH_InfoPerRL_PostTDD, DL_DPCH_InfoPerRL_PostTDD_sequence);

  return offset;
}


static const per_sequence_t DL_InformationPerRL_PostTDD_sequence[] = {
  { &hf_rrc_primaryCCPCH_Info, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PrimaryCCPCH_InfoPost },
  { &hf_rrc_dl_DPCH_InfoPerRL_06, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DL_DPCH_InfoPerRL_PostTDD },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_DL_InformationPerRL_PostTDD(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_DL_InformationPerRL_PostTDD, DL_InformationPerRL_PostTDD_sequence);

  return offset;
}


static const per_sequence_t T_tdd_sequence[] = {
  { &hf_rrc_ul_DPCH_Info_05 , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UL_DPCH_InfoPostTDD },
  { &hf_rrc_dl_CommonInformationPost, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DL_CommonInformationPost },
  { &hf_rrc_dl_InformationPerRL, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DL_InformationPerRL_PostTDD },
  { &hf_rrc_frequencyInfo_02, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_FrequencyInfoTDD },
  { &hf_rrc_primaryCCPCH_TX_Power, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PrimaryCCPCH_TX_Power },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_tdd(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_tdd, T_tdd_sequence);

  return offset;
}


static const value_string rrc_T_modeSpecificInfo_01_vals[] = {
  {   0, "fdd" },
  {   1, "tdd" },
  { 0, NULL }
};

static const per_choice_t T_modeSpecificInfo_01_choice[] = {
  {   0, &hf_rrc_fdd_07          , ASN1_NO_EXTENSIONS     , dissect_rrc_T_fdd_07 },
  {   1, &hf_rrc_tdd_01          , ASN1_NO_EXTENSIONS     , dissect_rrc_T_tdd },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_modeSpecificInfo_01(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_modeSpecificInfo_01, T_modeSpecificInfo_01_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t T_preconfiguration_sequence[] = {
  { &hf_rrc_preConfigMode   , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_preConfigMode },
  { &hf_rrc_rab_Info        , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RAB_Info_Post },
  { &hf_rrc_modeSpecificInfo_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_modeSpecificInfo_01 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_preconfiguration(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_preconfiguration, T_preconfiguration_sequence);

  return offset;
}


static const value_string rrc_T_specificationMode_vals[] = {
  {   0, "complete" },
  {   1, "preconfiguration" },
  { 0, NULL }
};

static const per_choice_t T_specificationMode_choice[] = {
  {   0, &hf_rrc_complete        , ASN1_NO_EXTENSIONS     , dissect_rrc_T_complete },
  {   1, &hf_rrc_preconfiguration, ASN1_NO_EXTENSIONS     , dissect_rrc_T_preconfiguration },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_specificationMode(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_specificationMode, T_specificationMode_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t HandoverToUTRANCommand_r3_IEs_sequence[] = {
  { &hf_rrc_new_U_RNTI_01   , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_U_RNTI_Short },
  { &hf_rrc_dummy_08        , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_ActivationTime },
  { &hf_rrc_cipheringAlgorithm, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CipheringAlgorithm },
  { &hf_rrc_specificationMode, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_specificationMode },
  { &hf_rrc_maxAllowedUL_TX_Power, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MaxAllowedUL_TX_Power },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_HandoverToUTRANCommand_r3_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_HandoverToUTRANCommand_r3_IEs, HandoverToUTRANCommand_r3_IEs_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions_37_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_nonCriticalExtensions_37(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_nonCriticalExtensions_37, T_nonCriticalExtensions_37_sequence);

  return offset;
}


static const per_sequence_t T_r3_08_sequence[] = {
  { &hf_rrc_handoverToUTRANCommand_r3, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_HandoverToUTRANCommand_r3_IEs },
  { &hf_rrc_nonCriticalExtensions_37, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_nonCriticalExtensions_37 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_r3_08(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_r3_08, T_r3_08_sequence);

  return offset;
}


static const per_sequence_t T_fdd_08_sequence[] = {
  { &hf_rrc_dummy_06        , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_PDSCH_Information },
  { &hf_rrc_dummy2_04       , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CPCH_SetInfo },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_fdd_08(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_fdd_08, T_fdd_08_sequence);

  return offset;
}


static const value_string rrc_T_modeSpecificInfo_02_vals[] = {
  {   0, "fdd" },
  {   1, "tdd" },
  { 0, NULL }
};

static const per_choice_t T_modeSpecificInfo_02_choice[] = {
  {   0, &hf_rrc_fdd_08          , ASN1_NO_EXTENSIONS     , dissect_rrc_T_fdd_08 },
  {   1, &hf_rrc_tdd             , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_modeSpecificInfo_02(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_modeSpecificInfo_02, T_modeSpecificInfo_02_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t T_complete_01_sequence[] = {
  { &hf_rrc_srb_InformationSetupList, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SRB_InformationSetupList },
  { &hf_rrc_rab_InformationSetupList_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RAB_InformationSetupList_r4 },
  { &hf_rrc_ul_CommonTransChInfo_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UL_CommonTransChInfo_r4 },
  { &hf_rrc_ul_AddReconfTransChInfoList, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UL_AddReconfTransChInfoList },
  { &hf_rrc_dl_CommonTransChInfo_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DL_CommonTransChInfo_r4 },
  { &hf_rrc_dl_AddReconfTransChInfoList_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DL_AddReconfTransChInfoList_r4 },
  { &hf_rrc_ul_DPCH_Info_06 , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UL_DPCH_Info_r4 },
  { &hf_rrc_modeSpecificInfo_02, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_modeSpecificInfo_02 },
  { &hf_rrc_dl_CommonInformation_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DL_CommonInformation_r4 },
  { &hf_rrc_dl_InformationPerRL_List_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DL_InformationPerRL_List_r4 },
  { &hf_rrc_frequencyInfo   , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_FrequencyInfo },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_complete_01(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_complete_01, T_complete_01_sequence);

  return offset;
}



static int
dissect_rrc_DefaultConfigIdentity_r4(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            0U, 12U, NULL, FALSE);

  return offset;
}


static const per_sequence_t T_defaultConfig_01_sequence[] = {
  { &hf_rrc_defaultConfigMode, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DefaultConfigMode },
  { &hf_rrc_defaultConfigIdentity_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DefaultConfigIdentity_r4 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_defaultConfig_01(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_defaultConfig_01, T_defaultConfig_01_sequence);

  return offset;
}


static const value_string rrc_T_preConfigMode_01_vals[] = {
  {   0, "predefinedConfigIdentity" },
  {   1, "defaultConfig" },
  { 0, NULL }
};

static const per_choice_t T_preConfigMode_01_choice[] = {
  {   0, &hf_rrc_predefinedConfigIdentity, ASN1_NO_EXTENSIONS     , dissect_rrc_PredefinedConfigIdentity },
  {   1, &hf_rrc_defaultConfig_01, ASN1_NO_EXTENSIONS     , dissect_rrc_T_defaultConfig_01 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_preConfigMode_01(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_preConfigMode_01, T_preConfigMode_01_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t T_fdd_09_sequence[] = {
  { &hf_rrc_ul_DPCH_Info_04 , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UL_DPCH_InfoPostFDD },
  { &hf_rrc_dl_CommonInformationPost, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DL_CommonInformationPost },
  { &hf_rrc_dl_InformationPerRL_List_07, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DL_InformationPerRL_ListPostFDD },
  { &hf_rrc_frequencyInfo_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_FrequencyInfoFDD },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_fdd_09(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_fdd_09, T_fdd_09_sequence);

  return offset;
}


static const per_sequence_t T_tdd384_sequence[] = {
  { &hf_rrc_ul_DPCH_Info_05 , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UL_DPCH_InfoPostTDD },
  { &hf_rrc_dl_InformationPerRL, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DL_InformationPerRL_PostTDD },
  { &hf_rrc_frequencyInfo_02, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_FrequencyInfoTDD },
  { &hf_rrc_primaryCCPCH_TX_Power, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PrimaryCCPCH_TX_Power },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_tdd384(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_tdd384, T_tdd384_sequence);

  return offset;
}


static const per_sequence_t UL_DPCH_PowerControlInfoPostTDD_LCR_r4_sequence[] = {
  { &hf_rrc_ul_TargetSIR    , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UL_TargetSIR },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UL_DPCH_PowerControlInfoPostTDD_LCR_r4(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UL_DPCH_PowerControlInfoPostTDD_LCR_r4, UL_DPCH_PowerControlInfoPostTDD_LCR_r4_sequence);

  return offset;
}


static const per_sequence_t T_enabled_03_sequence[] = {
  { &hf_rrc_ul_SynchronisationParameters, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_SynchronisationParameters_r4 },
  { &hf_rrc_synchronisationParameters, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_SynchronisationParameters_r4 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_enabled_03(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_enabled_03, T_enabled_03_sequence);

  return offset;
}


static const value_string rrc_UL_TimingAdvanceControl_LCR_r4_vals[] = {
  {   0, "disabled" },
  {   1, "enabled" },
  { 0, NULL }
};

static const per_choice_t UL_TimingAdvanceControl_LCR_r4_choice[] = {
  {   0, &hf_rrc_disabled        , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   1, &hf_rrc_enabled_03      , ASN1_NO_EXTENSIONS     , dissect_rrc_T_enabled_03 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_UL_TimingAdvanceControl_LCR_r4(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_UL_TimingAdvanceControl_LCR_r4, UL_TimingAdvanceControl_LCR_r4_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t UL_DPCH_InfoPostTDD_LCR_r4_sequence[] = {
  { &hf_rrc_ul_DPCH_PowerControlInfo_08, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UL_DPCH_PowerControlInfoPostTDD_LCR_r4 },
  { &hf_rrc_ul_TimingAdvance_04, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_TimingAdvanceControl_LCR_r4 },
  { &hf_rrc_ul_CCTrCH_TimeslotsCodes_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UplinkTimeslotsCodes_LCR_r4 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UL_DPCH_InfoPostTDD_LCR_r4(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UL_DPCH_InfoPostTDD_LCR_r4, UL_DPCH_InfoPostTDD_LCR_r4_sequence);

  return offset;
}


static const per_sequence_t PrimaryCCPCH_InfoPostTDD_LCR_r4_sequence[] = {
  { &hf_rrc_tstd_Indicator  , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { &hf_rrc_cellParametersID, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CellParametersID },
  { &hf_rrc_sctd_Indicator  , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_PrimaryCCPCH_InfoPostTDD_LCR_r4(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_PrimaryCCPCH_InfoPostTDD_LCR_r4, PrimaryCCPCH_InfoPostTDD_LCR_r4_sequence);

  return offset;
}


static const per_sequence_t DL_DPCH_InfoPerRL_PostTDD_LCR_r4_sequence[] = {
  { &hf_rrc_dl_CCTrCH_TimeslotsCodes_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DownlinkTimeslotsCodes_LCR_r4 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_DL_DPCH_InfoPerRL_PostTDD_LCR_r4(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_DL_DPCH_InfoPerRL_PostTDD_LCR_r4, DL_DPCH_InfoPerRL_PostTDD_LCR_r4_sequence);

  return offset;
}


static const per_sequence_t DL_InformationPerRL_PostTDD_LCR_r4_sequence[] = {
  { &hf_rrc_primaryCCPCH_Info_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PrimaryCCPCH_InfoPostTDD_LCR_r4 },
  { &hf_rrc_dl_DPCH_InfoPerRL_07, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DL_DPCH_InfoPerRL_PostTDD_LCR_r4 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_DL_InformationPerRL_PostTDD_LCR_r4(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_DL_InformationPerRL_PostTDD_LCR_r4, DL_InformationPerRL_PostTDD_LCR_r4_sequence);

  return offset;
}


static const per_sequence_t T_tdd128_sequence[] = {
  { &hf_rrc_ul_DPCH_Info_07 , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UL_DPCH_InfoPostTDD_LCR_r4 },
  { &hf_rrc_dl_InformationPerRL_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DL_InformationPerRL_PostTDD_LCR_r4 },
  { &hf_rrc_frequencyInfo_02, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_FrequencyInfoTDD },
  { &hf_rrc_primaryCCPCH_TX_Power, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PrimaryCCPCH_TX_Power },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_tdd128(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_tdd128, T_tdd128_sequence);

  return offset;
}


static const value_string rrc_T_tdd_01_vals[] = {
  {   0, "tdd384" },
  {   1, "tdd128" },
  { 0, NULL }
};

static const per_choice_t T_tdd_01_choice[] = {
  {   0, &hf_rrc_tdd384          , ASN1_NO_EXTENSIONS     , dissect_rrc_T_tdd384 },
  {   1, &hf_rrc_tdd128          , ASN1_NO_EXTENSIONS     , dissect_rrc_T_tdd128 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_tdd_01(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_tdd_01, T_tdd_01_choice,
                                 NULL);

  return offset;
}


static const value_string rrc_T_modeSpecificInfo_03_vals[] = {
  {   0, "fdd" },
  {   1, "tdd" },
  { 0, NULL }
};

static const per_choice_t T_modeSpecificInfo_03_choice[] = {
  {   0, &hf_rrc_fdd_09          , ASN1_NO_EXTENSIONS     , dissect_rrc_T_fdd_09 },
  {   1, &hf_rrc_tdd_02          , ASN1_NO_EXTENSIONS     , dissect_rrc_T_tdd_01 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_modeSpecificInfo_03(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_modeSpecificInfo_03, T_modeSpecificInfo_03_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t T_preconfiguration_01_sequence[] = {
  { &hf_rrc_preConfigMode_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_preConfigMode_01 },
  { &hf_rrc_rab_Info        , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RAB_Info_Post },
  { &hf_rrc_modeSpecificInfo_03, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_modeSpecificInfo_03 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_preconfiguration_01(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_preconfiguration_01, T_preconfiguration_01_sequence);

  return offset;
}


static const value_string rrc_T_specificationMode_01_vals[] = {
  {   0, "complete" },
  {   1, "preconfiguration" },
  { 0, NULL }
};

static const per_choice_t T_specificationMode_01_choice[] = {
  {   0, &hf_rrc_complete_01     , ASN1_NO_EXTENSIONS     , dissect_rrc_T_complete_01 },
  {   1, &hf_rrc_preconfiguration_01, ASN1_NO_EXTENSIONS     , dissect_rrc_T_preconfiguration_01 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_specificationMode_01(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_specificationMode_01, T_specificationMode_01_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t HandoverToUTRANCommand_r4_IEs_sequence[] = {
  { &hf_rrc_new_U_RNTI_01   , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_U_RNTI_Short },
  { &hf_rrc_cipheringAlgorithm, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CipheringAlgorithm },
  { &hf_rrc_specificationMode_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_specificationMode_01 },
  { &hf_rrc_maxAllowedUL_TX_Power, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MaxAllowedUL_TX_Power },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_HandoverToUTRANCommand_r4_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_HandoverToUTRANCommand_r4_IEs, HandoverToUTRANCommand_r4_IEs_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions_38_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_nonCriticalExtensions_38(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_nonCriticalExtensions_38, T_nonCriticalExtensions_38_sequence);

  return offset;
}


static const per_sequence_t T_r4_02_sequence[] = {
  { &hf_rrc_handoverToUTRANCommand_r4, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_HandoverToUTRANCommand_r4_IEs },
  { &hf_rrc_nonCriticalExtensions_38, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_nonCriticalExtensions_38 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_r4_02(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_r4_02, T_r4_02_sequence);

  return offset;
}


static const per_sequence_t T_fdd_10_sequence[] = {
  { &hf_rrc_dummy_06        , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_PDSCH_Information },
  { &hf_rrc_dummy2_04       , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CPCH_SetInfo },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_fdd_10(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_fdd_10, T_fdd_10_sequence);

  return offset;
}


static const value_string rrc_T_modeSpecificInfo_04_vals[] = {
  {   0, "fdd" },
  {   1, "tdd" },
  { 0, NULL }
};

static const per_choice_t T_modeSpecificInfo_04_choice[] = {
  {   0, &hf_rrc_fdd_10          , ASN1_NO_EXTENSIONS     , dissect_rrc_T_fdd_10 },
  {   1, &hf_rrc_tdd             , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_modeSpecificInfo_04(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_modeSpecificInfo_04, T_modeSpecificInfo_04_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t T_complete_02_sequence[] = {
  { &hf_rrc_srb_InformationSetupList_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SRB_InformationSetupList_r5 },
  { &hf_rrc_rab_InformationSetupList_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RAB_InformationSetupList_r5 },
  { &hf_rrc_ul_CommonTransChInfo_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UL_CommonTransChInfo_r4 },
  { &hf_rrc_ul_AddReconfTransChInfoList, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UL_AddReconfTransChInfoList },
  { &hf_rrc_dl_CommonTransChInfo_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DL_CommonTransChInfo_r4 },
  { &hf_rrc_dl_AddReconfTransChInfoList_02, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DL_AddReconfTransChInfoList_r5 },
  { &hf_rrc_ul_DPCH_Info_08 , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UL_DPCH_Info_r5 },
  { &hf_rrc_modeSpecificInfo_04, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_modeSpecificInfo_04 },
  { &hf_rrc_dl_CommonInformation_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DL_CommonInformation_r4 },
  { &hf_rrc_dl_InformationPerRL_List_02, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DL_InformationPerRL_List_r5 },
  { &hf_rrc_frequencyInfo   , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_FrequencyInfo },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_complete_02(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_complete_02, T_complete_02_sequence);

  return offset;
}


static const per_sequence_t T_defaultConfig_02_sequence[] = {
  { &hf_rrc_defaultConfigMode, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DefaultConfigMode },
  { &hf_rrc_defaultConfigIdentity_02, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DefaultConfigIdentity_r5 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_defaultConfig_02(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_defaultConfig_02, T_defaultConfig_02_sequence);

  return offset;
}


static const value_string rrc_T_preConfigMode_02_vals[] = {
  {   0, "predefinedConfigIdentity" },
  {   1, "defaultConfig" },
  { 0, NULL }
};

static const per_choice_t T_preConfigMode_02_choice[] = {
  {   0, &hf_rrc_predefinedConfigIdentity, ASN1_NO_EXTENSIONS     , dissect_rrc_PredefinedConfigIdentity },
  {   1, &hf_rrc_defaultConfig_02, ASN1_NO_EXTENSIONS     , dissect_rrc_T_defaultConfig_02 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_preConfigMode_02(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_preConfigMode_02, T_preConfigMode_02_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t T_fdd_11_sequence[] = {
  { &hf_rrc_ul_DPCH_Info_04 , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UL_DPCH_InfoPostFDD },
  { &hf_rrc_dl_CommonInformationPost, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DL_CommonInformationPost },
  { &hf_rrc_dl_InformationPerRL_List_07, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DL_InformationPerRL_ListPostFDD },
  { &hf_rrc_frequencyInfo_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_FrequencyInfoFDD },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_fdd_11(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_fdd_11, T_fdd_11_sequence);

  return offset;
}


static const per_sequence_t T_tdd384_01_sequence[] = {
  { &hf_rrc_ul_DPCH_Info_05 , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UL_DPCH_InfoPostTDD },
  { &hf_rrc_dl_InformationPerRL, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DL_InformationPerRL_PostTDD },
  { &hf_rrc_frequencyInfo_02, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_FrequencyInfoTDD },
  { &hf_rrc_primaryCCPCH_TX_Power, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PrimaryCCPCH_TX_Power },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_tdd384_01(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_tdd384_01, T_tdd384_01_sequence);

  return offset;
}


static const per_sequence_t T_tdd128_01_sequence[] = {
  { &hf_rrc_ul_DPCH_Info_07 , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UL_DPCH_InfoPostTDD_LCR_r4 },
  { &hf_rrc_dl_InformationPerRL_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DL_InformationPerRL_PostTDD_LCR_r4 },
  { &hf_rrc_frequencyInfo_02, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_FrequencyInfoTDD },
  { &hf_rrc_primaryCCPCH_TX_Power, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PrimaryCCPCH_TX_Power },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_tdd128_01(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_tdd128_01, T_tdd128_01_sequence);

  return offset;
}


static const value_string rrc_T_tdd_02_vals[] = {
  {   0, "tdd384" },
  {   1, "tdd128" },
  { 0, NULL }
};

static const per_choice_t T_tdd_02_choice[] = {
  {   0, &hf_rrc_tdd384_01       , ASN1_NO_EXTENSIONS     , dissect_rrc_T_tdd384_01 },
  {   1, &hf_rrc_tdd128_01       , ASN1_NO_EXTENSIONS     , dissect_rrc_T_tdd128_01 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_tdd_02(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_tdd_02, T_tdd_02_choice,
                                 NULL);

  return offset;
}


static const value_string rrc_T_modeSpecificInfo_05_vals[] = {
  {   0, "fdd" },
  {   1, "tdd" },
  { 0, NULL }
};

static const per_choice_t T_modeSpecificInfo_05_choice[] = {
  {   0, &hf_rrc_fdd_11          , ASN1_NO_EXTENSIONS     , dissect_rrc_T_fdd_11 },
  {   1, &hf_rrc_tdd_03          , ASN1_NO_EXTENSIONS     , dissect_rrc_T_tdd_02 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_modeSpecificInfo_05(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_modeSpecificInfo_05, T_modeSpecificInfo_05_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t T_preconfiguration_02_sequence[] = {
  { &hf_rrc_preConfigMode_02, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_preConfigMode_02 },
  { &hf_rrc_rab_Info        , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RAB_Info_Post },
  { &hf_rrc_modeSpecificInfo_05, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_modeSpecificInfo_05 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_preconfiguration_02(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_preconfiguration_02, T_preconfiguration_02_sequence);

  return offset;
}


static const value_string rrc_T_specificationMode_02_vals[] = {
  {   0, "complete" },
  {   1, "preconfiguration" },
  { 0, NULL }
};

static const per_choice_t T_specificationMode_02_choice[] = {
  {   0, &hf_rrc_complete_02     , ASN1_NO_EXTENSIONS     , dissect_rrc_T_complete_02 },
  {   1, &hf_rrc_preconfiguration_02, ASN1_NO_EXTENSIONS     , dissect_rrc_T_preconfiguration_02 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_specificationMode_02(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_specificationMode_02, T_specificationMode_02_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t HandoverToUTRANCommand_r5_IEs_sequence[] = {
  { &hf_rrc_new_U_RNTI_01   , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_U_RNTI_Short },
  { &hf_rrc_cipheringAlgorithm, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CipheringAlgorithm },
  { &hf_rrc_specificationMode_02, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_specificationMode_02 },
  { &hf_rrc_maxAllowedUL_TX_Power, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MaxAllowedUL_TX_Power },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_HandoverToUTRANCommand_r5_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_HandoverToUTRANCommand_r5_IEs, HandoverToUTRANCommand_r5_IEs_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions_39_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_nonCriticalExtensions_39(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_nonCriticalExtensions_39, T_nonCriticalExtensions_39_sequence);

  return offset;
}


static const per_sequence_t T_r5_02_sequence[] = {
  { &hf_rrc_handoverToUTRANCommand_r5, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_HandoverToUTRANCommand_r5_IEs },
  { &hf_rrc_nonCriticalExtensions_39, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_nonCriticalExtensions_39 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_r5_02(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_r5_02, T_r5_02_sequence);

  return offset;
}


static const per_sequence_t T_complete_03_sequence[] = {
  { &hf_rrc_srb_InformationSetupList_02, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SRB_InformationSetupList_r6 },
  { &hf_rrc_rab_InformationSetupList_03, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RAB_InformationSetupList_r6 },
  { &hf_rrc_ul_CommonTransChInfo_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UL_CommonTransChInfo_r4 },
  { &hf_rrc_ul_AddReconfTransChInfoList_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UL_AddReconfTransChInfoList_r6 },
  { &hf_rrc_dl_CommonTransChInfo_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DL_CommonTransChInfo_r4 },
  { &hf_rrc_dl_AddReconfTransChInfoList_02, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DL_AddReconfTransChInfoList_r5 },
  { &hf_rrc_ul_DPCH_Info    , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UL_DPCH_Info_r6 },
  { &hf_rrc_ul_EDCH_Information, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_EDCH_Information_r6 },
  { &hf_rrc_dl_HSPDSCH_Information_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_HSPDSCH_Information_r6 },
  { &hf_rrc_dl_CommonInformation_03, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DL_CommonInformation_r6 },
  { &hf_rrc_dl_InformationPerRL_List_03, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DL_InformationPerRL_List_r6 },
  { &hf_rrc_frequencyInfo   , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_FrequencyInfo },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_complete_03(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_complete_03, T_complete_03_sequence);

  return offset;
}


static const per_sequence_t T_defaultConfig_03_sequence[] = {
  { &hf_rrc_defaultConfigMode, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DefaultConfigMode },
  { &hf_rrc_defaultConfigIdentity_03, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DefaultConfigIdentity_r6 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_defaultConfig_03(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_defaultConfig_03, T_defaultConfig_03_sequence);

  return offset;
}


static const value_string rrc_T_preConfigMode_03_vals[] = {
  {   0, "predefinedConfigIdentity" },
  {   1, "defaultConfig" },
  { 0, NULL }
};

static const per_choice_t T_preConfigMode_03_choice[] = {
  {   0, &hf_rrc_predefinedConfigIdentity, ASN1_NO_EXTENSIONS     , dissect_rrc_PredefinedConfigIdentity },
  {   1, &hf_rrc_defaultConfig_03, ASN1_NO_EXTENSIONS     , dissect_rrc_T_defaultConfig_03 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_preConfigMode_03(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_preConfigMode_03, T_preConfigMode_03_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t T_fdd_12_sequence[] = {
  { &hf_rrc_ul_DPCH_Info_04 , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UL_DPCH_InfoPostFDD },
  { &hf_rrc_dl_CommonInformationPost, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DL_CommonInformationPost },
  { &hf_rrc_dl_InformationPerRL_List_07, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DL_InformationPerRL_ListPostFDD },
  { &hf_rrc_frequencyInfo_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_FrequencyInfoFDD },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_fdd_12(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_fdd_12, T_fdd_12_sequence);

  return offset;
}


static const per_sequence_t T_tdd384_02_sequence[] = {
  { &hf_rrc_ul_DPCH_Info_05 , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UL_DPCH_InfoPostTDD },
  { &hf_rrc_dl_InformationPerRL, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DL_InformationPerRL_PostTDD },
  { &hf_rrc_frequencyInfo_02, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_FrequencyInfoTDD },
  { &hf_rrc_primaryCCPCH_TX_Power, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PrimaryCCPCH_TX_Power },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_tdd384_02(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_tdd384_02, T_tdd384_02_sequence);

  return offset;
}


static const per_sequence_t T_tdd128_02_sequence[] = {
  { &hf_rrc_ul_DPCH_Info_07 , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UL_DPCH_InfoPostTDD_LCR_r4 },
  { &hf_rrc_dl_InformationPerRL_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DL_InformationPerRL_PostTDD_LCR_r4 },
  { &hf_rrc_frequencyInfo_02, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_FrequencyInfoTDD },
  { &hf_rrc_primaryCCPCH_TX_Power, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PrimaryCCPCH_TX_Power },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_tdd128_02(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_tdd128_02, T_tdd128_02_sequence);

  return offset;
}


static const value_string rrc_T_tdd_03_vals[] = {
  {   0, "tdd384" },
  {   1, "tdd128" },
  { 0, NULL }
};

static const per_choice_t T_tdd_03_choice[] = {
  {   0, &hf_rrc_tdd384_02       , ASN1_NO_EXTENSIONS     , dissect_rrc_T_tdd384_02 },
  {   1, &hf_rrc_tdd128_02       , ASN1_NO_EXTENSIONS     , dissect_rrc_T_tdd128_02 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_tdd_03(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_tdd_03, T_tdd_03_choice,
                                 NULL);

  return offset;
}


static const value_string rrc_T_modeSpecificInfo_06_vals[] = {
  {   0, "fdd" },
  {   1, "tdd" },
  { 0, NULL }
};

static const per_choice_t T_modeSpecificInfo_06_choice[] = {
  {   0, &hf_rrc_fdd_12          , ASN1_NO_EXTENSIONS     , dissect_rrc_T_fdd_12 },
  {   1, &hf_rrc_tdd_04          , ASN1_NO_EXTENSIONS     , dissect_rrc_T_tdd_03 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_modeSpecificInfo_06(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_modeSpecificInfo_06, T_modeSpecificInfo_06_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t T_preconfiguration_03_sequence[] = {
  { &hf_rrc_preConfigMode_03, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_preConfigMode_03 },
  { &hf_rrc_rab_Info        , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RAB_Info_Post },
  { &hf_rrc_modeSpecificInfo_06, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_modeSpecificInfo_06 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_preconfiguration_03(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_preconfiguration_03, T_preconfiguration_03_sequence);

  return offset;
}


static const value_string rrc_T_specificationMode_03_vals[] = {
  {   0, "complete" },
  {   1, "preconfiguration" },
  { 0, NULL }
};

static const per_choice_t T_specificationMode_03_choice[] = {
  {   0, &hf_rrc_complete_03     , ASN1_NO_EXTENSIONS     , dissect_rrc_T_complete_03 },
  {   1, &hf_rrc_preconfiguration_03, ASN1_NO_EXTENSIONS     , dissect_rrc_T_preconfiguration_03 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_specificationMode_03(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_specificationMode_03, T_specificationMode_03_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t HandoverToUTRANCommand_r6_IEs_sequence[] = {
  { &hf_rrc_new_U_RNTI_01   , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_U_RNTI_Short },
  { &hf_rrc_cipheringAlgorithm, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CipheringAlgorithm },
  { &hf_rrc_specificationMode_03, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_specificationMode_03 },
  { &hf_rrc_maxAllowedUL_TX_Power, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MaxAllowedUL_TX_Power },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_HandoverToUTRANCommand_r6_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_HandoverToUTRANCommand_r6_IEs, HandoverToUTRANCommand_r6_IEs_sequence);

  return offset;
}


static const per_sequence_t UE_HSPA_Identities_r6_sequence[] = {
  { &hf_rrc_new_H_RNTI      , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_H_RNTI },
  { &hf_rrc_newPrimary_E_RNTI, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_E_RNTI },
  { &hf_rrc_newSecondary_E_RNTI, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_E_RNTI },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UE_HSPA_Identities_r6(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UE_HSPA_Identities_r6, UE_HSPA_Identities_r6_sequence);

  return offset;
}


static const per_sequence_t HandoverToUTRANCommand_v6b0ext_IEs_sequence[] = {
  { &hf_rrc_ue_hspa_identities, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_HSPA_Identities_r6 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_HandoverToUTRANCommand_v6b0ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_HandoverToUTRANCommand_v6b0ext_IEs, HandoverToUTRANCommand_v6b0ext_IEs_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions_40_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_nonCriticalExtensions_40(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_nonCriticalExtensions_40, T_nonCriticalExtensions_40_sequence);

  return offset;
}


static const per_sequence_t T_v6b0NonCriticalExtensions_04_sequence[] = {
  { &hf_rrc_handoverToUTRANCommand_v6b0ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_HandoverToUTRANCommand_v6b0ext_IEs },
  { &hf_rrc_nonCriticalExtensions_40, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_nonCriticalExtensions_40 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v6b0NonCriticalExtensions_04(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v6b0NonCriticalExtensions_04, T_v6b0NonCriticalExtensions_04_sequence);

  return offset;
}


static const per_sequence_t T_r6_03_sequence[] = {
  { &hf_rrc_handoverToUTRANCommand_r6, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_HandoverToUTRANCommand_r6_IEs },
  { &hf_rrc_v6b0NonCriticalExtensions_04, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v6b0NonCriticalExtensions_04 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_r6_03(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_r6_03, T_r6_03_sequence);

  return offset;
}


static const per_sequence_t T_complete_04_sequence[] = {
  { &hf_rrc_srb_InformationSetupList_03, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SRB_InformationSetupList_r7 },
  { &hf_rrc_rab_InformationSetupList_04, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RAB_InformationSetupList_r7 },
  { &hf_rrc_ul_CommonTransChInfo_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UL_CommonTransChInfo_r4 },
  { &hf_rrc_ul_AddReconfTransChInfoList_02, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UL_AddReconfTransChInfoList_r7 },
  { &hf_rrc_dl_CommonTransChInfo_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DL_CommonTransChInfo_r4 },
  { &hf_rrc_dl_AddReconfTransChInfoList_03, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DL_AddReconfTransChInfoList_r7 },
  { &hf_rrc_ul_DPCH_Info_01 , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UL_DPCH_Info_r7 },
  { &hf_rrc_ul_EDCH_Information_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_EDCH_Information_r7 },
  { &hf_rrc_dl_HSPDSCH_Information_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_HSPDSCH_Information_r7 },
  { &hf_rrc_dl_CommonInformation_04, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DL_CommonInformation_r7 },
  { &hf_rrc_dl_InformationPerRL_List_04, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DL_InformationPerRL_List_r7 },
  { &hf_rrc_frequencyInfo   , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_FrequencyInfo },
  { &hf_rrc_multi_frequencyInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_Multi_frequencyInfo_LCR_r7 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_complete_04(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_complete_04, T_complete_04_sequence);

  return offset;
}


static const per_sequence_t T_defaultConfig_04_sequence[] = {
  { &hf_rrc_defaultConfigMode, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DefaultConfigMode },
  { &hf_rrc_defaultConfigIdentity_03, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DefaultConfigIdentity_r6 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_defaultConfig_04(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_defaultConfig_04, T_defaultConfig_04_sequence);

  return offset;
}


static const value_string rrc_T_preConfigMode_04_vals[] = {
  {   0, "predefinedConfigIdentity" },
  {   1, "defaultConfig" },
  { 0, NULL }
};

static const per_choice_t T_preConfigMode_04_choice[] = {
  {   0, &hf_rrc_predefinedConfigIdentity, ASN1_NO_EXTENSIONS     , dissect_rrc_PredefinedConfigIdentity },
  {   1, &hf_rrc_defaultConfig_04, ASN1_NO_EXTENSIONS     , dissect_rrc_T_defaultConfig_04 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_preConfigMode_04(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_preConfigMode_04, T_preConfigMode_04_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t T_fdd_13_sequence[] = {
  { &hf_rrc_ul_DPCH_Info_04 , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UL_DPCH_InfoPostFDD },
  { &hf_rrc_dl_CommonInformationPost, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DL_CommonInformationPost },
  { &hf_rrc_dl_InformationPerRL_List_07, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DL_InformationPerRL_ListPostFDD },
  { &hf_rrc_frequencyInfo_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_FrequencyInfoFDD },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_fdd_13(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_fdd_13, T_fdd_13_sequence);

  return offset;
}


static const per_sequence_t T_tdd384_03_sequence[] = {
  { &hf_rrc_ul_DPCH_Info_05 , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UL_DPCH_InfoPostTDD },
  { &hf_rrc_dl_InformationPerRL, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DL_InformationPerRL_PostTDD },
  { &hf_rrc_frequencyInfo_02, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_FrequencyInfoTDD },
  { &hf_rrc_primaryCCPCH_TX_Power, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PrimaryCCPCH_TX_Power },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_tdd384_03(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_tdd384_03, T_tdd384_03_sequence);

  return offset;
}


static const per_sequence_t T_tdd128_03_sequence[] = {
  { &hf_rrc_ul_DPCH_Info_01 , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UL_DPCH_Info_r7 },
  { &hf_rrc_dl_InformationPerRL_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DL_InformationPerRL_PostTDD_LCR_r4 },
  { &hf_rrc_frequencyInfo_02, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_FrequencyInfoTDD },
  { &hf_rrc_primaryCCPCH_TX_Power, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PrimaryCCPCH_TX_Power },
  { &hf_rrc_multi_frequencyInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_Multi_frequencyInfo_LCR_r7 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_tdd128_03(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_tdd128_03, T_tdd128_03_sequence);

  return offset;
}


static const per_sequence_t T_tdd768_sequence[] = {
  { &hf_rrc_ul_DPCH_Info_01 , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UL_DPCH_Info_r7 },
  { &hf_rrc_dl_InformationPerRL_02, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DL_InformationPerRL_List_r7 },
  { &hf_rrc_frequencyInfo_02, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_FrequencyInfoTDD },
  { &hf_rrc_primaryCCPCH_TX_Power, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PrimaryCCPCH_TX_Power },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_tdd768(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_tdd768, T_tdd768_sequence);

  return offset;
}


static const value_string rrc_T_tdd_04_vals[] = {
  {   0, "tdd384" },
  {   1, "tdd128" },
  {   2, "tdd768" },
  { 0, NULL }
};

static const per_choice_t T_tdd_04_choice[] = {
  {   0, &hf_rrc_tdd384_03       , ASN1_NO_EXTENSIONS     , dissect_rrc_T_tdd384_03 },
  {   1, &hf_rrc_tdd128_03       , ASN1_NO_EXTENSIONS     , dissect_rrc_T_tdd128_03 },
  {   2, &hf_rrc_tdd768          , ASN1_NO_EXTENSIONS     , dissect_rrc_T_tdd768 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_tdd_04(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_tdd_04, T_tdd_04_choice,
                                 NULL);

  return offset;
}


static const value_string rrc_T_modeSpecificInfo_07_vals[] = {
  {   0, "fdd" },
  {   1, "tdd" },
  { 0, NULL }
};

static const per_choice_t T_modeSpecificInfo_07_choice[] = {
  {   0, &hf_rrc_fdd_13          , ASN1_NO_EXTENSIONS     , dissect_rrc_T_fdd_13 },
  {   1, &hf_rrc_tdd_05          , ASN1_NO_EXTENSIONS     , dissect_rrc_T_tdd_04 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_modeSpecificInfo_07(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_modeSpecificInfo_07, T_modeSpecificInfo_07_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t T_preconfiguration_04_sequence[] = {
  { &hf_rrc_preConfigMode_04, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_preConfigMode_04 },
  { &hf_rrc_rab_Info        , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RAB_Info_Post },
  { &hf_rrc_modeSpecificInfo_07, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_modeSpecificInfo_07 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_preconfiguration_04(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_preconfiguration_04, T_preconfiguration_04_sequence);

  return offset;
}


static const value_string rrc_T_specificationMode_04_vals[] = {
  {   0, "complete" },
  {   1, "preconfiguration" },
  { 0, NULL }
};

static const per_choice_t T_specificationMode_04_choice[] = {
  {   0, &hf_rrc_complete_04     , ASN1_NO_EXTENSIONS     , dissect_rrc_T_complete_04 },
  {   1, &hf_rrc_preconfiguration_04, ASN1_NO_EXTENSIONS     , dissect_rrc_T_preconfiguration_04 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_specificationMode_04(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_specificationMode_04, T_specificationMode_04_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t HandoverToUTRANCommand_r7_IEs_sequence[] = {
  { &hf_rrc_new_U_RNTI_01   , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_U_RNTI_Short },
  { &hf_rrc_cipheringAlgorithm_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CipheringAlgorithm_r7 },
  { &hf_rrc_supportForChangeOfUE_Capability, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { &hf_rrc_new_H_RNTI      , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_H_RNTI },
  { &hf_rrc_newPrimary_E_RNTI, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_E_RNTI },
  { &hf_rrc_newSecondary_E_RNTI, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_E_RNTI },
  { &hf_rrc_specificationMode_04, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_specificationMode_04 },
  { &hf_rrc_maxAllowedUL_TX_Power, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MaxAllowedUL_TX_Power },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_HandoverToUTRANCommand_r7_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_HandoverToUTRANCommand_r7_IEs, HandoverToUTRANCommand_r7_IEs_sequence);

  return offset;
}


static const per_sequence_t HandoverToUTRANCommand_v780ext_IEs_sequence[] = {
  { &hf_rrc_hs_DSCH_TBSizeTable, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_HS_DSCH_TBSizeTable },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_HandoverToUTRANCommand_v780ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_HandoverToUTRANCommand_v780ext_IEs, HandoverToUTRANCommand_v780ext_IEs_sequence);

  return offset;
}


static const per_sequence_t HandoverToUTRANCommand_v820ext_IEs_sequence[] = {
  { &hf_rrc_rab_InformationSetupList_06, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RAB_InformationSetupList_v820ext },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_HandoverToUTRANCommand_v820ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_HandoverToUTRANCommand_v820ext_IEs, HandoverToUTRANCommand_v820ext_IEs_sequence);

  return offset;
}


static const per_sequence_t HandoverToUTRANCommand_v7d0ext_IEs_sequence[] = {
  { &hf_rrc_ul_EDCH_Information_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_EDCH_Information_ext },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_HandoverToUTRANCommand_v7d0ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_HandoverToUTRANCommand_v7d0ext_IEs, HandoverToUTRANCommand_v7d0ext_IEs_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions_41_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_nonCriticalExtensions_41(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_nonCriticalExtensions_41, T_nonCriticalExtensions_41_sequence);

  return offset;
}


static const per_sequence_t T_v7d0NonCriticalExtensions_02_sequence[] = {
  { &hf_rrc_handoverToUTRANCommand_v7d0ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_HandoverToUTRANCommand_v7d0ext_IEs },
  { &hf_rrc_nonCriticalExtensions_41, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_nonCriticalExtensions_41 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v7d0NonCriticalExtensions_02(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v7d0NonCriticalExtensions_02, T_v7d0NonCriticalExtensions_02_sequence);

  return offset;
}


static const per_sequence_t T_v820NonCriticalExtensions_sequence[] = {
  { &hf_rrc_handoverToUTRANCommand_v820ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_HandoverToUTRANCommand_v820ext_IEs },
  { &hf_rrc_v7d0NonCriticalExtensions_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v7d0NonCriticalExtensions_02 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v820NonCriticalExtensions(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v820NonCriticalExtensions, T_v820NonCriticalExtensions_sequence);

  return offset;
}


static const per_sequence_t T_v780NonCriticalExtensions_03_sequence[] = {
  { &hf_rrc_handoverToUTRANCommand_v780ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_HandoverToUTRANCommand_v780ext_IEs },
  { &hf_rrc_v820NonCriticalExtensions, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v820NonCriticalExtensions },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v780NonCriticalExtensions_03(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v780NonCriticalExtensions_03, T_v780NonCriticalExtensions_03_sequence);

  return offset;
}


static const per_sequence_t T_r7_03_sequence[] = {
  { &hf_rrc_handoverToUTRANCommand_r7, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_HandoverToUTRANCommand_r7_IEs },
  { &hf_rrc_v780NonCriticalExtensions_03, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v780NonCriticalExtensions_03 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_r7_03(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_r7_03, T_r7_03_sequence);

  return offset;
}


static const per_sequence_t T_complete_05_sequence[] = {
  { &hf_rrc_srb_InformationSetupList_04, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SRB_InformationSetupList_r8 },
  { &hf_rrc_rab_InformationSetupList_05, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RAB_InformationSetupList_r8 },
  { &hf_rrc_ul_CommonTransChInfo_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UL_CommonTransChInfo_r4 },
  { &hf_rrc_ul_AddReconfTransChInfoList_03, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UL_AddReconfTransChInfoList_r8 },
  { &hf_rrc_dl_CommonTransChInfo_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DL_CommonTransChInfo_r4 },
  { &hf_rrc_dl_AddReconfTransChInfoList_03, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DL_AddReconfTransChInfoList_r7 },
  { &hf_rrc_ul_DPCH_Info_01 , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UL_DPCH_Info_r7 },
  { &hf_rrc_ul_EDCH_Information_03, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_EDCH_Information_r8 },
  { &hf_rrc_dl_HSPDSCH_Information_03, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_HSPDSCH_Information_r8 },
  { &hf_rrc_dl_CommonInformation_05, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DL_CommonInformation_r8 },
  { &hf_rrc_dl_InformationPerRL_List_04, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DL_InformationPerRL_List_r7 },
  { &hf_rrc_frequencyInfo   , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_FrequencyInfo },
  { &hf_rrc_multi_frequencyInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_Multi_frequencyInfo_LCR_r7 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_complete_05(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_complete_05, T_complete_05_sequence);

  return offset;
}


static const per_sequence_t DLUL_HSPA_Information_r8_sequence[] = {
  { &hf_rrc_ul_EDCH_Information_03, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UL_EDCH_Information_r8 },
  { &hf_rrc_dl_CommonInformation_05, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DL_CommonInformation_r8 },
  { &hf_rrc_dl_InformationPerRL_List_04, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DL_InformationPerRL_List_r7 },
  { &hf_rrc_dl_HSPDSCH_Information_03, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DL_HSPDSCH_Information_r8 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_DLUL_HSPA_Information_r8(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_DLUL_HSPA_Information_r8, DLUL_HSPA_Information_r8_sequence);

  return offset;
}


static const per_sequence_t T_defaultConfig_05_sequence[] = {
  { &hf_rrc_defaultConfigMode, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DefaultConfigMode },
  { &hf_rrc_defaultConfigIdentity_03, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DefaultConfigIdentity_r6 },
  { &hf_rrc_dummy_09        , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DLUL_HSPA_Information_r8 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_defaultConfig_05(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_defaultConfig_05, T_defaultConfig_05_sequence);

  return offset;
}


static const value_string rrc_T_preConfigMode_05_vals[] = {
  {   0, "predefinedConfigIdentity" },
  {   1, "defaultConfig" },
  { 0, NULL }
};

static const per_choice_t T_preConfigMode_05_choice[] = {
  {   0, &hf_rrc_predefinedConfigIdentity, ASN1_NO_EXTENSIONS     , dissect_rrc_PredefinedConfigIdentity },
  {   1, &hf_rrc_defaultConfig_05, ASN1_NO_EXTENSIONS     , dissect_rrc_T_defaultConfig_05 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_preConfigMode_05(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_preConfigMode_05, T_preConfigMode_05_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t T_fdd_14_sequence[] = {
  { &hf_rrc_ul_DPCH_Info_04 , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UL_DPCH_InfoPostFDD },
  { &hf_rrc_dl_CommonInformationPost, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DL_CommonInformationPost },
  { &hf_rrc_dl_InformationPerRL_List_07, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DL_InformationPerRL_ListPostFDD },
  { &hf_rrc_frequencyInfo_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_FrequencyInfoFDD },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_fdd_14(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_fdd_14, T_fdd_14_sequence);

  return offset;
}


static const per_sequence_t T_tdd384_04_sequence[] = {
  { &hf_rrc_ul_DPCH_Info_05 , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UL_DPCH_InfoPostTDD },
  { &hf_rrc_dl_InformationPerRL, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DL_InformationPerRL_PostTDD },
  { &hf_rrc_frequencyInfo_02, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_FrequencyInfoTDD },
  { &hf_rrc_primaryCCPCH_TX_Power, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PrimaryCCPCH_TX_Power },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_tdd384_04(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_tdd384_04, T_tdd384_04_sequence);

  return offset;
}


static const per_sequence_t T_tdd128_04_sequence[] = {
  { &hf_rrc_ul_DPCH_Info_01 , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UL_DPCH_Info_r7 },
  { &hf_rrc_dl_InformationPerRL_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DL_InformationPerRL_PostTDD_LCR_r4 },
  { &hf_rrc_frequencyInfo_02, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_FrequencyInfoTDD },
  { &hf_rrc_primaryCCPCH_TX_Power, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PrimaryCCPCH_TX_Power },
  { &hf_rrc_multi_frequencyInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_Multi_frequencyInfo_LCR_r7 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_tdd128_04(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_tdd128_04, T_tdd128_04_sequence);

  return offset;
}


static const per_sequence_t T_tdd768_01_sequence[] = {
  { &hf_rrc_ul_DPCH_Info_01 , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UL_DPCH_Info_r7 },
  { &hf_rrc_dl_InformationPerRL_02, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DL_InformationPerRL_List_r7 },
  { &hf_rrc_frequencyInfo_02, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_FrequencyInfoTDD },
  { &hf_rrc_primaryCCPCH_TX_Power, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PrimaryCCPCH_TX_Power },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_tdd768_01(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_tdd768_01, T_tdd768_01_sequence);

  return offset;
}


static const value_string rrc_T_tdd_05_vals[] = {
  {   0, "tdd384" },
  {   1, "tdd128" },
  {   2, "tdd768" },
  { 0, NULL }
};

static const per_choice_t T_tdd_05_choice[] = {
  {   0, &hf_rrc_tdd384_04       , ASN1_NO_EXTENSIONS     , dissect_rrc_T_tdd384_04 },
  {   1, &hf_rrc_tdd128_04       , ASN1_NO_EXTENSIONS     , dissect_rrc_T_tdd128_04 },
  {   2, &hf_rrc_tdd768_01       , ASN1_NO_EXTENSIONS     , dissect_rrc_T_tdd768_01 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_tdd_05(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_tdd_05, T_tdd_05_choice,
                                 NULL);

  return offset;
}


static const value_string rrc_T_modeSpecificInfo_08_vals[] = {
  {   0, "fdd" },
  {   1, "tdd" },
  { 0, NULL }
};

static const per_choice_t T_modeSpecificInfo_08_choice[] = {
  {   0, &hf_rrc_fdd_14          , ASN1_NO_EXTENSIONS     , dissect_rrc_T_fdd_14 },
  {   1, &hf_rrc_tdd_06          , ASN1_NO_EXTENSIONS     , dissect_rrc_T_tdd_05 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_modeSpecificInfo_08(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_modeSpecificInfo_08, T_modeSpecificInfo_08_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t T_preconfiguration_05_sequence[] = {
  { &hf_rrc_preConfigMode_05, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_preConfigMode_05 },
  { &hf_rrc_rab_Info        , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RAB_Info_Post },
  { &hf_rrc_modeSpecificInfo_08, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_modeSpecificInfo_08 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_preconfiguration_05(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_preconfiguration_05, T_preconfiguration_05_sequence);

  return offset;
}


static const value_string rrc_T_specificationMode_05_vals[] = {
  {   0, "complete" },
  {   1, "preconfiguration" },
  { 0, NULL }
};

static const per_choice_t T_specificationMode_05_choice[] = {
  {   0, &hf_rrc_complete_05     , ASN1_NO_EXTENSIONS     , dissect_rrc_T_complete_05 },
  {   1, &hf_rrc_preconfiguration_05, ASN1_NO_EXTENSIONS     , dissect_rrc_T_preconfiguration_05 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_specificationMode_05(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_specificationMode_05, T_specificationMode_05_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t HandoverToUTRANCommand_r8_IEs_sequence[] = {
  { &hf_rrc_new_U_RNTI_01   , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_U_RNTI_Short },
  { &hf_rrc_cipheringAlgorithm_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CipheringAlgorithm_r7 },
  { &hf_rrc_supportForChangeOfUE_Capability, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { &hf_rrc_new_H_RNTI      , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_H_RNTI },
  { &hf_rrc_newPrimary_E_RNTI, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_E_RNTI },
  { &hf_rrc_newSecondary_E_RNTI, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_E_RNTI },
  { &hf_rrc_dummy_07        , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DefaultConfigForCellFACH },
  { &hf_rrc_specificationMode_05, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_specificationMode_05 },
  { &hf_rrc_maxAllowedUL_TX_Power, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MaxAllowedUL_TX_Power },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_HandoverToUTRANCommand_r8_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_HandoverToUTRANCommand_r8_IEs, HandoverToUTRANCommand_r8_IEs_sequence);

  return offset;
}


static const per_sequence_t HandoverToUTRANCommand_v890ext_IEs_sequence[] = {
  { &hf_rrc_dl_HSPDSCH_Information_04, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_HSPDSCH_Information_r8_ext },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_HandoverToUTRANCommand_v890ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_HandoverToUTRANCommand_v890ext_IEs, HandoverToUTRANCommand_v890ext_IEs_sequence);

  return offset;
}


static const per_sequence_t HandoverToUTRANCommand_v8a0ext_IEs_sequence[] = {
  { &hf_rrc_dl_HSPDSCH_Information_05, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_HSPDSCH_Information_r8_ext2 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_HandoverToUTRANCommand_v8a0ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_HandoverToUTRANCommand_v8a0ext_IEs, HandoverToUTRANCommand_v8a0ext_IEs_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions_42_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_nonCriticalExtensions_42(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_nonCriticalExtensions_42, T_nonCriticalExtensions_42_sequence);

  return offset;
}


static const per_sequence_t T_v8a0NonCriticalExtensions_02_sequence[] = {
  { &hf_rrc_handoverToUTRANCommand_v8a0ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_HandoverToUTRANCommand_v8a0ext_IEs },
  { &hf_rrc_nonCriticalExtensions_42, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_nonCriticalExtensions_42 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v8a0NonCriticalExtensions_02(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v8a0NonCriticalExtensions_02, T_v8a0NonCriticalExtensions_02_sequence);

  return offset;
}


static const per_sequence_t T_v890NonCriticalExtensions_03_sequence[] = {
  { &hf_rrc_handoverToUTRANCommand_v890ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_HandoverToUTRANCommand_v890ext_IEs },
  { &hf_rrc_v8a0NonCriticalExtensions_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v8a0NonCriticalExtensions_02 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v890NonCriticalExtensions_03(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v890NonCriticalExtensions_03, T_v890NonCriticalExtensions_03_sequence);

  return offset;
}


static const per_sequence_t T_v7d0NonCriticalExtensions_03_sequence[] = {
  { &hf_rrc_handoverToUTRANCommand_v7d0ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_HandoverToUTRANCommand_v7d0ext_IEs },
  { &hf_rrc_v890NonCriticalExtensions_03, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v890NonCriticalExtensions_03 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v7d0NonCriticalExtensions_03(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v7d0NonCriticalExtensions_03, T_v7d0NonCriticalExtensions_03_sequence);

  return offset;
}


static const per_sequence_t T_r8_03_sequence[] = {
  { &hf_rrc_handoverToUTRANCommand_r8, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_HandoverToUTRANCommand_r8_IEs },
  { &hf_rrc_v7d0NonCriticalExtensions_03, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v7d0NonCriticalExtensions_03 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_r8_03(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_r8_03, T_r8_03_sequence);

  return offset;
}


static const per_sequence_t T_complete_06_sequence[] = {
  { &hf_rrc_srb_InformationSetupList_04, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SRB_InformationSetupList_r8 },
  { &hf_rrc_rab_InformationSetupList_05, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RAB_InformationSetupList_r8 },
  { &hf_rrc_ul_CommonTransChInfo_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UL_CommonTransChInfo_r4 },
  { &hf_rrc_ul_AddReconfTransChInfoList_03, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UL_AddReconfTransChInfoList_r8 },
  { &hf_rrc_dl_CommonTransChInfo_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DL_CommonTransChInfo_r4 },
  { &hf_rrc_dl_AddReconfTransChInfoList_04, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DL_AddReconfTransChInfoList_r9 },
  { &hf_rrc_ul_DPCH_Info_01 , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UL_DPCH_Info_r7 },
  { &hf_rrc_ul_EDCH_Information_04, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_EDCH_Information_r9 },
  { &hf_rrc_dl_HSPDSCH_Information_06, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_HSPDSCH_Information_r9 },
  { &hf_rrc_dl_CommonInformation_05, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DL_CommonInformation_r8 },
  { &hf_rrc_dl_InformationPerRL_List_04, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DL_InformationPerRL_List_r7 },
  { &hf_rrc_frequencyInfo   , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_FrequencyInfo },
  { &hf_rrc_multi_frequencyInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_Multi_frequencyInfo_LCR_r7 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_complete_06(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_complete_06, T_complete_06_sequence);

  return offset;
}


static const per_sequence_t DLUL_HSPA_Information_r9_sequence[] = {
  { &hf_rrc_ul_EDCH_Information_04, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UL_EDCH_Information_r9 },
  { &hf_rrc_dl_CommonInformation_05, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DL_CommonInformation_r8 },
  { &hf_rrc_dl_InformationPerRL_List_04, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DL_InformationPerRL_List_r7 },
  { &hf_rrc_dl_HSPDSCH_Information_06, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DL_HSPDSCH_Information_r9 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_DLUL_HSPA_Information_r9(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_DLUL_HSPA_Information_r9, DLUL_HSPA_Information_r9_sequence);

  return offset;
}


static const per_sequence_t T_defaultConfig_06_sequence[] = {
  { &hf_rrc_defaultConfigMode, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DefaultConfigMode },
  { &hf_rrc_defaultConfigIdentity_03, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DefaultConfigIdentity_r6 },
  { &hf_rrc_dummy_10        , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DLUL_HSPA_Information_r9 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_defaultConfig_06(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_defaultConfig_06, T_defaultConfig_06_sequence);

  return offset;
}


static const value_string rrc_T_preConfigMode_06_vals[] = {
  {   0, "predefinedConfigIdentity" },
  {   1, "defaultConfig" },
  { 0, NULL }
};

static const per_choice_t T_preConfigMode_06_choice[] = {
  {   0, &hf_rrc_predefinedConfigIdentity, ASN1_NO_EXTENSIONS     , dissect_rrc_PredefinedConfigIdentity },
  {   1, &hf_rrc_defaultConfig_06, ASN1_NO_EXTENSIONS     , dissect_rrc_T_defaultConfig_06 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_preConfigMode_06(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_preConfigMode_06, T_preConfigMode_06_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t T_fdd_15_sequence[] = {
  { &hf_rrc_ul_DPCH_Info_04 , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UL_DPCH_InfoPostFDD },
  { &hf_rrc_dl_CommonInformationPost, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DL_CommonInformationPost },
  { &hf_rrc_dl_InformationPerRL_List_07, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DL_InformationPerRL_ListPostFDD },
  { &hf_rrc_frequencyInfo_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_FrequencyInfoFDD },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_fdd_15(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_fdd_15, T_fdd_15_sequence);

  return offset;
}


static const per_sequence_t T_tdd384_05_sequence[] = {
  { &hf_rrc_ul_DPCH_Info_05 , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UL_DPCH_InfoPostTDD },
  { &hf_rrc_dl_InformationPerRL, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DL_InformationPerRL_PostTDD },
  { &hf_rrc_frequencyInfo_02, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_FrequencyInfoTDD },
  { &hf_rrc_primaryCCPCH_TX_Power, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PrimaryCCPCH_TX_Power },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_tdd384_05(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_tdd384_05, T_tdd384_05_sequence);

  return offset;
}


static const per_sequence_t T_tdd128_05_sequence[] = {
  { &hf_rrc_ul_DPCH_Info_01 , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UL_DPCH_Info_r7 },
  { &hf_rrc_dl_InformationPerRL_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DL_InformationPerRL_PostTDD_LCR_r4 },
  { &hf_rrc_frequencyInfo_02, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_FrequencyInfoTDD },
  { &hf_rrc_primaryCCPCH_TX_Power, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PrimaryCCPCH_TX_Power },
  { &hf_rrc_multi_frequencyInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_Multi_frequencyInfo_LCR_r7 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_tdd128_05(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_tdd128_05, T_tdd128_05_sequence);

  return offset;
}


static const per_sequence_t T_tdd768_02_sequence[] = {
  { &hf_rrc_ul_DPCH_Info_01 , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UL_DPCH_Info_r7 },
  { &hf_rrc_dl_InformationPerRL_02, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DL_InformationPerRL_List_r7 },
  { &hf_rrc_frequencyInfo_02, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_FrequencyInfoTDD },
  { &hf_rrc_primaryCCPCH_TX_Power, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PrimaryCCPCH_TX_Power },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_tdd768_02(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_tdd768_02, T_tdd768_02_sequence);

  return offset;
}


static const value_string rrc_T_tdd_06_vals[] = {
  {   0, "tdd384" },
  {   1, "tdd128" },
  {   2, "tdd768" },
  { 0, NULL }
};

static const per_choice_t T_tdd_06_choice[] = {
  {   0, &hf_rrc_tdd384_05       , ASN1_NO_EXTENSIONS     , dissect_rrc_T_tdd384_05 },
  {   1, &hf_rrc_tdd128_05       , ASN1_NO_EXTENSIONS     , dissect_rrc_T_tdd128_05 },
  {   2, &hf_rrc_tdd768_02       , ASN1_NO_EXTENSIONS     , dissect_rrc_T_tdd768_02 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_tdd_06(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_tdd_06, T_tdd_06_choice,
                                 NULL);

  return offset;
}


static const value_string rrc_T_modeSpecificInfo_09_vals[] = {
  {   0, "fdd" },
  {   1, "tdd" },
  { 0, NULL }
};

static const per_choice_t T_modeSpecificInfo_09_choice[] = {
  {   0, &hf_rrc_fdd_15          , ASN1_NO_EXTENSIONS     , dissect_rrc_T_fdd_15 },
  {   1, &hf_rrc_tdd_07          , ASN1_NO_EXTENSIONS     , dissect_rrc_T_tdd_06 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_modeSpecificInfo_09(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_modeSpecificInfo_09, T_modeSpecificInfo_09_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t T_preconfiguration_06_sequence[] = {
  { &hf_rrc_preConfigMode_06, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_preConfigMode_06 },
  { &hf_rrc_rab_Info        , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RAB_Info_Post },
  { &hf_rrc_modeSpecificInfo_09, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_modeSpecificInfo_09 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_preconfiguration_06(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_preconfiguration_06, T_preconfiguration_06_sequence);

  return offset;
}


static const value_string rrc_T_specificationMode_06_vals[] = {
  {   0, "complete" },
  {   1, "preconfiguration" },
  { 0, NULL }
};

static const per_choice_t T_specificationMode_06_choice[] = {
  {   0, &hf_rrc_complete_06     , ASN1_NO_EXTENSIONS     , dissect_rrc_T_complete_06 },
  {   1, &hf_rrc_preconfiguration_06, ASN1_NO_EXTENSIONS     , dissect_rrc_T_preconfiguration_06 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_specificationMode_06(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_specificationMode_06, T_specificationMode_06_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t HandoverToUTRANCommand_r9_IEs_sequence[] = {
  { &hf_rrc_new_U_RNTI_01   , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_U_RNTI_Short },
  { &hf_rrc_cipheringAlgorithm_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CipheringAlgorithm_r7 },
  { &hf_rrc_supportForChangeOfUE_Capability, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { &hf_rrc_new_H_RNTI      , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_H_RNTI },
  { &hf_rrc_newPrimary_E_RNTI, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_E_RNTI },
  { &hf_rrc_newSecondary_E_RNTI, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_E_RNTI },
  { &hf_rrc_defaultConfigForCellFACH, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DefaultConfigForCellFACH },
  { &hf_rrc_specificationMode_06, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_specificationMode_06 },
  { &hf_rrc_maxAllowedUL_TX_Power, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MaxAllowedUL_TX_Power },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_HandoverToUTRANCommand_r9_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_HandoverToUTRANCommand_r9_IEs, HandoverToUTRANCommand_r9_IEs_sequence);

  return offset;
}


static const per_sequence_t HandoverToUTRANCommand_v9c0ext_IEs_sequence[] = {
  { &hf_rrc_dl_AddReconfTransChInfoList_05, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_AddReconfTransChInfoList_TDD128_v9c0ext },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_HandoverToUTRANCommand_v9c0ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_HandoverToUTRANCommand_v9c0ext_IEs, HandoverToUTRANCommand_v9c0ext_IEs_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions_43_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_nonCriticalExtensions_43(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_nonCriticalExtensions_43, T_nonCriticalExtensions_43_sequence);

  return offset;
}


static const per_sequence_t T_v9c0NonCriticalExtensions_02_sequence[] = {
  { &hf_rrc_handoverToUTRANCommand_v9c0ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_HandoverToUTRANCommand_v9c0ext_IEs },
  { &hf_rrc_nonCriticalExtensions_43, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_nonCriticalExtensions_43 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v9c0NonCriticalExtensions_02(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v9c0NonCriticalExtensions_02, T_v9c0NonCriticalExtensions_02_sequence);

  return offset;
}


static const per_sequence_t T_r9_03_sequence[] = {
  { &hf_rrc_handoverToUTRANCommand_r9, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_HandoverToUTRANCommand_r9_IEs },
  { &hf_rrc_v9c0NonCriticalExtensions_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v9c0NonCriticalExtensions_02 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_r9_03(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_r9_03, T_r9_03_sequence);

  return offset;
}


static const per_sequence_t T_complete_07_sequence[] = {
  { &hf_rrc_srb_InformationSetupList_04, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SRB_InformationSetupList_r8 },
  { &hf_rrc_rab_InformationSetupList_05, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RAB_InformationSetupList_r8 },
  { &hf_rrc_ul_CommonTransChInfo_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UL_CommonTransChInfo_r4 },
  { &hf_rrc_ul_AddReconfTransChInfoList_03, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UL_AddReconfTransChInfoList_r8 },
  { &hf_rrc_dl_CommonTransChInfo_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DL_CommonTransChInfo_r4 },
  { &hf_rrc_dl_AddReconfTransChInfoList_04, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DL_AddReconfTransChInfoList_r9 },
  { &hf_rrc_ul_DPCH_Info_01 , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UL_DPCH_Info_r7 },
  { &hf_rrc_ul_EDCH_Information_04, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_EDCH_Information_r9 },
  { &hf_rrc_dl_HSPDSCH_Information_06, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_HSPDSCH_Information_r9 },
  { &hf_rrc_dl_CommonInformation_06, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DL_CommonInformation_r10 },
  { &hf_rrc_dl_InformationPerRL_List_04, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DL_InformationPerRL_List_r7 },
  { &hf_rrc_frequencyInfo   , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_FrequencyInfo },
  { &hf_rrc_multi_frequencyInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_Multi_frequencyInfo_LCR_r7 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_complete_07(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_complete_07, T_complete_07_sequence);

  return offset;
}


static const per_sequence_t DLUL_HSPA_Information_r10_sequence[] = {
  { &hf_rrc_ul_EDCH_Information_04, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UL_EDCH_Information_r9 },
  { &hf_rrc_dl_CommonInformation_06, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DL_CommonInformation_r10 },
  { &hf_rrc_dl_InformationPerRL_List_04, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DL_InformationPerRL_List_r7 },
  { &hf_rrc_dl_HSPDSCH_Information_06, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DL_HSPDSCH_Information_r9 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_DLUL_HSPA_Information_r10(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_DLUL_HSPA_Information_r10, DLUL_HSPA_Information_r10_sequence);

  return offset;
}


static const per_sequence_t T_defaultConfig_07_sequence[] = {
  { &hf_rrc_defaultConfigMode, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DefaultConfigMode },
  { &hf_rrc_defaultConfigIdentity_03, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DefaultConfigIdentity_r6 },
  { &hf_rrc_dummy_11        , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DLUL_HSPA_Information_r10 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_defaultConfig_07(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_defaultConfig_07, T_defaultConfig_07_sequence);

  return offset;
}


static const value_string rrc_T_preConfigMode_07_vals[] = {
  {   0, "predefinedConfigIdentity" },
  {   1, "defaultConfig" },
  { 0, NULL }
};

static const per_choice_t T_preConfigMode_07_choice[] = {
  {   0, &hf_rrc_predefinedConfigIdentity, ASN1_NO_EXTENSIONS     , dissect_rrc_PredefinedConfigIdentity },
  {   1, &hf_rrc_defaultConfig_07, ASN1_NO_EXTENSIONS     , dissect_rrc_T_defaultConfig_07 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_preConfigMode_07(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_preConfigMode_07, T_preConfigMode_07_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t T_fdd_16_sequence[] = {
  { &hf_rrc_ul_DPCH_Info_04 , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UL_DPCH_InfoPostFDD },
  { &hf_rrc_dl_CommonInformationPost, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DL_CommonInformationPost },
  { &hf_rrc_dl_InformationPerRL_List_07, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DL_InformationPerRL_ListPostFDD },
  { &hf_rrc_frequencyInfo_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_FrequencyInfoFDD },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_fdd_16(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_fdd_16, T_fdd_16_sequence);

  return offset;
}


static const per_sequence_t T_tdd384_06_sequence[] = {
  { &hf_rrc_ul_DPCH_Info_05 , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UL_DPCH_InfoPostTDD },
  { &hf_rrc_dl_InformationPerRL, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DL_InformationPerRL_PostTDD },
  { &hf_rrc_frequencyInfo_02, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_FrequencyInfoTDD },
  { &hf_rrc_primaryCCPCH_TX_Power, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PrimaryCCPCH_TX_Power },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_tdd384_06(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_tdd384_06, T_tdd384_06_sequence);

  return offset;
}


static const per_sequence_t T_tdd128_06_sequence[] = {
  { &hf_rrc_ul_DPCH_Info_01 , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UL_DPCH_Info_r7 },
  { &hf_rrc_dl_InformationPerRL_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DL_InformationPerRL_PostTDD_LCR_r4 },
  { &hf_rrc_frequencyInfo_02, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_FrequencyInfoTDD },
  { &hf_rrc_primaryCCPCH_TX_Power, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PrimaryCCPCH_TX_Power },
  { &hf_rrc_multi_frequencyInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_Multi_frequencyInfo_LCR_r7 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_tdd128_06(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_tdd128_06, T_tdd128_06_sequence);

  return offset;
}


static const per_sequence_t T_tdd768_03_sequence[] = {
  { &hf_rrc_ul_DPCH_Info_01 , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UL_DPCH_Info_r7 },
  { &hf_rrc_dl_InformationPerRL_02, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DL_InformationPerRL_List_r7 },
  { &hf_rrc_frequencyInfo_02, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_FrequencyInfoTDD },
  { &hf_rrc_primaryCCPCH_TX_Power, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PrimaryCCPCH_TX_Power },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_tdd768_03(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_tdd768_03, T_tdd768_03_sequence);

  return offset;
}


static const value_string rrc_T_tdd_07_vals[] = {
  {   0, "tdd384" },
  {   1, "tdd128" },
  {   2, "tdd768" },
  { 0, NULL }
};

static const per_choice_t T_tdd_07_choice[] = {
  {   0, &hf_rrc_tdd384_06       , ASN1_NO_EXTENSIONS     , dissect_rrc_T_tdd384_06 },
  {   1, &hf_rrc_tdd128_06       , ASN1_NO_EXTENSIONS     , dissect_rrc_T_tdd128_06 },
  {   2, &hf_rrc_tdd768_03       , ASN1_NO_EXTENSIONS     , dissect_rrc_T_tdd768_03 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_tdd_07(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_tdd_07, T_tdd_07_choice,
                                 NULL);

  return offset;
}


static const value_string rrc_T_modeSpecificInfo_10_vals[] = {
  {   0, "fdd" },
  {   1, "tdd" },
  { 0, NULL }
};

static const per_choice_t T_modeSpecificInfo_10_choice[] = {
  {   0, &hf_rrc_fdd_16          , ASN1_NO_EXTENSIONS     , dissect_rrc_T_fdd_16 },
  {   1, &hf_rrc_tdd_08          , ASN1_NO_EXTENSIONS     , dissect_rrc_T_tdd_07 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_modeSpecificInfo_10(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_modeSpecificInfo_10, T_modeSpecificInfo_10_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t T_preconfiguration_07_sequence[] = {
  { &hf_rrc_preConfigMode_07, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_preConfigMode_07 },
  { &hf_rrc_rab_Info        , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RAB_Info_Post },
  { &hf_rrc_modeSpecificInfo_10, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_modeSpecificInfo_10 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_preconfiguration_07(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_preconfiguration_07, T_preconfiguration_07_sequence);

  return offset;
}


static const value_string rrc_T_specificationMode_07_vals[] = {
  {   0, "complete" },
  {   1, "preconfiguration" },
  { 0, NULL }
};

static const per_choice_t T_specificationMode_07_choice[] = {
  {   0, &hf_rrc_complete_07     , ASN1_NO_EXTENSIONS     , dissect_rrc_T_complete_07 },
  {   1, &hf_rrc_preconfiguration_07, ASN1_NO_EXTENSIONS     , dissect_rrc_T_preconfiguration_07 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_specificationMode_07(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_specificationMode_07, T_specificationMode_07_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t HandoverToUTRANCommand_r10_IEs_sequence[] = {
  { &hf_rrc_new_U_RNTI_01   , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_U_RNTI_Short },
  { &hf_rrc_cipheringAlgorithm_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CipheringAlgorithm_r7 },
  { &hf_rrc_supportForChangeOfUE_Capability, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { &hf_rrc_new_H_RNTI      , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_H_RNTI },
  { &hf_rrc_newPrimary_E_RNTI, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_E_RNTI },
  { &hf_rrc_newSecondary_E_RNTI, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_E_RNTI },
  { &hf_rrc_defaultConfigForCellFACH, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DefaultConfigForCellFACH },
  { &hf_rrc_specificationMode_07, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_specificationMode_07 },
  { &hf_rrc_maxAllowedUL_TX_Power, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MaxAllowedUL_TX_Power },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_HandoverToUTRANCommand_r10_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_HandoverToUTRANCommand_r10_IEs, HandoverToUTRANCommand_r10_IEs_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions_44_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_nonCriticalExtensions_44(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_nonCriticalExtensions_44, T_nonCriticalExtensions_44_sequence);

  return offset;
}


static const per_sequence_t T_v9c0NonCriticalExtensions_03_sequence[] = {
  { &hf_rrc_handoverToUTRANCommand_v9c0ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_HandoverToUTRANCommand_v9c0ext_IEs },
  { &hf_rrc_nonCriticalExtensions_44, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_nonCriticalExtensions_44 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v9c0NonCriticalExtensions_03(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v9c0NonCriticalExtensions_03, T_v9c0NonCriticalExtensions_03_sequence);

  return offset;
}


static const per_sequence_t T_r10_03_sequence[] = {
  { &hf_rrc_handoverToUTRANCommand_r10, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_HandoverToUTRANCommand_r10_IEs },
  { &hf_rrc_v9c0NonCriticalExtensions_03, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v9c0NonCriticalExtensions_03 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_r10_03(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_r10_03, T_r10_03_sequence);

  return offset;
}


static const per_sequence_t T_complete_08_sequence[] = {
  { &hf_rrc_srb_InformationSetupList_04, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SRB_InformationSetupList_r8 },
  { &hf_rrc_rab_InformationSetupList_07, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RAB_InformationSetupList_r11 },
  { &hf_rrc_ul_CommonTransChInfo_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UL_CommonTransChInfo_r4 },
  { &hf_rrc_ul_AddReconfTransChInfoList_03, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UL_AddReconfTransChInfoList_r8 },
  { &hf_rrc_dl_CommonTransChInfo_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DL_CommonTransChInfo_r4 },
  { &hf_rrc_dl_AddReconfTransChInfoList_06, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DL_AddReconfTransChInfoList_r11 },
  { &hf_rrc_ul_DPCH_Info_02 , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UL_DPCH_Info_r11 },
  { &hf_rrc_ul_EDCH_Information_05, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_EDCH_Information_r11 },
  { &hf_rrc_dl_HSPDSCH_Information_07, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_HSPDSCH_Information_r11 },
  { &hf_rrc_dl_CommonInformation_06, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DL_CommonInformation_r10 },
  { &hf_rrc_dl_InformationPerRL_List_04, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DL_InformationPerRL_List_r7 },
  { &hf_rrc_frequencyInfo   , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_FrequencyInfo },
  { &hf_rrc_multi_frequencyInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_Multi_frequencyInfo_LCR_r7 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_complete_08(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_complete_08, T_complete_08_sequence);

  return offset;
}


static const per_sequence_t T_defaultConfig_08_sequence[] = {
  { &hf_rrc_defaultConfigMode, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DefaultConfigMode },
  { &hf_rrc_defaultConfigIdentity_03, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DefaultConfigIdentity_r6 },
  { &hf_rrc_dummy_11        , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DLUL_HSPA_Information_r10 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_defaultConfig_08(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_defaultConfig_08, T_defaultConfig_08_sequence);

  return offset;
}


static const value_string rrc_T_preConfigMode_08_vals[] = {
  {   0, "predefinedConfigIdentity" },
  {   1, "defaultConfig" },
  { 0, NULL }
};

static const per_choice_t T_preConfigMode_08_choice[] = {
  {   0, &hf_rrc_predefinedConfigIdentity, ASN1_NO_EXTENSIONS     , dissect_rrc_PredefinedConfigIdentity },
  {   1, &hf_rrc_defaultConfig_08, ASN1_NO_EXTENSIONS     , dissect_rrc_T_defaultConfig_08 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_preConfigMode_08(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_preConfigMode_08, T_preConfigMode_08_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t T_fdd_17_sequence[] = {
  { &hf_rrc_ul_DPCH_Info_04 , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UL_DPCH_InfoPostFDD },
  { &hf_rrc_dl_CommonInformationPost, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DL_CommonInformationPost },
  { &hf_rrc_dl_InformationPerRL_List_07, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DL_InformationPerRL_ListPostFDD },
  { &hf_rrc_frequencyInfo_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_FrequencyInfoFDD },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_fdd_17(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_fdd_17, T_fdd_17_sequence);

  return offset;
}


static const per_sequence_t T_tdd384_07_sequence[] = {
  { &hf_rrc_ul_DPCH_Info_05 , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UL_DPCH_InfoPostTDD },
  { &hf_rrc_dl_InformationPerRL, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DL_InformationPerRL_PostTDD },
  { &hf_rrc_frequencyInfo_02, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_FrequencyInfoTDD },
  { &hf_rrc_primaryCCPCH_TX_Power, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PrimaryCCPCH_TX_Power },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_tdd384_07(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_tdd384_07, T_tdd384_07_sequence);

  return offset;
}


static const per_sequence_t T_tdd128_07_sequence[] = {
  { &hf_rrc_ul_DPCH_Info_01 , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UL_DPCH_Info_r7 },
  { &hf_rrc_dl_InformationPerRL_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DL_InformationPerRL_PostTDD_LCR_r4 },
  { &hf_rrc_frequencyInfo_02, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_FrequencyInfoTDD },
  { &hf_rrc_primaryCCPCH_TX_Power, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PrimaryCCPCH_TX_Power },
  { &hf_rrc_multi_frequencyInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_Multi_frequencyInfo_LCR_r7 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_tdd128_07(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_tdd128_07, T_tdd128_07_sequence);

  return offset;
}


static const per_sequence_t T_tdd768_04_sequence[] = {
  { &hf_rrc_ul_DPCH_Info_01 , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UL_DPCH_Info_r7 },
  { &hf_rrc_dl_InformationPerRL_02, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DL_InformationPerRL_List_r7 },
  { &hf_rrc_frequencyInfo_02, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_FrequencyInfoTDD },
  { &hf_rrc_primaryCCPCH_TX_Power, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PrimaryCCPCH_TX_Power },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_tdd768_04(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_tdd768_04, T_tdd768_04_sequence);

  return offset;
}


static const value_string rrc_T_tdd_08_vals[] = {
  {   0, "tdd384" },
  {   1, "tdd128" },
  {   2, "tdd768" },
  { 0, NULL }
};

static const per_choice_t T_tdd_08_choice[] = {
  {   0, &hf_rrc_tdd384_07       , ASN1_NO_EXTENSIONS     , dissect_rrc_T_tdd384_07 },
  {   1, &hf_rrc_tdd128_07       , ASN1_NO_EXTENSIONS     , dissect_rrc_T_tdd128_07 },
  {   2, &hf_rrc_tdd768_04       , ASN1_NO_EXTENSIONS     , dissect_rrc_T_tdd768_04 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_tdd_08(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_tdd_08, T_tdd_08_choice,
                                 NULL);

  return offset;
}


static const value_string rrc_T_modeSpecificInfo_11_vals[] = {
  {   0, "fdd" },
  {   1, "tdd" },
  { 0, NULL }
};

static const per_choice_t T_modeSpecificInfo_11_choice[] = {
  {   0, &hf_rrc_fdd_17          , ASN1_NO_EXTENSIONS     , dissect_rrc_T_fdd_17 },
  {   1, &hf_rrc_tdd_09          , ASN1_NO_EXTENSIONS     , dissect_rrc_T_tdd_08 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_modeSpecificInfo_11(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_modeSpecificInfo_11, T_modeSpecificInfo_11_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t T_preconfiguration_08_sequence[] = {
  { &hf_rrc_preConfigMode_08, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_preConfigMode_08 },
  { &hf_rrc_rab_Info        , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RAB_Info_Post },
  { &hf_rrc_modeSpecificInfo_11, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_modeSpecificInfo_11 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_preconfiguration_08(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_preconfiguration_08, T_preconfiguration_08_sequence);

  return offset;
}


static const value_string rrc_T_specificationMode_08_vals[] = {
  {   0, "complete" },
  {   1, "preconfiguration" },
  { 0, NULL }
};

static const per_choice_t T_specificationMode_08_choice[] = {
  {   0, &hf_rrc_complete_08     , ASN1_NO_EXTENSIONS     , dissect_rrc_T_complete_08 },
  {   1, &hf_rrc_preconfiguration_08, ASN1_NO_EXTENSIONS     , dissect_rrc_T_preconfiguration_08 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_specificationMode_08(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_specificationMode_08, T_specificationMode_08_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t HandoverToUTRANCommand_r11_IEs_sequence[] = {
  { &hf_rrc_new_U_RNTI_01   , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_U_RNTI_Short },
  { &hf_rrc_cipheringAlgorithm_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CipheringAlgorithm_r7 },
  { &hf_rrc_supportForChangeOfUE_Capability, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { &hf_rrc_new_H_RNTI      , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_H_RNTI },
  { &hf_rrc_newPrimary_E_RNTI, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_E_RNTI },
  { &hf_rrc_newSecondary_E_RNTI, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_E_RNTI },
  { &hf_rrc_defaultConfigForCellFACH, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DefaultConfigForCellFACH },
  { &hf_rrc_specificationMode_08, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_specificationMode_08 },
  { &hf_rrc_maxAllowedUL_TX_Power, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MaxAllowedUL_TX_Power },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_HandoverToUTRANCommand_r11_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_HandoverToUTRANCommand_r11_IEs, HandoverToUTRANCommand_r11_IEs_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions_45_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_nonCriticalExtensions_45(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_nonCriticalExtensions_45, T_nonCriticalExtensions_45_sequence);

  return offset;
}


static const per_sequence_t T_r11_03_sequence[] = {
  { &hf_rrc_handoverToUTRANCommand_r11, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_HandoverToUTRANCommand_r11_IEs },
  { &hf_rrc_handoverToUTRANCommand_r11_add_ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_BIT_STRING },
  { &hf_rrc_nonCriticalExtensions_45, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_nonCriticalExtensions_45 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_r11_03(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_r11_03, T_r11_03_sequence);

  return offset;
}


static const per_sequence_t T_criticalExtensions_37_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_criticalExtensions_37(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_criticalExtensions_37, T_criticalExtensions_37_sequence);

  return offset;
}


static const value_string rrc_T_criticalExtensions_36_vals[] = {
  {   0, "r11" },
  {   1, "criticalExtensions" },
  { 0, NULL }
};

static const per_choice_t T_criticalExtensions_36_choice[] = {
  {   0, &hf_rrc_r11_03          , ASN1_NO_EXTENSIONS     , dissect_rrc_T_r11_03 },
  {   1, &hf_rrc_criticalExtensions_38, ASN1_NO_EXTENSIONS     , dissect_rrc_T_criticalExtensions_37 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_criticalExtensions_36(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_criticalExtensions_36, T_criticalExtensions_36_choice,
                                 NULL);

  return offset;
}


static const value_string rrc_HandoverToUTRANCommand_LaterCriticalExtensions_vals[] = {
  {   0, "r10" },
  {   1, "criticalExtensions" },
  { 0, NULL }
};

static const per_choice_t HandoverToUTRANCommand_LaterCriticalExtensions_choice[] = {
  {   0, &hf_rrc_r10_03          , ASN1_NO_EXTENSIONS     , dissect_rrc_T_r10_03 },
  {   1, &hf_rrc_criticalExtensions_37, ASN1_NO_EXTENSIONS     , dissect_rrc_T_criticalExtensions_36 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_HandoverToUTRANCommand_LaterCriticalExtensions(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_HandoverToUTRANCommand_LaterCriticalExtensions, HandoverToUTRANCommand_LaterCriticalExtensions_choice,
                                 NULL);

  return offset;
}


static const value_string rrc_T_criticalExtensions_35_vals[] = {
  {   0, "r9" },
  {   1, "criticalExtensions" },
  { 0, NULL }
};

static const per_choice_t T_criticalExtensions_35_choice[] = {
  {   0, &hf_rrc_r9_03           , ASN1_NO_EXTENSIONS     , dissect_rrc_T_r9_03 },
  {   1, &hf_rrc_criticalExtensions_36, ASN1_NO_EXTENSIONS     , dissect_rrc_HandoverToUTRANCommand_LaterCriticalExtensions },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_criticalExtensions_35(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_criticalExtensions_35, T_criticalExtensions_35_choice,
                                 NULL);

  return offset;
}


static const value_string rrc_T_criticalExtensions_34_vals[] = {
  {   0, "r8" },
  {   1, "criticalExtensions" },
  { 0, NULL }
};

static const per_choice_t T_criticalExtensions_34_choice[] = {
  {   0, &hf_rrc_r8_03           , ASN1_NO_EXTENSIONS     , dissect_rrc_T_r8_03 },
  {   1, &hf_rrc_criticalExtensions_35, ASN1_NO_EXTENSIONS     , dissect_rrc_T_criticalExtensions_35 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_criticalExtensions_34(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_criticalExtensions_34, T_criticalExtensions_34_choice,
                                 NULL);

  return offset;
}


static const value_string rrc_T_criticalExtensions_33_vals[] = {
  {   0, "r7" },
  {   1, "criticalExtensions" },
  { 0, NULL }
};

static const per_choice_t T_criticalExtensions_33_choice[] = {
  {   0, &hf_rrc_r7_03           , ASN1_NO_EXTENSIONS     , dissect_rrc_T_r7_03 },
  {   1, &hf_rrc_criticalExtensions_34, ASN1_NO_EXTENSIONS     , dissect_rrc_T_criticalExtensions_34 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_criticalExtensions_33(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_criticalExtensions_33, T_criticalExtensions_33_choice,
                                 NULL);

  return offset;
}


static const value_string rrc_T_criticalExtensions_32_vals[] = {
  {   0, "r6" },
  {   1, "criticalExtensions" },
  { 0, NULL }
};

static const per_choice_t T_criticalExtensions_32_choice[] = {
  {   0, &hf_rrc_r6_03           , ASN1_NO_EXTENSIONS     , dissect_rrc_T_r6_03 },
  {   1, &hf_rrc_criticalExtensions_33, ASN1_NO_EXTENSIONS     , dissect_rrc_T_criticalExtensions_33 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_criticalExtensions_32(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_criticalExtensions_32, T_criticalExtensions_32_choice,
                                 NULL);

  return offset;
}


static const value_string rrc_T_criticalExtensions_31_vals[] = {
  {   0, "r5" },
  {   1, "criticalExtensions" },
  { 0, NULL }
};

static const per_choice_t T_criticalExtensions_31_choice[] = {
  {   0, &hf_rrc_r5_02           , ASN1_NO_EXTENSIONS     , dissect_rrc_T_r5_02 },
  {   1, &hf_rrc_criticalExtensions_32, ASN1_NO_EXTENSIONS     , dissect_rrc_T_criticalExtensions_32 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_criticalExtensions_31(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_criticalExtensions_31, T_criticalExtensions_31_choice,
                                 NULL);

  return offset;
}


static const value_string rrc_T_criticalExtensions_30_vals[] = {
  {   0, "r4" },
  {   1, "criticalExtensions" },
  { 0, NULL }
};

static const per_choice_t T_criticalExtensions_30_choice[] = {
  {   0, &hf_rrc_r4_02           , ASN1_NO_EXTENSIONS     , dissect_rrc_T_r4_02 },
  {   1, &hf_rrc_criticalExtensions_31, ASN1_NO_EXTENSIONS     , dissect_rrc_T_criticalExtensions_31 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_criticalExtensions_30(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_criticalExtensions_30, T_criticalExtensions_30_choice,
                                 NULL);

  return offset;
}


static const value_string rrc_HandoverToUTRANCommand_vals[] = {
  {   0, "r3" },
  {   1, "criticalExtensions" },
  { 0, NULL }
};

static const per_choice_t HandoverToUTRANCommand_choice[] = {
  {   0, &hf_rrc_r3_08           , ASN1_NO_EXTENSIONS     , dissect_rrc_T_r3_08 },
  {   1, &hf_rrc_criticalExtensions_30, ASN1_NO_EXTENSIONS     , dissect_rrc_T_criticalExtensions_30 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_HandoverToUTRANCommand(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
#line 886 "../../asn1/rrc/rrc.cnf"
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_HandoverToUTRANCommand, HandoverToUTRANCommand_choice,
                                 NULL);




  return offset;
}


static const per_sequence_t InitialDirectTransfer_v7g0ext_IEs_sequence[] = {
  { &hf_rrc_measuredResultsOnRACH_v7g0ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MeasuredResultsOnRACH_v7g0ext },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_InitialDirectTransfer_v7g0ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_InitialDirectTransfer_v7g0ext_IEs, InitialDirectTransfer_v7g0ext_IEs_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions_48_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_nonCriticalExtensions_48(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_nonCriticalExtensions_48, T_nonCriticalExtensions_48_sequence);

  return offset;
}


static const per_sequence_t InitialDirectTransfer_r3_add_ext_IEs_sequence[] = {
  { &hf_rrc_initialDirectTransfer_v7g0ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_InitialDirectTransfer_v7g0ext_IEs },
  { &hf_rrc_nonCriticalExtensions_48, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_nonCriticalExtensions_48 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_InitialDirectTransfer_r3_add_ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_InitialDirectTransfer_r3_add_ext_IEs, InitialDirectTransfer_r3_add_ext_IEs_sequence);

  return offset;
}


static const value_string rrc_T_other_vals[] = {
  {   0, "notStored" },
  {   1, "storedWithDifferentValueTag" },
  { 0, NULL }
};

static const per_choice_t T_other_choice[] = {
  {   0, &hf_rrc_notStored       , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   1, &hf_rrc_storedWithDifferentValueTag, ASN1_NO_EXTENSIONS     , dissect_rrc_PredefinedConfigValueTag },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_other(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_other, T_other_choice,
                                 NULL);

  return offset;
}


static const value_string rrc_PredefinedConfigStatusInfo_vals[] = {
  {   0, "storedWithValueTagSameAsPrevius" },
  {   1, "other" },
  { 0, NULL }
};

static const per_choice_t PredefinedConfigStatusInfo_choice[] = {
  {   0, &hf_rrc_storedWithValueTagSameAsPrevius, ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   1, &hf_rrc_other           , ASN1_NO_EXTENSIONS     , dissect_rrc_T_other },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_PredefinedConfigStatusInfo(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_PredefinedConfigStatusInfo, PredefinedConfigStatusInfo_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t PredefinedConfigStatusList_sequence_of[1] = {
  { &hf_rrc_PredefinedConfigStatusList_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PredefinedConfigStatusInfo },
};

static int
dissect_rrc_PredefinedConfigStatusList(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_PredefinedConfigStatusList, PredefinedConfigStatusList_sequence_of,
                                                  maxPredefConfig, maxPredefConfig, FALSE);

  return offset;
}


static const value_string rrc_T_predefinedConfigStatusList_vals[] = {
  {   0, "absent" },
  {   1, "present" },
  { 0, NULL }
};

static const per_choice_t T_predefinedConfigStatusList_choice[] = {
  {   0, &hf_rrc_absent          , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   1, &hf_rrc_present         , ASN1_NO_EXTENSIONS     , dissect_rrc_PredefinedConfigStatusList },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_predefinedConfigStatusList(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_predefinedConfigStatusList, T_predefinedConfigStatusList_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t UE_SecurityInformation_sequence[] = {
  { &hf_rrc_start_CS        , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_START_Value },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UE_SecurityInformation(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UE_SecurityInformation, UE_SecurityInformation_sequence);

  return offset;
}


static const value_string rrc_T_uE_SecurityInformation_vals[] = {
  {   0, "absent" },
  {   1, "present" },
  { 0, NULL }
};

static const per_choice_t T_uE_SecurityInformation_choice[] = {
  {   0, &hf_rrc_absent          , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   1, &hf_rrc_present_01      , ASN1_NO_EXTENSIONS     , dissect_rrc_UE_SecurityInformation },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_uE_SecurityInformation(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_uE_SecurityInformation, T_uE_SecurityInformation_choice,
                                 NULL);

  return offset;
}



static int
dissect_rrc_T_present(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
#line 155 "../../asn1/rrc/rrc.cnf"
  tvbuff_t *ue_radio_access_cap_info_tvb=NULL;

  offset = dissect_per_octet_string(tvb, offset, actx, tree, hf_index,
                                       0, 63, FALSE, &ue_radio_access_cap_info_tvb);


  if (ue_radio_access_cap_info_tvb)
    call_dissector(rrc_ue_radio_access_cap_info_handle,ue_radio_access_cap_info_tvb,actx->pinfo, tree);



  return offset;
}


static const value_string rrc_T_ue_CapabilityContainer_vals[] = {
  {   0, "absent" },
  {   1, "present" },
  { 0, NULL }
};

static const per_choice_t T_ue_CapabilityContainer_choice[] = {
  {   0, &hf_rrc_absent          , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   1, &hf_rrc_present_02      , ASN1_NO_EXTENSIONS     , dissect_rrc_T_present },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_ue_CapabilityContainer(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_ue_CapabilityContainer, T_ue_CapabilityContainer_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t InterRATHandoverInfo_v390ext_IEs_sequence[] = {
  { &hf_rrc_ue_RadioAccessCapability_v380ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_RadioAccessCapability_v380ext },
  { &hf_rrc_dl_PhysChCapabilityFDD_v380ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DL_PhysChCapabilityFDD_v380ext },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_InterRATHandoverInfo_v390ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_InterRATHandoverInfo_v390ext_IEs, InterRATHandoverInfo_v390ext_IEs_sequence);

  return offset;
}


static const per_sequence_t InterRATHandoverInfo_v3a0ext_IEs_sequence[] = {
  { &hf_rrc_ue_RadioAccessCapability_v3a0ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_RadioAccessCapability_v3a0ext },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_InterRATHandoverInfo_v3a0ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_InterRATHandoverInfo_v3a0ext_IEs, InterRATHandoverInfo_v3a0ext_IEs_sequence);

  return offset;
}



static int
dissect_rrc_UESpecificBehaviourInformation1interRAT(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_bit_string(tvb, offset, actx, tree, hf_index,
                                     8, 8, FALSE, NULL);

  return offset;
}


static const per_sequence_t InterRATHandoverInfo_v3d0ext_IEs_sequence[] = {
  { &hf_rrc_uESpecificBehaviourInformation1interRAT, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UESpecificBehaviourInformation1interRAT },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_InterRATHandoverInfo_v3d0ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_InterRATHandoverInfo_v3d0ext_IEs, InterRATHandoverInfo_v3d0ext_IEs_sequence);

  return offset;
}



static int
dissect_rrc_T_interRATHandoverInfo_r3_add_ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_bit_string_containing_pdu_new(tvb, offset, actx, tree, hf_index,
                                                              NO_BOUND, NO_BOUND, FALSE, dissect_InterRATHandoverInfo_r3_add_ext_IEs_PDU);

  return offset;
}


static const per_sequence_t InterRATHandoverInfo_v3g0ext_IEs_sequence[] = {
  { &hf_rrc_ue_RadioAccessCapability_v3g0ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_RadioAccessCapability_v3g0ext },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_InterRATHandoverInfo_v3g0ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_InterRATHandoverInfo_v3g0ext_IEs, InterRATHandoverInfo_v3g0ext_IEs_sequence);

  return offset;
}


static const per_sequence_t InterRATHandoverInfo_v4b0ext_IEs_sequence[] = {
  { &hf_rrc_accessStratumReleaseIndicator, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_AccessStratumReleaseIndicator },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_InterRATHandoverInfo_v4b0ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_InterRATHandoverInfo_v4b0ext_IEs, InterRATHandoverInfo_v4b0ext_IEs_sequence);

  return offset;
}


static const per_sequence_t InterRATHandoverInfo_v4d0ext_IEs_sequence[] = {
  { &hf_rrc_tdd128_RF_Capability, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RadioFrequencyBandTDDList },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_InterRATHandoverInfo_v4d0ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_InterRATHandoverInfo_v4d0ext_IEs, InterRATHandoverInfo_v4d0ext_IEs_sequence);

  return offset;
}


static const per_sequence_t PredefinedConfigValueTagList_sequence_of[1] = {
  { &hf_rrc_PredefinedConfigValueTagList_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PredefinedConfigValueTag },
};

static int
dissect_rrc_PredefinedConfigValueTagList(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_PredefinedConfigValueTagList, PredefinedConfigValueTagList_sequence_of,
                                                  1, maxPredefConfig, FALSE);

  return offset;
}


static const per_sequence_t PredefinedConfigSetWithDifferentValueTag_sequence[] = {
  { &hf_rrc_startPosition   , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_INTEGER_0_10 },
  { &hf_rrc_valueTagList    , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PredefinedConfigValueTagList },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_PredefinedConfigSetWithDifferentValueTag(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_PredefinedConfigSetWithDifferentValueTag, PredefinedConfigSetWithDifferentValueTag_sequence);

  return offset;
}


static const per_sequence_t PredefinedConfigSetsWithDifferentValueTag_sequence_of[1] = {
  { &hf_rrc_PredefinedConfigSetsWithDifferentValueTag_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PredefinedConfigSetWithDifferentValueTag },
};

static int
dissect_rrc_PredefinedConfigSetsWithDifferentValueTag(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_PredefinedConfigSetsWithDifferentValueTag, PredefinedConfigSetsWithDifferentValueTag_sequence_of,
                                                  1, 2, FALSE);

  return offset;
}


static const per_sequence_t PredefinedConfigStatusListVarSz_sequence_of[1] = {
  { &hf_rrc_PredefinedConfigStatusListVarSz_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PredefinedConfigStatusInfo },
};

static int
dissect_rrc_PredefinedConfigStatusListVarSz(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_PredefinedConfigStatusListVarSz, PredefinedConfigStatusListVarSz_sequence_of,
                                                  1, maxPredefConfig, FALSE);

  return offset;
}


static const per_sequence_t PredefinedConfigStatusListComp_sequence[] = {
  { &hf_rrc_setsWithDifferentValueTag, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PredefinedConfigSetsWithDifferentValueTag },
  { &hf_rrc_otherEntries    , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_PredefinedConfigStatusListVarSz },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_PredefinedConfigStatusListComp(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_PredefinedConfigStatusListComp, PredefinedConfigStatusListComp_sequence);

  return offset;
}


static const value_string rrc_RF_CapabBandFDDComp_vals[] = {
  {   0, "notSupported" },
  {   1, "default-TxRx-separation" },
  {   2, "spare2" },
  {   3, "spare1" },
  { 0, NULL }
};


static int
dissect_rrc_RF_CapabBandFDDComp(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     4, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t RF_CapabBandListFDDComp_sequence_of[1] = {
  { &hf_rrc_RF_CapabBandListFDDComp_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RF_CapabBandFDDComp },
};

static int
dissect_rrc_RF_CapabBandListFDDComp(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_RF_CapabBandListFDDComp, RF_CapabBandListFDDComp_sequence_of,
                                                  1, maxFreqBandsFDD, FALSE);

  return offset;
}


static const value_string rrc_T_fdd_47_vals[] = {
  {   0, "notSupported" },
  {   1, "supported" },
  { 0, NULL }
};

static const per_choice_t T_fdd_47_choice[] = {
  {   0, &hf_rrc_notSupported    , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   1, &hf_rrc_supported_06    , ASN1_NO_EXTENSIONS     , dissect_rrc_RF_CapabBandListFDDComp },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_fdd_47(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_fdd_47, T_fdd_47_choice,
                                 NULL);

  return offset;
}


static const value_string rrc_T_tdd384_RF_Capability_vals[] = {
  {   0, "notSupported" },
  {   1, "supported" },
  { 0, NULL }
};

static const per_choice_t T_tdd384_RF_Capability_choice[] = {
  {   0, &hf_rrc_notSupported    , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   1, &hf_rrc_supported_07    , ASN1_NO_EXTENSIONS     , dissect_rrc_RadioFrequencyBandTDDList },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_tdd384_RF_Capability(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_tdd384_RF_Capability, T_tdd384_RF_Capability_choice,
                                 NULL);

  return offset;
}


static const value_string rrc_T_tdd128_RF_Capability_vals[] = {
  {   0, "notSupported" },
  {   1, "supported" },
  { 0, NULL }
};

static const per_choice_t T_tdd128_RF_Capability_choice[] = {
  {   0, &hf_rrc_notSupported    , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   1, &hf_rrc_supported_07    , ASN1_NO_EXTENSIONS     , dissect_rrc_RadioFrequencyBandTDDList },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_tdd128_RF_Capability(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_tdd128_RF_Capability, T_tdd128_RF_Capability_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t RF_CapabilityComp_sequence[] = {
  { &hf_rrc_fdd_48          , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_fdd_47 },
  { &hf_rrc_tdd384_RF_Capability, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_tdd384_RF_Capability },
  { &hf_rrc_tdd128_RF_Capability_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_tdd128_RF_Capability },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_RF_CapabilityComp(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_RF_CapabilityComp, RF_CapabilityComp_sequence);

  return offset;
}


static const per_sequence_t UE_RadioAccessCapabilityComp_sequence[] = {
  { &hf_rrc_totalAM_RLCMemoryExceeds10kB, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { &hf_rrc_rf_CapabilityComp, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RF_CapabilityComp },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UE_RadioAccessCapabilityComp(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UE_RadioAccessCapabilityComp, UE_RadioAccessCapabilityComp_sequence);

  return offset;
}


static const per_sequence_t InterRATHandoverInfo_v590ext_IEs_sequence[] = {
  { &hf_rrc_predefinedConfigStatusListComp, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_PredefinedConfigStatusListComp },
  { &hf_rrc_ue_RadioAccessCapabilityComp, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_RadioAccessCapabilityComp },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_InterRATHandoverInfo_v590ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_InterRATHandoverInfo_v590ext_IEs, InterRATHandoverInfo_v590ext_IEs_sequence);

  return offset;
}


static const per_sequence_t UE_SecurityInformation2_sequence[] = {
  { &hf_rrc_start_PS        , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_START_Value },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UE_SecurityInformation2(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UE_SecurityInformation2, UE_SecurityInformation2_sequence);

  return offset;
}


static const per_sequence_t RF_CapabBandListFDDComp_ext_sequence_of[1] = {
  { &hf_rrc_RF_CapabBandListFDDComp_ext_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RF_CapabBandFDDComp },
};

static int
dissect_rrc_RF_CapabBandListFDDComp_ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_RF_CapabBandListFDDComp_ext, RF_CapabBandListFDDComp_ext_sequence_of,
                                                  1, maxFreqBandsFDD_ext, FALSE);

  return offset;
}


static const per_sequence_t UE_RadioAccessCapabilityComp_ext_sequence[] = {
  { &hf_rrc_rf_CapabilityFDDComp_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RF_CapabBandListFDDComp_ext },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UE_RadioAccessCapabilityComp_ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UE_RadioAccessCapabilityComp_ext, UE_RadioAccessCapabilityComp_ext_sequence);

  return offset;
}


static const per_sequence_t T_supported_06_sequence[] = {
  { &hf_rrc_edch_PhysicalLayerCategory, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_1_16 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_supported_06(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_supported_06, T_supported_06_sequence);

  return offset;
}


static const value_string rrc_T_fdd_edch_vals[] = {
  {   0, "supported" },
  {   1, "unsupported" },
  { 0, NULL }
};

static const per_choice_t T_fdd_edch_choice[] = {
  {   0, &hf_rrc_supported_12    , ASN1_NO_EXTENSIONS     , dissect_rrc_T_supported_06 },
  {   1, &hf_rrc_unsupported     , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_fdd_edch(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_fdd_edch, T_fdd_edch_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t PhysicalChannelCapability_edch_r6_sequence[] = {
  { &hf_rrc_fdd_edch        , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_fdd_edch },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_PhysicalChannelCapability_edch_r6(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_PhysicalChannelCapability_edch_r6, PhysicalChannelCapability_edch_r6_sequence);

  return offset;
}


static const per_sequence_t T_fddPhysicalChannelCapab_hspdsch_edch_sequence[] = {
  { &hf_rrc_dl_CapabilityWithSimultaneousHS_DSCHConfig, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_CapabilityWithSimultaneousHS_DSCHConfig },
  { &hf_rrc_physicalChannelCapabComp_hspdsch_r6, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_HSDSCH_physical_layer_category },
  { &hf_rrc_physicalChannelCapability_edch_r6, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PhysicalChannelCapability_edch_r6 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_fddPhysicalChannelCapab_hspdsch_edch(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_fddPhysicalChannelCapab_hspdsch_edch, T_fddPhysicalChannelCapab_hspdsch_edch_sequence);

  return offset;
}


static const per_sequence_t UE_RadioAccessCapabilityComp2_sequence[] = {
  { &hf_rrc_fddPhysicalChannelCapab_hspdsch_edch, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_fddPhysicalChannelCapab_hspdsch_edch },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UE_RadioAccessCapabilityComp2(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UE_RadioAccessCapabilityComp2, UE_RadioAccessCapabilityComp2_sequence);

  return offset;
}


static const per_sequence_t InterRATHandoverInfo_v690ext_IEs_sequence[] = {
  { &hf_rrc_ue_SecurityInformation2, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_SecurityInformation2 },
  { &hf_rrc_ue_RadioAccessCapabilityComp_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_RadioAccessCapabilityComp_ext },
  { &hf_rrc_ue_RadioAccessCapabilityComp2, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UE_RadioAccessCapabilityComp2 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_InterRATHandoverInfo_v690ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_InterRATHandoverInfo_v690ext_IEs, InterRATHandoverInfo_v690ext_IEs_sequence);

  return offset;
}


static const value_string rrc_T_supportForSIB11bis_vals[] = {
  {   0, "true" },
  { 0, NULL }
};


static int
dissect_rrc_T_supportForSIB11bis(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t InterRATHandoverInfo_v6b0ext_IEs_sequence[] = {
  { &hf_rrc_supportForSIB11bis, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_supportForSIB11bis },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_InterRATHandoverInfo_v6b0ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_InterRATHandoverInfo_v6b0ext_IEs, InterRATHandoverInfo_v6b0ext_IEs_sequence);

  return offset;
}


static const value_string rrc_T_supportForFDPCH_vals[] = {
  {   0, "true" },
  { 0, NULL }
};


static int
dissect_rrc_T_supportForFDPCH(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t InterRATHandoverInfo_v6e0ext_IEs_sequence[] = {
  { &hf_rrc_supportForFDPCH , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_supportForFDPCH },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_InterRATHandoverInfo_v6e0ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_InterRATHandoverInfo_v6e0ext_IEs, InterRATHandoverInfo_v6e0ext_IEs_sequence);

  return offset;
}


static const value_string rrc_RadioFrequencyBandTDDList_r7_vals[] = {
  {   0, "a" },
  {   1, "b" },
  {   2, "c" },
  {   3, "d" },
  {   4, "ab" },
  {   5, "ac" },
  {   6, "ad" },
  {   7, "bc" },
  {   8, "bd" },
  {   9, "cd" },
  {  10, "abc" },
  {  11, "abd" },
  {  12, "acd" },
  {  13, "bcd" },
  {  14, "abcd" },
  {  15, "notabcd" },
  { 0, NULL }
};


static int
dissect_rrc_RadioFrequencyBandTDDList_r7(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     16, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t T_tdd768RF_Capability_sequence[] = {
  { &hf_rrc_ue_PowerClass   , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UE_PowerClass },
  { &hf_rrc_radioFrequencyBandTDDList_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RadioFrequencyBandTDDList_r7 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_tdd768RF_Capability(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_tdd768RF_Capability, T_tdd768RF_Capability_sequence);

  return offset;
}


static const per_sequence_t RF_Capability_v770ext_sequence[] = {
  { &hf_rrc_tdd384RF_Capability, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RadioFrequencyBandTDDList_r7 },
  { &hf_rrc_tdd768RF_Capability, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_tdd768RF_Capability },
  { &hf_rrc_tdd128RF_Capability, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RadioFrequencyBandTDDList_r7 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_RF_Capability_v770ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_RF_Capability_v770ext, RF_Capability_v770ext_sequence);

  return offset;
}



static int
dissect_rrc_MaxPhysChPerFrame_768(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            1U, 448U, NULL, FALSE);

  return offset;
}


static const value_string rrc_MinimumSF_DL_768_vals[] = {
  {   0, "sf1" },
  {   1, "sf32" },
  { 0, NULL }
};


static int
dissect_rrc_MinimumSF_DL_768(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     2, NULL, FALSE, 0, NULL);

  return offset;
}



static int
dissect_rrc_MaxPhysChPerTS_768(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            1U, 32U, NULL, FALSE);

  return offset;
}


static const per_sequence_t DL_PhysChCapabilityInfoTDD_768_sequence[] = {
  { &hf_rrc_maxTS_PerFrame  , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MaxTS_PerFrame },
  { &hf_rrc_maxPhysChPerFrame_02, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MaxPhysChPerFrame_768 },
  { &hf_rrc_minimumSF_01    , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MinimumSF_DL_768 },
  { &hf_rrc_supportOfPDSCH  , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { &hf_rrc_maxPhysChPerTS_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MaxPhysChPerTS_768 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_DL_PhysChCapabilityInfoTDD_768(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_DL_PhysChCapabilityInfoTDD_768, DL_PhysChCapabilityInfoTDD_768_sequence);

  return offset;
}


static const per_sequence_t T_tddPhysChCapability_768_01_sequence[] = {
  { &hf_rrc_downlinkPhysChCapability_11, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DL_PhysChCapabilityInfoTDD_768 },
  { &hf_rrc_uplinkPhysChCapability_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UL_PhysChCapabilityTDD },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_tddPhysChCapability_768_01(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_tddPhysChCapability_768_01, T_tddPhysChCapability_768_01_sequence);

  return offset;
}



static int
dissect_rrc_MaxPhysChPerTimeslot_LCR_r7(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            1U, 4U, NULL, FALSE);

  return offset;
}


static const per_sequence_t UL_PhysChCapabilityInfoTDD_128_v770ext_sequence[] = {
  { &hf_rrc_maxPhysChPerTimeslot_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MaxPhysChPerTimeslot_LCR_r7 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UL_PhysChCapabilityInfoTDD_128_v770ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UL_PhysChCapabilityInfoTDD_128_v770ext, UL_PhysChCapabilityInfoTDD_128_v770ext_sequence);

  return offset;
}


static const per_sequence_t T_tddPhysChCapability_128_03_sequence[] = {
  { &hf_rrc_uplinkPhysChCapability_08, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UL_PhysChCapabilityInfoTDD_128_v770ext },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_tddPhysChCapability_128_03(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_tddPhysChCapability_128_03, T_tddPhysChCapability_128_03_sequence);

  return offset;
}


static const per_sequence_t PhysicalChannelCapabilityInfo_v770ext_sequence[] = {
  { &hf_rrc_tddPhysChCapability_768_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_tddPhysChCapability_768_01 },
  { &hf_rrc_tddPhysChCapability_128_03, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_tddPhysChCapability_128_03 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_PhysicalChannelCapabilityInfo_v770ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_PhysicalChannelCapabilityInfo_v770ext, PhysicalChannelCapabilityInfo_v770ext_sequence);

  return offset;
}


static const value_string rrc_T_ganssSupportIndication_vals[] = {
  {   0, "true" },
  { 0, NULL }
};


static int
dissect_rrc_T_ganssSupportIndication(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}


static const value_string rrc_T_mac_ehsSupport_02_vals[] = {
  {   0, "true" },
  { 0, NULL }
};


static int
dissect_rrc_T_mac_ehsSupport_02(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}


static const value_string rrc_UE_SpecificCapabilityInformation_LCRTDD_vals[] = {
  {   0, "nF" },
  {   1, "nFMc-TriRx-UniTx" },
  {   2, "nFMc-TriRx-TriTx" },
  {   3, "nFMc-HexRx-UniTx" },
  {   4, "nFMc-HexRx-TriTx" },
  {   5, "nFMc-HexRx-HexTx" },
  {   6, "nFMc-TwoRx-UniTx-Discontiguous" },
  {   7, "nFMc-TwoRx-TwoTx-Discontiguous" },
  {   8, "nFMc-TwoRx-UniTx-Contiguous" },
  {   9, "nFMc-TwoRx-TwoTx-Contiguous" },
  {  10, "spare6" },
  {  11, "spare5" },
  {  12, "spare4" },
  {  13, "spare3" },
  {  14, "spare2" },
  {  15, "spare1" },
  { 0, NULL }
};


static int
dissect_rrc_UE_SpecificCapabilityInformation_LCRTDD(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     16, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t UE_RadioAccessCapabilityInfo_v770ext_sequence[] = {
  { &hf_rrc_rf_Capability   , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RF_Capability_v770ext },
  { &hf_rrc_physicalChannelCapability_06, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PhysicalChannelCapabilityInfo_v770ext },
  { &hf_rrc_ganssSupportIndication, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_ganssSupportIndication },
  { &hf_rrc_mac_ehsSupport_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_mac_ehsSupport_02 },
  { &hf_rrc_ue_SpecificCapabilityInformation, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_SpecificCapabilityInformation_LCRTDD },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UE_RadioAccessCapabilityInfo_v770ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UE_RadioAccessCapabilityInfo_v770ext, UE_RadioAccessCapabilityInfo_v770ext_sequence);

  return offset;
}


static const value_string rrc_T_tdd768_RF_Capability_vals[] = {
  {   0, "notSupported" },
  {   1, "supported" },
  { 0, NULL }
};

static const per_choice_t T_tdd768_RF_Capability_choice[] = {
  {   0, &hf_rrc_notSupported    , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   1, &hf_rrc_supported_08    , ASN1_NO_EXTENSIONS     , dissect_rrc_RadioFrequencyBandTDDList_r7 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_tdd768_RF_Capability(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_tdd768_RF_Capability, T_tdd768_RF_Capability_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t RF_CapabilityComp_v770ext_sequence[] = {
  { &hf_rrc_tdd384_RF_Capability_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RadioFrequencyBandTDDList_r7 },
  { &hf_rrc_tdd768_RF_Capability, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_tdd768_RF_Capability },
  { &hf_rrc_tdd128_RF_Capability_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RadioFrequencyBandTDDList_r7 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_RF_CapabilityComp_v770ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_RF_CapabilityComp_v770ext, RF_CapabilityComp_v770ext_sequence);

  return offset;
}


static const value_string rrc_T_securityCapabilityIndication_vals[] = {
  {   0, "true" },
  { 0, NULL }
};


static int
dissect_rrc_T_securityCapabilityIndication(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}


static const value_string rrc_T_ganssSupportIndication_01_vals[] = {
  {   0, "true" },
  { 0, NULL }
};


static int
dissect_rrc_T_ganssSupportIndication_01(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}


static const value_string rrc_T_mac_ehsSupport_03_vals[] = {
  {   0, "true" },
  { 0, NULL }
};


static int
dissect_rrc_T_mac_ehsSupport_03(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t UE_RadioAccessCapabilityComp_v770ext_sequence[] = {
  { &hf_rrc_rf_CapabilityComp_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RF_CapabilityComp_v770ext },
  { &hf_rrc_securityCapabilityIndication, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_securityCapabilityIndication },
  { &hf_rrc_ganssSupportIndication_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_ganssSupportIndication_01 },
  { &hf_rrc_mac_ehsSupport_03, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_mac_ehsSupport_03 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UE_RadioAccessCapabilityComp_v770ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UE_RadioAccessCapabilityComp_v770ext, UE_RadioAccessCapabilityComp_v770ext_sequence);

  return offset;
}



static int
dissect_rrc_HSDSCH_physical_layer_category_ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            1U, 20U, NULL, FALSE);

  return offset;
}



static int
dissect_rrc_INTEGER_7(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            7U, 7U, NULL, FALSE);

  return offset;
}


static const value_string rrc_T_hsscchlessHsdschOperation_01_vals[] = {
  {   0, "true" },
  { 0, NULL }
};


static int
dissect_rrc_T_hsscchlessHsdschOperation_01(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}


static const value_string rrc_T_enhancedFdpch_01_vals[] = {
  {   0, "true" },
  { 0, NULL }
};


static int
dissect_rrc_T_enhancedFdpch_01(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}


static const value_string rrc_T_hsdschReception_CellFach_01_vals[] = {
  {   0, "true" },
  { 0, NULL }
};


static int
dissect_rrc_T_hsdschReception_CellFach_01(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}


static const value_string rrc_T_hsdschReception_CellUraPch_01_vals[] = {
  {   0, "true" },
  { 0, NULL }
};


static int
dissect_rrc_T_hsdschReception_CellUraPch_01(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}


static const value_string rrc_T_discontinuousDpcchTransmission_01_vals[] = {
  {   0, "true" },
  { 0, NULL }
};


static int
dissect_rrc_T_discontinuousDpcchTransmission_01(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}


static const value_string rrc_T_slotFormat4_vals[] = {
  {   0, "true" },
  { 0, NULL }
};


static int
dissect_rrc_T_slotFormat4(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}


static const value_string rrc_T_supportForCSVoiceoverHSPA_01_vals[] = {
  {   0, "true" },
  { 0, NULL }
};


static int
dissect_rrc_T_supportForCSVoiceoverHSPA_01(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t PDCP_Capability_v770ext_sequence[] = {
  { &hf_rrc_supportForCSVoiceoverHSPA_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_supportForCSVoiceoverHSPA_01 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_PDCP_Capability_v770ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_PDCP_Capability_v770ext, PDCP_Capability_v770ext_sequence);

  return offset;
}


static const per_sequence_t UE_RadioAccessCapabilityComp2_v770ext_sequence[] = {
  { &hf_rrc_hsdsch_physical_layer_category_ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_HSDSCH_physical_layer_category_ext },
  { &hf_rrc_edch_PhysicalLayerCategory_extension, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_INTEGER_7 },
  { &hf_rrc_hsscchlessHsdschOperation_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_hsscchlessHsdschOperation_01 },
  { &hf_rrc_enhancedFdpch_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_enhancedFdpch_01 },
  { &hf_rrc_hsdschReception_CellFach_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_hsdschReception_CellFach_01 },
  { &hf_rrc_hsdschReception_CellUraPch_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_hsdschReception_CellUraPch_01 },
  { &hf_rrc_discontinuousDpcchTransmission_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_discontinuousDpcchTransmission_01 },
  { &hf_rrc_slotFormat4     , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_slotFormat4 },
  { &hf_rrc_pdcp_Capability_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_PDCP_Capability_v770ext },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UE_RadioAccessCapabilityComp2_v770ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UE_RadioAccessCapabilityComp2_v770ext, UE_RadioAccessCapabilityComp2_v770ext_sequence);

  return offset;
}


static const per_sequence_t InterRATHandoverInfo_v770ext_IEs_sequence[] = {
  { &hf_rrc_ue_RadioAccessCapabilityInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_RadioAccessCapabilityInfo_v770ext },
  { &hf_rrc_ue_RadioAccessCapabilityComp_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_RadioAccessCapabilityComp_v770ext },
  { &hf_rrc_ue_RadioAccessCapabilityComp2_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_RadioAccessCapabilityComp2_v770ext },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_InterRATHandoverInfo_v770ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_InterRATHandoverInfo_v770ext_IEs, InterRATHandoverInfo_v770ext_IEs_sequence);

  return offset;
}


static const value_string rrc_T_supportForEDPCCHPowerBoosting_vals[] = {
  {   0, "true" },
  { 0, NULL }
};


static int
dissect_rrc_T_supportForEDPCCHPowerBoosting(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t InterRATHandoverInfo_v790ext_IEs_sequence[] = {
  { &hf_rrc_supportForEDPCCHPowerBoosting, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_supportForEDPCCHPowerBoosting },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_InterRATHandoverInfo_v790ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_InterRATHandoverInfo_v790ext_IEs, InterRATHandoverInfo_v790ext_IEs_sequence);

  return offset;
}


static const value_string rrc_RadioFrequencyBandFDD2_vals[] = {
  {   0, "bandVIII" },
  {   1, "bandIX" },
  {   2, "bandX" },
  {   3, "bandXI" },
  {   4, "bandXII" },
  {   5, "bandXIII" },
  {   6, "bandXIV" },
  {   7, "bandXV" },
  {   8, "bandXVI" },
  {   9, "bandXVII" },
  {  10, "bandXVIII" },
  {  11, "bandXIX" },
  {  12, "bandXX" },
  {  13, "bandXXI" },
  {  14, "bandXXII" },
  {  15, "extension-indicator" },
  { 0, NULL }
};


static int
dissect_rrc_RadioFrequencyBandFDD2(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     16, NULL, FALSE, 0, NULL);

  return offset;
}



static int
dissect_rrc_RadioFrequencyBandEUTRA(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            1U, 64U, NULL, FALSE);

  return offset;
}


static const per_sequence_t CompressedModeMeasCapabEUTRA_sequence[] = {
  { &hf_rrc_radioFrequencyBandEUTRA, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RadioFrequencyBandEUTRA },
  { &hf_rrc_compressedMode  , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_CompressedModeMeasCapabEUTRA(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_CompressedModeMeasCapabEUTRA, CompressedModeMeasCapabEUTRA_sequence);

  return offset;
}


static const per_sequence_t CompressedModeMeasCapabEUTRAList_sequence_of[1] = {
  { &hf_rrc_CompressedModeMeasCapabEUTRAList_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CompressedModeMeasCapabEUTRA },
};

static int
dissect_rrc_CompressedModeMeasCapabEUTRAList(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_CompressedModeMeasCapabEUTRAList, CompressedModeMeasCapabEUTRAList_sequence_of,
                                                  1, maxFreqBandsEUTRA, FALSE);

  return offset;
}


static const per_sequence_t MeasurementCapabilityExt3_sequence[] = {
  { &hf_rrc_compressedModeMeasCapabEUTRAList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CompressedModeMeasCapabEUTRAList },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_MeasurementCapabilityExt3(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_MeasurementCapabilityExt3, MeasurementCapabilityExt3_sequence);

  return offset;
}


static const per_sequence_t UE_RadioAccessCapabBandFDD3_sequence[] = {
  { &hf_rrc_radioFrequencyBandFDD, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RadioFrequencyBandFDD },
  { &hf_rrc_radioFrequencyBandFDD2, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RadioFrequencyBandFDD2 },
  { &hf_rrc_measurementCapability3, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MeasurementCapabilityExt3 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UE_RadioAccessCapabBandFDD3(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UE_RadioAccessCapabBandFDD3, UE_RadioAccessCapabBandFDD3_sequence);

  return offset;
}


static const per_sequence_t UE_RadioAccessCapabBandFDDList3_sequence_of[1] = {
  { &hf_rrc_UE_RadioAccessCapabBandFDDList3_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UE_RadioAccessCapabBandFDD3 },
};

static int
dissect_rrc_UE_RadioAccessCapabBandFDDList3(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_UE_RadioAccessCapabBandFDDList3, UE_RadioAccessCapabBandFDDList3_sequence_of,
                                                  1, maxFreqBandsFDD, FALSE);

  return offset;
}


static const per_sequence_t UE_RadioAccessCapability_v860ext_sequence[] = {
  { &hf_rrc_ue_RadioAccessCapabBandFDDList3, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UE_RadioAccessCapabBandFDDList3 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UE_RadioAccessCapability_v860ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UE_RadioAccessCapability_v860ext, UE_RadioAccessCapability_v860ext_sequence);

  return offset;
}



static int
dissect_rrc_HSDSCH_physical_layer_category_ext2(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            21U, 24U, NULL, FALSE);

  return offset;
}


static const value_string rrc_T_supportOfMACiis_04_vals[] = {
  {   0, "true" },
  { 0, NULL }
};


static int
dissect_rrc_T_supportOfMACiis_04(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t UE_RadioAccessCapabilityComp2_v860ext_sequence[] = {
  { &hf_rrc_hsdsch_physical_layer_category_ext2, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_HSDSCH_physical_layer_category_ext2 },
  { &hf_rrc_supportOfMACiis_04, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_supportOfMACiis_04 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UE_RadioAccessCapabilityComp2_v860ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UE_RadioAccessCapabilityComp2_v860ext, UE_RadioAccessCapabilityComp2_v860ext_sequence);

  return offset;
}


static const per_sequence_t InterRATHandoverInfo_v860ext_IEs_sequence[] = {
  { &hf_rrc_ue_RadioAccessCapabilityInfo_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_RadioAccessCapability_v860ext },
  { &hf_rrc_ue_RadioAccessCapabilityComp2_03, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_RadioAccessCapabilityComp2_v860ext },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_InterRATHandoverInfo_v860ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_InterRATHandoverInfo_v860ext_IEs, InterRATHandoverInfo_v860ext_IEs_sequence);

  return offset;
}


static const value_string rrc_T_supportForPriorityReselectionInUTRAN_01_vals[] = {
  {   0, "true" },
  { 0, NULL }
};


static int
dissect_rrc_T_supportForPriorityReselectionInUTRAN_01(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t UE_RadioAccessCapability_v880ext_sequence[] = {
  { &hf_rrc_supportForPriorityReselectionInUTRAN_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_supportForPriorityReselectionInUTRAN_01 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UE_RadioAccessCapability_v880ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UE_RadioAccessCapability_v880ext, UE_RadioAccessCapability_v880ext_sequence);

  return offset;
}


static const value_string rrc_RadioFrequencyBandTDDext_vals[] = {
  {   0, "e" },
  {   1, "f" },
  {   2, "g" },
  {   3, "h" },
  {   4, "i" },
  {   5, "j" },
  {   6, "k" },
  {   7, "l" },
  {   8, "m" },
  {   9, "n" },
  {  10, "o" },
  {  11, "p" },
  {  12, "spare4" },
  {  13, "spare3" },
  {  14, "spare2" },
  {  15, "spare1" },
  { 0, NULL }
};


static int
dissect_rrc_RadioFrequencyBandTDDext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     16, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t RadioFrequencyBandTDDextList_sequence_of[1] = {
  { &hf_rrc_RadioFrequencyBandTDDextList_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RadioFrequencyBandTDDext },
};

static int
dissect_rrc_RadioFrequencyBandTDDextList(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_RadioFrequencyBandTDDextList, RadioFrequencyBandTDDextList_sequence_of,
                                                  1, maxFreqBandsTDD_ext, FALSE);

  return offset;
}


static const value_string rrc_T_supportOfSFModeForHSPDSCHDualStream_01_vals[] = {
  {   0, "sf1" },
  {   1, "sf1sf16" },
  { 0, NULL }
};


static int
dissect_rrc_T_supportOfSFModeForHSPDSCHDualStream_01(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     2, NULL, FALSE, 0, NULL);

  return offset;
}


static const value_string rrc_T_supportOfCommonEDCH_04_vals[] = {
  {   0, "true" },
  { 0, NULL }
};


static int
dissect_rrc_T_supportOfCommonEDCH_04(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}


static const value_string rrc_T_supportOfMACiis_05_vals[] = {
  {   0, "true" },
  { 0, NULL }
};


static int
dissect_rrc_T_supportOfMACiis_05(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}


static const value_string rrc_T_supportOfSPSOperation_03_vals[] = {
  {   0, "true" },
  { 0, NULL }
};


static int
dissect_rrc_T_supportOfSPSOperation_03(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}


static const value_string rrc_T_supportOfControlChannelDRXOperation_02_vals[] = {
  {   0, "true" },
  { 0, NULL }
};


static int
dissect_rrc_T_supportOfControlChannelDRXOperation_02(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t UE_RadioAccessCapabilityComp_TDD128_sequence[] = {
  { &hf_rrc_tdd128RF_Capability_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RadioFrequencyBandTDDextList },
  { &hf_rrc_supportOfSFModeForHSPDSCHDualStream_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_supportOfSFModeForHSPDSCHDualStream_01 },
  { &hf_rrc_supportOfCommonEDCH_04, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_supportOfCommonEDCH_04 },
  { &hf_rrc_supportOfMACiis_05, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_supportOfMACiis_05 },
  { &hf_rrc_supportOfSPSOperation_03, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_supportOfSPSOperation_03 },
  { &hf_rrc_supportOfControlChannelDRXOperation_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_supportOfControlChannelDRXOperation_02 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UE_RadioAccessCapabilityComp_TDD128(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UE_RadioAccessCapabilityComp_TDD128, UE_RadioAccessCapabilityComp_TDD128_sequence);

  return offset;
}


static const per_sequence_t InterRATHandoverInfo_v880ext_IEs_sequence[] = {
  { &hf_rrc_ue_RadioAccessCapabilityInfo_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_RadioAccessCapability_v880ext },
  { &hf_rrc_ue_RadioAccessCapabilityInfo_TDD128, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_RadioAccessCapabilityComp_TDD128 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_InterRATHandoverInfo_v880ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_InterRATHandoverInfo_v880ext_IEs, InterRATHandoverInfo_v880ext_IEs_sequence);

  return offset;
}


static const value_string rrc_T_supportOfenhancedTS0_01_vals[] = {
  {   0, "true" },
  { 0, NULL }
};


static int
dissect_rrc_T_supportOfenhancedTS0_01(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t UE_RadioAccessCapability_v920ext_sequence[] = {
  { &hf_rrc_supportOfenhancedTS0_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_supportOfenhancedTS0_01 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UE_RadioAccessCapability_v920ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UE_RadioAccessCapability_v920ext, UE_RadioAccessCapability_v920ext_sequence);

  return offset;
}



static int
dissect_rrc_HSDSCH_physical_layer_category_ext3(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            25U, 28U, NULL, FALSE);

  return offset;
}



static int
dissect_rrc_INTEGER_8_9(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            8U, 9U, NULL, FALSE);

  return offset;
}


static const per_sequence_t UE_RadioAccessCapabilityComp2_v920ext_sequence[] = {
  { &hf_rrc_hsdsch_physical_layer_category_ext3, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_HSDSCH_physical_layer_category_ext3 },
  { &hf_rrc_edch_PhysicalLayerCategory_extension2, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_INTEGER_8_9 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UE_RadioAccessCapabilityComp2_v920ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UE_RadioAccessCapabilityComp2_v920ext, UE_RadioAccessCapabilityComp2_v920ext_sequence);

  return offset;
}


static const per_sequence_t InterRATHandoverInfo_v920ext_IEs_sequence[] = {
  { &hf_rrc_ue_RadioAccessCapabilityInfo_03, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_RadioAccessCapability_v920ext },
  { &hf_rrc_ue_RadioAccessCapabilityComp2_04, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_RadioAccessCapabilityComp2_v920ext },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_InterRATHandoverInfo_v920ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_InterRATHandoverInfo_v920ext_IEs, InterRATHandoverInfo_v920ext_IEs_sequence);

  return offset;
}


static const per_sequence_t IdleIntervalMeasCapabEUTRA_sequence[] = {
  { &hf_rrc_radioFrequencyBandEUTRA, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RadioFrequencyBandEUTRA },
  { &hf_rrc_idleInterval    , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_IdleIntervalMeasCapabEUTRA(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_IdleIntervalMeasCapabEUTRA, IdleIntervalMeasCapabEUTRA_sequence);

  return offset;
}


static const per_sequence_t IdleIntervalMeasCapabEUTRAList_sequence_of[1] = {
  { &hf_rrc_IdleIntervalMeasCapabEUTRAList_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_IdleIntervalMeasCapabEUTRA },
};

static int
dissect_rrc_IdleIntervalMeasCapabEUTRAList(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_IdleIntervalMeasCapabEUTRAList, IdleIntervalMeasCapabEUTRAList_sequence_of,
                                                  1, maxFreqBandsEUTRA, FALSE);

  return offset;
}


static const per_sequence_t MeasurementCapabilityTDD_sequence[] = {
  { &hf_rrc_idleIntervalMeasCapabEUTRAList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_IdleIntervalMeasCapabEUTRAList },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_MeasurementCapabilityTDD(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_MeasurementCapabilityTDD, MeasurementCapabilityTDD_sequence);

  return offset;
}



static int
dissect_rrc_HSDSCH_physical_layer_category_extension(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            1U, 64U, NULL, FALSE);

  return offset;
}


static const per_sequence_t UE_RadioAccessCapabilityInfo_TDD128_v8b0ext_sequence[] = {
  { &hf_rrc_measurementCapabilityTDD, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MeasurementCapabilityTDD },
  { &hf_rrc_hSDSCH_physical_layer_category_extension, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_HSDSCH_physical_layer_category_extension },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UE_RadioAccessCapabilityInfo_TDD128_v8b0ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UE_RadioAccessCapabilityInfo_TDD128_v8b0ext, UE_RadioAccessCapabilityInfo_TDD128_v8b0ext_sequence);

  return offset;
}


static const per_sequence_t InterRATHandoverInfo_v8b0ext_IEs_sequence[] = {
  { &hf_rrc_ue_RadioAccessCapabilityInfo_04, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_RadioAccessCapabilityInfo_TDD128_v8b0ext },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_InterRATHandoverInfo_v8b0ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_InterRATHandoverInfo_v8b0ext_IEs, InterRATHandoverInfo_v8b0ext_IEs_sequence);

  return offset;
}


static const per_sequence_t InterRATHandoverInfo_v950ext_IEs_sequence[] = {
  { &hf_rrc_pdcp_Capability , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_PDCP_Capability_r4_ext },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_InterRATHandoverInfo_v950ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_InterRATHandoverInfo_v950ext_IEs, InterRATHandoverInfo_v950ext_IEs_sequence);

  return offset;
}


static const value_string rrc_RadioFrequencyBandFDD3_vals[] = {
  {   0, "spare64" },
  {   1, "spare63" },
  {   2, "bandXXV" },
  {   3, "bandXXVI" },
  {   4, "spare60" },
  {   5, "spare59" },
  {   6, "spare58" },
  {   7, "spare57" },
  {   8, "spare56" },
  {   9, "spare55" },
  {  10, "spare54" },
  {  11, "spare53" },
  {  12, "spare52" },
  {  13, "spare51" },
  {  14, "spare50" },
  {  15, "spare49" },
  {  16, "spare48" },
  {  17, "spare47" },
  {  18, "spare46" },
  {  19, "spare45" },
  {  20, "spare44" },
  {  21, "spare43" },
  {  22, "spare42" },
  {  23, "spare41" },
  {  24, "spare40" },
  {  25, "spare39" },
  {  26, "spare38" },
  {  27, "spare37" },
  {  28, "spare36" },
  {  29, "spare35" },
  {  30, "spare34" },
  {  31, "spare33" },
  {  32, "spare32" },
  {  33, "spare31" },
  {  34, "spare30" },
  {  35, "spare29" },
  {  36, "spare28" },
  {  37, "spare27" },
  {  38, "spare26" },
  {  39, "spare25" },
  {  40, "spare24" },
  {  41, "spare23" },
  {  42, "spare22" },
  {  43, "spare21" },
  {  44, "spare20" },
  {  45, "spare19" },
  {  46, "spare18" },
  {  47, "spare17" },
  {  48, "spare16" },
  {  49, "spare15" },
  {  50, "spare14" },
  {  51, "spare13" },
  {  52, "spare12" },
  {  53, "spare11" },
  {  54, "spare10" },
  {  55, "spare9" },
  {  56, "spare8" },
  {  57, "spare7" },
  {  58, "spare6" },
  {  59, "spare5" },
  {  60, "spare4" },
  {  61, "spare3" },
  {  62, "spare2" },
  {  63, "extension-indicator" },
  { 0, NULL }
};

static value_string_ext rrc_RadioFrequencyBandFDD3_vals_ext = VALUE_STRING_EXT_INIT(rrc_RadioFrequencyBandFDD3_vals);


static int
dissect_rrc_RadioFrequencyBandFDD3(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     64, NULL, FALSE, 0, NULL);

  return offset;
}


static const value_string rrc_T_additionalSecondaryCells_vals[] = {
  {   0, "a1" },
  {   1, "a2" },
  { 0, NULL }
};


static int
dissect_rrc_T_additionalSecondaryCells(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     2, NULL, FALSE, 0, NULL);

  return offset;
}


static const value_string rrc_T_aggregatedCells_vals[] = {
  {   0, "nc-2c" },
  {   1, "nc-3c" },
  {   2, "nc-4c" },
  { 0, NULL }
};


static int
dissect_rrc_T_aggregatedCells(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     3, NULL, FALSE, 0, NULL);

  return offset;
}


static const value_string rrc_T_gapSize_01_vals[] = {
  {   0, "fiveMHz" },
  {   1, "tenMHz" },
  {   2, "anyGapSize" },
  {   3, "spare5" },
  {   4, "spare4" },
  {   5, "spare3" },
  {   6, "spare2" },
  {   7, "spare1" },
  { 0, NULL }
};


static int
dissect_rrc_T_gapSize_01(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     8, NULL, FALSE, 0, NULL);

  return offset;
}


static const value_string rrc_T_nonContiguousMultiCellCombination22_vals[] = {
  {   0, "true" },
  { 0, NULL }
};


static int
dissect_rrc_T_nonContiguousMultiCellCombination22(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}


static const value_string rrc_T_nonContiguousMultiCellCombination31_13_vals[] = {
  {   0, "true" },
  { 0, NULL }
};


static int
dissect_rrc_T_nonContiguousMultiCellCombination31_13(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t NonContiguousMultiCell_sequence[] = {
  { &hf_rrc_aggregatedCells , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_aggregatedCells },
  { &hf_rrc_gapSize_01      , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_gapSize_01 },
  { &hf_rrc_nonContiguousMultiCellCombination22, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_nonContiguousMultiCellCombination22 },
  { &hf_rrc_nonContiguousMultiCellCombination31_13, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_nonContiguousMultiCellCombination31_13 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_NonContiguousMultiCell(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_NonContiguousMultiCell, NonContiguousMultiCell_sequence);

  return offset;
}


static const per_sequence_t NonContiguousMultiCellCombinationList_sequence_of[1] = {
  { &hf_rrc_NonContiguousMultiCellCombinationList_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_NonContiguousMultiCell },
};

static int
dissect_rrc_NonContiguousMultiCellCombinationList(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_NonContiguousMultiCellCombinationList, NonContiguousMultiCellCombinationList_sequence_of,
                                                  1, maxNonContiguousMultiCellCombinations, FALSE);

  return offset;
}


static const per_sequence_t UE_RadioAccessCapabBandFDD4_sequence[] = {
  { &hf_rrc_radioFrequencyBandFDD, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RadioFrequencyBandFDD },
  { &hf_rrc_radioFrequencyBandFDD2, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RadioFrequencyBandFDD2 },
  { &hf_rrc_radioFrequencyBandFDD3, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RadioFrequencyBandFDD3 },
  { &hf_rrc_additionalSecondaryCells, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_additionalSecondaryCells },
  { &hf_rrc_nonContiguousMultiCellCombinationList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_NonContiguousMultiCellCombinationList },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UE_RadioAccessCapabBandFDD4(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UE_RadioAccessCapabBandFDD4, UE_RadioAccessCapabBandFDD4_sequence);

  return offset;
}


static const per_sequence_t UE_RadioAccessCapabBandFDDList4_sequence_of[1] = {
  { &hf_rrc_UE_RadioAccessCapabBandFDDList4_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UE_RadioAccessCapabBandFDD4 },
};

static int
dissect_rrc_UE_RadioAccessCapabBandFDDList4(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_UE_RadioAccessCapabBandFDDList4, UE_RadioAccessCapabBandFDDList4_sequence_of,
                                                  1, maxFreqBandsFDD, FALSE);

  return offset;
}


static const per_sequence_t UE_RadioAccessCapability_va40ext_sequence[] = {
  { &hf_rrc_ue_RadioAccessCapabBandFDDList4, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UE_RadioAccessCapabBandFDDList4 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UE_RadioAccessCapability_va40ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UE_RadioAccessCapability_va40ext, UE_RadioAccessCapability_va40ext_sequence);

  return offset;
}



static int
dissect_rrc_HSDSCH_physical_layer_category_ext4(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            29U, 30U, NULL, FALSE);

  return offset;
}



static int
dissect_rrc_HSDSCH_physical_layer_category_ext5(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            31U, 32U, NULL, FALSE);

  return offset;
}


static const per_sequence_t UE_RadioAccessCapabilityComp2_va40ext_sequence[] = {
  { &hf_rrc_hsdsch_physical_layer_category_ext4, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_HSDSCH_physical_layer_category_ext4 },
  { &hf_rrc_hsdsch_physical_layer_category_ext5, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_HSDSCH_physical_layer_category_ext5 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UE_RadioAccessCapabilityComp2_va40ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UE_RadioAccessCapabilityComp2_va40ext, UE_RadioAccessCapabilityComp2_va40ext_sequence);

  return offset;
}


static const per_sequence_t InterRATHandoverInfo_va40ext_IEs_sequence[] = {
  { &hf_rrc_ue_RadioAccessCapabilityInfo_05, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_RadioAccessCapability_va40ext },
  { &hf_rrc_ue_RadioAccessCapabilityComp2_05, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_RadioAccessCapabilityComp2_va40ext },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_InterRATHandoverInfo_va40ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_InterRATHandoverInfo_va40ext_IEs, InterRATHandoverInfo_va40ext_IEs_sequence);

  return offset;
}


static const value_string rrc_T_supportofULOLTD_vals[] = {
  {   0, "true" },
  { 0, NULL }
};


static int
dissect_rrc_T_supportofULOLTD(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t UE_RadioAccessCapabBandFDD4_va60ext_sequence[] = {
  { &hf_rrc_supportofULOLTD , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_supportofULOLTD },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UE_RadioAccessCapabBandFDD4_va60ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UE_RadioAccessCapabBandFDD4_va60ext, UE_RadioAccessCapabBandFDD4_va60ext_sequence);

  return offset;
}


static const per_sequence_t UE_RadioAccessCapabBandFDDList4_va60ext_sequence_of[1] = {
  { &hf_rrc_UE_RadioAccessCapabBandFDDList4_va60ext_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UE_RadioAccessCapabBandFDD4_va60ext },
};

static int
dissect_rrc_UE_RadioAccessCapabBandFDDList4_va60ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_UE_RadioAccessCapabBandFDDList4_va60ext, UE_RadioAccessCapabBandFDDList4_va60ext_sequence_of,
                                                  1, maxFreqBandsFDD, FALSE);

  return offset;
}


static const value_string rrc_T_additionalSecondaryCells_01_vals[] = {
  {   0, "a1" },
  {   1, "a2" },
  { 0, NULL }
};


static int
dissect_rrc_T_additionalSecondaryCells_01(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     2, NULL, FALSE, 0, NULL);

  return offset;
}


static const value_string rrc_T_supportofULOLTD_01_vals[] = {
  {   0, "true" },
  { 0, NULL }
};


static int
dissect_rrc_T_supportofULOLTD_01(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t UE_RadioAccessCapabBandFDD4_va80ext_sequence[] = {
  { &hf_rrc_radioFrequencyBandFDD, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RadioFrequencyBandFDD },
  { &hf_rrc_radioFrequencyBandFDD2, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RadioFrequencyBandFDD2 },
  { &hf_rrc_radioFrequencyBandFDD3, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RadioFrequencyBandFDD3 },
  { &hf_rrc_additionalSecondaryCells_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_additionalSecondaryCells_01 },
  { &hf_rrc_nonContiguousMultiCellCombinationList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_NonContiguousMultiCellCombinationList },
  { &hf_rrc_supportofULOLTD_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_supportofULOLTD_01 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UE_RadioAccessCapabBandFDD4_va80ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UE_RadioAccessCapabBandFDD4_va80ext, UE_RadioAccessCapabBandFDD4_va80ext_sequence);

  return offset;
}


static const per_sequence_t UE_RadioAccessCapabBandFDDList4_va80ext_sequence_of[1] = {
  { &hf_rrc_UE_RadioAccessCapabBandFDDList4_va80ext_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UE_RadioAccessCapabBandFDD4_va80ext },
};

static int
dissect_rrc_UE_RadioAccessCapabBandFDDList4_va80ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_UE_RadioAccessCapabBandFDDList4_va80ext, UE_RadioAccessCapabBandFDDList4_va80ext_sequence_of,
                                                  1, maxFreqBandsFDD_ext3, FALSE);

  return offset;
}


static const per_sequence_t UE_RadioAccessCapability_va80ext_sequence[] = {
  { &hf_rrc_ue_RadioAccessCapabBandFDDList4_va60ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_RadioAccessCapabBandFDDList4_va60ext },
  { &hf_rrc_ue_RadioAccessCapabBandFDDList4_va80ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_RadioAccessCapabBandFDDList4_va80ext },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UE_RadioAccessCapability_va80ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UE_RadioAccessCapability_va80ext, UE_RadioAccessCapability_va80ext_sequence);

  return offset;
}


static const per_sequence_t InterRATHandoverInfo_va80ext_IEs_sequence[] = {
  { &hf_rrc_ue_RadioAccessCapabilityInfo_va80ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_RadioAccessCapability_va80ext },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_InterRATHandoverInfo_va80ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_InterRATHandoverInfo_va80ext_IEs, InterRATHandoverInfo_va80ext_IEs_sequence);

  return offset;
}


static const value_string rrc_TotalRLC_AM_BufferSize_vbxyext_vals[] = {
  {   0, "kb3400" },
  {   1, "kb3500" },
  {   2, "kb4400" },
  {   3, "kb4500" },
  {   4, "spare4" },
  {   5, "spare3" },
  {   6, "spare2" },
  {   7, "spare1" },
  { 0, NULL }
};


static int
dissect_rrc_TotalRLC_AM_BufferSize_vbxyext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     8, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t RLC_Capability_vbxyext_sequence[] = {
  { &hf_rrc_totalRLC_AM_BufferSize_04, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_TotalRLC_AM_BufferSize_vbxyext },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_RLC_Capability_vbxyext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_RLC_Capability_vbxyext, RLC_Capability_vbxyext_sequence);

  return offset;
}


static const value_string rrc_T_additionalSecondaryCells2_vals[] = {
  {   0, "a3" },
  {   1, "a4" },
  {   2, "a5" },
  {   3, "a6" },
  { 0, NULL }
};


static int
dissect_rrc_T_additionalSecondaryCells2(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     4, NULL, FALSE, 0, NULL);

  return offset;
}


static const value_string rrc_T_supportOfULCLTD_vals[] = {
  {   0, "true" },
  { 0, NULL }
};


static int
dissect_rrc_T_supportOfULCLTD(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}



static int
dissect_rrc_INTEGER_2_8(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            2U, 8U, NULL, FALSE);

  return offset;
}



static int
dissect_rrc_INTEGER_1_4(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            1U, 4U, NULL, FALSE);

  return offset;
}


static const value_string rrc_MultiflowMIMOCapability_vals[] = {
  {   0, "single-stream" },
  {   1, "dual-stream" },
  { 0, NULL }
};


static int
dissect_rrc_MultiflowMIMOCapability(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     2, NULL, FALSE, 0, NULL);

  return offset;
}


static const value_string rrc_T_gapSize_vals[] = {
  {   0, "fiveMHz" },
  {   1, "tenMHz" },
  {   2, "anyGapSize" },
  {   3, "spare5" },
  {   4, "spare4" },
  {   5, "spare3" },
  {   6, "spare2" },
  {   7, "spare1" },
  { 0, NULL }
};


static int
dissect_rrc_T_gapSize(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     8, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t MultiflowNonContiguousCapability_sequence[] = {
  { &hf_rrc_gapSize         , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_gapSize },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_MultiflowNonContiguousCapability(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_MultiflowNonContiguousCapability, MultiflowNonContiguousCapability_sequence);

  return offset;
}


static const per_sequence_t MultiflowCapabilityBand_sequence[] = {
  { &hf_rrc_numberOfCells   , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_2_8 },
  { &hf_rrc_numberOfFrequencies, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_1_4 },
  { &hf_rrc_mimo            , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MultiflowMIMOCapability },
  { &hf_rrc_nonContiguousCapability, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MultiflowNonContiguousCapability },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_MultiflowCapabilityBand(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_MultiflowCapabilityBand, MultiflowCapabilityBand_sequence);

  return offset;
}


static const value_string rrc_T_mimo4x4CapabilityBand_vals[] = {
  {   0, "true" },
  { 0, NULL }
};


static int
dissect_rrc_T_mimo4x4CapabilityBand(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}


static const value_string rrc_T_supportOfULMIMO_vals[] = {
  {   0, "true" },
  { 0, NULL }
};


static int
dissect_rrc_T_supportOfULMIMO(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t UE_RadioAccessCapabBandFDD6_sequence[] = {
  { &hf_rrc_radioFrequencyBandFDD, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RadioFrequencyBandFDD },
  { &hf_rrc_radioFrequencyBandFDD2, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RadioFrequencyBandFDD2 },
  { &hf_rrc_radioFrequencyBandFDD3, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RadioFrequencyBandFDD3 },
  { &hf_rrc_additionalSecondaryCells2, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_additionalSecondaryCells2 },
  { &hf_rrc_supportOfULCLTD , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_supportOfULCLTD },
  { &hf_rrc_multiflowCapabilityBand, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MultiflowCapabilityBand },
  { &hf_rrc_mimo4x4CapabilityBand, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_mimo4x4CapabilityBand },
  { &hf_rrc_supportOfULMIMO , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_supportOfULMIMO },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UE_RadioAccessCapabBandFDD6(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UE_RadioAccessCapabBandFDD6, UE_RadioAccessCapabBandFDD6_sequence);

  return offset;
}


static const per_sequence_t UE_RadioAccessCapabBandFDDList6_sequence_of[1] = {
  { &hf_rrc_UE_RadioAccessCapabBandFDDList6_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UE_RadioAccessCapabBandFDD6 },
};

static int
dissect_rrc_UE_RadioAccessCapabBandFDDList6(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_UE_RadioAccessCapabBandFDDList6, UE_RadioAccessCapabBandFDDList6_sequence_of,
                                                  1, maxFreqBandsFDD3, FALSE);

  return offset;
}


static const per_sequence_t UE_RadioAccessCapability_vbxyext_sequence[] = {
  { &hf_rrc_rlc_Capability_03, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RLC_Capability_vbxyext },
  { &hf_rrc_ue_RadioAccessCapabBandFDDList6, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_RadioAccessCapabBandFDDList6 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UE_RadioAccessCapability_vbxyext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UE_RadioAccessCapability_vbxyext, UE_RadioAccessCapability_vbxyext_sequence);

  return offset;
}



static int
dissect_rrc_HSDSCH_physical_layer_category_ext6(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            33U, 34U, NULL, FALSE);

  return offset;
}



static int
dissect_rrc_HSDSCH_physical_layer_category_ext7(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            35U, 36U, NULL, FALSE);

  return offset;
}



static int
dissect_rrc_HSDSCH_physical_layer_category_ext8(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            37U, 38U, NULL, FALSE);

  return offset;
}



static int
dissect_rrc_INTEGER_10_12(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            10U, 12U, NULL, FALSE);

  return offset;
}


static const per_sequence_t UE_RadioAccessCapabilityComp2_vbxyext_sequence[] = {
  { &hf_rrc_hsdsch_physical_layer_category_ext6, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_HSDSCH_physical_layer_category_ext6 },
  { &hf_rrc_hsdsch_physical_layer_category_ext7, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_HSDSCH_physical_layer_category_ext7 },
  { &hf_rrc_hsdsch_physical_layer_category_ext8, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_HSDSCH_physical_layer_category_ext8 },
  { &hf_rrc_edch_PhysicalLayerCategory_extension3, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_INTEGER_10_12 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UE_RadioAccessCapabilityComp2_vbxyext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UE_RadioAccessCapabilityComp2_vbxyext, UE_RadioAccessCapabilityComp2_vbxyext_sequence);

  return offset;
}


static const per_sequence_t InterRATHandoverInfo_vbxyext_IEs_sequence[] = {
  { &hf_rrc_ue_RadioAccessCapabilityInfo_vbxyext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_RadioAccessCapability_vbxyext },
  { &hf_rrc_ue_RadioAccessCapabilityComp2_06, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_RadioAccessCapabilityComp2_vbxyext },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_InterRATHandoverInfo_vbxyext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_InterRATHandoverInfo_vbxyext_IEs, InterRATHandoverInfo_vbxyext_IEs_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions_57_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_nonCriticalExtensions_57(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_nonCriticalExtensions_57, T_nonCriticalExtensions_57_sequence);

  return offset;
}


static const per_sequence_t T_vbxyNonCriticalExtensions_06_sequence[] = {
  { &hf_rrc_interRATHandoverInfo_vbxyext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_InterRATHandoverInfo_vbxyext_IEs },
  { &hf_rrc_nonCriticalExtensions_57, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_nonCriticalExtensions_57 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_vbxyNonCriticalExtensions_06(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_vbxyNonCriticalExtensions_06, T_vbxyNonCriticalExtensions_06_sequence);

  return offset;
}


static const per_sequence_t T_va80NonCriticalExtensions_sequence[] = {
  { &hf_rrc_interRATHandoverInfo_va80ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_InterRATHandoverInfo_va80ext_IEs },
  { &hf_rrc_vbxyNonCriticalExtensions_06, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_vbxyNonCriticalExtensions_06 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_va80NonCriticalExtensions(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_va80NonCriticalExtensions, T_va80NonCriticalExtensions_sequence);

  return offset;
}


static const per_sequence_t T_va40NonCriticalExtensions_03_sequence[] = {
  { &hf_rrc_interRATHandoverInfo_va40ext_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_InterRATHandoverInfo_va40ext_IEs },
  { &hf_rrc_va80NonCriticalExtensions, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_va80NonCriticalExtensions },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_va40NonCriticalExtensions_03(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_va40NonCriticalExtensions_03, T_va40NonCriticalExtensions_03_sequence);

  return offset;
}


static const per_sequence_t T_v950NonCriticalExtensions_02_sequence[] = {
  { &hf_rrc_interRATHandoverInfo_v950ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_InterRATHandoverInfo_v950ext_IEs },
  { &hf_rrc_va40NonCriticalExtensions_03, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_va40NonCriticalExtensions_03 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v950NonCriticalExtensions_02(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v950NonCriticalExtensions_02, T_v950NonCriticalExtensions_02_sequence);

  return offset;
}


static const per_sequence_t T_v8b0NonCriticalExtensions_sequence[] = {
  { &hf_rrc_interRATHandoverInfo_v8b0ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_InterRATHandoverInfo_v8b0ext_IEs },
  { &hf_rrc_v950NonCriticalExtensions_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v950NonCriticalExtensions_02 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v8b0NonCriticalExtensions(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v8b0NonCriticalExtensions, T_v8b0NonCriticalExtensions_sequence);

  return offset;
}


static const per_sequence_t T_v920NonCriticalExtensions_sequence[] = {
  { &hf_rrc_interRATHandoverInfo_v920ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_InterRATHandoverInfo_v920ext_IEs },
  { &hf_rrc_v8b0NonCriticalExtensions, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v8b0NonCriticalExtensions },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v920NonCriticalExtensions(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v920NonCriticalExtensions, T_v920NonCriticalExtensions_sequence);

  return offset;
}


static const per_sequence_t T_v880NonCriticalExtensions_sequence[] = {
  { &hf_rrc_interRATHandoverInfo_v880ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_InterRATHandoverInfo_v880ext_IEs },
  { &hf_rrc_v920NonCriticalExtensions, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v920NonCriticalExtensions },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v880NonCriticalExtensions(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v880NonCriticalExtensions, T_v880NonCriticalExtensions_sequence);

  return offset;
}


static const per_sequence_t T_v860NonCriticalExtensions_07_sequence[] = {
  { &hf_rrc_interRATHandoverInfo_v860ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_InterRATHandoverInfo_v860ext_IEs },
  { &hf_rrc_v880NonCriticalExtensions, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v880NonCriticalExtensions },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v860NonCriticalExtensions_07(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v860NonCriticalExtensions_07, T_v860NonCriticalExtensions_07_sequence);

  return offset;
}


static const per_sequence_t T_v790nonCriticalExtensions_sequence[] = {
  { &hf_rrc_interRATHandoverInfo_v790ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_InterRATHandoverInfo_v790ext_IEs },
  { &hf_rrc_v860NonCriticalExtensions_07, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v860NonCriticalExtensions_07 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v790nonCriticalExtensions(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v790nonCriticalExtensions, T_v790nonCriticalExtensions_sequence);

  return offset;
}


static const per_sequence_t T_v770NonCriticalExtensions_02_sequence[] = {
  { &hf_rrc_interRATHandoverInfo_v770ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_InterRATHandoverInfo_v770ext_IEs },
  { &hf_rrc_v790nonCriticalExtensions, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v790nonCriticalExtensions },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v770NonCriticalExtensions_02(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v770NonCriticalExtensions_02, T_v770NonCriticalExtensions_02_sequence);

  return offset;
}


static const per_sequence_t T_v6e0NonCriticalExtensions_sequence[] = {
  { &hf_rrc_interRATHandoverInfo_v6e0ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_InterRATHandoverInfo_v6e0ext_IEs },
  { &hf_rrc_v770NonCriticalExtensions_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v770NonCriticalExtensions_02 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v6e0NonCriticalExtensions(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v6e0NonCriticalExtensions, T_v6e0NonCriticalExtensions_sequence);

  return offset;
}


static const per_sequence_t InterRATHandoverInfo_NonCriticalExtension_v6b0_IEs_sequence[] = {
  { &hf_rrc_interRATHandoverInfo_v6b0ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_InterRATHandoverInfo_v6b0ext_IEs },
  { &hf_rrc_v6e0NonCriticalExtensions, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v6e0NonCriticalExtensions },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_InterRATHandoverInfo_NonCriticalExtension_v6b0_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_InterRATHandoverInfo_NonCriticalExtension_v6b0_IEs, InterRATHandoverInfo_NonCriticalExtension_v6b0_IEs_sequence);

  return offset;
}


static const per_sequence_t T_v690NonCriticalExtensions_10_sequence[] = {
  { &hf_rrc_interRATHandoverInfo_v690ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_InterRATHandoverInfo_v690ext_IEs },
  { &hf_rrc_v6b0NonCriticalExtensions_05, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_InterRATHandoverInfo_NonCriticalExtension_v6b0_IEs },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v690NonCriticalExtensions_10(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v690NonCriticalExtensions_10, T_v690NonCriticalExtensions_10_sequence);

  return offset;
}


static const per_sequence_t T_v590NonCriticalExtensions_07_sequence[] = {
  { &hf_rrc_interRATHandoverInfo_v590ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_InterRATHandoverInfo_v590ext_IEs },
  { &hf_rrc_v690NonCriticalExtensions_10, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v690NonCriticalExtensions_10 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v590NonCriticalExtensions_07(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v590NonCriticalExtensions_07, T_v590NonCriticalExtensions_07_sequence);

  return offset;
}


static const per_sequence_t T_v4d0NonCriticalExtensions_02_sequence[] = {
  { &hf_rrc_interRATHandoverInfo_v4d0ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_InterRATHandoverInfo_v4d0ext_IEs },
  { &hf_rrc_v590NonCriticalExtensions_07, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v590NonCriticalExtensions_07 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v4d0NonCriticalExtensions_02(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v4d0NonCriticalExtensions_02, T_v4d0NonCriticalExtensions_02_sequence);

  return offset;
}


static const per_sequence_t T_v4b0NonCriticalExtensions_04_sequence[] = {
  { &hf_rrc_interRATHandoverInfo_v4b0ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_InterRATHandoverInfo_v4b0ext_IEs },
  { &hf_rrc_v4d0NonCriticalExtensions_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v4d0NonCriticalExtensions_02 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v4b0NonCriticalExtensions_04(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v4b0NonCriticalExtensions_04, T_v4b0NonCriticalExtensions_04_sequence);

  return offset;
}


static const per_sequence_t T_v3g0NonCriticalExtensions_sequence[] = {
  { &hf_rrc_interRATHandoverInfo_v3g0ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_InterRATHandoverInfo_v3g0ext_IEs },
  { &hf_rrc_v4b0NonCriticalExtensions_04, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v4b0NonCriticalExtensions_04 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v3g0NonCriticalExtensions(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v3g0NonCriticalExtensions, T_v3g0NonCriticalExtensions_sequence);

  return offset;
}


static const per_sequence_t T_laterNonCriticalExtensions_17_sequence[] = {
  { &hf_rrc_interRATHandoverInfo_v3d0ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_InterRATHandoverInfo_v3d0ext_IEs },
  { &hf_rrc_interRATHandoverInfo_r3_add_ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_interRATHandoverInfo_r3_add_ext },
  { &hf_rrc_v3g0NonCriticalExtensions, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v3g0NonCriticalExtensions },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_laterNonCriticalExtensions_17(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_laterNonCriticalExtensions_17, T_laterNonCriticalExtensions_17_sequence);

  return offset;
}


static const per_sequence_t T_v3a0NonCriticalExtensions_03_sequence[] = {
  { &hf_rrc_interRATHandoverInfo_v3a0ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_InterRATHandoverInfo_v3a0ext_IEs },
  { &hf_rrc_laterNonCriticalExtensions_17, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_laterNonCriticalExtensions_17 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v3a0NonCriticalExtensions_03(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v3a0NonCriticalExtensions_03, T_v3a0NonCriticalExtensions_03_sequence);

  return offset;
}


static const per_sequence_t T_present_01_sequence[] = {
  { &hf_rrc_interRATHandoverInfo_v390ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_InterRATHandoverInfo_v390ext_IEs },
  { &hf_rrc_v3a0NonCriticalExtensions_03, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v3a0NonCriticalExtensions_03 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_present_01(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_present_01, T_present_01_sequence);

  return offset;
}


static const value_string rrc_T_v390NonCriticalExtensions_vals[] = {
  {   0, "absent" },
  {   1, "present" },
  { 0, NULL }
};

static const per_choice_t T_v390NonCriticalExtensions_choice[] = {
  {   0, &hf_rrc_absent          , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   1, &hf_rrc_present_03      , ASN1_NO_EXTENSIONS     , dissect_rrc_T_present_01 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_v390NonCriticalExtensions(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_v390NonCriticalExtensions, T_v390NonCriticalExtensions_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t InterRATHandoverInfo_sequence[] = {
  { &hf_rrc_predefinedConfigStatusList, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_predefinedConfigStatusList },
  { &hf_rrc_uE_SecurityInformation, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_uE_SecurityInformation },
  { &hf_rrc_ue_CapabilityContainer, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_ue_CapabilityContainer },
  { &hf_rrc_v390NonCriticalExtensions, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_v390NonCriticalExtensions },
  { NULL, 0, 0, NULL }
};

int
dissect_rrc_InterRATHandoverInfo(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_InterRATHandoverInfo, InterRATHandoverInfo_sequence);

  return offset;
}


static const per_sequence_t T_fddRF_Capability_01_sequence[] = {
  { &hf_rrc_ue_PowerClass_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UE_PowerClassExt },
  { &hf_rrc_txRxFrequencySeparation, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TxRxFrequencySeparation },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_fddRF_Capability_01(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_fddRF_Capability_01, T_fddRF_Capability_01_sequence);

  return offset;
}


static const per_sequence_t CompressedModeMeasCapabFDD2_sequence[] = {
  { &hf_rrc_radioFrequencyBandFDD, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RadioFrequencyBandFDD },
  { &hf_rrc_radioFrequencyBandFDD2, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RadioFrequencyBandFDD2 },
  { &hf_rrc_dl_MeasurementsFDD, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { &hf_rrc_ul_MeasurementsFDD, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_CompressedModeMeasCapabFDD2(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_CompressedModeMeasCapabFDD2, CompressedModeMeasCapabFDD2_sequence);

  return offset;
}


static const per_sequence_t CompressedModeMeasCapabFDDList2_sequence_of[1] = {
  { &hf_rrc_CompressedModeMeasCapabFDDList2_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CompressedModeMeasCapabFDD2 },
};

static int
dissect_rrc_CompressedModeMeasCapabFDDList2(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_CompressedModeMeasCapabFDDList2, CompressedModeMeasCapabFDDList2_sequence_of,
                                                  1, maxFreqBandsFDD, FALSE);

  return offset;
}


static const per_sequence_t MeasurementCapabilityExt2_sequence[] = {
  { &hf_rrc_compressedModeMeasCapabFDDList_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CompressedModeMeasCapabFDDList2 },
  { &hf_rrc_compressedModeMeasCapabTDDList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CompressedModeMeasCapabTDDList },
  { &hf_rrc_compressedModeMeasCapabGSMList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CompressedModeMeasCapabGSMList },
  { &hf_rrc_compressedModeMeasCapabMC, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CompressedModeMeasCapabMC },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_MeasurementCapabilityExt2(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_MeasurementCapabilityExt2, MeasurementCapabilityExt2_sequence);

  return offset;
}


static const per_sequence_t UE_RadioAccessCapabBandFDD2_sequence[] = {
  { &hf_rrc_radioFrequencyBandFDD2, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RadioFrequencyBandFDD2 },
  { &hf_rrc_fddRF_Capability_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_fddRF_Capability_01 },
  { &hf_rrc_measurementCapability2, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MeasurementCapabilityExt2 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UE_RadioAccessCapabBandFDD2(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UE_RadioAccessCapabBandFDD2, UE_RadioAccessCapabBandFDD2_sequence);

  return offset;
}


static const per_sequence_t UE_RadioAccessCapabBandFDDList2_sequence_of[1] = {
  { &hf_rrc_UE_RadioAccessCapabBandFDDList2_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UE_RadioAccessCapabBandFDD2 },
};

static int
dissect_rrc_UE_RadioAccessCapabBandFDDList2(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_UE_RadioAccessCapabBandFDDList2, UE_RadioAccessCapabBandFDDList2_sequence_of,
                                                  1, maxFreqBandsFDD, FALSE);

  return offset;
}


static const per_sequence_t CompressedModeMeasCapabFDD_ext_sequence[] = {
  { &hf_rrc_radioFrequencyBandFDD2, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RadioFrequencyBandFDD2 },
  { &hf_rrc_dl_MeasurementsFDD, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { &hf_rrc_ul_MeasurementsFDD, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_CompressedModeMeasCapabFDD_ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_CompressedModeMeasCapabFDD_ext, CompressedModeMeasCapabFDD_ext_sequence);

  return offset;
}


static const per_sequence_t CompressedModeMeasCapabFDDList_ext_sequence_of[1] = {
  { &hf_rrc_CompressedModeMeasCapabFDDList_ext_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CompressedModeMeasCapabFDD_ext },
};

static int
dissect_rrc_CompressedModeMeasCapabFDDList_ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_CompressedModeMeasCapabFDDList_ext, CompressedModeMeasCapabFDDList_ext_sequence_of,
                                                  1, maxFreqBandsFDD, FALSE);

  return offset;
}


static const per_sequence_t UE_RadioAccessCapabBandFDD_ext_sequence[] = {
  { &hf_rrc_radioFrequencyBandFDD, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RadioFrequencyBandFDD },
  { &hf_rrc_compressedModeMeasCapabFDDList_ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CompressedModeMeasCapabFDDList_ext },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UE_RadioAccessCapabBandFDD_ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UE_RadioAccessCapabBandFDD_ext, UE_RadioAccessCapabBandFDD_ext_sequence);

  return offset;
}


static const per_sequence_t UE_RadioAccessCapabBandFDDList_ext_sequence_of[1] = {
  { &hf_rrc_UE_RadioAccessCapabBandFDDList_ext_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UE_RadioAccessCapabBandFDD_ext },
};

static int
dissect_rrc_UE_RadioAccessCapabBandFDDList_ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_UE_RadioAccessCapabBandFDDList_ext, UE_RadioAccessCapabBandFDDList_ext_sequence_of,
                                                  1, maxFreqBandsFDD, FALSE);

  return offset;
}


static const per_sequence_t UE_RadioAccessCapability_v650ext_sequence[] = {
  { &hf_rrc_ue_RadioAccessCapabBandFDDList2, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UE_RadioAccessCapabBandFDDList2 },
  { &hf_rrc_ue_RadioAccessCapabBandFDDList_ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_RadioAccessCapabBandFDDList_ext },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UE_RadioAccessCapability_v650ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UE_RadioAccessCapability_v650ext, UE_RadioAccessCapability_v650ext_sequence);

  return offset;
}


static const per_sequence_t InterRATHandoverInfo_v690ext1_IEs_sequence[] = {
  { &hf_rrc_ue_RadioAccessCapability_v650ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_RadioAccessCapability_v650ext },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_InterRATHandoverInfo_v690ext1_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_InterRATHandoverInfo_v690ext1_IEs, InterRATHandoverInfo_v690ext1_IEs_sequence);

  return offset;
}


static const value_string rrc_T_supportForTwoDRXSchemesInPCH_02_vals[] = {
  {   0, "true" },
  { 0, NULL }
};


static int
dissect_rrc_T_supportForTwoDRXSchemesInPCH_02(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}


static const value_string rrc_T_supportEDPDCHPowerInterpolation_vals[] = {
  {   0, "true" },
  { 0, NULL }
};


static int
dissect_rrc_T_supportEDPDCHPowerInterpolation(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t UE_RadioAccessCapability_v7e0ext_sequence[] = {
  { &hf_rrc_supportForTwoDRXSchemesInPCH_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_supportForTwoDRXSchemesInPCH_02 },
  { &hf_rrc_supportEDPDCHPowerInterpolation, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_supportEDPDCHPowerInterpolation },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UE_RadioAccessCapability_v7e0ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UE_RadioAccessCapability_v7e0ext, UE_RadioAccessCapability_v7e0ext_sequence);

  return offset;
}


static const per_sequence_t InterRATHandoverInfo_v7e0ext_IEs_sequence[] = {
  { &hf_rrc_ue_RadioAccessCapability, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_RadioAccessCapability_v7e0ext },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_InterRATHandoverInfo_v7e0ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_InterRATHandoverInfo_v7e0ext_IEs, InterRATHandoverInfo_v7e0ext_IEs_sequence);

  return offset;
}


static const value_string rrc_T_supportofTxDivOnNonMIMOChannel_vals[] = {
  {   0, "true" },
  { 0, NULL }
};


static int
dissect_rrc_T_supportofTxDivOnNonMIMOChannel(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}



static int
dissect_rrc_MultiCarrier_HSDSCH_physical_layer_category(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            1U, 64U, NULL, FALSE);

  return offset;
}



static int
dissect_rrc_INTEGER_1_6(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            1U, 6U, NULL, FALSE);

  return offset;
}


static const value_string rrc_T_supportForCSVoiceoverHSPA_02_vals[] = {
  {   0, "true" },
  { 0, NULL }
};


static int
dissect_rrc_T_supportForCSVoiceoverHSPA_02(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t UE_RadioAccessCapabilityComp_TDD128_v7f0ext_sequence[] = {
  { &hf_rrc_hSDSCH_physical_layer_category, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_HSDSCH_physical_layer_category },
  { &hf_rrc_multiCarrier_physical_layer_category, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MultiCarrier_HSDSCH_physical_layer_category },
  { &hf_rrc_tdd_edch_PhysicalLayerCategory, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_INTEGER_1_6 },
  { &hf_rrc_dl_CapabilityWithSimultaneousHS_DSCHConfig, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_CapabilityWithSimultaneousHS_DSCHConfig },
  { &hf_rrc_supportForCSVoiceoverHSPA_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_supportForCSVoiceoverHSPA_02 },
  { &hf_rrc_totalRLC_AM_BufferSize_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_TotalRLC_AM_BufferSize_r5_ext },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UE_RadioAccessCapabilityComp_TDD128_v7f0ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UE_RadioAccessCapabilityComp_TDD128_v7f0ext, UE_RadioAccessCapabilityComp_TDD128_v7f0ext_sequence);

  return offset;
}


static const per_sequence_t UE_RadioAccessCapability_v7f0ext_sequence[] = {
  { &hf_rrc_supportofTxDivOnNonMIMOChannel, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_supportofTxDivOnNonMIMOChannel },
  { &hf_rrc_physicalChannelCapability_LCR, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_PhysicalChannelCapability_LCR_r4 },
  { &hf_rrc_ue_RadioAccessCapabilityComp_TDD128, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_RadioAccessCapabilityComp_TDD128_v7f0ext },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UE_RadioAccessCapability_v7f0ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UE_RadioAccessCapability_v7f0ext, UE_RadioAccessCapability_v7f0ext_sequence);

  return offset;
}


static const value_string rrc_T_supportofTxDivOnNonMIMOChannel_01_vals[] = {
  {   0, "true" },
  { 0, NULL }
};


static int
dissect_rrc_T_supportofTxDivOnNonMIMOChannel_01(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t UE_RadioAccessCapabilityComp2_v7f0ext_sequence[] = {
  { &hf_rrc_supportofTxDivOnNonMIMOChannel_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_supportofTxDivOnNonMIMOChannel_01 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UE_RadioAccessCapabilityComp2_v7f0ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UE_RadioAccessCapabilityComp2_v7f0ext, UE_RadioAccessCapabilityComp2_v7f0ext_sequence);

  return offset;
}


static const per_sequence_t InterRATHandoverInfo_v7f0ext_IEs_sequence[] = {
  { &hf_rrc_ue_RadioAccessCapability_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_RadioAccessCapability_v7f0ext },
  { &hf_rrc_ue_RadioAccessCapabilityComp2_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_RadioAccessCapabilityComp2_v7f0ext },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_InterRATHandoverInfo_v7f0ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_InterRATHandoverInfo_v7f0ext_IEs, InterRATHandoverInfo_v7f0ext_IEs_sequence);

  return offset;
}


static const per_sequence_t T_fddRF_Capability_02_sequence[] = {
  { &hf_rrc_ue_PowerClass_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UE_PowerClassExt },
  { &hf_rrc_txRxFrequencySeparation, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TxRxFrequencySeparation },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_fddRF_Capability_02(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_fddRF_Capability_02, T_fddRF_Capability_02_sequence);

  return offset;
}


static const per_sequence_t CompressedModeMeasCapabFDD3_sequence[] = {
  { &hf_rrc_radioFrequencyBandFDD3, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RadioFrequencyBandFDD3 },
  { &hf_rrc_dl_MeasurementsFDD, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { &hf_rrc_ul_MeasurementsFDD, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_CompressedModeMeasCapabFDD3(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_CompressedModeMeasCapabFDD3, CompressedModeMeasCapabFDD3_sequence);

  return offset;
}


static const per_sequence_t CompressedModeMeasCapabFDDList3_sequence_of[1] = {
  { &hf_rrc_CompressedModeMeasCapabFDDList3_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CompressedModeMeasCapabFDD3 },
};

static int
dissect_rrc_CompressedModeMeasCapabFDDList3(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_CompressedModeMeasCapabFDDList3, CompressedModeMeasCapabFDDList3_sequence_of,
                                                  1, maxFreqBandsFDD_ext2, FALSE);

  return offset;
}


static const per_sequence_t MeasurementCapabilityExt4_sequence[] = {
  { &hf_rrc_compressedModeMeasCapabFDDList2, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CompressedModeMeasCapabFDDList2 },
  { &hf_rrc_compressedModeMeasCapabFDDList3, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CompressedModeMeasCapabFDDList3 },
  { &hf_rrc_compressedModeMeasCapabTDDList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CompressedModeMeasCapabTDDList },
  { &hf_rrc_compressedModeMeasCapabGSMList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CompressedModeMeasCapabGSMList },
  { &hf_rrc_compressedModeMeasCapabMC, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CompressedModeMeasCapabMC },
  { &hf_rrc_compressedModeMeasCapabEUTRAList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CompressedModeMeasCapabEUTRAList },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_MeasurementCapabilityExt4(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_MeasurementCapabilityExt4, MeasurementCapabilityExt4_sequence);

  return offset;
}


static const per_sequence_t UE_RadioAccessCapabBandFDD5_sequence[] = {
  { &hf_rrc_radioFrequencyBandFDD3, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RadioFrequencyBandFDD3 },
  { &hf_rrc_fddRF_Capability_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_fddRF_Capability_02 },
  { &hf_rrc_measurementCapability4, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MeasurementCapabilityExt4 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UE_RadioAccessCapabBandFDD5(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UE_RadioAccessCapabBandFDD5, UE_RadioAccessCapabBandFDD5_sequence);

  return offset;
}


static const per_sequence_t UE_RadioAccessCapabBandFDDList5_sequence_of[1] = {
  { &hf_rrc_UE_RadioAccessCapabBandFDDList5_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UE_RadioAccessCapabBandFDD5 },
};

static int
dissect_rrc_UE_RadioAccessCapabBandFDDList5(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_UE_RadioAccessCapabBandFDDList5, UE_RadioAccessCapabBandFDDList5_sequence_of,
                                                  1, maxFreqBandsFDD_ext2, FALSE);

  return offset;
}


static const per_sequence_t UE_RadioAccessCapabBandFDD_ext2_sequence[] = {
  { &hf_rrc_radioFrequencyBandFDD, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RadioFrequencyBandFDD },
  { &hf_rrc_radioFrequencyBandFDD2, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RadioFrequencyBandFDD2 },
  { &hf_rrc_compressedModeMeasCapabFDDList3, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CompressedModeMeasCapabFDDList3 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UE_RadioAccessCapabBandFDD_ext2(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UE_RadioAccessCapabBandFDD_ext2, UE_RadioAccessCapabBandFDD_ext2_sequence);

  return offset;
}


static const per_sequence_t UE_RadioAccessCapabBandFDDList_ext2_sequence_of[1] = {
  { &hf_rrc_UE_RadioAccessCapabBandFDDList_ext2_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UE_RadioAccessCapabBandFDD_ext2 },
};

static int
dissect_rrc_UE_RadioAccessCapabBandFDDList_ext2(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_UE_RadioAccessCapabBandFDDList_ext2, UE_RadioAccessCapabBandFDDList_ext2_sequence_of,
                                                  1, maxFreqBandsFDD2, FALSE);

  return offset;
}


static const per_sequence_t UE_RadioAccessCapabBand_va40ext_sequence[] = {
  { &hf_rrc_ue_RadioAccessCapabBandFDDList5, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UE_RadioAccessCapabBandFDDList5 },
  { &hf_rrc_ue_RadioAccessCapabBandFDDList_ext2, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_RadioAccessCapabBandFDDList_ext2 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UE_RadioAccessCapabBand_va40ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UE_RadioAccessCapabBand_va40ext, UE_RadioAccessCapabBand_va40ext_sequence);

  return offset;
}


static const per_sequence_t RF_CapabBandListFDDComp_ext2_sequence_of[1] = {
  { &hf_rrc_RF_CapabBandListFDDComp_ext2_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RF_CapabBandFDDComp },
};

static int
dissect_rrc_RF_CapabBandListFDDComp_ext2(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_RF_CapabBandListFDDComp_ext2, RF_CapabBandListFDDComp_ext2_sequence_of,
                                                  1, maxFreqBandsFDD_ext2, FALSE);

  return offset;
}


static const per_sequence_t InterRATHandoverInfoBand_va40ext_IEs_sequence[] = {
  { &hf_rrc_ue_RadioAccessCapabBand, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_RadioAccessCapabBand_va40ext },
  { &hf_rrc_rf_CapabilityFDDComp, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RF_CapabBandListFDDComp_ext2 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_InterRATHandoverInfoBand_va40ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_InterRATHandoverInfoBand_va40ext_IEs, InterRATHandoverInfoBand_va40ext_IEs_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions_56_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_nonCriticalExtensions_56(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_nonCriticalExtensions_56, T_nonCriticalExtensions_56_sequence);

  return offset;
}


static const per_sequence_t T_va40NonCriticalExtensions_02_sequence[] = {
  { &hf_rrc_interRATHandoverInfo_va40ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_InterRATHandoverInfoBand_va40ext_IEs },
  { &hf_rrc_nonCriticalExtensions_56, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_nonCriticalExtensions_56 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_va40NonCriticalExtensions_02(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_va40NonCriticalExtensions_02, T_va40NonCriticalExtensions_02_sequence);

  return offset;
}


static const per_sequence_t T_v7f0NonCriticalExtensions_04_sequence[] = {
  { &hf_rrc_interRATHandoverInfo_v7f0ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_InterRATHandoverInfo_v7f0ext_IEs },
  { &hf_rrc_va40NonCriticalExtensions_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_va40NonCriticalExtensions_02 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v7f0NonCriticalExtensions_04(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v7f0NonCriticalExtensions_04, T_v7f0NonCriticalExtensions_04_sequence);

  return offset;
}


static const per_sequence_t T_v7e0NonCriticalExtensions_sequence[] = {
  { &hf_rrc_interRATHandoverInfo_v7e0ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_InterRATHandoverInfo_v7e0ext_IEs },
  { &hf_rrc_v7f0NonCriticalExtensions_04, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v7f0NonCriticalExtensions_04 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v7e0NonCriticalExtensions(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v7e0NonCriticalExtensions, T_v7e0NonCriticalExtensions_sequence);

  return offset;
}


static const per_sequence_t InterRATHandoverInfo_r3_add_ext_IEs_sequence[] = {
  { &hf_rrc_interRATHandoverInfo_v690ext1, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_InterRATHandoverInfo_v690ext1_IEs },
  { &hf_rrc_v7e0NonCriticalExtensions, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v7e0NonCriticalExtensions },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_InterRATHandoverInfo_r3_add_ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_InterRATHandoverInfo_r3_add_ext_IEs, InterRATHandoverInfo_r3_add_ext_IEs_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions_115_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_nonCriticalExtensions_115(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_nonCriticalExtensions_115, T_nonCriticalExtensions_115_sequence);

  return offset;
}


static const per_sequence_t T_v7g0NonCriticalExtensions_13_sequence[] = {
  { &hf_rrc_radioBearerSetup_v7g0ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RadioBearerSetup_v7g0ext_IEs },
  { &hf_rrc_nonCriticalExtensions_115, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_nonCriticalExtensions_115 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v7g0NonCriticalExtensions_13(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v7g0NonCriticalExtensions_13, T_v7g0NonCriticalExtensions_13_sequence);

  return offset;
}


static const per_sequence_t T_v7f0NonCriticalExtensions_12_sequence[] = {
  { &hf_rrc_radioBearerSetup_v7f0ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RadioBearerSetup_v7f0ext_IEs },
  { &hf_rrc_v7g0NonCriticalExtensions_13, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v7g0NonCriticalExtensions_13 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v7f0NonCriticalExtensions_12(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v7f0NonCriticalExtensions_12, T_v7f0NonCriticalExtensions_12_sequence);

  return offset;
}


static const per_sequence_t RadioBearerSetup_r7_add_ext_IEs_sequence[] = {
  { &hf_rrc_radioBearerSetup_v7d0ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RadioBearerSetup_v7d0ext_IEs },
  { &hf_rrc_v7f0NonCriticalExtensions_12, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v7f0NonCriticalExtensions_12 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_RadioBearerSetup_r7_add_ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_RadioBearerSetup_r7_add_ext_IEs, RadioBearerSetup_r7_add_ext_IEs_sequence);

  return offset;
}


static const per_sequence_t RRCConnectionSetupComplete_v650ext_IEs_sequence[] = {
  { &hf_rrc_ue_RadioAccessCapability_v650ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UE_RadioAccessCapability_v650ext },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_RRCConnectionSetupComplete_v650ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_RRCConnectionSetupComplete_v650ext_IEs, RRCConnectionSetupComplete_v650ext_IEs_sequence);

  return offset;
}


static const value_string rrc_T_supportOfHandoverToGAN_vals[] = {
  {   0, "doesSupportHandoverToGAN" },
  { 0, NULL }
};


static int
dissect_rrc_T_supportOfHandoverToGAN(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t MultiModeRAT_Capability_v680ext_sequence[] = {
  { &hf_rrc_supportOfHandoverToGAN, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_supportOfHandoverToGAN },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_MultiModeRAT_Capability_v680ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_MultiModeRAT_Capability_v680ext, MultiModeRAT_Capability_v680ext_sequence);

  return offset;
}


static const per_sequence_t UE_RadioAccessCapability_v680ext_sequence[] = {
  { &hf_rrc_multiModeRAT_Capability_v680ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MultiModeRAT_Capability_v680ext },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UE_RadioAccessCapability_v680ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UE_RadioAccessCapability_v680ext, UE_RadioAccessCapability_v680ext_sequence);

  return offset;
}


static const per_sequence_t RRCConnectionSetupComplete_v680ext_IEs_sequence[] = {
  { &hf_rrc_ue_RadioAccessCapability_v680ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UE_RadioAccessCapability_v680ext },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_RRCConnectionSetupComplete_v680ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_RRCConnectionSetupComplete_v680ext_IEs, RRCConnectionSetupComplete_v680ext_IEs_sequence);

  return offset;
}


static const per_sequence_t RRCConnectionSetupComplete_v7e0ext_IEs_sequence[] = {
  { &hf_rrc_ue_RadioAccessCapability, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UE_RadioAccessCapability_v7e0ext },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_RRCConnectionSetupComplete_v7e0ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_RRCConnectionSetupComplete_v7e0ext_IEs, RRCConnectionSetupComplete_v7e0ext_IEs_sequence);

  return offset;
}


static const per_sequence_t RRCConnectionSetupComplete_v7f0ext_IEs_sequence[] = {
  { &hf_rrc_ue_RadioAccessCapability_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_RadioAccessCapability_v7f0ext },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_RRCConnectionSetupComplete_v7f0ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_RRCConnectionSetupComplete_v7f0ext_IEs, RRCConnectionSetupComplete_v7f0ext_IEs_sequence);

  return offset;
}


static const per_sequence_t RRCConnectionSetupCompleteBand_va40ext_IEs_sequence[] = {
  { &hf_rrc_ue_RadioAccessCapabBand, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_RadioAccessCapabBand_va40ext },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_RRCConnectionSetupCompleteBand_va40ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_RRCConnectionSetupCompleteBand_va40ext_IEs, RRCConnectionSetupCompleteBand_va40ext_IEs_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions_136_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_nonCriticalExtensions_136(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_nonCriticalExtensions_136, T_nonCriticalExtensions_136_sequence);

  return offset;
}


static const per_sequence_t T_va40NonCriticalExtensions_12_sequence[] = {
  { &hf_rrc_rrcConnectionSetupCompleteBand_va40ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RRCConnectionSetupCompleteBand_va40ext_IEs },
  { &hf_rrc_nonCriticalExtensions_136, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_nonCriticalExtensions_136 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_va40NonCriticalExtensions_12(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_va40NonCriticalExtensions_12, T_va40NonCriticalExtensions_12_sequence);

  return offset;
}


static const per_sequence_t T_v7f0NonCriticalExtensions_13_sequence[] = {
  { &hf_rrc_rrcConnectionSetupComplete_v7f0ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RRCConnectionSetupComplete_v7f0ext_IEs },
  { &hf_rrc_va40NonCriticalExtensions_13, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_va40NonCriticalExtensions_12 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v7f0NonCriticalExtensions_13(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v7f0NonCriticalExtensions_13, T_v7f0NonCriticalExtensions_13_sequence);

  return offset;
}


static const per_sequence_t T_v7e0NonCriticalExtensions_02_sequence[] = {
  { &hf_rrc_rrcConnectionSetupComplete_v7e0ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RRCConnectionSetupComplete_v7e0ext_IEs },
  { &hf_rrc_v7f0NonCriticalExtensions_13, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v7f0NonCriticalExtensions_13 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v7e0NonCriticalExtensions_02(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v7e0NonCriticalExtensions_02, T_v7e0NonCriticalExtensions_02_sequence);

  return offset;
}


static const per_sequence_t T_v680NonCriticalExtensions_sequence[] = {
  { &hf_rrc_rrcConnectionSetupComplete_v680ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RRCConnectionSetupComplete_v680ext_IEs },
  { &hf_rrc_v7e0NonCriticalExtensions_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v7e0NonCriticalExtensions_02 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v680NonCriticalExtensions(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v680NonCriticalExtensions, T_v680NonCriticalExtensions_sequence);

  return offset;
}


static const per_sequence_t RRCConnectionSetupComplete_r3_add_ext_IEs_sequence[] = {
  { &hf_rrc_rrcConnectionSetupComplete_v650ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RRCConnectionSetupComplete_v650ext_IEs },
  { &hf_rrc_v680NonCriticalExtensions, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v680NonCriticalExtensions },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_RRCConnectionSetupComplete_r3_add_ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_RRCConnectionSetupComplete_r3_add_ext_IEs, RRCConnectionSetupComplete_r3_add_ext_IEs_sequence);

  return offset;
}


static const per_sequence_t RRC_FailureInfo_r3_IEs_sequence[] = {
  { &hf_rrc_failureCauseWithProtErr, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_FailureCauseWithProtErr },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_RRC_FailureInfo_r3_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_RRC_FailureInfo_r3_IEs, RRC_FailureInfo_r3_IEs_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions_137_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_nonCriticalExtensions_137(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_nonCriticalExtensions_137, T_nonCriticalExtensions_137_sequence);

  return offset;
}


static const per_sequence_t T_laterNonCriticalExtensions_43_sequence[] = {
  { &hf_rrc_rrc_FailureInfo_r3_add_ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_BIT_STRING },
  { &hf_rrc_nonCriticalExtensions_137, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_nonCriticalExtensions_137 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_laterNonCriticalExtensions_43(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_laterNonCriticalExtensions_43, T_laterNonCriticalExtensions_43_sequence);

  return offset;
}


static const per_sequence_t T_r3_21_sequence[] = {
  { &hf_rrc_rRC_FailureInfo_r3, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RRC_FailureInfo_r3_IEs },
  { &hf_rrc_laterNonCriticalExtensions_43, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_laterNonCriticalExtensions_43 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_r3_21(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_r3_21, T_r3_21_sequence);

  return offset;
}


static const per_sequence_t T_criticalExtensions_107_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_criticalExtensions_107(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_criticalExtensions_107, T_criticalExtensions_107_sequence);

  return offset;
}


static const value_string rrc_RRC_FailureInfo_vals[] = {
  {   0, "r3" },
  {   1, "criticalExtensions" },
  { 0, NULL }
};

static const per_choice_t RRC_FailureInfo_choice[] = {
  {   0, &hf_rrc_r3_21           , ASN1_NO_EXTENSIONS     , dissect_rrc_T_r3_21 },
  {   1, &hf_rrc_criticalExtensions_108, ASN1_NO_EXTENSIONS     , dissect_rrc_T_criticalExtensions_107 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_RRC_FailureInfo(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_RRC_FailureInfo, RRC_FailureInfo_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t UECapabilityInformation_v650ext_IEs_sequence[] = {
  { &hf_rrc_ue_RadioAccessCapability_v650ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UE_RadioAccessCapability_v650ext },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UECapabilityInformation_v650ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UECapabilityInformation_v650ext_IEs, UECapabilityInformation_v650ext_IEs_sequence);

  return offset;
}


static const per_sequence_t UECapabilityInformation_v680ext_IEs_sequence[] = {
  { &hf_rrc_ue_RadioAccessCapability_v680ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UE_RadioAccessCapability_v680ext },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UECapabilityInformation_v680ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UECapabilityInformation_v680ext_IEs, UECapabilityInformation_v680ext_IEs_sequence);

  return offset;
}


static const per_sequence_t UECapabilityInformation_v7e0ext_IEs_sequence[] = {
  { &hf_rrc_ue_RadioAccessCapability, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UE_RadioAccessCapability_v7e0ext },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UECapabilityInformation_v7e0ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UECapabilityInformation_v7e0ext_IEs, UECapabilityInformation_v7e0ext_IEs_sequence);

  return offset;
}


static const per_sequence_t UECapabilityInformation_v7f0ext_IEs_sequence[] = {
  { &hf_rrc_ue_RadioAccessCapability_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_RadioAccessCapability_v7f0ext },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UECapabilityInformation_v7f0ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UECapabilityInformation_v7f0ext_IEs, UECapabilityInformation_v7f0ext_IEs_sequence);

  return offset;
}


static const per_sequence_t UECapabilityInformation_va40ext_IEs_sequence[] = {
  { &hf_rrc_ue_RadioAccessCapabBand, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_RadioAccessCapabBand_va40ext },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UECapabilityInformation_va40ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UECapabilityInformation_va40ext_IEs, UECapabilityInformation_va40ext_IEs_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions_161_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_nonCriticalExtensions_161(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_nonCriticalExtensions_161, T_nonCriticalExtensions_161_sequence);

  return offset;
}


static const per_sequence_t T_va40NonCriticalExtensions_14_sequence[] = {
  { &hf_rrc_ueCapabilityInformation_va40ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UECapabilityInformation_va40ext_IEs },
  { &hf_rrc_nonCriticalExtensions_161, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_nonCriticalExtensions_161 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_va40NonCriticalExtensions_14(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_va40NonCriticalExtensions_14, T_va40NonCriticalExtensions_14_sequence);

  return offset;
}


static const per_sequence_t T_v7f0NonCriticalExtensions_16_sequence[] = {
  { &hf_rrc_ueCapabilityInformation_v7f0ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UECapabilityInformation_v7f0ext_IEs },
  { &hf_rrc_va40NonCriticalExtensions_15, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_va40NonCriticalExtensions_14 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v7f0NonCriticalExtensions_16(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v7f0NonCriticalExtensions_16, T_v7f0NonCriticalExtensions_16_sequence);

  return offset;
}


static const per_sequence_t T_v7e0NonCriticalExtensions_03_sequence[] = {
  { &hf_rrc_ueCapabilityInformation_v7e0ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UECapabilityInformation_v7e0ext_IEs },
  { &hf_rrc_v7f0NonCriticalExtensions_16, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v7f0NonCriticalExtensions_16 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v7e0NonCriticalExtensions_03(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v7e0NonCriticalExtensions_03, T_v7e0NonCriticalExtensions_03_sequence);

  return offset;
}


static const per_sequence_t T_v680NonCriticalExtensions_01_sequence[] = {
  { &hf_rrc_ueCapabilityInformation_v680ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UECapabilityInformation_v680ext_IEs },
  { &hf_rrc_v7e0NonCriticalExtensions_03, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v7e0NonCriticalExtensions_03 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v680NonCriticalExtensions_01(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v680NonCriticalExtensions_01, T_v680NonCriticalExtensions_01_sequence);

  return offset;
}


static const per_sequence_t UECapabilityInformation_r3_add_ext_IEs_sequence[] = {
  { &hf_rrc_ueCapabilityInformation_v650ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UECapabilityInformation_v650ext_IEs },
  { &hf_rrc_v680NonCriticalExtensions_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v680NonCriticalExtensions_01 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UECapabilityInformation_r3_add_ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UECapabilityInformation_r3_add_ext_IEs, UECapabilityInformation_r3_add_ext_IEs_sequence);

  return offset;
}


static const value_string rrc_T_supportForTwoDRXSchemesInPCH_01_vals[] = {
  {   0, "true" },
  { 0, NULL }
};


static int
dissect_rrc_T_supportForTwoDRXSchemesInPCH_01(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t URAUpdate_v7e0ext_IEs_sequence[] = {
  { &hf_rrc_supportForTwoDRXSchemesInPCH_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_supportForTwoDRXSchemesInPCH_01 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_URAUpdate_v7e0ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_URAUpdate_v7e0ext_IEs, URAUpdate_v7e0ext_IEs_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions_168_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_nonCriticalExtensions_168(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_nonCriticalExtensions_168, T_nonCriticalExtensions_168_sequence);

  return offset;
}


static const per_sequence_t URAUpdate_r3_add_ext_IEs_sequence[] = {
  { &hf_rrc_uraUpdate_v7e0ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_URAUpdate_v7e0ext_IEs },
  { &hf_rrc_nonCriticalExtensions_168, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_nonCriticalExtensions_168 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_URAUpdate_r3_add_ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_URAUpdate_r3_add_ext_IEs, URAUpdate_r3_add_ext_IEs_sequence);

  return offset;
}



static int
dissect_rrc_T_mib(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_octet_string_containing_pdu_new(tvb, offset, actx, tree, hf_index,
                                                                NO_BOUND, NO_BOUND, FALSE, dissect_rrc_MasterInformationBlock_PDU);

  return offset;
}



static int
dissect_rrc_T_sysInfoTypeSB1(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_octet_string_containing_pdu_new(tvb, offset, actx, tree, hf_index,
                                                                NO_BOUND, NO_BOUND, FALSE, dissect_SysInfoTypeSB1_PDU);

  return offset;
}



static int
dissect_rrc_T_sysInfoTypeSB2(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_octet_string_containing_pdu_new(tvb, offset, actx, tree, hf_index,
                                                                NO_BOUND, NO_BOUND, FALSE, dissect_SysInfoTypeSB2_PDU);

  return offset;
}



static int
dissect_rrc_T_sysInfoType1(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_octet_string_containing_pdu_new(tvb, offset, actx, tree, hf_index,
                                                                NO_BOUND, NO_BOUND, FALSE, dissect_rrc_SysInfoType1_PDU);

  return offset;
}



static int
dissect_rrc_T_sysInfoType3(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_octet_string_containing_pdu_new(tvb, offset, actx, tree, hf_index,
                                                                NO_BOUND, NO_BOUND, FALSE, dissect_rrc_SysInfoType3_PDU);

  return offset;
}



static int
dissect_rrc_T_sysInfoType5(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_octet_string_containing_pdu_new(tvb, offset, actx, tree, hf_index,
                                                                NO_BOUND, NO_BOUND, FALSE, dissect_SysInfoType5_PDU);

  return offset;
}



static int
dissect_rrc_T_sysInfoType7(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_octet_string_containing_pdu_new(tvb, offset, actx, tree, hf_index,
                                                                NO_BOUND, NO_BOUND, FALSE, dissect_rrc_SysInfoType7_PDU);

  return offset;
}



static int
dissect_rrc_T_sysInfoType11(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_octet_string_containing_pdu_new(tvb, offset, actx, tree, hf_index,
                                                                NO_BOUND, NO_BOUND, FALSE, dissect_SysInfoType11_PDU);

  return offset;
}



static int
dissect_rrc_T_sysInfoType11bis(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_octet_string_containing_pdu_new(tvb, offset, actx, tree, hf_index,
                                                                NO_BOUND, NO_BOUND, FALSE, dissect_SysInfoType11bis_PDU);

  return offset;
}



static int
dissect_rrc_T_sysInfoType12(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_octet_string_containing_pdu_new(tvb, offset, actx, tree, hf_index,
                                                                NO_BOUND, NO_BOUND, FALSE, dissect_rrc_SysInfoType12_PDU);

  return offset;
}



static int
dissect_rrc_T_sysInfoType22(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_octet_string_containing_pdu_new(tvb, offset, actx, tree, hf_index,
                                                                NO_BOUND, NO_BOUND, FALSE, dissect_SysInfoType22_PDU);

  return offset;
}


static const per_sequence_t System_Information_Container_vbxyext_IEs_sequence[] = {
  { &hf_rrc_sysInfoType22   , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_sysInfoType22 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_System_Information_Container_vbxyext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_System_Information_Container_vbxyext_IEs, System_Information_Container_vbxyext_IEs_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions_188_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_nonCriticalExtensions_188(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_nonCriticalExtensions_188, T_nonCriticalExtensions_188_sequence);

  return offset;
}


static const per_sequence_t T_vbxyNonCriticalExtensions_16_sequence[] = {
  { &hf_rrc_system_Information_Container_vbxyext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_System_Information_Container_vbxyext_IEs },
  { &hf_rrc_nonCriticalExtensions_188, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_nonCriticalExtensions_188 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_vbxyNonCriticalExtensions_16(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_vbxyNonCriticalExtensions_16, T_vbxyNonCriticalExtensions_16_sequence);

  return offset;
}


static const per_sequence_t System_Information_Container_sequence[] = {
  { &hf_rrc_mib             , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_mib },
  { &hf_rrc_sysInfoTypeSB1  , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_sysInfoTypeSB1 },
  { &hf_rrc_sysInfoTypeSB2  , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_sysInfoTypeSB2 },
  { &hf_rrc_sysInfoType1    , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_sysInfoType1 },
  { &hf_rrc_sysInfoType3    , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_sysInfoType3 },
  { &hf_rrc_sysInfoType5    , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_sysInfoType5 },
  { &hf_rrc_sysInfoType7    , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_sysInfoType7 },
  { &hf_rrc_sysInfoType11   , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_sysInfoType11 },
  { &hf_rrc_sysInfoType11bis, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_sysInfoType11bis },
  { &hf_rrc_sysInfoType12   , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_sysInfoType12 },
  { &hf_rrc_vbxyNonCriticalExtensions_16, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_vbxyNonCriticalExtensions_16 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_System_Information_Container(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_System_Information_Container, System_Information_Container_sequence);

  return offset;
}



static int
dissect_rrc_ANSI_41_NAS_Parameter(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_bit_string(tvb, offset, actx, tree, hf_index,
                                     1, 2048, FALSE, NULL);

  return offset;
}



static int
dissect_rrc_NAS_SystemInformationANSI_41(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_rrc_ANSI_41_NAS_Parameter(tvb, offset, actx, tree, hf_index);

  return offset;
}


static const value_string rrc_T_cn_Type_vals[] = {
  {   0, "gsm-MAP" },
  {   1, "ansi-41" },
  { 0, NULL }
};

static const per_choice_t T_cn_Type_choice[] = {
  {   0, &hf_rrc_gsm_MAP         , ASN1_NO_EXTENSIONS     , dissect_rrc_NAS_SystemInformationGSM_MAP },
  {   1, &hf_rrc_ansi_41         , ASN1_NO_EXTENSIONS     , dissect_rrc_NAS_SystemInformationANSI_41 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_cn_Type(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_cn_Type, T_cn_Type_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t CN_DomainSysInfo_sequence[] = {
  { &hf_rrc_cn_DomainIdentity, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CN_DomainIdentity },
  { &hf_rrc_cn_Type         , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_cn_Type },
  { &hf_rrc_cn_DRX_CycleLengthCoeff, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CN_DRX_CycleLengthCoefficient },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_CN_DomainSysInfo(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_CN_DomainSysInfo, CN_DomainSysInfo_sequence);

  return offset;
}


static const per_sequence_t CN_DomainSysInfoList_sequence_of[1] = {
  { &hf_rrc_CN_DomainSysInfoList_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CN_DomainSysInfo },
};

static int
dissect_rrc_CN_DomainSysInfoList(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_CN_DomainSysInfoList, CN_DomainSysInfoList_sequence_of,
                                                  1, maxCNdomains, FALSE);

  return offset;
}


static const value_string rrc_T_eab_Category_vals[] = {
  {   0, "a" },
  {   1, "b" },
  {   2, "c" },
  { 0, NULL }
};


static int
dissect_rrc_T_eab_Category(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     3, NULL, FALSE, 0, NULL);

  return offset;
}


static const value_string rrc_AccessClassBarred_vals[] = {
  {   0, "barred" },
  {   1, "notBarred" },
  { 0, NULL }
};


static int
dissect_rrc_AccessClassBarred(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     2, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t EAB_AccessClassBarredList_sequence_of[1] = {
  { &hf_rrc_EAB_AccessClassBarredList_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_AccessClassBarred },
};

static int
dissect_rrc_EAB_AccessClassBarredList(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_EAB_AccessClassBarredList, EAB_AccessClassBarredList_sequence_of,
                                                  10, 10, FALSE);

  return offset;
}


static const per_sequence_t EAB_Configuration_sequence[] = {
  { &hf_rrc_eab_Category    , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_eab_Category },
  { &hf_rrc_eab_AccessClassBarredList, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_EAB_AccessClassBarredList },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_EAB_Configuration(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_EAB_Configuration, EAB_Configuration_sequence);

  return offset;
}


static const per_sequence_t DomainSpecificEAB_Parameters_sequence[] = {
  { &hf_rrc_cs_DomainSpecificEAB_Parameters, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_EAB_Configuration },
  { &hf_rrc_ps_DomainSpecificEAB_Parameters, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_EAB_Configuration },
  { &hf_rrc_commonEAB_Parameters, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_EAB_Configuration },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_DomainSpecificEAB_Parameters(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_DomainSpecificEAB_Parameters, DomainSpecificEAB_Parameters_sequence);

  return offset;
}


static const per_sequence_t EAB_ParametersPerPLMN_List_item_sequence[] = {
  { &hf_rrc_domainSpecificEAB_Parameters, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DomainSpecificEAB_Parameters },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_EAB_ParametersPerPLMN_List_item(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_EAB_ParametersPerPLMN_List_item, EAB_ParametersPerPLMN_List_item_sequence);

  return offset;
}


static const per_sequence_t EAB_ParametersPerPLMN_List_sequence_of[1] = {
  { &hf_rrc_EAB_ParametersPerPLMN_List_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_EAB_ParametersPerPLMN_List_item },
};

static int
dissect_rrc_EAB_ParametersPerPLMN_List(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_EAB_ParametersPerPLMN_List, EAB_ParametersPerPLMN_List_sequence_of,
                                                  1, 6, FALSE);

  return offset;
}


static const value_string rrc_EAB_Parameters_vals[] = {
  {   0, "eab-ParametersPerPLMN-List" },
  {   1, "eab-ParametersForAll" },
  { 0, NULL }
};

static const per_choice_t EAB_Parameters_choice[] = {
  {   0, &hf_rrc_eab_ParametersPerPLMN_List, ASN1_NO_EXTENSIONS     , dissect_rrc_EAB_ParametersPerPLMN_List },
  {   1, &hf_rrc_eab_ParametersForAll, ASN1_NO_EXTENSIONS     , dissect_rrc_DomainSpecificEAB_Parameters },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_EAB_Parameters(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_EAB_Parameters, EAB_Parameters_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t LocationRegistrationAccessClassBarredList_sequence_of[1] = {
  { &hf_rrc_LocationRegistrationAccessClassBarredList_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_AccessClassBarred },
};

static int
dissect_rrc_LocationRegistrationAccessClassBarredList(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_LocationRegistrationAccessClassBarredList, LocationRegistrationAccessClassBarredList_sequence_of,
                                                  15, 15, FALSE);

  return offset;
}


static const value_string rrc_LocationRegistrationParameters_vals[] = {
  {   0, "noRestriction" },
  {   1, "restriction" },
  { 0, NULL }
};

static const per_choice_t LocationRegistrationParameters_choice[] = {
  {   0, &hf_rrc_noRestriction   , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   1, &hf_rrc_restriction     , ASN1_NO_EXTENSIONS     , dissect_rrc_LocationRegistrationAccessClassBarredList },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_LocationRegistrationParameters(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_LocationRegistrationParameters, LocationRegistrationParameters_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t PLMN_IdentityWithOptionalMCC_r6_sequence[] = {
  { &hf_rrc_mcc             , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MCC },
  { &hf_rrc_mnc             , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MNC },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_PLMN_IdentityWithOptionalMCC_r6(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_PLMN_IdentityWithOptionalMCC_r6, PLMN_IdentityWithOptionalMCC_r6_sequence);

  return offset;
}


static const per_sequence_t SEQUENCE_SIZE_1_5_OF_PLMN_IdentityWithOptionalMCC_r6_sequence_of[1] = {
  { &hf_rrc_multiplePLMNs_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PLMN_IdentityWithOptionalMCC_r6 },
};

static int
dissect_rrc_SEQUENCE_SIZE_1_5_OF_PLMN_IdentityWithOptionalMCC_r6(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_SEQUENCE_SIZE_1_5_OF_PLMN_IdentityWithOptionalMCC_r6, SEQUENCE_SIZE_1_5_OF_PLMN_IdentityWithOptionalMCC_r6_sequence_of,
                                                  1, 5, FALSE);

  return offset;
}


static const per_sequence_t MultiplePLMN_List_r6_sequence[] = {
  { &hf_rrc_mibPLMN_Identity, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { &hf_rrc_multiplePLMNs   , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SEQUENCE_SIZE_1_5_OF_PLMN_IdentityWithOptionalMCC_r6 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_MultiplePLMN_List_r6(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_MultiplePLMN_List_r6, MultiplePLMN_List_r6_sequence);

  return offset;
}


static const value_string rrc_T_pagingResponseRestrictionIndicator_vals[] = {
  {   0, "all" },
  {   1, "cS" },
  {   2, "pS" },
  {   3, "none" },
  { 0, NULL }
};


static int
dissect_rrc_T_pagingResponseRestrictionIndicator(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     4, NULL, FALSE, 0, NULL);

  return offset;
}


static const value_string rrc_T_locationRegistrationRestrictionIndicator_vals[] = {
  {   0, "all" },
  {   1, "cS" },
  {   2, "pS" },
  { 0, NULL }
};


static int
dissect_rrc_T_locationRegistrationRestrictionIndicator(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     3, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t PagingPermissionWithAccessControlParameters_sequence[] = {
  { &hf_rrc_pagingResponseRestrictionIndicator, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_pagingResponseRestrictionIndicator },
  { &hf_rrc_locationRegistrationRestrictionIndicator, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_locationRegistrationRestrictionIndicator },
  { &hf_rrc_locationRegistration, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_LocationRegistrationParameters },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_PagingPermissionWithAccessControlParameters(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_PagingPermissionWithAccessControlParameters, PagingPermissionWithAccessControlParameters_sequence);

  return offset;
}


static const per_sequence_t PagingPermissionWithAccessControlList_sequence[] = {
  { &hf_rrc_pagingPermissionWithAccessControlParametersForOperator1, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_PagingPermissionWithAccessControlParameters },
  { &hf_rrc_pagingPermissionWithAccessControlParametersForOperator2, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_PagingPermissionWithAccessControlParameters },
  { &hf_rrc_pagingPermissionWithAccessControlParametersForOperator3, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_PagingPermissionWithAccessControlParameters },
  { &hf_rrc_pagingPermissionWithAccessControlParametersForOperator4, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_PagingPermissionWithAccessControlParameters },
  { &hf_rrc_pagingPermissionWithAccessControlParametersForOperator5, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_PagingPermissionWithAccessControlParameters },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_PagingPermissionWithAccessControlList(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_PagingPermissionWithAccessControlList, PagingPermissionWithAccessControlList_sequence);

  return offset;
}


static const value_string rrc_PagingPermissionWithAccessControlForSharedNetwork_vals[] = {
  {   0, "pagingPermissionWithAccessControlList" },
  {   1, "pagingPermissionWithAccessControlForAll" },
  { 0, NULL }
};

static const per_choice_t PagingPermissionWithAccessControlForSharedNetwork_choice[] = {
  {   0, &hf_rrc_pagingPermissionWithAccessControlList, ASN1_NO_EXTENSIONS     , dissect_rrc_PagingPermissionWithAccessControlList },
  {   1, &hf_rrc_pagingPermissionWithAccessControlForAll, ASN1_NO_EXTENSIONS     , dissect_rrc_PagingPermissionWithAccessControlParameters },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_PagingPermissionWithAccessControlForSharedNetwork(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_PagingPermissionWithAccessControlForSharedNetwork, PagingPermissionWithAccessControlForSharedNetwork_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t T_gsm_MAP_sequence[] = {
  { &hf_rrc_plmn_Identity   , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PLMN_Identity },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_gsm_MAP(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_gsm_MAP, T_gsm_MAP_sequence);

  return offset;
}



static int
dissect_rrc_P_REV(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_bit_string(tvb, offset, actx, tree, hf_index,
                                     8, 8, FALSE, NULL);

  return offset;
}



static int
dissect_rrc_Min_P_REV(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_bit_string(tvb, offset, actx, tree, hf_index,
                                     8, 8, FALSE, NULL);

  return offset;
}



static int
dissect_rrc_SID(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_bit_string(tvb, offset, actx, tree, hf_index,
                                     15, 15, FALSE, NULL);

  return offset;
}



static int
dissect_rrc_NID(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_bit_string(tvb, offset, actx, tree, hf_index,
                                     16, 16, FALSE, NULL);

  return offset;
}


static const per_sequence_t T_ansi_41_sequence[] = {
  { &hf_rrc_p_REV           , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_P_REV },
  { &hf_rrc_min_P_REV       , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_Min_P_REV },
  { &hf_rrc_sid             , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SID },
  { &hf_rrc_nid             , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_NID },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_ansi_41(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_ansi_41, T_ansi_41_sequence);

  return offset;
}


static const per_sequence_t T_gsm_MAP_and_ANSI_41_sequence[] = {
  { &hf_rrc_plmn_Identity   , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PLMN_Identity },
  { &hf_rrc_p_REV           , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_P_REV },
  { &hf_rrc_min_P_REV       , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_Min_P_REV },
  { &hf_rrc_sid             , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SID },
  { &hf_rrc_nid             , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_NID },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_gsm_MAP_and_ANSI_41(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_gsm_MAP_and_ANSI_41, T_gsm_MAP_and_ANSI_41_sequence);

  return offset;
}


static const value_string rrc_PLMN_Type_vals[] = {
  {   0, "gsm-MAP" },
  {   1, "ansi-41" },
  {   2, "gsm-MAP-and-ANSI-41" },
  {   3, "spare" },
  { 0, NULL }
};

static const per_choice_t PLMN_Type_choice[] = {
  {   0, &hf_rrc_gsm_MAP_01      , ASN1_NO_EXTENSIONS     , dissect_rrc_T_gsm_MAP },
  {   1, &hf_rrc_ansi_41_01      , ASN1_NO_EXTENSIONS     , dissect_rrc_T_ansi_41 },
  {   2, &hf_rrc_gsm_MAP_and_ANSI_41, ASN1_NO_EXTENSIONS     , dissect_rrc_T_gsm_MAP_and_ANSI_41 },
  {   3, &hf_rrc_spare           , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_PLMN_Type(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_PLMN_Type, PLMN_Type_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t AccessClassBarredList_sequence_of[1] = {
  { &hf_rrc_AccessClassBarredList_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_AccessClassBarred },
};

static int
dissect_rrc_AccessClassBarredList(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_AccessClassBarredList, AccessClassBarredList_sequence_of,
                                                  maxAC, maxAC, FALSE);

  return offset;
}


static const value_string rrc_AllowedIndicator_vals[] = {
  {   0, "allowed" },
  {   1, "notAllowed" },
  { 0, NULL }
};


static int
dissect_rrc_AllowedIndicator(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     2, NULL, FALSE, 0, NULL);

  return offset;
}


static const value_string rrc_T_Barred_vals[] = {
  {   0, "s10" },
  {   1, "s20" },
  {   2, "s40" },
  {   3, "s80" },
  {   4, "s160" },
  {   5, "s320" },
  {   6, "s640" },
  {   7, "s1280" },
  { 0, NULL }
};


static int
dissect_rrc_T_Barred(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     8, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t T_barred_sequence[] = {
  { &hf_rrc_intraFreqCellReselectionInd, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_AllowedIndicator },
  { &hf_rrc_t_Barred        , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_Barred },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_barred(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_barred, T_barred_sequence);

  return offset;
}


static const value_string rrc_CellBarred_vals[] = {
  {   0, "barred" },
  {   1, "notBarred" },
  { 0, NULL }
};

static const per_choice_t CellBarred_choice[] = {
  {   0, &hf_rrc_barred          , ASN1_NO_EXTENSIONS     , dissect_rrc_T_barred },
  {   1, &hf_rrc_notBarred       , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_CellBarred(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_CellBarred, CellBarred_choice,
                                 NULL);

  return offset;
}


static const value_string rrc_ReservedIndicator_vals[] = {
  {   0, "reserved" },
  {   1, "notReserved" },
  { 0, NULL }
};


static int
dissect_rrc_ReservedIndicator(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     2, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t CellAccessRestriction_sequence[] = {
  { &hf_rrc_cellBarred      , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CellBarred },
  { &hf_rrc_cellReservedForOperatorUse, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_ReservedIndicator },
  { &hf_rrc_cellReservationExtension, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_ReservedIndicator },
  { &hf_rrc_accessClassBarredList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_AccessClassBarredList },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_CellAccessRestriction(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_CellAccessRestriction, CellAccessRestriction_sequence);

  return offset;
}


static const value_string rrc_T_cellReservedForCSG_vals[] = {
  {   0, "true" },
  { 0, NULL }
};


static int
dissect_rrc_T_cellReservedForCSG(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t CellAccessRestriction_v870ext_sequence[] = {
  { &hf_rrc_cellReservedForCSG, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_cellReservedForCSG },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_CellAccessRestriction_v870ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_CellAccessRestriction_v870ext, CellAccessRestriction_v870ext_sequence);

  return offset;
}


static const value_string rrc_RAT_vals[] = {
  {   0, "utra-FDD" },
  {   1, "utra-TDD" },
  {   2, "gsm" },
  {   3, "cdma2000" },
  { 0, NULL }
};


static int
dissect_rrc_RAT(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     4, NULL, FALSE, 0, NULL);

  return offset;
}


static const value_string rrc_MappingFunctionType_vals[] = {
  {   0, "linear" },
  {   1, "functionType2" },
  {   2, "functionType3" },
  {   3, "functionType4" },
  { 0, NULL }
};


static int
dissect_rrc_MappingFunctionType(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     4, NULL, FALSE, 0, NULL);

  return offset;
}



static int
dissect_rrc_MapParameter(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            0U, 127U, NULL, FALSE);

  return offset;
}



static int
dissect_rrc_UpperLimit(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            1U, 91U, NULL, FALSE);

  return offset;
}


static const per_sequence_t MappingFunctionParameter_sequence[] = {
  { &hf_rrc_functionType    , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MappingFunctionType },
  { &hf_rrc_mapParameter1   , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MapParameter },
  { &hf_rrc_mapParameter2   , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MapParameter },
  { &hf_rrc_upperLimit      , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UpperLimit },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_MappingFunctionParameter(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_MappingFunctionParameter, MappingFunctionParameter_sequence);

  return offset;
}


static const per_sequence_t MappingFunctionParameterList_sequence_of[1] = {
  { &hf_rrc_MappingFunctionParameterList_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MappingFunctionParameter },
};

static int
dissect_rrc_MappingFunctionParameterList(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_MappingFunctionParameterList, MappingFunctionParameterList_sequence_of,
                                                  1, maxMeasIntervals, FALSE);

  return offset;
}


static const per_sequence_t Mapping_sequence[] = {
  { &hf_rrc_rat             , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RAT },
  { &hf_rrc_mappingFunctionParameterList, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MappingFunctionParameterList },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_Mapping(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_Mapping, Mapping_sequence);

  return offset;
}


static const per_sequence_t MappingInfo_sequence_of[1] = {
  { &hf_rrc_MappingInfo_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_Mapping },
};

static int
dissect_rrc_MappingInfo(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_MappingInfo, MappingInfo_sequence_of,
                                                  1, maxRAT, FALSE);

  return offset;
}



static int
dissect_rrc_Q_Hyst_S(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            0U, 20U, NULL, FALSE);

  return offset;
}


static const per_sequence_t T_cpich_Ec_N0_sequence[] = {
  { &hf_rrc_q_HYST_2_S      , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_Q_Hyst_S },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_cpich_Ec_N0(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_cpich_Ec_N0, T_cpich_Ec_N0_sequence);

  return offset;
}


static const value_string rrc_T_cellSelectQualityMeasure_vals[] = {
  {   0, "cpich-Ec-N0" },
  {   1, "cpich-RSCP" },
  { 0, NULL }
};

static const per_choice_t T_cellSelectQualityMeasure_choice[] = {
  {   0, &hf_rrc_cpich_Ec_N0     , ASN1_NO_EXTENSIONS     , dissect_rrc_T_cpich_Ec_N0 },
  {   1, &hf_rrc_cpich_RSCP      , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_cellSelectQualityMeasure(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_cellSelectQualityMeasure, T_cellSelectQualityMeasure_choice,
                                 NULL);

  return offset;
}



static int
dissect_rrc_S_SearchQual(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            -16, 10U, NULL, FALSE);

  return offset;
}



static int
dissect_rrc_S_SearchRXLEV(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            -53, 45U, NULL, FALSE);

  return offset;
}


static const value_string rrc_RAT_Identifier_vals[] = {
  {   0, "gsm" },
  {   1, "cdma2000" },
  { 0, NULL }
};


static int
dissect_rrc_RAT_Identifier(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     2, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t RAT_FDD_Info_sequence[] = {
  { &hf_rrc_rat_Identifier  , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RAT_Identifier },
  { &hf_rrc_s_SearchRAT     , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_S_SearchQual },
  { &hf_rrc_s_HCS_RAT       , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_S_SearchRXLEV },
  { &hf_rrc_s_Limit_SearchRAT, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_S_SearchQual },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_RAT_FDD_Info(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_RAT_FDD_Info, RAT_FDD_Info_sequence);

  return offset;
}


static const per_sequence_t RAT_FDD_InfoList_sequence_of[1] = {
  { &hf_rrc_RAT_FDD_InfoList_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RAT_FDD_Info },
};

static int
dissect_rrc_RAT_FDD_InfoList(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_RAT_FDD_InfoList, RAT_FDD_InfoList_sequence_of,
                                                  1, maxOtherRAT, FALSE);

  return offset;
}


static const per_sequence_t T_fdd_45_sequence[] = {
  { &hf_rrc_s_Intrasearch   , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_S_SearchQual },
  { &hf_rrc_s_Intersearch   , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_S_SearchQual },
  { &hf_rrc_s_SearchHCS     , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_S_SearchRXLEV },
  { &hf_rrc_rat_List        , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RAT_FDD_InfoList },
  { &hf_rrc_q_QualMin       , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_Q_QualMin },
  { &hf_rrc_q_RxlevMin      , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_Q_RxlevMin },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_fdd_45(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_fdd_45, T_fdd_45_sequence);

  return offset;
}


static const per_sequence_t RAT_TDD_Info_sequence[] = {
  { &hf_rrc_rat_Identifier  , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RAT_Identifier },
  { &hf_rrc_s_SearchRAT_01  , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_S_SearchRXLEV },
  { &hf_rrc_s_HCS_RAT       , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_S_SearchRXLEV },
  { &hf_rrc_s_Limit_SearchRAT_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_S_SearchRXLEV },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_RAT_TDD_Info(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_RAT_TDD_Info, RAT_TDD_Info_sequence);

  return offset;
}


static const per_sequence_t RAT_TDD_InfoList_sequence_of[1] = {
  { &hf_rrc_RAT_TDD_InfoList_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RAT_TDD_Info },
};

static int
dissect_rrc_RAT_TDD_InfoList(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_RAT_TDD_InfoList, RAT_TDD_InfoList_sequence_of,
                                                  1, maxOtherRAT, FALSE);

  return offset;
}


static const per_sequence_t T_tdd_10_sequence[] = {
  { &hf_rrc_s_Intrasearch_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_S_SearchRXLEV },
  { &hf_rrc_s_Intersearch_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_S_SearchRXLEV },
  { &hf_rrc_s_SearchHCS     , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_S_SearchRXLEV },
  { &hf_rrc_rat_List_01     , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RAT_TDD_InfoList },
  { &hf_rrc_q_RxlevMin      , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_Q_RxlevMin },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_tdd_10(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_tdd_10, T_tdd_10_sequence);

  return offset;
}


static const value_string rrc_T_modeSpecificInfo_16_vals[] = {
  {   0, "fdd" },
  {   1, "tdd" },
  { 0, NULL }
};

static const per_choice_t T_modeSpecificInfo_16_choice[] = {
  {   0, &hf_rrc_fdd_46          , ASN1_NO_EXTENSIONS     , dissect_rrc_T_fdd_45 },
  {   1, &hf_rrc_tdd_11          , ASN1_NO_EXTENSIONS     , dissect_rrc_T_tdd_10 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_modeSpecificInfo_16(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_modeSpecificInfo_16, T_modeSpecificInfo_16_choice,
                                 NULL);

  return offset;
}



static int
dissect_rrc_T_Reselection_S(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            0U, 31U, NULL, FALSE);

  return offset;
}


static const value_string rrc_T_CRMaxHyst_vals[] = {
  {   0, "notUsed" },
  {   1, "t10" },
  {   2, "t20" },
  {   3, "t30" },
  {   4, "t40" },
  {   5, "t50" },
  {   6, "t60" },
  {   7, "t70" },
  { 0, NULL }
};


static int
dissect_rrc_T_CRMaxHyst(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     8, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t N_CR_T_CRMaxHyst_sequence[] = {
  { &hf_rrc_n_CR            , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_INTEGER_1_16 },
  { &hf_rrc_t_CRMaxHyst     , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_CRMaxHyst },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_N_CR_T_CRMaxHyst(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_N_CR_T_CRMaxHyst, N_CR_T_CRMaxHyst_sequence);

  return offset;
}


static const value_string rrc_T_CRMax_vals[] = {
  {   0, "notUsed" },
  {   1, "t30" },
  {   2, "t60" },
  {   3, "t120" },
  {   4, "t180" },
  {   5, "t240" },
  { 0, NULL }
};

static const per_choice_t T_CRMax_choice[] = {
  {   0, &hf_rrc_notUsed         , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   1, &hf_rrc_t30             , ASN1_NO_EXTENSIONS     , dissect_rrc_N_CR_T_CRMaxHyst },
  {   2, &hf_rrc_t60             , ASN1_NO_EXTENSIONS     , dissect_rrc_N_CR_T_CRMaxHyst },
  {   3, &hf_rrc_t120            , ASN1_NO_EXTENSIONS     , dissect_rrc_N_CR_T_CRMaxHyst },
  {   4, &hf_rrc_t180            , ASN1_NO_EXTENSIONS     , dissect_rrc_N_CR_T_CRMaxHyst },
  {   5, &hf_rrc_t240            , ASN1_NO_EXTENSIONS     , dissect_rrc_N_CR_T_CRMaxHyst },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_CRMax(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_CRMax, T_CRMax_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t HCS_ServingCellInformation_sequence[] = {
  { &hf_rrc_hcs_PRIO        , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_HCS_PRIO },
  { &hf_rrc_q_HCS           , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_Q_HCS },
  { &hf_rrc_t_CR_Max        , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_CRMax },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_HCS_ServingCellInformation(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_HCS_ServingCellInformation, HCS_ServingCellInformation_sequence);

  return offset;
}


static const per_sequence_t CellSelectReselectInfoSIB_3_4_sequence[] = {
  { &hf_rrc_mappingInfo     , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MappingInfo },
  { &hf_rrc_cellSelectQualityMeasure, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_cellSelectQualityMeasure },
  { &hf_rrc_modeSpecificInfo_16, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_modeSpecificInfo_16 },
  { &hf_rrc_q_Hyst_l_S      , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_Q_Hyst_S },
  { &hf_rrc_t_Reselection_S , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_Reselection_S },
  { &hf_rrc_hcs_ServingCellInformation, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_HCS_ServingCellInformation },
  { &hf_rrc_maxAllowedUL_TX_Power, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MaxAllowedUL_TX_Power },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_CellSelectReselectInfoSIB_3_4(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_CellSelectReselectInfoSIB_3_4, CellSelectReselectInfoSIB_3_4_sequence);

  return offset;
}


static const per_sequence_t CSG_DedicatedFrequencyInfoList_sequence_of[1] = {
  { &hf_rrc_CSG_DedicatedFrequencyInfoList_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_FrequencyInfo },
};

static int
dissect_rrc_CSG_DedicatedFrequencyInfoList(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_CSG_DedicatedFrequencyInfoList, CSG_DedicatedFrequencyInfoList_sequence_of,
                                                  1, maxDedicatedCSGFreq, FALSE);

  return offset;
}


static const value_string rrc_T_numberOfPSCs_vals[] = {
  {   0, "psc5" },
  {   1, "psc10" },
  {   2, "psc15" },
  {   3, "psc20" },
  {   4, "psc30" },
  {   5, "psc40" },
  {   6, "psc50" },
  {   7, "psc64" },
  {   8, "psc80" },
  {   9, "psc120" },
  {  10, "psc160" },
  {  11, "psc256" },
  {  12, "alltheRest" },
  {  13, "spare3" },
  {  14, "spare2" },
  {  15, "spare1" },
  { 0, NULL }
};


static int
dissect_rrc_T_numberOfPSCs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     16, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t CSG_PSCSplitInfo_sequence[] = {
  { &hf_rrc_startPSC        , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_0_63 },
  { &hf_rrc_numberOfPSCs    , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_numberOfPSCs },
  { &hf_rrc_pscRange2Offset , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_INTEGER_1_63 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_CSG_PSCSplitInfo(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_CSG_PSCSplitInfo, CSG_PSCSplitInfo_sequence);

  return offset;
}


static const per_sequence_t T_restriction_sequence[] = {
  { &hf_rrc_domainSpecficAccessClassBarredList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_AccessClassBarredList },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_restriction(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_restriction, T_restriction_sequence);

  return offset;
}


static const value_string rrc_DomainSpecificAccessRestriction_v670ext_vals[] = {
  {   0, "noRestriction" },
  {   1, "restriction" },
  { 0, NULL }
};

static const per_choice_t DomainSpecificAccessRestriction_v670ext_choice[] = {
  {   0, &hf_rrc_noRestriction   , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   1, &hf_rrc_restriction_01  , ASN1_NO_EXTENSIONS     , dissect_rrc_T_restriction },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_DomainSpecificAccessRestriction_v670ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_DomainSpecificAccessRestriction_v670ext, DomainSpecificAccessRestriction_v670ext_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t DomainSpecificAccessRestrictionParam_v670ext_sequence[] = {
  { &hf_rrc_cSDomainSpecificAccessRestriction, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DomainSpecificAccessRestriction_v670ext },
  { &hf_rrc_pSDomainSpecificAccessRestriction, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DomainSpecificAccessRestriction_v670ext },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_DomainSpecificAccessRestrictionParam_v670ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_DomainSpecificAccessRestrictionParam_v670ext, DomainSpecificAccessRestrictionParam_v670ext_sequence);

  return offset;
}


static const per_sequence_t DomainSpecificAccessRestrictionList_v670ext_sequence[] = {
  { &hf_rrc_domainSpecificAccessRestrictionParametersForOperator1, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DomainSpecificAccessRestrictionParam_v670ext },
  { &hf_rrc_domainSpecificAccessRestrictionParametersForOperator2, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DomainSpecificAccessRestrictionParam_v670ext },
  { &hf_rrc_domainSpecificAccessRestrictionParametersForOperator3, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DomainSpecificAccessRestrictionParam_v670ext },
  { &hf_rrc_domainSpecificAccessRestrictionParametersForOperator4, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DomainSpecificAccessRestrictionParam_v670ext },
  { &hf_rrc_domainSpecificAccessRestrictionParametersForOperator5, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DomainSpecificAccessRestrictionParam_v670ext },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_DomainSpecificAccessRestrictionList_v670ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_DomainSpecificAccessRestrictionList_v670ext, DomainSpecificAccessRestrictionList_v670ext_sequence);

  return offset;
}


static const value_string rrc_DomainSpecificAccessRestrictionForSharedNetwork_v670ext_vals[] = {
  {   0, "domainSpecificAccessRestictionList" },
  {   1, "domainSpecificAccessRestictionParametersForAll" },
  { 0, NULL }
};

static const per_choice_t DomainSpecificAccessRestrictionForSharedNetwork_v670ext_choice[] = {
  {   0, &hf_rrc_domainSpecificAccessRestictionList, ASN1_NO_EXTENSIONS     , dissect_rrc_DomainSpecificAccessRestrictionList_v670ext },
  {   1, &hf_rrc_domainSpecificAccessRestictionParametersForAll, ASN1_NO_EXTENSIONS     , dissect_rrc_DomainSpecificAccessRestrictionParam_v670ext },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_DomainSpecificAccessRestrictionForSharedNetwork_v670ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_DomainSpecificAccessRestrictionForSharedNetwork_v670ext, DomainSpecificAccessRestrictionForSharedNetwork_v670ext_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t Mapping_LCR_r4_sequence[] = {
  { &hf_rrc_mappingFunctionParameterList, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MappingFunctionParameterList },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_Mapping_LCR_r4(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_Mapping_LCR_r4, Mapping_LCR_r4_sequence);

  return offset;
}



static int
dissect_rrc_OccurrenceSequenceNumberOfPICH(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            1U, maxSCCPCH, NULL, FALSE);

  return offset;
}



static int
dissect_rrc_Q_Hyst_S_Fine(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            0U, 40U, NULL, FALSE);

  return offset;
}



static int
dissect_rrc_SpeedDependentScalingFactor(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            0U, 10U, NULL, FALSE);

  return offset;
}



static int
dissect_rrc_T_Reselection_S_Fine(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            0U, 31U, NULL, FALSE);

  return offset;
}



static int
dissect_rrc_TreselectionScalingFactor(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            4U, 19U, NULL, FALSE);

  return offset;
}



static int
dissect_rrc_TreselectionScalingFactor2(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            1U, 16U, NULL, FALSE);

  return offset;
}


static const per_sequence_t URA_IdentityList_sequence_of[1] = {
  { &hf_rrc_URA_IdentityList_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_URA_Identity },
};

static int
dissect_rrc_URA_IdentityList(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_URA_IdentityList, URA_IdentityList_sequence_of,
                                                  1, maxURA, FALSE);

  return offset;
}



static int
dissect_rrc_N_AP_RetransMax(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            1U, 64U, NULL, FALSE);

  return offset;
}



static int
dissect_rrc_N_AccessFails(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            1U, 64U, NULL, FALSE);

  return offset;
}



static int
dissect_rrc_NF_BO_NoAICH(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            0U, 31U, NULL, FALSE);

  return offset;
}



static int
dissect_rrc_NS_BO_Busy(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            0U, 63U, NULL, FALSE);

  return offset;
}



static int
dissect_rrc_NF_BO_AllBusy(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            0U, 31U, NULL, FALSE);

  return offset;
}



static int
dissect_rrc_NF_BO_Mismatch(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            0U, 127U, NULL, FALSE);

  return offset;
}


static const value_string rrc_T_CPCH_vals[] = {
  {   0, "ct0" },
  {   1, "ct1" },
  { 0, NULL }
};


static int
dissect_rrc_T_CPCH(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     2, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t BackoffControlParams_sequence[] = {
  { &hf_rrc_n_AP_RetransMax , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_N_AP_RetransMax },
  { &hf_rrc_n_AccessFails   , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_N_AccessFails },
  { &hf_rrc_nf_BO_NoAICH    , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_NF_BO_NoAICH },
  { &hf_rrc_ns_BO_Busy      , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_NS_BO_Busy },
  { &hf_rrc_nf_BO_AllBusy   , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_NF_BO_AllBusy },
  { &hf_rrc_nf_BO_Mismatch  , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_NF_BO_Mismatch },
  { &hf_rrc_t_CPCH          , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_CPCH },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_BackoffControlParams(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_BackoffControlParams, BackoffControlParams_sequence);

  return offset;
}



static int
dissect_rrc_BandComb(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            1U, 256U, NULL, FALSE);

  return offset;
}


static const value_string rrc_T_adjacentFrequencyMeasurements_vals[] = {
  {   0, "true" },
  { 0, NULL }
};


static int
dissect_rrc_T_adjacentFrequencyMeasurements(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t CompressedModeMeasCapability_v860ext_sequence[] = {
  { &hf_rrc_adjacentFrequencyMeasurements, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_adjacentFrequencyMeasurements },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_CompressedModeMeasCapability_v860ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_CompressedModeMeasCapability_v860ext, CompressedModeMeasCapability_v860ext_sequence);

  return offset;
}


static const value_string rrc_T_interBandMeasurements_vals[] = {
  {   0, "true" },
  { 0, NULL }
};


static int
dissect_rrc_T_interBandMeasurements(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t CompressedModeMeasCapability_v920ext_sequence[] = {
  { &hf_rrc_interBandMeasurements, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_interBandMeasurements },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_CompressedModeMeasCapability_v920ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_CompressedModeMeasCapability_v920ext, CompressedModeMeasCapability_v920ext_sequence);

  return offset;
}


static const value_string rrc_T_enhancedInterFrequencyMeasurements_vals[] = {
  {   0, "true" },
  { 0, NULL }
};


static int
dissect_rrc_T_enhancedInterFrequencyMeasurements(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}


static const value_string rrc_T_freqSpecificCompressedMode_vals[] = {
  {   0, "true" },
  { 0, NULL }
};


static int
dissect_rrc_T_freqSpecificCompressedMode(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t CompressedModeMeasCapability_va40ext_sequence[] = {
  { &hf_rrc_enhancedInterFrequencyMeasurements, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_enhancedInterFrequencyMeasurements },
  { &hf_rrc_freqSpecificCompressedMode, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_freqSpecificCompressedMode },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_CompressedModeMeasCapability_va40ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_CompressedModeMeasCapability_va40ext, CompressedModeMeasCapability_va40ext_sequence);

  return offset;
}



static int
dissect_rrc_NS_IP(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            0U, 28U, NULL, FALSE);

  return offset;
}


static const per_sequence_t InitialPriorityDelayList_sequence_of[1] = {
  { &hf_rrc_InitialPriorityDelayList_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_NS_IP },
};

static int
dissect_rrc_InitialPriorityDelayList(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_InitialPriorityDelayList, InitialPriorityDelayList_sequence_of,
                                                  1, maxASC, FALSE);

  return offset;
}



static int
dissect_rrc_DL_DPCCH_BER(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            0U, 63U, NULL, FALSE);

  return offset;
}


static const per_sequence_t CPCH_Parameters_sequence[] = {
  { &hf_rrc_initialPriorityDelayList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_InitialPriorityDelayList },
  { &hf_rrc_backoffControlParams, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BackoffControlParams },
  { &hf_rrc_powerControlAlgorithm, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PowerControlAlgorithm },
  { &hf_rrc_dl_DPCCH_BER    , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DL_DPCCH_BER },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_CPCH_Parameters(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_CPCH_Parameters, CPCH_Parameters_sequence);

  return offset;
}


static const value_string rrc_T_supportOfIntraFreqProximityIndication_vals[] = {
  {   0, "true" },
  { 0, NULL }
};


static int
dissect_rrc_T_supportOfIntraFreqProximityIndication(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}


static const value_string rrc_T_supportOfInterFreqProximityIndication_vals[] = {
  {   0, "true" },
  { 0, NULL }
};


static int
dissect_rrc_T_supportOfInterFreqProximityIndication(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}


static const value_string rrc_T_supportOfE_UtraProximityIndication_vals[] = {
  {   0, "true" },
  { 0, NULL }
};


static int
dissect_rrc_T_supportOfE_UtraProximityIndication(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t CSG_ProximityIndicationCapability_sequence[] = {
  { &hf_rrc_supportOfIntraFreqProximityIndication, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_supportOfIntraFreqProximityIndication },
  { &hf_rrc_supportOfInterFreqProximityIndication, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_supportOfInterFreqProximityIndication },
  { &hf_rrc_supportOfE_UtraProximityIndication, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_supportOfE_UtraProximityIndication },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_CSG_ProximityIndicationCapability(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_CSG_ProximityIndicationCapability, CSG_ProximityIndicationCapability_sequence);

  return offset;
}


static const value_string rrc_T_hsscchlessHsdschOperation_vals[] = {
  {   0, "true" },
  { 0, NULL }
};


static int
dissect_rrc_T_hsscchlessHsdschOperation(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}


static const value_string rrc_T_enhancedFdpch_vals[] = {
  {   0, "true" },
  { 0, NULL }
};


static int
dissect_rrc_T_enhancedFdpch(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}


static const value_string rrc_T_hsdschReception_CellFach_vals[] = {
  {   0, "true" },
  { 0, NULL }
};


static int
dissect_rrc_T_hsdschReception_CellFach(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}


static const value_string rrc_T_hsdschReception_CellUraPch_vals[] = {
  {   0, "true" },
  { 0, NULL }
};


static int
dissect_rrc_T_hsdschReception_CellUraPch(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t DL_PhysChCapabilityFDD_v770ext_sequence[] = {
  { &hf_rrc_hsdsch_physical_layer_category_ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_HSDSCH_physical_layer_category_ext },
  { &hf_rrc_hsscchlessHsdschOperation, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_hsscchlessHsdschOperation },
  { &hf_rrc_enhancedFdpch   , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_enhancedFdpch },
  { &hf_rrc_hsdschReception_CellFach, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_hsdschReception_CellFach },
  { &hf_rrc_hsdschReception_CellUraPch, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_hsdschReception_CellUraPch },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_DL_PhysChCapabilityFDD_v770ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_DL_PhysChCapabilityFDD_v770ext, DL_PhysChCapabilityFDD_v770ext_sequence);

  return offset;
}


static const value_string rrc_T_supportOfTargetCellPreConfig_vals[] = {
  {   0, "true" },
  { 0, NULL }
};


static int
dissect_rrc_T_supportOfTargetCellPreConfig(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}


static const value_string rrc_T_supportOfHsdschDrxOperation_vals[] = {
  {   0, "true" },
  { 0, NULL }
};


static int
dissect_rrc_T_supportOfHsdschDrxOperation(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t DL_PhysChCapabilityFDD_v860ext_sequence[] = {
  { &hf_rrc_hsdsch_physical_layer_category_ext2, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_HSDSCH_physical_layer_category_ext2 },
  { &hf_rrc_supportOfTargetCellPreConfig, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_supportOfTargetCellPreConfig },
  { &hf_rrc_supportOfHsdschDrxOperation, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_supportOfHsdschDrxOperation },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_DL_PhysChCapabilityFDD_v860ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_DL_PhysChCapabilityFDD_v860ext, DL_PhysChCapabilityFDD_v860ext_sequence);

  return offset;
}


static const value_string rrc_T_supportOfMimoOnlySingleStream_vals[] = {
  {   0, "true" },
  { 0, NULL }
};


static int
dissect_rrc_T_supportOfMimoOnlySingleStream(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t DL_PhysChCapabilityFDD_v920ext_sequence[] = {
  { &hf_rrc_hsdsch_physical_layer_category_ext3, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_HSDSCH_physical_layer_category_ext3 },
  { &hf_rrc_supportOfMimoOnlySingleStream, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_supportOfMimoOnlySingleStream },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_DL_PhysChCapabilityFDD_v920ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_DL_PhysChCapabilityFDD_v920ext, DL_PhysChCapabilityFDD_v920ext_sequence);

  return offset;
}


static const per_sequence_t DL_PhysChCapabilityFDD_va40ext_sequence[] = {
  { &hf_rrc_hsdsch_physical_layer_category_ext4, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_HSDSCH_physical_layer_category_ext4 },
  { &hf_rrc_hsdsch_physical_layer_category_ext5, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_HSDSCH_physical_layer_category_ext5 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_DL_PhysChCapabilityFDD_va40ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_DL_PhysChCapabilityFDD_va40ext, DL_PhysChCapabilityFDD_va40ext_sequence);

  return offset;
}


static const value_string rrc_T_supportOf4x4MimoDualStreamRestriction_vals[] = {
  {   0, "true" },
  { 0, NULL }
};


static int
dissect_rrc_T_supportOf4x4MimoDualStreamRestriction(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}


static const value_string rrc_T_supportOfNodeBTrigHS_DPCCHTransmission_vals[] = {
  {   0, "true" },
  { 0, NULL }
};


static int
dissect_rrc_T_supportOfNodeBTrigHS_DPCCHTransmission(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}


static const value_string rrc_T_supportOfHS_DSCHSecondDRXOperation_vals[] = {
  {   0, "true" },
  { 0, NULL }
};


static int
dissect_rrc_T_supportOfHS_DSCHSecondDRXOperation(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t DL_PhysChCapabilityFDD_vbxyext_sequence[] = {
  { &hf_rrc_hsdsch_physical_layer_category_ext6, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_HSDSCH_physical_layer_category_ext6 },
  { &hf_rrc_hsdsch_physical_layer_category_ext7, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_HSDSCH_physical_layer_category_ext7 },
  { &hf_rrc_hsdsch_physical_layer_category_ext8, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_HSDSCH_physical_layer_category_ext8 },
  { &hf_rrc_supportOf4x4MimoDualStreamRestriction, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_supportOf4x4MimoDualStreamRestriction },
  { &hf_rrc_supportOfNodeBTrigHS_DPCCHTransmission, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_supportOfNodeBTrigHS_DPCCHTransmission },
  { &hf_rrc_supportOfHS_DSCHSecondDRXOperation, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_supportOfHS_DSCHSecondDRXOperation },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_DL_PhysChCapabilityFDD_vbxyext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_DL_PhysChCapabilityFDD_vbxyext, DL_PhysChCapabilityFDD_vbxyext_sequence);

  return offset;
}


static const per_sequence_t DL_PhysChCapabilityTDD_128_v770ext_sequence[] = {
  { &hf_rrc_multiCarrier_physical_layer_category, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MultiCarrier_HSDSCH_physical_layer_category },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_DL_PhysChCapabilityTDD_128_v770ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_DL_PhysChCapabilityTDD_128_v770ext, DL_PhysChCapabilityTDD_128_v770ext_sequence);

  return offset;
}



static int
dissect_rrc_MultiCarrier_HSDSCH_physical_layer_category_extension(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            1U, 36U, NULL, FALSE);

  return offset;
}


static const value_string rrc_T_supportOfSFModeForHSPDSCHDualStream_vals[] = {
  {   0, "sf1" },
  {   1, "sf1sf16" },
  { 0, NULL }
};


static int
dissect_rrc_T_supportOfSFModeForHSPDSCHDualStream(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     2, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t DL_PhysChCapabilityTDD_128_v860ext_sequence[] = {
  { &hf_rrc_hSDSCH_physical_layer_category_extension, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_HSDSCH_physical_layer_category_extension },
  { &hf_rrc_multiCarrier_physical_layer_category_extension, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MultiCarrier_HSDSCH_physical_layer_category_extension },
  { &hf_rrc_supportOfSFModeForHSPDSCHDualStream, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_supportOfSFModeForHSPDSCHDualStream },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_DL_PhysChCapabilityTDD_128_v860ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_DL_PhysChCapabilityTDD_128_v860ext, DL_PhysChCapabilityTDD_128_v860ext_sequence);

  return offset;
}



static int
dissect_rrc_MultiCarrier_HSDSCH_physical_layer_category_extension2(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            37U, 64U, NULL, FALSE);

  return offset;
}


static const per_sequence_t DL_PhysChCapabilityInfoTDD_128_va40ext_sequence[] = {
  { &hf_rrc_multiCarrier_physical_layer_category_extension2, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MultiCarrier_HSDSCH_physical_layer_category_extension2 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_DL_PhysChCapabilityInfoTDD_128_va40ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_DL_PhysChCapabilityInfoTDD_128_va40ext, DL_PhysChCapabilityInfoTDD_128_va40ext_sequence);

  return offset;
}


static const value_string rrc_T_tdd768_hspdsch_vals[] = {
  {   0, "supported" },
  {   1, "unsupported" },
  { 0, NULL }
};

static const per_choice_t T_tdd768_hspdsch_choice[] = {
  {   0, &hf_rrc_supported       , ASN1_NO_EXTENSIONS     , dissect_rrc_HSDSCH_physical_layer_category },
  {   1, &hf_rrc_unsupported     , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_tdd768_hspdsch(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_tdd768_hspdsch, T_tdd768_hspdsch_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t DL_PhysChCapabilityTDD_768_sequence[] = {
  { &hf_rrc_maxTS_PerFrame  , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MaxTS_PerFrame },
  { &hf_rrc_maxPhysChPerFrame_02, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MaxPhysChPerFrame_768 },
  { &hf_rrc_minimumSF_01    , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MinimumSF_DL_768 },
  { &hf_rrc_supportOfPDSCH  , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { &hf_rrc_tdd768_hspdsch  , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_tdd768_hspdsch },
  { &hf_rrc_maxPhysChPerTS_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MaxPhysChPerTS_768 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_DL_PhysChCapabilityTDD_768(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_DL_PhysChCapabilityTDD_768, DL_PhysChCapabilityTDD_768_sequence);

  return offset;
}



static int
dissect_rrc_TransmissionProbability(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            1U, 8U, NULL, FALSE);

  return offset;
}



static int
dissect_rrc_MaximumBitRate(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            0U, 32U, NULL, FALSE);

  return offset;
}


static const per_sequence_t DRAC_SysInfo_sequence[] = {
  { &hf_rrc_transmissionProbability, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TransmissionProbability },
  { &hf_rrc_maximumBitRate  , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MaximumBitRate },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_DRAC_SysInfo(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_DRAC_SysInfo, DRAC_SysInfo_sequence);

  return offset;
}


static const per_sequence_t DRAC_SysInfoList_sequence_of[1] = {
  { &hf_rrc_DRAC_SysInfoList_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DRAC_SysInfo },
};

static int
dissect_rrc_DRAC_SysInfoList(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_DRAC_SysInfoList, DRAC_SysInfoList_sequence_of,
                                                  1, maxDRACclasses, FALSE);

  return offset;
}


static const value_string rrc_FrequencyBandsIndicatorFDD_vals[] = {
  {   0, "frequencyBandsIndicator1" },
  {   1, "frequencyBandsIndicator2" },
  {   2, "frequencyBandsIndicator3" },
  { 0, NULL }
};

static const per_choice_t FrequencyBandsIndicatorFDD_choice[] = {
  {   0, &hf_rrc_frequencyBandsIndicator1, ASN1_NO_EXTENSIONS     , dissect_rrc_RadioFrequencyBandFDD },
  {   1, &hf_rrc_frequencyBandsIndicator2, ASN1_NO_EXTENSIONS     , dissect_rrc_RadioFrequencyBandFDD2 },
  {   2, &hf_rrc_frequencyBandsIndicator3, ASN1_NO_EXTENSIONS     , dissect_rrc_RadioFrequencyBandFDD3 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_FrequencyBandsIndicatorFDD(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_FrequencyBandsIndicatorFDD, FrequencyBandsIndicatorFDD_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t FrequencyBandsRedirectionFDD_sequence_of[1] = {
  { &hf_rrc_FrequencyBandsRedirectionFDD_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_FrequencyBandsIndicatorFDD },
};

static int
dissect_rrc_FrequencyBandsRedirectionFDD(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_FrequencyBandsRedirectionFDD, FrequencyBandsRedirectionFDD_sequence_of,
                                                  1, maxFreqBandsIndicatorSupport, FALSE);

  return offset;
}


static const per_sequence_t T_fdd_46_sequence[] = {
  { &hf_rrc_frequencyBandsSupportFDD, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_FrequencyBandsRedirectionFDD },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_fdd_46(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_fdd_46, T_fdd_46_sequence);

  return offset;
}


static const value_string rrc_RadioFrequencyBandTDD_r10_vals[] = {
  {   0, "a" },
  {   1, "b" },
  {   2, "c" },
  {   3, "d" },
  {   4, "e" },
  {   5, "f" },
  {   6, "spare2" },
  {   7, "spare1" },
  { 0, NULL }
};


static int
dissect_rrc_RadioFrequencyBandTDD_r10(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     8, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t FrequencyBandsIndicatorSupportTDD128_sequence[] = {
  { &hf_rrc_frequencyBandsIndicatorTDD, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RadioFrequencyBandTDD_r10 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_FrequencyBandsIndicatorSupportTDD128(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_FrequencyBandsIndicatorSupportTDD128, FrequencyBandsIndicatorSupportTDD128_sequence);

  return offset;
}


static const per_sequence_t FrequencyBandsIndicatorTDD128_sequence_of[1] = {
  { &hf_rrc_FrequencyBandsIndicatorTDD128_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_FrequencyBandsIndicatorSupportTDD128 },
};

static int
dissect_rrc_FrequencyBandsIndicatorTDD128(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_FrequencyBandsIndicatorTDD128, FrequencyBandsIndicatorTDD128_sequence_of,
                                                  1, maxFreqBandsIndicatorSupport, FALSE);

  return offset;
}


static const per_sequence_t T_tdd_11_sequence[] = {
  { &hf_rrc_frequencyBandsSupportTDD128, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_FrequencyBandsIndicatorTDD128 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_tdd_11(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_tdd_11, T_tdd_11_sequence);

  return offset;
}


static const value_string rrc_T_modeSpecificInfo_17_vals[] = {
  {   0, "fdd" },
  {   1, "tdd" },
  { 0, NULL }
};

static const per_choice_t T_modeSpecificInfo_17_choice[] = {
  {   0, &hf_rrc_fdd_47          , ASN1_NO_EXTENSIONS     , dissect_rrc_T_fdd_46 },
  {   1, &hf_rrc_tdd_12          , ASN1_NO_EXTENSIONS     , dissect_rrc_T_tdd_11 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_modeSpecificInfo_17(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_modeSpecificInfo_17, T_modeSpecificInfo_17_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t FrequencyBandsIndicatorSupport_sequence[] = {
  { &hf_rrc_modeSpecificInfo_17, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_modeSpecificInfo_17 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_FrequencyBandsIndicatorSupport(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_FrequencyBandsIndicatorSupport, FrequencyBandsIndicatorSupport_sequence);

  return offset;
}


static const value_string rrc_GANSS_Mode_vals[] = {
  {   0, "networkBased" },
  {   1, "uEBased" },
  {   2, "both" },
  {   3, "none" },
  { 0, NULL }
};


static int
dissect_rrc_GANSS_Mode(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     4, NULL, FALSE, 0, NULL);

  return offset;
}



static int
dissect_rrc_INTEGER_2_6(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            2U, 6U, NULL, FALSE);

  return offset;
}


static const per_sequence_t MultiflowBandCombination_sequence[] = {
  { &hf_rrc_numberOfCellsBandA, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_2_6 },
  { &hf_rrc_numberOfFrequenciesBandA, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_1_3 },
  { &hf_rrc_numberOfCellsBandB, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_2_6 },
  { &hf_rrc_numberOfFrequenciesBandB, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_1_3 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_MultiflowBandCombination(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_MultiflowBandCombination, MultiflowBandCombination_sequence);

  return offset;
}


static const per_sequence_t MultiflowBandCombList_sequence_of[1] = {
  { &hf_rrc_MultiflowBandCombList_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MultiflowBandCombination },
};

static int
dissect_rrc_MultiflowBandCombList(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_MultiflowBandCombList, MultiflowBandCombList_sequence_of,
                                                  1, 16, FALSE);

  return offset;
}


static const value_string rrc_T_longerHARQProcessing_vals[] = {
  {   0, "true" },
  { 0, NULL }
};


static int
dissect_rrc_T_longerHARQProcessing(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t MultiflowCapability_sequence[] = {
  { &hf_rrc_mimoInDifferentBands, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MultiflowMIMOCapability },
  { &hf_rrc_longerHARQProcessing, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_longerHARQProcessing },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_MultiflowCapability(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_MultiflowCapability, MultiflowCapability_sequence);

  return offset;
}


static const per_sequence_t MeasurementCapability_v860ext_sequence[] = {
  { &hf_rrc_downlinkCompressedMode_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CompressedModeMeasCapability_v860ext },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_MeasurementCapability_v860ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_MeasurementCapability_v860ext, MeasurementCapability_v860ext_sequence);

  return offset;
}


static const per_sequence_t MeasurementCapability_v920ext_sequence[] = {
  { &hf_rrc_downlinkCompressedMode_02, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CompressedModeMeasCapability_v920ext },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_MeasurementCapability_v920ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_MeasurementCapability_v920ext, MeasurementCapability_v920ext_sequence);

  return offset;
}


static const value_string rrc_T_interFrequencyDetectedSetMeasurements_vals[] = {
  {   0, "true" },
  { 0, NULL }
};


static int
dissect_rrc_T_interFrequencyDetectedSetMeasurements(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t MeasurementCapability_va40ext_sequence[] = {
  { &hf_rrc_interFrequencyDetectedSetMeasurements, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_interFrequencyDetectedSetMeasurements },
  { &hf_rrc_downlinkCompressedMode_03, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CompressedModeMeasCapability_va40ext },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_MeasurementCapability_va40ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_MeasurementCapability_va40ext, MeasurementCapability_va40ext_sequence);

  return offset;
}


static const value_string rrc_T_interFrequencyMeasOnConfigCarriersWithoutCompMode_vals[] = {
  {   0, "true" },
  { 0, NULL }
};


static int
dissect_rrc_T_interFrequencyMeasOnConfigCarriersWithoutCompMode(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t MeasurementCapability_vbxyext_sequence[] = {
  { &hf_rrc_interFrequencyMeasOnConfigCarriersWithoutCompMode, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_interFrequencyMeasOnConfigCarriersWithoutCompMode },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_MeasurementCapability_vbxyext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_MeasurementCapability_vbxyext, MeasurementCapability_vbxyext_sequence);

  return offset;
}


static const value_string rrc_T_supportOfPSHandoverToGAN_vals[] = {
  {   0, "doesSupportPSHandoverToGAN" },
  { 0, NULL }
};


static int
dissect_rrc_T_supportOfPSHandoverToGAN(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t MultiModeRAT_Capability_v770ext_sequence[] = {
  { &hf_rrc_supportOfPSHandoverToGAN, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_supportOfPSHandoverToGAN },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_MultiModeRAT_Capability_v770ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_MultiModeRAT_Capability_v770ext, MultiModeRAT_Capability_v770ext_sequence);

  return offset;
}


static const value_string rrc_T_supportOfEUTRAFDD_vals[] = {
  {   0, "doesSupportEUTRAFDD" },
  { 0, NULL }
};


static int
dissect_rrc_T_supportOfEUTRAFDD(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}


static const value_string rrc_T_supportOfInterRATHOToEUTRAFDD_vals[] = {
  {   0, "doesSupportInterRATHOToEUTRAFDD" },
  { 0, NULL }
};


static int
dissect_rrc_T_supportOfInterRATHOToEUTRAFDD(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}


static const value_string rrc_T_supportOfEUTRATDD_vals[] = {
  {   0, "doesSupportEUTRATDD" },
  { 0, NULL }
};


static int
dissect_rrc_T_supportOfEUTRATDD(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}


static const value_string rrc_T_supportOfInterRATHOToEUTRATDD_vals[] = {
  {   0, "doesSupportInterRATHOToEUTRATDD" },
  { 0, NULL }
};


static int
dissect_rrc_T_supportOfInterRATHOToEUTRATDD(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}



static int
dissect_rrc_T_eutraFeatureGroupIndicators(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
#line 627 "../../asn1/rrc/rrc.cnf"
  tvbuff_t *eutraFeatureGroupIndicators_tvb=NULL;
  offset = dissect_per_bit_string(tvb, offset, actx, tree, hf_index,
                                     4, 4, FALSE, &eutraFeatureGroupIndicators_tvb);

  if(eutraFeatureGroupIndicators_tvb){
    /* Dissect eutraFeatureGroupIndicators */
    proto_tree *subtree;
    gint curr_bit_offset = 0;
    subtree = proto_item_add_subtree(actx->created_item, ett_rrc_eutraFeatureGroupIndicators);
    proto_tree_add_bits_item(subtree, hf_rrc_eutra_feat_group_ind_1, eutraFeatureGroupIndicators_tvb, curr_bit_offset, 1, FALSE);
    curr_bit_offset++;
    proto_tree_add_bits_item(subtree, hf_rrc_eutra_feat_group_ind_2, eutraFeatureGroupIndicators_tvb, curr_bit_offset, 1, FALSE);
    curr_bit_offset++;
    proto_tree_add_bits_item(subtree, hf_rrc_eutra_feat_group_ind_3, eutraFeatureGroupIndicators_tvb, curr_bit_offset, 1, FALSE);
    curr_bit_offset++;
    proto_tree_add_bits_item(subtree, hf_rrc_eutra_feat_group_ind_4, eutraFeatureGroupIndicators_tvb, curr_bit_offset, 1, FALSE);
    curr_bit_offset++;
}



  return offset;
}


static const per_sequence_t MultiModeRAT_Capability_v860ext_sequence[] = {
  { &hf_rrc_supportOfEUTRAFDD, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_supportOfEUTRAFDD },
  { &hf_rrc_supportOfInterRATHOToEUTRAFDD, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_supportOfInterRATHOToEUTRAFDD },
  { &hf_rrc_supportOfEUTRATDD, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_supportOfEUTRATDD },
  { &hf_rrc_supportOfInterRATHOToEUTRATDD, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_supportOfInterRATHOToEUTRATDD },
  { &hf_rrc_eutraFeatureGroupIndicators, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_eutraFeatureGroupIndicators },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_MultiModeRAT_Capability_v860ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_MultiModeRAT_Capability_v860ext, MultiModeRAT_Capability_v860ext_sequence);

  return offset;
}


static const value_string rrc_T_supportOfEUTRAFDDMeasurementReportingInCELLFACH_vals[] = {
  {   0, "doesSupportOfEUTRAFDDMeasurementReportingInCELLFACH" },
  { 0, NULL }
};


static int
dissect_rrc_T_supportOfEUTRAFDDMeasurementReportingInCELLFACH(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}


static const value_string rrc_T_supportOfEUTRATDDMeasurementReportingInCELLFACH_vals[] = {
  {   0, "doesSupportOfEUTRATDDMeasurementReportingInCELLFACH" },
  { 0, NULL }
};


static int
dissect_rrc_T_supportOfEUTRATDDMeasurementReportingInCELLFACH(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t MultiModeRAT_Capability_vbxyext_sequence[] = {
  { &hf_rrc_supportOfEUTRAFDDMeasurementReportingInCELLFACH, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_supportOfEUTRAFDDMeasurementReportingInCELLFACH },
  { &hf_rrc_supportOfEUTRATDDMeasurementReportingInCELLFACH, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_supportOfEUTRATDDMeasurementReportingInCELLFACH },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_MultiModeRAT_Capability_vbxyext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_MultiModeRAT_Capability_vbxyext, MultiModeRAT_Capability_vbxyext_sequence);

  return offset;
}



static int
dissect_rrc_N_300(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            0U, 7U, NULL, FALSE);

  return offset;
}


static const value_string rrc_T_supportOfIntraFreqSIAcquisitionForHO_vals[] = {
  {   0, "true" },
  { 0, NULL }
};


static int
dissect_rrc_T_supportOfIntraFreqSIAcquisitionForHO(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}


static const value_string rrc_T_supportOfInterFreqSIAcquisitionForHO_vals[] = {
  {   0, "true" },
  { 0, NULL }
};


static int
dissect_rrc_T_supportOfInterFreqSIAcquisitionForHO(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}


static const value_string rrc_T_supportOfE_UtraSIAcquisitionForHO_vals[] = {
  {   0, "true" },
  { 0, NULL }
};


static int
dissect_rrc_T_supportOfE_UtraSIAcquisitionForHO(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t NeighCellSI_AcquisitionCapability_sequence[] = {
  { &hf_rrc_supportOfIntraFreqSIAcquisitionForHO, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_supportOfIntraFreqSIAcquisitionForHO },
  { &hf_rrc_supportOfInterFreqSIAcquisitionForHO, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_supportOfInterFreqSIAcquisitionForHO },
  { &hf_rrc_supportOfE_UtraSIAcquisitionForHO, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_supportOfE_UtraSIAcquisitionForHO },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_NeighCellSI_AcquisitionCapability(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_NeighCellSI_AcquisitionCapability, NeighCellSI_AcquisitionCapability_sequence);

  return offset;
}


static const value_string rrc_T_gANSS_Id_vals[] = {
  {   0, "sbas" },
  {   1, "modernizedGPS" },
  {   2, "qzss" },
  {   3, "glonass" },
  {   4, "spare4" },
  {   5, "spare3" },
  {   6, "spare2" },
  {   7, "spare1" },
  { 0, NULL }
};


static int
dissect_rrc_T_gANSS_Id(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     8, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t NetworkAssistedGANSS_Supported_List_item_sequence[] = {
  { &hf_rrc_gANSS_Id        , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_gANSS_Id },
  { &hf_rrc_gANSS_Mode      , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_GANSS_Mode },
  { &hf_rrc_gANSS_SignalId  , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_GANSS_Signal_Id },
  { &hf_rrc_supportForUE_GANSS_TimingOfCellFrames, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { &hf_rrc_supportForUE_GANSS_CarrierPhaseMeasurement, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_BOOLEAN },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_NetworkAssistedGANSS_Supported_List_item(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_NetworkAssistedGANSS_Supported_List_item, NetworkAssistedGANSS_Supported_List_item_sequence);

  return offset;
}


static const per_sequence_t NetworkAssistedGANSS_Supported_List_sequence_of[1] = {
  { &hf_rrc_NetworkAssistedGANSS_Supported_List_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_NetworkAssistedGANSS_Supported_List_item },
};

static int
dissect_rrc_NetworkAssistedGANSS_Supported_List(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_NetworkAssistedGANSS_Supported_List, NetworkAssistedGANSS_Supported_List_sequence_of,
                                                  1, maxGANSS, FALSE);

  return offset;
}


static const value_string rrc_T_non_native_AD_choices_supported_vals[] = {
  {   0, "true" },
  { 0, NULL }
};


static int
dissect_rrc_T_non_native_AD_choices_supported(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t NetworkAssistedGANSS_Supported_List_v860ext_item_sequence[] = {
  { &hf_rrc_sbas_Ids        , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_BIT_STRING_SIZE_8 },
  { &hf_rrc_gANSS_SignalIds , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_BIT_STRING_SIZE_8 },
  { &hf_rrc_non_native_AD_choices_supported, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_non_native_AD_choices_supported },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_NetworkAssistedGANSS_Supported_List_v860ext_item(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_NetworkAssistedGANSS_Supported_List_v860ext_item, NetworkAssistedGANSS_Supported_List_v860ext_item_sequence);

  return offset;
}


static const per_sequence_t NetworkAssistedGANSS_Supported_List_v860ext_sequence_of[1] = {
  { &hf_rrc_NetworkAssistedGANSS_Supported_List_v860ext_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_NetworkAssistedGANSS_Supported_List_v860ext_item },
};

static int
dissect_rrc_NetworkAssistedGANSS_Supported_List_v860ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_NetworkAssistedGANSS_Supported_List_v860ext, NetworkAssistedGANSS_Supported_List_v860ext_sequence_of,
                                                  1, maxGANSS, FALSE);

  return offset;
}


static const value_string rrc_T_discontinuousDpcchTransmission_02_vals[] = {
  {   0, "true" },
  { 0, NULL }
};


static int
dissect_rrc_T_discontinuousDpcchTransmission_02(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}


static const value_string rrc_T_slotFormat4_01_vals[] = {
  {   0, "true" },
  { 0, NULL }
};


static int
dissect_rrc_T_slotFormat4_01(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t UL_PhysChCapabilityFDD_v770ext_sequence[] = {
  { &hf_rrc_edch_PhysicalLayerCategory_extension, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_INTEGER_7 },
  { &hf_rrc_discontinuousDpcchTransmission_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_discontinuousDpcchTransmission_02 },
  { &hf_rrc_slotFormat4_01  , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_slotFormat4_01 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UL_PhysChCapabilityFDD_v770ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UL_PhysChCapabilityFDD_v770ext, UL_PhysChCapabilityFDD_v770ext_sequence);

  return offset;
}


static const per_sequence_t T_fddPhysChCapability_01_sequence[] = {
  { &hf_rrc_downlinkPhysChCapability_02, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DL_PhysChCapabilityFDD_v770ext },
  { &hf_rrc_uplinkPhysChCapability_02, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UL_PhysChCapabilityFDD_v770ext },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_fddPhysChCapability_01(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_fddPhysChCapability_01, T_fddPhysChCapability_01_sequence);

  return offset;
}


static const per_sequence_t T_supported_03_sequence[] = {
  { &hf_rrc_tdd_edch_PhysicalLayerCategory_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_1_16 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_supported_03(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_supported_03, T_supported_03_sequence);

  return offset;
}


static const value_string rrc_T_tdd384_edch_vals[] = {
  {   0, "supported" },
  {   1, "unsupported" },
  { 0, NULL }
};

static const per_choice_t T_tdd384_edch_choice[] = {
  {   0, &hf_rrc_supported_09    , ASN1_NO_EXTENSIONS     , dissect_rrc_T_supported_03 },
  {   1, &hf_rrc_unsupported     , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_tdd384_edch(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_tdd384_edch, T_tdd384_edch_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t UL_PhysChCapabilityTDD_384_v770ext_sequence[] = {
  { &hf_rrc_tdd384_edch     , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_tdd384_edch },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UL_PhysChCapabilityTDD_384_v770ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UL_PhysChCapabilityTDD_384_v770ext, UL_PhysChCapabilityTDD_384_v770ext_sequence);

  return offset;
}


static const per_sequence_t T_tddPhysChCapability_384_sequence[] = {
  { &hf_rrc_uplinkPhysChCapability_03, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UL_PhysChCapabilityTDD_384_v770ext },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_tddPhysChCapability_384(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_tddPhysChCapability_384, T_tddPhysChCapability_384_sequence);

  return offset;
}


static const per_sequence_t T_supported_05_sequence[] = {
  { &hf_rrc_tdd_edch_PhysicalLayerCategory_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_1_16 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_supported_05(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_supported_05, T_supported_05_sequence);

  return offset;
}


static const value_string rrc_T_tdd384_edch_01_vals[] = {
  {   0, "supported" },
  {   1, "unsupported" },
  { 0, NULL }
};

static const per_choice_t T_tdd384_edch_01_choice[] = {
  {   0, &hf_rrc_supported_11    , ASN1_NO_EXTENSIONS     , dissect_rrc_T_supported_05 },
  {   1, &hf_rrc_unsupported     , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_tdd384_edch_01(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_tdd384_edch_01, T_tdd384_edch_01_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t UL_PhysChCapabilityTDD_768_sequence[] = {
  { &hf_rrc_maxTS_PerFrame  , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MaxTS_PerFrame },
  { &hf_rrc_maxPhysChPerTimeslot, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MaxPhysChPerTimeslot },
  { &hf_rrc_minimumSF_02    , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MinimumSF_UL },
  { &hf_rrc_supportOfPUSCH  , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { &hf_rrc_tdd384_edch_01  , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_tdd384_edch_01 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UL_PhysChCapabilityTDD_768(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UL_PhysChCapabilityTDD_768, UL_PhysChCapabilityTDD_768_sequence);

  return offset;
}


static const per_sequence_t T_tddPhysChCapability_768_sequence[] = {
  { &hf_rrc_downlinkPhysChCapability_03, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DL_PhysChCapabilityTDD_768 },
  { &hf_rrc_uplinkPhysChCapability_04, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UL_PhysChCapabilityTDD_768 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_tddPhysChCapability_768(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_tddPhysChCapability_768, T_tddPhysChCapability_768_sequence);

  return offset;
}


static const per_sequence_t T_supported_04_sequence[] = {
  { &hf_rrc_tdd_edch_PhysicalLayerCategory, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_1_6 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_supported_04(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_supported_04, T_supported_04_sequence);

  return offset;
}


static const value_string rrc_T_tdd128_edch_vals[] = {
  {   0, "supported" },
  {   1, "unsupported" },
  { 0, NULL }
};

static const per_choice_t T_tdd128_edch_choice[] = {
  {   0, &hf_rrc_supported_10    , ASN1_NO_EXTENSIONS     , dissect_rrc_T_supported_04 },
  {   1, &hf_rrc_unsupported     , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_tdd128_edch(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_tdd128_edch, T_tdd128_edch_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t UL_PhysChCapabilityTDD_128_v770ext_sequence[] = {
  { &hf_rrc_maxPhysChPerTimeslot_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MaxPhysChPerTimeslot_LCR_r7 },
  { &hf_rrc_tdd128_edch     , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_tdd128_edch },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UL_PhysChCapabilityTDD_128_v770ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UL_PhysChCapabilityTDD_128_v770ext, UL_PhysChCapabilityTDD_128_v770ext_sequence);

  return offset;
}


static const per_sequence_t T_tddPhysChCapability_128_sequence[] = {
  { &hf_rrc_downlinkPhysChCapability_04, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DL_PhysChCapabilityTDD_128_v770ext },
  { &hf_rrc_uplinkPhysChCapability_05, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UL_PhysChCapabilityTDD_128_v770ext },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_tddPhysChCapability_128(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_tddPhysChCapability_128, T_tddPhysChCapability_128_sequence);

  return offset;
}


static const per_sequence_t PhysicalChannelCapability_v770ext_sequence[] = {
  { &hf_rrc_fddPhysChCapability_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_fddPhysChCapability_01 },
  { &hf_rrc_tddPhysChCapability_384, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_tddPhysChCapability_384 },
  { &hf_rrc_tddPhysChCapability_768, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_tddPhysChCapability_768 },
  { &hf_rrc_tddPhysChCapability_128, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_tddPhysChCapability_128 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_PhysicalChannelCapability_v770ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_PhysicalChannelCapability_v770ext, PhysicalChannelCapability_v770ext_sequence);

  return offset;
}


static const per_sequence_t T_fddPhysChCapability_02_sequence[] = {
  { &hf_rrc_downlinkPhysChCapability_05, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DL_PhysChCapabilityFDD_v860ext },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_fddPhysChCapability_02(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_fddPhysChCapability_02, T_fddPhysChCapability_02_sequence);

  return offset;
}


static const per_sequence_t T_tddPhysChCapability_128_01_sequence[] = {
  { &hf_rrc_downlinkPhysChCapability_06, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DL_PhysChCapabilityTDD_128_v860ext },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_tddPhysChCapability_128_01(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_tddPhysChCapability_128_01, T_tddPhysChCapability_128_01_sequence);

  return offset;
}


static const per_sequence_t PhysicalChannelCapability_v860ext_sequence[] = {
  { &hf_rrc_fddPhysChCapability_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_fddPhysChCapability_02 },
  { &hf_rrc_tddPhysChCapability_128_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_tddPhysChCapability_128_01 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_PhysicalChannelCapability_v860ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_PhysicalChannelCapability_v860ext, PhysicalChannelCapability_v860ext_sequence);

  return offset;
}


static const per_sequence_t UL_PhysChCapabilityFDD_v920ext_sequence[] = {
  { &hf_rrc_edch_PhysicalLayerCategory_extension2, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_INTEGER_8_9 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UL_PhysChCapabilityFDD_v920ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UL_PhysChCapabilityFDD_v920ext, UL_PhysChCapabilityFDD_v920ext_sequence);

  return offset;
}


static const per_sequence_t T_fddPhysChCapability_03_sequence[] = {
  { &hf_rrc_downlinkPhysChCapability_07, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DL_PhysChCapabilityFDD_v920ext },
  { &hf_rrc_uplinkPhysChCapability_06, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UL_PhysChCapabilityFDD_v920ext },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_fddPhysChCapability_03(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_fddPhysChCapability_03, T_fddPhysChCapability_03_sequence);

  return offset;
}


static const per_sequence_t PhysicalChannelCapability_v920ext_sequence[] = {
  { &hf_rrc_fddPhysChCapability_03, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_fddPhysChCapability_03 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_PhysicalChannelCapability_v920ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_PhysicalChannelCapability_v920ext, PhysicalChannelCapability_v920ext_sequence);

  return offset;
}


static const per_sequence_t T_fddPhysChCapability_04_sequence[] = {
  { &hf_rrc_downlinkPhysChCapability_08, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DL_PhysChCapabilityFDD_va40ext },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_fddPhysChCapability_04(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_fddPhysChCapability_04, T_fddPhysChCapability_04_sequence);

  return offset;
}


static const value_string rrc_T_maxNumberCarrierForMCHSUPA_TDD_vals[] = {
  {   0, "nf-2" },
  {   1, "nf-3" },
  {   2, "nf-6" },
  {   3, "spare" },
  { 0, NULL }
};


static int
dissect_rrc_T_maxNumberCarrierForMCHSUPA_TDD(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     4, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t UL_PhysChCapabilityInfoTDD_128_va40ext_sequence[] = {
  { &hf_rrc_multiCarrier_EdchPhysicalLayerCategory_128, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_INTEGER_1_8 },
  { &hf_rrc_maxNumberCarrierForMCHSUPA_TDD, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_maxNumberCarrierForMCHSUPA_TDD },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UL_PhysChCapabilityInfoTDD_128_va40ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UL_PhysChCapabilityInfoTDD_128_va40ext, UL_PhysChCapabilityInfoTDD_128_va40ext_sequence);

  return offset;
}


static const per_sequence_t T_tddPhysChCapability_128_02_sequence[] = {
  { &hf_rrc_downlinkPhysChCapability_09, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DL_PhysChCapabilityInfoTDD_128_va40ext },
  { &hf_rrc_uplinkPhysChCapability_07, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UL_PhysChCapabilityInfoTDD_128_va40ext },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_tddPhysChCapability_128_02(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_tddPhysChCapability_128_02, T_tddPhysChCapability_128_02_sequence);

  return offset;
}


static const per_sequence_t PhysicalChannelCapability_va40ext_sequence[] = {
  { &hf_rrc_fddPhysChCapability_04, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_fddPhysChCapability_04 },
  { &hf_rrc_tddPhysChCapability_128_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_tddPhysChCapability_128_02 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_PhysicalChannelCapability_va40ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_PhysicalChannelCapability_va40ext, PhysicalChannelCapability_va40ext_sequence);

  return offset;
}


static const per_sequence_t T_fddPhysChCapability_05_sequence[] = {
  { &hf_rrc_downlinkPhysChCapability_10, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DL_PhysChCapabilityFDD_vbxyext },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_fddPhysChCapability_05(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_fddPhysChCapability_05, T_fddPhysChCapability_05_sequence);

  return offset;
}


static const per_sequence_t PhysicalChannelCapability_vbxyext_sequence[] = {
  { &hf_rrc_fddPhysChCapability_05, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_fddPhysChCapability_05 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_PhysicalChannelCapability_vbxyext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_PhysicalChannelCapability_vbxyext, PhysicalChannelCapability_vbxyext_sequence);

  return offset;
}


static const value_string rrc_T_numberOfRepetitionsPerSFNPeriod_vals[] = {
  {   0, "c2" },
  {   1, "c3" },
  {   2, "c4" },
  {   3, "c5" },
  {   4, "c6" },
  {   5, "c7" },
  {   6, "c8" },
  {   7, "c9" },
  {   8, "c10" },
  {   9, "c12" },
  {  10, "c14" },
  {  11, "c16" },
  {  12, "c18" },
  {  13, "c20" },
  {  14, "c24" },
  {  15, "c28" },
  {  16, "c32" },
  {  17, "c36" },
  {  18, "c40" },
  {  19, "c48" },
  {  20, "c56" },
  {  21, "c64" },
  {  22, "c72" },
  {  23, "c80" },
  { 0, NULL }
};

static value_string_ext rrc_T_numberOfRepetitionsPerSFNPeriod_vals_ext = VALUE_STRING_EXT_INIT(rrc_T_numberOfRepetitionsPerSFNPeriod_vals);


static int
dissect_rrc_T_numberOfRepetitionsPerSFNPeriod(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     24, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t PNBSCH_Allocation_r4_sequence[] = {
  { &hf_rrc_numberOfRepetitionsPerSFNPeriod, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_numberOfRepetitionsPerSFNPeriod },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_PNBSCH_Allocation_r4(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_PNBSCH_Allocation_r4, PNBSCH_Allocation_r4_sequence);

  return offset;
}


static const per_sequence_t RF_Capability_v860ext_sequence[] = {
  { &hf_rrc_tdd128RF_Capability_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RadioFrequencyBandTDDextList },
  { &hf_rrc_tdd384RF_Capability_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RadioFrequencyBandTDDextList },
  { &hf_rrc_tdd768RF_Capability_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RadioFrequencyBandTDDextList },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_RF_Capability_v860ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_RF_Capability_v860ext, RF_Capability_v860ext_sequence);

  return offset;
}


static const value_string rrc_T_supportOfMFBI_vals[] = {
  {   0, "true" },
  { 0, NULL }
};


static int
dissect_rrc_T_supportOfMFBI(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t RF_Capability_vaa0ext_sequence[] = {
  { &hf_rrc_supportOfMFBI   , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_supportOfMFBI },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_RF_Capability_vaa0ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_RF_Capability_vaa0ext, RF_Capability_vaa0ext_sequence);

  return offset;
}


static const per_sequence_t RLC_Capability_v770ext_sequence[] = {
  { &hf_rrc_supportOfTwoLogicalChannel, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_RLC_Capability_v770ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_RLC_Capability_v770ext, RLC_Capability_v770ext_sequence);

  return offset;
}


static const value_string rrc_TotalRLC_AM_BufferSize_v920ext_vals[] = {
  {   0, "kb1150" },
  {   1, "kb1250" },
  { 0, NULL }
};


static int
dissect_rrc_TotalRLC_AM_BufferSize_v920ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     2, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t RLC_Capability_v920ext_sequence[] = {
  { &hf_rrc_totalRLC_AM_BufferSize_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_TotalRLC_AM_BufferSize_v920ext },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_RLC_Capability_v920ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_RLC_Capability_v920ext, RLC_Capability_v920ext_sequence);

  return offset;
}


static const value_string rrc_TotalRLC_AM_BufferSize_va40ext_vals[] = {
  {   0, "kb1500" },
  {   1, "kb1800" },
  {   2, "kb2300" },
  {   3, "kb2550" },
  { 0, NULL }
};


static int
dissect_rrc_TotalRLC_AM_BufferSize_va40ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     4, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t RLC_Capability_va40ext_sequence[] = {
  { &hf_rrc_totalRLC_AM_BufferSize_03, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_TotalRLC_AM_BufferSize_va40ext },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_RLC_Capability_va40ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_RLC_Capability_va40ext, RLC_Capability_va40ext_sequence);

  return offset;
}


static const value_string rrc_T_300_vals[] = {
  {   0, "ms100" },
  {   1, "ms200" },
  {   2, "ms400" },
  {   3, "ms600" },
  {   4, "ms800" },
  {   5, "ms1000" },
  {   6, "ms1200" },
  {   7, "ms1400" },
  {   8, "ms1600" },
  {   9, "ms1800" },
  {  10, "ms2000" },
  {  11, "ms3000" },
  {  12, "ms4000" },
  {  13, "ms6000" },
  {  14, "ms8000" },
  { 0, NULL }
};


static int
dissect_rrc_T_300(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     15, NULL, FALSE, 0, NULL);

  return offset;
}


static const value_string rrc_T_321_vals[] = {
  {   0, "ms100" },
  {   1, "ms200" },
  {   2, "ms400" },
  {   3, "ms800" },
  { 0, NULL }
};


static int
dissect_rrc_T_321(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     4, NULL, FALSE, 0, NULL);

  return offset;
}


static const value_string rrc_T_328_vals[] = {
  {   0, "ms20" },
  {   1, "ms40" },
  {   2, "ms60" },
  {   3, "ms80" },
  { 0, NULL }
};


static int
dissect_rrc_T_328(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     4, NULL, FALSE, 0, NULL);

  return offset;
}


static const value_string rrc_T_329_vals[] = {
  {   0, "s05p" },
  {   1, "s1" },
  {   2, "s2" },
  {   3, "s4" },
  { 0, NULL }
};


static int
dissect_rrc_T_329(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     4, NULL, FALSE, 0, NULL);

  return offset;
}


static const value_string rrc_T_supportOfLoggedMeasurementsIdlePCH_vals[] = {
  {   0, "true" },
  { 0, NULL }
};


static int
dissect_rrc_T_supportOfLoggedMeasurementsIdlePCH(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t UE_BasedNetworkPerformanceMeasurementsParameters_sequence[] = {
  { &hf_rrc_supportOfLoggedMeasurementsIdlePCH, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_supportOfLoggedMeasurementsIdlePCH },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UE_BasedNetworkPerformanceMeasurementsParameters(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UE_BasedNetworkPerformanceMeasurementsParameters, UE_BasedNetworkPerformanceMeasurementsParameters_sequence);

  return offset;
}


static const value_string rrc_T_deviceType_vals[] = {
  {   0, "doesNotBenefitFromBatteryConsumptionOptimisation" },
  { 0, NULL }
};


static int
dissect_rrc_T_deviceType(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t UE_RadioAccessCapability_v690ext_sequence[] = {
  { &hf_rrc_physicalchannelcapability_edch, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PhysicalChannelCapability_edch_r6 },
  { &hf_rrc_deviceType      , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_deviceType },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UE_RadioAccessCapability_v690ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UE_RadioAccessCapability_v690ext, UE_RadioAccessCapability_v690ext_sequence);

  return offset;
}


static const value_string rrc_T_supportOfInter_RAT_PS_Handover_vals[] = {
  {   0, "doesSupporInter-RAT-PS-Handover" },
  { 0, NULL }
};


static int
dissect_rrc_T_supportOfInter_RAT_PS_Handover(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}



static int
dissect_rrc_GSM_MS_RadioAccessCapability(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
#line 587 "../../asn1/rrc/rrc.cnf"
  tvbuff_t *parameter_tvb=NULL;

  offset = dissect_per_octet_string(tvb, offset, actx, tree, hf_index,
                                       1, 64, FALSE, &parameter_tvb);

  if (parameter_tvb){
    de_gmm_ms_radio_acc_cap(parameter_tvb, tree, actx->pinfo, 0, tvb_length(parameter_tvb), NULL, 0);
  }



  return offset;
}


static const per_sequence_t InterRAT_UE_RadioAccessCapability_v690ext_sequence[] = {
  { &hf_rrc_supportOfInter_RAT_PS_Handover, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_supportOfInter_RAT_PS_Handover },
  { &hf_rrc_gsm_MS_RadioAccessCapability, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_GSM_MS_RadioAccessCapability },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_InterRAT_UE_RadioAccessCapability_v690ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_InterRAT_UE_RadioAccessCapability_v690ext, InterRAT_UE_RadioAccessCapability_v690ext_sequence);

  return offset;
}


static const value_string rrc_T_supportForSIB11bis_01_vals[] = {
  {   0, "true" },
  { 0, NULL }
};


static int
dissect_rrc_T_supportForSIB11bis_01(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t UE_RadioAccessCapability_v6b0ext_IEs_sequence[] = {
  { &hf_rrc_supportForSIB11bis_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_supportForSIB11bis_01 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UE_RadioAccessCapability_v6b0ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UE_RadioAccessCapability_v6b0ext_IEs, UE_RadioAccessCapability_v6b0ext_IEs_sequence);

  return offset;
}


static const value_string rrc_T_supportForFDPCH_02_vals[] = {
  {   0, "true" },
  { 0, NULL }
};


static int
dissect_rrc_T_supportForFDPCH_02(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t UE_RadioAccessCapability_v6e0ext_IEs_sequence[] = {
  { &hf_rrc_supportForFDPCH_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_supportForFDPCH_02 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UE_RadioAccessCapability_v6e0ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UE_RadioAccessCapability_v6e0ext_IEs, UE_RadioAccessCapability_v6e0ext_IEs_sequence);

  return offset;
}


static const per_sequence_t UE_GANSSPositioning_Capability_sequence[] = {
  { &hf_rrc_networkAssistedGANSS_supportedList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_NetworkAssistedGANSS_Supported_List },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UE_GANSSPositioning_Capability(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UE_GANSSPositioning_Capability, UE_GANSSPositioning_Capability_sequence);

  return offset;
}


static const per_sequence_t UE_PositioningCapability_v770ext_sequence[] = {
  { &hf_rrc_ue_GANSSPositioning_Capability, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_GANSSPositioning_Capability },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UE_PositioningCapability_v770ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UE_PositioningCapability_v770ext, UE_PositioningCapability_v770ext_sequence);

  return offset;
}


static const value_string rrc_T_mac_ehsSupport_01_vals[] = {
  {   0, "true" },
  { 0, NULL }
};


static int
dissect_rrc_T_mac_ehsSupport_01(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t UE_RadioAccessCapability_v770ext_IEs_sequence[] = {
  { &hf_rrc_pdcp_Capability_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_PDCP_Capability_v770ext },
  { &hf_rrc_rlc_Capability  , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RLC_Capability_v770ext },
  { &hf_rrc_rf_Capability   , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RF_Capability_v770ext },
  { &hf_rrc_physicalChannelCapability, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PhysicalChannelCapability_v770ext },
  { &hf_rrc_multiModeRAT_Capability, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MultiModeRAT_Capability_v770ext },
  { &hf_rrc_ue_PositioningCapability, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UE_PositioningCapability_v770ext },
  { &hf_rrc_mac_ehsSupport_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_mac_ehsSupport_01 },
  { &hf_rrc_ue_specificCapabilityInformation, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_SpecificCapabilityInformation_LCRTDD },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UE_RadioAccessCapability_v770ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UE_RadioAccessCapability_v770ext_IEs, UE_RadioAccessCapability_v770ext_IEs_sequence);

  return offset;
}


static const value_string rrc_T_supportForEDPCCHPowerBoosting_01_vals[] = {
  {   0, "true" },
  { 0, NULL }
};


static int
dissect_rrc_T_supportForEDPCCHPowerBoosting_01(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t UE_RadioAccessCapability_v790ext_IEs_sequence[] = {
  { &hf_rrc_supportForEDPCCHPowerBoosting_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_supportForEDPCCHPowerBoosting_01 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UE_RadioAccessCapability_v790ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UE_RadioAccessCapability_v790ext_IEs, UE_RadioAccessCapability_v790ext_IEs_sequence);

  return offset;
}


static const per_sequence_t UE_GANSSPositioning_Capability_v860ext_IEs_sequence[] = {
  { &hf_rrc_networkAssistedGANSS_supportedList_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_NetworkAssistedGANSS_Supported_List_v860ext },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UE_GANSSPositioning_Capability_v860ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UE_GANSSPositioning_Capability_v860ext_IEs, UE_GANSSPositioning_Capability_v860ext_IEs_sequence);

  return offset;
}


static const per_sequence_t UE_PositioningCapability_v860ext_sequence[] = {
  { &hf_rrc_ue_GANSSPositioning_Capability_v860ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_GANSSPositioning_Capability_v860ext_IEs },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UE_PositioningCapability_v860ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UE_PositioningCapability_v860ext, UE_PositioningCapability_v860ext_sequence);

  return offset;
}


static const value_string rrc_T_supportOfCommonEDCH_03_vals[] = {
  {   0, "true" },
  { 0, NULL }
};


static int
dissect_rrc_T_supportOfCommonEDCH_03(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}


static const value_string rrc_T_supportOfMACiis_03_vals[] = {
  {   0, "true" },
  { 0, NULL }
};


static int
dissect_rrc_T_supportOfMACiis_03(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}


static const value_string rrc_T_supportOfSPSOperation_02_vals[] = {
  {   0, "true" },
  { 0, NULL }
};


static int
dissect_rrc_T_supportOfSPSOperation_02(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}


static const value_string rrc_T_supportOfControlChannelDRXOperation_01_vals[] = {
  {   0, "true" },
  { 0, NULL }
};


static int
dissect_rrc_T_supportOfControlChannelDRXOperation_01(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}


static const value_string rrc_T_supportOfCSG_01_vals[] = {
  {   0, "true" },
  { 0, NULL }
};


static int
dissect_rrc_T_supportOfCSG_01(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t UE_RadioAccessCapability_v860ext_IEs_sequence[] = {
  { &hf_rrc_rf_Capability_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RF_Capability_v860ext },
  { &hf_rrc_physicalChannelCapability_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PhysicalChannelCapability_v860ext },
  { &hf_rrc_multiModeRAT_Capability_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MultiModeRAT_Capability_v860ext },
  { &hf_rrc_ue_PositioningCapability_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UE_PositioningCapability_v860ext },
  { &hf_rrc_measurementCapability, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MeasurementCapability_v860ext },
  { &hf_rrc_measurementCapabilityTDD, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MeasurementCapabilityTDD },
  { &hf_rrc_supportOfCommonEDCH_03, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_supportOfCommonEDCH_03 },
  { &hf_rrc_supportOfMACiis_03, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_supportOfMACiis_03 },
  { &hf_rrc_supportOfSPSOperation_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_supportOfSPSOperation_02 },
  { &hf_rrc_supportOfControlChannelDRXOperation_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_supportOfControlChannelDRXOperation_01 },
  { &hf_rrc_supportOfCSG_01 , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_supportOfCSG_01 },
  { &hf_rrc_ue_RadioAccessCapabBandFDDList3, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_RadioAccessCapabBandFDDList3 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UE_RadioAccessCapability_v860ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UE_RadioAccessCapability_v860ext_IEs, UE_RadioAccessCapability_v860ext_IEs_sequence);

  return offset;
}



static int
dissect_rrc_T_ue_EUTRA_Capability(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
#line 603 "../../asn1/rrc/rrc.cnf"
  tvbuff_t *ue_eutra_cap_tvb = NULL;
  offset = dissect_per_octet_string(tvb, offset, actx, tree, hf_index,
                                       NO_BOUND, NO_BOUND, FALSE, &ue_eutra_cap_tvb);

  if (ue_eutra_cap_tvb && lte_rrc_ue_eutra_cap_handle)
    call_dissector(lte_rrc_ue_eutra_cap_handle, ue_eutra_cap_tvb, actx->pinfo, tree);



  return offset;
}


static const per_sequence_t EUTRA_RadioAccessCapability_sequence[] = {
  { &hf_rrc_ue_EUTRA_Capability, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_ue_EUTRA_Capability },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_EUTRA_RadioAccessCapability(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_EUTRA_RadioAccessCapability, EUTRA_RadioAccessCapability_sequence);

  return offset;
}


static const per_sequence_t InterRAT_UE_RadioAccessCapability_v860ext_sequence[] = {
  { &hf_rrc_eutra_RadioAccessCapability, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_EUTRA_RadioAccessCapability },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_InterRAT_UE_RadioAccessCapability_v860ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_InterRAT_UE_RadioAccessCapability_v860ext, InterRAT_UE_RadioAccessCapability_v860ext_sequence);

  return offset;
}


static const value_string rrc_T_supportForPriorityReselectionInUTRAN_vals[] = {
  {   0, "true" },
  { 0, NULL }
};


static int
dissect_rrc_T_supportForPriorityReselectionInUTRAN(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t UE_RadioAccessCapability_v880ext_IEs_sequence[] = {
  { &hf_rrc_supportForPriorityReselectionInUTRAN, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_supportForPriorityReselectionInUTRAN },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UE_RadioAccessCapability_v880ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UE_RadioAccessCapability_v880ext_IEs, UE_RadioAccessCapability_v880ext_IEs_sequence);

  return offset;
}


static const value_string rrc_T_supportCellSpecificTxDiversityinDC_Operation_vals[] = {
  {   0, "true" },
  { 0, NULL }
};


static int
dissect_rrc_T_supportCellSpecificTxDiversityinDC_Operation(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t UE_RadioAccessCapability_v890ext_IEs_sequence[] = {
  { &hf_rrc_supportCellSpecificTxDiversityinDC_Operation, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_supportCellSpecificTxDiversityinDC_Operation },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UE_RadioAccessCapability_v890ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UE_RadioAccessCapability_v890ext_IEs, UE_RadioAccessCapability_v890ext_IEs_sequence);

  return offset;
}


static const per_sequence_t UE_RadioAccessCapabBandCombList_sequence_of[1] = {
  { &hf_rrc_UE_RadioAccessCapabBandCombList_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BandComb },
};

static int
dissect_rrc_UE_RadioAccessCapabBandCombList(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_UE_RadioAccessCapabBandCombList, UE_RadioAccessCapabBandCombList_sequence_of,
                                                  1, 16, FALSE);

  return offset;
}


static const value_string rrc_T_supportOfenhancedTS0_vals[] = {
  {   0, "true" },
  { 0, NULL }
};


static int
dissect_rrc_T_supportOfenhancedTS0(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t UE_RadioAccessCapability_v920ext_IEs_sequence[] = {
  { &hf_rrc_ue_RadioAccessCapabBandCombList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_RadioAccessCapabBandCombList },
  { &hf_rrc_physicalChannelCapability_02, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PhysicalChannelCapability_v920ext },
  { &hf_rrc_rlc_Capability_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RLC_Capability_v920ext },
  { &hf_rrc_supportOfenhancedTS0, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_supportOfenhancedTS0 },
  { &hf_rrc_measurementCapability_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MeasurementCapability_v920ext },
  { &hf_rrc_csgProximityIndicationCapability, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CSG_ProximityIndicationCapability },
  { &hf_rrc_neighCellSI_AcquisitionCapability, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_NeighCellSI_AcquisitionCapability },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UE_RadioAccessCapability_v920ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UE_RadioAccessCapability_v920ext_IEs, UE_RadioAccessCapability_v920ext_IEs_sequence);

  return offset;
}


static const value_string rrc_T_ue_ExtendedMeasurementSupport_vals[] = {
  {   0, "true" },
  { 0, NULL }
};


static int
dissect_rrc_T_ue_ExtendedMeasurementSupport(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t UE_RadioAccessCapability_v970ext_IEs_sequence[] = {
  { &hf_rrc_ue_ExtendedMeasurementSupport, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_ue_ExtendedMeasurementSupport },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UE_RadioAccessCapability_v970ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UE_RadioAccessCapability_v970ext_IEs, UE_RadioAccessCapability_v970ext_IEs_sequence);

  return offset;
}


static const per_sequence_t SupportedCarrierCombination_sequence[] = {
  { &hf_rrc_carrierCombination12, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { &hf_rrc_carrierCombination21, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { &hf_rrc_carrierCombination13, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { &hf_rrc_carrierCombination31, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { &hf_rrc_carrierCombination22, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_SupportedCarrierCombination(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_SupportedCarrierCombination, SupportedCarrierCombination_sequence);

  return offset;
}


static const per_sequence_t UE_RadioAccessCapabBandCombList_va40ext_sequence_of[1] = {
  { &hf_rrc_UE_RadioAccessCapabBandCombList_va40ext_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SupportedCarrierCombination },
};

static int
dissect_rrc_UE_RadioAccessCapabBandCombList_va40ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_UE_RadioAccessCapabBandCombList_va40ext, UE_RadioAccessCapabBandCombList_va40ext_sequence_of,
                                                  1, 16, FALSE);

  return offset;
}


static const value_string rrc_T_supportOfMUMIMO_vals[] = {
  {   0, "uplink" },
  {   1, "uplink-and-downlink" },
  { 0, NULL }
};


static int
dissect_rrc_T_supportOfMUMIMO(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     2, NULL, FALSE, 0, NULL);

  return offset;
}


static const value_string rrc_T_supportForDualCellMimoInDifferentBands_vals[] = {
  {   0, "true" },
  { 0, NULL }
};


static int
dissect_rrc_T_supportForDualCellMimoInDifferentBands(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}


static const value_string rrc_T_supportOfUTRANANR_vals[] = {
  {   0, "true" },
  { 0, NULL }
};


static int
dissect_rrc_T_supportOfUTRANANR(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t UE_RadioAccessCapability_va40ext_IEs_sequence[] = {
  { &hf_rrc_ue_RadioAccessCapabBandCombList_va40ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_RadioAccessCapabBandCombList_va40ext },
  { &hf_rrc_physicalChannelCapability_03, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PhysicalChannelCapability_va40ext },
  { &hf_rrc_rlc_Capability_02, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RLC_Capability_va40ext },
  { &hf_rrc_measurementCapability_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MeasurementCapability_va40ext },
  { &hf_rrc_supportOfMUMIMO , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_supportOfMUMIMO },
  { &hf_rrc_supportForDualCellMimoInDifferentBands, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_supportForDualCellMimoInDifferentBands },
  { &hf_rrc_ue_BasedNetworkPerformanceMeasurementsParameters, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_BasedNetworkPerformanceMeasurementsParameters },
  { &hf_rrc_supportOfUTRANANR, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_supportOfUTRANANR },
  { &hf_rrc_ue_RadioAccessCapabBandFDDList4, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_RadioAccessCapabBandFDDList4 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UE_RadioAccessCapability_va40ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UE_RadioAccessCapability_va40ext_IEs, UE_RadioAccessCapability_va40ext_IEs_sequence);

  return offset;
}


static const per_sequence_t UE_RadioAccessCapability_va60ext_IEs_sequence[] = {
  { &hf_rrc_ue_RadioAccessCapabBandFDDList4_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_RadioAccessCapabBandFDDList4_va60ext },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UE_RadioAccessCapability_va60ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UE_RadioAccessCapability_va60ext_IEs, UE_RadioAccessCapability_va60ext_IEs_sequence);

  return offset;
}


static const per_sequence_t UE_RadioAccessCapability_va80ext_IEs_sequence[] = {
  { &hf_rrc_ue_RadioAccessCapabBandFDDList4_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_RadioAccessCapabBandFDDList4_va80ext },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UE_RadioAccessCapability_va80ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UE_RadioAccessCapability_va80ext_IEs, UE_RadioAccessCapability_va80ext_IEs_sequence);

  return offset;
}


static const value_string rrc_T_voiceOverUTRA_PS_HS_Support_vals[] = {
  {   0, "true" },
  { 0, NULL }
};


static int
dissect_rrc_T_voiceOverUTRA_PS_HS_Support(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}


static const value_string rrc_T_srvcc_SupportFromUTRA_to_UTRA_vals[] = {
  {   0, "true" },
  { 0, NULL }
};


static int
dissect_rrc_T_srvcc_SupportFromUTRA_to_UTRA(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}


static const value_string rrc_T_srvcc_SupportFromUTRA_to_GERAN_vals[] = {
  {   0, "true" },
  { 0, NULL }
};


static int
dissect_rrc_T_srvcc_SupportFromUTRA_to_GERAN(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}


static const value_string rrc_T_ue_supportMac_ehsWindowSizeExtension_vals[] = {
  {   0, "true" },
  { 0, NULL }
};


static int
dissect_rrc_T_ue_supportMac_ehsWindowSizeExtension(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t UE_RadioAccessCapability_v9c0ext_IEs_sequence[] = {
  { &hf_rrc_voiceOverUTRA_PS_HS_Support, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_voiceOverUTRA_PS_HS_Support },
  { &hf_rrc_srvcc_SupportFromUTRA_to_UTRA, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_srvcc_SupportFromUTRA_to_UTRA },
  { &hf_rrc_srvcc_SupportFromUTRA_to_GERAN, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_srvcc_SupportFromUTRA_to_GERAN },
  { &hf_rrc_ue_supportMac_ehsWindowSizeExtension, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_ue_supportMac_ehsWindowSizeExtension },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UE_RadioAccessCapability_v9c0ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UE_RadioAccessCapability_v9c0ext_IEs, UE_RadioAccessCapability_v9c0ext_IEs_sequence);

  return offset;
}


static const value_string rrc_T_supportOfUMRLCre_establish_via_reconfig_vals[] = {
  {   0, "true" },
  { 0, NULL }
};


static int
dissect_rrc_T_supportOfUMRLCre_establish_via_reconfig(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t UE_RadioAccessCapability_vaa0ext_IEs_sequence[] = {
  { &hf_rrc_rf_capability   , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RF_Capability_vaa0ext },
  { &hf_rrc_supportOfUMRLCre_establish_via_reconfig, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_supportOfUMRLCre_establish_via_reconfig },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UE_RadioAccessCapability_vaa0ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UE_RadioAccessCapability_vaa0ext_IEs, UE_RadioAccessCapability_vaa0ext_IEs_sequence);

  return offset;
}


static const value_string rrc_T_supportOfCommonERGCH_vals[] = {
  {   0, "true" },
  { 0, NULL }
};


static int
dissect_rrc_T_supportOfCommonERGCH(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}


static const value_string rrc_T_supportOfFallbackR99PRACH_vals[] = {
  {   0, "true" },
  { 0, NULL }
};


static int
dissect_rrc_T_supportOfFallbackR99PRACH(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}


static const value_string rrc_T_supportOfConcurrent2ms10msTTI_vals[] = {
  {   0, "true" },
  { 0, NULL }
};


static int
dissect_rrc_T_supportOfConcurrent2ms10msTTI(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}


static const value_string rrc_T_supportOfTTIalignmentAndPerHARQActivation_vals[] = {
  {   0, "true" },
  { 0, NULL }
};


static int
dissect_rrc_T_supportOfTTIalignmentAndPerHARQActivation(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t SupportedCarrierCombination_ext_sequence[] = {
  { &hf_rrc_carrierCombination14, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { &hf_rrc_carrierCombination41, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { &hf_rrc_carrierCombination15, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { &hf_rrc_carrierCombination51, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { &hf_rrc_carrierCombination16, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { &hf_rrc_carrierCombination61, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { &hf_rrc_carrierCombination17, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { &hf_rrc_carrierCombination71, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { &hf_rrc_carrierCombination23, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { &hf_rrc_carrierCombination32, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { &hf_rrc_carrierCombination24, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { &hf_rrc_carrierCombination42, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { &hf_rrc_carrierCombination25, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { &hf_rrc_carrierCombination52, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { &hf_rrc_carrierCombination26, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { &hf_rrc_carrierCombination62, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { &hf_rrc_carrierCombination33, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { &hf_rrc_carrierCombination34, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { &hf_rrc_carrierCombination43, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { &hf_rrc_carrierCombination44, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { &hf_rrc_carrierCombination35, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { &hf_rrc_carrierCombination53, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_SupportedCarrierCombination_ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_SupportedCarrierCombination_ext, SupportedCarrierCombination_ext_sequence);

  return offset;
}


static const per_sequence_t UE_RadioAccessCapabBandCombList_vbxyext_sequence_of[1] = {
  { &hf_rrc_UE_RadioAccessCapabBandCombList_vbxyext_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SupportedCarrierCombination_ext },
};

static int
dissect_rrc_UE_RadioAccessCapabBandCombList_vbxyext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_UE_RadioAccessCapabBandCombList_vbxyext, UE_RadioAccessCapabBandCombList_vbxyext_sequence_of,
                                                  1, 16, FALSE);

  return offset;
}


static const value_string rrc_T_rsrvcc_SupportFromUTRA_CS_HS_to_EUTRA_FDD_vals[] = {
  {   0, "true" },
  { 0, NULL }
};


static int
dissect_rrc_T_rsrvcc_SupportFromUTRA_CS_HS_to_EUTRA_FDD(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}


static const value_string rrc_T_rsrvcc_SupportFromUTRA_CS_HS_to_EUTRA_TDD_vals[] = {
  {   0, "true" },
  { 0, NULL }
};


static int
dissect_rrc_T_rsrvcc_SupportFromUTRA_CS_HS_to_EUTRA_TDD(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t UE_RadioAccessCapability_vbxyext_IEs_sequence[] = {
  { &hf_rrc_multiModeRAT_Capability_02, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MultiModeRAT_Capability_vbxyext },
  { &hf_rrc_supportOfCommonERGCH, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_supportOfCommonERGCH },
  { &hf_rrc_supportOfFallbackR99PRACH, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_supportOfFallbackR99PRACH },
  { &hf_rrc_supportOfConcurrent2ms10msTTI, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_supportOfConcurrent2ms10msTTI },
  { &hf_rrc_supportOfTTIalignmentAndPerHARQActivation, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_supportOfTTIalignmentAndPerHARQActivation },
  { &hf_rrc_ue_RadioAccessCapabBandCombList_vbxyext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_RadioAccessCapabBandCombList_vbxyext },
  { &hf_rrc_multiflowBandCombList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MultiflowBandCombList },
  { &hf_rrc_physicalChannelCapability_04, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PhysicalChannelCapability_vbxyext },
  { &hf_rrc_rlc_Capability_03, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RLC_Capability_vbxyext },
  { &hf_rrc_multiflowCapability, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MultiflowCapability },
  { &hf_rrc_ue_RadioAccessCapabBandFDDList6, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_RadioAccessCapabBandFDDList6 },
  { &hf_rrc_measurementCapability_03, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MeasurementCapability_vbxyext },
  { &hf_rrc_rsrvcc_SupportFromUTRA_CS_HS_to_EUTRA_FDD, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_rsrvcc_SupportFromUTRA_CS_HS_to_EUTRA_FDD },
  { &hf_rrc_rsrvcc_SupportFromUTRA_CS_HS_to_EUTRA_TDD, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_rsrvcc_SupportFromUTRA_CS_HS_to_EUTRA_TDD },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UE_RadioAccessCapability_vbxyext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UE_RadioAccessCapability_vbxyext_IEs, UE_RadioAccessCapability_vbxyext_IEs_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions_194_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_nonCriticalExtensions_194(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_nonCriticalExtensions_194, T_nonCriticalExtensions_194_sequence);

  return offset;
}


static const per_sequence_t T_vbxyNonCriticalExtensions_18_sequence[] = {
  { &hf_rrc_ue_RadioAccessCapability_vbxyext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UE_RadioAccessCapability_vbxyext_IEs },
  { &hf_rrc_nonCriticalExtensions_194, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_nonCriticalExtensions_194 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_vbxyNonCriticalExtensions_18(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_vbxyNonCriticalExtensions_18, T_vbxyNonCriticalExtensions_18_sequence);

  return offset;
}


static const per_sequence_t T_vaa0NonCriticalExtensions_02_sequence[] = {
  { &hf_rrc_ue_RadioAccessCapability_vaa0ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UE_RadioAccessCapability_vaa0ext_IEs },
  { &hf_rrc_vbxyNonCriticalExtensions_18, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_vbxyNonCriticalExtensions_18 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_vaa0NonCriticalExtensions_02(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_vaa0NonCriticalExtensions_02, T_vaa0NonCriticalExtensions_02_sequence);

  return offset;
}


static const per_sequence_t T_v9c0NonCriticalExtensions_12_sequence[] = {
  { &hf_rrc_ue_RadioAccessCapability_v9c0ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UE_RadioAccessCapability_v9c0ext_IEs },
  { &hf_rrc_vaa0NonCriticalExtensions_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_vaa0NonCriticalExtensions_02 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v9c0NonCriticalExtensions_12(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v9c0NonCriticalExtensions_12, T_v9c0NonCriticalExtensions_12_sequence);

  return offset;
}


static const per_sequence_t UE_RadioAccessCapability_LaterNonCriticalExtensions_sequence[] = {
  { &hf_rrc_v9c0NonCriticalExtensions_12, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_v9c0NonCriticalExtensions_12 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UE_RadioAccessCapability_LaterNonCriticalExtensions(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UE_RadioAccessCapability_LaterNonCriticalExtensions, UE_RadioAccessCapability_LaterNonCriticalExtensions_sequence);

  return offset;
}


static const per_sequence_t T_va80NonCriticalExtensions_01_sequence[] = {
  { &hf_rrc_ue_RadioAccessCapability_va80ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UE_RadioAccessCapability_va80ext_IEs },
  { &hf_rrc_laterNonCriticalExtensions_67, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_RadioAccessCapability_LaterNonCriticalExtensions },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_va80NonCriticalExtensions_01(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_va80NonCriticalExtensions_01, T_va80NonCriticalExtensions_01_sequence);

  return offset;
}


static const per_sequence_t T_va60NonCriticalExtensions_sequence[] = {
  { &hf_rrc_ue_RadioAccessCapability_va60ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UE_RadioAccessCapability_va60ext_IEs },
  { &hf_rrc_va80NonCriticalExtensions_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_va80NonCriticalExtensions_01 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_va60NonCriticalExtensions(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_va60NonCriticalExtensions, T_va60NonCriticalExtensions_sequence);

  return offset;
}


static const per_sequence_t T_va40NonCriticalExtensions_17_sequence[] = {
  { &hf_rrc_ue_RadioAccessCapability_va40ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UE_RadioAccessCapability_va40ext_IEs },
  { &hf_rrc_va60NonCriticalExtensions, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_va60NonCriticalExtensions },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_va40NonCriticalExtensions_17(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_va40NonCriticalExtensions_17, T_va40NonCriticalExtensions_17_sequence);

  return offset;
}


static const per_sequence_t T_v970NonCriticalExtensions_03_sequence[] = {
  { &hf_rrc_ue_RadioAccessCapability_v970ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UE_RadioAccessCapability_v970ext_IEs },
  { &hf_rrc_va40NonCriticalExtensions_18, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_va40NonCriticalExtensions_17 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v970NonCriticalExtensions_03(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v970NonCriticalExtensions_03, T_v970NonCriticalExtensions_03_sequence);

  return offset;
}


static const per_sequence_t T_v920NonCriticalExtensions_03_sequence[] = {
  { &hf_rrc_ue_RadioAccessCapability_v920ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UE_RadioAccessCapability_v920ext_IEs },
  { &hf_rrc_v970NonCriticalExtensions_03, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v970NonCriticalExtensions_03 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v920NonCriticalExtensions_03(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v920NonCriticalExtensions_03, T_v920NonCriticalExtensions_03_sequence);

  return offset;
}


static const per_sequence_t T_v890NonCriticalExtensions_10_sequence[] = {
  { &hf_rrc_ue_RadioAccessCapability_v890ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UE_RadioAccessCapability_v890ext_IEs },
  { &hf_rrc_v920NonCriticalExtensions_03, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v920NonCriticalExtensions_03 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v890NonCriticalExtensions_10(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v890NonCriticalExtensions_10, T_v890NonCriticalExtensions_10_sequence);

  return offset;
}


static const per_sequence_t T_v880NonCriticalExtensions_01_sequence[] = {
  { &hf_rrc_ue_RadioAccessCapability_v880ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UE_RadioAccessCapability_v880ext_IEs },
  { &hf_rrc_v890NonCriticalExtensions_10, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v890NonCriticalExtensions_10 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v880NonCriticalExtensions_01(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v880NonCriticalExtensions_01, T_v880NonCriticalExtensions_01_sequence);

  return offset;
}


static const per_sequence_t T_v860NonCriticalExtensions_25_sequence[] = {
  { &hf_rrc_ue_RadioAccessCapability_v860ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UE_RadioAccessCapability_v860ext_IEs },
  { &hf_rrc_ue_RATSpecificCapability_v860ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_InterRAT_UE_RadioAccessCapability_v860ext },
  { &hf_rrc_v880NonCriticalExtensions_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v880NonCriticalExtensions_01 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v860NonCriticalExtensions_25(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v860NonCriticalExtensions_25, T_v860NonCriticalExtensions_25_sequence);

  return offset;
}


static const per_sequence_t T_v790NonCriticalExtensions_01_sequence[] = {
  { &hf_rrc_ue_RadioAccessCapability_v790ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UE_RadioAccessCapability_v790ext_IEs },
  { &hf_rrc_v860NonCriticalExtensions_25, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v860NonCriticalExtensions_25 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v790NonCriticalExtensions_01(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v790NonCriticalExtensions_01, T_v790NonCriticalExtensions_01_sequence);

  return offset;
}


static const per_sequence_t T_v770NonCriticalExtensions_39_sequence[] = {
  { &hf_rrc_ue_RadioAccessCapability_v770ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UE_RadioAccessCapability_v770ext_IEs },
  { &hf_rrc_v790NonCriticalExtensions_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v790NonCriticalExtensions_01 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v770NonCriticalExtensions_39(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v770NonCriticalExtensions_39, T_v770NonCriticalExtensions_39_sequence);

  return offset;
}


static const per_sequence_t T_v6e0NonCriticalExtensions_02_sequence[] = {
  { &hf_rrc_ue_RadioAccessCapability_v6e0ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UE_RadioAccessCapability_v6e0ext_IEs },
  { &hf_rrc_v770NonCriticalExtensions_39, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v770NonCriticalExtensions_39 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v6e0NonCriticalExtensions_02(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v6e0NonCriticalExtensions_02, T_v6e0NonCriticalExtensions_02_sequence);

  return offset;
}


static const per_sequence_t T_v6b0NonCriticalExtensions_14_sequence[] = {
  { &hf_rrc_ue_RadioAccessCapability_v6b0ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UE_RadioAccessCapability_v6b0ext_IEs },
  { &hf_rrc_v6e0NonCriticalExtensions_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v6e0NonCriticalExtensions_02 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v6b0NonCriticalExtensions_14(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v6b0NonCriticalExtensions_14, T_v6b0NonCriticalExtensions_14_sequence);

  return offset;
}


static const per_sequence_t UE_CapabilityContainer_IEs_sequence[] = {
  { &hf_rrc_ue_RadioAccessCapability_v690ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UE_RadioAccessCapability_v690ext },
  { &hf_rrc_ue_RATSpecificCapability_v690ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_InterRAT_UE_RadioAccessCapability_v690ext },
  { &hf_rrc_v6b0NonCriticalExtensions_15, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v6b0NonCriticalExtensions_14 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UE_CapabilityContainer_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UE_CapabilityContainer_IEs, UE_CapabilityContainer_IEs_sequence);

  return offset;
}


static const per_sequence_t UE_IdleTimersAndConstants_sequence[] = {
  { &hf_rrc_t_300           , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_300 },
  { &hf_rrc_n_300           , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_N_300 },
  { &hf_rrc_t_312           , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_312 },
  { &hf_rrc_n_312           , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_N_312 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UE_IdleTimersAndConstants(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UE_IdleTimersAndConstants, UE_IdleTimersAndConstants_sequence);

  return offset;
}


static const per_sequence_t UE_IdleTimersAndConstants_v3a0ext_sequence[] = {
  { &hf_rrc_n_312_01        , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_N_312ext },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UE_IdleTimersAndConstants_v3a0ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UE_IdleTimersAndConstants_v3a0ext, UE_IdleTimersAndConstants_v3a0ext_sequence);

  return offset;
}


static const per_sequence_t UE_RadioAccessCapabilityInfo_sequence[] = {
  { &hf_rrc_ue_RadioAccessCapability_02, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UE_RadioAccessCapability },
  { &hf_rrc_ue_RadioAccessCapability_v370ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UE_RadioAccessCapability_v370ext },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UE_RadioAccessCapabilityInfo(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UE_RadioAccessCapabilityInfo, UE_RadioAccessCapabilityInfo_sequence);

  return offset;
}


static const per_sequence_t CommonRBMappingInfo_sequence[] = {
  { &hf_rrc_logicalChannelIdentity, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_LogicalChannelIdentity },
  { &hf_rrc_mac_ehs_QueueId , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MAC_ehs_QueueId },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_CommonRBMappingInfo(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_CommonRBMappingInfo, CommonRBMappingInfo_sequence);

  return offset;
}


static const per_sequence_t PredefinedRB_Configuration_sequence[] = {
  { &hf_rrc_re_EstablishmentTimer, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_Re_EstablishmentTimer },
  { &hf_rrc_srb_InformationList, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SRB_InformationSetupList },
  { &hf_rrc_rb_InformationList, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RB_InformationSetupList },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_PredefinedRB_Configuration(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_PredefinedRB_Configuration, PredefinedRB_Configuration_sequence);

  return offset;
}


static const per_sequence_t PreDefTransChConfiguration_sequence[] = {
  { &hf_rrc_ul_CommonTransChInfo, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UL_CommonTransChInfo },
  { &hf_rrc_ul_AddReconfTrChInfoList, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UL_AddReconfTransChInfoList },
  { &hf_rrc_dl_CommonTransChInfo, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DL_CommonTransChInfo },
  { &hf_rrc_dl_TrChInfoList , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DL_AddReconfTransChInfoList },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_PreDefTransChConfiguration(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_PreDefTransChConfiguration, PreDefTransChConfiguration_sequence);

  return offset;
}


static const per_sequence_t T_fdd_127_sequence[] = {
  { &hf_rrc_powerControlAlgorithm, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PowerControlAlgorithm },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_fdd_127(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_fdd_127, T_fdd_127_sequence);

  return offset;
}


static const per_sequence_t T_tdd_77_sequence[] = {
  { &hf_rrc_dpch_ConstantValue, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_ConstantValueTdd },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_tdd_77(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_tdd_77, T_tdd_77_sequence);

  return offset;
}


static const value_string rrc_UL_DPCH_PowerControlInfoPredef_vals[] = {
  {   0, "fdd" },
  {   1, "tdd" },
  { 0, NULL }
};

static const per_choice_t UL_DPCH_PowerControlInfoPredef_choice[] = {
  {   0, &hf_rrc_fdd_130         , ASN1_NO_EXTENSIONS     , dissect_rrc_T_fdd_127 },
  {   1, &hf_rrc_tdd_83          , ASN1_NO_EXTENSIONS     , dissect_rrc_T_tdd_77 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_UL_DPCH_PowerControlInfoPredef(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_UL_DPCH_PowerControlInfoPredef, UL_DPCH_PowerControlInfoPredef_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t T_fdd_119_sequence[] = {
  { &hf_rrc_tfci_Existence  , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { &hf_rrc_puncturingLimit , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PuncturingLimit },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_fdd_119(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_fdd_119, T_fdd_119_sequence);

  return offset;
}


static const per_sequence_t T_tdd_70_sequence[] = {
  { &hf_rrc_commonTimeslotInfo, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CommonTimeslotInfo },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_tdd_70(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_tdd_70, T_tdd_70_sequence);

  return offset;
}


static const value_string rrc_T_modeSpecificInfo_87_vals[] = {
  {   0, "fdd" },
  {   1, "tdd" },
  { 0, NULL }
};

static const per_choice_t T_modeSpecificInfo_87_choice[] = {
  {   0, &hf_rrc_fdd_122         , ASN1_NO_EXTENSIONS     , dissect_rrc_T_fdd_119 },
  {   1, &hf_rrc_tdd_76          , ASN1_NO_EXTENSIONS     , dissect_rrc_T_tdd_70 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_modeSpecificInfo_87(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_modeSpecificInfo_87, T_modeSpecificInfo_87_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t UL_DPCH_InfoPredef_sequence[] = {
  { &hf_rrc_ul_DPCH_PowerControlInfo_09, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UL_DPCH_PowerControlInfoPredef },
  { &hf_rrc_modeSpecificInfo_87, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_modeSpecificInfo_87 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UL_DPCH_InfoPredef(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UL_DPCH_InfoPredef, UL_DPCH_InfoPredef_sequence);

  return offset;
}


static const per_sequence_t T_fdd_69_sequence[] = {
  { &hf_rrc_spreadingFactorAndPilot, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SF512_AndPilot },
  { &hf_rrc_positionFixedOrFlexible, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PositionFixedOrFlexible },
  { &hf_rrc_tfci_Existence  , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_fdd_69(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_fdd_69, T_fdd_69_sequence);

  return offset;
}


static const per_sequence_t T_tdd_31_sequence[] = {
  { &hf_rrc_commonTimeslotInfo, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CommonTimeslotInfo },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_tdd_31(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_tdd_31, T_tdd_31_sequence);

  return offset;
}


static const value_string rrc_T_modeSpecificInfo_39_vals[] = {
  {   0, "fdd" },
  {   1, "tdd" },
  { 0, NULL }
};

static const per_choice_t T_modeSpecificInfo_39_choice[] = {
  {   0, &hf_rrc_fdd_70          , ASN1_NO_EXTENSIONS     , dissect_rrc_T_fdd_69 },
  {   1, &hf_rrc_tdd_32          , ASN1_NO_EXTENSIONS     , dissect_rrc_T_tdd_31 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_modeSpecificInfo_39(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_modeSpecificInfo_39, T_modeSpecificInfo_39_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t DL_DPCH_InfoCommonPredef_sequence[] = {
  { &hf_rrc_modeSpecificInfo_39, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_modeSpecificInfo_39 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_DL_DPCH_InfoCommonPredef(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_DL_DPCH_InfoCommonPredef, DL_DPCH_InfoCommonPredef_sequence);

  return offset;
}


static const per_sequence_t DL_CommonInformationPredef_sequence[] = {
  { &hf_rrc_dl_DPCH_InfoCommon_04, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_DPCH_InfoCommonPredef },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_DL_CommonInformationPredef(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_DL_CommonInformationPredef, DL_CommonInformationPredef_sequence);

  return offset;
}


static const per_sequence_t PreDefPhyChConfiguration_sequence[] = {
  { &hf_rrc_ul_DPCH_InfoPredef, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UL_DPCH_InfoPredef },
  { &hf_rrc_dl_CommonInformationPredef, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_CommonInformationPredef },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_PreDefPhyChConfiguration(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_PreDefPhyChConfiguration, PreDefPhyChConfiguration_sequence);

  return offset;
}


static const per_sequence_t PreDefRadioConfiguration_sequence[] = {
  { &hf_rrc_predefinedRB_Configuration, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PredefinedRB_Configuration },
  { &hf_rrc_preDefTransChConfiguration, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PreDefTransChConfiguration },
  { &hf_rrc_preDefPhyChConfiguration, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PreDefPhyChConfiguration },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_PreDefRadioConfiguration(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_PreDefRadioConfiguration, PreDefRadioConfiguration_sequence);

  return offset;
}


static const per_sequence_t T_fdd_120_sequence[] = {
  { &hf_rrc_numberOfTPC_Bits, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_NumberOfTPC_Bits },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_fdd_120(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_fdd_120, T_fdd_120_sequence);

  return offset;
}


static const value_string rrc_T_modeSpecificInfo_88_vals[] = {
  {   0, "fdd" },
  {   1, "tdd" },
  { 0, NULL }
};

static const per_choice_t T_modeSpecificInfo_88_choice[] = {
  {   0, &hf_rrc_fdd_123         , ASN1_NO_EXTENSIONS     , dissect_rrc_T_fdd_120 },
  {   1, &hf_rrc_tdd             , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_modeSpecificInfo_88(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_modeSpecificInfo_88, T_modeSpecificInfo_88_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t UL_DPCH_InfoPredef_v770ext_sequence[] = {
  { &hf_rrc_modeSpecificInfo_88, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_modeSpecificInfo_88 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UL_DPCH_InfoPredef_v770ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UL_DPCH_InfoPredef_v770ext, UL_DPCH_InfoPredef_v770ext_sequence);

  return offset;
}


static const per_sequence_t PreDefPhyChConfiguration_v770ext_sequence[] = {
  { &hf_rrc_ul_DPCH_InfoPredef_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UL_DPCH_InfoPredef_v770ext },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_PreDefPhyChConfiguration_v770ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_PreDefPhyChConfiguration_v770ext, PreDefPhyChConfiguration_v770ext_sequence);

  return offset;
}


static const per_sequence_t PreDefRadioConfiguration_v770ext_sequence[] = {
  { &hf_rrc_preDefPhyChConfiguration_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PreDefPhyChConfiguration_v770ext },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_PreDefRadioConfiguration_v770ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_PreDefRadioConfiguration_v770ext, PreDefRadioConfiguration_v770ext_sequence);

  return offset;
}


static const per_sequence_t PreDefRadioConfiguration_v920ext_sequence[] = {
  { &hf_rrc_mac_ehsWindowSize, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MAC_hs_WindowSize_r9 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_PreDefRadioConfiguration_v920ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_PreDefRadioConfiguration_v920ext, PreDefRadioConfiguration_v920ext_sequence);

  return offset;
}


static const value_string rrc_TimerOSD_r6_vals[] = {
  {   0, "ms40" },
  {   1, "ms80" },
  {   2, "ms120" },
  {   3, "ms160" },
  {   4, "ms240" },
  {   5, "ms320" },
  {   6, "ms480" },
  {   7, "ms640" },
  {   8, "ms960" },
  {   9, "ms1280" },
  {  10, "ms1920" },
  {  11, "ms2560" },
  {  12, "ms3840" },
  {  13, "ms5120" },
  { 0, NULL }
};


static int
dissect_rrc_TimerOSD_r6(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     14, NULL, FALSE, 0, NULL);

  return offset;
}


static const value_string rrc_WindowSizeOSD_r6_vals[] = {
  {   0, "ws8" },
  {   1, "ws16" },
  {   2, "ws32" },
  {   3, "ws40" },
  {   4, "ws48" },
  {   5, "ws56" },
  {   6, "ws64" },
  { 0, NULL }
};


static int
dissect_rrc_WindowSizeOSD_r6(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     7, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t UM_RLC_OutOSeqDelivery_Info_r6_sequence[] = {
  { &hf_rrc_timer_OSD       , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_TimerOSD_r6 },
  { &hf_rrc_windowSize_OSD  , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_WindowSizeOSD_r6 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UM_RLC_OutOSeqDelivery_Info_r6(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UM_RLC_OutOSeqDelivery_Info_r6, UM_RLC_OutOSeqDelivery_Info_r6_sequence);

  return offset;
}


static const per_sequence_t RLC_Info_MCCH_r6_sequence[] = {
  { &hf_rrc_dl_UM_RLC_LI_size, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DL_UM_RLC_LI_size },
  { &hf_rrc_dl_UM_RLC_OutOSeqDelivery_Info, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UM_RLC_OutOSeqDelivery_Info_r6 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_RLC_Info_MCCH_r6(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_RLC_Info_MCCH_r6, RLC_Info_MCCH_r6_sequence);

  return offset;
}


static const per_sequence_t Common_E_DCH_MAC_d_Flow_sequence[] = {
  { &hf_rrc_mac_d_FlowIdentity, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_E_DCH_MAC_d_FlowIdentity },
  { &hf_rrc_mac_d_FlowPowerOffset, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_E_DCH_MAC_d_FlowPowerOffset },
  { &hf_rrc_mac_d_FlowMaxRetrans, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_E_DCH_MAC_d_FlowMaxRetrans },
  { &hf_rrc_mac_d_FlowMultiplexingList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_E_DCH_MAC_d_FlowMultiplexingList },
  { &hf_rrc_e_dch_mac_d_flow_retransmission_timer, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_E_DCH_MAC_d_FlowRetransTimer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_Common_E_DCH_MAC_d_Flow(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_Common_E_DCH_MAC_d_Flow, Common_E_DCH_MAC_d_Flow_sequence);

  return offset;
}


static const per_sequence_t Common_E_DCH_MAC_d_FlowList_sequence_of[1] = {
  { &hf_rrc_Common_E_DCH_MAC_d_FlowList_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_Common_E_DCH_MAC_d_Flow },
};

static int
dissect_rrc_Common_E_DCH_MAC_d_FlowList(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_Common_E_DCH_MAC_d_FlowList, Common_E_DCH_MAC_d_FlowList_sequence_of,
                                                  1, maxE_DCHMACdFlow, FALSE);

  return offset;
}


static const per_sequence_t Common_E_DCH_MAC_d_Flow_Info_ConcurrentTTI_sequence[] = {
  { &hf_rrc_mac_d_FlowIdentity, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_E_DCH_MAC_d_FlowIdentity },
  { &hf_rrc_mac_d_FlowPowerOffset, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_E_DCH_MAC_d_FlowPowerOffset },
  { &hf_rrc_mac_d_FlowMaxRetrans, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_E_DCH_MAC_d_FlowMaxRetrans },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_Common_E_DCH_MAC_d_Flow_Info_ConcurrentTTI(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_Common_E_DCH_MAC_d_Flow_Info_ConcurrentTTI, Common_E_DCH_MAC_d_Flow_Info_ConcurrentTTI_sequence);

  return offset;
}


static const per_sequence_t Common_E_DCH_MAC_d_Flow_Info_List_ConcurrentTTI_sequence_of[1] = {
  { &hf_rrc_Common_E_DCH_MAC_d_Flow_Info_List_ConcurrentTTI_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_Common_E_DCH_MAC_d_Flow_Info_ConcurrentTTI },
};

static int
dissect_rrc_Common_E_DCH_MAC_d_Flow_Info_List_ConcurrentTTI(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_Common_E_DCH_MAC_d_Flow_Info_List_ConcurrentTTI, Common_E_DCH_MAC_d_Flow_Info_List_ConcurrentTTI_sequence_of,
                                                  1, maxE_DCHMACdFlow, FALSE);

  return offset;
}


static const value_string rrc_T_tti_01_vals[] = {
  {   0, "tti5" },
  {   1, "tti10" },
  {   2, "tti20" },
  {   3, "tti40" },
  {   4, "tti80" },
  {   5, "dynamic" },
  { 0, NULL }
};

static const per_choice_t T_tti_01_choice[] = {
  {   0, &hf_rrc_tti5            , ASN1_NO_EXTENSIONS     , dissect_rrc_CommonDynamicTF_InfoList },
  {   1, &hf_rrc_tti10           , ASN1_NO_EXTENSIONS     , dissect_rrc_CommonDynamicTF_InfoList },
  {   2, &hf_rrc_tti20           , ASN1_NO_EXTENSIONS     , dissect_rrc_CommonDynamicTF_InfoList },
  {   3, &hf_rrc_tti40           , ASN1_NO_EXTENSIONS     , dissect_rrc_CommonDynamicTF_InfoList },
  {   4, &hf_rrc_tti80           , ASN1_NO_EXTENSIONS     , dissect_rrc_CommonDynamicTF_InfoList },
  {   5, &hf_rrc_dynamic         , ASN1_NO_EXTENSIONS     , dissect_rrc_CommonDynamicTF_InfoList_DynamicTTI },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_tti_01(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_tti_01, T_tti_01_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t CommonTransChTFS_LCR_sequence[] = {
  { &hf_rrc_tti_01          , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_tti_01 },
  { &hf_rrc_semistaticTF_Information, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SemistaticTF_Information },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_CommonTransChTFS_LCR(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_CommonTransChTFS_LCR, CommonTransChTFS_LCR_sequence);

  return offset;
}


static const per_sequence_t Common_MAC_ehs_ReorderingQueue_sequence[] = {
  { &hf_rrc_mac_ehs_QueueId , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MAC_ehs_QueueId },
  { &hf_rrc_t1_ReleaseTimer , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T1_ReleaseTimer },
  { &hf_rrc_reorderingResetTimer, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_Treset_ResetTimer },
  { &hf_rrc_mac_ehsWindowSize_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MAC_hs_WindowSize },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_Common_MAC_ehs_ReorderingQueue(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_Common_MAC_ehs_ReorderingQueue, Common_MAC_ehs_ReorderingQueue_sequence);

  return offset;
}


static const per_sequence_t Common_MAC_ehs_ReorderingQueueList_sequence_of[1] = {
  { &hf_rrc_Common_MAC_ehs_ReorderingQueueList_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_Common_MAC_ehs_ReorderingQueue },
};

static int
dissect_rrc_Common_MAC_ehs_ReorderingQueueList(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_Common_MAC_ehs_ReorderingQueueList, Common_MAC_ehs_ReorderingQueueList_sequence_of,
                                                  1, maxCommonQueueID, FALSE);

  return offset;
}


static const value_string rrc_T_fixed_TTI_Selection_vals[] = {
  {   0, "tti2" },
  {   1, "tti10" },
  { 0, NULL }
};


static int
dissect_rrc_T_fixed_TTI_Selection(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     2, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t T_fixed_sequence[] = {
  { &hf_rrc_fixed_TTI_Selection, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_fixed_TTI_Selection },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_fixed(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_fixed, T_fixed_sequence);

  return offset;
}


static const per_sequence_t T_threshold_based_sequence[] = {
  { &hf_rrc_common_E_DCH_TTI_Selection_Thresh, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_0_15 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_threshold_based(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_threshold_based, T_threshold_based_sequence);

  return offset;
}


static const value_string rrc_T_tti_selection_vals[] = {
  {   0, "fixed" },
  {   1, "threshold-based" },
  { 0, NULL }
};

static const per_choice_t T_tti_selection_choice[] = {
  {   0, &hf_rrc_fixed           , ASN1_NO_EXTENSIONS     , dissect_rrc_T_fixed },
  {   1, &hf_rrc_threshold_based , ASN1_NO_EXTENSIONS     , dissect_rrc_T_threshold_based },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_tti_selection(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_tti_selection, T_tti_selection_choice,
                                 NULL);

  return offset;
}



static int
dissect_rrc_INTEGER_0_maxEDCHs_1(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            0U, maxEDCHs_1, NULL, FALSE);

  return offset;
}


static const per_sequence_t UL_DPCHpowerControlInfoConcurrentTTI_sequence[] = {
  { &hf_rrc_deltaACK        , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DeltaACK },
  { &hf_rrc_deltaNACK       , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DeltaNACK },
  { &hf_rrc_ack_NACK_repetition_factor, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_ACK_NACK_repetitionFactor },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UL_DPCHpowerControlInfoConcurrentTTI(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UL_DPCHpowerControlInfoConcurrentTTI, UL_DPCHpowerControlInfoConcurrentTTI_sequence);

  return offset;
}


static const value_string rrc_T_max_CCCH_ResourceAllocation_01_vals[] = {
  {   0, "tti8" },
  {   1, "tti12" },
  {   2, "tti16" },
  {   3, "tti20" },
  {   4, "tti24" },
  {   5, "tti32" },
  {   6, "tti40" },
  {   7, "tti80" },
  { 0, NULL }
};


static int
dissect_rrc_T_max_CCCH_ResourceAllocation_01(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     8, NULL, FALSE, 0, NULL);

  return offset;
}



static int
dissect_rrc_INTEGER_8_24(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            8U, 24U, NULL, FALSE);

  return offset;
}


static const value_string rrc_T_e_dch_TransmitContinuationOffset_01_vals[] = {
  {   0, "tti0" },
  {   1, "tti4" },
  {   2, "tti8" },
  {   3, "tti16" },
  {   4, "tti24" },
  {   5, "tti40" },
  {   6, "tti80" },
  {   7, "infinity" },
  { 0, NULL }
};


static int
dissect_rrc_T_e_dch_TransmitContinuationOffset_01(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     8, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t Measurement_Feedback_Info_ConcurrentTTI_sequence[] = {
  { &hf_rrc_feedback_cycle_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_Feedback_cycle_r7 },
  { &hf_rrc_cqi_RepetitionFactor, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CQI_RepetitionFactor },
  { &hf_rrc_deltaCQI        , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DeltaCQI },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_Measurement_Feedback_Info_ConcurrentTTI(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_Measurement_Feedback_Info_ConcurrentTTI, Measurement_Feedback_Info_ConcurrentTTI_sequence);

  return offset;
}


static const per_sequence_t CommonEDCHSystemInfoParamConcurrentTTI_sequence[] = {
  { &hf_rrc_common_E_DCH_MAC_d_Flow_Info_List_ConcurrentTTI, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_Common_E_DCH_MAC_d_Flow_Info_List_ConcurrentTTI },
  { &hf_rrc_initialServingGrantValue, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_INTEGER_0_37 },
  { &hf_rrc_e_agch_Information_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_E_AGCH_Information },
  { &hf_rrc_ul_DPCHpowerControlInfoConcurrentTTI, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_DPCHpowerControlInfoConcurrentTTI },
  { &hf_rrc_e_DPCCH_DPCCH_PowerOffset, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_E_DPCCH_DPCCH_PowerOffset },
  { &hf_rrc_e_dpdch_Info    , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_E_DPDCH_Info_r8 },
  { &hf_rrc_additional_E_DCH_TransmitBackoff, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_INTEGER_0_15 },
  { &hf_rrc_max_CCCH_ResourceAllocation_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_max_CCCH_ResourceAllocation_01 },
  { &hf_rrc_max_PeriodForCollisionResolution, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_INTEGER_8_24 },
  { &hf_rrc_e_dch_TransmitContinuationOffset_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_e_dch_TransmitContinuationOffset_01 },
  { &hf_rrc_measurement_Feedback_Info_ConcurrentTTI, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_Measurement_Feedback_Info_ConcurrentTTI },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_CommonEDCHSystemInfoParamConcurrentTTI(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_CommonEDCHSystemInfoParamConcurrentTTI, CommonEDCHSystemInfoParamConcurrentTTI_sequence);

  return offset;
}


static const per_sequence_t Concurrent_Deployment_2ms_10ms_TTI_sequence[] = {
  { &hf_rrc_tti_selection   , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_tti_selection },
  { &hf_rrc_concurrent_TTI_Partition_Index, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_0_maxEDCHs_1 },
  { &hf_rrc_commonEDCHSystemInfoParamConcurrentTTI, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CommonEDCHSystemInfoParamConcurrentTTI },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_Concurrent_Deployment_2ms_10ms_TTI(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_Concurrent_Deployment_2ms_10ms_TTI, Concurrent_Deployment_2ms_10ms_TTI_sequence);

  return offset;
}


static const value_string rrc_TransportFormatSet_LCR_vals[] = {
  {   0, "dedicatedTransChTFS" },
  {   1, "commonTransChTFS-LCR" },
  { 0, NULL }
};

static const per_choice_t TransportFormatSet_LCR_choice[] = {
  {   0, &hf_rrc_dedicatedTransChTFS, ASN1_NO_EXTENSIONS     , dissect_rrc_DedicatedTransChTFS },
  {   1, &hf_rrc_commonTransChTFS_LCR, ASN1_NO_EXTENSIONS     , dissect_rrc_CommonTransChTFS_LCR },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_TransportFormatSet_LCR(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_TransportFormatSet_LCR, TransportFormatSet_LCR_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t USCH_TransportChannelsInfo_item_sequence[] = {
  { &hf_rrc_usch_TransportChannelIdentity, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TransportChannelIdentity },
  { &hf_rrc_usch_TFS        , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TransportFormatSet },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_USCH_TransportChannelsInfo_item(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_USCH_TransportChannelsInfo_item, USCH_TransportChannelsInfo_item_sequence);

  return offset;
}


static const per_sequence_t USCH_TransportChannelsInfo_sequence_of[1] = {
  { &hf_rrc_USCH_TransportChannelsInfo_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_USCH_TransportChannelsInfo_item },
};

static int
dissect_rrc_USCH_TransportChannelsInfo(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_USCH_TransportChannelsInfo, USCH_TransportChannelsInfo_sequence_of,
                                                  1, maxTrCH, FALSE);

  return offset;
}



static int
dissect_rrc_AC_To_ASC_Mapping(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            0U, 7U, NULL, FALSE);

  return offset;
}


static const per_sequence_t AC_To_ASC_MappingTable_sequence_of[1] = {
  { &hf_rrc_AC_To_ASC_MappingTable_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_AC_To_ASC_Mapping },
};

static int
dissect_rrc_AC_To_ASC_MappingTable(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_AC_To_ASC_MappingTable, AC_To_ASC_MappingTable_sequence_of,
                                                  maxASCmap, maxASCmap, FALSE);

  return offset;
}



static int
dissect_rrc_T_assignedSubChannelNumber(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_bit_string(tvb, offset, actx, tree, hf_index,
                                     4, 4, FALSE, NULL);

  return offset;
}


static const per_sequence_t AccessServiceClass_FDD_sequence[] = {
  { &hf_rrc_availableSignatureStartIndex, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_0_15 },
  { &hf_rrc_availableSignatureEndIndex, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_0_15 },
  { &hf_rrc_assignedSubChannelNumber, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_assignedSubChannelNumber },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_AccessServiceClass_FDD(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_AccessServiceClass_FDD, AccessServiceClass_FDD_sequence);

  return offset;
}



static int
dissect_rrc_T_channelisationCodeIndices(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_bit_string(tvb, offset, actx, tree, hf_index,
                                     8, 8, FALSE, NULL);

  return offset;
}


static const value_string rrc_T_subchannels_vals[] = {
  {   0, "subch0" },
  {   1, "subch1" },
  { 0, NULL }
};


static int
dissect_rrc_T_subchannels(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     2, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t T_size2_sequence[] = {
  { &hf_rrc_subchannels     , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_subchannels },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_size2(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_size2, T_size2_sequence);

  return offset;
}



static int
dissect_rrc_T_subchannels_01(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_bit_string(tvb, offset, actx, tree, hf_index,
                                     4, 4, FALSE, NULL);

  return offset;
}


static const per_sequence_t T_size4_sequence[] = {
  { &hf_rrc_subchannels_01  , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_subchannels_01 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_size4(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_size4, T_size4_sequence);

  return offset;
}



static int
dissect_rrc_T_subchannels_02(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_bit_string(tvb, offset, actx, tree, hf_index,
                                     8, 8, FALSE, NULL);

  return offset;
}


static const per_sequence_t T_size8_sequence[] = {
  { &hf_rrc_subchannels_02  , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_subchannels_02 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_size8(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_size8, T_size8_sequence);

  return offset;
}


static const value_string rrc_T_subchannelSize_vals[] = {
  {   0, "size1" },
  {   1, "size2" },
  {   2, "size4" },
  {   3, "size8" },
  { 0, NULL }
};

static const per_choice_t T_subchannelSize_choice[] = {
  {   0, &hf_rrc_size1           , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   1, &hf_rrc_size2           , ASN1_NO_EXTENSIONS     , dissect_rrc_T_size2 },
  {   2, &hf_rrc_size4           , ASN1_NO_EXTENSIONS     , dissect_rrc_T_size4 },
  {   3, &hf_rrc_size8           , ASN1_NO_EXTENSIONS     , dissect_rrc_T_size8 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_subchannelSize(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_subchannelSize, T_subchannelSize_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t AccessServiceClass_TDD_sequence[] = {
  { &hf_rrc_channelisationCodeIndices, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_channelisationCodeIndices },
  { &hf_rrc_subchannelSize  , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_subchannelSize },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_AccessServiceClass_TDD(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_AccessServiceClass_TDD, AccessServiceClass_TDD_sequence);

  return offset;
}



static int
dissect_rrc_T_channelisationCodeIndices_01(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_bit_string(tvb, offset, actx, tree, hf_index,
                                     16, 16, FALSE, NULL);

  return offset;
}


static const value_string rrc_T_subchannels_03_vals[] = {
  {   0, "subch0" },
  {   1, "subch1" },
  { 0, NULL }
};


static int
dissect_rrc_T_subchannels_03(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     2, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t T_size2_01_sequence[] = {
  { &hf_rrc_subchannels_03  , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_subchannels_03 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_size2_01(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_size2_01, T_size2_01_sequence);

  return offset;
}



static int
dissect_rrc_T_subchannels_04(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_bit_string(tvb, offset, actx, tree, hf_index,
                                     4, 4, FALSE, NULL);

  return offset;
}


static const per_sequence_t T_size4_01_sequence[] = {
  { &hf_rrc_subchannels_04  , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_subchannels_04 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_size4_01(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_size4_01, T_size4_01_sequence);

  return offset;
}



static int
dissect_rrc_T_subchannels_05(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_bit_string(tvb, offset, actx, tree, hf_index,
                                     8, 8, FALSE, NULL);

  return offset;
}


static const per_sequence_t T_size8_01_sequence[] = {
  { &hf_rrc_subchannels_05  , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_subchannels_05 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_size8_01(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_size8_01, T_size8_01_sequence);

  return offset;
}



static int
dissect_rrc_T_subchannels_06(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_bit_string(tvb, offset, actx, tree, hf_index,
                                     16, 16, FALSE, NULL);

  return offset;
}


static const per_sequence_t T_size16_sequence[] = {
  { &hf_rrc_subchannels_06  , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_subchannels_06 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_size16(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_size16, T_size16_sequence);

  return offset;
}


static const value_string rrc_T_subchannelSize_01_vals[] = {
  {   0, "size1" },
  {   1, "size2" },
  {   2, "size4" },
  {   3, "size8" },
  {   4, "size16" },
  { 0, NULL }
};

static const per_choice_t T_subchannelSize_01_choice[] = {
  {   0, &hf_rrc_size1           , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   1, &hf_rrc_size2_01        , ASN1_NO_EXTENSIONS     , dissect_rrc_T_size2_01 },
  {   2, &hf_rrc_size4_01        , ASN1_NO_EXTENSIONS     , dissect_rrc_T_size4_01 },
  {   3, &hf_rrc_size8_01        , ASN1_NO_EXTENSIONS     , dissect_rrc_T_size8_01 },
  {   4, &hf_rrc_size16          , ASN1_NO_EXTENSIONS     , dissect_rrc_T_size16 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_subchannelSize_01(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_subchannelSize_01, T_subchannelSize_01_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t AccessServiceClass_TDD_r7_sequence[] = {
  { &hf_rrc_channelisationCodeIndices_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_channelisationCodeIndices_01 },
  { &hf_rrc_subchannelSize_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_subchannelSize_01 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_AccessServiceClass_TDD_r7(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_AccessServiceClass_TDD_r7, AccessServiceClass_TDD_r7_sequence);

  return offset;
}


static const per_sequence_t DynamicTFInformationCCCH_sequence[] = {
  { &hf_rrc_octetModeRLC_SizeInfoType2, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_OctetModeRLC_SizeInfoType2 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_DynamicTFInformationCCCH(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_DynamicTFInformationCCCH, DynamicTFInformationCCCH_sequence);

  return offset;
}


static const per_sequence_t AdditionalPRACH_TF_and_TFCS_CCCH_IEs_sequence[] = {
  { &hf_rrc_powerOffsetInformation, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PowerOffsetInformation },
  { &hf_rrc_dynamicTFInformationCCCH, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DynamicTFInformationCCCH },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_AdditionalPRACH_TF_and_TFCS_CCCH_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_AdditionalPRACH_TF_and_TFCS_CCCH_IEs, AdditionalPRACH_TF_and_TFCS_CCCH_IEs_sequence);

  return offset;
}


static const per_sequence_t AdditionalPRACH_TF_and_TFCS_CCCH_sequence[] = {
  { &hf_rrc_additionalPRACH_TF_and_TFCS_CCCH_IEs, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_AdditionalPRACH_TF_and_TFCS_CCCH_IEs },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_AdditionalPRACH_TF_and_TFCS_CCCH(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_AdditionalPRACH_TF_and_TFCS_CCCH, AdditionalPRACH_TF_and_TFCS_CCCH_sequence);

  return offset;
}


static const per_sequence_t AdditionalPRACH_TF_and_TFCS_CCCH_List_sequence_of[1] = {
  { &hf_rrc_AdditionalPRACH_TF_and_TFCS_CCCH_List_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_AdditionalPRACH_TF_and_TFCS_CCCH },
};

static int
dissect_rrc_AdditionalPRACH_TF_and_TFCS_CCCH_List(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_AdditionalPRACH_TF_and_TFCS_CCCH_List, AdditionalPRACH_TF_and_TFCS_CCCH_List_sequence_of,
                                                  1, maxPRACH, FALSE);

  return offset;
}


static const value_string rrc_AICH_TransmissionTiming_vals[] = {
  {   0, "e0" },
  {   1, "e1" },
  { 0, NULL }
};


static int
dissect_rrc_AICH_TransmissionTiming(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     2, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t AICH_Info_sequence[] = {
  { &hf_rrc_channelisationCode256, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_ChannelisationCode256 },
  { &hf_rrc_sttd_Indicator  , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { &hf_rrc_aich_TransmissionTiming, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_AICH_TransmissionTiming },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_AICH_Info(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_AICH_Info, AICH_Info_sequence);

  return offset;
}


static const per_sequence_t AICH_Info_Compressed_sequence[] = {
  { &hf_rrc_channelisationCode256, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_ChannelisationCode256 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_AICH_Info_Compressed(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_AICH_Info_Compressed, AICH_Info_Compressed_sequence);

  return offset;
}



static int
dissect_rrc_AICH_PowerOffset(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            -22, 5U, NULL, FALSE);

  return offset;
}


static const per_sequence_t ASCSetting_FDD_sequence[] = {
  { &hf_rrc_accessServiceClass_FDD, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_AccessServiceClass_FDD },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_ASCSetting_FDD(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_ASCSetting_FDD, ASCSetting_FDD_sequence);

  return offset;
}


static const per_sequence_t ASCSetting_TDD_sequence[] = {
  { &hf_rrc_accessServiceClass_TDD, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_AccessServiceClass_TDD },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_ASCSetting_TDD(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_ASCSetting_TDD, ASCSetting_TDD_sequence);

  return offset;
}


static const per_sequence_t ASCSetting_TDD_r7_sequence[] = {
  { &hf_rrc_accessServiceClass_TDD_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_AccessServiceClass_TDD_r7 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_ASCSetting_TDD_r7(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_ASCSetting_TDD_r7, ASCSetting_TDD_r7_sequence);

  return offset;
}



static int
dissect_rrc_AvailableSignatures(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_bit_string(tvb, offset, actx, tree, hf_index,
                                     16, 16, FALSE, NULL);

  return offset;
}



static int
dissect_rrc_AvailableSubChannelNumbers(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_bit_string(tvb, offset, actx, tree, hf_index,
                                     12, 12, FALSE, NULL);

  return offset;
}


static const per_sequence_t CodeResourceInformation_TDD128_sequence[] = {
  { &hf_rrc_start_code      , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_HS_ChannelisationCode_LCR },
  { &hf_rrc_stop_code       , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_HS_ChannelisationCode_LCR },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_CodeResourceInformation_TDD128(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_CodeResourceInformation_TDD128, CodeResourceInformation_TDD128_sequence);

  return offset;
}


static const per_sequence_t E_RGCH_Information_CommonEdch_sequence[] = {
  { &hf_rrc_signatureSequence_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_E_HICH_RGCH_SignatureSequence },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_E_RGCH_Information_CommonEdch(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_E_RGCH_Information_CommonEdch, E_RGCH_Information_CommonEdch_sequence);

  return offset;
}


static const per_sequence_t E_HICH_Information_CommonEdch_sequence[] = {
  { &hf_rrc_channelisationCode_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_E_HICH_ChannelisationCode },
  { &hf_rrc_signatureSequence_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_E_HICH_RGCH_SignatureSequence },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_E_HICH_Information_CommonEdch(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_E_HICH_Information_CommonEdch, E_HICH_Information_CommonEdch_sequence);

  return offset;
}


static const per_sequence_t UL_DPCH_CodeInfoForCommonEDCH_sequence[] = {
  { &hf_rrc_ul_DPCCHscramblingCodeType, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_ScramblingCodeType },
  { &hf_rrc_ul_DPCCHscramblingCode, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_ScramblingCode },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UL_DPCH_CodeInfoForCommonEDCH(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UL_DPCH_CodeInfoForCommonEDCH, UL_DPCH_CodeInfoForCommonEDCH_sequence);

  return offset;
}


static const per_sequence_t Common_E_DCH_ResourceInfoList_sequence[] = {
  { &hf_rrc_s_offset        , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_INTEGER_0_9 },
  { &hf_rrc_f_dpch_ChannelisationCodeNumber, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_INTEGER_0_255 },
  { &hf_rrc_e_RGCH_Information_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_E_RGCH_Information_CommonEdch },
  { &hf_rrc_e_hich_Info     , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_E_HICH_Information_CommonEdch },
  { &hf_rrc_ul_DPCH_CodeInfoForCommonEDCH, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UL_DPCH_CodeInfoForCommonEDCH },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_Common_E_DCH_ResourceInfoList(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_Common_E_DCH_ResourceInfoList, Common_E_DCH_ResourceInfoList_sequence);

  return offset;
}


static const per_sequence_t TwoMsHarqConfiguration_sequence[] = {
  { &hf_rrc_harqProcessAllocation, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_BIT_STRING_SIZE_8 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_TwoMsHarqConfiguration(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_TwoMsHarqConfiguration, TwoMsHarqConfiguration_sequence);

  return offset;
}


static const per_sequence_t Common_E_DCH_ResourceInfoListExt_sequence[] = {
  { &hf_rrc_twoMsHarqConfiguration, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TwoMsHarqConfiguration },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_Common_E_DCH_ResourceInfoListExt(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_Common_E_DCH_ResourceInfoListExt, Common_E_DCH_ResourceInfoListExt_sequence);

  return offset;
}



static int
dissect_rrc_INTEGER_1_maxERNTIgroup(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            1U, maxERNTIgroup, NULL, FALSE);

  return offset;
}



static int
dissect_rrc_INTEGER_1_maxERNTIperGroup(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            1U, maxERNTIperGroup, NULL, FALSE);

  return offset;
}


static const per_sequence_t Common_E_RNTI_Info_item_sequence[] = {
  { &hf_rrc_starting_E_RNTI , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_E_RNTI },
  { &hf_rrc_number_of_group , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_1_maxERNTIgroup },
  { &hf_rrc_number_of_ENRTI_per_group, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_1_maxERNTIperGroup },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_Common_E_RNTI_Info_item(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_Common_E_RNTI_Info_item, Common_E_RNTI_Info_item_sequence);

  return offset;
}


static const per_sequence_t Common_E_RNTI_Info_sequence_of[1] = {
  { &hf_rrc_Common_E_RNTI_Info_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_Common_E_RNTI_Info_item },
};

static int
dissect_rrc_Common_E_RNTI_Info(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_Common_E_RNTI_Info, Common_E_RNTI_Info_sequence_of,
                                                  1, maxERUCCH, FALSE);

  return offset;
}


static const per_sequence_t SEQUENCE_SIZE_1_maxEDCHs_OF_Common_E_DCH_ResourceInfoListExt_sequence_of[1] = {
  { &hf_rrc_common_E_DCH_ResourceInfoListExt_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_Common_E_DCH_ResourceInfoListExt },
};

static int
dissect_rrc_SEQUENCE_SIZE_1_maxEDCHs_OF_Common_E_DCH_ResourceInfoListExt(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_SEQUENCE_SIZE_1_maxEDCHs_OF_Common_E_DCH_ResourceInfoListExt, SEQUENCE_SIZE_1_maxEDCHs_OF_Common_E_DCH_ResourceInfoListExt_sequence_of,
                                                  1, maxEDCHs, FALSE);

  return offset;
}


static const per_sequence_t ScheduledTransmissionConfiguration_sequence[] = {
  { &hf_rrc_common_E_DCH_ResourceInfoListExt, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SEQUENCE_SIZE_1_maxEDCHs_OF_Common_E_DCH_ResourceInfoListExt },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_ScheduledTransmissionConfiguration(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_ScheduledTransmissionConfiguration, ScheduledTransmissionConfiguration_sequence);

  return offset;
}


static const per_sequence_t CommonEDCHResourceConfigInfoListExt_sequence[] = {
  { &hf_rrc_scheduledTransmissionConfiguration, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_ScheduledTransmissionConfiguration },
  { &hf_rrc_cOffset         , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_INTEGER_0_29 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_CommonEDCHResourceConfigInfoListExt(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_CommonEDCHResourceConfigInfoListExt, CommonEDCHResourceConfigInfoListExt_sequence);

  return offset;
}



static int
dissect_rrc_UL_Interference(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            -110, -70, NULL, FALSE);

  return offset;
}


static const per_sequence_t T_dummy_14_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_dummy_14(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_dummy_14, T_dummy_14_sequence);

  return offset;
}


static const value_string rrc_T_t_RUCCH_02_vals[] = {
  {   0, "ms20" },
  {   1, "ms40" },
  {   2, "ms60" },
  {   3, "ms80" },
  {   4, "ms120" },
  {   5, "ms160" },
  {   6, "ms200" },
  {   7, "ms240" },
  {   8, "ms280" },
  {   9, "ms320" },
  {  10, "ms400" },
  {  11, "ms500" },
  {  12, "ms600" },
  {  13, "ms800" },
  {  14, "ms1000" },
  {  15, "ms2000" },
  { 0, NULL }
};


static int
dissect_rrc_T_t_RUCCH_02(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     16, NULL, FALSE, 0, NULL);

  return offset;
}


static const value_string rrc_T_t_WAIT_01_vals[] = {
  {   0, "everyEDCHTTI" },
  {   1, "ms40" },
  {   2, "ms80" },
  {   3, "ms160" },
  {   4, "ms320" },
  {   5, "ms640" },
  {   6, "ms1000" },
  {   7, "ms2000" },
  { 0, NULL }
};


static int
dissect_rrc_T_t_WAIT_01(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     8, NULL, FALSE, 0, NULL);

  return offset;
}


static const value_string rrc_T_t_SI_01_vals[] = {
  {   0, "everyEDCHTTI" },
  {   1, "ms20" },
  {   2, "ms40" },
  {   3, "ms60" },
  {   4, "ms80" },
  {   5, "ms160" },
  {   6, "ms200" },
  { 0, NULL }
};


static int
dissect_rrc_T_t_SI_01(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     7, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t E_RUCCH_Info_TDD128_sequence[] = {
  { &hf_rrc_t_RUCCH_02      , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_t_RUCCH_02 },
  { &hf_rrc_n_RUCCH         , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_INTEGER_0_7 },
  { &hf_rrc_t_WAIT_01       , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_t_WAIT_01 },
  { &hf_rrc_t_SI_01         , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_t_SI_01 },
  { &hf_rrc_extendedEstimationWindow, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_INTEGER_2_5 },
  { &hf_rrc_e_RUCCH_AccessServiceClass, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_E_RUCCH_AccessServiceClass },
  { &hf_rrc_e_RUCCH_PersistenceScalingFactor_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_PersistenceScalingFactorList },
  { &hf_rrc_sync_UL_InfoForE_RUCCH, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_SYNC_UL_InfoForE_RUCCH },
  { &hf_rrc_prach_information, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_PRACH_Information_LCR_List },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_E_RUCCH_Info_TDD128(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_E_RUCCH_Info_TDD128, E_RUCCH_Info_TDD128_sequence);

  return offset;
}


static const value_string rrc_T_snpl_ReportType_01_vals[] = {
  {   0, "type1" },
  {   1, "type2" },
  { 0, NULL }
};


static int
dissect_rrc_T_snpl_ReportType_01(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     2, NULL, FALSE, 0, NULL);

  return offset;
}


static const value_string rrc_T_retransTimerForSchedInfo_01_vals[] = {
  {   0, "ms10" },
  {   1, "ms15" },
  {   2, "ms20" },
  {   3, "ms25" },
  {   4, "ms30" },
  {   5, "ms35" },
  {   6, "ms40" },
  {   7, "ms45" },
  {   8, "ms50" },
  {   9, "ms55" },
  {  10, "ms60" },
  {  11, "ms65" },
  {  12, "ms70" },
  {  13, "ms75" },
  {  14, "ms80" },
  {  15, "ms85" },
  {  16, "ms90" },
  {  17, "ms95" },
  {  18, "ms100" },
  {  19, "ms110" },
  {  20, "ms120" },
  {  21, "ms140" },
  {  22, "ms160" },
  {  23, "ms200" },
  {  24, "ms240" },
  {  25, "ms280" },
  {  26, "ms320" },
  {  27, "ms400" },
  {  28, "ms480" },
  {  29, "ms560" },
  { 0, NULL }
};

static value_string_ext rrc_T_retransTimerForSchedInfo_01_vals_ext = VALUE_STRING_EXT_INIT(rrc_T_retransTimerForSchedInfo_01_vals);


static int
dissect_rrc_T_retransTimerForSchedInfo_01(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     30, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t E_PUCH_Info_TDD128_sequence[] = {
  { &hf_rrc_e_TFCS_Info     , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_E_TFCS_Info },
  { &hf_rrc_snpl_ReportType_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_snpl_ReportType_01 },
  { &hf_rrc_prxdes_base     , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_M112_M50 },
  { &hf_rrc_beaconPLEst_01  , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_BOOLEAN },
  { &hf_rrc_tpc_StepSize    , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TPC_StepSizeTDD },
  { &hf_rrc_pebase_PowerControlGAP_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_INTEGER_1_255 },
  { &hf_rrc_ul_SynchronisationParameters, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_SynchronisationParameters_r4 },
  { &hf_rrc_e_PUCH_TS_ConfigurationList_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SEQUENCE_SIZE_1_maxTS_LCR_1_OF_E_PUCH_TS_Slots_LCR },
  { &hf_rrc_minimum_Allowed_Code_Rate, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_0_63 },
  { &hf_rrc_maximum_Allowed_Code_Rate, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_0_63 },
  { &hf_rrc_maximumNumOfRetransSchedInfo, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_0_15 },
  { &hf_rrc_retransTimerForSchedInfo_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_retransTimerForSchedInfo_01 },
  { &hf_rrc_dummy_44        , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_INTEGER_0_6 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_E_PUCH_Info_TDD128(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_E_PUCH_Info_TDD128, E_PUCH_Info_TDD128_sequence);

  return offset;
}


static const value_string rrc_T_harq_Info_03_vals[] = {
  {   0, "rv0" },
  {   1, "rvtable" },
  { 0, NULL }
};


static int
dissect_rrc_T_harq_Info_03(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     2, NULL, FALSE, 0, NULL);

  return offset;
}


static const value_string rrc_T_harq_retransmission_timer_vals[] = {
  {   0, "ms10" },
  {   1, "ms15" },
  {   2, "ms20" },
  {   3, "ms25" },
  {   4, "ms30" },
  {   5, "ms35" },
  {   6, "ms40" },
  {   7, "ms45" },
  {   8, "ms50" },
  {   9, "ms55" },
  {  10, "ms60" },
  {  11, "ms65" },
  {  12, "ms70" },
  {  13, "ms75" },
  {  14, "ms80" },
  {  15, "ms85" },
  {  16, "ms90" },
  {  17, "ms95" },
  {  18, "ms100" },
  {  19, "ms110" },
  {  20, "ms120" },
  {  21, "ms140" },
  {  22, "ms160" },
  { 0, NULL }
};

static value_string_ext rrc_T_harq_retransmission_timer_vals_ext = VALUE_STRING_EXT_INIT(rrc_T_harq_retransmission_timer_vals);


static int
dissect_rrc_T_harq_retransmission_timer(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     23, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t T_ccch_transmission_Info_sequence[] = {
  { &hf_rrc_common_e_rnti_Info, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_Common_E_RNTI_Info },
  { &hf_rrc_harq_MaximumNumberOfRetransmissions, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_0_7 },
  { &hf_rrc_harq_retransmission_timer, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_harq_retransmission_timer },
  { &hf_rrc_harq_power_offset, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_0_6 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_ccch_transmission_Info(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_ccch_transmission_Info, T_ccch_transmission_Info_sequence);

  return offset;
}


static const per_sequence_t T_tdd128_11_sequence[] = {
  { &hf_rrc_e_RUCCH_Info    , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_E_RUCCH_Info_TDD128 },
  { &hf_rrc_e_PUCH_Info     , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_E_PUCH_Info_TDD128 },
  { &hf_rrc_e_hich_Information, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_E_HICH_Information_TDD128 },
  { &hf_rrc_e_agch_Information, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_E_AGCH_Information_TDD128 },
  { &hf_rrc_harq_Info_03    , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_harq_Info_03 },
  { &hf_rrc_ccch_transmission_Info, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_ccch_transmission_Info },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_tdd128_11(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_tdd128_11, T_tdd128_11_sequence);

  return offset;
}


static const value_string rrc_T_tdd_19_vals[] = {
  {   0, "tdd768" },
  {   1, "tdd384" },
  {   2, "tdd128" },
  { 0, NULL }
};

static const per_choice_t T_tdd_19_choice[] = {
  {   0, &hf_rrc_tdd768_06       , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   1, &hf_rrc_tdd384_11       , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   2, &hf_rrc_tdd128_11       , ASN1_NO_EXTENSIONS     , dissect_rrc_T_tdd128_11 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_tdd_19(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_tdd_19, T_tdd_19_choice,
                                 NULL);

  return offset;
}


static const value_string rrc_T_modeSpecificInfo_27_vals[] = {
  {   0, "dummy" },
  {   1, "tdd" },
  { 0, NULL }
};

static const per_choice_t T_modeSpecificInfo_27_choice[] = {
  {   0, &hf_rrc_dummy_37        , ASN1_NO_EXTENSIONS     , dissect_rrc_T_dummy_14 },
  {   1, &hf_rrc_tdd_20          , ASN1_NO_EXTENSIONS     , dissect_rrc_T_tdd_19 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_modeSpecificInfo_27(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_modeSpecificInfo_27, T_modeSpecificInfo_27_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t CommonEDCHSystemInfo_sequence[] = {
  { &hf_rrc_ul_InterferenceForCommonEDCH, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_Interference },
  { &hf_rrc_common_E_DCH_MAC_d_FlowList, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_Common_E_DCH_MAC_d_FlowList },
  { &hf_rrc_modeSpecificInfo_27, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_modeSpecificInfo_27 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_CommonEDCHSystemInfo(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_CommonEDCHSystemInfo, CommonEDCHSystemInfo_sequence);

  return offset;
}



static int
dissect_rrc_PreambleScramblingCodeWordNumber(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            0U, 15U, NULL, FALSE);

  return offset;
}


static const per_sequence_t SEQUENCE_SIZE_1_maxASC_OF_ASCSetting_FDD_sequence_of[1] = {
  { &hf_rrc_fdd_item        , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_ASCSetting_FDD },
};

static int
dissect_rrc_SEQUENCE_SIZE_1_maxASC_OF_ASCSetting_FDD(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_SEQUENCE_SIZE_1_maxASC_OF_ASCSetting_FDD, SEQUENCE_SIZE_1_maxASC_OF_ASCSetting_FDD_sequence_of,
                                                  1, maxASC, FALSE);

  return offset;
}


static const per_sequence_t SEQUENCE_SIZE_1_maxASC_OF_ASCSetting_TDD_r7_sequence_of[1] = {
  { &hf_rrc_tdd_item_01     , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_ASCSetting_TDD_r7 },
};

static int
dissect_rrc_SEQUENCE_SIZE_1_maxASC_OF_ASCSetting_TDD_r7(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_SEQUENCE_SIZE_1_maxASC_OF_ASCSetting_TDD_r7, SEQUENCE_SIZE_1_maxASC_OF_ASCSetting_TDD_r7_sequence_of,
                                                  1, maxASC, FALSE);

  return offset;
}


static const value_string rrc_PRACH_Partitioning_r7_vals[] = {
  {   0, "fdd" },
  {   1, "tdd" },
  { 0, NULL }
};

static const per_choice_t PRACH_Partitioning_r7_choice[] = {
  {   0, &hf_rrc_fdd_105         , ASN1_NO_EXTENSIONS     , dissect_rrc_SEQUENCE_SIZE_1_maxASC_OF_ASCSetting_FDD },
  {   1, &hf_rrc_tdd_63          , ASN1_NO_EXTENSIONS     , dissect_rrc_SEQUENCE_SIZE_1_maxASC_OF_ASCSetting_TDD_r7 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_PRACH_Partitioning_r7(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_PRACH_Partitioning_r7, PRACH_Partitioning_r7_choice,
                                 NULL);

  return offset;
}



static int
dissect_rrc_PowerRampStep(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            1U, 8U, NULL, FALSE);

  return offset;
}



static int
dissect_rrc_PreambleRetransMax(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            1U, 64U, NULL, FALSE);

  return offset;
}


static const per_sequence_t PRACH_PowerOffset_sequence[] = {
  { &hf_rrc_powerRampStep   , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PowerRampStep },
  { &hf_rrc_preambleRetransMax, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PreambleRetransMax },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_PRACH_PowerOffset(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_PRACH_PowerOffset, PRACH_PowerOffset_sequence);

  return offset;
}



static int
dissect_rrc_NB01(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            0U, 50U, NULL, FALSE);

  return offset;
}


static const per_sequence_t RACH_TransmissionParameters_sequence[] = {
  { &hf_rrc_mmax            , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_1_32 },
  { &hf_rrc_nb01Min         , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_NB01 },
  { &hf_rrc_nb01Max         , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_NB01 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_RACH_TransmissionParameters(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_RACH_TransmissionParameters, RACH_TransmissionParameters_sequence);

  return offset;
}



static int
dissect_rrc_INTEGER_M5_10(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            -5, 10U, NULL, FALSE);

  return offset;
}


static const per_sequence_t PRACH_PreambleForEnhancedUplink_sequence[] = {
  { &hf_rrc_availableSignatures, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_AvailableSignatures },
  { &hf_rrc_e_ai_Indication , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { &hf_rrc_preambleScramblingCodeWordNumber, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_PreambleScramblingCodeWordNumber },
  { &hf_rrc_availableSubChannelNumbers, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_AvailableSubChannelNumbers },
  { &hf_rrc_prach_Partitioning, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_PRACH_Partitioning_r7 },
  { &hf_rrc_persistenceScalingFactorList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_PersistenceScalingFactorList },
  { &hf_rrc_ac_To_ASC_MappingTable, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_AC_To_ASC_MappingTable },
  { &hf_rrc_primaryCPICH_TX_Power, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_PrimaryCPICH_TX_Power },
  { &hf_rrc_constantValue   , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_ConstantValue },
  { &hf_rrc_prach_PowerOffset, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_PRACH_PowerOffset },
  { &hf_rrc_rach_TransmissionParameters, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RACH_TransmissionParameters },
  { &hf_rrc_aich_Info       , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_AICH_Info },
  { &hf_rrc_powerOffsetPp_e , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_M5_10 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_PRACH_PreambleForEnhancedUplink(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_PRACH_PreambleForEnhancedUplink, PRACH_PreambleForEnhancedUplink_sequence);

  return offset;
}


static const value_string rrc_T_harq_Info_04_vals[] = {
  {   0, "rv0" },
  {   1, "rvtable" },
  { 0, NULL }
};


static int
dissect_rrc_T_harq_Info_04(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     2, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t UL_DPCHpowerControlInfoForCommonEDCH_sequence[] = {
  { &hf_rrc_powerControlAlgorithm, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PowerControlAlgorithm },
  { &hf_rrc_deltaACK        , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DeltaACK },
  { &hf_rrc_deltaNACK       , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DeltaNACK },
  { &hf_rrc_ack_NACK_repetition_factor, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_ACK_NACK_repetitionFactor },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UL_DPCHpowerControlInfoForCommonEDCH(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UL_DPCHpowerControlInfoForCommonEDCH, UL_DPCHpowerControlInfoForCommonEDCH_sequence);

  return offset;
}


static const value_string rrc_T_max_CCCH_ResourceAllocation_vals[] = {
  {   0, "tti8" },
  {   1, "tti12" },
  {   2, "tti16" },
  {   3, "tti20" },
  {   4, "tti24" },
  {   5, "tti32" },
  {   6, "tti40" },
  {   7, "tti80" },
  { 0, NULL }
};


static int
dissect_rrc_T_max_CCCH_ResourceAllocation(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     8, NULL, FALSE, 0, NULL);

  return offset;
}


static const value_string rrc_T_e_dch_TransmitContinuationOffset_vals[] = {
  {   0, "tti0" },
  {   1, "tti4" },
  {   2, "tti8" },
  {   3, "tti16" },
  {   4, "tti24" },
  {   5, "tti40" },
  {   6, "tti80" },
  {   7, "infinity" },
  { 0, NULL }
};


static int
dissect_rrc_T_e_dch_TransmitContinuationOffset(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     8, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t SEQUENCE_SIZE_1_maxEDCHs_OF_Common_E_DCH_ResourceInfoList_sequence_of[1] = {
  { &hf_rrc_common_E_DCH_ResourceInfoList_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_Common_E_DCH_ResourceInfoList },
};

static int
dissect_rrc_SEQUENCE_SIZE_1_maxEDCHs_OF_Common_E_DCH_ResourceInfoList(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_SEQUENCE_SIZE_1_maxEDCHs_OF_Common_E_DCH_ResourceInfoList, SEQUENCE_SIZE_1_maxEDCHs_OF_Common_E_DCH_ResourceInfoList_sequence_of,
                                                  1, maxEDCHs, FALSE);

  return offset;
}


static const per_sequence_t CommonEDCHSystemInfoFDD_sequence[] = {
  { &hf_rrc_ul_InterferenceForCommonEDCH, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_Interference },
  { &hf_rrc_common_E_DCH_MAC_d_FlowList, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_Common_E_DCH_MAC_d_FlowList },
  { &hf_rrc_prach_PreambleForEnhancedUplink, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PRACH_PreambleForEnhancedUplink },
  { &hf_rrc_initialServingGrantValue, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_0_37 },
  { &hf_rrc_e_dch_TTI       , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_E_DCH_TTI },
  { &hf_rrc_e_agch_Information_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_E_AGCH_Information },
  { &hf_rrc_harq_Info_04    , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_harq_Info_04 },
  { &hf_rrc_ul_DPCHpowerControlInfoForCommonEDCH, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UL_DPCHpowerControlInfoForCommonEDCH },
  { &hf_rrc_e_dpcch_Info    , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_E_DPCCH_Info_r7 },
  { &hf_rrc_e_dpdch_Info    , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_E_DPDCH_Info_r8 },
  { &hf_rrc_dl_FDPCH_TPCcommandErrorRate, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_1_10 },
  { &hf_rrc_additional_E_DCH_TransmitBackoff, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_0_15 },
  { &hf_rrc_max_CCCH_ResourceAllocation, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_max_CCCH_ResourceAllocation },
  { &hf_rrc_max_PeriodForCollisionResolution, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_8_24 },
  { &hf_rrc_e_dch_TransmitContinuationOffset, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_e_dch_TransmitContinuationOffset },
  { &hf_rrc_ack_nack_support_on_HS_DPCCH, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { &hf_rrc_measurement_Feedback_Info, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_Measurement_Feedback_Info_r7 },
  { &hf_rrc_common_E_DCH_ResourceInfoList, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SEQUENCE_SIZE_1_maxEDCHs_OF_Common_E_DCH_ResourceInfoList },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_CommonEDCHSystemInfoFDD(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_CommonEDCHSystemInfoFDD, CommonEDCHSystemInfoFDD_sequence);

  return offset;
}


static const per_sequence_t DynamicPersistenceLevelTF_List_sequence_of[1] = {
  { &hf_rrc_DynamicPersistenceLevelTF_List_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DynamicPersistenceLevel },
};

static int
dissect_rrc_DynamicPersistenceLevelTF_List(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_DynamicPersistenceLevelTF_List, DynamicPersistenceLevelTF_List_sequence_of,
                                                  1, maxTF_CPCH, FALSE);

  return offset;
}


static const per_sequence_t CPCH_PersistenceLevels_sequence[] = {
  { &hf_rrc_cpch_SetID      , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CPCH_SetID },
  { &hf_rrc_dynamicPersistenceLevelTF_List, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DynamicPersistenceLevelTF_List },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_CPCH_PersistenceLevels(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_CPCH_PersistenceLevels, CPCH_PersistenceLevels_sequence);

  return offset;
}


static const per_sequence_t CPCH_PersistenceLevelsList_sequence_of[1] = {
  { &hf_rrc_CPCH_PersistenceLevelsList_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CPCH_PersistenceLevels },
};

static int
dissect_rrc_CPCH_PersistenceLevelsList(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_CPCH_PersistenceLevelsList, CPCH_PersistenceLevelsList_sequence_of,
                                                  1, maxCPCHsets, FALSE);

  return offset;
}


static const per_sequence_t CPCH_SetInfoList_sequence_of[1] = {
  { &hf_rrc_CPCH_SetInfoList_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CPCH_SetInfo },
};

static int
dissect_rrc_CPCH_SetInfoList(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_CPCH_SetInfoList, CPCH_SetInfoList_sequence_of,
                                                  1, maxCPCHsets, FALSE);

  return offset;
}



static int
dissect_rrc_CSICH_PowerOffset(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            -10, 5U, NULL, FALSE);

  return offset;
}


static const per_sequence_t DSCH_TransportChannelsInfo_item_sequence[] = {
  { &hf_rrc_dsch_transport_channel_identity, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TransportChannelIdentity },
  { &hf_rrc_dsch_TFS        , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TransportFormatSet },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_DSCH_TransportChannelsInfo_item(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_DSCH_TransportChannelsInfo_item, DSCH_TransportChannelsInfo_item_sequence);

  return offset;
}


static const per_sequence_t DSCH_TransportChannelsInfo_sequence_of[1] = {
  { &hf_rrc_DSCH_TransportChannelsInfo_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DSCH_TransportChannelsInfo_item },
};

static int
dissect_rrc_DSCH_TransportChannelsInfo(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_DSCH_TransportChannelsInfo, DSCH_TransportChannelsInfo_sequence_of,
                                                  1, maxTrCH, FALSE);

  return offset;
}


static const per_sequence_t DynamicPersistenceLevelList_sequence_of[1] = {
  { &hf_rrc_DynamicPersistenceLevelList_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DynamicPersistenceLevel },
};

static int
dissect_rrc_DynamicPersistenceLevelList(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_DynamicPersistenceLevelList, DynamicPersistenceLevelList_sequence_of,
                                                  1, maxPRACH, FALSE);

  return offset;
}


static const per_sequence_t Fallback_R99_PRACH_info_IEs_sequence[] = {
  { &hf_rrc_cCCH_Fallback   , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { &hf_rrc_dCCH_Fallback   , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_Fallback_R99_PRACH_info_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_Fallback_R99_PRACH_info_IEs, Fallback_R99_PRACH_info_IEs_sequence);

  return offset;
}


static const per_sequence_t HS_SCCH_SystemInfo_sequence[] = {
  { &hf_rrc_dl_ScramblingCode, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_SecondaryScramblingCode },
  { &hf_rrc_hS_SCCHChannelisationCodeInfo, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SEQUENCE_SIZE_1_maxHSSCCHs_OF_HS_SCCH_Codes },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_HS_SCCH_SystemInfo(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_HS_SCCH_SystemInfo, HS_SCCH_SystemInfo_sequence);

  return offset;
}


static const per_sequence_t SEQUENCE_SIZE_1_maxCommonHRNTI_OF_H_RNTI_sequence_of[1] = {
  { &hf_rrc_common_H_RNTI_information_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_H_RNTI },
};

static int
dissect_rrc_SEQUENCE_SIZE_1_maxCommonHRNTI_OF_H_RNTI(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_SEQUENCE_SIZE_1_maxCommonHRNTI_OF_H_RNTI, SEQUENCE_SIZE_1_maxCommonHRNTI_OF_H_RNTI_sequence_of,
                                                  1, maxCommonHRNTI, FALSE);

  return offset;
}


static const per_sequence_t HS_DSCH_CommonSystemInformation_sequence[] = {
  { &hf_rrc_ccch_MappingInfo, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CommonRBMappingInfo },
  { &hf_rrc_srb1_MappingInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CommonRBMappingInfo },
  { &hf_rrc_common_MAC_ehs_ReorderingQueueList, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_Common_MAC_ehs_ReorderingQueueList },
  { &hf_rrc_hs_scch_SystemInfo, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_HS_SCCH_SystemInfo },
  { &hf_rrc_harq_SystemInfo , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_HARQ_Info },
  { &hf_rrc_common_H_RNTI_information, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SEQUENCE_SIZE_1_maxCommonHRNTI_OF_H_RNTI },
  { &hf_rrc_bcchSpecific_H_RNTI, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_H_RNTI },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_HS_DSCH_CommonSystemInformation(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_HS_DSCH_CommonSystemInformation, HS_DSCH_CommonSystemInformation_sequence);

  return offset;
}


static const value_string rrc_T_tpc_step_size_03_vals[] = {
  {   0, "s1" },
  {   1, "s2" },
  {   2, "s3" },
  {   3, "spare1" },
  { 0, NULL }
};


static int
dissect_rrc_T_tpc_step_size_03(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     4, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t HS_SCCH_SystemInfo_TDD128_sequence[] = {
  { &hf_rrc_hs_SCCH_SetConfiguration, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SEQUENCE_SIZE_1_maxHSSCCHs_OF_HS_SCCH_TDD128_r6 },
  { &hf_rrc_power_level_HSSICH, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_M120_M58 },
  { &hf_rrc_nack_ack_power_offset, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_M7_8 },
  { &hf_rrc_tpc_step_size_03, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_tpc_step_size_03 },
  { &hf_rrc_bler_target     , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_Bler_Target },
  { &hf_rrc_power_control_gap, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_INTEGER_1_255 },
  { &hf_rrc_pathloss_compensation_switch, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_BOOLEAN },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_HS_SCCH_SystemInfo_TDD128(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_HS_SCCH_SystemInfo_TDD128, HS_SCCH_SystemInfo_TDD128_sequence);

  return offset;
}


static const per_sequence_t HS_DSCH_CommonSystemInformation_TDD128_sequence[] = {
  { &hf_rrc_ccch_MappingInfo, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CommonRBMappingInfo },
  { &hf_rrc_srb1_MappingInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CommonRBMappingInfo },
  { &hf_rrc_common_MAC_ehs_ReorderingQueueList, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_Common_MAC_ehs_ReorderingQueueList },
  { &hf_rrc_hs_scch_SystemInfo_tdd128, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_HS_SCCH_SystemInfo_TDD128 },
  { &hf_rrc_harq_SystemInfo , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_HARQ_Info },
  { &hf_rrc_hs_pdsch_MidambleConfiguration, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_HS_PDSCH_Midamble_Configuration_TDD128 },
  { &hf_rrc_common_H_RNTI_information, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SEQUENCE_SIZE_1_maxCommonHRNTI_OF_H_RNTI },
  { &hf_rrc_bcchSpecific_H_RNTI, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_H_RNTI },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_HS_DSCH_CommonSystemInformation_TDD128(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_HS_DSCH_CommonSystemInformation_TDD128, HS_DSCH_CommonSystemInformation_TDD128_sequence);

  return offset;
}


static const value_string rrc_HS_DSCH_DrxBurstFach_vals[] = {
  {   0, "f1" },
  {   1, "f2" },
  {   2, "f4" },
  {   3, "f8" },
  {   4, "f16" },
  {   5, "spare3" },
  {   6, "spare2" },
  {   7, "spare1" },
  { 0, NULL }
};


static int
dissect_rrc_HS_DSCH_DrxBurstFach(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     8, NULL, FALSE, 0, NULL);

  return offset;
}


static const value_string rrc_HS_DSCH_DrxCycleFach_vals[] = {
  {   0, "f4" },
  {   1, "f8" },
  {   2, "f16" },
  {   3, "f32" },
  { 0, NULL }
};


static int
dissect_rrc_HS_DSCH_DrxCycleFach(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     4, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t HS_DSCH_DrxCellfach_info_sequence[] = {
  { &hf_rrc_t_321           , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_321 },
  { &hf_rrc_hs_dsch_DrxCycleFach, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_HS_DSCH_DrxCycleFach },
  { &hf_rrc_hs_dsch_DrxBurstFach, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_HS_DSCH_DrxBurstFach },
  { &hf_rrc_drxInterruption_hs_dsch, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_HS_DSCH_DrxCellfach_info(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_HS_DSCH_DrxCellfach_info, HS_DSCH_DrxCellfach_info_sequence);

  return offset;
}


static const per_sequence_t HS_DSCH_DrxCellfach_info_TDD128_sequence[] = {
  { &hf_rrc_t_321           , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_321 },
  { &hf_rrc_hs_dsch_DrxCycleFach, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_HS_DSCH_DrxCycleFach },
  { &hf_rrc_hs_dsch_DrxBurstFach, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_HS_DSCH_DrxBurstFach },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_HS_DSCH_DrxCellfach_info_TDD128(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_HS_DSCH_DrxCellfach_info_TDD128, HS_DSCH_DrxCellfach_info_TDD128_sequence);

  return offset;
}


static const value_string rrc_HS_DSCH_SecondDrx_CycleFach_vals[] = {
  {   0, "f64" },
  {   1, "f128" },
  {   2, "f256" },
  {   3, "f512" },
  { 0, NULL }
};


static int
dissect_rrc_HS_DSCH_SecondDrx_CycleFach(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     4, NULL, FALSE, 0, NULL);

  return offset;
}


static const value_string rrc_HS_DSCH_Second_Rx_BurstFach_vals[] = {
  {   0, "f1" },
  {   1, "f2" },
  { 0, NULL }
};


static int
dissect_rrc_HS_DSCH_Second_Rx_BurstFach(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     2, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t T_one_level_DRX_sequence[] = {
  { &hf_rrc_hs_dsch_Second_Rx_BurstFach, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_HS_DSCH_Second_Rx_BurstFach },
  { &hf_rrc_t329            , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_329 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_one_level_DRX(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_one_level_DRX, T_one_level_DRX_sequence);

  return offset;
}


static const value_string rrc_HS_DSCH_First_Rx_BurstFach_vals[] = {
  {   0, "f0p4" },
  {   1, "f0p8" },
  { 0, NULL }
};


static int
dissect_rrc_HS_DSCH_First_Rx_BurstFach(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     2, NULL, FALSE, 0, NULL);

  return offset;
}


static const value_string rrc_HS_DSCH_First_Drx_CycleFach_vals[] = {
  {   0, "f2" },
  {   1, "f4" },
  {   2, "f8" },
  {   3, "f16" },
  {   4, "f32" },
  {   5, "f64" },
  { 0, NULL }
};


static int
dissect_rrc_HS_DSCH_First_Drx_CycleFach(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     6, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t T_two_level_DRX_sequence[] = {
  { &hf_rrc_t328            , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_328 },
  { &hf_rrc_hs_dsch_First_Rx_BurstFach, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_HS_DSCH_First_Rx_BurstFach },
  { &hf_rrc_hs_dsch_First_Drx_CycleFach, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_HS_DSCH_First_Drx_CycleFach },
  { &hf_rrc_hs_dsch_Second_Rx_BurstFach, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_HS_DSCH_Second_Rx_BurstFach },
  { &hf_rrc_t329            , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_329 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_two_level_DRX(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_two_level_DRX, T_two_level_DRX_sequence);

  return offset;
}


static const value_string rrc_T_drx_level_vals[] = {
  {   0, "one-level-DRX" },
  {   1, "two-level-DRX" },
  { 0, NULL }
};

static const per_choice_t T_drx_level_choice[] = {
  {   0, &hf_rrc_one_level_DRX   , ASN1_NO_EXTENSIONS     , dissect_rrc_T_one_level_DRX },
  {   1, &hf_rrc_two_level_DRX   , ASN1_NO_EXTENSIONS     , dissect_rrc_T_two_level_DRX },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_drx_level(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_drx_level, T_drx_level_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t HS_DSCH_DrxCellfach_SecondDrx_info_sequence[] = {
  { &hf_rrc_hs_dsch_SecondDrx_CycleFach, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_HS_DSCH_SecondDrx_CycleFach },
  { &hf_rrc_drx_level       , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_drx_level },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_HS_DSCH_DrxCellfach_SecondDrx_info(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_HS_DSCH_DrxCellfach_SecondDrx_info, HS_DSCH_DrxCellfach_SecondDrx_info_sequence);

  return offset;
}


static const value_string rrc_PI_CountPerFrame_vals[] = {
  {   0, "e18" },
  {   1, "e36" },
  {   2, "e72" },
  {   3, "e144" },
  { 0, NULL }
};


static int
dissect_rrc_PI_CountPerFrame(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     4, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t T_fdd_102_sequence[] = {
  { &hf_rrc_channelisationCode256, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_ChannelisationCode256 },
  { &hf_rrc_pi_CountPerFrame, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PI_CountPerFrame },
  { &hf_rrc_sttd_Indicator  , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_fdd_102(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_fdd_102, T_fdd_102_sequence);

  return offset;
}


static const value_string rrc_TDD_PICH_CCode_vals[] = {
  {   0, "cc16-1" },
  {   1, "cc16-2" },
  {   2, "cc16-3" },
  {   3, "cc16-4" },
  {   4, "cc16-5" },
  {   5, "cc16-6" },
  {   6, "cc16-7" },
  {   7, "cc16-8" },
  {   8, "cc16-9" },
  {   9, "cc16-10" },
  {  10, "cc16-11" },
  {  11, "cc16-12" },
  {  12, "cc16-13" },
  {  13, "cc16-14" },
  {  14, "cc16-15" },
  {  15, "cc16-16" },
  { 0, NULL }
};


static int
dissect_rrc_TDD_PICH_CCode(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     16, NULL, FALSE, 0, NULL);

  return offset;
}


static const value_string rrc_RepPerLengthOffset_PICH_vals[] = {
  {   0, "rpp4-2" },
  {   1, "rpp8-2" },
  {   2, "rpp8-4" },
  {   3, "rpp16-2" },
  {   4, "rpp16-4" },
  {   5, "rpp32-2" },
  {   6, "rpp32-4" },
  {   7, "rpp64-2" },
  {   8, "rpp64-4" },
  { 0, NULL }
};

static const per_choice_t RepPerLengthOffset_PICH_choice[] = {
  {   0, &hf_rrc_rpp4_2          , ASN1_NO_EXTENSIONS     , dissect_rrc_INTEGER_0_3 },
  {   1, &hf_rrc_rpp8_2          , ASN1_NO_EXTENSIONS     , dissect_rrc_INTEGER_0_7 },
  {   2, &hf_rrc_rpp8_4          , ASN1_NO_EXTENSIONS     , dissect_rrc_INTEGER_0_7 },
  {   3, &hf_rrc_rpp16_2         , ASN1_NO_EXTENSIONS     , dissect_rrc_INTEGER_0_15 },
  {   4, &hf_rrc_rpp16_4         , ASN1_NO_EXTENSIONS     , dissect_rrc_INTEGER_0_15 },
  {   5, &hf_rrc_rpp32_2         , ASN1_NO_EXTENSIONS     , dissect_rrc_INTEGER_0_31 },
  {   6, &hf_rrc_rpp32_4         , ASN1_NO_EXTENSIONS     , dissect_rrc_INTEGER_0_31 },
  {   7, &hf_rrc_rpp64_2         , ASN1_NO_EXTENSIONS     , dissect_rrc_INTEGER_0_63 },
  {   8, &hf_rrc_rpp64_4         , ASN1_NO_EXTENSIONS     , dissect_rrc_INTEGER_0_63 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_RepPerLengthOffset_PICH(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_RepPerLengthOffset_PICH, RepPerLengthOffset_PICH_choice,
                                 NULL);

  return offset;
}


static const value_string rrc_PagingIndicatorLength_vals[] = {
  {   0, "pi4" },
  {   1, "pi8" },
  {   2, "pi16" },
  { 0, NULL }
};


static int
dissect_rrc_PagingIndicatorLength(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     3, NULL, FALSE, 0, NULL);

  return offset;
}


static const value_string rrc_N_GAP_vals[] = {
  {   0, "f2" },
  {   1, "f4" },
  {   2, "f8" },
  { 0, NULL }
};


static int
dissect_rrc_N_GAP(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     3, NULL, FALSE, 0, NULL);

  return offset;
}



static int
dissect_rrc_N_PCH(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            1U, 8U, NULL, FALSE);

  return offset;
}


static const per_sequence_t T_tdd_57_sequence[] = {
  { &hf_rrc_channelisationCode_07, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_TDD_PICH_CCode },
  { &hf_rrc_timeslot        , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_TimeslotNumber },
  { &hf_rrc_midambleShiftAndBurstType_04, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MidambleShiftAndBurstType },
  { &hf_rrc_repetitionPeriodLengthOffset, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RepPerLengthOffset_PICH },
  { &hf_rrc_pagingIndicatorLength, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_PagingIndicatorLength },
  { &hf_rrc_n_GAP           , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_N_GAP },
  { &hf_rrc_n_PCH           , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_N_PCH },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_tdd_57(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_tdd_57, T_tdd_57_sequence);

  return offset;
}


static const value_string rrc_PICH_Info_vals[] = {
  {   0, "fdd" },
  {   1, "tdd" },
  { 0, NULL }
};

static const per_choice_t PICH_Info_choice[] = {
  {   0, &hf_rrc_fdd_104         , ASN1_NO_EXTENSIONS     , dissect_rrc_T_fdd_102 },
  {   1, &hf_rrc_tdd_61          , ASN1_NO_EXTENSIONS     , dissect_rrc_T_tdd_57 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_PICH_Info(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_PICH_Info, PICH_Info_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t PICH_ForHSDPASupportedPaging_sequence[] = {
  { &hf_rrc_hsdpa_AssociatedPichInfo, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PICH_Info },
  { &hf_rrc_hs_pdschChannelisationCode, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_1_15 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_PICH_ForHSDPASupportedPaging(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_PICH_ForHSDPASupportedPaging, PICH_ForHSDPASupportedPaging_sequence);

  return offset;
}


static const per_sequence_t SEQUENCE_SIZE_1_maxSCCPCH_OF_PICH_ForHSDPASupportedPaging_sequence_of[1] = {
  { &hf_rrc_pich_ForHSDPASupportedPagingList_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PICH_ForHSDPASupportedPaging },
};

static int
dissect_rrc_SEQUENCE_SIZE_1_maxSCCPCH_OF_PICH_ForHSDPASupportedPaging(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_SEQUENCE_SIZE_1_maxSCCPCH_OF_PICH_ForHSDPASupportedPaging, SEQUENCE_SIZE_1_maxSCCPCH_OF_PICH_ForHSDPASupportedPaging_sequence_of,
                                                  1, maxSCCPCH, FALSE);

  return offset;
}



static int
dissect_rrc_TransportBlockSizeIndex(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            1U, 32U, NULL, FALSE);

  return offset;
}


static const per_sequence_t SEQUENCE_SIZE_1_2_OF_TransportBlockSizeIndex_sequence_of[1] = {
  { &hf_rrc_transportBlockSizeList_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TransportBlockSizeIndex },
};

static int
dissect_rrc_SEQUENCE_SIZE_1_2_OF_TransportBlockSizeIndex(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_SEQUENCE_SIZE_1_2_OF_TransportBlockSizeIndex, SEQUENCE_SIZE_1_2_OF_TransportBlockSizeIndex_sequence_of,
                                                  1, 2, FALSE);

  return offset;
}


static const per_sequence_t HS_DSCH_PagingSystemInformation_sequence[] = {
  { &hf_rrc_dlScramblingCode, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_SecondaryScramblingCode },
  { &hf_rrc_pich_ForHSDPASupportedPagingList, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SEQUENCE_SIZE_1_maxSCCPCH_OF_PICH_ForHSDPASupportedPaging },
  { &hf_rrc_numberOfPcchTransmissions, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_1_5 },
  { &hf_rrc_transportBlockSizeList, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SEQUENCE_SIZE_1_2_OF_TransportBlockSizeIndex },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_HS_DSCH_PagingSystemInformation(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_HS_DSCH_PagingSystemInformation, HS_DSCH_PagingSystemInformation_sequence);

  return offset;
}


static const per_sequence_t T_implicit_02_sequence[] = {
  { &hf_rrc_occurrenceSequenceNumberOfPICH, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_OccurrenceSequenceNumberOfPICH },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_implicit_02(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_implicit_02, T_implicit_02_sequence);

  return offset;
}


static const per_sequence_t PichChannelisationCodeList_LCR_r4_sequence_of[1] = {
  { &hf_rrc_PichChannelisationCodeList_LCR_r4_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DL_TS_ChannelisationCode },
};

static int
dissect_rrc_PichChannelisationCodeList_LCR_r4(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_PichChannelisationCodeList_LCR_r4, PichChannelisationCodeList_LCR_r4_sequence_of,
                                                  1, 2, FALSE);

  return offset;
}


static const per_sequence_t PICH_Info_LCR_r4_sequence[] = {
  { &hf_rrc_timeslot_01     , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_TimeslotNumber_LCR_r4 },
  { &hf_rrc_pichChannelisationCodeList_LCR_r4, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PichChannelisationCodeList_LCR_r4 },
  { &hf_rrc_midambleShiftAndBurstType_03, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MidambleShiftAndBurstType_LCR_r4 },
  { &hf_rrc_repetitionPeriodLengthOffset, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RepPerLengthOffset_PICH },
  { &hf_rrc_pagingIndicatorLength, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_PagingIndicatorLength },
  { &hf_rrc_n_GAP           , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_N_GAP },
  { &hf_rrc_n_PCH           , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_N_PCH },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_PICH_Info_LCR_r4(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_PICH_Info_LCR_r4, PICH_Info_LCR_r4_sequence);

  return offset;
}


static const value_string rrc_PICH_ForHSDPASupportedPaging_TDD128_vals[] = {
  {   0, "implicit" },
  {   1, "explicit" },
  { 0, NULL }
};

static const per_choice_t PICH_ForHSDPASupportedPaging_TDD128_choice[] = {
  {   0, &hf_rrc_implicit_03     , ASN1_NO_EXTENSIONS     , dissect_rrc_T_implicit_02 },
  {   1, &hf_rrc_explicit_05     , ASN1_NO_EXTENSIONS     , dissect_rrc_PICH_Info_LCR_r4 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_PICH_ForHSDPASupportedPaging_TDD128(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_PICH_ForHSDPASupportedPaging_TDD128, PICH_ForHSDPASupportedPaging_TDD128_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t SEQUENCE_SIZE_1_maxSCCPCH_OF_PICH_ForHSDPASupportedPaging_TDD128_sequence_of[1] = {
  { &hf_rrc_pich_ForHsdschList_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PICH_ForHSDPASupportedPaging_TDD128 },
};

static int
dissect_rrc_SEQUENCE_SIZE_1_maxSCCPCH_OF_PICH_ForHSDPASupportedPaging_TDD128(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_SEQUENCE_SIZE_1_maxSCCPCH_OF_PICH_ForHSDPASupportedPaging_TDD128, SEQUENCE_SIZE_1_maxSCCPCH_OF_PICH_ForHSDPASupportedPaging_TDD128_sequence_of,
                                                  1, maxSCCPCH, FALSE);

  return offset;
}


static const per_sequence_t T_paging_associatedHspdschInfo_item_sequence[] = {
  { &hf_rrc_hs_pdsch_MidambleConfiguration, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_HS_PDSCH_Midamble_Configuration_TDD128 },
  { &hf_rrc_timeslotResourceRelatedInfo_02, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_6 },
  { &hf_rrc_codeResourceInformation, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CodeResourceInformation_TDD128 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_paging_associatedHspdschInfo_item(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_paging_associatedHspdschInfo_item, T_paging_associatedHspdschInfo_item_sequence);

  return offset;
}


static const per_sequence_t T_paging_associatedHspdschInfo_sequence_of[1] = {
  { &hf_rrc_paging_associatedHspdschInfo_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_paging_associatedHspdschInfo_item },
};

static int
dissect_rrc_T_paging_associatedHspdschInfo(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_T_paging_associatedHspdschInfo, T_paging_associatedHspdschInfo_sequence_of,
                                                  1, maxSCCPCH, FALSE);

  return offset;
}


static const per_sequence_t PCCH_InformationList_sequence[] = {
  { &hf_rrc_paging_associatedHspdschInfo, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_paging_associatedHspdschInfo },
  { &hf_rrc_paging_sub_Channel_size, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_1_3 },
  { &hf_rrc_transportBlockSizeList, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SEQUENCE_SIZE_1_2_OF_TransportBlockSizeIndex },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_PCCH_InformationList(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_PCCH_InformationList, PCCH_InformationList_sequence);

  return offset;
}


static const per_sequence_t HS_DSCH_PagingSystemInformation_TDD128_sequence[] = {
  { &hf_rrc_pich_ForHsdschList, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SEQUENCE_SIZE_1_maxSCCPCH_OF_PICH_ForHSDPASupportedPaging_TDD128 },
  { &hf_rrc_dtch_DCCH_reception_window_size, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_1_16 },
  { &hf_rrc_pcch_InformationList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_PCCH_InformationList },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_HS_DSCH_PagingSystemInformation_TDD128(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_HS_DSCH_PagingSystemInformation_TDD128, HS_DSCH_PagingSystemInformation_TDD128_sequence);

  return offset;
}


static const value_string rrc_T_modulation_02_vals[] = {
  {   0, "mod-QPSK" },
  {   1, "mod-8PSK" },
  { 0, NULL }
};


static int
dissect_rrc_T_modulation_02(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     2, NULL, FALSE, 0, NULL);

  return offset;
}


static const value_string rrc_T_ss_TPC_Symbols_01_vals[] = {
  {   0, "zero" },
  {   1, "one" },
  {   2, "sixteenOverSF" },
  { 0, NULL }
};


static int
dissect_rrc_T_ss_TPC_Symbols_01(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     3, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t IndividualTimeslotInfo_LCR_r4_ext_sequence[] = {
  { &hf_rrc_midambleShiftAndBurstType_03, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MidambleShiftAndBurstType_LCR_r4 },
  { &hf_rrc_modulation_02   , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_modulation_02 },
  { &hf_rrc_ss_TPC_Symbols_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_ss_TPC_Symbols_01 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_IndividualTimeslotInfo_LCR_r4_ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_IndividualTimeslotInfo_LCR_r4_ext, IndividualTimeslotInfo_LCR_r4_ext_sequence);

  return offset;
}


static const value_string rrc_T_hs_dpcch_TransmitContinuationOffset_vals[] = {
  {   0, "ms10" },
  {   1, "ms20" },
  {   2, "ms30" },
  {   3, "ms40" },
  {   4, "ms80" },
  {   5, "ms160" },
  {   6, "ms320" },
  {   7, "ms800" },
  { 0, NULL }
};


static int
dissect_rrc_T_hs_dpcch_TransmitContinuationOffset(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     8, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t NodeB_Trigger_HS_DPCCH_Transmission_sequence[] = {
  { &hf_rrc_hs_dpcch_TransmitContinuationOffset, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_hs_dpcch_TransmitContinuationOffset },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_NodeB_Trigger_HS_DPCCH_Transmission(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_NodeB_Trigger_HS_DPCCH_Transmission, NodeB_Trigger_HS_DPCCH_Transmission_sequence);

  return offset;
}


static const per_sequence_t OpenLoopPowerControl_TDD_sequence[] = {
  { &hf_rrc_primaryCCPCH_TX_Power, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PrimaryCCPCH_TX_Power },
  { &hf_rrc_alpha           , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_Alpha },
  { &hf_rrc_prach_ConstantValue, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_ConstantValueTdd },
  { &hf_rrc_dpch_ConstantValue, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_ConstantValueTdd },
  { &hf_rrc_pusch_ConstantValue, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_ConstantValueTdd },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_OpenLoopPowerControl_TDD(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_OpenLoopPowerControl_TDD, OpenLoopPowerControl_TDD_sequence);

  return offset;
}


static const per_sequence_t PDSCH_Info_LCR_r4_sequence[] = {
  { &hf_rrc_tfcs_ID_01      , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_TFCS_IdentityPlain },
  { &hf_rrc_commonTimeslotInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CommonTimeslotInfo },
  { &hf_rrc_pdsch_TimeslotsCodes_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DownlinkTimeslotsCodes_LCR_r4 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_PDSCH_Info_LCR_r4(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_PDSCH_Info_LCR_r4, PDSCH_Info_LCR_r4_sequence);

  return offset;
}


static const per_sequence_t PDSCH_SysInfo_sequence[] = {
  { &hf_rrc_pdsch_Identity  , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PDSCH_Identity },
  { &hf_rrc_pdsch_Info      , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PDSCH_Info },
  { &hf_rrc_dsch_TFS        , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_TransportFormatSet },
  { &hf_rrc_dsch_TFCS       , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_TFCS },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_PDSCH_SysInfo(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_PDSCH_SysInfo, PDSCH_SysInfo_sequence);

  return offset;
}


static const per_sequence_t PDSCH_SysInfo_VHCR_r7_sequence[] = {
  { &hf_rrc_pdsch_Identity  , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PDSCH_Identity },
  { &hf_rrc_pdsch_Info_02   , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PDSCH_Info_r7 },
  { &hf_rrc_dsch_TransportChannelsInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DSCH_TransportChannelsInfo },
  { &hf_rrc_dsch_TFCS       , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_TFCS },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_PDSCH_SysInfo_VHCR_r7(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_PDSCH_SysInfo_VHCR_r7, PDSCH_SysInfo_VHCR_r7_sequence);

  return offset;
}


static const per_sequence_t PDSCH_SysInfo_HCR_r5_sequence[] = {
  { &hf_rrc_pdsch_Identity  , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PDSCH_Identity },
  { &hf_rrc_pdsch_Info      , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PDSCH_Info },
  { &hf_rrc_dsch_TransportChannelsInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DSCH_TransportChannelsInfo },
  { &hf_rrc_dsch_TFCS       , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_TFCS },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_PDSCH_SysInfo_HCR_r5(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_PDSCH_SysInfo_HCR_r5, PDSCH_SysInfo_HCR_r5_sequence);

  return offset;
}


static const per_sequence_t PDSCH_SysInfo_LCR_r4_sequence[] = {
  { &hf_rrc_pdsch_Identity  , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PDSCH_Identity },
  { &hf_rrc_pdsch_Info_03   , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PDSCH_Info_LCR_r4 },
  { &hf_rrc_dsch_TFS        , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_TransportFormatSet },
  { &hf_rrc_dsch_TFCS       , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_TFCS },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_PDSCH_SysInfo_LCR_r4(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_PDSCH_SysInfo_LCR_r4, PDSCH_SysInfo_LCR_r4_sequence);

  return offset;
}


static const per_sequence_t PDSCH_SysInfoList_sequence_of[1] = {
  { &hf_rrc_PDSCH_SysInfoList_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PDSCH_SysInfo },
};

static int
dissect_rrc_PDSCH_SysInfoList(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_PDSCH_SysInfoList, PDSCH_SysInfoList_sequence_of,
                                                  1, maxPDSCH, FALSE);

  return offset;
}


static const per_sequence_t PDSCH_SysInfoList_VHCR_r7_sequence_of[1] = {
  { &hf_rrc_PDSCH_SysInfoList_VHCR_r7_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PDSCH_SysInfo_VHCR_r7 },
};

static int
dissect_rrc_PDSCH_SysInfoList_VHCR_r7(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_PDSCH_SysInfoList_VHCR_r7, PDSCH_SysInfoList_VHCR_r7_sequence_of,
                                                  1, maxPDSCH, FALSE);

  return offset;
}


static const per_sequence_t PDSCH_SysInfoList_HCR_r5_sequence_of[1] = {
  { &hf_rrc_PDSCH_SysInfoList_HCR_r5_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PDSCH_SysInfo_HCR_r5 },
};

static int
dissect_rrc_PDSCH_SysInfoList_HCR_r5(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_PDSCH_SysInfoList_HCR_r5, PDSCH_SysInfoList_HCR_r5_sequence_of,
                                                  1, maxPDSCH, FALSE);

  return offset;
}


static const per_sequence_t PDSCH_SysInfoList_LCR_r4_sequence_of[1] = {
  { &hf_rrc_PDSCH_SysInfoList_LCR_r4_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PDSCH_SysInfo_LCR_r4 },
};

static int
dissect_rrc_PDSCH_SysInfoList_LCR_r4(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_PDSCH_SysInfoList_LCR_r4, PDSCH_SysInfoList_LCR_r4_sequence_of,
                                                  1, maxPDSCH, FALSE);

  return offset;
}


static const per_sequence_t SFN_TimeInfo_sequence[] = {
  { &hf_rrc_activationTimeSFN, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_0_4095 },
  { &hf_rrc_physChDuration  , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DurationTimeInfo },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_SFN_TimeInfo(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_SFN_TimeInfo, SFN_TimeInfo_sequence);

  return offset;
}


static const per_sequence_t PDSCH_SysInfoList_SFN_item_sequence[] = {
  { &hf_rrc_pdsch_SysInfo   , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PDSCH_SysInfo },
  { &hf_rrc_sfn_TimeInfo    , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_SFN_TimeInfo },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_PDSCH_SysInfoList_SFN_item(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_PDSCH_SysInfoList_SFN_item, PDSCH_SysInfoList_SFN_item_sequence);

  return offset;
}


static const per_sequence_t PDSCH_SysInfoList_SFN_sequence_of[1] = {
  { &hf_rrc_PDSCH_SysInfoList_SFN_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PDSCH_SysInfoList_SFN_item },
};

static int
dissect_rrc_PDSCH_SysInfoList_SFN(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_PDSCH_SysInfoList_SFN, PDSCH_SysInfoList_SFN_sequence_of,
                                                  1, maxPDSCH, FALSE);

  return offset;
}


static const per_sequence_t PDSCH_SysInfoList_SFN_HCR_r5_item_sequence[] = {
  { &hf_rrc_pdsch_SysInfo_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PDSCH_SysInfo_HCR_r5 },
  { &hf_rrc_sfn_TimeInfo    , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_SFN_TimeInfo },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_PDSCH_SysInfoList_SFN_HCR_r5_item(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_PDSCH_SysInfoList_SFN_HCR_r5_item, PDSCH_SysInfoList_SFN_HCR_r5_item_sequence);

  return offset;
}


static const per_sequence_t PDSCH_SysInfoList_SFN_HCR_r5_sequence_of[1] = {
  { &hf_rrc_PDSCH_SysInfoList_SFN_HCR_r5_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PDSCH_SysInfoList_SFN_HCR_r5_item },
};

static int
dissect_rrc_PDSCH_SysInfoList_SFN_HCR_r5(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_PDSCH_SysInfoList_SFN_HCR_r5, PDSCH_SysInfoList_SFN_HCR_r5_sequence_of,
                                                  1, maxPDSCH, FALSE);

  return offset;
}


static const per_sequence_t PDSCH_SysInfoList_SFN_LCR_r4_item_sequence[] = {
  { &hf_rrc_pdsch_SysInfo_02, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PDSCH_SysInfo_LCR_r4 },
  { &hf_rrc_sfn_TimeInfo    , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_SFN_TimeInfo },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_PDSCH_SysInfoList_SFN_LCR_r4_item(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_PDSCH_SysInfoList_SFN_LCR_r4_item, PDSCH_SysInfoList_SFN_LCR_r4_item_sequence);

  return offset;
}


static const per_sequence_t PDSCH_SysInfoList_SFN_LCR_r4_sequence_of[1] = {
  { &hf_rrc_PDSCH_SysInfoList_SFN_LCR_r4_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PDSCH_SysInfoList_SFN_LCR_r4_item },
};

static int
dissect_rrc_PDSCH_SysInfoList_SFN_LCR_r4(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_PDSCH_SysInfoList_SFN_LCR_r4, PDSCH_SysInfoList_SFN_LCR_r4_sequence_of,
                                                  1, maxPDSCH, FALSE);

  return offset;
}


static const value_string rrc_TDD768_PICH_CCode_vals[] = {
  {   0, "cc32-1" },
  {   1, "cc32-2" },
  {   2, "cc32-3" },
  {   3, "cc32-4" },
  {   4, "cc32-5" },
  {   5, "cc32-6" },
  {   6, "cc32-7" },
  {   7, "cc32-8" },
  {   8, "cc32-9" },
  {   9, "cc32-10" },
  {  10, "cc32-11" },
  {  11, "cc32-12" },
  {  12, "cc32-13" },
  {  13, "cc32-14" },
  {  14, "cc32-15" },
  {  15, "cc32-16" },
  {  16, "cc32-17" },
  {  17, "cc32-18" },
  {  18, "cc32-19" },
  {  19, "cc32-20" },
  {  20, "cc32-21" },
  {  21, "cc32-22" },
  {  22, "cc32-23" },
  {  23, "cc32-24" },
  {  24, "cc32-25" },
  {  25, "cc32-26" },
  {  26, "cc32-27" },
  {  27, "cc32-28" },
  {  28, "cc32-29" },
  {  29, "cc32-30" },
  {  30, "cc32-31" },
  {  31, "cc32-32" },
  { 0, NULL }
};

static value_string_ext rrc_TDD768_PICH_CCode_vals_ext = VALUE_STRING_EXT_INIT(rrc_TDD768_PICH_CCode_vals);


static int
dissect_rrc_TDD768_PICH_CCode(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     32, NULL, FALSE, 0, NULL);

  return offset;
}


static const value_string rrc_T_channelisationCode_vals[] = {
  {   0, "tdd384" },
  {   1, "tdd768" },
  { 0, NULL }
};

static const per_choice_t T_channelisationCode_choice[] = {
  {   0, &hf_rrc_tdd384_31       , ASN1_NO_EXTENSIONS     , dissect_rrc_TDD_PICH_CCode },
  {   1, &hf_rrc_tdd768_20       , ASN1_NO_EXTENSIONS     , dissect_rrc_TDD768_PICH_CCode },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_channelisationCode(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_channelisationCode, T_channelisationCode_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t PICH_Info_HCR_VHCR_r7_sequence[] = {
  { &hf_rrc_channelisationCode_08, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_channelisationCode },
  { &hf_rrc_timeslot        , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_TimeslotNumber },
  { &hf_rrc_midambleShiftAndBurstType_04, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MidambleShiftAndBurstType },
  { &hf_rrc_repetitionPeriodLengthOffset, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RepPerLengthOffset_PICH },
  { &hf_rrc_pagingIndicatorLength, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_PagingIndicatorLength },
  { &hf_rrc_n_GAP           , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_N_GAP },
  { &hf_rrc_n_PCH           , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_N_PCH },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_PICH_Info_HCR_VHCR_r7(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_PICH_Info_HCR_VHCR_r7, PICH_Info_HCR_VHCR_r7_sequence);

  return offset;
}



static int
dissect_rrc_PICH_PowerOffset(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            -10, 5U, NULL, FALSE);

  return offset;
}


static const per_sequence_t PRACH_ChanCodes_LCR_r4_sequence_of[1] = {
  { &hf_rrc_PRACH_ChanCodes_LCR_r4_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TDD_PRACH_CCode_LCR_r4 },
};

static int
dissect_rrc_PRACH_ChanCodes_LCR_r4(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_PRACH_ChanCodes_LCR_r4, PRACH_ChanCodes_LCR_r4_sequence_of,
                                                  1, 4, FALSE);

  return offset;
}


static const per_sequence_t PRACH_Definition_LCR_r4_sequence[] = {
  { &hf_rrc_timeslot_02     , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TimeslotNumber_PRACH_LCR_r4 },
  { &hf_rrc_prach_ChanCodes_LCR, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PRACH_ChanCodes_LCR_r4 },
  { &hf_rrc_midambleShiftAndBurstType_03, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MidambleShiftAndBurstType_LCR_r4 },
  { &hf_rrc_fpach_Info      , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_FPACH_Info_r4 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_PRACH_Definition_LCR_r4(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_PRACH_Definition_LCR_r4, PRACH_Definition_LCR_r4_sequence);

  return offset;
}


static const value_string rrc_PRACH_Midamble_vals[] = {
  {   0, "direct" },
  {   1, "direct-Inverted" },
  { 0, NULL }
};


static int
dissect_rrc_PRACH_Midamble(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     2, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t SEQUENCE_SIZE_1_maxASC_OF_ASCSetting_TDD_sequence_of[1] = {
  { &hf_rrc_tdd_item        , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_ASCSetting_TDD },
};

static int
dissect_rrc_SEQUENCE_SIZE_1_maxASC_OF_ASCSetting_TDD(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_SEQUENCE_SIZE_1_maxASC_OF_ASCSetting_TDD, SEQUENCE_SIZE_1_maxASC_OF_ASCSetting_TDD_sequence_of,
                                                  1, maxASC, FALSE);

  return offset;
}


static const value_string rrc_PRACH_Partitioning_vals[] = {
  {   0, "fdd" },
  {   1, "tdd" },
  { 0, NULL }
};

static const per_choice_t PRACH_Partitioning_choice[] = {
  {   0, &hf_rrc_fdd_105         , ASN1_NO_EXTENSIONS     , dissect_rrc_SEQUENCE_SIZE_1_maxASC_OF_ASCSetting_FDD },
  {   1, &hf_rrc_tdd_62          , ASN1_NO_EXTENSIONS     , dissect_rrc_SEQUENCE_SIZE_1_maxASC_OF_ASCSetting_TDD },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_PRACH_Partitioning(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_PRACH_Partitioning, PRACH_Partitioning_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t PRACH_PreambleForEnhancedUplinkExt_sequence[] = {
  { &hf_rrc_availableSignatures, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_AvailableSignatures },
  { &hf_rrc_preambleScramblingCodeWordNumber, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PreambleScramblingCodeWordNumber },
  { &hf_rrc_aich_Info_01    , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_AICH_Info_Compressed },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_PRACH_PreambleForEnhancedUplinkExt(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_PRACH_PreambleForEnhancedUplinkExt, PRACH_PreambleForEnhancedUplinkExt_sequence);

  return offset;
}



static int
dissect_rrc_INTEGER_0_5(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            0U, 5U, NULL, FALSE);

  return offset;
}


static const per_sequence_t PRACH_PreambleForEnhancedUplinkExtWithWeight_sequence[] = {
  { &hf_rrc_prach_PreambleForEnhancedUplinkExt, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PRACH_PreambleForEnhancedUplinkExt },
  { &hf_rrc_weight          , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_INTEGER_0_5 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_PRACH_PreambleForEnhancedUplinkExtWithWeight(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_PRACH_PreambleForEnhancedUplinkExtWithWeight, PRACH_PreambleForEnhancedUplinkExtWithWeight_sequence);

  return offset;
}


static const per_sequence_t SEQUENCE_SIZE_1_maxPRACH_EUL_OF_PRACH_PreambleForEnhancedUplinkExtWithWeight_sequence_of[1] = {
  { &hf_rrc_prach_PreambleForEnhancedUplinkList_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PRACH_PreambleForEnhancedUplinkExtWithWeight },
};

static int
dissect_rrc_SEQUENCE_SIZE_1_maxPRACH_EUL_OF_PRACH_PreambleForEnhancedUplinkExtWithWeight(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_SEQUENCE_SIZE_1_maxPRACH_EUL_OF_PRACH_PreambleForEnhancedUplinkExtWithWeight, SEQUENCE_SIZE_1_maxPRACH_EUL_OF_PRACH_PreambleForEnhancedUplinkExtWithWeight_sequence_of,
                                                  1, maxPRACH_EUL, FALSE);

  return offset;
}


static const per_sequence_t PRACH_PreambleForEnhancedUplinkExtList_sequence[] = {
  { &hf_rrc_prach_PreambleForEnhancedUplinkList, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SEQUENCE_SIZE_1_maxPRACH_EUL_OF_PRACH_PreambleForEnhancedUplinkExtWithWeight },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_PRACH_PreambleForEnhancedUplinkExtList(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_PRACH_PreambleForEnhancedUplinkExtList, PRACH_PreambleForEnhancedUplinkExtList_sequence);

  return offset;
}



static int
dissect_rrc_PRACH_PreambleForEnhancedUplinkListType1Ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_rrc_PRACH_PreambleForEnhancedUplinkExtList(tvb, offset, actx, tree, hf_index);

  return offset;
}



static int
dissect_rrc_PRACH_PreambleForEnhancedUplinkListType2Ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_rrc_PRACH_PreambleForEnhancedUplinkExtList(tvb, offset, actx, tree, hf_index);

  return offset;
}



static int
dissect_rrc_PRACH_PreambleForEnhancedUplinkListType3Ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_rrc_PRACH_PreambleForEnhancedUplinkExtList(tvb, offset, actx, tree, hf_index);

  return offset;
}


static const value_string rrc_SF_PRACH_vals[] = {
  {   0, "sfpr32" },
  {   1, "sfpr64" },
  {   2, "sfpr128" },
  {   3, "sfpr256" },
  { 0, NULL }
};


static int
dissect_rrc_SF_PRACH(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     4, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t T_fdd_103_sequence[] = {
  { &hf_rrc_availableSignatures, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_AvailableSignatures },
  { &hf_rrc_availableSF     , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SF_PRACH },
  { &hf_rrc_preambleScramblingCodeWordNumber, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PreambleScramblingCodeWordNumber },
  { &hf_rrc_puncturingLimit , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PuncturingLimit },
  { &hf_rrc_availableSubChannelNumbers, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_AvailableSubChannelNumbers },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_fdd_103(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_fdd_103, T_fdd_103_sequence);

  return offset;
}


static const value_string rrc_TDD_PRACH_CCode8_vals[] = {
  {   0, "cc8-1" },
  {   1, "cc8-2" },
  {   2, "cc8-3" },
  {   3, "cc8-4" },
  {   4, "cc8-5" },
  {   5, "cc8-6" },
  {   6, "cc8-7" },
  {   7, "cc8-8" },
  { 0, NULL }
};


static int
dissect_rrc_TDD_PRACH_CCode8(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     8, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t SEQUENCE_SIZE_1_8_OF_TDD_PRACH_CCode8_sequence_of[1] = {
  { &hf_rrc_sf8_item        , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TDD_PRACH_CCode8 },
};

static int
dissect_rrc_SEQUENCE_SIZE_1_8_OF_TDD_PRACH_CCode8(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_SEQUENCE_SIZE_1_8_OF_TDD_PRACH_CCode8, SEQUENCE_SIZE_1_8_OF_TDD_PRACH_CCode8_sequence_of,
                                                  1, 8, FALSE);

  return offset;
}


static const value_string rrc_TDD_PRACH_CCode16_vals[] = {
  {   0, "cc16-1" },
  {   1, "cc16-2" },
  {   2, "cc16-3" },
  {   3, "cc16-4" },
  {   4, "cc16-5" },
  {   5, "cc16-6" },
  {   6, "cc16-7" },
  {   7, "cc16-8" },
  {   8, "cc16-9" },
  {   9, "cc16-10" },
  {  10, "cc16-11" },
  {  11, "cc16-12" },
  {  12, "cc16-13" },
  {  13, "cc16-14" },
  {  14, "cc16-15" },
  {  15, "cc16-16" },
  { 0, NULL }
};


static int
dissect_rrc_TDD_PRACH_CCode16(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     16, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t SEQUENCE_SIZE_1_8_OF_TDD_PRACH_CCode16_sequence_of[1] = {
  { &hf_rrc_sf16_item       , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TDD_PRACH_CCode16 },
};

static int
dissect_rrc_SEQUENCE_SIZE_1_8_OF_TDD_PRACH_CCode16(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_SEQUENCE_SIZE_1_8_OF_TDD_PRACH_CCode16, SEQUENCE_SIZE_1_8_OF_TDD_PRACH_CCode16_sequence_of,
                                                  1, 8, FALSE);

  return offset;
}


static const value_string rrc_TDD_PRACH_CCodeList_vals[] = {
  {   0, "sf8" },
  {   1, "sf16" },
  { 0, NULL }
};

static const per_choice_t TDD_PRACH_CCodeList_choice[] = {
  {   0, &hf_rrc_sf8_01          , ASN1_NO_EXTENSIONS     , dissect_rrc_SEQUENCE_SIZE_1_8_OF_TDD_PRACH_CCode8 },
  {   1, &hf_rrc_sf16_01         , ASN1_NO_EXTENSIONS     , dissect_rrc_SEQUENCE_SIZE_1_8_OF_TDD_PRACH_CCode16 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_TDD_PRACH_CCodeList(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_TDD_PRACH_CCodeList, TDD_PRACH_CCodeList_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t T_tdd_58_sequence[] = {
  { &hf_rrc_timeslot        , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TimeslotNumber },
  { &hf_rrc_channelisationCodeList, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TDD_PRACH_CCodeList },
  { &hf_rrc_prach_Midamble  , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PRACH_Midamble },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_tdd_58(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_tdd_58, T_tdd_58_sequence);

  return offset;
}


static const value_string rrc_T_modeSpecificInfo_72_vals[] = {
  {   0, "fdd" },
  {   1, "tdd" },
  { 0, NULL }
};

static const per_choice_t T_modeSpecificInfo_72_choice[] = {
  {   0, &hf_rrc_fdd_106         , ASN1_NO_EXTENSIONS     , dissect_rrc_T_fdd_103 },
  {   1, &hf_rrc_tdd_64          , ASN1_NO_EXTENSIONS     , dissect_rrc_T_tdd_58 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_modeSpecificInfo_72(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_modeSpecificInfo_72, T_modeSpecificInfo_72_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t PRACH_RACH_Info_sequence[] = {
  { &hf_rrc_modeSpecificInfo_72, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_modeSpecificInfo_72 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_PRACH_RACH_Info(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_PRACH_RACH_Info, PRACH_RACH_Info_sequence);

  return offset;
}


static const value_string rrc_TDD768_PRACH_CCode32_vals[] = {
  {   0, "cc32-1" },
  {   1, "cc32-2" },
  {   2, "cc32-3" },
  {   3, "cc32-4" },
  {   4, "cc32-5" },
  {   5, "cc32-6" },
  {   6, "cc32-7" },
  {   7, "cc32-8" },
  {   8, "cc32-9" },
  {   9, "cc32-10" },
  {  10, "cc32-11" },
  {  11, "cc32-12" },
  {  12, "cc32-13" },
  {  13, "cc32-14" },
  {  14, "cc32-15" },
  {  15, "cc32-16" },
  { 0, NULL }
};


static int
dissect_rrc_TDD768_PRACH_CCode32(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     16, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t SEQUENCE_SIZE_1_16_OF_TDD768_PRACH_CCode32_sequence_of[1] = {
  { &hf_rrc_sf32_item       , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TDD768_PRACH_CCode32 },
};

static int
dissect_rrc_SEQUENCE_SIZE_1_16_OF_TDD768_PRACH_CCode32(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_SEQUENCE_SIZE_1_16_OF_TDD768_PRACH_CCode32, SEQUENCE_SIZE_1_16_OF_TDD768_PRACH_CCode32_sequence_of,
                                                  1, 16, FALSE);

  return offset;
}


static const value_string rrc_TDD768_PRACH_CCode16_vals[] = {
  {   0, "cc16-1" },
  {   1, "cc16-2" },
  {   2, "cc16-3" },
  {   3, "cc16-4" },
  {   4, "cc16-5" },
  {   5, "cc16-6" },
  {   6, "cc16-7" },
  {   7, "cc16-8" },
  {   8, "cc16-9" },
  {   9, "cc16-10" },
  {  10, "cc16-11" },
  {  11, "cc16-12" },
  {  12, "cc16-13" },
  {  13, "cc16-14" },
  {  14, "cc16-15" },
  {  15, "cc16-16" },
  { 0, NULL }
};


static int
dissect_rrc_TDD768_PRACH_CCode16(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     16, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t SEQUENCE_SIZE_1_16_OF_TDD768_PRACH_CCode16_sequence_of[1] = {
  { &hf_rrc_sf16_item_01    , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TDD768_PRACH_CCode16 },
};

static int
dissect_rrc_SEQUENCE_SIZE_1_16_OF_TDD768_PRACH_CCode16(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_SEQUENCE_SIZE_1_16_OF_TDD768_PRACH_CCode16, SEQUENCE_SIZE_1_16_OF_TDD768_PRACH_CCode16_sequence_of,
                                                  1, 16, FALSE);

  return offset;
}


static const value_string rrc_TDD768_PRACH_CCodeList_vals[] = {
  {   0, "sf32" },
  {   1, "sf16" },
  { 0, NULL }
};

static const per_choice_t TDD768_PRACH_CCodeList_choice[] = {
  {   0, &hf_rrc_sf32_01         , ASN1_NO_EXTENSIONS     , dissect_rrc_SEQUENCE_SIZE_1_16_OF_TDD768_PRACH_CCode32 },
  {   1, &hf_rrc_sf16_02         , ASN1_NO_EXTENSIONS     , dissect_rrc_SEQUENCE_SIZE_1_16_OF_TDD768_PRACH_CCode16 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_TDD768_PRACH_CCodeList(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_TDD768_PRACH_CCodeList, TDD768_PRACH_CCodeList_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t PRACH_RACH_Info_VHCR_r7_sequence[] = {
  { &hf_rrc_timeslot        , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TimeslotNumber },
  { &hf_rrc_channelisationCodeList_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TDD768_PRACH_CCodeList },
  { &hf_rrc_prach_Midamble  , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PRACH_Midamble },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_PRACH_RACH_Info_VHCR_r7(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_PRACH_RACH_Info_VHCR_r7, PRACH_RACH_Info_VHCR_r7_sequence);

  return offset;
}


static const per_sequence_t PRACH_RACH_Info_LCR_v770ext_sequence[] = {
  { &hf_rrc_e_RUCCH_Sync_UL_Codes_Bitmap, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_Sync_UL_Codes_Bitmap },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_PRACH_RACH_Info_LCR_v770ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_PRACH_RACH_Info_LCR_v770ext, PRACH_RACH_Info_LCR_v770ext_sequence);

  return offset;
}


static const value_string rrc_T_max_SYNC_UL_Transmissions_01_vals[] = {
  {   0, "tr1" },
  {   1, "tr2" },
  {   2, "tr4" },
  {   3, "tr8" },
  { 0, NULL }
};


static int
dissect_rrc_T_max_SYNC_UL_Transmissions_01(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     4, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t SYNC_UL_Info_r4_sequence[] = {
  { &hf_rrc_sync_UL_Codes_Bitmap, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_Sync_UL_Codes_Bitmap },
  { &hf_rrc_prxUpPCHdes     , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_0_62 },
  { &hf_rrc_powerRampStep_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_0_3 },
  { &hf_rrc_max_SYNC_UL_Transmissions_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_max_SYNC_UL_Transmissions_01 },
  { &hf_rrc_mmax            , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_1_32 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_SYNC_UL_Info_r4(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_SYNC_UL_Info_r4, SYNC_UL_Info_r4_sequence);

  return offset;
}


static const per_sequence_t SEQUENCE_SIZE_1_maxPRACH_FPACH_OF_PRACH_Definition_LCR_r4_sequence_of[1] = {
  { &hf_rrc_prach_DefinitionList_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PRACH_Definition_LCR_r4 },
};

static int
dissect_rrc_SEQUENCE_SIZE_1_maxPRACH_FPACH_OF_PRACH_Definition_LCR_r4(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_SEQUENCE_SIZE_1_maxPRACH_FPACH_OF_PRACH_Definition_LCR_r4, SEQUENCE_SIZE_1_maxPRACH_FPACH_OF_PRACH_Definition_LCR_r4_sequence_of,
                                                  1, maxPRACH_FPACH, FALSE);

  return offset;
}


static const per_sequence_t PRACH_RACH_Info_LCR_r4_sequence[] = {
  { &hf_rrc_sync_UL_Info    , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SYNC_UL_Info_r4 },
  { &hf_rrc_prach_DefinitionList, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SEQUENCE_SIZE_1_maxPRACH_FPACH_OF_PRACH_Definition_LCR_r4 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_PRACH_RACH_Info_LCR_r4(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_PRACH_RACH_Info_LCR_r4, PRACH_RACH_Info_LCR_r4_sequence);

  return offset;
}


static const per_sequence_t T_fdd_104_sequence[] = {
  { &hf_rrc_primaryCPICH_TX_Power, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_PrimaryCPICH_TX_Power },
  { &hf_rrc_constantValue   , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_ConstantValue },
  { &hf_rrc_prach_PowerOffset, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_PRACH_PowerOffset },
  { &hf_rrc_rach_TransmissionParameters, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RACH_TransmissionParameters },
  { &hf_rrc_aich_Info       , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_AICH_Info },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_fdd_104(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_fdd_104, T_fdd_104_sequence);

  return offset;
}


static const value_string rrc_T_modeSpecificInfo_73_vals[] = {
  {   0, "fdd" },
  {   1, "tdd" },
  { 0, NULL }
};

static const per_choice_t T_modeSpecificInfo_73_choice[] = {
  {   0, &hf_rrc_fdd_107         , ASN1_NO_EXTENSIONS     , dissect_rrc_T_fdd_104 },
  {   1, &hf_rrc_tdd             , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_modeSpecificInfo_73(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_modeSpecificInfo_73, T_modeSpecificInfo_73_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t PRACH_SystemInformation_sequence[] = {
  { &hf_rrc_prach_RACH_Info , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PRACH_RACH_Info },
  { &hf_rrc_transportChannelIdentity, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TransportChannelIdentity },
  { &hf_rrc_rach_TransportFormatSet, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_TransportFormatSet },
  { &hf_rrc_rach_TFCS       , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_TFCS },
  { &hf_rrc_prach_Partitioning_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_PRACH_Partitioning },
  { &hf_rrc_persistenceScalingFactorList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_PersistenceScalingFactorList },
  { &hf_rrc_ac_To_ASC_MappingTable, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_AC_To_ASC_MappingTable },
  { &hf_rrc_modeSpecificInfo_73, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_modeSpecificInfo_73 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_PRACH_SystemInformation(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_PRACH_SystemInformation, PRACH_SystemInformation_sequence);

  return offset;
}


static const per_sequence_t PRACH_SystemInformation_VHCR_r7_sequence[] = {
  { &hf_rrc_prach_RACH_Info_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PRACH_RACH_Info_VHCR_r7 },
  { &hf_rrc_prach_Partitioning, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_PRACH_Partitioning_r7 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_PRACH_SystemInformation_VHCR_r7(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_PRACH_SystemInformation_VHCR_r7, PRACH_SystemInformation_VHCR_r7_sequence);

  return offset;
}


static const per_sequence_t PRACH_SystemInformation_LCR_r4_sequence[] = {
  { &hf_rrc_prach_RACH_Info_LCR, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PRACH_RACH_Info_LCR_r4 },
  { &hf_rrc_rach_TransportFormatSet_LCR, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_TransportFormatSet_LCR },
  { &hf_rrc_prach_Partitioning_LCR, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_PRACH_Partitioning_LCR_r4 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_PRACH_SystemInformation_LCR_r4(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_PRACH_SystemInformation_LCR_r4, PRACH_SystemInformation_LCR_r4_sequence);

  return offset;
}


static const per_sequence_t PRACH_SystemInformation_LCR_v770ext_sequence[] = {
  { &hf_rrc_prach_RACH_Info_LCR_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PRACH_RACH_Info_LCR_v770ext },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_PRACH_SystemInformation_LCR_v770ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_PRACH_SystemInformation_LCR_v770ext, PRACH_SystemInformation_LCR_v770ext_sequence);

  return offset;
}


static const per_sequence_t PRACH_SystemInformationList_sequence_of[1] = {
  { &hf_rrc_PRACH_SystemInformationList_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PRACH_SystemInformation },
};

static int
dissect_rrc_PRACH_SystemInformationList(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_PRACH_SystemInformationList, PRACH_SystemInformationList_sequence_of,
                                                  1, maxPRACH, FALSE);

  return offset;
}


static const per_sequence_t PRACH_SystemInformationList_VHCR_r7_sequence_of[1] = {
  { &hf_rrc_PRACH_SystemInformationList_VHCR_r7_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PRACH_SystemInformation_VHCR_r7 },
};

static int
dissect_rrc_PRACH_SystemInformationList_VHCR_r7(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_PRACH_SystemInformationList_VHCR_r7, PRACH_SystemInformationList_VHCR_r7_sequence_of,
                                                  1, maxPRACH, FALSE);

  return offset;
}


static const per_sequence_t PRACH_SystemInformationList_LCR_r4_sequence_of[1] = {
  { &hf_rrc_PRACH_SystemInformationList_LCR_r4_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PRACH_SystemInformation_LCR_r4 },
};

static int
dissect_rrc_PRACH_SystemInformationList_LCR_r4(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_PRACH_SystemInformationList_LCR_r4, PRACH_SystemInformationList_LCR_r4_sequence_of,
                                                  1, maxPRACH, FALSE);

  return offset;
}


static const per_sequence_t PRACH_SystemInformationList_LCR_v770ext_sequence_of[1] = {
  { &hf_rrc_PRACH_SystemInformationList_LCR_v770ext_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PRACH_SystemInformation_LCR_v770ext },
};

static int
dissect_rrc_PRACH_SystemInformationList_LCR_v770ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_PRACH_SystemInformationList_LCR_v770ext, PRACH_SystemInformationList_LCR_v770ext_sequence_of,
                                                  1, maxPRACH, FALSE);

  return offset;
}


static const per_sequence_t PrimaryCCPCH_Info_LCR_r4_ext_sequence[] = {
  { &hf_rrc_tstd_Indicator  , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_PrimaryCCPCH_Info_LCR_r4_ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_PrimaryCCPCH_Info_LCR_r4_ext, PrimaryCCPCH_Info_LCR_r4_ext_sequence);

  return offset;
}


static const per_sequence_t PUSCH_Info_LCR_r4_sequence[] = {
  { &hf_rrc_tfcs_ID_01      , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_TFCS_IdentityPlain },
  { &hf_rrc_commonTimeslotInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CommonTimeslotInfo },
  { &hf_rrc_pusch_TimeslotsCodes_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UplinkTimeslotsCodes_LCR_r4 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_PUSCH_Info_LCR_r4(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_PUSCH_Info_LCR_r4, PUSCH_Info_LCR_r4_sequence);

  return offset;
}


static const per_sequence_t PUSCH_SysInfo_sequence[] = {
  { &hf_rrc_pusch_Identity  , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PUSCH_Identity },
  { &hf_rrc_pusch_Info      , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PUSCH_Info },
  { &hf_rrc_usch_TFS        , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_TransportFormatSet },
  { &hf_rrc_usch_TFCS       , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_TFCS },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_PUSCH_SysInfo(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_PUSCH_SysInfo, PUSCH_SysInfo_sequence);

  return offset;
}


static const per_sequence_t PUSCH_SysInfo_VHCR_sequence[] = {
  { &hf_rrc_pusch_Identity  , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PUSCH_Identity },
  { &hf_rrc_pusch_Info_VHCR , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PUSCH_Info_VHCR },
  { &hf_rrc_usch_TransportChannelsInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_USCH_TransportChannelsInfo },
  { &hf_rrc_usch_TFCS       , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_TFCS },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_PUSCH_SysInfo_VHCR(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_PUSCH_SysInfo_VHCR, PUSCH_SysInfo_VHCR_sequence);

  return offset;
}


static const per_sequence_t PUSCH_SysInfo_HCR_r5_sequence[] = {
  { &hf_rrc_pusch_Identity  , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PUSCH_Identity },
  { &hf_rrc_pusch_Info      , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PUSCH_Info },
  { &hf_rrc_usch_TransportChannelsInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_USCH_TransportChannelsInfo },
  { &hf_rrc_usch_TFCS       , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_TFCS },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_PUSCH_SysInfo_HCR_r5(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_PUSCH_SysInfo_HCR_r5, PUSCH_SysInfo_HCR_r5_sequence);

  return offset;
}


static const per_sequence_t PUSCH_SysInfo_LCR_r4_sequence[] = {
  { &hf_rrc_pusch_Identity  , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PUSCH_Identity },
  { &hf_rrc_pusch_Info_03   , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PUSCH_Info_LCR_r4 },
  { &hf_rrc_usch_TFS        , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_TransportFormatSet },
  { &hf_rrc_usch_TFCS       , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_TFCS },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_PUSCH_SysInfo_LCR_r4(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_PUSCH_SysInfo_LCR_r4, PUSCH_SysInfo_LCR_r4_sequence);

  return offset;
}


static const per_sequence_t PUSCH_SysInfoList_sequence_of[1] = {
  { &hf_rrc_PUSCH_SysInfoList_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PUSCH_SysInfo },
};

static int
dissect_rrc_PUSCH_SysInfoList(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_PUSCH_SysInfoList, PUSCH_SysInfoList_sequence_of,
                                                  1, maxPUSCH, FALSE);

  return offset;
}


static const per_sequence_t PUSCH_SysInfoList_HCR_r5_sequence_of[1] = {
  { &hf_rrc_PUSCH_SysInfoList_HCR_r5_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PUSCH_SysInfo_HCR_r5 },
};

static int
dissect_rrc_PUSCH_SysInfoList_HCR_r5(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_PUSCH_SysInfoList_HCR_r5, PUSCH_SysInfoList_HCR_r5_sequence_of,
                                                  1, maxPUSCH, FALSE);

  return offset;
}


static const per_sequence_t PUSCH_SysInfoList_LCR_r4_sequence_of[1] = {
  { &hf_rrc_PUSCH_SysInfoList_LCR_r4_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PUSCH_SysInfo_LCR_r4 },
};

static int
dissect_rrc_PUSCH_SysInfoList_LCR_r4(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_PUSCH_SysInfoList_LCR_r4, PUSCH_SysInfoList_LCR_r4_sequence_of,
                                                  1, maxPUSCH, FALSE);

  return offset;
}


static const per_sequence_t PUSCH_SysInfoList_SFN_item_sequence[] = {
  { &hf_rrc_pusch_SysInfo   , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PUSCH_SysInfo },
  { &hf_rrc_sfn_TimeInfo    , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_SFN_TimeInfo },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_PUSCH_SysInfoList_SFN_item(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_PUSCH_SysInfoList_SFN_item, PUSCH_SysInfoList_SFN_item_sequence);

  return offset;
}


static const per_sequence_t PUSCH_SysInfoList_SFN_sequence_of[1] = {
  { &hf_rrc_PUSCH_SysInfoList_SFN_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PUSCH_SysInfoList_SFN_item },
};

static int
dissect_rrc_PUSCH_SysInfoList_SFN(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_PUSCH_SysInfoList_SFN, PUSCH_SysInfoList_SFN_sequence_of,
                                                  1, maxPUSCH, FALSE);

  return offset;
}


static const per_sequence_t PUSCH_SysInfoList_SFN_HCR_r5_item_sequence[] = {
  { &hf_rrc_pusch_SysInfo_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PUSCH_SysInfo_HCR_r5 },
  { &hf_rrc_sfn_TimeInfo    , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_SFN_TimeInfo },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_PUSCH_SysInfoList_SFN_HCR_r5_item(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_PUSCH_SysInfoList_SFN_HCR_r5_item, PUSCH_SysInfoList_SFN_HCR_r5_item_sequence);

  return offset;
}


static const per_sequence_t PUSCH_SysInfoList_SFN_HCR_r5_sequence_of[1] = {
  { &hf_rrc_PUSCH_SysInfoList_SFN_HCR_r5_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PUSCH_SysInfoList_SFN_HCR_r5_item },
};

static int
dissect_rrc_PUSCH_SysInfoList_SFN_HCR_r5(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_PUSCH_SysInfoList_SFN_HCR_r5, PUSCH_SysInfoList_SFN_HCR_r5_sequence_of,
                                                  1, maxPUSCH, FALSE);

  return offset;
}


static const per_sequence_t PUSCH_SysInfoList_SFN_LCR_r4_item_sequence[] = {
  { &hf_rrc_pusch_SysInfo_02, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PUSCH_SysInfo_LCR_r4 },
  { &hf_rrc_sfn_TimeInfo    , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_SFN_TimeInfo },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_PUSCH_SysInfoList_SFN_LCR_r4_item(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_PUSCH_SysInfoList_SFN_LCR_r4_item, PUSCH_SysInfoList_SFN_LCR_r4_item_sequence);

  return offset;
}


static const per_sequence_t PUSCH_SysInfoList_SFN_LCR_r4_sequence_of[1] = {
  { &hf_rrc_PUSCH_SysInfoList_SFN_LCR_r4_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PUSCH_SysInfoList_SFN_LCR_r4_item },
};

static int
dissect_rrc_PUSCH_SysInfoList_SFN_LCR_r4(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_PUSCH_SysInfoList_SFN_LCR_r4, PUSCH_SysInfoList_SFN_LCR_r4_sequence_of,
                                                  1, maxPUSCH, FALSE);

  return offset;
}


static const per_sequence_t PUSCH_SysInfoList_SFN_VHCR_item_sequence[] = {
  { &hf_rrc_pusch_SysInfo_VHCR, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PUSCH_SysInfo_VHCR },
  { &hf_rrc_sfn_TimeInfo    , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_SFN_TimeInfo },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_PUSCH_SysInfoList_SFN_VHCR_item(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_PUSCH_SysInfoList_SFN_VHCR_item, PUSCH_SysInfoList_SFN_VHCR_item_sequence);

  return offset;
}


static const per_sequence_t PUSCH_SysInfoList_SFN_VHCR_sequence_of[1] = {
  { &hf_rrc_PUSCH_SysInfoList_SFN_VHCR_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PUSCH_SysInfoList_SFN_VHCR_item },
};

static int
dissect_rrc_PUSCH_SysInfoList_SFN_VHCR(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_PUSCH_SysInfoList_SFN_VHCR, PUSCH_SysInfoList_SFN_VHCR_sequence_of,
                                                  1, maxPUSCH, FALSE);

  return offset;
}


static const value_string rrc_SCCPCH_ChannelisationCode_VHCR_vals[] = {
  {   0, "cc32-1" },
  {   1, "cc32-2" },
  {   2, "cc32-3" },
  {   3, "cc32-4" },
  {   4, "cc32-5" },
  {   5, "cc32-6" },
  {   6, "cc32-7" },
  {   7, "cc32-8" },
  {   8, "cc32-9" },
  {   9, "cc32-10" },
  {  10, "cc32-11" },
  {  11, "cc32-12" },
  {  12, "cc32-13" },
  {  13, "cc32-14" },
  {  14, "cc32-15" },
  {  15, "cc32-16" },
  {  16, "cc32-17" },
  {  17, "cc32-18" },
  {  18, "cc32-19" },
  {  19, "cc32-20" },
  {  20, "cc32-21" },
  {  21, "cc32-22" },
  {  22, "cc32-23" },
  {  23, "cc32-24" },
  {  24, "cc32-25" },
  {  25, "cc32-26" },
  {  26, "cc32-27" },
  {  27, "cc32-28" },
  {  28, "cc32-29" },
  {  29, "cc32-30" },
  {  30, "cc32-31" },
  {  31, "cc32-32" },
  { 0, NULL }
};

static value_string_ext rrc_SCCPCH_ChannelisationCode_VHCR_vals_ext = VALUE_STRING_EXT_INIT(rrc_SCCPCH_ChannelisationCode_VHCR_vals);


static int
dissect_rrc_SCCPCH_ChannelisationCode_VHCR(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     32, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t SCCPCH_ChannelisationCodeList_VHCR_sequence_of[1] = {
  { &hf_rrc_SCCPCH_ChannelisationCodeList_VHCR_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SCCPCH_ChannelisationCode_VHCR },
};

static int
dissect_rrc_SCCPCH_ChannelisationCodeList_VHCR(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_SCCPCH_ChannelisationCodeList_VHCR, SCCPCH_ChannelisationCodeList_VHCR_sequence_of,
                                                  1, 32, FALSE);

  return offset;
}


static const per_sequence_t SCCPCH_SystemInformation_sequence[] = {
  { &hf_rrc_secondaryCCPCH_Info, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SecondaryCCPCH_Info },
  { &hf_rrc_tfcs            , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_TFCS },
  { &hf_rrc_fach_PCH_InformationList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_FACH_PCH_InformationList },
  { &hf_rrc_pich_Info       , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_PICH_Info },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_SCCPCH_SystemInformation(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_SCCPCH_SystemInformation, SCCPCH_SystemInformation_sequence);

  return offset;
}


static const per_sequence_t SecondaryCCPCH_Info_LCR_r4_ext_sequence[] = {
  { &hf_rrc_individualTimeslotLCR_Ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_IndividualTimeslotInfo_LCR_r4_ext },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_SecondaryCCPCH_Info_LCR_r4_ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_SecondaryCCPCH_Info_LCR_r4_ext, SecondaryCCPCH_Info_LCR_r4_ext_sequence);

  return offset;
}


static const per_sequence_t SCCPCH_SystemInformation_LCR_r4_ext_sequence[] = {
  { &hf_rrc_secondaryCCPCH_LCR_Extensions, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SecondaryCCPCH_Info_LCR_r4_ext },
  { &hf_rrc_pich_Info_01    , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_PICH_Info_LCR_r4 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_SCCPCH_SystemInformation_LCR_r4_ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_SCCPCH_SystemInformation_LCR_r4_ext, SCCPCH_SystemInformation_LCR_r4_ext_sequence);

  return offset;
}


static const per_sequence_t T_tdd384_31_sequence[] = {
  { &hf_rrc_individualTimeslotInfo_03, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_IndividualTimeslotInfo_r7 },
  { &hf_rrc_channelisationCode_09, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SCCPCH_ChannelisationCodeList },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_tdd384_31(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_tdd384_31, T_tdd384_31_sequence);

  return offset;
}


static const per_sequence_t T_tdd768_18_sequence[] = {
  { &hf_rrc_individualTimeslotInfo_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_IndividualTimeslotInfo_VHCR },
  { &hf_rrc_channelisationCode_10, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SCCPCH_ChannelisationCodeList_VHCR },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_tdd768_18(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_tdd768_18, T_tdd768_18_sequence);

  return offset;
}


static const value_string rrc_T_modeSpecificInfo_78_vals[] = {
  {   0, "tdd384" },
  {   1, "tdd768" },
  { 0, NULL }
};

static const per_choice_t T_modeSpecificInfo_78_choice[] = {
  {   0, &hf_rrc_tdd384_34       , ASN1_NO_EXTENSIONS     , dissect_rrc_T_tdd384_31 },
  {   1, &hf_rrc_tdd768_21       , ASN1_NO_EXTENSIONS     , dissect_rrc_T_tdd768_18 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_modeSpecificInfo_78(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_modeSpecificInfo_78, T_modeSpecificInfo_78_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t SecondaryCCPCH_Info_HCR_VHCR_r7_sequence[] = {
  { &hf_rrc_modeSpecificInfo_78, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_modeSpecificInfo_78 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_SecondaryCCPCH_Info_HCR_VHCR_r7(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_SecondaryCCPCH_Info_HCR_VHCR_r7, SecondaryCCPCH_Info_HCR_VHCR_r7_sequence);

  return offset;
}


static const per_sequence_t SCCPCH_SystemInformation_HCR_VHCR_r7_sequence[] = {
  { &hf_rrc_secondaryCCPCH_Info_02, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SecondaryCCPCH_Info_HCR_VHCR_r7 },
  { &hf_rrc_tfcs            , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_TFCS },
  { &hf_rrc_fach_PCH_InformationList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_FACH_PCH_InformationList },
  { &hf_rrc_pich_Info_02    , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_PICH_Info_HCR_VHCR_r7 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_SCCPCH_SystemInformation_HCR_VHCR_r7(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_SCCPCH_SystemInformation_HCR_VHCR_r7, SCCPCH_SystemInformation_HCR_VHCR_r7_sequence);

  return offset;
}


static const per_sequence_t SCCPCH_SystemInformationList_sequence_of[1] = {
  { &hf_rrc_SCCPCH_SystemInformationList_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SCCPCH_SystemInformation },
};

static int
dissect_rrc_SCCPCH_SystemInformationList(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_SCCPCH_SystemInformationList, SCCPCH_SystemInformationList_sequence_of,
                                                  1, maxSCCPCH, FALSE);

  return offset;
}


static const per_sequence_t SCCPCH_SystemInformationList_HCR_VHCR_r7_sequence_of[1] = {
  { &hf_rrc_SCCPCH_SystemInformationList_HCR_VHCR_r7_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SCCPCH_SystemInformation_HCR_VHCR_r7 },
};

static int
dissect_rrc_SCCPCH_SystemInformationList_HCR_VHCR_r7(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_SCCPCH_SystemInformationList_HCR_VHCR_r7, SCCPCH_SystemInformationList_HCR_VHCR_r7_sequence_of,
                                                  1, maxSCCPCH, FALSE);

  return offset;
}


static const per_sequence_t SCCPCH_SystemInformationList_LCR_r4_ext_sequence_of[1] = {
  { &hf_rrc_SCCPCH_SystemInformationList_LCR_r4_ext_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SCCPCH_SystemInformation_LCR_r4_ext },
};

static int
dissect_rrc_SCCPCH_SystemInformationList_LCR_r4_ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_SCCPCH_SystemInformationList_LCR_r4_ext, SCCPCH_SystemInformationList_LCR_r4_ext_sequence_of,
                                                  1, maxSCCPCH, FALSE);

  return offset;
}



static int
dissect_rrc_INTEGER_7_10(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            7U, 10U, NULL, FALSE);

  return offset;
}


static const per_sequence_t MBMS_MCCH_ConfigurationInfo_r6_sequence[] = {
  { &hf_rrc_accessInfoPeriodCoefficient, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_0_3 },
  { &hf_rrc_repetitionPeriodCoefficient, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_0_3 },
  { &hf_rrc_modificationPeriodCoefficient, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_7_10 },
  { &hf_rrc_rlc_Info_06     , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RLC_Info_MCCH_r6 },
  { &hf_rrc_tctf_Presence   , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MBMS_TCTF_Presence },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_MBMS_MCCH_ConfigurationInfo_r6(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_MBMS_MCCH_ConfigurationInfo_r6, MBMS_MCCH_ConfigurationInfo_r6_sequence);

  return offset;
}


static const per_sequence_t T_fachCarryingMCCH_sequence[] = {
  { &hf_rrc_mcch_transportFormatSet, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TransportFormatSet },
  { &hf_rrc_mcch_ConfigurationInfo, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MBMS_MCCH_ConfigurationInfo_r6 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_fachCarryingMCCH(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_fachCarryingMCCH, T_fachCarryingMCCH_sequence);

  return offset;
}


static const per_sequence_t MBMS_FACHCarryingMTCH_List_sequence_of[1] = {
  { &hf_rrc_MBMS_FACHCarryingMTCH_List_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TransportFormatSet },
};

static int
dissect_rrc_MBMS_FACHCarryingMTCH_List(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_MBMS_FACHCarryingMTCH_List, MBMS_FACHCarryingMTCH_List_sequence_of,
                                                  1, maxFACHPCH, FALSE);

  return offset;
}


static const per_sequence_t T_fachCarryingMSCH_sequence[] = {
  { &hf_rrc_msch_transportFormatSet, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TransportFormatSet },
  { &hf_rrc_dummy_45        , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MBMS_MSCH_ConfigurationInfo_r6 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_fachCarryingMSCH(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_fachCarryingMSCH, T_fachCarryingMSCH_sequence);

  return offset;
}


static const per_sequence_t SCCPCH_SystemInformation_MBMS_r6_sequence[] = {
  { &hf_rrc_secondaryCCPCHInfo_MBMS, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SecondaryCCPCHInfo_MBMS_r6 },
  { &hf_rrc_transportFormatCombinationSet, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TFCS },
  { &hf_rrc_fachCarryingMCCH, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_fachCarryingMCCH },
  { &hf_rrc_fachCarryingMTCH_List, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MBMS_FACHCarryingMTCH_List },
  { &hf_rrc_fachCarryingMSCH, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_fachCarryingMSCH },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_SCCPCH_SystemInformation_MBMS_r6(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_SCCPCH_SystemInformation_MBMS_r6, SCCPCH_SystemInformation_MBMS_r6_sequence);

  return offset;
}


static const per_sequence_t T_fachCarryingMCCH_01_sequence[] = {
  { &hf_rrc_mcch_transportFormatSet, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TransportFormatSet },
  { &hf_rrc_mcch_ConfigurationInfo, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MBMS_MCCH_ConfigurationInfo_r6 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_fachCarryingMCCH_01(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_fachCarryingMCCH_01, T_fachCarryingMCCH_01_sequence);

  return offset;
}


static const per_sequence_t T_fachCarryingMSCH_01_sequence[] = {
  { &hf_rrc_msch_transportFormatSet, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TransportFormatSet },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_fachCarryingMSCH_01(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_fachCarryingMSCH_01, T_fachCarryingMSCH_01_sequence);

  return offset;
}


static const per_sequence_t SCCPCH_SystemInformation_MBMS_r7_sequence[] = {
  { &hf_rrc_secondaryCCPCHInfo_MBMS_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SecondaryCCPCHInfo_MBMS_r7 },
  { &hf_rrc_transportFormatCombinationSet, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TFCS },
  { &hf_rrc_fachCarryingMCCH_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_fachCarryingMCCH_01 },
  { &hf_rrc_fachCarryingMTCH_List, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MBMS_FACHCarryingMTCH_List },
  { &hf_rrc_fachCarryingMSCH_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_fachCarryingMSCH_01 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_SCCPCH_SystemInformation_MBMS_r7(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_SCCPCH_SystemInformation_MBMS_r7, SCCPCH_SystemInformation_MBMS_r7_sequence);

  return offset;
}


static const per_sequence_t TDD_MBSFNTSlotInfo_sequence[] = {
  { &hf_rrc_timeSlotNumber  , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TimeslotNumber },
  { &hf_rrc_cellParametersID, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CellParametersID },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_TDD_MBSFNTSlotInfo(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_TDD_MBSFNTSlotInfo, TDD_MBSFNTSlotInfo_sequence);

  return offset;
}


static const per_sequence_t TDD_MBSFNInformation_sequence_of[1] = {
  { &hf_rrc_TDD_MBSFNInformation_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TDD_MBSFNTSlotInfo },
};

static int
dissect_rrc_TDD_MBSFNInformation(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_TDD_MBSFNInformation, TDD_MBSFNInformation_sequence_of,
                                                  1, maxTS, FALSE);

  return offset;
}


static const value_string rrc_CellDCH_ReportCriteria_vals[] = {
  {   0, "intraFreqReportingCriteria" },
  {   1, "periodicalReportingCriteria" },
  { 0, NULL }
};

static const per_choice_t CellDCH_ReportCriteria_choice[] = {
  {   0, &hf_rrc_intraFreqReportingCriteria, ASN1_NO_EXTENSIONS     , dissect_rrc_IntraFreqReportingCriteria },
  {   1, &hf_rrc_periodicalReportingCriteria, ASN1_NO_EXTENSIONS     , dissect_rrc_PeriodicalReportingCriteria },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_CellDCH_ReportCriteria(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_CellDCH_ReportCriteria, CellDCH_ReportCriteria_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t ForbiddenAffectCell_LCR_r4_sequence[] = {
  { &hf_rrc_tdd_107         , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PrimaryCCPCH_Info_LCR_r4 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_ForbiddenAffectCell_LCR_r4(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_ForbiddenAffectCell_LCR_r4, ForbiddenAffectCell_LCR_r4_sequence);

  return offset;
}


static const per_sequence_t ForbiddenAffectCellList_LCR_r4_sequence_of[1] = {
  { &hf_rrc_ForbiddenAffectCellList_LCR_r4_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_ForbiddenAffectCell_LCR_r4 },
};

static int
dissect_rrc_ForbiddenAffectCellList_LCR_r4(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_ForbiddenAffectCellList_LCR_r4, ForbiddenAffectCellList_LCR_r4_sequence_of,
                                                  1, maxCellMeas, FALSE);

  return offset;
}


static const per_sequence_t Event1a_LCR_r4_sequence[] = {
  { &hf_rrc_triggeringCondition, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TriggeringCondition2 },
  { &hf_rrc_reportingRange  , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_ReportingRange },
  { &hf_rrc_forbiddenAffectCellList_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_ForbiddenAffectCellList_LCR_r4 },
  { &hf_rrc_w               , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_W },
  { &hf_rrc_reportDeactivationThreshold, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_ReportDeactivationThreshold },
  { &hf_rrc_reportingAmount , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_ReportingAmount },
  { &hf_rrc_reportingInterval, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_ReportingInterval },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_Event1a_LCR_r4(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_Event1a_LCR_r4, Event1a_LCR_r4_sequence);

  return offset;
}


static const per_sequence_t Event1b_LCR_r4_sequence[] = {
  { &hf_rrc_triggeringCondition_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TriggeringCondition1 },
  { &hf_rrc_reportingRange  , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_ReportingRange },
  { &hf_rrc_forbiddenAffectCellList_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_ForbiddenAffectCellList_LCR_r4 },
  { &hf_rrc_w               , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_W },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_Event1b_LCR_r4(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_Event1b_LCR_r4, Event1b_LCR_r4_sequence);

  return offset;
}


static const value_string rrc_IntraFreqEvent_LCR_r4_vals[] = {
  {   0, "e1a" },
  {   1, "e1b" },
  {   2, "e1c" },
  {   3, "e1d" },
  {   4, "e1e" },
  {   5, "e1f" },
  {   6, "e1g" },
  {   7, "e1h" },
  {   8, "e1i" },
  { 0, NULL }
};

static const per_choice_t IntraFreqEvent_LCR_r4_choice[] = {
  {   0, &hf_rrc_e1a_02          , ASN1_NO_EXTENSIONS     , dissect_rrc_Event1a_LCR_r4 },
  {   1, &hf_rrc_e1b_02          , ASN1_NO_EXTENSIONS     , dissect_rrc_Event1b_LCR_r4 },
  {   2, &hf_rrc_e1c             , ASN1_NO_EXTENSIONS     , dissect_rrc_Event1c },
  {   3, &hf_rrc_e1d             , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   4, &hf_rrc_e1e             , ASN1_NO_EXTENSIONS     , dissect_rrc_Event1e },
  {   5, &hf_rrc_e1f             , ASN1_NO_EXTENSIONS     , dissect_rrc_Event1f },
  {   6, &hf_rrc_e1g             , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   7, &hf_rrc_e1h             , ASN1_NO_EXTENSIONS     , dissect_rrc_ThresholdUsedFrequency },
  {   8, &hf_rrc_e1i             , ASN1_NO_EXTENSIONS     , dissect_rrc_ThresholdUsedFrequency },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_IntraFreqEvent_LCR_r4(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_IntraFreqEvent_LCR_r4, IntraFreqEvent_LCR_r4_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t IntraFreqEventCriteria_LCR_r4_sequence[] = {
  { &hf_rrc_event_02        , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_IntraFreqEvent_LCR_r4 },
  { &hf_rrc_hysteresis_01   , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_Hysteresis },
  { &hf_rrc_timeToTrigger   , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TimeToTrigger },
  { &hf_rrc_reportingCellStatus, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_ReportingCellStatus },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_IntraFreqEventCriteria_LCR_r4(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_IntraFreqEventCriteria_LCR_r4, IntraFreqEventCriteria_LCR_r4_sequence);

  return offset;
}


static const per_sequence_t IntraFreqEventCriteriaList_LCR_r4_sequence_of[1] = {
  { &hf_rrc_IntraFreqEventCriteriaList_LCR_r4_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_IntraFreqEventCriteria_LCR_r4 },
};

static int
dissect_rrc_IntraFreqEventCriteriaList_LCR_r4(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_IntraFreqEventCriteriaList_LCR_r4, IntraFreqEventCriteriaList_LCR_r4_sequence_of,
                                                  1, maxMeasEvent, FALSE);

  return offset;
}


static const per_sequence_t IntraFreqReportingCriteria_LCR_r4_sequence[] = {
  { &hf_rrc_eventCriteriaList_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_IntraFreqEventCriteriaList_LCR_r4 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_IntraFreqReportingCriteria_LCR_r4(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_IntraFreqReportingCriteria_LCR_r4, IntraFreqReportingCriteria_LCR_r4_sequence);

  return offset;
}


static const value_string rrc_CellDCH_ReportCriteria_LCR_r4_vals[] = {
  {   0, "intraFreqReportingCriteria" },
  {   1, "periodicalReportingCriteria" },
  { 0, NULL }
};

static const per_choice_t CellDCH_ReportCriteria_LCR_r4_choice[] = {
  {   0, &hf_rrc_intraFreqReportingCriteria_01, ASN1_NO_EXTENSIONS     , dissect_rrc_IntraFreqReportingCriteria_LCR_r4 },
  {   1, &hf_rrc_periodicalReportingCriteria, ASN1_NO_EXTENSIONS     , dissect_rrc_PeriodicalReportingCriteria },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_CellDCH_ReportCriteria_LCR_r4(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_CellDCH_ReportCriteria_LCR_r4, CellDCH_ReportCriteria_LCR_r4_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t T_fdd_135_sequence[] = {
  { &hf_rrc_primaryCPICH_Info, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_PrimaryCPICH_Info },
  { &hf_rrc_primaryCPICH_TX_Power, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_PrimaryCPICH_TX_Power },
  { &hf_rrc_readSFN_Indicator, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { &hf_rrc_tx_DiversityIndicator, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_fdd_135(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_fdd_135, T_fdd_135_sequence);

  return offset;
}


static const per_sequence_t T_tdd_86_sequence[] = {
  { &hf_rrc_primaryCCPCH_Info_02, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PrimaryCCPCH_Info },
  { &hf_rrc_primaryCCPCH_TX_Power, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_PrimaryCCPCH_TX_Power },
  { &hf_rrc_timeslotInfoList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_TimeslotInfoList },
  { &hf_rrc_readSFN_Indicator, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_tdd_86(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_tdd_86, T_tdd_86_sequence);

  return offset;
}


static const value_string rrc_T_modeSpecificInfo_96_vals[] = {
  {   0, "fdd" },
  {   1, "tdd" },
  { 0, NULL }
};

static const per_choice_t T_modeSpecificInfo_96_choice[] = {
  {   0, &hf_rrc_fdd_138         , ASN1_NO_EXTENSIONS     , dissect_rrc_T_fdd_135 },
  {   1, &hf_rrc_tdd_92          , ASN1_NO_EXTENSIONS     , dissect_rrc_T_tdd_86 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_modeSpecificInfo_96(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_modeSpecificInfo_96, T_modeSpecificInfo_96_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t T_fdd_144_sequence[] = {
  { &hf_rrc_q_QualMin       , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_Q_QualMin },
  { &hf_rrc_q_RxlevMin      , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_Q_RxlevMin },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_fdd_144(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_fdd_144, T_fdd_144_sequence);

  return offset;
}


static const per_sequence_t T_tdd_95_sequence[] = {
  { &hf_rrc_q_RxlevMin      , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_Q_RxlevMin },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_tdd_95(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_tdd_95, T_tdd_95_sequence);

  return offset;
}


static const per_sequence_t T_gsm_04_sequence[] = {
  { &hf_rrc_q_RxlevMin      , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_Q_RxlevMin },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_gsm_04(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_gsm_04, T_gsm_04_sequence);

  return offset;
}


static const value_string rrc_T_modeSpecificInfo_105_vals[] = {
  {   0, "fdd" },
  {   1, "tdd" },
  {   2, "gsm" },
  { 0, NULL }
};

static const per_choice_t T_modeSpecificInfo_105_choice[] = {
  {   0, &hf_rrc_fdd_149         , ASN1_NO_EXTENSIONS     , dissect_rrc_T_fdd_144 },
  {   1, &hf_rrc_tdd_102         , ASN1_NO_EXTENSIONS     , dissect_rrc_T_tdd_95 },
  {   2, &hf_rrc_gsm_04          , ASN1_NO_EXTENSIONS     , dissect_rrc_T_gsm_04 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_modeSpecificInfo_105(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_modeSpecificInfo_105, T_modeSpecificInfo_105_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t CellSelectReselectInfoSIB_11_12_RSCP_sequence[] = {
  { &hf_rrc_q_OffsetS_N     , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_Q_OffsetS_N },
  { &hf_rrc_maxAllowedUL_TX_Power, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MaxAllowedUL_TX_Power },
  { &hf_rrc_modeSpecificInfo_105, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_modeSpecificInfo_105 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_CellSelectReselectInfoSIB_11_12_RSCP(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_CellSelectReselectInfoSIB_11_12_RSCP, CellSelectReselectInfoSIB_11_12_RSCP_sequence);

  return offset;
}


static const per_sequence_t CellInfoSI_RSCP_sequence[] = {
  { &hf_rrc_cellIndividualOffset, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CellIndividualOffset },
  { &hf_rrc_referenceTimeDifferenceToCell, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_ReferenceTimeDifferenceToCell },
  { &hf_rrc_modeSpecificInfo_96, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_modeSpecificInfo_96 },
  { &hf_rrc_cellSelectionReselectionInfo_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CellSelectReselectInfoSIB_11_12_RSCP },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_CellInfoSI_RSCP(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_CellInfoSI_RSCP, CellInfoSI_RSCP_sequence);

  return offset;
}


static const per_sequence_t TimeslotInfoList_LCR_r4_sequence_of[1] = {
  { &hf_rrc_TimeslotInfoList_LCR_r4_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TimeslotInfo_LCR_r4 },
};

static int
dissect_rrc_TimeslotInfoList_LCR_r4(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_TimeslotInfoList_LCR_r4, TimeslotInfoList_LCR_r4_sequence_of,
                                                  1, maxTS_LCR, FALSE);

  return offset;
}


static const per_sequence_t CellInfoSI_RSCP_LCR_r4_sequence[] = {
  { &hf_rrc_cellIndividualOffset, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CellIndividualOffset },
  { &hf_rrc_referenceTimeDifferenceToCell, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_ReferenceTimeDifferenceToCell },
  { &hf_rrc_primaryCCPCH_Info_04, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PrimaryCCPCH_Info_LCR_r4 },
  { &hf_rrc_primaryCCPCH_TX_Power, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_PrimaryCCPCH_TX_Power },
  { &hf_rrc_timeslotInfoList_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_TimeslotInfoList_LCR_r4 },
  { &hf_rrc_readSFN_Indicator, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { &hf_rrc_cellSelectionReselectionInfo_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CellSelectReselectInfoSIB_11_12_RSCP },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_CellInfoSI_RSCP_LCR_r4(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_CellInfoSI_RSCP_LCR_r4, CellInfoSI_RSCP_LCR_r4_sequence);

  return offset;
}


static const per_sequence_t T_fdd_136_sequence[] = {
  { &hf_rrc_primaryCPICH_Info, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_PrimaryCPICH_Info },
  { &hf_rrc_primaryCPICH_TX_Power, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_PrimaryCPICH_TX_Power },
  { &hf_rrc_readSFN_Indicator, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { &hf_rrc_tx_DiversityIndicator, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_fdd_136(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_fdd_136, T_fdd_136_sequence);

  return offset;
}


static const per_sequence_t T_tdd_87_sequence[] = {
  { &hf_rrc_primaryCCPCH_Info_02, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PrimaryCCPCH_Info },
  { &hf_rrc_primaryCCPCH_TX_Power, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_PrimaryCCPCH_TX_Power },
  { &hf_rrc_timeslotInfoList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_TimeslotInfoList },
  { &hf_rrc_readSFN_Indicator, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_tdd_87(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_tdd_87, T_tdd_87_sequence);

  return offset;
}


static const value_string rrc_T_modeSpecificInfo_97_vals[] = {
  {   0, "fdd" },
  {   1, "tdd" },
  { 0, NULL }
};

static const per_choice_t T_modeSpecificInfo_97_choice[] = {
  {   0, &hf_rrc_fdd_139         , ASN1_NO_EXTENSIONS     , dissect_rrc_T_fdd_136 },
  {   1, &hf_rrc_tdd_93          , ASN1_NO_EXTENSIONS     , dissect_rrc_T_tdd_87 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_modeSpecificInfo_97(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_modeSpecificInfo_97, T_modeSpecificInfo_97_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t T_fdd_145_sequence[] = {
  { &hf_rrc_q_QualMin       , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_Q_QualMin },
  { &hf_rrc_q_RxlevMin      , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_Q_RxlevMin },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_fdd_145(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_fdd_145, T_fdd_145_sequence);

  return offset;
}


static const per_sequence_t T_tdd_96_sequence[] = {
  { &hf_rrc_q_RxlevMin      , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_Q_RxlevMin },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_tdd_96(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_tdd_96, T_tdd_96_sequence);

  return offset;
}


static const per_sequence_t T_gsm_05_sequence[] = {
  { &hf_rrc_q_RxlevMin      , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_Q_RxlevMin },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_gsm_05(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_gsm_05, T_gsm_05_sequence);

  return offset;
}


static const value_string rrc_T_modeSpecificInfo_106_vals[] = {
  {   0, "fdd" },
  {   1, "tdd" },
  {   2, "gsm" },
  { 0, NULL }
};

static const per_choice_t T_modeSpecificInfo_106_choice[] = {
  {   0, &hf_rrc_fdd_150         , ASN1_NO_EXTENSIONS     , dissect_rrc_T_fdd_145 },
  {   1, &hf_rrc_tdd_103         , ASN1_NO_EXTENSIONS     , dissect_rrc_T_tdd_96 },
  {   2, &hf_rrc_gsm_05          , ASN1_NO_EXTENSIONS     , dissect_rrc_T_gsm_05 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_modeSpecificInfo_106(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_modeSpecificInfo_106, T_modeSpecificInfo_106_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t CellSelectReselectInfoSIB_11_12_ECN0_sequence[] = {
  { &hf_rrc_q_Offset1S_N    , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_Q_OffsetS_N },
  { &hf_rrc_q_Offset2S_N    , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_Q_OffsetS_N },
  { &hf_rrc_maxAllowedUL_TX_Power, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MaxAllowedUL_TX_Power },
  { &hf_rrc_modeSpecificInfo_106, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_modeSpecificInfo_106 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_CellSelectReselectInfoSIB_11_12_ECN0(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_CellSelectReselectInfoSIB_11_12_ECN0, CellSelectReselectInfoSIB_11_12_ECN0_sequence);

  return offset;
}


static const per_sequence_t CellInfoSI_ECN0_sequence[] = {
  { &hf_rrc_cellIndividualOffset, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CellIndividualOffset },
  { &hf_rrc_referenceTimeDifferenceToCell, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_ReferenceTimeDifferenceToCell },
  { &hf_rrc_modeSpecificInfo_97, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_modeSpecificInfo_97 },
  { &hf_rrc_cellSelectionReselectionInfo_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CellSelectReselectInfoSIB_11_12_ECN0 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_CellInfoSI_ECN0(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_CellInfoSI_ECN0, CellInfoSI_ECN0_sequence);

  return offset;
}


static const per_sequence_t CellInfoSI_ECN0_LCR_r4_sequence[] = {
  { &hf_rrc_cellIndividualOffset, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CellIndividualOffset },
  { &hf_rrc_referenceTimeDifferenceToCell, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_ReferenceTimeDifferenceToCell },
  { &hf_rrc_primaryCCPCH_Info_04, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PrimaryCCPCH_Info_LCR_r4 },
  { &hf_rrc_primaryCCPCH_TX_Power, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_PrimaryCCPCH_TX_Power },
  { &hf_rrc_timeslotInfoList_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_TimeslotInfoList_LCR_r4 },
  { &hf_rrc_readSFN_Indicator, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { &hf_rrc_cellSelectionReselectionInfo_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CellSelectReselectInfoSIB_11_12_ECN0 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_CellInfoSI_ECN0_LCR_r4(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_CellInfoSI_ECN0_LCR_r4, CellInfoSI_ECN0_LCR_r4_sequence);

  return offset;
}


static const per_sequence_t T_fdd_137_sequence[] = {
  { &hf_rrc_primaryCPICH_Info, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_PrimaryCPICH_Info },
  { &hf_rrc_primaryCPICH_TX_Power, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_PrimaryCPICH_TX_Power },
  { &hf_rrc_readSFN_Indicator, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { &hf_rrc_tx_DiversityIndicator, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_fdd_137(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_fdd_137, T_fdd_137_sequence);

  return offset;
}


static const per_sequence_t T_tdd_88_sequence[] = {
  { &hf_rrc_primaryCCPCH_Info_02, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PrimaryCCPCH_Info },
  { &hf_rrc_primaryCCPCH_TX_Power, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_PrimaryCCPCH_TX_Power },
  { &hf_rrc_timeslotInfoList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_TimeslotInfoList },
  { &hf_rrc_readSFN_Indicator, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_tdd_88(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_tdd_88, T_tdd_88_sequence);

  return offset;
}


static const value_string rrc_T_modeSpecificInfo_98_vals[] = {
  {   0, "fdd" },
  {   1, "tdd" },
  { 0, NULL }
};

static const per_choice_t T_modeSpecificInfo_98_choice[] = {
  {   0, &hf_rrc_fdd_140         , ASN1_NO_EXTENSIONS     , dissect_rrc_T_fdd_137 },
  {   1, &hf_rrc_tdd_94          , ASN1_NO_EXTENSIONS     , dissect_rrc_T_tdd_88 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_modeSpecificInfo_98(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_modeSpecificInfo_98, T_modeSpecificInfo_98_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t T_fdd_146_sequence[] = {
  { &hf_rrc_q_QualMin       , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_Q_QualMin },
  { &hf_rrc_q_RxlevMin      , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_Q_RxlevMin },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_fdd_146(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_fdd_146, T_fdd_146_sequence);

  return offset;
}


static const per_sequence_t T_tdd_97_sequence[] = {
  { &hf_rrc_q_RxlevMin      , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_Q_RxlevMin },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_tdd_97(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_tdd_97, T_tdd_97_sequence);

  return offset;
}


static const per_sequence_t T_gsm_06_sequence[] = {
  { &hf_rrc_q_RxlevMin      , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_Q_RxlevMin },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_gsm_06(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_gsm_06, T_gsm_06_sequence);

  return offset;
}


static const value_string rrc_T_modeSpecificInfo_107_vals[] = {
  {   0, "fdd" },
  {   1, "tdd" },
  {   2, "gsm" },
  { 0, NULL }
};

static const per_choice_t T_modeSpecificInfo_107_choice[] = {
  {   0, &hf_rrc_fdd_151         , ASN1_NO_EXTENSIONS     , dissect_rrc_T_fdd_146 },
  {   1, &hf_rrc_tdd_104         , ASN1_NO_EXTENSIONS     , dissect_rrc_T_tdd_97 },
  {   2, &hf_rrc_gsm_06          , ASN1_NO_EXTENSIONS     , dissect_rrc_T_gsm_06 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_modeSpecificInfo_107(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_modeSpecificInfo_107, T_modeSpecificInfo_107_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t CellSelectReselectInfoSIB_11_12_HCS_RSCP_sequence[] = {
  { &hf_rrc_q_OffsetS_N     , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_Q_OffsetS_N },
  { &hf_rrc_maxAllowedUL_TX_Power, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MaxAllowedUL_TX_Power },
  { &hf_rrc_hcs_NeighbouringCellInformation_RSCP, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_HCS_NeighbouringCellInformation_RSCP },
  { &hf_rrc_modeSpecificInfo_107, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_modeSpecificInfo_107 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_CellSelectReselectInfoSIB_11_12_HCS_RSCP(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_CellSelectReselectInfoSIB_11_12_HCS_RSCP, CellSelectReselectInfoSIB_11_12_HCS_RSCP_sequence);

  return offset;
}


static const per_sequence_t CellInfoSI_HCS_RSCP_sequence[] = {
  { &hf_rrc_cellIndividualOffset, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CellIndividualOffset },
  { &hf_rrc_referenceTimeDifferenceToCell, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_ReferenceTimeDifferenceToCell },
  { &hf_rrc_modeSpecificInfo_98, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_modeSpecificInfo_98 },
  { &hf_rrc_cellSelectionReselectionInfo_03, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CellSelectReselectInfoSIB_11_12_HCS_RSCP },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_CellInfoSI_HCS_RSCP(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_CellInfoSI_HCS_RSCP, CellInfoSI_HCS_RSCP_sequence);

  return offset;
}


static const per_sequence_t CellInfoSI_HCS_RSCP_LCR_r4_sequence[] = {
  { &hf_rrc_cellIndividualOffset, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CellIndividualOffset },
  { &hf_rrc_referenceTimeDifferenceToCell, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_ReferenceTimeDifferenceToCell },
  { &hf_rrc_primaryCCPCH_Info_04, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PrimaryCCPCH_Info_LCR_r4 },
  { &hf_rrc_primaryCCPCH_TX_Power, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_PrimaryCCPCH_TX_Power },
  { &hf_rrc_timeslotInfoList_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_TimeslotInfoList_LCR_r4 },
  { &hf_rrc_readSFN_Indicator, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { &hf_rrc_cellSelectionReselectionInfo_03, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CellSelectReselectInfoSIB_11_12_HCS_RSCP },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_CellInfoSI_HCS_RSCP_LCR_r4(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_CellInfoSI_HCS_RSCP_LCR_r4, CellInfoSI_HCS_RSCP_LCR_r4_sequence);

  return offset;
}


static const per_sequence_t T_fdd_138_sequence[] = {
  { &hf_rrc_primaryCPICH_Info, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_PrimaryCPICH_Info },
  { &hf_rrc_primaryCPICH_TX_Power, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_PrimaryCPICH_TX_Power },
  { &hf_rrc_readSFN_Indicator, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { &hf_rrc_tx_DiversityIndicator, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_fdd_138(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_fdd_138, T_fdd_138_sequence);

  return offset;
}


static const per_sequence_t T_tdd_89_sequence[] = {
  { &hf_rrc_primaryCCPCH_Info_02, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PrimaryCCPCH_Info },
  { &hf_rrc_primaryCCPCH_TX_Power, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_PrimaryCCPCH_TX_Power },
  { &hf_rrc_timeslotInfoList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_TimeslotInfoList },
  { &hf_rrc_readSFN_Indicator, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_tdd_89(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_tdd_89, T_tdd_89_sequence);

  return offset;
}


static const value_string rrc_T_modeSpecificInfo_99_vals[] = {
  {   0, "fdd" },
  {   1, "tdd" },
  { 0, NULL }
};

static const per_choice_t T_modeSpecificInfo_99_choice[] = {
  {   0, &hf_rrc_fdd_141         , ASN1_NO_EXTENSIONS     , dissect_rrc_T_fdd_138 },
  {   1, &hf_rrc_tdd_95          , ASN1_NO_EXTENSIONS     , dissect_rrc_T_tdd_89 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_modeSpecificInfo_99(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_modeSpecificInfo_99, T_modeSpecificInfo_99_choice,
                                 NULL);

  return offset;
}


static const value_string rrc_TemporaryOffset2_vals[] = {
  {   0, "to2" },
  {   1, "to3" },
  {   2, "to4" },
  {   3, "to6" },
  {   4, "to8" },
  {   5, "to10" },
  {   6, "to12" },
  {   7, "infinite" },
  { 0, NULL }
};


static int
dissect_rrc_TemporaryOffset2(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     8, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t TemporaryOffsetList_sequence[] = {
  { &hf_rrc_temporaryOffset1, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TemporaryOffset1 },
  { &hf_rrc_temporaryOffset2, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TemporaryOffset2 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_TemporaryOffsetList(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_TemporaryOffsetList, TemporaryOffsetList_sequence);

  return offset;
}


static const value_string rrc_PenaltyTime_ECN0_vals[] = {
  {   0, "notUsed" },
  {   1, "pt10" },
  {   2, "pt20" },
  {   3, "pt30" },
  {   4, "pt40" },
  {   5, "pt50" },
  {   6, "pt60" },
  { 0, NULL }
};

static const per_choice_t PenaltyTime_ECN0_choice[] = {
  {   0, &hf_rrc_notUsed         , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   1, &hf_rrc_pt10_01         , ASN1_NO_EXTENSIONS     , dissect_rrc_TemporaryOffsetList },
  {   2, &hf_rrc_pt20_01         , ASN1_NO_EXTENSIONS     , dissect_rrc_TemporaryOffsetList },
  {   3, &hf_rrc_pt30_01         , ASN1_NO_EXTENSIONS     , dissect_rrc_TemporaryOffsetList },
  {   4, &hf_rrc_pt40_01         , ASN1_NO_EXTENSIONS     , dissect_rrc_TemporaryOffsetList },
  {   5, &hf_rrc_pt50_01         , ASN1_NO_EXTENSIONS     , dissect_rrc_TemporaryOffsetList },
  {   6, &hf_rrc_pt60_01         , ASN1_NO_EXTENSIONS     , dissect_rrc_TemporaryOffsetList },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_PenaltyTime_ECN0(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_PenaltyTime_ECN0, PenaltyTime_ECN0_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t HCS_CellReselectInformation_ECN0_sequence[] = {
  { &hf_rrc_penaltyTime_01  , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PenaltyTime_ECN0 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_HCS_CellReselectInformation_ECN0(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_HCS_CellReselectInformation_ECN0, HCS_CellReselectInformation_ECN0_sequence);

  return offset;
}


static const per_sequence_t HCS_NeighbouringCellInformation_ECN0_sequence[] = {
  { &hf_rrc_hcs_PRIO        , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_HCS_PRIO },
  { &hf_rrc_q_HCS           , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_Q_HCS },
  { &hf_rrc_hcs_CellReselectInformation_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_HCS_CellReselectInformation_ECN0 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_HCS_NeighbouringCellInformation_ECN0(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_HCS_NeighbouringCellInformation_ECN0, HCS_NeighbouringCellInformation_ECN0_sequence);

  return offset;
}


static const per_sequence_t T_fdd_147_sequence[] = {
  { &hf_rrc_q_QualMin       , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_Q_QualMin },
  { &hf_rrc_q_RxlevMin      , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_Q_RxlevMin },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_fdd_147(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_fdd_147, T_fdd_147_sequence);

  return offset;
}


static const per_sequence_t T_tdd_98_sequence[] = {
  { &hf_rrc_q_RxlevMin      , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_Q_RxlevMin },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_tdd_98(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_tdd_98, T_tdd_98_sequence);

  return offset;
}


static const per_sequence_t T_gsm_07_sequence[] = {
  { &hf_rrc_q_RxlevMin      , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_Q_RxlevMin },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_gsm_07(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_gsm_07, T_gsm_07_sequence);

  return offset;
}


static const value_string rrc_T_modeSpecificInfo_108_vals[] = {
  {   0, "fdd" },
  {   1, "tdd" },
  {   2, "gsm" },
  { 0, NULL }
};

static const per_choice_t T_modeSpecificInfo_108_choice[] = {
  {   0, &hf_rrc_fdd_152         , ASN1_NO_EXTENSIONS     , dissect_rrc_T_fdd_147 },
  {   1, &hf_rrc_tdd_105         , ASN1_NO_EXTENSIONS     , dissect_rrc_T_tdd_98 },
  {   2, &hf_rrc_gsm_07          , ASN1_NO_EXTENSIONS     , dissect_rrc_T_gsm_07 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_modeSpecificInfo_108(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_modeSpecificInfo_108, T_modeSpecificInfo_108_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t CellSelectReselectInfoSIB_11_12_HCS_ECN0_sequence[] = {
  { &hf_rrc_q_Offset1S_N    , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_Q_OffsetS_N },
  { &hf_rrc_q_Offset2S_N    , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_Q_OffsetS_N },
  { &hf_rrc_maxAllowedUL_TX_Power, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MaxAllowedUL_TX_Power },
  { &hf_rrc_hcs_NeighbouringCellInformation_ECN0, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_HCS_NeighbouringCellInformation_ECN0 },
  { &hf_rrc_modeSpecificInfo_108, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_modeSpecificInfo_108 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_CellSelectReselectInfoSIB_11_12_HCS_ECN0(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_CellSelectReselectInfoSIB_11_12_HCS_ECN0, CellSelectReselectInfoSIB_11_12_HCS_ECN0_sequence);

  return offset;
}


static const per_sequence_t CellInfoSI_HCS_ECN0_sequence[] = {
  { &hf_rrc_cellIndividualOffset, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CellIndividualOffset },
  { &hf_rrc_referenceTimeDifferenceToCell, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_ReferenceTimeDifferenceToCell },
  { &hf_rrc_modeSpecificInfo_99, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_modeSpecificInfo_99 },
  { &hf_rrc_cellSelectionReselectionInfo_04, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CellSelectReselectInfoSIB_11_12_HCS_ECN0 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_CellInfoSI_HCS_ECN0(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_CellInfoSI_HCS_ECN0, CellInfoSI_HCS_ECN0_sequence);

  return offset;
}


static const per_sequence_t CellInfoSI_HCS_ECN0_LCR_r4_sequence[] = {
  { &hf_rrc_cellIndividualOffset, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CellIndividualOffset },
  { &hf_rrc_referenceTimeDifferenceToCell, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_ReferenceTimeDifferenceToCell },
  { &hf_rrc_primaryCCPCH_Info_04, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PrimaryCCPCH_Info_LCR_r4 },
  { &hf_rrc_primaryCCPCH_TX_Power, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_PrimaryCCPCH_TX_Power },
  { &hf_rrc_timeslotInfoList_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_TimeslotInfoList_LCR_r4 },
  { &hf_rrc_readSFN_Indicator, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { &hf_rrc_cellSelectionReselectionInfo_04, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CellSelectReselectInfoSIB_11_12_HCS_ECN0 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_CellInfoSI_HCS_ECN0_LCR_r4(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_CellInfoSI_HCS_ECN0_LCR_r4, CellInfoSI_HCS_ECN0_LCR_r4_sequence);

  return offset;
}



static int
dissect_rrc_DeltaQrxlevmin(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            -2, -1, NULL, FALSE);

  return offset;
}


static const per_sequence_t CellSelectReselectInfo_v590ext_sequence[] = {
  { &hf_rrc_deltaQrxlevmin  , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DeltaQrxlevmin },
  { &hf_rrc_deltaQhcs       , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DeltaRSCP },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_CellSelectReselectInfo_v590ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_CellSelectReselectInfo_v590ext, CellSelectReselectInfo_v590ext_sequence);

  return offset;
}


static const per_sequence_t CellSelectReselectInfoPCHFACH_v5b0ext_sequence[] = {
  { &hf_rrc_q_Hyst_l_S_PCH  , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_Q_Hyst_S_Fine },
  { &hf_rrc_q_Hyst_l_S_FACH , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_Q_Hyst_S_Fine },
  { &hf_rrc_q_Hyst_2_S_PCH  , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_Q_Hyst_S_Fine },
  { &hf_rrc_q_Hyst_2_S_FACH , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_Q_Hyst_S_Fine },
  { &hf_rrc_t_Reselection_S_PCH, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_Reselection_S },
  { &hf_rrc_t_Reselection_S_FACH, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_Reselection_S_Fine },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_CellSelectReselectInfoPCHFACH_v5b0ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_CellSelectReselectInfoPCHFACH_v5b0ext, CellSelectReselectInfoPCHFACH_v5b0ext_sequence);

  return offset;
}


static const per_sequence_t CellSelectReselectInfoTreselectionScaling_v5c0ext_sequence[] = {
  { &hf_rrc_non_HCS_t_CR_Max, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_CRMax },
  { &hf_rrc_speedDependentScalingFactor, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_SpeedDependentScalingFactor },
  { &hf_rrc_interFrequencyTreselectionScalingFactor, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_TreselectionScalingFactor },
  { &hf_rrc_interRATTreselectionScalingFactor, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_TreselectionScalingFactor },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_CellSelectReselectInfoTreselectionScaling_v5c0ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_CellSelectReselectInfoTreselectionScaling_v5c0ext, CellSelectReselectInfoTreselectionScaling_v5c0ext_sequence);

  return offset;
}



static int
dissect_rrc_INTEGER_M70_M22(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            -70, -22, NULL, FALSE);

  return offset;
}


static const per_sequence_t EUTRA_FrequencyAndPriorityInfo_sequence[] = {
  { &hf_rrc_earfcn          , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_EARFCN },
  { &hf_rrc_measurementBandwidth, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_EUTRA_MeasurementBandwidth },
  { &hf_rrc_priority        , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_0_maxPrio_1 },
  { &hf_rrc_qRxLevMinEUTRA  , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_M70_M22 },
  { &hf_rrc_threshXhigh     , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_0_31 },
  { &hf_rrc_threshXlow      , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_0_31 },
  { &hf_rrc_eutra_blackListedCellList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_EUTRA_BlacklistedCellPerFreqList },
  { &hf_rrc_eutraDetection  , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_EUTRA_FrequencyAndPriorityInfo(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_EUTRA_FrequencyAndPriorityInfo, EUTRA_FrequencyAndPriorityInfo_sequence);

  return offset;
}



static int
dissect_rrc_INTEGER_M34_M3(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            -34, -3, NULL, FALSE);

  return offset;
}


static const per_sequence_t EUTRA_FrequencyAndPriorityInfo_v920ext_sequence[] = {
  { &hf_rrc_qqualMinEUTRA   , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_INTEGER_M34_M3 },
  { &hf_rrc_threshXhigh2    , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_INTEGER_0_31 },
  { &hf_rrc_threshXlow2     , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_INTEGER_0_31 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_EUTRA_FrequencyAndPriorityInfo_v920ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_EUTRA_FrequencyAndPriorityInfo_v920ext, EUTRA_FrequencyAndPriorityInfo_v920ext_sequence);

  return offset;
}


static const per_sequence_t EUTRA_FrequencyAndPriorityInfoList_sequence_of[1] = {
  { &hf_rrc_EUTRA_FrequencyAndPriorityInfoList_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_EUTRA_FrequencyAndPriorityInfo },
};

static int
dissect_rrc_EUTRA_FrequencyAndPriorityInfoList(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_EUTRA_FrequencyAndPriorityInfoList, EUTRA_FrequencyAndPriorityInfoList_sequence_of,
                                                  1, maxNumEUTRAFreqs, FALSE);

  return offset;
}


static const per_sequence_t EUTRA_FrequencyAndPriorityInfoList_v920ext_sequence_of[1] = {
  { &hf_rrc_EUTRA_FrequencyAndPriorityInfoList_v920ext_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_EUTRA_FrequencyAndPriorityInfo_v920ext },
};

static int
dissect_rrc_EUTRA_FrequencyAndPriorityInfoList_v920ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_EUTRA_FrequencyAndPriorityInfoList_v920ext, EUTRA_FrequencyAndPriorityInfoList_v920ext_sequence_of,
                                                  1, maxNumEUTRAFreqs, FALSE);

  return offset;
}


static const per_sequence_t EUTRA_FrequencyAndPriorityInfoList_vbxyext_sequence[] = {
  { &hf_rrc_eutraTreselectionScalingFactor, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_TreselectionScalingFactor2 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_EUTRA_FrequencyAndPriorityInfoList_vbxyext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_EUTRA_FrequencyAndPriorityInfoList_vbxyext, EUTRA_FrequencyAndPriorityInfoList_vbxyext_sequence);

  return offset;
}



static int
dissect_rrc_BIT_STRING_SIZE_maxNumEUTRAFreqs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_bit_string(tvb, offset, actx, tree, hf_index,
                                     maxNumEUTRAFreqs, maxNumEUTRAFreqs, FALSE, NULL);

  return offset;
}


static const value_string rrc_T_eutra_FrequencyRepQuantityRACH_vals[] = {
  {   0, "rsrp" },
  {   1, "rsrq" },
  { 0, NULL }
};


static int
dissect_rrc_T_eutra_FrequencyRepQuantityRACH(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     2, NULL, FALSE, 0, NULL);

  return offset;
}


static const value_string rrc_T_rachReportingPriority_vals[] = {
  {   0, "intraEUTRAInter" },
  {   1, "interIntraEUTRA" },
  {   2, "interEUTRAIntra" },
  {   3, "eUTRAIntraInter" },
  {   4, "eUTRAInterIntra" },
  {   5, "spare3" },
  {   6, "spare2" },
  {   7, "spare1" },
  { 0, NULL }
};


static int
dissect_rrc_T_rachReportingPriority(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     8, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t EUTRA_FrequencyRACHReportingInfo_sequence[] = {
  { &hf_rrc_eutra_FrequencyListIndicator, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_maxNumEUTRAFreqs },
  { &hf_rrc_eutra_FrequencyRepQuantityRACH, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_eutra_FrequencyRepQuantityRACH },
  { &hf_rrc_eutra_FrequencyRACHReportingThreshold, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_0_97 },
  { &hf_rrc_rachReportingPriority, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_rachReportingPriority },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_EUTRA_FrequencyRACHReportingInfo(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_EUTRA_FrequencyRACHReportingInfo, EUTRA_FrequencyRACHReportingInfo_sequence);

  return offset;
}


static const value_string rrc_RAT_Type_vals[] = {
  {   0, "gsm" },
  {   1, "is2000" },
  { 0, NULL }
};


static int
dissect_rrc_RAT_Type(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     2, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t SEQUENCE_SIZE_1_maxOtherRAT_OF_RAT_Type_sequence_of[1] = {
  { &hf_rrc_inter_RAT_meas_ind_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RAT_Type },
};

static int
dissect_rrc_SEQUENCE_SIZE_1_maxOtherRAT_OF_RAT_Type(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_SEQUENCE_SIZE_1_maxOtherRAT_OF_RAT_Type, SEQUENCE_SIZE_1_maxOtherRAT_OF_RAT_Type_sequence_of,
                                                  1, maxOtherRAT, FALSE);

  return offset;
}


static const per_sequence_t FACH_MeasurementOccasionInfo_sequence[] = {
  { &hf_rrc_fACH_meas_occasion_coeff, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_INTEGER_1_12 },
  { &hf_rrc_inter_freq_FDD_meas_ind, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { &hf_rrc_inter_freq_TDD_meas_ind, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { &hf_rrc_inter_RAT_meas_ind, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_SEQUENCE_SIZE_1_maxOtherRAT_OF_RAT_Type },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_FACH_MeasurementOccasionInfo(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_FACH_MeasurementOccasionInfo, FACH_MeasurementOccasionInfo_sequence);

  return offset;
}


static const per_sequence_t FACH_MeasurementOccasionInfo_LCR_r4_ext_sequence[] = {
  { &hf_rrc_inter_freq_TDD128_meas_ind, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_FACH_MeasurementOccasionInfo_LCR_r4_ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_FACH_MeasurementOccasionInfo_LCR_r4_ext, FACH_MeasurementOccasionInfo_LCR_r4_ext_sequence);

  return offset;
}



static int
dissect_rrc_INTEGER_M58_M13(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            -58, -13, NULL, FALSE);

  return offset;
}


static const per_sequence_t GSM_PriorityInfo_sequence[] = {
  { &hf_rrc_gsmCellGroup    , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_GSM_CellGroup },
  { &hf_rrc_priority        , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_0_maxPrio_1 },
  { &hf_rrc_qRxLevMinGSM    , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_M58_M13 },
  { &hf_rrc_threshXhigh     , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_0_31 },
  { &hf_rrc_threshXlow      , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_0_31 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_GSM_PriorityInfo(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_GSM_PriorityInfo, GSM_PriorityInfo_sequence);

  return offset;
}


static const per_sequence_t GSM_PriorityInfoList_sequence_of[1] = {
  { &hf_rrc_GSM_PriorityInfoList_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_GSM_PriorityInfo },
};

static int
dissect_rrc_GSM_PriorityInfoList(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_GSM_PriorityInfoList, GSM_PriorityInfoList_sequence_of,
                                                  1, maxNumGSMCellGroup, FALSE);

  return offset;
}


static const per_sequence_t GSM_PriorityInfoList_vbxyext_sequence[] = {
  { &hf_rrc_gsmTreselectionScalingFactor, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_TreselectionScalingFactor2 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_GSM_PriorityInfoList_vbxyext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_GSM_PriorityInfoList_vbxyext, GSM_PriorityInfoList_vbxyext_sequence);

  return offset;
}


static const per_sequence_t NewInterFreqCellSI_RSCP_sequence[] = {
  { &hf_rrc_interFreqCellID , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_InterFreqCellID },
  { &hf_rrc_frequencyInfo   , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_FrequencyInfo },
  { &hf_rrc_cellInfo_03     , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CellInfoSI_RSCP },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_NewInterFreqCellSI_RSCP(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_NewInterFreqCellSI_RSCP, NewInterFreqCellSI_RSCP_sequence);

  return offset;
}


static const per_sequence_t NewInterFreqCellSI_List_RSCP_sequence_of[1] = {
  { &hf_rrc_NewInterFreqCellSI_List_RSCP_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_NewInterFreqCellSI_RSCP },
};

static int
dissect_rrc_NewInterFreqCellSI_List_RSCP(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_NewInterFreqCellSI_List_RSCP, NewInterFreqCellSI_List_RSCP_sequence_of,
                                                  1, maxCellMeas, FALSE);

  return offset;
}


static const per_sequence_t InterFreqCellInfoSI_List_RSCP_sequence[] = {
  { &hf_rrc_removedInterFreqCellList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RemovedInterFreqCellList },
  { &hf_rrc_newInterFreqCellList_07, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_NewInterFreqCellSI_List_RSCP },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_InterFreqCellInfoSI_List_RSCP(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_InterFreqCellInfoSI_List_RSCP, InterFreqCellInfoSI_List_RSCP_sequence);

  return offset;
}


static const per_sequence_t NewInterFreqCellSI_ECN0_sequence[] = {
  { &hf_rrc_interFreqCellID , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_InterFreqCellID },
  { &hf_rrc_frequencyInfo   , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_FrequencyInfo },
  { &hf_rrc_cellInfo_04     , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CellInfoSI_ECN0 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_NewInterFreqCellSI_ECN0(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_NewInterFreqCellSI_ECN0, NewInterFreqCellSI_ECN0_sequence);

  return offset;
}


static const per_sequence_t NewInterFreqCellSI_List_ECN0_sequence_of[1] = {
  { &hf_rrc_NewInterFreqCellSI_List_ECN0_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_NewInterFreqCellSI_ECN0 },
};

static int
dissect_rrc_NewInterFreqCellSI_List_ECN0(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_NewInterFreqCellSI_List_ECN0, NewInterFreqCellSI_List_ECN0_sequence_of,
                                                  1, maxCellMeas, FALSE);

  return offset;
}


static const per_sequence_t InterFreqCellInfoSI_List_ECN0_sequence[] = {
  { &hf_rrc_removedInterFreqCellList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RemovedInterFreqCellList },
  { &hf_rrc_newInterFreqCellList_08, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_NewInterFreqCellSI_List_ECN0 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_InterFreqCellInfoSI_List_ECN0(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_InterFreqCellInfoSI_List_ECN0, InterFreqCellInfoSI_List_ECN0_sequence);

  return offset;
}


static const per_sequence_t NewInterFreqCellSI_HCS_RSCP_sequence[] = {
  { &hf_rrc_interFreqCellID , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_InterFreqCellID },
  { &hf_rrc_frequencyInfo   , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_FrequencyInfo },
  { &hf_rrc_cellInfo_05     , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CellInfoSI_HCS_RSCP },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_NewInterFreqCellSI_HCS_RSCP(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_NewInterFreqCellSI_HCS_RSCP, NewInterFreqCellSI_HCS_RSCP_sequence);

  return offset;
}


static const per_sequence_t NewInterFreqCellSI_List_HCS_RSCP_sequence_of[1] = {
  { &hf_rrc_NewInterFreqCellSI_List_HCS_RSCP_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_NewInterFreqCellSI_HCS_RSCP },
};

static int
dissect_rrc_NewInterFreqCellSI_List_HCS_RSCP(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_NewInterFreqCellSI_List_HCS_RSCP, NewInterFreqCellSI_List_HCS_RSCP_sequence_of,
                                                  1, maxCellMeas, FALSE);

  return offset;
}


static const per_sequence_t InterFreqCellInfoSI_List_HCS_RSCP_sequence[] = {
  { &hf_rrc_removedInterFreqCellList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RemovedInterFreqCellList },
  { &hf_rrc_newInterFreqCellList_09, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_NewInterFreqCellSI_List_HCS_RSCP },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_InterFreqCellInfoSI_List_HCS_RSCP(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_InterFreqCellInfoSI_List_HCS_RSCP, InterFreqCellInfoSI_List_HCS_RSCP_sequence);

  return offset;
}


static const per_sequence_t NewInterFreqCellSI_HCS_ECN0_sequence[] = {
  { &hf_rrc_interFreqCellID , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_InterFreqCellID },
  { &hf_rrc_frequencyInfo   , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_FrequencyInfo },
  { &hf_rrc_cellInfo_06     , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CellInfoSI_HCS_ECN0 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_NewInterFreqCellSI_HCS_ECN0(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_NewInterFreqCellSI_HCS_ECN0, NewInterFreqCellSI_HCS_ECN0_sequence);

  return offset;
}


static const per_sequence_t NewInterFreqCellSI_List_HCS_ECN0_sequence_of[1] = {
  { &hf_rrc_NewInterFreqCellSI_List_HCS_ECN0_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_NewInterFreqCellSI_HCS_ECN0 },
};

static int
dissect_rrc_NewInterFreqCellSI_List_HCS_ECN0(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_NewInterFreqCellSI_List_HCS_ECN0, NewInterFreqCellSI_List_HCS_ECN0_sequence_of,
                                                  1, maxCellMeas, FALSE);

  return offset;
}


static const per_sequence_t InterFreqCellInfoSI_List_HCS_ECN0_sequence[] = {
  { &hf_rrc_removedInterFreqCellList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RemovedInterFreqCellList },
  { &hf_rrc_newInterFreqCellList_10, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_NewInterFreqCellSI_List_HCS_ECN0 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_InterFreqCellInfoSI_List_HCS_ECN0(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_InterFreqCellInfoSI_List_HCS_ECN0, InterFreqCellInfoSI_List_HCS_ECN0_sequence);

  return offset;
}


static const per_sequence_t NewInterFreqCellSI_RSCP_LCR_r4_sequence[] = {
  { &hf_rrc_interFreqCellID , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_InterFreqCellID },
  { &hf_rrc_frequencyInfo   , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_FrequencyInfo },
  { &hf_rrc_cellInfo_07     , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CellInfoSI_RSCP_LCR_r4 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_NewInterFreqCellSI_RSCP_LCR_r4(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_NewInterFreqCellSI_RSCP_LCR_r4, NewInterFreqCellSI_RSCP_LCR_r4_sequence);

  return offset;
}


static const per_sequence_t NewInterFreqCellSI_List_RSCP_LCR_r4_sequence_of[1] = {
  { &hf_rrc_NewInterFreqCellSI_List_RSCP_LCR_r4_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_NewInterFreqCellSI_RSCP_LCR_r4 },
};

static int
dissect_rrc_NewInterFreqCellSI_List_RSCP_LCR_r4(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_NewInterFreqCellSI_List_RSCP_LCR_r4, NewInterFreqCellSI_List_RSCP_LCR_r4_sequence_of,
                                                  1, maxCellMeas, FALSE);

  return offset;
}


static const per_sequence_t InterFreqCellInfoSI_List_RSCP_LCR_sequence[] = {
  { &hf_rrc_removedInterFreqCellList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RemovedInterFreqCellList },
  { &hf_rrc_newInterFreqCellList_11, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_NewInterFreqCellSI_List_RSCP_LCR_r4 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_InterFreqCellInfoSI_List_RSCP_LCR(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_InterFreqCellInfoSI_List_RSCP_LCR, InterFreqCellInfoSI_List_RSCP_LCR_sequence);

  return offset;
}


static const per_sequence_t NewInterFreqCellSI_ECN0_LCR_r4_sequence[] = {
  { &hf_rrc_interFreqCellID , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_InterFreqCellID },
  { &hf_rrc_frequencyInfo   , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_FrequencyInfo },
  { &hf_rrc_cellInfo_08     , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CellInfoSI_ECN0_LCR_r4 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_NewInterFreqCellSI_ECN0_LCR_r4(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_NewInterFreqCellSI_ECN0_LCR_r4, NewInterFreqCellSI_ECN0_LCR_r4_sequence);

  return offset;
}


static const per_sequence_t NewInterFreqCellSI_List_ECN0_LCR_r4_sequence_of[1] = {
  { &hf_rrc_NewInterFreqCellSI_List_ECN0_LCR_r4_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_NewInterFreqCellSI_ECN0_LCR_r4 },
};

static int
dissect_rrc_NewInterFreqCellSI_List_ECN0_LCR_r4(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_NewInterFreqCellSI_List_ECN0_LCR_r4, NewInterFreqCellSI_List_ECN0_LCR_r4_sequence_of,
                                                  1, maxCellMeas, FALSE);

  return offset;
}


static const per_sequence_t InterFreqCellInfoSI_List_ECN0_LCR_sequence[] = {
  { &hf_rrc_removedInterFreqCellList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RemovedInterFreqCellList },
  { &hf_rrc_newInterFreqCellList_12, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_NewInterFreqCellSI_List_ECN0_LCR_r4 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_InterFreqCellInfoSI_List_ECN0_LCR(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_InterFreqCellInfoSI_List_ECN0_LCR, InterFreqCellInfoSI_List_ECN0_LCR_sequence);

  return offset;
}


static const per_sequence_t NewInterFreqCellSI_HCS_RSCP_LCR_r4_sequence[] = {
  { &hf_rrc_interFreqCellID , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_InterFreqCellID },
  { &hf_rrc_frequencyInfo   , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_FrequencyInfo },
  { &hf_rrc_cellInfo_09     , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CellInfoSI_HCS_RSCP_LCR_r4 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_NewInterFreqCellSI_HCS_RSCP_LCR_r4(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_NewInterFreqCellSI_HCS_RSCP_LCR_r4, NewInterFreqCellSI_HCS_RSCP_LCR_r4_sequence);

  return offset;
}


static const per_sequence_t NewInterFreqCellSI_List_HCS_RSCP_LCR_r4_sequence_of[1] = {
  { &hf_rrc_NewInterFreqCellSI_List_HCS_RSCP_LCR_r4_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_NewInterFreqCellSI_HCS_RSCP_LCR_r4 },
};

static int
dissect_rrc_NewInterFreqCellSI_List_HCS_RSCP_LCR_r4(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_NewInterFreqCellSI_List_HCS_RSCP_LCR_r4, NewInterFreqCellSI_List_HCS_RSCP_LCR_r4_sequence_of,
                                                  1, maxCellMeas, FALSE);

  return offset;
}


static const per_sequence_t InterFreqCellInfoSI_List_HCS_RSCP_LCR_sequence[] = {
  { &hf_rrc_removedInterFreqCellList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RemovedInterFreqCellList },
  { &hf_rrc_newInterFreqCellList_13, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_NewInterFreqCellSI_List_HCS_RSCP_LCR_r4 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_InterFreqCellInfoSI_List_HCS_RSCP_LCR(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_InterFreqCellInfoSI_List_HCS_RSCP_LCR, InterFreqCellInfoSI_List_HCS_RSCP_LCR_sequence);

  return offset;
}


static const per_sequence_t NewInterFreqCellSI_HCS_ECN0_LCR_r4_sequence[] = {
  { &hf_rrc_interFreqCellID , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_InterFreqCellID },
  { &hf_rrc_frequencyInfo   , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_FrequencyInfo },
  { &hf_rrc_cellInfo_10     , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CellInfoSI_HCS_ECN0_LCR_r4 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_NewInterFreqCellSI_HCS_ECN0_LCR_r4(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_NewInterFreqCellSI_HCS_ECN0_LCR_r4, NewInterFreqCellSI_HCS_ECN0_LCR_r4_sequence);

  return offset;
}


static const per_sequence_t NewInterFreqCellSI_List_HCS_ECN0_LCR_r4_sequence_of[1] = {
  { &hf_rrc_NewInterFreqCellSI_List_HCS_ECN0_LCR_r4_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_NewInterFreqCellSI_HCS_ECN0_LCR_r4 },
};

static int
dissect_rrc_NewInterFreqCellSI_List_HCS_ECN0_LCR_r4(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_NewInterFreqCellSI_List_HCS_ECN0_LCR_r4, NewInterFreqCellSI_List_HCS_ECN0_LCR_r4_sequence_of,
                                                  1, maxCellMeas, FALSE);

  return offset;
}


static const per_sequence_t InterFreqCellInfoSI_List_HCS_ECN0_LCR_sequence[] = {
  { &hf_rrc_removedInterFreqCellList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RemovedInterFreqCellList },
  { &hf_rrc_newInterFreqCellList_14, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_NewInterFreqCellSI_List_HCS_ECN0_LCR_r4 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_InterFreqCellInfoSI_List_HCS_ECN0_LCR(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_InterFreqCellInfoSI_List_HCS_ECN0_LCR, InterFreqCellInfoSI_List_HCS_ECN0_LCR_sequence);

  return offset;
}


static const per_sequence_t InterFreqMeasurementSysInfo_RSCP_sequence[] = {
  { &hf_rrc_interFreqCellInfoSI_List, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_InterFreqCellInfoSI_List_RSCP },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_InterFreqMeasurementSysInfo_RSCP(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_InterFreqMeasurementSysInfo_RSCP, InterFreqMeasurementSysInfo_RSCP_sequence);

  return offset;
}


static const per_sequence_t InterFreqMeasurementSysInfo_ECN0_sequence[] = {
  { &hf_rrc_interFreqCellInfoSI_List_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_InterFreqCellInfoSI_List_ECN0 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_InterFreqMeasurementSysInfo_ECN0(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_InterFreqMeasurementSysInfo_ECN0, InterFreqMeasurementSysInfo_ECN0_sequence);

  return offset;
}


static const per_sequence_t InterFreqMeasurementSysInfo_HCS_RSCP_sequence[] = {
  { &hf_rrc_interFreqCellInfoSI_List_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_InterFreqCellInfoSI_List_HCS_RSCP },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_InterFreqMeasurementSysInfo_HCS_RSCP(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_InterFreqMeasurementSysInfo_HCS_RSCP, InterFreqMeasurementSysInfo_HCS_RSCP_sequence);

  return offset;
}


static const per_sequence_t InterFreqMeasurementSysInfo_HCS_ECN0_sequence[] = {
  { &hf_rrc_interFreqCellInfoSI_List_03, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_InterFreqCellInfoSI_List_HCS_ECN0 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_InterFreqMeasurementSysInfo_HCS_ECN0(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_InterFreqMeasurementSysInfo_HCS_ECN0, InterFreqMeasurementSysInfo_HCS_ECN0_sequence);

  return offset;
}


static const per_sequence_t InterFreqMeasurementSysInfo_RSCP_LCR_r4_sequence[] = {
  { &hf_rrc_interFreqCellInfoSI_List_04, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_InterFreqCellInfoSI_List_RSCP_LCR },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_InterFreqMeasurementSysInfo_RSCP_LCR_r4(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_InterFreqMeasurementSysInfo_RSCP_LCR_r4, InterFreqMeasurementSysInfo_RSCP_LCR_r4_sequence);

  return offset;
}


static const per_sequence_t InterFreqMeasurementSysInfo_ECN0_LCR_r4_sequence[] = {
  { &hf_rrc_interFreqCellInfoSI_List_05, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_InterFreqCellInfoSI_List_ECN0_LCR },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_InterFreqMeasurementSysInfo_ECN0_LCR_r4(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_InterFreqMeasurementSysInfo_ECN0_LCR_r4, InterFreqMeasurementSysInfo_ECN0_LCR_r4_sequence);

  return offset;
}


static const per_sequence_t InterFreqMeasurementSysInfo_HCS_RSCP_LCR_r4_sequence[] = {
  { &hf_rrc_interFreqCellInfoSI_List_06, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_InterFreqCellInfoSI_List_HCS_RSCP_LCR },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_InterFreqMeasurementSysInfo_HCS_RSCP_LCR_r4(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_InterFreqMeasurementSysInfo_HCS_RSCP_LCR_r4, InterFreqMeasurementSysInfo_HCS_RSCP_LCR_r4_sequence);

  return offset;
}


static const per_sequence_t InterFreqMeasurementSysInfo_HCS_ECN0_LCR_r4_sequence[] = {
  { &hf_rrc_interFreqCellInfoSI_List_07, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_InterFreqCellInfoSI_List_HCS_ECN0_LCR },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_InterFreqMeasurementSysInfo_HCS_ECN0_LCR_r4(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_InterFreqMeasurementSysInfo_HCS_ECN0_LCR_r4, InterFreqMeasurementSysInfo_HCS_ECN0_LCR_r4_sequence);

  return offset;
}


static const value_string rrc_InterFreqRepQuantityRACH_FDD_vals[] = {
  {   0, "cpich-EcN0" },
  {   1, "cpich-RSCP" },
  { 0, NULL }
};


static int
dissect_rrc_InterFreqRepQuantityRACH_FDD(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     2, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t T_fdd_152_sequence[] = {
  { &hf_rrc_interFreqRepQuantityRACH_FDD, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_InterFreqRepQuantityRACH_FDD },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_fdd_152(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_fdd_152, T_fdd_152_sequence);

  return offset;
}


static const value_string rrc_InterFreqRepQuantityRACH_TDD_vals[] = {
  {   0, "dummy" },
  {   1, "primaryCCPCH-RSCP" },
  { 0, NULL }
};


static int
dissect_rrc_InterFreqRepQuantityRACH_TDD(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     2, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t InterFreqRepQuantityRACH_TDDList_sequence_of[1] = {
  { &hf_rrc_InterFreqRepQuantityRACH_TDDList_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_InterFreqRepQuantityRACH_TDD },
};

static int
dissect_rrc_InterFreqRepQuantityRACH_TDDList(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_InterFreqRepQuantityRACH_TDDList, InterFreqRepQuantityRACH_TDDList_sequence_of,
                                                  1, 2, FALSE);

  return offset;
}


static const per_sequence_t T_tdd_101_sequence[] = {
  { &hf_rrc_interFreqRepQuantityRACH_TDDList, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_InterFreqRepQuantityRACH_TDDList },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_tdd_101(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_tdd_101, T_tdd_101_sequence);

  return offset;
}



static int
dissect_rrc_MaxReportedCellsOnRACHinterFreq(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            1U, 8U, NULL, FALSE);

  return offset;
}


static const value_string rrc_T_modeSpecificInfo_113_vals[] = {
  {   0, "fdd" },
  {   1, "tdd" },
  {   2, "interFreqRACHReportingThreshold" },
  {   3, "maxReportedCellsOnRACHinterFreq" },
  { 0, NULL }
};

static const per_choice_t T_modeSpecificInfo_113_choice[] = {
  {   0, &hf_rrc_fdd_158         , ASN1_NO_EXTENSIONS     , dissect_rrc_T_fdd_152 },
  {   1, &hf_rrc_tdd_109         , ASN1_NO_EXTENSIONS     , dissect_rrc_T_tdd_101 },
  {   2, &hf_rrc_interFreqRACHReportingThreshold, ASN1_NO_EXTENSIONS     , dissect_rrc_Threshold },
  {   3, &hf_rrc_maxReportedCellsOnRACHinterFreq, ASN1_NO_EXTENSIONS     , dissect_rrc_MaxReportedCellsOnRACHinterFreq },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_modeSpecificInfo_113(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_modeSpecificInfo_113, T_modeSpecificInfo_113_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t Dummy_InterFreqRACHReportingInfo_sequence[] = {
  { &hf_rrc_modeSpecificInfo_113, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_modeSpecificInfo_113 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_Dummy_InterFreqRACHReportingInfo(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_Dummy_InterFreqRACHReportingInfo, Dummy_InterFreqRACHReportingInfo_sequence);

  return offset;
}


static const per_sequence_t T_fdd_153_sequence[] = {
  { &hf_rrc_interFreqRepQuantityRACH_FDD, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_InterFreqRepQuantityRACH_FDD },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_fdd_153(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_fdd_153, T_fdd_153_sequence);

  return offset;
}


static const per_sequence_t T_tdd_102_sequence[] = {
  { &hf_rrc_interFreqRepQuantityRACH_TDDList, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_InterFreqRepQuantityRACH_TDDList },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_tdd_102(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_tdd_102, T_tdd_102_sequence);

  return offset;
}


static const value_string rrc_T_modeSpecificInfo_114_vals[] = {
  {   0, "fdd" },
  {   1, "tdd" },
  { 0, NULL }
};

static const per_choice_t T_modeSpecificInfo_114_choice[] = {
  {   0, &hf_rrc_fdd_159         , ASN1_NO_EXTENSIONS     , dissect_rrc_T_fdd_153 },
  {   1, &hf_rrc_tdd_110         , ASN1_NO_EXTENSIONS     , dissect_rrc_T_tdd_102 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_modeSpecificInfo_114(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_modeSpecificInfo_114, T_modeSpecificInfo_114_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t InterFreqRACHReportingInfo_sequence[] = {
  { &hf_rrc_modeSpecificInfo_114, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_modeSpecificInfo_114 },
  { &hf_rrc_interFreqRACHReportingThreshold, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_Threshold },
  { &hf_rrc_maxReportedCellsOnRACHinterFreq, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MaxReportedCellsOnRACHinterFreq },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_InterFreqRACHReportingInfo(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_InterFreqRACHReportingInfo, InterFreqRACHReportingInfo_sequence);

  return offset;
}


static const per_sequence_t T_gsm_14_sequence[] = {
  { &hf_rrc_cellSelectionReselectionInfo_05, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CellSelectReselectInfoSIB_11_12 },
  { &hf_rrc_interRATCellIndividualOffset, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_InterRATCellIndividualOffset },
  { &hf_rrc_bsic            , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BSIC },
  { &hf_rrc_frequency_band  , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_Frequency_Band },
  { &hf_rrc_bcch_ARFCN      , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BCCH_ARFCN },
  { &hf_rrc_dummy_24        , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_NULL },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_gsm_14(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_gsm_14, T_gsm_14_sequence);

  return offset;
}


static const per_sequence_t T_is_2000_03_sequence[] = {
  { &hf_rrc_is_2000SpecificMeasInfo, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_IS_2000SpecificMeasInfo },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_is_2000_03(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_is_2000_03, T_is_2000_03_sequence);

  return offset;
}


static const value_string rrc_T_technologySpecificInfo_02_vals[] = {
  {   0, "gsm" },
  {   1, "is-2000" },
  {   2, "absent" },
  {   3, "spare1" },
  { 0, NULL }
};

static const per_choice_t T_technologySpecificInfo_02_choice[] = {
  {   0, &hf_rrc_gsm_16          , ASN1_NO_EXTENSIONS     , dissect_rrc_T_gsm_14 },
  {   1, &hf_rrc_is_2000_04      , ASN1_NO_EXTENSIONS     , dissect_rrc_T_is_2000_03 },
  {   2, &hf_rrc_absent          , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   3, &hf_rrc_spare1          , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_technologySpecificInfo_02(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_technologySpecificInfo_02, T_technologySpecificInfo_02_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t NewInterRATCell_B_sequence[] = {
  { &hf_rrc_interRATCellID  , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_InterRATCellID },
  { &hf_rrc_technologySpecificInfo_02, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_technologySpecificInfo_02 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_NewInterRATCell_B(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_NewInterRATCell_B, NewInterRATCell_B_sequence);

  return offset;
}


static const per_sequence_t NewInterRATCellList_B_sequence_of[1] = {
  { &hf_rrc_NewInterRATCellList_B_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_NewInterRATCell_B },
};

static int
dissect_rrc_NewInterRATCellList_B(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_NewInterRATCellList_B, NewInterRATCellList_B_sequence_of,
                                                  1, maxCellMeas, FALSE);

  return offset;
}


static const per_sequence_t InterRATCellInfoList_B_sequence[] = {
  { &hf_rrc_removedInterRATCellList, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RemovedInterRATCellList },
  { &hf_rrc_newInterRATCellList_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_NewInterRATCellList_B },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_InterRATCellInfoList_B(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_InterRATCellInfoList_B, InterRATCellInfoList_B_sequence);

  return offset;
}


static const per_sequence_t InterRATMeasurementSysInfo_sequence[] = {
  { &hf_rrc_interRATCellInfoList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_InterRATCellInfoList },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_InterRATMeasurementSysInfo(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_InterRATMeasurementSysInfo, InterRATMeasurementSysInfo_sequence);

  return offset;
}


static const per_sequence_t InterRATMeasurementSysInfo_B_sequence[] = {
  { &hf_rrc_interRATCellInfoList_03, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_InterRATCellInfoList_B },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_InterRATMeasurementSysInfo_B(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_InterRATMeasurementSysInfo_B, InterRATMeasurementSysInfo_B_sequence);

  return offset;
}


static const per_sequence_t NewIntraFreqCellSI_RSCP_sequence[] = {
  { &hf_rrc_intraFreqCellID , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_IntraFreqCellID },
  { &hf_rrc_cellInfo_03     , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CellInfoSI_RSCP },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_NewIntraFreqCellSI_RSCP(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_NewIntraFreqCellSI_RSCP, NewIntraFreqCellSI_RSCP_sequence);

  return offset;
}


static const per_sequence_t NewIntraFreqCellSI_List_RSCP_sequence_of[1] = {
  { &hf_rrc_NewIntraFreqCellSI_List_RSCP_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_NewIntraFreqCellSI_RSCP },
};

static int
dissect_rrc_NewIntraFreqCellSI_List_RSCP(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_NewIntraFreqCellSI_List_RSCP, NewIntraFreqCellSI_List_RSCP_sequence_of,
                                                  1, maxCellMeas, FALSE);

  return offset;
}


static const per_sequence_t IntraFreqCellInfoSI_List_RSCP_sequence[] = {
  { &hf_rrc_removedIntraFreqCellList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RemovedIntraFreqCellList },
  { &hf_rrc_newIntraFreqCellList_06, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_NewIntraFreqCellSI_List_RSCP },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_IntraFreqCellInfoSI_List_RSCP(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_IntraFreqCellInfoSI_List_RSCP, IntraFreqCellInfoSI_List_RSCP_sequence);

  return offset;
}


static const per_sequence_t NewIntraFreqCellSI_ECN0_sequence[] = {
  { &hf_rrc_intraFreqCellID , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_IntraFreqCellID },
  { &hf_rrc_cellInfo_04     , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CellInfoSI_ECN0 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_NewIntraFreqCellSI_ECN0(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_NewIntraFreqCellSI_ECN0, NewIntraFreqCellSI_ECN0_sequence);

  return offset;
}


static const per_sequence_t NewIntraFreqCellSI_List_ECN0_sequence_of[1] = {
  { &hf_rrc_NewIntraFreqCellSI_List_ECN0_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_NewIntraFreqCellSI_ECN0 },
};

static int
dissect_rrc_NewIntraFreqCellSI_List_ECN0(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_NewIntraFreqCellSI_List_ECN0, NewIntraFreqCellSI_List_ECN0_sequence_of,
                                                  1, maxCellMeas, FALSE);

  return offset;
}


static const per_sequence_t IntraFreqCellInfoSI_List_ECN0_sequence[] = {
  { &hf_rrc_removedIntraFreqCellList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RemovedIntraFreqCellList },
  { &hf_rrc_newIntraFreqCellList_07, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_NewIntraFreqCellSI_List_ECN0 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_IntraFreqCellInfoSI_List_ECN0(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_IntraFreqCellInfoSI_List_ECN0, IntraFreqCellInfoSI_List_ECN0_sequence);

  return offset;
}


static const per_sequence_t NewIntraFreqCellSI_HCS_RSCP_sequence[] = {
  { &hf_rrc_intraFreqCellID , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_IntraFreqCellID },
  { &hf_rrc_cellInfo_05     , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CellInfoSI_HCS_RSCP },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_NewIntraFreqCellSI_HCS_RSCP(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_NewIntraFreqCellSI_HCS_RSCP, NewIntraFreqCellSI_HCS_RSCP_sequence);

  return offset;
}


static const per_sequence_t NewIntraFreqCellSI_List_HCS_RSCP_sequence_of[1] = {
  { &hf_rrc_NewIntraFreqCellSI_List_HCS_RSCP_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_NewIntraFreqCellSI_HCS_RSCP },
};

static int
dissect_rrc_NewIntraFreqCellSI_List_HCS_RSCP(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_NewIntraFreqCellSI_List_HCS_RSCP, NewIntraFreqCellSI_List_HCS_RSCP_sequence_of,
                                                  1, maxCellMeas, FALSE);

  return offset;
}


static const per_sequence_t IntraFreqCellInfoSI_List_HCS_RSCP_sequence[] = {
  { &hf_rrc_removedIntraFreqCellList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RemovedIntraFreqCellList },
  { &hf_rrc_newIntraFreqCellList_08, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_NewIntraFreqCellSI_List_HCS_RSCP },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_IntraFreqCellInfoSI_List_HCS_RSCP(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_IntraFreqCellInfoSI_List_HCS_RSCP, IntraFreqCellInfoSI_List_HCS_RSCP_sequence);

  return offset;
}


static const per_sequence_t NewIntraFreqCellSI_HCS_ECN0_sequence[] = {
  { &hf_rrc_intraFreqCellID , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_IntraFreqCellID },
  { &hf_rrc_cellInfo_06     , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CellInfoSI_HCS_ECN0 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_NewIntraFreqCellSI_HCS_ECN0(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_NewIntraFreqCellSI_HCS_ECN0, NewIntraFreqCellSI_HCS_ECN0_sequence);

  return offset;
}


static const per_sequence_t NewIntraFreqCellSI_List_HCS_ECN0_sequence_of[1] = {
  { &hf_rrc_NewIntraFreqCellSI_List_HCS_ECN0_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_NewIntraFreqCellSI_HCS_ECN0 },
};

static int
dissect_rrc_NewIntraFreqCellSI_List_HCS_ECN0(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_NewIntraFreqCellSI_List_HCS_ECN0, NewIntraFreqCellSI_List_HCS_ECN0_sequence_of,
                                                  1, maxCellMeas, FALSE);

  return offset;
}


static const per_sequence_t IntraFreqCellInfoSI_List_HCS_ECN0_sequence[] = {
  { &hf_rrc_removedIntraFreqCellList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RemovedIntraFreqCellList },
  { &hf_rrc_newIntraFreqCellList_09, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_NewIntraFreqCellSI_List_HCS_ECN0 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_IntraFreqCellInfoSI_List_HCS_ECN0(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_IntraFreqCellInfoSI_List_HCS_ECN0, IntraFreqCellInfoSI_List_HCS_ECN0_sequence);

  return offset;
}


static const per_sequence_t NewIntraFreqCellSI_RSCP_LCR_r4_sequence[] = {
  { &hf_rrc_intraFreqCellID , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_IntraFreqCellID },
  { &hf_rrc_cellInfo_07     , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CellInfoSI_RSCP_LCR_r4 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_NewIntraFreqCellSI_RSCP_LCR_r4(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_NewIntraFreqCellSI_RSCP_LCR_r4, NewIntraFreqCellSI_RSCP_LCR_r4_sequence);

  return offset;
}


static const per_sequence_t NewIntraFreqCellSI_List_RSCP_LCR_r4_sequence_of[1] = {
  { &hf_rrc_NewIntraFreqCellSI_List_RSCP_LCR_r4_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_NewIntraFreqCellSI_RSCP_LCR_r4 },
};

static int
dissect_rrc_NewIntraFreqCellSI_List_RSCP_LCR_r4(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_NewIntraFreqCellSI_List_RSCP_LCR_r4, NewIntraFreqCellSI_List_RSCP_LCR_r4_sequence_of,
                                                  1, maxCellMeas, FALSE);

  return offset;
}


static const per_sequence_t IntraFreqCellInfoSI_List_RSCP_LCR_r4_sequence[] = {
  { &hf_rrc_removedIntraFreqCellList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RemovedIntraFreqCellList },
  { &hf_rrc_newIntraFreqCellList_10, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_NewIntraFreqCellSI_List_RSCP_LCR_r4 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_IntraFreqCellInfoSI_List_RSCP_LCR_r4(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_IntraFreqCellInfoSI_List_RSCP_LCR_r4, IntraFreqCellInfoSI_List_RSCP_LCR_r4_sequence);

  return offset;
}


static const per_sequence_t NewIntraFreqCellSI_ECN0_LCR_r4_sequence[] = {
  { &hf_rrc_intraFreqCellID , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_IntraFreqCellID },
  { &hf_rrc_cellInfo_08     , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CellInfoSI_ECN0_LCR_r4 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_NewIntraFreqCellSI_ECN0_LCR_r4(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_NewIntraFreqCellSI_ECN0_LCR_r4, NewIntraFreqCellSI_ECN0_LCR_r4_sequence);

  return offset;
}


static const per_sequence_t NewIntraFreqCellSI_List_ECN0_LCR_r4_sequence_of[1] = {
  { &hf_rrc_NewIntraFreqCellSI_List_ECN0_LCR_r4_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_NewIntraFreqCellSI_ECN0_LCR_r4 },
};

static int
dissect_rrc_NewIntraFreqCellSI_List_ECN0_LCR_r4(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_NewIntraFreqCellSI_List_ECN0_LCR_r4, NewIntraFreqCellSI_List_ECN0_LCR_r4_sequence_of,
                                                  1, maxCellMeas, FALSE);

  return offset;
}


static const per_sequence_t IntraFreqCellInfoSI_List_ECN0_LCR_r4_sequence[] = {
  { &hf_rrc_removedIntraFreqCellList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RemovedIntraFreqCellList },
  { &hf_rrc_newIntraFreqCellList_11, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_NewIntraFreqCellSI_List_ECN0_LCR_r4 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_IntraFreqCellInfoSI_List_ECN0_LCR_r4(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_IntraFreqCellInfoSI_List_ECN0_LCR_r4, IntraFreqCellInfoSI_List_ECN0_LCR_r4_sequence);

  return offset;
}


static const per_sequence_t NewIntraFreqCellSI_HCS_RSCP_LCR_r4_sequence[] = {
  { &hf_rrc_intraFreqCellID , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_IntraFreqCellID },
  { &hf_rrc_cellInfo_09     , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CellInfoSI_HCS_RSCP_LCR_r4 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_NewIntraFreqCellSI_HCS_RSCP_LCR_r4(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_NewIntraFreqCellSI_HCS_RSCP_LCR_r4, NewIntraFreqCellSI_HCS_RSCP_LCR_r4_sequence);

  return offset;
}


static const per_sequence_t NewIntraFreqCellSI_List_HCS_RSCP_LCR_r4_sequence_of[1] = {
  { &hf_rrc_NewIntraFreqCellSI_List_HCS_RSCP_LCR_r4_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_NewIntraFreqCellSI_HCS_RSCP_LCR_r4 },
};

static int
dissect_rrc_NewIntraFreqCellSI_List_HCS_RSCP_LCR_r4(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_NewIntraFreqCellSI_List_HCS_RSCP_LCR_r4, NewIntraFreqCellSI_List_HCS_RSCP_LCR_r4_sequence_of,
                                                  1, maxCellMeas, FALSE);

  return offset;
}


static const per_sequence_t IntraFreqCellInfoSI_List_HCS_RSCP_LCR_r4_sequence[] = {
  { &hf_rrc_removedIntraFreqCellList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RemovedIntraFreqCellList },
  { &hf_rrc_newIntraFreqCellList_12, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_NewIntraFreqCellSI_List_HCS_RSCP_LCR_r4 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_IntraFreqCellInfoSI_List_HCS_RSCP_LCR_r4(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_IntraFreqCellInfoSI_List_HCS_RSCP_LCR_r4, IntraFreqCellInfoSI_List_HCS_RSCP_LCR_r4_sequence);

  return offset;
}


static const per_sequence_t NewIntraFreqCellSI_HCS_ECN0_LCR_r4_sequence[] = {
  { &hf_rrc_intraFreqCellID , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_IntraFreqCellID },
  { &hf_rrc_cellInfo_10     , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CellInfoSI_HCS_ECN0_LCR_r4 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_NewIntraFreqCellSI_HCS_ECN0_LCR_r4(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_NewIntraFreqCellSI_HCS_ECN0_LCR_r4, NewIntraFreqCellSI_HCS_ECN0_LCR_r4_sequence);

  return offset;
}


static const per_sequence_t NewIntraFreqCellSI_List_HCS_ECN0_LCR_r4_sequence_of[1] = {
  { &hf_rrc_NewIntraFreqCellSI_List_HCS_ECN0_LCR_r4_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_NewIntraFreqCellSI_HCS_ECN0_LCR_r4 },
};

static int
dissect_rrc_NewIntraFreqCellSI_List_HCS_ECN0_LCR_r4(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_NewIntraFreqCellSI_List_HCS_ECN0_LCR_r4, NewIntraFreqCellSI_List_HCS_ECN0_LCR_r4_sequence_of,
                                                  1, maxCellMeas, FALSE);

  return offset;
}


static const per_sequence_t IntraFreqCellInfoSI_List_HCS_ECN0_LCR_r4_sequence[] = {
  { &hf_rrc_removedIntraFreqCellList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RemovedIntraFreqCellList },
  { &hf_rrc_newIntraFreqCellList_13, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_NewIntraFreqCellSI_List_HCS_ECN0_LCR_r4 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_IntraFreqCellInfoSI_List_HCS_ECN0_LCR_r4(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_IntraFreqCellInfoSI_List_HCS_ECN0_LCR_r4, IntraFreqCellInfoSI_List_HCS_ECN0_LCR_r4_sequence);

  return offset;
}


static const value_string rrc_IntraFreqMeasQuantity_FDD_sib3_vals[] = {
  {   0, "cpich-Ec-N0" },
  {   1, "cpich-RSCP" },
  { 0, NULL }
};


static int
dissect_rrc_IntraFreqMeasQuantity_FDD_sib3(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     2, NULL, FALSE, 0, NULL);

  return offset;
}


static const value_string rrc_IntraFreqMeasQuantity_TDD_sib3List_item_vals[] = {
  {   0, "primaryCCPCH-RSCP" },
  {   1, "timeslotISCP" },
  { 0, NULL }
};


static int
dissect_rrc_IntraFreqMeasQuantity_TDD_sib3List_item(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     2, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t IntraFreqMeasQuantity_TDD_sib3List_sequence_of[1] = {
  { &hf_rrc_IntraFreqMeasQuantity_TDD_sib3List_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_IntraFreqMeasQuantity_TDD_sib3List_item },
};

static int
dissect_rrc_IntraFreqMeasQuantity_TDD_sib3List(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_IntraFreqMeasQuantity_TDD_sib3List, IntraFreqMeasQuantity_TDD_sib3List_sequence_of,
                                                  1, 2, FALSE);

  return offset;
}


static const value_string rrc_IntraFreqRepQuantityRACH_FDD_vals[] = {
  {   0, "cpich-EcN0" },
  {   1, "cpich-RSCP" },
  {   2, "pathloss" },
  {   3, "noReport" },
  { 0, NULL }
};


static int
dissect_rrc_IntraFreqRepQuantityRACH_FDD(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     4, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t T_fdd_156_sequence[] = {
  { &hf_rrc_intraFreqRepQuantityRACH_FDD, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_IntraFreqRepQuantityRACH_FDD },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_fdd_156(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_fdd_156, T_fdd_156_sequence);

  return offset;
}


static const value_string rrc_IntraFreqRepQuantityRACH_TDD_vals[] = {
  {   0, "timeslotISCP" },
  {   1, "primaryCCPCH-RSCP" },
  {   2, "noReport" },
  { 0, NULL }
};


static int
dissect_rrc_IntraFreqRepQuantityRACH_TDD(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     3, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t IntraFreqRepQuantityRACH_TDDList_sequence_of[1] = {
  { &hf_rrc_IntraFreqRepQuantityRACH_TDDList_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_IntraFreqRepQuantityRACH_TDD },
};

static int
dissect_rrc_IntraFreqRepQuantityRACH_TDDList(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_IntraFreqRepQuantityRACH_TDDList, IntraFreqRepQuantityRACH_TDDList_sequence_of,
                                                  1, 2, FALSE);

  return offset;
}


static const per_sequence_t T_tdd_104_sequence[] = {
  { &hf_rrc_intraFreqRepQuantityRACH_TDDList, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_IntraFreqRepQuantityRACH_TDDList },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_tdd_104(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_tdd_104, T_tdd_104_sequence);

  return offset;
}


static const value_string rrc_T_modeSpecificInfo_117_vals[] = {
  {   0, "fdd" },
  {   1, "tdd" },
  { 0, NULL }
};

static const per_choice_t T_modeSpecificInfo_117_choice[] = {
  {   0, &hf_rrc_fdd_162         , ASN1_NO_EXTENSIONS     , dissect_rrc_T_fdd_156 },
  {   1, &hf_rrc_tdd_112         , ASN1_NO_EXTENSIONS     , dissect_rrc_T_tdd_104 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_modeSpecificInfo_117(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_modeSpecificInfo_117, T_modeSpecificInfo_117_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t IntraFreqReportingQuantityForRACH_sequence[] = {
  { &hf_rrc_sfn_SFN_OTD_Type, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SFN_SFN_OTD_Type },
  { &hf_rrc_modeSpecificInfo_117, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_modeSpecificInfo_117 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_IntraFreqReportingQuantityForRACH(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_IntraFreqReportingQuantityForRACH, IntraFreqReportingQuantityForRACH_sequence);

  return offset;
}


static const value_string rrc_MaxReportedCellsOnRACH_vals[] = {
  {   0, "noReport" },
  {   1, "currentCell" },
  {   2, "currentAnd-1-BestNeighbour" },
  {   3, "currentAnd-2-BestNeighbour" },
  {   4, "currentAnd-3-BestNeighbour" },
  {   5, "currentAnd-4-BestNeighbour" },
  {   6, "currentAnd-5-BestNeighbour" },
  {   7, "currentAnd-6-BestNeighbour" },
  { 0, NULL }
};


static int
dissect_rrc_MaxReportedCellsOnRACH(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     8, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t ReportingInfoForCellDCH_sequence[] = {
  { &hf_rrc_intraFreqReportingQuantity, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_IntraFreqReportingQuantity },
  { &hf_rrc_measurementReportingMode, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MeasurementReportingMode },
  { &hf_rrc_reportCriteria_16, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CellDCH_ReportCriteria },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_ReportingInfoForCellDCH(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_ReportingInfoForCellDCH, ReportingInfoForCellDCH_sequence);

  return offset;
}


static const per_sequence_t IntraFreqMeasurementSysInfo_RSCP_sequence[] = {
  { &hf_rrc_intraFreqMeasurementID, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MeasurementIdentity },
  { &hf_rrc_intraFreqCellInfoSI_List, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_IntraFreqCellInfoSI_List_RSCP },
  { &hf_rrc_intraFreqMeasQuantity, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_IntraFreqMeasQuantity },
  { &hf_rrc_intraFreqReportingQuantityForRACH, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_IntraFreqReportingQuantityForRACH },
  { &hf_rrc_maxReportedCellsOnRACH, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MaxReportedCellsOnRACH },
  { &hf_rrc_reportingInfoForCellDCH, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_ReportingInfoForCellDCH },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_IntraFreqMeasurementSysInfo_RSCP(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_IntraFreqMeasurementSysInfo_RSCP, IntraFreqMeasurementSysInfo_RSCP_sequence);

  return offset;
}


static const per_sequence_t IntraFreqMeasurementSysInfo_ECN0_sequence[] = {
  { &hf_rrc_intraFreqMeasurementID, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MeasurementIdentity },
  { &hf_rrc_intraFreqCellInfoSI_List_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_IntraFreqCellInfoSI_List_ECN0 },
  { &hf_rrc_intraFreqMeasQuantity, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_IntraFreqMeasQuantity },
  { &hf_rrc_intraFreqReportingQuantityForRACH, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_IntraFreqReportingQuantityForRACH },
  { &hf_rrc_maxReportedCellsOnRACH, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MaxReportedCellsOnRACH },
  { &hf_rrc_reportingInfoForCellDCH, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_ReportingInfoForCellDCH },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_IntraFreqMeasurementSysInfo_ECN0(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_IntraFreqMeasurementSysInfo_ECN0, IntraFreqMeasurementSysInfo_ECN0_sequence);

  return offset;
}


static const per_sequence_t IntraFreqMeasurementSysInfo_HCS_RSCP_sequence[] = {
  { &hf_rrc_intraFreqMeasurementID, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MeasurementIdentity },
  { &hf_rrc_intraFreqCellInfoSI_List_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_IntraFreqCellInfoSI_List_HCS_RSCP },
  { &hf_rrc_intraFreqMeasQuantity, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_IntraFreqMeasQuantity },
  { &hf_rrc_intraFreqReportingQuantityForRACH, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_IntraFreqReportingQuantityForRACH },
  { &hf_rrc_maxReportedCellsOnRACH, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MaxReportedCellsOnRACH },
  { &hf_rrc_reportingInfoForCellDCH, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_ReportingInfoForCellDCH },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_IntraFreqMeasurementSysInfo_HCS_RSCP(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_IntraFreqMeasurementSysInfo_HCS_RSCP, IntraFreqMeasurementSysInfo_HCS_RSCP_sequence);

  return offset;
}


static const per_sequence_t IntraFreqMeasurementSysInfo_HCS_ECN0_sequence[] = {
  { &hf_rrc_intraFreqMeasurementID, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MeasurementIdentity },
  { &hf_rrc_intraFreqCellInfoSI_List_03, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_IntraFreqCellInfoSI_List_HCS_ECN0 },
  { &hf_rrc_intraFreqMeasQuantity, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_IntraFreqMeasQuantity },
  { &hf_rrc_intraFreqReportingQuantityForRACH, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_IntraFreqReportingQuantityForRACH },
  { &hf_rrc_maxReportedCellsOnRACH, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MaxReportedCellsOnRACH },
  { &hf_rrc_reportingInfoForCellDCH, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_ReportingInfoForCellDCH },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_IntraFreqMeasurementSysInfo_HCS_ECN0(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_IntraFreqMeasurementSysInfo_HCS_ECN0, IntraFreqMeasurementSysInfo_HCS_ECN0_sequence);

  return offset;
}


static const per_sequence_t ReportingInfoForCellDCH_LCR_r4_sequence[] = {
  { &hf_rrc_intraFreqReportingQuantity, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_IntraFreqReportingQuantity },
  { &hf_rrc_measurementReportingMode, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MeasurementReportingMode },
  { &hf_rrc_reportCriteria_17, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CellDCH_ReportCriteria_LCR_r4 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_ReportingInfoForCellDCH_LCR_r4(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_ReportingInfoForCellDCH_LCR_r4, ReportingInfoForCellDCH_LCR_r4_sequence);

  return offset;
}


static const per_sequence_t IntraFreqMeasurementSysInfo_RSCP_LCR_r4_sequence[] = {
  { &hf_rrc_intraFreqMeasurementID, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MeasurementIdentity },
  { &hf_rrc_intraFreqCellInfoSI_List_04, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_IntraFreqCellInfoSI_List_RSCP_LCR_r4 },
  { &hf_rrc_intraFreqMeasQuantity, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_IntraFreqMeasQuantity },
  { &hf_rrc_intraFreqReportingQuantityForRACH, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_IntraFreqReportingQuantityForRACH },
  { &hf_rrc_maxReportedCellsOnRACH, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MaxReportedCellsOnRACH },
  { &hf_rrc_reportingInfoForCellDCH_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_ReportingInfoForCellDCH_LCR_r4 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_IntraFreqMeasurementSysInfo_RSCP_LCR_r4(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_IntraFreqMeasurementSysInfo_RSCP_LCR_r4, IntraFreqMeasurementSysInfo_RSCP_LCR_r4_sequence);

  return offset;
}


static const per_sequence_t IntraFreqMeasurementSysInfo_ECN0_LCR_r4_sequence[] = {
  { &hf_rrc_intraFreqMeasurementID, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MeasurementIdentity },
  { &hf_rrc_intraFreqCellInfoSI_List_05, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_IntraFreqCellInfoSI_List_ECN0_LCR_r4 },
  { &hf_rrc_intraFreqMeasQuantity, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_IntraFreqMeasQuantity },
  { &hf_rrc_intraFreqReportingQuantityForRACH, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_IntraFreqReportingQuantityForRACH },
  { &hf_rrc_maxReportedCellsOnRACH, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MaxReportedCellsOnRACH },
  { &hf_rrc_reportingInfoForCellDCH_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_ReportingInfoForCellDCH_LCR_r4 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_IntraFreqMeasurementSysInfo_ECN0_LCR_r4(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_IntraFreqMeasurementSysInfo_ECN0_LCR_r4, IntraFreqMeasurementSysInfo_ECN0_LCR_r4_sequence);

  return offset;
}


static const per_sequence_t IntraFreqMeasurementSysInfo_HCS_RSCP_LCR_r4_sequence[] = {
  { &hf_rrc_intraFreqMeasurementID, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MeasurementIdentity },
  { &hf_rrc_intraFreqCellInfoSI_List_06, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_IntraFreqCellInfoSI_List_HCS_RSCP_LCR_r4 },
  { &hf_rrc_intraFreqMeasQuantity, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_IntraFreqMeasQuantity },
  { &hf_rrc_intraFreqReportingQuantityForRACH, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_IntraFreqReportingQuantityForRACH },
  { &hf_rrc_maxReportedCellsOnRACH, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MaxReportedCellsOnRACH },
  { &hf_rrc_reportingInfoForCellDCH_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_ReportingInfoForCellDCH_LCR_r4 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_IntraFreqMeasurementSysInfo_HCS_RSCP_LCR_r4(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_IntraFreqMeasurementSysInfo_HCS_RSCP_LCR_r4, IntraFreqMeasurementSysInfo_HCS_RSCP_LCR_r4_sequence);

  return offset;
}


static const per_sequence_t IntraFreqMeasurementSysInfo_HCS_ECN0_LCR_r4_sequence[] = {
  { &hf_rrc_intraFreqMeasurementID, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MeasurementIdentity },
  { &hf_rrc_intraFreqCellInfoSI_List_07, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_IntraFreqCellInfoSI_List_HCS_ECN0_LCR_r4 },
  { &hf_rrc_intraFreqMeasQuantity, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_IntraFreqMeasQuantity },
  { &hf_rrc_intraFreqReportingQuantityForRACH, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_IntraFreqReportingQuantityForRACH },
  { &hf_rrc_maxReportedCellsOnRACH, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MaxReportedCellsOnRACH },
  { &hf_rrc_reportingInfoForCellDCH_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_ReportingInfoForCellDCH_LCR_r4 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_IntraFreqMeasurementSysInfo_HCS_ECN0_LCR_r4(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_IntraFreqMeasurementSysInfo_HCS_ECN0_LCR_r4, IntraFreqMeasurementSysInfo_HCS_ECN0_LCR_r4_sequence);

  return offset;
}


static const per_sequence_t T_cpich_RSCP_sequence[] = {
  { &hf_rrc_intraFreqMeasurementSysInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_IntraFreqMeasurementSysInfo_RSCP },
  { &hf_rrc_interFreqMeasurementSysInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_InterFreqMeasurementSysInfo_RSCP },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_cpich_RSCP(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_cpich_RSCP, T_cpich_RSCP_sequence);

  return offset;
}


static const per_sequence_t T_cpich_Ec_N0_01_sequence[] = {
  { &hf_rrc_intraFreqMeasurementSysInfo_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_IntraFreqMeasurementSysInfo_ECN0 },
  { &hf_rrc_interFreqMeasurementSysInfo_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_InterFreqMeasurementSysInfo_ECN0 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_cpich_Ec_N0_01(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_cpich_Ec_N0_01, T_cpich_Ec_N0_01_sequence);

  return offset;
}


static const value_string rrc_T_cellSelectQualityMeasure_01_vals[] = {
  {   0, "cpich-RSCP" },
  {   1, "cpich-Ec-N0" },
  { 0, NULL }
};

static const per_choice_t T_cellSelectQualityMeasure_01_choice[] = {
  {   0, &hf_rrc_cpich_RSCP_02   , ASN1_NO_EXTENSIONS     , dissect_rrc_T_cpich_RSCP },
  {   1, &hf_rrc_cpich_Ec_N0_02  , ASN1_NO_EXTENSIONS     , dissect_rrc_T_cpich_Ec_N0_01 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_cellSelectQualityMeasure_01(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_cellSelectQualityMeasure_01, T_cellSelectQualityMeasure_01_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t T_hcs_not_used_sequence[] = {
  { &hf_rrc_cellSelectQualityMeasure_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_cellSelectQualityMeasure_01 },
  { &hf_rrc_interRATMeasurementSysInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_InterRATMeasurementSysInfo_B },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_hcs_not_used(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_hcs_not_used, T_hcs_not_used_sequence);

  return offset;
}


static const per_sequence_t T_cpich_RSCP_01_sequence[] = {
  { &hf_rrc_intraFreqMeasurementSysInfo_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_IntraFreqMeasurementSysInfo_HCS_RSCP },
  { &hf_rrc_interFreqMeasurementSysInfo_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_InterFreqMeasurementSysInfo_HCS_RSCP },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_cpich_RSCP_01(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_cpich_RSCP_01, T_cpich_RSCP_01_sequence);

  return offset;
}


static const per_sequence_t T_cpich_Ec_N0_02_sequence[] = {
  { &hf_rrc_intraFreqMeasurementSysInfo_03, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_IntraFreqMeasurementSysInfo_HCS_ECN0 },
  { &hf_rrc_interFreqMeasurementSysInfo_03, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_InterFreqMeasurementSysInfo_HCS_ECN0 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_cpich_Ec_N0_02(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_cpich_Ec_N0_02, T_cpich_Ec_N0_02_sequence);

  return offset;
}


static const value_string rrc_T_cellSelectQualityMeasure_02_vals[] = {
  {   0, "cpich-RSCP" },
  {   1, "cpich-Ec-N0" },
  { 0, NULL }
};

static const per_choice_t T_cellSelectQualityMeasure_02_choice[] = {
  {   0, &hf_rrc_cpich_RSCP_03   , ASN1_NO_EXTENSIONS     , dissect_rrc_T_cpich_RSCP_01 },
  {   1, &hf_rrc_cpich_Ec_N0_03  , ASN1_NO_EXTENSIONS     , dissect_rrc_T_cpich_Ec_N0_02 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_cellSelectQualityMeasure_02(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_cellSelectQualityMeasure_02, T_cellSelectQualityMeasure_02_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t T_hcs_used_sequence[] = {
  { &hf_rrc_cellSelectQualityMeasure_02, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_cellSelectQualityMeasure_02 },
  { &hf_rrc_interRATMeasurementSysInfo_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_InterRATMeasurementSysInfo },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_hcs_used(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_hcs_used, T_hcs_used_sequence);

  return offset;
}


static const value_string rrc_T_use_of_HCS_vals[] = {
  {   0, "hcs-not-used" },
  {   1, "hcs-used" },
  { 0, NULL }
};

static const per_choice_t T_use_of_HCS_choice[] = {
  {   0, &hf_rrc_hcs_not_used    , ASN1_NO_EXTENSIONS     , dissect_rrc_T_hcs_not_used },
  {   1, &hf_rrc_hcs_used        , ASN1_NO_EXTENSIONS     , dissect_rrc_T_hcs_used },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_use_of_HCS(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_use_of_HCS, T_use_of_HCS_choice,
                                 NULL);

  return offset;
}


static const value_string rrc_TrafficVolumeReportCriteriaSysInfo_vals[] = {
  {   0, "trafficVolumeReportingCriteria" },
  {   1, "periodicalReportingCriteria" },
  { 0, NULL }
};

static const per_choice_t TrafficVolumeReportCriteriaSysInfo_choice[] = {
  {   0, &hf_rrc_trafficVolumeReportingCriteria, ASN1_NO_EXTENSIONS     , dissect_rrc_TrafficVolumeReportingCriteria },
  {   1, &hf_rrc_periodicalReportingCriteria, ASN1_NO_EXTENSIONS     , dissect_rrc_PeriodicalReportingCriteria },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_TrafficVolumeReportCriteriaSysInfo(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_TrafficVolumeReportCriteriaSysInfo, TrafficVolumeReportCriteriaSysInfo_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t TrafficVolumeMeasSysInfo_sequence[] = {
  { &hf_rrc_trafficVolumeMeasurementID, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MeasurementIdentity },
  { &hf_rrc_trafficVolumeMeasurementObjectList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_TrafficVolumeMeasurementObjectList },
  { &hf_rrc_trafficVolumeMeasQuantity, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_TrafficVolumeMeasQuantity },
  { &hf_rrc_trafficVolumeReportingQuantity, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_TrafficVolumeReportingQuantity },
  { &hf_rrc_dummy_53        , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_TrafficVolumeReportingCriteria },
  { &hf_rrc_measurementValidity, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MeasurementValidity },
  { &hf_rrc_measurementReportingMode, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MeasurementReportingMode },
  { &hf_rrc_reportCriteriaSysInf, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TrafficVolumeReportCriteriaSysInfo },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_TrafficVolumeMeasSysInfo(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_TrafficVolumeMeasSysInfo, TrafficVolumeMeasSysInfo_sequence);

  return offset;
}


static const per_sequence_t UE_InternalMeasurementSysInfo_sequence[] = {
  { &hf_rrc_ue_InternalMeasurementID, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MeasurementIdentity },
  { &hf_rrc_ue_InternalMeasQuantity, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UE_InternalMeasQuantity },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UE_InternalMeasurementSysInfo(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UE_InternalMeasurementSysInfo, UE_InternalMeasurementSysInfo_sequence);

  return offset;
}


static const per_sequence_t MeasurementControlSysInfo_sequence[] = {
  { &hf_rrc_use_of_HCS      , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_use_of_HCS },
  { &hf_rrc_trafficVolumeMeasSysInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_TrafficVolumeMeasSysInfo },
  { &hf_rrc_dummy_52        , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_InternalMeasurementSysInfo },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_MeasurementControlSysInfo(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_MeasurementControlSysInfo, MeasurementControlSysInfo_sequence);

  return offset;
}


static const per_sequence_t T_cpich_RSCP_02_sequence[] = {
  { &hf_rrc_newIntraFreqCellList_06, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_NewIntraFreqCellSI_List_RSCP },
  { &hf_rrc_newInterFreqCellList_07, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_NewInterFreqCellSI_List_RSCP },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_cpich_RSCP_02(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_cpich_RSCP_02, T_cpich_RSCP_02_sequence);

  return offset;
}


static const per_sequence_t T_cpich_Ec_N0_03_sequence[] = {
  { &hf_rrc_newIntraFreqCellList_07, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_NewIntraFreqCellSI_List_ECN0 },
  { &hf_rrc_newInterFreqCellList_08, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_NewInterFreqCellSI_List_ECN0 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_cpich_Ec_N0_03(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_cpich_Ec_N0_03, T_cpich_Ec_N0_03_sequence);

  return offset;
}


static const value_string rrc_T_cellSelectQualityMeasure_03_vals[] = {
  {   0, "cpich-RSCP" },
  {   1, "cpich-Ec-N0" },
  { 0, NULL }
};

static const per_choice_t T_cellSelectQualityMeasure_03_choice[] = {
  {   0, &hf_rrc_cpich_RSCP_04   , ASN1_NO_EXTENSIONS     , dissect_rrc_T_cpich_RSCP_02 },
  {   1, &hf_rrc_cpich_Ec_N0_04  , ASN1_NO_EXTENSIONS     , dissect_rrc_T_cpich_Ec_N0_03 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_cellSelectQualityMeasure_03(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_cellSelectQualityMeasure_03, T_cellSelectQualityMeasure_03_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t T_hcs_not_used_01_sequence[] = {
  { &hf_rrc_cellSelectQualityMeasure_03, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_cellSelectQualityMeasure_03 },
  { &hf_rrc_newInterRATCellList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_NewInterRATCellList },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_hcs_not_used_01(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_hcs_not_used_01, T_hcs_not_used_01_sequence);

  return offset;
}


static const per_sequence_t T_cpich_RSCP_03_sequence[] = {
  { &hf_rrc_newIntraFreqCellList_08, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_NewIntraFreqCellSI_List_HCS_RSCP },
  { &hf_rrc_newInterFreqCellList_09, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_NewInterFreqCellSI_List_HCS_RSCP },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_cpich_RSCP_03(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_cpich_RSCP_03, T_cpich_RSCP_03_sequence);

  return offset;
}


static const per_sequence_t T_cpich_Ec_N0_04_sequence[] = {
  { &hf_rrc_newIntraFreqCellList_09, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_NewIntraFreqCellSI_List_HCS_ECN0 },
  { &hf_rrc_newInterFreqCellList_10, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_NewInterFreqCellSI_List_HCS_ECN0 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_cpich_Ec_N0_04(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_cpich_Ec_N0_04, T_cpich_Ec_N0_04_sequence);

  return offset;
}


static const value_string rrc_T_cellSelectQualityMeasure_04_vals[] = {
  {   0, "cpich-RSCP" },
  {   1, "cpich-Ec-N0" },
  { 0, NULL }
};

static const per_choice_t T_cellSelectQualityMeasure_04_choice[] = {
  {   0, &hf_rrc_cpich_RSCP_05   , ASN1_NO_EXTENSIONS     , dissect_rrc_T_cpich_RSCP_03 },
  {   1, &hf_rrc_cpich_Ec_N0_05  , ASN1_NO_EXTENSIONS     , dissect_rrc_T_cpich_Ec_N0_04 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_cellSelectQualityMeasure_04(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_cellSelectQualityMeasure_04, T_cellSelectQualityMeasure_04_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t T_hcs_used_01_sequence[] = {
  { &hf_rrc_cellSelectQualityMeasure_04, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_cellSelectQualityMeasure_04 },
  { &hf_rrc_newInterRATCellList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_NewInterRATCellList },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_hcs_used_01(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_hcs_used_01, T_hcs_used_01_sequence);

  return offset;
}


static const value_string rrc_T_use_of_HCS_01_vals[] = {
  {   0, "hcs-not-used" },
  {   1, "hcs-used" },
  { 0, NULL }
};

static const per_choice_t T_use_of_HCS_01_choice[] = {
  {   0, &hf_rrc_hcs_not_used_01 , ASN1_NO_EXTENSIONS     , dissect_rrc_T_hcs_not_used_01 },
  {   1, &hf_rrc_hcs_used_01     , ASN1_NO_EXTENSIONS     , dissect_rrc_T_hcs_used_01 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_use_of_HCS_01(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_use_of_HCS_01, T_use_of_HCS_01_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t MeasurementControlSysInfoExtension_sequence[] = {
  { &hf_rrc_use_of_HCS_01   , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_use_of_HCS_01 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_MeasurementControlSysInfoExtension(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_MeasurementControlSysInfoExtension, MeasurementControlSysInfoExtension_sequence);

  return offset;
}


static const per_sequence_t SEQUENCE_SIZE_1_maxCellMeas_OF_CellSelectReselectInfo_v590ext_sequence_of[1] = {
  { &hf_rrc_newIntraFrequencyCellInfoListAddon_r5_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CellSelectReselectInfo_v590ext },
};

static int
dissect_rrc_SEQUENCE_SIZE_1_maxCellMeas_OF_CellSelectReselectInfo_v590ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_SEQUENCE_SIZE_1_maxCellMeas_OF_CellSelectReselectInfo_v590ext, SEQUENCE_SIZE_1_maxCellMeas_OF_CellSelectReselectInfo_v590ext_sequence_of,
                                                  1, maxCellMeas, FALSE);

  return offset;
}


static const per_sequence_t MeasurementControlSysInfoExtensionAddon_r5_sequence[] = {
  { &hf_rrc_newIntraFrequencyCellInfoListAddon_r5, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_SEQUENCE_SIZE_1_maxCellMeas_OF_CellSelectReselectInfo_v590ext },
  { &hf_rrc_newInterFrequencyCellInfoListAddon_r5, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_SEQUENCE_SIZE_1_maxCellMeas_OF_CellSelectReselectInfo_v590ext },
  { &hf_rrc_newInterRATCellInfoListAddon_r5, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_SEQUENCE_SIZE_1_maxCellMeas_OF_CellSelectReselectInfo_v590ext },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_MeasurementControlSysInfoExtensionAddon_r5(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_MeasurementControlSysInfoExtensionAddon_r5, MeasurementControlSysInfoExtensionAddon_r5_sequence);

  return offset;
}


static const per_sequence_t T_cpich_RSCP_04_sequence[] = {
  { &hf_rrc_newIntraFreqCellList_10, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_NewIntraFreqCellSI_List_RSCP_LCR_r4 },
  { &hf_rrc_newInterFreqCellList_11, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_NewInterFreqCellSI_List_RSCP_LCR_r4 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_cpich_RSCP_04(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_cpich_RSCP_04, T_cpich_RSCP_04_sequence);

  return offset;
}


static const per_sequence_t T_cpich_Ec_N0_05_sequence[] = {
  { &hf_rrc_newIntraFreqCellList_11, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_NewIntraFreqCellSI_List_ECN0_LCR_r4 },
  { &hf_rrc_newInterFreqCellList_12, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_NewInterFreqCellSI_List_ECN0_LCR_r4 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_cpich_Ec_N0_05(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_cpich_Ec_N0_05, T_cpich_Ec_N0_05_sequence);

  return offset;
}


static const value_string rrc_T_cellSelectQualityMeasure_05_vals[] = {
  {   0, "cpich-RSCP" },
  {   1, "cpich-Ec-N0" },
  { 0, NULL }
};

static const per_choice_t T_cellSelectQualityMeasure_05_choice[] = {
  {   0, &hf_rrc_cpich_RSCP_06   , ASN1_NO_EXTENSIONS     , dissect_rrc_T_cpich_RSCP_04 },
  {   1, &hf_rrc_cpich_Ec_N0_06  , ASN1_NO_EXTENSIONS     , dissect_rrc_T_cpich_Ec_N0_05 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_cellSelectQualityMeasure_05(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_cellSelectQualityMeasure_05, T_cellSelectQualityMeasure_05_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t T_hcs_not_used_02_sequence[] = {
  { &hf_rrc_cellSelectQualityMeasure_05, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_cellSelectQualityMeasure_05 },
  { &hf_rrc_newInterRATCellList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_NewInterRATCellList },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_hcs_not_used_02(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_hcs_not_used_02, T_hcs_not_used_02_sequence);

  return offset;
}


static const per_sequence_t T_cpich_RSCP_05_sequence[] = {
  { &hf_rrc_newIntraFreqCellList_12, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_NewIntraFreqCellSI_List_HCS_RSCP_LCR_r4 },
  { &hf_rrc_newInterFreqCellList_13, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_NewInterFreqCellSI_List_HCS_RSCP_LCR_r4 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_cpich_RSCP_05(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_cpich_RSCP_05, T_cpich_RSCP_05_sequence);

  return offset;
}


static const per_sequence_t T_cpich_Ec_N0_06_sequence[] = {
  { &hf_rrc_newIntraFreqCellList_13, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_NewIntraFreqCellSI_List_HCS_ECN0_LCR_r4 },
  { &hf_rrc_newInterFreqCellList_14, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_NewInterFreqCellSI_List_HCS_ECN0_LCR_r4 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_cpich_Ec_N0_06(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_cpich_Ec_N0_06, T_cpich_Ec_N0_06_sequence);

  return offset;
}


static const value_string rrc_T_cellSelectQualityMeasure_06_vals[] = {
  {   0, "cpich-RSCP" },
  {   1, "cpich-Ec-N0" },
  { 0, NULL }
};

static const per_choice_t T_cellSelectQualityMeasure_06_choice[] = {
  {   0, &hf_rrc_cpich_RSCP_07   , ASN1_NO_EXTENSIONS     , dissect_rrc_T_cpich_RSCP_05 },
  {   1, &hf_rrc_cpich_Ec_N0_07  , ASN1_NO_EXTENSIONS     , dissect_rrc_T_cpich_Ec_N0_06 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_cellSelectQualityMeasure_06(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_cellSelectQualityMeasure_06, T_cellSelectQualityMeasure_06_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t T_hcs_used_02_sequence[] = {
  { &hf_rrc_cellSelectQualityMeasure_06, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_cellSelectQualityMeasure_06 },
  { &hf_rrc_newInterRATCellList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_NewInterRATCellList },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_hcs_used_02(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_hcs_used_02, T_hcs_used_02_sequence);

  return offset;
}


static const value_string rrc_T_use_of_HCS_02_vals[] = {
  {   0, "hcs-not-used" },
  {   1, "hcs-used" },
  { 0, NULL }
};

static const per_choice_t T_use_of_HCS_02_choice[] = {
  {   0, &hf_rrc_hcs_not_used_02 , ASN1_NO_EXTENSIONS     , dissect_rrc_T_hcs_not_used_02 },
  {   1, &hf_rrc_hcs_used_02     , ASN1_NO_EXTENSIONS     , dissect_rrc_T_hcs_used_02 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_use_of_HCS_02(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_use_of_HCS_02, T_use_of_HCS_02_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t MeasurementControlSysInfoExtension_LCR_r4_sequence[] = {
  { &hf_rrc_use_of_HCS_02   , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_use_of_HCS_02 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_MeasurementControlSysInfoExtension_LCR_r4(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_MeasurementControlSysInfoExtension_LCR_r4, MeasurementControlSysInfoExtension_LCR_r4_sequence);

  return offset;
}


static const per_sequence_t T_cpich_RSCP_06_sequence[] = {
  { &hf_rrc_intraFreqMeasurementSysInfo_04, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_IntraFreqMeasurementSysInfo_RSCP_LCR_r4 },
  { &hf_rrc_interFreqMeasurementSysInfo_04, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_InterFreqMeasurementSysInfo_RSCP_LCR_r4 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_cpich_RSCP_06(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_cpich_RSCP_06, T_cpich_RSCP_06_sequence);

  return offset;
}


static const per_sequence_t T_cpich_Ec_N0_07_sequence[] = {
  { &hf_rrc_intraFreqMeasurementSysInfo_05, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_IntraFreqMeasurementSysInfo_ECN0_LCR_r4 },
  { &hf_rrc_interFreqMeasurementSysInfo_05, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_InterFreqMeasurementSysInfo_ECN0_LCR_r4 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_cpich_Ec_N0_07(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_cpich_Ec_N0_07, T_cpich_Ec_N0_07_sequence);

  return offset;
}


static const value_string rrc_T_cellSelectQualityMeasure_07_vals[] = {
  {   0, "cpich-RSCP" },
  {   1, "cpich-Ec-N0" },
  { 0, NULL }
};

static const per_choice_t T_cellSelectQualityMeasure_07_choice[] = {
  {   0, &hf_rrc_cpich_RSCP_08   , ASN1_NO_EXTENSIONS     , dissect_rrc_T_cpich_RSCP_06 },
  {   1, &hf_rrc_cpich_Ec_N0_08  , ASN1_NO_EXTENSIONS     , dissect_rrc_T_cpich_Ec_N0_07 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_cellSelectQualityMeasure_07(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_cellSelectQualityMeasure_07, T_cellSelectQualityMeasure_07_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t T_hcs_not_used_03_sequence[] = {
  { &hf_rrc_cellSelectQualityMeasure_07, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_cellSelectQualityMeasure_07 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_hcs_not_used_03(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_hcs_not_used_03, T_hcs_not_used_03_sequence);

  return offset;
}


static const per_sequence_t T_cpich_RSCP_07_sequence[] = {
  { &hf_rrc_intraFreqMeasurementSysInfo_06, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_IntraFreqMeasurementSysInfo_HCS_RSCP_LCR_r4 },
  { &hf_rrc_interFreqMeasurementSysInfo_06, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_InterFreqMeasurementSysInfo_HCS_RSCP_LCR_r4 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_cpich_RSCP_07(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_cpich_RSCP_07, T_cpich_RSCP_07_sequence);

  return offset;
}


static const per_sequence_t T_cpich_Ec_N0_08_sequence[] = {
  { &hf_rrc_intraFreqMeasurementSysInfo_07, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_IntraFreqMeasurementSysInfo_HCS_ECN0_LCR_r4 },
  { &hf_rrc_interFreqMeasurementSysInfo_07, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_InterFreqMeasurementSysInfo_HCS_ECN0_LCR_r4 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_cpich_Ec_N0_08(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_cpich_Ec_N0_08, T_cpich_Ec_N0_08_sequence);

  return offset;
}


static const value_string rrc_T_cellSelectQualityMeasure_08_vals[] = {
  {   0, "cpich-RSCP" },
  {   1, "cpich-Ec-N0" },
  { 0, NULL }
};

static const per_choice_t T_cellSelectQualityMeasure_08_choice[] = {
  {   0, &hf_rrc_cpich_RSCP_09   , ASN1_NO_EXTENSIONS     , dissect_rrc_T_cpich_RSCP_07 },
  {   1, &hf_rrc_cpich_Ec_N0_09  , ASN1_NO_EXTENSIONS     , dissect_rrc_T_cpich_Ec_N0_08 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_cellSelectQualityMeasure_08(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_cellSelectQualityMeasure_08, T_cellSelectQualityMeasure_08_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t T_hcs_used_03_sequence[] = {
  { &hf_rrc_cellSelectQualityMeasure_08, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_cellSelectQualityMeasure_08 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_hcs_used_03(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_hcs_used_03, T_hcs_used_03_sequence);

  return offset;
}


static const value_string rrc_T_use_of_HCS_03_vals[] = {
  {   0, "hcs-not-used" },
  {   1, "hcs-used" },
  { 0, NULL }
};

static const per_choice_t T_use_of_HCS_03_choice[] = {
  {   0, &hf_rrc_hcs_not_used_03 , ASN1_NO_EXTENSIONS     , dissect_rrc_T_hcs_not_used_03 },
  {   1, &hf_rrc_hcs_used_03     , ASN1_NO_EXTENSIONS     , dissect_rrc_T_hcs_used_03 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_use_of_HCS_03(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_use_of_HCS_03, T_use_of_HCS_03_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t MeasurementControlSysInfo_LCR_r4_ext_sequence[] = {
  { &hf_rrc_use_of_HCS_03   , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_use_of_HCS_03 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_MeasurementControlSysInfo_LCR_r4_ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_MeasurementControlSysInfo_LCR_r4_ext, MeasurementControlSysInfo_LCR_r4_ext_sequence);

  return offset;
}


static const per_sequence_t MultipleEUTRAFrequencyBandIndicatorList_sequence_of[1] = {
  { &hf_rrc_MultipleEUTRAFrequencyBandIndicatorList_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RadioFrequencyBandEUTRA },
};

static int
dissect_rrc_MultipleEUTRAFrequencyBandIndicatorList(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_MultipleEUTRAFrequencyBandIndicatorList, MultipleEUTRAFrequencyBandIndicatorList_sequence_of,
                                                  1, maxMultipleFrequencyBandsEUTRA, FALSE);

  return offset;
}


static const per_sequence_t MultipleEUTRAFrequencyBandInfo_sequence[] = {
  { &hf_rrc_multipleEUTRAFrequencyBandIndicatorlist, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MultipleEUTRAFrequencyBandIndicatorList },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_MultipleEUTRAFrequencyBandInfo(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_MultipleEUTRAFrequencyBandInfo, MultipleEUTRAFrequencyBandInfo_sequence);

  return offset;
}


static const per_sequence_t MultipleEUTRAFrequencyInfoList_sequence_of[1] = {
  { &hf_rrc_MultipleEUTRAFrequencyInfoList_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MultipleEUTRAFrequencyBandInfo },
};

static int
dissect_rrc_MultipleEUTRAFrequencyInfoList(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_MultipleEUTRAFrequencyInfoList, MultipleEUTRAFrequencyInfoList_sequence_of,
                                                  1, maxNumEUTRAFreqs, FALSE);

  return offset;
}


static const per_sequence_t MultipleFrequencyBandIndicatorListFDD_sequence_of[1] = {
  { &hf_rrc_MultipleFrequencyBandIndicatorListFDD_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_FrequencyBandsIndicatorFDD },
};

static int
dissect_rrc_MultipleFrequencyBandIndicatorListFDD(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_MultipleFrequencyBandIndicatorListFDD, MultipleFrequencyBandIndicatorListFDD_sequence_of,
                                                  1, maxMultipleFrequencyBandsFDD, FALSE);

  return offset;
}


static const per_sequence_t MultipleFrequencyBandInfo_sequence[] = {
  { &hf_rrc_multipleFrequencyBandIndicatorListFDD, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MultipleFrequencyBandIndicatorListFDD },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_MultipleFrequencyBandInfo(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_MultipleFrequencyBandInfo, MultipleFrequencyBandInfo_sequence);

  return offset;
}


static const per_sequence_t MultipleFrequencyInfoListFDD_sequence_of[1] = {
  { &hf_rrc_MultipleFrequencyInfoListFDD_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MultipleFrequencyBandInfo },
};

static int
dissect_rrc_MultipleFrequencyInfoListFDD(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_MultipleFrequencyInfoListFDD, MultipleFrequencyInfoListFDD_sequence_of,
                                                  1, maxNumFDDFreqs, FALSE);

  return offset;
}


static const per_sequence_t PLMNsOfIntraFreqCellsList_item_sequence[] = {
  { &hf_rrc_plmn_Identity   , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_PLMN_Identity },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_PLMNsOfIntraFreqCellsList_item(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_PLMNsOfIntraFreqCellsList_item, PLMNsOfIntraFreqCellsList_item_sequence);

  return offset;
}


static const per_sequence_t PLMNsOfIntraFreqCellsList_sequence_of[1] = {
  { &hf_rrc_PLMNsOfIntraFreqCellsList_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PLMNsOfIntraFreqCellsList_item },
};

static int
dissect_rrc_PLMNsOfIntraFreqCellsList(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_PLMNsOfIntraFreqCellsList, PLMNsOfIntraFreqCellsList_sequence_of,
                                                  1, maxCellMeas, FALSE);

  return offset;
}


static const per_sequence_t PLMNsOfInterFreqCellsList_item_sequence[] = {
  { &hf_rrc_plmn_Identity   , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_PLMN_Identity },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_PLMNsOfInterFreqCellsList_item(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_PLMNsOfInterFreqCellsList_item, PLMNsOfInterFreqCellsList_item_sequence);

  return offset;
}


static const per_sequence_t PLMNsOfInterFreqCellsList_sequence_of[1] = {
  { &hf_rrc_PLMNsOfInterFreqCellsList_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PLMNsOfInterFreqCellsList_item },
};

static int
dissect_rrc_PLMNsOfInterFreqCellsList(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_PLMNsOfInterFreqCellsList, PLMNsOfInterFreqCellsList_sequence_of,
                                                  1, maxCellMeas, FALSE);

  return offset;
}


static const per_sequence_t PLMNsOfInterRATCellsList_item_sequence[] = {
  { &hf_rrc_plmn_Identity   , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_PLMN_Identity },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_PLMNsOfInterRATCellsList_item(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_PLMNsOfInterRATCellsList_item, PLMNsOfInterRATCellsList_item_sequence);

  return offset;
}


static const per_sequence_t PLMNsOfInterRATCellsList_sequence_of[1] = {
  { &hf_rrc_PLMNsOfInterRATCellsList_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PLMNsOfInterRATCellsList_item },
};

static int
dissect_rrc_PLMNsOfInterRATCellsList(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_PLMNsOfInterRATCellsList, PLMNsOfInterRATCellsList_sequence_of,
                                                  1, maxCellMeas, FALSE);

  return offset;
}


static const per_sequence_t PLMNIdentitiesOfNeighbourCells_sequence[] = {
  { &hf_rrc_plmnsOfIntraFreqCellsList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_PLMNsOfIntraFreqCellsList },
  { &hf_rrc_plmnsOfInterFreqCellsList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_PLMNsOfInterFreqCellsList },
  { &hf_rrc_plmnsOfInterRATCellsList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_PLMNsOfInterRATCellsList },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_PLMNIdentitiesOfNeighbourCells(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_PLMNIdentitiesOfNeighbourCells, PLMNIdentitiesOfNeighbourCells_sequence);

  return offset;
}


static const per_sequence_t SEQUENCE_SIZE_1_6_OF_PLMN_IdentityWithOptionalMCC_r6_sequence_of[1] = {
  { &hf_rrc_multiplePLMN_list_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PLMN_IdentityWithOptionalMCC_r6 },
};

static int
dissect_rrc_SEQUENCE_SIZE_1_6_OF_PLMN_IdentityWithOptionalMCC_r6(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_SEQUENCE_SIZE_1_6_OF_PLMN_IdentityWithOptionalMCC_r6, SEQUENCE_SIZE_1_6_OF_PLMN_IdentityWithOptionalMCC_r6_sequence_of,
                                                  1, 6, FALSE);

  return offset;
}


static const per_sequence_t MultiplePLMNsOfIntraFreqCellsList_item_sequence[] = {
  { &hf_rrc_multiplePLMN_list, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_SEQUENCE_SIZE_1_6_OF_PLMN_IdentityWithOptionalMCC_r6 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_MultiplePLMNsOfIntraFreqCellsList_item(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_MultiplePLMNsOfIntraFreqCellsList_item, MultiplePLMNsOfIntraFreqCellsList_item_sequence);

  return offset;
}


static const per_sequence_t MultiplePLMNsOfIntraFreqCellsList_sequence_of[1] = {
  { &hf_rrc_MultiplePLMNsOfIntraFreqCellsList_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MultiplePLMNsOfIntraFreqCellsList_item },
};

static int
dissect_rrc_MultiplePLMNsOfIntraFreqCellsList(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_MultiplePLMNsOfIntraFreqCellsList, MultiplePLMNsOfIntraFreqCellsList_sequence_of,
                                                  1, maxCellMeas, FALSE);

  return offset;
}


static const per_sequence_t MultiplePLMNsOfInterFreqCellsList_item_sequence[] = {
  { &hf_rrc_multiplePLMN_list, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_SEQUENCE_SIZE_1_6_OF_PLMN_IdentityWithOptionalMCC_r6 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_MultiplePLMNsOfInterFreqCellsList_item(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_MultiplePLMNsOfInterFreqCellsList_item, MultiplePLMNsOfInterFreqCellsList_item_sequence);

  return offset;
}


static const per_sequence_t MultiplePLMNsOfInterFreqCellsList_sequence_of[1] = {
  { &hf_rrc_MultiplePLMNsOfInterFreqCellsList_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MultiplePLMNsOfInterFreqCellsList_item },
};

static int
dissect_rrc_MultiplePLMNsOfInterFreqCellsList(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_MultiplePLMNsOfInterFreqCellsList, MultiplePLMNsOfInterFreqCellsList_sequence_of,
                                                  1, maxCellMeas, FALSE);

  return offset;
}


static const per_sequence_t PLMNIdentitiesOfNeighbourCells_v860ext_sequence[] = {
  { &hf_rrc_multipleplmnsOfIntraFreqCellsList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MultiplePLMNsOfIntraFreqCellsList },
  { &hf_rrc_multipleplmnsOfInterFreqCellsList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MultiplePLMNsOfInterFreqCellsList },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_PLMNIdentitiesOfNeighbourCells_v860ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_PLMNIdentitiesOfNeighbourCells_v860ext, PLMNIdentitiesOfNeighbourCells_v860ext_sequence);

  return offset;
}



static int
dissect_rrc_Q_QualMin_Offset(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            1U, 16U, NULL, FALSE);

  return offset;
}



static int
dissect_rrc_Q_RxlevMin_Offset(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            1U, 8U, NULL, FALSE);

  return offset;
}


static const per_sequence_t UE_Positioning_CipherParameters_sequence[] = {
  { &hf_rrc_cipheringKeyFlag, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_1 },
  { &hf_rrc_cipheringSerialNumber, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_0_65535 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UE_Positioning_CipherParameters(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UE_Positioning_CipherParameters, UE_Positioning_CipherParameters_sequence);

  return offset;
}


static const per_sequence_t UE_Positioning_GANSS_TimeModels_sequence_of[1] = {
  { &hf_rrc_UE_Positioning_GANSS_TimeModels_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UE_Positioning_GANSS_TimeModel },
};

static int
dissect_rrc_UE_Positioning_GANSS_TimeModels(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_UE_Positioning_GANSS_TimeModels, UE_Positioning_GANSS_TimeModels_sequence_of,
                                                  1, maxGANSS_1, FALSE);

  return offset;
}


static const per_sequence_t UE_Positioning_GANSS_TimeModels_va40ext_sequence_of[1] = {
  { &hf_rrc_UE_Positioning_GANSS_TimeModels_va40ext_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UE_Positioning_GANSS_TimeModel_va40ext },
};

static int
dissect_rrc_UE_Positioning_GANSS_TimeModels_va40ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_UE_Positioning_GANSS_TimeModels_va40ext, UE_Positioning_GANSS_TimeModels_va40ext_sequence_of,
                                                  1, maxGANSS_1, FALSE);

  return offset;
}



static int
dissect_rrc_INTEGER_M60_M13(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            -60, -13, NULL, FALSE);

  return offset;
}


static const per_sequence_t UTRAN_FDD_Frequency_sequence[] = {
  { &hf_rrc_uarfcn          , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UARFCN },
  { &hf_rrc_priority        , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_0_maxPrio_1 },
  { &hf_rrc_threshXhigh     , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_0_31 },
  { &hf_rrc_threshXlow      , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_0_31 },
  { &hf_rrc_qQualMinFDD     , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_INTEGER_M24_0 },
  { &hf_rrc_qRxLevMinFDD    , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_INTEGER_M60_M13 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UTRAN_FDD_Frequency(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UTRAN_FDD_Frequency, UTRAN_FDD_Frequency_sequence);

  return offset;
}


static const per_sequence_t UTRAN_FDD_FrequencyList_sequence_of[1] = {
  { &hf_rrc_UTRAN_FDD_FrequencyList_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UTRAN_FDD_Frequency },
};

static int
dissect_rrc_UTRAN_FDD_FrequencyList(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_UTRAN_FDD_FrequencyList, UTRAN_FDD_FrequencyList_sequence_of,
                                                  1, maxNumFDDFreqs, FALSE);

  return offset;
}


static const per_sequence_t UTRAN_TDD_Frequency_sequence[] = {
  { &hf_rrc_uarfcn          , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UARFCN },
  { &hf_rrc_priority        , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_0_maxPrio_1 },
  { &hf_rrc_threshXhigh     , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_0_31 },
  { &hf_rrc_threshXlow      , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_0_31 },
  { &hf_rrc_qRxLevMinTDD    , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_INTEGER_M60_M13 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UTRAN_TDD_Frequency(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UTRAN_TDD_Frequency, UTRAN_TDD_Frequency_sequence);

  return offset;
}


static const per_sequence_t UTRAN_TDD_FrequencyList_sequence_of[1] = {
  { &hf_rrc_UTRAN_TDD_FrequencyList_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UTRAN_TDD_Frequency },
};

static int
dissect_rrc_UTRAN_TDD_FrequencyList(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_UTRAN_TDD_FrequencyList, UTRAN_TDD_FrequencyList_sequence_of,
                                                  1, maxNumTDDFreqs, FALSE);

  return offset;
}


static const per_sequence_t T_utra_ServingCell_sequence[] = {
  { &hf_rrc_priority        , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_0_maxPrio_1 },
  { &hf_rrc_s_PrioritySearch1, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_0_31 },
  { &hf_rrc_s_PrioritySearch2, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_INTEGER_0_7 },
  { &hf_rrc_threshServingLow, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_0_31 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_utra_ServingCell(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_utra_ServingCell, T_utra_ServingCell_sequence);

  return offset;
}


static const per_sequence_t UTRA_PriorityInfoList_sequence[] = {
  { &hf_rrc_utra_ServingCell, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_utra_ServingCell },
  { &hf_rrc_utran_FDD_FrequencyList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UTRAN_FDD_FrequencyList },
  { &hf_rrc_utran_TDD_FrequencyList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UTRAN_TDD_FrequencyList },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UTRA_PriorityInfoList(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UTRA_PriorityInfoList, UTRA_PriorityInfoList_sequence);

  return offset;
}


static const per_sequence_t UTRA_PriorityInfoList_v920ext_sequence[] = {
  { &hf_rrc_threshServingLow2, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_INTEGER_0_31 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UTRA_PriorityInfoList_v920ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UTRA_PriorityInfoList_v920ext, UTRA_PriorityInfoList_v920ext_sequence);

  return offset;
}


static const per_sequence_t CBS_DRX_Level1Information_sequence[] = {
  { &hf_rrc_ctch_AllocationPeriod, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_1_256 },
  { &hf_rrc_cbs_FrameOffset , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_0_255 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_CBS_DRX_Level1Information(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_CBS_DRX_Level1Information, CBS_DRX_Level1Information_sequence);

  return offset;
}


static const value_string rrc_CBS_DRX_Level1Information_extension_r6_vals[] = {
  {   0, "p8" },
  {   1, "p16" },
  {   2, "p32" },
  {   3, "p64" },
  {   4, "p128" },
  {   5, "p256" },
  { 0, NULL }
};


static int
dissect_rrc_CBS_DRX_Level1Information_extension_r6(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     6, NULL, FALSE, 0, NULL);

  return offset;
}


static const value_string rrc_T_dataTransmFreqGranularity_vals[] = {
  {   0, "ms100" },
  {   1, "ms250" },
  {   2, "ms500" },
  {   3, "ms1000" },
  {   4, "ms2000" },
  {   5, "ms5000" },
  {   6, "ms10000" },
  {   7, "ms60000" },
  { 0, NULL }
};


static int
dissect_rrc_T_dataTransmFreqGranularity(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     8, NULL, FALSE, 0, NULL);

  return offset;
}



static int
dissect_rrc_INTEGER_1_610(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            1U, 610U, NULL, FALSE);

  return offset;
}


static const per_sequence_t DataTransmFrequency_sequence[] = {
  { &hf_rrc_dataTransmFreqGranularity, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_dataTransmFreqGranularity },
  { &hf_rrc_numberOfDataTransmOcc, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_1_610 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_DataTransmFrequency(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_DataTransmFrequency, DataTransmFrequency_sequence);

  return offset;
}



static int
dissect_rrc_INTEGER_1_120(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            1U, 120U, NULL, FALSE);

  return offset;
}


static const per_sequence_t DataVolumePerRB_sequence[] = {
  { &hf_rrc_rb_Identity     , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RB_Identity },
  { &hf_rrc_dataVolume      , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_0_4294967295 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_DataVolumePerRB(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_DataVolumePerRB, DataVolumePerRB_sequence);

  return offset;
}


static const per_sequence_t DataVolumePerRB_List_sequence_of[1] = {
  { &hf_rrc_DataVolumePerRB_List_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DataVolumePerRB },
};

static int
dissect_rrc_DataVolumePerRB_List(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_DataVolumePerRB_List, DataVolumePerRB_List_sequence_of,
                                                  1, maxRB, FALSE);

  return offset;
}


static const per_sequence_t DataVolumeHistory_sequence[] = {
  { &hf_rrc_dataVolumeMontoringWindow, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_1_120 },
  { &hf_rrc_dataTransmFrequency, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DataTransmFrequency },
  { &hf_rrc_dataVolumePerRB , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DataVolumePerRB_List },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_DataVolumeHistory(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_DataVolumeHistory, DataVolumeHistory_sequence);

  return offset;
}


static const per_sequence_t T_fdd_184_sequence[] = {
  { &hf_rrc_intraFreqMeasQuantity_FDD_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_IntraFreqMeasQuantity_FDD_sib3 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_fdd_184(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_fdd_184, T_fdd_184_sequence);

  return offset;
}


static const per_sequence_t T_tdd_133_sequence[] = {
  { &hf_rrc_intraFreqMeasQuantity_TDDList_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_IntraFreqMeasQuantity_TDD_sib3List },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_tdd_133(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_tdd_133, T_tdd_133_sequence);

  return offset;
}


static const value_string rrc_T_modeSpecificInfo_149_vals[] = {
  {   0, "fdd" },
  {   1, "tdd" },
  { 0, NULL }
};

static const per_choice_t T_modeSpecificInfo_149_choice[] = {
  {   0, &hf_rrc_fdd_190         , ASN1_NO_EXTENSIONS     , dissect_rrc_T_fdd_184 },
  {   1, &hf_rrc_tdd_141         , ASN1_NO_EXTENSIONS     , dissect_rrc_T_tdd_133 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_modeSpecificInfo_149(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_modeSpecificInfo_149, T_modeSpecificInfo_149_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t DeferredMeasurementControlReadingSupport_sequence[] = {
  { &hf_rrc_modeSpecificInfo_149, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_modeSpecificInfo_149 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_DeferredMeasurementControlReadingSupport(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_DeferredMeasurementControlReadingSupport, DeferredMeasurementControlReadingSupport_sequence);

  return offset;
}



static int
dissect_rrc_ExpirationTimeFactor(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            1U, 8U, NULL, FALSE);

  return offset;
}


static const value_string rrc_SIB_TypeExtGANSS_vals[] = {
  {   0, "systemInfoType15-1bis" },
  {   1, "systemInfoType15-2bis" },
  {   2, "systemInfoType15-2ter" },
  {   3, "systemInfoType15-3bis" },
  {   4, "systemInfoType15-6" },
  {   5, "systemInfoType15-7" },
  {   6, "systemInfoType15-8" },
  {   7, "spare9" },
  {   8, "spare8" },
  {   9, "spare7" },
  {  10, "spare6" },
  {  11, "spare5" },
  {  12, "spare4" },
  {  13, "spare3" },
  {  14, "spare2" },
  {  15, "spare1" },
  { 0, NULL }
};

static const per_choice_t SIB_TypeExtGANSS_choice[] = {
  {   0, &hf_rrc_systemInfoType15_1bis, ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   1, &hf_rrc_systemInfoType15_2bis, ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   2, &hf_rrc_systemInfoType15_2ter, ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   3, &hf_rrc_systemInfoType15_3bis, ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   4, &hf_rrc_systemInfoType15_6, ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   5, &hf_rrc_systemInfoType15_7, ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   6, &hf_rrc_systemInfoType15_8, ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   7, &hf_rrc_spare9          , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   8, &hf_rrc_spare8          , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   9, &hf_rrc_spare7          , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {  10, &hf_rrc_spare6          , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {  11, &hf_rrc_spare5          , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {  12, &hf_rrc_spare4          , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {  13, &hf_rrc_spare3          , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {  14, &hf_rrc_spare2          , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {  15, &hf_rrc_spare1          , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_SIB_TypeExtGANSS(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_SIB_TypeExtGANSS, SIB_TypeExtGANSS_choice,
                                 NULL);

  return offset;
}


static const value_string rrc_ValueTagInfo_vals[] = {
  {   0, "none" },
  {   1, "cellValueTag" },
  {   2, "plmn-ValueTag" },
  {   3, "sIBOccurrenceIdentityAndValueTag" },
  { 0, NULL }
};

static const per_choice_t ValueTagInfo_choice[] = {
  {   0, &hf_rrc_none            , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   1, &hf_rrc_cellValueTag    , ASN1_NO_EXTENSIONS     , dissect_rrc_CellValueTag },
  {   2, &hf_rrc_plmn_ValueTag   , ASN1_NO_EXTENSIONS     , dissect_rrc_PLMN_ValueTag },
  {   3, &hf_rrc_sIBOccurrenceIdentityAndValueTag, ASN1_NO_EXTENSIONS     , dissect_rrc_SIBOccurrenceIdentityAndValueTag },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_ValueTagInfo(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_ValueTagInfo, ValueTagInfo_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t ExtGANSS_SchedulingInfo_sequence[] = {
  { &hf_rrc_extensionGANSS_SIBType, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SIB_TypeExtGANSS },
  { &hf_rrc_schedulingInfo  , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SchedulingInformation },
  { &hf_rrc_valueTagInfo    , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_ValueTagInfo },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_ExtGANSS_SchedulingInfo(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_ExtGANSS_SchedulingInfo, ExtGANSS_SchedulingInfo_sequence);

  return offset;
}


static const per_sequence_t SEQUENCE_SIZE_1_maxSIB_OF_ExtGANSS_SchedulingInfo_sequence_of[1] = {
  { &hf_rrc_ganssScheduling_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_ExtGANSS_SchedulingInfo },
};

static int
dissect_rrc_SEQUENCE_SIZE_1_maxSIB_OF_ExtGANSS_SchedulingInfo(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_SEQUENCE_SIZE_1_maxSIB_OF_ExtGANSS_SchedulingInfo, SEQUENCE_SIZE_1_maxSIB_OF_ExtGANSS_SchedulingInfo_sequence_of,
                                                  1, maxSIB, FALSE);

  return offset;
}


static const per_sequence_t ExtGANSS_SIBTypeInfoSchedulingInfo_sequence[] = {
  { &hf_rrc_ganssID         , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_INTEGER_0_7 },
  { &hf_rrc_sbasID          , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_Positioning_GANSS_SBAS_ID },
  { &hf_rrc_ganssScheduling , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SEQUENCE_SIZE_1_maxSIB_OF_ExtGANSS_SchedulingInfo },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_ExtGANSS_SIBTypeInfoSchedulingInfo(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_ExtGANSS_SIBTypeInfoSchedulingInfo, ExtGANSS_SIBTypeInfoSchedulingInfo_sequence);

  return offset;
}


static const per_sequence_t ExtGANSS_SIBTypeInfoSchedulingInfoList_sequence_of[1] = {
  { &hf_rrc_ExtGANSS_SIBTypeInfoSchedulingInfoList_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_ExtGANSS_SIBTypeInfoSchedulingInfo },
};

static int
dissect_rrc_ExtGANSS_SIBTypeInfoSchedulingInfoList(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_ExtGANSS_SIBTypeInfoSchedulingInfoList, ExtGANSS_SIBTypeInfoSchedulingInfoList_sequence_of,
                                                  1, maxGANSS, FALSE);

  return offset;
}


static const value_string rrc_SIB_TypeExt_vals[] = {
  {   0, "systemInfoType11bis" },
  {   1, "systemInfoType15bis" },
  {   2, "systemInfoType15-1bis" },
  {   3, "systemInfoType15-2bis" },
  {   4, "systemInfoType15-3bis" },
  {   5, "systemInfoType15-6" },
  {   6, "systemInfoType15-7" },
  {   7, "systemInfoType15-8" },
  { 0, NULL }
};

static const per_choice_t SIB_TypeExt_choice[] = {
  {   0, &hf_rrc_systemInfoType11bis, ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   1, &hf_rrc_systemInfoType15bis, ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   2, &hf_rrc_systemInfoType15_1bis, ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   3, &hf_rrc_systemInfoType15_2bis, ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   4, &hf_rrc_systemInfoType15_3bis, ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   5, &hf_rrc_systemInfoType15_6, ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   6, &hf_rrc_systemInfoType15_7, ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   7, &hf_rrc_systemInfoType15_8, ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_SIB_TypeExt(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_SIB_TypeExt, SIB_TypeExt_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t ExtSIBTypeInfoSchedulingInfo_sequence[] = {
  { &hf_rrc_extensionSIB_Type, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SIB_TypeExt },
  { &hf_rrc_schedulingInfo  , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SchedulingInformation },
  { &hf_rrc_valueTagInfo    , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_ValueTagInfo },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_ExtSIBTypeInfoSchedulingInfo(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_ExtSIBTypeInfoSchedulingInfo, ExtSIBTypeInfoSchedulingInfo_sequence);

  return offset;
}


static const per_sequence_t ExtSIBTypeInfoSchedulingInfo_List_sequence_of[1] = {
  { &hf_rrc_ExtSIBTypeInfoSchedulingInfo_List_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_ExtSIBTypeInfoSchedulingInfo },
};

static int
dissect_rrc_ExtSIBTypeInfoSchedulingInfo_List(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_ExtSIBTypeInfoSchedulingInfo_List, ExtSIBTypeInfoSchedulingInfo_List_sequence_of,
                                                  1, maxSIB, FALSE);

  return offset;
}


static const value_string rrc_SIB_TypeExt2_vals[] = {
  {   0, "systemInfoType19" },
  {   1, "systemInfoType15-2ter" },
  {   2, "systemInfoType20" },
  {   3, "systemInfoType21" },
  {   4, "systemInfoType22" },
  {   5, "spare3" },
  {   6, "spare2" },
  {   7, "spare1" },
  { 0, NULL }
};

static const per_choice_t SIB_TypeExt2_choice[] = {
  {   0, &hf_rrc_systemInfoType19, ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   1, &hf_rrc_systemInfoType15_2ter, ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   2, &hf_rrc_systemInfoType20, ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   3, &hf_rrc_systemInfoType21, ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   4, &hf_rrc_systemInfoType22, ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   5, &hf_rrc_spare3          , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   6, &hf_rrc_spare2          , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   7, &hf_rrc_spare1          , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_SIB_TypeExt2(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_SIB_TypeExt2, SIB_TypeExt2_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t ExtSIBTypeInfoSchedulingInfo2_sequence[] = {
  { &hf_rrc_extensionSIB_Type2, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SIB_TypeExt2 },
  { &hf_rrc_schedulingInfo  , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SchedulingInformation },
  { &hf_rrc_valueTagInfo    , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_ValueTagInfo },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_ExtSIBTypeInfoSchedulingInfo2(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_ExtSIBTypeInfoSchedulingInfo2, ExtSIBTypeInfoSchedulingInfo2_sequence);

  return offset;
}


static const per_sequence_t ExtSIBTypeInfoSchedulingInfo_List2_sequence_of[1] = {
  { &hf_rrc_ExtSIBTypeInfoSchedulingInfo_List2_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_ExtSIBTypeInfoSchedulingInfo2 },
};

static int
dissect_rrc_ExtSIBTypeInfoSchedulingInfo_List2(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_ExtSIBTypeInfoSchedulingInfo_List2, ExtSIBTypeInfoSchedulingInfo_List2_sequence_of,
                                                  1, maxSIB, FALSE);

  return offset;
}



static int
dissect_rrc_HNBName(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
#line 648 "../../asn1/rrc/rrc.cnf"
  tvbuff_t *hnbname_tvb = NULL;

  offset = dissect_per_octet_string(tvb, offset, actx, tree, -1,
                                       1, maxHNBNameSize, FALSE, &hnbname_tvb);

  actx->created_item = proto_tree_add_unicode_string(tree, hf_index, hnbname_tvb, 0, -1,
                                tvb_get_ephemeral_string_enc(hnbname_tvb, 0, tvb_length(hnbname_tvb), ENC_UTF_8 | ENC_NA));



  return offset;
}


static const value_string rrc_SIBSb_TypeAndTag_vals[] = {
  {   0, "sysInfoType1" },
  {   1, "sysInfoType2" },
  {   2, "sysInfoType3" },
  {   3, "sysInfoType4" },
  {   4, "sysInfoType5" },
  {   5, "sysInfoType6" },
  {   6, "sysInfoType7" },
  {   7, "dummy" },
  {   8, "dummy2" },
  {   9, "dummy3" },
  {  10, "sysInfoType11" },
  {  11, "sysInfoType12" },
  {  12, "sysInfoType13" },
  {  13, "sysInfoType13-1" },
  {  14, "sysInfoType13-2" },
  {  15, "sysInfoType13-3" },
  {  16, "sysInfoType13-4" },
  {  17, "sysInfoType14" },
  {  18, "sysInfoType15" },
  {  19, "sysInfoType16" },
  {  20, "sysInfoType17" },
  {  21, "sysInfoTypeSB1" },
  {  22, "sysInfoTypeSB2" },
  {  23, "sysInfoType15-1" },
  {  24, "sysInfoType15-2" },
  {  25, "sysInfoType15-3" },
  {  26, "sysInfoType15-4" },
  {  27, "sysInfoType18" },
  {  28, "sysInfoType15-5" },
  {  29, "sysInfoType5bis" },
  {  30, "spare2" },
  {  31, "spare1" },
  { 0, NULL }
};

static value_string_ext rrc_SIBSb_TypeAndTag_vals_ext = VALUE_STRING_EXT_INIT(rrc_SIBSb_TypeAndTag_vals);

static const per_choice_t SIBSb_TypeAndTag_choice[] = {
  {   0, &hf_rrc_sysInfoType1_01 , ASN1_NO_EXTENSIONS     , dissect_rrc_PLMN_ValueTag },
  {   1, &hf_rrc_sysInfoType2    , ASN1_NO_EXTENSIONS     , dissect_rrc_CellValueTag },
  {   2, &hf_rrc_sysInfoType3_01 , ASN1_NO_EXTENSIONS     , dissect_rrc_CellValueTag },
  {   3, &hf_rrc_sysInfoType4    , ASN1_NO_EXTENSIONS     , dissect_rrc_CellValueTag },
  {   4, &hf_rrc_sysInfoType5_01 , ASN1_NO_EXTENSIONS     , dissect_rrc_CellValueTag },
  {   5, &hf_rrc_sysInfoType6    , ASN1_NO_EXTENSIONS     , dissect_rrc_CellValueTag },
  {   6, &hf_rrc_sysInfoType7_01 , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   7, &hf_rrc_dummy_55        , ASN1_NO_EXTENSIONS     , dissect_rrc_CellValueTag },
  {   8, &hf_rrc_dummy2_14       , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   9, &hf_rrc_dummy3_03       , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {  10, &hf_rrc_sysInfoType11_01, ASN1_NO_EXTENSIONS     , dissect_rrc_CellValueTag },
  {  11, &hf_rrc_sysInfoType12_01, ASN1_NO_EXTENSIONS     , dissect_rrc_CellValueTag },
  {  12, &hf_rrc_sysInfoType13   , ASN1_NO_EXTENSIONS     , dissect_rrc_CellValueTag },
  {  13, &hf_rrc_sysInfoType13_1 , ASN1_NO_EXTENSIONS     , dissect_rrc_CellValueTag },
  {  14, &hf_rrc_sysInfoType13_2 , ASN1_NO_EXTENSIONS     , dissect_rrc_CellValueTag },
  {  15, &hf_rrc_sysInfoType13_3 , ASN1_NO_EXTENSIONS     , dissect_rrc_CellValueTag },
  {  16, &hf_rrc_sysInfoType13_4 , ASN1_NO_EXTENSIONS     , dissect_rrc_CellValueTag },
  {  17, &hf_rrc_sysInfoType14   , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {  18, &hf_rrc_sysInfoType15   , ASN1_NO_EXTENSIONS     , dissect_rrc_CellValueTag },
  {  19, &hf_rrc_sysInfoType16   , ASN1_NO_EXTENSIONS     , dissect_rrc_PredefinedConfigIdentityAndValueTag },
  {  20, &hf_rrc_sysInfoType17   , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {  21, &hf_rrc_sysInfoTypeSB1_01, ASN1_NO_EXTENSIONS     , dissect_rrc_CellValueTag },
  {  22, &hf_rrc_sysInfoTypeSB2_01, ASN1_NO_EXTENSIONS     , dissect_rrc_CellValueTag },
  {  23, &hf_rrc_sysInfoType15_1 , ASN1_NO_EXTENSIONS     , dissect_rrc_CellValueTag },
  {  24, &hf_rrc_sysInfoType15_2 , ASN1_NO_EXTENSIONS     , dissect_rrc_SIBOccurrenceIdentityAndValueTag },
  {  25, &hf_rrc_sysInfoType15_3 , ASN1_NO_EXTENSIONS     , dissect_rrc_SIBOccurrenceIdentityAndValueTag },
  {  26, &hf_rrc_sysInfoType15_4 , ASN1_NO_EXTENSIONS     , dissect_rrc_CellValueTag },
  {  27, &hf_rrc_sysInfoType18   , ASN1_NO_EXTENSIONS     , dissect_rrc_CellValueTag },
  {  28, &hf_rrc_sysInfoType15_5 , ASN1_NO_EXTENSIONS     , dissect_rrc_CellValueTag },
  {  29, &hf_rrc_sysInfoType5bis , ASN1_NO_EXTENSIONS     , dissect_rrc_CellValueTag },
  {  30, &hf_rrc_spare2          , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {  31, &hf_rrc_spare1          , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_SIBSb_TypeAndTag(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_SIBSb_TypeAndTag, SIBSb_TypeAndTag_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t SchedulingInformationSIBSb_sequence[] = {
  { &hf_rrc_sibSb_Type      , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SIBSb_TypeAndTag },
  { &hf_rrc_scheduling_01   , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SchedulingInformation },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_SchedulingInformationSIBSb(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_SchedulingInformationSIBSb, SchedulingInformationSIBSb_sequence);

  return offset;
}


static const per_sequence_t SIBSb_ReferenceList_sequence_of[1] = {
  { &hf_rrc_SIBSb_ReferenceList_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SchedulingInformationSIBSb },
};

static int
dissect_rrc_SIBSb_ReferenceList(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_SIBSb_ReferenceList, SIBSb_ReferenceList_sequence_of,
                                                  1, maxSIB, FALSE);

  return offset;
}


static const per_sequence_t MasterInformationBlock_v690ext_sequence[] = {
  { &hf_rrc_multiplePLMN_List, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MultiplePLMN_List_r6 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_MasterInformationBlock_v690ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_MasterInformationBlock_v690ext, MasterInformationBlock_v690ext_sequence);

  return offset;
}


static const per_sequence_t MasterInformationBlock_v6b0ext_IEs_sequence[] = {
  { &hf_rrc_extSIBTypeInfoSchedulingInfo_List, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_ExtSIBTypeInfoSchedulingInfo_List },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_MasterInformationBlock_v6b0ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_MasterInformationBlock_v6b0ext_IEs, MasterInformationBlock_v6b0ext_IEs_sequence);

  return offset;
}


static const value_string rrc_T_csg_Indicator_vals[] = {
  {   0, "true" },
  { 0, NULL }
};


static int
dissect_rrc_T_csg_Indicator(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t MasterInformationBlock_v860ext_IEs_sequence[] = {
  { &hf_rrc_extSIBTypeInfoSchedulingInfo_List_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_ExtSIBTypeInfoSchedulingInfo_List2 },
  { &hf_rrc_extGANSS_SIBTypeInfoSchedulingInfoList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_ExtGANSS_SIBTypeInfoSchedulingInfoList },
  { &hf_rrc_csg_Indicator   , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_csg_Indicator },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_MasterInformationBlock_v860ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_MasterInformationBlock_v860ext_IEs, MasterInformationBlock_v860ext_IEs_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions_195_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_nonCriticalExtensions_195(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_nonCriticalExtensions_195, T_nonCriticalExtensions_195_sequence);

  return offset;
}


static const per_sequence_t T_v860NonCriticalExtensions_26_sequence[] = {
  { &hf_rrc_masterInformationBlock_v860ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MasterInformationBlock_v860ext_IEs },
  { &hf_rrc_nonCriticalExtensions_195, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_nonCriticalExtensions_195 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v860NonCriticalExtensions_26(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v860NonCriticalExtensions_26, T_v860NonCriticalExtensions_26_sequence);

  return offset;
}


static const per_sequence_t T_v6b0NonCriticalExtensions_15_sequence[] = {
  { &hf_rrc_masterInformationBlock_v6b0ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MasterInformationBlock_v6b0ext_IEs },
  { &hf_rrc_v860NonCriticalExtensions_26, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v860NonCriticalExtensions_26 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v6b0NonCriticalExtensions_15(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v6b0NonCriticalExtensions_15, T_v6b0NonCriticalExtensions_15_sequence);

  return offset;
}


static const per_sequence_t T_v690NonCriticalExtensions_48_sequence[] = {
  { &hf_rrc_masterInformationBlock_v690ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MasterInformationBlock_v690ext },
  { &hf_rrc_v6b0NonCriticalExtensions_16, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v6b0NonCriticalExtensions_15 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v690NonCriticalExtensions_48(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v690NonCriticalExtensions_48, T_v690NonCriticalExtensions_48_sequence);

  return offset;
}


static const per_sequence_t MasterInformationBlock_sequence[] = {
  { &hf_rrc_mib_ValueTag    , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MIB_ValueTag },
  { &hf_rrc_plmn_Type       , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PLMN_Type },
  { &hf_rrc_sibSb_ReferenceList, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SIBSb_ReferenceList },
  { &hf_rrc_v690NonCriticalExtensions_48, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v690NonCriticalExtensions_48 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_MasterInformationBlock(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_MasterInformationBlock, MasterInformationBlock_sequence);

  return offset;
}


static const per_sequence_t SIB_ReferenceList_sequence_of[1] = {
  { &hf_rrc_SIB_ReferenceList_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SchedulingInformationSIB },
};

static int
dissect_rrc_SIB_ReferenceList(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_SIB_ReferenceList, SIB_ReferenceList_sequence_of,
                                                  1, maxSIB, FALSE);

  return offset;
}



static int
dissect_rrc_T_cn_CommonGSM_MAP_NAS_SysInfo_03(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
#line 670 "../../asn1/rrc/rrc.cnf"
  rrc_nas_sys_info_gsm_map_type = RRC_NAS_SYS_INFO_CN_COMMON;
  offset = dissect_rrc_NAS_SystemInformationGSM_MAP(tvb, offset, actx, tree, hf_index);




  return offset;
}


static const per_sequence_t SysInfoType1_v3a0ext_IEs_sequence[] = {
  { &hf_rrc_ue_ConnTimersAndConstants_v3a0ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UE_ConnTimersAndConstants_v3a0ext },
  { &hf_rrc_ue_IdleTimersAndConstants_v3a0ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UE_IdleTimersAndConstants_v3a0ext },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_SysInfoType1_v3a0ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_SysInfoType1_v3a0ext_IEs, SysInfoType1_v3a0ext_IEs_sequence);

  return offset;
}


static const per_sequence_t SysInfoType1_v860ext_IEs_sequence[] = {
  { &hf_rrc_ue_ConnTimersAndConstants_02, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UE_ConnTimersAndConstants_v860ext },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_SysInfoType1_v860ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_SysInfoType1_v860ext_IEs, SysInfoType1_v860ext_IEs_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions_196_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_nonCriticalExtensions_196(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_nonCriticalExtensions_196, T_nonCriticalExtensions_196_sequence);

  return offset;
}


static const per_sequence_t T_v860NonCriticalExtentions_02_sequence[] = {
  { &hf_rrc_sysInfoType1_v860ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SysInfoType1_v860ext_IEs },
  { &hf_rrc_nonCriticalExtensions_196, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_nonCriticalExtensions_196 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v860NonCriticalExtentions_02(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v860NonCriticalExtentions_02, T_v860NonCriticalExtentions_02_sequence);

  return offset;
}


static const per_sequence_t T_v3a0NonCriticalExtensions_12_sequence[] = {
  { &hf_rrc_sysInfoType1_v3a0ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SysInfoType1_v3a0ext_IEs },
  { &hf_rrc_v860NonCriticalExtentions_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v860NonCriticalExtentions_02 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v3a0NonCriticalExtensions_12(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v3a0NonCriticalExtensions_12, T_v3a0NonCriticalExtensions_12_sequence);

  return offset;
}


static const per_sequence_t SysInfoType1_sequence[] = {
  { &hf_rrc_cn_CommonGSM_MAP_NAS_SysInfo_03, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_cn_CommonGSM_MAP_NAS_SysInfo_03 },
  { &hf_rrc_cn_DomainSysInfoList, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CN_DomainSysInfoList },
  { &hf_rrc_ue_ConnTimersAndConstants, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_ConnTimersAndConstants },
  { &hf_rrc_ue_IdleTimersAndConstants, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_IdleTimersAndConstants },
  { &hf_rrc_v3a0NonCriticalExtensions_12, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v3a0NonCriticalExtensions_12 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_SysInfoType1(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_SysInfoType1, SysInfoType1_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions_197_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_nonCriticalExtensions_197(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_nonCriticalExtensions_197, T_nonCriticalExtensions_197_sequence);

  return offset;
}


static const per_sequence_t SysInfoType2_sequence[] = {
  { &hf_rrc_ura_IdentityList, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_URA_IdentityList },
  { &hf_rrc_nonCriticalExtensions_197, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_nonCriticalExtensions_197 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_SysInfoType2(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_SysInfoType2, SysInfoType2_sequence);

  return offset;
}


static const per_sequence_t SysInfoType3_v4b0ext_IEs_sequence[] = {
  { &hf_rrc_mapping_LCR     , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_Mapping_LCR_r4 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_SysInfoType3_v4b0ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_SysInfoType3_v4b0ext_IEs, SysInfoType3_v4b0ext_IEs_sequence);

  return offset;
}


static const per_sequence_t SysInfoType3_v590ext_sequence[] = {
  { &hf_rrc_cellSelectReselectInfo_v590ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CellSelectReselectInfo_v590ext },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_SysInfoType3_v590ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_SysInfoType3_v590ext, SysInfoType3_v590ext_sequence);

  return offset;
}


static const per_sequence_t SysInfoType3_v5c0ext_IEs_sequence[] = {
  { &hf_rrc_cellSelectReselectInfoTreselectionScaling_v5c0ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CellSelectReselectInfoTreselectionScaling_v5c0ext },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_SysInfoType3_v5c0ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_SysInfoType3_v5c0ext_IEs, SysInfoType3_v5c0ext_IEs_sequence);

  return offset;
}


static const per_sequence_t SysInfoType3_v670ext_sequence[] = {
  { &hf_rrc_domainSpecificAccessRestrictionParametersForPLMNOfMIB, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DomainSpecificAccessRestrictionParam_v670ext },
  { &hf_rrc_domainSpecificAccessRestictionForSharedNetwork, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DomainSpecificAccessRestrictionForSharedNetwork_v670ext },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_SysInfoType3_v670ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_SysInfoType3_v670ext, SysInfoType3_v670ext_sequence);

  return offset;
}


static const value_string rrc_MBSFNOnlyService_vals[] = {
  {   0, "true" },
  { 0, NULL }
};


static int
dissect_rrc_MBSFNOnlyService(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t SysInfoType3_v770ext_IEs_sequence[] = {
  { &hf_rrc_deferredMeasurementControlReadingSupport, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DeferredMeasurementControlReadingSupport },
  { &hf_rrc_q_QualMin_Offset, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_Q_QualMin_Offset },
  { &hf_rrc_q_RxlevMin_Offset, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_Q_RxlevMin_Offset },
  { &hf_rrc_mbsfnOnlyService, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MBSFNOnlyService },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_SysInfoType3_v770ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_SysInfoType3_v770ext_IEs, SysInfoType3_v770ext_IEs_sequence);

  return offset;
}


static const per_sequence_t SysInfoType3_v830ext_IEs_sequence[] = {
  { &hf_rrc_pagingPermissionWithAccessControlParametersForPLMNOfMIB, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_PagingPermissionWithAccessControlParameters },
  { &hf_rrc_pagingPermissionWithAccessControlParametersForSharedNetwork, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_PagingPermissionWithAccessControlForSharedNetwork },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_SysInfoType3_v830ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_SysInfoType3_v830ext_IEs, SysInfoType3_v830ext_IEs_sequence);

  return offset;
}


static const per_sequence_t SysInfoType3_v860ext_IEs_sequence[] = {
  { &hf_rrc_csgIdentity     , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CSG_Identity },
  { &hf_rrc_csg_PSCSplitInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CSG_PSCSplitInfo },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_SysInfoType3_v860ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_SysInfoType3_v860ext_IEs, SysInfoType3_v860ext_IEs_sequence);

  return offset;
}


static const per_sequence_t SysInfoType3_v870ext_IEs_sequence[] = {
  { &hf_rrc_cellAccessRestriction_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CellAccessRestriction_v870ext },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_SysInfoType3_v870ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_SysInfoType3_v870ext_IEs, SysInfoType3_v870ext_IEs_sequence);

  return offset;
}


static const value_string rrc_T_imsEmergencySupportIndicator_vals[] = {
  {   0, "supported" },
  { 0, NULL }
};


static int
dissect_rrc_T_imsEmergencySupportIndicator(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t SysInfoType3_v920ext_IEs_sequence[] = {
  { &hf_rrc_imsEmergencySupportIndicator, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_imsEmergencySupportIndicator },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_SysInfoType3_v920ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_SysInfoType3_v920ext_IEs, SysInfoType3_v920ext_IEs_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions_198_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_nonCriticalExtensions_198(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_nonCriticalExtensions_198, T_nonCriticalExtensions_198_sequence);

  return offset;
}


static const per_sequence_t T_v900NonCriticalExtension_sequence[] = {
  { &hf_rrc_sysInfoType3_v920ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SysInfoType3_v920ext_IEs },
  { &hf_rrc_nonCriticalExtensions_198, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_nonCriticalExtensions_198 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v900NonCriticalExtension(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v900NonCriticalExtension, T_v900NonCriticalExtension_sequence);

  return offset;
}


static const per_sequence_t T_v870NonCriticalExtension_sequence[] = {
  { &hf_rrc_sysInfoType3_v870ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SysInfoType3_v870ext_IEs },
  { &hf_rrc_v900NonCriticalExtension, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v900NonCriticalExtension },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v870NonCriticalExtension(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v870NonCriticalExtension, T_v870NonCriticalExtension_sequence);

  return offset;
}


static const per_sequence_t T_v860NonCriticalExtension_01_sequence[] = {
  { &hf_rrc_sysInfoType3_v860ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SysInfoType3_v860ext_IEs },
  { &hf_rrc_v870NonCriticalExtension, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v870NonCriticalExtension },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v860NonCriticalExtension_01(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v860NonCriticalExtension_01, T_v860NonCriticalExtension_01_sequence);

  return offset;
}


static const per_sequence_t T_v830NonCriticalExtension_sequence[] = {
  { &hf_rrc_sysInfoType3_v830ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SysInfoType3_v830ext_IEs },
  { &hf_rrc_v860NonCriticalExtension_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v860NonCriticalExtension_01 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v830NonCriticalExtension(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v830NonCriticalExtension, T_v830NonCriticalExtension_sequence);

  return offset;
}


static const per_sequence_t T_v770NonCriticalExtension_02_sequence[] = {
  { &hf_rrc_sysInfoType3_v770ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SysInfoType3_v770ext_IEs },
  { &hf_rrc_v830NonCriticalExtension, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v830NonCriticalExtension },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v770NonCriticalExtension_02(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v770NonCriticalExtension_02, T_v770NonCriticalExtension_02_sequence);

  return offset;
}


static const per_sequence_t T_v670NonCriticalExtension_sequence[] = {
  { &hf_rrc_sysInfoType3_v670ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SysInfoType3_v670ext },
  { &hf_rrc_v770NonCriticalExtension_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v770NonCriticalExtension_02 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v670NonCriticalExtension(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v670NonCriticalExtension, T_v670NonCriticalExtension_sequence);

  return offset;
}


static const per_sequence_t T_v5c0NoncriticalExtension_sequence[] = {
  { &hf_rrc_sysInfoType3_v5c0ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SysInfoType3_v5c0ext_IEs },
  { &hf_rrc_v670NonCriticalExtension, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v670NonCriticalExtension },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v5c0NoncriticalExtension(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v5c0NoncriticalExtension, T_v5c0NoncriticalExtension_sequence);

  return offset;
}


static const per_sequence_t T_v590NonCriticalExtension_sequence[] = {
  { &hf_rrc_sysInfoType3_v590ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SysInfoType3_v590ext },
  { &hf_rrc_v5c0NoncriticalExtension, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v5c0NoncriticalExtension },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v590NonCriticalExtension(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v590NonCriticalExtension, T_v590NonCriticalExtension_sequence);

  return offset;
}


static const per_sequence_t T_v4b0NonCriticalExtensions_17_sequence[] = {
  { &hf_rrc_sysInfoType3_v4b0ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SysInfoType3_v4b0ext_IEs },
  { &hf_rrc_v590NonCriticalExtension, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v590NonCriticalExtension },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v4b0NonCriticalExtensions_17(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v4b0NonCriticalExtensions_17, T_v4b0NonCriticalExtensions_17_sequence);

  return offset;
}


static const per_sequence_t SysInfoType3_sequence[] = {
  { &hf_rrc_sib4indicator   , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { &hf_rrc_cellIdentity    , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CellIdentity },
  { &hf_rrc_cellSelectReselectInfo, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CellSelectReselectInfoSIB_3_4 },
  { &hf_rrc_cellAccessRestriction, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CellAccessRestriction },
  { &hf_rrc_v4b0NonCriticalExtensions_17, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v4b0NonCriticalExtensions_17 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_SysInfoType3(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_SysInfoType3, SysInfoType3_sequence);

  return offset;
}


static const per_sequence_t SysInfoType4_v4b0ext_IEs_sequence[] = {
  { &hf_rrc_mapping_LCR     , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_Mapping_LCR_r4 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_SysInfoType4_v4b0ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_SysInfoType4_v4b0ext_IEs, SysInfoType4_v4b0ext_IEs_sequence);

  return offset;
}


static const per_sequence_t SysInfoType4_v590ext_sequence[] = {
  { &hf_rrc_cellSelectReselectInfo_v590ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CellSelectReselectInfo_v590ext },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_SysInfoType4_v590ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_SysInfoType4_v590ext, SysInfoType4_v590ext_sequence);

  return offset;
}


static const per_sequence_t SysInfoType4_v5b0ext_IEs_sequence[] = {
  { &hf_rrc_cellSelectReselectInfoPCHFACH_v5b0ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CellSelectReselectInfoPCHFACH_v5b0ext },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_SysInfoType4_v5b0ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_SysInfoType4_v5b0ext_IEs, SysInfoType4_v5b0ext_IEs_sequence);

  return offset;
}


static const per_sequence_t SysInfoType4_v5c0ext_IEs_sequence[] = {
  { &hf_rrc_cellSelectReselectInfoTreselectionScaling_v5c0ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CellSelectReselectInfoTreselectionScaling_v5c0ext },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_SysInfoType4_v5c0ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_SysInfoType4_v5c0ext_IEs, SysInfoType4_v5c0ext_IEs_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions_199_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_nonCriticalExtensions_199(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_nonCriticalExtensions_199, T_nonCriticalExtensions_199_sequence);

  return offset;
}


static const per_sequence_t T_v5c0NonCriticalExtension_sequence[] = {
  { &hf_rrc_sysInfoType4_v5c0ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SysInfoType4_v5c0ext_IEs },
  { &hf_rrc_nonCriticalExtensions_199, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_nonCriticalExtensions_199 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v5c0NonCriticalExtension(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v5c0NonCriticalExtension, T_v5c0NonCriticalExtension_sequence);

  return offset;
}


static const per_sequence_t T_v5b0NonCriticalExtension_sequence[] = {
  { &hf_rrc_sysInfoType4_v5b0ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SysInfoType4_v5b0ext_IEs },
  { &hf_rrc_v5c0NonCriticalExtension, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v5c0NonCriticalExtension },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v5b0NonCriticalExtension(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v5b0NonCriticalExtension, T_v5b0NonCriticalExtension_sequence);

  return offset;
}


static const per_sequence_t T_v590NonCriticalExtension_01_sequence[] = {
  { &hf_rrc_sysInfoType4_v590ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SysInfoType4_v590ext },
  { &hf_rrc_v5b0NonCriticalExtension, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v5b0NonCriticalExtension },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v590NonCriticalExtension_01(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v590NonCriticalExtension_01, T_v590NonCriticalExtension_01_sequence);

  return offset;
}


static const per_sequence_t T_v4b0NonCriticalExtensions_18_sequence[] = {
  { &hf_rrc_sysInfoType4_v4b0ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SysInfoType4_v4b0ext_IEs },
  { &hf_rrc_v590NonCriticalExtension_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v590NonCriticalExtension_01 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v4b0NonCriticalExtensions_18(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v4b0NonCriticalExtensions_18, T_v4b0NonCriticalExtensions_18_sequence);

  return offset;
}


static const per_sequence_t SysInfoType4_sequence[] = {
  { &hf_rrc_cellIdentity    , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CellIdentity },
  { &hf_rrc_cellSelectReselectInfo, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CellSelectReselectInfoSIB_3_4 },
  { &hf_rrc_cellAccessRestriction, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CellAccessRestriction },
  { &hf_rrc_v4b0NonCriticalExtensions_18, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v4b0NonCriticalExtensions_18 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_SysInfoType4(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_SysInfoType4, SysInfoType4_sequence);

  return offset;
}


static const per_sequence_t T_fdd_185_sequence[] = {
  { &hf_rrc_aich_PowerOffset, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_AICH_PowerOffset },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_fdd_185(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_fdd_185, T_fdd_185_sequence);

  return offset;
}


static const per_sequence_t T_tdd_134_sequence[] = {
  { &hf_rrc_pusch_SysInfoList_SFN, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_PUSCH_SysInfoList_SFN },
  { &hf_rrc_pdsch_SysInfoList_SFN, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_PDSCH_SysInfoList_SFN },
  { &hf_rrc_openLoopPowerControl_TDD, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_OpenLoopPowerControl_TDD },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_tdd_134(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_tdd_134, T_tdd_134_sequence);

  return offset;
}


static const value_string rrc_T_modeSpecificInfo_150_vals[] = {
  {   0, "fdd" },
  {   1, "tdd" },
  { 0, NULL }
};

static const per_choice_t T_modeSpecificInfo_150_choice[] = {
  {   0, &hf_rrc_fdd_191         , ASN1_NO_EXTENSIONS     , dissect_rrc_T_fdd_185 },
  {   1, &hf_rrc_tdd_142         , ASN1_NO_EXTENSIONS     , dissect_rrc_T_tdd_134 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_modeSpecificInfo_150(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_modeSpecificInfo_150, T_modeSpecificInfo_150_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t T_tdd128SpecificInfo_sequence[] = {
  { &hf_rrc_pusch_SysInfoList_SFN_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_PUSCH_SysInfoList_SFN_LCR_r4 },
  { &hf_rrc_pdsch_SysInfoList_SFN_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_PDSCH_SysInfoList_SFN_LCR_r4 },
  { &hf_rrc_pCCPCH_LCR_Extensions, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_PrimaryCCPCH_Info_LCR_r4_ext },
  { &hf_rrc_sCCPCH_LCR_ExtensionsList, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SCCPCH_SystemInformationList_LCR_r4_ext },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_tdd128SpecificInfo(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_tdd128SpecificInfo, T_tdd128SpecificInfo_sequence);

  return offset;
}


static const per_sequence_t SysInfoType5_v4b0ext_IEs_sequence[] = {
  { &hf_rrc_pNBSCH_Allocation_r4, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_PNBSCH_Allocation_r4 },
  { &hf_rrc_openLoopPowerControl_IPDL_TDD, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_OpenLoopPowerControl_IPDL_TDD_r4 },
  { &hf_rrc_prach_SystemInformationList_LCR_r4, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_PRACH_SystemInformationList_LCR_r4 },
  { &hf_rrc_tdd128SpecificInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_tdd128SpecificInfo },
  { &hf_rrc_frequencyBandIndicator, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RadioFrequencyBandFDD },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_SysInfoType5_v4b0ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_SysInfoType5_v4b0ext_IEs, SysInfoType5_v4b0ext_IEs_sequence);

  return offset;
}


static const per_sequence_t T_hcr_r5_SpecificInfo_sequence[] = {
  { &hf_rrc_pusch_SysInfoList_SFN_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_PUSCH_SysInfoList_SFN_HCR_r5 },
  { &hf_rrc_pdsch_SysInfoList_SFN_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_PDSCH_SysInfoList_SFN_HCR_r5 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_hcr_r5_SpecificInfo(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_hcr_r5_SpecificInfo, T_hcr_r5_SpecificInfo_sequence);

  return offset;
}


static const per_sequence_t SysInfoType5_v590ext_IEs_sequence[] = {
  { &hf_rrc_hcr_r5_SpecificInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_hcr_r5_SpecificInfo },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_SysInfoType5_v590ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_SysInfoType5_v590ext_IEs, SysInfoType5_v590ext_IEs_sequence);

  return offset;
}


static const per_sequence_t SysInfoType5_v650ext_IEs_sequence[] = {
  { &hf_rrc_frequencyBandIndicator2, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RadioFrequencyBandFDD2 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_SysInfoType5_v650ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_SysInfoType5_v650ext_IEs, SysInfoType5_v650ext_IEs_sequence);

  return offset;
}


static const value_string rrc_T_hsdpa_CellIndicator_vals[] = {
  {   0, "hsdpa-CapableCell" },
  { 0, NULL }
};


static int
dissect_rrc_T_hsdpa_CellIndicator(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t SysInfoType5_v680ext_IEs_sequence[] = {
  { &hf_rrc_hsdpa_CellIndicator, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_hsdpa_CellIndicator },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_SysInfoType5_v680ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_SysInfoType5_v680ext_IEs, SysInfoType5_v680ext_IEs_sequence);

  return offset;
}


static const value_string rrc_T_edch_CellIndicator_vals[] = {
  {   0, "edch-CapableCell" },
  { 0, NULL }
};


static int
dissect_rrc_T_edch_CellIndicator(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}


static const value_string rrc_T_sccpch_SystemInformation_MBMS_vals[] = {
  {   0, "mcchOnSCCPCHusedForNonMBMS" },
  {   1, "mcchOnSCCPCHusedOnlyForMBMS" },
  { 0, NULL }
};

static const per_choice_t T_sccpch_SystemInformation_MBMS_choice[] = {
  {   0, &hf_rrc_mcchOnSCCPCHusedForNonMBMS, ASN1_NO_EXTENSIONS     , dissect_rrc_MBMS_MCCH_ConfigurationInfo_r6 },
  {   1, &hf_rrc_mcchOnSCCPCHusedOnlyForMBMS, ASN1_NO_EXTENSIONS     , dissect_rrc_SCCPCH_SystemInformation_MBMS_r6 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_sccpch_SystemInformation_MBMS(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_sccpch_SystemInformation_MBMS, T_sccpch_SystemInformation_MBMS_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t SysInfoType5_v690ext_IEs_sequence[] = {
  { &hf_rrc_edch_CellIndicator, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_edch_CellIndicator },
  { &hf_rrc_sccpch_SystemInformation_MBMS, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_sccpch_SystemInformation_MBMS },
  { &hf_rrc_additionalPRACH_TF_and_TFCS_CCCH_List, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_AdditionalPRACH_TF_and_TFCS_CCCH_List },
  { &hf_rrc_cBS_DRX_Level1Information_extension, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CBS_DRX_Level1Information_extension_r6 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_SysInfoType5_v690ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_SysInfoType5_v690ext_IEs, SysInfoType5_v690ext_IEs_sequence);

  return offset;
}


static const per_sequence_t T_fdd_186_sequence[] = {
  { &hf_rrc_hs_dsch_CommonSystemInformation, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_HS_DSCH_CommonSystemInformation },
  { &hf_rrc_hs_dsch_PagingSystemInformation, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_HS_DSCH_PagingSystemInformation },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_fdd_186(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_fdd_186, T_fdd_186_sequence);

  return offset;
}


static const per_sequence_t T_tdd768_24_sequence[] = {
  { &hf_rrc_pusch_SysInfoList_SFN_03, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_PUSCH_SysInfoList_SFN_VHCR },
  { &hf_rrc_pdsch_SysInfoList_SFN_03, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_PDSCH_SysInfoList_VHCR_r7 },
  { &hf_rrc_prach_SystemInformationList_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_PRACH_SystemInformationList_VHCR_r7 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_tdd768_24(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_tdd768_24, T_tdd768_24_sequence);

  return offset;
}


static const per_sequence_t T_tdd128_68_sequence[] = {
  { &hf_rrc_prach_SystemInformationList_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_PRACH_SystemInformationList_LCR_v770ext },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_tdd128_68(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_tdd128_68, T_tdd128_68_sequence);

  return offset;
}


static const value_string rrc_T_modeSpecificInfo_151_vals[] = {
  {   0, "fdd" },
  {   1, "tdd768" },
  {   2, "tdd128" },
  { 0, NULL }
};

static const per_choice_t T_modeSpecificInfo_151_choice[] = {
  {   0, &hf_rrc_fdd_192         , ASN1_NO_EXTENSIONS     , dissect_rrc_T_fdd_186 },
  {   1, &hf_rrc_tdd768_27       , ASN1_NO_EXTENSIONS     , dissect_rrc_T_tdd768_24 },
  {   2, &hf_rrc_tdd128_71       , ASN1_NO_EXTENSIONS     , dissect_rrc_T_tdd128_68 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_modeSpecificInfo_151(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_modeSpecificInfo_151, T_modeSpecificInfo_151_choice,
                                 NULL);

  return offset;
}


static const value_string rrc_T_sccpch_SystemInformation_MBMS_01_vals[] = {
  {   0, "mcchOnSCCPCHusedOnlyForMBMS" },
  { 0, NULL }
};

static const per_choice_t T_sccpch_SystemInformation_MBMS_01_choice[] = {
  {   0, &hf_rrc_mcchOnSCCPCHusedOnlyForMBMS_01, ASN1_NO_EXTENSIONS     , dissect_rrc_SCCPCH_SystemInformation_MBMS_r7 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_sccpch_SystemInformation_MBMS_01(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_sccpch_SystemInformation_MBMS_01, T_sccpch_SystemInformation_MBMS_01_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t SysInfoType5_v770ext_IEs_sequence[] = {
  { &hf_rrc_modeSpecificInfo_151, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_modeSpecificInfo_151 },
  { &hf_rrc_sccpch_SystemInformationList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_SCCPCH_SystemInformationList_HCR_VHCR_r7 },
  { &hf_rrc_sccpch_SystemInformation_MBMS_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_sccpch_SystemInformation_MBMS_01 },
  { &hf_rrc_tDD_MBSFNInformation, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_TDD_MBSFNInformation },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_SysInfoType5_v770ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_SysInfoType5_v770ext_IEs, SysInfoType5_v770ext_IEs_sequence);

  return offset;
}


static const per_sequence_t T_dummy_15_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_dummy_15(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_dummy_15, T_dummy_15_sequence);

  return offset;
}


static const per_sequence_t T_fdd_187_sequence[] = {
  { &hf_rrc_hs_dsch_DrxCellfach_info, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_HS_DSCH_DrxCellfach_info },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_fdd_187(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_fdd_187, T_fdd_187_sequence);

  return offset;
}


static const per_sequence_t T_hs_dsch_CommonSysInfo_sequence[] = {
  { &hf_rrc_hs_dsch_CommonSystemInformation_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_HS_DSCH_CommonSystemInformation_TDD128 },
  { &hf_rrc_hs_dsch_PagingSystemInformation_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_HS_DSCH_PagingSystemInformation_TDD128 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_hs_dsch_CommonSysInfo(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_hs_dsch_CommonSysInfo, T_hs_dsch_CommonSysInfo_sequence);

  return offset;
}


static const per_sequence_t T_tdd128_69_sequence[] = {
  { &hf_rrc_hs_dsch_CommonSysInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_hs_dsch_CommonSysInfo },
  { &hf_rrc_hs_dsch_DrxCellfach_info_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_HS_DSCH_DrxCellfach_info_TDD128 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_tdd128_69(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_tdd128_69, T_tdd128_69_sequence);

  return offset;
}


static const value_string rrc_T_modeSpecificInfo_152_vals[] = {
  {   0, "fdd" },
  {   1, "tdd128" },
  { 0, NULL }
};

static const per_choice_t T_modeSpecificInfo_152_choice[] = {
  {   0, &hf_rrc_fdd_193         , ASN1_NO_EXTENSIONS     , dissect_rrc_T_fdd_187 },
  {   1, &hf_rrc_tdd128_72       , ASN1_NO_EXTENSIONS     , dissect_rrc_T_tdd128_69 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_modeSpecificInfo_152(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_modeSpecificInfo_152, T_modeSpecificInfo_152_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t SysInfoType5_v860ext_IEs_sequence[] = {
  { &hf_rrc_dummy_56        , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_dummy_15 },
  { &hf_rrc_modeSpecificInfo_152, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_modeSpecificInfo_152 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_SysInfoType5_v860ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_SysInfoType5_v860ext_IEs, SysInfoType5_v860ext_IEs_sequence);

  return offset;
}


static const per_sequence_t SysInfoType5_v890ext_IEs_sequence[] = {
  { &hf_rrc_secondFrequencyInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_FrequencyInfoTDD },
  { &hf_rrc_commonEDCHSystemInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CommonEDCHSystemInfo },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_SysInfoType5_v890ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_SysInfoType5_v890ext_IEs, SysInfoType5_v890ext_IEs_sequence);

  return offset;
}


static const value_string rrc_T_tresetUsageIndicator_vals[] = {
  {   0, "true" },
  { 0, NULL }
};


static int
dissect_rrc_T_tresetUsageIndicator(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t SysInfoType5_v8b0ext_IEs_sequence[] = {
  { &hf_rrc_tresetUsageIndicator, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_tresetUsageIndicator },
  { &hf_rrc_upPCHpositionInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UpPCHposition_LCR },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_SysInfoType5_v8b0ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_SysInfoType5_v8b0ext_IEs, SysInfoType5_v8b0ext_IEs_sequence);

  return offset;
}


static const per_sequence_t SysInfoType5_v8d0ext_IEs_sequence[] = {
  { &hf_rrc_commonEDCHSystemInfoFDD, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CommonEDCHSystemInfoFDD },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_SysInfoType5_v8d0ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_SysInfoType5_v8d0ext_IEs, SysInfoType5_v8d0ext_IEs_sequence);

  return offset;
}


static const per_sequence_t SysInfoType5_va40ext_IEs_sequence[] = {
  { &hf_rrc_frequencyBandsIndicatorSupport, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_FrequencyBandsIndicatorSupport },
  { &hf_rrc_frequencyBandIndicator3, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RadioFrequencyBandFDD3 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_SysInfoType5_va40ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_SysInfoType5_va40ext_IEs, SysInfoType5_va40ext_IEs_sequence);

  return offset;
}


static const per_sequence_t SysInfoType5_va80ext_IEs_sequence[] = {
  { &hf_rrc_multipleFrequencyBandIndicatorListFDD, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MultipleFrequencyBandIndicatorListFDD },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_SysInfoType5_va80ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_SysInfoType5_va80ext_IEs, SysInfoType5_va80ext_IEs_sequence);

  return offset;
}


static const per_sequence_t SysInfoType5_vbxyext_IEs_sequence[] = {
  { &hf_rrc_sib22indicator  , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_SysInfoType5_vbxyext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_SysInfoType5_vbxyext_IEs, SysInfoType5_vbxyext_IEs_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions_200_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_nonCriticalExtensions_200(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_nonCriticalExtensions_200, T_nonCriticalExtensions_200_sequence);

  return offset;
}


static const per_sequence_t SysInfoType5_NonCriticalExtensions_vbxy_IEs_sequence[] = {
  { &hf_rrc_sysInfoType5_vbxyext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SysInfoType5_vbxyext_IEs },
  { &hf_rrc_nonCriticalExtensions_200, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_nonCriticalExtensions_200 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_SysInfoType5_NonCriticalExtensions_vbxy_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_SysInfoType5_NonCriticalExtensions_vbxy_IEs, SysInfoType5_NonCriticalExtensions_vbxy_IEs_sequence);

  return offset;
}


static const per_sequence_t T_va80NonCriticalExtensions_02_sequence[] = {
  { &hf_rrc_sysInfoType5_va80ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SysInfoType5_va80ext_IEs },
  { &hf_rrc_vbxyNonCriticalExtensions_19, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_SysInfoType5_NonCriticalExtensions_vbxy_IEs },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_va80NonCriticalExtensions_02(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_va80NonCriticalExtensions_02, T_va80NonCriticalExtensions_02_sequence);

  return offset;
}


static const per_sequence_t T_va40NonCriticalExtensions_18_sequence[] = {
  { &hf_rrc_sysInfoType5_va40ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SysInfoType5_va40ext_IEs },
  { &hf_rrc_va80NonCriticalExtensions_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_va80NonCriticalExtensions_02 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_va40NonCriticalExtensions_18(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_va40NonCriticalExtensions_18, T_va40NonCriticalExtensions_18_sequence);

  return offset;
}


static const per_sequence_t T_v8d0NonCriticalExtensions_sequence[] = {
  { &hf_rrc_sysInfoType5_v8d0ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SysInfoType5_v8d0ext_IEs },
  { &hf_rrc_va40NonCriticalExtensions_19, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_va40NonCriticalExtensions_18 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v8d0NonCriticalExtensions(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v8d0NonCriticalExtensions, T_v8d0NonCriticalExtensions_sequence);

  return offset;
}


static const per_sequence_t T_v8b0NonCriticalExtensions_01_sequence[] = {
  { &hf_rrc_sysInfoType5_v8b0ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SysInfoType5_v8b0ext_IEs },
  { &hf_rrc_v8d0NonCriticalExtensions, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v8d0NonCriticalExtensions },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v8b0NonCriticalExtensions_01(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v8b0NonCriticalExtensions_01, T_v8b0NonCriticalExtensions_01_sequence);

  return offset;
}


static const per_sequence_t T_v890NonCriticalExtensions_11_sequence[] = {
  { &hf_rrc_sysInfoType5_v890ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SysInfoType5_v890ext_IEs },
  { &hf_rrc_v8b0NonCriticalExtensions_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v8b0NonCriticalExtensions_01 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v890NonCriticalExtensions_11(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v890NonCriticalExtensions_11, T_v890NonCriticalExtensions_11_sequence);

  return offset;
}


static const per_sequence_t T_v860NonCriticalExtensions_27_sequence[] = {
  { &hf_rrc_sysInfoType5_v860ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SysInfoType5_v860ext_IEs },
  { &hf_rrc_v890NonCriticalExtensions_11, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v890NonCriticalExtensions_11 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v860NonCriticalExtensions_27(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v860NonCriticalExtensions_27, T_v860NonCriticalExtensions_27_sequence);

  return offset;
}


static const per_sequence_t T_v770NonCriticalExtensions_40_sequence[] = {
  { &hf_rrc_sysInfoType5_v770ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SysInfoType5_v770ext_IEs },
  { &hf_rrc_v860NonCriticalExtensions_27, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v860NonCriticalExtensions_27 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v770NonCriticalExtensions_40(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v770NonCriticalExtensions_40, T_v770NonCriticalExtensions_40_sequence);

  return offset;
}


static const per_sequence_t T_v690NonCriticalExtensions_49_sequence[] = {
  { &hf_rrc_sysInfoType5_v690ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SysInfoType5_v690ext_IEs },
  { &hf_rrc_v770NonCriticalExtensions_40, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v770NonCriticalExtensions_40 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v690NonCriticalExtensions_49(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v690NonCriticalExtensions_49, T_v690NonCriticalExtensions_49_sequence);

  return offset;
}


static const per_sequence_t T_v680NonCriticalExtensions_02_sequence[] = {
  { &hf_rrc_sysInfoType5_v680ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_SysInfoType5_v680ext_IEs },
  { &hf_rrc_v690NonCriticalExtensions_49, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v690NonCriticalExtensions_49 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v680NonCriticalExtensions_02(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v680NonCriticalExtensions_02, T_v680NonCriticalExtensions_02_sequence);

  return offset;
}


static const per_sequence_t T_v650NonCriticalExtensions_sequence[] = {
  { &hf_rrc_sysInfoType5_v650ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_SysInfoType5_v650ext_IEs },
  { &hf_rrc_v680NonCriticalExtensions_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v680NonCriticalExtensions_02 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v650NonCriticalExtensions(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v650NonCriticalExtensions, T_v650NonCriticalExtensions_sequence);

  return offset;
}


static const per_sequence_t T_v590NonCriticalExtensions_28_sequence[] = {
  { &hf_rrc_sysInfoType5_v590ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_SysInfoType5_v590ext_IEs },
  { &hf_rrc_v650NonCriticalExtensions, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v650NonCriticalExtensions },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v590NonCriticalExtensions_28(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v590NonCriticalExtensions_28, T_v590NonCriticalExtensions_28_sequence);

  return offset;
}


static const per_sequence_t T_v4b0NonCriticalExtensions_19_sequence[] = {
  { &hf_rrc_sysInfoType5_v4b0ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_SysInfoType5_v4b0ext_IEs },
  { &hf_rrc_v590NonCriticalExtensions_28, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v590NonCriticalExtensions_28 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v4b0NonCriticalExtensions_19(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v4b0NonCriticalExtensions_19, T_v4b0NonCriticalExtensions_19_sequence);

  return offset;
}


static const per_sequence_t SysInfoType5_sequence[] = {
  { &hf_rrc_sib6indicator   , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { &hf_rrc_pich_PowerOffset, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PICH_PowerOffset },
  { &hf_rrc_modeSpecificInfo_150, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_modeSpecificInfo_150 },
  { &hf_rrc_primaryCCPCH_Info_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_PrimaryCCPCH_Info },
  { &hf_rrc_prach_SystemInformationList, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PRACH_SystemInformationList },
  { &hf_rrc_sCCPCH_SystemInformationList, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SCCPCH_SystemInformationList },
  { &hf_rrc_cbs_DRX_Level1Information, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CBS_DRX_Level1Information },
  { &hf_rrc_v4b0NonCriticalExtensions_19, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v4b0NonCriticalExtensions_19 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_SysInfoType5(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_SysInfoType5, SysInfoType5_sequence);

  return offset;
}



static int
dissect_rrc_SysInfoType5bis(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_rrc_SysInfoType5(tvb, offset, actx, tree, hf_index);

  return offset;
}


static const per_sequence_t T_fdd_188_sequence[] = {
  { &hf_rrc_aich_PowerOffset, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_AICH_PowerOffset },
  { &hf_rrc_dummy_57        , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CSICH_PowerOffset },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_fdd_188(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_fdd_188, T_fdd_188_sequence);

  return offset;
}


static const per_sequence_t T_tdd_135_sequence[] = {
  { &hf_rrc_pusch_SysInfoList_SFN, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_PUSCH_SysInfoList_SFN },
  { &hf_rrc_pdsch_SysInfoList_SFN, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_PDSCH_SysInfoList_SFN },
  { &hf_rrc_openLoopPowerControl_TDD, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_OpenLoopPowerControl_TDD },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_tdd_135(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_tdd_135, T_tdd_135_sequence);

  return offset;
}


static const value_string rrc_T_modeSpecificInfo_153_vals[] = {
  {   0, "fdd" },
  {   1, "tdd" },
  { 0, NULL }
};

static const per_choice_t T_modeSpecificInfo_153_choice[] = {
  {   0, &hf_rrc_fdd_194         , ASN1_NO_EXTENSIONS     , dissect_rrc_T_fdd_188 },
  {   1, &hf_rrc_tdd_143         , ASN1_NO_EXTENSIONS     , dissect_rrc_T_tdd_135 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_modeSpecificInfo_153(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_modeSpecificInfo_153, T_modeSpecificInfo_153_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t T_tdd128SpecificInfo_01_sequence[] = {
  { &hf_rrc_pusch_SysInfoList_SFN_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_PUSCH_SysInfoList_SFN_LCR_r4 },
  { &hf_rrc_pdsch_SysInfoList_SFN_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_PDSCH_SysInfoList_SFN_LCR_r4 },
  { &hf_rrc_pCCPCH_LCR_Extensions, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_PrimaryCCPCH_Info_LCR_r4_ext },
  { &hf_rrc_sCCPCH_LCR_ExtensionsList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_SCCPCH_SystemInformationList_LCR_r4_ext },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_tdd128SpecificInfo_01(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_tdd128SpecificInfo_01, T_tdd128SpecificInfo_01_sequence);

  return offset;
}


static const per_sequence_t SysInfoType6_v4b0ext_IEs_sequence[] = {
  { &hf_rrc_openLoopPowerControl_IPDL_TDD, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_OpenLoopPowerControl_IPDL_TDD_r4 },
  { &hf_rrc_prach_SystemInformationList_LCR_r4, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_PRACH_SystemInformationList_LCR_r4 },
  { &hf_rrc_tdd128SpecificInfo_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_tdd128SpecificInfo_01 },
  { &hf_rrc_frequencyBandIndicator, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RadioFrequencyBandFDD },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_SysInfoType6_v4b0ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_SysInfoType6_v4b0ext_IEs, SysInfoType6_v4b0ext_IEs_sequence);

  return offset;
}


static const per_sequence_t T_hcr_r5_SpecificInfo_01_sequence[] = {
  { &hf_rrc_pusch_SysInfoList_SFN_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_PUSCH_SysInfoList_SFN_HCR_r5 },
  { &hf_rrc_pdsch_SysInfoList_SFN_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_PDSCH_SysInfoList_SFN_HCR_r5 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_hcr_r5_SpecificInfo_01(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_hcr_r5_SpecificInfo_01, T_hcr_r5_SpecificInfo_01_sequence);

  return offset;
}


static const per_sequence_t SysInfoType6_v590ext_IEs_sequence[] = {
  { &hf_rrc_hcr_r5_SpecificInfo_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_hcr_r5_SpecificInfo_01 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_SysInfoType6_v590ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_SysInfoType6_v590ext_IEs, SysInfoType6_v590ext_IEs_sequence);

  return offset;
}


static const per_sequence_t SysInfoType6_v650ext_IEs_sequence[] = {
  { &hf_rrc_frequencyBandIndicator2, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RadioFrequencyBandFDD2 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_SysInfoType6_v650ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_SysInfoType6_v650ext_IEs, SysInfoType6_v650ext_IEs_sequence);

  return offset;
}


static const per_sequence_t SysInfoType6_v690ext_IEs_sequence[] = {
  { &hf_rrc_additionalPRACH_TF_and_TFCS_CCCH_List, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_AdditionalPRACH_TF_and_TFCS_CCCH_List },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_SysInfoType6_v690ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_SysInfoType6_v690ext_IEs, SysInfoType6_v690ext_IEs_sequence);

  return offset;
}


static const per_sequence_t T_tdd768SpecificInfo_sequence[] = {
  { &hf_rrc_pusch_SysInfoList_SFN_03, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_PUSCH_SysInfoList_SFN_VHCR },
  { &hf_rrc_pdsch_SysInfoList_SFN_03, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_PDSCH_SysInfoList_VHCR_r7 },
  { &hf_rrc_prach_SystemInformationList_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_PRACH_SystemInformationList_VHCR_r7 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_tdd768SpecificInfo(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_tdd768SpecificInfo, T_tdd768SpecificInfo_sequence);

  return offset;
}


static const per_sequence_t SysInfoType6_v770ext_IEs_sequence[] = {
  { &hf_rrc_tdd768SpecificInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_tdd768SpecificInfo },
  { &hf_rrc_sccpch_SystemInformationList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_SCCPCH_SystemInformationList_HCR_VHCR_r7 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_SysInfoType6_v770ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_SysInfoType6_v770ext_IEs, SysInfoType6_v770ext_IEs_sequence);

  return offset;
}


static const per_sequence_t SysInfoType6_va40ext_IEs_sequence[] = {
  { &hf_rrc_frequencyBandIndicator3, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RadioFrequencyBandFDD3 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_SysInfoType6_va40ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_SysInfoType6_va40ext_IEs, SysInfoType6_va40ext_IEs_sequence);

  return offset;
}


static const per_sequence_t SysInfoType6_va80ext_IEs_sequence[] = {
  { &hf_rrc_multipleFrequencyBandIndicatorListFDD, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MultipleFrequencyBandIndicatorListFDD },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_SysInfoType6_va80ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_SysInfoType6_va80ext_IEs, SysInfoType6_va80ext_IEs_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions_201_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_nonCriticalExtensions_201(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_nonCriticalExtensions_201, T_nonCriticalExtensions_201_sequence);

  return offset;
}


static const per_sequence_t T_va80NonCriticalExtensions_03_sequence[] = {
  { &hf_rrc_sysInfoType6_va80ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SysInfoType6_va80ext_IEs },
  { &hf_rrc_nonCriticalExtensions_201, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_nonCriticalExtensions_201 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_va80NonCriticalExtensions_03(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_va80NonCriticalExtensions_03, T_va80NonCriticalExtensions_03_sequence);

  return offset;
}


static const per_sequence_t T_va40NonCriticalExtensions_19_sequence[] = {
  { &hf_rrc_sysInfoType6_va40ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SysInfoType6_va40ext_IEs },
  { &hf_rrc_va80NonCriticalExtensions_03, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_va80NonCriticalExtensions_03 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_va40NonCriticalExtensions_19(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_va40NonCriticalExtensions_19, T_va40NonCriticalExtensions_19_sequence);

  return offset;
}


static const per_sequence_t T_v770NonCriticalExtensions_41_sequence[] = {
  { &hf_rrc_sysInfoType6_v770ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SysInfoType6_v770ext_IEs },
  { &hf_rrc_va40NonCriticalExtensions_20, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_va40NonCriticalExtensions_19 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v770NonCriticalExtensions_41(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v770NonCriticalExtensions_41, T_v770NonCriticalExtensions_41_sequence);

  return offset;
}


static const per_sequence_t T_v690nonCriticalExtensions_sequence[] = {
  { &hf_rrc_sysInfoType6_v690ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SysInfoType6_v690ext_IEs },
  { &hf_rrc_v770NonCriticalExtensions_41, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v770NonCriticalExtensions_41 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v690nonCriticalExtensions(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v690nonCriticalExtensions, T_v690nonCriticalExtensions_sequence);

  return offset;
}


static const per_sequence_t T_v650nonCriticalExtensions_sequence[] = {
  { &hf_rrc_sysInfoType6_v650ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_SysInfoType6_v650ext_IEs },
  { &hf_rrc_v690nonCriticalExtensions, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v690nonCriticalExtensions },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v650nonCriticalExtensions(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v650nonCriticalExtensions, T_v650nonCriticalExtensions_sequence);

  return offset;
}


static const per_sequence_t T_v590NonCriticalExtensions_29_sequence[] = {
  { &hf_rrc_sysInfoType6_v590ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_SysInfoType6_v590ext_IEs },
  { &hf_rrc_v650nonCriticalExtensions, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v650nonCriticalExtensions },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v590NonCriticalExtensions_29(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v590NonCriticalExtensions_29, T_v590NonCriticalExtensions_29_sequence);

  return offset;
}


static const per_sequence_t T_v4b0NonCriticalExtensions_20_sequence[] = {
  { &hf_rrc_sysInfoType6_v4b0ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_SysInfoType6_v4b0ext_IEs },
  { &hf_rrc_v590NonCriticalExtensions_29, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v590NonCriticalExtensions_29 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v4b0NonCriticalExtensions_20(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v4b0NonCriticalExtensions_20, T_v4b0NonCriticalExtensions_20_sequence);

  return offset;
}


static const per_sequence_t SysInfoType6_sequence[] = {
  { &hf_rrc_pich_PowerOffset, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PICH_PowerOffset },
  { &hf_rrc_modeSpecificInfo_153, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_modeSpecificInfo_153 },
  { &hf_rrc_primaryCCPCH_Info_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_PrimaryCCPCH_Info },
  { &hf_rrc_prach_SystemInformationList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_PRACH_SystemInformationList },
  { &hf_rrc_sCCPCH_SystemInformationList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_SCCPCH_SystemInformationList },
  { &hf_rrc_cbs_DRX_Level1Information, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CBS_DRX_Level1Information },
  { &hf_rrc_v4b0NonCriticalExtensions_20, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v4b0NonCriticalExtensions_20 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_SysInfoType6(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_SysInfoType6, SysInfoType6_sequence);

  return offset;
}


static const per_sequence_t T_fdd_189_sequence[] = {
  { &hf_rrc_ul_Interference , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UL_Interference },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_fdd_189(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_fdd_189, T_fdd_189_sequence);

  return offset;
}


static const value_string rrc_T_modeSpecificInfo_154_vals[] = {
  {   0, "fdd" },
  {   1, "tdd" },
  { 0, NULL }
};

static const per_choice_t T_modeSpecificInfo_154_choice[] = {
  {   0, &hf_rrc_fdd_195         , ASN1_NO_EXTENSIONS     , dissect_rrc_T_fdd_189 },
  {   1, &hf_rrc_tdd             , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_modeSpecificInfo_154(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_modeSpecificInfo_154, T_modeSpecificInfo_154_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions_202_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_nonCriticalExtensions_202(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_nonCriticalExtensions_202, T_nonCriticalExtensions_202_sequence);

  return offset;
}


static const per_sequence_t SysInfoType7_sequence[] = {
  { &hf_rrc_modeSpecificInfo_154, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_modeSpecificInfo_154 },
  { &hf_rrc_prach_Information_SIB5_List, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DynamicPersistenceLevelList },
  { &hf_rrc_prach_Information_SIB6_List, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DynamicPersistenceLevelList },
  { &hf_rrc_expirationTimeFactor, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_ExpirationTimeFactor },
  { &hf_rrc_nonCriticalExtensions_202, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_nonCriticalExtensions_202 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_SysInfoType7(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_SysInfoType7, SysInfoType7_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions_203_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_nonCriticalExtensions_203(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_nonCriticalExtensions_203, T_nonCriticalExtensions_203_sequence);

  return offset;
}


static const per_sequence_t SysInfoType8_sequence[] = {
  { &hf_rrc_dummy1_07       , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CPCH_Parameters },
  { &hf_rrc_dummy2_15       , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CPCH_SetInfoList },
  { &hf_rrc_dummy3_04       , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CSICH_PowerOffset },
  { &hf_rrc_nonCriticalExtensions_203, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_nonCriticalExtensions_203 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_SysInfoType8(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_SysInfoType8, SysInfoType8_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions_204_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_nonCriticalExtensions_204(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_nonCriticalExtensions_204, T_nonCriticalExtensions_204_sequence);

  return offset;
}


static const per_sequence_t SysInfoType9_sequence[] = {
  { &hf_rrc_dummy_58        , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CPCH_PersistenceLevelsList },
  { &hf_rrc_nonCriticalExtensions_204, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_nonCriticalExtensions_204 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_SysInfoType9(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_SysInfoType9, SysInfoType9_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions_205_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_nonCriticalExtensions_205(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_nonCriticalExtensions_205, T_nonCriticalExtensions_205_sequence);

  return offset;
}


static const per_sequence_t SysInfoType10_sequence[] = {
  { &hf_rrc_dummy_59        , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DRAC_SysInfoList },
  { &hf_rrc_nonCriticalExtensions_205, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_nonCriticalExtensions_205 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_SysInfoType10(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_SysInfoType10, SysInfoType10_sequence);

  return offset;
}


static const per_sequence_t SysInfoType11_v4b0ext_IEs_sequence[] = {
  { &hf_rrc_fach_MeasurementOccasionInfo_LCR_Ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_FACH_MeasurementOccasionInfo_LCR_r4_ext },
  { &hf_rrc_measurementControlSysInfo_LCR, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MeasurementControlSysInfo_LCR_r4_ext },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_SysInfoType11_v4b0ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_SysInfoType11_v4b0ext_IEs, SysInfoType11_v4b0ext_IEs_sequence);

  return offset;
}


static const per_sequence_t SysInfoType11_v590ext_IEs_sequence[] = {
  { &hf_rrc_newIntraFrequencyCellInfoList_v590ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_SEQUENCE_SIZE_1_maxCellMeas_OF_CellSelectReselectInfo_v590ext },
  { &hf_rrc_newInterFrequencyCellInfoList_v590ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_SEQUENCE_SIZE_1_maxCellMeas_OF_CellSelectReselectInfo_v590ext },
  { &hf_rrc_newInterRATCellInfoList_v590ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_SEQUENCE_SIZE_1_maxCellMeas_OF_CellSelectReselectInfo_v590ext },
  { &hf_rrc_intraFreqEventCriteriaList_v590ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_Intra_FreqEventCriteriaList_v590ext },
  { &hf_rrc_intraFreqReportingCriteria_1b_r5, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_IntraFreqReportingCriteria_1b_r5 },
  { &hf_rrc_intraFreqEvent_1d_r5, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_IntraFreqEvent_1d_r5 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_SysInfoType11_v590ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_SysInfoType11_v590ext_IEs, SysInfoType11_v590ext_IEs_sequence);

  return offset;
}


static const per_sequence_t SysInfoType11_v690ext_IEs_sequence[] = {
  { &hf_rrc_dummy_60        , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_Dummy_InterFreqRACHReportingInfo },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_SysInfoType11_v690ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_SysInfoType11_v690ext_IEs, SysInfoType11_v690ext_IEs_sequence);

  return offset;
}


static const per_sequence_t SysInfoType11_v6b0ext_IEs_sequence[] = {
  { &hf_rrc_interFreqRACHReportingInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_InterFreqRACHReportingInfo },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_SysInfoType11_v6b0ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_SysInfoType11_v6b0ext_IEs, SysInfoType11_v6b0ext_IEs_sequence);

  return offset;
}


static const per_sequence_t MBSFNFrequency_sequence[] = {
  { &hf_rrc_frequencyInfo   , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_FrequencyInfo },
  { &hf_rrc_cellParametersID, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CellParametersID },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_MBSFNFrequency(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_MBSFNFrequency, MBSFNFrequency_sequence);

  return offset;
}


static const per_sequence_t MBSFNFrequencyList_sequence_of[1] = {
  { &hf_rrc_MBSFNFrequencyList_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MBSFNFrequency },
};

static int
dissect_rrc_MBSFNFrequencyList(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_MBSFNFrequencyList, MBSFNFrequencyList_sequence_of,
                                                  0, maxMBSFNClusters, FALSE);

  return offset;
}


static const per_sequence_t SysInfoType11_v770ext_IEs_sequence[] = {
  { &hf_rrc_mbsfnFrequencyList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MBSFNFrequencyList },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_SysInfoType11_v770ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_SysInfoType11_v770ext_IEs, SysInfoType11_v770ext_IEs_sequence);

  return offset;
}


static const per_sequence_t SysInfoType11_v7b0ext_IEs_sequence[] = {
  { &hf_rrc_newInterFreqCellList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_NewInterFreqCellList_v7b0ext },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_SysInfoType11_v7b0ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_SysInfoType11_v7b0ext_IEs, SysInfoType11_v7b0ext_IEs_sequence);

  return offset;
}


static const value_string rrc_T_imb_Indication_vals[] = {
  {   0, "true" },
  { 0, NULL }
};


static int
dissect_rrc_T_imb_Indication(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t MBSFNFrequency_v860ext_sequence[] = {
  { &hf_rrc_imb_Indication  , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_imb_Indication },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_MBSFNFrequency_v860ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_MBSFNFrequency_v860ext, MBSFNFrequency_v860ext_sequence);

  return offset;
}


static const per_sequence_t MBSFNFrequencyList_v860ext_sequence_of[1] = {
  { &hf_rrc_MBSFNFrequencyList_v860ext_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MBSFNFrequency_v860ext },
};

static int
dissect_rrc_MBSFNFrequencyList_v860ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_MBSFNFrequencyList_v860ext, MBSFNFrequencyList_v860ext_sequence_of,
                                                  0, maxMBSFNClusters, FALSE);

  return offset;
}


static const per_sequence_t SysInfoType11_v860ext_IEs_sequence[] = {
  { &hf_rrc_mbsfnFrequencyList_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MBSFNFrequencyList_v860ext },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_SysInfoType11_v860ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_SysInfoType11_v860ext_IEs, SysInfoType11_v860ext_IEs_sequence);

  return offset;
}


static const per_sequence_t SysInfoType11_va80ext_IEs_sequence[] = {
  { &hf_rrc_multipleFrequencyInfoListFDD, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MultipleFrequencyInfoListFDD },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_SysInfoType11_va80ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_SysInfoType11_va80ext_IEs, SysInfoType11_va80ext_IEs_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions_206_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_nonCriticalExtensions_206(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_nonCriticalExtensions_206, T_nonCriticalExtensions_206_sequence);

  return offset;
}


static const per_sequence_t T_va80NonCriticalExtensions_04_sequence[] = {
  { &hf_rrc_sysInfoType11_va80ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SysInfoType11_va80ext_IEs },
  { &hf_rrc_nonCriticalExtensions_206, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_nonCriticalExtensions_206 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_va80NonCriticalExtensions_04(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_va80NonCriticalExtensions_04, T_va80NonCriticalExtensions_04_sequence);

  return offset;
}


static const per_sequence_t T_v860NonCriticalExtensions_28_sequence[] = {
  { &hf_rrc_sysInfoType11_v860ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SysInfoType11_v860ext_IEs },
  { &hf_rrc_va80NonCriticalExtensions_04, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_va80NonCriticalExtensions_04 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v860NonCriticalExtensions_28(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v860NonCriticalExtensions_28, T_v860NonCriticalExtensions_28_sequence);

  return offset;
}


static const per_sequence_t T_v7b0NonCriticalExtensions_02_sequence[] = {
  { &hf_rrc_sysInfoType11_v7b0ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SysInfoType11_v7b0ext_IEs },
  { &hf_rrc_v860NonCriticalExtensions_28, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v860NonCriticalExtensions_28 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v7b0NonCriticalExtensions_02(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v7b0NonCriticalExtensions_02, T_v7b0NonCriticalExtensions_02_sequence);

  return offset;
}


static const per_sequence_t T_v770NonCriticalExtensions_42_sequence[] = {
  { &hf_rrc_sysInfoType11_v770ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SysInfoType11_v770ext_IEs },
  { &hf_rrc_v7b0NonCriticalExtensions_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v7b0NonCriticalExtensions_02 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v770NonCriticalExtensions_42(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v770NonCriticalExtensions_42, T_v770NonCriticalExtensions_42_sequence);

  return offset;
}


static const per_sequence_t T_v6b0NonCriticalExtensions_16_sequence[] = {
  { &hf_rrc_sysInfoType11_v6b0ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SysInfoType11_v6b0ext_IEs },
  { &hf_rrc_v770NonCriticalExtensions_42, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v770NonCriticalExtensions_42 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v6b0NonCriticalExtensions_16(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v6b0NonCriticalExtensions_16, T_v6b0NonCriticalExtensions_16_sequence);

  return offset;
}


static const per_sequence_t T_v690NonCriticalExtensions_50_sequence[] = {
  { &hf_rrc_sysInfoType11_v690ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SysInfoType11_v690ext_IEs },
  { &hf_rrc_v6b0NonCriticalExtensions_17, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v6b0NonCriticalExtensions_16 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v690NonCriticalExtensions_50(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v690NonCriticalExtensions_50, T_v690NonCriticalExtensions_50_sequence);

  return offset;
}


static const per_sequence_t T_v590NonCriticalExtension_02_sequence[] = {
  { &hf_rrc_sysInfoType11_v590ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SysInfoType11_v590ext_IEs },
  { &hf_rrc_v690NonCriticalExtensions_50, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v690NonCriticalExtensions_50 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v590NonCriticalExtension_02(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v590NonCriticalExtension_02, T_v590NonCriticalExtension_02_sequence);

  return offset;
}


static const per_sequence_t T_v4b0NonCriticalExtensions_21_sequence[] = {
  { &hf_rrc_sysInfoType11_v4b0ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_SysInfoType11_v4b0ext_IEs },
  { &hf_rrc_v590NonCriticalExtension_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v590NonCriticalExtension_02 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v4b0NonCriticalExtensions_21(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v4b0NonCriticalExtensions_21, T_v4b0NonCriticalExtensions_21_sequence);

  return offset;
}


static const per_sequence_t SysInfoType11_sequence[] = {
  { &hf_rrc_sib12indicator  , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { &hf_rrc_fach_MeasurementOccasionInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_FACH_MeasurementOccasionInfo },
  { &hf_rrc_measurementControlSysInfo, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MeasurementControlSysInfo },
  { &hf_rrc_v4b0NonCriticalExtensions_21, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v4b0NonCriticalExtensions_21 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_SysInfoType11(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_SysInfoType11, SysInfoType11_sequence);

  return offset;
}


static const per_sequence_t SysInfoType11bis_v7b0ext_IEs_sequence[] = {
  { &hf_rrc_newInterFreqCellList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_NewInterFreqCellList_v7b0ext },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_SysInfoType11bis_v7b0ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_SysInfoType11bis_v7b0ext_IEs, SysInfoType11bis_v7b0ext_IEs_sequence);

  return offset;
}


static const per_sequence_t SysInfoType11bis_v860ext_IEs_sequence[] = {
  { &hf_rrc_csg_PSCSplitInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CSG_PSCSplitInfo },
  { &hf_rrc_csg_DedicatedFrequencyInfoList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CSG_DedicatedFrequencyInfoList },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_SysInfoType11bis_v860ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_SysInfoType11bis_v860ext_IEs, SysInfoType11bis_v860ext_IEs_sequence);

  return offset;
}


static const per_sequence_t SysInfoType11bis_va80ext_IEs_sequence[] = {
  { &hf_rrc_multipleFrequencyInfoListFDD, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MultipleFrequencyInfoListFDD },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_SysInfoType11bis_va80ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_SysInfoType11bis_va80ext_IEs, SysInfoType11bis_va80ext_IEs_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions_207_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_nonCriticalExtensions_207(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_nonCriticalExtensions_207, T_nonCriticalExtensions_207_sequence);

  return offset;
}


static const per_sequence_t T_va80NonCriticalExtensions_05_sequence[] = {
  { &hf_rrc_sysInfoType11bis_va80ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SysInfoType11bis_va80ext_IEs },
  { &hf_rrc_nonCriticalExtensions_207, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_nonCriticalExtensions_207 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_va80NonCriticalExtensions_05(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_va80NonCriticalExtensions_05, T_va80NonCriticalExtensions_05_sequence);

  return offset;
}


static const per_sequence_t T_v860NonCriticalExtensions_29_sequence[] = {
  { &hf_rrc_sysInfoType11bis_v860ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SysInfoType11bis_v860ext_IEs },
  { &hf_rrc_va80NonCriticalExtensions_05, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_va80NonCriticalExtensions_05 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v860NonCriticalExtensions_29(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v860NonCriticalExtensions_29, T_v860NonCriticalExtensions_29_sequence);

  return offset;
}


static const per_sequence_t T_v7b0NonCriticalExtensions_03_sequence[] = {
  { &hf_rrc_sysInfoType11bis_v7b0ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SysInfoType11bis_v7b0ext_IEs },
  { &hf_rrc_v860NonCriticalExtensions_29, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v860NonCriticalExtensions_29 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v7b0NonCriticalExtensions_03(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v7b0NonCriticalExtensions_03, T_v7b0NonCriticalExtensions_03_sequence);

  return offset;
}


static const per_sequence_t SysInfoType11bis_sequence[] = {
  { &hf_rrc_measurementControlSysInfo_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MeasurementControlSysInfoExtension },
  { &hf_rrc_measurementControlSysInfo_LCR_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MeasurementControlSysInfoExtension_LCR_r4 },
  { &hf_rrc_measurementControlSysInfoExtensionAddon_r5, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MeasurementControlSysInfoExtensionAddon_r5 },
  { &hf_rrc_v7b0NonCriticalExtensions_03, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v7b0NonCriticalExtensions_03 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_SysInfoType11bis(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_SysInfoType11bis, SysInfoType11bis_sequence);

  return offset;
}


static const per_sequence_t SysInfoType12_v4b0ext_IEs_sequence[] = {
  { &hf_rrc_fach_MeasurementOccasionInfo_LCR_Ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_FACH_MeasurementOccasionInfo_LCR_r4_ext },
  { &hf_rrc_measurementControlSysInfo_LCR, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MeasurementControlSysInfo_LCR_r4_ext },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_SysInfoType12_v4b0ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_SysInfoType12_v4b0ext_IEs, SysInfoType12_v4b0ext_IEs_sequence);

  return offset;
}


static const per_sequence_t SysInfoType12_v590ext_IEs_sequence[] = {
  { &hf_rrc_newIntraFrequencyCellInfoList_v590ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_SEQUENCE_SIZE_1_maxCellMeas_OF_CellSelectReselectInfo_v590ext },
  { &hf_rrc_newInterFrequencyCellInfoList_v590ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_SEQUENCE_SIZE_1_maxCellMeas_OF_CellSelectReselectInfo_v590ext },
  { &hf_rrc_newInterRATCellInfoList_v590ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_SEQUENCE_SIZE_1_maxCellMeas_OF_CellSelectReselectInfo_v590ext },
  { &hf_rrc_intraFreqEventCriteriaList_v590ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_Intra_FreqEventCriteriaList_v590ext },
  { &hf_rrc_intraFreqReportingCriteria_1b_r5, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_IntraFreqReportingCriteria_1b_r5 },
  { &hf_rrc_intraFreqEvent_1d_r5, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_IntraFreqEvent_1d_r5 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_SysInfoType12_v590ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_SysInfoType12_v590ext_IEs, SysInfoType12_v590ext_IEs_sequence);

  return offset;
}


static const per_sequence_t SysInfoType12_v690ext_IEs_sequence[] = {
  { &hf_rrc_dummy_60        , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_Dummy_InterFreqRACHReportingInfo },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_SysInfoType12_v690ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_SysInfoType12_v690ext_IEs, SysInfoType12_v690ext_IEs_sequence);

  return offset;
}


static const per_sequence_t SysInfoType12_v6b0ext_IEs_sequence[] = {
  { &hf_rrc_interFreqRACHReportingInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_InterFreqRACHReportingInfo },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_SysInfoType12_v6b0ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_SysInfoType12_v6b0ext_IEs, SysInfoType12_v6b0ext_IEs_sequence);

  return offset;
}


static const per_sequence_t SysInfoType12_v7b0ext_IEs_sequence[] = {
  { &hf_rrc_newInterFreqCellList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_NewInterFreqCellList_v7b0ext },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_SysInfoType12_v7b0ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_SysInfoType12_v7b0ext_IEs, SysInfoType12_v7b0ext_IEs_sequence);

  return offset;
}


static const per_sequence_t SysInfoType12_va80ext_IEs_sequence[] = {
  { &hf_rrc_multipleFrequencyInfoListFDD, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MultipleFrequencyInfoListFDD },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_SysInfoType12_va80ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_SysInfoType12_va80ext_IEs, SysInfoType12_va80ext_IEs_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions_208_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_nonCriticalExtensions_208(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_nonCriticalExtensions_208, T_nonCriticalExtensions_208_sequence);

  return offset;
}


static const per_sequence_t T_va80NonCriticalExtensions_06_sequence[] = {
  { &hf_rrc_sysInfoType12_va80ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SysInfoType12_va80ext_IEs },
  { &hf_rrc_nonCriticalExtensions_208, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_nonCriticalExtensions_208 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_va80NonCriticalExtensions_06(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_va80NonCriticalExtensions_06, T_va80NonCriticalExtensions_06_sequence);

  return offset;
}


static const per_sequence_t T_v7b0NonCriticalExtensions_04_sequence[] = {
  { &hf_rrc_sysInfoType12_v7b0ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SysInfoType12_v7b0ext_IEs },
  { &hf_rrc_va80NonCriticalExtensions_06, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_va80NonCriticalExtensions_06 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v7b0NonCriticalExtensions_04(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v7b0NonCriticalExtensions_04, T_v7b0NonCriticalExtensions_04_sequence);

  return offset;
}


static const per_sequence_t T_v6b0NonCriticalExtensions_17_sequence[] = {
  { &hf_rrc_sysInfoType12_v6b0ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SysInfoType12_v6b0ext_IEs },
  { &hf_rrc_v7b0NonCriticalExtensions_04, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v7b0NonCriticalExtensions_04 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v6b0NonCriticalExtensions_17(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v6b0NonCriticalExtensions_17, T_v6b0NonCriticalExtensions_17_sequence);

  return offset;
}


static const per_sequence_t T_v690NonCriticalExtensions_51_sequence[] = {
  { &hf_rrc_sysInfoType12_v690ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SysInfoType12_v690ext_IEs },
  { &hf_rrc_v6b0NonCriticalExtensions_18, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v6b0NonCriticalExtensions_17 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v690NonCriticalExtensions_51(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v690NonCriticalExtensions_51, T_v690NonCriticalExtensions_51_sequence);

  return offset;
}


static const per_sequence_t T_v590NonCriticalExtension_03_sequence[] = {
  { &hf_rrc_sysInfoType12_v590ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SysInfoType12_v590ext_IEs },
  { &hf_rrc_v690NonCriticalExtensions_51, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v690NonCriticalExtensions_51 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v590NonCriticalExtension_03(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v590NonCriticalExtension_03, T_v590NonCriticalExtension_03_sequence);

  return offset;
}


static const per_sequence_t T_v4b0NonCriticalExtensions_22_sequence[] = {
  { &hf_rrc_sysInfoType12_v4b0ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_SysInfoType12_v4b0ext_IEs },
  { &hf_rrc_v590NonCriticalExtension_03, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v590NonCriticalExtension_03 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v4b0NonCriticalExtensions_22(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v4b0NonCriticalExtensions_22, T_v4b0NonCriticalExtensions_22_sequence);

  return offset;
}


static const per_sequence_t SysInfoType12_sequence[] = {
  { &hf_rrc_fach_MeasurementOccasionInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_FACH_MeasurementOccasionInfo },
  { &hf_rrc_measurementControlSysInfo, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MeasurementControlSysInfo },
  { &hf_rrc_v4b0NonCriticalExtensions_22, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v4b0NonCriticalExtensions_22 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_SysInfoType12(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_SysInfoType12, SysInfoType12_sequence);

  return offset;
}


static const per_sequence_t SysInfoType13_v3a0ext_IEs_sequence[] = {
  { &hf_rrc_ue_IdleTimersAndConstants_v3a0ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UE_IdleTimersAndConstants_v3a0ext },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_SysInfoType13_v3a0ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_SysInfoType13_v3a0ext_IEs, SysInfoType13_v3a0ext_IEs_sequence);

  return offset;
}


static const per_sequence_t SysInfoType13_v4b0ext_IEs_sequence[] = {
  { &hf_rrc_capabilityUpdateRequirement_r4Ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CapabilityUpdateRequirement_r4_ext },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_SysInfoType13_v4b0ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_SysInfoType13_v4b0ext_IEs, SysInfoType13_v4b0ext_IEs_sequence);

  return offset;
}


static const per_sequence_t SysInfoType13_v770ext_IEs_sequence[] = {
  { &hf_rrc_capabilityUpdateRequirement_05, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CapabilityUpdateRequirement_v770ext },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_SysInfoType13_v770ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_SysInfoType13_v770ext_IEs, SysInfoType13_v770ext_IEs_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions_209_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_nonCriticalExtensions_209(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_nonCriticalExtensions_209, T_nonCriticalExtensions_209_sequence);

  return offset;
}


static const per_sequence_t T_v770NonCriticalExtensions_43_sequence[] = {
  { &hf_rrc_sysInfoType13_v770ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SysInfoType13_v770ext_IEs },
  { &hf_rrc_nonCriticalExtensions_209, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_nonCriticalExtensions_209 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v770NonCriticalExtensions_43(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v770NonCriticalExtensions_43, T_v770NonCriticalExtensions_43_sequence);

  return offset;
}


static const per_sequence_t T_v4b0NonCriticalExtensions_23_sequence[] = {
  { &hf_rrc_sysInfoType13_v4b0ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SysInfoType13_v4b0ext_IEs },
  { &hf_rrc_v770NonCriticalExtensions_43, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v770NonCriticalExtensions_43 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v4b0NonCriticalExtensions_23(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v4b0NonCriticalExtensions_23, T_v4b0NonCriticalExtensions_23_sequence);

  return offset;
}


static const per_sequence_t T_v3a0NonCriticalExtensions_13_sequence[] = {
  { &hf_rrc_sysInfoType13_v3a0ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SysInfoType13_v3a0ext_IEs },
  { &hf_rrc_v4b0NonCriticalExtensions_23, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v4b0NonCriticalExtensions_23 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v3a0NonCriticalExtensions_13(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v3a0NonCriticalExtensions_13, T_v3a0NonCriticalExtensions_13_sequence);

  return offset;
}


static const per_sequence_t SysInfoType13_sequence[] = {
  { &hf_rrc_cn_DomainSysInfoList, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CN_DomainSysInfoList },
  { &hf_rrc_ue_IdleTimersAndConstants, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_IdleTimersAndConstants },
  { &hf_rrc_capabilityUpdateRequirement, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CapabilityUpdateRequirement },
  { &hf_rrc_v3a0NonCriticalExtensions_13, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v3a0NonCriticalExtensions_13 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_SysInfoType13(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_SysInfoType13, SysInfoType13_sequence);

  return offset;
}



static int
dissect_rrc_ANSI_41_RAND_Information(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_rrc_ANSI_41_NAS_Parameter(tvb, offset, actx, tree, hf_index);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions_210_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_nonCriticalExtensions_210(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_nonCriticalExtensions_210, T_nonCriticalExtensions_210_sequence);

  return offset;
}


static const per_sequence_t SysInfoType13_1_sequence[] = {
  { &hf_rrc_ansi_41_RAND_Information, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_ANSI_41_RAND_Information },
  { &hf_rrc_nonCriticalExtensions_210, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_nonCriticalExtensions_210 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_SysInfoType13_1(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_SysInfoType13_1, SysInfoType13_1_sequence);

  return offset;
}



static int
dissect_rrc_ANSI_41_UserZoneID_Information(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_rrc_ANSI_41_NAS_Parameter(tvb, offset, actx, tree, hf_index);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions_211_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_nonCriticalExtensions_211(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_nonCriticalExtensions_211, T_nonCriticalExtensions_211_sequence);

  return offset;
}


static const per_sequence_t SysInfoType13_2_sequence[] = {
  { &hf_rrc_ansi_41_UserZoneID_Information, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_ANSI_41_UserZoneID_Information },
  { &hf_rrc_nonCriticalExtensions_211, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_nonCriticalExtensions_211 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_SysInfoType13_2(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_SysInfoType13_2, SysInfoType13_2_sequence);

  return offset;
}



static int
dissect_rrc_ANSI_41_PrivateNeighbourListInfo(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_rrc_ANSI_41_NAS_Parameter(tvb, offset, actx, tree, hf_index);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions_212_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_nonCriticalExtensions_212(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_nonCriticalExtensions_212, T_nonCriticalExtensions_212_sequence);

  return offset;
}


static const per_sequence_t SysInfoType13_3_sequence[] = {
  { &hf_rrc_ansi_41_PrivateNeighbourListInfo, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_ANSI_41_PrivateNeighbourListInfo },
  { &hf_rrc_nonCriticalExtensions_212, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_nonCriticalExtensions_212 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_SysInfoType13_3(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_SysInfoType13_3, SysInfoType13_3_sequence);

  return offset;
}



static int
dissect_rrc_ANSI_41_GlobalServiceRedirectInfo(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_rrc_ANSI_41_NAS_Parameter(tvb, offset, actx, tree, hf_index);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions_213_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_nonCriticalExtensions_213(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_nonCriticalExtensions_213, T_nonCriticalExtensions_213_sequence);

  return offset;
}


static const per_sequence_t SysInfoType13_4_sequence[] = {
  { &hf_rrc_ansi_41_GlobalServiceRedirectInfo, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_ANSI_41_GlobalServiceRedirectInfo },
  { &hf_rrc_nonCriticalExtensions_213, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_nonCriticalExtensions_213 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_SysInfoType13_4(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_SysInfoType13_4, SysInfoType13_4_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions_214_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_nonCriticalExtensions_214(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_nonCriticalExtensions_214, T_nonCriticalExtensions_214_sequence);

  return offset;
}


static const per_sequence_t SysInfoType14_sequence[] = {
  { &hf_rrc_individualTS_InterferenceList, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_IndividualTS_InterferenceList },
  { &hf_rrc_expirationTimeFactor, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_ExpirationTimeFactor },
  { &hf_rrc_nonCriticalExtensions_214, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_nonCriticalExtensions_214 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_SysInfoType14(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_SysInfoType14, SysInfoType14_sequence);

  return offset;
}


static const per_sequence_t SysInfoType15_v4b0ext_IEs_sequence[] = {
  { &hf_rrc_up_Ipdl_Parameters_TDD, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_Positioning_IPDL_Parameters_TDD_r4_ext },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_SysInfoType15_v4b0ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_SysInfoType15_v4b0ext_IEs, SysInfoType15_v4b0ext_IEs_sequence);

  return offset;
}


static const per_sequence_t SysInfoType15_v770ext_IEs_sequence[] = {
  { &hf_rrc_ue_Positioning_GPS_ReferenceTime, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UE_Positioning_GPS_ReferenceTime_v770ext },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_SysInfoType15_v770ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_SysInfoType15_v770ext_IEs, SysInfoType15_v770ext_IEs_sequence);

  return offset;
}


static const per_sequence_t SysInfoType15_va40ext_IEs_sequence[] = {
  { &hf_rrc_ue_Positioning_GPS_ReferenceTime_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UE_Positioning_GPS_ReferenceTime_va40ext },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_SysInfoType15_va40ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_SysInfoType15_va40ext_IEs, SysInfoType15_va40ext_IEs_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions_215_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_nonCriticalExtensions_215(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_nonCriticalExtensions_215, T_nonCriticalExtensions_215_sequence);

  return offset;
}


static const per_sequence_t T_va40NonCriticalExtensions_20_sequence[] = {
  { &hf_rrc_sysInfoType15_va40ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SysInfoType15_va40ext_IEs },
  { &hf_rrc_nonCriticalExtensions_215, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_nonCriticalExtensions_215 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_va40NonCriticalExtensions_20(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_va40NonCriticalExtensions_20, T_va40NonCriticalExtensions_20_sequence);

  return offset;
}


static const per_sequence_t T_v770NonCriticalExtensions_44_sequence[] = {
  { &hf_rrc_sysInfoType15_v770ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SysInfoType15_v770ext_IEs },
  { &hf_rrc_va40NonCriticalExtensions_21, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_va40NonCriticalExtensions_20 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v770NonCriticalExtensions_44(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v770NonCriticalExtensions_44, T_v770NonCriticalExtensions_44_sequence);

  return offset;
}


static const per_sequence_t T_v4b0NonCriticalExtensions_24_sequence[] = {
  { &hf_rrc_sysInfoType15_v4b0ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SysInfoType15_v4b0ext_IEs },
  { &hf_rrc_v770NonCriticalExtensions_44, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v770NonCriticalExtensions_44 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v4b0NonCriticalExtensions_24(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v4b0NonCriticalExtensions_24, T_v4b0NonCriticalExtensions_24_sequence);

  return offset;
}


static const per_sequence_t SysInfoType15_sequence[] = {
  { &hf_rrc_ue_positioning_GPS_CipherParameters, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_Positioning_CipherParameters },
  { &hf_rrc_ue_positioning_GPS_ReferenceLocation, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_ReferenceLocation },
  { &hf_rrc_ue_positioning_GPS_ReferenceTime, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UE_Positioning_GPS_ReferenceTime },
  { &hf_rrc_ue_positioning_GPS_Real_timeIntegrity, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_BadSatList },
  { &hf_rrc_v4b0NonCriticalExtensions_24, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v4b0NonCriticalExtensions_24 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_SysInfoType15(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_SysInfoType15, SysInfoType15_sequence);

  return offset;
}


static const per_sequence_t SysInfoType15bis_v860ext_IEs_sequence[] = {
  { &hf_rrc_ue_Positioning_GANSS_AddIonoModel, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_Positioning_GANSS_AddIonoModel },
  { &hf_rrc_ue_Positioning_GANSS_EarthOrientationPara, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_Positioning_GANSS_EarthOrientPara },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_SysInfoType15bis_v860ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_SysInfoType15bis_v860ext_IEs, SysInfoType15bis_v860ext_IEs_sequence);

  return offset;
}


static const per_sequence_t SysInfoType15bis_va40ext_IEs_sequence[] = {
  { &hf_rrc_ue_positioning_GANSS_ReferenceTime_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UE_Positioning_GANSS_ReferenceTime_va40ext },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_SysInfoType15bis_va40ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_SysInfoType15bis_va40ext_IEs, SysInfoType15bis_va40ext_IEs_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions_216_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_nonCriticalExtensions_216(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_nonCriticalExtensions_216, T_nonCriticalExtensions_216_sequence);

  return offset;
}


static const per_sequence_t T_va40NonCriticalExtensions_21_sequence[] = {
  { &hf_rrc_sysInfoType15bis_va40ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SysInfoType15bis_va40ext_IEs },
  { &hf_rrc_nonCriticalExtensions_216, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_nonCriticalExtensions_216 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_va40NonCriticalExtensions_21(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_va40NonCriticalExtensions_21, T_va40NonCriticalExtensions_21_sequence);

  return offset;
}


static const per_sequence_t T_v860NonCriticalExtensions_30_sequence[] = {
  { &hf_rrc_sysInfoType15bis_v860ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SysInfoType15bis_v860ext_IEs },
  { &hf_rrc_va40NonCriticalExtensions_22, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_va40NonCriticalExtensions_21 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v860NonCriticalExtensions_30(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v860NonCriticalExtensions_30, T_v860NonCriticalExtensions_30_sequence);

  return offset;
}


static const per_sequence_t SysInfoType15bis_sequence[] = {
  { &hf_rrc_ue_positioning_GANSS_ReferencePosition, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_ReferenceLocationGANSS },
  { &hf_rrc_ue_positioning_GANSS_ReferenceTime, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UE_Positioning_GANSS_ReferenceTime },
  { &hf_rrc_ue_positioning_GANSS_IonosphericModel, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_Positioning_GANSS_IonosphericModel },
  { &hf_rrc_v860NonCriticalExtensions_30, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v860NonCriticalExtensions_30 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_SysInfoType15bis(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_SysInfoType15bis, SysInfoType15bis_sequence);

  return offset;
}


static const per_sequence_t SysInfoType15_1_v920ext_IEs_sequence[] = {
  { &hf_rrc_ue_positioning_GPS_DGPS_Corrections_02, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UE_Positioning_GPS_DGPS_Corrections_v920ext },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_SysInfoType15_1_v920ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_SysInfoType15_1_v920ext_IEs, SysInfoType15_1_v920ext_IEs_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions_217_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_nonCriticalExtensions_217(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_nonCriticalExtensions_217, T_nonCriticalExtensions_217_sequence);

  return offset;
}


static const per_sequence_t T_v920NonCriticalExtensions_04_sequence[] = {
  { &hf_rrc_sysInfoType15_1_v920ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SysInfoType15_1_v920ext_IEs },
  { &hf_rrc_nonCriticalExtensions_217, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_nonCriticalExtensions_217 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v920NonCriticalExtensions_04(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v920NonCriticalExtensions_04, T_v920NonCriticalExtensions_04_sequence);

  return offset;
}


static const per_sequence_t SysInfoType15_1_sequence[] = {
  { &hf_rrc_ue_positioning_GPS_DGPS_Corrections, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UE_Positioning_GPS_DGPS_Corrections },
  { &hf_rrc_v920NonCriticalExtensions_04, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v920NonCriticalExtensions_04 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_SysInfoType15_1(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_SysInfoType15_1, SysInfoType15_1_sequence);

  return offset;
}


static const per_sequence_t SysInfoType15_1bis_v920ext_IEs_sequence[] = {
  { &hf_rrc_ue_positioning_GANSS_DGANSS_Corrections_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UE_Positioning_DGANSSCorrections_v920ext },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_SysInfoType15_1bis_v920ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_SysInfoType15_1bis_v920ext_IEs, SysInfoType15_1bis_v920ext_IEs_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions_218_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_nonCriticalExtensions_218(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_nonCriticalExtensions_218, T_nonCriticalExtensions_218_sequence);

  return offset;
}


static const per_sequence_t T_v920NonCriticalExtensions_05_sequence[] = {
  { &hf_rrc_sysInfoType15_1bis_v920ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SysInfoType15_1bis_v920ext_IEs },
  { &hf_rrc_nonCriticalExtensions_218, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_nonCriticalExtensions_218 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v920NonCriticalExtensions_05(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v920NonCriticalExtensions_05, T_v920NonCriticalExtensions_05_sequence);

  return offset;
}


static const per_sequence_t SysInfoType15_1bis_sequence[] = {
  { &hf_rrc_ue_positioning_GANSS_DGANSS_Corrections, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UE_Positioning_DGANSSCorrections },
  { &hf_rrc_v920NonCriticalExtensions_05, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v920NonCriticalExtensions_05 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_SysInfoType15_1bis(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_SysInfoType15_1bis, SysInfoType15_1bis_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions_219_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_nonCriticalExtensions_219(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_nonCriticalExtensions_219, T_nonCriticalExtensions_219_sequence);

  return offset;
}


static const per_sequence_t SysInfoType15_2_sequence[] = {
  { &hf_rrc_transmissionTOW , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_GPS_TOW_1sec },
  { &hf_rrc_satID           , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SatID },
  { &hf_rrc_ephemerisParameter, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_EphemerisParameter },
  { &hf_rrc_nonCriticalExtensions_219, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_nonCriticalExtensions_219 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_SysInfoType15_2(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_SysInfoType15_2, SysInfoType15_2_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions_220_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_nonCriticalExtensions_220(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_nonCriticalExtensions_220, T_nonCriticalExtensions_220_sequence);

  return offset;
}


static const per_sequence_t SysInfoType15_2bis_sequence[] = {
  { &hf_rrc_ue_positioning_GANSS_navigationModel, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UE_Positioning_GANSS_NavigationModel },
  { &hf_rrc_nonCriticalExtensions_220, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_nonCriticalExtensions_220 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_SysInfoType15_2bis(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_SysInfoType15_2bis, SysInfoType15_2bis_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions_221_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_nonCriticalExtensions_221(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_nonCriticalExtensions_221, T_nonCriticalExtensions_221_sequence);

  return offset;
}


static const per_sequence_t SysInfoType15_2ter_sequence[] = {
  { &hf_rrc_ue_positioning_GANSS_AddNavigationModels, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UE_Positioning_GANSS_AddNavigationModels },
  { &hf_rrc_nonCriticalExtensions_221, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_nonCriticalExtensions_221 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_SysInfoType15_2ter(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_SysInfoType15_2ter, SysInfoType15_2ter_sequence);

  return offset;
}



static int
dissect_rrc_BIT_STRING_SIZE_1_32(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_bit_string(tvb, offset, actx, tree, hf_index,
                                     1, 32, FALSE, NULL);

  return offset;
}


static const per_sequence_t SysInfoType15_3_va40ext_IEs_sequence[] = {
  { &hf_rrc_ue_positioning_GPS_Almanac_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_Positioning_GPS_Almanac_va40ext },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_SysInfoType15_3_va40ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_SysInfoType15_3_va40ext_IEs, SysInfoType15_3_va40ext_IEs_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions_222_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_nonCriticalExtensions_222(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_nonCriticalExtensions_222, T_nonCriticalExtensions_222_sequence);

  return offset;
}


static const per_sequence_t T_va40NonCriticalExtensions_22_sequence[] = {
  { &hf_rrc_sysInfoType15_3_va40ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SysInfoType15_3_va40ext_IEs },
  { &hf_rrc_nonCriticalExtensions_222, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_nonCriticalExtensions_222 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_va40NonCriticalExtensions_22(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_va40NonCriticalExtensions_22, T_va40NonCriticalExtensions_22_sequence);

  return offset;
}


static const per_sequence_t SysInfoType15_3_sequence[] = {
  { &hf_rrc_transmissionTOW , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_GPS_TOW_1sec },
  { &hf_rrc_ue_positioning_GPS_Almanac, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_Positioning_GPS_Almanac },
  { &hf_rrc_ue_positioning_GPS_IonosphericModel, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_Positioning_GPS_IonosphericModel },
  { &hf_rrc_ue_positioning_GPS_UTC_Model, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_Positioning_GPS_UTC_Model },
  { &hf_rrc_satMask         , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_BIT_STRING_SIZE_1_32 },
  { &hf_rrc_lsbTOW          , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_BIT_STRING_SIZE_8 },
  { &hf_rrc_va40NonCriticalExtensions_23, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_va40NonCriticalExtensions_22 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_SysInfoType15_3(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_SysInfoType15_3, SysInfoType15_3_sequence);

  return offset;
}


static const per_sequence_t SysInfoType15_3bis_v860ext_IEs_sequence[] = {
  { &hf_rrc_ue_Positioning_GANSS_Almanac, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_Positioning_GANSS_Almanac_v860ext },
  { &hf_rrc_ue_Positioning_GANSS_AddUTCModels, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_Positioning_GANSS_AddUTCModels },
  { &hf_rrc_ue_Positioning_GANSS_AuxiliaryInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_Positioning_GANSS_AuxiliaryInfo },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_SysInfoType15_3bis_v860ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_SysInfoType15_3bis_v860ext_IEs, SysInfoType15_3bis_v860ext_IEs_sequence);

  return offset;
}


static const per_sequence_t SysInfoType15_3bis_va40ext_IEs_sequence[] = {
  { &hf_rrc_ue_Positioning_GANSS_Almanac_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_Positioning_GANSS_Almanac_va40ext },
  { &hf_rrc_ue_positioning_GANSS_TimeModels_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_Positioning_GANSS_TimeModels_va40ext },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_SysInfoType15_3bis_va40ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_SysInfoType15_3bis_va40ext_IEs, SysInfoType15_3bis_va40ext_IEs_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions_223_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_nonCriticalExtensions_223(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_nonCriticalExtensions_223, T_nonCriticalExtensions_223_sequence);

  return offset;
}


static const per_sequence_t T_va40NonCriticalExtensions_23_sequence[] = {
  { &hf_rrc_sysInfoType15_3bis_va40ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SysInfoType15_3bis_va40ext_IEs },
  { &hf_rrc_nonCriticalExtensions_223, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_nonCriticalExtensions_223 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_va40NonCriticalExtensions_23(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_va40NonCriticalExtensions_23, T_va40NonCriticalExtensions_23_sequence);

  return offset;
}


static const per_sequence_t T_v860NonCriticalExtensions_31_sequence[] = {
  { &hf_rrc_sysInfoType15_3bis_v860ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SysInfoType15_3bis_v860ext_IEs },
  { &hf_rrc_va40NonCriticalExtensions_24, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_va40NonCriticalExtensions_23 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v860NonCriticalExtensions_31(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v860NonCriticalExtensions_31, T_v860NonCriticalExtensions_31_sequence);

  return offset;
}


static const per_sequence_t SysInfoType15_3bis_sequence[] = {
  { &hf_rrc_ue_positioning_GANSS_Almanac, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_Positioning_GANSS_Almanac },
  { &hf_rrc_ue_positioning_GANSS_TimeModels, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_Positioning_GANSS_TimeModels },
  { &hf_rrc_ue_positioning_GANSS_UTC_Model, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_Positioning_GANSS_UTCModel },
  { &hf_rrc_v860NonCriticalExtensions_31, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v860NonCriticalExtensions_31 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_SysInfoType15_3bis(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_SysInfoType15_3bis, SysInfoType15_3bis_sequence);

  return offset;
}


static const per_sequence_t SysInfoType15_4_v3a0ext_sequence[] = {
  { &hf_rrc_sfn_Offset_Validity, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_SFN_Offset_Validity },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_SysInfoType15_4_v3a0ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_SysInfoType15_4_v3a0ext, SysInfoType15_4_v3a0ext_sequence);

  return offset;
}


static const per_sequence_t SysInfoType15_4_v4b0ext_sequence[] = {
  { &hf_rrc_ue_Positioning_OTDOA_AssistanceData_r4ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_Positioning_OTDOA_AssistanceData_r4ext },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_SysInfoType15_4_v4b0ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_SysInfoType15_4_v4b0ext, SysInfoType15_4_v4b0ext_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions_224_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_nonCriticalExtensions_224(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_nonCriticalExtensions_224, T_nonCriticalExtensions_224_sequence);

  return offset;
}


static const per_sequence_t T_v4b0NonCriticalExtensions_25_sequence[] = {
  { &hf_rrc_sysInfoType15_4_v4b0ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SysInfoType15_4_v4b0ext },
  { &hf_rrc_nonCriticalExtensions_224, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_nonCriticalExtensions_224 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v4b0NonCriticalExtensions_25(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v4b0NonCriticalExtensions_25, T_v4b0NonCriticalExtensions_25_sequence);

  return offset;
}


static const per_sequence_t T_v3a0NonCriticalExtensions_14_sequence[] = {
  { &hf_rrc_sysInfoType15_4_v3a0ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SysInfoType15_4_v3a0ext },
  { &hf_rrc_v4b0NonCriticalExtensions_25, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v4b0NonCriticalExtensions_25 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v3a0NonCriticalExtensions_14(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v3a0NonCriticalExtensions_14, T_v3a0NonCriticalExtensions_14_sequence);

  return offset;
}


static const per_sequence_t SysInfoType15_4_sequence[] = {
  { &hf_rrc_ue_positioning_OTDOA_CipherParameters, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_Positioning_CipherParameters },
  { &hf_rrc_ue_positioning_OTDOA_AssistanceData, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UE_Positioning_OTDOA_AssistanceData },
  { &hf_rrc_v3a0NonCriticalExtensions_14, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v3a0NonCriticalExtensions_14 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_SysInfoType15_4(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_SysInfoType15_4, SysInfoType15_4_sequence);

  return offset;
}


static const per_sequence_t SysInfoType15_5_v3a0ext_sequence[] = {
  { &hf_rrc_sfn_Offset_Validity, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_SFN_Offset_Validity },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_SysInfoType15_5_v3a0ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_SysInfoType15_5_v3a0ext, SysInfoType15_5_v3a0ext_sequence);

  return offset;
}


static const per_sequence_t SysInfoType15_5_v770ext_IEs_sequence[] = {
  { &hf_rrc_ue_Positioning_OTDOA_AssistanceData_UEB_ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_Positioning_OTDOA_AssistanceData_UEB_ext },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_SysInfoType15_5_v770ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_SysInfoType15_5_v770ext_IEs, SysInfoType15_5_v770ext_IEs_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions_225_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_nonCriticalExtensions_225(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_nonCriticalExtensions_225, T_nonCriticalExtensions_225_sequence);

  return offset;
}


static const per_sequence_t T_v770NonCriticalExtensions_45_sequence[] = {
  { &hf_rrc_sysInfoType15_5_v770ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SysInfoType15_5_v770ext_IEs },
  { &hf_rrc_nonCriticalExtensions_225, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_nonCriticalExtensions_225 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v770NonCriticalExtensions_45(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v770NonCriticalExtensions_45, T_v770NonCriticalExtensions_45_sequence);

  return offset;
}


static const per_sequence_t T_v3a0NonCriticalExtensions_15_sequence[] = {
  { &hf_rrc_sysInfoType15_5_v3a0ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SysInfoType15_5_v3a0ext },
  { &hf_rrc_v770NonCriticalExtensions_45, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v770NonCriticalExtensions_45 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v3a0NonCriticalExtensions_15(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v3a0NonCriticalExtensions_15, T_v3a0NonCriticalExtensions_15_sequence);

  return offset;
}


static const per_sequence_t SysInfoType15_5_sequence[] = {
  { &hf_rrc_ue_positioning_OTDOA_AssistanceData_UEB, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UE_Positioning_OTDOA_AssistanceData_UEB },
  { &hf_rrc_v3a0NonCriticalExtensions_15, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v3a0NonCriticalExtensions_15 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_SysInfoType15_5(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_SysInfoType15_5, SysInfoType15_5_sequence);

  return offset;
}


static const per_sequence_t SysInfoType15_6_va40ext_IEs_sequence[] = {
  { &hf_rrc_ue_positioning_GANSS_ReferenceMeasurementInformation_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UE_Positioning_GANSS_ReferenceMeasurementInfo_va40ext },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_SysInfoType15_6_va40ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_SysInfoType15_6_va40ext_IEs, SysInfoType15_6_va40ext_IEs_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions_226_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_nonCriticalExtensions_226(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_nonCriticalExtensions_226, T_nonCriticalExtensions_226_sequence);

  return offset;
}


static const per_sequence_t T_va40NonCriticalExtensions_24_sequence[] = {
  { &hf_rrc_sysInfoType15_6_va40ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SysInfoType15_6_va40ext_IEs },
  { &hf_rrc_nonCriticalExtensions_226, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_nonCriticalExtensions_226 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_va40NonCriticalExtensions_24(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_va40NonCriticalExtensions_24, T_va40NonCriticalExtensions_24_sequence);

  return offset;
}


static const per_sequence_t SysInfoType15_6_sequence[] = {
  { &hf_rrc_ue_positioning_GANSS_TOD, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_0_86399 },
  { &hf_rrc_ue_positioning_GANSS_ReferenceMeasurementInformation, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UE_Positioning_GANSS_ReferenceMeasurementInfo },
  { &hf_rrc_va40NonCriticalExtensions_25, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_va40NonCriticalExtensions_24 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_SysInfoType15_6(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_SysInfoType15_6, SysInfoType15_6_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions_227_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_nonCriticalExtensions_227(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_nonCriticalExtensions_227, T_nonCriticalExtensions_227_sequence);

  return offset;
}


static const per_sequence_t SysInfoType15_7_sequence[] = {
  { &hf_rrc_ue_positioning_GANSS_DataBitAssistance, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UE_Positioning_GANSS_Data_Bit_Assistance },
  { &hf_rrc_nonCriticalExtensions_227, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_nonCriticalExtensions_227 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_SysInfoType15_7(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_SysInfoType15_7, SysInfoType15_7_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions_228_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_nonCriticalExtensions_228(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_nonCriticalExtensions_228, T_nonCriticalExtensions_228_sequence);

  return offset;
}


static const per_sequence_t SysInfoType15_8_sequence[] = {
  { &hf_rrc_ue_positioning_GANSS_DataCipheringInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_Positioning_CipherParameters },
  { &hf_rrc_ue_positioning_GANSS_realTimeIntegrity, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_Positioning_GANSS_RealTimeIntegrity },
  { &hf_rrc_nonCriticalExtensions_228, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_nonCriticalExtensions_228 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_SysInfoType15_8(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_SysInfoType15_8, SysInfoType15_8_sequence);

  return offset;
}


static const per_sequence_t SysInfoType16_v770ext_IEs_sequence[] = {
  { &hf_rrc_preDefinedRadioConfiguration_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PreDefRadioConfiguration_v770ext },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_SysInfoType16_v770ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_SysInfoType16_v770ext_IEs, SysInfoType16_v770ext_IEs_sequence);

  return offset;
}


static const per_sequence_t SysInfoType16_v920ext_IEs_sequence[] = {
  { &hf_rrc_preDefinedRadioConfiguration_02, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PreDefRadioConfiguration_v920ext },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_SysInfoType16_v920ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_SysInfoType16_v920ext_IEs, SysInfoType16_v920ext_IEs_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions_229_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_nonCriticalExtensions_229(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_nonCriticalExtensions_229, T_nonCriticalExtensions_229_sequence);

  return offset;
}


static const per_sequence_t T_v920NonCriticalExtensions_06_sequence[] = {
  { &hf_rrc_sysInfoType16_v920ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SysInfoType16_v920ext_IEs },
  { &hf_rrc_nonCriticalExtensions_229, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_nonCriticalExtensions_229 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v920NonCriticalExtensions_06(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v920NonCriticalExtensions_06, T_v920NonCriticalExtensions_06_sequence);

  return offset;
}


static const per_sequence_t T_v770NonCriticalExtensions_46_sequence[] = {
  { &hf_rrc_sysInfoType16_v770ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SysInfoType16_v770ext_IEs },
  { &hf_rrc_v920NonCriticalExtensions_06, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v920NonCriticalExtensions_06 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v770NonCriticalExtensions_46(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v770NonCriticalExtensions_46, T_v770NonCriticalExtensions_46_sequence);

  return offset;
}


static const per_sequence_t SysInfoType16_sequence[] = {
  { &hf_rrc_preDefinedRadioConfiguration, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PreDefRadioConfiguration },
  { &hf_rrc_v770NonCriticalExtensions_46, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v770NonCriticalExtensions_46 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_SysInfoType16(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_SysInfoType16, SysInfoType16_sequence);

  return offset;
}


static const per_sequence_t T_tdd128SpecificInfo_02_sequence[] = {
  { &hf_rrc_pusch_SysInfoList_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_PUSCH_SysInfoList_LCR_r4 },
  { &hf_rrc_pdsch_SysInfoList_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_PDSCH_SysInfoList_LCR_r4 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_tdd128SpecificInfo_02(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_tdd128SpecificInfo_02, T_tdd128SpecificInfo_02_sequence);

  return offset;
}


static const per_sequence_t SysInfoType17_v4b0ext_IEs_sequence[] = {
  { &hf_rrc_tdd128SpecificInfo_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_tdd128SpecificInfo_02 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_SysInfoType17_v4b0ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_SysInfoType17_v4b0ext_IEs, SysInfoType17_v4b0ext_IEs_sequence);

  return offset;
}


static const per_sequence_t T_hcr_r5_SpecificInfo_02_sequence[] = {
  { &hf_rrc_pusch_SysInfoList_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_PUSCH_SysInfoList_HCR_r5 },
  { &hf_rrc_pdsch_SysInfoList_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_PDSCH_SysInfoList_HCR_r5 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_hcr_r5_SpecificInfo_02(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_hcr_r5_SpecificInfo_02, T_hcr_r5_SpecificInfo_02_sequence);

  return offset;
}


static const per_sequence_t SysInfoType17_v590ext_IEs_sequence[] = {
  { &hf_rrc_hcr_r5_SpecificInfo_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_hcr_r5_SpecificInfo_02 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_SysInfoType17_v590ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_SysInfoType17_v590ext_IEs, SysInfoType17_v590ext_IEs_sequence);

  return offset;
}


static const per_sequence_t T_tdd768SpecificInfo_01_sequence[] = {
  { &hf_rrc_pusch_SysInfoList_SFN_03, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_PUSCH_SysInfoList_SFN_VHCR },
  { &hf_rrc_pdsch_SysInfoList_SFN_03, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_PDSCH_SysInfoList_VHCR_r7 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_tdd768SpecificInfo_01(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_tdd768SpecificInfo_01, T_tdd768SpecificInfo_01_sequence);

  return offset;
}


static const per_sequence_t SysInfoType17_v770ext_IEs_sequence[] = {
  { &hf_rrc_tdd768SpecificInfo_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_tdd768SpecificInfo_01 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_SysInfoType17_v770ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_SysInfoType17_v770ext_IEs, SysInfoType17_v770ext_IEs_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions_230_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_nonCriticalExtensions_230(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_nonCriticalExtensions_230, T_nonCriticalExtensions_230_sequence);

  return offset;
}


static const per_sequence_t T_v770NonCriticalExtensions_47_sequence[] = {
  { &hf_rrc_sysInfoType17_v770ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SysInfoType17_v770ext_IEs },
  { &hf_rrc_nonCriticalExtensions_230, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_nonCriticalExtensions_230 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v770NonCriticalExtensions_47(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v770NonCriticalExtensions_47, T_v770NonCriticalExtensions_47_sequence);

  return offset;
}


static const per_sequence_t T_v590NonCriticalExtensions_30_sequence[] = {
  { &hf_rrc_sysInfoType17_v590ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_SysInfoType17_v590ext_IEs },
  { &hf_rrc_v770NonCriticalExtensions_47, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v770NonCriticalExtensions_47 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v590NonCriticalExtensions_30(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v590NonCriticalExtensions_30, T_v590NonCriticalExtensions_30_sequence);

  return offset;
}


static const per_sequence_t T_v4b0NonCriticalExtensions_26_sequence[] = {
  { &hf_rrc_sysInfoType17_v4b0ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SysInfoType17_v4b0ext_IEs },
  { &hf_rrc_v590NonCriticalExtensions_30, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v590NonCriticalExtensions_30 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v4b0NonCriticalExtensions_26(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v4b0NonCriticalExtensions_26, T_v4b0NonCriticalExtensions_26_sequence);

  return offset;
}


static const per_sequence_t SysInfoType17_sequence[] = {
  { &hf_rrc_pusch_SysInfoList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_PUSCH_SysInfoList },
  { &hf_rrc_pdsch_SysInfoList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_PDSCH_SysInfoList },
  { &hf_rrc_v4b0NonCriticalExtensions_26, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v4b0NonCriticalExtensions_26 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_SysInfoType17(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_SysInfoType17, SysInfoType17_sequence);

  return offset;
}


static const per_sequence_t SysInfoType18_v6b0ext_sequence[] = {
  { &hf_rrc_idleModePLMNIdentitiesSIB11bis, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_PLMNIdentitiesOfNeighbourCells },
  { &hf_rrc_connectedModePLMNIdentitiesSIB11bis, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_PLMNIdentitiesOfNeighbourCells },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_SysInfoType18_v6b0ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_SysInfoType18_v6b0ext, SysInfoType18_v6b0ext_sequence);

  return offset;
}


static const per_sequence_t SysInfoType18_v860ext_sequence[] = {
  { &hf_rrc_idleModePLMNIdentities_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_PLMNIdentitiesOfNeighbourCells_v860ext },
  { &hf_rrc_connectedModePLMNIdentities_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_PLMNIdentitiesOfNeighbourCells_v860ext },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_SysInfoType18_v860ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_SysInfoType18_v860ext, SysInfoType18_v860ext_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions_231_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_nonCriticalExtensions_231(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_nonCriticalExtensions_231, T_nonCriticalExtensions_231_sequence);

  return offset;
}


static const per_sequence_t T_v860NonCriticalExtensions_32_sequence[] = {
  { &hf_rrc_sysInfoType18_v860ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SysInfoType18_v860ext },
  { &hf_rrc_nonCriticalExtensions_231, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_nonCriticalExtensions_231 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v860NonCriticalExtensions_32(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v860NonCriticalExtensions_32, T_v860NonCriticalExtensions_32_sequence);

  return offset;
}


static const per_sequence_t T_v6b0NonCriticalExtensions_18_sequence[] = {
  { &hf_rrc_sysInfoType18_v6b0ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SysInfoType18_v6b0ext },
  { &hf_rrc_v860NonCriticalExtensions_32, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v860NonCriticalExtensions_32 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v6b0NonCriticalExtensions_18(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v6b0NonCriticalExtensions_18, T_v6b0NonCriticalExtensions_18_sequence);

  return offset;
}


static const per_sequence_t SysInfoType18_sequence[] = {
  { &hf_rrc_idleModePLMNIdentities, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_PLMNIdentitiesOfNeighbourCells },
  { &hf_rrc_connectedModePLMNIdentities, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_PLMNIdentitiesOfNeighbourCells },
  { &hf_rrc_v6b0NonCriticalExtensions_19, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v6b0NonCriticalExtensions_18 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_SysInfoType18(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_SysInfoType18, SysInfoType18_sequence);

  return offset;
}


static const per_sequence_t SysInfoType19_v920ext_sequence[] = {
  { &hf_rrc_utra_PriorityInfoList_v920ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UTRA_PriorityInfoList_v920ext },
  { &hf_rrc_eutra_FrequencyAndPriorityInfoList_v920ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_EUTRA_FrequencyAndPriorityInfoList_v920ext },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_SysInfoType19_v920ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_SysInfoType19_v920ext, SysInfoType19_v920ext_sequence);

  return offset;
}


static const per_sequence_t SysInfoType19_va80ext_sequence[] = {
  { &hf_rrc_multipleEutraFrequencyInfoList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MultipleEUTRAFrequencyInfoList },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_SysInfoType19_va80ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_SysInfoType19_va80ext, SysInfoType19_va80ext_sequence);

  return offset;
}


static const value_string rrc_T_cell_fach_meas_ind_vals[] = {
  {   0, "high-priority-layers" },
  {   1, "all-layers" },
  { 0, NULL }
};


static int
dissect_rrc_T_cell_fach_meas_ind(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     2, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t SysInfoType19_vb30ext_sequence[] = {
  { &hf_rrc_cell_fach_meas_ind, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_cell_fach_meas_ind },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_SysInfoType19_vb30ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_SysInfoType19_vb30ext, SysInfoType19_vb30ext_sequence);

  return offset;
}


static const per_sequence_t SysInfoType19_vbxyext_sequence[] = {
  { &hf_rrc_gsm_PriorityInfoList_vbxyext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_GSM_PriorityInfoList_vbxyext },
  { &hf_rrc_eutra_FrequencyAndPriorityInfoList_vbxyext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_EUTRA_FrequencyAndPriorityInfoList_vbxyext },
  { &hf_rrc_eutra_FrequencyRACHReportingInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_EUTRA_FrequencyRACHReportingInfo },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_SysInfoType19_vbxyext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_SysInfoType19_vbxyext, SysInfoType19_vbxyext_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions_232_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_nonCriticalExtensions_232(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_nonCriticalExtensions_232, T_nonCriticalExtensions_232_sequence);

  return offset;
}


static const per_sequence_t T_vbxyNonCriticalExtensions_19_sequence[] = {
  { &hf_rrc_sysInfoType19_vbxyext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SysInfoType19_vbxyext },
  { &hf_rrc_nonCriticalExtensions_232, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_nonCriticalExtensions_232 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_vbxyNonCriticalExtensions_19(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_vbxyNonCriticalExtensions_19, T_vbxyNonCriticalExtensions_19_sequence);

  return offset;
}


static const per_sequence_t T_vb30NonCriticalExtensions_sequence[] = {
  { &hf_rrc_sysInfoType19_vb30ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SysInfoType19_vb30ext },
  { &hf_rrc_vbxyNonCriticalExtensions_20, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_vbxyNonCriticalExtensions_19 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_vb30NonCriticalExtensions(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_vb30NonCriticalExtensions, T_vb30NonCriticalExtensions_sequence);

  return offset;
}


static const per_sequence_t T_va80NonCriticalExtensions_07_sequence[] = {
  { &hf_rrc_sysInfoType19_va80ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SysInfoType19_va80ext },
  { &hf_rrc_vb30NonCriticalExtensions, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_vb30NonCriticalExtensions },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_va80NonCriticalExtensions_07(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_va80NonCriticalExtensions_07, T_va80NonCriticalExtensions_07_sequence);

  return offset;
}


static const per_sequence_t T_v920NonCriticalExtensions_07_sequence[] = {
  { &hf_rrc_sysInfoType19_v920ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SysInfoType19_v920ext },
  { &hf_rrc_va80NonCriticalExtensions_07, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_va80NonCriticalExtensions_07 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v920NonCriticalExtensions_07(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v920NonCriticalExtensions_07, T_v920NonCriticalExtensions_07_sequence);

  return offset;
}


static const per_sequence_t SysInfoType19_sequence[] = {
  { &hf_rrc_utra_PriorityInfoList, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UTRA_PriorityInfoList },
  { &hf_rrc_gsm_PriorityInfoList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_GSM_PriorityInfoList },
  { &hf_rrc_eutra_FrequencyAndPriorityInfoList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_EUTRA_FrequencyAndPriorityInfoList },
  { &hf_rrc_v920NonCriticalExtensions_07, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v920NonCriticalExtensions_07 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_SysInfoType19(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_SysInfoType19, SysInfoType19_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions_233_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_nonCriticalExtensions_233(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_nonCriticalExtensions_233, T_nonCriticalExtensions_233_sequence);

  return offset;
}


static const per_sequence_t SysInfoType20_sequence[] = {
  { &hf_rrc_hNBName         , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_HNBName },
  { &hf_rrc_nonCriticalExtensions_233, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_nonCriticalExtensions_233 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_SysInfoType20(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_SysInfoType20, SysInfoType20_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions_234_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_nonCriticalExtensions_234(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_nonCriticalExtensions_234, T_nonCriticalExtensions_234_sequence);

  return offset;
}


static const per_sequence_t SysInfoType21_sequence[] = {
  { &hf_rrc_eab_Parameters  , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_EAB_Parameters },
  { &hf_rrc_nonCriticalExtensions_234, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_nonCriticalExtensions_234 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_SysInfoType21(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_SysInfoType21, SysInfoType21_sequence);

  return offset;
}


static const per_sequence_t T_fdd_190_sequence[] = {
  { &hf_rrc_prach_PreambleForEnhancedUplinkListType1Ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_PRACH_PreambleForEnhancedUplinkListType1Ext },
  { &hf_rrc_prach_PreambleForEnhancedUplinkListType2Ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_PRACH_PreambleForEnhancedUplinkListType2Ext },
  { &hf_rrc_prach_PreambleForEnhancedUplinkListType3Ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_PRACH_PreambleForEnhancedUplinkListType3Ext },
  { &hf_rrc_concurrent_Deployment_2ms_10ms_TTI, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_Concurrent_Deployment_2ms_10ms_TTI },
  { &hf_rrc_nodeB_Trigger_HS_DPCCH_Transmission, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_NodeB_Trigger_HS_DPCCH_Transmission },
  { &hf_rrc_fallback_R99_PRACH_info, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_Fallback_R99_PRACH_info_IEs },
  { &hf_rrc_commonEDCHResourceConfigInfoListExt, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CommonEDCHResourceConfigInfoListExt },
  { &hf_rrc_hs_dsch_DrxCellfach_SecondDrx_info, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_HS_DSCH_DrxCellfach_SecondDrx_info },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_fdd_190(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_fdd_190, T_fdd_190_sequence);

  return offset;
}


static const value_string rrc_T_modeSpecificInfo_155_vals[] = {
  {   0, "fdd" },
  {   1, "tdd" },
  { 0, NULL }
};

static const per_choice_t T_modeSpecificInfo_155_choice[] = {
  {   0, &hf_rrc_fdd_196         , ASN1_NO_EXTENSIONS     , dissect_rrc_T_fdd_190 },
  {   1, &hf_rrc_tdd             , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_modeSpecificInfo_155(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_modeSpecificInfo_155, T_modeSpecificInfo_155_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions_235_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_nonCriticalExtensions_235(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_nonCriticalExtensions_235, T_nonCriticalExtensions_235_sequence);

  return offset;
}


static const per_sequence_t SysInfoType22_sequence[] = {
  { &hf_rrc_modeSpecificInfo_155, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_modeSpecificInfo_155 },
  { &hf_rrc_nonCriticalExtensions_235, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_nonCriticalExtensions_235 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_SysInfoType22(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_SysInfoType22, SysInfoType22_sequence);

  return offset;
}


static const per_sequence_t SysInfoTypeSB1_v6b0ext_sequence[] = {
  { &hf_rrc_extSIBTypeInfoSchedulingInfo_List, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_ExtSIBTypeInfoSchedulingInfo_List },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_SysInfoTypeSB1_v6b0ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_SysInfoTypeSB1_v6b0ext, SysInfoTypeSB1_v6b0ext_sequence);

  return offset;
}


static const per_sequence_t SysInfoTypeSB1_v860ext_sequence[] = {
  { &hf_rrc_extSIBTypeInfoSchedulingInfo_List_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_ExtSIBTypeInfoSchedulingInfo_List2 },
  { &hf_rrc_extGANSS_SIBTypeInfoSchedulingInfoList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_ExtGANSS_SIBTypeInfoSchedulingInfoList },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_SysInfoTypeSB1_v860ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_SysInfoTypeSB1_v860ext, SysInfoTypeSB1_v860ext_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions_236_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_nonCriticalExtensions_236(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_nonCriticalExtensions_236, T_nonCriticalExtensions_236_sequence);

  return offset;
}


static const per_sequence_t T_v860NonCriticalExtensions_33_sequence[] = {
  { &hf_rrc_sysInfoTypeSB1_v860ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SysInfoTypeSB1_v860ext },
  { &hf_rrc_nonCriticalExtensions_236, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_nonCriticalExtensions_236 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v860NonCriticalExtensions_33(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v860NonCriticalExtensions_33, T_v860NonCriticalExtensions_33_sequence);

  return offset;
}


static const per_sequence_t T_v6b0NonCriticalExtensions_19_sequence[] = {
  { &hf_rrc_sysInfoTypeSB1_v6b0ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SysInfoTypeSB1_v6b0ext },
  { &hf_rrc_v860NonCriticalExtensions_33, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v860NonCriticalExtensions_33 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v6b0NonCriticalExtensions_19(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v6b0NonCriticalExtensions_19, T_v6b0NonCriticalExtensions_19_sequence);

  return offset;
}


static const per_sequence_t SysInfoTypeSB1_sequence[] = {
  { &hf_rrc_sib_ReferenceList, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SIB_ReferenceList },
  { &hf_rrc_v6b0NonCriticalExtensions_20, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v6b0NonCriticalExtensions_19 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_SysInfoTypeSB1(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_SysInfoTypeSB1, SysInfoTypeSB1_sequence);

  return offset;
}


static const per_sequence_t SysInfoTypeSB2_v6b0ext_sequence[] = {
  { &hf_rrc_extSIBTypeInfoSchedulingInfo_List, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_ExtSIBTypeInfoSchedulingInfo_List },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_SysInfoTypeSB2_v6b0ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_SysInfoTypeSB2_v6b0ext, SysInfoTypeSB2_v6b0ext_sequence);

  return offset;
}


static const per_sequence_t SysInfoTypeSB2_v860ext_sequence[] = {
  { &hf_rrc_extSIBTypeInfoSchedulingInfo_List_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_ExtSIBTypeInfoSchedulingInfo_List2 },
  { &hf_rrc_extGANSS_SIBTypeInfoSchedulingInfoList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_ExtGANSS_SIBTypeInfoSchedulingInfoList },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_SysInfoTypeSB2_v860ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_SysInfoTypeSB2_v860ext, SysInfoTypeSB2_v860ext_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions_237_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_nonCriticalExtensions_237(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_nonCriticalExtensions_237, T_nonCriticalExtensions_237_sequence);

  return offset;
}


static const per_sequence_t T_v860NonCriticalExtensions_34_sequence[] = {
  { &hf_rrc_sysInfoTypeSB2_v860ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SysInfoTypeSB2_v860ext },
  { &hf_rrc_nonCriticalExtensions_237, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_nonCriticalExtensions_237 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v860NonCriticalExtensions_34(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v860NonCriticalExtensions_34, T_v860NonCriticalExtensions_34_sequence);

  return offset;
}


static const per_sequence_t T_v6b0NonCriticalExtensions_20_sequence[] = {
  { &hf_rrc_sysInfoTypeSB2_v6b0ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SysInfoTypeSB2_v6b0ext },
  { &hf_rrc_v860NonCriticalExtensions_34, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v860NonCriticalExtensions_34 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v6b0NonCriticalExtensions_20(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v6b0NonCriticalExtensions_20, T_v6b0NonCriticalExtensions_20_sequence);

  return offset;
}


static const per_sequence_t SysInfoTypeSB2_sequence[] = {
  { &hf_rrc_sib_ReferenceList, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SIB_ReferenceList },
  { &hf_rrc_v6b0NonCriticalExtensions_21, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v6b0NonCriticalExtensions_20 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_SysInfoTypeSB2(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_SysInfoTypeSB2, SysInfoTypeSB2_sequence);

  return offset;
}


static const per_sequence_t UE_HistoryInformation_sequence[] = {
  { &hf_rrc_ue_InactivityPeriod, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_1_120 },
  { &hf_rrc_ueMobilityStateIndicator, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_High_MobilityDetected },
  { &hf_rrc_ul_dataVolumeHistory, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DataVolumeHistory },
  { &hf_rrc_dl_dataVolumeHistory, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DataVolumeHistory },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UE_HistoryInformation(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UE_HistoryInformation, UE_HistoryInformation_sequence);

  return offset;
}



static int
dissect_rrc_T_interRATHandoverInfo(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
#line 595 "../../asn1/rrc/rrc.cnf"
  tvbuff_t *parameter_tvb=NULL;

  offset = dissect_per_octet_string(tvb, offset, actx, tree, hf_index,
                                       0, 255, FALSE, &parameter_tvb);

  if (parameter_tvb){
    dissect_rrc_InterRATHandoverInfo_PDU(parameter_tvb, actx->pinfo, tree, NULL);
  }



  return offset;
}


static const per_sequence_t InterRATHandoverInfoWithInterRATCapabilities_r3_IEs_sequence[] = {
  { &hf_rrc_ue_RATSpecificCapability, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_InterRAT_UE_RadioAccessCapabilityList },
  { &hf_rrc_interRATHandoverInfo_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_interRATHandoverInfo },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_InterRATHandoverInfoWithInterRATCapabilities_r3_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_InterRATHandoverInfoWithInterRATCapabilities_r3_IEs, InterRATHandoverInfoWithInterRATCapabilities_r3_IEs_sequence);

  return offset;
}


static const per_sequence_t InterRATHandoverInfoWithInterRATCapabilities_v390ext_IEs_sequence[] = {
  { &hf_rrc_failureCauseWithProtErr, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_FailureCauseWithProtErr },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_InterRATHandoverInfoWithInterRATCapabilities_v390ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_InterRATHandoverInfoWithInterRATCapabilities_v390ext_IEs, InterRATHandoverInfoWithInterRATCapabilities_v390ext_IEs_sequence);

  return offset;
}


static const per_sequence_t InterRATHandoverInfoWithInterRATCapabilities_v690ext_IEs_sequence[] = {
  { &hf_rrc_ue_RATSpecificCapability_v690ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_InterRAT_UE_RadioAccessCapability_v690ext },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_InterRATHandoverInfoWithInterRATCapabilities_v690ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_InterRATHandoverInfoWithInterRATCapabilities_v690ext_IEs, InterRATHandoverInfoWithInterRATCapabilities_v690ext_IEs_sequence);

  return offset;
}


static const per_sequence_t InterRATHandoverInfoWithInterRATCapabilities_v860ext_IEs_sequence[] = {
  { &hf_rrc_ue_RATSpecificCapability_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_InterRAT_UE_RadioAccessCapability_v860ext },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_InterRATHandoverInfoWithInterRATCapabilities_v860ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_InterRATHandoverInfoWithInterRATCapabilities_v860ext_IEs, InterRATHandoverInfoWithInterRATCapabilities_v860ext_IEs_sequence);

  return offset;
}


static const per_sequence_t InterRATHandoverInfoWithInterRATCapabilities_v920ext_IEs_sequence[] = {
  { &hf_rrc_ue_Inactivity_Period, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_INTEGER_1_120 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_InterRATHandoverInfoWithInterRATCapabilities_v920ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_InterRATHandoverInfoWithInterRATCapabilities_v920ext_IEs, InterRATHandoverInfoWithInterRATCapabilities_v920ext_IEs_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions_238_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_nonCriticalExtensions_238(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_nonCriticalExtensions_238, T_nonCriticalExtensions_238_sequence);

  return offset;
}


static const per_sequence_t T_v920NonCriticalExtensions_08_sequence[] = {
  { &hf_rrc_interRATHandoverInfoWithInterRATCapabilities_v920ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_InterRATHandoverInfoWithInterRATCapabilities_v920ext_IEs },
  { &hf_rrc_nonCriticalExtensions_238, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_nonCriticalExtensions_238 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v920NonCriticalExtensions_08(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v920NonCriticalExtensions_08, T_v920NonCriticalExtensions_08_sequence);

  return offset;
}


static const per_sequence_t T_v860NonCriticalExtensions_35_sequence[] = {
  { &hf_rrc_interRATHandoverInfoWithInterRATCapabilities_v860ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_InterRATHandoverInfoWithInterRATCapabilities_v860ext_IEs },
  { &hf_rrc_v920NonCriticalExtensions_08, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v920NonCriticalExtensions_08 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v860NonCriticalExtensions_35(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v860NonCriticalExtensions_35, T_v860NonCriticalExtensions_35_sequence);

  return offset;
}


static const per_sequence_t T_v690NonCriticalExtensions_52_sequence[] = {
  { &hf_rrc_interRATHandoverInfoWithInterRATCapabilities_v690ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_InterRATHandoverInfoWithInterRATCapabilities_v690ext_IEs },
  { &hf_rrc_v860NonCriticalExtensions_35, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v860NonCriticalExtensions_35 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v690NonCriticalExtensions_52(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v690NonCriticalExtensions_52, T_v690NonCriticalExtensions_52_sequence);

  return offset;
}


static const per_sequence_t T_v390NonCriticalExtensions_01_sequence[] = {
  { &hf_rrc_interRATHandoverInfoWithInterRATCapabilities_v390ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_InterRATHandoverInfoWithInterRATCapabilities_v390ext_IEs },
  { &hf_rrc_v690NonCriticalExtensions_52, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v690NonCriticalExtensions_52 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v390NonCriticalExtensions_01(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v390NonCriticalExtensions_01, T_v390NonCriticalExtensions_01_sequence);

  return offset;
}


static const per_sequence_t T_r3_31_sequence[] = {
  { &hf_rrc_interRATHandoverInfo_r3, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_InterRATHandoverInfoWithInterRATCapabilities_r3_IEs },
  { &hf_rrc_v390NonCriticalExtensions_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v390NonCriticalExtensions_01 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_r3_31(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_r3_31, T_r3_31_sequence);

  return offset;
}


static const per_sequence_t T_criticalExtensions_139_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_criticalExtensions_139(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_criticalExtensions_139, T_criticalExtensions_139_sequence);

  return offset;
}


static const value_string rrc_InterRATHandoverInfoWithInterRATCapabilities_r3_vals[] = {
  {   0, "r3" },
  {   1, "criticalExtensions" },
  { 0, NULL }
};

static const per_choice_t InterRATHandoverInfoWithInterRATCapabilities_r3_choice[] = {
  {   0, &hf_rrc_r3_31           , ASN1_NO_EXTENSIONS     , dissect_rrc_T_r3_31 },
  {   1, &hf_rrc_criticalExtensions_140, ASN1_NO_EXTENSIONS     , dissect_rrc_T_criticalExtensions_139 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_InterRATHandoverInfoWithInterRATCapabilities_r3(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_InterRATHandoverInfoWithInterRATCapabilities_r3, InterRATHandoverInfoWithInterRATCapabilities_r3_choice,
                                 NULL);

  return offset;
}


static const value_string rrc_StateOfRRC_vals[] = {
  {   0, "cell-DCH" },
  {   1, "cell-FACH" },
  {   2, "cell-PCH" },
  {   3, "ura-PCH" },
  { 0, NULL }
};


static int
dissect_rrc_StateOfRRC(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     4, NULL, FALSE, 0, NULL);

  return offset;
}


static const value_string rrc_StateOfRRC_Procedure_vals[] = {
  {   0, "awaitNoRRC-Message" },
  {   1, "awaitRB-ReleaseComplete" },
  {   2, "awaitRB-SetupComplete" },
  {   3, "awaitRB-ReconfigurationComplete" },
  {   4, "awaitTransportCH-ReconfigurationComplete" },
  {   5, "awaitPhysicalCH-ReconfigurationComplete" },
  {   6, "awaitActiveSetUpdateComplete" },
  {   7, "awaitHandoverComplete" },
  {   8, "sendCellUpdateConfirm" },
  {   9, "sendUraUpdateConfirm" },
  {  10, "dummy" },
  {  11, "otherStates" },
  { 0, NULL }
};


static int
dissect_rrc_StateOfRRC_Procedure(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     12, NULL, FALSE, 0, NULL);

  return offset;
}


static const value_string rrc_CipheringStatus_vals[] = {
  {   0, "started" },
  {   1, "notStarted" },
  { 0, NULL }
};


static int
dissect_rrc_CipheringStatus(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     2, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t CalculationTimeForCiphering_sequence[] = {
  { &hf_rrc_cell_Id         , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CellIdentity },
  { &hf_rrc_sfn             , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_0_4095 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_CalculationTimeForCiphering(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_CalculationTimeForCiphering, CalculationTimeForCiphering_sequence);

  return offset;
}



static int
dissect_rrc_BIT_STRING_SIZE_20_25(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_bit_string(tvb, offset, actx, tree, hf_index,
                                     20, 25, FALSE, NULL);

  return offset;
}


static const per_sequence_t CipheringInfoPerRB_sequence[] = {
  { &hf_rrc_dl_HFN          , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_20_25 },
  { &hf_rrc_ul_HFN          , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_20_25 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_CipheringInfoPerRB(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_CipheringInfoPerRB, CipheringInfoPerRB_sequence);

  return offset;
}


static const per_sequence_t CipheringInfoPerRB_List_sequence_of[1] = {
  { &hf_rrc_CipheringInfoPerRB_List_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CipheringInfoPerRB },
};

static int
dissect_rrc_CipheringInfoPerRB_List(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_CipheringInfoPerRB_List, CipheringInfoPerRB_List_sequence_of,
                                                  1, maxRB, FALSE);

  return offset;
}


static const per_sequence_t COUNT_CSingle_sequence[] = {
  { &hf_rrc_cn_DomainIdentity, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CN_DomainIdentity },
  { &hf_rrc_count_C         , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_32 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_COUNT_CSingle(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_COUNT_CSingle, COUNT_CSingle_sequence);

  return offset;
}


static const per_sequence_t COUNT_C_List_sequence_of[1] = {
  { &hf_rrc_COUNT_C_List_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_COUNT_CSingle },
};

static int
dissect_rrc_COUNT_C_List(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_COUNT_C_List, COUNT_C_List_sequence_of,
                                                  1, maxCNdomains, FALSE);

  return offset;
}


static const value_string rrc_IntegrityProtectionStatus_vals[] = {
  {   0, "started" },
  {   1, "notStarted" },
  { 0, NULL }
};


static int
dissect_rrc_IntegrityProtectionStatus(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     2, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t SRB_SpecificIntegrityProtInfo_sequence[] = {
  { &hf_rrc_ul_RRC_HFN      , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_28 },
  { &hf_rrc_dl_RRC_HFN      , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_28 },
  { &hf_rrc_ul_RRC_SequenceNumber, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RRC_MessageSequenceNumber },
  { &hf_rrc_dl_RRC_SequenceNumber, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RRC_MessageSequenceNumber },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_SRB_SpecificIntegrityProtInfo(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_SRB_SpecificIntegrityProtInfo, SRB_SpecificIntegrityProtInfo_sequence);

  return offset;
}


static const per_sequence_t SRB_SpecificIntegrityProtInfoList_sequence_of[1] = {
  { &hf_rrc_SRB_SpecificIntegrityProtInfoList_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SRB_SpecificIntegrityProtInfo },
};

static int
dissect_rrc_SRB_SpecificIntegrityProtInfoList(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_SRB_SpecificIntegrityProtInfoList, SRB_SpecificIntegrityProtInfoList_sequence_of,
                                                  4, maxSRBsetup, FALSE);

  return offset;
}



static int
dissect_rrc_ImplementationSpecificParams(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_bit_string(tvb, offset, actx, tree, hf_index,
                                     1, 512, FALSE, NULL);

  return offset;
}


static const per_sequence_t UE_Positioning_LastKnownPos_sequence[] = {
  { &hf_rrc_sfn             , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_0_4095 },
  { &hf_rrc_cell_id         , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CellIdentity },
  { &hf_rrc_positionEstimate, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PositionEstimate },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UE_Positioning_LastKnownPos(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UE_Positioning_LastKnownPos, UE_Positioning_LastKnownPos_sequence);

  return offset;
}


static const value_string rrc_MeasurementCommandWithType_vals[] = {
  {   0, "setup" },
  {   1, "modify" },
  {   2, "release" },
  { 0, NULL }
};

static const per_choice_t MeasurementCommandWithType_choice[] = {
  {   0, &hf_rrc_setup           , ASN1_NO_EXTENSIONS     , dissect_rrc_MeasurementType },
  {   1, &hf_rrc_modify_09       , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   2, &hf_rrc_release_01      , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_MeasurementCommandWithType(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_MeasurementCommandWithType, MeasurementCommandWithType_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t OngoingMeasRep_sequence[] = {
  { &hf_rrc_measurementIdentity, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MeasurementIdentity },
  { &hf_rrc_measurementCommandWithType, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MeasurementCommandWithType },
  { &hf_rrc_measurementReportingMode, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MeasurementReportingMode },
  { &hf_rrc_additionalMeasurementID_List, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_AdditionalMeasurementID_List },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_OngoingMeasRep(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_OngoingMeasRep, OngoingMeasRep_sequence);

  return offset;
}


static const per_sequence_t OngoingMeasRepList_sequence_of[1] = {
  { &hf_rrc_OngoingMeasRepList_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_OngoingMeasRep },
};

static int
dissect_rrc_OngoingMeasRepList(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_OngoingMeasRepList, OngoingMeasRepList_sequence_of,
                                                  1, maxNoOfMeas, FALSE);

  return offset;
}


static const per_sequence_t T_fdd_194_sequence[] = {
  { &hf_rrc_dummy_05        , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CPCH_SetID },
  { &hf_rrc_dummy2_02       , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DRAC_StaticInformationList },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_fdd_194(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_fdd_194, T_fdd_194_sequence);

  return offset;
}


static const value_string rrc_T_modeSpecificInfo_156_vals[] = {
  {   0, "fdd" },
  {   1, "tdd" },
  { 0, NULL }
};

static const per_choice_t T_modeSpecificInfo_156_choice[] = {
  {   0, &hf_rrc_fdd_200         , ASN1_NO_EXTENSIONS     , dissect_rrc_T_fdd_194 },
  {   1, &hf_rrc_tdd             , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_modeSpecificInfo_156(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_modeSpecificInfo_156, T_modeSpecificInfo_156_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t SRNC_RelocationInfo_r3_IEs_sequence[] = {
  { &hf_rrc_stateOfRRC      , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_StateOfRRC },
  { &hf_rrc_stateOfRRC_Procedure, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_StateOfRRC_Procedure },
  { &hf_rrc_cipheringStatus , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CipheringStatus },
  { &hf_rrc_calculationTimeForCiphering, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CalculationTimeForCiphering },
  { &hf_rrc_cipheringInfoPerRB_List, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CipheringInfoPerRB_List },
  { &hf_rrc_count_C_List    , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_COUNT_C_List },
  { &hf_rrc_integrityProtectionStatus, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_IntegrityProtectionStatus },
  { &hf_rrc_srb_SpecificIntegrityProtInfo, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SRB_SpecificIntegrityProtInfoList },
  { &hf_rrc_implementationSpecificParams, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_ImplementationSpecificParams },
  { &hf_rrc_u_RNTI          , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_U_RNTI },
  { &hf_rrc_c_RNTI          , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_C_RNTI },
  { &hf_rrc_ue_RadioAccessCapability_02, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UE_RadioAccessCapability },
  { &hf_rrc_ue_Positioning_LastKnownPos, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_Positioning_LastKnownPos },
  { &hf_rrc_ue_RATSpecificCapability, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_InterRAT_UE_RadioAccessCapabilityList },
  { &hf_rrc_ura_Identity    , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_URA_Identity },
  { &hf_rrc_cn_CommonGSM_MAP_NAS_SysInfo_04, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_NAS_SystemInformationGSM_MAP },
  { &hf_rrc_cn_DomainInformationList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CN_DomainInformationList },
  { &hf_rrc_ongoingMeasRepList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_OngoingMeasRepList },
  { &hf_rrc_predefinedConfigStatusList_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PredefinedConfigStatusList },
  { &hf_rrc_srb_InformationList, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SRB_InformationSetupList },
  { &hf_rrc_rab_InformationList_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RAB_InformationSetupList },
  { &hf_rrc_ul_CommonTransChInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_CommonTransChInfo },
  { &hf_rrc_ul_TransChInfoList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_AddReconfTransChInfoList },
  { &hf_rrc_modeSpecificInfo_156, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_modeSpecificInfo_156 },
  { &hf_rrc_dl_CommonTransChInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_CommonTransChInfo },
  { &hf_rrc_dl_TransChInfoList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_AddReconfTransChInfoList },
  { &hf_rrc_measurementReport, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MeasurementReport },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_SRNC_RelocationInfo_r3_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_SRNC_RelocationInfo_r3_IEs, SRNC_RelocationInfo_r3_IEs_sequence);

  return offset;
}


static const per_sequence_t CipheringStatusCNdomain_sequence[] = {
  { &hf_rrc_cn_DomainIdentity, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CN_DomainIdentity },
  { &hf_rrc_cipheringStatus , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CipheringStatus },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_CipheringStatusCNdomain(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_CipheringStatusCNdomain, CipheringStatusCNdomain_sequence);

  return offset;
}


static const per_sequence_t CipheringStatusList_sequence_of[1] = {
  { &hf_rrc_CipheringStatusList_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CipheringStatusCNdomain },
};

static int
dissect_rrc_CipheringStatusList(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_CipheringStatusList, CipheringStatusList_sequence_of,
                                                  1, maxCNdomains, FALSE);

  return offset;
}


static const per_sequence_t SRNC_RelocationInfo_v380ext_IEs_sequence[] = {
  { &hf_rrc_cn_DomainIdentity, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CN_DomainIdentity },
  { &hf_rrc_cipheringStatusList, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CipheringStatusList },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_SRNC_RelocationInfo_v380ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_SRNC_RelocationInfo_v380ext_IEs, SRNC_RelocationInfo_v380ext_IEs_sequence);

  return offset;
}


static const per_sequence_t CN_DomainInformation_v390ext_sequence[] = {
  { &hf_rrc_cn_DRX_CycleLengthCoeff, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CN_DRX_CycleLengthCoefficient },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_CN_DomainInformation_v390ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_CN_DomainInformation_v390ext, CN_DomainInformation_v390ext_sequence);

  return offset;
}


static const per_sequence_t CN_DomainInformationList_v390ext_sequence_of[1] = {
  { &hf_rrc_CN_DomainInformationList_v390ext_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CN_DomainInformation_v390ext },
};

static int
dissect_rrc_CN_DomainInformationList_v390ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_CN_DomainInformationList_v390ext, CN_DomainInformationList_v390ext_sequence_of,
                                                  1, maxCNdomains, FALSE);

  return offset;
}


static const per_sequence_t SRNC_RelocationInfo_v390ext_IEs_sequence[] = {
  { &hf_rrc_cn_DomainInformationList_v390ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CN_DomainInformationList_v390ext },
  { &hf_rrc_ue_RadioAccessCapability_v370ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_RadioAccessCapability_v370ext },
  { &hf_rrc_ue_RadioAccessCapability_v380ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_RadioAccessCapability_v380ext },
  { &hf_rrc_dl_PhysChCapabilityFDD_v380ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DL_PhysChCapabilityFDD_v380ext },
  { &hf_rrc_failureCauseWithProtErr, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_FailureCauseWithProtErr },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_SRNC_RelocationInfo_v390ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_SRNC_RelocationInfo_v390ext_IEs, SRNC_RelocationInfo_v390ext_IEs_sequence);

  return offset;
}


static const per_sequence_t CipheringInfoPerRB_List_v3a0ext_sequence[] = {
  { &hf_rrc_dl_UM_SN        , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_7 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_CipheringInfoPerRB_List_v3a0ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_CipheringInfoPerRB_List_v3a0ext, CipheringInfoPerRB_List_v3a0ext_sequence);

  return offset;
}


static const per_sequence_t SRNC_RelocationInfo_v3a0ext_IEs_sequence[] = {
  { &hf_rrc_cipheringInfoForSRB1_v3a0ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CipheringInfoPerRB_List_v3a0ext },
  { &hf_rrc_ue_RadioAccessCapability_v3a0ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_RadioAccessCapability_v3a0ext },
  { &hf_rrc_startValueForCiphering_v3a0ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_START_Value },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_SRNC_RelocationInfo_v3a0ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_SRNC_RelocationInfo_v3a0ext_IEs, SRNC_RelocationInfo_v3a0ext_IEs_sequence);

  return offset;
}


static const per_sequence_t STARTList2_sequence_of[1] = {
  { &hf_rrc_STARTList2_item , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_STARTSingle },
};

static int
dissect_rrc_STARTList2(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_STARTList2, STARTList2_sequence_of,
                                                  2, maxCNdomains, FALSE);

  return offset;
}


static const per_sequence_t SRNC_RelocationInfo_v3b0ext_IEs_sequence[] = {
  { &hf_rrc_cn_DomainIdentity, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CN_DomainIdentity },
  { &hf_rrc_startValueForCiphering_v3b0ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_STARTList2 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_SRNC_RelocationInfo_v3b0ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_SRNC_RelocationInfo_v3b0ext_IEs, SRNC_RelocationInfo_v3b0ext_IEs_sequence);

  return offset;
}


static const per_sequence_t SRNC_RelocationInfo_v3c0ext_IEs_sequence[] = {
  { &hf_rrc_rb_IdentityForHOMessage, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RB_Identity },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_SRNC_RelocationInfo_v3c0ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_SRNC_RelocationInfo_v3c0ext_IEs, SRNC_RelocationInfo_v3c0ext_IEs_sequence);

  return offset;
}


static const per_sequence_t SRNC_RelocationInfo_v3d0ext_IEs_sequence[] = {
  { &hf_rrc_uESpecificBehaviourInformation1idle, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UESpecificBehaviourInformation1idle },
  { &hf_rrc_uESpecificBehaviourInformation1interRAT, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UESpecificBehaviourInformation1interRAT },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_SRNC_RelocationInfo_v3d0ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_SRNC_RelocationInfo_v3d0ext_IEs, SRNC_RelocationInfo_v3d0ext_IEs_sequence);

  return offset;
}



static int
dissect_rrc_T_sRNC_RelocationInfo_r3_add_ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_bit_string_containing_pdu_new(tvb, offset, actx, tree, hf_index,
                                                              NO_BOUND, NO_BOUND, FALSE, dissect_SRNC_RelocationInfo_v3h0ext_IEs_PDU);

  return offset;
}


static const per_sequence_t SRNC_RelocationInfo_v3g0ext_IEs_sequence[] = {
  { &hf_rrc_ue_RadioAccessCapability_v3g0ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_RadioAccessCapability_v3g0ext },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_SRNC_RelocationInfo_v3g0ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_SRNC_RelocationInfo_v3g0ext_IEs, SRNC_RelocationInfo_v3g0ext_IEs_sequence);

  return offset;
}


static const per_sequence_t SRNC_RelocationInfo_v4b0ext_IEs_sequence[] = {
  { &hf_rrc_ue_RadioAccessCapability_v4b0ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_RadioAccessCapability_v4b0ext },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_SRNC_RelocationInfo_v4b0ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_SRNC_RelocationInfo_v4b0ext_IEs, SRNC_RelocationInfo_v4b0ext_IEs_sequence);

  return offset;
}


static const per_sequence_t SRNC_RelocationInfo_v590ext_IEs_sequence[] = {
  { &hf_rrc_ue_RadioAccessCapability_v590ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_RadioAccessCapability_v590ext },
  { &hf_rrc_ue_RATSpecificCapability_v590ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_InterRAT_UE_RadioAccessCapability_v590ext },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_SRNC_RelocationInfo_v590ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_SRNC_RelocationInfo_v590ext_IEs, SRNC_RelocationInfo_v590ext_IEs_sequence);

  return offset;
}


static const per_sequence_t T_active_sequence[] = {
  { &hf_rrc_tgcfn           , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TGCFN },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_active(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_active, T_active_sequence);

  return offset;
}


static const value_string rrc_T_current_tgps_Status_vals[] = {
  {   0, "active" },
  {   1, "inactive" },
  { 0, NULL }
};

static const per_choice_t T_current_tgps_Status_choice[] = {
  {   0, &hf_rrc_active          , ASN1_NO_EXTENSIONS     , dissect_rrc_T_active },
  {   1, &hf_rrc_inactive        , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_current_tgps_Status(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_current_tgps_Status, T_current_tgps_Status_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t StoredTGP_Sequence_sequence[] = {
  { &hf_rrc_tgpsi           , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TGPSI },
  { &hf_rrc_current_tgps_Status, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_current_tgps_Status },
  { &hf_rrc_tgps_ConfigurationParams, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_TGPS_ConfigurationParams },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_StoredTGP_Sequence(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_StoredTGP_Sequence, StoredTGP_Sequence_sequence);

  return offset;
}


static const per_sequence_t StoredTGP_SequenceList_sequence_of[1] = {
  { &hf_rrc_StoredTGP_SequenceList_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_StoredTGP_Sequence },
};

static int
dissect_rrc_StoredTGP_SequenceList(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_StoredTGP_SequenceList, StoredTGP_SequenceList_sequence_of,
                                                  1, maxTGPS, FALSE);

  return offset;
}


static const per_sequence_t CodeChangeStatus_sequence[] = {
  { &hf_rrc_primaryCPICH_Info, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PrimaryCPICH_Info },
  { &hf_rrc_scramblingCodeChange, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_ScramblingCodeChange },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_CodeChangeStatus(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_CodeChangeStatus, CodeChangeStatus_sequence);

  return offset;
}


static const per_sequence_t CodeChangeStatusList_sequence_of[1] = {
  { &hf_rrc_CodeChangeStatusList_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CodeChangeStatus },
};

static int
dissect_rrc_CodeChangeStatusList(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_CodeChangeStatusList, CodeChangeStatusList_sequence_of,
                                                  1, maxRL, FALSE);

  return offset;
}


static const per_sequence_t StoredCompressedModeInfo_sequence[] = {
  { &hf_rrc_storedTGP_SequenceList, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_StoredTGP_SequenceList },
  { &hf_rrc_codeChangeStatusList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CodeChangeStatusList },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_StoredCompressedModeInfo(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_StoredCompressedModeInfo, StoredCompressedModeInfo_sequence);

  return offset;
}


static const per_sequence_t SRNC_RelocationInfo_v5a0ext_IEs_sequence[] = {
  { &hf_rrc_storedCompressedModeInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_StoredCompressedModeInfo },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_SRNC_RelocationInfo_v5a0ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_SRNC_RelocationInfo_v5a0ext_IEs, SRNC_RelocationInfo_v5a0ext_IEs_sequence);

  return offset;
}


static const per_sequence_t SRNC_RelocationInfo_v5b0ext_IEs_sequence[] = {
  { &hf_rrc_interRATCellInfoIndication, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_InterRATCellInfoIndication },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_SRNC_RelocationInfo_v5b0ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_SRNC_RelocationInfo_v5b0ext_IEs, SRNC_RelocationInfo_v5b0ext_IEs_sequence);

  return offset;
}


static const per_sequence_t SRNC_RelocationInfo_v5c0ext_IEs_sequence[] = {
  { &hf_rrc_ue_RadioAccessCapability_v5c0ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_RadioAccessCapability_v5c0ext },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_SRNC_RelocationInfo_v5c0ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_SRNC_RelocationInfo_v5c0ext_IEs, SRNC_RelocationInfo_v5c0ext_IEs_sequence);

  return offset;
}



static int
dissect_rrc_T_ueCapabilityContainer_02(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_bit_string_containing_pdu_new(tvb, offset, actx, tree, hf_index,
                                                              NO_BOUND, NO_BOUND, FALSE, dissect_UE_CapabilityContainer_IEs_PDU);

  return offset;
}



static int
dissect_rrc_T_ueCapabilityContainer_RSC(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_bit_string_containing_pdu_new(tvb, offset, actx, tree, hf_index,
                                                              NO_BOUND, NO_BOUND, FALSE, dissect_RRCConnectionSetupComplete_r3_add_ext_IEs_PDU);

  return offset;
}



static int
dissect_rrc_T_ueCapabilityContainer_UCI(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_bit_string_containing_pdu_new(tvb, offset, actx, tree, hf_index,
                                                              NO_BOUND, NO_BOUND, FALSE, dissect_UECapabilityInformation_r3_add_ext_IEs_PDU);

  return offset;
}


static const per_sequence_t IntraFreqReportingCriteria_r6_ext_sequence[] = {
  { &hf_rrc_event_06        , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_Event1j_r6 },
  { &hf_rrc_hysteresis_01   , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_Hysteresis },
  { &hf_rrc_timeToTrigger   , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TimeToTrigger },
  { &hf_rrc_reportingCellStatus, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_ReportingCellStatus },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_IntraFreqReportingCriteria_r6_ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_IntraFreqReportingCriteria_r6_ext, IntraFreqReportingCriteria_r6_ext_sequence);

  return offset;
}


static const per_sequence_t SRNC_RelocationInfo_v690ext_IEs_sequence[] = {
  { &hf_rrc_ueCapabilityContainer_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_ueCapabilityContainer_02 },
  { &hf_rrc_ueCapabilityContainer_RSC, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_ueCapabilityContainer_RSC },
  { &hf_rrc_ueCapabilityContainer_UCI, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_ueCapabilityContainer_UCI },
  { &hf_rrc_rab_InformationSetupList_08, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RAB_InformationSetupList_r6_ext },
  { &hf_rrc_measuredResultsOnRACHinterFreq, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MeasuredResultsOnRACHinterFreq },
  { &hf_rrc_mbms_JoinedInformation, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MBMS_JoinedInformation_r6 },
  { &hf_rrc_intraFreqReportingCriteria_08, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_IntraFreqReportingCriteria_r6_ext },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_SRNC_RelocationInfo_v690ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_SRNC_RelocationInfo_v690ext_IEs, SRNC_RelocationInfo_v690ext_IEs_sequence);

  return offset;
}


static const per_sequence_t SRNC_RelocationInfo_v6b0ext_IEs_sequence[] = {
  { &hf_rrc_rab_InformationSetupListExt, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RAB_InformationSetupList_v6b0ext },
  { &hf_rrc_mbmsSelectedServiceInfo, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MBMS_SelectedServiceInfo },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_SRNC_RelocationInfo_v6b0ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_SRNC_RelocationInfo_v6b0ext_IEs, SRNC_RelocationInfo_v6b0ext_IEs_sequence);

  return offset;
}


static const per_sequence_t SRNC_RelocationInfo_v770ext_IEs_sequence[] = {
  { &hf_rrc_thresholdSFN_GPS_TOW_us, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_ThresholdSFN_GPS_TOW_us },
  { &hf_rrc_srns_t_305      , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_305 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_SRNC_RelocationInfo_v770ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_SRNC_RelocationInfo_v770ext_IEs, SRNC_RelocationInfo_v770ext_IEs_sequence);

  return offset;
}


static const per_sequence_t SRNC_RelocationInfo_v7e0ext_IEs_sequence[] = {
  { &hf_rrc_ue_RadioAccessCapability, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UE_RadioAccessCapability_v7e0ext },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_SRNC_RelocationInfo_v7e0ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_SRNC_RelocationInfo_v7e0ext_IEs, SRNC_RelocationInfo_v7e0ext_IEs_sequence);

  return offset;
}


static const per_sequence_t SRNC_RelocationInfo_v7f0ext_IEs_sequence[] = {
  { &hf_rrc_ue_RadioAccessCapability_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_RadioAccessCapability_v7f0ext },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_SRNC_RelocationInfo_v7f0ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_SRNC_RelocationInfo_v7f0ext_IEs, SRNC_RelocationInfo_v7f0ext_IEs_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions_240_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_nonCriticalExtensions_240(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_nonCriticalExtensions_240, T_nonCriticalExtensions_240_sequence);

  return offset;
}


static const per_sequence_t T_v7f0NonCriticalExtensions_17_sequence[] = {
  { &hf_rrc_sRNC_RelocationInfo_v7f0ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SRNC_RelocationInfo_v7f0ext_IEs },
  { &hf_rrc_nonCriticalExtensions_240, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_nonCriticalExtensions_240 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v7f0NonCriticalExtensions_17(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v7f0NonCriticalExtensions_17, T_v7f0NonCriticalExtensions_17_sequence);

  return offset;
}


static const per_sequence_t T_v7e0NonCriticalExtensions_04_sequence[] = {
  { &hf_rrc_sRNC_RelocationInfo_v7e0ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SRNC_RelocationInfo_v7e0ext_IEs },
  { &hf_rrc_v7f0NonCriticalExtensions_17, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v7f0NonCriticalExtensions_17 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v7e0NonCriticalExtensions_04(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v7e0NonCriticalExtensions_04, T_v7e0NonCriticalExtensions_04_sequence);

  return offset;
}


static const per_sequence_t T_v770NonCriticalExtensions_48_sequence[] = {
  { &hf_rrc_sRNC_RelocationInfo_v770ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SRNC_RelocationInfo_v770ext_IEs },
  { &hf_rrc_v7e0NonCriticalExtensions_04, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v7e0NonCriticalExtensions_04 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v770NonCriticalExtensions_48(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v770NonCriticalExtensions_48, T_v770NonCriticalExtensions_48_sequence);

  return offset;
}


static const per_sequence_t T_v6b0NonCriticalExtensions_21_sequence[] = {
  { &hf_rrc_sRNC_RelocationInfo_v6b0ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SRNC_RelocationInfo_v6b0ext_IEs },
  { &hf_rrc_v770NonCriticalExtensions_48, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v770NonCriticalExtensions_48 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v6b0NonCriticalExtensions_21(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v6b0NonCriticalExtensions_21, T_v6b0NonCriticalExtensions_21_sequence);

  return offset;
}


static const per_sequence_t T_v690NonCriticalExtensions_53_sequence[] = {
  { &hf_rrc_sRNC_RelocationInfo_v690ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SRNC_RelocationInfo_v690ext_IEs },
  { &hf_rrc_v6b0NonCriticalExtensions_22, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v6b0NonCriticalExtensions_21 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v690NonCriticalExtensions_53(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v690NonCriticalExtensions_53, T_v690NonCriticalExtensions_53_sequence);

  return offset;
}


static const per_sequence_t T_v5c0NonCriticalExtensions_02_sequence[] = {
  { &hf_rrc_sRNC_RelocationInfo_v5c0ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SRNC_RelocationInfo_v5c0ext_IEs },
  { &hf_rrc_v690NonCriticalExtensions_53, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v690NonCriticalExtensions_53 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v5c0NonCriticalExtensions_02(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v5c0NonCriticalExtensions_02, T_v5c0NonCriticalExtensions_02_sequence);

  return offset;
}


static const per_sequence_t T_v5b0NonCriticalExtensions_03_sequence[] = {
  { &hf_rrc_sRNC_RelocationInfo_v5b0ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SRNC_RelocationInfo_v5b0ext_IEs },
  { &hf_rrc_v5c0NonCriticalExtensions_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v5c0NonCriticalExtensions_02 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v5b0NonCriticalExtensions_03(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v5b0NonCriticalExtensions_03, T_v5b0NonCriticalExtensions_03_sequence);

  return offset;
}


static const per_sequence_t T_v5a0NonCriticalExtensions_sequence[] = {
  { &hf_rrc_sRNC_RelocationInfo_v5a0ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SRNC_RelocationInfo_v5a0ext_IEs },
  { &hf_rrc_v5b0NonCriticalExtensions_03, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v5b0NonCriticalExtensions_03 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v5a0NonCriticalExtensions(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v5a0NonCriticalExtensions, T_v5a0NonCriticalExtensions_sequence);

  return offset;
}


static const per_sequence_t T_v590NonCriticalExtensions_31_sequence[] = {
  { &hf_rrc_sRNC_RelocationInfo_v590ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SRNC_RelocationInfo_v590ext_IEs },
  { &hf_rrc_v5a0NonCriticalExtensions, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v5a0NonCriticalExtensions },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v590NonCriticalExtensions_31(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v590NonCriticalExtensions_31, T_v590NonCriticalExtensions_31_sequence);

  return offset;
}


static const per_sequence_t T_v4b0NonCriticalExtensions_27_sequence[] = {
  { &hf_rrc_sRNC_RelocationInfo_v4b0ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SRNC_RelocationInfo_v4b0ext_IEs },
  { &hf_rrc_v590NonCriticalExtensions_31, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v590NonCriticalExtensions_31 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v4b0NonCriticalExtensions_27(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v4b0NonCriticalExtensions_27, T_v4b0NonCriticalExtensions_27_sequence);

  return offset;
}


static const per_sequence_t T_v3g0NonCriticalExtensions_02_sequence[] = {
  { &hf_rrc_sRNC_RelocationInfo_v3g0ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SRNC_RelocationInfo_v3g0ext_IEs },
  { &hf_rrc_v4b0NonCriticalExtensions_27, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v4b0NonCriticalExtensions_27 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v3g0NonCriticalExtensions_02(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v3g0NonCriticalExtensions_02, T_v3g0NonCriticalExtensions_02_sequence);

  return offset;
}


static const per_sequence_t T_laterNonCriticalExtensions_67_sequence[] = {
  { &hf_rrc_sRNC_RelocationInfo_v3d0ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SRNC_RelocationInfo_v3d0ext_IEs },
  { &hf_rrc_sRNC_RelocationInfo_r3_add_ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_sRNC_RelocationInfo_r3_add_ext },
  { &hf_rrc_v3g0NonCriticalExtensions_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v3g0NonCriticalExtensions_02 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_laterNonCriticalExtensions_67(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_laterNonCriticalExtensions_67, T_laterNonCriticalExtensions_67_sequence);

  return offset;
}


static const per_sequence_t T_v3c0NonCriticalExtensions_sequence[] = {
  { &hf_rrc_sRNC_RelocationInfo_v3c0ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SRNC_RelocationInfo_v3c0ext_IEs },
  { &hf_rrc_laterNonCriticalExtensions_68, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_laterNonCriticalExtensions_67 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v3c0NonCriticalExtensions(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v3c0NonCriticalExtensions, T_v3c0NonCriticalExtensions_sequence);

  return offset;
}


static const per_sequence_t T_v3b0NonCriticalExtensions_sequence[] = {
  { &hf_rrc_sRNC_RelocationInfo_v3b0ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SRNC_RelocationInfo_v3b0ext_IEs },
  { &hf_rrc_v3c0NonCriticalExtensions, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v3c0NonCriticalExtensions },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v3b0NonCriticalExtensions(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v3b0NonCriticalExtensions, T_v3b0NonCriticalExtensions_sequence);

  return offset;
}


static const per_sequence_t T_v3a0NonCriticalExtensions_16_sequence[] = {
  { &hf_rrc_sRNC_RelocationInfo_v3a0ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SRNC_RelocationInfo_v3a0ext_IEs },
  { &hf_rrc_v3b0NonCriticalExtensions, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v3b0NonCriticalExtensions },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v3a0NonCriticalExtensions_16(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v3a0NonCriticalExtensions_16, T_v3a0NonCriticalExtensions_16_sequence);

  return offset;
}


static const per_sequence_t T_v390NonCriticalExtensions_02_sequence[] = {
  { &hf_rrc_sRNC_RelocationInfo_v390ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SRNC_RelocationInfo_v390ext_IEs },
  { &hf_rrc_v3a0NonCriticalExtensions_16, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v3a0NonCriticalExtensions_16 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v390NonCriticalExtensions_02(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v390NonCriticalExtensions_02, T_v390NonCriticalExtensions_02_sequence);

  return offset;
}


static const per_sequence_t T_v380NonCriticalExtensions_02_sequence[] = {
  { &hf_rrc_sRNC_RelocationInfo_v380ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SRNC_RelocationInfo_v380ext_IEs },
  { &hf_rrc_v390NonCriticalExtensions_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v390NonCriticalExtensions_02 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v380NonCriticalExtensions_02(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v380NonCriticalExtensions_02, T_v380NonCriticalExtensions_02_sequence);

  return offset;
}


static const per_sequence_t T_r3_32_sequence[] = {
  { &hf_rrc_sRNC_RelocationInfo_r3, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SRNC_RelocationInfo_r3_IEs },
  { &hf_rrc_v380NonCriticalExtensions_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v380NonCriticalExtensions_02 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_r3_32(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_r3_32, T_r3_32_sequence);

  return offset;
}


static const per_sequence_t CipheringStatusCNdomain_r4_sequence[] = {
  { &hf_rrc_cn_DomainIdentity, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CN_DomainIdentity },
  { &hf_rrc_cipheringStatus , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CipheringStatus },
  { &hf_rrc_start_Value     , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_START_Value },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_CipheringStatusCNdomain_r4(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_CipheringStatusCNdomain_r4, CipheringStatusCNdomain_r4_sequence);

  return offset;
}


static const per_sequence_t CipheringStatusList_r4_sequence_of[1] = {
  { &hf_rrc_CipheringStatusList_r4_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CipheringStatusCNdomain_r4 },
};

static int
dissect_rrc_CipheringStatusList_r4(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_CipheringStatusList_r4, CipheringStatusList_r4_sequence_of,
                                                  1, maxCNdomains, FALSE);

  return offset;
}


static const per_sequence_t CipheringInfoPerRB_r4_sequence[] = {
  { &hf_rrc_rb_Identity     , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RB_Identity },
  { &hf_rrc_dl_HFN          , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_20_25 },
  { &hf_rrc_dl_UM_SN        , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_BIT_STRING_SIZE_7 },
  { &hf_rrc_ul_HFN          , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BIT_STRING_SIZE_20_25 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_CipheringInfoPerRB_r4(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_CipheringInfoPerRB_r4, CipheringInfoPerRB_r4_sequence);

  return offset;
}


static const per_sequence_t CipheringInfoPerRB_List_r4_sequence_of[1] = {
  { &hf_rrc_CipheringInfoPerRB_List_r4_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CipheringInfoPerRB_r4 },
};

static int
dissect_rrc_CipheringInfoPerRB_List_r4(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_CipheringInfoPerRB_List_r4, CipheringInfoPerRB_List_r4_sequence_of,
                                                  1, maxRB, FALSE);

  return offset;
}


static const value_string rrc_T_supportForRfc2507_01_vals[] = {
  {   0, "notSupported" },
  {   1, "supported" },
  { 0, NULL }
};

static const per_choice_t T_supportForRfc2507_01_choice[] = {
  {   0, &hf_rrc_notSupported    , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   1, &hf_rrc_supported_01    , ASN1_NO_EXTENSIONS     , dissect_rrc_MaxHcContextSpace },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_supportForRfc2507_01(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_supportForRfc2507_01, T_supportForRfc2507_01_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t T_supported_08_sequence[] = {
  { &hf_rrc_maxROHC_ContextSessions, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MaxROHC_ContextSessions_r4 },
  { &hf_rrc_reverseCompressionDepth, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_INTEGER_0_65535 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_supported_08(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_supported_08, T_supported_08_sequence);

  return offset;
}


static const value_string rrc_T_supportForRfc3095_01_vals[] = {
  {   0, "notSupported" },
  {   1, "supported" },
  { 0, NULL }
};

static const per_choice_t T_supportForRfc3095_01_choice[] = {
  {   0, &hf_rrc_notSupported    , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   1, &hf_rrc_supported_15    , ASN1_NO_EXTENSIONS     , dissect_rrc_T_supported_08 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_supportForRfc3095_01(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_supportForRfc3095_01, T_supportForRfc3095_01_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t PDCP_Capability_r4_sequence[] = {
  { &hf_rrc_losslessSRNS_RelocationSupport, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { &hf_rrc_supportForRfc2507_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_supportForRfc2507_01 },
  { &hf_rrc_supportForRfc3095_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_supportForRfc3095_01 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_PDCP_Capability_r4(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_PDCP_Capability_r4, PDCP_Capability_r4_sequence);

  return offset;
}


static const per_sequence_t T_fddRF_Capability_04_sequence[] = {
  { &hf_rrc_ue_PowerClass_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UE_PowerClassExt },
  { &hf_rrc_txRxFrequencySeparation, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TxRxFrequencySeparation },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_fddRF_Capability_04(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_fddRF_Capability_04, T_fddRF_Capability_04_sequence);

  return offset;
}


static const per_sequence_t T_tdd384_RF_Capability_01_sequence[] = {
  { &hf_rrc_ue_PowerClass_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UE_PowerClassExt },
  { &hf_rrc_radioFrequencyBandTDDList, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RadioFrequencyBandTDDList },
  { &hf_rrc_chipRateCapability, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_ChipRateCapability },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_tdd384_RF_Capability_01(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_tdd384_RF_Capability_01, T_tdd384_RF_Capability_01_sequence);

  return offset;
}


static const per_sequence_t T_tdd128_RF_Capability_01_sequence[] = {
  { &hf_rrc_ue_PowerClass_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UE_PowerClassExt },
  { &hf_rrc_radioFrequencyBandTDDList, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RadioFrequencyBandTDDList },
  { &hf_rrc_chipRateCapability, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_ChipRateCapability },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_tdd128_RF_Capability_01(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_tdd128_RF_Capability_01, T_tdd128_RF_Capability_01_sequence);

  return offset;
}


static const per_sequence_t RF_Capability_r4_sequence[] = {
  { &hf_rrc_fddRF_Capability_04, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_fddRF_Capability_04 },
  { &hf_rrc_tdd384_RF_Capability_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_tdd384_RF_Capability_01 },
  { &hf_rrc_tdd128_RF_Capability_03, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_tdd128_RF_Capability_01 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_RF_Capability_r4(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_RF_Capability_r4, RF_Capability_r4_sequence);

  return offset;
}


static const per_sequence_t DL_PhysChCapabilityFDD_r4_sequence[] = {
  { &hf_rrc_maxNoDPCH_PDSCH_Codes, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_1_8 },
  { &hf_rrc_maxNoPhysChBitsReceived, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MaxNoPhysChBitsReceived },
  { &hf_rrc_supportForSF_512, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { &hf_rrc_dummy_28        , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { &hf_rrc_dummy2_05       , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SimultaneousSCCPCH_DPCH_Reception },
  { &hf_rrc_dummy3_05       , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_SupportOfDedicatedPilotsForChEstimation },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_DL_PhysChCapabilityFDD_r4(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_DL_PhysChCapabilityFDD_r4, DL_PhysChCapabilityFDD_r4_sequence);

  return offset;
}


static const per_sequence_t T_fddPhysChCapability_06_sequence[] = {
  { &hf_rrc_downlinkPhysChCapability_13, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DL_PhysChCapabilityFDD_r4 },
  { &hf_rrc_uplinkPhysChCapability, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UL_PhysChCapabilityFDD },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_fddPhysChCapability_06(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_fddPhysChCapability_06, T_fddPhysChCapability_06_sequence);

  return offset;
}


static const per_sequence_t T_tdd384_PhysChCapability_sequence[] = {
  { &hf_rrc_downlinkPhysChCapability_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DL_PhysChCapabilityTDD },
  { &hf_rrc_uplinkPhysChCapability_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UL_PhysChCapabilityTDD },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_tdd384_PhysChCapability(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_tdd384_PhysChCapability, T_tdd384_PhysChCapability_sequence);

  return offset;
}


static const per_sequence_t T_tdd128_PhysChCapability_01_sequence[] = {
  { &hf_rrc_downlinkPhysChCapability_12, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DL_PhysChCapabilityTDD_LCR_r4 },
  { &hf_rrc_uplinkPhysChCapability_09, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UL_PhysChCapabilityTDD_LCR_r4 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_tdd128_PhysChCapability_01(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_tdd128_PhysChCapability_01, T_tdd128_PhysChCapability_01_sequence);

  return offset;
}


static const per_sequence_t PhysicalChannelCapability_r4_sequence[] = {
  { &hf_rrc_fddPhysChCapability_06, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_fddPhysChCapability_06 },
  { &hf_rrc_tdd384_PhysChCapability, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_tdd384_PhysChCapability },
  { &hf_rrc_tdd128_PhysChCapability_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_tdd128_PhysChCapability_01 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_PhysicalChannelCapability_r4(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_PhysicalChannelCapability_r4, PhysicalChannelCapability_r4_sequence);

  return offset;
}


static const value_string rrc_T_validity_CellPCH_UraPCH_01_vals[] = {
  {   0, "true" },
  { 0, NULL }
};


static int
dissect_rrc_T_validity_CellPCH_UraPCH_01(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}


static const value_string rrc_T_sfn_sfnType2Capability_01_vals[] = {
  {   0, "true" },
  { 0, NULL }
};


static int
dissect_rrc_T_sfn_sfnType2Capability_01(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t UE_Positioning_Capability_r4_sequence[] = {
  { &hf_rrc_standaloneLocMethodsSupported, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { &hf_rrc_ue_BasedOTDOA_Supported, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { &hf_rrc_networkAssistedGPS_Supported, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_NetworkAssistedGPS_Supported },
  { &hf_rrc_supportForUE_GPS_TimingOfCellFrames, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { &hf_rrc_supportForIPDL  , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { &hf_rrc_rx_tx_TimeDifferenceType2Capable, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { &hf_rrc_validity_CellPCH_UraPCH_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_validity_CellPCH_UraPCH_01 },
  { &hf_rrc_sfn_sfnType2Capability_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_sfn_sfnType2Capability_01 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UE_Positioning_Capability_r4(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UE_Positioning_Capability_r4, UE_Positioning_Capability_r4_sequence);

  return offset;
}


static const per_sequence_t CompressedModeMeasCapability_r4_sequence[] = {
  { &hf_rrc_fdd_Measurements, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { &hf_rrc_tdd384_Measurements, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_BOOLEAN },
  { &hf_rrc_tdd128_Measurements, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_BOOLEAN },
  { &hf_rrc_gsm_Measurements, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_GSM_Measurements },
  { &hf_rrc_multiCarrierMeasurements, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_BOOLEAN },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_CompressedModeMeasCapability_r4(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_CompressedModeMeasCapability_r4, CompressedModeMeasCapability_r4_sequence);

  return offset;
}


static const per_sequence_t MeasurementCapability_r4_sequence[] = {
  { &hf_rrc_downlinkCompressedMode_04, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CompressedModeMeasCapability_r4 },
  { &hf_rrc_uplinkCompressedMode_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CompressedModeMeasCapability_r4 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_MeasurementCapability_r4(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_MeasurementCapability_r4, MeasurementCapability_r4_sequence);

  return offset;
}


static const per_sequence_t UE_RadioAccessCapability_r4_sequence[] = {
  { &hf_rrc_accessStratumReleaseIndicator, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_AccessStratumReleaseIndicator },
  { &hf_rrc_pdcp_Capability_03, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PDCP_Capability_r4 },
  { &hf_rrc_rlc_Capability_04, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RLC_Capability },
  { &hf_rrc_transportChannelCapability, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TransportChannelCapability },
  { &hf_rrc_rf_Capability_04, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RF_Capability_r4 },
  { &hf_rrc_physicalChannelCapability_08, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PhysicalChannelCapability_r4 },
  { &hf_rrc_ue_MultiModeRAT_Capability, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UE_MultiModeRAT_Capability },
  { &hf_rrc_securityCapability, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SecurityCapability },
  { &hf_rrc_ue_positioning_Capability_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UE_Positioning_Capability_r4 },
  { &hf_rrc_measurementCapability_06, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MeasurementCapability_r4 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UE_RadioAccessCapability_r4(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UE_RadioAccessCapability_r4, UE_RadioAccessCapability_r4_sequence);

  return offset;
}


static const value_string rrc_MeasurementCommandWithType_r4_vals[] = {
  {   0, "setup" },
  {   1, "modify" },
  {   2, "release" },
  { 0, NULL }
};

static const per_choice_t MeasurementCommandWithType_r4_choice[] = {
  {   0, &hf_rrc_setup_01        , ASN1_NO_EXTENSIONS     , dissect_rrc_MeasurementType_r4 },
  {   1, &hf_rrc_modify_09       , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   2, &hf_rrc_release_01      , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_MeasurementCommandWithType_r4(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_MeasurementCommandWithType_r4, MeasurementCommandWithType_r4_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t OngoingMeasRep_r4_sequence[] = {
  { &hf_rrc_measurementIdentity, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MeasurementIdentity },
  { &hf_rrc_measurementCommandWithType_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MeasurementCommandWithType_r4 },
  { &hf_rrc_measurementReportingMode, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MeasurementReportingMode },
  { &hf_rrc_additionalMeasurementID_List, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_AdditionalMeasurementID_List },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_OngoingMeasRep_r4(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_OngoingMeasRep_r4, OngoingMeasRep_r4_sequence);

  return offset;
}


static const per_sequence_t OngoingMeasRepList_r4_sequence_of[1] = {
  { &hf_rrc_OngoingMeasRepList_r4_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_OngoingMeasRep_r4 },
};

static int
dissect_rrc_OngoingMeasRepList_r4(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_OngoingMeasRepList_r4, OngoingMeasRepList_r4_sequence_of,
                                                  1, maxNoOfMeas, FALSE);

  return offset;
}


static const per_sequence_t T_fdd_195_sequence[] = {
  { &hf_rrc_dummy1          , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CPCH_SetID },
  { &hf_rrc_dummy2_02       , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DRAC_StaticInformationList },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_fdd_195(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_fdd_195, T_fdd_195_sequence);

  return offset;
}


static const value_string rrc_T_dummy_17_vals[] = {
  {   0, "fdd" },
  {   1, "tdd" },
  { 0, NULL }
};

static const per_choice_t T_dummy_17_choice[] = {
  {   0, &hf_rrc_fdd_201         , ASN1_NO_EXTENSIONS     , dissect_rrc_T_fdd_195 },
  {   1, &hf_rrc_tdd             , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_dummy_17(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_dummy_17, T_dummy_17_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t SRNC_RelocationInfo_r4_IEs_sequence[] = {
  { &hf_rrc_rb_IdentityForHOMessage, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RB_Identity },
  { &hf_rrc_stateOfRRC      , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_StateOfRRC },
  { &hf_rrc_stateOfRRC_Procedure, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_StateOfRRC_Procedure },
  { &hf_rrc_cipheringStatusList_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CipheringStatusList_r4 },
  { &hf_rrc_latestConfiguredCN_Domain, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CN_DomainIdentity },
  { &hf_rrc_calculationTimeForCiphering, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CalculationTimeForCiphering },
  { &hf_rrc_count_C_List    , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_COUNT_C_List },
  { &hf_rrc_cipheringInfoPerRB_List_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CipheringInfoPerRB_List_r4 },
  { &hf_rrc_integrityProtectionStatus, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_IntegrityProtectionStatus },
  { &hf_rrc_srb_SpecificIntegrityProtInfo, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SRB_SpecificIntegrityProtInfoList },
  { &hf_rrc_implementationSpecificParams, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_ImplementationSpecificParams },
  { &hf_rrc_u_RNTI          , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_U_RNTI },
  { &hf_rrc_c_RNTI          , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_C_RNTI },
  { &hf_rrc_ue_RadioAccessCapability_03, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UE_RadioAccessCapability_r4 },
  { &hf_rrc_ue_RadioAccessCapability_ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_RadioAccessCapabBandFDDList },
  { &hf_rrc_ue_Positioning_LastKnownPos, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_Positioning_LastKnownPos },
  { &hf_rrc_uESpecificBehaviourInformation1idle, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UESpecificBehaviourInformation1idle },
  { &hf_rrc_uESpecificBehaviourInformation1interRAT, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UESpecificBehaviourInformation1interRAT },
  { &hf_rrc_ue_RATSpecificCapability, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_InterRAT_UE_RadioAccessCapabilityList },
  { &hf_rrc_ura_Identity    , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_URA_Identity },
  { &hf_rrc_cn_CommonGSM_MAP_NAS_SysInfo_04, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_NAS_SystemInformationGSM_MAP },
  { &hf_rrc_cn_DomainInformationList_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CN_DomainInformationListFull },
  { &hf_rrc_ongoingMeasRepList_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_OngoingMeasRepList_r4 },
  { &hf_rrc_predefinedConfigStatusList_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PredefinedConfigStatusList },
  { &hf_rrc_srb_InformationList, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SRB_InformationSetupList },
  { &hf_rrc_rab_InformationList_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RAB_InformationSetupList_r4 },
  { &hf_rrc_ul_CommonTransChInfo_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_CommonTransChInfo_r4 },
  { &hf_rrc_ul_TransChInfoList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_AddReconfTransChInfoList },
  { &hf_rrc_dummy_62        , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_dummy_17 },
  { &hf_rrc_dl_CommonTransChInfo_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_CommonTransChInfo_r4 },
  { &hf_rrc_dl_TransChInfoList_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_AddReconfTransChInfoList_r4 },
  { &hf_rrc_measurementReport, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MeasurementReport },
  { &hf_rrc_failureCause    , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_FailureCauseWithProtErr },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_SRNC_RelocationInfo_r4_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_SRNC_RelocationInfo_r4_IEs, SRNC_RelocationInfo_r4_IEs_sequence);

  return offset;
}


static const per_sequence_t TPC_Combination_Info_sequence[] = {
  { &hf_rrc_primaryCPICH_Info, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PrimaryCPICH_Info },
  { &hf_rrc_tpc_CombinationIndex, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TPC_CombinationIndex },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_TPC_Combination_Info(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_TPC_Combination_Info, TPC_Combination_Info_sequence);

  return offset;
}


static const per_sequence_t TPC_CombinationInfoList_sequence_of[1] = {
  { &hf_rrc_TPC_CombinationInfoList_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TPC_Combination_Info },
};

static int
dissect_rrc_TPC_CombinationInfoList(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_TPC_CombinationInfoList, TPC_CombinationInfoList_sequence_of,
                                                  1, maxRL, FALSE);

  return offset;
}


static const per_sequence_t SRNC_RelocationInfo_v4d0ext_IEs_sequence[] = {
  { &hf_rrc_tpc_CombinationInfoList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_TPC_CombinationInfoList },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_SRNC_RelocationInfo_v4d0ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_SRNC_RelocationInfo_v4d0ext_IEs, SRNC_RelocationInfo_v4d0ext_IEs_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions_241_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_nonCriticalExtensions_241(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_nonCriticalExtensions_241, T_nonCriticalExtensions_241_sequence);

  return offset;
}


static const per_sequence_t T_v7f0NonCriticalExtensions_18_sequence[] = {
  { &hf_rrc_sRNC_RelocationInfo_v7f0ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SRNC_RelocationInfo_v7f0ext_IEs },
  { &hf_rrc_nonCriticalExtensions_241, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_nonCriticalExtensions_241 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v7f0NonCriticalExtensions_18(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v7f0NonCriticalExtensions_18, T_v7f0NonCriticalExtensions_18_sequence);

  return offset;
}


static const per_sequence_t T_v7e0NonCriticalExtensions_05_sequence[] = {
  { &hf_rrc_sRNC_RelocationInfo_v7e0ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SRNC_RelocationInfo_v7e0ext_IEs },
  { &hf_rrc_v7f0NonCriticalExtensions_18, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v7f0NonCriticalExtensions_18 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v7e0NonCriticalExtensions_05(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v7e0NonCriticalExtensions_05, T_v7e0NonCriticalExtensions_05_sequence);

  return offset;
}


static const per_sequence_t T_v770NonCriticalExtensions_49_sequence[] = {
  { &hf_rrc_sRNC_RelocationInfo_v770ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SRNC_RelocationInfo_v770ext_IEs },
  { &hf_rrc_v7e0NonCriticalExtensions_05, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v7e0NonCriticalExtensions_05 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v770NonCriticalExtensions_49(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v770NonCriticalExtensions_49, T_v770NonCriticalExtensions_49_sequence);

  return offset;
}


static const per_sequence_t T_v6b0NonCriticalExtensions_22_sequence[] = {
  { &hf_rrc_sRNC_RelocationInfo_v6b0ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SRNC_RelocationInfo_v6b0ext_IEs },
  { &hf_rrc_v770NonCriticalExtensions_49, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v770NonCriticalExtensions_49 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v6b0NonCriticalExtensions_22(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v6b0NonCriticalExtensions_22, T_v6b0NonCriticalExtensions_22_sequence);

  return offset;
}


static const per_sequence_t T_v690NonCriticalExtensions_54_sequence[] = {
  { &hf_rrc_sRNC_RelocationInfo_v690ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SRNC_RelocationInfo_v690ext_IEs },
  { &hf_rrc_v6b0NonCriticalExtensions_23, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v6b0NonCriticalExtensions_22 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v690NonCriticalExtensions_54(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v690NonCriticalExtensions_54, T_v690NonCriticalExtensions_54_sequence);

  return offset;
}


static const per_sequence_t T_v5c0NonCriticalExtensions_03_sequence[] = {
  { &hf_rrc_sRNC_RelocationInfo_v5c0ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SRNC_RelocationInfo_v5c0ext_IEs },
  { &hf_rrc_v690NonCriticalExtensions_54, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v690NonCriticalExtensions_54 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v5c0NonCriticalExtensions_03(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v5c0NonCriticalExtensions_03, T_v5c0NonCriticalExtensions_03_sequence);

  return offset;
}


static const per_sequence_t T_v5b0NonCriticalExtensions_04_sequence[] = {
  { &hf_rrc_sRNC_RelocationInfo_v5b0ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SRNC_RelocationInfo_v5b0ext_IEs },
  { &hf_rrc_v5c0NonCriticalExtensions_03, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v5c0NonCriticalExtensions_03 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v5b0NonCriticalExtensions_04(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v5b0NonCriticalExtensions_04, T_v5b0NonCriticalExtensions_04_sequence);

  return offset;
}


static const per_sequence_t T_v5a0NonCriticalExtensions_01_sequence[] = {
  { &hf_rrc_sRNC_RelocationInfo_v5a0ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SRNC_RelocationInfo_v5a0ext_IEs },
  { &hf_rrc_v5b0NonCriticalExtensions_04, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v5b0NonCriticalExtensions_04 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v5a0NonCriticalExtensions_01(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v5a0NonCriticalExtensions_01, T_v5a0NonCriticalExtensions_01_sequence);

  return offset;
}


static const per_sequence_t T_v590NonCriticalExtensions_32_sequence[] = {
  { &hf_rrc_sRNC_RelocationInfo_v590ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SRNC_RelocationInfo_v590ext_IEs },
  { &hf_rrc_v5a0NonCriticalExtensions_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v5a0NonCriticalExtensions_01 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v590NonCriticalExtensions_32(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v590NonCriticalExtensions_32, T_v590NonCriticalExtensions_32_sequence);

  return offset;
}


static const per_sequence_t T_v4d0NonCriticalExtensions_14_sequence[] = {
  { &hf_rrc_sRNC_RelocationInfo_v4d0ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SRNC_RelocationInfo_v4d0ext_IEs },
  { &hf_rrc_sRNC_RelocationInfo_r4_add_ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_BIT_STRING },
  { &hf_rrc_v590NonCriticalExtensions_32, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v590NonCriticalExtensions_32 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v4d0NonCriticalExtensions_14(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v4d0NonCriticalExtensions_14, T_v4d0NonCriticalExtensions_14_sequence);

  return offset;
}


static const per_sequence_t T_r4_14_sequence[] = {
  { &hf_rrc_sRNC_RelocationInfo_r4, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SRNC_RelocationInfo_r4_IEs },
  { &hf_rrc_v4d0NonCriticalExtensions_14, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v4d0NonCriticalExtensions_14 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_r4_14(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_r4_14, T_r4_14_sequence);

  return offset;
}


static const value_string rrc_MaxHcContextSpace_r5_vals[] = {
  {   0, "dummy" },
  {   1, "by1024" },
  {   2, "by2048" },
  {   3, "by4096" },
  {   4, "by8192" },
  {   5, "by16384" },
  {   6, "by32768" },
  {   7, "by65536" },
  {   8, "by131072" },
  { 0, NULL }
};


static int
dissect_rrc_MaxHcContextSpace_r5(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     9, NULL, FALSE, 0, NULL);

  return offset;
}


static const value_string rrc_T_supportForRfc2507_02_vals[] = {
  {   0, "notSupported" },
  {   1, "supported" },
  { 0, NULL }
};

static const per_choice_t T_supportForRfc2507_02_choice[] = {
  {   0, &hf_rrc_notSupported    , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   1, &hf_rrc_supported_16    , ASN1_NO_EXTENSIONS     , dissect_rrc_MaxHcContextSpace_r5 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_supportForRfc2507_02(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_supportForRfc2507_02, T_supportForRfc2507_02_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t T_supported_09_sequence[] = {
  { &hf_rrc_maxROHC_ContextSessions, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MaxROHC_ContextSessions_r4 },
  { &hf_rrc_reverseCompressionDepth, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_INTEGER_0_65535 },
  { &hf_rrc_supportForRfc3095ContextRelocation, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_supported_09(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_supported_09, T_supported_09_sequence);

  return offset;
}


static const value_string rrc_T_supportForRfc3095_02_vals[] = {
  {   0, "notSupported" },
  {   1, "supported" },
  { 0, NULL }
};

static const per_choice_t T_supportForRfc3095_02_choice[] = {
  {   0, &hf_rrc_notSupported    , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   1, &hf_rrc_supported_17    , ASN1_NO_EXTENSIONS     , dissect_rrc_T_supported_09 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_supportForRfc3095_02(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_supportForRfc3095_02, T_supportForRfc3095_02_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t PDCP_Capability_r5_sequence[] = {
  { &hf_rrc_losslessSRNS_RelocationSupport, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { &hf_rrc_supportForRfc2507_02, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_supportForRfc2507_02 },
  { &hf_rrc_supportForRfc3095_02, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_supportForRfc3095_02 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_PDCP_Capability_r5(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_PDCP_Capability_r5, PDCP_Capability_r5_sequence);

  return offset;
}


static const value_string rrc_TotalRLC_AM_BufferSize_r5_vals[] = {
  {   0, "kb10" },
  {   1, "kb50" },
  {   2, "kb100" },
  {   3, "kb150" },
  {   4, "kb200" },
  {   5, "kb300" },
  {   6, "kb400" },
  {   7, "kb500" },
  {   8, "kb750" },
  {   9, "kb1000" },
  { 0, NULL }
};


static int
dissect_rrc_TotalRLC_AM_BufferSize_r5(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     10, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t RLC_Capability_r5_sequence[] = {
  { &hf_rrc_totalRLC_AM_BufferSize_05, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TotalRLC_AM_BufferSize_r5 },
  { &hf_rrc_maximumRLC_WindowSize, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MaximumRLC_WindowSize },
  { &hf_rrc_maximumAM_EntityNumber, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MaximumAM_EntityNumberRLC_Cap },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_RLC_Capability_r5(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_RLC_Capability_r5, RLC_Capability_r5_sequence);

  return offset;
}


static const per_sequence_t T_supported_07_sequence[] = {
  { &hf_rrc_hsdsch_physical_layer_category, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_HSDSCH_physical_layer_category },
  { &hf_rrc_dummy_28        , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { &hf_rrc_dummy2_06       , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_supported_07(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_supported_07, T_supported_07_sequence);

  return offset;
}


static const value_string rrc_T_fdd_hspdsch_01_vals[] = {
  {   0, "supported" },
  {   1, "unsupported" },
  { 0, NULL }
};

static const per_choice_t T_fdd_hspdsch_01_choice[] = {
  {   0, &hf_rrc_supported_14    , ASN1_NO_EXTENSIONS     , dissect_rrc_T_supported_07 },
  {   1, &hf_rrc_unsupported     , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_fdd_hspdsch_01(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_fdd_hspdsch_01, T_fdd_hspdsch_01_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t DL_PhysChCapabilityFDD_r5_sequence[] = {
  { &hf_rrc_maxNoDPCH_PDSCH_Codes, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_1_8 },
  { &hf_rrc_maxNoPhysChBitsReceived, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MaxNoPhysChBitsReceived },
  { &hf_rrc_supportForSF_512, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { &hf_rrc_dummy_28        , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { &hf_rrc_dummy2_05       , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SimultaneousSCCPCH_DPCH_Reception },
  { &hf_rrc_dummy3_05       , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_SupportOfDedicatedPilotsForChEstimation },
  { &hf_rrc_fdd_hspdsch_01  , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_fdd_hspdsch_01 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_DL_PhysChCapabilityFDD_r5(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_DL_PhysChCapabilityFDD_r5, DL_PhysChCapabilityFDD_r5_sequence);

  return offset;
}


static const per_sequence_t T_fddPhysChCapability_07_sequence[] = {
  { &hf_rrc_downlinkPhysChCapability_14, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DL_PhysChCapabilityFDD_r5 },
  { &hf_rrc_uplinkPhysChCapability, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UL_PhysChCapabilityFDD },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_fddPhysChCapability_07(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_fddPhysChCapability_07, T_fddPhysChCapability_07_sequence);

  return offset;
}


static const value_string rrc_T_tdd384_hspdsch_01_vals[] = {
  {   0, "supported" },
  {   1, "unsupported" },
  { 0, NULL }
};

static const per_choice_t T_tdd384_hspdsch_01_choice[] = {
  {   0, &hf_rrc_supported       , ASN1_NO_EXTENSIONS     , dissect_rrc_HSDSCH_physical_layer_category },
  {   1, &hf_rrc_unsupported     , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_tdd384_hspdsch_01(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_tdd384_hspdsch_01, T_tdd384_hspdsch_01_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t DL_PhysChCapabilityTDD_r5_sequence[] = {
  { &hf_rrc_maxTS_PerFrame  , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MaxTS_PerFrame },
  { &hf_rrc_maxPhysChPerFrame, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MaxPhysChPerFrame },
  { &hf_rrc_minimumSF       , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MinimumSF_DL },
  { &hf_rrc_supportOfPDSCH  , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { &hf_rrc_maxPhysChPerTS  , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MaxPhysChPerTS },
  { &hf_rrc_tdd384_hspdsch_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_tdd384_hspdsch_01 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_DL_PhysChCapabilityTDD_r5(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_DL_PhysChCapabilityTDD_r5, DL_PhysChCapabilityTDD_r5_sequence);

  return offset;
}


static const per_sequence_t T_tdd384_PhysChCapability_01_sequence[] = {
  { &hf_rrc_downlinkPhysChCapability_15, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DL_PhysChCapabilityTDD_r5 },
  { &hf_rrc_uplinkPhysChCapability_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UL_PhysChCapabilityTDD },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_tdd384_PhysChCapability_01(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_tdd384_PhysChCapability_01, T_tdd384_PhysChCapability_01_sequence);

  return offset;
}


static const value_string rrc_T_tdd128_hspdsch_01_vals[] = {
  {   0, "supported" },
  {   1, "unsupported" },
  { 0, NULL }
};

static const per_choice_t T_tdd128_hspdsch_01_choice[] = {
  {   0, &hf_rrc_supported       , ASN1_NO_EXTENSIONS     , dissect_rrc_HSDSCH_physical_layer_category },
  {   1, &hf_rrc_unsupported     , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_tdd128_hspdsch_01(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_tdd128_hspdsch_01, T_tdd128_hspdsch_01_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t DL_PhysChCapabilityTDD_LCR_r5_sequence[] = {
  { &hf_rrc_maxTS_PerSubFrame, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MaxTS_PerSubFrame_r4 },
  { &hf_rrc_maxPhysChPerFrame_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MaxPhysChPerSubFrame_r4 },
  { &hf_rrc_minimumSF       , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MinimumSF_DL },
  { &hf_rrc_supportOfPDSCH  , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { &hf_rrc_maxPhysChPerTS  , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MaxPhysChPerTS },
  { &hf_rrc_supportOf8PSK   , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { &hf_rrc_tdd128_hspdsch_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_tdd128_hspdsch_01 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_DL_PhysChCapabilityTDD_LCR_r5(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_DL_PhysChCapabilityTDD_LCR_r5, DL_PhysChCapabilityTDD_LCR_r5_sequence);

  return offset;
}


static const per_sequence_t T_tdd128_PhysChCapability_02_sequence[] = {
  { &hf_rrc_downlinkPhysChCapability_16, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_DL_PhysChCapabilityTDD_LCR_r5 },
  { &hf_rrc_uplinkPhysChCapability_09, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UL_PhysChCapabilityTDD_LCR_r4 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_tdd128_PhysChCapability_02(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_tdd128_PhysChCapability_02, T_tdd128_PhysChCapability_02_sequence);

  return offset;
}


static const per_sequence_t PhysicalChannelCapability_r5_sequence[] = {
  { &hf_rrc_fddPhysChCapability_07, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_fddPhysChCapability_07 },
  { &hf_rrc_tdd384_PhysChCapability_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_tdd384_PhysChCapability_01 },
  { &hf_rrc_tdd128_PhysChCapability_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_tdd128_PhysChCapability_02 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_PhysicalChannelCapability_r5(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_PhysicalChannelCapability_r5, PhysicalChannelCapability_r5_sequence);

  return offset;
}


static const per_sequence_t UE_MultiModeRAT_Capability_r5_sequence[] = {
  { &hf_rrc_multiRAT_CapabilityList, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MultiRAT_Capability },
  { &hf_rrc_multiModeCapability, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MultiModeCapability },
  { &hf_rrc_supportOfUTRAN_ToGERAN_NACC, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UE_MultiModeRAT_Capability_r5(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UE_MultiModeRAT_Capability_r5, UE_MultiModeRAT_Capability_r5_sequence);

  return offset;
}


static const per_sequence_t UE_RadioAccessCapability_r5_sequence[] = {
  { &hf_rrc_accessStratumReleaseIndicator, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_AccessStratumReleaseIndicator },
  { &hf_rrc_dl_CapabilityWithSimultaneousHS_DSCHConfig, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_CapabilityWithSimultaneousHS_DSCHConfig },
  { &hf_rrc_pdcp_Capability_04, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PDCP_Capability_r5 },
  { &hf_rrc_rlc_Capability_05, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RLC_Capability_r5 },
  { &hf_rrc_transportChannelCapability, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TransportChannelCapability },
  { &hf_rrc_rf_Capability_04, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RF_Capability_r4 },
  { &hf_rrc_physicalChannelCapability_09, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PhysicalChannelCapability_r5 },
  { &hf_rrc_ue_MultiModeRAT_Capability_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UE_MultiModeRAT_Capability_r5 },
  { &hf_rrc_securityCapability, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SecurityCapability },
  { &hf_rrc_ue_positioning_Capability_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UE_Positioning_Capability_r4 },
  { &hf_rrc_measurementCapability_06, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MeasurementCapability_r4 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UE_RadioAccessCapability_r5(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UE_RadioAccessCapability_r5, UE_RadioAccessCapability_r5_sequence);

  return offset;
}


static const per_sequence_t InterRAT_UE_RadioAccessCapabilityList_r5_sequence[] = {
  { &hf_rrc_interRAT_UE_RadioAccessCapability, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_InterRAT_UE_RadioAccessCapabilityList },
  { &hf_rrc_geranIu_RadioAccessCapability, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_GERANIu_RadioAccessCapability },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_InterRAT_UE_RadioAccessCapabilityList_r5(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_InterRAT_UE_RadioAccessCapabilityList_r5, InterRAT_UE_RadioAccessCapabilityList_r5_sequence);

  return offset;
}


static const value_string rrc_T_measurementCommand_v590ext_01_vals[] = {
  {   0, "intra-frequency" },
  {   1, "inter-frequency" },
  { 0, NULL }
};

static const per_choice_t T_measurementCommand_v590ext_01_choice[] = {
  {   0, &hf_rrc_intra_frequency , ASN1_NO_EXTENSIONS     , dissect_rrc_Intra_FreqEventCriteriaList_v590ext },
  {   1, &hf_rrc_inter_frequency , ASN1_NO_EXTENSIONS     , dissect_rrc_Inter_FreqEventCriteriaList_v590ext },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_measurementCommand_v590ext_01(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_measurementCommand_v590ext_01, T_measurementCommand_v590ext_01_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t OngoingMeasRep_r5_sequence[] = {
  { &hf_rrc_measurementIdentity, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MeasurementIdentity },
  { &hf_rrc_measurementCommandWithType_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MeasurementCommandWithType_r4 },
  { &hf_rrc_measurementReportingMode, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MeasurementReportingMode },
  { &hf_rrc_additionalMeasurementID_List, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_AdditionalMeasurementID_List },
  { &hf_rrc_measurementCommand_v590ext_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_measurementCommand_v590ext_01 },
  { &hf_rrc_intraFreqReportingCriteria_1b_r5, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_IntraFreqReportingCriteria_1b_r5 },
  { &hf_rrc_intraFreqEvent_1d_r5, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_IntraFreqEvent_1d_r5 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_OngoingMeasRep_r5(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_OngoingMeasRep_r5, OngoingMeasRep_r5_sequence);

  return offset;
}


static const per_sequence_t OngoingMeasRepList_r5_sequence_of[1] = {
  { &hf_rrc_OngoingMeasRepList_r5_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_OngoingMeasRep_r5 },
};

static int
dissect_rrc_OngoingMeasRepList_r5(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_OngoingMeasRepList_r5, OngoingMeasRepList_r5_sequence_of,
                                                  1, maxNoOfMeas, FALSE);

  return offset;
}


static const per_sequence_t T_fdd_196_sequence[] = {
  { &hf_rrc_dummy1          , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CPCH_SetID },
  { &hf_rrc_dummy2_02       , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DRAC_StaticInformationList },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_fdd_196(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_fdd_196, T_fdd_196_sequence);

  return offset;
}


static const value_string rrc_T_dummy_18_vals[] = {
  {   0, "fdd" },
  {   1, "tdd" },
  { 0, NULL }
};

static const per_choice_t T_dummy_18_choice[] = {
  {   0, &hf_rrc_fdd_202         , ASN1_NO_EXTENSIONS     , dissect_rrc_T_fdd_196 },
  {   1, &hf_rrc_tdd             , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_dummy_18(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_dummy_18, T_dummy_18_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t SRNC_RelocationInfo_r5_IEs_sequence[] = {
  { &hf_rrc_rb_IdentityForHOMessage, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RB_Identity },
  { &hf_rrc_stateOfRRC      , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_StateOfRRC },
  { &hf_rrc_stateOfRRC_Procedure, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_StateOfRRC_Procedure },
  { &hf_rrc_cipheringStatusList_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CipheringStatusList_r4 },
  { &hf_rrc_latestConfiguredCN_Domain, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CN_DomainIdentity },
  { &hf_rrc_calculationTimeForCiphering, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CalculationTimeForCiphering },
  { &hf_rrc_count_C_List    , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_COUNT_C_List },
  { &hf_rrc_cipheringInfoPerRB_List_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CipheringInfoPerRB_List_r4 },
  { &hf_rrc_integrityProtectionStatus, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_IntegrityProtectionStatus },
  { &hf_rrc_srb_SpecificIntegrityProtInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_SRB_SpecificIntegrityProtInfoList },
  { &hf_rrc_implementationSpecificParams, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_ImplementationSpecificParams },
  { &hf_rrc_u_RNTI          , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_U_RNTI },
  { &hf_rrc_c_RNTI          , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_C_RNTI },
  { &hf_rrc_ue_RadioAccessCapability_04, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UE_RadioAccessCapability_r5 },
  { &hf_rrc_ue_RadioAccessCapability_ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_RadioAccessCapabBandFDDList },
  { &hf_rrc_ue_Positioning_LastKnownPos, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_Positioning_LastKnownPos },
  { &hf_rrc_uESpecificBehaviourInformation1idle, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UESpecificBehaviourInformation1idle },
  { &hf_rrc_uESpecificBehaviourInformation1interRAT, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UESpecificBehaviourInformation1interRAT },
  { &hf_rrc_ue_RATSpecificCapability_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_InterRAT_UE_RadioAccessCapabilityList_r5 },
  { &hf_rrc_ura_Identity    , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_URA_Identity },
  { &hf_rrc_cn_CommonGSM_MAP_NAS_SysInfo_04, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_NAS_SystemInformationGSM_MAP },
  { &hf_rrc_cn_DomainInformationList_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CN_DomainInformationListFull },
  { &hf_rrc_ongoingMeasRepList_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_OngoingMeasRepList_r5 },
  { &hf_rrc_predefinedConfigStatusList_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PredefinedConfigStatusList },
  { &hf_rrc_srb_InformationList_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SRB_InformationSetupList_r5 },
  { &hf_rrc_rab_InformationList_03, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RAB_InformationSetupList_r5 },
  { &hf_rrc_ul_CommonTransChInfo_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_CommonTransChInfo_r4 },
  { &hf_rrc_ul_TransChInfoList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_AddReconfTransChInfoList },
  { &hf_rrc_dummy_63        , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_dummy_18 },
  { &hf_rrc_dl_CommonTransChInfo_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_CommonTransChInfo_r4 },
  { &hf_rrc_dl_TransChInfoList_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_AddReconfTransChInfoList_r5 },
  { &hf_rrc_tpc_CombinationInfoList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_TPC_CombinationInfoList },
  { &hf_rrc_measurementReport, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MeasurementReport },
  { &hf_rrc_failureCause    , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_FailureCauseWithProtErr },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_SRNC_RelocationInfo_r5_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_SRNC_RelocationInfo_r5_IEs, SRNC_RelocationInfo_r5_IEs_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions_242_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_nonCriticalExtensions_242(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_nonCriticalExtensions_242, T_nonCriticalExtensions_242_sequence);

  return offset;
}


static const per_sequence_t T_v7f0NonCriticalExtensions_19_sequence[] = {
  { &hf_rrc_sRNC_RelocationInfo_v7f0ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SRNC_RelocationInfo_v7f0ext_IEs },
  { &hf_rrc_nonCriticalExtensions_242, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_nonCriticalExtensions_242 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v7f0NonCriticalExtensions_19(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v7f0NonCriticalExtensions_19, T_v7f0NonCriticalExtensions_19_sequence);

  return offset;
}


static const per_sequence_t T_v7e0NonCriticalExtensions_06_sequence[] = {
  { &hf_rrc_sRNC_RelocationInfo_v7e0ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SRNC_RelocationInfo_v7e0ext_IEs },
  { &hf_rrc_v7f0NonCriticalExtensions_19, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v7f0NonCriticalExtensions_19 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v7e0NonCriticalExtensions_06(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v7e0NonCriticalExtensions_06, T_v7e0NonCriticalExtensions_06_sequence);

  return offset;
}


static const per_sequence_t T_v770NonCriticalExtensions_50_sequence[] = {
  { &hf_rrc_sRNC_RelocationInfo_v770ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SRNC_RelocationInfo_v770ext_IEs },
  { &hf_rrc_v7e0NonCriticalExtensions_06, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v7e0NonCriticalExtensions_06 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v770NonCriticalExtensions_50(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v770NonCriticalExtensions_50, T_v770NonCriticalExtensions_50_sequence);

  return offset;
}


static const per_sequence_t T_v6b0NonCriticalExtensions_23_sequence[] = {
  { &hf_rrc_sRNC_RelocationInfo_v6b0ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SRNC_RelocationInfo_v6b0ext_IEs },
  { &hf_rrc_v770NonCriticalExtensions_50, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v770NonCriticalExtensions_50 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v6b0NonCriticalExtensions_23(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v6b0NonCriticalExtensions_23, T_v6b0NonCriticalExtensions_23_sequence);

  return offset;
}


static const per_sequence_t T_v690NonCriticalExtensions_55_sequence[] = {
  { &hf_rrc_sRNC_RelocationInfo_v690ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SRNC_RelocationInfo_v690ext_IEs },
  { &hf_rrc_v6b0NonCriticalExtensions_24, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v6b0NonCriticalExtensions_23 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v690NonCriticalExtensions_55(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v690NonCriticalExtensions_55, T_v690NonCriticalExtensions_55_sequence);

  return offset;
}


static const per_sequence_t T_v5c0NonCriticalExtensions_04_sequence[] = {
  { &hf_rrc_sRNC_RelocationInfo_v5c0ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SRNC_RelocationInfo_v5c0ext_IEs },
  { &hf_rrc_v690NonCriticalExtensions_55, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v690NonCriticalExtensions_55 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v5c0NonCriticalExtensions_04(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v5c0NonCriticalExtensions_04, T_v5c0NonCriticalExtensions_04_sequence);

  return offset;
}


static const per_sequence_t T_v5b0NonCriticalExtensions_05_sequence[] = {
  { &hf_rrc_sRNC_RelocationInfo_v5b0ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SRNC_RelocationInfo_v5b0ext_IEs },
  { &hf_rrc_v5c0NonCriticalExtensions_04, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v5c0NonCriticalExtensions_04 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v5b0NonCriticalExtensions_05(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v5b0NonCriticalExtensions_05, T_v5b0NonCriticalExtensions_05_sequence);

  return offset;
}


static const per_sequence_t T_v5a0NonCriticalExtensions_02_sequence[] = {
  { &hf_rrc_sRNC_RelocationInfo_v5a0ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SRNC_RelocationInfo_v5a0ext_IEs },
  { &hf_rrc_v5b0NonCriticalExtensions_05, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v5b0NonCriticalExtensions_05 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v5a0NonCriticalExtensions_02(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v5a0NonCriticalExtensions_02, T_v5a0NonCriticalExtensions_02_sequence);

  return offset;
}


static const per_sequence_t T_r5_15_sequence[] = {
  { &hf_rrc_sRNC_RelocationInfo_r5, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SRNC_RelocationInfo_r5_IEs },
  { &hf_rrc_sRNC_RelocationInfo_r5_add_ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_BIT_STRING },
  { &hf_rrc_v5a0NonCriticalExtensions_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v5a0NonCriticalExtensions_02 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_r5_15(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_r5_15, T_r5_15_sequence);

  return offset;
}


static const value_string rrc_T_losslessDLRLC_PDUSizeChange_01_vals[] = {
  {   0, "true" },
  { 0, NULL }
};


static int
dissect_rrc_T_losslessDLRLC_PDUSizeChange_01(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}


static const value_string rrc_T_supportForRfc2507_03_vals[] = {
  {   0, "notSupported" },
  {   1, "supported" },
  { 0, NULL }
};

static const per_choice_t T_supportForRfc2507_03_choice[] = {
  {   0, &hf_rrc_notSupported    , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   1, &hf_rrc_supported_16    , ASN1_NO_EXTENSIONS     , dissect_rrc_MaxHcContextSpace_r5 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_supportForRfc2507_03(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_supportForRfc2507_03, T_supportForRfc2507_03_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t T_supported_10_sequence[] = {
  { &hf_rrc_maxROHC_ContextSessions, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MaxROHC_ContextSessions_r4 },
  { &hf_rrc_reverseCompressionDepth, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_INTEGER_0_65535 },
  { &hf_rrc_supportForRfc3095ContextRelocation, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_supported_10(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_supported_10, T_supported_10_sequence);

  return offset;
}


static const value_string rrc_T_supportForRfc3095_03_vals[] = {
  {   0, "notSupported" },
  {   1, "supported" },
  { 0, NULL }
};

static const per_choice_t T_supportForRfc3095_03_choice[] = {
  {   0, &hf_rrc_notSupported    , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   1, &hf_rrc_supported_18    , ASN1_NO_EXTENSIONS     , dissect_rrc_T_supported_10 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_supportForRfc3095_03(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_supportForRfc3095_03, T_supportForRfc3095_03_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t PDCP_Capability_r6_sequence[] = {
  { &hf_rrc_losslessSRNS_RelocationSupport, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { &hf_rrc_losslessDLRLC_PDUSizeChange_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_losslessDLRLC_PDUSizeChange_01 },
  { &hf_rrc_supportForRfc2507_03, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_supportForRfc2507_03 },
  { &hf_rrc_supportForRfc3095_03, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_supportForRfc3095_03 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_PDCP_Capability_r6(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_PDCP_Capability_r6, PDCP_Capability_r6_sequence);

  return offset;
}


static const per_sequence_t UE_RadioAccessCapability_r6_sequence[] = {
  { &hf_rrc_accessStratumReleaseIndicator, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_AccessStratumReleaseIndicator },
  { &hf_rrc_dl_CapabilityWithSimultaneousHS_DSCHConfig, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_CapabilityWithSimultaneousHS_DSCHConfig },
  { &hf_rrc_pdcp_Capability_05, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PDCP_Capability_r6 },
  { &hf_rrc_rlc_Capability_05, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RLC_Capability_r5 },
  { &hf_rrc_transportChannelCapability, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TransportChannelCapability },
  { &hf_rrc_rf_Capability_04, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RF_Capability_r4 },
  { &hf_rrc_physicalChannelCapability_09, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PhysicalChannelCapability_r5 },
  { &hf_rrc_ue_MultiModeRAT_Capability_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UE_MultiModeRAT_Capability_r5 },
  { &hf_rrc_securityCapability, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SecurityCapability },
  { &hf_rrc_ue_positioning_Capability_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UE_Positioning_Capability_r4 },
  { &hf_rrc_measurementCapability_06, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MeasurementCapability_r4 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UE_RadioAccessCapability_r6(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UE_RadioAccessCapability_r6, UE_RadioAccessCapability_r6_sequence);

  return offset;
}



static int
dissect_rrc_T_ueCapabilityContainer_03(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_bit_string_containing_pdu_new(tvb, offset, actx, tree, hf_index,
                                                              NO_BOUND, NO_BOUND, FALSE, dissect_UE_CapabilityContainer_IEs_PDU);

  return offset;
}



static int
dissect_rrc_T_ueCapabilityContainer_RSC_01(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_bit_string_containing_pdu_new(tvb, offset, actx, tree, hf_index,
                                                              NO_BOUND, NO_BOUND, FALSE, dissect_RRCConnectionSetupComplete_r3_add_ext_IEs_PDU);

  return offset;
}



static int
dissect_rrc_T_ueCapabilityContainer_UCI_01(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_bit_string_containing_pdu_new(tvb, offset, actx, tree, hf_index,
                                                              NO_BOUND, NO_BOUND, FALSE, dissect_UECapabilityInformation_r3_add_ext_IEs_PDU);

  return offset;
}


static const value_string rrc_MeasurementCommandWithType_r6_vals[] = {
  {   0, "setup" },
  {   1, "modify" },
  {   2, "release" },
  { 0, NULL }
};

static const per_choice_t MeasurementCommandWithType_r6_choice[] = {
  {   0, &hf_rrc_setup_02        , ASN1_NO_EXTENSIONS     , dissect_rrc_MeasurementType_r6 },
  {   1, &hf_rrc_modify_09       , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   2, &hf_rrc_release_01      , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_MeasurementCommandWithType_r6(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_MeasurementCommandWithType_r6, MeasurementCommandWithType_r6_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t OngoingMeasRep_r6_sequence[] = {
  { &hf_rrc_measurementIdentity, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MeasurementIdentity },
  { &hf_rrc_measurementCommandWithType_02, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MeasurementCommandWithType_r6 },
  { &hf_rrc_measurementReportingMode, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MeasurementReportingMode },
  { &hf_rrc_additionalMeasurementID_List, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_AdditionalMeasurementID_List },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_OngoingMeasRep_r6(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_OngoingMeasRep_r6, OngoingMeasRep_r6_sequence);

  return offset;
}


static const per_sequence_t OngoingMeasRepList_r6_sequence_of[1] = {
  { &hf_rrc_OngoingMeasRepList_r6_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_OngoingMeasRep_r6 },
};

static int
dissect_rrc_OngoingMeasRepList_r6(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_OngoingMeasRepList_r6, OngoingMeasRepList_r6_sequence_of,
                                                  1, maxNoOfMeas, FALSE);

  return offset;
}



static int
dissect_rrc_T_measurementReport(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_bit_string_containing_pdu_new(tvb, offset, actx, tree, hf_index,
                                                              NO_BOUND, NO_BOUND, FALSE, dissect_MeasurementReport_PDU);

  return offset;
}


static const per_sequence_t SRNC_RelocationInfo_r6_IEs_sequence[] = {
  { &hf_rrc_rb_IdentityForHOMessage, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RB_Identity },
  { &hf_rrc_stateOfRRC      , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_StateOfRRC },
  { &hf_rrc_stateOfRRC_Procedure, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_StateOfRRC_Procedure },
  { &hf_rrc_cipheringStatusList_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CipheringStatusList_r4 },
  { &hf_rrc_latestConfiguredCN_Domain, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CN_DomainIdentity },
  { &hf_rrc_calculationTimeForCiphering, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CalculationTimeForCiphering },
  { &hf_rrc_count_C_List    , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_COUNT_C_List },
  { &hf_rrc_cipheringInfoPerRB_List_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CipheringInfoPerRB_List_r4 },
  { &hf_rrc_integrityProtectionStatus, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_IntegrityProtectionStatus },
  { &hf_rrc_srb_SpecificIntegrityProtInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_SRB_SpecificIntegrityProtInfoList },
  { &hf_rrc_implementationSpecificParams, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_ImplementationSpecificParams },
  { &hf_rrc_u_RNTI          , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_U_RNTI },
  { &hf_rrc_c_RNTI          , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_C_RNTI },
  { &hf_rrc_ue_RadioAccessCapability_05, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UE_RadioAccessCapability_r6 },
  { &hf_rrc_ue_RadioAccessCapability_ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_RadioAccessCapabBandFDDList },
  { &hf_rrc_ue_Positioning_LastKnownPos, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_Positioning_LastKnownPos },
  { &hf_rrc_uESpecificBehaviourInformation1idle, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UESpecificBehaviourInformation1idle },
  { &hf_rrc_uESpecificBehaviourInformation1interRAT, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UESpecificBehaviourInformation1interRAT },
  { &hf_rrc_ueCapabilityContainer_03, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_ueCapabilityContainer_03 },
  { &hf_rrc_ueCapabilityContainer_RSC_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_ueCapabilityContainer_RSC_01 },
  { &hf_rrc_ueCapabilityContainer_UCI_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_ueCapabilityContainer_UCI_01 },
  { &hf_rrc_ue_RATSpecificCapability_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_InterRAT_UE_RadioAccessCapabilityList_r5 },
  { &hf_rrc_ura_Identity    , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_URA_Identity },
  { &hf_rrc_cn_CommonGSM_MAP_NAS_SysInfo_04, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_NAS_SystemInformationGSM_MAP },
  { &hf_rrc_cn_DomainInformationList_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CN_DomainInformationListFull },
  { &hf_rrc_ongoingMeasRepList_03, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_OngoingMeasRepList_r6 },
  { &hf_rrc_interRATCellInfoIndication, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_InterRATCellInfoIndication },
  { &hf_rrc_predefinedConfigStatusList_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PredefinedConfigStatusList },
  { &hf_rrc_srb_InformationList_02, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SRB_InformationSetupList_r6 },
  { &hf_rrc_rab_InformationList_04, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RAB_InformationSetupList_r6 },
  { &hf_rrc_ul_CommonTransChInfo_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_CommonTransChInfo_r4 },
  { &hf_rrc_ul_TransChInfoList_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_AddReconfTransChInfoList_r6 },
  { &hf_rrc_dl_CommonTransChInfo_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_CommonTransChInfo_r4 },
  { &hf_rrc_dl_TransChInfoList_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_AddReconfTransChInfoList_r5 },
  { &hf_rrc_tpc_CombinationInfoList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_TPC_CombinationInfoList },
  { &hf_rrc_storedCompressedModeInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_StoredCompressedModeInfo },
  { &hf_rrc_measurementReport_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_measurementReport },
  { &hf_rrc_failureCause    , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_FailureCauseWithProtErr },
  { &hf_rrc_mbms_JoinedInformation, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MBMS_JoinedInformation_r6 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_SRNC_RelocationInfo_r6_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_SRNC_RelocationInfo_r6_IEs, SRNC_RelocationInfo_r6_IEs_sequence);

  return offset;
}



static int
dissect_rrc_T_sRNC_RelocationInfo_r6_add_ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_bit_string_containing_pdu_new(tvb, offset, actx, tree, hf_index,
                                                              NO_BOUND, NO_BOUND, FALSE, dissect_SRNC_RelocationInfo_r6_add_ext_IEs_PDU);

  return offset;
}


static const per_sequence_t SRNC_RelocationInfo_v820ext_IEs_sequence[] = {
  { &hf_rrc_rab_InformationList_07, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RAB_InformationSetupList_v820ext },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_SRNC_RelocationInfo_v820ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_SRNC_RelocationInfo_v820ext_IEs, SRNC_RelocationInfo_v820ext_IEs_sequence);

  return offset;
}


static const per_sequence_t E_RGCH_Combination_Info_sequence[] = {
  { &hf_rrc_primaryCPICH_Info, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PrimaryCPICH_Info },
  { &hf_rrc_rg_CombinationIndex_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_0_5 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_E_RGCH_Combination_Info(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_E_RGCH_Combination_Info, E_RGCH_Combination_Info_sequence);

  return offset;
}


static const per_sequence_t E_RGCH_CombinationInfoList_sequence_of[1] = {
  { &hf_rrc_E_RGCH_CombinationInfoList_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_E_RGCH_Combination_Info },
};

static int
dissect_rrc_E_RGCH_CombinationInfoList(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_E_RGCH_CombinationInfoList, E_RGCH_CombinationInfoList_sequence_of,
                                                  1, maxEDCHRL, FALSE);

  return offset;
}


static const per_sequence_t SRNC_RelocationInfo_v860ext_IEs_sequence[] = {
  { &hf_rrc_e_RGCH_CombinationInfoList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_E_RGCH_CombinationInfoList },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_SRNC_RelocationInfo_v860ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_SRNC_RelocationInfo_v860ext_IEs, SRNC_RelocationInfo_v860ext_IEs_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions_243_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_nonCriticalExtensions_243(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_nonCriticalExtensions_243, T_nonCriticalExtensions_243_sequence);

  return offset;
}


static const per_sequence_t T_v860NonCriticalExtensions_36_sequence[] = {
  { &hf_rrc_sRNC_RelocationInfo_v860ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SRNC_RelocationInfo_v860ext_IEs },
  { &hf_rrc_nonCriticalExtensions_243, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_nonCriticalExtensions_243 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v860NonCriticalExtensions_36(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v860NonCriticalExtensions_36, T_v860NonCriticalExtensions_36_sequence);

  return offset;
}


static const per_sequence_t T_v820NonCriticalExtensions_03_sequence[] = {
  { &hf_rrc_srnc_RelocationInfo_v820ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SRNC_RelocationInfo_v820ext_IEs },
  { &hf_rrc_v860NonCriticalExtensions_36, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v860NonCriticalExtensions_36 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v820NonCriticalExtensions_03(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v820NonCriticalExtensions_03, T_v820NonCriticalExtensions_03_sequence);

  return offset;
}


static const per_sequence_t T_v770NonCriticalExtensions_51_sequence[] = {
  { &hf_rrc_sRNC_RelocationInfo_v770ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SRNC_RelocationInfo_v770ext_IEs },
  { &hf_rrc_v820NonCriticalExtensions_03, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v820NonCriticalExtensions_03 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v770NonCriticalExtensions_51(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v770NonCriticalExtensions_51, T_v770NonCriticalExtensions_51_sequence);

  return offset;
}


static const per_sequence_t T_v6b0NonCriticalExtensions_24_sequence[] = {
  { &hf_rrc_sRNC_RelocationInfo_v6b0ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SRNC_RelocationInfo_v6b0ext_IEs },
  { &hf_rrc_v770NonCriticalExtensions_51, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v770NonCriticalExtensions_51 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v6b0NonCriticalExtensions_24(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v6b0NonCriticalExtensions_24, T_v6b0NonCriticalExtensions_24_sequence);

  return offset;
}


static const per_sequence_t T_r6_12_sequence[] = {
  { &hf_rrc_sRNC_RelocationInfo_r6, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SRNC_RelocationInfo_r6_IEs },
  { &hf_rrc_sRNC_RelocationInfo_r6_add_ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_sRNC_RelocationInfo_r6_add_ext },
  { &hf_rrc_v6b0NonCriticalExtensions_25, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v6b0NonCriticalExtensions_24 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_r6_12(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_r6_12, T_r6_12_sequence);

  return offset;
}



static int
dissect_rrc_T_ueCapabilityContainer_04(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_bit_string_containing_pdu_new(tvb, offset, actx, tree, hf_index,
                                                              NO_BOUND, NO_BOUND, FALSE, dissect_UE_CapabilityContainer_IEs_PDU);

  return offset;
}



static int
dissect_rrc_T_ueCapabilityContainer_RSC_02(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_bit_string_containing_pdu_new(tvb, offset, actx, tree, hf_index,
                                                              NO_BOUND, NO_BOUND, FALSE, dissect_RRCConnectionSetupComplete_r3_add_ext_IEs_PDU);

  return offset;
}



static int
dissect_rrc_T_ueCapabilityContainer_UCI_02(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_bit_string_containing_pdu_new(tvb, offset, actx, tree, hf_index,
                                                              NO_BOUND, NO_BOUND, FALSE, dissect_UECapabilityInformation_r3_add_ext_IEs_PDU);

  return offset;
}


static const value_string rrc_MeasurementCommandWithType_r7_vals[] = {
  {   0, "setup" },
  {   1, "modify" },
  {   2, "release" },
  { 0, NULL }
};

static const per_choice_t MeasurementCommandWithType_r7_choice[] = {
  {   0, &hf_rrc_setup_03        , ASN1_NO_EXTENSIONS     , dissect_rrc_MeasurementType_r7 },
  {   1, &hf_rrc_modify_09       , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   2, &hf_rrc_release_01      , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_MeasurementCommandWithType_r7(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_MeasurementCommandWithType_r7, MeasurementCommandWithType_r7_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t OngoingMeasRep_r7_sequence[] = {
  { &hf_rrc_measurementIdentity, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MeasurementIdentity },
  { &hf_rrc_measurementCommandWithType_03, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MeasurementCommandWithType_r7 },
  { &hf_rrc_measurementReportingMode, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MeasurementReportingMode },
  { &hf_rrc_additionalMeasurementID_List, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_AdditionalMeasurementID_List },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_OngoingMeasRep_r7(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_OngoingMeasRep_r7, OngoingMeasRep_r7_sequence);

  return offset;
}


static const per_sequence_t OngoingMeasRepList_r7_sequence_of[1] = {
  { &hf_rrc_OngoingMeasRepList_r7_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_OngoingMeasRep_r7 },
};

static int
dissect_rrc_OngoingMeasRepList_r7(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_OngoingMeasRepList_r7, OngoingMeasRepList_r7_sequence_of,
                                                  1, maxNoOfMeas, FALSE);

  return offset;
}



static int
dissect_rrc_T_measurementReport_01(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_bit_string_containing_pdu_new(tvb, offset, actx, tree, hf_index,
                                                              NO_BOUND, NO_BOUND, FALSE, dissect_MeasurementReport_PDU);

  return offset;
}


static const per_sequence_t SRNC_RelocationInfo_r7_IEs_sequence[] = {
  { &hf_rrc_rb_IdentityForHOMessage, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RB_Identity },
  { &hf_rrc_stateOfRRC      , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_StateOfRRC },
  { &hf_rrc_stateOfRRC_Procedure, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_StateOfRRC_Procedure },
  { &hf_rrc_cipheringStatusList_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CipheringStatusList_r4 },
  { &hf_rrc_latestConfiguredCN_Domain, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CN_DomainIdentity },
  { &hf_rrc_calculationTimeForCiphering, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CalculationTimeForCiphering },
  { &hf_rrc_count_C_List    , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_COUNT_C_List },
  { &hf_rrc_cipheringInfoPerRB_List_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CipheringInfoPerRB_List_r4 },
  { &hf_rrc_integrityProtectionStatus, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_IntegrityProtectionStatus },
  { &hf_rrc_srb_SpecificIntegrityProtInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_SRB_SpecificIntegrityProtInfoList },
  { &hf_rrc_implementationSpecificParams, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_ImplementationSpecificParams },
  { &hf_rrc_u_RNTI          , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_U_RNTI },
  { &hf_rrc_c_RNTI          , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_C_RNTI },
  { &hf_rrc_ue_RadioAccessCapability_05, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UE_RadioAccessCapability_r6 },
  { &hf_rrc_ue_RadioAccessCapability_ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_RadioAccessCapabBandFDDList },
  { &hf_rrc_ue_Positioning_LastKnownPos, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_Positioning_LastKnownPos },
  { &hf_rrc_uESpecificBehaviourInformation1idle, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UESpecificBehaviourInformation1idle },
  { &hf_rrc_uESpecificBehaviourInformation1interRAT, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UESpecificBehaviourInformation1interRAT },
  { &hf_rrc_ueCapabilityContainer_04, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_ueCapabilityContainer_04 },
  { &hf_rrc_ueCapabilityContainer_RSC_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_ueCapabilityContainer_RSC_02 },
  { &hf_rrc_ueCapabilityContainer_UCI_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_ueCapabilityContainer_UCI_02 },
  { &hf_rrc_ue_RATSpecificCapability_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_InterRAT_UE_RadioAccessCapabilityList_r5 },
  { &hf_rrc_ura_Identity    , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_URA_Identity },
  { &hf_rrc_srns_t_305      , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_305 },
  { &hf_rrc_cn_CommonGSM_MAP_NAS_SysInfo_04, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_NAS_SystemInformationGSM_MAP },
  { &hf_rrc_cn_DomainInformationList_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CN_DomainInformationListFull },
  { &hf_rrc_ongoingMeasRepList_04, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_OngoingMeasRepList_r7 },
  { &hf_rrc_interRATCellInfoIndication, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_InterRATCellInfoIndication },
  { &hf_rrc_predefinedConfigStatusList_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PredefinedConfigStatusList },
  { &hf_rrc_srb_InformationList_03, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SRB_InformationSetupList_r7 },
  { &hf_rrc_rab_InformationList_05, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RAB_InformationSetupList_r7 },
  { &hf_rrc_ul_CommonTransChInfo_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_CommonTransChInfo_r4 },
  { &hf_rrc_ul_TransChInfoList_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_AddReconfTransChInfoList_r7 },
  { &hf_rrc_dl_CommonTransChInfo_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_CommonTransChInfo_r4 },
  { &hf_rrc_dl_TransChInfoList_03, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_AddReconfTransChInfoList_r7 },
  { &hf_rrc_tpc_CombinationInfoList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_TPC_CombinationInfoList },
  { &hf_rrc_storedCompressedModeInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_StoredCompressedModeInfo },
  { &hf_rrc_measurementReport_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_measurementReport_01 },
  { &hf_rrc_failureCause    , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_FailureCauseWithProtErr },
  { &hf_rrc_mbms_JoinedInformation, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MBMS_JoinedInformation_r6 },
  { &hf_rrc_mbmsSelectedServiceInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MBMS_SelectedServiceInfo },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_SRNC_RelocationInfo_r7_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_SRNC_RelocationInfo_r7_IEs, SRNC_RelocationInfo_r7_IEs_sequence);

  return offset;
}



static int
dissect_rrc_T_sRNC_RelocationInfo_r7_add_ext(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_bit_string_containing_pdu_new(tvb, offset, actx, tree, hf_index,
                                                              NO_BOUND, NO_BOUND, FALSE, dissect_SRNC_RelocationInfo_r7_add_ext_IEs_PDU);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions_244_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_nonCriticalExtensions_244(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_nonCriticalExtensions_244, T_nonCriticalExtensions_244_sequence);

  return offset;
}


static const per_sequence_t T_v860NonCriticalExtensions_37_sequence[] = {
  { &hf_rrc_sRNC_RelocationInfo_v860ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SRNC_RelocationInfo_v860ext_IEs },
  { &hf_rrc_nonCriticalExtensions_244, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_nonCriticalExtensions_244 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v860NonCriticalExtensions_37(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v860NonCriticalExtensions_37, T_v860NonCriticalExtensions_37_sequence);

  return offset;
}


static const per_sequence_t T_v820NonCriticalExtensions_04_sequence[] = {
  { &hf_rrc_srnc_RelocationInfo_v820ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SRNC_RelocationInfo_v820ext_IEs },
  { &hf_rrc_v860NonCriticalExtensions_37, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v860NonCriticalExtensions_37 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v820NonCriticalExtensions_04(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v820NonCriticalExtensions_04, T_v820NonCriticalExtensions_04_sequence);

  return offset;
}


static const per_sequence_t T_r7_16_sequence[] = {
  { &hf_rrc_sRNC_RelocationInfo_r7, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SRNC_RelocationInfo_r7_IEs },
  { &hf_rrc_sRNC_RelocationInfo_r7_add_ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_sRNC_RelocationInfo_r7_add_ext },
  { &hf_rrc_v820NonCriticalExtensions_04, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v820NonCriticalExtensions_04 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_r7_16(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_r7_16, T_r7_16_sequence);

  return offset;
}



static int
dissect_rrc_T_ueCapabilityContainer_05(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_bit_string_containing_pdu_new(tvb, offset, actx, tree, hf_index,
                                                              NO_BOUND, NO_BOUND, FALSE, dissect_UE_CapabilityContainer_IEs_PDU);

  return offset;
}



static int
dissect_rrc_T_ueCapabilityContainer_RSC_03(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_bit_string_containing_pdu_new(tvb, offset, actx, tree, hf_index,
                                                              NO_BOUND, NO_BOUND, FALSE, dissect_RRCConnectionSetupComplete_r3_add_ext_IEs_PDU);

  return offset;
}



static int
dissect_rrc_T_ueCapabilityContainer_UCI_03(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_bit_string_containing_pdu_new(tvb, offset, actx, tree, hf_index,
                                                              NO_BOUND, NO_BOUND, FALSE, dissect_UECapabilityInformation_r3_add_ext_IEs_PDU);

  return offset;
}


static const value_string rrc_MeasurementCommandWithType_r8_vals[] = {
  {   0, "setup" },
  {   1, "modify" },
  {   2, "release" },
  { 0, NULL }
};

static const per_choice_t MeasurementCommandWithType_r8_choice[] = {
  {   0, &hf_rrc_setup_04        , ASN1_NO_EXTENSIONS     , dissect_rrc_MeasurementType_r8 },
  {   1, &hf_rrc_modify_09       , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   2, &hf_rrc_release_01      , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_MeasurementCommandWithType_r8(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_MeasurementCommandWithType_r8, MeasurementCommandWithType_r8_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t OngoingMeasRep_r8_sequence[] = {
  { &hf_rrc_measurementIdentity, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MeasurementIdentity },
  { &hf_rrc_measurementCommandWithType_04, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MeasurementCommandWithType_r8 },
  { &hf_rrc_measurementReportingMode, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MeasurementReportingMode },
  { &hf_rrc_additionalMeasurementID_List, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_AdditionalMeasurementID_List },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_OngoingMeasRep_r8(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_OngoingMeasRep_r8, OngoingMeasRep_r8_sequence);

  return offset;
}


static const per_sequence_t OngoingMeasRepList_r8_sequence_of[1] = {
  { &hf_rrc_OngoingMeasRepList_r8_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_OngoingMeasRep_r8 },
};

static int
dissect_rrc_OngoingMeasRepList_r8(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_OngoingMeasRepList_r8, OngoingMeasRepList_r8_sequence_of,
                                                  1, maxNoOfMeas, FALSE);

  return offset;
}


static const per_sequence_t T_active_01_sequence[] = {
  { &hf_rrc_tgcfn           , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TGCFN },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_active_01(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_active_01, T_active_01_sequence);

  return offset;
}


static const value_string rrc_T_current_tgps_Status_01_vals[] = {
  {   0, "active" },
  {   1, "inactive" },
  { 0, NULL }
};

static const per_choice_t T_current_tgps_Status_01_choice[] = {
  {   0, &hf_rrc_active_01       , ASN1_NO_EXTENSIONS     , dissect_rrc_T_active_01 },
  {   1, &hf_rrc_inactive        , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_current_tgps_Status_01(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_current_tgps_Status_01, T_current_tgps_Status_01_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t StoredTGP_Sequence_r8_sequence[] = {
  { &hf_rrc_tgpsi           , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TGPSI },
  { &hf_rrc_current_tgps_Status_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_current_tgps_Status_01 },
  { &hf_rrc_tgps_ConfigurationParams_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_TGPS_ConfigurationParams_r8 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_StoredTGP_Sequence_r8(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_StoredTGP_Sequence_r8, StoredTGP_Sequence_r8_sequence);

  return offset;
}


static const per_sequence_t StoredTGP_SequenceList_r8_sequence_of[1] = {
  { &hf_rrc_StoredTGP_SequenceList_r8_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_StoredTGP_Sequence_r8 },
};

static int
dissect_rrc_StoredTGP_SequenceList_r8(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_StoredTGP_SequenceList_r8, StoredTGP_SequenceList_r8_sequence_of,
                                                  1, maxTGPS, FALSE);

  return offset;
}


static const per_sequence_t StoredCompressedModeInfo_r8_sequence[] = {
  { &hf_rrc_storedTGP_SequenceList_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_StoredTGP_SequenceList_r8 },
  { &hf_rrc_codeChangeStatusList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CodeChangeStatusList },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_StoredCompressedModeInfo_r8(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_StoredCompressedModeInfo_r8, StoredCompressedModeInfo_r8_sequence);

  return offset;
}



static int
dissect_rrc_T_measurementReport_02(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_bit_string_containing_pdu_new(tvb, offset, actx, tree, hf_index,
                                                              NO_BOUND, NO_BOUND, FALSE, dissect_MeasurementReport_PDU);

  return offset;
}


static const per_sequence_t SRNC_RelocationInfo_r8_IEs_sequence[] = {
  { &hf_rrc_rb_IdentityForHOMessage, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RB_Identity },
  { &hf_rrc_stateOfRRC      , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_StateOfRRC },
  { &hf_rrc_stateOfRRC_Procedure, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_StateOfRRC_Procedure },
  { &hf_rrc_cipheringStatusList_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CipheringStatusList_r4 },
  { &hf_rrc_latestConfiguredCN_Domain, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CN_DomainIdentity },
  { &hf_rrc_calculationTimeForCiphering, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CalculationTimeForCiphering },
  { &hf_rrc_count_C_List    , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_COUNT_C_List },
  { &hf_rrc_cipheringInfoPerRB_List_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CipheringInfoPerRB_List_r4 },
  { &hf_rrc_integrityProtectionStatus, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_IntegrityProtectionStatus },
  { &hf_rrc_srb_SpecificIntegrityProtInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_SRB_SpecificIntegrityProtInfoList },
  { &hf_rrc_implementationSpecificParams, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_ImplementationSpecificParams },
  { &hf_rrc_u_RNTI          , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_U_RNTI },
  { &hf_rrc_c_RNTI          , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_C_RNTI },
  { &hf_rrc_ue_RadioAccessCapability_05, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UE_RadioAccessCapability_r6 },
  { &hf_rrc_ue_RadioAccessCapability_ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_RadioAccessCapabBandFDDList },
  { &hf_rrc_ue_Positioning_LastKnownPos, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_Positioning_LastKnownPos },
  { &hf_rrc_uESpecificBehaviourInformation1idle, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UESpecificBehaviourInformation1idle },
  { &hf_rrc_uESpecificBehaviourInformation1interRAT, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UESpecificBehaviourInformation1interRAT },
  { &hf_rrc_ueCapabilityContainer_05, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_ueCapabilityContainer_05 },
  { &hf_rrc_ueCapabilityContainer_RSC_03, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_ueCapabilityContainer_RSC_03 },
  { &hf_rrc_ueCapabilityContainer_UCI_03, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_ueCapabilityContainer_UCI_03 },
  { &hf_rrc_ue_RATSpecificCapability_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_InterRAT_UE_RadioAccessCapabilityList_r5 },
  { &hf_rrc_ura_Identity    , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_URA_Identity },
  { &hf_rrc_srns_t_305      , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_305 },
  { &hf_rrc_cn_CommonGSM_MAP_NAS_SysInfo_04, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_NAS_SystemInformationGSM_MAP },
  { &hf_rrc_cn_DomainInformationList_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CN_DomainInformationListFull },
  { &hf_rrc_ongoingMeasRepList_05, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_OngoingMeasRepList_r8 },
  { &hf_rrc_interRATCellInfoIndication, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_InterRATCellInfoIndication },
  { &hf_rrc_predefinedConfigStatusList_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PredefinedConfigStatusList },
  { &hf_rrc_srb_InformationList_04, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SRB_InformationSetupList_r8 },
  { &hf_rrc_rab_InformationList_06, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RAB_InformationSetupList_r8 },
  { &hf_rrc_ul_CommonTransChInfo_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_CommonTransChInfo_r4 },
  { &hf_rrc_ul_TransChInfoList_03, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_AddReconfTransChInfoList_r8 },
  { &hf_rrc_dl_CommonTransChInfo_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_CommonTransChInfo_r4 },
  { &hf_rrc_dl_TransChInfoList_03, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_AddReconfTransChInfoList_r7 },
  { &hf_rrc_tpc_CombinationInfoList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_TPC_CombinationInfoList },
  { &hf_rrc_e_RGCH_CombinationInfoList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_E_RGCH_CombinationInfoList },
  { &hf_rrc_storedCompressedModeInfo_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_StoredCompressedModeInfo_r8 },
  { &hf_rrc_measurementReport_03, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_measurementReport_02 },
  { &hf_rrc_failureCause    , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_FailureCauseWithProtErr },
  { &hf_rrc_ue_HistoryInformation, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_HistoryInformation },
  { &hf_rrc_mbms_JoinedInformation, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MBMS_JoinedInformation_r6 },
  { &hf_rrc_mbmsSelectedServiceInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MBMS_SelectedServiceInfo },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_SRNC_RelocationInfo_r8_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_SRNC_RelocationInfo_r8_IEs, SRNC_RelocationInfo_r8_IEs_sequence);

  return offset;
}


static const per_sequence_t SRNC_RelocationInfo_v8d0ext_IEs_sequence[] = {
  { &hf_rrc_pdcp_ROHC_TargetMode, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_PDCP_ROHC_TargetMode },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_SRNC_RelocationInfo_v8d0ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_SRNC_RelocationInfo_v8d0ext_IEs, SRNC_RelocationInfo_v8d0ext_IEs_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions_245_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_nonCriticalExtensions_245(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_nonCriticalExtensions_245, T_nonCriticalExtensions_245_sequence);

  return offset;
}


static const per_sequence_t T_v8d0NonCriticalExtensions_01_sequence[] = {
  { &hf_rrc_sRNC_RelocationInfo_v8d0ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SRNC_RelocationInfo_v8d0ext_IEs },
  { &hf_rrc_nonCriticalExtensions_245, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_nonCriticalExtensions_245 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v8d0NonCriticalExtensions_01(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v8d0NonCriticalExtensions_01, T_v8d0NonCriticalExtensions_01_sequence);

  return offset;
}


static const per_sequence_t T_v7f0NonCriticalExtensions_20_sequence[] = {
  { &hf_rrc_sRNC_RelocationInfo_v7f0ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SRNC_RelocationInfo_v7f0ext_IEs },
  { &hf_rrc_v8d0NonCriticalExtensions_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v8d0NonCriticalExtensions_01 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v7f0NonCriticalExtensions_20(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v7f0NonCriticalExtensions_20, T_v7f0NonCriticalExtensions_20_sequence);

  return offset;
}


static const per_sequence_t T_v7e0NonCriticalExtensions_07_sequence[] = {
  { &hf_rrc_sRNC_RelocationInfo_v7e0ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SRNC_RelocationInfo_v7e0ext_IEs },
  { &hf_rrc_v7f0NonCriticalExtensions_20, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v7f0NonCriticalExtensions_20 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v7e0NonCriticalExtensions_07(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v7e0NonCriticalExtensions_07, T_v7e0NonCriticalExtensions_07_sequence);

  return offset;
}


static const per_sequence_t T_r8_12_sequence[] = {
  { &hf_rrc_sRNC_RelocationInfo_r8, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SRNC_RelocationInfo_r8_IEs },
  { &hf_rrc_sRNC_RelocationInfo_r8_add_ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_BIT_STRING },
  { &hf_rrc_v7e0NonCriticalExtensions_07, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v7e0NonCriticalExtensions_07 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_r8_12(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_r8_12, T_r8_12_sequence);

  return offset;
}



static int
dissect_rrc_T_ueCapabilityContainer_06(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_bit_string_containing_pdu_new(tvb, offset, actx, tree, hf_index,
                                                              NO_BOUND, NO_BOUND, FALSE, dissect_UE_CapabilityContainer_IEs_PDU);

  return offset;
}



static int
dissect_rrc_T_ueCapabilityContainer_RSC_04(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_bit_string_containing_pdu_new(tvb, offset, actx, tree, hf_index,
                                                              NO_BOUND, NO_BOUND, FALSE, dissect_RRCConnectionSetupComplete_r3_add_ext_IEs_PDU);

  return offset;
}



static int
dissect_rrc_T_ueCapabilityContainer_UCI_04(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_bit_string_containing_pdu_new(tvb, offset, actx, tree, hf_index,
                                                              NO_BOUND, NO_BOUND, FALSE, dissect_UECapabilityInformation_r3_add_ext_IEs_PDU);

  return offset;
}


static const value_string rrc_MeasurementCommandWithType_r9_vals[] = {
  {   0, "setup" },
  {   1, "modify" },
  {   2, "release" },
  { 0, NULL }
};

static const per_choice_t MeasurementCommandWithType_r9_choice[] = {
  {   0, &hf_rrc_setup_05        , ASN1_NO_EXTENSIONS     , dissect_rrc_MeasurementType_r9 },
  {   1, &hf_rrc_modify_09       , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   2, &hf_rrc_release_01      , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_MeasurementCommandWithType_r9(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_MeasurementCommandWithType_r9, MeasurementCommandWithType_r9_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t OngoingMeasRep_r9_sequence[] = {
  { &hf_rrc_measurementIdentity, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MeasurementIdentity },
  { &hf_rrc_measurementCommandWithType_05, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MeasurementCommandWithType_r9 },
  { &hf_rrc_measurementReportingMode, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MeasurementReportingMode },
  { &hf_rrc_additionalMeasurementID_List, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_AdditionalMeasurementID_List },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_OngoingMeasRep_r9(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_OngoingMeasRep_r9, OngoingMeasRep_r9_sequence);

  return offset;
}


static const per_sequence_t OngoingMeasRepList_r9_sequence_of[1] = {
  { &hf_rrc_OngoingMeasRepList_r9_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_OngoingMeasRep_r9 },
};

static int
dissect_rrc_OngoingMeasRepList_r9(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_OngoingMeasRepList_r9, OngoingMeasRepList_r9_sequence_of,
                                                  1, maxNoOfMeas, FALSE);

  return offset;
}


static const per_sequence_t TPC_Combination_Info_r9_sequence[] = {
  { &hf_rrc_primaryCPICH_Info, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PrimaryCPICH_Info },
  { &hf_rrc_tpc_CombinationIndex, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TPC_CombinationIndex },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_TPC_Combination_Info_r9(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_TPC_Combination_Info_r9, TPC_Combination_Info_r9_sequence);

  return offset;
}


static const per_sequence_t TPC_CombinationInfoList_r9_sequence_of[1] = {
  { &hf_rrc_TPC_CombinationInfoList_r9_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_TPC_Combination_Info_r9 },
};

static int
dissect_rrc_TPC_CombinationInfoList_r9(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_TPC_CombinationInfoList_r9, TPC_CombinationInfoList_r9_sequence_of,
                                                  1, maxEDCHRL, FALSE);

  return offset;
}


static const per_sequence_t E_RGCH_Combination_Info_r9_sequence[] = {
  { &hf_rrc_primaryCPICH_Info, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PrimaryCPICH_Info },
  { &hf_rrc_rg_CombinationIndex_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_0_5 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_E_RGCH_Combination_Info_r9(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_E_RGCH_Combination_Info_r9, E_RGCH_Combination_Info_r9_sequence);

  return offset;
}


static const per_sequence_t E_RGCH_CombinationInfoList_r9_sequence_of[1] = {
  { &hf_rrc_E_RGCH_CombinationInfoList_r9_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_E_RGCH_Combination_Info_r9 },
};

static int
dissect_rrc_E_RGCH_CombinationInfoList_r9(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_E_RGCH_CombinationInfoList_r9, E_RGCH_CombinationInfoList_r9_sequence_of,
                                                  1, maxEDCHRL, FALSE);

  return offset;
}



static int
dissect_rrc_T_measurementReport_03(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_bit_string_containing_pdu_new(tvb, offset, actx, tree, hf_index,
                                                              NO_BOUND, NO_BOUND, FALSE, dissect_MeasurementReport_PDU);

  return offset;
}


static const per_sequence_t SRNC_RelocationInfo_r9_IEs_sequence[] = {
  { &hf_rrc_rb_IdentityForHOMessage, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RB_Identity },
  { &hf_rrc_stateOfRRC      , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_StateOfRRC },
  { &hf_rrc_stateOfRRC_Procedure, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_StateOfRRC_Procedure },
  { &hf_rrc_cipheringStatusList_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CipheringStatusList_r4 },
  { &hf_rrc_latestConfiguredCN_Domain, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CN_DomainIdentity },
  { &hf_rrc_calculationTimeForCiphering, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CalculationTimeForCiphering },
  { &hf_rrc_count_C_List    , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_COUNT_C_List },
  { &hf_rrc_cipheringInfoPerRB_List_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CipheringInfoPerRB_List_r4 },
  { &hf_rrc_integrityProtectionStatus, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_IntegrityProtectionStatus },
  { &hf_rrc_srb_SpecificIntegrityProtInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_SRB_SpecificIntegrityProtInfoList },
  { &hf_rrc_implementationSpecificParams, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_ImplementationSpecificParams },
  { &hf_rrc_u_RNTI          , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_U_RNTI },
  { &hf_rrc_c_RNTI          , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_C_RNTI },
  { &hf_rrc_ue_RadioAccessCapability_05, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UE_RadioAccessCapability_r6 },
  { &hf_rrc_ue_RadioAccessCapability_ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_RadioAccessCapabBandFDDList },
  { &hf_rrc_ue_Positioning_LastKnownPos, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_Positioning_LastKnownPos },
  { &hf_rrc_uESpecificBehaviourInformation1idle, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UESpecificBehaviourInformation1idle },
  { &hf_rrc_uESpecificBehaviourInformation1interRAT, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UESpecificBehaviourInformation1interRAT },
  { &hf_rrc_ueCapabilityContainer_06, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_ueCapabilityContainer_06 },
  { &hf_rrc_ueCapabilityContainer_RSC_04, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_ueCapabilityContainer_RSC_04 },
  { &hf_rrc_ueCapabilityContainer_UCI_04, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_ueCapabilityContainer_UCI_04 },
  { &hf_rrc_ue_RATSpecificCapability_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_InterRAT_UE_RadioAccessCapabilityList_r5 },
  { &hf_rrc_ura_Identity    , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_URA_Identity },
  { &hf_rrc_srns_t_305      , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_305 },
  { &hf_rrc_cn_CommonGSM_MAP_NAS_SysInfo_04, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_NAS_SystemInformationGSM_MAP },
  { &hf_rrc_cn_DomainInformationList_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CN_DomainInformationListFull },
  { &hf_rrc_ongoingMeasRepList_06, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_OngoingMeasRepList_r9 },
  { &hf_rrc_interRATCellInfoIndication, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_InterRATCellInfoIndication },
  { &hf_rrc_predefinedConfigStatusList_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PredefinedConfigStatusList },
  { &hf_rrc_srb_InformationList_04, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SRB_InformationSetupList_r8 },
  { &hf_rrc_rab_InformationList_06, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RAB_InformationSetupList_r8 },
  { &hf_rrc_ul_CommonTransChInfo_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_CommonTransChInfo_r4 },
  { &hf_rrc_ul_TransChInfoList_03, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_AddReconfTransChInfoList_r8 },
  { &hf_rrc_dl_CommonTransChInfo_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_CommonTransChInfo_r4 },
  { &hf_rrc_dl_TransChInfoList_04, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_AddReconfTransChInfoList_r9 },
  { &hf_rrc_tpc_CombinationInfoList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_TPC_CombinationInfoList },
  { &hf_rrc_e_RGCH_CombinationInfoList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_E_RGCH_CombinationInfoList },
  { &hf_rrc_storedCompressedModeInfo_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_StoredCompressedModeInfo_r8 },
  { &hf_rrc_secondary_tpc_CombinationInfoList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_TPC_CombinationInfoList_r9 },
  { &hf_rrc_secondary_e_RGCH_CombinationInfoList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_E_RGCH_CombinationInfoList_r9 },
  { &hf_rrc_measurementReport_04, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_measurementReport_03 },
  { &hf_rrc_failureCause    , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_FailureCauseWithProtErr },
  { &hf_rrc_ue_HistoryInformation, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_HistoryInformation },
  { &hf_rrc_mbms_JoinedInformation, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MBMS_JoinedInformation_r6 },
  { &hf_rrc_mbmsSelectedServiceInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MBMS_SelectedServiceInfo },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_SRNC_RelocationInfo_r9_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_SRNC_RelocationInfo_r9_IEs, SRNC_RelocationInfo_r9_IEs_sequence);

  return offset;
}


static const per_sequence_t OngoingMeasRep_v970ext_IEs_sequence[] = {
  { &hf_rrc_measurementIdentity_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MeasurementIdentity_r9 },
  { &hf_rrc_additionalMeasurementID_List_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_AdditionalMeasurementID_List_r9 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_OngoingMeasRep_v970ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_OngoingMeasRep_v970ext_IEs, OngoingMeasRep_v970ext_IEs_sequence);

  return offset;
}


static const per_sequence_t OngoingMeasRepList_v970ext_IE_sequence_of[1] = {
  { &hf_rrc_OngoingMeasRepList_v970ext_IE_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_OngoingMeasRep_v970ext_IEs },
};

static int
dissect_rrc_OngoingMeasRepList_v970ext_IE(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_OngoingMeasRepList_v970ext_IE, OngoingMeasRepList_v970ext_IE_sequence_of,
                                                  1, maxNoOfMeas, FALSE);

  return offset;
}


static const per_sequence_t SRNC_RelocationInfo_v970ext_IEs_sequence[] = {
  { &hf_rrc_ongoingMeasRepList_07, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_OngoingMeasRepList_v970ext_IE },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_SRNC_RelocationInfo_v970ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_SRNC_RelocationInfo_v970ext_IEs, SRNC_RelocationInfo_v970ext_IEs_sequence);

  return offset;
}


static const per_sequence_t SRNC_RelocationInfo_v9c0ext_IEs_sequence[] = {
  { &hf_rrc_dl_AddReconfTransChInfoList_05, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_AddReconfTransChInfoList_TDD128_v9c0ext },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_SRNC_RelocationInfo_v9c0ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_SRNC_RelocationInfo_v9c0ext_IEs, SRNC_RelocationInfo_v9c0ext_IEs_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions_246_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_nonCriticalExtensions_246(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_nonCriticalExtensions_246, T_nonCriticalExtensions_246_sequence);

  return offset;
}


static const per_sequence_t T_v9c0NonCriticalExtensions_13_sequence[] = {
  { &hf_rrc_sRNC_RelocationInfo_v9c0ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SRNC_RelocationInfo_v9c0ext_IEs },
  { &hf_rrc_nonCriticalExtensions_246, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_nonCriticalExtensions_246 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v9c0NonCriticalExtensions_13(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v9c0NonCriticalExtensions_13, T_v9c0NonCriticalExtensions_13_sequence);

  return offset;
}


static const per_sequence_t T_v970NonCriticalExtensions_04_sequence[] = {
  { &hf_rrc_sRNC_RelocationInfo_v970ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SRNC_RelocationInfo_v970ext_IEs },
  { &hf_rrc_v9c0NonCriticalExtensions_13, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v9c0NonCriticalExtensions_13 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v970NonCriticalExtensions_04(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v970NonCriticalExtensions_04, T_v970NonCriticalExtensions_04_sequence);

  return offset;
}


static const per_sequence_t T_v8d0NonCriticalExtensions_02_sequence[] = {
  { &hf_rrc_sRNC_RelocationInfo_v8d0ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SRNC_RelocationInfo_v8d0ext_IEs },
  { &hf_rrc_v970NonCriticalExtensions_04, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v970NonCriticalExtensions_04 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v8d0NonCriticalExtensions_02(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v8d0NonCriticalExtensions_02, T_v8d0NonCriticalExtensions_02_sequence);

  return offset;
}


static const per_sequence_t T_r9_11_sequence[] = {
  { &hf_rrc_sRNC_RelocationInfo_r9, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SRNC_RelocationInfo_r9_IEs },
  { &hf_rrc_sRNC_RelocationInfo_r9_add_ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_BIT_STRING },
  { &hf_rrc_v8d0NonCriticalExtensions_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v8d0NonCriticalExtensions_02 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_r9_11(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_r9_11, T_r9_11_sequence);

  return offset;
}



static int
dissect_rrc_T_ueCapabilityContainer_07(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_bit_string_containing_pdu_new(tvb, offset, actx, tree, hf_index,
                                                              NO_BOUND, NO_BOUND, FALSE, dissect_UE_CapabilityContainer_IEs_PDU);

  return offset;
}



static int
dissect_rrc_T_ueCapabilityContainer_RSC_05(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_bit_string_containing_pdu_new(tvb, offset, actx, tree, hf_index,
                                                              NO_BOUND, NO_BOUND, FALSE, dissect_RRCConnectionSetupComplete_r3_add_ext_IEs_PDU);

  return offset;
}



static int
dissect_rrc_T_ueCapabilityContainer_UCI_05(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_bit_string_containing_pdu_new(tvb, offset, actx, tree, hf_index,
                                                              NO_BOUND, NO_BOUND, FALSE, dissect_UECapabilityInformation_r3_add_ext_IEs_PDU);

  return offset;
}


static const value_string rrc_MeasurementCommandWithType_r10_vals[] = {
  {   0, "setup" },
  {   1, "modify" },
  {   2, "release" },
  { 0, NULL }
};

static const per_choice_t MeasurementCommandWithType_r10_choice[] = {
  {   0, &hf_rrc_setup_06        , ASN1_NO_EXTENSIONS     , dissect_rrc_MeasurementType_r10 },
  {   1, &hf_rrc_modify_09       , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   2, &hf_rrc_release_01      , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_MeasurementCommandWithType_r10(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_MeasurementCommandWithType_r10, MeasurementCommandWithType_r10_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t OngoingMeasRep_r10_sequence[] = {
  { &hf_rrc_measurementIdentity, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MeasurementIdentity },
  { &hf_rrc_measurementCommandWithType_06, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MeasurementCommandWithType_r10 },
  { &hf_rrc_measurementReportingMode, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MeasurementReportingMode },
  { &hf_rrc_additionalMeasurementID_List, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_AdditionalMeasurementID_List },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_OngoingMeasRep_r10(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_OngoingMeasRep_r10, OngoingMeasRep_r10_sequence);

  return offset;
}


static const per_sequence_t OngoingMeasRepList_r10_sequence_of[1] = {
  { &hf_rrc_OngoingMeasRepList_r10_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_OngoingMeasRep_r10 },
};

static int
dissect_rrc_OngoingMeasRepList_r10(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_OngoingMeasRepList_r10, OngoingMeasRepList_r10_sequence_of,
                                                  1, maxNoOfMeas, FALSE);

  return offset;
}



static int
dissect_rrc_T_measurementReport_04(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_bit_string_containing_pdu_new(tvb, offset, actx, tree, hf_index,
                                                              NO_BOUND, NO_BOUND, FALSE, dissect_MeasurementReport_PDU);

  return offset;
}


static const per_sequence_t SRNC_RelocationInfo_r10_IEs_sequence[] = {
  { &hf_rrc_rb_IdentityForHOMessage, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RB_Identity },
  { &hf_rrc_stateOfRRC      , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_StateOfRRC },
  { &hf_rrc_stateOfRRC_Procedure, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_StateOfRRC_Procedure },
  { &hf_rrc_cipheringStatusList_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CipheringStatusList_r4 },
  { &hf_rrc_latestConfiguredCN_Domain, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CN_DomainIdentity },
  { &hf_rrc_calculationTimeForCiphering, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CalculationTimeForCiphering },
  { &hf_rrc_count_C_List    , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_COUNT_C_List },
  { &hf_rrc_cipheringInfoPerRB_List_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CipheringInfoPerRB_List_r4 },
  { &hf_rrc_integrityProtectionStatus, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_IntegrityProtectionStatus },
  { &hf_rrc_srb_SpecificIntegrityProtInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_SRB_SpecificIntegrityProtInfoList },
  { &hf_rrc_implementationSpecificParams, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_ImplementationSpecificParams },
  { &hf_rrc_u_RNTI          , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_U_RNTI },
  { &hf_rrc_c_RNTI          , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_C_RNTI },
  { &hf_rrc_ue_RadioAccessCapability_05, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UE_RadioAccessCapability_r6 },
  { &hf_rrc_ue_RadioAccessCapability_ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_RadioAccessCapabBandFDDList },
  { &hf_rrc_ue_Positioning_LastKnownPos, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_Positioning_LastKnownPos },
  { &hf_rrc_uESpecificBehaviourInformation1idle, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UESpecificBehaviourInformation1idle },
  { &hf_rrc_uESpecificBehaviourInformation1interRAT, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UESpecificBehaviourInformation1interRAT },
  { &hf_rrc_ueCapabilityContainer_07, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_ueCapabilityContainer_07 },
  { &hf_rrc_ueCapabilityContainer_RSC_05, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_ueCapabilityContainer_RSC_05 },
  { &hf_rrc_ueCapabilityContainer_UCI_05, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_ueCapabilityContainer_UCI_05 },
  { &hf_rrc_ue_RATSpecificCapability_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_InterRAT_UE_RadioAccessCapabilityList_r5 },
  { &hf_rrc_ura_Identity    , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_URA_Identity },
  { &hf_rrc_srns_t_305      , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_305 },
  { &hf_rrc_cn_CommonGSM_MAP_NAS_SysInfo_04, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_NAS_SystemInformationGSM_MAP },
  { &hf_rrc_cn_DomainInformationList_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CN_DomainInformationListFull },
  { &hf_rrc_ongoingMeasRepList_08, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_OngoingMeasRepList_r10 },
  { &hf_rrc_interRATCellInfoIndication, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_InterRATCellInfoIndication },
  { &hf_rrc_predefinedConfigStatusList_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PredefinedConfigStatusList },
  { &hf_rrc_srb_InformationList_04, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SRB_InformationSetupList_r8 },
  { &hf_rrc_rab_InformationList_06, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RAB_InformationSetupList_r8 },
  { &hf_rrc_pdcp_ROHC_TargetMode, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_PDCP_ROHC_TargetMode },
  { &hf_rrc_ul_CommonTransChInfo_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_CommonTransChInfo_r4 },
  { &hf_rrc_ul_TransChInfoList_03, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_AddReconfTransChInfoList_r8 },
  { &hf_rrc_dl_CommonTransChInfo_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_CommonTransChInfo_r4 },
  { &hf_rrc_dl_TransChInfoList_04, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_AddReconfTransChInfoList_r9 },
  { &hf_rrc_tpc_CombinationInfoList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_TPC_CombinationInfoList },
  { &hf_rrc_e_RGCH_CombinationInfoList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_E_RGCH_CombinationInfoList },
  { &hf_rrc_storedCompressedModeInfo_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_StoredCompressedModeInfo_r8 },
  { &hf_rrc_secondary_tpc_CombinationInfoList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_TPC_CombinationInfoList_r9 },
  { &hf_rrc_secondary_e_RGCH_CombinationInfoList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_E_RGCH_CombinationInfoList_r9 },
  { &hf_rrc_measurementReport_05, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_measurementReport_04 },
  { &hf_rrc_failureCause    , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_FailureCauseWithProtErr },
  { &hf_rrc_ue_HistoryInformation, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_HistoryInformation },
  { &hf_rrc_mbms_JoinedInformation, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MBMS_JoinedInformation_r6 },
  { &hf_rrc_mbmsSelectedServiceInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MBMS_SelectedServiceInfo },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_SRNC_RelocationInfo_r10_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_SRNC_RelocationInfo_r10_IEs, SRNC_RelocationInfo_r10_IEs_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions_247_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_nonCriticalExtensions_247(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_nonCriticalExtensions_247, T_nonCriticalExtensions_247_sequence);

  return offset;
}


static const per_sequence_t T_r10_13_sequence[] = {
  { &hf_rrc_sRNC_RelocationInfo_r10, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SRNC_RelocationInfo_r10_IEs },
  { &hf_rrc_sRNC_RelocationInfo_r10_add_ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_BIT_STRING },
  { &hf_rrc_nonCriticalExtensions_247, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_nonCriticalExtensions_247 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_r10_13(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_r10_13, T_r10_13_sequence);

  return offset;
}



static int
dissect_rrc_T_ueCapabilityContainer_08(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_bit_string_containing_pdu_new(tvb, offset, actx, tree, hf_index,
                                                              NO_BOUND, NO_BOUND, FALSE, dissect_UE_CapabilityContainer_IEs_PDU);

  return offset;
}



static int
dissect_rrc_T_ueCapabilityContainer_RSC_06(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_bit_string_containing_pdu_new(tvb, offset, actx, tree, hf_index,
                                                              NO_BOUND, NO_BOUND, FALSE, dissect_RRCConnectionSetupComplete_r3_add_ext_IEs_PDU);

  return offset;
}



static int
dissect_rrc_T_ueCapabilityContainer_UCI_06(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_bit_string_containing_pdu_new(tvb, offset, actx, tree, hf_index,
                                                              NO_BOUND, NO_BOUND, FALSE, dissect_UECapabilityInformation_r3_add_ext_IEs_PDU);

  return offset;
}


static const value_string rrc_MeasurementCommandWithType_r11_vals[] = {
  {   0, "setup" },
  {   1, "modify" },
  {   2, "release" },
  { 0, NULL }
};

static const per_choice_t MeasurementCommandWithType_r11_choice[] = {
  {   0, &hf_rrc_setup_07        , ASN1_NO_EXTENSIONS     , dissect_rrc_MeasurementType_r11 },
  {   1, &hf_rrc_modify_09       , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  {   2, &hf_rrc_release_01      , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_MeasurementCommandWithType_r11(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_MeasurementCommandWithType_r11, MeasurementCommandWithType_r11_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t OngoingMeasRep_r11_sequence[] = {
  { &hf_rrc_measurementIdentity_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MeasurementIdentity_r9 },
  { &hf_rrc_measurementCommandWithType_07, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_MeasurementCommandWithType_r11 },
  { &hf_rrc_measurementReportingMode, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MeasurementReportingMode },
  { &hf_rrc_additionalMeasurementID_List_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_AdditionalMeasurementID_List_r9 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_OngoingMeasRep_r11(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_OngoingMeasRep_r11, OngoingMeasRep_r11_sequence);

  return offset;
}


static const per_sequence_t OngoingMeasRepList_r11_sequence_of[1] = {
  { &hf_rrc_OngoingMeasRepList_r11_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_OngoingMeasRep_r11 },
};

static int
dissect_rrc_OngoingMeasRepList_r11(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_OngoingMeasRepList_r11, OngoingMeasRepList_r11_sequence_of,
                                                  1, maxNoOfMeas, FALSE);

  return offset;
}



static int
dissect_rrc_T_measurementReport_05(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_bit_string_containing_pdu_new(tvb, offset, actx, tree, hf_index,
                                                              NO_BOUND, NO_BOUND, FALSE, dissect_MeasurementReport_PDU);

  return offset;
}


static const per_sequence_t SRNC_RelocationInfo_r11_IEs_sequence[] = {
  { &hf_rrc_rb_IdentityForHOMessage, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RB_Identity },
  { &hf_rrc_stateOfRRC      , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_StateOfRRC },
  { &hf_rrc_stateOfRRC_Procedure, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_StateOfRRC_Procedure },
  { &hf_rrc_cipheringStatusList_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CipheringStatusList_r4 },
  { &hf_rrc_latestConfiguredCN_Domain, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_CN_DomainIdentity },
  { &hf_rrc_calculationTimeForCiphering, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CalculationTimeForCiphering },
  { &hf_rrc_count_C_List    , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_COUNT_C_List },
  { &hf_rrc_cipheringInfoPerRB_List_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CipheringInfoPerRB_List_r4 },
  { &hf_rrc_integrityProtectionStatus, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_IntegrityProtectionStatus },
  { &hf_rrc_srb_SpecificIntegrityProtInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_SRB_SpecificIntegrityProtInfoList },
  { &hf_rrc_implementationSpecificParams, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_ImplementationSpecificParams },
  { &hf_rrc_u_RNTI          , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_U_RNTI },
  { &hf_rrc_c_RNTI          , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_C_RNTI },
  { &hf_rrc_ue_RadioAccessCapability_05, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_UE_RadioAccessCapability_r6 },
  { &hf_rrc_ue_RadioAccessCapability_ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_RadioAccessCapabBandFDDList },
  { &hf_rrc_ue_Positioning_LastKnownPos, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_Positioning_LastKnownPos },
  { &hf_rrc_uESpecificBehaviourInformation1idle, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UESpecificBehaviourInformation1idle },
  { &hf_rrc_uESpecificBehaviourInformation1interRAT, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UESpecificBehaviourInformation1interRAT },
  { &hf_rrc_ueCapabilityContainer_08, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_ueCapabilityContainer_08 },
  { &hf_rrc_ueCapabilityContainer_RSC_06, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_ueCapabilityContainer_RSC_06 },
  { &hf_rrc_ueCapabilityContainer_UCI_06, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_ueCapabilityContainer_UCI_06 },
  { &hf_rrc_ue_RATSpecificCapability_02, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_InterRAT_UE_RadioAccessCapabilityList_r5 },
  { &hf_rrc_ura_Identity    , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_URA_Identity },
  { &hf_rrc_srns_t_305      , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_305 },
  { &hf_rrc_cn_CommonGSM_MAP_NAS_SysInfo_04, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_NAS_SystemInformationGSM_MAP },
  { &hf_rrc_cn_DomainInformationList_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_CN_DomainInformationListFull },
  { &hf_rrc_ongoingMeasRepList_09, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_OngoingMeasRepList_r11 },
  { &hf_rrc_interRATCellInfoIndication, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_InterRATCellInfoIndication },
  { &hf_rrc_predefinedConfigStatusList_01, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_PredefinedConfigStatusList },
  { &hf_rrc_srb_InformationList_04, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SRB_InformationSetupList_r8 },
  { &hf_rrc_rab_InformationList_08, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_RAB_InformationSetupList_r11 },
  { &hf_rrc_pdcp_ROHC_TargetMode, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_PDCP_ROHC_TargetMode },
  { &hf_rrc_ul_CommonTransChInfo_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_CommonTransChInfo_r4 },
  { &hf_rrc_ul_TransChInfoList_03, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_AddReconfTransChInfoList_r8 },
  { &hf_rrc_dl_CommonTransChInfo_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_CommonTransChInfo_r4 },
  { &hf_rrc_dl_TransChInfoList_05, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_AddReconfTransChInfoList_r11 },
  { &hf_rrc_tpc_CombinationInfoList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_TPC_CombinationInfoList },
  { &hf_rrc_e_RGCH_CombinationInfoList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_E_RGCH_CombinationInfoList },
  { &hf_rrc_storedCompressedModeInfo_01, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_StoredCompressedModeInfo_r8 },
  { &hf_rrc_secondary_tpc_CombinationInfoList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_TPC_CombinationInfoList_r9 },
  { &hf_rrc_secondary_e_RGCH_CombinationInfoList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_E_RGCH_CombinationInfoList_r9 },
  { &hf_rrc_measurementReport_06, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_measurementReport_05 },
  { &hf_rrc_failureCause    , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_FailureCauseWithProtErr },
  { &hf_rrc_ue_HistoryInformation, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UE_HistoryInformation },
  { &hf_rrc_mbms_JoinedInformation, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MBMS_JoinedInformation_r6 },
  { &hf_rrc_mbmsSelectedServiceInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_MBMS_SelectedServiceInfo },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_SRNC_RelocationInfo_r11_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_SRNC_RelocationInfo_r11_IEs, SRNC_RelocationInfo_r11_IEs_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions_248_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_nonCriticalExtensions_248(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_nonCriticalExtensions_248, T_nonCriticalExtensions_248_sequence);

  return offset;
}


static const per_sequence_t T_v9c0NonCriticalExtensions_14_sequence[] = {
  { &hf_rrc_sRNC_RelocationInfo_v9c0ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SRNC_RelocationInfo_v9c0ext_IEs },
  { &hf_rrc_nonCriticalExtensions_248, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_nonCriticalExtensions_248 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v9c0NonCriticalExtensions_14(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v9c0NonCriticalExtensions_14, T_v9c0NonCriticalExtensions_14_sequence);

  return offset;
}


static const per_sequence_t T_r11_14_sequence[] = {
  { &hf_rrc_sRNC_RelocationInfo_r11, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SRNC_RelocationInfo_r11_IEs },
  { &hf_rrc_sRNC_RelocationInfo_r11_add_ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_BIT_STRING },
  { &hf_rrc_v9c0NonCriticalExtensions_14, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v9c0NonCriticalExtensions_14 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_r11_14(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_r11_14, T_r11_14_sequence);

  return offset;
}


static const per_sequence_t T_criticalExtensions_148_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_criticalExtensions_148(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_criticalExtensions_148, T_criticalExtensions_148_sequence);

  return offset;
}


static const value_string rrc_T_criticalExtensions_147_vals[] = {
  {   0, "r11" },
  {   1, "criticalExtensions" },
  { 0, NULL }
};

static const per_choice_t T_criticalExtensions_147_choice[] = {
  {   0, &hf_rrc_r11_14          , ASN1_NO_EXTENSIONS     , dissect_rrc_T_r11_14 },
  {   1, &hf_rrc_criticalExtensions_149, ASN1_NO_EXTENSIONS     , dissect_rrc_T_criticalExtensions_148 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_criticalExtensions_147(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_criticalExtensions_147, T_criticalExtensions_147_choice,
                                 NULL);

  return offset;
}


static const value_string rrc_T_criticalExtensions_146_vals[] = {
  {   0, "r10" },
  {   1, "criticalExtensions" },
  { 0, NULL }
};

static const per_choice_t T_criticalExtensions_146_choice[] = {
  {   0, &hf_rrc_r10_13          , ASN1_NO_EXTENSIONS     , dissect_rrc_T_r10_13 },
  {   1, &hf_rrc_criticalExtensions_148, ASN1_NO_EXTENSIONS     , dissect_rrc_T_criticalExtensions_147 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_criticalExtensions_146(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_criticalExtensions_146, T_criticalExtensions_146_choice,
                                 NULL);

  return offset;
}


static const value_string rrc_T_criticalExtensions_145_vals[] = {
  {   0, "r9" },
  {   1, "criticalExtensions" },
  { 0, NULL }
};

static const per_choice_t T_criticalExtensions_145_choice[] = {
  {   0, &hf_rrc_r9_11           , ASN1_NO_EXTENSIONS     , dissect_rrc_T_r9_11 },
  {   1, &hf_rrc_criticalExtensions_147, ASN1_NO_EXTENSIONS     , dissect_rrc_T_criticalExtensions_146 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_criticalExtensions_145(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_criticalExtensions_145, T_criticalExtensions_145_choice,
                                 NULL);

  return offset;
}


static const value_string rrc_T_criticalExtensions_144_vals[] = {
  {   0, "r8" },
  {   1, "criticalExtensions" },
  { 0, NULL }
};

static const per_choice_t T_criticalExtensions_144_choice[] = {
  {   0, &hf_rrc_r8_12           , ASN1_NO_EXTENSIONS     , dissect_rrc_T_r8_12 },
  {   1, &hf_rrc_criticalExtensions_146, ASN1_NO_EXTENSIONS     , dissect_rrc_T_criticalExtensions_145 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_criticalExtensions_144(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_criticalExtensions_144, T_criticalExtensions_144_choice,
                                 NULL);

  return offset;
}


static const value_string rrc_T_criticalExtensions_143_vals[] = {
  {   0, "r7" },
  {   1, "criticalExtensions" },
  { 0, NULL }
};

static const per_choice_t T_criticalExtensions_143_choice[] = {
  {   0, &hf_rrc_r7_16           , ASN1_NO_EXTENSIONS     , dissect_rrc_T_r7_16 },
  {   1, &hf_rrc_criticalExtensions_145, ASN1_NO_EXTENSIONS     , dissect_rrc_T_criticalExtensions_144 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_criticalExtensions_143(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_criticalExtensions_143, T_criticalExtensions_143_choice,
                                 NULL);

  return offset;
}


static const value_string rrc_T_criticalExtensions_142_vals[] = {
  {   0, "r6" },
  {   1, "criticalExtensions" },
  { 0, NULL }
};

static const per_choice_t T_criticalExtensions_142_choice[] = {
  {   0, &hf_rrc_r6_12           , ASN1_NO_EXTENSIONS     , dissect_rrc_T_r6_12 },
  {   1, &hf_rrc_criticalExtensions_144, ASN1_NO_EXTENSIONS     , dissect_rrc_T_criticalExtensions_143 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_criticalExtensions_142(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_criticalExtensions_142, T_criticalExtensions_142_choice,
                                 NULL);

  return offset;
}


static const value_string rrc_T_criticalExtensions_141_vals[] = {
  {   0, "r5" },
  {   1, "criticalExtensions" },
  { 0, NULL }
};

static const per_choice_t T_criticalExtensions_141_choice[] = {
  {   0, &hf_rrc_r5_15           , ASN1_NO_EXTENSIONS     , dissect_rrc_T_r5_15 },
  {   1, &hf_rrc_criticalExtensions_143, ASN1_NO_EXTENSIONS     , dissect_rrc_T_criticalExtensions_142 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_criticalExtensions_141(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_criticalExtensions_141, T_criticalExtensions_141_choice,
                                 NULL);

  return offset;
}


static const value_string rrc_T_later_than_r3_28_vals[] = {
  {   0, "r4" },
  {   1, "criticalExtensions" },
  { 0, NULL }
};

static const per_choice_t T_later_than_r3_28_choice[] = {
  {   0, &hf_rrc_r4_14           , ASN1_NO_EXTENSIONS     , dissect_rrc_T_r4_14 },
  {   1, &hf_rrc_criticalExtensions_142, ASN1_NO_EXTENSIONS     , dissect_rrc_T_criticalExtensions_141 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_T_later_than_r3_28(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_T_later_than_r3_28, T_later_than_r3_28_choice,
                                 NULL);

  return offset;
}


static const value_string rrc_SRNC_RelocationInfo_r3_vals[] = {
  {   0, "r3" },
  {   1, "later-than-r3" },
  { 0, NULL }
};

static const per_choice_t SRNC_RelocationInfo_r3_choice[] = {
  {   0, &hf_rrc_r3_32           , ASN1_NO_EXTENSIONS     , dissect_rrc_T_r3_32 },
  {   1, &hf_rrc_later_than_r3_28, ASN1_NO_EXTENSIONS     , dissect_rrc_T_later_than_r3_28 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_SRNC_RelocationInfo_r3(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_SRNC_RelocationInfo_r3, SRNC_RelocationInfo_r3_choice,
                                 NULL);

  return offset;
}


static const value_string rrc_T_dl_mode_vals[] = {
  {   0, "u" },
  {   1, "o" },
  {   2, "r" },
  { 0, NULL }
};


static int
dissect_rrc_T_dl_mode(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     3, NULL, FALSE, 0, NULL);

  return offset;
}



static int
dissect_rrc_OCTET_STRING_SIZE_1_3000(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_octet_string(tvb, offset, actx, tree, hf_index,
                                       1, 3000, FALSE, NULL);

  return offset;
}


static const per_sequence_t DL_RFC3095_Context_sequence[] = {
  { &hf_rrc_rfc3095_Context_Identity, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_0_16383 },
  { &hf_rrc_dl_mode         , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_dl_mode },
  { &hf_rrc_dl_ref_ir       , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_OCTET_STRING_SIZE_1_3000 },
  { &hf_rrc_dl_ref_time     , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_INTEGER_0_4294967295 },
  { &hf_rrc_dl_curr_time    , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_INTEGER_0_4294967295 },
  { &hf_rrc_dl_syn_offset_id, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_INTEGER_0_65535 },
  { &hf_rrc_dl_syn_slope_ts , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_INTEGER_0_4294967295 },
  { &hf_rrc_dl_dyn_changed  , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_BOOLEAN },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_DL_RFC3095_Context(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_DL_RFC3095_Context, DL_RFC3095_Context_sequence);

  return offset;
}


static const value_string rrc_T_ul_mode_vals[] = {
  {   0, "u" },
  {   1, "o" },
  {   2, "r" },
  { 0, NULL }
};


static int
dissect_rrc_T_ul_mode(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     3, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t UL_RFC3095_Context_sequence[] = {
  { &hf_rrc_rfc3095_Context_Identity, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_INTEGER_0_16383 },
  { &hf_rrc_ul_mode         , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_T_ul_mode },
  { &hf_rrc_ul_ref_ir       , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_OCTET_STRING_SIZE_1_3000 },
  { &hf_rrc_ul_ref_time     , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_INTEGER_0_4294967295 },
  { &hf_rrc_ul_curr_time    , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_INTEGER_0_4294967295 },
  { &hf_rrc_ul_syn_offset_id, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_INTEGER_0_65535 },
  { &hf_rrc_ul_syn_slope_ts , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_INTEGER_0_4294967295 },
  { &hf_rrc_ul_ref_sn_1     , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_INTEGER_0_65535 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_UL_RFC3095_Context(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_UL_RFC3095_Context, UL_RFC3095_Context_sequence);

  return offset;
}


static const per_sequence_t RFC3095_Context_List_item_sequence[] = {
  { &hf_rrc_dl_RFC3095_Context, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_DL_RFC3095_Context },
  { &hf_rrc_ul_RFC3095_Context, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_UL_RFC3095_Context },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_RFC3095_Context_List_item(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_RFC3095_Context_List_item, RFC3095_Context_List_item_sequence);

  return offset;
}


static const per_sequence_t RFC3095_Context_List_sequence_of[1] = {
  { &hf_rrc_RFC3095_Context_List_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RFC3095_Context_List_item },
};

static int
dissect_rrc_RFC3095_Context_List(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_RFC3095_Context_List, RFC3095_Context_List_sequence_of,
                                                  1, maxRFC3095_CID, FALSE);

  return offset;
}


static const per_sequence_t RFC3095_ContextInfo_sequence[] = {
  { &hf_rrc_rb_Identity     , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RB_Identity },
  { &hf_rrc_rfc3095_Context_List, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RFC3095_Context_List },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_RFC3095_ContextInfo(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_RFC3095_ContextInfo, RFC3095_ContextInfo_sequence);

  return offset;
}


static const per_sequence_t RFC3095_ContextInfoList_r5_sequence_of[1] = {
  { &hf_rrc_RFC3095_ContextInfoList_r5_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RFC3095_ContextInfo },
};

static int
dissect_rrc_RFC3095_ContextInfoList_r5(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rrc_RFC3095_ContextInfoList_r5, RFC3095_ContextInfoList_r5_sequence_of,
                                                  1, maxRBallRABs, FALSE);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions_239_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_nonCriticalExtensions_239(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_nonCriticalExtensions_239, T_nonCriticalExtensions_239_sequence);

  return offset;
}


static const per_sequence_t T_r5_14_sequence[] = {
  { &hf_rrc_rFC3095_ContextInfoList_r5, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_RFC3095_ContextInfoList_r5 },
  { &hf_rrc_nonCriticalExtensions_239, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_nonCriticalExtensions_239 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_r5_14(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_r5_14, T_r5_14_sequence);

  return offset;
}


static const per_sequence_t T_criticalExtensions_140_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_criticalExtensions_140(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_criticalExtensions_140, T_criticalExtensions_140_sequence);

  return offset;
}


static const value_string rrc_RFC3095_ContextInfo_r5_vals[] = {
  {   0, "r5" },
  {   1, "criticalExtensions" },
  { 0, NULL }
};

static const per_choice_t RFC3095_ContextInfo_r5_choice[] = {
  {   0, &hf_rrc_r5_14           , ASN1_NO_EXTENSIONS     , dissect_rrc_T_r5_14 },
  {   1, &hf_rrc_criticalExtensions_141, ASN1_NO_EXTENSIONS     , dissect_rrc_T_criticalExtensions_140 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_RFC3095_ContextInfo_r5(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_RFC3095_ContextInfo_r5, RFC3095_ContextInfo_r5_choice,
                                 NULL);

  return offset;
}


static const value_string rrc_ToTargetRNC_Container_vals[] = {
  {   0, "interRATHandoverInfo" },
  {   1, "srncRelocation" },
  {   2, "rfc3095-ContextInfo" },
  {   3, "extension" },
  { 0, NULL }
};

static const per_choice_t ToTargetRNC_Container_choice[] = {
  {   0, &hf_rrc_interRATHandoverInfo, ASN1_NO_EXTENSIONS     , dissect_rrc_InterRATHandoverInfoWithInterRATCapabilities_r3 },
  {   1, &hf_rrc_srncRelocation  , ASN1_NO_EXTENSIONS     , dissect_rrc_SRNC_RelocationInfo_r3 },
  {   2, &hf_rrc_rfc3095_ContextInfo, ASN1_NO_EXTENSIONS     , dissect_rrc_RFC3095_ContextInfo_r5 },
  {   3, &hf_rrc_extension       , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_ToTargetRNC_Container(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_ToTargetRNC_Container, ToTargetRNC_Container_choice,
                                 NULL);

  return offset;
}



static int
dissect_rrc_T_dL_DCCHmessage(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
#line 507 "../../asn1/rrc/rrc.cnf"
  tvbuff_t *dl_dcch_message_tvb=NULL;

  offset = dissect_per_octet_string(tvb, offset, actx, tree, hf_index,
                                       NO_BOUND, NO_BOUND, FALSE, &dl_dcch_message_tvb);


  if (dl_dcch_message_tvb)
    call_dissector(rrc_dl_dcch_handle,dl_dcch_message_tvb,actx->pinfo, proto_tree_get_root(tree));



  return offset;
}


static const value_string rrc_TargetRNC_ToSourceRNC_Container_vals[] = {
  {   0, "radioBearerSetup" },
  {   1, "radioBearerReconfiguration" },
  {   2, "radioBearerRelease" },
  {   3, "transportChannelReconfiguration" },
  {   4, "physicalChannelReconfiguration" },
  {   5, "rrc-FailureInfo" },
  {   6, "dL-DCCHmessage" },
  {   7, "extension" },
  { 0, NULL }
};

static const per_choice_t TargetRNC_ToSourceRNC_Container_choice[] = {
  {   0, &hf_rrc_radioBearerSetup, ASN1_NO_EXTENSIONS     , dissect_rrc_RadioBearerSetup },
  {   1, &hf_rrc_radioBearerReconfiguration, ASN1_NO_EXTENSIONS     , dissect_rrc_RadioBearerReconfiguration },
  {   2, &hf_rrc_radioBearerRelease, ASN1_NO_EXTENSIONS     , dissect_rrc_RadioBearerRelease },
  {   3, &hf_rrc_transportChannelReconfiguration, ASN1_NO_EXTENSIONS     , dissect_rrc_TransportChannelReconfiguration },
  {   4, &hf_rrc_physicalChannelReconfiguration, ASN1_NO_EXTENSIONS     , dissect_rrc_PhysicalChannelReconfiguration },
  {   5, &hf_rrc_rrc_FailureInfo , ASN1_NO_EXTENSIONS     , dissect_rrc_RRC_FailureInfo },
  {   6, &hf_rrc_dL_DCCHmessage  , ASN1_NO_EXTENSIONS     , dissect_rrc_T_dL_DCCHmessage },
  {   7, &hf_rrc_extension       , ASN1_NO_EXTENSIONS     , dissect_rrc_NULL },
  { 0, NULL, 0, NULL }
};

static int
dissect_rrc_TargetRNC_ToSourceRNC_Container(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rrc_TargetRNC_ToSourceRNC_Container, TargetRNC_ToSourceRNC_Container_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t T_nonCriticalExtension_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_nonCriticalExtension(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_nonCriticalExtension, T_nonCriticalExtension_sequence);

  return offset;
}


static const per_sequence_t SRNC_RelocationInfo_v3h0ext_IEs_sequence[] = {
  { &hf_rrc_tpc_CombinationInfoList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_TPC_CombinationInfoList },
  { &hf_rrc_nonCriticalExtension, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_nonCriticalExtension },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_SRNC_RelocationInfo_v3h0ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_SRNC_RelocationInfo_v3h0ext_IEs, SRNC_RelocationInfo_v3h0ext_IEs_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions_249_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_nonCriticalExtensions_249(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_nonCriticalExtensions_249, T_nonCriticalExtensions_249_sequence);

  return offset;
}


static const per_sequence_t T_v7f0NonCriticalExtensions_21_sequence[] = {
  { &hf_rrc_sRNC_RelocationInfo_v7f0ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SRNC_RelocationInfo_v7f0ext_IEs },
  { &hf_rrc_nonCriticalExtensions_249, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_nonCriticalExtensions_249 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v7f0NonCriticalExtensions_21(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v7f0NonCriticalExtensions_21, T_v7f0NonCriticalExtensions_21_sequence);

  return offset;
}


static const per_sequence_t SRNC_RelocationInfo_r6_add_ext_IEs_sequence[] = {
  { &hf_rrc_sRNC_RelocationInfo_v7e0ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SRNC_RelocationInfo_v7e0ext_IEs },
  { &hf_rrc_v7f0NonCriticalExtensions_21, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v7f0NonCriticalExtensions_21 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_SRNC_RelocationInfo_r6_add_ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_SRNC_RelocationInfo_r6_add_ext_IEs, SRNC_RelocationInfo_r6_add_ext_IEs_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions_250_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_nonCriticalExtensions_250(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_nonCriticalExtensions_250, T_nonCriticalExtensions_250_sequence);

  return offset;
}


static const per_sequence_t T_v7f0NonCriticalExtensions_22_sequence[] = {
  { &hf_rrc_sRNC_RelocationInfo_v7f0ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SRNC_RelocationInfo_v7f0ext_IEs },
  { &hf_rrc_nonCriticalExtensions_250, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_nonCriticalExtensions_250 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_T_v7f0NonCriticalExtensions_22(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_T_v7f0NonCriticalExtensions_22, T_v7f0NonCriticalExtensions_22_sequence);

  return offset;
}


static const per_sequence_t SRNC_RelocationInfo_r7_add_ext_IEs_sequence[] = {
  { &hf_rrc_sRNC_RelocationInfo_v7e0ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rrc_SRNC_RelocationInfo_v7e0ext_IEs },
  { &hf_rrc_v7f0NonCriticalExtensions_22, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rrc_T_v7f0NonCriticalExtensions_22 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rrc_SRNC_RelocationInfo_r7_add_ext_IEs(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rrc_SRNC_RelocationInfo_r7_add_ext_IEs, SRNC_RelocationInfo_r7_add_ext_IEs_sequence);

  return offset;
}

/*--- PDUs ---*/

static int dissect_DL_DCCH_Message_PDU(tvbuff_t *tvb _U_, packet_info *pinfo _U_, proto_tree *tree _U_, void *data _U_) {
  int offset = 0;
  asn1_ctx_t asn1_ctx;
  asn1_ctx_init(&asn1_ctx, ASN1_ENC_PER, FALSE, pinfo);
  offset = dissect_rrc_DL_DCCH_Message(tvb, offset, &asn1_ctx, tree, hf_rrc_DL_DCCH_Message_PDU);
  offset += 7; offset >>= 3;
  return offset;
}
static int dissect_UL_DCCH_Message_PDU(tvbuff_t *tvb _U_, packet_info *pinfo _U_, proto_tree *tree _U_, void *data _U_) {
  int offset = 0;
  asn1_ctx_t asn1_ctx;
  asn1_ctx_init(&asn1_ctx, ASN1_ENC_PER, FALSE, pinfo);
  offset = dissect_rrc_UL_DCCH_Message(tvb, offset, &asn1_ctx, tree, hf_rrc_UL_DCCH_Message_PDU);
  offset += 7; offset >>= 3;
  return offset;
}
static int dissect_DL_CCCH_Message_PDU(tvbuff_t *tvb _U_, packet_info *pinfo _U_, proto_tree *tree _U_, void *data _U_) {
  int offset = 0;
  asn1_ctx_t asn1_ctx;
  asn1_ctx_init(&asn1_ctx, ASN1_ENC_PER, FALSE, pinfo);
  offset = dissect_rrc_DL_CCCH_Message(tvb, offset, &asn1_ctx, tree, hf_rrc_DL_CCCH_Message_PDU);
  offset += 7; offset >>= 3;
  return offset;
}
static int dissect_UL_CCCH_Message_PDU(tvbuff_t *tvb _U_, packet_info *pinfo _U_, proto_tree *tree _U_, void *data _U_) {
  int offset = 0;
  asn1_ctx_t asn1_ctx;
  asn1_ctx_init(&asn1_ctx, ASN1_ENC_PER, FALSE, pinfo);
  offset = dissect_rrc_UL_CCCH_Message(tvb, offset, &asn1_ctx, tree, hf_rrc_UL_CCCH_Message_PDU);
  offset += 7; offset >>= 3;
  return offset;
}
static int dissect_PCCH_Message_PDU(tvbuff_t *tvb _U_, packet_info *pinfo _U_, proto_tree *tree _U_, void *data _U_) {
  int offset = 0;
  asn1_ctx_t asn1_ctx;
  asn1_ctx_init(&asn1_ctx, ASN1_ENC_PER, FALSE, pinfo);
  offset = dissect_rrc_PCCH_Message(tvb, offset, &asn1_ctx, tree, hf_rrc_PCCH_Message_PDU);
  offset += 7; offset >>= 3;
  return offset;
}
static int dissect_DL_SHCCH_Message_PDU(tvbuff_t *tvb _U_, packet_info *pinfo _U_, proto_tree *tree _U_, void *data _U_) {
  int offset = 0;
  asn1_ctx_t asn1_ctx;
  asn1_ctx_init(&asn1_ctx, ASN1_ENC_PER, FALSE, pinfo);
  offset = dissect_rrc_DL_SHCCH_Message(tvb, offset, &asn1_ctx, tree, hf_rrc_DL_SHCCH_Message_PDU);
  offset += 7; offset >>= 3;
  return offset;
}
static int dissect_UL_SHCCH_Message_PDU(tvbuff_t *tvb _U_, packet_info *pinfo _U_, proto_tree *tree _U_, void *data _U_) {
  int offset = 0;
  asn1_ctx_t asn1_ctx;
  asn1_ctx_init(&asn1_ctx, ASN1_ENC_PER, FALSE, pinfo);
  offset = dissect_rrc_UL_SHCCH_Message(tvb, offset, &asn1_ctx, tree, hf_rrc_UL_SHCCH_Message_PDU);
  offset += 7; offset >>= 3;
  return offset;
}
static int dissect_BCCH_FACH_Message_PDU(tvbuff_t *tvb _U_, packet_info *pinfo _U_, proto_tree *tree _U_, void *data _U_) {
  int offset = 0;
  asn1_ctx_t asn1_ctx;
  asn1_ctx_init(&asn1_ctx, ASN1_ENC_PER, FALSE, pinfo);
  offset = dissect_rrc_BCCH_FACH_Message(tvb, offset, &asn1_ctx, tree, hf_rrc_BCCH_FACH_Message_PDU);
  offset += 7; offset >>= 3;
  return offset;
}
static int dissect_BCCH_BCH_Message_PDU(tvbuff_t *tvb _U_, packet_info *pinfo _U_, proto_tree *tree _U_, void *data _U_) {
  int offset = 0;
  asn1_ctx_t asn1_ctx;
  asn1_ctx_init(&asn1_ctx, ASN1_ENC_PER, FALSE, pinfo);
  offset = dissect_rrc_BCCH_BCH_Message(tvb, offset, &asn1_ctx, tree, hf_rrc_BCCH_BCH_Message_PDU);
  offset += 7; offset >>= 3;
  return offset;
}
static int dissect_MCCH_Message_PDU(tvbuff_t *tvb _U_, packet_info *pinfo _U_, proto_tree *tree _U_, void *data _U_) {
  int offset = 0;
  asn1_ctx_t asn1_ctx;
  asn1_ctx_init(&asn1_ctx, ASN1_ENC_PER, FALSE, pinfo);
  offset = dissect_rrc_MCCH_Message(tvb, offset, &asn1_ctx, tree, hf_rrc_MCCH_Message_PDU);
  offset += 7; offset >>= 3;
  return offset;
}
static int dissect_MSCH_Message_PDU(tvbuff_t *tvb _U_, packet_info *pinfo _U_, proto_tree *tree _U_, void *data _U_) {
  int offset = 0;
  asn1_ctx_t asn1_ctx;
  asn1_ctx_init(&asn1_ctx, ASN1_ENC_PER, FALSE, pinfo);
  offset = dissect_rrc_MSCH_Message(tvb, offset, &asn1_ctx, tree, hf_rrc_MSCH_Message_PDU);
  offset += 7; offset >>= 3;
  return offset;
}
int dissect_rrc_HandoverToUTRANCommand_PDU(tvbuff_t *tvb _U_, packet_info *pinfo _U_, proto_tree *tree _U_, void *data _U_) {
  int offset = 0;
  asn1_ctx_t asn1_ctx;
  asn1_ctx_init(&asn1_ctx, ASN1_ENC_PER, FALSE, pinfo);
  offset = dissect_rrc_HandoverToUTRANCommand(tvb, offset, &asn1_ctx, tree, hf_rrc_rrc_HandoverToUTRANCommand_PDU);
  offset += 7; offset >>= 3;
  return offset;
}
int dissect_rrc_InterRATHandoverInfo_PDU(tvbuff_t *tvb _U_, packet_info *pinfo _U_, proto_tree *tree _U_, void *data _U_) {
  int offset = 0;
  asn1_ctx_t asn1_ctx;
  asn1_ctx_init(&asn1_ctx, ASN1_ENC_PER, FALSE, pinfo);
  offset = dissect_rrc_InterRATHandoverInfo(tvb, offset, &asn1_ctx, tree, hf_rrc_rrc_InterRATHandoverInfo_PDU);
  offset += 7; offset >>= 3;
  return offset;
}
static int dissect_SystemInformation_BCH_PDU(tvbuff_t *tvb _U_, packet_info *pinfo _U_, proto_tree *tree _U_, void *data _U_) {
  int offset = 0;
  asn1_ctx_t asn1_ctx;
  asn1_ctx_init(&asn1_ctx, ASN1_ENC_PER, FALSE, pinfo);
  offset = dissect_rrc_SystemInformation_BCH(tvb, offset, &asn1_ctx, tree, hf_rrc_SystemInformation_BCH_PDU);
  offset += 7; offset >>= 3;
  return offset;
}
static int dissect_System_Information_Container_PDU(tvbuff_t *tvb _U_, packet_info *pinfo _U_, proto_tree *tree _U_, void *data _U_) {
  int offset = 0;
  asn1_ctx_t asn1_ctx;
  asn1_ctx_init(&asn1_ctx, ASN1_ENC_PER, FALSE, pinfo);
  offset = dissect_rrc_System_Information_Container(tvb, offset, &asn1_ctx, tree, hf_rrc_System_Information_Container_PDU);
  offset += 7; offset >>= 3;
  return offset;
}
static int dissect_UE_RadioAccessCapabilityInfo_PDU(tvbuff_t *tvb _U_, packet_info *pinfo _U_, proto_tree *tree _U_, void *data _U_) {
  int offset = 0;
  asn1_ctx_t asn1_ctx;
  asn1_ctx_init(&asn1_ctx, ASN1_ENC_PER, FALSE, pinfo);
  offset = dissect_rrc_UE_RadioAccessCapabilityInfo(tvb, offset, &asn1_ctx, tree, hf_rrc_UE_RadioAccessCapabilityInfo_PDU);
  offset += 7; offset >>= 3;
  return offset;
}
int dissect_rrc_MasterInformationBlock_PDU(tvbuff_t *tvb _U_, packet_info *pinfo _U_, proto_tree *tree _U_, void *data _U_) {
  int offset = 0;
  asn1_ctx_t asn1_ctx;
  asn1_ctx_init(&asn1_ctx, ASN1_ENC_PER, FALSE, pinfo);
  offset = dissect_rrc_MasterInformationBlock(tvb, offset, &asn1_ctx, tree, hf_rrc_rrc_MasterInformationBlock_PDU);
  offset += 7; offset >>= 3;
  return offset;
}
int dissect_rrc_SysInfoType1_PDU(tvbuff_t *tvb _U_, packet_info *pinfo _U_, proto_tree *tree _U_, void *data _U_) {
  int offset = 0;
  asn1_ctx_t asn1_ctx;
  asn1_ctx_init(&asn1_ctx, ASN1_ENC_PER, FALSE, pinfo);
  offset = dissect_rrc_SysInfoType1(tvb, offset, &asn1_ctx, tree, hf_rrc_rrc_SysInfoType1_PDU);
  offset += 7; offset >>= 3;
  return offset;
}
int dissect_rrc_SysInfoType2_PDU(tvbuff_t *tvb _U_, packet_info *pinfo _U_, proto_tree *tree _U_, void *data _U_) {
  int offset = 0;
  asn1_ctx_t asn1_ctx;
  asn1_ctx_init(&asn1_ctx, ASN1_ENC_PER, FALSE, pinfo);
  offset = dissect_rrc_SysInfoType2(tvb, offset, &asn1_ctx, tree, hf_rrc_rrc_SysInfoType2_PDU);
  offset += 7; offset >>= 3;
  return offset;
}
int dissect_rrc_SysInfoType3_PDU(tvbuff_t *tvb _U_, packet_info *pinfo _U_, proto_tree *tree _U_, void *data _U_) {
  int offset = 0;
  asn1_ctx_t asn1_ctx;
  asn1_ctx_init(&asn1_ctx, ASN1_ENC_PER, FALSE, pinfo);
  offset = dissect_rrc_SysInfoType3(tvb, offset, &asn1_ctx, tree, hf_rrc_rrc_SysInfoType3_PDU);
  offset += 7; offset >>= 3;
  return offset;
}
static int dissect_SysInfoType4_PDU(tvbuff_t *tvb _U_, packet_info *pinfo _U_, proto_tree *tree _U_, void *data _U_) {
  int offset = 0;
  asn1_ctx_t asn1_ctx;
  asn1_ctx_init(&asn1_ctx, ASN1_ENC_PER, FALSE, pinfo);
  offset = dissect_rrc_SysInfoType4(tvb, offset, &asn1_ctx, tree, hf_rrc_SysInfoType4_PDU);
  offset += 7; offset >>= 3;
  return offset;
}
static int dissect_SysInfoType5_PDU(tvbuff_t *tvb _U_, packet_info *pinfo _U_, proto_tree *tree _U_, void *data _U_) {
  int offset = 0;
  asn1_ctx_t asn1_ctx;
  asn1_ctx_init(&asn1_ctx, ASN1_ENC_PER, FALSE, pinfo);
  offset = dissect_rrc_SysInfoType5(tvb, offset, &asn1_ctx, tree, hf_rrc_SysInfoType5_PDU);
  offset += 7; offset >>= 3;
  return offset;
}
static int dissect_SysInfoType5bis_PDU(tvbuff_t *tvb _U_, packet_info *pinfo _U_, proto_tree *tree _U_, void *data _U_) {
  int offset = 0;
  asn1_ctx_t asn1_ctx;
  asn1_ctx_init(&asn1_ctx, ASN1_ENC_PER, FALSE, pinfo);
  offset = dissect_rrc_SysInfoType5bis(tvb, offset, &asn1_ctx, tree, hf_rrc_SysInfoType5bis_PDU);
  offset += 7; offset >>= 3;
  return offset;
}
static int dissect_SysInfoType6_PDU(tvbuff_t *tvb _U_, packet_info *pinfo _U_, proto_tree *tree _U_, void *data _U_) {
  int offset = 0;
  asn1_ctx_t asn1_ctx;
  asn1_ctx_init(&asn1_ctx, ASN1_ENC_PER, FALSE, pinfo);
  offset = dissect_rrc_SysInfoType6(tvb, offset, &asn1_ctx, tree, hf_rrc_SysInfoType6_PDU);
  offset += 7; offset >>= 3;
  return offset;
}
int dissect_rrc_SysInfoType7_PDU(tvbuff_t *tvb _U_, packet_info *pinfo _U_, proto_tree *tree _U_, void *data _U_) {
  int offset = 0;
  asn1_ctx_t asn1_ctx;
  asn1_ctx_init(&asn1_ctx, ASN1_ENC_PER, FALSE, pinfo);
  offset = dissect_rrc_SysInfoType7(tvb, offset, &asn1_ctx, tree, hf_rrc_rrc_SysInfoType7_PDU);
  offset += 7; offset >>= 3;
  return offset;
}
static int dissect_SysInfoType8_PDU(tvbuff_t *tvb _U_, packet_info *pinfo _U_, proto_tree *tree _U_, void *data _U_) {
  int offset = 0;
  asn1_ctx_t asn1_ctx;
  asn1_ctx_init(&asn1_ctx, ASN1_ENC_PER, FALSE, pinfo);
  offset = dissect_rrc_SysInfoType8(tvb, offset, &asn1_ctx, tree, hf_rrc_SysInfoType8_PDU);
  offset += 7; offset >>= 3;
  return offset;
}
static int dissect_SysInfoType9_PDU(tvbuff_t *tvb _U_, packet_info *pinfo _U_, proto_tree *tree _U_, void *data _U_) {
  int offset = 0;
  asn1_ctx_t asn1_ctx;
  asn1_ctx_init(&asn1_ctx, ASN1_ENC_PER, FALSE, pinfo);
  offset = dissect_rrc_SysInfoType9(tvb, offset, &asn1_ctx, tree, hf_rrc_SysInfoType9_PDU);
  offset += 7; offset >>= 3;
  return offset;
}
static int dissect_SysInfoType10_PDU(tvbuff_t *tvb _U_, packet_info *pinfo _U_, proto_tree *tree _U_, void *data _U_) {
  int offset = 0;
  asn1_ctx_t asn1_ctx;
  asn1_ctx_init(&asn1_ctx, ASN1_ENC_PER, FALSE, pinfo);
  offset = dissect_rrc_SysInfoType10(tvb, offset, &asn1_ctx, tree, hf_rrc_SysInfoType10_PDU);
  offset += 7; offset >>= 3;
  return offset;
}
static int dissect_SysInfoType11_PDU(tvbuff_t *tvb _U_, packet_info *pinfo _U_, proto_tree *tree _U_, void *data _U_) {
  int offset = 0;
  asn1_ctx_t asn1_ctx;
  asn1_ctx_init(&asn1_ctx, ASN1_ENC_PER, FALSE, pinfo);
  offset = dissect_rrc_SysInfoType11(tvb, offset, &asn1_ctx, tree, hf_rrc_SysInfoType11_PDU);
  offset += 7; offset >>= 3;
  return offset;
}
static int dissect_SysInfoType11bis_PDU(tvbuff_t *tvb _U_, packet_info *pinfo _U_, proto_tree *tree _U_, void *data _U_) {
  int offset = 0;
  asn1_ctx_t asn1_ctx;
  asn1_ctx_init(&asn1_ctx, ASN1_ENC_PER, FALSE, pinfo);
  offset = dissect_rrc_SysInfoType11bis(tvb, offset, &asn1_ctx, tree, hf_rrc_SysInfoType11bis_PDU);
  offset += 7; offset >>= 3;
  return offset;
}
int dissect_rrc_SysInfoType12_PDU(tvbuff_t *tvb _U_, packet_info *pinfo _U_, proto_tree *tree _U_, void *data _U_) {
  int offset = 0;
  asn1_ctx_t asn1_ctx;
  asn1_ctx_init(&asn1_ctx, ASN1_ENC_PER, FALSE, pinfo);
  offset = dissect_rrc_SysInfoType12(tvb, offset, &asn1_ctx, tree, hf_rrc_rrc_SysInfoType12_PDU);
  offset += 7; offset >>= 3;
  return offset;
}
static int dissect_SysInfoType13_PDU(tvbuff_t *tvb _U_, packet_info *pinfo _U_, proto_tree *tree _U_, void *data _U_) {
  int offset = 0;
  asn1_ctx_t asn1_ctx;
  asn1_ctx_init(&asn1_ctx, ASN1_ENC_PER, FALSE, pinfo);
  offset = dissect_rrc_SysInfoType13(tvb, offset, &asn1_ctx, tree, hf_rrc_SysInfoType13_PDU);
  offset += 7; offset >>= 3;
  return offset;
}
static int dissect_SysInfoType13_1_PDU(tvbuff_t *tvb _U_, packet_info *pinfo _U_, proto_tree *tree _U_, void *data _U_) {
  int offset = 0;
  asn1_ctx_t asn1_ctx;
  asn1_ctx_init(&asn1_ctx, ASN1_ENC_PER, FALSE, pinfo);
  offset = dissect_rrc_SysInfoType13_1(tvb, offset, &asn1_ctx, tree, hf_rrc_SysInfoType13_1_PDU);
  offset += 7; offset >>= 3;
  return offset;
}
static int dissect_SysInfoType13_2_PDU(tvbuff_t *tvb _U_, packet_info *pinfo _U_, proto_tree *tree _U_, void *data _U_) {
  int offset = 0;
  asn1_ctx_t asn1_ctx;
  asn1_ctx_init(&asn1_ctx, ASN1_ENC_PER, FALSE, pinfo);
  offset = dissect_rrc_SysInfoType13_2(tvb, offset, &asn1_ctx, tree, hf_rrc_SysInfoType13_2_PDU);
  offset += 7; offset >>= 3;
  return offset;
}
static int dissect_SysInfoType13_3_PDU(tvbuff_t *tvb _U_, packet_info *pinfo _U_, proto_tree *tree _U_, void *data _U_) {
  int offset = 0;
  asn1_ctx_t asn1_ctx;
  asn1_ctx_init(&asn1_ctx, ASN1_ENC_PER, FALSE, pinfo);
  offset = dissect_rrc_SysInfoType13_3(tvb, offset, &asn1_ctx, tree, hf_rrc_SysInfoType13_3_PDU);
  offset += 7; offset >>= 3;
  return offset;
}
static int dissect_SysInfoType13_4_PDU(tvbuff_t *tvb _U_, packet_info *pinfo _U_, proto_tree *tree _U_, void *data _U_) {
  int offset = 0;
  asn1_ctx_t asn1_ctx;
  asn1_ctx_init(&asn1_ctx, ASN1_ENC_PER, FALSE, pinfo);
  offset = dissect_rrc_SysInfoType13_4(tvb, offset, &asn1_ctx, tree, hf_rrc_SysInfoType13_4_PDU);
  offset += 7; offset >>= 3;
  return offset;
}
static int dissect_SysInfoType14_PDU(tvbuff_t *tvb _U_, packet_info *pinfo _U_, proto_tree *tree _U_, void *data _U_) {
  int offset = 0;
  asn1_ctx_t asn1_ctx;
  asn1_ctx_init(&asn1_ctx, ASN1_ENC_PER, FALSE, pinfo);
  offset = dissect_rrc_SysInfoType14(tvb, offset, &asn1_ctx, tree, hf_rrc_SysInfoType14_PDU);
  offset += 7; offset >>= 3;
  return offset;
}
static int dissect_SysInfoType15_PDU(tvbuff_t *tvb _U_, packet_info *pinfo _U_, proto_tree *tree _U_, void *data _U_) {
  int offset = 0;
  asn1_ctx_t asn1_ctx;
  asn1_ctx_init(&asn1_ctx, ASN1_ENC_PER, FALSE, pinfo);
  offset = dissect_rrc_SysInfoType15(tvb, offset, &asn1_ctx, tree, hf_rrc_SysInfoType15_PDU);
  offset += 7; offset >>= 3;
  return offset;
}
static int dissect_SysInfoType15bis_PDU(tvbuff_t *tvb _U_, packet_info *pinfo _U_, proto_tree *tree _U_, void *data _U_) {
  int offset = 0;
  asn1_ctx_t asn1_ctx;
  asn1_ctx_init(&asn1_ctx, ASN1_ENC_PER, FALSE, pinfo);
  offset = dissect_rrc_SysInfoType15bis(tvb, offset, &asn1_ctx, tree, hf_rrc_SysInfoType15bis_PDU);
  offset += 7; offset >>= 3;
  return offset;
}
static int dissect_SysInfoType15_1_PDU(tvbuff_t *tvb _U_, packet_info *pinfo _U_, proto_tree *tree _U_, void *data _U_) {
  int offset = 0;
  asn1_ctx_t asn1_ctx;
  asn1_ctx_init(&asn1_ctx, ASN1_ENC_PER, FALSE, pinfo);
  offset = dissect_rrc_SysInfoType15_1(tvb, offset, &asn1_ctx, tree, hf_rrc_SysInfoType15_1_PDU);
  offset += 7; offset >>= 3;
  return offset;
}
static int dissect_SysInfoType15_1bis_PDU(tvbuff_t *tvb _U_, packet_info *pinfo _U_, proto_tree *tree _U_, void *data _U_) {
  int offset = 0;
  asn1_ctx_t asn1_ctx;
  asn1_ctx_init(&asn1_ctx, ASN1_ENC_PER, FALSE, pinfo);
  offset = dissect_rrc_SysInfoType15_1bis(tvb, offset, &asn1_ctx, tree, hf_rrc_SysInfoType15_1bis_PDU);
  offset += 7; offset >>= 3;
  return offset;
}
static int dissect_SysInfoType15_2_PDU(tvbuff_t *tvb _U_, packet_info *pinfo _U_, proto_tree *tree _U_, void *data _U_) {
  int offset = 0;
  asn1_ctx_t asn1_ctx;
  asn1_ctx_init(&asn1_ctx, ASN1_ENC_PER, FALSE, pinfo);
  offset = dissect_rrc_SysInfoType15_2(tvb, offset, &asn1_ctx, tree, hf_rrc_SysInfoType15_2_PDU);
  offset += 7; offset >>= 3;
  return offset;
}
static int dissect_SysInfoType15_2bis_PDU(tvbuff_t *tvb _U_, packet_info *pinfo _U_, proto_tree *tree _U_, void *data _U_) {
  int offset = 0;
  asn1_ctx_t asn1_ctx;
  asn1_ctx_init(&asn1_ctx, ASN1_ENC_PER, FALSE, pinfo);
  offset = dissect_rrc_SysInfoType15_2bis(tvb, offset, &asn1_ctx, tree, hf_rrc_SysInfoType15_2bis_PDU);
  offset += 7; offset >>= 3;
  return offset;
}
static int dissect_SysInfoType15_2ter_PDU(tvbuff_t *tvb _U_, packet_info *pinfo _U_, proto_tree *tree _U_, void *data _U_) {
  int offset = 0;
  asn1_ctx_t asn1_ctx;
  asn1_ctx_init(&asn1_ctx, ASN1_ENC_PER, FALSE, pinfo);
  offset = dissect_rrc_SysInfoType15_2ter(tvb, offset, &asn1_ctx, tree, hf_rrc_SysInfoType15_2ter_PDU);
  offset += 7; offset >>= 3;
  return offset;
}
static int dissect_SysInfoType15_3_PDU(tvbuff_t *tvb _U_, packet_info *pinfo _U_, proto_tree *tree _U_, void *data _U_) {
  int offset = 0;
  asn1_ctx_t asn1_ctx;
  asn1_ctx_init(&asn1_ctx, ASN1_ENC_PER, FALSE, pinfo);
  offset = dissect_rrc_SysInfoType15_3(tvb, offset, &asn1_ctx, tree, hf_rrc_SysInfoType15_3_PDU);
  offset += 7; offset >>= 3;
  return offset;
}
static int dissect_SysInfoType15_3bis_PDU(tvbuff_t *tvb _U_, packet_info *pinfo _U_, proto_tree *tree _U_, void *data _U_) {
  int offset = 0;
  asn1_ctx_t asn1_ctx;
  asn1_ctx_init(&asn1_ctx, ASN1_ENC_PER, FALSE, pinfo);
  offset = dissect_rrc_SysInfoType15_3bis(tvb, offset, &asn1_ctx, tree, hf_rrc_SysInfoType15_3bis_PDU);
  offset += 7; offset >>= 3;
  return offset;
}
static int dissect_SysInfoType15_4_PDU(tvbuff_t *tvb _U_, packet_info *pinfo _U_, proto_tree *tree _U_, void *data _U_) {
  int offset = 0;
  asn1_ctx_t asn1_ctx;
  asn1_ctx_init(&asn1_ctx, ASN1_ENC_PER, FALSE, pinfo);
  offset = dissect_rrc_SysInfoType15_4(tvb, offset, &asn1_ctx, tree, hf_rrc_SysInfoType15_4_PDU);
  offset += 7; offset >>= 3;
  return offset;
}
static int dissect_SysInfoType15_5_PDU(tvbuff_t *tvb _U_, packet_info *pinfo _U_, proto_tree *tree _U_, void *data _U_) {
  int offset = 0;
  asn1_ctx_t asn1_ctx;
  asn1_ctx_init(&asn1_ctx, ASN1_ENC_PER, FALSE, pinfo);
  offset = dissect_rrc_SysInfoType15_5(tvb, offset, &asn1_ctx, tree, hf_rrc_SysInfoType15_5_PDU);
  offset += 7; offset >>= 3;
  return offset;
}
static int dissect_SysInfoType15_6_PDU(tvbuff_t *tvb _U_, packet_info *pinfo _U_, proto_tree *tree _U_, void *data _U_) {
  int offset = 0;
  asn1_ctx_t asn1_ctx;
  asn1_ctx_init(&asn1_ctx, ASN1_ENC_PER, FALSE, pinfo);
  offset = dissect_rrc_SysInfoType15_6(tvb, offset, &asn1_ctx, tree, hf_rrc_SysInfoType15_6_PDU);
  offset += 7; offset >>= 3;
  return offset;
}
static int dissect_SysInfoType15_7_PDU(tvbuff_t *tvb _U_, packet_info *pinfo _U_, proto_tree *tree _U_, void *data _U_) {
  int offset = 0;
  asn1_ctx_t asn1_ctx;
  asn1_ctx_init(&asn1_ctx, ASN1_ENC_PER, FALSE, pinfo);
  offset = dissect_rrc_SysInfoType15_7(tvb, offset, &asn1_ctx, tree, hf_rrc_SysInfoType15_7_PDU);
  offset += 7; offset >>= 3;
  return offset;
}
static int dissect_SysInfoType15_8_PDU(tvbuff_t *tvb _U_, packet_info *pinfo _U_, proto_tree *tree _U_, void *data _U_) {
  int offset = 0;
  asn1_ctx_t asn1_ctx;
  asn1_ctx_init(&asn1_ctx, ASN1_ENC_PER, FALSE, pinfo);
  offset = dissect_rrc_SysInfoType15_8(tvb, offset, &asn1_ctx, tree, hf_rrc_SysInfoType15_8_PDU);
  offset += 7; offset >>= 3;
  return offset;
}
static int dissect_SysInfoType16_PDU(tvbuff_t *tvb _U_, packet_info *pinfo _U_, proto_tree *tree _U_, void *data _U_) {
  int offset = 0;
  asn1_ctx_t asn1_ctx;
  asn1_ctx_init(&asn1_ctx, ASN1_ENC_PER, FALSE, pinfo);
  offset = dissect_rrc_SysInfoType16(tvb, offset, &asn1_ctx, tree, hf_rrc_SysInfoType16_PDU);
  offset += 7; offset >>= 3;
  return offset;
}
static int dissect_SysInfoType17_PDU(tvbuff_t *tvb _U_, packet_info *pinfo _U_, proto_tree *tree _U_, void *data _U_) {
  int offset = 0;
  asn1_ctx_t asn1_ctx;
  asn1_ctx_init(&asn1_ctx, ASN1_ENC_PER, FALSE, pinfo);
  offset = dissect_rrc_SysInfoType17(tvb, offset, &asn1_ctx, tree, hf_rrc_SysInfoType17_PDU);
  offset += 7; offset >>= 3;
  return offset;
}
static int dissect_SysInfoType18_PDU(tvbuff_t *tvb _U_, packet_info *pinfo _U_, proto_tree *tree _U_, void *data _U_) {
  int offset = 0;
  asn1_ctx_t asn1_ctx;
  asn1_ctx_init(&asn1_ctx, ASN1_ENC_PER, FALSE, pinfo);
  offset = dissect_rrc_SysInfoType18(tvb, offset, &asn1_ctx, tree, hf_rrc_SysInfoType18_PDU);
  offset += 7; offset >>= 3;
  return offset;
}
static int dissect_SysInfoType19_PDU(tvbuff_t *tvb _U_, packet_info *pinfo _U_, proto_tree *tree _U_, void *data _U_) {
  int offset = 0;
  asn1_ctx_t asn1_ctx;
  asn1_ctx_init(&asn1_ctx, ASN1_ENC_PER, FALSE, pinfo);
  offset = dissect_rrc_SysInfoType19(tvb, offset, &asn1_ctx, tree, hf_rrc_SysInfoType19_PDU);
  offset += 7; offset >>= 3;
  return offset;
}
static int dissect_SysInfoType20_PDU(tvbuff_t *tvb _U_, packet_info *pinfo _U_, proto_tree *tree _U_, void *data _U_) {
  int offset = 0;
  asn1_ctx_t asn1_ctx;
  asn1_ctx_init(&asn1_ctx, ASN1_ENC_PER, FALSE, pinfo);
  offset = dissect_rrc_SysInfoType20(tvb, offset, &asn1_ctx, tree, hf_rrc_SysInfoType20_PDU);
  offset += 7; offset >>= 3;
  return offset;
}
static int dissect_SysInfoType21_PDU(tvbuff_t *tvb _U_, packet_info *pinfo _U_, proto_tree *tree _U_, void *data _U_) {
  int offset = 0;
  asn1_ctx_t asn1_ctx;
  asn1_ctx_init(&asn1_ctx, ASN1_ENC_PER, FALSE, pinfo);
  offset = dissect_rrc_SysInfoType21(tvb, offset, &asn1_ctx, tree, hf_rrc_SysInfoType21_PDU);
  offset += 7; offset >>= 3;
  return offset;
}
static int dissect_SysInfoType22_PDU(tvbuff_t *tvb _U_, packet_info *pinfo _U_, proto_tree *tree _U_, void *data _U_) {
  int offset = 0;
  asn1_ctx_t asn1_ctx;
  asn1_ctx_init(&asn1_ctx, ASN1_ENC_PER, FALSE, pinfo);
  offset = dissect_rrc_SysInfoType22(tvb, offset, &asn1_ctx, tree, hf_rrc_SysInfoType22_PDU);
  offset += 7; offset >>= 3;
  return offset;
}
static int dissect_SysInfoTypeSB1_PDU(tvbuff_t *tvb _U_, packet_info *pinfo _U_, proto_tree *tree _U_, void *data _U_) {
  int offset = 0;
  asn1_ctx_t asn1_ctx;
  asn1_ctx_init(&asn1_ctx, ASN1_ENC_PER, FALSE, pinfo);
  offset = dissect_rrc_SysInfoTypeSB1(tvb, offset, &asn1_ctx, tree, hf_rrc_SysInfoTypeSB1_PDU);
  offset += 7; offset >>= 3;
  return offset;
}
static int dissect_SysInfoTypeSB2_PDU(tvbuff_t *tvb _U_, packet_info *pinfo _U_, proto_tree *tree _U_, void *data _U_) {
  int offset = 0;
  asn1_ctx_t asn1_ctx;
  asn1_ctx_init(&asn1_ctx, ASN1_ENC_PER, FALSE, pinfo);
  offset = dissect_rrc_SysInfoTypeSB2(tvb, offset, &asn1_ctx, tree, hf_rrc_SysInfoTypeSB2_PDU);
  offset += 7; offset >>= 3;
  return offset;
}
int dissect_rrc_ToTargetRNC_Container_PDU(tvbuff_t *tvb _U_, packet_info *pinfo _U_, proto_tree *tree _U_, void *data _U_) {
  int offset = 0;
  asn1_ctx_t asn1_ctx;
  asn1_ctx_init(&asn1_ctx, ASN1_ENC_PER, FALSE, pinfo);
  offset = dissect_rrc_ToTargetRNC_Container(tvb, offset, &asn1_ctx, tree, hf_rrc_rrc_ToTargetRNC_Container_PDU);
  offset += 7; offset >>= 3;
  return offset;
}
int dissect_rrc_TargetRNC_ToSourceRNC_Container_PDU(tvbuff_t *tvb _U_, packet_info *pinfo _U_, proto_tree *tree _U_, void *data _U_) {
  int offset = 0;
  asn1_ctx_t asn1_ctx;
  asn1_ctx_init(&asn1_ctx, ASN1_ENC_PER, FALSE, pinfo);
  offset = dissect_rrc_TargetRNC_ToSourceRNC_Container(tvb, offset, &asn1_ctx, tree, hf_rrc_rrc_TargetRNC_ToSourceRNC_Container_PDU);
  offset += 7; offset >>= 3;
  return offset;
}
static int dissect_CellUpdate_r3_add_ext_IEs_PDU(tvbuff_t *tvb _U_, packet_info *pinfo _U_, proto_tree *tree _U_, void *data _U_) {
  int offset = 0;
  asn1_ctx_t asn1_ctx;
  asn1_ctx_init(&asn1_ctx, ASN1_ENC_PER, FALSE, pinfo);
  offset = dissect_rrc_CellUpdate_r3_add_ext_IEs(tvb, offset, &asn1_ctx, tree, hf_rrc_CellUpdate_r3_add_ext_IEs_PDU);
  offset += 7; offset >>= 3;
  return offset;
}
static int dissect_CellUpdateConfirm_r7_add_ext_IEs_PDU(tvbuff_t *tvb _U_, packet_info *pinfo _U_, proto_tree *tree _U_, void *data _U_) {
  int offset = 0;
  asn1_ctx_t asn1_ctx;
  asn1_ctx_init(&asn1_ctx, ASN1_ENC_PER, FALSE, pinfo);
  offset = dissect_rrc_CellUpdateConfirm_r7_add_ext_IEs(tvb, offset, &asn1_ctx, tree, hf_rrc_CellUpdateConfirm_r7_add_ext_IEs_PDU);
  offset += 7; offset >>= 3;
  return offset;
}
static int dissect_InitialDirectTransfer_r3_add_ext_IEs_PDU(tvbuff_t *tvb _U_, packet_info *pinfo _U_, proto_tree *tree _U_, void *data _U_) {
  int offset = 0;
  asn1_ctx_t asn1_ctx;
  asn1_ctx_init(&asn1_ctx, ASN1_ENC_PER, FALSE, pinfo);
  offset = dissect_rrc_InitialDirectTransfer_r3_add_ext_IEs(tvb, offset, &asn1_ctx, tree, hf_rrc_InitialDirectTransfer_r3_add_ext_IEs_PDU);
  offset += 7; offset >>= 3;
  return offset;
}
static int dissect_InterRATHandoverInfo_r3_add_ext_IEs_PDU(tvbuff_t *tvb _U_, packet_info *pinfo _U_, proto_tree *tree _U_, void *data _U_) {
  int offset = 0;
  asn1_ctx_t asn1_ctx;
  asn1_ctx_init(&asn1_ctx, ASN1_ENC_PER, FALSE, pinfo);
  offset = dissect_rrc_InterRATHandoverInfo_r3_add_ext_IEs(tvb, offset, &asn1_ctx, tree, hf_rrc_InterRATHandoverInfo_r3_add_ext_IEs_PDU);
  offset += 7; offset >>= 3;
  return offset;
}
static int dissect_RadioBearerSetup_r7_add_ext_IEs_PDU(tvbuff_t *tvb _U_, packet_info *pinfo _U_, proto_tree *tree _U_, void *data _U_) {
  int offset = 0;
  asn1_ctx_t asn1_ctx;
  asn1_ctx_init(&asn1_ctx, ASN1_ENC_PER, FALSE, pinfo);
  offset = dissect_rrc_RadioBearerSetup_r7_add_ext_IEs(tvb, offset, &asn1_ctx, tree, hf_rrc_RadioBearerSetup_r7_add_ext_IEs_PDU);
  offset += 7; offset >>= 3;
  return offset;
}
static int dissect_RRCConnectionSetupComplete_r3_add_ext_IEs_PDU(tvbuff_t *tvb _U_, packet_info *pinfo _U_, proto_tree *tree _U_, void *data _U_) {
  int offset = 0;
  asn1_ctx_t asn1_ctx;
  asn1_ctx_init(&asn1_ctx, ASN1_ENC_PER, FALSE, pinfo);
  offset = dissect_rrc_RRCConnectionSetupComplete_r3_add_ext_IEs(tvb, offset, &asn1_ctx, tree, hf_rrc_RRCConnectionSetupComplete_r3_add_ext_IEs_PDU);
  offset += 7; offset >>= 3;
  return offset;
}
static int dissect_UE_CapabilityContainer_IEs_PDU(tvbuff_t *tvb _U_, packet_info *pinfo _U_, proto_tree *tree _U_, void *data _U_) {
  int offset = 0;
  asn1_ctx_t asn1_ctx;
  asn1_ctx_init(&asn1_ctx, ASN1_ENC_PER, FALSE, pinfo);
  offset = dissect_rrc_UE_CapabilityContainer_IEs(tvb, offset, &asn1_ctx, tree, hf_rrc_UE_CapabilityContainer_IEs_PDU);
  offset += 7; offset >>= 3;
  return offset;
}
static int dissect_UECapabilityInformation_r3_add_ext_IEs_PDU(tvbuff_t *tvb _U_, packet_info *pinfo _U_, proto_tree *tree _U_, void *data _U_) {
  int offset = 0;
  asn1_ctx_t asn1_ctx;
  asn1_ctx_init(&asn1_ctx, ASN1_ENC_PER, FALSE, pinfo);
  offset = dissect_rrc_UECapabilityInformation_r3_add_ext_IEs(tvb, offset, &asn1_ctx, tree, hf_rrc_UECapabilityInformation_r3_add_ext_IEs_PDU);
  offset += 7; offset >>= 3;
  return offset;
}
static int dissect_URAUpdate_r3_add_ext_IEs_PDU(tvbuff_t *tvb _U_, packet_info *pinfo _U_, proto_tree *tree _U_, void *data _U_) {
  int offset = 0;
  asn1_ctx_t asn1_ctx;
  asn1_ctx_init(&asn1_ctx, ASN1_ENC_PER, FALSE, pinfo);
  offset = dissect_rrc_URAUpdate_r3_add_ext_IEs(tvb, offset, &asn1_ctx, tree, hf_rrc_URAUpdate_r3_add_ext_IEs_PDU);
  offset += 7; offset >>= 3;
  return offset;
}
static int dissect_SRNC_RelocationInfo_v3h0ext_IEs_PDU(tvbuff_t *tvb _U_, packet_info *pinfo _U_, proto_tree *tree _U_, void *data _U_) {
  int offset = 0;
  asn1_ctx_t asn1_ctx;
  asn1_ctx_init(&asn1_ctx, ASN1_ENC_PER, FALSE, pinfo);
  offset = dissect_rrc_SRNC_RelocationInfo_v3h0ext_IEs(tvb, offset, &asn1_ctx, tree, hf_rrc_SRNC_RelocationInfo_v3h0ext_IEs_PDU);
  offset += 7; offset >>= 3;
  return offset;
}
static int dissect_SRNC_RelocationInfo_r6_add_ext_IEs_PDU(tvbuff_t *tvb _U_, packet_info *pinfo _U_, proto_tree *tree _U_, void *data _U_) {
  int offset = 0;
  asn1_ctx_t asn1_ctx;
  asn1_ctx_init(&asn1_ctx, ASN1_ENC_PER, FALSE, pinfo);
  offset = dissect_rrc_SRNC_RelocationInfo_r6_add_ext_IEs(tvb, offset, &asn1_ctx, tree, hf_rrc_SRNC_RelocationInfo_r6_add_ext_IEs_PDU);
  offset += 7; offset >>= 3;
  return offset;
}
static int dissect_SRNC_RelocationInfo_r7_add_ext_IEs_PDU(tvbuff_t *tvb _U_, packet_info *pinfo _U_, proto_tree *tree _U_, void *data _U_) {
  int offset = 0;
  asn1_ctx_t asn1_ctx;
  asn1_ctx_init(&asn1_ctx, ASN1_ENC_PER, FALSE, pinfo);
  offset = dissect_rrc_SRNC_RelocationInfo_r7_add_ext_IEs(tvb, offset, &asn1_ctx, tree, hf_rrc_SRNC_RelocationInfo_r7_add_ext_IEs_PDU);
  offset += 7; offset >>= 3;
  return offset;
}
static int dissect_MeasurementReport_PDU(tvbuff_t *tvb _U_, packet_info *pinfo _U_, proto_tree *tree _U_, void *data _U_) {
  int offset = 0;
  asn1_ctx_t asn1_ctx;
  asn1_ctx_init(&asn1_ctx, ASN1_ENC_PER, FALSE, pinfo);
  offset = dissect_rrc_MeasurementReport(tvb, offset, &asn1_ctx, tree, hf_rrc_MeasurementReport_PDU);
  offset += 7; offset >>= 3;
  return offset;
}


/*--- End of included file: packet-rrc-fn.c ---*/
#line 181 "../../asn1/rrc/packet-rrc-template.c"

#include "packet-rrc.h"


static void
dissect_rrc(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
{
    /* FIX ME Currently don't know the 'starting point' of this protocol
     * exported DL-DCCH-Message is the entry point.
     */
    proto_item    *rrc_item = NULL;
    proto_tree    *rrc_tree = NULL;
    struct rrc_info *rrcinf;

    top_tree = tree;
    rrcinf = p_get_proto_data(pinfo->fd, proto_rrc);

    /* make entry in the Protocol column on summary display */
    col_set_str(pinfo->cinfo, COL_PROTOCOL, "RRC");

    /*Clear memory*/
    memset(num_chans_per_flow,0,sizeof(guint8)*RRC_MAX_NUM_HSDHSCH_MACDFLOW);

    /* create the rrc protocol tree */
    rrc_item = proto_tree_add_item(tree, proto_rrc, tvb, 0, -1, ENC_NA);
    rrc_tree = proto_item_add_subtree(rrc_item, ett_rrc);

    if (rrcinf) {
        switch (rrcinf->msgtype[pinfo->fd->subnum]) {
            case RRC_MESSAGE_TYPE_PCCH:
                call_dissector(rrc_pcch_handle, tvb, pinfo, rrc_tree);
                break;
            case RRC_MESSAGE_TYPE_UL_CCCH:
                call_dissector(rrc_ul_ccch_handle, tvb, pinfo, rrc_tree);
                break;
            case RRC_MESSAGE_TYPE_DL_CCCH:
                call_dissector(rrc_dl_ccch_handle, tvb, pinfo, rrc_tree);
                break;
            case RRC_MESSAGE_TYPE_UL_DCCH:
                call_dissector(rrc_ul_dcch_handle, tvb, pinfo, rrc_tree);
                break;
            case RRC_MESSAGE_TYPE_DL_DCCH:
                call_dissector(rrc_dl_dcch_handle, tvb, pinfo, rrc_tree);
                break;
            case RRC_MESSAGE_TYPE_BCCH_FACH:
                call_dissector(rrc_bcch_fach_handle, tvb, pinfo, rrc_tree);
                break;
            default:
                ;
        }
    }
}

void rrc_init(void){
    /*Cleanup*/
    if(hsdsch_muxed_flows){
        g_tree_destroy(hsdsch_muxed_flows);
    }
    if(rrc_ciph_inf){
        g_tree_destroy(rrc_ciph_inf);
    }
    /*Initialize structure for muxed flow indication*/
    hsdsch_muxed_flows = g_tree_new_full(rrc_key_cmp,
                       NULL,      /* data pointer, optional */
                       rrc_free_key,
                       rrc_free_value);

    /*Initialize structure for muxed flow indication*/
    rrc_ciph_inf = g_tree_new_full(rrc_key_cmp,
                       NULL,      /* data pointer, optional */
                       NULL,
                       rrc_free_value);
}
/*--- proto_register_rrc -------------------------------------------*/
void proto_register_rrc(void) {

  /* List of fields */
  static hf_register_info hf[] = {


/*--- Included file: packet-rrc-hfarr.c ---*/
#line 1 "../../asn1/rrc/packet-rrc-hfarr.c"
    { &hf_rrc_DL_DCCH_Message_PDU,
      { "DL-DCCH-Message", "rrc.DL_DCCH_Message",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_UL_DCCH_Message_PDU,
      { "UL-DCCH-Message", "rrc.UL_DCCH_Message",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_DL_CCCH_Message_PDU,
      { "DL-CCCH-Message", "rrc.DL_CCCH_Message",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_UL_CCCH_Message_PDU,
      { "UL-CCCH-Message", "rrc.UL_CCCH_Message",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_PCCH_Message_PDU,
      { "PCCH-Message", "rrc.PCCH_Message",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_DL_SHCCH_Message_PDU,
      { "DL-SHCCH-Message", "rrc.DL_SHCCH_Message",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_UL_SHCCH_Message_PDU,
      { "UL-SHCCH-Message", "rrc.UL_SHCCH_Message",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_BCCH_FACH_Message_PDU,
      { "BCCH-FACH-Message", "rrc.BCCH_FACH_Message",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_BCCH_BCH_Message_PDU,
      { "BCCH-BCH-Message", "rrc.BCCH_BCH_Message",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_MCCH_Message_PDU,
      { "MCCH-Message", "rrc.MCCH_Message",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_MSCH_Message_PDU,
      { "MSCH-Message", "rrc.MSCH_Message",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_rrc_HandoverToUTRANCommand_PDU,
      { "HandoverToUTRANCommand", "rrc.HandoverToUTRANCommand",
        FT_UINT32, BASE_DEC, VALS(rrc_HandoverToUTRANCommand_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_rrc_InterRATHandoverInfo_PDU,
      { "InterRATHandoverInfo", "rrc.InterRATHandoverInfo",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_SystemInformation_BCH_PDU,
      { "SystemInformation-BCH", "rrc.SystemInformation_BCH",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_System_Information_Container_PDU,
      { "System-Information-Container", "rrc.System_Information_Container",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_UE_RadioAccessCapabilityInfo_PDU,
      { "UE-RadioAccessCapabilityInfo", "rrc.UE_RadioAccessCapabilityInfo",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_rrc_MasterInformationBlock_PDU,
      { "MasterInformationBlock", "rrc.MasterInformationBlock",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_rrc_SysInfoType1_PDU,
      { "SysInfoType1", "rrc.SysInfoType1",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_rrc_SysInfoType2_PDU,
      { "SysInfoType2", "rrc.SysInfoType2",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_rrc_SysInfoType3_PDU,
      { "SysInfoType3", "rrc.SysInfoType3",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_SysInfoType4_PDU,
      { "SysInfoType4", "rrc.SysInfoType4",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_SysInfoType5_PDU,
      { "SysInfoType5", "rrc.SysInfoType5",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_SysInfoType5bis_PDU,
      { "SysInfoType5bis", "rrc.SysInfoType5bis",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_SysInfoType6_PDU,
      { "SysInfoType6", "rrc.SysInfoType6",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_rrc_SysInfoType7_PDU,
      { "SysInfoType7", "rrc.SysInfoType7",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_SysInfoType8_PDU,
      { "SysInfoType8", "rrc.SysInfoType8",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_SysInfoType9_PDU,
      { "SysInfoType9", "rrc.SysInfoType9",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_SysInfoType10_PDU,
      { "SysInfoType10", "rrc.SysInfoType10",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_SysInfoType11_PDU,
      { "SysInfoType11", "rrc.SysInfoType11",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_SysInfoType11bis_PDU,
      { "SysInfoType11bis", "rrc.SysInfoType11bis",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_rrc_SysInfoType12_PDU,
      { "SysInfoType12", "rrc.SysInfoType12",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_SysInfoType13_PDU,
      { "SysInfoType13", "rrc.SysInfoType13",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_SysInfoType13_1_PDU,
      { "SysInfoType13-1", "rrc.SysInfoType13_1",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_SysInfoType13_2_PDU,
      { "SysInfoType13-2", "rrc.SysInfoType13_2",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_SysInfoType13_3_PDU,
      { "SysInfoType13-3", "rrc.SysInfoType13_3",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_SysInfoType13_4_PDU,
      { "SysInfoType13-4", "rrc.SysInfoType13_4",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_SysInfoType14_PDU,
      { "SysInfoType14", "rrc.SysInfoType14",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_SysInfoType15_PDU,
      { "SysInfoType15", "rrc.SysInfoType15",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_SysInfoType15bis_PDU,
      { "SysInfoType15bis", "rrc.SysInfoType15bis",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_SysInfoType15_1_PDU,
      { "SysInfoType15-1", "rrc.SysInfoType15_1",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_SysInfoType15_1bis_PDU,
      { "SysInfoType15-1bis", "rrc.SysInfoType15_1bis",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_SysInfoType15_2_PDU,
      { "SysInfoType15-2", "rrc.SysInfoType15_2",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_SysInfoType15_2bis_PDU,
      { "SysInfoType15-2bis", "rrc.SysInfoType15_2bis",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_SysInfoType15_2ter_PDU,
      { "SysInfoType15-2ter", "rrc.SysInfoType15_2ter",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_SysInfoType15_3_PDU,
      { "SysInfoType15-3", "rrc.SysInfoType15_3",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_SysInfoType15_3bis_PDU,
      { "SysInfoType15-3bis", "rrc.SysInfoType15_3bis",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_SysInfoType15_4_PDU,
      { "SysInfoType15-4", "rrc.SysInfoType15_4",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_SysInfoType15_5_PDU,
      { "SysInfoType15-5", "rrc.SysInfoType15_5",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_SysInfoType15_6_PDU,
      { "SysInfoType15-6", "rrc.SysInfoType15_6",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_SysInfoType15_7_PDU,
      { "SysInfoType15-7", "rrc.SysInfoType15_7",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_SysInfoType15_8_PDU,
      { "SysInfoType15-8", "rrc.SysInfoType15_8",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_SysInfoType16_PDU,
      { "SysInfoType16", "rrc.SysInfoType16",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_SysInfoType17_PDU,
      { "SysInfoType17", "rrc.SysInfoType17",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_SysInfoType18_PDU,
      { "SysInfoType18", "rrc.SysInfoType18",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_SysInfoType19_PDU,
      { "SysInfoType19", "rrc.SysInfoType19",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_SysInfoType20_PDU,
      { "SysInfoType20", "rrc.SysInfoType20",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_SysInfoType21_PDU,
      { "SysInfoType21", "rrc.SysInfoType21",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_SysInfoType22_PDU,
      { "SysInfoType22", "rrc.SysInfoType22",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_SysInfoTypeSB1_PDU,
      { "SysInfoTypeSB1", "rrc.SysInfoTypeSB1",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_SysInfoTypeSB2_PDU,
      { "SysInfoTypeSB2", "rrc.SysInfoTypeSB2",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_rrc_ToTargetRNC_Container_PDU,
      { "ToTargetRNC-Container", "rrc.ToTargetRNC_Container",
        FT_UINT32, BASE_DEC, VALS(rrc_ToTargetRNC_Container_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_rrc_TargetRNC_ToSourceRNC_Container_PDU,
      { "TargetRNC-ToSourceRNC-Container", "rrc.TargetRNC_ToSourceRNC_Container",
        FT_UINT32, BASE_DEC, VALS(rrc_TargetRNC_ToSourceRNC_Container_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_CellUpdate_r3_add_ext_IEs_PDU,
      { "CellUpdate-r3-add-ext-IEs", "rrc.CellUpdate_r3_add_ext_IEs",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_CellUpdateConfirm_r7_add_ext_IEs_PDU,
      { "CellUpdateConfirm-r7-add-ext-IEs", "rrc.CellUpdateConfirm_r7_add_ext_IEs",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_InitialDirectTransfer_r3_add_ext_IEs_PDU,
      { "InitialDirectTransfer-r3-add-ext-IEs", "rrc.InitialDirectTransfer_r3_add_ext_IEs",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_InterRATHandoverInfo_r3_add_ext_IEs_PDU,
      { "InterRATHandoverInfo-r3-add-ext-IEs", "rrc.InterRATHandoverInfo_r3_add_ext_IEs",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_RadioBearerSetup_r7_add_ext_IEs_PDU,
      { "RadioBearerSetup-r7-add-ext-IEs", "rrc.RadioBearerSetup_r7_add_ext_IEs",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_RRCConnectionSetupComplete_r3_add_ext_IEs_PDU,
      { "RRCConnectionSetupComplete-r3-add-ext-IEs", "rrc.RRCConnectionSetupComplete_r3_add_ext_IEs",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_UE_CapabilityContainer_IEs_PDU,
      { "UE-CapabilityContainer-IEs", "rrc.UE_CapabilityContainer_IEs",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_UECapabilityInformation_r3_add_ext_IEs_PDU,
      { "UECapabilityInformation-r3-add-ext-IEs", "rrc.UECapabilityInformation_r3_add_ext_IEs",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_URAUpdate_r3_add_ext_IEs_PDU,
      { "URAUpdate-r3-add-ext-IEs", "rrc.URAUpdate_r3_add_ext_IEs",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_SRNC_RelocationInfo_v3h0ext_IEs_PDU,
      { "SRNC-RelocationInfo-v3h0ext-IEs", "rrc.SRNC_RelocationInfo_v3h0ext_IEs",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_SRNC_RelocationInfo_r6_add_ext_IEs_PDU,
      { "SRNC-RelocationInfo-r6-add-ext-IEs", "rrc.SRNC_RelocationInfo_r6_add_ext_IEs",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_SRNC_RelocationInfo_r7_add_ext_IEs_PDU,
      { "SRNC-RelocationInfo-r7-add-ext-IEs", "rrc.SRNC_RelocationInfo_r7_add_ext_IEs",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_MeasurementReport_PDU,
      { "MeasurementReport", "rrc.MeasurementReport",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_integrityCheckInfo,
      { "integrityCheckInfo", "rrc.integrityCheckInfo",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_message,
      { "message", "rrc.message",
        FT_UINT32, BASE_DEC|BASE_EXT_STRING, &rrc_DL_DCCH_MessageType_vals_ext, 0,
        "DL_DCCH_MessageType", HFILL }},
    { &hf_rrc_activeSetUpdate,
      { "activeSetUpdate", "rrc.activeSetUpdate",
        FT_UINT32, BASE_DEC, VALS(rrc_ActiveSetUpdate_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_assistanceDataDelivery,
      { "assistanceDataDelivery", "rrc.assistanceDataDelivery",
        FT_UINT32, BASE_DEC, VALS(rrc_AssistanceDataDelivery_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_cellChangeOrderFromUTRAN,
      { "cellChangeOrderFromUTRAN", "rrc.cellChangeOrderFromUTRAN",
        FT_UINT32, BASE_DEC, VALS(rrc_CellChangeOrderFromUTRAN_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_cellUpdateConfirm,
      { "cellUpdateConfirm", "rrc.cellUpdateConfirm",
        FT_UINT32, BASE_DEC, VALS(rrc_CellUpdateConfirm_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_counterCheck,
      { "counterCheck", "rrc.counterCheck",
        FT_UINT32, BASE_DEC, VALS(rrc_CounterCheck_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_downlinkDirectTransfer,
      { "downlinkDirectTransfer", "rrc.downlinkDirectTransfer",
        FT_UINT32, BASE_DEC, VALS(rrc_DownlinkDirectTransfer_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_handoverFromUTRANCommand_GSM,
      { "handoverFromUTRANCommand-GSM", "rrc.handoverFromUTRANCommand_GSM",
        FT_UINT32, BASE_DEC, VALS(rrc_HandoverFromUTRANCommand_GSM_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_handoverFromUTRANCommand_CDMA2000,
      { "handoverFromUTRANCommand-CDMA2000", "rrc.handoverFromUTRANCommand_CDMA2000",
        FT_UINT32, BASE_DEC, VALS(rrc_HandoverFromUTRANCommand_CDMA2000_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_measurementControl,
      { "measurementControl", "rrc.measurementControl",
        FT_UINT32, BASE_DEC, VALS(rrc_MeasurementControl_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_pagingType2,
      { "pagingType2", "rrc.pagingType2",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_physicalChannelReconfiguration,
      { "physicalChannelReconfiguration", "rrc.physicalChannelReconfiguration",
        FT_UINT32, BASE_DEC, VALS(rrc_PhysicalChannelReconfiguration_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_physicalSharedChannelAllocation,
      { "physicalSharedChannelAllocation", "rrc.physicalSharedChannelAllocation",
        FT_UINT32, BASE_DEC, VALS(rrc_PhysicalSharedChannelAllocation_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_radioBearerReconfiguration,
      { "radioBearerReconfiguration", "rrc.radioBearerReconfiguration",
        FT_UINT32, BASE_DEC, VALS(rrc_RadioBearerReconfiguration_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_radioBearerRelease,
      { "radioBearerRelease", "rrc.radioBearerRelease",
        FT_UINT32, BASE_DEC, VALS(rrc_RadioBearerRelease_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_radioBearerSetup,
      { "radioBearerSetup", "rrc.radioBearerSetup",
        FT_UINT32, BASE_DEC, VALS(rrc_RadioBearerSetup_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_rrcConnectionRelease,
      { "rrcConnectionRelease", "rrc.rrcConnectionRelease",
        FT_UINT32, BASE_DEC, VALS(rrc_RRCConnectionRelease_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_securityModeCommand,
      { "securityModeCommand", "rrc.securityModeCommand",
        FT_UINT32, BASE_DEC, VALS(rrc_SecurityModeCommand_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_signallingConnectionRelease,
      { "signallingConnectionRelease", "rrc.signallingConnectionRelease",
        FT_UINT32, BASE_DEC, VALS(rrc_SignallingConnectionRelease_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_transportChannelReconfiguration,
      { "transportChannelReconfiguration", "rrc.transportChannelReconfiguration",
        FT_UINT32, BASE_DEC, VALS(rrc_TransportChannelReconfiguration_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_transportFormatCombinationControl,
      { "transportFormatCombinationControl", "rrc.transportFormatCombinationControl",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_ueCapabilityEnquiry,
      { "ueCapabilityEnquiry", "rrc.ueCapabilityEnquiry",
        FT_UINT32, BASE_DEC, VALS(rrc_UECapabilityEnquiry_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_ueCapabilityInformationConfirm,
      { "ueCapabilityInformationConfirm", "rrc.ueCapabilityInformationConfirm",
        FT_UINT32, BASE_DEC, VALS(rrc_UECapabilityInformationConfirm_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_uplinkPhysicalChannelControl,
      { "uplinkPhysicalChannelControl", "rrc.uplinkPhysicalChannelControl",
        FT_UINT32, BASE_DEC, VALS(rrc_UplinkPhysicalChannelControl_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_uraUpdateConfirm,
      { "uraUpdateConfirm", "rrc.uraUpdateConfirm",
        FT_UINT32, BASE_DEC, VALS(rrc_URAUpdateConfirm_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_utranMobilityInformation,
      { "utranMobilityInformation", "rrc.utranMobilityInformation",
        FT_UINT32, BASE_DEC, VALS(rrc_UTRANMobilityInformation_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_handoverFromUTRANCommand_GERANIu,
      { "handoverFromUTRANCommand-GERANIu", "rrc.handoverFromUTRANCommand_GERANIu",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_mbmsModifiedServicesInformation,
      { "mbmsModifiedServicesInformation", "rrc.mbmsModifiedServicesInformation",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_etwsPrimaryNotificationWithSecurity,
      { "etwsPrimaryNotificationWithSecurity", "rrc.etwsPrimaryNotificationWithSecurity",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_handoverFromUTRANCommand_EUTRA,
      { "handoverFromUTRANCommand-EUTRA", "rrc.handoverFromUTRANCommand_EUTRA",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_ueInformationRequest,
      { "ueInformationRequest", "rrc.ueInformationRequest",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_loggingMeasurementConfiguration,
      { "loggingMeasurementConfiguration", "rrc.loggingMeasurementConfiguration",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_spare1,
      { "spare1", "rrc.spare1",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_message_01,
      { "message", "rrc.message",
        FT_UINT32, BASE_DEC|BASE_EXT_STRING, &rrc_UL_DCCH_MessageType_vals_ext, 0,
        "UL_DCCH_MessageType", HFILL }},
    { &hf_rrc_activeSetUpdateComplete,
      { "activeSetUpdateComplete", "rrc.activeSetUpdateComplete",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_activeSetUpdateFailure,
      { "activeSetUpdateFailure", "rrc.activeSetUpdateFailure",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_cellChangeOrderFromUTRANFailure,
      { "cellChangeOrderFromUTRANFailure", "rrc.cellChangeOrderFromUTRANFailure",
        FT_UINT32, BASE_DEC, VALS(rrc_CellChangeOrderFromUTRANFailure_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_counterCheckResponse,
      { "counterCheckResponse", "rrc.counterCheckResponse",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_handoverToUTRANComplete,
      { "handoverToUTRANComplete", "rrc.handoverToUTRANComplete",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_initialDirectTransfer,
      { "initialDirectTransfer", "rrc.initialDirectTransfer",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_handoverFromUTRANFailure,
      { "handoverFromUTRANFailure", "rrc.handoverFromUTRANFailure",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_measurementControlFailure,
      { "measurementControlFailure", "rrc.measurementControlFailure",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_measurementReport,
      { "measurementReport", "rrc.measurementReport",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_physicalChannelReconfigurationComplete,
      { "physicalChannelReconfigurationComplete", "rrc.physicalChannelReconfigurationComplete",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_physicalChannelReconfigurationFailure,
      { "physicalChannelReconfigurationFailure", "rrc.physicalChannelReconfigurationFailure",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_radioBearerReconfigurationComplete,
      { "radioBearerReconfigurationComplete", "rrc.radioBearerReconfigurationComplete",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_radioBearerReconfigurationFailure,
      { "radioBearerReconfigurationFailure", "rrc.radioBearerReconfigurationFailure",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_radioBearerReleaseComplete,
      { "radioBearerReleaseComplete", "rrc.radioBearerReleaseComplete",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_radioBearerReleaseFailure,
      { "radioBearerReleaseFailure", "rrc.radioBearerReleaseFailure",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_radioBearerSetupComplete,
      { "radioBearerSetupComplete", "rrc.radioBearerSetupComplete",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_radioBearerSetupFailure,
      { "radioBearerSetupFailure", "rrc.radioBearerSetupFailure",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_rrcConnectionReleaseComplete,
      { "rrcConnectionReleaseComplete", "rrc.rrcConnectionReleaseComplete",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_rrcConnectionSetupComplete,
      { "rrcConnectionSetupComplete", "rrc.rrcConnectionSetupComplete",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_rrcStatus,
      { "rrcStatus", "rrc.rrcStatus",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_securityModeComplete,
      { "securityModeComplete", "rrc.securityModeComplete",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_securityModeFailure,
      { "securityModeFailure", "rrc.securityModeFailure",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_signallingConnectionReleaseIndication,
      { "signallingConnectionReleaseIndication", "rrc.signallingConnectionReleaseIndication",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_transportChannelReconfigurationComplete,
      { "transportChannelReconfigurationComplete", "rrc.transportChannelReconfigurationComplete",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_transportChannelReconfigurationFailure,
      { "transportChannelReconfigurationFailure", "rrc.transportChannelReconfigurationFailure",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_transportFormatCombinationControlFailure,
      { "transportFormatCombinationControlFailure", "rrc.transportFormatCombinationControlFailure",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_ueCapabilityInformation,
      { "ueCapabilityInformation", "rrc.ueCapabilityInformation",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_uplinkDirectTransfer,
      { "uplinkDirectTransfer", "rrc.uplinkDirectTransfer",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_utranMobilityInformationConfirm,
      { "utranMobilityInformationConfirm", "rrc.utranMobilityInformationConfirm",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_utranMobilityInformationFailure,
      { "utranMobilityInformationFailure", "rrc.utranMobilityInformationFailure",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_mbmsModificationRequest,
      { "mbmsModificationRequest", "rrc.mbmsModificationRequest",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_ul_DCCH_MessageType_ext,
      { "ul-DCCH-MessageType-ext", "rrc.ul_DCCH_MessageType_ext",
        FT_UINT32, BASE_DEC, VALS(rrc_UL_DCCH_MessageType_ext_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_ueInformationResponse,
      { "ueInformationResponse", "rrc.ueInformationResponse",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_spare15,
      { "spare15", "rrc.spare15",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_spare14,
      { "spare14", "rrc.spare14",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_spare13,
      { "spare13", "rrc.spare13",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_spare12,
      { "spare12", "rrc.spare12",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_spare11,
      { "spare11", "rrc.spare11",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_spare10,
      { "spare10", "rrc.spare10",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_spare9,
      { "spare9", "rrc.spare9",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_spare8,
      { "spare8", "rrc.spare8",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_spare7,
      { "spare7", "rrc.spare7",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_spare6,
      { "spare6", "rrc.spare6",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_spare5,
      { "spare5", "rrc.spare5",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_spare4,
      { "spare4", "rrc.spare4",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_spare3,
      { "spare3", "rrc.spare3",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_spare2,
      { "spare2", "rrc.spare2",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_message_02,
      { "message", "rrc.message",
        FT_UINT32, BASE_DEC, VALS(rrc_DL_CCCH_MessageType_vals), 0,
        "DL_CCCH_MessageType", HFILL }},
    { &hf_rrc_cellUpdateConfirm_01,
      { "cellUpdateConfirm", "rrc.cellUpdateConfirm",
        FT_UINT32, BASE_DEC, VALS(rrc_CellUpdateConfirm_CCCH_vals), 0,
        "CellUpdateConfirm_CCCH", HFILL }},
    { &hf_rrc_rrcConnectionReject,
      { "rrcConnectionReject", "rrc.rrcConnectionReject",
        FT_UINT32, BASE_DEC, VALS(rrc_RRCConnectionReject_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_rrcConnectionRelease_01,
      { "rrcConnectionRelease", "rrc.rrcConnectionRelease",
        FT_UINT32, BASE_DEC, VALS(rrc_RRCConnectionRelease_CCCH_vals), 0,
        "RRCConnectionRelease_CCCH", HFILL }},
    { &hf_rrc_rrcConnectionSetup,
      { "rrcConnectionSetup", "rrc.rrcConnectionSetup",
        FT_UINT32, BASE_DEC, VALS(rrc_RRCConnectionSetup_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_uraUpdateConfirm_01,
      { "uraUpdateConfirm", "rrc.uraUpdateConfirm",
        FT_UINT32, BASE_DEC, VALS(rrc_URAUpdateConfirm_CCCH_vals), 0,
        "URAUpdateConfirm_CCCH", HFILL }},
    { &hf_rrc_message_03,
      { "message", "rrc.message",
        FT_UINT32, BASE_DEC, VALS(rrc_UL_CCCH_MessageType_vals), 0,
        "UL_CCCH_MessageType", HFILL }},
    { &hf_rrc_cellUpdate,
      { "cellUpdate", "rrc.cellUpdate",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_rrcConnectionRequest,
      { "rrcConnectionRequest", "rrc.rrcConnectionRequest",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_uraUpdate,
      { "uraUpdate", "rrc.uraUpdate",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_spare,
      { "spare", "rrc.spare",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_message_04,
      { "message", "rrc.message",
        FT_UINT32, BASE_DEC, VALS(rrc_PCCH_MessageType_vals), 0,
        "PCCH_MessageType", HFILL }},
    { &hf_rrc_pagingType1,
      { "pagingType1", "rrc.pagingType1",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_message_05,
      { "message", "rrc.message",
        FT_UINT32, BASE_DEC, VALS(rrc_DL_SHCCH_MessageType_vals), 0,
        "DL_SHCCH_MessageType", HFILL }},
    { &hf_rrc_message_06,
      { "message", "rrc.message",
        FT_UINT32, BASE_DEC, VALS(rrc_UL_SHCCH_MessageType_vals), 0,
        "UL_SHCCH_MessageType", HFILL }},
    { &hf_rrc_puschCapacityRequest,
      { "puschCapacityRequest", "rrc.puschCapacityRequest",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_message_07,
      { "message", "rrc.message",
        FT_UINT32, BASE_DEC, VALS(rrc_BCCH_FACH_MessageType_vals), 0,
        "BCCH_FACH_MessageType", HFILL }},
    { &hf_rrc_dummy,
      { "dummy", "rrc.dummy",
        FT_NONE, BASE_NONE, NULL, 0,
        "SystemInformation_FACH", HFILL }},
    { &hf_rrc_systemInformationChangeIndication,
      { "systemInformationChangeIndication", "rrc.systemInformationChangeIndication",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_message_08,
      { "message", "rrc.message",
        FT_NONE, BASE_NONE, NULL, 0,
        "SystemInformation_BCH", HFILL }},
    { &hf_rrc_message_09,
      { "message", "rrc.message",
        FT_UINT32, BASE_DEC, VALS(rrc_MCCH_MessageType_vals), 0,
        "MCCH_MessageType", HFILL }},
    { &hf_rrc_mbmsAccessInformation,
      { "mbmsAccessInformation", "rrc.mbmsAccessInformation",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_mbmsCommonPTMRBInformation,
      { "mbmsCommonPTMRBInformation", "rrc.mbmsCommonPTMRBInformation",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_mbmsCurrentCellPTMRBInformation,
      { "mbmsCurrentCellPTMRBInformation", "rrc.mbmsCurrentCellPTMRBInformation",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_mbmsGeneralInformation,
      { "mbmsGeneralInformation", "rrc.mbmsGeneralInformation",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_mbmsNeighbouringCellPTMRBInformation,
      { "mbmsNeighbouringCellPTMRBInformation", "rrc.mbmsNeighbouringCellPTMRBInformation",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_mbmsUnmodifiedServicesInformation,
      { "mbmsUnmodifiedServicesInformation", "rrc.mbmsUnmodifiedServicesInformation",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_message_10,
      { "message", "rrc.message",
        FT_UINT32, BASE_DEC, VALS(rrc_MSCH_MessageType_vals), 0,
        "MSCH_MessageType", HFILL }},
    { &hf_rrc_mbmsSchedulingInformation,
      { "mbmsSchedulingInformation", "rrc.mbmsSchedulingInformation",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_r3,
      { "r3", "rrc.r3",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_activeSetUpdate_r3,
      { "activeSetUpdate-r3", "rrc.activeSetUpdate_r3",
        FT_NONE, BASE_NONE, NULL, 0,
        "ActiveSetUpdate_r3_IEs", HFILL }},
    { &hf_rrc_laterNonCriticalExtensions,
      { "laterNonCriticalExtensions", "rrc.laterNonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_activeSetUpdate_r3_add_ext,
      { "activeSetUpdate-r3-add-ext", "rrc.activeSetUpdate_r3_add_ext",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING", HFILL }},
    { &hf_rrc_v4b0NonCriticalExtensions,
      { "v4b0NonCriticalExtensions", "rrc.v4b0NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_activeSetUpdate_v4b0ext,
      { "activeSetUpdate-v4b0ext", "rrc.activeSetUpdate_v4b0ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "ActiveSetUpdate_v4b0ext_IEs", HFILL }},
    { &hf_rrc_v590NonCriticalExtensions,
      { "v590NonCriticalExtensions", "rrc.v590NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_activeSetUpdate_v590ext,
      { "activeSetUpdate-v590ext", "rrc.activeSetUpdate_v590ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "ActiveSetUpdate_v590ext_IEs", HFILL }},
    { &hf_rrc_v690NonCriticalExtensions,
      { "v690NonCriticalExtensions", "rrc.v690NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_activeSetUpdate_v690ext,
      { "activeSetUpdate-v690ext", "rrc.activeSetUpdate_v690ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "ActiveSetUpdate_v690ext_IEs", HFILL }},
    { &hf_rrc_nonCriticalExtensions,
      { "nonCriticalExtensions", "rrc.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_later_than_r3,
      { "later-than-r3", "rrc.later_than_r3",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_rrc_TransactionIdentifier,
      { "rrc-TransactionIdentifier", "rrc.rrc_TransactionIdentifier",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_criticalExtensions,
      { "criticalExtensions", "rrc.criticalExtensions",
        FT_UINT32, BASE_DEC, VALS(rrc_T_criticalExtensions_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_r6,
      { "r6", "rrc.r6",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_activeSetUpdate_r6,
      { "activeSetUpdate-r6", "rrc.activeSetUpdate_r6",
        FT_NONE, BASE_NONE, NULL, 0,
        "ActiveSetUpdate_r6_IEs", HFILL }},
    { &hf_rrc_activeSetUpdate_r6_add_ext,
      { "activeSetUpdate-r6-add-ext", "rrc.activeSetUpdate_r6_add_ext",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING", HFILL }},
    { &hf_rrc_v6b0NonCriticalExtensions,
      { "v6b0NonCriticalExtensions", "rrc.v6b0NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_activeSetUpdate_v6b0ext,
      { "activeSetUpdate-v6b0ext", "rrc.activeSetUpdate_v6b0ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "ActiveSetUpdate_v6b0ext_IEs", HFILL }},
    { &hf_rrc_nonCriticalExtensions_01,
      { "nonCriticalExtensions", "rrc.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_nonCriticalExtensions_01", HFILL }},
    { &hf_rrc_criticalExtensions_01,
      { "criticalExtensions", "rrc.criticalExtensions",
        FT_UINT32, BASE_DEC, VALS(rrc_T_criticalExtensions_01_vals), 0,
        "T_criticalExtensions_01", HFILL }},
    { &hf_rrc_r7,
      { "r7", "rrc.r7",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_activeSetUpdate_r7,
      { "activeSetUpdate-r7", "rrc.activeSetUpdate_r7",
        FT_NONE, BASE_NONE, NULL, 0,
        "ActiveSetUpdate_r7_IEs", HFILL }},
    { &hf_rrc_activeSetUpdate_r7_add_ext,
      { "activeSetUpdate-r7-add-ext", "rrc.activeSetUpdate_r7_add_ext",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING", HFILL }},
    { &hf_rrc_v780NonCriticalExtensions,
      { "v780NonCriticalExtensions", "rrc.v780NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_activeSetUpdate_v780ext,
      { "activeSetUpdate-v780ext", "rrc.activeSetUpdate_v780ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "ActiveSetUpdate_v780ext_IEs", HFILL }},
    { &hf_rrc_v7f0NonCriticalExtensions,
      { "v7f0NonCriticalExtensions", "rrc.v7f0NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_activeSetUdpate_v7f0ext,
      { "activeSetUdpate-v7f0ext", "rrc.activeSetUdpate_v7f0ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "ActiveSetUpdate_v7f0ext_IEs", HFILL }},
    { &hf_rrc_v7g0NonCriticalExtensions,
      { "v7g0NonCriticalExtensions", "rrc.v7g0NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_activeSetUdpate_v7g0ext,
      { "activeSetUdpate-v7g0ext", "rrc.activeSetUdpate_v7g0ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "ActiveSetUpdate_v7g0ext_IEs", HFILL }},
    { &hf_rrc_nonCriticalExtensions_02,
      { "nonCriticalExtensions", "rrc.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_nonCriticalExtensions_02", HFILL }},
    { &hf_rrc_criticalExtensions_02,
      { "criticalExtensions", "rrc.criticalExtensions",
        FT_UINT32, BASE_DEC, VALS(rrc_T_criticalExtensions_02_vals), 0,
        "T_criticalExtensions_02", HFILL }},
    { &hf_rrc_r8,
      { "r8", "rrc.r8",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_activeSetUpdate_r8,
      { "activeSetUpdate-r8", "rrc.activeSetUpdate_r8",
        FT_NONE, BASE_NONE, NULL, 0,
        "ActiveSetUpdate_r8_IEs", HFILL }},
    { &hf_rrc_activeSetUpdate_r8_add_ext,
      { "activeSetUpdate-r8-add-ext", "rrc.activeSetUpdate_r8_add_ext",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING", HFILL }},
    { &hf_rrc_v890NonCriticalExtensions,
      { "v890NonCriticalExtensions", "rrc.v890NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_activeSetUpdate_v890ext,
      { "activeSetUpdate-v890ext", "rrc.activeSetUpdate_v890ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "ActiveSetUpdate_v890ext_IEs", HFILL }},
    { &hf_rrc_v7g0NonCriticalExtensions_01,
      { "v7g0NonCriticalExtensions", "rrc.v7g0NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v7g0NonCriticalExtensions_01", HFILL }},
    { &hf_rrc_activeSetUpdate_v7g0ext,
      { "activeSetUpdate-v7g0ext", "rrc.activeSetUpdate_v7g0ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "ActiveSetUpdate_v7g0ext_IEs", HFILL }},
    { &hf_rrc_nonCriticalExtensions_03,
      { "nonCriticalExtensions", "rrc.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_nonCriticalExtensions_03", HFILL }},
    { &hf_rrc_criticalExtensions_03,
      { "criticalExtensions", "rrc.criticalExtensions",
        FT_UINT32, BASE_DEC, VALS(rrc_T_criticalExtensions_03_vals), 0,
        "T_criticalExtensions_03", HFILL }},
    { &hf_rrc_r9,
      { "r9", "rrc.r9",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_activeSetUpdate_r9,
      { "activeSetUpdate-r9", "rrc.activeSetUpdate_r9",
        FT_NONE, BASE_NONE, NULL, 0,
        "ActiveSetUpdate_r9_IEs", HFILL }},
    { &hf_rrc_activeSetUpdate_r9_add_ext,
      { "activeSetUpdate-r9-add-ext", "rrc.activeSetUpdate_r9_add_ext",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING", HFILL }},
    { &hf_rrc_v950NonCriticalExtensions,
      { "v950NonCriticalExtensions", "rrc.v950NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_activeSetUpdate_v950ext,
      { "activeSetUpdate-v950ext", "rrc.activeSetUpdate_v950ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "ActiveSetUpdate_v950ext_IEs", HFILL }},
    { &hf_rrc_nonCriticalExtensions_04,
      { "nonCriticalExtensions", "rrc.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_nonCriticalExtensions_04", HFILL }},
    { &hf_rrc_criticalExtensions_04,
      { "criticalExtensions", "rrc.criticalExtensions",
        FT_UINT32, BASE_DEC, VALS(rrc_T_criticalExtensions_04_vals), 0,
        "T_criticalExtensions_04", HFILL }},
    { &hf_rrc_r10,
      { "r10", "rrc.r10",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_activeSetUpdate_r10,
      { "activeSetUpdate-r10", "rrc.activeSetUpdate_r10",
        FT_NONE, BASE_NONE, NULL, 0,
        "ActiveSetUpdate_r10_IEs", HFILL }},
    { &hf_rrc_activeSetUpdate_r10_add_ext,
      { "activeSetUpdate-r10-add-ext", "rrc.activeSetUpdate_r10_add_ext",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING", HFILL }},
    { &hf_rrc_vbxyNonCriticalExtensions,
      { "vbxyNonCriticalExtensions", "rrc.vbxyNonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_activeSetUpdate_vbxyext,
      { "activeSetUpdate-vbxyext", "rrc.activeSetUpdate_vbxyext",
        FT_NONE, BASE_NONE, NULL, 0,
        "ActiveSetUpdate_vbxyext_IEs", HFILL }},
    { &hf_rrc_nonCriticalExtensions_05,
      { "nonCriticalExtensions", "rrc.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_nonCriticalExtensions_05", HFILL }},
    { &hf_rrc_criticalExtensions_05,
      { "criticalExtensions", "rrc.criticalExtensions",
        FT_UINT32, BASE_DEC, VALS(rrc_T_criticalExtensions_05_vals), 0,
        "T_criticalExtensions_05", HFILL }},
    { &hf_rrc_r11,
      { "r11", "rrc.r11",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_activeSetUpdate_r11,
      { "activeSetUpdate-r11", "rrc.activeSetUpdate_r11",
        FT_NONE, BASE_NONE, NULL, 0,
        "ActiveSetUpdate_r11_IEs", HFILL }},
    { &hf_rrc_activeSetUpdate_r11_add_ext,
      { "activeSetUpdate-r11-add-ext", "rrc.activeSetUpdate_r11_add_ext",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING", HFILL }},
    { &hf_rrc_nonCriticalExtensions_06,
      { "nonCriticalExtensions", "rrc.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_nonCriticalExtensions_06", HFILL }},
    { &hf_rrc_criticalExtensions_06,
      { "criticalExtensions", "rrc.criticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_criticalExtensions_06", HFILL }},
    { &hf_rrc_dummy_01,
      { "dummy", "rrc.dummy",
        FT_NONE, BASE_NONE, NULL, 0,
        "IntegrityProtectionModeInfo", HFILL }},
    { &hf_rrc_dummy2,
      { "dummy2", "rrc.dummy2",
        FT_NONE, BASE_NONE, NULL, 0,
        "CipheringModeInfo", HFILL }},
    { &hf_rrc_activationTime,
      { "activationTime", "rrc.activationTime",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_newU_RNTI,
      { "newU-RNTI", "rrc.newU_RNTI",
        FT_NONE, BASE_NONE, NULL, 0,
        "U_RNTI", HFILL }},
    { &hf_rrc_cn_InformationInfo,
      { "cn-InformationInfo", "rrc.cn_InformationInfo",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_dummy3,
      { "dummy3", "rrc.dummy3",
        FT_NONE, BASE_NONE, NULL, 0,
        "DL_CounterSynchronisationInfo", HFILL }},
    { &hf_rrc_maxAllowedUL_TX_Power,
      { "maxAllowedUL-TX-Power", "rrc.maxAllowedUL_TX_Power",
        FT_INT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_rl_AdditionInformationList,
      { "rl-AdditionInformationList", "rrc.rl_AdditionInformationList",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_rl_RemovalInformationList,
      { "rl-RemovalInformationList", "rrc.rl_RemovalInformationList",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_tx_DiversityMode,
      { "tx-DiversityMode", "rrc.tx_DiversityMode",
        FT_UINT32, BASE_DEC, VALS(rrc_TX_DiversityMode_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_dummy4,
      { "dummy4", "rrc.dummy4",
        FT_NONE, BASE_NONE, NULL, 0,
        "SSDT_Information", HFILL }},
    { &hf_rrc_dummy_02,
      { "dummy", "rrc.dummy",
        FT_UINT32, BASE_DEC, VALS(rrc_SSDT_UL_vals), 0,
        "SSDT_UL", HFILL }},
    { &hf_rrc_cell_id_PerRL_List,
      { "cell-id-PerRL-List", "rrc.cell_id_PerRL_List",
        FT_UINT32, BASE_DEC, NULL, 0,
        "CellIdentity_PerRL_List", HFILL }},
    { &hf_rrc_dpc_Mode,
      { "dpc-Mode", "rrc.dpc_Mode",
        FT_UINT32, BASE_DEC, VALS(rrc_DPC_Mode_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_dl_TPC_PowerOffsetPerRL_List,
      { "dl-TPC-PowerOffsetPerRL-List", "rrc.dl_TPC_PowerOffsetPerRL_List",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_primary_plmn_Identity,
      { "primary-plmn-Identity", "rrc.primary_plmn_Identity",
        FT_NONE, BASE_NONE, NULL, 0,
        "PLMN_Identity", HFILL }},
    { &hf_rrc_newH_RNTI,
      { "newH-RNTI", "rrc.newH_RNTI",
        FT_BYTES, BASE_NONE, NULL, 0,
        "H_RNTI", HFILL }},
    { &hf_rrc_newPrimary_E_RNTI,
      { "newPrimary-E-RNTI", "rrc.newPrimary_E_RNTI",
        FT_BYTES, BASE_NONE, NULL, 0,
        "E_RNTI", HFILL }},
    { &hf_rrc_newSecondary_E_RNTI,
      { "newSecondary-E-RNTI", "rrc.newSecondary_E_RNTI",
        FT_BYTES, BASE_NONE, NULL, 0,
        "E_RNTI", HFILL }},
    { &hf_rrc_cn_InformationInfo_01,
      { "cn-InformationInfo", "rrc.cn_InformationInfo",
        FT_NONE, BASE_NONE, NULL, 0,
        "CN_InformationInfo_r6", HFILL }},
    { &hf_rrc_rl_AdditionInformationList_01,
      { "rl-AdditionInformationList", "rrc.rl_AdditionInformationList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "RL_AdditionInformationList_r6", HFILL }},
    { &hf_rrc_serving_HSDSCH_CellInformation,
      { "serving-HSDSCH-CellInformation", "rrc.serving_HSDSCH_CellInformation",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_e_dch_ReconfigurationInfo,
      { "e-dch-ReconfigurationInfo", "rrc.e_dch_ReconfigurationInfo",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_rl_AdditionInformation_list_v6b0ext,
      { "rl-AdditionInformation-list-v6b0ext", "rrc.rl_AdditionInformation_list_v6b0ext",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_dtx_drx_TimingInfo,
      { "dtx-drx-TimingInfo", "rrc.dtx_drx_TimingInfo",
        FT_NONE, BASE_NONE, NULL, 0,
        "DTX_DRX_TimingInfo_r7", HFILL }},
    { &hf_rrc_dtx_drx_Info,
      { "dtx-drx-Info", "rrc.dtx_drx_Info",
        FT_NONE, BASE_NONE, NULL, 0,
        "DTX_DRX_Info_r7", HFILL }},
    { &hf_rrc_hs_scch_LessInfo,
      { "hs-scch-LessInfo", "rrc.hs_scch_LessInfo",
        FT_NONE, BASE_NONE, NULL, 0,
        "HS_SCCH_LessInfo_r7", HFILL }},
    { &hf_rrc_mimoParameters,
      { "mimoParameters", "rrc.mimoParameters",
        FT_NONE, BASE_NONE, NULL, 0,
        "MIMO_Parameters_r7", HFILL }},
    { &hf_rrc_rl_AdditionInformationList_02,
      { "rl-AdditionInformationList", "rrc.rl_AdditionInformationList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "RL_AdditionInformationList_r7", HFILL }},
    { &hf_rrc_serving_HSDSCH_CellInformation_01,
      { "serving-HSDSCH-CellInformation", "rrc.serving_HSDSCH_CellInformation",
        FT_NONE, BASE_NONE, NULL, 0,
        "Serving_HSDSCH_CellInformation_r7", HFILL }},
    { &hf_rrc_e_dch_ReconfigurationInfo_01,
      { "e-dch-ReconfigurationInfo", "rrc.e_dch_ReconfigurationInfo",
        FT_NONE, BASE_NONE, NULL, 0,
        "E_DCH_ReconfigurationInfo_r7", HFILL }},
    { &hf_rrc_ul_16QAM_Config,
      { "ul-16QAM-Config", "rrc.ul_16QAM_Config",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_e_dch_ReconfInfoSameCell,
      { "e-dch-ReconfInfoSameCell", "rrc.e_dch_ReconfInfoSameCell",
        FT_NONE, BASE_NONE, NULL, 0,
        "E_DCH_RL_InfoSameServingCell", HFILL }},
    { &hf_rrc_e_TFC_Boost_Info,
      { "e-TFC-Boost-Info", "rrc.e_TFC_Boost_Info",
        FT_NONE, BASE_NONE, NULL, 0,
        "E_TFC_Boost_Info_r7", HFILL }},
    { &hf_rrc_e_DPDCH_PowerInterpolation,
      { "e-DPDCH-PowerInterpolation", "rrc.e_DPDCH_PowerInterpolation",
        FT_BOOLEAN, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_hs_DSCH_TBSizeTable,
      { "hs-DSCH-TBSizeTable", "rrc.hs_DSCH_TBSizeTable",
        FT_UINT32, BASE_DEC, VALS(rrc_HS_DSCH_TBSizeTable_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_mimoParameters_01,
      { "mimoParameters", "rrc.mimoParameters",
        FT_NONE, BASE_NONE, NULL, 0,
        "MIMO_Parameters_v7f0ext", HFILL }},
    { &hf_rrc_mimoParameters_02,
      { "mimoParameters", "rrc.mimoParameters",
        FT_NONE, BASE_NONE, NULL, 0,
        "MIMO_Parameters_v7g0ext", HFILL }},
    { &hf_rrc_mimoParameters_03,
      { "mimoParameters", "rrc.mimoParameters",
        FT_NONE, BASE_NONE, NULL, 0,
        "MIMO_Parameters_r8", HFILL }},
    { &hf_rrc_rl_AdditionInformationList_03,
      { "rl-AdditionInformationList", "rrc.rl_AdditionInformationList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "RL_AdditionInformationList_r8", HFILL }},
    { &hf_rrc_servingCellChangeParameters,
      { "servingCellChangeParameters", "rrc.servingCellChangeParameters",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_serving_HSDSCH_CellInformation_02,
      { "serving-HSDSCH-CellInformation", "rrc.serving_HSDSCH_CellInformation",
        FT_NONE, BASE_NONE, NULL, 0,
        "Serving_HSDSCH_CellInformation_r8", HFILL }},
    { &hf_rrc_dl_SecondaryCellInfoFDD,
      { "dl-SecondaryCellInfoFDD", "rrc.dl_SecondaryCellInfoFDD",
        FT_UINT32, BASE_DEC, VALS(rrc_DL_SecondaryCellInfoFDD_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_dl_SecondaryCellInfoFDD_v890ext,
      { "dl-SecondaryCellInfoFDD-v890ext", "rrc.dl_SecondaryCellInfoFDD_v890ext",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_rl_AdditionInformationList_04,
      { "rl-AdditionInformationList", "rrc.rl_AdditionInformationList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "RL_AdditionInformationList_v890ext", HFILL }},
    { &hf_rrc_mimoParameters_04,
      { "mimoParameters", "rrc.mimoParameters",
        FT_NONE, BASE_NONE, NULL, 0,
        "MIMO_Parameters_r9", HFILL }},
    { &hf_rrc_ul_SecondaryCellInfoFDD,
      { "ul-SecondaryCellInfoFDD", "rrc.ul_SecondaryCellInfoFDD",
        FT_UINT32, BASE_DEC, VALS(rrc_UL_SecondaryCellInfoFDD_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_e_dch_ReconfigurationInfo_SecULFrequency,
      { "e-dch-ReconfigurationInfo-SecULFrequency", "rrc.e_dch_ReconfigurationInfo_SecULFrequency",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_rl_AdditionInformationList_05,
      { "rl-AdditionInformationList", "rrc.rl_AdditionInformationList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "RL_AdditionInformationList_r9", HFILL }},
    { &hf_rrc_rl_AdditionInformationList_SecULFreq,
      { "rl-AdditionInformationList-SecULFreq", "rrc.rl_AdditionInformationList_SecULFreq",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_rl_RemovalInformationList_SecULFreq,
      { "rl-RemovalInformationList-SecULFreq", "rrc.rl_RemovalInformationList_SecULFreq",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_serving_HSDSCH_CellInformation_03,
      { "serving-HSDSCH-CellInformation", "rrc.serving_HSDSCH_CellInformation",
        FT_NONE, BASE_NONE, NULL, 0,
        "Serving_HSDSCH_CellInformation_r9", HFILL }},
    { &hf_rrc_dl_SecondaryCellInfoFDD_01,
      { "dl-SecondaryCellInfoFDD", "rrc.dl_SecondaryCellInfoFDD",
        FT_NONE, BASE_NONE, NULL, 0,
        "DL_SecondaryCellInfoFDD_r9", HFILL }},
    { &hf_rrc_secondaryCellMimoParameters,
      { "secondaryCellMimoParameters", "rrc.secondaryCellMimoParameters",
        FT_NONE, BASE_NONE, NULL, 0,
        "SecondaryCellMIMOparametersFDD_v950ext", HFILL }},
    { &hf_rrc_rl_AdditionInformationList_06,
      { "rl-AdditionInformationList", "rrc.rl_AdditionInformationList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "RL_AdditionInformationList_v950ext", HFILL }},
    { &hf_rrc_rl_AdditionInformationList_07,
      { "rl-AdditionInformationList", "rrc.rl_AdditionInformationList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "RL_AdditionInformationList_r10", HFILL }},
    { &hf_rrc_dl_SecondaryCellInfoFDD_02,
      { "dl-SecondaryCellInfoFDD", "rrc.dl_SecondaryCellInfoFDD",
        FT_NONE, BASE_NONE, NULL, 0,
        "DL_SecondaryCellInfoFDD_r10", HFILL }},
    { &hf_rrc_additionalDLSecCellInfoListFDD,
      { "additionalDLSecCellInfoListFDD", "rrc.additionalDLSecCellInfoListFDD",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_ulOLTDInfoFDD,
      { "ulOLTDInfoFDD", "rrc.ulOLTDInfoFDD",
        FT_NONE, BASE_NONE, NULL, 0,
        "UL_OLTD_InfoFDD", HFILL }},
    { &hf_rrc_rl_AdditionInformationList_08,
      { "rl-AdditionInformationList", "rrc.rl_AdditionInformationList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "RL_AdditionInformationList_vbxyext", HFILL }},
    { &hf_rrc_mimo4x4Parameters,
      { "mimo4x4Parameters", "rrc.mimo4x4Parameters",
        FT_UINT32, BASE_DEC, VALS(rrc_MIMO4x4_Parameters_vals), 0,
        "MIMO4x4_Parameters", HFILL }},
    { &hf_rrc_ul_CLTD_InfoFDD,
      { "ul-CLTD-InfoFDD", "rrc.ul_CLTD_InfoFDD",
        FT_UINT32, BASE_DEC, VALS(rrc_UL_CLTD_InfoFDD_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_f_tpich_ReconfigurationInfo,
      { "f-tpich-ReconfigurationInfo", "rrc.f_tpich_ReconfigurationInfo",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_ul_OLTD_InfoFDD,
      { "ul-OLTD-InfoFDD", "rrc.ul_OLTD_InfoFDD",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_rl_AdditionInformationList_09,
      { "rl-AdditionInformationList", "rrc.rl_AdditionInformationList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "RL_AdditionInformationList_r11", HFILL }},
    { &hf_rrc_serving_HSDSCH_CellInformation_04,
      { "serving-HSDSCH-CellInformation", "rrc.serving_HSDSCH_CellInformation",
        FT_NONE, BASE_NONE, NULL, 0,
        "Serving_HSDSCH_CellInformation_r11", HFILL }},
    { &hf_rrc_e_dch_ReconfigurationInfo_02,
      { "e-dch-ReconfigurationInfo", "rrc.e_dch_ReconfigurationInfo",
        FT_NONE, BASE_NONE, NULL, 0,
        "E_DCH_ReconfigurationInfo_r11", HFILL }},
    { &hf_rrc_ul_64QAM_Config,
      { "ul-64QAM-Config", "rrc.ul_64QAM_Config",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_ul_MIMO_InfoFDD,
      { "ul-MIMO-InfoFDD", "rrc.ul_MIMO_InfoFDD",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_dl_SecondaryCellInfoFDD_03,
      { "dl-SecondaryCellInfoFDD", "rrc.dl_SecondaryCellInfoFDD",
        FT_NONE, BASE_NONE, NULL, 0,
        "DL_SecondaryCellInfoFDD_r11", HFILL }},
    { &hf_rrc_additionalDLSecCellInfoListFDD_01,
      { "additionalDLSecCellInfoListFDD", "rrc.additionalDLSecCellInfoListFDD",
        FT_UINT32, BASE_DEC, NULL, 0,
        "AdditionalDLSecCellInfoListFDD_r11", HFILL }},
    { &hf_rrc_additionalDLSecCellInfoListFDD2,
      { "additionalDLSecCellInfoListFDD2", "rrc.additionalDLSecCellInfoListFDD2",
        FT_UINT32, BASE_DEC, NULL, 0,
        "AdditionalDLSecCellInfoListFDD2_r11", HFILL }},
    { &hf_rrc_dummy_03,
      { "dummy", "rrc.dummy",
        FT_NONE, BASE_NONE, NULL, 0,
        "IntegrityProtActivationInfo", HFILL }},
    { &hf_rrc_dummy2_01,
      { "dummy2", "rrc.dummy2",
        FT_UINT32, BASE_DEC, NULL, 0,
        "RB_ActivationTimeInfoList", HFILL }},
    { &hf_rrc_dummy3_01,
      { "dummy3", "rrc.dummy3",
        FT_NONE, BASE_NONE, NULL, 0,
        "UL_CounterSynchronisationInfo", HFILL }},
    { &hf_rrc_laterNonCriticalExtensions_01,
      { "laterNonCriticalExtensions", "rrc.laterNonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_laterNonCriticalExtensions_01", HFILL }},
    { &hf_rrc_activeSetUpdateComplete_r3_add_ext,
      { "activeSetUpdateComplete-r3-add-ext", "rrc.activeSetUpdateComplete_r3_add_ext",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING", HFILL }},
    { &hf_rrc_nonCriticalExtensions_07,
      { "nonCriticalExtensions", "rrc.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_nonCriticalExtensions_07", HFILL }},
    { &hf_rrc_failureCause,
      { "failureCause", "rrc.failureCause",
        FT_UINT32, BASE_DEC, VALS(rrc_FailureCauseWithProtErr_vals), 0,
        "FailureCauseWithProtErr", HFILL }},
    { &hf_rrc_laterNonCriticalExtensions_02,
      { "laterNonCriticalExtensions", "rrc.laterNonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_laterNonCriticalExtensions_02", HFILL }},
    { &hf_rrc_activeSetUpdateFailure_r3_add_ext,
      { "activeSetUpdateFailure-r3-add-ext", "rrc.activeSetUpdateFailure_r3_add_ext",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING", HFILL }},
    { &hf_rrc_nonCriticalExtensions_08,
      { "nonCriticalExtensions", "rrc.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_nonCriticalExtensions_08", HFILL }},
    { &hf_rrc_r3_01,
      { "r3", "rrc.r3",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_r3_01", HFILL }},
    { &hf_rrc_assistanceDataDelivery_r3,
      { "assistanceDataDelivery-r3", "rrc.assistanceDataDelivery_r3",
        FT_NONE, BASE_NONE, NULL, 0,
        "AssistanceDataDelivery_r3_IEs", HFILL }},
    { &hf_rrc_v3a0NonCriticalExtensions,
      { "v3a0NonCriticalExtensions", "rrc.v3a0NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_assistanceDataDelivery_v3a0ext,
      { "assistanceDataDelivery-v3a0ext", "rrc.assistanceDataDelivery_v3a0ext",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_laterNonCriticalExtensions_03,
      { "laterNonCriticalExtensions", "rrc.laterNonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_laterNonCriticalExtensions_03", HFILL }},
    { &hf_rrc_assistanceDataDelivery_r3_add_ext,
      { "assistanceDataDelivery-r3-add-ext", "rrc.assistanceDataDelivery_r3_add_ext",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING", HFILL }},
    { &hf_rrc_v4b0NonCriticalExtensions_01,
      { "v4b0NonCriticalExtensions", "rrc.v4b0NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v4b0NonCriticalExtensions_01", HFILL }},
    { &hf_rrc_assistanceDataDelivery_v4b0ext,
      { "assistanceDataDelivery-v4b0ext", "rrc.assistanceDataDelivery_v4b0ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "AssistanceDataDelivery_v4b0ext_IEs", HFILL }},
    { &hf_rrc_v770NonCriticalExtension,
      { "v770NonCriticalExtension", "rrc.v770NonCriticalExtension",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_assistanceDataDelivery_v770ext,
      { "assistanceDataDelivery-v770ext", "rrc.assistanceDataDelivery_v770ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "AssistanceDataDelivery_v770ext_IEs", HFILL }},
    { &hf_rrc_v860NonCriticalExtension,
      { "v860NonCriticalExtension", "rrc.v860NonCriticalExtension",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_assistanceDataDelivery_v860ext,
      { "assistanceDataDelivery-v860ext", "rrc.assistanceDataDelivery_v860ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "AssistanceDataDelivery_v860ext_IEs", HFILL }},
    { &hf_rrc_v920NonCriticalExtension,
      { "v920NonCriticalExtension", "rrc.v920NonCriticalExtension",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_assistanceDataDelivery_v920ext,
      { "assistanceDataDelivery-v920ext", "rrc.assistanceDataDelivery_v920ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "AssistanceDataDelivery_v920ext_IEs", HFILL }},
    { &hf_rrc_va40NonCriticalExtension,
      { "va40NonCriticalExtension", "rrc.va40NonCriticalExtension",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_assistanceDataDelivery_va40ext,
      { "assistanceDataDelivery-va40ext", "rrc.assistanceDataDelivery_va40ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "AssistanceDataDelivery_va40ext_IEs", HFILL }},
    { &hf_rrc_nonCriticalExtensions_09,
      { "nonCriticalExtensions", "rrc.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_nonCriticalExtensions_09", HFILL }},
    { &hf_rrc_later_than_r3_01,
      { "later-than-r3", "rrc.later_than_r3",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_later_than_r3_01", HFILL }},
    { &hf_rrc_criticalExtensions_07,
      { "criticalExtensions", "rrc.criticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_criticalExtensions_07", HFILL }},
    { &hf_rrc_ue_positioning_GPS_AssistanceData,
      { "ue-positioning-GPS-AssistanceData", "rrc.ue_positioning_GPS_AssistanceData",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_ue_positioning_OTDOA_AssistanceData_UEB,
      { "ue-positioning-OTDOA-AssistanceData-UEB", "rrc.ue_positioning_OTDOA_AssistanceData_UEB",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_sfn_Offset_Validity,
      { "sfn-Offset-Validity", "rrc.sfn_Offset_Validity",
        FT_UINT32, BASE_DEC, VALS(rrc_SFN_Offset_Validity_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_ue_Positioning_OTDOA_AssistanceData_r4ext,
      { "ue-Positioning-OTDOA-AssistanceData-r4ext", "rrc.ue_Positioning_OTDOA_AssistanceData_r4ext",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_ue_Positioning_OTDOA_AssistanceData_UEB_ext,
      { "ue-Positioning-OTDOA-AssistanceData-UEB-ext", "rrc.ue_Positioning_OTDOA_AssistanceData_UEB_ext",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_ue_Positioning_GPS_AssistanceData,
      { "ue-Positioning-GPS-AssistanceData", "rrc.ue_Positioning_GPS_AssistanceData",
        FT_NONE, BASE_NONE, NULL, 0,
        "UE_Positioning_GPS_AssistanceData_v770ext", HFILL }},
    { &hf_rrc_ue_positioning_GANSS_AssistanceData,
      { "ue-positioning-GANSS-AssistanceData", "rrc.ue_positioning_GANSS_AssistanceData",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_ue_positioning_GANSS_AssistanceData_v860ext,
      { "ue-positioning-GANSS-AssistanceData-v860ext", "rrc.ue_positioning_GANSS_AssistanceData_v860ext",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_ue_positioning_GPS_AssistanceData_v920ext,
      { "ue-positioning-GPS-AssistanceData-v920ext", "rrc.ue_positioning_GPS_AssistanceData_v920ext",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_ue_positioning_GANSS_AssistanceData_v920ext,
      { "ue-positioning-GANSS-AssistanceData-v920ext", "rrc.ue_positioning_GANSS_AssistanceData_v920ext",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_ue_positioning_GPS_AssistanceData_va40ext,
      { "ue-positioning-GPS-AssistanceData-va40ext", "rrc.ue_positioning_GPS_AssistanceData_va40ext",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_ue_positioning_GANSS_AssistanceData_va40ext,
      { "ue-positioning-GANSS-AssistanceData-va40ext", "rrc.ue_positioning_GANSS_AssistanceData_va40ext",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_r3_02,
      { "r3", "rrc.r3",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_r3_02", HFILL }},
    { &hf_rrc_cellChangeOrderFromUTRAN_IEs,
      { "cellChangeOrderFromUTRAN-IEs", "rrc.cellChangeOrderFromUTRAN_IEs",
        FT_NONE, BASE_NONE, NULL, 0,
        "CellChangeOrderFromUTRAN_r3_IEs", HFILL }},
    { &hf_rrc_laterNonCriticalExtensions_04,
      { "laterNonCriticalExtensions", "rrc.laterNonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_laterNonCriticalExtensions_04", HFILL }},
    { &hf_rrc_cellChangeOrderFromUTRAN_r3_add_ext,
      { "cellChangeOrderFromUTRAN-r3-add-ext", "rrc.cellChangeOrderFromUTRAN_r3_add_ext",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING", HFILL }},
    { &hf_rrc_v590NonCriticalExtensions_01,
      { "v590NonCriticalExtensions", "rrc.v590NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v590NonCriticalExtensions_01", HFILL }},
    { &hf_rrc_cellChangeOrderFromUTRAN_v590ext,
      { "cellChangeOrderFromUTRAN-v590ext", "rrc.cellChangeOrderFromUTRAN_v590ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "CellChangeOrderFromUTRAN_v590ext_IEs", HFILL }},
    { &hf_rrc_nonCriticalExtensions_10,
      { "nonCriticalExtensions", "rrc.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_nonCriticalExtensions_10", HFILL }},
    { &hf_rrc_later_than_r3_02,
      { "later-than-r3", "rrc.later_than_r3",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_later_than_r3_02", HFILL }},
    { &hf_rrc_criticalExtensions_08,
      { "criticalExtensions", "rrc.criticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_criticalExtensions_08", HFILL }},
    { &hf_rrc_rab_InformationList,
      { "rab-InformationList", "rrc.rab_InformationList",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_interRAT_TargetCellDescription,
      { "interRAT-TargetCellDescription", "rrc.interRAT_TargetCellDescription",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_geran_SystemInfoType,
      { "geran-SystemInfoType", "rrc.geran_SystemInfoType",
        FT_UINT32, BASE_DEC, VALS(rrc_T_geran_SystemInfoType_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_sI,
      { "sI", "rrc.sI",
        FT_UINT32, BASE_DEC, NULL, 0,
        "GERAN_SystemInformation", HFILL }},
    { &hf_rrc_pSI,
      { "pSI", "rrc.pSI",
        FT_UINT32, BASE_DEC, NULL, 0,
        "GERAN_SystemInformation", HFILL }},
    { &hf_rrc_r3_03,
      { "r3", "rrc.r3",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_r3_03", HFILL }},
    { &hf_rrc_cellChangeOrderFromUTRANFailure_r3,
      { "cellChangeOrderFromUTRANFailure-r3", "rrc.cellChangeOrderFromUTRANFailure_r3",
        FT_NONE, BASE_NONE, NULL, 0,
        "CellChangeOrderFromUTRANFailure_r3_IEs", HFILL }},
    { &hf_rrc_laterNonCriticalExtensions_05,
      { "laterNonCriticalExtensions", "rrc.laterNonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_laterNonCriticalExtensions_05", HFILL }},
    { &hf_rrc_cellChangeOrderFromUTRANFailure_r3_add_ext,
      { "cellChangeOrderFromUTRANFailure-r3-add-ext", "rrc.cellChangeOrderFromUTRANFailure_r3_add_ext",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING", HFILL }},
    { &hf_rrc_nonCriticalExtensions_11,
      { "nonCriticalExtensions", "rrc.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_nonCriticalExtensions_11", HFILL }},
    { &hf_rrc_dummy_04,
      { "dummy", "rrc.dummy",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_criticalExtensions_09,
      { "criticalExtensions", "rrc.criticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_criticalExtensions_09", HFILL }},
    { &hf_rrc_interRAT_ChangeFailureCause,
      { "interRAT-ChangeFailureCause", "rrc.interRAT_ChangeFailureCause",
        FT_UINT32, BASE_DEC, VALS(rrc_InterRAT_ChangeFailureCause_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_u_RNTI,
      { "u-RNTI", "rrc.u_RNTI",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_startList,
      { "startList", "rrc.startList",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_am_RLC_ErrorIndicationRb2_3or4,
      { "am-RLC-ErrorIndicationRb2-3or4", "rrc.am_RLC_ErrorIndicationRb2_3or4",
        FT_BOOLEAN, BASE_NONE, NULL, 0,
        "BOOLEAN", HFILL }},
    { &hf_rrc_am_RLC_ErrorIndicationRb5orAbove,
      { "am-RLC-ErrorIndicationRb5orAbove", "rrc.am_RLC_ErrorIndicationRb5orAbove",
        FT_BOOLEAN, BASE_NONE, NULL, 0,
        "BOOLEAN", HFILL }},
    { &hf_rrc_cellUpdateCause,
      { "cellUpdateCause", "rrc.cellUpdateCause",
        FT_UINT32, BASE_DEC, VALS(rrc_CellUpdateCause_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_failureCause_01,
      { "failureCause", "rrc.failureCause",
        FT_NONE, BASE_NONE, NULL, 0,
        "FailureCauseWithProtErrTrId", HFILL }},
    { &hf_rrc_rb_timer_indicator,
      { "rb-timer-indicator", "rrc.rb_timer_indicator",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_measuredResultsOnRACH,
      { "measuredResultsOnRACH", "rrc.measuredResultsOnRACH",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_laterNonCriticalExtensions_06,
      { "laterNonCriticalExtensions", "rrc.laterNonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_laterNonCriticalExtensions_06", HFILL }},
    { &hf_rrc_cellUpdate_r3_add_ext,
      { "cellUpdate-r3-add-ext", "rrc.cellUpdate_r3_add_ext",
        FT_BYTES, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_v590NonCriticalExtensions_02,
      { "v590NonCriticalExtensions", "rrc.v590NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v590NonCriticalExtensions_02", HFILL }},
    { &hf_rrc_cellUpdate_v590ext,
      { "cellUpdate-v590ext", "rrc.cellUpdate_v590ext",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_v690NonCriticalExtensions_01,
      { "v690NonCriticalExtensions", "rrc.v690NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v690NonCriticalExtensions_01", HFILL }},
    { &hf_rrc_cellUpdate_v690ext,
      { "cellUpdate-v690ext", "rrc.cellUpdate_v690ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "CellUpdate_v690ext_IEs", HFILL }},
    { &hf_rrc_v6b0NonCriticalExtensions_01,
      { "v6b0NonCriticalExtensions", "rrc.v6b0NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v6b0NonCriticalExtensions_01", HFILL }},
    { &hf_rrc_cellUpdate_v6b0ext,
      { "cellUpdate-v6b0ext", "rrc.cellUpdate_v6b0ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "CellUpdate_v6b0ext_IEs", HFILL }},
    { &hf_rrc_v770NonCriticalExtensions,
      { "v770NonCriticalExtensions", "rrc.v770NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_cellUpdate_v770ext,
      { "cellUpdate-v770ext", "rrc.cellUpdate_v770ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "CellUpdate_v770ext_IEs", HFILL }},
    { &hf_rrc_v860NonCriticalExtensions,
      { "v860NonCriticalExtensions", "rrc.v860NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_cellUpdate_v860ext,
      { "cellUpdate-v860ext", "rrc.cellUpdate_v860ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "CellUpdate_v860ext_IEs", HFILL }},
    { &hf_rrc_va40NonCriticalExtensions,
      { "va40NonCriticalExtensions", "rrc.va40NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_cellUpdate_va40ext,
      { "cellUpdate-va40ext", "rrc.cellUpdate_va40ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "CellUpdate_va40ext_IEs", HFILL }},
    { &hf_rrc_vbxyNonCriticalExtensions_01,
      { "vbxyNonCriticalExtensions", "rrc.vbxyNonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_vbxyNonCriticalExtensions_01", HFILL }},
    { &hf_rrc_cellUpdate_vbxyext,
      { "cellUpdate-vbxyext", "rrc.cellUpdate_vbxyext",
        FT_NONE, BASE_NONE, NULL, 0,
        "CellUpdate_vbxyext_IEs", HFILL }},
    { &hf_rrc_nonCriticalExtensions_12,
      { "nonCriticalExtensions", "rrc.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_nonCriticalExtensions_12", HFILL }},
    { &hf_rrc_cellUpdate_v7e0ext,
      { "cellUpdate-v7e0ext", "rrc.cellUpdate_v7e0ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "CellUpdate_v7e0ext_IEs", HFILL }},
    { &hf_rrc_v7g0NonCriticalExtensions_02,
      { "v7g0NonCriticalExtensions", "rrc.v7g0NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v7g0NonCriticalExtensions_02", HFILL }},
    { &hf_rrc_cellUpdate_v7g0ext,
      { "cellUpdate-v7g0ext", "rrc.cellUpdate_v7g0ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "CellUpdate_v7g0ext_IEs", HFILL }},
    { &hf_rrc_nonCriticalExtensions_13,
      { "nonCriticalExtensions", "rrc.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_nonCriticalExtensions_13", HFILL }},
    { &hf_rrc_establishmentCause,
      { "establishmentCause", "rrc.establishmentCause",
        FT_UINT32, BASE_DEC|BASE_EXT_STRING, &rrc_EstablishmentCause_vals_ext, 0,
        NULL, HFILL }},
    { &hf_rrc_cellUpdateCause_ext,
      { "cellUpdateCause-ext", "rrc.cellUpdateCause_ext",
        FT_UINT32, BASE_DEC, VALS(rrc_CellUpdateCause_ext_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_trafficVolumeIndicator,
      { "trafficVolumeIndicator", "rrc.trafficVolumeIndicator",
        FT_UINT32, BASE_DEC, VALS(rrc_T_trafficVolumeIndicator_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_measuredResultsOnRACHinterFreq,
      { "measuredResultsOnRACHinterFreq", "rrc.measuredResultsOnRACHinterFreq",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_reconfigurationStatusIndicator,
      { "reconfigurationStatusIndicator", "rrc.reconfigurationStatusIndicator",
        FT_UINT32, BASE_DEC, VALS(rrc_T_reconfigurationStatusIndicator_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_mbmsSelectedServices,
      { "mbmsSelectedServices", "rrc.mbmsSelectedServices",
        FT_NONE, BASE_NONE, NULL, 0,
        "MBMS_SelectedServicesShort", HFILL }},
    { &hf_rrc_csCallType,
      { "csCallType", "rrc.csCallType",
        FT_UINT32, BASE_DEC, VALS(rrc_T_csCallType_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_hspdschReception_CellFach,
      { "hspdschReception-CellFach", "rrc.hspdschReception_CellFach",
        FT_UINT32, BASE_DEC, VALS(rrc_T_hspdschReception_CellFach_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_ueMobilityStateIndicator,
      { "ueMobilityStateIndicator", "rrc.ueMobilityStateIndicator",
        FT_UINT32, BASE_DEC, VALS(rrc_High_MobilityDetected_vals), 0,
        "High_MobilityDetected", HFILL }},
    { &hf_rrc_capabilityChangeIndicator,
      { "capabilityChangeIndicator", "rrc.capabilityChangeIndicator",
        FT_UINT32, BASE_DEC, VALS(rrc_T_capabilityChangeIndicator_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_supportForTwoDRXSchemesInPCH,
      { "supportForTwoDRXSchemesInPCH", "rrc.supportForTwoDRXSchemesInPCH",
        FT_UINT32, BASE_DEC, VALS(rrc_T_supportForTwoDRXSchemesInPCH_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_measuredResultsOnRACH_v7g0ext,
      { "measuredResultsOnRACH-v7g0ext", "rrc.measuredResultsOnRACH_v7g0ext",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_supportOfCommonEDCH,
      { "supportOfCommonEDCH", "rrc.supportOfCommonEDCH",
        FT_UINT32, BASE_DEC, VALS(rrc_T_supportOfCommonEDCH_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_supportOfHS_DSCHDRXOperation,
      { "supportOfHS-DSCHDRXOperation", "rrc.supportOfHS_DSCHDRXOperation",
        FT_UINT32, BASE_DEC, VALS(rrc_T_supportOfHS_DSCHDRXOperation_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_supportOfMACiis,
      { "supportOfMACiis", "rrc.supportOfMACiis",
        FT_UINT32, BASE_DEC, VALS(rrc_T_supportOfMACiis_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_supportOfSPSOperation,
      { "supportOfSPSOperation", "rrc.supportOfSPSOperation",
        FT_UINT32, BASE_DEC, VALS(rrc_T_supportOfSPSOperation_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_supportOfControlChannelDRXOperation,
      { "supportOfControlChannelDRXOperation", "rrc.supportOfControlChannelDRXOperation",
        FT_UINT32, BASE_DEC, VALS(rrc_T_supportOfControlChannelDRXOperation_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_securityRevertStatusIndicator,
      { "securityRevertStatusIndicator", "rrc.securityRevertStatusIndicator",
        FT_UINT32, BASE_DEC, VALS(rrc_T_securityRevertStatusIndicator_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_loggedMeasAvailable,
      { "loggedMeasAvailable", "rrc.loggedMeasAvailable",
        FT_UINT32, BASE_DEC, VALS(rrc_T_loggedMeasAvailable_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_loggedANRResultsAvailable,
      { "loggedANRResultsAvailable", "rrc.loggedANRResultsAvailable",
        FT_UINT32, BASE_DEC, VALS(rrc_T_loggedANRResultsAvailable_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_measuredResultsOnRACHeutraFreq,
      { "measuredResultsOnRACHeutraFreq", "rrc.measuredResultsOnRACHeutraFreq",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_r3_04,
      { "r3", "rrc.r3",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_r3_04", HFILL }},
    { &hf_rrc_cellUpdateConfirm_r3,
      { "cellUpdateConfirm-r3", "rrc.cellUpdateConfirm_r3",
        FT_NONE, BASE_NONE, NULL, 0,
        "CellUpdateConfirm_r3_IEs", HFILL }},
    { &hf_rrc_v3a0NonCriticalExtensions_01,
      { "v3a0NonCriticalExtensions", "rrc.v3a0NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v3a0NonCriticalExtensions_01", HFILL }},
    { &hf_rrc_cellUpdateConfirm_v3a0ext,
      { "cellUpdateConfirm-v3a0ext", "rrc.cellUpdateConfirm_v3a0ext",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_laterNonCriticalExtensions_07,
      { "laterNonCriticalExtensions", "rrc.laterNonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_laterNonCriticalExtensions_07", HFILL }},
    { &hf_rrc_cellUpdateConfirm_r3_add_ext,
      { "cellUpdateConfirm-r3-add-ext", "rrc.cellUpdateConfirm_r3_add_ext",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING", HFILL }},
    { &hf_rrc_v4b0NonCriticalExtensions_02,
      { "v4b0NonCriticalExtensions", "rrc.v4b0NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v4b0NonCriticalExtensions_02", HFILL }},
    { &hf_rrc_cellUpdateConfirm_v4b0ext,
      { "cellUpdateConfirm-v4b0ext", "rrc.cellUpdateConfirm_v4b0ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "CellUpdateConfirm_v4b0ext_IEs", HFILL }},
    { &hf_rrc_v590NonCriticalExtenstions,
      { "v590NonCriticalExtenstions", "rrc.v590NonCriticalExtenstions",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_cellUpdateConfirm_v590ext,
      { "cellUpdateConfirm-v590ext", "rrc.cellUpdateConfirm_v590ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "CellUpdateConfirm_v590ext_IEs", HFILL }},
    { &hf_rrc_v5d0NonCriticalExtenstions,
      { "v5d0NonCriticalExtenstions", "rrc.v5d0NonCriticalExtenstions",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_cellUpdateConfirm_v5d0ext,
      { "cellUpdateConfirm-v5d0ext", "rrc.cellUpdateConfirm_v5d0ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "CellUpdateConfirm_v5d0ext_IEs", HFILL }},
    { &hf_rrc_v690NonCriticalExtensions_02,
      { "v690NonCriticalExtensions", "rrc.v690NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v690NonCriticalExtensions_02", HFILL }},
    { &hf_rrc_cellUpdateConfirm_v690ext,
      { "cellUpdateConfirm-v690ext", "rrc.cellUpdateConfirm_v690ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "CellUpdateConfirm_v690ext_IEs", HFILL }},
    { &hf_rrc_nonCriticalExtensions_14,
      { "nonCriticalExtensions", "rrc.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_nonCriticalExtensions_14", HFILL }},
    { &hf_rrc_later_than_r3_03,
      { "later-than-r3", "rrc.later_than_r3",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_later_than_r3_03", HFILL }},
    { &hf_rrc_criticalExtensions_10,
      { "criticalExtensions", "rrc.criticalExtensions",
        FT_UINT32, BASE_DEC, VALS(rrc_T_criticalExtensions_10_vals), 0,
        "T_criticalExtensions_10", HFILL }},
    { &hf_rrc_r4,
      { "r4", "rrc.r4",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_cellUpdateConfirm_r4,
      { "cellUpdateConfirm-r4", "rrc.cellUpdateConfirm_r4",
        FT_NONE, BASE_NONE, NULL, 0,
        "CellUpdateConfirm_r4_IEs", HFILL }},
    { &hf_rrc_v4d0NonCriticalExtensions,
      { "v4d0NonCriticalExtensions", "rrc.v4d0NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_cellUpdateConfirm_r4_add_ext,
      { "cellUpdateConfirm-r4-add-ext", "rrc.cellUpdateConfirm_r4_add_ext",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING", HFILL }},
    { &hf_rrc_v590NonCriticalExtenstions_01,
      { "v590NonCriticalExtenstions", "rrc.v590NonCriticalExtenstions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v590NonCriticalExtenstions_01", HFILL }},
    { &hf_rrc_v5d0NonCriticalExtenstions_01,
      { "v5d0NonCriticalExtenstions", "rrc.v5d0NonCriticalExtenstions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v5d0NonCriticalExtenstions_01", HFILL }},
    { &hf_rrc_v690NonCriticalExtensions_03,
      { "v690NonCriticalExtensions", "rrc.v690NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v690NonCriticalExtensions_03", HFILL }},
    { &hf_rrc_nonCriticalExtensions_15,
      { "nonCriticalExtensions", "rrc.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_nonCriticalExtensions_15", HFILL }},
    { &hf_rrc_criticalExtensions_11,
      { "criticalExtensions", "rrc.criticalExtensions",
        FT_UINT32, BASE_DEC, VALS(rrc_T_criticalExtensions_11_vals), 0,
        "T_criticalExtensions_11", HFILL }},
    { &hf_rrc_r5,
      { "r5", "rrc.r5",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_cellUpdateConfirm_r5,
      { "cellUpdateConfirm-r5", "rrc.cellUpdateConfirm_r5",
        FT_NONE, BASE_NONE, NULL, 0,
        "CellUpdateConfirm_r5_IEs", HFILL }},
    { &hf_rrc_cellUpdateConfirm_r5_add_ext,
      { "cellUpdateConfirm-r5-add-ext", "rrc.cellUpdateConfirm_r5_add_ext",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING", HFILL }},
    { &hf_rrc_v5d0NonCriticalExtenstions_02,
      { "v5d0NonCriticalExtenstions", "rrc.v5d0NonCriticalExtenstions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v5d0NonCriticalExtenstions_02", HFILL }},
    { &hf_rrc_v690NonCriticalExtensions_04,
      { "v690NonCriticalExtensions", "rrc.v690NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v690NonCriticalExtensions_04", HFILL }},
    { &hf_rrc_nonCriticalExtensions_16,
      { "nonCriticalExtensions", "rrc.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_nonCriticalExtensions_16", HFILL }},
    { &hf_rrc_criticalExtensions_12,
      { "criticalExtensions", "rrc.criticalExtensions",
        FT_UINT32, BASE_DEC, VALS(rrc_T_criticalExtensions_12_vals), 0,
        "T_criticalExtensions_12", HFILL }},
    { &hf_rrc_r6_01,
      { "r6", "rrc.r6",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_r6_01", HFILL }},
    { &hf_rrc_cellUpdateConfirm_r6,
      { "cellUpdateConfirm-r6", "rrc.cellUpdateConfirm_r6",
        FT_NONE, BASE_NONE, NULL, 0,
        "CellUpdateConfirm_r6_IEs", HFILL }},
    { &hf_rrc_cellUpdateConfirm_r6_add_ext,
      { "cellUpdateConfirm-r6-add-ext", "rrc.cellUpdateConfirm_r6_add_ext",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING", HFILL }},
    { &hf_rrc_v6b0NonCriticalExtensions_02,
      { "v6b0NonCriticalExtensions", "rrc.v6b0NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v6b0NonCriticalExtensions_02", HFILL }},
    { &hf_rrc_cellUpdateConfirm_v6b0ext,
      { "cellUpdateConfirm-v6b0ext", "rrc.cellUpdateConfirm_v6b0ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "CellUpdateConfirm_v6b0ext_IEs", HFILL }},
    { &hf_rrc_nonCriticalExtensions_17,
      { "nonCriticalExtensions", "rrc.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_nonCriticalExtensions_17", HFILL }},
    { &hf_rrc_criticalExtensions_13,
      { "criticalExtensions", "rrc.criticalExtensions",
        FT_UINT32, BASE_DEC, VALS(rrc_T_criticalExtensions_13_vals), 0,
        "T_criticalExtensions_13", HFILL }},
    { &hf_rrc_r7_01,
      { "r7", "rrc.r7",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_r7_01", HFILL }},
    { &hf_rrc_cellUpdateConfirm_r7,
      { "cellUpdateConfirm-r7", "rrc.cellUpdateConfirm_r7",
        FT_NONE, BASE_NONE, NULL, 0,
        "CellUpdateConfirm_r7_IEs", HFILL }},
    { &hf_rrc_cellUpdateConfirm_r7_add_ext,
      { "cellUpdateConfirm-r7-add-ext", "rrc.cellUpdateConfirm_r7_add_ext",
        FT_BYTES, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_v780NonCriticalExtensions_01,
      { "v780NonCriticalExtensions", "rrc.v780NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v780NonCriticalExtensions_01", HFILL }},
    { &hf_rrc_cellUpdateConfirm_v780ext,
      { "cellUpdateConfirm-v780ext", "rrc.cellUpdateConfirm_v780ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "CellUpdateConfirm_v780ext_IEs", HFILL }},
    { &hf_rrc_v860NonCriticalExtensions_01,
      { "v860NonCriticalExtensions", "rrc.v860NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v860NonCriticalExtensions_01", HFILL }},
    { &hf_rrc_cellUpdateConfirm_v860ext,
      { "cellUpdateConfirm-v860ext", "rrc.cellUpdateConfirm_v860ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "CellUpdateConfirm_v860ext_IEs", HFILL }},
    { &hf_rrc_nonCriticalExtensions_18,
      { "nonCriticalExtensions", "rrc.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_nonCriticalExtensions_18", HFILL }},
    { &hf_rrc_criticalExtensions_14,
      { "criticalExtensions", "rrc.criticalExtensions",
        FT_UINT32, BASE_DEC, VALS(rrc_T_criticalExtensions_14_vals), 0,
        "T_criticalExtensions_14", HFILL }},
    { &hf_rrc_r8_01,
      { "r8", "rrc.r8",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_r8_01", HFILL }},
    { &hf_rrc_cellUpdateConfirm_r8,
      { "cellUpdateConfirm-r8", "rrc.cellUpdateConfirm_r8",
        FT_NONE, BASE_NONE, NULL, 0,
        "CellUpdateConfirm_r8_IEs", HFILL }},
    { &hf_rrc_cellUpdateConfirm_r8_add_ext,
      { "cellUpdateConfirm-r8-add-ext", "rrc.cellUpdateConfirm_r8_add_ext",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING", HFILL }},
    { &hf_rrc_v7d0NonCriticalExtensions,
      { "v7d0NonCriticalExtensions", "rrc.v7d0NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_cellUpdateConfirm_v7d0ext,
      { "cellUpdateConfirm-v7d0ext", "rrc.cellUpdateConfirm_v7d0ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "CellUpdateConfirm_v7d0ext_IEs", HFILL }},
    { &hf_rrc_v7f0NonCriticalExtensions_01,
      { "v7f0NonCriticalExtensions", "rrc.v7f0NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v7f0NonCriticalExtensions_01", HFILL }},
    { &hf_rrc_cellUpdateConfirm_v7f0ext,
      { "cellUpdateConfirm-v7f0ext", "rrc.cellUpdateConfirm_v7f0ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "CellUpdateConfirm_v7f0ext_IEs", HFILL }},
    { &hf_rrc_v890NonCriticalExtensions_01,
      { "v890NonCriticalExtensions", "rrc.v890NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v890NonCriticalExtensions_01", HFILL }},
    { &hf_rrc_cellUpdateConfirm_v890ext,
      { "cellUpdateConfirm-v890ext", "rrc.cellUpdateConfirm_v890ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "CellUpdateConfirm_v890ext_IEs", HFILL }},
    { &hf_rrc_v7g0NonCriticalExtensions_03,
      { "v7g0NonCriticalExtensions", "rrc.v7g0NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v7g0NonCriticalExtensions_03", HFILL }},
    { &hf_rrc_cellUpdateConfirm_v7g0ext,
      { "cellUpdateConfirm-v7g0ext", "rrc.cellUpdateConfirm_v7g0ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "CellUpdateConfirm_v7g0ext_IEs", HFILL }},
    { &hf_rrc_v8a0NonCriticalExtensions,
      { "v8a0NonCriticalExtensions", "rrc.v8a0NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_cellUpdateConfirm_v8a0ext,
      { "cellUpdateConfirm-v8a0ext", "rrc.cellUpdateConfirm_v8a0ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "CellUpdateConfirm_v8a0ext_IEs", HFILL }},
    { &hf_rrc_nonCriticalExtensions_19,
      { "nonCriticalExtensions", "rrc.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_nonCriticalExtensions_19", HFILL }},
    { &hf_rrc_criticalExtensions_15,
      { "criticalExtensions", "rrc.criticalExtensions",
        FT_UINT32, BASE_DEC, VALS(rrc_T_criticalExtensions_15_vals), 0,
        "T_criticalExtensions_15", HFILL }},
    { &hf_rrc_r9_01,
      { "r9", "rrc.r9",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_r9_01", HFILL }},
    { &hf_rrc_cellUpdateConfirm_r9,
      { "cellUpdateConfirm-r9", "rrc.cellUpdateConfirm_r9",
        FT_NONE, BASE_NONE, NULL, 0,
        "CellUpdateConfirm_r9_IEs", HFILL }},
    { &hf_rrc_cellUpdateConfirm_r9_add_ext,
      { "cellUpdateConfirm-r9-add-ext", "rrc.cellUpdateConfirm_r9_add_ext",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING", HFILL }},
    { &hf_rrc_v950NonCriticalExtensions_01,
      { "v950NonCriticalExtensions", "rrc.v950NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v950NonCriticalExtensions_01", HFILL }},
    { &hf_rrc_cellUpdateConfirm_v950ext,
      { "cellUpdateConfirm-v950ext", "rrc.cellUpdateConfirm_v950ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "CellUpdateConfirm_v950ext_IEs", HFILL }},
    { &hf_rrc_v9c0NonCriticalExtensions,
      { "v9c0NonCriticalExtensions", "rrc.v9c0NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_cellUpdateConfirm_v9c0ext,
      { "cellUpdateConfirm-v9c0ext", "rrc.cellUpdateConfirm_v9c0ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "CellUpdateConfirm_v9c0ext_IEs", HFILL }},
    { &hf_rrc_nonCriticalExtensions_20,
      { "nonCriticalExtensions", "rrc.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_nonCriticalExtensions_20", HFILL }},
    { &hf_rrc_criticalExtensions_16,
      { "criticalExtensions", "rrc.criticalExtensions",
        FT_UINT32, BASE_DEC, VALS(rrc_T_criticalExtensions_16_vals), 0,
        "T_criticalExtensions_16", HFILL }},
    { &hf_rrc_r10_01,
      { "r10", "rrc.r10",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_r10_01", HFILL }},
    { &hf_rrc_cellUpdateConfirm_r10,
      { "cellUpdateConfirm-r10", "rrc.cellUpdateConfirm_r10",
        FT_NONE, BASE_NONE, NULL, 0,
        "CellUpdateConfirm_r10_IEs", HFILL }},
    { &hf_rrc_cellUpdateConfirm_r10_add_ext,
      { "cellUpdateConfirm-r10-add-ext", "rrc.cellUpdateConfirm_r10_add_ext",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING", HFILL }},
    { &hf_rrc_v9c0NonCriticalExtensions_01,
      { "v9c0NonCriticalExtensions", "rrc.v9c0NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v9c0NonCriticalExtensions_01", HFILL }},
    { &hf_rrc_vbxyNonCriticalExtensions_02,
      { "vbxyNonCriticalExtensions", "rrc.vbxyNonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_vbxyNonCriticalExtensions_02", HFILL }},
    { &hf_rrc_cellUpdateConfirm_vbxyext,
      { "cellUpdateConfirm-vbxyext", "rrc.cellUpdateConfirm_vbxyext",
        FT_NONE, BASE_NONE, NULL, 0,
        "CellUpdateConfirm_vbxyext_IEs", HFILL }},
    { &hf_rrc_nonCriticalExtensions_21,
      { "nonCriticalExtensions", "rrc.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_nonCriticalExtensions_21", HFILL }},
    { &hf_rrc_criticalExtensions_17,
      { "criticalExtensions", "rrc.criticalExtensions",
        FT_UINT32, BASE_DEC, VALS(rrc_T_criticalExtensions_17_vals), 0,
        "T_criticalExtensions_17", HFILL }},
    { &hf_rrc_r11_01,
      { "r11", "rrc.r11",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_r11_01", HFILL }},
    { &hf_rrc_cellUpdateConfirm_r11,
      { "cellUpdateConfirm-r11", "rrc.cellUpdateConfirm_r11",
        FT_NONE, BASE_NONE, NULL, 0,
        "CellUpdateConfirm_r11_IEs", HFILL }},
    { &hf_rrc_cellUpdateConfirm_r11_add_ext,
      { "cellUpdateConfirm-r11-add-ext", "rrc.cellUpdateConfirm_r11_add_ext",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING", HFILL }},
    { &hf_rrc_nonCriticalExtensions_22,
      { "nonCriticalExtensions", "rrc.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_nonCriticalExtensions_22", HFILL }},
    { &hf_rrc_criticalExtensions_18,
      { "criticalExtensions", "rrc.criticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_criticalExtensions_18", HFILL }},
    { &hf_rrc_integrityProtectionModeInfo,
      { "integrityProtectionModeInfo", "rrc.integrityProtectionModeInfo",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_cipheringModeInfo,
      { "cipheringModeInfo", "rrc.cipheringModeInfo",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_new_U_RNTI,
      { "new-U-RNTI", "rrc.new_U_RNTI",
        FT_NONE, BASE_NONE, NULL, 0,
        "U_RNTI", HFILL }},
    { &hf_rrc_new_C_RNTI,
      { "new-C-RNTI", "rrc.new_C_RNTI",
        FT_BYTES, BASE_NONE, NULL, 0,
        "C_RNTI", HFILL }},
    { &hf_rrc_rrc_StateIndicator,
      { "rrc-StateIndicator", "rrc.rrc_StateIndicator",
        FT_UINT32, BASE_DEC, VALS(rrc_RRC_StateIndicator_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_utran_DRX_CycleLengthCoeff,
      { "utran-DRX-CycleLengthCoeff", "rrc.utran_DRX_CycleLengthCoeff",
        FT_UINT32, BASE_DEC, NULL, 0,
        "UTRAN_DRX_CycleLengthCoefficient", HFILL }},
    { &hf_rrc_rlc_Re_establishIndicatorRb2_3or4,
      { "rlc-Re-establishIndicatorRb2-3or4", "rrc.rlc_Re_establishIndicatorRb2_3or4",
        FT_BOOLEAN, BASE_NONE, NULL, 0,
        "BOOLEAN", HFILL }},
    { &hf_rrc_rlc_Re_establishIndicatorRb5orAbove,
      { "rlc-Re-establishIndicatorRb5orAbove", "rrc.rlc_Re_establishIndicatorRb5orAbove",
        FT_BOOLEAN, BASE_NONE, NULL, 0,
        "BOOLEAN", HFILL }},
    { &hf_rrc_ura_Identity,
      { "ura-Identity", "rrc.ura_Identity",
        FT_BYTES, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_rb_InformationReleaseList,
      { "rb-InformationReleaseList", "rrc.rb_InformationReleaseList",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_rb_InformationReconfigList,
      { "rb-InformationReconfigList", "rrc.rb_InformationReconfigList",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_rb_InformationAffectedList,
      { "rb-InformationAffectedList", "rrc.rb_InformationAffectedList",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_dl_CounterSynchronisationInfo,
      { "dl-CounterSynchronisationInfo", "rrc.dl_CounterSynchronisationInfo",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_ul_CommonTransChInfo,
      { "ul-CommonTransChInfo", "rrc.ul_CommonTransChInfo",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_ul_deletedTransChInfoList,
      { "ul-deletedTransChInfoList", "rrc.ul_deletedTransChInfoList",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_ul_AddReconfTransChInfoList,
      { "ul-AddReconfTransChInfoList", "rrc.ul_AddReconfTransChInfoList",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_modeSpecificTransChInfo,
      { "modeSpecificTransChInfo", "rrc.modeSpecificTransChInfo",
        FT_UINT32, BASE_DEC, VALS(rrc_T_modeSpecificTransChInfo_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_fdd,
      { "fdd", "rrc.fdd",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_dummy_05,
      { "dummy", "rrc.dummy",
        FT_UINT32, BASE_DEC, NULL, 0,
        "CPCH_SetID", HFILL }},
    { &hf_rrc_dummy2_02,
      { "dummy2", "rrc.dummy2",
        FT_UINT32, BASE_DEC, NULL, 0,
        "DRAC_StaticInformationList", HFILL }},
    { &hf_rrc_tdd,
      { "tdd", "rrc.tdd",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_dl_CommonTransChInfo,
      { "dl-CommonTransChInfo", "rrc.dl_CommonTransChInfo",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_dl_DeletedTransChInfoList,
      { "dl-DeletedTransChInfoList", "rrc.dl_DeletedTransChInfoList",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_dl_AddReconfTransChInfoList,
      { "dl-AddReconfTransChInfoList", "rrc.dl_AddReconfTransChInfoList",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_frequencyInfo,
      { "frequencyInfo", "rrc.frequencyInfo",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_ul_ChannelRequirement,
      { "ul-ChannelRequirement", "rrc.ul_ChannelRequirement",
        FT_UINT32, BASE_DEC, VALS(rrc_UL_ChannelRequirement_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_modeSpecificPhysChInfo,
      { "modeSpecificPhysChInfo", "rrc.modeSpecificPhysChInfo",
        FT_UINT32, BASE_DEC, VALS(rrc_T_modeSpecificPhysChInfo_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_fdd_01,
      { "fdd", "rrc.fdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_fdd_01", HFILL }},
    { &hf_rrc_dummy_06,
      { "dummy", "rrc.dummy",
        FT_NONE, BASE_NONE, NULL, 0,
        "DL_PDSCH_Information", HFILL }},
    { &hf_rrc_dl_CommonInformation,
      { "dl-CommonInformation", "rrc.dl_CommonInformation",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_dl_InformationPerRL_List,
      { "dl-InformationPerRL-List", "rrc.dl_InformationPerRL_List",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_new_DSCH_RNTI,
      { "new-DSCH-RNTI", "rrc.new_DSCH_RNTI",
        FT_BYTES, BASE_NONE, NULL, 0,
        "DSCH_RNTI", HFILL }},
    { &hf_rrc_pdcp_ROHC_TargetMode,
      { "pdcp-ROHC-TargetMode", "rrc.pdcp_ROHC_TargetMode",
        FT_UINT32, BASE_DEC, VALS(rrc_PDCP_ROHC_TargetMode_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_rb_InformationReconfigList_01,
      { "rb-InformationReconfigList", "rrc.rb_InformationReconfigList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "RB_InformationReconfigList_r4", HFILL }},
    { &hf_rrc_ul_CommonTransChInfo_01,
      { "ul-CommonTransChInfo", "rrc.ul_CommonTransChInfo",
        FT_NONE, BASE_NONE, NULL, 0,
        "UL_CommonTransChInfo_r4", HFILL }},
    { &hf_rrc_modeSpecificTransChInfo_01,
      { "modeSpecificTransChInfo", "rrc.modeSpecificTransChInfo",
        FT_UINT32, BASE_DEC, VALS(rrc_T_modeSpecificTransChInfo_01_vals), 0,
        "T_modeSpecificTransChInfo_01", HFILL }},
    { &hf_rrc_fdd_02,
      { "fdd", "rrc.fdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_fdd_02", HFILL }},
    { &hf_rrc_dl_CommonTransChInfo_01,
      { "dl-CommonTransChInfo", "rrc.dl_CommonTransChInfo",
        FT_NONE, BASE_NONE, NULL, 0,
        "DL_CommonTransChInfo_r4", HFILL }},
    { &hf_rrc_dl_AddReconfTransChInfoList_01,
      { "dl-AddReconfTransChInfoList", "rrc.dl_AddReconfTransChInfoList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "DL_AddReconfTransChInfoList_r4", HFILL }},
    { &hf_rrc_ul_ChannelRequirement_01,
      { "ul-ChannelRequirement", "rrc.ul_ChannelRequirement",
        FT_UINT32, BASE_DEC, VALS(rrc_UL_ChannelRequirement_r4_vals), 0,
        "UL_ChannelRequirement_r4", HFILL }},
    { &hf_rrc_modeSpecificPhysChInfo_01,
      { "modeSpecificPhysChInfo", "rrc.modeSpecificPhysChInfo",
        FT_UINT32, BASE_DEC, VALS(rrc_T_modeSpecificPhysChInfo_01_vals), 0,
        "T_modeSpecificPhysChInfo_01", HFILL }},
    { &hf_rrc_fdd_03,
      { "fdd", "rrc.fdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_fdd_03", HFILL }},
    { &hf_rrc_dl_CommonInformation_01,
      { "dl-CommonInformation", "rrc.dl_CommonInformation",
        FT_NONE, BASE_NONE, NULL, 0,
        "DL_CommonInformation_r4", HFILL }},
    { &hf_rrc_dl_InformationPerRL_List_01,
      { "dl-InformationPerRL-List", "rrc.dl_InformationPerRL_List",
        FT_UINT32, BASE_DEC, NULL, 0,
        "DL_InformationPerRL_List_r4", HFILL }},
    { &hf_rrc_new_H_RNTI,
      { "new-H-RNTI", "rrc.new_H_RNTI",
        FT_BYTES, BASE_NONE, NULL, 0,
        "H_RNTI", HFILL }},
    { &hf_rrc_rb_InformationReconfigList_02,
      { "rb-InformationReconfigList", "rrc.rb_InformationReconfigList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "RB_InformationReconfigList_r5", HFILL }},
    { &hf_rrc_rb_InformationAffectedList_01,
      { "rb-InformationAffectedList", "rrc.rb_InformationAffectedList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "RB_InformationAffectedList_r5", HFILL }},
    { &hf_rrc_dl_CounterSynchronisationInfo_01,
      { "dl-CounterSynchronisationInfo", "rrc.dl_CounterSynchronisationInfo",
        FT_NONE, BASE_NONE, NULL, 0,
        "DL_CounterSynchronisationInfo_r5", HFILL }},
    { &hf_rrc_modeSpecificTransChInfo_02,
      { "modeSpecificTransChInfo", "rrc.modeSpecificTransChInfo",
        FT_UINT32, BASE_DEC, VALS(rrc_T_modeSpecificTransChInfo_02_vals), 0,
        "T_modeSpecificTransChInfo_02", HFILL }},
    { &hf_rrc_fdd_04,
      { "fdd", "rrc.fdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_fdd_04", HFILL }},
    { &hf_rrc_dl_DeletedTransChInfoList_01,
      { "dl-DeletedTransChInfoList", "rrc.dl_DeletedTransChInfoList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "DL_DeletedTransChInfoList_r5", HFILL }},
    { &hf_rrc_dl_AddReconfTransChInfoList_02,
      { "dl-AddReconfTransChInfoList", "rrc.dl_AddReconfTransChInfoList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "DL_AddReconfTransChInfoList_r5", HFILL }},
    { &hf_rrc_ul_ChannelRequirement_02,
      { "ul-ChannelRequirement", "rrc.ul_ChannelRequirement",
        FT_UINT32, BASE_DEC, VALS(rrc_UL_ChannelRequirement_r5_vals), 0,
        "UL_ChannelRequirement_r5", HFILL }},
    { &hf_rrc_modeSpecificPhysChInfo_02,
      { "modeSpecificPhysChInfo", "rrc.modeSpecificPhysChInfo",
        FT_UINT32, BASE_DEC, VALS(rrc_T_modeSpecificPhysChInfo_02_vals), 0,
        "T_modeSpecificPhysChInfo_02", HFILL }},
    { &hf_rrc_fdd_05,
      { "fdd", "rrc.fdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_fdd_05", HFILL }},
    { &hf_rrc_dl_HSPDSCH_Information,
      { "dl-HSPDSCH-Information", "rrc.dl_HSPDSCH_Information",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_dl_CommonInformation_02,
      { "dl-CommonInformation", "rrc.dl_CommonInformation",
        FT_NONE, BASE_NONE, NULL, 0,
        "DL_CommonInformation_r5", HFILL }},
    { &hf_rrc_dl_InformationPerRL_List_02,
      { "dl-InformationPerRL-List", "rrc.dl_InformationPerRL_List",
        FT_UINT32, BASE_DEC, NULL, 0,
        "DL_InformationPerRL_List_r5", HFILL }},
    { &hf_rrc_rb_InformationReconfigList_03,
      { "rb-InformationReconfigList", "rrc.rb_InformationReconfigList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "RB_InformationReconfigList_r6", HFILL }},
    { &hf_rrc_rb_InformationAffectedList_02,
      { "rb-InformationAffectedList", "rrc.rb_InformationAffectedList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "RB_InformationAffectedList_r6", HFILL }},
    { &hf_rrc_ul_deletedTransChInfoList_01,
      { "ul-deletedTransChInfoList", "rrc.ul_deletedTransChInfoList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "UL_DeletedTransChInfoList_r6", HFILL }},
    { &hf_rrc_ul_AddReconfTransChInfoList_01,
      { "ul-AddReconfTransChInfoList", "rrc.ul_AddReconfTransChInfoList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "UL_AddReconfTransChInfoList_r6", HFILL }},
    { &hf_rrc_ul_DPCH_Info,
      { "ul-DPCH-Info", "rrc.ul_DPCH_Info",
        FT_NONE, BASE_NONE, NULL, 0,
        "UL_DPCH_Info_r6", HFILL }},
    { &hf_rrc_ul_EDCH_Information,
      { "ul-EDCH-Information", "rrc.ul_EDCH_Information",
        FT_NONE, BASE_NONE, NULL, 0,
        "UL_EDCH_Information_r6", HFILL }},
    { &hf_rrc_dl_HSPDSCH_Information_01,
      { "dl-HSPDSCH-Information", "rrc.dl_HSPDSCH_Information",
        FT_NONE, BASE_NONE, NULL, 0,
        "DL_HSPDSCH_Information_r6", HFILL }},
    { &hf_rrc_dl_CommonInformation_03,
      { "dl-CommonInformation", "rrc.dl_CommonInformation",
        FT_NONE, BASE_NONE, NULL, 0,
        "DL_CommonInformation_r6", HFILL }},
    { &hf_rrc_dl_InformationPerRL_List_03,
      { "dl-InformationPerRL-List", "rrc.dl_InformationPerRL_List",
        FT_UINT32, BASE_DEC, NULL, 0,
        "DL_InformationPerRL_List_r6", HFILL }},
    { &hf_rrc_mbms_PL_ServiceRestrictInfo,
      { "mbms-PL-ServiceRestrictInfo", "rrc.mbms_PL_ServiceRestrictInfo",
        FT_UINT32, BASE_DEC, VALS(rrc_MBMS_PL_ServiceRestrictInfo_r6_vals), 0,
        "MBMS_PL_ServiceRestrictInfo_r6", HFILL }},
    { &hf_rrc_harq_Preamble_Mode,
      { "harq-Preamble-Mode", "rrc.harq_Preamble_Mode",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_beaconPLEst,
      { "beaconPLEst", "rrc.beaconPLEst",
        FT_UINT32, BASE_DEC, VALS(rrc_BEACON_PL_Est_vals), 0,
        "BEACON_PL_Est", HFILL }},
    { &hf_rrc_postVerificationPeriod,
      { "postVerificationPeriod", "rrc.postVerificationPeriod",
        FT_UINT32, BASE_DEC, VALS(rrc_T_postVerificationPeriod_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_dhs_sync,
      { "dhs-sync", "rrc.dhs_sync",
        FT_INT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_dl_InformationPerRL_List_v6b0ext,
      { "dl-InformationPerRL-List-v6b0ext", "rrc.dl_InformationPerRL_List_v6b0ext",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_integrityProtectionModeInfo_01,
      { "integrityProtectionModeInfo", "rrc.integrityProtectionModeInfo",
        FT_NONE, BASE_NONE, NULL, 0,
        "IntegrityProtectionModeInfo_r7", HFILL }},
    { &hf_rrc_cipheringModeInfo_01,
      { "cipheringModeInfo", "rrc.cipheringModeInfo",
        FT_NONE, BASE_NONE, NULL, 0,
        "CipheringModeInfo_r7", HFILL }},
    { &hf_rrc_utran_DRX_CycleLengthCoeff_01,
      { "utran-DRX-CycleLengthCoeff", "rrc.utran_DRX_CycleLengthCoeff",
        FT_NONE, BASE_NONE, NULL, 0,
        "UTRAN_DRX_CycleLengthCoefficient_r7", HFILL }},
    { &hf_rrc_waitTime,
      { "waitTime", "rrc.waitTime",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_supportForChangeOfUE_Capability,
      { "supportForChangeOfUE-Capability", "rrc.supportForChangeOfUE_Capability",
        FT_BOOLEAN, BASE_NONE, NULL, 0,
        "BOOLEAN", HFILL }},
    { &hf_rrc_rb_InformationReconfigList_04,
      { "rb-InformationReconfigList", "rrc.rb_InformationReconfigList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "RB_InformationReconfigList_r7", HFILL }},
    { &hf_rrc_rb_InformationAffectedList_03,
      { "rb-InformationAffectedList", "rrc.rb_InformationAffectedList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "RB_InformationAffectedList_r7", HFILL }},
    { &hf_rrc_ul_AddReconfTransChInfoList_02,
      { "ul-AddReconfTransChInfoList", "rrc.ul_AddReconfTransChInfoList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "UL_AddReconfTransChInfoList_r7", HFILL }},
    { &hf_rrc_dl_DeletedTransChInfoList_02,
      { "dl-DeletedTransChInfoList", "rrc.dl_DeletedTransChInfoList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "DL_DeletedTransChInfoList_r7", HFILL }},
    { &hf_rrc_dl_AddReconfTransChInfoList_03,
      { "dl-AddReconfTransChInfoList", "rrc.dl_AddReconfTransChInfoList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "DL_AddReconfTransChInfoList_r7", HFILL }},
    { &hf_rrc_multi_frequencyInfo,
      { "multi-frequencyInfo", "rrc.multi_frequencyInfo",
        FT_NONE, BASE_NONE, NULL, 0,
        "Multi_frequencyInfo_LCR_r7", HFILL }},
    { &hf_rrc_ul_DPCH_Info_01,
      { "ul-DPCH-Info", "rrc.ul_DPCH_Info",
        FT_NONE, BASE_NONE, NULL, 0,
        "UL_DPCH_Info_r7", HFILL }},
    { &hf_rrc_ul_EDCH_Information_01,
      { "ul-EDCH-Information", "rrc.ul_EDCH_Information",
        FT_NONE, BASE_NONE, NULL, 0,
        "UL_EDCH_Information_r7", HFILL }},
    { &hf_rrc_dl_HSPDSCH_Information_02,
      { "dl-HSPDSCH-Information", "rrc.dl_HSPDSCH_Information",
        FT_NONE, BASE_NONE, NULL, 0,
        "DL_HSPDSCH_Information_r7", HFILL }},
    { &hf_rrc_dl_CommonInformation_04,
      { "dl-CommonInformation", "rrc.dl_CommonInformation",
        FT_NONE, BASE_NONE, NULL, 0,
        "DL_CommonInformation_r7", HFILL }},
    { &hf_rrc_dl_InformationPerRL_List_04,
      { "dl-InformationPerRL-List", "rrc.dl_InformationPerRL_List",
        FT_UINT32, BASE_DEC, NULL, 0,
        "DL_InformationPerRL_List_r7", HFILL }},
    { &hf_rrc_v7f0NonCriticalExtensions_02,
      { "v7f0NonCriticalExtensions", "rrc.v7f0NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v7f0NonCriticalExtensions_02", HFILL }},
    { &hf_rrc_v7g0NonCriticalExtensions_04,
      { "v7g0NonCriticalExtensions", "rrc.v7g0NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v7g0NonCriticalExtensions_04", HFILL }},
    { &hf_rrc_nonCriticalExtensions_23,
      { "nonCriticalExtensions", "rrc.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_nonCriticalExtensions_23", HFILL }},
    { &hf_rrc_ul_EDCH_Information_02,
      { "ul-EDCH-Information", "rrc.ul_EDCH_Information",
        FT_UINT32, BASE_DEC, VALS(rrc_UL_EDCH_Information_ext_vals), 0,
        "UL_EDCH_Information_ext", HFILL }},
    { &hf_rrc_dummy_07,
      { "dummy", "rrc.dummy",
        FT_NONE, BASE_NONE, NULL, 0,
        "DefaultConfigForCellFACH", HFILL }},
    { &hf_rrc_rab_InformationSetup,
      { "rab-InformationSetup", "rrc.rab_InformationSetup",
        FT_NONE, BASE_NONE, NULL, 0,
        "RAB_InformationSetup_r8", HFILL }},
    { &hf_rrc_rb_InformationReconfigList_05,
      { "rb-InformationReconfigList", "rrc.rb_InformationReconfigList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "RB_InformationReconfigList_r8", HFILL }},
    { &hf_rrc_rb_InformationAffectedList_04,
      { "rb-InformationAffectedList", "rrc.rb_InformationAffectedList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "RB_InformationAffectedList_r8", HFILL }},
    { &hf_rrc_ul_AddReconfTransChInfoList_03,
      { "ul-AddReconfTransChInfoList", "rrc.ul_AddReconfTransChInfoList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "UL_AddReconfTransChInfoList_r8", HFILL }},
    { &hf_rrc_ul_EDCH_Information_03,
      { "ul-EDCH-Information", "rrc.ul_EDCH_Information",
        FT_NONE, BASE_NONE, NULL, 0,
        "UL_EDCH_Information_r8", HFILL }},
    { &hf_rrc_dl_HSPDSCH_Information_03,
      { "dl-HSPDSCH-Information", "rrc.dl_HSPDSCH_Information",
        FT_NONE, BASE_NONE, NULL, 0,
        "DL_HSPDSCH_Information_r8", HFILL }},
    { &hf_rrc_dl_CommonInformation_05,
      { "dl-CommonInformation", "rrc.dl_CommonInformation",
        FT_NONE, BASE_NONE, NULL, 0,
        "DL_CommonInformation_r8", HFILL }},
    { &hf_rrc_dl_InformationPerRL_List_05,
      { "dl-InformationPerRL-List", "rrc.dl_InformationPerRL_List",
        FT_UINT32, BASE_DEC, NULL, 0,
        "DL_InformationPerRL_List_r8", HFILL }},
    { &hf_rrc_controlChannelDRXInfo_TDD128,
      { "controlChannelDRXInfo-TDD128", "rrc.controlChannelDRXInfo_TDD128",
        FT_NONE, BASE_NONE, NULL, 0,
        "ControlChannelDRXInfo_TDD128_r8", HFILL }},
    { &hf_rrc_sps_Information_TDD128,
      { "sps-Information-TDD128", "rrc.sps_Information_TDD128",
        FT_NONE, BASE_NONE, NULL, 0,
        "SPS_Information_TDD128_r8", HFILL }},
    { &hf_rrc_rab_InformationSetup_01,
      { "rab-InformationSetup", "rrc.rab_InformationSetup",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_rab_InformationSetup_r7,
      { "rab-InformationSetup-r7", "rrc.rab_InformationSetup_r7",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_rab_InformationSetup_v820ext,
      { "rab-InformationSetup-v820ext", "rrc.rab_InformationSetup_v820ext",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_dl_HSPDSCH_Information_04,
      { "dl-HSPDSCH-Information", "rrc.dl_HSPDSCH_Information",
        FT_NONE, BASE_NONE, NULL, 0,
        "DL_HSPDSCH_Information_r8_ext", HFILL }},
    { &hf_rrc_dl_HSPDSCH_Information_05,
      { "dl-HSPDSCH-Information", "rrc.dl_HSPDSCH_Information",
        FT_NONE, BASE_NONE, NULL, 0,
        "DL_HSPDSCH_Information_r8_ext2", HFILL }},
    { &hf_rrc_dl_AddReconfTransChInfoList_04,
      { "dl-AddReconfTransChInfoList", "rrc.dl_AddReconfTransChInfoList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "DL_AddReconfTransChInfoList_r9", HFILL }},
    { &hf_rrc_ul_EDCH_Information_04,
      { "ul-EDCH-Information", "rrc.ul_EDCH_Information",
        FT_NONE, BASE_NONE, NULL, 0,
        "UL_EDCH_Information_r9", HFILL }},
    { &hf_rrc_dl_HSPDSCH_Information_06,
      { "dl-HSPDSCH-Information", "rrc.dl_HSPDSCH_Information",
        FT_NONE, BASE_NONE, NULL, 0,
        "DL_HSPDSCH_Information_r9", HFILL }},
    { &hf_rrc_cellDCHMeasOccasionInfo_TDD128,
      { "cellDCHMeasOccasionInfo-TDD128", "rrc.cellDCHMeasOccasionInfo_TDD128",
        FT_NONE, BASE_NONE, NULL, 0,
        "CellDCHMeasOccasionInfo_TDD128_r9", HFILL }},
    { &hf_rrc_dl_AddReconfTransChInfoList_05,
      { "dl-AddReconfTransChInfoList", "rrc.dl_AddReconfTransChInfoList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "DL_AddReconfTransChInfoList_TDD128_v9c0ext", HFILL }},
    { &hf_rrc_dummy2_03,
      { "dummy2", "rrc.dummy2",
        FT_NONE, BASE_NONE, NULL, 0,
        "SR_VCC_Info", HFILL }},
    { &hf_rrc_ul_MulticarrierEDCHInfo_TDD128,
      { "ul-MulticarrierEDCHInfo-TDD128", "rrc.ul_MulticarrierEDCHInfo_TDD128",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_dl_CommonInformation_06,
      { "dl-CommonInformation", "rrc.dl_CommonInformation",
        FT_NONE, BASE_NONE, NULL, 0,
        "DL_CommonInformation_r10", HFILL }},
    { &hf_rrc_mu_MIMO_Info_TDD128,
      { "mu-MIMO-Info-TDD128", "rrc.mu_MIMO_Info_TDD128",
        FT_UINT32, BASE_DEC, VALS(rrc_MU_MIMO_Info_TDD128_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_rb_InformationReconfigList_06,
      { "rb-InformationReconfigList", "rrc.rb_InformationReconfigList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "RB_InformationReconfigList_r11", HFILL }},
    { &hf_rrc_dl_AddReconfTransChInfoList_06,
      { "dl-AddReconfTransChInfoList", "rrc.dl_AddReconfTransChInfoList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "DL_AddReconfTransChInfoList_r11", HFILL }},
    { &hf_rrc_ul_DPCH_Info_02,
      { "ul-DPCH-Info", "rrc.ul_DPCH_Info",
        FT_NONE, BASE_NONE, NULL, 0,
        "UL_DPCH_Info_r11", HFILL }},
    { &hf_rrc_ul_EDCH_Information_05,
      { "ul-EDCH-Information", "rrc.ul_EDCH_Information",
        FT_NONE, BASE_NONE, NULL, 0,
        "UL_EDCH_Information_r11", HFILL }},
    { &hf_rrc_dl_HSPDSCH_Information_07,
      { "dl-HSPDSCH-Information", "rrc.dl_HSPDSCH_Information",
        FT_NONE, BASE_NONE, NULL, 0,
        "DL_HSPDSCH_Information_r11", HFILL }},
    { &hf_rrc_dl_CommonInformation_07,
      { "dl-CommonInformation", "rrc.dl_CommonInformation",
        FT_NONE, BASE_NONE, NULL, 0,
        "DL_CommonInformation_r11", HFILL }},
    { &hf_rrc_dl_InformationPerRL_List_06,
      { "dl-InformationPerRL-List", "rrc.dl_InformationPerRL_List",
        FT_UINT32, BASE_DEC, NULL, 0,
        "DL_InformationPerRL_List_r11", HFILL }},
    { &hf_rrc_commonERGCHInfoFDD,
      { "commonERGCHInfoFDD", "rrc.commonERGCHInfoFDD",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_r3_05,
      { "r3", "rrc.r3",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_r3_05", HFILL }},
    { &hf_rrc_laterNonCriticalExtensions_08,
      { "laterNonCriticalExtensions", "rrc.laterNonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_laterNonCriticalExtensions_08", HFILL }},
    { &hf_rrc_cellUpdateConfirm_CCCH_r3_add_ext,
      { "cellUpdateConfirm-CCCH-r3-add-ext", "rrc.cellUpdateConfirm_CCCH_r3_add_ext",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING", HFILL }},
    { &hf_rrc_v4b0NonCriticalExtensions_03,
      { "v4b0NonCriticalExtensions", "rrc.v4b0NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v4b0NonCriticalExtensions_03", HFILL }},
    { &hf_rrc_v590NonCriticalExtensions_03,
      { "v590NonCriticalExtensions", "rrc.v590NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v590NonCriticalExtensions_03", HFILL }},
    { &hf_rrc_v5d0NonCriticalExtenstions_03,
      { "v5d0NonCriticalExtenstions", "rrc.v5d0NonCriticalExtenstions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v5d0NonCriticalExtenstions_03", HFILL }},
    { &hf_rrc_v690NonCriticalExtensions_05,
      { "v690NonCriticalExtensions", "rrc.v690NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v690NonCriticalExtensions_05", HFILL }},
    { &hf_rrc_nonCriticalExtensions_24,
      { "nonCriticalExtensions", "rrc.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_nonCriticalExtensions_24", HFILL }},
    { &hf_rrc_later_than_r3_04,
      { "later-than-r3", "rrc.later_than_r3",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_later_than_r3_04", HFILL }},
    { &hf_rrc_criticalExtensions_19,
      { "criticalExtensions", "rrc.criticalExtensions",
        FT_UINT32, BASE_DEC, VALS(rrc_T_criticalExtensions_19_vals), 0,
        "T_criticalExtensions_19", HFILL }},
    { &hf_rrc_r4_01,
      { "r4", "rrc.r4",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_r4_01", HFILL }},
    { &hf_rrc_v4d0NonCriticalExtensions_01,
      { "v4d0NonCriticalExtensions", "rrc.v4d0NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v4d0NonCriticalExtensions_01", HFILL }},
    { &hf_rrc_cellUpdateConfirm_CCCH_r4_add_ext,
      { "cellUpdateConfirm-CCCH-r4-add-ext", "rrc.cellUpdateConfirm_CCCH_r4_add_ext",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING", HFILL }},
    { &hf_rrc_v590NonCriticalExtensions_04,
      { "v590NonCriticalExtensions", "rrc.v590NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v590NonCriticalExtensions_04", HFILL }},
    { &hf_rrc_v5d0NonCriticalExtenstions_04,
      { "v5d0NonCriticalExtenstions", "rrc.v5d0NonCriticalExtenstions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v5d0NonCriticalExtenstions_04", HFILL }},
    { &hf_rrc_v690NonCriticalExtensions_06,
      { "v690NonCriticalExtensions", "rrc.v690NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v690NonCriticalExtensions_06", HFILL }},
    { &hf_rrc_nonCriticalExtensions_25,
      { "nonCriticalExtensions", "rrc.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_nonCriticalExtensions_25", HFILL }},
    { &hf_rrc_criticalExtensions_20,
      { "criticalExtensions", "rrc.criticalExtensions",
        FT_UINT32, BASE_DEC, VALS(rrc_T_criticalExtensions_20_vals), 0,
        "T_criticalExtensions_20", HFILL }},
    { &hf_rrc_r5_01,
      { "r5", "rrc.r5",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_r5_01", HFILL }},
    { &hf_rrc_cellUpdateConfirm_CCCH_r5_add_ext,
      { "cellUpdateConfirm-CCCH-r5-add-ext", "rrc.cellUpdateConfirm_CCCH_r5_add_ext",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING", HFILL }},
    { &hf_rrc_v5d0NonCriticalExtenstions_05,
      { "v5d0NonCriticalExtenstions", "rrc.v5d0NonCriticalExtenstions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v5d0NonCriticalExtenstions_05", HFILL }},
    { &hf_rrc_v690NonCriticalExtensions_07,
      { "v690NonCriticalExtensions", "rrc.v690NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v690NonCriticalExtensions_07", HFILL }},
    { &hf_rrc_nonCriticalExtensions_26,
      { "nonCriticalExtensions", "rrc.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_nonCriticalExtensions_26", HFILL }},
    { &hf_rrc_criticalExtensions_21,
      { "criticalExtensions", "rrc.criticalExtensions",
        FT_UINT32, BASE_DEC, VALS(rrc_T_criticalExtensions_21_vals), 0,
        "T_criticalExtensions_21", HFILL }},
    { &hf_rrc_r6_02,
      { "r6", "rrc.r6",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_r6_02", HFILL }},
    { &hf_rrc_v6b0NonCriticalExtensions_03,
      { "v6b0NonCriticalExtensions", "rrc.v6b0NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v6b0NonCriticalExtensions_03", HFILL }},
    { &hf_rrc_nonCriticalExtensions_27,
      { "nonCriticalExtensions", "rrc.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_nonCriticalExtensions_27", HFILL }},
    { &hf_rrc_criticalExtensions_22,
      { "criticalExtensions", "rrc.criticalExtensions",
        FT_UINT32, BASE_DEC, VALS(rrc_T_criticalExtensions_22_vals), 0,
        "T_criticalExtensions_22", HFILL }},
    { &hf_rrc_r7_02,
      { "r7", "rrc.r7",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_r7_02", HFILL }},
    { &hf_rrc_cellUpdateConfirm_r7_add_ext_01,
      { "cellUpdateConfirm-r7-add-ext", "rrc.cellUpdateConfirm_r7_add_ext",
        FT_BYTES, BASE_NONE, NULL, 0,
        "T_cellUpdateConfirm_r7_add_ext_01", HFILL }},
    { &hf_rrc_v780NonCriticalExtensions_02,
      { "v780NonCriticalExtensions", "rrc.v780NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v780NonCriticalExtensions_02", HFILL }},
    { &hf_rrc_v860NonCriticalExtensions_02,
      { "v860NonCriticalExtensions", "rrc.v860NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v860NonCriticalExtensions_02", HFILL }},
    { &hf_rrc_nonCriticalExtensions_28,
      { "nonCriticalExtensions", "rrc.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_nonCriticalExtensions_28", HFILL }},
    { &hf_rrc_criticalExtensions_23,
      { "criticalExtensions", "rrc.criticalExtensions",
        FT_UINT32, BASE_DEC, VALS(rrc_T_criticalExtensions_23_vals), 0,
        "T_criticalExtensions_23", HFILL }},
    { &hf_rrc_r8_02,
      { "r8", "rrc.r8",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_r8_02", HFILL }},
    { &hf_rrc_v7d0NonCriticalExtensions_01,
      { "v7d0NonCriticalExtensions", "rrc.v7d0NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v7d0NonCriticalExtensions_01", HFILL }},
    { &hf_rrc_v890NonCriticalExtensions_02,
      { "v890NonCriticalExtensions", "rrc.v890NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v890NonCriticalExtensions_02", HFILL }},
    { &hf_rrc_v8a0NonCriticalExtensions_01,
      { "v8a0NonCriticalExtensions", "rrc.v8a0NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v8a0NonCriticalExtensions_01", HFILL }},
    { &hf_rrc_v7f0NonCriticalExtensions_03,
      { "v7f0NonCriticalExtensions", "rrc.v7f0NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v7f0NonCriticalExtensions_03", HFILL }},
    { &hf_rrc_v7g0NonCriticalExtensions_05,
      { "v7g0NonCriticalExtensions", "rrc.v7g0NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v7g0NonCriticalExtensions_05", HFILL }},
    { &hf_rrc_nonCriticalExtensions_29,
      { "nonCriticalExtensions", "rrc.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_nonCriticalExtensions_29", HFILL }},
    { &hf_rrc_criticalExtensions_24,
      { "criticalExtensions", "rrc.criticalExtensions",
        FT_UINT32, BASE_DEC, VALS(rrc_T_criticalExtensions_24_vals), 0,
        "T_criticalExtensions_24", HFILL }},
    { &hf_rrc_r9_02,
      { "r9", "rrc.r9",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_r9_02", HFILL }},
    { &hf_rrc_nonCriticalExtensions_30,
      { "nonCriticalExtensions", "rrc.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_nonCriticalExtensions_30", HFILL }},
    { &hf_rrc_criticalExtensions_25,
      { "criticalExtensions", "rrc.criticalExtensions",
        FT_UINT32, BASE_DEC, VALS(rrc_T_criticalExtensions_25_vals), 0,
        "T_criticalExtensions_25", HFILL }},
    { &hf_rrc_r10_02,
      { "r10", "rrc.r10",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_r10_02", HFILL }},
    { &hf_rrc_vbxyNonCriticalExtensions_03,
      { "vbxyNonCriticalExtensions", "rrc.vbxyNonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_vbxyNonCriticalExtensions_03", HFILL }},
    { &hf_rrc_nonCriticalExtensions_31,
      { "nonCriticalExtensions", "rrc.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_nonCriticalExtensions_31", HFILL }},
    { &hf_rrc_criticalExtensions_26,
      { "criticalExtensions", "rrc.criticalExtensions",
        FT_UINT32, BASE_DEC, VALS(rrc_T_criticalExtensions_26_vals), 0,
        "T_criticalExtensions_26", HFILL }},
    { &hf_rrc_r11_02,
      { "r11", "rrc.r11",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_r11_02", HFILL }},
    { &hf_rrc_nonCriticalExtensions_32,
      { "nonCriticalExtensions", "rrc.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_nonCriticalExtensions_32", HFILL }},
    { &hf_rrc_criticalExtensions_27,
      { "criticalExtensions", "rrc.criticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_criticalExtensions_27", HFILL }},
    { &hf_rrc_r3_06,
      { "r3", "rrc.r3",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_r3_06", HFILL }},
    { &hf_rrc_counterCheck_r3,
      { "counterCheck-r3", "rrc.counterCheck_r3",
        FT_NONE, BASE_NONE, NULL, 0,
        "CounterCheck_r3_IEs", HFILL }},
    { &hf_rrc_laterNonCriticalExtensions_09,
      { "laterNonCriticalExtensions", "rrc.laterNonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_laterNonCriticalExtensions_09", HFILL }},
    { &hf_rrc_counterCheck_r3_add_ext,
      { "counterCheck-r3-add-ext", "rrc.counterCheck_r3_add_ext",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING", HFILL }},
    { &hf_rrc_nonCriticalExtensions_33,
      { "nonCriticalExtensions", "rrc.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_nonCriticalExtensions_33", HFILL }},
    { &hf_rrc_later_than_r3_05,
      { "later-than-r3", "rrc.later_than_r3",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_later_than_r3_05", HFILL }},
    { &hf_rrc_criticalExtensions_28,
      { "criticalExtensions", "rrc.criticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_criticalExtensions_28", HFILL }},
    { &hf_rrc_rb_COUNT_C_MSB_InformationList,
      { "rb-COUNT-C-MSB-InformationList", "rrc.rb_COUNT_C_MSB_InformationList",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_rb_COUNT_C_InformationList,
      { "rb-COUNT-C-InformationList", "rrc.rb_COUNT_C_InformationList",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_laterNonCriticalExtensions_10,
      { "laterNonCriticalExtensions", "rrc.laterNonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_laterNonCriticalExtensions_10", HFILL }},
    { &hf_rrc_counterCheckResponse_r3_add_ext,
      { "counterCheckResponse-r3-add-ext", "rrc.counterCheckResponse_r3_add_ext",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING", HFILL }},
    { &hf_rrc_nonCriticalExtensions_34,
      { "nonCriticalExtensions", "rrc.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_nonCriticalExtensions_34", HFILL }},
    { &hf_rrc_r3_07,
      { "r3", "rrc.r3",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_r3_07", HFILL }},
    { &hf_rrc_downlinkDirectTransfer_r3,
      { "downlinkDirectTransfer-r3", "rrc.downlinkDirectTransfer_r3",
        FT_NONE, BASE_NONE, NULL, 0,
        "DownlinkDirectTransfer_r3_IEs", HFILL }},
    { &hf_rrc_laterNonCriticalExtensions_11,
      { "laterNonCriticalExtensions", "rrc.laterNonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_laterNonCriticalExtensions_11", HFILL }},
    { &hf_rrc_downlinkDirectTransfer_r3_add_ext,
      { "downlinkDirectTransfer-r3-add-ext", "rrc.downlinkDirectTransfer_r3_add_ext",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING", HFILL }},
    { &hf_rrc_nonCriticalExtensions_35,
      { "nonCriticalExtensions", "rrc.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_nonCriticalExtensions_35", HFILL }},
    { &hf_rrc_later_than_r3_06,
      { "later-than-r3", "rrc.later_than_r3",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_later_than_r3_06", HFILL }},
    { &hf_rrc_criticalExtensions_29,
      { "criticalExtensions", "rrc.criticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_criticalExtensions_29", HFILL }},
    { &hf_rrc_cn_DomainIdentity,
      { "cn-DomainIdentity", "rrc.cn_DomainIdentity",
        FT_UINT32, BASE_DEC, VALS(rrc_CN_DomainIdentity_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_nas_Message,
      { "nas-Message", "rrc.nas_Message",
        FT_BYTES, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_etws_Information,
      { "etws-Information", "rrc.etws_Information",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_etws_WarningSecurityInfo,
      { "etws-WarningSecurityInfo", "rrc.etws_WarningSecurityInfo",
        FT_BYTES, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_nonCriticalExtensions_36,
      { "nonCriticalExtensions", "rrc.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_nonCriticalExtensions_36", HFILL }},
    { &hf_rrc_r3_08,
      { "r3", "rrc.r3",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_r3_08", HFILL }},
    { &hf_rrc_handoverToUTRANCommand_r3,
      { "handoverToUTRANCommand-r3", "rrc.handoverToUTRANCommand_r3",
        FT_NONE, BASE_NONE, NULL, 0,
        "HandoverToUTRANCommand_r3_IEs", HFILL }},
    { &hf_rrc_nonCriticalExtensions_37,
      { "nonCriticalExtensions", "rrc.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_nonCriticalExtensions_37", HFILL }},
    { &hf_rrc_criticalExtensions_30,
      { "criticalExtensions", "rrc.criticalExtensions",
        FT_UINT32, BASE_DEC, VALS(rrc_T_criticalExtensions_30_vals), 0,
        "T_criticalExtensions_30", HFILL }},
    { &hf_rrc_r4_02,
      { "r4", "rrc.r4",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_r4_02", HFILL }},
    { &hf_rrc_handoverToUTRANCommand_r4,
      { "handoverToUTRANCommand-r4", "rrc.handoverToUTRANCommand_r4",
        FT_NONE, BASE_NONE, NULL, 0,
        "HandoverToUTRANCommand_r4_IEs", HFILL }},
    { &hf_rrc_nonCriticalExtensions_38,
      { "nonCriticalExtensions", "rrc.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_nonCriticalExtensions_38", HFILL }},
    { &hf_rrc_criticalExtensions_31,
      { "criticalExtensions", "rrc.criticalExtensions",
        FT_UINT32, BASE_DEC, VALS(rrc_T_criticalExtensions_31_vals), 0,
        "T_criticalExtensions_31", HFILL }},
    { &hf_rrc_r5_02,
      { "r5", "rrc.r5",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_r5_02", HFILL }},
    { &hf_rrc_handoverToUTRANCommand_r5,
      { "handoverToUTRANCommand-r5", "rrc.handoverToUTRANCommand_r5",
        FT_NONE, BASE_NONE, NULL, 0,
        "HandoverToUTRANCommand_r5_IEs", HFILL }},
    { &hf_rrc_nonCriticalExtensions_39,
      { "nonCriticalExtensions", "rrc.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_nonCriticalExtensions_39", HFILL }},
    { &hf_rrc_criticalExtensions_32,
      { "criticalExtensions", "rrc.criticalExtensions",
        FT_UINT32, BASE_DEC, VALS(rrc_T_criticalExtensions_32_vals), 0,
        "T_criticalExtensions_32", HFILL }},
    { &hf_rrc_r6_03,
      { "r6", "rrc.r6",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_r6_03", HFILL }},
    { &hf_rrc_handoverToUTRANCommand_r6,
      { "handoverToUTRANCommand-r6", "rrc.handoverToUTRANCommand_r6",
        FT_NONE, BASE_NONE, NULL, 0,
        "HandoverToUTRANCommand_r6_IEs", HFILL }},
    { &hf_rrc_v6b0NonCriticalExtensions_04,
      { "v6b0NonCriticalExtensions", "rrc.v6b0NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v6b0NonCriticalExtensions_04", HFILL }},
    { &hf_rrc_handoverToUTRANCommand_v6b0ext,
      { "handoverToUTRANCommand-v6b0ext", "rrc.handoverToUTRANCommand_v6b0ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "HandoverToUTRANCommand_v6b0ext_IEs", HFILL }},
    { &hf_rrc_nonCriticalExtensions_40,
      { "nonCriticalExtensions", "rrc.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_nonCriticalExtensions_40", HFILL }},
    { &hf_rrc_criticalExtensions_33,
      { "criticalExtensions", "rrc.criticalExtensions",
        FT_UINT32, BASE_DEC, VALS(rrc_T_criticalExtensions_33_vals), 0,
        "T_criticalExtensions_33", HFILL }},
    { &hf_rrc_r7_03,
      { "r7", "rrc.r7",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_r7_03", HFILL }},
    { &hf_rrc_handoverToUTRANCommand_r7,
      { "handoverToUTRANCommand-r7", "rrc.handoverToUTRANCommand_r7",
        FT_NONE, BASE_NONE, NULL, 0,
        "HandoverToUTRANCommand_r7_IEs", HFILL }},
    { &hf_rrc_v780NonCriticalExtensions_03,
      { "v780NonCriticalExtensions", "rrc.v780NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v780NonCriticalExtensions_03", HFILL }},
    { &hf_rrc_handoverToUTRANCommand_v780ext,
      { "handoverToUTRANCommand-v780ext", "rrc.handoverToUTRANCommand_v780ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "HandoverToUTRANCommand_v780ext_IEs", HFILL }},
    { &hf_rrc_v820NonCriticalExtensions,
      { "v820NonCriticalExtensions", "rrc.v820NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_handoverToUTRANCommand_v820ext,
      { "handoverToUTRANCommand-v820ext", "rrc.handoverToUTRANCommand_v820ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "HandoverToUTRANCommand_v820ext_IEs", HFILL }},
    { &hf_rrc_v7d0NonCriticalExtensions_02,
      { "v7d0NonCriticalExtensions", "rrc.v7d0NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v7d0NonCriticalExtensions_02", HFILL }},
    { &hf_rrc_handoverToUTRANCommand_v7d0ext,
      { "handoverToUTRANCommand-v7d0ext", "rrc.handoverToUTRANCommand_v7d0ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "HandoverToUTRANCommand_v7d0ext_IEs", HFILL }},
    { &hf_rrc_nonCriticalExtensions_41,
      { "nonCriticalExtensions", "rrc.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_nonCriticalExtensions_41", HFILL }},
    { &hf_rrc_criticalExtensions_34,
      { "criticalExtensions", "rrc.criticalExtensions",
        FT_UINT32, BASE_DEC, VALS(rrc_T_criticalExtensions_34_vals), 0,
        "T_criticalExtensions_34", HFILL }},
    { &hf_rrc_r8_03,
      { "r8", "rrc.r8",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_r8_03", HFILL }},
    { &hf_rrc_handoverToUTRANCommand_r8,
      { "handoverToUTRANCommand-r8", "rrc.handoverToUTRANCommand_r8",
        FT_NONE, BASE_NONE, NULL, 0,
        "HandoverToUTRANCommand_r8_IEs", HFILL }},
    { &hf_rrc_v7d0NonCriticalExtensions_03,
      { "v7d0NonCriticalExtensions", "rrc.v7d0NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v7d0NonCriticalExtensions_03", HFILL }},
    { &hf_rrc_v890NonCriticalExtensions_03,
      { "v890NonCriticalExtensions", "rrc.v890NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v890NonCriticalExtensions_03", HFILL }},
    { &hf_rrc_handoverToUTRANCommand_v890ext,
      { "handoverToUTRANCommand-v890ext", "rrc.handoverToUTRANCommand_v890ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "HandoverToUTRANCommand_v890ext_IEs", HFILL }},
    { &hf_rrc_v8a0NonCriticalExtensions_02,
      { "v8a0NonCriticalExtensions", "rrc.v8a0NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v8a0NonCriticalExtensions_02", HFILL }},
    { &hf_rrc_handoverToUTRANCommand_v8a0ext,
      { "handoverToUTRANCommand-v8a0ext", "rrc.handoverToUTRANCommand_v8a0ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "HandoverToUTRANCommand_v8a0ext_IEs", HFILL }},
    { &hf_rrc_nonCriticalExtensions_42,
      { "nonCriticalExtensions", "rrc.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_nonCriticalExtensions_42", HFILL }},
    { &hf_rrc_criticalExtensions_35,
      { "criticalExtensions", "rrc.criticalExtensions",
        FT_UINT32, BASE_DEC, VALS(rrc_T_criticalExtensions_35_vals), 0,
        "T_criticalExtensions_35", HFILL }},
    { &hf_rrc_r9_03,
      { "r9", "rrc.r9",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_r9_03", HFILL }},
    { &hf_rrc_handoverToUTRANCommand_r9,
      { "handoverToUTRANCommand-r9", "rrc.handoverToUTRANCommand_r9",
        FT_NONE, BASE_NONE, NULL, 0,
        "HandoverToUTRANCommand_r9_IEs", HFILL }},
    { &hf_rrc_v9c0NonCriticalExtensions_02,
      { "v9c0NonCriticalExtensions", "rrc.v9c0NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v9c0NonCriticalExtensions_02", HFILL }},
    { &hf_rrc_handoverToUTRANCommand_v9c0ext,
      { "handoverToUTRANCommand-v9c0ext", "rrc.handoverToUTRANCommand_v9c0ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "HandoverToUTRANCommand_v9c0ext_IEs", HFILL }},
    { &hf_rrc_nonCriticalExtensions_43,
      { "nonCriticalExtensions", "rrc.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_nonCriticalExtensions_43", HFILL }},
    { &hf_rrc_criticalExtensions_36,
      { "criticalExtensions", "rrc.criticalExtensions",
        FT_UINT32, BASE_DEC, VALS(rrc_HandoverToUTRANCommand_LaterCriticalExtensions_vals), 0,
        "HandoverToUTRANCommand_LaterCriticalExtensions", HFILL }},
    { &hf_rrc_new_U_RNTI_01,
      { "new-U-RNTI", "rrc.new_U_RNTI",
        FT_NONE, BASE_NONE, NULL, 0,
        "U_RNTI_Short", HFILL }},
    { &hf_rrc_dummy_08,
      { "dummy", "rrc.dummy",
        FT_UINT32, BASE_DEC, NULL, 0,
        "ActivationTime", HFILL }},
    { &hf_rrc_cipheringAlgorithm,
      { "cipheringAlgorithm", "rrc.cipheringAlgorithm",
        FT_UINT32, BASE_DEC, VALS(rrc_CipheringAlgorithm_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_specificationMode,
      { "specificationMode", "rrc.specificationMode",
        FT_UINT32, BASE_DEC, VALS(rrc_T_specificationMode_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_complete,
      { "complete", "rrc.complete",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_srb_InformationSetupList,
      { "srb-InformationSetupList", "rrc.srb_InformationSetupList",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_rab_InformationSetupList,
      { "rab-InformationSetupList", "rrc.rab_InformationSetupList",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_ul_DPCH_Info_03,
      { "ul-DPCH-Info", "rrc.ul_DPCH_Info",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_modeSpecificInfo,
      { "modeSpecificInfo", "rrc.modeSpecificInfo",
        FT_UINT32, BASE_DEC, VALS(rrc_T_modeSpecificInfo_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_fdd_06,
      { "fdd", "rrc.fdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_fdd_06", HFILL }},
    { &hf_rrc_dummy2_04,
      { "dummy2", "rrc.dummy2",
        FT_NONE, BASE_NONE, NULL, 0,
        "CPCH_SetInfo", HFILL }},
    { &hf_rrc_preconfiguration,
      { "preconfiguration", "rrc.preconfiguration",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_preConfigMode,
      { "preConfigMode", "rrc.preConfigMode",
        FT_UINT32, BASE_DEC, VALS(rrc_T_preConfigMode_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_predefinedConfigIdentity,
      { "predefinedConfigIdentity", "rrc.predefinedConfigIdentity",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_defaultConfig,
      { "defaultConfig", "rrc.defaultConfig",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_defaultConfigMode,
      { "defaultConfigMode", "rrc.defaultConfigMode",
        FT_UINT32, BASE_DEC, VALS(rrc_DefaultConfigMode_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_defaultConfigIdentity,
      { "defaultConfigIdentity", "rrc.defaultConfigIdentity",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_rab_Info,
      { "rab-Info", "rrc.rab_Info",
        FT_NONE, BASE_NONE, NULL, 0,
        "RAB_Info_Post", HFILL }},
    { &hf_rrc_modeSpecificInfo_01,
      { "modeSpecificInfo", "rrc.modeSpecificInfo",
        FT_UINT32, BASE_DEC, VALS(rrc_T_modeSpecificInfo_01_vals), 0,
        "T_modeSpecificInfo_01", HFILL }},
    { &hf_rrc_fdd_07,
      { "fdd", "rrc.fdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_fdd_07", HFILL }},
    { &hf_rrc_ul_DPCH_Info_04,
      { "ul-DPCH-Info", "rrc.ul_DPCH_Info",
        FT_NONE, BASE_NONE, NULL, 0,
        "UL_DPCH_InfoPostFDD", HFILL }},
    { &hf_rrc_dl_CommonInformationPost,
      { "dl-CommonInformationPost", "rrc.dl_CommonInformationPost",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_dl_InformationPerRL_List_07,
      { "dl-InformationPerRL-List", "rrc.dl_InformationPerRL_List",
        FT_UINT32, BASE_DEC, NULL, 0,
        "DL_InformationPerRL_ListPostFDD", HFILL }},
    { &hf_rrc_frequencyInfo_01,
      { "frequencyInfo", "rrc.frequencyInfo",
        FT_NONE, BASE_NONE, NULL, 0,
        "FrequencyInfoFDD", HFILL }},
    { &hf_rrc_tdd_01,
      { "tdd", "rrc.tdd",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_ul_DPCH_Info_05,
      { "ul-DPCH-Info", "rrc.ul_DPCH_Info",
        FT_NONE, BASE_NONE, NULL, 0,
        "UL_DPCH_InfoPostTDD", HFILL }},
    { &hf_rrc_dl_InformationPerRL,
      { "dl-InformationPerRL", "rrc.dl_InformationPerRL",
        FT_NONE, BASE_NONE, NULL, 0,
        "DL_InformationPerRL_PostTDD", HFILL }},
    { &hf_rrc_frequencyInfo_02,
      { "frequencyInfo", "rrc.frequencyInfo",
        FT_NONE, BASE_NONE, NULL, 0,
        "FrequencyInfoTDD", HFILL }},
    { &hf_rrc_primaryCCPCH_TX_Power,
      { "primaryCCPCH-TX-Power", "rrc.primaryCCPCH_TX_Power",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_specificationMode_01,
      { "specificationMode", "rrc.specificationMode",
        FT_UINT32, BASE_DEC, VALS(rrc_T_specificationMode_01_vals), 0,
        "T_specificationMode_01", HFILL }},
    { &hf_rrc_complete_01,
      { "complete", "rrc.complete",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_complete_01", HFILL }},
    { &hf_rrc_rab_InformationSetupList_01,
      { "rab-InformationSetupList", "rrc.rab_InformationSetupList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "RAB_InformationSetupList_r4", HFILL }},
    { &hf_rrc_ul_DPCH_Info_06,
      { "ul-DPCH-Info", "rrc.ul_DPCH_Info",
        FT_NONE, BASE_NONE, NULL, 0,
        "UL_DPCH_Info_r4", HFILL }},
    { &hf_rrc_modeSpecificInfo_02,
      { "modeSpecificInfo", "rrc.modeSpecificInfo",
        FT_UINT32, BASE_DEC, VALS(rrc_T_modeSpecificInfo_02_vals), 0,
        "T_modeSpecificInfo_02", HFILL }},
    { &hf_rrc_fdd_08,
      { "fdd", "rrc.fdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_fdd_08", HFILL }},
    { &hf_rrc_preconfiguration_01,
      { "preconfiguration", "rrc.preconfiguration",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_preconfiguration_01", HFILL }},
    { &hf_rrc_preConfigMode_01,
      { "preConfigMode", "rrc.preConfigMode",
        FT_UINT32, BASE_DEC, VALS(rrc_T_preConfigMode_01_vals), 0,
        "T_preConfigMode_01", HFILL }},
    { &hf_rrc_defaultConfig_01,
      { "defaultConfig", "rrc.defaultConfig",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_defaultConfig_01", HFILL }},
    { &hf_rrc_defaultConfigIdentity_01,
      { "defaultConfigIdentity", "rrc.defaultConfigIdentity",
        FT_UINT32, BASE_DEC, NULL, 0,
        "DefaultConfigIdentity_r4", HFILL }},
    { &hf_rrc_modeSpecificInfo_03,
      { "modeSpecificInfo", "rrc.modeSpecificInfo",
        FT_UINT32, BASE_DEC, VALS(rrc_T_modeSpecificInfo_03_vals), 0,
        "T_modeSpecificInfo_03", HFILL }},
    { &hf_rrc_fdd_09,
      { "fdd", "rrc.fdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_fdd_09", HFILL }},
    { &hf_rrc_tdd_02,
      { "tdd", "rrc.tdd",
        FT_UINT32, BASE_DEC, VALS(rrc_T_tdd_01_vals), 0,
        "T_tdd_01", HFILL }},
    { &hf_rrc_tdd384,
      { "tdd384", "rrc.tdd384",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_tdd128,
      { "tdd128", "rrc.tdd128",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_ul_DPCH_Info_07,
      { "ul-DPCH-Info", "rrc.ul_DPCH_Info",
        FT_NONE, BASE_NONE, NULL, 0,
        "UL_DPCH_InfoPostTDD_LCR_r4", HFILL }},
    { &hf_rrc_dl_InformationPerRL_01,
      { "dl-InformationPerRL", "rrc.dl_InformationPerRL",
        FT_NONE, BASE_NONE, NULL, 0,
        "DL_InformationPerRL_PostTDD_LCR_r4", HFILL }},
    { &hf_rrc_specificationMode_02,
      { "specificationMode", "rrc.specificationMode",
        FT_UINT32, BASE_DEC, VALS(rrc_T_specificationMode_02_vals), 0,
        "T_specificationMode_02", HFILL }},
    { &hf_rrc_complete_02,
      { "complete", "rrc.complete",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_complete_02", HFILL }},
    { &hf_rrc_srb_InformationSetupList_01,
      { "srb-InformationSetupList", "rrc.srb_InformationSetupList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "SRB_InformationSetupList_r5", HFILL }},
    { &hf_rrc_rab_InformationSetupList_02,
      { "rab-InformationSetupList", "rrc.rab_InformationSetupList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "RAB_InformationSetupList_r5", HFILL }},
    { &hf_rrc_ul_DPCH_Info_08,
      { "ul-DPCH-Info", "rrc.ul_DPCH_Info",
        FT_NONE, BASE_NONE, NULL, 0,
        "UL_DPCH_Info_r5", HFILL }},
    { &hf_rrc_modeSpecificInfo_04,
      { "modeSpecificInfo", "rrc.modeSpecificInfo",
        FT_UINT32, BASE_DEC, VALS(rrc_T_modeSpecificInfo_04_vals), 0,
        "T_modeSpecificInfo_04", HFILL }},
    { &hf_rrc_fdd_10,
      { "fdd", "rrc.fdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_fdd_10", HFILL }},
    { &hf_rrc_preconfiguration_02,
      { "preconfiguration", "rrc.preconfiguration",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_preconfiguration_02", HFILL }},
    { &hf_rrc_preConfigMode_02,
      { "preConfigMode", "rrc.preConfigMode",
        FT_UINT32, BASE_DEC, VALS(rrc_T_preConfigMode_02_vals), 0,
        "T_preConfigMode_02", HFILL }},
    { &hf_rrc_defaultConfig_02,
      { "defaultConfig", "rrc.defaultConfig",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_defaultConfig_02", HFILL }},
    { &hf_rrc_defaultConfigIdentity_02,
      { "defaultConfigIdentity", "rrc.defaultConfigIdentity",
        FT_UINT32, BASE_DEC, NULL, 0,
        "DefaultConfigIdentity_r5", HFILL }},
    { &hf_rrc_modeSpecificInfo_05,
      { "modeSpecificInfo", "rrc.modeSpecificInfo",
        FT_UINT32, BASE_DEC, VALS(rrc_T_modeSpecificInfo_05_vals), 0,
        "T_modeSpecificInfo_05", HFILL }},
    { &hf_rrc_fdd_11,
      { "fdd", "rrc.fdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_fdd_11", HFILL }},
    { &hf_rrc_tdd_03,
      { "tdd", "rrc.tdd",
        FT_UINT32, BASE_DEC, VALS(rrc_T_tdd_02_vals), 0,
        "T_tdd_02", HFILL }},
    { &hf_rrc_tdd384_01,
      { "tdd384", "rrc.tdd384",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_tdd384_01", HFILL }},
    { &hf_rrc_tdd128_01,
      { "tdd128", "rrc.tdd128",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_tdd128_01", HFILL }},
    { &hf_rrc_specificationMode_03,
      { "specificationMode", "rrc.specificationMode",
        FT_UINT32, BASE_DEC, VALS(rrc_T_specificationMode_03_vals), 0,
        "T_specificationMode_03", HFILL }},
    { &hf_rrc_complete_03,
      { "complete", "rrc.complete",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_complete_03", HFILL }},
    { &hf_rrc_srb_InformationSetupList_02,
      { "srb-InformationSetupList", "rrc.srb_InformationSetupList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "SRB_InformationSetupList_r6", HFILL }},
    { &hf_rrc_rab_InformationSetupList_03,
      { "rab-InformationSetupList", "rrc.rab_InformationSetupList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "RAB_InformationSetupList_r6", HFILL }},
    { &hf_rrc_preconfiguration_03,
      { "preconfiguration", "rrc.preconfiguration",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_preconfiguration_03", HFILL }},
    { &hf_rrc_preConfigMode_03,
      { "preConfigMode", "rrc.preConfigMode",
        FT_UINT32, BASE_DEC, VALS(rrc_T_preConfigMode_03_vals), 0,
        "T_preConfigMode_03", HFILL }},
    { &hf_rrc_defaultConfig_03,
      { "defaultConfig", "rrc.defaultConfig",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_defaultConfig_03", HFILL }},
    { &hf_rrc_defaultConfigIdentity_03,
      { "defaultConfigIdentity", "rrc.defaultConfigIdentity",
        FT_UINT32, BASE_DEC, NULL, 0,
        "DefaultConfigIdentity_r6", HFILL }},
    { &hf_rrc_modeSpecificInfo_06,
      { "modeSpecificInfo", "rrc.modeSpecificInfo",
        FT_UINT32, BASE_DEC, VALS(rrc_T_modeSpecificInfo_06_vals), 0,
        "T_modeSpecificInfo_06", HFILL }},
    { &hf_rrc_fdd_12,
      { "fdd", "rrc.fdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_fdd_12", HFILL }},
    { &hf_rrc_tdd_04,
      { "tdd", "rrc.tdd",
        FT_UINT32, BASE_DEC, VALS(rrc_T_tdd_03_vals), 0,
        "T_tdd_03", HFILL }},
    { &hf_rrc_tdd384_02,
      { "tdd384", "rrc.tdd384",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_tdd384_02", HFILL }},
    { &hf_rrc_tdd128_02,
      { "tdd128", "rrc.tdd128",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_tdd128_02", HFILL }},
    { &hf_rrc_ue_hspa_identities,
      { "ue-hspa-identities", "rrc.ue_hspa_identities",
        FT_NONE, BASE_NONE, NULL, 0,
        "UE_HSPA_Identities_r6", HFILL }},
    { &hf_rrc_cipheringAlgorithm_01,
      { "cipheringAlgorithm", "rrc.cipheringAlgorithm",
        FT_UINT32, BASE_DEC, VALS(rrc_CipheringAlgorithm_r7_vals), 0,
        "CipheringAlgorithm_r7", HFILL }},
    { &hf_rrc_specificationMode_04,
      { "specificationMode", "rrc.specificationMode",
        FT_UINT32, BASE_DEC, VALS(rrc_T_specificationMode_04_vals), 0,
        "T_specificationMode_04", HFILL }},
    { &hf_rrc_complete_04,
      { "complete", "rrc.complete",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_complete_04", HFILL }},
    { &hf_rrc_srb_InformationSetupList_03,
      { "srb-InformationSetupList", "rrc.srb_InformationSetupList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "SRB_InformationSetupList_r7", HFILL }},
    { &hf_rrc_rab_InformationSetupList_04,
      { "rab-InformationSetupList", "rrc.rab_InformationSetupList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "RAB_InformationSetupList_r7", HFILL }},
    { &hf_rrc_preconfiguration_04,
      { "preconfiguration", "rrc.preconfiguration",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_preconfiguration_04", HFILL }},
    { &hf_rrc_preConfigMode_04,
      { "preConfigMode", "rrc.preConfigMode",
        FT_UINT32, BASE_DEC, VALS(rrc_T_preConfigMode_04_vals), 0,
        "T_preConfigMode_04", HFILL }},
    { &hf_rrc_defaultConfig_04,
      { "defaultConfig", "rrc.defaultConfig",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_defaultConfig_04", HFILL }},
    { &hf_rrc_modeSpecificInfo_07,
      { "modeSpecificInfo", "rrc.modeSpecificInfo",
        FT_UINT32, BASE_DEC, VALS(rrc_T_modeSpecificInfo_07_vals), 0,
        "T_modeSpecificInfo_07", HFILL }},
    { &hf_rrc_fdd_13,
      { "fdd", "rrc.fdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_fdd_13", HFILL }},
    { &hf_rrc_tdd_05,
      { "tdd", "rrc.tdd",
        FT_UINT32, BASE_DEC, VALS(rrc_T_tdd_04_vals), 0,
        "T_tdd_04", HFILL }},
    { &hf_rrc_tdd384_03,
      { "tdd384", "rrc.tdd384",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_tdd384_03", HFILL }},
    { &hf_rrc_tdd128_03,
      { "tdd128", "rrc.tdd128",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_tdd128_03", HFILL }},
    { &hf_rrc_tdd768,
      { "tdd768", "rrc.tdd768",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_dl_InformationPerRL_02,
      { "dl-InformationPerRL", "rrc.dl_InformationPerRL",
        FT_UINT32, BASE_DEC, NULL, 0,
        "DL_InformationPerRL_List_r7", HFILL }},
    { &hf_rrc_specificationMode_05,
      { "specificationMode", "rrc.specificationMode",
        FT_UINT32, BASE_DEC, VALS(rrc_T_specificationMode_05_vals), 0,
        "T_specificationMode_05", HFILL }},
    { &hf_rrc_complete_05,
      { "complete", "rrc.complete",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_complete_05", HFILL }},
    { &hf_rrc_srb_InformationSetupList_04,
      { "srb-InformationSetupList", "rrc.srb_InformationSetupList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "SRB_InformationSetupList_r8", HFILL }},
    { &hf_rrc_rab_InformationSetupList_05,
      { "rab-InformationSetupList", "rrc.rab_InformationSetupList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "RAB_InformationSetupList_r8", HFILL }},
    { &hf_rrc_preconfiguration_05,
      { "preconfiguration", "rrc.preconfiguration",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_preconfiguration_05", HFILL }},
    { &hf_rrc_preConfigMode_05,
      { "preConfigMode", "rrc.preConfigMode",
        FT_UINT32, BASE_DEC, VALS(rrc_T_preConfigMode_05_vals), 0,
        "T_preConfigMode_05", HFILL }},
    { &hf_rrc_defaultConfig_05,
      { "defaultConfig", "rrc.defaultConfig",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_defaultConfig_05", HFILL }},
    { &hf_rrc_dummy_09,
      { "dummy", "rrc.dummy",
        FT_NONE, BASE_NONE, NULL, 0,
        "DLUL_HSPA_Information_r8", HFILL }},
    { &hf_rrc_modeSpecificInfo_08,
      { "modeSpecificInfo", "rrc.modeSpecificInfo",
        FT_UINT32, BASE_DEC, VALS(rrc_T_modeSpecificInfo_08_vals), 0,
        "T_modeSpecificInfo_08", HFILL }},
    { &hf_rrc_fdd_14,
      { "fdd", "rrc.fdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_fdd_14", HFILL }},
    { &hf_rrc_tdd_06,
      { "tdd", "rrc.tdd",
        FT_UINT32, BASE_DEC, VALS(rrc_T_tdd_05_vals), 0,
        "T_tdd_05", HFILL }},
    { &hf_rrc_tdd384_04,
      { "tdd384", "rrc.tdd384",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_tdd384_04", HFILL }},
    { &hf_rrc_tdd128_04,
      { "tdd128", "rrc.tdd128",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_tdd128_04", HFILL }},
    { &hf_rrc_tdd768_01,
      { "tdd768", "rrc.tdd768",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_tdd768_01", HFILL }},
    { &hf_rrc_rab_InformationSetupList_06,
      { "rab-InformationSetupList", "rrc.rab_InformationSetupList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "RAB_InformationSetupList_v820ext", HFILL }},
    { &hf_rrc_defaultConfigForCellFACH,
      { "defaultConfigForCellFACH", "rrc.defaultConfigForCellFACH",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_specificationMode_06,
      { "specificationMode", "rrc.specificationMode",
        FT_UINT32, BASE_DEC, VALS(rrc_T_specificationMode_06_vals), 0,
        "T_specificationMode_06", HFILL }},
    { &hf_rrc_complete_06,
      { "complete", "rrc.complete",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_complete_06", HFILL }},
    { &hf_rrc_preconfiguration_06,
      { "preconfiguration", "rrc.preconfiguration",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_preconfiguration_06", HFILL }},
    { &hf_rrc_preConfigMode_06,
      { "preConfigMode", "rrc.preConfigMode",
        FT_UINT32, BASE_DEC, VALS(rrc_T_preConfigMode_06_vals), 0,
        "T_preConfigMode_06", HFILL }},
    { &hf_rrc_defaultConfig_06,
      { "defaultConfig", "rrc.defaultConfig",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_defaultConfig_06", HFILL }},
    { &hf_rrc_dummy_10,
      { "dummy", "rrc.dummy",
        FT_NONE, BASE_NONE, NULL, 0,
        "DLUL_HSPA_Information_r9", HFILL }},
    { &hf_rrc_modeSpecificInfo_09,
      { "modeSpecificInfo", "rrc.modeSpecificInfo",
        FT_UINT32, BASE_DEC, VALS(rrc_T_modeSpecificInfo_09_vals), 0,
        "T_modeSpecificInfo_09", HFILL }},
    { &hf_rrc_fdd_15,
      { "fdd", "rrc.fdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_fdd_15", HFILL }},
    { &hf_rrc_tdd_07,
      { "tdd", "rrc.tdd",
        FT_UINT32, BASE_DEC, VALS(rrc_T_tdd_06_vals), 0,
        "T_tdd_06", HFILL }},
    { &hf_rrc_tdd384_05,
      { "tdd384", "rrc.tdd384",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_tdd384_05", HFILL }},
    { &hf_rrc_tdd128_05,
      { "tdd128", "rrc.tdd128",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_tdd128_05", HFILL }},
    { &hf_rrc_tdd768_02,
      { "tdd768", "rrc.tdd768",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_tdd768_02", HFILL }},
    { &hf_rrc_r10_03,
      { "r10", "rrc.r10",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_r10_03", HFILL }},
    { &hf_rrc_handoverToUTRANCommand_r10,
      { "handoverToUTRANCommand-r10", "rrc.handoverToUTRANCommand_r10",
        FT_NONE, BASE_NONE, NULL, 0,
        "HandoverToUTRANCommand_r10_IEs", HFILL }},
    { &hf_rrc_v9c0NonCriticalExtensions_03,
      { "v9c0NonCriticalExtensions", "rrc.v9c0NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v9c0NonCriticalExtensions_03", HFILL }},
    { &hf_rrc_nonCriticalExtensions_44,
      { "nonCriticalExtensions", "rrc.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_nonCriticalExtensions_44", HFILL }},
    { &hf_rrc_criticalExtensions_37,
      { "criticalExtensions", "rrc.criticalExtensions",
        FT_UINT32, BASE_DEC, VALS(rrc_T_criticalExtensions_36_vals), 0,
        "T_criticalExtensions_36", HFILL }},
    { &hf_rrc_r11_03,
      { "r11", "rrc.r11",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_r11_03", HFILL }},
    { &hf_rrc_handoverToUTRANCommand_r11,
      { "handoverToUTRANCommand-r11", "rrc.handoverToUTRANCommand_r11",
        FT_NONE, BASE_NONE, NULL, 0,
        "HandoverToUTRANCommand_r11_IEs", HFILL }},
    { &hf_rrc_handoverToUTRANCommand_r11_add_ext,
      { "handoverToUTRANCommand-r11-add-ext", "rrc.handoverToUTRANCommand_r11_add_ext",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING", HFILL }},
    { &hf_rrc_nonCriticalExtensions_45,
      { "nonCriticalExtensions", "rrc.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_nonCriticalExtensions_45", HFILL }},
    { &hf_rrc_criticalExtensions_38,
      { "criticalExtensions", "rrc.criticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_criticalExtensions_37", HFILL }},
    { &hf_rrc_specificationMode_07,
      { "specificationMode", "rrc.specificationMode",
        FT_UINT32, BASE_DEC, VALS(rrc_T_specificationMode_07_vals), 0,
        "T_specificationMode_07", HFILL }},
    { &hf_rrc_complete_07,
      { "complete", "rrc.complete",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_complete_07", HFILL }},
    { &hf_rrc_preconfiguration_07,
      { "preconfiguration", "rrc.preconfiguration",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_preconfiguration_07", HFILL }},
    { &hf_rrc_preConfigMode_07,
      { "preConfigMode", "rrc.preConfigMode",
        FT_UINT32, BASE_DEC, VALS(rrc_T_preConfigMode_07_vals), 0,
        "T_preConfigMode_07", HFILL }},
    { &hf_rrc_defaultConfig_07,
      { "defaultConfig", "rrc.defaultConfig",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_defaultConfig_07", HFILL }},
    { &hf_rrc_dummy_11,
      { "dummy", "rrc.dummy",
        FT_NONE, BASE_NONE, NULL, 0,
        "DLUL_HSPA_Information_r10", HFILL }},
    { &hf_rrc_modeSpecificInfo_10,
      { "modeSpecificInfo", "rrc.modeSpecificInfo",
        FT_UINT32, BASE_DEC, VALS(rrc_T_modeSpecificInfo_10_vals), 0,
        "T_modeSpecificInfo_10", HFILL }},
    { &hf_rrc_fdd_16,
      { "fdd", "rrc.fdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_fdd_16", HFILL }},
    { &hf_rrc_tdd_08,
      { "tdd", "rrc.tdd",
        FT_UINT32, BASE_DEC, VALS(rrc_T_tdd_07_vals), 0,
        "T_tdd_07", HFILL }},
    { &hf_rrc_tdd384_06,
      { "tdd384", "rrc.tdd384",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_tdd384_06", HFILL }},
    { &hf_rrc_tdd128_06,
      { "tdd128", "rrc.tdd128",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_tdd128_06", HFILL }},
    { &hf_rrc_tdd768_03,
      { "tdd768", "rrc.tdd768",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_tdd768_03", HFILL }},
    { &hf_rrc_specificationMode_08,
      { "specificationMode", "rrc.specificationMode",
        FT_UINT32, BASE_DEC, VALS(rrc_T_specificationMode_08_vals), 0,
        "T_specificationMode_08", HFILL }},
    { &hf_rrc_complete_08,
      { "complete", "rrc.complete",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_complete_08", HFILL }},
    { &hf_rrc_rab_InformationSetupList_07,
      { "rab-InformationSetupList", "rrc.rab_InformationSetupList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "RAB_InformationSetupList_r11", HFILL }},
    { &hf_rrc_preconfiguration_08,
      { "preconfiguration", "rrc.preconfiguration",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_preconfiguration_08", HFILL }},
    { &hf_rrc_preConfigMode_08,
      { "preConfigMode", "rrc.preConfigMode",
        FT_UINT32, BASE_DEC, VALS(rrc_T_preConfigMode_08_vals), 0,
        "T_preConfigMode_08", HFILL }},
    { &hf_rrc_defaultConfig_08,
      { "defaultConfig", "rrc.defaultConfig",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_defaultConfig_08", HFILL }},
    { &hf_rrc_modeSpecificInfo_11,
      { "modeSpecificInfo", "rrc.modeSpecificInfo",
        FT_UINT32, BASE_DEC, VALS(rrc_T_modeSpecificInfo_11_vals), 0,
        "T_modeSpecificInfo_11", HFILL }},
    { &hf_rrc_fdd_17,
      { "fdd", "rrc.fdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_fdd_17", HFILL }},
    { &hf_rrc_tdd_09,
      { "tdd", "rrc.tdd",
        FT_UINT32, BASE_DEC, VALS(rrc_T_tdd_08_vals), 0,
        "T_tdd_08", HFILL }},
    { &hf_rrc_tdd384_07,
      { "tdd384", "rrc.tdd384",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_tdd384_07", HFILL }},
    { &hf_rrc_tdd128_07,
      { "tdd128", "rrc.tdd128",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_tdd128_07", HFILL }},
    { &hf_rrc_tdd768_04,
      { "tdd768", "rrc.tdd768",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_tdd768_04", HFILL }},
    { &hf_rrc_count_C_ActivationTime,
      { "count-C-ActivationTime", "rrc.count_C_ActivationTime",
        FT_UINT32, BASE_DEC, NULL, 0,
        "ActivationTime", HFILL }},
    { &hf_rrc_laterNonCriticalExtensions_12,
      { "laterNonCriticalExtensions", "rrc.laterNonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_laterNonCriticalExtensions_12", HFILL }},
    { &hf_rrc_handoverToUTRANComplete_r3_add_ext,
      { "handoverToUTRANComplete-r3-add-ext", "rrc.handoverToUTRANComplete_r3_add_ext",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING", HFILL }},
    { &hf_rrc_va40NonCriticalExtensions_01,
      { "va40NonCriticalExtensions", "rrc.va40NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_va40NonCriticalExtensions_01", HFILL }},
    { &hf_rrc_handoverToUTRANComplete_va40ext,
      { "handoverToUTRANComplete-va40ext", "rrc.handoverToUTRANComplete_va40ext",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_vbxyNonCriticalExtensions_04,
      { "vbxyNonCriticalExtensions", "rrc.vbxyNonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_vbxyNonCriticalExtensions_04", HFILL }},
    { &hf_rrc_handoverToUTRANComplete_vbxyext,
      { "handoverToUTRANComplete-vbxyext", "rrc.handoverToUTRANComplete_vbxyext",
        FT_NONE, BASE_NONE, NULL, 0,
        "HandoverToUTRANComplete_vbxyext_IEs", HFILL }},
    { &hf_rrc_nonCriticalExtensions_46,
      { "nonCriticalExtensions", "rrc.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_nonCriticalExtensions_46", HFILL }},
    { &hf_rrc_loggedMeasAvailable_01,
      { "loggedMeasAvailable", "rrc.loggedMeasAvailable",
        FT_UINT32, BASE_DEC, VALS(rrc_T_loggedMeasAvailable_01_vals), 0,
        "T_loggedMeasAvailable_01", HFILL }},
    { &hf_rrc_loggedConnectionEstablishmentFailureInfoAvailable,
      { "loggedConnectionEstablishmentFailureInfoAvailable", "rrc.loggedConnectionEstablishmentFailureInfoAvailable",
        FT_UINT32, BASE_DEC, VALS(rrc_T_loggedConnectionEstablishmentFailureInfoAvailable_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_intraDomainNasNodeSelector,
      { "intraDomainNasNodeSelector", "rrc.intraDomainNasNodeSelector",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_v3a0NonCriticalExtensions_02,
      { "v3a0NonCriticalExtensions", "rrc.v3a0NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v3a0NonCriticalExtensions_02", HFILL }},
    { &hf_rrc_initialDirectTransfer_v3a0ext,
      { "initialDirectTransfer-v3a0ext", "rrc.initialDirectTransfer_v3a0ext",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_laterNonCriticalExtensions_13,
      { "laterNonCriticalExtensions", "rrc.laterNonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_laterNonCriticalExtensions_13", HFILL }},
    { &hf_rrc_initialDirectTransfer_r3_add_ext,
      { "initialDirectTransfer-r3-add-ext", "rrc.initialDirectTransfer_r3_add_ext",
        FT_BYTES, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_v590NonCriticalExtensions_05,
      { "v590NonCriticalExtensions", "rrc.v590NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v590NonCriticalExtensions_05", HFILL }},
    { &hf_rrc_initialDirectTransfer_v590ext,
      { "initialDirectTransfer-v590ext", "rrc.initialDirectTransfer_v590ext",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_v690NonCriticalExtensions_08,
      { "v690NonCriticalExtensions", "rrc.v690NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v690NonCriticalExtensions_08", HFILL }},
    { &hf_rrc_initialDirectTransfer_v690ext,
      { "initialDirectTransfer-v690ext", "rrc.initialDirectTransfer_v690ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "InitialDirectTransfer_v690ext_IEs", HFILL }},
    { &hf_rrc_v770NonCriticalExtensions_01,
      { "v770NonCriticalExtensions", "rrc.v770NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v770NonCriticalExtensions_01", HFILL }},
    { &hf_rrc_initialDirectTransfer_v770ext,
      { "initialDirectTransfer-v770ext", "rrc.initialDirectTransfer_v770ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "InitialDirectTransfer_v770ext_IEs", HFILL }},
    { &hf_rrc_v860NonCriticalExtensions_03,
      { "v860NonCriticalExtensions", "rrc.v860NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v860NonCriticalExtensions_03", HFILL }},
    { &hf_rrc_initialDirectTransfer_v860ext,
      { "initialDirectTransfer-v860ext", "rrc.initialDirectTransfer_v860ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "InitialDirectTransfer_v860ext_IEs", HFILL }},
    { &hf_rrc_vbxyNonCriticalExtensions_05,
      { "vbxyNonCriticalExtensions", "rrc.vbxyNonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_vbxyNonCriticalExtensions_05", HFILL }},
    { &hf_rrc_initialDirectTransfer_vbxyext,
      { "initialDirectTransfer-vbxyext", "rrc.initialDirectTransfer_vbxyext",
        FT_NONE, BASE_NONE, NULL, 0,
        "InitialDirectTransfer_vbxyext_IEs", HFILL }},
    { &hf_rrc_nonCriticalExtensions_47,
      { "nonCriticalExtensions", "rrc.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_nonCriticalExtensions_47", HFILL }},
    { &hf_rrc_start_Value,
      { "start-Value", "rrc.start_Value",
        FT_BYTES, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_plmn_Identity,
      { "plmn-Identity", "rrc.plmn_Identity",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_mbms_JoinedInformation,
      { "mbms-JoinedInformation", "rrc.mbms_JoinedInformation",
        FT_NONE, BASE_NONE, NULL, 0,
        "MBMS_JoinedInformation_r6", HFILL }},
    { &hf_rrc_csCallType_01,
      { "csCallType", "rrc.csCallType",
        FT_UINT32, BASE_DEC, VALS(rrc_T_csCallType_01_vals), 0,
        "T_csCallType_01", HFILL }},
    { &hf_rrc_supportOfCSG,
      { "supportOfCSG", "rrc.supportOfCSG",
        FT_UINT32, BASE_DEC, VALS(rrc_T_supportOfCSG_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_initialDirectTransfer_v7g0ext,
      { "initialDirectTransfer-v7g0ext", "rrc.initialDirectTransfer_v7g0ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "InitialDirectTransfer_v7g0ext_IEs", HFILL }},
    { &hf_rrc_nonCriticalExtensions_48,
      { "nonCriticalExtensions", "rrc.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_nonCriticalExtensions_48", HFILL }},
    { &hf_rrc_r3_09,
      { "r3", "rrc.r3",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_r3_09", HFILL }},
    { &hf_rrc_handoverFromUTRANCommand_GSM_r3,
      { "handoverFromUTRANCommand-GSM-r3", "rrc.handoverFromUTRANCommand_GSM_r3",
        FT_NONE, BASE_NONE, NULL, 0,
        "HandoverFromUTRANCommand_GSM_r3_IEs", HFILL }},
    { &hf_rrc_laterNonCriticalExtensions_14,
      { "laterNonCriticalExtensions", "rrc.laterNonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_laterNonCriticalExtensions_14", HFILL }},
    { &hf_rrc_handoverFromUTRANCommand_GSM_r3_add_ext,
      { "handoverFromUTRANCommand-GSM-r3-add-ext", "rrc.handoverFromUTRANCommand_GSM_r3_add_ext",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING", HFILL }},
    { &hf_rrc_v690NonCriticalExtensions_09,
      { "v690NonCriticalExtensions", "rrc.v690NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v690NonCriticalExtensions_09", HFILL }},
    { &hf_rrc_handoverFromUTRANCommand_GSM_v690ext,
      { "handoverFromUTRANCommand-GSM-v690ext", "rrc.handoverFromUTRANCommand_GSM_v690ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "HandoverFromUTRANCommand_GSM_v690ext_IEs", HFILL }},
    { &hf_rrc_v860NonCriticalExtensions_04,
      { "v860NonCriticalExtensions", "rrc.v860NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v860NonCriticalExtensions_04", HFILL }},
    { &hf_rrc_handoverFromUTRANCommand_GSM_v860ext,
      { "handoverFromUTRANCommand-GSM-v860ext", "rrc.handoverFromUTRANCommand_GSM_v860ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "HandoverFromUTRANCommand_GSM_v860ext_IEs", HFILL }},
    { &hf_rrc_nonCriticalExtensions_49,
      { "nonCriticalExtensions", "rrc.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_nonCriticalExtensions_49", HFILL }},
    { &hf_rrc_later_than_r3_07,
      { "later-than-r3", "rrc.later_than_r3",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_later_than_r3_07", HFILL }},
    { &hf_rrc_criticalExtensions_39,
      { "criticalExtensions", "rrc.criticalExtensions",
        FT_UINT32, BASE_DEC, VALS(rrc_T_criticalExtensions_38_vals), 0,
        "T_criticalExtensions_38", HFILL }},
    { &hf_rrc_r6_04,
      { "r6", "rrc.r6",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_r6_04", HFILL }},
    { &hf_rrc_handoverFromUTRANCommand_GSM_r6,
      { "handoverFromUTRANCommand-GSM-r6", "rrc.handoverFromUTRANCommand_GSM_r6",
        FT_NONE, BASE_NONE, NULL, 0,
        "HandoverFromUTRANCommand_GSM_r6_IEs", HFILL }},
    { &hf_rrc_handoverFromUTRANCommand_GSM_r6_add_ext,
      { "handoverFromUTRANCommand-GSM-r6-add-ext", "rrc.handoverFromUTRANCommand_GSM_r6_add_ext",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING", HFILL }},
    { &hf_rrc_v860NonCriticalExtensions_05,
      { "v860NonCriticalExtensions", "rrc.v860NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v860NonCriticalExtensions_05", HFILL }},
    { &hf_rrc_nonCriticalExtensions_50,
      { "nonCriticalExtensions", "rrc.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_nonCriticalExtensions_50", HFILL }},
    { &hf_rrc_criticalExtensions_40,
      { "criticalExtensions", "rrc.criticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_criticalExtensions_39", HFILL }},
    { &hf_rrc_toHandoverRAB_Info,
      { "toHandoverRAB-Info", "rrc.toHandoverRAB_Info",
        FT_NONE, BASE_NONE, NULL, 0,
        "RAB_Info", HFILL }},
    { &hf_rrc_frequency_band,
      { "frequency-band", "rrc.frequency_band",
        FT_UINT32, BASE_DEC, VALS(rrc_Frequency_Band_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_gsm_message,
      { "gsm-message", "rrc.gsm_message",
        FT_UINT32, BASE_DEC, VALS(rrc_T_gsm_message_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_single_GSM_Message_r3,
      { "single-GSM-Message", "rrc.single_GSM_Message",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_single_GSM_Message_r3", HFILL }},
    { &hf_rrc_gsm_MessageList_r3,
      { "gsm-MessageList", "rrc.gsm_MessageList",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_gsm_MessageList_r3", HFILL }},
    { &hf_rrc_gsm_Messages,
      { "gsm-Messages", "rrc.gsm_Messages",
        FT_UINT32, BASE_DEC, NULL, 0,
        "GSM_MessageList", HFILL }},
    { &hf_rrc_toHandoverRAB_Info_01,
      { "toHandoverRAB-Info", "rrc.toHandoverRAB_Info",
        FT_UINT32, BASE_DEC, NULL, 0,
        "RAB_InformationList_r6", HFILL }},
    { &hf_rrc_gsm_message_01,
      { "gsm-message", "rrc.gsm_message",
        FT_UINT32, BASE_DEC, VALS(rrc_T_gsm_message_01_vals), 0,
        "T_gsm_message_01", HFILL }},
    { &hf_rrc_single_GSM_Message_r6,
      { "single-GSM-Message", "rrc.single_GSM_Message",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_single_GSM_Message_r6", HFILL }},
    { &hf_rrc_gsm_MessageList_r6,
      { "gsm-MessageList", "rrc.gsm_MessageList",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_gsm_MessageList_r6", HFILL }},
    { &hf_rrc_geran_SystemInfoType_01,
      { "geran-SystemInfoType", "rrc.geran_SystemInfoType",
        FT_UINT32, BASE_DEC, VALS(rrc_T_geran_SystemInfoType_01_vals), 0,
        "T_geran_SystemInfoType_01", HFILL }},
    { &hf_rrc_geran_SystemInfoType_02,
      { "geran-SystemInfoType", "rrc.geran_SystemInfoType",
        FT_UINT32, BASE_DEC, VALS(rrc_T_geran_SystemInfoType_02_vals), 0,
        "T_geran_SystemInfoType_02", HFILL }},
    { &hf_rrc_sr_vcc_SecurityRABInfo,
      { "sr-vcc-SecurityRABInfo", "rrc.sr_vcc_SecurityRABInfo",
        FT_NONE, BASE_NONE, NULL, 0,
        "SR_VCC_SecurityRABInfo_v860ext", HFILL }},
    { &hf_rrc_handoverFromUTRANCommand_GERANIu_01,
      { "handoverFromUTRANCommand-GERANIu", "rrc.handoverFromUTRANCommand_GERANIu",
        FT_UINT32, BASE_DEC, VALS(rrc_T_handoverFromUTRANCommand_GERANIu_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_r5_03,
      { "r5", "rrc.r5",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_r5_03", HFILL }},
    { &hf_rrc_handoverFromUTRANCommand_GERANIu_r5,
      { "handoverFromUTRANCommand-GERANIu-r5", "rrc.handoverFromUTRANCommand_GERANIu_r5",
        FT_NONE, BASE_NONE, NULL, 0,
        "HandoverFromUTRANCommand_GERANIu_r5_IEs", HFILL }},
    { &hf_rrc_nonCriticalExtensions_51,
      { "nonCriticalExtensions", "rrc.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_nonCriticalExtensions_51", HFILL }},
    { &hf_rrc_later_than_r5,
      { "later-than-r5", "rrc.later_than_r5",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_criticalExtensions_41,
      { "criticalExtensions", "rrc.criticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_criticalExtensions_40", HFILL }},
    { &hf_rrc_frequency_Band,
      { "frequency-Band", "rrc.frequency_Band",
        FT_UINT32, BASE_DEC, VALS(rrc_Frequency_Band_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_geranIu_Message,
      { "geranIu-Message", "rrc.geranIu_Message",
        FT_UINT32, BASE_DEC, VALS(rrc_T_geranIu_Message_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_single_GERANIu_Message,
      { "single-GERANIu-Message", "rrc.single_GERANIu_Message",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_geranIu_MessageList,
      { "geranIu-MessageList", "rrc.geranIu_MessageList",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_geranIu_Messages,
      { "geranIu-Messages", "rrc.geranIu_Messages",
        FT_UINT32, BASE_DEC, NULL, 0,
        "GERANIu_MessageList", HFILL }},
    { &hf_rrc_r3_10,
      { "r3", "rrc.r3",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_r3_10", HFILL }},
    { &hf_rrc_handoverFromUTRANCommand_CDMA2000_r3,
      { "handoverFromUTRANCommand-CDMA2000-r3", "rrc.handoverFromUTRANCommand_CDMA2000_r3",
        FT_NONE, BASE_NONE, NULL, 0,
        "HandoverFromUTRANCommand_CDMA2000_r3_IEs", HFILL }},
    { &hf_rrc_laterNonCriticalExtensions_15,
      { "laterNonCriticalExtensions", "rrc.laterNonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_laterNonCriticalExtensions_15", HFILL }},
    { &hf_rrc_handoverFromUTRANCommand_CDMA2000_r3_add_ext,
      { "handoverFromUTRANCommand-CDMA2000-r3-add-ext", "rrc.handoverFromUTRANCommand_CDMA2000_r3_add_ext",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING", HFILL }},
    { &hf_rrc_nonCriticalExtensions_52,
      { "nonCriticalExtensions", "rrc.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_nonCriticalExtensions_52", HFILL }},
    { &hf_rrc_later_than_r3_08,
      { "later-than-r3", "rrc.later_than_r3",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_later_than_r3_08", HFILL }},
    { &hf_rrc_criticalExtensions_42,
      { "criticalExtensions", "rrc.criticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_criticalExtensions_41", HFILL }},
    { &hf_rrc_cdma2000_MessageList,
      { "cdma2000-MessageList", "rrc.cdma2000_MessageList",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_criticalExtensions_43,
      { "criticalExtensions", "rrc.criticalExtensions",
        FT_UINT32, BASE_DEC, VALS(rrc_T_criticalExtensions_42_vals), 0,
        "T_criticalExtensions_42", HFILL }},
    { &hf_rrc_r8_04,
      { "r8", "rrc.r8",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_r8_04", HFILL }},
    { &hf_rrc_handoverFromUTRANCommand_EUTRA_r8,
      { "handoverFromUTRANCommand-EUTRA-r8", "rrc.handoverFromUTRANCommand_EUTRA_r8",
        FT_NONE, BASE_NONE, NULL, 0,
        "HandoverFromUTRANCommand_EUTRA_r8_IEs", HFILL }},
    { &hf_rrc_handoverFromUTRANCommand_EUTRA_r8_add_ext,
      { "handoverFromUTRANCommand-EUTRA-r8-add-ext", "rrc.handoverFromUTRANCommand_EUTRA_r8_add_ext",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING", HFILL }},
    { &hf_rrc_nonCriticalExtensions_53,
      { "nonCriticalExtensions", "rrc.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_nonCriticalExtensions_53", HFILL }},
    { &hf_rrc_criticalExtensions_44,
      { "criticalExtensions", "rrc.criticalExtensions",
        FT_UINT32, BASE_DEC, VALS(rrc_T_criticalExtensions_43_vals), 0,
        "T_criticalExtensions_43", HFILL }},
    { &hf_rrc_r11_04,
      { "r11", "rrc.r11",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_r11_04", HFILL }},
    { &hf_rrc_handoverFromUTRANCommand_EUTRA_r11,
      { "handoverFromUTRANCommand-EUTRA-r11", "rrc.handoverFromUTRANCommand_EUTRA_r11",
        FT_NONE, BASE_NONE, NULL, 0,
        "HandoverFromUTRANCommand_EUTRA_r11_IEs", HFILL }},
    { &hf_rrc_handoverFromUTRANCommand_EUTRA_r11_add_ext,
      { "handoverFromUTRANCommand-EUTRA-r11-add-ext", "rrc.handoverFromUTRANCommand_EUTRA_r11_add_ext",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING", HFILL }},
    { &hf_rrc_nonCriticalExtensions_54,
      { "nonCriticalExtensions", "rrc.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_nonCriticalExtensions_54", HFILL }},
    { &hf_rrc_criticalExtensions_45,
      { "criticalExtensions", "rrc.criticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_criticalExtensions_44", HFILL }},
    { &hf_rrc_toHandoverRAB_Info_02,
      { "toHandoverRAB-Info", "rrc.toHandoverRAB_Info",
        FT_UINT32, BASE_DEC, NULL, 0,
        "RAB_InformationList", HFILL }},
    { &hf_rrc_eutra_Message,
      { "eutra-Message", "rrc.eutra_Message",
        FT_BYTES, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_rsr_vcc_Info,
      { "rsr-vcc-Info", "rrc.rsr_vcc_Info",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_eutra_Message_01,
      { "eutra-Message", "rrc.eutra_Message",
        FT_BYTES, BASE_NONE, NULL, 0,
        "T_eutra_Message_01", HFILL }},
    { &hf_rrc_interRAT_HO_FailureCause,
      { "interRAT-HO-FailureCause", "rrc.interRAT_HO_FailureCause",
        FT_UINT32, BASE_DEC, VALS(rrc_InterRAT_HO_FailureCause_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_interRATMessage,
      { "interRATMessage", "rrc.interRATMessage",
        FT_UINT32, BASE_DEC, VALS(rrc_T_interRATMessage_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_gsm,
      { "gsm", "rrc.gsm",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_gsm_MessageList_gsm,
      { "gsm-MessageList", "rrc.gsm_MessageList",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_cdma2000,
      { "cdma2000", "rrc.cdma2000",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_laterNonCriticalExtensions_16,
      { "laterNonCriticalExtensions", "rrc.laterNonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_laterNonCriticalExtensions_16", HFILL }},
    { &hf_rrc_handoverFromUTRANFailure_r3_add_ext,
      { "handoverFromUTRANFailure-r3-add-ext", "rrc.handoverFromUTRANFailure_r3_add_ext",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING", HFILL }},
    { &hf_rrc_v590NonCriticalExtensions_06,
      { "v590NonCriticalExtensions", "rrc.v590NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v590NonCriticalExtensions_06", HFILL }},
    { &hf_rrc_handoverFromUTRANFailure_v590ext,
      { "handoverFromUTRANFailure-v590ext", "rrc.handoverFromUTRANFailure_v590ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "HandoverFromUtranFailure_v590ext_IEs", HFILL }},
    { &hf_rrc_v860NonCriticalExtensions_06,
      { "v860NonCriticalExtensions", "rrc.v860NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v860NonCriticalExtensions_06", HFILL }},
    { &hf_rrc_handoverFromUTRANFailure_v860ext,
      { "handoverFromUTRANFailure-v860ext", "rrc.handoverFromUTRANFailure_v860ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "HandoverFromUtranFailure_v860ext_IEs", HFILL }},
    { &hf_rrc_nonCriticalExtensions_55,
      { "nonCriticalExtensions", "rrc.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_nonCriticalExtensions_55", HFILL }},
    { &hf_rrc_geranIu_MessageList_01,
      { "geranIu-MessageList", "rrc.geranIu_MessageList",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_eutra_Message_02,
      { "eutra-Message", "rrc.eutra_Message",
        FT_BYTES, BASE_NONE, NULL, 0,
        "T_eutra_Message_02", HFILL }},
    { &hf_rrc_predefinedConfigStatusList,
      { "predefinedConfigStatusList", "rrc.predefinedConfigStatusList",
        FT_UINT32, BASE_DEC, VALS(rrc_T_predefinedConfigStatusList_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_absent,
      { "absent", "rrc.absent",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_present,
      { "present", "rrc.present",
        FT_UINT32, BASE_DEC, NULL, 0,
        "PredefinedConfigStatusList", HFILL }},
    { &hf_rrc_uE_SecurityInformation,
      { "uE-SecurityInformation", "rrc.uE_SecurityInformation",
        FT_UINT32, BASE_DEC, VALS(rrc_T_uE_SecurityInformation_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_present_01,
      { "present", "rrc.present",
        FT_NONE, BASE_NONE, NULL, 0,
        "UE_SecurityInformation", HFILL }},
    { &hf_rrc_ue_CapabilityContainer,
      { "ue-CapabilityContainer", "rrc.ue_CapabilityContainer",
        FT_UINT32, BASE_DEC, VALS(rrc_T_ue_CapabilityContainer_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_present_02,
      { "present", "rrc.present",
        FT_BYTES, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_v390NonCriticalExtensions,
      { "v390NonCriticalExtensions", "rrc.v390NonCriticalExtensions",
        FT_UINT32, BASE_DEC, VALS(rrc_T_v390NonCriticalExtensions_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_present_03,
      { "present", "rrc.present",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_present_01", HFILL }},
    { &hf_rrc_interRATHandoverInfo_v390ext,
      { "interRATHandoverInfo-v390ext", "rrc.interRATHandoverInfo_v390ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "InterRATHandoverInfo_v390ext_IEs", HFILL }},
    { &hf_rrc_v3a0NonCriticalExtensions_03,
      { "v3a0NonCriticalExtensions", "rrc.v3a0NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v3a0NonCriticalExtensions_03", HFILL }},
    { &hf_rrc_interRATHandoverInfo_v3a0ext,
      { "interRATHandoverInfo-v3a0ext", "rrc.interRATHandoverInfo_v3a0ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "InterRATHandoverInfo_v3a0ext_IEs", HFILL }},
    { &hf_rrc_laterNonCriticalExtensions_17,
      { "laterNonCriticalExtensions", "rrc.laterNonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_laterNonCriticalExtensions_17", HFILL }},
    { &hf_rrc_interRATHandoverInfo_v3d0ext,
      { "interRATHandoverInfo-v3d0ext", "rrc.interRATHandoverInfo_v3d0ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "InterRATHandoverInfo_v3d0ext_IEs", HFILL }},
    { &hf_rrc_interRATHandoverInfo_r3_add_ext,
      { "interRATHandoverInfo-r3-add-ext", "rrc.interRATHandoverInfo_r3_add_ext",
        FT_BYTES, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_v3g0NonCriticalExtensions,
      { "v3g0NonCriticalExtensions", "rrc.v3g0NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_interRATHandoverInfo_v3g0ext,
      { "interRATHandoverInfo-v3g0ext", "rrc.interRATHandoverInfo_v3g0ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "InterRATHandoverInfo_v3g0ext_IEs", HFILL }},
    { &hf_rrc_v4b0NonCriticalExtensions_04,
      { "v4b0NonCriticalExtensions", "rrc.v4b0NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v4b0NonCriticalExtensions_04", HFILL }},
    { &hf_rrc_interRATHandoverInfo_v4b0ext,
      { "interRATHandoverInfo-v4b0ext", "rrc.interRATHandoverInfo_v4b0ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "InterRATHandoverInfo_v4b0ext_IEs", HFILL }},
    { &hf_rrc_v4d0NonCriticalExtensions_02,
      { "v4d0NonCriticalExtensions", "rrc.v4d0NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v4d0NonCriticalExtensions_02", HFILL }},
    { &hf_rrc_interRATHandoverInfo_v4d0ext,
      { "interRATHandoverInfo-v4d0ext", "rrc.interRATHandoverInfo_v4d0ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "InterRATHandoverInfo_v4d0ext_IEs", HFILL }},
    { &hf_rrc_v590NonCriticalExtensions_07,
      { "v590NonCriticalExtensions", "rrc.v590NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v590NonCriticalExtensions_07", HFILL }},
    { &hf_rrc_interRATHandoverInfo_v590ext,
      { "interRATHandoverInfo-v590ext", "rrc.interRATHandoverInfo_v590ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "InterRATHandoverInfo_v590ext_IEs", HFILL }},
    { &hf_rrc_v690NonCriticalExtensions_10,
      { "v690NonCriticalExtensions", "rrc.v690NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v690NonCriticalExtensions_10", HFILL }},
    { &hf_rrc_interRATHandoverInfo_v690ext,
      { "interRATHandoverInfo-v690ext", "rrc.interRATHandoverInfo_v690ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "InterRATHandoverInfo_v690ext_IEs", HFILL }},
    { &hf_rrc_v6b0NonCriticalExtensions_05,
      { "v6b0NonCriticalExtensions", "rrc.v6b0NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "InterRATHandoverInfo_NonCriticalExtension_v6b0_IEs", HFILL }},
    { &hf_rrc_ue_RadioAccessCapability_v380ext,
      { "ue-RadioAccessCapability-v380ext", "rrc.ue_RadioAccessCapability_v380ext",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_dl_PhysChCapabilityFDD_v380ext,
      { "dl-PhysChCapabilityFDD-v380ext", "rrc.dl_PhysChCapabilityFDD_v380ext",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_ue_RadioAccessCapability_v3a0ext,
      { "ue-RadioAccessCapability-v3a0ext", "rrc.ue_RadioAccessCapability_v3a0ext",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_uESpecificBehaviourInformation1interRAT,
      { "uESpecificBehaviourInformation1interRAT", "rrc.uESpecificBehaviourInformation1interRAT",
        FT_BYTES, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_ue_RadioAccessCapability_v3g0ext,
      { "ue-RadioAccessCapability-v3g0ext", "rrc.ue_RadioAccessCapability_v3g0ext",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_interRATHandoverInfo_v690ext1,
      { "interRATHandoverInfo-v690ext1", "rrc.interRATHandoverInfo_v690ext1",
        FT_NONE, BASE_NONE, NULL, 0,
        "InterRATHandoverInfo_v690ext1_IEs", HFILL }},
    { &hf_rrc_v7e0NonCriticalExtensions,
      { "v7e0NonCriticalExtensions", "rrc.v7e0NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_interRATHandoverInfo_v7e0ext,
      { "interRATHandoverInfo-v7e0ext", "rrc.interRATHandoverInfo_v7e0ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "InterRATHandoverInfo_v7e0ext_IEs", HFILL }},
    { &hf_rrc_v7f0NonCriticalExtensions_04,
      { "v7f0NonCriticalExtensions", "rrc.v7f0NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v7f0NonCriticalExtensions_04", HFILL }},
    { &hf_rrc_interRATHandoverInfo_v7f0ext,
      { "interRATHandoverInfo-v7f0ext", "rrc.interRATHandoverInfo_v7f0ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "InterRATHandoverInfo_v7f0ext_IEs", HFILL }},
    { &hf_rrc_va40NonCriticalExtensions_02,
      { "va40NonCriticalExtensions", "rrc.va40NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_va40NonCriticalExtensions_02", HFILL }},
    { &hf_rrc_interRATHandoverInfo_va40ext,
      { "interRATHandoverInfo-va40ext", "rrc.interRATHandoverInfo_va40ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "InterRATHandoverInfoBand_va40ext_IEs", HFILL }},
    { &hf_rrc_nonCriticalExtensions_56,
      { "nonCriticalExtensions", "rrc.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_nonCriticalExtensions_56", HFILL }},
    { &hf_rrc_accessStratumReleaseIndicator,
      { "accessStratumReleaseIndicator", "rrc.accessStratumReleaseIndicator",
        FT_UINT32, BASE_DEC, VALS(rrc_AccessStratumReleaseIndicator_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_tdd128_RF_Capability,
      { "tdd128-RF-Capability", "rrc.tdd128_RF_Capability",
        FT_UINT32, BASE_DEC, VALS(rrc_RadioFrequencyBandTDDList_vals), 0,
        "RadioFrequencyBandTDDList", HFILL }},
    { &hf_rrc_predefinedConfigStatusListComp,
      { "predefinedConfigStatusListComp", "rrc.predefinedConfigStatusListComp",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_ue_RadioAccessCapabilityComp,
      { "ue-RadioAccessCapabilityComp", "rrc.ue_RadioAccessCapabilityComp",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_ue_RadioAccessCapability_v650ext,
      { "ue-RadioAccessCapability-v650ext", "rrc.ue_RadioAccessCapability_v650ext",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_ue_SecurityInformation2,
      { "ue-SecurityInformation2", "rrc.ue_SecurityInformation2",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_ue_RadioAccessCapabilityComp_01,
      { "ue-RadioAccessCapabilityComp", "rrc.ue_RadioAccessCapabilityComp",
        FT_NONE, BASE_NONE, NULL, 0,
        "UE_RadioAccessCapabilityComp_ext", HFILL }},
    { &hf_rrc_ue_RadioAccessCapabilityComp2,
      { "ue-RadioAccessCapabilityComp2", "rrc.ue_RadioAccessCapabilityComp2",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_interRATHandoverInfo_v6b0ext,
      { "interRATHandoverInfo-v6b0ext", "rrc.interRATHandoverInfo_v6b0ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "InterRATHandoverInfo_v6b0ext_IEs", HFILL }},
    { &hf_rrc_v6e0NonCriticalExtensions,
      { "v6e0NonCriticalExtensions", "rrc.v6e0NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_interRATHandoverInfo_v6e0ext,
      { "interRATHandoverInfo-v6e0ext", "rrc.interRATHandoverInfo_v6e0ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "InterRATHandoverInfo_v6e0ext_IEs", HFILL }},
    { &hf_rrc_v770NonCriticalExtensions_02,
      { "v770NonCriticalExtensions", "rrc.v770NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v770NonCriticalExtensions_02", HFILL }},
    { &hf_rrc_interRATHandoverInfo_v770ext,
      { "interRATHandoverInfo-v770ext", "rrc.interRATHandoverInfo_v770ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "InterRATHandoverInfo_v770ext_IEs", HFILL }},
    { &hf_rrc_v790nonCriticalExtensions,
      { "v790nonCriticalExtensions", "rrc.v790nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_interRATHandoverInfo_v790ext,
      { "interRATHandoverInfo-v790ext", "rrc.interRATHandoverInfo_v790ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "InterRATHandoverInfo_v790ext_IEs", HFILL }},
    { &hf_rrc_v860NonCriticalExtensions_07,
      { "v860NonCriticalExtensions", "rrc.v860NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v860NonCriticalExtensions_07", HFILL }},
    { &hf_rrc_interRATHandoverInfo_v860ext,
      { "interRATHandoverInfo-v860ext", "rrc.interRATHandoverInfo_v860ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "InterRATHandoverInfo_v860ext_IEs", HFILL }},
    { &hf_rrc_v880NonCriticalExtensions,
      { "v880NonCriticalExtensions", "rrc.v880NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_interRATHandoverInfo_v880ext,
      { "interRATHandoverInfo-v880ext", "rrc.interRATHandoverInfo_v880ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "InterRATHandoverInfo_v880ext_IEs", HFILL }},
    { &hf_rrc_v920NonCriticalExtensions,
      { "v920NonCriticalExtensions", "rrc.v920NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_interRATHandoverInfo_v920ext,
      { "interRATHandoverInfo-v920ext", "rrc.interRATHandoverInfo_v920ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "InterRATHandoverInfo_v920ext_IEs", HFILL }},
    { &hf_rrc_v8b0NonCriticalExtensions,
      { "v8b0NonCriticalExtensions", "rrc.v8b0NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_interRATHandoverInfo_v8b0ext,
      { "interRATHandoverInfo-v8b0ext", "rrc.interRATHandoverInfo_v8b0ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "InterRATHandoverInfo_v8b0ext_IEs", HFILL }},
    { &hf_rrc_v950NonCriticalExtensions_02,
      { "v950NonCriticalExtensions", "rrc.v950NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v950NonCriticalExtensions_02", HFILL }},
    { &hf_rrc_interRATHandoverInfo_v950ext,
      { "interRATHandoverInfo-v950ext", "rrc.interRATHandoverInfo_v950ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "InterRATHandoverInfo_v950ext_IEs", HFILL }},
    { &hf_rrc_va40NonCriticalExtensions_03,
      { "va40NonCriticalExtensions", "rrc.va40NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_va40NonCriticalExtensions_03", HFILL }},
    { &hf_rrc_interRATHandoverInfo_va40ext_01,
      { "interRATHandoverInfo-va40ext", "rrc.interRATHandoverInfo_va40ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "InterRATHandoverInfo_va40ext_IEs", HFILL }},
    { &hf_rrc_va80NonCriticalExtensions,
      { "va80NonCriticalExtensions", "rrc.va80NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_interRATHandoverInfo_va80ext,
      { "interRATHandoverInfo-va80ext", "rrc.interRATHandoverInfo_va80ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "InterRATHandoverInfo_va80ext_IEs", HFILL }},
    { &hf_rrc_vbxyNonCriticalExtensions_06,
      { "vbxyNonCriticalExtensions", "rrc.vbxyNonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_vbxyNonCriticalExtensions_06", HFILL }},
    { &hf_rrc_interRATHandoverInfo_vbxyext,
      { "interRATHandoverInfo-vbxyext", "rrc.interRATHandoverInfo_vbxyext",
        FT_NONE, BASE_NONE, NULL, 0,
        "InterRATHandoverInfo_vbxyext_IEs", HFILL }},
    { &hf_rrc_nonCriticalExtensions_57,
      { "nonCriticalExtensions", "rrc.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_nonCriticalExtensions_57", HFILL }},
    { &hf_rrc_supportForSIB11bis,
      { "supportForSIB11bis", "rrc.supportForSIB11bis",
        FT_UINT32, BASE_DEC, VALS(rrc_T_supportForSIB11bis_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_supportForFDPCH,
      { "supportForFDPCH", "rrc.supportForFDPCH",
        FT_UINT32, BASE_DEC, VALS(rrc_T_supportForFDPCH_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_ue_RadioAccessCapabilityInfo,
      { "ue-RadioAccessCapabilityInfo", "rrc.ue_RadioAccessCapabilityInfo",
        FT_NONE, BASE_NONE, NULL, 0,
        "UE_RadioAccessCapabilityInfo_v770ext", HFILL }},
    { &hf_rrc_ue_RadioAccessCapabilityComp_02,
      { "ue-RadioAccessCapabilityComp", "rrc.ue_RadioAccessCapabilityComp",
        FT_NONE, BASE_NONE, NULL, 0,
        "UE_RadioAccessCapabilityComp_v770ext", HFILL }},
    { &hf_rrc_ue_RadioAccessCapabilityComp2_01,
      { "ue-RadioAccessCapabilityComp2", "rrc.ue_RadioAccessCapabilityComp2",
        FT_NONE, BASE_NONE, NULL, 0,
        "UE_RadioAccessCapabilityComp2_v770ext", HFILL }},
    { &hf_rrc_supportForEDPCCHPowerBoosting,
      { "supportForEDPCCHPowerBoosting", "rrc.supportForEDPCCHPowerBoosting",
        FT_UINT32, BASE_DEC, VALS(rrc_T_supportForEDPCCHPowerBoosting_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_ue_RadioAccessCapability,
      { "ue-RadioAccessCapability", "rrc.ue_RadioAccessCapability",
        FT_NONE, BASE_NONE, NULL, 0,
        "UE_RadioAccessCapability_v7e0ext", HFILL }},
    { &hf_rrc_ue_RadioAccessCapability_01,
      { "ue-RadioAccessCapability", "rrc.ue_RadioAccessCapability",
        FT_NONE, BASE_NONE, NULL, 0,
        "UE_RadioAccessCapability_v7f0ext", HFILL }},
    { &hf_rrc_ue_RadioAccessCapabilityComp2_02,
      { "ue-RadioAccessCapabilityComp2", "rrc.ue_RadioAccessCapabilityComp2",
        FT_NONE, BASE_NONE, NULL, 0,
        "UE_RadioAccessCapabilityComp2_v7f0ext", HFILL }},
    { &hf_rrc_ue_RadioAccessCapabilityInfo_01,
      { "ue-RadioAccessCapabilityInfo", "rrc.ue_RadioAccessCapabilityInfo",
        FT_NONE, BASE_NONE, NULL, 0,
        "UE_RadioAccessCapability_v860ext", HFILL }},
    { &hf_rrc_ue_RadioAccessCapabilityComp2_03,
      { "ue-RadioAccessCapabilityComp2", "rrc.ue_RadioAccessCapabilityComp2",
        FT_NONE, BASE_NONE, NULL, 0,
        "UE_RadioAccessCapabilityComp2_v860ext", HFILL }},
    { &hf_rrc_ue_RadioAccessCapabilityInfo_02,
      { "ue-RadioAccessCapabilityInfo", "rrc.ue_RadioAccessCapabilityInfo",
        FT_NONE, BASE_NONE, NULL, 0,
        "UE_RadioAccessCapability_v880ext", HFILL }},
    { &hf_rrc_ue_RadioAccessCapabilityInfo_TDD128,
      { "ue-RadioAccessCapabilityInfo-TDD128", "rrc.ue_RadioAccessCapabilityInfo_TDD128",
        FT_NONE, BASE_NONE, NULL, 0,
        "UE_RadioAccessCapabilityComp_TDD128", HFILL }},
    { &hf_rrc_ue_RadioAccessCapabilityInfo_03,
      { "ue-RadioAccessCapabilityInfo", "rrc.ue_RadioAccessCapabilityInfo",
        FT_NONE, BASE_NONE, NULL, 0,
        "UE_RadioAccessCapability_v920ext", HFILL }},
    { &hf_rrc_ue_RadioAccessCapabilityComp2_04,
      { "ue-RadioAccessCapabilityComp2", "rrc.ue_RadioAccessCapabilityComp2",
        FT_NONE, BASE_NONE, NULL, 0,
        "UE_RadioAccessCapabilityComp2_v920ext", HFILL }},
    { &hf_rrc_ue_RadioAccessCapabilityInfo_04,
      { "ue-RadioAccessCapabilityInfo", "rrc.ue_RadioAccessCapabilityInfo",
        FT_NONE, BASE_NONE, NULL, 0,
        "UE_RadioAccessCapabilityInfo_TDD128_v8b0ext", HFILL }},
    { &hf_rrc_pdcp_Capability,
      { "pdcp-Capability", "rrc.pdcp_Capability",
        FT_NONE, BASE_NONE, NULL, 0,
        "PDCP_Capability_r4_ext", HFILL }},
    { &hf_rrc_ue_RadioAccessCapabilityInfo_05,
      { "ue-RadioAccessCapabilityInfo", "rrc.ue_RadioAccessCapabilityInfo",
        FT_NONE, BASE_NONE, NULL, 0,
        "UE_RadioAccessCapability_va40ext", HFILL }},
    { &hf_rrc_ue_RadioAccessCapabilityComp2_05,
      { "ue-RadioAccessCapabilityComp2", "rrc.ue_RadioAccessCapabilityComp2",
        FT_NONE, BASE_NONE, NULL, 0,
        "UE_RadioAccessCapabilityComp2_va40ext", HFILL }},
    { &hf_rrc_ue_RadioAccessCapabBand,
      { "ue-RadioAccessCapabBand", "rrc.ue_RadioAccessCapabBand",
        FT_NONE, BASE_NONE, NULL, 0,
        "UE_RadioAccessCapabBand_va40ext", HFILL }},
    { &hf_rrc_rf_CapabilityFDDComp,
      { "rf-CapabilityFDDComp", "rrc.rf_CapabilityFDDComp",
        FT_UINT32, BASE_DEC, NULL, 0,
        "RF_CapabBandListFDDComp_ext2", HFILL }},
    { &hf_rrc_ue_RadioAccessCapabilityInfo_va80ext,
      { "ue-RadioAccessCapabilityInfo-va80ext", "rrc.ue_RadioAccessCapabilityInfo_va80ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "UE_RadioAccessCapability_va80ext", HFILL }},
    { &hf_rrc_ue_RadioAccessCapabilityInfo_vbxyext,
      { "ue-RadioAccessCapabilityInfo-vbxyext", "rrc.ue_RadioAccessCapabilityInfo_vbxyext",
        FT_NONE, BASE_NONE, NULL, 0,
        "UE_RadioAccessCapability_vbxyext", HFILL }},
    { &hf_rrc_ue_RadioAccessCapabilityComp2_06,
      { "ue-RadioAccessCapabilityComp2", "rrc.ue_RadioAccessCapabilityComp2",
        FT_NONE, BASE_NONE, NULL, 0,
        "UE_RadioAccessCapabilityComp2_vbxyext", HFILL }},
    { &hf_rrc_r3_11,
      { "r3", "rrc.r3",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_r3_11", HFILL }},
    { &hf_rrc_measurementControl_r3,
      { "measurementControl-r3", "rrc.measurementControl_r3",
        FT_NONE, BASE_NONE, NULL, 0,
        "MeasurementControl_r3_IEs", HFILL }},
    { &hf_rrc_v390nonCriticalExtensions,
      { "v390nonCriticalExtensions", "rrc.v390nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_measurementControl_v390ext,
      { "measurementControl-v390ext", "rrc.measurementControl_v390ext",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_v3a0NonCriticalExtensions_04,
      { "v3a0NonCriticalExtensions", "rrc.v3a0NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v3a0NonCriticalExtensions_04", HFILL }},
    { &hf_rrc_measurementControl_v3a0ext,
      { "measurementControl-v3a0ext", "rrc.measurementControl_v3a0ext",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_laterNonCriticalExtensions_18,
      { "laterNonCriticalExtensions", "rrc.laterNonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_laterNonCriticalExtensions_18", HFILL }},
    { &hf_rrc_measurementControl_r3_add_ext,
      { "measurementControl-r3-add-ext", "rrc.measurementControl_r3_add_ext",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING", HFILL }},
    { &hf_rrc_v4b0NonCriticalExtensions_05,
      { "v4b0NonCriticalExtensions", "rrc.v4b0NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v4b0NonCriticalExtensions_05", HFILL }},
    { &hf_rrc_v590NonCriticalExtensions_08,
      { "v590NonCriticalExtensions", "rrc.v590NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v590NonCriticalExtensions_08", HFILL }},
    { &hf_rrc_measurementControl_v590ext,
      { "measurementControl-v590ext", "rrc.measurementControl_v590ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "MeasurementControl_v590ext_IEs", HFILL }},
    { &hf_rrc_v5b0NonCriticalExtensions,
      { "v5b0NonCriticalExtensions", "rrc.v5b0NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_measurementControl_v5b0ext,
      { "measurementControl-v5b0ext", "rrc.measurementControl_v5b0ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "MeasurementControl_v5b0ext_IEs", HFILL }},
    { &hf_rrc_nonCriticalExtensions_58,
      { "nonCriticalExtensions", "rrc.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_nonCriticalExtensions_58", HFILL }},
    { &hf_rrc_later_than_r3_09,
      { "later-than-r3", "rrc.later_than_r3",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_later_than_r3_09", HFILL }},
    { &hf_rrc_criticalExtensions_46,
      { "criticalExtensions", "rrc.criticalExtensions",
        FT_UINT32, BASE_DEC, VALS(rrc_T_criticalExtensions_45_vals), 0,
        "T_criticalExtensions_45", HFILL }},
    { &hf_rrc_r4_03,
      { "r4", "rrc.r4",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_r4_03", HFILL }},
    { &hf_rrc_measurementControl_r4,
      { "measurementControl-r4", "rrc.measurementControl_r4",
        FT_NONE, BASE_NONE, NULL, 0,
        "MeasurementControl_r4_IEs", HFILL }},
    { &hf_rrc_v4d0NonCriticalExtensions_03,
      { "v4d0NonCriticalExtensions", "rrc.v4d0NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v4d0NonCriticalExtensions_03", HFILL }},
    { &hf_rrc_measurementControl_r4_add_ext,
      { "measurementControl-r4-add-ext", "rrc.measurementControl_r4_add_ext",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING", HFILL }},
    { &hf_rrc_v590NonCriticalExtensions_09,
      { "v590NonCriticalExtensions", "rrc.v590NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v590NonCriticalExtensions_09", HFILL }},
    { &hf_rrc_v5b0NonCriticalExtensions_01,
      { "v5b0NonCriticalExtensions", "rrc.v5b0NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v5b0NonCriticalExtensions_01", HFILL }},
    { &hf_rrc_nonCriticalExtensions_59,
      { "nonCriticalExtensions", "rrc.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_nonCriticalExtensions_59", HFILL }},
    { &hf_rrc_later_than_r4,
      { "later-than-r4", "rrc.later_than_r4",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_rrc_TransactionIdentifier_MSP,
      { "rrc-TransactionIdentifier-MSP", "rrc.rrc_TransactionIdentifier_MSP",
        FT_UINT32, BASE_DEC, NULL, 0,
        "RRC_TransactionIdentifier", HFILL }},
    { &hf_rrc_criticalExtensions_47,
      { "criticalExtensions", "rrc.criticalExtensions",
        FT_UINT32, BASE_DEC, VALS(rrc_T_criticalExtensions_46_vals), 0,
        "T_criticalExtensions_46", HFILL }},
    { &hf_rrc_r6_05,
      { "r6", "rrc.r6",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_r6_05", HFILL }},
    { &hf_rrc_measurementControl_r6,
      { "measurementControl-r6", "rrc.measurementControl_r6",
        FT_NONE, BASE_NONE, NULL, 0,
        "MeasurementControl_r6_IEs", HFILL }},
    { &hf_rrc_v6a0NonCriticalExtensions,
      { "v6a0NonCriticalExtensions", "rrc.v6a0NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_measurementControl_v6a0ext,
      { "measurementControl-v6a0ext", "rrc.measurementControl_v6a0ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "MeasurementControl_v6a0ext_IEs", HFILL }},
    { &hf_rrc_nonCriticalExtensions_60,
      { "nonCriticalExtensions", "rrc.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_nonCriticalExtensions_60", HFILL }},
    { &hf_rrc_criticalExtensions_48,
      { "criticalExtensions", "rrc.criticalExtensions",
        FT_UINT32, BASE_DEC, VALS(rrc_T_criticalExtensions_47_vals), 0,
        "T_criticalExtensions_47", HFILL }},
    { &hf_rrc_r7_04,
      { "r7", "rrc.r7",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_r7_04", HFILL }},
    { &hf_rrc_measurementControl_r7,
      { "measurementControl-r7", "rrc.measurementControl_r7",
        FT_NONE, BASE_NONE, NULL, 0,
        "MeasurementControl_r7_IEs", HFILL }},
    { &hf_rrc_measurementControl_r7_add_ext,
      { "measurementControl-r7-add-ext", "rrc.measurementControl_r7_add_ext",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING", HFILL }},
    { &hf_rrc_v7b0NonCriticalExtensions,
      { "v7b0NonCriticalExtensions", "rrc.v7b0NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_measurementControl_v7b0ext,
      { "measurementControl-v7b0ext", "rrc.measurementControl_v7b0ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "MeasurementControl_v7b0ext_IEs", HFILL }},
    { &hf_rrc_nonCriticalExtensions_61,
      { "nonCriticalExtensions", "rrc.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_nonCriticalExtensions_61", HFILL }},
    { &hf_rrc_criticalExtensions_49,
      { "criticalExtensions", "rrc.criticalExtensions",
        FT_UINT32, BASE_DEC, VALS(rrc_T_criticalExtensions_48_vals), 0,
        "T_criticalExtensions_48", HFILL }},
    { &hf_rrc_r8_05,
      { "r8", "rrc.r8",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_r8_05", HFILL }},
    { &hf_rrc_measurementControl_r8,
      { "measurementControl-r8", "rrc.measurementControl_r8",
        FT_NONE, BASE_NONE, NULL, 0,
        "MeasurementControl_r8_IEs", HFILL }},
    { &hf_rrc_measurementControl_r8_add_ext,
      { "measurementControl-r8-add-ext", "rrc.measurementControl_r8_add_ext",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING", HFILL }},
    { &hf_rrc_v8a0NonCriticalExtensions_03,
      { "v8a0NonCriticalExtensions", "rrc.v8a0NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v8a0NonCriticalExtensions_03", HFILL }},
    { &hf_rrc_measurementControl_v8a0ext,
      { "measurementControl-v8a0ext", "rrc.measurementControl_v8a0ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "MeasurementControl_v8a0ext_IEs", HFILL }},
    { &hf_rrc_nonCriticalExtensions_62,
      { "nonCriticalExtensions", "rrc.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_nonCriticalExtensions_62", HFILL }},
    { &hf_rrc_criticalExtensions_50,
      { "criticalExtensions", "rrc.criticalExtensions",
        FT_UINT32, BASE_DEC, VALS(rrc_T_criticalExtensions_49_vals), 0,
        "T_criticalExtensions_49", HFILL }},
    { &hf_rrc_r9_04,
      { "r9", "rrc.r9",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_r9_04", HFILL }},
    { &hf_rrc_measurementControl_r9,
      { "measurementControl-r9", "rrc.measurementControl_r9",
        FT_NONE, BASE_NONE, NULL, 0,
        "MeasurementControl_r9_IEs", HFILL }},
    { &hf_rrc_measurementControl_r9_add_ext,
      { "measurementControl-r9-add-ext", "rrc.measurementControl_r9_add_ext",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING", HFILL }},
    { &hf_rrc_v970NonCriticalExtensions,
      { "v970NonCriticalExtensions", "rrc.v970NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_measurementControl_v970ext,
      { "measurementControl-v970ext", "rrc.measurementControl_v970ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "MeasurementControl_v970ext_IEs", HFILL }},
    { &hf_rrc_nonCriticalExtensions_63,
      { "nonCriticalExtensions", "rrc.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_nonCriticalExtensions_63", HFILL }},
    { &hf_rrc_criticalExtensions_51,
      { "criticalExtensions", "rrc.criticalExtensions",
        FT_UINT32, BASE_DEC, VALS(rrc_T_criticalExtensions_50_vals), 0,
        "T_criticalExtensions_50", HFILL }},
    { &hf_rrc_r10_04,
      { "r10", "rrc.r10",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_r10_04", HFILL }},
    { &hf_rrc_measurementControl_r10,
      { "measurementControl-r10", "rrc.measurementControl_r10",
        FT_NONE, BASE_NONE, NULL, 0,
        "MeasurementControl_r10_IEs", HFILL }},
    { &hf_rrc_measurementControl_r10_add_ext,
      { "measurementControl-r10-add-ext", "rrc.measurementControl_r10_add_ext",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING", HFILL }},
    { &hf_rrc_vaa0NonCriticalExtensions,
      { "vaa0NonCriticalExtensions", "rrc.vaa0NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_measurementControl_vaa0ext,
      { "measurementControl-vaa0ext", "rrc.measurementControl_vaa0ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "MeasurementControl_vaa0ext_IEs", HFILL }},
    { &hf_rrc_nonCriticalExtensions_64,
      { "nonCriticalExtensions", "rrc.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_nonCriticalExtensions_64", HFILL }},
    { &hf_rrc_criticalExtensions_52,
      { "criticalExtensions", "rrc.criticalExtensions",
        FT_UINT32, BASE_DEC, VALS(rrc_T_criticalExtensions_51_vals), 0,
        "T_criticalExtensions_51", HFILL }},
    { &hf_rrc_r11_05,
      { "r11", "rrc.r11",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_r11_05", HFILL }},
    { &hf_rrc_measurementControl_r11,
      { "measurementControl-r11", "rrc.measurementControl_r11",
        FT_NONE, BASE_NONE, NULL, 0,
        "MeasurementControl_r11_IEs", HFILL }},
    { &hf_rrc_measurementControl_r11_add_ext,
      { "measurementControl-r11-add-ext", "rrc.measurementControl_r11_add_ext",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING", HFILL }},
    { &hf_rrc_nonCriticalExtensions_65,
      { "nonCriticalExtensions", "rrc.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_nonCriticalExtensions_65", HFILL }},
    { &hf_rrc_criticalExtensions_53,
      { "criticalExtensions", "rrc.criticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_criticalExtensions_52", HFILL }},
    { &hf_rrc_measurementIdentity,
      { "measurementIdentity", "rrc.measurementIdentity",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_measurementCommand,
      { "measurementCommand", "rrc.measurementCommand",
        FT_UINT32, BASE_DEC, VALS(rrc_MeasurementCommand_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_measurementReportingMode,
      { "measurementReportingMode", "rrc.measurementReportingMode",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_additionalMeasurementList,
      { "additionalMeasurementList", "rrc.additionalMeasurementList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "AdditionalMeasurementID_List", HFILL }},
    { &hf_rrc_dpch_CompressedModeStatusInfo,
      { "dpch-CompressedModeStatusInfo", "rrc.dpch_CompressedModeStatusInfo",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_ue_Positioning_Measurement_v390ext,
      { "ue-Positioning-Measurement-v390ext", "rrc.ue_Positioning_Measurement_v390ext",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_measurementCommand_01,
      { "measurementCommand", "rrc.measurementCommand",
        FT_UINT32, BASE_DEC, VALS(rrc_MeasurementCommand_r4_vals), 0,
        "MeasurementCommand_r4", HFILL }},
    { &hf_rrc_measurementCommand_v590ext,
      { "measurementCommand-v590ext", "rrc.measurementCommand_v590ext",
        FT_UINT32, BASE_DEC, VALS(rrc_T_measurementCommand_v590ext_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_intra_frequency,
      { "intra-frequency", "rrc.intra_frequency",
        FT_UINT32, BASE_DEC, NULL, 0,
        "Intra_FreqEventCriteriaList_v590ext", HFILL }},
    { &hf_rrc_inter_frequency,
      { "inter-frequency", "rrc.inter_frequency",
        FT_UINT32, BASE_DEC, NULL, 0,
        "Inter_FreqEventCriteriaList_v590ext", HFILL }},
    { &hf_rrc_intraFreqReportingCriteria_1b_r5,
      { "intraFreqReportingCriteria-1b-r5", "rrc.intraFreqReportingCriteria_1b_r5",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_intraFreqEvent_1d_r5,
      { "intraFreqEvent-1d-r5", "rrc.intraFreqEvent_1d_r5",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_rrc_TransactionIdentifier_MSP_v590ext,
      { "rrc-TransactionIdentifier-MSP-v590ext", "rrc.rrc_TransactionIdentifier_MSP_v590ext",
        FT_UINT32, BASE_DEC, NULL, 0,
        "RRC_TransactionIdentifier", HFILL }},
    { &hf_rrc_interRATCellInfoIndication,
      { "interRATCellInfoIndication", "rrc.interRATCellInfoIndication",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_measurementCommand_02,
      { "measurementCommand", "rrc.measurementCommand",
        FT_UINT32, BASE_DEC, VALS(rrc_MeasurementCommand_r6_vals), 0,
        "MeasurementCommand_r6", HFILL }},
    { &hf_rrc_measurementCommand_03,
      { "measurementCommand", "rrc.measurementCommand",
        FT_UINT32, BASE_DEC, VALS(rrc_MeasurementCommand_r7_vals), 0,
        "MeasurementCommand_r7", HFILL }},
    { &hf_rrc_newInterFreqCellList,
      { "newInterFreqCellList", "rrc.newInterFreqCellList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "NewInterFreqCellList_v7b0ext", HFILL }},
    { &hf_rrc_measurementCommand_04,
      { "measurementCommand", "rrc.measurementCommand",
        FT_UINT32, BASE_DEC, VALS(rrc_MeasurementCommand_r8_vals), 0,
        "MeasurementCommand_r8", HFILL }},
    { &hf_rrc_newIntraFreqCellList,
      { "newIntraFreqCellList", "rrc.newIntraFreqCellList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "NewIntraFreqCellList_LCR_v8a0ext", HFILL }},
    { &hf_rrc_newInterFreqCellList_01,
      { "newInterFreqCellList", "rrc.newInterFreqCellList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "NewInterFreqCellList_LCR_v8a0ext", HFILL }},
    { &hf_rrc_measurementCommand_05,
      { "measurementCommand", "rrc.measurementCommand",
        FT_UINT32, BASE_DEC, VALS(rrc_MeasurementCommand_r9_vals), 0,
        "MeasurementCommand_r9", HFILL }},
    { &hf_rrc_measurementIdentity_01,
      { "measurementIdentity", "rrc.measurementIdentity",
        FT_UINT32, BASE_DEC, NULL, 0,
        "MeasurementIdentity_r9", HFILL }},
    { &hf_rrc_additionalMeasurementList_01,
      { "additionalMeasurementList", "rrc.additionalMeasurementList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "AdditionalMeasurementID_List_r9", HFILL }},
    { &hf_rrc_measurementCommand_06,
      { "measurementCommand", "rrc.measurementCommand",
        FT_UINT32, BASE_DEC, VALS(rrc_MeasurementCommand_r10_vals), 0,
        "MeasurementCommand_r10", HFILL }},
    { &hf_rrc_dpch_CompressedModeStatusInfo_01,
      { "dpch-CompressedModeStatusInfo", "rrc.dpch_CompressedModeStatusInfo",
        FT_NONE, BASE_NONE, NULL, 0,
        "DPCH_CompressedModeStatusInfo_r10", HFILL }},
    { &hf_rrc_periodicalReportingCriteriaOnSecULFreq,
      { "periodicalReportingCriteriaOnSecULFreq", "rrc.periodicalReportingCriteriaOnSecULFreq",
        FT_NONE, BASE_NONE, NULL, 0,
        "PeriodicalWithReportingCellStatusOnSecULFreq", HFILL }},
    { &hf_rrc_measurementCommand_07,
      { "measurementCommand", "rrc.measurementCommand",
        FT_UINT32, BASE_DEC, VALS(rrc_MeasurementCommand_r11_vals), 0,
        "MeasurementCommand_r11", HFILL }},
    { &hf_rrc_laterNonCriticalExtensions_19,
      { "laterNonCriticalExtensions", "rrc.laterNonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_laterNonCriticalExtensions_19", HFILL }},
    { &hf_rrc_measurementControlFailure_r3_add_ext,
      { "measurementControlFailure-r3-add-ext", "rrc.measurementControlFailure_r3_add_ext",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING", HFILL }},
    { &hf_rrc_v590NonCriticalExtensions_10,
      { "v590NonCriticalExtensions", "rrc.v590NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v590NonCriticalExtensions_10", HFILL }},
    { &hf_rrc_measurementControlFailure_v590ext,
      { "measurementControlFailure-v590ext", "rrc.measurementControlFailure_v590ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "MeasurementControlFailure_v590ext_IEs", HFILL }},
    { &hf_rrc_nonCriticalExtensions_66,
      { "nonCriticalExtensions", "rrc.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_nonCriticalExtensions_66", HFILL }},
    { &hf_rrc_measuredResults,
      { "measuredResults", "rrc.measuredResults",
        FT_UINT32, BASE_DEC, VALS(rrc_MeasuredResults_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_additionalMeasuredResults,
      { "additionalMeasuredResults", "rrc.additionalMeasuredResults",
        FT_UINT32, BASE_DEC, NULL, 0,
        "MeasuredResultsList", HFILL }},
    { &hf_rrc_eventResults,
      { "eventResults", "rrc.eventResults",
        FT_UINT32, BASE_DEC, VALS(rrc_EventResults_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_v390nonCriticalExtensions_01,
      { "v390nonCriticalExtensions", "rrc.v390nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v390nonCriticalExtensions_01", HFILL }},
    { &hf_rrc_measurementReport_v390ext,
      { "measurementReport-v390ext", "rrc.measurementReport_v390ext",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_laterNonCriticalExtensions_20,
      { "laterNonCriticalExtensions", "rrc.laterNonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_laterNonCriticalExtensions_20", HFILL }},
    { &hf_rrc_measurementReport_r3_add_ext,
      { "measurementReport-r3-add-ext", "rrc.measurementReport_r3_add_ext",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING", HFILL }},
    { &hf_rrc_v4b0NonCriticalExtensions_06,
      { "v4b0NonCriticalExtensions", "rrc.v4b0NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v4b0NonCriticalExtensions_06", HFILL }},
    { &hf_rrc_measurementReport_v4b0ext,
      { "measurementReport-v4b0ext", "rrc.measurementReport_v4b0ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "MeasurementReport_v4b0ext_IEs", HFILL }},
    { &hf_rrc_v590NonCriticalExtensions_11,
      { "v590NonCriticalExtensions", "rrc.v590NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v590NonCriticalExtensions_11", HFILL }},
    { &hf_rrc_measurementReport_v590ext,
      { "measurementReport-v590ext", "rrc.measurementReport_v590ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "MeasurementReport_v590ext_IEs", HFILL }},
    { &hf_rrc_v5b0NonCriticalExtensions_02,
      { "v5b0NonCriticalExtensions", "rrc.v5b0NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v5b0NonCriticalExtensions_02", HFILL }},
    { &hf_rrc_measurementReport_v5b0ext,
      { "measurementReport-v5b0ext", "rrc.measurementReport_v5b0ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "MeasurementReport_v5b0ext_IEs", HFILL }},
    { &hf_rrc_v690NonCriticalExtensions_11,
      { "v690NonCriticalExtensions", "rrc.v690NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v690NonCriticalExtensions_11", HFILL }},
    { &hf_rrc_measurementReport_v690ext,
      { "measurementReport-v690ext", "rrc.measurementReport_v690ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "MeasurementReport_v690ext_IEs", HFILL }},
    { &hf_rrc_v770NonCriticalExtensions_03,
      { "v770NonCriticalExtensions", "rrc.v770NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v770NonCriticalExtensions_03", HFILL }},
    { &hf_rrc_measurementReport_v770ext,
      { "measurementReport-v770ext", "rrc.measurementReport_v770ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "MeasurementReport_v770ext_IEs", HFILL }},
    { &hf_rrc_v860NonCriticalExtensions_08,
      { "v860NonCriticalExtensions", "rrc.v860NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v860NonCriticalExtensions_08", HFILL }},
    { &hf_rrc_measurementReport_v860ext,
      { "measurementReport-v860ext", "rrc.measurementReport_v860ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "MeasurementReport_v860ext_IEs", HFILL }},
    { &hf_rrc_v920NonCriticalExtensions_01,
      { "v920NonCriticalExtensions", "rrc.v920NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v920NonCriticalExtensions_01", HFILL }},
    { &hf_rrc_measurementReport_v920ext,
      { "measurementReport-v920ext", "rrc.measurementReport_v920ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "MeasurementReport_v920ext_IEs", HFILL }},
    { &hf_rrc_v970NonCriticalExtensions_01,
      { "v970NonCriticalExtensions", "rrc.v970NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v970NonCriticalExtensions_01", HFILL }},
    { &hf_rrc_measurementReport_v970ext_IE,
      { "measurementReport-v970ext-IE", "rrc.measurementReport_v970ext_IE",
        FT_NONE, BASE_NONE, NULL, 0,
        "MeasurementReport_v970ext_IEs", HFILL }},
    { &hf_rrc_va40NonCriticalExtensions_04,
      { "va40NonCriticalExtensions", "rrc.va40NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_va40NonCriticalExtensions_04", HFILL }},
    { &hf_rrc_measurementReport_va40ext,
      { "measurementReport-va40ext", "rrc.measurementReport_va40ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "MeasurementReport_va40ext_IEs", HFILL }},
    { &hf_rrc_vbxyNonCriticalExtensions_07,
      { "vbxyNonCriticalExtensions", "rrc.vbxyNonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_vbxyNonCriticalExtensions_07", HFILL }},
    { &hf_rrc_measurementReport_vbxyext,
      { "measurementReport-vbxyext", "rrc.measurementReport_vbxyext",
        FT_NONE, BASE_NONE, NULL, 0,
        "MeasurementReport_vbxyext_IEs", HFILL }},
    { &hf_rrc_nonCriticalExtensions_67,
      { "nonCriticalExtensions", "rrc.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_nonCriticalExtensions_67", HFILL }},
    { &hf_rrc_measuredResults_v390ext,
      { "measuredResults-v390ext", "rrc.measuredResults_v390ext",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_interFreqEventResults_LCR,
      { "interFreqEventResults-LCR", "rrc.interFreqEventResults_LCR",
        FT_NONE, BASE_NONE, NULL, 0,
        "InterFreqEventResults_LCR_r4_ext", HFILL }},
    { &hf_rrc_additionalMeasuredResults_LCR,
      { "additionalMeasuredResults-LCR", "rrc.additionalMeasuredResults_LCR",
        FT_UINT32, BASE_DEC, NULL, 0,
        "MeasuredResultsList_LCR_r4_ext", HFILL }},
    { &hf_rrc_dummy_12,
      { "dummy", "rrc.dummy",
        FT_NONE, BASE_NONE, NULL, 0,
        "PrimaryCPICH_Info", HFILL }},
    { &hf_rrc_measuredResults_v590ext,
      { "measuredResults-v590ext", "rrc.measuredResults_v590ext",
        FT_UINT32, BASE_DEC, VALS(rrc_MeasuredResults_v590ext_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_measuredResults_01,
      { "measuredResults", "rrc.measuredResults",
        FT_UINT32, BASE_DEC, VALS(rrc_MeasuredResults_v770ext_vals), 0,
        "MeasuredResults_v770ext", HFILL }},
    { &hf_rrc_additionalMeasuredResults_01,
      { "additionalMeasuredResults", "rrc.additionalMeasuredResults",
        FT_UINT32, BASE_DEC, NULL, 0,
        "MeasuredResultsList_v770xet", HFILL }},
    { &hf_rrc_eventResults_01,
      { "eventResults", "rrc.eventResults",
        FT_UINT32, BASE_DEC, VALS(rrc_EventResults_v770ext_vals), 0,
        "EventResults_v770ext", HFILL }},
    { &hf_rrc_ue_Positioning_OTDOA_MeasuredResults,
      { "ue-Positioning-OTDOA-MeasuredResults", "rrc.ue_Positioning_OTDOA_MeasuredResults",
        FT_NONE, BASE_NONE, NULL, 0,
        "UE_Positioning_OTDOA_MeasuredResultsTDD_ext", HFILL }},
    { &hf_rrc_measuredResults_02,
      { "measuredResults", "rrc.measuredResults",
        FT_UINT32, BASE_DEC, VALS(rrc_MeasuredResults_v860ext_vals), 0,
        "MeasuredResults_v860ext", HFILL }},
    { &hf_rrc_additionalMeasuredResults_02,
      { "additionalMeasuredResults", "rrc.additionalMeasuredResults",
        FT_UINT32, BASE_DEC, NULL, 0,
        "MeasuredResultsList_v860ext", HFILL }},
    { &hf_rrc_eventResults_02,
      { "eventResults", "rrc.eventResults",
        FT_UINT32, BASE_DEC, VALS(rrc_EventResults_v860ext_vals), 0,
        "EventResults_v860ext", HFILL }},
    { &hf_rrc_eutra_MeasuredResults,
      { "eutra-MeasuredResults", "rrc.eutra_MeasuredResults",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_eutra_EventResults,
      { "eutra-EventResults", "rrc.eutra_EventResults",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_cSGProximityIndication,
      { "cSGProximityIndication", "rrc.cSGProximityIndication",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_measuredResults_03,
      { "measuredResults", "rrc.measuredResults",
        FT_UINT32, BASE_DEC, VALS(rrc_MeasuredResults_v920ext_vals), 0,
        "MeasuredResults_v920ext", HFILL }},
    { &hf_rrc_measuredResultsOnSecUlFreq,
      { "measuredResultsOnSecUlFreq", "rrc.measuredResultsOnSecUlFreq",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_additionalMeasuredResults_03,
      { "additionalMeasuredResults", "rrc.additionalMeasuredResults",
        FT_UINT32, BASE_DEC, NULL, 0,
        "MeasuredResultsList_v920ext", HFILL }},
    { &hf_rrc_additionalMeasuredResultsOnSecUlFreq,
      { "additionalMeasuredResultsOnSecUlFreq", "rrc.additionalMeasuredResultsOnSecUlFreq",
        FT_UINT32, BASE_DEC, NULL, 0,
        "MeasuredResultsListOnSecUlFreq", HFILL }},
    { &hf_rrc_eventResultsOnSecUlFreq,
      { "eventResultsOnSecUlFreq", "rrc.eventResultsOnSecUlFreq",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_eutra_MeasuredResults_01,
      { "eutra-MeasuredResults", "rrc.eutra_MeasuredResults",
        FT_NONE, BASE_NONE, NULL, 0,
        "EUTRA_MeasuredResults_v920ext", HFILL }},
    { &hf_rrc_measurementIdentity_02,
      { "measurementIdentity", "rrc.measurementIdentity",
        FT_UINT32, BASE_DEC, NULL, 0,
        "MeasurementIdentityExt", HFILL }},
    { &hf_rrc_eventResults_03,
      { "eventResults", "rrc.eventResults",
        FT_UINT32, BASE_DEC, VALS(rrc_EventResults_va40ext_vals), 0,
        "EventResults_va40ext", HFILL }},
    { &hf_rrc_loggedMeasAvailable_02,
      { "loggedMeasAvailable", "rrc.loggedMeasAvailable",
        FT_UINT32, BASE_DEC, VALS(rrc_T_loggedMeasAvailable_02_vals), 0,
        "T_loggedMeasAvailable_02", HFILL }},
    { &hf_rrc_loggedANRResultsAvailable_01,
      { "loggedANRResultsAvailable", "rrc.loggedANRResultsAvailable",
        FT_UINT32, BASE_DEC, VALS(rrc_T_loggedANRResultsAvailable_01_vals), 0,
        "T_loggedANRResultsAvailable_01", HFILL }},
    { &hf_rrc_eutra_EventResultsForCELLFACH,
      { "eutra-EventResultsForCELLFACH", "rrc.eutra_EventResultsForCELLFACH",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_pagingRecordList,
      { "pagingRecordList", "rrc.pagingRecordList",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_bcch_ModificationInfo,
      { "bcch-ModificationInfo", "rrc.bcch_ModificationInfo",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_laterNonCriticalExtensions_21,
      { "laterNonCriticalExtensions", "rrc.laterNonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_laterNonCriticalExtensions_21", HFILL }},
    { &hf_rrc_pagingType1_r3_add_ext,
      { "pagingType1-r3-add-ext", "rrc.pagingType1_r3_add_ext",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING", HFILL }},
    { &hf_rrc_v590NonCriticalExtensions_12,
      { "v590NonCriticalExtensions", "rrc.v590NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v590NonCriticalExtensions_12", HFILL }},
    { &hf_rrc_pagingType1_v590ext,
      { "pagingType1-v590ext", "rrc.pagingType1_v590ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "PagingType1_v590ext_IEs", HFILL }},
    { &hf_rrc_v860NonCriticalExtensions_09,
      { "v860NonCriticalExtensions", "rrc.v860NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v860NonCriticalExtensions_09", HFILL }},
    { &hf_rrc_pagingType1_v860ext,
      { "pagingType1-v860ext", "rrc.pagingType1_v860ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "PagingType1_v860ext_IEs", HFILL }},
    { &hf_rrc_nonCriticalExtensions_68,
      { "nonCriticalExtensions", "rrc.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_nonCriticalExtensions_68", HFILL }},
    { &hf_rrc_pagingRecord2List,
      { "pagingRecord2List", "rrc.pagingRecord2List",
        FT_UINT32, BASE_DEC, NULL, 0,
        "PagingRecord2List_r5", HFILL }},
    { &hf_rrc_pagingCause,
      { "pagingCause", "rrc.pagingCause",
        FT_UINT32, BASE_DEC, VALS(rrc_PagingCause_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_pagingRecordTypeID,
      { "pagingRecordTypeID", "rrc.pagingRecordTypeID",
        FT_UINT32, BASE_DEC, VALS(rrc_PagingRecordTypeID_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_laterNonCriticalExtensions_22,
      { "laterNonCriticalExtensions", "rrc.laterNonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_laterNonCriticalExtensions_22", HFILL }},
    { &hf_rrc_pagingType2_r3_add_ext,
      { "pagingType2-r3-add-ext", "rrc.pagingType2_r3_add_ext",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING", HFILL }},
    { &hf_rrc_nonCriticalExtensions_69,
      { "nonCriticalExtensions", "rrc.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_nonCriticalExtensions_69", HFILL }},
    { &hf_rrc_r3_12,
      { "r3", "rrc.r3",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_r3_12", HFILL }},
    { &hf_rrc_physicalChannelReconfiguration_r3,
      { "physicalChannelReconfiguration-r3", "rrc.physicalChannelReconfiguration_r3",
        FT_NONE, BASE_NONE, NULL, 0,
        "PhysicalChannelReconfiguration_r3_IEs", HFILL }},
    { &hf_rrc_v3a0NonCriticalExtensions_05,
      { "v3a0NonCriticalExtensions", "rrc.v3a0NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v3a0NonCriticalExtensions_05", HFILL }},
    { &hf_rrc_physicalChannelReconfiguration_v3a0ext,
      { "physicalChannelReconfiguration-v3a0ext", "rrc.physicalChannelReconfiguration_v3a0ext",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_laterNonCriticalExtensions_23,
      { "laterNonCriticalExtensions", "rrc.laterNonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_laterNonCriticalExtensions_23", HFILL }},
    { &hf_rrc_physicalChannelReconfiguration_r3_add_ext,
      { "physicalChannelReconfiguration-r3-add-ext", "rrc.physicalChannelReconfiguration_r3_add_ext",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING", HFILL }},
    { &hf_rrc_v4b0NonCriticalExtenstions,
      { "v4b0NonCriticalExtenstions", "rrc.v4b0NonCriticalExtenstions",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_physicalChannelReconfiguration_v4b0ext,
      { "physicalChannelReconfiguration-v4b0ext", "rrc.physicalChannelReconfiguration_v4b0ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "PhysicalChannelReconfiguration_v4b0ext_IEs", HFILL }},
    { &hf_rrc_v590NonCriticalExtenstions_02,
      { "v590NonCriticalExtenstions", "rrc.v590NonCriticalExtenstions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v590NonCriticalExtenstions_02", HFILL }},
    { &hf_rrc_physicalChannelReconfiguration_v590ext,
      { "physicalChannelReconfiguration-v590ext", "rrc.physicalChannelReconfiguration_v590ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "PhysicalChannelReconfiguration_v590ext_IEs", HFILL }},
    { &hf_rrc_v690NonCriticalExtensions_12,
      { "v690NonCriticalExtensions", "rrc.v690NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v690NonCriticalExtensions_12", HFILL }},
    { &hf_rrc_physicalChannelReconfiguration_v690ext,
      { "physicalChannelReconfiguration-v690ext", "rrc.physicalChannelReconfiguration_v690ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "PhysicalChannelReconfiguration_v690ext_IEs", HFILL }},
    { &hf_rrc_v770NonCriticalExtensions_04,
      { "v770NonCriticalExtensions", "rrc.v770NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v770NonCriticalExtensions_04", HFILL }},
    { &hf_rrc_physicalChannelReconfiguration_v770ext,
      { "physicalChannelReconfiguration-v770ext", "rrc.physicalChannelReconfiguration_v770ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "PhysicalChannelReconfiguration_v770ext_IEs", HFILL }},
    { &hf_rrc_nonCriticalExtensions_70,
      { "nonCriticalExtensions", "rrc.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_nonCriticalExtensions_70", HFILL }},
    { &hf_rrc_later_than_r3_10,
      { "later-than-r3", "rrc.later_than_r3",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_later_than_r3_10", HFILL }},
    { &hf_rrc_criticalExtensions_54,
      { "criticalExtensions", "rrc.criticalExtensions",
        FT_UINT32, BASE_DEC, VALS(rrc_T_criticalExtensions_53_vals), 0,
        "T_criticalExtensions_53", HFILL }},
    { &hf_rrc_r4_04,
      { "r4", "rrc.r4",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_r4_04", HFILL }},
    { &hf_rrc_physicalChannelReconfiguration_r4,
      { "physicalChannelReconfiguration-r4", "rrc.physicalChannelReconfiguration_r4",
        FT_NONE, BASE_NONE, NULL, 0,
        "PhysicalChannelReconfiguration_r4_IEs", HFILL }},
    { &hf_rrc_v4d0NonCriticalExtensions_04,
      { "v4d0NonCriticalExtensions", "rrc.v4d0NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v4d0NonCriticalExtensions_04", HFILL }},
    { &hf_rrc_physicalChannelReconfiguration_r4_add_ext,
      { "physicalChannelReconfiguration-r4-add-ext", "rrc.physicalChannelReconfiguration_r4_add_ext",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING", HFILL }},
    { &hf_rrc_v590NonCriticalExtenstions_03,
      { "v590NonCriticalExtenstions", "rrc.v590NonCriticalExtenstions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v590NonCriticalExtenstions_03", HFILL }},
    { &hf_rrc_v690NonCriticalExtensions_13,
      { "v690NonCriticalExtensions", "rrc.v690NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v690NonCriticalExtensions_13", HFILL }},
    { &hf_rrc_v770NonCriticalExtensions_05,
      { "v770NonCriticalExtensions", "rrc.v770NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v770NonCriticalExtensions_05", HFILL }},
    { &hf_rrc_nonCriticalExtensions_71,
      { "nonCriticalExtensions", "rrc.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_nonCriticalExtensions_71", HFILL }},
    { &hf_rrc_criticalExtensions_55,
      { "criticalExtensions", "rrc.criticalExtensions",
        FT_UINT32, BASE_DEC, VALS(rrc_T_criticalExtensions_54_vals), 0,
        "T_criticalExtensions_54", HFILL }},
    { &hf_rrc_r5_04,
      { "r5", "rrc.r5",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_r5_04", HFILL }},
    { &hf_rrc_physicalChannelReconfiguration_r5,
      { "physicalChannelReconfiguration-r5", "rrc.physicalChannelReconfiguration_r5",
        FT_NONE, BASE_NONE, NULL, 0,
        "PhysicalChannelReconfiguration_r5_IEs", HFILL }},
    { &hf_rrc_physicalChannelReconfiguration_r5_add_ext,
      { "physicalChannelReconfiguration-r5-add-ext", "rrc.physicalChannelReconfiguration_r5_add_ext",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING", HFILL }},
    { &hf_rrc_v690NonCriticalExtensions_14,
      { "v690NonCriticalExtensions", "rrc.v690NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v690NonCriticalExtensions_14", HFILL }},
    { &hf_rrc_v770NonCriticalExtensions_06,
      { "v770NonCriticalExtensions", "rrc.v770NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v770NonCriticalExtensions_06", HFILL }},
    { &hf_rrc_nonCriticalExtensions_72,
      { "nonCriticalExtensions", "rrc.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_nonCriticalExtensions_72", HFILL }},
    { &hf_rrc_criticalExtensions_56,
      { "criticalExtensions", "rrc.criticalExtensions",
        FT_UINT32, BASE_DEC, VALS(rrc_T_criticalExtensions_55_vals), 0,
        "T_criticalExtensions_55", HFILL }},
    { &hf_rrc_r6_06,
      { "r6", "rrc.r6",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_r6_06", HFILL }},
    { &hf_rrc_physicalChannelReconfiguration_r6,
      { "physicalChannelReconfiguration-r6", "rrc.physicalChannelReconfiguration_r6",
        FT_NONE, BASE_NONE, NULL, 0,
        "PhysicalChannelReconfiguration_r6_IEs", HFILL }},
    { &hf_rrc_physicalChannelReconfiguration_r6_add_ext,
      { "physicalChannelReconfiguration-r6-add-ext", "rrc.physicalChannelReconfiguration_r6_add_ext",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING", HFILL }},
    { &hf_rrc_v6b0NonCriticalExtensions_06,
      { "v6b0NonCriticalExtensions", "rrc.v6b0NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v6b0NonCriticalExtensions_05", HFILL }},
    { &hf_rrc_physicalChannelReconfiguration_v6b0ext,
      { "physicalChannelReconfiguration-v6b0ext", "rrc.physicalChannelReconfiguration_v6b0ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "PhysicalChannelReconfiguration_v6b0ext_IEs", HFILL }},
    { &hf_rrc_v770NonCriticalExtensions_07,
      { "v770NonCriticalExtensions", "rrc.v770NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v770NonCriticalExtensions_07", HFILL }},
    { &hf_rrc_nonCriticalExtensions_73,
      { "nonCriticalExtensions", "rrc.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_nonCriticalExtensions_73", HFILL }},
    { &hf_rrc_criticalExtensions_57,
      { "criticalExtensions", "rrc.criticalExtensions",
        FT_UINT32, BASE_DEC, VALS(rrc_T_criticalExtensions_56_vals), 0,
        "T_criticalExtensions_56", HFILL }},
    { &hf_rrc_r7_05,
      { "r7", "rrc.r7",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_r7_05", HFILL }},
    { &hf_rrc_physicalChannelReconfiguration_r7,
      { "physicalChannelReconfiguration-r7", "rrc.physicalChannelReconfiguration_r7",
        FT_NONE, BASE_NONE, NULL, 0,
        "PhysicalChannelReconfiguration_r7_IEs", HFILL }},
    { &hf_rrc_physicalChannelReconfiguration_r7_add_ext,
      { "physicalChannelReconfiguration-r7-add-ext", "rrc.physicalChannelReconfiguration_r7_add_ext",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING", HFILL }},
    { &hf_rrc_v780NonCriticalExtensions_04,
      { "v780NonCriticalExtensions", "rrc.v780NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v780NonCriticalExtensions_04", HFILL }},
    { &hf_rrc_physicalChannelReconfiguration_v780ext,
      { "physicalChannelReconfiguration-v780ext", "rrc.physicalChannelReconfiguration_v780ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "PhysicalChannelReconfiguration_v780ext_IEs", HFILL }},
    { &hf_rrc_v7d0NonCriticalExtensions_04,
      { "v7d0NonCriticalExtensions", "rrc.v7d0NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v7d0NonCriticalExtensions_04", HFILL }},
    { &hf_rrc_physicalChannelReconfiguration_v7d0ext,
      { "physicalChannelReconfiguration-v7d0ext", "rrc.physicalChannelReconfiguration_v7d0ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "PhysicalChannelReconfiguration_v7d0ext_IEs", HFILL }},
    { &hf_rrc_v7f0NonCriticalExtensions_05,
      { "v7f0NonCriticalExtensions", "rrc.v7f0NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v7f0NonCriticalExtensions_05", HFILL }},
    { &hf_rrc_physicalChannelReconfiguration_v7f0ext,
      { "physicalChannelReconfiguration-v7f0ext", "rrc.physicalChannelReconfiguration_v7f0ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "PhysicalChannelReconfiguration_v7f0ext_IEs", HFILL }},
    { &hf_rrc_v7g0NonCriticalExtensions_06,
      { "v7g0NonCriticalExtensions", "rrc.v7g0NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v7g0NonCriticalExtensions_06", HFILL }},
    { &hf_rrc_physicalChannelReconfiguration_v7g0ext,
      { "physicalChannelReconfiguration-v7g0ext", "rrc.physicalChannelReconfiguration_v7g0ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "PhysicalChannelReconfiguration_v7g0ext_IEs", HFILL }},
    { &hf_rrc_nonCriticalExtensions_74,
      { "nonCriticalExtensions", "rrc.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_nonCriticalExtensions_74", HFILL }},
    { &hf_rrc_criticalExtensions_58,
      { "criticalExtensions", "rrc.criticalExtensions",
        FT_UINT32, BASE_DEC, VALS(rrc_T_criticalExtensions_57_vals), 0,
        "T_criticalExtensions_57", HFILL }},
    { &hf_rrc_r8_06,
      { "r8", "rrc.r8",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_r8_06", HFILL }},
    { &hf_rrc_physicalChannelReconfiguration_r8,
      { "physicalChannelReconfiguration-r8", "rrc.physicalChannelReconfiguration_r8",
        FT_NONE, BASE_NONE, NULL, 0,
        "PhysicalChannelReconfiguration_r8_IEs", HFILL }},
    { &hf_rrc_physicalChannelReconfiguration_r8_add_ext,
      { "physicalChannelReconfiguration-r8-add-ext", "rrc.physicalChannelReconfiguration_r8_add_ext",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING", HFILL }},
    { &hf_rrc_v7d0NonCriticalExtensions_05,
      { "v7d0NonCriticalExtensions", "rrc.v7d0NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v7d0NonCriticalExtensions_05", HFILL }},
    { &hf_rrc_v7f0NonCriticalExtensions_06,
      { "v7f0NonCriticalExtensions", "rrc.v7f0NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v7f0NonCriticalExtensions_06", HFILL }},
    { &hf_rrc_v890NonCriticalExtensions_04,
      { "v890NonCriticalExtensions", "rrc.v890NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v890NonCriticalExtensions_04", HFILL }},
    { &hf_rrc_physicalChannelReconfiguration_v890ext,
      { "physicalChannelReconfiguration-v890ext", "rrc.physicalChannelReconfiguration_v890ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "PhysicalChannelReconfiguration_v890ext_IEs", HFILL }},
    { &hf_rrc_v7g0NonCriticalExtensions_07,
      { "v7g0NonCriticalExtensions", "rrc.v7g0NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v7g0NonCriticalExtensions_07", HFILL }},
    { &hf_rrc_v8a0NonCriticalExtensions_04,
      { "v8a0NonCriticalExtensions", "rrc.v8a0NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v8a0NonCriticalExtensions_04", HFILL }},
    { &hf_rrc_physicalChannelReconfiguration_v8a0ext,
      { "physicalChannelReconfiguration-v8a0ext", "rrc.physicalChannelReconfiguration_v8a0ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "PhysicalChannelReconfiguration_v8a0ext_IEs", HFILL }},
    { &hf_rrc_nonCriticalExtensions_75,
      { "nonCriticalExtensions", "rrc.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_nonCriticalExtensions_75", HFILL }},
    { &hf_rrc_criticalExtensions_59,
      { "criticalExtensions", "rrc.criticalExtensions",
        FT_UINT32, BASE_DEC, VALS(rrc_T_criticalExtensions_58_vals), 0,
        "T_criticalExtensions_58", HFILL }},
    { &hf_rrc_r9_05,
      { "r9", "rrc.r9",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_r9_05", HFILL }},
    { &hf_rrc_physicalChannelReconfiguration_r9,
      { "physicalChannelReconfiguration-r9", "rrc.physicalChannelReconfiguration_r9",
        FT_NONE, BASE_NONE, NULL, 0,
        "PhysicalChannelReconfiguration_r9_IEs", HFILL }},
    { &hf_rrc_physicalChannelReconfiguration_r9_add_ext,
      { "physicalChannelReconfiguration-r9-add-ext", "rrc.physicalChannelReconfiguration_r9_add_ext",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING", HFILL }},
    { &hf_rrc_v950NonCriticalExtensions_03,
      { "v950NonCriticalExtensions", "rrc.v950NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v950NonCriticalExtensions_03", HFILL }},
    { &hf_rrc_physicalChannelReconfiguration_v950ext,
      { "physicalChannelReconfiguration-v950ext", "rrc.physicalChannelReconfiguration_v950ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "PhysicalChannelReconfiguration_v950ext_IEs", HFILL }},
    { &hf_rrc_nonCriticalExtensions_76,
      { "nonCriticalExtensions", "rrc.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_nonCriticalExtensions_76", HFILL }},
    { &hf_rrc_criticalExtensions_60,
      { "criticalExtensions", "rrc.criticalExtensions",
        FT_UINT32, BASE_DEC, VALS(rrc_T_criticalExtensions_59_vals), 0,
        "T_criticalExtensions_59", HFILL }},
    { &hf_rrc_r10_05,
      { "r10", "rrc.r10",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_r10_05", HFILL }},
    { &hf_rrc_physicalChannelReconfiguration_r10,
      { "physicalChannelReconfiguration-r10", "rrc.physicalChannelReconfiguration_r10",
        FT_NONE, BASE_NONE, NULL, 0,
        "PhysicalChannelReconfiguration_r10_IEs", HFILL }},
    { &hf_rrc_physicalChannelReconfiguration_r10_add_ext,
      { "physicalChannelReconfiguration-r10-add-ext", "rrc.physicalChannelReconfiguration_r10_add_ext",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING", HFILL }},
    { &hf_rrc_vbxyNonCriticalExtensions_08,
      { "vbxyNonCriticalExtensions", "rrc.vbxyNonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_vbxyNonCriticalExtensions_08", HFILL }},
    { &hf_rrc_physicalChannelReconfiguration_vbxyext,
      { "physicalChannelReconfiguration-vbxyext", "rrc.physicalChannelReconfiguration_vbxyext",
        FT_NONE, BASE_NONE, NULL, 0,
        "PhysicalChannelReconfiguration_vbxyext_IEs", HFILL }},
    { &hf_rrc_nonCriticalExtensions_77,
      { "nonCriticalExtensions", "rrc.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_nonCriticalExtensions_77", HFILL }},
    { &hf_rrc_criticalExtensions_61,
      { "criticalExtensions", "rrc.criticalExtensions",
        FT_UINT32, BASE_DEC, VALS(rrc_T_criticalExtensions_60_vals), 0,
        "T_criticalExtensions_60", HFILL }},
    { &hf_rrc_r11_06,
      { "r11", "rrc.r11",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_r11_06", HFILL }},
    { &hf_rrc_physicalChannelReconfiguration_r11,
      { "physicalChannelReconfiguration-r11", "rrc.physicalChannelReconfiguration_r11",
        FT_NONE, BASE_NONE, NULL, 0,
        "PhysicalChannelReconfiguration_r11_IEs", HFILL }},
    { &hf_rrc_physicalChannelReconfiguration_r11_add_ext,
      { "physicalChannelReconfiguration-r11-add-ext", "rrc.physicalChannelReconfiguration_r11_add_ext",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING", HFILL }},
    { &hf_rrc_nonCriticalExtensions_78,
      { "nonCriticalExtensions", "rrc.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_nonCriticalExtensions_78", HFILL }},
    { &hf_rrc_criticalExtensions_62,
      { "criticalExtensions", "rrc.criticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_criticalExtensions_61", HFILL }},
    { &hf_rrc_ul_ChannelRequirement_03,
      { "ul-ChannelRequirement", "rrc.ul_ChannelRequirement",
        FT_UINT32, BASE_DEC, VALS(rrc_UL_ChannelRequirementWithCPCH_SetID_vals), 0,
        "UL_ChannelRequirementWithCPCH_SetID", HFILL }},
    { &hf_rrc_modeSpecificInfo_12,
      { "modeSpecificInfo", "rrc.modeSpecificInfo",
        FT_UINT32, BASE_DEC, VALS(rrc_T_modeSpecificInfo_12_vals), 0,
        "T_modeSpecificInfo_12", HFILL }},
    { &hf_rrc_fdd_18,
      { "fdd", "rrc.fdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_fdd_18", HFILL }},
    { &hf_rrc_ul_ChannelRequirement_04,
      { "ul-ChannelRequirement", "rrc.ul_ChannelRequirement",
        FT_UINT32, BASE_DEC, VALS(rrc_UL_ChannelRequirementWithCPCH_SetID_r4_vals), 0,
        "UL_ChannelRequirementWithCPCH_SetID_r4", HFILL }},
    { &hf_rrc_modeSpecificInfo_13,
      { "modeSpecificInfo", "rrc.modeSpecificInfo",
        FT_UINT32, BASE_DEC, VALS(rrc_T_modeSpecificInfo_13_vals), 0,
        "T_modeSpecificInfo_13", HFILL }},
    { &hf_rrc_fdd_19,
      { "fdd", "rrc.fdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_fdd_19", HFILL }},
    { &hf_rrc_ul_ChannelRequirement_05,
      { "ul-ChannelRequirement", "rrc.ul_ChannelRequirement",
        FT_UINT32, BASE_DEC, VALS(rrc_UL_ChannelRequirementWithCPCH_SetID_r5_vals), 0,
        "UL_ChannelRequirementWithCPCH_SetID_r5", HFILL }},
    { &hf_rrc_modeSpecificInfo_14,
      { "modeSpecificInfo", "rrc.modeSpecificInfo",
        FT_UINT32, BASE_DEC, VALS(rrc_T_modeSpecificInfo_14_vals), 0,
        "T_modeSpecificInfo_14", HFILL }},
    { &hf_rrc_fdd_20,
      { "fdd", "rrc.fdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_fdd_20", HFILL }},
    { &hf_rrc_delayRestrictionFlag,
      { "delayRestrictionFlag", "rrc.delayRestrictionFlag",
        FT_UINT32, BASE_DEC, VALS(rrc_DelayRestrictionFlag_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_postVerificationPeriod_01,
      { "postVerificationPeriod", "rrc.postVerificationPeriod",
        FT_UINT32, BASE_DEC, VALS(rrc_T_postVerificationPeriod_01_vals), 0,
        "T_postVerificationPeriod_01", HFILL }},
    { &hf_rrc_timingMaintainedSynchInd,
      { "timingMaintainedSynchInd", "rrc.timingMaintainedSynchInd",
        FT_UINT32, BASE_DEC, VALS(rrc_TimingMaintainedSynchInd_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_responseToChangeOfUE_Capability,
      { "responseToChangeOfUE-Capability", "rrc.responseToChangeOfUE_Capability",
        FT_UINT32, BASE_DEC, VALS(rrc_T_responseToChangeOfUE_Capability_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_harqInfo,
      { "harqInfo", "rrc.harqInfo",
        FT_NONE, BASE_NONE, NULL, 0,
        "HARQ_Info_r7", HFILL }},
    { &hf_rrc_responseToChangeOfUE_Capability_01,
      { "responseToChangeOfUE-Capability", "rrc.responseToChangeOfUE_Capability",
        FT_UINT32, BASE_DEC, VALS(rrc_T_responseToChangeOfUE_Capability_01_vals), 0,
        "T_responseToChangeOfUE_Capability_01", HFILL }},
    { &hf_rrc_responseToChangeOfUE_Capability_02,
      { "responseToChangeOfUE-Capability", "rrc.responseToChangeOfUE_Capability",
        FT_UINT32, BASE_DEC, VALS(rrc_T_responseToChangeOfUE_Capability_02_vals), 0,
        "T_responseToChangeOfUE_Capability_02", HFILL }},
    { &hf_rrc_responseToChangeOfUE_Capability_03,
      { "responseToChangeOfUE-Capability", "rrc.responseToChangeOfUE_Capability",
        FT_UINT32, BASE_DEC, VALS(rrc_T_responseToChangeOfUE_Capability_03_vals), 0,
        "T_responseToChangeOfUE_Capability_03", HFILL }},
    { &hf_rrc_responseToChangeOfUE_Capability_04,
      { "responseToChangeOfUE-Capability", "rrc.responseToChangeOfUE_Capability",
        FT_UINT32, BASE_DEC, VALS(rrc_T_responseToChangeOfUE_Capability_04_vals), 0,
        "T_responseToChangeOfUE_Capability_04", HFILL }},
    { &hf_rrc_harqInfo_01,
      { "harqInfo", "rrc.harqInfo",
        FT_NONE, BASE_NONE, NULL, 0,
        "HARQ_Info_r11", HFILL }},
    { &hf_rrc_ul_IntegProtActivationInfo,
      { "ul-IntegProtActivationInfo", "rrc.ul_IntegProtActivationInfo",
        FT_NONE, BASE_NONE, NULL, 0,
        "IntegrityProtActivationInfo", HFILL }},
    { &hf_rrc_ul_TimingAdvance,
      { "ul-TimingAdvance", "rrc.ul_TimingAdvance",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_dummy_13,
      { "dummy", "rrc.dummy",
        FT_UINT32, BASE_DEC, NULL, 0,
        "RB_ActivationTimeInfoList", HFILL }},
    { &hf_rrc_ul_CounterSynchronisationInfo,
      { "ul-CounterSynchronisationInfo", "rrc.ul_CounterSynchronisationInfo",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_laterNonCriticalExtensions_24,
      { "laterNonCriticalExtensions", "rrc.laterNonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_laterNonCriticalExtensions_24", HFILL }},
    { &hf_rrc_physicalChannelReconfigurationComplete_r3_add_ext,
      { "physicalChannelReconfigurationComplete-r3-add-ext", "rrc.physicalChannelReconfigurationComplete_r3_add_ext",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING", HFILL }},
    { &hf_rrc_v770NonCriticalExtensions_08,
      { "v770NonCriticalExtensions", "rrc.v770NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v770NonCriticalExtensions_08", HFILL }},
    { &hf_rrc_physicalChannelReconfigurationComplete_v770ext,
      { "physicalChannelReconfigurationComplete-v770ext", "rrc.physicalChannelReconfigurationComplete_v770ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "PhysicalChannelReconfigurationComplete_v770ext_IEs", HFILL }},
    { &hf_rrc_nonCriticalExtensions_79,
      { "nonCriticalExtensions", "rrc.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_nonCriticalExtensions_79", HFILL }},
    { &hf_rrc_ext_UL_TimingAdvance,
      { "ext-UL-TimingAdvance", "rrc.ext_UL_TimingAdvance",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_deferredMeasurementControlReading,
      { "deferredMeasurementControlReading", "rrc.deferredMeasurementControlReading",
        FT_UINT32, BASE_DEC, VALS(rrc_T_deferredMeasurementControlReading_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_laterNonCriticalExtensions_25,
      { "laterNonCriticalExtensions", "rrc.laterNonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_laterNonCriticalExtensions_25", HFILL }},
    { &hf_rrc_physicalChannelReconfigurationFailure_r3_add_ext,
      { "physicalChannelReconfigurationFailure-r3-add-ext", "rrc.physicalChannelReconfigurationFailure_r3_add_ext",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING", HFILL }},
    { &hf_rrc_nonCriticalExtensions_80,
      { "nonCriticalExtensions", "rrc.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_nonCriticalExtensions_80", HFILL }},
    { &hf_rrc_r3_13,
      { "r3", "rrc.r3",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_r3_13", HFILL }},
    { &hf_rrc_physicalSharedChannelAllocation_r3,
      { "physicalSharedChannelAllocation-r3", "rrc.physicalSharedChannelAllocation_r3",
        FT_NONE, BASE_NONE, NULL, 0,
        "PhysicalSharedChannelAllocation_r3_IEs", HFILL }},
    { &hf_rrc_laterNonCriticalExtensions_26,
      { "laterNonCriticalExtensions", "rrc.laterNonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_laterNonCriticalExtensions_26", HFILL }},
    { &hf_rrc_physicalSharedChannelAllocation_r3_add_ext,
      { "physicalSharedChannelAllocation-r3-add-ext", "rrc.physicalSharedChannelAllocation_r3_add_ext",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING", HFILL }},
    { &hf_rrc_nonCriticalExtensions_81,
      { "nonCriticalExtensions", "rrc.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_nonCriticalExtensions_81", HFILL }},
    { &hf_rrc_later_than_r3_11,
      { "later-than-r3", "rrc.later_than_r3",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_later_than_r3_11", HFILL }},
    { &hf_rrc_dsch_RNTI,
      { "dsch-RNTI", "rrc.dsch_RNTI",
        FT_BYTES, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_criticalExtensions_63,
      { "criticalExtensions", "rrc.criticalExtensions",
        FT_UINT32, BASE_DEC, VALS(rrc_T_criticalExtensions_62_vals), 0,
        "T_criticalExtensions_62", HFILL }},
    { &hf_rrc_r4_05,
      { "r4", "rrc.r4",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_r4_05", HFILL }},
    { &hf_rrc_physicalSharedChannelAllocation_r4,
      { "physicalSharedChannelAllocation-r4", "rrc.physicalSharedChannelAllocation_r4",
        FT_NONE, BASE_NONE, NULL, 0,
        "PhysicalSharedChannelAllocation_r4_IEs", HFILL }},
    { &hf_rrc_v4d0NonCriticalExtensions_05,
      { "v4d0NonCriticalExtensions", "rrc.v4d0NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v4d0NonCriticalExtensions_05", HFILL }},
    { &hf_rrc_physicalSharedChannelAllocation_r4_add_ext,
      { "physicalSharedChannelAllocation-r4-add-ext", "rrc.physicalSharedChannelAllocation_r4_add_ext",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING", HFILL }},
    { &hf_rrc_v690NonCriticalExtensions_15,
      { "v690NonCriticalExtensions", "rrc.v690NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v690NonCriticalExtensions_15", HFILL }},
    { &hf_rrc_physicalSharedChannelAllocation_v690ext,
      { "physicalSharedChannelAllocation-v690ext", "rrc.physicalSharedChannelAllocation_v690ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "PhysicalSharedChannelAllocation_v690ext_IEs", HFILL }},
    { &hf_rrc_v770NonCriticalExtensions_09,
      { "v770NonCriticalExtensions", "rrc.v770NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v770NonCriticalExtensions_09", HFILL }},
    { &hf_rrc_physicalSharedChannelAllocation_v770ext,
      { "physicalSharedChannelAllocation-v770ext", "rrc.physicalSharedChannelAllocation_v770ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "PhysicalSharedChannelAllocation_v770ext_IEs", HFILL }},
    { &hf_rrc_nonCriticalExtensions_82,
      { "nonCriticalExtensions", "rrc.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_nonCriticalExtensions_82", HFILL }},
    { &hf_rrc_criticalExtensions_64,
      { "criticalExtensions", "rrc.criticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_criticalExtensions_63", HFILL }},
    { &hf_rrc_ul_TimingAdvance_01,
      { "ul-TimingAdvance", "rrc.ul_TimingAdvance",
        FT_UINT32, BASE_DEC, VALS(rrc_UL_TimingAdvanceControl_vals), 0,
        "UL_TimingAdvanceControl", HFILL }},
    { &hf_rrc_pusch_CapacityAllocationInfo,
      { "pusch-CapacityAllocationInfo", "rrc.pusch_CapacityAllocationInfo",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_pdsch_CapacityAllocationInfo,
      { "pdsch-CapacityAllocationInfo", "rrc.pdsch_CapacityAllocationInfo",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_confirmRequest,
      { "confirmRequest", "rrc.confirmRequest",
        FT_UINT32, BASE_DEC, VALS(rrc_T_confirmRequest_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_trafficVolumeReportRequest,
      { "trafficVolumeReportRequest", "rrc.trafficVolumeReportRequest",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_255", HFILL }},
    { &hf_rrc_iscpTimeslotList,
      { "iscpTimeslotList", "rrc.iscpTimeslotList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "TimeslotList", HFILL }},
    { &hf_rrc_requestPCCPCHRSCP,
      { "requestPCCPCHRSCP", "rrc.requestPCCPCHRSCP",
        FT_BOOLEAN, BASE_NONE, NULL, 0,
        "BOOLEAN", HFILL }},
    { &hf_rrc_ul_TimingAdvance_02,
      { "ul-TimingAdvance", "rrc.ul_TimingAdvance",
        FT_UINT32, BASE_DEC, VALS(rrc_UL_TimingAdvanceControl_r4_vals), 0,
        "UL_TimingAdvanceControl_r4", HFILL }},
    { &hf_rrc_pusch_CapacityAllocationInfo_01,
      { "pusch-CapacityAllocationInfo", "rrc.pusch_CapacityAllocationInfo",
        FT_NONE, BASE_NONE, NULL, 0,
        "PUSCH_CapacityAllocationInfo_r4", HFILL }},
    { &hf_rrc_pdsch_CapacityAllocationInfo_01,
      { "pdsch-CapacityAllocationInfo", "rrc.pdsch_CapacityAllocationInfo",
        FT_NONE, BASE_NONE, NULL, 0,
        "PDSCH_CapacityAllocationInfo_r4", HFILL }},
    { &hf_rrc_confirmRequest_01,
      { "confirmRequest", "rrc.confirmRequest",
        FT_UINT32, BASE_DEC, VALS(rrc_T_confirmRequest_01_vals), 0,
        "T_confirmRequest_01", HFILL }},
    { &hf_rrc_iscpTimeslotList_01,
      { "iscpTimeslotList", "rrc.iscpTimeslotList",
        FT_UINT32, BASE_DEC, VALS(rrc_TimeslotList_r4_vals), 0,
        "TimeslotList_r4", HFILL }},
    { &hf_rrc_ul_TimingAdvance_03,
      { "ul-TimingAdvance", "rrc.ul_TimingAdvance",
        FT_UINT32, BASE_DEC, VALS(rrc_UL_TimingAdvanceControl_r7_vals), 0,
        "UL_TimingAdvanceControl_r7", HFILL }},
    { &hf_rrc_pusch_CapacityAllocationInfo_02,
      { "pusch-CapacityAllocationInfo", "rrc.pusch_CapacityAllocationInfo",
        FT_NONE, BASE_NONE, NULL, 0,
        "PUSCH_CapacityAllocationInfo_r7", HFILL }},
    { &hf_rrc_pdsch_CapacityAllocationInfo_02,
      { "pdsch-CapacityAllocationInfo", "rrc.pdsch_CapacityAllocationInfo",
        FT_NONE, BASE_NONE, NULL, 0,
        "PDSCH_CapacityAllocationInfo_r7", HFILL }},
    { &hf_rrc_trafficVolume,
      { "trafficVolume", "rrc.trafficVolume",
        FT_UINT32, BASE_DEC, NULL, 0,
        "TrafficVolumeMeasuredResultsList", HFILL }},
    { &hf_rrc_timeslotListWithISCP,
      { "timeslotListWithISCP", "rrc.timeslotListWithISCP",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_primaryCCPCH_RSCP,
      { "primaryCCPCH-RSCP", "rrc.primaryCCPCH_RSCP",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_allocationConfirmation,
      { "allocationConfirmation", "rrc.allocationConfirmation",
        FT_UINT32, BASE_DEC, VALS(rrc_T_allocationConfirmation_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_pdschConfirmation,
      { "pdschConfirmation", "rrc.pdschConfirmation",
        FT_UINT32, BASE_DEC, NULL, 0,
        "PDSCH_Identity", HFILL }},
    { &hf_rrc_puschConfirmation,
      { "puschConfirmation", "rrc.puschConfirmation",
        FT_UINT32, BASE_DEC, NULL, 0,
        "PUSCH_Identity", HFILL }},
    { &hf_rrc_protocolErrorIndicator,
      { "protocolErrorIndicator", "rrc.protocolErrorIndicator",
        FT_UINT32, BASE_DEC, VALS(rrc_ProtocolErrorIndicatorWithMoreInfo_vals), 0,
        "ProtocolErrorIndicatorWithMoreInfo", HFILL }},
    { &hf_rrc_laterNonCriticalExtensions_27,
      { "laterNonCriticalExtensions", "rrc.laterNonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_laterNonCriticalExtensions_27", HFILL }},
    { &hf_rrc_puschCapacityRequest_r3_add_ext,
      { "puschCapacityRequest-r3-add-ext", "rrc.puschCapacityRequest_r3_add_ext",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING", HFILL }},
    { &hf_rrc_v590NonCriticalExtensions_13,
      { "v590NonCriticalExtensions", "rrc.v590NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v590NonCriticalExtensions_13", HFILL }},
    { &hf_rrc_puschCapacityRequest_v590ext,
      { "puschCapacityRequest-v590ext", "rrc.puschCapacityRequest_v590ext",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_nonCriticalExtensions_83,
      { "nonCriticalExtensions", "rrc.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_nonCriticalExtensions_83", HFILL }},
    { &hf_rrc_primaryCCPCH_RSCP_delta,
      { "primaryCCPCH-RSCP-delta", "rrc.primaryCCPCH_RSCP_delta",
        FT_INT32, BASE_DEC, NULL, 0,
        "DeltaRSCP", HFILL }},
    { &hf_rrc_r3_14,
      { "r3", "rrc.r3",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_r3_14", HFILL }},
    { &hf_rrc_radioBearerReconfiguration_r3,
      { "radioBearerReconfiguration-r3", "rrc.radioBearerReconfiguration_r3",
        FT_NONE, BASE_NONE, NULL, 0,
        "RadioBearerReconfiguration_r3_IEs", HFILL }},
    { &hf_rrc_v3aoNonCriticalExtensions,
      { "v3aoNonCriticalExtensions", "rrc.v3aoNonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_radioBearerReconfiguration_v3a0ext,
      { "radioBearerReconfiguration-v3a0ext", "rrc.radioBearerReconfiguration_v3a0ext",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_laterNonCriticalExtensions_28,
      { "laterNonCriticalExtensions", "rrc.laterNonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_laterNonCriticalExtensions_28", HFILL }},
    { &hf_rrc_radioBearerReconfiguration_r3_add_ext,
      { "radioBearerReconfiguration-r3-add-ext", "rrc.radioBearerReconfiguration_r3_add_ext",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING", HFILL }},
    { &hf_rrc_v4b0NonCriticalExtensions_07,
      { "v4b0NonCriticalExtensions", "rrc.v4b0NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v4b0NonCriticalExtensions_07", HFILL }},
    { &hf_rrc_radioBearerReconfiguration_v4b0ext,
      { "radioBearerReconfiguration-v4b0ext", "rrc.radioBearerReconfiguration_v4b0ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "RadioBearerReconfiguration_v4b0ext_IEs", HFILL }},
    { &hf_rrc_v590NonCriticalExtensions_14,
      { "v590NonCriticalExtensions", "rrc.v590NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v590NonCriticalExtensions_14", HFILL }},
    { &hf_rrc_radioBearerReconfiguration_v590ext,
      { "radioBearerReconfiguration-v590ext", "rrc.radioBearerReconfiguration_v590ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "RadioBearerReconfiguration_v590ext_IEs", HFILL }},
    { &hf_rrc_v5d0NonCriticalExtenstions_06,
      { "v5d0NonCriticalExtenstions", "rrc.v5d0NonCriticalExtenstions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v5d0NonCriticalExtenstions_06", HFILL }},
    { &hf_rrc_radioBearerReconfiguration_v5d0ext,
      { "radioBearerReconfiguration-v5d0ext", "rrc.radioBearerReconfiguration_v5d0ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "RadioBearerReconfiguration_v5d0ext_IEs", HFILL }},
    { &hf_rrc_v690NonCriticalExtensions_16,
      { "v690NonCriticalExtensions", "rrc.v690NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v690NonCriticalExtensions_16", HFILL }},
    { &hf_rrc_radioBearerReconfiguration_v690ext,
      { "radioBearerReconfiguration-v690ext", "rrc.radioBearerReconfiguration_v690ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "RadioBearerReconfiguration_v690ext_IEs", HFILL }},
    { &hf_rrc_v770NonCriticalExtensions_10,
      { "v770NonCriticalExtensions", "rrc.v770NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v770NonCriticalExtensions_10", HFILL }},
    { &hf_rrc_radioBearerReconfiguration_v770ext,
      { "radioBearerReconfiguration-v770ext", "rrc.radioBearerReconfiguration_v770ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "RadioBearerReconfiguration_v770ext_IEs", HFILL }},
    { &hf_rrc_nonCriticalExtensions_84,
      { "nonCriticalExtensions", "rrc.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_nonCriticalExtensions_84", HFILL }},
    { &hf_rrc_later_than_r3_12,
      { "later-than-r3", "rrc.later_than_r3",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_later_than_r3_12", HFILL }},
    { &hf_rrc_criticalExtensions_65,
      { "criticalExtensions", "rrc.criticalExtensions",
        FT_UINT32, BASE_DEC, VALS(rrc_T_criticalExtensions_64_vals), 0,
        "T_criticalExtensions_64", HFILL }},
    { &hf_rrc_r4_06,
      { "r4", "rrc.r4",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_r4_06", HFILL }},
    { &hf_rrc_radioBearerReconfiguration_r4,
      { "radioBearerReconfiguration-r4", "rrc.radioBearerReconfiguration_r4",
        FT_NONE, BASE_NONE, NULL, 0,
        "RadioBearerReconfiguration_r4_IEs", HFILL }},
    { &hf_rrc_v4d0NonCriticalExtensions_06,
      { "v4d0NonCriticalExtensions", "rrc.v4d0NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v4d0NonCriticalExtensions_06", HFILL }},
    { &hf_rrc_radioBearerReconfiguration_r4_add_ext,
      { "radioBearerReconfiguration-r4-add-ext", "rrc.radioBearerReconfiguration_r4_add_ext",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING", HFILL }},
    { &hf_rrc_v590NonCriticalExtensions_15,
      { "v590NonCriticalExtensions", "rrc.v590NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v590NonCriticalExtensions_15", HFILL }},
    { &hf_rrc_v5d0NonCriticalExtenstions_07,
      { "v5d0NonCriticalExtenstions", "rrc.v5d0NonCriticalExtenstions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v5d0NonCriticalExtenstions_07", HFILL }},
    { &hf_rrc_v690NonCriticalExtensions_17,
      { "v690NonCriticalExtensions", "rrc.v690NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v690NonCriticalExtensions_17", HFILL }},
    { &hf_rrc_v770NonCriticalExtensions_11,
      { "v770NonCriticalExtensions", "rrc.v770NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v770NonCriticalExtensions_11", HFILL }},
    { &hf_rrc_nonCriticalExtensions_85,
      { "nonCriticalExtensions", "rrc.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_nonCriticalExtensions_85", HFILL }},
    { &hf_rrc_criticalExtensions_66,
      { "criticalExtensions", "rrc.criticalExtensions",
        FT_UINT32, BASE_DEC, VALS(rrc_T_criticalExtensions_65_vals), 0,
        "T_criticalExtensions_65", HFILL }},
    { &hf_rrc_r5_05,
      { "r5", "rrc.r5",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_r5_05", HFILL }},
    { &hf_rrc_radioBearerReconfiguration_r5,
      { "radioBearerReconfiguration-r5", "rrc.radioBearerReconfiguration_r5",
        FT_NONE, BASE_NONE, NULL, 0,
        "RadioBearerReconfiguration_r5_IEs", HFILL }},
    { &hf_rrc_radioBearerReconfiguration_r5_add_ext,
      { "radioBearerReconfiguration-r5-add-ext", "rrc.radioBearerReconfiguration_r5_add_ext",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING", HFILL }},
    { &hf_rrc_v5d0NonCriticalExtenstions_08,
      { "v5d0NonCriticalExtenstions", "rrc.v5d0NonCriticalExtenstions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v5d0NonCriticalExtenstions_08", HFILL }},
    { &hf_rrc_v690NonCriticalExtensions_18,
      { "v690NonCriticalExtensions", "rrc.v690NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v690NonCriticalExtensions_18", HFILL }},
    { &hf_rrc_v770NonCriticalExtensions_12,
      { "v770NonCriticalExtensions", "rrc.v770NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v770NonCriticalExtensions_12", HFILL }},
    { &hf_rrc_nonCriticalExtensions_86,
      { "nonCriticalExtensions", "rrc.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_nonCriticalExtensions_86", HFILL }},
    { &hf_rrc_criticalExtensions_67,
      { "criticalExtensions", "rrc.criticalExtensions",
        FT_UINT32, BASE_DEC, VALS(rrc_T_criticalExtensions_66_vals), 0,
        "T_criticalExtensions_66", HFILL }},
    { &hf_rrc_r6_07,
      { "r6", "rrc.r6",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_r6_07", HFILL }},
    { &hf_rrc_radioBearerReconfiguration_r6,
      { "radioBearerReconfiguration-r6", "rrc.radioBearerReconfiguration_r6",
        FT_NONE, BASE_NONE, NULL, 0,
        "RadioBearerReconfiguration_r6_IEs", HFILL }},
    { &hf_rrc_radioBearerReconfiguration_r6_add_ext,
      { "radioBearerReconfiguration-r6-add-ext", "rrc.radioBearerReconfiguration_r6_add_ext",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING", HFILL }},
    { &hf_rrc_v6b0NonCriticalExtensions_07,
      { "v6b0NonCriticalExtensions", "rrc.v6b0NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v6b0NonCriticalExtensions_06", HFILL }},
    { &hf_rrc_radioBearerReconfiguration_v6b0ext,
      { "radioBearerReconfiguration-v6b0ext", "rrc.radioBearerReconfiguration_v6b0ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "RadioBearerReconfiguration_v6b0ext_IEs", HFILL }},
    { &hf_rrc_v6f0NonCriticalExtensions,
      { "v6f0NonCriticalExtensions", "rrc.v6f0NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_radioBearerRconfiguration_v6f0ext,
      { "radioBearerRconfiguration-v6f0ext", "rrc.radioBearerRconfiguration_v6f0ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "RadioBearerReconfiguration_v6f0ext_IEs", HFILL }},
    { &hf_rrc_v770NonCriticalExtensions_13,
      { "v770NonCriticalExtensions", "rrc.v770NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v770NonCriticalExtensions_13", HFILL }},
    { &hf_rrc_nonCriticalExtensions_87,
      { "nonCriticalExtensions", "rrc.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_nonCriticalExtensions_87", HFILL }},
    { &hf_rrc_criticalExtensions_68,
      { "criticalExtensions", "rrc.criticalExtensions",
        FT_UINT32, BASE_DEC, VALS(rrc_T_criticalExtensions_67_vals), 0,
        "T_criticalExtensions_67", HFILL }},
    { &hf_rrc_r7_06,
      { "r7", "rrc.r7",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_r7_06", HFILL }},
    { &hf_rrc_radioBearerReconfiguration_r7,
      { "radioBearerReconfiguration-r7", "rrc.radioBearerReconfiguration_r7",
        FT_NONE, BASE_NONE, NULL, 0,
        "RadioBearerReconfiguration_r7_IEs", HFILL }},
    { &hf_rrc_v780NonCriticalExtensions_05,
      { "v780NonCriticalExtensions", "rrc.v780NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v780NonCriticalExtensions_05", HFILL }},
    { &hf_rrc_radioBearerReconfiguration_v780ext,
      { "radioBearerReconfiguration-v780ext", "rrc.radioBearerReconfiguration_v780ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "RadioBearerReconfiguration_v780ext_IEs", HFILL }},
    { &hf_rrc_v790NonCriticalExtensions,
      { "v790NonCriticalExtensions", "rrc.v790NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_radioBearerReconfiguration_v790ext,
      { "radioBearerReconfiguration-v790ext", "rrc.radioBearerReconfiguration_v790ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "RadioBearerReconfiguration_v790ext_IEs", HFILL }},
    { &hf_rrc_v7d0NonCriticalExtensions_06,
      { "v7d0NonCriticalExtensions", "rrc.v7d0NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v7d0NonCriticalExtensions_06", HFILL }},
    { &hf_rrc_radioBearerReconfiguration_v7d0ext,
      { "radioBearerReconfiguration-v7d0ext", "rrc.radioBearerReconfiguration_v7d0ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "RadioBearerReconfiguration_v7d0ext_IEs", HFILL }},
    { &hf_rrc_v7f0NonCriticalExtensions_07,
      { "v7f0NonCriticalExtensions", "rrc.v7f0NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v7f0NonCriticalExtensions_07", HFILL }},
    { &hf_rrc_radioBearerReconfiguration_v7f0ext,
      { "radioBearerReconfiguration-v7f0ext", "rrc.radioBearerReconfiguration_v7f0ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "RadioBearerReconfiguration_v7f0ext_IEs", HFILL }},
    { &hf_rrc_v7g0NonCriticalExtensions_08,
      { "v7g0NonCriticalExtensions", "rrc.v7g0NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v7g0NonCriticalExtensions_08", HFILL }},
    { &hf_rrc_radioBearerReconfiguration_v7g0ext,
      { "radioBearerReconfiguration-v7g0ext", "rrc.radioBearerReconfiguration_v7g0ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "RadioBearerReconfiguration_v7g0ext_IEs", HFILL }},
    { &hf_rrc_nonCriticalExtensions_88,
      { "nonCriticalExtensions", "rrc.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_nonCriticalExtensions_88", HFILL }},
    { &hf_rrc_criticalExtensions_69,
      { "criticalExtensions", "rrc.criticalExtensions",
        FT_UINT32, BASE_DEC, VALS(rrc_T_criticalExtensions_68_vals), 0,
        "T_criticalExtensions_68", HFILL }},
    { &hf_rrc_r8_07,
      { "r8", "rrc.r8",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_r8_07", HFILL }},
    { &hf_rrc_radioBearerReconfiguration_r8,
      { "radioBearerReconfiguration-r8", "rrc.radioBearerReconfiguration_r8",
        FT_NONE, BASE_NONE, NULL, 0,
        "RadioBearerReconfiguration_r8_IEs", HFILL }},
    { &hf_rrc_radioBearerReconfiguration_r8_add_ext,
      { "radioBearerReconfiguration-r8-add-ext", "rrc.radioBearerReconfiguration_r8_add_ext",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING", HFILL }},
    { &hf_rrc_v7d0NonCriticalExtensions_07,
      { "v7d0NonCriticalExtensions", "rrc.v7d0NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v7d0NonCriticalExtensions_07", HFILL }},
    { &hf_rrc_v7f0NonCriticalExtensions_08,
      { "v7f0NonCriticalExtensions", "rrc.v7f0NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v7f0NonCriticalExtensions_08", HFILL }},
    { &hf_rrc_v890NonCriticalExtensions_05,
      { "v890NonCriticalExtensions", "rrc.v890NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v890NonCriticalExtensions_05", HFILL }},
    { &hf_rrc_radioBearerReconfiguration_v890ext,
      { "radioBearerReconfiguration-v890ext", "rrc.radioBearerReconfiguration_v890ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "RadioBearerReconfiguration_v890ext_IEs", HFILL }},
    { &hf_rrc_v7g0NonCriticalExtensions_09,
      { "v7g0NonCriticalExtensions", "rrc.v7g0NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v7g0NonCriticalExtensions_09", HFILL }},
    { &hf_rrc_v8a0NonCriticalExtensions_05,
      { "v8a0NonCriticalExtensions", "rrc.v8a0NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v8a0NonCriticalExtensions_05", HFILL }},
    { &hf_rrc_radioBearerReconfiguration_v8a0ext,
      { "radioBearerReconfiguration-v8a0ext", "rrc.radioBearerReconfiguration_v8a0ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "RadioBearerReconfiguration_v8a0ext_IEs", HFILL }},
    { &hf_rrc_nonCriticalExtensions_89,
      { "nonCriticalExtensions", "rrc.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_nonCriticalExtensions_89", HFILL }},
    { &hf_rrc_criticalExtensions_70,
      { "criticalExtensions", "rrc.criticalExtensions",
        FT_UINT32, BASE_DEC, VALS(rrc_T_criticalExtensions_69_vals), 0,
        "T_criticalExtensions_69", HFILL }},
    { &hf_rrc_r9_06,
      { "r9", "rrc.r9",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_r9_06", HFILL }},
    { &hf_rrc_radioBearerReconfiguration_r9,
      { "radioBearerReconfiguration-r9", "rrc.radioBearerReconfiguration_r9",
        FT_NONE, BASE_NONE, NULL, 0,
        "RadioBearerReconfiguration_r9_IEs", HFILL }},
    { &hf_rrc_radioBearerReconfiguration_r9_add_ext,
      { "radioBearerReconfiguration-r9-add-ext", "rrc.radioBearerReconfiguration_r9_add_ext",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING", HFILL }},
    { &hf_rrc_v950NonCriticalExtensions_04,
      { "v950NonCriticalExtensions", "rrc.v950NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v950NonCriticalExtensions_04", HFILL }},
    { &hf_rrc_radioBearerReconfiguration_v950ext,
      { "radioBearerReconfiguration-v950ext", "rrc.radioBearerReconfiguration_v950ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "RadioBearerReconfiguration_v950ext_IEs", HFILL }},
    { &hf_rrc_v9c0NonCriticalExtensions_04,
      { "v9c0NonCriticalExtensions", "rrc.v9c0NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v9c0NonCriticalExtensions_04", HFILL }},
    { &hf_rrc_radioBearerReconfiguration_v9c0ext,
      { "radioBearerReconfiguration-v9c0ext", "rrc.radioBearerReconfiguration_v9c0ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "RadioBearerReconfiguration_v9c0ext_IEs", HFILL }},
    { &hf_rrc_nonCriticalExtensions_90,
      { "nonCriticalExtensions", "rrc.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_nonCriticalExtensions_90", HFILL }},
    { &hf_rrc_criticalExtensions_71,
      { "criticalExtensions", "rrc.criticalExtensions",
        FT_UINT32, BASE_DEC, VALS(rrc_T_criticalExtensions_70_vals), 0,
        "T_criticalExtensions_70", HFILL }},
    { &hf_rrc_r10_06,
      { "r10", "rrc.r10",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_r10_06", HFILL }},
    { &hf_rrc_radioBearerReconfiguration_r10,
      { "radioBearerReconfiguration-r10", "rrc.radioBearerReconfiguration_r10",
        FT_NONE, BASE_NONE, NULL, 0,
        "RadioBearerReconfiguration_r10_IEs", HFILL }},
    { &hf_rrc_radioBearerReconfiguration_r10_add_ext,
      { "radioBearerReconfiguration-r10-add-ext", "rrc.radioBearerReconfiguration_r10_add_ext",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING", HFILL }},
    { &hf_rrc_v9c0NonCriticalExtensions_05,
      { "v9c0NonCriticalExtensions", "rrc.v9c0NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v9c0NonCriticalExtensions_05", HFILL }},
    { &hf_rrc_vaa0NonCriticalExtensions_01,
      { "vaa0NonCriticalExtensions", "rrc.vaa0NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_vaa0NonCriticalExtensions_01", HFILL }},
    { &hf_rrc_radioBearerReconfiguration_vaa0ext,
      { "radioBearerReconfiguration-vaa0ext", "rrc.radioBearerReconfiguration_vaa0ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "RadioBearerReconfiguration_vaa0ext_IEs", HFILL }},
    { &hf_rrc_vbxyNonCriticalExtensions_09,
      { "vbxyNonCriticalExtensions", "rrc.vbxyNonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_vbxyNonCriticalExtensions_09", HFILL }},
    { &hf_rrc_radioBearerReconfiguration_vbxyext,
      { "radioBearerReconfiguration-vbxyext", "rrc.radioBearerReconfiguration_vbxyext",
        FT_NONE, BASE_NONE, NULL, 0,
        "RadioBearerReconfiguration_vbxyext_IEs", HFILL }},
    { &hf_rrc_nonCriticalExtensions_91,
      { "nonCriticalExtensions", "rrc.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_nonCriticalExtensions_91", HFILL }},
    { &hf_rrc_criticalExtensions_72,
      { "criticalExtensions", "rrc.criticalExtensions",
        FT_UINT32, BASE_DEC, VALS(rrc_T_criticalExtensions_71_vals), 0,
        "T_criticalExtensions_71", HFILL }},
    { &hf_rrc_r11_07,
      { "r11", "rrc.r11",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_r11_07", HFILL }},
    { &hf_rrc_radioBearerReconfiguration_r11,
      { "radioBearerReconfiguration-r11", "rrc.radioBearerReconfiguration_r11",
        FT_NONE, BASE_NONE, NULL, 0,
        "RadioBearerReconfiguration_r11_IEs", HFILL }},
    { &hf_rrc_radioBearerReconfiguration_r11_add_ext,
      { "radioBearerReconfiguration-r11-add-ext", "rrc.radioBearerReconfiguration_r11_add_ext",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING", HFILL }},
    { &hf_rrc_nonCriticalExtensions_92,
      { "nonCriticalExtensions", "rrc.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_nonCriticalExtensions_92", HFILL }},
    { &hf_rrc_criticalExtensions_73,
      { "criticalExtensions", "rrc.criticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_criticalExtensions_72", HFILL }},
    { &hf_rrc_umRlc_ReEstablishmentRBList,
      { "umRlc-ReEstablishmentRBList", "rrc.umRlc_ReEstablishmentRBList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "RB_IdentityList", HFILL }},
    { &hf_rrc_rab_InformationReconfigList,
      { "rab-InformationReconfigList", "rrc.rab_InformationReconfigList",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_dummy_14,
      { "dummy", "rrc.dummy",
        FT_UINT32, BASE_DEC, VALS(rrc_T_dummy_01_vals), 0,
        "T_dummy_01", HFILL }},
    { &hf_rrc_fdd_21,
      { "fdd", "rrc.fdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_fdd_21", HFILL }},
    { &hf_rrc_dummy1,
      { "dummy1", "rrc.dummy1",
        FT_UINT32, BASE_DEC, NULL, 0,
        "CPCH_SetID", HFILL }},
    { &hf_rrc_dl_AddReconfTransChInfoList_07,
      { "dl-AddReconfTransChInfoList", "rrc.dl_AddReconfTransChInfoList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "DL_AddReconfTransChInfo2List", HFILL }},
    { &hf_rrc_modeSpecificPhysChInfo_03,
      { "modeSpecificPhysChInfo", "rrc.modeSpecificPhysChInfo",
        FT_UINT32, BASE_DEC, VALS(rrc_T_modeSpecificPhysChInfo_03_vals), 0,
        "T_modeSpecificPhysChInfo_03", HFILL }},
    { &hf_rrc_fdd_22,
      { "fdd", "rrc.fdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_fdd_22", HFILL }},
    { &hf_rrc_dummy_15,
      { "dummy", "rrc.dummy",
        FT_UINT32, BASE_DEC, VALS(rrc_T_dummy_02_vals), 0,
        "T_dummy_02", HFILL }},
    { &hf_rrc_fdd_23,
      { "fdd", "rrc.fdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_fdd_23", HFILL }},
    { &hf_rrc_modeSpecificPhysChInfo_04,
      { "modeSpecificPhysChInfo", "rrc.modeSpecificPhysChInfo",
        FT_UINT32, BASE_DEC, VALS(rrc_T_modeSpecificPhysChInfo_04_vals), 0,
        "T_modeSpecificPhysChInfo_04", HFILL }},
    { &hf_rrc_fdd_24,
      { "fdd", "rrc.fdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_fdd_24", HFILL }},
    { &hf_rrc_specificationMode_09,
      { "specificationMode", "rrc.specificationMode",
        FT_UINT32, BASE_DEC, VALS(rrc_T_specificationMode_09_vals), 0,
        "T_specificationMode_09", HFILL }},
    { &hf_rrc_complete_09,
      { "complete", "rrc.complete",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_complete_09", HFILL }},
    { &hf_rrc_rb_PDCPContextRelocationList,
      { "rb-PDCPContextRelocationList", "rrc.rb_PDCPContextRelocationList",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_dummy_16,
      { "dummy", "rrc.dummy",
        FT_UINT32, BASE_DEC, VALS(rrc_T_dummy_03_vals), 0,
        "T_dummy_03", HFILL }},
    { &hf_rrc_fdd_25,
      { "fdd", "rrc.fdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_fdd_25", HFILL }},
    { &hf_rrc_preconfiguration_09,
      { "preconfiguration", "rrc.preconfiguration",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_preconfiguration_09", HFILL }},
    { &hf_rrc_preConfigMode_09,
      { "preConfigMode", "rrc.preConfigMode",
        FT_UINT32, BASE_DEC, VALS(rrc_T_preConfigMode_09_vals), 0,
        "T_preConfigMode_09", HFILL }},
    { &hf_rrc_defaultConfig_09,
      { "defaultConfig", "rrc.defaultConfig",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_defaultConfig_09", HFILL }},
    { &hf_rrc_modeSpecificPhysChInfo_05,
      { "modeSpecificPhysChInfo", "rrc.modeSpecificPhysChInfo",
        FT_UINT32, BASE_DEC, VALS(rrc_T_modeSpecificPhysChInfo_05_vals), 0,
        "T_modeSpecificPhysChInfo_05", HFILL }},
    { &hf_rrc_fdd_26,
      { "fdd", "rrc.fdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_fdd_26", HFILL }},
    { &hf_rrc_specificationMode_10,
      { "specificationMode", "rrc.specificationMode",
        FT_UINT32, BASE_DEC, VALS(rrc_T_specificationMode_10_vals), 0,
        "T_specificationMode_10", HFILL }},
    { &hf_rrc_complete_10,
      { "complete", "rrc.complete",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_complete_10", HFILL }},
    { &hf_rrc_preconfiguration_10,
      { "preconfiguration", "rrc.preconfiguration",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_preconfiguration_10", HFILL }},
    { &hf_rrc_preConfigMode_10,
      { "preConfigMode", "rrc.preConfigMode",
        FT_UINT32, BASE_DEC, VALS(rrc_T_preConfigMode_10_vals), 0,
        "T_preConfigMode_10", HFILL }},
    { &hf_rrc_defaultConfig_10,
      { "defaultConfig", "rrc.defaultConfig",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_defaultConfig_10", HFILL }},
    { &hf_rrc_postVerificationPeriod_02,
      { "postVerificationPeriod", "rrc.postVerificationPeriod",
        FT_UINT32, BASE_DEC, VALS(rrc_T_postVerificationPeriod_02_vals), 0,
        "T_postVerificationPeriod_02", HFILL }},
    { &hf_rrc_specificationMode_11,
      { "specificationMode", "rrc.specificationMode",
        FT_UINT32, BASE_DEC, VALS(rrc_T_specificationMode_11_vals), 0,
        "T_specificationMode_11", HFILL }},
    { &hf_rrc_complete_11,
      { "complete", "rrc.complete",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_complete_11", HFILL }},
    { &hf_rrc_rab_InformationMBMSPtpList,
      { "rab-InformationMBMSPtpList", "rrc.rab_InformationMBMSPtpList",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_responseToChangeOfUE_Capability_05,
      { "responseToChangeOfUE-Capability", "rrc.responseToChangeOfUE_Capability",
        FT_UINT32, BASE_DEC, VALS(rrc_T_responseToChangeOfUE_Capability_05_vals), 0,
        "T_responseToChangeOfUE_Capability_05", HFILL }},
    { &hf_rrc_specificationMode_12,
      { "specificationMode", "rrc.specificationMode",
        FT_UINT32, BASE_DEC, VALS(rrc_T_specificationMode_12_vals), 0,
        "T_specificationMode_12", HFILL }},
    { &hf_rrc_complete_12,
      { "complete", "rrc.complete",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_complete_12", HFILL }},
    { &hf_rrc_preconfiguration_11,
      { "preconfiguration", "rrc.preconfiguration",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_preconfiguration_11", HFILL }},
    { &hf_rrc_preConfigMode_11,
      { "preConfigMode", "rrc.preConfigMode",
        FT_UINT32, BASE_DEC, VALS(rrc_T_preConfigMode_11_vals), 0,
        "T_preConfigMode_11", HFILL }},
    { &hf_rrc_defaultConfig_11,
      { "defaultConfig", "rrc.defaultConfig",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_defaultConfig_11", HFILL }},
    { &hf_rrc_radioBearerReconfiguration_r7_add_ext,
      { "radioBearerReconfiguration-r7-add-ext", "rrc.radioBearerReconfiguration_r7_add_ext",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING", HFILL }},
    { &hf_rrc_responseToChangeOfUE_Capability_06,
      { "responseToChangeOfUE-Capability", "rrc.responseToChangeOfUE_Capability",
        FT_UINT32, BASE_DEC, VALS(rrc_T_responseToChangeOfUE_Capability_06_vals), 0,
        "T_responseToChangeOfUE_Capability_06", HFILL }},
    { &hf_rrc_specificationMode_13,
      { "specificationMode", "rrc.specificationMode",
        FT_UINT32, BASE_DEC, VALS(rrc_T_specificationMode_13_vals), 0,
        "T_specificationMode_13", HFILL }},
    { &hf_rrc_complete_13,
      { "complete", "rrc.complete",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_complete_13", HFILL }},
    { &hf_rrc_rab_InformationReconfigList_01,
      { "rab-InformationReconfigList", "rrc.rab_InformationReconfigList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "RAB_InformationReconfigList_r8", HFILL }},
    { &hf_rrc_preconfiguration_12,
      { "preconfiguration", "rrc.preconfiguration",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_preconfiguration_12", HFILL }},
    { &hf_rrc_preConfigMode_12,
      { "preConfigMode", "rrc.preConfigMode",
        FT_UINT32, BASE_DEC, VALS(rrc_T_preConfigMode_12_vals), 0,
        "T_preConfigMode_12", HFILL }},
    { &hf_rrc_defaultConfig_12,
      { "defaultConfig", "rrc.defaultConfig",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_defaultConfig_12", HFILL }},
    { &hf_rrc_responseToChangeOfUE_Capability_07,
      { "responseToChangeOfUE-Capability", "rrc.responseToChangeOfUE_Capability",
        FT_UINT32, BASE_DEC, VALS(rrc_T_responseToChangeOfUE_Capability_07_vals), 0,
        "T_responseToChangeOfUE_Capability_07", HFILL }},
    { &hf_rrc_specificationMode_14,
      { "specificationMode", "rrc.specificationMode",
        FT_UINT32, BASE_DEC, VALS(rrc_T_specificationMode_14_vals), 0,
        "T_specificationMode_14", HFILL }},
    { &hf_rrc_complete_14,
      { "complete", "rrc.complete",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_complete_14", HFILL }},
    { &hf_rrc_preconfiguration_13,
      { "preconfiguration", "rrc.preconfiguration",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_preconfiguration_13", HFILL }},
    { &hf_rrc_preConfigMode_13,
      { "preConfigMode", "rrc.preConfigMode",
        FT_UINT32, BASE_DEC, VALS(rrc_T_preConfigMode_13_vals), 0,
        "T_preConfigMode_13", HFILL }},
    { &hf_rrc_defaultConfig_13,
      { "defaultConfig", "rrc.defaultConfig",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_defaultConfig_13", HFILL }},
    { &hf_rrc_responseToChangeOfUE_Capability_08,
      { "responseToChangeOfUE-Capability", "rrc.responseToChangeOfUE_Capability",
        FT_UINT32, BASE_DEC, VALS(rrc_T_responseToChangeOfUE_Capability_08_vals), 0,
        "T_responseToChangeOfUE_Capability_08", HFILL }},
    { &hf_rrc_specificationMode_15,
      { "specificationMode", "rrc.specificationMode",
        FT_UINT32, BASE_DEC, VALS(rrc_T_specificationMode_15_vals), 0,
        "T_specificationMode_15", HFILL }},
    { &hf_rrc_complete_15,
      { "complete", "rrc.complete",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_complete_15", HFILL }},
    { &hf_rrc_preconfiguration_14,
      { "preconfiguration", "rrc.preconfiguration",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_preconfiguration_14", HFILL }},
    { &hf_rrc_preConfigMode_14,
      { "preConfigMode", "rrc.preConfigMode",
        FT_UINT32, BASE_DEC, VALS(rrc_T_preConfigMode_14_vals), 0,
        "T_preConfigMode_14", HFILL }},
    { &hf_rrc_defaultConfig_14,
      { "defaultConfig", "rrc.defaultConfig",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_defaultConfig_14", HFILL }},
    { &hf_rrc_responseToChangeOfUE_Capability_09,
      { "responseToChangeOfUE-Capability", "rrc.responseToChangeOfUE_Capability",
        FT_UINT32, BASE_DEC, VALS(rrc_T_responseToChangeOfUE_Capability_09_vals), 0,
        "T_responseToChangeOfUE_Capability_09", HFILL }},
    { &hf_rrc_specificationMode_16,
      { "specificationMode", "rrc.specificationMode",
        FT_UINT32, BASE_DEC, VALS(rrc_T_specificationMode_16_vals), 0,
        "T_specificationMode_16", HFILL }},
    { &hf_rrc_complete_16,
      { "complete", "rrc.complete",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_complete_16", HFILL }},
    { &hf_rrc_preconfiguration_15,
      { "preconfiguration", "rrc.preconfiguration",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_preconfiguration_15", HFILL }},
    { &hf_rrc_preConfigMode_15,
      { "preConfigMode", "rrc.preConfigMode",
        FT_UINT32, BASE_DEC, VALS(rrc_T_preConfigMode_15_vals), 0,
        "T_preConfigMode_15", HFILL }},
    { &hf_rrc_defaultConfig_15,
      { "defaultConfig", "rrc.defaultConfig",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_defaultConfig_15", HFILL }},
    { &hf_rrc_laterNonCriticalExtensions_29,
      { "laterNonCriticalExtensions", "rrc.laterNonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_laterNonCriticalExtensions_29", HFILL }},
    { &hf_rrc_radioBearerReconfigurationComplete_r3_add_ext,
      { "radioBearerReconfigurationComplete-r3-add-ext", "rrc.radioBearerReconfigurationComplete_r3_add_ext",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING", HFILL }},
    { &hf_rrc_v770NonCriticalExtensions_14,
      { "v770NonCriticalExtensions", "rrc.v770NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v770NonCriticalExtensions_14", HFILL }},
    { &hf_rrc_radioBearerReconfigurationComplete_v770ext,
      { "radioBearerReconfigurationComplete-v770ext", "rrc.radioBearerReconfigurationComplete_v770ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "RadioBearerReconfigurationComplete_v770ext_IEs", HFILL }},
    { &hf_rrc_nonCriticalExtensions_93,
      { "nonCriticalExtensions", "rrc.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_nonCriticalExtensions_93", HFILL }},
    { &hf_rrc_deferredMeasurementControlReading_01,
      { "deferredMeasurementControlReading", "rrc.deferredMeasurementControlReading",
        FT_UINT32, BASE_DEC, VALS(rrc_T_deferredMeasurementControlReading_01_vals), 0,
        "T_deferredMeasurementControlReading_01", HFILL }},
    { &hf_rrc_potentiallySuccesfulBearerList,
      { "potentiallySuccesfulBearerList", "rrc.potentiallySuccesfulBearerList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "RB_IdentityList", HFILL }},
    { &hf_rrc_laterNonCriticalExtensions_30,
      { "laterNonCriticalExtensions", "rrc.laterNonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_laterNonCriticalExtensions_30", HFILL }},
    { &hf_rrc_radioBearerReconfigurationFailure_r3_add_ext,
      { "radioBearerReconfigurationFailure-r3-add-ext", "rrc.radioBearerReconfigurationFailure_r3_add_ext",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING", HFILL }},
    { &hf_rrc_nonCriticalExtensions_94,
      { "nonCriticalExtensions", "rrc.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_nonCriticalExtensions_94", HFILL }},
    { &hf_rrc_r3_15,
      { "r3", "rrc.r3",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_r3_15", HFILL }},
    { &hf_rrc_radioBearerRelease_r3,
      { "radioBearerRelease-r3", "rrc.radioBearerRelease_r3",
        FT_NONE, BASE_NONE, NULL, 0,
        "RadioBearerRelease_r3_IEs", HFILL }},
    { &hf_rrc_v3a0NonCriticalExtensions_06,
      { "v3a0NonCriticalExtensions", "rrc.v3a0NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v3a0NonCriticalExtensions_06", HFILL }},
    { &hf_rrc_radioBearerRelease_v3a0ext,
      { "radioBearerRelease-v3a0ext", "rrc.radioBearerRelease_v3a0ext",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_laterNonCriticalExtensions_31,
      { "laterNonCriticalExtensions", "rrc.laterNonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_laterNonCriticalExtensions_31", HFILL }},
    { &hf_rrc_radioBearerRelease_r3_add_ext,
      { "radioBearerRelease-r3-add-ext", "rrc.radioBearerRelease_r3_add_ext",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING", HFILL }},
    { &hf_rrc_v4b0NonCriticalExtensions_08,
      { "v4b0NonCriticalExtensions", "rrc.v4b0NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v4b0NonCriticalExtensions_08", HFILL }},
    { &hf_rrc_radioBearerRelease_v4b0ext,
      { "radioBearerRelease-v4b0ext", "rrc.radioBearerRelease_v4b0ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "RadioBearerRelease_v4b0ext_IEs", HFILL }},
    { &hf_rrc_v590NonCriticalExtensions_16,
      { "v590NonCriticalExtensions", "rrc.v590NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v590NonCriticalExtensions_16", HFILL }},
    { &hf_rrc_radioBearerRelease_v590ext,
      { "radioBearerRelease-v590ext", "rrc.radioBearerRelease_v590ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "RadioBearerRelease_v590ext_IEs", HFILL }},
    { &hf_rrc_v690NonCriticalExtensions_19,
      { "v690NonCriticalExtensions", "rrc.v690NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v690NonCriticalExtensions_19", HFILL }},
    { &hf_rrc_radioBearerRelease_v690ext,
      { "radioBearerRelease-v690ext", "rrc.radioBearerRelease_v690ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "RadioBearerRelease_v690ext_IEs", HFILL }},
    { &hf_rrc_v770NonCriticalExtensions_15,
      { "v770NonCriticalExtensions", "rrc.v770NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v770NonCriticalExtensions_15", HFILL }},
    { &hf_rrc_radioBearerRelease_v770ext,
      { "radioBearerRelease-v770ext", "rrc.radioBearerRelease_v770ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "RadioBearerRelease_v770ext_IEs", HFILL }},
    { &hf_rrc_nonCriticalExtensions_95,
      { "nonCriticalExtensions", "rrc.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_nonCriticalExtensions_95", HFILL }},
    { &hf_rrc_later_than_r3_13,
      { "later-than-r3", "rrc.later_than_r3",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_later_than_r3_13", HFILL }},
    { &hf_rrc_criticalExtensions_74,
      { "criticalExtensions", "rrc.criticalExtensions",
        FT_UINT32, BASE_DEC, VALS(rrc_T_criticalExtensions_73_vals), 0,
        "T_criticalExtensions_73", HFILL }},
    { &hf_rrc_r4_07,
      { "r4", "rrc.r4",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_r4_07", HFILL }},
    { &hf_rrc_radioBearerRelease_r4,
      { "radioBearerRelease-r4", "rrc.radioBearerRelease_r4",
        FT_NONE, BASE_NONE, NULL, 0,
        "RadioBearerRelease_r4_IEs", HFILL }},
    { &hf_rrc_v4d0NonCriticalExtensions_07,
      { "v4d0NonCriticalExtensions", "rrc.v4d0NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v4d0NonCriticalExtensions_07", HFILL }},
    { &hf_rrc_radioBearerRelease_r4_add_ext,
      { "radioBearerRelease-r4-add-ext", "rrc.radioBearerRelease_r4_add_ext",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING", HFILL }},
    { &hf_rrc_v590NonCriticalExtensions_17,
      { "v590NonCriticalExtensions", "rrc.v590NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v590NonCriticalExtensions_17", HFILL }},
    { &hf_rrc_v690NonCriticalExtensions_20,
      { "v690NonCriticalExtensions", "rrc.v690NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v690NonCriticalExtensions_20", HFILL }},
    { &hf_rrc_v770NonCriticalExtensions_16,
      { "v770NonCriticalExtensions", "rrc.v770NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v770NonCriticalExtensions_16", HFILL }},
    { &hf_rrc_nonCriticalExtensions_96,
      { "nonCriticalExtensions", "rrc.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_nonCriticalExtensions_96", HFILL }},
    { &hf_rrc_criticalExtensions_75,
      { "criticalExtensions", "rrc.criticalExtensions",
        FT_UINT32, BASE_DEC, VALS(rrc_T_criticalExtensions_74_vals), 0,
        "T_criticalExtensions_74", HFILL }},
    { &hf_rrc_r5_06,
      { "r5", "rrc.r5",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_r5_06", HFILL }},
    { &hf_rrc_radioBearerRelease_r5,
      { "radioBearerRelease-r5", "rrc.radioBearerRelease_r5",
        FT_NONE, BASE_NONE, NULL, 0,
        "RadioBearerRelease_r5_IEs", HFILL }},
    { &hf_rrc_radioBearerRelease_r5_add_ext,
      { "radioBearerRelease-r5-add-ext", "rrc.radioBearerRelease_r5_add_ext",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING", HFILL }},
    { &hf_rrc_v690NonCriticalExtensions_21,
      { "v690NonCriticalExtensions", "rrc.v690NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v690NonCriticalExtensions_21", HFILL }},
    { &hf_rrc_v770NonCriticalExtensions_17,
      { "v770NonCriticalExtensions", "rrc.v770NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v770NonCriticalExtensions_17", HFILL }},
    { &hf_rrc_nonCriticalExtensions_97,
      { "nonCriticalExtensions", "rrc.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_nonCriticalExtensions_97", HFILL }},
    { &hf_rrc_criticalExtensions_76,
      { "criticalExtensions", "rrc.criticalExtensions",
        FT_UINT32, BASE_DEC, VALS(rrc_T_criticalExtensions_75_vals), 0,
        "T_criticalExtensions_75", HFILL }},
    { &hf_rrc_r6_08,
      { "r6", "rrc.r6",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_r6_08", HFILL }},
    { &hf_rrc_radioBearerRelease_r6,
      { "radioBearerRelease-r6", "rrc.radioBearerRelease_r6",
        FT_NONE, BASE_NONE, NULL, 0,
        "RadioBearerRelease_r6_IEs", HFILL }},
    { &hf_rrc_radioBearerRelease_r6_add_ext,
      { "radioBearerRelease-r6-add-ext", "rrc.radioBearerRelease_r6_add_ext",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING", HFILL }},
    { &hf_rrc_v6b0NonCriticalExtensions_08,
      { "v6b0NonCriticalExtensions", "rrc.v6b0NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v6b0NonCriticalExtensions_07", HFILL }},
    { &hf_rrc_radioBearerRelease_v6b0ext,
      { "radioBearerRelease-v6b0ext", "rrc.radioBearerRelease_v6b0ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "RadioBearerRelease_v6b0ext_IEs", HFILL }},
    { &hf_rrc_v770NonCriticalExtensions_18,
      { "v770NonCriticalExtensions", "rrc.v770NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v770NonCriticalExtensions_18", HFILL }},
    { &hf_rrc_nonCriticalExtensions_98,
      { "nonCriticalExtensions", "rrc.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_nonCriticalExtensions_98", HFILL }},
    { &hf_rrc_criticalExtensions_77,
      { "criticalExtensions", "rrc.criticalExtensions",
        FT_UINT32, BASE_DEC, VALS(rrc_T_criticalExtensions_76_vals), 0,
        "T_criticalExtensions_76", HFILL }},
    { &hf_rrc_r7_07,
      { "r7", "rrc.r7",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_r7_07", HFILL }},
    { &hf_rrc_radioBearerRelease_r7,
      { "radioBearerRelease-r7", "rrc.radioBearerRelease_r7",
        FT_NONE, BASE_NONE, NULL, 0,
        "RadioBearerRelease_r7_IEs", HFILL }},
    { &hf_rrc_radioBearerRelease_r7_add_ext,
      { "radioBearerRelease-r7-add-ext", "rrc.radioBearerRelease_r7_add_ext",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING", HFILL }},
    { &hf_rrc_v780NonCriticalExtensions_06,
      { "v780NonCriticalExtensions", "rrc.v780NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v780NonCriticalExtensions_06", HFILL }},
    { &hf_rrc_radioBearerRelease_v780ext,
      { "radioBearerRelease-v780ext", "rrc.radioBearerRelease_v780ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "RadioBearerRelease_v780ext_IEs", HFILL }},
    { &hf_rrc_v7d0NonCriticalExtensions_08,
      { "v7d0NonCriticalExtensions", "rrc.v7d0NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v7d0NonCriticalExtensions_08", HFILL }},
    { &hf_rrc_radioBearerRelease_v7d0ext,
      { "radioBearerRelease-v7d0ext", "rrc.radioBearerRelease_v7d0ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "RadioBearerRelease_v7d0ext_IEs", HFILL }},
    { &hf_rrc_v7f0NonCriticalExtensions_09,
      { "v7f0NonCriticalExtensions", "rrc.v7f0NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v7f0NonCriticalExtensions_09", HFILL }},
    { &hf_rrc_radioBearerRelease_v7f0ext,
      { "radioBearerRelease-v7f0ext", "rrc.radioBearerRelease_v7f0ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "RadioBearerRelease_v7f0ext_IEs", HFILL }},
    { &hf_rrc_v7g0NonCriticalExtensions_10,
      { "v7g0NonCriticalExtensions", "rrc.v7g0NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v7g0NonCriticalExtensions_10", HFILL }},
    { &hf_rrc_radioBearerRelease_v7g0ext,
      { "radioBearerRelease-v7g0ext", "rrc.radioBearerRelease_v7g0ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "RadioBearerRelease_v7g0ext_IEs", HFILL }},
    { &hf_rrc_nonCriticalExtensions_99,
      { "nonCriticalExtensions", "rrc.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_nonCriticalExtensions_99", HFILL }},
    { &hf_rrc_criticalExtensions_78,
      { "criticalExtensions", "rrc.criticalExtensions",
        FT_UINT32, BASE_DEC, VALS(rrc_T_criticalExtensions_77_vals), 0,
        "T_criticalExtensions_77", HFILL }},
    { &hf_rrc_r8_08,
      { "r8", "rrc.r8",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_r8_08", HFILL }},
    { &hf_rrc_radioBearerRelease_r8,
      { "radioBearerRelease-r8", "rrc.radioBearerRelease_r8",
        FT_NONE, BASE_NONE, NULL, 0,
        "RadioBearerRelease_r8_IEs", HFILL }},
    { &hf_rrc_radioBearerRelease_r8_add_ext,
      { "radioBearerRelease-r8-add-ext", "rrc.radioBearerRelease_r8_add_ext",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING", HFILL }},
    { &hf_rrc_v7d0NonCriticalExtensions_09,
      { "v7d0NonCriticalExtensions", "rrc.v7d0NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v7d0NonCriticalExtensions_09", HFILL }},
    { &hf_rrc_v7f0NonCriticalExtensions_10,
      { "v7f0NonCriticalExtensions", "rrc.v7f0NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v7f0NonCriticalExtensions_10", HFILL }},
    { &hf_rrc_v890NonCriticalExtensions_06,
      { "v890NonCriticalExtensions", "rrc.v890NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v890NonCriticalExtensions_06", HFILL }},
    { &hf_rrc_radioBearerRelease_v890ext,
      { "radioBearerRelease-v890ext", "rrc.radioBearerRelease_v890ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "RadioBearerRelease_v890ext_IEs", HFILL }},
    { &hf_rrc_v7g0NonCriticalExtensions_11,
      { "v7g0NonCriticalExtensions", "rrc.v7g0NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v7g0NonCriticalExtensions_11", HFILL }},
    { &hf_rrc_v8a0NonCriticalExtensions_06,
      { "v8a0NonCriticalExtensions", "rrc.v8a0NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v8a0NonCriticalExtensions_06", HFILL }},
    { &hf_rrc_radioBearerRelease_v8a0ext,
      { "radioBearerRelease-v8a0ext", "rrc.radioBearerRelease_v8a0ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "RadioBearerRelease_v8a0ext_IEs", HFILL }},
    { &hf_rrc_nonCriticalExtensions_100,
      { "nonCriticalExtensions", "rrc.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_nonCriticalExtensions_100", HFILL }},
    { &hf_rrc_criticalExtensions_79,
      { "criticalExtensions", "rrc.criticalExtensions",
        FT_UINT32, BASE_DEC, VALS(rrc_T_criticalExtensions_78_vals), 0,
        "T_criticalExtensions_78", HFILL }},
    { &hf_rrc_r9_07,
      { "r9", "rrc.r9",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_r9_07", HFILL }},
    { &hf_rrc_radioBearerRelease_r9,
      { "radioBearerRelease-r9", "rrc.radioBearerRelease_r9",
        FT_NONE, BASE_NONE, NULL, 0,
        "RadioBearerRelease_r9_IEs", HFILL }},
    { &hf_rrc_radioBearerRelease_r9_add_ext,
      { "radioBearerRelease-r9-add-ext", "rrc.radioBearerRelease_r9_add_ext",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING", HFILL }},
    { &hf_rrc_v950NonCriticalExtensions_05,
      { "v950NonCriticalExtensions", "rrc.v950NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v950NonCriticalExtensions_05", HFILL }},
    { &hf_rrc_radioBearerRelease_v950ext,
      { "radioBearerRelease-v950ext", "rrc.radioBearerRelease_v950ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "RadioBearerRelease_v950ext_IEs", HFILL }},
    { &hf_rrc_v970NonCriticalExtensions_02,
      { "v970NonCriticalExtensions", "rrc.v970NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v970NonCriticalExtensions_02", HFILL }},
    { &hf_rrc_radioBearerRelease_v970ext,
      { "radioBearerRelease-v970ext", "rrc.radioBearerRelease_v970ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "RadioBearerRelease_v970ext_IEs", HFILL }},
    { &hf_rrc_v9c0NonCriticalExtensions_06,
      { "v9c0NonCriticalExtensions", "rrc.v9c0NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v9c0NonCriticalExtensions_06", HFILL }},
    { &hf_rrc_radioBearerRelease_v9c0ext,
      { "radioBearerRelease-v9c0ext", "rrc.radioBearerRelease_v9c0ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "RadioBearerRelease_v9c0ext_IEs", HFILL }},
    { &hf_rrc_nonCriticalExtensions_101,
      { "nonCriticalExtensions", "rrc.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_nonCriticalExtensions_101", HFILL }},
    { &hf_rrc_criticalExtensions_80,
      { "criticalExtensions", "rrc.criticalExtensions",
        FT_UINT32, BASE_DEC, VALS(rrc_T_criticalExtensions_79_vals), 0,
        "T_criticalExtensions_79", HFILL }},
    { &hf_rrc_r10_07,
      { "r10", "rrc.r10",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_r10_07", HFILL }},
    { &hf_rrc_radioBearerRelease_r10,
      { "radioBearerRelease-r10", "rrc.radioBearerRelease_r10",
        FT_NONE, BASE_NONE, NULL, 0,
        "RadioBearerRelease_r10_IEs", HFILL }},
    { &hf_rrc_radioBearerRelease_r10_add_ext,
      { "radioBearerRelease-r10-add-ext", "rrc.radioBearerRelease_r10_add_ext",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING", HFILL }},
    { &hf_rrc_v9c0NonCriticalExtensions_07,
      { "v9c0NonCriticalExtensions", "rrc.v9c0NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v9c0NonCriticalExtensions_07", HFILL }},
    { &hf_rrc_vbxyNonCriticalExtensions_10,
      { "vbxyNonCriticalExtensions", "rrc.vbxyNonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_vbxyNonCriticalExtensions_10", HFILL }},
    { &hf_rrc_radioBearerRelease_vbxyext,
      { "radioBearerRelease-vbxyext", "rrc.radioBearerRelease_vbxyext",
        FT_NONE, BASE_NONE, NULL, 0,
        "RadioBearerRelease_vbxyext_IEs", HFILL }},
    { &hf_rrc_nonCriticalExtensions_102,
      { "nonCriticalExtensions", "rrc.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_nonCriticalExtensions_102", HFILL }},
    { &hf_rrc_criticalExtensions_81,
      { "criticalExtensions", "rrc.criticalExtensions",
        FT_UINT32, BASE_DEC, VALS(rrc_T_criticalExtensions_80_vals), 0,
        "T_criticalExtensions_80", HFILL }},
    { &hf_rrc_r11_08,
      { "r11", "rrc.r11",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_r11_08", HFILL }},
    { &hf_rrc_radioBearerRelease_r11,
      { "radioBearerRelease-r11", "rrc.radioBearerRelease_r11",
        FT_NONE, BASE_NONE, NULL, 0,
        "RadioBearerRelease_r11_IEs", HFILL }},
    { &hf_rrc_radioBearerRelease_r11_add_ext,
      { "radioBearerRelease-r11-add-ext", "rrc.radioBearerRelease_r11_add_ext",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING", HFILL }},
    { &hf_rrc_nonCriticalExtensions_103,
      { "nonCriticalExtensions", "rrc.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_nonCriticalExtensions_103", HFILL }},
    { &hf_rrc_criticalExtensions_82,
      { "criticalExtensions", "rrc.criticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_criticalExtensions_81", HFILL }},
    { &hf_rrc_signallingConnectionRelIndication,
      { "signallingConnectionRelIndication", "rrc.signallingConnectionRelIndication",
        FT_UINT32, BASE_DEC, VALS(rrc_CN_DomainIdentity_vals), 0,
        "CN_DomainIdentity", HFILL }},
    { &hf_rrc_dummy_17,
      { "dummy", "rrc.dummy",
        FT_UINT32, BASE_DEC, VALS(rrc_T_dummy_04_vals), 0,
        "T_dummy_04", HFILL }},
    { &hf_rrc_fdd_27,
      { "fdd", "rrc.fdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_fdd_27", HFILL }},
    { &hf_rrc_modeSpecificPhysChInfo_06,
      { "modeSpecificPhysChInfo", "rrc.modeSpecificPhysChInfo",
        FT_UINT32, BASE_DEC, VALS(rrc_T_modeSpecificPhysChInfo_06_vals), 0,
        "T_modeSpecificPhysChInfo_06", HFILL }},
    { &hf_rrc_fdd_28,
      { "fdd", "rrc.fdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_fdd_28", HFILL }},
    { &hf_rrc_dummy_18,
      { "dummy", "rrc.dummy",
        FT_UINT32, BASE_DEC, VALS(rrc_T_dummy_05_vals), 0,
        "T_dummy_05", HFILL }},
    { &hf_rrc_fdd_29,
      { "fdd", "rrc.fdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_fdd_29", HFILL }},
    { &hf_rrc_modeSpecificPhysChInfo_07,
      { "modeSpecificPhysChInfo", "rrc.modeSpecificPhysChInfo",
        FT_UINT32, BASE_DEC, VALS(rrc_T_modeSpecificPhysChInfo_07_vals), 0,
        "T_modeSpecificPhysChInfo_07", HFILL }},
    { &hf_rrc_fdd_30,
      { "fdd", "rrc.fdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_fdd_30", HFILL }},
    { &hf_rrc_dummy_19,
      { "dummy", "rrc.dummy",
        FT_UINT32, BASE_DEC, VALS(rrc_T_dummy_06_vals), 0,
        "T_dummy_06", HFILL }},
    { &hf_rrc_fdd_31,
      { "fdd", "rrc.fdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_fdd_31", HFILL }},
    { &hf_rrc_modeSpecificPhysChInfo_08,
      { "modeSpecificPhysChInfo", "rrc.modeSpecificPhysChInfo",
        FT_UINT32, BASE_DEC, VALS(rrc_T_modeSpecificPhysChInfo_08_vals), 0,
        "T_modeSpecificPhysChInfo_08", HFILL }},
    { &hf_rrc_fdd_32,
      { "fdd", "rrc.fdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_fdd_32", HFILL }},
    { &hf_rrc_postVerificationPeriod_03,
      { "postVerificationPeriod", "rrc.postVerificationPeriod",
        FT_UINT32, BASE_DEC, VALS(rrc_T_postVerificationPeriod_03_vals), 0,
        "T_postVerificationPeriod_03", HFILL }},
    { &hf_rrc_mbms_RB_ListReleasedToChangeTransferMode,
      { "mbms-RB-ListReleasedToChangeTransferMode", "rrc.mbms_RB_ListReleasedToChangeTransferMode",
        FT_UINT32, BASE_DEC, NULL, 0,
        "RB_InformationReleaseList", HFILL }},
    { &hf_rrc_laterNonCriticalExtensions_32,
      { "laterNonCriticalExtensions", "rrc.laterNonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_laterNonCriticalExtensions_32", HFILL }},
    { &hf_rrc_radioBearerReleaseComplete_r3_add_ext,
      { "radioBearerReleaseComplete-r3-add-ext", "rrc.radioBearerReleaseComplete_r3_add_ext",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING", HFILL }},
    { &hf_rrc_v770NonCriticalExtensions_19,
      { "v770NonCriticalExtensions", "rrc.v770NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v770NonCriticalExtensions_19", HFILL }},
    { &hf_rrc_radioBearerReleaseComplete_v770ext,
      { "radioBearerReleaseComplete-v770ext", "rrc.radioBearerReleaseComplete_v770ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "RadioBearerReleaseComplete_v770ext_IEs", HFILL }},
    { &hf_rrc_nonCriticalExtensions_104,
      { "nonCriticalExtensions", "rrc.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_nonCriticalExtensions_104", HFILL }},
    { &hf_rrc_deferredMeasurementControlReading_02,
      { "deferredMeasurementControlReading", "rrc.deferredMeasurementControlReading",
        FT_UINT32, BASE_DEC, VALS(rrc_T_deferredMeasurementControlReading_02_vals), 0,
        "T_deferredMeasurementControlReading_02", HFILL }},
    { &hf_rrc_laterNonCriticalExtensions_33,
      { "laterNonCriticalExtensions", "rrc.laterNonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_laterNonCriticalExtensions_33", HFILL }},
    { &hf_rrc_radioBearerReleaseFailure_r3_add_ext,
      { "radioBearerReleaseFailure-r3-add-ext", "rrc.radioBearerReleaseFailure_r3_add_ext",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING", HFILL }},
    { &hf_rrc_nonCriticalExtensions_105,
      { "nonCriticalExtensions", "rrc.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_nonCriticalExtensions_105", HFILL }},
    { &hf_rrc_r3_16,
      { "r3", "rrc.r3",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_r3_16", HFILL }},
    { &hf_rrc_radioBearerSetup_r3,
      { "radioBearerSetup-r3", "rrc.radioBearerSetup_r3",
        FT_NONE, BASE_NONE, NULL, 0,
        "RadioBearerSetup_r3_IEs", HFILL }},
    { &hf_rrc_v3a0NonCriticalExtensions_07,
      { "v3a0NonCriticalExtensions", "rrc.v3a0NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v3a0NonCriticalExtensions_07", HFILL }},
    { &hf_rrc_radioBearerSetup_v3a0ext,
      { "radioBearerSetup-v3a0ext", "rrc.radioBearerSetup_v3a0ext",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_laterNonCriticalExtensions_34,
      { "laterNonCriticalExtensions", "rrc.laterNonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_laterNonCriticalExtensions_34", HFILL }},
    { &hf_rrc_radioBearerSetup_r3_add_ext,
      { "radioBearerSetup-r3-add-ext", "rrc.radioBearerSetup_r3_add_ext",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING", HFILL }},
    { &hf_rrc_v4b0NonCriticalExtensions_09,
      { "v4b0NonCriticalExtensions", "rrc.v4b0NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v4b0NonCriticalExtensions_09", HFILL }},
    { &hf_rrc_radioBearerSetup_v4b0ext,
      { "radioBearerSetup-v4b0ext", "rrc.radioBearerSetup_v4b0ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "RadioBearerSetup_v4b0ext_IEs", HFILL }},
    { &hf_rrc_v590NonCriticalExtensions_18,
      { "v590NonCriticalExtensions", "rrc.v590NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v590NonCriticalExtensions_18", HFILL }},
    { &hf_rrc_radioBearerSetup_v590ext,
      { "radioBearerSetup-v590ext", "rrc.radioBearerSetup_v590ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "RadioBearerSetup_v590ext_IEs", HFILL }},
    { &hf_rrc_v5d0NonCriticalExtenstions_09,
      { "v5d0NonCriticalExtenstions", "rrc.v5d0NonCriticalExtenstions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v5d0NonCriticalExtenstions_09", HFILL }},
    { &hf_rrc_radioBearerSetup_v5d0ext,
      { "radioBearerSetup-v5d0ext", "rrc.radioBearerSetup_v5d0ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "RadioBearerSetup_v5d0ext_IEs", HFILL }},
    { &hf_rrc_v690NonCriticalExtensions_22,
      { "v690NonCriticalExtensions", "rrc.v690NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v690NonCriticalExtensions_22", HFILL }},
    { &hf_rrc_radioBearerSetup_v690ext,
      { "radioBearerSetup-v690ext", "rrc.radioBearerSetup_v690ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "RadioBearerSetup_v690ext_IEs", HFILL }},
    { &hf_rrc_nonCriticalExtensions_106,
      { "nonCriticalExtensions", "rrc.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_nonCriticalExtensions_106", HFILL }},
    { &hf_rrc_later_than_r3_14,
      { "later-than-r3", "rrc.later_than_r3",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_later_than_r3_14", HFILL }},
    { &hf_rrc_criticalExtensions_83,
      { "criticalExtensions", "rrc.criticalExtensions",
        FT_UINT32, BASE_DEC, VALS(rrc_T_criticalExtensions_82_vals), 0,
        "T_criticalExtensions_82", HFILL }},
    { &hf_rrc_r4_08,
      { "r4", "rrc.r4",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_r4_08", HFILL }},
    { &hf_rrc_radioBearerSetup_r4,
      { "radioBearerSetup-r4", "rrc.radioBearerSetup_r4",
        FT_NONE, BASE_NONE, NULL, 0,
        "RadioBearerSetup_r4_IEs", HFILL }},
    { &hf_rrc_v4d0NonCriticalExtensions_08,
      { "v4d0NonCriticalExtensions", "rrc.v4d0NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v4d0NonCriticalExtensions_08", HFILL }},
    { &hf_rrc_radioBearerSetup_r4_add_ext,
      { "radioBearerSetup-r4-add-ext", "rrc.radioBearerSetup_r4_add_ext",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING", HFILL }},
    { &hf_rrc_v590NonCriticalExtensions_19,
      { "v590NonCriticalExtensions", "rrc.v590NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v590NonCriticalExtensions_19", HFILL }},
    { &hf_rrc_v5d0NonCriticalExtenstions_10,
      { "v5d0NonCriticalExtenstions", "rrc.v5d0NonCriticalExtenstions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v5d0NonCriticalExtenstions_10", HFILL }},
    { &hf_rrc_v690NonCriticalExtensions_23,
      { "v690NonCriticalExtensions", "rrc.v690NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v690NonCriticalExtensions_23", HFILL }},
    { &hf_rrc_nonCriticalExtensions_107,
      { "nonCriticalExtensions", "rrc.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_nonCriticalExtensions_107", HFILL }},
    { &hf_rrc_criticalExtensions_84,
      { "criticalExtensions", "rrc.criticalExtensions",
        FT_UINT32, BASE_DEC, VALS(rrc_T_criticalExtensions_83_vals), 0,
        "T_criticalExtensions_83", HFILL }},
    { &hf_rrc_r5_07,
      { "r5", "rrc.r5",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_r5_07", HFILL }},
    { &hf_rrc_radioBearerSetup_r5,
      { "radioBearerSetup-r5", "rrc.radioBearerSetup_r5",
        FT_NONE, BASE_NONE, NULL, 0,
        "RadioBearerSetup_r5_IEs", HFILL }},
    { &hf_rrc_radioBearerSetup_r5_add_ext,
      { "radioBearerSetup-r5-add-ext", "rrc.radioBearerSetup_r5_add_ext",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING", HFILL }},
    { &hf_rrc_v5d0NonCriticalExtenstions_11,
      { "v5d0NonCriticalExtenstions", "rrc.v5d0NonCriticalExtenstions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v5d0NonCriticalExtenstions_11", HFILL }},
    { &hf_rrc_v690NonCriticalExtensions_24,
      { "v690NonCriticalExtensions", "rrc.v690NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v690NonCriticalExtensions_24", HFILL }},
    { &hf_rrc_nonCriticalExtensions_108,
      { "nonCriticalExtensions", "rrc.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_nonCriticalExtensions_108", HFILL }},
    { &hf_rrc_criticalExtensions_85,
      { "criticalExtensions", "rrc.criticalExtensions",
        FT_UINT32, BASE_DEC, VALS(rrc_T_criticalExtensions_84_vals), 0,
        "T_criticalExtensions_84", HFILL }},
    { &hf_rrc_r6_09,
      { "r6", "rrc.r6",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_r6_09", HFILL }},
    { &hf_rrc_radioBearerSetup_r6,
      { "radioBearerSetup-r6", "rrc.radioBearerSetup_r6",
        FT_NONE, BASE_NONE, NULL, 0,
        "RadioBearerSetup_r6_IEs", HFILL }},
    { &hf_rrc_radioBearerSetup_r6_add_ext,
      { "radioBearerSetup-r6-add-ext", "rrc.radioBearerSetup_r6_add_ext",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING", HFILL }},
    { &hf_rrc_v6b0NonCriticalExtensions_09,
      { "v6b0NonCriticalExtensions", "rrc.v6b0NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v6b0NonCriticalExtensions_08", HFILL }},
    { &hf_rrc_radioBearerSetup_v6b0ext,
      { "radioBearerSetup-v6b0ext", "rrc.radioBearerSetup_v6b0ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "RadioBearerSetup_v6b0ext_IEs", HFILL }},
    { &hf_rrc_nonCriticalExtensions_109,
      { "nonCriticalExtensions", "rrc.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_nonCriticalExtensions_109", HFILL }},
    { &hf_rrc_criticalExtensions_86,
      { "criticalExtensions", "rrc.criticalExtensions",
        FT_UINT32, BASE_DEC, VALS(rrc_T_criticalExtensions_85_vals), 0,
        "T_criticalExtensions_85", HFILL }},
    { &hf_rrc_r7_08,
      { "r7", "rrc.r7",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_r7_08", HFILL }},
    { &hf_rrc_radioBearerSetup_r7,
      { "radioBearerSetup-r7", "rrc.radioBearerSetup_r7",
        FT_NONE, BASE_NONE, NULL, 0,
        "RadioBearerSetup_r7_IEs", HFILL }},
    { &hf_rrc_radioBearerSetup_r7_add_ext,
      { "radioBearerSetup-r7-add-ext", "rrc.radioBearerSetup_r7_add_ext",
        FT_BYTES, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_v780NonCriticalExtensions_07,
      { "v780NonCriticalExtensions", "rrc.v780NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v780NonCriticalExtensions_07", HFILL }},
    { &hf_rrc_radioBearerSetup_v780ext,
      { "radioBearerSetup-v780ext", "rrc.radioBearerSetup_v780ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "RadioBearerSetup_v780ext_IEs", HFILL }},
    { &hf_rrc_v820NonCriticalExtensions_01,
      { "v820NonCriticalExtensions", "rrc.v820NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v820NonCriticalExtensions_01", HFILL }},
    { &hf_rrc_radioBearerSetup_v820ext,
      { "radioBearerSetup-v820ext", "rrc.radioBearerSetup_v820ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "RadioBearerSetup_v820ext_IEs", HFILL }},
    { &hf_rrc_nonCriticalExtensions_110,
      { "nonCriticalExtensions", "rrc.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_nonCriticalExtensions_110", HFILL }},
    { &hf_rrc_criticalExtensions_87,
      { "criticalExtensions", "rrc.criticalExtensions",
        FT_UINT32, BASE_DEC, VALS(rrc_T_criticalExtensions_86_vals), 0,
        "T_criticalExtensions_86", HFILL }},
    { &hf_rrc_r8_09,
      { "r8", "rrc.r8",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_r8_09", HFILL }},
    { &hf_rrc_radioBearerSetup_r8,
      { "radioBearerSetup-r8", "rrc.radioBearerSetup_r8",
        FT_NONE, BASE_NONE, NULL, 0,
        "RadioBearerSetup_r8_IEs", HFILL }},
    { &hf_rrc_radioBearerSetup_r8_add_ext,
      { "radioBearerSetup-r8-add-ext", "rrc.radioBearerSetup_r8_add_ext",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING", HFILL }},
    { &hf_rrc_v7d0NonCriticalExtensions_10,
      { "v7d0NonCriticalExtensions", "rrc.v7d0NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v7d0NonCriticalExtensions_10", HFILL }},
    { &hf_rrc_radioBearerSetup_v7d0ext,
      { "radioBearerSetup-v7d0ext", "rrc.radioBearerSetup_v7d0ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "RadioBearerSetup_v7d0ext_IEs", HFILL }},
    { &hf_rrc_v7f0NonCriticalExtensions_11,
      { "v7f0NonCriticalExtensions", "rrc.v7f0NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v7f0NonCriticalExtensions_11", HFILL }},
    { &hf_rrc_radioBearerSetup_v7f0ext,
      { "radioBearerSetup-v7f0ext", "rrc.radioBearerSetup_v7f0ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "RadioBearerSetup_v7f0ext_IEs", HFILL }},
    { &hf_rrc_v890NonCriticalExtensions_07,
      { "v890NonCriticalExtensions", "rrc.v890NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v890NonCriticalExtensions_07", HFILL }},
    { &hf_rrc_radioBearerSetup_v890ext,
      { "radioBearerSetup-v890ext", "rrc.radioBearerSetup_v890ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "RadioBearerSetup_v890ext_IEs", HFILL }},
    { &hf_rrc_v7g0NonCriticalExtensions_12,
      { "v7g0NonCriticalExtensions", "rrc.v7g0NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v7g0NonCriticalExtensions_12", HFILL }},
    { &hf_rrc_radioBearerSetup_v7g0ext,
      { "radioBearerSetup-v7g0ext", "rrc.radioBearerSetup_v7g0ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "RadioBearerSetup_v7g0ext_IEs", HFILL }},
    { &hf_rrc_v8a0NonCriticalExtensions_07,
      { "v8a0NonCriticalExtensions", "rrc.v8a0NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v8a0NonCriticalExtensions_07", HFILL }},
    { &hf_rrc_radioBearerSetup_v8a0ext,
      { "radioBearerSetup-v8a0ext", "rrc.radioBearerSetup_v8a0ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "RadioBearerSetup_v8a0ext_IEs", HFILL }},
    { &hf_rrc_nonCriticalExtensions_111,
      { "nonCriticalExtensions", "rrc.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_nonCriticalExtensions_111", HFILL }},
    { &hf_rrc_criticalExtensions_88,
      { "criticalExtensions", "rrc.criticalExtensions",
        FT_UINT32, BASE_DEC, VALS(rrc_T_criticalExtensions_87_vals), 0,
        "T_criticalExtensions_87", HFILL }},
    { &hf_rrc_r9_08,
      { "r9", "rrc.r9",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_r9_08", HFILL }},
    { &hf_rrc_radioBearerSetup_r9,
      { "radioBearerSetup-r9", "rrc.radioBearerSetup_r9",
        FT_NONE, BASE_NONE, NULL, 0,
        "RadioBearerSetup_r9_IEs", HFILL }},
    { &hf_rrc_radioBearerSetup_r9_add_ext,
      { "radioBearerSetup-r9-add-ext", "rrc.radioBearerSetup_r9_add_ext",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING", HFILL }},
    { &hf_rrc_v950NonCriticalExtensions_06,
      { "v950NonCriticalExtensions", "rrc.v950NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v950NonCriticalExtensions_06", HFILL }},
    { &hf_rrc_radioBearerSetup_v950ext,
      { "radioBearerSetup-v950ext", "rrc.radioBearerSetup_v950ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "RadioBearerSetup_v950ext_IEs", HFILL }},
    { &hf_rrc_v9c0NonCriticalExtensions_08,
      { "v9c0NonCriticalExtensions", "rrc.v9c0NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v9c0NonCriticalExtensions_08", HFILL }},
    { &hf_rrc_radioBearerSetup_v9c0ext,
      { "radioBearerSetup-v9c0ext", "rrc.radioBearerSetup_v9c0ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "RadioBearerSetup_v9c0ext_IEs", HFILL }},
    { &hf_rrc_nonCriticalExtensions_112,
      { "nonCriticalExtensions", "rrc.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_nonCriticalExtensions_112", HFILL }},
    { &hf_rrc_criticalExtensions_89,
      { "criticalExtensions", "rrc.criticalExtensions",
        FT_UINT32, BASE_DEC, VALS(rrc_T_criticalExtensions_88_vals), 0,
        "T_criticalExtensions_88", HFILL }},
    { &hf_rrc_r10_08,
      { "r10", "rrc.r10",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_r10_08", HFILL }},
    { &hf_rrc_radioBearerSetup_r10,
      { "radioBearerSetup-r10", "rrc.radioBearerSetup_r10",
        FT_NONE, BASE_NONE, NULL, 0,
        "RadioBearerSetup_r10_IEs", HFILL }},
    { &hf_rrc_radioBearerSetup_r10_add_ext,
      { "radioBearerSetup-r10-add-ext", "rrc.radioBearerSetup_r10_add_ext",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING", HFILL }},
    { &hf_rrc_v9c0NonCriticalExtensions_09,
      { "v9c0NonCriticalExtensions", "rrc.v9c0NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v9c0NonCriticalExtensions_09", HFILL }},
    { &hf_rrc_vbxyNonCriticalExtensions_11,
      { "vbxyNonCriticalExtensions", "rrc.vbxyNonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_vbxyNonCriticalExtensions_11", HFILL }},
    { &hf_rrc_radioBearerSetup_vbxyext,
      { "radioBearerSetup-vbxyext", "rrc.radioBearerSetup_vbxyext",
        FT_NONE, BASE_NONE, NULL, 0,
        "RadioBearerSetup_vbxyext_IEs", HFILL }},
    { &hf_rrc_nonCriticalExtensions_113,
      { "nonCriticalExtensions", "rrc.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_nonCriticalExtensions_113", HFILL }},
    { &hf_rrc_criticalExtensions_90,
      { "criticalExtensions", "rrc.criticalExtensions",
        FT_UINT32, BASE_DEC, VALS(rrc_T_criticalExtensions_89_vals), 0,
        "T_criticalExtensions_89", HFILL }},
    { &hf_rrc_r11_09,
      { "r11", "rrc.r11",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_r11_09", HFILL }},
    { &hf_rrc_radioBearerSetup_r11,
      { "radioBearerSetup-r11", "rrc.radioBearerSetup_r11",
        FT_NONE, BASE_NONE, NULL, 0,
        "RadioBearerSetup_r11_IEs", HFILL }},
    { &hf_rrc_radioBearerSetup_r11_add_ext,
      { "radioBearerSetup-r11-add-ext", "rrc.radioBearerSetup_r11_add_ext",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING", HFILL }},
    { &hf_rrc_nonCriticalExtensions_114,
      { "nonCriticalExtensions", "rrc.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_nonCriticalExtensions_114", HFILL }},
    { &hf_rrc_criticalExtensions_91,
      { "criticalExtensions", "rrc.criticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_criticalExtensions_90", HFILL }},
    { &hf_rrc_dummy_20,
      { "dummy", "rrc.dummy",
        FT_UINT32, BASE_DEC, VALS(rrc_T_dummy_07_vals), 0,
        "T_dummy_07", HFILL }},
    { &hf_rrc_fdd_33,
      { "fdd", "rrc.fdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_fdd_33", HFILL }},
    { &hf_rrc_modeSpecificPhysChInfo_09,
      { "modeSpecificPhysChInfo", "rrc.modeSpecificPhysChInfo",
        FT_UINT32, BASE_DEC, VALS(rrc_T_modeSpecificPhysChInfo_09_vals), 0,
        "T_modeSpecificPhysChInfo_09", HFILL }},
    { &hf_rrc_fdd_34,
      { "fdd", "rrc.fdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_fdd_34", HFILL }},
    { &hf_rrc_dummy_21,
      { "dummy", "rrc.dummy",
        FT_UINT32, BASE_DEC, VALS(rrc_T_dummy_08_vals), 0,
        "T_dummy_08", HFILL }},
    { &hf_rrc_fdd_35,
      { "fdd", "rrc.fdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_fdd_35", HFILL }},
    { &hf_rrc_modeSpecificPhysChInfo_10,
      { "modeSpecificPhysChInfo", "rrc.modeSpecificPhysChInfo",
        FT_UINT32, BASE_DEC, VALS(rrc_T_modeSpecificPhysChInfo_10_vals), 0,
        "T_modeSpecificPhysChInfo_10", HFILL }},
    { &hf_rrc_fdd_36,
      { "fdd", "rrc.fdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_fdd_36", HFILL }},
    { &hf_rrc_dummy_22,
      { "dummy", "rrc.dummy",
        FT_UINT32, BASE_DEC, VALS(rrc_T_dummy_09_vals), 0,
        "T_dummy_09", HFILL }},
    { &hf_rrc_fdd_37,
      { "fdd", "rrc.fdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_fdd_37", HFILL }},
    { &hf_rrc_modeSpecificPhysChInfo_11,
      { "modeSpecificPhysChInfo", "rrc.modeSpecificPhysChInfo",
        FT_UINT32, BASE_DEC, VALS(rrc_T_modeSpecificPhysChInfo_11_vals), 0,
        "T_modeSpecificPhysChInfo_11", HFILL }},
    { &hf_rrc_fdd_38,
      { "fdd", "rrc.fdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_fdd_38", HFILL }},
    { &hf_rrc_postVerificationPeriod_04,
      { "postVerificationPeriod", "rrc.postVerificationPeriod",
        FT_UINT32, BASE_DEC, VALS(rrc_T_postVerificationPeriod_04_vals), 0,
        "T_postVerificationPeriod_04", HFILL }},
    { &hf_rrc_rab_InformationSetupList_08,
      { "rab-InformationSetupList", "rrc.rab_InformationSetupList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "RAB_InformationSetupList_r6_ext", HFILL }},
    { &hf_rrc_specificationMode_17,
      { "specificationMode", "rrc.specificationMode",
        FT_UINT32, BASE_DEC, VALS(rrc_T_specificationMode_17_vals), 0,
        "T_specificationMode_17", HFILL }},
    { &hf_rrc_complete_17,
      { "complete", "rrc.complete",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_complete_17", HFILL }},
    { &hf_rrc_dummy_23,
      { "dummy", "rrc.dummy",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_dummy_10", HFILL }},
    { &hf_rrc_rab_Info_01,
      { "rab-Info", "rrc.rab_Info",
        FT_NONE, BASE_NONE, NULL, 0,
        "RAB_Info_r6", HFILL }},
    { &hf_rrc_rb_InformationChangedList,
      { "rb-InformationChangedList", "rrc.rb_InformationChangedList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "RB_InformationChangedList_r6", HFILL }},
    { &hf_rrc_powerOffsetInfoShort,
      { "powerOffsetInfoShort", "rrc.powerOffsetInfoShort",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_rab_InformationSetupListExt,
      { "rab-InformationSetupListExt", "rrc.rab_InformationSetupListExt",
        FT_UINT32, BASE_DEC, NULL, 0,
        "RAB_InformationSetupList_v6b0ext", HFILL }},
    { &hf_rrc_specificationMode_18,
      { "specificationMode", "rrc.specificationMode",
        FT_UINT32, BASE_DEC, VALS(rrc_T_specificationMode_18_vals), 0,
        "T_specificationMode_18", HFILL }},
    { &hf_rrc_complete_18,
      { "complete", "rrc.complete",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_complete_18", HFILL }},
    { &hf_rrc_dummy_24,
      { "dummy", "rrc.dummy",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_v7f0NonCriticalExtensions_12,
      { "v7f0NonCriticalExtensions", "rrc.v7f0NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v7f0NonCriticalExtensions_12", HFILL }},
    { &hf_rrc_v7g0NonCriticalExtensions_13,
      { "v7g0NonCriticalExtensions", "rrc.v7g0NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v7g0NonCriticalExtensions_13", HFILL }},
    { &hf_rrc_nonCriticalExtensions_115,
      { "nonCriticalExtensions", "rrc.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_nonCriticalExtensions_115", HFILL }},
    { &hf_rrc_sr_vcc_Info,
      { "sr-vcc-Info", "rrc.sr_vcc_Info",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_laterNonCriticalExtensions_35,
      { "laterNonCriticalExtensions", "rrc.laterNonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_laterNonCriticalExtensions_35", HFILL }},
    { &hf_rrc_radioBearerSetupComplete_r3_add_ext,
      { "radioBearerSetupComplete-r3-add-ext", "rrc.radioBearerSetupComplete_r3_add_ext",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING", HFILL }},
    { &hf_rrc_v770NonCriticalExtensions_20,
      { "v770NonCriticalExtensions", "rrc.v770NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v770NonCriticalExtensions_20", HFILL }},
    { &hf_rrc_radioBearerSetupComplete_v770ext,
      { "radioBearerSetupComplete-v770ext", "rrc.radioBearerSetupComplete_v770ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "RadioBearerSetupComplete_v770ext_IEs", HFILL }},
    { &hf_rrc_nonCriticalExtensions_116,
      { "nonCriticalExtensions", "rrc.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_nonCriticalExtensions_116", HFILL }},
    { &hf_rrc_deferredMeasurementControlReading_03,
      { "deferredMeasurementControlReading", "rrc.deferredMeasurementControlReading",
        FT_UINT32, BASE_DEC, VALS(rrc_T_deferredMeasurementControlReading_03_vals), 0,
        "T_deferredMeasurementControlReading_03", HFILL }},
    { &hf_rrc_laterNonCriticalExtensions_36,
      { "laterNonCriticalExtensions", "rrc.laterNonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_laterNonCriticalExtensions_36", HFILL }},
    { &hf_rrc_radioBearerSetupFailure_r3_add_ext,
      { "radioBearerSetupFailure-r3-add-ext", "rrc.radioBearerSetupFailure_r3_add_ext",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING", HFILL }},
    { &hf_rrc_nonCriticalExtensions_117,
      { "nonCriticalExtensions", "rrc.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_nonCriticalExtensions_117", HFILL }},
    { &hf_rrc_r3_17,
      { "r3", "rrc.r3",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_r3_17", HFILL }},
    { &hf_rrc_rrcConnectionReject_r3,
      { "rrcConnectionReject-r3", "rrc.rrcConnectionReject_r3",
        FT_NONE, BASE_NONE, NULL, 0,
        "RRCConnectionReject_r3_IEs", HFILL }},
    { &hf_rrc_laterNonCriticalExtensions_37,
      { "laterNonCriticalExtensions", "rrc.laterNonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_laterNonCriticalExtensions_37", HFILL }},
    { &hf_rrc_rrcConnectionReject_r3_add_ext,
      { "rrcConnectionReject-r3-add-ext", "rrc.rrcConnectionReject_r3_add_ext",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING", HFILL }},
    { &hf_rrc_v690NonCriticalExtensions_25,
      { "v690NonCriticalExtensions", "rrc.v690NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v690NonCriticalExtensions_25", HFILL }},
    { &hf_rrc_rrcConnectionReject_v690ext,
      { "rrcConnectionReject-v690ext", "rrc.rrcConnectionReject_v690ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "RRCConnectionReject_v690ext_IEs", HFILL }},
    { &hf_rrc_v6f0NonCriticalExtensions_01,
      { "v6f0NonCriticalExtensions", "rrc.v6f0NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v6f0NonCriticalExtensions_01", HFILL }},
    { &hf_rrc_rrcConnectionReject_v6f0ext,
      { "rrcConnectionReject-v6f0ext", "rrc.rrcConnectionReject_v6f0ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "RRCConnectionReject_v6f0ext_IEs", HFILL }},
    { &hf_rrc_v860NonCriticalExtensions_10,
      { "v860NonCriticalExtensions", "rrc.v860NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v860NonCriticalExtensions_10", HFILL }},
    { &hf_rrc_rrcConnectionReject_v860ext,
      { "rrcConnectionReject-v860ext", "rrc.rrcConnectionReject_v860ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "RRCConnectionReject_v860ext_IEs", HFILL }},
    { &hf_rrc_va40NonCriticalExtensions_05,
      { "va40NonCriticalExtensions", "rrc.va40NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_va40NonCriticalExtensions_05", HFILL }},
    { &hf_rrc_rrcConnectionReject_va40ext,
      { "rrcConnectionReject-va40ext", "rrc.rrcConnectionReject_va40ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "RRCConnectionReject_va40ext_IEs", HFILL }},
    { &hf_rrc_nonCriticalExtensions_118,
      { "nonCriticalExtensions", "rrc.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_nonCriticalExtensions_118", HFILL }},
    { &hf_rrc_later_than_r3_15,
      { "later-than-r3", "rrc.later_than_r3",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_later_than_r3_15", HFILL }},
    { &hf_rrc_initialUE_Identity,
      { "initialUE-Identity", "rrc.initialUE_Identity",
        FT_UINT32, BASE_DEC, VALS(rrc_InitialUE_Identity_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_criticalExtensions_92,
      { "criticalExtensions", "rrc.criticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_criticalExtensions_91", HFILL }},
    { &hf_rrc_rejectionCause,
      { "rejectionCause", "rrc.rejectionCause",
        FT_UINT32, BASE_DEC, VALS(rrc_RejectionCause_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_redirectionInfo,
      { "redirectionInfo", "rrc.redirectionInfo",
        FT_UINT32, BASE_DEC, VALS(rrc_RedirectionInfo_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_redirectionInfo_v690ext,
      { "redirectionInfo-v690ext", "rrc.redirectionInfo_v690ext",
        FT_UINT32, BASE_DEC, NULL, 0,
        "GSM_TargetCellInfoList", HFILL }},
    { &hf_rrc_countingCompletion,
      { "countingCompletion", "rrc.countingCompletion",
        FT_UINT32, BASE_DEC, VALS(rrc_T_countingCompletion_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_redirectionInfo_01,
      { "redirectionInfo", "rrc.redirectionInfo",
        FT_UINT32, BASE_DEC, VALS(rrc_RedirectionInfo_v860ext_vals), 0,
        "RedirectionInfo_v860ext", HFILL }},
    { &hf_rrc_extendedWaitTime,
      { "extendedWaitTime", "rrc.extendedWaitTime",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_r3_18,
      { "r3", "rrc.r3",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_r3_18", HFILL }},
    { &hf_rrc_rrcConnectionRelease_r3,
      { "rrcConnectionRelease-r3", "rrc.rrcConnectionRelease_r3",
        FT_NONE, BASE_NONE, NULL, 0,
        "RRCConnectionRelease_r3_IEs", HFILL }},
    { &hf_rrc_laterNonCriticalExtensions_38,
      { "laterNonCriticalExtensions", "rrc.laterNonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_laterNonCriticalExtensions_38", HFILL }},
    { &hf_rrc_rrcConnectionRelease_r3_add_ext,
      { "rrcConnectionRelease-r3-add-ext", "rrc.rrcConnectionRelease_r3_add_ext",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING", HFILL }},
    { &hf_rrc_v690NonCriticalExtensions_26,
      { "v690NonCriticalExtensions", "rrc.v690NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v690NonCriticalExtensions_26", HFILL }},
    { &hf_rrc_rrcConnectionRelease_v690ext,
      { "rrcConnectionRelease-v690ext", "rrc.rrcConnectionRelease_v690ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "RRCConnectionRelease_v690ext_IEs", HFILL }},
    { &hf_rrc_v770NonCriticalExtensions_21,
      { "v770NonCriticalExtensions", "rrc.v770NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v770NonCriticalExtensions_21", HFILL }},
    { &hf_rrc_rrcConnectionRelease_v770ext,
      { "rrcConnectionRelease-v770ext", "rrc.rrcConnectionRelease_v770ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "RRCConnectionRelease_v770ext_IEs", HFILL }},
    { &hf_rrc_v860NonCriticalExtensions_11,
      { "v860NonCriticalExtensions", "rrc.v860NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v860NonCriticalExtensions_11", HFILL }},
    { &hf_rrc_rrcConnectionRelease_v860ext,
      { "rrcConnectionRelease-v860ext", "rrc.rrcConnectionRelease_v860ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "RRCConnectionRelease_v860ext_IEs", HFILL }},
    { &hf_rrc_va40NonCriticalExtensions_06,
      { "va40NonCriticalExtensions", "rrc.va40NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_va40NonCriticalExtensions_06", HFILL }},
    { &hf_rrc_rrcConnectionRelease_va40ext,
      { "rrcConnectionRelease-va40ext", "rrc.rrcConnectionRelease_va40ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "RRCConnectionRelease_va40ext_IEs", HFILL }},
    { &hf_rrc_nonCriticalExtensions_119,
      { "nonCriticalExtensions", "rrc.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_nonCriticalExtensions_119", HFILL }},
    { &hf_rrc_later_than_r3_16,
      { "later-than-r3", "rrc.later_than_r3",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_later_than_r3_16", HFILL }},
    { &hf_rrc_criticalExtensions_93,
      { "criticalExtensions", "rrc.criticalExtensions",
        FT_UINT32, BASE_DEC, VALS(rrc_T_criticalExtensions_92_vals), 0,
        "T_criticalExtensions_92", HFILL }},
    { &hf_rrc_r4_09,
      { "r4", "rrc.r4",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_r4_09", HFILL }},
    { &hf_rrc_rrcConnectionRelease_r4,
      { "rrcConnectionRelease-r4", "rrc.rrcConnectionRelease_r4",
        FT_NONE, BASE_NONE, NULL, 0,
        "RRCConnectionRelease_r4_IEs", HFILL }},
    { &hf_rrc_v4d0NonCriticalExtensions_09,
      { "v4d0NonCriticalExtensions", "rrc.v4d0NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v4d0NonCriticalExtensions_09", HFILL }},
    { &hf_rrc_rrcConnectionRelease_r4_add_ext,
      { "rrcConnectionRelease-r4-add-ext", "rrc.rrcConnectionRelease_r4_add_ext",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING", HFILL }},
    { &hf_rrc_v690NonCriticalExtensions_27,
      { "v690NonCriticalExtensions", "rrc.v690NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v690NonCriticalExtensions_27", HFILL }},
    { &hf_rrc_v770NonCriticalExtensions_22,
      { "v770NonCriticalExtensions", "rrc.v770NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v770NonCriticalExtensions_22", HFILL }},
    { &hf_rrc_v860NonCriticalExtensions_12,
      { "v860NonCriticalExtensions", "rrc.v860NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v860NonCriticalExtensions_12", HFILL }},
    { &hf_rrc_va40NonCriticalExtensions_07,
      { "va40NonCriticalExtensions", "rrc.va40NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_va40NonCriticalExtensions_07", HFILL }},
    { &hf_rrc_nonCriticalExtensions_120,
      { "nonCriticalExtensions", "rrc.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_nonCriticalExtensions_120", HFILL }},
    { &hf_rrc_criticalExtensions_94,
      { "criticalExtensions", "rrc.criticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_criticalExtensions_93", HFILL }},
    { &hf_rrc_n_308,
      { "n-308", "rrc.n_308",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_releaseCause,
      { "releaseCause", "rrc.releaseCause",
        FT_UINT32, BASE_DEC, VALS(rrc_ReleaseCause_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_rplmn_information,
      { "rplmn-information", "rrc.rplmn_information",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_rplmn_information_01,
      { "rplmn-information", "rrc.rplmn_information",
        FT_NONE, BASE_NONE, NULL, 0,
        "Rplmn_Information_r4", HFILL }},
    { &hf_rrc_redirectionInfo_v690ext_01,
      { "redirectionInfo-v690ext", "rrc.redirectionInfo_v690ext",
        FT_UINT32, BASE_DEC, VALS(rrc_RedirectionInfo_r6_vals), 0,
        "RedirectionInfo_r6", HFILL }},
    { &hf_rrc_r3_19,
      { "r3", "rrc.r3",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_r3_19", HFILL }},
    { &hf_rrc_rrcConnectionRelease_CCCH_r3,
      { "rrcConnectionRelease-CCCH-r3", "rrc.rrcConnectionRelease_CCCH_r3",
        FT_NONE, BASE_NONE, NULL, 0,
        "RRCConnectionRelease_CCCH_r3_IEs", HFILL }},
    { &hf_rrc_laterNonCriticalExtensions_39,
      { "laterNonCriticalExtensions", "rrc.laterNonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_laterNonCriticalExtensions_39", HFILL }},
    { &hf_rrc_rrcConnectionRelease_CCCH_r3_add_ext,
      { "rrcConnectionRelease-CCCH-r3-add-ext", "rrc.rrcConnectionRelease_CCCH_r3_add_ext",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING", HFILL }},
    { &hf_rrc_v690NonCriticalExtensions_28,
      { "v690NonCriticalExtensions", "rrc.v690NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v690NonCriticalExtensions_28", HFILL }},
    { &hf_rrc_rrcConnectionRelease_v690ext_01,
      { "rrcConnectionRelease-v690ext", "rrc.rrcConnectionRelease_v690ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "RRCConnectionRelease_CCCH_v690ext_IEs", HFILL }},
    { &hf_rrc_v860NonCriticalExtensions_13,
      { "v860NonCriticalExtensions", "rrc.v860NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v860NonCriticalExtensions_13", HFILL }},
    { &hf_rrc_rrcConnectionRelease_v860ext_01,
      { "rrcConnectionRelease-v860ext", "rrc.rrcConnectionRelease_v860ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "RRCConnectionRelease_CCCH_v860ext_IEs", HFILL }},
    { &hf_rrc_va40NonCriticalExtensions_08,
      { "va40NonCriticalExtensions", "rrc.va40NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_va40NonCriticalExtensions_08", HFILL }},
    { &hf_rrc_nonCriticalExtensions_121,
      { "nonCriticalExtensions", "rrc.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_nonCriticalExtensions_121", HFILL }},
    { &hf_rrc_later_than_r3_17,
      { "later-than-r3", "rrc.later_than_r3",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_later_than_r3_17", HFILL }},
    { &hf_rrc_criticalExtensions_95,
      { "criticalExtensions", "rrc.criticalExtensions",
        FT_UINT32, BASE_DEC, VALS(rrc_T_criticalExtensions_94_vals), 0,
        "T_criticalExtensions_94", HFILL }},
    { &hf_rrc_r4_10,
      { "r4", "rrc.r4",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_r4_10", HFILL }},
    { &hf_rrc_rrcConnectionRelease_CCCH_r4,
      { "rrcConnectionRelease-CCCH-r4", "rrc.rrcConnectionRelease_CCCH_r4",
        FT_NONE, BASE_NONE, NULL, 0,
        "RRCConnectionRelease_CCCH_r4_IEs", HFILL }},
    { &hf_rrc_v4d0NonCriticalExtensions_10,
      { "v4d0NonCriticalExtensions", "rrc.v4d0NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v4d0NonCriticalExtensions_10", HFILL }},
    { &hf_rrc_rrcConnectionRelease_CCCH_r4_add_ext,
      { "rrcConnectionRelease-CCCH-r4-add-ext", "rrc.rrcConnectionRelease_CCCH_r4_add_ext",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING", HFILL }},
    { &hf_rrc_v690NonCriticalExtensions_29,
      { "v690NonCriticalExtensions", "rrc.v690NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v690NonCriticalExtensions_29", HFILL }},
    { &hf_rrc_v860NonCriticalExtensions_14,
      { "v860NonCriticalExtensions", "rrc.v860NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v860NonCriticalExtensions_14", HFILL }},
    { &hf_rrc_va40NonCriticalExtensions_09,
      { "va40NonCriticalExtensions", "rrc.va40NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_va40NonCriticalExtensions_09", HFILL }},
    { &hf_rrc_nonCriticalExtensions_122,
      { "nonCriticalExtensions", "rrc.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_nonCriticalExtensions_122", HFILL }},
    { &hf_rrc_criticalExtensions_96,
      { "criticalExtensions", "rrc.criticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_criticalExtensions_95", HFILL }},
    { &hf_rrc_groupIdentity,
      { "groupIdentity", "rrc.groupIdentity",
        FT_UINT32, BASE_DEC, NULL, 0,
        "SEQUENCE_SIZE_1_maxURNTI_Group_OF_GroupReleaseInformation", HFILL }},
    { &hf_rrc_groupIdentity_item,
      { "GroupReleaseInformation", "rrc.GroupReleaseInformation",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_criticalExtensions_97,
      { "criticalExtensions", "rrc.criticalExtensions",
        FT_UINT32, BASE_DEC, VALS(rrc_T_criticalExtensions_96_vals), 0,
        "T_criticalExtensions_96", HFILL }},
    { &hf_rrc_r5_08,
      { "r5", "rrc.r5",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_r5_08", HFILL }},
    { &hf_rrc_rrcConnectionRelease_CCCH_r5,
      { "rrcConnectionRelease-CCCH-r5", "rrc.rrcConnectionRelease_CCCH_r5",
        FT_NONE, BASE_NONE, NULL, 0,
        "RRCConnectionRelease_CCCH_r5_IEs", HFILL }},
    { &hf_rrc_rrcConnectionRelease_CCCH_r5_add_ext,
      { "rrcConnectionRelease-CCCH-r5-add-ext", "rrc.rrcConnectionRelease_CCCH_r5_add_ext",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING", HFILL }},
    { &hf_rrc_v690NonCriticalExtensions_30,
      { "v690NonCriticalExtensions", "rrc.v690NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v690NonCriticalExtensions_30", HFILL }},
    { &hf_rrc_v860NonCriticalExtensions_15,
      { "v860NonCriticalExtensions", "rrc.v860NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v860NonCriticalExtensions_15", HFILL }},
    { &hf_rrc_va40NonCriticalExtensions_10,
      { "va40NonCriticalExtensions", "rrc.va40NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_va40NonCriticalExtensions_10", HFILL }},
    { &hf_rrc_nonCriticalExtensions_123,
      { "nonCriticalExtensions", "rrc.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_nonCriticalExtensions_123", HFILL }},
    { &hf_rrc_criticalExtensions_98,
      { "criticalExtensions", "rrc.criticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_criticalExtensions_97", HFILL }},
    { &hf_rrc_rrcConnectionRelease_02,
      { "rrcConnectionRelease", "rrc.rrcConnectionRelease",
        FT_NONE, BASE_NONE, NULL, 0,
        "RRCConnectionRelease_r3_IEs", HFILL }},
    { &hf_rrc_rrcConnectionRelease_03,
      { "rrcConnectionRelease", "rrc.rrcConnectionRelease",
        FT_NONE, BASE_NONE, NULL, 0,
        "RRCConnectionRelease_r4_IEs", HFILL }},
    { &hf_rrc_errorIndication,
      { "errorIndication", "rrc.errorIndication",
        FT_UINT32, BASE_DEC, VALS(rrc_FailureCauseWithProtErr_vals), 0,
        "FailureCauseWithProtErr", HFILL }},
    { &hf_rrc_laterNonCriticalExtensions_40,
      { "laterNonCriticalExtensions", "rrc.laterNonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_laterNonCriticalExtensions_40", HFILL }},
    { &hf_rrc_rrcConnectionReleaseComplete_r3_add_ext,
      { "rrcConnectionReleaseComplete-r3-add-ext", "rrc.rrcConnectionReleaseComplete_r3_add_ext",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING", HFILL }},
    { &hf_rrc_nonCriticalExtensions_124,
      { "nonCriticalExtensions", "rrc.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_nonCriticalExtensions_124", HFILL }},
    { &hf_rrc_protocolErrorIndicator_01,
      { "protocolErrorIndicator", "rrc.protocolErrorIndicator",
        FT_UINT32, BASE_DEC, VALS(rrc_ProtocolErrorIndicator_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_v3d0NonCriticalExtensions,
      { "v3d0NonCriticalExtensions", "rrc.v3d0NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_rRCConnectionRequest_v3d0ext,
      { "rRCConnectionRequest-v3d0ext", "rrc.rRCConnectionRequest_v3d0ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "RRCConnectionRequest_v3d0ext_IEs", HFILL }},
    { &hf_rrc_v4b0NonCriticalExtensions_10,
      { "v4b0NonCriticalExtensions", "rrc.v4b0NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v4b0NonCriticalExtensions_10", HFILL }},
    { &hf_rrc_rrcConnectionRequest_v4b0ext,
      { "rrcConnectionRequest-v4b0ext", "rrc.rrcConnectionRequest_v4b0ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "RRCConnectionRequest_v4b0ext_IEs", HFILL }},
    { &hf_rrc_v590NonCriticalExtensions_20,
      { "v590NonCriticalExtensions", "rrc.v590NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v590NonCriticalExtensions_20", HFILL }},
    { &hf_rrc_rrcConnectionRequest_v590ext,
      { "rrcConnectionRequest-v590ext", "rrc.rrcConnectionRequest_v590ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "RRCConnectionRequest_v590ext_IEs", HFILL }},
    { &hf_rrc_v690NonCriticalExtensions_31,
      { "v690NonCriticalExtensions", "rrc.v690NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v690NonCriticalExtensions_31", HFILL }},
    { &hf_rrc_rrcConnectionRequest_v690ext,
      { "rrcConnectionRequest-v690ext", "rrc.rrcConnectionRequest_v690ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "RRCConnectionRequest_v690ext_IEs", HFILL }},
    { &hf_rrc_v6b0NonCriticalExtensions_10,
      { "v6b0NonCriticalExtensions", "rrc.v6b0NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v6b0NonCriticalExtensions_09", HFILL }},
    { &hf_rrc_rrcConnectionRequest_v6b0ext,
      { "rrcConnectionRequest-v6b0ext", "rrc.rrcConnectionRequest_v6b0ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "RRCConnectionRequest_v6b0ext_IEs", HFILL }},
    { &hf_rrc_v6e0NonCriticalExtensions_01,
      { "v6e0NonCriticalExtensions", "rrc.v6e0NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v6e0NonCriticalExtensions_01", HFILL }},
    { &hf_rrc_rrcConnectionRequest_v6e0ext,
      { "rrcConnectionRequest-v6e0ext", "rrc.rrcConnectionRequest_v6e0ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "RRCConnectionRequest_v6e0ext_IEs", HFILL }},
    { &hf_rrc_v770NonCriticalExtensions_23,
      { "v770NonCriticalExtensions", "rrc.v770NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v770NonCriticalExtensions_23", HFILL }},
    { &hf_rrc_rrcConnectionRequest_v770ext,
      { "rrcConnectionRequest-v770ext", "rrc.rrcConnectionRequest_v770ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "RRCConnectionRequest_v770ext_IEs", HFILL }},
    { &hf_rrc_v7b0NonCriticalExtensions_01,
      { "v7b0NonCriticalExtensions", "rrc.v7b0NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v7b0NonCriticalExtensions_01", HFILL }},
    { &hf_rrc_rrcConnectionRequest_v7b0ext,
      { "rrcConnectionRequest-v7b0ext", "rrc.rrcConnectionRequest_v7b0ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "RRCConnectionRequest_v7b0ext_IEs", HFILL }},
    { &hf_rrc_v860NonCriticalExtensions_16,
      { "v860NonCriticalExtensions", "rrc.v860NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v860NonCriticalExtensions_16", HFILL }},
    { &hf_rrc_rrcConnectionRequest_v860ext,
      { "rrcConnectionRequest-v860ext", "rrc.rrcConnectionRequest_v860ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "RRCConnectionRequest_v860ext_IEs", HFILL }},
    { &hf_rrc_v7e0NonCriticalExtensions_01,
      { "v7e0NonCriticalExtensions", "rrc.v7e0NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v7e0NonCriticalExtensions_01", HFILL }},
    { &hf_rrc_rrcConnectionRequest_v7e0ext,
      { "rrcConnectionRequest-v7e0ext", "rrc.rrcConnectionRequest_v7e0ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "RRCConnectionRequest_v7e0ext_IEs", HFILL }},
    { &hf_rrc_v7g0NonCriticalExtensions_14,
      { "v7g0NonCriticalExtensions", "rrc.v7g0NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v7g0NonCriticalExtensions_14", HFILL }},
    { &hf_rrc_rrcConnectionRequest_v7g0ext,
      { "rrcConnectionRequest-v7g0ext", "rrc.rrcConnectionRequest_v7g0ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "RRCConnectionRequest_v7g0ext_IEs", HFILL }},
    { &hf_rrc_v920NonCriticalExtensions_02,
      { "v920NonCriticalExtensions", "rrc.v920NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v920NonCriticalExtensions_02", HFILL }},
    { &hf_rrc_rrcConnectionRequest_v920ext,
      { "rrcConnectionRequest-v920ext", "rrc.rrcConnectionRequest_v920ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "RRCConnectionRequest_v920ext_IEs", HFILL }},
    { &hf_rrc_v940NonCriticalExtensions,
      { "v940NonCriticalExtensions", "rrc.v940NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_rrcConnectionRequest_v940ext,
      { "rrcConnectionRequest-v940ext", "rrc.rrcConnectionRequest_v940ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "RRCConnectionRequest_v940ext_IEs", HFILL }},
    { &hf_rrc_va40NonCriticalExtensions_11,
      { "va40NonCriticalExtensions", "rrc.va40NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "RRCConnectionRequest_NonCriticalExts_va40_IEs", HFILL }},
    { &hf_rrc_uESpecificBehaviourInformation1idle,
      { "uESpecificBehaviourInformation1idle", "rrc.uESpecificBehaviourInformation1idle",
        FT_BYTES, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_predefinedConfigStatusInfo,
      { "predefinedConfigStatusInfo", "rrc.predefinedConfigStatusInfo",
        FT_BOOLEAN, BASE_NONE, NULL, 0,
        "BOOLEAN", HFILL }},
    { &hf_rrc_ueCapabilityIndication,
      { "ueCapabilityIndication", "rrc.ueCapabilityIndication",
        FT_UINT32, BASE_DEC, VALS(rrc_T_ueCapabilityIndication_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_domainIndicator,
      { "domainIndicator", "rrc.domainIndicator",
        FT_UINT32, BASE_DEC, VALS(rrc_T_domainIndicator_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_cs_domain,
      { "cs-domain", "rrc.cs_domain",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_csCallType_02,
      { "csCallType", "rrc.csCallType",
        FT_UINT32, BASE_DEC, VALS(rrc_T_csCallType_02_vals), 0,
        "T_csCallType_02", HFILL }},
    { &hf_rrc_ps_domain,
      { "ps-domain", "rrc.ps_domain",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_supportForFDPCH_01,
      { "supportForFDPCH", "rrc.supportForFDPCH",
        FT_UINT32, BASE_DEC, VALS(rrc_T_supportForFDPCH_01_vals), 0,
        "T_supportForFDPCH_01", HFILL }},
    { &hf_rrc_hspdschReception_CellFach_01,
      { "hspdschReception-CellFach", "rrc.hspdschReception_CellFach",
        FT_UINT32, BASE_DEC, VALS(rrc_T_hspdschReception_CellFach_01_vals), 0,
        "T_hspdschReception_CellFach_01", HFILL }},
    { &hf_rrc_mac_ehsSupport,
      { "mac-ehsSupport", "rrc.mac_ehsSupport",
        FT_UINT32, BASE_DEC, VALS(rrc_T_mac_ehsSupport_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_discontinuousDpcchTransmission,
      { "discontinuousDpcchTransmission", "rrc.discontinuousDpcchTransmission",
        FT_UINT32, BASE_DEC, VALS(rrc_T_discontinuousDpcchTransmission_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_supportForE_FDPCH,
      { "supportForE-FDPCH", "rrc.supportForE_FDPCH",
        FT_UINT32, BASE_DEC, VALS(rrc_T_supportForE_FDPCH_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_supportForCSVoiceoverHSPA,
      { "supportForCSVoiceoverHSPA", "rrc.supportForCSVoiceoverHSPA",
        FT_UINT32, BASE_DEC, VALS(rrc_T_supportForCSVoiceoverHSPA_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_supportOfCommonEDCH_01,
      { "supportOfCommonEDCH", "rrc.supportOfCommonEDCH",
        FT_UINT32, BASE_DEC, VALS(rrc_T_supportOfCommonEDCH_01_vals), 0,
        "T_supportOfCommonEDCH_01", HFILL }},
    { &hf_rrc_multiCellSupport,
      { "multiCellSupport", "rrc.multiCellSupport",
        FT_UINT32, BASE_DEC, VALS(rrc_T_multiCellSupport_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_pre_redirectionInfo,
      { "pre-redirectionInfo", "rrc.pre_redirectionInfo",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_supportOfMACiis_01,
      { "supportOfMACiis", "rrc.supportOfMACiis",
        FT_UINT32, BASE_DEC, VALS(rrc_T_supportOfMACiis_01_vals), 0,
        "T_supportOfMACiis_01", HFILL }},
    { &hf_rrc_supportOfSPSOperation_01,
      { "supportOfSPSOperation", "rrc.supportOfSPSOperation",
        FT_UINT32, BASE_DEC, VALS(rrc_T_supportOfSPSOperation_01_vals), 0,
        "T_supportOfSPSOperation_01", HFILL }},
    { &hf_rrc_supportOfDualCellMIMO,
      { "supportOfDualCellMIMO", "rrc.supportOfDualCellMIMO",
        FT_UINT32, BASE_DEC, VALS(rrc_T_supportOfDualCellMIMO_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_systemInformationContainerStoredIndicator,
      { "systemInformationContainerStoredIndicator", "rrc.systemInformationContainerStoredIndicator",
        FT_UINT32, BASE_DEC, VALS(rrc_T_systemInformationContainerStoredIndicator_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_cSFBIndication,
      { "cSFBIndication", "rrc.cSFBIndication",
        FT_UINT32, BASE_DEC, VALS(rrc_T_cSFBIndication_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_rrcConnectionRequest_va40ext,
      { "rrcConnectionRequest-va40ext", "rrc.rrcConnectionRequest_va40ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "RRCConnectionRequest_va40ext_IEs", HFILL }},
    { &hf_rrc_v9b0NonCriticalExtensions,
      { "v9b0NonCriticalExtensions", "rrc.v9b0NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_rrcConnectionRequest_v9b0ext,
      { "rrcConnectionRequest-v9b0ext", "rrc.rrcConnectionRequest_v9b0ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "RRCConnectionRequest_v9b0ext_IEs", HFILL }},
    { &hf_rrc_vbxyNonCriticalExtensions_12,
      { "vbxyNonCriticalExtensions", "rrc.vbxyNonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_vbxyNonCriticalExtensions_12", HFILL }},
    { &hf_rrc_rrcConnectionRequest_vbxyext,
      { "rrcConnectionRequest-vbxyext", "rrc.rrcConnectionRequest_vbxyext",
        FT_NONE, BASE_NONE, NULL, 0,
        "RRCConnectionRequest_vbxyext_IEs", HFILL }},
    { &hf_rrc_nonCriticalExtensions_125,
      { "nonCriticalExtensions", "rrc.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_nonCriticalExtensions_125", HFILL }},
    { &hf_rrc_supportOfMoreThanTwoCells,
      { "supportOfMoreThanTwoCells", "rrc.supportOfMoreThanTwoCells",
        FT_UINT32, BASE_DEC, VALS(rrc_T_supportOfMoreThanTwoCells_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_supportOf1stFrequencyBand,
      { "supportOf1stFrequencyBand", "rrc.supportOf1stFrequencyBand",
        FT_UINT32, BASE_DEC, VALS(rrc_T_supportOf1stFrequencyBand_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_supportOf2ndFrequencyBand,
      { "supportOf2ndFrequencyBand", "rrc.supportOf2ndFrequencyBand",
        FT_UINT32, BASE_DEC, VALS(rrc_T_supportOf2ndFrequencyBand_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_r3_20,
      { "r3", "rrc.r3",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_r3_20", HFILL }},
    { &hf_rrc_rrcConnectionSetup_r3,
      { "rrcConnectionSetup-r3", "rrc.rrcConnectionSetup_r3",
        FT_NONE, BASE_NONE, NULL, 0,
        "RRCConnectionSetup_r3_IEs", HFILL }},
    { &hf_rrc_laterNonCriticalExtensions_41,
      { "laterNonCriticalExtensions", "rrc.laterNonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_laterNonCriticalExtensions_41", HFILL }},
    { &hf_rrc_rrcConnectionSetup_r3_add_ext,
      { "rrcConnectionSetup-r3-add-ext", "rrc.rrcConnectionSetup_r3_add_ext",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING", HFILL }},
    { &hf_rrc_v4b0NonCriticalExtensions_11,
      { "v4b0NonCriticalExtensions", "rrc.v4b0NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v4b0NonCriticalExtensions_11", HFILL }},
    { &hf_rrc_rrcConnectionSetup_v4b0ext,
      { "rrcConnectionSetup-v4b0ext", "rrc.rrcConnectionSetup_v4b0ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "RRCConnectionSetup_v4b0ext_IEs", HFILL }},
    { &hf_rrc_v590NonCriticalExtensions_21,
      { "v590NonCriticalExtensions", "rrc.v590NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v590NonCriticalExtensions_21", HFILL }},
    { &hf_rrc_rrcConnectionSetup_v590ext,
      { "rrcConnectionSetup-v590ext", "rrc.rrcConnectionSetup_v590ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "RRCConnectionSetup_v590ext_IEs", HFILL }},
    { &hf_rrc_v690NonCriticalExtensions_32,
      { "v690NonCriticalExtensions", "rrc.v690NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v690NonCriticalExtensions_32", HFILL }},
    { &hf_rrc_rrcConnectionSetup_v690ext,
      { "rrcConnectionSetup-v690ext", "rrc.rrcConnectionSetup_v690ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "RRCConnectionSetup_v690ext_IEs", HFILL }},
    { &hf_rrc_nonCriticalExtensions_126,
      { "nonCriticalExtensions", "rrc.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_nonCriticalExtensions_126", HFILL }},
    { &hf_rrc_later_than_r3_18,
      { "later-than-r3", "rrc.later_than_r3",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_later_than_r3_18", HFILL }},
    { &hf_rrc_criticalExtensions_99,
      { "criticalExtensions", "rrc.criticalExtensions",
        FT_UINT32, BASE_DEC, VALS(rrc_T_criticalExtensions_98_vals), 0,
        "T_criticalExtensions_98", HFILL }},
    { &hf_rrc_r4_11,
      { "r4", "rrc.r4",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_r4_11", HFILL }},
    { &hf_rrc_rrcConnectionSetup_r4,
      { "rrcConnectionSetup-r4", "rrc.rrcConnectionSetup_r4",
        FT_NONE, BASE_NONE, NULL, 0,
        "RRCConnectionSetup_r4_IEs", HFILL }},
    { &hf_rrc_v4d0NonCriticalExtensions_11,
      { "v4d0NonCriticalExtensions", "rrc.v4d0NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v4d0NonCriticalExtensions_11", HFILL }},
    { &hf_rrc_rrcConnectionSetup_r4_add_ext,
      { "rrcConnectionSetup-r4-add-ext", "rrc.rrcConnectionSetup_r4_add_ext",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING", HFILL }},
    { &hf_rrc_v590NonCriticalExtensions_22,
      { "v590NonCriticalExtensions", "rrc.v590NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v590NonCriticalExtensions_22", HFILL }},
    { &hf_rrc_v690NonCriticalExtensions_33,
      { "v690NonCriticalExtensions", "rrc.v690NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v690NonCriticalExtensions_33", HFILL }},
    { &hf_rrc_nonCriticalExtensions_127,
      { "nonCriticalExtensions", "rrc.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_nonCriticalExtensions_127", HFILL }},
    { &hf_rrc_criticalExtensions_100,
      { "criticalExtensions", "rrc.criticalExtensions",
        FT_UINT32, BASE_DEC, VALS(rrc_T_criticalExtensions_99_vals), 0,
        "T_criticalExtensions_99", HFILL }},
    { &hf_rrc_r5_09,
      { "r5", "rrc.r5",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_r5_09", HFILL }},
    { &hf_rrc_rrcConnectionSetup_r5,
      { "rrcConnectionSetup-r5", "rrc.rrcConnectionSetup_r5",
        FT_NONE, BASE_NONE, NULL, 0,
        "RRCConnectionSetup_r5_IEs", HFILL }},
    { &hf_rrc_rrcConnectionSetup_r5_add_ext,
      { "rrcConnectionSetup-r5-add-ext", "rrc.rrcConnectionSetup_r5_add_ext",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING", HFILL }},
    { &hf_rrc_v690NonCriticalExtensions_34,
      { "v690NonCriticalExtensions", "rrc.v690NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v690NonCriticalExtensions_34", HFILL }},
    { &hf_rrc_nonCriticalExtensions_128,
      { "nonCriticalExtensions", "rrc.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_nonCriticalExtensions_128", HFILL }},
    { &hf_rrc_criticalExtensions_101,
      { "criticalExtensions", "rrc.criticalExtensions",
        FT_UINT32, BASE_DEC, VALS(rrc_T_criticalExtensions_100_vals), 0,
        "T_criticalExtensions_100", HFILL }},
    { &hf_rrc_r6_10,
      { "r6", "rrc.r6",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_r6_10", HFILL }},
    { &hf_rrc_rrcConnectionSetup_r6,
      { "rrcConnectionSetup-r6", "rrc.rrcConnectionSetup_r6",
        FT_NONE, BASE_NONE, NULL, 0,
        "RRCConnectionSetup_r6_IEs", HFILL }},
    { &hf_rrc_rrcConnectionSetup_r6_add_ext,
      { "rrcConnectionSetup-r6-add-ext", "rrc.rrcConnectionSetup_r6_add_ext",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING", HFILL }},
    { &hf_rrc_v6b0NonCriticalExtensions_11,
      { "v6b0NonCriticalExtensions", "rrc.v6b0NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v6b0NonCriticalExtensions_10", HFILL }},
    { &hf_rrc_rrcConnectionSetup_v6b0ext,
      { "rrcConnectionSetup-v6b0ext", "rrc.rrcConnectionSetup_v6b0ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "RRCConnectionSetup_v6b0ext_IEs", HFILL }},
    { &hf_rrc_nonCriticalExtensions_129,
      { "nonCriticalExtensions", "rrc.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_nonCriticalExtensions_129", HFILL }},
    { &hf_rrc_criticalExtensions_102,
      { "criticalExtensions", "rrc.criticalExtensions",
        FT_UINT32, BASE_DEC, VALS(rrc_T_criticalExtensions_101_vals), 0,
        "T_criticalExtensions_101", HFILL }},
    { &hf_rrc_r7_09,
      { "r7", "rrc.r7",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_r7_09", HFILL }},
    { &hf_rrc_rrcConnectionSetup_r7,
      { "rrcConnectionSetup-r7", "rrc.rrcConnectionSetup_r7",
        FT_NONE, BASE_NONE, NULL, 0,
        "RRCConnectionSetup_r7_IEs", HFILL }},
    { &hf_rrc_rrcConnectionSetup_r7_add_ext,
      { "rrcConnectionSetup-r7-add-ext", "rrc.rrcConnectionSetup_r7_add_ext",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING", HFILL }},
    { &hf_rrc_v780NonCriticalExtensions_08,
      { "v780NonCriticalExtensions", "rrc.v780NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v780NonCriticalExtensions_08", HFILL }},
    { &hf_rrc_rrcConnectionSetup_v780ext,
      { "rrcConnectionSetup-v780ext", "rrc.rrcConnectionSetup_v780ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "RRCConnectionSetup_v780ext_IEs", HFILL }},
    { &hf_rrc_v7d0NonCriticalExtensions_11,
      { "v7d0NonCriticalExtensions", "rrc.v7d0NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v7d0NonCriticalExtensions_11", HFILL }},
    { &hf_rrc_rrcConnectionSetup_v7d0ext,
      { "rrcConnectionSetup-v7d0ext", "rrc.rrcConnectionSetup_v7d0ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "RRCConnectionSetup_v7d0ext_IEs", HFILL }},
    { &hf_rrc_nonCriticalExtensions_130,
      { "nonCriticalExtensions", "rrc.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_nonCriticalExtensions_130", HFILL }},
    { &hf_rrc_criticalExtensions_103,
      { "criticalExtensions", "rrc.criticalExtensions",
        FT_UINT32, BASE_DEC, VALS(rrc_T_criticalExtensions_102_vals), 0,
        "T_criticalExtensions_102", HFILL }},
    { &hf_rrc_r8_10,
      { "r8", "rrc.r8",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_r8_10", HFILL }},
    { &hf_rrc_rrcConnectionSetup_r8,
      { "rrcConnectionSetup-r8", "rrc.rrcConnectionSetup_r8",
        FT_NONE, BASE_NONE, NULL, 0,
        "RRCConnectionSetup_r8_IEs", HFILL }},
    { &hf_rrc_rrcConnectionSetup_r8_add_ext,
      { "rrcConnectionSetup-r8-add-ext", "rrc.rrcConnectionSetup_r8_add_ext",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING", HFILL }},
    { &hf_rrc_v7d0NonCriticalExtensions_12,
      { "v7d0NonCriticalExtensions", "rrc.v7d0NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v7d0NonCriticalExtensions_12", HFILL }},
    { &hf_rrc_v890NonCriticalExtensions_08,
      { "v890NonCriticalExtensions", "rrc.v890NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v890NonCriticalExtensions_08", HFILL }},
    { &hf_rrc_rrcConnectionSetup_v890ext,
      { "rrcConnectionSetup-v890ext", "rrc.rrcConnectionSetup_v890ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "RRCConnectionSetup_v890ext_IEs", HFILL }},
    { &hf_rrc_v8a0NonCriticalExtensions_08,
      { "v8a0NonCriticalExtensions", "rrc.v8a0NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v8a0NonCriticalExtensions_08", HFILL }},
    { &hf_rrc_rrcConnectionSetup_v8a0ext,
      { "rrcConnectionSetup-v8a0ext", "rrc.rrcConnectionSetup_v8a0ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "RRCConnectionSetup_v8a0ext_IEs", HFILL }},
    { &hf_rrc_nonCriticalExtensions_131,
      { "nonCriticalExtensions", "rrc.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_nonCriticalExtensions_131", HFILL }},
    { &hf_rrc_criticalExtensions_104,
      { "criticalExtensions", "rrc.criticalExtensions",
        FT_UINT32, BASE_DEC, VALS(rrc_T_criticalExtensions_103_vals), 0,
        "T_criticalExtensions_103", HFILL }},
    { &hf_rrc_r9_09,
      { "r9", "rrc.r9",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_r9_09", HFILL }},
    { &hf_rrc_rrcConnectionSetup_r9,
      { "rrcConnectionSetup-r9", "rrc.rrcConnectionSetup_r9",
        FT_NONE, BASE_NONE, NULL, 0,
        "RRCConnectionSetup_r9_IEs", HFILL }},
    { &hf_rrc_rrcConnectionSetup_r9_add_ext,
      { "rrcConnectionSetup-r9-add-ext", "rrc.rrcConnectionSetup_r9_add_ext",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING", HFILL }},
    { &hf_rrc_v950NonCriticalExtensions_07,
      { "v950NonCriticalExtensions", "rrc.v950NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v950NonCriticalExtensions_07", HFILL }},
    { &hf_rrc_rrcConnectionSetup_v950ext,
      { "rrcConnectionSetup-v950ext", "rrc.rrcConnectionSetup_v950ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "RRCConnectionSetup_v950ext_IEs", HFILL }},
    { &hf_rrc_nonCriticalExtensions_132,
      { "nonCriticalExtensions", "rrc.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_nonCriticalExtensions_132", HFILL }},
    { &hf_rrc_criticalExtensions_105,
      { "criticalExtensions", "rrc.criticalExtensions",
        FT_UINT32, BASE_DEC, VALS(rrc_T_criticalExtensions_104_vals), 0,
        "T_criticalExtensions_104", HFILL }},
    { &hf_rrc_r10_09,
      { "r10", "rrc.r10",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_r10_09", HFILL }},
    { &hf_rrc_rrcConnectionSetup_r10,
      { "rrcConnectionSetup-r10", "rrc.rrcConnectionSetup_r10",
        FT_NONE, BASE_NONE, NULL, 0,
        "RRCConnectionSetup_r10_IEs", HFILL }},
    { &hf_rrc_rrcConnectionSetup_r10_add_ext,
      { "rrcConnectionSetup-r10-add-ext", "rrc.rrcConnectionSetup_r10_add_ext",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING", HFILL }},
    { &hf_rrc_nonCriticalExtensions_133,
      { "nonCriticalExtensions", "rrc.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_nonCriticalExtensions_133", HFILL }},
    { &hf_rrc_criticalExtensions_106,
      { "criticalExtensions", "rrc.criticalExtensions",
        FT_UINT32, BASE_DEC, VALS(rrc_T_criticalExtensions_105_vals), 0,
        "T_criticalExtensions_105", HFILL }},
    { &hf_rrc_r11_10,
      { "r11", "rrc.r11",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_r11_10", HFILL }},
    { &hf_rrc_rrcConnectionSetup_r11,
      { "rrcConnectionSetup-r11", "rrc.rrcConnectionSetup_r11",
        FT_NONE, BASE_NONE, NULL, 0,
        "RRCConnectionSetup_r11_IEs", HFILL }},
    { &hf_rrc_rrcConnectionSetup_r11_add_ext,
      { "rrcConnectionSetup-r11-add-ext", "rrc.rrcConnectionSetup_r11_add_ext",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING", HFILL }},
    { &hf_rrc_nonCriticalExtensions_134,
      { "nonCriticalExtensions", "rrc.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_nonCriticalExtensions_134", HFILL }},
    { &hf_rrc_criticalExtensions_107,
      { "criticalExtensions", "rrc.criticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_criticalExtensions_106", HFILL }},
    { &hf_rrc_new_c_RNTI,
      { "new-c-RNTI", "rrc.new_c_RNTI",
        FT_BYTES, BASE_NONE, NULL, 0,
        "C_RNTI", HFILL }},
    { &hf_rrc_capabilityUpdateRequirement,
      { "capabilityUpdateRequirement", "rrc.capabilityUpdateRequirement",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_srb_InformationSetupList_05,
      { "srb-InformationSetupList", "rrc.srb_InformationSetupList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "SRB_InformationSetupList2", HFILL }},
    { &hf_rrc_capabilityUpdateRequirement_r4_ext,
      { "capabilityUpdateRequirement-r4-ext", "rrc.capabilityUpdateRequirement_r4_ext",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_systemSpecificCapUpdateReq,
      { "systemSpecificCapUpdateReq", "rrc.systemSpecificCapUpdateReq",
        FT_UINT32, BASE_DEC, VALS(rrc_SystemSpecificCapUpdateReq_v590ext_vals), 0,
        "SystemSpecificCapUpdateReq_v590ext", HFILL }},
    { &hf_rrc_capabilityUpdateRequirement_01,
      { "capabilityUpdateRequirement", "rrc.capabilityUpdateRequirement",
        FT_NONE, BASE_NONE, NULL, 0,
        "CapabilityUpdateRequirement_r4", HFILL }},
    { &hf_rrc_capabilityUpdateRequirement_02,
      { "capabilityUpdateRequirement", "rrc.capabilityUpdateRequirement",
        FT_NONE, BASE_NONE, NULL, 0,
        "CapabilityUpdateRequirement_r5", HFILL }},
    { &hf_rrc_specificationMode_19,
      { "specificationMode", "rrc.specificationMode",
        FT_UINT32, BASE_DEC, VALS(rrc_T_specificationMode_19_vals), 0,
        "T_specificationMode_19", HFILL }},
    { &hf_rrc_complete_19,
      { "complete", "rrc.complete",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_complete_19", HFILL }},
    { &hf_rrc_preconfiguration_16,
      { "preconfiguration", "rrc.preconfiguration",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_preconfiguration_16", HFILL }},
    { &hf_rrc_preConfigMode_16,
      { "preConfigMode", "rrc.preConfigMode",
        FT_UINT32, BASE_DEC, VALS(rrc_T_preConfigMode_16_vals), 0,
        "T_preConfigMode_16", HFILL }},
    { &hf_rrc_defaultConfig_16,
      { "defaultConfig", "rrc.defaultConfig",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_defaultConfig_16", HFILL }},
    { &hf_rrc_dl_InformationPerRL_List_08,
      { "dl-InformationPerRL-List", "rrc.dl_InformationPerRL_List",
        FT_UINT32, BASE_DEC, NULL, 0,
        "DL_InformationPerRL_List_r5bis", HFILL }},
    { &hf_rrc_postVerificationPeriod_05,
      { "postVerificationPeriod", "rrc.postVerificationPeriod",
        FT_UINT32, BASE_DEC, VALS(rrc_T_postVerificationPeriod_05_vals), 0,
        "T_postVerificationPeriod_05", HFILL }},
    { &hf_rrc_specificationMode_20,
      { "specificationMode", "rrc.specificationMode",
        FT_UINT32, BASE_DEC, VALS(rrc_T_specificationMode_20_vals), 0,
        "T_specificationMode_20", HFILL }},
    { &hf_rrc_complete_20,
      { "complete", "rrc.complete",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_complete_20", HFILL }},
    { &hf_rrc_srb_InformationSetupList_06,
      { "srb-InformationSetupList", "rrc.srb_InformationSetupList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "SRB_InformationSetupList2_r6", HFILL }},
    { &hf_rrc_preconfiguration_17,
      { "preconfiguration", "rrc.preconfiguration",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_preconfiguration_17", HFILL }},
    { &hf_rrc_preConfigMode_17,
      { "preConfigMode", "rrc.preConfigMode",
        FT_UINT32, BASE_DEC, VALS(rrc_T_preConfigMode_17_vals), 0,
        "T_preConfigMode_17", HFILL }},
    { &hf_rrc_defaultConfig_17,
      { "defaultConfig", "rrc.defaultConfig",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_defaultConfig_17", HFILL }},
    { &hf_rrc_capabilityUpdateRequirement_03,
      { "capabilityUpdateRequirement", "rrc.capabilityUpdateRequirement",
        FT_NONE, BASE_NONE, NULL, 0,
        "CapabilityUpdateRequirement_r7", HFILL }},
    { &hf_rrc_specificationMode_21,
      { "specificationMode", "rrc.specificationMode",
        FT_UINT32, BASE_DEC, VALS(rrc_T_specificationMode_21_vals), 0,
        "T_specificationMode_21", HFILL }},
    { &hf_rrc_complete_21,
      { "complete", "rrc.complete",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_complete_21", HFILL }},
    { &hf_rrc_srb_InformationSetupList_07,
      { "srb-InformationSetupList", "rrc.srb_InformationSetupList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "SRB_InformationSetupList2_r7", HFILL }},
    { &hf_rrc_preconfiguration_18,
      { "preconfiguration", "rrc.preconfiguration",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_preconfiguration_18", HFILL }},
    { &hf_rrc_preConfigMode_18,
      { "preConfigMode", "rrc.preConfigMode",
        FT_UINT32, BASE_DEC, VALS(rrc_T_preConfigMode_18_vals), 0,
        "T_preConfigMode_18", HFILL }},
    { &hf_rrc_defaultConfig_18,
      { "defaultConfig", "rrc.defaultConfig",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_defaultConfig_18", HFILL }},
    { &hf_rrc_capabilityUpdateRequirement_04,
      { "capabilityUpdateRequirement", "rrc.capabilityUpdateRequirement",
        FT_NONE, BASE_NONE, NULL, 0,
        "CapabilityUpdateRequirement_r8", HFILL }},
    { &hf_rrc_specificationMode_22,
      { "specificationMode", "rrc.specificationMode",
        FT_UINT32, BASE_DEC, VALS(rrc_T_specificationMode_22_vals), 0,
        "T_specificationMode_22", HFILL }},
    { &hf_rrc_complete_22,
      { "complete", "rrc.complete",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_complete_22", HFILL }},
    { &hf_rrc_srb_InformationSetupList_08,
      { "srb-InformationSetupList", "rrc.srb_InformationSetupList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "SRB_InformationSetupList2_r8", HFILL }},
    { &hf_rrc_preconfiguration_19,
      { "preconfiguration", "rrc.preconfiguration",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_preconfiguration_19", HFILL }},
    { &hf_rrc_preConfigMode_19,
      { "preConfigMode", "rrc.preConfigMode",
        FT_UINT32, BASE_DEC, VALS(rrc_T_preConfigMode_19_vals), 0,
        "T_preConfigMode_19", HFILL }},
    { &hf_rrc_defaultConfig_19,
      { "defaultConfig", "rrc.defaultConfig",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_defaultConfig_19", HFILL }},
    { &hf_rrc_specificationMode_23,
      { "specificationMode", "rrc.specificationMode",
        FT_UINT32, BASE_DEC, VALS(rrc_T_specificationMode_23_vals), 0,
        "T_specificationMode_23", HFILL }},
    { &hf_rrc_complete_23,
      { "complete", "rrc.complete",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_complete_23", HFILL }},
    { &hf_rrc_preconfiguration_20,
      { "preconfiguration", "rrc.preconfiguration",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_preconfiguration_20", HFILL }},
    { &hf_rrc_preConfigMode_20,
      { "preConfigMode", "rrc.preConfigMode",
        FT_UINT32, BASE_DEC, VALS(rrc_T_preConfigMode_20_vals), 0,
        "T_preConfigMode_20", HFILL }},
    { &hf_rrc_defaultConfig_20,
      { "defaultConfig", "rrc.defaultConfig",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_defaultConfig_20", HFILL }},
    { &hf_rrc_specificationMode_24,
      { "specificationMode", "rrc.specificationMode",
        FT_UINT32, BASE_DEC, VALS(rrc_T_specificationMode_24_vals), 0,
        "T_specificationMode_24", HFILL }},
    { &hf_rrc_complete_24,
      { "complete", "rrc.complete",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_complete_24", HFILL }},
    { &hf_rrc_preconfiguration_21,
      { "preconfiguration", "rrc.preconfiguration",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_preconfiguration_21", HFILL }},
    { &hf_rrc_preConfigMode_21,
      { "preConfigMode", "rrc.preConfigMode",
        FT_UINT32, BASE_DEC, VALS(rrc_T_preConfigMode_21_vals), 0,
        "T_preConfigMode_21", HFILL }},
    { &hf_rrc_defaultConfig_21,
      { "defaultConfig", "rrc.defaultConfig",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_defaultConfig_21", HFILL }},
    { &hf_rrc_specificationMode_25,
      { "specificationMode", "rrc.specificationMode",
        FT_UINT32, BASE_DEC, VALS(rrc_T_specificationMode_25_vals), 0,
        "T_specificationMode_25", HFILL }},
    { &hf_rrc_complete_25,
      { "complete", "rrc.complete",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_complete_25", HFILL }},
    { &hf_rrc_preconfiguration_22,
      { "preconfiguration", "rrc.preconfiguration",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_preconfiguration_22", HFILL }},
    { &hf_rrc_preConfigMode_22,
      { "preConfigMode", "rrc.preConfigMode",
        FT_UINT32, BASE_DEC, VALS(rrc_T_preConfigMode_22_vals), 0,
        "T_preConfigMode_22", HFILL }},
    { &hf_rrc_defaultConfig_22,
      { "defaultConfig", "rrc.defaultConfig",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_defaultConfig_22", HFILL }},
    { &hf_rrc_ue_RadioAccessCapability_02,
      { "ue-RadioAccessCapability", "rrc.ue_RadioAccessCapability",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_ue_RATSpecificCapability,
      { "ue-RATSpecificCapability", "rrc.ue_RATSpecificCapability",
        FT_UINT32, BASE_DEC, NULL, 0,
        "InterRAT_UE_RadioAccessCapabilityList", HFILL }},
    { &hf_rrc_v370NonCriticalExtensions,
      { "v370NonCriticalExtensions", "rrc.v370NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_rrcConnectionSetupComplete_v370ext,
      { "rrcConnectionSetupComplete-v370ext", "rrc.rrcConnectionSetupComplete_v370ext",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_v380NonCriticalExtensions,
      { "v380NonCriticalExtensions", "rrc.v380NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_rrcConnectionSetupComplete_v380ext,
      { "rrcConnectionSetupComplete-v380ext", "rrc.rrcConnectionSetupComplete_v380ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "RRCConnectionSetupComplete_v380ext_IEs", HFILL }},
    { &hf_rrc_v3a0NonCriticalExtensions_08,
      { "v3a0NonCriticalExtensions", "rrc.v3a0NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v3a0NonCriticalExtensions_08", HFILL }},
    { &hf_rrc_rrcConnectionSetupComplete_v3a0ext,
      { "rrcConnectionSetupComplete-v3a0ext", "rrc.rrcConnectionSetupComplete_v3a0ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "RRCConnectionSetupComplete_v3a0ext_IEs", HFILL }},
    { &hf_rrc_laterNonCriticalExtensions_42,
      { "laterNonCriticalExtensions", "rrc.laterNonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_laterNonCriticalExtensions_42", HFILL }},
    { &hf_rrc_rrcConnectionSetupComplete_r3_add_ext,
      { "rrcConnectionSetupComplete-r3-add-ext", "rrc.rrcConnectionSetupComplete_r3_add_ext",
        FT_BYTES, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_v3g0NonCriticalExtensions_01,
      { "v3g0NonCriticalExtensions", "rrc.v3g0NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v3g0NonCriticalExtensions_01", HFILL }},
    { &hf_rrc_rrcConnectionSetupComplete_v3g0ext,
      { "rrcConnectionSetupComplete-v3g0ext", "rrc.rrcConnectionSetupComplete_v3g0ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "RRCConnectionSetupComplete_v3g0ext_IEs", HFILL }},
    { &hf_rrc_v4b0NonCriticalExtensions_12,
      { "v4b0NonCriticalExtensions", "rrc.v4b0NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v4b0NonCriticalExtensions_12", HFILL }},
    { &hf_rrc_rrcConnectionSetupComplete_v4b0ext,
      { "rrcConnectionSetupComplete-v4b0ext", "rrc.rrcConnectionSetupComplete_v4b0ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "RRCConnectionSetupComplete_v4b0ext_IEs", HFILL }},
    { &hf_rrc_v590NonCriticalExtensions_23,
      { "v590NonCriticalExtensions", "rrc.v590NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v590NonCriticalExtensions_23", HFILL }},
    { &hf_rrc_rrcConnectionSetupComplete_v590ext,
      { "rrcConnectionSetupComplete-v590ext", "rrc.rrcConnectionSetupComplete_v590ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "RRCConnectionSetupComplete_v590ext_IEs", HFILL }},
    { &hf_rrc_v5c0NonCriticalExtensions,
      { "v5c0NonCriticalExtensions", "rrc.v5c0NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_rrcConnectionSetupComplete_v5c0ext,
      { "rrcConnectionSetupComplete-v5c0ext", "rrc.rrcConnectionSetupComplete_v5c0ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "RRCConnectionSetupComplete_v5c0ext_IEs", HFILL }},
    { &hf_rrc_v690NonCriticalExtensions_35,
      { "v690NonCriticalExtensions", "rrc.v690NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v690NonCriticalExtensions_35", HFILL }},
    { &hf_rrc_rrcConnectionSetupComplete_v690ext,
      { "rrcConnectionSetupComplete-v690ext", "rrc.rrcConnectionSetupComplete_v690ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "RRCConnectionSetupComplete_v690ext_IEs", HFILL }},
    { &hf_rrc_v770NonCriticalExtensions_24,
      { "v770NonCriticalExtensions", "rrc.v770NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v770NonCriticalExtensions_24", HFILL }},
    { &hf_rrc_rrcConectionSetupComplete_v770ext,
      { "rrcConectionSetupComplete-v770ext", "rrc.rrcConectionSetupComplete_v770ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "RRCConnectionSetupComplete_v770ext_IEs", HFILL }},
    { &hf_rrc_va40NonCriticalExtensions_12,
      { "va40NonCriticalExtensions", "rrc.va40NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_va40NonCriticalExtensions_11", HFILL }},
    { &hf_rrc_rrcConectionSetupComplete_va40ext,
      { "rrcConectionSetupComplete-va40ext", "rrc.rrcConectionSetupComplete_va40ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "RRCConnectionSetupComplete_va40ext_IEs", HFILL }},
    { &hf_rrc_vbxyNonCriticalExtensions_13,
      { "vbxyNonCriticalExtensions", "rrc.vbxyNonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_vbxyNonCriticalExtensions_13", HFILL }},
    { &hf_rrc_rrcConnectionSetupComplete_vbxyext,
      { "rrcConnectionSetupComplete-vbxyext", "rrc.rrcConnectionSetupComplete_vbxyext",
        FT_NONE, BASE_NONE, NULL, 0,
        "RRCConnectionSetupComplete_vbxyext_IEs", HFILL }},
    { &hf_rrc_nonCriticalExtensions_135,
      { "nonCriticalExtensions", "rrc.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_nonCriticalExtensions_135", HFILL }},
    { &hf_rrc_ue_RadioAccessCapability_v370ext,
      { "ue-RadioAccessCapability-v370ext", "rrc.ue_RadioAccessCapability_v370ext",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_rrcConnectionSetupComplete_v650ext,
      { "rrcConnectionSetupComplete-v650ext", "rrc.rrcConnectionSetupComplete_v650ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "RRCConnectionSetupComplete_v650ext_IEs", HFILL }},
    { &hf_rrc_v680NonCriticalExtensions,
      { "v680NonCriticalExtensions", "rrc.v680NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_rrcConnectionSetupComplete_v680ext,
      { "rrcConnectionSetupComplete-v680ext", "rrc.rrcConnectionSetupComplete_v680ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "RRCConnectionSetupComplete_v680ext_IEs", HFILL }},
    { &hf_rrc_v7e0NonCriticalExtensions_02,
      { "v7e0NonCriticalExtensions", "rrc.v7e0NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v7e0NonCriticalExtensions_02", HFILL }},
    { &hf_rrc_rrcConnectionSetupComplete_v7e0ext,
      { "rrcConnectionSetupComplete-v7e0ext", "rrc.rrcConnectionSetupComplete_v7e0ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "RRCConnectionSetupComplete_v7e0ext_IEs", HFILL }},
    { &hf_rrc_v7f0NonCriticalExtensions_13,
      { "v7f0NonCriticalExtensions", "rrc.v7f0NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v7f0NonCriticalExtensions_13", HFILL }},
    { &hf_rrc_rrcConnectionSetupComplete_v7f0ext,
      { "rrcConnectionSetupComplete-v7f0ext", "rrc.rrcConnectionSetupComplete_v7f0ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "RRCConnectionSetupComplete_v7f0ext_IEs", HFILL }},
    { &hf_rrc_va40NonCriticalExtensions_13,
      { "va40NonCriticalExtensions", "rrc.va40NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_va40NonCriticalExtensions_12", HFILL }},
    { &hf_rrc_rrcConnectionSetupCompleteBand_va40ext,
      { "rrcConnectionSetupCompleteBand-va40ext", "rrc.rrcConnectionSetupCompleteBand_va40ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "RRCConnectionSetupCompleteBand_va40ext_IEs", HFILL }},
    { &hf_rrc_nonCriticalExtensions_136,
      { "nonCriticalExtensions", "rrc.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_nonCriticalExtensions_136", HFILL }},
    { &hf_rrc_ue_RadioAccessCapability_v4b0ext,
      { "ue-RadioAccessCapability-v4b0ext", "rrc.ue_RadioAccessCapability_v4b0ext",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_ue_RadioAccessCapability_v590ext,
      { "ue-RadioAccessCapability-v590ext", "rrc.ue_RadioAccessCapability_v590ext",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_ue_RATSpecificCapability_v590ext,
      { "ue-RATSpecificCapability-v590ext", "rrc.ue_RATSpecificCapability_v590ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "InterRAT_UE_RadioAccessCapability_v590ext", HFILL }},
    { &hf_rrc_ue_RadioAccessCapability_v5c0ext,
      { "ue-RadioAccessCapability-v5c0ext", "rrc.ue_RadioAccessCapability_v5c0ext",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_ue_RadioAccessCapability_v680ext,
      { "ue-RadioAccessCapability-v680ext", "rrc.ue_RadioAccessCapability_v680ext",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_ueCapabilityContainer,
      { "ueCapabilityContainer", "rrc.ueCapabilityContainer",
        FT_BYTES, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_deferredMeasurementControlReading_04,
      { "deferredMeasurementControlReading", "rrc.deferredMeasurementControlReading",
        FT_UINT32, BASE_DEC, VALS(rrc_T_deferredMeasurementControlReading_04_vals), 0,
        "T_deferredMeasurementControlReading_04", HFILL }},
    { &hf_rrc_loggedMeasAvailable_03,
      { "loggedMeasAvailable", "rrc.loggedMeasAvailable",
        FT_UINT32, BASE_DEC, VALS(rrc_T_loggedMeasAvailable_03_vals), 0,
        "T_loggedMeasAvailable_03", HFILL }},
    { &hf_rrc_loggedANRResultsAvailable_02,
      { "loggedANRResultsAvailable", "rrc.loggedANRResultsAvailable",
        FT_UINT32, BASE_DEC, VALS(rrc_T_loggedANRResultsAvailable_02_vals), 0,
        "T_loggedANRResultsAvailable_02", HFILL }},
    { &hf_rrc_loggedConnectionEstablishmentFailureInfoAvailable_01,
      { "loggedConnectionEstablishmentFailureInfoAvailable", "rrc.loggedConnectionEstablishmentFailureInfoAvailable",
        FT_UINT32, BASE_DEC, VALS(rrc_T_loggedConnectionEstablishmentFailureInfoAvailable_01_vals), 0,
        "T_loggedConnectionEstablishmentFailureInfoAvailable_01", HFILL }},
    { &hf_rrc_r3_21,
      { "r3", "rrc.r3",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_r3_21", HFILL }},
    { &hf_rrc_rRC_FailureInfo_r3,
      { "rRC-FailureInfo-r3", "rrc.rRC_FailureInfo_r3",
        FT_NONE, BASE_NONE, NULL, 0,
        "RRC_FailureInfo_r3_IEs", HFILL }},
    { &hf_rrc_laterNonCriticalExtensions_43,
      { "laterNonCriticalExtensions", "rrc.laterNonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_laterNonCriticalExtensions_43", HFILL }},
    { &hf_rrc_rrc_FailureInfo_r3_add_ext,
      { "rrc-FailureInfo-r3-add-ext", "rrc.rrc_FailureInfo_r3_add_ext",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING", HFILL }},
    { &hf_rrc_nonCriticalExtensions_137,
      { "nonCriticalExtensions", "rrc.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_nonCriticalExtensions_137", HFILL }},
    { &hf_rrc_criticalExtensions_108,
      { "criticalExtensions", "rrc.criticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_criticalExtensions_107", HFILL }},
    { &hf_rrc_failureCauseWithProtErr,
      { "failureCauseWithProtErr", "rrc.failureCauseWithProtErr",
        FT_UINT32, BASE_DEC, VALS(rrc_FailureCauseWithProtErr_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_protocolErrorInformation,
      { "protocolErrorInformation", "rrc.protocolErrorInformation",
        FT_NONE, BASE_NONE, NULL, 0,
        "ProtocolErrorMoreInformation", HFILL }},
    { &hf_rrc_laterNonCriticalExtensions_44,
      { "laterNonCriticalExtensions", "rrc.laterNonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_laterNonCriticalExtensions_44", HFILL }},
    { &hf_rrc_rrcStatus_r3_add_ext,
      { "rrcStatus-r3-add-ext", "rrc.rrcStatus_r3_add_ext",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING", HFILL }},
    { &hf_rrc_nonCriticalExtensions_138,
      { "nonCriticalExtensions", "rrc.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_nonCriticalExtensions_138", HFILL }},
    { &hf_rrc_r3_22,
      { "r3", "rrc.r3",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_r3_22", HFILL }},
    { &hf_rrc_securityModeCommand_r3,
      { "securityModeCommand-r3", "rrc.securityModeCommand_r3",
        FT_NONE, BASE_NONE, NULL, 0,
        "SecurityModeCommand_r3_IEs", HFILL }},
    { &hf_rrc_laterNonCriticalExtensions_45,
      { "laterNonCriticalExtensions", "rrc.laterNonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_laterNonCriticalExtensions_45", HFILL }},
    { &hf_rrc_securityModeCommand_r3_add_ext,
      { "securityModeCommand-r3-add-ext", "rrc.securityModeCommand_r3_add_ext",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING", HFILL }},
    { &hf_rrc_nonCriticalExtensions_139,
      { "nonCriticalExtensions", "rrc.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_nonCriticalExtensions_139", HFILL }},
    { &hf_rrc_later_than_r3_19,
      { "later-than-r3", "rrc.later_than_r3",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_later_than_r3_19", HFILL }},
    { &hf_rrc_criticalExtensions_109,
      { "criticalExtensions", "rrc.criticalExtensions",
        FT_UINT32, BASE_DEC, VALS(rrc_T_criticalExtensions_108_vals), 0,
        "T_criticalExtensions_108", HFILL }},
    { &hf_rrc_r7_10,
      { "r7", "rrc.r7",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_r7_10", HFILL }},
    { &hf_rrc_securityModeCommand_r7,
      { "securityModeCommand-r7", "rrc.securityModeCommand_r7",
        FT_NONE, BASE_NONE, NULL, 0,
        "SecurityModeCommand_r7_IEs", HFILL }},
    { &hf_rrc_securityModeCommand_r7_add_ext,
      { "securityModeCommand-r7-add-ext", "rrc.securityModeCommand_r7_add_ext",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING", HFILL }},
    { &hf_rrc_nonCriticalExtensions_140,
      { "nonCriticalExtensions", "rrc.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_nonCriticalExtensions_140", HFILL }},
    { &hf_rrc_criticalExtensions_110,
      { "criticalExtensions", "rrc.criticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_criticalExtensions_109", HFILL }},
    { &hf_rrc_securityCapability,
      { "securityCapability", "rrc.securityCapability",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_ue_SystemSpecificSecurityCap,
      { "ue-SystemSpecificSecurityCap", "rrc.ue_SystemSpecificSecurityCap",
        FT_UINT32, BASE_DEC, NULL, 0,
        "InterRAT_UE_SecurityCapList", HFILL }},
    { &hf_rrc_rb_UL_CiphActivationTimeInfo,
      { "rb-UL-CiphActivationTimeInfo", "rrc.rb_UL_CiphActivationTimeInfo",
        FT_UINT32, BASE_DEC, NULL, 0,
        "RB_ActivationTimeInfoList", HFILL }},
    { &hf_rrc_laterNonCriticalExtensions_46,
      { "laterNonCriticalExtensions", "rrc.laterNonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_laterNonCriticalExtensions_46", HFILL }},
    { &hf_rrc_securityModeComplete_r3_add_ext,
      { "securityModeComplete-r3-add-ext", "rrc.securityModeComplete_r3_add_ext",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING", HFILL }},
    { &hf_rrc_nonCriticalExtensions_141,
      { "nonCriticalExtensions", "rrc.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_nonCriticalExtensions_141", HFILL }},
    { &hf_rrc_laterNonCriticalExtensions_47,
      { "laterNonCriticalExtensions", "rrc.laterNonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_laterNonCriticalExtensions_47", HFILL }},
    { &hf_rrc_securityModeFailure_r3_add_ext,
      { "securityModeFailure-r3-add-ext", "rrc.securityModeFailure_r3_add_ext",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING", HFILL }},
    { &hf_rrc_nonCriticalExtensions_142,
      { "nonCriticalExtensions", "rrc.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_nonCriticalExtensions_142", HFILL }},
    { &hf_rrc_r3_23,
      { "r3", "rrc.r3",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_r3_23", HFILL }},
    { &hf_rrc_signallingConnectionRelease_r3,
      { "signallingConnectionRelease-r3", "rrc.signallingConnectionRelease_r3",
        FT_NONE, BASE_NONE, NULL, 0,
        "SignallingConnectionRelease_r3_IEs", HFILL }},
    { &hf_rrc_laterNonCriticalExtensions_48,
      { "laterNonCriticalExtensions", "rrc.laterNonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_laterNonCriticalExtensions_48", HFILL }},
    { &hf_rrc_signallingConnectionRelease_r3_add_ext,
      { "signallingConnectionRelease-r3-add-ext", "rrc.signallingConnectionRelease_r3_add_ext",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING", HFILL }},
    { &hf_rrc_va40NonCriticalExtensions_14,
      { "va40NonCriticalExtensions", "rrc.va40NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_va40NonCriticalExtensions_13", HFILL }},
    { &hf_rrc_signallingConnectionRelease_va40ext,
      { "signallingConnectionRelease-va40ext", "rrc.signallingConnectionRelease_va40ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "SignallingConnectionRelease_va40ext_IEs", HFILL }},
    { &hf_rrc_nonCriticalExtensions_143,
      { "nonCriticalExtensions", "rrc.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_nonCriticalExtensions_143", HFILL }},
    { &hf_rrc_later_than_r3_20,
      { "later-than-r3", "rrc.later_than_r3",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_later_than_r3_20", HFILL }},
    { &hf_rrc_criticalExtensions_111,
      { "criticalExtensions", "rrc.criticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_criticalExtensions_110", HFILL }},
    { &hf_rrc_laterNonCriticalExtensions_49,
      { "laterNonCriticalExtensions", "rrc.laterNonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_laterNonCriticalExtensions_49", HFILL }},
    { &hf_rrc_signallingConnectionReleaseIndication_r3_add_ext,
      { "signallingConnectionReleaseIndication-r3-add-ext", "rrc.signallingConnectionReleaseIndication_r3_add_ext",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING", HFILL }},
    { &hf_rrc_v860nonCriticalExtentions,
      { "v860nonCriticalExtentions", "rrc.v860nonCriticalExtentions",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_signallingConnectionReleaseIndication_v860ext,
      { "signallingConnectionReleaseIndication-v860ext", "rrc.signallingConnectionReleaseIndication_v860ext",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_nonCriticalExtensions_144,
      { "nonCriticalExtensions", "rrc.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_nonCriticalExtensions_144", HFILL }},
    { &hf_rrc_signallingConnectionReleaseIndicationCause,
      { "signallingConnectionReleaseIndicationCause", "rrc.signallingConnectionReleaseIndicationCause",
        FT_UINT32, BASE_DEC, VALS(rrc_T_signallingConnectionReleaseIndicationCause_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_sfn_Prime,
      { "sfn-Prime", "rrc.sfn_Prime",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_payload,
      { "payload", "rrc.payload",
        FT_UINT32, BASE_DEC, VALS(rrc_T_payload_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_noSegment,
      { "noSegment", "rrc.noSegment",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_firstSegment,
      { "firstSegment", "rrc.firstSegment",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_subsequentSegment,
      { "subsequentSegment", "rrc.subsequentSegment",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_lastSegmentShort,
      { "lastSegmentShort", "rrc.lastSegmentShort",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_lastAndFirst,
      { "lastAndFirst", "rrc.lastAndFirst",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_firstSegment_01,
      { "firstSegment", "rrc.firstSegment",
        FT_NONE, BASE_NONE, NULL, 0,
        "FirstSegmentShort", HFILL }},
    { &hf_rrc_lastAndComplete,
      { "lastAndComplete", "rrc.lastAndComplete",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_completeSIB_List,
      { "completeSIB-List", "rrc.completeSIB_List",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_lastAndCompleteAndFirst,
      { "lastAndCompleteAndFirst", "rrc.lastAndCompleteAndFirst",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_completeAndFirst,
      { "completeAndFirst", "rrc.completeAndFirst",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_completeSIB,
      { "completeSIB", "rrc.completeSIB",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_lastSegment,
      { "lastSegment", "rrc.lastSegment",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_payload_01,
      { "payload", "rrc.payload",
        FT_UINT32, BASE_DEC, VALS(rrc_T_payload_01_vals), 0,
        "T_payload_01", HFILL }},
    { &hf_rrc_lastAndFirst_01,
      { "lastAndFirst", "rrc.lastAndFirst",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_lastAndFirst_01", HFILL }},
    { &hf_rrc_lastAndComplete_01,
      { "lastAndComplete", "rrc.lastAndComplete",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_lastAndComplete_01", HFILL }},
    { &hf_rrc_lastAndCompleteAndFirst_01,
      { "lastAndCompleteAndFirst", "rrc.lastAndCompleteAndFirst",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_lastAndCompleteAndFirst_01", HFILL }},
    { &hf_rrc_completeAndFirst_01,
      { "completeAndFirst", "rrc.completeAndFirst",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_completeAndFirst_01", HFILL }},
    { &hf_rrc_sib_Type,
      { "sib-Type", "rrc.sib_Type",
        FT_UINT32, BASE_DEC|BASE_EXT_STRING, &rrc_SIB_Type_vals_ext, 0,
        NULL, HFILL }},
    { &hf_rrc_seg_Count,
      { "seg-Count", "rrc.seg_Count",
        FT_UINT32, BASE_DEC, NULL, 0,
        "SegCount", HFILL }},
    { &hf_rrc_sib_Data_fixed,
      { "sib-Data-fixed", "rrc.sib_Data_fixed",
        FT_BYTES, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_sib_Data_variable,
      { "sib-Data-variable", "rrc.sib_Data_variable",
        FT_BYTES, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_segmentIndex,
      { "segmentIndex", "rrc.segmentIndex",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_CompleteSIB_List_item,
      { "CompleteSIBshort", "rrc.CompleteSIBshort",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_sib_Data_fixed_01,
      { "sib-Data-fixed", "rrc.sib_Data_fixed",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_226", HFILL }},
    { &hf_rrc_laterNonCriticalExtensions_50,
      { "laterNonCriticalExtensions", "rrc.laterNonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_laterNonCriticalExtensions_50", HFILL }},
    { &hf_rrc_systemInformationChangeIndication_r3_add_ext,
      { "systemInformationChangeIndication-r3-add-ext", "rrc.systemInformationChangeIndication_r3_add_ext",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING", HFILL }},
    { &hf_rrc_v860NonCriticalExtensions_17,
      { "v860NonCriticalExtensions", "rrc.v860NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v860NonCriticalExtensions_17", HFILL }},
    { &hf_rrc_systemInformationChangeIndication_v860ext,
      { "systemInformationChangeIndication-v860ext", "rrc.systemInformationChangeIndication_v860ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "SystemInformationChangeIndication_v860ext_IEs", HFILL }},
    { &hf_rrc_nonCriticalExtensions_145,
      { "nonCriticalExtensions", "rrc.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_nonCriticalExtensions_145", HFILL }},
    { &hf_rrc_r3_24,
      { "r3", "rrc.r3",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_r3_24", HFILL }},
    { &hf_rrc_transportChannelReconfiguration_r3,
      { "transportChannelReconfiguration-r3", "rrc.transportChannelReconfiguration_r3",
        FT_NONE, BASE_NONE, NULL, 0,
        "TransportChannelReconfiguration_r3_IEs", HFILL }},
    { &hf_rrc_v3a0NonCriticalExtensions_09,
      { "v3a0NonCriticalExtensions", "rrc.v3a0NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v3a0NonCriticalExtensions_09", HFILL }},
    { &hf_rrc_transportChannelReconfiguration_v3a0ext,
      { "transportChannelReconfiguration-v3a0ext", "rrc.transportChannelReconfiguration_v3a0ext",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_laterNonCriticalExtensions_51,
      { "laterNonCriticalExtensions", "rrc.laterNonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_laterNonCriticalExtensions_51", HFILL }},
    { &hf_rrc_transportChannelReconfiguration_r3_add_ext,
      { "transportChannelReconfiguration-r3-add-ext", "rrc.transportChannelReconfiguration_r3_add_ext",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING", HFILL }},
    { &hf_rrc_v4b0NonCriticalExtensions_13,
      { "v4b0NonCriticalExtensions", "rrc.v4b0NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v4b0NonCriticalExtensions_13", HFILL }},
    { &hf_rrc_transportChannelReconfiguration_v4b0ext,
      { "transportChannelReconfiguration-v4b0ext", "rrc.transportChannelReconfiguration_v4b0ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "TransportChannelReconfiguration_v4b0ext_IEs", HFILL }},
    { &hf_rrc_v590NonCriticalExtensions_24,
      { "v590NonCriticalExtensions", "rrc.v590NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v590NonCriticalExtensions_24", HFILL }},
    { &hf_rrc_transportChannelReconfiguration_v590ext,
      { "transportChannelReconfiguration-v590ext", "rrc.transportChannelReconfiguration_v590ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "TransportChannelReconfiguration_v590ext_IEs", HFILL }},
    { &hf_rrc_v690NonCriticalExtensions_36,
      { "v690NonCriticalExtensions", "rrc.v690NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v690NonCriticalExtensions_36", HFILL }},
    { &hf_rrc_transportChannelReconfiguration_v690ext,
      { "transportChannelReconfiguration-v690ext", "rrc.transportChannelReconfiguration_v690ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "TransportChannelReconfiguration_v690ext_IEs", HFILL }},
    { &hf_rrc_v770NonCriticalExtensions_25,
      { "v770NonCriticalExtensions", "rrc.v770NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v770NonCriticalExtensions_25", HFILL }},
    { &hf_rrc_transportChannelReconfiguration_v770ext,
      { "transportChannelReconfiguration-v770ext", "rrc.transportChannelReconfiguration_v770ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "TransportChannelReconfiguration_v770ext_IEs", HFILL }},
    { &hf_rrc_nonCriticalExtensions_146,
      { "nonCriticalExtensions", "rrc.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_nonCriticalExtensions_146", HFILL }},
    { &hf_rrc_later_than_r3_21,
      { "later-than-r3", "rrc.later_than_r3",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_later_than_r3_21", HFILL }},
    { &hf_rrc_criticalExtensions_112,
      { "criticalExtensions", "rrc.criticalExtensions",
        FT_UINT32, BASE_DEC, VALS(rrc_T_criticalExtensions_111_vals), 0,
        "T_criticalExtensions_111", HFILL }},
    { &hf_rrc_r4_12,
      { "r4", "rrc.r4",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_r4_12", HFILL }},
    { &hf_rrc_transportChannelReconfiguration_r4,
      { "transportChannelReconfiguration-r4", "rrc.transportChannelReconfiguration_r4",
        FT_NONE, BASE_NONE, NULL, 0,
        "TransportChannelReconfiguration_r4_IEs", HFILL }},
    { &hf_rrc_v4d0NonCriticalExtensions_12,
      { "v4d0NonCriticalExtensions", "rrc.v4d0NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v4d0NonCriticalExtensions_12", HFILL }},
    { &hf_rrc_transportChannelReconfiguration_r4_add_ext,
      { "transportChannelReconfiguration-r4-add-ext", "rrc.transportChannelReconfiguration_r4_add_ext",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING", HFILL }},
    { &hf_rrc_v590NonCriticalExtensions_25,
      { "v590NonCriticalExtensions", "rrc.v590NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v590NonCriticalExtensions_25", HFILL }},
    { &hf_rrc_v690NonCriticalExtensions_37,
      { "v690NonCriticalExtensions", "rrc.v690NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v690NonCriticalExtensions_37", HFILL }},
    { &hf_rrc_v770NonCriticalExtensions_26,
      { "v770NonCriticalExtensions", "rrc.v770NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v770NonCriticalExtensions_26", HFILL }},
    { &hf_rrc_nonCriticalExtensions_147,
      { "nonCriticalExtensions", "rrc.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_nonCriticalExtensions_147", HFILL }},
    { &hf_rrc_criticalExtensions_113,
      { "criticalExtensions", "rrc.criticalExtensions",
        FT_UINT32, BASE_DEC, VALS(rrc_T_criticalExtensions_112_vals), 0,
        "T_criticalExtensions_112", HFILL }},
    { &hf_rrc_r5_10,
      { "r5", "rrc.r5",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_r5_10", HFILL }},
    { &hf_rrc_transportChannelReconfiguration_r5,
      { "transportChannelReconfiguration-r5", "rrc.transportChannelReconfiguration_r5",
        FT_NONE, BASE_NONE, NULL, 0,
        "TransportChannelReconfiguration_r5_IEs", HFILL }},
    { &hf_rrc_transportChannelReconfiguration_r5_add_ext,
      { "transportChannelReconfiguration-r5-add-ext", "rrc.transportChannelReconfiguration_r5_add_ext",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING", HFILL }},
    { &hf_rrc_v690NonCriticalExtensions_38,
      { "v690NonCriticalExtensions", "rrc.v690NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v690NonCriticalExtensions_38", HFILL }},
    { &hf_rrc_v770NonCriticalExtensions_27,
      { "v770NonCriticalExtensions", "rrc.v770NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v770NonCriticalExtensions_27", HFILL }},
    { &hf_rrc_nonCriticalExtensions_148,
      { "nonCriticalExtensions", "rrc.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_nonCriticalExtensions_148", HFILL }},
    { &hf_rrc_criticalExtensions_114,
      { "criticalExtensions", "rrc.criticalExtensions",
        FT_UINT32, BASE_DEC, VALS(rrc_T_criticalExtensions_113_vals), 0,
        "T_criticalExtensions_113", HFILL }},
    { &hf_rrc_r6_11,
      { "r6", "rrc.r6",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_r6_11", HFILL }},
    { &hf_rrc_transportChannelReconfiguration_r6,
      { "transportChannelReconfiguration-r6", "rrc.transportChannelReconfiguration_r6",
        FT_NONE, BASE_NONE, NULL, 0,
        "TransportChannelReconfiguration_r6_IEs", HFILL }},
    { &hf_rrc_transportChannelReconfiguration_r6_add_ext,
      { "transportChannelReconfiguration-r6-add-ext", "rrc.transportChannelReconfiguration_r6_add_ext",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING", HFILL }},
    { &hf_rrc_v6b0NonCriticalExtensions_12,
      { "v6b0NonCriticalExtensions", "rrc.v6b0NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v6b0NonCriticalExtensions_11", HFILL }},
    { &hf_rrc_transportChannelReconfiguration_v6b0ext,
      { "transportChannelReconfiguration-v6b0ext", "rrc.transportChannelReconfiguration_v6b0ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "TransportChannelReconfiguration_v6b0ext_IEs", HFILL }},
    { &hf_rrc_v770NonCriticalExtensions_28,
      { "v770NonCriticalExtensions", "rrc.v770NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v770NonCriticalExtensions_28", HFILL }},
    { &hf_rrc_nonCriticalExtensions_149,
      { "nonCriticalExtensions", "rrc.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_nonCriticalExtensions_149", HFILL }},
    { &hf_rrc_criticalExtensions_115,
      { "criticalExtensions", "rrc.criticalExtensions",
        FT_UINT32, BASE_DEC, VALS(rrc_T_criticalExtensions_114_vals), 0,
        "T_criticalExtensions_114", HFILL }},
    { &hf_rrc_r7_11,
      { "r7", "rrc.r7",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_r7_11", HFILL }},
    { &hf_rrc_transportChannelReconfiguration_r7,
      { "transportChannelReconfiguration-r7", "rrc.transportChannelReconfiguration_r7",
        FT_NONE, BASE_NONE, NULL, 0,
        "TransportChannelReconfiguration_r7_IEs", HFILL }},
    { &hf_rrc_transportChannelReconfiguration_r7_add_ext,
      { "transportChannelReconfiguration-r7-add-ext", "rrc.transportChannelReconfiguration_r7_add_ext",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING", HFILL }},
    { &hf_rrc_v780NonCriticalExtensions_09,
      { "v780NonCriticalExtensions", "rrc.v780NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v780NonCriticalExtensions_09", HFILL }},
    { &hf_rrc_transportChannelReconfiguration_v780ext,
      { "transportChannelReconfiguration-v780ext", "rrc.transportChannelReconfiguration_v780ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "TransportChannelReconfiguration_v780ext_IEs", HFILL }},
    { &hf_rrc_v7d0NonCriticalExtensions_13,
      { "v7d0NonCriticalExtensions", "rrc.v7d0NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v7d0NonCriticalExtensions_13", HFILL }},
    { &hf_rrc_transportChannelReconfiguration_v7d0ext,
      { "transportChannelReconfiguration-v7d0ext", "rrc.transportChannelReconfiguration_v7d0ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "TransportChannelReconfiguration_v7d0ext_IEs", HFILL }},
    { &hf_rrc_v7f0NonCriticalExtensions_14,
      { "v7f0NonCriticalExtensions", "rrc.v7f0NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v7f0NonCriticalExtensions_14", HFILL }},
    { &hf_rrc_transportChannelReconfiguration_v7f0ext,
      { "transportChannelReconfiguration-v7f0ext", "rrc.transportChannelReconfiguration_v7f0ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "TransportChannelReconfiguration_v7f0ext_IEs", HFILL }},
    { &hf_rrc_v7g0NonCriticalExtensions_15,
      { "v7g0NonCriticalExtensions", "rrc.v7g0NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v7g0NonCriticalExtensions_15", HFILL }},
    { &hf_rrc_transportChannelReconfiguration_v7g0ext,
      { "transportChannelReconfiguration-v7g0ext", "rrc.transportChannelReconfiguration_v7g0ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "TransportChannelReconfiguration_v7g0ext_IEs", HFILL }},
    { &hf_rrc_nonCriticalExtensions_150,
      { "nonCriticalExtensions", "rrc.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_nonCriticalExtensions_150", HFILL }},
    { &hf_rrc_criticalExtensions_116,
      { "criticalExtensions", "rrc.criticalExtensions",
        FT_UINT32, BASE_DEC, VALS(rrc_T_criticalExtensions_115_vals), 0,
        "T_criticalExtensions_115", HFILL }},
    { &hf_rrc_r8_11,
      { "r8", "rrc.r8",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_r8_11", HFILL }},
    { &hf_rrc_transportChannelReconfiguration_r8,
      { "transportChannelReconfiguration-r8", "rrc.transportChannelReconfiguration_r8",
        FT_NONE, BASE_NONE, NULL, 0,
        "TransportChannelReconfiguration_r8_IEs", HFILL }},
    { &hf_rrc_transportChannelReconfiguration_r8_add_ext,
      { "transportChannelReconfiguration-r8-add-ext", "rrc.transportChannelReconfiguration_r8_add_ext",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING", HFILL }},
    { &hf_rrc_v7d0NonCriticalExtensions_14,
      { "v7d0NonCriticalExtensions", "rrc.v7d0NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v7d0NonCriticalExtensions_14", HFILL }},
    { &hf_rrc_v7f0NonCriticalExtensions_15,
      { "v7f0NonCriticalExtensions", "rrc.v7f0NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v7f0NonCriticalExtensions_15", HFILL }},
    { &hf_rrc_v890NonCriticalExtensions_09,
      { "v890NonCriticalExtensions", "rrc.v890NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v890NonCriticalExtensions_09", HFILL }},
    { &hf_rrc_transportChannelReconfiguration_v890ext,
      { "transportChannelReconfiguration-v890ext", "rrc.transportChannelReconfiguration_v890ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "TransportChannelReconfiguration_v890ext_IEs", HFILL }},
    { &hf_rrc_v7g0NonCriticalExtensions_16,
      { "v7g0NonCriticalExtensions", "rrc.v7g0NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v7g0NonCriticalExtensions_16", HFILL }},
    { &hf_rrc_v8a0NonCriticalExtensions_09,
      { "v8a0NonCriticalExtensions", "rrc.v8a0NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v8a0NonCriticalExtensions_09", HFILL }},
    { &hf_rrc_transportChannelReconfiguration_v8a0ext,
      { "transportChannelReconfiguration-v8a0ext", "rrc.transportChannelReconfiguration_v8a0ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "TransportChannelReconfiguration_v8a0ext_IEs", HFILL }},
    { &hf_rrc_nonCriticalExtensions_151,
      { "nonCriticalExtensions", "rrc.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_nonCriticalExtensions_151", HFILL }},
    { &hf_rrc_criticalExtensions_117,
      { "criticalExtensions", "rrc.criticalExtensions",
        FT_UINT32, BASE_DEC, VALS(rrc_T_criticalExtensions_116_vals), 0,
        "T_criticalExtensions_116", HFILL }},
    { &hf_rrc_r9_10,
      { "r9", "rrc.r9",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_r9_10", HFILL }},
    { &hf_rrc_transportChannelReconfiguration_r9,
      { "transportChannelReconfiguration-r9", "rrc.transportChannelReconfiguration_r9",
        FT_NONE, BASE_NONE, NULL, 0,
        "TransportChannelReconfiguration_r9_IEs", HFILL }},
    { &hf_rrc_transportChannelReconfiguration_r9_add_ext,
      { "transportChannelReconfiguration-r9-add-ext", "rrc.transportChannelReconfiguration_r9_add_ext",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING", HFILL }},
    { &hf_rrc_v950NonCriticalExtensions_08,
      { "v950NonCriticalExtensions", "rrc.v950NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v950NonCriticalExtensions_08", HFILL }},
    { &hf_rrc_transportChannelReconfiguration_v950ext,
      { "transportChannelReconfiguration-v950ext", "rrc.transportChannelReconfiguration_v950ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "TransportChannelReconfiguration_v950ext_IEs", HFILL }},
    { &hf_rrc_v9c0NonCriticalExtensions_10,
      { "v9c0NonCriticalExtensions", "rrc.v9c0NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v9c0NonCriticalExtensions_10", HFILL }},
    { &hf_rrc_transportChannelReconfiguration_v9c0ext,
      { "transportChannelReconfiguration-v9c0ext", "rrc.transportChannelReconfiguration_v9c0ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "TransportChannelReconfiguration_v9c0ext_IEs", HFILL }},
    { &hf_rrc_nonCriticalExtensions_152,
      { "nonCriticalExtensions", "rrc.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_nonCriticalExtensions_152", HFILL }},
    { &hf_rrc_criticalExtensions_118,
      { "criticalExtensions", "rrc.criticalExtensions",
        FT_UINT32, BASE_DEC, VALS(rrc_T_criticalExtensions_117_vals), 0,
        "T_criticalExtensions_117", HFILL }},
    { &hf_rrc_r10_10,
      { "r10", "rrc.r10",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_r10_10", HFILL }},
    { &hf_rrc_transportChannelReconfiguration_r10,
      { "transportChannelReconfiguration-r10", "rrc.transportChannelReconfiguration_r10",
        FT_NONE, BASE_NONE, NULL, 0,
        "TransportChannelReconfiguration_r10_IEs", HFILL }},
    { &hf_rrc_transportChannelReconfiguration_r10_add_ext,
      { "transportChannelReconfiguration-r10-add-ext", "rrc.transportChannelReconfiguration_r10_add_ext",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING", HFILL }},
    { &hf_rrc_v9c0NonCriticalExtensions_11,
      { "v9c0NonCriticalExtensions", "rrc.v9c0NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v9c0NonCriticalExtensions_11", HFILL }},
    { &hf_rrc_vbxyNonCriticalExtensions_14,
      { "vbxyNonCriticalExtensions", "rrc.vbxyNonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_vbxyNonCriticalExtensions_14", HFILL }},
    { &hf_rrc_transportChannelReconfiguration_vbxyext,
      { "transportChannelReconfiguration-vbxyext", "rrc.transportChannelReconfiguration_vbxyext",
        FT_NONE, BASE_NONE, NULL, 0,
        "TransportChannelReconfiguration_vbxyext_IEs", HFILL }},
    { &hf_rrc_nonCriticalExtensions_153,
      { "nonCriticalExtensions", "rrc.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_nonCriticalExtensions_153", HFILL }},
    { &hf_rrc_criticalExtensions_119,
      { "criticalExtensions", "rrc.criticalExtensions",
        FT_UINT32, BASE_DEC, VALS(rrc_T_criticalExtensions_118_vals), 0,
        "T_criticalExtensions_118", HFILL }},
    { &hf_rrc_r11_11,
      { "r11", "rrc.r11",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_r11_11", HFILL }},
    { &hf_rrc_transportChannelReconfiguration_r11,
      { "transportChannelReconfiguration-r11", "rrc.transportChannelReconfiguration_r11",
        FT_NONE, BASE_NONE, NULL, 0,
        "TransportChannelReconfiguration_r11_IEs", HFILL }},
    { &hf_rrc_transportChannelReconfiguration_r11_add_ext,
      { "transportChannelReconfiguration-r11-add-ext", "rrc.transportChannelReconfiguration_r11_add_ext",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING", HFILL }},
    { &hf_rrc_nonCriticalExtensions_154,
      { "nonCriticalExtensions", "rrc.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_nonCriticalExtensions_154", HFILL }},
    { &hf_rrc_criticalExtensions_120,
      { "criticalExtensions", "rrc.criticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_criticalExtensions_119", HFILL }},
    { &hf_rrc_dummy_25,
      { "dummy", "rrc.dummy",
        FT_UINT32, BASE_DEC, VALS(rrc_T_dummy_11_vals), 0,
        "T_dummy_11", HFILL }},
    { &hf_rrc_fdd_39,
      { "fdd", "rrc.fdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_fdd_39", HFILL }},
    { &hf_rrc_modeSpecificPhysChInfo_12,
      { "modeSpecificPhysChInfo", "rrc.modeSpecificPhysChInfo",
        FT_UINT32, BASE_DEC, VALS(rrc_T_modeSpecificPhysChInfo_12_vals), 0,
        "T_modeSpecificPhysChInfo_12", HFILL }},
    { &hf_rrc_fdd_40,
      { "fdd", "rrc.fdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_fdd_40", HFILL }},
    { &hf_rrc_dummy_26,
      { "dummy", "rrc.dummy",
        FT_UINT32, BASE_DEC, VALS(rrc_T_dummy_12_vals), 0,
        "T_dummy_12", HFILL }},
    { &hf_rrc_fdd_41,
      { "fdd", "rrc.fdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_fdd_41", HFILL }},
    { &hf_rrc_modeSpecificPhysChInfo_13,
      { "modeSpecificPhysChInfo", "rrc.modeSpecificPhysChInfo",
        FT_UINT32, BASE_DEC, VALS(rrc_T_modeSpecificPhysChInfo_13_vals), 0,
        "T_modeSpecificPhysChInfo_13", HFILL }},
    { &hf_rrc_fdd_42,
      { "fdd", "rrc.fdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_fdd_42", HFILL }},
    { &hf_rrc_dummy_27,
      { "dummy", "rrc.dummy",
        FT_UINT32, BASE_DEC, VALS(rrc_T_dummy_13_vals), 0,
        "T_dummy_13", HFILL }},
    { &hf_rrc_fdd_43,
      { "fdd", "rrc.fdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_fdd_43", HFILL }},
    { &hf_rrc_modeSpecificPhysChInfo_14,
      { "modeSpecificPhysChInfo", "rrc.modeSpecificPhysChInfo",
        FT_UINT32, BASE_DEC, VALS(rrc_T_modeSpecificPhysChInfo_14_vals), 0,
        "T_modeSpecificPhysChInfo_14", HFILL }},
    { &hf_rrc_fdd_44,
      { "fdd", "rrc.fdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_fdd_44", HFILL }},
    { &hf_rrc_postVerificationPeriod_06,
      { "postVerificationPeriod", "rrc.postVerificationPeriod",
        FT_UINT32, BASE_DEC, VALS(rrc_T_postVerificationPeriod_06_vals), 0,
        "T_postVerificationPeriod_06", HFILL }},
    { &hf_rrc_responseToChangeOfUE_Capability_10,
      { "responseToChangeOfUE-Capability", "rrc.responseToChangeOfUE_Capability",
        FT_UINT32, BASE_DEC, VALS(rrc_T_responseToChangeOfUE_Capability_10_vals), 0,
        "T_responseToChangeOfUE_Capability_10", HFILL }},
    { &hf_rrc_responseToChangeOfUE_Capability_11,
      { "responseToChangeOfUE-Capability", "rrc.responseToChangeOfUE_Capability",
        FT_UINT32, BASE_DEC, VALS(rrc_T_responseToChangeOfUE_Capability_11_vals), 0,
        "T_responseToChangeOfUE_Capability_11", HFILL }},
    { &hf_rrc_responseToChangeOfUE_Capability_12,
      { "responseToChangeOfUE-Capability", "rrc.responseToChangeOfUE_Capability",
        FT_UINT32, BASE_DEC, VALS(rrc_T_responseToChangeOfUE_Capability_12_vals), 0,
        "T_responseToChangeOfUE_Capability_12", HFILL }},
    { &hf_rrc_responseToChangeOfUE_Capability_13,
      { "responseToChangeOfUE-Capability", "rrc.responseToChangeOfUE_Capability",
        FT_UINT32, BASE_DEC, VALS(rrc_T_responseToChangeOfUE_Capability_13_vals), 0,
        "T_responseToChangeOfUE_Capability_13", HFILL }},
    { &hf_rrc_responseToChangeOfUE_Capability_14,
      { "responseToChangeOfUE-Capability", "rrc.responseToChangeOfUE_Capability",
        FT_UINT32, BASE_DEC, VALS(rrc_T_responseToChangeOfUE_Capability_14_vals), 0,
        "T_responseToChangeOfUE_Capability_14", HFILL }},
    { &hf_rrc_laterNonCriticalExtensions_52,
      { "laterNonCriticalExtensions", "rrc.laterNonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_laterNonCriticalExtensions_52", HFILL }},
    { &hf_rrc_transportChannelReconfigurationComplete_r3_add_ext,
      { "transportChannelReconfigurationComplete-r3-add-ext", "rrc.transportChannelReconfigurationComplete_r3_add_ext",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING", HFILL }},
    { &hf_rrc_v770NonCriticalExtensions_29,
      { "v770NonCriticalExtensions", "rrc.v770NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v770NonCriticalExtensions_29", HFILL }},
    { &hf_rrc_transportChannelReconfigurationComplete_v770ext,
      { "transportChannelReconfigurationComplete-v770ext", "rrc.transportChannelReconfigurationComplete_v770ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "TransportChannelReconfigurationComplete_v770ext_IEs", HFILL }},
    { &hf_rrc_nonCriticalExtensions_155,
      { "nonCriticalExtensions", "rrc.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_nonCriticalExtensions_155", HFILL }},
    { &hf_rrc_deferredMeasurementControlReading_05,
      { "deferredMeasurementControlReading", "rrc.deferredMeasurementControlReading",
        FT_UINT32, BASE_DEC, VALS(rrc_T_deferredMeasurementControlReading_05_vals), 0,
        "T_deferredMeasurementControlReading_05", HFILL }},
    { &hf_rrc_laterNonCriticalExtensions_53,
      { "laterNonCriticalExtensions", "rrc.laterNonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_laterNonCriticalExtensions_53", HFILL }},
    { &hf_rrc_transportChannelReconfigurationFailure_r3_add_ext,
      { "transportChannelReconfigurationFailure-r3-add-ext", "rrc.transportChannelReconfigurationFailure_r3_add_ext",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING", HFILL }},
    { &hf_rrc_nonCriticalExtensions_156,
      { "nonCriticalExtensions", "rrc.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_nonCriticalExtensions_156", HFILL }},
    { &hf_rrc_modeSpecificInfo_15,
      { "modeSpecificInfo", "rrc.modeSpecificInfo",
        FT_UINT32, BASE_DEC, VALS(rrc_T_modeSpecificInfo_15_vals), 0,
        "T_modeSpecificInfo_15", HFILL }},
    { &hf_rrc_fdd_45,
      { "fdd", "rrc.fdd",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_tdd_10,
      { "tdd", "rrc.tdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_tdd_09", HFILL }},
    { &hf_rrc_tfcs_ID,
      { "tfcs-ID", "rrc.tfcs_ID",
        FT_NONE, BASE_NONE, NULL, 0,
        "TFCS_Identity", HFILL }},
    { &hf_rrc_dpch_TFCS_InUplink,
      { "dpch-TFCS-InUplink", "rrc.dpch_TFCS_InUplink",
        FT_UINT32, BASE_DEC, VALS(rrc_TFC_Subset_vals), 0,
        "TFC_Subset", HFILL }},
    { &hf_rrc_activationTimeForTFCSubset,
      { "activationTimeForTFCSubset", "rrc.activationTimeForTFCSubset",
        FT_UINT32, BASE_DEC, NULL, 0,
        "ActivationTime", HFILL }},
    { &hf_rrc_tfc_ControlDuration,
      { "tfc-ControlDuration", "rrc.tfc_ControlDuration",
        FT_UINT32, BASE_DEC, VALS(rrc_TFC_ControlDuration_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_laterNonCriticalExtensions_54,
      { "laterNonCriticalExtensions", "rrc.laterNonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_laterNonCriticalExtensions_54", HFILL }},
    { &hf_rrc_transportFormatCombinationControl_r3_add_ext,
      { "transportFormatCombinationControl-r3-add-ext", "rrc.transportFormatCombinationControl_r3_add_ext",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING", HFILL }},
    { &hf_rrc_v820NonCriticalExtensions_02,
      { "v820NonCriticalExtensions", "rrc.v820NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v820NonCriticalExtensions_02", HFILL }},
    { &hf_rrc_transportformatcombinationcontrol_v820ext,
      { "transportformatcombinationcontrol-v820ext", "rrc.transportformatcombinationcontrol_v820ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "TransportFormatCombinationControl_v820ext_IEs", HFILL }},
    { &hf_rrc_nonCriticalExtensions_157,
      { "nonCriticalExtensions", "rrc.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_nonCriticalExtensions_157", HFILL }},
    { &hf_rrc_ul_AMR_Rate,
      { "ul-AMR-Rate", "rrc.ul_AMR_Rate",
        FT_UINT32, BASE_DEC, VALS(rrc_UL_AMR_Rate_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_laterNonCriticalExtensions_55,
      { "laterNonCriticalExtensions", "rrc.laterNonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_laterNonCriticalExtensions_55", HFILL }},
    { &hf_rrc_transportFormatCombinationControlFailure_r3_add_ext,
      { "transportFormatCombinationControlFailure-r3-add-ext", "rrc.transportFormatCombinationControlFailure_r3_add_ext",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING", HFILL }},
    { &hf_rrc_nonCriticalExtensions_158,
      { "nonCriticalExtensions", "rrc.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_nonCriticalExtensions_158", HFILL }},
    { &hf_rrc_r3_25,
      { "r3", "rrc.r3",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_r3_25", HFILL }},
    { &hf_rrc_ueCapabilityEnquiry_r3,
      { "ueCapabilityEnquiry-r3", "rrc.ueCapabilityEnquiry_r3",
        FT_NONE, BASE_NONE, NULL, 0,
        "UECapabilityEnquiry_r3_IEs", HFILL }},
    { &hf_rrc_laterNonCriticalExtensions_56,
      { "laterNonCriticalExtensions", "rrc.laterNonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_laterNonCriticalExtensions_56", HFILL }},
    { &hf_rrc_ueCapabilityEnquiry_r3_add_ext,
      { "ueCapabilityEnquiry-r3-add-ext", "rrc.ueCapabilityEnquiry_r3_add_ext",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING", HFILL }},
    { &hf_rrc_v4b0NonCriticalExtensions_14,
      { "v4b0NonCriticalExtensions", "rrc.v4b0NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v4b0NonCriticalExtensions_14", HFILL }},
    { &hf_rrc_ueCapabilityEnquiry_v4b0ext,
      { "ueCapabilityEnquiry-v4b0ext", "rrc.ueCapabilityEnquiry_v4b0ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "UECapabilityEnquiry_v4b0ext_IEs", HFILL }},
    { &hf_rrc_v590NonCriticalExtensions_26,
      { "v590NonCriticalExtensions", "rrc.v590NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v590NonCriticalExtensions_26", HFILL }},
    { &hf_rrc_ueCapabilityEnquiry_v590ext,
      { "ueCapabilityEnquiry-v590ext", "rrc.ueCapabilityEnquiry_v590ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "UECapabilityEnquiry_v590ext_IEs", HFILL }},
    { &hf_rrc_v770NonCriticalExtensions_30,
      { "v770NonCriticalExtensions", "rrc.v770NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v770NonCriticalExtensions_30", HFILL }},
    { &hf_rrc_ueCapabilityEnquiry_v770ext,
      { "ueCapabilityEnquiry-v770ext", "rrc.ueCapabilityEnquiry_v770ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "UECapabilityEnquiry_v770ext_IEs", HFILL }},
    { &hf_rrc_v860NonCriticalExtensions_18,
      { "v860NonCriticalExtensions", "rrc.v860NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v860NonCriticalExtensions_18", HFILL }},
    { &hf_rrc_ueCapabilityEnquiry_v860ext,
      { "ueCapabilityEnquiry-v860ext", "rrc.ueCapabilityEnquiry_v860ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "UECapabilityEnquiry_v860ext_IEs", HFILL }},
    { &hf_rrc_nonCriticalExtensions_159,
      { "nonCriticalExtensions", "rrc.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_nonCriticalExtensions_159", HFILL }},
    { &hf_rrc_later_than_r3_22,
      { "later-than-r3", "rrc.later_than_r3",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_later_than_r3_22", HFILL }},
    { &hf_rrc_criticalExtensions_121,
      { "criticalExtensions", "rrc.criticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_criticalExtensions_120", HFILL }},
    { &hf_rrc_capabilityUpdateRequirement_05,
      { "capabilityUpdateRequirement", "rrc.capabilityUpdateRequirement",
        FT_NONE, BASE_NONE, NULL, 0,
        "CapabilityUpdateRequirement_v770ext", HFILL }},
    { &hf_rrc_capabilityUpdateRequirement_06,
      { "capabilityUpdateRequirement", "rrc.capabilityUpdateRequirement",
        FT_NONE, BASE_NONE, NULL, 0,
        "CapabilityUpdateRequirement_v860ext", HFILL }},
    { &hf_rrc_v370NonCriticalExtensions_01,
      { "v370NonCriticalExtensions", "rrc.v370NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v370NonCriticalExtensions_01", HFILL }},
    { &hf_rrc_ueCapabilityInformation_v370ext,
      { "ueCapabilityInformation-v370ext", "rrc.ueCapabilityInformation_v370ext",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_v380NonCriticalExtensions_01,
      { "v380NonCriticalExtensions", "rrc.v380NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v380NonCriticalExtensions_01", HFILL }},
    { &hf_rrc_ueCapabilityInformation_v380ext,
      { "ueCapabilityInformation-v380ext", "rrc.ueCapabilityInformation_v380ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "UECapabilityInformation_v380ext_IEs", HFILL }},
    { &hf_rrc_v3a0NonCriticalExtensions_10,
      { "v3a0NonCriticalExtensions", "rrc.v3a0NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v3a0NonCriticalExtensions_10", HFILL }},
    { &hf_rrc_ueCapabilityInformation_v3a0ext,
      { "ueCapabilityInformation-v3a0ext", "rrc.ueCapabilityInformation_v3a0ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "UECapabilityInformation_v3a0ext_IEs", HFILL }},
    { &hf_rrc_laterNonCriticalExtensions_57,
      { "laterNonCriticalExtensions", "rrc.laterNonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_laterNonCriticalExtensions_57", HFILL }},
    { &hf_rrc_ueCapabilityInformation_r3_add_ext,
      { "ueCapabilityInformation-r3-add-ext", "rrc.ueCapabilityInformation_r3_add_ext",
        FT_BYTES, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_v4b0NonCriticalExtensions_15,
      { "v4b0NonCriticalExtensions", "rrc.v4b0NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v4b0NonCriticalExtensions_15", HFILL }},
    { &hf_rrc_ueCapabilityInformation_v4b0ext,
      { "ueCapabilityInformation-v4b0ext", "rrc.ueCapabilityInformation_v4b0ext",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_v590NonCriticalExtensions_27,
      { "v590NonCriticalExtensions", "rrc.v590NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v590NonCriticalExtensions_27", HFILL }},
    { &hf_rrc_ueCapabilityInformation_v590ext,
      { "ueCapabilityInformation-v590ext", "rrc.ueCapabilityInformation_v590ext",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_v5c0NonCriticalExtensions_01,
      { "v5c0NonCriticalExtensions", "rrc.v5c0NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v5c0NonCriticalExtensions_01", HFILL }},
    { &hf_rrc_ueCapabilityInformation_v5c0ext,
      { "ueCapabilityInformation-v5c0ext", "rrc.ueCapabilityInformation_v5c0ext",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_v690NonCriticalExtensions_39,
      { "v690NonCriticalExtensions", "rrc.v690NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v690NonCriticalExtensions_39", HFILL }},
    { &hf_rrc_ueCapabilityInformation_v690ext,
      { "ueCapabilityInformation-v690ext", "rrc.ueCapabilityInformation_v690ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "UECapabilityInformation_v690ext_IEs", HFILL }},
    { &hf_rrc_nonCriticalExtensions_160,
      { "nonCriticalExtensions", "rrc.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_nonCriticalExtensions_160", HFILL }},
    { &hf_rrc_ueCapabilityInformation_v650ext,
      { "ueCapabilityInformation-v650ext", "rrc.ueCapabilityInformation_v650ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "UECapabilityInformation_v650ext_IEs", HFILL }},
    { &hf_rrc_v680NonCriticalExtensions_01,
      { "v680NonCriticalExtensions", "rrc.v680NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v680NonCriticalExtensions_01", HFILL }},
    { &hf_rrc_ueCapabilityInformation_v680ext,
      { "ueCapabilityInformation-v680ext", "rrc.ueCapabilityInformation_v680ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "UECapabilityInformation_v680ext_IEs", HFILL }},
    { &hf_rrc_v7e0NonCriticalExtensions_03,
      { "v7e0NonCriticalExtensions", "rrc.v7e0NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v7e0NonCriticalExtensions_03", HFILL }},
    { &hf_rrc_ueCapabilityInformation_v7e0ext,
      { "ueCapabilityInformation-v7e0ext", "rrc.ueCapabilityInformation_v7e0ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "UECapabilityInformation_v7e0ext_IEs", HFILL }},
    { &hf_rrc_v7f0NonCriticalExtensions_16,
      { "v7f0NonCriticalExtensions", "rrc.v7f0NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v7f0NonCriticalExtensions_16", HFILL }},
    { &hf_rrc_ueCapabilityInformation_v7f0ext,
      { "ueCapabilityInformation-v7f0ext", "rrc.ueCapabilityInformation_v7f0ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "UECapabilityInformation_v7f0ext_IEs", HFILL }},
    { &hf_rrc_va40NonCriticalExtensions_15,
      { "va40NonCriticalExtensions", "rrc.va40NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_va40NonCriticalExtensions_14", HFILL }},
    { &hf_rrc_ueCapabilityInformation_va40ext,
      { "ueCapabilityInformation-va40ext", "rrc.ueCapabilityInformation_va40ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "UECapabilityInformation_va40ext_IEs", HFILL }},
    { &hf_rrc_nonCriticalExtensions_161,
      { "nonCriticalExtensions", "rrc.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_nonCriticalExtensions_161", HFILL }},
    { &hf_rrc_ueCapabilityContainer_01,
      { "ueCapabilityContainer", "rrc.ueCapabilityContainer",
        FT_BYTES, BASE_NONE, NULL, 0,
        "T_ueCapabilityContainer_01", HFILL }},
    { &hf_rrc_r3_26,
      { "r3", "rrc.r3",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_r3_26", HFILL }},
    { &hf_rrc_ueCapabilityInformationConfirm_r3,
      { "ueCapabilityInformationConfirm-r3", "rrc.ueCapabilityInformationConfirm_r3",
        FT_NONE, BASE_NONE, NULL, 0,
        "UECapabilityInformationConfirm_r3_IEs", HFILL }},
    { &hf_rrc_laterNonCriticalExtensions_58,
      { "laterNonCriticalExtensions", "rrc.laterNonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_laterNonCriticalExtensions_58", HFILL }},
    { &hf_rrc_ueCapabilityInformationConfirm_r3_add_ext,
      { "ueCapabilityInformationConfirm-r3-add-ext", "rrc.ueCapabilityInformationConfirm_r3_add_ext",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING", HFILL }},
    { &hf_rrc_v770NonCriticalExtensions_31,
      { "v770NonCriticalExtensions", "rrc.v770NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v770NonCriticalExtensions_31", HFILL }},
    { &hf_rrc_ueCapabilityInformationConfirm_v770ext,
      { "ueCapabilityInformationConfirm-v770ext", "rrc.ueCapabilityInformationConfirm_v770ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "UECapabilityInformationConfirm_v770ext_IEs", HFILL }},
    { &hf_rrc_nonCriticalExtensions_162,
      { "nonCriticalExtensions", "rrc.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_nonCriticalExtensions_162", HFILL }},
    { &hf_rrc_later_than_r3_23,
      { "later-than-r3", "rrc.later_than_r3",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_later_than_r3_23", HFILL }},
    { &hf_rrc_criticalExtensions_122,
      { "criticalExtensions", "rrc.criticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_criticalExtensions_121", HFILL }},
    { &hf_rrc_acceptanceOfChangeOfCapability,
      { "acceptanceOfChangeOfCapability", "rrc.acceptanceOfChangeOfCapability",
        FT_UINT32, BASE_DEC, VALS(rrc_T_acceptanceOfChangeOfCapability_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_laterNonCriticalExtensions_59,
      { "laterNonCriticalExtensions", "rrc.laterNonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_laterNonCriticalExtensions_59", HFILL }},
    { &hf_rrc_uplinkDirectTransfer_r3_add_ext,
      { "uplinkDirectTransfer-r3-add-ext", "rrc.uplinkDirectTransfer_r3_add_ext",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING", HFILL }},
    { &hf_rrc_v690NonCriticalExtensions_40,
      { "v690NonCriticalExtensions", "rrc.v690NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v690NonCriticalExtensions_40", HFILL }},
    { &hf_rrc_uplinkDirectTransfer_v690ext,
      { "uplinkDirectTransfer-v690ext", "rrc.uplinkDirectTransfer_v690ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "UplinkDirectTransfer_v690ext_IEs", HFILL }},
    { &hf_rrc_v7g0NonCriticalExtensions_17,
      { "v7g0NonCriticalExtensions", "rrc.v7g0NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v7g0NonCriticalExtensions_17", HFILL }},
    { &hf_rrc_uplinkDirectTransfer_v7g0ext,
      { "uplinkDirectTransfer-v7g0ext", "rrc.uplinkDirectTransfer_v7g0ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "UplinkDirectTransfer_v7g0ext_IEs", HFILL }},
    { &hf_rrc_vbxyNonCriticalExtensions_15,
      { "vbxyNonCriticalExtensions", "rrc.vbxyNonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_vbxyNonCriticalExtensions_15", HFILL }},
    { &hf_rrc_uplinkDirectTransfer_vbxyext,
      { "uplinkDirectTransfer-vbxyext", "rrc.uplinkDirectTransfer_vbxyext",
        FT_NONE, BASE_NONE, NULL, 0,
        "UplinkDirectTransfer_vbxyext_IEs", HFILL }},
    { &hf_rrc_nonCriticalExtensions_163,
      { "nonCriticalExtensions", "rrc.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_nonCriticalExtensions_163", HFILL }},
    { &hf_rrc_r3_27,
      { "r3", "rrc.r3",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_r3_27", HFILL }},
    { &hf_rrc_uplinkPhysicalChannelControl_r3,
      { "uplinkPhysicalChannelControl-r3", "rrc.uplinkPhysicalChannelControl_r3",
        FT_NONE, BASE_NONE, NULL, 0,
        "UplinkPhysicalChannelControl_r3_IEs", HFILL }},
    { &hf_rrc_laterNonCriticalExtensions_60,
      { "laterNonCriticalExtensions", "rrc.laterNonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_laterNonCriticalExtensions_60", HFILL }},
    { &hf_rrc_uplinkPhysicalChannelControl_r3_add_ext,
      { "uplinkPhysicalChannelControl-r3-add-ext", "rrc.uplinkPhysicalChannelControl_r3_add_ext",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING", HFILL }},
    { &hf_rrc_v4b0NonCriticalExtensions_16,
      { "v4b0NonCriticalExtensions", "rrc.v4b0NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v4b0NonCriticalExtensions_16", HFILL }},
    { &hf_rrc_uplinkPysicalChannelControl_v4b0ext,
      { "uplinkPysicalChannelControl-v4b0ext", "rrc.uplinkPysicalChannelControl_v4b0ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "UplinkPhysicalChannelControl_v4b0ext_IEs", HFILL }},
    { &hf_rrc_noncriticalExtensions,
      { "noncriticalExtensions", "rrc.noncriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_later_than_r3_24,
      { "later-than-r3", "rrc.later_than_r3",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_later_than_r3_24", HFILL }},
    { &hf_rrc_criticalExtensions_123,
      { "criticalExtensions", "rrc.criticalExtensions",
        FT_UINT32, BASE_DEC, VALS(rrc_T_criticalExtensions_122_vals), 0,
        "T_criticalExtensions_122", HFILL }},
    { &hf_rrc_r4_13,
      { "r4", "rrc.r4",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_r4_13", HFILL }},
    { &hf_rrc_uplinkPhysicalChannelControl_r4,
      { "uplinkPhysicalChannelControl-r4", "rrc.uplinkPhysicalChannelControl_r4",
        FT_NONE, BASE_NONE, NULL, 0,
        "UplinkPhysicalChannelControl_r4_IEs", HFILL }},
    { &hf_rrc_v4d0NonCriticalExtensions_13,
      { "v4d0NonCriticalExtensions", "rrc.v4d0NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v4d0NonCriticalExtensions_13", HFILL }},
    { &hf_rrc_uplinkPhysicalChannelControl_r4_add_ext,
      { "uplinkPhysicalChannelControl-r4-add-ext", "rrc.uplinkPhysicalChannelControl_r4_add_ext",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING", HFILL }},
    { &hf_rrc_v690NonCriticalExtensions_41,
      { "v690NonCriticalExtensions", "rrc.v690NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v690NonCriticalExtensions_41", HFILL }},
    { &hf_rrc_uplinkPhysicalChannelControl_v690ext,
      { "uplinkPhysicalChannelControl-v690ext", "rrc.uplinkPhysicalChannelControl_v690ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "UplinkPhysicalChannelControl_v690ext_IEs", HFILL }},
    { &hf_rrc_nonCriticalExtensions_164,
      { "nonCriticalExtensions", "rrc.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_nonCriticalExtensions_164", HFILL }},
    { &hf_rrc_criticalExtensions_124,
      { "criticalExtensions", "rrc.criticalExtensions",
        FT_UINT32, BASE_DEC, VALS(rrc_T_criticalExtensions_123_vals), 0,
        "T_criticalExtensions_123", HFILL }},
    { &hf_rrc_r5_11,
      { "r5", "rrc.r5",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_r5_11", HFILL }},
    { &hf_rrc_uplinkPhysicalChannelControl_r5,
      { "uplinkPhysicalChannelControl-r5", "rrc.uplinkPhysicalChannelControl_r5",
        FT_NONE, BASE_NONE, NULL, 0,
        "UplinkPhysicalChannelControl_r5_IEs", HFILL }},
    { &hf_rrc_uplinkPhysicalChannelControl_r5_add_ext,
      { "uplinkPhysicalChannelControl-r5-add-ext", "rrc.uplinkPhysicalChannelControl_r5_add_ext",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING", HFILL }},
    { &hf_rrc_v690NonCriticalExtensions_42,
      { "v690NonCriticalExtensions", "rrc.v690NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v690NonCriticalExtensions_42", HFILL }},
    { &hf_rrc_v6a0NonCriticalExtensions_01,
      { "v6a0NonCriticalExtensions", "rrc.v6a0NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v6a0NonCriticalExtensions_01", HFILL }},
    { &hf_rrc_uplinkPhysicalChannelControl_v6a0ext,
      { "uplinkPhysicalChannelControl-v6a0ext", "rrc.uplinkPhysicalChannelControl_v6a0ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "UplinkPhysicalChannelControl_v6a0ext_IEs", HFILL }},
    { &hf_rrc_nonCriticalExtensions_165,
      { "nonCriticalExtensions", "rrc.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_nonCriticalExtensions_165", HFILL }},
    { &hf_rrc_criticalExtensions_125,
      { "criticalExtensions", "rrc.criticalExtensions",
        FT_UINT32, BASE_DEC, VALS(rrc_T_criticalExtensions_124_vals), 0,
        "T_criticalExtensions_124", HFILL }},
    { &hf_rrc_r7_12,
      { "r7", "rrc.r7",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_r7_12", HFILL }},
    { &hf_rrc_uplinkPhysicalChannelControl_r7,
      { "uplinkPhysicalChannelControl-r7", "rrc.uplinkPhysicalChannelControl_r7",
        FT_NONE, BASE_NONE, NULL, 0,
        "UplinkPhysicalChannelControl_r7_IEs", HFILL }},
    { &hf_rrc_uplinkPhysicalChannelControl_r7_add_ext,
      { "uplinkPhysicalChannelControl-r7-add-ext", "rrc.uplinkPhysicalChannelControl_r7_add_ext",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING", HFILL }},
    { &hf_rrc_nonCriticalExtensions_166,
      { "nonCriticalExtensions", "rrc.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_nonCriticalExtensions_166", HFILL }},
    { &hf_rrc_criticalExtensions_126,
      { "criticalExtensions", "rrc.criticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_criticalExtensions_125", HFILL }},
    { &hf_rrc_ccTrCH_PowerControlInfo,
      { "ccTrCH-PowerControlInfo", "rrc.ccTrCH_PowerControlInfo",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_timingAdvance,
      { "timingAdvance", "rrc.timingAdvance",
        FT_UINT32, BASE_DEC, VALS(rrc_UL_TimingAdvanceControl_vals), 0,
        "UL_TimingAdvanceControl", HFILL }},
    { &hf_rrc_alpha,
      { "alpha", "rrc.alpha",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_specialBurstScheduling,
      { "specialBurstScheduling", "rrc.specialBurstScheduling",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_prach_ConstantValue,
      { "prach-ConstantValue", "rrc.prach_ConstantValue",
        FT_INT32, BASE_DEC, NULL, 0,
        "ConstantValueTdd", HFILL }},
    { &hf_rrc_pusch_ConstantValue,
      { "pusch-ConstantValue", "rrc.pusch_ConstantValue",
        FT_INT32, BASE_DEC, NULL, 0,
        "ConstantValueTdd", HFILL }},
    { &hf_rrc_openLoopPowerControl_IPDL_TDD,
      { "openLoopPowerControl-IPDL-TDD", "rrc.openLoopPowerControl_IPDL_TDD",
        FT_NONE, BASE_NONE, NULL, 0,
        "OpenLoopPowerControl_IPDL_TDD_r4", HFILL }},
    { &hf_rrc_ccTrCH_PowerControlInfo_01,
      { "ccTrCH-PowerControlInfo", "rrc.ccTrCH_PowerControlInfo",
        FT_NONE, BASE_NONE, NULL, 0,
        "CCTrCH_PowerControlInfo_r4", HFILL }},
    { &hf_rrc_tddOption,
      { "tddOption", "rrc.tddOption",
        FT_UINT32, BASE_DEC, VALS(rrc_T_tddOption_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_tdd384_08,
      { "tdd384", "rrc.tdd384",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_tdd384_08", HFILL }},
    { &hf_rrc_timingAdvance_01,
      { "timingAdvance", "rrc.timingAdvance",
        FT_UINT32, BASE_DEC, VALS(rrc_UL_TimingAdvanceControl_r4_vals), 0,
        "UL_TimingAdvanceControl_r4", HFILL }},
    { &hf_rrc_tdd128_08,
      { "tdd128", "rrc.tdd128",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_tdd128_08", HFILL }},
    { &hf_rrc_ul_SynchronisationParameters,
      { "ul-SynchronisationParameters", "rrc.ul_SynchronisationParameters",
        FT_NONE, BASE_NONE, NULL, 0,
        "UL_SynchronisationParameters_r4", HFILL }},
    { &hf_rrc_ccTrCH_PowerControlInfo_02,
      { "ccTrCH-PowerControlInfo", "rrc.ccTrCH_PowerControlInfo",
        FT_NONE, BASE_NONE, NULL, 0,
        "CCTrCH_PowerControlInfo_r5", HFILL }},
    { &hf_rrc_tddOption_01,
      { "tddOption", "rrc.tddOption",
        FT_UINT32, BASE_DEC, VALS(rrc_T_tddOption_01_vals), 0,
        "T_tddOption_01", HFILL }},
    { &hf_rrc_tdd384_09,
      { "tdd384", "rrc.tdd384",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_tdd384_09", HFILL }},
    { &hf_rrc_hs_SICH_PowerControl,
      { "hs-SICH-PowerControl", "rrc.hs_SICH_PowerControl",
        FT_NONE, BASE_NONE, NULL, 0,
        "HS_SICH_Power_Control_Info_TDD384", HFILL }},
    { &hf_rrc_tdd128_09,
      { "tdd128", "rrc.tdd128",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_tdd128_09", HFILL }},
    { &hf_rrc_desired_HS_SICH_PowerLevel,
      { "desired-HS-SICH-PowerLevel", "rrc.desired_HS_SICH_PowerLevel",
        FT_INT32, BASE_DEC, NULL, 0,
        "INTEGER_M120_M58", HFILL }},
    { &hf_rrc_tpc_Step_Size,
      { "tpc-Step-Size", "rrc.tpc_Step_Size",
        FT_UINT32, BASE_DEC, VALS(rrc_T_tpc_Step_Size_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_ccTrCH_PowerControlInfo_03,
      { "ccTrCH-PowerControlInfo", "rrc.ccTrCH_PowerControlInfo",
        FT_NONE, BASE_NONE, NULL, 0,
        "CCTrCH_PowerControlInfo_r7", HFILL }},
    { &hf_rrc_tddOption_02,
      { "tddOption", "rrc.tddOption",
        FT_UINT32, BASE_DEC, VALS(rrc_T_tddOption_02_vals), 0,
        "T_tddOption_02", HFILL }},
    { &hf_rrc_tdd384_10,
      { "tdd384", "rrc.tdd384",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_tdd384_10", HFILL }},
    { &hf_rrc_tdd768_05,
      { "tdd768", "rrc.tdd768",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_tdd768_05", HFILL }},
    { &hf_rrc_timingAdvance_02,
      { "timingAdvance", "rrc.timingAdvance",
        FT_UINT32, BASE_DEC, VALS(rrc_UL_TimingAdvanceControl_r7_vals), 0,
        "UL_TimingAdvanceControl_r7", HFILL }},
    { &hf_rrc_hs_SICH_PowerControl_01,
      { "hs-SICH-PowerControl", "rrc.hs_SICH_PowerControl",
        FT_NONE, BASE_NONE, NULL, 0,
        "HS_SICH_Power_Control_Info_TDD768", HFILL }},
    { &hf_rrc_tdd128_10,
      { "tdd128", "rrc.tdd128",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_tdd128_10", HFILL }},
    { &hf_rrc_tpc_Step_Size_01,
      { "tpc-Step-Size", "rrc.tpc_Step_Size",
        FT_UINT32, BASE_DEC, VALS(rrc_T_tpc_Step_Size_01_vals), 0,
        "T_tpc_Step_Size_01", HFILL }},
    { &hf_rrc_ura_UpdateCause,
      { "ura-UpdateCause", "rrc.ura_UpdateCause",
        FT_UINT32, BASE_DEC, VALS(rrc_URA_UpdateCause_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_laterNonCriticalExtensions_61,
      { "laterNonCriticalExtensions", "rrc.laterNonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_laterNonCriticalExtensions_61", HFILL }},
    { &hf_rrc_uraUpdate_r3_add_ext,
      { "uraUpdate-r3-add-ext", "rrc.uraUpdate_r3_add_ext",
        FT_BYTES, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_v770NonCriticalExtensions_32,
      { "v770NonCriticalExtensions", "rrc.v770NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v770NonCriticalExtensions_32", HFILL }},
    { &hf_rrc_uraUpdate_v770ext,
      { "uraUpdate-v770ext", "rrc.uraUpdate_v770ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "UraUpdate_v770ext_IEs", HFILL }},
    { &hf_rrc_v860NonCriticalExtensions_19,
      { "v860NonCriticalExtensions", "rrc.v860NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v860NonCriticalExtensions_19", HFILL }},
    { &hf_rrc_uraUpdate_v860ext,
      { "uraUpdate-v860ext", "rrc.uraUpdate_v860ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "URAUpdate_v860ext_IEs", HFILL }},
    { &hf_rrc_va40NonCriticalExtensions_16,
      { "va40NonCriticalExtensions", "rrc.va40NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_va40NonCriticalExtensions_15", HFILL }},
    { &hf_rrc_uraUpdate_va40ext,
      { "uraUpdate-va40ext", "rrc.uraUpdate_va40ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "URAUpdate_va40ext_IEs", HFILL }},
    { &hf_rrc_nonCriticalExtensions_167,
      { "nonCriticalExtensions", "rrc.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_nonCriticalExtensions_167", HFILL }},
    { &hf_rrc_uraUpdate_v7e0ext,
      { "uraUpdate-v7e0ext", "rrc.uraUpdate_v7e0ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "URAUpdate_v7e0ext_IEs", HFILL }},
    { &hf_rrc_nonCriticalExtensions_168,
      { "nonCriticalExtensions", "rrc.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_nonCriticalExtensions_168", HFILL }},
    { &hf_rrc_support_hsdschReception_CellUraPch,
      { "support-hsdschReception-CellUraPch", "rrc.support_hsdschReception_CellUraPch",
        FT_UINT32, BASE_DEC, VALS(rrc_T_support_hsdschReception_CellUraPch_vals), 0,
        "T_support_hsdschReception_CellUraPch", HFILL }},
    { &hf_rrc_support_hsdschReception_CellFach,
      { "support-hsdschReception-CellFach", "rrc.support_hsdschReception_CellFach",
        FT_UINT32, BASE_DEC, VALS(rrc_T_support_hsdschReception_CellFach_vals), 0,
        "T_support_hsdschReception_CellFach", HFILL }},
    { &hf_rrc_supportForTwoDRXSchemesInPCH_01,
      { "supportForTwoDRXSchemesInPCH", "rrc.supportForTwoDRXSchemesInPCH",
        FT_UINT32, BASE_DEC, VALS(rrc_T_supportForTwoDRXSchemesInPCH_01_vals), 0,
        "T_supportForTwoDRXSchemesInPCH_01", HFILL }},
    { &hf_rrc_supportOfHS_DSCHDRXOperation_01,
      { "supportOfHS-DSCHDRXOperation", "rrc.supportOfHS_DSCHDRXOperation",
        FT_UINT32, BASE_DEC, VALS(rrc_T_supportOfHS_DSCHDRXOperation_01_vals), 0,
        "T_supportOfHS_DSCHDRXOperation_01", HFILL }},
    { &hf_rrc_supportOfCommonEDCH_02,
      { "supportOfCommonEDCH", "rrc.supportOfCommonEDCH",
        FT_UINT32, BASE_DEC, VALS(rrc_T_supportOfCommonEDCH_02_vals), 0,
        "T_supportOfCommonEDCH_02", HFILL }},
    { &hf_rrc_supportOfMACiis_02,
      { "supportOfMACiis", "rrc.supportOfMACiis",
        FT_UINT32, BASE_DEC, VALS(rrc_T_supportOfMACiis_02_vals), 0,
        "T_supportOfMACiis_02", HFILL }},
    { &hf_rrc_loggedMeasAvailable_04,
      { "loggedMeasAvailable", "rrc.loggedMeasAvailable",
        FT_UINT32, BASE_DEC, VALS(rrc_T_loggedMeasAvailable_04_vals), 0,
        "T_loggedMeasAvailable_04", HFILL }},
    { &hf_rrc_loggedANRResultsAvailable_03,
      { "loggedANRResultsAvailable", "rrc.loggedANRResultsAvailable",
        FT_UINT32, BASE_DEC, VALS(rrc_T_loggedANRResultsAvailable_03_vals), 0,
        "T_loggedANRResultsAvailable_03", HFILL }},
    { &hf_rrc_r3_28,
      { "r3", "rrc.r3",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_r3_28", HFILL }},
    { &hf_rrc_uraUpdateConfirm_r3,
      { "uraUpdateConfirm-r3", "rrc.uraUpdateConfirm_r3",
        FT_NONE, BASE_NONE, NULL, 0,
        "URAUpdateConfirm_r3_IEs", HFILL }},
    { &hf_rrc_laterNonCriticalExtensions_62,
      { "laterNonCriticalExtensions", "rrc.laterNonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_laterNonCriticalExtensions_62", HFILL }},
    { &hf_rrc_uraUpdateConfirm_r3_add_ext,
      { "uraUpdateConfirm-r3-add-ext", "rrc.uraUpdateConfirm_r3_add_ext",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING", HFILL }},
    { &hf_rrc_v690NonCriticalExtensions_43,
      { "v690NonCriticalExtensions", "rrc.v690NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v690NonCriticalExtensions_43", HFILL }},
    { &hf_rrc_uraUpdateConfirm_v690ext,
      { "uraUpdateConfirm-v690ext", "rrc.uraUpdateConfirm_v690ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "URAUpdateConfirm_v690ext_IEs", HFILL }},
    { &hf_rrc_nonCriticalExtensions_169,
      { "nonCriticalExtensions", "rrc.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_nonCriticalExtensions_169", HFILL }},
    { &hf_rrc_later_than_r3_25,
      { "later-than-r3", "rrc.later_than_r3",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_later_than_r3_25", HFILL }},
    { &hf_rrc_criticalExtensions_127,
      { "criticalExtensions", "rrc.criticalExtensions",
        FT_UINT32, BASE_DEC, VALS(rrc_T_criticalExtensions_126_vals), 0,
        "T_criticalExtensions_126", HFILL }},
    { &hf_rrc_r5_12,
      { "r5", "rrc.r5",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_r5_12", HFILL }},
    { &hf_rrc_uraUpdateConfirm_r5,
      { "uraUpdateConfirm-r5", "rrc.uraUpdateConfirm_r5",
        FT_NONE, BASE_NONE, NULL, 0,
        "URAUpdateConfirm_r5_IEs", HFILL }},
    { &hf_rrc_v690NonCriticalExtensions_44,
      { "v690NonCriticalExtensions", "rrc.v690NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v690NonCriticalExtensions_44", HFILL }},
    { &hf_rrc_nonCriticalExtensions_170,
      { "nonCriticalExtensions", "rrc.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_nonCriticalExtensions_170", HFILL }},
    { &hf_rrc_criticalExtensions_128,
      { "criticalExtensions", "rrc.criticalExtensions",
        FT_UINT32, BASE_DEC, VALS(rrc_T_criticalExtensions_127_vals), 0,
        "T_criticalExtensions_127", HFILL }},
    { &hf_rrc_r7_13,
      { "r7", "rrc.r7",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_r7_13", HFILL }},
    { &hf_rrc_uraUpdateConfirm_r7,
      { "uraUpdateConfirm-r7", "rrc.uraUpdateConfirm_r7",
        FT_NONE, BASE_NONE, NULL, 0,
        "URAUpdateConfirm_r7_IEs", HFILL }},
    { &hf_rrc_uraUpdateConfirm_r7_add_ext,
      { "uraUpdateConfirm-r7-add-ext", "rrc.uraUpdateConfirm_r7_add_ext",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING", HFILL }},
    { &hf_rrc_v860NonCriticalExtensions_20,
      { "v860NonCriticalExtensions", "rrc.v860NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v860NonCriticalExtensions_20", HFILL }},
    { &hf_rrc_uraUpdateConfirm_v860ext,
      { "uraUpdateConfirm-v860ext", "rrc.uraUpdateConfirm_v860ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "URAUpdateConfirm_v860ext_IEs", HFILL }},
    { &hf_rrc_nonCriticalExtensions_171,
      { "nonCriticalExtensions", "rrc.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_nonCriticalExtensions_171", HFILL }},
    { &hf_rrc_criticalExtensions_129,
      { "criticalExtensions", "rrc.criticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_criticalExtensions_128", HFILL }},
    { &hf_rrc_r3_29,
      { "r3", "rrc.r3",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_r3_29", HFILL }},
    { &hf_rrc_uraUpdateConfirm_CCCH_r3,
      { "uraUpdateConfirm-CCCH-r3", "rrc.uraUpdateConfirm_CCCH_r3",
        FT_NONE, BASE_NONE, NULL, 0,
        "URAUpdateConfirm_CCCH_r3_IEs", HFILL }},
    { &hf_rrc_laterNonCriticalExtensions_63,
      { "laterNonCriticalExtensions", "rrc.laterNonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_laterNonCriticalExtensions_63", HFILL }},
    { &hf_rrc_uraUpdateConfirm_CCCH_r3_add_ext,
      { "uraUpdateConfirm-CCCH-r3-add-ext", "rrc.uraUpdateConfirm_CCCH_r3_add_ext",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING", HFILL }},
    { &hf_rrc_v690NonCriticalExtensions_45,
      { "v690NonCriticalExtensions", "rrc.v690NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v690NonCriticalExtensions_45", HFILL }},
    { &hf_rrc_v860NonCriticalExtensions_21,
      { "v860NonCriticalExtensions", "rrc.v860NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v860NonCriticalExtensions_21", HFILL }},
    { &hf_rrc_nonCriticalExtensions_172,
      { "nonCriticalExtensions", "rrc.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_nonCriticalExtensions_172", HFILL }},
    { &hf_rrc_later_than_r3_26,
      { "later-than-r3", "rrc.later_than_r3",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_later_than_r3_26", HFILL }},
    { &hf_rrc_criticalExtensions_130,
      { "criticalExtensions", "rrc.criticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_criticalExtensions_129", HFILL }},
    { &hf_rrc_uraUpdateConfirm_02,
      { "uraUpdateConfirm", "rrc.uraUpdateConfirm",
        FT_NONE, BASE_NONE, NULL, 0,
        "URAUpdateConfirm_r3_IEs", HFILL }},
    { &hf_rrc_r3_30,
      { "r3", "rrc.r3",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_r3_30", HFILL }},
    { &hf_rrc_utranMobilityInformation_r3,
      { "utranMobilityInformation-r3", "rrc.utranMobilityInformation_r3",
        FT_NONE, BASE_NONE, NULL, 0,
        "UTRANMobilityInformation_r3_IEs", HFILL }},
    { &hf_rrc_v3a0NonCriticalExtensions_11,
      { "v3a0NonCriticalExtensions", "rrc.v3a0NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v3a0NonCriticalExtensions_11", HFILL }},
    { &hf_rrc_utranMobilityInformation_v3a0ext,
      { "utranMobilityInformation-v3a0ext", "rrc.utranMobilityInformation_v3a0ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "UTRANMobilityInformation_v3a0ext_IEs", HFILL }},
    { &hf_rrc_laterNonCriticalExtensions_64,
      { "laterNonCriticalExtensions", "rrc.laterNonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_laterNonCriticalExtensions_64", HFILL }},
    { &hf_rrc_utranMobilityInformation_r3_add_ext,
      { "utranMobilityInformation-r3-add-ext", "rrc.utranMobilityInformation_r3_add_ext",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING", HFILL }},
    { &hf_rrc_v690NonCriticalExtensions_46,
      { "v690NonCriticalExtensions", "rrc.v690NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v690NonCriticalExtensions_46", HFILL }},
    { &hf_rrc_utranMobilityInformation_v690ext,
      { "utranMobilityInformation-v690ext", "rrc.utranMobilityInformation_v690ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "UtranMobilityInformation_v690ext_IEs", HFILL }},
    { &hf_rrc_v860NonCriticalExtentions,
      { "v860NonCriticalExtentions", "rrc.v860NonCriticalExtentions",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_utranMobilityInformation_v860ext,
      { "utranMobilityInformation-v860ext", "rrc.utranMobilityInformation_v860ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "UTRANMobilityInformation_v860ext1_IEs", HFILL }},
    { &hf_rrc_nonCriticalExtensions_173,
      { "nonCriticalExtensions", "rrc.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_nonCriticalExtensions_173", HFILL }},
    { &hf_rrc_later_than_r3_27,
      { "later-than-r3", "rrc.later_than_r3",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_later_than_r3_27", HFILL }},
    { &hf_rrc_criticalExtensions_131,
      { "criticalExtensions", "rrc.criticalExtensions",
        FT_UINT32, BASE_DEC, VALS(rrc_T_criticalExtensions_130_vals), 0,
        "T_criticalExtensions_130", HFILL }},
    { &hf_rrc_r5_13,
      { "r5", "rrc.r5",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_r5_13", HFILL }},
    { &hf_rrc_utranMobilityInformation_r5,
      { "utranMobilityInformation-r5", "rrc.utranMobilityInformation_r5",
        FT_NONE, BASE_NONE, NULL, 0,
        "UTRANMobilityInformation_r5_IEs", HFILL }},
    { &hf_rrc_v690NonCriticalExtensions_47,
      { "v690NonCriticalExtensions", "rrc.v690NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v690NonCriticalExtensions_47", HFILL }},
    { &hf_rrc_v860NonCriticalExtentions_01,
      { "v860NonCriticalExtentions", "rrc.v860NonCriticalExtentions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v860NonCriticalExtentions_01", HFILL }},
    { &hf_rrc_nonCriticalExtensions_174,
      { "nonCriticalExtensions", "rrc.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_nonCriticalExtensions_174", HFILL }},
    { &hf_rrc_criticalExtensions_132,
      { "criticalExtensions", "rrc.criticalExtensions",
        FT_UINT32, BASE_DEC, VALS(rrc_T_criticalExtensions_131_vals), 0,
        "T_criticalExtensions_131", HFILL }},
    { &hf_rrc_r7_14,
      { "r7", "rrc.r7",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_r7_14", HFILL }},
    { &hf_rrc_utranMobilityInformation_r7,
      { "utranMobilityInformation-r7", "rrc.utranMobilityInformation_r7",
        FT_NONE, BASE_NONE, NULL, 0,
        "UTRANMobilityInformation_r7_IEs", HFILL }},
    { &hf_rrc_utranMobilityInformation_r7_add_ext,
      { "utranMobilityInformation-r7-add-ext", "rrc.utranMobilityInformation_r7_add_ext",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING", HFILL }},
    { &hf_rrc_v860NonCriticalExtensions_22,
      { "v860NonCriticalExtensions", "rrc.v860NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v860NonCriticalExtensions_22", HFILL }},
    { &hf_rrc_utranMobilityInformation_v860ext_01,
      { "utranMobilityInformation-v860ext", "rrc.utranMobilityInformation_v860ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "UTRANMobilityInformation_v860ext2_IEs", HFILL }},
    { &hf_rrc_nonCriticalExtensions_175,
      { "nonCriticalExtensions", "rrc.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_nonCriticalExtensions_175", HFILL }},
    { &hf_rrc_criticalExtensions_133,
      { "criticalExtensions", "rrc.criticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_criticalExtensions_132", HFILL }},
    { &hf_rrc_ue_ConnTimersAndConstants,
      { "ue-ConnTimersAndConstants", "rrc.ue_ConnTimersAndConstants",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_cn_InformationInfo_02,
      { "cn-InformationInfo", "rrc.cn_InformationInfo",
        FT_NONE, BASE_NONE, NULL, 0,
        "CN_InformationInfoFull", HFILL }},
    { &hf_rrc_nonCriticalExtensions_176,
      { "nonCriticalExtensions", "rrc.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_nonCriticalExtensions_176", HFILL }},
    { &hf_rrc_ue_ConnTimersAndConstants_v3a0ext,
      { "ue-ConnTimersAndConstants-v3a0ext", "rrc.ue_ConnTimersAndConstants_v3a0ext",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_ue_ConnTimersAndConstants_01,
      { "ue-ConnTimersAndConstants", "rrc.ue_ConnTimersAndConstants",
        FT_NONE, BASE_NONE, NULL, 0,
        "UE_ConnTimersAndConstants_r5", HFILL }},
    { &hf_rrc_ue_ConnTimersAndConstants_02,
      { "ue-ConnTimersAndConstants", "rrc.ue_ConnTimersAndConstants",
        FT_NONE, BASE_NONE, NULL, 0,
        "UE_ConnTimersAndConstants_v860ext", HFILL }},
    { &hf_rrc_dedicatedPriorityInformation,
      { "dedicatedPriorityInformation", "rrc.dedicatedPriorityInformation",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_laterNonCriticalExtensions_65,
      { "laterNonCriticalExtensions", "rrc.laterNonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_laterNonCriticalExtensions_65", HFILL }},
    { &hf_rrc_utranMobilityInformationConfirm_r3_add_ext,
      { "utranMobilityInformationConfirm-r3-add-ext", "rrc.utranMobilityInformationConfirm_r3_add_ext",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING", HFILL }},
    { &hf_rrc_v770NonCriticalExtension_01,
      { "v770NonCriticalExtension", "rrc.v770NonCriticalExtension",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v770NonCriticalExtension_01", HFILL }},
    { &hf_rrc_utranMobilityInformationConfirm_v770ext,
      { "utranMobilityInformationConfirm-v770ext", "rrc.utranMobilityInformationConfirm_v770ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "UTRANMobilityInformationConfirm_v770ext_IEs", HFILL }},
    { &hf_rrc_va40NonCriticalExtensions_17,
      { "va40NonCriticalExtensions", "rrc.va40NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_va40NonCriticalExtensions_16", HFILL }},
    { &hf_rrc_utranMobilityInformationConfirm_va40ext,
      { "utranMobilityInformationConfirm-va40ext", "rrc.utranMobilityInformationConfirm_va40ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "UTRANMobilityInformationConfirm_va40ext_IEs", HFILL }},
    { &hf_rrc_nonCriticalExtensions_177,
      { "nonCriticalExtensions", "rrc.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_nonCriticalExtensions_177", HFILL }},
    { &hf_rrc_deferredMeasurementControlReading_06,
      { "deferredMeasurementControlReading", "rrc.deferredMeasurementControlReading",
        FT_UINT32, BASE_DEC, VALS(rrc_T_deferredMeasurementControlReading_06_vals), 0,
        "T_deferredMeasurementControlReading_06", HFILL }},
    { &hf_rrc_loggedMeasAvailable_05,
      { "loggedMeasAvailable", "rrc.loggedMeasAvailable",
        FT_UINT32, BASE_DEC, VALS(rrc_T_loggedMeasAvailable_05_vals), 0,
        "T_loggedMeasAvailable_05", HFILL }},
    { &hf_rrc_laterNonCriticalExtensions_66,
      { "laterNonCriticalExtensions", "rrc.laterNonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_laterNonCriticalExtensions_66", HFILL }},
    { &hf_rrc_utranMobilityInformationFailure_r3_add_ext,
      { "utranMobilityInformationFailure-r3-add-ext", "rrc.utranMobilityInformationFailure_r3_add_ext",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING", HFILL }},
    { &hf_rrc_nonCriticalExtensions_178,
      { "nonCriticalExtensions", "rrc.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_nonCriticalExtensions_178", HFILL }},
    { &hf_rrc_mbms_ServiceAccessInfoList,
      { "mbms-ServiceAccessInfoList", "rrc.mbms_ServiceAccessInfoList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "MBMS_ServiceAccessInfoList_r6", HFILL }},
    { &hf_rrc_nonCriticalExtensions_179,
      { "nonCriticalExtensions", "rrc.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_nonCriticalExtensions_179", HFILL }},
    { &hf_rrc_mbms_CommonRBInformationList,
      { "mbms-CommonRBInformationList", "rrc.mbms_CommonRBInformationList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "MBMS_CommonRBInformationList_r6", HFILL }},
    { &hf_rrc_mbms_TranspChInfoForEachTrCh,
      { "mbms-TranspChInfoForEachTrCh", "rrc.mbms_TranspChInfoForEachTrCh",
        FT_UINT32, BASE_DEC, NULL, 0,
        "MBMS_TranspChInfoForEachTrCh_r6", HFILL }},
    { &hf_rrc_mbms_TranspChInfoForEachCCTrCh,
      { "mbms-TranspChInfoForEachCCTrCh", "rrc.mbms_TranspChInfoForEachCCTrCh",
        FT_UINT32, BASE_DEC, NULL, 0,
        "MBMS_TranspChInfoForEachCCTrCh_r6", HFILL }},
    { &hf_rrc_mbms_PhyChInformationList,
      { "mbms-PhyChInformationList", "rrc.mbms_PhyChInformationList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "MBMS_PhyChInformationList_r6", HFILL }},
    { &hf_rrc_v770NonCriticalExtensions_33,
      { "v770NonCriticalExtensions", "rrc.v770NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v770NonCriticalExtensions_33", HFILL }},
    { &hf_rrc_mbmsCommonPTMRBInformation_v770ext,
      { "mbmsCommonPTMRBInformation-v770ext", "rrc.mbmsCommonPTMRBInformation_v770ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "MBMSCommonPTMRBInformation_v770ext_IEs", HFILL }},
    { &hf_rrc_v780NonCriticalExtensions_10,
      { "v780NonCriticalExtensions", "rrc.v780NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v780NonCriticalExtensions_10", HFILL }},
    { &hf_rrc_mbmsCommonPTMRBInformation_v780ext,
      { "mbmsCommonPTMRBInformation-v780ext", "rrc.mbmsCommonPTMRBInformation_v780ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "MBMSCommonPTMRBInformation_v780ext_IEs", HFILL }},
    { &hf_rrc_v860NonCriticalExtensions_23,
      { "v860NonCriticalExtensions", "rrc.v860NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v860NonCriticalExtensions_23", HFILL }},
    { &hf_rrc_mbmsCommonPTMRBInformation_v860ext,
      { "mbmsCommonPTMRBInformation-v860ext", "rrc.mbmsCommonPTMRBInformation_v860ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "MBMSCommonPTMRBInformation_v860ext_IEs", HFILL }},
    { &hf_rrc_nonCriticalExtensions_180,
      { "nonCriticalExtensions", "rrc.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_nonCriticalExtensions_180", HFILL }},
    { &hf_rrc_mbms_PhyChInformationList_r7,
      { "mbms-PhyChInformationList-r7", "rrc.mbms_PhyChInformationList_r7",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_mbsfn_TDDInformation_LCR,
      { "mbsfn-TDDInformation-LCR", "rrc.mbsfn_TDDInformation_LCR",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_mbms_PhyChInformationList_01,
      { "mbms-PhyChInformationList", "rrc.mbms_PhyChInformationList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "MBMS_PhyChInformationList_IMB384", HFILL }},
    { &hf_rrc_mbms_CurrentCell_SCCPCHList,
      { "mbms-CurrentCell-SCCPCHList", "rrc.mbms_CurrentCell_SCCPCHList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "MBMS_CurrentCell_SCCPCHList_r6", HFILL }},
    { &hf_rrc_mbms_SIBType5_SCCPCHList,
      { "mbms-SIBType5-SCCPCHList", "rrc.mbms_SIBType5_SCCPCHList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "MBMS_SIBType5_SCCPCHList_r6", HFILL }},
    { &hf_rrc_v770NonCriticalExtensions_34,
      { "v770NonCriticalExtensions", "rrc.v770NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v770NonCriticalExtensions_34", HFILL }},
    { &hf_rrc_mbmsCurrentCellPTMRBInfo_v770ext,
      { "mbmsCurrentCellPTMRBInfo-v770ext", "rrc.mbmsCurrentCellPTMRBInfo_v770ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "MBMSCurrentCellPTMRBInfo_v770ext_IEs", HFILL }},
    { &hf_rrc_nonCriticalExtensions_181,
      { "nonCriticalExtensions", "rrc.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_nonCriticalExtensions_181", HFILL }},
    { &hf_rrc_mbsfn_TDM_Info_List,
      { "mbsfn-TDM-Info-List", "rrc.mbsfn_TDM_Info_List",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_mbms_PreferredFrequencyInfo,
      { "mbms-PreferredFrequencyInfo", "rrc.mbms_PreferredFrequencyInfo",
        FT_UINT32, BASE_DEC, NULL, 0,
        "MBMS_PreferredFrequencyList_r6", HFILL }},
    { &hf_rrc_mbms_TimersAndCounters,
      { "mbms-TimersAndCounters", "rrc.mbms_TimersAndCounters",
        FT_NONE, BASE_NONE, NULL, 0,
        "MBMS_TimersAndCounters_r6", HFILL }},
    { &hf_rrc_michConfigurationInfo,
      { "michConfigurationInfo", "rrc.michConfigurationInfo",
        FT_NONE, BASE_NONE, NULL, 0,
        "MBMS_MICHConfigurationInfo_r6", HFILL }},
    { &hf_rrc_cellGroupIdentity,
      { "cellGroupIdentity", "rrc.cellGroupIdentity",
        FT_BYTES, BASE_NONE, NULL, 0,
        "MBMS_CellGroupIdentity_r6", HFILL }},
    { &hf_rrc_mschDefaultConfigurationInfo,
      { "mschDefaultConfigurationInfo", "rrc.mschDefaultConfigurationInfo",
        FT_NONE, BASE_NONE, NULL, 0,
        "MBMS_MSCH_ConfigurationInfo_r6", HFILL }},
    { &hf_rrc_v6b0NonCriticalExtensions_13,
      { "v6b0NonCriticalExtensions", "rrc.v6b0NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v6b0NonCriticalExtensions_12", HFILL }},
    { &hf_rrc_mbmsGeneralInformation_v6b0ext,
      { "mbmsGeneralInformation-v6b0ext", "rrc.mbmsGeneralInformation_v6b0ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "MBMSGeneralInformation_v6b0ext_IEs", HFILL }},
    { &hf_rrc_v770NonCriticalExtensions_35,
      { "v770NonCriticalExtensions", "rrc.v770NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v770NonCriticalExtensions_35", HFILL }},
    { &hf_rrc_mbmsGeneralInformation_v770ext,
      { "mbmsGeneralInformation-v770ext", "rrc.mbmsGeneralInformation_v770ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "MBMSGeneralInformation_v770ext_IEs", HFILL }},
    { &hf_rrc_v860NonCriticalExtensions_24,
      { "v860NonCriticalExtensions", "rrc.v860NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v860NonCriticalExtensions_24", HFILL }},
    { &hf_rrc_mbmsGeneralInformation_v860ext,
      { "mbmsGeneralInformation-v860ext", "rrc.mbmsGeneralInformation_v860ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "MBMSGeneralInformation_v860ext_IEs", HFILL }},
    { &hf_rrc_v890NoncriticalExtensions,
      { "v890NoncriticalExtensions", "rrc.v890NoncriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_mbmsGeneralInformation_v890ext,
      { "mbmsGeneralInformation-v890ext", "rrc.mbmsGeneralInformation_v890ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "MBMSGeneralInformation_v890ext_IEs", HFILL }},
    { &hf_rrc_nonCriticalExtensions_182,
      { "nonCriticalExtensions", "rrc.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_nonCriticalExtensions_182", HFILL }},
    { &hf_rrc_indicateChangeInSelectedServices,
      { "indicateChangeInSelectedServices", "rrc.indicateChangeInSelectedServices",
        FT_BOOLEAN, BASE_NONE, NULL, 0,
        "BOOLEAN", HFILL }},
    { &hf_rrc_mbmsMICHConfiguration,
      { "mbmsMICHConfiguration", "rrc.mbmsMICHConfiguration",
        FT_NONE, BASE_NONE, NULL, 0,
        "MBMS_MICHConfigurationInfo_v770ext", HFILL }},
    { &hf_rrc_mbsfnInterFrequencyNeighbourList,
      { "mbsfnInterFrequencyNeighbourList", "rrc.mbsfnInterFrequencyNeighbourList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "MBSFN_InterFrequencyNeighbourList_r7", HFILL }},
    { &hf_rrc_mbsfnInterFrequencyNeighbourList_01,
      { "mbsfnInterFrequencyNeighbourList", "rrc.mbsfnInterFrequencyNeighbourList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "MBSFN_InterFrequencyNeighbourList_v860ext", HFILL }},
    { &hf_rrc_mbmsNetworkStandardTimeInformation_LCR,
      { "mbmsNetworkStandardTimeInformation-LCR", "rrc.mbmsNetworkStandardTimeInformation_LCR",
        FT_NONE, BASE_NONE, NULL, 0,
        "MBMS_NetworkStandardTimeInformation_LCR_v890ext", HFILL }},
    { &hf_rrc_mbmsMICHConfiguration_01,
      { "mbmsMICHConfiguration", "rrc.mbmsMICHConfiguration",
        FT_NONE, BASE_NONE, NULL, 0,
        "MBMS_MICHConfigurationInfo_v890ext", HFILL }},
    { &hf_rrc_mbms_PreferredFreqRequest,
      { "mbms-PreferredFreqRequest", "rrc.mbms_PreferredFreqRequest",
        FT_NONE, BASE_NONE, NULL, 0,
        "MBMS_ServiceIdentity_r6", HFILL }},
    { &hf_rrc_v6b0NonCriticalExtensions_14,
      { "v6b0NonCriticalExtensions", "rrc.v6b0NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v6b0NonCriticalExtensions_13", HFILL }},
    { &hf_rrc_mbmsModificationRequest_v6b0ext,
      { "mbmsModificationRequest-v6b0ext", "rrc.mbmsModificationRequest_v6b0ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "MBMSModificationRequest_v6b0ext_IEs", HFILL }},
    { &hf_rrc_v6f0NonCriticalExtensions_02,
      { "v6f0NonCriticalExtensions", "rrc.v6f0NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v6f0NonCriticalExtensions_02", HFILL }},
    { &hf_rrc_mbmsModificationRequest_v6f0ext,
      { "mbmsModificationRequest-v6f0ext", "rrc.mbmsModificationRequest_v6f0ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "MBMSModificationRequest_v6f0ext_IEs", HFILL }},
    { &hf_rrc_nonCriticalExtensions_183,
      { "nonCriticalExtensions", "rrc.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_nonCriticalExtensions_183", HFILL }},
    { &hf_rrc_mbmsSelectedServiceInfo,
      { "mbmsSelectedServiceInfo", "rrc.mbmsSelectedServiceInfo",
        FT_NONE, BASE_NONE, NULL, 0,
        "MBMS_SelectedServiceInfo", HFILL }},
    { &hf_rrc_mbmsSupportOfServiceChangeForAPtpRB,
      { "mbmsSupportOfServiceChangeForAPtpRB", "rrc.mbmsSupportOfServiceChangeForAPtpRB",
        FT_UINT32, BASE_DEC, VALS(rrc_T_mbmsSupportOfServiceChangeForAPtpRB_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_modifedServiceList,
      { "modifedServiceList", "rrc.modifedServiceList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "MBMS_ModifedServiceList_r6", HFILL }},
    { &hf_rrc_mbms_ReacquireMCCH,
      { "mbms-ReacquireMCCH", "rrc.mbms_ReacquireMCCH",
        FT_UINT32, BASE_DEC, VALS(rrc_T_mbms_ReacquireMCCH_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_mbms_DynamicPersistenceLevel,
      { "mbms-DynamicPersistenceLevel", "rrc.mbms_DynamicPersistenceLevel",
        FT_UINT32, BASE_DEC, NULL, 0,
        "DynamicPersistenceLevel", HFILL }},
    { &hf_rrc_endOfModifiedMCCHInformation,
      { "endOfModifiedMCCHInformation", "rrc.endOfModifiedMCCHInformation",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_1_16", HFILL }},
    { &hf_rrc_mbmsNumberOfNeighbourCells,
      { "mbmsNumberOfNeighbourCells", "rrc.mbmsNumberOfNeighbourCells",
        FT_UINT32, BASE_DEC, NULL, 0,
        "MBMS_NumberOfNeighbourCells_r6", HFILL }},
    { &hf_rrc_mbms_AllUnmodifiedPTMServices,
      { "mbms-AllUnmodifiedPTMServices", "rrc.mbms_AllUnmodifiedPTMServices",
        FT_UINT32, BASE_DEC, VALS(rrc_T_mbms_AllUnmodifiedPTMServices_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_mbms_PTMActivationTime,
      { "mbms-PTMActivationTime", "rrc.mbms_PTMActivationTime",
        FT_UINT32, BASE_DEC, NULL, 0,
        "MBMS_PTMActivationTime_r6", HFILL }},
    { &hf_rrc_v770NonCriticalExtensions_36,
      { "v770NonCriticalExtensions", "rrc.v770NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v770NonCriticalExtensions_36", HFILL }},
    { &hf_rrc_mbmsModifiedServicesInformation_v770ext,
      { "mbmsModifiedServicesInformation-v770ext", "rrc.mbmsModifiedServicesInformation_v770ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "MBMSModifiedServicesInformation_v770ext_IEs", HFILL }},
    { &hf_rrc_v7c0NonCriticalExtensions,
      { "v7c0NonCriticalExtensions", "rrc.v7c0NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_mbmsModifiedServicesInformation_v7c0ext,
      { "mbmsModifiedServicesInformation-v7c0ext", "rrc.mbmsModifiedServicesInformation_v7c0ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "MBMSModifiedServicesInformation_v7c0ext_IEs", HFILL }},
    { &hf_rrc_nonCriticalExtensions_184,
      { "nonCriticalExtensions", "rrc.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_nonCriticalExtensions_184", HFILL }},
    { &hf_rrc_modifiedServiceList,
      { "modifiedServiceList", "rrc.modifiedServiceList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "MBMS_ModifiedServiceList_v770ext", HFILL }},
    { &hf_rrc_mib_ValueTag,
      { "mib-ValueTag", "rrc.mib_ValueTag",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_modifiedServiceList_01,
      { "modifiedServiceList", "rrc.modifiedServiceList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "MBMS_ModifiedServiceList_LCR_v7c0ext", HFILL }},
    { &hf_rrc_neighbouringCellIdentity,
      { "neighbouringCellIdentity", "rrc.neighbouringCellIdentity",
        FT_UINT32, BASE_DEC, NULL, 0,
        "IntraFreqCellID", HFILL }},
    { &hf_rrc_neighbouringCellSCCPCHList,
      { "neighbouringCellSCCPCHList", "rrc.neighbouringCellSCCPCHList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "MBMS_NeighbouringCellSCCPCHList_r6", HFILL }},
    { &hf_rrc_v770NonCriticalExtensions_37,
      { "v770NonCriticalExtensions", "rrc.v770NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v770NonCriticalExtensions_37", HFILL }},
    { &hf_rrc_mbmsNeighbouringCellPTMRBInformation_v770ext,
      { "mbmsNeighbouringCellPTMRBInformation-v770ext", "rrc.mbmsNeighbouringCellPTMRBInformation_v770ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "MBMSNeighbouringCellPTMRBInformation_v770ext_IEs", HFILL }},
    { &hf_rrc_nonCriticalExtensions_185,
      { "nonCriticalExtensions", "rrc.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_nonCriticalExtensions_185", HFILL }},
    { &hf_rrc_neighbouringCellSCCPCHList_01,
      { "neighbouringCellSCCPCHList", "rrc.neighbouringCellSCCPCHList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "MBMS_NeighbouringCellSCCPCHList_v770ext", HFILL }},
    { &hf_rrc_serviceSchedulingInfoList,
      { "serviceSchedulingInfoList", "rrc.serviceSchedulingInfoList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "MBMS_ServiceSchedulingInfoList_r6", HFILL }},
    { &hf_rrc_nonCriticalExtensions_186,
      { "nonCriticalExtensions", "rrc.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_nonCriticalExtensions_186", HFILL }},
    { &hf_rrc_unmodifiedServiceList,
      { "unmodifiedServiceList", "rrc.unmodifiedServiceList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "MBMS_UnmodifiedServiceList_r6", HFILL }},
    { &hf_rrc_v770NonCriticalExtensions_38,
      { "v770NonCriticalExtensions", "rrc.v770NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v770NonCriticalExtensions_38", HFILL }},
    { &hf_rrc_mbmsUnmodifiedServicesInformation_v770ext,
      { "mbmsUnmodifiedServicesInformation-v770ext", "rrc.mbmsUnmodifiedServicesInformation_v770ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "MBMSUnmodifiedServicesInformation_v770ext_IEs", HFILL }},
    { &hf_rrc_nonCriticalExtensions_187,
      { "nonCriticalExtensions", "rrc.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_nonCriticalExtensions_187", HFILL }},
    { &hf_rrc_unmodifiedServiceList_01,
      { "unmodifiedServiceList", "rrc.unmodifiedServiceList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "MBMS_UnmodifiedServiceList_v770ext", HFILL }},
    { &hf_rrc_mib,
      { "mib", "rrc.mib",
        FT_BYTES, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_sysInfoTypeSB1,
      { "sysInfoTypeSB1", "rrc.sysInfoTypeSB1",
        FT_BYTES, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_sysInfoTypeSB2,
      { "sysInfoTypeSB2", "rrc.sysInfoTypeSB2",
        FT_BYTES, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_sysInfoType1,
      { "sysInfoType1", "rrc.sysInfoType1",
        FT_BYTES, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_sysInfoType3,
      { "sysInfoType3", "rrc.sysInfoType3",
        FT_BYTES, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_sysInfoType5,
      { "sysInfoType5", "rrc.sysInfoType5",
        FT_BYTES, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_sysInfoType7,
      { "sysInfoType7", "rrc.sysInfoType7",
        FT_BYTES, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_sysInfoType11,
      { "sysInfoType11", "rrc.sysInfoType11",
        FT_BYTES, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_sysInfoType11bis,
      { "sysInfoType11bis", "rrc.sysInfoType11bis",
        FT_BYTES, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_sysInfoType12,
      { "sysInfoType12", "rrc.sysInfoType12",
        FT_BYTES, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_vbxyNonCriticalExtensions_16,
      { "vbxyNonCriticalExtensions", "rrc.vbxyNonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_vbxyNonCriticalExtensions_16", HFILL }},
    { &hf_rrc_system_Information_Container_vbxyext,
      { "system-Information-Container-vbxyext", "rrc.system_Information_Container_vbxyext",
        FT_NONE, BASE_NONE, NULL, 0,
        "System_Information_Container_vbxyext_IEs", HFILL }},
    { &hf_rrc_nonCriticalExtensions_188,
      { "nonCriticalExtensions", "rrc.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_nonCriticalExtensions_188", HFILL }},
    { &hf_rrc_sysInfoType22,
      { "sysInfoType22", "rrc.sysInfoType22",
        FT_BYTES, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_criticalExtensions_134,
      { "criticalExtensions", "rrc.criticalExtensions",
        FT_UINT32, BASE_DEC, VALS(rrc_T_criticalExtensions_133_vals), 0,
        "T_criticalExtensions_133", HFILL }},
    { &hf_rrc_r10_11,
      { "r10", "rrc.r10",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_r10_11", HFILL }},
    { &hf_rrc_loggingMeasurementConfiguration_r10_IEs,
      { "loggingMeasurementConfiguration-r10-IEs", "rrc.loggingMeasurementConfiguration_r10_IEs",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_loggingMeasurementConfiguration_r10_add_ext,
      { "loggingMeasurementConfiguration-r10-add-ext", "rrc.loggingMeasurementConfiguration_r10_add_ext",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING", HFILL }},
    { &hf_rrc_nonCriticalExtensions_189,
      { "nonCriticalExtensions", "rrc.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_nonCriticalExtensions_189", HFILL }},
    { &hf_rrc_criticalExtensions_135,
      { "criticalExtensions", "rrc.criticalExtensions",
        FT_UINT32, BASE_DEC, VALS(rrc_T_criticalExtensions_134_vals), 0,
        "T_criticalExtensions_134", HFILL }},
    { &hf_rrc_r11_12,
      { "r11", "rrc.r11",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_r11_12", HFILL }},
    { &hf_rrc_loggingMeasurementConfiguration_r11_IEs,
      { "loggingMeasurementConfiguration-r11-IEs", "rrc.loggingMeasurementConfiguration_r11_IEs",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_loggingMeasurementConfiguration_r11_add_ext,
      { "loggingMeasurementConfiguration-r11-add-ext", "rrc.loggingMeasurementConfiguration_r11_add_ext",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING", HFILL }},
    { &hf_rrc_nonCriticalExtensions_190,
      { "nonCriticalExtensions", "rrc.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_nonCriticalExtensions_190", HFILL }},
    { &hf_rrc_criticalExtensions_136,
      { "criticalExtensions", "rrc.criticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_criticalExtensions_135", HFILL }},
    { &hf_rrc_loggedMeasurementsConfigurationInfo,
      { "loggedMeasurementsConfigurationInfo", "rrc.loggedMeasurementsConfigurationInfo",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_loggedANRConfigurationInfo,
      { "loggedANRConfigurationInfo", "rrc.loggedANRConfigurationInfo",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_loggedMeasurementsConfigurationInfo_01,
      { "loggedMeasurementsConfigurationInfo", "rrc.loggedMeasurementsConfigurationInfo",
        FT_NONE, BASE_NONE, NULL, 0,
        "LoggedMeasurementsConfigurationInfo_r11", HFILL }},
    { &hf_rrc_loggedMeasReport,
      { "loggedMeasReport", "rrc.loggedMeasReport",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_loggedANRReportInfoList,
      { "loggedANRReportInfoList", "rrc.loggedANRReportInfoList",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_vbxyNonCriticalExtensions_17,
      { "vbxyNonCriticalExtensions", "rrc.vbxyNonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_vbxyNonCriticalExtensions_17", HFILL }},
    { &hf_rrc_ueInformationResponse_vbxyext,
      { "ueInformationResponse-vbxyext", "rrc.ueInformationResponse_vbxyext",
        FT_NONE, BASE_NONE, NULL, 0,
        "UEInformationResponse_vbxyext_IEs", HFILL }},
    { &hf_rrc_nonCriticalExtensions_191,
      { "nonCriticalExtensions", "rrc.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_nonCriticalExtensions_191", HFILL }},
    { &hf_rrc_loggedMeasReport_01,
      { "loggedMeasReport", "rrc.loggedMeasReport",
        FT_NONE, BASE_NONE, NULL, 0,
        "LoggedMeasReport_vbxyext", HFILL }},
    { &hf_rrc_loggedConnectionEstablishmentFailureReport,
      { "loggedConnectionEstablishmentFailureReport", "rrc.loggedConnectionEstablishmentFailureReport",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_criticalExtensions_137,
      { "criticalExtensions", "rrc.criticalExtensions",
        FT_UINT32, BASE_DEC, VALS(rrc_T_criticalExtensions_136_vals), 0,
        "T_criticalExtensions_136", HFILL }},
    { &hf_rrc_r10_12,
      { "r10", "rrc.r10",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_r10_12", HFILL }},
    { &hf_rrc_ueInformationRequest_r10,
      { "ueInformationRequest-r10", "rrc.ueInformationRequest_r10",
        FT_NONE, BASE_NONE, NULL, 0,
        "UEInformationRequest_r10_IEs", HFILL }},
    { &hf_rrc_ueInformationRequest_r10_add_ext,
      { "ueInformationRequest-r10-add-ext", "rrc.ueInformationRequest_r10_add_ext",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING", HFILL }},
    { &hf_rrc_nonCriticalExtensions_192,
      { "nonCriticalExtensions", "rrc.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_nonCriticalExtensions_192", HFILL }},
    { &hf_rrc_criticalExtensions_138,
      { "criticalExtensions", "rrc.criticalExtensions",
        FT_UINT32, BASE_DEC, VALS(rrc_T_criticalExtensions_137_vals), 0,
        "T_criticalExtensions_137", HFILL }},
    { &hf_rrc_r11_13,
      { "r11", "rrc.r11",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_r11_13", HFILL }},
    { &hf_rrc_ueInformationRequest_r11,
      { "ueInformationRequest-r11", "rrc.ueInformationRequest_r11",
        FT_NONE, BASE_NONE, NULL, 0,
        "UEInformationRequest_r11_IEs", HFILL }},
    { &hf_rrc_ueInformationRequest_r11_add_ext,
      { "ueInformationRequest-r11-add-ext", "rrc.ueInformationRequest_r11_add_ext",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING", HFILL }},
    { &hf_rrc_nonCriticalExtensions_193,
      { "nonCriticalExtensions", "rrc.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_nonCriticalExtensions_193", HFILL }},
    { &hf_rrc_criticalExtensions_139,
      { "criticalExtensions", "rrc.criticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_criticalExtensions_138", HFILL }},
    { &hf_rrc_loggedMeasReportRequest,
      { "loggedMeasReportRequest", "rrc.loggedMeasReportRequest",
        FT_UINT32, BASE_DEC, VALS(rrc_T_loggedMeasReportRequest_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_loggedANRReportRequest,
      { "loggedANRReportRequest", "rrc.loggedANRReportRequest",
        FT_UINT32, BASE_DEC, VALS(rrc_T_loggedANRReportRequest_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_loggedMeasReportRequest_01,
      { "loggedMeasReportRequest", "rrc.loggedMeasReportRequest",
        FT_UINT32, BASE_DEC, VALS(rrc_T_loggedMeasReportRequest_01_vals), 0,
        "T_loggedMeasReportRequest_01", HFILL }},
    { &hf_rrc_loggedANRReportRequest_01,
      { "loggedANRReportRequest", "rrc.loggedANRReportRequest",
        FT_UINT32, BASE_DEC, VALS(rrc_T_loggedANRReportRequest_01_vals), 0,
        "T_loggedANRReportRequest_01", HFILL }},
    { &hf_rrc_loggedConnectionEstablishmentFailureRequest,
      { "loggedConnectionEstablishmentFailureRequest", "rrc.loggedConnectionEstablishmentFailureRequest",
        FT_UINT32, BASE_DEC, VALS(rrc_T_loggedConnectionEstablishmentFailureRequest_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_cn_DomainSpecificNAS_Info,
      { "cn-DomainSpecificNAS-Info", "rrc.cn_DomainSpecificNAS_Info",
        FT_BYTES, BASE_NONE, NULL, 0,
        "NAS_SystemInformationGSM_MAP", HFILL }},
    { &hf_rrc_cn_DRX_CycleLengthCoeff,
      { "cn-DRX-CycleLengthCoeff", "rrc.cn_DRX_CycleLengthCoeff",
        FT_UINT32, BASE_DEC, NULL, 0,
        "CN_DRX_CycleLengthCoefficient", HFILL }},
    { &hf_rrc_CN_DomainInformationList_item,
      { "CN-DomainInformation", "rrc.CN_DomainInformation",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_CN_DomainInformationListFull_item,
      { "CN-DomainInformationFull", "rrc.CN_DomainInformationFull",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_cn_Type,
      { "cn-Type", "rrc.cn_Type",
        FT_UINT32, BASE_DEC, VALS(rrc_T_cn_Type_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_gsm_MAP,
      { "gsm-MAP", "rrc.gsm_MAP",
        FT_BYTES, BASE_NONE, NULL, 0,
        "NAS_SystemInformationGSM_MAP", HFILL }},
    { &hf_rrc_ansi_41,
      { "ansi-41", "rrc.ansi_41",
        FT_BYTES, BASE_NONE, NULL, 0,
        "NAS_SystemInformationANSI_41", HFILL }},
    { &hf_rrc_CN_DomainSysInfoList_item,
      { "CN-DomainSysInfo", "rrc.CN_DomainSysInfo",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_cn_CommonGSM_MAP_NAS_SysInfo,
      { "cn-CommonGSM-MAP-NAS-SysInfo", "rrc.cn_CommonGSM_MAP_NAS_SysInfo",
        FT_BYTES, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_cn_DomainInformationList,
      { "cn-DomainInformationList", "rrc.cn_DomainInformationList",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_cn_CommonGSM_MAP_NAS_SysInfo_01,
      { "cn-CommonGSM-MAP-NAS-SysInfo", "rrc.cn_CommonGSM_MAP_NAS_SysInfo",
        FT_BYTES, BASE_NONE, NULL, 0,
        "T_cn_CommonGSM_MAP_NAS_SysInfo_01", HFILL }},
    { &hf_rrc_cn_CommonGSM_MAP_NAS_SysInfo_02,
      { "cn-CommonGSM-MAP-NAS-SysInfo", "rrc.cn_CommonGSM_MAP_NAS_SysInfo",
        FT_BYTES, BASE_NONE, NULL, 0,
        "T_cn_CommonGSM_MAP_NAS_SysInfo_02", HFILL }},
    { &hf_rrc_cn_DomainInformationListFull,
      { "cn-DomainInformationListFull", "rrc.cn_DomainInformationListFull",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_cs_DomainSpecificEAB_Parameters,
      { "cs-DomainSpecificEAB-Parameters", "rrc.cs_DomainSpecificEAB_Parameters",
        FT_NONE, BASE_NONE, NULL, 0,
        "EAB_Configuration", HFILL }},
    { &hf_rrc_ps_DomainSpecificEAB_Parameters,
      { "ps-DomainSpecificEAB-Parameters", "rrc.ps_DomainSpecificEAB_Parameters",
        FT_NONE, BASE_NONE, NULL, 0,
        "EAB_Configuration", HFILL }},
    { &hf_rrc_commonEAB_Parameters,
      { "commonEAB-Parameters", "rrc.commonEAB_Parameters",
        FT_NONE, BASE_NONE, NULL, 0,
        "EAB_Configuration", HFILL }},
    { &hf_rrc_EAB_AccessClassBarredList_item,
      { "AccessClassBarred", "rrc.AccessClassBarred",
        FT_UINT32, BASE_DEC, VALS(rrc_AccessClassBarred_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_eab_Category,
      { "eab-Category", "rrc.eab_Category",
        FT_UINT32, BASE_DEC, VALS(rrc_T_eab_Category_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_eab_AccessClassBarredList,
      { "eab-AccessClassBarredList", "rrc.eab_AccessClassBarredList",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_eab_ParametersPerPLMN_List,
      { "eab-ParametersPerPLMN-List", "rrc.eab_ParametersPerPLMN_List",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_eab_ParametersForAll,
      { "eab-ParametersForAll", "rrc.eab_ParametersForAll",
        FT_NONE, BASE_NONE, NULL, 0,
        "DomainSpecificEAB_Parameters", HFILL }},
    { &hf_rrc_EAB_ParametersPerPLMN_List_item,
      { "EAB-ParametersPerPLMN-List item", "rrc.EAB_ParametersPerPLMN_List_item",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_domainSpecificEAB_Parameters,
      { "domainSpecificEAB-Parameters", "rrc.domainSpecificEAB_Parameters",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_ec_N0,
      { "ec-N0", "rrc.ec_N0",
        FT_INT32, BASE_DEC, NULL, 0,
        "INTEGER_M24_0", HFILL }},
    { &hf_rrc_routingbasis,
      { "routingbasis", "rrc.routingbasis",
        FT_UINT32, BASE_DEC, VALS(rrc_T_routingbasis_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_localPTMSI,
      { "localPTMSI", "rrc.localPTMSI",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_routingparameter,
      { "routingparameter", "rrc.routingparameter",
        FT_BYTES, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_tMSIofsamePLMN,
      { "tMSIofsamePLMN", "rrc.tMSIofsamePLMN",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_tMSIofdifferentPLMN,
      { "tMSIofdifferentPLMN", "rrc.tMSIofdifferentPLMN",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_iMSIresponsetopaging,
      { "iMSIresponsetopaging", "rrc.iMSIresponsetopaging",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_iMSIcauseUEinitiatedEvent,
      { "iMSIcauseUEinitiatedEvent", "rrc.iMSIcauseUEinitiatedEvent",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_iMEI,
      { "iMEI", "rrc.iMEI",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_spare2_01,
      { "spare2", "rrc.spare2",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_spare1_01,
      { "spare1", "rrc.spare1",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_dummy_28,
      { "dummy", "rrc.dummy",
        FT_BOOLEAN, BASE_NONE, NULL, 0,
        "BOOLEAN", HFILL }},
    { &hf_rrc_IMEI_item,
      { "IMEI-Digit", "rrc.IMEI_Digit",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_IMSI_GSM_MAP_item,
      { "Digit", "rrc.Digit",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_version,
      { "version", "rrc.version",
        FT_UINT32, BASE_DEC, VALS(rrc_T_version_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_release99,
      { "release99", "rrc.release99",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_cn_Type_01,
      { "cn-Type", "rrc.cn_Type",
        FT_UINT32, BASE_DEC, VALS(rrc_T_cn_Type_01_vals), 0,
        "T_cn_Type_01", HFILL }},
    { &hf_rrc_gsm_Map_IDNNS,
      { "gsm-Map-IDNNS", "rrc.gsm_Map_IDNNS",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_ansi_41_IDNNS,
      { "ansi-41-IDNNS", "rrc.ansi_41_IDNNS",
        FT_BYTES, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_later,
      { "later", "rrc.later",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_futurecoding,
      { "futurecoding", "rrc.futurecoding",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_15", HFILL }},
    { &hf_rrc_lac,
      { "lac", "rrc.lac",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_16", HFILL }},
    { &hf_rrc_LocationRegistrationAccessClassBarredList_item,
      { "AccessClassBarred", "rrc.AccessClassBarred",
        FT_UINT32, BASE_DEC, VALS(rrc_AccessClassBarred_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_noRestriction,
      { "noRestriction", "rrc.noRestriction",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_restriction,
      { "restriction", "rrc.restriction",
        FT_UINT32, BASE_DEC, NULL, 0,
        "LocationRegistrationAccessClassBarredList", HFILL }},
    { &hf_rrc_MCC_item,
      { "Digit", "rrc.Digit",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_MNC_item,
      { "Digit", "rrc.Digit",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_mibPLMN_Identity,
      { "mibPLMN-Identity", "rrc.mibPLMN_Identity",
        FT_BOOLEAN, BASE_NONE, NULL, 0,
        "BOOLEAN", HFILL }},
    { &hf_rrc_multiplePLMNs,
      { "multiplePLMNs", "rrc.multiplePLMNs",
        FT_UINT32, BASE_DEC, NULL, 0,
        "SEQUENCE_SIZE_1_5_OF_PLMN_IdentityWithOptionalMCC_r6", HFILL }},
    { &hf_rrc_multiplePLMNs_item,
      { "PLMN-IdentityWithOptionalMCC-r6", "rrc.PLMN_IdentityWithOptionalMCC_r6",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_pagingPermissionWithAccessControlList,
      { "pagingPermissionWithAccessControlList", "rrc.pagingPermissionWithAccessControlList",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_pagingPermissionWithAccessControlForAll,
      { "pagingPermissionWithAccessControlForAll", "rrc.pagingPermissionWithAccessControlForAll",
        FT_NONE, BASE_NONE, NULL, 0,
        "PagingPermissionWithAccessControlParameters", HFILL }},
    { &hf_rrc_pagingPermissionWithAccessControlParametersForOperator1,
      { "pagingPermissionWithAccessControlParametersForOperator1", "rrc.pagingPermissionWithAccessControlParametersForOperator1",
        FT_NONE, BASE_NONE, NULL, 0,
        "PagingPermissionWithAccessControlParameters", HFILL }},
    { &hf_rrc_pagingPermissionWithAccessControlParametersForOperator2,
      { "pagingPermissionWithAccessControlParametersForOperator2", "rrc.pagingPermissionWithAccessControlParametersForOperator2",
        FT_NONE, BASE_NONE, NULL, 0,
        "PagingPermissionWithAccessControlParameters", HFILL }},
    { &hf_rrc_pagingPermissionWithAccessControlParametersForOperator3,
      { "pagingPermissionWithAccessControlParametersForOperator3", "rrc.pagingPermissionWithAccessControlParametersForOperator3",
        FT_NONE, BASE_NONE, NULL, 0,
        "PagingPermissionWithAccessControlParameters", HFILL }},
    { &hf_rrc_pagingPermissionWithAccessControlParametersForOperator4,
      { "pagingPermissionWithAccessControlParametersForOperator4", "rrc.pagingPermissionWithAccessControlParametersForOperator4",
        FT_NONE, BASE_NONE, NULL, 0,
        "PagingPermissionWithAccessControlParameters", HFILL }},
    { &hf_rrc_pagingPermissionWithAccessControlParametersForOperator5,
      { "pagingPermissionWithAccessControlParametersForOperator5", "rrc.pagingPermissionWithAccessControlParametersForOperator5",
        FT_NONE, BASE_NONE, NULL, 0,
        "PagingPermissionWithAccessControlParameters", HFILL }},
    { &hf_rrc_pagingResponseRestrictionIndicator,
      { "pagingResponseRestrictionIndicator", "rrc.pagingResponseRestrictionIndicator",
        FT_UINT32, BASE_DEC, VALS(rrc_T_pagingResponseRestrictionIndicator_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_locationRegistrationRestrictionIndicator,
      { "locationRegistrationRestrictionIndicator", "rrc.locationRegistrationRestrictionIndicator",
        FT_UINT32, BASE_DEC, VALS(rrc_T_locationRegistrationRestrictionIndicator_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_locationRegistration,
      { "locationRegistration", "rrc.locationRegistration",
        FT_UINT32, BASE_DEC, VALS(rrc_LocationRegistrationParameters_vals), 0,
        "LocationRegistrationParameters", HFILL }},
    { &hf_rrc_mcc,
      { "mcc", "rrc.mcc",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_mnc,
      { "mnc", "rrc.mnc",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_gsm_MAP_01,
      { "gsm-MAP", "rrc.gsm_MAP",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_ansi_41_01,
      { "ansi-41", "rrc.ansi_41",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_p_REV,
      { "p-REV", "rrc.p_REV",
        FT_BYTES, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_min_P_REV,
      { "min-P-REV", "rrc.min_P_REV",
        FT_BYTES, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_sid,
      { "sid", "rrc.sid",
        FT_BYTES, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_nid,
      { "nid", "rrc.nid",
        FT_BYTES, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_gsm_MAP_and_ANSI_41,
      { "gsm-MAP-and-ANSI-41", "rrc.gsm_MAP_and_ANSI_41",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_gsm_MAP_RAB_Identity,
      { "gsm-MAP-RAB-Identity", "rrc.gsm_MAP_RAB_Identity",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_8", HFILL }},
    { &hf_rrc_ansi_41_RAB_Identity,
      { "ansi-41-RAB-Identity", "rrc.ansi_41_RAB_Identity",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_8", HFILL }},
    { &hf_rrc_lai,
      { "lai", "rrc.lai",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_rac,
      { "rac", "rrc.rac",
        FT_BYTES, BASE_NONE, NULL, 0,
        "RoutingAreaCode", HFILL }},
    { &hf_rrc_AccessClassBarredList_item,
      { "AccessClassBarred", "rrc.AccessClassBarred",
        FT_UINT32, BASE_DEC, VALS(rrc_AccessClassBarred_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_cellBarred,
      { "cellBarred", "rrc.cellBarred",
        FT_UINT32, BASE_DEC, VALS(rrc_CellBarred_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_cellReservedForOperatorUse,
      { "cellReservedForOperatorUse", "rrc.cellReservedForOperatorUse",
        FT_UINT32, BASE_DEC, VALS(rrc_ReservedIndicator_vals), 0,
        "ReservedIndicator", HFILL }},
    { &hf_rrc_cellReservationExtension,
      { "cellReservationExtension", "rrc.cellReservationExtension",
        FT_UINT32, BASE_DEC, VALS(rrc_ReservedIndicator_vals), 0,
        "ReservedIndicator", HFILL }},
    { &hf_rrc_accessClassBarredList,
      { "accessClassBarredList", "rrc.accessClassBarredList",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_cellReservedForCSG,
      { "cellReservedForCSG", "rrc.cellReservedForCSG",
        FT_UINT32, BASE_DEC, VALS(rrc_T_cellReservedForCSG_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_barred,
      { "barred", "rrc.barred",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_intraFreqCellReselectionInd,
      { "intraFreqCellReselectionInd", "rrc.intraFreqCellReselectionInd",
        FT_UINT32, BASE_DEC, VALS(rrc_AllowedIndicator_vals), 0,
        "AllowedIndicator", HFILL }},
    { &hf_rrc_t_Barred,
      { "t-Barred", "rrc.t_Barred",
        FT_UINT32, BASE_DEC, VALS(rrc_T_Barred_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_notBarred,
      { "notBarred", "rrc.notBarred",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_CellIdentity_PerRL_List_item,
      { "CellIdentity", "rrc.CellIdentity",
        FT_BYTES, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_cell_Identity,
      { "cell-Identity", "rrc.cell_Identity",
        FT_BYTES, BASE_NONE, NULL, 0,
        "CellIdentity", HFILL }},
    { &hf_rrc_mappingInfo,
      { "mappingInfo", "rrc.mappingInfo",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_cellSelectQualityMeasure,
      { "cellSelectQualityMeasure", "rrc.cellSelectQualityMeasure",
        FT_UINT32, BASE_DEC, VALS(rrc_T_cellSelectQualityMeasure_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_cpich_Ec_N0,
      { "cpich-Ec-N0", "rrc.cpich_Ec_N0",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_q_HYST_2_S,
      { "q-HYST-2-S", "rrc.q_HYST_2_S",
        FT_UINT32, BASE_DEC, NULL, 0,
        "Q_Hyst_S", HFILL }},
    { &hf_rrc_cpich_RSCP,
      { "cpich-RSCP", "rrc.cpich_RSCP",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_modeSpecificInfo_16,
      { "modeSpecificInfo", "rrc.modeSpecificInfo",
        FT_UINT32, BASE_DEC, VALS(rrc_T_modeSpecificInfo_16_vals), 0,
        "T_modeSpecificInfo_16", HFILL }},
    { &hf_rrc_fdd_46,
      { "fdd", "rrc.fdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_fdd_45", HFILL }},
    { &hf_rrc_s_Intrasearch,
      { "s-Intrasearch", "rrc.s_Intrasearch",
        FT_INT32, BASE_DEC, NULL, 0,
        "S_SearchQual", HFILL }},
    { &hf_rrc_s_Intersearch,
      { "s-Intersearch", "rrc.s_Intersearch",
        FT_INT32, BASE_DEC, NULL, 0,
        "S_SearchQual", HFILL }},
    { &hf_rrc_s_SearchHCS,
      { "s-SearchHCS", "rrc.s_SearchHCS",
        FT_INT32, BASE_DEC, NULL, 0,
        "S_SearchRXLEV", HFILL }},
    { &hf_rrc_rat_List,
      { "rat-List", "rrc.rat_List",
        FT_UINT32, BASE_DEC, NULL, 0,
        "RAT_FDD_InfoList", HFILL }},
    { &hf_rrc_q_QualMin,
      { "q-QualMin", "rrc.q_QualMin",
        FT_INT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_q_RxlevMin,
      { "q-RxlevMin", "rrc.q_RxlevMin",
        FT_INT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_tdd_11,
      { "tdd", "rrc.tdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_tdd_10", HFILL }},
    { &hf_rrc_s_Intrasearch_01,
      { "s-Intrasearch", "rrc.s_Intrasearch",
        FT_INT32, BASE_DEC, NULL, 0,
        "S_SearchRXLEV", HFILL }},
    { &hf_rrc_s_Intersearch_01,
      { "s-Intersearch", "rrc.s_Intersearch",
        FT_INT32, BASE_DEC, NULL, 0,
        "S_SearchRXLEV", HFILL }},
    { &hf_rrc_rat_List_01,
      { "rat-List", "rrc.rat_List",
        FT_UINT32, BASE_DEC, NULL, 0,
        "RAT_TDD_InfoList", HFILL }},
    { &hf_rrc_q_Hyst_l_S,
      { "q-Hyst-l-S", "rrc.q_Hyst_l_S",
        FT_UINT32, BASE_DEC, NULL, 0,
        "Q_Hyst_S", HFILL }},
    { &hf_rrc_t_Reselection_S,
      { "t-Reselection-S", "rrc.t_Reselection_S",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_hcs_ServingCellInformation,
      { "hcs-ServingCellInformation", "rrc.hcs_ServingCellInformation",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_CSG_DedicatedFrequencyInfoList_item,
      { "FrequencyInfo", "rrc.FrequencyInfo",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_startPSC,
      { "startPSC", "rrc.startPSC",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_63", HFILL }},
    { &hf_rrc_numberOfPSCs,
      { "numberOfPSCs", "rrc.numberOfPSCs",
        FT_UINT32, BASE_DEC, VALS(rrc_T_numberOfPSCs_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_pscRange2Offset,
      { "pscRange2Offset", "rrc.pscRange2Offset",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_1_63", HFILL }},
    { &hf_rrc_domainSpecificAccessRestictionList,
      { "domainSpecificAccessRestictionList", "rrc.domainSpecificAccessRestictionList",
        FT_NONE, BASE_NONE, NULL, 0,
        "DomainSpecificAccessRestrictionList_v670ext", HFILL }},
    { &hf_rrc_domainSpecificAccessRestictionParametersForAll,
      { "domainSpecificAccessRestictionParametersForAll", "rrc.domainSpecificAccessRestictionParametersForAll",
        FT_NONE, BASE_NONE, NULL, 0,
        "DomainSpecificAccessRestrictionParam_v670ext", HFILL }},
    { &hf_rrc_domainSpecificAccessRestrictionParametersForOperator1,
      { "domainSpecificAccessRestrictionParametersForOperator1", "rrc.domainSpecificAccessRestrictionParametersForOperator1",
        FT_NONE, BASE_NONE, NULL, 0,
        "DomainSpecificAccessRestrictionParam_v670ext", HFILL }},
    { &hf_rrc_domainSpecificAccessRestrictionParametersForOperator2,
      { "domainSpecificAccessRestrictionParametersForOperator2", "rrc.domainSpecificAccessRestrictionParametersForOperator2",
        FT_NONE, BASE_NONE, NULL, 0,
        "DomainSpecificAccessRestrictionParam_v670ext", HFILL }},
    { &hf_rrc_domainSpecificAccessRestrictionParametersForOperator3,
      { "domainSpecificAccessRestrictionParametersForOperator3", "rrc.domainSpecificAccessRestrictionParametersForOperator3",
        FT_NONE, BASE_NONE, NULL, 0,
        "DomainSpecificAccessRestrictionParam_v670ext", HFILL }},
    { &hf_rrc_domainSpecificAccessRestrictionParametersForOperator4,
      { "domainSpecificAccessRestrictionParametersForOperator4", "rrc.domainSpecificAccessRestrictionParametersForOperator4",
        FT_NONE, BASE_NONE, NULL, 0,
        "DomainSpecificAccessRestrictionParam_v670ext", HFILL }},
    { &hf_rrc_domainSpecificAccessRestrictionParametersForOperator5,
      { "domainSpecificAccessRestrictionParametersForOperator5", "rrc.domainSpecificAccessRestrictionParametersForOperator5",
        FT_NONE, BASE_NONE, NULL, 0,
        "DomainSpecificAccessRestrictionParam_v670ext", HFILL }},
    { &hf_rrc_cSDomainSpecificAccessRestriction,
      { "cSDomainSpecificAccessRestriction", "rrc.cSDomainSpecificAccessRestriction",
        FT_UINT32, BASE_DEC, VALS(rrc_DomainSpecificAccessRestriction_v670ext_vals), 0,
        "DomainSpecificAccessRestriction_v670ext", HFILL }},
    { &hf_rrc_pSDomainSpecificAccessRestriction,
      { "pSDomainSpecificAccessRestriction", "rrc.pSDomainSpecificAccessRestriction",
        FT_UINT32, BASE_DEC, VALS(rrc_DomainSpecificAccessRestriction_v670ext_vals), 0,
        "DomainSpecificAccessRestriction_v670ext", HFILL }},
    { &hf_rrc_restriction_01,
      { "restriction", "rrc.restriction",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_domainSpecficAccessClassBarredList,
      { "domainSpecficAccessClassBarredList", "rrc.domainSpecficAccessClassBarredList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "AccessClassBarredList", HFILL }},
    { &hf_rrc_rat,
      { "rat", "rrc.rat",
        FT_UINT32, BASE_DEC, VALS(rrc_RAT_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_mappingFunctionParameterList,
      { "mappingFunctionParameterList", "rrc.mappingFunctionParameterList",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_functionType,
      { "functionType", "rrc.functionType",
        FT_UINT32, BASE_DEC, VALS(rrc_MappingFunctionType_vals), 0,
        "MappingFunctionType", HFILL }},
    { &hf_rrc_mapParameter1,
      { "mapParameter1", "rrc.mapParameter1",
        FT_UINT32, BASE_DEC, NULL, 0,
        "MapParameter", HFILL }},
    { &hf_rrc_mapParameter2,
      { "mapParameter2", "rrc.mapParameter2",
        FT_UINT32, BASE_DEC, NULL, 0,
        "MapParameter", HFILL }},
    { &hf_rrc_upperLimit,
      { "upperLimit", "rrc.upperLimit",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_MappingFunctionParameterList_item,
      { "MappingFunctionParameter", "rrc.MappingFunctionParameter",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_MappingInfo_item,
      { "Mapping", "rrc.Mapping",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_action,
      { "action", "rrc.action",
        FT_UINT32, BASE_DEC, VALS(rrc_T_action_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_clearDedicatedPriorities,
      { "clearDedicatedPriorities", "rrc.clearDedicatedPriorities",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_configureDedicatedPriorities,
      { "configureDedicatedPriorities", "rrc.configureDedicatedPriorities",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_t_322,
      { "t-322", "rrc.t_322",
        FT_UINT32, BASE_DEC, VALS(rrc_T_322_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_priorityLevelList,
      { "priorityLevelList", "rrc.priorityLevelList",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_eutraDetection,
      { "eutraDetection", "rrc.eutraDetection",
        FT_BOOLEAN, BASE_NONE, NULL, 0,
        "BOOLEAN", HFILL }},
    { &hf_rrc_priority,
      { "priority", "rrc.priority",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_maxPrio_1", HFILL }},
    { &hf_rrc_radioAccessTechnology,
      { "radioAccessTechnology", "rrc.radioAccessTechnology",
        FT_UINT32, BASE_DEC, VALS(rrc_T_radioAccessTechnology_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_utraFDD,
      { "utraFDD", "rrc.utraFDD",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_utraFDD_item,
      { "utraFDD item", "rrc.utraFDD_item",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_uarfcn,
      { "uarfcn", "rrc.uarfcn",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_utraTDD,
      { "utraTDD", "rrc.utraTDD",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_utraTDD_item,
      { "utraTDD item", "rrc.utraTDD_item",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_eutra,
      { "eutra", "rrc.eutra",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_eutra_item,
      { "eutra item", "rrc.eutra_item",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_earfcn,
      { "earfcn", "rrc.earfcn",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_gsm_01,
      { "gsm", "rrc.gsm",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_gsm_01", HFILL }},
    { &hf_rrc_gsm_CellGroup,
      { "gsm-CellGroup", "rrc.gsm_CellGroup",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_PriorityLevelList_item,
      { "PriorityLevel", "rrc.PriorityLevel",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_rat_Identifier,
      { "rat-Identifier", "rrc.rat_Identifier",
        FT_UINT32, BASE_DEC, VALS(rrc_RAT_Identifier_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_s_SearchRAT,
      { "s-SearchRAT", "rrc.s_SearchRAT",
        FT_INT32, BASE_DEC, NULL, 0,
        "S_SearchQual", HFILL }},
    { &hf_rrc_s_HCS_RAT,
      { "s-HCS-RAT", "rrc.s_HCS_RAT",
        FT_INT32, BASE_DEC, NULL, 0,
        "S_SearchRXLEV", HFILL }},
    { &hf_rrc_s_Limit_SearchRAT,
      { "s-Limit-SearchRAT", "rrc.s_Limit_SearchRAT",
        FT_INT32, BASE_DEC, NULL, 0,
        "S_SearchQual", HFILL }},
    { &hf_rrc_RAT_FDD_InfoList_item,
      { "RAT-FDD-Info", "rrc.RAT_FDD_Info",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_s_SearchRAT_01,
      { "s-SearchRAT", "rrc.s_SearchRAT",
        FT_INT32, BASE_DEC, NULL, 0,
        "S_SearchRXLEV", HFILL }},
    { &hf_rrc_s_Limit_SearchRAT_01,
      { "s-Limit-SearchRAT", "rrc.s_Limit_SearchRAT",
        FT_INT32, BASE_DEC, NULL, 0,
        "S_SearchRXLEV", HFILL }},
    { &hf_rrc_RAT_TDD_InfoList_item,
      { "RAT-TDD-Info", "rrc.RAT_TDD_Info",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_URA_IdentityList_item,
      { "URA-Identity", "rrc.URA_Identity",
        FT_BYTES, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_n_AP_RetransMax,
      { "n-AP-RetransMax", "rrc.n_AP_RetransMax",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_n_AccessFails,
      { "n-AccessFails", "rrc.n_AccessFails",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_nf_BO_NoAICH,
      { "nf-BO-NoAICH", "rrc.nf_BO_NoAICH",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_ns_BO_Busy,
      { "ns-BO-Busy", "rrc.ns_BO_Busy",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_nf_BO_AllBusy,
      { "nf-BO-AllBusy", "rrc.nf_BO_AllBusy",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_nf_BO_Mismatch,
      { "nf-BO-Mismatch", "rrc.nf_BO_Mismatch",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_t_CPCH,
      { "t-CPCH", "rrc.t_CPCH",
        FT_UINT32, BASE_DEC, VALS(rrc_T_CPCH_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_ue_RadioCapabilityFDDUpdateRequirement,
      { "ue-RadioCapabilityFDDUpdateRequirement", "rrc.ue_RadioCapabilityFDDUpdateRequirement",
        FT_BOOLEAN, BASE_NONE, NULL, 0,
        "BOOLEAN", HFILL }},
    { &hf_rrc_ue_RadioCapabilityTDDUpdateRequirement,
      { "ue-RadioCapabilityTDDUpdateRequirement", "rrc.ue_RadioCapabilityTDDUpdateRequirement",
        FT_BOOLEAN, BASE_NONE, NULL, 0,
        "BOOLEAN", HFILL }},
    { &hf_rrc_systemSpecificCapUpdateReqList,
      { "systemSpecificCapUpdateReqList", "rrc.systemSpecificCapUpdateReqList",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_ue_RadioCapabilityUpdateRequirement_TDD128,
      { "ue-RadioCapabilityUpdateRequirement-TDD128", "rrc.ue_RadioCapabilityUpdateRequirement_TDD128",
        FT_BOOLEAN, BASE_NONE, NULL, 0,
        "BOOLEAN", HFILL }},
    { &hf_rrc_ue_RadioCapabilityFDDUpdateRequirement_FDD,
      { "ue-RadioCapabilityFDDUpdateRequirement-FDD", "rrc.ue_RadioCapabilityFDDUpdateRequirement_FDD",
        FT_BOOLEAN, BASE_NONE, NULL, 0,
        "BOOLEAN", HFILL }},
    { &hf_rrc_ue_RadioCapabilityTDDUpdateRequirement_TDD384,
      { "ue-RadioCapabilityTDDUpdateRequirement-TDD384", "rrc.ue_RadioCapabilityTDDUpdateRequirement_TDD384",
        FT_BOOLEAN, BASE_NONE, NULL, 0,
        "BOOLEAN", HFILL }},
    { &hf_rrc_ue_RadioCapabilityTDDUpdateRequirement_TDD128,
      { "ue-RadioCapabilityTDDUpdateRequirement-TDD128", "rrc.ue_RadioCapabilityTDDUpdateRequirement_TDD128",
        FT_BOOLEAN, BASE_NONE, NULL, 0,
        "BOOLEAN", HFILL }},
    { &hf_rrc_systemSpecificCapUpdateReqList_01,
      { "systemSpecificCapUpdateReqList", "rrc.systemSpecificCapUpdateReqList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "SystemSpecificCapUpdateReqList_r5", HFILL }},
    { &hf_rrc_ue_RadioCapabilityTDDUpdateRequirement_TDD768,
      { "ue-RadioCapabilityTDDUpdateRequirement-TDD768", "rrc.ue_RadioCapabilityTDDUpdateRequirement_TDD768",
        FT_BOOLEAN, BASE_NONE, NULL, 0,
        "BOOLEAN", HFILL }},
    { &hf_rrc_systemSpecificCapUpdateReqList_02,
      { "systemSpecificCapUpdateReqList", "rrc.systemSpecificCapUpdateReqList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "SystemSpecificCapUpdateReqList_r8", HFILL }},
    { &hf_rrc_startRestart,
      { "startRestart", "rrc.startRestart",
        FT_UINT32, BASE_DEC, VALS(rrc_CipheringAlgorithm_vals), 0,
        "CipheringAlgorithm", HFILL }},
    { &hf_rrc_startRestart_01,
      { "startRestart", "rrc.startRestart",
        FT_UINT32, BASE_DEC, VALS(rrc_CipheringAlgorithm_r7_vals), 0,
        "CipheringAlgorithm_r7", HFILL }},
    { &hf_rrc_cipheringModeCommand,
      { "cipheringModeCommand", "rrc.cipheringModeCommand",
        FT_UINT32, BASE_DEC, VALS(rrc_CipheringModeCommand_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_activationTimeForDPCH,
      { "activationTimeForDPCH", "rrc.activationTimeForDPCH",
        FT_UINT32, BASE_DEC, NULL, 0,
        "ActivationTime", HFILL }},
    { &hf_rrc_rb_DL_CiphActivationTimeInfo,
      { "rb-DL-CiphActivationTimeInfo", "rrc.rb_DL_CiphActivationTimeInfo",
        FT_UINT32, BASE_DEC, NULL, 0,
        "RB_ActivationTimeInfoList", HFILL }},
    { &hf_rrc_cipheringModeCommand_01,
      { "cipheringModeCommand", "rrc.cipheringModeCommand",
        FT_NONE, BASE_NONE, NULL, 0,
        "CipheringModeCommand_r7", HFILL }},
    { &hf_rrc_imsi_GSM_MAP,
      { "imsi-GSM-MAP", "rrc.imsi_GSM_MAP",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_tmsi_GSM_MAP,
      { "tmsi-GSM-MAP", "rrc.tmsi_GSM_MAP",
        FT_BYTES, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_p_TMSI_GSM_MAP,
      { "p-TMSI-GSM-MAP", "rrc.p_TMSI_GSM_MAP",
        FT_BYTES, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_imsi_DS_41,
      { "imsi-DS-41", "rrc.imsi_DS_41",
        FT_BYTES, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_tmsi_DS_41,
      { "tmsi-DS-41", "rrc.tmsi_DS_41",
        FT_BYTES, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_fdd_Measurements,
      { "fdd-Measurements", "rrc.fdd_Measurements",
        FT_BOOLEAN, BASE_NONE, NULL, 0,
        "BOOLEAN", HFILL }},
    { &hf_rrc_tdd_Measurements,
      { "tdd-Measurements", "rrc.tdd_Measurements",
        FT_BOOLEAN, BASE_NONE, NULL, 0,
        "BOOLEAN", HFILL }},
    { &hf_rrc_gsm_Measurements,
      { "gsm-Measurements", "rrc.gsm_Measurements",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_multiCarrierMeasurements,
      { "multiCarrierMeasurements", "rrc.multiCarrierMeasurements",
        FT_BOOLEAN, BASE_NONE, NULL, 0,
        "BOOLEAN", HFILL }},
    { &hf_rrc_tdd128_Measurements,
      { "tdd128-Measurements", "rrc.tdd128_Measurements",
        FT_BOOLEAN, BASE_NONE, NULL, 0,
        "BOOLEAN", HFILL }},
    { &hf_rrc_adjacentFrequencyMeasurements,
      { "adjacentFrequencyMeasurements", "rrc.adjacentFrequencyMeasurements",
        FT_UINT32, BASE_DEC, VALS(rrc_T_adjacentFrequencyMeasurements_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_interBandMeasurements,
      { "interBandMeasurements", "rrc.interBandMeasurements",
        FT_UINT32, BASE_DEC, VALS(rrc_T_interBandMeasurements_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_enhancedInterFrequencyMeasurements,
      { "enhancedInterFrequencyMeasurements", "rrc.enhancedInterFrequencyMeasurements",
        FT_UINT32, BASE_DEC, VALS(rrc_T_enhancedInterFrequencyMeasurements_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_freqSpecificCompressedMode,
      { "freqSpecificCompressedMode", "rrc.freqSpecificCompressedMode",
        FT_UINT32, BASE_DEC, VALS(rrc_T_freqSpecificCompressedMode_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_radioFrequencyBandEUTRA,
      { "radioFrequencyBandEUTRA", "rrc.radioFrequencyBandEUTRA",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_compressedMode,
      { "compressedMode", "rrc.compressedMode",
        FT_BOOLEAN, BASE_NONE, NULL, 0,
        "BOOLEAN", HFILL }},
    { &hf_rrc_CompressedModeMeasCapabEUTRAList_item,
      { "CompressedModeMeasCapabEUTRA", "rrc.CompressedModeMeasCapabEUTRA",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_CompressedModeMeasCapabFDDList_item,
      { "CompressedModeMeasCapabFDD", "rrc.CompressedModeMeasCapabFDD",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_CompressedModeMeasCapabFDDList2_item,
      { "CompressedModeMeasCapabFDD2", "rrc.CompressedModeMeasCapabFDD2",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_CompressedModeMeasCapabFDDList3_item,
      { "CompressedModeMeasCapabFDD3", "rrc.CompressedModeMeasCapabFDD3",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_CompressedModeMeasCapabFDDList_ext_item,
      { "CompressedModeMeasCapabFDD-ext", "rrc.CompressedModeMeasCapabFDD_ext",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_radioFrequencyBandFDD,
      { "radioFrequencyBandFDD", "rrc.radioFrequencyBandFDD",
        FT_UINT32, BASE_DEC, VALS(rrc_RadioFrequencyBandFDD_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_dl_MeasurementsFDD,
      { "dl-MeasurementsFDD", "rrc.dl_MeasurementsFDD",
        FT_BOOLEAN, BASE_NONE, NULL, 0,
        "BOOLEAN", HFILL }},
    { &hf_rrc_ul_MeasurementsFDD,
      { "ul-MeasurementsFDD", "rrc.ul_MeasurementsFDD",
        FT_BOOLEAN, BASE_NONE, NULL, 0,
        "BOOLEAN", HFILL }},
    { &hf_rrc_radioFrequencyBandFDD2,
      { "radioFrequencyBandFDD2", "rrc.radioFrequencyBandFDD2",
        FT_UINT32, BASE_DEC, VALS(rrc_RadioFrequencyBandFDD2_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_radioFrequencyBandFDD3,
      { "radioFrequencyBandFDD3", "rrc.radioFrequencyBandFDD3",
        FT_UINT32, BASE_DEC|BASE_EXT_STRING, &rrc_RadioFrequencyBandFDD3_vals_ext, 0,
        NULL, HFILL }},
    { &hf_rrc_CompressedModeMeasCapabTDDList_item,
      { "CompressedModeMeasCapabTDD", "rrc.CompressedModeMeasCapabTDD",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_radioFrequencyBandTDD,
      { "radioFrequencyBandTDD", "rrc.radioFrequencyBandTDD",
        FT_UINT32, BASE_DEC, VALS(rrc_RadioFrequencyBandTDD_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_dl_MeasurementsTDD,
      { "dl-MeasurementsTDD", "rrc.dl_MeasurementsTDD",
        FT_BOOLEAN, BASE_NONE, NULL, 0,
        "BOOLEAN", HFILL }},
    { &hf_rrc_ul_MeasurementsTDD,
      { "ul-MeasurementsTDD", "rrc.ul_MeasurementsTDD",
        FT_BOOLEAN, BASE_NONE, NULL, 0,
        "BOOLEAN", HFILL }},
    { &hf_rrc_CompressedModeMeasCapabGSMList_item,
      { "CompressedModeMeasCapabGSM", "rrc.CompressedModeMeasCapabGSM",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_radioFrequencyBandGSM,
      { "radioFrequencyBandGSM", "rrc.radioFrequencyBandGSM",
        FT_UINT32, BASE_DEC, VALS(rrc_RadioFrequencyBandGSM_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_dl_MeasurementsGSM,
      { "dl-MeasurementsGSM", "rrc.dl_MeasurementsGSM",
        FT_BOOLEAN, BASE_NONE, NULL, 0,
        "BOOLEAN", HFILL }},
    { &hf_rrc_ul_MeasurementsGSM,
      { "ul-MeasurementsGSM", "rrc.ul_MeasurementsGSM",
        FT_BOOLEAN, BASE_NONE, NULL, 0,
        "BOOLEAN", HFILL }},
    { &hf_rrc_dl_MeasurementsMC,
      { "dl-MeasurementsMC", "rrc.dl_MeasurementsMC",
        FT_BOOLEAN, BASE_NONE, NULL, 0,
        "BOOLEAN", HFILL }},
    { &hf_rrc_ul_MeasurementsMC,
      { "ul-MeasurementsMC", "rrc.ul_MeasurementsMC",
        FT_BOOLEAN, BASE_NONE, NULL, 0,
        "BOOLEAN", HFILL }},
    { &hf_rrc_initialPriorityDelayList,
      { "initialPriorityDelayList", "rrc.initialPriorityDelayList",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_backoffControlParams,
      { "backoffControlParams", "rrc.backoffControlParams",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_powerControlAlgorithm,
      { "powerControlAlgorithm", "rrc.powerControlAlgorithm",
        FT_UINT32, BASE_DEC, VALS(rrc_PowerControlAlgorithm_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_dl_DPCCH_BER,
      { "dl-DPCCH-BER", "rrc.dl_DPCCH_BER",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_supportOfIntraFreqProximityIndication,
      { "supportOfIntraFreqProximityIndication", "rrc.supportOfIntraFreqProximityIndication",
        FT_UINT32, BASE_DEC, VALS(rrc_T_supportOfIntraFreqProximityIndication_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_supportOfInterFreqProximityIndication,
      { "supportOfInterFreqProximityIndication", "rrc.supportOfInterFreqProximityIndication",
        FT_UINT32, BASE_DEC, VALS(rrc_T_supportOfInterFreqProximityIndication_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_supportOfE_UtraProximityIndication,
      { "supportOfE-UtraProximityIndication", "rrc.supportOfE_UtraProximityIndication",
        FT_UINT32, BASE_DEC, VALS(rrc_T_supportOfE_UtraProximityIndication_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_DL_InformationPerSecondaryRL_List_item,
      { "DL-InformationPerSecondaryRL", "rrc.DL_InformationPerSecondaryRL",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_primaryCPICH_Info,
      { "primaryCPICH-Info", "rrc.primaryCPICH_Info",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_cell_id,
      { "cell-id", "rrc.cell_id",
        FT_BYTES, BASE_NONE, NULL, 0,
        "CellIdentity", HFILL }},
    { &hf_rrc_dl_FDPCHInfoPerRL_SecULFreq,
      { "dl-FDPCHInfoPerRL-SecULFreq", "rrc.dl_FDPCHInfoPerRL_SecULFreq",
        FT_NONE, BASE_NONE, NULL, 0,
        "DL_FDPCH_InfoPerRL_r7", HFILL }},
    { &hf_rrc_e_AGCH_Information,
      { "e-AGCH-Information", "rrc.e_AGCH_Information",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_e_HICH_Information,
      { "e-HICH-Information", "rrc.e_HICH_Information",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_e_RGCH_Info,
      { "e-RGCH-Info", "rrc.e_RGCH_Info",
        FT_UINT32, BASE_DEC, VALS(rrc_T_e_RGCH_Info_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_e_RGCH_Information,
      { "e-RGCH-Information", "rrc.e_RGCH_Information",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_releaseIndicator,
      { "releaseIndicator", "rrc.releaseIndicator",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_maxNoDPCH_PDSCH_Codes,
      { "maxNoDPCH-PDSCH-Codes", "rrc.maxNoDPCH_PDSCH_Codes",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_1_8", HFILL }},
    { &hf_rrc_maxNoPhysChBitsReceived,
      { "maxNoPhysChBitsReceived", "rrc.maxNoPhysChBitsReceived",
        FT_UINT32, BASE_DEC, VALS(rrc_MaxNoPhysChBitsReceived_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_supportForSF_512,
      { "supportForSF-512", "rrc.supportForSF_512",
        FT_BOOLEAN, BASE_NONE, NULL, 0,
        "BOOLEAN", HFILL }},
    { &hf_rrc_dummy2_05,
      { "dummy2", "rrc.dummy2",
        FT_UINT32, BASE_DEC, VALS(rrc_SimultaneousSCCPCH_DPCH_Reception_vals), 0,
        "SimultaneousSCCPCH_DPCH_Reception", HFILL }},
    { &hf_rrc_dummy_29,
      { "dummy", "rrc.dummy",
        FT_UINT32, BASE_DEC, VALS(rrc_SupportOfDedicatedPilotsForChEstimation_vals), 0,
        "SupportOfDedicatedPilotsForChEstimation", HFILL }},
    { &hf_rrc_hsdsch_physical_layer_category_ext,
      { "hsdsch-physical-layer-category-ext", "rrc.hsdsch_physical_layer_category_ext",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_hsscchlessHsdschOperation,
      { "hsscchlessHsdschOperation", "rrc.hsscchlessHsdschOperation",
        FT_UINT32, BASE_DEC, VALS(rrc_T_hsscchlessHsdschOperation_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_enhancedFdpch,
      { "enhancedFdpch", "rrc.enhancedFdpch",
        FT_UINT32, BASE_DEC, VALS(rrc_T_enhancedFdpch_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_hsdschReception_CellFach,
      { "hsdschReception-CellFach", "rrc.hsdschReception_CellFach",
        FT_UINT32, BASE_DEC, VALS(rrc_T_hsdschReception_CellFach_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_hsdschReception_CellUraPch,
      { "hsdschReception-CellUraPch", "rrc.hsdschReception_CellUraPch",
        FT_UINT32, BASE_DEC, VALS(rrc_T_hsdschReception_CellUraPch_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_hsdsch_physical_layer_category_ext2,
      { "hsdsch-physical-layer-category-ext2", "rrc.hsdsch_physical_layer_category_ext2",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_supportOfTargetCellPreConfig,
      { "supportOfTargetCellPreConfig", "rrc.supportOfTargetCellPreConfig",
        FT_UINT32, BASE_DEC, VALS(rrc_T_supportOfTargetCellPreConfig_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_supportOfHsdschDrxOperation,
      { "supportOfHsdschDrxOperation", "rrc.supportOfHsdschDrxOperation",
        FT_UINT32, BASE_DEC, VALS(rrc_T_supportOfHsdschDrxOperation_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_hsdsch_physical_layer_category_ext3,
      { "hsdsch-physical-layer-category-ext3", "rrc.hsdsch_physical_layer_category_ext3",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_supportOfMimoOnlySingleStream,
      { "supportOfMimoOnlySingleStream", "rrc.supportOfMimoOnlySingleStream",
        FT_UINT32, BASE_DEC, VALS(rrc_T_supportOfMimoOnlySingleStream_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_hsdsch_physical_layer_category_ext4,
      { "hsdsch-physical-layer-category-ext4", "rrc.hsdsch_physical_layer_category_ext4",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_hsdsch_physical_layer_category_ext5,
      { "hsdsch-physical-layer-category-ext5", "rrc.hsdsch_physical_layer_category_ext5",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_hsdsch_physical_layer_category_ext6,
      { "hsdsch-physical-layer-category-ext6", "rrc.hsdsch_physical_layer_category_ext6",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_hsdsch_physical_layer_category_ext7,
      { "hsdsch-physical-layer-category-ext7", "rrc.hsdsch_physical_layer_category_ext7",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_hsdsch_physical_layer_category_ext8,
      { "hsdsch-physical-layer-category-ext8", "rrc.hsdsch_physical_layer_category_ext8",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_supportOf4x4MimoDualStreamRestriction,
      { "supportOf4x4MimoDualStreamRestriction", "rrc.supportOf4x4MimoDualStreamRestriction",
        FT_UINT32, BASE_DEC, VALS(rrc_T_supportOf4x4MimoDualStreamRestriction_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_supportOfNodeBTrigHS_DPCCHTransmission,
      { "supportOfNodeBTrigHS-DPCCHTransmission", "rrc.supportOfNodeBTrigHS_DPCCHTransmission",
        FT_UINT32, BASE_DEC, VALS(rrc_T_supportOfNodeBTrigHS_DPCCHTransmission_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_supportOfHS_DSCHSecondDRXOperation,
      { "supportOfHS-DSCHSecondDRXOperation", "rrc.supportOfHS_DSCHSecondDRXOperation",
        FT_UINT32, BASE_DEC, VALS(rrc_T_supportOfHS_DSCHSecondDRXOperation_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_maxTS_PerFrame,
      { "maxTS-PerFrame", "rrc.maxTS_PerFrame",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_maxPhysChPerFrame,
      { "maxPhysChPerFrame", "rrc.maxPhysChPerFrame",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_minimumSF,
      { "minimumSF", "rrc.minimumSF",
        FT_UINT32, BASE_DEC, VALS(rrc_MinimumSF_DL_vals), 0,
        "MinimumSF_DL", HFILL }},
    { &hf_rrc_supportOfPDSCH,
      { "supportOfPDSCH", "rrc.supportOfPDSCH",
        FT_BOOLEAN, BASE_NONE, NULL, 0,
        "BOOLEAN", HFILL }},
    { &hf_rrc_maxPhysChPerTS,
      { "maxPhysChPerTS", "rrc.maxPhysChPerTS",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_maxTS_PerSubFrame,
      { "maxTS-PerSubFrame", "rrc.maxTS_PerSubFrame",
        FT_UINT32, BASE_DEC, NULL, 0,
        "MaxTS_PerSubFrame_r4", HFILL }},
    { &hf_rrc_maxPhysChPerFrame_01,
      { "maxPhysChPerFrame", "rrc.maxPhysChPerFrame",
        FT_UINT32, BASE_DEC, NULL, 0,
        "MaxPhysChPerSubFrame_r4", HFILL }},
    { &hf_rrc_supportOf8PSK,
      { "supportOf8PSK", "rrc.supportOf8PSK",
        FT_BOOLEAN, BASE_NONE, NULL, 0,
        "BOOLEAN", HFILL }},
    { &hf_rrc_multiCarrier_physical_layer_category,
      { "multiCarrier-physical-layer-category", "rrc.multiCarrier_physical_layer_category",
        FT_UINT32, BASE_DEC, NULL, 0,
        "MultiCarrier_HSDSCH_physical_layer_category", HFILL }},
    { &hf_rrc_hSDSCH_physical_layer_category_extension,
      { "hSDSCH-physical-layer-category-extension", "rrc.hSDSCH_physical_layer_category_extension",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_multiCarrier_physical_layer_category_extension,
      { "multiCarrier-physical-layer-category-extension", "rrc.multiCarrier_physical_layer_category_extension",
        FT_UINT32, BASE_DEC, NULL, 0,
        "MultiCarrier_HSDSCH_physical_layer_category_extension", HFILL }},
    { &hf_rrc_supportOfSFModeForHSPDSCHDualStream,
      { "supportOfSFModeForHSPDSCHDualStream", "rrc.supportOfSFModeForHSPDSCHDualStream",
        FT_UINT32, BASE_DEC, VALS(rrc_T_supportOfSFModeForHSPDSCHDualStream_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_multiCarrier_physical_layer_category_extension2,
      { "multiCarrier-physical-layer-category-extension2", "rrc.multiCarrier_physical_layer_category_extension2",
        FT_UINT32, BASE_DEC, NULL, 0,
        "MultiCarrier_HSDSCH_physical_layer_category_extension2", HFILL }},
    { &hf_rrc_maxPhysChPerFrame_02,
      { "maxPhysChPerFrame", "rrc.maxPhysChPerFrame",
        FT_UINT32, BASE_DEC, NULL, 0,
        "MaxPhysChPerFrame_768", HFILL }},
    { &hf_rrc_minimumSF_01,
      { "minimumSF", "rrc.minimumSF",
        FT_UINT32, BASE_DEC, VALS(rrc_MinimumSF_DL_768_vals), 0,
        "MinimumSF_DL_768", HFILL }},
    { &hf_rrc_tdd768_hspdsch,
      { "tdd768-hspdsch", "rrc.tdd768_hspdsch",
        FT_UINT32, BASE_DEC, VALS(rrc_T_tdd768_hspdsch_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_supported,
      { "supported", "rrc.supported",
        FT_UINT32, BASE_DEC, NULL, 0,
        "HSDSCH_physical_layer_category", HFILL }},
    { &hf_rrc_unsupported,
      { "unsupported", "rrc.unsupported",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_maxPhysChPerTS_01,
      { "maxPhysChPerTS", "rrc.maxPhysChPerTS",
        FT_UINT32, BASE_DEC, NULL, 0,
        "MaxPhysChPerTS_768", HFILL }},
    { &hf_rrc_maxNoBitsReceived,
      { "maxNoBitsReceived", "rrc.maxNoBitsReceived",
        FT_UINT32, BASE_DEC, VALS(rrc_MaxNoBits_vals), 0,
        "MaxNoBits", HFILL }},
    { &hf_rrc_maxConvCodeBitsReceived,
      { "maxConvCodeBitsReceived", "rrc.maxConvCodeBitsReceived",
        FT_UINT32, BASE_DEC, VALS(rrc_MaxNoBits_vals), 0,
        "MaxNoBits", HFILL }},
    { &hf_rrc_turboDecodingSupport,
      { "turboDecodingSupport", "rrc.turboDecodingSupport",
        FT_UINT32, BASE_DEC, VALS(rrc_TurboSupport_vals), 0,
        "TurboSupport", HFILL }},
    { &hf_rrc_maxSimultaneousTransChs,
      { "maxSimultaneousTransChs", "rrc.maxSimultaneousTransChs",
        FT_UINT32, BASE_DEC, VALS(rrc_MaxSimultaneousTransChsDL_vals), 0,
        "MaxSimultaneousTransChsDL", HFILL }},
    { &hf_rrc_maxSimultaneousCCTrCH_Count,
      { "maxSimultaneousCCTrCH-Count", "rrc.maxSimultaneousCCTrCH_Count",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_maxReceivedTransportBlocks,
      { "maxReceivedTransportBlocks", "rrc.maxReceivedTransportBlocks",
        FT_UINT32, BASE_DEC, VALS(rrc_MaxTransportBlocksDL_vals), 0,
        "MaxTransportBlocksDL", HFILL }},
    { &hf_rrc_maxNumberOfTFC,
      { "maxNumberOfTFC", "rrc.maxNumberOfTFC",
        FT_UINT32, BASE_DEC, VALS(rrc_MaxNumberOfTFC_DL_vals), 0,
        "MaxNumberOfTFC_DL", HFILL }},
    { &hf_rrc_maxNumberOfTF,
      { "maxNumberOfTF", "rrc.maxNumberOfTF",
        FT_UINT32, BASE_DEC, VALS(rrc_MaxNumberOfTF_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_transmissionProbability,
      { "transmissionProbability", "rrc.transmissionProbability",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_maximumBitRate,
      { "maximumBitRate", "rrc.maximumBitRate",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_DRAC_SysInfoList_item,
      { "DRAC-SysInfo", "rrc.DRAC_SysInfo",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_configurationUnsupported,
      { "configurationUnsupported", "rrc.configurationUnsupported",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_physicalChannelFailure,
      { "physicalChannelFailure", "rrc.physicalChannelFailure",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_incompatibleSimultaneousReconfiguration,
      { "incompatibleSimultaneousReconfiguration", "rrc.incompatibleSimultaneousReconfiguration",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_compressedModeRuntimeError,
      { "compressedModeRuntimeError", "rrc.compressedModeRuntimeError",
        FT_UINT32, BASE_DEC, NULL, 0,
        "TGPSI", HFILL }},
    { &hf_rrc_protocolError,
      { "protocolError", "rrc.protocolError",
        FT_NONE, BASE_NONE, NULL, 0,
        "ProtocolErrorInformation", HFILL }},
    { &hf_rrc_cellUpdateOccurred,
      { "cellUpdateOccurred", "rrc.cellUpdateOccurred",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_invalidConfiguration,
      { "invalidConfiguration", "rrc.invalidConfiguration",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_configurationIncomplete,
      { "configurationIncomplete", "rrc.configurationIncomplete",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_unsupportedMeasurement,
      { "unsupportedMeasurement", "rrc.unsupportedMeasurement",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_mbmsSessionAlreadyReceivedCorrectly,
      { "mbmsSessionAlreadyReceivedCorrectly", "rrc.mbmsSessionAlreadyReceivedCorrectly",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_lowerPriorityMBMSService,
      { "lowerPriorityMBMSService", "rrc.lowerPriorityMBMSService",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_modeSpecificInfo_17,
      { "modeSpecificInfo", "rrc.modeSpecificInfo",
        FT_UINT32, BASE_DEC, VALS(rrc_T_modeSpecificInfo_17_vals), 0,
        "T_modeSpecificInfo_17", HFILL }},
    { &hf_rrc_fdd_47,
      { "fdd", "rrc.fdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_fdd_46", HFILL }},
    { &hf_rrc_frequencyBandsSupportFDD,
      { "frequencyBandsSupportFDD", "rrc.frequencyBandsSupportFDD",
        FT_UINT32, BASE_DEC, NULL, 0,
        "FrequencyBandsRedirectionFDD", HFILL }},
    { &hf_rrc_tdd_12,
      { "tdd", "rrc.tdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_tdd_11", HFILL }},
    { &hf_rrc_frequencyBandsSupportTDD128,
      { "frequencyBandsSupportTDD128", "rrc.frequencyBandsSupportTDD128",
        FT_UINT32, BASE_DEC, NULL, 0,
        "FrequencyBandsIndicatorTDD128", HFILL }},
    { &hf_rrc_FrequencyBandsIndicatorTDD128_item,
      { "FrequencyBandsIndicatorSupportTDD128", "rrc.FrequencyBandsIndicatorSupportTDD128",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_frequencyBandsIndicatorTDD,
      { "frequencyBandsIndicatorTDD", "rrc.frequencyBandsIndicatorTDD",
        FT_UINT32, BASE_DEC, VALS(rrc_RadioFrequencyBandTDD_r10_vals), 0,
        "RadioFrequencyBandTDD_r10", HFILL }},
    { &hf_rrc_FrequencyBandsRedirectionFDD_item,
      { "FrequencyBandsIndicatorFDD", "rrc.FrequencyBandsIndicatorFDD",
        FT_UINT32, BASE_DEC, VALS(rrc_FrequencyBandsIndicatorFDD_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_frequencyBandsIndicator1,
      { "frequencyBandsIndicator1", "rrc.frequencyBandsIndicator1",
        FT_UINT32, BASE_DEC, VALS(rrc_RadioFrequencyBandFDD_vals), 0,
        "RadioFrequencyBandFDD", HFILL }},
    { &hf_rrc_frequencyBandsIndicator2,
      { "frequencyBandsIndicator2", "rrc.frequencyBandsIndicator2",
        FT_UINT32, BASE_DEC, VALS(rrc_RadioFrequencyBandFDD2_vals), 0,
        "RadioFrequencyBandFDD2", HFILL }},
    { &hf_rrc_frequencyBandsIndicator3,
      { "frequencyBandsIndicator3", "rrc.frequencyBandsIndicator3",
        FT_UINT32, BASE_DEC|BASE_EXT_STRING, &rrc_RadioFrequencyBandFDD3_vals_ext, 0,
        "RadioFrequencyBandFDD3", HFILL }},
    { &hf_rrc_rrc_ConnectionReleaseInformation,
      { "rrc-ConnectionReleaseInformation", "rrc.rrc_ConnectionReleaseInformation",
        FT_UINT32, BASE_DEC, VALS(rrc_RRC_ConnectionReleaseInformation_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_groupReleaseInformation,
      { "groupReleaseInformation", "rrc.groupReleaseInformation",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_uRNTI_Group,
      { "uRNTI-Group", "rrc.uRNTI_Group",
        FT_UINT32, BASE_DEC|BASE_EXT_STRING, &rrc_U_RNTI_Group_vals_ext, 0,
        "U_RNTI_Group", HFILL }},
    { &hf_rrc_gsm900,
      { "gsm900", "rrc.gsm900",
        FT_BOOLEAN, BASE_NONE, NULL, 0,
        "BOOLEAN", HFILL }},
    { &hf_rrc_dcs1800,
      { "dcs1800", "rrc.dcs1800",
        FT_BOOLEAN, BASE_NONE, NULL, 0,
        "BOOLEAN", HFILL }},
    { &hf_rrc_gsm1900,
      { "gsm1900", "rrc.gsm1900",
        FT_BOOLEAN, BASE_NONE, NULL, 0,
        "BOOLEAN", HFILL }},
    { &hf_rrc_numberOfCellsBandA,
      { "numberOfCellsBandA", "rrc.numberOfCellsBandA",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_2_6", HFILL }},
    { &hf_rrc_numberOfFrequenciesBandA,
      { "numberOfFrequenciesBandA", "rrc.numberOfFrequenciesBandA",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_1_3", HFILL }},
    { &hf_rrc_numberOfCellsBandB,
      { "numberOfCellsBandB", "rrc.numberOfCellsBandB",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_2_6", HFILL }},
    { &hf_rrc_numberOfFrequenciesBandB,
      { "numberOfFrequenciesBandB", "rrc.numberOfFrequenciesBandB",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_1_3", HFILL }},
    { &hf_rrc_MultiflowBandCombList_item,
      { "MultiflowBandCombination", "rrc.MultiflowBandCombination",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_mimoInDifferentBands,
      { "mimoInDifferentBands", "rrc.mimoInDifferentBands",
        FT_UINT32, BASE_DEC, VALS(rrc_MultiflowMIMOCapability_vals), 0,
        "MultiflowMIMOCapability", HFILL }},
    { &hf_rrc_longerHARQProcessing,
      { "longerHARQProcessing", "rrc.longerHARQProcessing",
        FT_UINT32, BASE_DEC, VALS(rrc_T_longerHARQProcessing_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_numberOfCells,
      { "numberOfCells", "rrc.numberOfCells",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_2_8", HFILL }},
    { &hf_rrc_numberOfFrequencies,
      { "numberOfFrequencies", "rrc.numberOfFrequencies",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_1_4", HFILL }},
    { &hf_rrc_mimo,
      { "mimo", "rrc.mimo",
        FT_UINT32, BASE_DEC, VALS(rrc_MultiflowMIMOCapability_vals), 0,
        "MultiflowMIMOCapability", HFILL }},
    { &hf_rrc_nonContiguousCapability,
      { "nonContiguousCapability", "rrc.nonContiguousCapability",
        FT_NONE, BASE_NONE, NULL, 0,
        "MultiflowNonContiguousCapability", HFILL }},
    { &hf_rrc_gapSize,
      { "gapSize", "rrc.gapSize",
        FT_UINT32, BASE_DEC, VALS(rrc_T_gapSize_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_idleInterval,
      { "idleInterval", "rrc.idleInterval",
        FT_BOOLEAN, BASE_NONE, NULL, 0,
        "BOOLEAN", HFILL }},
    { &hf_rrc_IdleIntervalMeasCapabEUTRAList_item,
      { "IdleIntervalMeasCapabEUTRA", "rrc.IdleIntervalMeasCapabEUTRA",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_esn_DS_41,
      { "esn-DS-41", "rrc.esn_DS_41",
        FT_BYTES, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_InitialPriorityDelayList_item,
      { "NS-IP", "rrc.NS_IP",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_imsi,
      { "imsi", "rrc.imsi",
        FT_UINT32, BASE_DEC, NULL, 0,
        "IMSI_GSM_MAP", HFILL }},
    { &hf_rrc_tmsi_and_LAI,
      { "tmsi-and-LAI", "rrc.tmsi_and_LAI",
        FT_NONE, BASE_NONE, NULL, 0,
        "TMSI_and_LAI_GSM_MAP", HFILL }},
    { &hf_rrc_p_TMSI_and_RAI,
      { "p-TMSI-and-RAI", "rrc.p_TMSI_and_RAI",
        FT_NONE, BASE_NONE, NULL, 0,
        "P_TMSI_and_RAI_GSM_MAP", HFILL }},
    { &hf_rrc_imei,
      { "imei", "rrc.imei",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_imsi_and_ESN_DS_41,
      { "imsi-and-ESN-DS-41", "rrc.imsi_and_ESN_DS_41",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_messageAuthenticationCode,
      { "messageAuthenticationCode", "rrc.messageAuthenticationCode",
        FT_BYTES, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_rrc_MessageSequenceNumber,
      { "rrc-MessageSequenceNumber", "rrc.rrc_MessageSequenceNumber",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_rrc_MessageSequenceNumberList,
      { "rrc-MessageSequenceNumberList", "rrc.rrc_MessageSequenceNumberList",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_startIntegrityProtection,
      { "startIntegrityProtection", "rrc.startIntegrityProtection",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_integrityProtInitNumber,
      { "integrityProtInitNumber", "rrc.integrityProtInitNumber",
        FT_BYTES, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_modify,
      { "modify", "rrc.modify",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_dl_IntegrityProtActivationInfo,
      { "dl-IntegrityProtActivationInfo", "rrc.dl_IntegrityProtActivationInfo",
        FT_NONE, BASE_NONE, NULL, 0,
        "IntegrityProtActivationInfo", HFILL }},
    { &hf_rrc_integrityProtectionModeCommand,
      { "integrityProtectionModeCommand", "rrc.integrityProtectionModeCommand",
        FT_UINT32, BASE_DEC, VALS(rrc_IntegrityProtectionModeCommand_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_integrityProtectionAlgorithm,
      { "integrityProtectionAlgorithm", "rrc.integrityProtectionAlgorithm",
        FT_UINT32, BASE_DEC, VALS(rrc_IntegrityProtectionAlgorithm_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_integrityProtectionAlgorithm_01,
      { "integrityProtectionAlgorithm", "rrc.integrityProtectionAlgorithm",
        FT_UINT32, BASE_DEC, VALS(rrc_IntegrityProtectionAlgorithm_r7_vals), 0,
        "IntegrityProtectionAlgorithm_r7", HFILL }},
    { &hf_rrc_downlinkCompressedMode,
      { "downlinkCompressedMode", "rrc.downlinkCompressedMode",
        FT_NONE, BASE_NONE, NULL, 0,
        "CompressedModeMeasCapability", HFILL }},
    { &hf_rrc_uplinkCompressedMode,
      { "uplinkCompressedMode", "rrc.uplinkCompressedMode",
        FT_NONE, BASE_NONE, NULL, 0,
        "CompressedModeMeasCapability", HFILL }},
    { &hf_rrc_compressedModeMeasCapabFDDList,
      { "compressedModeMeasCapabFDDList", "rrc.compressedModeMeasCapabFDDList",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_compressedModeMeasCapabTDDList,
      { "compressedModeMeasCapabTDDList", "rrc.compressedModeMeasCapabTDDList",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_compressedModeMeasCapabGSMList,
      { "compressedModeMeasCapabGSMList", "rrc.compressedModeMeasCapabGSMList",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_compressedModeMeasCapabMC,
      { "compressedModeMeasCapabMC", "rrc.compressedModeMeasCapabMC",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_compressedModeMeasCapabFDDList_01,
      { "compressedModeMeasCapabFDDList", "rrc.compressedModeMeasCapabFDDList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "CompressedModeMeasCapabFDDList2", HFILL }},
    { &hf_rrc_compressedModeMeasCapabEUTRAList,
      { "compressedModeMeasCapabEUTRAList", "rrc.compressedModeMeasCapabEUTRAList",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_compressedModeMeasCapabFDDList2,
      { "compressedModeMeasCapabFDDList2", "rrc.compressedModeMeasCapabFDDList2",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_compressedModeMeasCapabFDDList3,
      { "compressedModeMeasCapabFDDList3", "rrc.compressedModeMeasCapabFDDList3",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_downlinkCompressedMode_LCR,
      { "downlinkCompressedMode-LCR", "rrc.downlinkCompressedMode_LCR",
        FT_NONE, BASE_NONE, NULL, 0,
        "CompressedModeMeasCapability_LCR_r4", HFILL }},
    { &hf_rrc_uplinkCompressedMode_LCR,
      { "uplinkCompressedMode-LCR", "rrc.uplinkCompressedMode_LCR",
        FT_NONE, BASE_NONE, NULL, 0,
        "CompressedModeMeasCapability_LCR_r4", HFILL }},
    { &hf_rrc_downlinkCompressedMode_01,
      { "downlinkCompressedMode", "rrc.downlinkCompressedMode",
        FT_NONE, BASE_NONE, NULL, 0,
        "CompressedModeMeasCapability_v860ext", HFILL }},
    { &hf_rrc_downlinkCompressedMode_02,
      { "downlinkCompressedMode", "rrc.downlinkCompressedMode",
        FT_NONE, BASE_NONE, NULL, 0,
        "CompressedModeMeasCapability_v920ext", HFILL }},
    { &hf_rrc_interFrequencyDetectedSetMeasurements,
      { "interFrequencyDetectedSetMeasurements", "rrc.interFrequencyDetectedSetMeasurements",
        FT_UINT32, BASE_DEC, VALS(rrc_T_interFrequencyDetectedSetMeasurements_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_downlinkCompressedMode_03,
      { "downlinkCompressedMode", "rrc.downlinkCompressedMode",
        FT_NONE, BASE_NONE, NULL, 0,
        "CompressedModeMeasCapability_va40ext", HFILL }},
    { &hf_rrc_interFrequencyMeasOnConfigCarriersWithoutCompMode,
      { "interFrequencyMeasOnConfigCarriersWithoutCompMode", "rrc.interFrequencyMeasOnConfigCarriersWithoutCompMode",
        FT_UINT32, BASE_DEC, VALS(rrc_T_interFrequencyMeasOnConfigCarriersWithoutCompMode_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_idleIntervalMeasCapabEUTRAList,
      { "idleIntervalMeasCapabEUTRAList", "rrc.idleIntervalMeasCapabEUTRAList",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_NonContiguousMultiCellCombinationList_item,
      { "NonContiguousMultiCell", "rrc.NonContiguousMultiCell",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_aggregatedCells,
      { "aggregatedCells", "rrc.aggregatedCells",
        FT_UINT32, BASE_DEC, VALS(rrc_T_aggregatedCells_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_gapSize_01,
      { "gapSize", "rrc.gapSize",
        FT_UINT32, BASE_DEC, VALS(rrc_T_gapSize_01_vals), 0,
        "T_gapSize_01", HFILL }},
    { &hf_rrc_nonContiguousMultiCellCombination22,
      { "nonContiguousMultiCellCombination22", "rrc.nonContiguousMultiCellCombination22",
        FT_UINT32, BASE_DEC, VALS(rrc_T_nonContiguousMultiCellCombination22_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_nonContiguousMultiCellCombination31_13,
      { "nonContiguousMultiCellCombination31-13", "rrc.nonContiguousMultiCellCombination31_13",
        FT_UINT32, BASE_DEC, VALS(rrc_T_nonContiguousMultiCellCombination31_13_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_supportOfGSM,
      { "supportOfGSM", "rrc.supportOfGSM",
        FT_BOOLEAN, BASE_NONE, NULL, 0,
        "BOOLEAN", HFILL }},
    { &hf_rrc_supportOfMulticarrier,
      { "supportOfMulticarrier", "rrc.supportOfMulticarrier",
        FT_BOOLEAN, BASE_NONE, NULL, 0,
        "BOOLEAN", HFILL }},
    { &hf_rrc_supportOfUTRAN_ToGERAN_NACC,
      { "supportOfUTRAN-ToGERAN-NACC", "rrc.supportOfUTRAN_ToGERAN_NACC",
        FT_BOOLEAN, BASE_NONE, NULL, 0,
        "BOOLEAN", HFILL }},
    { &hf_rrc_supportOfHandoverToGAN,
      { "supportOfHandoverToGAN", "rrc.supportOfHandoverToGAN",
        FT_UINT32, BASE_DEC, VALS(rrc_T_supportOfHandoverToGAN_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_supportOfPSHandoverToGAN,
      { "supportOfPSHandoverToGAN", "rrc.supportOfPSHandoverToGAN",
        FT_UINT32, BASE_DEC, VALS(rrc_T_supportOfPSHandoverToGAN_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_supportOfEUTRAFDD,
      { "supportOfEUTRAFDD", "rrc.supportOfEUTRAFDD",
        FT_UINT32, BASE_DEC, VALS(rrc_T_supportOfEUTRAFDD_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_supportOfInterRATHOToEUTRAFDD,
      { "supportOfInterRATHOToEUTRAFDD", "rrc.supportOfInterRATHOToEUTRAFDD",
        FT_UINT32, BASE_DEC, VALS(rrc_T_supportOfInterRATHOToEUTRAFDD_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_supportOfEUTRATDD,
      { "supportOfEUTRATDD", "rrc.supportOfEUTRATDD",
        FT_UINT32, BASE_DEC, VALS(rrc_T_supportOfEUTRATDD_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_supportOfInterRATHOToEUTRATDD,
      { "supportOfInterRATHOToEUTRATDD", "rrc.supportOfInterRATHOToEUTRATDD",
        FT_UINT32, BASE_DEC, VALS(rrc_T_supportOfInterRATHOToEUTRATDD_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_eutraFeatureGroupIndicators,
      { "eutraFeatureGroupIndicators", "rrc.eutraFeatureGroupIndicators",
        FT_BYTES, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_supportOfEUTRAFDDMeasurementReportingInCELLFACH,
      { "supportOfEUTRAFDDMeasurementReportingInCELLFACH", "rrc.supportOfEUTRAFDDMeasurementReportingInCELLFACH",
        FT_UINT32, BASE_DEC, VALS(rrc_T_supportOfEUTRAFDDMeasurementReportingInCELLFACH_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_supportOfEUTRATDDMeasurementReportingInCELLFACH,
      { "supportOfEUTRATDDMeasurementReportingInCELLFACH", "rrc.supportOfEUTRATDDMeasurementReportingInCELLFACH",
        FT_UINT32, BASE_DEC, VALS(rrc_T_supportOfEUTRATDDMeasurementReportingInCELLFACH_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_supportOfIntraFreqSIAcquisitionForHO,
      { "supportOfIntraFreqSIAcquisitionForHO", "rrc.supportOfIntraFreqSIAcquisitionForHO",
        FT_UINT32, BASE_DEC, VALS(rrc_T_supportOfIntraFreqSIAcquisitionForHO_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_supportOfInterFreqSIAcquisitionForHO,
      { "supportOfInterFreqSIAcquisitionForHO", "rrc.supportOfInterFreqSIAcquisitionForHO",
        FT_UINT32, BASE_DEC, VALS(rrc_T_supportOfInterFreqSIAcquisitionForHO_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_supportOfE_UtraSIAcquisitionForHO,
      { "supportOfE-UtraSIAcquisitionForHO", "rrc.supportOfE_UtraSIAcquisitionForHO",
        FT_UINT32, BASE_DEC, VALS(rrc_T_supportOfE_UtraSIAcquisitionForHO_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_NetworkAssistedGANSS_Supported_List_item,
      { "NetworkAssistedGANSS-Supported-List item", "rrc.NetworkAssistedGANSS_Supported_List_item",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_gANSS_Id,
      { "gANSS-Id", "rrc.gANSS_Id",
        FT_UINT32, BASE_DEC, VALS(rrc_T_gANSS_Id_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_gANSS_Mode,
      { "gANSS-Mode", "rrc.gANSS_Mode",
        FT_UINT32, BASE_DEC, VALS(rrc_GANSS_Mode_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_gANSS_SignalId,
      { "gANSS-SignalId", "rrc.gANSS_SignalId",
        FT_UINT32, BASE_DEC, NULL, 0,
        "GANSS_Signal_Id", HFILL }},
    { &hf_rrc_supportForUE_GANSS_TimingOfCellFrames,
      { "supportForUE-GANSS-TimingOfCellFrames", "rrc.supportForUE_GANSS_TimingOfCellFrames",
        FT_BOOLEAN, BASE_NONE, NULL, 0,
        "BOOLEAN", HFILL }},
    { &hf_rrc_supportForUE_GANSS_CarrierPhaseMeasurement,
      { "supportForUE-GANSS-CarrierPhaseMeasurement", "rrc.supportForUE_GANSS_CarrierPhaseMeasurement",
        FT_BOOLEAN, BASE_NONE, NULL, 0,
        "BOOLEAN", HFILL }},
    { &hf_rrc_NetworkAssistedGANSS_Supported_List_v860ext_item,
      { "NetworkAssistedGANSS-Supported-List-v860ext item", "rrc.NetworkAssistedGANSS_Supported_List_v860ext_item",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_sbas_Ids,
      { "sbas-Ids", "rrc.sbas_Ids",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_8", HFILL }},
    { &hf_rrc_gANSS_SignalIds,
      { "gANSS-SignalIds", "rrc.gANSS_SignalIds",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_8", HFILL }},
    { &hf_rrc_non_native_AD_choices_supported,
      { "non-native-AD-choices-supported", "rrc.non_native_AD_choices_supported",
        FT_UINT32, BASE_DEC, VALS(rrc_T_non_native_AD_choices_supported_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_p_TMSI,
      { "p-TMSI", "rrc.p_TMSI",
        FT_BYTES, BASE_NONE, NULL, 0,
        "P_TMSI_GSM_MAP", HFILL }},
    { &hf_rrc_rai,
      { "rai", "rrc.rai",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_cn_Identity,
      { "cn-Identity", "rrc.cn_Identity",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_cn_pagedUE_Identity,
      { "cn-pagedUE-Identity", "rrc.cn_pagedUE_Identity",
        FT_UINT32, BASE_DEC, VALS(rrc_CN_PagedUE_Identity_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_utran_Identity,
      { "utran-Identity", "rrc.utran_Identity",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_cn_OriginatedPage_connectedMode_UE,
      { "cn-OriginatedPage-connectedMode-UE", "rrc.cn_OriginatedPage_connectedMode_UE",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_utran_SingleUE_Identity,
      { "utran-SingleUE-Identity", "rrc.utran_SingleUE_Identity",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_cn_OriginatedPage_connectedMode_UE_01,
      { "cn-OriginatedPage-connectedMode-UE", "rrc.cn_OriginatedPage_connectedMode_UE",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_cn_OriginatedPage_connectedMode_UE_01", HFILL }},
    { &hf_rrc_utran_GroupIdentity,
      { "utran-GroupIdentity", "rrc.utran_GroupIdentity",
        FT_UINT32, BASE_DEC, NULL, 0,
        "SEQUENCE_SIZE_1_maxURNTI_Group_OF_GroupIdentityWithReleaseInformation", HFILL }},
    { &hf_rrc_utran_GroupIdentity_item,
      { "GroupIdentityWithReleaseInformation", "rrc.GroupIdentityWithReleaseInformation",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_PagingRecordList_item,
      { "PagingRecord", "rrc.PagingRecord",
        FT_UINT32, BASE_DEC, VALS(rrc_PagingRecord_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_PagingRecord2List_r5_item,
      { "PagingRecord2-r5", "rrc.PagingRecord2_r5",
        FT_UINT32, BASE_DEC, VALS(rrc_PagingRecord2_r5_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_losslessSRNS_RelocationSupport,
      { "losslessSRNS-RelocationSupport", "rrc.losslessSRNS_RelocationSupport",
        FT_BOOLEAN, BASE_NONE, NULL, 0,
        "BOOLEAN", HFILL }},
    { &hf_rrc_supportForRfc2507,
      { "supportForRfc2507", "rrc.supportForRfc2507",
        FT_UINT32, BASE_DEC, VALS(rrc_T_supportForRfc2507_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_notSupported,
      { "notSupported", "rrc.notSupported",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_supported_01,
      { "supported", "rrc.supported",
        FT_UINT32, BASE_DEC, VALS(rrc_MaxHcContextSpace_vals), 0,
        "MaxHcContextSpace", HFILL }},
    { &hf_rrc_supportForRfc3095,
      { "supportForRfc3095", "rrc.supportForRfc3095",
        FT_UINT32, BASE_DEC, VALS(rrc_T_supportForRfc3095_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_supported_02,
      { "supported", "rrc.supported",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_maxROHC_ContextSessions,
      { "maxROHC-ContextSessions", "rrc.maxROHC_ContextSessions",
        FT_UINT32, BASE_DEC, VALS(rrc_MaxROHC_ContextSessions_r4_vals), 0,
        "MaxROHC_ContextSessions_r4", HFILL }},
    { &hf_rrc_reverseCompressionDepth,
      { "reverseCompressionDepth", "rrc.reverseCompressionDepth",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_65535", HFILL }},
    { &hf_rrc_supportForRfc3095ContextRelocation,
      { "supportForRfc3095ContextRelocation", "rrc.supportForRfc3095ContextRelocation",
        FT_BOOLEAN, BASE_NONE, NULL, 0,
        "BOOLEAN", HFILL }},
    { &hf_rrc_maxHcContextSpace,
      { "maxHcContextSpace", "rrc.maxHcContextSpace",
        FT_UINT32, BASE_DEC, VALS(rrc_MaxHcContextSpace_r5_ext_vals), 0,
        "MaxHcContextSpace_r5_ext", HFILL }},
    { &hf_rrc_losslessDLRLC_PDUSizeChange,
      { "losslessDLRLC-PDUSizeChange", "rrc.losslessDLRLC_PDUSizeChange",
        FT_UINT32, BASE_DEC, VALS(rrc_T_losslessDLRLC_PDUSizeChange_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_supportForCSVoiceoverHSPA_01,
      { "supportForCSVoiceoverHSPA", "rrc.supportForCSVoiceoverHSPA",
        FT_UINT32, BASE_DEC, VALS(rrc_T_supportForCSVoiceoverHSPA_01_vals), 0,
        "T_supportForCSVoiceoverHSPA_01", HFILL }},
    { &hf_rrc_fddPhysChCapability,
      { "fddPhysChCapability", "rrc.fddPhysChCapability",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_downlinkPhysChCapability,
      { "downlinkPhysChCapability", "rrc.downlinkPhysChCapability",
        FT_NONE, BASE_NONE, NULL, 0,
        "DL_PhysChCapabilityFDD", HFILL }},
    { &hf_rrc_uplinkPhysChCapability,
      { "uplinkPhysChCapability", "rrc.uplinkPhysChCapability",
        FT_NONE, BASE_NONE, NULL, 0,
        "UL_PhysChCapabilityFDD", HFILL }},
    { &hf_rrc_tddPhysChCapability,
      { "tddPhysChCapability", "rrc.tddPhysChCapability",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_downlinkPhysChCapability_01,
      { "downlinkPhysChCapability", "rrc.downlinkPhysChCapability",
        FT_NONE, BASE_NONE, NULL, 0,
        "DL_PhysChCapabilityTDD", HFILL }},
    { &hf_rrc_uplinkPhysChCapability_01,
      { "uplinkPhysChCapability", "rrc.uplinkPhysChCapability",
        FT_NONE, BASE_NONE, NULL, 0,
        "UL_PhysChCapabilityTDD", HFILL }},
    { &hf_rrc_fddPhysChCapability_01,
      { "fddPhysChCapability", "rrc.fddPhysChCapability",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_fddPhysChCapability_01", HFILL }},
    { &hf_rrc_downlinkPhysChCapability_02,
      { "downlinkPhysChCapability", "rrc.downlinkPhysChCapability",
        FT_NONE, BASE_NONE, NULL, 0,
        "DL_PhysChCapabilityFDD_v770ext", HFILL }},
    { &hf_rrc_uplinkPhysChCapability_02,
      { "uplinkPhysChCapability", "rrc.uplinkPhysChCapability",
        FT_NONE, BASE_NONE, NULL, 0,
        "UL_PhysChCapabilityFDD_v770ext", HFILL }},
    { &hf_rrc_tddPhysChCapability_384,
      { "tddPhysChCapability-384", "rrc.tddPhysChCapability_384",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_uplinkPhysChCapability_03,
      { "uplinkPhysChCapability", "rrc.uplinkPhysChCapability",
        FT_NONE, BASE_NONE, NULL, 0,
        "UL_PhysChCapabilityTDD_384_v770ext", HFILL }},
    { &hf_rrc_tddPhysChCapability_768,
      { "tddPhysChCapability-768", "rrc.tddPhysChCapability_768",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_downlinkPhysChCapability_03,
      { "downlinkPhysChCapability", "rrc.downlinkPhysChCapability",
        FT_NONE, BASE_NONE, NULL, 0,
        "DL_PhysChCapabilityTDD_768", HFILL }},
    { &hf_rrc_uplinkPhysChCapability_04,
      { "uplinkPhysChCapability", "rrc.uplinkPhysChCapability",
        FT_NONE, BASE_NONE, NULL, 0,
        "UL_PhysChCapabilityTDD_768", HFILL }},
    { &hf_rrc_tddPhysChCapability_128,
      { "tddPhysChCapability-128", "rrc.tddPhysChCapability_128",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_downlinkPhysChCapability_04,
      { "downlinkPhysChCapability", "rrc.downlinkPhysChCapability",
        FT_NONE, BASE_NONE, NULL, 0,
        "DL_PhysChCapabilityTDD_128_v770ext", HFILL }},
    { &hf_rrc_uplinkPhysChCapability_05,
      { "uplinkPhysChCapability", "rrc.uplinkPhysChCapability",
        FT_NONE, BASE_NONE, NULL, 0,
        "UL_PhysChCapabilityTDD_128_v770ext", HFILL }},
    { &hf_rrc_fddPhysChCapability_02,
      { "fddPhysChCapability", "rrc.fddPhysChCapability",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_fddPhysChCapability_02", HFILL }},
    { &hf_rrc_downlinkPhysChCapability_05,
      { "downlinkPhysChCapability", "rrc.downlinkPhysChCapability",
        FT_NONE, BASE_NONE, NULL, 0,
        "DL_PhysChCapabilityFDD_v860ext", HFILL }},
    { &hf_rrc_tddPhysChCapability_128_01,
      { "tddPhysChCapability-128", "rrc.tddPhysChCapability_128",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_tddPhysChCapability_128_01", HFILL }},
    { &hf_rrc_downlinkPhysChCapability_06,
      { "downlinkPhysChCapability", "rrc.downlinkPhysChCapability",
        FT_NONE, BASE_NONE, NULL, 0,
        "DL_PhysChCapabilityTDD_128_v860ext", HFILL }},
    { &hf_rrc_fddPhysChCapability_03,
      { "fddPhysChCapability", "rrc.fddPhysChCapability",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_fddPhysChCapability_03", HFILL }},
    { &hf_rrc_downlinkPhysChCapability_07,
      { "downlinkPhysChCapability", "rrc.downlinkPhysChCapability",
        FT_NONE, BASE_NONE, NULL, 0,
        "DL_PhysChCapabilityFDD_v920ext", HFILL }},
    { &hf_rrc_uplinkPhysChCapability_06,
      { "uplinkPhysChCapability", "rrc.uplinkPhysChCapability",
        FT_NONE, BASE_NONE, NULL, 0,
        "UL_PhysChCapabilityFDD_v920ext", HFILL }},
    { &hf_rrc_fddPhysChCapability_04,
      { "fddPhysChCapability", "rrc.fddPhysChCapability",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_fddPhysChCapability_04", HFILL }},
    { &hf_rrc_downlinkPhysChCapability_08,
      { "downlinkPhysChCapability", "rrc.downlinkPhysChCapability",
        FT_NONE, BASE_NONE, NULL, 0,
        "DL_PhysChCapabilityFDD_va40ext", HFILL }},
    { &hf_rrc_tddPhysChCapability_128_02,
      { "tddPhysChCapability-128", "rrc.tddPhysChCapability_128",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_tddPhysChCapability_128_02", HFILL }},
    { &hf_rrc_downlinkPhysChCapability_09,
      { "downlinkPhysChCapability", "rrc.downlinkPhysChCapability",
        FT_NONE, BASE_NONE, NULL, 0,
        "DL_PhysChCapabilityInfoTDD_128_va40ext", HFILL }},
    { &hf_rrc_uplinkPhysChCapability_07,
      { "uplinkPhysChCapability", "rrc.uplinkPhysChCapability",
        FT_NONE, BASE_NONE, NULL, 0,
        "UL_PhysChCapabilityInfoTDD_128_va40ext", HFILL }},
    { &hf_rrc_fddPhysChCapability_05,
      { "fddPhysChCapability", "rrc.fddPhysChCapability",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_fddPhysChCapability_05", HFILL }},
    { &hf_rrc_downlinkPhysChCapability_10,
      { "downlinkPhysChCapability", "rrc.downlinkPhysChCapability",
        FT_NONE, BASE_NONE, NULL, 0,
        "DL_PhysChCapabilityFDD_vbxyext", HFILL }},
    { &hf_rrc_tddPhysChCapability_768_01,
      { "tddPhysChCapability-768", "rrc.tddPhysChCapability_768",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_tddPhysChCapability_768_01", HFILL }},
    { &hf_rrc_downlinkPhysChCapability_11,
      { "downlinkPhysChCapability", "rrc.downlinkPhysChCapability",
        FT_NONE, BASE_NONE, NULL, 0,
        "DL_PhysChCapabilityInfoTDD_768", HFILL }},
    { &hf_rrc_tddPhysChCapability_128_03,
      { "tddPhysChCapability-128", "rrc.tddPhysChCapability_128",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_tddPhysChCapability_128_03", HFILL }},
    { &hf_rrc_uplinkPhysChCapability_08,
      { "uplinkPhysChCapability", "rrc.uplinkPhysChCapability",
        FT_NONE, BASE_NONE, NULL, 0,
        "UL_PhysChCapabilityInfoTDD_128_v770ext", HFILL }},
    { &hf_rrc_tdd128_PhysChCapability,
      { "tdd128-PhysChCapability", "rrc.tdd128_PhysChCapability",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_downlinkPhysChCapability_12,
      { "downlinkPhysChCapability", "rrc.downlinkPhysChCapability",
        FT_NONE, BASE_NONE, NULL, 0,
        "DL_PhysChCapabilityTDD_LCR_r4", HFILL }},
    { &hf_rrc_uplinkPhysChCapability_09,
      { "uplinkPhysChCapability", "rrc.uplinkPhysChCapability",
        FT_NONE, BASE_NONE, NULL, 0,
        "UL_PhysChCapabilityTDD_LCR_r4", HFILL }},
    { &hf_rrc_fdd_hspdsch,
      { "fdd-hspdsch", "rrc.fdd_hspdsch",
        FT_UINT32, BASE_DEC, VALS(rrc_T_fdd_hspdsch_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_supported_03,
      { "supported", "rrc.supported",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_supported_01", HFILL }},
    { &hf_rrc_hsdsch_physical_layer_category,
      { "hsdsch-physical-layer-category", "rrc.hsdsch_physical_layer_category",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_dummy2_06,
      { "dummy2", "rrc.dummy2",
        FT_BOOLEAN, BASE_NONE, NULL, 0,
        "BOOLEAN", HFILL }},
    { &hf_rrc_tdd384_hspdsch,
      { "tdd384-hspdsch", "rrc.tdd384_hspdsch",
        FT_UINT32, BASE_DEC, VALS(rrc_T_tdd384_hspdsch_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_tdd128_hspdsch,
      { "tdd128-hspdsch", "rrc.tdd128_hspdsch",
        FT_UINT32, BASE_DEC, VALS(rrc_T_tdd128_hspdsch_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_numberOfRepetitionsPerSFNPeriod,
      { "numberOfRepetitionsPerSFNPeriod", "rrc.numberOfRepetitionsPerSFNPeriod",
        FT_UINT32, BASE_DEC|BASE_EXT_STRING, &rrc_T_numberOfRepetitionsPerSFNPeriod_vals_ext, 0,
        NULL, HFILL }},
    { &hf_rrc_supportEUTRA_FDD,
      { "supportEUTRA-FDD", "rrc.supportEUTRA_FDD",
        FT_BOOLEAN, BASE_NONE, NULL, 0,
        "BOOLEAN", HFILL }},
    { &hf_rrc_supportEUTRA_TDD,
      { "supportEUTRA-TDD", "rrc.supportEUTRA_TDD",
        FT_BOOLEAN, BASE_NONE, NULL, 0,
        "BOOLEAN", HFILL }},
    { &hf_rrc_noError,
      { "noError", "rrc.noError",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_errorOccurred,
      { "errorOccurred", "rrc.errorOccurred",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_protocolErrorInformation_01,
      { "protocolErrorInformation", "rrc.protocolErrorInformation",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_diagnosticsType,
      { "diagnosticsType", "rrc.diagnosticsType",
        FT_UINT32, BASE_DEC, VALS(rrc_T_diagnosticsType_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_type1,
      { "type1", "rrc.type1",
        FT_UINT32, BASE_DEC, VALS(rrc_T_type1_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_asn1_ViolationOrEncodingError,
      { "asn1-ViolationOrEncodingError", "rrc.asn1_ViolationOrEncodingError",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_messageTypeNonexistent,
      { "messageTypeNonexistent", "rrc.messageTypeNonexistent",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_messageNotCompatibleWithReceiverState,
      { "messageNotCompatibleWithReceiverState", "rrc.messageNotCompatibleWithReceiverState",
        FT_NONE, BASE_NONE, NULL, 0,
        "IdentificationOfReceivedMessage", HFILL }},
    { &hf_rrc_ie_ValueNotComprehended,
      { "ie-ValueNotComprehended", "rrc.ie_ValueNotComprehended",
        FT_NONE, BASE_NONE, NULL, 0,
        "IdentificationOfReceivedMessage", HFILL }},
    { &hf_rrc_conditionalInformationElementError,
      { "conditionalInformationElementError", "rrc.conditionalInformationElementError",
        FT_NONE, BASE_NONE, NULL, 0,
        "IdentificationOfReceivedMessage", HFILL }},
    { &hf_rrc_messageExtensionNotComprehended,
      { "messageExtensionNotComprehended", "rrc.messageExtensionNotComprehended",
        FT_NONE, BASE_NONE, NULL, 0,
        "IdentificationOfReceivedMessage", HFILL }},
    { &hf_rrc_RadioFrequencyBandTDDextList_item,
      { "RadioFrequencyBandTDDext", "rrc.RadioFrequencyBandTDDext",
        FT_UINT32, BASE_DEC, VALS(rrc_RadioFrequencyBandTDDext_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_t314_expired,
      { "t314-expired", "rrc.t314_expired",
        FT_BOOLEAN, BASE_NONE, NULL, 0,
        "BOOLEAN", HFILL }},
    { &hf_rrc_t315_expired,
      { "t315-expired", "rrc.t315_expired",
        FT_BOOLEAN, BASE_NONE, NULL, 0,
        "BOOLEAN", HFILL }},
    { &hf_rrc_interRATInfo,
      { "interRATInfo", "rrc.interRATInfo",
        FT_UINT32, BASE_DEC, VALS(rrc_InterRATInfo_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_interRATInfo_01,
      { "interRATInfo", "rrc.interRATInfo",
        FT_NONE, BASE_NONE, NULL, 0,
        "InterRATInfo_r6", HFILL }},
    { &hf_rrc_frequencyInfo_03,
      { "frequencyInfo", "rrc.frequencyInfo",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_interRATInfo_02,
      { "interRATInfo", "rrc.interRATInfo",
        FT_UINT32, BASE_DEC, VALS(rrc_InterRATInfo_v860ext_vals), 0,
        "InterRATInfo_v860ext", HFILL }},
    { &hf_rrc_fddRF_Capability,
      { "fddRF-Capability", "rrc.fddRF_Capability",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_ue_PowerClass,
      { "ue-PowerClass", "rrc.ue_PowerClass",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_txRxFrequencySeparation,
      { "txRxFrequencySeparation", "rrc.txRxFrequencySeparation",
        FT_UINT32, BASE_DEC, VALS(rrc_TxRxFrequencySeparation_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_tddRF_Capability,
      { "tddRF-Capability", "rrc.tddRF_Capability",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_radioFrequencyTDDBandList,
      { "radioFrequencyTDDBandList", "rrc.radioFrequencyTDDBandList",
        FT_UINT32, BASE_DEC, VALS(rrc_RadioFrequencyBandTDDList_vals), 0,
        "RadioFrequencyBandTDDList", HFILL }},
    { &hf_rrc_chipRateCapability,
      { "chipRateCapability", "rrc.chipRateCapability",
        FT_UINT32, BASE_DEC, VALS(rrc_ChipRateCapability_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_tddRF_Capability_01,
      { "tddRF-Capability", "rrc.tddRF_Capability",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_tddRF_Capability_01", HFILL }},
    { &hf_rrc_radioFrequencyBandTDDList,
      { "radioFrequencyBandTDDList", "rrc.radioFrequencyBandTDDList",
        FT_UINT32, BASE_DEC, VALS(rrc_RadioFrequencyBandTDDList_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_tdd384RF_Capability,
      { "tdd384RF-Capability", "rrc.tdd384RF_Capability",
        FT_UINT32, BASE_DEC, VALS(rrc_RadioFrequencyBandTDDList_r7_vals), 0,
        "RadioFrequencyBandTDDList_r7", HFILL }},
    { &hf_rrc_tdd768RF_Capability,
      { "tdd768RF-Capability", "rrc.tdd768RF_Capability",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_radioFrequencyBandTDDList_01,
      { "radioFrequencyBandTDDList", "rrc.radioFrequencyBandTDDList",
        FT_UINT32, BASE_DEC, VALS(rrc_RadioFrequencyBandTDDList_r7_vals), 0,
        "RadioFrequencyBandTDDList_r7", HFILL }},
    { &hf_rrc_tdd128RF_Capability,
      { "tdd128RF-Capability", "rrc.tdd128RF_Capability",
        FT_UINT32, BASE_DEC, VALS(rrc_RadioFrequencyBandTDDList_r7_vals), 0,
        "RadioFrequencyBandTDDList_r7", HFILL }},
    { &hf_rrc_tdd128RF_Capability_01,
      { "tdd128RF-Capability", "rrc.tdd128RF_Capability",
        FT_UINT32, BASE_DEC, NULL, 0,
        "RadioFrequencyBandTDDextList", HFILL }},
    { &hf_rrc_tdd384RF_Capability_01,
      { "tdd384RF-Capability", "rrc.tdd384RF_Capability",
        FT_UINT32, BASE_DEC, NULL, 0,
        "RadioFrequencyBandTDDextList", HFILL }},
    { &hf_rrc_tdd768RF_Capability_01,
      { "tdd768RF-Capability", "rrc.tdd768RF_Capability",
        FT_UINT32, BASE_DEC, NULL, 0,
        "RadioFrequencyBandTDDextList", HFILL }},
    { &hf_rrc_supportOfMFBI,
      { "supportOfMFBI", "rrc.supportOfMFBI",
        FT_UINT32, BASE_DEC, VALS(rrc_T_supportOfMFBI_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_totalRLC_AM_BufferSize,
      { "totalRLC-AM-BufferSize", "rrc.totalRLC_AM_BufferSize",
        FT_UINT32, BASE_DEC, VALS(rrc_TotalRLC_AM_BufferSize_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_maximumRLC_WindowSize,
      { "maximumRLC-WindowSize", "rrc.maximumRLC_WindowSize",
        FT_UINT32, BASE_DEC, VALS(rrc_MaximumRLC_WindowSize_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_maximumAM_EntityNumber,
      { "maximumAM-EntityNumber", "rrc.maximumAM_EntityNumber",
        FT_UINT32, BASE_DEC, VALS(rrc_MaximumAM_EntityNumberRLC_Cap_vals), 0,
        "MaximumAM_EntityNumberRLC_Cap", HFILL }},
    { &hf_rrc_totalRLC_AM_BufferSize_01,
      { "totalRLC-AM-BufferSize", "rrc.totalRLC_AM_BufferSize",
        FT_UINT32, BASE_DEC, VALS(rrc_TotalRLC_AM_BufferSize_r5_ext_vals), 0,
        "TotalRLC_AM_BufferSize_r5_ext", HFILL }},
    { &hf_rrc_supportOfTwoLogicalChannel,
      { "supportOfTwoLogicalChannel", "rrc.supportOfTwoLogicalChannel",
        FT_BOOLEAN, BASE_NONE, NULL, 0,
        "BOOLEAN", HFILL }},
    { &hf_rrc_totalRLC_AM_BufferSize_02,
      { "totalRLC-AM-BufferSize", "rrc.totalRLC_AM_BufferSize",
        FT_UINT32, BASE_DEC, VALS(rrc_TotalRLC_AM_BufferSize_v920ext_vals), 0,
        "TotalRLC_AM_BufferSize_v920ext", HFILL }},
    { &hf_rrc_totalRLC_AM_BufferSize_03,
      { "totalRLC-AM-BufferSize", "rrc.totalRLC_AM_BufferSize",
        FT_UINT32, BASE_DEC, VALS(rrc_TotalRLC_AM_BufferSize_va40ext_vals), 0,
        "TotalRLC_AM_BufferSize_va40ext", HFILL }},
    { &hf_rrc_totalRLC_AM_BufferSize_04,
      { "totalRLC-AM-BufferSize", "rrc.totalRLC_AM_BufferSize",
        FT_UINT32, BASE_DEC, VALS(rrc_TotalRLC_AM_BufferSize_vbxyext_vals), 0,
        "TotalRLC_AM_BufferSize_vbxyext", HFILL }},
    { &hf_rrc_noRelease,
      { "noRelease", "rrc.noRelease",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_release,
      { "release", "rrc.release",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_RRC_MessageSequenceNumberList_item,
      { "RRC-MessageSequenceNumber", "rrc.RRC_MessageSequenceNumber",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_nonce,
      { "nonce", "rrc.nonce",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_128", HFILL }},
    { &hf_rrc_ims_Information,
      { "ims-Information", "rrc.ims_Information",
        FT_BYTES, BASE_NONE, NULL, 0,
        "OCTET_STRING_SIZE_1_32", HFILL }},
    { &hf_rrc_cipheringAlgorithmCap,
      { "cipheringAlgorithmCap", "rrc.cipheringAlgorithmCap",
        FT_BYTES, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_integrityProtectionAlgorithmCap,
      { "integrityProtectionAlgorithmCap", "rrc.integrityProtectionAlgorithmCap",
        FT_BYTES, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_primary_E_RNTI,
      { "primary-E-RNTI", "rrc.primary_E_RNTI",
        FT_BYTES, BASE_NONE, NULL, 0,
        "E_RNTI", HFILL }},
    { &hf_rrc_secondary_E_RNTI,
      { "secondary-E-RNTI", "rrc.secondary_E_RNTI",
        FT_BYTES, BASE_NONE, NULL, 0,
        "E_RNTI", HFILL }},
    { &hf_rrc_scramblingCodeType,
      { "scramblingCodeType", "rrc.scramblingCodeType",
        FT_UINT32, BASE_DEC, VALS(rrc_ScramblingCodeType_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_scramblingCodeNumber,
      { "scramblingCodeNumber", "rrc.scramblingCodeNumber",
        FT_UINT32, BASE_DEC, NULL, 0,
        "UL_ScramblingCode", HFILL }},
    { &hf_rrc_ms2_SchedTransmGrantHARQAlloc,
      { "ms2-SchedTransmGrantHARQAlloc", "rrc.ms2_SchedTransmGrantHARQAlloc",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_8", HFILL }},
    { &hf_rrc_servingGrant,
      { "servingGrant", "rrc.servingGrant",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_primary_Secondary_GrantSelector,
      { "primary-Secondary-GrantSelector", "rrc.primary_Secondary_GrantSelector",
        FT_UINT32, BASE_DEC, VALS(rrc_T_primary_Secondary_GrantSelector_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_minReduced_E_DPDCH_GainFactor,
      { "minReduced-E-DPDCH-GainFactor", "rrc.minReduced_E_DPDCH_GainFactor",
        FT_UINT32, BASE_DEC, VALS(rrc_MinReduced_E_DPDCH_GainFactor_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_e_DCH_minimumSet_E_TFCI,
      { "e-DCH-minimumSet-E-TFCI", "rrc.e_DCH_minimumSet_E_TFCI",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_dpcchPowerOffset_SecondaryULFrequency,
      { "dpcchPowerOffset-SecondaryULFrequency", "rrc.dpcchPowerOffset_SecondaryULFrequency",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_7", HFILL }},
    { &hf_rrc_pc_Preamble,
      { "pc-Preamble", "rrc.pc_Preamble",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_deltaACK,
      { "deltaACK", "rrc.deltaACK",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_deltaNACK,
      { "deltaNACK", "rrc.deltaNACK",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_dl_hspdsch_Information,
      { "dl-hspdsch-Information", "rrc.dl_hspdsch_Information",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_harqInfo_02,
      { "harqInfo", "rrc.harqInfo",
        FT_NONE, BASE_NONE, NULL, 0,
        "HARQ_Info", HFILL }},
    { &hf_rrc_mac_hsResetIndicator,
      { "mac-hsResetIndicator", "rrc.mac_hsResetIndicator",
        FT_UINT32, BASE_DEC, VALS(rrc_T_mac_hsResetIndicator_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_dl_hspdsch_Information_01,
      { "dl-hspdsch-Information", "rrc.dl_hspdsch_Information",
        FT_NONE, BASE_NONE, NULL, 0,
        "DL_HSPDSCH_Information_r7", HFILL }},
    { &hf_rrc_mac_hsResetIndicator_01,
      { "mac-hsResetIndicator", "rrc.mac_hsResetIndicator",
        FT_UINT32, BASE_DEC, VALS(rrc_T_mac_hsResetIndicator_01_vals), 0,
        "T_mac_hsResetIndicator_01", HFILL }},
    { &hf_rrc_dl_hspdsch_Information_02,
      { "dl-hspdsch-Information", "rrc.dl_hspdsch_Information",
        FT_NONE, BASE_NONE, NULL, 0,
        "DL_HSPDSCH_Information_r8", HFILL }},
    { &hf_rrc_mac_hsResetIndicator_02,
      { "mac-hsResetIndicator", "rrc.mac_hsResetIndicator",
        FT_UINT32, BASE_DEC, VALS(rrc_T_mac_hsResetIndicator_02_vals), 0,
        "T_mac_hsResetIndicator_02", HFILL }},
    { &hf_rrc_dl_hspdsch_Information_03,
      { "dl-hspdsch-Information", "rrc.dl_hspdsch_Information",
        FT_NONE, BASE_NONE, NULL, 0,
        "DL_HSPDSCH_Information_r9", HFILL }},
    { &hf_rrc_mac_hsResetIndicator_03,
      { "mac-hsResetIndicator", "rrc.mac_hsResetIndicator",
        FT_UINT32, BASE_DEC, VALS(rrc_T_mac_hsResetIndicator_03_vals), 0,
        "T_mac_hsResetIndicator_03", HFILL }},
    { &hf_rrc_deltaACK_01,
      { "deltaACK", "rrc.deltaACK",
        FT_UINT32, BASE_DEC, NULL, 0,
        "DeltaACK_r11", HFILL }},
    { &hf_rrc_deltaNACK_01,
      { "deltaNACK", "rrc.deltaNACK",
        FT_UINT32, BASE_DEC, NULL, 0,
        "DeltaNACK_r11", HFILL }},
    { &hf_rrc_dl_hspdsch_Information_04,
      { "dl-hspdsch-Information", "rrc.dl_hspdsch_Information",
        FT_NONE, BASE_NONE, NULL, 0,
        "DL_HSPDSCH_Information_r11", HFILL }},
    { &hf_rrc_mac_hsResetIndicator_04,
      { "mac-hsResetIndicator", "rrc.mac_hsResetIndicator",
        FT_UINT32, BASE_DEC, VALS(rrc_T_mac_hsResetIndicator_04_vals), 0,
        "T_mac_hsResetIndicator_04", HFILL }},
    { &hf_rrc_mac_hsResetIndicator_assisting,
      { "mac-hsResetIndicator-assisting", "rrc.mac_hsResetIndicator_assisting",
        FT_UINT32, BASE_DEC, VALS(rrc_T_mac_hsResetIndicator_assisting_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_supported_04,
      { "supported", "rrc.supported",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_supported_02", HFILL }},
    { &hf_rrc_maxNoSCCPCH_RL,
      { "maxNoSCCPCH-RL", "rrc.maxNoSCCPCH_RL",
        FT_UINT32, BASE_DEC, VALS(rrc_MaxNoSCCPCH_RL_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_simultaneousSCCPCH_DPCH_DPDCH_Reception,
      { "simultaneousSCCPCH-DPCH-DPDCH-Reception", "rrc.simultaneousSCCPCH_DPCH_DPDCH_Reception",
        FT_BOOLEAN, BASE_NONE, NULL, 0,
        "BOOLEAN", HFILL }},
    { &hf_rrc_rab_InfoReplace,
      { "rab-InfoReplace", "rrc.rab_InfoReplace",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_STARTList_item,
      { "STARTSingle", "rrc.STARTSingle",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_SystemSpecificCapUpdateReqList_item,
      { "SystemSpecificCapUpdateReq", "rrc.SystemSpecificCapUpdateReq",
        FT_UINT32, BASE_DEC, VALS(rrc_SystemSpecificCapUpdateReq_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_SystemSpecificCapUpdateReqList_r5_item,
      { "SystemSpecificCapUpdateReq-r5", "rrc.SystemSpecificCapUpdateReq_r5",
        FT_UINT32, BASE_DEC, VALS(rrc_SystemSpecificCapUpdateReq_r5_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_SystemSpecificCapUpdateReqList_r8_item,
      { "SystemSpecificCapUpdateReq-r8", "rrc.SystemSpecificCapUpdateReq_r8",
        FT_UINT32, BASE_DEC, VALS(rrc_SystemSpecificCapUpdateReq_r8_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_tmsi,
      { "tmsi", "rrc.tmsi",
        FT_BYTES, BASE_NONE, NULL, 0,
        "TMSI_GSM_MAP", HFILL }},
    { &hf_rrc_dl_TransChCapability,
      { "dl-TransChCapability", "rrc.dl_TransChCapability",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_ul_TransChCapability,
      { "ul-TransChCapability", "rrc.ul_TransChCapability",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_supported_05,
      { "supported", "rrc.supported",
        FT_UINT32, BASE_DEC, VALS(rrc_MaxNoBits_vals), 0,
        "MaxNoBits", HFILL }},
    { &hf_rrc_srnc_Identity,
      { "srnc-Identity", "rrc.srnc_Identity",
        FT_BYTES, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_s_RNTI,
      { "s-RNTI", "rrc.s_RNTI",
        FT_BYTES, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_all,
      { "all", "rrc.all",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_u_RNTI_BitMaskIndex_b1,
      { "u-RNTI-BitMaskIndex-b1", "rrc.u_RNTI_BitMaskIndex_b1",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_31", HFILL }},
    { &hf_rrc_u_RNTI_BitMaskIndex_b2,
      { "u-RNTI-BitMaskIndex-b2", "rrc.u_RNTI_BitMaskIndex_b2",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_30", HFILL }},
    { &hf_rrc_u_RNTI_BitMaskIndex_b3,
      { "u-RNTI-BitMaskIndex-b3", "rrc.u_RNTI_BitMaskIndex_b3",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_29", HFILL }},
    { &hf_rrc_u_RNTI_BitMaskIndex_b4,
      { "u-RNTI-BitMaskIndex-b4", "rrc.u_RNTI_BitMaskIndex_b4",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_28", HFILL }},
    { &hf_rrc_u_RNTI_BitMaskIndex_b5,
      { "u-RNTI-BitMaskIndex-b5", "rrc.u_RNTI_BitMaskIndex_b5",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_27", HFILL }},
    { &hf_rrc_u_RNTI_BitMaskIndex_b6,
      { "u-RNTI-BitMaskIndex-b6", "rrc.u_RNTI_BitMaskIndex_b6",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_26", HFILL }},
    { &hf_rrc_u_RNTI_BitMaskIndex_b7,
      { "u-RNTI-BitMaskIndex-b7", "rrc.u_RNTI_BitMaskIndex_b7",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_25", HFILL }},
    { &hf_rrc_u_RNTI_BitMaskIndex_b8,
      { "u-RNTI-BitMaskIndex-b8", "rrc.u_RNTI_BitMaskIndex_b8",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_24", HFILL }},
    { &hf_rrc_u_RNTI_BitMaskIndex_b9,
      { "u-RNTI-BitMaskIndex-b9", "rrc.u_RNTI_BitMaskIndex_b9",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_23", HFILL }},
    { &hf_rrc_u_RNTI_BitMaskIndex_b10,
      { "u-RNTI-BitMaskIndex-b10", "rrc.u_RNTI_BitMaskIndex_b10",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_22", HFILL }},
    { &hf_rrc_u_RNTI_BitMaskIndex_b11,
      { "u-RNTI-BitMaskIndex-b11", "rrc.u_RNTI_BitMaskIndex_b11",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_21", HFILL }},
    { &hf_rrc_u_RNTI_BitMaskIndex_b12,
      { "u-RNTI-BitMaskIndex-b12", "rrc.u_RNTI_BitMaskIndex_b12",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_20", HFILL }},
    { &hf_rrc_u_RNTI_BitMaskIndex_b13,
      { "u-RNTI-BitMaskIndex-b13", "rrc.u_RNTI_BitMaskIndex_b13",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_19", HFILL }},
    { &hf_rrc_u_RNTI_BitMaskIndex_b14,
      { "u-RNTI-BitMaskIndex-b14", "rrc.u_RNTI_BitMaskIndex_b14",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_18", HFILL }},
    { &hf_rrc_u_RNTI_BitMaskIndex_b15,
      { "u-RNTI-BitMaskIndex-b15", "rrc.u_RNTI_BitMaskIndex_b15",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_17", HFILL }},
    { &hf_rrc_u_RNTI_BitMaskIndex_b16,
      { "u-RNTI-BitMaskIndex-b16", "rrc.u_RNTI_BitMaskIndex_b16",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_16", HFILL }},
    { &hf_rrc_u_RNTI_BitMaskIndex_b17,
      { "u-RNTI-BitMaskIndex-b17", "rrc.u_RNTI_BitMaskIndex_b17",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_15", HFILL }},
    { &hf_rrc_u_RNTI_BitMaskIndex_b18,
      { "u-RNTI-BitMaskIndex-b18", "rrc.u_RNTI_BitMaskIndex_b18",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_14", HFILL }},
    { &hf_rrc_u_RNTI_BitMaskIndex_b19,
      { "u-RNTI-BitMaskIndex-b19", "rrc.u_RNTI_BitMaskIndex_b19",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_13", HFILL }},
    { &hf_rrc_u_RNTI_BitMaskIndex_b20,
      { "u-RNTI-BitMaskIndex-b20", "rrc.u_RNTI_BitMaskIndex_b20",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_12", HFILL }},
    { &hf_rrc_u_RNTI_BitMaskIndex_b21,
      { "u-RNTI-BitMaskIndex-b21", "rrc.u_RNTI_BitMaskIndex_b21",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_11", HFILL }},
    { &hf_rrc_u_RNTI_BitMaskIndex_b22,
      { "u-RNTI-BitMaskIndex-b22", "rrc.u_RNTI_BitMaskIndex_b22",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_10", HFILL }},
    { &hf_rrc_u_RNTI_BitMaskIndex_b23,
      { "u-RNTI-BitMaskIndex-b23", "rrc.u_RNTI_BitMaskIndex_b23",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_9", HFILL }},
    { &hf_rrc_u_RNTI_BitMaskIndex_b24,
      { "u-RNTI-BitMaskIndex-b24", "rrc.u_RNTI_BitMaskIndex_b24",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_8", HFILL }},
    { &hf_rrc_u_RNTI_BitMaskIndex_b25,
      { "u-RNTI-BitMaskIndex-b25", "rrc.u_RNTI_BitMaskIndex_b25",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_7", HFILL }},
    { &hf_rrc_u_RNTI_BitMaskIndex_b26,
      { "u-RNTI-BitMaskIndex-b26", "rrc.u_RNTI_BitMaskIndex_b26",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_6", HFILL }},
    { &hf_rrc_u_RNTI_BitMaskIndex_b27,
      { "u-RNTI-BitMaskIndex-b27", "rrc.u_RNTI_BitMaskIndex_b27",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_5", HFILL }},
    { &hf_rrc_u_RNTI_BitMaskIndex_b28,
      { "u-RNTI-BitMaskIndex-b28", "rrc.u_RNTI_BitMaskIndex_b28",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_4", HFILL }},
    { &hf_rrc_u_RNTI_BitMaskIndex_b29,
      { "u-RNTI-BitMaskIndex-b29", "rrc.u_RNTI_BitMaskIndex_b29",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_3", HFILL }},
    { &hf_rrc_u_RNTI_BitMaskIndex_b30,
      { "u-RNTI-BitMaskIndex-b30", "rrc.u_RNTI_BitMaskIndex_b30",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_2", HFILL }},
    { &hf_rrc_u_RNTI_BitMaskIndex_b31,
      { "u-RNTI-BitMaskIndex-b31", "rrc.u_RNTI_BitMaskIndex_b31",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_1", HFILL }},
    { &hf_rrc_s_RNTI_2,
      { "s-RNTI-2", "rrc.s_RNTI_2",
        FT_BYTES, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_supportOfLoggedMeasurementsIdlePCH,
      { "supportOfLoggedMeasurementsIdlePCH", "rrc.supportOfLoggedMeasurementsIdlePCH",
        FT_UINT32, BASE_DEC, VALS(rrc_T_supportOfLoggedMeasurementsIdlePCH_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_ue_RadioAccessCapability_v690ext,
      { "ue-RadioAccessCapability-v690ext", "rrc.ue_RadioAccessCapability_v690ext",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_ue_RATSpecificCapability_v690ext,
      { "ue-RATSpecificCapability-v690ext", "rrc.ue_RATSpecificCapability_v690ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "InterRAT_UE_RadioAccessCapability_v690ext", HFILL }},
    { &hf_rrc_v6b0NonCriticalExtensions_15,
      { "v6b0NonCriticalExtensions", "rrc.v6b0NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v6b0NonCriticalExtensions_14", HFILL }},
    { &hf_rrc_ue_RadioAccessCapability_v6b0ext,
      { "ue-RadioAccessCapability-v6b0ext", "rrc.ue_RadioAccessCapability_v6b0ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "UE_RadioAccessCapability_v6b0ext_IEs", HFILL }},
    { &hf_rrc_v6e0NonCriticalExtensions_02,
      { "v6e0NonCriticalExtensions", "rrc.v6e0NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v6e0NonCriticalExtensions_02", HFILL }},
    { &hf_rrc_ue_RadioAccessCapability_v6e0ext,
      { "ue-RadioAccessCapability-v6e0ext", "rrc.ue_RadioAccessCapability_v6e0ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "UE_RadioAccessCapability_v6e0ext_IEs", HFILL }},
    { &hf_rrc_v770NonCriticalExtensions_39,
      { "v770NonCriticalExtensions", "rrc.v770NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v770NonCriticalExtensions_39", HFILL }},
    { &hf_rrc_ue_RadioAccessCapability_v770ext,
      { "ue-RadioAccessCapability-v770ext", "rrc.ue_RadioAccessCapability_v770ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "UE_RadioAccessCapability_v770ext_IEs", HFILL }},
    { &hf_rrc_v790NonCriticalExtensions_01,
      { "v790NonCriticalExtensions", "rrc.v790NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v790NonCriticalExtensions_01", HFILL }},
    { &hf_rrc_ue_RadioAccessCapability_v790ext,
      { "ue-RadioAccessCapability-v790ext", "rrc.ue_RadioAccessCapability_v790ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "UE_RadioAccessCapability_v790ext_IEs", HFILL }},
    { &hf_rrc_v860NonCriticalExtensions_25,
      { "v860NonCriticalExtensions", "rrc.v860NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v860NonCriticalExtensions_25", HFILL }},
    { &hf_rrc_ue_RadioAccessCapability_v860ext,
      { "ue-RadioAccessCapability-v860ext", "rrc.ue_RadioAccessCapability_v860ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "UE_RadioAccessCapability_v860ext_IEs", HFILL }},
    { &hf_rrc_ue_RATSpecificCapability_v860ext,
      { "ue-RATSpecificCapability-v860ext", "rrc.ue_RATSpecificCapability_v860ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "InterRAT_UE_RadioAccessCapability_v860ext", HFILL }},
    { &hf_rrc_v880NonCriticalExtensions_01,
      { "v880NonCriticalExtensions", "rrc.v880NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v880NonCriticalExtensions_01", HFILL }},
    { &hf_rrc_ue_RadioAccessCapability_v880ext,
      { "ue-RadioAccessCapability-v880ext", "rrc.ue_RadioAccessCapability_v880ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "UE_RadioAccessCapability_v880ext_IEs", HFILL }},
    { &hf_rrc_v890NonCriticalExtensions_10,
      { "v890NonCriticalExtensions", "rrc.v890NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v890NonCriticalExtensions_10", HFILL }},
    { &hf_rrc_ue_RadioAccessCapability_v890ext,
      { "ue-RadioAccessCapability-v890ext", "rrc.ue_RadioAccessCapability_v890ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "UE_RadioAccessCapability_v890ext_IEs", HFILL }},
    { &hf_rrc_v920NonCriticalExtensions_03,
      { "v920NonCriticalExtensions", "rrc.v920NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v920NonCriticalExtensions_03", HFILL }},
    { &hf_rrc_ue_RadioAccessCapability_v920ext,
      { "ue-RadioAccessCapability-v920ext", "rrc.ue_RadioAccessCapability_v920ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "UE_RadioAccessCapability_v920ext_IEs", HFILL }},
    { &hf_rrc_v970NonCriticalExtensions_03,
      { "v970NonCriticalExtensions", "rrc.v970NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v970NonCriticalExtensions_03", HFILL }},
    { &hf_rrc_ue_RadioAccessCapability_v970ext,
      { "ue-RadioAccessCapability-v970ext", "rrc.ue_RadioAccessCapability_v970ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "UE_RadioAccessCapability_v970ext_IEs", HFILL }},
    { &hf_rrc_va40NonCriticalExtensions_18,
      { "va40NonCriticalExtensions", "rrc.va40NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_va40NonCriticalExtensions_17", HFILL }},
    { &hf_rrc_ue_RadioAccessCapability_va40ext,
      { "ue-RadioAccessCapability-va40ext", "rrc.ue_RadioAccessCapability_va40ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "UE_RadioAccessCapability_va40ext_IEs", HFILL }},
    { &hf_rrc_va60NonCriticalExtensions,
      { "va60NonCriticalExtensions", "rrc.va60NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_ue_RadioAccessCapability_va60ext,
      { "ue-RadioAccessCapability-va60ext", "rrc.ue_RadioAccessCapability_va60ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "UE_RadioAccessCapability_va60ext_IEs", HFILL }},
    { &hf_rrc_va80NonCriticalExtensions_01,
      { "va80NonCriticalExtensions", "rrc.va80NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_va80NonCriticalExtensions_01", HFILL }},
    { &hf_rrc_ue_RadioAccessCapability_va80ext,
      { "ue-RadioAccessCapability-va80ext", "rrc.ue_RadioAccessCapability_va80ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "UE_RadioAccessCapability_va80ext_IEs", HFILL }},
    { &hf_rrc_laterNonCriticalExtensions_67,
      { "laterNonCriticalExtensions", "rrc.laterNonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "UE_RadioAccessCapability_LaterNonCriticalExtensions", HFILL }},
    { &hf_rrc_supportForSIB11bis_01,
      { "supportForSIB11bis", "rrc.supportForSIB11bis",
        FT_UINT32, BASE_DEC, VALS(rrc_T_supportForSIB11bis_01_vals), 0,
        "T_supportForSIB11bis_01", HFILL }},
    { &hf_rrc_supportForFDPCH_02,
      { "supportForFDPCH", "rrc.supportForFDPCH",
        FT_UINT32, BASE_DEC, VALS(rrc_T_supportForFDPCH_02_vals), 0,
        "T_supportForFDPCH_02", HFILL }},
    { &hf_rrc_pdcp_Capability_01,
      { "pdcp-Capability", "rrc.pdcp_Capability",
        FT_NONE, BASE_NONE, NULL, 0,
        "PDCP_Capability_v770ext", HFILL }},
    { &hf_rrc_rlc_Capability,
      { "rlc-Capability", "rrc.rlc_Capability",
        FT_NONE, BASE_NONE, NULL, 0,
        "RLC_Capability_v770ext", HFILL }},
    { &hf_rrc_rf_Capability,
      { "rf-Capability", "rrc.rf_Capability",
        FT_NONE, BASE_NONE, NULL, 0,
        "RF_Capability_v770ext", HFILL }},
    { &hf_rrc_physicalChannelCapability,
      { "physicalChannelCapability", "rrc.physicalChannelCapability",
        FT_NONE, BASE_NONE, NULL, 0,
        "PhysicalChannelCapability_v770ext", HFILL }},
    { &hf_rrc_multiModeRAT_Capability,
      { "multiModeRAT-Capability", "rrc.multiModeRAT_Capability",
        FT_NONE, BASE_NONE, NULL, 0,
        "MultiModeRAT_Capability_v770ext", HFILL }},
    { &hf_rrc_ue_PositioningCapability,
      { "ue-PositioningCapability", "rrc.ue_PositioningCapability",
        FT_NONE, BASE_NONE, NULL, 0,
        "UE_PositioningCapability_v770ext", HFILL }},
    { &hf_rrc_mac_ehsSupport_01,
      { "mac-ehsSupport", "rrc.mac_ehsSupport",
        FT_UINT32, BASE_DEC, VALS(rrc_T_mac_ehsSupport_01_vals), 0,
        "T_mac_ehsSupport_01", HFILL }},
    { &hf_rrc_ue_specificCapabilityInformation,
      { "ue-specificCapabilityInformation", "rrc.ue_specificCapabilityInformation",
        FT_UINT32, BASE_DEC, VALS(rrc_UE_SpecificCapabilityInformation_LCRTDD_vals), 0,
        "UE_SpecificCapabilityInformation_LCRTDD", HFILL }},
    { &hf_rrc_supportForEDPCCHPowerBoosting_01,
      { "supportForEDPCCHPowerBoosting", "rrc.supportForEDPCCHPowerBoosting",
        FT_UINT32, BASE_DEC, VALS(rrc_T_supportForEDPCCHPowerBoosting_01_vals), 0,
        "T_supportForEDPCCHPowerBoosting_01", HFILL }},
    { &hf_rrc_rf_Capability_01,
      { "rf-Capability", "rrc.rf_Capability",
        FT_NONE, BASE_NONE, NULL, 0,
        "RF_Capability_v860ext", HFILL }},
    { &hf_rrc_physicalChannelCapability_01,
      { "physicalChannelCapability", "rrc.physicalChannelCapability",
        FT_NONE, BASE_NONE, NULL, 0,
        "PhysicalChannelCapability_v860ext", HFILL }},
    { &hf_rrc_multiModeRAT_Capability_01,
      { "multiModeRAT-Capability", "rrc.multiModeRAT_Capability",
        FT_NONE, BASE_NONE, NULL, 0,
        "MultiModeRAT_Capability_v860ext", HFILL }},
    { &hf_rrc_ue_PositioningCapability_01,
      { "ue-PositioningCapability", "rrc.ue_PositioningCapability",
        FT_NONE, BASE_NONE, NULL, 0,
        "UE_PositioningCapability_v860ext", HFILL }},
    { &hf_rrc_measurementCapability,
      { "measurementCapability", "rrc.measurementCapability",
        FT_NONE, BASE_NONE, NULL, 0,
        "MeasurementCapability_v860ext", HFILL }},
    { &hf_rrc_measurementCapabilityTDD,
      { "measurementCapabilityTDD", "rrc.measurementCapabilityTDD",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_supportOfCommonEDCH_03,
      { "supportOfCommonEDCH", "rrc.supportOfCommonEDCH",
        FT_UINT32, BASE_DEC, VALS(rrc_T_supportOfCommonEDCH_03_vals), 0,
        "T_supportOfCommonEDCH_03", HFILL }},
    { &hf_rrc_supportOfMACiis_03,
      { "supportOfMACiis", "rrc.supportOfMACiis",
        FT_UINT32, BASE_DEC, VALS(rrc_T_supportOfMACiis_03_vals), 0,
        "T_supportOfMACiis_03", HFILL }},
    { &hf_rrc_supportOfSPSOperation_02,
      { "supportOfSPSOperation", "rrc.supportOfSPSOperation",
        FT_UINT32, BASE_DEC, VALS(rrc_T_supportOfSPSOperation_02_vals), 0,
        "T_supportOfSPSOperation_02", HFILL }},
    { &hf_rrc_supportOfControlChannelDRXOperation_01,
      { "supportOfControlChannelDRXOperation", "rrc.supportOfControlChannelDRXOperation",
        FT_UINT32, BASE_DEC, VALS(rrc_T_supportOfControlChannelDRXOperation_01_vals), 0,
        "T_supportOfControlChannelDRXOperation_01", HFILL }},
    { &hf_rrc_supportOfCSG_01,
      { "supportOfCSG", "rrc.supportOfCSG",
        FT_UINT32, BASE_DEC, VALS(rrc_T_supportOfCSG_01_vals), 0,
        "T_supportOfCSG_01", HFILL }},
    { &hf_rrc_ue_RadioAccessCapabBandFDDList3,
      { "ue-RadioAccessCapabBandFDDList3", "rrc.ue_RadioAccessCapabBandFDDList3",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_supportForPriorityReselectionInUTRAN,
      { "supportForPriorityReselectionInUTRAN", "rrc.supportForPriorityReselectionInUTRAN",
        FT_UINT32, BASE_DEC, VALS(rrc_T_supportForPriorityReselectionInUTRAN_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_supportCellSpecificTxDiversityinDC_Operation,
      { "supportCellSpecificTxDiversityinDC-Operation", "rrc.supportCellSpecificTxDiversityinDC_Operation",
        FT_UINT32, BASE_DEC, VALS(rrc_T_supportCellSpecificTxDiversityinDC_Operation_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_ue_RadioAccessCapabBandCombList,
      { "ue-RadioAccessCapabBandCombList", "rrc.ue_RadioAccessCapabBandCombList",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_physicalChannelCapability_02,
      { "physicalChannelCapability", "rrc.physicalChannelCapability",
        FT_NONE, BASE_NONE, NULL, 0,
        "PhysicalChannelCapability_v920ext", HFILL }},
    { &hf_rrc_rlc_Capability_01,
      { "rlc-Capability", "rrc.rlc_Capability",
        FT_NONE, BASE_NONE, NULL, 0,
        "RLC_Capability_v920ext", HFILL }},
    { &hf_rrc_supportOfenhancedTS0,
      { "supportOfenhancedTS0", "rrc.supportOfenhancedTS0",
        FT_UINT32, BASE_DEC, VALS(rrc_T_supportOfenhancedTS0_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_measurementCapability_01,
      { "measurementCapability", "rrc.measurementCapability",
        FT_NONE, BASE_NONE, NULL, 0,
        "MeasurementCapability_v920ext", HFILL }},
    { &hf_rrc_csgProximityIndicationCapability,
      { "csgProximityIndicationCapability", "rrc.csgProximityIndicationCapability",
        FT_NONE, BASE_NONE, NULL, 0,
        "CSG_ProximityIndicationCapability", HFILL }},
    { &hf_rrc_neighCellSI_AcquisitionCapability,
      { "neighCellSI-AcquisitionCapability", "rrc.neighCellSI_AcquisitionCapability",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_ue_ExtendedMeasurementSupport,
      { "ue-ExtendedMeasurementSupport", "rrc.ue_ExtendedMeasurementSupport",
        FT_UINT32, BASE_DEC, VALS(rrc_T_ue_ExtendedMeasurementSupport_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_voiceOverUTRA_PS_HS_Support,
      { "voiceOverUTRA-PS-HS-Support", "rrc.voiceOverUTRA_PS_HS_Support",
        FT_UINT32, BASE_DEC, VALS(rrc_T_voiceOverUTRA_PS_HS_Support_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_srvcc_SupportFromUTRA_to_UTRA,
      { "srvcc-SupportFromUTRA-to-UTRA", "rrc.srvcc_SupportFromUTRA_to_UTRA",
        FT_UINT32, BASE_DEC, VALS(rrc_T_srvcc_SupportFromUTRA_to_UTRA_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_srvcc_SupportFromUTRA_to_GERAN,
      { "srvcc-SupportFromUTRA-to-GERAN", "rrc.srvcc_SupportFromUTRA_to_GERAN",
        FT_UINT32, BASE_DEC, VALS(rrc_T_srvcc_SupportFromUTRA_to_GERAN_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_ue_supportMac_ehsWindowSizeExtension,
      { "ue-supportMac-ehsWindowSizeExtension", "rrc.ue_supportMac_ehsWindowSizeExtension",
        FT_UINT32, BASE_DEC, VALS(rrc_T_ue_supportMac_ehsWindowSizeExtension_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_v9c0NonCriticalExtensions_12,
      { "v9c0NonCriticalExtensions", "rrc.v9c0NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v9c0NonCriticalExtensions_12", HFILL }},
    { &hf_rrc_ue_RadioAccessCapability_v9c0ext,
      { "ue-RadioAccessCapability-v9c0ext", "rrc.ue_RadioAccessCapability_v9c0ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "UE_RadioAccessCapability_v9c0ext_IEs", HFILL }},
    { &hf_rrc_vaa0NonCriticalExtensions_02,
      { "vaa0NonCriticalExtensions", "rrc.vaa0NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_vaa0NonCriticalExtensions_02", HFILL }},
    { &hf_rrc_ue_RadioAccessCapability_vaa0ext,
      { "ue-RadioAccessCapability-vaa0ext", "rrc.ue_RadioAccessCapability_vaa0ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "UE_RadioAccessCapability_vaa0ext_IEs", HFILL }},
    { &hf_rrc_vbxyNonCriticalExtensions_18,
      { "vbxyNonCriticalExtensions", "rrc.vbxyNonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_vbxyNonCriticalExtensions_18", HFILL }},
    { &hf_rrc_ue_RadioAccessCapability_vbxyext,
      { "ue-RadioAccessCapability-vbxyext", "rrc.ue_RadioAccessCapability_vbxyext",
        FT_NONE, BASE_NONE, NULL, 0,
        "UE_RadioAccessCapability_vbxyext_IEs", HFILL }},
    { &hf_rrc_nonCriticalExtensions_194,
      { "nonCriticalExtensions", "rrc.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_nonCriticalExtensions_194", HFILL }},
    { &hf_rrc_ue_RadioAccessCapabBandCombList_va40ext,
      { "ue-RadioAccessCapabBandCombList-va40ext", "rrc.ue_RadioAccessCapabBandCombList_va40ext",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_physicalChannelCapability_03,
      { "physicalChannelCapability", "rrc.physicalChannelCapability",
        FT_NONE, BASE_NONE, NULL, 0,
        "PhysicalChannelCapability_va40ext", HFILL }},
    { &hf_rrc_rlc_Capability_02,
      { "rlc-Capability", "rrc.rlc_Capability",
        FT_NONE, BASE_NONE, NULL, 0,
        "RLC_Capability_va40ext", HFILL }},
    { &hf_rrc_measurementCapability_02,
      { "measurementCapability", "rrc.measurementCapability",
        FT_NONE, BASE_NONE, NULL, 0,
        "MeasurementCapability_va40ext", HFILL }},
    { &hf_rrc_supportOfMUMIMO,
      { "supportOfMUMIMO", "rrc.supportOfMUMIMO",
        FT_UINT32, BASE_DEC, VALS(rrc_T_supportOfMUMIMO_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_supportForDualCellMimoInDifferentBands,
      { "supportForDualCellMimoInDifferentBands", "rrc.supportForDualCellMimoInDifferentBands",
        FT_UINT32, BASE_DEC, VALS(rrc_T_supportForDualCellMimoInDifferentBands_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_ue_BasedNetworkPerformanceMeasurementsParameters,
      { "ue-BasedNetworkPerformanceMeasurementsParameters", "rrc.ue_BasedNetworkPerformanceMeasurementsParameters",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_supportOfUTRANANR,
      { "supportOfUTRANANR", "rrc.supportOfUTRANANR",
        FT_UINT32, BASE_DEC, VALS(rrc_T_supportOfUTRANANR_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_ue_RadioAccessCapabBandFDDList4,
      { "ue-RadioAccessCapabBandFDDList4", "rrc.ue_RadioAccessCapabBandFDDList4",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_ue_RadioAccessCapabBandFDDList4_01,
      { "ue-RadioAccessCapabBandFDDList4", "rrc.ue_RadioAccessCapabBandFDDList4",
        FT_UINT32, BASE_DEC, NULL, 0,
        "UE_RadioAccessCapabBandFDDList4_va60ext", HFILL }},
    { &hf_rrc_ue_RadioAccessCapabBandFDDList4_02,
      { "ue-RadioAccessCapabBandFDDList4", "rrc.ue_RadioAccessCapabBandFDDList4",
        FT_UINT32, BASE_DEC, NULL, 0,
        "UE_RadioAccessCapabBandFDDList4_va80ext", HFILL }},
    { &hf_rrc_rf_capability,
      { "rf-capability", "rrc.rf_capability",
        FT_NONE, BASE_NONE, NULL, 0,
        "RF_Capability_vaa0ext", HFILL }},
    { &hf_rrc_supportOfUMRLCre_establish_via_reconfig,
      { "supportOfUMRLCre-establish-via-reconfig", "rrc.supportOfUMRLCre_establish_via_reconfig",
        FT_UINT32, BASE_DEC, VALS(rrc_T_supportOfUMRLCre_establish_via_reconfig_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_multiModeRAT_Capability_02,
      { "multiModeRAT-Capability", "rrc.multiModeRAT_Capability",
        FT_NONE, BASE_NONE, NULL, 0,
        "MultiModeRAT_Capability_vbxyext", HFILL }},
    { &hf_rrc_supportOfCommonERGCH,
      { "supportOfCommonERGCH", "rrc.supportOfCommonERGCH",
        FT_UINT32, BASE_DEC, VALS(rrc_T_supportOfCommonERGCH_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_supportOfFallbackR99PRACH,
      { "supportOfFallbackR99PRACH", "rrc.supportOfFallbackR99PRACH",
        FT_UINT32, BASE_DEC, VALS(rrc_T_supportOfFallbackR99PRACH_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_supportOfConcurrent2ms10msTTI,
      { "supportOfConcurrent2ms10msTTI", "rrc.supportOfConcurrent2ms10msTTI",
        FT_UINT32, BASE_DEC, VALS(rrc_T_supportOfConcurrent2ms10msTTI_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_supportOfTTIalignmentAndPerHARQActivation,
      { "supportOfTTIalignmentAndPerHARQActivation", "rrc.supportOfTTIalignmentAndPerHARQActivation",
        FT_UINT32, BASE_DEC, VALS(rrc_T_supportOfTTIalignmentAndPerHARQActivation_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_ue_RadioAccessCapabBandCombList_vbxyext,
      { "ue-RadioAccessCapabBandCombList-vbxyext", "rrc.ue_RadioAccessCapabBandCombList_vbxyext",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_multiflowBandCombList,
      { "multiflowBandCombList", "rrc.multiflowBandCombList",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_physicalChannelCapability_04,
      { "physicalChannelCapability", "rrc.physicalChannelCapability",
        FT_NONE, BASE_NONE, NULL, 0,
        "PhysicalChannelCapability_vbxyext", HFILL }},
    { &hf_rrc_rlc_Capability_03,
      { "rlc-Capability", "rrc.rlc_Capability",
        FT_NONE, BASE_NONE, NULL, 0,
        "RLC_Capability_vbxyext", HFILL }},
    { &hf_rrc_multiflowCapability,
      { "multiflowCapability", "rrc.multiflowCapability",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_ue_RadioAccessCapabBandFDDList6,
      { "ue-RadioAccessCapabBandFDDList6", "rrc.ue_RadioAccessCapabBandFDDList6",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_measurementCapability_03,
      { "measurementCapability", "rrc.measurementCapability",
        FT_NONE, BASE_NONE, NULL, 0,
        "MeasurementCapability_vbxyext", HFILL }},
    { &hf_rrc_rsrvcc_SupportFromUTRA_CS_HS_to_EUTRA_FDD,
      { "rsrvcc-SupportFromUTRA-CS-HS-to-EUTRA-FDD", "rrc.rsrvcc_SupportFromUTRA_CS_HS_to_EUTRA_FDD",
        FT_UINT32, BASE_DEC, VALS(rrc_T_rsrvcc_SupportFromUTRA_CS_HS_to_EUTRA_FDD_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_rsrvcc_SupportFromUTRA_CS_HS_to_EUTRA_TDD,
      { "rsrvcc-SupportFromUTRA-CS-HS-to-EUTRA-TDD", "rrc.rsrvcc_SupportFromUTRA_CS_HS_to_EUTRA_TDD",
        FT_UINT32, BASE_DEC, VALS(rrc_T_rsrvcc_SupportFromUTRA_CS_HS_to_EUTRA_TDD_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_t_301,
      { "t-301", "rrc.t_301",
        FT_UINT32, BASE_DEC, VALS(rrc_T_301_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_n_301,
      { "n-301", "rrc.n_301",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_t_302,
      { "t-302", "rrc.t_302",
        FT_UINT32, BASE_DEC, VALS(rrc_T_302_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_n_302,
      { "n-302", "rrc.n_302",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_t_304,
      { "t-304", "rrc.t_304",
        FT_UINT32, BASE_DEC, VALS(rrc_T_304_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_n_304,
      { "n-304", "rrc.n_304",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_t_305,
      { "t-305", "rrc.t_305",
        FT_UINT32, BASE_DEC, VALS(rrc_T_305_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_t_307,
      { "t-307", "rrc.t_307",
        FT_UINT32, BASE_DEC, VALS(rrc_T_307_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_t_308,
      { "t-308", "rrc.t_308",
        FT_UINT32, BASE_DEC, VALS(rrc_T_308_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_t_309,
      { "t-309", "rrc.t_309",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_t_310,
      { "t-310", "rrc.t_310",
        FT_UINT32, BASE_DEC, VALS(rrc_T_310_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_n_310,
      { "n-310", "rrc.n_310",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_t_311,
      { "t-311", "rrc.t_311",
        FT_UINT32, BASE_DEC, VALS(rrc_T_311_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_t_312,
      { "t-312", "rrc.t_312",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_n_312,
      { "n-312", "rrc.n_312",
        FT_UINT32, BASE_DEC, VALS(rrc_N_312_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_t_313,
      { "t-313", "rrc.t_313",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_n_313,
      { "n-313", "rrc.n_313",
        FT_UINT32, BASE_DEC, VALS(rrc_N_313_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_t_314,
      { "t-314", "rrc.t_314",
        FT_UINT32, BASE_DEC, VALS(rrc_T_314_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_t_315,
      { "t-315", "rrc.t_315",
        FT_UINT32, BASE_DEC, VALS(rrc_T_315_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_n_315,
      { "n-315", "rrc.n_315",
        FT_UINT32, BASE_DEC, VALS(rrc_N_315_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_t_316,
      { "t-316", "rrc.t_316",
        FT_UINT32, BASE_DEC, VALS(rrc_T_316_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_t_317,
      { "t-317", "rrc.t_317",
        FT_UINT32, BASE_DEC, VALS(rrc_T_317_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_n_312_01,
      { "n-312", "rrc.n_312",
        FT_UINT32, BASE_DEC, VALS(rrc_N_312ext_vals), 0,
        "N_312ext", HFILL }},
    { &hf_rrc_n_315_01,
      { "n-315", "rrc.n_315",
        FT_UINT32, BASE_DEC, VALS(rrc_N_315ext_vals), 0,
        "N_315ext", HFILL }},
    { &hf_rrc_n_312_02,
      { "n-312", "rrc.n_312",
        FT_UINT32, BASE_DEC, VALS(rrc_N_312_r5_vals), 0,
        "N_312_r5", HFILL }},
    { &hf_rrc_n_315_02,
      { "n-315", "rrc.n_315",
        FT_UINT32, BASE_DEC, VALS(rrc_N_315_r5_vals), 0,
        "N_315_r5", HFILL }},
    { &hf_rrc_t_323,
      { "t-323", "rrc.t_323",
        FT_UINT32, BASE_DEC, VALS(rrc_T_323_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_networkAssistedGANSS_supportedList,
      { "networkAssistedGANSS-supportedList", "rrc.networkAssistedGANSS_supportedList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "NetworkAssistedGANSS_Supported_List", HFILL }},
    { &hf_rrc_networkAssistedGANSS_supportedList_01,
      { "networkAssistedGANSS-supportedList", "rrc.networkAssistedGANSS_supportedList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "NetworkAssistedGANSS_Supported_List_v860ext", HFILL }},
    { &hf_rrc_t_300,
      { "t-300", "rrc.t_300",
        FT_UINT32, BASE_DEC, VALS(rrc_T_300_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_n_300,
      { "n-300", "rrc.n_300",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_multiRAT_CapabilityList,
      { "multiRAT-CapabilityList", "rrc.multiRAT_CapabilityList",
        FT_NONE, BASE_NONE, NULL, 0,
        "MultiRAT_Capability", HFILL }},
    { &hf_rrc_multiModeCapability,
      { "multiModeCapability", "rrc.multiModeCapability",
        FT_UINT32, BASE_DEC, VALS(rrc_MultiModeCapability_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_pdcp_Capability_02,
      { "pdcp-Capability", "rrc.pdcp_Capability",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_rlc_Capability_04,
      { "rlc-Capability", "rrc.rlc_Capability",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_transportChannelCapability,
      { "transportChannelCapability", "rrc.transportChannelCapability",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_rf_Capability_02,
      { "rf-Capability", "rrc.rf_Capability",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_physicalChannelCapability_05,
      { "physicalChannelCapability", "rrc.physicalChannelCapability",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_ue_MultiModeRAT_Capability,
      { "ue-MultiModeRAT-Capability", "rrc.ue_MultiModeRAT_Capability",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_ue_positioning_Capability,
      { "ue-positioning-Capability", "rrc.ue_positioning_Capability",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_measurementCapability_04,
      { "measurementCapability", "rrc.measurementCapability",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_ue_RadioAccessCapabBandFDDList,
      { "ue-RadioAccessCapabBandFDDList", "rrc.ue_RadioAccessCapabBandFDDList",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_ue_PositioningCapabilityExt_v380,
      { "ue-PositioningCapabilityExt-v380", "rrc.ue_PositioningCapabilityExt_v380",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_ue_PositioningCapabilityExt_v3a0,
      { "ue-PositioningCapabilityExt-v3a0", "rrc.ue_PositioningCapabilityExt_v3a0",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_ue_PositioningCapabilityExt_v3g0,
      { "ue-PositioningCapabilityExt-v3g0", "rrc.ue_PositioningCapabilityExt_v3g0",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_ue_RadioAccessCapabBandFDDList2,
      { "ue-RadioAccessCapabBandFDDList2", "rrc.ue_RadioAccessCapabBandFDDList2",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_ue_RadioAccessCapabBandFDDList_ext,
      { "ue-RadioAccessCapabBandFDDList-ext", "rrc.ue_RadioAccessCapabBandFDDList_ext",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_physicalchannelcapability_edch,
      { "physicalchannelcapability-edch", "rrc.physicalchannelcapability_edch",
        FT_NONE, BASE_NONE, NULL, 0,
        "PhysicalChannelCapability_edch_r6", HFILL }},
    { &hf_rrc_deviceType,
      { "deviceType", "rrc.deviceType",
        FT_UINT32, BASE_DEC, VALS(rrc_T_deviceType_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_supportForTwoDRXSchemesInPCH_02,
      { "supportForTwoDRXSchemesInPCH", "rrc.supportForTwoDRXSchemesInPCH",
        FT_UINT32, BASE_DEC, VALS(rrc_T_supportForTwoDRXSchemesInPCH_02_vals), 0,
        "T_supportForTwoDRXSchemesInPCH_02", HFILL }},
    { &hf_rrc_supportEDPDCHPowerInterpolation,
      { "supportEDPDCHPowerInterpolation", "rrc.supportEDPDCHPowerInterpolation",
        FT_UINT32, BASE_DEC, VALS(rrc_T_supportEDPDCHPowerInterpolation_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_supportofTxDivOnNonMIMOChannel,
      { "supportofTxDivOnNonMIMOChannel", "rrc.supportofTxDivOnNonMIMOChannel",
        FT_UINT32, BASE_DEC, VALS(rrc_T_supportofTxDivOnNonMIMOChannel_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_physicalChannelCapability_LCR,
      { "physicalChannelCapability-LCR", "rrc.physicalChannelCapability_LCR",
        FT_NONE, BASE_NONE, NULL, 0,
        "PhysicalChannelCapability_LCR_r4", HFILL }},
    { &hf_rrc_ue_RadioAccessCapabilityComp_TDD128,
      { "ue-RadioAccessCapabilityComp-TDD128", "rrc.ue_RadioAccessCapabilityComp_TDD128",
        FT_NONE, BASE_NONE, NULL, 0,
        "UE_RadioAccessCapabilityComp_TDD128_v7f0ext", HFILL }},
    { &hf_rrc_physicalChannelCapability_06,
      { "physicalChannelCapability", "rrc.physicalChannelCapability",
        FT_NONE, BASE_NONE, NULL, 0,
        "PhysicalChannelCapabilityInfo_v770ext", HFILL }},
    { &hf_rrc_ganssSupportIndication,
      { "ganssSupportIndication", "rrc.ganssSupportIndication",
        FT_UINT32, BASE_DEC, VALS(rrc_T_ganssSupportIndication_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_mac_ehsSupport_02,
      { "mac-ehsSupport", "rrc.mac_ehsSupport",
        FT_UINT32, BASE_DEC, VALS(rrc_T_mac_ehsSupport_02_vals), 0,
        "T_mac_ehsSupport_02", HFILL }},
    { &hf_rrc_ue_SpecificCapabilityInformation,
      { "ue-SpecificCapabilityInformation", "rrc.ue_SpecificCapabilityInformation",
        FT_UINT32, BASE_DEC, VALS(rrc_UE_SpecificCapabilityInformation_LCRTDD_vals), 0,
        "UE_SpecificCapabilityInformation_LCRTDD", HFILL }},
    { &hf_rrc_supportForPriorityReselectionInUTRAN_01,
      { "supportForPriorityReselectionInUTRAN", "rrc.supportForPriorityReselectionInUTRAN",
        FT_UINT32, BASE_DEC, VALS(rrc_T_supportForPriorityReselectionInUTRAN_01_vals), 0,
        "T_supportForPriorityReselectionInUTRAN_01", HFILL }},
    { &hf_rrc_supportOfenhancedTS0_01,
      { "supportOfenhancedTS0", "rrc.supportOfenhancedTS0",
        FT_UINT32, BASE_DEC, VALS(rrc_T_supportOfenhancedTS0_01_vals), 0,
        "T_supportOfenhancedTS0_01", HFILL }},
    { &hf_rrc_ue_RadioAccessCapabBandFDDList4_va60ext,
      { "ue-RadioAccessCapabBandFDDList4-va60ext", "rrc.ue_RadioAccessCapabBandFDDList4_va60ext",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_ue_RadioAccessCapabBandFDDList4_va80ext,
      { "ue-RadioAccessCapabBandFDDList4-va80ext", "rrc.ue_RadioAccessCapabBandFDDList4_va80ext",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_ue_RadioAccessCapabBandFDDList5,
      { "ue-RadioAccessCapabBandFDDList5", "rrc.ue_RadioAccessCapabBandFDDList5",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_ue_RadioAccessCapabBandFDDList_ext2,
      { "ue-RadioAccessCapabBandFDDList-ext2", "rrc.ue_RadioAccessCapabBandFDDList_ext2",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_UE_RadioAccessCapabBandFDDList2_item,
      { "UE-RadioAccessCapabBandFDD2", "rrc.UE_RadioAccessCapabBandFDD2",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_UE_RadioAccessCapabBandFDDList3_item,
      { "UE-RadioAccessCapabBandFDD3", "rrc.UE_RadioAccessCapabBandFDD3",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_UE_RadioAccessCapabBandFDDList4_item,
      { "UE-RadioAccessCapabBandFDD4", "rrc.UE_RadioAccessCapabBandFDD4",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_UE_RadioAccessCapabBandFDDList4_va60ext_item,
      { "UE-RadioAccessCapabBandFDD4-va60ext", "rrc.UE_RadioAccessCapabBandFDD4_va60ext",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_UE_RadioAccessCapabBandFDDList4_va80ext_item,
      { "UE-RadioAccessCapabBandFDD4-va80ext", "rrc.UE_RadioAccessCapabBandFDD4_va80ext",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_UE_RadioAccessCapabBandFDDList5_item,
      { "UE-RadioAccessCapabBandFDD5", "rrc.UE_RadioAccessCapabBandFDD5",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_UE_RadioAccessCapabBandFDDList6_item,
      { "UE-RadioAccessCapabBandFDD6", "rrc.UE_RadioAccessCapabBandFDD6",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_fddRF_Capability_01,
      { "fddRF-Capability", "rrc.fddRF_Capability",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_fddRF_Capability_01", HFILL }},
    { &hf_rrc_ue_PowerClass_01,
      { "ue-PowerClass", "rrc.ue_PowerClass",
        FT_UINT32, BASE_DEC, VALS(rrc_UE_PowerClassExt_vals), 0,
        "UE_PowerClassExt", HFILL }},
    { &hf_rrc_measurementCapability2,
      { "measurementCapability2", "rrc.measurementCapability2",
        FT_NONE, BASE_NONE, NULL, 0,
        "MeasurementCapabilityExt2", HFILL }},
    { &hf_rrc_measurementCapability3,
      { "measurementCapability3", "rrc.measurementCapability3",
        FT_NONE, BASE_NONE, NULL, 0,
        "MeasurementCapabilityExt3", HFILL }},
    { &hf_rrc_additionalSecondaryCells,
      { "additionalSecondaryCells", "rrc.additionalSecondaryCells",
        FT_UINT32, BASE_DEC, VALS(rrc_T_additionalSecondaryCells_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_nonContiguousMultiCellCombinationList,
      { "nonContiguousMultiCellCombinationList", "rrc.nonContiguousMultiCellCombinationList",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_supportofULOLTD,
      { "supportofULOLTD", "rrc.supportofULOLTD",
        FT_UINT32, BASE_DEC, VALS(rrc_T_supportofULOLTD_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_additionalSecondaryCells_01,
      { "additionalSecondaryCells", "rrc.additionalSecondaryCells",
        FT_UINT32, BASE_DEC, VALS(rrc_T_additionalSecondaryCells_01_vals), 0,
        "T_additionalSecondaryCells_01", HFILL }},
    { &hf_rrc_supportofULOLTD_01,
      { "supportofULOLTD", "rrc.supportofULOLTD",
        FT_UINT32, BASE_DEC, VALS(rrc_T_supportofULOLTD_01_vals), 0,
        "T_supportofULOLTD_01", HFILL }},
    { &hf_rrc_fddRF_Capability_02,
      { "fddRF-Capability", "rrc.fddRF_Capability",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_fddRF_Capability_02", HFILL }},
    { &hf_rrc_measurementCapability4,
      { "measurementCapability4", "rrc.measurementCapability4",
        FT_NONE, BASE_NONE, NULL, 0,
        "MeasurementCapabilityExt4", HFILL }},
    { &hf_rrc_additionalSecondaryCells2,
      { "additionalSecondaryCells2", "rrc.additionalSecondaryCells2",
        FT_UINT32, BASE_DEC, VALS(rrc_T_additionalSecondaryCells2_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_supportOfULCLTD,
      { "supportOfULCLTD", "rrc.supportOfULCLTD",
        FT_UINT32, BASE_DEC, VALS(rrc_T_supportOfULCLTD_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_multiflowCapabilityBand,
      { "multiflowCapabilityBand", "rrc.multiflowCapabilityBand",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_mimo4x4CapabilityBand,
      { "mimo4x4CapabilityBand", "rrc.mimo4x4CapabilityBand",
        FT_UINT32, BASE_DEC, VALS(rrc_T_mimo4x4CapabilityBand_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_supportOfULMIMO,
      { "supportOfULMIMO", "rrc.supportOfULMIMO",
        FT_UINT32, BASE_DEC, VALS(rrc_T_supportOfULMIMO_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_rx_tx_TimeDifferenceType2Capable,
      { "rx-tx-TimeDifferenceType2Capable", "rrc.rx_tx_TimeDifferenceType2Capable",
        FT_BOOLEAN, BASE_NONE, NULL, 0,
        "BOOLEAN", HFILL }},
    { &hf_rrc_validity_CellPCH_UraPCH,
      { "validity-CellPCH-UraPCH", "rrc.validity_CellPCH_UraPCH",
        FT_UINT32, BASE_DEC, VALS(rrc_T_validity_CellPCH_UraPCH_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_sfn_sfnType2Capability,
      { "sfn-sfnType2Capability", "rrc.sfn_sfnType2Capability",
        FT_UINT32, BASE_DEC, VALS(rrc_T_sfn_sfnType2Capability_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_ue_GANSSPositioning_Capability,
      { "ue-GANSSPositioning-Capability", "rrc.ue_GANSSPositioning_Capability",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_ue_GANSSPositioning_Capability_v860ext,
      { "ue-GANSSPositioning-Capability-v860ext", "rrc.ue_GANSSPositioning_Capability_v860ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "UE_GANSSPositioning_Capability_v860ext_IEs", HFILL }},
    { &hf_rrc_UE_RadioAccessCapabBandCombList_item,
      { "BandComb", "rrc.BandComb",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_UE_RadioAccessCapabBandFDDList_item,
      { "UE-RadioAccessCapabBandFDD", "rrc.UE_RadioAccessCapabBandFDD",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_UE_RadioAccessCapabBandFDDList_ext_item,
      { "UE-RadioAccessCapabBandFDD-ext", "rrc.UE_RadioAccessCapabBandFDD_ext",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_UE_RadioAccessCapabBandFDDList_ext2_item,
      { "UE-RadioAccessCapabBandFDD-ext2", "rrc.UE_RadioAccessCapabBandFDD_ext2",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_fddRF_Capability_03,
      { "fddRF-Capability", "rrc.fddRF_Capability",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_fddRF_Capability_03", HFILL }},
    { &hf_rrc_measurementCapability_05,
      { "measurementCapability", "rrc.measurementCapability",
        FT_NONE, BASE_NONE, NULL, 0,
        "MeasurementCapabilityExt", HFILL }},
    { &hf_rrc_compressedModeMeasCapabFDDList_ext,
      { "compressedModeMeasCapabFDDList-ext", "rrc.compressedModeMeasCapabFDDList_ext",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_pdcp_Capability_r4_ext,
      { "pdcp-Capability-r4-ext", "rrc.pdcp_Capability_r4_ext",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_tdd_CapabilityExt,
      { "tdd-CapabilityExt", "rrc.tdd_CapabilityExt",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_rf_Capability_03,
      { "rf-Capability", "rrc.rf_Capability",
        FT_NONE, BASE_NONE, NULL, 0,
        "RF_Capability_r4_ext", HFILL }},
    { &hf_rrc_measurementCapability_r4_ext,
      { "measurementCapability-r4-ext", "rrc.measurementCapability_r4_ext",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_totalAM_RLCMemoryExceeds10kB,
      { "totalAM-RLCMemoryExceeds10kB", "rrc.totalAM_RLCMemoryExceeds10kB",
        FT_BOOLEAN, BASE_NONE, NULL, 0,
        "BOOLEAN", HFILL }},
    { &hf_rrc_rf_CapabilityComp,
      { "rf-CapabilityComp", "rrc.rf_CapabilityComp",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_rf_CapabilityFDDComp_01,
      { "rf-CapabilityFDDComp", "rrc.rf_CapabilityFDDComp",
        FT_UINT32, BASE_DEC, NULL, 0,
        "RF_CapabBandListFDDComp_ext", HFILL }},
    { &hf_rrc_hSDSCH_physical_layer_category,
      { "hSDSCH-physical-layer-category", "rrc.hSDSCH_physical_layer_category",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_tdd_edch_PhysicalLayerCategory,
      { "tdd-edch-PhysicalLayerCategory", "rrc.tdd_edch_PhysicalLayerCategory",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_1_6", HFILL }},
    { &hf_rrc_dl_CapabilityWithSimultaneousHS_DSCHConfig,
      { "dl-CapabilityWithSimultaneousHS-DSCHConfig", "rrc.dl_CapabilityWithSimultaneousHS_DSCHConfig",
        FT_UINT32, BASE_DEC, VALS(rrc_DL_CapabilityWithSimultaneousHS_DSCHConfig_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_supportForCSVoiceoverHSPA_02,
      { "supportForCSVoiceoverHSPA", "rrc.supportForCSVoiceoverHSPA",
        FT_UINT32, BASE_DEC, VALS(rrc_T_supportForCSVoiceoverHSPA_02_vals), 0,
        "T_supportForCSVoiceoverHSPA_02", HFILL }},
    { &hf_rrc_rf_CapabilityComp_01,
      { "rf-CapabilityComp", "rrc.rf_CapabilityComp",
        FT_NONE, BASE_NONE, NULL, 0,
        "RF_CapabilityComp_v770ext", HFILL }},
    { &hf_rrc_securityCapabilityIndication,
      { "securityCapabilityIndication", "rrc.securityCapabilityIndication",
        FT_UINT32, BASE_DEC, VALS(rrc_T_securityCapabilityIndication_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_ganssSupportIndication_01,
      { "ganssSupportIndication", "rrc.ganssSupportIndication",
        FT_UINT32, BASE_DEC, VALS(rrc_T_ganssSupportIndication_01_vals), 0,
        "T_ganssSupportIndication_01", HFILL }},
    { &hf_rrc_mac_ehsSupport_03,
      { "mac-ehsSupport", "rrc.mac_ehsSupport",
        FT_UINT32, BASE_DEC, VALS(rrc_T_mac_ehsSupport_03_vals), 0,
        "T_mac_ehsSupport_03", HFILL }},
    { &hf_rrc_fddPhysicalChannelCapab_hspdsch_edch,
      { "fddPhysicalChannelCapab-hspdsch-edch", "rrc.fddPhysicalChannelCapab_hspdsch_edch",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_physicalChannelCapabComp_hspdsch_r6,
      { "physicalChannelCapabComp-hspdsch-r6", "rrc.physicalChannelCapabComp_hspdsch_r6",
        FT_UINT32, BASE_DEC, NULL, 0,
        "HSDSCH_physical_layer_category", HFILL }},
    { &hf_rrc_physicalChannelCapability_edch_r6,
      { "physicalChannelCapability-edch-r6", "rrc.physicalChannelCapability_edch_r6",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_edch_PhysicalLayerCategory_extension,
      { "edch-PhysicalLayerCategory-extension", "rrc.edch_PhysicalLayerCategory_extension",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_7", HFILL }},
    { &hf_rrc_hsscchlessHsdschOperation_01,
      { "hsscchlessHsdschOperation", "rrc.hsscchlessHsdschOperation",
        FT_UINT32, BASE_DEC, VALS(rrc_T_hsscchlessHsdschOperation_01_vals), 0,
        "T_hsscchlessHsdschOperation_01", HFILL }},
    { &hf_rrc_enhancedFdpch_01,
      { "enhancedFdpch", "rrc.enhancedFdpch",
        FT_UINT32, BASE_DEC, VALS(rrc_T_enhancedFdpch_01_vals), 0,
        "T_enhancedFdpch_01", HFILL }},
    { &hf_rrc_hsdschReception_CellFach_01,
      { "hsdschReception-CellFach", "rrc.hsdschReception_CellFach",
        FT_UINT32, BASE_DEC, VALS(rrc_T_hsdschReception_CellFach_01_vals), 0,
        "T_hsdschReception_CellFach_01", HFILL }},
    { &hf_rrc_hsdschReception_CellUraPch_01,
      { "hsdschReception-CellUraPch", "rrc.hsdschReception_CellUraPch",
        FT_UINT32, BASE_DEC, VALS(rrc_T_hsdschReception_CellUraPch_01_vals), 0,
        "T_hsdschReception_CellUraPch_01", HFILL }},
    { &hf_rrc_discontinuousDpcchTransmission_01,
      { "discontinuousDpcchTransmission", "rrc.discontinuousDpcchTransmission",
        FT_UINT32, BASE_DEC, VALS(rrc_T_discontinuousDpcchTransmission_01_vals), 0,
        "T_discontinuousDpcchTransmission_01", HFILL }},
    { &hf_rrc_slotFormat4,
      { "slotFormat4", "rrc.slotFormat4",
        FT_UINT32, BASE_DEC, VALS(rrc_T_slotFormat4_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_supportofTxDivOnNonMIMOChannel_01,
      { "supportofTxDivOnNonMIMOChannel", "rrc.supportofTxDivOnNonMIMOChannel",
        FT_UINT32, BASE_DEC, VALS(rrc_T_supportofTxDivOnNonMIMOChannel_01_vals), 0,
        "T_supportofTxDivOnNonMIMOChannel_01", HFILL }},
    { &hf_rrc_supportOfMACiis_04,
      { "supportOfMACiis", "rrc.supportOfMACiis",
        FT_UINT32, BASE_DEC, VALS(rrc_T_supportOfMACiis_04_vals), 0,
        "T_supportOfMACiis_04", HFILL }},
    { &hf_rrc_edch_PhysicalLayerCategory_extension2,
      { "edch-PhysicalLayerCategory-extension2", "rrc.edch_PhysicalLayerCategory_extension2",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_8_9", HFILL }},
    { &hf_rrc_edch_PhysicalLayerCategory_extension3,
      { "edch-PhysicalLayerCategory-extension3", "rrc.edch_PhysicalLayerCategory_extension3",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_10_12", HFILL }},
    { &hf_rrc_supportOfSFModeForHSPDSCHDualStream_01,
      { "supportOfSFModeForHSPDSCHDualStream", "rrc.supportOfSFModeForHSPDSCHDualStream",
        FT_UINT32, BASE_DEC, VALS(rrc_T_supportOfSFModeForHSPDSCHDualStream_01_vals), 0,
        "T_supportOfSFModeForHSPDSCHDualStream_01", HFILL }},
    { &hf_rrc_supportOfCommonEDCH_04,
      { "supportOfCommonEDCH", "rrc.supportOfCommonEDCH",
        FT_UINT32, BASE_DEC, VALS(rrc_T_supportOfCommonEDCH_04_vals), 0,
        "T_supportOfCommonEDCH_04", HFILL }},
    { &hf_rrc_supportOfMACiis_05,
      { "supportOfMACiis", "rrc.supportOfMACiis",
        FT_UINT32, BASE_DEC, VALS(rrc_T_supportOfMACiis_05_vals), 0,
        "T_supportOfMACiis_05", HFILL }},
    { &hf_rrc_supportOfSPSOperation_03,
      { "supportOfSPSOperation", "rrc.supportOfSPSOperation",
        FT_UINT32, BASE_DEC, VALS(rrc_T_supportOfSPSOperation_03_vals), 0,
        "T_supportOfSPSOperation_03", HFILL }},
    { &hf_rrc_supportOfControlChannelDRXOperation_02,
      { "supportOfControlChannelDRXOperation", "rrc.supportOfControlChannelDRXOperation",
        FT_UINT32, BASE_DEC, VALS(rrc_T_supportOfControlChannelDRXOperation_02_vals), 0,
        "T_supportOfControlChannelDRXOperation_02", HFILL }},
    { &hf_rrc_fdd_48,
      { "fdd", "rrc.fdd",
        FT_UINT32, BASE_DEC, VALS(rrc_T_fdd_47_vals), 0,
        "T_fdd_47", HFILL }},
    { &hf_rrc_supported_06,
      { "supported", "rrc.supported",
        FT_UINT32, BASE_DEC, NULL, 0,
        "RF_CapabBandListFDDComp", HFILL }},
    { &hf_rrc_tdd384_RF_Capability,
      { "tdd384-RF-Capability", "rrc.tdd384_RF_Capability",
        FT_UINT32, BASE_DEC, VALS(rrc_T_tdd384_RF_Capability_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_supported_07,
      { "supported", "rrc.supported",
        FT_UINT32, BASE_DEC, VALS(rrc_RadioFrequencyBandTDDList_vals), 0,
        "RadioFrequencyBandTDDList", HFILL }},
    { &hf_rrc_tdd128_RF_Capability_01,
      { "tdd128-RF-Capability", "rrc.tdd128_RF_Capability",
        FT_UINT32, BASE_DEC, VALS(rrc_T_tdd128_RF_Capability_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_tdd384_RF_Capability_01,
      { "tdd384-RF-Capability", "rrc.tdd384_RF_Capability",
        FT_UINT32, BASE_DEC, VALS(rrc_RadioFrequencyBandTDDList_r7_vals), 0,
        "RadioFrequencyBandTDDList_r7", HFILL }},
    { &hf_rrc_tdd768_RF_Capability,
      { "tdd768-RF-Capability", "rrc.tdd768_RF_Capability",
        FT_UINT32, BASE_DEC, VALS(rrc_T_tdd768_RF_Capability_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_supported_08,
      { "supported", "rrc.supported",
        FT_UINT32, BASE_DEC, VALS(rrc_RadioFrequencyBandTDDList_r7_vals), 0,
        "RadioFrequencyBandTDDList_r7", HFILL }},
    { &hf_rrc_tdd128_RF_Capability_02,
      { "tdd128-RF-Capability", "rrc.tdd128_RF_Capability",
        FT_UINT32, BASE_DEC, VALS(rrc_RadioFrequencyBandTDDList_r7_vals), 0,
        "RadioFrequencyBandTDDList_r7", HFILL }},
    { &hf_rrc_RF_CapabBandListFDDComp_item,
      { "RF-CapabBandFDDComp", "rrc.RF_CapabBandFDDComp",
        FT_UINT32, BASE_DEC, VALS(rrc_RF_CapabBandFDDComp_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_RF_CapabBandListFDDComp_ext_item,
      { "RF-CapabBandFDDComp", "rrc.RF_CapabBandFDDComp",
        FT_UINT32, BASE_DEC, VALS(rrc_RF_CapabBandFDDComp_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_RF_CapabBandListFDDComp_ext2_item,
      { "RF-CapabBandFDDComp", "rrc.RF_CapabBandFDDComp",
        FT_UINT32, BASE_DEC, VALS(rrc_RF_CapabBandFDDComp_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_UE_RadioAccessCapabBandCombList_va40ext_item,
      { "SupportedCarrierCombination", "rrc.SupportedCarrierCombination",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_UE_RadioAccessCapabBandCombList_vbxyext_item,
      { "SupportedCarrierCombination-ext", "rrc.SupportedCarrierCombination_ext",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_carrierCombination12,
      { "carrierCombination12", "rrc.carrierCombination12",
        FT_BOOLEAN, BASE_NONE, NULL, 0,
        "BOOLEAN", HFILL }},
    { &hf_rrc_carrierCombination21,
      { "carrierCombination21", "rrc.carrierCombination21",
        FT_BOOLEAN, BASE_NONE, NULL, 0,
        "BOOLEAN", HFILL }},
    { &hf_rrc_carrierCombination13,
      { "carrierCombination13", "rrc.carrierCombination13",
        FT_BOOLEAN, BASE_NONE, NULL, 0,
        "BOOLEAN", HFILL }},
    { &hf_rrc_carrierCombination31,
      { "carrierCombination31", "rrc.carrierCombination31",
        FT_BOOLEAN, BASE_NONE, NULL, 0,
        "BOOLEAN", HFILL }},
    { &hf_rrc_carrierCombination22,
      { "carrierCombination22", "rrc.carrierCombination22",
        FT_BOOLEAN, BASE_NONE, NULL, 0,
        "BOOLEAN", HFILL }},
    { &hf_rrc_carrierCombination14,
      { "carrierCombination14", "rrc.carrierCombination14",
        FT_BOOLEAN, BASE_NONE, NULL, 0,
        "BOOLEAN", HFILL }},
    { &hf_rrc_carrierCombination41,
      { "carrierCombination41", "rrc.carrierCombination41",
        FT_BOOLEAN, BASE_NONE, NULL, 0,
        "BOOLEAN", HFILL }},
    { &hf_rrc_carrierCombination15,
      { "carrierCombination15", "rrc.carrierCombination15",
        FT_BOOLEAN, BASE_NONE, NULL, 0,
        "BOOLEAN", HFILL }},
    { &hf_rrc_carrierCombination51,
      { "carrierCombination51", "rrc.carrierCombination51",
        FT_BOOLEAN, BASE_NONE, NULL, 0,
        "BOOLEAN", HFILL }},
    { &hf_rrc_carrierCombination16,
      { "carrierCombination16", "rrc.carrierCombination16",
        FT_BOOLEAN, BASE_NONE, NULL, 0,
        "BOOLEAN", HFILL }},
    { &hf_rrc_carrierCombination61,
      { "carrierCombination61", "rrc.carrierCombination61",
        FT_BOOLEAN, BASE_NONE, NULL, 0,
        "BOOLEAN", HFILL }},
    { &hf_rrc_carrierCombination17,
      { "carrierCombination17", "rrc.carrierCombination17",
        FT_BOOLEAN, BASE_NONE, NULL, 0,
        "BOOLEAN", HFILL }},
    { &hf_rrc_carrierCombination71,
      { "carrierCombination71", "rrc.carrierCombination71",
        FT_BOOLEAN, BASE_NONE, NULL, 0,
        "BOOLEAN", HFILL }},
    { &hf_rrc_carrierCombination23,
      { "carrierCombination23", "rrc.carrierCombination23",
        FT_BOOLEAN, BASE_NONE, NULL, 0,
        "BOOLEAN", HFILL }},
    { &hf_rrc_carrierCombination32,
      { "carrierCombination32", "rrc.carrierCombination32",
        FT_BOOLEAN, BASE_NONE, NULL, 0,
        "BOOLEAN", HFILL }},
    { &hf_rrc_carrierCombination24,
      { "carrierCombination24", "rrc.carrierCombination24",
        FT_BOOLEAN, BASE_NONE, NULL, 0,
        "BOOLEAN", HFILL }},
    { &hf_rrc_carrierCombination42,
      { "carrierCombination42", "rrc.carrierCombination42",
        FT_BOOLEAN, BASE_NONE, NULL, 0,
        "BOOLEAN", HFILL }},
    { &hf_rrc_carrierCombination25,
      { "carrierCombination25", "rrc.carrierCombination25",
        FT_BOOLEAN, BASE_NONE, NULL, 0,
        "BOOLEAN", HFILL }},
    { &hf_rrc_carrierCombination52,
      { "carrierCombination52", "rrc.carrierCombination52",
        FT_BOOLEAN, BASE_NONE, NULL, 0,
        "BOOLEAN", HFILL }},
    { &hf_rrc_carrierCombination26,
      { "carrierCombination26", "rrc.carrierCombination26",
        FT_BOOLEAN, BASE_NONE, NULL, 0,
        "BOOLEAN", HFILL }},
    { &hf_rrc_carrierCombination62,
      { "carrierCombination62", "rrc.carrierCombination62",
        FT_BOOLEAN, BASE_NONE, NULL, 0,
        "BOOLEAN", HFILL }},
    { &hf_rrc_carrierCombination33,
      { "carrierCombination33", "rrc.carrierCombination33",
        FT_BOOLEAN, BASE_NONE, NULL, 0,
        "BOOLEAN", HFILL }},
    { &hf_rrc_carrierCombination34,
      { "carrierCombination34", "rrc.carrierCombination34",
        FT_BOOLEAN, BASE_NONE, NULL, 0,
        "BOOLEAN", HFILL }},
    { &hf_rrc_carrierCombination43,
      { "carrierCombination43", "rrc.carrierCombination43",
        FT_BOOLEAN, BASE_NONE, NULL, 0,
        "BOOLEAN", HFILL }},
    { &hf_rrc_carrierCombination44,
      { "carrierCombination44", "rrc.carrierCombination44",
        FT_BOOLEAN, BASE_NONE, NULL, 0,
        "BOOLEAN", HFILL }},
    { &hf_rrc_carrierCombination35,
      { "carrierCombination35", "rrc.carrierCombination35",
        FT_BOOLEAN, BASE_NONE, NULL, 0,
        "BOOLEAN", HFILL }},
    { &hf_rrc_carrierCombination53,
      { "carrierCombination53", "rrc.carrierCombination53",
        FT_BOOLEAN, BASE_NONE, NULL, 0,
        "BOOLEAN", HFILL }},
    { &hf_rrc_pdcp_Capability_r5_ext,
      { "pdcp-Capability-r5-ext", "rrc.pdcp_Capability_r5_ext",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_rlc_Capability_r5_ext,
      { "rlc-Capability-r5-ext", "rrc.rlc_Capability_r5_ext",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_physicalChannelCapability_07,
      { "physicalChannelCapability", "rrc.physicalChannelCapability",
        FT_NONE, BASE_NONE, NULL, 0,
        "PhysicalChannelCapability_hspdsch_r5", HFILL }},
    { &hf_rrc_multiModeRAT_Capability_v590ext,
      { "multiModeRAT-Capability-v590ext", "rrc.multiModeRAT_Capability_v590ext",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_pdcp_Capability_r5_ext2,
      { "pdcp-Capability-r5-ext2", "rrc.pdcp_Capability_r5_ext2",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_multiModeRAT_Capability_v680ext,
      { "multiModeRAT-Capability-v680ext", "rrc.multiModeRAT_Capability_v680ext",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_ulOLTDActivation,
      { "ulOLTDActivation", "rrc.ulOLTDActivation",
        FT_UINT32, BASE_DEC, VALS(rrc_T_ulOLTDActivation_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_maxNoDPDCH_BitsTransmitted,
      { "maxNoDPDCH-BitsTransmitted", "rrc.maxNoDPDCH_BitsTransmitted",
        FT_UINT32, BASE_DEC, VALS(rrc_MaxNoDPDCH_BitsTransmitted_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_discontinuousDpcchTransmission_02,
      { "discontinuousDpcchTransmission", "rrc.discontinuousDpcchTransmission",
        FT_UINT32, BASE_DEC, VALS(rrc_T_discontinuousDpcchTransmission_02_vals), 0,
        "T_discontinuousDpcchTransmission_02", HFILL }},
    { &hf_rrc_slotFormat4_01,
      { "slotFormat4", "rrc.slotFormat4",
        FT_UINT32, BASE_DEC, VALS(rrc_T_slotFormat4_01_vals), 0,
        "T_slotFormat4_01", HFILL }},
    { &hf_rrc_maxPhysChPerTimeslot,
      { "maxPhysChPerTimeslot", "rrc.maxPhysChPerTimeslot",
        FT_UINT32, BASE_DEC, VALS(rrc_MaxPhysChPerTimeslot_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_minimumSF_02,
      { "minimumSF", "rrc.minimumSF",
        FT_UINT32, BASE_DEC, VALS(rrc_MinimumSF_UL_vals), 0,
        "MinimumSF_UL", HFILL }},
    { &hf_rrc_supportOfPUSCH,
      { "supportOfPUSCH", "rrc.supportOfPUSCH",
        FT_BOOLEAN, BASE_NONE, NULL, 0,
        "BOOLEAN", HFILL }},
    { &hf_rrc_tdd384_edch,
      { "tdd384-edch", "rrc.tdd384_edch",
        FT_UINT32, BASE_DEC, VALS(rrc_T_tdd384_edch_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_supported_09,
      { "supported", "rrc.supported",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_supported_03", HFILL }},
    { &hf_rrc_tdd_edch_PhysicalLayerCategory_01,
      { "tdd-edch-PhysicalLayerCategory", "rrc.tdd_edch_PhysicalLayerCategory",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_1_16", HFILL }},
    { &hf_rrc_maxPhysChPerTimeslot_01,
      { "maxPhysChPerTimeslot", "rrc.maxPhysChPerTimeslot",
        FT_UINT32, BASE_DEC, NULL, 0,
        "MaxPhysChPerTimeslot_LCR_r7", HFILL }},
    { &hf_rrc_tdd128_edch,
      { "tdd128-edch", "rrc.tdd128_edch",
        FT_UINT32, BASE_DEC, VALS(rrc_T_tdd128_edch_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_supported_10,
      { "supported", "rrc.supported",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_supported_04", HFILL }},
    { &hf_rrc_multiCarrier_EdchPhysicalLayerCategory_128,
      { "multiCarrier-EdchPhysicalLayerCategory-128", "rrc.multiCarrier_EdchPhysicalLayerCategory_128",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_1_8", HFILL }},
    { &hf_rrc_maxNumberCarrierForMCHSUPA_TDD,
      { "maxNumberCarrierForMCHSUPA-TDD", "rrc.maxNumberCarrierForMCHSUPA_TDD",
        FT_UINT32, BASE_DEC, VALS(rrc_T_maxNumberCarrierForMCHSUPA_TDD_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_tdd384_edch_01,
      { "tdd384-edch", "rrc.tdd384_edch",
        FT_UINT32, BASE_DEC, VALS(rrc_T_tdd384_edch_01_vals), 0,
        "T_tdd384_edch_01", HFILL }},
    { &hf_rrc_supported_11,
      { "supported", "rrc.supported",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_supported_05", HFILL }},
    { &hf_rrc_continue,
      { "continue", "rrc.continue",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_newConfiguration,
      { "newConfiguration", "rrc.newConfiguration",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_secondaryServingEDCHCell_Info,
      { "secondaryServingEDCHCell-Info", "rrc.secondaryServingEDCHCell_Info",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_secondaryEDCH_Info_Common,
      { "secondaryEDCH-Info-Common", "rrc.secondaryEDCH_Info_Common",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_dl_InformationPerSecondaryRL_List,
      { "dl-InformationPerSecondaryRL-List", "rrc.dl_InformationPerSecondaryRL_List",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_fdd_edch,
      { "fdd-edch", "rrc.fdd_edch",
        FT_UINT32, BASE_DEC, VALS(rrc_T_fdd_edch_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_supported_12,
      { "supported", "rrc.supported",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_supported_06", HFILL }},
    { &hf_rrc_edch_PhysicalLayerCategory,
      { "edch-PhysicalLayerCategory", "rrc.edch_PhysicalLayerCategory",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_1_16", HFILL }},
    { &hf_rrc_maxNoBitsTransmitted,
      { "maxNoBitsTransmitted", "rrc.maxNoBitsTransmitted",
        FT_UINT32, BASE_DEC, VALS(rrc_MaxNoBits_vals), 0,
        "MaxNoBits", HFILL }},
    { &hf_rrc_maxConvCodeBitsTransmitted,
      { "maxConvCodeBitsTransmitted", "rrc.maxConvCodeBitsTransmitted",
        FT_UINT32, BASE_DEC, VALS(rrc_MaxNoBits_vals), 0,
        "MaxNoBits", HFILL }},
    { &hf_rrc_turboEncodingSupport,
      { "turboEncodingSupport", "rrc.turboEncodingSupport",
        FT_UINT32, BASE_DEC, VALS(rrc_TurboSupport_vals), 0,
        "TurboSupport", HFILL }},
    { &hf_rrc_maxSimultaneousTransChs_01,
      { "maxSimultaneousTransChs", "rrc.maxSimultaneousTransChs",
        FT_UINT32, BASE_DEC, VALS(rrc_MaxSimultaneousTransChsUL_vals), 0,
        "MaxSimultaneousTransChsUL", HFILL }},
    { &hf_rrc_modeSpecificInfo_18,
      { "modeSpecificInfo", "rrc.modeSpecificInfo",
        FT_UINT32, BASE_DEC, VALS(rrc_T_modeSpecificInfo_18_vals), 0,
        "T_modeSpecificInfo_18", HFILL }},
    { &hf_rrc_tdd_13,
      { "tdd", "rrc.tdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_tdd_12", HFILL }},
    { &hf_rrc_maxTransmittedBlocks,
      { "maxTransmittedBlocks", "rrc.maxTransmittedBlocks",
        FT_UINT32, BASE_DEC, VALS(rrc_MaxTransportBlocksUL_vals), 0,
        "MaxTransportBlocksUL", HFILL }},
    { &hf_rrc_maxNumberOfTFC_01,
      { "maxNumberOfTFC", "rrc.maxNumberOfTFC",
        FT_UINT32, BASE_DEC, VALS(rrc_MaxNumberOfTFC_UL_vals), 0,
        "MaxNumberOfTFC_UL", HFILL }},
    { &hf_rrc_standaloneLocMethodsSupported,
      { "standaloneLocMethodsSupported", "rrc.standaloneLocMethodsSupported",
        FT_BOOLEAN, BASE_NONE, NULL, 0,
        "BOOLEAN", HFILL }},
    { &hf_rrc_ue_BasedOTDOA_Supported,
      { "ue-BasedOTDOA-Supported", "rrc.ue_BasedOTDOA_Supported",
        FT_BOOLEAN, BASE_NONE, NULL, 0,
        "BOOLEAN", HFILL }},
    { &hf_rrc_networkAssistedGPS_Supported,
      { "networkAssistedGPS-Supported", "rrc.networkAssistedGPS_Supported",
        FT_UINT32, BASE_DEC, VALS(rrc_NetworkAssistedGPS_Supported_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_supportForUE_GPS_TimingOfCellFrames,
      { "supportForUE-GPS-TimingOfCellFrames", "rrc.supportForUE_GPS_TimingOfCellFrames",
        FT_BOOLEAN, BASE_NONE, NULL, 0,
        "BOOLEAN", HFILL }},
    { &hf_rrc_supportForIPDL,
      { "supportForIPDL", "rrc.supportForIPDL",
        FT_BOOLEAN, BASE_NONE, NULL, 0,
        "BOOLEAN", HFILL }},
    { &hf_rrc_start_CS,
      { "start-CS", "rrc.start_CS",
        FT_BYTES, BASE_NONE, NULL, 0,
        "START_Value", HFILL }},
    { &hf_rrc_start_PS,
      { "start-PS", "rrc.start_PS",
        FT_BYTES, BASE_NONE, NULL, 0,
        "START_Value", HFILL }},
    { &hf_rrc_drx_CycleLengthCoefficient,
      { "drx-CycleLengthCoefficient", "rrc.drx_CycleLengthCoefficient",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_3_9", HFILL }},
    { &hf_rrc_drx_CycleLengthCoefficient2,
      { "drx-CycleLengthCoefficient2", "rrc.drx_CycleLengthCoefficient2",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_3_9", HFILL }},
    { &hf_rrc_timeForDRXCycle2,
      { "timeForDRXCycle2", "rrc.timeForDRXCycle2",
        FT_UINT32, BASE_DEC, VALS(rrc_T_319_vals), 0,
        "T_319", HFILL }},
    { &hf_rrc_rfc2507_Info,
      { "rfc2507-Info", "rrc.rfc2507_Info",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_rfc3095_Info,
      { "rfc3095-Info", "rrc.rfc3095_Info",
        FT_NONE, BASE_NONE, NULL, 0,
        "RFC3095_Info_r4", HFILL }},
    { &hf_rrc_logicalChannelIdentity,
      { "logicalChannelIdentity", "rrc.logicalChannelIdentity",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_mac_ehs_QueueId,
      { "mac-ehs-QueueId", "rrc.mac_ehs_QueueId",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_maxCS_Delay,
      { "maxCS-Delay", "rrc.maxCS_Delay",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_defaultConfigIdForCellFACH,
      { "defaultConfigIdForCellFACH", "rrc.defaultConfigIdForCellFACH",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_inSequenceDelivery,
      { "inSequenceDelivery", "rrc.inSequenceDelivery",
        FT_BOOLEAN, BASE_NONE, NULL, 0,
        "BOOLEAN", HFILL }},
    { &hf_rrc_receivingWindowSize,
      { "receivingWindowSize", "rrc.receivingWindowSize",
        FT_UINT32, BASE_DEC, VALS(rrc_ReceivingWindowSize_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_dl_RLC_StatusInfo,
      { "dl-RLC-StatusInfo", "rrc.dl_RLC_StatusInfo",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_dl_RLC_PDU_size,
      { "dl-RLC-PDU-size", "rrc.dl_RLC_PDU_size",
        FT_UINT32, BASE_DEC, VALS(rrc_OctetModeRLC_SizeInfoType1_vals), 0,
        "OctetModeRLC_SizeInfoType1", HFILL }},
    { &hf_rrc_dl_RLC_PDU_size_01,
      { "dl-RLC-PDU-size", "rrc.dl_RLC_PDU_size",
        FT_UINT32, BASE_DEC, VALS(rrc_T_dl_RLC_PDU_size_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_fixedSize,
      { "fixedSize", "rrc.fixedSize",
        FT_UINT32, BASE_DEC, VALS(rrc_OctetModeRLC_SizeInfoType1_vals), 0,
        "OctetModeRLC_SizeInfoType1", HFILL }},
    { &hf_rrc_flexibleSize,
      { "flexibleSize", "rrc.flexibleSize",
        FT_UINT32, BASE_DEC, VALS(rrc_T_flexibleSize_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_dl_RLC_PDU_size_02,
      { "dl-RLC-PDU-size", "rrc.dl_RLC_PDU_size",
        FT_UINT32, BASE_DEC, VALS(rrc_T_dl_RLC_PDU_size_01_vals), 0,
        "T_dl_RLC_PDU_size_01", HFILL }},
    { &hf_rrc_flexibleSize_01,
      { "flexibleSize", "rrc.flexibleSize",
        FT_UINT32, BASE_DEC, VALS(rrc_T_flexibleSize_01_vals), 0,
        "T_flexibleSize_01", HFILL }},
    { &hf_rrc_dl_RLC_StatusInfo_01,
      { "dl-RLC-StatusInfo", "rrc.dl_RLC_StatusInfo",
        FT_NONE, BASE_NONE, NULL, 0,
        "DL_RLC_StatusInfo_r11", HFILL }},
    { &hf_rrc_rB_WithPDCP_InfoList,
      { "rB-WithPDCP-InfoList", "rrc.rB_WithPDCP_InfoList",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_rb_WithPDCP_InfoList,
      { "rb-WithPDCP-InfoList", "rrc.rb_WithPDCP_InfoList",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_dl_TransportChannelType,
      { "dl-TransportChannelType", "rrc.dl_TransportChannelType",
        FT_UINT32, BASE_DEC, VALS(rrc_DL_TransportChannelType_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_dl_TransportChannelType_01,
      { "dl-TransportChannelType", "rrc.dl_TransportChannelType",
        FT_UINT32, BASE_DEC, VALS(rrc_DL_TransportChannelType_r5_vals), 0,
        "DL_TransportChannelType_r5", HFILL }},
    { &hf_rrc_dl_TransportChannelType_02,
      { "dl-TransportChannelType", "rrc.dl_TransportChannelType",
        FT_UINT32, BASE_DEC, VALS(rrc_DL_TransportChannelType_r7_vals), 0,
        "DL_TransportChannelType_r7", HFILL }},
    { &hf_rrc_DL_LogicalChannelMappingList_item,
      { "DL-LogicalChannelMapping", "rrc.DL_LogicalChannelMapping",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_DL_LogicalChannelMappingList_r5_item,
      { "DL-LogicalChannelMapping-r5", "rrc.DL_LogicalChannelMapping_r5",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_DL_LogicalChannelMappingList_r7_item,
      { "DL-LogicalChannelMapping-r7", "rrc.DL_LogicalChannelMapping_r7",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_dummy_30,
      { "dummy", "rrc.dummy",
        FT_UINT32, BASE_DEC, VALS(rrc_CID_InclusionInfo_r4_vals), 0,
        "CID_InclusionInfo_r4", HFILL }},
    { &hf_rrc_max_CID,
      { "max-CID", "rrc.max_CID",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_1_16383", HFILL }},
    { &hf_rrc_reverseDecompressionDepth,
      { "reverseDecompressionDepth", "rrc.reverseDecompressionDepth",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_65535", HFILL }},
    { &hf_rrc_dl_AM_RLC_Mode,
      { "dl-AM-RLC-Mode", "rrc.dl_AM_RLC_Mode",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_dl_UM_RLC_Mode,
      { "dl-UM-RLC-Mode", "rrc.dl_UM_RLC_Mode",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_dl_TM_RLC_Mode,
      { "dl-TM-RLC-Mode", "rrc.dl_TM_RLC_Mode",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_dl_AM_RLC_Mode_01,
      { "dl-AM-RLC-Mode", "rrc.dl_AM_RLC_Mode",
        FT_NONE, BASE_NONE, NULL, 0,
        "DL_AM_RLC_Mode_r5", HFILL }},
    { &hf_rrc_dl_UM_RLC_Mode_01,
      { "dl-UM-RLC-Mode", "rrc.dl_UM_RLC_Mode",
        FT_NONE, BASE_NONE, NULL, 0,
        "DL_UM_RLC_Mode_r5", HFILL }},
    { &hf_rrc_dl_UM_RLC_Mode_02,
      { "dl-UM-RLC-Mode", "rrc.dl_UM_RLC_Mode",
        FT_NONE, BASE_NONE, NULL, 0,
        "DL_UM_RLC_Mode_r6", HFILL }},
    { &hf_rrc_dl_AM_RLC_Mode_02,
      { "dl-AM-RLC-Mode", "rrc.dl_AM_RLC_Mode",
        FT_NONE, BASE_NONE, NULL, 0,
        "DL_AM_RLC_Mode_r7", HFILL }},
    { &hf_rrc_dl_AM_RLC_Mode_03,
      { "dl-AM-RLC-Mode", "rrc.dl_AM_RLC_Mode",
        FT_NONE, BASE_NONE, NULL, 0,
        "DL_AM_RLC_Mode_r11", HFILL }},
    { &hf_rrc_timerStatusProhibit,
      { "timerStatusProhibit", "rrc.timerStatusProhibit",
        FT_UINT32, BASE_DEC|BASE_EXT_STRING, &rrc_TimerStatusProhibit_vals_ext, 0,
        NULL, HFILL }},
    { &hf_rrc_dummy_31,
      { "dummy", "rrc.dummy",
        FT_UINT32, BASE_DEC, VALS(rrc_TimerEPC_vals), 0,
        "TimerEPC", HFILL }},
    { &hf_rrc_missingPDU_Indicator,
      { "missingPDU-Indicator", "rrc.missingPDU_Indicator",
        FT_BOOLEAN, BASE_NONE, NULL, 0,
        "BOOLEAN", HFILL }},
    { &hf_rrc_timerStatusPeriodic,
      { "timerStatusPeriodic", "rrc.timerStatusPeriodic",
        FT_UINT32, BASE_DEC, VALS(rrc_TimerStatusPeriodic_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_timerReordering,
      { "timerReordering", "rrc.timerReordering",
        FT_UINT32, BASE_DEC, VALS(rrc_TimerReordering_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_segmentationIndication,
      { "segmentationIndication", "rrc.segmentationIndication",
        FT_BOOLEAN, BASE_NONE, NULL, 0,
        "BOOLEAN", HFILL }},
    { &hf_rrc_dch,
      { "dch", "rrc.dch",
        FT_UINT32, BASE_DEC, NULL, 0,
        "TransportChannelIdentity", HFILL }},
    { &hf_rrc_fach,
      { "fach", "rrc.fach",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_dsch,
      { "dsch", "rrc.dsch",
        FT_UINT32, BASE_DEC, NULL, 0,
        "TransportChannelIdentity", HFILL }},
    { &hf_rrc_dch_and_dsch,
      { "dch-and-dsch", "rrc.dch_and_dsch",
        FT_NONE, BASE_NONE, NULL, 0,
        "TransportChannelIdentityDCHandDSCH", HFILL }},
    { &hf_rrc_hsdsch,
      { "hsdsch", "rrc.hsdsch",
        FT_UINT32, BASE_DEC, NULL, 0,
        "MAC_d_FlowIdentity", HFILL }},
    { &hf_rrc_dch_and_hsdsch,
      { "dch-and-hsdsch", "rrc.dch_and_hsdsch",
        FT_NONE, BASE_NONE, NULL, 0,
        "MAC_d_FlowIdentityDCHandHSDSCH", HFILL }},
    { &hf_rrc_hsdsch_01,
      { "hsdsch", "rrc.hsdsch",
        FT_UINT32, BASE_DEC, VALS(rrc_T_hsdsch_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_mac_hs,
      { "mac-hs", "rrc.mac_hs",
        FT_UINT32, BASE_DEC, NULL, 0,
        "MAC_d_FlowIdentity", HFILL }},
    { &hf_rrc_mac_ehs,
      { "mac-ehs", "rrc.mac_ehs",
        FT_UINT32, BASE_DEC, NULL, 0,
        "MAC_ehs_QueueId", HFILL }},
    { &hf_rrc_dch_and_hsdsch_01,
      { "dch-and-hsdsch", "rrc.dch_and_hsdsch",
        FT_UINT32, BASE_DEC, VALS(rrc_T_dch_and_hsdsch_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_mac_hs_01,
      { "mac-hs", "rrc.mac_hs",
        FT_NONE, BASE_NONE, NULL, 0,
        "MAC_d_FlowIdentityDCHandHSDSCH", HFILL }},
    { &hf_rrc_mac_ehs_01,
      { "mac-ehs", "rrc.mac_ehs",
        FT_NONE, BASE_NONE, NULL, 0,
        "MAC_ehs_QueueIdDCHandHSDSCH", HFILL }},
    { &hf_rrc_dl_UM_RLC_LI_size,
      { "dl-UM-RLC-LI-size", "rrc.dl_UM_RLC_LI_size",
        FT_UINT32, BASE_DEC, VALS(rrc_DL_UM_RLC_LI_size_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_dl_Reception_Window_Size,
      { "dl-Reception-Window-Size", "rrc.dl_Reception_Window_Size",
        FT_UINT32, BASE_DEC, VALS(rrc_DL_Reception_Window_Size_r6_vals), 0,
        "DL_Reception_Window_Size_r6", HFILL }},
    { &hf_rrc_timerMRW,
      { "timerMRW", "rrc.timerMRW",
        FT_UINT32, BASE_DEC, VALS(rrc_TimerMRW_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_timerDiscard,
      { "timerDiscard", "rrc.timerDiscard",
        FT_UINT32, BASE_DEC, VALS(rrc_TimerDiscard_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_maxMRW,
      { "maxMRW", "rrc.maxMRW",
        FT_UINT32, BASE_DEC, VALS(rrc_MaxMRW_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_algorithmSpecificInfo,
      { "algorithmSpecificInfo", "rrc.algorithmSpecificInfo",
        FT_UINT32, BASE_DEC, VALS(rrc_AlgorithmSpecificInfo_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_HeaderCompressionInfoList_item,
      { "HeaderCompressionInfo", "rrc.HeaderCompressionInfo",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_algorithmSpecificInfo_01,
      { "algorithmSpecificInfo", "rrc.algorithmSpecificInfo",
        FT_UINT32, BASE_DEC, VALS(rrc_AlgorithmSpecificInfo_r4_vals), 0,
        "AlgorithmSpecificInfo_r4", HFILL }},
    { &hf_rrc_HeaderCompressionInfoList_r4_item,
      { "HeaderCompressionInfo-r4", "rrc.HeaderCompressionInfo_r4",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_supported_13,
      { "supported", "rrc.supported",
        FT_UINT32, BASE_DEC, VALS(rrc_MaxPDCP_SN_WindowSize_vals), 0,
        "MaxPDCP_SN_WindowSize", HFILL }},
    { &hf_rrc_maxDAT,
      { "maxDAT", "rrc.maxDAT",
        FT_UINT32, BASE_DEC, VALS(rrc_MaxDAT_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_losslessSRNS_RelocSupport,
      { "losslessSRNS-RelocSupport", "rrc.losslessSRNS_RelocSupport",
        FT_UINT32, BASE_DEC, VALS(rrc_LosslessSRNS_RelocSupport_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_pdcp_PDU_Header,
      { "pdcp-PDU-Header", "rrc.pdcp_PDU_Header",
        FT_UINT32, BASE_DEC, VALS(rrc_PDCP_PDU_Header_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_headerCompressionInfoList,
      { "headerCompressionInfoList", "rrc.headerCompressionInfoList",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_headerCompressionInfoList_01,
      { "headerCompressionInfoList", "rrc.headerCompressionInfoList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "HeaderCompressionInfoList_r4", HFILL }},
    { &hf_rrc_pdcp_Info,
      { "pdcp-Info", "rrc.pdcp_Info",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_dummy_32,
      { "dummy", "rrc.dummy",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_65535", HFILL }},
    { &hf_rrc_pdcp_Info_01,
      { "pdcp-Info", "rrc.pdcp_Info",
        FT_NONE, BASE_NONE, NULL, 0,
        "PDCP_Info_r4", HFILL }},
    { &hf_rrc_timerPollProhibit,
      { "timerPollProhibit", "rrc.timerPollProhibit",
        FT_UINT32, BASE_DEC|BASE_EXT_STRING, &rrc_TimerPollProhibit_vals_ext, 0,
        NULL, HFILL }},
    { &hf_rrc_timerPoll,
      { "timerPoll", "rrc.timerPoll",
        FT_UINT32, BASE_DEC|BASE_EXT_STRING, &rrc_TimerPoll_vals_ext, 0,
        NULL, HFILL }},
    { &hf_rrc_poll_PDU,
      { "poll-PDU", "rrc.poll_PDU",
        FT_UINT32, BASE_DEC, VALS(rrc_Poll_PDU_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_poll_SDU,
      { "poll-SDU", "rrc.poll_SDU",
        FT_UINT32, BASE_DEC, VALS(rrc_Poll_SDU_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_lastTransmissionPDU_Poll,
      { "lastTransmissionPDU-Poll", "rrc.lastTransmissionPDU_Poll",
        FT_BOOLEAN, BASE_NONE, NULL, 0,
        "BOOLEAN", HFILL }},
    { &hf_rrc_lastRetransmissionPDU_Poll,
      { "lastRetransmissionPDU-Poll", "rrc.lastRetransmissionPDU_Poll",
        FT_BOOLEAN, BASE_NONE, NULL, 0,
        "BOOLEAN", HFILL }},
    { &hf_rrc_pollWindow,
      { "pollWindow", "rrc.pollWindow",
        FT_UINT32, BASE_DEC, VALS(rrc_PollWindow_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_timerPollPeriodic,
      { "timerPollPeriodic", "rrc.timerPollPeriodic",
        FT_UINT32, BASE_DEC, VALS(rrc_TimerPollPeriodic_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_re_EstablishmentTimer,
      { "re-EstablishmentTimer", "rrc.re_EstablishmentTimer",
        FT_UINT32, BASE_DEC, VALS(rrc_Re_EstablishmentTimer_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_srb_InformationList,
      { "srb-InformationList", "rrc.srb_InformationList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "SRB_InformationSetupList", HFILL }},
    { &hf_rrc_rb_InformationList,
      { "rb-InformationList", "rrc.rb_InformationList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "RB_InformationSetupList", HFILL }},
    { &hf_rrc_predefinedRB_Configuration,
      { "predefinedRB-Configuration", "rrc.predefinedRB_Configuration",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_preDefTransChConfiguration,
      { "preDefTransChConfiguration", "rrc.preDefTransChConfiguration",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_preDefPhyChConfiguration,
      { "preDefPhyChConfiguration", "rrc.preDefPhyChConfiguration",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_preDefPhyChConfiguration_01,
      { "preDefPhyChConfiguration", "rrc.preDefPhyChConfiguration",
        FT_NONE, BASE_NONE, NULL, 0,
        "PreDefPhyChConfiguration_v770ext", HFILL }},
    { &hf_rrc_mac_ehsWindowSize,
      { "mac-ehsWindowSize", "rrc.mac_ehsWindowSize",
        FT_UINT32, BASE_DEC, VALS(rrc_MAC_hs_WindowSize_r9_vals), 0,
        "MAC_hs_WindowSize_r9", HFILL }},
    { &hf_rrc_PredefinedConfigStatusList_item,
      { "PredefinedConfigStatusInfo", "rrc.PredefinedConfigStatusInfo",
        FT_UINT32, BASE_DEC, VALS(rrc_PredefinedConfigStatusInfo_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_storedWithValueTagSameAsPrevius,
      { "storedWithValueTagSameAsPrevius", "rrc.storedWithValueTagSameAsPrevius",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_other,
      { "other", "rrc.other",
        FT_UINT32, BASE_DEC, VALS(rrc_T_other_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_notStored,
      { "notStored", "rrc.notStored",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_storedWithDifferentValueTag,
      { "storedWithDifferentValueTag", "rrc.storedWithDifferentValueTag",
        FT_UINT32, BASE_DEC, NULL, 0,
        "PredefinedConfigValueTag", HFILL }},
    { &hf_rrc_setsWithDifferentValueTag,
      { "setsWithDifferentValueTag", "rrc.setsWithDifferentValueTag",
        FT_UINT32, BASE_DEC, NULL, 0,
        "PredefinedConfigSetsWithDifferentValueTag", HFILL }},
    { &hf_rrc_otherEntries,
      { "otherEntries", "rrc.otherEntries",
        FT_UINT32, BASE_DEC, NULL, 0,
        "PredefinedConfigStatusListVarSz", HFILL }},
    { &hf_rrc_PredefinedConfigSetsWithDifferentValueTag_item,
      { "PredefinedConfigSetWithDifferentValueTag", "rrc.PredefinedConfigSetWithDifferentValueTag",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_startPosition,
      { "startPosition", "rrc.startPosition",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_10", HFILL }},
    { &hf_rrc_valueTagList,
      { "valueTagList", "rrc.valueTagList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "PredefinedConfigValueTagList", HFILL }},
    { &hf_rrc_PredefinedConfigValueTagList_item,
      { "PredefinedConfigValueTag", "rrc.PredefinedConfigValueTag",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_PredefinedConfigStatusListVarSz_item,
      { "PredefinedConfigStatusInfo", "rrc.PredefinedConfigStatusInfo",
        FT_UINT32, BASE_DEC, VALS(rrc_PredefinedConfigStatusInfo_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_rab_Identity,
      { "rab-Identity", "rrc.rab_Identity",
        FT_UINT32, BASE_DEC, VALS(rrc_RAB_Identity_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_nas_Synchronisation_Indicator,
      { "nas-Synchronisation-Indicator", "rrc.nas_Synchronisation_Indicator",
        FT_BYTES, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_mbms_SessionIdentity,
      { "mbms-SessionIdentity", "rrc.mbms_SessionIdentity",
        FT_BYTES, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_mbms_ServiceIdentity,
      { "mbms-ServiceIdentity", "rrc.mbms_ServiceIdentity",
        FT_BYTES, BASE_NONE, NULL, 0,
        "OCTET_STRING_SIZE_3", HFILL }},
    { &hf_rrc_RAB_InformationList_item,
      { "RAB-Info", "rrc.RAB_Info",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_RAB_InformationList_r6_item,
      { "RAB-Info-r6", "rrc.RAB_Info_r6",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_RAB_InformationReconfigList_item,
      { "RAB-InformationReconfig", "rrc.RAB_InformationReconfig",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_RAB_InformationReconfigList_r8_item,
      { "RAB-InformationReconfig-r8", "rrc.RAB_InformationReconfig_r8",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_cs_HSPA_Information,
      { "cs-HSPA-Information", "rrc.cs_HSPA_Information",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_rb_Identity,
      { "rb-Identity", "rrc.rb_Identity",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_RAB_InformationMBMSPtpList_item,
      { "RAB-InformationMBMSPtp", "rrc.RAB_InformationMBMSPtp",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_rab_Info_02,
      { "rab-Info", "rrc.rab_Info",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_rb_InformationSetupList,
      { "rb-InformationSetupList", "rrc.rb_InformationSetupList",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_rb_InformationSetupList_01,
      { "rb-InformationSetupList", "rrc.rb_InformationSetupList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "RB_InformationSetupList_r4", HFILL }},
    { &hf_rrc_rb_InformationSetupList_02,
      { "rb-InformationSetupList", "rrc.rb_InformationSetupList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "RB_InformationSetupList_r5", HFILL }},
    { &hf_rrc_rab_Info_r6_ext,
      { "rab-Info-r6-ext", "rrc.rab_Info_r6_ext",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_rb_InformationSetupList_03,
      { "rb-InformationSetupList", "rrc.rb_InformationSetupList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "RB_InformationSetupList_r6", HFILL }},
    { &hf_rrc_rab_Info_v6b0ext,
      { "rab-Info-v6b0ext", "rrc.rab_Info_v6b0ext",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_rab_Info_03,
      { "rab-Info", "rrc.rab_Info",
        FT_NONE, BASE_NONE, NULL, 0,
        "RAB_Info_r7", HFILL }},
    { &hf_rrc_rb_InformationSetupList_04,
      { "rb-InformationSetupList", "rrc.rb_InformationSetupList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "RB_InformationSetupList_r7", HFILL }},
    { &hf_rrc_rb_InformationSetupList_05,
      { "rb-InformationSetupList", "rrc.rb_InformationSetupList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "RB_InformationSetupList_r8", HFILL }},
    { &hf_rrc_rb_InformationSetupList_06,
      { "rb-InformationSetupList", "rrc.rb_InformationSetupList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "RB_InformationSetupList_r11", HFILL }},
    { &hf_rrc_RAB_InformationSetupList_item,
      { "RAB-InformationSetup", "rrc.RAB_InformationSetup",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_RAB_InformationSetupList_r4_item,
      { "RAB-InformationSetup-r4", "rrc.RAB_InformationSetup_r4",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_RAB_InformationSetupList_r5_item,
      { "RAB-InformationSetup-r5", "rrc.RAB_InformationSetup_r5",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_RAB_InformationSetupList_r6_item,
      { "RAB-InformationSetup-r6", "rrc.RAB_InformationSetup_r6",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_RAB_InformationSetupList_r6_ext_item,
      { "RAB-InformationSetup-r6-ext", "rrc.RAB_InformationSetup_r6_ext",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_RAB_InformationSetupList_v6b0ext_item,
      { "RAB-InformationSetup-v6b0ext", "rrc.RAB_InformationSetup_v6b0ext",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_RAB_InformationSetupList_r7_item,
      { "RAB-InformationSetup-r7", "rrc.RAB_InformationSetup_r7",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_RAB_InformationSetupList_r8_item,
      { "RAB-InformationSetup-r8", "rrc.RAB_InformationSetup_r8",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_RAB_InformationSetupList_v820ext_item,
      { "RAB-InformationSetup-v820ext", "rrc.RAB_InformationSetup_v820ext",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_RAB_InformationSetupList_r11_item,
      { "RAB-InformationSetup-r11", "rrc.RAB_InformationSetup_r11",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_rlc_SequenceNumber,
      { "rlc-SequenceNumber", "rrc.rlc_SequenceNumber",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_RB_ActivationTimeInfoList_item,
      { "RB-ActivationTimeInfo", "rrc.RB_ActivationTimeInfo",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_count_C_UL,
      { "count-C-UL", "rrc.count_C_UL",
        FT_UINT32, BASE_DEC, NULL, 0,
        "COUNT_C", HFILL }},
    { &hf_rrc_count_C_DL,
      { "count-C-DL", "rrc.count_C_DL",
        FT_UINT32, BASE_DEC, NULL, 0,
        "COUNT_C", HFILL }},
    { &hf_rrc_RB_COUNT_C_InformationList_item,
      { "RB-COUNT-C-Information", "rrc.RB_COUNT_C_Information",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_count_C_MSB_UL,
      { "count-C-MSB-UL", "rrc.count_C_MSB_UL",
        FT_UINT32, BASE_DEC, NULL, 0,
        "COUNT_C_MSB", HFILL }},
    { &hf_rrc_count_C_MSB_DL,
      { "count-C-MSB-DL", "rrc.count_C_MSB_DL",
        FT_UINT32, BASE_DEC, NULL, 0,
        "COUNT_C_MSB", HFILL }},
    { &hf_rrc_RB_COUNT_C_MSB_InformationList_item,
      { "RB-COUNT-C-MSB-Information", "rrc.RB_COUNT_C_MSB_Information",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_RB_IdentityList_item,
      { "RB-Identity", "rrc.RB_Identity",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_rb_MappingInfo,
      { "rb-MappingInfo", "rrc.rb_MappingInfo",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_rb_MappingInfo_01,
      { "rb-MappingInfo", "rrc.rb_MappingInfo",
        FT_UINT32, BASE_DEC, NULL, 0,
        "RB_MappingInfo_r5", HFILL }},
    { &hf_rrc_rb_MappingInfo_02,
      { "rb-MappingInfo", "rrc.rb_MappingInfo",
        FT_UINT32, BASE_DEC, NULL, 0,
        "RB_MappingInfo_r6", HFILL }},
    { &hf_rrc_rb_MappingInfo_03,
      { "rb-MappingInfo", "rrc.rb_MappingInfo",
        FT_UINT32, BASE_DEC, NULL, 0,
        "RB_MappingInfo_r7", HFILL }},
    { &hf_rrc_rb_MappingInfo_04,
      { "rb-MappingInfo", "rrc.rb_MappingInfo",
        FT_UINT32, BASE_DEC, NULL, 0,
        "RB_MappingInfo_r8", HFILL }},
    { &hf_rrc_RB_InformationAffectedList_item,
      { "RB-InformationAffected", "rrc.RB_InformationAffected",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_RB_InformationAffectedList_r5_item,
      { "RB-InformationAffected-r5", "rrc.RB_InformationAffected_r5",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_RB_InformationAffectedList_r6_item,
      { "RB-InformationAffected-r6", "rrc.RB_InformationAffected_r6",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_RB_InformationAffectedList_r7_item,
      { "RB-InformationAffected-r7", "rrc.RB_InformationAffected_r7",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_RB_InformationAffectedList_r8_item,
      { "RB-InformationAffected-r8", "rrc.RB_InformationAffected_r8",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_rb_Change,
      { "rb-Change", "rrc.rb_Change",
        FT_UINT32, BASE_DEC, VALS(rrc_T_rb_Change_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_release_01,
      { "release", "rrc.release",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_re_mapToDefaultRb,
      { "re-mapToDefaultRb", "rrc.re_mapToDefaultRb",
        FT_UINT32, BASE_DEC, NULL, 0,
        "RB_Identity", HFILL }},
    { &hf_rrc_RB_InformationChangedList_r6_item,
      { "RB-InformationChanged-r6", "rrc.RB_InformationChanged_r6",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_pdcp_Info_02,
      { "pdcp-Info", "rrc.pdcp_Info",
        FT_NONE, BASE_NONE, NULL, 0,
        "PDCP_InfoReconfig", HFILL }},
    { &hf_rrc_pdcp_SN_Info,
      { "pdcp-SN-Info", "rrc.pdcp_SN_Info",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_rlc_Info,
      { "rlc-Info", "rrc.rlc_Info",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_rb_StopContinue,
      { "rb-StopContinue", "rrc.rb_StopContinue",
        FT_UINT32, BASE_DEC, VALS(rrc_RB_StopContinue_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_pdcp_Info_03,
      { "pdcp-Info", "rrc.pdcp_Info",
        FT_NONE, BASE_NONE, NULL, 0,
        "PDCP_InfoReconfig_r4", HFILL }},
    { &hf_rrc_rlc_Info_01,
      { "rlc-Info", "rrc.rlc_Info",
        FT_NONE, BASE_NONE, NULL, 0,
        "RLC_Info_r5", HFILL }},
    { &hf_rrc_rlc_Info_02,
      { "rlc-Info", "rrc.rlc_Info",
        FT_NONE, BASE_NONE, NULL, 0,
        "RLC_Info_r6", HFILL }},
    { &hf_rrc_rlc_Info_03,
      { "rlc-Info", "rrc.rlc_Info",
        FT_NONE, BASE_NONE, NULL, 0,
        "RLC_Info_r7", HFILL }},
    { &hf_rrc_rlc_Info_04,
      { "rlc-Info", "rrc.rlc_Info",
        FT_NONE, BASE_NONE, NULL, 0,
        "RLC_Info_r11", HFILL }},
    { &hf_rrc_RB_InformationReconfigList_item,
      { "RB-InformationReconfig", "rrc.RB_InformationReconfig",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_RB_InformationReconfigList_r4_item,
      { "RB-InformationReconfig-r4", "rrc.RB_InformationReconfig_r4",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_RB_InformationReconfigList_r5_item,
      { "RB-InformationReconfig-r5", "rrc.RB_InformationReconfig_r5",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_RB_InformationReconfigList_r6_item,
      { "RB-InformationReconfig-r6", "rrc.RB_InformationReconfig_r6",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_RB_InformationReconfigList_r7_item,
      { "RB-InformationReconfig-r7", "rrc.RB_InformationReconfig_r7",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_RB_InformationReconfigList_r8_item,
      { "RB-InformationReconfig-r8", "rrc.RB_InformationReconfig_r8",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_RB_InformationReconfigList_r11_item,
      { "RB-InformationReconfig-r11", "rrc.RB_InformationReconfig_r11",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_RB_InformationReleaseList_item,
      { "RB-Identity", "rrc.RB_Identity",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_rlc_InfoChoice,
      { "rlc-InfoChoice", "rrc.rlc_InfoChoice",
        FT_UINT32, BASE_DEC, VALS(rrc_RLC_InfoChoice_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_rlc_InfoChoice_01,
      { "rlc-InfoChoice", "rrc.rlc_InfoChoice",
        FT_UINT32, BASE_DEC, VALS(rrc_RLC_InfoChoice_r5_vals), 0,
        "RLC_InfoChoice_r5", HFILL }},
    { &hf_rrc_rlc_InfoChoice_02,
      { "rlc-InfoChoice", "rrc.rlc_InfoChoice",
        FT_UINT32, BASE_DEC, VALS(rrc_RLC_InfoChoice_r6_vals), 0,
        "RLC_InfoChoice_r6", HFILL }},
    { &hf_rrc_rlc_InfoChoice_03,
      { "rlc-InfoChoice", "rrc.rlc_InfoChoice",
        FT_UINT32, BASE_DEC, VALS(rrc_RLC_InfoChoice_r7_vals), 0,
        "RLC_InfoChoice_r7", HFILL }},
    { &hf_rrc_rlc_InfoChoice_04,
      { "rlc-InfoChoice", "rrc.rlc_InfoChoice",
        FT_UINT32, BASE_DEC, VALS(rrc_RLC_InfoChoice_r11_vals), 0,
        "RLC_InfoChoice_r11", HFILL }},
    { &hf_rrc_RB_InformationSetupList_item,
      { "RB-InformationSetup", "rrc.RB_InformationSetup",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_RB_InformationSetupList_r4_item,
      { "RB-InformationSetup-r4", "rrc.RB_InformationSetup_r4",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_RB_InformationSetupList_r5_item,
      { "RB-InformationSetup-r5", "rrc.RB_InformationSetup_r5",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_RB_InformationSetupList_r6_item,
      { "RB-InformationSetup-r6", "rrc.RB_InformationSetup_r6",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_RB_InformationSetupList_r7_item,
      { "RB-InformationSetup-r7", "rrc.RB_InformationSetup_r7",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_RB_InformationSetupList_r8_item,
      { "RB-InformationSetup-r8", "rrc.RB_InformationSetup_r8",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_RB_InformationSetupList_r11_item,
      { "RB-InformationSetup-r11", "rrc.RB_InformationSetup_r11",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_RB_MappingInfo_item,
      { "RB-MappingOption", "rrc.RB_MappingOption",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_RB_MappingInfo_r5_item,
      { "RB-MappingOption-r5", "rrc.RB_MappingOption_r5",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_RB_MappingInfo_r6_item,
      { "RB-MappingOption-r6", "rrc.RB_MappingOption_r6",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_RB_MappingInfo_r7_item,
      { "RB-MappingOption-r7", "rrc.RB_MappingOption_r7",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_RB_MappingInfo_r8_item,
      { "RB-MappingOption-r8", "rrc.RB_MappingOption_r8",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_ul_LogicalChannelMappings,
      { "ul-LogicalChannelMappings", "rrc.ul_LogicalChannelMappings",
        FT_UINT32, BASE_DEC, VALS(rrc_UL_LogicalChannelMappings_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_dl_LogicalChannelMappingList,
      { "dl-LogicalChannelMappingList", "rrc.dl_LogicalChannelMappingList",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_dl_LogicalChannelMappingList_01,
      { "dl-LogicalChannelMappingList", "rrc.dl_LogicalChannelMappingList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "DL_LogicalChannelMappingList_r5", HFILL }},
    { &hf_rrc_ul_LogicalChannelMappings_01,
      { "ul-LogicalChannelMappings", "rrc.ul_LogicalChannelMappings",
        FT_UINT32, BASE_DEC, VALS(rrc_UL_LogicalChannelMappings_r6_vals), 0,
        "UL_LogicalChannelMappings_r6", HFILL }},
    { &hf_rrc_dl_LogicalChannelMappingList_02,
      { "dl-LogicalChannelMappingList", "rrc.dl_LogicalChannelMappingList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "DL_LogicalChannelMappingList_r7", HFILL }},
    { &hf_rrc_ul_LogicalChannelMappings_02,
      { "ul-LogicalChannelMappings", "rrc.ul_LogicalChannelMappings",
        FT_UINT32, BASE_DEC, VALS(rrc_UL_LogicalChannelMappings_r8_vals), 0,
        "UL_LogicalChannelMappings_r8", HFILL }},
    { &hf_rrc_dl_RFC3095_Context_Relocation,
      { "dl-RFC3095-Context-Relocation", "rrc.dl_RFC3095_Context_Relocation",
        FT_BOOLEAN, BASE_NONE, NULL, 0,
        "BOOLEAN", HFILL }},
    { &hf_rrc_ul_RFC3095_Context_Relocation,
      { "ul-RFC3095-Context-Relocation", "rrc.ul_RFC3095_Context_Relocation",
        FT_BOOLEAN, BASE_NONE, NULL, 0,
        "BOOLEAN", HFILL }},
    { &hf_rrc_RB_PDCPContextRelocationList_item,
      { "RB-PDCPContextRelocation", "rrc.RB_PDCPContextRelocation",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_RB_WithPDCP_InfoList_item,
      { "RB-WithPDCP-Info", "rrc.RB_WithPDCP_Info",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_f_MAX_PERIOD,
      { "f-MAX-PERIOD", "rrc.f_MAX_PERIOD",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_1_65535", HFILL }},
    { &hf_rrc_f_MAX_TIME,
      { "f-MAX-TIME", "rrc.f_MAX_TIME",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_1_255", HFILL }},
    { &hf_rrc_max_HEADER,
      { "max-HEADER", "rrc.max_HEADER",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_60_65535", HFILL }},
    { &hf_rrc_tcp_SPACE,
      { "tcp-SPACE", "rrc.tcp_SPACE",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_3_255", HFILL }},
    { &hf_rrc_non_TCP_SPACE,
      { "non-TCP-SPACE", "rrc.non_TCP_SPACE",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_3_65535", HFILL }},
    { &hf_rrc_expectReordering,
      { "expectReordering", "rrc.expectReordering",
        FT_UINT32, BASE_DEC, VALS(rrc_ExpectReordering_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_rohcProfileList,
      { "rohcProfileList", "rrc.rohcProfileList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "ROHC_ProfileList_r4", HFILL }},
    { &hf_rrc_ul_RFC3095,
      { "ul-RFC3095", "rrc.ul_RFC3095",
        FT_NONE, BASE_NONE, NULL, 0,
        "UL_RFC3095_r4", HFILL }},
    { &hf_rrc_dl_RFC3095,
      { "dl-RFC3095", "rrc.dl_RFC3095",
        FT_NONE, BASE_NONE, NULL, 0,
        "DL_RFC3095_r4", HFILL }},
    { &hf_rrc_ul_RLC_Mode,
      { "ul-RLC-Mode", "rrc.ul_RLC_Mode",
        FT_UINT32, BASE_DEC, VALS(rrc_UL_RLC_Mode_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_dl_RLC_Mode,
      { "dl-RLC-Mode", "rrc.dl_RLC_Mode",
        FT_UINT32, BASE_DEC, VALS(rrc_DL_RLC_Mode_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_dl_RLC_Mode_01,
      { "dl-RLC-Mode", "rrc.dl_RLC_Mode",
        FT_UINT32, BASE_DEC, VALS(rrc_DL_RLC_Mode_r5_vals), 0,
        "DL_RLC_Mode_r5", HFILL }},
    { &hf_rrc_rlc_OneSidedReEst,
      { "rlc-OneSidedReEst", "rrc.rlc_OneSidedReEst",
        FT_BOOLEAN, BASE_NONE, NULL, 0,
        "BOOLEAN", HFILL }},
    { &hf_rrc_dl_RLC_Mode_02,
      { "dl-RLC-Mode", "rrc.dl_RLC_Mode",
        FT_UINT32, BASE_DEC, VALS(rrc_DL_RLC_Mode_r6_vals), 0,
        "DL_RLC_Mode_r6", HFILL }},
    { &hf_rrc_altE_bitInterpretation,
      { "altE-bitInterpretation", "rrc.altE_bitInterpretation",
        FT_UINT32, BASE_DEC, VALS(rrc_T_altE_bitInterpretation_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_dl_RLC_Mode_03,
      { "dl-RLC-Mode", "rrc.dl_RLC_Mode",
        FT_UINT32, BASE_DEC, VALS(rrc_DL_RLC_Mode_r7_vals), 0,
        "DL_RLC_Mode_r7", HFILL }},
    { &hf_rrc_altE_bitInterpretation_01,
      { "altE-bitInterpretation", "rrc.altE_bitInterpretation",
        FT_UINT32, BASE_DEC, VALS(rrc_T_altE_bitInterpretation_01_vals), 0,
        "T_altE_bitInterpretation_01", HFILL }},
    { &hf_rrc_useSpecialValueOfHEField,
      { "useSpecialValueOfHEField", "rrc.useSpecialValueOfHEField",
        FT_UINT32, BASE_DEC, VALS(rrc_T_useSpecialValueOfHEField_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_dl_RLC_Mode_04,
      { "dl-RLC-Mode", "rrc.dl_RLC_Mode",
        FT_UINT32, BASE_DEC, VALS(rrc_DL_RLC_Mode_r11_vals), 0,
        "DL_RLC_Mode_r11", HFILL }},
    { &hf_rrc_altE_bitInterpretation_02,
      { "altE-bitInterpretation", "rrc.altE_bitInterpretation",
        FT_UINT32, BASE_DEC, VALS(rrc_T_altE_bitInterpretation_02_vals), 0,
        "T_altE_bitInterpretation_02", HFILL }},
    { &hf_rrc_useSpecialValueOfHEField_01,
      { "useSpecialValueOfHEField", "rrc.useSpecialValueOfHEField",
        FT_UINT32, BASE_DEC, VALS(rrc_T_useSpecialValueOfHEField_01_vals), 0,
        "T_useSpecialValueOfHEField_01", HFILL }},
    { &hf_rrc_dl_UM_RLC_OutOSeqDelivery_Info,
      { "dl-UM-RLC-OutOSeqDelivery-Info", "rrc.dl_UM_RLC_OutOSeqDelivery_Info",
        FT_NONE, BASE_NONE, NULL, 0,
        "UM_RLC_OutOSeqDelivery_Info_r6", HFILL }},
    { &hf_rrc_dl_UM_RLC_DuplAvoid_Reord_Info,
      { "dl-UM-RLC-DuplAvoid-Reord-Info", "rrc.dl_UM_RLC_DuplAvoid_Reord_Info",
        FT_NONE, BASE_NONE, NULL, 0,
        "UM_RLC_DuplAvoid_Reord_Info_r6", HFILL }},
    { &hf_rrc_same_as_RB,
      { "same-as-RB", "rrc.same_as_RB",
        FT_UINT32, BASE_DEC, NULL, 0,
        "RB_Identity", HFILL }},
    { &hf_rrc_lengthIndicatorSize,
      { "lengthIndicatorSize", "rrc.lengthIndicatorSize",
        FT_UINT32, BASE_DEC, VALS(rrc_T_lengthIndicatorSize_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_minRLC_PDU_Size,
      { "minRLC-PDU-Size", "rrc.minRLC_PDU_Size",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_1503", HFILL }},
    { &hf_rrc_largestRLC_PDU_Size,
      { "largestRLC-PDU-Size", "rrc.largestRLC_PDU_Size",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_1503", HFILL }},
    { &hf_rrc_RLC_PDU_SizeList_item,
      { "RLC-PDU-Size", "rrc.RLC_PDU_Size",
        FT_UINT32, BASE_DEC, VALS(rrc_OctetModeRLC_SizeInfoType1_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_rlc_SizeIndex,
      { "rlc-SizeIndex", "rrc.rlc_SizeIndex",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_1_maxTF", HFILL }},
    { &hf_rrc_RLC_SizeExplicitList_item,
      { "RLC-SizeInfo", "rrc.RLC_SizeInfo",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_ROHC_ProfileList_r4_item,
      { "ROHC-Profile-r4", "rrc.ROHC_Profile_r4",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_ROHC_PacketSizeList_r4_item,
      { "ROHC-PacketSize-r4", "rrc.ROHC_PacketSize_r4",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_SRB_InformationSetupList_item,
      { "SRB-InformationSetup", "rrc.SRB_InformationSetup",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_SRB_InformationSetupList_r5_item,
      { "SRB-InformationSetup-r5", "rrc.SRB_InformationSetup_r5",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_SRB_InformationSetupList_r6_item,
      { "SRB-InformationSetup-r6", "rrc.SRB_InformationSetup_r6",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_SRB_InformationSetupList_r7_item,
      { "SRB-InformationSetup-r7", "rrc.SRB_InformationSetup_r7",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_SRB_InformationSetupList_r8_item,
      { "SRB-InformationSetup-r8", "rrc.SRB_InformationSetup_r8",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_SRB_InformationSetupList2_item,
      { "SRB-InformationSetup", "rrc.SRB_InformationSetup",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_SRB_InformationSetupList2_r6_item,
      { "SRB-InformationSetup-r6", "rrc.SRB_InformationSetup_r6",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_SRB_InformationSetupList2_r7_item,
      { "SRB-InformationSetup-r7", "rrc.SRB_InformationSetup_r7",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_SRB_InformationSetupList2_r8_item,
      { "SRB-InformationSetup-r8", "rrc.SRB_InformationSetup_r8",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_timerBasedExplicit,
      { "timerBasedExplicit", "rrc.timerBasedExplicit",
        FT_NONE, BASE_NONE, NULL, 0,
        "ExplicitDiscard", HFILL }},
    { &hf_rrc_timerBasedNoExplicit,
      { "timerBasedNoExplicit", "rrc.timerBasedNoExplicit",
        FT_UINT32, BASE_DEC, VALS(rrc_NoExplicitDiscard_vals), 0,
        "NoExplicitDiscard", HFILL }},
    { &hf_rrc_maxDAT_Retransmissions,
      { "maxDAT-Retransmissions", "rrc.maxDAT_Retransmissions",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_noDiscard,
      { "noDiscard", "rrc.noDiscard",
        FT_UINT32, BASE_DEC, VALS(rrc_MaxDAT_vals), 0,
        "MaxDAT", HFILL }},
    { &hf_rrc_transmissionRLC_Discard,
      { "transmissionRLC-Discard", "rrc.transmissionRLC_Discard",
        FT_UINT32, BASE_DEC, VALS(rrc_TransmissionRLC_Discard_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_transmissionWindowSize,
      { "transmissionWindowSize", "rrc.transmissionWindowSize",
        FT_UINT32, BASE_DEC, VALS(rrc_TransmissionWindowSize_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_timerRST,
      { "timerRST", "rrc.timerRST",
        FT_UINT32, BASE_DEC, VALS(rrc_TimerRST_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_max_RST,
      { "max-RST", "rrc.max_RST",
        FT_UINT32, BASE_DEC, VALS(rrc_MaxRST_vals), 0,
        "MaxRST", HFILL }},
    { &hf_rrc_pollingInfo,
      { "pollingInfo", "rrc.pollingInfo",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_ul_TransportChannelType,
      { "ul-TransportChannelType", "rrc.ul_TransportChannelType",
        FT_UINT32, BASE_DEC, VALS(rrc_UL_TransportChannelType_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_rlc_SizeList,
      { "rlc-SizeList", "rrc.rlc_SizeList",
        FT_UINT32, BASE_DEC, VALS(rrc_T_rlc_SizeList_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_allSizes,
      { "allSizes", "rrc.allSizes",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_configured,
      { "configured", "rrc.configured",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_explicitList,
      { "explicitList", "rrc.explicitList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "RLC_SizeExplicitList", HFILL }},
    { &hf_rrc_mac_LogicalChannelPriority,
      { "mac-LogicalChannelPriority", "rrc.mac_LogicalChannelPriority",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_ul_TrCH_Type,
      { "ul-TrCH-Type", "rrc.ul_TrCH_Type",
        FT_UINT32, BASE_DEC, VALS(rrc_T_ul_TrCH_Type_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_dch_rach_usch,
      { "dch-rach-usch", "rrc.dch_rach_usch",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_rlc_SizeList_01,
      { "rlc-SizeList", "rrc.rlc_SizeList",
        FT_UINT32, BASE_DEC, VALS(rrc_T_rlc_SizeList_01_vals), 0,
        "T_rlc_SizeList_01", HFILL }},
    { &hf_rrc_e_dch,
      { "e-dch", "rrc.e_dch",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_e_DCH_MAC_d_FlowIdentity,
      { "e-DCH-MAC-d-FlowIdentity", "rrc.e_DCH_MAC_d_FlowIdentity",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_ddi,
      { "ddi", "rrc.ddi",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_rlc_PDU_SizeList,
      { "rlc-PDU-SizeList", "rrc.rlc_PDU_SizeList",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_includeInSchedulingInfo,
      { "includeInSchedulingInfo", "rrc.includeInSchedulingInfo",
        FT_BOOLEAN, BASE_NONE, NULL, 0,
        "BOOLEAN", HFILL }},
    { &hf_rrc_ul_TrCH_Type_01,
      { "ul-TrCH-Type", "rrc.ul_TrCH_Type",
        FT_UINT32, BASE_DEC, VALS(rrc_T_ul_TrCH_Type_01_vals), 0,
        "T_ul_TrCH_Type_01", HFILL }},
    { &hf_rrc_dch_rach_usch_01,
      { "dch-rach-usch", "rrc.dch_rach_usch",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_dch_rach_usch_01", HFILL }},
    { &hf_rrc_rlc_SizeList_02,
      { "rlc-SizeList", "rrc.rlc_SizeList",
        FT_UINT32, BASE_DEC, VALS(rrc_T_rlc_SizeList_02_vals), 0,
        "T_rlc_SizeList_02", HFILL }},
    { &hf_rrc_e_dch_01,
      { "e-dch", "rrc.e_dch",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_e_dch_01", HFILL }},
    { &hf_rrc_rlc_PDU_Size,
      { "rlc-PDU-Size", "rrc.rlc_PDU_Size",
        FT_UINT32, BASE_DEC, VALS(rrc_T_rlc_PDU_Size_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_fixedSize_01,
      { "fixedSize", "rrc.fixedSize",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_flexibleSize_02,
      { "flexibleSize", "rrc.flexibleSize",
        FT_NONE, BASE_NONE, NULL, 0,
        "RLC_PDU_SizeConstraint", HFILL }},
    { &hf_rrc_rlc_LogicalChannelMappingIndicator,
      { "rlc-LogicalChannelMappingIndicator", "rrc.rlc_LogicalChannelMappingIndicator",
        FT_BOOLEAN, BASE_NONE, NULL, 0,
        "BOOLEAN", HFILL }},
    { &hf_rrc_ul_LogicalChannelMapping,
      { "ul-LogicalChannelMapping", "rrc.ul_LogicalChannelMapping",
        FT_UINT32, BASE_DEC, NULL, 0,
        "SEQUENCE_SIZE_maxLoCHperRLC_OF_UL_LogicalChannelMapping", HFILL }},
    { &hf_rrc_ul_LogicalChannelMapping_item,
      { "UL-LogicalChannelMapping", "rrc.UL_LogicalChannelMapping",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_ul_LogicalChannelMapping_01,
      { "ul-LogicalChannelMapping", "rrc.ul_LogicalChannelMapping",
        FT_UINT32, BASE_DEC, NULL, 0,
        "SEQUENCE_SIZE_maxLoCHperRLC_OF_UL_LogicalChannelMapping_r6", HFILL }},
    { &hf_rrc_ul_LogicalChannelMapping_item_01,
      { "UL-LogicalChannelMapping-r6", "rrc.UL_LogicalChannelMapping_r6",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_ul_LogicalChannelMapping_02,
      { "ul-LogicalChannelMapping", "rrc.ul_LogicalChannelMapping",
        FT_UINT32, BASE_DEC, NULL, 0,
        "SEQUENCE_SIZE_maxLoCHperRLC_OF_UL_LogicalChannelMapping_r8", HFILL }},
    { &hf_rrc_ul_LogicalChannelMapping_item_02,
      { "UL-LogicalChannelMapping-r8", "rrc.UL_LogicalChannelMapping_r8",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_oneLogicalChannel,
      { "oneLogicalChannel", "rrc.oneLogicalChannel",
        FT_NONE, BASE_NONE, NULL, 0,
        "UL_LogicalChannelMapping", HFILL }},
    { &hf_rrc_twoLogicalChannels,
      { "twoLogicalChannels", "rrc.twoLogicalChannels",
        FT_NONE, BASE_NONE, NULL, 0,
        "UL_LogicalChannelMappingList", HFILL }},
    { &hf_rrc_oneLogicalChannel_01,
      { "oneLogicalChannel", "rrc.oneLogicalChannel",
        FT_NONE, BASE_NONE, NULL, 0,
        "UL_LogicalChannelMapping_r6", HFILL }},
    { &hf_rrc_twoLogicalChannels_01,
      { "twoLogicalChannels", "rrc.twoLogicalChannels",
        FT_NONE, BASE_NONE, NULL, 0,
        "UL_LogicalChannelMappingList_r6", HFILL }},
    { &hf_rrc_oneLogicalChannel_02,
      { "oneLogicalChannel", "rrc.oneLogicalChannel",
        FT_NONE, BASE_NONE, NULL, 0,
        "UL_LogicalChannelMapping_r8", HFILL }},
    { &hf_rrc_twoLogicalChannels_02,
      { "twoLogicalChannels", "rrc.twoLogicalChannels",
        FT_NONE, BASE_NONE, NULL, 0,
        "UL_LogicalChannelMappingList_r8", HFILL }},
    { &hf_rrc_dummy1_01,
      { "dummy1", "rrc.dummy1",
        FT_UINT32, BASE_DEC, VALS(rrc_CID_InclusionInfo_r4_vals), 0,
        "CID_InclusionInfo_r4", HFILL }},
    { &hf_rrc_dummy_33,
      { "dummy", "rrc.dummy",
        FT_UINT32, BASE_DEC, NULL, 0,
        "ROHC_PacketSizeList_r4", HFILL }},
    { &hf_rrc_ul_AM_RLC_Mode,
      { "ul-AM-RLC-Mode", "rrc.ul_AM_RLC_Mode",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_ul_UM_RLC_Mode,
      { "ul-UM-RLC-Mode", "rrc.ul_UM_RLC_Mode",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_ul_TM_RLC_Mode,
      { "ul-TM-RLC-Mode", "rrc.ul_TM_RLC_Mode",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_rach,
      { "rach", "rrc.rach",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_usch,
      { "usch", "rrc.usch",
        FT_UINT32, BASE_DEC, NULL, 0,
        "TransportChannelIdentity", HFILL }},
    { &hf_rrc_timer_DAR,
      { "timer-DAR", "rrc.timer_DAR",
        FT_UINT32, BASE_DEC, VALS(rrc_TimerDAR_r6_vals), 0,
        "TimerDAR_r6", HFILL }},
    { &hf_rrc_widowSize_DAR,
      { "widowSize-DAR", "rrc.widowSize_DAR",
        FT_UINT32, BASE_DEC, VALS(rrc_WindowSizeDAR_r6_vals), 0,
        "WindowSizeDAR_r6", HFILL }},
    { &hf_rrc_timer_OSD,
      { "timer-OSD", "rrc.timer_OSD",
        FT_UINT32, BASE_DEC, VALS(rrc_TimerOSD_r6_vals), 0,
        "TimerOSD_r6", HFILL }},
    { &hf_rrc_windowSize_OSD,
      { "windowSize-OSD", "rrc.windowSize_OSD",
        FT_UINT32, BASE_DEC, VALS(rrc_WindowSizeOSD_r6_vals), 0,
        "WindowSizeOSD_r6", HFILL }},
    { &hf_rrc_mac_hs_AddReconfQueue_List,
      { "mac-hs-AddReconfQueue-List", "rrc.mac_hs_AddReconfQueue_List",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_mac_hs_DelQueue_List,
      { "mac-hs-DelQueue-List", "rrc.mac_hs_DelQueue_List",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_mac_ehs_AddReconfQueue_List,
      { "mac-ehs-AddReconfQueue-List", "rrc.mac_ehs_AddReconfQueue_List",
        FT_UINT32, BASE_DEC, NULL, 0,
        "MAC_ehs_AddReconfReordQ_List", HFILL }},
    { &hf_rrc_dummy_34,
      { "dummy", "rrc.dummy",
        FT_UINT32, BASE_DEC, NULL, 0,
        "MAC_ehs_DelReordQ_List", HFILL }},
    { &hf_rrc_mac_ehs_AddReconfQueue_List_01,
      { "mac-ehs-AddReconfQueue-List", "rrc.mac_ehs_AddReconfQueue_List",
        FT_UINT32, BASE_DEC, NULL, 0,
        "MAC_ehs_AddReconfReordQ_List_r9", HFILL }},
    { &hf_rrc_mac_ehs_AddReconfQueue_List_02,
      { "mac-ehs-AddReconfQueue-List", "rrc.mac_ehs_AddReconfQueue_List",
        FT_UINT32, BASE_DEC, NULL, 0,
        "MAC_ehs_AddReconfReordQ_List_TDD128_v9c0ext", HFILL }},
    { &hf_rrc_mac_ehs_AddReconfQueue_List_03,
      { "mac-ehs-AddReconfQueue-List", "rrc.mac_ehs_AddReconfQueue_List",
        FT_UINT32, BASE_DEC, NULL, 0,
        "MAC_ehs_AddReconfReordQ_List_r11", HFILL }},
    { &hf_rrc_AllowedTFC_List_item,
      { "TFC-Value", "rrc.TFC_Value",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_AllowedTFI_List_item,
      { "AllowedTFI-List item", "rrc.AllowedTFI_List_item",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_31", HFILL }},
    { &hf_rrc_sizeType1,
      { "sizeType1", "rrc.sizeType1",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_127", HFILL }},
    { &hf_rrc_sizeType2,
      { "sizeType2", "rrc.sizeType2",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_part1,
      { "part1", "rrc.part1",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_15", HFILL }},
    { &hf_rrc_part2,
      { "part2", "rrc.part2",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_1_7", HFILL }},
    { &hf_rrc_sizeType3,
      { "sizeType3", "rrc.sizeType3",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_part1_01,
      { "part1", "rrc.part1",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_47", HFILL }},
    { &hf_rrc_part2_01,
      { "part2", "rrc.part2",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_1_15", HFILL }},
    { &hf_rrc_sizeType4,
      { "sizeType4", "rrc.sizeType4",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_part1_02,
      { "part1", "rrc.part1",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_62", HFILL }},
    { &hf_rrc_part2_02,
      { "part2", "rrc.part2",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_1_63", HFILL }},
    { &hf_rrc_noCoding,
      { "noCoding", "rrc.noCoding",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_convolutional,
      { "convolutional", "rrc.convolutional",
        FT_UINT32, BASE_DEC, VALS(rrc_CodingRate_vals), 0,
        "CodingRate", HFILL }},
    { &hf_rrc_turbo,
      { "turbo", "rrc.turbo",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_mac_d_FlowIdentity,
      { "mac-d-FlowIdentity", "rrc.mac_d_FlowIdentity",
        FT_UINT32, BASE_DEC, NULL, 0,
        "E_DCH_MAC_d_FlowIdentity", HFILL }},
    { &hf_rrc_mac_d_FlowPowerOffset,
      { "mac-d-FlowPowerOffset", "rrc.mac_d_FlowPowerOffset",
        FT_UINT32, BASE_DEC, NULL, 0,
        "E_DCH_MAC_d_FlowPowerOffset", HFILL }},
    { &hf_rrc_mac_d_FlowMaxRetrans,
      { "mac-d-FlowMaxRetrans", "rrc.mac_d_FlowMaxRetrans",
        FT_UINT32, BASE_DEC, NULL, 0,
        "E_DCH_MAC_d_FlowMaxRetrans", HFILL }},
    { &hf_rrc_mac_d_FlowMultiplexingList,
      { "mac-d-FlowMultiplexingList", "rrc.mac_d_FlowMultiplexingList",
        FT_BYTES, BASE_NONE, NULL, 0,
        "E_DCH_MAC_d_FlowMultiplexingList", HFILL }},
    { &hf_rrc_e_dch_mac_d_flow_retransmission_timer,
      { "e-dch-mac-d-flow-retransmission-timer", "rrc.e_dch_mac_d_flow_retransmission_timer",
        FT_UINT32, BASE_DEC|BASE_EXT_STRING, &rrc_E_DCH_MAC_d_FlowRetransTimer_vals_ext, 0,
        "E_DCH_MAC_d_FlowRetransTimer", HFILL }},
    { &hf_rrc_Common_E_DCH_MAC_d_FlowList_item,
      { "Common-E-DCH-MAC-d-Flow", "rrc.Common_E_DCH_MAC_d_Flow",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_Common_E_DCH_MAC_d_Flow_Info_List_ConcurrentTTI_item,
      { "Common-E-DCH-MAC-d-Flow-Info-ConcurrentTTI", "rrc.Common_E_DCH_MAC_d_Flow_Info_ConcurrentTTI",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_rlc_Size,
      { "rlc-Size", "rrc.rlc_Size",
        FT_UINT32, BASE_DEC, VALS(rrc_T_rlc_Size_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_fdd_49,
      { "fdd", "rrc.fdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_fdd_48", HFILL }},
    { &hf_rrc_octetModeRLC_SizeInfoType2,
      { "octetModeRLC-SizeInfoType2", "rrc.octetModeRLC_SizeInfoType2",
        FT_UINT32, BASE_DEC, VALS(rrc_OctetModeRLC_SizeInfoType2_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_tdd_14,
      { "tdd", "rrc.tdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_tdd_13", HFILL }},
    { &hf_rrc_commonTDD_Choice,
      { "commonTDD-Choice", "rrc.commonTDD_Choice",
        FT_UINT32, BASE_DEC, VALS(rrc_T_commonTDD_Choice_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_bitModeRLC_SizeInfo,
      { "bitModeRLC-SizeInfo", "rrc.bitModeRLC_SizeInfo",
        FT_UINT32, BASE_DEC, VALS(rrc_BitModeRLC_SizeInfo_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_octetModeRLC_SizeInfoType1,
      { "octetModeRLC-SizeInfoType1", "rrc.octetModeRLC_SizeInfoType1",
        FT_UINT32, BASE_DEC, VALS(rrc_OctetModeRLC_SizeInfoType1_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_numberOfTbSizeList,
      { "numberOfTbSizeList", "rrc.numberOfTbSizeList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "SEQUENCE_SIZE_1_maxTF_OF_NumberOfTransportBlocks", HFILL }},
    { &hf_rrc_numberOfTbSizeList_item,
      { "NumberOfTransportBlocks", "rrc.NumberOfTransportBlocks",
        FT_UINT32, BASE_DEC, VALS(rrc_NumberOfTransportBlocks_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_logicalChannelList,
      { "logicalChannelList", "rrc.logicalChannelList",
        FT_UINT32, BASE_DEC, VALS(rrc_LogicalChannelList_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_commonTDD_Choice_01,
      { "commonTDD-Choice", "rrc.commonTDD_Choice",
        FT_UINT32, BASE_DEC, VALS(rrc_T_commonTDD_Choice_01_vals), 0,
        "T_commonTDD_Choice_01", HFILL }},
    { &hf_rrc_numberOfTbSizeAndTTIList,
      { "numberOfTbSizeAndTTIList", "rrc.numberOfTbSizeAndTTIList",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_CommonDynamicTF_InfoList_item,
      { "CommonDynamicTF-Info", "rrc.CommonDynamicTF_Info",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_CommonDynamicTF_InfoList_DynamicTTI_item,
      { "CommonDynamicTF-Info-DynamicTTI", "rrc.CommonDynamicTF_Info_DynamicTTI",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_tti,
      { "tti", "rrc.tti",
        FT_UINT32, BASE_DEC, VALS(rrc_T_tti_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_tti10,
      { "tti10", "rrc.tti10",
        FT_UINT32, BASE_DEC, NULL, 0,
        "CommonDynamicTF_InfoList", HFILL }},
    { &hf_rrc_tti20,
      { "tti20", "rrc.tti20",
        FT_UINT32, BASE_DEC, NULL, 0,
        "CommonDynamicTF_InfoList", HFILL }},
    { &hf_rrc_tti40,
      { "tti40", "rrc.tti40",
        FT_UINT32, BASE_DEC, NULL, 0,
        "CommonDynamicTF_InfoList", HFILL }},
    { &hf_rrc_tti80,
      { "tti80", "rrc.tti80",
        FT_UINT32, BASE_DEC, NULL, 0,
        "CommonDynamicTF_InfoList", HFILL }},
    { &hf_rrc_dynamic,
      { "dynamic", "rrc.dynamic",
        FT_UINT32, BASE_DEC, NULL, 0,
        "CommonDynamicTF_InfoList_DynamicTTI", HFILL }},
    { &hf_rrc_semistaticTF_Information,
      { "semistaticTF-Information", "rrc.semistaticTF_Information",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_tti_01,
      { "tti", "rrc.tti",
        FT_UINT32, BASE_DEC, VALS(rrc_T_tti_01_vals), 0,
        "T_tti_01", HFILL }},
    { &hf_rrc_tti5,
      { "tti5", "rrc.tti5",
        FT_UINT32, BASE_DEC, NULL, 0,
        "CommonDynamicTF_InfoList", HFILL }},
    { &hf_rrc_t1_ReleaseTimer,
      { "t1-ReleaseTimer", "rrc.t1_ReleaseTimer",
        FT_UINT32, BASE_DEC, VALS(rrc_T1_ReleaseTimer_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_reorderingResetTimer,
      { "reorderingResetTimer", "rrc.reorderingResetTimer",
        FT_UINT32, BASE_DEC, VALS(rrc_Treset_ResetTimer_vals), 0,
        "Treset_ResetTimer", HFILL }},
    { &hf_rrc_mac_ehsWindowSize_01,
      { "mac-ehsWindowSize", "rrc.mac_ehsWindowSize",
        FT_UINT32, BASE_DEC, VALS(rrc_MAC_hs_WindowSize_vals), 0,
        "MAC_hs_WindowSize", HFILL }},
    { &hf_rrc_Common_MAC_ehs_ReorderingQueueList_item,
      { "Common-MAC-ehs-ReorderingQueue", "rrc.Common_MAC_ehs_ReorderingQueue",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_tti_selection,
      { "tti-selection", "rrc.tti_selection",
        FT_UINT32, BASE_DEC, VALS(rrc_T_tti_selection_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_fixed,
      { "fixed", "rrc.fixed",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_fixed_TTI_Selection,
      { "fixed-TTI-Selection", "rrc.fixed_TTI_Selection",
        FT_UINT32, BASE_DEC, VALS(rrc_T_fixed_TTI_Selection_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_threshold_based,
      { "threshold-based", "rrc.threshold_based",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_common_E_DCH_TTI_Selection_Thresh,
      { "common-E-DCH-TTI-Selection-Thresh", "rrc.common_E_DCH_TTI_Selection_Thresh",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_15", HFILL }},
    { &hf_rrc_concurrent_TTI_Partition_Index,
      { "concurrent-TTI-Partition-Index", "rrc.concurrent_TTI_Partition_Index",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_maxEDCHs_1", HFILL }},
    { &hf_rrc_commonEDCHSystemInfoParamConcurrentTTI,
      { "commonEDCHSystemInfoParamConcurrentTTI", "rrc.commonEDCHSystemInfoParamConcurrentTTI",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_rlc_Size_01,
      { "rlc-Size", "rrc.rlc_Size",
        FT_UINT32, BASE_DEC, VALS(rrc_T_rlc_Size_01_vals), 0,
        "T_rlc_Size_01", HFILL }},
    { &hf_rrc_bitMode,
      { "bitMode", "rrc.bitMode",
        FT_UINT32, BASE_DEC, VALS(rrc_BitModeRLC_SizeInfo_vals), 0,
        "BitModeRLC_SizeInfo", HFILL }},
    { &hf_rrc_octetModeType1,
      { "octetModeType1", "rrc.octetModeType1",
        FT_UINT32, BASE_DEC, VALS(rrc_OctetModeRLC_SizeInfoType1_vals), 0,
        "OctetModeRLC_SizeInfoType1", HFILL }},
    { &hf_rrc_rlc_Size_02,
      { "rlc-Size", "rrc.rlc_Size",
        FT_UINT32, BASE_DEC, VALS(rrc_T_rlc_Size_02_vals), 0,
        "T_rlc_Size_02", HFILL }},
    { &hf_rrc_DedicatedDynamicTF_InfoList_item,
      { "DedicatedDynamicTF-Info", "rrc.DedicatedDynamicTF_Info",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_DedicatedDynamicTF_InfoList_DynamicTTI_item,
      { "DedicatedDynamicTF-Info-DynamicTTI", "rrc.DedicatedDynamicTF_Info_DynamicTTI",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_tti_02,
      { "tti", "rrc.tti",
        FT_UINT32, BASE_DEC, VALS(rrc_T_tti_02_vals), 0,
        "T_tti_02", HFILL }},
    { &hf_rrc_tti10_01,
      { "tti10", "rrc.tti10",
        FT_UINT32, BASE_DEC, NULL, 0,
        "DedicatedDynamicTF_InfoList", HFILL }},
    { &hf_rrc_tti20_01,
      { "tti20", "rrc.tti20",
        FT_UINT32, BASE_DEC, NULL, 0,
        "DedicatedDynamicTF_InfoList", HFILL }},
    { &hf_rrc_tti40_01,
      { "tti40", "rrc.tti40",
        FT_UINT32, BASE_DEC, NULL, 0,
        "DedicatedDynamicTF_InfoList", HFILL }},
    { &hf_rrc_tti80_01,
      { "tti80", "rrc.tti80",
        FT_UINT32, BASE_DEC, NULL, 0,
        "DedicatedDynamicTF_InfoList", HFILL }},
    { &hf_rrc_dynamic_01,
      { "dynamic", "rrc.dynamic",
        FT_UINT32, BASE_DEC, NULL, 0,
        "DedicatedDynamicTF_InfoList_DynamicTTI", HFILL }},
    { &hf_rrc_DL_AddReconfTransChInfo2List_item,
      { "DL-AddReconfTransChInformation2", "rrc.DL_AddReconfTransChInformation2",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_DL_AddReconfTransChInfoList_item,
      { "DL-AddReconfTransChInformation", "rrc.DL_AddReconfTransChInformation",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_DL_AddReconfTransChInfoList_r4_item,
      { "DL-AddReconfTransChInformation-r4", "rrc.DL_AddReconfTransChInformation_r4",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_DL_AddReconfTransChInfoList_r5_item,
      { "DL-AddReconfTransChInformation-r5", "rrc.DL_AddReconfTransChInformation_r5",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_DL_AddReconfTransChInfoList_r7_item,
      { "DL-AddReconfTransChInformation-r7", "rrc.DL_AddReconfTransChInformation_r7",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_DL_AddReconfTransChInfoList_r9_item,
      { "DL-AddReconfTransChInformation-r9", "rrc.DL_AddReconfTransChInformation_r9",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_DL_AddReconfTransChInfoList_TDD128_v9c0ext_item,
      { "DL-AddReconfTransChInformation-TDD128-v9c0ext", "rrc.DL_AddReconfTransChInformation_TDD128_v9c0ext",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_DL_AddReconfTransChInfoList_r11_item,
      { "DL-AddReconfTransChInformation-r11", "rrc.DL_AddReconfTransChInformation_r11",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_dl_TransportChannelType_03,
      { "dl-TransportChannelType", "rrc.dl_TransportChannelType",
        FT_UINT32, BASE_DEC, VALS(rrc_DL_TrCH_Type_vals), 0,
        "DL_TrCH_Type", HFILL }},
    { &hf_rrc_dl_transportChannelIdentity,
      { "dl-transportChannelIdentity", "rrc.dl_transportChannelIdentity",
        FT_UINT32, BASE_DEC, NULL, 0,
        "TransportChannelIdentity", HFILL }},
    { &hf_rrc_tfs_SignallingMode,
      { "tfs-SignallingMode", "rrc.tfs_SignallingMode",
        FT_UINT32, BASE_DEC, VALS(rrc_T_tfs_SignallingMode_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_explicit_config,
      { "explicit-config", "rrc.explicit_config",
        FT_UINT32, BASE_DEC, VALS(rrc_TransportFormatSet_vals), 0,
        "TransportFormatSet", HFILL }},
    { &hf_rrc_sameAsULTrCH,
      { "sameAsULTrCH", "rrc.sameAsULTrCH",
        FT_NONE, BASE_NONE, NULL, 0,
        "UL_TransportChannelIdentity", HFILL }},
    { &hf_rrc_dch_QualityTarget,
      { "dch-QualityTarget", "rrc.dch_QualityTarget",
        FT_NONE, BASE_NONE, NULL, 0,
        "QualityTarget", HFILL }},
    { &hf_rrc_dummy_35,
      { "dummy", "rrc.dummy",
        FT_NONE, BASE_NONE, NULL, 0,
        "TM_SignallingInfo", HFILL }},
    { &hf_rrc_tfs_SignallingMode_01,
      { "tfs-SignallingMode", "rrc.tfs_SignallingMode",
        FT_UINT32, BASE_DEC, VALS(rrc_T_tfs_SignallingMode_01_vals), 0,
        "T_tfs_SignallingMode_01", HFILL }},
    { &hf_rrc_dl_TransportChannelType_04,
      { "dl-TransportChannelType", "rrc.dl_TransportChannelType",
        FT_UINT32, BASE_DEC, VALS(rrc_DL_TrCH_TypeId1_r5_vals), 0,
        "DL_TrCH_TypeId1_r5", HFILL }},
    { &hf_rrc_tfs_SignallingMode_02,
      { "tfs-SignallingMode", "rrc.tfs_SignallingMode",
        FT_UINT32, BASE_DEC, VALS(rrc_T_tfs_SignallingMode_02_vals), 0,
        "T_tfs_SignallingMode_02", HFILL }},
    { &hf_rrc_hsdsch_02,
      { "hsdsch", "rrc.hsdsch",
        FT_NONE, BASE_NONE, NULL, 0,
        "HSDSCH_Info", HFILL }},
    { &hf_rrc_tfs_SignallingMode_03,
      { "tfs-SignallingMode", "rrc.tfs_SignallingMode",
        FT_UINT32, BASE_DEC, VALS(rrc_T_tfs_SignallingMode_03_vals), 0,
        "T_tfs_SignallingMode_03", HFILL }},
    { &hf_rrc_hsdsch_03,
      { "hsdsch", "rrc.hsdsch",
        FT_NONE, BASE_NONE, NULL, 0,
        "HSDSCH_Info_r7", HFILL }},
    { &hf_rrc_tfs_SignallingMode_04,
      { "tfs-SignallingMode", "rrc.tfs_SignallingMode",
        FT_UINT32, BASE_DEC, VALS(rrc_T_tfs_SignallingMode_04_vals), 0,
        "T_tfs_SignallingMode_04", HFILL }},
    { &hf_rrc_hsdsch_04,
      { "hsdsch", "rrc.hsdsch",
        FT_NONE, BASE_NONE, NULL, 0,
        "HSDSCH_Info_r9", HFILL }},
    { &hf_rrc_hsdsch_05,
      { "hsdsch", "rrc.hsdsch",
        FT_NONE, BASE_NONE, NULL, 0,
        "HSDSCH_Info_TDD128_v9c0ext", HFILL }},
    { &hf_rrc_tfs_SignallingMode_05,
      { "tfs-SignallingMode", "rrc.tfs_SignallingMode",
        FT_UINT32, BASE_DEC, VALS(rrc_T_tfs_SignallingMode_05_vals), 0,
        "T_tfs_SignallingMode_05", HFILL }},
    { &hf_rrc_hsdsch_06,
      { "hsdsch", "rrc.hsdsch",
        FT_NONE, BASE_NONE, NULL, 0,
        "HSDSCH_Info_r11", HFILL }},
    { &hf_rrc_transportChannelIdentity,
      { "transportChannelIdentity", "rrc.transportChannelIdentity",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_tfs_SignallingMode_06,
      { "tfs-SignallingMode", "rrc.tfs_SignallingMode",
        FT_UINT32, BASE_DEC, VALS(rrc_T_tfs_SignallingMode_06_vals), 0,
        "T_tfs_SignallingMode_06", HFILL }},
    { &hf_rrc_qualityTarget,
      { "qualityTarget", "rrc.qualityTarget",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_sccpch_TFCS,
      { "sccpch-TFCS", "rrc.sccpch_TFCS",
        FT_UINT32, BASE_DEC, VALS(rrc_TFCS_vals), 0,
        "TFCS", HFILL }},
    { &hf_rrc_modeSpecificInfo_19,
      { "modeSpecificInfo", "rrc.modeSpecificInfo",
        FT_UINT32, BASE_DEC, VALS(rrc_T_modeSpecificInfo_19_vals), 0,
        "T_modeSpecificInfo_19", HFILL }},
    { &hf_rrc_fdd_50,
      { "fdd", "rrc.fdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_fdd_49", HFILL }},
    { &hf_rrc_dl_Parameters,
      { "dl-Parameters", "rrc.dl_Parameters",
        FT_UINT32, BASE_DEC, VALS(rrc_T_dl_Parameters_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_dl_DCH_TFCS,
      { "dl-DCH-TFCS", "rrc.dl_DCH_TFCS",
        FT_UINT32, BASE_DEC, VALS(rrc_TFCS_vals), 0,
        "TFCS", HFILL }},
    { &hf_rrc_sameAsUL,
      { "sameAsUL", "rrc.sameAsUL",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_tdd_15,
      { "tdd", "rrc.tdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_tdd_14", HFILL }},
    { &hf_rrc_individualDL_CCTrCH_InfoList,
      { "individualDL-CCTrCH-InfoList", "rrc.individualDL_CCTrCH_InfoList",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_modeSpecificInfo_20,
      { "modeSpecificInfo", "rrc.modeSpecificInfo",
        FT_UINT32, BASE_DEC, VALS(rrc_T_modeSpecificInfo_20_vals), 0,
        "T_modeSpecificInfo_20", HFILL }},
    { &hf_rrc_fdd_51,
      { "fdd", "rrc.fdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_fdd_50", HFILL }},
    { &hf_rrc_dl_Parameters_01,
      { "dl-Parameters", "rrc.dl_Parameters",
        FT_UINT32, BASE_DEC, VALS(rrc_T_dl_Parameters_01_vals), 0,
        "T_dl_Parameters_01", HFILL }},
    { &hf_rrc_dl_DCH_TFCS_01,
      { "dl-DCH-TFCS", "rrc.dl_DCH_TFCS",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_tfcs,
      { "tfcs", "rrc.tfcs",
        FT_UINT32, BASE_DEC, VALS(rrc_TFCS_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_tdd_16,
      { "tdd", "rrc.tdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_tdd_15", HFILL }},
    { &hf_rrc_DL_DeletedTransChInfoList_item,
      { "DL-TransportChannelIdentity", "rrc.DL_TransportChannelIdentity",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_DL_DeletedTransChInfoList_r5_item,
      { "DL-TransportChannelIdentity-r5", "rrc.DL_TransportChannelIdentity_r5",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_DL_DeletedTransChInfoList_r7_item,
      { "DL-TransportChannelIdentity-r7", "rrc.DL_TransportChannelIdentity_r7",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_dl_TransportChannelIdentity,
      { "dl-TransportChannelIdentity", "rrc.dl_TransportChannelIdentity",
        FT_UINT32, BASE_DEC, NULL, 0,
        "TransportChannelIdentity", HFILL }},
    { &hf_rrc_dl_TransportChannelType_05,
      { "dl-TransportChannelType", "rrc.dl_TransportChannelType",
        FT_UINT32, BASE_DEC, VALS(rrc_DL_TrCH_TypeId2_r5_vals), 0,
        "DL_TrCH_TypeId2_r5", HFILL }},
    { &hf_rrc_dl_TransportChannelType_06,
      { "dl-TransportChannelType", "rrc.dl_TransportChannelType",
        FT_UINT32, BASE_DEC, VALS(rrc_DL_TrCH_TypeId2_r7_vals), 0,
        "DL_TrCH_TypeId2_r7", HFILL }},
    { &hf_rrc_hsdsch_07,
      { "hsdsch", "rrc.hsdsch",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_hsdsch_08,
      { "hsdsch", "rrc.hsdsch",
        FT_UINT32, BASE_DEC, VALS(rrc_T_hsdsch_01_vals), 0,
        "T_hsdsch_01", HFILL }},
    { &hf_rrc_transmissionTimeValidity,
      { "transmissionTimeValidity", "rrc.transmissionTimeValidity",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_timeDurationBeforeRetry,
      { "timeDurationBeforeRetry", "rrc.timeDurationBeforeRetry",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_drac_ClassIdentity,
      { "drac-ClassIdentity", "rrc.drac_ClassIdentity",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_DRAC_StaticInformationList_item,
      { "DRAC-StaticInformation", "rrc.DRAC_StaticInformation",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_transmissionGrantType,
      { "transmissionGrantType", "rrc.transmissionGrantType",
        FT_UINT32, BASE_DEC, VALS(rrc_T_transmissionGrantType_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_non_ScheduledTransGrantInfo,
      { "non-ScheduledTransGrantInfo", "rrc.non_ScheduledTransGrantInfo",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_maxMAC_e_PDUContents,
      { "maxMAC-e-PDUContents", "rrc.maxMAC_e_PDUContents",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_1_19982", HFILL }},
    { &hf_rrc_ms2_NonSchedTransmGrantHARQAlloc,
      { "ms2-NonSchedTransmGrantHARQAlloc", "rrc.ms2_NonSchedTransmGrantHARQAlloc",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_8", HFILL }},
    { &hf_rrc_scheduledTransmissionGrantInfo,
      { "scheduledTransmissionGrantInfo", "rrc.scheduledTransmissionGrantInfo",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_mac_d_FlowRetransTimer,
      { "mac-d-FlowRetransTimer", "rrc.mac_d_FlowRetransTimer",
        FT_UINT32, BASE_DEC|BASE_EXT_STRING, &rrc_E_DCH_MAC_d_FlowRetransTimer_vals_ext, 0,
        "E_DCH_MAC_d_FlowRetransTimer", HFILL }},
    { &hf_rrc_transmissionGrantType_01,
      { "transmissionGrantType", "rrc.transmissionGrantType",
        FT_UINT32, BASE_DEC, VALS(rrc_T_transmissionGrantType_01_vals), 0,
        "T_transmissionGrantType_01", HFILL }},
    { &hf_rrc_non_ScheduledTransGrantInfo_01,
      { "non-ScheduledTransGrantInfo", "rrc.non_ScheduledTransGrantInfo",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_non_ScheduledTransGrantInfo_01", HFILL }},
    { &hf_rrc_modeSpecificInfo_21,
      { "modeSpecificInfo", "rrc.modeSpecificInfo",
        FT_UINT32, BASE_DEC, VALS(rrc_T_modeSpecificInfo_21_vals), 0,
        "T_modeSpecificInfo_21", HFILL }},
    { &hf_rrc_fdd_52,
      { "fdd", "rrc.fdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_fdd_51", HFILL }},
    { &hf_rrc_E_DCH_AddReconf_MAC_d_FlowList_item,
      { "E-DCH-AddReconf-MAC-d-Flow", "rrc.E_DCH_AddReconf_MAC_d_Flow",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_E_DCH_AddReconf_MAC_d_FlowList_r7_item,
      { "E-DCH-AddReconf-MAC-d-Flow-r7", "rrc.E_DCH_AddReconf_MAC_d_Flow_r7",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_complete_26,
      { "complete", "rrc.complete",
        FT_NONE, BASE_NONE, NULL, 0,
        "TFCS_ReconfAdd", HFILL }},
    { &hf_rrc_addition,
      { "addition", "rrc.addition",
        FT_NONE, BASE_NONE, NULL, 0,
        "TFCS_ReconfAdd", HFILL }},
    { &hf_rrc_removal,
      { "removal", "rrc.removal",
        FT_UINT32, BASE_DEC, NULL, 0,
        "TFCS_RemovalList", HFILL }},
    { &hf_rrc_replacement,
      { "replacement", "rrc.replacement",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_tfcsRemoval,
      { "tfcsRemoval", "rrc.tfcsRemoval",
        FT_UINT32, BASE_DEC, NULL, 0,
        "TFCS_RemovalList", HFILL }},
    { &hf_rrc_tfcsAdd,
      { "tfcsAdd", "rrc.tfcsAdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "TFCS_ReconfAdd", HFILL }},
    { &hf_rrc_signalledGainFactors,
      { "signalledGainFactors", "rrc.signalledGainFactors",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_computedGainFactors,
      { "computedGainFactors", "rrc.computedGainFactors",
        FT_UINT32, BASE_DEC, NULL, 0,
        "ReferenceTFC_ID", HFILL }},
    { &hf_rrc_addOrReconfMAC_dFlow,
      { "addOrReconfMAC-dFlow", "rrc.addOrReconfMAC_dFlow",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_dl_MAC_HeaderType,
      { "dl-MAC-HeaderType", "rrc.dl_MAC_HeaderType",
        FT_UINT32, BASE_DEC, VALS(rrc_T_dl_MAC_HeaderType_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_mac_hs_02,
      { "mac-hs", "rrc.mac_hs",
        FT_NONE, BASE_NONE, NULL, 0,
        "AddOrReconfMAC_dFlow", HFILL }},
    { &hf_rrc_mac_ehs_02,
      { "mac-ehs", "rrc.mac_ehs",
        FT_NONE, BASE_NONE, NULL, 0,
        "AddOrReconfMAC_ehs_ReordQ", HFILL }},
    { &hf_rrc_dl_MAC_HeaderType_01,
      { "dl-MAC-HeaderType", "rrc.dl_MAC_HeaderType",
        FT_UINT32, BASE_DEC, VALS(rrc_T_dl_MAC_HeaderType_01_vals), 0,
        "T_dl_MAC_HeaderType_01", HFILL }},
    { &hf_rrc_mac_ehs_03,
      { "mac-ehs", "rrc.mac_ehs",
        FT_NONE, BASE_NONE, NULL, 0,
        "AddOrReconfMAC_ehs_ReordQ_r9", HFILL }},
    { &hf_rrc_mac_ehs_04,
      { "mac-ehs", "rrc.mac_ehs",
        FT_NONE, BASE_NONE, NULL, 0,
        "AddOrReconfMAC_ehs_ReordQ_TDD128_v9c0ext", HFILL }},
    { &hf_rrc_dl_MAC_HeaderType_02,
      { "dl-MAC-HeaderType", "rrc.dl_MAC_HeaderType",
        FT_UINT32, BASE_DEC, VALS(rrc_T_dl_MAC_HeaderType_02_vals), 0,
        "T_dl_MAC_HeaderType_02", HFILL }},
    { &hf_rrc_mac_ehs_05,
      { "mac-ehs", "rrc.mac_ehs",
        FT_NONE, BASE_NONE, NULL, 0,
        "AddOrReconfMAC_ehs_ReordQ_r11", HFILL }},
    { &hf_rrc_numberOfProcesses,
      { "numberOfProcesses", "rrc.numberOfProcesses",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_1_8", HFILL }},
    { &hf_rrc_memoryPartitioning,
      { "memoryPartitioning", "rrc.memoryPartitioning",
        FT_UINT32, BASE_DEC, VALS(rrc_T_memoryPartitioning_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_implicit,
      { "implicit", "rrc.implicit",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_explicit,
      { "explicit", "rrc.explicit",
        FT_UINT32, BASE_DEC, NULL, 0,
        "SEQUENCE_SIZE_1_maxHProcesses_OF_HARQMemorySize", HFILL }},
    { &hf_rrc_explicit_item,
      { "HARQMemorySize", "rrc.HARQMemorySize",
        FT_UINT32, BASE_DEC|BASE_EXT_STRING, &rrc_HARQMemorySize_vals_ext, 0,
        NULL, HFILL }},
    { &hf_rrc_numberOfProcesses_01,
      { "numberOfProcesses", "rrc.numberOfProcesses",
        FT_UINT32, BASE_DEC, VALS(rrc_T_numberOfProcesses_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_memoryPartitioning_01,
      { "memoryPartitioning", "rrc.memoryPartitioning",
        FT_UINT32, BASE_DEC, VALS(rrc_T_memoryPartitioning_01_vals), 0,
        "T_memoryPartitioning_01", HFILL }},
    { &hf_rrc_explicit_01,
      { "explicit", "rrc.explicit",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_memorySize,
      { "memorySize", "rrc.memorySize",
        FT_UINT32, BASE_DEC, NULL, 0,
        "SEQUENCE_SIZE_1_maxHProcesses_OF_HARQMemorySize", HFILL }},
    { &hf_rrc_memorySize_item,
      { "HARQMemorySize", "rrc.HARQMemorySize",
        FT_UINT32, BASE_DEC|BASE_EXT_STRING, &rrc_HARQMemorySize_vals_ext, 0,
        NULL, HFILL }},
    { &hf_rrc_additionalMemorySizesForMIMO,
      { "additionalMemorySizesForMIMO", "rrc.additionalMemorySizesForMIMO",
        FT_UINT32, BASE_DEC, NULL, 0,
        "SEQUENCE_SIZE_1_maxHProcesses_OF_HARQMemorySize", HFILL }},
    { &hf_rrc_additionalMemorySizesForMIMO_item,
      { "HARQMemorySize", "rrc.HARQMemorySize",
        FT_UINT32, BASE_DEC|BASE_EXT_STRING, &rrc_HARQMemorySize_vals_ext, 0,
        NULL, HFILL }},
    { &hf_rrc_numberOfProcesses_02,
      { "numberOfProcesses", "rrc.numberOfProcesses",
        FT_UINT32, BASE_DEC, VALS(rrc_T_numberOfProcesses_01_vals), 0,
        "T_numberOfProcesses_01", HFILL }},
    { &hf_rrc_memoryPartitioning_02,
      { "memoryPartitioning", "rrc.memoryPartitioning",
        FT_UINT32, BASE_DEC, VALS(rrc_T_memoryPartitioning_02_vals), 0,
        "T_memoryPartitioning_02", HFILL }},
    { &hf_rrc_explicit_02,
      { "explicit", "rrc.explicit",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_explicit_01", HFILL }},
    { &hf_rrc_additionalMemorySizesThirdMIMOStream,
      { "additionalMemorySizesThirdMIMOStream", "rrc.additionalMemorySizesThirdMIMOStream",
        FT_UINT32, BASE_DEC, NULL, 0,
        "SEQUENCE_SIZE_1_maxHProcesses_OF_HARQMemorySize", HFILL }},
    { &hf_rrc_additionalMemorySizesThirdMIMOStream_item,
      { "HARQMemorySize", "rrc.HARQMemorySize",
        FT_UINT32, BASE_DEC|BASE_EXT_STRING, &rrc_HARQMemorySize_vals_ext, 0,
        NULL, HFILL }},
    { &hf_rrc_additionalMemorySizesFourthMIMOStream,
      { "additionalMemorySizesFourthMIMOStream", "rrc.additionalMemorySizesFourthMIMOStream",
        FT_UINT32, BASE_DEC, NULL, 0,
        "SEQUENCE_SIZE_1_maxHProcesses_OF_HARQMemorySize", HFILL }},
    { &hf_rrc_additionalMemorySizesFourthMIMOStream_item,
      { "HARQMemorySize", "rrc.HARQMemorySize",
        FT_UINT32, BASE_DEC|BASE_EXT_STRING, &rrc_HARQMemorySize_vals_ext, 0,
        NULL, HFILL }},
    { &hf_rrc_dl_TFCS_Identity,
      { "dl-TFCS-Identity", "rrc.dl_TFCS_Identity",
        FT_NONE, BASE_NONE, NULL, 0,
        "TFCS_Identity", HFILL }},
    { &hf_rrc_tfcs_SignallingMode,
      { "tfcs-SignallingMode", "rrc.tfcs_SignallingMode",
        FT_UINT32, BASE_DEC, VALS(rrc_T_tfcs_SignallingMode_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_explicit_config_01,
      { "explicit-config", "rrc.explicit_config",
        FT_UINT32, BASE_DEC, VALS(rrc_TFCS_vals), 0,
        "TFCS", HFILL }},
    { &hf_rrc_sameAsUL_01,
      { "sameAsUL", "rrc.sameAsUL",
        FT_NONE, BASE_NONE, NULL, 0,
        "TFCS_Identity", HFILL }},
    { &hf_rrc_IndividualDL_CCTrCH_InfoList_item,
      { "IndividualDL-CCTrCH-Info", "rrc.IndividualDL_CCTrCH_Info",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_ul_TFCS_Identity,
      { "ul-TFCS-Identity", "rrc.ul_TFCS_Identity",
        FT_NONE, BASE_NONE, NULL, 0,
        "TFCS_Identity", HFILL }},
    { &hf_rrc_ul_TFCS,
      { "ul-TFCS", "rrc.ul_TFCS",
        FT_UINT32, BASE_DEC, VALS(rrc_TFCS_vals), 0,
        "TFCS", HFILL }},
    { &hf_rrc_tfc_Subset,
      { "tfc-Subset", "rrc.tfc_Subset",
        FT_UINT32, BASE_DEC, VALS(rrc_TFC_Subset_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_IndividualUL_CCTrCH_InfoList_item,
      { "IndividualUL-CCTrCH-Info", "rrc.IndividualUL_CCTrCH_Info",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_logChOfRb,
      { "logChOfRb", "rrc.logChOfRb",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_1", HFILL }},
    { &hf_rrc_explicitList_01,
      { "explicitList", "rrc.explicitList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "SEQUENCE_SIZE_1_15_OF_LogicalChannelByRB", HFILL }},
    { &hf_rrc_explicitList_item,
      { "LogicalChannelByRB", "rrc.LogicalChannelByRB",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_dch_transport_ch_id,
      { "dch-transport-ch-id", "rrc.dch_transport_ch_id",
        FT_UINT32, BASE_DEC, NULL, 0,
        "TransportChannelIdentity", HFILL }},
    { &hf_rrc_hsdsch_mac_d_flow_id,
      { "hsdsch-mac-d-flow-id", "rrc.hsdsch_mac_d_flow_id",
        FT_UINT32, BASE_DEC, NULL, 0,
        "MAC_d_FlowIdentity", HFILL }},
    { &hf_rrc_MAC_d_PDU_SizeInfo_List_item,
      { "MAC-d-PDUsizeInfo", "rrc.MAC_d_PDUsizeInfo",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_mac_d_PDU_Size,
      { "mac-d-PDU-Size", "rrc.mac_d_PDU_Size",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_1_5000", HFILL }},
    { &hf_rrc_mac_d_PDU_Index,
      { "mac-d-PDU-Index", "rrc.mac_d_PDU_Index",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_7", HFILL }},
    { &hf_rrc_MAC_hs_AddReconfQueue_List_item,
      { "MAC-hs-AddReconfQueue", "rrc.MAC_hs_AddReconfQueue",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_mac_hsQueueId,
      { "mac-hsQueueId", "rrc.mac_hsQueueId",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_7", HFILL }},
    { &hf_rrc_mac_dFlowId,
      { "mac-dFlowId", "rrc.mac_dFlowId",
        FT_UINT32, BASE_DEC, NULL, 0,
        "MAC_d_FlowIdentity", HFILL }},
    { &hf_rrc_reorderingReleaseTimer,
      { "reorderingReleaseTimer", "rrc.reorderingReleaseTimer",
        FT_UINT32, BASE_DEC, VALS(rrc_T1_ReleaseTimer_vals), 0,
        "T1_ReleaseTimer", HFILL }},
    { &hf_rrc_mac_hsWindowSize,
      { "mac-hsWindowSize", "rrc.mac_hsWindowSize",
        FT_UINT32, BASE_DEC, VALS(rrc_MAC_hs_WindowSize_vals), 0,
        "MAC_hs_WindowSize", HFILL }},
    { &hf_rrc_mac_d_PDU_SizeInfo_List,
      { "mac-d-PDU-SizeInfo-List", "rrc.mac_d_PDU_SizeInfo_List",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_MAC_hs_DelQueue_List_item,
      { "MAC-hs-DelQueue", "rrc.MAC_hs_DelQueue",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_MAC_ehs_AddReconfReordQ_List_item,
      { "MAC-ehs-AddReconfReordQ", "rrc.MAC_ehs_AddReconfReordQ",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_MAC_ehs_AddReconfReordQ_List_r9_item,
      { "MAC-ehs-AddReconfReordQ-r9", "rrc.MAC_ehs_AddReconfReordQ_r9",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_MAC_ehs_AddReconfReordQ_List_TDD128_v9c0ext_item,
      { "MAC-ehs-AddReconfReordQ-TDD128-v9c0ext", "rrc.MAC_ehs_AddReconfReordQ_TDD128_v9c0ext",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_MAC_ehs_AddReconfReordQ_List_r11_item,
      { "MAC-ehs-AddReconfReordQ-r11", "rrc.MAC_ehs_AddReconfReordQ_r11",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_mac_ehsWindowSize_extension,
      { "mac-ehsWindowSize-extension", "rrc.mac_ehsWindowSize_extension",
        FT_UINT32, BASE_DEC, VALS(rrc_MAC_ehs_WindowSize_TDD128_v9c0ext_vals), 0,
        "MAC_ehs_WindowSize_TDD128_v9c0ext", HFILL }},
    { &hf_rrc_mac_ehsWindowSize_02,
      { "mac-ehsWindowSize", "rrc.mac_ehsWindowSize",
        FT_UINT32, BASE_DEC, VALS(rrc_MAC_hs_WindowSize_r11_vals), 0,
        "MAC_hs_WindowSize_r11", HFILL }},
    { &hf_rrc_MAC_ehs_DelReordQ_List_item,
      { "MAC-ehs-DelReordQ", "rrc.MAC_ehs_DelReordQ",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_hsdsch_mac_ehs_QueueId,
      { "hsdsch-mac-ehs-QueueId", "rrc.hsdsch_mac_ehs_QueueId",
        FT_UINT32, BASE_DEC, NULL, 0,
        "MAC_ehs_QueueId", HFILL }},
    { &hf_rrc_NumberOfTbSizeAndTTIList_item,
      { "NumberOfTbSizeAndTTIList item", "rrc.NumberOfTbSizeAndTTIList_item",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_numberOfTransportBlocks,
      { "numberOfTransportBlocks", "rrc.numberOfTransportBlocks",
        FT_UINT32, BASE_DEC, VALS(rrc_NumberOfTransportBlocks_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_transmissionTimeInterval,
      { "transmissionTimeInterval", "rrc.transmissionTimeInterval",
        FT_UINT32, BASE_DEC, VALS(rrc_TransmissionTimeInterval_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_Non_allowedTFC_List_item,
      { "TFC-Value", "rrc.TFC_Value",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_zero,
      { "zero", "rrc.zero",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_one,
      { "one", "rrc.one",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_small,
      { "small", "rrc.small",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_2_17", HFILL }},
    { &hf_rrc_large,
      { "large", "rrc.large",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_18_512", HFILL }},
    { &hf_rrc_sizeType1_01,
      { "sizeType1", "rrc.sizeType1",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_31", HFILL }},
    { &hf_rrc_sizeType2_01,
      { "sizeType2", "rrc.sizeType2",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_sizeType2_01", HFILL }},
    { &hf_rrc_part1_03,
      { "part1", "rrc.part1",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_23", HFILL }},
    { &hf_rrc_part2_03,
      { "part2", "rrc.part2",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_1_3", HFILL }},
    { &hf_rrc_sizeType3_01,
      { "sizeType3", "rrc.sizeType3",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_sizeType3_01", HFILL }},
    { &hf_rrc_part1_04,
      { "part1", "rrc.part1",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_61", HFILL }},
    { &hf_rrc_sizeType2_02,
      { "sizeType2", "rrc.sizeType2",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_63", HFILL }},
    { &hf_rrc_sizeType3_02,
      { "sizeType3", "rrc.sizeType3",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_56", HFILL }},
    { &hf_rrc_referenceTFC,
      { "referenceTFC", "rrc.referenceTFC",
        FT_UINT32, BASE_DEC, NULL, 0,
        "TFC_Value", HFILL }},
    { &hf_rrc_modeSpecificInfo_22,
      { "modeSpecificInfo", "rrc.modeSpecificInfo",
        FT_UINT32, BASE_DEC, VALS(rrc_T_modeSpecificInfo_22_vals), 0,
        "T_modeSpecificInfo_22", HFILL }},
    { &hf_rrc_fdd_53,
      { "fdd", "rrc.fdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_fdd_52", HFILL }},
    { &hf_rrc_gainFactorBetaC,
      { "gainFactorBetaC", "rrc.gainFactorBetaC",
        FT_UINT32, BASE_DEC, NULL, 0,
        "GainFactor", HFILL }},
    { &hf_rrc_gainFactorBetaD,
      { "gainFactorBetaD", "rrc.gainFactorBetaD",
        FT_UINT32, BASE_DEC, NULL, 0,
        "GainFactor", HFILL }},
    { &hf_rrc_gainFactorInformation,
      { "gainFactorInformation", "rrc.gainFactorInformation",
        FT_UINT32, BASE_DEC, VALS(rrc_GainFactorInformation_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_powerOffsetPp_m,
      { "powerOffsetPp-m", "rrc.powerOffsetPp_m",
        FT_INT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_ul_AddReconfTrChInfoList,
      { "ul-AddReconfTrChInfoList", "rrc.ul_AddReconfTrChInfoList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "UL_AddReconfTransChInfoList", HFILL }},
    { &hf_rrc_dl_TrChInfoList,
      { "dl-TrChInfoList", "rrc.dl_TrChInfoList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "DL_AddReconfTransChInfoList", HFILL }},
    { &hf_rrc_bler_QualityValue,
      { "bler-QualityValue", "rrc.bler_QualityValue",
        FT_INT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_ul_TransportChannelType_01,
      { "ul-TransportChannelType", "rrc.ul_TransportChannelType",
        FT_UINT32, BASE_DEC, VALS(rrc_UL_TrCH_Type_vals), 0,
        "UL_TrCH_Type", HFILL }},
    { &hf_rrc_restrictedTrChIdentity,
      { "restrictedTrChIdentity", "rrc.restrictedTrChIdentity",
        FT_UINT32, BASE_DEC, NULL, 0,
        "TransportChannelIdentity", HFILL }},
    { &hf_rrc_allowedTFI_List,
      { "allowedTFI-List", "rrc.allowedTFI_List",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_RestrictedTrChInfoList_item,
      { "RestrictedTrChInfo", "rrc.RestrictedTrChInfo",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_channelCodingType,
      { "channelCodingType", "rrc.channelCodingType",
        FT_UINT32, BASE_DEC, VALS(rrc_ChannelCodingType_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_rateMatchingAttribute,
      { "rateMatchingAttribute", "rrc.rateMatchingAttribute",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_crc_Size,
      { "crc-Size", "rrc.crc_Size",
        FT_UINT32, BASE_DEC, VALS(rrc_CRC_Size_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_modeSpecificInfo_23,
      { "modeSpecificInfo", "rrc.modeSpecificInfo",
        FT_UINT32, BASE_DEC, VALS(rrc_T_modeSpecificInfo_23_vals), 0,
        "T_modeSpecificInfo_23", HFILL }},
    { &hf_rrc_fdd_54,
      { "fdd", "rrc.fdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_fdd_53", HFILL }},
    { &hf_rrc_referenceTFC_ID,
      { "referenceTFC-ID", "rrc.referenceTFC_ID",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_splitType,
      { "splitType", "rrc.splitType",
        FT_UINT32, BASE_DEC, VALS(rrc_SplitType_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_tfci_Field2_Length,
      { "tfci-Field2-Length", "rrc.tfci_Field2_Length",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_1_10", HFILL }},
    { &hf_rrc_tfci_Field1_Information,
      { "tfci-Field1-Information", "rrc.tfci_Field1_Information",
        FT_UINT32, BASE_DEC, VALS(rrc_ExplicitTFCS_Configuration_vals), 0,
        "ExplicitTFCS_Configuration", HFILL }},
    { &hf_rrc_tfci_Field2_Information,
      { "tfci-Field2-Information", "rrc.tfci_Field2_Information",
        FT_UINT32, BASE_DEC, VALS(rrc_TFCI_Field2_Information_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_minimumAllowedTFC_Number,
      { "minimumAllowedTFC-Number", "rrc.minimumAllowedTFC_Number",
        FT_UINT32, BASE_DEC, NULL, 0,
        "TFC_Value", HFILL }},
    { &hf_rrc_allowedTFC_List,
      { "allowedTFC-List", "rrc.allowedTFC_List",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_non_allowedTFC_List,
      { "non-allowedTFC-List", "rrc.non_allowedTFC_List",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_restrictedTrChInfoList,
      { "restrictedTrChInfoList", "rrc.restrictedTrChInfoList",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_fullTFCS,
      { "fullTFCS", "rrc.fullTFCS",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_TFC_SubsetList_item,
      { "TFC-SubsetList item", "rrc.TFC_SubsetList_item",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_modeSpecificInfo_24,
      { "modeSpecificInfo", "rrc.modeSpecificInfo",
        FT_UINT32, BASE_DEC, VALS(rrc_T_modeSpecificInfo_24_vals), 0,
        "T_modeSpecificInfo_24", HFILL }},
    { &hf_rrc_tdd_17,
      { "tdd", "rrc.tdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_tdd_16", HFILL }},
    { &hf_rrc_tfci_Range,
      { "tfci-Range", "rrc.tfci_Range",
        FT_UINT32, BASE_DEC, NULL, 0,
        "TFCI_RangeList", HFILL }},
    { &hf_rrc_explicit_config_02,
      { "explicit-config", "rrc.explicit_config",
        FT_UINT32, BASE_DEC, VALS(rrc_ExplicitTFCS_Configuration_vals), 0,
        "ExplicitTFCS_Configuration", HFILL }},
    { &hf_rrc_maxTFCIField2Value,
      { "maxTFCIField2Value", "rrc.maxTFCIField2Value",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_1_1023", HFILL }},
    { &hf_rrc_tfcs_InfoForDSCH,
      { "tfcs-InfoForDSCH", "rrc.tfcs_InfoForDSCH",
        FT_UINT32, BASE_DEC, VALS(rrc_TFCS_InfoForDSCH_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_TFCI_RangeList_item,
      { "TFCI-Range", "rrc.TFCI_Range",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_normalTFCI_Signalling,
      { "normalTFCI-Signalling", "rrc.normalTFCI_Signalling",
        FT_UINT32, BASE_DEC, VALS(rrc_ExplicitTFCS_Configuration_vals), 0,
        "ExplicitTFCS_Configuration", HFILL }},
    { &hf_rrc_dummy_36,
      { "dummy", "rrc.dummy",
        FT_NONE, BASE_NONE, NULL, 0,
        "SplitTFCI_Signalling", HFILL }},
    { &hf_rrc_tfcs_ID_01,
      { "tfcs-ID", "rrc.tfcs_ID",
        FT_UINT32, BASE_DEC, NULL, 0,
        "TFCS_IdentityPlain", HFILL }},
    { &hf_rrc_sharedChannelIndicator,
      { "sharedChannelIndicator", "rrc.sharedChannelIndicator",
        FT_BOOLEAN, BASE_NONE, NULL, 0,
        "BOOLEAN", HFILL }},
    { &hf_rrc_ctfc2bit,
      { "ctfc2bit", "rrc.ctfc2bit",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_3", HFILL }},
    { &hf_rrc_ctfc4bit,
      { "ctfc4bit", "rrc.ctfc4bit",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_15", HFILL }},
    { &hf_rrc_ctfc6bit,
      { "ctfc6bit", "rrc.ctfc6bit",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_63", HFILL }},
    { &hf_rrc_ctfc8bit,
      { "ctfc8bit", "rrc.ctfc8bit",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_255", HFILL }},
    { &hf_rrc_ctfc12bit,
      { "ctfc12bit", "rrc.ctfc12bit",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_4095", HFILL }},
    { &hf_rrc_ctfc16bit,
      { "ctfc16bit", "rrc.ctfc16bit",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_65535", HFILL }},
    { &hf_rrc_ctfc24bit,
      { "ctfc24bit", "rrc.ctfc24bit",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_16777215", HFILL }},
    { &hf_rrc_ctfcSize,
      { "ctfcSize", "rrc.ctfcSize",
        FT_UINT32, BASE_DEC, VALS(rrc_T_ctfcSize_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_ctfc2Bit,
      { "ctfc2Bit", "rrc.ctfc2Bit",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_ctfc2Bit_item,
      { "ctfc2Bit item", "rrc.ctfc2Bit_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_ctfc2Bit_item", HFILL }},
    { &hf_rrc_ctfc2,
      { "ctfc2", "rrc.ctfc2",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_3", HFILL }},
    { &hf_rrc_powerOffsetInformation,
      { "powerOffsetInformation", "rrc.powerOffsetInformation",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_ctfc4Bit,
      { "ctfc4Bit", "rrc.ctfc4Bit",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_ctfc4Bit_item,
      { "ctfc4Bit item", "rrc.ctfc4Bit_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_ctfc4Bit_item", HFILL }},
    { &hf_rrc_ctfc4,
      { "ctfc4", "rrc.ctfc4",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_15", HFILL }},
    { &hf_rrc_ctfc6Bit,
      { "ctfc6Bit", "rrc.ctfc6Bit",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_ctfc6Bit_item,
      { "ctfc6Bit item", "rrc.ctfc6Bit_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_ctfc6Bit_item", HFILL }},
    { &hf_rrc_ctfc6,
      { "ctfc6", "rrc.ctfc6",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_63", HFILL }},
    { &hf_rrc_ctfc8Bit,
      { "ctfc8Bit", "rrc.ctfc8Bit",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_ctfc8Bit_item,
      { "ctfc8Bit item", "rrc.ctfc8Bit_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_ctfc8Bit_item", HFILL }},
    { &hf_rrc_ctfc8,
      { "ctfc8", "rrc.ctfc8",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_255", HFILL }},
    { &hf_rrc_ctfc12Bit,
      { "ctfc12Bit", "rrc.ctfc12Bit",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_ctfc12Bit_item,
      { "ctfc12Bit item", "rrc.ctfc12Bit_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_ctfc12Bit_item", HFILL }},
    { &hf_rrc_ctfc12,
      { "ctfc12", "rrc.ctfc12",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_4095", HFILL }},
    { &hf_rrc_ctfc16Bit,
      { "ctfc16Bit", "rrc.ctfc16Bit",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_ctfc16Bit_item,
      { "ctfc16Bit item", "rrc.ctfc16Bit_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_ctfc16Bit_item", HFILL }},
    { &hf_rrc_ctfc16,
      { "ctfc16", "rrc.ctfc16",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_65535", HFILL }},
    { &hf_rrc_ctfc24Bit,
      { "ctfc24Bit", "rrc.ctfc24Bit",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_ctfc24Bit_item,
      { "ctfc24Bit item", "rrc.ctfc24Bit_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_ctfc24Bit_item", HFILL }},
    { &hf_rrc_ctfc24,
      { "ctfc24", "rrc.ctfc24",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_16777215", HFILL }},
    { &hf_rrc_tfci,
      { "tfci", "rrc.tfci",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_1023", HFILL }},
    { &hf_rrc_TFCS_RemovalList_item,
      { "TFCS-Removal", "rrc.TFCS_Removal",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_messType,
      { "messType", "rrc.messType",
        FT_UINT32, BASE_DEC, VALS(rrc_MessType_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_tm_SignallingMode,
      { "tm-SignallingMode", "rrc.tm_SignallingMode",
        FT_UINT32, BASE_DEC, VALS(rrc_T_tm_SignallingMode_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_mode1,
      { "mode1", "rrc.mode1",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_mode2,
      { "mode2", "rrc.mode2",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_ul_controlledTrChList,
      { "ul-controlledTrChList", "rrc.ul_controlledTrChList",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_dsch_transport_ch_id,
      { "dsch-transport-ch-id", "rrc.dsch_transport_ch_id",
        FT_UINT32, BASE_DEC, NULL, 0,
        "TransportChannelIdentity", HFILL }},
    { &hf_rrc_dedicatedTransChTFS,
      { "dedicatedTransChTFS", "rrc.dedicatedTransChTFS",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_commonTransChTFS,
      { "commonTransChTFS", "rrc.commonTransChTFS",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_commonTransChTFS_LCR,
      { "commonTransChTFS-LCR", "rrc.commonTransChTFS_LCR",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_UL_AddReconfTransChInfoList_item,
      { "UL-AddReconfTransChInformation", "rrc.UL_AddReconfTransChInformation",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_UL_AddReconfTransChInfoList_r6_item,
      { "UL-AddReconfTransChInformation-r6", "rrc.UL_AddReconfTransChInformation_r6",
        FT_UINT32, BASE_DEC, VALS(rrc_UL_AddReconfTransChInformation_r6_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_UL_AddReconfTransChInfoList_r7_item,
      { "UL-AddReconfTransChInformation-r7", "rrc.UL_AddReconfTransChInformation_r7",
        FT_UINT32, BASE_DEC, VALS(rrc_UL_AddReconfTransChInformation_r7_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_UL_AddReconfTransChInfoList_r8_item,
      { "UL-AddReconfTransChInformation-r8", "rrc.UL_AddReconfTransChInformation_r8",
        FT_UINT32, BASE_DEC, VALS(rrc_UL_AddReconfTransChInformation_r8_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_transportFormatSet,
      { "transportFormatSet", "rrc.transportFormatSet",
        FT_UINT32, BASE_DEC, VALS(rrc_TransportFormatSet_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_dch_usch,
      { "dch-usch", "rrc.dch_usch",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_e_dch_02,
      { "e-dch", "rrc.e_dch",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_e_dch_02", HFILL }},
    { &hf_rrc_tti_03,
      { "tti", "rrc.tti",
        FT_UINT32, BASE_DEC, VALS(rrc_E_DCH_TTI_vals), 0,
        "E_DCH_TTI", HFILL }},
    { &hf_rrc_harq_Info,
      { "harq-Info", "rrc.harq_Info",
        FT_UINT32, BASE_DEC, VALS(rrc_T_harq_Info_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_addReconf_MAC_d_FlowList,
      { "addReconf-MAC-d-FlowList", "rrc.addReconf_MAC_d_FlowList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "E_DCH_AddReconf_MAC_d_FlowList", HFILL }},
    { &hf_rrc_dch_usch_01,
      { "dch-usch", "rrc.dch_usch",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_dch_usch_01", HFILL }},
    { &hf_rrc_e_dch_03,
      { "e-dch", "rrc.e_dch",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_e_dch_03", HFILL }},
    { &hf_rrc_modeSpecific,
      { "modeSpecific", "rrc.modeSpecific",
        FT_UINT32, BASE_DEC, VALS(rrc_T_modeSpecific_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_fdd_55,
      { "fdd", "rrc.fdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_fdd_54", HFILL }},
    { &hf_rrc_harq_Info_01,
      { "harq-Info", "rrc.harq_Info",
        FT_UINT32, BASE_DEC, VALS(rrc_T_harq_Info_01_vals), 0,
        "T_harq_Info_01", HFILL }},
    { &hf_rrc_addReconf_MAC_d_FlowList_01,
      { "addReconf-MAC-d-FlowList", "rrc.addReconf_MAC_d_FlowList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "E_DCH_AddReconf_MAC_d_FlowList_r7", HFILL }},
    { &hf_rrc_dch_usch_02,
      { "dch-usch", "rrc.dch_usch",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_dch_usch_02", HFILL }},
    { &hf_rrc_e_dch_04,
      { "e-dch", "rrc.e_dch",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_e_dch_04", HFILL }},
    { &hf_rrc_ul_MAC_HeaderType,
      { "ul-MAC-HeaderType", "rrc.ul_MAC_HeaderType",
        FT_UINT32, BASE_DEC, VALS(rrc_T_ul_MAC_HeaderType_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_modeSpecific_01,
      { "modeSpecific", "rrc.modeSpecific",
        FT_UINT32, BASE_DEC, VALS(rrc_T_modeSpecific_01_vals), 0,
        "T_modeSpecific_01", HFILL }},
    { &hf_rrc_fdd_56,
      { "fdd", "rrc.fdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_fdd_55", HFILL }},
    { &hf_rrc_harq_Info_02,
      { "harq-Info", "rrc.harq_Info",
        FT_UINT32, BASE_DEC, VALS(rrc_T_harq_Info_02_vals), 0,
        "T_harq_Info_02", HFILL }},
    { &hf_rrc_prach_TFCS,
      { "prach-TFCS", "rrc.prach_TFCS",
        FT_UINT32, BASE_DEC, VALS(rrc_TFCS_vals), 0,
        "TFCS", HFILL }},
    { &hf_rrc_modeSpecificInfo_25,
      { "modeSpecificInfo", "rrc.modeSpecificInfo",
        FT_UINT32, BASE_DEC, VALS(rrc_T_modeSpecificInfo_25_vals), 0,
        "T_modeSpecificInfo_25", HFILL }},
    { &hf_rrc_fdd_57,
      { "fdd", "rrc.fdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_fdd_56", HFILL }},
    { &hf_rrc_tdd_18,
      { "tdd", "rrc.tdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_tdd_17", HFILL }},
    { &hf_rrc_individualUL_CCTrCH_InfoList,
      { "individualUL-CCTrCH-InfoList", "rrc.individualUL_CCTrCH_InfoList",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_modeSpecificInfo_26,
      { "modeSpecificInfo", "rrc.modeSpecificInfo",
        FT_UINT32, BASE_DEC, VALS(rrc_T_modeSpecificInfo_26_vals), 0,
        "T_modeSpecificInfo_26", HFILL }},
    { &hf_rrc_fdd_58,
      { "fdd", "rrc.fdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_fdd_57", HFILL }},
    { &hf_rrc_tdd_19,
      { "tdd", "rrc.tdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_tdd_18", HFILL }},
    { &hf_rrc_tfc_SubsetList,
      { "tfc-SubsetList", "rrc.tfc_SubsetList",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_UL_ControlledTrChList_item,
      { "TransportChannelIdentity", "rrc.TransportChannelIdentity",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_UL_DeletedTransChInfoList_item,
      { "UL-TransportChannelIdentity", "rrc.UL_TransportChannelIdentity",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_UL_DeletedTransChInfoList_r6_item,
      { "UL-TransportChannelIdentity-r6", "rrc.UL_TransportChannelIdentity_r6",
        FT_UINT32, BASE_DEC, VALS(rrc_UL_TransportChannelIdentity_r6_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_ul_TransportChannelIdentity,
      { "ul-TransportChannelIdentity", "rrc.ul_TransportChannelIdentity",
        FT_UINT32, BASE_DEC, NULL, 0,
        "TransportChannelIdentity", HFILL }},
    { &hf_rrc_dch_usch_03,
      { "dch-usch", "rrc.dch_usch",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_dch_usch_03", HFILL }},
    { &hf_rrc_e_dch_05,
      { "e-dch", "rrc.e_dch",
        FT_UINT32, BASE_DEC, NULL, 0,
        "E_DCH_MAC_d_FlowIdentity", HFILL }},
    { &hf_rrc_USCH_TransportChannelsInfo_item,
      { "USCH-TransportChannelsInfo item", "rrc.USCH_TransportChannelsInfo_item",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_usch_TransportChannelIdentity,
      { "usch-TransportChannelIdentity", "rrc.usch_TransportChannelIdentity",
        FT_UINT32, BASE_DEC, NULL, 0,
        "TransportChannelIdentity", HFILL }},
    { &hf_rrc_usch_TFS,
      { "usch-TFS", "rrc.usch_TFS",
        FT_UINT32, BASE_DEC, VALS(rrc_TransportFormatSet_vals), 0,
        "TransportFormatSet", HFILL }},
    { &hf_rrc_AC_To_ASC_MappingTable_item,
      { "AC-To-ASC-Mapping", "rrc.AC_To_ASC_Mapping",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_availableSignatureStartIndex,
      { "availableSignatureStartIndex", "rrc.availableSignatureStartIndex",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_15", HFILL }},
    { &hf_rrc_availableSignatureEndIndex,
      { "availableSignatureEndIndex", "rrc.availableSignatureEndIndex",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_15", HFILL }},
    { &hf_rrc_assignedSubChannelNumber,
      { "assignedSubChannelNumber", "rrc.assignedSubChannelNumber",
        FT_BYTES, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_channelisationCodeIndices,
      { "channelisationCodeIndices", "rrc.channelisationCodeIndices",
        FT_BYTES, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_subchannelSize,
      { "subchannelSize", "rrc.subchannelSize",
        FT_UINT32, BASE_DEC, VALS(rrc_T_subchannelSize_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_size1,
      { "size1", "rrc.size1",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_size2,
      { "size2", "rrc.size2",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_subchannels,
      { "subchannels", "rrc.subchannels",
        FT_UINT32, BASE_DEC, VALS(rrc_T_subchannels_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_size4,
      { "size4", "rrc.size4",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_subchannels_01,
      { "subchannels", "rrc.subchannels",
        FT_BYTES, BASE_NONE, NULL, 0,
        "T_subchannels_01", HFILL }},
    { &hf_rrc_size8,
      { "size8", "rrc.size8",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_subchannels_02,
      { "subchannels", "rrc.subchannels",
        FT_BYTES, BASE_NONE, NULL, 0,
        "T_subchannels_02", HFILL }},
    { &hf_rrc_channelisationCodeIndices_01,
      { "channelisationCodeIndices", "rrc.channelisationCodeIndices",
        FT_BYTES, BASE_NONE, NULL, 0,
        "T_channelisationCodeIndices_01", HFILL }},
    { &hf_rrc_subchannelSize_01,
      { "subchannelSize", "rrc.subchannelSize",
        FT_UINT32, BASE_DEC, VALS(rrc_T_subchannelSize_01_vals), 0,
        "T_subchannelSize_01", HFILL }},
    { &hf_rrc_size2_01,
      { "size2", "rrc.size2",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_size2_01", HFILL }},
    { &hf_rrc_subchannels_03,
      { "subchannels", "rrc.subchannels",
        FT_UINT32, BASE_DEC, VALS(rrc_T_subchannels_03_vals), 0,
        "T_subchannels_03", HFILL }},
    { &hf_rrc_size4_01,
      { "size4", "rrc.size4",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_size4_01", HFILL }},
    { &hf_rrc_subchannels_04,
      { "subchannels", "rrc.subchannels",
        FT_BYTES, BASE_NONE, NULL, 0,
        "T_subchannels_04", HFILL }},
    { &hf_rrc_size8_01,
      { "size8", "rrc.size8",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_size8_01", HFILL }},
    { &hf_rrc_subchannels_05,
      { "subchannels", "rrc.subchannels",
        FT_BYTES, BASE_NONE, NULL, 0,
        "T_subchannels_05", HFILL }},
    { &hf_rrc_size16,
      { "size16", "rrc.size16",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_subchannels_06,
      { "subchannels", "rrc.subchannels",
        FT_BYTES, BASE_NONE, NULL, 0,
        "T_subchannels_06", HFILL }},
    { &hf_rrc_availableSYNC_UlCodesIndics,
      { "availableSYNC-UlCodesIndics", "rrc.availableSYNC_UlCodesIndics",
        FT_BYTES, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_subchannelSize_02,
      { "subchannelSize", "rrc.subchannelSize",
        FT_UINT32, BASE_DEC, VALS(rrc_T_subchannelSize_02_vals), 0,
        "T_subchannelSize_02", HFILL }},
    { &hf_rrc_size2_02,
      { "size2", "rrc.size2",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_size2_02", HFILL }},
    { &hf_rrc_subchannels_07,
      { "subchannels", "rrc.subchannels",
        FT_UINT32, BASE_DEC, VALS(rrc_T_subchannels_07_vals), 0,
        "T_subchannels_07", HFILL }},
    { &hf_rrc_size4_02,
      { "size4", "rrc.size4",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_size4_02", HFILL }},
    { &hf_rrc_subchannels_08,
      { "subchannels", "rrc.subchannels",
        FT_BYTES, BASE_NONE, NULL, 0,
        "T_subchannels_08", HFILL }},
    { &hf_rrc_size8_02,
      { "size8", "rrc.size8",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_size8_02", HFILL }},
    { &hf_rrc_subchannels_09,
      { "subchannels", "rrc.subchannels",
        FT_BYTES, BASE_NONE, NULL, 0,
        "T_subchannels_09", HFILL }},
    { &hf_rrc_dynamicTFInformationCCCH,
      { "dynamicTFInformationCCCH", "rrc.dynamicTFInformationCCCH",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_additionalPRACH_TF_and_TFCS_CCCH_IEs,
      { "additionalPRACH-TF-and-TFCS-CCCH-IEs", "rrc.additionalPRACH_TF_and_TFCS_CCCH_IEs",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_AdditionalPRACH_TF_and_TFCS_CCCH_List_item,
      { "AdditionalPRACH-TF-and-TFCS-CCCH", "rrc.AdditionalPRACH_TF_and_TFCS_CCCH",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_AdditionalDLSecCellInfoListFDD_item,
      { "AdditionalDLSecCellInfoFDD", "rrc.AdditionalDLSecCellInfoFDD",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_AdditionalDLSecCellInfoListFDD_r11_item,
      { "AdditionalDLSecCellInfoFDD-r11", "rrc.AdditionalDLSecCellInfoFDD_r11",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_AdditionalDLSecCellInfoListFDD2_r11_item,
      { "AdditionalDLSecCellInfoFDD-r11", "rrc.AdditionalDLSecCellInfoFDD_r11",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_channelisationCode256,
      { "channelisationCode256", "rrc.channelisationCode256",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_sttd_Indicator,
      { "sttd-Indicator", "rrc.sttd_Indicator",
        FT_BOOLEAN, BASE_NONE, NULL, 0,
        "BOOLEAN", HFILL }},
    { &hf_rrc_aich_TransmissionTiming,
      { "aich-TransmissionTiming", "rrc.aich_TransmissionTiming",
        FT_UINT32, BASE_DEC, VALS(rrc_AICH_TransmissionTiming_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_allocationActivationTime,
      { "allocationActivationTime", "rrc.allocationActivationTime",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_255", HFILL }},
    { &hf_rrc_allocationDuration,
      { "allocationDuration", "rrc.allocationDuration",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_1_256", HFILL }},
    { &hf_rrc_antenna3And4_S_CPICH,
      { "antenna3And4-S-CPICH", "rrc.antenna3And4_S_CPICH",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_antenna3And4_D_CPICH,
      { "antenna3And4-D-CPICH", "rrc.antenna3And4_D_CPICH",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_antenna3_S_CPICH,
      { "antenna3-S-CPICH", "rrc.antenna3_S_CPICH",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_antenna4_S_CPICH,
      { "antenna4-S-CPICH", "rrc.antenna4_S_CPICH",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_s_cpich_PowerOffset_Mimo,
      { "s-cpich-PowerOffset-Mimo", "rrc.s_cpich_PowerOffset_Mimo",
        FT_INT32, BASE_DEC, NULL, 0,
        "S_CPICH_PowerOffset_4x4MIMO", HFILL }},
    { &hf_rrc_antenna3_D_CPICH,
      { "antenna3-D-CPICH", "rrc.antenna3_D_CPICH",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_antenna4_D_CPICH,
      { "antenna4-D-CPICH", "rrc.antenna4_D_CPICH",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_d_cpich_PowerOffset_Mimo,
      { "d-cpich-PowerOffset-Mimo", "rrc.d_cpich_PowerOffset_Mimo",
        FT_INT32, BASE_DEC, NULL, 0,
        "D_CPICH_PowerOffset_4x4MIMO", HFILL }},
    { &hf_rrc_initialD_CPICHStatus,
      { "initialD-CPICHStatus", "rrc.initialD_CPICHStatus",
        FT_UINT32, BASE_DEC, VALS(rrc_T_initialD_CPICHStatus_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_channelisationCode,
      { "channelisationCode", "rrc.channelisationCode",
        FT_UINT32, BASE_DEC, NULL, 0,
        "ChannelisationCode256", HFILL }},
    { &hf_rrc_ap_Signature,
      { "ap-Signature", "rrc.ap_Signature",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_availableAP_SubchannelList,
      { "availableAP-SubchannelList", "rrc.availableAP_SubchannelList",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_accessServiceClass_FDD,
      { "accessServiceClass-FDD", "rrc.accessServiceClass_FDD",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_accessServiceClass_TDD,
      { "accessServiceClass-TDD", "rrc.accessServiceClass_TDD",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_accessServiceClass_TDD_01,
      { "accessServiceClass-TDD", "rrc.accessServiceClass_TDD",
        FT_NONE, BASE_NONE, NULL, 0,
        "AccessServiceClass_TDD_r7", HFILL }},
    { &hf_rrc_accessServiceClass_TDD_LCR,
      { "accessServiceClass-TDD-LCR", "rrc.accessServiceClass_TDD_LCR",
        FT_NONE, BASE_NONE, NULL, 0,
        "AccessServiceClass_TDD_LCR_r4", HFILL }},
    { &hf_rrc_AvailableAP_Signature_VCAMList_item,
      { "AP-Signature-VCAM", "rrc.AP_Signature_VCAM",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_AvailableAP_SignatureList_item,
      { "AP-Signature", "rrc.AP_Signature",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_AvailableAP_SubchannelList_item,
      { "AP-Subchannel", "rrc.AP_Subchannel",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_AvailableMinimumSF_ListVCAM_item,
      { "AvailableMinimumSF-VCAM", "rrc.AvailableMinimumSF_VCAM",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_minimumSpreadingFactor,
      { "minimumSpreadingFactor", "rrc.minimumSpreadingFactor",
        FT_UINT32, BASE_DEC, VALS(rrc_MinimumSpreadingFactor_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_nf_Max,
      { "nf-Max", "rrc.nf_Max",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_maxAvailablePCPCH_Number,
      { "maxAvailablePCPCH-Number", "rrc.maxAvailablePCPCH_Number",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_availableAP_Signature_VCAMList,
      { "availableAP-Signature-VCAMList", "rrc.availableAP_Signature_VCAMList",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_tfcs_Identity,
      { "tfcs-Identity", "rrc.tfcs_Identity",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_ul_DPCH_PowerControlInfo,
      { "ul-DPCH-PowerControlInfo", "rrc.ul_DPCH_PowerControlInfo",
        FT_UINT32, BASE_DEC, VALS(rrc_UL_DPCH_PowerControlInfo_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_ul_DPCH_PowerControlInfo_01,
      { "ul-DPCH-PowerControlInfo", "rrc.ul_DPCH_PowerControlInfo",
        FT_UINT32, BASE_DEC, VALS(rrc_UL_DPCH_PowerControlInfo_r4_vals), 0,
        "UL_DPCH_PowerControlInfo_r4", HFILL }},
    { &hf_rrc_ul_DPCH_PowerControlInfo_02,
      { "ul-DPCH-PowerControlInfo", "rrc.ul_DPCH_PowerControlInfo",
        FT_UINT32, BASE_DEC, VALS(rrc_UL_DPCH_PowerControlInfo_r5_vals), 0,
        "UL_DPCH_PowerControlInfo_r5", HFILL }},
    { &hf_rrc_ul_DPCH_PowerControlInfo_03,
      { "ul-DPCH-PowerControlInfo", "rrc.ul_DPCH_PowerControlInfo",
        FT_UINT32, BASE_DEC, VALS(rrc_UL_DPCH_PowerControlInfo_r7_vals), 0,
        "UL_DPCH_PowerControlInfo_r7", HFILL }},
    { &hf_rrc_CD_AccessSlotSubchannelList_item,
      { "CD-AccessSlotSubchannel", "rrc.CD_AccessSlotSubchannel",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_CD_SignatureCodeList_item,
      { "CD-SignatureCode", "rrc.CD_SignatureCode",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_burstType,
      { "burstType", "rrc.burstType",
        FT_UINT32, BASE_DEC, VALS(rrc_BurstType_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_midambleShift,
      { "midambleShift", "rrc.midambleShift",
        FT_UINT32, BASE_DEC, NULL, 0,
        "MidambleShiftLong", HFILL }},
    { &hf_rrc_timeslot,
      { "timeslot", "rrc.timeslot",
        FT_UINT32, BASE_DEC, NULL, 0,
        "TimeslotNumber", HFILL }},
    { &hf_rrc_cellParametersID,
      { "cellParametersID", "rrc.cellParametersID",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_notActive,
      { "notActive", "rrc.notActive",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_isActive,
      { "isActive", "rrc.isActive",
        FT_UINT32, BASE_DEC, NULL, 0,
        "AvailableMinimumSF_ListVCAM", HFILL }},
    { &hf_rrc_availableAP_SignatureList,
      { "availableAP-SignatureList", "rrc.availableAP_SignatureList",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_pdsch_CodeMapList,
      { "pdsch-CodeMapList", "rrc.pdsch_CodeMapList",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_start_code,
      { "start-code", "rrc.start_code",
        FT_UINT32, BASE_DEC, VALS(rrc_HS_ChannelisationCode_LCR_vals), 0,
        "HS_ChannelisationCode_LCR", HFILL }},
    { &hf_rrc_stop_code,
      { "stop-code", "rrc.stop_code",
        FT_UINT32, BASE_DEC, VALS(rrc_HS_ChannelisationCode_LCR_vals), 0,
        "HS_ChannelisationCode_LCR", HFILL }},
    { &hf_rrc_s_offset,
      { "s-offset", "rrc.s_offset",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_9", HFILL }},
    { &hf_rrc_f_dpch_ChannelisationCodeNumber,
      { "f-dpch-ChannelisationCodeNumber", "rrc.f_dpch_ChannelisationCodeNumber",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_255", HFILL }},
    { &hf_rrc_e_RGCH_Information_01,
      { "e-RGCH-Information", "rrc.e_RGCH_Information",
        FT_NONE, BASE_NONE, NULL, 0,
        "E_RGCH_Information_CommonEdch", HFILL }},
    { &hf_rrc_e_hich_Info,
      { "e-hich-Info", "rrc.e_hich_Info",
        FT_NONE, BASE_NONE, NULL, 0,
        "E_HICH_Information_CommonEdch", HFILL }},
    { &hf_rrc_ul_DPCH_CodeInfoForCommonEDCH,
      { "ul-DPCH-CodeInfoForCommonEDCH", "rrc.ul_DPCH_CodeInfoForCommonEDCH",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_twoMsHarqConfiguration,
      { "twoMsHarqConfiguration", "rrc.twoMsHarqConfiguration",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_Common_E_RNTI_Info_item,
      { "Common-E-RNTI-Info item", "rrc.Common_E_RNTI_Info_item",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_starting_E_RNTI,
      { "starting-E-RNTI", "rrc.starting_E_RNTI",
        FT_BYTES, BASE_NONE, NULL, 0,
        "E_RNTI", HFILL }},
    { &hf_rrc_number_of_group,
      { "number-of-group", "rrc.number_of_group",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_1_maxERNTIgroup", HFILL }},
    { &hf_rrc_number_of_ENRTI_per_group,
      { "number-of-ENRTI-per-group", "rrc.number_of_ENRTI_per_group",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_1_maxERNTIperGroup", HFILL }},
    { &hf_rrc_scheduledTransmissionConfiguration,
      { "scheduledTransmissionConfiguration", "rrc.scheduledTransmissionConfiguration",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_cOffset,
      { "cOffset", "rrc.cOffset",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_29", HFILL }},
    { &hf_rrc_ul_InterferenceForCommonEDCH,
      { "ul-InterferenceForCommonEDCH", "rrc.ul_InterferenceForCommonEDCH",
        FT_INT32, BASE_DEC, NULL, 0,
        "UL_Interference", HFILL }},
    { &hf_rrc_common_E_DCH_MAC_d_FlowList,
      { "common-E-DCH-MAC-d-FlowList", "rrc.common_E_DCH_MAC_d_FlowList",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_modeSpecificInfo_27,
      { "modeSpecificInfo", "rrc.modeSpecificInfo",
        FT_UINT32, BASE_DEC, VALS(rrc_T_modeSpecificInfo_27_vals), 0,
        "T_modeSpecificInfo_27", HFILL }},
    { &hf_rrc_dummy_37,
      { "dummy", "rrc.dummy",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_dummy_14", HFILL }},
    { &hf_rrc_tdd_20,
      { "tdd", "rrc.tdd",
        FT_UINT32, BASE_DEC, VALS(rrc_T_tdd_19_vals), 0,
        "T_tdd_19", HFILL }},
    { &hf_rrc_tdd768_06,
      { "tdd768", "rrc.tdd768",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_tdd384_11,
      { "tdd384", "rrc.tdd384",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_tdd128_11,
      { "tdd128", "rrc.tdd128",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_tdd128_11", HFILL }},
    { &hf_rrc_e_RUCCH_Info,
      { "e-RUCCH-Info", "rrc.e_RUCCH_Info",
        FT_NONE, BASE_NONE, NULL, 0,
        "E_RUCCH_Info_TDD128", HFILL }},
    { &hf_rrc_e_PUCH_Info,
      { "e-PUCH-Info", "rrc.e_PUCH_Info",
        FT_NONE, BASE_NONE, NULL, 0,
        "E_PUCH_Info_TDD128", HFILL }},
    { &hf_rrc_e_hich_Information,
      { "e-hich-Information", "rrc.e_hich_Information",
        FT_NONE, BASE_NONE, NULL, 0,
        "E_HICH_Information_TDD128", HFILL }},
    { &hf_rrc_e_agch_Information,
      { "e-agch-Information", "rrc.e_agch_Information",
        FT_NONE, BASE_NONE, NULL, 0,
        "E_AGCH_Information_TDD128", HFILL }},
    { &hf_rrc_harq_Info_03,
      { "harq-Info", "rrc.harq_Info",
        FT_UINT32, BASE_DEC, VALS(rrc_T_harq_Info_03_vals), 0,
        "T_harq_Info_03", HFILL }},
    { &hf_rrc_ccch_transmission_Info,
      { "ccch-transmission-Info", "rrc.ccch_transmission_Info",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_common_e_rnti_Info,
      { "common-e-rnti-Info", "rrc.common_e_rnti_Info",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_harq_MaximumNumberOfRetransmissions,
      { "harq-MaximumNumberOfRetransmissions", "rrc.harq_MaximumNumberOfRetransmissions",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_7", HFILL }},
    { &hf_rrc_harq_retransmission_timer,
      { "harq-retransmission-timer", "rrc.harq_retransmission_timer",
        FT_UINT32, BASE_DEC|BASE_EXT_STRING, &rrc_T_harq_retransmission_timer_vals_ext, 0,
        NULL, HFILL }},
    { &hf_rrc_harq_power_offset,
      { "harq-power-offset", "rrc.harq_power_offset",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_6", HFILL }},
    { &hf_rrc_prach_PreambleForEnhancedUplink,
      { "prach-PreambleForEnhancedUplink", "rrc.prach_PreambleForEnhancedUplink",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_initialServingGrantValue,
      { "initialServingGrantValue", "rrc.initialServingGrantValue",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_37", HFILL }},
    { &hf_rrc_e_dch_TTI,
      { "e-dch-TTI", "rrc.e_dch_TTI",
        FT_UINT32, BASE_DEC, VALS(rrc_E_DCH_TTI_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_e_agch_Information_01,
      { "e-agch-Information", "rrc.e_agch_Information",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_harq_Info_04,
      { "harq-Info", "rrc.harq_Info",
        FT_UINT32, BASE_DEC, VALS(rrc_T_harq_Info_04_vals), 0,
        "T_harq_Info_04", HFILL }},
    { &hf_rrc_ul_DPCHpowerControlInfoForCommonEDCH,
      { "ul-DPCHpowerControlInfoForCommonEDCH", "rrc.ul_DPCHpowerControlInfoForCommonEDCH",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_e_dpcch_Info,
      { "e-dpcch-Info", "rrc.e_dpcch_Info",
        FT_NONE, BASE_NONE, NULL, 0,
        "E_DPCCH_Info_r7", HFILL }},
    { &hf_rrc_e_dpdch_Info,
      { "e-dpdch-Info", "rrc.e_dpdch_Info",
        FT_NONE, BASE_NONE, NULL, 0,
        "E_DPDCH_Info_r8", HFILL }},
    { &hf_rrc_dl_FDPCH_TPCcommandErrorRate,
      { "dl-FDPCH-TPCcommandErrorRate", "rrc.dl_FDPCH_TPCcommandErrorRate",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_1_10", HFILL }},
    { &hf_rrc_additional_E_DCH_TransmitBackoff,
      { "additional-E-DCH-TransmitBackoff", "rrc.additional_E_DCH_TransmitBackoff",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_15", HFILL }},
    { &hf_rrc_max_CCCH_ResourceAllocation,
      { "max-CCCH-ResourceAllocation", "rrc.max_CCCH_ResourceAllocation",
        FT_UINT32, BASE_DEC, VALS(rrc_T_max_CCCH_ResourceAllocation_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_max_PeriodForCollisionResolution,
      { "max-PeriodForCollisionResolution", "rrc.max_PeriodForCollisionResolution",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_8_24", HFILL }},
    { &hf_rrc_e_dch_TransmitContinuationOffset,
      { "e-dch-TransmitContinuationOffset", "rrc.e_dch_TransmitContinuationOffset",
        FT_UINT32, BASE_DEC, VALS(rrc_T_e_dch_TransmitContinuationOffset_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_ack_nack_support_on_HS_DPCCH,
      { "ack-nack-support-on-HS-DPCCH", "rrc.ack_nack_support_on_HS_DPCCH",
        FT_BOOLEAN, BASE_NONE, NULL, 0,
        "BOOLEAN", HFILL }},
    { &hf_rrc_measurement_Feedback_Info,
      { "measurement-Feedback-Info", "rrc.measurement_Feedback_Info",
        FT_NONE, BASE_NONE, NULL, 0,
        "Measurement_Feedback_Info_r7", HFILL }},
    { &hf_rrc_common_E_DCH_ResourceInfoList,
      { "common-E-DCH-ResourceInfoList", "rrc.common_E_DCH_ResourceInfoList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "SEQUENCE_SIZE_1_maxEDCHs_OF_Common_E_DCH_ResourceInfoList", HFILL }},
    { &hf_rrc_common_E_DCH_ResourceInfoList_item,
      { "Common-E-DCH-ResourceInfoList", "rrc.Common_E_DCH_ResourceInfoList",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_common_E_DCH_MAC_d_Flow_Info_List_ConcurrentTTI,
      { "common-E-DCH-MAC-d-Flow-Info-List-ConcurrentTTI", "rrc.common_E_DCH_MAC_d_Flow_Info_List_ConcurrentTTI",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_ul_DPCHpowerControlInfoConcurrentTTI,
      { "ul-DPCHpowerControlInfoConcurrentTTI", "rrc.ul_DPCHpowerControlInfoConcurrentTTI",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_e_DPCCH_DPCCH_PowerOffset,
      { "e-DPCCH-DPCCH-PowerOffset", "rrc.e_DPCCH_DPCCH_PowerOffset",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_max_CCCH_ResourceAllocation_01,
      { "max-CCCH-ResourceAllocation", "rrc.max_CCCH_ResourceAllocation",
        FT_UINT32, BASE_DEC, VALS(rrc_T_max_CCCH_ResourceAllocation_01_vals), 0,
        "T_max_CCCH_ResourceAllocation_01", HFILL }},
    { &hf_rrc_e_dch_TransmitContinuationOffset_01,
      { "e-dch-TransmitContinuationOffset", "rrc.e_dch_TransmitContinuationOffset",
        FT_UINT32, BASE_DEC, VALS(rrc_T_e_dch_TransmitContinuationOffset_01_vals), 0,
        "T_e_dch_TransmitContinuationOffset_01", HFILL }},
    { &hf_rrc_measurement_Feedback_Info_ConcurrentTTI,
      { "measurement-Feedback-Info-ConcurrentTTI", "rrc.measurement_Feedback_Info_ConcurrentTTI",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_channelisationCode_01,
      { "channelisationCode", "rrc.channelisationCode",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_127", HFILL }},
    { &hf_rrc_signatureSequence,
      { "signatureSequence", "rrc.signatureSequence",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_39", HFILL }},
    { &hf_rrc_configurationInfo,
      { "configurationInfo", "rrc.configurationInfo",
        FT_UINT32, BASE_DEC, VALS(rrc_T_configurationInfo_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_newConfiguration_01,
      { "newConfiguration", "rrc.newConfiguration",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_newConfiguration_01", HFILL }},
    { &hf_rrc_eRGCHNeighbourCellList,
      { "eRGCHNeighbourCellList", "rrc.eRGCHNeighbourCellList",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_maxCellMeas", HFILL }},
    { &hf_rrc_commonERGCHChannelConfigList,
      { "commonERGCHChannelConfigList", "rrc.commonERGCHChannelConfigList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "SEQUENCE_SIZE_1_maxCellMeas_OF_CommonERGCHChannelConfig", HFILL }},
    { &hf_rrc_commonERGCHChannelConfigList_item,
      { "CommonERGCHChannelConfig", "rrc.CommonERGCHChannelConfig",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_minimumServingGrantValue,
      { "minimumServingGrantValue", "rrc.minimumServingGrantValue",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_37", HFILL }},
    { &hf_rrc_reportingRange,
      { "reportingRange", "rrc.reportingRange",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_filterCoefficient,
      { "filterCoefficient", "rrc.filterCoefficient",
        FT_UINT32, BASE_DEC, VALS(rrc_FilterCoefficient_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_secondInterleavingMode,
      { "secondInterleavingMode", "rrc.secondInterleavingMode",
        FT_UINT32, BASE_DEC, VALS(rrc_SecondInterleavingMode_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_tfci_Coding,
      { "tfci-Coding", "rrc.tfci_Coding",
        FT_UINT32, BASE_DEC, VALS(rrc_TFCI_Coding_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_puncturingLimit,
      { "puncturingLimit", "rrc.puncturingLimit",
        FT_UINT32, BASE_DEC, VALS(rrc_PuncturingLimit_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_repetitionPeriodAndLength,
      { "repetitionPeriodAndLength", "rrc.repetitionPeriodAndLength",
        FT_UINT32, BASE_DEC, VALS(rrc_RepetitionPeriodAndLength_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_repetitionPeriodLengthAndOffset,
      { "repetitionPeriodLengthAndOffset", "rrc.repetitionPeriodLengthAndOffset",
        FT_UINT32, BASE_DEC, VALS(rrc_RepetitionPeriodLengthAndOffset_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_controlChannelDrxOperation,
      { "controlChannelDrxOperation", "rrc.controlChannelDrxOperation",
        FT_UINT32, BASE_DEC, VALS(rrc_T_controlChannelDrxOperation_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_continue_01,
      { "continue", "rrc.continue",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_enablingDelay,
      { "enablingDelay", "rrc.enablingDelay",
        FT_UINT32, BASE_DEC, VALS(rrc_EnablingDelay_TDD128_vals), 0,
        "EnablingDelay_TDD128", HFILL }},
    { &hf_rrc_newOperation,
      { "newOperation", "rrc.newOperation",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_hS_SCCH_Drx_Info,
      { "hS-SCCH-Drx-Info", "rrc.hS_SCCH_Drx_Info",
        FT_NONE, BASE_NONE, NULL, 0,
        "HS_SCCH_DRX_Info_TDD128", HFILL }},
    { &hf_rrc_e_AGCH_Drx_Info,
      { "e-AGCH-Drx-Info", "rrc.e_AGCH_Drx_Info",
        FT_NONE, BASE_NONE, NULL, 0,
        "E_AGCH_DRX_Info_TDD128", HFILL }},
    { &hf_rrc_cpch_SetID,
      { "cpch-SetID", "rrc.cpch_SetID",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_dynamicPersistenceLevelTF_List,
      { "dynamicPersistenceLevelTF-List", "rrc.dynamicPersistenceLevelTF_List",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_CPCH_PersistenceLevelsList_item,
      { "CPCH-PersistenceLevels", "rrc.CPCH_PersistenceLevels",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_ap_PreambleScramblingCode,
      { "ap-PreambleScramblingCode", "rrc.ap_PreambleScramblingCode",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_ap_AICH_ChannelisationCode,
      { "ap-AICH-ChannelisationCode", "rrc.ap_AICH_ChannelisationCode",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_cd_PreambleScramblingCode,
      { "cd-PreambleScramblingCode", "rrc.cd_PreambleScramblingCode",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_cd_CA_ICH_ChannelisationCode,
      { "cd-CA-ICH-ChannelisationCode", "rrc.cd_CA_ICH_ChannelisationCode",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_cd_AccessSlotSubchannelList,
      { "cd-AccessSlotSubchannelList", "rrc.cd_AccessSlotSubchannelList",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_cd_SignatureCodeList,
      { "cd-SignatureCodeList", "rrc.cd_SignatureCodeList",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_deltaPp_m,
      { "deltaPp-m", "rrc.deltaPp_m",
        FT_INT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_ul_DPCCH_SlotFormat,
      { "ul-DPCCH-SlotFormat", "rrc.ul_DPCCH_SlotFormat",
        FT_UINT32, BASE_DEC, VALS(rrc_UL_DPCCH_SlotFormat_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_n_StartMessage,
      { "n-StartMessage", "rrc.n_StartMessage",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_n_EOT,
      { "n-EOT", "rrc.n_EOT",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_channelAssignmentActive,
      { "channelAssignmentActive", "rrc.channelAssignmentActive",
        FT_UINT32, BASE_DEC, VALS(rrc_ChannelAssignmentActive_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_cpch_StatusIndicationMode,
      { "cpch-StatusIndicationMode", "rrc.cpch_StatusIndicationMode",
        FT_UINT32, BASE_DEC, VALS(rrc_CPCH_StatusIndicationMode_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_pcpch_ChannelInfoList,
      { "pcpch-ChannelInfoList", "rrc.pcpch_ChannelInfoList",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_CPCH_SetInfoList_item,
      { "CPCH-SetInfo", "rrc.CPCH_SetInfo",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_timeInfo,
      { "timeInfo", "rrc.timeInfo",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_commonTimeslotInfo,
      { "commonTimeslotInfo", "rrc.commonTimeslotInfo",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_dl_CCTrCH_TimeslotsCodes,
      { "dl-CCTrCH-TimeslotsCodes", "rrc.dl_CCTrCH_TimeslotsCodes",
        FT_NONE, BASE_NONE, NULL, 0,
        "DownlinkTimeslotsCodes", HFILL }},
    { &hf_rrc_ul_CCTrChTPCList,
      { "ul-CCTrChTPCList", "rrc.ul_CCTrChTPCList",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_tddOption_03,
      { "tddOption", "rrc.tddOption",
        FT_UINT32, BASE_DEC, VALS(rrc_T_tddOption_03_vals), 0,
        "T_tddOption_03", HFILL }},
    { &hf_rrc_tdd384_12,
      { "tdd384", "rrc.tdd384",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_tdd384_11", HFILL }},
    { &hf_rrc_tdd128_12,
      { "tdd128", "rrc.tdd128",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_tdd128_12", HFILL }},
    { &hf_rrc_dl_CCTrCH_TimeslotsCodes_01,
      { "dl-CCTrCH-TimeslotsCodes", "rrc.dl_CCTrCH_TimeslotsCodes",
        FT_NONE, BASE_NONE, NULL, 0,
        "DownlinkTimeslotsCodes_LCR_r4", HFILL }},
    { &hf_rrc_tddOption_04,
      { "tddOption", "rrc.tddOption",
        FT_UINT32, BASE_DEC, VALS(rrc_T_tddOption_04_vals), 0,
        "T_tddOption_04", HFILL }},
    { &hf_rrc_tdd384_13,
      { "tdd384", "rrc.tdd384",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_tdd384_12", HFILL }},
    { &hf_rrc_dl_CCTrCH_TimeslotsCodes_02,
      { "dl-CCTrCH-TimeslotsCodes", "rrc.dl_CCTrCH_TimeslotsCodes",
        FT_NONE, BASE_NONE, NULL, 0,
        "DownlinkTimeslotsCodes_r7", HFILL }},
    { &hf_rrc_tdd768_07,
      { "tdd768", "rrc.tdd768",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_tdd768_06", HFILL }},
    { &hf_rrc_dl_CCTrCH_TimeslotsCodes_03,
      { "dl-CCTrCH-TimeslotsCodes", "rrc.dl_CCTrCH_TimeslotsCodes",
        FT_NONE, BASE_NONE, NULL, 0,
        "DownlinkTimeslotsCodes_VHCR", HFILL }},
    { &hf_rrc_tdd128_13,
      { "tdd128", "rrc.tdd128",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_tdd128_13", HFILL }},
    { &hf_rrc_DL_CCTrChList_item,
      { "DL-CCTrCh", "rrc.DL_CCTrCh",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_DL_CCTrChList_r7_item,
      { "DL-CCTrCh-r7", "rrc.DL_CCTrCh_r7",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_DL_CCTrChList_r4_item,
      { "DL-CCTrCh-r4", "rrc.DL_CCTrCh_r4",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_DL_CCTrChListToRemove_item,
      { "TFCS-IdentityPlain", "rrc.TFCS_IdentityPlain",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_secondaryScramblingCode,
      { "secondaryScramblingCode", "rrc.secondaryScramblingCode",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_sf_AndCodeNumber,
      { "sf-AndCodeNumber", "rrc.sf_AndCodeNumber",
        FT_UINT32, BASE_DEC, VALS(rrc_SF512_AndCodeNumber_vals), 0,
        "SF512_AndCodeNumber", HFILL }},
    { &hf_rrc_scramblingCodeChange,
      { "scramblingCodeChange", "rrc.scramblingCodeChange",
        FT_UINT32, BASE_DEC, VALS(rrc_ScramblingCodeChange_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_DL_ChannelisationCodeList_item,
      { "DL-ChannelisationCode", "rrc.DL_ChannelisationCode",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_dl_DPCH_InfoCommon,
      { "dl-DPCH-InfoCommon", "rrc.dl_DPCH_InfoCommon",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_modeSpecificInfo_28,
      { "modeSpecificInfo", "rrc.modeSpecificInfo",
        FT_UINT32, BASE_DEC, VALS(rrc_T_modeSpecificInfo_28_vals), 0,
        "T_modeSpecificInfo_28", HFILL }},
    { &hf_rrc_fdd_59,
      { "fdd", "rrc.fdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_fdd_58", HFILL }},
    { &hf_rrc_defaultDPCH_OffsetValue,
      { "defaultDPCH-OffsetValue", "rrc.defaultDPCH_OffsetValue",
        FT_UINT32, BASE_DEC, NULL, 0,
        "DefaultDPCH_OffsetValueFDD", HFILL }},
    { &hf_rrc_dpch_CompressedModeInfo,
      { "dpch-CompressedModeInfo", "rrc.dpch_CompressedModeInfo",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_dummy_38,
      { "dummy", "rrc.dummy",
        FT_NONE, BASE_NONE, NULL, 0,
        "SSDT_Information", HFILL }},
    { &hf_rrc_tdd_21,
      { "tdd", "rrc.tdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_tdd_20", HFILL }},
    { &hf_rrc_defaultDPCH_OffsetValue_01,
      { "defaultDPCH-OffsetValue", "rrc.defaultDPCH_OffsetValue",
        FT_UINT32, BASE_DEC, NULL, 0,
        "DefaultDPCH_OffsetValueTDD", HFILL }},
    { &hf_rrc_dl_DPCH_InfoCommon_01,
      { "dl-DPCH-InfoCommon", "rrc.dl_DPCH_InfoCommon",
        FT_NONE, BASE_NONE, NULL, 0,
        "DL_DPCH_InfoCommon_r4", HFILL }},
    { &hf_rrc_modeSpecificInfo_29,
      { "modeSpecificInfo", "rrc.modeSpecificInfo",
        FT_UINT32, BASE_DEC, VALS(rrc_T_modeSpecificInfo_29_vals), 0,
        "T_modeSpecificInfo_29", HFILL }},
    { &hf_rrc_fdd_60,
      { "fdd", "rrc.fdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_fdd_59", HFILL }},
    { &hf_rrc_dummy_39,
      { "dummy", "rrc.dummy",
        FT_NONE, BASE_NONE, NULL, 0,
        "SSDT_Information_r4", HFILL }},
    { &hf_rrc_tdd_22,
      { "tdd", "rrc.tdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_tdd_21", HFILL }},
    { &hf_rrc_tddOption_05,
      { "tddOption", "rrc.tddOption",
        FT_UINT32, BASE_DEC, VALS(rrc_T_tddOption_05_vals), 0,
        "T_tddOption_05", HFILL }},
    { &hf_rrc_tdd128_14,
      { "tdd128", "rrc.tdd128",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_tdd128_14", HFILL }},
    { &hf_rrc_tstd_Indicator,
      { "tstd-Indicator", "rrc.tstd_Indicator",
        FT_BOOLEAN, BASE_NONE, NULL, 0,
        "BOOLEAN", HFILL }},
    { &hf_rrc_modeSpecificInfo_30,
      { "modeSpecificInfo", "rrc.modeSpecificInfo",
        FT_UINT32, BASE_DEC, VALS(rrc_T_modeSpecificInfo_30_vals), 0,
        "T_modeSpecificInfo_30", HFILL }},
    { &hf_rrc_fdd_61,
      { "fdd", "rrc.fdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_fdd_60", HFILL }},
    { &hf_rrc_tdd_23,
      { "tdd", "rrc.tdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_tdd_22", HFILL }},
    { &hf_rrc_tddOption_06,
      { "tddOption", "rrc.tddOption",
        FT_UINT32, BASE_DEC, VALS(rrc_T_tddOption_06_vals), 0,
        "T_tddOption_06", HFILL }},
    { &hf_rrc_tdd128_15,
      { "tdd128", "rrc.tdd128",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_tdd128_15", HFILL }},
    { &hf_rrc_mac_hsResetIndicator_05,
      { "mac-hsResetIndicator", "rrc.mac_hsResetIndicator",
        FT_UINT32, BASE_DEC, VALS(rrc_T_mac_hsResetIndicator_05_vals), 0,
        "T_mac_hsResetIndicator_05", HFILL }},
    { &hf_rrc_dl_dpchInfoCommon,
      { "dl-dpchInfoCommon", "rrc.dl_dpchInfoCommon",
        FT_UINT32, BASE_DEC, VALS(rrc_T_dl_dpchInfoCommon_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_dl_DPCH_InfoCommon_02,
      { "dl-DPCH-InfoCommon", "rrc.dl_DPCH_InfoCommon",
        FT_NONE, BASE_NONE, NULL, 0,
        "DL_DPCH_InfoCommon_r6", HFILL }},
    { &hf_rrc_dl_FDPCH_InfoCommon,
      { "dl-FDPCH-InfoCommon", "rrc.dl_FDPCH_InfoCommon",
        FT_NONE, BASE_NONE, NULL, 0,
        "DL_FDPCH_InfoCommon_r6", HFILL }},
    { &hf_rrc_modeSpecificInfo_31,
      { "modeSpecificInfo", "rrc.modeSpecificInfo",
        FT_UINT32, BASE_DEC, VALS(rrc_T_modeSpecificInfo_31_vals), 0,
        "T_modeSpecificInfo_31", HFILL }},
    { &hf_rrc_fdd_62,
      { "fdd", "rrc.fdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_fdd_61", HFILL }},
    { &hf_rrc_tdd_24,
      { "tdd", "rrc.tdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_tdd_23", HFILL }},
    { &hf_rrc_tddOption_07,
      { "tddOption", "rrc.tddOption",
        FT_UINT32, BASE_DEC, VALS(rrc_T_tddOption_07_vals), 0,
        "T_tddOption_07", HFILL }},
    { &hf_rrc_tdd128_16,
      { "tdd128", "rrc.tdd128",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_tdd128_16", HFILL }},
    { &hf_rrc_mac_hsResetIndicator_06,
      { "mac-hsResetIndicator", "rrc.mac_hsResetIndicator",
        FT_UINT32, BASE_DEC, VALS(rrc_T_mac_hsResetIndicator_06_vals), 0,
        "T_mac_hsResetIndicator_06", HFILL }},
    { &hf_rrc_postVerificationPeriod_07,
      { "postVerificationPeriod", "rrc.postVerificationPeriod",
        FT_UINT32, BASE_DEC, VALS(rrc_T_postVerificationPeriod_07_vals), 0,
        "T_postVerificationPeriod_07", HFILL }},
    { &hf_rrc_dl_dpchInfoCommon_01,
      { "dl-dpchInfoCommon", "rrc.dl_dpchInfoCommon",
        FT_UINT32, BASE_DEC, VALS(rrc_T_dl_dpchInfoCommon_01_vals), 0,
        "T_dl_dpchInfoCommon_01", HFILL }},
    { &hf_rrc_modeSpecificInfo_32,
      { "modeSpecificInfo", "rrc.modeSpecificInfo",
        FT_UINT32, BASE_DEC, VALS(rrc_T_modeSpecificInfo_32_vals), 0,
        "T_modeSpecificInfo_32", HFILL }},
    { &hf_rrc_fdd_63,
      { "fdd", "rrc.fdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_fdd_62", HFILL }},
    { &hf_rrc_tdd_25,
      { "tdd", "rrc.tdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_tdd_24", HFILL }},
    { &hf_rrc_tddOption_08,
      { "tddOption", "rrc.tddOption",
        FT_UINT32, BASE_DEC, VALS(rrc_T_tddOption_08_vals), 0,
        "T_tddOption_08", HFILL }},
    { &hf_rrc_tdd128_17,
      { "tdd128", "rrc.tdd128",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_tdd128_17", HFILL }},
    { &hf_rrc_mac_hsResetIndicator_07,
      { "mac-hsResetIndicator", "rrc.mac_hsResetIndicator",
        FT_UINT32, BASE_DEC, VALS(rrc_T_mac_hsResetIndicator_07_vals), 0,
        "T_mac_hsResetIndicator_07", HFILL }},
    { &hf_rrc_postVerificationPeriod_08,
      { "postVerificationPeriod", "rrc.postVerificationPeriod",
        FT_UINT32, BASE_DEC, VALS(rrc_T_postVerificationPeriod_08_vals), 0,
        "T_postVerificationPeriod_08", HFILL }},
    { &hf_rrc_dl_dpchInfoCommon_02,
      { "dl-dpchInfoCommon", "rrc.dl_dpchInfoCommon",
        FT_UINT32, BASE_DEC, VALS(rrc_T_dl_dpchInfoCommon_02_vals), 0,
        "T_dl_dpchInfoCommon_02", HFILL }},
    { &hf_rrc_modeSpecificInfo_33,
      { "modeSpecificInfo", "rrc.modeSpecificInfo",
        FT_UINT32, BASE_DEC, VALS(rrc_T_modeSpecificInfo_33_vals), 0,
        "T_modeSpecificInfo_33", HFILL }},
    { &hf_rrc_fdd_64,
      { "fdd", "rrc.fdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_fdd_63", HFILL }},
    { &hf_rrc_dpch_CompressedModeInfo_01,
      { "dpch-CompressedModeInfo", "rrc.dpch_CompressedModeInfo",
        FT_NONE, BASE_NONE, NULL, 0,
        "DPCH_CompressedModeInfo_r8", HFILL }},
    { &hf_rrc_tdd_26,
      { "tdd", "rrc.tdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_tdd_25", HFILL }},
    { &hf_rrc_tddOption_09,
      { "tddOption", "rrc.tddOption",
        FT_UINT32, BASE_DEC, VALS(rrc_T_tddOption_09_vals), 0,
        "T_tddOption_09", HFILL }},
    { &hf_rrc_tdd128_18,
      { "tdd128", "rrc.tdd128",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_tdd128_18", HFILL }},
    { &hf_rrc_mac_hsResetIndicator_08,
      { "mac-hsResetIndicator", "rrc.mac_hsResetIndicator",
        FT_UINT32, BASE_DEC, VALS(rrc_T_mac_hsResetIndicator_08_vals), 0,
        "T_mac_hsResetIndicator_08", HFILL }},
    { &hf_rrc_postVerificationPeriod_09,
      { "postVerificationPeriod", "rrc.postVerificationPeriod",
        FT_UINT32, BASE_DEC, VALS(rrc_T_postVerificationPeriod_09_vals), 0,
        "T_postVerificationPeriod_09", HFILL }},
    { &hf_rrc_dl_dpchInfoCommon_03,
      { "dl-dpchInfoCommon", "rrc.dl_dpchInfoCommon",
        FT_UINT32, BASE_DEC, VALS(rrc_T_dl_dpchInfoCommon_03_vals), 0,
        "T_dl_dpchInfoCommon_03", HFILL }},
    { &hf_rrc_modeSpecificInfo_34,
      { "modeSpecificInfo", "rrc.modeSpecificInfo",
        FT_UINT32, BASE_DEC, VALS(rrc_T_modeSpecificInfo_34_vals), 0,
        "T_modeSpecificInfo_34", HFILL }},
    { &hf_rrc_fdd_65,
      { "fdd", "rrc.fdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_fdd_64", HFILL }},
    { &hf_rrc_dpch_CompressedModeInfo_02,
      { "dpch-CompressedModeInfo", "rrc.dpch_CompressedModeInfo",
        FT_NONE, BASE_NONE, NULL, 0,
        "DPCH_CompressedModeInfo_r10", HFILL }},
    { &hf_rrc_tdd_27,
      { "tdd", "rrc.tdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_tdd_26", HFILL }},
    { &hf_rrc_tddOption_10,
      { "tddOption", "rrc.tddOption",
        FT_UINT32, BASE_DEC, VALS(rrc_T_tddOption_10_vals), 0,
        "T_tddOption_10", HFILL }},
    { &hf_rrc_tdd128_19,
      { "tdd128", "rrc.tdd128",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_tdd128_19", HFILL }},
    { &hf_rrc_mac_hsResetIndicator_09,
      { "mac-hsResetIndicator", "rrc.mac_hsResetIndicator",
        FT_UINT32, BASE_DEC, VALS(rrc_T_mac_hsResetIndicator_09_vals), 0,
        "T_mac_hsResetIndicator_09", HFILL }},
    { &hf_rrc_postVerificationPeriod_10,
      { "postVerificationPeriod", "rrc.postVerificationPeriod",
        FT_UINT32, BASE_DEC, VALS(rrc_T_postVerificationPeriod_10_vals), 0,
        "T_postVerificationPeriod_10", HFILL }},
    { &hf_rrc_dl_dpchInfoCommon_04,
      { "dl-dpchInfoCommon", "rrc.dl_dpchInfoCommon",
        FT_UINT32, BASE_DEC, VALS(rrc_T_dl_dpchInfoCommon_04_vals), 0,
        "T_dl_dpchInfoCommon_04", HFILL }},
    { &hf_rrc_modeSpecificInfo_35,
      { "modeSpecificInfo", "rrc.modeSpecificInfo",
        FT_UINT32, BASE_DEC, VALS(rrc_T_modeSpecificInfo_35_vals), 0,
        "T_modeSpecificInfo_35", HFILL }},
    { &hf_rrc_fdd_66,
      { "fdd", "rrc.fdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_fdd_65", HFILL }},
    { &hf_rrc_tdd_28,
      { "tdd", "rrc.tdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_tdd_27", HFILL }},
    { &hf_rrc_tddOption_11,
      { "tddOption", "rrc.tddOption",
        FT_UINT32, BASE_DEC, VALS(rrc_T_tddOption_11_vals), 0,
        "T_tddOption_11", HFILL }},
    { &hf_rrc_tdd128_20,
      { "tdd128", "rrc.tdd128",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_tdd128_20", HFILL }},
    { &hf_rrc_mac_hsResetIndicator_10,
      { "mac-hsResetIndicator", "rrc.mac_hsResetIndicator",
        FT_UINT32, BASE_DEC, VALS(rrc_T_mac_hsResetIndicator_10_vals), 0,
        "T_mac_hsResetIndicator_10", HFILL }},
    { &hf_rrc_postVerificationPeriod_11,
      { "postVerificationPeriod", "rrc.postVerificationPeriod",
        FT_UINT32, BASE_DEC, VALS(rrc_T_postVerificationPeriod_11_vals), 0,
        "T_postVerificationPeriod_11", HFILL }},
    { &hf_rrc_mac_hsResetIndicator_assisting_01,
      { "mac-hsResetIndicator-assisting", "rrc.mac_hsResetIndicator_assisting",
        FT_UINT32, BASE_DEC, VALS(rrc_T_mac_hsResetIndicator_assisting_01_vals), 0,
        "T_mac_hsResetIndicator_assisting_01", HFILL }},
    { &hf_rrc_dl_DPCH_InfoCommon_03,
      { "dl-DPCH-InfoCommon", "rrc.dl_DPCH_InfoCommon",
        FT_NONE, BASE_NONE, NULL, 0,
        "DL_DPCH_InfoCommonPost", HFILL }},
    { &hf_rrc_dl_DPCH_InfoCommon_04,
      { "dl-DPCH-InfoCommon", "rrc.dl_DPCH_InfoCommon",
        FT_NONE, BASE_NONE, NULL, 0,
        "DL_DPCH_InfoCommonPredef", HFILL }},
    { &hf_rrc_cfnHandling,
      { "cfnHandling", "rrc.cfnHandling",
        FT_UINT32, BASE_DEC, VALS(rrc_T_cfnHandling_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_maintain,
      { "maintain", "rrc.maintain",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_initialise,
      { "initialise", "rrc.initialise",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_dummy_40,
      { "dummy", "rrc.dummy",
        FT_UINT32, BASE_DEC, NULL, 0,
        "Cfntargetsfnframeoffset", HFILL }},
    { &hf_rrc_modeSpecificInfo_36,
      { "modeSpecificInfo", "rrc.modeSpecificInfo",
        FT_UINT32, BASE_DEC, VALS(rrc_T_modeSpecificInfo_36_vals), 0,
        "T_modeSpecificInfo_36", HFILL }},
    { &hf_rrc_fdd_67,
      { "fdd", "rrc.fdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_fdd_66", HFILL }},
    { &hf_rrc_dl_DPCH_PowerControlInfo,
      { "dl-DPCH-PowerControlInfo", "rrc.dl_DPCH_PowerControlInfo",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_powerOffsetPilot_pdpdch,
      { "powerOffsetPilot-pdpdch", "rrc.powerOffsetPilot_pdpdch",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_dl_rate_matching_restriction,
      { "dl-rate-matching-restriction", "rrc.dl_rate_matching_restriction",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_spreadingFactorAndPilot,
      { "spreadingFactorAndPilot", "rrc.spreadingFactorAndPilot",
        FT_UINT32, BASE_DEC, VALS(rrc_SF512_AndPilot_vals), 0,
        "SF512_AndPilot", HFILL }},
    { &hf_rrc_positionFixedOrFlexible,
      { "positionFixedOrFlexible", "rrc.positionFixedOrFlexible",
        FT_UINT32, BASE_DEC, VALS(rrc_PositionFixedOrFlexible_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_tfci_Existence,
      { "tfci-Existence", "rrc.tfci_Existence",
        FT_BOOLEAN, BASE_NONE, NULL, 0,
        "BOOLEAN", HFILL }},
    { &hf_rrc_tdd_29,
      { "tdd", "rrc.tdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_tdd_28", HFILL }},
    { &hf_rrc_cfnHandling_01,
      { "cfnHandling", "rrc.cfnHandling",
        FT_UINT32, BASE_DEC, VALS(rrc_T_cfnHandling_01_vals), 0,
        "T_cfnHandling_01", HFILL }},
    { &hf_rrc_initialise_01,
      { "initialise", "rrc.initialise",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_initialise_01", HFILL }},
    { &hf_rrc_modeSpecificInfo_37,
      { "modeSpecificInfo", "rrc.modeSpecificInfo",
        FT_UINT32, BASE_DEC, VALS(rrc_T_modeSpecificInfo_37_vals), 0,
        "T_modeSpecificInfo_37", HFILL }},
    { &hf_rrc_fdd_68,
      { "fdd", "rrc.fdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_fdd_67", HFILL }},
    { &hf_rrc_tdd_30,
      { "tdd", "rrc.tdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_tdd_29", HFILL }},
    { &hf_rrc_mac_d_HFN_initial_value,
      { "mac-d-HFN-initial-value", "rrc.mac_d_HFN_initial_value",
        FT_BYTES, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_cfnHandling_02,
      { "cfnHandling", "rrc.cfnHandling",
        FT_UINT32, BASE_DEC, VALS(rrc_T_cfnHandling_02_vals), 0,
        "T_cfnHandling_02", HFILL }},
    { &hf_rrc_maintain_01,
      { "maintain", "rrc.maintain",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_timingmaintainedsynchind,
      { "timingmaintainedsynchind", "rrc.timingmaintainedsynchind",
        FT_UINT32, BASE_DEC, VALS(rrc_TimingMaintainedSynchInd_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_initialise_02,
      { "initialise", "rrc.initialise",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_modeSpecificInfo_38,
      { "modeSpecificInfo", "rrc.modeSpecificInfo",
        FT_UINT32, BASE_DEC, VALS(rrc_T_modeSpecificInfo_38_vals), 0,
        "T_modeSpecificInfo_38", HFILL }},
    { &hf_rrc_fdd_69,
      { "fdd", "rrc.fdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_fdd_68", HFILL }},
    { &hf_rrc_tdd_31,
      { "tdd", "rrc.tdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_tdd_30", HFILL }},
    { &hf_rrc_modeSpecificInfo_39,
      { "modeSpecificInfo", "rrc.modeSpecificInfo",
        FT_UINT32, BASE_DEC, VALS(rrc_T_modeSpecificInfo_39_vals), 0,
        "T_modeSpecificInfo_39", HFILL }},
    { &hf_rrc_fdd_70,
      { "fdd", "rrc.fdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_fdd_69", HFILL }},
    { &hf_rrc_tdd_32,
      { "tdd", "rrc.tdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_tdd_31", HFILL }},
    { &hf_rrc_fdd_71,
      { "fdd", "rrc.fdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_fdd_70", HFILL }},
    { &hf_rrc_pCPICH_UsageForChannelEst,
      { "pCPICH-UsageForChannelEst", "rrc.pCPICH_UsageForChannelEst",
        FT_UINT32, BASE_DEC, VALS(rrc_PCPICH_UsageForChannelEst_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_dpch_FrameOffset,
      { "dpch-FrameOffset", "rrc.dpch_FrameOffset",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_secondaryCPICH_Info,
      { "secondaryCPICH-Info", "rrc.secondaryCPICH_Info",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_dl_ChannelisationCodeList,
      { "dl-ChannelisationCodeList", "rrc.dl_ChannelisationCodeList",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_tpc_CombinationIndex,
      { "tpc-CombinationIndex", "rrc.tpc_CombinationIndex",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_dummy_41,
      { "dummy", "rrc.dummy",
        FT_UINT32, BASE_DEC, VALS(rrc_SSDT_CellIdentity_vals), 0,
        "SSDT_CellIdentity", HFILL }},
    { &hf_rrc_closedLoopTimingAdjMode,
      { "closedLoopTimingAdjMode", "rrc.closedLoopTimingAdjMode",
        FT_UINT32, BASE_DEC, VALS(rrc_ClosedLoopTimingAdjMode_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_tdd_33,
      { "tdd", "rrc.tdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_tdd_32", HFILL }},
    { &hf_rrc_dl_CCTrChListToEstablish,
      { "dl-CCTrChListToEstablish", "rrc.dl_CCTrChListToEstablish",
        FT_UINT32, BASE_DEC, NULL, 0,
        "DL_CCTrChList", HFILL }},
    { &hf_rrc_dl_CCTrChListToRemove,
      { "dl-CCTrChListToRemove", "rrc.dl_CCTrChListToRemove",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_fdd_72,
      { "fdd", "rrc.fdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_fdd_71", HFILL }},
    { &hf_rrc_tdd_34,
      { "tdd", "rrc.tdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_tdd_33", HFILL }},
    { &hf_rrc_dl_CCTrChListToEstablish_01,
      { "dl-CCTrChListToEstablish", "rrc.dl_CCTrChListToEstablish",
        FT_UINT32, BASE_DEC, NULL, 0,
        "DL_CCTrChList_r4", HFILL }},
    { &hf_rrc_fdd_73,
      { "fdd", "rrc.fdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_fdd_72", HFILL }},
    { &hf_rrc_powerOffsetTPC_pdpdch,
      { "powerOffsetTPC-pdpdch", "rrc.powerOffsetTPC_pdpdch",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_tdd_35,
      { "tdd", "rrc.tdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_tdd_34", HFILL }},
    { &hf_rrc_fdd_74,
      { "fdd", "rrc.fdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_fdd_73", HFILL }},
    { &hf_rrc_tdd_36,
      { "tdd", "rrc.tdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_tdd_35", HFILL }},
    { &hf_rrc_fdd_75,
      { "fdd", "rrc.fdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_fdd_74", HFILL }},
    { &hf_rrc_tdd_37,
      { "tdd", "rrc.tdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_tdd_36", HFILL }},
    { &hf_rrc_dl_CCTrChListToEstablish_02,
      { "dl-CCTrChListToEstablish", "rrc.dl_CCTrChListToEstablish",
        FT_UINT32, BASE_DEC, NULL, 0,
        "DL_CCTrChList_r7", HFILL }},
    { &hf_rrc_fdpch_FrameOffset,
      { "fdpch-FrameOffset", "rrc.fdpch_FrameOffset",
        FT_UINT32, BASE_DEC, NULL, 0,
        "DPCH_FrameOffset", HFILL }},
    { &hf_rrc_dl_ChannelisationCode,
      { "dl-ChannelisationCode", "rrc.dl_ChannelisationCode",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_255", HFILL }},
    { &hf_rrc_fdpch_SlotFormat,
      { "fdpch-SlotFormat", "rrc.fdpch_SlotFormat",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_sttdIndication,
      { "sttdIndication", "rrc.sttdIndication",
        FT_UINT32, BASE_DEC, VALS(rrc_STTDIndication_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_dl_ChannelisationCode_01,
      { "dl-ChannelisationCode", "rrc.dl_ChannelisationCode",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_dl_DPCH_TimeslotsCodes,
      { "dl-DPCH-TimeslotsCodes", "rrc.dl_DPCH_TimeslotsCodes",
        FT_NONE, BASE_NONE, NULL, 0,
        "DownlinkTimeslotsCodes", HFILL }},
    { &hf_rrc_modeSpecificInfo_40,
      { "modeSpecificInfo", "rrc.modeSpecificInfo",
        FT_UINT32, BASE_DEC, VALS(rrc_T_modeSpecificInfo_40_vals), 0,
        "T_modeSpecificInfo_40", HFILL }},
    { &hf_rrc_fdd_76,
      { "fdd", "rrc.fdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_fdd_75", HFILL }},
    { &hf_rrc_tdd_38,
      { "tdd", "rrc.tdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_tdd_37", HFILL }},
    { &hf_rrc_tpc_StepSizeTDD,
      { "tpc-StepSizeTDD", "rrc.tpc_StepSizeTDD",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_cfnHandling_03,
      { "cfnHandling", "rrc.cfnHandling",
        FT_UINT32, BASE_DEC, VALS(rrc_T_cfnHandling_03_vals), 0,
        "T_cfnHandling_03", HFILL }},
    { &hf_rrc_maintain_02,
      { "maintain", "rrc.maintain",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_maintain_01", HFILL }},
    { &hf_rrc_dl_FDPCH_PowerControlInfo,
      { "dl-FDPCH-PowerControlInfo", "rrc.dl_FDPCH_PowerControlInfo",
        FT_NONE, BASE_NONE, NULL, 0,
        "DL_DPCH_PowerControlInfo", HFILL }},
    { &hf_rrc_dl_FDPCH_TPCcommandErrorRate_01,
      { "dl-FDPCH-TPCcommandErrorRate", "rrc.dl_FDPCH_TPCcommandErrorRate",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_1_16", HFILL }},
    { &hf_rrc_hs_scch_Info,
      { "hs-scch-Info", "rrc.hs_scch_Info",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_measurement_feedback_Info,
      { "measurement-feedback-Info", "rrc.measurement_feedback_Info",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_modeSpecificInfo_41,
      { "modeSpecificInfo", "rrc.modeSpecificInfo",
        FT_UINT32, BASE_DEC, VALS(rrc_T_modeSpecificInfo_41_vals), 0,
        "T_modeSpecificInfo_41", HFILL }},
    { &hf_rrc_tdd_39,
      { "tdd", "rrc.tdd",
        FT_UINT32, BASE_DEC, VALS(rrc_T_tdd_38_vals), 0,
        "T_tdd_38", HFILL }},
    { &hf_rrc_tdd384_14,
      { "tdd384", "rrc.tdd384",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_tdd384_13", HFILL }},
    { &hf_rrc_dl_HSPDSCH_TS_Configuration,
      { "dl-HSPDSCH-TS-Configuration", "rrc.dl_HSPDSCH_TS_Configuration",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_tdd128_21,
      { "tdd128", "rrc.tdd128",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_tdd128_21", HFILL }},
    { &hf_rrc_hs_PDSCH_Midamble_Configuration_tdd128,
      { "hs-PDSCH-Midamble-Configuration-tdd128", "rrc.hs_PDSCH_Midamble_Configuration_tdd128",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_hs_scch_Info_01,
      { "hs-scch-Info", "rrc.hs_scch_Info",
        FT_NONE, BASE_NONE, NULL, 0,
        "HS_SCCH_Info_r6", HFILL }},
    { &hf_rrc_modeSpecificInfo_42,
      { "modeSpecificInfo", "rrc.modeSpecificInfo",
        FT_UINT32, BASE_DEC, VALS(rrc_T_modeSpecificInfo_42_vals), 0,
        "T_modeSpecificInfo_42", HFILL }},
    { &hf_rrc_tdd_40,
      { "tdd", "rrc.tdd",
        FT_UINT32, BASE_DEC, VALS(rrc_T_tdd_39_vals), 0,
        "T_tdd_39", HFILL }},
    { &hf_rrc_tdd384_15,
      { "tdd384", "rrc.tdd384",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_tdd384_14", HFILL }},
    { &hf_rrc_tdd128_22,
      { "tdd128", "rrc.tdd128",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_tdd128_22", HFILL }},
    { &hf_rrc_hs_scch_Info_02,
      { "hs-scch-Info", "rrc.hs_scch_Info",
        FT_NONE, BASE_NONE, NULL, 0,
        "HS_SCCH_Info_r7", HFILL }},
    { &hf_rrc_measurement_feedback_Info_01,
      { "measurement-feedback-Info", "rrc.measurement_feedback_Info",
        FT_NONE, BASE_NONE, NULL, 0,
        "Measurement_Feedback_Info_r7", HFILL }},
    { &hf_rrc_modeSpecificInfo_43,
      { "modeSpecificInfo", "rrc.modeSpecificInfo",
        FT_UINT32, BASE_DEC, VALS(rrc_T_modeSpecificInfo_43_vals), 0,
        "T_modeSpecificInfo_43", HFILL }},
    { &hf_rrc_tdd_41,
      { "tdd", "rrc.tdd",
        FT_UINT32, BASE_DEC, VALS(rrc_T_tdd_40_vals), 0,
        "T_tdd_40", HFILL }},
    { &hf_rrc_tdd384_16,
      { "tdd384", "rrc.tdd384",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_tdd384_15", HFILL }},
    { &hf_rrc_tdd768_08,
      { "tdd768", "rrc.tdd768",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_tdd768_07", HFILL }},
    { &hf_rrc_dl_HSPDSCH_TS_Configuration_01,
      { "dl-HSPDSCH-TS-Configuration", "rrc.dl_HSPDSCH_TS_Configuration",
        FT_UINT32, BASE_DEC, NULL, 0,
        "DL_HSPDSCH_TS_Configuration_VHCR", HFILL }},
    { &hf_rrc_tdd128_23,
      { "tdd128", "rrc.tdd128",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_tdd128_23", HFILL }},
    { &hf_rrc_dl_MultiCarrier_Information,
      { "dl-MultiCarrier-Information", "rrc.dl_MultiCarrier_Information",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_fdd_77,
      { "fdd", "rrc.fdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_fdd_76", HFILL }},
    { &hf_rrc_dl_64QAM_Configured,
      { "dl-64QAM-Configured", "rrc.dl_64QAM_Configured",
        FT_UINT32, BASE_DEC, VALS(rrc_T_dl_64QAM_Configured_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_modeSpecificInfo_44,
      { "modeSpecificInfo", "rrc.modeSpecificInfo",
        FT_UINT32, BASE_DEC, VALS(rrc_T_modeSpecificInfo_44_vals), 0,
        "T_modeSpecificInfo_44", HFILL }},
    { &hf_rrc_tdd_42,
      { "tdd", "rrc.tdd",
        FT_UINT32, BASE_DEC, VALS(rrc_T_tdd_41_vals), 0,
        "T_tdd_41", HFILL }},
    { &hf_rrc_tdd384_17,
      { "tdd384", "rrc.tdd384",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_tdd384_16", HFILL }},
    { &hf_rrc_tdd768_09,
      { "tdd768", "rrc.tdd768",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_tdd768_08", HFILL }},
    { &hf_rrc_tdd128_24,
      { "tdd128", "rrc.tdd128",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_tdd128_24", HFILL }},
    { &hf_rrc_fdd_78,
      { "fdd", "rrc.fdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_fdd_77", HFILL }},
    { &hf_rrc_dl_64QAM_Configured_01,
      { "dl-64QAM-Configured", "rrc.dl_64QAM_Configured",
        FT_UINT32, BASE_DEC, VALS(rrc_T_dl_64QAM_Configured_01_vals), 0,
        "T_dl_64QAM_Configured_01", HFILL }},
    { &hf_rrc_modeSpecificInfo_45,
      { "modeSpecificInfo", "rrc.modeSpecificInfo",
        FT_UINT32, BASE_DEC, VALS(rrc_T_modeSpecificInfo_45_vals), 0,
        "T_modeSpecificInfo_45", HFILL }},
    { &hf_rrc_tdd_43,
      { "tdd", "rrc.tdd",
        FT_UINT32, BASE_DEC, VALS(rrc_T_tdd_42_vals), 0,
        "T_tdd_42", HFILL }},
    { &hf_rrc_tdd128_25,
      { "tdd128", "rrc.tdd128",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_tdd128_25", HFILL }},
    { &hf_rrc_outofSyncWindow,
      { "outofSyncWindow", "rrc.outofSyncWindow",
        FT_UINT32, BASE_DEC, VALS(rrc_OutofSyncWindow_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_hs_scch_Info_03,
      { "hs-scch-Info", "rrc.hs_scch_Info",
        FT_NONE, BASE_NONE, NULL, 0,
        "HS_SCCH_Info_r8_ext", HFILL }},
    { &hf_rrc_hs_scch_Info_04,
      { "hs-scch-Info", "rrc.hs_scch_Info",
        FT_NONE, BASE_NONE, NULL, 0,
        "HS_SCCH_Info_r9", HFILL }},
    { &hf_rrc_modeSpecificInfo_46,
      { "modeSpecificInfo", "rrc.modeSpecificInfo",
        FT_UINT32, BASE_DEC, VALS(rrc_T_modeSpecificInfo_46_vals), 0,
        "T_modeSpecificInfo_46", HFILL }},
    { &hf_rrc_tdd_44,
      { "tdd", "rrc.tdd",
        FT_UINT32, BASE_DEC, VALS(rrc_T_tdd_43_vals), 0,
        "T_tdd_43", HFILL }},
    { &hf_rrc_tdd384_18,
      { "tdd384", "rrc.tdd384",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_tdd384_17", HFILL }},
    { &hf_rrc_tdd768_10,
      { "tdd768", "rrc.tdd768",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_tdd768_09", HFILL }},
    { &hf_rrc_tdd128_26,
      { "tdd128", "rrc.tdd128",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_tdd128_26", HFILL }},
    { &hf_rrc_tS0_Indicator,
      { "tS0-Indicator", "rrc.tS0_Indicator",
        FT_UINT32, BASE_DEC, VALS(rrc_T_tS0_Indicator_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_fdd_79,
      { "fdd", "rrc.fdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_fdd_78", HFILL }},
    { &hf_rrc_dl_64QAM_Configured_02,
      { "dl-64QAM-Configured", "rrc.dl_64QAM_Configured",
        FT_UINT32, BASE_DEC, VALS(rrc_T_dl_64QAM_Configured_02_vals), 0,
        "T_dl_64QAM_Configured_02", HFILL }},
    { &hf_rrc_measurement_feedback_Info_02,
      { "measurement-feedback-Info", "rrc.measurement_feedback_Info",
        FT_NONE, BASE_NONE, NULL, 0,
        "Measurement_Feedback_Info_r11", HFILL }},
    { &hf_rrc_modeSpecificInfo_47,
      { "modeSpecificInfo", "rrc.modeSpecificInfo",
        FT_UINT32, BASE_DEC, VALS(rrc_T_modeSpecificInfo_47_vals), 0,
        "T_modeSpecificInfo_47", HFILL }},
    { &hf_rrc_tdd_45,
      { "tdd", "rrc.tdd",
        FT_UINT32, BASE_DEC, VALS(rrc_T_tdd_44_vals), 0,
        "T_tdd_44", HFILL }},
    { &hf_rrc_tdd384_19,
      { "tdd384", "rrc.tdd384",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_tdd384_18", HFILL }},
    { &hf_rrc_tdd768_11,
      { "tdd768", "rrc.tdd768",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_tdd768_10", HFILL }},
    { &hf_rrc_tdd128_27,
      { "tdd128", "rrc.tdd128",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_tdd128_27", HFILL }},
    { &hf_rrc_tS0_Indicator_01,
      { "tS0-Indicator", "rrc.tS0_Indicator",
        FT_UINT32, BASE_DEC, VALS(rrc_T_tS0_Indicator_01_vals), 0,
        "T_tS0_Indicator_01", HFILL }},
    { &hf_rrc_fdd_80,
      { "fdd", "rrc.fdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_fdd_79", HFILL }},
    { &hf_rrc_dl_64QAM_Configured_03,
      { "dl-64QAM-Configured", "rrc.dl_64QAM_Configured",
        FT_UINT32, BASE_DEC, VALS(rrc_T_dl_64QAM_Configured_03_vals), 0,
        "T_dl_64QAM_Configured_03", HFILL }},
    { &hf_rrc_DL_HSPDSCH_MultiCarrier_Information_item,
      { "DL-HSPDSCH-MultiCarrier-Information item", "rrc.DL_HSPDSCH_MultiCarrier_Information_item",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_uarfcn_Carrier,
      { "uarfcn-Carrier", "rrc.uarfcn_Carrier",
        FT_UINT32, BASE_DEC, NULL, 0,
        "UARFCN", HFILL }},
    { &hf_rrc_hs_PDSCH_Midamble_Configuration,
      { "hs-PDSCH-Midamble-Configuration", "rrc.hs_PDSCH_Midamble_Configuration",
        FT_NONE, BASE_NONE, NULL, 0,
        "HS_PDSCH_Midamble_Configuration_TDD128", HFILL }},
    { &hf_rrc_hs_SCCH_TDD128_MultiCarrier,
      { "hs-SCCH-TDD128-MultiCarrier", "rrc.hs_SCCH_TDD128_MultiCarrier",
        FT_UINT32, BASE_DEC, NULL, 0,
        "SEQUENCE_SIZE_1_maxHSSCCHs_OF_HS_SCCH_TDD128_MultiCarrier", HFILL }},
    { &hf_rrc_hs_SCCH_TDD128_MultiCarrier_item,
      { "HS-SCCH-TDD128-MultiCarrier", "rrc.HS_SCCH_TDD128_MultiCarrier",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_DL_HSPDSCH_TS_Configuration_item,
      { "DL-HSPDSCH-TS-Configuration item", "rrc.DL_HSPDSCH_TS_Configuration_item",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_midambleShiftAndBurstType,
      { "midambleShiftAndBurstType", "rrc.midambleShiftAndBurstType",
        FT_NONE, BASE_NONE, NULL, 0,
        "MidambleShiftAndBurstType_DL", HFILL }},
    { &hf_rrc_DL_HSPDSCH_TS_Configuration_VHCR_item,
      { "DL-HSPDSCH-TS-Configuration-VHCR item", "rrc.DL_HSPDSCH_TS_Configuration_VHCR_item",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_midambleShiftAndBurstType_01,
      { "midambleShiftAndBurstType", "rrc.midambleShiftAndBurstType",
        FT_NONE, BASE_NONE, NULL, 0,
        "MidambleShiftAndBurstType_DL_VHCR", HFILL }},
    { &hf_rrc_modeSpecificInfo_48,
      { "modeSpecificInfo", "rrc.modeSpecificInfo",
        FT_UINT32, BASE_DEC, VALS(rrc_T_modeSpecificInfo_48_vals), 0,
        "T_modeSpecificInfo_48", HFILL }},
    { &hf_rrc_fdd_81,
      { "fdd", "rrc.fdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_fdd_80", HFILL }},
    { &hf_rrc_dummy1_02,
      { "dummy1", "rrc.dummy1",
        FT_NONE, BASE_NONE, NULL, 0,
        "PDSCH_SHO_DCH_Info", HFILL }},
    { &hf_rrc_dummy2_07,
      { "dummy2", "rrc.dummy2",
        FT_NONE, BASE_NONE, NULL, 0,
        "PDSCH_CodeMapping", HFILL }},
    { &hf_rrc_tdd_46,
      { "tdd", "rrc.tdd",
        FT_UINT32, BASE_DEC, VALS(rrc_PrimaryCCPCH_Info_vals), 0,
        "PrimaryCCPCH_Info", HFILL }},
    { &hf_rrc_dl_DPCH_InfoPerRL,
      { "dl-DPCH-InfoPerRL", "rrc.dl_DPCH_InfoPerRL",
        FT_UINT32, BASE_DEC, VALS(rrc_DL_DPCH_InfoPerRL_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_dummy_42,
      { "dummy", "rrc.dummy",
        FT_NONE, BASE_NONE, NULL, 0,
        "SCCPCH_InfoForFACH", HFILL }},
    { &hf_rrc_modeSpecificInfo_49,
      { "modeSpecificInfo", "rrc.modeSpecificInfo",
        FT_UINT32, BASE_DEC, VALS(rrc_T_modeSpecificInfo_49_vals), 0,
        "T_modeSpecificInfo_49", HFILL }},
    { &hf_rrc_fdd_82,
      { "fdd", "rrc.fdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_fdd_81", HFILL }},
    { &hf_rrc_tdd_47,
      { "tdd", "rrc.tdd",
        FT_UINT32, BASE_DEC, VALS(rrc_PrimaryCCPCH_Info_r4_vals), 0,
        "PrimaryCCPCH_Info_r4", HFILL }},
    { &hf_rrc_dl_DPCH_InfoPerRL_01,
      { "dl-DPCH-InfoPerRL", "rrc.dl_DPCH_InfoPerRL",
        FT_UINT32, BASE_DEC, VALS(rrc_DL_DPCH_InfoPerRL_r4_vals), 0,
        "DL_DPCH_InfoPerRL_r4", HFILL }},
    { &hf_rrc_dummy_43,
      { "dummy", "rrc.dummy",
        FT_NONE, BASE_NONE, NULL, 0,
        "SCCPCH_InfoForFACH_r4", HFILL }},
    { &hf_rrc_modeSpecificInfo_50,
      { "modeSpecificInfo", "rrc.modeSpecificInfo",
        FT_UINT32, BASE_DEC, VALS(rrc_T_modeSpecificInfo_50_vals), 0,
        "T_modeSpecificInfo_50", HFILL }},
    { &hf_rrc_fdd_83,
      { "fdd", "rrc.fdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_fdd_82", HFILL }},
    { &hf_rrc_servingHSDSCH_RL_indicator,
      { "servingHSDSCH-RL-indicator", "rrc.servingHSDSCH_RL_indicator",
        FT_BOOLEAN, BASE_NONE, NULL, 0,
        "BOOLEAN", HFILL }},
    { &hf_rrc_dl_DPCH_InfoPerRL_02,
      { "dl-DPCH-InfoPerRL", "rrc.dl_DPCH_InfoPerRL",
        FT_UINT32, BASE_DEC, VALS(rrc_DL_DPCH_InfoPerRL_r5_vals), 0,
        "DL_DPCH_InfoPerRL_r5", HFILL }},
    { &hf_rrc_modeSpecificInfo_51,
      { "modeSpecificInfo", "rrc.modeSpecificInfo",
        FT_UINT32, BASE_DEC, VALS(rrc_T_modeSpecificInfo_51_vals), 0,
        "T_modeSpecificInfo_51", HFILL }},
    { &hf_rrc_fdd_84,
      { "fdd", "rrc.fdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_fdd_83", HFILL }},
    { &hf_rrc_modeSpecificInfo_52,
      { "modeSpecificInfo", "rrc.modeSpecificInfo",
        FT_UINT32, BASE_DEC, VALS(rrc_T_modeSpecificInfo_52_vals), 0,
        "T_modeSpecificInfo_52", HFILL }},
    { &hf_rrc_fdd_85,
      { "fdd", "rrc.fdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_fdd_84", HFILL }},
    { &hf_rrc_servingEDCH_RL_indicator,
      { "servingEDCH-RL-indicator", "rrc.servingEDCH_RL_indicator",
        FT_BOOLEAN, BASE_NONE, NULL, 0,
        "BOOLEAN", HFILL }},
    { &hf_rrc_dl_dpchInfo,
      { "dl-dpchInfo", "rrc.dl_dpchInfo",
        FT_UINT32, BASE_DEC, VALS(rrc_T_dl_dpchInfo_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_dl_DPCH_InfoPerRL_03,
      { "dl-DPCH-InfoPerRL", "rrc.dl_DPCH_InfoPerRL",
        FT_UINT32, BASE_DEC, VALS(rrc_DL_DPCH_InfoPerRL_r6_vals), 0,
        "DL_DPCH_InfoPerRL_r6", HFILL }},
    { &hf_rrc_dl_FDPCH_InfoPerRL,
      { "dl-FDPCH-InfoPerRL", "rrc.dl_FDPCH_InfoPerRL",
        FT_NONE, BASE_NONE, NULL, 0,
        "DL_FDPCH_InfoPerRL_r6", HFILL }},
    { &hf_rrc_e_HICH_Info,
      { "e-HICH-Info", "rrc.e_HICH_Info",
        FT_UINT32, BASE_DEC, VALS(rrc_T_e_HICH_Info_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_e_RGCH_Info_01,
      { "e-RGCH-Info", "rrc.e_RGCH_Info",
        FT_UINT32, BASE_DEC, VALS(rrc_T_e_RGCH_Info_01_vals), 0,
        "T_e_RGCH_Info_01", HFILL }},
    { &hf_rrc_modeSpecificInfo_53,
      { "modeSpecificInfo", "rrc.modeSpecificInfo",
        FT_UINT32, BASE_DEC, VALS(rrc_T_modeSpecificInfo_53_vals), 0,
        "T_modeSpecificInfo_53", HFILL }},
    { &hf_rrc_fdd_86,
      { "fdd", "rrc.fdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_fdd_85", HFILL }},
    { &hf_rrc_dl_dpchInfo_01,
      { "dl-dpchInfo", "rrc.dl_dpchInfo",
        FT_UINT32, BASE_DEC, VALS(rrc_T_dl_dpchInfo_01_vals), 0,
        "T_dl_dpchInfo_01", HFILL }},
    { &hf_rrc_dl_DPCH_InfoPerRL_04,
      { "dl-DPCH-InfoPerRL", "rrc.dl_DPCH_InfoPerRL",
        FT_UINT32, BASE_DEC, VALS(rrc_DL_DPCH_InfoPerRL_r7_vals), 0,
        "DL_DPCH_InfoPerRL_r7", HFILL }},
    { &hf_rrc_dl_FDPCH_InfoPerRL_01,
      { "dl-FDPCH-InfoPerRL", "rrc.dl_FDPCH_InfoPerRL",
        FT_NONE, BASE_NONE, NULL, 0,
        "DL_FDPCH_InfoPerRL_r7", HFILL }},
    { &hf_rrc_e_AGCH_Information_01,
      { "e-AGCH-Information", "rrc.e_AGCH_Information",
        FT_NONE, BASE_NONE, NULL, 0,
        "E_AGCH_Information_r7", HFILL }},
    { &hf_rrc_modeSpecificInfo2,
      { "modeSpecificInfo2", "rrc.modeSpecificInfo2",
        FT_UINT32, BASE_DEC, VALS(rrc_T_modeSpecificInfo2_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_fdd_87,
      { "fdd", "rrc.fdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_fdd_86", HFILL }},
    { &hf_rrc_e_HICH_Info_01,
      { "e-HICH-Info", "rrc.e_HICH_Info",
        FT_UINT32, BASE_DEC, VALS(rrc_T_e_HICH_Info_01_vals), 0,
        "T_e_HICH_Info_01", HFILL }},
    { &hf_rrc_e_RGCH_Info_02,
      { "e-RGCH-Info", "rrc.e_RGCH_Info",
        FT_UINT32, BASE_DEC, VALS(rrc_T_e_RGCH_Info_02_vals), 0,
        "T_e_RGCH_Info_02", HFILL }},
    { &hf_rrc_tdd_48,
      { "tdd", "rrc.tdd",
        FT_UINT32, BASE_DEC, VALS(rrc_T_tdd_45_vals), 0,
        "T_tdd_45", HFILL }},
    { &hf_rrc_tdd384_tdd768,
      { "tdd384-tdd768", "rrc.tdd384_tdd768",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_e_HICH_Info_02,
      { "e-HICH-Info", "rrc.e_HICH_Info",
        FT_NONE, BASE_NONE, NULL, 0,
        "E_HICH_Information_TDD384_768", HFILL }},
    { &hf_rrc_tdd128_28,
      { "tdd128", "rrc.tdd128",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_tdd128_28", HFILL }},
    { &hf_rrc_e_HICH_Info_03,
      { "e-HICH-Info", "rrc.e_HICH_Info",
        FT_NONE, BASE_NONE, NULL, 0,
        "E_HICH_Information_TDD128", HFILL }},
    { &hf_rrc_modeSpecificInfo_54,
      { "modeSpecificInfo", "rrc.modeSpecificInfo",
        FT_UINT32, BASE_DEC, VALS(rrc_T_modeSpecificInfo_54_vals), 0,
        "T_modeSpecificInfo_54", HFILL }},
    { &hf_rrc_fdd_88,
      { "fdd", "rrc.fdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_fdd_87", HFILL }},
    { &hf_rrc_dl_dpchInfo_02,
      { "dl-dpchInfo", "rrc.dl_dpchInfo",
        FT_UINT32, BASE_DEC, VALS(rrc_T_dl_dpchInfo_02_vals), 0,
        "T_dl_dpchInfo_02", HFILL }},
    { &hf_rrc_e_AGCH_Information_02,
      { "e-AGCH-Information", "rrc.e_AGCH_Information",
        FT_NONE, BASE_NONE, NULL, 0,
        "E_AGCH_Information_r8", HFILL }},
    { &hf_rrc_modeSpecificInfo2_01,
      { "modeSpecificInfo2", "rrc.modeSpecificInfo2",
        FT_UINT32, BASE_DEC, VALS(rrc_T_modeSpecificInfo2_01_vals), 0,
        "T_modeSpecificInfo2_01", HFILL }},
    { &hf_rrc_fdd_89,
      { "fdd", "rrc.fdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_fdd_88", HFILL }},
    { &hf_rrc_e_HICH_Info_04,
      { "e-HICH-Info", "rrc.e_HICH_Info",
        FT_UINT32, BASE_DEC, VALS(rrc_T_e_HICH_Info_02_vals), 0,
        "T_e_HICH_Info_02", HFILL }},
    { &hf_rrc_e_RGCH_Info_03,
      { "e-RGCH-Info", "rrc.e_RGCH_Info",
        FT_UINT32, BASE_DEC, VALS(rrc_T_e_RGCH_Info_03_vals), 0,
        "T_e_RGCH_Info_03", HFILL }},
    { &hf_rrc_tdd_49,
      { "tdd", "rrc.tdd",
        FT_UINT32, BASE_DEC, VALS(rrc_T_tdd_46_vals), 0,
        "T_tdd_46", HFILL }},
    { &hf_rrc_tdd384_tdd768_01,
      { "tdd384-tdd768", "rrc.tdd384_tdd768",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_tdd384_tdd768_01", HFILL }},
    { &hf_rrc_tdd128_29,
      { "tdd128", "rrc.tdd128",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_tdd128_29", HFILL }},
    { &hf_rrc_modeSpecificInfo_55,
      { "modeSpecificInfo", "rrc.modeSpecificInfo",
        FT_UINT32, BASE_DEC, VALS(rrc_T_modeSpecificInfo_55_vals), 0,
        "T_modeSpecificInfo_55", HFILL }},
    { &hf_rrc_fdd_90,
      { "fdd", "rrc.fdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_fdd_89", HFILL }},
    { &hf_rrc_dl_dpchInfo_03,
      { "dl-dpchInfo", "rrc.dl_dpchInfo",
        FT_UINT32, BASE_DEC, VALS(rrc_T_dl_dpchInfo_03_vals), 0,
        "T_dl_dpchInfo_03", HFILL }},
    { &hf_rrc_e_ROCH_Information,
      { "e-ROCH-Information", "rrc.e_ROCH_Information",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_modeSpecificInfo2_02,
      { "modeSpecificInfo2", "rrc.modeSpecificInfo2",
        FT_UINT32, BASE_DEC, VALS(rrc_T_modeSpecificInfo2_02_vals), 0,
        "T_modeSpecificInfo2_02", HFILL }},
    { &hf_rrc_fdd_91,
      { "fdd", "rrc.fdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_fdd_90", HFILL }},
    { &hf_rrc_e_HICH_Info_05,
      { "e-HICH-Info", "rrc.e_HICH_Info",
        FT_UINT32, BASE_DEC, VALS(rrc_T_e_HICH_Info_03_vals), 0,
        "T_e_HICH_Info_03", HFILL }},
    { &hf_rrc_e_HICH_Information_01,
      { "e-HICH-Information", "rrc.e_HICH_Information",
        FT_NONE, BASE_NONE, NULL, 0,
        "E_HICH_Information_r11", HFILL }},
    { &hf_rrc_secondaryReleaseIndicator,
      { "secondaryReleaseIndicator", "rrc.secondaryReleaseIndicator",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_e_RGCH_Info_04,
      { "e-RGCH-Info", "rrc.e_RGCH_Info",
        FT_UINT32, BASE_DEC, VALS(rrc_T_e_RGCH_Info_04_vals), 0,
        "T_e_RGCH_Info_04", HFILL }},
    { &hf_rrc_f_TPICH_Info,
      { "f-TPICH-Info", "rrc.f_TPICH_Info",
        FT_UINT32, BASE_DEC, VALS(rrc_T_f_TPICH_Info_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_f_TPICH_Information,
      { "f-TPICH-Information", "rrc.f_TPICH_Information",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_tdd_50,
      { "tdd", "rrc.tdd",
        FT_UINT32, BASE_DEC, VALS(rrc_T_tdd_47_vals), 0,
        "T_tdd_47", HFILL }},
    { &hf_rrc_tdd384_tdd768_02,
      { "tdd384-tdd768", "rrc.tdd384_tdd768",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_tdd384_tdd768_02", HFILL }},
    { &hf_rrc_tdd128_30,
      { "tdd128", "rrc.tdd128",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_tdd128_30", HFILL }},
    { &hf_rrc_DL_InformationPerRL_List_item,
      { "DL-InformationPerRL", "rrc.DL_InformationPerRL",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_DL_InformationPerRL_List_r4_item,
      { "DL-InformationPerRL-r4", "rrc.DL_InformationPerRL_r4",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_DL_InformationPerRL_List_r5_item,
      { "DL-InformationPerRL-r5", "rrc.DL_InformationPerRL_r5",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_DL_InformationPerRL_List_r6_item,
      { "DL-InformationPerRL-r6", "rrc.DL_InformationPerRL_r6",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_DL_InformationPerRL_List_v6b0ext_item,
      { "DL-InformationPerRL-v6b0ext", "rrc.DL_InformationPerRL_v6b0ext",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_DL_InformationPerRL_List_r5bis_item,
      { "DL-InformationPerRL-r5bis", "rrc.DL_InformationPerRL_r5bis",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_DL_InformationPerRL_List_r7_item,
      { "DL-InformationPerRL-r7", "rrc.DL_InformationPerRL_r7",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_DL_InformationPerRL_List_r8_item,
      { "DL-InformationPerRL-r8", "rrc.DL_InformationPerRL_r8",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_DL_InformationPerRL_List_r11_item,
      { "DL-InformationPerRL-r11", "rrc.DL_InformationPerRL_r11",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_DL_InformationPerRL_ListPostFDD_item,
      { "DL-InformationPerRL-PostFDD", "rrc.DL_InformationPerRL_PostFDD",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_dl_DPCH_InfoPerRL_05,
      { "dl-DPCH-InfoPerRL", "rrc.dl_DPCH_InfoPerRL",
        FT_NONE, BASE_NONE, NULL, 0,
        "DL_DPCH_InfoPerRL_PostFDD", HFILL }},
    { &hf_rrc_primaryCCPCH_Info,
      { "primaryCCPCH-Info", "rrc.primaryCCPCH_Info",
        FT_NONE, BASE_NONE, NULL, 0,
        "PrimaryCCPCH_InfoPost", HFILL }},
    { &hf_rrc_dl_DPCH_InfoPerRL_06,
      { "dl-DPCH-InfoPerRL", "rrc.dl_DPCH_InfoPerRL",
        FT_NONE, BASE_NONE, NULL, 0,
        "DL_DPCH_InfoPerRL_PostTDD", HFILL }},
    { &hf_rrc_primaryCCPCH_Info_01,
      { "primaryCCPCH-Info", "rrc.primaryCCPCH_Info",
        FT_NONE, BASE_NONE, NULL, 0,
        "PrimaryCCPCH_InfoPostTDD_LCR_r4", HFILL }},
    { &hf_rrc_dl_DPCH_InfoPerRL_07,
      { "dl-DPCH-InfoPerRL", "rrc.dl_DPCH_InfoPerRL",
        FT_NONE, BASE_NONE, NULL, 0,
        "DL_DPCH_InfoPerRL_PostTDD_LCR_r4", HFILL }},
    { &hf_rrc_tsn_Length,
      { "tsn-Length", "rrc.tsn_Length",
        FT_UINT32, BASE_DEC, VALS(rrc_T_tsn_Length_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_multiCarrierNumber,
      { "multiCarrierNumber", "rrc.multiCarrierNumber",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_1_maxTDD128Carrier", HFILL }},
    { &hf_rrc_dl_HSPDSCH_MultiCarrier_Information,
      { "dl-HSPDSCH-MultiCarrier-Information", "rrc.dl_HSPDSCH_MultiCarrier_Information",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_restrictedTrCH_InfoList,
      { "restrictedTrCH-InfoList", "rrc.restrictedTrCH_InfoList",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_newConfiguration_02,
      { "newConfiguration", "rrc.newConfiguration",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_newConfiguration_02", HFILL }},
    { &hf_rrc_dl_64QAM_Configured_04,
      { "dl-64QAM-Configured", "rrc.dl_64QAM_Configured",
        FT_UINT32, BASE_DEC, VALS(rrc_T_dl_64QAM_Configured_04_vals), 0,
        "T_dl_64QAM_Configured_04", HFILL }},
    { &hf_rrc_dl_ScramblingCode,
      { "dl-ScramblingCode", "rrc.dl_ScramblingCode",
        FT_UINT32, BASE_DEC, NULL, 0,
        "SecondaryScramblingCode", HFILL }},
    { &hf_rrc_hS_SCCHChannelisationCodeInfo,
      { "hS-SCCHChannelisationCodeInfo", "rrc.hS_SCCHChannelisationCodeInfo",
        FT_UINT32, BASE_DEC, NULL, 0,
        "SEQUENCE_SIZE_1_maxHSSCCHs_OF_HS_SCCH_Codes", HFILL }},
    { &hf_rrc_hS_SCCHChannelisationCodeInfo_item,
      { "HS-SCCH-Codes", "rrc.HS_SCCH_Codes",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_measurementPowerOffset,
      { "measurementPowerOffset", "rrc.measurementPowerOffset",
        FT_INT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_uarfcn_DL,
      { "uarfcn-DL", "rrc.uarfcn_DL",
        FT_UINT32, BASE_DEC, NULL, 0,
        "UARFCN", HFILL }},
    { &hf_rrc_differentTxModeFromServingHS_DSCHCell,
      { "differentTxModeFromServingHS-DSCHCell", "rrc.differentTxModeFromServingHS_DSCHCell",
        FT_UINT32, BASE_DEC, VALS(rrc_T_differentTxModeFromServingHS_DSCHCell_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_configurationInfo_01,
      { "configurationInfo", "rrc.configurationInfo",
        FT_UINT32, BASE_DEC, VALS(rrc_T_configurationInfo_01_vals), 0,
        "T_configurationInfo_01", HFILL }},
    { &hf_rrc_newConfiguration_03,
      { "newConfiguration", "rrc.newConfiguration",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_newConfiguration_03", HFILL }},
    { &hf_rrc_dl_64QAM_Configured_05,
      { "dl-64QAM-Configured", "rrc.dl_64QAM_Configured",
        FT_UINT32, BASE_DEC, VALS(rrc_T_dl_64QAM_Configured_05_vals), 0,
        "T_dl_64QAM_Configured_05", HFILL }},
    { &hf_rrc_differentTxModeFromServingHS_DSCHCell_01,
      { "differentTxModeFromServingHS-DSCHCell", "rrc.differentTxModeFromServingHS_DSCHCell",
        FT_UINT32, BASE_DEC, VALS(rrc_T_differentTxModeFromServingHS_DSCHCell_01_vals), 0,
        "T_differentTxModeFromServingHS_DSCHCell_01", HFILL }},
    { &hf_rrc_secondaryCellMIMOparameters,
      { "secondaryCellMIMOparameters", "rrc.secondaryCellMIMOparameters",
        FT_UINT32, BASE_DEC, VALS(rrc_SecondaryCellMIMOparametersFDD_vals), 0,
        "SecondaryCellMIMOparametersFDD", HFILL }},
    { &hf_rrc_configurationInfo_02,
      { "configurationInfo", "rrc.configurationInfo",
        FT_UINT32, BASE_DEC, VALS(rrc_T_configurationInfo_02_vals), 0,
        "T_configurationInfo_02", HFILL }},
    { &hf_rrc_newConfiguration_04,
      { "newConfiguration", "rrc.newConfiguration",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_newConfiguration_04", HFILL }},
    { &hf_rrc_dl_64QAM_Configured_06,
      { "dl-64QAM-Configured", "rrc.dl_64QAM_Configured",
        FT_UINT32, BASE_DEC, VALS(rrc_T_dl_64QAM_Configured_06_vals), 0,
        "T_dl_64QAM_Configured_06", HFILL }},
    { &hf_rrc_differentTxModeFromServingHS_DSCHCell_02,
      { "differentTxModeFromServingHS-DSCHCell", "rrc.differentTxModeFromServingHS_DSCHCell",
        FT_UINT32, BASE_DEC, VALS(rrc_T_differentTxModeFromServingHS_DSCHCell_02_vals), 0,
        "T_differentTxModeFromServingHS_DSCHCell_02", HFILL }},
    { &hf_rrc_secondaryCellMIMOparameters_01,
      { "secondaryCellMIMOparameters", "rrc.secondaryCellMIMOparameters",
        FT_UINT32, BASE_DEC, VALS(rrc_SecondaryCellMIMOparametersFDD_r10_vals), 0,
        "SecondaryCellMIMOparametersFDD_r10", HFILL }},
    { &hf_rrc_configurationInfo_03,
      { "configurationInfo", "rrc.configurationInfo",
        FT_UINT32, BASE_DEC, VALS(rrc_T_configurationInfo_03_vals), 0,
        "T_configurationInfo_03", HFILL }},
    { &hf_rrc_newConfiguration_05,
      { "newConfiguration", "rrc.newConfiguration",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_newConfiguration_05", HFILL }},
    { &hf_rrc_dl_64QAM_Configured_07,
      { "dl-64QAM-Configured", "rrc.dl_64QAM_Configured",
        FT_UINT32, BASE_DEC, VALS(rrc_T_dl_64QAM_Configured_07_vals), 0,
        "T_dl_64QAM_Configured_07", HFILL }},
    { &hf_rrc_measurementPowerOffset1,
      { "measurementPowerOffset1", "rrc.measurementPowerOffset1",
        FT_INT32, BASE_DEC, NULL, 0,
        "MeasurementPowerOffset", HFILL }},
    { &hf_rrc_differentTxModeFromServingHS_DSCHCell_03,
      { "differentTxModeFromServingHS-DSCHCell", "rrc.differentTxModeFromServingHS_DSCHCell",
        FT_UINT32, BASE_DEC, VALS(rrc_T_differentTxModeFromServingHS_DSCHCell_03_vals), 0,
        "T_differentTxModeFromServingHS_DSCHCell_03", HFILL }},
    { &hf_rrc_multiflowConfigurationInfo,
      { "multiflowConfigurationInfo", "rrc.multiflowConfigurationInfo",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_secondaryCell4x4MIMOparameters,
      { "secondaryCell4x4MIMOparameters", "rrc.secondaryCell4x4MIMOparameters",
        FT_UINT32, BASE_DEC, VALS(rrc_SecondaryCell4x4MIMOparametersFDD_vals), 0,
        "SecondaryCell4x4MIMOparametersFDD", HFILL }},
    { &hf_rrc_DL_TPC_PowerOffsetPerRL_List_item,
      { "DL-TPC-PowerOffsetPerRL", "rrc.DL_TPC_PowerOffsetPerRL",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_codesRepresentation,
      { "codesRepresentation", "rrc.codesRepresentation",
        FT_UINT32, BASE_DEC, VALS(rrc_T_codesRepresentation_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_consecutive,
      { "consecutive", "rrc.consecutive",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_firstChannelisationCode,
      { "firstChannelisationCode", "rrc.firstChannelisationCode",
        FT_UINT32, BASE_DEC, VALS(rrc_DL_TS_ChannelisationCode_vals), 0,
        "DL_TS_ChannelisationCode", HFILL }},
    { &hf_rrc_lastChannelisationCode,
      { "lastChannelisationCode", "rrc.lastChannelisationCode",
        FT_UINT32, BASE_DEC, VALS(rrc_DL_TS_ChannelisationCode_vals), 0,
        "DL_TS_ChannelisationCode", HFILL }},
    { &hf_rrc_bitmap,
      { "bitmap", "rrc.bitmap",
        FT_BYTES, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_codesRepresentation_01,
      { "codesRepresentation", "rrc.codesRepresentation",
        FT_UINT32, BASE_DEC, VALS(rrc_T_codesRepresentation_01_vals), 0,
        "T_codesRepresentation_01", HFILL }},
    { &hf_rrc_consecutive_01,
      { "consecutive", "rrc.consecutive",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_consecutive_01", HFILL }},
    { &hf_rrc_firstChannelisationCode_01,
      { "firstChannelisationCode", "rrc.firstChannelisationCode",
        FT_UINT32, BASE_DEC|BASE_EXT_STRING, &rrc_DL_TS_ChannelisationCode_VHCR_vals_ext, 0,
        "DL_TS_ChannelisationCode_VHCR", HFILL }},
    { &hf_rrc_lastChannelisationCode_01,
      { "lastChannelisationCode", "rrc.lastChannelisationCode",
        FT_UINT32, BASE_DEC|BASE_EXT_STRING, &rrc_DL_TS_ChannelisationCode_VHCR_vals_ext, 0,
        "DL_TS_ChannelisationCode_VHCR", HFILL }},
    { &hf_rrc_bitmap_01,
      { "bitmap", "rrc.bitmap",
        FT_BYTES, BASE_NONE, NULL, 0,
        "T_bitmap_01", HFILL }},
    { &hf_rrc_firstChannelisationCode_02,
      { "firstChannelisationCode", "rrc.firstChannelisationCode",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_1_15", HFILL }},
    { &hf_rrc_lastChannelisationCode_02,
      { "lastChannelisationCode", "rrc.lastChannelisationCode",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_1_15", HFILL }},
    { &hf_rrc_parameters,
      { "parameters", "rrc.parameters",
        FT_UINT32, BASE_DEC, VALS(rrc_T_parameters_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_sameAsLast,
      { "sameAsLast", "rrc.sameAsLast",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_timeslotNumber,
      { "timeslotNumber", "rrc.timeslotNumber",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_newParameters,
      { "newParameters", "rrc.newParameters",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_individualTimeslotInfo,
      { "individualTimeslotInfo", "rrc.individualTimeslotInfo",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_dl_TS_ChannelisationCodesShort,
      { "dl-TS-ChannelisationCodesShort", "rrc.dl_TS_ChannelisationCodesShort",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_parameters_01,
      { "parameters", "rrc.parameters",
        FT_UINT32, BASE_DEC, VALS(rrc_T_parameters_01_vals), 0,
        "T_parameters_01", HFILL }},
    { &hf_rrc_sameAsLast_01,
      { "sameAsLast", "rrc.sameAsLast",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_sameAsLast_01", HFILL }},
    { &hf_rrc_newParameters_01,
      { "newParameters", "rrc.newParameters",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_newParameters_01", HFILL }},
    { &hf_rrc_individualTimeslotInfo_01,
      { "individualTimeslotInfo", "rrc.individualTimeslotInfo",
        FT_NONE, BASE_NONE, NULL, 0,
        "IndividualTimeslotInfo_VHCR", HFILL }},
    { &hf_rrc_dl_TS_ChannelisationCodesShort_01,
      { "dl-TS-ChannelisationCodesShort", "rrc.dl_TS_ChannelisationCodesShort",
        FT_NONE, BASE_NONE, NULL, 0,
        "DL_TS_ChannelisationCodesShort_VHCR", HFILL }},
    { &hf_rrc_parameters_02,
      { "parameters", "rrc.parameters",
        FT_UINT32, BASE_DEC, VALS(rrc_T_parameters_02_vals), 0,
        "T_parameters_02", HFILL }},
    { &hf_rrc_sameAsLast_02,
      { "sameAsLast", "rrc.sameAsLast",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_sameAsLast_02", HFILL }},
    { &hf_rrc_timeslotNumber_01,
      { "timeslotNumber", "rrc.timeslotNumber",
        FT_UINT32, BASE_DEC, NULL, 0,
        "TimeslotNumber_LCR_r4", HFILL }},
    { &hf_rrc_newParameters_02,
      { "newParameters", "rrc.newParameters",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_newParameters_02", HFILL }},
    { &hf_rrc_individualTimeslotInfo_02,
      { "individualTimeslotInfo", "rrc.individualTimeslotInfo",
        FT_NONE, BASE_NONE, NULL, 0,
        "IndividualTimeslotInfo_LCR_r4", HFILL }},
    { &hf_rrc_parameters_03,
      { "parameters", "rrc.parameters",
        FT_UINT32, BASE_DEC, VALS(rrc_T_parameters_03_vals), 0,
        "T_parameters_03", HFILL }},
    { &hf_rrc_sameAsLast_03,
      { "sameAsLast", "rrc.sameAsLast",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_sameAsLast_03", HFILL }},
    { &hf_rrc_newParameters_03,
      { "newParameters", "rrc.newParameters",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_newParameters_03", HFILL }},
    { &hf_rrc_individualTimeslotInfo_03,
      { "individualTimeslotInfo", "rrc.individualTimeslotInfo",
        FT_NONE, BASE_NONE, NULL, 0,
        "IndividualTimeslotInfo_r7", HFILL }},
    { &hf_rrc_firstIndividualTimeslotInfo,
      { "firstIndividualTimeslotInfo", "rrc.firstIndividualTimeslotInfo",
        FT_NONE, BASE_NONE, NULL, 0,
        "IndividualTimeslotInfo", HFILL }},
    { &hf_rrc_moreTimeslots,
      { "moreTimeslots", "rrc.moreTimeslots",
        FT_UINT32, BASE_DEC, VALS(rrc_T_moreTimeslots_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_noMore,
      { "noMore", "rrc.noMore",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_additionalTimeslots,
      { "additionalTimeslots", "rrc.additionalTimeslots",
        FT_UINT32, BASE_DEC, VALS(rrc_T_additionalTimeslots_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_consecutive_02,
      { "consecutive", "rrc.consecutive",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_1_maxTS_1", HFILL }},
    { &hf_rrc_timeslotList,
      { "timeslotList", "rrc.timeslotList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "SEQUENCE_SIZE_1_maxTS_1_OF_DownlinkAdditionalTimeslots", HFILL }},
    { &hf_rrc_timeslotList_item,
      { "DownlinkAdditionalTimeslots", "rrc.DownlinkAdditionalTimeslots",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_firstIndividualTimeslotInfo_01,
      { "firstIndividualTimeslotInfo", "rrc.firstIndividualTimeslotInfo",
        FT_NONE, BASE_NONE, NULL, 0,
        "IndividualTimeslotInfo_VHCR", HFILL }},
    { &hf_rrc_moreTimeslots_01,
      { "moreTimeslots", "rrc.moreTimeslots",
        FT_UINT32, BASE_DEC, VALS(rrc_T_moreTimeslots_01_vals), 0,
        "T_moreTimeslots_01", HFILL }},
    { &hf_rrc_additionalTimeslots_01,
      { "additionalTimeslots", "rrc.additionalTimeslots",
        FT_UINT32, BASE_DEC, VALS(rrc_T_additionalTimeslots_01_vals), 0,
        "T_additionalTimeslots_01", HFILL }},
    { &hf_rrc_timeslotList_01,
      { "timeslotList", "rrc.timeslotList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "SEQUENCE_SIZE_1_maxTS_1_OF_DownlinkAdditionalTimeslots_VHCR", HFILL }},
    { &hf_rrc_timeslotList_item_01,
      { "DownlinkAdditionalTimeslots-VHCR", "rrc.DownlinkAdditionalTimeslots_VHCR",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_firstIndividualTimeslotInfo_02,
      { "firstIndividualTimeslotInfo", "rrc.firstIndividualTimeslotInfo",
        FT_NONE, BASE_NONE, NULL, 0,
        "IndividualTimeslotInfo_LCR_r4", HFILL }},
    { &hf_rrc_moreTimeslots_02,
      { "moreTimeslots", "rrc.moreTimeslots",
        FT_UINT32, BASE_DEC, VALS(rrc_T_moreTimeslots_02_vals), 0,
        "T_moreTimeslots_02", HFILL }},
    { &hf_rrc_additionalTimeslots_02,
      { "additionalTimeslots", "rrc.additionalTimeslots",
        FT_UINT32, BASE_DEC, VALS(rrc_T_additionalTimeslots_02_vals), 0,
        "T_additionalTimeslots_02", HFILL }},
    { &hf_rrc_consecutive_03,
      { "consecutive", "rrc.consecutive",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_1_maxTS_LCR_1", HFILL }},
    { &hf_rrc_timeslotList_02,
      { "timeslotList", "rrc.timeslotList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "SEQUENCE_SIZE_1_maxTS_LCR_1_OF_DownlinkAdditionalTimeslots_LCR_r4", HFILL }},
    { &hf_rrc_timeslotList_item_02,
      { "DownlinkAdditionalTimeslots-LCR-r4", "rrc.DownlinkAdditionalTimeslots_LCR_r4",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_firstIndividualTimeslotInfo_03,
      { "firstIndividualTimeslotInfo", "rrc.firstIndividualTimeslotInfo",
        FT_NONE, BASE_NONE, NULL, 0,
        "IndividualTimeslotInfo_r7", HFILL }},
    { &hf_rrc_moreTimeslots_03,
      { "moreTimeslots", "rrc.moreTimeslots",
        FT_UINT32, BASE_DEC, VALS(rrc_T_moreTimeslots_03_vals), 0,
        "T_moreTimeslots_03", HFILL }},
    { &hf_rrc_additionalTimeslots_03,
      { "additionalTimeslots", "rrc.additionalTimeslots",
        FT_UINT32, BASE_DEC, VALS(rrc_T_additionalTimeslots_03_vals), 0,
        "T_additionalTimeslots_03", HFILL }},
    { &hf_rrc_timeslotList_03,
      { "timeslotList", "rrc.timeslotList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "SEQUENCE_SIZE_1_maxTS_1_OF_DownlinkAdditionalTimeslots_r7", HFILL }},
    { &hf_rrc_timeslotList_item_03,
      { "DownlinkAdditionalTimeslots-r7", "rrc.DownlinkAdditionalTimeslots_r7",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_tgp_SequenceList,
      { "tgp-SequenceList", "rrc.tgp_SequenceList",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_tgp_SequenceList_01,
      { "tgp-SequenceList", "rrc.tgp_SequenceList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "TGP_SequenceList_r8", HFILL }},
    { &hf_rrc_tgp_SequenceList_02,
      { "tgp-SequenceList", "rrc.tgp_SequenceList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "TGP_SequenceList_r10", HFILL }},
    { &hf_rrc_tgps_Reconfiguration_CFN,
      { "tgps-Reconfiguration-CFN", "rrc.tgps_Reconfiguration_CFN",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_tgp_SequenceShortList,
      { "tgp-SequenceShortList", "rrc.tgp_SequenceShortList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "SEQUENCE_SIZE_1_maxTGPS_OF_TGP_SequenceShort", HFILL }},
    { &hf_rrc_tgp_SequenceShortList_item,
      { "TGP-SequenceShort", "rrc.TGP_SequenceShort",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_tgp_SequenceShortList_01,
      { "tgp-SequenceShortList", "rrc.tgp_SequenceShortList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "SEQUENCE_SIZE_1_maxTGPS_OF_TGP_SequenceShort_r10", HFILL }},
    { &hf_rrc_tgp_SequenceShortList_item_01,
      { "TGP-SequenceShort-r10", "rrc.TGP_SequenceShort_r10",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_ue_drx_Cycle,
      { "ue-drx-Cycle", "rrc.ue_drx_Cycle",
        FT_UINT32, BASE_DEC, VALS(rrc_UE_DRX_Cycle_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_ue_drx_Cycle_InactivityThreshold,
      { "ue-drx-Cycle-InactivityThreshold", "rrc.ue_drx_Cycle_InactivityThreshold",
        FT_UINT32, BASE_DEC, VALS(rrc_UE_DRX_Cycle_InactivityThreshold_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_ue_GrantMonitoring_InactivityThreshold,
      { "ue-GrantMonitoring-InactivityThreshold", "rrc.ue_GrantMonitoring_InactivityThreshold",
        FT_UINT32, BASE_DEC, VALS(rrc_UE_GrantMonitoring_InactivityThreshold_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_ue_drx_GrantMonitoring,
      { "ue-drx-GrantMonitoring", "rrc.ue_drx_GrantMonitoring",
        FT_BOOLEAN, BASE_NONE, NULL, 0,
        "BOOLEAN", HFILL }},
    { &hf_rrc_maxTFCI_Field2Value,
      { "maxTFCI-Field2Value", "rrc.maxTFCI_Field2Value",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_spreadingFactor,
      { "spreadingFactor", "rrc.spreadingFactor",
        FT_UINT32, BASE_DEC, VALS(rrc_SF_PDSCH_vals), 0,
        "SF_PDSCH", HFILL }},
    { &hf_rrc_codeNumber,
      { "codeNumber", "rrc.codeNumber",
        FT_UINT32, BASE_DEC, NULL, 0,
        "CodeNumberDSCH", HFILL }},
    { &hf_rrc_multiCodeInfo,
      { "multiCodeInfo", "rrc.multiCodeInfo",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_DSCH_MappingList_item,
      { "DSCH-Mapping", "rrc.DSCH_Mapping",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_DSCH_TransportChannelsInfo_item,
      { "DSCH-TransportChannelsInfo item", "rrc.DSCH_TransportChannelsInfo_item",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_dsch_transport_channel_identity,
      { "dsch-transport-channel-identity", "rrc.dsch_transport_channel_identity",
        FT_UINT32, BASE_DEC, NULL, 0,
        "TransportChannelIdentity", HFILL }},
    { &hf_rrc_dsch_TFS,
      { "dsch-TFS", "rrc.dsch_TFS",
        FT_UINT32, BASE_DEC, VALS(rrc_TransportFormatSet_vals), 0,
        "TransportFormatSet", HFILL }},
    { &hf_rrc_dtx_Info,
      { "dtx-Info", "rrc.dtx_Info",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_drx_Info,
      { "drx-Info", "rrc.drx_Info",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_uplink_DPCCHSlotFormatInformation,
      { "uplink-DPCCHSlotFormatInformation", "rrc.uplink_DPCCHSlotFormatInformation",
        FT_UINT32, BASE_DEC, VALS(rrc_Uplink_DPCCH_Slot_Format_Information_vals), 0,
        "Uplink_DPCCH_Slot_Format_Information", HFILL }},
    { &hf_rrc_ue_dtx_Cycle1_10ms,
      { "ue-dtx-Cycle1-10ms", "rrc.ue_dtx_Cycle1_10ms",
        FT_UINT32, BASE_DEC, VALS(rrc_UE_DTX_Cycle1_10ms_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_ue_dtx_Cycle2_10ms,
      { "ue-dtx-Cycle2-10ms", "rrc.ue_dtx_Cycle2_10ms",
        FT_UINT32, BASE_DEC, VALS(rrc_UE_DTX_Cycle2_10ms_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_mac_dtx_Cycle_10ms,
      { "mac-dtx-Cycle-10ms", "rrc.mac_dtx_Cycle_10ms",
        FT_UINT32, BASE_DEC, VALS(rrc_MAC_DTX_Cycle_10ms_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_ue_dtx_Cycle1_2ms,
      { "ue-dtx-Cycle1-2ms", "rrc.ue_dtx_Cycle1_2ms",
        FT_UINT32, BASE_DEC, VALS(rrc_UE_DTX_Cycle1_2ms_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_ue_dtx_Cycle2_2ms,
      { "ue-dtx-Cycle2-2ms", "rrc.ue_dtx_Cycle2_2ms",
        FT_UINT32, BASE_DEC, VALS(rrc_UE_DTX_Cycle2_2ms_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_mac_dtx_Cycle_2ms,
      { "mac-dtx-Cycle-2ms", "rrc.mac_dtx_Cycle_2ms",
        FT_UINT32, BASE_DEC, VALS(rrc_MAC_DTX_Cycle_2ms_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_e_dch_TTI_Length,
      { "e-dch-TTI-Length", "rrc.e_dch_TTI_Length",
        FT_UINT32, BASE_DEC, VALS(rrc_T_e_dch_TTI_Length_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_dtx_e_dch_TTI_10ms,
      { "dtx-e-dch-TTI-10ms", "rrc.dtx_e_dch_TTI_10ms",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_dtx_e_dch_TTI_2ms,
      { "dtx-e-dch-TTI-2ms", "rrc.dtx_e_dch_TTI_2ms",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_ue_dtx_cycle2InactivityThreshold,
      { "ue-dtx-cycle2InactivityThreshold", "rrc.ue_dtx_cycle2InactivityThreshold",
        FT_UINT32, BASE_DEC, VALS(rrc_UE_DTX_Cycle2InactivityThreshold_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_ue_dtx_cycle2DefaultSG,
      { "ue-dtx-cycle2DefaultSG", "rrc.ue_dtx_cycle2DefaultSG",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_38", HFILL }},
    { &hf_rrc_ue_dtx_long_preamble_length,
      { "ue-dtx-long-preamble-length", "rrc.ue_dtx_long_preamble_length",
        FT_UINT32, BASE_DEC, VALS(rrc_UE_DTX_long_preamble_length_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_mac_InactivityThreshold,
      { "mac-InactivityThreshold", "rrc.mac_InactivityThreshold",
        FT_UINT32, BASE_DEC, VALS(rrc_MAC_InactivityThreshold_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_cqi_dtx_Timer,
      { "cqi-dtx-Timer", "rrc.cqi_dtx_Timer",
        FT_UINT32, BASE_DEC, VALS(rrc_CQI_DTX_Timer_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_ue_dpcch_Burst1,
      { "ue-dpcch-Burst1", "rrc.ue_dpcch_Burst1",
        FT_UINT32, BASE_DEC, VALS(rrc_UE_DPCCH_Burst_vals), 0,
        "UE_DPCCH_Burst", HFILL }},
    { &hf_rrc_ue_dpcch_Burst2,
      { "ue-dpcch-Burst2", "rrc.ue_dpcch_Burst2",
        FT_UINT32, BASE_DEC, VALS(rrc_UE_DPCCH_Burst_vals), 0,
        "UE_DPCCH_Burst", HFILL }},
    { &hf_rrc_DynamicPersistenceLevelList_item,
      { "DynamicPersistenceLevel", "rrc.DynamicPersistenceLevel",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_DynamicPersistenceLevelTF_List_item,
      { "DynamicPersistenceLevel", "rrc.DynamicPersistenceLevel",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_e_AGCH_DRX_InfoType,
      { "e-AGCH-DRX-InfoType", "rrc.e_AGCH_DRX_InfoType",
        FT_UINT32, BASE_DEC, VALS(rrc_T_e_AGCH_DRX_InfoType_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_sameAsHS_SCCH,
      { "sameAsHS-SCCH", "rrc.sameAsHS_SCCH",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_e_AGCH_DRX_Parameters,
      { "e-AGCH-DRX-Parameters", "rrc.e_AGCH_DRX_Parameters",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_e_AGCH_DRX_Cycle,
      { "e-AGCH-DRX-Cycle", "rrc.e_AGCH_DRX_Cycle",
        FT_UINT32, BASE_DEC, VALS(rrc_ControlChannelDRXCycle_TDD128_vals), 0,
        "ControlChannelDRXCycle_TDD128", HFILL }},
    { &hf_rrc_e_AGCH_InactivityMonitorThreshold,
      { "e-AGCH-InactivityMonitorThreshold", "rrc.e_AGCH_InactivityMonitorThreshold",
        FT_UINT32, BASE_DEC, VALS(rrc_E_AGCH_InactivityMonitorThreshold_TDD128_vals), 0,
        "E_AGCH_InactivityMonitorThreshold_TDD128", HFILL }},
    { &hf_rrc_e_AGCH_DRX_Offset,
      { "e-AGCH-DRX-Offset", "rrc.e_AGCH_DRX_Offset",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_63", HFILL }},
    { &hf_rrc_tS_number,
      { "tS-number", "rrc.tS_number",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_14", HFILL }},
    { &hf_rrc_channelisation_code,
      { "channelisation-code", "rrc.channelisation_code",
        FT_UINT32, BASE_DEC, VALS(rrc_DL_TS_ChannelisationCode_vals), 0,
        "DL_TS_ChannelisationCode", HFILL }},
    { &hf_rrc_midambleShiftAndBurstType_02,
      { "midambleShiftAndBurstType", "rrc.midambleShiftAndBurstType",
        FT_NONE, BASE_NONE, NULL, 0,
        "MidambleShiftAndBurstType_EDCH", HFILL }},
    { &hf_rrc_channelisation_code_01,
      { "channelisation-code", "rrc.channelisation_code",
        FT_UINT32, BASE_DEC|BASE_EXT_STRING, &rrc_DL_TS_ChannelisationCode_VHCR_vals_ext, 0,
        "DL_TS_ChannelisationCode_VHCR", HFILL }},
    { &hf_rrc_firstChannelisationCode_03,
      { "firstChannelisationCode", "rrc.firstChannelisationCode",
        FT_UINT32, BASE_DEC, VALS(rrc_HS_ChannelisationCode_LCR_vals), 0,
        "HS_ChannelisationCode_LCR", HFILL }},
    { &hf_rrc_secondChannelisationCode,
      { "secondChannelisationCode", "rrc.secondChannelisationCode",
        FT_UINT32, BASE_DEC, VALS(rrc_HS_ChannelisationCode_LCR_vals), 0,
        "HS_ChannelisationCode_LCR", HFILL }},
    { &hf_rrc_midambleAllocationMode,
      { "midambleAllocationMode", "rrc.midambleAllocationMode",
        FT_UINT32, BASE_DEC, VALS(rrc_T_midambleAllocationMode_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_defaultMidamble,
      { "defaultMidamble", "rrc.defaultMidamble",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_commonMidamble,
      { "commonMidamble", "rrc.commonMidamble",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_ueSpecificMidamble,
      { "ueSpecificMidamble", "rrc.ueSpecificMidamble",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_15", HFILL }},
    { &hf_rrc_midambleConfiguration,
      { "midambleConfiguration", "rrc.midambleConfiguration",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_1_8", HFILL }},
    { &hf_rrc_e_AGCH_ChannelisationCode,
      { "e-AGCH-ChannelisationCode", "rrc.e_AGCH_ChannelisationCode",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_modeSpecific_02,
      { "modeSpecific", "rrc.modeSpecific",
        FT_UINT32, BASE_DEC, VALS(rrc_T_modeSpecific_02_vals), 0,
        "T_modeSpecific_02", HFILL }},
    { &hf_rrc_fdd_92,
      { "fdd", "rrc.fdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_fdd_91", HFILL }},
    { &hf_rrc_tdd_51,
      { "tdd", "rrc.tdd",
        FT_UINT32, BASE_DEC, VALS(rrc_T_tdd_48_vals), 0,
        "T_tdd_48", HFILL }},
    { &hf_rrc_tdd384_20,
      { "tdd384", "rrc.tdd384",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_tdd384_19", HFILL }},
    { &hf_rrc_long_Term_Grant_Indicator,
      { "long-Term-Grant-Indicator", "rrc.long_Term_Grant_Indicator",
        FT_BOOLEAN, BASE_NONE, NULL, 0,
        "BOOLEAN", HFILL }},
    { &hf_rrc_length_of_TTRI_field,
      { "length-of-TTRI-field", "rrc.length_of_TTRI_field",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_1_12", HFILL }},
    { &hf_rrc_e_AGCH_Set_Config,
      { "e-AGCH-Set-Config", "rrc.e_AGCH_Set_Config",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_e_AGCH_BLER_Target,
      { "e-AGCH-BLER-Target", "rrc.e_AGCH_BLER_Target",
        FT_INT32, BASE_DEC, NULL, 0,
        "Bler_Target", HFILL }},
    { &hf_rrc_tdd768_12,
      { "tdd768", "rrc.tdd768",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_tdd768_11", HFILL }},
    { &hf_rrc_e_AGCH_Set_Config_01,
      { "e-AGCH-Set-Config", "rrc.e_AGCH_Set_Config",
        FT_UINT32, BASE_DEC, NULL, 0,
        "E_AGCH_Set_Config_VHCR", HFILL }},
    { &hf_rrc_tdd128_31,
      { "tdd128", "rrc.tdd128",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_tdd128_31", HFILL }},
    { &hf_rrc_rdi_Indicator,
      { "rdi-Indicator", "rrc.rdi_Indicator",
        FT_BOOLEAN, BASE_NONE, NULL, 0,
        "BOOLEAN", HFILL }},
    { &hf_rrc_tpc_StepSize,
      { "tpc-StepSize", "rrc.tpc_StepSize",
        FT_UINT32, BASE_DEC, NULL, 0,
        "TPC_StepSizeTDD", HFILL }},
    { &hf_rrc_e_AGCH_Set_Config_02,
      { "e-AGCH-Set-Config", "rrc.e_AGCH_Set_Config",
        FT_UINT32, BASE_DEC, NULL, 0,
        "E_AGCH_Set_Config_LCR", HFILL }},
    { &hf_rrc_modeSpecific_03,
      { "modeSpecific", "rrc.modeSpecific",
        FT_UINT32, BASE_DEC, VALS(rrc_T_modeSpecific_03_vals), 0,
        "T_modeSpecific_03", HFILL }},
    { &hf_rrc_fdd_93,
      { "fdd", "rrc.fdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_fdd_92", HFILL }},
    { &hf_rrc_tdd_52,
      { "tdd", "rrc.tdd",
        FT_UINT32, BASE_DEC, VALS(rrc_T_tdd_49_vals), 0,
        "T_tdd_49", HFILL }},
    { &hf_rrc_tdd384_21,
      { "tdd384", "rrc.tdd384",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_tdd384_20", HFILL }},
    { &hf_rrc_tdd768_13,
      { "tdd768", "rrc.tdd768",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_tdd768_12", HFILL }},
    { &hf_rrc_tdd128_32,
      { "tdd128", "rrc.tdd128",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_tdd128_32", HFILL }},
    { &hf_rrc_E_AGCH_Set_Config_item,
      { "E-AGCH-Individual", "rrc.E_AGCH_Individual",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_E_AGCH_Set_Config_VHCR_item,
      { "E-AGCH-Individual-VHCR", "rrc.E_AGCH_Individual_VHCR",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_E_AGCH_Set_Config_LCR_item,
      { "E-AGCH-Individual-LCR", "rrc.E_AGCH_Individual_LCR",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_e_ROCH_ChannelisationCode,
      { "e-ROCH-ChannelisationCode", "rrc.e_ROCH_ChannelisationCode",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_e_roch_e_rnti,
      { "e-roch-e-rnti", "rrc.e_roch_e_rnti",
        FT_BYTES, BASE_NONE, NULL, 0,
        "E_RNTI", HFILL }},
    { &hf_rrc_e_DCH_RL_InfoNewServingCell,
      { "e-DCH-RL-InfoNewServingCell", "rrc.e_DCH_RL_InfoNewServingCell",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_e_DCH_RL_InfoOtherCellList,
      { "e-DCH-RL-InfoOtherCellList", "rrc.e_DCH_RL_InfoOtherCellList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "SEQUENCE_SIZE_1_maxEDCHRL_OF_E_DCH_RL_InfoOtherCell", HFILL }},
    { &hf_rrc_e_DCH_RL_InfoOtherCellList_item,
      { "E-DCH-RL-InfoOtherCell", "rrc.E_DCH_RL_InfoOtherCell",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_e_DCH_RL_InfoNewServingCell_01,
      { "e-DCH-RL-InfoNewServingCell", "rrc.e_DCH_RL_InfoNewServingCell",
        FT_NONE, BASE_NONE, NULL, 0,
        "E_DCH_RL_InfoNewServingCell_r7", HFILL }},
    { &hf_rrc_e_DCH_RL_InfoNewServingCell_02,
      { "e-DCH-RL-InfoNewServingCell", "rrc.e_DCH_RL_InfoNewServingCell",
        FT_NONE, BASE_NONE, NULL, 0,
        "E_DCH_RL_InfoNewServingCell_r11", HFILL }},
    { &hf_rrc_e_DCH_RL_InfoOtherCellList_01,
      { "e-DCH-RL-InfoOtherCellList", "rrc.e_DCH_RL_InfoOtherCellList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "SEQUENCE_SIZE_1_maxEDCHRL_OF_E_DCH_RL_InfoOtherCell_r11", HFILL }},
    { &hf_rrc_e_DCH_RL_InfoOtherCellList_item_01,
      { "E-DCH-RL-InfoOtherCell-r11", "rrc.E_DCH_RL_InfoOtherCell_r11",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_e_DCH_RL_InfoNewSecServingCell,
      { "e-DCH-RL-InfoNewSecServingCell", "rrc.e_DCH_RL_InfoNewSecServingCell",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_e_DCH_RL_InfoOtherCellList_SecULFreq,
      { "e-DCH-RL-InfoOtherCellList-SecULFreq", "rrc.e_DCH_RL_InfoOtherCellList_SecULFreq",
        FT_UINT32, BASE_DEC, NULL, 0,
        "SEQUENCE_SIZE_1_maxEDCHRL_OF_E_DCH_RL_InfoOtherCell_SecULFreq", HFILL }},
    { &hf_rrc_e_DCH_RL_InfoOtherCellList_SecULFreq_item,
      { "E-DCH-RL-InfoOtherCell-SecULFreq", "rrc.E_DCH_RL_InfoOtherCell_SecULFreq",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_servingGrant_01,
      { "servingGrant", "rrc.servingGrant",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_servingGrant_01", HFILL }},
    { &hf_rrc_value,
      { "value", "rrc.value",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_38", HFILL }},
    { &hf_rrc_primary_Secondary_GrantSelector_01,
      { "primary-Secondary-GrantSelector", "rrc.primary_Secondary_GrantSelector",
        FT_UINT32, BASE_DEC, VALS(rrc_T_primary_Secondary_GrantSelector_01_vals), 0,
        "T_primary_Secondary_GrantSelector_01", HFILL }},
    { &hf_rrc_reference_E_TFCIs,
      { "reference-E-TFCIs", "rrc.reference_E_TFCIs",
        FT_UINT32, BASE_DEC, NULL, 0,
        "E_DPDCH_Reference_E_TFCIList", HFILL }},
    { &hf_rrc_powerOffsetForSchedInfo,
      { "powerOffsetForSchedInfo", "rrc.powerOffsetForSchedInfo",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_6", HFILL }},
    { &hf_rrc_threeIndexStepThreshold,
      { "threeIndexStepThreshold", "rrc.threeIndexStepThreshold",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_37", HFILL }},
    { &hf_rrc_twoIndexStepThreshold,
      { "twoIndexStepThreshold", "rrc.twoIndexStepThreshold",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_37", HFILL }},
    { &hf_rrc_e_RGCH_Info_05,
      { "e-RGCH-Info", "rrc.e_RGCH_Info",
        FT_UINT32, BASE_DEC, VALS(rrc_T_e_RGCH_Info_05_vals), 0,
        "T_e_RGCH_Info_05", HFILL }},
    { &hf_rrc_servingGrant_02,
      { "servingGrant", "rrc.servingGrant",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_servingGrant_02", HFILL }},
    { &hf_rrc_primary_Secondary_GrantSelector_02,
      { "primary-Secondary-GrantSelector", "rrc.primary_Secondary_GrantSelector",
        FT_UINT32, BASE_DEC, VALS(rrc_T_primary_Secondary_GrantSelector_02_vals), 0,
        "T_primary_Secondary_GrantSelector_02", HFILL }},
    { &hf_rrc_reference_E_TFCIs_01,
      { "reference-E-TFCIs", "rrc.reference_E_TFCIs",
        FT_UINT32, BASE_DEC, NULL, 0,
        "E_DPDCH_Reference_E_TFCIList_r7", HFILL }},
    { &hf_rrc_e_RGCH_Info_06,
      { "e-RGCH-Info", "rrc.e_RGCH_Info",
        FT_UINT32, BASE_DEC, VALS(rrc_T_e_RGCH_Info_06_vals), 0,
        "T_e_RGCH_Info_06", HFILL }},
    { &hf_rrc_servingGrant_03,
      { "servingGrant", "rrc.servingGrant",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_servingGrant_03", HFILL }},
    { &hf_rrc_primary_Secondary_GrantSelector_03,
      { "primary-Secondary-GrantSelector", "rrc.primary_Secondary_GrantSelector",
        FT_UINT32, BASE_DEC, VALS(rrc_T_primary_Secondary_GrantSelector_03_vals), 0,
        "T_primary_Secondary_GrantSelector_03", HFILL }},
    { &hf_rrc_e_RGCH_Info_07,
      { "e-RGCH-Info", "rrc.e_RGCH_Info",
        FT_UINT32, BASE_DEC, VALS(rrc_T_e_RGCH_Info_07_vals), 0,
        "T_e_RGCH_Info_07", HFILL }},
    { &hf_rrc_e_RGCH_Info_08,
      { "e-RGCH-Info", "rrc.e_RGCH_Info",
        FT_UINT32, BASE_DEC, VALS(rrc_T_e_RGCH_Info_08_vals), 0,
        "T_e_RGCH_Info_08", HFILL }},
    { &hf_rrc_e_HICH_Info_06,
      { "e-HICH-Info", "rrc.e_HICH_Info",
        FT_UINT32, BASE_DEC, VALS(rrc_T_e_HICH_Info_04_vals), 0,
        "T_e_HICH_Info_04", HFILL }},
    { &hf_rrc_e_RGCH_Info_09,
      { "e-RGCH-Info", "rrc.e_RGCH_Info",
        FT_UINT32, BASE_DEC, VALS(rrc_T_e_RGCH_Info_09_vals), 0,
        "T_e_RGCH_Info_09", HFILL }},
    { &hf_rrc_e_HICH_Info_07,
      { "e-HICH-Info", "rrc.e_HICH_Info",
        FT_UINT32, BASE_DEC, VALS(rrc_T_e_HICH_Info_05_vals), 0,
        "T_e_HICH_Info_05", HFILL }},
    { &hf_rrc_e_RGCH_Info_10,
      { "e-RGCH-Info", "rrc.e_RGCH_Info",
        FT_UINT32, BASE_DEC, VALS(rrc_T_e_RGCH_Info_10_vals), 0,
        "T_e_RGCH_Info_10", HFILL }},
    { &hf_rrc_e_RGCH_Info_11,
      { "e-RGCH-Info", "rrc.e_RGCH_Info",
        FT_UINT32, BASE_DEC, VALS(rrc_T_e_RGCH_Info_11_vals), 0,
        "T_e_RGCH_Info_11", HFILL }},
    { &hf_rrc_e_dch_SPS_Operation,
      { "e-dch-SPS-Operation", "rrc.e_dch_SPS_Operation",
        FT_UINT32, BASE_DEC, VALS(rrc_T_e_dch_SPS_Operation_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_newOperation_01,
      { "newOperation", "rrc.newOperation",
        FT_NONE, BASE_NONE, NULL, 0,
        "E_DCH_SPS_NewOperation_TDD128", HFILL }},
    { &hf_rrc_e_hich_Info_01,
      { "e-hich-Info", "rrc.e_hich_Info",
        FT_NONE, BASE_NONE, NULL, 0,
        "E_HICH_Information_For_SPS_TDD128", HFILL }},
    { &hf_rrc_e_dch_TxPattern,
      { "e-dch-TxPattern", "rrc.e_dch_TxPattern",
        FT_UINT32, BASE_DEC, NULL, 0,
        "E_DCH_TxPatternList_TDD128", HFILL }},
    { &hf_rrc_initialSPSInfoForEDCH,
      { "initialSPSInfoForEDCH", "rrc.initialSPSInfoForEDCH",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_n_E_UCCH,
      { "n-E-UCCH", "rrc.n_E_UCCH",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_1_8", HFILL }},
    { &hf_rrc_codeResourceInfo,
      { "codeResourceInfo", "rrc.codeResourceInfo",
        FT_UINT32, BASE_DEC|BASE_EXT_STRING, &rrc_UL_TS_ChannelisationCode_vals_ext, 0,
        "UL_TS_ChannelisationCode", HFILL }},
    { &hf_rrc_timeslotResourceRelatedInfo,
      { "timeslotResourceRelatedInfo", "rrc.timeslotResourceRelatedInfo",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_5", HFILL }},
    { &hf_rrc_powerResourceRelatedInfo,
      { "powerResourceRelatedInfo", "rrc.powerResourceRelatedInfo",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_1_32", HFILL }},
    { &hf_rrc_subframeNum,
      { "subframeNum", "rrc.subframeNum",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_1", HFILL }},
    { &hf_rrc_initialTxPatternIndex,
      { "initialTxPatternIndex", "rrc.initialTxPatternIndex",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_maxEDCHTxPattern_TDD128_1", HFILL }},
    { &hf_rrc_E_DCH_TxPatternList_TDD128_item,
      { "E-DCH-TxPatternList-TDD128 item", "rrc.E_DCH_TxPatternList_TDD128_item",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_repetitionPeriodAndLength_01,
      { "repetitionPeriodAndLength", "rrc.repetitionPeriodAndLength",
        FT_UINT32, BASE_DEC, VALS(rrc_RepetitionPeriodAndLengthForSPS_vals), 0,
        "RepetitionPeriodAndLengthForSPS", HFILL }},
    { &hf_rrc_happyBit_DelayCondition,
      { "happyBit-DelayCondition", "rrc.happyBit_DelayCondition",
        FT_UINT32, BASE_DEC, VALS(rrc_HappyBit_DelayCondition_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_e_TFCI_TableIndex,
      { "e-TFCI-TableIndex", "rrc.e_TFCI_TableIndex",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_e_DCH_MinimumSet_E_TFCI,
      { "e-DCH-MinimumSet-E-TFCI", "rrc.e_DCH_MinimumSet_E_TFCI",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_maxChannelisationCodes,
      { "maxChannelisationCodes", "rrc.maxChannelisationCodes",
        FT_UINT32, BASE_DEC, VALS(rrc_E_DPDCH_MaxChannelisationCodes_vals), 0,
        "E_DPDCH_MaxChannelisationCodes", HFILL }},
    { &hf_rrc_pl_NonMax,
      { "pl-NonMax", "rrc.pl_NonMax",
        FT_UINT32, BASE_DEC, NULL, 0,
        "E_DPDCH_PL_NonMax", HFILL }},
    { &hf_rrc_schedulingInfoConfiguration,
      { "schedulingInfoConfiguration", "rrc.schedulingInfoConfiguration",
        FT_NONE, BASE_NONE, NULL, 0,
        "E_DPDCH_SchedulingInfoConfiguration", HFILL }},
    { &hf_rrc_reference_E_TFCI,
      { "reference-E-TFCI", "rrc.reference_E_TFCI",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_127", HFILL }},
    { &hf_rrc_reference_E_TFCI_PO,
      { "reference-E-TFCI-PO", "rrc.reference_E_TFCI_PO",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_29", HFILL }},
    { &hf_rrc_reference_E_TFCI_PO_r7,
      { "reference-E-TFCI-PO-r7", "rrc.reference_E_TFCI_PO_r7",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_31", HFILL }},
    { &hf_rrc_E_DPDCH_Reference_E_TFCIList_item,
      { "E-DPDCH-Reference-E-TFCI", "rrc.E_DPDCH_Reference_E_TFCI",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_E_DPDCH_Reference_E_TFCIList_r7_item,
      { "E-DPDCH-Reference-E-TFCI-r7", "rrc.E_DPDCH_Reference_E_TFCI_r7",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_periodicityOfSchedInfo_NoGrant,
      { "periodicityOfSchedInfo-NoGrant", "rrc.periodicityOfSchedInfo_NoGrant",
        FT_UINT32, BASE_DEC, VALS(rrc_E_DPDCH_PeriodicyOfSchedInfo_vals), 0,
        "E_DPDCH_PeriodicyOfSchedInfo", HFILL }},
    { &hf_rrc_periodicityOfSchedInfo_Grant,
      { "periodicityOfSchedInfo-Grant", "rrc.periodicityOfSchedInfo_Grant",
        FT_UINT32, BASE_DEC, VALS(rrc_E_DPDCH_PeriodicyOfSchedInfo_vals), 0,
        "E_DPDCH_PeriodicyOfSchedInfo", HFILL }},
    { &hf_rrc_servingGrant_04,
      { "servingGrant", "rrc.servingGrant",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_servingGrant_04", HFILL }},
    { &hf_rrc_primary_Secondary_GrantSelector_04,
      { "primary-Secondary-GrantSelector", "rrc.primary_Secondary_GrantSelector",
        FT_UINT32, BASE_DEC, VALS(rrc_T_primary_Secondary_GrantSelector_04_vals), 0,
        "T_primary_Secondary_GrantSelector_04", HFILL }},
    { &hf_rrc_channelisationCode_02,
      { "channelisationCode", "rrc.channelisationCode",
        FT_UINT32, BASE_DEC, NULL, 0,
        "E_HICH_ChannelisationCode", HFILL }},
    { &hf_rrc_signatureSequence_01,
      { "signatureSequence", "rrc.signatureSequence",
        FT_UINT32, BASE_DEC, NULL, 0,
        "E_HICH_RGCH_SignatureSequence", HFILL }},
    { &hf_rrc_secondarySignatureSequence,
      { "secondarySignatureSequence", "rrc.secondarySignatureSequence",
        FT_UINT32, BASE_DEC, NULL, 0,
        "E_HICH_RGCH_SignatureSequence", HFILL }},
    { &hf_rrc_n_E_HICH,
      { "n-E-HICH", "rrc.n_E_HICH",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_4_44", HFILL }},
    { &hf_rrc_tS_Number,
      { "tS-Number", "rrc.tS_Number",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_14", HFILL }},
    { &hf_rrc_modeSpecificInfo_56,
      { "modeSpecificInfo", "rrc.modeSpecificInfo",
        FT_UINT32, BASE_DEC, VALS(rrc_T_modeSpecificInfo_56_vals), 0,
        "T_modeSpecificInfo_56", HFILL }},
    { &hf_rrc_tdd384_22,
      { "tdd384", "rrc.tdd384",
        FT_UINT32, BASE_DEC, VALS(rrc_DL_TS_ChannelisationCode_vals), 0,
        "DL_TS_ChannelisationCode", HFILL }},
    { &hf_rrc_tdd768_14,
      { "tdd768", "rrc.tdd768",
        FT_UINT32, BASE_DEC|BASE_EXT_STRING, &rrc_DL_TS_ChannelisationCode_VHCR_vals_ext, 0,
        "DL_TS_ChannelisationCode_VHCR", HFILL }},
    { &hf_rrc_burst_Type,
      { "burst-Type", "rrc.burst_Type",
        FT_UINT32, BASE_DEC, VALS(rrc_T_burst_Type_vals), 0,
        "T_burst_Type", HFILL }},
    { &hf_rrc_midamble_Allocation_Mode,
      { "midamble-Allocation-Mode", "rrc.midamble_Allocation_Mode",
        FT_UINT32, BASE_DEC, VALS(rrc_T_midamble_Allocation_Mode_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_ei,
      { "ei", "rrc.ei",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_3", HFILL }},
    { &hf_rrc_channelisation_Code,
      { "channelisation-Code", "rrc.channelisation_Code",
        FT_UINT32, BASE_DEC, VALS(rrc_HS_ChannelisationCode_LCR_vals), 0,
        "HS_ChannelisationCode_LCR", HFILL }},
    { &hf_rrc_midambleAllocationMode_01,
      { "midambleAllocationMode", "rrc.midambleAllocationMode",
        FT_UINT32, BASE_DEC, VALS(rrc_T_midambleAllocationMode_01_vals), 0,
        "T_midambleAllocationMode_01", HFILL }},
    { &hf_rrc_n_E_HICH_01,
      { "n-E-HICH", "rrc.n_E_HICH",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_4_15", HFILL }},
    { &hf_rrc_e_HICH_InfoList,
      { "e-HICH-InfoList", "rrc.e_HICH_InfoList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "E_HICH_Information_LCR_List", HFILL }},
    { &hf_rrc_E_HICH_Information_LCR_List_item,
      { "E-HICH-Information-LCR", "rrc.E_HICH_Information_LCR",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_configurationmode,
      { "configurationmode", "rrc.configurationmode",
        FT_UINT32, BASE_DEC, VALS(rrc_T_configurationmode_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_implicit_01,
      { "implicit", "rrc.implicit",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_explicit_03,
      { "explicit", "rrc.explicit",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_explicit_02", HFILL }},
    { &hf_rrc_midambleAllocationMode_02,
      { "midambleAllocationMode", "rrc.midambleAllocationMode",
        FT_UINT32, BASE_DEC, VALS(rrc_T_midambleAllocationMode_02_vals), 0,
        "T_midambleAllocationMode_02", HFILL }},
    { &hf_rrc_signatureSequenceGroupIndex,
      { "signatureSequenceGroupIndex", "rrc.signatureSequenceGroupIndex",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_19", HFILL }},
    { &hf_rrc_e_TFCS_Info,
      { "e-TFCS-Info", "rrc.e_TFCS_Info",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_modeSpecificInfo_57,
      { "modeSpecificInfo", "rrc.modeSpecificInfo",
        FT_UINT32, BASE_DEC, VALS(rrc_T_modeSpecificInfo_57_vals), 0,
        "T_modeSpecificInfo_57", HFILL }},
    { &hf_rrc_tdd348_tdd768,
      { "tdd348-tdd768", "rrc.tdd348_tdd768",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_noSlotsForTFCIandTPC,
      { "noSlotsForTFCIandTPC", "rrc.noSlotsForTFCIandTPC",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_1_12", HFILL }},
    { &hf_rrc_e_PUCH_ContantValue,
      { "e-PUCH-ContantValue", "rrc.e_PUCH_ContantValue",
        FT_INT32, BASE_DEC, NULL, 0,
        "INTEGER_M35_10", HFILL }},
    { &hf_rrc_e_PUCH_TS_ConfigurationList,
      { "e-PUCH-TS-ConfigurationList", "rrc.e_PUCH_TS_ConfigurationList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "SEQUENCE_SIZE_1_maxTS_2_OF_E_PUCH_TS_Slots", HFILL }},
    { &hf_rrc_e_PUCH_TS_ConfigurationList_item,
      { "E-PUCH-TS-Slots", "rrc.E_PUCH_TS_Slots",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_e_PUCH_CodeHopping,
      { "e-PUCH-CodeHopping", "rrc.e_PUCH_CodeHopping",
        FT_BOOLEAN, BASE_NONE, NULL, 0,
        "BOOLEAN", HFILL }},
    { &hf_rrc_e_PUCH_TPC_Step_Size,
      { "e-PUCH-TPC-Step-Size", "rrc.e_PUCH_TPC_Step_Size",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_1_3", HFILL }},
    { &hf_rrc_minimum_Allowed_Code_Rate,
      { "minimum-Allowed-Code-Rate", "rrc.minimum_Allowed_Code_Rate",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_63", HFILL }},
    { &hf_rrc_maximum_Allowed_Code_Rate,
      { "maximum-Allowed-Code-Rate", "rrc.maximum_Allowed_Code_Rate",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_63", HFILL }},
    { &hf_rrc_tdd128_33,
      { "tdd128", "rrc.tdd128",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_tdd128_33", HFILL }},
    { &hf_rrc_snpl_ReportType,
      { "snpl-ReportType", "rrc.snpl_ReportType",
        FT_UINT32, BASE_DEC, VALS(rrc_T_snpl_ReportType_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_prxdes_base,
      { "prxdes-base", "rrc.prxdes_base",
        FT_INT32, BASE_DEC, NULL, 0,
        "INTEGER_M112_M50", HFILL }},
    { &hf_rrc_beaconPLEst_01,
      { "beaconPLEst", "rrc.beaconPLEst",
        FT_BOOLEAN, BASE_NONE, NULL, 0,
        "BOOLEAN", HFILL }},
    { &hf_rrc_pebase_PowerControlGAP,
      { "pebase-PowerControlGAP", "rrc.pebase_PowerControlGAP",
        FT_UINT32, BASE_DEC, NULL, 0,
        "PowerControlGAP", HFILL }},
    { &hf_rrc_e_PUCH_TS_ConfigurationList_01,
      { "e-PUCH-TS-ConfigurationList", "rrc.e_PUCH_TS_ConfigurationList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "SEQUENCE_SIZE_1_maxTS_LCR_1_OF_E_PUCH_TS_Slots_LCR", HFILL }},
    { &hf_rrc_e_PUCH_TS_ConfigurationList_item_01,
      { "E-PUCH-TS-Slots-LCR", "rrc.E_PUCH_TS_Slots_LCR",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_maximumNumOfRetransSchedInfo,
      { "maximumNumOfRetransSchedInfo", "rrc.maximumNumOfRetransSchedInfo",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_15", HFILL }},
    { &hf_rrc_retransTimerForSchedInfo,
      { "retransTimerForSchedInfo", "rrc.retransTimerForSchedInfo",
        FT_UINT32, BASE_DEC|BASE_EXT_STRING, &rrc_T_retransTimerForSchedInfo_vals_ext, 0,
        NULL, HFILL }},
    { &hf_rrc_snpl_ReportType_01,
      { "snpl-ReportType", "rrc.snpl_ReportType",
        FT_UINT32, BASE_DEC, VALS(rrc_T_snpl_ReportType_01_vals), 0,
        "T_snpl_ReportType_01", HFILL }},
    { &hf_rrc_pebase_PowerControlGAP_01,
      { "pebase-PowerControlGAP", "rrc.pebase_PowerControlGAP",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_1_255", HFILL }},
    { &hf_rrc_retransTimerForSchedInfo_01,
      { "retransTimerForSchedInfo", "rrc.retransTimerForSchedInfo",
        FT_UINT32, BASE_DEC|BASE_EXT_STRING, &rrc_T_retransTimerForSchedInfo_01_vals_ext, 0,
        "T_retransTimerForSchedInfo_01", HFILL }},
    { &hf_rrc_dummy_44,
      { "dummy", "rrc.dummy",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_6", HFILL }},
    { &hf_rrc_ts_Number,
      { "ts-Number", "rrc.ts_Number",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_14", HFILL }},
    { &hf_rrc_timeslotNumber_02,
      { "timeslotNumber", "rrc.timeslotNumber",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_1_5", HFILL }},
    { &hf_rrc_midambleAllocationMode_03,
      { "midambleAllocationMode", "rrc.midambleAllocationMode",
        FT_UINT32, BASE_DEC, VALS(rrc_T_midambleAllocationMode_03_vals), 0,
        "T_midambleAllocationMode_03", HFILL }},
    { &hf_rrc_rg_CombinationIndex,
      { "rg-CombinationIndex", "rrc.rg_CombinationIndex",
        FT_UINT32, BASE_DEC, NULL, 0,
        "E_RGCH_CombinationIndex", HFILL }},
    { &hf_rrc_modeSpecificInfo_58,
      { "modeSpecificInfo", "rrc.modeSpecificInfo",
        FT_UINT32, BASE_DEC, VALS(rrc_T_modeSpecificInfo_58_vals), 0,
        "T_modeSpecificInfo_58", HFILL }},
    { &hf_rrc_tdd384_tdd768_03,
      { "tdd384-tdd768", "rrc.tdd384_tdd768",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_tdd384_tdd768_03", HFILL }},
    { &hf_rrc_e_RUCCH_ConstantValue,
      { "e-RUCCH-ConstantValue", "rrc.e_RUCCH_ConstantValue",
        FT_INT32, BASE_DEC, NULL, 0,
        "INTEGER_M35_10", HFILL }},
    { &hf_rrc_e_RUCCH_PersistenceScalingFactor,
      { "e-RUCCH-PersistenceScalingFactor", "rrc.e_RUCCH_PersistenceScalingFactor",
        FT_UINT32, BASE_DEC, VALS(rrc_PersistenceScalingFactor_vals), 0,
        "PersistenceScalingFactor", HFILL }},
    { &hf_rrc_t_RUCCH,
      { "t-RUCCH", "rrc.t_RUCCH",
        FT_UINT32, BASE_DEC, VALS(rrc_T_t_RUCCH_vals), 0,
        "T_t_RUCCH", HFILL }},
    { &hf_rrc_e_RUCCH_TS_Number,
      { "e-RUCCH-TS-Number", "rrc.e_RUCCH_TS_Number",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_14", HFILL }},
    { &hf_rrc_e_RUCCH_Midamble,
      { "e-RUCCH-Midamble", "rrc.e_RUCCH_Midamble",
        FT_UINT32, BASE_DEC, VALS(rrc_T_e_RUCCH_Midamble_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_t_adv,
      { "t-adv", "rrc.t_adv",
        FT_UINT32, BASE_DEC, VALS(rrc_T_t_adv_vals), 0,
        "T_t_adv", HFILL }},
    { &hf_rrc_t_SCHED,
      { "t-SCHED", "rrc.t_SCHED",
        FT_UINT32, BASE_DEC, VALS(rrc_T_t_SCHED_vals), 0,
        "T_t_SCHED", HFILL }},
    { &hf_rrc_modeSpecificInfo_59,
      { "modeSpecificInfo", "rrc.modeSpecificInfo",
        FT_UINT32, BASE_DEC, VALS(rrc_T_modeSpecificInfo_59_vals), 0,
        "T_modeSpecificInfo_59", HFILL }},
    { &hf_rrc_tdd384_23,
      { "tdd384", "rrc.tdd384",
        FT_UINT32, BASE_DEC, VALS(rrc_T_tdd384_21_vals), 0,
        "T_tdd384_21", HFILL }},
    { &hf_rrc_sF16,
      { "sF16", "rrc.sF16",
        FT_UINT32, BASE_DEC, NULL, 0,
        "SEQUENCE_SIZE_1_8_OF_SF16Codes", HFILL }},
    { &hf_rrc_sF16_item,
      { "SF16Codes", "rrc.SF16Codes",
        FT_UINT32, BASE_DEC, VALS(rrc_SF16Codes_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_sF8,
      { "sF8", "rrc.sF8",
        FT_UINT32, BASE_DEC, NULL, 0,
        "SEQUENCE_SIZE_1_8_OF_SF8Codes", HFILL }},
    { &hf_rrc_sF8_item,
      { "SF8Codes", "rrc.SF8Codes",
        FT_UINT32, BASE_DEC, VALS(rrc_SF8Codes_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_tdd768_15,
      { "tdd768", "rrc.tdd768",
        FT_UINT32, BASE_DEC, VALS(rrc_T_tdd768_13_vals), 0,
        "T_tdd768_13", HFILL }},
    { &hf_rrc_sF32,
      { "sF32", "rrc.sF32",
        FT_UINT32, BASE_DEC, NULL, 0,
        "SEQUENCE_SIZE_1_16_OF_SF32Codes", HFILL }},
    { &hf_rrc_sF32_item,
      { "SF32Codes", "rrc.SF32Codes",
        FT_UINT32, BASE_DEC, VALS(rrc_SF32Codes_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_sF816,
      { "sF816", "rrc.sF816",
        FT_UINT32, BASE_DEC, NULL, 0,
        "SEQUENCE_SIZE_1_16_OF_SF16Codes2", HFILL }},
    { &hf_rrc_sF816_item,
      { "SF16Codes2", "rrc.SF16Codes2",
        FT_UINT32, BASE_DEC, VALS(rrc_SF16Codes2_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_tdd128_34,
      { "tdd128", "rrc.tdd128",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_tdd128_34", HFILL }},
    { &hf_rrc_t_RUCCH_01,
      { "t-RUCCH", "rrc.t_RUCCH",
        FT_UINT32, BASE_DEC, VALS(rrc_T_t_RUCCH_01_vals), 0,
        "T_t_RUCCH_01", HFILL }},
    { &hf_rrc_n_RUCCH,
      { "n-RUCCH", "rrc.n_RUCCH",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_7", HFILL }},
    { &hf_rrc_t_WAIT,
      { "t-WAIT", "rrc.t_WAIT",
        FT_UINT32, BASE_DEC, VALS(rrc_T_t_WAIT_vals), 0,
        "T_t_WAIT", HFILL }},
    { &hf_rrc_t_SI,
      { "t-SI", "rrc.t_SI",
        FT_UINT32, BASE_DEC, VALS(rrc_T_t_SI_vals), 0,
        "T_t_SI", HFILL }},
    { &hf_rrc_extendedEstimationWindow,
      { "extendedEstimationWindow", "rrc.extendedEstimationWindow",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_2_5", HFILL }},
    { &hf_rrc_e_RUCCH_AccessServiceClass,
      { "e-RUCCH-AccessServiceClass", "rrc.e_RUCCH_AccessServiceClass",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_e_RUCCH_PersistenceScalingFactor_01,
      { "e-RUCCH-PersistenceScalingFactor", "rrc.e_RUCCH_PersistenceScalingFactor",
        FT_UINT32, BASE_DEC, NULL, 0,
        "PersistenceScalingFactorList", HFILL }},
    { &hf_rrc_sync_UL_InfoForE_RUCCH,
      { "sync-UL-InfoForE-RUCCH", "rrc.sync_UL_InfoForE_RUCCH",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_prach_information,
      { "prach-information", "rrc.prach_information",
        FT_UINT32, BASE_DEC, NULL, 0,
        "PRACH_Information_LCR_List", HFILL }},
    { &hf_rrc_t_RUCCH_02,
      { "t-RUCCH", "rrc.t_RUCCH",
        FT_UINT32, BASE_DEC, VALS(rrc_T_t_RUCCH_02_vals), 0,
        "T_t_RUCCH_02", HFILL }},
    { &hf_rrc_t_WAIT_01,
      { "t-WAIT", "rrc.t_WAIT",
        FT_UINT32, BASE_DEC, VALS(rrc_T_t_WAIT_01_vals), 0,
        "T_t_WAIT_01", HFILL }},
    { &hf_rrc_t_SI_01,
      { "t-SI", "rrc.t_SI",
        FT_UINT32, BASE_DEC, VALS(rrc_T_t_SI_01_vals), 0,
        "T_t_SI_01", HFILL }},
    { &hf_rrc_e_TFCI_Boost,
      { "e-TFCI-Boost", "rrc.e_TFCI_Boost",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_127", HFILL }},
    { &hf_rrc_delta_T2TP,
      { "delta-T2TP", "rrc.delta_T2TP",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_6", HFILL }},
    { &hf_rrc_reference_Beta_QPSK_List,
      { "reference-Beta-QPSK-List", "rrc.reference_Beta_QPSK_List",
        FT_UINT32, BASE_DEC, NULL, 0,
        "SEQUENCE_SIZE_1_8_OF_Reference_Beta_QPSK", HFILL }},
    { &hf_rrc_reference_Beta_QPSK_List_item,
      { "Reference-Beta-QPSK", "rrc.Reference_Beta_QPSK",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_reference_Beta_16QAM_List,
      { "reference-Beta-16QAM-List", "rrc.reference_Beta_16QAM_List",
        FT_UINT32, BASE_DEC, NULL, 0,
        "SEQUENCE_SIZE_1_8_OF_Reference_Beta_16QAM", HFILL }},
    { &hf_rrc_reference_Beta_16QAM_List_item,
      { "Reference-Beta-16QAM", "rrc.Reference_Beta_16QAM",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_modeSpecificInfo_60,
      { "modeSpecificInfo", "rrc.modeSpecificInfo",
        FT_UINT32, BASE_DEC, VALS(rrc_T_modeSpecificInfo_60_vals), 0,
        "T_modeSpecificInfo_60", HFILL }},
    { &hf_rrc_tdd384_24,
      { "tdd384", "rrc.tdd384",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_tdd384_22", HFILL }},
    { &hf_rrc_ex_ul_TimingAdvance,
      { "ex-ul-TimingAdvance", "rrc.ex_ul_TimingAdvance",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_255", HFILL }},
    { &hf_rrc_tdd768_16,
      { "tdd768", "rrc.tdd768",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_tdd768_14", HFILL }},
    { &hf_rrc_ex_ul_TimingAdvance_01,
      { "ex-ul-TimingAdvance", "rrc.ex_ul_TimingAdvance",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_511", HFILL }},
    { &hf_rrc_ctch_Indicator,
      { "ctch-Indicator", "rrc.ctch_Indicator",
        FT_BOOLEAN, BASE_NONE, NULL, 0,
        "BOOLEAN", HFILL }},
    { &hf_rrc_FACH_PCH_InformationList_item,
      { "FACH-PCH-Information", "rrc.FACH_PCH_Information",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_cCCH_Fallback,
      { "cCCH-Fallback", "rrc.cCCH_Fallback",
        FT_BOOLEAN, BASE_NONE, NULL, 0,
        "BOOLEAN", HFILL }},
    { &hf_rrc_dCCH_Fallback,
      { "dCCH-Fallback", "rrc.dCCH_Fallback",
        FT_BOOLEAN, BASE_NONE, NULL, 0,
        "BOOLEAN", HFILL }},
    { &hf_rrc_timeslot_01,
      { "timeslot", "rrc.timeslot",
        FT_UINT32, BASE_DEC, NULL, 0,
        "TimeslotNumber_LCR_r4", HFILL }},
    { &hf_rrc_channelisationCode_03,
      { "channelisationCode", "rrc.channelisationCode",
        FT_UINT32, BASE_DEC, VALS(rrc_TDD_FPACH_CCode16_r4_vals), 0,
        "TDD_FPACH_CCode16_r4", HFILL }},
    { &hf_rrc_midambleShiftAndBurstType_03,
      { "midambleShiftAndBurstType", "rrc.midambleShiftAndBurstType",
        FT_NONE, BASE_NONE, NULL, 0,
        "MidambleShiftAndBurstType_LCR_r4", HFILL }},
    { &hf_rrc_wi,
      { "wi", "rrc.wi",
        FT_UINT32, BASE_DEC, NULL, 0,
        "Wi_LCR", HFILL }},
    { &hf_rrc_f_tpich_Information,
      { "f-tpich-Information", "rrc.f_tpich_Information",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_f_tpich_Info,
      { "f-tpich-Info", "rrc.f_tpich_Info",
        FT_UINT32, BASE_DEC, VALS(rrc_T_f_tpich_Info_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_ftpich_SlotFormat,
      { "ftpich-SlotFormat", "rrc.ftpich_SlotFormat",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_ftpich_CodeNumber,
      { "ftpich-CodeNumber", "rrc.ftpich_CodeNumber",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_255", HFILL }},
    { &hf_rrc_ftpich_FrameOffset,
      { "ftpich-FrameOffset", "rrc.ftpich_FrameOffset",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_f_tpich_InfoNewServingCell,
      { "f-tpich-InfoNewServingCell", "rrc.f_tpich_InfoNewServingCell",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_f_tpich_InfoOtherCellList,
      { "f-tpich-InfoOtherCellList", "rrc.f_tpich_InfoOtherCellList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "SEQUENCE_SIZE_1_maxRL_OF_F_TPICH_InfoOtherCell", HFILL }},
    { &hf_rrc_f_tpich_InfoOtherCellList_item,
      { "F-TPICH-InfoOtherCell", "rrc.F_TPICH_InfoOtherCell",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_FreqIndexListForEnhancedMeas_item,
      { "FrequencyIndexForEnhancedMeas", "rrc.FrequencyIndexForEnhancedMeas",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_modeSpecificInfo_61,
      { "modeSpecificInfo", "rrc.modeSpecificInfo",
        FT_UINT32, BASE_DEC, VALS(rrc_T_modeSpecificInfo_61_vals), 0,
        "T_modeSpecificInfo_61", HFILL }},
    { &hf_rrc_fdd_94,
      { "fdd", "rrc.fdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "FrequencyInfoFDD", HFILL }},
    { &hf_rrc_tdd_53,
      { "tdd", "rrc.tdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "FrequencyInfoTDD", HFILL }},
    { &hf_rrc_uarfcn_UL,
      { "uarfcn-UL", "rrc.uarfcn_UL",
        FT_UINT32, BASE_DEC, NULL, 0,
        "UARFCN", HFILL }},
    { &hf_rrc_uarfcn_Nt,
      { "uarfcn-Nt", "rrc.uarfcn_Nt",
        FT_UINT32, BASE_DEC, NULL, 0,
        "UARFCN", HFILL }},
    { &hf_rrc_startCode,
      { "startCode", "rrc.startCode",
        FT_UINT32, BASE_DEC, VALS(rrc_HS_ChannelisationCode_LCR_vals), 0,
        "HS_ChannelisationCode_LCR", HFILL }},
    { &hf_rrc_stopCode,
      { "stopCode", "rrc.stopCode",
        FT_UINT32, BASE_DEC, VALS(rrc_HS_ChannelisationCode_LCR_vals), 0,
        "HS_ChannelisationCode_LCR", HFILL }},
    { &hf_rrc_ccch_MappingInfo,
      { "ccch-MappingInfo", "rrc.ccch_MappingInfo",
        FT_NONE, BASE_NONE, NULL, 0,
        "CommonRBMappingInfo", HFILL }},
    { &hf_rrc_srb1_MappingInfo,
      { "srb1-MappingInfo", "rrc.srb1_MappingInfo",
        FT_NONE, BASE_NONE, NULL, 0,
        "CommonRBMappingInfo", HFILL }},
    { &hf_rrc_common_MAC_ehs_ReorderingQueueList,
      { "common-MAC-ehs-ReorderingQueueList", "rrc.common_MAC_ehs_ReorderingQueueList",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_hs_scch_SystemInfo,
      { "hs-scch-SystemInfo", "rrc.hs_scch_SystemInfo",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_harq_SystemInfo,
      { "harq-SystemInfo", "rrc.harq_SystemInfo",
        FT_NONE, BASE_NONE, NULL, 0,
        "HARQ_Info", HFILL }},
    { &hf_rrc_common_H_RNTI_information,
      { "common-H-RNTI-information", "rrc.common_H_RNTI_information",
        FT_UINT32, BASE_DEC, NULL, 0,
        "SEQUENCE_SIZE_1_maxCommonHRNTI_OF_H_RNTI", HFILL }},
    { &hf_rrc_common_H_RNTI_information_item,
      { "H-RNTI", "rrc.H_RNTI",
        FT_BYTES, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_bcchSpecific_H_RNTI,
      { "bcchSpecific-H-RNTI", "rrc.bcchSpecific_H_RNTI",
        FT_BYTES, BASE_NONE, NULL, 0,
        "H_RNTI", HFILL }},
    { &hf_rrc_hs_scch_SystemInfo_tdd128,
      { "hs-scch-SystemInfo-tdd128", "rrc.hs_scch_SystemInfo_tdd128",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_hs_pdsch_MidambleConfiguration,
      { "hs-pdsch-MidambleConfiguration", "rrc.hs_pdsch_MidambleConfiguration",
        FT_NONE, BASE_NONE, NULL, 0,
        "HS_PDSCH_Midamble_Configuration_TDD128", HFILL }},
    { &hf_rrc_t_321,
      { "t-321", "rrc.t_321",
        FT_UINT32, BASE_DEC, VALS(rrc_T_321_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_hs_dsch_DrxCycleFach,
      { "hs-dsch-DrxCycleFach", "rrc.hs_dsch_DrxCycleFach",
        FT_UINT32, BASE_DEC, VALS(rrc_HS_DSCH_DrxCycleFach_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_hs_dsch_DrxBurstFach,
      { "hs-dsch-DrxBurstFach", "rrc.hs_dsch_DrxBurstFach",
        FT_UINT32, BASE_DEC, VALS(rrc_HS_DSCH_DrxBurstFach_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_drxInterruption_hs_dsch,
      { "drxInterruption-hs-dsch", "rrc.drxInterruption_hs_dsch",
        FT_BOOLEAN, BASE_NONE, NULL, 0,
        "BOOLEAN", HFILL }},
    { &hf_rrc_hs_dsch_SecondDrx_CycleFach,
      { "hs-dsch-SecondDrx-CycleFach", "rrc.hs_dsch_SecondDrx_CycleFach",
        FT_UINT32, BASE_DEC, VALS(rrc_HS_DSCH_SecondDrx_CycleFach_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_drx_level,
      { "drx-level", "rrc.drx_level",
        FT_UINT32, BASE_DEC, VALS(rrc_T_drx_level_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_one_level_DRX,
      { "one-level-DRX", "rrc.one_level_DRX",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_hs_dsch_Second_Rx_BurstFach,
      { "hs-dsch-Second-Rx-BurstFach", "rrc.hs_dsch_Second_Rx_BurstFach",
        FT_UINT32, BASE_DEC, VALS(rrc_HS_DSCH_Second_Rx_BurstFach_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_t329,
      { "t329", "rrc.t329",
        FT_UINT32, BASE_DEC, VALS(rrc_T_329_vals), 0,
        "T_329", HFILL }},
    { &hf_rrc_two_level_DRX,
      { "two-level-DRX", "rrc.two_level_DRX",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_t328,
      { "t328", "rrc.t328",
        FT_UINT32, BASE_DEC, VALS(rrc_T_328_vals), 0,
        "T_328", HFILL }},
    { &hf_rrc_hs_dsch_First_Rx_BurstFach,
      { "hs-dsch-First-Rx-BurstFach", "rrc.hs_dsch_First_Rx_BurstFach",
        FT_UINT32, BASE_DEC, VALS(rrc_HS_DSCH_First_Rx_BurstFach_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_hs_dsch_First_Drx_CycleFach,
      { "hs-dsch-First-Drx-CycleFach", "rrc.hs_dsch_First_Drx_CycleFach",
        FT_UINT32, BASE_DEC, VALS(rrc_HS_DSCH_First_Drx_CycleFach_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_dlScramblingCode,
      { "dlScramblingCode", "rrc.dlScramblingCode",
        FT_UINT32, BASE_DEC, NULL, 0,
        "SecondaryScramblingCode", HFILL }},
    { &hf_rrc_pich_ForHSDPASupportedPagingList,
      { "pich-ForHSDPASupportedPagingList", "rrc.pich_ForHSDPASupportedPagingList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "SEQUENCE_SIZE_1_maxSCCPCH_OF_PICH_ForHSDPASupportedPaging", HFILL }},
    { &hf_rrc_pich_ForHSDPASupportedPagingList_item,
      { "PICH-ForHSDPASupportedPaging", "rrc.PICH_ForHSDPASupportedPaging",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_numberOfPcchTransmissions,
      { "numberOfPcchTransmissions", "rrc.numberOfPcchTransmissions",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_1_5", HFILL }},
    { &hf_rrc_transportBlockSizeList,
      { "transportBlockSizeList", "rrc.transportBlockSizeList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "SEQUENCE_SIZE_1_2_OF_TransportBlockSizeIndex", HFILL }},
    { &hf_rrc_transportBlockSizeList_item,
      { "TransportBlockSizeIndex", "rrc.TransportBlockSizeIndex",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_pich_ForHsdschList,
      { "pich-ForHsdschList", "rrc.pich_ForHsdschList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "SEQUENCE_SIZE_1_maxSCCPCH_OF_PICH_ForHSDPASupportedPaging_TDD128", HFILL }},
    { &hf_rrc_pich_ForHsdschList_item,
      { "PICH-ForHSDPASupportedPaging-TDD128", "rrc.PICH_ForHSDPASupportedPaging_TDD128",
        FT_UINT32, BASE_DEC, VALS(rrc_PICH_ForHSDPASupportedPaging_TDD128_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_dtch_DCCH_reception_window_size,
      { "dtch-DCCH-reception-window-size", "rrc.dtch_DCCH_reception_window_size",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_1_16", HFILL }},
    { &hf_rrc_pcch_InformationList,
      { "pcch-InformationList", "rrc.pcch_InformationList",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_HS_DSCH_RxPatternList_TDD128_item,
      { "HS-DSCH-RxPatternList-TDD128 item", "rrc.HS_DSCH_RxPatternList_TDD128_item",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_HS_DSCH_TbsList_TDD128_item,
      { "HS-DSCH-TbsList-TDD128 item", "rrc.HS_DSCH_TbsList_TDD128_item",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_hs_dsch_TBSizeIndex,
      { "hs-dsch-TBSizeIndex", "rrc.hs_dsch_TBSizeIndex",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_1_63", HFILL }},
    { &hf_rrc_hs_dsch_SPS_Operation,
      { "hs-dsch-SPS-Operation", "rrc.hs_dsch_SPS_Operation",
        FT_UINT32, BASE_DEC, VALS(rrc_T_hs_dsch_SPS_Operation_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_newOperation_02,
      { "newOperation", "rrc.newOperation",
        FT_NONE, BASE_NONE, NULL, 0,
        "HS_DSCH_SPS_NewOperation_TDD128", HFILL }},
    { &hf_rrc_hs_dsch_TbsList,
      { "hs-dsch-TbsList", "rrc.hs_dsch_TbsList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "HS_DSCH_TbsList_TDD128", HFILL }},
    { &hf_rrc_hs_dsch_RxPatternList,
      { "hs-dsch-RxPatternList", "rrc.hs_dsch_RxPatternList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "HS_DSCH_RxPatternList_TDD128", HFILL }},
    { &hf_rrc_hARQInfoForSPS,
      { "hARQInfoForSPS", "rrc.hARQInfoForSPS",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_memorySize_01,
      { "memorySize", "rrc.memorySize",
        FT_UINT32, BASE_DEC|BASE_EXT_STRING, &rrc_HARQMemorySize_vals_ext, 0,
        "HARQMemorySize", HFILL }},
    { &hf_rrc_hs_sich_List,
      { "hs-sich-List", "rrc.hs_sich_List",
        FT_UINT32, BASE_DEC, NULL, 0,
        "HS_SICH_List_TDD128", HFILL }},
    { &hf_rrc_initialSPSInfoForHSDSCH,
      { "initialSPSInfoForHSDSCH", "rrc.initialSPSInfoForHSDSCH",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_timeslotInfo,
      { "timeslotInfo", "rrc.timeslotInfo",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_5", HFILL }},
    { &hf_rrc_codeResourceInfo_01,
      { "codeResourceInfo", "rrc.codeResourceInfo",
        FT_NONE, BASE_NONE, NULL, 0,
        "HS_ChannelisationCodeSetInfo_LCR", HFILL }},
    { &hf_rrc_sfnNum,
      { "sfnNum", "rrc.sfnNum",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_1", HFILL }},
    { &hf_rrc_initialRxPatternIndex,
      { "initialRxPatternIndex", "rrc.initialRxPatternIndex",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_maxRxPatternForHSDSCH_TDD128_1", HFILL }},
    { &hf_rrc_initialTfsIndex,
      { "initialTfsIndex", "rrc.initialTfsIndex",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_maxTbsForHSDSCH_TDD128_1", HFILL }},
    { &hf_rrc_modulation,
      { "modulation", "rrc.modulation",
        FT_UINT32, BASE_DEC, VALS(rrc_T_modulation_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_hs_sich_Index,
      { "hs-sich-Index", "rrc.hs_sich_Index",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_maxHSSICH_TDD128_1", HFILL }},
    { &hf_rrc_midambleAllocationMode_04,
      { "midambleAllocationMode", "rrc.midambleAllocationMode",
        FT_UINT32, BASE_DEC, VALS(rrc_T_midambleAllocationMode_04_vals), 0,
        "T_midambleAllocationMode_04", HFILL }},
    { &hf_rrc_modeSpecificInfo_62,
      { "modeSpecificInfo", "rrc.modeSpecificInfo",
        FT_UINT32, BASE_DEC, VALS(rrc_T_modeSpecificInfo_62_vals), 0,
        "T_modeSpecificInfo_62", HFILL }},
    { &hf_rrc_fdd_95,
      { "fdd", "rrc.fdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_fdd_93", HFILL }},
    { &hf_rrc_tdd_54,
      { "tdd", "rrc.tdd",
        FT_UINT32, BASE_DEC, VALS(rrc_T_tdd_50_vals), 0,
        "T_tdd_50", HFILL }},
    { &hf_rrc_tdd384_25,
      { "tdd384", "rrc.tdd384",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_tdd384_23", HFILL }},
    { &hf_rrc_nack_ack_power_offset,
      { "nack-ack-power-offset", "rrc.nack_ack_power_offset",
        FT_INT32, BASE_DEC, NULL, 0,
        "INTEGER_M7_8", HFILL }},
    { &hf_rrc_hs_SICH_PowerControl_Info,
      { "hs-SICH-PowerControl-Info", "rrc.hs_SICH_PowerControl_Info",
        FT_NONE, BASE_NONE, NULL, 0,
        "HS_SICH_Power_Control_Info_TDD384", HFILL }},
    { &hf_rrc_hS_SCCH_SetConfiguration,
      { "hS-SCCH-SetConfiguration", "rrc.hS_SCCH_SetConfiguration",
        FT_UINT32, BASE_DEC, NULL, 0,
        "SEQUENCE_SIZE_1_maxHSSCCHs_OF_HS_SCCH_TDD384", HFILL }},
    { &hf_rrc_hS_SCCH_SetConfiguration_item,
      { "HS-SCCH-TDD384", "rrc.HS_SCCH_TDD384",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_tdd128_35,
      { "tdd128", "rrc.tdd128",
        FT_UINT32, BASE_DEC, NULL, 0,
        "SEQUENCE_SIZE_1_maxHSSCCHs_OF_HS_SCCH_TDD128", HFILL }},
    { &hf_rrc_tdd128_item,
      { "HS-SCCH-TDD128", "rrc.HS_SCCH_TDD128",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_modeSpecificInfo_63,
      { "modeSpecificInfo", "rrc.modeSpecificInfo",
        FT_UINT32, BASE_DEC, VALS(rrc_T_modeSpecificInfo_63_vals), 0,
        "T_modeSpecificInfo_63", HFILL }},
    { &hf_rrc_fdd_96,
      { "fdd", "rrc.fdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_fdd_94", HFILL }},
    { &hf_rrc_tdd_55,
      { "tdd", "rrc.tdd",
        FT_UINT32, BASE_DEC, VALS(rrc_T_tdd_51_vals), 0,
        "T_tdd_51", HFILL }},
    { &hf_rrc_tdd384_26,
      { "tdd384", "rrc.tdd384",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_tdd384_24", HFILL }},
    { &hf_rrc_bler_target,
      { "bler-target", "rrc.bler_target",
        FT_INT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_hS_SCCH_SetConfiguration_01,
      { "hS-SCCH-SetConfiguration", "rrc.hS_SCCH_SetConfiguration",
        FT_UINT32, BASE_DEC, NULL, 0,
        "SEQUENCE_SIZE_1_maxHSSCCHs_OF_HS_SCCH_TDD384_r6", HFILL }},
    { &hf_rrc_hS_SCCH_SetConfiguration_item_01,
      { "HS-SCCH-TDD384-r6", "rrc.HS_SCCH_TDD384_r6",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_tdd128_36,
      { "tdd128", "rrc.tdd128",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_tdd128_35", HFILL }},
    { &hf_rrc_power_level_HSSICH,
      { "power-level-HSSICH", "rrc.power_level_HSSICH",
        FT_INT32, BASE_DEC, NULL, 0,
        "INTEGER_M120_M58", HFILL }},
    { &hf_rrc_tpc_step_size,
      { "tpc-step-size", "rrc.tpc_step_size",
        FT_UINT32, BASE_DEC, VALS(rrc_T_tpc_step_size_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_hS_SCCH_SetConfiguration_02,
      { "hS-SCCH-SetConfiguration", "rrc.hS_SCCH_SetConfiguration",
        FT_UINT32, BASE_DEC, NULL, 0,
        "SEQUENCE_SIZE_1_maxHSSCCHs_OF_HS_SCCH_TDD128_r6", HFILL }},
    { &hf_rrc_hS_SCCH_SetConfiguration_item_02,
      { "HS-SCCH-TDD128-r6", "rrc.HS_SCCH_TDD128_r6",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_modeSpecificInfo_64,
      { "modeSpecificInfo", "rrc.modeSpecificInfo",
        FT_UINT32, BASE_DEC, VALS(rrc_T_modeSpecificInfo_64_vals), 0,
        "T_modeSpecificInfo_64", HFILL }},
    { &hf_rrc_fdd_97,
      { "fdd", "rrc.fdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_fdd_95", HFILL }},
    { &hf_rrc_tdd_56,
      { "tdd", "rrc.tdd",
        FT_UINT32, BASE_DEC, VALS(rrc_T_tdd_52_vals), 0,
        "T_tdd_52", HFILL }},
    { &hf_rrc_tdd384_27,
      { "tdd384", "rrc.tdd384",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_tdd384_25", HFILL }},
    { &hf_rrc_tdd768_17,
      { "tdd768", "rrc.tdd768",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_tdd768_15", HFILL }},
    { &hf_rrc_hs_SICH_PowerControl_Info_01,
      { "hs-SICH-PowerControl-Info", "rrc.hs_SICH_PowerControl_Info",
        FT_NONE, BASE_NONE, NULL, 0,
        "HS_SICH_Power_Control_Info_TDD768", HFILL }},
    { &hf_rrc_hS_SCCH_SetConfiguration_03,
      { "hS-SCCH-SetConfiguration", "rrc.hS_SCCH_SetConfiguration",
        FT_UINT32, BASE_DEC, NULL, 0,
        "SEQUENCE_SIZE_1_maxHSSCCHs_OF_HS_SCCH_TDD768", HFILL }},
    { &hf_rrc_hS_SCCH_SetConfiguration_item_03,
      { "HS-SCCH-TDD768", "rrc.HS_SCCH_TDD768",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_tdd128_37,
      { "tdd128", "rrc.tdd128",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_tdd128_36", HFILL }},
    { &hf_rrc_tpc_step_size_01,
      { "tpc-step-size", "rrc.tpc_step_size",
        FT_UINT32, BASE_DEC, VALS(rrc_T_tpc_step_size_01_vals), 0,
        "T_tpc_step_size_01", HFILL }},
    { &hf_rrc_powerControlGAP,
      { "powerControlGAP", "rrc.powerControlGAP",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_pathlossCompensationSwitch,
      { "pathlossCompensationSwitch", "rrc.pathlossCompensationSwitch",
        FT_BOOLEAN, BASE_NONE, NULL, 0,
        "BOOLEAN", HFILL }},
    { &hf_rrc_modeSpecificInfo_65,
      { "modeSpecificInfo", "rrc.modeSpecificInfo",
        FT_UINT32, BASE_DEC, VALS(rrc_T_modeSpecificInfo_65_vals), 0,
        "T_modeSpecificInfo_65", HFILL }},
    { &hf_rrc_tdd_57,
      { "tdd", "rrc.tdd",
        FT_UINT32, BASE_DEC, VALS(rrc_T_tdd_53_vals), 0,
        "T_tdd_53", HFILL }},
    { &hf_rrc_tdd128_38,
      { "tdd128", "rrc.tdd128",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_tdd128_37", HFILL }},
    { &hf_rrc_hS_SCCH_tpc_step_size,
      { "hS-SCCH-tpc-step-size", "rrc.hS_SCCH_tpc_step_size",
        FT_UINT32, BASE_DEC, VALS(rrc_T_hS_SCCH_tpc_step_size_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_modeSpecificInfo_66,
      { "modeSpecificInfo", "rrc.modeSpecificInfo",
        FT_UINT32, BASE_DEC, VALS(rrc_T_modeSpecificInfo_66_vals), 0,
        "T_modeSpecificInfo_66", HFILL }},
    { &hf_rrc_fdd_98,
      { "fdd", "rrc.fdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_fdd_96", HFILL }},
    { &hf_rrc_tdd_58,
      { "tdd", "rrc.tdd",
        FT_UINT32, BASE_DEC, VALS(rrc_T_tdd_54_vals), 0,
        "T_tdd_54", HFILL }},
    { &hf_rrc_tdd384_28,
      { "tdd384", "rrc.tdd384",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_tdd384_26", HFILL }},
    { &hf_rrc_tdd768_18,
      { "tdd768", "rrc.tdd768",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_tdd768_16", HFILL }},
    { &hf_rrc_tdd128_39,
      { "tdd128", "rrc.tdd128",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_tdd128_38", HFILL }},
    { &hf_rrc_tpc_step_size_02,
      { "tpc-step-size", "rrc.tpc_step_size",
        FT_UINT32, BASE_DEC, VALS(rrc_T_tpc_step_size_02_vals), 0,
        "T_tpc_step_size_02", HFILL }},
    { &hf_rrc_hS_SCCH_tpc_step_size_01,
      { "hS-SCCH-tpc-step-size", "rrc.hS_SCCH_tpc_step_size",
        FT_UINT32, BASE_DEC, VALS(rrc_T_hS_SCCH_tpc_step_size_01_vals), 0,
        "T_hS_SCCH_tpc_step_size_01", HFILL }},
    { &hf_rrc_hS_SCCH_DRX_Cycle,
      { "hS-SCCH-DRX-Cycle", "rrc.hS_SCCH_DRX_Cycle",
        FT_UINT32, BASE_DEC, VALS(rrc_ControlChannelDRXCycle_TDD128_vals), 0,
        "ControlChannelDRXCycle_TDD128", HFILL }},
    { &hf_rrc_hS_SCCH_DRX_InactivityThreshold,
      { "hS-SCCH-DRX-InactivityThreshold", "rrc.hS_SCCH_DRX_InactivityThreshold",
        FT_UINT32, BASE_DEC, VALS(rrc_HS_SCCH_DRX_InactivityThreshold_TDD128_vals), 0,
        "HS_SCCH_DRX_InactivityThreshold_TDD128", HFILL }},
    { &hf_rrc_hS_SCCH_DRX_Offset,
      { "hS-SCCH-DRX-Offset", "rrc.hS_SCCH_DRX_Offset",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_63", HFILL }},
    { &hf_rrc_hs_pdsch_CodeIndex,
      { "hs-pdsch-CodeIndex", "rrc.hs_pdsch_CodeIndex",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_1_15", HFILL }},
    { &hf_rrc_hs_scch_LessTFS,
      { "hs-scch-LessTFS", "rrc.hs_scch_LessTFS",
        FT_UINT32, BASE_DEC, NULL, 0,
        "HS_SCCH_LessTFSList", HFILL }},
    { &hf_rrc_hs_scchLessOperation,
      { "hs-scchLessOperation", "rrc.hs_scchLessOperation",
        FT_UINT32, BASE_DEC, VALS(rrc_T_hs_scchLessOperation_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_newOperation_03,
      { "newOperation", "rrc.newOperation",
        FT_NONE, BASE_NONE, NULL, 0,
        "HS_SCCH_Less_NewOperation", HFILL }},
    { &hf_rrc_HS_SCCH_LessTFSList_item,
      { "HS-SCCH-LessTFSList item", "rrc.HS_SCCH_LessTFSList_item",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_hs_scch_LessTFSI,
      { "hs-scch-LessTFSI", "rrc.hs_scch_LessTFSI",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_1_90", HFILL }},
    { &hf_rrc_hs_scch_LessSecondCodeSupport,
      { "hs-scch-LessSecondCodeSupport", "rrc.hs_scch_LessSecondCodeSupport",
        FT_BOOLEAN, BASE_NONE, NULL, 0,
        "BOOLEAN", HFILL }},
    { &hf_rrc_hs_SCCH_SetConfiguration,
      { "hs-SCCH-SetConfiguration", "rrc.hs_SCCH_SetConfiguration",
        FT_UINT32, BASE_DEC, NULL, 0,
        "SEQUENCE_SIZE_1_maxHSSCCHs_OF_HS_SCCH_TDD128_r6", HFILL }},
    { &hf_rrc_hs_SCCH_SetConfiguration_item,
      { "HS-SCCH-TDD128-r6", "rrc.HS_SCCH_TDD128_r6",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_tpc_step_size_03,
      { "tpc-step-size", "rrc.tpc_step_size",
        FT_UINT32, BASE_DEC, VALS(rrc_T_tpc_step_size_03_vals), 0,
        "T_tpc_step_size_03", HFILL }},
    { &hf_rrc_power_control_gap,
      { "power-control-gap", "rrc.power_control_gap",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_1_255", HFILL }},
    { &hf_rrc_pathloss_compensation_switch,
      { "pathloss-compensation-switch", "rrc.pathloss_compensation_switch",
        FT_BOOLEAN, BASE_NONE, NULL, 0,
        "BOOLEAN", HFILL }},
    { &hf_rrc_midambleAllocationMode_05,
      { "midambleAllocationMode", "rrc.midambleAllocationMode",
        FT_UINT32, BASE_DEC, VALS(rrc_T_midambleAllocationMode_05_vals), 0,
        "T_midambleAllocationMode_05", HFILL }},
    { &hf_rrc_hs_sich_configuration,
      { "hs-sich-configuration", "rrc.hs_sich_configuration",
        FT_NONE, BASE_NONE, NULL, 0,
        "HS_SICH_Configuration_TDD128", HFILL }},
    { &hf_rrc_midambleAllocationMode_06,
      { "midambleAllocationMode", "rrc.midambleAllocationMode",
        FT_UINT32, BASE_DEC, VALS(rrc_T_midambleAllocationMode_06_vals), 0,
        "T_midambleAllocationMode_06", HFILL }},
    { &hf_rrc_hs_sich_configuration_01,
      { "hs-sich-configuration", "rrc.hs_sich_configuration",
        FT_NONE, BASE_NONE, NULL, 0,
        "HS_SICH_Configuration_TDD128_r6", HFILL }},
    { &hf_rrc_uarfcn_HS_SCCH_Rx,
      { "uarfcn-HS-SCCH-Rx", "rrc.uarfcn_HS_SCCH_Rx",
        FT_UINT32, BASE_DEC, NULL, 0,
        "UARFCN", HFILL }},
    { &hf_rrc_midambleAllocationMode_07,
      { "midambleAllocationMode", "rrc.midambleAllocationMode",
        FT_UINT32, BASE_DEC, VALS(rrc_T_midambleAllocationMode_07_vals), 0,
        "T_midambleAllocationMode_07", HFILL }},
    { &hf_rrc_channelisationCode_04,
      { "channelisationCode", "rrc.channelisationCode",
        FT_UINT32, BASE_DEC, VALS(rrc_HS_ChannelisationCode_LCR_vals), 0,
        "HS_ChannelisationCode_LCR", HFILL }},
    { &hf_rrc_midambleAllocationMode_08,
      { "midambleAllocationMode", "rrc.midambleAllocationMode",
        FT_UINT32, BASE_DEC, VALS(rrc_T_midambleAllocationMode_08_vals), 0,
        "T_midambleAllocationMode_08", HFILL }},
    { &hf_rrc_ueSpecificMidamble_01,
      { "ueSpecificMidamble", "rrc.ueSpecificMidamble",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_tpc_step_size_04,
      { "tpc-step-size", "rrc.tpc_step_size",
        FT_UINT32, BASE_DEC, VALS(rrc_T_tpc_step_size_04_vals), 0,
        "T_tpc_step_size_04", HFILL }},
    { &hf_rrc_midambleAllocationMode_09,
      { "midambleAllocationMode", "rrc.midambleAllocationMode",
        FT_UINT32, BASE_DEC, VALS(rrc_T_midambleAllocationMode_09_vals), 0,
        "T_midambleAllocationMode_09", HFILL }},
    { &hf_rrc_ueSpecificMidamble_02,
      { "ueSpecificMidamble", "rrc.ueSpecificMidamble",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_ueSpecificMidamble_01", HFILL }},
    { &hf_rrc_HS_SICH_List_TDD128_item,
      { "HS-SICH-List-TDD128 item", "rrc.HS_SICH_List_TDD128_item",
        FT_UINT32, BASE_DEC, VALS(rrc_HS_SICH_List_TDD128_item_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_implicit_02,
      { "implicit", "rrc.implicit",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_implicit_01", HFILL }},
    { &hf_rrc_hS_SCCH_Index,
      { "hS-SCCH-Index", "rrc.hS_SCCH_Index",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_maxHSSCCHs_1", HFILL }},
    { &hf_rrc_explicit_04,
      { "explicit", "rrc.explicit",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_explicit_03", HFILL }},
    { &hf_rrc_hS_SICH_Info,
      { "hS-SICH-Info", "rrc.hS_SICH_Info",
        FT_NONE, BASE_NONE, NULL, 0,
        "HS_SICH_Configuration_TDD128_r6", HFILL }},
    { &hf_rrc_HS_SICH_ReferenceSignalInfoList_item,
      { "HS-SICH-ReferenceSignalInfoList item", "rrc.HS_SICH_ReferenceSignalInfoList_item",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_midambleShift_01,
      { "midambleShift", "rrc.midambleShift",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_15", HFILL }},
    { &hf_rrc_timeSlot,
      { "timeSlot", "rrc.timeSlot",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_1_5", HFILL }},
    { &hf_rrc_channelisationCode_05,
      { "channelisationCode", "rrc.channelisationCode",
        FT_UINT32, BASE_DEC, VALS(rrc_DL_TS_ChannelisationCode_vals), 0,
        "DL_TS_ChannelisationCode", HFILL }},
    { &hf_rrc_midambleAllocationMode_10,
      { "midambleAllocationMode", "rrc.midambleAllocationMode",
        FT_UINT32, BASE_DEC, VALS(rrc_T_midambleAllocationMode_10_vals), 0,
        "T_midambleAllocationMode_10", HFILL }},
    { &hf_rrc_ueSpecificMidamble_03,
      { "ueSpecificMidamble", "rrc.ueSpecificMidamble",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_ueSpecificMidamble_02", HFILL }},
    { &hf_rrc_midambleconfiguration,
      { "midambleconfiguration", "rrc.midambleconfiguration",
        FT_UINT32, BASE_DEC, VALS(rrc_MidambleConfigurationBurstType1and3_vals), 0,
        "MidambleConfigurationBurstType1and3", HFILL }},
    { &hf_rrc_hs_sich_configuration_02,
      { "hs-sich-configuration", "rrc.hs_sich_configuration",
        FT_NONE, BASE_NONE, NULL, 0,
        "HS_SICH_Configuration_TDD384", HFILL }},
    { &hf_rrc_midambleAllocationMode_11,
      { "midambleAllocationMode", "rrc.midambleAllocationMode",
        FT_UINT32, BASE_DEC, VALS(rrc_T_midambleAllocationMode_11_vals), 0,
        "T_midambleAllocationMode_11", HFILL }},
    { &hf_rrc_ueSpecificMidamble_04,
      { "ueSpecificMidamble", "rrc.ueSpecificMidamble",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_ueSpecificMidamble_03", HFILL }},
    { &hf_rrc_channelisationCode_06,
      { "channelisationCode", "rrc.channelisationCode",
        FT_UINT32, BASE_DEC|BASE_EXT_STRING, &rrc_DL_TS_ChannelisationCode_VHCR_vals_ext, 0,
        "DL_TS_ChannelisationCode_VHCR", HFILL }},
    { &hf_rrc_midambleAllocationMode_12,
      { "midambleAllocationMode", "rrc.midambleAllocationMode",
        FT_UINT32, BASE_DEC, VALS(rrc_T_midambleAllocationMode_12_vals), 0,
        "T_midambleAllocationMode_12", HFILL }},
    { &hf_rrc_ueSpecificMidamble_05,
      { "ueSpecificMidamble", "rrc.ueSpecificMidamble",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_ueSpecificMidamble_04", HFILL }},
    { &hf_rrc_hs_sich_configuration_03,
      { "hs-sich-configuration", "rrc.hs_sich_configuration",
        FT_NONE, BASE_NONE, NULL, 0,
        "HS_SICH_Configuration_TDD768", HFILL }},
    { &hf_rrc_midambleAllocationMode_13,
      { "midambleAllocationMode", "rrc.midambleAllocationMode",
        FT_UINT32, BASE_DEC, VALS(rrc_T_midambleAllocationMode_13_vals), 0,
        "T_midambleAllocationMode_13", HFILL }},
    { &hf_rrc_ueSpecificMidamble_06,
      { "ueSpecificMidamble", "rrc.ueSpecificMidamble",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_ueSpecificMidamble_05", HFILL }},
    { &hf_rrc_midambleAllocationMode_14,
      { "midambleAllocationMode", "rrc.midambleAllocationMode",
        FT_UINT32, BASE_DEC, VALS(rrc_T_midambleAllocationMode_14_vals), 0,
        "T_midambleAllocationMode_14", HFILL }},
    { &hf_rrc_ueSpecificMidamble_07,
      { "ueSpecificMidamble", "rrc.ueSpecificMidamble",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_ueSpecificMidamble_06", HFILL }},
    { &hf_rrc_ul_target_SIR,
      { "ul-target-SIR", "rrc.ul_target_SIR",
        FT_INT32, BASE_DEC, NULL, 0,
        "INTEGER_M22_40", HFILL }},
    { &hf_rrc_hs_sich_ConstantValue,
      { "hs-sich-ConstantValue", "rrc.hs_sich_ConstantValue",
        FT_INT32, BASE_DEC, NULL, 0,
        "ConstantValue", HFILL }},
    { &hf_rrc_midambleShiftAndBurstType_04,
      { "midambleShiftAndBurstType", "rrc.midambleShiftAndBurstType",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_midambleShiftAndBurstType_VHCR,
      { "midambleShiftAndBurstType-VHCR", "rrc.midambleShiftAndBurstType_VHCR",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_modulation_01,
      { "modulation", "rrc.modulation",
        FT_UINT32, BASE_DEC, VALS(rrc_T_modulation_01_vals), 0,
        "T_modulation_01", HFILL }},
    { &hf_rrc_ss_TPC_Symbols,
      { "ss-TPC-Symbols", "rrc.ss_TPC_Symbols",
        FT_UINT32, BASE_DEC, VALS(rrc_T_ss_TPC_Symbols_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_additionalSS_TPC_Symbols,
      { "additionalSS-TPC-Symbols", "rrc.additionalSS_TPC_Symbols",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_1_15", HFILL }},
    { &hf_rrc_modulation_02,
      { "modulation", "rrc.modulation",
        FT_UINT32, BASE_DEC, VALS(rrc_T_modulation_02_vals), 0,
        "T_modulation_02", HFILL }},
    { &hf_rrc_ss_TPC_Symbols_01,
      { "ss-TPC-Symbols", "rrc.ss_TPC_Symbols",
        FT_UINT32, BASE_DEC, VALS(rrc_T_ss_TPC_Symbols_01_vals), 0,
        "T_ss_TPC_Symbols_01", HFILL }},
    { &hf_rrc_midambleShiftAndBurstType_05,
      { "midambleShiftAndBurstType", "rrc.midambleShiftAndBurstType",
        FT_NONE, BASE_NONE, NULL, 0,
        "MidambleShiftAndBurstType_r7", HFILL }},
    { &hf_rrc_ul_TimeslotInterference,
      { "ul-TimeslotInterference", "rrc.ul_TimeslotInterference",
        FT_INT32, BASE_DEC, NULL, 0,
        "TDD_UL_Interference", HFILL }},
    { &hf_rrc_IndividualTS_InterferenceList_item,
      { "IndividualTS-Interference", "rrc.IndividualTS_Interference",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_modeSpecificInfo_67,
      { "modeSpecificInfo", "rrc.modeSpecificInfo",
        FT_UINT32, BASE_DEC, VALS(rrc_T_modeSpecificInfo_67_vals), 0,
        "T_modeSpecificInfo_67", HFILL }},
    { &hf_rrc_fdd_99,
      { "fdd", "rrc.fdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_fdd_97", HFILL }},
    { &hf_rrc_feedback_cycle,
      { "feedback-cycle", "rrc.feedback_cycle",
        FT_UINT32, BASE_DEC, VALS(rrc_Feedback_cycle_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_cqi_RepetitionFactor,
      { "cqi-RepetitionFactor", "rrc.cqi_RepetitionFactor",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_deltaCQI,
      { "deltaCQI", "rrc.deltaCQI",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_modeSpecificInfo_68,
      { "modeSpecificInfo", "rrc.modeSpecificInfo",
        FT_UINT32, BASE_DEC, VALS(rrc_T_modeSpecificInfo_68_vals), 0,
        "T_modeSpecificInfo_68", HFILL }},
    { &hf_rrc_fdd_100,
      { "fdd", "rrc.fdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_fdd_98", HFILL }},
    { &hf_rrc_feedback_cycle_01,
      { "feedback-cycle", "rrc.feedback_cycle",
        FT_UINT32, BASE_DEC, VALS(rrc_Feedback_cycle_r7_vals), 0,
        "Feedback_cycle_r7", HFILL }},
    { &hf_rrc_modeSpecificInfo_69,
      { "modeSpecificInfo", "rrc.modeSpecificInfo",
        FT_UINT32, BASE_DEC, VALS(rrc_T_modeSpecificInfo_69_vals), 0,
        "T_modeSpecificInfo_69", HFILL }},
    { &hf_rrc_fdd_101,
      { "fdd", "rrc.fdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_fdd_99", HFILL }},
    { &hf_rrc_cqi_RepetitionFactor_Assisting,
      { "cqi-RepetitionFactor-Assisting", "rrc.cqi_RepetitionFactor_Assisting",
        FT_UINT32, BASE_DEC, NULL, 0,
        "CQI_RepetitionFactor", HFILL }},
    { &hf_rrc_deltaCQI_01,
      { "deltaCQI", "rrc.deltaCQI",
        FT_UINT32, BASE_DEC, NULL, 0,
        "DeltaCQI_r11", HFILL }},
    { &hf_rrc_burstType_01,
      { "burstType", "rrc.burstType",
        FT_UINT32, BASE_DEC, VALS(rrc_T_burstType_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_type1_01,
      { "type1", "rrc.type1",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_type1_01", HFILL }},
    { &hf_rrc_midambleConfigurationBurstType1and3,
      { "midambleConfigurationBurstType1and3", "rrc.midambleConfigurationBurstType1and3",
        FT_UINT32, BASE_DEC, VALS(rrc_MidambleConfigurationBurstType1and3_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_midambleAllocationMode_15,
      { "midambleAllocationMode", "rrc.midambleAllocationMode",
        FT_UINT32, BASE_DEC, VALS(rrc_T_midambleAllocationMode_15_vals), 0,
        "T_midambleAllocationMode_15", HFILL }},
    { &hf_rrc_ueSpecificMidamble_08,
      { "ueSpecificMidamble", "rrc.ueSpecificMidamble",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_ueSpecificMidamble_07", HFILL }},
    { &hf_rrc_type2,
      { "type2", "rrc.type2",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_midambleConfigurationBurstType2,
      { "midambleConfigurationBurstType2", "rrc.midambleConfigurationBurstType2",
        FT_UINT32, BASE_DEC, VALS(rrc_MidambleConfigurationBurstType2_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_midambleAllocationMode_16,
      { "midambleAllocationMode", "rrc.midambleAllocationMode",
        FT_UINT32, BASE_DEC, VALS(rrc_T_midambleAllocationMode_16_vals), 0,
        "T_midambleAllocationMode_16", HFILL }},
    { &hf_rrc_ueSpecificMidamble_09,
      { "ueSpecificMidamble", "rrc.ueSpecificMidamble",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_ueSpecificMidamble_08", HFILL }},
    { &hf_rrc_midambleShift_02,
      { "midambleShift", "rrc.midambleShift",
        FT_UINT32, BASE_DEC, NULL, 0,
        "MidambleShiftShort", HFILL }},
    { &hf_rrc_type3,
      { "type3", "rrc.type3",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_midambleAllocationMode_17,
      { "midambleAllocationMode", "rrc.midambleAllocationMode",
        FT_UINT32, BASE_DEC, VALS(rrc_T_midambleAllocationMode_17_vals), 0,
        "T_midambleAllocationMode_17", HFILL }},
    { &hf_rrc_ueSpecificMidamble_10,
      { "ueSpecificMidamble", "rrc.ueSpecificMidamble",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_ueSpecificMidamble_09", HFILL }},
    { &hf_rrc_burstType_02,
      { "burstType", "rrc.burstType",
        FT_UINT32, BASE_DEC, VALS(rrc_T_burstType_01_vals), 0,
        "T_burstType_01", HFILL }},
    { &hf_rrc_type1_02,
      { "type1", "rrc.type1",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_type1_02", HFILL }},
    { &hf_rrc_midambleAllocationMode_18,
      { "midambleAllocationMode", "rrc.midambleAllocationMode",
        FT_UINT32, BASE_DEC, VALS(rrc_T_midambleAllocationMode_18_vals), 0,
        "T_midambleAllocationMode_18", HFILL }},
    { &hf_rrc_ueSpecificMidamble_11,
      { "ueSpecificMidamble", "rrc.ueSpecificMidamble",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_ueSpecificMidamble_10", HFILL }},
    { &hf_rrc_type2_01,
      { "type2", "rrc.type2",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_type2_01", HFILL }},
    { &hf_rrc_midambleConfigurationBurstType2_01,
      { "midambleConfigurationBurstType2", "rrc.midambleConfigurationBurstType2",
        FT_UINT32, BASE_DEC, VALS(rrc_MidambleConfigurationBurstType2_VHCR_vals), 0,
        "MidambleConfigurationBurstType2_VHCR", HFILL }},
    { &hf_rrc_midambleAllocationMode_19,
      { "midambleAllocationMode", "rrc.midambleAllocationMode",
        FT_UINT32, BASE_DEC, VALS(rrc_T_midambleAllocationMode_19_vals), 0,
        "T_midambleAllocationMode_19", HFILL }},
    { &hf_rrc_ueSpecificMidamble_12,
      { "ueSpecificMidamble", "rrc.ueSpecificMidamble",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_ueSpecificMidamble_11", HFILL }},
    { &hf_rrc_midambleShift_03,
      { "midambleShift", "rrc.midambleShift",
        FT_UINT32, BASE_DEC, NULL, 0,
        "MidambleShiftShort_VHCR", HFILL }},
    { &hf_rrc_type3_01,
      { "type3", "rrc.type3",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_type3_01", HFILL }},
    { &hf_rrc_midambleAllocationMode_20,
      { "midambleAllocationMode", "rrc.midambleAllocationMode",
        FT_UINT32, BASE_DEC, VALS(rrc_T_midambleAllocationMode_20_vals), 0,
        "T_midambleAllocationMode_20", HFILL }},
    { &hf_rrc_ueSpecificMidamble_13,
      { "ueSpecificMidamble", "rrc.ueSpecificMidamble",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_ueSpecificMidamble_12", HFILL }},
    { &hf_rrc_mbsfnBurstType4,
      { "mbsfnBurstType4", "rrc.mbsfnBurstType4",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_burstType_03,
      { "burstType", "rrc.burstType",
        FT_UINT32, BASE_DEC, VALS(rrc_T_burstType_02_vals), 0,
        "T_burstType_02", HFILL }},
    { &hf_rrc_type1_03,
      { "type1", "rrc.type1",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_type1_03", HFILL }},
    { &hf_rrc_midambleAllocationMode_21,
      { "midambleAllocationMode", "rrc.midambleAllocationMode",
        FT_UINT32, BASE_DEC, VALS(rrc_T_midambleAllocationMode_21_vals), 0,
        "T_midambleAllocationMode_21", HFILL }},
    { &hf_rrc_ueSpecificMidamble_14,
      { "ueSpecificMidamble", "rrc.ueSpecificMidamble",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_ueSpecificMidamble_13", HFILL }},
    { &hf_rrc_type2_02,
      { "type2", "rrc.type2",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_type2_02", HFILL }},
    { &hf_rrc_midambleAllocationMode_22,
      { "midambleAllocationMode", "rrc.midambleAllocationMode",
        FT_UINT32, BASE_DEC, VALS(rrc_T_midambleAllocationMode_22_vals), 0,
        "T_midambleAllocationMode_22", HFILL }},
    { &hf_rrc_ueSpecificMidamble_15,
      { "ueSpecificMidamble", "rrc.ueSpecificMidamble",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_ueSpecificMidamble_14", HFILL }},
    { &hf_rrc_type3_02,
      { "type3", "rrc.type3",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_type3_02", HFILL }},
    { &hf_rrc_midambleAllocationMode_23,
      { "midambleAllocationMode", "rrc.midambleAllocationMode",
        FT_UINT32, BASE_DEC, VALS(rrc_T_midambleAllocationMode_23_vals), 0,
        "T_midambleAllocationMode_23", HFILL }},
    { &hf_rrc_ueSpecificMidamble_16,
      { "ueSpecificMidamble", "rrc.ueSpecificMidamble",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_ueSpecificMidamble_15", HFILL }},
    { &hf_rrc_burstType_04,
      { "burstType", "rrc.burstType",
        FT_UINT32, BASE_DEC, VALS(rrc_T_burstType_03_vals), 0,
        "T_burstType_03", HFILL }},
    { &hf_rrc_type1_04,
      { "type1", "rrc.type1",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_type1_04", HFILL }},
    { &hf_rrc_midambleAllocationMode_24,
      { "midambleAllocationMode", "rrc.midambleAllocationMode",
        FT_UINT32, BASE_DEC, VALS(rrc_T_midambleAllocationMode_24_vals), 0,
        "T_midambleAllocationMode_24", HFILL }},
    { &hf_rrc_ueSpecificMidamble_17,
      { "ueSpecificMidamble", "rrc.ueSpecificMidamble",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_ueSpecificMidamble_16", HFILL }},
    { &hf_rrc_type2_03,
      { "type2", "rrc.type2",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_type2_03", HFILL }},
    { &hf_rrc_midambleAllocationMode_25,
      { "midambleAllocationMode", "rrc.midambleAllocationMode",
        FT_UINT32, BASE_DEC, VALS(rrc_T_midambleAllocationMode_25_vals), 0,
        "T_midambleAllocationMode_25", HFILL }},
    { &hf_rrc_ueSpecificMidamble_18,
      { "ueSpecificMidamble", "rrc.ueSpecificMidamble",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_ueSpecificMidamble_17", HFILL }},
    { &hf_rrc_burstType_05,
      { "burstType", "rrc.burstType",
        FT_UINT32, BASE_DEC, VALS(rrc_T_burstType_04_vals), 0,
        "T_burstType_04", HFILL }},
    { &hf_rrc_type1_05,
      { "type1", "rrc.type1",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_type1_05", HFILL }},
    { &hf_rrc_midambleAllocationMode_26,
      { "midambleAllocationMode", "rrc.midambleAllocationMode",
        FT_UINT32, BASE_DEC, VALS(rrc_T_midambleAllocationMode_26_vals), 0,
        "T_midambleAllocationMode_26", HFILL }},
    { &hf_rrc_ueSpecificMidamble_19,
      { "ueSpecificMidamble", "rrc.ueSpecificMidamble",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_ueSpecificMidamble_18", HFILL }},
    { &hf_rrc_type2_04,
      { "type2", "rrc.type2",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_type2_04", HFILL }},
    { &hf_rrc_midambleAllocationMode_27,
      { "midambleAllocationMode", "rrc.midambleAllocationMode",
        FT_UINT32, BASE_DEC, VALS(rrc_T_midambleAllocationMode_27_vals), 0,
        "T_midambleAllocationMode_27", HFILL }},
    { &hf_rrc_ueSpecificMidamble_20,
      { "ueSpecificMidamble", "rrc.ueSpecificMidamble",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_ueSpecificMidamble_19", HFILL }},
    { &hf_rrc_midambleAllocationMode_28,
      { "midambleAllocationMode", "rrc.midambleAllocationMode",
        FT_UINT32, BASE_DEC, VALS(rrc_T_midambleAllocationMode_28_vals), 0,
        "T_midambleAllocationMode_28", HFILL }},
    { &hf_rrc_ueSpecificMidamble_21,
      { "ueSpecificMidamble", "rrc.ueSpecificMidamble",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_ueSpecificMidamble_20", HFILL }},
    { &hf_rrc_burstType_06,
      { "burstType", "rrc.burstType",
        FT_UINT32, BASE_DEC, VALS(rrc_T_burstType_05_vals), 0,
        "T_burstType_05", HFILL }},
    { &hf_rrc_type1_06,
      { "type1", "rrc.type1",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_type1_06", HFILL }},
    { &hf_rrc_midambleConfigurationBurstType1,
      { "midambleConfigurationBurstType1", "rrc.midambleConfigurationBurstType1",
        FT_UINT32, BASE_DEC, VALS(rrc_MidambleConfigurationBurstType1_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_midambleAllocationMode_29,
      { "midambleAllocationMode", "rrc.midambleAllocationMode",
        FT_UINT32, BASE_DEC, VALS(rrc_T_midambleAllocationMode_29_vals), 0,
        "T_midambleAllocationMode_29", HFILL }},
    { &hf_rrc_ueSpecificMidamble_22,
      { "ueSpecificMidamble", "rrc.ueSpecificMidamble",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_ueSpecificMidamble_21", HFILL }},
    { &hf_rrc_type2_05,
      { "type2", "rrc.type2",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_type2_05", HFILL }},
    { &hf_rrc_midambleAllocationMode_30,
      { "midambleAllocationMode", "rrc.midambleAllocationMode",
        FT_UINT32, BASE_DEC, VALS(rrc_T_midambleAllocationMode_30_vals), 0,
        "T_midambleAllocationMode_30", HFILL }},
    { &hf_rrc_ueSpecificMidamble_23,
      { "ueSpecificMidamble", "rrc.ueSpecificMidamble",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_ueSpecificMidamble_22", HFILL }},
    { &hf_rrc_mimoOperation,
      { "mimoOperation", "rrc.mimoOperation",
        FT_UINT32, BASE_DEC, VALS(rrc_MIMO_Operation_vals), 0,
        "MIMO_Operation", HFILL }},
    { &hf_rrc_mimoN_M_Ratio,
      { "mimoN-M-Ratio", "rrc.mimoN_M_Ratio",
        FT_UINT32, BASE_DEC, VALS(rrc_MIMO_N_M_Ratio_vals), 0,
        "MIMO_N_M_Ratio", HFILL }},
    { &hf_rrc_mimoPilotConfiguration,
      { "mimoPilotConfiguration", "rrc.mimoPilotConfiguration",
        FT_NONE, BASE_NONE, NULL, 0,
        "MIMO_PilotConfiguration", HFILL }},
    { &hf_rrc_newConfiguration_06,
      { "newConfiguration", "rrc.newConfiguration",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_newConfiguration_06", HFILL }},
    { &hf_rrc_mimo4x4N_M_Ratio,
      { "mimo4x4N-M-Ratio", "rrc.mimo4x4N_M_Ratio",
        FT_UINT32, BASE_DEC, VALS(rrc_MIMO_N_M_Ratio_vals), 0,
        "MIMO_N_M_Ratio", HFILL }},
    { &hf_rrc_mimo4x4PilotConfiguration,
      { "mimo4x4PilotConfiguration", "rrc.mimo4x4PilotConfiguration",
        FT_NONE, BASE_NONE, NULL, 0,
        "MIMO4x4_PilotConfiguration", HFILL }},
    { &hf_rrc_precodingWeightSetRestriction,
      { "precodingWeightSetRestriction", "rrc.precodingWeightSetRestriction",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_64", HFILL }},
    { &hf_rrc_precodingWeightSetRestriction_01,
      { "precodingWeightSetRestriction", "rrc.precodingWeightSetRestriction",
        FT_UINT32, BASE_DEC, VALS(rrc_T_precodingWeightSetRestriction_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_modeSpecificInfo_70,
      { "modeSpecificInfo", "rrc.modeSpecificInfo",
        FT_UINT32, BASE_DEC, VALS(rrc_T_modeSpecificInfo_70_vals), 0,
        "T_modeSpecificInfo_70", HFILL }},
    { &hf_rrc_fdd_102,
      { "fdd", "rrc.fdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_fdd_100", HFILL }},
    { &hf_rrc_tdd_59,
      { "tdd", "rrc.tdd",
        FT_UINT32, BASE_DEC, VALS(rrc_T_tdd_55_vals), 0,
        "T_tdd_55", HFILL }},
    { &hf_rrc_tdd128_40,
      { "tdd128", "rrc.tdd128",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_tdd128_39", HFILL }},
    { &hf_rrc_mimoSFModeForHSPDSCHDualStream,
      { "mimoSFModeForHSPDSCHDualStream", "rrc.mimoSFModeForHSPDSCHDualStream",
        FT_UINT32, BASE_DEC, VALS(rrc_T_mimoSFModeForHSPDSCHDualStream_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_hs_sich_ReferenceSignalInfoList,
      { "hs-sich-ReferenceSignalInfoList", "rrc.hs_sich_ReferenceSignalInfoList",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_tdd384_tdd768_04,
      { "tdd384-tdd768", "rrc.tdd384_tdd768",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_modeSpecificInfo_71,
      { "modeSpecificInfo", "rrc.modeSpecificInfo",
        FT_UINT32, BASE_DEC, VALS(rrc_T_modeSpecificInfo_71_vals), 0,
        "T_modeSpecificInfo_71", HFILL }},
    { &hf_rrc_fdd_103,
      { "fdd", "rrc.fdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_fdd_101", HFILL }},
    { &hf_rrc_mimoPilotConfiguration_01,
      { "mimoPilotConfiguration", "rrc.mimoPilotConfiguration",
        FT_NONE, BASE_NONE, NULL, 0,
        "MIMO_PilotConfiguration_r9", HFILL }},
    { &hf_rrc_precodingWeightSetRestriction_02,
      { "precodingWeightSetRestriction", "rrc.precodingWeightSetRestriction",
        FT_UINT32, BASE_DEC, VALS(rrc_T_precodingWeightSetRestriction_01_vals), 0,
        "T_precodingWeightSetRestriction_01", HFILL }},
    { &hf_rrc_tdd_60,
      { "tdd", "rrc.tdd",
        FT_UINT32, BASE_DEC, VALS(rrc_T_tdd_56_vals), 0,
        "T_tdd_56", HFILL }},
    { &hf_rrc_tdd128_41,
      { "tdd128", "rrc.tdd128",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_tdd128_40", HFILL }},
    { &hf_rrc_mimoSFModeForHSPDSCHDualStream_01,
      { "mimoSFModeForHSPDSCHDualStream", "rrc.mimoSFModeForHSPDSCHDualStream",
        FT_UINT32, BASE_DEC, VALS(rrc_T_mimoSFModeForHSPDSCHDualStream_01_vals), 0,
        "T_mimoSFModeForHSPDSCHDualStream_01", HFILL }},
    { &hf_rrc_secondCPICH_Pattern,
      { "secondCPICH-Pattern", "rrc.secondCPICH_Pattern",
        FT_UINT32, BASE_DEC, VALS(rrc_T_secondCPICH_Pattern_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_normalPattern,
      { "normalPattern", "rrc.normalPattern",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_diversityPattern,
      { "diversityPattern", "rrc.diversityPattern",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_s_cpich_PowerOffset_Mimo_01,
      { "s-cpich-PowerOffset-Mimo", "rrc.s_cpich_PowerOffset_Mimo",
        FT_INT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_secondCPICH_Pattern_01,
      { "secondCPICH-Pattern", "rrc.secondCPICH_Pattern",
        FT_UINT32, BASE_DEC, VALS(rrc_T_secondCPICH_Pattern_01_vals), 0,
        "T_secondCPICH_Pattern_01", HFILL }},
    { &hf_rrc_diversityPattern_01,
      { "diversityPattern", "rrc.diversityPattern",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_diversityPattern_01", HFILL }},
    { &hf_rrc_antenna2,
      { "antenna2", "rrc.antenna2",
        FT_NONE, BASE_NONE, NULL, 0,
        "MIMO_PilotConfiguration_r9", HFILL }},
    { &hf_rrc_antenna3And4,
      { "antenna3And4", "rrc.antenna3And4",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_secondFrequencyInfo,
      { "secondFrequencyInfo", "rrc.secondFrequencyInfo",
        FT_NONE, BASE_NONE, NULL, 0,
        "FrequencyInfoTDD", HFILL }},
    { &hf_rrc_fPachFrequencyInfo,
      { "fPachFrequencyInfo", "rrc.fPachFrequencyInfo",
        FT_NONE, BASE_NONE, NULL, 0,
        "FrequencyInfoTDD", HFILL }},
    { &hf_rrc_upPCHpositionInfo,
      { "upPCHpositionInfo", "rrc.upPCHpositionInfo",
        FT_UINT32, BASE_DEC, NULL, 0,
        "UpPCHposition_LCR", HFILL }},
    { &hf_rrc_cellType,
      { "cellType", "rrc.cellType",
        FT_UINT32, BASE_DEC, VALS(rrc_MultiflowCellType_vals), 0,
        "MultiflowCellType", HFILL }},
    { &hf_rrc_timeReferenceCell,
      { "timeReferenceCell", "rrc.timeReferenceCell",
        FT_UINT32, BASE_DEC, VALS(rrc_T_timeReferenceCell_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_newConfiguration_07,
      { "newConfiguration", "rrc.newConfiguration",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_newConfiguration_07", HFILL }},
    { &hf_rrc_mu_MIMO_Operation,
      { "mu-MIMO-Operation", "rrc.mu_MIMO_Operation",
        FT_UINT32, BASE_DEC, VALS(rrc_MU_MIMO_Operation_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_standaloneMidambleInfo,
      { "standaloneMidambleInfo", "rrc.standaloneMidambleInfo",
        FT_NONE, BASE_NONE, NULL, 0,
        "StandaloneMidambleInfo_TDD128", HFILL }},
    { &hf_rrc_enablingDelay_01,
      { "enablingDelay", "rrc.enablingDelay",
        FT_UINT32, BASE_DEC, VALS(rrc_EnablingDelay_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_ue_dtx_drx_Offset,
      { "ue-dtx-drx-Offset", "rrc.ue_dtx_drx_Offset",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_hs_dpcch_TransmitContinuationOffset,
      { "hs-dpcch-TransmitContinuationOffset", "rrc.hs_dpcch_TransmitContinuationOffset",
        FT_UINT32, BASE_DEC, VALS(rrc_T_hs_dpcch_TransmitContinuationOffset_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_tdd384_768,
      { "tdd384-768", "rrc.tdd384_768",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_timeslotResourceRelatedInfo_01,
      { "timeslotResourceRelatedInfo", "rrc.timeslotResourceRelatedInfo",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_13", HFILL }},
    { &hf_rrc_tdd128_42,
      { "tdd128", "rrc.tdd128",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_tdd128_41", HFILL }},
    { &hf_rrc_e_HICH_Info_08,
      { "e-HICH-Info", "rrc.e_HICH_Info",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_e_HICH_Info_06", HFILL }},
    { &hf_rrc_midambleAllocationMode_31,
      { "midambleAllocationMode", "rrc.midambleAllocationMode",
        FT_UINT32, BASE_DEC, VALS(rrc_T_midambleAllocationMode_31_vals), 0,
        "T_midambleAllocationMode_31", HFILL }},
    { &hf_rrc_tdd384_768_01,
      { "tdd384-768", "rrc.tdd384_768",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_tdd128_43,
      { "tdd128", "rrc.tdd128",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_tdd128_42", HFILL }},
    { &hf_rrc_t_SI_nst,
      { "t-SI-nst", "rrc.t_SI_nst",
        FT_UINT32, BASE_DEC, VALS(rrc_T_t_SI_nst_vals), 0,
        "T_t_SI_nst", HFILL }},
    { &hf_rrc_tdd384_768_02,
      { "tdd384-768", "rrc.tdd384_768",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_tdd384_768_01", HFILL }},
    { &hf_rrc_tdd128_44,
      { "tdd128", "rrc.tdd128",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_tdd128_43", HFILL }},
    { &hf_rrc_e_HICH_Info_09,
      { "e-HICH-Info", "rrc.e_HICH_Info",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_e_HICH_Info_07", HFILL }},
    { &hf_rrc_midambleAllocationMode_32,
      { "midambleAllocationMode", "rrc.midambleAllocationMode",
        FT_UINT32, BASE_DEC, VALS(rrc_T_midambleAllocationMode_32_vals), 0,
        "T_midambleAllocationMode_32", HFILL }},
    { &hf_rrc_t_SI_nst_01,
      { "t-SI-nst", "rrc.t_SI_nst",
        FT_UINT32, BASE_DEC, VALS(rrc_T_t_SI_nst_01_vals), 0,
        "T_t_SI_nst_01", HFILL }},
    { &hf_rrc_dpch_ConstantValue,
      { "dpch-ConstantValue", "rrc.dpch_ConstantValue",
        FT_INT32, BASE_DEC, NULL, 0,
        "ConstantValueTdd", HFILL }},
    { &hf_rrc_ipdl_alpha,
      { "ipdl-alpha", "rrc.ipdl_alpha",
        FT_UINT32, BASE_DEC, NULL, 0,
        "Alpha", HFILL }},
    { &hf_rrc_maxPowerIncrease,
      { "maxPowerIncrease", "rrc.maxPowerIncrease",
        FT_UINT32, BASE_DEC, NULL, 0,
        "MaxPowerIncrease_r4", HFILL }},
    { &hf_rrc_paging_associatedHspdschInfo,
      { "paging-associatedHspdschInfo", "rrc.paging_associatedHspdschInfo",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_paging_associatedHspdschInfo_item,
      { "paging-associatedHspdschInfo item", "rrc.paging_associatedHspdschInfo_item",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_timeslotResourceRelatedInfo_02,
      { "timeslotResourceRelatedInfo", "rrc.timeslotResourceRelatedInfo",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_6", HFILL }},
    { &hf_rrc_codeResourceInformation,
      { "codeResourceInformation", "rrc.codeResourceInformation",
        FT_NONE, BASE_NONE, NULL, 0,
        "CodeResourceInformation_TDD128", HFILL }},
    { &hf_rrc_paging_sub_Channel_size,
      { "paging-sub-Channel-size", "rrc.paging_sub_Channel_size",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_1_3", HFILL }},
    { &hf_rrc_pcpch_UL_ScramblingCode,
      { "pcpch-UL-ScramblingCode", "rrc.pcpch_UL_ScramblingCode",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_79", HFILL }},
    { &hf_rrc_pcpch_DL_ChannelisationCode,
      { "pcpch-DL-ChannelisationCode", "rrc.pcpch_DL_ChannelisationCode",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_511", HFILL }},
    { &hf_rrc_pcpch_DL_ScramblingCode,
      { "pcpch-DL-ScramblingCode", "rrc.pcpch_DL_ScramblingCode",
        FT_UINT32, BASE_DEC, NULL, 0,
        "SecondaryScramblingCode", HFILL }},
    { &hf_rrc_pcp_Length,
      { "pcp-Length", "rrc.pcp_Length",
        FT_UINT32, BASE_DEC, VALS(rrc_PCP_Length_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_ucsm_Info,
      { "ucsm-Info", "rrc.ucsm_Info",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_PCPCH_ChannelInfoList_item,
      { "PCPCH-ChannelInfo", "rrc.PCPCH_ChannelInfo",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_pdsch_PowerControlInfo,
      { "pdsch-PowerControlInfo", "rrc.pdsch_PowerControlInfo",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_pdsch_AllocationPeriodInfo,
      { "pdsch-AllocationPeriodInfo", "rrc.pdsch_AllocationPeriodInfo",
        FT_NONE, BASE_NONE, NULL, 0,
        "AllocationPeriodInfo", HFILL }},
    { &hf_rrc_configuration,
      { "configuration", "rrc.configuration",
        FT_UINT32, BASE_DEC, VALS(rrc_T_configuration_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_old_Configuration,
      { "old-Configuration", "rrc.old_Configuration",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_pdsch_Identity,
      { "pdsch-Identity", "rrc.pdsch_Identity",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_new_Configuration,
      { "new-Configuration", "rrc.new_Configuration",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_pdsch_Info,
      { "pdsch-Info", "rrc.pdsch_Info",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_configuration_01,
      { "configuration", "rrc.configuration",
        FT_UINT32, BASE_DEC, VALS(rrc_T_configuration_01_vals), 0,
        "T_configuration_01", HFILL }},
    { &hf_rrc_old_Configuration_01,
      { "old-Configuration", "rrc.old_Configuration",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_old_Configuration_01", HFILL }},
    { &hf_rrc_new_Configuration_01,
      { "new-Configuration", "rrc.new_Configuration",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_new_Configuration_01", HFILL }},
    { &hf_rrc_pdsch_Info_01,
      { "pdsch-Info", "rrc.pdsch_Info",
        FT_NONE, BASE_NONE, NULL, 0,
        "PDSCH_Info_r4", HFILL }},
    { &hf_rrc_configuration_02,
      { "configuration", "rrc.configuration",
        FT_UINT32, BASE_DEC, VALS(rrc_T_configuration_02_vals), 0,
        "T_configuration_02", HFILL }},
    { &hf_rrc_old_Configuration_02,
      { "old-Configuration", "rrc.old_Configuration",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_old_Configuration_02", HFILL }},
    { &hf_rrc_new_Configuration_02,
      { "new-Configuration", "rrc.new_Configuration",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_new_Configuration_02", HFILL }},
    { &hf_rrc_pdsch_Info_02,
      { "pdsch-Info", "rrc.pdsch_Info",
        FT_NONE, BASE_NONE, NULL, 0,
        "PDSCH_Info_r7", HFILL }},
    { &hf_rrc_PDSCH_CodeInfoList_item,
      { "PDSCH-CodeInfo", "rrc.PDSCH_CodeInfo",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_codeNumberStart,
      { "codeNumberStart", "rrc.codeNumberStart",
        FT_UINT32, BASE_DEC, NULL, 0,
        "CodeNumberDSCH", HFILL }},
    { &hf_rrc_codeNumberStop,
      { "codeNumberStop", "rrc.codeNumberStop",
        FT_UINT32, BASE_DEC, NULL, 0,
        "CodeNumberDSCH", HFILL }},
    { &hf_rrc_PDSCH_CodeMapList_item,
      { "PDSCH-CodeMap", "rrc.PDSCH_CodeMap",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_signallingMethod,
      { "signallingMethod", "rrc.signallingMethod",
        FT_UINT32, BASE_DEC, VALS(rrc_T_signallingMethod_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_codeRange,
      { "codeRange", "rrc.codeRange",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_tfci_Range_01,
      { "tfci-Range", "rrc.tfci_Range",
        FT_UINT32, BASE_DEC, NULL, 0,
        "DSCH_MappingList", HFILL }},
    { &hf_rrc_explicit_config_03,
      { "explicit-config", "rrc.explicit_config",
        FT_UINT32, BASE_DEC, NULL, 0,
        "PDSCH_CodeInfoList", HFILL }},
    { &hf_rrc_replace,
      { "replace", "rrc.replace",
        FT_UINT32, BASE_DEC, NULL, 0,
        "ReplacedPDSCH_CodeInfoList", HFILL }},
    { &hf_rrc_pdsch_TimeslotsCodes,
      { "pdsch-TimeslotsCodes", "rrc.pdsch_TimeslotsCodes",
        FT_NONE, BASE_NONE, NULL, 0,
        "DownlinkTimeslotsCodes", HFILL }},
    { &hf_rrc_tddOption_12,
      { "tddOption", "rrc.tddOption",
        FT_UINT32, BASE_DEC, VALS(rrc_T_tddOption_12_vals), 0,
        "T_tddOption_12", HFILL }},
    { &hf_rrc_tdd384_29,
      { "tdd384", "rrc.tdd384",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_tdd384_27", HFILL }},
    { &hf_rrc_tdd128_45,
      { "tdd128", "rrc.tdd128",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_tdd128_44", HFILL }},
    { &hf_rrc_pdsch_TimeslotsCodes_01,
      { "pdsch-TimeslotsCodes", "rrc.pdsch_TimeslotsCodes",
        FT_NONE, BASE_NONE, NULL, 0,
        "DownlinkTimeslotsCodes_LCR_r4", HFILL }},
    { &hf_rrc_tddOption_13,
      { "tddOption", "rrc.tddOption",
        FT_UINT32, BASE_DEC, VALS(rrc_T_tddOption_13_vals), 0,
        "T_tddOption_13", HFILL }},
    { &hf_rrc_tdd384_30,
      { "tdd384", "rrc.tdd384",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_tdd384_28", HFILL }},
    { &hf_rrc_tdd768_19,
      { "tdd768", "rrc.tdd768",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_tdd768_17", HFILL }},
    { &hf_rrc_pdsch_TimeslotsCodes_02,
      { "pdsch-TimeslotsCodes", "rrc.pdsch_TimeslotsCodes",
        FT_NONE, BASE_NONE, NULL, 0,
        "DownlinkTimeslotsCodes_VHCR", HFILL }},
    { &hf_rrc_tdd128_46,
      { "tdd128", "rrc.tdd128",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_tdd128_45", HFILL }},
    { &hf_rrc_dsch_RadioLinkIdentifier,
      { "dsch-RadioLinkIdentifier", "rrc.dsch_RadioLinkIdentifier",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_rl_IdentifierList,
      { "rl-IdentifierList", "rrc.rl_IdentifierList",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_dsch_TFCS,
      { "dsch-TFCS", "rrc.dsch_TFCS",
        FT_UINT32, BASE_DEC, VALS(rrc_TFCS_vals), 0,
        "TFCS", HFILL }},
    { &hf_rrc_dsch_TransportChannelsInfo,
      { "dsch-TransportChannelsInfo", "rrc.dsch_TransportChannelsInfo",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_pdsch_Info_03,
      { "pdsch-Info", "rrc.pdsch_Info",
        FT_NONE, BASE_NONE, NULL, 0,
        "PDSCH_Info_LCR_r4", HFILL }},
    { &hf_rrc_PDSCH_SysInfoList_item,
      { "PDSCH-SysInfo", "rrc.PDSCH_SysInfo",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_PDSCH_SysInfoList_VHCR_r7_item,
      { "PDSCH-SysInfo-VHCR-r7", "rrc.PDSCH_SysInfo_VHCR_r7",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_PDSCH_SysInfoList_HCR_r5_item,
      { "PDSCH-SysInfo-HCR-r5", "rrc.PDSCH_SysInfo_HCR_r5",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_PDSCH_SysInfoList_LCR_r4_item,
      { "PDSCH-SysInfo-LCR-r4", "rrc.PDSCH_SysInfo_LCR_r4",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_PDSCH_SysInfoList_SFN_item,
      { "PDSCH-SysInfoList-SFN item", "rrc.PDSCH_SysInfoList_SFN_item",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_pdsch_SysInfo,
      { "pdsch-SysInfo", "rrc.pdsch_SysInfo",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_sfn_TimeInfo,
      { "sfn-TimeInfo", "rrc.sfn_TimeInfo",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_PDSCH_SysInfoList_SFN_HCR_r5_item,
      { "PDSCH-SysInfoList-SFN-HCR-r5 item", "rrc.PDSCH_SysInfoList_SFN_HCR_r5_item",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_pdsch_SysInfo_01,
      { "pdsch-SysInfo", "rrc.pdsch_SysInfo",
        FT_NONE, BASE_NONE, NULL, 0,
        "PDSCH_SysInfo_HCR_r5", HFILL }},
    { &hf_rrc_PDSCH_SysInfoList_SFN_LCR_r4_item,
      { "PDSCH-SysInfoList-SFN-LCR-r4 item", "rrc.PDSCH_SysInfoList_SFN_LCR_r4_item",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_pdsch_SysInfo_02,
      { "pdsch-SysInfo", "rrc.pdsch_SysInfo",
        FT_NONE, BASE_NONE, NULL, 0,
        "PDSCH_SysInfo_LCR_r4", HFILL }},
    { &hf_rrc_PersistenceScalingFactorList_item,
      { "PersistenceScalingFactor", "rrc.PersistenceScalingFactor",
        FT_UINT32, BASE_DEC, VALS(rrc_PersistenceScalingFactor_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_PichChannelisationCodeList_LCR_r4_item,
      { "DL-TS-ChannelisationCode", "rrc.DL_TS_ChannelisationCode",
        FT_UINT32, BASE_DEC, VALS(rrc_DL_TS_ChannelisationCode_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_hsdpa_AssociatedPichInfo,
      { "hsdpa-AssociatedPichInfo", "rrc.hsdpa_AssociatedPichInfo",
        FT_UINT32, BASE_DEC, VALS(rrc_PICH_Info_vals), 0,
        "PICH_Info", HFILL }},
    { &hf_rrc_hs_pdschChannelisationCode,
      { "hs-pdschChannelisationCode", "rrc.hs_pdschChannelisationCode",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_1_15", HFILL }},
    { &hf_rrc_implicit_03,
      { "implicit", "rrc.implicit",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_implicit_02", HFILL }},
    { &hf_rrc_occurrenceSequenceNumberOfPICH,
      { "occurrenceSequenceNumberOfPICH", "rrc.occurrenceSequenceNumberOfPICH",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_explicit_05,
      { "explicit", "rrc.explicit",
        FT_NONE, BASE_NONE, NULL, 0,
        "PICH_Info_LCR_r4", HFILL }},
    { &hf_rrc_fdd_104,
      { "fdd", "rrc.fdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_fdd_102", HFILL }},
    { &hf_rrc_pi_CountPerFrame,
      { "pi-CountPerFrame", "rrc.pi_CountPerFrame",
        FT_UINT32, BASE_DEC, VALS(rrc_PI_CountPerFrame_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_tdd_61,
      { "tdd", "rrc.tdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_tdd_57", HFILL }},
    { &hf_rrc_channelisationCode_07,
      { "channelisationCode", "rrc.channelisationCode",
        FT_UINT32, BASE_DEC, VALS(rrc_TDD_PICH_CCode_vals), 0,
        "TDD_PICH_CCode", HFILL }},
    { &hf_rrc_repetitionPeriodLengthOffset,
      { "repetitionPeriodLengthOffset", "rrc.repetitionPeriodLengthOffset",
        FT_UINT32, BASE_DEC, VALS(rrc_RepPerLengthOffset_PICH_vals), 0,
        "RepPerLengthOffset_PICH", HFILL }},
    { &hf_rrc_pagingIndicatorLength,
      { "pagingIndicatorLength", "rrc.pagingIndicatorLength",
        FT_UINT32, BASE_DEC, VALS(rrc_PagingIndicatorLength_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_n_GAP,
      { "n-GAP", "rrc.n_GAP",
        FT_UINT32, BASE_DEC, VALS(rrc_N_GAP_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_n_PCH,
      { "n-PCH", "rrc.n_PCH",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_channelisationCode_08,
      { "channelisationCode", "rrc.channelisationCode",
        FT_UINT32, BASE_DEC, VALS(rrc_T_channelisationCode_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_tdd384_31,
      { "tdd384", "rrc.tdd384",
        FT_UINT32, BASE_DEC, VALS(rrc_TDD_PICH_CCode_vals), 0,
        "TDD_PICH_CCode", HFILL }},
    { &hf_rrc_tdd768_20,
      { "tdd768", "rrc.tdd768",
        FT_UINT32, BASE_DEC|BASE_EXT_STRING, &rrc_TDD768_PICH_CCode_vals_ext, 0,
        "TDD768_PICH_CCode", HFILL }},
    { &hf_rrc_pichChannelisationCodeList_LCR_r4,
      { "pichChannelisationCodeList-LCR-r4", "rrc.pichChannelisationCodeList_LCR_r4",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_plcchSequenceNumber,
      { "plcchSequenceNumber", "rrc.plcchSequenceNumber",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_1_14", HFILL }},
    { &hf_rrc_tpcCommandTargetRate,
      { "tpcCommandTargetRate", "rrc.tpcCommandTargetRate",
        FT_UINT32, BASE_DEC, NULL, 0,
        "TPC_CommandTargetRate", HFILL }},
    { &hf_rrc_algorithm1,
      { "algorithm1", "rrc.algorithm1",
        FT_UINT32, BASE_DEC, NULL, 0,
        "TPC_StepSizeFDD", HFILL }},
    { &hf_rrc_algorithm2,
      { "algorithm2", "rrc.algorithm2",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_PRACH_ChanCodes_LCR_r4_item,
      { "TDD-PRACH-CCode-LCR-r4", "rrc.TDD_PRACH_CCode_LCR_r4",
        FT_UINT32, BASE_DEC|BASE_EXT_STRING, &rrc_TDD_PRACH_CCode_LCR_r4_vals_ext, 0,
        NULL, HFILL }},
    { &hf_rrc_PRACH_ChanCodes_List_LCR_item,
      { "TDD-PRACH-CCode-LCR-r4", "rrc.TDD_PRACH_CCode_LCR_r4",
        FT_UINT32, BASE_DEC|BASE_EXT_STRING, &rrc_TDD_PRACH_CCode_LCR_r4_vals_ext, 0,
        NULL, HFILL }},
    { &hf_rrc_timeslot_02,
      { "timeslot", "rrc.timeslot",
        FT_UINT32, BASE_DEC, NULL, 0,
        "TimeslotNumber_PRACH_LCR_r4", HFILL }},
    { &hf_rrc_prach_ChanCodes_LCR,
      { "prach-ChanCodes-LCR", "rrc.prach_ChanCodes_LCR",
        FT_UINT32, BASE_DEC, NULL, 0,
        "PRACH_ChanCodes_LCR_r4", HFILL }},
    { &hf_rrc_fpach_Info,
      { "fpach-Info", "rrc.fpach_Info",
        FT_NONE, BASE_NONE, NULL, 0,
        "FPACH_Info_r4", HFILL }},
    { &hf_rrc_prach_ChanCodes_list_LCR,
      { "prach-ChanCodes-list-LCR", "rrc.prach_ChanCodes_list_LCR",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_PRACH_Information_LCR_List_item,
      { "PRACH-Information-LCR", "rrc.PRACH_Information_LCR",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_fdd_105,
      { "fdd", "rrc.fdd",
        FT_UINT32, BASE_DEC, NULL, 0,
        "SEQUENCE_SIZE_1_maxASC_OF_ASCSetting_FDD", HFILL }},
    { &hf_rrc_fdd_item,
      { "ASCSetting-FDD", "rrc.ASCSetting_FDD",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_tdd_62,
      { "tdd", "rrc.tdd",
        FT_UINT32, BASE_DEC, NULL, 0,
        "SEQUENCE_SIZE_1_maxASC_OF_ASCSetting_TDD", HFILL }},
    { &hf_rrc_tdd_item,
      { "ASCSetting-TDD", "rrc.ASCSetting_TDD",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_tdd_63,
      { "tdd", "rrc.tdd",
        FT_UINT32, BASE_DEC, NULL, 0,
        "SEQUENCE_SIZE_1_maxASC_OF_ASCSetting_TDD_r7", HFILL }},
    { &hf_rrc_tdd_item_01,
      { "ASCSetting-TDD-r7", "rrc.ASCSetting_TDD_r7",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_PRACH_Partitioning_LCR_r4_item,
      { "ASCSetting-TDD-LCR-r4", "rrc.ASCSetting_TDD_LCR_r4",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_powerRampStep,
      { "powerRampStep", "rrc.powerRampStep",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_preambleRetransMax,
      { "preambleRetransMax", "rrc.preambleRetransMax",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_availableSignatures,
      { "availableSignatures", "rrc.availableSignatures",
        FT_BYTES, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_e_ai_Indication,
      { "e-ai-Indication", "rrc.e_ai_Indication",
        FT_BOOLEAN, BASE_NONE, NULL, 0,
        "BOOLEAN", HFILL }},
    { &hf_rrc_preambleScramblingCodeWordNumber,
      { "preambleScramblingCodeWordNumber", "rrc.preambleScramblingCodeWordNumber",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_availableSubChannelNumbers,
      { "availableSubChannelNumbers", "rrc.availableSubChannelNumbers",
        FT_BYTES, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_prach_Partitioning,
      { "prach-Partitioning", "rrc.prach_Partitioning",
        FT_UINT32, BASE_DEC, VALS(rrc_PRACH_Partitioning_r7_vals), 0,
        "PRACH_Partitioning_r7", HFILL }},
    { &hf_rrc_persistenceScalingFactorList,
      { "persistenceScalingFactorList", "rrc.persistenceScalingFactorList",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_ac_To_ASC_MappingTable,
      { "ac-To-ASC-MappingTable", "rrc.ac_To_ASC_MappingTable",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_primaryCPICH_TX_Power,
      { "primaryCPICH-TX-Power", "rrc.primaryCPICH_TX_Power",
        FT_INT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_constantValue,
      { "constantValue", "rrc.constantValue",
        FT_INT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_prach_PowerOffset,
      { "prach-PowerOffset", "rrc.prach_PowerOffset",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_rach_TransmissionParameters,
      { "rach-TransmissionParameters", "rrc.rach_TransmissionParameters",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_aich_Info,
      { "aich-Info", "rrc.aich_Info",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_powerOffsetPp_e,
      { "powerOffsetPp-e", "rrc.powerOffsetPp_e",
        FT_INT32, BASE_DEC, NULL, 0,
        "INTEGER_M5_10", HFILL }},
    { &hf_rrc_aich_Info_01,
      { "aich-Info", "rrc.aich_Info",
        FT_NONE, BASE_NONE, NULL, 0,
        "AICH_Info_Compressed", HFILL }},
    { &hf_rrc_prach_PreambleForEnhancedUplinkList,
      { "prach-PreambleForEnhancedUplinkList", "rrc.prach_PreambleForEnhancedUplinkList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "SEQUENCE_SIZE_1_maxPRACH_EUL_OF_PRACH_PreambleForEnhancedUplinkExtWithWeight", HFILL }},
    { &hf_rrc_prach_PreambleForEnhancedUplinkList_item,
      { "PRACH-PreambleForEnhancedUplinkExtWithWeight", "rrc.PRACH_PreambleForEnhancedUplinkExtWithWeight",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_prach_PreambleForEnhancedUplinkExt,
      { "prach-PreambleForEnhancedUplinkExt", "rrc.prach_PreambleForEnhancedUplinkExt",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_weight,
      { "weight", "rrc.weight",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_5", HFILL }},
    { &hf_rrc_modeSpecificInfo_72,
      { "modeSpecificInfo", "rrc.modeSpecificInfo",
        FT_UINT32, BASE_DEC, VALS(rrc_T_modeSpecificInfo_72_vals), 0,
        "T_modeSpecificInfo_72", HFILL }},
    { &hf_rrc_fdd_106,
      { "fdd", "rrc.fdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_fdd_103", HFILL }},
    { &hf_rrc_availableSF,
      { "availableSF", "rrc.availableSF",
        FT_UINT32, BASE_DEC, VALS(rrc_SF_PRACH_vals), 0,
        "SF_PRACH", HFILL }},
    { &hf_rrc_tdd_64,
      { "tdd", "rrc.tdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_tdd_58", HFILL }},
    { &hf_rrc_channelisationCodeList,
      { "channelisationCodeList", "rrc.channelisationCodeList",
        FT_UINT32, BASE_DEC, VALS(rrc_TDD_PRACH_CCodeList_vals), 0,
        "TDD_PRACH_CCodeList", HFILL }},
    { &hf_rrc_prach_Midamble,
      { "prach-Midamble", "rrc.prach_Midamble",
        FT_UINT32, BASE_DEC, VALS(rrc_PRACH_Midamble_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_channelisationCodeList_01,
      { "channelisationCodeList", "rrc.channelisationCodeList",
        FT_UINT32, BASE_DEC, VALS(rrc_TDD768_PRACH_CCodeList_vals), 0,
        "TDD768_PRACH_CCodeList", HFILL }},
    { &hf_rrc_e_RUCCH_Sync_UL_Codes_Bitmap,
      { "e-RUCCH-Sync-UL-Codes-Bitmap", "rrc.e_RUCCH_Sync_UL_Codes_Bitmap",
        FT_BYTES, BASE_NONE, NULL, 0,
        "Sync_UL_Codes_Bitmap", HFILL }},
    { &hf_rrc_sync_UL_Info,
      { "sync-UL-Info", "rrc.sync_UL_Info",
        FT_NONE, BASE_NONE, NULL, 0,
        "SYNC_UL_Info_r4", HFILL }},
    { &hf_rrc_prach_DefinitionList,
      { "prach-DefinitionList", "rrc.prach_DefinitionList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "SEQUENCE_SIZE_1_maxPRACH_FPACH_OF_PRACH_Definition_LCR_r4", HFILL }},
    { &hf_rrc_prach_DefinitionList_item,
      { "PRACH-Definition-LCR-r4", "rrc.PRACH_Definition_LCR_r4",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_prach_RACH_Info,
      { "prach-RACH-Info", "rrc.prach_RACH_Info",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_rach_TransportFormatSet,
      { "rach-TransportFormatSet", "rrc.rach_TransportFormatSet",
        FT_UINT32, BASE_DEC, VALS(rrc_TransportFormatSet_vals), 0,
        "TransportFormatSet", HFILL }},
    { &hf_rrc_rach_TFCS,
      { "rach-TFCS", "rrc.rach_TFCS",
        FT_UINT32, BASE_DEC, VALS(rrc_TFCS_vals), 0,
        "TFCS", HFILL }},
    { &hf_rrc_prach_Partitioning_01,
      { "prach-Partitioning", "rrc.prach_Partitioning",
        FT_UINT32, BASE_DEC, VALS(rrc_PRACH_Partitioning_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_modeSpecificInfo_73,
      { "modeSpecificInfo", "rrc.modeSpecificInfo",
        FT_UINT32, BASE_DEC, VALS(rrc_T_modeSpecificInfo_73_vals), 0,
        "T_modeSpecificInfo_73", HFILL }},
    { &hf_rrc_fdd_107,
      { "fdd", "rrc.fdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_fdd_104", HFILL }},
    { &hf_rrc_prach_RACH_Info_01,
      { "prach-RACH-Info", "rrc.prach_RACH_Info",
        FT_NONE, BASE_NONE, NULL, 0,
        "PRACH_RACH_Info_VHCR_r7", HFILL }},
    { &hf_rrc_prach_RACH_Info_LCR,
      { "prach-RACH-Info-LCR", "rrc.prach_RACH_Info_LCR",
        FT_NONE, BASE_NONE, NULL, 0,
        "PRACH_RACH_Info_LCR_r4", HFILL }},
    { &hf_rrc_rach_TransportFormatSet_LCR,
      { "rach-TransportFormatSet-LCR", "rrc.rach_TransportFormatSet_LCR",
        FT_UINT32, BASE_DEC, VALS(rrc_TransportFormatSet_LCR_vals), 0,
        "TransportFormatSet_LCR", HFILL }},
    { &hf_rrc_prach_Partitioning_LCR,
      { "prach-Partitioning-LCR", "rrc.prach_Partitioning_LCR",
        FT_UINT32, BASE_DEC, NULL, 0,
        "PRACH_Partitioning_LCR_r4", HFILL }},
    { &hf_rrc_prach_RACH_Info_LCR_01,
      { "prach-RACH-Info-LCR", "rrc.prach_RACH_Info_LCR",
        FT_NONE, BASE_NONE, NULL, 0,
        "PRACH_RACH_Info_LCR_v770ext", HFILL }},
    { &hf_rrc_PRACH_SystemInformationList_item,
      { "PRACH-SystemInformation", "rrc.PRACH_SystemInformation",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_PRACH_SystemInformationList_VHCR_r7_item,
      { "PRACH-SystemInformation-VHCR-r7", "rrc.PRACH_SystemInformation_VHCR_r7",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_PRACH_SystemInformationList_LCR_r4_item,
      { "PRACH-SystemInformation-LCR-r4", "rrc.PRACH_SystemInformation_LCR_r4",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_PRACH_SystemInformationList_LCR_v770ext_item,
      { "PRACH-SystemInformation-LCR-v770ext", "rrc.PRACH_SystemInformation_LCR_v770ext",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_ul_DPCH_InfoPredef,
      { "ul-DPCH-InfoPredef", "rrc.ul_DPCH_InfoPredef",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_dl_CommonInformationPredef,
      { "dl-CommonInformationPredef", "rrc.dl_CommonInformationPredef",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_ul_DPCH_InfoPredef_01,
      { "ul-DPCH-InfoPredef", "rrc.ul_DPCH_InfoPredef",
        FT_NONE, BASE_NONE, NULL, 0,
        "UL_DPCH_InfoPredef_v770ext", HFILL }},
    { &hf_rrc_fdd_108,
      { "fdd", "rrc.fdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_fdd_105", HFILL }},
    { &hf_rrc_tx_DiversityIndicator,
      { "tx-DiversityIndicator", "rrc.tx_DiversityIndicator",
        FT_BOOLEAN, BASE_NONE, NULL, 0,
        "BOOLEAN", HFILL }},
    { &hf_rrc_tdd_65,
      { "tdd", "rrc.tdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_tdd_59", HFILL }},
    { &hf_rrc_syncCase,
      { "syncCase", "rrc.syncCase",
        FT_UINT32, BASE_DEC, VALS(rrc_T_syncCase_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_syncCase1,
      { "syncCase1", "rrc.syncCase1",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_syncCase2,
      { "syncCase2", "rrc.syncCase2",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_timeslotSync2,
      { "timeslotSync2", "rrc.timeslotSync2",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_sctd_Indicator,
      { "sctd-Indicator", "rrc.sctd_Indicator",
        FT_BOOLEAN, BASE_NONE, NULL, 0,
        "BOOLEAN", HFILL }},
    { &hf_rrc_fdd_109,
      { "fdd", "rrc.fdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_fdd_106", HFILL }},
    { &hf_rrc_tdd_66,
      { "tdd", "rrc.tdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_tdd_60", HFILL }},
    { &hf_rrc_tddOption_14,
      { "tddOption", "rrc.tddOption",
        FT_UINT32, BASE_DEC, VALS(rrc_T_tddOption_14_vals), 0,
        "T_tddOption_14", HFILL }},
    { &hf_rrc_tdd384_tdd768_05,
      { "tdd384-tdd768", "rrc.tdd384_tdd768",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_tdd384_tdd768_04", HFILL }},
    { &hf_rrc_syncCase_01,
      { "syncCase", "rrc.syncCase",
        FT_UINT32, BASE_DEC, VALS(rrc_T_syncCase_01_vals), 0,
        "T_syncCase_01", HFILL }},
    { &hf_rrc_syncCase1_01,
      { "syncCase1", "rrc.syncCase1",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_syncCase1_01", HFILL }},
    { &hf_rrc_syncCase2_01,
      { "syncCase2", "rrc.syncCase2",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_syncCase2_01", HFILL }},
    { &hf_rrc_tdd128_47,
      { "tdd128", "rrc.tdd128",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_tdd128_46", HFILL }},
    { &hf_rrc_syncCase_02,
      { "syncCase", "rrc.syncCase",
        FT_UINT32, BASE_DEC, VALS(rrc_T_syncCase_02_vals), 0,
        "T_syncCase_02", HFILL }},
    { &hf_rrc_syncCase1_02,
      { "syncCase1", "rrc.syncCase1",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_syncCase1_02", HFILL }},
    { &hf_rrc_syncCase2_02,
      { "syncCase2", "rrc.syncCase2",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_syncCase2_02", HFILL }},
    { &hf_rrc_primaryScramblingCode,
      { "primaryScramblingCode", "rrc.primaryScramblingCode",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_pusch_Allocation,
      { "pusch-Allocation", "rrc.pusch_Allocation",
        FT_UINT32, BASE_DEC, VALS(rrc_T_pusch_Allocation_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_pusch_AllocationPending,
      { "pusch-AllocationPending", "rrc.pusch_AllocationPending",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_pusch_AllocationAssignment,
      { "pusch-AllocationAssignment", "rrc.pusch_AllocationAssignment",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_pusch_AllocationPeriodInfo,
      { "pusch-AllocationPeriodInfo", "rrc.pusch_AllocationPeriodInfo",
        FT_NONE, BASE_NONE, NULL, 0,
        "AllocationPeriodInfo", HFILL }},
    { &hf_rrc_pusch_PowerControlInfo,
      { "pusch-PowerControlInfo", "rrc.pusch_PowerControlInfo",
        FT_UINT32, BASE_DEC, NULL, 0,
        "UL_TargetSIR", HFILL }},
    { &hf_rrc_configuration_03,
      { "configuration", "rrc.configuration",
        FT_UINT32, BASE_DEC, VALS(rrc_T_configuration_03_vals), 0,
        "T_configuration_03", HFILL }},
    { &hf_rrc_old_Configuration_03,
      { "old-Configuration", "rrc.old_Configuration",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_old_Configuration_03", HFILL }},
    { &hf_rrc_pusch_Identity,
      { "pusch-Identity", "rrc.pusch_Identity",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_new_Configuration_03,
      { "new-Configuration", "rrc.new_Configuration",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_new_Configuration_03", HFILL }},
    { &hf_rrc_pusch_Info,
      { "pusch-Info", "rrc.pusch_Info",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_pusch_Allocation_01,
      { "pusch-Allocation", "rrc.pusch_Allocation",
        FT_UINT32, BASE_DEC, VALS(rrc_T_pusch_Allocation_01_vals), 0,
        "T_pusch_Allocation_01", HFILL }},
    { &hf_rrc_pusch_AllocationAssignment_01,
      { "pusch-AllocationAssignment", "rrc.pusch_AllocationAssignment",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_pusch_AllocationAssignment_01", HFILL }},
    { &hf_rrc_pusch_PowerControlInfo_01,
      { "pusch-PowerControlInfo", "rrc.pusch_PowerControlInfo",
        FT_NONE, BASE_NONE, NULL, 0,
        "PUSCH_PowerControlInfo_r4", HFILL }},
    { &hf_rrc_configuration_04,
      { "configuration", "rrc.configuration",
        FT_UINT32, BASE_DEC, VALS(rrc_T_configuration_04_vals), 0,
        "T_configuration_04", HFILL }},
    { &hf_rrc_old_Configuration_04,
      { "old-Configuration", "rrc.old_Configuration",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_old_Configuration_04", HFILL }},
    { &hf_rrc_new_Configuration_04,
      { "new-Configuration", "rrc.new_Configuration",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_new_Configuration_04", HFILL }},
    { &hf_rrc_pusch_Info_01,
      { "pusch-Info", "rrc.pusch_Info",
        FT_NONE, BASE_NONE, NULL, 0,
        "PUSCH_Info_r4", HFILL }},
    { &hf_rrc_pusch_Allocation_02,
      { "pusch-Allocation", "rrc.pusch_Allocation",
        FT_UINT32, BASE_DEC, VALS(rrc_T_pusch_Allocation_02_vals), 0,
        "T_pusch_Allocation_02", HFILL }},
    { &hf_rrc_pusch_AllocationAssignment_02,
      { "pusch-AllocationAssignment", "rrc.pusch_AllocationAssignment",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_pusch_AllocationAssignment_02", HFILL }},
    { &hf_rrc_pusch_PowerControlInfo_02,
      { "pusch-PowerControlInfo", "rrc.pusch_PowerControlInfo",
        FT_NONE, BASE_NONE, NULL, 0,
        "PUSCH_PowerControlInfo_r7", HFILL }},
    { &hf_rrc_configuration_05,
      { "configuration", "rrc.configuration",
        FT_UINT32, BASE_DEC, VALS(rrc_T_configuration_05_vals), 0,
        "T_configuration_05", HFILL }},
    { &hf_rrc_old_Configuration_05,
      { "old-Configuration", "rrc.old_Configuration",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_old_Configuration_05", HFILL }},
    { &hf_rrc_new_Configuration_05,
      { "new-Configuration", "rrc.new_Configuration",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_new_Configuration_05", HFILL }},
    { &hf_rrc_pusch_Info_02,
      { "pusch-Info", "rrc.pusch_Info",
        FT_NONE, BASE_NONE, NULL, 0,
        "PUSCH_Info_VHCR", HFILL }},
    { &hf_rrc_pusch_TimeslotsCodes,
      { "pusch-TimeslotsCodes", "rrc.pusch_TimeslotsCodes",
        FT_NONE, BASE_NONE, NULL, 0,
        "UplinkTimeslotsCodes", HFILL }},
    { &hf_rrc_tddOption_15,
      { "tddOption", "rrc.tddOption",
        FT_UINT32, BASE_DEC, VALS(rrc_T_tddOption_15_vals), 0,
        "T_tddOption_15", HFILL }},
    { &hf_rrc_tdd384_32,
      { "tdd384", "rrc.tdd384",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_tdd384_29", HFILL }},
    { &hf_rrc_tdd128_48,
      { "tdd128", "rrc.tdd128",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_tdd128_47", HFILL }},
    { &hf_rrc_pusch_TimeslotsCodes_01,
      { "pusch-TimeslotsCodes", "rrc.pusch_TimeslotsCodes",
        FT_NONE, BASE_NONE, NULL, 0,
        "UplinkTimeslotsCodes_LCR_r4", HFILL }},
    { &hf_rrc_pusch_TimeslotsCodes_VHCR,
      { "pusch-TimeslotsCodes-VHCR", "rrc.pusch_TimeslotsCodes_VHCR",
        FT_NONE, BASE_NONE, NULL, 0,
        "UplinkTimeslotsCodes_VHCR", HFILL }},
    { &hf_rrc_ul_TargetSIR,
      { "ul-TargetSIR", "rrc.ul_TargetSIR",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_tddOption_16,
      { "tddOption", "rrc.tddOption",
        FT_UINT32, BASE_DEC, VALS(rrc_T_tddOption_16_vals), 0,
        "T_tddOption_16", HFILL }},
    { &hf_rrc_tdd128_49,
      { "tdd128", "rrc.tdd128",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_tdd128_48", HFILL }},
    { &hf_rrc_tddOption_17,
      { "tddOption", "rrc.tddOption",
        FT_UINT32, BASE_DEC, VALS(rrc_T_tddOption_17_vals), 0,
        "T_tddOption_17", HFILL }},
    { &hf_rrc_tdd128_50,
      { "tdd128", "rrc.tdd128",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_tdd128_49", HFILL }},
    { &hf_rrc_usch_TFCS,
      { "usch-TFCS", "rrc.usch_TFCS",
        FT_UINT32, BASE_DEC, VALS(rrc_TFCS_vals), 0,
        "TFCS", HFILL }},
    { &hf_rrc_pusch_Info_VHCR,
      { "pusch-Info-VHCR", "rrc.pusch_Info_VHCR",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_usch_TransportChannelsInfo,
      { "usch-TransportChannelsInfo", "rrc.usch_TransportChannelsInfo",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_pusch_Info_03,
      { "pusch-Info", "rrc.pusch_Info",
        FT_NONE, BASE_NONE, NULL, 0,
        "PUSCH_Info_LCR_r4", HFILL }},
    { &hf_rrc_PUSCH_SysInfoList_item,
      { "PUSCH-SysInfo", "rrc.PUSCH_SysInfo",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_PUSCH_SysInfoList_HCR_r5_item,
      { "PUSCH-SysInfo-HCR-r5", "rrc.PUSCH_SysInfo_HCR_r5",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_PUSCH_SysInfoList_LCR_r4_item,
      { "PUSCH-SysInfo-LCR-r4", "rrc.PUSCH_SysInfo_LCR_r4",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_PUSCH_SysInfoList_SFN_item,
      { "PUSCH-SysInfoList-SFN item", "rrc.PUSCH_SysInfoList_SFN_item",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_pusch_SysInfo,
      { "pusch-SysInfo", "rrc.pusch_SysInfo",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_PUSCH_SysInfoList_SFN_HCR_r5_item,
      { "PUSCH-SysInfoList-SFN-HCR-r5 item", "rrc.PUSCH_SysInfoList_SFN_HCR_r5_item",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_pusch_SysInfo_01,
      { "pusch-SysInfo", "rrc.pusch_SysInfo",
        FT_NONE, BASE_NONE, NULL, 0,
        "PUSCH_SysInfo_HCR_r5", HFILL }},
    { &hf_rrc_PUSCH_SysInfoList_SFN_LCR_r4_item,
      { "PUSCH-SysInfoList-SFN-LCR-r4 item", "rrc.PUSCH_SysInfoList_SFN_LCR_r4_item",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_pusch_SysInfo_02,
      { "pusch-SysInfo", "rrc.pusch_SysInfo",
        FT_NONE, BASE_NONE, NULL, 0,
        "PUSCH_SysInfo_LCR_r4", HFILL }},
    { &hf_rrc_PUSCH_SysInfoList_SFN_VHCR_item,
      { "PUSCH-SysInfoList-SFN-VHCR item", "rrc.PUSCH_SysInfoList_SFN_VHCR_item",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_pusch_SysInfo_VHCR,
      { "pusch-SysInfo-VHCR", "rrc.pusch_SysInfo_VHCR",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_mmax,
      { "mmax", "rrc.mmax",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_1_32", HFILL }},
    { &hf_rrc_nb01Min,
      { "nb01Min", "rrc.nb01Min",
        FT_UINT32, BASE_DEC, NULL, 0,
        "NB01", HFILL }},
    { &hf_rrc_nb01Max,
      { "nb01Max", "rrc.nb01Max",
        FT_UINT32, BASE_DEC, NULL, 0,
        "NB01", HFILL }},
    { &hf_rrc_reference_Code_Rate,
      { "reference-Code-Rate", "rrc.reference_Code_Rate",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_10", HFILL }},
    { &hf_rrc_reference_Beta,
      { "reference-Beta", "rrc.reference_Beta",
        FT_INT32, BASE_DEC, NULL, 0,
        "INTEGER_M15_16", HFILL }},
    { &hf_rrc_repetitionPeriod1,
      { "repetitionPeriod1", "rrc.repetitionPeriod1",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_repetitionPeriod2,
      { "repetitionPeriod2", "rrc.repetitionPeriod2",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_1_1", HFILL }},
    { &hf_rrc_repetitionPeriod4,
      { "repetitionPeriod4", "rrc.repetitionPeriod4",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_1_3", HFILL }},
    { &hf_rrc_repetitionPeriod8,
      { "repetitionPeriod8", "rrc.repetitionPeriod8",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_1_7", HFILL }},
    { &hf_rrc_repetitionPeriod16,
      { "repetitionPeriod16", "rrc.repetitionPeriod16",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_1_15", HFILL }},
    { &hf_rrc_repetitionPeriod32,
      { "repetitionPeriod32", "rrc.repetitionPeriod32",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_1_31", HFILL }},
    { &hf_rrc_repetitionPeriod64,
      { "repetitionPeriod64", "rrc.repetitionPeriod64",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_1_63", HFILL }},
    { &hf_rrc_repetitionPeriod2_01,
      { "repetitionPeriod2", "rrc.repetitionPeriod2",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_length,
      { "length", "rrc.length",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_offset,
      { "offset", "rrc.offset",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_1", HFILL }},
    { &hf_rrc_repetitionPeriod4_01,
      { "repetitionPeriod4", "rrc.repetitionPeriod4",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_length_01,
      { "length", "rrc.length",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_1_3", HFILL }},
    { &hf_rrc_offset_01,
      { "offset", "rrc.offset",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_3", HFILL }},
    { &hf_rrc_repetitionPeriod8_01,
      { "repetitionPeriod8", "rrc.repetitionPeriod8",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_length_02,
      { "length", "rrc.length",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_1_7", HFILL }},
    { &hf_rrc_offset_02,
      { "offset", "rrc.offset",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_7", HFILL }},
    { &hf_rrc_repetitionPeriod16_01,
      { "repetitionPeriod16", "rrc.repetitionPeriod16",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_length_03,
      { "length", "rrc.length",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_1_15", HFILL }},
    { &hf_rrc_offset_03,
      { "offset", "rrc.offset",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_15", HFILL }},
    { &hf_rrc_repetitionPeriod32_01,
      { "repetitionPeriod32", "rrc.repetitionPeriod32",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_length_04,
      { "length", "rrc.length",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_1_31", HFILL }},
    { &hf_rrc_offset_04,
      { "offset", "rrc.offset",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_31", HFILL }},
    { &hf_rrc_repetitionPeriod64_01,
      { "repetitionPeriod64", "rrc.repetitionPeriod64",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_length_05,
      { "length", "rrc.length",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_1_63", HFILL }},
    { &hf_rrc_offset_05,
      { "offset", "rrc.offset",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_63", HFILL }},
    { &hf_rrc_tfci_Field2,
      { "tfci-Field2", "rrc.tfci_Field2",
        FT_UINT32, BASE_DEC, NULL, 0,
        "MaxTFCI_Field2Value", HFILL }},
    { &hf_rrc_ReplacedPDSCH_CodeInfoList_item,
      { "ReplacedPDSCH-CodeInfo", "rrc.ReplacedPDSCH_CodeInfo",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_rpp4_2,
      { "rpp4-2", "rrc.rpp4_2",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_3", HFILL }},
    { &hf_rrc_rpp8_2,
      { "rpp8-2", "rrc.rpp8_2",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_7", HFILL }},
    { &hf_rrc_rpp8_4,
      { "rpp8-4", "rrc.rpp8_4",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_7", HFILL }},
    { &hf_rrc_rpp16_2,
      { "rpp16-2", "rrc.rpp16_2",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_15", HFILL }},
    { &hf_rrc_rpp16_4,
      { "rpp16-4", "rrc.rpp16_4",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_15", HFILL }},
    { &hf_rrc_rpp32_2,
      { "rpp32-2", "rrc.rpp32_2",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_31", HFILL }},
    { &hf_rrc_rpp32_4,
      { "rpp32-4", "rrc.rpp32_4",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_31", HFILL }},
    { &hf_rrc_rpp64_2,
      { "rpp64-2", "rrc.rpp64_2",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_63", HFILL }},
    { &hf_rrc_rpp64_4,
      { "rpp64-4", "rrc.rpp64_4",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_63", HFILL }},
    { &hf_rrc_dl_restrictedTrCh_Type,
      { "dl-restrictedTrCh-Type", "rrc.dl_restrictedTrCh_Type",
        FT_UINT32, BASE_DEC, VALS(rrc_DL_TrCH_Type_vals), 0,
        "DL_TrCH_Type", HFILL }},
    { &hf_rrc_restrictedDL_TrCH_Identity,
      { "restrictedDL-TrCH-Identity", "rrc.restrictedDL_TrCH_Identity",
        FT_UINT32, BASE_DEC, NULL, 0,
        "TransportChannelIdentity", HFILL }},
    { &hf_rrc_allowedTFIList,
      { "allowedTFIList", "rrc.allowedTFIList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "AllowedTFI_List", HFILL }},
    { &hf_rrc_RestrictedTrCH_InfoList_item,
      { "RestrictedTrCH", "rrc.RestrictedTrCH",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_dummy2_08,
      { "dummy2", "rrc.dummy2",
        FT_NONE, BASE_NONE, NULL, 0,
        "SCCPCH_InfoForFACH", HFILL }},
    { &hf_rrc_cell_Id,
      { "cell-Id", "rrc.cell_Id",
        FT_BYTES, BASE_NONE, NULL, 0,
        "CellIdentity", HFILL }},
    { &hf_rrc_dl_dpchInfo_04,
      { "dl-dpchInfo", "rrc.dl_dpchInfo",
        FT_UINT32, BASE_DEC, VALS(rrc_T_dl_dpchInfo_04_vals), 0,
        "T_dl_dpchInfo_04", HFILL }},
    { &hf_rrc_dl_dpchInfo_05,
      { "dl-dpchInfo", "rrc.dl_dpchInfo",
        FT_UINT32, BASE_DEC, VALS(rrc_T_dl_dpchInfo_05_vals), 0,
        "T_dl_dpchInfo_05", HFILL }},
    { &hf_rrc_dl_dpchInfo_06,
      { "dl-dpchInfo", "rrc.dl_dpchInfo",
        FT_UINT32, BASE_DEC, VALS(rrc_T_dl_dpchInfo_06_vals), 0,
        "T_dl_dpchInfo_06", HFILL }},
    { &hf_rrc_targetCellPreconfigInfo,
      { "targetCellPreconfigInfo", "rrc.targetCellPreconfigInfo",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_dl_dpchInfo_07,
      { "dl-dpchInfo", "rrc.dl_dpchInfo",
        FT_UINT32, BASE_DEC, VALS(rrc_T_dl_dpchInfo_07_vals), 0,
        "T_dl_dpchInfo_07", HFILL }},
    { &hf_rrc_targetCellPreconfigInfo_01,
      { "targetCellPreconfigInfo", "rrc.targetCellPreconfigInfo",
        FT_NONE, BASE_NONE, NULL, 0,
        "TargetCellPreconfigInfo_r9", HFILL }},
    { &hf_rrc_dl_dpchInfo_08,
      { "dl-dpchInfo", "rrc.dl_dpchInfo",
        FT_UINT32, BASE_DEC, VALS(rrc_T_dl_dpchInfo_08_vals), 0,
        "T_dl_dpchInfo_08", HFILL }},
    { &hf_rrc_targetCellPreconfigInfo_02,
      { "targetCellPreconfigInfo", "rrc.targetCellPreconfigInfo",
        FT_NONE, BASE_NONE, NULL, 0,
        "TargetCellPreconfigInfo_r10", HFILL }},
    { &hf_rrc_targetCellPreconfigInfo_03,
      { "targetCellPreconfigInfo", "rrc.targetCellPreconfigInfo",
        FT_NONE, BASE_NONE, NULL, 0,
        "TargetCellPreconfigInfo_vbxyext", HFILL }},
    { &hf_rrc_dl_dpchInfo_09,
      { "dl-dpchInfo", "rrc.dl_dpchInfo",
        FT_UINT32, BASE_DEC, VALS(rrc_T_dl_dpchInfo_09_vals), 0,
        "T_dl_dpchInfo_09", HFILL }},
    { &hf_rrc_targetCellPreconfigInfo_04,
      { "targetCellPreconfigInfo", "rrc.targetCellPreconfigInfo",
        FT_NONE, BASE_NONE, NULL, 0,
        "TargetCellPreconfigInfo_r11", HFILL }},
    { &hf_rrc_targetCellPreconfigInfo_05,
      { "targetCellPreconfigInfo", "rrc.targetCellPreconfigInfo",
        FT_NONE, BASE_NONE, NULL, 0,
        "TargetCellPreconfigInfo_v890ext", HFILL }},
    { &hf_rrc_targetCellPreconfigInfo_06,
      { "targetCellPreconfigInfo", "rrc.targetCellPreconfigInfo",
        FT_NONE, BASE_NONE, NULL, 0,
        "TargetCellPreconfigInfo_v950ext", HFILL }},
    { &hf_rrc_RL_AdditionInformationList_item,
      { "RL-AdditionInformation", "rrc.RL_AdditionInformation",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_RL_AdditionInformationList_r6_item,
      { "RL-AdditionInformation-r6", "rrc.RL_AdditionInformation_r6",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_RL_AdditionInformationList_r7_item,
      { "RL-AdditionInformation-r7", "rrc.RL_AdditionInformation_r7",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_RL_AdditionInformation_list_v6b0ext_item,
      { "RL-AdditionInformation-v6b0ext", "rrc.RL_AdditionInformation_v6b0ext",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_RL_AdditionInformationList_r8_item,
      { "RL-AdditionInformation-r8", "rrc.RL_AdditionInformation_r8",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_RL_AdditionInformationList_v890ext_item,
      { "RL-AdditionInformation-v890ext", "rrc.RL_AdditionInformation_v890ext",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_RL_AdditionInformationList_r9_item,
      { "RL-AdditionInformation-r9", "rrc.RL_AdditionInformation_r9",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_RL_AdditionInformationList_v950ext_item,
      { "RL-AdditionInformation-v950ext", "rrc.RL_AdditionInformation_v950ext",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_RL_AdditionInformationList_r10_item,
      { "RL-AdditionInformation-r10", "rrc.RL_AdditionInformation_r10",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_RL_AdditionInformationList_vbxyext_item,
      { "RL-AdditionInformation-vbxyext", "rrc.RL_AdditionInformation_vbxyext",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_RL_AdditionInformationList_r11_item,
      { "RL-AdditionInformation-r11", "rrc.RL_AdditionInformation_r11",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_RL_AdditionInformationList_SecULFreq_item,
      { "RL-AdditionInformation-SecULFreq", "rrc.RL_AdditionInformation_SecULFreq",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_RL_IdentifierList_item,
      { "PrimaryCPICH-Info", "rrc.PrimaryCPICH_Info",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_RL_RemovalInformationList_item,
      { "PrimaryCPICH-Info", "rrc.PrimaryCPICH_Info",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_RL_RemovalInformationList_SecULFreq_item,
      { "PrimaryCPICH-Info", "rrc.PrimaryCPICH_Info",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_SCCPCH_ChannelisationCodeList_item,
      { "SCCPCH-ChannelisationCode", "rrc.SCCPCH_ChannelisationCode",
        FT_UINT32, BASE_DEC, VALS(rrc_SCCPCH_ChannelisationCode_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_SCCPCH_ChannelisationCodeList_VHCR_item,
      { "SCCPCH-ChannelisationCode-VHCR", "rrc.SCCPCH_ChannelisationCode_VHCR",
        FT_UINT32, BASE_DEC|BASE_EXT_STRING, &rrc_SCCPCH_ChannelisationCode_VHCR_vals_ext, 0,
        NULL, HFILL }},
    { &hf_rrc_secondaryCCPCH_Info,
      { "secondaryCCPCH-Info", "rrc.secondaryCCPCH_Info",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_modeSpecificInfo_74,
      { "modeSpecificInfo", "rrc.modeSpecificInfo",
        FT_UINT32, BASE_DEC, VALS(rrc_T_modeSpecificInfo_74_vals), 0,
        "T_modeSpecificInfo_74", HFILL }},
    { &hf_rrc_fdd_110,
      { "fdd", "rrc.fdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_fdd_107", HFILL }},
    { &hf_rrc_fach_PCH_InformationList,
      { "fach-PCH-InformationList", "rrc.fach_PCH_InformationList",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_sib_ReferenceListFACH,
      { "sib-ReferenceListFACH", "rrc.sib_ReferenceListFACH",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_tdd_67,
      { "tdd", "rrc.tdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_tdd_61", HFILL }},
    { &hf_rrc_secondaryCCPCH_Info_01,
      { "secondaryCCPCH-Info", "rrc.secondaryCCPCH_Info",
        FT_NONE, BASE_NONE, NULL, 0,
        "SecondaryCCPCH_Info_r4", HFILL }},
    { &hf_rrc_modeSpecificInfo_75,
      { "modeSpecificInfo", "rrc.modeSpecificInfo",
        FT_UINT32, BASE_DEC, VALS(rrc_T_modeSpecificInfo_75_vals), 0,
        "T_modeSpecificInfo_75", HFILL }},
    { &hf_rrc_fdd_111,
      { "fdd", "rrc.fdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_fdd_108", HFILL }},
    { &hf_rrc_pich_Info,
      { "pich-Info", "rrc.pich_Info",
        FT_UINT32, BASE_DEC, VALS(rrc_PICH_Info_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_secondaryCCPCH_LCR_Extensions,
      { "secondaryCCPCH-LCR-Extensions", "rrc.secondaryCCPCH_LCR_Extensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "SecondaryCCPCH_Info_LCR_r4_ext", HFILL }},
    { &hf_rrc_pich_Info_01,
      { "pich-Info", "rrc.pich_Info",
        FT_NONE, BASE_NONE, NULL, 0,
        "PICH_Info_LCR_r4", HFILL }},
    { &hf_rrc_secondaryCCPCH_Info_02,
      { "secondaryCCPCH-Info", "rrc.secondaryCCPCH_Info",
        FT_NONE, BASE_NONE, NULL, 0,
        "SecondaryCCPCH_Info_HCR_VHCR_r7", HFILL }},
    { &hf_rrc_pich_Info_02,
      { "pich-Info", "rrc.pich_Info",
        FT_NONE, BASE_NONE, NULL, 0,
        "PICH_Info_HCR_VHCR_r7", HFILL }},
    { &hf_rrc_SCCPCH_SystemInformationList_item,
      { "SCCPCH-SystemInformation", "rrc.SCCPCH_SystemInformation",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_SCCPCH_SystemInformationList_HCR_VHCR_r7_item,
      { "SCCPCH-SystemInformation-HCR-VHCR-r7", "rrc.SCCPCH_SystemInformation_HCR_VHCR_r7",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_SCCPCH_SystemInformationList_LCR_r4_ext_item,
      { "SCCPCH-SystemInformation-LCR-r4-ext", "rrc.SCCPCH_SystemInformation_LCR_r4_ext",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_secondaryCCPCHInfo_MBMS,
      { "secondaryCCPCHInfo-MBMS", "rrc.secondaryCCPCHInfo_MBMS",
        FT_NONE, BASE_NONE, NULL, 0,
        "SecondaryCCPCHInfo_MBMS_r6", HFILL }},
    { &hf_rrc_transportFormatCombinationSet,
      { "transportFormatCombinationSet", "rrc.transportFormatCombinationSet",
        FT_UINT32, BASE_DEC, VALS(rrc_TFCS_vals), 0,
        "TFCS", HFILL }},
    { &hf_rrc_fachCarryingMCCH,
      { "fachCarryingMCCH", "rrc.fachCarryingMCCH",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_mcch_transportFormatSet,
      { "mcch-transportFormatSet", "rrc.mcch_transportFormatSet",
        FT_UINT32, BASE_DEC, VALS(rrc_TransportFormatSet_vals), 0,
        "TransportFormatSet", HFILL }},
    { &hf_rrc_mcch_ConfigurationInfo,
      { "mcch-ConfigurationInfo", "rrc.mcch_ConfigurationInfo",
        FT_NONE, BASE_NONE, NULL, 0,
        "MBMS_MCCH_ConfigurationInfo_r6", HFILL }},
    { &hf_rrc_fachCarryingMTCH_List,
      { "fachCarryingMTCH-List", "rrc.fachCarryingMTCH_List",
        FT_UINT32, BASE_DEC, NULL, 0,
        "MBMS_FACHCarryingMTCH_List", HFILL }},
    { &hf_rrc_fachCarryingMSCH,
      { "fachCarryingMSCH", "rrc.fachCarryingMSCH",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_msch_transportFormatSet,
      { "msch-transportFormatSet", "rrc.msch_transportFormatSet",
        FT_UINT32, BASE_DEC, VALS(rrc_TransportFormatSet_vals), 0,
        "TransportFormatSet", HFILL }},
    { &hf_rrc_dummy_45,
      { "dummy", "rrc.dummy",
        FT_NONE, BASE_NONE, NULL, 0,
        "MBMS_MSCH_ConfigurationInfo_r6", HFILL }},
    { &hf_rrc_secondaryCCPCHInfo_MBMS_01,
      { "secondaryCCPCHInfo-MBMS", "rrc.secondaryCCPCHInfo_MBMS",
        FT_NONE, BASE_NONE, NULL, 0,
        "SecondaryCCPCHInfo_MBMS_r7", HFILL }},
    { &hf_rrc_fachCarryingMCCH_01,
      { "fachCarryingMCCH", "rrc.fachCarryingMCCH",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_fachCarryingMCCH_01", HFILL }},
    { &hf_rrc_fachCarryingMSCH_01,
      { "fachCarryingMSCH", "rrc.fachCarryingMSCH",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_fachCarryingMSCH_01", HFILL }},
    { &hf_rrc_common_E_DCH_ResourceInfoListExt,
      { "common-E-DCH-ResourceInfoListExt", "rrc.common_E_DCH_ResourceInfoListExt",
        FT_UINT32, BASE_DEC, NULL, 0,
        "SEQUENCE_SIZE_1_maxEDCHs_OF_Common_E_DCH_ResourceInfoListExt", HFILL }},
    { &hf_rrc_common_E_DCH_ResourceInfoListExt_item,
      { "Common-E-DCH-ResourceInfoListExt", "rrc.Common_E_DCH_ResourceInfoListExt",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_modeSpecificInfo_76,
      { "modeSpecificInfo", "rrc.modeSpecificInfo",
        FT_UINT32, BASE_DEC, VALS(rrc_T_modeSpecificInfo_76_vals), 0,
        "T_modeSpecificInfo_76", HFILL }},
    { &hf_rrc_fdd_112,
      { "fdd", "rrc.fdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_fdd_109", HFILL }},
    { &hf_rrc_dummy1_03,
      { "dummy1", "rrc.dummy1",
        FT_UINT32, BASE_DEC, VALS(rrc_PCPICH_UsageForChannelEst_vals), 0,
        "PCPICH_UsageForChannelEst", HFILL }},
    { &hf_rrc_dummy2_09,
      { "dummy2", "rrc.dummy2",
        FT_NONE, BASE_NONE, NULL, 0,
        "SecondaryCPICH_Info", HFILL }},
    { &hf_rrc_sf_AndCodeNumber_01,
      { "sf-AndCodeNumber", "rrc.sf_AndCodeNumber",
        FT_UINT32, BASE_DEC, VALS(rrc_SF256_AndCodeNumber_vals), 0,
        "SF256_AndCodeNumber", HFILL }},
    { &hf_rrc_pilotSymbolExistence,
      { "pilotSymbolExistence", "rrc.pilotSymbolExistence",
        FT_BOOLEAN, BASE_NONE, NULL, 0,
        "BOOLEAN", HFILL }},
    { &hf_rrc_timingOffset,
      { "timingOffset", "rrc.timingOffset",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_tdd_68,
      { "tdd", "rrc.tdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_tdd_62", HFILL }},
    { &hf_rrc_commonTimeslotInfo_01,
      { "commonTimeslotInfo", "rrc.commonTimeslotInfo",
        FT_NONE, BASE_NONE, NULL, 0,
        "CommonTimeslotInfoSCCPCH", HFILL }},
    { &hf_rrc_channelisationCode_09,
      { "channelisationCode", "rrc.channelisationCode",
        FT_UINT32, BASE_DEC, NULL, 0,
        "SCCPCH_ChannelisationCodeList", HFILL }},
    { &hf_rrc_modeSpecificInfo_77,
      { "modeSpecificInfo", "rrc.modeSpecificInfo",
        FT_UINT32, BASE_DEC, VALS(rrc_T_modeSpecificInfo_77_vals), 0,
        "T_modeSpecificInfo_77", HFILL }},
    { &hf_rrc_fdd_113,
      { "fdd", "rrc.fdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_fdd_110", HFILL }},
    { &hf_rrc_tdd_69,
      { "tdd", "rrc.tdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_tdd_63", HFILL }},
    { &hf_rrc_tddOption_18,
      { "tddOption", "rrc.tddOption",
        FT_UINT32, BASE_DEC, VALS(rrc_T_tddOption_18_vals), 0,
        "T_tddOption_18", HFILL }},
    { &hf_rrc_tdd384_33,
      { "tdd384", "rrc.tdd384",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_tdd384_30", HFILL }},
    { &hf_rrc_tdd128_51,
      { "tdd128", "rrc.tdd128",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_tdd128_50", HFILL }},
    { &hf_rrc_modeSpecificInfo_78,
      { "modeSpecificInfo", "rrc.modeSpecificInfo",
        FT_UINT32, BASE_DEC, VALS(rrc_T_modeSpecificInfo_78_vals), 0,
        "T_modeSpecificInfo_78", HFILL }},
    { &hf_rrc_tdd384_34,
      { "tdd384", "rrc.tdd384",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_tdd384_31", HFILL }},
    { &hf_rrc_tdd768_21,
      { "tdd768", "rrc.tdd768",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_tdd768_18", HFILL }},
    { &hf_rrc_channelisationCode_10,
      { "channelisationCode", "rrc.channelisationCode",
        FT_UINT32, BASE_DEC, NULL, 0,
        "SCCPCH_ChannelisationCodeList_VHCR", HFILL }},
    { &hf_rrc_individualTimeslotLCR_Ext,
      { "individualTimeslotLCR-Ext", "rrc.individualTimeslotLCR_Ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "IndividualTimeslotInfo_LCR_r4_ext", HFILL }},
    { &hf_rrc_subFrameNumber,
      { "subFrameNumber", "rrc.subFrameNumber",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_4", HFILL }},
    { &hf_rrc_dl_ChannelisationCodes,
      { "dl-ChannelisationCodes", "rrc.dl_ChannelisationCodes",
        FT_NONE, BASE_NONE, NULL, 0,
        "DL_ChannelCodes_MBSFN_IMB384", HFILL }},
    { &hf_rrc_modulation_03,
      { "modulation", "rrc.modulation",
        FT_UINT32, BASE_DEC, VALS(rrc_T_modulation_03_vals), 0,
        "T_modulation_03", HFILL }},
    { &hf_rrc_modQPSK,
      { "modQPSK", "rrc.modQPSK",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_mod16QAM,
      { "mod16QAM", "rrc.mod16QAM",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_cpich_SecCCPCH_PowerOffset,
      { "cpich-SecCCPCH-PowerOffset", "rrc.cpich_SecCCPCH_PowerOffset",
        FT_INT32, BASE_DEC, NULL, 0,
        "INTEGER_M11_4", HFILL }},
    { &hf_rrc_modeSpecificInfo_79,
      { "modeSpecificInfo", "rrc.modeSpecificInfo",
        FT_UINT32, BASE_DEC, VALS(rrc_T_modeSpecificInfo_79_vals), 0,
        "T_modeSpecificInfo_79", HFILL }},
    { &hf_rrc_fdd_114,
      { "fdd", "rrc.fdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_fdd_111", HFILL }},
    { &hf_rrc_tdd384_35,
      { "tdd384", "rrc.tdd384",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_tdd384_32", HFILL }},
    { &hf_rrc_commonTimeslotInfoMBMS,
      { "commonTimeslotInfoMBMS", "rrc.commonTimeslotInfoMBMS",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_downlinkTimeslotsCodes,
      { "downlinkTimeslotsCodes", "rrc.downlinkTimeslotsCodes",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_tdd128_52,
      { "tdd128", "rrc.tdd128",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_tdd128_51", HFILL }},
    { &hf_rrc_downlinkTimeslotsCodes_01,
      { "downlinkTimeslotsCodes", "rrc.downlinkTimeslotsCodes",
        FT_NONE, BASE_NONE, NULL, 0,
        "DownlinkTimeslotsCodes_LCR_r4", HFILL }},
    { &hf_rrc_modeSpecificInfo_80,
      { "modeSpecificInfo", "rrc.modeSpecificInfo",
        FT_UINT32, BASE_DEC, VALS(rrc_T_modeSpecificInfo_80_vals), 0,
        "T_modeSpecificInfo_80", HFILL }},
    { &hf_rrc_fdd_115,
      { "fdd", "rrc.fdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_fdd_112", HFILL }},
    { &hf_rrc_modulation_04,
      { "modulation", "rrc.modulation",
        FT_UINT32, BASE_DEC, VALS(rrc_T_modulation_04_vals), 0,
        "T_modulation_04", HFILL }},
    { &hf_rrc_mod16QAM_01,
      { "mod16QAM", "rrc.mod16QAM",
        FT_INT32, BASE_DEC, NULL, 0,
        "INTEGER_M11_4", HFILL }},
    { &hf_rrc_tdd384_36,
      { "tdd384", "rrc.tdd384",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_tdd384_33", HFILL }},
    { &hf_rrc_downlinkTimeslotsCodes_02,
      { "downlinkTimeslotsCodes", "rrc.downlinkTimeslotsCodes",
        FT_NONE, BASE_NONE, NULL, 0,
        "DownlinkTimeslotsCodes_r7", HFILL }},
    { &hf_rrc_modulation_05,
      { "modulation", "rrc.modulation",
        FT_UINT32, BASE_DEC, VALS(rrc_T_modulation_05_vals), 0,
        "T_modulation_05", HFILL }},
    { &hf_rrc_tdd768_22,
      { "tdd768", "rrc.tdd768",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_tdd768_19", HFILL }},
    { &hf_rrc_downlinkTimeslotsCodes_03,
      { "downlinkTimeslotsCodes", "rrc.downlinkTimeslotsCodes",
        FT_NONE, BASE_NONE, NULL, 0,
        "DownlinkTimeslotsCodes_VHCR", HFILL }},
    { &hf_rrc_modulation_06,
      { "modulation", "rrc.modulation",
        FT_UINT32, BASE_DEC, VALS(rrc_T_modulation_06_vals), 0,
        "T_modulation_06", HFILL }},
    { &hf_rrc_tdd128_53,
      { "tdd128", "rrc.tdd128",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_tdd128_52", HFILL }},
    { &hf_rrc_mbsfnSpecialTimeSlot,
      { "mbsfnSpecialTimeSlot", "rrc.mbsfnSpecialTimeSlot",
        FT_UINT32, BASE_DEC, VALS(rrc_TimeSlotLCR_ext_vals), 0,
        "TimeSlotLCR_ext", HFILL }},
    { &hf_rrc_modulation_07,
      { "modulation", "rrc.modulation",
        FT_UINT32, BASE_DEC, VALS(rrc_T_modulation_07_vals), 0,
        "T_modulation_07", HFILL }},
    { &hf_rrc_secondaryDL_ScramblingCode,
      { "secondaryDL-ScramblingCode", "rrc.secondaryDL_ScramblingCode",
        FT_UINT32, BASE_DEC, NULL, 0,
        "SecondaryScramblingCode", HFILL }},
    { &hf_rrc_newConfiguration_08,
      { "newConfiguration", "rrc.newConfiguration",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_newConfiguration_08", HFILL }},
    { &hf_rrc_precodingWeightSetRestriction_03,
      { "precodingWeightSetRestriction", "rrc.precodingWeightSetRestriction",
        FT_UINT32, BASE_DEC, VALS(rrc_T_precodingWeightSetRestriction_02_vals), 0,
        "T_precodingWeightSetRestriction_02", HFILL }},
    { &hf_rrc_newConfiguration_09,
      { "newConfiguration", "rrc.newConfiguration",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_newConfiguration_09", HFILL }},
    { &hf_rrc_precodingWeightSetRestriction_04,
      { "precodingWeightSetRestriction", "rrc.precodingWeightSetRestriction",
        FT_UINT32, BASE_DEC, VALS(rrc_T_precodingWeightSetRestriction_03_vals), 0,
        "T_precodingWeightSetRestriction_03", HFILL }},
    { &hf_rrc_newConfiguration_10,
      { "newConfiguration", "rrc.newConfiguration",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_newConfiguration_10", HFILL }},
    { &hf_rrc_mimoPilotConfiguration_02,
      { "mimoPilotConfiguration", "rrc.mimoPilotConfiguration",
        FT_NONE, BASE_NONE, NULL, 0,
        "MIMO4x4_PilotConfiguration", HFILL }},
    { &hf_rrc_servingCellChangeMACreset,
      { "servingCellChangeMACreset", "rrc.servingCellChangeMACreset",
        FT_BOOLEAN, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_servingCellChangeMsgType,
      { "servingCellChangeMsgType", "rrc.servingCellChangeMsgType",
        FT_UINT32, BASE_DEC, VALS(rrc_ServingCellChangeMsgType_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_servingCellChangeTrId,
      { "servingCellChangeTrId", "rrc.servingCellChangeTrId",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_sf4,
      { "sf4", "rrc.sf4",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_3", HFILL }},
    { &hf_rrc_sf8,
      { "sf8", "rrc.sf8",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_7", HFILL }},
    { &hf_rrc_sf16,
      { "sf16", "rrc.sf16",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_15", HFILL }},
    { &hf_rrc_sf32,
      { "sf32", "rrc.sf32",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_31", HFILL }},
    { &hf_rrc_sf64,
      { "sf64", "rrc.sf64",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_63", HFILL }},
    { &hf_rrc_sf128,
      { "sf128", "rrc.sf128",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_127", HFILL }},
    { &hf_rrc_sf256,
      { "sf256", "rrc.sf256",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_255", HFILL }},
    { &hf_rrc_sf512,
      { "sf512", "rrc.sf512",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_511", HFILL }},
    { &hf_rrc_sfd4,
      { "sfd4", "rrc.sfd4",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_sfd8,
      { "sfd8", "rrc.sfd8",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_sfd16,
      { "sfd16", "rrc.sfd16",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_sfd32,
      { "sfd32", "rrc.sfd32",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_sfd64,
      { "sfd64", "rrc.sfd64",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_sfd128,
      { "sfd128", "rrc.sfd128",
        FT_UINT32, BASE_DEC, VALS(rrc_PilotBits128_vals), 0,
        "PilotBits128", HFILL }},
    { &hf_rrc_sfd256,
      { "sfd256", "rrc.sfd256",
        FT_UINT32, BASE_DEC, VALS(rrc_PilotBits256_vals), 0,
        "PilotBits256", HFILL }},
    { &hf_rrc_sfd512,
      { "sfd512", "rrc.sfd512",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_activationTimeSFN,
      { "activationTimeSFN", "rrc.activationTimeSFN",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_4095", HFILL }},
    { &hf_rrc_physChDuration,
      { "physChDuration", "rrc.physChDuration",
        FT_UINT32, BASE_DEC, NULL, 0,
        "DurationTimeInfo", HFILL }},
    { &hf_rrc_e_dch_SPS_Info,
      { "e-dch-SPS-Info", "rrc.e_dch_SPS_Info",
        FT_NONE, BASE_NONE, NULL, 0,
        "E_DCH_SPS_Information_TDD128", HFILL }},
    { &hf_rrc_hs_dsch_SPS_Info,
      { "hs-dsch-SPS-Info", "rrc.hs_dsch_SPS_Info",
        FT_NONE, BASE_NONE, NULL, 0,
        "HS_DSCH_SPS_Information_TDD128", HFILL }},
    { &hf_rrc_s_Field,
      { "s-Field", "rrc.s_Field",
        FT_UINT32, BASE_DEC, VALS(rrc_S_Field_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_codeWordSet,
      { "codeWordSet", "rrc.codeWordSet",
        FT_UINT32, BASE_DEC, VALS(rrc_CodeWordSet_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_ssdt_UL_r4,
      { "ssdt-UL-r4", "rrc.ssdt_UL_r4",
        FT_UINT32, BASE_DEC, VALS(rrc_SSDT_UL_vals), 0,
        "SSDT_UL", HFILL }},
    { &hf_rrc_timeSlotInformation,
      { "timeSlotInformation", "rrc.timeSlotInformation",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_5", HFILL }},
    { &hf_rrc_repetitionPeriod,
      { "repetitionPeriod", "rrc.repetitionPeriod",
        FT_UINT32, BASE_DEC, VALS(rrc_T_repetitionPeriod_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_referenceBeta,
      { "referenceBeta", "rrc.referenceBeta",
        FT_INT32, BASE_DEC, NULL, 0,
        "INTEGER_M15_16", HFILL }},
    { &hf_rrc_sync_UL_CodesBitmap,
      { "sync-UL-CodesBitmap", "rrc.sync_UL_CodesBitmap",
        FT_BYTES, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_prxUpPCHdes,
      { "prxUpPCHdes", "rrc.prxUpPCHdes",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_62", HFILL }},
    { &hf_rrc_sync_UL_Procedure,
      { "sync-UL-Procedure", "rrc.sync_UL_Procedure",
        FT_NONE, BASE_NONE, NULL, 0,
        "SYNC_UL_Procedure_r4", HFILL }},
    { &hf_rrc_max_SYNC_UL_Transmissions,
      { "max-SYNC-UL-Transmissions", "rrc.max_SYNC_UL_Transmissions",
        FT_UINT32, BASE_DEC, VALS(rrc_T_max_SYNC_UL_Transmissions_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_powerRampStep_01,
      { "powerRampStep", "rrc.powerRampStep",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_3", HFILL }},
    { &hf_rrc_sync_UL_Codes_Bitmap,
      { "sync-UL-Codes-Bitmap", "rrc.sync_UL_Codes_Bitmap",
        FT_BYTES, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_max_SYNC_UL_Transmissions_01,
      { "max-SYNC-UL-Transmissions", "rrc.max_SYNC_UL_Transmissions",
        FT_UINT32, BASE_DEC, VALS(rrc_T_max_SYNC_UL_Transmissions_01_vals), 0,
        "T_max_SYNC_UL_Transmissions_01", HFILL }},
    { &hf_rrc_max_SYNC_UL_Transmissions_02,
      { "max-SYNC-UL-Transmissions", "rrc.max_SYNC_UL_Transmissions",
        FT_UINT32, BASE_DEC, VALS(rrc_T_max_SYNC_UL_Transmissions_02_vals), 0,
        "T_max_SYNC_UL_Transmissions_02", HFILL }},
    { &hf_rrc_activationTimeOffset,
      { "activationTimeOffset", "rrc.activationTimeOffset",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_uL_OLTD_InfoFDD,
      { "uL-OLTD-InfoFDD", "rrc.uL_OLTD_InfoFDD",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_ul_MIMO_Info,
      { "ul-MIMO-Info", "rrc.ul_MIMO_Info",
        FT_NONE, BASE_NONE, NULL, 0,
        "UL_MIMO_InfoFDD", HFILL }},
    { &hf_rrc_TDD_MBSFNInformation_item,
      { "TDD-MBSFNTSlotInfo", "rrc.TDD_MBSFNTSlotInfo",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_timeSlotNumber,
      { "timeSlotNumber", "rrc.timeSlotNumber",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_sf8_01,
      { "sf8", "rrc.sf8",
        FT_UINT32, BASE_DEC, NULL, 0,
        "SEQUENCE_SIZE_1_8_OF_TDD_PRACH_CCode8", HFILL }},
    { &hf_rrc_sf8_item,
      { "TDD-PRACH-CCode8", "rrc.TDD_PRACH_CCode8",
        FT_UINT32, BASE_DEC, VALS(rrc_TDD_PRACH_CCode8_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_sf16_01,
      { "sf16", "rrc.sf16",
        FT_UINT32, BASE_DEC, NULL, 0,
        "SEQUENCE_SIZE_1_8_OF_TDD_PRACH_CCode16", HFILL }},
    { &hf_rrc_sf16_item,
      { "TDD-PRACH-CCode16", "rrc.TDD_PRACH_CCode16",
        FT_UINT32, BASE_DEC, VALS(rrc_TDD_PRACH_CCode16_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_sf32_01,
      { "sf32", "rrc.sf32",
        FT_UINT32, BASE_DEC, NULL, 0,
        "SEQUENCE_SIZE_1_16_OF_TDD768_PRACH_CCode32", HFILL }},
    { &hf_rrc_sf32_item,
      { "TDD768-PRACH-CCode32", "rrc.TDD768_PRACH_CCode32",
        FT_UINT32, BASE_DEC, VALS(rrc_TDD768_PRACH_CCode32_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_sf16_02,
      { "sf16", "rrc.sf16",
        FT_UINT32, BASE_DEC, NULL, 0,
        "SEQUENCE_SIZE_1_16_OF_TDD768_PRACH_CCode16", HFILL }},
    { &hf_rrc_sf16_item_01,
      { "TDD768-PRACH-CCode16", "rrc.TDD768_PRACH_CCode16",
        FT_UINT32, BASE_DEC, VALS(rrc_TDD768_PRACH_CCode16_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_tgpsi,
      { "tgpsi", "rrc.tgpsi",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_tgps_Status,
      { "tgps-Status", "rrc.tgps_Status",
        FT_UINT32, BASE_DEC, VALS(rrc_T_tgps_Status_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_activate,
      { "activate", "rrc.activate",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_tgcfn,
      { "tgcfn", "rrc.tgcfn",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_deactivate,
      { "deactivate", "rrc.deactivate",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_tgps_ConfigurationParams,
      { "tgps-ConfigurationParams", "rrc.tgps_ConfigurationParams",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_tgps_Status_01,
      { "tgps-Status", "rrc.tgps_Status",
        FT_UINT32, BASE_DEC, VALS(rrc_T_tgps_Status_01_vals), 0,
        "T_tgps_Status_01", HFILL }},
    { &hf_rrc_activate_01,
      { "activate", "rrc.activate",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_activate_01", HFILL }},
    { &hf_rrc_tgps_ConfigurationParams_01,
      { "tgps-ConfigurationParams", "rrc.tgps_ConfigurationParams",
        FT_NONE, BASE_NONE, NULL, 0,
        "TGPS_ConfigurationParams_r8", HFILL }},
    { &hf_rrc_tgps_Status_02,
      { "tgps-Status", "rrc.tgps_Status",
        FT_UINT32, BASE_DEC, VALS(rrc_T_tgps_Status_02_vals), 0,
        "T_tgps_Status_02", HFILL }},
    { &hf_rrc_activate_02,
      { "activate", "rrc.activate",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_activate_02", HFILL }},
    { &hf_rrc_tgps_ConfigurationParams_02,
      { "tgps-ConfigurationParams", "rrc.tgps_ConfigurationParams",
        FT_NONE, BASE_NONE, NULL, 0,
        "TGPS_ConfigurationParams_r10", HFILL }},
    { &hf_rrc_TGP_SequenceList_item,
      { "TGP-Sequence", "rrc.TGP_Sequence",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_TGP_SequenceList_r8_item,
      { "TGP-Sequence-r8", "rrc.TGP_Sequence_r8",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_TGP_SequenceList_r10_item,
      { "TGP-Sequence-r10", "rrc.TGP_Sequence_r10",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_tgps_Status_03,
      { "tgps-Status", "rrc.tgps_Status",
        FT_UINT32, BASE_DEC, VALS(rrc_T_tgps_Status_03_vals), 0,
        "T_tgps_Status_03", HFILL }},
    { &hf_rrc_activate_03,
      { "activate", "rrc.activate",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_activate_03", HFILL }},
    { &hf_rrc_tgps_Status_04,
      { "tgps-Status", "rrc.tgps_Status",
        FT_UINT32, BASE_DEC, VALS(rrc_T_tgps_Status_04_vals), 0,
        "T_tgps_Status_04", HFILL }},
    { &hf_rrc_activate_04,
      { "activate", "rrc.activate",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_activate_04", HFILL }},
    { &hf_rrc_freqSpecificCompressedMode_01,
      { "freqSpecificCompressedMode", "rrc.freqSpecificCompressedMode",
        FT_UINT32, BASE_DEC, VALS(rrc_T_freqSpecificCompressedMode_01_vals), 0,
        "T_freqSpecificCompressedMode_01", HFILL }},
    { &hf_rrc_tgmp,
      { "tgmp", "rrc.tgmp",
        FT_UINT32, BASE_DEC, VALS(rrc_TGMP_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_tgprc,
      { "tgprc", "rrc.tgprc",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_tgsn,
      { "tgsn", "rrc.tgsn",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_tgl1,
      { "tgl1", "rrc.tgl1",
        FT_UINT32, BASE_DEC, NULL, 0,
        "TGL", HFILL }},
    { &hf_rrc_tgl2,
      { "tgl2", "rrc.tgl2",
        FT_UINT32, BASE_DEC, NULL, 0,
        "TGL", HFILL }},
    { &hf_rrc_tgd,
      { "tgd", "rrc.tgd",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_tgpl1,
      { "tgpl1", "rrc.tgpl1",
        FT_UINT32, BASE_DEC, NULL, 0,
        "TGPL", HFILL }},
    { &hf_rrc_dummy_46,
      { "dummy", "rrc.dummy",
        FT_UINT32, BASE_DEC, NULL, 0,
        "TGPL", HFILL }},
    { &hf_rrc_rpp,
      { "rpp", "rrc.rpp",
        FT_UINT32, BASE_DEC, VALS(rrc_RPP_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_itp,
      { "itp", "rrc.itp",
        FT_UINT32, BASE_DEC, VALS(rrc_ITP_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_ul_DL_Mode,
      { "ul-DL-Mode", "rrc.ul_DL_Mode",
        FT_UINT32, BASE_DEC, VALS(rrc_UL_DL_Mode_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_dl_FrameType,
      { "dl-FrameType", "rrc.dl_FrameType",
        FT_UINT32, BASE_DEC, VALS(rrc_DL_FrameType_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_deltaSIR1,
      { "deltaSIR1", "rrc.deltaSIR1",
        FT_UINT32, BASE_DEC, NULL, 0,
        "DeltaSIR", HFILL }},
    { &hf_rrc_deltaSIRAfter1,
      { "deltaSIRAfter1", "rrc.deltaSIRAfter1",
        FT_UINT32, BASE_DEC, NULL, 0,
        "DeltaSIR", HFILL }},
    { &hf_rrc_deltaSIR2,
      { "deltaSIR2", "rrc.deltaSIR2",
        FT_UINT32, BASE_DEC, NULL, 0,
        "DeltaSIR", HFILL }},
    { &hf_rrc_deltaSIRAfter2,
      { "deltaSIRAfter2", "rrc.deltaSIRAfter2",
        FT_UINT32, BASE_DEC, NULL, 0,
        "DeltaSIR", HFILL }},
    { &hf_rrc_nidentifyAbort,
      { "nidentifyAbort", "rrc.nidentifyAbort",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_treconfirmAbort,
      { "treconfirmAbort", "rrc.treconfirmAbort",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_tgmp_01,
      { "tgmp", "rrc.tgmp",
        FT_UINT32, BASE_DEC, VALS(rrc_TGMP_r8_vals), 0,
        "TGMP_r8", HFILL }},
    { &hf_rrc_freqSpecificCompressedMode_02,
      { "freqSpecificCompressedMode", "rrc.freqSpecificCompressedMode",
        FT_UINT32, BASE_DEC, VALS(rrc_T_freqSpecificCompressedMode_02_vals), 0,
        "T_freqSpecificCompressedMode_02", HFILL }},
    { &hf_rrc_durationTimeInfo,
      { "durationTimeInfo", "rrc.durationTimeInfo",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_TimeslotList_item,
      { "TimeslotNumber", "rrc.TimeslotNumber",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_tdd384_37,
      { "tdd384", "rrc.tdd384",
        FT_UINT32, BASE_DEC, NULL, 0,
        "SEQUENCE_SIZE_1_maxTS_OF_TimeslotNumber", HFILL }},
    { &hf_rrc_tdd384_item,
      { "TimeslotNumber", "rrc.TimeslotNumber",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_tdd128_54,
      { "tdd128", "rrc.tdd128",
        FT_UINT32, BASE_DEC, NULL, 0,
        "SEQUENCE_SIZE_1_maxTS_LCR_OF_TimeslotNumber_LCR_r4", HFILL }},
    { &hf_rrc_tdd128_item_01,
      { "TimeslotNumber-LCR-r4", "rrc.TimeslotNumber_LCR_r4",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_timing,
      { "timing", "rrc.timing",
        FT_UINT32, BASE_DEC, VALS(rrc_T_timing_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_newTiming,
      { "newTiming", "rrc.newTiming",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_harqProcessAllocation,
      { "harqProcessAllocation", "rrc.harqProcessAllocation",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_8", HFILL }},
    { &hf_rrc_channelReqParamsForUCSM,
      { "channelReqParamsForUCSM", "rrc.channelReqParamsForUCSM",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_ul_16QAM_Settings,
      { "ul-16QAM-Settings", "rrc.ul_16QAM_Settings",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_mac_es_e_resetIndicator,
      { "mac-es-e-resetIndicator", "rrc.mac_es_e_resetIndicator",
        FT_UINT32, BASE_DEC, VALS(rrc_T_mac_es_e_resetIndicator_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_beta_Ed_Gain_E_AGCH_Table_Selection,
      { "beta-Ed-Gain-E-AGCH-Table-Selection", "rrc.beta_Ed_Gain_E_AGCH_Table_Selection",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_1", HFILL }},
    { &hf_rrc_ul_64QAM_Settings,
      { "ul-64QAM-Settings", "rrc.ul_64QAM_Settings",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_mac_is_i_resetIndicator,
      { "mac-is-i-resetIndicator", "rrc.mac_is_i_resetIndicator",
        FT_UINT32, BASE_DEC, VALS(rrc_T_mac_is_i_resetIndicator_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_ul_CCTrCH_TimeslotsCodes,
      { "ul-CCTrCH-TimeslotsCodes", "rrc.ul_CCTrCH_TimeslotsCodes",
        FT_NONE, BASE_NONE, NULL, 0,
        "UplinkTimeslotsCodes", HFILL }},
    { &hf_rrc_tddOption_19,
      { "tddOption", "rrc.tddOption",
        FT_UINT32, BASE_DEC, VALS(rrc_T_tddOption_19_vals), 0,
        "T_tddOption_19", HFILL }},
    { &hf_rrc_tdd384_38,
      { "tdd384", "rrc.tdd384",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_tdd384_34", HFILL }},
    { &hf_rrc_tdd128_55,
      { "tdd128", "rrc.tdd128",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_tdd128_53", HFILL }},
    { &hf_rrc_ul_CCTrCH_TimeslotsCodes_01,
      { "ul-CCTrCH-TimeslotsCodes", "rrc.ul_CCTrCH_TimeslotsCodes",
        FT_NONE, BASE_NONE, NULL, 0,
        "UplinkTimeslotsCodes_LCR_r4", HFILL }},
    { &hf_rrc_tddOption_20,
      { "tddOption", "rrc.tddOption",
        FT_UINT32, BASE_DEC, VALS(rrc_T_tddOption_20_vals), 0,
        "T_tddOption_20", HFILL }},
    { &hf_rrc_tdd384_39,
      { "tdd384", "rrc.tdd384",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_tdd384_35", HFILL }},
    { &hf_rrc_tdd768_23,
      { "tdd768", "rrc.tdd768",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_tdd768_20", HFILL }},
    { &hf_rrc_ul_CCTrCH_TimeslotsCodes_02,
      { "ul-CCTrCH-TimeslotsCodes", "rrc.ul_CCTrCH_TimeslotsCodes",
        FT_NONE, BASE_NONE, NULL, 0,
        "UplinkTimeslotsCodes_VHCR", HFILL }},
    { &hf_rrc_tdd128_56,
      { "tdd128", "rrc.tdd128",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_tdd128_54", HFILL }},
    { &hf_rrc_ul_CCTrCH_TimeslotsCodes_03,
      { "ul-CCTrCH-TimeslotsCodes", "rrc.ul_CCTrCH_TimeslotsCodes",
        FT_NONE, BASE_NONE, NULL, 0,
        "UplinkTimeslotsCodes_LCR_r7", HFILL }},
    { &hf_rrc_UL_CCTrCHList_item,
      { "UL-CCTrCH", "rrc.UL_CCTrCH",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_UL_CCTrCHList_r4_item,
      { "UL-CCTrCH-r4", "rrc.UL_CCTrCH_r4",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_UL_CCTrCHList_r7_item,
      { "UL-CCTrCH-r7", "rrc.UL_CCTrCH_r7",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_UL_CCTrCHListToRemove_item,
      { "TFCS-IdentityPlain", "rrc.TFCS_IdentityPlain",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_UL_CCTrChTPCList_item,
      { "TFCS-Identity", "rrc.TFCS_Identity",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_dummy_47,
      { "dummy", "rrc.dummy",
        FT_NONE, BASE_NONE, NULL, 0,
        "CPCH_SetInfo", HFILL }},
    { &hf_rrc_dummy1_04,
      { "dummy1", "rrc.dummy1",
        FT_NONE, BASE_NONE, NULL, 0,
        "CPCH_SetInfo", HFILL }},
    { &hf_rrc_dummy2_10,
      { "dummy2", "rrc.dummy2",
        FT_UINT32, BASE_DEC, NULL, 0,
        "CPCH_SetID", HFILL }},
    { &hf_rrc_configurationInfo_04,
      { "configurationInfo", "rrc.configurationInfo",
        FT_UINT32, BASE_DEC, VALS(rrc_T_configurationInfo_04_vals), 0,
        "T_configurationInfo_04", HFILL }},
    { &hf_rrc_newConfiguration_11,
      { "newConfiguration", "rrc.newConfiguration",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_newConfiguration_11", HFILL }},
    { &hf_rrc_s_dpcch_Info,
      { "s-dpcch-Info", "rrc.s_dpcch_Info",
        FT_NONE, BASE_NONE, NULL, 0,
        "S_DPCCH_Information", HFILL }},
    { &hf_rrc_initial_CLTD_ActivationState,
      { "initial-CLTD-ActivationState", "rrc.initial_CLTD_ActivationState",
        FT_UINT32, BASE_DEC, VALS(rrc_T_initial_CLTD_ActivationState_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_configurationInfo_05,
      { "configurationInfo", "rrc.configurationInfo",
        FT_UINT32, BASE_DEC, VALS(rrc_T_configurationInfo_05_vals), 0,
        "T_configurationInfo_05", HFILL }},
    { &hf_rrc_newConfiguration_12,
      { "newConfiguration", "rrc.newConfiguration",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_newConfiguration_12", HFILL }},
    { &hf_rrc_s_e_dpcch_power_offset,
      { "s-e-dpcch-power-offset", "rrc.s_e_dpcch_power_offset",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_17", HFILL }},
    { &hf_rrc_rank2_minimum_E_TFCI,
      { "rank2-minimum-E-TFCI", "rrc.rank2_minimum_E_TFCI",
        FT_UINT32, BASE_DEC, NULL, 0,
        "E_DCH_MinimumSet_E_TFCI", HFILL }},
    { &hf_rrc_inter_stream_compensation,
      { "inter-stream-compensation", "rrc.inter_stream_compensation",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_15", HFILL }},
    { &hf_rrc_ul,
      { "ul", "rrc.ul",
        FT_UINT32, BASE_DEC, VALS(rrc_UL_CompressedModeMethod_vals), 0,
        "UL_CompressedModeMethod", HFILL }},
    { &hf_rrc_dl,
      { "dl", "rrc.dl",
        FT_UINT32, BASE_DEC, VALS(rrc_DL_CompressedModeMethod_vals), 0,
        "DL_CompressedModeMethod", HFILL }},
    { &hf_rrc_ul_and_dl,
      { "ul-and-dl", "rrc.ul_and_dl",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_ul_DPCCHscramblingCodeType,
      { "ul-DPCCHscramblingCodeType", "rrc.ul_DPCCHscramblingCodeType",
        FT_UINT32, BASE_DEC, VALS(rrc_ScramblingCodeType_vals), 0,
        "ScramblingCodeType", HFILL }},
    { &hf_rrc_ul_DPCCHscramblingCode,
      { "ul-DPCCHscramblingCode", "rrc.ul_DPCCHscramblingCode",
        FT_UINT32, BASE_DEC, NULL, 0,
        "UL_ScramblingCode", HFILL }},
    { &hf_rrc_modeSpecificInfo_81,
      { "modeSpecificInfo", "rrc.modeSpecificInfo",
        FT_UINT32, BASE_DEC, VALS(rrc_T_modeSpecificInfo_81_vals), 0,
        "T_modeSpecificInfo_81", HFILL }},
    { &hf_rrc_fdd_116,
      { "fdd", "rrc.fdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_fdd_113", HFILL }},
    { &hf_rrc_scramblingCode,
      { "scramblingCode", "rrc.scramblingCode",
        FT_UINT32, BASE_DEC, NULL, 0,
        "UL_ScramblingCode", HFILL }},
    { &hf_rrc_numberOfDPDCH,
      { "numberOfDPDCH", "rrc.numberOfDPDCH",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_spreadingFactor_01,
      { "spreadingFactor", "rrc.spreadingFactor",
        FT_UINT32, BASE_DEC, VALS(rrc_SpreadingFactor_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_numberOfFBI_Bits,
      { "numberOfFBI-Bits", "rrc.numberOfFBI_Bits",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_tdd_70,
      { "tdd", "rrc.tdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_tdd_64", HFILL }},
    { &hf_rrc_ul_CCTrCHList,
      { "ul-CCTrCHList", "rrc.ul_CCTrCHList",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_ul_CCTrCHListToRemove,
      { "ul-CCTrCHListToRemove", "rrc.ul_CCTrCHListToRemove",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_modeSpecificInfo_82,
      { "modeSpecificInfo", "rrc.modeSpecificInfo",
        FT_UINT32, BASE_DEC, VALS(rrc_T_modeSpecificInfo_82_vals), 0,
        "T_modeSpecificInfo_82", HFILL }},
    { &hf_rrc_fdd_117,
      { "fdd", "rrc.fdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_fdd_114", HFILL }},
    { &hf_rrc_tdd_71,
      { "tdd", "rrc.tdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_tdd_65", HFILL }},
    { &hf_rrc_ul_CCTrCHList_01,
      { "ul-CCTrCHList", "rrc.ul_CCTrCHList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "UL_CCTrCHList_r4", HFILL }},
    { &hf_rrc_modeSpecificInfo_83,
      { "modeSpecificInfo", "rrc.modeSpecificInfo",
        FT_UINT32, BASE_DEC, VALS(rrc_T_modeSpecificInfo_83_vals), 0,
        "T_modeSpecificInfo_83", HFILL }},
    { &hf_rrc_fdd_118,
      { "fdd", "rrc.fdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_fdd_115", HFILL }},
    { &hf_rrc_tdd_72,
      { "tdd", "rrc.tdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_tdd_66", HFILL }},
    { &hf_rrc_ul_DPCH_PowerControlInfo_04,
      { "ul-DPCH-PowerControlInfo", "rrc.ul_DPCH_PowerControlInfo",
        FT_UINT32, BASE_DEC, VALS(rrc_UL_DPCH_PowerControlInfo_r6_vals), 0,
        "UL_DPCH_PowerControlInfo_r6", HFILL }},
    { &hf_rrc_modeSpecificInfo_84,
      { "modeSpecificInfo", "rrc.modeSpecificInfo",
        FT_UINT32, BASE_DEC, VALS(rrc_T_modeSpecificInfo_84_vals), 0,
        "T_modeSpecificInfo_84", HFILL }},
    { &hf_rrc_fdd_119,
      { "fdd", "rrc.fdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_fdd_116", HFILL }},
    { &hf_rrc_dpdchPresence,
      { "dpdchPresence", "rrc.dpdchPresence",
        FT_UINT32, BASE_DEC, VALS(rrc_T_dpdchPresence_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_present_04,
      { "present", "rrc.present",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_present_02", HFILL }},
    { &hf_rrc_notPresent,
      { "notPresent", "rrc.notPresent",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_tdd_73,
      { "tdd", "rrc.tdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_tdd_67", HFILL }},
    { &hf_rrc_modeSpecificInfo_85,
      { "modeSpecificInfo", "rrc.modeSpecificInfo",
        FT_UINT32, BASE_DEC, VALS(rrc_T_modeSpecificInfo_85_vals), 0,
        "T_modeSpecificInfo_85", HFILL }},
    { &hf_rrc_fdd_120,
      { "fdd", "rrc.fdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_fdd_117", HFILL }},
    { &hf_rrc_dpdchPresence_01,
      { "dpdchPresence", "rrc.dpdchPresence",
        FT_UINT32, BASE_DEC, VALS(rrc_T_dpdchPresence_01_vals), 0,
        "T_dpdchPresence_01", HFILL }},
    { &hf_rrc_present_05,
      { "present", "rrc.present",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_present_03", HFILL }},
    { &hf_rrc_numberOfTPC_Bits,
      { "numberOfTPC-Bits", "rrc.numberOfTPC_Bits",
        FT_UINT32, BASE_DEC, VALS(rrc_NumberOfTPC_Bits_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_notPresent_01,
      { "notPresent", "rrc.notPresent",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_notPresent_01", HFILL }},
    { &hf_rrc_tdd_74,
      { "tdd", "rrc.tdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_tdd_68", HFILL }},
    { &hf_rrc_ul_CCTrCHList_02,
      { "ul-CCTrCHList", "rrc.ul_CCTrCHList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "UL_CCTrCHList_r7", HFILL }},
    { &hf_rrc_ul_DPCH_PowerControlInfo_05,
      { "ul-DPCH-PowerControlInfo", "rrc.ul_DPCH_PowerControlInfo",
        FT_UINT32, BASE_DEC, VALS(rrc_UL_DPCH_PowerControlInfo_r11_vals), 0,
        "UL_DPCH_PowerControlInfo_r11", HFILL }},
    { &hf_rrc_modeSpecificInfo_86,
      { "modeSpecificInfo", "rrc.modeSpecificInfo",
        FT_UINT32, BASE_DEC, VALS(rrc_T_modeSpecificInfo_86_vals), 0,
        "T_modeSpecificInfo_86", HFILL }},
    { &hf_rrc_fdd_121,
      { "fdd", "rrc.fdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_fdd_118", HFILL }},
    { &hf_rrc_dpdchPresence_02,
      { "dpdchPresence", "rrc.dpdchPresence",
        FT_UINT32, BASE_DEC, VALS(rrc_T_dpdchPresence_02_vals), 0,
        "T_dpdchPresence_02", HFILL }},
    { &hf_rrc_present_06,
      { "present", "rrc.present",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_present_04", HFILL }},
    { &hf_rrc_notPresent_02,
      { "notPresent", "rrc.notPresent",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_notPresent_02", HFILL }},
    { &hf_rrc_tdd_75,
      { "tdd", "rrc.tdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_tdd_69", HFILL }},
    { &hf_rrc_ul_DPCH_PowerControlInfo_06,
      { "ul-DPCH-PowerControlInfo", "rrc.ul_DPCH_PowerControlInfo",
        FT_NONE, BASE_NONE, NULL, 0,
        "UL_DPCH_PowerControlInfoPostFDD", HFILL }},
    { &hf_rrc_reducedScramblingCodeNumber,
      { "reducedScramblingCodeNumber", "rrc.reducedScramblingCodeNumber",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_ul_DPCH_PowerControlInfo_07,
      { "ul-DPCH-PowerControlInfo", "rrc.ul_DPCH_PowerControlInfo",
        FT_NONE, BASE_NONE, NULL, 0,
        "UL_DPCH_PowerControlInfoPostTDD", HFILL }},
    { &hf_rrc_ul_DPCH_PowerControlInfo_08,
      { "ul-DPCH-PowerControlInfo", "rrc.ul_DPCH_PowerControlInfo",
        FT_NONE, BASE_NONE, NULL, 0,
        "UL_DPCH_PowerControlInfoPostTDD_LCR_r4", HFILL }},
    { &hf_rrc_ul_TimingAdvance_04,
      { "ul-TimingAdvance", "rrc.ul_TimingAdvance",
        FT_UINT32, BASE_DEC, VALS(rrc_UL_TimingAdvanceControl_LCR_r4_vals), 0,
        "UL_TimingAdvanceControl_LCR_r4", HFILL }},
    { &hf_rrc_ul_DPCH_PowerControlInfo_09,
      { "ul-DPCH-PowerControlInfo", "rrc.ul_DPCH_PowerControlInfo",
        FT_UINT32, BASE_DEC, VALS(rrc_UL_DPCH_PowerControlInfoPredef_vals), 0,
        "UL_DPCH_PowerControlInfoPredef", HFILL }},
    { &hf_rrc_modeSpecificInfo_87,
      { "modeSpecificInfo", "rrc.modeSpecificInfo",
        FT_UINT32, BASE_DEC, VALS(rrc_T_modeSpecificInfo_87_vals), 0,
        "T_modeSpecificInfo_87", HFILL }},
    { &hf_rrc_fdd_122,
      { "fdd", "rrc.fdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_fdd_119", HFILL }},
    { &hf_rrc_tdd_76,
      { "tdd", "rrc.tdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_tdd_70", HFILL }},
    { &hf_rrc_modeSpecificInfo_88,
      { "modeSpecificInfo", "rrc.modeSpecificInfo",
        FT_UINT32, BASE_DEC, VALS(rrc_T_modeSpecificInfo_88_vals), 0,
        "T_modeSpecificInfo_88", HFILL }},
    { &hf_rrc_fdd_123,
      { "fdd", "rrc.fdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_fdd_120", HFILL }},
    { &hf_rrc_fdd_124,
      { "fdd", "rrc.fdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_fdd_121", HFILL }},
    { &hf_rrc_dpcch_PowerOffset,
      { "dpcch-PowerOffset", "rrc.dpcch_PowerOffset",
        FT_INT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_sRB_delay,
      { "sRB-delay", "rrc.sRB_delay",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_tdd_77,
      { "tdd", "rrc.tdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_tdd_71", HFILL }},
    { &hf_rrc_ul_OL_PC_Signalling,
      { "ul-OL-PC-Signalling", "rrc.ul_OL_PC_Signalling",
        FT_UINT32, BASE_DEC, VALS(rrc_T_ul_OL_PC_Signalling_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_broadcast_UL_OL_PC_info,
      { "broadcast-UL-OL-PC-info", "rrc.broadcast_UL_OL_PC_info",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_individuallySignalled,
      { "individuallySignalled", "rrc.individuallySignalled",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_individualTS_InterferenceList,
      { "individualTS-InterferenceList", "rrc.individualTS_InterferenceList",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_fdd_125,
      { "fdd", "rrc.fdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_fdd_122", HFILL }},
    { &hf_rrc_tdd_78,
      { "tdd", "rrc.tdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_tdd_72", HFILL }},
    { &hf_rrc_ul_OL_PC_Signalling_01,
      { "ul-OL-PC-Signalling", "rrc.ul_OL_PC_Signalling",
        FT_UINT32, BASE_DEC, VALS(rrc_T_ul_OL_PC_Signalling_01_vals), 0,
        "T_ul_OL_PC_Signalling_01", HFILL }},
    { &hf_rrc_individuallySignalled_01,
      { "individuallySignalled", "rrc.individuallySignalled",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_individuallySignalled_01", HFILL }},
    { &hf_rrc_tddOption_21,
      { "tddOption", "rrc.tddOption",
        FT_UINT32, BASE_DEC, VALS(rrc_T_tddOption_21_vals), 0,
        "T_tddOption_21", HFILL }},
    { &hf_rrc_tdd384_40,
      { "tdd384", "rrc.tdd384",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_tdd384_36", HFILL }},
    { &hf_rrc_dpch_ConstantValue_01,
      { "dpch-ConstantValue", "rrc.dpch_ConstantValue",
        FT_INT32, BASE_DEC, NULL, 0,
        "ConstantValue", HFILL }},
    { &hf_rrc_tdd128_57,
      { "tdd128", "rrc.tdd128",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_tdd128_55", HFILL }},
    { &hf_rrc_fdd_126,
      { "fdd", "rrc.fdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_fdd_123", HFILL }},
    { &hf_rrc_ack_NACK_repetition_factor,
      { "ack-NACK-repetition-factor", "rrc.ack_NACK_repetition_factor",
        FT_UINT32, BASE_DEC, NULL, 0,
        "ACK_NACK_repetitionFactor", HFILL }},
    { &hf_rrc_tdd_79,
      { "tdd", "rrc.tdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_tdd_73", HFILL }},
    { &hf_rrc_ul_OL_PC_Signalling_02,
      { "ul-OL-PC-Signalling", "rrc.ul_OL_PC_Signalling",
        FT_UINT32, BASE_DEC, VALS(rrc_T_ul_OL_PC_Signalling_02_vals), 0,
        "T_ul_OL_PC_Signalling_02", HFILL }},
    { &hf_rrc_individuallySignalled_02,
      { "individuallySignalled", "rrc.individuallySignalled",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_individuallySignalled_02", HFILL }},
    { &hf_rrc_tddOption_22,
      { "tddOption", "rrc.tddOption",
        FT_UINT32, BASE_DEC, VALS(rrc_T_tddOption_22_vals), 0,
        "T_tddOption_22", HFILL }},
    { &hf_rrc_tdd384_41,
      { "tdd384", "rrc.tdd384",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_tdd384_37", HFILL }},
    { &hf_rrc_tdd128_58,
      { "tdd128", "rrc.tdd128",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_tdd128_56", HFILL }},
    { &hf_rrc_fdd_127,
      { "fdd", "rrc.fdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_fdd_124", HFILL }},
    { &hf_rrc_tdd_80,
      { "tdd", "rrc.tdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_tdd_74", HFILL }},
    { &hf_rrc_ul_OL_PC_Signalling_03,
      { "ul-OL-PC-Signalling", "rrc.ul_OL_PC_Signalling",
        FT_UINT32, BASE_DEC, VALS(rrc_T_ul_OL_PC_Signalling_03_vals), 0,
        "T_ul_OL_PC_Signalling_03", HFILL }},
    { &hf_rrc_individuallySignalled_03,
      { "individuallySignalled", "rrc.individuallySignalled",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_individuallySignalled_03", HFILL }},
    { &hf_rrc_tddOption_23,
      { "tddOption", "rrc.tddOption",
        FT_UINT32, BASE_DEC, VALS(rrc_T_tddOption_23_vals), 0,
        "T_tddOption_23", HFILL }},
    { &hf_rrc_tdd384_42,
      { "tdd384", "rrc.tdd384",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_tdd384_38", HFILL }},
    { &hf_rrc_tdd128_59,
      { "tdd128", "rrc.tdd128",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_tdd128_57", HFILL }},
    { &hf_rrc_fdd_128,
      { "fdd", "rrc.fdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_fdd_125", HFILL }},
    { &hf_rrc_tdd_81,
      { "tdd", "rrc.tdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_tdd_75", HFILL }},
    { &hf_rrc_ul_OL_PC_Signalling_04,
      { "ul-OL-PC-Signalling", "rrc.ul_OL_PC_Signalling",
        FT_UINT32, BASE_DEC, VALS(rrc_T_ul_OL_PC_Signalling_04_vals), 0,
        "T_ul_OL_PC_Signalling_04", HFILL }},
    { &hf_rrc_individuallySignalled_04,
      { "individuallySignalled", "rrc.individuallySignalled",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_individuallySignalled_04", HFILL }},
    { &hf_rrc_tddOption_24,
      { "tddOption", "rrc.tddOption",
        FT_UINT32, BASE_DEC, VALS(rrc_T_tddOption_24_vals), 0,
        "T_tddOption_24", HFILL }},
    { &hf_rrc_tdd384_43,
      { "tdd384", "rrc.tdd384",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_tdd384_39", HFILL }},
    { &hf_rrc_tdd768_24,
      { "tdd768", "rrc.tdd768",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_tdd768_21", HFILL }},
    { &hf_rrc_tdd128_60,
      { "tdd128", "rrc.tdd128",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_tdd128_58", HFILL }},
    { &hf_rrc_fdd_129,
      { "fdd", "rrc.fdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_fdd_126", HFILL }},
    { &hf_rrc_ack_NACK_repetition_factor_assisting,
      { "ack-NACK-repetition-factor-assisting", "rrc.ack_NACK_repetition_factor_assisting",
        FT_UINT32, BASE_DEC, NULL, 0,
        "ACK_NACK_repetitionFactor", HFILL }},
    { &hf_rrc_tdd_82,
      { "tdd", "rrc.tdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_tdd_76", HFILL }},
    { &hf_rrc_ul_OL_PC_Signalling_05,
      { "ul-OL-PC-Signalling", "rrc.ul_OL_PC_Signalling",
        FT_UINT32, BASE_DEC, VALS(rrc_T_ul_OL_PC_Signalling_05_vals), 0,
        "T_ul_OL_PC_Signalling_05", HFILL }},
    { &hf_rrc_individuallySignalled_05,
      { "individuallySignalled", "rrc.individuallySignalled",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_individuallySignalled_05", HFILL }},
    { &hf_rrc_tddOption_25,
      { "tddOption", "rrc.tddOption",
        FT_UINT32, BASE_DEC, VALS(rrc_T_tddOption_25_vals), 0,
        "T_tddOption_25", HFILL }},
    { &hf_rrc_tdd384_44,
      { "tdd384", "rrc.tdd384",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_tdd384_40", HFILL }},
    { &hf_rrc_tdd768_25,
      { "tdd768", "rrc.tdd768",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_tdd768_22", HFILL }},
    { &hf_rrc_tdd128_61,
      { "tdd128", "rrc.tdd128",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_tdd128_59", HFILL }},
    { &hf_rrc_dpcch_PowerOffset_01,
      { "dpcch-PowerOffset", "rrc.dpcch_PowerOffset",
        FT_INT32, BASE_DEC, NULL, 0,
        "DPCCH_PowerOffset2", HFILL }},
    { &hf_rrc_fdd_130,
      { "fdd", "rrc.fdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_fdd_127", HFILL }},
    { &hf_rrc_tdd_83,
      { "tdd", "rrc.tdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_tdd_77", HFILL }},
    { &hf_rrc_tdd_84,
      { "tdd", "rrc.tdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_tdd_78", HFILL }},
    { &hf_rrc_non_ScheduledTransGrantInfo_02,
      { "non-ScheduledTransGrantInfo", "rrc.non_ScheduledTransGrantInfo",
        FT_UINT32, BASE_DEC, VALS(rrc_Non_ScheduledTransGrantInfoTDD_ext_vals), 0,
        "Non_ScheduledTransGrantInfoTDD_ext", HFILL }},
    { &hf_rrc_mac_es_e_resetIndicator_01,
      { "mac-es-e-resetIndicator", "rrc.mac_es_e_resetIndicator",
        FT_UINT32, BASE_DEC, VALS(rrc_T_mac_es_e_resetIndicator_01_vals), 0,
        "T_mac_es_e_resetIndicator_01", HFILL }},
    { &hf_rrc_e_DPCCH_Info,
      { "e-DPCCH-Info", "rrc.e_DPCCH_Info",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_e_DPDCH_Info,
      { "e-DPDCH-Info", "rrc.e_DPDCH_Info",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_schedulingTransmConfiguration,
      { "schedulingTransmConfiguration", "rrc.schedulingTransmConfiguration",
        FT_NONE, BASE_NONE, NULL, 0,
        "E_DPDCH_SchedulingTransmConfiguration", HFILL }},
    { &hf_rrc_mac_es_e_resetIndicator_02,
      { "mac-es-e-resetIndicator", "rrc.mac_es_e_resetIndicator",
        FT_UINT32, BASE_DEC, VALS(rrc_T_mac_es_e_resetIndicator_02_vals), 0,
        "T_mac_es_e_resetIndicator_02", HFILL }},
    { &hf_rrc_modeSpecificInfo_89,
      { "modeSpecificInfo", "rrc.modeSpecificInfo",
        FT_UINT32, BASE_DEC, VALS(rrc_T_modeSpecificInfo_89_vals), 0,
        "T_modeSpecificInfo_89", HFILL }},
    { &hf_rrc_fdd_131,
      { "fdd", "rrc.fdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_fdd_128", HFILL }},
    { &hf_rrc_e_DPCCH_Info_01,
      { "e-DPCCH-Info", "rrc.e_DPCCH_Info",
        FT_NONE, BASE_NONE, NULL, 0,
        "E_DPCCH_Info_r7", HFILL }},
    { &hf_rrc_e_DPDCH_Info_01,
      { "e-DPDCH-Info", "rrc.e_DPDCH_Info",
        FT_NONE, BASE_NONE, NULL, 0,
        "E_DPDCH_Info_r7", HFILL }},
    { &hf_rrc_tdd_85,
      { "tdd", "rrc.tdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_tdd_79", HFILL }},
    { &hf_rrc_e_RUCCH_Info_01,
      { "e-RUCCH-Info", "rrc.e_RUCCH_Info",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_e_PUCH_Info_01,
      { "e-PUCH-Info", "rrc.e_PUCH_Info",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_non_ScheduledTransGrantInfo_03,
      { "non-ScheduledTransGrantInfo", "rrc.non_ScheduledTransGrantInfo",
        FT_UINT32, BASE_DEC, VALS(rrc_Non_ScheduledTransGrantInfoTDD_vals), 0,
        "Non_ScheduledTransGrantInfoTDD", HFILL }},
    { &hf_rrc_mac_es_e_resetIndicator_03,
      { "mac-es-e-resetIndicator", "rrc.mac_es_e_resetIndicator",
        FT_UINT32, BASE_DEC, VALS(rrc_T_mac_es_e_resetIndicator_03_vals), 0,
        "T_mac_es_e_resetIndicator_03", HFILL }},
    { &hf_rrc_modeSpecificInfo_90,
      { "modeSpecificInfo", "rrc.modeSpecificInfo",
        FT_UINT32, BASE_DEC, VALS(rrc_T_modeSpecificInfo_90_vals), 0,
        "T_modeSpecificInfo_90", HFILL }},
    { &hf_rrc_fdd_132,
      { "fdd", "rrc.fdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_fdd_129", HFILL }},
    { &hf_rrc_e_DPDCH_Info_02,
      { "e-DPDCH-Info", "rrc.e_DPDCH_Info",
        FT_NONE, BASE_NONE, NULL, 0,
        "E_DPDCH_Info_r8", HFILL }},
    { &hf_rrc_tdd_86,
      { "tdd", "rrc.tdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_tdd_80", HFILL }},
    { &hf_rrc_mac_es_e_resetIndicator_04,
      { "mac-es-e-resetIndicator", "rrc.mac_es_e_resetIndicator",
        FT_UINT32, BASE_DEC, VALS(rrc_T_mac_es_e_resetIndicator_04_vals), 0,
        "T_mac_es_e_resetIndicator_04", HFILL }},
    { &hf_rrc_modeSpecificInfo_91,
      { "modeSpecificInfo", "rrc.modeSpecificInfo",
        FT_UINT32, BASE_DEC, VALS(rrc_T_modeSpecificInfo_91_vals), 0,
        "T_modeSpecificInfo_91", HFILL }},
    { &hf_rrc_fdd_133,
      { "fdd", "rrc.fdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_fdd_130", HFILL }},
    { &hf_rrc_tdd_87,
      { "tdd", "rrc.tdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_tdd_81", HFILL }},
    { &hf_rrc_non_ScheduledTransGrantInfo_04,
      { "non-ScheduledTransGrantInfo", "rrc.non_ScheduledTransGrantInfo",
        FT_UINT32, BASE_DEC, VALS(rrc_Non_ScheduledTransGrantInfoTDD_r9_vals), 0,
        "Non_ScheduledTransGrantInfoTDD_r9", HFILL }},
    { &hf_rrc_mac_es_e_resetIndicator_05,
      { "mac-es-e-resetIndicator", "rrc.mac_es_e_resetIndicator",
        FT_UINT32, BASE_DEC, VALS(rrc_T_mac_es_e_resetIndicator_05_vals), 0,
        "T_mac_es_e_resetIndicator_05", HFILL }},
    { &hf_rrc_modeSpecificInfo_92,
      { "modeSpecificInfo", "rrc.modeSpecificInfo",
        FT_UINT32, BASE_DEC, VALS(rrc_T_modeSpecificInfo_92_vals), 0,
        "T_modeSpecificInfo_92", HFILL }},
    { &hf_rrc_fdd_134,
      { "fdd", "rrc.fdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_fdd_131", HFILL }},
    { &hf_rrc_tdd_88,
      { "tdd", "rrc.tdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_tdd_82", HFILL }},
    { &hf_rrc_e_PUCH_info,
      { "e-PUCH-info", "rrc.e_PUCH_info",
        FT_NONE, BASE_NONE, NULL, 0,
        "E_PUCH_Info_MulticarrierEDCH_TDD128", HFILL }},
    { &hf_rrc_e_AGCH_info,
      { "e-AGCH-info", "rrc.e_AGCH_info",
        FT_NONE, BASE_NONE, NULL, 0,
        "E_AGCH_Information_TDD128", HFILL }},
    { &hf_rrc_e_HICH_info,
      { "e-HICH-info", "rrc.e_HICH_info",
        FT_NONE, BASE_NONE, NULL, 0,
        "E_HICH_Information_TDD128", HFILL }},
    { &hf_rrc_UL_MulticarrierEDCH_Infolist_TDD128_item,
      { "UL-MulticarrierEDCH-InfolistItem-TDD128", "rrc.UL_MulticarrierEDCH_InfolistItem_TDD128",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_UL_MulticarrierEDCH_Deletelist_TDD128_item,
      { "UARFCN", "rrc.UARFCN",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_configuration_info,
      { "configuration-info", "rrc.configuration_info",
        FT_UINT32, BASE_DEC, VALS(rrc_T_configuration_info_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_newConfiguration_13,
      { "newConfiguration", "rrc.newConfiguration",
        FT_NONE, BASE_NONE, NULL, 0,
        "UL_MulticarrierEDCH_NewConfigurationInfo_TDD128", HFILL }},
    { &hf_rrc_tsn_Length_01,
      { "tsn-Length", "rrc.tsn_Length",
        FT_UINT32, BASE_DEC, VALS(rrc_T_tsn_Length_01_vals), 0,
        "T_tsn_Length_01", HFILL }},
    { &hf_rrc_ul_MulticarrierEDCH_Infolist,
      { "ul-MulticarrierEDCH-Infolist", "rrc.ul_MulticarrierEDCH_Infolist",
        FT_UINT32, BASE_DEC, NULL, 0,
        "UL_MulticarrierEDCH_Infolist_TDD128", HFILL }},
    { &hf_rrc_ul_MulticarrierEDCH_Deletelist,
      { "ul-MulticarrierEDCH-Deletelist", "rrc.ul_MulticarrierEDCH_Deletelist",
        FT_UINT32, BASE_DEC, NULL, 0,
        "UL_MulticarrierEDCH_Deletelist_TDD128", HFILL }},
    { &hf_rrc_stepSize,
      { "stepSize", "rrc.stepSize",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_1_8", HFILL }},
    { &hf_rrc_frequency,
      { "frequency", "rrc.frequency",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_1_8", HFILL }},
    { &hf_rrc_disabled,
      { "disabled", "rrc.disabled",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_enabled,
      { "enabled", "rrc.enabled",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_enabled_01,
      { "enabled", "rrc.enabled",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_enabled_01", HFILL }},
    { &hf_rrc_tddOption_26,
      { "tddOption", "rrc.tddOption",
        FT_UINT32, BASE_DEC, VALS(rrc_T_tddOption_26_vals), 0,
        "T_tddOption_26", HFILL }},
    { &hf_rrc_tdd384_45,
      { "tdd384", "rrc.tdd384",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_tdd384_41", HFILL }},
    { &hf_rrc_tdd128_62,
      { "tdd128", "rrc.tdd128",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_tdd128_60", HFILL }},
    { &hf_rrc_synchronisationParameters,
      { "synchronisationParameters", "rrc.synchronisationParameters",
        FT_NONE, BASE_NONE, NULL, 0,
        "SynchronisationParameters_r4", HFILL }},
    { &hf_rrc_enabled_02,
      { "enabled", "rrc.enabled",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_enabled_02", HFILL }},
    { &hf_rrc_tddOption_27,
      { "tddOption", "rrc.tddOption",
        FT_UINT32, BASE_DEC, VALS(rrc_T_tddOption_27_vals), 0,
        "T_tddOption_27", HFILL }},
    { &hf_rrc_tdd384_46,
      { "tdd384", "rrc.tdd384",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_tdd384_42", HFILL }},
    { &hf_rrc_tdd768_26,
      { "tdd768", "rrc.tdd768",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_tdd768_23", HFILL }},
    { &hf_rrc_tdd128_63,
      { "tdd128", "rrc.tdd128",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_tdd128_61", HFILL }},
    { &hf_rrc_enabled_03,
      { "enabled", "rrc.enabled",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_enabled_03", HFILL }},
    { &hf_rrc_UL_TS_ChannelisationCodeList_item,
      { "UL-TS-ChannelisationCode", "rrc.UL_TS_ChannelisationCode",
        FT_UINT32, BASE_DEC|BASE_EXT_STRING, &rrc_UL_TS_ChannelisationCode_vals_ext, 0,
        NULL, HFILL }},
    { &hf_rrc_UL_TS_ChannelisationCodeList_r7_item,
      { "UL-TS-ChannelisationCodeList-r7 item", "rrc.UL_TS_ChannelisationCodeList_r7_item",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_ul_TS_Channelisation_Code,
      { "ul-TS-Channelisation-Code", "rrc.ul_TS_Channelisation_Code",
        FT_UINT32, BASE_DEC|BASE_EXT_STRING, &rrc_UL_TS_ChannelisationCode_vals_ext, 0,
        "UL_TS_ChannelisationCode", HFILL }},
    { &hf_rrc_plcch_info,
      { "plcch-info", "rrc.plcch_info",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_UL_TS_ChannelisationCodeList_VHCR_item,
      { "UL-TS-ChannelisationCode-VHCR", "rrc.UL_TS_ChannelisationCode_VHCR",
        FT_UINT32, BASE_DEC|BASE_EXT_STRING, &rrc_UL_TS_ChannelisationCode_VHCR_vals_ext, 0,
        NULL, HFILL }},
    { &hf_rrc_parameters_04,
      { "parameters", "rrc.parameters",
        FT_UINT32, BASE_DEC, VALS(rrc_T_parameters_04_vals), 0,
        "T_parameters_04", HFILL }},
    { &hf_rrc_sameAsLast_04,
      { "sameAsLast", "rrc.sameAsLast",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_sameAsLast_04", HFILL }},
    { &hf_rrc_newParameters_04,
      { "newParameters", "rrc.newParameters",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_newParameters_04", HFILL }},
    { &hf_rrc_ul_TS_ChannelisationCodeList,
      { "ul-TS-ChannelisationCodeList", "rrc.ul_TS_ChannelisationCodeList",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_parameters_05,
      { "parameters", "rrc.parameters",
        FT_UINT32, BASE_DEC, VALS(rrc_T_parameters_05_vals), 0,
        "T_parameters_05", HFILL }},
    { &hf_rrc_sameAsLast_05,
      { "sameAsLast", "rrc.sameAsLast",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_sameAsLast_05", HFILL }},
    { &hf_rrc_newParameters_05,
      { "newParameters", "rrc.newParameters",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_newParameters_05", HFILL }},
    { &hf_rrc_parameters_06,
      { "parameters", "rrc.parameters",
        FT_UINT32, BASE_DEC, VALS(rrc_T_parameters_06_vals), 0,
        "T_parameters_06", HFILL }},
    { &hf_rrc_sameAsLast_06,
      { "sameAsLast", "rrc.sameAsLast",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_sameAsLast_06", HFILL }},
    { &hf_rrc_newParameters_06,
      { "newParameters", "rrc.newParameters",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_newParameters_06", HFILL }},
    { &hf_rrc_ul_TS_ChannelisationCodeList_01,
      { "ul-TS-ChannelisationCodeList", "rrc.ul_TS_ChannelisationCodeList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "UL_TS_ChannelisationCodeList_r7", HFILL }},
    { &hf_rrc_parameters_07,
      { "parameters", "rrc.parameters",
        FT_UINT32, BASE_DEC, VALS(rrc_T_parameters_07_vals), 0,
        "T_parameters_07", HFILL }},
    { &hf_rrc_sameAsLast_07,
      { "sameAsLast", "rrc.sameAsLast",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_sameAsLast_07", HFILL }},
    { &hf_rrc_newParameters_07,
      { "newParameters", "rrc.newParameters",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_newParameters_07", HFILL }},
    { &hf_rrc_dynamicSFusage,
      { "dynamicSFusage", "rrc.dynamicSFusage",
        FT_BOOLEAN, BASE_NONE, NULL, 0,
        "BOOLEAN", HFILL }},
    { &hf_rrc_moreTimeslots_04,
      { "moreTimeslots", "rrc.moreTimeslots",
        FT_UINT32, BASE_DEC, VALS(rrc_T_moreTimeslots_04_vals), 0,
        "T_moreTimeslots_04", HFILL }},
    { &hf_rrc_additionalTimeslots_04,
      { "additionalTimeslots", "rrc.additionalTimeslots",
        FT_UINT32, BASE_DEC, VALS(rrc_T_additionalTimeslots_04_vals), 0,
        "T_additionalTimeslots_04", HFILL }},
    { &hf_rrc_consecutive_04,
      { "consecutive", "rrc.consecutive",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_consecutive_02", HFILL }},
    { &hf_rrc_numAdditionalTimeslots,
      { "numAdditionalTimeslots", "rrc.numAdditionalTimeslots",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_1_maxTS_1", HFILL }},
    { &hf_rrc_timeslotList_04,
      { "timeslotList", "rrc.timeslotList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "SEQUENCE_SIZE_1_maxTS_1_OF_UplinkAdditionalTimeslots", HFILL }},
    { &hf_rrc_timeslotList_item_04,
      { "UplinkAdditionalTimeslots", "rrc.UplinkAdditionalTimeslots",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_ul_TS_ChannelisationCodeList_02,
      { "ul-TS-ChannelisationCodeList", "rrc.ul_TS_ChannelisationCodeList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "UL_TS_ChannelisationCodeList_VHCR", HFILL }},
    { &hf_rrc_moreTimeslots_05,
      { "moreTimeslots", "rrc.moreTimeslots",
        FT_UINT32, BASE_DEC, VALS(rrc_T_moreTimeslots_05_vals), 0,
        "T_moreTimeslots_05", HFILL }},
    { &hf_rrc_additionalTimeslots_05,
      { "additionalTimeslots", "rrc.additionalTimeslots",
        FT_UINT32, BASE_DEC, VALS(rrc_T_additionalTimeslots_05_vals), 0,
        "T_additionalTimeslots_05", HFILL }},
    { &hf_rrc_consecutive_05,
      { "consecutive", "rrc.consecutive",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_consecutive_03", HFILL }},
    { &hf_rrc_timeslotList_05,
      { "timeslotList", "rrc.timeslotList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "SEQUENCE_SIZE_1_maxTS_1_OF_UplinkAdditionalTimeslots_VHCR", HFILL }},
    { &hf_rrc_timeslotList_item_05,
      { "UplinkAdditionalTimeslots-VHCR", "rrc.UplinkAdditionalTimeslots_VHCR",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_moreTimeslots_06,
      { "moreTimeslots", "rrc.moreTimeslots",
        FT_UINT32, BASE_DEC, VALS(rrc_T_moreTimeslots_06_vals), 0,
        "T_moreTimeslots_06", HFILL }},
    { &hf_rrc_additionalTimeslots_06,
      { "additionalTimeslots", "rrc.additionalTimeslots",
        FT_UINT32, BASE_DEC, VALS(rrc_T_additionalTimeslots_06_vals), 0,
        "T_additionalTimeslots_06", HFILL }},
    { &hf_rrc_consecutive_06,
      { "consecutive", "rrc.consecutive",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_consecutive_04", HFILL }},
    { &hf_rrc_numAdditionalTimeslots_01,
      { "numAdditionalTimeslots", "rrc.numAdditionalTimeslots",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_1_maxTS_LCR_1", HFILL }},
    { &hf_rrc_timeslotList_06,
      { "timeslotList", "rrc.timeslotList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "SEQUENCE_SIZE_1_maxTS_LCR_1_OF_UplinkAdditionalTimeslots_LCR_r4", HFILL }},
    { &hf_rrc_timeslotList_item_06,
      { "UplinkAdditionalTimeslots-LCR-r4", "rrc.UplinkAdditionalTimeslots_LCR_r4",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_moreTimeslots_07,
      { "moreTimeslots", "rrc.moreTimeslots",
        FT_UINT32, BASE_DEC, VALS(rrc_T_moreTimeslots_07_vals), 0,
        "T_moreTimeslots_07", HFILL }},
    { &hf_rrc_additionalTimeslots_07,
      { "additionalTimeslots", "rrc.additionalTimeslots",
        FT_UINT32, BASE_DEC, VALS(rrc_T_additionalTimeslots_07_vals), 0,
        "T_additionalTimeslots_07", HFILL }},
    { &hf_rrc_consecutive_07,
      { "consecutive", "rrc.consecutive",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_consecutive_05", HFILL }},
    { &hf_rrc_timeslotList_07,
      { "timeslotList", "rrc.timeslotList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "SEQUENCE_SIZE_1_maxTS_LCR_1_OF_UplinkAdditionalTimeslots_LCR_r7", HFILL }},
    { &hf_rrc_timeslotList_item_07,
      { "UplinkAdditionalTimeslots-LCR-r7", "rrc.UplinkAdditionalTimeslots_LCR_r7",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_satID,
      { "satID", "rrc.satID",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_doppler0thOrder,
      { "doppler0thOrder", "rrc.doppler0thOrder",
        FT_INT32, BASE_DEC, NULL, 0,
        "INTEGER_M2048_2047", HFILL }},
    { &hf_rrc_extraDopplerInfo,
      { "extraDopplerInfo", "rrc.extraDopplerInfo",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_codePhase,
      { "codePhase", "rrc.codePhase",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_1022", HFILL }},
    { &hf_rrc_integerCodePhase,
      { "integerCodePhase", "rrc.integerCodePhase",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_19", HFILL }},
    { &hf_rrc_gps_BitNumber,
      { "gps-BitNumber", "rrc.gps_BitNumber",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_3", HFILL }},
    { &hf_rrc_codePhaseSearchWindow,
      { "codePhaseSearchWindow", "rrc.codePhaseSearchWindow",
        FT_UINT32, BASE_DEC, VALS(rrc_CodePhaseSearchWindow_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_azimuthAndElevation,
      { "azimuthAndElevation", "rrc.azimuthAndElevation",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_azimuthAndElevation_01,
      { "azimuthAndElevation", "rrc.azimuthAndElevation",
        FT_NONE, BASE_NONE, NULL, 0,
        "AzimuthAndElevation_va40ext", HFILL }},
    { &hf_rrc_azimuthAndElevation_02,
      { "azimuthAndElevation", "rrc.azimuthAndElevation",
        FT_NONE, BASE_NONE, NULL, 0,
        "AzimuthAndElevation_r10", HFILL }},
    { &hf_rrc_AcquisitionSatInfoList_item,
      { "AcquisitionSatInfo", "rrc.AcquisitionSatInfo",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_AcquisitionSatInfoList_va40ext_item,
      { "AcquisitionSatInfo-va40ext", "rrc.AcquisitionSatInfo_va40ext",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_AcquisitionSatInfoList_r10_item,
      { "AcquisitionSatInfo-r10", "rrc.AcquisitionSatInfo_r10",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_AdditionalMeasurementID_List_item,
      { "MeasurementIdentity", "rrc.MeasurementIdentity",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_AdditionalMeasurementID_List_r9_item,
      { "MeasurementIdentity-r9", "rrc.MeasurementIdentity_r9",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_dataID,
      { "dataID", "rrc.dataID",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_3", HFILL }},
    { &hf_rrc_e,
      { "e", "rrc.e",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_16", HFILL }},
    { &hf_rrc_t_oa,
      { "t-oa", "rrc.t_oa",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_8", HFILL }},
    { &hf_rrc_deltaI,
      { "deltaI", "rrc.deltaI",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_16", HFILL }},
    { &hf_rrc_omegaDot,
      { "omegaDot", "rrc.omegaDot",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_16", HFILL }},
    { &hf_rrc_satHealth,
      { "satHealth", "rrc.satHealth",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_8", HFILL }},
    { &hf_rrc_a_Sqrt,
      { "a-Sqrt", "rrc.a_Sqrt",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_24", HFILL }},
    { &hf_rrc_omega0,
      { "omega0", "rrc.omega0",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_24", HFILL }},
    { &hf_rrc_m0,
      { "m0", "rrc.m0",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_24", HFILL }},
    { &hf_rrc_omega,
      { "omega", "rrc.omega",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_24", HFILL }},
    { &hf_rrc_af0,
      { "af0", "rrc.af0",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_11", HFILL }},
    { &hf_rrc_af1,
      { "af1", "rrc.af1",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_11", HFILL }},
    { &hf_rrc_AlmanacSatInfoList_item,
      { "AlmanacSatInfo", "rrc.AlmanacSatInfo",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_sat_info_SBASecefList,
      { "sat-info-SBASecefList", "rrc.sat_info_SBASecefList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "GANSS_SAT_Info_Almanac_SBASecefList", HFILL }},
    { &hf_rrc_sat_info_GLOkpList,
      { "sat-info-GLOkpList", "rrc.sat_info_GLOkpList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "GANSS_SAT_Info_Almanac_GLOkpList", HFILL }},
    { &hf_rrc_t_oa_01,
      { "t-oa", "rrc.t_oa",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_147", HFILL }},
    { &hf_rrc_iod_a,
      { "iod-a", "rrc.iod_a",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_3", HFILL }},
    { &hf_rrc_sat_info_kpList,
      { "sat-info-kpList", "rrc.sat_info_kpList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "GANSS_SAT_Info_Almanac_KpList", HFILL }},
    { &hf_rrc_sat_info_MIDIkpList,
      { "sat-info-MIDIkpList", "rrc.sat_info_MIDIkpList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "GANSS_SAT_Info_Almanac_MIDIkpList", HFILL }},
    { &hf_rrc_sat_info_NAVkpList,
      { "sat-info-NAVkpList", "rrc.sat_info_NAVkpList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "GANSS_SAT_Info_Almanac_NAVkpList", HFILL }},
    { &hf_rrc_sat_info_REDkpList,
      { "sat-info-REDkpList", "rrc.sat_info_REDkpList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "GANSS_SAT_Info_Almanac_REDkpList", HFILL }},
    { &hf_rrc_AuxInfoGANSS_ID1_item,
      { "AuxInfoGANSS-ID1-element", "rrc.AuxInfoGANSS_ID1_element",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_svID,
      { "svID", "rrc.svID",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_63", HFILL }},
    { &hf_rrc_signalsAvailable,
      { "signalsAvailable", "rrc.signalsAvailable",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_8", HFILL }},
    { &hf_rrc_AuxInfoGANSS_ID3_item,
      { "AuxInfoGANSS-ID3-element", "rrc.AuxInfoGANSS_ID3_element",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_channelNumber,
      { "channelNumber", "rrc.channelNumber",
        FT_INT32, BASE_DEC, NULL, 0,
        "INTEGER_M7_13", HFILL }},
    { &hf_rrc_azimuth,
      { "azimuth", "rrc.azimuth",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_31", HFILL }},
    { &hf_rrc_elevation,
      { "elevation", "rrc.elevation",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_7", HFILL }},
    { &hf_rrc_azimuthLSB,
      { "azimuthLSB", "rrc.azimuthLSB",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_15", HFILL }},
    { &hf_rrc_elevationLSB,
      { "elevationLSB", "rrc.elevationLSB",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_15", HFILL }},
    { &hf_rrc_BadSatList_item,
      { "BadSatList item", "rrc.BadSatList_item",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_63", HFILL }},
    { &hf_rrc_dl_TransportChannelBLER,
      { "dl-TransportChannelBLER", "rrc.dl_TransportChannelBLER",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_BLER_MeasurementResultsList_item,
      { "BLER-MeasurementResults", "rrc.BLER_MeasurementResults",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_BLER_TransChIdList_item,
      { "TransportChannelIdentity", "rrc.TransportChannelIdentity",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_verifiedBSIC,
      { "verifiedBSIC", "rrc.verifiedBSIC",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_maxCellMeas", HFILL }},
    { &hf_rrc_nonVerifiedBSIC,
      { "nonVerifiedBSIC", "rrc.nonVerifiedBSIC",
        FT_UINT32, BASE_DEC, NULL, 0,
        "BCCH_ARFCN", HFILL }},
    { &hf_rrc_burstStart,
      { "burstStart", "rrc.burstStart",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_15", HFILL }},
    { &hf_rrc_burstLength,
      { "burstLength", "rrc.burstLength",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_10_25", HFILL }},
    { &hf_rrc_burstFreq,
      { "burstFreq", "rrc.burstFreq",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_1_16", HFILL }},
    { &hf_rrc_intraFreqReportingCriteria,
      { "intraFreqReportingCriteria", "rrc.intraFreqReportingCriteria",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_periodicalReportingCriteria,
      { "periodicalReportingCriteria", "rrc.periodicalReportingCriteria",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_intraFreqReportingCriteria_01,
      { "intraFreqReportingCriteria", "rrc.intraFreqReportingCriteria",
        FT_NONE, BASE_NONE, NULL, 0,
        "IntraFreqReportingCriteria_LCR_r4", HFILL }},
    { &hf_rrc_patternIdentifier,
      { "patternIdentifier", "rrc.patternIdentifier",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_maxMeasOccasionPattern_1", HFILL }},
    { &hf_rrc_statusFlag,
      { "statusFlag", "rrc.statusFlag",
        FT_UINT32, BASE_DEC, VALS(rrc_T_statusFlag_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_measurementPurpose,
      { "measurementPurpose", "rrc.measurementPurpose",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_5", HFILL }},
    { &hf_rrc_measurementOccasionPatternParameter,
      { "measurementOccasionPatternParameter", "rrc.measurementOccasionPatternParameter",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_cellDCHMeasOccasionSequenceList,
      { "cellDCHMeasOccasionSequenceList", "rrc.cellDCHMeasOccasionSequenceList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "SEQUENCE_SIZE_1_maxMeasOccasionPattern_OF_CellDCHMeasOccasionPattern_LCR", HFILL }},
    { &hf_rrc_cellDCHMeasOccasionSequenceList_item,
      { "CellDCHMeasOccasionPattern-LCR", "rrc.CellDCHMeasOccasionPattern_LCR",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_cellIndividualOffset,
      { "cellIndividualOffset", "rrc.cellIndividualOffset",
        FT_INT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_referenceTimeDifferenceToCell,
      { "referenceTimeDifferenceToCell", "rrc.referenceTimeDifferenceToCell",
        FT_UINT32, BASE_DEC, VALS(rrc_ReferenceTimeDifferenceToCell_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_modeSpecificInfo_93,
      { "modeSpecificInfo", "rrc.modeSpecificInfo",
        FT_UINT32, BASE_DEC, VALS(rrc_T_modeSpecificInfo_93_vals), 0,
        "T_modeSpecificInfo_93", HFILL }},
    { &hf_rrc_fdd_135,
      { "fdd", "rrc.fdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_fdd_132", HFILL }},
    { &hf_rrc_readSFN_Indicator,
      { "readSFN-Indicator", "rrc.readSFN_Indicator",
        FT_BOOLEAN, BASE_NONE, NULL, 0,
        "BOOLEAN", HFILL }},
    { &hf_rrc_tdd_89,
      { "tdd", "rrc.tdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_tdd_83", HFILL }},
    { &hf_rrc_primaryCCPCH_Info_02,
      { "primaryCCPCH-Info", "rrc.primaryCCPCH_Info",
        FT_UINT32, BASE_DEC, VALS(rrc_PrimaryCCPCH_Info_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_timeslotInfoList,
      { "timeslotInfoList", "rrc.timeslotInfoList",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_modeSpecificInfo_94,
      { "modeSpecificInfo", "rrc.modeSpecificInfo",
        FT_UINT32, BASE_DEC, VALS(rrc_T_modeSpecificInfo_94_vals), 0,
        "T_modeSpecificInfo_94", HFILL }},
    { &hf_rrc_fdd_136,
      { "fdd", "rrc.fdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_fdd_133", HFILL }},
    { &hf_rrc_tdd_90,
      { "tdd", "rrc.tdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_tdd_84", HFILL }},
    { &hf_rrc_primaryCCPCH_Info_03,
      { "primaryCCPCH-Info", "rrc.primaryCCPCH_Info",
        FT_UINT32, BASE_DEC, VALS(rrc_PrimaryCCPCH_Info_r4_vals), 0,
        "PrimaryCCPCH_Info_r4", HFILL }},
    { &hf_rrc_timeslotInfoList_01,
      { "timeslotInfoList", "rrc.timeslotInfoList",
        FT_UINT32, BASE_DEC, VALS(rrc_TimeslotInfoList_r4_vals), 0,
        "TimeslotInfoList_r4", HFILL }},
    { &hf_rrc_cellSelectionReselectionInfo,
      { "cellSelectionReselectionInfo", "rrc.cellSelectionReselectionInfo",
        FT_NONE, BASE_NONE, NULL, 0,
        "CellSelectReselectInfoMC_RSCP", HFILL }},
    { &hf_rrc_modeSpecificInfo_95,
      { "modeSpecificInfo", "rrc.modeSpecificInfo",
        FT_UINT32, BASE_DEC, VALS(rrc_T_modeSpecificInfo_95_vals), 0,
        "T_modeSpecificInfo_95", HFILL }},
    { &hf_rrc_fdd_137,
      { "fdd", "rrc.fdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_fdd_134", HFILL }},
    { &hf_rrc_tdd_91,
      { "tdd", "rrc.tdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_tdd_85", HFILL }},
    { &hf_rrc_CellInfoListToBeExcluded_r11_item,
      { "PrimaryCPICH-Info", "rrc.PrimaryCPICH_Info",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_modeSpecificInfo_96,
      { "modeSpecificInfo", "rrc.modeSpecificInfo",
        FT_UINT32, BASE_DEC, VALS(rrc_T_modeSpecificInfo_96_vals), 0,
        "T_modeSpecificInfo_96", HFILL }},
    { &hf_rrc_fdd_138,
      { "fdd", "rrc.fdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_fdd_135", HFILL }},
    { &hf_rrc_tdd_92,
      { "tdd", "rrc.tdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_tdd_86", HFILL }},
    { &hf_rrc_cellSelectionReselectionInfo_01,
      { "cellSelectionReselectionInfo", "rrc.cellSelectionReselectionInfo",
        FT_NONE, BASE_NONE, NULL, 0,
        "CellSelectReselectInfoSIB_11_12_RSCP", HFILL }},
    { &hf_rrc_primaryCCPCH_Info_04,
      { "primaryCCPCH-Info", "rrc.primaryCCPCH_Info",
        FT_NONE, BASE_NONE, NULL, 0,
        "PrimaryCCPCH_Info_LCR_r4", HFILL }},
    { &hf_rrc_timeslotInfoList_02,
      { "timeslotInfoList", "rrc.timeslotInfoList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "TimeslotInfoList_LCR_r4", HFILL }},
    { &hf_rrc_modeSpecificInfo_97,
      { "modeSpecificInfo", "rrc.modeSpecificInfo",
        FT_UINT32, BASE_DEC, VALS(rrc_T_modeSpecificInfo_97_vals), 0,
        "T_modeSpecificInfo_97", HFILL }},
    { &hf_rrc_fdd_139,
      { "fdd", "rrc.fdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_fdd_136", HFILL }},
    { &hf_rrc_tdd_93,
      { "tdd", "rrc.tdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_tdd_87", HFILL }},
    { &hf_rrc_cellSelectionReselectionInfo_02,
      { "cellSelectionReselectionInfo", "rrc.cellSelectionReselectionInfo",
        FT_NONE, BASE_NONE, NULL, 0,
        "CellSelectReselectInfoSIB_11_12_ECN0", HFILL }},
    { &hf_rrc_modeSpecificInfo_98,
      { "modeSpecificInfo", "rrc.modeSpecificInfo",
        FT_UINT32, BASE_DEC, VALS(rrc_T_modeSpecificInfo_98_vals), 0,
        "T_modeSpecificInfo_98", HFILL }},
    { &hf_rrc_fdd_140,
      { "fdd", "rrc.fdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_fdd_137", HFILL }},
    { &hf_rrc_tdd_94,
      { "tdd", "rrc.tdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_tdd_88", HFILL }},
    { &hf_rrc_cellSelectionReselectionInfo_03,
      { "cellSelectionReselectionInfo", "rrc.cellSelectionReselectionInfo",
        FT_NONE, BASE_NONE, NULL, 0,
        "CellSelectReselectInfoSIB_11_12_HCS_RSCP", HFILL }},
    { &hf_rrc_modeSpecificInfo_99,
      { "modeSpecificInfo", "rrc.modeSpecificInfo",
        FT_UINT32, BASE_DEC, VALS(rrc_T_modeSpecificInfo_99_vals), 0,
        "T_modeSpecificInfo_99", HFILL }},
    { &hf_rrc_fdd_141,
      { "fdd", "rrc.fdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_fdd_138", HFILL }},
    { &hf_rrc_tdd_95,
      { "tdd", "rrc.tdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_tdd_89", HFILL }},
    { &hf_rrc_cellSelectionReselectionInfo_04,
      { "cellSelectionReselectionInfo", "rrc.cellSelectionReselectionInfo",
        FT_NONE, BASE_NONE, NULL, 0,
        "CellSelectReselectInfoSIB_11_12_HCS_ECN0", HFILL }},
    { &hf_rrc_cellIdentity,
      { "cellIdentity", "rrc.cellIdentity",
        FT_BYTES, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_dummy_48,
      { "dummy", "rrc.dummy",
        FT_UINT32, BASE_DEC, VALS(rrc_SFN_SFN_ObsTimeDifference_vals), 0,
        "SFN_SFN_ObsTimeDifference", HFILL }},
    { &hf_rrc_cellSynchronisationInfo,
      { "cellSynchronisationInfo", "rrc.cellSynchronisationInfo",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_modeSpecificInfo_100,
      { "modeSpecificInfo", "rrc.modeSpecificInfo",
        FT_UINT32, BASE_DEC, VALS(rrc_T_modeSpecificInfo_100_vals), 0,
        "T_modeSpecificInfo_100", HFILL }},
    { &hf_rrc_fdd_142,
      { "fdd", "rrc.fdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_fdd_139", HFILL }},
    { &hf_rrc_cpich_Ec_N0_01,
      { "cpich-Ec-N0", "rrc.cpich_Ec_N0",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_cpich_RSCP_01,
      { "cpich-RSCP", "rrc.cpich_RSCP",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_pathloss,
      { "pathloss", "rrc.pathloss",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_tdd_96,
      { "tdd", "rrc.tdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_tdd_90", HFILL }},
    { &hf_rrc_proposedTGSN,
      { "proposedTGSN", "rrc.proposedTGSN",
        FT_UINT32, BASE_DEC, NULL, 0,
        "TGSN", HFILL }},
    { &hf_rrc_timeslotISCP_List,
      { "timeslotISCP-List", "rrc.timeslotISCP_List",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_modeSpecificInfo_101,
      { "modeSpecificInfo", "rrc.modeSpecificInfo",
        FT_UINT32, BASE_DEC, VALS(rrc_T_modeSpecificInfo_101_vals), 0,
        "T_modeSpecificInfo_101", HFILL }},
    { &hf_rrc_fdd_143,
      { "fdd", "rrc.fdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_fdd_140", HFILL }},
    { &hf_rrc_deltaRSCPPerCell,
      { "deltaRSCPPerCell", "rrc.deltaRSCPPerCell",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_tdd_97,
      { "tdd", "rrc.tdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_tdd_91", HFILL }},
    { &hf_rrc_csgIdentity,
      { "csgIdentity", "rrc.csgIdentity",
        FT_BYTES, BASE_NONE, NULL, 0,
        "CSG_Identity", HFILL }},
    { &hf_rrc_csgMemberIndication,
      { "csgMemberIndication", "rrc.csgMemberIndication",
        FT_UINT32, BASE_DEC, VALS(rrc_T_csgMemberIndication_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_fdd_144,
      { "fdd", "rrc.fdd",
        FT_UINT32, BASE_DEC, NULL, 0,
        "SEQUENCE_SIZE_1_maxCellMeas_OF_PrimaryCPICH_Info", HFILL }},
    { &hf_rrc_fdd_item_01,
      { "PrimaryCPICH-Info", "rrc.PrimaryCPICH_Info",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_tdd_98,
      { "tdd", "rrc.tdd",
        FT_UINT32, BASE_DEC, NULL, 0,
        "SEQUENCE_SIZE_1_maxCellMeas_OF_PrimaryCCPCH_Info", HFILL }},
    { &hf_rrc_tdd_item_02,
      { "PrimaryCCPCH-Info", "rrc.PrimaryCCPCH_Info",
        FT_UINT32, BASE_DEC, VALS(rrc_PrimaryCCPCH_Info_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_fdd_145,
      { "fdd", "rrc.fdd",
        FT_UINT32, BASE_DEC, NULL, 0,
        "SEQUENCE_SIZE_1_maxCellMeasOnSecULFreq_OF_PrimaryCPICH_Info", HFILL }},
    { &hf_rrc_CellMeasurementEventResults_LCR_r4_item,
      { "PrimaryCCPCH-Info-LCR-r4", "rrc.PrimaryCCPCH_Info_LCR_r4",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_dummy_49,
      { "dummy", "rrc.dummy",
        FT_UINT32, BASE_DEC, VALS(rrc_SFN_SFN_OTD_Type_vals), 0,
        "SFN_SFN_OTD_Type", HFILL }},
    { &hf_rrc_cellIdentity_reportingIndicator,
      { "cellIdentity-reportingIndicator", "rrc.cellIdentity_reportingIndicator",
        FT_BOOLEAN, BASE_NONE, NULL, 0,
        "BOOLEAN", HFILL }},
    { &hf_rrc_cellSynchronisationInfoReportingIndicator,
      { "cellSynchronisationInfoReportingIndicator", "rrc.cellSynchronisationInfoReportingIndicator",
        FT_BOOLEAN, BASE_NONE, NULL, 0,
        "BOOLEAN", HFILL }},
    { &hf_rrc_modeSpecificInfo_102,
      { "modeSpecificInfo", "rrc.modeSpecificInfo",
        FT_UINT32, BASE_DEC, VALS(rrc_T_modeSpecificInfo_102_vals), 0,
        "T_modeSpecificInfo_102", HFILL }},
    { &hf_rrc_fdd_146,
      { "fdd", "rrc.fdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_fdd_141", HFILL }},
    { &hf_rrc_cpich_Ec_N0_reportingIndicator,
      { "cpich-Ec-N0-reportingIndicator", "rrc.cpich_Ec_N0_reportingIndicator",
        FT_BOOLEAN, BASE_NONE, NULL, 0,
        "BOOLEAN", HFILL }},
    { &hf_rrc_cpich_RSCP_reportingIndicator,
      { "cpich-RSCP-reportingIndicator", "rrc.cpich_RSCP_reportingIndicator",
        FT_BOOLEAN, BASE_NONE, NULL, 0,
        "BOOLEAN", HFILL }},
    { &hf_rrc_pathloss_reportingIndicator,
      { "pathloss-reportingIndicator", "rrc.pathloss_reportingIndicator",
        FT_BOOLEAN, BASE_NONE, NULL, 0,
        "BOOLEAN", HFILL }},
    { &hf_rrc_tdd_99,
      { "tdd", "rrc.tdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_tdd_92", HFILL }},
    { &hf_rrc_timeslotISCP_reportingIndicator,
      { "timeslotISCP-reportingIndicator", "rrc.timeslotISCP_reportingIndicator",
        FT_BOOLEAN, BASE_NONE, NULL, 0,
        "BOOLEAN", HFILL }},
    { &hf_rrc_proposedTGSN_ReportingRequired,
      { "proposedTGSN-ReportingRequired", "rrc.proposedTGSN_ReportingRequired",
        FT_BOOLEAN, BASE_NONE, NULL, 0,
        "BOOLEAN", HFILL }},
    { &hf_rrc_primaryCCPCH_RSCP_reportingIndicator,
      { "primaryCCPCH-RSCP-reportingIndicator", "rrc.primaryCCPCH_RSCP_reportingIndicator",
        FT_BOOLEAN, BASE_NONE, NULL, 0,
        "BOOLEAN", HFILL }},
    { &hf_rrc_q_OffsetS_N,
      { "q-OffsetS-N", "rrc.q_OffsetS_N",
        FT_INT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_hcs_NeighbouringCellInformation_RSCP,
      { "hcs-NeighbouringCellInformation-RSCP", "rrc.hcs_NeighbouringCellInformation_RSCP",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_modeSpecificInfo_103,
      { "modeSpecificInfo", "rrc.modeSpecificInfo",
        FT_UINT32, BASE_DEC, VALS(rrc_T_modeSpecificInfo_103_vals), 0,
        "T_modeSpecificInfo_103", HFILL }},
    { &hf_rrc_fdd_147,
      { "fdd", "rrc.fdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_fdd_142", HFILL }},
    { &hf_rrc_tdd_100,
      { "tdd", "rrc.tdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_tdd_93", HFILL }},
    { &hf_rrc_gsm_02,
      { "gsm", "rrc.gsm",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_gsm_02", HFILL }},
    { &hf_rrc_q_Offset1S_N,
      { "q-Offset1S-N", "rrc.q_Offset1S_N",
        FT_INT32, BASE_DEC, NULL, 0,
        "Q_OffsetS_N", HFILL }},
    { &hf_rrc_q_Offset2S_N,
      { "q-Offset2S-N", "rrc.q_Offset2S_N",
        FT_INT32, BASE_DEC, NULL, 0,
        "Q_OffsetS_N", HFILL }},
    { &hf_rrc_modeSpecificInfo_104,
      { "modeSpecificInfo", "rrc.modeSpecificInfo",
        FT_UINT32, BASE_DEC, VALS(rrc_T_modeSpecificInfo_104_vals), 0,
        "T_modeSpecificInfo_104", HFILL }},
    { &hf_rrc_fdd_148,
      { "fdd", "rrc.fdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_fdd_143", HFILL }},
    { &hf_rrc_tdd_101,
      { "tdd", "rrc.tdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_tdd_94", HFILL }},
    { &hf_rrc_gsm_03,
      { "gsm", "rrc.gsm",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_gsm_03", HFILL }},
    { &hf_rrc_modeSpecificInfo_105,
      { "modeSpecificInfo", "rrc.modeSpecificInfo",
        FT_UINT32, BASE_DEC, VALS(rrc_T_modeSpecificInfo_105_vals), 0,
        "T_modeSpecificInfo_105", HFILL }},
    { &hf_rrc_fdd_149,
      { "fdd", "rrc.fdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_fdd_144", HFILL }},
    { &hf_rrc_tdd_102,
      { "tdd", "rrc.tdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_tdd_95", HFILL }},
    { &hf_rrc_gsm_04,
      { "gsm", "rrc.gsm",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_gsm_04", HFILL }},
    { &hf_rrc_modeSpecificInfo_106,
      { "modeSpecificInfo", "rrc.modeSpecificInfo",
        FT_UINT32, BASE_DEC, VALS(rrc_T_modeSpecificInfo_106_vals), 0,
        "T_modeSpecificInfo_106", HFILL }},
    { &hf_rrc_fdd_150,
      { "fdd", "rrc.fdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_fdd_145", HFILL }},
    { &hf_rrc_tdd_103,
      { "tdd", "rrc.tdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_tdd_96", HFILL }},
    { &hf_rrc_gsm_05,
      { "gsm", "rrc.gsm",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_gsm_05", HFILL }},
    { &hf_rrc_modeSpecificInfo_107,
      { "modeSpecificInfo", "rrc.modeSpecificInfo",
        FT_UINT32, BASE_DEC, VALS(rrc_T_modeSpecificInfo_107_vals), 0,
        "T_modeSpecificInfo_107", HFILL }},
    { &hf_rrc_fdd_151,
      { "fdd", "rrc.fdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_fdd_146", HFILL }},
    { &hf_rrc_tdd_104,
      { "tdd", "rrc.tdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_tdd_97", HFILL }},
    { &hf_rrc_gsm_06,
      { "gsm", "rrc.gsm",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_gsm_06", HFILL }},
    { &hf_rrc_hcs_NeighbouringCellInformation_ECN0,
      { "hcs-NeighbouringCellInformation-ECN0", "rrc.hcs_NeighbouringCellInformation_ECN0",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_modeSpecificInfo_108,
      { "modeSpecificInfo", "rrc.modeSpecificInfo",
        FT_UINT32, BASE_DEC, VALS(rrc_T_modeSpecificInfo_108_vals), 0,
        "T_modeSpecificInfo_108", HFILL }},
    { &hf_rrc_fdd_152,
      { "fdd", "rrc.fdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_fdd_147", HFILL }},
    { &hf_rrc_tdd_105,
      { "tdd", "rrc.tdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_tdd_98", HFILL }},
    { &hf_rrc_gsm_07,
      { "gsm", "rrc.gsm",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_gsm_07", HFILL }},
    { &hf_rrc_deltaQrxlevmin,
      { "deltaQrxlevmin", "rrc.deltaQrxlevmin",
        FT_INT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_deltaQhcs,
      { "deltaQhcs", "rrc.deltaQhcs",
        FT_INT32, BASE_DEC, NULL, 0,
        "DeltaRSCP", HFILL }},
    { &hf_rrc_q_Hyst_l_S_PCH,
      { "q-Hyst-l-S-PCH", "rrc.q_Hyst_l_S_PCH",
        FT_UINT32, BASE_DEC, NULL, 0,
        "Q_Hyst_S_Fine", HFILL }},
    { &hf_rrc_q_Hyst_l_S_FACH,
      { "q-Hyst-l-S-FACH", "rrc.q_Hyst_l_S_FACH",
        FT_UINT32, BASE_DEC, NULL, 0,
        "Q_Hyst_S_Fine", HFILL }},
    { &hf_rrc_q_Hyst_2_S_PCH,
      { "q-Hyst-2-S-PCH", "rrc.q_Hyst_2_S_PCH",
        FT_UINT32, BASE_DEC, NULL, 0,
        "Q_Hyst_S_Fine", HFILL }},
    { &hf_rrc_q_Hyst_2_S_FACH,
      { "q-Hyst-2-S-FACH", "rrc.q_Hyst_2_S_FACH",
        FT_UINT32, BASE_DEC, NULL, 0,
        "Q_Hyst_S_Fine", HFILL }},
    { &hf_rrc_t_Reselection_S_PCH,
      { "t-Reselection-S-PCH", "rrc.t_Reselection_S_PCH",
        FT_UINT32, BASE_DEC, NULL, 0,
        "T_Reselection_S", HFILL }},
    { &hf_rrc_t_Reselection_S_FACH,
      { "t-Reselection-S-FACH", "rrc.t_Reselection_S_FACH",
        FT_UINT32, BASE_DEC, NULL, 0,
        "T_Reselection_S_Fine", HFILL }},
    { &hf_rrc_non_HCS_t_CR_Max,
      { "non-HCS-t-CR-Max", "rrc.non_HCS_t_CR_Max",
        FT_UINT32, BASE_DEC, VALS(rrc_T_CRMax_vals), 0,
        "T_CRMax", HFILL }},
    { &hf_rrc_speedDependentScalingFactor,
      { "speedDependentScalingFactor", "rrc.speedDependentScalingFactor",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_interFrequencyTreselectionScalingFactor,
      { "interFrequencyTreselectionScalingFactor", "rrc.interFrequencyTreselectionScalingFactor",
        FT_UINT32, BASE_DEC, NULL, 0,
        "TreselectionScalingFactor", HFILL }},
    { &hf_rrc_interRATTreselectionScalingFactor,
      { "interRATTreselectionScalingFactor", "rrc.interRATTreselectionScalingFactor",
        FT_UINT32, BASE_DEC, NULL, 0,
        "TreselectionScalingFactor", HFILL }},
    { &hf_rrc_CellsForInterFreqMeasList_item,
      { "InterFreqCellID", "rrc.InterFreqCellID",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_CellsForInterRATMeasList_item,
      { "InterRATCellID", "rrc.InterRATCellID",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_CellsForIntraFreqMeasList_item,
      { "IntraFreqCellID", "rrc.IntraFreqCellID",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_CellsForIntraFreqMeasListOnSecULFreq_item,
      { "IntraFreqCellIDOnSecULFreq", "rrc.IntraFreqCellIDOnSecULFreq",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_modeSpecificInfo_109,
      { "modeSpecificInfo", "rrc.modeSpecificInfo",
        FT_UINT32, BASE_DEC, VALS(rrc_T_modeSpecificInfo_109_vals), 0,
        "T_modeSpecificInfo_109", HFILL }},
    { &hf_rrc_fdd_153,
      { "fdd", "rrc.fdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_fdd_148", HFILL }},
    { &hf_rrc_countC_SFN_Frame_difference,
      { "countC-SFN-Frame-difference", "rrc.countC_SFN_Frame_difference",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_tm,
      { "tm", "rrc.tm",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_38399", HFILL }},
    { &hf_rrc_tdd_106,
      { "tdd", "rrc.tdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_tdd_99", HFILL }},
    { &hf_rrc_bsicReported,
      { "bsicReported", "rrc.bsicReported",
        FT_UINT32, BASE_DEC, VALS(rrc_BSICReported_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_CellToReportList_item,
      { "CellToReport", "rrc.CellToReport",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_cnavToc,
      { "cnavToc", "rrc.cnavToc",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_11", HFILL }},
    { &hf_rrc_cnavTop,
      { "cnavTop", "rrc.cnavTop",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_11", HFILL }},
    { &hf_rrc_cnavURA0,
      { "cnavURA0", "rrc.cnavURA0",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_5", HFILL }},
    { &hf_rrc_cnavURA1,
      { "cnavURA1", "rrc.cnavURA1",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_3", HFILL }},
    { &hf_rrc_cnavURA2,
      { "cnavURA2", "rrc.cnavURA2",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_3", HFILL }},
    { &hf_rrc_cnavAf2,
      { "cnavAf2", "rrc.cnavAf2",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_10", HFILL }},
    { &hf_rrc_cnavAf1,
      { "cnavAf1", "rrc.cnavAf1",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_20", HFILL }},
    { &hf_rrc_cnavAf0,
      { "cnavAf0", "rrc.cnavAf0",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_26", HFILL }},
    { &hf_rrc_cnavTgd,
      { "cnavTgd", "rrc.cnavTgd",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_13", HFILL }},
    { &hf_rrc_cnavISCl1cp,
      { "cnavISCl1cp", "rrc.cnavISCl1cp",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_13", HFILL }},
    { &hf_rrc_cnavISCl1cd,
      { "cnavISCl1cd", "rrc.cnavISCl1cd",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_13", HFILL }},
    { &hf_rrc_cnavISCl1ca,
      { "cnavISCl1ca", "rrc.cnavISCl1ca",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_13", HFILL }},
    { &hf_rrc_cnavISCl2c,
      { "cnavISCl2c", "rrc.cnavISCl2c",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_13", HFILL }},
    { &hf_rrc_cnavISCl5i5,
      { "cnavISCl5i5", "rrc.cnavISCl5i5",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_13", HFILL }},
    { &hf_rrc_cnavISCl5q5,
      { "cnavISCl5q5", "rrc.cnavISCl5q5",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_13", HFILL }},
    { &hf_rrc_countC_SFN_High,
      { "countC-SFN-High", "rrc.countC_SFN_High",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_15", HFILL }},
    { &hf_rrc_off,
      { "off", "rrc.off",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_255", HFILL }},
    { &hf_rrc_modeSpecificInfo_110,
      { "modeSpecificInfo", "rrc.modeSpecificInfo",
        FT_UINT32, BASE_DEC, VALS(rrc_T_modeSpecificInfo_110_vals), 0,
        "T_modeSpecificInfo_110", HFILL }},
    { &hf_rrc_fdd_154,
      { "fdd", "rrc.fdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_fdd_149", HFILL }},
    { &hf_rrc_startPSC_01,
      { "startPSC", "rrc.startPSC",
        FT_NONE, BASE_NONE, NULL, 0,
        "PrimaryCPICH_Info", HFILL }},
    { &hf_rrc_numberOfPSCs_01,
      { "numberOfPSCs", "rrc.numberOfPSCs",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_1_512", HFILL }},
    { &hf_rrc_CSGCellInfoList_item,
      { "CSGCellInfo", "rrc.CSGCellInfo",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_cSGInterFreqCellInfoListperFreq,
      { "cSGInterFreqCellInfoListperFreq", "rrc.cSGInterFreqCellInfoListperFreq",
        FT_UINT32, BASE_DEC, NULL, 0,
        "CSGCellInfoList", HFILL }},
    { &hf_rrc_CSGInterFreqCellInfoList_item,
      { "CSGInterFreqCellInfo", "rrc.CSGInterFreqCellInfo",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_uTRACSGProximityDetec,
      { "uTRACSGProximityDetec", "rrc.uTRACSGProximityDetec",
        FT_UINT32, BASE_DEC, VALS(rrc_T_uTRACSGProximityDetec_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_e_UTRACSGProximityDetec,
      { "e-UTRACSGProximityDetec", "rrc.e_UTRACSGProximityDetec",
        FT_UINT32, BASE_DEC, VALS(rrc_T_e_UTRACSGProximityDetec_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_cSGproximityInd,
      { "cSGproximityInd", "rrc.cSGproximityInd",
        FT_UINT32, BASE_DEC, VALS(rrc_T_cSGproximityInd_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_radioAccessTechnology_01,
      { "radioAccessTechnology", "rrc.radioAccessTechnology",
        FT_UINT32, BASE_DEC, VALS(rrc_T_radioAccessTechnology_01_vals), 0,
        "T_radioAccessTechnology_01", HFILL }},
    { &hf_rrc_uTRA,
      { "uTRA", "rrc.uTRA",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_cSGFrequencyInfoUTRA,
      { "cSGFrequencyInfoUTRA", "rrc.cSGFrequencyInfoUTRA",
        FT_NONE, BASE_NONE, NULL, 0,
        "FrequencyInfo", HFILL }},
    { &hf_rrc_e_UTRA,
      { "e-UTRA", "rrc.e_UTRA",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_cSGFrequencyInfoEUTRA,
      { "cSGFrequencyInfoEUTRA", "rrc.cSGFrequencyInfoEUTRA",
        FT_UINT32, BASE_DEC, NULL, 0,
        "EARFCN", HFILL }},
    { &hf_rrc_ganss_signal_id,
      { "ganss-signal-id", "rrc.ganss_signal_id",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_data_bits,
      { "data-bits", "rrc.data_bits",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_1_1024", HFILL }},
    { &hf_rrc_DataBitAssistanceList_item,
      { "DataBitAssistanceSat", "rrc.DataBitAssistanceSat",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_satID_01,
      { "satID", "rrc.satID",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_63", HFILL }},
    { &hf_rrc_dataBitAssistanceSgnList,
      { "dataBitAssistanceSgnList", "rrc.dataBitAssistanceSgnList",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_DataBitAssistanceSgnList_item,
      { "DataBitAssistance", "rrc.DataBitAssistance",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_deltaRSCP,
      { "deltaRSCP", "rrc.deltaRSCP",
        FT_INT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_b1,
      { "b1", "rrc.b1",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_11", HFILL }},
    { &hf_rrc_b2,
      { "b2", "rrc.b2",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_10", HFILL }},
    { &hf_rrc_ganssSignalId,
      { "ganssSignalId", "rrc.ganssSignalId",
        FT_UINT32, BASE_DEC, NULL, 0,
        "GANSS_Signal_Id", HFILL }},
    { &hf_rrc_ganssStatusHealth,
      { "ganssStatusHealth", "rrc.ganssStatusHealth",
        FT_UINT32, BASE_DEC, VALS(rrc_GANSS_Status_Health_vals), 0,
        "GANSS_Status_Health", HFILL }},
    { &hf_rrc_dgansssignalInformationList,
      { "dgansssignalInformationList", "rrc.dgansssignalInformationList",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_dgansssignalInformationList_01,
      { "dgansssignalInformationList", "rrc.dgansssignalInformationList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "DGANSSSignalInformationList_r9", HFILL }},
    { &hf_rrc_dgansssignalInformationList_02,
      { "dgansssignalInformationList", "rrc.dgansssignalInformationList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "DGANSSSignalInformationList_v920ext", HFILL }},
    { &hf_rrc_DGANSSInfoList_item,
      { "DGANSSInfo", "rrc.DGANSSInfo",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_DGANSSInfoList_r9_item,
      { "DGANSSInfo-r9", "rrc.DGANSSInfo_r9",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_DGANSSInfoList_v920ext_item,
      { "DGANSSInfo-v920ext", "rrc.DGANSSInfo_v920ext",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_satId,
      { "satId", "rrc.satId",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_63", HFILL }},
    { &hf_rrc_iode_dganss,
      { "iode-dganss", "rrc.iode_dganss",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_10", HFILL }},
    { &hf_rrc_udre,
      { "udre", "rrc.udre",
        FT_UINT32, BASE_DEC, VALS(rrc_UDRE_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_ganss_prc,
      { "ganss-prc", "rrc.ganss_prc",
        FT_INT32, BASE_DEC, NULL, 0,
        "INTEGER_M2047_2047", HFILL }},
    { &hf_rrc_ganss_rrc,
      { "ganss-rrc", "rrc.ganss_rrc",
        FT_INT32, BASE_DEC, NULL, 0,
        "INTEGER_M127_127", HFILL }},
    { &hf_rrc_udreGrowthRate,
      { "udreGrowthRate", "rrc.udreGrowthRate",
        FT_UINT32, BASE_DEC, VALS(rrc_UDREGrowthRate_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_udreValidityTime,
      { "udreValidityTime", "rrc.udreValidityTime",
        FT_UINT32, BASE_DEC, VALS(rrc_UDREValidityTime_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_DGANSSSignalInformationList_item,
      { "DGANSSSignalInformation", "rrc.DGANSSSignalInformation",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_DGANSSSignalInformationList_r9_item,
      { "DGANSSSignalInformation-r9", "rrc.DGANSSSignalInformation_r9",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_DGANSSSignalInformationList_v920ext_item,
      { "DGANSSSignalInformation-v920ext", "rrc.DGANSSSignalInformation_v920ext",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_iode,
      { "iode", "rrc.iode",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_prc,
      { "prc", "rrc.prc",
        FT_INT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_rrc,
      { "rrc", "rrc.rrc",
        FT_INT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_dummy1_05,
      { "dummy1", "rrc.dummy1",
        FT_INT32, BASE_DEC, NULL, 0,
        "DeltaPRC", HFILL }},
    { &hf_rrc_dummy2_11,
      { "dummy2", "rrc.dummy2",
        FT_INT32, BASE_DEC, NULL, 0,
        "DeltaRRC", HFILL }},
    { &hf_rrc_dummy3_02,
      { "dummy3", "rrc.dummy3",
        FT_INT32, BASE_DEC, NULL, 0,
        "DeltaPRC", HFILL }},
    { &hf_rrc_dummy4_01,
      { "dummy4", "rrc.dummy4",
        FT_INT32, BASE_DEC, NULL, 0,
        "DeltaRRC", HFILL }},
    { &hf_rrc_DGPS_CorrectionSatInfoList_item,
      { "DGPS-CorrectionSatInfo", "rrc.DGPS_CorrectionSatInfo",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_DGPS_CorrectionSatInfoList_r9_item,
      { "DGPS-CorrectionSatInfo-r9", "rrc.DGPS_CorrectionSatInfo_r9",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_DGPS_CorrectionSatInfoList_v920ext_item,
      { "DGPS-CorrectionSatInfo-v920ext", "rrc.DGPS_CorrectionSatInfo_v920ext",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_latitudeSign,
      { "latitudeSign", "rrc.latitudeSign",
        FT_UINT32, BASE_DEC, VALS(rrc_T_latitudeSign_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_latitude,
      { "latitude", "rrc.latitude",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_8388607", HFILL }},
    { &hf_rrc_longitude,
      { "longitude", "rrc.longitude",
        FT_INT32, BASE_DEC, NULL, 0,
        "INTEGER_M8388608_8388607", HFILL }},
    { &hf_rrc_latitudeSign_01,
      { "latitudeSign", "rrc.latitudeSign",
        FT_UINT32, BASE_DEC, VALS(rrc_T_latitudeSign_01_vals), 0,
        "T_latitudeSign_01", HFILL }},
    { &hf_rrc_altitudeDirection,
      { "altitudeDirection", "rrc.altitudeDirection",
        FT_UINT32, BASE_DEC, VALS(rrc_T_altitudeDirection_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_altitude,
      { "altitude", "rrc.altitude",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_32767", HFILL }},
    { &hf_rrc_latitudeSign_02,
      { "latitudeSign", "rrc.latitudeSign",
        FT_UINT32, BASE_DEC, VALS(rrc_T_latitudeSign_02_vals), 0,
        "T_latitudeSign_02", HFILL }},
    { &hf_rrc_altitudeDirection_01,
      { "altitudeDirection", "rrc.altitudeDirection",
        FT_UINT32, BASE_DEC, VALS(rrc_T_altitudeDirection_01_vals), 0,
        "T_altitudeDirection_01", HFILL }},
    { &hf_rrc_uncertaintySemiMajor,
      { "uncertaintySemiMajor", "rrc.uncertaintySemiMajor",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_127", HFILL }},
    { &hf_rrc_uncertaintySemiMinor,
      { "uncertaintySemiMinor", "rrc.uncertaintySemiMinor",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_127", HFILL }},
    { &hf_rrc_orientationMajorAxis,
      { "orientationMajorAxis", "rrc.orientationMajorAxis",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_89", HFILL }},
    { &hf_rrc_uncertaintyAltitude,
      { "uncertaintyAltitude", "rrc.uncertaintyAltitude",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_127", HFILL }},
    { &hf_rrc_confidence,
      { "confidence", "rrc.confidence",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_100", HFILL }},
    { &hf_rrc_latitudeSign_03,
      { "latitudeSign", "rrc.latitudeSign",
        FT_UINT32, BASE_DEC, VALS(rrc_T_latitudeSign_03_vals), 0,
        "T_latitudeSign_03", HFILL }},
    { &hf_rrc_uncertaintyCode,
      { "uncertaintyCode", "rrc.uncertaintyCode",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_127", HFILL }},
    { &hf_rrc_latitudeSign_04,
      { "latitudeSign", "rrc.latitudeSign",
        FT_UINT32, BASE_DEC, VALS(rrc_T_latitudeSign_04_vals), 0,
        "T_latitudeSign_04", HFILL }},
    { &hf_rrc_reportedCells,
      { "reportedCells", "rrc.reportedCells",
        FT_UINT32, BASE_DEC, NULL, 0,
        "SEQUENCE_SIZE_1_maxReportedEUTRACellPerFreq_OF_EUTRA_PhysicalCellIdentity", HFILL }},
    { &hf_rrc_reportedCells_item,
      { "EUTRA-PhysicalCellIdentity", "rrc.EUTRA_PhysicalCellIdentity",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_Eutra_EventResultList_item,
      { "Eutra-EventResult", "rrc.Eutra_EventResult",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_eventID,
      { "eventID", "rrc.eventID",
        FT_UINT32, BASE_DEC, VALS(rrc_EventIDInterRAT_vals), 0,
        "EventIDInterRAT", HFILL }},
    { &hf_rrc_eutra_EventResultsList,
      { "eutra-EventResultsList", "rrc.eutra_EventResultsList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "Eutra_EventResultList", HFILL }},
    { &hf_rrc_eutraFrequencyIndicator,
      { "eutraFrequencyIndicator", "rrc.eutraFrequencyIndicator",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_maxNumEUTRAFreqs_FACH", HFILL }},
    { &hf_rrc_measurementBandwidth,
      { "measurementBandwidth", "rrc.measurementBandwidth",
        FT_UINT32, BASE_DEC, VALS(rrc_EUTRA_MeasurementBandwidth_vals), 0,
        "EUTRA_MeasurementBandwidth", HFILL }},
    { &hf_rrc_qRxLevMinEUTRA,
      { "qRxLevMinEUTRA", "rrc.qRxLevMinEUTRA",
        FT_INT32, BASE_DEC, NULL, 0,
        "INTEGER_M70_M22", HFILL }},
    { &hf_rrc_threshXhigh,
      { "threshXhigh", "rrc.threshXhigh",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_31", HFILL }},
    { &hf_rrc_threshXlow,
      { "threshXlow", "rrc.threshXlow",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_31", HFILL }},
    { &hf_rrc_eutra_blackListedCellList,
      { "eutra-blackListedCellList", "rrc.eutra_blackListedCellList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "EUTRA_BlacklistedCellPerFreqList", HFILL }},
    { &hf_rrc_qqualMinEUTRA,
      { "qqualMinEUTRA", "rrc.qqualMinEUTRA",
        FT_INT32, BASE_DEC, NULL, 0,
        "INTEGER_M34_M3", HFILL }},
    { &hf_rrc_threshXhigh2,
      { "threshXhigh2", "rrc.threshXhigh2",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_31", HFILL }},
    { &hf_rrc_threshXlow2,
      { "threshXlow2", "rrc.threshXlow2",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_31", HFILL }},
    { &hf_rrc_EUTRA_FrequencyAndPriorityInfoList_item,
      { "EUTRA-FrequencyAndPriorityInfo", "rrc.EUTRA_FrequencyAndPriorityInfo",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_EUTRA_FrequencyAndPriorityInfoList_v920ext_item,
      { "EUTRA-FrequencyAndPriorityInfo-v920ext", "rrc.EUTRA_FrequencyAndPriorityInfo_v920ext",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_eutraTreselectionScalingFactor,
      { "eutraTreselectionScalingFactor", "rrc.eutraTreselectionScalingFactor",
        FT_UINT32, BASE_DEC, NULL, 0,
        "TreselectionScalingFactor2", HFILL }},
    { &hf_rrc_eutraFrequencyRemoval,
      { "eutraFrequencyRemoval", "rrc.eutraFrequencyRemoval",
        FT_UINT32, BASE_DEC, VALS(rrc_EUTRA_FrequencyRemoval_vals), 0,
        "EUTRA_FrequencyRemoval", HFILL }},
    { &hf_rrc_eutraNewFrequencies,
      { "eutraNewFrequencies", "rrc.eutraNewFrequencies",
        FT_UINT32, BASE_DEC, NULL, 0,
        "EUTRA_FrequencyInfoList", HFILL }},
    { &hf_rrc_eutraSIAcquisition,
      { "eutraSIAcquisition", "rrc.eutraSIAcquisition",
        FT_NONE, BASE_NONE, NULL, 0,
        "EUTRA_SIAcquisition", HFILL }},
    { &hf_rrc_eutraNewFrequencies_01,
      { "eutraNewFrequencies", "rrc.eutraNewFrequencies",
        FT_UINT32, BASE_DEC, NULL, 0,
        "EUTRA_FrequencyInfoList_r11", HFILL }},
    { &hf_rrc_physicalCellIdentity,
      { "physicalCellIdentity", "rrc.physicalCellIdentity",
        FT_UINT32, BASE_DEC, NULL, 0,
        "EUTRA_PhysicalCellIdentity", HFILL }},
    { &hf_rrc_rSRP,
      { "rSRP", "rrc.rSRP",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_97", HFILL }},
    { &hf_rrc_rSRQ,
      { "rSRQ", "rrc.rSRQ",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_33", HFILL }},
    { &hf_rrc_measuredEUTRACells,
      { "measuredEUTRACells", "rrc.measuredEUTRACells",
        FT_UINT32, BASE_DEC, NULL, 0,
        "SEQUENCE_SIZE_1_maxReportedEUTRACellPerFreq_OF_EUTRA_MeasuredCells", HFILL }},
    { &hf_rrc_measuredEUTRACells_item,
      { "EUTRA-MeasuredCells", "rrc.EUTRA_MeasuredCells",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_Eutra_MeasuredResultList_item,
      { "Eutra-MeasuredResult", "rrc.Eutra_MeasuredResult",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_eutraMeasuredResultList,
      { "eutraMeasuredResultList", "rrc.eutraMeasuredResultList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "Eutra_MeasuredResultList", HFILL }},
    { &hf_rrc_eutraSIacquisitionResults,
      { "eutraSIacquisitionResults", "rrc.eutraSIacquisitionResults",
        FT_NONE, BASE_NONE, NULL, 0,
        "EUTRA_SIacquisitionResults", HFILL }},
    { &hf_rrc_measuredEUTRACells_v920ext,
      { "measuredEUTRACells-v920ext", "rrc.measuredEUTRACells_v920ext",
        FT_UINT32, BASE_DEC, NULL, 0,
        "SEQUENCE_SIZE_1_maxReportedEUTRACellPerFreq_OF_EUTRA_MeasuredCells_v920ext", HFILL }},
    { &hf_rrc_measuredEUTRACells_v920ext_item,
      { "EUTRA-MeasuredCells-v920ext", "rrc.EUTRA_MeasuredCells_v920ext",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_Eutra_MeasuredResultList_v920ext_item,
      { "Eutra-MeasuredResult-v920ext", "rrc.Eutra_MeasuredResult_v920ext",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_eutraMeasuredResultList_v920ext,
      { "eutraMeasuredResultList-v920ext", "rrc.eutraMeasuredResultList_v920ext",
        FT_UINT32, BASE_DEC, NULL, 0,
        "Eutra_MeasuredResultList_v920ext", HFILL }},
    { &hf_rrc_cgiInfo,
      { "cgiInfo", "rrc.cgiInfo",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_cellIdentity_01,
      { "cellIdentity", "rrc.cellIdentity",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_28", HFILL }},
    { &hf_rrc_trackingAreaCode,
      { "trackingAreaCode", "rrc.trackingAreaCode",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_16", HFILL }},
    { &hf_rrc_csgMemberIndication_01,
      { "csgMemberIndication", "rrc.csgMemberIndication",
        FT_UINT32, BASE_DEC, VALS(rrc_T_csgMemberIndication_01_vals), 0,
        "T_csgMemberIndication_01", HFILL }},
    { &hf_rrc_removeAllFrequencies,
      { "removeAllFrequencies", "rrc.removeAllFrequencies",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_removeSomeFrequencies,
      { "removeSomeFrequencies", "rrc.removeSomeFrequencies",
        FT_UINT32, BASE_DEC, NULL, 0,
        "SEQUENCE_SIZE_1_maxNumEUTRAFreqs_OF_EARFCN", HFILL }},
    { &hf_rrc_removeSomeFrequencies_item,
      { "EARFCN", "rrc.EARFCN",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_removeNoFrequencies,
      { "removeNoFrequencies", "rrc.removeNoFrequencies",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_EUTRA_FrequencyInfoList_item,
      { "EUTRA-FrequencyInfo", "rrc.EUTRA_FrequencyInfo",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_EUTRA_FrequencyInfoList_r11_item,
      { "EUTRA-FrequencyInfo-r11", "rrc.EUTRA_FrequencyInfo_r11",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_widebandRSRQMeasurement,
      { "widebandRSRQMeasurement", "rrc.widebandRSRQMeasurement",
        FT_UINT32, BASE_DEC, VALS(rrc_T_widebandRSRQMeasurement_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_eutra_FrequencyListIndicator,
      { "eutra-FrequencyListIndicator", "rrc.eutra_FrequencyListIndicator",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_maxNumEUTRAFreqs", HFILL }},
    { &hf_rrc_eutra_FrequencyRepQuantityRACH,
      { "eutra-FrequencyRepQuantityRACH", "rrc.eutra_FrequencyRepQuantityRACH",
        FT_UINT32, BASE_DEC, VALS(rrc_T_eutra_FrequencyRepQuantityRACH_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_eutra_FrequencyRACHReportingThreshold,
      { "eutra-FrequencyRACHReportingThreshold", "rrc.eutra_FrequencyRACHReportingThreshold",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_97", HFILL }},
    { &hf_rrc_rachReportingPriority,
      { "rachReportingPriority", "rrc.rachReportingPriority",
        FT_UINT32, BASE_DEC, VALS(rrc_T_rachReportingPriority_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_eutraFrequenciesForMeasurement,
      { "eutraFrequenciesForMeasurement", "rrc.eutraFrequenciesForMeasurement",
        FT_UINT32, BASE_DEC, NULL, 0,
        "SEQUENCE_SIZE_1_maxNumEUTRAFreqs_FACH_OF_EARFCN", HFILL }},
    { &hf_rrc_eutraFrequenciesForMeasurement_item,
      { "EARFCN", "rrc.EARFCN",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_reportCriteria,
      { "reportCriteria", "rrc.reportCriteria",
        FT_UINT32, BASE_DEC, VALS(rrc_T_reportCriteria_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_measurementQuantity,
      { "measurementQuantity", "rrc.measurementQuantity",
        FT_UINT32, BASE_DEC, VALS(rrc_T_measurementQuantity_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_reportingThreshold,
      { "reportingThreshold", "rrc.reportingThreshold",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_97", HFILL }},
    { &hf_rrc_triggeringCondition,
      { "triggeringCondition", "rrc.triggeringCondition",
        FT_UINT32, BASE_DEC, VALS(rrc_TriggeringCondition2_vals), 0,
        "TriggeringCondition2", HFILL }},
    { &hf_rrc_forbiddenAffectCellList,
      { "forbiddenAffectCellList", "rrc.forbiddenAffectCellList",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_w,
      { "w", "rrc.w",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_reportDeactivationThreshold,
      { "reportDeactivationThreshold", "rrc.reportDeactivationThreshold",
        FT_UINT32, BASE_DEC, VALS(rrc_ReportDeactivationThreshold_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_reportingAmount,
      { "reportingAmount", "rrc.reportingAmount",
        FT_UINT32, BASE_DEC, VALS(rrc_ReportingAmount_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_reportingInterval,
      { "reportingInterval", "rrc.reportingInterval",
        FT_UINT32, BASE_DEC, VALS(rrc_ReportingInterval_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_forbiddenAffectCellList_01,
      { "forbiddenAffectCellList", "rrc.forbiddenAffectCellList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "ForbiddenAffectCellList_r4", HFILL }},
    { &hf_rrc_forbiddenAffectCellListOnSecULFreq,
      { "forbiddenAffectCellListOnSecULFreq", "rrc.forbiddenAffectCellListOnSecULFreq",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_forbiddenAffectCellList_02,
      { "forbiddenAffectCellList", "rrc.forbiddenAffectCellList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "ForbiddenAffectCellList_LCR_r4", HFILL }},
    { &hf_rrc_triggeringCondition_01,
      { "triggeringCondition", "rrc.triggeringCondition",
        FT_UINT32, BASE_DEC, VALS(rrc_TriggeringCondition1_vals), 0,
        "TriggeringCondition1", HFILL }},
    { &hf_rrc_periodicReportingInfo_1b,
      { "periodicReportingInfo-1b", "rrc.periodicReportingInfo_1b",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_replacementActivationThreshold,
      { "replacementActivationThreshold", "rrc.replacementActivationThreshold",
        FT_UINT32, BASE_DEC, VALS(rrc_ReplacementActivationThreshold_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_useCIO,
      { "useCIO", "rrc.useCIO",
        FT_BOOLEAN, BASE_NONE, NULL, 0,
        "BOOLEAN", HFILL }},
    { &hf_rrc_thresholdUsedFrequency,
      { "thresholdUsedFrequency", "rrc.thresholdUsedFrequency",
        FT_INT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_thresholdUsedFrequency_01,
      { "thresholdUsedFrequency", "rrc.thresholdUsedFrequency",
        FT_INT32, BASE_DEC, NULL, 0,
        "ThresholdUsedFrequency_r6", HFILL }},
    { &hf_rrc_dummy_50,
      { "dummy", "rrc.dummy",
        FT_INT32, BASE_DEC, NULL, 0,
        "Threshold", HFILL }},
    { &hf_rrc_usedFreqW,
      { "usedFreqW", "rrc.usedFreqW",
        FT_UINT32, BASE_DEC, NULL, 0,
        "W", HFILL }},
    { &hf_rrc_hysteresis,
      { "hysteresis", "rrc.hysteresis",
        FT_UINT32, BASE_DEC, NULL, 0,
        "HysteresisInterFreq", HFILL }},
    { &hf_rrc_timeToTrigger,
      { "timeToTrigger", "rrc.timeToTrigger",
        FT_UINT32, BASE_DEC, VALS(rrc_TimeToTrigger_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_reportingCellStatus,
      { "reportingCellStatus", "rrc.reportingCellStatus",
        FT_UINT32, BASE_DEC, VALS(rrc_ReportingCellStatus_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_nonUsedFreqParameterList,
      { "nonUsedFreqParameterList", "rrc.nonUsedFreqParameterList",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_nonUsedFreqParameterList_01,
      { "nonUsedFreqParameterList", "rrc.nonUsedFreqParameterList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "NonUsedFreqWList_r6", HFILL }},
    { &hf_rrc_reportingCellStatus_01,
      { "reportingCellStatus", "rrc.reportingCellStatus",
        FT_UINT32, BASE_DEC, VALS(rrc_ReportingCellStatus_r10_vals), 0,
        "ReportingCellStatus_r10", HFILL }},
    { &hf_rrc_nonUsedFreqParameterList_02,
      { "nonUsedFreqParameterList", "rrc.nonUsedFreqParameterList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "NonUsedFreq2aParameterList_r10", HFILL }},
    { &hf_rrc_nonUsedFreqParameterList_03,
      { "nonUsedFreqParameterList", "rrc.nonUsedFreqParameterList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "NonUsedFreq2aParameterList_r11", HFILL }},
    { &hf_rrc_usedFreqThreshold,
      { "usedFreqThreshold", "rrc.usedFreqThreshold",
        FT_INT32, BASE_DEC, NULL, 0,
        "Threshold", HFILL }},
    { &hf_rrc_usedFreqThreshold_01,
      { "usedFreqThreshold", "rrc.usedFreqThreshold",
        FT_INT32, BASE_DEC, NULL, 0,
        "Threshold_r6", HFILL }},
    { &hf_rrc_nonUsedFreqParameterList_04,
      { "nonUsedFreqParameterList", "rrc.nonUsedFreqParameterList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "NonUsedFreqParameterList_r6", HFILL }},
    { &hf_rrc_nonUsedFreqParameterList_05,
      { "nonUsedFreqParameterList", "rrc.nonUsedFreqParameterList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "NonUsedFreqParameterList_r10", HFILL }},
    { &hf_rrc_nonUsedFreqParameterList_06,
      { "nonUsedFreqParameterList", "rrc.nonUsedFreqParameterList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "NonUsedFreqParameterList_r11", HFILL }},
    { &hf_rrc_thresholdOwnSystem,
      { "thresholdOwnSystem", "rrc.thresholdOwnSystem",
        FT_INT32, BASE_DEC, NULL, 0,
        "Threshold", HFILL }},
    { &hf_rrc_thresholdOtherSystem,
      { "thresholdOtherSystem", "rrc.thresholdOtherSystem",
        FT_INT32, BASE_DEC, NULL, 0,
        "Threshold", HFILL }},
    { &hf_rrc_hysteresis_01,
      { "hysteresis", "rrc.hysteresis",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_intraFreqEventResults,
      { "intraFreqEventResults", "rrc.intraFreqEventResults",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_interFreqEventResults,
      { "interFreqEventResults", "rrc.interFreqEventResults",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_interRATEventResults,
      { "interRATEventResults", "rrc.interRATEventResults",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_trafficVolumeEventResults,
      { "trafficVolumeEventResults", "rrc.trafficVolumeEventResults",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_qualityEventResults,
      { "qualityEventResults", "rrc.qualityEventResults",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_ue_InternalEventResults,
      { "ue-InternalEventResults", "rrc.ue_InternalEventResults",
        FT_UINT32, BASE_DEC, VALS(rrc_UE_InternalEventResults_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_ue_positioning_MeasurementEventResults,
      { "ue-positioning-MeasurementEventResults", "rrc.ue_positioning_MeasurementEventResults",
        FT_UINT32, BASE_DEC, VALS(rrc_UE_Positioning_MeasurementEventResults_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_ue_positioning_MeasurementEventResults_01,
      { "ue-positioning-MeasurementEventResults", "rrc.ue_positioning_MeasurementEventResults",
        FT_UINT32, BASE_DEC, VALS(rrc_UE_Positioning_MeasurementEventResults_v770ext_vals), 0,
        "UE_Positioning_MeasurementEventResults_v770ext", HFILL }},
    { &hf_rrc_ue_positioning_MeasurementEventResults_02,
      { "ue-positioning-MeasurementEventResults", "rrc.ue_positioning_MeasurementEventResults",
        FT_UINT32, BASE_DEC, VALS(rrc_UE_Positioning_MeasurementEventResults_v860ext_vals), 0,
        "UE_Positioning_MeasurementEventResults_v860ext", HFILL }},
    { &hf_rrc_interFreqEventResults_01,
      { "interFreqEventResults", "rrc.interFreqEventResults",
        FT_NONE, BASE_NONE, NULL, 0,
        "InterFreqEventResults_va40ext", HFILL }},
    { &hf_rrc_intraFreqEventResults_01,
      { "intraFreqEventResults", "rrc.intraFreqEventResults",
        FT_NONE, BASE_NONE, NULL, 0,
        "IntraFreqEventResultsOnSecUlFreq", HFILL }},
    { &hf_rrc_dopplerFirstOrder,
      { "dopplerFirstOrder", "rrc.dopplerFirstOrder",
        FT_INT32, BASE_DEC, NULL, 0,
        "INTEGER_M42_21", HFILL }},
    { &hf_rrc_dopplerUncertainty,
      { "dopplerUncertainty", "rrc.dopplerUncertainty",
        FT_UINT32, BASE_DEC, VALS(rrc_T_dopplerUncertainty_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_doppler1stOrder,
      { "doppler1stOrder", "rrc.doppler1stOrder",
        FT_INT32, BASE_DEC, NULL, 0,
        "INTEGER_M42_21", HFILL }},
    { &hf_rrc_dopplerUncertainty_01,
      { "dopplerUncertainty", "rrc.dopplerUncertainty",
        FT_UINT32, BASE_DEC, VALS(rrc_DopplerUncertainty_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_fACH_meas_occasion_coeff,
      { "fACH-meas-occasion-coeff", "rrc.fACH_meas_occasion_coeff",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_1_12", HFILL }},
    { &hf_rrc_inter_freq_FDD_meas_ind,
      { "inter-freq-FDD-meas-ind", "rrc.inter_freq_FDD_meas_ind",
        FT_BOOLEAN, BASE_NONE, NULL, 0,
        "BOOLEAN", HFILL }},
    { &hf_rrc_inter_freq_TDD_meas_ind,
      { "inter-freq-TDD-meas-ind", "rrc.inter_freq_TDD_meas_ind",
        FT_BOOLEAN, BASE_NONE, NULL, 0,
        "BOOLEAN", HFILL }},
    { &hf_rrc_inter_RAT_meas_ind,
      { "inter-RAT-meas-ind", "rrc.inter_RAT_meas_ind",
        FT_UINT32, BASE_DEC, NULL, 0,
        "SEQUENCE_SIZE_1_maxOtherRAT_OF_RAT_Type", HFILL }},
    { &hf_rrc_inter_RAT_meas_ind_item,
      { "RAT-Type", "rrc.RAT_Type",
        FT_UINT32, BASE_DEC, VALS(rrc_RAT_Type_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_inter_freq_TDD128_meas_ind,
      { "inter-freq-TDD128-meas-ind", "rrc.inter_freq_TDD128_meas_ind",
        FT_BOOLEAN, BASE_NONE, NULL, 0,
        "BOOLEAN", HFILL }},
    { &hf_rrc_fdd_155,
      { "fdd", "rrc.fdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "PrimaryCPICH_Info", HFILL }},
    { &hf_rrc_tdd_107,
      { "tdd", "rrc.tdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "PrimaryCCPCH_Info_LCR_r4", HFILL }},
    { &hf_rrc_ForbiddenAffectCellList_item,
      { "ForbiddenAffectCell", "rrc.ForbiddenAffectCell",
        FT_UINT32, BASE_DEC, VALS(rrc_ForbiddenAffectCell_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_ForbiddenAffectCellList_r4_item,
      { "ForbiddenAffectCell-r4", "rrc.ForbiddenAffectCell_r4",
        FT_UINT32, BASE_DEC, VALS(rrc_ForbiddenAffectCell_r4_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_ForbiddenAffectCellList_LCR_r4_item,
      { "ForbiddenAffectCell-LCR-r4", "rrc.ForbiddenAffectCell_LCR_r4",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_ForbiddenAffectCellListOnSecULFreq_item,
      { "ForbiddenAffectCellOnSecULFreq", "rrc.ForbiddenAffectCellOnSecULFreq",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_primaryCPICH,
      { "primaryCPICH", "rrc.primaryCPICH",
        FT_NONE, BASE_NONE, NULL, 0,
        "PrimaryCPICH_Info", HFILL }},
    { &hf_rrc_ganssTod,
      { "ganssTod", "rrc.ganssTod",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_86399", HFILL }},
    { &hf_rrc_dataBitAssistance,
      { "dataBitAssistance", "rrc.dataBitAssistance",
        FT_NONE, BASE_NONE, NULL, 0,
        "ReqDataBitAssistance", HFILL }},
    { &hf_rrc_ganssId,
      { "ganssId", "rrc.ganssId",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_7", HFILL }},
    { &hf_rrc_ganssTimeModelsList,
      { "ganssTimeModelsList", "rrc.ganssTimeModelsList",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_uePositioningDGANSSCorrections,
      { "uePositioningDGANSSCorrections", "rrc.uePositioningDGANSSCorrections",
        FT_NONE, BASE_NONE, NULL, 0,
        "UE_Positioning_DGANSSCorrections", HFILL }},
    { &hf_rrc_uePositioningGANSSNavigationModel,
      { "uePositioningGANSSNavigationModel", "rrc.uePositioningGANSSNavigationModel",
        FT_NONE, BASE_NONE, NULL, 0,
        "UE_Positioning_GANSS_NavigationModel", HFILL }},
    { &hf_rrc_uePositioningGANSSRealTimeIntegrity,
      { "uePositioningGANSSRealTimeIntegrity", "rrc.uePositioningGANSSRealTimeIntegrity",
        FT_UINT32, BASE_DEC, NULL, 0,
        "UE_Positioning_GANSS_RealTimeIntegrity", HFILL }},
    { &hf_rrc_uePositioningGANSSDataBitAssistance,
      { "uePositioningGANSSDataBitAssistance", "rrc.uePositioningGANSSDataBitAssistance",
        FT_NONE, BASE_NONE, NULL, 0,
        "UE_Positioning_GANSS_Data_Bit_Assistance", HFILL }},
    { &hf_rrc_uePositioningGANSSReferenceMeasurementInfo,
      { "uePositioningGANSSReferenceMeasurementInfo", "rrc.uePositioningGANSSReferenceMeasurementInfo",
        FT_NONE, BASE_NONE, NULL, 0,
        "UE_Positioning_GANSS_ReferenceMeasurementInfo", HFILL }},
    { &hf_rrc_uePositioningGANSSAlmanac,
      { "uePositioningGANSSAlmanac", "rrc.uePositioningGANSSAlmanac",
        FT_NONE, BASE_NONE, NULL, 0,
        "UE_Positioning_GANSS_Almanac", HFILL }},
    { &hf_rrc_uePositioningGANSSUTCModel,
      { "uePositioningGANSSUTCModel", "rrc.uePositioningGANSSUTCModel",
        FT_NONE, BASE_NONE, NULL, 0,
        "UE_Positioning_GANSS_UTCModel", HFILL }},
    { &hf_rrc_uePositiningGANSSsbasID,
      { "uePositiningGANSSsbasID", "rrc.uePositiningGANSSsbasID",
        FT_UINT32, BASE_DEC, VALS(rrc_UE_Positioning_GANSS_SBAS_ID_vals), 0,
        "UE_Positioning_GANSS_SBAS_ID", HFILL }},
    { &hf_rrc_uePositioningGANSSAddNavigationModels,
      { "uePositioningGANSSAddNavigationModels", "rrc.uePositioningGANSSAddNavigationModels",
        FT_NONE, BASE_NONE, NULL, 0,
        "UE_Positioning_GANSS_AddNavigationModels", HFILL }},
    { &hf_rrc_uePositioningGANSSAlmanac_01,
      { "uePositioningGANSSAlmanac", "rrc.uePositioningGANSSAlmanac",
        FT_NONE, BASE_NONE, NULL, 0,
        "UE_Positioning_GANSS_Almanac_v860ext", HFILL }},
    { &hf_rrc_uePositioningGANSSAddUTCModels,
      { "uePositioningGANSSAddUTCModels", "rrc.uePositioningGANSSAddUTCModels",
        FT_NONE, BASE_NONE, NULL, 0,
        "UE_Positioning_GANSS_AddUTCModels", HFILL }},
    { &hf_rrc_uePositioningGANSSAuxiliaryInfo,
      { "uePositioningGANSSAuxiliaryInfo", "rrc.uePositioningGANSSAuxiliaryInfo",
        FT_UINT32, BASE_DEC, VALS(rrc_UE_Positioning_GANSS_AuxiliaryInfo_vals), 0,
        "UE_Positioning_GANSS_AuxiliaryInfo", HFILL }},
    { &hf_rrc_uePositioningGANSSAlmanac_02,
      { "uePositioningGANSSAlmanac", "rrc.uePositioningGANSSAlmanac",
        FT_NONE, BASE_NONE, NULL, 0,
        "UE_Positioning_GANSS_Almanac_r8", HFILL }},
    { &hf_rrc_uePositioningDGANSSCorrections_01,
      { "uePositioningDGANSSCorrections", "rrc.uePositioningDGANSSCorrections",
        FT_NONE, BASE_NONE, NULL, 0,
        "UE_Positioning_DGANSSCorrections_r9", HFILL }},
    { &hf_rrc_uePositioningDGANSSCorrections_02,
      { "uePositioningDGANSSCorrections", "rrc.uePositioningDGANSSCorrections",
        FT_NONE, BASE_NONE, NULL, 0,
        "UE_Positioning_DGANSSCorrections_v920ext", HFILL }},
    { &hf_rrc_ganssTimeModelsList_01,
      { "ganssTimeModelsList", "rrc.ganssTimeModelsList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "GANSSTimeModelsList_va40ext", HFILL }},
    { &hf_rrc_uePositioningGANSSReferenceMeasurementInfo_01,
      { "uePositioningGANSSReferenceMeasurementInfo", "rrc.uePositioningGANSSReferenceMeasurementInfo",
        FT_NONE, BASE_NONE, NULL, 0,
        "UE_Positioning_GANSS_ReferenceMeasurementInfo_va40ext", HFILL }},
    { &hf_rrc_uePositioningGANSSAlmanac_03,
      { "uePositioningGANSSAlmanac", "rrc.uePositioningGANSSAlmanac",
        FT_NONE, BASE_NONE, NULL, 0,
        "UE_Positioning_GANSS_Almanac_va40ext", HFILL }},
    { &hf_rrc_ganssTimeModelsList_02,
      { "ganssTimeModelsList", "rrc.ganssTimeModelsList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "GANSSTimeModelsList_r10", HFILL }},
    { &hf_rrc_uePositioningGANSSReferenceMeasurementInfo_02,
      { "uePositioningGANSSReferenceMeasurementInfo", "rrc.uePositioningGANSSReferenceMeasurementInfo",
        FT_NONE, BASE_NONE, NULL, 0,
        "UE_Positioning_GANSS_ReferenceMeasurementInfo_r10", HFILL }},
    { &hf_rrc_uePositioningGANSSAlmanac_04,
      { "uePositioningGANSSAlmanac", "rrc.uePositioningGANSSAlmanac",
        FT_NONE, BASE_NONE, NULL, 0,
        "UE_Positioning_GANSS_Almanac_r10", HFILL }},
    { &hf_rrc_GANSSGenericDataList_item,
      { "GANSSGenericData", "rrc.GANSSGenericData",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_GANSSGenericDataList_v860ext_item,
      { "GANSSGenericData-v860ext", "rrc.GANSSGenericData_v860ext",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_GANSSGenericDataList_r8_item,
      { "GANSSGenericData-r8", "rrc.GANSSGenericData_r8",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_GANSSGenericDataList_r9_item,
      { "GANSSGenericData-r9", "rrc.GANSSGenericData_r9",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_GANSSGenericDataList_v920ext_item,
      { "GANSSGenericData-v920ext", "rrc.GANSSGenericData_v920ext",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_GANSSGenericDataList_va40ext_item,
      { "GANSSGenericData-va40ext", "rrc.GANSSGenericData_va40ext",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_GANSSGenericDataList_r10_item,
      { "GANSSGenericData-r10", "rrc.GANSSGenericData_r10",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_GANSSGenericMeasurementInfo_item,
      { "GANSSGenericMeasurementInfo item", "rrc.GANSSGenericMeasurementInfo_item",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_ganssMeasurementSignalList,
      { "ganssMeasurementSignalList", "rrc.ganssMeasurementSignalList",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_GANSSGenericMeasurementInfo_v860ext_item,
      { "GANSSGenericMeasurementInfo-v860ext item", "rrc.GANSSGenericMeasurementInfo_v860ext_item",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_ganssMeasurementSignalList_01,
      { "ganssMeasurementSignalList", "rrc.ganssMeasurementSignalList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "GANSSMeasurementSignalList_v860ext", HFILL }},
    { &hf_rrc_GANSSMeasurementParameters_item,
      { "GANSSMeasurementParameters item", "rrc.GANSSMeasurementParameters_item",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_cSurNzero,
      { "cSurNzero", "rrc.cSurNzero",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_63", HFILL }},
    { &hf_rrc_multipathIndicator,
      { "multipathIndicator", "rrc.multipathIndicator",
        FT_UINT32, BASE_DEC, VALS(rrc_T_multipathIndicator_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_carrierQualityIndication,
      { "carrierQualityIndication", "rrc.carrierQualityIndication",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_2", HFILL }},
    { &hf_rrc_ganssCodePhase,
      { "ganssCodePhase", "rrc.ganssCodePhase",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_2097151", HFILL }},
    { &hf_rrc_ganssIntegerCodePhase,
      { "ganssIntegerCodePhase", "rrc.ganssIntegerCodePhase",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_63", HFILL }},
    { &hf_rrc_codePhaseRmsError,
      { "codePhaseRmsError", "rrc.codePhaseRmsError",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_63", HFILL }},
    { &hf_rrc_doppler,
      { "doppler", "rrc.doppler",
        FT_INT32, BASE_DEC, NULL, 0,
        "INTEGER_M32768_32767", HFILL }},
    { &hf_rrc_adr,
      { "adr", "rrc.adr",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_33554431", HFILL }},
    { &hf_rrc_GANSSMeasurementParameters_v860ext_item,
      { "GANSSMeasurementParameters-v860ext item", "rrc.GANSSMeasurementParameters_v860ext_item",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_ganssIntegerCodePhaseExt,
      { "ganssIntegerCodePhaseExt", "rrc.ganssIntegerCodePhaseExt",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_64_127", HFILL }},
    { &hf_rrc_GANSSMeasurementSignalList_item,
      { "GANSSMeasurementSignalList item", "rrc.GANSSMeasurementSignalList_item",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_ganssCodePhaseAmbiguity,
      { "ganssCodePhaseAmbiguity", "rrc.ganssCodePhaseAmbiguity",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_31", HFILL }},
    { &hf_rrc_ganssMeasurementParameters,
      { "ganssMeasurementParameters", "rrc.ganssMeasurementParameters",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_GANSSMeasurementSignalList_v860ext_item,
      { "GANSSMeasurementSignalList-v860ext item", "rrc.GANSSMeasurementSignalList_v860ext_item",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_ganssCodePhaseAmbiguityExt,
      { "ganssCodePhaseAmbiguityExt", "rrc.ganssCodePhaseAmbiguityExt",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_32_127", HFILL }},
    { &hf_rrc_ganssMeasurementParameters_01,
      { "ganssMeasurementParameters", "rrc.ganssMeasurementParameters",
        FT_UINT32, BASE_DEC, NULL, 0,
        "GANSSMeasurementParameters_v860ext", HFILL }},
    { &hf_rrc_ganssWeek,
      { "ganssWeek", "rrc.ganssWeek",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_4095", HFILL }},
    { &hf_rrc_ganssToe,
      { "ganssToe", "rrc.ganssToe",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_167", HFILL }},
    { &hf_rrc_t_toeLimit,
      { "t-toeLimit", "rrc.t_toeLimit",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_10", HFILL }},
    { &hf_rrc_satellitesListRelatedDataList,
      { "satellitesListRelatedDataList", "rrc.satellitesListRelatedDataList",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_gANSS_tod,
      { "gANSS-tod", "rrc.gANSS_tod",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_3599999", HFILL }},
    { &hf_rrc_gANSS_timeId,
      { "gANSS-timeId", "rrc.gANSS_timeId",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_7", HFILL }},
    { &hf_rrc_gANSS_tod_uncertainty,
      { "gANSS-tod-uncertainty", "rrc.gANSS_tod_uncertainty",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_127", HFILL }},
    { &hf_rrc_ganssRealTimeIntegrity,
      { "ganssRealTimeIntegrity", "rrc.ganssRealTimeIntegrity",
        FT_BOOLEAN, BASE_NONE, NULL, 0,
        "BOOLEAN", HFILL }},
    { &hf_rrc_ganssDifferentialCorrection,
      { "ganssDifferentialCorrection", "rrc.ganssDifferentialCorrection",
        FT_BYTES, BASE_NONE, NULL, 0,
        "DGANSS_Sig_Id_Req", HFILL }},
    { &hf_rrc_ganssAlmanac,
      { "ganssAlmanac", "rrc.ganssAlmanac",
        FT_BOOLEAN, BASE_NONE, NULL, 0,
        "BOOLEAN", HFILL }},
    { &hf_rrc_ganssNavigationModel,
      { "ganssNavigationModel", "rrc.ganssNavigationModel",
        FT_BOOLEAN, BASE_NONE, NULL, 0,
        "BOOLEAN", HFILL }},
    { &hf_rrc_ganssTimeModelGNSS_GNSS,
      { "ganssTimeModelGNSS-GNSS", "rrc.ganssTimeModelGNSS_GNSS",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_8", HFILL }},
    { &hf_rrc_ganssReferenceMeasurementInfo,
      { "ganssReferenceMeasurementInfo", "rrc.ganssReferenceMeasurementInfo",
        FT_BOOLEAN, BASE_NONE, NULL, 0,
        "BOOLEAN", HFILL }},
    { &hf_rrc_ganssDataBits,
      { "ganssDataBits", "rrc.ganssDataBits",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_ganssUTCModel,
      { "ganssUTCModel", "rrc.ganssUTCModel",
        FT_BOOLEAN, BASE_NONE, NULL, 0,
        "BOOLEAN", HFILL }},
    { &hf_rrc_ganssNavigationModelAdditionalData,
      { "ganssNavigationModelAdditionalData", "rrc.ganssNavigationModelAdditionalData",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_sbasId,
      { "sbasId", "rrc.sbasId",
        FT_UINT32, BASE_DEC, VALS(rrc_UE_Positioning_GANSS_SBAS_ID_vals), 0,
        "UE_Positioning_GANSS_SBAS_ID", HFILL }},
    { &hf_rrc_ganssAddNavigationModel,
      { "ganssAddNavigationModel", "rrc.ganssAddNavigationModel",
        FT_UINT32, BASE_DEC, VALS(rrc_T_ganssAddNavigationModel_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_ganssAddUTCmodel,
      { "ganssAddUTCmodel", "rrc.ganssAddUTCmodel",
        FT_UINT32, BASE_DEC, VALS(rrc_T_ganssAddUTCmodel_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_ganssAuxInfo,
      { "ganssAuxInfo", "rrc.ganssAuxInfo",
        FT_UINT32, BASE_DEC, VALS(rrc_T_ganssAuxInfo_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_ganssAddADchoices,
      { "ganssAddADchoices", "rrc.ganssAddADchoices",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_orbitModelID,
      { "orbitModelID", "rrc.orbitModelID",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_7", HFILL }},
    { &hf_rrc_clockModelID,
      { "clockModelID", "rrc.clockModelID",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_7", HFILL }},
    { &hf_rrc_utcModelID,
      { "utcModelID", "rrc.utcModelID",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_7", HFILL }},
    { &hf_rrc_almanacModelID,
      { "almanacModelID", "rrc.almanacModelID",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_7", HFILL }},
    { &hf_rrc_GanssRequestedGenericAssistanceDataList_item,
      { "GanssReqGenericData", "rrc.GanssReqGenericData",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_GanssRequestedGenericAssistanceDataList_v860ext_item,
      { "GanssReqGenericData-v860ext", "rrc.GanssReqGenericData_v860ext",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_ganssSatId,
      { "ganssSatId", "rrc.ganssSatId",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_63", HFILL }},
    { &hf_rrc_dopplerZeroOrder,
      { "dopplerZeroOrder", "rrc.dopplerZeroOrder",
        FT_INT32, BASE_DEC, NULL, 0,
        "INTEGER_M2048_2047", HFILL }},
    { &hf_rrc_extraDoppler,
      { "extraDoppler", "rrc.extraDoppler",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_codePhase_01,
      { "codePhase", "rrc.codePhase",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_1023", HFILL }},
    { &hf_rrc_integerCodePhase_01,
      { "integerCodePhase", "rrc.integerCodePhase",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_127", HFILL }},
    { &hf_rrc_codePhaseSearchWindow_01,
      { "codePhaseSearchWindow", "rrc.codePhaseSearchWindow",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_5", HFILL }},
    { &hf_rrc_azimuthandElevation,
      { "azimuthandElevation", "rrc.azimuthandElevation",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_azimuthandElevation_01,
      { "azimuthandElevation", "rrc.azimuthandElevation",
        FT_NONE, BASE_NONE, NULL, 0,
        "AzimuthAndElevation_r10", HFILL }},
    { &hf_rrc_GANSSSatelliteInformationList_item,
      { "GANSSSatelliteInformation", "rrc.GANSSSatelliteInformation",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_GANSSSatelliteInformationList_va40ext_item,
      { "GANSSSatelliteInformation-va40ext", "rrc.GANSSSatelliteInformation_va40ext",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_GANSSSatelliteInformationList_r10_item,
      { "GANSSSatelliteInformation-r10", "rrc.GANSSSatelliteInformation_r10",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_GANSSTimeModelsList_item,
      { "UE-Positioning-GANSS-TimeModel", "rrc.UE_Positioning_GANSS_TimeModel",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_GANSSTimeModelsList_va40ext_item,
      { "UE-Positioning-GANSS-TimeModel-va40ext", "rrc.UE_Positioning_GANSS_TimeModel_va40ext",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_GANSSTimeModelsList_r10_item,
      { "UE-Positioning-GANSS-TimeModel-r10", "rrc.UE_Positioning_GANSS_TimeModel_r10",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_gloAlmNA,
      { "gloAlmNA", "rrc.gloAlmNA",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_11", HFILL }},
    { &hf_rrc_gloAlmnA,
      { "gloAlmnA", "rrc.gloAlmnA",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_5", HFILL }},
    { &hf_rrc_gloAlmHA,
      { "gloAlmHA", "rrc.gloAlmHA",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_5", HFILL }},
    { &hf_rrc_gloAlmLambdaA,
      { "gloAlmLambdaA", "rrc.gloAlmLambdaA",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_21", HFILL }},
    { &hf_rrc_gloAlmTlambdaA,
      { "gloAlmTlambdaA", "rrc.gloAlmTlambdaA",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_21", HFILL }},
    { &hf_rrc_gloAlmDeltaIA,
      { "gloAlmDeltaIA", "rrc.gloAlmDeltaIA",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_18", HFILL }},
    { &hf_rrc_gloAkmDeltaTA,
      { "gloAkmDeltaTA", "rrc.gloAkmDeltaTA",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_22", HFILL }},
    { &hf_rrc_gloAlmDeltaTdotA,
      { "gloAlmDeltaTdotA", "rrc.gloAlmDeltaTdotA",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_7", HFILL }},
    { &hf_rrc_gloAlmEpsilonA,
      { "gloAlmEpsilonA", "rrc.gloAlmEpsilonA",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_15", HFILL }},
    { &hf_rrc_gloAlmOmegaA,
      { "gloAlmOmegaA", "rrc.gloAlmOmegaA",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_16", HFILL }},
    { &hf_rrc_gloAlmTauA,
      { "gloAlmTauA", "rrc.gloAlmTauA",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_10", HFILL }},
    { &hf_rrc_gloAlmCA,
      { "gloAlmCA", "rrc.gloAlmCA",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_1", HFILL }},
    { &hf_rrc_gloAlmMA,
      { "gloAlmMA", "rrc.gloAlmMA",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_2", HFILL }},
    { &hf_rrc_svId,
      { "svId", "rrc.svId",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_63", HFILL }},
    { &hf_rrc_ganss_alm_e,
      { "ganss-alm-e", "rrc.ganss_alm_e",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_11", HFILL }},
    { &hf_rrc_ganss_delta_I_alm,
      { "ganss-delta-I-alm", "rrc.ganss_delta_I_alm",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_11", HFILL }},
    { &hf_rrc_ganss_omegadot_alm,
      { "ganss-omegadot-alm", "rrc.ganss_omegadot_alm",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_11", HFILL }},
    { &hf_rrc_ganss_svhealth_alm,
      { "ganss-svhealth-alm", "rrc.ganss_svhealth_alm",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_4", HFILL }},
    { &hf_rrc_ganss_delta_a_sqrt_alm,
      { "ganss-delta-a-sqrt-alm", "rrc.ganss_delta_a_sqrt_alm",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_17", HFILL }},
    { &hf_rrc_ganss_omegazero_alm,
      { "ganss-omegazero-alm", "rrc.ganss_omegazero_alm",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_16", HFILL }},
    { &hf_rrc_ganss_m_zero_alm,
      { "ganss-m-zero-alm", "rrc.ganss_m_zero_alm",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_16", HFILL }},
    { &hf_rrc_ganss_omega_alm,
      { "ganss-omega-alm", "rrc.ganss_omega_alm",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_16", HFILL }},
    { &hf_rrc_ganss_af_zero_alm,
      { "ganss-af-zero-alm", "rrc.ganss_af_zero_alm",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_14", HFILL }},
    { &hf_rrc_ganss_af_one_alm,
      { "ganss-af-one-alm", "rrc.ganss_af_one_alm",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_11", HFILL }},
    { &hf_rrc_midiAlmE,
      { "midiAlmE", "rrc.midiAlmE",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_11", HFILL }},
    { &hf_rrc_midiAlmDeltaI,
      { "midiAlmDeltaI", "rrc.midiAlmDeltaI",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_11", HFILL }},
    { &hf_rrc_midiAlmOmegaDot,
      { "midiAlmOmegaDot", "rrc.midiAlmOmegaDot",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_11", HFILL }},
    { &hf_rrc_midiAlmSqrtA,
      { "midiAlmSqrtA", "rrc.midiAlmSqrtA",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_17", HFILL }},
    { &hf_rrc_midiAlmOmega0,
      { "midiAlmOmega0", "rrc.midiAlmOmega0",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_16", HFILL }},
    { &hf_rrc_midiAlmOmega,
      { "midiAlmOmega", "rrc.midiAlmOmega",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_16", HFILL }},
    { &hf_rrc_midiAlmMo,
      { "midiAlmMo", "rrc.midiAlmMo",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_16", HFILL }},
    { &hf_rrc_midiAlmaf0,
      { "midiAlmaf0", "rrc.midiAlmaf0",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_11", HFILL }},
    { &hf_rrc_midiAlmaf1,
      { "midiAlmaf1", "rrc.midiAlmaf1",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_10", HFILL }},
    { &hf_rrc_midiAlmL1Health,
      { "midiAlmL1Health", "rrc.midiAlmL1Health",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_1", HFILL }},
    { &hf_rrc_midiAlmL2Health,
      { "midiAlmL2Health", "rrc.midiAlmL2Health",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_1", HFILL }},
    { &hf_rrc_midiAlmL5Health,
      { "midiAlmL5Health", "rrc.midiAlmL5Health",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_1", HFILL }},
    { &hf_rrc_navAlmE,
      { "navAlmE", "rrc.navAlmE",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_16", HFILL }},
    { &hf_rrc_navAlmDeltaI,
      { "navAlmDeltaI", "rrc.navAlmDeltaI",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_16", HFILL }},
    { &hf_rrc_navAlmOMEGADOT,
      { "navAlmOMEGADOT", "rrc.navAlmOMEGADOT",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_16", HFILL }},
    { &hf_rrc_navAlmSVHealth,
      { "navAlmSVHealth", "rrc.navAlmSVHealth",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_8", HFILL }},
    { &hf_rrc_navAlmSqrtA,
      { "navAlmSqrtA", "rrc.navAlmSqrtA",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_24", HFILL }},
    { &hf_rrc_navAlmOMEGAo,
      { "navAlmOMEGAo", "rrc.navAlmOMEGAo",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_24", HFILL }},
    { &hf_rrc_navAlmOmega,
      { "navAlmOmega", "rrc.navAlmOmega",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_24", HFILL }},
    { &hf_rrc_navAlmMo,
      { "navAlmMo", "rrc.navAlmMo",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_24", HFILL }},
    { &hf_rrc_navAlmaf0,
      { "navAlmaf0", "rrc.navAlmaf0",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_11", HFILL }},
    { &hf_rrc_navAlmaf1,
      { "navAlmaf1", "rrc.navAlmaf1",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_11", HFILL }},
    { &hf_rrc_redAlmDeltaA,
      { "redAlmDeltaA", "rrc.redAlmDeltaA",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_8", HFILL }},
    { &hf_rrc_redAlmOmega0,
      { "redAlmOmega0", "rrc.redAlmOmega0",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_7", HFILL }},
    { &hf_rrc_redAlmPhi0,
      { "redAlmPhi0", "rrc.redAlmPhi0",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_7", HFILL }},
    { &hf_rrc_redAlmL1Health,
      { "redAlmL1Health", "rrc.redAlmL1Health",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_1", HFILL }},
    { &hf_rrc_redAlmL2Health,
      { "redAlmL2Health", "rrc.redAlmL2Health",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_1", HFILL }},
    { &hf_rrc_redAlmL5Health,
      { "redAlmL5Health", "rrc.redAlmL5Health",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_1", HFILL }},
    { &hf_rrc_sbasAlmDataID,
      { "sbasAlmDataID", "rrc.sbasAlmDataID",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_2", HFILL }},
    { &hf_rrc_sbasAlmHealth,
      { "sbasAlmHealth", "rrc.sbasAlmHealth",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_8", HFILL }},
    { &hf_rrc_sbasAlmXg,
      { "sbasAlmXg", "rrc.sbasAlmXg",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_15", HFILL }},
    { &hf_rrc_sbasAlmYg,
      { "sbasAlmYg", "rrc.sbasAlmYg",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_15", HFILL }},
    { &hf_rrc_sbasAlmZg,
      { "sbasAlmZg", "rrc.sbasAlmZg",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_9", HFILL }},
    { &hf_rrc_sbasAlmXgdot,
      { "sbasAlmXgdot", "rrc.sbasAlmXgdot",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_3", HFILL }},
    { &hf_rrc_sbasAlmYgDot,
      { "sbasAlmYgDot", "rrc.sbasAlmYgDot",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_3", HFILL }},
    { &hf_rrc_sbasAlmZgDot,
      { "sbasAlmZgDot", "rrc.sbasAlmZgDot",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_4", HFILL }},
    { &hf_rrc_sbasAlmTo,
      { "sbasAlmTo", "rrc.sbasAlmTo",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_11", HFILL }},
    { &hf_rrc_GANSS_SAT_Info_Almanac_GLOkpList_item,
      { "GANSS-SAT-Info-Almanac-GLOkp", "rrc.GANSS_SAT_Info_Almanac_GLOkp",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_GANSS_SAT_Info_Almanac_KpList_item,
      { "GANSS-SAT-Info-Almanac-Kp", "rrc.GANSS_SAT_Info_Almanac_Kp",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_GANSS_SAT_Info_Almanac_MIDIkpList_item,
      { "GANSS-SAT-Info-Almanac-MIDIkp", "rrc.GANSS_SAT_Info_Almanac_MIDIkp",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_GANSS_SAT_Info_Almanac_NAVkpList_item,
      { "GANSS-SAT-Info-Almanac-NAVkp", "rrc.GANSS_SAT_Info_Almanac_NAVkp",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_GANSS_SAT_Info_Almanac_REDkpList_item,
      { "GANSS-SAT-Info-Almanac-REDkp", "rrc.GANSS_SAT_Info_Almanac_REDkp",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_GANSS_SAT_Info_Almanac_SBASecefList_item,
      { "GANSS-SAT-Info-Almanac-SBASecef", "rrc.GANSS_SAT_Info_Almanac_SBASecef",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_svHealth,
      { "svHealth", "rrc.svHealth",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_6", HFILL }},
    { &hf_rrc_iod,
      { "iod", "rrc.iod",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_11", HFILL }},
    { &hf_rrc_ganssClockModel,
      { "ganssClockModel", "rrc.ganssClockModel",
        FT_NONE, BASE_NONE, NULL, 0,
        "UE_Positioning_GANSS_AddClockModels", HFILL }},
    { &hf_rrc_ganssOrbitModel,
      { "ganssOrbitModel", "rrc.ganssOrbitModel",
        FT_NONE, BASE_NONE, NULL, 0,
        "UE_Positioning_GANSS_AddOrbitModels", HFILL }},
    { &hf_rrc_svHealth_01,
      { "svHealth", "rrc.svHealth",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_5", HFILL }},
    { &hf_rrc_iod_01,
      { "iod", "rrc.iod",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_10", HFILL }},
    { &hf_rrc_ganssClockModel_01,
      { "ganssClockModel", "rrc.ganssClockModel",
        FT_NONE, BASE_NONE, NULL, 0,
        "UE_Positioning_GANSS_ClockModel", HFILL }},
    { &hf_rrc_ganssOrbitModel_01,
      { "ganssOrbitModel", "rrc.ganssOrbitModel",
        FT_NONE, BASE_NONE, NULL, 0,
        "UE_Positioning_GANSS_OrbitModel", HFILL }},
    { &hf_rrc_Ganss_Sat_Info_AddNavList_item,
      { "Ganss-Sat-Info-AddNav", "rrc.Ganss_Sat_Info_AddNav",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_Ganss_Sat_Info_NavList_item,
      { "Ganss-Sat-Info-Nav", "rrc.Ganss_Sat_Info_Nav",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_storm_flag_one,
      { "storm-flag-one", "rrc.storm_flag_one",
        FT_BOOLEAN, BASE_NONE, NULL, 0,
        "BOOLEAN", HFILL }},
    { &hf_rrc_storm_flag_two,
      { "storm-flag-two", "rrc.storm_flag_two",
        FT_BOOLEAN, BASE_NONE, NULL, 0,
        "BOOLEAN", HFILL }},
    { &hf_rrc_storm_flag_three,
      { "storm-flag-three", "rrc.storm_flag_three",
        FT_BOOLEAN, BASE_NONE, NULL, 0,
        "BOOLEAN", HFILL }},
    { &hf_rrc_storm_flag_four,
      { "storm-flag-four", "rrc.storm_flag_four",
        FT_BOOLEAN, BASE_NONE, NULL, 0,
        "BOOLEAN", HFILL }},
    { &hf_rrc_storm_flag_five,
      { "storm-flag-five", "rrc.storm_flag_five",
        FT_BOOLEAN, BASE_NONE, NULL, 0,
        "BOOLEAN", HFILL }},
    { &hf_rrc_gloTau,
      { "gloTau", "rrc.gloTau",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_22", HFILL }},
    { &hf_rrc_gloGamma,
      { "gloGamma", "rrc.gloGamma",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_11", HFILL }},
    { &hf_rrc_gloDeltaTau,
      { "gloDeltaTau", "rrc.gloDeltaTau",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_5", HFILL }},
    { &hf_rrc_satelliteID,
      { "satelliteID", "rrc.satelliteID",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_63", HFILL }},
    { &hf_rrc_c_N0,
      { "c-N0", "rrc.c_N0",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_63", HFILL }},
    { &hf_rrc_doppler_01,
      { "doppler", "rrc.doppler",
        FT_INT32, BASE_DEC, NULL, 0,
        "INTEGER_M32768_32768", HFILL }},
    { &hf_rrc_wholeGPS_Chips,
      { "wholeGPS-Chips", "rrc.wholeGPS_Chips",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_1022", HFILL }},
    { &hf_rrc_fractionalGPS_Chips,
      { "fractionalGPS-Chips", "rrc.fractionalGPS_Chips",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_1023", HFILL }},
    { &hf_rrc_multipathIndicator_01,
      { "multipathIndicator", "rrc.multipathIndicator",
        FT_UINT32, BASE_DEC, VALS(rrc_MultipathIndicator_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_pseudorangeRMS_Error,
      { "pseudorangeRMS-Error", "rrc.pseudorangeRMS_Error",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_63", HFILL }},
    { &hf_rrc_GPS_MeasurementParamList_item,
      { "GPS-MeasurementParam", "rrc.GPS_MeasurementParam",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_startingARFCN,
      { "startingARFCN", "rrc.startingARFCN",
        FT_UINT32, BASE_DEC, NULL, 0,
        "BCCH_ARFCN", HFILL }},
    { &hf_rrc_bandIndicator,
      { "bandIndicator", "rrc.bandIndicator",
        FT_UINT32, BASE_DEC, VALS(rrc_T_bandIndicator_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_followingARFCNs,
      { "followingARFCNs", "rrc.followingARFCNs",
        FT_UINT32, BASE_DEC, VALS(rrc_T_followingARFCNs_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_explicitListOfARFCNs,
      { "explicitListOfARFCNs", "rrc.explicitListOfARFCNs",
        FT_UINT32, BASE_DEC, NULL, 0,
        "SEQUENCE_SIZE_0_31_OF_BCCH_ARFCN", HFILL }},
    { &hf_rrc_explicitListOfARFCNs_item,
      { "BCCH-ARFCN", "rrc.BCCH_ARFCN",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_equallySpacedARFCNs,
      { "equallySpacedARFCNs", "rrc.equallySpacedARFCNs",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_arfcn_Spacing,
      { "arfcn-Spacing", "rrc.arfcn_Spacing",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_1_8", HFILL }},
    { &hf_rrc_numberOfFollowingARFCNs,
      { "numberOfFollowingARFCNs", "rrc.numberOfFollowingARFCNs",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_31", HFILL }},
    { &hf_rrc_variableBitMapOfARFCNs,
      { "variableBitMapOfARFCNs", "rrc.variableBitMapOfARFCNs",
        FT_BYTES, BASE_NONE, NULL, 0,
        "OCTET_STRING_SIZE_1_16", HFILL }},
    { &hf_rrc_continuousRangeOfARFCNs,
      { "continuousRangeOfARFCNs", "rrc.continuousRangeOfARFCNs",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_endingARFCN,
      { "endingARFCN", "rrc.endingARFCN",
        FT_UINT32, BASE_DEC, NULL, 0,
        "BCCH_ARFCN", HFILL }},
    { &hf_rrc_gsm_CarrierRSSI,
      { "gsm-CarrierRSSI", "rrc.gsm_CarrierRSSI",
        FT_BYTES, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_dummy_51,
      { "dummy", "rrc.dummy",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_46_173", HFILL }},
    { &hf_rrc_dummy2_12,
      { "dummy2", "rrc.dummy2",
        FT_UINT32, BASE_DEC, NULL, 0,
        "ObservedTimeDifferenceToGSM", HFILL }},
    { &hf_rrc_GSM_MeasuredResultsList_item,
      { "GSM-MeasuredResults", "rrc.GSM_MeasuredResults",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_tlm_Message,
      { "tlm-Message", "rrc.tlm_Message",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_14", HFILL }},
    { &hf_rrc_tlm_Reserved,
      { "tlm-Reserved", "rrc.tlm_Reserved",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_2", HFILL }},
    { &hf_rrc_alert,
      { "alert", "rrc.alert",
        FT_BOOLEAN, BASE_NONE, NULL, 0,
        "BOOLEAN", HFILL }},
    { &hf_rrc_antiSpoof,
      { "antiSpoof", "rrc.antiSpoof",
        FT_BOOLEAN, BASE_NONE, NULL, 0,
        "BOOLEAN", HFILL }},
    { &hf_rrc_GPS_TOW_AssistList_item,
      { "GPS-TOW-Assist", "rrc.GPS_TOW_Assist",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_gsmCellGroup,
      { "gsmCellGroup", "rrc.gsmCellGroup",
        FT_NONE, BASE_NONE, NULL, 0,
        "GSM_CellGroup", HFILL }},
    { &hf_rrc_qRxLevMinGSM,
      { "qRxLevMinGSM", "rrc.qRxLevMinGSM",
        FT_INT32, BASE_DEC, NULL, 0,
        "INTEGER_M58_M13", HFILL }},
    { &hf_rrc_GSM_PriorityInfoList_item,
      { "GSM-PriorityInfo", "rrc.GSM_PriorityInfo",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_gsmTreselectionScalingFactor,
      { "gsmTreselectionScalingFactor", "rrc.gsmTreselectionScalingFactor",
        FT_UINT32, BASE_DEC, NULL, 0,
        "TreselectionScalingFactor2", HFILL }},
    { &hf_rrc_penaltyTime,
      { "penaltyTime", "rrc.penaltyTime",
        FT_UINT32, BASE_DEC, VALS(rrc_PenaltyTime_RSCP_vals), 0,
        "PenaltyTime_RSCP", HFILL }},
    { &hf_rrc_penaltyTime_01,
      { "penaltyTime", "rrc.penaltyTime",
        FT_UINT32, BASE_DEC, VALS(rrc_PenaltyTime_ECN0_vals), 0,
        "PenaltyTime_ECN0", HFILL }},
    { &hf_rrc_hcs_PRIO,
      { "hcs-PRIO", "rrc.hcs_PRIO",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_q_HCS,
      { "q-HCS", "rrc.q_HCS",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_hcs_CellReselectInformation,
      { "hcs-CellReselectInformation", "rrc.hcs_CellReselectInformation",
        FT_NONE, BASE_NONE, NULL, 0,
        "HCS_CellReselectInformation_RSCP", HFILL }},
    { &hf_rrc_hcs_CellReselectInformation_01,
      { "hcs-CellReselectInformation", "rrc.hcs_CellReselectInformation",
        FT_NONE, BASE_NONE, NULL, 0,
        "HCS_CellReselectInformation_ECN0", HFILL }},
    { &hf_rrc_t_CR_Max,
      { "t-CR-Max", "rrc.t_CR_Max",
        FT_UINT32, BASE_DEC, VALS(rrc_T_CRMax_vals), 0,
        "T_CRMax", HFILL }},
    { &hf_rrc_bearing,
      { "bearing", "rrc.bearing",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_359", HFILL }},
    { &hf_rrc_horizontalSpeed,
      { "horizontalSpeed", "rrc.horizontalSpeed",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_2047", HFILL }},
    { &hf_rrc_verticalSpeedDirection,
      { "verticalSpeedDirection", "rrc.verticalSpeedDirection",
        FT_UINT32, BASE_DEC, VALS(rrc_T_verticalSpeedDirection_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_verticalSpeed,
      { "verticalSpeed", "rrc.verticalSpeed",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_255", HFILL }},
    { &hf_rrc_horizontalSpeedUncertainty,
      { "horizontalSpeedUncertainty", "rrc.horizontalSpeedUncertainty",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_255", HFILL }},
    { &hf_rrc_verticalSpeedDirection_01,
      { "verticalSpeedDirection", "rrc.verticalSpeedDirection",
        FT_UINT32, BASE_DEC, VALS(rrc_T_verticalSpeedDirection_01_vals), 0,
        "T_verticalSpeedDirection_01", HFILL }},
    { &hf_rrc_horizontalUncertaintySpeed,
      { "horizontalUncertaintySpeed", "rrc.horizontalUncertaintySpeed",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_255", HFILL }},
    { &hf_rrc_verticalUncertaintySpeed,
      { "verticalUncertaintySpeed", "rrc.verticalUncertaintySpeed",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_255", HFILL }},
    { &hf_rrc_k,
      { "k", "rrc.k",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_2_3", HFILL }},
    { &hf_rrc_nonFreqRelatedEventResults,
      { "nonFreqRelatedEventResults", "rrc.nonFreqRelatedEventResults",
        FT_UINT32, BASE_DEC, VALS(rrc_CellMeasurementEventResults_vals), 0,
        "CellMeasurementEventResults", HFILL }},
    { &hf_rrc_nonFreqRelatedEventResults_01,
      { "nonFreqRelatedEventResults", "rrc.nonFreqRelatedEventResults",
        FT_UINT32, BASE_DEC, NULL, 0,
        "CellMeasurementEventResults_LCR_r4", HFILL }},
    { &hf_rrc_removedInterFreqCellList,
      { "removedInterFreqCellList", "rrc.removedInterFreqCellList",
        FT_UINT32, BASE_DEC, VALS(rrc_RemovedInterFreqCellList_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_newInterFreqCellList_02,
      { "newInterFreqCellList", "rrc.newInterFreqCellList",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_cellsForInterFreqMeasList,
      { "cellsForInterFreqMeasList", "rrc.cellsForInterFreqMeasList",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_newInterFreqCellList_03,
      { "newInterFreqCellList", "rrc.newInterFreqCellList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "NewInterFreqCellList_r4", HFILL }},
    { &hf_rrc_newInterFreqCellList_04,
      { "newInterFreqCellList", "rrc.newInterFreqCellList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "NewInterFreqCellList_r8", HFILL }},
    { &hf_rrc_newInterFreqCellList_05,
      { "newInterFreqCellList", "rrc.newInterFreqCellList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "NewInterFreqCellList_r9", HFILL }},
    { &hf_rrc_cSGInterFreqCellInfoList,
      { "cSGInterFreqCellInfoList", "rrc.cSGInterFreqCellInfoList",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_interFreqSIAcquisition,
      { "interFreqSIAcquisition", "rrc.interFreqSIAcquisition",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_newInterFreqCellList_06,
      { "newInterFreqCellList", "rrc.newInterFreqCellList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "NewInterFreqCellList_r10", HFILL }},
    { &hf_rrc_modeSpecificInfo_111,
      { "modeSpecificInfo", "rrc.modeSpecificInfo",
        FT_UINT32, BASE_DEC, VALS(rrc_T_modeSpecificInfo_111_vals), 0,
        "T_modeSpecificInfo_111", HFILL }},
    { &hf_rrc_fdd_156,
      { "fdd", "rrc.fdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_fdd_150", HFILL }},
    { &hf_rrc_primaryScramblingCode_01,
      { "primaryScramblingCode", "rrc.primaryScramblingCode",
        FT_NONE, BASE_NONE, NULL, 0,
        "PrimaryCPICH_Info", HFILL }},
    { &hf_rrc_newInterFreqCellList_07,
      { "newInterFreqCellList", "rrc.newInterFreqCellList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "NewInterFreqCellSI_List_RSCP", HFILL }},
    { &hf_rrc_newInterFreqCellList_08,
      { "newInterFreqCellList", "rrc.newInterFreqCellList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "NewInterFreqCellSI_List_ECN0", HFILL }},
    { &hf_rrc_newInterFreqCellList_09,
      { "newInterFreqCellList", "rrc.newInterFreqCellList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "NewInterFreqCellSI_List_HCS_RSCP", HFILL }},
    { &hf_rrc_newInterFreqCellList_10,
      { "newInterFreqCellList", "rrc.newInterFreqCellList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "NewInterFreqCellSI_List_HCS_ECN0", HFILL }},
    { &hf_rrc_newInterFreqCellList_11,
      { "newInterFreqCellList", "rrc.newInterFreqCellList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "NewInterFreqCellSI_List_RSCP_LCR_r4", HFILL }},
    { &hf_rrc_newInterFreqCellList_12,
      { "newInterFreqCellList", "rrc.newInterFreqCellList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "NewInterFreqCellSI_List_ECN0_LCR_r4", HFILL }},
    { &hf_rrc_newInterFreqCellList_13,
      { "newInterFreqCellList", "rrc.newInterFreqCellList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "NewInterFreqCellSI_List_HCS_RSCP_LCR_r4", HFILL }},
    { &hf_rrc_newInterFreqCellList_14,
      { "newInterFreqCellList", "rrc.newInterFreqCellList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "NewInterFreqCellSI_List_HCS_ECN0_LCR_r4", HFILL }},
    { &hf_rrc_InterFreqCellList_item,
      { "InterFreqCell", "rrc.InterFreqCell",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_InterFreqCellList_LCR_r4_ext_item,
      { "InterFreqCell-LCR-r4", "rrc.InterFreqCell_LCR_r4",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_InterFreqCellMeasuredResultsList_item,
      { "CellMeasuredResults", "rrc.CellMeasuredResults",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_InterFreqCellMeasuredResultsList_v920ext_item,
      { "CellMeasuredResults-v920ext", "rrc.CellMeasuredResults_v920ext",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_event2a,
      { "event2a", "rrc.event2a",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_event2b,
      { "event2b", "rrc.event2b",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_event2c,
      { "event2c", "rrc.event2c",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_event2d,
      { "event2d", "rrc.event2d",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_event2e,
      { "event2e", "rrc.event2e",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_event2f,
      { "event2f", "rrc.event2f",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_event2a_01,
      { "event2a", "rrc.event2a",
        FT_NONE, BASE_NONE, NULL, 0,
        "Event2a_r6", HFILL }},
    { &hf_rrc_event2b_01,
      { "event2b", "rrc.event2b",
        FT_NONE, BASE_NONE, NULL, 0,
        "Event2b_r6", HFILL }},
    { &hf_rrc_event2c_01,
      { "event2c", "rrc.event2c",
        FT_NONE, BASE_NONE, NULL, 0,
        "Event2c_r6", HFILL }},
    { &hf_rrc_event2d_01,
      { "event2d", "rrc.event2d",
        FT_NONE, BASE_NONE, NULL, 0,
        "Event2d_r6", HFILL }},
    { &hf_rrc_event2e_01,
      { "event2e", "rrc.event2e",
        FT_NONE, BASE_NONE, NULL, 0,
        "Event2e_r6", HFILL }},
    { &hf_rrc_event2f_01,
      { "event2f", "rrc.event2f",
        FT_NONE, BASE_NONE, NULL, 0,
        "Event2f_r6", HFILL }},
    { &hf_rrc_event2a_02,
      { "event2a", "rrc.event2a",
        FT_NONE, BASE_NONE, NULL, 0,
        "Event2a_r10", HFILL }},
    { &hf_rrc_event2b_02,
      { "event2b", "rrc.event2b",
        FT_NONE, BASE_NONE, NULL, 0,
        "Event2b_r10", HFILL }},
    { &hf_rrc_event2c_02,
      { "event2c", "rrc.event2c",
        FT_NONE, BASE_NONE, NULL, 0,
        "Event2c_r10", HFILL }},
    { &hf_rrc_event2d_02,
      { "event2d", "rrc.event2d",
        FT_NONE, BASE_NONE, NULL, 0,
        "Event2d_r10", HFILL }},
    { &hf_rrc_event2e_02,
      { "event2e", "rrc.event2e",
        FT_NONE, BASE_NONE, NULL, 0,
        "Event2e_r10", HFILL }},
    { &hf_rrc_event2f_02,
      { "event2f", "rrc.event2f",
        FT_NONE, BASE_NONE, NULL, 0,
        "Event2f_r10", HFILL }},
    { &hf_rrc_event2a_03,
      { "event2a", "rrc.event2a",
        FT_NONE, BASE_NONE, NULL, 0,
        "Event2a_r11", HFILL }},
    { &hf_rrc_event2b_03,
      { "event2b", "rrc.event2b",
        FT_NONE, BASE_NONE, NULL, 0,
        "Event2b_r11", HFILL }},
    { &hf_rrc_event2c_03,
      { "event2c", "rrc.event2c",
        FT_NONE, BASE_NONE, NULL, 0,
        "Event2c_r11", HFILL }},
    { &hf_rrc_event2e_03,
      { "event2e", "rrc.event2e",
        FT_NONE, BASE_NONE, NULL, 0,
        "Event2e_r11", HFILL }},
    { &hf_rrc_InterFreqEventList_item,
      { "InterFreqEvent", "rrc.InterFreqEvent",
        FT_UINT32, BASE_DEC, VALS(rrc_InterFreqEvent_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_InterFreqEventList_r6_item,
      { "InterFreqEvent-r6", "rrc.InterFreqEvent_r6",
        FT_UINT32, BASE_DEC, VALS(rrc_InterFreqEvent_r6_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_InterFreqEventList_r10_item,
      { "InterFreqEvent-r10", "rrc.InterFreqEvent_r10",
        FT_UINT32, BASE_DEC, VALS(rrc_InterFreqEvent_r10_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_InterFreqEventList_r11_item,
      { "InterFreqEvent-r11", "rrc.InterFreqEvent_r11",
        FT_UINT32, BASE_DEC, VALS(rrc_InterFreqEvent_r11_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_InterFrequencyMeasuredResultsList_v590ext_item,
      { "DeltaRSCPPerCell", "rrc.DeltaRSCPPerCell",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_threholdUsedFrequency_delta,
      { "threholdUsedFrequency-delta", "rrc.threholdUsedFrequency_delta",
        FT_INT32, BASE_DEC, NULL, 0,
        "DeltaRSCP", HFILL }},
    { &hf_rrc_threholdNonUsedFrequency_deltaList,
      { "threholdNonUsedFrequency-deltaList", "rrc.threholdNonUsedFrequency_deltaList",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_Inter_FreqEventCriteriaList_v590ext_item,
      { "Inter-FreqEventCriteria-v590ext", "rrc.Inter_FreqEventCriteria_v590ext",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_Intra_FreqEventCriteriaList_v590ext_item,
      { "DeltaRSCP", "rrc.DeltaRSCP",
        FT_INT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_IntraFrequencyMeasuredResultsList_v590ext_item,
      { "DeltaRSCPPerCell", "rrc.DeltaRSCPPerCell",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_eventID_01,
      { "eventID", "rrc.eventID",
        FT_UINT32, BASE_DEC, VALS(rrc_EventIDInterFreq_vals), 0,
        "EventIDInterFreq", HFILL }},
    { &hf_rrc_interFreqCellList,
      { "interFreqCellList", "rrc.interFreqCellList",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_detectedSetTrigger,
      { "detectedSetTrigger", "rrc.detectedSetTrigger",
        FT_UINT32, BASE_DEC, VALS(rrc_T_detectedSetTrigger_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_interFreqCellList_01,
      { "interFreqCellList", "rrc.interFreqCellList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "InterFreqCellList_LCR_r4_ext", HFILL }},
    { &hf_rrc_reportingCriteria,
      { "reportingCriteria", "rrc.reportingCriteria",
        FT_UINT32, BASE_DEC, VALS(rrc_T_reportingCriteria_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_intraFreqReportingCriteria_02,
      { "intraFreqReportingCriteria", "rrc.intraFreqReportingCriteria",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_intraFreqMeasQuantity,
      { "intraFreqMeasQuantity", "rrc.intraFreqMeasQuantity",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_interFreqReportingCriteria,
      { "interFreqReportingCriteria", "rrc.interFreqReportingCriteria",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_modeSpecificInfo_112,
      { "modeSpecificInfo", "rrc.modeSpecificInfo",
        FT_UINT32, BASE_DEC, VALS(rrc_T_modeSpecificInfo_112_vals), 0,
        "T_modeSpecificInfo_112", HFILL }},
    { &hf_rrc_fdd_157,
      { "fdd", "rrc.fdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_fdd_151", HFILL }},
    { &hf_rrc_freqQualityEstimateQuantity_FDD,
      { "freqQualityEstimateQuantity-FDD", "rrc.freqQualityEstimateQuantity_FDD",
        FT_UINT32, BASE_DEC, VALS(rrc_FreqQualityEstimateQuantity_FDD_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_tdd_108,
      { "tdd", "rrc.tdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_tdd_100", HFILL }},
    { &hf_rrc_freqQualityEstimateQuantity_TDD,
      { "freqQualityEstimateQuantity-TDD", "rrc.freqQualityEstimateQuantity_TDD",
        FT_UINT32, BASE_DEC, VALS(rrc_FreqQualityEstimateQuantity_TDD_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_utra_CarrierRSSI,
      { "utra-CarrierRSSI", "rrc.utra_CarrierRSSI",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_interFreqCellMeasuredResultsList,
      { "interFreqCellMeasuredResultsList", "rrc.interFreqCellMeasuredResultsList",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_interFreqCellMeasuredResultsList_01,
      { "interFreqCellMeasuredResultsList", "rrc.interFreqCellMeasuredResultsList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "InterFreqCellMeasuredResultsList_v920ext", HFILL }},
    { &hf_rrc_InterFreqMeasuredResultsList_item,
      { "InterFreqMeasuredResults", "rrc.InterFreqMeasuredResults",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_InterFreqMeasuredResultsList_v920ext_item,
      { "InterFreqMeasuredResults-v920ext", "rrc.InterFreqMeasuredResults_v920ext",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_interFreqCellInfoSI_List,
      { "interFreqCellInfoSI-List", "rrc.interFreqCellInfoSI_List",
        FT_NONE, BASE_NONE, NULL, 0,
        "InterFreqCellInfoSI_List_RSCP", HFILL }},
    { &hf_rrc_interFreqCellInfoSI_List_01,
      { "interFreqCellInfoSI-List", "rrc.interFreqCellInfoSI_List",
        FT_NONE, BASE_NONE, NULL, 0,
        "InterFreqCellInfoSI_List_ECN0", HFILL }},
    { &hf_rrc_interFreqCellInfoSI_List_02,
      { "interFreqCellInfoSI-List", "rrc.interFreqCellInfoSI_List",
        FT_NONE, BASE_NONE, NULL, 0,
        "InterFreqCellInfoSI_List_HCS_RSCP", HFILL }},
    { &hf_rrc_interFreqCellInfoSI_List_03,
      { "interFreqCellInfoSI-List", "rrc.interFreqCellInfoSI_List",
        FT_NONE, BASE_NONE, NULL, 0,
        "InterFreqCellInfoSI_List_HCS_ECN0", HFILL }},
    { &hf_rrc_interFreqCellInfoSI_List_04,
      { "interFreqCellInfoSI-List", "rrc.interFreqCellInfoSI_List",
        FT_NONE, BASE_NONE, NULL, 0,
        "InterFreqCellInfoSI_List_RSCP_LCR", HFILL }},
    { &hf_rrc_interFreqCellInfoSI_List_05,
      { "interFreqCellInfoSI-List", "rrc.interFreqCellInfoSI_List",
        FT_NONE, BASE_NONE, NULL, 0,
        "InterFreqCellInfoSI_List_ECN0_LCR", HFILL }},
    { &hf_rrc_interFreqCellInfoSI_List_06,
      { "interFreqCellInfoSI-List", "rrc.interFreqCellInfoSI_List",
        FT_NONE, BASE_NONE, NULL, 0,
        "InterFreqCellInfoSI_List_HCS_RSCP_LCR", HFILL }},
    { &hf_rrc_interFreqCellInfoSI_List_07,
      { "interFreqCellInfoSI-List", "rrc.interFreqCellInfoSI_List",
        FT_NONE, BASE_NONE, NULL, 0,
        "InterFreqCellInfoSI_List_HCS_ECN0_LCR", HFILL }},
    { &hf_rrc_InterFreqRACHRepCellsList_item,
      { "InterFreqCellID", "rrc.InterFreqCellID",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_modeSpecificInfo_113,
      { "modeSpecificInfo", "rrc.modeSpecificInfo",
        FT_UINT32, BASE_DEC, VALS(rrc_T_modeSpecificInfo_113_vals), 0,
        "T_modeSpecificInfo_113", HFILL }},
    { &hf_rrc_fdd_158,
      { "fdd", "rrc.fdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_fdd_152", HFILL }},
    { &hf_rrc_interFreqRepQuantityRACH_FDD,
      { "interFreqRepQuantityRACH-FDD", "rrc.interFreqRepQuantityRACH_FDD",
        FT_UINT32, BASE_DEC, VALS(rrc_InterFreqRepQuantityRACH_FDD_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_tdd_109,
      { "tdd", "rrc.tdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_tdd_101", HFILL }},
    { &hf_rrc_interFreqRepQuantityRACH_TDDList,
      { "interFreqRepQuantityRACH-TDDList", "rrc.interFreqRepQuantityRACH_TDDList",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_interFreqRACHReportingThreshold,
      { "interFreqRACHReportingThreshold", "rrc.interFreqRACHReportingThreshold",
        FT_INT32, BASE_DEC, NULL, 0,
        "Threshold", HFILL }},
    { &hf_rrc_maxReportedCellsOnRACHinterFreq,
      { "maxReportedCellsOnRACHinterFreq", "rrc.maxReportedCellsOnRACHinterFreq",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_modeSpecificInfo_114,
      { "modeSpecificInfo", "rrc.modeSpecificInfo",
        FT_UINT32, BASE_DEC, VALS(rrc_T_modeSpecificInfo_114_vals), 0,
        "T_modeSpecificInfo_114", HFILL }},
    { &hf_rrc_fdd_159,
      { "fdd", "rrc.fdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_fdd_153", HFILL }},
    { &hf_rrc_tdd_110,
      { "tdd", "rrc.tdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_tdd_102", HFILL }},
    { &hf_rrc_interFreqReportingCriteria_01,
      { "interFreqReportingCriteria", "rrc.interFreqReportingCriteria",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_periodicalReportingCriteria_01,
      { "periodicalReportingCriteria", "rrc.periodicalReportingCriteria",
        FT_NONE, BASE_NONE, NULL, 0,
        "PeriodicalWithReportingCellStatus", HFILL }},
    { &hf_rrc_noReporting,
      { "noReporting", "rrc.noReporting",
        FT_NONE, BASE_NONE, NULL, 0,
        "ReportingCellStatusOpt", HFILL }},
    { &hf_rrc_intraFreqReportingCriteria_03,
      { "intraFreqReportingCriteria", "rrc.intraFreqReportingCriteria",
        FT_NONE, BASE_NONE, NULL, 0,
        "IntraFreqReportingCriteria_r4", HFILL }},
    { &hf_rrc_intraFreqReportingCriteria_04,
      { "intraFreqReportingCriteria", "rrc.intraFreqReportingCriteria",
        FT_NONE, BASE_NONE, NULL, 0,
        "IntraFreqReportingCriteria_r6", HFILL }},
    { &hf_rrc_interFreqReportingCriteria_02,
      { "interFreqReportingCriteria", "rrc.interFreqReportingCriteria",
        FT_NONE, BASE_NONE, NULL, 0,
        "InterFreqReportingCriteria_r6", HFILL }},
    { &hf_rrc_intraFreqReportingCriteria_05,
      { "intraFreqReportingCriteria", "rrc.intraFreqReportingCriteria",
        FT_NONE, BASE_NONE, NULL, 0,
        "IntraFreqReportingCriteria_r7", HFILL }},
    { &hf_rrc_intraFreqReportingCriteria_06,
      { "intraFreqReportingCriteria", "rrc.intraFreqReportingCriteria",
        FT_NONE, BASE_NONE, NULL, 0,
        "IntraFreqReportingCriteria_r9", HFILL }},
    { &hf_rrc_interFreqReportingCriteria_03,
      { "interFreqReportingCriteria", "rrc.interFreqReportingCriteria",
        FT_NONE, BASE_NONE, NULL, 0,
        "InterFreqReportingCriteria_r10", HFILL }},
    { &hf_rrc_periodicalReportingCriteria_02,
      { "periodicalReportingCriteria", "rrc.periodicalReportingCriteria",
        FT_NONE, BASE_NONE, NULL, 0,
        "PeriodicalWithReportingCellStatus_r10", HFILL }},
    { &hf_rrc_noReporting_01,
      { "noReporting", "rrc.noReporting",
        FT_NONE, BASE_NONE, NULL, 0,
        "ReportingCellStatusOpt_r10", HFILL }},
    { &hf_rrc_intraFreqReportingCriteria_07,
      { "intraFreqReportingCriteria", "rrc.intraFreqReportingCriteria",
        FT_NONE, BASE_NONE, NULL, 0,
        "IntraFreqReportingCriteria_r11", HFILL }},
    { &hf_rrc_interFreqReportingCriteria_04,
      { "interFreqReportingCriteria", "rrc.interFreqReportingCriteria",
        FT_NONE, BASE_NONE, NULL, 0,
        "InterFreqReportingCriteria_r11", HFILL }},
    { &hf_rrc_interFreqEventList,
      { "interFreqEventList", "rrc.interFreqEventList",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_interFreqEventList_01,
      { "interFreqEventList", "rrc.interFreqEventList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "InterFreqEventList_r6", HFILL }},
    { &hf_rrc_interFreqEventList_02,
      { "interFreqEventList", "rrc.interFreqEventList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "InterFreqEventList_r10", HFILL }},
    { &hf_rrc_interFreqEventList_03,
      { "interFreqEventList", "rrc.interFreqEventList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "InterFreqEventList_r11", HFILL }},
    { &hf_rrc_utra_Carrier_RSSI,
      { "utra-Carrier-RSSI", "rrc.utra_Carrier_RSSI",
        FT_BOOLEAN, BASE_NONE, NULL, 0,
        "BOOLEAN", HFILL }},
    { &hf_rrc_frequencyQualityEstimate,
      { "frequencyQualityEstimate", "rrc.frequencyQualityEstimate",
        FT_BOOLEAN, BASE_NONE, NULL, 0,
        "BOOLEAN", HFILL }},
    { &hf_rrc_nonFreqRelatedQuantities,
      { "nonFreqRelatedQuantities", "rrc.nonFreqRelatedQuantities",
        FT_NONE, BASE_NONE, NULL, 0,
        "CellReportingQuantities", HFILL }},
    { &hf_rrc_InterFreqRepQuantityRACH_TDDList_item,
      { "InterFreqRepQuantityRACH-TDD", "rrc.InterFreqRepQuantityRACH_TDD",
        FT_UINT32, BASE_DEC, VALS(rrc_InterFreqRepQuantityRACH_TDD_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_interFreqCellInfoList,
      { "interFreqCellInfoList", "rrc.interFreqCellInfoList",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_interFreqMeasQuantity,
      { "interFreqMeasQuantity", "rrc.interFreqMeasQuantity",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_interFreqReportingQuantity,
      { "interFreqReportingQuantity", "rrc.interFreqReportingQuantity",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_measurementValidity,
      { "measurementValidity", "rrc.measurementValidity",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_interFreqSetUpdate,
      { "interFreqSetUpdate", "rrc.interFreqSetUpdate",
        FT_UINT32, BASE_DEC, VALS(rrc_UE_AutonomousUpdateMode_vals), 0,
        "UE_AutonomousUpdateMode", HFILL }},
    { &hf_rrc_reportCriteria_01,
      { "reportCriteria", "rrc.reportCriteria",
        FT_UINT32, BASE_DEC, VALS(rrc_InterFreqReportCriteria_vals), 0,
        "InterFreqReportCriteria", HFILL }},
    { &hf_rrc_interFreqCellInfoList_01,
      { "interFreqCellInfoList", "rrc.interFreqCellInfoList",
        FT_NONE, BASE_NONE, NULL, 0,
        "InterFreqCellInfoList_r4", HFILL }},
    { &hf_rrc_reportCriteria_02,
      { "reportCriteria", "rrc.reportCriteria",
        FT_UINT32, BASE_DEC, VALS(rrc_InterFreqReportCriteria_r4_vals), 0,
        "InterFreqReportCriteria_r4", HFILL }},
    { &hf_rrc_reportCriteria_03,
      { "reportCriteria", "rrc.reportCriteria",
        FT_UINT32, BASE_DEC, VALS(rrc_InterFreqReportCriteria_r6_vals), 0,
        "InterFreqReportCriteria_r6", HFILL }},
    { &hf_rrc_reportCriteria_04,
      { "reportCriteria", "rrc.reportCriteria",
        FT_UINT32, BASE_DEC, VALS(rrc_InterFreqReportCriteria_r7_vals), 0,
        "InterFreqReportCriteria_r7", HFILL }},
    { &hf_rrc_interFreqCellInfoList_02,
      { "interFreqCellInfoList", "rrc.interFreqCellInfoList",
        FT_NONE, BASE_NONE, NULL, 0,
        "InterFreqCellInfoList_r8", HFILL }},
    { &hf_rrc_adjacentFrequencyIndex,
      { "adjacentFrequencyIndex", "rrc.adjacentFrequencyIndex",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_31", HFILL }},
    { &hf_rrc_interFreqCellInfoList_03,
      { "interFreqCellInfoList", "rrc.interFreqCellInfoList",
        FT_NONE, BASE_NONE, NULL, 0,
        "InterFreqCellInfoList_r9", HFILL }},
    { &hf_rrc_interBandFrequencyIndex,
      { "interBandFrequencyIndex", "rrc.interBandFrequencyIndex",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_31", HFILL }},
    { &hf_rrc_reportCriteria_05,
      { "reportCriteria", "rrc.reportCriteria",
        FT_UINT32, BASE_DEC, VALS(rrc_InterFreqReportCriteria_r9_vals), 0,
        "InterFreqReportCriteria_r9", HFILL }},
    { &hf_rrc_interFreqCellInfoList_04,
      { "interFreqCellInfoList", "rrc.interFreqCellInfoList",
        FT_NONE, BASE_NONE, NULL, 0,
        "InterFreqCellInfoList_r10", HFILL }},
    { &hf_rrc_freqIndexListForEnhancedMeas,
      { "freqIndexListForEnhancedMeas", "rrc.freqIndexListForEnhancedMeas",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_reportCriteria_06,
      { "reportCriteria", "rrc.reportCriteria",
        FT_UINT32, BASE_DEC, VALS(rrc_InterFreqReportCriteria_r10_vals), 0,
        "InterFreqReportCriteria_r10", HFILL }},
    { &hf_rrc_reportCriteria_07,
      { "reportCriteria", "rrc.reportCriteria",
        FT_UINT32, BASE_DEC, VALS(rrc_InterFreqReportCriteria_r11_vals), 0,
        "InterFreqReportCriteria_r11", HFILL }},
    { &hf_rrc_technologySpecificInfo,
      { "technologySpecificInfo", "rrc.technologySpecificInfo",
        FT_UINT32, BASE_DEC, VALS(rrc_T_technologySpecificInfo_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_gsm_08,
      { "gsm", "rrc.gsm",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_gsm_08", HFILL }},
    { &hf_rrc_bsic,
      { "bsic", "rrc.bsic",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_bcch_ARFCN,
      { "bcch-ARFCN", "rrc.bcch_ARFCN",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_ncMode,
      { "ncMode", "rrc.ncMode",
        FT_BYTES, BASE_NONE, NULL, 0,
        "NC_Mode", HFILL }},
    { &hf_rrc_is_2000,
      { "is-2000", "rrc.is_2000",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_removedInterRATCellList,
      { "removedInterRATCellList", "rrc.removedInterRATCellList",
        FT_UINT32, BASE_DEC, VALS(rrc_RemovedInterRATCellList_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_newInterRATCellList,
      { "newInterRATCellList", "rrc.newInterRATCellList",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_cellsForInterRATMeasList,
      { "cellsForInterRATMeasList", "rrc.cellsForInterRATMeasList",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_newInterRATCellList_01,
      { "newInterRATCellList", "rrc.newInterRATCellList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "NewInterRATCellList_B", HFILL }},
    { &hf_rrc_interRATCellInfoIndication_r6,
      { "interRATCellInfoIndication-r6", "rrc.interRATCellInfoIndication_r6",
        FT_UINT32, BASE_DEC, NULL, 0,
        "InterRATCellInfoIndication", HFILL }},
    { &hf_rrc_event3a,
      { "event3a", "rrc.event3a",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_event3b,
      { "event3b", "rrc.event3b",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_event3c,
      { "event3c", "rrc.event3c",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_event3d,
      { "event3d", "rrc.event3d",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_InterRATEventList_item,
      { "InterRATEvent", "rrc.InterRATEvent",
        FT_UINT32, BASE_DEC, VALS(rrc_InterRATEvent_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_cellToReportList,
      { "cellToReportList", "rrc.cellToReportList",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_rat_01,
      { "rat", "rrc.rat",
        FT_UINT32, BASE_DEC, VALS(rrc_InterRATInfo_vals), 0,
        "InterRATInfo", HFILL }},
    { &hf_rrc_gsm_TargetCellInfoList,
      { "gsm-TargetCellInfoList", "rrc.gsm_TargetCellInfoList",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_gsm_09,
      { "gsm", "rrc.gsm",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_eutra_01,
      { "eutra", "rrc.eutra",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_eutra_01", HFILL }},
    { &hf_rrc_eutra_TargetFreqInfoList,
      { "eutra-TargetFreqInfoList", "rrc.eutra_TargetFreqInfoList",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_measQuantityUTRAN_QualityEstimate,
      { "measQuantityUTRAN-QualityEstimate", "rrc.measQuantityUTRAN_QualityEstimate",
        FT_NONE, BASE_NONE, NULL, 0,
        "IntraFreqMeasQuantity", HFILL }},
    { &hf_rrc_ratSpecificInfo,
      { "ratSpecificInfo", "rrc.ratSpecificInfo",
        FT_UINT32, BASE_DEC, VALS(rrc_T_ratSpecificInfo_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_gsm_10,
      { "gsm", "rrc.gsm",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_gsm_09", HFILL }},
    { &hf_rrc_measurementQuantity_01,
      { "measurementQuantity", "rrc.measurementQuantity",
        FT_UINT32, BASE_DEC, VALS(rrc_MeasurementQuantityGSM_vals), 0,
        "MeasurementQuantityGSM", HFILL }},
    { &hf_rrc_bsic_VerificationRequired,
      { "bsic-VerificationRequired", "rrc.bsic_VerificationRequired",
        FT_UINT32, BASE_DEC, VALS(rrc_BSIC_VerificationRequired_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_is_2000_01,
      { "is-2000", "rrc.is_2000",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_tadd_EcIo,
      { "tadd-EcIo", "rrc.tadd_EcIo",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_63", HFILL }},
    { &hf_rrc_tcomp_EcIo,
      { "tcomp-EcIo", "rrc.tcomp_EcIo",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_15", HFILL }},
    { &hf_rrc_softSlope,
      { "softSlope", "rrc.softSlope",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_63", HFILL }},
    { &hf_rrc_addIntercept,
      { "addIntercept", "rrc.addIntercept",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_63", HFILL }},
    { &hf_rrc_ratSpecificInfo_01,
      { "ratSpecificInfo", "rrc.ratSpecificInfo",
        FT_UINT32, BASE_DEC, VALS(rrc_T_ratSpecificInfo_01_vals), 0,
        "T_ratSpecificInfo_01", HFILL }},
    { &hf_rrc_gsm_11,
      { "gsm", "rrc.gsm",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_gsm_10", HFILL }},
    { &hf_rrc_is_2000_02,
      { "is-2000", "rrc.is_2000",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_is_2000_01", HFILL }},
    { &hf_rrc_e_UTRA_01,
      { "e-UTRA", "rrc.e_UTRA",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_e_UTRA_01", HFILL }},
    { &hf_rrc_measurementQuantity_02,
      { "measurementQuantity", "rrc.measurementQuantity",
        FT_UINT32, BASE_DEC, VALS(rrc_MeasurementQuantityEUTRA_vals), 0,
        "MeasurementQuantityEUTRA", HFILL }},
    { &hf_rrc_gsm_12,
      { "gsm", "rrc.gsm",
        FT_UINT32, BASE_DEC, NULL, 0,
        "GSM_MeasuredResultsList", HFILL }},
    { &hf_rrc_InterRATMeasuredResultsList_item,
      { "InterRATMeasuredResults", "rrc.InterRATMeasuredResults",
        FT_UINT32, BASE_DEC, VALS(rrc_InterRATMeasuredResults_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_interRATCellInfoList,
      { "interRATCellInfoList", "rrc.interRATCellInfoList",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_interRATMeasQuantity,
      { "interRATMeasQuantity", "rrc.interRATMeasQuantity",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_interRATReportingQuantity,
      { "interRATReportingQuantity", "rrc.interRATReportingQuantity",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_reportCriteria_08,
      { "reportCriteria", "rrc.reportCriteria",
        FT_UINT32, BASE_DEC, VALS(rrc_InterRATReportCriteria_vals), 0,
        "InterRATReportCriteria", HFILL }},
    { &hf_rrc_interRATCellInfoList_01,
      { "interRATCellInfoList", "rrc.interRATCellInfoList",
        FT_NONE, BASE_NONE, NULL, 0,
        "InterRATCellInfoList_r4", HFILL }},
    { &hf_rrc_interRATCellInfoList_02,
      { "interRATCellInfoList", "rrc.interRATCellInfoList",
        FT_NONE, BASE_NONE, NULL, 0,
        "InterRATCellInfoList_r6", HFILL }},
    { &hf_rrc_interRATMeasurementObjects,
      { "interRATMeasurementObjects", "rrc.interRATMeasurementObjects",
        FT_UINT32, BASE_DEC, VALS(rrc_T_interRATMeasurementObjects_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_eutra_FrequencyList,
      { "eutra-FrequencyList", "rrc.eutra_FrequencyList",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_interRATMeasQuantity_01,
      { "interRATMeasQuantity", "rrc.interRATMeasQuantity",
        FT_NONE, BASE_NONE, NULL, 0,
        "InterRATMeasQuantity_r8", HFILL }},
    { &hf_rrc_interRATReportingQuantity_01,
      { "interRATReportingQuantity", "rrc.interRATReportingQuantity",
        FT_NONE, BASE_NONE, NULL, 0,
        "InterRATReportingQuantity_r8", HFILL }},
    { &hf_rrc_idleIntervalInfo,
      { "idleIntervalInfo", "rrc.idleIntervalInfo",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_interRATMeasurementObjects_01,
      { "interRATMeasurementObjects", "rrc.interRATMeasurementObjects",
        FT_UINT32, BASE_DEC, VALS(rrc_T_interRATMeasurementObjects_01_vals), 0,
        "T_interRATMeasurementObjects_01", HFILL }},
    { &hf_rrc_eutra_FrequencyList_01,
      { "eutra-FrequencyList", "rrc.eutra_FrequencyList",
        FT_NONE, BASE_NONE, NULL, 0,
        "EUTRA_FrequencyList_r9", HFILL }},
    { &hf_rrc_interRATMeasurementObjects_02,
      { "interRATMeasurementObjects", "rrc.interRATMeasurementObjects",
        FT_UINT32, BASE_DEC, VALS(rrc_T_interRATMeasurementObjects_02_vals), 0,
        "T_interRATMeasurementObjects_02", HFILL }},
    { &hf_rrc_eutra_FrequencyList_02,
      { "eutra-FrequencyList", "rrc.eutra_FrequencyList",
        FT_NONE, BASE_NONE, NULL, 0,
        "EUTRA_FrequencyList_r11", HFILL }},
    { &hf_rrc_interRATCellInfoList_03,
      { "interRATCellInfoList", "rrc.interRATCellInfoList",
        FT_NONE, BASE_NONE, NULL, 0,
        "InterRATCellInfoList_B", HFILL }},
    { &hf_rrc_interRATReportingCriteria,
      { "interRATReportingCriteria", "rrc.interRATReportingCriteria",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_interRATEventList,
      { "interRATEventList", "rrc.interRATEventList",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_utran_EstimatedQuality,
      { "utran-EstimatedQuality", "rrc.utran_EstimatedQuality",
        FT_BOOLEAN, BASE_NONE, NULL, 0,
        "BOOLEAN", HFILL }},
    { &hf_rrc_ratSpecificInfo_02,
      { "ratSpecificInfo", "rrc.ratSpecificInfo",
        FT_UINT32, BASE_DEC, VALS(rrc_T_ratSpecificInfo_02_vals), 0,
        "T_ratSpecificInfo_02", HFILL }},
    { &hf_rrc_gsm_13,
      { "gsm", "rrc.gsm",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_gsm_11", HFILL }},
    { &hf_rrc_gsm_Carrier_RSSI,
      { "gsm-Carrier-RSSI", "rrc.gsm_Carrier_RSSI",
        FT_BOOLEAN, BASE_NONE, NULL, 0,
        "BOOLEAN", HFILL }},
    { &hf_rrc_ratSpecificInfo_03,
      { "ratSpecificInfo", "rrc.ratSpecificInfo",
        FT_UINT32, BASE_DEC, VALS(rrc_T_ratSpecificInfo_03_vals), 0,
        "T_ratSpecificInfo_03", HFILL }},
    { &hf_rrc_gsm_14,
      { "gsm", "rrc.gsm",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_gsm_12", HFILL }},
    { &hf_rrc_eutra_02,
      { "eutra", "rrc.eutra",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_eutra_02", HFILL }},
    { &hf_rrc_reportingQuantity,
      { "reportingQuantity", "rrc.reportingQuantity",
        FT_UINT32, BASE_DEC, VALS(rrc_T_reportingQuantity_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_removedIntraFreqCellList,
      { "removedIntraFreqCellList", "rrc.removedIntraFreqCellList",
        FT_UINT32, BASE_DEC, VALS(rrc_RemovedIntraFreqCellList_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_newIntraFreqCellList_01,
      { "newIntraFreqCellList", "rrc.newIntraFreqCellList",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_cellsForIntraFreqMeasList,
      { "cellsForIntraFreqMeasList", "rrc.cellsForIntraFreqMeasList",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_newIntraFreqCellList_02,
      { "newIntraFreqCellList", "rrc.newIntraFreqCellList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "NewIntraFreqCellList_r4", HFILL }},
    { &hf_rrc_removedIntraFreqCellList_01,
      { "removedIntraFreqCellList", "rrc.removedIntraFreqCellList",
        FT_UINT32, BASE_DEC, VALS(rrc_RemovedIntraFreqCellListOnSecULFreq_vals), 0,
        "RemovedIntraFreqCellListOnSecULFreq", HFILL }},
    { &hf_rrc_newIntraFreqCellList_03,
      { "newIntraFreqCellList", "rrc.newIntraFreqCellList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "NewIntraFreqCellListOnSecULFreq", HFILL }},
    { &hf_rrc_cellsForIntraFreqMeasList_01,
      { "cellsForIntraFreqMeasList", "rrc.cellsForIntraFreqMeasList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "CellsForIntraFreqMeasListOnSecULFreq", HFILL }},
    { &hf_rrc_intraFreqCellInfoListOnSecULFreq,
      { "intraFreqCellInfoListOnSecULFreq", "rrc.intraFreqCellInfoListOnSecULFreq",
        FT_NONE, BASE_NONE, NULL, 0,
        "IntraFreqCellInfoListInfoOnSecULFreq", HFILL }},
    { &hf_rrc_newIntraFreqCellList_04,
      { "newIntraFreqCellList", "rrc.newIntraFreqCellList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "NewIntraFreqCellList_r9", HFILL }},
    { &hf_rrc_cSGIntraFreqCellInfoList,
      { "cSGIntraFreqCellInfoList", "rrc.cSGIntraFreqCellInfoList",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_intraFreqSIAcquisition,
      { "intraFreqSIAcquisition", "rrc.intraFreqSIAcquisition",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_newIntraFreqCellList_05,
      { "newIntraFreqCellList", "rrc.newIntraFreqCellList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "NewIntraFreqCellList_r10", HFILL }},
    { &hf_rrc_IntraFreqSIAcquisitionInfo_item,
      { "PrimaryCPICH-Info", "rrc.PrimaryCPICH_Info",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_modeSpecificInfo_115,
      { "modeSpecificInfo", "rrc.modeSpecificInfo",
        FT_UINT32, BASE_DEC, VALS(rrc_T_modeSpecificInfo_115_vals), 0,
        "T_modeSpecificInfo_115", HFILL }},
    { &hf_rrc_fdd_160,
      { "fdd", "rrc.fdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_fdd_154", HFILL }},
    { &hf_rrc_intraFreqSIAcquisitionInfo,
      { "intraFreqSIAcquisitionInfo", "rrc.intraFreqSIAcquisitionInfo",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_newIntraFreqCellList_06,
      { "newIntraFreqCellList", "rrc.newIntraFreqCellList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "NewIntraFreqCellSI_List_RSCP", HFILL }},
    { &hf_rrc_newIntraFreqCellList_07,
      { "newIntraFreqCellList", "rrc.newIntraFreqCellList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "NewIntraFreqCellSI_List_ECN0", HFILL }},
    { &hf_rrc_newIntraFreqCellList_08,
      { "newIntraFreqCellList", "rrc.newIntraFreqCellList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "NewIntraFreqCellSI_List_HCS_RSCP", HFILL }},
    { &hf_rrc_newIntraFreqCellList_09,
      { "newIntraFreqCellList", "rrc.newIntraFreqCellList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "NewIntraFreqCellSI_List_HCS_ECN0", HFILL }},
    { &hf_rrc_newIntraFreqCellList_10,
      { "newIntraFreqCellList", "rrc.newIntraFreqCellList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "NewIntraFreqCellSI_List_RSCP_LCR_r4", HFILL }},
    { &hf_rrc_newIntraFreqCellList_11,
      { "newIntraFreqCellList", "rrc.newIntraFreqCellList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "NewIntraFreqCellSI_List_ECN0_LCR_r4", HFILL }},
    { &hf_rrc_newIntraFreqCellList_12,
      { "newIntraFreqCellList", "rrc.newIntraFreqCellList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "NewIntraFreqCellSI_List_HCS_RSCP_LCR_r4", HFILL }},
    { &hf_rrc_newIntraFreqCellList_13,
      { "newIntraFreqCellList", "rrc.newIntraFreqCellList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "NewIntraFreqCellSI_List_HCS_ECN0_LCR_r4", HFILL }},
    { &hf_rrc_e1a,
      { "e1a", "rrc.e1a",
        FT_NONE, BASE_NONE, NULL, 0,
        "Event1a", HFILL }},
    { &hf_rrc_e1b,
      { "e1b", "rrc.e1b",
        FT_NONE, BASE_NONE, NULL, 0,
        "Event1b", HFILL }},
    { &hf_rrc_e1c,
      { "e1c", "rrc.e1c",
        FT_NONE, BASE_NONE, NULL, 0,
        "Event1c", HFILL }},
    { &hf_rrc_e1d,
      { "e1d", "rrc.e1d",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_e1e,
      { "e1e", "rrc.e1e",
        FT_NONE, BASE_NONE, NULL, 0,
        "Event1e", HFILL }},
    { &hf_rrc_e1f,
      { "e1f", "rrc.e1f",
        FT_NONE, BASE_NONE, NULL, 0,
        "Event1f", HFILL }},
    { &hf_rrc_e1g,
      { "e1g", "rrc.e1g",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_e1h,
      { "e1h", "rrc.e1h",
        FT_INT32, BASE_DEC, NULL, 0,
        "ThresholdUsedFrequency", HFILL }},
    { &hf_rrc_e1i,
      { "e1i", "rrc.e1i",
        FT_INT32, BASE_DEC, NULL, 0,
        "ThresholdUsedFrequency", HFILL }},
    { &hf_rrc_e1a_01,
      { "e1a", "rrc.e1a",
        FT_NONE, BASE_NONE, NULL, 0,
        "Event1a_r4", HFILL }},
    { &hf_rrc_e1b_01,
      { "e1b", "rrc.e1b",
        FT_NONE, BASE_NONE, NULL, 0,
        "Event1b_r4", HFILL }},
    { &hf_rrc_e1a_02,
      { "e1a", "rrc.e1a",
        FT_NONE, BASE_NONE, NULL, 0,
        "Event1a_LCR_r4", HFILL }},
    { &hf_rrc_e1b_02,
      { "e1b", "rrc.e1b",
        FT_NONE, BASE_NONE, NULL, 0,
        "Event1b_LCR_r4", HFILL }},
    { &hf_rrc_e1d_01,
      { "e1d", "rrc.e1d",
        FT_NONE, BASE_NONE, NULL, 0,
        "Event1d", HFILL }},
    { &hf_rrc_e1e_01,
      { "e1e", "rrc.e1e",
        FT_NONE, BASE_NONE, NULL, 0,
        "Event1e_r6", HFILL }},
    { &hf_rrc_e1f_01,
      { "e1f", "rrc.e1f",
        FT_NONE, BASE_NONE, NULL, 0,
        "Event1f_r6", HFILL }},
    { &hf_rrc_e1h_01,
      { "e1h", "rrc.e1h",
        FT_INT32, BASE_DEC, NULL, 0,
        "ThresholdUsedFrequency_r6", HFILL }},
    { &hf_rrc_e1i_01,
      { "e1i", "rrc.e1i",
        FT_INT32, BASE_DEC, NULL, 0,
        "ThresholdUsedFrequency_r6", HFILL }},
    { &hf_rrc_e1j,
      { "e1j", "rrc.e1j",
        FT_NONE, BASE_NONE, NULL, 0,
        "Event1j_r6", HFILL }},
    { &hf_rrc_e1b_03,
      { "e1b", "rrc.e1b",
        FT_NONE, BASE_NONE, NULL, 0,
        "Event1b_r7", HFILL }},
    { &hf_rrc_e1a_03,
      { "e1a", "rrc.e1a",
        FT_NONE, BASE_NONE, NULL, 0,
        "Event1a_OnSecUlFreq_r9", HFILL }},
    { &hf_rrc_e1b_04,
      { "e1b", "rrc.e1b",
        FT_NONE, BASE_NONE, NULL, 0,
        "Event1b_OnSecUlFreq_r9", HFILL }},
    { &hf_rrc_event,
      { "event", "rrc.event",
        FT_UINT32, BASE_DEC, VALS(rrc_IntraFreqEvent_vals), 0,
        "IntraFreqEvent", HFILL }},
    { &hf_rrc_event_01,
      { "event", "rrc.event",
        FT_UINT32, BASE_DEC, VALS(rrc_IntraFreqEvent_r4_vals), 0,
        "IntraFreqEvent_r4", HFILL }},
    { &hf_rrc_event_02,
      { "event", "rrc.event",
        FT_UINT32, BASE_DEC, VALS(rrc_IntraFreqEvent_LCR_r4_vals), 0,
        "IntraFreqEvent_LCR_r4", HFILL }},
    { &hf_rrc_event_03,
      { "event", "rrc.event",
        FT_UINT32, BASE_DEC, VALS(rrc_IntraFreqEvent_r6_vals), 0,
        "IntraFreqEvent_r6", HFILL }},
    { &hf_rrc_event_04,
      { "event", "rrc.event",
        FT_UINT32, BASE_DEC, VALS(rrc_IntraFreqEvent_r7_vals), 0,
        "IntraFreqEvent_r7", HFILL }},
    { &hf_rrc_event_05,
      { "event", "rrc.event",
        FT_UINT32, BASE_DEC, VALS(rrc_IntraFreqEventOnSecULFreq_vals), 0,
        "IntraFreqEventOnSecULFreq", HFILL }},
    { &hf_rrc_IntraFreqEventCriteriaList_item,
      { "IntraFreqEventCriteria", "rrc.IntraFreqEventCriteria",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_IntraFreqEventCriteriaList_r4_item,
      { "IntraFreqEventCriteria-r4", "rrc.IntraFreqEventCriteria_r4",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_IntraFreqEventCriteriaList_LCR_r4_item,
      { "IntraFreqEventCriteria-LCR-r4", "rrc.IntraFreqEventCriteria_LCR_r4",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_IntraFreqEventCriteriaList_r6_item,
      { "IntraFreqEventCriteria-r6", "rrc.IntraFreqEventCriteria_r6",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_IntraFreqEventCriteriaList_r7_item,
      { "IntraFreqEventCriteria-r7", "rrc.IntraFreqEventCriteria_r7",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_intraFreqEventCriteria,
      { "intraFreqEventCriteria", "rrc.intraFreqEventCriteria",
        FT_UINT32, BASE_DEC, NULL, 0,
        "SEQUENCE_SIZE_1_maxMeasEventOnSecULFreq_OF_IntraFreqEventCriteriaOnSecULFreq", HFILL }},
    { &hf_rrc_intraFreqEventCriteria_item,
      { "IntraFreqEventCriteriaOnSecULFreq", "rrc.IntraFreqEventCriteriaOnSecULFreq",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_eventID_02,
      { "eventID", "rrc.eventID",
        FT_UINT32, BASE_DEC, VALS(rrc_EventIDIntraFreq_vals), 0,
        "EventIDIntraFreq", HFILL }},
    { &hf_rrc_cellMeasurementEventResults,
      { "cellMeasurementEventResults", "rrc.cellMeasurementEventResults",
        FT_UINT32, BASE_DEC, VALS(rrc_CellMeasurementEventResults_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_cellMeasurementEventResultsOnSecUlFreq,
      { "cellMeasurementEventResultsOnSecUlFreq", "rrc.cellMeasurementEventResultsOnSecUlFreq",
        FT_UINT32, BASE_DEC, VALS(rrc_CellMeasurementEventResultsOnSecUlFreq_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_modeSpecificInfo_116,
      { "modeSpecificInfo", "rrc.modeSpecificInfo",
        FT_UINT32, BASE_DEC, VALS(rrc_T_modeSpecificInfo_116_vals), 0,
        "T_modeSpecificInfo_116", HFILL }},
    { &hf_rrc_fdd_161,
      { "fdd", "rrc.fdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_fdd_155", HFILL }},
    { &hf_rrc_intraFreqMeasQuantity_FDD,
      { "intraFreqMeasQuantity-FDD", "rrc.intraFreqMeasQuantity_FDD",
        FT_UINT32, BASE_DEC, VALS(rrc_IntraFreqMeasQuantity_FDD_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_tdd_111,
      { "tdd", "rrc.tdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_tdd_103", HFILL }},
    { &hf_rrc_intraFreqMeasQuantity_TDDList,
      { "intraFreqMeasQuantity-TDDList", "rrc.intraFreqMeasQuantity_TDDList",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_IntraFreqMeasQuantity_TDDList_item,
      { "IntraFreqMeasQuantity-TDD", "rrc.IntraFreqMeasQuantity_TDD",
        FT_UINT32, BASE_DEC, VALS(rrc_IntraFreqMeasQuantity_TDD_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_IntraFreqMeasQuantity_TDD_sib3List_item,
      { "IntraFreqMeasQuantity-TDD-sib3List item", "rrc.IntraFreqMeasQuantity_TDD_sib3List_item",
        FT_UINT32, BASE_DEC, VALS(rrc_IntraFreqMeasQuantity_TDD_sib3List_item_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_IntraFreqMeasuredResultsList_item,
      { "CellMeasuredResults", "rrc.CellMeasuredResults",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_IntraFreqMeasuredResultsList_v920ext_item,
      { "CellMeasuredResults-v920ext", "rrc.CellMeasuredResults_v920ext",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_intraFreqMeasurementID,
      { "intraFreqMeasurementID", "rrc.intraFreqMeasurementID",
        FT_UINT32, BASE_DEC, NULL, 0,
        "MeasurementIdentity", HFILL }},
    { &hf_rrc_intraFreqCellInfoSI_List,
      { "intraFreqCellInfoSI-List", "rrc.intraFreqCellInfoSI_List",
        FT_NONE, BASE_NONE, NULL, 0,
        "IntraFreqCellInfoSI_List_RSCP", HFILL }},
    { &hf_rrc_intraFreqReportingQuantityForRACH,
      { "intraFreqReportingQuantityForRACH", "rrc.intraFreqReportingQuantityForRACH",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_maxReportedCellsOnRACH,
      { "maxReportedCellsOnRACH", "rrc.maxReportedCellsOnRACH",
        FT_UINT32, BASE_DEC, VALS(rrc_MaxReportedCellsOnRACH_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_reportingInfoForCellDCH,
      { "reportingInfoForCellDCH", "rrc.reportingInfoForCellDCH",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_intraFreqCellInfoSI_List_01,
      { "intraFreqCellInfoSI-List", "rrc.intraFreqCellInfoSI_List",
        FT_NONE, BASE_NONE, NULL, 0,
        "IntraFreqCellInfoSI_List_ECN0", HFILL }},
    { &hf_rrc_intraFreqCellInfoSI_List_02,
      { "intraFreqCellInfoSI-List", "rrc.intraFreqCellInfoSI_List",
        FT_NONE, BASE_NONE, NULL, 0,
        "IntraFreqCellInfoSI_List_HCS_RSCP", HFILL }},
    { &hf_rrc_intraFreqCellInfoSI_List_03,
      { "intraFreqCellInfoSI-List", "rrc.intraFreqCellInfoSI_List",
        FT_NONE, BASE_NONE, NULL, 0,
        "IntraFreqCellInfoSI_List_HCS_ECN0", HFILL }},
    { &hf_rrc_intraFreqCellInfoSI_List_04,
      { "intraFreqCellInfoSI-List", "rrc.intraFreqCellInfoSI_List",
        FT_NONE, BASE_NONE, NULL, 0,
        "IntraFreqCellInfoSI_List_RSCP_LCR_r4", HFILL }},
    { &hf_rrc_reportingInfoForCellDCH_01,
      { "reportingInfoForCellDCH", "rrc.reportingInfoForCellDCH",
        FT_NONE, BASE_NONE, NULL, 0,
        "ReportingInfoForCellDCH_LCR_r4", HFILL }},
    { &hf_rrc_intraFreqCellInfoSI_List_05,
      { "intraFreqCellInfoSI-List", "rrc.intraFreqCellInfoSI_List",
        FT_NONE, BASE_NONE, NULL, 0,
        "IntraFreqCellInfoSI_List_ECN0_LCR_r4", HFILL }},
    { &hf_rrc_intraFreqCellInfoSI_List_06,
      { "intraFreqCellInfoSI-List", "rrc.intraFreqCellInfoSI_List",
        FT_NONE, BASE_NONE, NULL, 0,
        "IntraFreqCellInfoSI_List_HCS_RSCP_LCR_r4", HFILL }},
    { &hf_rrc_intraFreqCellInfoSI_List_07,
      { "intraFreqCellInfoSI-List", "rrc.intraFreqCellInfoSI_List",
        FT_NONE, BASE_NONE, NULL, 0,
        "IntraFreqCellInfoSI_List_HCS_ECN0_LCR_r4", HFILL }},
    { &hf_rrc_eventCriteriaList,
      { "eventCriteriaList", "rrc.eventCriteriaList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "IntraFreqEventCriteriaList", HFILL }},
    { &hf_rrc_eventCriteriaList_01,
      { "eventCriteriaList", "rrc.eventCriteriaList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "IntraFreqEventCriteriaList_r4", HFILL }},
    { &hf_rrc_eventCriteriaList_02,
      { "eventCriteriaList", "rrc.eventCriteriaList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "IntraFreqEventCriteriaList_LCR_r4", HFILL }},
    { &hf_rrc_eventCriteriaList_03,
      { "eventCriteriaList", "rrc.eventCriteriaList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "IntraFreqEventCriteriaList_r6", HFILL }},
    { &hf_rrc_eventCriteriaList_04,
      { "eventCriteriaList", "rrc.eventCriteriaList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "IntraFreqEventCriteriaList_r7", HFILL }},
    { &hf_rrc_eventCriteriaListOnSecULFreq,
      { "eventCriteriaListOnSecULFreq", "rrc.eventCriteriaListOnSecULFreq",
        FT_NONE, BASE_NONE, NULL, 0,
        "IntraFreqEventCriteriaListOnSecULFreq", HFILL }},
    { &hf_rrc_excludedCellInfoList,
      { "excludedCellInfoList", "rrc.excludedCellInfoList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "CellInfoListToBeExcluded_r11", HFILL }},
    { &hf_rrc_activeSetReportingQuantities,
      { "activeSetReportingQuantities", "rrc.activeSetReportingQuantities",
        FT_NONE, BASE_NONE, NULL, 0,
        "CellReportingQuantities", HFILL }},
    { &hf_rrc_monitoredSetReportingQuantities,
      { "monitoredSetReportingQuantities", "rrc.monitoredSetReportingQuantities",
        FT_NONE, BASE_NONE, NULL, 0,
        "CellReportingQuantities", HFILL }},
    { &hf_rrc_detectedSetReportingQuantities,
      { "detectedSetReportingQuantities", "rrc.detectedSetReportingQuantities",
        FT_NONE, BASE_NONE, NULL, 0,
        "CellReportingQuantities", HFILL }},
    { &hf_rrc_sfn_SFN_OTD_Type,
      { "sfn-SFN-OTD-Type", "rrc.sfn_SFN_OTD_Type",
        FT_UINT32, BASE_DEC, VALS(rrc_SFN_SFN_OTD_Type_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_modeSpecificInfo_117,
      { "modeSpecificInfo", "rrc.modeSpecificInfo",
        FT_UINT32, BASE_DEC, VALS(rrc_T_modeSpecificInfo_117_vals), 0,
        "T_modeSpecificInfo_117", HFILL }},
    { &hf_rrc_fdd_162,
      { "fdd", "rrc.fdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_fdd_156", HFILL }},
    { &hf_rrc_intraFreqRepQuantityRACH_FDD,
      { "intraFreqRepQuantityRACH-FDD", "rrc.intraFreqRepQuantityRACH_FDD",
        FT_UINT32, BASE_DEC, VALS(rrc_IntraFreqRepQuantityRACH_FDD_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_tdd_112,
      { "tdd", "rrc.tdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_tdd_104", HFILL }},
    { &hf_rrc_intraFreqRepQuantityRACH_TDDList,
      { "intraFreqRepQuantityRACH-TDDList", "rrc.intraFreqRepQuantityRACH_TDDList",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_IntraFreqRepQuantityRACH_TDDList_item,
      { "IntraFreqRepQuantityRACH-TDD", "rrc.IntraFreqRepQuantityRACH_TDD",
        FT_UINT32, BASE_DEC, VALS(rrc_IntraFreqRepQuantityRACH_TDD_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_intraFreqCellInfoList,
      { "intraFreqCellInfoList", "rrc.intraFreqCellInfoList",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_intraFreqReportingQuantity,
      { "intraFreqReportingQuantity", "rrc.intraFreqReportingQuantity",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_reportCriteria_09,
      { "reportCriteria", "rrc.reportCriteria",
        FT_UINT32, BASE_DEC, VALS(rrc_IntraFreqReportCriteria_vals), 0,
        "IntraFreqReportCriteria", HFILL }},
    { &hf_rrc_intraFreqCellInfoList_01,
      { "intraFreqCellInfoList", "rrc.intraFreqCellInfoList",
        FT_NONE, BASE_NONE, NULL, 0,
        "IntraFreqCellInfoList_r4", HFILL }},
    { &hf_rrc_reportCriteria_10,
      { "reportCriteria", "rrc.reportCriteria",
        FT_UINT32, BASE_DEC, VALS(rrc_IntraFreqReportCriteria_r4_vals), 0,
        "IntraFreqReportCriteria_r4", HFILL }},
    { &hf_rrc_reportCriteria_11,
      { "reportCriteria", "rrc.reportCriteria",
        FT_UINT32, BASE_DEC, VALS(rrc_IntraFreqReportCriteria_r6_vals), 0,
        "IntraFreqReportCriteria_r6", HFILL }},
    { &hf_rrc_reportCriteria_12,
      { "reportCriteria", "rrc.reportCriteria",
        FT_UINT32, BASE_DEC, VALS(rrc_IntraFreqReportCriteria_r7_vals), 0,
        "IntraFreqReportCriteria_r7", HFILL }},
    { &hf_rrc_intraFreqCellInfoList_02,
      { "intraFreqCellInfoList", "rrc.intraFreqCellInfoList",
        FT_NONE, BASE_NONE, NULL, 0,
        "IntraFreqCellInfoList_r9", HFILL }},
    { &hf_rrc_intraFreqCellInfoListOnSecULFreq_01,
      { "intraFreqCellInfoListOnSecULFreq", "rrc.intraFreqCellInfoListOnSecULFreq",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_reportCriteria_13,
      { "reportCriteria", "rrc.reportCriteria",
        FT_UINT32, BASE_DEC, VALS(rrc_IntraFreqReportCriteria_r9_vals), 0,
        "IntraFreqReportCriteria_r9", HFILL }},
    { &hf_rrc_intraFreqCellInfoList_03,
      { "intraFreqCellInfoList", "rrc.intraFreqCellInfoList",
        FT_NONE, BASE_NONE, NULL, 0,
        "IntraFreqCellInfoList_r10", HFILL }},
    { &hf_rrc_reportCriteria_14,
      { "reportCriteria", "rrc.reportCriteria",
        FT_UINT32, BASE_DEC, VALS(rrc_IntraFreqReportCriteria_r11_vals), 0,
        "IntraFreqReportCriteria_r11", HFILL }},
    { &hf_rrc_IntraFrequencyMeasuredResultsListOnSecULFreq_item,
      { "CellMeasuredResults-r9", "rrc.CellMeasuredResults_r9",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_loggingAbsoluteThreshold,
      { "loggingAbsoluteThreshold", "rrc.loggingAbsoluteThreshold",
        FT_UINT32, BASE_DEC, VALS(rrc_LoggingAbsoluteThreshold_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_loggingRelativeThreshold,
      { "loggingRelativeThreshold", "rrc.loggingRelativeThreshold",
        FT_INT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_loggingDuration,
      { "loggingDuration", "rrc.loggingDuration",
        FT_UINT32, BASE_DEC, VALS(rrc_T_loggingDuration_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_intraUTRAANR,
      { "intraUTRAANR", "rrc.intraUTRAANR",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_interRATANRforEUTRAIndicator,
      { "interRATANRforEUTRAIndicator", "rrc.interRATANRforEUTRAIndicator",
        FT_UINT32, BASE_DEC, VALS(rrc_T_interRATANRforEUTRAIndicator_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_interRATANRforGSMIndicator,
      { "interRATANRforGSMIndicator", "rrc.interRATANRforGSMIndicator",
        FT_UINT32, BASE_DEC, VALS(rrc_T_interRATANRforGSMIndicator_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_LoggedANRReportInfoList_item,
      { "LoggedANRReportInfo", "rrc.LoggedANRReportInfo",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_servingCell,
      { "servingCell", "rrc.servingCell",
        FT_BYTES, BASE_NONE, NULL, 0,
        "CellIdentity", HFILL }},
    { &hf_rrc_loggedCellInfo,
      { "loggedCellInfo", "rrc.loggedCellInfo",
        FT_UINT32, BASE_DEC, VALS(rrc_LoggedCellInfo_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_loggedUTRACellInfo,
      { "loggedUTRACellInfo", "rrc.loggedUTRACellInfo",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_loggedEUTRACellInfo,
      { "loggedEUTRACellInfo", "rrc.loggedEUTRACellInfo",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_loggedGSMCellInfo,
      { "loggedGSMCellInfo", "rrc.loggedGSMCellInfo",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_loggedConnectionEstablishmentFailureAccessInfo_FDD,
      { "loggedConnectionEstablishmentFailureAccessInfo-FDD", "rrc.loggedConnectionEstablishmentFailureAccessInfo_FDD",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_loggedMeasFailedCellMeas,
      { "loggedMeasFailedCellMeas", "rrc.loggedMeasFailedCellMeas",
        FT_NONE, BASE_NONE, NULL, 0,
        "LoggedMeasServingCellMeas_FDD", HFILL }},
    { &hf_rrc_loggedMeasIntrafreqNeighbourList,
      { "loggedMeasIntrafreqNeighbourList", "rrc.loggedMeasIntrafreqNeighbourList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "LoggedMeasIntrafreqNeighbourList_FDD", HFILL }},
    { &hf_rrc_loggedMeasInterfreqList,
      { "loggedMeasInterfreqList", "rrc.loggedMeasInterfreqList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "LoggedMeasInterfreqList_FDD", HFILL }},
    { &hf_rrc_loggedMeasInterRATNeighbourMeas,
      { "loggedMeasInterRATNeighbourMeas", "rrc.loggedMeasInterRATNeighbourMeas",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_loggedConnectionEstablishmentFailureLocationInfo,
      { "loggedConnectionEstablishmentFailureLocationInfo", "rrc.loggedConnectionEstablishmentFailureLocationInfo",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_loggedConnectionEstablishmentFailureAccessInfo_TDD,
      { "loggedConnectionEstablishmentFailureAccessInfo-TDD", "rrc.loggedConnectionEstablishmentFailureAccessInfo_TDD",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_loggedMeasFailedCellMeas_01,
      { "loggedMeasFailedCellMeas", "rrc.loggedMeasFailedCellMeas",
        FT_NONE, BASE_NONE, NULL, 0,
        "LoggedMeasServingCellMeas_TDD128", HFILL }},
    { &hf_rrc_loggedMeasIntrafreqNeighbourList_01,
      { "loggedMeasIntrafreqNeighbourList", "rrc.loggedMeasIntrafreqNeighbourList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "LoggedMeasIntrafreqNeighbourMeasList_TDD128", HFILL }},
    { &hf_rrc_loggedMeasInterfreqList_01,
      { "loggedMeasInterfreqList", "rrc.loggedMeasInterfreqList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "LoggedMeasInterfreqList_TDD128", HFILL }},
    { &hf_rrc_numberOfRRCMsgTransmitted,
      { "numberOfRRCMsgTransmitted", "rrc.numberOfRRCMsgTransmitted",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_8", HFILL }},
    { &hf_rrc_fpachReceived,
      { "fpachReceived", "rrc.fpachReceived",
        FT_BOOLEAN, BASE_NONE, NULL, 0,
        "BOOLEAN", HFILL }},
    { &hf_rrc_e_RUCCHFailure,
      { "e-RUCCHFailure", "rrc.e_RUCCHFailure",
        FT_BOOLEAN, BASE_NONE, NULL, 0,
        "BOOLEAN", HFILL }},
    { &hf_rrc_location,
      { "location", "rrc.location",
        FT_UINT32, BASE_DEC, VALS(rrc_T_location_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_ellipsoidPoint,
      { "ellipsoidPoint", "rrc.ellipsoidPoint",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_ellipsoidPointUncertCircle,
      { "ellipsoidPointUncertCircle", "rrc.ellipsoidPointUncertCircle",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_ellipsoidPointUncertEllipse,
      { "ellipsoidPointUncertEllipse", "rrc.ellipsoidPointUncertEllipse",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_ellipsoidPointAltitude,
      { "ellipsoidPointAltitude", "rrc.ellipsoidPointAltitude",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_ellipsoidPointAltitudeEllipse,
      { "ellipsoidPointAltitudeEllipse", "rrc.ellipsoidPointAltitudeEllipse",
        FT_NONE, BASE_NONE, NULL, 0,
        "EllipsoidPointAltitudeEllipsoide", HFILL }},
    { &hf_rrc_horizontalVelocity,
      { "horizontalVelocity", "rrc.horizontalVelocity",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_timeSinceFailure,
      { "timeSinceFailure", "rrc.timeSinceFailure",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_1_172800", HFILL }},
    { &hf_rrc_modeSpecificInfo_118,
      { "modeSpecificInfo", "rrc.modeSpecificInfo",
        FT_UINT32, BASE_DEC, VALS(rrc_T_modeSpecificInfo_118_vals), 0,
        "T_modeSpecificInfo_118", HFILL }},
    { &hf_rrc_fdd_163,
      { "fdd", "rrc.fdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_fdd_157", HFILL }},
    { &hf_rrc_loggedConnectionEstablishmentFailureInfo,
      { "loggedConnectionEstablishmentFailureInfo", "rrc.loggedConnectionEstablishmentFailureInfo",
        FT_NONE, BASE_NONE, NULL, 0,
        "LoggedConnectionEstablishmentFailureInfo_FDD", HFILL }},
    { &hf_rrc_tdd_113,
      { "tdd", "rrc.tdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_tdd_105", HFILL }},
    { &hf_rrc_loggedConnectionEstablishmentFailureInfo_01,
      { "loggedConnectionEstablishmentFailureInfo", "rrc.loggedConnectionEstablishmentFailureInfo",
        FT_NONE, BASE_NONE, NULL, 0,
        "LoggedConnectionEstablishmentFailureInfo_TDD128", HFILL }},
    { &hf_rrc_eutraCellIdentity,
      { "eutraCellIdentity", "rrc.eutraCellIdentity",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_28", HFILL }},
    { &hf_rrc_gsmCellIdentity,
      { "gsmCellIdentity", "rrc.gsmCellIdentity",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_16", HFILL }},
    { &hf_rrc_bandIndicator_01,
      { "bandIndicator", "rrc.bandIndicator",
        FT_UINT32, BASE_DEC, VALS(rrc_Frequency_Band_vals), 0,
        "Frequency_Band", HFILL }},
    { &hf_rrc_absoluteTimeInfo,
      { "absoluteTimeInfo", "rrc.absoluteTimeInfo",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_48", HFILL }},
    { &hf_rrc_loggingDuration_01,
      { "loggingDuration", "rrc.loggingDuration",
        FT_UINT32, BASE_DEC, VALS(rrc_T_loggingDuration_01_vals), 0,
        "T_loggingDuration_01", HFILL }},
    { &hf_rrc_loggingInterval,
      { "loggingInterval", "rrc.loggingInterval",
        FT_UINT32, BASE_DEC, VALS(rrc_T_loggingInterval_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_traceReference,
      { "traceReference", "rrc.traceReference",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_traceRecordingSession,
      { "traceRecordingSession", "rrc.traceRecordingSession",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_tce_Id,
      { "tce-Id", "rrc.tce_Id",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_areaConfiguration,
      { "areaConfiguration", "rrc.areaConfiguration",
        FT_UINT32, BASE_DEC, VALS(rrc_T_areaConfiguration_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_cellIDList,
      { "cellIDList", "rrc.cellIDList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "SEQUENCE_SIZE_1_32_OF_CellIdentity", HFILL }},
    { &hf_rrc_cellIDList_item,
      { "CellIdentity", "rrc.CellIdentity",
        FT_BYTES, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_locationAreaList,
      { "locationAreaList", "rrc.locationAreaList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "SEQUENCE_SIZE_1_8_OF_LAI", HFILL }},
    { &hf_rrc_locationAreaList_item,
      { "LAI", "rrc.LAI",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_routingAreaList,
      { "routingAreaList", "rrc.routingAreaList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "SEQUENCE_SIZE_1_8_OF_RAI", HFILL }},
    { &hf_rrc_routingAreaList_item,
      { "RAI", "rrc.RAI",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_loggingDuration_02,
      { "loggingDuration", "rrc.loggingDuration",
        FT_UINT32, BASE_DEC, VALS(rrc_T_loggingDuration_02_vals), 0,
        "T_loggingDuration_02", HFILL }},
    { &hf_rrc_loggingInterval_01,
      { "loggingInterval", "rrc.loggingInterval",
        FT_UINT32, BASE_DEC, VALS(rrc_T_loggingInterval_01_vals), 0,
        "T_loggingInterval_01", HFILL }},
    { &hf_rrc_plmnList,
      { "plmnList", "rrc.plmnList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "SEQUENCE_SIZE_1_15_OF_PLMN_Identity", HFILL }},
    { &hf_rrc_plmnList_item,
      { "PLMN-Identity", "rrc.PLMN_Identity",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_areaConfiguration_01,
      { "areaConfiguration", "rrc.areaConfiguration",
        FT_UINT32, BASE_DEC, VALS(rrc_T_areaConfiguration_01_vals), 0,
        "T_areaConfiguration_01", HFILL }},
    { &hf_rrc_cellIDList_01,
      { "cellIDList", "rrc.cellIDList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "SEQUENCE_SIZE_1_32_OF_CellIDListItem", HFILL }},
    { &hf_rrc_cellIDList_item_01,
      { "CellIDListItem", "rrc.CellIDListItem",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_LoggedMeasInterfreqList_FDD_item,
      { "LoggedMeasInterfreqInfo-FDD", "rrc.LoggedMeasInterfreqInfo_FDD",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_loggedMeasInterfreqNeighbourList,
      { "loggedMeasInterfreqNeighbourList", "rrc.loggedMeasInterfreqNeighbourList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "SEQUENCE_SIZE_1_maxnumLoggedMeas_OF_LoggedMeasNeighbourInfo_FDD", HFILL }},
    { &hf_rrc_loggedMeasInterfreqNeighbourList_item,
      { "LoggedMeasNeighbourInfo-FDD", "rrc.LoggedMeasNeighbourInfo_FDD",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_loggedMeasInterfreqNeighbourList_01,
      { "loggedMeasInterfreqNeighbourList", "rrc.loggedMeasInterfreqNeighbourList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "LoggedMeasInterfreqNeighbourList_TDD128", HFILL }},
    { &hf_rrc_loggedMeasEUTRAFreqList,
      { "loggedMeasEUTRAFreqList", "rrc.loggedMeasEUTRAFreqList",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_loggedMeasGSMNeighbourCellsList,
      { "loggedMeasGSMNeighbourCellsList", "rrc.loggedMeasGSMNeighbourCellsList",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_LoggedMeasIntrafreqNeighbourList_FDD_item,
      { "LoggedMeasNeighbourInfo-FDD", "rrc.LoggedMeasNeighbourInfo_FDD",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_LoggedMeasIntrafreqNeighbourMeasList_TDD128_item,
      { "LoggedMeasNeighbourMeas-TDD128", "rrc.LoggedMeasNeighbourMeas_TDD128",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_LoggedMeasGSMNeighbourCellsList_item,
      { "LoggedMeasGSMNeighbourCellsinfo", "rrc.LoggedMeasGSMNeighbourCellsinfo",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_LoggedMeasEUTRAFreqList_item,
      { "LoggedMeasEUTRAFreqInfo", "rrc.LoggedMeasEUTRAFreqInfo",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_loggedMeasEUTRAFreqNeighbourInfoList,
      { "loggedMeasEUTRAFreqNeighbourInfoList", "rrc.loggedMeasEUTRAFreqNeighbourInfoList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "SEQUENCE_SIZE_1_maxnumLoggedMeas_OF_LoggedMeasEUTRAFreqNeighbourInfo", HFILL }},
    { &hf_rrc_loggedMeasEUTRAFreqNeighbourInfoList_item,
      { "LoggedMeasEUTRAFreqNeighbourInfo", "rrc.LoggedMeasEUTRAFreqNeighbourInfo",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_relativeTimeStamp,
      { "relativeTimeStamp", "rrc.relativeTimeStamp",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_7200", HFILL }},
    { &hf_rrc_loggedMeasServingCellMeas,
      { "loggedMeasServingCellMeas", "rrc.loggedMeasServingCellMeas",
        FT_NONE, BASE_NONE, NULL, 0,
        "LoggedMeasServingCellMeas_FDD", HFILL }},
    { &hf_rrc_loggedMeasLocationInfo,
      { "loggedMeasLocationInfo", "rrc.loggedMeasLocationInfo",
        FT_UINT32, BASE_DEC, VALS(rrc_LoggedMeasLocationInfo_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_loggedMeasServingCellMeas_01,
      { "loggedMeasServingCellMeas", "rrc.loggedMeasServingCellMeas",
        FT_NONE, BASE_NONE, NULL, 0,
        "LoggedMeasServingCellMeas_FDD_vbxyext", HFILL }},
    { &hf_rrc_loggedMeasLocationInfo_01,
      { "loggedMeasLocationInfo", "rrc.loggedMeasLocationInfo",
        FT_UINT32, BASE_DEC, VALS(rrc_LoggedMeasLocationInfo_vbxyext_vals), 0,
        "LoggedMeasLocationInfo_vbxyext", HFILL }},
    { &hf_rrc_loggedMeasServingCellMeas_02,
      { "loggedMeasServingCellMeas", "rrc.loggedMeasServingCellMeas",
        FT_NONE, BASE_NONE, NULL, 0,
        "LoggedMeasServingCellMeas_TDD128", HFILL }},
    { &hf_rrc_loggedMeasServingCellMeas_03,
      { "loggedMeasServingCellMeas", "rrc.loggedMeasServingCellMeas",
        FT_NONE, BASE_NONE, NULL, 0,
        "LoggedMeasServingCellMeas_TDD128_vbxyext", HFILL }},
    { &hf_rrc_LoggedMeasInterfreqList_TDD128_item,
      { "LoggedMeasInterfreqInfo-TDD128", "rrc.LoggedMeasInterfreqInfo_TDD128",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_LoggedMeasInterfreqNeighbourList_TDD128_item,
      { "LoggedMeasNeighbourMeas-TDD128", "rrc.LoggedMeasNeighbourMeas_TDD128",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_loggedMeasInfoList,
      { "loggedMeasInfoList", "rrc.loggedMeasInfoList",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_loggedMeasAvailable_06,
      { "loggedMeasAvailable", "rrc.loggedMeasAvailable",
        FT_UINT32, BASE_DEC, VALS(rrc_T_loggedMeasAvailable_06_vals), 0,
        "T_loggedMeasAvailable_06", HFILL }},
    { &hf_rrc_loggedMeasInfoList_01,
      { "loggedMeasInfoList", "rrc.loggedMeasInfoList",
        FT_NONE, BASE_NONE, NULL, 0,
        "LoggedMeasInfoList_vbxyext", HFILL }},
    { &hf_rrc_LoggedMeasInfoList_FDD_item,
      { "LoggedMeasInfo-FDD", "rrc.LoggedMeasInfo_FDD",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_LoggedMeasInfoList_FDD_vbxyext_item,
      { "LoggedMeasInfo-FDD-vbxyext", "rrc.LoggedMeasInfo_FDD_vbxyext",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_LoggedMeasInfoList_TDD128_item,
      { "LoggedMeasInfo-TDD128", "rrc.LoggedMeasInfo_TDD128",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_LoggedMeasInfoList_TDD128_vbxyext_item,
      { "LoggedMeasInfo-TDD128-vbxyext", "rrc.LoggedMeasInfo_TDD128_vbxyext",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_modeSpecificInfo_119,
      { "modeSpecificInfo", "rrc.modeSpecificInfo",
        FT_UINT32, BASE_DEC, VALS(rrc_T_modeSpecificInfo_119_vals), 0,
        "T_modeSpecificInfo_119", HFILL }},
    { &hf_rrc_fdd_164,
      { "fdd", "rrc.fdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_fdd_158", HFILL }},
    { &hf_rrc_loggedMeasInfoList_02,
      { "loggedMeasInfoList", "rrc.loggedMeasInfoList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "LoggedMeasInfoList_FDD", HFILL }},
    { &hf_rrc_tdd_114,
      { "tdd", "rrc.tdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_tdd_106", HFILL }},
    { &hf_rrc_loggedMeasInfoList_03,
      { "loggedMeasInfoList", "rrc.loggedMeasInfoList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "LoggedMeasInfoList_TDD128", HFILL }},
    { &hf_rrc_modeSpecificInfo_120,
      { "modeSpecificInfo", "rrc.modeSpecificInfo",
        FT_UINT32, BASE_DEC, VALS(rrc_T_modeSpecificInfo_120_vals), 0,
        "T_modeSpecificInfo_120", HFILL }},
    { &hf_rrc_fdd_165,
      { "fdd", "rrc.fdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_fdd_159", HFILL }},
    { &hf_rrc_loggedMeasInfoList_04,
      { "loggedMeasInfoList", "rrc.loggedMeasInfoList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "LoggedMeasInfoList_FDD_vbxyext", HFILL }},
    { &hf_rrc_tdd_115,
      { "tdd", "rrc.tdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_tdd_107", HFILL }},
    { &hf_rrc_loggedMeasInfoList_05,
      { "loggedMeasInfoList", "rrc.loggedMeasInfoList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "LoggedMeasInfoList_TDD128_vbxyext", HFILL }},
    { &hf_rrc_plmnIdentity,
      { "plmnIdentity", "rrc.plmnIdentity",
        FT_NONE, BASE_NONE, NULL, 0,
        "PLMN_Identity", HFILL }},
    { &hf_rrc_rscpforANR,
      { "rscpforANR", "rrc.rscpforANR",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_ec_N0forANR,
      { "ec-N0forANR", "rrc.ec_N0forANR",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_toe_nav,
      { "toe-nav", "rrc.toe_nav",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_14", HFILL }},
    { &hf_rrc_ganss_omega_nav,
      { "ganss-omega-nav", "rrc.ganss_omega_nav",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_32", HFILL }},
    { &hf_rrc_delta_n_nav,
      { "delta-n-nav", "rrc.delta_n_nav",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_16", HFILL }},
    { &hf_rrc_m_zero_nav,
      { "m-zero-nav", "rrc.m_zero_nav",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_32", HFILL }},
    { &hf_rrc_omegadot_nav,
      { "omegadot-nav", "rrc.omegadot_nav",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_24", HFILL }},
    { &hf_rrc_ganss_e_nav,
      { "ganss-e-nav", "rrc.ganss_e_nav",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_32", HFILL }},
    { &hf_rrc_idot_nav,
      { "idot-nav", "rrc.idot_nav",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_14", HFILL }},
    { &hf_rrc_a_sqrt_nav,
      { "a-sqrt-nav", "rrc.a_sqrt_nav",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_32", HFILL }},
    { &hf_rrc_i_zero_nav,
      { "i-zero-nav", "rrc.i_zero_nav",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_32", HFILL }},
    { &hf_rrc_omega_zero_nav,
      { "omega-zero-nav", "rrc.omega_zero_nav",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_32", HFILL }},
    { &hf_rrc_c_rs_nav,
      { "c-rs-nav", "rrc.c_rs_nav",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_16", HFILL }},
    { &hf_rrc_c_is_nav,
      { "c-is-nav", "rrc.c_is_nav",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_16", HFILL }},
    { &hf_rrc_c_us_nav,
      { "c-us-nav", "rrc.c_us_nav",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_16", HFILL }},
    { &hf_rrc_c_rc_nav,
      { "c-rc-nav", "rrc.c_rc_nav",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_16", HFILL }},
    { &hf_rrc_c_ic_nav,
      { "c-ic-nav", "rrc.c_ic_nav",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_16", HFILL }},
    { &hf_rrc_c_uc_nav,
      { "c-uc-nav", "rrc.c_uc_nav",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_16", HFILL }},
    { &hf_rrc_intraFreqMeasuredResultsList,
      { "intraFreqMeasuredResultsList", "rrc.intraFreqMeasuredResultsList",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_interFreqMeasuredResultsList,
      { "interFreqMeasuredResultsList", "rrc.interFreqMeasuredResultsList",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_interRATMeasuredResultsList,
      { "interRATMeasuredResultsList", "rrc.interRATMeasuredResultsList",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_trafficVolumeMeasuredResultsList,
      { "trafficVolumeMeasuredResultsList", "rrc.trafficVolumeMeasuredResultsList",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_qualityMeasuredResults,
      { "qualityMeasuredResults", "rrc.qualityMeasuredResults",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_ue_InternalMeasuredResults,
      { "ue-InternalMeasuredResults", "rrc.ue_InternalMeasuredResults",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_ue_positioning_MeasuredResults,
      { "ue-positioning-MeasuredResults", "rrc.ue_positioning_MeasuredResults",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_intraFreqMeasuredResultsList_01,
      { "intraFreqMeasuredResultsList", "rrc.intraFreqMeasuredResultsList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "IntraFrequencyMeasuredResultsListOnSecULFreq", HFILL }},
    { &hf_rrc_ue_positioning_MeasuredResults_v390ext,
      { "ue-positioning-MeasuredResults-v390ext", "rrc.ue_positioning_MeasuredResults_v390ext",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_intraFrequencyMeasuredResultsList,
      { "intraFrequencyMeasuredResultsList", "rrc.intraFrequencyMeasuredResultsList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "IntraFrequencyMeasuredResultsList_v590ext", HFILL }},
    { &hf_rrc_interFrequencyMeasuredResultsList,
      { "interFrequencyMeasuredResultsList", "rrc.interFrequencyMeasuredResultsList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "InterFrequencyMeasuredResultsList_v590ext", HFILL }},
    { &hf_rrc_ueInternalMeasuredResults,
      { "ueInternalMeasuredResults", "rrc.ueInternalMeasuredResults",
        FT_NONE, BASE_NONE, NULL, 0,
        "UE_InternalMeasuredResults_v770ext", HFILL }},
    { &hf_rrc_ue_positioning_MeasuredResults_01,
      { "ue-positioning-MeasuredResults", "rrc.ue_positioning_MeasuredResults",
        FT_NONE, BASE_NONE, NULL, 0,
        "UE_Positioning_MeasuredResults_v770ext", HFILL }},
    { &hf_rrc_ue_positioning_MeasuredResults_02,
      { "ue-positioning-MeasuredResults", "rrc.ue_positioning_MeasuredResults",
        FT_NONE, BASE_NONE, NULL, 0,
        "UE_Positioning_MeasuredResults_v860ext", HFILL }},
    { &hf_rrc_MeasuredResultsList_v920ext_item,
      { "MeasuredResults-v920ext", "rrc.MeasuredResults_v920ext",
        FT_UINT32, BASE_DEC, VALS(rrc_MeasuredResults_v920ext_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_MeasuredResultsListOnSecUlFreq_item,
      { "MeasuredResultsOnSecUlFreq", "rrc.MeasuredResultsOnSecUlFreq",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_intraFreqMeasuredResultsList_02,
      { "intraFreqMeasuredResultsList", "rrc.intraFreqMeasuredResultsList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "IntraFreqMeasuredResultsList_v920ext", HFILL }},
    { &hf_rrc_interFreqMeasuredResultsList_01,
      { "interFreqMeasuredResultsList", "rrc.interFreqMeasuredResultsList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "InterFreqMeasuredResultsList_v920ext", HFILL }},
    { &hf_rrc_ue_InternalMeasuredResults_01,
      { "ue-InternalMeasuredResults", "rrc.ue_InternalMeasuredResults",
        FT_NONE, BASE_NONE, NULL, 0,
        "UE_InternalMeasuredResults_LCR_r4", HFILL }},
    { &hf_rrc_ue_positioniing_MeasuredResults,
      { "ue-positioniing-MeasuredResults", "rrc.ue_positioniing_MeasuredResults",
        FT_NONE, BASE_NONE, NULL, 0,
        "UE_Positioning_MeasuredResults", HFILL }},
    { &hf_rrc_MeasuredResultsList_item,
      { "MeasuredResults", "rrc.MeasuredResults",
        FT_UINT32, BASE_DEC, VALS(rrc_MeasuredResults_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_MeasuredResultsList_LCR_r4_ext_item,
      { "MeasuredResults-LCR-r4", "rrc.MeasuredResults_LCR_r4",
        FT_UINT32, BASE_DEC, VALS(rrc_MeasuredResults_LCR_r4_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_MeasuredResultsList_v770xet_item,
      { "MeasuredResultsList-v770xet item", "rrc.MeasuredResultsList_v770xet_item",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_MeasuredResultsList_v860ext_item,
      { "MeasuredResultsList-v860ext item", "rrc.MeasuredResultsList_v860ext_item",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_currentCell,
      { "currentCell", "rrc.currentCell",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_modeSpecificInfo_121,
      { "modeSpecificInfo", "rrc.modeSpecificInfo",
        FT_UINT32, BASE_DEC, VALS(rrc_T_modeSpecificInfo_121_vals), 0,
        "T_modeSpecificInfo_121", HFILL }},
    { &hf_rrc_fdd_166,
      { "fdd", "rrc.fdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_fdd_160", HFILL }},
    { &hf_rrc_measurementQuantity_03,
      { "measurementQuantity", "rrc.measurementQuantity",
        FT_UINT32, BASE_DEC, VALS(rrc_T_measurementQuantity_01_vals), 0,
        "T_measurementQuantity_01", HFILL }},
    { &hf_rrc_tdd_116,
      { "tdd", "rrc.tdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_tdd_108", HFILL }},
    { &hf_rrc_timeslotISCP,
      { "timeslotISCP", "rrc.timeslotISCP",
        FT_UINT32, BASE_DEC, NULL, 0,
        "TimeslotISCP_List", HFILL }},
    { &hf_rrc_monitoredCells,
      { "monitoredCells", "rrc.monitoredCells",
        FT_UINT32, BASE_DEC, NULL, 0,
        "MonitoredCellRACH_List", HFILL }},
    { &hf_rrc_currentCell_DeltaRSCP,
      { "currentCell-DeltaRSCP", "rrc.currentCell_DeltaRSCP",
        FT_NONE, BASE_NONE, NULL, 0,
        "DeltaRSCPPerCell", HFILL }},
    { &hf_rrc_monitoredCellRACH_List_v7g0ext,
      { "monitoredCellRACH-List-v7g0ext", "rrc.monitoredCellRACH_List_v7g0ext",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_MonitoredCellRACH_List_v7g0ext_item,
      { "DeltaRSCPPerCell", "rrc.DeltaRSCPPerCell",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_interFreqCellIndication_SIB11,
      { "interFreqCellIndication-SIB11", "rrc.interFreqCellIndication_SIB11",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_1", HFILL }},
    { &hf_rrc_interFreqCellIndication_SIB12,
      { "interFreqCellIndication-SIB12", "rrc.interFreqCellIndication_SIB12",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_1", HFILL }},
    { &hf_rrc_interFreqRACHRepCellsList,
      { "interFreqRACHRepCellsList", "rrc.interFreqRACHRepCellsList",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_eutraFrequencyIndication_SIB19,
      { "eutraFrequencyIndication-SIB19", "rrc.eutraFrequencyIndication_SIB19",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_1", HFILL }},
    { &hf_rrc_setup,
      { "setup", "rrc.setup",
        FT_UINT32, BASE_DEC, VALS(rrc_MeasurementType_vals), 0,
        "MeasurementType", HFILL }},
    { &hf_rrc_modify_01,
      { "modify", "rrc.modify",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_modify_01", HFILL }},
    { &hf_rrc_measurementType,
      { "measurementType", "rrc.measurementType",
        FT_UINT32, BASE_DEC, VALS(rrc_MeasurementType_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_setup_01,
      { "setup", "rrc.setup",
        FT_UINT32, BASE_DEC, VALS(rrc_MeasurementType_r4_vals), 0,
        "MeasurementType_r4", HFILL }},
    { &hf_rrc_modify_02,
      { "modify", "rrc.modify",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_modify_02", HFILL }},
    { &hf_rrc_measurementType_01,
      { "measurementType", "rrc.measurementType",
        FT_UINT32, BASE_DEC, VALS(rrc_MeasurementType_r4_vals), 0,
        "MeasurementType_r4", HFILL }},
    { &hf_rrc_setup_02,
      { "setup", "rrc.setup",
        FT_UINT32, BASE_DEC, VALS(rrc_MeasurementType_r6_vals), 0,
        "MeasurementType_r6", HFILL }},
    { &hf_rrc_modify_03,
      { "modify", "rrc.modify",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_modify_03", HFILL }},
    { &hf_rrc_measurementType_02,
      { "measurementType", "rrc.measurementType",
        FT_UINT32, BASE_DEC, VALS(rrc_MeasurementType_r6_vals), 0,
        "MeasurementType_r6", HFILL }},
    { &hf_rrc_setup_03,
      { "setup", "rrc.setup",
        FT_UINT32, BASE_DEC, VALS(rrc_MeasurementType_r7_vals), 0,
        "MeasurementType_r7", HFILL }},
    { &hf_rrc_modify_04,
      { "modify", "rrc.modify",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_modify_04", HFILL }},
    { &hf_rrc_measurementType_03,
      { "measurementType", "rrc.measurementType",
        FT_UINT32, BASE_DEC, VALS(rrc_MeasurementType_r7_vals), 0,
        "MeasurementType_r7", HFILL }},
    { &hf_rrc_setup_04,
      { "setup", "rrc.setup",
        FT_UINT32, BASE_DEC, VALS(rrc_MeasurementType_r8_vals), 0,
        "MeasurementType_r8", HFILL }},
    { &hf_rrc_modify_05,
      { "modify", "rrc.modify",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_modify_05", HFILL }},
    { &hf_rrc_measurementType_04,
      { "measurementType", "rrc.measurementType",
        FT_UINT32, BASE_DEC, VALS(rrc_MeasurementType_r8_vals), 0,
        "MeasurementType_r8", HFILL }},
    { &hf_rrc_setup_05,
      { "setup", "rrc.setup",
        FT_UINT32, BASE_DEC, VALS(rrc_MeasurementType_r9_vals), 0,
        "MeasurementType_r9", HFILL }},
    { &hf_rrc_modify_06,
      { "modify", "rrc.modify",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_modify_06", HFILL }},
    { &hf_rrc_measurementType_05,
      { "measurementType", "rrc.measurementType",
        FT_UINT32, BASE_DEC, VALS(rrc_MeasurementType_r9_vals), 0,
        "MeasurementType_r9", HFILL }},
    { &hf_rrc_setup_06,
      { "setup", "rrc.setup",
        FT_UINT32, BASE_DEC, VALS(rrc_MeasurementType_r10_vals), 0,
        "MeasurementType_r10", HFILL }},
    { &hf_rrc_modify_07,
      { "modify", "rrc.modify",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_modify_07", HFILL }},
    { &hf_rrc_measurementType_06,
      { "measurementType", "rrc.measurementType",
        FT_UINT32, BASE_DEC, VALS(rrc_MeasurementType_r10_vals), 0,
        "MeasurementType_r10", HFILL }},
    { &hf_rrc_setup_07,
      { "setup", "rrc.setup",
        FT_UINT32, BASE_DEC, VALS(rrc_MeasurementType_r11_vals), 0,
        "MeasurementType_r11", HFILL }},
    { &hf_rrc_modify_08,
      { "modify", "rrc.modify",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_modify_08", HFILL }},
    { &hf_rrc_measurementType_07,
      { "measurementType", "rrc.measurementType",
        FT_UINT32, BASE_DEC, VALS(rrc_MeasurementType_r11_vals), 0,
        "MeasurementType_r11", HFILL }},
    { &hf_rrc_use_of_HCS,
      { "use-of-HCS", "rrc.use_of_HCS",
        FT_UINT32, BASE_DEC, VALS(rrc_T_use_of_HCS_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_hcs_not_used,
      { "hcs-not-used", "rrc.hcs_not_used",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_hcs_not_used", HFILL }},
    { &hf_rrc_cellSelectQualityMeasure_01,
      { "cellSelectQualityMeasure", "rrc.cellSelectQualityMeasure",
        FT_UINT32, BASE_DEC, VALS(rrc_T_cellSelectQualityMeasure_01_vals), 0,
        "T_cellSelectQualityMeasure_01", HFILL }},
    { &hf_rrc_cpich_RSCP_02,
      { "cpich-RSCP", "rrc.cpich_RSCP",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_intraFreqMeasurementSysInfo,
      { "intraFreqMeasurementSysInfo", "rrc.intraFreqMeasurementSysInfo",
        FT_NONE, BASE_NONE, NULL, 0,
        "IntraFreqMeasurementSysInfo_RSCP", HFILL }},
    { &hf_rrc_interFreqMeasurementSysInfo,
      { "interFreqMeasurementSysInfo", "rrc.interFreqMeasurementSysInfo",
        FT_NONE, BASE_NONE, NULL, 0,
        "InterFreqMeasurementSysInfo_RSCP", HFILL }},
    { &hf_rrc_cpich_Ec_N0_02,
      { "cpich-Ec-N0", "rrc.cpich_Ec_N0",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_cpich_Ec_N0_01", HFILL }},
    { &hf_rrc_intraFreqMeasurementSysInfo_01,
      { "intraFreqMeasurementSysInfo", "rrc.intraFreqMeasurementSysInfo",
        FT_NONE, BASE_NONE, NULL, 0,
        "IntraFreqMeasurementSysInfo_ECN0", HFILL }},
    { &hf_rrc_interFreqMeasurementSysInfo_01,
      { "interFreqMeasurementSysInfo", "rrc.interFreqMeasurementSysInfo",
        FT_NONE, BASE_NONE, NULL, 0,
        "InterFreqMeasurementSysInfo_ECN0", HFILL }},
    { &hf_rrc_interRATMeasurementSysInfo,
      { "interRATMeasurementSysInfo", "rrc.interRATMeasurementSysInfo",
        FT_NONE, BASE_NONE, NULL, 0,
        "InterRATMeasurementSysInfo_B", HFILL }},
    { &hf_rrc_hcs_used,
      { "hcs-used", "rrc.hcs_used",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_cellSelectQualityMeasure_02,
      { "cellSelectQualityMeasure", "rrc.cellSelectQualityMeasure",
        FT_UINT32, BASE_DEC, VALS(rrc_T_cellSelectQualityMeasure_02_vals), 0,
        "T_cellSelectQualityMeasure_02", HFILL }},
    { &hf_rrc_cpich_RSCP_03,
      { "cpich-RSCP", "rrc.cpich_RSCP",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_cpich_RSCP_01", HFILL }},
    { &hf_rrc_intraFreqMeasurementSysInfo_02,
      { "intraFreqMeasurementSysInfo", "rrc.intraFreqMeasurementSysInfo",
        FT_NONE, BASE_NONE, NULL, 0,
        "IntraFreqMeasurementSysInfo_HCS_RSCP", HFILL }},
    { &hf_rrc_interFreqMeasurementSysInfo_02,
      { "interFreqMeasurementSysInfo", "rrc.interFreqMeasurementSysInfo",
        FT_NONE, BASE_NONE, NULL, 0,
        "InterFreqMeasurementSysInfo_HCS_RSCP", HFILL }},
    { &hf_rrc_cpich_Ec_N0_03,
      { "cpich-Ec-N0", "rrc.cpich_Ec_N0",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_cpich_Ec_N0_02", HFILL }},
    { &hf_rrc_intraFreqMeasurementSysInfo_03,
      { "intraFreqMeasurementSysInfo", "rrc.intraFreqMeasurementSysInfo",
        FT_NONE, BASE_NONE, NULL, 0,
        "IntraFreqMeasurementSysInfo_HCS_ECN0", HFILL }},
    { &hf_rrc_interFreqMeasurementSysInfo_03,
      { "interFreqMeasurementSysInfo", "rrc.interFreqMeasurementSysInfo",
        FT_NONE, BASE_NONE, NULL, 0,
        "InterFreqMeasurementSysInfo_HCS_ECN0", HFILL }},
    { &hf_rrc_interRATMeasurementSysInfo_01,
      { "interRATMeasurementSysInfo", "rrc.interRATMeasurementSysInfo",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_trafficVolumeMeasSysInfo,
      { "trafficVolumeMeasSysInfo", "rrc.trafficVolumeMeasSysInfo",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_dummy_52,
      { "dummy", "rrc.dummy",
        FT_NONE, BASE_NONE, NULL, 0,
        "UE_InternalMeasurementSysInfo", HFILL }},
    { &hf_rrc_use_of_HCS_01,
      { "use-of-HCS", "rrc.use_of_HCS",
        FT_UINT32, BASE_DEC, VALS(rrc_T_use_of_HCS_01_vals), 0,
        "T_use_of_HCS_01", HFILL }},
    { &hf_rrc_hcs_not_used_01,
      { "hcs-not-used", "rrc.hcs_not_used",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_hcs_not_used_01", HFILL }},
    { &hf_rrc_cellSelectQualityMeasure_03,
      { "cellSelectQualityMeasure", "rrc.cellSelectQualityMeasure",
        FT_UINT32, BASE_DEC, VALS(rrc_T_cellSelectQualityMeasure_03_vals), 0,
        "T_cellSelectQualityMeasure_03", HFILL }},
    { &hf_rrc_cpich_RSCP_04,
      { "cpich-RSCP", "rrc.cpich_RSCP",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_cpich_RSCP_02", HFILL }},
    { &hf_rrc_cpich_Ec_N0_04,
      { "cpich-Ec-N0", "rrc.cpich_Ec_N0",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_cpich_Ec_N0_03", HFILL }},
    { &hf_rrc_hcs_used_01,
      { "hcs-used", "rrc.hcs_used",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_hcs_used_01", HFILL }},
    { &hf_rrc_cellSelectQualityMeasure_04,
      { "cellSelectQualityMeasure", "rrc.cellSelectQualityMeasure",
        FT_UINT32, BASE_DEC, VALS(rrc_T_cellSelectQualityMeasure_04_vals), 0,
        "T_cellSelectQualityMeasure_04", HFILL }},
    { &hf_rrc_cpich_RSCP_05,
      { "cpich-RSCP", "rrc.cpich_RSCP",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_cpich_RSCP_03", HFILL }},
    { &hf_rrc_cpich_Ec_N0_05,
      { "cpich-Ec-N0", "rrc.cpich_Ec_N0",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_cpich_Ec_N0_04", HFILL }},
    { &hf_rrc_newIntraFrequencyCellInfoListAddon_r5,
      { "newIntraFrequencyCellInfoListAddon-r5", "rrc.newIntraFrequencyCellInfoListAddon_r5",
        FT_UINT32, BASE_DEC, NULL, 0,
        "SEQUENCE_SIZE_1_maxCellMeas_OF_CellSelectReselectInfo_v590ext", HFILL }},
    { &hf_rrc_newIntraFrequencyCellInfoListAddon_r5_item,
      { "CellSelectReselectInfo-v590ext", "rrc.CellSelectReselectInfo_v590ext",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_newInterFrequencyCellInfoListAddon_r5,
      { "newInterFrequencyCellInfoListAddon-r5", "rrc.newInterFrequencyCellInfoListAddon_r5",
        FT_UINT32, BASE_DEC, NULL, 0,
        "SEQUENCE_SIZE_1_maxCellMeas_OF_CellSelectReselectInfo_v590ext", HFILL }},
    { &hf_rrc_newInterFrequencyCellInfoListAddon_r5_item,
      { "CellSelectReselectInfo-v590ext", "rrc.CellSelectReselectInfo_v590ext",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_newInterRATCellInfoListAddon_r5,
      { "newInterRATCellInfoListAddon-r5", "rrc.newInterRATCellInfoListAddon_r5",
        FT_UINT32, BASE_DEC, NULL, 0,
        "SEQUENCE_SIZE_1_maxCellMeas_OF_CellSelectReselectInfo_v590ext", HFILL }},
    { &hf_rrc_newInterRATCellInfoListAddon_r5_item,
      { "CellSelectReselectInfo-v590ext", "rrc.CellSelectReselectInfo_v590ext",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_use_of_HCS_02,
      { "use-of-HCS", "rrc.use_of_HCS",
        FT_UINT32, BASE_DEC, VALS(rrc_T_use_of_HCS_02_vals), 0,
        "T_use_of_HCS_02", HFILL }},
    { &hf_rrc_hcs_not_used_02,
      { "hcs-not-used", "rrc.hcs_not_used",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_hcs_not_used_02", HFILL }},
    { &hf_rrc_cellSelectQualityMeasure_05,
      { "cellSelectQualityMeasure", "rrc.cellSelectQualityMeasure",
        FT_UINT32, BASE_DEC, VALS(rrc_T_cellSelectQualityMeasure_05_vals), 0,
        "T_cellSelectQualityMeasure_05", HFILL }},
    { &hf_rrc_cpich_RSCP_06,
      { "cpich-RSCP", "rrc.cpich_RSCP",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_cpich_RSCP_04", HFILL }},
    { &hf_rrc_cpich_Ec_N0_06,
      { "cpich-Ec-N0", "rrc.cpich_Ec_N0",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_cpich_Ec_N0_05", HFILL }},
    { &hf_rrc_hcs_used_02,
      { "hcs-used", "rrc.hcs_used",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_hcs_used_02", HFILL }},
    { &hf_rrc_cellSelectQualityMeasure_06,
      { "cellSelectQualityMeasure", "rrc.cellSelectQualityMeasure",
        FT_UINT32, BASE_DEC, VALS(rrc_T_cellSelectQualityMeasure_06_vals), 0,
        "T_cellSelectQualityMeasure_06", HFILL }},
    { &hf_rrc_cpich_RSCP_07,
      { "cpich-RSCP", "rrc.cpich_RSCP",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_cpich_RSCP_05", HFILL }},
    { &hf_rrc_cpich_Ec_N0_07,
      { "cpich-Ec-N0", "rrc.cpich_Ec_N0",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_cpich_Ec_N0_06", HFILL }},
    { &hf_rrc_use_of_HCS_03,
      { "use-of-HCS", "rrc.use_of_HCS",
        FT_UINT32, BASE_DEC, VALS(rrc_T_use_of_HCS_03_vals), 0,
        "T_use_of_HCS_03", HFILL }},
    { &hf_rrc_hcs_not_used_03,
      { "hcs-not-used", "rrc.hcs_not_used",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_hcs_not_used_03", HFILL }},
    { &hf_rrc_cellSelectQualityMeasure_07,
      { "cellSelectQualityMeasure", "rrc.cellSelectQualityMeasure",
        FT_UINT32, BASE_DEC, VALS(rrc_T_cellSelectQualityMeasure_07_vals), 0,
        "T_cellSelectQualityMeasure_07", HFILL }},
    { &hf_rrc_cpich_RSCP_08,
      { "cpich-RSCP", "rrc.cpich_RSCP",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_cpich_RSCP_06", HFILL }},
    { &hf_rrc_intraFreqMeasurementSysInfo_04,
      { "intraFreqMeasurementSysInfo", "rrc.intraFreqMeasurementSysInfo",
        FT_NONE, BASE_NONE, NULL, 0,
        "IntraFreqMeasurementSysInfo_RSCP_LCR_r4", HFILL }},
    { &hf_rrc_interFreqMeasurementSysInfo_04,
      { "interFreqMeasurementSysInfo", "rrc.interFreqMeasurementSysInfo",
        FT_NONE, BASE_NONE, NULL, 0,
        "InterFreqMeasurementSysInfo_RSCP_LCR_r4", HFILL }},
    { &hf_rrc_cpich_Ec_N0_08,
      { "cpich-Ec-N0", "rrc.cpich_Ec_N0",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_cpich_Ec_N0_07", HFILL }},
    { &hf_rrc_intraFreqMeasurementSysInfo_05,
      { "intraFreqMeasurementSysInfo", "rrc.intraFreqMeasurementSysInfo",
        FT_NONE, BASE_NONE, NULL, 0,
        "IntraFreqMeasurementSysInfo_ECN0_LCR_r4", HFILL }},
    { &hf_rrc_interFreqMeasurementSysInfo_05,
      { "interFreqMeasurementSysInfo", "rrc.interFreqMeasurementSysInfo",
        FT_NONE, BASE_NONE, NULL, 0,
        "InterFreqMeasurementSysInfo_ECN0_LCR_r4", HFILL }},
    { &hf_rrc_hcs_used_03,
      { "hcs-used", "rrc.hcs_used",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_hcs_used_03", HFILL }},
    { &hf_rrc_cellSelectQualityMeasure_08,
      { "cellSelectQualityMeasure", "rrc.cellSelectQualityMeasure",
        FT_UINT32, BASE_DEC, VALS(rrc_T_cellSelectQualityMeasure_08_vals), 0,
        "T_cellSelectQualityMeasure_08", HFILL }},
    { &hf_rrc_cpich_RSCP_09,
      { "cpich-RSCP", "rrc.cpich_RSCP",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_cpich_RSCP_07", HFILL }},
    { &hf_rrc_intraFreqMeasurementSysInfo_06,
      { "intraFreqMeasurementSysInfo", "rrc.intraFreqMeasurementSysInfo",
        FT_NONE, BASE_NONE, NULL, 0,
        "IntraFreqMeasurementSysInfo_HCS_RSCP_LCR_r4", HFILL }},
    { &hf_rrc_interFreqMeasurementSysInfo_06,
      { "interFreqMeasurementSysInfo", "rrc.interFreqMeasurementSysInfo",
        FT_NONE, BASE_NONE, NULL, 0,
        "InterFreqMeasurementSysInfo_HCS_RSCP_LCR_r4", HFILL }},
    { &hf_rrc_cpich_Ec_N0_09,
      { "cpich-Ec-N0", "rrc.cpich_Ec_N0",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_cpich_Ec_N0_08", HFILL }},
    { &hf_rrc_intraFreqMeasurementSysInfo_07,
      { "intraFreqMeasurementSysInfo", "rrc.intraFreqMeasurementSysInfo",
        FT_NONE, BASE_NONE, NULL, 0,
        "IntraFreqMeasurementSysInfo_HCS_ECN0_LCR_r4", HFILL }},
    { &hf_rrc_interFreqMeasurementSysInfo_07,
      { "interFreqMeasurementSysInfo", "rrc.interFreqMeasurementSysInfo",
        FT_NONE, BASE_NONE, NULL, 0,
        "InterFreqMeasurementSysInfo_HCS_ECN0_LCR_r4", HFILL }},
    { &hf_rrc_measurement_Occasion_Coeff,
      { "measurement-Occasion-Coeff", "rrc.measurement_Occasion_Coeff",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_1_9", HFILL }},
    { &hf_rrc_measurement_Occasion_Offset,
      { "measurement-Occasion-Offset", "rrc.measurement_Occasion_Offset",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_511", HFILL }},
    { &hf_rrc_measurement_Occasion_Length,
      { "measurement-Occasion-Length", "rrc.measurement_Occasion_Length",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_1_512", HFILL }},
    { &hf_rrc_timeslotBitmap,
      { "timeslotBitmap", "rrc.timeslotBitmap",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_7", HFILL }},
    { &hf_rrc_measurementReportTransferMode,
      { "measurementReportTransferMode", "rrc.measurementReportTransferMode",
        FT_UINT32, BASE_DEC, VALS(rrc_TransferMode_vals), 0,
        "TransferMode", HFILL }},
    { &hf_rrc_periodicalOrEventTrigger,
      { "periodicalOrEventTrigger", "rrc.periodicalOrEventTrigger",
        FT_UINT32, BASE_DEC, VALS(rrc_PeriodicalOrEventTrigger_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_intraFrequencyMeasurement,
      { "intraFrequencyMeasurement", "rrc.intraFrequencyMeasurement",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_interFrequencyMeasurement,
      { "interFrequencyMeasurement", "rrc.interFrequencyMeasurement",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_interRATMeasurement,
      { "interRATMeasurement", "rrc.interRATMeasurement",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_ue_positioning_Measurement,
      { "ue-positioning-Measurement", "rrc.ue_positioning_Measurement",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_trafficVolumeMeasurement,
      { "trafficVolumeMeasurement", "rrc.trafficVolumeMeasurement",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_qualityMeasurement,
      { "qualityMeasurement", "rrc.qualityMeasurement",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_ue_InternalMeasurement,
      { "ue-InternalMeasurement", "rrc.ue_InternalMeasurement",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_intraFrequencyMeasurement_01,
      { "intraFrequencyMeasurement", "rrc.intraFrequencyMeasurement",
        FT_NONE, BASE_NONE, NULL, 0,
        "IntraFrequencyMeasurement_r4", HFILL }},
    { &hf_rrc_interFrequencyMeasurement_01,
      { "interFrequencyMeasurement", "rrc.interFrequencyMeasurement",
        FT_NONE, BASE_NONE, NULL, 0,
        "InterFrequencyMeasurement_r4", HFILL }},
    { &hf_rrc_interRATMeasurement_01,
      { "interRATMeasurement", "rrc.interRATMeasurement",
        FT_NONE, BASE_NONE, NULL, 0,
        "InterRATMeasurement_r4", HFILL }},
    { &hf_rrc_up_Measurement,
      { "up-Measurement", "rrc.up_Measurement",
        FT_NONE, BASE_NONE, NULL, 0,
        "UE_Positioning_Measurement_r4", HFILL }},
    { &hf_rrc_ue_InternalMeasurement_01,
      { "ue-InternalMeasurement", "rrc.ue_InternalMeasurement",
        FT_NONE, BASE_NONE, NULL, 0,
        "UE_InternalMeasurement_r4", HFILL }},
    { &hf_rrc_intraFrequencyMeasurement_02,
      { "intraFrequencyMeasurement", "rrc.intraFrequencyMeasurement",
        FT_NONE, BASE_NONE, NULL, 0,
        "IntraFrequencyMeasurement_r6", HFILL }},
    { &hf_rrc_interFrequencyMeasurement_02,
      { "interFrequencyMeasurement", "rrc.interFrequencyMeasurement",
        FT_NONE, BASE_NONE, NULL, 0,
        "InterFrequencyMeasurement_r6", HFILL }},
    { &hf_rrc_interRATMeasurement_02,
      { "interRATMeasurement", "rrc.interRATMeasurement",
        FT_NONE, BASE_NONE, NULL, 0,
        "InterRATMeasurement_r6", HFILL }},
    { &hf_rrc_intraFrequencyMeasurement_03,
      { "intraFrequencyMeasurement", "rrc.intraFrequencyMeasurement",
        FT_NONE, BASE_NONE, NULL, 0,
        "IntraFrequencyMeasurement_r7", HFILL }},
    { &hf_rrc_interFrequencyMeasurement_03,
      { "interFrequencyMeasurement", "rrc.interFrequencyMeasurement",
        FT_NONE, BASE_NONE, NULL, 0,
        "InterFrequencyMeasurement_r7", HFILL }},
    { &hf_rrc_up_Measurement_01,
      { "up-Measurement", "rrc.up_Measurement",
        FT_NONE, BASE_NONE, NULL, 0,
        "UE_Positioning_Measurement_r7", HFILL }},
    { &hf_rrc_interFrequencyMeasurement_04,
      { "interFrequencyMeasurement", "rrc.interFrequencyMeasurement",
        FT_NONE, BASE_NONE, NULL, 0,
        "InterFrequencyMeasurement_r8", HFILL }},
    { &hf_rrc_interRATMeasurement_03,
      { "interRATMeasurement", "rrc.interRATMeasurement",
        FT_NONE, BASE_NONE, NULL, 0,
        "InterRATMeasurement_r8", HFILL }},
    { &hf_rrc_up_Measurement_02,
      { "up-Measurement", "rrc.up_Measurement",
        FT_NONE, BASE_NONE, NULL, 0,
        "UE_Positioning_Measurement_r8", HFILL }},
    { &hf_rrc_intraFrequencyMeasurement_04,
      { "intraFrequencyMeasurement", "rrc.intraFrequencyMeasurement",
        FT_NONE, BASE_NONE, NULL, 0,
        "IntraFrequencyMeasurement_r9", HFILL }},
    { &hf_rrc_interFrequencyMeasurement_05,
      { "interFrequencyMeasurement", "rrc.interFrequencyMeasurement",
        FT_NONE, BASE_NONE, NULL, 0,
        "InterFrequencyMeasurement_r9", HFILL }},
    { &hf_rrc_interRATMeasurement_04,
      { "interRATMeasurement", "rrc.interRATMeasurement",
        FT_NONE, BASE_NONE, NULL, 0,
        "InterRATMeasurement_r9", HFILL }},
    { &hf_rrc_up_Measurement_03,
      { "up-Measurement", "rrc.up_Measurement",
        FT_NONE, BASE_NONE, NULL, 0,
        "UE_Positioning_Measurement_r9", HFILL }},
    { &hf_rrc_csgProximityDetection,
      { "csgProximityDetection", "rrc.csgProximityDetection",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_intraFrequencyMeasurement_05,
      { "intraFrequencyMeasurement", "rrc.intraFrequencyMeasurement",
        FT_NONE, BASE_NONE, NULL, 0,
        "IntraFrequencyMeasurement_r10", HFILL }},
    { &hf_rrc_interFrequencyMeasurement_06,
      { "interFrequencyMeasurement", "rrc.interFrequencyMeasurement",
        FT_NONE, BASE_NONE, NULL, 0,
        "InterFrequencyMeasurement_r10", HFILL }},
    { &hf_rrc_up_Measurement_04,
      { "up-Measurement", "rrc.up_Measurement",
        FT_NONE, BASE_NONE, NULL, 0,
        "UE_Positioning_Measurement_r10", HFILL }},
    { &hf_rrc_intraFrequencyMeasurement_06,
      { "intraFrequencyMeasurement", "rrc.intraFrequencyMeasurement",
        FT_NONE, BASE_NONE, NULL, 0,
        "IntraFrequencyMeasurement_r11", HFILL }},
    { &hf_rrc_interFrequencyMeasurement_07,
      { "interFrequencyMeasurement", "rrc.interFrequencyMeasurement",
        FT_NONE, BASE_NONE, NULL, 0,
        "InterFrequencyMeasurement_r11", HFILL }},
    { &hf_rrc_interRATMeasurement_05,
      { "interRATMeasurement", "rrc.interRATMeasurement",
        FT_NONE, BASE_NONE, NULL, 0,
        "InterRATMeasurement_r11", HFILL }},
    { &hf_rrc_eutraMeasurementForCELLFACH,
      { "eutraMeasurementForCELLFACH", "rrc.eutraMeasurementForCELLFACH",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_ue_State,
      { "ue-State", "rrc.ue_State",
        FT_UINT32, BASE_DEC, VALS(rrc_T_ue_State_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_MonitoredCellRACH_List_item,
      { "MonitoredCellRACH-Result", "rrc.MonitoredCellRACH_Result",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_sfn_SFN_ObsTimeDifference,
      { "sfn-SFN-ObsTimeDifference", "rrc.sfn_SFN_ObsTimeDifference",
        FT_UINT32, BASE_DEC, VALS(rrc_SFN_SFN_ObsTimeDifference_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_modeSpecificInfo_122,
      { "modeSpecificInfo", "rrc.modeSpecificInfo",
        FT_UINT32, BASE_DEC, VALS(rrc_T_modeSpecificInfo_122_vals), 0,
        "T_modeSpecificInfo_122", HFILL }},
    { &hf_rrc_fdd_167,
      { "fdd", "rrc.fdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_fdd_161", HFILL }},
    { &hf_rrc_measurementQuantity_04,
      { "measurementQuantity", "rrc.measurementQuantity",
        FT_UINT32, BASE_DEC, VALS(rrc_T_measurementQuantity_02_vals), 0,
        "T_measurementQuantity_02", HFILL }},
    { &hf_rrc_tdd_117,
      { "tdd", "rrc.tdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_tdd_109", HFILL }},
    { &hf_rrc_MultipleEUTRAFrequencyBandIndicatorList_item,
      { "RadioFrequencyBandEUTRA", "rrc.RadioFrequencyBandEUTRA",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_multipleEUTRAFrequencyBandIndicatorlist,
      { "multipleEUTRAFrequencyBandIndicatorlist", "rrc.multipleEUTRAFrequencyBandIndicatorlist",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_MultipleEUTRAFrequencyInfoList_item,
      { "MultipleEUTRAFrequencyBandInfo", "rrc.MultipleEUTRAFrequencyBandInfo",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_MultipleFrequencyBandIndicatorListFDD_item,
      { "FrequencyBandsIndicatorFDD", "rrc.FrequencyBandsIndicatorFDD",
        FT_UINT32, BASE_DEC, VALS(rrc_FrequencyBandsIndicatorFDD_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_multipleFrequencyBandIndicatorListFDD,
      { "multipleFrequencyBandIndicatorListFDD", "rrc.multipleFrequencyBandIndicatorListFDD",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_MultipleFrequencyInfoListFDD_item,
      { "MultipleFrequencyBandInfo", "rrc.MultipleFrequencyBandInfo",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_n_CR,
      { "n-CR", "rrc.n_CR",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_1_16", HFILL }},
    { &hf_rrc_t_CRMaxHyst,
      { "t-CRMaxHyst", "rrc.t_CRMaxHyst",
        FT_UINT32, BASE_DEC, VALS(rrc_T_CRMaxHyst_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_navToc,
      { "navToc", "rrc.navToc",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_16", HFILL }},
    { &hf_rrc_navaf2,
      { "navaf2", "rrc.navaf2",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_8", HFILL }},
    { &hf_rrc_navaf1,
      { "navaf1", "rrc.navaf1",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_16", HFILL }},
    { &hf_rrc_navaf0,
      { "navaf0", "rrc.navaf0",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_22", HFILL }},
    { &hf_rrc_navTgd,
      { "navTgd", "rrc.navTgd",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_8", HFILL }},
    { &hf_rrc_satelliteStatus,
      { "satelliteStatus", "rrc.satelliteStatus",
        FT_UINT32, BASE_DEC, VALS(rrc_SatelliteStatus_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_ephemerisParameter,
      { "ephemerisParameter", "rrc.ephemerisParameter",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_NavigationModelSatInfoList_item,
      { "NavigationModelSatInfo", "rrc.NavigationModelSatInfo",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_cnavURAindex,
      { "cnavURAindex", "rrc.cnavURAindex",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_5", HFILL }},
    { &hf_rrc_cnavDeltaA,
      { "cnavDeltaA", "rrc.cnavDeltaA",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_26", HFILL }},
    { &hf_rrc_cnavAdot,
      { "cnavAdot", "rrc.cnavAdot",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_25", HFILL }},
    { &hf_rrc_cnavDeltaNo,
      { "cnavDeltaNo", "rrc.cnavDeltaNo",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_17", HFILL }},
    { &hf_rrc_cnavDeltaNoDot,
      { "cnavDeltaNoDot", "rrc.cnavDeltaNoDot",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_23", HFILL }},
    { &hf_rrc_cnavMo,
      { "cnavMo", "rrc.cnavMo",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_33", HFILL }},
    { &hf_rrc_cnavE,
      { "cnavE", "rrc.cnavE",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_33", HFILL }},
    { &hf_rrc_cnavOmega,
      { "cnavOmega", "rrc.cnavOmega",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_33", HFILL }},
    { &hf_rrc_cnavOMEGA0,
      { "cnavOMEGA0", "rrc.cnavOMEGA0",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_33", HFILL }},
    { &hf_rrc_cnavDeltaOmegaDot,
      { "cnavDeltaOmegaDot", "rrc.cnavDeltaOmegaDot",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_17", HFILL }},
    { &hf_rrc_cnavIo,
      { "cnavIo", "rrc.cnavIo",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_33", HFILL }},
    { &hf_rrc_cnavIoDot,
      { "cnavIoDot", "rrc.cnavIoDot",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_15", HFILL }},
    { &hf_rrc_cnavCis,
      { "cnavCis", "rrc.cnavCis",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_16", HFILL }},
    { &hf_rrc_cnavCic,
      { "cnavCic", "rrc.cnavCic",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_16", HFILL }},
    { &hf_rrc_cnavCrs,
      { "cnavCrs", "rrc.cnavCrs",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_24", HFILL }},
    { &hf_rrc_cnavCrc,
      { "cnavCrc", "rrc.cnavCrc",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_24", HFILL }},
    { &hf_rrc_cnavCus,
      { "cnavCus", "rrc.cnavCus",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_21", HFILL }},
    { &hf_rrc_cnavCuc,
      { "cnavCuc", "rrc.cnavCuc",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_21", HFILL }},
    { &hf_rrc_gloEn,
      { "gloEn", "rrc.gloEn",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_5", HFILL }},
    { &hf_rrc_gloP1,
      { "gloP1", "rrc.gloP1",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_2", HFILL }},
    { &hf_rrc_gloP2,
      { "gloP2", "rrc.gloP2",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_1", HFILL }},
    { &hf_rrc_gloM,
      { "gloM", "rrc.gloM",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_2", HFILL }},
    { &hf_rrc_gloX,
      { "gloX", "rrc.gloX",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_27", HFILL }},
    { &hf_rrc_gloXdot,
      { "gloXdot", "rrc.gloXdot",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_24", HFILL }},
    { &hf_rrc_gloXdotdot,
      { "gloXdotdot", "rrc.gloXdotdot",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_5", HFILL }},
    { &hf_rrc_gloY,
      { "gloY", "rrc.gloY",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_27", HFILL }},
    { &hf_rrc_gloYdot,
      { "gloYdot", "rrc.gloYdot",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_24", HFILL }},
    { &hf_rrc_gloYdotdot,
      { "gloYdotdot", "rrc.gloYdotdot",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_5", HFILL }},
    { &hf_rrc_gloZ,
      { "gloZ", "rrc.gloZ",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_27", HFILL }},
    { &hf_rrc_gloZdot,
      { "gloZdot", "rrc.gloZdot",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_24", HFILL }},
    { &hf_rrc_gloZdotdot,
      { "gloZdotdot", "rrc.gloZdotdot",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_5", HFILL }},
    { &hf_rrc_navURA,
      { "navURA", "rrc.navURA",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_4", HFILL }},
    { &hf_rrc_navFitFlag,
      { "navFitFlag", "rrc.navFitFlag",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_1", HFILL }},
    { &hf_rrc_navToe,
      { "navToe", "rrc.navToe",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_16", HFILL }},
    { &hf_rrc_navOmega,
      { "navOmega", "rrc.navOmega",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_32", HFILL }},
    { &hf_rrc_navDeltaN,
      { "navDeltaN", "rrc.navDeltaN",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_16", HFILL }},
    { &hf_rrc_navM0,
      { "navM0", "rrc.navM0",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_32", HFILL }},
    { &hf_rrc_navOmegaADot,
      { "navOmegaADot", "rrc.navOmegaADot",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_24", HFILL }},
    { &hf_rrc_navE,
      { "navE", "rrc.navE",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_32", HFILL }},
    { &hf_rrc_navIDot,
      { "navIDot", "rrc.navIDot",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_14", HFILL }},
    { &hf_rrc_navAPowerHalf,
      { "navAPowerHalf", "rrc.navAPowerHalf",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_32", HFILL }},
    { &hf_rrc_navI0,
      { "navI0", "rrc.navI0",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_32", HFILL }},
    { &hf_rrc_navOmegaA0,
      { "navOmegaA0", "rrc.navOmegaA0",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_32", HFILL }},
    { &hf_rrc_navCrs,
      { "navCrs", "rrc.navCrs",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_16", HFILL }},
    { &hf_rrc_navCis,
      { "navCis", "rrc.navCis",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_16", HFILL }},
    { &hf_rrc_navCus,
      { "navCus", "rrc.navCus",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_16", HFILL }},
    { &hf_rrc_navCrc,
      { "navCrc", "rrc.navCrc",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_16", HFILL }},
    { &hf_rrc_navCic,
      { "navCic", "rrc.navCic",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_16", HFILL }},
    { &hf_rrc_navCuc,
      { "navCuc", "rrc.navCuc",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_16", HFILL }},
    { &hf_rrc_sbasTo,
      { "sbasTo", "rrc.sbasTo",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_13", HFILL }},
    { &hf_rrc_sbasAccuracy,
      { "sbasAccuracy", "rrc.sbasAccuracy",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_4", HFILL }},
    { &hf_rrc_sbasXg,
      { "sbasXg", "rrc.sbasXg",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_30", HFILL }},
    { &hf_rrc_sbasYg,
      { "sbasYg", "rrc.sbasYg",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_30", HFILL }},
    { &hf_rrc_sbasZg,
      { "sbasZg", "rrc.sbasZg",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_25", HFILL }},
    { &hf_rrc_sbasXgDot,
      { "sbasXgDot", "rrc.sbasXgDot",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_17", HFILL }},
    { &hf_rrc_sbasYgDot,
      { "sbasYgDot", "rrc.sbasYgDot",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_17", HFILL }},
    { &hf_rrc_sbasZgDot,
      { "sbasZgDot", "rrc.sbasZgDot",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_18", HFILL }},
    { &hf_rrc_sbasXgDotDot,
      { "sbasXgDotDot", "rrc.sbasXgDotDot",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_10", HFILL }},
    { &hf_rrc_sbagYgDotDot,
      { "sbagYgDotDot", "rrc.sbagYgDotDot",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_10", HFILL }},
    { &hf_rrc_sbasZgDotDot,
      { "sbasZgDotDot", "rrc.sbasZgDotDot",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_10", HFILL }},
    { &hf_rrc_codeOnL2,
      { "codeOnL2", "rrc.codeOnL2",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_2", HFILL }},
    { &hf_rrc_uraIndex,
      { "uraIndex", "rrc.uraIndex",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_4", HFILL }},
    { &hf_rrc_satHealth_01,
      { "satHealth", "rrc.satHealth",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_6", HFILL }},
    { &hf_rrc_iodc,
      { "iodc", "rrc.iodc",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_10", HFILL }},
    { &hf_rrc_l2Pflag,
      { "l2Pflag", "rrc.l2Pflag",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_1", HFILL }},
    { &hf_rrc_sf1Revd,
      { "sf1Revd", "rrc.sf1Revd",
        FT_NONE, BASE_NONE, NULL, 0,
        "SubFrame1Reserved", HFILL }},
    { &hf_rrc_t_GD,
      { "t-GD", "rrc.t_GD",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_8", HFILL }},
    { &hf_rrc_t_oc,
      { "t-oc", "rrc.t_oc",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_16", HFILL }},
    { &hf_rrc_af2,
      { "af2", "rrc.af2",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_8", HFILL }},
    { &hf_rrc_af1_01,
      { "af1", "rrc.af1",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_16", HFILL }},
    { &hf_rrc_af0_01,
      { "af0", "rrc.af0",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_22", HFILL }},
    { &hf_rrc_c_rs,
      { "c-rs", "rrc.c_rs",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_16", HFILL }},
    { &hf_rrc_delta_n,
      { "delta-n", "rrc.delta_n",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_16", HFILL }},
    { &hf_rrc_m0_01,
      { "m0", "rrc.m0",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_32", HFILL }},
    { &hf_rrc_c_uc,
      { "c-uc", "rrc.c_uc",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_16", HFILL }},
    { &hf_rrc_e_01,
      { "e", "rrc.e",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_32", HFILL }},
    { &hf_rrc_c_us,
      { "c-us", "rrc.c_us",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_16", HFILL }},
    { &hf_rrc_a_Sqrt_01,
      { "a-Sqrt", "rrc.a_Sqrt",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_32", HFILL }},
    { &hf_rrc_t_oe,
      { "t-oe", "rrc.t_oe",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_16", HFILL }},
    { &hf_rrc_fitInterval,
      { "fitInterval", "rrc.fitInterval",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_1", HFILL }},
    { &hf_rrc_aodo,
      { "aodo", "rrc.aodo",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_5", HFILL }},
    { &hf_rrc_c_ic,
      { "c-ic", "rrc.c_ic",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_16", HFILL }},
    { &hf_rrc_omega0_01,
      { "omega0", "rrc.omega0",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_32", HFILL }},
    { &hf_rrc_c_is,
      { "c-is", "rrc.c_is",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_16", HFILL }},
    { &hf_rrc_i0,
      { "i0", "rrc.i0",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_32", HFILL }},
    { &hf_rrc_c_rc,
      { "c-rc", "rrc.c_rc",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_16", HFILL }},
    { &hf_rrc_omega_01,
      { "omega", "rrc.omega",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_32", HFILL }},
    { &hf_rrc_omegaDot_01,
      { "omegaDot", "rrc.omegaDot",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_24", HFILL }},
    { &hf_rrc_iDot,
      { "iDot", "rrc.iDot",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_14", HFILL }},
    { &hf_rrc_modeSpecificInfo_123,
      { "modeSpecificInfo", "rrc.modeSpecificInfo",
        FT_UINT32, BASE_DEC, VALS(rrc_T_modeSpecificInfo_123_vals), 0,
        "T_modeSpecificInfo_123", HFILL }},
    { &hf_rrc_fdd_168,
      { "fdd", "rrc.fdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_fdd_162", HFILL }},
    { &hf_rrc_neighbourIdentity,
      { "neighbourIdentity", "rrc.neighbourIdentity",
        FT_NONE, BASE_NONE, NULL, 0,
        "PrimaryCPICH_Info", HFILL }},
    { &hf_rrc_uE_RX_TX_TimeDifferenceType2Info,
      { "uE-RX-TX-TimeDifferenceType2Info", "rrc.uE_RX_TX_TimeDifferenceType2Info",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_tdd_118,
      { "tdd", "rrc.tdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_tdd_110", HFILL }},
    { &hf_rrc_neighbourAndChannelIdentity,
      { "neighbourAndChannelIdentity", "rrc.neighbourAndChannelIdentity",
        FT_NONE, BASE_NONE, NULL, 0,
        "CellAndChannelIdentity", HFILL }},
    { &hf_rrc_neighbourQuality,
      { "neighbourQuality", "rrc.neighbourQuality",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_sfn_SFN_ObsTimeDifference2,
      { "sfn-SFN-ObsTimeDifference2", "rrc.sfn_SFN_ObsTimeDifference2",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_modeSpecificInfo_124,
      { "modeSpecificInfo", "rrc.modeSpecificInfo",
        FT_UINT32, BASE_DEC, VALS(rrc_T_modeSpecificInfo_124_vals), 0,
        "T_modeSpecificInfo_124", HFILL }},
    { &hf_rrc_fdd_169,
      { "fdd", "rrc.fdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_fdd_163", HFILL }},
    { &hf_rrc_NeighbourList_item,
      { "Neighbour", "rrc.Neighbour",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_NeighbourList_TDD_r7_item,
      { "Neighbour-TDD-r7", "rrc.Neighbour_TDD_r7",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_NeighbourList_v390ext_item,
      { "Neighbour-v390ext", "rrc.Neighbour_v390ext",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_ue_Positioning_OTDOA_Quality,
      { "ue-Positioning-OTDOA-Quality", "rrc.ue_Positioning_OTDOA_Quality",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_interFreqCellID,
      { "interFreqCellID", "rrc.interFreqCellID",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_cellInfo,
      { "cellInfo", "rrc.cellInfo",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_cellInfo_01,
      { "cellInfo", "rrc.cellInfo",
        FT_NONE, BASE_NONE, NULL, 0,
        "CellInfo_r4", HFILL }},
    { &hf_rrc_intraSecondaryFreqIndicator,
      { "intraSecondaryFreqIndicator", "rrc.intraSecondaryFreqIndicator",
        FT_BOOLEAN, BASE_NONE, NULL, 0,
        "BOOLEAN", HFILL }},
    { &hf_rrc_cellInfo_LCR_r8,
      { "cellInfo-LCR-r8", "rrc.cellInfo_LCR_r8",
        FT_NONE, BASE_NONE, NULL, 0,
        "CellInfo_LCR_r8_ext", HFILL }},
    { &hf_rrc_modeSpecificInfo_125,
      { "modeSpecificInfo", "rrc.modeSpecificInfo",
        FT_UINT32, BASE_DEC, VALS(rrc_T_modeSpecificInfo_125_vals), 0,
        "T_modeSpecificInfo_125", HFILL }},
    { &hf_rrc_noInfo,
      { "noInfo", "rrc.noInfo",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_tdd128_64,
      { "tdd128", "rrc.tdd128",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_tdd128_62", HFILL }},
    { &hf_rrc_cellInfo_02,
      { "cellInfo", "rrc.cellInfo",
        FT_NONE, BASE_NONE, NULL, 0,
        "CellInfo_r9", HFILL }},
    { &hf_rrc_modeSpecificInfo_126,
      { "modeSpecificInfo", "rrc.modeSpecificInfo",
        FT_UINT32, BASE_DEC, VALS(rrc_T_modeSpecificInfo_126_vals), 0,
        "T_modeSpecificInfo_126", HFILL }},
    { &hf_rrc_tdd128_65,
      { "tdd128", "rrc.tdd128",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_tdd128_63", HFILL }},
    { &hf_rrc_modeSpecificInfo_127,
      { "modeSpecificInfo", "rrc.modeSpecificInfo",
        FT_UINT32, BASE_DEC, VALS(rrc_T_modeSpecificInfo_127_vals), 0,
        "T_modeSpecificInfo_127", HFILL }},
    { &hf_rrc_tdd128_66,
      { "tdd128", "rrc.tdd128",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_tdd128_64", HFILL }},
    { &hf_rrc_sNPLMonitorSetIndicator_TDD128,
      { "sNPLMonitorSetIndicator-TDD128", "rrc.sNPLMonitorSetIndicator_TDD128",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_5", HFILL }},
    { &hf_rrc_NewInterFreqCellList_item,
      { "NewInterFreqCell", "rrc.NewInterFreqCell",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_NewInterFreqCellList_r4_item,
      { "NewInterFreqCell-r4", "rrc.NewInterFreqCell_r4",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_NewInterFreqCellList_v7b0ext_item,
      { "NewInterFreqCell-v7b0ext", "rrc.NewInterFreqCell_v7b0ext",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_NewInterFreqCellList_LCR_v8a0ext_item,
      { "NewInterFreqCell-LCR-v8a0ext", "rrc.NewInterFreqCell_LCR_v8a0ext",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_NewInterFreqCellList_r8_item,
      { "NewInterFreqCell-r8", "rrc.NewInterFreqCell_r8",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_NewInterFreqCellList_r9_item,
      { "NewInterFreqCell-r9", "rrc.NewInterFreqCell_r9",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_NewInterFreqCellList_r10_item,
      { "NewInterFreqCell-r10", "rrc.NewInterFreqCell_r10",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_cellInfo_03,
      { "cellInfo", "rrc.cellInfo",
        FT_NONE, BASE_NONE, NULL, 0,
        "CellInfoSI_RSCP", HFILL }},
    { &hf_rrc_cellInfo_04,
      { "cellInfo", "rrc.cellInfo",
        FT_NONE, BASE_NONE, NULL, 0,
        "CellInfoSI_ECN0", HFILL }},
    { &hf_rrc_cellInfo_05,
      { "cellInfo", "rrc.cellInfo",
        FT_NONE, BASE_NONE, NULL, 0,
        "CellInfoSI_HCS_RSCP", HFILL }},
    { &hf_rrc_cellInfo_06,
      { "cellInfo", "rrc.cellInfo",
        FT_NONE, BASE_NONE, NULL, 0,
        "CellInfoSI_HCS_ECN0", HFILL }},
    { &hf_rrc_cellInfo_07,
      { "cellInfo", "rrc.cellInfo",
        FT_NONE, BASE_NONE, NULL, 0,
        "CellInfoSI_RSCP_LCR_r4", HFILL }},
    { &hf_rrc_cellInfo_08,
      { "cellInfo", "rrc.cellInfo",
        FT_NONE, BASE_NONE, NULL, 0,
        "CellInfoSI_ECN0_LCR_r4", HFILL }},
    { &hf_rrc_cellInfo_09,
      { "cellInfo", "rrc.cellInfo",
        FT_NONE, BASE_NONE, NULL, 0,
        "CellInfoSI_HCS_RSCP_LCR_r4", HFILL }},
    { &hf_rrc_cellInfo_10,
      { "cellInfo", "rrc.cellInfo",
        FT_NONE, BASE_NONE, NULL, 0,
        "CellInfoSI_HCS_ECN0_LCR_r4", HFILL }},
    { &hf_rrc_NewInterFreqCellSI_List_ECN0_item,
      { "NewInterFreqCellSI-ECN0", "rrc.NewInterFreqCellSI_ECN0",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_NewInterFreqCellSI_List_HCS_RSCP_item,
      { "NewInterFreqCellSI-HCS-RSCP", "rrc.NewInterFreqCellSI_HCS_RSCP",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_NewInterFreqCellSI_List_HCS_ECN0_item,
      { "NewInterFreqCellSI-HCS-ECN0", "rrc.NewInterFreqCellSI_HCS_ECN0",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_NewInterFreqCellSI_List_RSCP_item,
      { "NewInterFreqCellSI-RSCP", "rrc.NewInterFreqCellSI_RSCP",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_NewInterFreqCellSI_List_ECN0_LCR_r4_item,
      { "NewInterFreqCellSI-ECN0-LCR-r4", "rrc.NewInterFreqCellSI_ECN0_LCR_r4",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_NewInterFreqCellSI_List_HCS_RSCP_LCR_r4_item,
      { "NewInterFreqCellSI-HCS-RSCP-LCR-r4", "rrc.NewInterFreqCellSI_HCS_RSCP_LCR_r4",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_NewInterFreqCellSI_List_HCS_ECN0_LCR_r4_item,
      { "NewInterFreqCellSI-HCS-ECN0-LCR-r4", "rrc.NewInterFreqCellSI_HCS_ECN0_LCR_r4",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_NewInterFreqCellSI_List_RSCP_LCR_r4_item,
      { "NewInterFreqCellSI-RSCP-LCR-r4", "rrc.NewInterFreqCellSI_RSCP_LCR_r4",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_interRATCellID,
      { "interRATCellID", "rrc.interRATCellID",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_technologySpecificInfo_01,
      { "technologySpecificInfo", "rrc.technologySpecificInfo",
        FT_UINT32, BASE_DEC, VALS(rrc_T_technologySpecificInfo_01_vals), 0,
        "T_technologySpecificInfo_01", HFILL }},
    { &hf_rrc_gsm_15,
      { "gsm", "rrc.gsm",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_gsm_13", HFILL }},
    { &hf_rrc_cellSelectionReselectionInfo_05,
      { "cellSelectionReselectionInfo", "rrc.cellSelectionReselectionInfo",
        FT_NONE, BASE_NONE, NULL, 0,
        "CellSelectReselectInfoSIB_11_12", HFILL }},
    { &hf_rrc_interRATCellIndividualOffset,
      { "interRATCellIndividualOffset", "rrc.interRATCellIndividualOffset",
        FT_INT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_is_2000_03,
      { "is-2000", "rrc.is_2000",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_is_2000_02", HFILL }},
    { &hf_rrc_is_2000SpecificMeasInfo,
      { "is-2000SpecificMeasInfo", "rrc.is_2000SpecificMeasInfo",
        FT_UINT32, BASE_DEC, VALS(rrc_IS_2000SpecificMeasInfo_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_technologySpecificInfo_02,
      { "technologySpecificInfo", "rrc.technologySpecificInfo",
        FT_UINT32, BASE_DEC, VALS(rrc_T_technologySpecificInfo_02_vals), 0,
        "T_technologySpecificInfo_02", HFILL }},
    { &hf_rrc_gsm_16,
      { "gsm", "rrc.gsm",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_gsm_14", HFILL }},
    { &hf_rrc_is_2000_04,
      { "is-2000", "rrc.is_2000",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_is_2000_03", HFILL }},
    { &hf_rrc_NewInterRATCellList_item,
      { "NewInterRATCell", "rrc.NewInterRATCell",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_NewInterRATCellList_B_item,
      { "NewInterRATCell-B", "rrc.NewInterRATCell_B",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_intraFreqCellID,
      { "intraFreqCellID", "rrc.intraFreqCellID",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_modeSpecificInfo_128,
      { "modeSpecificInfo", "rrc.modeSpecificInfo",
        FT_UINT32, BASE_DEC, VALS(rrc_T_modeSpecificInfo_128_vals), 0,
        "T_modeSpecificInfo_128", HFILL }},
    { &hf_rrc_tdd128_67,
      { "tdd128", "rrc.tdd128",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_tdd128_65", HFILL }},
    { &hf_rrc_intraFreqCellIDOnSecULFreq,
      { "intraFreqCellIDOnSecULFreq", "rrc.intraFreqCellIDOnSecULFreq",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_NewIntraFreqCellList_item,
      { "NewIntraFreqCell", "rrc.NewIntraFreqCell",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_NewIntraFreqCellList_r4_item,
      { "NewIntraFreqCell-r4", "rrc.NewIntraFreqCell_r4",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_NewIntraFreqCellList_r9_item,
      { "NewIntraFreqCell-r9", "rrc.NewIntraFreqCell_r9",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_NewIntraFreqCellList_r10_item,
      { "NewIntraFreqCell-r10", "rrc.NewIntraFreqCell_r10",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_NewIntraFreqCellList_LCR_v8a0ext_item,
      { "NewIntraFreqCell-LCR-v8a0ext", "rrc.NewIntraFreqCell_LCR_v8a0ext",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_NewIntraFreqCellListOnSecULFreq_item,
      { "NewIntraFreqCellOnSecULFreq", "rrc.NewIntraFreqCellOnSecULFreq",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_NewIntraFreqCellSI_List_RSCP_item,
      { "NewIntraFreqCellSI-RSCP", "rrc.NewIntraFreqCellSI_RSCP",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_NewIntraFreqCellSI_List_ECN0_item,
      { "NewIntraFreqCellSI-ECN0", "rrc.NewIntraFreqCellSI_ECN0",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_NewIntraFreqCellSI_List_HCS_RSCP_item,
      { "NewIntraFreqCellSI-HCS-RSCP", "rrc.NewIntraFreqCellSI_HCS_RSCP",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_NewIntraFreqCellSI_List_HCS_ECN0_item,
      { "NewIntraFreqCellSI-HCS-ECN0", "rrc.NewIntraFreqCellSI_HCS_ECN0",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_NewIntraFreqCellSI_List_RSCP_LCR_r4_item,
      { "NewIntraFreqCellSI-RSCP-LCR-r4", "rrc.NewIntraFreqCellSI_RSCP_LCR_r4",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_NewIntraFreqCellSI_List_ECN0_LCR_r4_item,
      { "NewIntraFreqCellSI-ECN0-LCR-r4", "rrc.NewIntraFreqCellSI_ECN0_LCR_r4",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_NewIntraFreqCellSI_List_HCS_RSCP_LCR_r4_item,
      { "NewIntraFreqCellSI-HCS-RSCP-LCR-r4", "rrc.NewIntraFreqCellSI_HCS_RSCP_LCR_r4",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_NewIntraFreqCellSI_List_HCS_ECN0_LCR_r4_item,
      { "NewIntraFreqCellSI-HCS-ECN0-LCR-r4", "rrc.NewIntraFreqCellSI_HCS_ECN0_LCR_r4",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_nonUsedFreqThreshold,
      { "nonUsedFreqThreshold", "rrc.nonUsedFreqThreshold",
        FT_INT32, BASE_DEC, NULL, 0,
        "Threshold", HFILL }},
    { &hf_rrc_nonUsedFreqW,
      { "nonUsedFreqW", "rrc.nonUsedFreqW",
        FT_UINT32, BASE_DEC, NULL, 0,
        "W", HFILL }},
    { &hf_rrc_nonUsedFreqThreshold_01,
      { "nonUsedFreqThreshold", "rrc.nonUsedFreqThreshold",
        FT_INT32, BASE_DEC, NULL, 0,
        "Threshold_r6", HFILL }},
    { &hf_rrc_nonUsedFreqTriggeringConditionDetectedCells,
      { "nonUsedFreqTriggeringConditionDetectedCells", "rrc.nonUsedFreqTriggeringConditionDetectedCells",
        FT_UINT32, BASE_DEC, VALS(rrc_T_nonUsedFreqTriggeringConditionDetectedCells_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_nonUsedFreqTriggeringConditionDetectedCells_01,
      { "nonUsedFreqTriggeringConditionDetectedCells", "rrc.nonUsedFreqTriggeringConditionDetectedCells",
        FT_UINT32, BASE_DEC, VALS(rrc_T_nonUsedFreqTriggeringConditionDetectedCells_01_vals), 0,
        "T_nonUsedFreqTriggeringConditionDetectedCells_01", HFILL }},
    { &hf_rrc_nonUsedFreqTriggeringConditionDetectedCells_02,
      { "nonUsedFreqTriggeringConditionDetectedCells", "rrc.nonUsedFreqTriggeringConditionDetectedCells",
        FT_UINT32, BASE_DEC, VALS(rrc_T_nonUsedFreqTriggeringConditionDetectedCells_02_vals), 0,
        "T_nonUsedFreqTriggeringConditionDetectedCells_02", HFILL }},
    { &hf_rrc_nonUsedFreqTriggeringConditionDetectedCells_03,
      { "nonUsedFreqTriggeringConditionDetectedCells", "rrc.nonUsedFreqTriggeringConditionDetectedCells",
        FT_UINT32, BASE_DEC, VALS(rrc_T_nonUsedFreqTriggeringConditionDetectedCells_03_vals), 0,
        "T_nonUsedFreqTriggeringConditionDetectedCells_03", HFILL }},
    { &hf_rrc_NonUsedFreqParameterList_item,
      { "NonUsedFreqParameter", "rrc.NonUsedFreqParameter",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_NonUsedFreqParameterList_r6_item,
      { "NonUsedFreqParameter-r6", "rrc.NonUsedFreqParameter_r6",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_NonUsedFreqParameterList_r10_item,
      { "NonUsedFreqParameter-r10", "rrc.NonUsedFreqParameter_r10",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_NonUsedFreqParameterList_r11_item,
      { "NonUsedFreqParameter-r11", "rrc.NonUsedFreqParameter_r11",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_NonUsedFreq2aParameterList_r10_item,
      { "NonUsedFreq2aParameter-r10", "rrc.NonUsedFreq2aParameter_r10",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_NonUsedFreq2aParameterList_r11_item,
      { "NonUsedFreq2aParameter-r11", "rrc.NonUsedFreq2aParameter_r11",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_NonUsedFreqWList_r6_item,
      { "W", "rrc.W",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_notUsed,
      { "notUsed", "rrc.notUsed",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_pt10,
      { "pt10", "rrc.pt10",
        FT_UINT32, BASE_DEC, VALS(rrc_TemporaryOffset1_vals), 0,
        "TemporaryOffset1", HFILL }},
    { &hf_rrc_pt20,
      { "pt20", "rrc.pt20",
        FT_UINT32, BASE_DEC, VALS(rrc_TemporaryOffset1_vals), 0,
        "TemporaryOffset1", HFILL }},
    { &hf_rrc_pt30,
      { "pt30", "rrc.pt30",
        FT_UINT32, BASE_DEC, VALS(rrc_TemporaryOffset1_vals), 0,
        "TemporaryOffset1", HFILL }},
    { &hf_rrc_pt40,
      { "pt40", "rrc.pt40",
        FT_UINT32, BASE_DEC, VALS(rrc_TemporaryOffset1_vals), 0,
        "TemporaryOffset1", HFILL }},
    { &hf_rrc_pt50,
      { "pt50", "rrc.pt50",
        FT_UINT32, BASE_DEC, VALS(rrc_TemporaryOffset1_vals), 0,
        "TemporaryOffset1", HFILL }},
    { &hf_rrc_pt60,
      { "pt60", "rrc.pt60",
        FT_UINT32, BASE_DEC, VALS(rrc_TemporaryOffset1_vals), 0,
        "TemporaryOffset1", HFILL }},
    { &hf_rrc_pt10_01,
      { "pt10", "rrc.pt10",
        FT_NONE, BASE_NONE, NULL, 0,
        "TemporaryOffsetList", HFILL }},
    { &hf_rrc_pt20_01,
      { "pt20", "rrc.pt20",
        FT_NONE, BASE_NONE, NULL, 0,
        "TemporaryOffsetList", HFILL }},
    { &hf_rrc_pt30_01,
      { "pt30", "rrc.pt30",
        FT_NONE, BASE_NONE, NULL, 0,
        "TemporaryOffsetList", HFILL }},
    { &hf_rrc_pt40_01,
      { "pt40", "rrc.pt40",
        FT_NONE, BASE_NONE, NULL, 0,
        "TemporaryOffsetList", HFILL }},
    { &hf_rrc_pt50_01,
      { "pt50", "rrc.pt50",
        FT_NONE, BASE_NONE, NULL, 0,
        "TemporaryOffsetList", HFILL }},
    { &hf_rrc_pt60_01,
      { "pt60", "rrc.pt60",
        FT_NONE, BASE_NONE, NULL, 0,
        "TemporaryOffsetList", HFILL }},
    { &hf_rrc_reportingInterval_01,
      { "reportingInterval", "rrc.reportingInterval",
        FT_UINT32, BASE_DEC, VALS(rrc_ReportingIntervalLong_vals), 0,
        "ReportingIntervalLong", HFILL }},
    { &hf_rrc_periodicalWithReportingCellStatus,
      { "periodicalWithReportingCellStatus", "rrc.periodicalWithReportingCellStatus",
        FT_NONE, BASE_NONE, NULL, 0,
        "PeriodicalWithReportingCellStatus_r10", HFILL }},
    { &hf_rrc_plmnsOfIntraFreqCellsList,
      { "plmnsOfIntraFreqCellsList", "rrc.plmnsOfIntraFreqCellsList",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_plmnsOfInterFreqCellsList,
      { "plmnsOfInterFreqCellsList", "rrc.plmnsOfInterFreqCellsList",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_plmnsOfInterRATCellsList,
      { "plmnsOfInterRATCellsList", "rrc.plmnsOfInterRATCellsList",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_multipleplmnsOfIntraFreqCellsList,
      { "multipleplmnsOfIntraFreqCellsList", "rrc.multipleplmnsOfIntraFreqCellsList",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_multipleplmnsOfInterFreqCellsList,
      { "multipleplmnsOfInterFreqCellsList", "rrc.multipleplmnsOfInterFreqCellsList",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_PLMNsOfInterFreqCellsList_item,
      { "PLMNsOfInterFreqCellsList item", "rrc.PLMNsOfInterFreqCellsList_item",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_MultiplePLMNsOfInterFreqCellsList_item,
      { "MultiplePLMNsOfInterFreqCellsList item", "rrc.MultiplePLMNsOfInterFreqCellsList_item",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_multiplePLMN_list,
      { "multiplePLMN-list", "rrc.multiplePLMN_list",
        FT_UINT32, BASE_DEC, NULL, 0,
        "SEQUENCE_SIZE_1_6_OF_PLMN_IdentityWithOptionalMCC_r6", HFILL }},
    { &hf_rrc_multiplePLMN_list_item,
      { "PLMN-IdentityWithOptionalMCC-r6", "rrc.PLMN_IdentityWithOptionalMCC_r6",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_PLMNsOfIntraFreqCellsList_item,
      { "PLMNsOfIntraFreqCellsList item", "rrc.PLMNsOfIntraFreqCellsList_item",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_MultiplePLMNsOfIntraFreqCellsList_item,
      { "MultiplePLMNsOfIntraFreqCellsList item", "rrc.MultiplePLMNsOfIntraFreqCellsList_item",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_PLMNsOfInterRATCellsList_item,
      { "PLMNsOfInterRATCellsList item", "rrc.PLMNsOfInterRATCellsList_item",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_QualityEventResults_item,
      { "TransportChannelIdentity", "rrc.TransportChannelIdentity",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_blerMeasurementResultsList,
      { "blerMeasurementResultsList", "rrc.blerMeasurementResultsList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "BLER_MeasurementResultsList", HFILL }},
    { &hf_rrc_modeSpecificInfo_129,
      { "modeSpecificInfo", "rrc.modeSpecificInfo",
        FT_UINT32, BASE_DEC, VALS(rrc_T_modeSpecificInfo_129_vals), 0,
        "T_modeSpecificInfo_129", HFILL }},
    { &hf_rrc_tdd_119,
      { "tdd", "rrc.tdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_tdd_111", HFILL }},
    { &hf_rrc_sir_MeasurementResults,
      { "sir-MeasurementResults", "rrc.sir_MeasurementResults",
        FT_UINT32, BASE_DEC, NULL, 0,
        "SIR_MeasurementList", HFILL }},
    { &hf_rrc_qualityReportingQuantity,
      { "qualityReportingQuantity", "rrc.qualityReportingQuantity",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_reportCriteria_15,
      { "reportCriteria", "rrc.reportCriteria",
        FT_UINT32, BASE_DEC, VALS(rrc_QualityReportCriteria_vals), 0,
        "QualityReportCriteria", HFILL }},
    { &hf_rrc_qualityReportingCriteria,
      { "qualityReportingCriteria", "rrc.qualityReportingCriteria",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_noReporting_02,
      { "noReporting", "rrc.noReporting",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_QualityReportingCriteria_item,
      { "QualityReportingCriteriaSingle", "rrc.QualityReportingCriteriaSingle",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_totalCRC,
      { "totalCRC", "rrc.totalCRC",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_1_512", HFILL }},
    { &hf_rrc_badCRC,
      { "badCRC", "rrc.badCRC",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_1_512", HFILL }},
    { &hf_rrc_pendingAfterTrigger,
      { "pendingAfterTrigger", "rrc.pendingAfterTrigger",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_1_512", HFILL }},
    { &hf_rrc_dl_TransChBLER,
      { "dl-TransChBLER", "rrc.dl_TransChBLER",
        FT_BOOLEAN, BASE_NONE, NULL, 0,
        "BOOLEAN", HFILL }},
    { &hf_rrc_bler_dl_TransChIdList,
      { "bler-dl-TransChIdList", "rrc.bler_dl_TransChIdList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "BLER_TransChIdList", HFILL }},
    { &hf_rrc_modeSpecificInfo_130,
      { "modeSpecificInfo", "rrc.modeSpecificInfo",
        FT_UINT32, BASE_DEC, VALS(rrc_T_modeSpecificInfo_130_vals), 0,
        "T_modeSpecificInfo_130", HFILL }},
    { &hf_rrc_tdd_120,
      { "tdd", "rrc.tdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_tdd_112", HFILL }},
    { &hf_rrc_sir_TFCS_List,
      { "sir-TFCS-List", "rrc.sir_TFCS_List",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_ellipsoidPointWithAltitude,
      { "ellipsoidPointWithAltitude", "rrc.ellipsoidPointWithAltitude",
        FT_NONE, BASE_NONE, NULL, 0,
        "EllipsoidPointAltitude", HFILL }},
    { &hf_rrc_ellipsoidPointAltitudeEllipsoide,
      { "ellipsoidPointAltitudeEllipsoide", "rrc.ellipsoidPointAltitudeEllipsoide",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_accuracy40,
      { "accuracy40", "rrc.accuracy40",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_960", HFILL }},
    { &hf_rrc_accuracy256,
      { "accuracy256", "rrc.accuracy256",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_150", HFILL }},
    { &hf_rrc_accuracy2560,
      { "accuracy2560", "rrc.accuracy2560",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_15", HFILL }},
    { &hf_rrc_removeAllInterFreqCells,
      { "removeAllInterFreqCells", "rrc.removeAllInterFreqCells",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_removeSomeInterFreqCells,
      { "removeSomeInterFreqCells", "rrc.removeSomeInterFreqCells",
        FT_UINT32, BASE_DEC, NULL, 0,
        "SEQUENCE_SIZE_1_maxCellMeas_OF_InterFreqCellID", HFILL }},
    { &hf_rrc_removeSomeInterFreqCells_item,
      { "InterFreqCellID", "rrc.InterFreqCellID",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_removeNoInterFreqCells,
      { "removeNoInterFreqCells", "rrc.removeNoInterFreqCells",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_removeAllInterRATCells,
      { "removeAllInterRATCells", "rrc.removeAllInterRATCells",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_removeSomeInterRATCells,
      { "removeSomeInterRATCells", "rrc.removeSomeInterRATCells",
        FT_UINT32, BASE_DEC, NULL, 0,
        "SEQUENCE_SIZE_1_maxCellMeas_OF_InterRATCellID", HFILL }},
    { &hf_rrc_removeSomeInterRATCells_item,
      { "InterRATCellID", "rrc.InterRATCellID",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_removeNoInterRATCells,
      { "removeNoInterRATCells", "rrc.removeNoInterRATCells",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_removeAllIntraFreqCells,
      { "removeAllIntraFreqCells", "rrc.removeAllIntraFreqCells",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_removeSomeIntraFreqCells,
      { "removeSomeIntraFreqCells", "rrc.removeSomeIntraFreqCells",
        FT_UINT32, BASE_DEC, NULL, 0,
        "SEQUENCE_SIZE_1_maxCellMeas_OF_IntraFreqCellID", HFILL }},
    { &hf_rrc_removeSomeIntraFreqCells_item,
      { "IntraFreqCellID", "rrc.IntraFreqCellID",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_removeNoIntraFreqCells,
      { "removeNoIntraFreqCells", "rrc.removeNoIntraFreqCells",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_removeSomeIntraFreqCells_01,
      { "removeSomeIntraFreqCells", "rrc.removeSomeIntraFreqCells",
        FT_UINT32, BASE_DEC, NULL, 0,
        "SEQUENCE_SIZE_1_maxCellMeasOnSecULFreq_OF_IntraFreqCellIDOnSecULFreq", HFILL }},
    { &hf_rrc_removeSomeIntraFreqCells_item_01,
      { "IntraFreqCellIDOnSecULFreq", "rrc.IntraFreqCellIDOnSecULFreq",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_withinActiveSet,
      { "withinActiveSet", "rrc.withinActiveSet",
        FT_UINT32, BASE_DEC, VALS(rrc_MaxNumberOfReportingCellsType1_vals), 0,
        "MaxNumberOfReportingCellsType1", HFILL }},
    { &hf_rrc_withinMonitoredSetUsedFreq,
      { "withinMonitoredSetUsedFreq", "rrc.withinMonitoredSetUsedFreq",
        FT_UINT32, BASE_DEC, VALS(rrc_MaxNumberOfReportingCellsType1_vals), 0,
        "MaxNumberOfReportingCellsType1", HFILL }},
    { &hf_rrc_withinActiveAndOrMonitoredUsedFreq,
      { "withinActiveAndOrMonitoredUsedFreq", "rrc.withinActiveAndOrMonitoredUsedFreq",
        FT_UINT32, BASE_DEC, VALS(rrc_MaxNumberOfReportingCellsType1_vals), 0,
        "MaxNumberOfReportingCellsType1", HFILL }},
    { &hf_rrc_withinDetectedSetUsedFreq,
      { "withinDetectedSetUsedFreq", "rrc.withinDetectedSetUsedFreq",
        FT_UINT32, BASE_DEC, VALS(rrc_MaxNumberOfReportingCellsType1_vals), 0,
        "MaxNumberOfReportingCellsType1", HFILL }},
    { &hf_rrc_withinMonitoredAndOrDetectedUsedFreq,
      { "withinMonitoredAndOrDetectedUsedFreq", "rrc.withinMonitoredAndOrDetectedUsedFreq",
        FT_UINT32, BASE_DEC, VALS(rrc_MaxNumberOfReportingCellsType1_vals), 0,
        "MaxNumberOfReportingCellsType1", HFILL }},
    { &hf_rrc_allActiveplusMonitoredSet,
      { "allActiveplusMonitoredSet", "rrc.allActiveplusMonitoredSet",
        FT_UINT32, BASE_DEC, VALS(rrc_MaxNumberOfReportingCellsType3_vals), 0,
        "MaxNumberOfReportingCellsType3", HFILL }},
    { &hf_rrc_allActivePlusDetectedSet,
      { "allActivePlusDetectedSet", "rrc.allActivePlusDetectedSet",
        FT_UINT32, BASE_DEC, VALS(rrc_MaxNumberOfReportingCellsType3_vals), 0,
        "MaxNumberOfReportingCellsType3", HFILL }},
    { &hf_rrc_allActivePlusMonitoredAndOrDetectedSet,
      { "allActivePlusMonitoredAndOrDetectedSet", "rrc.allActivePlusMonitoredAndOrDetectedSet",
        FT_UINT32, BASE_DEC, VALS(rrc_MaxNumberOfReportingCellsType3_vals), 0,
        "MaxNumberOfReportingCellsType3", HFILL }},
    { &hf_rrc_withinVirtualActSet,
      { "withinVirtualActSet", "rrc.withinVirtualActSet",
        FT_UINT32, BASE_DEC, VALS(rrc_MaxNumberOfReportingCellsType1_vals), 0,
        "MaxNumberOfReportingCellsType1", HFILL }},
    { &hf_rrc_withinMonitoredSetNonUsedFreq,
      { "withinMonitoredSetNonUsedFreq", "rrc.withinMonitoredSetNonUsedFreq",
        FT_UINT32, BASE_DEC, VALS(rrc_MaxNumberOfReportingCellsType1_vals), 0,
        "MaxNumberOfReportingCellsType1", HFILL }},
    { &hf_rrc_withinMonitoredAndOrVirtualActiveSetNonUsedFreq,
      { "withinMonitoredAndOrVirtualActiveSetNonUsedFreq", "rrc.withinMonitoredAndOrVirtualActiveSetNonUsedFreq",
        FT_UINT32, BASE_DEC, VALS(rrc_MaxNumberOfReportingCellsType1_vals), 0,
        "MaxNumberOfReportingCellsType1", HFILL }},
    { &hf_rrc_allVirtualActSetplusMonitoredSetNonUsedFreq,
      { "allVirtualActSetplusMonitoredSetNonUsedFreq", "rrc.allVirtualActSetplusMonitoredSetNonUsedFreq",
        FT_UINT32, BASE_DEC, VALS(rrc_MaxNumberOfReportingCellsType3_vals), 0,
        "MaxNumberOfReportingCellsType3", HFILL }},
    { &hf_rrc_withinActSetOrVirtualActSet_InterRATcells,
      { "withinActSetOrVirtualActSet-InterRATcells", "rrc.withinActSetOrVirtualActSet_InterRATcells",
        FT_UINT32, BASE_DEC, VALS(rrc_MaxNumberOfReportingCellsType2_vals), 0,
        "MaxNumberOfReportingCellsType2", HFILL }},
    { &hf_rrc_withinActSetAndOrMonitoredUsedFreqOrVirtualActSetAndOrMonitoredNonUsedFreq,
      { "withinActSetAndOrMonitoredUsedFreqOrVirtualActSetAndOrMonitoredNonUsedFreq", "rrc.withinActSetAndOrMonitoredUsedFreqOrVirtualActSetAndOrMonitoredNonUsedFreq",
        FT_UINT32, BASE_DEC, VALS(rrc_MaxNumberOfReportingCellsType2_vals), 0,
        "MaxNumberOfReportingCellsType2", HFILL }},
    { &hf_rrc_allVirtualActSetplusMonitoredSetplusDetectedSetNonUsedFreq,
      { "allVirtualActSetplusMonitoredSetplusDetectedSetNonUsedFreq", "rrc.allVirtualActSetplusMonitoredSetplusDetectedSetNonUsedFreq",
        FT_UINT32, BASE_DEC, VALS(rrc_MaxNumberOfReportingCellsType3_vals), 0,
        "MaxNumberOfReportingCellsType3", HFILL }},
    { &hf_rrc_reportCriteria_16,
      { "reportCriteria", "rrc.reportCriteria",
        FT_UINT32, BASE_DEC, VALS(rrc_CellDCH_ReportCriteria_vals), 0,
        "CellDCH_ReportCriteria", HFILL }},
    { &hf_rrc_reportCriteria_17,
      { "reportCriteria", "rrc.reportCriteria",
        FT_UINT32, BASE_DEC, VALS(rrc_CellDCH_ReportCriteria_LCR_r4_vals), 0,
        "CellDCH_ReportCriteria_LCR_r4", HFILL }},
    { &hf_rrc_ganssSignalID,
      { "ganssSignalID", "rrc.ganssSignalID",
        FT_BYTES, BASE_NONE, NULL, 0,
        "DGANSS_Sig_Id_Req", HFILL }},
    { &hf_rrc_ganssDataBitInterval,
      { "ganssDataBitInterval", "rrc.ganssDataBitInterval",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_15", HFILL }},
    { &hf_rrc_ganssSatelliteInfo,
      { "ganssSatelliteInfo", "rrc.ganssSatelliteInfo",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_ganssSatelliteInfo_item,
      { "ganssSatelliteInfo item", "rrc.ganssSatelliteInfo_item",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_63", HFILL }},
    { &hf_rrc_RL_AdditionInfoList_item,
      { "PrimaryCPICH-Info", "rrc.PrimaryCPICH_Info",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_rl_AdditionInfoList,
      { "rl-AdditionInfoList", "rrc.rl_AdditionInfoList",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_rL_RemovalInformationList,
      { "rL-RemovalInformationList", "rrc.rL_RemovalInformationList",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_rSCP,
      { "rSCP", "rrc.rSCP",
        FT_INT32, BASE_DEC, NULL, 0,
        "INTEGER_M120_M25", HFILL }},
    { &hf_rrc_SatDataList_item,
      { "SatData", "rrc.SatData",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_iod_02,
      { "iod", "rrc.iod",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_1023", HFILL }},
    { &hf_rrc_SatellitesListRelatedDataList_item,
      { "SatellitesListRelatedData", "rrc.SatellitesListRelatedData",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_t_oc_01,
      { "t-oc", "rrc.t_oc",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_14", HFILL }},
    { &hf_rrc_af2_01,
      { "af2", "rrc.af2",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_12", HFILL }},
    { &hf_rrc_af1_02,
      { "af1", "rrc.af1",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_18", HFILL }},
    { &hf_rrc_af0_02,
      { "af0", "rrc.af0",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_28", HFILL }},
    { &hf_rrc_tgd_01,
      { "tgd", "rrc.tgd",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_10", HFILL }},
    { &hf_rrc_model_id,
      { "model-id", "rrc.model_id",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_1", HFILL }},
    { &hf_rrc_Satellite_clock_modelList_item,
      { "Satellite-clock-model", "rrc.Satellite_clock_model",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_sbasAgfo,
      { "sbasAgfo", "rrc.sbasAgfo",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_12", HFILL }},
    { &hf_rrc_sbasAgf1,
      { "sbasAgf1", "rrc.sbasAgf1",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_8", HFILL }},
    { &hf_rrc_s_DPCCH_DPCCH_PowerOffset,
      { "s-DPCCH-DPCCH-PowerOffset", "rrc.s_DPCCH_DPCCH_PowerOffset",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_type1_07,
      { "type1", "rrc.type1",
        FT_UINT32, BASE_DEC, NULL, 0,
        "SFN_SFN_ObsTimeDifference1", HFILL }},
    { &hf_rrc_type2_06,
      { "type2", "rrc.type2",
        FT_UINT32, BASE_DEC, NULL, 0,
        "SFN_SFN_ObsTimeDifference2", HFILL }},
    { &hf_rrc_sfn_Offset,
      { "sfn-Offset", "rrc.sfn_Offset",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_4095", HFILL }},
    { &hf_rrc_sfn_sfn_Reltimedifference,
      { "sfn-sfn-Reltimedifference", "rrc.sfn_sfn_Reltimedifference",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_38399", HFILL }},
    { &hf_rrc_SIR_MeasurementList_item,
      { "SIR-MeasurementResults", "rrc.SIR_MeasurementResults",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_sir_TimeslotList,
      { "sir-TimeslotList", "rrc.sir_TimeslotList",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_SIR_TFCS_List_item,
      { "SIR-TFCS", "rrc.SIR_TFCS",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_SIR_TimeslotList_item,
      { "SIR", "rrc.SIR",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_reserved1,
      { "reserved1", "rrc.reserved1",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_23", HFILL }},
    { &hf_rrc_reserved2,
      { "reserved2", "rrc.reserved2",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_24", HFILL }},
    { &hf_rrc_reserved3,
      { "reserved3", "rrc.reserved3",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_24", HFILL }},
    { &hf_rrc_reserved4,
      { "reserved4", "rrc.reserved4",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_16", HFILL }},
    { &hf_rrc_t_ADV,
      { "t-ADV", "rrc.t_ADV",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_2047", HFILL }},
    { &hf_rrc_sfn,
      { "sfn", "rrc.sfn",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_4095", HFILL }},
    { &hf_rrc_t_ADV_01,
      { "t-ADV", "rrc.t_ADV",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_8191", HFILL }},
    { &hf_rrc_t30,
      { "t30", "rrc.t30",
        FT_NONE, BASE_NONE, NULL, 0,
        "N_CR_T_CRMaxHyst", HFILL }},
    { &hf_rrc_t60,
      { "t60", "rrc.t60",
        FT_NONE, BASE_NONE, NULL, 0,
        "N_CR_T_CRMaxHyst", HFILL }},
    { &hf_rrc_t120,
      { "t120", "rrc.t120",
        FT_NONE, BASE_NONE, NULL, 0,
        "N_CR_T_CRMaxHyst", HFILL }},
    { &hf_rrc_t180,
      { "t180", "rrc.t180",
        FT_NONE, BASE_NONE, NULL, 0,
        "N_CR_T_CRMaxHyst", HFILL }},
    { &hf_rrc_t240,
      { "t240", "rrc.t240",
        FT_NONE, BASE_NONE, NULL, 0,
        "N_CR_T_CRMaxHyst", HFILL }},
    { &hf_rrc_tce_Id_01,
      { "tce-Id", "rrc.tce_Id",
        FT_BYTES, BASE_NONE, NULL, 0,
        "OCTET_STRING_SIZE_1", HFILL }},
    { &hf_rrc_temporaryOffset1,
      { "temporaryOffset1", "rrc.temporaryOffset1",
        FT_UINT32, BASE_DEC, VALS(rrc_TemporaryOffset1_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_temporaryOffset2,
      { "temporaryOffset2", "rrc.temporaryOffset2",
        FT_UINT32, BASE_DEC, VALS(rrc_TemporaryOffset2_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_ThreholdNonUsedFrequency_deltaList_item,
      { "DeltaRSCPPerCell", "rrc.DeltaRSCPPerCell",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_TimeslotInfoList_item,
      { "TimeslotInfo", "rrc.TimeslotInfo",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_TimeslotInfoList_LCR_r4_item,
      { "TimeslotInfo-LCR-r4", "rrc.TimeslotInfo_LCR_r4",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_tdd384_47,
      { "tdd384", "rrc.tdd384",
        FT_UINT32, BASE_DEC, NULL, 0,
        "SEQUENCE_SIZE_1_maxTS_OF_TimeslotInfo", HFILL }},
    { &hf_rrc_tdd384_item_01,
      { "TimeslotInfo", "rrc.TimeslotInfo",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_tdd128_68,
      { "tdd128", "rrc.tdd128",
        FT_UINT32, BASE_DEC, NULL, 0,
        "SEQUENCE_SIZE_1_maxTS_LCR_OF_TimeslotInfo_LCR_r4", HFILL }},
    { &hf_rrc_tdd128_item_02,
      { "TimeslotInfo-LCR-r4", "rrc.TimeslotInfo_LCR_r4",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_TimeslotISCP_List_item,
      { "TimeslotISCP", "rrc.TimeslotISCP",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_TimeslotListWithISCP_item,
      { "TimeslotWithISCP", "rrc.TimeslotWithISCP",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_timeslotISCP_01,
      { "timeslotISCP", "rrc.timeslotISCP",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_traceId,
      { "traceId", "rrc.traceId",
        FT_BYTES, BASE_NONE, NULL, 0,
        "OCTET_STRING_SIZE_3", HFILL }},
    { &hf_rrc_traceRecordingSession_01,
      { "traceRecordingSession", "rrc.traceRecordingSession",
        FT_BYTES, BASE_NONE, NULL, 0,
        "OCTET_STRING_SIZE_2", HFILL }},
    { &hf_rrc_eventID_03,
      { "eventID", "rrc.eventID",
        FT_UINT32, BASE_DEC, VALS(rrc_TrafficVolumeEventType_vals), 0,
        "TrafficVolumeEventType", HFILL }},
    { &hf_rrc_reportingThreshold_01,
      { "reportingThreshold", "rrc.reportingThreshold",
        FT_UINT32, BASE_DEC|BASE_EXT_STRING, &rrc_TrafficVolumeThreshold_vals_ext, 0,
        "TrafficVolumeThreshold", HFILL }},
    { &hf_rrc_pendingTimeAfterTrigger,
      { "pendingTimeAfterTrigger", "rrc.pendingTimeAfterTrigger",
        FT_UINT32, BASE_DEC, VALS(rrc_PendingTimeAfterTrigger_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_tx_InterruptionAfterTrigger,
      { "tx-InterruptionAfterTrigger", "rrc.tx_InterruptionAfterTrigger",
        FT_UINT32, BASE_DEC, VALS(rrc_TX_InterruptionAfterTrigger_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_ul_transportChannelCausingEvent,
      { "ul-transportChannelCausingEvent", "rrc.ul_transportChannelCausingEvent",
        FT_UINT32, BASE_DEC, VALS(rrc_UL_TrCH_Identity_vals), 0,
        "UL_TrCH_Identity", HFILL }},
    { &hf_rrc_trafficVolumeEventIdentity,
      { "trafficVolumeEventIdentity", "rrc.trafficVolumeEventIdentity",
        FT_UINT32, BASE_DEC, VALS(rrc_TrafficVolumeEventType_vals), 0,
        "TrafficVolumeEventType", HFILL }},
    { &hf_rrc_rlc_BufferPayload,
      { "rlc-BufferPayload", "rrc.rlc_BufferPayload",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_averageRLC_BufferPayload,
      { "averageRLC-BufferPayload", "rrc.averageRLC_BufferPayload",
        FT_UINT32, BASE_DEC, NULL, 0,
        "TimeInterval", HFILL }},
    { &hf_rrc_varianceOfRLC_BufferPayload,
      { "varianceOfRLC-BufferPayload", "rrc.varianceOfRLC_BufferPayload",
        FT_UINT32, BASE_DEC, NULL, 0,
        "TimeInterval", HFILL }},
    { &hf_rrc_trafficVolumeMeasurementID,
      { "trafficVolumeMeasurementID", "rrc.trafficVolumeMeasurementID",
        FT_UINT32, BASE_DEC, NULL, 0,
        "MeasurementIdentity", HFILL }},
    { &hf_rrc_trafficVolumeMeasurementObjectList,
      { "trafficVolumeMeasurementObjectList", "rrc.trafficVolumeMeasurementObjectList",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_trafficVolumeMeasQuantity,
      { "trafficVolumeMeasQuantity", "rrc.trafficVolumeMeasQuantity",
        FT_UINT32, BASE_DEC, VALS(rrc_TrafficVolumeMeasQuantity_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_trafficVolumeReportingQuantity,
      { "trafficVolumeReportingQuantity", "rrc.trafficVolumeReportingQuantity",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_dummy_53,
      { "dummy", "rrc.dummy",
        FT_NONE, BASE_NONE, NULL, 0,
        "TrafficVolumeReportingCriteria", HFILL }},
    { &hf_rrc_reportCriteriaSysInf,
      { "reportCriteriaSysInf", "rrc.reportCriteriaSysInf",
        FT_UINT32, BASE_DEC, VALS(rrc_TrafficVolumeReportCriteriaSysInfo_vals), 0,
        "TrafficVolumeReportCriteriaSysInfo", HFILL }},
    { &hf_rrc_rlc_BuffersPayload,
      { "rlc-BuffersPayload", "rrc.rlc_BuffersPayload",
        FT_UINT32, BASE_DEC|BASE_EXT_STRING, &rrc_RLC_BuffersPayload_vals_ext, 0,
        NULL, HFILL }},
    { &hf_rrc_averageRLC_BufferPayload_01,
      { "averageRLC-BufferPayload", "rrc.averageRLC_BufferPayload",
        FT_UINT32, BASE_DEC|BASE_EXT_STRING, &rrc_AverageRLC_BufferPayload_vals_ext, 0,
        NULL, HFILL }},
    { &hf_rrc_varianceOfRLC_BufferPayload_01,
      { "varianceOfRLC-BufferPayload", "rrc.varianceOfRLC_BufferPayload",
        FT_UINT32, BASE_DEC, VALS(rrc_VarianceOfRLC_BufferPayload_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_TrafficVolumeMeasuredResultsList_item,
      { "TrafficVolumeMeasuredResults", "rrc.TrafficVolumeMeasuredResults",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_reportCriteria_18,
      { "reportCriteria", "rrc.reportCriteria",
        FT_UINT32, BASE_DEC, VALS(rrc_TrafficVolumeReportCriteria_vals), 0,
        "TrafficVolumeReportCriteria", HFILL }},
    { &hf_rrc_TrafficVolumeMeasurementObjectList_item,
      { "UL-TrCH-Identity", "rrc.UL_TrCH_Identity",
        FT_UINT32, BASE_DEC, VALS(rrc_UL_TrCH_Identity_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_trafficVolumeReportingCriteria,
      { "trafficVolumeReportingCriteria", "rrc.trafficVolumeReportingCriteria",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_transChCriteriaList,
      { "transChCriteriaList", "rrc.transChCriteriaList",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_rlc_RB_BufferPayload,
      { "rlc-RB-BufferPayload", "rrc.rlc_RB_BufferPayload",
        FT_BOOLEAN, BASE_NONE, NULL, 0,
        "BOOLEAN", HFILL }},
    { &hf_rrc_rlc_RB_BufferPayloadAverage,
      { "rlc-RB-BufferPayloadAverage", "rrc.rlc_RB_BufferPayloadAverage",
        FT_BOOLEAN, BASE_NONE, NULL, 0,
        "BOOLEAN", HFILL }},
    { &hf_rrc_rlc_RB_BufferPayloadVariance,
      { "rlc-RB-BufferPayloadVariance", "rrc.rlc_RB_BufferPayloadVariance",
        FT_BOOLEAN, BASE_NONE, NULL, 0,
        "BOOLEAN", HFILL }},
    { &hf_rrc_ul_transportChannelID,
      { "ul-transportChannelID", "rrc.ul_transportChannelID",
        FT_UINT32, BASE_DEC, VALS(rrc_UL_TrCH_Identity_vals), 0,
        "UL_TrCH_Identity", HFILL }},
    { &hf_rrc_eventSpecificParameters,
      { "eventSpecificParameters", "rrc.eventSpecificParameters",
        FT_UINT32, BASE_DEC, NULL, 0,
        "SEQUENCE_SIZE_1_maxMeasParEvent_OF_TrafficVolumeEventParam", HFILL }},
    { &hf_rrc_eventSpecificParameters_item,
      { "TrafficVolumeEventParam", "rrc.TrafficVolumeEventParam",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_TransChCriteriaList_item,
      { "TransChCriteria", "rrc.TransChCriteria",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_transmittedPowerThreshold,
      { "transmittedPowerThreshold", "rrc.transmittedPowerThreshold",
        FT_INT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_ue_RX_TX_TimeDifferenceThreshold,
      { "ue-RX-TX-TimeDifferenceThreshold", "rrc.ue_RX_TX_TimeDifferenceThreshold",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_onWithNoReporting,
      { "onWithNoReporting", "rrc.onWithNoReporting",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_dummy2_13,
      { "dummy2", "rrc.dummy2",
        FT_NONE, BASE_NONE, NULL, 0,
        "RL_InformationLists", HFILL }},
    { &hf_rrc_event6a,
      { "event6a", "rrc.event6a",
        FT_NONE, BASE_NONE, NULL, 0,
        "UE_6AB_Event", HFILL }},
    { &hf_rrc_event6b,
      { "event6b", "rrc.event6b",
        FT_NONE, BASE_NONE, NULL, 0,
        "UE_6AB_Event", HFILL }},
    { &hf_rrc_event6c,
      { "event6c", "rrc.event6c",
        FT_UINT32, BASE_DEC, VALS(rrc_TimeToTrigger_vals), 0,
        "TimeToTrigger", HFILL }},
    { &hf_rrc_event6d,
      { "event6d", "rrc.event6d",
        FT_UINT32, BASE_DEC, VALS(rrc_TimeToTrigger_vals), 0,
        "TimeToTrigger", HFILL }},
    { &hf_rrc_event6e,
      { "event6e", "rrc.event6e",
        FT_UINT32, BASE_DEC, VALS(rrc_TimeToTrigger_vals), 0,
        "TimeToTrigger", HFILL }},
    { &hf_rrc_event6f,
      { "event6f", "rrc.event6f",
        FT_NONE, BASE_NONE, NULL, 0,
        "UE_6FG_Event", HFILL }},
    { &hf_rrc_event6g,
      { "event6g", "rrc.event6g",
        FT_NONE, BASE_NONE, NULL, 0,
        "UE_6FG_Event", HFILL }},
    { &hf_rrc_UE_InternalEventParamList_item,
      { "UE-InternalEventParam", "rrc.UE_InternalEventParam",
        FT_UINT32, BASE_DEC, VALS(rrc_UE_InternalEventParam_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_event6a_01,
      { "event6a", "rrc.event6a",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_event6b_01,
      { "event6b", "rrc.event6b",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_event6c_01,
      { "event6c", "rrc.event6c",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_event6d_01,
      { "event6d", "rrc.event6d",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_event6e_01,
      { "event6e", "rrc.event6e",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_event6f_01,
      { "event6f", "rrc.event6f",
        FT_NONE, BASE_NONE, NULL, 0,
        "PrimaryCPICH_Info", HFILL }},
    { &hf_rrc_event6g_01,
      { "event6g", "rrc.event6g",
        FT_NONE, BASE_NONE, NULL, 0,
        "PrimaryCPICH_Info", HFILL }},
    { &hf_rrc_measurementQuantity_05,
      { "measurementQuantity", "rrc.measurementQuantity",
        FT_UINT32, BASE_DEC, VALS(rrc_UE_MeasurementQuantity_vals), 0,
        "UE_MeasurementQuantity", HFILL }},
    { &hf_rrc_modeSpecificInfo_131,
      { "modeSpecificInfo", "rrc.modeSpecificInfo",
        FT_UINT32, BASE_DEC, VALS(rrc_T_modeSpecificInfo_131_vals), 0,
        "T_modeSpecificInfo_131", HFILL }},
    { &hf_rrc_fdd_170,
      { "fdd", "rrc.fdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_fdd_164", HFILL }},
    { &hf_rrc_ue_TransmittedPowerFDD,
      { "ue-TransmittedPowerFDD", "rrc.ue_TransmittedPowerFDD",
        FT_UINT32, BASE_DEC, NULL, 0,
        "UE_TransmittedPower", HFILL }},
    { &hf_rrc_ue_RX_TX_ReportEntryList,
      { "ue-RX-TX-ReportEntryList", "rrc.ue_RX_TX_ReportEntryList",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_tdd_121,
      { "tdd", "rrc.tdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_tdd_113", HFILL }},
    { &hf_rrc_ue_TransmittedPowerTDD_List,
      { "ue-TransmittedPowerTDD-List", "rrc.ue_TransmittedPowerTDD_List",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_appliedTA,
      { "appliedTA", "rrc.appliedTA",
        FT_UINT32, BASE_DEC, NULL, 0,
        "UL_TimingAdvance", HFILL }},
    { &hf_rrc_t_ADVinfo,
      { "t-ADVinfo", "rrc.t_ADVinfo",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_modeSpecificInfo_132,
      { "modeSpecificInfo", "rrc.modeSpecificInfo",
        FT_UINT32, BASE_DEC, VALS(rrc_T_modeSpecificInfo_132_vals), 0,
        "T_modeSpecificInfo_132", HFILL }},
    { &hf_rrc_tdd384_768_03,
      { "tdd384-768", "rrc.tdd384_768",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_tdd384_768_02", HFILL }},
    { &hf_rrc_appliedTA_01,
      { "appliedTA", "rrc.appliedTA",
        FT_NONE, BASE_NONE, NULL, 0,
        "EXT_UL_TimingAdvance", HFILL }},
    { &hf_rrc_tdd128_69,
      { "tdd128", "rrc.tdd128",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_tdd128_66", HFILL }},
    { &hf_rrc_t_ADVinfo_01,
      { "t-ADVinfo", "rrc.t_ADVinfo",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_ADVinfo_ext", HFILL }},
    { &hf_rrc_ue_InternalMeasQuantity,
      { "ue-InternalMeasQuantity", "rrc.ue_InternalMeasQuantity",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_ue_InternalReportingQuantity,
      { "ue-InternalReportingQuantity", "rrc.ue_InternalReportingQuantity",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_reportCriteria_19,
      { "reportCriteria", "rrc.reportCriteria",
        FT_UINT32, BASE_DEC, VALS(rrc_UE_InternalReportCriteria_vals), 0,
        "UE_InternalReportCriteria", HFILL }},
    { &hf_rrc_ue_InternalReportingQuantity_01,
      { "ue-InternalReportingQuantity", "rrc.ue_InternalReportingQuantity",
        FT_NONE, BASE_NONE, NULL, 0,
        "UE_InternalReportingQuantity_r4", HFILL }},
    { &hf_rrc_ue_InternalMeasurementID,
      { "ue-InternalMeasurementID", "rrc.ue_InternalMeasurementID",
        FT_UINT32, BASE_DEC, NULL, 0,
        "MeasurementIdentity", HFILL }},
    { &hf_rrc_ue_InternalReportingCriteria,
      { "ue-InternalReportingCriteria", "rrc.ue_InternalReportingCriteria",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_ue_InternalEventParamList,
      { "ue-InternalEventParamList", "rrc.ue_InternalEventParamList",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_ue_TransmittedPower,
      { "ue-TransmittedPower", "rrc.ue_TransmittedPower",
        FT_BOOLEAN, BASE_NONE, NULL, 0,
        "BOOLEAN", HFILL }},
    { &hf_rrc_modeSpecificInfo_133,
      { "modeSpecificInfo", "rrc.modeSpecificInfo",
        FT_UINT32, BASE_DEC, VALS(rrc_T_modeSpecificInfo_133_vals), 0,
        "T_modeSpecificInfo_133", HFILL }},
    { &hf_rrc_fdd_171,
      { "fdd", "rrc.fdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_fdd_165", HFILL }},
    { &hf_rrc_ue_RX_TX_TimeDifference,
      { "ue-RX-TX-TimeDifference", "rrc.ue_RX_TX_TimeDifference",
        FT_BOOLEAN, BASE_NONE, NULL, 0,
        "BOOLEAN", HFILL }},
    { &hf_rrc_tdd_122,
      { "tdd", "rrc.tdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_tdd_114", HFILL }},
    { &hf_rrc_appliedTA_02,
      { "appliedTA", "rrc.appliedTA",
        FT_BOOLEAN, BASE_NONE, NULL, 0,
        "BOOLEAN", HFILL }},
    { &hf_rrc_modeSpecificInfo_134,
      { "modeSpecificInfo", "rrc.modeSpecificInfo",
        FT_UINT32, BASE_DEC, VALS(rrc_T_modeSpecificInfo_134_vals), 0,
        "T_modeSpecificInfo_134", HFILL }},
    { &hf_rrc_fdd_172,
      { "fdd", "rrc.fdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_fdd_166", HFILL }},
    { &hf_rrc_tdd_123,
      { "tdd", "rrc.tdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_tdd_115", HFILL }},
    { &hf_rrc_tddOption_28,
      { "tddOption", "rrc.tddOption",
        FT_UINT32, BASE_DEC, VALS(rrc_T_tddOption_28_vals), 0,
        "T_tddOption_28", HFILL }},
    { &hf_rrc_tdd384_48,
      { "tdd384", "rrc.tdd384",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_tdd384_43", HFILL }},
    { &hf_rrc_tdd128_70,
      { "tdd128", "rrc.tdd128",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_tdd128_67", HFILL }},
    { &hf_rrc_t_ADVinfo_02,
      { "t-ADVinfo", "rrc.t_ADVinfo",
        FT_BOOLEAN, BASE_NONE, NULL, 0,
        "BOOLEAN", HFILL }},
    { &hf_rrc_ue_RX_TX_TimeDifferenceType1,
      { "ue-RX-TX-TimeDifferenceType1", "rrc.ue_RX_TX_TimeDifferenceType1",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_UE_RX_TX_ReportEntryList_item,
      { "UE-RX-TX-ReportEntry", "rrc.UE_RX_TX_ReportEntry",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_ue_RX_TX_TimeDifferenceType2,
      { "ue-RX-TX-TimeDifferenceType2", "rrc.ue_RX_TX_TimeDifferenceType2",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_UE_TransmittedPowerTDD_List_item,
      { "UE-TransmittedPower", "rrc.UE_TransmittedPower",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_rachorcpch,
      { "rachorcpch", "rrc.rachorcpch",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_cipheringKeyFlag,
      { "cipheringKeyFlag", "rrc.cipheringKeyFlag",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_1", HFILL }},
    { &hf_rrc_cipheringSerialNumber,
      { "cipheringSerialNumber", "rrc.cipheringSerialNumber",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_65535", HFILL }},
    { &hf_rrc_dganssreferencetime,
      { "dganssreferencetime", "rrc.dganssreferencetime",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_119", HFILL }},
    { &hf_rrc_dganssInfoList,
      { "dganssInfoList", "rrc.dganssInfoList",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_dganssInfoList_01,
      { "dganssInfoList", "rrc.dganssInfoList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "DGANSSInfoList_r9", HFILL }},
    { &hf_rrc_dganssInfoList_02,
      { "dganssInfoList", "rrc.dganssInfoList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "DGANSSInfoList_v920ext", HFILL }},
    { &hf_rrc_errorReason,
      { "errorReason", "rrc.errorReason",
        FT_UINT32, BASE_DEC, VALS(rrc_UE_Positioning_ErrorCause_vals), 0,
        "UE_Positioning_ErrorCause", HFILL }},
    { &hf_rrc_ue_positioning_GPS_additionalAssistanceDataRequest,
      { "ue-positioning-GPS-additionalAssistanceDataRequest", "rrc.ue_positioning_GPS_additionalAssistanceDataRequest",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_errorReason_01,
      { "errorReason", "rrc.errorReason",
        FT_UINT32, BASE_DEC, VALS(rrc_UE_Positioning_ErrorCause_r7_vals), 0,
        "UE_Positioning_ErrorCause_r7", HFILL }},
    { &hf_rrc_ue_positioning_GANSS_additionalAssistanceDataRequest,
      { "ue-positioning-GANSS-additionalAssistanceDataRequest", "rrc.ue_positioning_GANSS_additionalAssistanceDataRequest",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_ue_positioning_GANSS_additionalAssistanceDataRequest_01,
      { "ue-positioning-GANSS-additionalAssistanceDataRequest", "rrc.ue_positioning_GANSS_additionalAssistanceDataRequest",
        FT_NONE, BASE_NONE, NULL, 0,
        "UE_Positioning_GANSS_AdditionalAssistanceDataRequest_v860ext", HFILL }},
    { &hf_rrc_reportFirstFix,
      { "reportFirstFix", "rrc.reportFirstFix",
        FT_BOOLEAN, BASE_NONE, NULL, 0,
        "BOOLEAN", HFILL }},
    { &hf_rrc_measurementInterval,
      { "measurementInterval", "rrc.measurementInterval",
        FT_UINT32, BASE_DEC, VALS(rrc_UE_Positioning_MeasurementInterval_vals), 0,
        "UE_Positioning_MeasurementInterval", HFILL }},
    { &hf_rrc_eventSpecificInfo,
      { "eventSpecificInfo", "rrc.eventSpecificInfo",
        FT_UINT32, BASE_DEC, VALS(rrc_UE_Positioning_EventSpecificInfo_vals), 0,
        "UE_Positioning_EventSpecificInfo", HFILL }},
    { &hf_rrc_eventSpecificInfo_01,
      { "eventSpecificInfo", "rrc.eventSpecificInfo",
        FT_UINT32, BASE_DEC, VALS(rrc_UE_Positioning_EventSpecificInfo_r7_vals), 0,
        "UE_Positioning_EventSpecificInfo_r7", HFILL }},
    { &hf_rrc_UE_Positioning_EventParamList_item,
      { "UE-Positioning-EventParam", "rrc.UE_Positioning_EventParam",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_UE_Positioning_EventParamList_r7_item,
      { "UE-Positioning-EventParam-r7", "rrc.UE_Positioning_EventParam_r7",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_e7a,
      { "e7a", "rrc.e7a",
        FT_UINT32, BASE_DEC, VALS(rrc_ThresholdPositionChange_vals), 0,
        "ThresholdPositionChange", HFILL }},
    { &hf_rrc_e7b,
      { "e7b", "rrc.e7b",
        FT_UINT32, BASE_DEC, VALS(rrc_ThresholdSFN_SFN_Change_vals), 0,
        "ThresholdSFN_SFN_Change", HFILL }},
    { &hf_rrc_e7c,
      { "e7c", "rrc.e7c",
        FT_UINT32, BASE_DEC, VALS(rrc_ThresholdSFN_GPS_TOW_vals), 0,
        "ThresholdSFN_GPS_TOW", HFILL }},
    { &hf_rrc_e7c_01,
      { "e7c", "rrc.e7c",
        FT_UINT32, BASE_DEC, VALS(rrc_ThresholdSFN_GPS_TOW_us_vals), 0,
        "ThresholdSFN_GPS_TOW_us", HFILL }},
    { &hf_rrc_e7d,
      { "e7d", "rrc.e7d",
        FT_UINT32, BASE_DEC, VALS(rrc_ThresholdSFN_GANSS_TOW_vals), 0,
        "ThresholdSFN_GANSS_TOW", HFILL }},
    { &hf_rrc_utcModel1,
      { "utcModel1", "rrc.utcModel1",
        FT_NONE, BASE_NONE, NULL, 0,
        "UTCmodelSet1", HFILL }},
    { &hf_rrc_utcModel2,
      { "utcModel2", "rrc.utcModel2",
        FT_NONE, BASE_NONE, NULL, 0,
        "UTCmodelSet2", HFILL }},
    { &hf_rrc_utcModel3,
      { "utcModel3", "rrc.utcModel3",
        FT_NONE, BASE_NONE, NULL, 0,
        "UTCmodelSet3", HFILL }},
    { &hf_rrc_ganss_wk_number,
      { "ganss-wk-number", "rrc.ganss_wk_number",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_255", HFILL }},
    { &hf_rrc_alm_keplerianParameters,
      { "alm-keplerianParameters", "rrc.alm_keplerianParameters",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_alm_keplerianNAVAlmanac,
      { "alm-keplerianNAVAlmanac", "rrc.alm_keplerianNAVAlmanac",
        FT_NONE, BASE_NONE, NULL, 0,
        "ALM_NAVKeplerianSet", HFILL }},
    { &hf_rrc_alm_keplerianReducedAlmanac,
      { "alm-keplerianReducedAlmanac", "rrc.alm_keplerianReducedAlmanac",
        FT_NONE, BASE_NONE, NULL, 0,
        "ALM_ReducedKeplerianSet", HFILL }},
    { &hf_rrc_alm_keplerianMidiAlmanac,
      { "alm-keplerianMidiAlmanac", "rrc.alm_keplerianMidiAlmanac",
        FT_NONE, BASE_NONE, NULL, 0,
        "ALM_MidiAlmanacSet", HFILL }},
    { &hf_rrc_alm_keplerianGLONASS,
      { "alm-keplerianGLONASS", "rrc.alm_keplerianGLONASS",
        FT_NONE, BASE_NONE, NULL, 0,
        "ALM_GlonassAlmanacSet", HFILL }},
    { &hf_rrc_alm_ecefSBASAlmanac,
      { "alm-ecefSBASAlmanac", "rrc.alm_ecefSBASAlmanac",
        FT_NONE, BASE_NONE, NULL, 0,
        "ALM_ECEFsbasAlmanacSet", HFILL }},
    { &hf_rrc_complete_Almanac_Provided,
      { "complete-Almanac-Provided", "rrc.complete_Almanac_Provided",
        FT_BOOLEAN, BASE_NONE, NULL, 0,
        "BOOLEAN", HFILL }},
    { &hf_rrc_ganssReferenceTime,
      { "ganssReferenceTime", "rrc.ganssReferenceTime",
        FT_BOOLEAN, BASE_NONE, NULL, 0,
        "BOOLEAN", HFILL }},
    { &hf_rrc_ganssreferenceLocation,
      { "ganssreferenceLocation", "rrc.ganssreferenceLocation",
        FT_BOOLEAN, BASE_NONE, NULL, 0,
        "BOOLEAN", HFILL }},
    { &hf_rrc_ganssIonosphericModel,
      { "ganssIonosphericModel", "rrc.ganssIonosphericModel",
        FT_BOOLEAN, BASE_NONE, NULL, 0,
        "BOOLEAN", HFILL }},
    { &hf_rrc_ganssRequestedGenericAssistanceDataList,
      { "ganssRequestedGenericAssistanceDataList", "rrc.ganssRequestedGenericAssistanceDataList",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_ganssAddIonoModelReq,
      { "ganssAddIonoModelReq", "rrc.ganssAddIonoModelReq",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_2", HFILL }},
    { &hf_rrc_ganssEOPreq,
      { "ganssEOPreq", "rrc.ganssEOPreq",
        FT_UINT32, BASE_DEC, VALS(rrc_T_ganssEOPreq_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_ganssRequestedGenericAssistanceDataList_01,
      { "ganssRequestedGenericAssistanceDataList", "rrc.ganssRequestedGenericAssistanceDataList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "GanssRequestedGenericAssistanceDataList_v860ext", HFILL }},
    { &hf_rrc_dataID_01,
      { "dataID", "rrc.dataID",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_2", HFILL }},
    { &hf_rrc_alpha_beta_parameters,
      { "alpha-beta-parameters", "rrc.alpha_beta_parameters",
        FT_NONE, BASE_NONE, NULL, 0,
        "UE_Positioning_GPS_IonosphericModel", HFILL }},
    { &hf_rrc_non_broadcastIndication,
      { "non-broadcastIndication", "rrc.non_broadcastIndication",
        FT_UINT32, BASE_DEC, VALS(rrc_T_non_broadcastIndication_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_ganssSatInfoNavList,
      { "ganssSatInfoNavList", "rrc.ganssSatInfoNavList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "Ganss_Sat_Info_AddNavList", HFILL }},
    { &hf_rrc_ue_positioning_GANSS_ReferenceTime,
      { "ue-positioning-GANSS-ReferenceTime", "rrc.ue_positioning_GANSS_ReferenceTime",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_uePositioningGanssReferencePosition,
      { "uePositioningGanssReferencePosition", "rrc.uePositioningGanssReferencePosition",
        FT_NONE, BASE_NONE, NULL, 0,
        "ReferenceLocationGANSS", HFILL }},
    { &hf_rrc_uePositioningGanssIonosphericModel,
      { "uePositioningGanssIonosphericModel", "rrc.uePositioningGanssIonosphericModel",
        FT_NONE, BASE_NONE, NULL, 0,
        "UE_Positioning_GANSS_IonosphericModel", HFILL }},
    { &hf_rrc_ganssGenericDataList,
      { "ganssGenericDataList", "rrc.ganssGenericDataList",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_uePositioningGanssAddIonoModel,
      { "uePositioningGanssAddIonoModel", "rrc.uePositioningGanssAddIonoModel",
        FT_NONE, BASE_NONE, NULL, 0,
        "UE_Positioning_GANSS_AddIonoModel", HFILL }},
    { &hf_rrc_uePositioningGanssEarthOrientationPara,
      { "uePositioningGanssEarthOrientationPara", "rrc.uePositioningGanssEarthOrientationPara",
        FT_NONE, BASE_NONE, NULL, 0,
        "UE_Positioning_GANSS_EarthOrientPara", HFILL }},
    { &hf_rrc_ganssGenericDataList_01,
      { "ganssGenericDataList", "rrc.ganssGenericDataList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "GANSSGenericDataList_v860ext", HFILL }},
    { &hf_rrc_ganssGenericDataList_02,
      { "ganssGenericDataList", "rrc.ganssGenericDataList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "GANSSGenericDataList_r8", HFILL }},
    { &hf_rrc_ganssGenericDataList_03,
      { "ganssGenericDataList", "rrc.ganssGenericDataList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "GANSSGenericDataList_r9", HFILL }},
    { &hf_rrc_ganssGenericDataList_04,
      { "ganssGenericDataList", "rrc.ganssGenericDataList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "GANSSGenericDataList_v920ext", HFILL }},
    { &hf_rrc_ue_positioning_GANSS_ReferenceTime_01,
      { "ue-positioning-GANSS-ReferenceTime", "rrc.ue_positioning_GANSS_ReferenceTime",
        FT_NONE, BASE_NONE, NULL, 0,
        "UE_Positioning_GANSS_ReferenceTime_va40ext", HFILL }},
    { &hf_rrc_ganssGenericDataList_05,
      { "ganssGenericDataList", "rrc.ganssGenericDataList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "GANSSGenericDataList_va40ext", HFILL }},
    { &hf_rrc_ue_positioning_GANSS_ReferenceTime_02,
      { "ue-positioning-GANSS-ReferenceTime", "rrc.ue_positioning_GANSS_ReferenceTime",
        FT_NONE, BASE_NONE, NULL, 0,
        "UE_Positioning_GANSS_ReferenceTime_r10", HFILL }},
    { &hf_rrc_ganssGenericDataList_06,
      { "ganssGenericDataList", "rrc.ganssGenericDataList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "GANSSGenericDataList_r10", HFILL }},
    { &hf_rrc_ganssID1,
      { "ganssID1", "rrc.ganssID1",
        FT_UINT32, BASE_DEC, NULL, 0,
        "AuxInfoGANSS_ID1", HFILL }},
    { &hf_rrc_ganssID3,
      { "ganssID3", "rrc.ganssID3",
        FT_UINT32, BASE_DEC, NULL, 0,
        "AuxInfoGANSS_ID3", HFILL }},
    { &hf_rrc_satellite_clock_modelList,
      { "satellite-clock-modelList", "rrc.satellite_clock_modelList",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_navClockModel,
      { "navClockModel", "rrc.navClockModel",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_cnavClockModel,
      { "cnavClockModel", "rrc.cnavClockModel",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_glonassClockModel,
      { "glonassClockModel", "rrc.glonassClockModel",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_sbasClockModel,
      { "sbasClockModel", "rrc.sbasClockModel",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_navKeplerianSet,
      { "navKeplerianSet", "rrc.navKeplerianSet",
        FT_NONE, BASE_NONE, NULL, 0,
        "NavModel_NAVKeplerianSet", HFILL }},
    { &hf_rrc_cnavKeplerianSet,
      { "cnavKeplerianSet", "rrc.cnavKeplerianSet",
        FT_NONE, BASE_NONE, NULL, 0,
        "NavModel_CNAVKeplerianSet", HFILL }},
    { &hf_rrc_glonassECEF,
      { "glonassECEF", "rrc.glonassECEF",
        FT_NONE, BASE_NONE, NULL, 0,
        "NavModel_GLONASSecef", HFILL }},
    { &hf_rrc_sbasECEF,
      { "sbasECEF", "rrc.sbasECEF",
        FT_NONE, BASE_NONE, NULL, 0,
        "NavModel_SBASecef", HFILL }},
    { &hf_rrc_ganss_tod,
      { "ganss-tod", "rrc.ganss_tod",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_59", HFILL }},
    { &hf_rrc_dataBitAssistanceList,
      { "dataBitAssistanceList", "rrc.dataBitAssistanceList",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_teop,
      { "teop", "rrc.teop",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_16", HFILL }},
    { &hf_rrc_pmX,
      { "pmX", "rrc.pmX",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_21", HFILL }},
    { &hf_rrc_pmXdot,
      { "pmXdot", "rrc.pmXdot",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_15", HFILL }},
    { &hf_rrc_pmY,
      { "pmY", "rrc.pmY",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_21", HFILL }},
    { &hf_rrc_pmYdot,
      { "pmYdot", "rrc.pmYdot",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_15", HFILL }},
    { &hf_rrc_deltaUT1,
      { "deltaUT1", "rrc.deltaUT1",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_31", HFILL }},
    { &hf_rrc_deltaUT1dot,
      { "deltaUT1dot", "rrc.deltaUT1dot",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_19", HFILL }},
    { &hf_rrc_alpha_zero_ionos,
      { "alpha-zero-ionos", "rrc.alpha_zero_ionos",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_12", HFILL }},
    { &hf_rrc_alpha_one_ionos,
      { "alpha-one-ionos", "rrc.alpha_one_ionos",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_12", HFILL }},
    { &hf_rrc_alpha_two_ionos,
      { "alpha-two-ionos", "rrc.alpha_two_ionos",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_12", HFILL }},
    { &hf_rrc_gANSS_storm_flags,
      { "gANSS-storm-flags", "rrc.gANSS_storm_flags",
        FT_NONE, BASE_NONE, NULL, 0,
        "GANSS_Storm_Flag", HFILL }},
    { &hf_rrc_referenceTime,
      { "referenceTime", "rrc.referenceTime",
        FT_UINT32, BASE_DEC, VALS(rrc_T_referenceTime_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_utran_GANSSReferenceTimeResult,
      { "utran-GANSSReferenceTimeResult", "rrc.utran_GANSSReferenceTimeResult",
        FT_NONE, BASE_NONE, NULL, 0,
        "UTRAN_GANSSReferenceTime", HFILL }},
    { &hf_rrc_ganssReferenceTimeOnly,
      { "ganssReferenceTimeOnly", "rrc.ganssReferenceTimeOnly",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_ganssGenericMeasurementInfo,
      { "ganssGenericMeasurementInfo", "rrc.ganssGenericMeasurementInfo",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_ganssGenericMeasurementInfo_01,
      { "ganssGenericMeasurementInfo", "rrc.ganssGenericMeasurementInfo",
        FT_UINT32, BASE_DEC, NULL, 0,
        "GANSSGenericMeasurementInfo_v860ext", HFILL }},
    { &hf_rrc_non_broadcastIndication_01,
      { "non-broadcastIndication", "rrc.non_broadcastIndication",
        FT_UINT32, BASE_DEC, VALS(rrc_T_non_broadcastIndication_01_vals), 0,
        "T_non_broadcastIndication_01", HFILL }},
    { &hf_rrc_ganssSatInfoNavList_01,
      { "ganssSatInfoNavList", "rrc.ganssSatInfoNavList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "Ganss_Sat_Info_NavList", HFILL }},
    { &hf_rrc_keplerianParameters,
      { "keplerianParameters", "rrc.keplerianParameters",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_UE_Positioning_GANSS_RealTimeIntegrity_item,
      { "UE-Positioning-GANSS-RealTimeIntegrity item", "rrc.UE_Positioning_GANSS_RealTimeIntegrity_item",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_bad_ganss_satId,
      { "bad-ganss-satId", "rrc.bad_ganss_satId",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_63", HFILL }},
    { &hf_rrc_bad_ganss_signalId,
      { "bad-ganss-signalId", "rrc.bad_ganss_signalId",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_8", HFILL }},
    { &hf_rrc_satelliteInformationList,
      { "satelliteInformationList", "rrc.satelliteInformationList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "GANSSSatelliteInformationList", HFILL }},
    { &hf_rrc_satelliteInformationList_01,
      { "satelliteInformationList", "rrc.satelliteInformationList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "GANSSSatelliteInformationList_va40ext", HFILL }},
    { &hf_rrc_satelliteInformationList_02,
      { "satelliteInformationList", "rrc.satelliteInformationList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "GANSSSatelliteInformationList_r10", HFILL }},
    { &hf_rrc_ganssDay,
      { "ganssDay", "rrc.ganssDay",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_8191", HFILL }},
    { &hf_rrc_ganssTodUncertainty,
      { "ganssTodUncertainty", "rrc.ganssTodUncertainty",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_127", HFILL }},
    { &hf_rrc_ganssTimeId,
      { "ganssTimeId", "rrc.ganssTimeId",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_7", HFILL }},
    { &hf_rrc_utran_ganssreferenceTime,
      { "utran-ganssreferenceTime", "rrc.utran_ganssreferenceTime",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_timingOfCellFrames,
      { "timingOfCellFrames", "rrc.timingOfCellFrames",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_3999999", HFILL }},
    { &hf_rrc_mode,
      { "mode", "rrc.mode",
        FT_UINT32, BASE_DEC, VALS(rrc_T_mode_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_fdd_173,
      { "fdd", "rrc.fdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_fdd_167", HFILL }},
    { &hf_rrc_primary_CPICH_Info,
      { "primary-CPICH-Info", "rrc.primary_CPICH_Info",
        FT_NONE, BASE_NONE, NULL, 0,
        "PrimaryCPICH_Info", HFILL }},
    { &hf_rrc_tdd_124,
      { "tdd", "rrc.tdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_tdd_116", HFILL }},
    { &hf_rrc_cellParameters,
      { "cellParameters", "rrc.cellParameters",
        FT_UINT32, BASE_DEC, NULL, 0,
        "CellParametersID", HFILL }},
    { &hf_rrc_referenceSfn,
      { "referenceSfn", "rrc.referenceSfn",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_4095", HFILL }},
    { &hf_rrc_tutran_ganss_driftRate,
      { "tutran-ganss-driftRate", "rrc.tutran_ganss_driftRate",
        FT_UINT32, BASE_DEC, VALS(rrc_Tutran_Ganss_DriftRate_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_ganss_Day_Cycle_Number,
      { "ganss-Day-Cycle-Number", "rrc.ganss_Day_Cycle_Number",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_7", HFILL }},
    { &hf_rrc_utran_ganssreferenceTime_01,
      { "utran-ganssreferenceTime", "rrc.utran_ganssreferenceTime",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_utran_ganssreferenceTime_01", HFILL }},
    { &hf_rrc_mode_01,
      { "mode", "rrc.mode",
        FT_UINT32, BASE_DEC, VALS(rrc_T_mode_01_vals), 0,
        "T_mode_01", HFILL }},
    { &hf_rrc_fdd_174,
      { "fdd", "rrc.fdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_fdd_168", HFILL }},
    { &hf_rrc_tdd_125,
      { "tdd", "rrc.tdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_tdd_117", HFILL }},
    { &hf_rrc_ganss_timeModelreferenceTime,
      { "ganss-timeModelreferenceTime", "rrc.ganss_timeModelreferenceTime",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_37799", HFILL }},
    { &hf_rrc_ganss_t_a0,
      { "ganss-t-a0", "rrc.ganss_t_a0",
        FT_INT32, BASE_DEC, NULL, 0,
        "INTEGER_M2147483648_2147483647", HFILL }},
    { &hf_rrc_ganss_t_a1,
      { "ganss-t-a1", "rrc.ganss_t_a1",
        FT_INT32, BASE_DEC, NULL, 0,
        "INTEGER_M8388608_8388607", HFILL }},
    { &hf_rrc_ganss_t_a2,
      { "ganss-t-a2", "rrc.ganss_t_a2",
        FT_INT32, BASE_DEC, NULL, 0,
        "INTEGER_M64_63", HFILL }},
    { &hf_rrc_gnss_to_id,
      { "gnss-to-id", "rrc.gnss_to_id",
        FT_UINT32, BASE_DEC, VALS(rrc_T_gnss_to_id_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_ganss_wk_number_01,
      { "ganss-wk-number", "rrc.ganss_wk_number",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_8191", HFILL }},
    { &hf_rrc_deltaT,
      { "deltaT", "rrc.deltaT",
        FT_INT32, BASE_DEC, NULL, 0,
        "INTEGER_M128_127", HFILL }},
    { &hf_rrc_gnss_to_id_01,
      { "gnss-to-id", "rrc.gnss_to_id",
        FT_UINT32, BASE_DEC, VALS(rrc_T_gnss_to_id_01_vals), 0,
        "T_gnss_to_id_01", HFILL }},
    { &hf_rrc_UE_Positioning_GANSS_TimeModels_item,
      { "UE-Positioning-GANSS-TimeModel", "rrc.UE_Positioning_GANSS_TimeModel",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_UE_Positioning_GANSS_TimeModels_va40ext_item,
      { "UE-Positioning-GANSS-TimeModel-va40ext", "rrc.UE_Positioning_GANSS_TimeModel_va40ext",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_a_one_utc,
      { "a-one-utc", "rrc.a_one_utc",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_24", HFILL }},
    { &hf_rrc_a_zero_utc,
      { "a-zero-utc", "rrc.a_zero_utc",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_32", HFILL }},
    { &hf_rrc_t_ot_utc,
      { "t-ot-utc", "rrc.t_ot_utc",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_8", HFILL }},
    { &hf_rrc_delta_t_ls_utc,
      { "delta-t-ls-utc", "rrc.delta_t_ls_utc",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_8", HFILL }},
    { &hf_rrc_w_n_t_utc,
      { "w-n-t-utc", "rrc.w_n_t_utc",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_8", HFILL }},
    { &hf_rrc_w_n_lsf_utc,
      { "w-n-lsf-utc", "rrc.w_n_lsf_utc",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_8", HFILL }},
    { &hf_rrc_dn_utc,
      { "dn-utc", "rrc.dn_utc",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_8", HFILL }},
    { &hf_rrc_delta_t_lsf_utc,
      { "delta-t-lsf-utc", "rrc.delta_t_lsf_utc",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_8", HFILL }},
    { &hf_rrc_gps_ReferenceTime,
      { "gps-ReferenceTime", "rrc.gps_ReferenceTime",
        FT_UINT32, BASE_DEC, NULL, 0,
        "GPS_TOW_1msec", HFILL }},
    { &hf_rrc_utran_GPSReferenceTime,
      { "utran-GPSReferenceTime", "rrc.utran_GPSReferenceTime",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_satelliteInformationList_03,
      { "satelliteInformationList", "rrc.satelliteInformationList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "AcquisitionSatInfoList", HFILL }},
    { &hf_rrc_ue_Positioning_GPS_ReferenceTimeUncertainty,
      { "ue-Positioning-GPS-ReferenceTimeUncertainty", "rrc.ue_Positioning_GPS_ReferenceTimeUncertainty",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_satelliteInformationList_04,
      { "satelliteInformationList", "rrc.satelliteInformationList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "AcquisitionSatInfoList_va40ext", HFILL }},
    { &hf_rrc_satelliteInformationList_05,
      { "satelliteInformationList", "rrc.satelliteInformationList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "AcquisitionSatInfoList_r10", HFILL }},
    { &hf_rrc_almanacRequest,
      { "almanacRequest", "rrc.almanacRequest",
        FT_BOOLEAN, BASE_NONE, NULL, 0,
        "BOOLEAN", HFILL }},
    { &hf_rrc_utcModelRequest,
      { "utcModelRequest", "rrc.utcModelRequest",
        FT_BOOLEAN, BASE_NONE, NULL, 0,
        "BOOLEAN", HFILL }},
    { &hf_rrc_ionosphericModelRequest,
      { "ionosphericModelRequest", "rrc.ionosphericModelRequest",
        FT_BOOLEAN, BASE_NONE, NULL, 0,
        "BOOLEAN", HFILL }},
    { &hf_rrc_navigationModelRequest,
      { "navigationModelRequest", "rrc.navigationModelRequest",
        FT_BOOLEAN, BASE_NONE, NULL, 0,
        "BOOLEAN", HFILL }},
    { &hf_rrc_dgpsCorrectionsRequest,
      { "dgpsCorrectionsRequest", "rrc.dgpsCorrectionsRequest",
        FT_BOOLEAN, BASE_NONE, NULL, 0,
        "BOOLEAN", HFILL }},
    { &hf_rrc_referenceLocationRequest,
      { "referenceLocationRequest", "rrc.referenceLocationRequest",
        FT_BOOLEAN, BASE_NONE, NULL, 0,
        "BOOLEAN", HFILL }},
    { &hf_rrc_referenceTimeRequest,
      { "referenceTimeRequest", "rrc.referenceTimeRequest",
        FT_BOOLEAN, BASE_NONE, NULL, 0,
        "BOOLEAN", HFILL }},
    { &hf_rrc_aquisitionAssistanceRequest,
      { "aquisitionAssistanceRequest", "rrc.aquisitionAssistanceRequest",
        FT_BOOLEAN, BASE_NONE, NULL, 0,
        "BOOLEAN", HFILL }},
    { &hf_rrc_realTimeIntegrityRequest,
      { "realTimeIntegrityRequest", "rrc.realTimeIntegrityRequest",
        FT_BOOLEAN, BASE_NONE, NULL, 0,
        "BOOLEAN", HFILL }},
    { &hf_rrc_navModelAddDataRequest,
      { "navModelAddDataRequest", "rrc.navModelAddDataRequest",
        FT_NONE, BASE_NONE, NULL, 0,
        "UE_Positioning_GPS_NavModelAddDataReq", HFILL }},
    { &hf_rrc_wn_a,
      { "wn-a", "rrc.wn_a",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_8", HFILL }},
    { &hf_rrc_almanacSatInfoList,
      { "almanacSatInfoList", "rrc.almanacSatInfoList",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_sv_GlobalHealth,
      { "sv-GlobalHealth", "rrc.sv_GlobalHealth",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_364", HFILL }},
    { &hf_rrc_ue_positioning_GPS_ReferenceTime,
      { "ue-positioning-GPS-ReferenceTime", "rrc.ue_positioning_GPS_ReferenceTime",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_ue_positioning_GPS_ReferenceLocation,
      { "ue-positioning-GPS-ReferenceLocation", "rrc.ue_positioning_GPS_ReferenceLocation",
        FT_NONE, BASE_NONE, NULL, 0,
        "ReferenceLocation", HFILL }},
    { &hf_rrc_ue_positioning_GPS_DGPS_Corrections,
      { "ue-positioning-GPS-DGPS-Corrections", "rrc.ue_positioning_GPS_DGPS_Corrections",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_ue_positioning_GPS_NavigationModel,
      { "ue-positioning-GPS-NavigationModel", "rrc.ue_positioning_GPS_NavigationModel",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_ue_positioning_GPS_IonosphericModel,
      { "ue-positioning-GPS-IonosphericModel", "rrc.ue_positioning_GPS_IonosphericModel",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_ue_positioning_GPS_UTC_Model,
      { "ue-positioning-GPS-UTC-Model", "rrc.ue_positioning_GPS_UTC_Model",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_ue_positioning_GPS_Almanac,
      { "ue-positioning-GPS-Almanac", "rrc.ue_positioning_GPS_Almanac",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_ue_positioning_GPS_AcquisitionAssistance,
      { "ue-positioning-GPS-AcquisitionAssistance", "rrc.ue_positioning_GPS_AcquisitionAssistance",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_ue_positioning_GPS_Real_timeIntegrity,
      { "ue-positioning-GPS-Real-timeIntegrity", "rrc.ue_positioning_GPS_Real_timeIntegrity",
        FT_UINT32, BASE_DEC, NULL, 0,
        "BadSatList", HFILL }},
    { &hf_rrc_dummy_54,
      { "dummy", "rrc.dummy",
        FT_NONE, BASE_NONE, NULL, 0,
        "UE_Positioning_GPS_ReferenceCellInfo", HFILL }},
    { &hf_rrc_ue_positioning_GPS_ReferenceTime_01,
      { "ue-positioning-GPS-ReferenceTime", "rrc.ue_positioning_GPS_ReferenceTime",
        FT_NONE, BASE_NONE, NULL, 0,
        "UE_Positioning_GPS_ReferenceTime_v770ext", HFILL }},
    { &hf_rrc_ue_positioning_GPS_AcquisitionAssistance_01,
      { "ue-positioning-GPS-AcquisitionAssistance", "rrc.ue_positioning_GPS_AcquisitionAssistance",
        FT_NONE, BASE_NONE, NULL, 0,
        "UE_Positioning_GPS_AcquisitionAssistance_v770ext", HFILL }},
    { &hf_rrc_ue_positioning_GPS_ReferenceTime_02,
      { "ue-positioning-GPS-ReferenceTime", "rrc.ue_positioning_GPS_ReferenceTime",
        FT_NONE, BASE_NONE, NULL, 0,
        "UE_Positioning_GPS_ReferenceTime_r7", HFILL }},
    { &hf_rrc_ue_positioning_GPS_AcquisitionAssistance_02,
      { "ue-positioning-GPS-AcquisitionAssistance", "rrc.ue_positioning_GPS_AcquisitionAssistance",
        FT_NONE, BASE_NONE, NULL, 0,
        "UE_Positioning_GPS_AcquisitionAssistance_r7", HFILL }},
    { &hf_rrc_ue_positioning_GPS_DGPS_Corrections_01,
      { "ue-positioning-GPS-DGPS-Corrections", "rrc.ue_positioning_GPS_DGPS_Corrections",
        FT_NONE, BASE_NONE, NULL, 0,
        "UE_Positioning_GPS_DGPS_Corrections_r9", HFILL }},
    { &hf_rrc_ue_positioning_GPS_DGPS_Corrections_02,
      { "ue-positioning-GPS-DGPS-Corrections", "rrc.ue_positioning_GPS_DGPS_Corrections",
        FT_NONE, BASE_NONE, NULL, 0,
        "UE_Positioning_GPS_DGPS_Corrections_v920ext", HFILL }},
    { &hf_rrc_ue_positioning_GPS_ReferenceTime_03,
      { "ue-positioning-GPS-ReferenceTime", "rrc.ue_positioning_GPS_ReferenceTime",
        FT_NONE, BASE_NONE, NULL, 0,
        "UE_Positioning_GPS_ReferenceTime_va40ext", HFILL }},
    { &hf_rrc_ue_positioning_GPS_Almanac_01,
      { "ue-positioning-GPS-Almanac", "rrc.ue_positioning_GPS_Almanac",
        FT_NONE, BASE_NONE, NULL, 0,
        "UE_Positioning_GPS_Almanac_va40ext", HFILL }},
    { &hf_rrc_ue_positioning_GPS_AcquisitionAssistance_03,
      { "ue-positioning-GPS-AcquisitionAssistance", "rrc.ue_positioning_GPS_AcquisitionAssistance",
        FT_NONE, BASE_NONE, NULL, 0,
        "UE_Positioning_GPS_AcquisitionAssistance_va40ext", HFILL }},
    { &hf_rrc_ue_positioning_GPS_ReferenceTime_04,
      { "ue-positioning-GPS-ReferenceTime", "rrc.ue_positioning_GPS_ReferenceTime",
        FT_NONE, BASE_NONE, NULL, 0,
        "UE_Positioning_GPS_ReferenceTime_r10", HFILL }},
    { &hf_rrc_ue_positioning_GPS_Almanac_02,
      { "ue-positioning-GPS-Almanac", "rrc.ue_positioning_GPS_Almanac",
        FT_NONE, BASE_NONE, NULL, 0,
        "UE_Positioning_GPS_Almanac_r10", HFILL }},
    { &hf_rrc_ue_positioning_GPS_AcquisitionAssistance_04,
      { "ue-positioning-GPS-AcquisitionAssistance", "rrc.ue_positioning_GPS_AcquisitionAssistance",
        FT_NONE, BASE_NONE, NULL, 0,
        "UE_Positioning_GPS_AcquisitionAssistance_r10", HFILL }},
    { &hf_rrc_gps_TOW,
      { "gps-TOW", "rrc.gps_TOW",
        FT_UINT32, BASE_DEC, NULL, 0,
        "GPS_TOW_1sec", HFILL }},
    { &hf_rrc_statusHealth,
      { "statusHealth", "rrc.statusHealth",
        FT_UINT32, BASE_DEC, VALS(rrc_DiffCorrectionStatus_vals), 0,
        "DiffCorrectionStatus", HFILL }},
    { &hf_rrc_dgps_CorrectionSatInfoList,
      { "dgps-CorrectionSatInfoList", "rrc.dgps_CorrectionSatInfoList",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_dgps_CorrectionSatInfoList_01,
      { "dgps-CorrectionSatInfoList", "rrc.dgps_CorrectionSatInfoList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "DGPS_CorrectionSatInfoList_r9", HFILL }},
    { &hf_rrc_dgps_CorrectionSatInfoList_02,
      { "dgps-CorrectionSatInfoList", "rrc.dgps_CorrectionSatInfoList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "DGPS_CorrectionSatInfoList_v920ext", HFILL }},
    { &hf_rrc_alpha0,
      { "alpha0", "rrc.alpha0",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_8", HFILL }},
    { &hf_rrc_alpha1,
      { "alpha1", "rrc.alpha1",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_8", HFILL }},
    { &hf_rrc_alpha2,
      { "alpha2", "rrc.alpha2",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_8", HFILL }},
    { &hf_rrc_alpha3,
      { "alpha3", "rrc.alpha3",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_8", HFILL }},
    { &hf_rrc_beta0,
      { "beta0", "rrc.beta0",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_8", HFILL }},
    { &hf_rrc_beta1,
      { "beta1", "rrc.beta1",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_8", HFILL }},
    { &hf_rrc_beta2,
      { "beta2", "rrc.beta2",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_8", HFILL }},
    { &hf_rrc_beta3,
      { "beta3", "rrc.beta3",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_8", HFILL }},
    { &hf_rrc_referenceTime_01,
      { "referenceTime", "rrc.referenceTime",
        FT_UINT32, BASE_DEC, VALS(rrc_T_referenceTime_01_vals), 0,
        "T_referenceTime_01", HFILL }},
    { &hf_rrc_utran_GPSReferenceTimeResult,
      { "utran-GPSReferenceTimeResult", "rrc.utran_GPSReferenceTimeResult",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_gps_ReferenceTimeOnly,
      { "gps-ReferenceTimeOnly", "rrc.gps_ReferenceTimeOnly",
        FT_UINT32, BASE_DEC, NULL, 0,
        "GPS_TOW_1msec", HFILL }},
    { &hf_rrc_gps_MeasurementParamList,
      { "gps-MeasurementParamList", "rrc.gps_MeasurementParamList",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_navigationModelSatInfoList,
      { "navigationModelSatInfoList", "rrc.navigationModelSatInfoList",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_gps_Week,
      { "gps-Week", "rrc.gps_Week",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_1023", HFILL }},
    { &hf_rrc_gps_Toe,
      { "gps-Toe", "rrc.gps_Toe",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_255", HFILL }},
    { &hf_rrc_tToeLimit,
      { "tToeLimit", "rrc.tToeLimit",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_15", HFILL }},
    { &hf_rrc_satDataList,
      { "satDataList", "rrc.satDataList",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_modeSpecificInfo_135,
      { "modeSpecificInfo", "rrc.modeSpecificInfo",
        FT_UINT32, BASE_DEC, VALS(rrc_T_modeSpecificInfo_135_vals), 0,
        "T_modeSpecificInfo_135", HFILL }},
    { &hf_rrc_fdd_175,
      { "fdd", "rrc.fdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_fdd_169", HFILL }},
    { &hf_rrc_referenceIdentity,
      { "referenceIdentity", "rrc.referenceIdentity",
        FT_NONE, BASE_NONE, NULL, 0,
        "PrimaryCPICH_Info", HFILL }},
    { &hf_rrc_tdd_126,
      { "tdd", "rrc.tdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_tdd_118", HFILL }},
    { &hf_rrc_referenceIdentity_01,
      { "referenceIdentity", "rrc.referenceIdentity",
        FT_UINT32, BASE_DEC, NULL, 0,
        "CellParametersID", HFILL }},
    { &hf_rrc_gps_tow_1msec,
      { "gps-tow-1msec", "rrc.gps_tow_1msec",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_sfn_tow_Uncertainty,
      { "sfn-tow-Uncertainty", "rrc.sfn_tow_Uncertainty",
        FT_UINT32, BASE_DEC, VALS(rrc_SFN_TOW_Uncertainty_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_utran_GPS_DriftRate,
      { "utran-GPS-DriftRate", "rrc.utran_GPS_DriftRate",
        FT_UINT32, BASE_DEC, VALS(rrc_UTRAN_GPS_DriftRate_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_gps_TOW_AssistList,
      { "gps-TOW-AssistList", "rrc.gps_TOW_AssistList",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_gps_Week_Cycle_Number,
      { "gps-Week-Cycle-Number", "rrc.gps_Week_Cycle_Number",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_7", HFILL }},
    { &hf_rrc_a1,
      { "a1", "rrc.a1",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_24", HFILL }},
    { &hf_rrc_a0,
      { "a0", "rrc.a0",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_32", HFILL }},
    { &hf_rrc_t_ot,
      { "t-ot", "rrc.t_ot",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_8", HFILL }},
    { &hf_rrc_wn_t,
      { "wn-t", "rrc.wn_t",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_8", HFILL }},
    { &hf_rrc_delta_t_LS,
      { "delta-t-LS", "rrc.delta_t_LS",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_8", HFILL }},
    { &hf_rrc_wn_lsf,
      { "wn-lsf", "rrc.wn_lsf",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_8", HFILL }},
    { &hf_rrc_dn,
      { "dn", "rrc.dn",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_8", HFILL }},
    { &hf_rrc_delta_t_LSF,
      { "delta-t-LSF", "rrc.delta_t_LSF",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_8", HFILL }},
    { &hf_rrc_ip_Spacing,
      { "ip-Spacing", "rrc.ip_Spacing",
        FT_UINT32, BASE_DEC, VALS(rrc_IP_Spacing_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_ip_Length,
      { "ip-Length", "rrc.ip_Length",
        FT_UINT32, BASE_DEC, VALS(rrc_IP_Length_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_ip_Offset,
      { "ip-Offset", "rrc.ip_Offset",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_9", HFILL }},
    { &hf_rrc_seed,
      { "seed", "rrc.seed",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_63", HFILL }},
    { &hf_rrc_burstModeParameters,
      { "burstModeParameters", "rrc.burstModeParameters",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_modeSpecificInfo_136,
      { "modeSpecificInfo", "rrc.modeSpecificInfo",
        FT_UINT32, BASE_DEC, VALS(rrc_T_modeSpecificInfo_136_vals), 0,
        "T_modeSpecificInfo_136", HFILL }},
    { &hf_rrc_fdd_176,
      { "fdd", "rrc.fdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_fdd_170", HFILL }},
    { &hf_rrc_tdd_127,
      { "tdd", "rrc.tdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_tdd_119", HFILL }},
    { &hf_rrc_ip_Spacing_TDD,
      { "ip-Spacing-TDD", "rrc.ip_Spacing_TDD",
        FT_UINT32, BASE_DEC, VALS(rrc_IP_Spacing_TDD_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_ip_slot,
      { "ip-slot", "rrc.ip_slot",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_14", HFILL }},
    { &hf_rrc_ip_Start,
      { "ip-Start", "rrc.ip_Start",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_4095", HFILL }},
    { &hf_rrc_ip_PCCPCG,
      { "ip-PCCPCG", "rrc.ip_PCCPCG",
        FT_BOOLEAN, BASE_NONE, NULL, 0,
        "IP_PCCPCH_r4", HFILL }},
    { &hf_rrc_ip_Spacing_01,
      { "ip-Spacing", "rrc.ip_Spacing",
        FT_UINT32, BASE_DEC, VALS(rrc_IP_Spacing_TDD_vals), 0,
        "IP_Spacing_TDD", HFILL }},
    { &hf_rrc_ue_positioning_OTDOA_Measurement,
      { "ue-positioning-OTDOA-Measurement", "rrc.ue_positioning_OTDOA_Measurement",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_ue_positioning_PositionEstimateInfo,
      { "ue-positioning-PositionEstimateInfo", "rrc.ue_positioning_PositionEstimateInfo",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_ue_positioning_GPS_Measurement,
      { "ue-positioning-GPS-Measurement", "rrc.ue_positioning_GPS_Measurement",
        FT_NONE, BASE_NONE, NULL, 0,
        "UE_Positioning_GPS_MeasurementResults", HFILL }},
    { &hf_rrc_ue_positioning_Error,
      { "ue-positioning-Error", "rrc.ue_positioning_Error",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_ue_positioning_PositionEstimateInfo_01,
      { "ue-positioning-PositionEstimateInfo", "rrc.ue_positioning_PositionEstimateInfo",
        FT_NONE, BASE_NONE, NULL, 0,
        "UE_Positioning_PositionEstimateInfo_v770ext", HFILL }},
    { &hf_rrc_ue_positioning_GPS_Measurement_01,
      { "ue-positioning-GPS-Measurement", "rrc.ue_positioning_GPS_Measurement",
        FT_NONE, BASE_NONE, NULL, 0,
        "UE_Positioning_GPS_MeasurementResults_v770ext", HFILL }},
    { &hf_rrc_ue_positioning_Error_01,
      { "ue-positioning-Error", "rrc.ue_positioning_Error",
        FT_NONE, BASE_NONE, NULL, 0,
        "UE_Positioning_Error_r7", HFILL }},
    { &hf_rrc_ue_positioning_Ganss_MeasuredResults,
      { "ue-positioning-Ganss-MeasuredResults", "rrc.ue_positioning_Ganss_MeasuredResults",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_ue_Positioning_OTDOA_Measurement_v390ext,
      { "ue-Positioning-OTDOA-Measurement-v390ext", "rrc.ue_Positioning_OTDOA_Measurement_v390ext",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_ue_positioning_Error_02,
      { "ue-positioning-Error", "rrc.ue_positioning_Error",
        FT_NONE, BASE_NONE, NULL, 0,
        "UE_Positioning_Error_v860ext", HFILL }},
    { &hf_rrc_ue_positioning_Ganss_MeasurementResults,
      { "ue-positioning-Ganss-MeasurementResults", "rrc.ue_positioning_Ganss_MeasurementResults",
        FT_NONE, BASE_NONE, NULL, 0,
        "UE_Positioning_GANSS_MeasuredResults_v860ext", HFILL }},
    { &hf_rrc_ue_positioning_ReportingQuantity,
      { "ue-positioning-ReportingQuantity", "rrc.ue_positioning_ReportingQuantity",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_reportCriteria_20,
      { "reportCriteria", "rrc.reportCriteria",
        FT_UINT32, BASE_DEC, VALS(rrc_UE_Positioning_ReportCriteria_vals), 0,
        "UE_Positioning_ReportCriteria", HFILL }},
    { &hf_rrc_ue_positioning_OTDOA_AssistanceData,
      { "ue-positioning-OTDOA-AssistanceData", "rrc.ue_positioning_OTDOA_AssistanceData",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_ue_positioning_ReportingQuantity_v390ext,
      { "ue-positioning-ReportingQuantity-v390ext", "rrc.ue_positioning_ReportingQuantity_v390ext",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_ue_positioning_ReportingQuantity_01,
      { "ue-positioning-ReportingQuantity", "rrc.ue_positioning_ReportingQuantity",
        FT_NONE, BASE_NONE, NULL, 0,
        "UE_Positioning_ReportingQuantity_r4", HFILL }},
    { &hf_rrc_ue_positioning_OTDOA_AssistanceData_01,
      { "ue-positioning-OTDOA-AssistanceData", "rrc.ue_positioning_OTDOA_AssistanceData",
        FT_NONE, BASE_NONE, NULL, 0,
        "UE_Positioning_OTDOA_AssistanceData_r4", HFILL }},
    { &hf_rrc_ue_positioning_ReportingQuantity_02,
      { "ue-positioning-ReportingQuantity", "rrc.ue_positioning_ReportingQuantity",
        FT_NONE, BASE_NONE, NULL, 0,
        "UE_Positioning_ReportingQuantity_r7", HFILL }},
    { &hf_rrc_reportCriteria_21,
      { "reportCriteria", "rrc.reportCriteria",
        FT_UINT32, BASE_DEC, VALS(rrc_UE_Positioning_ReportCriteria_r7_vals), 0,
        "UE_Positioning_ReportCriteria_r7", HFILL }},
    { &hf_rrc_ue_positioning_OTDOA_AssistanceData_02,
      { "ue-positioning-OTDOA-AssistanceData", "rrc.ue_positioning_OTDOA_AssistanceData",
        FT_NONE, BASE_NONE, NULL, 0,
        "UE_Positioning_OTDOA_AssistanceData_r7", HFILL }},
    { &hf_rrc_ue_positioning_GPS_AssistanceData_01,
      { "ue-positioning-GPS-AssistanceData", "rrc.ue_positioning_GPS_AssistanceData",
        FT_NONE, BASE_NONE, NULL, 0,
        "UE_Positioning_GPS_AssistanceData_r7", HFILL }},
    { &hf_rrc_ue_positioning_ReportingQuantity_03,
      { "ue-positioning-ReportingQuantity", "rrc.ue_positioning_ReportingQuantity",
        FT_NONE, BASE_NONE, NULL, 0,
        "UE_Positioning_ReportingQuantity_r8", HFILL }},
    { &hf_rrc_ue_positioning_GANSS_AssistanceData_01,
      { "ue-positioning-GANSS-AssistanceData", "rrc.ue_positioning_GANSS_AssistanceData",
        FT_NONE, BASE_NONE, NULL, 0,
        "UE_Positioning_GANSS_AssistanceData_r8", HFILL }},
    { &hf_rrc_ue_positioning_GPS_AssistanceData_02,
      { "ue-positioning-GPS-AssistanceData", "rrc.ue_positioning_GPS_AssistanceData",
        FT_NONE, BASE_NONE, NULL, 0,
        "UE_Positioning_GPS_AssistanceData_r9", HFILL }},
    { &hf_rrc_ue_positioning_GANSS_AssistanceData_02,
      { "ue-positioning-GANSS-AssistanceData", "rrc.ue_positioning_GANSS_AssistanceData",
        FT_NONE, BASE_NONE, NULL, 0,
        "UE_Positioning_GANSS_AssistanceData_r9", HFILL }},
    { &hf_rrc_ue_positioning_GPS_AssistanceData_03,
      { "ue-positioning-GPS-AssistanceData", "rrc.ue_positioning_GPS_AssistanceData",
        FT_NONE, BASE_NONE, NULL, 0,
        "UE_Positioning_GPS_AssistanceData_r10", HFILL }},
    { &hf_rrc_ue_positioning_GANSS_AssistanceData_03,
      { "ue-positioning-GANSS-AssistanceData", "rrc.ue_positioning_GANSS_AssistanceData",
        FT_NONE, BASE_NONE, NULL, 0,
        "UE_Positioning_GANSS_AssistanceData_r10", HFILL }},
    { &hf_rrc_event7a,
      { "event7a", "rrc.event7a",
        FT_NONE, BASE_NONE, NULL, 0,
        "UE_Positioning_PositionEstimateInfo", HFILL }},
    { &hf_rrc_event7b,
      { "event7b", "rrc.event7b",
        FT_NONE, BASE_NONE, NULL, 0,
        "UE_Positioning_OTDOA_Measurement", HFILL }},
    { &hf_rrc_event7c,
      { "event7c", "rrc.event7c",
        FT_NONE, BASE_NONE, NULL, 0,
        "UE_Positioning_GPS_MeasurementResults", HFILL }},
    { &hf_rrc_additionalOrReplacedPosMeasEvent,
      { "additionalOrReplacedPosMeasEvent", "rrc.additionalOrReplacedPosMeasEvent",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_event7a_01,
      { "event7a", "rrc.event7a",
        FT_NONE, BASE_NONE, NULL, 0,
        "UE_Positioning_PositionEstimateInfo_v770ext", HFILL }},
    { &hf_rrc_event7c_01,
      { "event7c", "rrc.event7c",
        FT_NONE, BASE_NONE, NULL, 0,
        "UE_Positioning_GPS_MeasurementResults_v770ext", HFILL }},
    { &hf_rrc_event7d,
      { "event7d", "rrc.event7d",
        FT_NONE, BASE_NONE, NULL, 0,
        "UE_Positioning_GANSS_MeasuredResults", HFILL }},
    { &hf_rrc_event7d_01,
      { "event7d", "rrc.event7d",
        FT_NONE, BASE_NONE, NULL, 0,
        "UE_Positioning_GANSS_MeasuredResults_v860ext", HFILL }},
    { &hf_rrc_ue_positioning_OTDOA_ReferenceCellInfo,
      { "ue-positioning-OTDOA-ReferenceCellInfo", "rrc.ue_positioning_OTDOA_ReferenceCellInfo",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_ue_positioning_OTDOA_NeighbourCellList,
      { "ue-positioning-OTDOA-NeighbourCellList", "rrc.ue_positioning_OTDOA_NeighbourCellList",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_ue_positioning_OTDOA_ReferenceCellInfo_01,
      { "ue-positioning-OTDOA-ReferenceCellInfo", "rrc.ue_positioning_OTDOA_ReferenceCellInfo",
        FT_NONE, BASE_NONE, NULL, 0,
        "UE_Positioning_OTDOA_ReferenceCellInfo_r4", HFILL }},
    { &hf_rrc_ue_positioning_OTDOA_NeighbourCellList_01,
      { "ue-positioning-OTDOA-NeighbourCellList", "rrc.ue_positioning_OTDOA_NeighbourCellList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "UE_Positioning_OTDOA_NeighbourCellList_r4", HFILL }},
    { &hf_rrc_ue_Positioning_IPDL_Parameters_TDD_r4_ext,
      { "ue-Positioning-IPDL-Parameters-TDD-r4-ext", "rrc.ue_Positioning_IPDL_Parameters_TDD_r4_ext",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_ue_Positioning_IPDL_Parameters_TDDList_r4_ext,
      { "ue-Positioning-IPDL-Parameters-TDDList-r4-ext", "rrc.ue_Positioning_IPDL_Parameters_TDDList_r4_ext",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_ue_positioning_OTDOA_ReferenceCellInfo_UEB,
      { "ue-positioning-OTDOA-ReferenceCellInfo-UEB", "rrc.ue_positioning_OTDOA_ReferenceCellInfo_UEB",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_ue_positioning_OTDOA_NeighbourCellList_UEB,
      { "ue-positioning-OTDOA-NeighbourCellList-UEB", "rrc.ue_positioning_OTDOA_NeighbourCellList_UEB",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_ue_positioning_OTDOA_ReferenceCellInfo_UEB_ext,
      { "ue-positioning-OTDOA-ReferenceCellInfo-UEB-ext", "rrc.ue_positioning_OTDOA_ReferenceCellInfo_UEB_ext",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_ue_positioning_OTDOA_NeighbourCellList_UEB_ext,
      { "ue-positioning-OTDOA-NeighbourCellList-UEB-ext", "rrc.ue_positioning_OTDOA_NeighbourCellList_UEB_ext",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_ue_positioning_OTDOA_ReferenceCellInfo_02,
      { "ue-positioning-OTDOA-ReferenceCellInfo", "rrc.ue_positioning_OTDOA_ReferenceCellInfo",
        FT_NONE, BASE_NONE, NULL, 0,
        "UE_Positioning_OTDOA_ReferenceCellInfo_r7", HFILL }},
    { &hf_rrc_ue_positioning_OTDOA_NeighbourCellList_02,
      { "ue-positioning-OTDOA-NeighbourCellList", "rrc.ue_positioning_OTDOA_NeighbourCellList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "UE_Positioning_OTDOA_NeighbourCellList_r7", HFILL }},
    { &hf_rrc_UE_Positioning_IPDL_Parameters_TDDList_r4_ext_item,
      { "UE-Positioning-IPDL-Parameters-TDD-r4-ext", "rrc.UE_Positioning_IPDL_Parameters_TDD_r4_ext",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_neighbourList,
      { "neighbourList", "rrc.neighbourList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "NeighbourList_TDD_r7", HFILL }},
    { &hf_rrc_modeSpecificInfo_137,
      { "modeSpecificInfo", "rrc.modeSpecificInfo",
        FT_UINT32, BASE_DEC, VALS(rrc_T_modeSpecificInfo_137_vals), 0,
        "T_modeSpecificInfo_137", HFILL }},
    { &hf_rrc_fdd_177,
      { "fdd", "rrc.fdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_fdd_171", HFILL }},
    { &hf_rrc_referenceCellIDentity,
      { "referenceCellIDentity", "rrc.referenceCellIDentity",
        FT_NONE, BASE_NONE, NULL, 0,
        "PrimaryCPICH_Info", HFILL }},
    { &hf_rrc_ue_RX_TX_TimeDifferenceType2Info,
      { "ue-RX-TX-TimeDifferenceType2Info", "rrc.ue_RX_TX_TimeDifferenceType2Info",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_tdd_128,
      { "tdd", "rrc.tdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_tdd_120", HFILL }},
    { &hf_rrc_referenceCellIdentity,
      { "referenceCellIdentity", "rrc.referenceCellIdentity",
        FT_UINT32, BASE_DEC, NULL, 0,
        "CellParametersID", HFILL }},
    { &hf_rrc_neighbourList_01,
      { "neighbourList", "rrc.neighbourList",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_neighbourList_v390ext,
      { "neighbourList-v390ext", "rrc.neighbourList_v390ext",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_modeSpecificInfo_138,
      { "modeSpecificInfo", "rrc.modeSpecificInfo",
        FT_UINT32, BASE_DEC, VALS(rrc_T_modeSpecificInfo_138_vals), 0,
        "T_modeSpecificInfo_138", HFILL }},
    { &hf_rrc_fdd_178,
      { "fdd", "rrc.fdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_fdd_172", HFILL }},
    { &hf_rrc_tdd_129,
      { "tdd", "rrc.tdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_tdd_121", HFILL }},
    { &hf_rrc_cellAndChannelIdentity,
      { "cellAndChannelIdentity", "rrc.cellAndChannelIdentity",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_ue_positioning_IPDL_Paremeters,
      { "ue-positioning-IPDL-Paremeters", "rrc.ue_positioning_IPDL_Paremeters",
        FT_NONE, BASE_NONE, NULL, 0,
        "UE_Positioning_IPDL_Parameters", HFILL }},
    { &hf_rrc_sfn_SFN_RelTimeDifference,
      { "sfn-SFN-RelTimeDifference", "rrc.sfn_SFN_RelTimeDifference",
        FT_NONE, BASE_NONE, NULL, 0,
        "SFN_SFN_RelTimeDifference1", HFILL }},
    { &hf_rrc_sfn_SFN_Drift,
      { "sfn-SFN-Drift", "rrc.sfn_SFN_Drift",
        FT_UINT32, BASE_DEC|BASE_EXT_STRING, &rrc_SFN_SFN_Drift_vals_ext, 0,
        NULL, HFILL }},
    { &hf_rrc_searchWindowSize,
      { "searchWindowSize", "rrc.searchWindowSize",
        FT_UINT32, BASE_DEC, VALS(rrc_OTDOA_SearchWindowSize_vals), 0,
        "OTDOA_SearchWindowSize", HFILL }},
    { &hf_rrc_positioningMode,
      { "positioningMode", "rrc.positioningMode",
        FT_UINT32, BASE_DEC, VALS(rrc_T_positioningMode_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_ueBased,
      { "ueBased", "rrc.ueBased",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_ueAssisted,
      { "ueAssisted", "rrc.ueAssisted",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_modeSpecificInfo_139,
      { "modeSpecificInfo", "rrc.modeSpecificInfo",
        FT_UINT32, BASE_DEC, VALS(rrc_T_modeSpecificInfo_139_vals), 0,
        "T_modeSpecificInfo_139", HFILL }},
    { &hf_rrc_fdd_179,
      { "fdd", "rrc.fdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_fdd_173", HFILL }},
    { &hf_rrc_tdd_130,
      { "tdd", "rrc.tdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_tdd_122", HFILL }},
    { &hf_rrc_ue_positioning_IPDL_Paremeters_01,
      { "ue-positioning-IPDL-Paremeters", "rrc.ue_positioning_IPDL_Paremeters",
        FT_NONE, BASE_NONE, NULL, 0,
        "UE_Positioning_IPDL_Parameters_r4", HFILL }},
    { &hf_rrc_positioningMode_01,
      { "positioningMode", "rrc.positioningMode",
        FT_UINT32, BASE_DEC, VALS(rrc_T_positioningMode_01_vals), 0,
        "T_positioningMode_01", HFILL }},
    { &hf_rrc_ueBased_01,
      { "ueBased", "rrc.ueBased",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_ueBased_01", HFILL }},
    { &hf_rrc_relativeNorth,
      { "relativeNorth", "rrc.relativeNorth",
        FT_INT32, BASE_DEC, NULL, 0,
        "INTEGER_M20000_20000", HFILL }},
    { &hf_rrc_relativeEast,
      { "relativeEast", "rrc.relativeEast",
        FT_INT32, BASE_DEC, NULL, 0,
        "INTEGER_M20000_20000", HFILL }},
    { &hf_rrc_relativeAltitude,
      { "relativeAltitude", "rrc.relativeAltitude",
        FT_INT32, BASE_DEC, NULL, 0,
        "INTEGER_M4000_4000", HFILL }},
    { &hf_rrc_fineSFN_SFN,
      { "fineSFN-SFN", "rrc.fineSFN_SFN",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_roundTripTime,
      { "roundTripTime", "rrc.roundTripTime",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_32766", HFILL }},
    { &hf_rrc_ueAssisted_01,
      { "ueAssisted", "rrc.ueAssisted",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_ueAssisted_01", HFILL }},
    { &hf_rrc_modeSpecificInfo_140,
      { "modeSpecificInfo", "rrc.modeSpecificInfo",
        FT_UINT32, BASE_DEC, VALS(rrc_T_modeSpecificInfo_140_vals), 0,
        "T_modeSpecificInfo_140", HFILL }},
    { &hf_rrc_fdd_180,
      { "fdd", "rrc.fdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_fdd_174", HFILL }},
    { &hf_rrc_tdd_131,
      { "tdd", "rrc.tdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_tdd_123", HFILL }},
    { &hf_rrc_roundTripTimeExtension,
      { "roundTripTimeExtension", "rrc.roundTripTimeExtension",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_70274", HFILL }},
    { &hf_rrc_modeSpecificInfo_141,
      { "modeSpecificInfo", "rrc.modeSpecificInfo",
        FT_UINT32, BASE_DEC, VALS(rrc_T_modeSpecificInfo_141_vals), 0,
        "T_modeSpecificInfo_141", HFILL }},
    { &hf_rrc_fdd_181,
      { "fdd", "rrc.fdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_fdd_175", HFILL }},
    { &hf_rrc_tdd_132,
      { "tdd", "rrc.tdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_tdd_124", HFILL }},
    { &hf_rrc_positioningMode_02,
      { "positioningMode", "rrc.positioningMode",
        FT_UINT32, BASE_DEC, VALS(rrc_T_positioningMode_02_vals), 0,
        "T_positioningMode_02", HFILL }},
    { &hf_rrc_ueBased_02,
      { "ueBased", "rrc.ueBased",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_ueBased_02", HFILL }},
    { &hf_rrc_ueAssisted_02,
      { "ueAssisted", "rrc.ueAssisted",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_ueAssisted_02", HFILL }},
    { &hf_rrc_UE_Positioning_OTDOA_NeighbourCellList_item,
      { "UE-Positioning-OTDOA-NeighbourCellInfo", "rrc.UE_Positioning_OTDOA_NeighbourCellInfo",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_UE_Positioning_OTDOA_NeighbourCellList_r4_item,
      { "UE-Positioning-OTDOA-NeighbourCellInfo-r4", "rrc.UE_Positioning_OTDOA_NeighbourCellInfo_r4",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_UE_Positioning_OTDOA_NeighbourCellList_UEB_item,
      { "UE-Positioning-OTDOA-NeighbourCellInfo-UEB", "rrc.UE_Positioning_OTDOA_NeighbourCellInfo_UEB",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_UE_Positioning_OTDOA_NeighbourCellList_UEB_ext_item,
      { "UE-Positioning-OTDOA-NeighbourCellInfo-UEB-ext", "rrc.UE_Positioning_OTDOA_NeighbourCellInfo_UEB_ext",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_UE_Positioning_OTDOA_NeighbourCellList_r7_item,
      { "UE-Positioning-OTDOA-NeighbourCellInfo-r7", "rrc.UE_Positioning_OTDOA_NeighbourCellInfo_r7",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_stdResolution,
      { "stdResolution", "rrc.stdResolution",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_2", HFILL }},
    { &hf_rrc_numberOfOTDOA_Measurements,
      { "numberOfOTDOA-Measurements", "rrc.numberOfOTDOA_Measurements",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_3", HFILL }},
    { &hf_rrc_stdOfOTDOA_Measurements,
      { "stdOfOTDOA-Measurements", "rrc.stdOfOTDOA_Measurements",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_5", HFILL }},
    { &hf_rrc_modeSpecificInfo_142,
      { "modeSpecificInfo", "rrc.modeSpecificInfo",
        FT_UINT32, BASE_DEC, VALS(rrc_T_modeSpecificInfo_142_vals), 0,
        "T_modeSpecificInfo_142", HFILL }},
    { &hf_rrc_fdd_182,
      { "fdd", "rrc.fdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_fdd_176", HFILL }},
    { &hf_rrc_tdd_133,
      { "tdd", "rrc.tdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_tdd_125", HFILL }},
    { &hf_rrc_positioningMode_03,
      { "positioningMode", "rrc.positioningMode",
        FT_UINT32, BASE_DEC, VALS(rrc_T_positioningMode_03_vals), 0,
        "T_positioningMode_03", HFILL }},
    { &hf_rrc_ueBased_03,
      { "ueBased", "rrc.ueBased",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_ueBased_03", HFILL }},
    { &hf_rrc_ueAssisted_03,
      { "ueAssisted", "rrc.ueAssisted",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_ueAssisted_03", HFILL }},
    { &hf_rrc_modeSpecificInfo_143,
      { "modeSpecificInfo", "rrc.modeSpecificInfo",
        FT_UINT32, BASE_DEC, VALS(rrc_T_modeSpecificInfo_143_vals), 0,
        "T_modeSpecificInfo_143", HFILL }},
    { &hf_rrc_fdd_183,
      { "fdd", "rrc.fdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_fdd_177", HFILL }},
    { &hf_rrc_tdd_134,
      { "tdd", "rrc.tdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_tdd_126", HFILL }},
    { &hf_rrc_positioningMode_04,
      { "positioningMode", "rrc.positioningMode",
        FT_UINT32, BASE_DEC, VALS(rrc_T_positioningMode_04_vals), 0,
        "T_positioningMode_04", HFILL }},
    { &hf_rrc_ueBased_04,
      { "ueBased", "rrc.ueBased",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_ueBased_04", HFILL }},
    { &hf_rrc_cellPosition,
      { "cellPosition", "rrc.cellPosition",
        FT_UINT32, BASE_DEC, VALS(rrc_ReferenceCellPosition_vals), 0,
        "ReferenceCellPosition", HFILL }},
    { &hf_rrc_ueAssisted_04,
      { "ueAssisted", "rrc.ueAssisted",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_ueAssisted_04", HFILL }},
    { &hf_rrc_modeSpecificInfo_144,
      { "modeSpecificInfo", "rrc.modeSpecificInfo",
        FT_UINT32, BASE_DEC, VALS(rrc_T_modeSpecificInfo_144_vals), 0,
        "T_modeSpecificInfo_144", HFILL }},
    { &hf_rrc_fdd_184,
      { "fdd", "rrc.fdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_fdd_178", HFILL }},
    { &hf_rrc_tdd_135,
      { "tdd", "rrc.tdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_tdd_127", HFILL }},
    { &hf_rrc_modeSpecificInfo_145,
      { "modeSpecificInfo", "rrc.modeSpecificInfo",
        FT_UINT32, BASE_DEC, VALS(rrc_T_modeSpecificInfo_145_vals), 0,
        "T_modeSpecificInfo_145", HFILL }},
    { &hf_rrc_fdd_185,
      { "fdd", "rrc.fdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_fdd_179", HFILL }},
    { &hf_rrc_tdd_136,
      { "tdd", "rrc.tdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_tdd_128", HFILL }},
    { &hf_rrc_positioningMode_05,
      { "positioningMode", "rrc.positioningMode",
        FT_UINT32, BASE_DEC, VALS(rrc_T_positioningMode_05_vals), 0,
        "T_positioningMode_05", HFILL }},
    { &hf_rrc_ueBased_05,
      { "ueBased", "rrc.ueBased",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_ueBased_05", HFILL }},
    { &hf_rrc_ueAssisted_05,
      { "ueAssisted", "rrc.ueAssisted",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_ueAssisted_05", HFILL }},
    { &hf_rrc_referenceTime_02,
      { "referenceTime", "rrc.referenceTime",
        FT_UINT32, BASE_DEC, VALS(rrc_T_referenceTime_02_vals), 0,
        "T_referenceTime_02", HFILL }},
    { &hf_rrc_cell_Timing,
      { "cell-Timing", "rrc.cell_Timing",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_modeSpecificInfo_146,
      { "modeSpecificInfo", "rrc.modeSpecificInfo",
        FT_UINT32, BASE_DEC, VALS(rrc_T_modeSpecificInfo_146_vals), 0,
        "T_modeSpecificInfo_146", HFILL }},
    { &hf_rrc_fdd_186,
      { "fdd", "rrc.fdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_fdd_180", HFILL }},
    { &hf_rrc_tdd_137,
      { "tdd", "rrc.tdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_tdd_129", HFILL }},
    { &hf_rrc_positionEstimate,
      { "positionEstimate", "rrc.positionEstimate",
        FT_UINT32, BASE_DEC, VALS(rrc_PositionEstimate_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_referenceTimeOptions,
      { "referenceTimeOptions", "rrc.referenceTimeOptions",
        FT_UINT32, BASE_DEC, VALS(rrc_T_referenceTimeOptions_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_earlier_than_r7,
      { "earlier-than-r7", "rrc.earlier_than_r7",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_r7_15,
      { "r7", "rrc.r7",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_r7_15", HFILL }},
    { &hf_rrc_referenceTime_03,
      { "referenceTime", "rrc.referenceTime",
        FT_UINT32, BASE_DEC, VALS(rrc_T_referenceTime_03_vals), 0,
        "T_referenceTime_03", HFILL }},
    { &hf_rrc_positionData,
      { "positionData", "rrc.positionData",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_16", HFILL }},
    { &hf_rrc_velocityEstimate,
      { "velocityEstimate", "rrc.velocityEstimate",
        FT_UINT32, BASE_DEC, VALS(rrc_VelocityEstimate_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_ue_positioning_ReportingCriteria,
      { "ue-positioning-ReportingCriteria", "rrc.ue_positioning_ReportingCriteria",
        FT_UINT32, BASE_DEC, NULL, 0,
        "UE_Positioning_EventParamList", HFILL }},
    { &hf_rrc_ue_positioning_ReportingCriteria_01,
      { "ue-positioning-ReportingCriteria", "rrc.ue_positioning_ReportingCriteria",
        FT_UINT32, BASE_DEC, NULL, 0,
        "UE_Positioning_EventParamList_r7", HFILL }},
    { &hf_rrc_methodType,
      { "methodType", "rrc.methodType",
        FT_UINT32, BASE_DEC, VALS(rrc_UE_Positioning_MethodType_vals), 0,
        "UE_Positioning_MethodType", HFILL }},
    { &hf_rrc_positioningMethod,
      { "positioningMethod", "rrc.positioningMethod",
        FT_UINT32, BASE_DEC, VALS(rrc_PositioningMethod_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_dummy1_06,
      { "dummy1", "rrc.dummy1",
        FT_UINT32, BASE_DEC, VALS(rrc_UE_Positioning_ResponseTime_vals), 0,
        "UE_Positioning_ResponseTime", HFILL }},
    { &hf_rrc_horizontal_Accuracy,
      { "horizontal-Accuracy", "rrc.horizontal_Accuracy",
        FT_BYTES, BASE_NONE, NULL, 0,
        "UE_Positioning_Accuracy", HFILL }},
    { &hf_rrc_gps_TimingOfCellWanted,
      { "gps-TimingOfCellWanted", "rrc.gps_TimingOfCellWanted",
        FT_BOOLEAN, BASE_NONE, NULL, 0,
        "BOOLEAN", HFILL }},
    { &hf_rrc_additionalAssistanceDataRequest,
      { "additionalAssistanceDataRequest", "rrc.additionalAssistanceDataRequest",
        FT_BOOLEAN, BASE_NONE, NULL, 0,
        "BOOLEAN", HFILL }},
    { &hf_rrc_environmentCharacterisation,
      { "environmentCharacterisation", "rrc.environmentCharacterisation",
        FT_UINT32, BASE_DEC, VALS(rrc_EnvironmentCharacterisation_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_vertical_Accuracy,
      { "vertical-Accuracy", "rrc.vertical_Accuracy",
        FT_BYTES, BASE_NONE, NULL, 0,
        "UE_Positioning_Accuracy", HFILL }},
    { &hf_rrc_horizontalAccuracy,
      { "horizontalAccuracy", "rrc.horizontalAccuracy",
        FT_BYTES, BASE_NONE, NULL, 0,
        "UE_Positioning_Accuracy", HFILL }},
    { &hf_rrc_verticalAccuracy,
      { "verticalAccuracy", "rrc.verticalAccuracy",
        FT_BYTES, BASE_NONE, NULL, 0,
        "UE_Positioning_Accuracy", HFILL }},
    { &hf_rrc_additionalAssistanceDataReq,
      { "additionalAssistanceDataReq", "rrc.additionalAssistanceDataReq",
        FT_BOOLEAN, BASE_NONE, NULL, 0,
        "BOOLEAN", HFILL }},
    { &hf_rrc_velocityRequested,
      { "velocityRequested", "rrc.velocityRequested",
        FT_UINT32, BASE_DEC, VALS(rrc_T_velocityRequested_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_gANSSPositioningMethods,
      { "gANSSPositioningMethods", "rrc.gANSSPositioningMethods",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_16", HFILL }},
    { &hf_rrc_gANSSTimingOfCellWanted,
      { "gANSSTimingOfCellWanted", "rrc.gANSSTimingOfCellWanted",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_8", HFILL }},
    { &hf_rrc_gANSSCarrierPhaseMeasurementRequested,
      { "gANSSCarrierPhaseMeasurementRequested", "rrc.gANSSCarrierPhaseMeasurementRequested",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_8", HFILL }},
    { &hf_rrc_velocityRequested_01,
      { "velocityRequested", "rrc.velocityRequested",
        FT_UINT32, BASE_DEC, VALS(rrc_T_velocityRequested_01_vals), 0,
        "T_velocityRequested_01", HFILL }},
    { &hf_rrc_gANSSMultiFreqMeasurementRequested,
      { "gANSSMultiFreqMeasurementRequested", "rrc.gANSSMultiFreqMeasurementRequested",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_8", HFILL }},
    { &hf_rrc_utcA0,
      { "utcA0", "rrc.utcA0",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_16", HFILL }},
    { &hf_rrc_utcA1,
      { "utcA1", "rrc.utcA1",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_13", HFILL }},
    { &hf_rrc_utcA2,
      { "utcA2", "rrc.utcA2",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_7", HFILL }},
    { &hf_rrc_utcDeltaTls,
      { "utcDeltaTls", "rrc.utcDeltaTls",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_8", HFILL }},
    { &hf_rrc_utcTot,
      { "utcTot", "rrc.utcTot",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_16", HFILL }},
    { &hf_rrc_utcWNot,
      { "utcWNot", "rrc.utcWNot",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_13", HFILL }},
    { &hf_rrc_utcWNlsf,
      { "utcWNlsf", "rrc.utcWNlsf",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_8", HFILL }},
    { &hf_rrc_utcDN,
      { "utcDN", "rrc.utcDN",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_4", HFILL }},
    { &hf_rrc_utcDeltaTlsf,
      { "utcDeltaTlsf", "rrc.utcDeltaTlsf",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_8", HFILL }},
    { &hf_rrc_nA,
      { "nA", "rrc.nA",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_11", HFILL }},
    { &hf_rrc_tauC,
      { "tauC", "rrc.tauC",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_32", HFILL }},
    { &hf_rrc_deltaUT1_01,
      { "deltaUT1", "rrc.deltaUT1",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_kp,
      { "kp", "rrc.kp",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_2", HFILL }},
    { &hf_rrc_utcA1wnt,
      { "utcA1wnt", "rrc.utcA1wnt",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_24", HFILL }},
    { &hf_rrc_utcA0wnt,
      { "utcA0wnt", "rrc.utcA0wnt",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_32", HFILL }},
    { &hf_rrc_utcTot_01,
      { "utcTot", "rrc.utcTot",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_8", HFILL }},
    { &hf_rrc_utcWNt,
      { "utcWNt", "rrc.utcWNt",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_8", HFILL }},
    { &hf_rrc_utcDN_01,
      { "utcDN", "rrc.utcDN",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_8", HFILL }},
    { &hf_rrc_utcStandardID,
      { "utcStandardID", "rrc.utcStandardID",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_3", HFILL }},
    { &hf_rrc_ue_GANSSTimingOfCellFrames,
      { "ue-GANSSTimingOfCellFrames", "rrc.ue_GANSSTimingOfCellFrames",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_ms_Part,
      { "ms-Part", "rrc.ms_Part",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_80", HFILL }},
    { &hf_rrc_ls_Part,
      { "ls-Part", "rrc.ls_Part",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_4294967295", HFILL }},
    { &hf_rrc_gANSS_TimeId,
      { "gANSS-TimeId", "rrc.gANSS_TimeId",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_7", HFILL }},
    { &hf_rrc_gANSS_TimeUncertainty,
      { "gANSS-TimeUncertainty", "rrc.gANSS_TimeUncertainty",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_127", HFILL }},
    { &hf_rrc_mode_02,
      { "mode", "rrc.mode",
        FT_UINT32, BASE_DEC, VALS(rrc_T_mode_02_vals), 0,
        "T_mode_02", HFILL }},
    { &hf_rrc_fdd_187,
      { "fdd", "rrc.fdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_fdd_181", HFILL }},
    { &hf_rrc_tdd_138,
      { "tdd", "rrc.tdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_tdd_130", HFILL }},
    { &hf_rrc_UTRAN_FDD_FrequencyList_item,
      { "UTRAN-FDD-Frequency", "rrc.UTRAN_FDD_Frequency",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_qQualMinFDD,
      { "qQualMinFDD", "rrc.qQualMinFDD",
        FT_INT32, BASE_DEC, NULL, 0,
        "INTEGER_M24_0", HFILL }},
    { &hf_rrc_qRxLevMinFDD,
      { "qRxLevMinFDD", "rrc.qRxLevMinFDD",
        FT_INT32, BASE_DEC, NULL, 0,
        "INTEGER_M60_M13", HFILL }},
    { &hf_rrc_UTRAN_TDD_FrequencyList_item,
      { "UTRAN-TDD-Frequency", "rrc.UTRAN_TDD_Frequency",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_qRxLevMinTDD,
      { "qRxLevMinTDD", "rrc.qRxLevMinTDD",
        FT_INT32, BASE_DEC, NULL, 0,
        "INTEGER_M60_M13", HFILL }},
    { &hf_rrc_utran_GPSTimingOfCell,
      { "utran-GPSTimingOfCell", "rrc.utran_GPSTimingOfCell",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_ms_part,
      { "ms-part", "rrc.ms_part",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_1023", HFILL }},
    { &hf_rrc_ls_part,
      { "ls-part", "rrc.ls_part",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_4294967295", HFILL }},
    { &hf_rrc_modeSpecificInfo_147,
      { "modeSpecificInfo", "rrc.modeSpecificInfo",
        FT_UINT32, BASE_DEC, VALS(rrc_T_modeSpecificInfo_147_vals), 0,
        "T_modeSpecificInfo_147", HFILL }},
    { &hf_rrc_fdd_188,
      { "fdd", "rrc.fdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_fdd_182", HFILL }},
    { &hf_rrc_tdd_139,
      { "tdd", "rrc.tdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_tdd_131", HFILL }},
    { &hf_rrc_ue_GPSTimingOfCell,
      { "ue-GPSTimingOfCell", "rrc.ue_GPSTimingOfCell",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_ms_part_01,
      { "ms-part", "rrc.ms_part",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_16383", HFILL }},
    { &hf_rrc_modeSpecificInfo_148,
      { "modeSpecificInfo", "rrc.modeSpecificInfo",
        FT_UINT32, BASE_DEC, VALS(rrc_T_modeSpecificInfo_148_vals), 0,
        "T_modeSpecificInfo_148", HFILL }},
    { &hf_rrc_fdd_189,
      { "fdd", "rrc.fdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_fdd_183", HFILL }},
    { &hf_rrc_tdd_140,
      { "tdd", "rrc.tdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_tdd_132", HFILL }},
    { &hf_rrc_utra_ServingCell,
      { "utra-ServingCell", "rrc.utra_ServingCell",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_s_PrioritySearch1,
      { "s-PrioritySearch1", "rrc.s_PrioritySearch1",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_31", HFILL }},
    { &hf_rrc_s_PrioritySearch2,
      { "s-PrioritySearch2", "rrc.s_PrioritySearch2",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_7", HFILL }},
    { &hf_rrc_threshServingLow,
      { "threshServingLow", "rrc.threshServingLow",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_31", HFILL }},
    { &hf_rrc_utran_FDD_FrequencyList,
      { "utran-FDD-FrequencyList", "rrc.utran_FDD_FrequencyList",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_utran_TDD_FrequencyList,
      { "utran-TDD-FrequencyList", "rrc.utran_TDD_FrequencyList",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_threshServingLow2,
      { "threshServingLow2", "rrc.threshServingLow2",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_31", HFILL }},
    { &hf_rrc_horizontalWithVerticalVelocity,
      { "horizontalWithVerticalVelocity", "rrc.horizontalWithVerticalVelocity",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_horizontalVelocityWithUncertainty,
      { "horizontalVelocityWithUncertainty", "rrc.horizontalVelocityWithUncertainty",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_horizontalWithVerticalVelocityAndUncertainty,
      { "horizontalWithVerticalVelocityAndUncertainty", "rrc.horizontalWithVerticalVelocityAndUncertainty",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_bcch_ModificationTime,
      { "bcch-ModificationTime", "rrc.bcch_ModificationTime",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_ncc,
      { "ncc", "rrc.ncc",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_bcc,
      { "bcc", "rrc.bcc",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_ctch_AllocationPeriod,
      { "ctch-AllocationPeriod", "rrc.ctch_AllocationPeriod",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_1_256", HFILL }},
    { &hf_rrc_cbs_FrameOffset,
      { "cbs-FrameOffset", "rrc.cbs_FrameOffset",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_255", HFILL }},
    { &hf_rrc_msg_Type,
      { "msg-Type", "rrc.msg_Type",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_8", HFILL }},
    { &hf_rrc_payload_02,
      { "payload", "rrc.payload",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_1_512", HFILL }},
    { &hf_rrc_CDMA2000_MessageList_item,
      { "CDMA2000-Message", "rrc.CDMA2000_Message",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_CDMA2000_UMTS_Frequency_List_item,
      { "FrequencyInfoCDMA2000", "rrc.FrequencyInfoCDMA2000",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_dataTransmFreqGranularity,
      { "dataTransmFreqGranularity", "rrc.dataTransmFreqGranularity",
        FT_UINT32, BASE_DEC, VALS(rrc_T_dataTransmFreqGranularity_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_numberOfDataTransmOcc,
      { "numberOfDataTransmOcc", "rrc.numberOfDataTransmOcc",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_1_610", HFILL }},
    { &hf_rrc_dataVolumeMontoringWindow,
      { "dataVolumeMontoringWindow", "rrc.dataVolumeMontoringWindow",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_1_120", HFILL }},
    { &hf_rrc_dataTransmFrequency,
      { "dataTransmFrequency", "rrc.dataTransmFrequency",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_dataVolumePerRB,
      { "dataVolumePerRB", "rrc.dataVolumePerRB",
        FT_UINT32, BASE_DEC, NULL, 0,
        "DataVolumePerRB_List", HFILL }},
    { &hf_rrc_dataVolume,
      { "dataVolume", "rrc.dataVolume",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_4294967295", HFILL }},
    { &hf_rrc_DataVolumePerRB_List_item,
      { "DataVolumePerRB", "rrc.DataVolumePerRB",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_modeSpecificInfo_149,
      { "modeSpecificInfo", "rrc.modeSpecificInfo",
        FT_UINT32, BASE_DEC, VALS(rrc_T_modeSpecificInfo_149_vals), 0,
        "T_modeSpecificInfo_149", HFILL }},
    { &hf_rrc_fdd_190,
      { "fdd", "rrc.fdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_fdd_184", HFILL }},
    { &hf_rrc_intraFreqMeasQuantity_FDD_01,
      { "intraFreqMeasQuantity-FDD", "rrc.intraFreqMeasQuantity_FDD",
        FT_UINT32, BASE_DEC, VALS(rrc_IntraFreqMeasQuantity_FDD_sib3_vals), 0,
        "IntraFreqMeasQuantity_FDD_sib3", HFILL }},
    { &hf_rrc_tdd_141,
      { "tdd", "rrc.tdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_tdd_133", HFILL }},
    { &hf_rrc_intraFreqMeasQuantity_TDDList_01,
      { "intraFreqMeasQuantity-TDDList", "rrc.intraFreqMeasQuantity_TDDList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "IntraFreqMeasQuantity_TDD_sib3List", HFILL }},
    { &hf_rrc_warningType,
      { "warningType", "rrc.warningType",
        FT_BYTES, BASE_NONE, NULL, 0,
        "OCTET_STRING_SIZE_1_2", HFILL }},
    { &hf_rrc_messageIdentifier,
      { "messageIdentifier", "rrc.messageIdentifier",
        FT_BYTES, BASE_NONE, NULL, 0,
        "OCTET_STRING_SIZE_2", HFILL }},
    { &hf_rrc_serialNumber,
      { "serialNumber", "rrc.serialNumber",
        FT_BYTES, BASE_NONE, NULL, 0,
        "OCTET_STRING_SIZE_2", HFILL }},
    { &hf_rrc_EUTRA_BlacklistedCellPerFreqList_item,
      { "EUTRA-BlacklistedCell", "rrc.EUTRA_BlacklistedCell",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_ue_EUTRA_Capability,
      { "ue-EUTRA-Capability", "rrc.ue_EUTRA_Capability",
        FT_BYTES, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_EUTRA_TargetFreqInfoList_item,
      { "EUTRA-TargetFreqInfo", "rrc.EUTRA_TargetFreqInfo",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_dlEUTRACarrierFreq,
      { "dlEUTRACarrierFreq", "rrc.dlEUTRACarrierFreq",
        FT_UINT32, BASE_DEC, NULL, 0,
        "EARFCN", HFILL }},
    { &hf_rrc_eutraBlacklistedCellPerFreqList,
      { "eutraBlacklistedCellPerFreqList", "rrc.eutraBlacklistedCellPerFreqList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "EUTRA_BlacklistedCellPerFreqList", HFILL }},
    { &hf_rrc_extensionGANSS_SIBType,
      { "extensionGANSS-SIBType", "rrc.extensionGANSS_SIBType",
        FT_UINT32, BASE_DEC, VALS(rrc_SIB_TypeExtGANSS_vals), 0,
        "SIB_TypeExtGANSS", HFILL }},
    { &hf_rrc_schedulingInfo,
      { "schedulingInfo", "rrc.schedulingInfo",
        FT_NONE, BASE_NONE, NULL, 0,
        "SchedulingInformation", HFILL }},
    { &hf_rrc_valueTagInfo,
      { "valueTagInfo", "rrc.valueTagInfo",
        FT_UINT32, BASE_DEC, VALS(rrc_ValueTagInfo_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_ExtGANSS_SIBTypeInfoSchedulingInfoList_item,
      { "ExtGANSS-SIBTypeInfoSchedulingInfo", "rrc.ExtGANSS_SIBTypeInfoSchedulingInfo",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_ganssID,
      { "ganssID", "rrc.ganssID",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_7", HFILL }},
    { &hf_rrc_sbasID,
      { "sbasID", "rrc.sbasID",
        FT_UINT32, BASE_DEC, VALS(rrc_UE_Positioning_GANSS_SBAS_ID_vals), 0,
        "UE_Positioning_GANSS_SBAS_ID", HFILL }},
    { &hf_rrc_ganssScheduling,
      { "ganssScheduling", "rrc.ganssScheduling",
        FT_UINT32, BASE_DEC, NULL, 0,
        "SEQUENCE_SIZE_1_maxSIB_OF_ExtGANSS_SchedulingInfo", HFILL }},
    { &hf_rrc_ganssScheduling_item,
      { "ExtGANSS-SchedulingInfo", "rrc.ExtGANSS_SchedulingInfo",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_ExtSIBTypeInfoSchedulingInfo_List_item,
      { "ExtSIBTypeInfoSchedulingInfo", "rrc.ExtSIBTypeInfoSchedulingInfo",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_extensionSIB_Type,
      { "extensionSIB-Type", "rrc.extensionSIB_Type",
        FT_UINT32, BASE_DEC, VALS(rrc_SIB_TypeExt_vals), 0,
        "SIB_TypeExt", HFILL }},
    { &hf_rrc_ExtSIBTypeInfoSchedulingInfo_List2_item,
      { "ExtSIBTypeInfoSchedulingInfo2", "rrc.ExtSIBTypeInfoSchedulingInfo2",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_extensionSIB_Type2,
      { "extensionSIB-Type2", "rrc.extensionSIB_Type2",
        FT_UINT32, BASE_DEC, VALS(rrc_SIB_TypeExt2_vals), 0,
        "SIB_TypeExt2", HFILL }},
    { &hf_rrc_FDD_UMTS_Frequency_List_item,
      { "FrequencyInfoFDD", "rrc.FrequencyInfoFDD",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_band_Class,
      { "band-Class", "rrc.band_Class",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_5", HFILL }},
    { &hf_rrc_cdma_Freq,
      { "cdma-Freq", "rrc.cdma_Freq",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_11", HFILL }},
    { &hf_rrc_GERAN_SystemInformation_item,
      { "GERAN-SystemInfoBlock", "rrc.GERAN_SystemInfoBlock",
        FT_BYTES, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_gsmLowRangeUARFCN,
      { "gsmLowRangeUARFCN", "rrc.gsmLowRangeUARFCN",
        FT_UINT32, BASE_DEC, NULL, 0,
        "UARFCN", HFILL }},
    { &hf_rrc_gsmUpRangeUARFCN,
      { "gsmUpRangeUARFCN", "rrc.gsmUpRangeUARFCN",
        FT_UINT32, BASE_DEC, NULL, 0,
        "UARFCN", HFILL }},
    { &hf_rrc_GSM_BA_Range_List_item,
      { "GSM-BA-Range", "rrc.GSM_BA_Range",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_GSM_MessageList_item,
      { "GSM-MessageList item", "rrc.GSM_MessageList_item",
        FT_BYTES, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_GSM_TargetCellInfoList_item,
      { "GSM-TargetCellInfo", "rrc.GSM_TargetCellInfo",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_receivedMessageType,
      { "receivedMessageType", "rrc.receivedMessageType",
        FT_UINT32, BASE_DEC|BASE_EXT_STRING, &rrc_ReceivedMessageType_vals_ext, 0,
        NULL, HFILL }},
    { &hf_rrc_configurationUnacceptable,
      { "configurationUnacceptable", "rrc.configurationUnacceptable",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_unspecified,
      { "unspecified", "rrc.unspecified",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_GERANIu_MessageList_item,
      { "GERANIu-MessageList item", "rrc.GERANIu_MessageList_item",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_1_32768", HFILL }},
    { &hf_rrc_gsm_17,
      { "gsm", "rrc.gsm",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_gsm_15", HFILL }},
    { &hf_rrc_gsm_Classmark2,
      { "gsm-Classmark2", "rrc.gsm_Classmark2",
        FT_BYTES, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_gsm_Classmark3,
      { "gsm-Classmark3", "rrc.gsm_Classmark3",
        FT_BYTES, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_cdma2000_01,
      { "cdma2000", "rrc.cdma2000",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_cdma2000_01", HFILL }},
    { &hf_rrc_InterRAT_UE_RadioAccessCapabilityList_item,
      { "InterRAT-UE-RadioAccessCapability", "rrc.InterRAT_UE_RadioAccessCapability",
        FT_UINT32, BASE_DEC, VALS(rrc_InterRAT_UE_RadioAccessCapability_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_geranIu_RadioAccessCapability,
      { "geranIu-RadioAccessCapability", "rrc.geranIu_RadioAccessCapability",
        FT_BYTES, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_supportOfInter_RAT_PS_Handover,
      { "supportOfInter-RAT-PS-Handover", "rrc.supportOfInter_RAT_PS_Handover",
        FT_UINT32, BASE_DEC, VALS(rrc_T_supportOfInter_RAT_PS_Handover_vals), 0,
        "T_supportOfInter_RAT_PS_Handover", HFILL }},
    { &hf_rrc_gsm_MS_RadioAccessCapability,
      { "gsm-MS-RadioAccessCapability", "rrc.gsm_MS_RadioAccessCapability",
        FT_BYTES, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_eutra_RadioAccessCapability,
      { "eutra-RadioAccessCapability", "rrc.eutra_RadioAccessCapability",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_gsm_18,
      { "gsm", "rrc.gsm",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_gsm_16", HFILL }},
    { &hf_rrc_gsmSecurityCapability,
      { "gsmSecurityCapability", "rrc.gsmSecurityCapability",
        FT_BYTES, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_InterRAT_UE_SecurityCapList_item,
      { "InterRAT-UE-SecurityCapability", "rrc.InterRAT_UE_SecurityCapability",
        FT_UINT32, BASE_DEC, VALS(rrc_InterRAT_UE_SecurityCapability_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_interRAT_ProtocolError,
      { "interRAT-ProtocolError", "rrc.interRAT_ProtocolError",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_plmn_Type,
      { "plmn-Type", "rrc.plmn_Type",
        FT_UINT32, BASE_DEC, VALS(rrc_PLMN_Type_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_sibSb_ReferenceList,
      { "sibSb-ReferenceList", "rrc.sibSb_ReferenceList",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_v690NonCriticalExtensions_48,
      { "v690NonCriticalExtensions", "rrc.v690NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v690NonCriticalExtensions_48", HFILL }},
    { &hf_rrc_masterInformationBlock_v690ext,
      { "masterInformationBlock-v690ext", "rrc.masterInformationBlock_v690ext",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_v6b0NonCriticalExtensions_16,
      { "v6b0NonCriticalExtensions", "rrc.v6b0NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v6b0NonCriticalExtensions_15", HFILL }},
    { &hf_rrc_masterInformationBlock_v6b0ext,
      { "masterInformationBlock-v6b0ext", "rrc.masterInformationBlock_v6b0ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "MasterInformationBlock_v6b0ext_IEs", HFILL }},
    { &hf_rrc_v860NonCriticalExtensions_26,
      { "v860NonCriticalExtensions", "rrc.v860NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v860NonCriticalExtensions_26", HFILL }},
    { &hf_rrc_masterInformationBlock_v860ext,
      { "masterInformationBlock-v860ext", "rrc.masterInformationBlock_v860ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "MasterInformationBlock_v860ext_IEs", HFILL }},
    { &hf_rrc_nonCriticalExtensions_195,
      { "nonCriticalExtensions", "rrc.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_nonCriticalExtensions_195", HFILL }},
    { &hf_rrc_multiplePLMN_List,
      { "multiplePLMN-List", "rrc.multiplePLMN_List",
        FT_NONE, BASE_NONE, NULL, 0,
        "MultiplePLMN_List_r6", HFILL }},
    { &hf_rrc_extSIBTypeInfoSchedulingInfo_List,
      { "extSIBTypeInfoSchedulingInfo-List", "rrc.extSIBTypeInfoSchedulingInfo_List",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_extSIBTypeInfoSchedulingInfo_List_01,
      { "extSIBTypeInfoSchedulingInfo-List", "rrc.extSIBTypeInfoSchedulingInfo_List",
        FT_UINT32, BASE_DEC, NULL, 0,
        "ExtSIBTypeInfoSchedulingInfo_List2", HFILL }},
    { &hf_rrc_extGANSS_SIBTypeInfoSchedulingInfoList,
      { "extGANSS-SIBTypeInfoSchedulingInfoList", "rrc.extGANSS_SIBTypeInfoSchedulingInfoList",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_csg_Indicator,
      { "csg-Indicator", "rrc.csg_Indicator",
        FT_UINT32, BASE_DEC, VALS(rrc_T_csg_Indicator_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_predefinedConfigValueTag,
      { "predefinedConfigValueTag", "rrc.predefinedConfigValueTag",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_diagnosticsType_01,
      { "diagnosticsType", "rrc.diagnosticsType",
        FT_UINT32, BASE_DEC, VALS(rrc_T_diagnosticsType_01_vals), 0,
        "T_diagnosticsType_01", HFILL }},
    { &hf_rrc_type1_08,
      { "type1", "rrc.type1",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_type1_07", HFILL }},
    { &hf_rrc_protocolErrorCause,
      { "protocolErrorCause", "rrc.protocolErrorCause",
        FT_UINT32, BASE_DEC, VALS(rrc_ProtocolErrorCause_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_gsm_BA_Range_List,
      { "gsm-BA-Range-List", "rrc.gsm_BA_Range_List",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_fdd_UMTS_Frequency_List,
      { "fdd-UMTS-Frequency-List", "rrc.fdd_UMTS_Frequency_List",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_tdd_UMTS_Frequency_List,
      { "tdd-UMTS-Frequency-List", "rrc.tdd_UMTS_Frequency_List",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_cdma2000_UMTS_Frequency_List,
      { "cdma2000-UMTS-Frequency-List", "rrc.cdma2000_UMTS_Frequency_List",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_tdd384_UMTS_Frequency_List,
      { "tdd384-UMTS-Frequency-List", "rrc.tdd384_UMTS_Frequency_List",
        FT_UINT32, BASE_DEC, NULL, 0,
        "TDD_UMTS_Frequency_List", HFILL }},
    { &hf_rrc_tdd128_UMTS_Frequency_List,
      { "tdd128-UMTS-Frequency-List", "rrc.tdd128_UMTS_Frequency_List",
        FT_UINT32, BASE_DEC, NULL, 0,
        "TDD_UMTS_Frequency_List", HFILL }},
    { &hf_rrc_scheduling,
      { "scheduling", "rrc.scheduling",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_segCount,
      { "segCount", "rrc.segCount",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_sib_Pos,
      { "sib-Pos", "rrc.sib_Pos",
        FT_UINT32, BASE_DEC, VALS(rrc_T_sib_Pos_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_rep4,
      { "rep4", "rrc.rep4",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_1", HFILL }},
    { &hf_rrc_rep8,
      { "rep8", "rrc.rep8",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_3", HFILL }},
    { &hf_rrc_rep16,
      { "rep16", "rrc.rep16",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_7", HFILL }},
    { &hf_rrc_rep32,
      { "rep32", "rrc.rep32",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_15", HFILL }},
    { &hf_rrc_rep64,
      { "rep64", "rrc.rep64",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_31", HFILL }},
    { &hf_rrc_rep128,
      { "rep128", "rrc.rep128",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_63", HFILL }},
    { &hf_rrc_rep256,
      { "rep256", "rrc.rep256",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_127", HFILL }},
    { &hf_rrc_rep512,
      { "rep512", "rrc.rep512",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_255", HFILL }},
    { &hf_rrc_rep1024,
      { "rep1024", "rrc.rep1024",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_511", HFILL }},
    { &hf_rrc_rep2048,
      { "rep2048", "rrc.rep2048",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_1023", HFILL }},
    { &hf_rrc_rep4096,
      { "rep4096", "rrc.rep4096",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_2047", HFILL }},
    { &hf_rrc_sib_PosOffsetInfo,
      { "sib-PosOffsetInfo", "rrc.sib_PosOffsetInfo",
        FT_UINT32, BASE_DEC, NULL, 0,
        "SibOFF_List", HFILL }},
    { &hf_rrc_sib_Type_01,
      { "sib-Type", "rrc.sib_Type",
        FT_UINT32, BASE_DEC|BASE_EXT_STRING, &rrc_SIB_TypeAndTag_vals_ext, 0,
        "SIB_TypeAndTag", HFILL }},
    { &hf_rrc_scheduling_01,
      { "scheduling", "rrc.scheduling",
        FT_NONE, BASE_NONE, NULL, 0,
        "SchedulingInformation", HFILL }},
    { &hf_rrc_sibSb_Type,
      { "sibSb-Type", "rrc.sibSb_Type",
        FT_UINT32, BASE_DEC|BASE_EXT_STRING, &rrc_SIBSb_TypeAndTag_vals_ext, 0,
        "SIBSb_TypeAndTag", HFILL }},
    { &hf_rrc_sibOccurIdentity,
      { "sibOccurIdentity", "rrc.sibOccurIdentity",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_sibOccurValueTag,
      { "sibOccurValueTag", "rrc.sibOccurValueTag",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_SIB_ReferenceList_item,
      { "SchedulingInformationSIB", "rrc.SchedulingInformationSIB",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_SIBSb_ReferenceList_item,
      { "SchedulingInformationSIBSb", "rrc.SchedulingInformationSIBSb",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_SIB_ReferenceListFACH_item,
      { "SchedulingInformationSIB", "rrc.SchedulingInformationSIB",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_sysInfoType1_01,
      { "sysInfoType1", "rrc.sysInfoType1",
        FT_UINT32, BASE_DEC, NULL, 0,
        "PLMN_ValueTag", HFILL }},
    { &hf_rrc_sysInfoType2,
      { "sysInfoType2", "rrc.sysInfoType2",
        FT_UINT32, BASE_DEC, NULL, 0,
        "CellValueTag", HFILL }},
    { &hf_rrc_sysInfoType3_01,
      { "sysInfoType3", "rrc.sysInfoType3",
        FT_UINT32, BASE_DEC, NULL, 0,
        "CellValueTag", HFILL }},
    { &hf_rrc_sysInfoType4,
      { "sysInfoType4", "rrc.sysInfoType4",
        FT_UINT32, BASE_DEC, NULL, 0,
        "CellValueTag", HFILL }},
    { &hf_rrc_sysInfoType5_01,
      { "sysInfoType5", "rrc.sysInfoType5",
        FT_UINT32, BASE_DEC, NULL, 0,
        "CellValueTag", HFILL }},
    { &hf_rrc_sysInfoType6,
      { "sysInfoType6", "rrc.sysInfoType6",
        FT_UINT32, BASE_DEC, NULL, 0,
        "CellValueTag", HFILL }},
    { &hf_rrc_sysInfoType7_01,
      { "sysInfoType7", "rrc.sysInfoType7",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_dummy_55,
      { "dummy", "rrc.dummy",
        FT_UINT32, BASE_DEC, NULL, 0,
        "CellValueTag", HFILL }},
    { &hf_rrc_dummy2_14,
      { "dummy2", "rrc.dummy2",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_dummy3_03,
      { "dummy3", "rrc.dummy3",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_sysInfoType11_01,
      { "sysInfoType11", "rrc.sysInfoType11",
        FT_UINT32, BASE_DEC, NULL, 0,
        "CellValueTag", HFILL }},
    { &hf_rrc_sysInfoType12_01,
      { "sysInfoType12", "rrc.sysInfoType12",
        FT_UINT32, BASE_DEC, NULL, 0,
        "CellValueTag", HFILL }},
    { &hf_rrc_sysInfoType13,
      { "sysInfoType13", "rrc.sysInfoType13",
        FT_UINT32, BASE_DEC, NULL, 0,
        "CellValueTag", HFILL }},
    { &hf_rrc_sysInfoType13_1,
      { "sysInfoType13-1", "rrc.sysInfoType13_1",
        FT_UINT32, BASE_DEC, NULL, 0,
        "CellValueTag", HFILL }},
    { &hf_rrc_sysInfoType13_2,
      { "sysInfoType13-2", "rrc.sysInfoType13_2",
        FT_UINT32, BASE_DEC, NULL, 0,
        "CellValueTag", HFILL }},
    { &hf_rrc_sysInfoType13_3,
      { "sysInfoType13-3", "rrc.sysInfoType13_3",
        FT_UINT32, BASE_DEC, NULL, 0,
        "CellValueTag", HFILL }},
    { &hf_rrc_sysInfoType13_4,
      { "sysInfoType13-4", "rrc.sysInfoType13_4",
        FT_UINT32, BASE_DEC, NULL, 0,
        "CellValueTag", HFILL }},
    { &hf_rrc_sysInfoType14,
      { "sysInfoType14", "rrc.sysInfoType14",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_sysInfoType15,
      { "sysInfoType15", "rrc.sysInfoType15",
        FT_UINT32, BASE_DEC, NULL, 0,
        "CellValueTag", HFILL }},
    { &hf_rrc_sysInfoType16,
      { "sysInfoType16", "rrc.sysInfoType16",
        FT_NONE, BASE_NONE, NULL, 0,
        "PredefinedConfigIdentityAndValueTag", HFILL }},
    { &hf_rrc_sysInfoType17,
      { "sysInfoType17", "rrc.sysInfoType17",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_sysInfoType15_1,
      { "sysInfoType15-1", "rrc.sysInfoType15_1",
        FT_UINT32, BASE_DEC, NULL, 0,
        "CellValueTag", HFILL }},
    { &hf_rrc_sysInfoType15_2,
      { "sysInfoType15-2", "rrc.sysInfoType15_2",
        FT_NONE, BASE_NONE, NULL, 0,
        "SIBOccurrenceIdentityAndValueTag", HFILL }},
    { &hf_rrc_sysInfoType15_3,
      { "sysInfoType15-3", "rrc.sysInfoType15_3",
        FT_NONE, BASE_NONE, NULL, 0,
        "SIBOccurrenceIdentityAndValueTag", HFILL }},
    { &hf_rrc_sysInfoType15_4,
      { "sysInfoType15-4", "rrc.sysInfoType15_4",
        FT_UINT32, BASE_DEC, NULL, 0,
        "CellValueTag", HFILL }},
    { &hf_rrc_sysInfoType18,
      { "sysInfoType18", "rrc.sysInfoType18",
        FT_UINT32, BASE_DEC, NULL, 0,
        "CellValueTag", HFILL }},
    { &hf_rrc_sysInfoType15_5,
      { "sysInfoType15-5", "rrc.sysInfoType15_5",
        FT_UINT32, BASE_DEC, NULL, 0,
        "CellValueTag", HFILL }},
    { &hf_rrc_sysInfoType5bis,
      { "sysInfoType5bis", "rrc.sysInfoType5bis",
        FT_UINT32, BASE_DEC, NULL, 0,
        "CellValueTag", HFILL }},
    { &hf_rrc_sysInfoTypeSB1_01,
      { "sysInfoTypeSB1", "rrc.sysInfoTypeSB1",
        FT_UINT32, BASE_DEC, NULL, 0,
        "CellValueTag", HFILL }},
    { &hf_rrc_sysInfoTypeSB2_01,
      { "sysInfoTypeSB2", "rrc.sysInfoTypeSB2",
        FT_UINT32, BASE_DEC, NULL, 0,
        "CellValueTag", HFILL }},
    { &hf_rrc_systemInfoType11bis,
      { "systemInfoType11bis", "rrc.systemInfoType11bis",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_systemInfoType15bis,
      { "systemInfoType15bis", "rrc.systemInfoType15bis",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_systemInfoType15_1bis,
      { "systemInfoType15-1bis", "rrc.systemInfoType15_1bis",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_systemInfoType15_2bis,
      { "systemInfoType15-2bis", "rrc.systemInfoType15_2bis",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_systemInfoType15_3bis,
      { "systemInfoType15-3bis", "rrc.systemInfoType15_3bis",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_systemInfoType15_6,
      { "systemInfoType15-6", "rrc.systemInfoType15_6",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_systemInfoType15_7,
      { "systemInfoType15-7", "rrc.systemInfoType15_7",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_systemInfoType15_8,
      { "systemInfoType15-8", "rrc.systemInfoType15_8",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_systemInfoType19,
      { "systemInfoType19", "rrc.systemInfoType19",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_systemInfoType15_2ter,
      { "systemInfoType15-2ter", "rrc.systemInfoType15_2ter",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_systemInfoType20,
      { "systemInfoType20", "rrc.systemInfoType20",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_systemInfoType21,
      { "systemInfoType21", "rrc.systemInfoType21",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_systemInfoType22,
      { "systemInfoType22", "rrc.systemInfoType22",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_SibOFF_List_item,
      { "SibOFF", "rrc.SibOFF",
        FT_UINT32, BASE_DEC, VALS(rrc_SibOFF_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_cn_CommonGSM_MAP_NAS_SysInfo_03,
      { "cn-CommonGSM-MAP-NAS-SysInfo", "rrc.cn_CommonGSM_MAP_NAS_SysInfo",
        FT_BYTES, BASE_NONE, NULL, 0,
        "T_cn_CommonGSM_MAP_NAS_SysInfo_03", HFILL }},
    { &hf_rrc_cn_DomainSysInfoList,
      { "cn-DomainSysInfoList", "rrc.cn_DomainSysInfoList",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_ue_IdleTimersAndConstants,
      { "ue-IdleTimersAndConstants", "rrc.ue_IdleTimersAndConstants",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_v3a0NonCriticalExtensions_12,
      { "v3a0NonCriticalExtensions", "rrc.v3a0NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v3a0NonCriticalExtensions_12", HFILL }},
    { &hf_rrc_sysInfoType1_v3a0ext,
      { "sysInfoType1-v3a0ext", "rrc.sysInfoType1_v3a0ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "SysInfoType1_v3a0ext_IEs", HFILL }},
    { &hf_rrc_v860NonCriticalExtentions_02,
      { "v860NonCriticalExtentions", "rrc.v860NonCriticalExtentions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v860NonCriticalExtentions_02", HFILL }},
    { &hf_rrc_sysInfoType1_v860ext,
      { "sysInfoType1-v860ext", "rrc.sysInfoType1_v860ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "SysInfoType1_v860ext_IEs", HFILL }},
    { &hf_rrc_nonCriticalExtensions_196,
      { "nonCriticalExtensions", "rrc.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_nonCriticalExtensions_196", HFILL }},
    { &hf_rrc_ue_IdleTimersAndConstants_v3a0ext,
      { "ue-IdleTimersAndConstants-v3a0ext", "rrc.ue_IdleTimersAndConstants_v3a0ext",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_ura_IdentityList,
      { "ura-IdentityList", "rrc.ura_IdentityList",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_nonCriticalExtensions_197,
      { "nonCriticalExtensions", "rrc.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_nonCriticalExtensions_197", HFILL }},
    { &hf_rrc_sib4indicator,
      { "sib4indicator", "rrc.sib4indicator",
        FT_BOOLEAN, BASE_NONE, NULL, 0,
        "BOOLEAN", HFILL }},
    { &hf_rrc_cellSelectReselectInfo,
      { "cellSelectReselectInfo", "rrc.cellSelectReselectInfo",
        FT_NONE, BASE_NONE, NULL, 0,
        "CellSelectReselectInfoSIB_3_4", HFILL }},
    { &hf_rrc_cellAccessRestriction,
      { "cellAccessRestriction", "rrc.cellAccessRestriction",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_v4b0NonCriticalExtensions_17,
      { "v4b0NonCriticalExtensions", "rrc.v4b0NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v4b0NonCriticalExtensions_17", HFILL }},
    { &hf_rrc_sysInfoType3_v4b0ext,
      { "sysInfoType3-v4b0ext", "rrc.sysInfoType3_v4b0ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "SysInfoType3_v4b0ext_IEs", HFILL }},
    { &hf_rrc_v590NonCriticalExtension,
      { "v590NonCriticalExtension", "rrc.v590NonCriticalExtension",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_sysInfoType3_v590ext,
      { "sysInfoType3-v590ext", "rrc.sysInfoType3_v590ext",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_v5c0NoncriticalExtension,
      { "v5c0NoncriticalExtension", "rrc.v5c0NoncriticalExtension",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_sysInfoType3_v5c0ext,
      { "sysInfoType3-v5c0ext", "rrc.sysInfoType3_v5c0ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "SysInfoType3_v5c0ext_IEs", HFILL }},
    { &hf_rrc_v670NonCriticalExtension,
      { "v670NonCriticalExtension", "rrc.v670NonCriticalExtension",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_sysInfoType3_v670ext,
      { "sysInfoType3-v670ext", "rrc.sysInfoType3_v670ext",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_v770NonCriticalExtension_02,
      { "v770NonCriticalExtension", "rrc.v770NonCriticalExtension",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v770NonCriticalExtension_02", HFILL }},
    { &hf_rrc_sysInfoType3_v770ext,
      { "sysInfoType3-v770ext", "rrc.sysInfoType3_v770ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "SysInfoType3_v770ext_IEs", HFILL }},
    { &hf_rrc_v830NonCriticalExtension,
      { "v830NonCriticalExtension", "rrc.v830NonCriticalExtension",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_sysInfoType3_v830ext,
      { "sysInfoType3-v830ext", "rrc.sysInfoType3_v830ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "SysInfoType3_v830ext_IEs", HFILL }},
    { &hf_rrc_v860NonCriticalExtension_01,
      { "v860NonCriticalExtension", "rrc.v860NonCriticalExtension",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v860NonCriticalExtension_01", HFILL }},
    { &hf_rrc_sysInfoType3_v860ext,
      { "sysInfoType3-v860ext", "rrc.sysInfoType3_v860ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "SysInfoType3_v860ext_IEs", HFILL }},
    { &hf_rrc_v870NonCriticalExtension,
      { "v870NonCriticalExtension", "rrc.v870NonCriticalExtension",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_sysInfoType3_v870ext,
      { "sysInfoType3-v870ext", "rrc.sysInfoType3_v870ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "SysInfoType3_v870ext_IEs", HFILL }},
    { &hf_rrc_v900NonCriticalExtension,
      { "v900NonCriticalExtension", "rrc.v900NonCriticalExtension",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_sysInfoType3_v920ext,
      { "sysInfoType3-v920ext", "rrc.sysInfoType3_v920ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "SysInfoType3_v920ext_IEs", HFILL }},
    { &hf_rrc_nonCriticalExtensions_198,
      { "nonCriticalExtensions", "rrc.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_nonCriticalExtensions_198", HFILL }},
    { &hf_rrc_mapping_LCR,
      { "mapping-LCR", "rrc.mapping_LCR",
        FT_NONE, BASE_NONE, NULL, 0,
        "Mapping_LCR_r4", HFILL }},
    { &hf_rrc_cellSelectReselectInfo_v590ext,
      { "cellSelectReselectInfo-v590ext", "rrc.cellSelectReselectInfo_v590ext",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_cellSelectReselectInfoTreselectionScaling_v5c0ext,
      { "cellSelectReselectInfoTreselectionScaling-v5c0ext", "rrc.cellSelectReselectInfoTreselectionScaling_v5c0ext",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_domainSpecificAccessRestrictionParametersForPLMNOfMIB,
      { "domainSpecificAccessRestrictionParametersForPLMNOfMIB", "rrc.domainSpecificAccessRestrictionParametersForPLMNOfMIB",
        FT_NONE, BASE_NONE, NULL, 0,
        "DomainSpecificAccessRestrictionParam_v670ext", HFILL }},
    { &hf_rrc_domainSpecificAccessRestictionForSharedNetwork,
      { "domainSpecificAccessRestictionForSharedNetwork", "rrc.domainSpecificAccessRestictionForSharedNetwork",
        FT_UINT32, BASE_DEC, VALS(rrc_DomainSpecificAccessRestrictionForSharedNetwork_v670ext_vals), 0,
        "DomainSpecificAccessRestrictionForSharedNetwork_v670ext", HFILL }},
    { &hf_rrc_deferredMeasurementControlReadingSupport,
      { "deferredMeasurementControlReadingSupport", "rrc.deferredMeasurementControlReadingSupport",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_q_QualMin_Offset,
      { "q-QualMin-Offset", "rrc.q_QualMin_Offset",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_q_RxlevMin_Offset,
      { "q-RxlevMin-Offset", "rrc.q_RxlevMin_Offset",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_mbsfnOnlyService,
      { "mbsfnOnlyService", "rrc.mbsfnOnlyService",
        FT_UINT32, BASE_DEC, VALS(rrc_MBSFNOnlyService_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_pagingPermissionWithAccessControlParametersForPLMNOfMIB,
      { "pagingPermissionWithAccessControlParametersForPLMNOfMIB", "rrc.pagingPermissionWithAccessControlParametersForPLMNOfMIB",
        FT_NONE, BASE_NONE, NULL, 0,
        "PagingPermissionWithAccessControlParameters", HFILL }},
    { &hf_rrc_pagingPermissionWithAccessControlParametersForSharedNetwork,
      { "pagingPermissionWithAccessControlParametersForSharedNetwork", "rrc.pagingPermissionWithAccessControlParametersForSharedNetwork",
        FT_UINT32, BASE_DEC, VALS(rrc_PagingPermissionWithAccessControlForSharedNetwork_vals), 0,
        "PagingPermissionWithAccessControlForSharedNetwork", HFILL }},
    { &hf_rrc_csg_PSCSplitInfo,
      { "csg-PSCSplitInfo", "rrc.csg_PSCSplitInfo",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_cellAccessRestriction_01,
      { "cellAccessRestriction", "rrc.cellAccessRestriction",
        FT_NONE, BASE_NONE, NULL, 0,
        "CellAccessRestriction_v870ext", HFILL }},
    { &hf_rrc_imsEmergencySupportIndicator,
      { "imsEmergencySupportIndicator", "rrc.imsEmergencySupportIndicator",
        FT_UINT32, BASE_DEC, VALS(rrc_T_imsEmergencySupportIndicator_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_v4b0NonCriticalExtensions_18,
      { "v4b0NonCriticalExtensions", "rrc.v4b0NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v4b0NonCriticalExtensions_18", HFILL }},
    { &hf_rrc_sysInfoType4_v4b0ext,
      { "sysInfoType4-v4b0ext", "rrc.sysInfoType4_v4b0ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "SysInfoType4_v4b0ext_IEs", HFILL }},
    { &hf_rrc_v590NonCriticalExtension_01,
      { "v590NonCriticalExtension", "rrc.v590NonCriticalExtension",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v590NonCriticalExtension_01", HFILL }},
    { &hf_rrc_sysInfoType4_v590ext,
      { "sysInfoType4-v590ext", "rrc.sysInfoType4_v590ext",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_v5b0NonCriticalExtension,
      { "v5b0NonCriticalExtension", "rrc.v5b0NonCriticalExtension",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_sysInfoType4_v5b0ext,
      { "sysInfoType4-v5b0ext", "rrc.sysInfoType4_v5b0ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "SysInfoType4_v5b0ext_IEs", HFILL }},
    { &hf_rrc_v5c0NonCriticalExtension,
      { "v5c0NonCriticalExtension", "rrc.v5c0NonCriticalExtension",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_sysInfoType4_v5c0ext,
      { "sysInfoType4-v5c0ext", "rrc.sysInfoType4_v5c0ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "SysInfoType4_v5c0ext_IEs", HFILL }},
    { &hf_rrc_nonCriticalExtensions_199,
      { "nonCriticalExtensions", "rrc.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_nonCriticalExtensions_199", HFILL }},
    { &hf_rrc_cellSelectReselectInfoPCHFACH_v5b0ext,
      { "cellSelectReselectInfoPCHFACH-v5b0ext", "rrc.cellSelectReselectInfoPCHFACH_v5b0ext",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_sib6indicator,
      { "sib6indicator", "rrc.sib6indicator",
        FT_BOOLEAN, BASE_NONE, NULL, 0,
        "BOOLEAN", HFILL }},
    { &hf_rrc_pich_PowerOffset,
      { "pich-PowerOffset", "rrc.pich_PowerOffset",
        FT_INT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_modeSpecificInfo_150,
      { "modeSpecificInfo", "rrc.modeSpecificInfo",
        FT_UINT32, BASE_DEC, VALS(rrc_T_modeSpecificInfo_150_vals), 0,
        "T_modeSpecificInfo_150", HFILL }},
    { &hf_rrc_fdd_191,
      { "fdd", "rrc.fdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_fdd_185", HFILL }},
    { &hf_rrc_aich_PowerOffset,
      { "aich-PowerOffset", "rrc.aich_PowerOffset",
        FT_INT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_tdd_142,
      { "tdd", "rrc.tdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_tdd_134", HFILL }},
    { &hf_rrc_pusch_SysInfoList_SFN,
      { "pusch-SysInfoList-SFN", "rrc.pusch_SysInfoList_SFN",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_pdsch_SysInfoList_SFN,
      { "pdsch-SysInfoList-SFN", "rrc.pdsch_SysInfoList_SFN",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_openLoopPowerControl_TDD,
      { "openLoopPowerControl-TDD", "rrc.openLoopPowerControl_TDD",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_prach_SystemInformationList,
      { "prach-SystemInformationList", "rrc.prach_SystemInformationList",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_sCCPCH_SystemInformationList,
      { "sCCPCH-SystemInformationList", "rrc.sCCPCH_SystemInformationList",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_cbs_DRX_Level1Information,
      { "cbs-DRX-Level1Information", "rrc.cbs_DRX_Level1Information",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_v4b0NonCriticalExtensions_19,
      { "v4b0NonCriticalExtensions", "rrc.v4b0NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v4b0NonCriticalExtensions_19", HFILL }},
    { &hf_rrc_sysInfoType5_v4b0ext,
      { "sysInfoType5-v4b0ext", "rrc.sysInfoType5_v4b0ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "SysInfoType5_v4b0ext_IEs", HFILL }},
    { &hf_rrc_v590NonCriticalExtensions_28,
      { "v590NonCriticalExtensions", "rrc.v590NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v590NonCriticalExtensions_28", HFILL }},
    { &hf_rrc_sysInfoType5_v590ext,
      { "sysInfoType5-v590ext", "rrc.sysInfoType5_v590ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "SysInfoType5_v590ext_IEs", HFILL }},
    { &hf_rrc_v650NonCriticalExtensions,
      { "v650NonCriticalExtensions", "rrc.v650NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_sysInfoType5_v650ext,
      { "sysInfoType5-v650ext", "rrc.sysInfoType5_v650ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "SysInfoType5_v650ext_IEs", HFILL }},
    { &hf_rrc_v680NonCriticalExtensions_02,
      { "v680NonCriticalExtensions", "rrc.v680NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v680NonCriticalExtensions_02", HFILL }},
    { &hf_rrc_sysInfoType5_v680ext,
      { "sysInfoType5-v680ext", "rrc.sysInfoType5_v680ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "SysInfoType5_v680ext_IEs", HFILL }},
    { &hf_rrc_v690NonCriticalExtensions_49,
      { "v690NonCriticalExtensions", "rrc.v690NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v690NonCriticalExtensions_49", HFILL }},
    { &hf_rrc_sysInfoType5_v690ext,
      { "sysInfoType5-v690ext", "rrc.sysInfoType5_v690ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "SysInfoType5_v690ext_IEs", HFILL }},
    { &hf_rrc_v770NonCriticalExtensions_40,
      { "v770NonCriticalExtensions", "rrc.v770NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v770NonCriticalExtensions_40", HFILL }},
    { &hf_rrc_sysInfoType5_v770ext,
      { "sysInfoType5-v770ext", "rrc.sysInfoType5_v770ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "SysInfoType5_v770ext_IEs", HFILL }},
    { &hf_rrc_v860NonCriticalExtensions_27,
      { "v860NonCriticalExtensions", "rrc.v860NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v860NonCriticalExtensions_27", HFILL }},
    { &hf_rrc_sysInfoType5_v860ext,
      { "sysInfoType5-v860ext", "rrc.sysInfoType5_v860ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "SysInfoType5_v860ext_IEs", HFILL }},
    { &hf_rrc_v890NonCriticalExtensions_11,
      { "v890NonCriticalExtensions", "rrc.v890NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v890NonCriticalExtensions_11", HFILL }},
    { &hf_rrc_sysInfoType5_v890ext,
      { "sysInfoType5-v890ext", "rrc.sysInfoType5_v890ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "SysInfoType5_v890ext_IEs", HFILL }},
    { &hf_rrc_v8b0NonCriticalExtensions_01,
      { "v8b0NonCriticalExtensions", "rrc.v8b0NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v8b0NonCriticalExtensions_01", HFILL }},
    { &hf_rrc_sysInfoType5_v8b0ext,
      { "sysInfoType5-v8b0ext", "rrc.sysInfoType5_v8b0ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "SysInfoType5_v8b0ext_IEs", HFILL }},
    { &hf_rrc_v8d0NonCriticalExtensions,
      { "v8d0NonCriticalExtensions", "rrc.v8d0NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_sysInfoType5_v8d0ext,
      { "sysInfoType5-v8d0ext", "rrc.sysInfoType5_v8d0ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "SysInfoType5_v8d0ext_IEs", HFILL }},
    { &hf_rrc_va40NonCriticalExtensions_19,
      { "va40NonCriticalExtensions", "rrc.va40NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_va40NonCriticalExtensions_18", HFILL }},
    { &hf_rrc_sysInfoType5_va40ext,
      { "sysInfoType5-va40ext", "rrc.sysInfoType5_va40ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "SysInfoType5_va40ext_IEs", HFILL }},
    { &hf_rrc_va80NonCriticalExtensions_02,
      { "va80NonCriticalExtensions", "rrc.va80NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_va80NonCriticalExtensions_02", HFILL }},
    { &hf_rrc_sysInfoType5_va80ext,
      { "sysInfoType5-va80ext", "rrc.sysInfoType5_va80ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "SysInfoType5_va80ext_IEs", HFILL }},
    { &hf_rrc_vbxyNonCriticalExtensions_19,
      { "vbxyNonCriticalExtensions", "rrc.vbxyNonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "SysInfoType5_NonCriticalExtensions_vbxy_IEs", HFILL }},
    { &hf_rrc_pNBSCH_Allocation_r4,
      { "pNBSCH-Allocation-r4", "rrc.pNBSCH_Allocation_r4",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_prach_SystemInformationList_LCR_r4,
      { "prach-SystemInformationList-LCR-r4", "rrc.prach_SystemInformationList_LCR_r4",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_tdd128SpecificInfo,
      { "tdd128SpecificInfo", "rrc.tdd128SpecificInfo",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_pusch_SysInfoList_SFN_01,
      { "pusch-SysInfoList-SFN", "rrc.pusch_SysInfoList_SFN",
        FT_UINT32, BASE_DEC, NULL, 0,
        "PUSCH_SysInfoList_SFN_LCR_r4", HFILL }},
    { &hf_rrc_pdsch_SysInfoList_SFN_01,
      { "pdsch-SysInfoList-SFN", "rrc.pdsch_SysInfoList_SFN",
        FT_UINT32, BASE_DEC, NULL, 0,
        "PDSCH_SysInfoList_SFN_LCR_r4", HFILL }},
    { &hf_rrc_pCCPCH_LCR_Extensions,
      { "pCCPCH-LCR-Extensions", "rrc.pCCPCH_LCR_Extensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "PrimaryCCPCH_Info_LCR_r4_ext", HFILL }},
    { &hf_rrc_sCCPCH_LCR_ExtensionsList,
      { "sCCPCH-LCR-ExtensionsList", "rrc.sCCPCH_LCR_ExtensionsList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "SCCPCH_SystemInformationList_LCR_r4_ext", HFILL }},
    { &hf_rrc_frequencyBandIndicator,
      { "frequencyBandIndicator", "rrc.frequencyBandIndicator",
        FT_UINT32, BASE_DEC, VALS(rrc_RadioFrequencyBandFDD_vals), 0,
        "RadioFrequencyBandFDD", HFILL }},
    { &hf_rrc_hcr_r5_SpecificInfo,
      { "hcr-r5-SpecificInfo", "rrc.hcr_r5_SpecificInfo",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_pusch_SysInfoList_SFN_02,
      { "pusch-SysInfoList-SFN", "rrc.pusch_SysInfoList_SFN",
        FT_UINT32, BASE_DEC, NULL, 0,
        "PUSCH_SysInfoList_SFN_HCR_r5", HFILL }},
    { &hf_rrc_pdsch_SysInfoList_SFN_02,
      { "pdsch-SysInfoList-SFN", "rrc.pdsch_SysInfoList_SFN",
        FT_UINT32, BASE_DEC, NULL, 0,
        "PDSCH_SysInfoList_SFN_HCR_r5", HFILL }},
    { &hf_rrc_frequencyBandIndicator2,
      { "frequencyBandIndicator2", "rrc.frequencyBandIndicator2",
        FT_UINT32, BASE_DEC, VALS(rrc_RadioFrequencyBandFDD2_vals), 0,
        "RadioFrequencyBandFDD2", HFILL }},
    { &hf_rrc_hsdpa_CellIndicator,
      { "hsdpa-CellIndicator", "rrc.hsdpa_CellIndicator",
        FT_UINT32, BASE_DEC, VALS(rrc_T_hsdpa_CellIndicator_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_edch_CellIndicator,
      { "edch-CellIndicator", "rrc.edch_CellIndicator",
        FT_UINT32, BASE_DEC, VALS(rrc_T_edch_CellIndicator_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_sccpch_SystemInformation_MBMS,
      { "sccpch-SystemInformation-MBMS", "rrc.sccpch_SystemInformation_MBMS",
        FT_UINT32, BASE_DEC, VALS(rrc_T_sccpch_SystemInformation_MBMS_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_mcchOnSCCPCHusedForNonMBMS,
      { "mcchOnSCCPCHusedForNonMBMS", "rrc.mcchOnSCCPCHusedForNonMBMS",
        FT_NONE, BASE_NONE, NULL, 0,
        "MBMS_MCCH_ConfigurationInfo_r6", HFILL }},
    { &hf_rrc_mcchOnSCCPCHusedOnlyForMBMS,
      { "mcchOnSCCPCHusedOnlyForMBMS", "rrc.mcchOnSCCPCHusedOnlyForMBMS",
        FT_NONE, BASE_NONE, NULL, 0,
        "SCCPCH_SystemInformation_MBMS_r6", HFILL }},
    { &hf_rrc_additionalPRACH_TF_and_TFCS_CCCH_List,
      { "additionalPRACH-TF-and-TFCS-CCCH-List", "rrc.additionalPRACH_TF_and_TFCS_CCCH_List",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_cBS_DRX_Level1Information_extension,
      { "cBS-DRX-Level1Information-extension", "rrc.cBS_DRX_Level1Information_extension",
        FT_UINT32, BASE_DEC, VALS(rrc_CBS_DRX_Level1Information_extension_r6_vals), 0,
        "CBS_DRX_Level1Information_extension_r6", HFILL }},
    { &hf_rrc_modeSpecificInfo_151,
      { "modeSpecificInfo", "rrc.modeSpecificInfo",
        FT_UINT32, BASE_DEC, VALS(rrc_T_modeSpecificInfo_151_vals), 0,
        "T_modeSpecificInfo_151", HFILL }},
    { &hf_rrc_fdd_192,
      { "fdd", "rrc.fdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_fdd_186", HFILL }},
    { &hf_rrc_hs_dsch_CommonSystemInformation,
      { "hs-dsch-CommonSystemInformation", "rrc.hs_dsch_CommonSystemInformation",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_hs_dsch_PagingSystemInformation,
      { "hs-dsch-PagingSystemInformation", "rrc.hs_dsch_PagingSystemInformation",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_tdd768_27,
      { "tdd768", "rrc.tdd768",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_tdd768_24", HFILL }},
    { &hf_rrc_pusch_SysInfoList_SFN_03,
      { "pusch-SysInfoList-SFN", "rrc.pusch_SysInfoList_SFN",
        FT_UINT32, BASE_DEC, NULL, 0,
        "PUSCH_SysInfoList_SFN_VHCR", HFILL }},
    { &hf_rrc_pdsch_SysInfoList_SFN_03,
      { "pdsch-SysInfoList-SFN", "rrc.pdsch_SysInfoList_SFN",
        FT_UINT32, BASE_DEC, NULL, 0,
        "PDSCH_SysInfoList_VHCR_r7", HFILL }},
    { &hf_rrc_prach_SystemInformationList_01,
      { "prach-SystemInformationList", "rrc.prach_SystemInformationList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "PRACH_SystemInformationList_VHCR_r7", HFILL }},
    { &hf_rrc_tdd128_71,
      { "tdd128", "rrc.tdd128",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_tdd128_68", HFILL }},
    { &hf_rrc_prach_SystemInformationList_02,
      { "prach-SystemInformationList", "rrc.prach_SystemInformationList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "PRACH_SystemInformationList_LCR_v770ext", HFILL }},
    { &hf_rrc_sccpch_SystemInformationList,
      { "sccpch-SystemInformationList", "rrc.sccpch_SystemInformationList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "SCCPCH_SystemInformationList_HCR_VHCR_r7", HFILL }},
    { &hf_rrc_sccpch_SystemInformation_MBMS_01,
      { "sccpch-SystemInformation-MBMS", "rrc.sccpch_SystemInformation_MBMS",
        FT_UINT32, BASE_DEC, VALS(rrc_T_sccpch_SystemInformation_MBMS_01_vals), 0,
        "T_sccpch_SystemInformation_MBMS_01", HFILL }},
    { &hf_rrc_mcchOnSCCPCHusedOnlyForMBMS_01,
      { "mcchOnSCCPCHusedOnlyForMBMS", "rrc.mcchOnSCCPCHusedOnlyForMBMS",
        FT_NONE, BASE_NONE, NULL, 0,
        "SCCPCH_SystemInformation_MBMS_r7", HFILL }},
    { &hf_rrc_tDD_MBSFNInformation,
      { "tDD-MBSFNInformation", "rrc.tDD_MBSFNInformation",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_dummy_56,
      { "dummy", "rrc.dummy",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_dummy_15", HFILL }},
    { &hf_rrc_modeSpecificInfo_152,
      { "modeSpecificInfo", "rrc.modeSpecificInfo",
        FT_UINT32, BASE_DEC, VALS(rrc_T_modeSpecificInfo_152_vals), 0,
        "T_modeSpecificInfo_152", HFILL }},
    { &hf_rrc_fdd_193,
      { "fdd", "rrc.fdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_fdd_187", HFILL }},
    { &hf_rrc_hs_dsch_DrxCellfach_info,
      { "hs-dsch-DrxCellfach-info", "rrc.hs_dsch_DrxCellfach_info",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_tdd128_72,
      { "tdd128", "rrc.tdd128",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_tdd128_69", HFILL }},
    { &hf_rrc_hs_dsch_CommonSysInfo,
      { "hs-dsch-CommonSysInfo", "rrc.hs_dsch_CommonSysInfo",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_hs_dsch_CommonSystemInformation_01,
      { "hs-dsch-CommonSystemInformation", "rrc.hs_dsch_CommonSystemInformation",
        FT_NONE, BASE_NONE, NULL, 0,
        "HS_DSCH_CommonSystemInformation_TDD128", HFILL }},
    { &hf_rrc_hs_dsch_PagingSystemInformation_01,
      { "hs-dsch-PagingSystemInformation", "rrc.hs_dsch_PagingSystemInformation",
        FT_NONE, BASE_NONE, NULL, 0,
        "HS_DSCH_PagingSystemInformation_TDD128", HFILL }},
    { &hf_rrc_hs_dsch_DrxCellfach_info_01,
      { "hs-dsch-DrxCellfach-info", "rrc.hs_dsch_DrxCellfach_info",
        FT_NONE, BASE_NONE, NULL, 0,
        "HS_DSCH_DrxCellfach_info_TDD128", HFILL }},
    { &hf_rrc_commonEDCHSystemInfo,
      { "commonEDCHSystemInfo", "rrc.commonEDCHSystemInfo",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_tresetUsageIndicator,
      { "tresetUsageIndicator", "rrc.tresetUsageIndicator",
        FT_UINT32, BASE_DEC, VALS(rrc_T_tresetUsageIndicator_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_commonEDCHSystemInfoFDD,
      { "commonEDCHSystemInfoFDD", "rrc.commonEDCHSystemInfoFDD",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_frequencyBandsIndicatorSupport,
      { "frequencyBandsIndicatorSupport", "rrc.frequencyBandsIndicatorSupport",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_frequencyBandIndicator3,
      { "frequencyBandIndicator3", "rrc.frequencyBandIndicator3",
        FT_UINT32, BASE_DEC|BASE_EXT_STRING, &rrc_RadioFrequencyBandFDD3_vals_ext, 0,
        "RadioFrequencyBandFDD3", HFILL }},
    { &hf_rrc_sysInfoType5_vbxyext,
      { "sysInfoType5-vbxyext", "rrc.sysInfoType5_vbxyext",
        FT_NONE, BASE_NONE, NULL, 0,
        "SysInfoType5_vbxyext_IEs", HFILL }},
    { &hf_rrc_nonCriticalExtensions_200,
      { "nonCriticalExtensions", "rrc.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_nonCriticalExtensions_200", HFILL }},
    { &hf_rrc_sib22indicator,
      { "sib22indicator", "rrc.sib22indicator",
        FT_BOOLEAN, BASE_NONE, NULL, 0,
        "BOOLEAN", HFILL }},
    { &hf_rrc_modeSpecificInfo_153,
      { "modeSpecificInfo", "rrc.modeSpecificInfo",
        FT_UINT32, BASE_DEC, VALS(rrc_T_modeSpecificInfo_153_vals), 0,
        "T_modeSpecificInfo_153", HFILL }},
    { &hf_rrc_fdd_194,
      { "fdd", "rrc.fdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_fdd_188", HFILL }},
    { &hf_rrc_dummy_57,
      { "dummy", "rrc.dummy",
        FT_INT32, BASE_DEC, NULL, 0,
        "CSICH_PowerOffset", HFILL }},
    { &hf_rrc_tdd_143,
      { "tdd", "rrc.tdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_tdd_135", HFILL }},
    { &hf_rrc_v4b0NonCriticalExtensions_20,
      { "v4b0NonCriticalExtensions", "rrc.v4b0NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v4b0NonCriticalExtensions_20", HFILL }},
    { &hf_rrc_sysInfoType6_v4b0ext,
      { "sysInfoType6-v4b0ext", "rrc.sysInfoType6_v4b0ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "SysInfoType6_v4b0ext_IEs", HFILL }},
    { &hf_rrc_v590NonCriticalExtensions_29,
      { "v590NonCriticalExtensions", "rrc.v590NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v590NonCriticalExtensions_29", HFILL }},
    { &hf_rrc_sysInfoType6_v590ext,
      { "sysInfoType6-v590ext", "rrc.sysInfoType6_v590ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "SysInfoType6_v590ext_IEs", HFILL }},
    { &hf_rrc_v650nonCriticalExtensions,
      { "v650nonCriticalExtensions", "rrc.v650nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_sysInfoType6_v650ext,
      { "sysInfoType6-v650ext", "rrc.sysInfoType6_v650ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "SysInfoType6_v650ext_IEs", HFILL }},
    { &hf_rrc_v690nonCriticalExtensions,
      { "v690nonCriticalExtensions", "rrc.v690nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_sysInfoType6_v690ext,
      { "sysInfoType6-v690ext", "rrc.sysInfoType6_v690ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "SysInfoType6_v690ext_IEs", HFILL }},
    { &hf_rrc_v770NonCriticalExtensions_41,
      { "v770NonCriticalExtensions", "rrc.v770NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v770NonCriticalExtensions_41", HFILL }},
    { &hf_rrc_sysInfoType6_v770ext,
      { "sysInfoType6-v770ext", "rrc.sysInfoType6_v770ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "SysInfoType6_v770ext_IEs", HFILL }},
    { &hf_rrc_va40NonCriticalExtensions_20,
      { "va40NonCriticalExtensions", "rrc.va40NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_va40NonCriticalExtensions_19", HFILL }},
    { &hf_rrc_sysInfoType6_va40ext,
      { "sysInfoType6-va40ext", "rrc.sysInfoType6_va40ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "SysInfoType6_va40ext_IEs", HFILL }},
    { &hf_rrc_va80NonCriticalExtensions_03,
      { "va80NonCriticalExtensions", "rrc.va80NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_va80NonCriticalExtensions_03", HFILL }},
    { &hf_rrc_sysInfoType6_va80ext,
      { "sysInfoType6-va80ext", "rrc.sysInfoType6_va80ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "SysInfoType6_va80ext_IEs", HFILL }},
    { &hf_rrc_nonCriticalExtensions_201,
      { "nonCriticalExtensions", "rrc.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_nonCriticalExtensions_201", HFILL }},
    { &hf_rrc_tdd128SpecificInfo_01,
      { "tdd128SpecificInfo", "rrc.tdd128SpecificInfo",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_tdd128SpecificInfo_01", HFILL }},
    { &hf_rrc_hcr_r5_SpecificInfo_01,
      { "hcr-r5-SpecificInfo", "rrc.hcr_r5_SpecificInfo",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_hcr_r5_SpecificInfo_01", HFILL }},
    { &hf_rrc_tdd768SpecificInfo,
      { "tdd768SpecificInfo", "rrc.tdd768SpecificInfo",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_modeSpecificInfo_154,
      { "modeSpecificInfo", "rrc.modeSpecificInfo",
        FT_UINT32, BASE_DEC, VALS(rrc_T_modeSpecificInfo_154_vals), 0,
        "T_modeSpecificInfo_154", HFILL }},
    { &hf_rrc_fdd_195,
      { "fdd", "rrc.fdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_fdd_189", HFILL }},
    { &hf_rrc_ul_Interference,
      { "ul-Interference", "rrc.ul_Interference",
        FT_INT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_prach_Information_SIB5_List,
      { "prach-Information-SIB5-List", "rrc.prach_Information_SIB5_List",
        FT_UINT32, BASE_DEC, NULL, 0,
        "DynamicPersistenceLevelList", HFILL }},
    { &hf_rrc_prach_Information_SIB6_List,
      { "prach-Information-SIB6-List", "rrc.prach_Information_SIB6_List",
        FT_UINT32, BASE_DEC, NULL, 0,
        "DynamicPersistenceLevelList", HFILL }},
    { &hf_rrc_expirationTimeFactor,
      { "expirationTimeFactor", "rrc.expirationTimeFactor",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_nonCriticalExtensions_202,
      { "nonCriticalExtensions", "rrc.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_nonCriticalExtensions_202", HFILL }},
    { &hf_rrc_dummy1_07,
      { "dummy1", "rrc.dummy1",
        FT_NONE, BASE_NONE, NULL, 0,
        "CPCH_Parameters", HFILL }},
    { &hf_rrc_dummy2_15,
      { "dummy2", "rrc.dummy2",
        FT_UINT32, BASE_DEC, NULL, 0,
        "CPCH_SetInfoList", HFILL }},
    { &hf_rrc_dummy3_04,
      { "dummy3", "rrc.dummy3",
        FT_INT32, BASE_DEC, NULL, 0,
        "CSICH_PowerOffset", HFILL }},
    { &hf_rrc_nonCriticalExtensions_203,
      { "nonCriticalExtensions", "rrc.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_nonCriticalExtensions_203", HFILL }},
    { &hf_rrc_dummy_58,
      { "dummy", "rrc.dummy",
        FT_UINT32, BASE_DEC, NULL, 0,
        "CPCH_PersistenceLevelsList", HFILL }},
    { &hf_rrc_nonCriticalExtensions_204,
      { "nonCriticalExtensions", "rrc.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_nonCriticalExtensions_204", HFILL }},
    { &hf_rrc_dummy_59,
      { "dummy", "rrc.dummy",
        FT_UINT32, BASE_DEC, NULL, 0,
        "DRAC_SysInfoList", HFILL }},
    { &hf_rrc_nonCriticalExtensions_205,
      { "nonCriticalExtensions", "rrc.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_nonCriticalExtensions_205", HFILL }},
    { &hf_rrc_sib12indicator,
      { "sib12indicator", "rrc.sib12indicator",
        FT_BOOLEAN, BASE_NONE, NULL, 0,
        "BOOLEAN", HFILL }},
    { &hf_rrc_fach_MeasurementOccasionInfo,
      { "fach-MeasurementOccasionInfo", "rrc.fach_MeasurementOccasionInfo",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_measurementControlSysInfo,
      { "measurementControlSysInfo", "rrc.measurementControlSysInfo",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_v4b0NonCriticalExtensions_21,
      { "v4b0NonCriticalExtensions", "rrc.v4b0NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v4b0NonCriticalExtensions_21", HFILL }},
    { &hf_rrc_sysInfoType11_v4b0ext,
      { "sysInfoType11-v4b0ext", "rrc.sysInfoType11_v4b0ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "SysInfoType11_v4b0ext_IEs", HFILL }},
    { &hf_rrc_v590NonCriticalExtension_02,
      { "v590NonCriticalExtension", "rrc.v590NonCriticalExtension",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v590NonCriticalExtension_02", HFILL }},
    { &hf_rrc_sysInfoType11_v590ext,
      { "sysInfoType11-v590ext", "rrc.sysInfoType11_v590ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "SysInfoType11_v590ext_IEs", HFILL }},
    { &hf_rrc_v690NonCriticalExtensions_50,
      { "v690NonCriticalExtensions", "rrc.v690NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v690NonCriticalExtensions_50", HFILL }},
    { &hf_rrc_sysInfoType11_v690ext,
      { "sysInfoType11-v690ext", "rrc.sysInfoType11_v690ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "SysInfoType11_v690ext_IEs", HFILL }},
    { &hf_rrc_v6b0NonCriticalExtensions_17,
      { "v6b0NonCriticalExtensions", "rrc.v6b0NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v6b0NonCriticalExtensions_16", HFILL }},
    { &hf_rrc_sysInfoType11_v6b0ext,
      { "sysInfoType11-v6b0ext", "rrc.sysInfoType11_v6b0ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "SysInfoType11_v6b0ext_IEs", HFILL }},
    { &hf_rrc_v770NonCriticalExtensions_42,
      { "v770NonCriticalExtensions", "rrc.v770NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v770NonCriticalExtensions_42", HFILL }},
    { &hf_rrc_sysInfoType11_v770ext,
      { "sysInfoType11-v770ext", "rrc.sysInfoType11_v770ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "SysInfoType11_v770ext_IEs", HFILL }},
    { &hf_rrc_v7b0NonCriticalExtensions_02,
      { "v7b0NonCriticalExtensions", "rrc.v7b0NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v7b0NonCriticalExtensions_02", HFILL }},
    { &hf_rrc_sysInfoType11_v7b0ext,
      { "sysInfoType11-v7b0ext", "rrc.sysInfoType11_v7b0ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "SysInfoType11_v7b0ext_IEs", HFILL }},
    { &hf_rrc_v860NonCriticalExtensions_28,
      { "v860NonCriticalExtensions", "rrc.v860NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v860NonCriticalExtensions_28", HFILL }},
    { &hf_rrc_sysInfoType11_v860ext,
      { "sysInfoType11-v860ext", "rrc.sysInfoType11_v860ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "SysInfoType11_v860ext_IEs", HFILL }},
    { &hf_rrc_va80NonCriticalExtensions_04,
      { "va80NonCriticalExtensions", "rrc.va80NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_va80NonCriticalExtensions_04", HFILL }},
    { &hf_rrc_sysInfoType11_va80ext,
      { "sysInfoType11-va80ext", "rrc.sysInfoType11_va80ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "SysInfoType11_va80ext_IEs", HFILL }},
    { &hf_rrc_nonCriticalExtensions_206,
      { "nonCriticalExtensions", "rrc.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_nonCriticalExtensions_206", HFILL }},
    { &hf_rrc_fach_MeasurementOccasionInfo_LCR_Ext,
      { "fach-MeasurementOccasionInfo-LCR-Ext", "rrc.fach_MeasurementOccasionInfo_LCR_Ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "FACH_MeasurementOccasionInfo_LCR_r4_ext", HFILL }},
    { &hf_rrc_measurementControlSysInfo_LCR,
      { "measurementControlSysInfo-LCR", "rrc.measurementControlSysInfo_LCR",
        FT_NONE, BASE_NONE, NULL, 0,
        "MeasurementControlSysInfo_LCR_r4_ext", HFILL }},
    { &hf_rrc_newIntraFrequencyCellInfoList_v590ext,
      { "newIntraFrequencyCellInfoList-v590ext", "rrc.newIntraFrequencyCellInfoList_v590ext",
        FT_UINT32, BASE_DEC, NULL, 0,
        "SEQUENCE_SIZE_1_maxCellMeas_OF_CellSelectReselectInfo_v590ext", HFILL }},
    { &hf_rrc_newIntraFrequencyCellInfoList_v590ext_item,
      { "CellSelectReselectInfo-v590ext", "rrc.CellSelectReselectInfo_v590ext",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_newInterFrequencyCellInfoList_v590ext,
      { "newInterFrequencyCellInfoList-v590ext", "rrc.newInterFrequencyCellInfoList_v590ext",
        FT_UINT32, BASE_DEC, NULL, 0,
        "SEQUENCE_SIZE_1_maxCellMeas_OF_CellSelectReselectInfo_v590ext", HFILL }},
    { &hf_rrc_newInterFrequencyCellInfoList_v590ext_item,
      { "CellSelectReselectInfo-v590ext", "rrc.CellSelectReselectInfo_v590ext",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_newInterRATCellInfoList_v590ext,
      { "newInterRATCellInfoList-v590ext", "rrc.newInterRATCellInfoList_v590ext",
        FT_UINT32, BASE_DEC, NULL, 0,
        "SEQUENCE_SIZE_1_maxCellMeas_OF_CellSelectReselectInfo_v590ext", HFILL }},
    { &hf_rrc_newInterRATCellInfoList_v590ext_item,
      { "CellSelectReselectInfo-v590ext", "rrc.CellSelectReselectInfo_v590ext",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_intraFreqEventCriteriaList_v590ext,
      { "intraFreqEventCriteriaList-v590ext", "rrc.intraFreqEventCriteriaList_v590ext",
        FT_UINT32, BASE_DEC, NULL, 0,
        "Intra_FreqEventCriteriaList_v590ext", HFILL }},
    { &hf_rrc_dummy_60,
      { "dummy", "rrc.dummy",
        FT_NONE, BASE_NONE, NULL, 0,
        "Dummy_InterFreqRACHReportingInfo", HFILL }},
    { &hf_rrc_interFreqRACHReportingInfo,
      { "interFreqRACHReportingInfo", "rrc.interFreqRACHReportingInfo",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_mbsfnFrequencyList,
      { "mbsfnFrequencyList", "rrc.mbsfnFrequencyList",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_mbsfnFrequencyList_01,
      { "mbsfnFrequencyList", "rrc.mbsfnFrequencyList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "MBSFNFrequencyList_v860ext", HFILL }},
    { &hf_rrc_multipleFrequencyInfoListFDD,
      { "multipleFrequencyInfoListFDD", "rrc.multipleFrequencyInfoListFDD",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_measurementControlSysInfo_01,
      { "measurementControlSysInfo", "rrc.measurementControlSysInfo",
        FT_NONE, BASE_NONE, NULL, 0,
        "MeasurementControlSysInfoExtension", HFILL }},
    { &hf_rrc_measurementControlSysInfo_LCR_01,
      { "measurementControlSysInfo-LCR", "rrc.measurementControlSysInfo_LCR",
        FT_NONE, BASE_NONE, NULL, 0,
        "MeasurementControlSysInfoExtension_LCR_r4", HFILL }},
    { &hf_rrc_measurementControlSysInfoExtensionAddon_r5,
      { "measurementControlSysInfoExtensionAddon-r5", "rrc.measurementControlSysInfoExtensionAddon_r5",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_v7b0NonCriticalExtensions_03,
      { "v7b0NonCriticalExtensions", "rrc.v7b0NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v7b0NonCriticalExtensions_03", HFILL }},
    { &hf_rrc_sysInfoType11bis_v7b0ext,
      { "sysInfoType11bis-v7b0ext", "rrc.sysInfoType11bis_v7b0ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "SysInfoType11bis_v7b0ext_IEs", HFILL }},
    { &hf_rrc_v860NonCriticalExtensions_29,
      { "v860NonCriticalExtensions", "rrc.v860NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v860NonCriticalExtensions_29", HFILL }},
    { &hf_rrc_sysInfoType11bis_v860ext,
      { "sysInfoType11bis-v860ext", "rrc.sysInfoType11bis_v860ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "SysInfoType11bis_v860ext_IEs", HFILL }},
    { &hf_rrc_va80NonCriticalExtensions_05,
      { "va80NonCriticalExtensions", "rrc.va80NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_va80NonCriticalExtensions_05", HFILL }},
    { &hf_rrc_sysInfoType11bis_va80ext,
      { "sysInfoType11bis-va80ext", "rrc.sysInfoType11bis_va80ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "SysInfoType11bis_va80ext_IEs", HFILL }},
    { &hf_rrc_nonCriticalExtensions_207,
      { "nonCriticalExtensions", "rrc.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_nonCriticalExtensions_207", HFILL }},
    { &hf_rrc_csg_DedicatedFrequencyInfoList,
      { "csg-DedicatedFrequencyInfoList", "rrc.csg_DedicatedFrequencyInfoList",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_v4b0NonCriticalExtensions_22,
      { "v4b0NonCriticalExtensions", "rrc.v4b0NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v4b0NonCriticalExtensions_22", HFILL }},
    { &hf_rrc_sysInfoType12_v4b0ext,
      { "sysInfoType12-v4b0ext", "rrc.sysInfoType12_v4b0ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "SysInfoType12_v4b0ext_IEs", HFILL }},
    { &hf_rrc_v590NonCriticalExtension_03,
      { "v590NonCriticalExtension", "rrc.v590NonCriticalExtension",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v590NonCriticalExtension_03", HFILL }},
    { &hf_rrc_sysInfoType12_v590ext,
      { "sysInfoType12-v590ext", "rrc.sysInfoType12_v590ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "SysInfoType12_v590ext_IEs", HFILL }},
    { &hf_rrc_v690NonCriticalExtensions_51,
      { "v690NonCriticalExtensions", "rrc.v690NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v690NonCriticalExtensions_51", HFILL }},
    { &hf_rrc_sysInfoType12_v690ext,
      { "sysInfoType12-v690ext", "rrc.sysInfoType12_v690ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "SysInfoType12_v690ext_IEs", HFILL }},
    { &hf_rrc_v6b0NonCriticalExtensions_18,
      { "v6b0NonCriticalExtensions", "rrc.v6b0NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v6b0NonCriticalExtensions_17", HFILL }},
    { &hf_rrc_sysInfoType12_v6b0ext,
      { "sysInfoType12-v6b0ext", "rrc.sysInfoType12_v6b0ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "SysInfoType12_v6b0ext_IEs", HFILL }},
    { &hf_rrc_v7b0NonCriticalExtensions_04,
      { "v7b0NonCriticalExtensions", "rrc.v7b0NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v7b0NonCriticalExtensions_04", HFILL }},
    { &hf_rrc_sysInfoType12_v7b0ext,
      { "sysInfoType12-v7b0ext", "rrc.sysInfoType12_v7b0ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "SysInfoType12_v7b0ext_IEs", HFILL }},
    { &hf_rrc_va80NonCriticalExtensions_06,
      { "va80NonCriticalExtensions", "rrc.va80NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_va80NonCriticalExtensions_06", HFILL }},
    { &hf_rrc_sysInfoType12_va80ext,
      { "sysInfoType12-va80ext", "rrc.sysInfoType12_va80ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "SysInfoType12_va80ext_IEs", HFILL }},
    { &hf_rrc_nonCriticalExtensions_208,
      { "nonCriticalExtensions", "rrc.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_nonCriticalExtensions_208", HFILL }},
    { &hf_rrc_v3a0NonCriticalExtensions_13,
      { "v3a0NonCriticalExtensions", "rrc.v3a0NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v3a0NonCriticalExtensions_13", HFILL }},
    { &hf_rrc_sysInfoType13_v3a0ext,
      { "sysInfoType13-v3a0ext", "rrc.sysInfoType13_v3a0ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "SysInfoType13_v3a0ext_IEs", HFILL }},
    { &hf_rrc_v4b0NonCriticalExtensions_23,
      { "v4b0NonCriticalExtensions", "rrc.v4b0NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v4b0NonCriticalExtensions_23", HFILL }},
    { &hf_rrc_sysInfoType13_v4b0ext,
      { "sysInfoType13-v4b0ext", "rrc.sysInfoType13_v4b0ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "SysInfoType13_v4b0ext_IEs", HFILL }},
    { &hf_rrc_v770NonCriticalExtensions_43,
      { "v770NonCriticalExtensions", "rrc.v770NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v770NonCriticalExtensions_43", HFILL }},
    { &hf_rrc_sysInfoType13_v770ext,
      { "sysInfoType13-v770ext", "rrc.sysInfoType13_v770ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "SysInfoType13_v770ext_IEs", HFILL }},
    { &hf_rrc_nonCriticalExtensions_209,
      { "nonCriticalExtensions", "rrc.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_nonCriticalExtensions_209", HFILL }},
    { &hf_rrc_capabilityUpdateRequirement_r4Ext,
      { "capabilityUpdateRequirement-r4Ext", "rrc.capabilityUpdateRequirement_r4Ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "CapabilityUpdateRequirement_r4_ext", HFILL }},
    { &hf_rrc_ansi_41_RAND_Information,
      { "ansi-41-RAND-Information", "rrc.ansi_41_RAND_Information",
        FT_BYTES, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_nonCriticalExtensions_210,
      { "nonCriticalExtensions", "rrc.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_nonCriticalExtensions_210", HFILL }},
    { &hf_rrc_ansi_41_UserZoneID_Information,
      { "ansi-41-UserZoneID-Information", "rrc.ansi_41_UserZoneID_Information",
        FT_BYTES, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_nonCriticalExtensions_211,
      { "nonCriticalExtensions", "rrc.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_nonCriticalExtensions_211", HFILL }},
    { &hf_rrc_ansi_41_PrivateNeighbourListInfo,
      { "ansi-41-PrivateNeighbourListInfo", "rrc.ansi_41_PrivateNeighbourListInfo",
        FT_BYTES, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_nonCriticalExtensions_212,
      { "nonCriticalExtensions", "rrc.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_nonCriticalExtensions_212", HFILL }},
    { &hf_rrc_ansi_41_GlobalServiceRedirectInfo,
      { "ansi-41-GlobalServiceRedirectInfo", "rrc.ansi_41_GlobalServiceRedirectInfo",
        FT_BYTES, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_nonCriticalExtensions_213,
      { "nonCriticalExtensions", "rrc.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_nonCriticalExtensions_213", HFILL }},
    { &hf_rrc_nonCriticalExtensions_214,
      { "nonCriticalExtensions", "rrc.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_nonCriticalExtensions_214", HFILL }},
    { &hf_rrc_ue_positioning_GPS_CipherParameters,
      { "ue-positioning-GPS-CipherParameters", "rrc.ue_positioning_GPS_CipherParameters",
        FT_NONE, BASE_NONE, NULL, 0,
        "UE_Positioning_CipherParameters", HFILL }},
    { &hf_rrc_v4b0NonCriticalExtensions_24,
      { "v4b0NonCriticalExtensions", "rrc.v4b0NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v4b0NonCriticalExtensions_24", HFILL }},
    { &hf_rrc_sysInfoType15_v4b0ext,
      { "sysInfoType15-v4b0ext", "rrc.sysInfoType15_v4b0ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "SysInfoType15_v4b0ext_IEs", HFILL }},
    { &hf_rrc_v770NonCriticalExtensions_44,
      { "v770NonCriticalExtensions", "rrc.v770NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v770NonCriticalExtensions_44", HFILL }},
    { &hf_rrc_sysInfoType15_v770ext,
      { "sysInfoType15-v770ext", "rrc.sysInfoType15_v770ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "SysInfoType15_v770ext_IEs", HFILL }},
    { &hf_rrc_va40NonCriticalExtensions_21,
      { "va40NonCriticalExtensions", "rrc.va40NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_va40NonCriticalExtensions_20", HFILL }},
    { &hf_rrc_sysInfoType15_va40ext,
      { "sysInfoType15-va40ext", "rrc.sysInfoType15_va40ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "SysInfoType15_va40ext_IEs", HFILL }},
    { &hf_rrc_nonCriticalExtensions_215,
      { "nonCriticalExtensions", "rrc.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_nonCriticalExtensions_215", HFILL }},
    { &hf_rrc_up_Ipdl_Parameters_TDD,
      { "up-Ipdl-Parameters-TDD", "rrc.up_Ipdl_Parameters_TDD",
        FT_NONE, BASE_NONE, NULL, 0,
        "UE_Positioning_IPDL_Parameters_TDD_r4_ext", HFILL }},
    { &hf_rrc_ue_Positioning_GPS_ReferenceTime,
      { "ue-Positioning-GPS-ReferenceTime", "rrc.ue_Positioning_GPS_ReferenceTime",
        FT_NONE, BASE_NONE, NULL, 0,
        "UE_Positioning_GPS_ReferenceTime_v770ext", HFILL }},
    { &hf_rrc_ue_Positioning_GPS_ReferenceTime_01,
      { "ue-Positioning-GPS-ReferenceTime", "rrc.ue_Positioning_GPS_ReferenceTime",
        FT_NONE, BASE_NONE, NULL, 0,
        "UE_Positioning_GPS_ReferenceTime_va40ext", HFILL }},
    { &hf_rrc_ue_positioning_GANSS_ReferencePosition,
      { "ue-positioning-GANSS-ReferencePosition", "rrc.ue_positioning_GANSS_ReferencePosition",
        FT_NONE, BASE_NONE, NULL, 0,
        "ReferenceLocationGANSS", HFILL }},
    { &hf_rrc_ue_positioning_GANSS_IonosphericModel,
      { "ue-positioning-GANSS-IonosphericModel", "rrc.ue_positioning_GANSS_IonosphericModel",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_v860NonCriticalExtensions_30,
      { "v860NonCriticalExtensions", "rrc.v860NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v860NonCriticalExtensions_30", HFILL }},
    { &hf_rrc_sysInfoType15bis_v860ext,
      { "sysInfoType15bis-v860ext", "rrc.sysInfoType15bis_v860ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "SysInfoType15bis_v860ext_IEs", HFILL }},
    { &hf_rrc_va40NonCriticalExtensions_22,
      { "va40NonCriticalExtensions", "rrc.va40NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_va40NonCriticalExtensions_21", HFILL }},
    { &hf_rrc_sysInfoType15bis_va40ext,
      { "sysInfoType15bis-va40ext", "rrc.sysInfoType15bis_va40ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "SysInfoType15bis_va40ext_IEs", HFILL }},
    { &hf_rrc_nonCriticalExtensions_216,
      { "nonCriticalExtensions", "rrc.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_nonCriticalExtensions_216", HFILL }},
    { &hf_rrc_ue_Positioning_GANSS_AddIonoModel,
      { "ue-Positioning-GANSS-AddIonoModel", "rrc.ue_Positioning_GANSS_AddIonoModel",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_ue_Positioning_GANSS_EarthOrientationPara,
      { "ue-Positioning-GANSS-EarthOrientationPara", "rrc.ue_Positioning_GANSS_EarthOrientationPara",
        FT_NONE, BASE_NONE, NULL, 0,
        "UE_Positioning_GANSS_EarthOrientPara", HFILL }},
    { &hf_rrc_v920NonCriticalExtensions_04,
      { "v920NonCriticalExtensions", "rrc.v920NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v920NonCriticalExtensions_04", HFILL }},
    { &hf_rrc_sysInfoType15_1_v920ext,
      { "sysInfoType15-1-v920ext", "rrc.sysInfoType15_1_v920ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "SysInfoType15_1_v920ext_IEs", HFILL }},
    { &hf_rrc_nonCriticalExtensions_217,
      { "nonCriticalExtensions", "rrc.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_nonCriticalExtensions_217", HFILL }},
    { &hf_rrc_ue_positioning_GANSS_DGANSS_Corrections,
      { "ue-positioning-GANSS-DGANSS-Corrections", "rrc.ue_positioning_GANSS_DGANSS_Corrections",
        FT_NONE, BASE_NONE, NULL, 0,
        "UE_Positioning_DGANSSCorrections", HFILL }},
    { &hf_rrc_v920NonCriticalExtensions_05,
      { "v920NonCriticalExtensions", "rrc.v920NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v920NonCriticalExtensions_05", HFILL }},
    { &hf_rrc_sysInfoType15_1bis_v920ext,
      { "sysInfoType15-1bis-v920ext", "rrc.sysInfoType15_1bis_v920ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "SysInfoType15_1bis_v920ext_IEs", HFILL }},
    { &hf_rrc_nonCriticalExtensions_218,
      { "nonCriticalExtensions", "rrc.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_nonCriticalExtensions_218", HFILL }},
    { &hf_rrc_ue_positioning_GANSS_DGANSS_Corrections_01,
      { "ue-positioning-GANSS-DGANSS-Corrections", "rrc.ue_positioning_GANSS_DGANSS_Corrections",
        FT_NONE, BASE_NONE, NULL, 0,
        "UE_Positioning_DGANSSCorrections_v920ext", HFILL }},
    { &hf_rrc_transmissionTOW,
      { "transmissionTOW", "rrc.transmissionTOW",
        FT_UINT32, BASE_DEC, NULL, 0,
        "GPS_TOW_1sec", HFILL }},
    { &hf_rrc_nonCriticalExtensions_219,
      { "nonCriticalExtensions", "rrc.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_nonCriticalExtensions_219", HFILL }},
    { &hf_rrc_ue_positioning_GANSS_navigationModel,
      { "ue-positioning-GANSS-navigationModel", "rrc.ue_positioning_GANSS_navigationModel",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_nonCriticalExtensions_220,
      { "nonCriticalExtensions", "rrc.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_nonCriticalExtensions_220", HFILL }},
    { &hf_rrc_ue_positioning_GANSS_AddNavigationModels,
      { "ue-positioning-GANSS-AddNavigationModels", "rrc.ue_positioning_GANSS_AddNavigationModels",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_nonCriticalExtensions_221,
      { "nonCriticalExtensions", "rrc.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_nonCriticalExtensions_221", HFILL }},
    { &hf_rrc_satMask,
      { "satMask", "rrc.satMask",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_1_32", HFILL }},
    { &hf_rrc_lsbTOW,
      { "lsbTOW", "rrc.lsbTOW",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_8", HFILL }},
    { &hf_rrc_va40NonCriticalExtensions_23,
      { "va40NonCriticalExtensions", "rrc.va40NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_va40NonCriticalExtensions_22", HFILL }},
    { &hf_rrc_sysInfoType15_3_va40ext,
      { "sysInfoType15-3-va40ext", "rrc.sysInfoType15_3_va40ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "SysInfoType15_3_va40ext_IEs", HFILL }},
    { &hf_rrc_nonCriticalExtensions_222,
      { "nonCriticalExtensions", "rrc.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_nonCriticalExtensions_222", HFILL }},
    { &hf_rrc_ue_positioning_GANSS_Almanac,
      { "ue-positioning-GANSS-Almanac", "rrc.ue_positioning_GANSS_Almanac",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_ue_positioning_GANSS_TimeModels,
      { "ue-positioning-GANSS-TimeModels", "rrc.ue_positioning_GANSS_TimeModels",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_ue_positioning_GANSS_UTC_Model,
      { "ue-positioning-GANSS-UTC-Model", "rrc.ue_positioning_GANSS_UTC_Model",
        FT_NONE, BASE_NONE, NULL, 0,
        "UE_Positioning_GANSS_UTCModel", HFILL }},
    { &hf_rrc_v860NonCriticalExtensions_31,
      { "v860NonCriticalExtensions", "rrc.v860NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v860NonCriticalExtensions_31", HFILL }},
    { &hf_rrc_sysInfoType15_3bis_v860ext,
      { "sysInfoType15-3bis-v860ext", "rrc.sysInfoType15_3bis_v860ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "SysInfoType15_3bis_v860ext_IEs", HFILL }},
    { &hf_rrc_va40NonCriticalExtensions_24,
      { "va40NonCriticalExtensions", "rrc.va40NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_va40NonCriticalExtensions_23", HFILL }},
    { &hf_rrc_sysInfoType15_3bis_va40ext,
      { "sysInfoType15-3bis-va40ext", "rrc.sysInfoType15_3bis_va40ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "SysInfoType15_3bis_va40ext_IEs", HFILL }},
    { &hf_rrc_nonCriticalExtensions_223,
      { "nonCriticalExtensions", "rrc.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_nonCriticalExtensions_223", HFILL }},
    { &hf_rrc_ue_Positioning_GANSS_Almanac,
      { "ue-Positioning-GANSS-Almanac", "rrc.ue_Positioning_GANSS_Almanac",
        FT_NONE, BASE_NONE, NULL, 0,
        "UE_Positioning_GANSS_Almanac_v860ext", HFILL }},
    { &hf_rrc_ue_Positioning_GANSS_AddUTCModels,
      { "ue-Positioning-GANSS-AddUTCModels", "rrc.ue_Positioning_GANSS_AddUTCModels",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_ue_Positioning_GANSS_AuxiliaryInfo,
      { "ue-Positioning-GANSS-AuxiliaryInfo", "rrc.ue_Positioning_GANSS_AuxiliaryInfo",
        FT_UINT32, BASE_DEC, VALS(rrc_UE_Positioning_GANSS_AuxiliaryInfo_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_ue_Positioning_GANSS_Almanac_01,
      { "ue-Positioning-GANSS-Almanac", "rrc.ue_Positioning_GANSS_Almanac",
        FT_NONE, BASE_NONE, NULL, 0,
        "UE_Positioning_GANSS_Almanac_va40ext", HFILL }},
    { &hf_rrc_ue_positioning_GANSS_TimeModels_01,
      { "ue-positioning-GANSS-TimeModels", "rrc.ue_positioning_GANSS_TimeModels",
        FT_UINT32, BASE_DEC, NULL, 0,
        "UE_Positioning_GANSS_TimeModels_va40ext", HFILL }},
    { &hf_rrc_ue_positioning_OTDOA_CipherParameters,
      { "ue-positioning-OTDOA-CipherParameters", "rrc.ue_positioning_OTDOA_CipherParameters",
        FT_NONE, BASE_NONE, NULL, 0,
        "UE_Positioning_CipherParameters", HFILL }},
    { &hf_rrc_v3a0NonCriticalExtensions_14,
      { "v3a0NonCriticalExtensions", "rrc.v3a0NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v3a0NonCriticalExtensions_14", HFILL }},
    { &hf_rrc_sysInfoType15_4_v3a0ext,
      { "sysInfoType15-4-v3a0ext", "rrc.sysInfoType15_4_v3a0ext",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_v4b0NonCriticalExtensions_25,
      { "v4b0NonCriticalExtensions", "rrc.v4b0NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v4b0NonCriticalExtensions_25", HFILL }},
    { &hf_rrc_sysInfoType15_4_v4b0ext,
      { "sysInfoType15-4-v4b0ext", "rrc.sysInfoType15_4_v4b0ext",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_nonCriticalExtensions_224,
      { "nonCriticalExtensions", "rrc.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_nonCriticalExtensions_224", HFILL }},
    { &hf_rrc_v3a0NonCriticalExtensions_15,
      { "v3a0NonCriticalExtensions", "rrc.v3a0NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v3a0NonCriticalExtensions_15", HFILL }},
    { &hf_rrc_sysInfoType15_5_v3a0ext,
      { "sysInfoType15-5-v3a0ext", "rrc.sysInfoType15_5_v3a0ext",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_v770NonCriticalExtensions_45,
      { "v770NonCriticalExtensions", "rrc.v770NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v770NonCriticalExtensions_45", HFILL }},
    { &hf_rrc_sysInfoType15_5_v770ext,
      { "sysInfoType15-5-v770ext", "rrc.sysInfoType15_5_v770ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "SysInfoType15_5_v770ext_IEs", HFILL }},
    { &hf_rrc_nonCriticalExtensions_225,
      { "nonCriticalExtensions", "rrc.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_nonCriticalExtensions_225", HFILL }},
    { &hf_rrc_ue_positioning_GANSS_TOD,
      { "ue-positioning-GANSS-TOD", "rrc.ue_positioning_GANSS_TOD",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_86399", HFILL }},
    { &hf_rrc_ue_positioning_GANSS_ReferenceMeasurementInformation,
      { "ue-positioning-GANSS-ReferenceMeasurementInformation", "rrc.ue_positioning_GANSS_ReferenceMeasurementInformation",
        FT_NONE, BASE_NONE, NULL, 0,
        "UE_Positioning_GANSS_ReferenceMeasurementInfo", HFILL }},
    { &hf_rrc_va40NonCriticalExtensions_25,
      { "va40NonCriticalExtensions", "rrc.va40NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_va40NonCriticalExtensions_24", HFILL }},
    { &hf_rrc_sysInfoType15_6_va40ext,
      { "sysInfoType15-6-va40ext", "rrc.sysInfoType15_6_va40ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "SysInfoType15_6_va40ext_IEs", HFILL }},
    { &hf_rrc_nonCriticalExtensions_226,
      { "nonCriticalExtensions", "rrc.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_nonCriticalExtensions_226", HFILL }},
    { &hf_rrc_ue_positioning_GANSS_ReferenceMeasurementInformation_01,
      { "ue-positioning-GANSS-ReferenceMeasurementInformation", "rrc.ue_positioning_GANSS_ReferenceMeasurementInformation",
        FT_NONE, BASE_NONE, NULL, 0,
        "UE_Positioning_GANSS_ReferenceMeasurementInfo_va40ext", HFILL }},
    { &hf_rrc_ue_positioning_GANSS_DataBitAssistance,
      { "ue-positioning-GANSS-DataBitAssistance", "rrc.ue_positioning_GANSS_DataBitAssistance",
        FT_NONE, BASE_NONE, NULL, 0,
        "UE_Positioning_GANSS_Data_Bit_Assistance", HFILL }},
    { &hf_rrc_nonCriticalExtensions_227,
      { "nonCriticalExtensions", "rrc.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_nonCriticalExtensions_227", HFILL }},
    { &hf_rrc_ue_positioning_GANSS_DataCipheringInfo,
      { "ue-positioning-GANSS-DataCipheringInfo", "rrc.ue_positioning_GANSS_DataCipheringInfo",
        FT_NONE, BASE_NONE, NULL, 0,
        "UE_Positioning_CipherParameters", HFILL }},
    { &hf_rrc_ue_positioning_GANSS_realTimeIntegrity,
      { "ue-positioning-GANSS-realTimeIntegrity", "rrc.ue_positioning_GANSS_realTimeIntegrity",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_nonCriticalExtensions_228,
      { "nonCriticalExtensions", "rrc.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_nonCriticalExtensions_228", HFILL }},
    { &hf_rrc_preDefinedRadioConfiguration,
      { "preDefinedRadioConfiguration", "rrc.preDefinedRadioConfiguration",
        FT_NONE, BASE_NONE, NULL, 0,
        "PreDefRadioConfiguration", HFILL }},
    { &hf_rrc_v770NonCriticalExtensions_46,
      { "v770NonCriticalExtensions", "rrc.v770NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v770NonCriticalExtensions_46", HFILL }},
    { &hf_rrc_sysInfoType16_v770ext,
      { "sysInfoType16-v770ext", "rrc.sysInfoType16_v770ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "SysInfoType16_v770ext_IEs", HFILL }},
    { &hf_rrc_v920NonCriticalExtensions_06,
      { "v920NonCriticalExtensions", "rrc.v920NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v920NonCriticalExtensions_06", HFILL }},
    { &hf_rrc_sysInfoType16_v920ext,
      { "sysInfoType16-v920ext", "rrc.sysInfoType16_v920ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "SysInfoType16_v920ext_IEs", HFILL }},
    { &hf_rrc_nonCriticalExtensions_229,
      { "nonCriticalExtensions", "rrc.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_nonCriticalExtensions_229", HFILL }},
    { &hf_rrc_preDefinedRadioConfiguration_01,
      { "preDefinedRadioConfiguration", "rrc.preDefinedRadioConfiguration",
        FT_NONE, BASE_NONE, NULL, 0,
        "PreDefRadioConfiguration_v770ext", HFILL }},
    { &hf_rrc_preDefinedRadioConfiguration_02,
      { "preDefinedRadioConfiguration", "rrc.preDefinedRadioConfiguration",
        FT_NONE, BASE_NONE, NULL, 0,
        "PreDefRadioConfiguration_v920ext", HFILL }},
    { &hf_rrc_pusch_SysInfoList,
      { "pusch-SysInfoList", "rrc.pusch_SysInfoList",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_pdsch_SysInfoList,
      { "pdsch-SysInfoList", "rrc.pdsch_SysInfoList",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_v4b0NonCriticalExtensions_26,
      { "v4b0NonCriticalExtensions", "rrc.v4b0NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v4b0NonCriticalExtensions_26", HFILL }},
    { &hf_rrc_sysInfoType17_v4b0ext,
      { "sysInfoType17-v4b0ext", "rrc.sysInfoType17_v4b0ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "SysInfoType17_v4b0ext_IEs", HFILL }},
    { &hf_rrc_v590NonCriticalExtensions_30,
      { "v590NonCriticalExtensions", "rrc.v590NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v590NonCriticalExtensions_30", HFILL }},
    { &hf_rrc_sysInfoType17_v590ext,
      { "sysInfoType17-v590ext", "rrc.sysInfoType17_v590ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "SysInfoType17_v590ext_IEs", HFILL }},
    { &hf_rrc_v770NonCriticalExtensions_47,
      { "v770NonCriticalExtensions", "rrc.v770NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v770NonCriticalExtensions_47", HFILL }},
    { &hf_rrc_sysInfoType17_v770ext,
      { "sysInfoType17-v770ext", "rrc.sysInfoType17_v770ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "SysInfoType17_v770ext_IEs", HFILL }},
    { &hf_rrc_nonCriticalExtensions_230,
      { "nonCriticalExtensions", "rrc.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_nonCriticalExtensions_230", HFILL }},
    { &hf_rrc_tdd128SpecificInfo_02,
      { "tdd128SpecificInfo", "rrc.tdd128SpecificInfo",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_tdd128SpecificInfo_02", HFILL }},
    { &hf_rrc_pusch_SysInfoList_01,
      { "pusch-SysInfoList", "rrc.pusch_SysInfoList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "PUSCH_SysInfoList_LCR_r4", HFILL }},
    { &hf_rrc_pdsch_SysInfoList_01,
      { "pdsch-SysInfoList", "rrc.pdsch_SysInfoList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "PDSCH_SysInfoList_LCR_r4", HFILL }},
    { &hf_rrc_hcr_r5_SpecificInfo_02,
      { "hcr-r5-SpecificInfo", "rrc.hcr_r5_SpecificInfo",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_hcr_r5_SpecificInfo_02", HFILL }},
    { &hf_rrc_pusch_SysInfoList_02,
      { "pusch-SysInfoList", "rrc.pusch_SysInfoList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "PUSCH_SysInfoList_HCR_r5", HFILL }},
    { &hf_rrc_pdsch_SysInfoList_02,
      { "pdsch-SysInfoList", "rrc.pdsch_SysInfoList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "PDSCH_SysInfoList_HCR_r5", HFILL }},
    { &hf_rrc_tdd768SpecificInfo_01,
      { "tdd768SpecificInfo", "rrc.tdd768SpecificInfo",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_tdd768SpecificInfo_01", HFILL }},
    { &hf_rrc_idleModePLMNIdentities,
      { "idleModePLMNIdentities", "rrc.idleModePLMNIdentities",
        FT_NONE, BASE_NONE, NULL, 0,
        "PLMNIdentitiesOfNeighbourCells", HFILL }},
    { &hf_rrc_connectedModePLMNIdentities,
      { "connectedModePLMNIdentities", "rrc.connectedModePLMNIdentities",
        FT_NONE, BASE_NONE, NULL, 0,
        "PLMNIdentitiesOfNeighbourCells", HFILL }},
    { &hf_rrc_v6b0NonCriticalExtensions_19,
      { "v6b0NonCriticalExtensions", "rrc.v6b0NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v6b0NonCriticalExtensions_18", HFILL }},
    { &hf_rrc_sysInfoType18_v6b0ext,
      { "sysInfoType18-v6b0ext", "rrc.sysInfoType18_v6b0ext",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_v860NonCriticalExtensions_32,
      { "v860NonCriticalExtensions", "rrc.v860NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v860NonCriticalExtensions_32", HFILL }},
    { &hf_rrc_sysInfoType18_v860ext,
      { "sysInfoType18-v860ext", "rrc.sysInfoType18_v860ext",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_nonCriticalExtensions_231,
      { "nonCriticalExtensions", "rrc.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_nonCriticalExtensions_231", HFILL }},
    { &hf_rrc_idleModePLMNIdentitiesSIB11bis,
      { "idleModePLMNIdentitiesSIB11bis", "rrc.idleModePLMNIdentitiesSIB11bis",
        FT_NONE, BASE_NONE, NULL, 0,
        "PLMNIdentitiesOfNeighbourCells", HFILL }},
    { &hf_rrc_connectedModePLMNIdentitiesSIB11bis,
      { "connectedModePLMNIdentitiesSIB11bis", "rrc.connectedModePLMNIdentitiesSIB11bis",
        FT_NONE, BASE_NONE, NULL, 0,
        "PLMNIdentitiesOfNeighbourCells", HFILL }},
    { &hf_rrc_idleModePLMNIdentities_01,
      { "idleModePLMNIdentities", "rrc.idleModePLMNIdentities",
        FT_NONE, BASE_NONE, NULL, 0,
        "PLMNIdentitiesOfNeighbourCells_v860ext", HFILL }},
    { &hf_rrc_connectedModePLMNIdentities_01,
      { "connectedModePLMNIdentities", "rrc.connectedModePLMNIdentities",
        FT_NONE, BASE_NONE, NULL, 0,
        "PLMNIdentitiesOfNeighbourCells_v860ext", HFILL }},
    { &hf_rrc_utra_PriorityInfoList,
      { "utra-PriorityInfoList", "rrc.utra_PriorityInfoList",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_gsm_PriorityInfoList,
      { "gsm-PriorityInfoList", "rrc.gsm_PriorityInfoList",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_eutra_FrequencyAndPriorityInfoList,
      { "eutra-FrequencyAndPriorityInfoList", "rrc.eutra_FrequencyAndPriorityInfoList",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_v920NonCriticalExtensions_07,
      { "v920NonCriticalExtensions", "rrc.v920NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v920NonCriticalExtensions_07", HFILL }},
    { &hf_rrc_sysInfoType19_v920ext,
      { "sysInfoType19-v920ext", "rrc.sysInfoType19_v920ext",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_va80NonCriticalExtensions_07,
      { "va80NonCriticalExtensions", "rrc.va80NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_va80NonCriticalExtensions_07", HFILL }},
    { &hf_rrc_sysInfoType19_va80ext,
      { "sysInfoType19-va80ext", "rrc.sysInfoType19_va80ext",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_vb30NonCriticalExtensions,
      { "vb30NonCriticalExtensions", "rrc.vb30NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_sysInfoType19_vb30ext,
      { "sysInfoType19-vb30ext", "rrc.sysInfoType19_vb30ext",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_vbxyNonCriticalExtensions_20,
      { "vbxyNonCriticalExtensions", "rrc.vbxyNonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_vbxyNonCriticalExtensions_19", HFILL }},
    { &hf_rrc_sysInfoType19_vbxyext,
      { "sysInfoType19-vbxyext", "rrc.sysInfoType19_vbxyext",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_nonCriticalExtensions_232,
      { "nonCriticalExtensions", "rrc.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_nonCriticalExtensions_232", HFILL }},
    { &hf_rrc_utra_PriorityInfoList_v920ext,
      { "utra-PriorityInfoList-v920ext", "rrc.utra_PriorityInfoList_v920ext",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_eutra_FrequencyAndPriorityInfoList_v920ext,
      { "eutra-FrequencyAndPriorityInfoList-v920ext", "rrc.eutra_FrequencyAndPriorityInfoList_v920ext",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_multipleEutraFrequencyInfoList,
      { "multipleEutraFrequencyInfoList", "rrc.multipleEutraFrequencyInfoList",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_cell_fach_meas_ind,
      { "cell-fach-meas-ind", "rrc.cell_fach_meas_ind",
        FT_UINT32, BASE_DEC, VALS(rrc_T_cell_fach_meas_ind_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_gsm_PriorityInfoList_vbxyext,
      { "gsm-PriorityInfoList-vbxyext", "rrc.gsm_PriorityInfoList_vbxyext",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_eutra_FrequencyAndPriorityInfoList_vbxyext,
      { "eutra-FrequencyAndPriorityInfoList-vbxyext", "rrc.eutra_FrequencyAndPriorityInfoList_vbxyext",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_eutra_FrequencyRACHReportingInfo,
      { "eutra-FrequencyRACHReportingInfo", "rrc.eutra_FrequencyRACHReportingInfo",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_hNBName,
      { "hNBName", "rrc.hNBName",
        FT_STRING, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_nonCriticalExtensions_233,
      { "nonCriticalExtensions", "rrc.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_nonCriticalExtensions_233", HFILL }},
    { &hf_rrc_eab_Parameters,
      { "eab-Parameters", "rrc.eab_Parameters",
        FT_UINT32, BASE_DEC, VALS(rrc_EAB_Parameters_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_nonCriticalExtensions_234,
      { "nonCriticalExtensions", "rrc.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_nonCriticalExtensions_234", HFILL }},
    { &hf_rrc_modeSpecificInfo_155,
      { "modeSpecificInfo", "rrc.modeSpecificInfo",
        FT_UINT32, BASE_DEC, VALS(rrc_T_modeSpecificInfo_155_vals), 0,
        "T_modeSpecificInfo_155", HFILL }},
    { &hf_rrc_fdd_196,
      { "fdd", "rrc.fdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_fdd_190", HFILL }},
    { &hf_rrc_prach_PreambleForEnhancedUplinkListType1Ext,
      { "prach-PreambleForEnhancedUplinkListType1Ext", "rrc.prach_PreambleForEnhancedUplinkListType1Ext",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_prach_PreambleForEnhancedUplinkListType2Ext,
      { "prach-PreambleForEnhancedUplinkListType2Ext", "rrc.prach_PreambleForEnhancedUplinkListType2Ext",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_prach_PreambleForEnhancedUplinkListType3Ext,
      { "prach-PreambleForEnhancedUplinkListType3Ext", "rrc.prach_PreambleForEnhancedUplinkListType3Ext",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_concurrent_Deployment_2ms_10ms_TTI,
      { "concurrent-Deployment-2ms-10ms-TTI", "rrc.concurrent_Deployment_2ms_10ms_TTI",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_nodeB_Trigger_HS_DPCCH_Transmission,
      { "nodeB-Trigger-HS-DPCCH-Transmission", "rrc.nodeB_Trigger_HS_DPCCH_Transmission",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_fallback_R99_PRACH_info,
      { "fallback-R99-PRACH-info", "rrc.fallback_R99_PRACH_info",
        FT_NONE, BASE_NONE, NULL, 0,
        "Fallback_R99_PRACH_info_IEs", HFILL }},
    { &hf_rrc_commonEDCHResourceConfigInfoListExt,
      { "commonEDCHResourceConfigInfoListExt", "rrc.commonEDCHResourceConfigInfoListExt",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_hs_dsch_DrxCellfach_SecondDrx_info,
      { "hs-dsch-DrxCellfach-SecondDrx-info", "rrc.hs_dsch_DrxCellfach_SecondDrx_info",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_nonCriticalExtensions_235,
      { "nonCriticalExtensions", "rrc.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_nonCriticalExtensions_235", HFILL }},
    { &hf_rrc_sib_ReferenceList,
      { "sib-ReferenceList", "rrc.sib_ReferenceList",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_v6b0NonCriticalExtensions_20,
      { "v6b0NonCriticalExtensions", "rrc.v6b0NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v6b0NonCriticalExtensions_19", HFILL }},
    { &hf_rrc_sysInfoTypeSB1_v6b0ext,
      { "sysInfoTypeSB1-v6b0ext", "rrc.sysInfoTypeSB1_v6b0ext",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_v860NonCriticalExtensions_33,
      { "v860NonCriticalExtensions", "rrc.v860NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v860NonCriticalExtensions_33", HFILL }},
    { &hf_rrc_sysInfoTypeSB1_v860ext,
      { "sysInfoTypeSB1-v860ext", "rrc.sysInfoTypeSB1_v860ext",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_nonCriticalExtensions_236,
      { "nonCriticalExtensions", "rrc.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_nonCriticalExtensions_236", HFILL }},
    { &hf_rrc_v6b0NonCriticalExtensions_21,
      { "v6b0NonCriticalExtensions", "rrc.v6b0NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v6b0NonCriticalExtensions_20", HFILL }},
    { &hf_rrc_sysInfoTypeSB2_v6b0ext,
      { "sysInfoTypeSB2-v6b0ext", "rrc.sysInfoTypeSB2_v6b0ext",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_v860NonCriticalExtensions_34,
      { "v860NonCriticalExtensions", "rrc.v860NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v860NonCriticalExtensions_34", HFILL }},
    { &hf_rrc_sysInfoTypeSB2_v860ext,
      { "sysInfoTypeSB2-v860ext", "rrc.sysInfoTypeSB2_v860ext",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_nonCriticalExtensions_237,
      { "nonCriticalExtensions", "rrc.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_nonCriticalExtensions_237", HFILL }},
    { &hf_rrc_TDD_UMTS_Frequency_List_item,
      { "FrequencyInfoTDD", "rrc.FrequencyInfoTDD",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_ue_InactivityPeriod,
      { "ue-InactivityPeriod", "rrc.ue_InactivityPeriod",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_1_120", HFILL }},
    { &hf_rrc_ul_dataVolumeHistory,
      { "ul-dataVolumeHistory", "rrc.ul_dataVolumeHistory",
        FT_NONE, BASE_NONE, NULL, 0,
        "DataVolumeHistory", HFILL }},
    { &hf_rrc_dl_dataVolumeHistory,
      { "dl-dataVolumeHistory", "rrc.dl_dataVolumeHistory",
        FT_NONE, BASE_NONE, NULL, 0,
        "DataVolumeHistory", HFILL }},
    { &hf_rrc_none,
      { "none", "rrc.none",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_cellValueTag,
      { "cellValueTag", "rrc.cellValueTag",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_plmn_ValueTag,
      { "plmn-ValueTag", "rrc.plmn_ValueTag",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_sIBOccurrenceIdentityAndValueTag,
      { "sIBOccurrenceIdentityAndValueTag", "rrc.sIBOccurrenceIdentityAndValueTag",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_commonRBIdentity,
      { "commonRBIdentity", "rrc.commonRBIdentity",
        FT_UINT32, BASE_DEC, NULL, 0,
        "MBMS_CommonRBIdentity", HFILL }},
    { &hf_rrc_rlc_Info_05,
      { "rlc-Info", "rrc.rlc_Info",
        FT_NONE, BASE_NONE, NULL, 0,
        "RLC_Info_MTCH_r6", HFILL }},
    { &hf_rrc_MBMS_CommonRBInformationList_r6_item,
      { "MBMS-CommonRBInformation-r6", "rrc.MBMS_CommonRBInformation_r6",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_countingForUraPCH,
      { "countingForUraPCH", "rrc.countingForUraPCH",
        FT_BOOLEAN, BASE_NONE, NULL, 0,
        "BOOLEAN", HFILL }},
    { &hf_rrc_countingForCellPCH,
      { "countingForCellPCH", "rrc.countingForCellPCH",
        FT_BOOLEAN, BASE_NONE, NULL, 0,
        "BOOLEAN", HFILL }},
    { &hf_rrc_countingForCellFACH,
      { "countingForCellFACH", "rrc.countingForCellFACH",
        FT_BOOLEAN, BASE_NONE, NULL, 0,
        "BOOLEAN", HFILL }},
    { &hf_rrc_sccpchIdentity,
      { "sccpchIdentity", "rrc.sccpchIdentity",
        FT_UINT32, BASE_DEC, NULL, 0,
        "MBMS_SCCPCHIdentity", HFILL }},
    { &hf_rrc_secondaryCCPCH_Info_03,
      { "secondaryCCPCH-Info", "rrc.secondaryCCPCH_Info",
        FT_UINT32, BASE_DEC, NULL, 0,
        "MBMS_CommonPhyChIdentity", HFILL }},
    { &hf_rrc_softComb_TimingOffset,
      { "softComb-TimingOffset", "rrc.softComb_TimingOffset",
        FT_UINT32, BASE_DEC, VALS(rrc_MBMS_SoftComb_TimingOffset_vals), 0,
        "MBMS_SoftComb_TimingOffset", HFILL }},
    { &hf_rrc_transpCh_InfoCommonForAllTrCh,
      { "transpCh-InfoCommonForAllTrCh", "rrc.transpCh_InfoCommonForAllTrCh",
        FT_UINT32, BASE_DEC, NULL, 0,
        "MBMS_CommonCCTrChIdentity", HFILL }},
    { &hf_rrc_transpCHInformation,
      { "transpCHInformation", "rrc.transpCHInformation",
        FT_UINT32, BASE_DEC, NULL, 0,
        "MBMS_TrCHInformation_CurrList", HFILL }},
    { &hf_rrc_MBMS_CurrentCell_SCCPCHList_r6_item,
      { "MBMS-CurrentCell-SCCPCH-r6", "rrc.MBMS_CurrentCell_SCCPCH_r6",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_MBMS_FACHCarryingMTCH_List_item,
      { "TransportFormatSet", "rrc.TransportFormatSet",
        FT_UINT32, BASE_DEC, VALS(rrc_TransportFormatSet_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_cycleOffset,
      { "cycleOffset", "rrc.cycleOffset",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_7", HFILL }},
    { &hf_rrc_mtch_L1CombiningPeriodList,
      { "mtch-L1CombiningPeriodList", "rrc.mtch_L1CombiningPeriodList",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_mtch_L1CombiningPeriodList_item,
      { "mtch-L1CombiningPeriodList item", "rrc.mtch_L1CombiningPeriodList_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_mtch_L1CombiningPeriodList_item", HFILL }},
    { &hf_rrc_periodStart,
      { "periodStart", "rrc.periodStart",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_7", HFILL }},
    { &hf_rrc_periodDuration,
      { "periodDuration", "rrc.periodDuration",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_1_8", HFILL }},
    { &hf_rrc_cycleOffset_01,
      { "cycleOffset", "rrc.cycleOffset",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_15", HFILL }},
    { &hf_rrc_mtch_L1CombiningPeriodList_01,
      { "mtch-L1CombiningPeriodList", "rrc.mtch_L1CombiningPeriodList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "T_mtch_L1CombiningPeriodList_01", HFILL }},
    { &hf_rrc_mtch_L1CombiningPeriodList_item_01,
      { "mtch-L1CombiningPeriodList item", "rrc.mtch_L1CombiningPeriodList_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_mtch_L1CombiningPeriodList_item_01", HFILL }},
    { &hf_rrc_periodStart_01,
      { "periodStart", "rrc.periodStart",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_15", HFILL }},
    { &hf_rrc_periodDuration_01,
      { "periodDuration", "rrc.periodDuration",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_1_16", HFILL }},
    { &hf_rrc_cycleOffset_02,
      { "cycleOffset", "rrc.cycleOffset",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_31", HFILL }},
    { &hf_rrc_mtch_L1CombiningPeriodList_02,
      { "mtch-L1CombiningPeriodList", "rrc.mtch_L1CombiningPeriodList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "T_mtch_L1CombiningPeriodList_02", HFILL }},
    { &hf_rrc_mtch_L1CombiningPeriodList_item_02,
      { "mtch-L1CombiningPeriodList item", "rrc.mtch_L1CombiningPeriodList_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_mtch_L1CombiningPeriodList_item_02", HFILL }},
    { &hf_rrc_periodStart_02,
      { "periodStart", "rrc.periodStart",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_31", HFILL }},
    { &hf_rrc_periodDuration_02,
      { "periodDuration", "rrc.periodDuration",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_1_32", HFILL }},
    { &hf_rrc_cycleOffset_03,
      { "cycleOffset", "rrc.cycleOffset",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_63", HFILL }},
    { &hf_rrc_mtch_L1CombiningPeriodList_03,
      { "mtch-L1CombiningPeriodList", "rrc.mtch_L1CombiningPeriodList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "T_mtch_L1CombiningPeriodList_03", HFILL }},
    { &hf_rrc_mtch_L1CombiningPeriodList_item_03,
      { "mtch-L1CombiningPeriodList item", "rrc.mtch_L1CombiningPeriodList_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_mtch_L1CombiningPeriodList_item_03", HFILL }},
    { &hf_rrc_periodStart_03,
      { "periodStart", "rrc.periodStart",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_63", HFILL }},
    { &hf_rrc_periodDuration_03,
      { "periodDuration", "rrc.periodDuration",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_1_64", HFILL }},
    { &hf_rrc_cycleOffset_04,
      { "cycleOffset", "rrc.cycleOffset",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_127", HFILL }},
    { &hf_rrc_mtch_L1CombiningPeriodList_04,
      { "mtch-L1CombiningPeriodList", "rrc.mtch_L1CombiningPeriodList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "T_mtch_L1CombiningPeriodList_04", HFILL }},
    { &hf_rrc_mtch_L1CombiningPeriodList_item_04,
      { "mtch-L1CombiningPeriodList item", "rrc.mtch_L1CombiningPeriodList_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_mtch_L1CombiningPeriodList_item_04", HFILL }},
    { &hf_rrc_periodStart_04,
      { "periodStart", "rrc.periodStart",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_127", HFILL }},
    { &hf_rrc_periodDuration_04,
      { "periodDuration", "rrc.periodDuration",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_1_128", HFILL }},
    { &hf_rrc_cycleOffset_05,
      { "cycleOffset", "rrc.cycleOffset",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_255", HFILL }},
    { &hf_rrc_mtch_L1CombiningPeriodList_05,
      { "mtch-L1CombiningPeriodList", "rrc.mtch_L1CombiningPeriodList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "T_mtch_L1CombiningPeriodList_05", HFILL }},
    { &hf_rrc_mtch_L1CombiningPeriodList_item_05,
      { "mtch-L1CombiningPeriodList item", "rrc.mtch_L1CombiningPeriodList_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_mtch_L1CombiningPeriodList_item_05", HFILL }},
    { &hf_rrc_periodStart_05,
      { "periodStart", "rrc.periodStart",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_255", HFILL }},
    { &hf_rrc_periodDuration_05,
      { "periodDuration", "rrc.periodDuration",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_1_256", HFILL }},
    { &hf_rrc_cycleLength_32,
      { "cycleLength-32", "rrc.cycleLength_32",
        FT_NONE, BASE_NONE, NULL, 0,
        "MBMS_L1CombiningSchedule_32", HFILL }},
    { &hf_rrc_cycleLength_64,
      { "cycleLength-64", "rrc.cycleLength_64",
        FT_NONE, BASE_NONE, NULL, 0,
        "MBMS_L1CombiningSchedule_64", HFILL }},
    { &hf_rrc_cycleLength_128,
      { "cycleLength-128", "rrc.cycleLength_128",
        FT_NONE, BASE_NONE, NULL, 0,
        "MBMS_L1CombiningSchedule_128", HFILL }},
    { &hf_rrc_cycleLength_256,
      { "cycleLength-256", "rrc.cycleLength_256",
        FT_NONE, BASE_NONE, NULL, 0,
        "MBMS_L1CombiningSchedule_256", HFILL }},
    { &hf_rrc_cycleLength_512,
      { "cycleLength-512", "rrc.cycleLength_512",
        FT_NONE, BASE_NONE, NULL, 0,
        "MBMS_L1CombiningSchedule_512", HFILL }},
    { &hf_rrc_cycleLength_1024,
      { "cycleLength-1024", "rrc.cycleLength_1024",
        FT_NONE, BASE_NONE, NULL, 0,
        "MBMS_L1CombiningSchedule_1024", HFILL }},
    { &hf_rrc_sameAsCurrent,
      { "sameAsCurrent", "rrc.sameAsCurrent",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_currentCell_SCCPCH,
      { "currentCell-SCCPCH", "rrc.currentCell_SCCPCH",
        FT_UINT32, BASE_DEC, NULL, 0,
        "MBMS_SCCPCHIdentity", HFILL }},
    { &hf_rrc_msch_ConfigurationInfo,
      { "msch-ConfigurationInfo", "rrc.msch_ConfigurationInfo",
        FT_NONE, BASE_NONE, NULL, 0,
        "MBMS_MSCH_ConfigurationInfo_r6", HFILL }},
    { &hf_rrc_different,
      { "different", "rrc.different",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_transpCHInformation_01,
      { "transpCHInformation", "rrc.transpCHInformation",
        FT_UINT32, BASE_DEC, NULL, 0,
        "MBMS_TrCHInformation_NeighbList", HFILL }},
    { &hf_rrc_accessInfoPeriodCoefficient,
      { "accessInfoPeriodCoefficient", "rrc.accessInfoPeriodCoefficient",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_3", HFILL }},
    { &hf_rrc_repetitionPeriodCoefficient,
      { "repetitionPeriodCoefficient", "rrc.repetitionPeriodCoefficient",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_3", HFILL }},
    { &hf_rrc_modificationPeriodCoefficient,
      { "modificationPeriodCoefficient", "rrc.modificationPeriodCoefficient",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_7_10", HFILL }},
    { &hf_rrc_rlc_Info_06,
      { "rlc-Info", "rrc.rlc_Info",
        FT_NONE, BASE_NONE, NULL, 0,
        "RLC_Info_MCCH_r6", HFILL }},
    { &hf_rrc_tctf_Presence,
      { "tctf-Presence", "rrc.tctf_Presence",
        FT_UINT32, BASE_DEC, VALS(rrc_MBMS_TCTF_Presence_vals), 0,
        "MBMS_TCTF_Presence", HFILL }},
    { &hf_rrc_michPowerOffset,
      { "michPowerOffset", "rrc.michPowerOffset",
        FT_INT32, BASE_DEC, NULL, 0,
        "MBMS_MICHPowerOffset", HFILL }},
    { &hf_rrc_mode_03,
      { "mode", "rrc.mode",
        FT_UINT32, BASE_DEC, VALS(rrc_T_mode_03_vals), 0,
        "T_mode_03", HFILL }},
    { &hf_rrc_fdd_197,
      { "fdd", "rrc.fdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_fdd_191", HFILL }},
    { &hf_rrc_ni_CountPerFrame,
      { "ni-CountPerFrame", "rrc.ni_CountPerFrame",
        FT_UINT32, BASE_DEC, VALS(rrc_MBMS_NI_CountPerFrame_vals), 0,
        "MBMS_NI_CountPerFrame", HFILL }},
    { &hf_rrc_tdd384_49,
      { "tdd384", "rrc.tdd384",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_tdd384_44", HFILL }},
    { &hf_rrc_repetitionPeriodLengthOffset_01,
      { "repetitionPeriodLengthOffset", "rrc.repetitionPeriodLengthOffset",
        FT_UINT32, BASE_DEC, VALS(rrc_RepPerLengthOffset_MICH_vals), 0,
        "RepPerLengthOffset_MICH", HFILL }},
    { &hf_rrc_mbmsNotificationIndLength,
      { "mbmsNotificationIndLength", "rrc.mbmsNotificationIndLength",
        FT_UINT32, BASE_DEC, VALS(rrc_MBMS_MICHNotificationIndLength_vals), 0,
        "MBMS_MICHNotificationIndLength", HFILL }},
    { &hf_rrc_tdd128_73,
      { "tdd128", "rrc.tdd128",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_tdd128_70", HFILL }},
    { &hf_rrc_channelisationCodeList_02,
      { "channelisationCodeList", "rrc.channelisationCodeList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "SEQUENCE_SIZE_1_2_OF_DL_TS_ChannelisationCode", HFILL }},
    { &hf_rrc_channelisationCodeList_item,
      { "DL-TS-ChannelisationCode", "rrc.DL_TS_ChannelisationCode",
        FT_UINT32, BASE_DEC, VALS(rrc_DL_TS_ChannelisationCode_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_mode_04,
      { "mode", "rrc.mode",
        FT_UINT32, BASE_DEC, VALS(rrc_T_mode_04_vals), 0,
        "T_mode_04", HFILL }},
    { &hf_rrc_tdd384_50,
      { "tdd384", "rrc.tdd384",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_tdd384_45", HFILL }},
    { &hf_rrc_tdd768_28,
      { "tdd768", "rrc.tdd768",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_tdd768_25", HFILL }},
    { &hf_rrc_midambleShiftAndBurstType_06,
      { "midambleShiftAndBurstType", "rrc.midambleShiftAndBurstType",
        FT_NONE, BASE_NONE, NULL, 0,
        "MidambleShiftAndBurstType_VHCR", HFILL }},
    { &hf_rrc_tdd128_74,
      { "tdd128", "rrc.tdd128",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_tdd128_71", HFILL }},
    { &hf_rrc_mode_05,
      { "mode", "rrc.mode",
        FT_UINT32, BASE_DEC, VALS(rrc_T_mode_05_vals), 0,
        "T_mode_05", HFILL }},
    { &hf_rrc_imb384,
      { "imb384", "rrc.imb384",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_ni_CountPerFrame_01,
      { "ni-CountPerFrame", "rrc.ni_CountPerFrame",
        FT_UINT32, BASE_DEC, VALS(rrc_MBMS_NI_CountPerFrame_IMB384_vals), 0,
        "MBMS_NI_CountPerFrame_IMB384", HFILL }},
    { &hf_rrc_mbms_TransmissionIdentity,
      { "mbms-TransmissionIdentity", "rrc.mbms_TransmissionIdentity",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_mbms_RequiredUEAction,
      { "mbms-RequiredUEAction", "rrc.mbms_RequiredUEAction",
        FT_UINT32, BASE_DEC, VALS(rrc_MBMS_RequiredUEAction_Mod_vals), 0,
        "MBMS_RequiredUEAction_Mod", HFILL }},
    { &hf_rrc_mbms_PreferredFrequency,
      { "mbms-PreferredFrequency", "rrc.mbms_PreferredFrequency",
        FT_UINT32, BASE_DEC, VALS(rrc_T_mbms_PreferredFrequency_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_mcch,
      { "mcch", "rrc.mcch",
        FT_UINT32, BASE_DEC, NULL, 0,
        "MBMS_PFLIndex", HFILL }},
    { &hf_rrc_dcch,
      { "dcch", "rrc.dcch",
        FT_NONE, BASE_NONE, NULL, 0,
        "MBMS_PFLInfo", HFILL }},
    { &hf_rrc_dummy_61,
      { "dummy", "rrc.dummy",
        FT_UINT32, BASE_DEC, VALS(rrc_T_dummy_16_vals), 0,
        "T_dummy_16", HFILL }},
    { &hf_rrc_continueMCCHReading,
      { "continueMCCHReading", "rrc.continueMCCHReading",
        FT_BOOLEAN, BASE_NONE, NULL, 0,
        "BOOLEAN", HFILL }},
    { &hf_rrc_MBMS_ModifedServiceList_r6_item,
      { "MBMS-ModifedService-r6", "rrc.MBMS_ModifedService_r6",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_mbsfnClusterFrequency,
      { "mbsfnClusterFrequency", "rrc.mbsfnClusterFrequency",
        FT_UINT32, BASE_DEC, NULL, 0,
        "MBSFN_ClusterFrequency_r7", HFILL }},
    { &hf_rrc_rbReleaseCause,
      { "rbReleaseCause", "rrc.rbReleaseCause",
        FT_UINT32, BASE_DEC, VALS(rrc_MBMS_PTM_RBReleaseCause_LCR_r7_vals), 0,
        "MBMS_PTM_RBReleaseCause_LCR_r7", HFILL }},
    { &hf_rrc_MBMS_ModifiedServiceList_v770ext_item,
      { "MBMS-ModifiedService-v770ext", "rrc.MBMS_ModifiedService_v770ext",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_MBMS_ModifiedServiceList_LCR_v7c0ext_item,
      { "MBMS-ModifiedService-LCR-v7c0ext", "rrc.MBMS_ModifiedService_LCR_v7c0ext",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_mschShedulingInfo,
      { "mschShedulingInfo", "rrc.mschShedulingInfo",
        FT_UINT32, BASE_DEC, VALS(rrc_MBMS_MSCHSchedulingInfo_vals), 0,
        "MBMS_MSCHSchedulingInfo", HFILL }},
    { &hf_rrc_rlc_Info_07,
      { "rlc-Info", "rrc.rlc_Info",
        FT_NONE, BASE_NONE, NULL, 0,
        "RLC_Info_MSCH_r6", HFILL }},
    { &hf_rrc_schedulingPeriod_32_Offset,
      { "schedulingPeriod-32-Offset", "rrc.schedulingPeriod_32_Offset",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_31", HFILL }},
    { &hf_rrc_schedulingPeriod_64_Offset,
      { "schedulingPeriod-64-Offset", "rrc.schedulingPeriod_64_Offset",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_63", HFILL }},
    { &hf_rrc_schedulingPeriod_128_Offset,
      { "schedulingPeriod-128-Offset", "rrc.schedulingPeriod_128_Offset",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_127", HFILL }},
    { &hf_rrc_schedulingPeriod_256_Offset,
      { "schedulingPeriod-256-Offset", "rrc.schedulingPeriod_256_Offset",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_255", HFILL }},
    { &hf_rrc_schedulingPeriod_512_Offset,
      { "schedulingPeriod-512-Offset", "rrc.schedulingPeriod_512_Offset",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_511", HFILL }},
    { &hf_rrc_schedulingPeriod_1024_Offset,
      { "schedulingPeriod-1024-Offset", "rrc.schedulingPeriod_1024_Offset",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_1023", HFILL }},
    { &hf_rrc_secondaryCCPCHPwrOffsetDiff,
      { "secondaryCCPCHPwrOffsetDiff", "rrc.secondaryCCPCHPwrOffsetDiff",
        FT_UINT32, BASE_DEC, VALS(rrc_MBMS_SCCPCHPwrOffsetDiff_vals), 0,
        "MBMS_SCCPCHPwrOffsetDiff", HFILL }},
    { &hf_rrc_layer1Combining,
      { "layer1Combining", "rrc.layer1Combining",
        FT_UINT32, BASE_DEC, VALS(rrc_T_layer1Combining_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_fdd_198,
      { "fdd", "rrc.fdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_fdd_192", HFILL }},
    { &hf_rrc_mbms_L1CombiningTransmTimeDiff,
      { "mbms-L1CombiningTransmTimeDiff", "rrc.mbms_L1CombiningTransmTimeDiff",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_mbms_L1CombiningSchedule,
      { "mbms-L1CombiningSchedule", "rrc.mbms_L1CombiningSchedule",
        FT_UINT32, BASE_DEC, VALS(rrc_MBMS_L1CombiningSchedule_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_mbms_L23Configuration,
      { "mbms-L23Configuration", "rrc.mbms_L23Configuration",
        FT_UINT32, BASE_DEC, VALS(rrc_MBMS_L23Configuration_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_secondaryCCPCH_InfoDiff,
      { "secondaryCCPCH-InfoDiff", "rrc.secondaryCCPCH_InfoDiff",
        FT_NONE, BASE_NONE, NULL, 0,
        "SecondaryCCPCHInfoDiff_MBMS", HFILL }},
    { &hf_rrc_layer1Combining_01,
      { "layer1Combining", "rrc.layer1Combining",
        FT_UINT32, BASE_DEC, VALS(rrc_T_layer1Combining_01_vals), 0,
        "T_layer1Combining_01", HFILL }},
    { &hf_rrc_fdd_199,
      { "fdd", "rrc.fdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_fdd_193", HFILL }},
    { &hf_rrc_MBMS_NeighbouringCellSCCPCHList_r6_item,
      { "MBMS-NeighbouringCellSCCPCH-r6", "rrc.MBMS_NeighbouringCellSCCPCH_r6",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_MBMS_NeighbouringCellSCCPCHList_v770ext_item,
      { "MBMS-NeighbouringCellSCCPCH-v770ext", "rrc.MBMS_NeighbouringCellSCCPCH_v770ext",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_networkStandardTime,
      { "networkStandardTime", "rrc.networkStandardTime",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_40", HFILL }},
    { &hf_rrc_correlativeSFN,
      { "correlativeSFN", "rrc.correlativeSFN",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_4095", HFILL }},
    { &hf_rrc_mbms_CommonPhyChIdentity,
      { "mbms-CommonPhyChIdentity", "rrc.mbms_CommonPhyChIdentity",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_secondaryCCPCHInfo_MBMS_02,
      { "secondaryCCPCHInfo-MBMS", "rrc.secondaryCCPCHInfo_MBMS",
        FT_NONE, BASE_NONE, NULL, 0,
        "SecondaryCCPCHFrameType2Info", HFILL }},
    { &hf_rrc_MBMS_PhyChInformationList_IMB384_item,
      { "MBMS-PhyChInformation-IMB384", "rrc.MBMS_PhyChInformation_IMB384",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_MBMS_PhyChInformationList_r6_item,
      { "MBMS-PhyChInformation-r6", "rrc.MBMS_PhyChInformation_r6",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_MBMS_PhyChInformationList_r7_item,
      { "MBMS-PhyChInformation-r7", "rrc.MBMS_PhyChInformation_r7",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_mbmsPreferredFrequency,
      { "mbmsPreferredFrequency", "rrc.mbmsPreferredFrequency",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_1_maxMBMS_Freq", HFILL }},
    { &hf_rrc_layerConvergenceInformation,
      { "layerConvergenceInformation", "rrc.layerConvergenceInformation",
        FT_UINT32, BASE_DEC, VALS(rrc_T_layerConvergenceInformation_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_mbms_Qoffset,
      { "mbms-Qoffset", "rrc.mbms_Qoffset",
        FT_UINT32, BASE_DEC, VALS(rrc_MBMS_Qoffset_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_mbms_HCSoffset,
      { "mbms-HCSoffset", "rrc.mbms_HCSoffset",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_7", HFILL }},
    { &hf_rrc_MBMS_PreferredFrequencyList_r6_item,
      { "MBMS-PreferredFrequencyInfo-r6", "rrc.MBMS_PreferredFrequencyInfo_r6",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_rbInformation,
      { "rbInformation", "rrc.rbInformation",
        FT_UINT32, BASE_DEC, NULL, 0,
        "MBMS_CommonRBIdentity", HFILL }},
    { &hf_rrc_shortTransmissionID,
      { "shortTransmissionID", "rrc.shortTransmissionID",
        FT_UINT32, BASE_DEC, NULL, 0,
        "MBMS_ShortTransmissionID", HFILL }},
    { &hf_rrc_logicalChIdentity,
      { "logicalChIdentity", "rrc.logicalChIdentity",
        FT_UINT32, BASE_DEC, NULL, 0,
        "MBMS_LogicalChIdentity", HFILL }},
    { &hf_rrc_MBMS_PTM_RBInformation_CList_item,
      { "MBMS-PTM-RBInformation-C", "rrc.MBMS_PTM_RBInformation_C",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_layer1_CombiningStatus,
      { "layer1-CombiningStatus", "rrc.layer1_CombiningStatus",
        FT_BOOLEAN, BASE_NONE, NULL, 0,
        "BOOLEAN", HFILL }},
    { &hf_rrc_MBMS_PTM_RBInformation_NList_item,
      { "MBMS-PTM-RBInformation-N", "rrc.MBMS_PTM_RBInformation_N",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_status,
      { "status", "rrc.status",
        FT_UINT32, BASE_DEC, VALS(rrc_T_status_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_some,
      { "some", "rrc.some",
        FT_UINT32, BASE_DEC, NULL, 0,
        "MBMS_SelectedServicesListFull", HFILL }},
    { &hf_rrc_MBMS_SelectedServicesListFull_item,
      { "MBMS-ServiceIdentity-r6", "rrc.MBMS_ServiceIdentity_r6",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_MBMS_SelectedServicesListShort_item,
      { "MBMS-ShortTransmissionID", "rrc.MBMS_ShortTransmissionID",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_mbms_SelectedServicesList,
      { "mbms-SelectedServicesList", "rrc.mbms_SelectedServicesList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "MBMS_SelectedServicesListShort", HFILL }},
    { &hf_rrc_modificationPeriodIdentity,
      { "modificationPeriodIdentity", "rrc.modificationPeriodIdentity",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_1", HFILL }},
    { &hf_rrc_accessprobabilityFactor_Idle,
      { "accessprobabilityFactor-Idle", "rrc.accessprobabilityFactor_Idle",
        FT_UINT32, BASE_DEC|BASE_EXT_STRING, &rrc_MBMS_AccessProbabilityFactor_vals_ext, 0,
        "MBMS_AccessProbabilityFactor", HFILL }},
    { &hf_rrc_accessprobabilityFactor_Connected,
      { "accessprobabilityFactor-Connected", "rrc.accessprobabilityFactor_Connected",
        FT_UINT32, BASE_DEC|BASE_EXT_STRING, &rrc_MBMS_AccessProbabilityFactor_vals_ext, 0,
        "MBMS_AccessProbabilityFactor", HFILL }},
    { &hf_rrc_mbms_ConnectedModeCountingScope,
      { "mbms-ConnectedModeCountingScope", "rrc.mbms_ConnectedModeCountingScope",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_MBMS_ServiceAccessInfoList_r6_item,
      { "MBMS-ServiceAccessInfo-r6", "rrc.MBMS_ServiceAccessInfo_r6",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_serviceIdentity,
      { "serviceIdentity", "rrc.serviceIdentity",
        FT_BYTES, BASE_NONE, NULL, 0,
        "OCTET_STRING_SIZE_3", HFILL }},
    { &hf_rrc_plmn_Identity_01,
      { "plmn-Identity", "rrc.plmn_Identity",
        FT_UINT32, BASE_DEC, VALS(rrc_T_plmn_Identity_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_sameAsMIB_PLMN_Id,
      { "sameAsMIB-PLMN-Id", "rrc.sameAsMIB_PLMN_Id",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_other_01,
      { "other", "rrc.other",
        FT_UINT32, BASE_DEC, VALS(rrc_T_other_01_vals), 0,
        "T_other_01", HFILL }},
    { &hf_rrc_sameAsMIB_MultiPLMN_Id,
      { "sameAsMIB-MultiPLMN-Id", "rrc.sameAsMIB_MultiPLMN_Id",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_1_5", HFILL }},
    { &hf_rrc_explicitPLMN_Id,
      { "explicitPLMN-Id", "rrc.explicitPLMN_Id",
        FT_NONE, BASE_NONE, NULL, 0,
        "PLMN_Identity", HFILL }},
    { &hf_rrc_mbms_ServiceTransmInfoList,
      { "mbms-ServiceTransmInfoList", "rrc.mbms_ServiceTransmInfoList",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_nextSchedulingperiod,
      { "nextSchedulingperiod", "rrc.nextSchedulingperiod",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_31", HFILL }},
    { &hf_rrc_MBMS_ServiceSchedulingInfoList_r6_item,
      { "MBMS-ServiceSchedulingInfo-r6", "rrc.MBMS_ServiceSchedulingInfo_r6",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_start,
      { "start", "rrc.start",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_255", HFILL }},
    { &hf_rrc_duration,
      { "duration", "rrc.duration",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_1_256", HFILL }},
    { &hf_rrc_MBMS_ServiceTransmInfoList_item,
      { "MBMS-ServiceTransmInfo", "rrc.MBMS_ServiceTransmInfo",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_transpCHInformation_02,
      { "transpCHInformation", "rrc.transpCHInformation",
        FT_UINT32, BASE_DEC, NULL, 0,
        "MBMS_TrCHInformation_SIB5List", HFILL }},
    { &hf_rrc_MBMS_SIBType5_SCCPCHList_r6_item,
      { "MBMS-SIBType5-SCCPCH-r6", "rrc.MBMS_SIBType5_SCCPCH_r6",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_t_318,
      { "t-318", "rrc.t_318",
        FT_UINT32, BASE_DEC, VALS(rrc_T_318_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_mbms_ServiceIdentity_01,
      { "mbms-ServiceIdentity", "rrc.mbms_ServiceIdentity",
        FT_NONE, BASE_NONE, NULL, 0,
        "MBMS_ServiceIdentity_r6", HFILL }},
    { &hf_rrc_commonCCTrChIdentity,
      { "commonCCTrChIdentity", "rrc.commonCCTrChIdentity",
        FT_UINT32, BASE_DEC, NULL, 0,
        "MBMS_CommonCCTrChIdentity", HFILL }},
    { &hf_rrc_MBMS_TranspChInfoForEachCCTrCh_r6_item,
      { "MBMS-TranspChInfoForCCTrCh-r6", "rrc.MBMS_TranspChInfoForCCTrCh_r6",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_MBMS_TranspChInfoForEachTrCh_r6_item,
      { "MBMS-TranspChInfoForTrCh-r6", "rrc.MBMS_TranspChInfoForTrCh_r6",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_commonTrChIdentity,
      { "commonTrChIdentity", "rrc.commonTrChIdentity",
        FT_UINT32, BASE_DEC, NULL, 0,
        "MBMS_CommonTrChIdentity", HFILL }},
    { &hf_rrc_transpCh_Info,
      { "transpCh-Info", "rrc.transpCh_Info",
        FT_UINT32, BASE_DEC, NULL, 0,
        "MBMS_CommonTrChIdentity", HFILL }},
    { &hf_rrc_rbInformation_01,
      { "rbInformation", "rrc.rbInformation",
        FT_UINT32, BASE_DEC, NULL, 0,
        "MBMS_PTM_RBInformation_CList", HFILL }},
    { &hf_rrc_MBMS_TrCHInformation_CurrList_item,
      { "MBMS-TrCHInformation-Curr", "rrc.MBMS_TrCHInformation_Curr",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_transpCh_CombiningStatus,
      { "transpCh-CombiningStatus", "rrc.transpCh_CombiningStatus",
        FT_BOOLEAN, BASE_NONE, NULL, 0,
        "BOOLEAN", HFILL }},
    { &hf_rrc_rbInformation_02,
      { "rbInformation", "rrc.rbInformation",
        FT_UINT32, BASE_DEC, NULL, 0,
        "MBMS_PTM_RBInformation_NList", HFILL }},
    { &hf_rrc_MBMS_TrCHInformation_NeighbList_item,
      { "MBMS-TrCHInformation-Neighb", "rrc.MBMS_TrCHInformation_Neighb",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_transpCh_Identity,
      { "transpCh-Identity", "rrc.transpCh_Identity",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_1_maxFACHPCH", HFILL }},
    { &hf_rrc_MBMS_TrCHInformation_SIB5List_item,
      { "MBMS-TrCHInformation-SIB5", "rrc.MBMS_TrCHInformation_SIB5",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_mbms_RequiredUEAction_01,
      { "mbms-RequiredUEAction", "rrc.mbms_RequiredUEAction",
        FT_UINT32, BASE_DEC, VALS(rrc_MBMS_RequiredUEAction_UMod_vals), 0,
        "MBMS_RequiredUEAction_UMod", HFILL }},
    { &hf_rrc_mbms_PreferredFrequency_01,
      { "mbms-PreferredFrequency", "rrc.mbms_PreferredFrequency",
        FT_UINT32, BASE_DEC, NULL, 0,
        "MBMS_PFLIndex", HFILL }},
    { &hf_rrc_MBMS_UnmodifiedServiceList_r6_item,
      { "MBMS-UnmodifiedService-r6", "rrc.MBMS_UnmodifiedService_r6",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_MBMS_UnmodifiedServiceList_v770ext_item,
      { "MBMS-UnmodifiedService-v770ext", "rrc.MBMS_UnmodifiedService_v770ext",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_imb_Indication,
      { "imb-Indication", "rrc.imb_Indication",
        FT_UINT32, BASE_DEC, VALS(rrc_T_imb_Indication_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_MBSFNFrequencyList_item,
      { "MBSFNFrequency", "rrc.MBSFNFrequency",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_MBSFNFrequencyList_v860ext_item,
      { "MBSFNFrequency-v860ext", "rrc.MBSFNFrequency_v860ext",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_notificationOfAllMBSFNServicesInTheBand,
      { "notificationOfAllMBSFNServicesInTheBand", "rrc.notificationOfAllMBSFNServicesInTheBand",
        FT_UINT32, BASE_DEC, VALS(rrc_T_notificationOfAllMBSFNServicesInTheBand_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_mbsfnFrequency,
      { "mbsfnFrequency", "rrc.mbsfnFrequency",
        FT_NONE, BASE_NONE, NULL, 0,
        "FrequencyInfo", HFILL }},
    { &hf_rrc_mbsfnServicesNotification,
      { "mbsfnServicesNotification", "rrc.mbsfnServicesNotification",
        FT_UINT32, BASE_DEC, VALS(rrc_T_mbsfnServicesNotification_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_mbsfnServicesNotified,
      { "mbsfnServicesNotified", "rrc.mbsfnServicesNotified",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_mbsfnServicesNotNotified,
      { "mbsfnServicesNotNotified", "rrc.mbsfnServicesNotNotified",
        FT_NONE, BASE_NONE, NULL, 0,
        "MBSFNservicesNotNotified_r7", HFILL }},
    { &hf_rrc_imb_Indication_01,
      { "imb-Indication", "rrc.imb_Indication",
        FT_UINT32, BASE_DEC, VALS(rrc_T_imb_Indication_01_vals), 0,
        "T_imb_Indication_01", HFILL }},
    { &hf_rrc_MBSFN_InterFrequencyNeighbourList_r7_item,
      { "MBSFNInterFrequencyNeighbour-r7", "rrc.MBSFNInterFrequencyNeighbour_r7",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_MBSFN_InterFrequencyNeighbourList_v860ext_item,
      { "MBSFNInterFrequencyNeighbour-v860ext", "rrc.MBSFNInterFrequencyNeighbour_v860ext",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_MBSFN_TDDInformation_item,
      { "MBSFN-TDDTimeSlotInfo", "rrc.MBSFN_TDDTimeSlotInfo",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_MBSFN_TDDInformation_LCR_item,
      { "MBSFN-TDDTimeSlotInfo-LCR", "rrc.MBSFN_TDDTimeSlotInfo_LCR",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_timeSlotNumber_01,
      { "timeSlotNumber", "rrc.timeSlotNumber",
        FT_UINT32, BASE_DEC, NULL, 0,
        "TimeslotNumber_LCR_r4", HFILL }},
    { &hf_rrc_frequencyIndex,
      { "frequencyIndex", "rrc.frequencyIndex",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_1_maxMBSFNClusters", HFILL }},
    { &hf_rrc_timeSlotList,
      { "timeSlotList", "rrc.timeSlotList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "MBSFN_TDDInformation", HFILL }},
    { &hf_rrc_tDMPeriod,
      { "tDMPeriod", "rrc.tDMPeriod",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_2_9", HFILL }},
    { &hf_rrc_tDMOffset,
      { "tDMOffset", "rrc.tDMOffset",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_8", HFILL }},
    { &hf_rrc_tDMLength,
      { "tDMLength", "rrc.tDMLength",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_1_8", HFILL }},
    { &hf_rrc_MBSFN_TDM_Info_List_item,
      { "MBSFN-TDM-Info", "rrc.MBSFN_TDM_Info",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_interRATHandoverInfo,
      { "interRATHandoverInfo", "rrc.interRATHandoverInfo",
        FT_UINT32, BASE_DEC, VALS(rrc_InterRATHandoverInfoWithInterRATCapabilities_r3_vals), 0,
        "InterRATHandoverInfoWithInterRATCapabilities_r3", HFILL }},
    { &hf_rrc_srncRelocation,
      { "srncRelocation", "rrc.srncRelocation",
        FT_UINT32, BASE_DEC, VALS(rrc_SRNC_RelocationInfo_r3_vals), 0,
        "SRNC_RelocationInfo_r3", HFILL }},
    { &hf_rrc_rfc3095_ContextInfo,
      { "rfc3095-ContextInfo", "rrc.rfc3095_ContextInfo",
        FT_UINT32, BASE_DEC, VALS(rrc_RFC3095_ContextInfo_r5_vals), 0,
        "RFC3095_ContextInfo_r5", HFILL }},
    { &hf_rrc_extension,
      { "extension", "rrc.extension",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_rrc_FailureInfo,
      { "rrc-FailureInfo", "rrc.rrc_FailureInfo",
        FT_UINT32, BASE_DEC, VALS(rrc_RRC_FailureInfo_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_dL_DCCHmessage,
      { "dL-DCCHmessage", "rrc.dL_DCCHmessage",
        FT_BYTES, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_r3_31,
      { "r3", "rrc.r3",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_r3_31", HFILL }},
    { &hf_rrc_interRATHandoverInfo_r3,
      { "interRATHandoverInfo-r3", "rrc.interRATHandoverInfo_r3",
        FT_NONE, BASE_NONE, NULL, 0,
        "InterRATHandoverInfoWithInterRATCapabilities_r3_IEs", HFILL }},
    { &hf_rrc_v390NonCriticalExtensions_01,
      { "v390NonCriticalExtensions", "rrc.v390NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v390NonCriticalExtensions_01", HFILL }},
    { &hf_rrc_interRATHandoverInfoWithInterRATCapabilities_v390ext,
      { "interRATHandoverInfoWithInterRATCapabilities-v390ext", "rrc.interRATHandoverInfoWithInterRATCapabilities_v390ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "InterRATHandoverInfoWithInterRATCapabilities_v390ext_IEs", HFILL }},
    { &hf_rrc_v690NonCriticalExtensions_52,
      { "v690NonCriticalExtensions", "rrc.v690NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v690NonCriticalExtensions_52", HFILL }},
    { &hf_rrc_interRATHandoverInfoWithInterRATCapabilities_v690ext,
      { "interRATHandoverInfoWithInterRATCapabilities-v690ext", "rrc.interRATHandoverInfoWithInterRATCapabilities_v690ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "InterRATHandoverInfoWithInterRATCapabilities_v690ext_IEs", HFILL }},
    { &hf_rrc_v860NonCriticalExtensions_35,
      { "v860NonCriticalExtensions", "rrc.v860NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v860NonCriticalExtensions_35", HFILL }},
    { &hf_rrc_interRATHandoverInfoWithInterRATCapabilities_v860ext,
      { "interRATHandoverInfoWithInterRATCapabilities-v860ext", "rrc.interRATHandoverInfoWithInterRATCapabilities_v860ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "InterRATHandoverInfoWithInterRATCapabilities_v860ext_IEs", HFILL }},
    { &hf_rrc_v920NonCriticalExtensions_08,
      { "v920NonCriticalExtensions", "rrc.v920NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v920NonCriticalExtensions_08", HFILL }},
    { &hf_rrc_interRATHandoverInfoWithInterRATCapabilities_v920ext,
      { "interRATHandoverInfoWithInterRATCapabilities-v920ext", "rrc.interRATHandoverInfoWithInterRATCapabilities_v920ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "InterRATHandoverInfoWithInterRATCapabilities_v920ext_IEs", HFILL }},
    { &hf_rrc_nonCriticalExtensions_238,
      { "nonCriticalExtensions", "rrc.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_nonCriticalExtensions_238", HFILL }},
    { &hf_rrc_criticalExtensions_140,
      { "criticalExtensions", "rrc.criticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_criticalExtensions_139", HFILL }},
    { &hf_rrc_interRATHandoverInfo_01,
      { "interRATHandoverInfo", "rrc.interRATHandoverInfo",
        FT_BYTES, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_ue_RATSpecificCapability_01,
      { "ue-RATSpecificCapability", "rrc.ue_RATSpecificCapability",
        FT_NONE, BASE_NONE, NULL, 0,
        "InterRAT_UE_RadioAccessCapability_v860ext", HFILL }},
    { &hf_rrc_ue_Inactivity_Period,
      { "ue-Inactivity-Period", "rrc.ue_Inactivity_Period",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_1_120", HFILL }},
    { &hf_rrc_r5_14,
      { "r5", "rrc.r5",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_r5_14", HFILL }},
    { &hf_rrc_rFC3095_ContextInfoList_r5,
      { "rFC3095-ContextInfoList-r5", "rrc.rFC3095_ContextInfoList_r5",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_nonCriticalExtensions_239,
      { "nonCriticalExtensions", "rrc.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_nonCriticalExtensions_239", HFILL }},
    { &hf_rrc_criticalExtensions_141,
      { "criticalExtensions", "rrc.criticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_criticalExtensions_140", HFILL }},
    { &hf_rrc_RFC3095_ContextInfoList_r5_item,
      { "RFC3095-ContextInfo", "rrc.RFC3095_ContextInfo",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_r3_32,
      { "r3", "rrc.r3",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_r3_32", HFILL }},
    { &hf_rrc_sRNC_RelocationInfo_r3,
      { "sRNC-RelocationInfo-r3", "rrc.sRNC_RelocationInfo_r3",
        FT_NONE, BASE_NONE, NULL, 0,
        "SRNC_RelocationInfo_r3_IEs", HFILL }},
    { &hf_rrc_v380NonCriticalExtensions_02,
      { "v380NonCriticalExtensions", "rrc.v380NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v380NonCriticalExtensions_02", HFILL }},
    { &hf_rrc_sRNC_RelocationInfo_v380ext,
      { "sRNC-RelocationInfo-v380ext", "rrc.sRNC_RelocationInfo_v380ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "SRNC_RelocationInfo_v380ext_IEs", HFILL }},
    { &hf_rrc_v390NonCriticalExtensions_02,
      { "v390NonCriticalExtensions", "rrc.v390NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v390NonCriticalExtensions_02", HFILL }},
    { &hf_rrc_sRNC_RelocationInfo_v390ext,
      { "sRNC-RelocationInfo-v390ext", "rrc.sRNC_RelocationInfo_v390ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "SRNC_RelocationInfo_v390ext_IEs", HFILL }},
    { &hf_rrc_v3a0NonCriticalExtensions_16,
      { "v3a0NonCriticalExtensions", "rrc.v3a0NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v3a0NonCriticalExtensions_16", HFILL }},
    { &hf_rrc_sRNC_RelocationInfo_v3a0ext,
      { "sRNC-RelocationInfo-v3a0ext", "rrc.sRNC_RelocationInfo_v3a0ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "SRNC_RelocationInfo_v3a0ext_IEs", HFILL }},
    { &hf_rrc_v3b0NonCriticalExtensions,
      { "v3b0NonCriticalExtensions", "rrc.v3b0NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_sRNC_RelocationInfo_v3b0ext,
      { "sRNC-RelocationInfo-v3b0ext", "rrc.sRNC_RelocationInfo_v3b0ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "SRNC_RelocationInfo_v3b0ext_IEs", HFILL }},
    { &hf_rrc_v3c0NonCriticalExtensions,
      { "v3c0NonCriticalExtensions", "rrc.v3c0NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_sRNC_RelocationInfo_v3c0ext,
      { "sRNC-RelocationInfo-v3c0ext", "rrc.sRNC_RelocationInfo_v3c0ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "SRNC_RelocationInfo_v3c0ext_IEs", HFILL }},
    { &hf_rrc_laterNonCriticalExtensions_68,
      { "laterNonCriticalExtensions", "rrc.laterNonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_laterNonCriticalExtensions_67", HFILL }},
    { &hf_rrc_sRNC_RelocationInfo_v3d0ext,
      { "sRNC-RelocationInfo-v3d0ext", "rrc.sRNC_RelocationInfo_v3d0ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "SRNC_RelocationInfo_v3d0ext_IEs", HFILL }},
    { &hf_rrc_sRNC_RelocationInfo_r3_add_ext,
      { "sRNC-RelocationInfo-r3-add-ext", "rrc.sRNC_RelocationInfo_r3_add_ext",
        FT_BYTES, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_v3g0NonCriticalExtensions_02,
      { "v3g0NonCriticalExtensions", "rrc.v3g0NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v3g0NonCriticalExtensions_02", HFILL }},
    { &hf_rrc_sRNC_RelocationInfo_v3g0ext,
      { "sRNC-RelocationInfo-v3g0ext", "rrc.sRNC_RelocationInfo_v3g0ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "SRNC_RelocationInfo_v3g0ext_IEs", HFILL }},
    { &hf_rrc_v4b0NonCriticalExtensions_27,
      { "v4b0NonCriticalExtensions", "rrc.v4b0NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v4b0NonCriticalExtensions_27", HFILL }},
    { &hf_rrc_sRNC_RelocationInfo_v4b0ext,
      { "sRNC-RelocationInfo-v4b0ext", "rrc.sRNC_RelocationInfo_v4b0ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "SRNC_RelocationInfo_v4b0ext_IEs", HFILL }},
    { &hf_rrc_v590NonCriticalExtensions_31,
      { "v590NonCriticalExtensions", "rrc.v590NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v590NonCriticalExtensions_31", HFILL }},
    { &hf_rrc_sRNC_RelocationInfo_v590ext,
      { "sRNC-RelocationInfo-v590ext", "rrc.sRNC_RelocationInfo_v590ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "SRNC_RelocationInfo_v590ext_IEs", HFILL }},
    { &hf_rrc_v5a0NonCriticalExtensions,
      { "v5a0NonCriticalExtensions", "rrc.v5a0NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_sRNC_RelocationInfo_v5a0ext,
      { "sRNC-RelocationInfo-v5a0ext", "rrc.sRNC_RelocationInfo_v5a0ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "SRNC_RelocationInfo_v5a0ext_IEs", HFILL }},
    { &hf_rrc_v5b0NonCriticalExtensions_03,
      { "v5b0NonCriticalExtensions", "rrc.v5b0NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v5b0NonCriticalExtensions_03", HFILL }},
    { &hf_rrc_sRNC_RelocationInfo_v5b0ext,
      { "sRNC-RelocationInfo-v5b0ext", "rrc.sRNC_RelocationInfo_v5b0ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "SRNC_RelocationInfo_v5b0ext_IEs", HFILL }},
    { &hf_rrc_v5c0NonCriticalExtensions_02,
      { "v5c0NonCriticalExtensions", "rrc.v5c0NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v5c0NonCriticalExtensions_02", HFILL }},
    { &hf_rrc_sRNC_RelocationInfo_v5c0ext,
      { "sRNC-RelocationInfo-v5c0ext", "rrc.sRNC_RelocationInfo_v5c0ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "SRNC_RelocationInfo_v5c0ext_IEs", HFILL }},
    { &hf_rrc_v690NonCriticalExtensions_53,
      { "v690NonCriticalExtensions", "rrc.v690NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v690NonCriticalExtensions_53", HFILL }},
    { &hf_rrc_sRNC_RelocationInfo_v690ext,
      { "sRNC-RelocationInfo-v690ext", "rrc.sRNC_RelocationInfo_v690ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "SRNC_RelocationInfo_v690ext_IEs", HFILL }},
    { &hf_rrc_v6b0NonCriticalExtensions_22,
      { "v6b0NonCriticalExtensions", "rrc.v6b0NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v6b0NonCriticalExtensions_21", HFILL }},
    { &hf_rrc_sRNC_RelocationInfo_v6b0ext,
      { "sRNC-RelocationInfo-v6b0ext", "rrc.sRNC_RelocationInfo_v6b0ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "SRNC_RelocationInfo_v6b0ext_IEs", HFILL }},
    { &hf_rrc_v770NonCriticalExtensions_48,
      { "v770NonCriticalExtensions", "rrc.v770NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v770NonCriticalExtensions_48", HFILL }},
    { &hf_rrc_sRNC_RelocationInfo_v770ext,
      { "sRNC-RelocationInfo-v770ext", "rrc.sRNC_RelocationInfo_v770ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "SRNC_RelocationInfo_v770ext_IEs", HFILL }},
    { &hf_rrc_v7e0NonCriticalExtensions_04,
      { "v7e0NonCriticalExtensions", "rrc.v7e0NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v7e0NonCriticalExtensions_04", HFILL }},
    { &hf_rrc_sRNC_RelocationInfo_v7e0ext,
      { "sRNC-RelocationInfo-v7e0ext", "rrc.sRNC_RelocationInfo_v7e0ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "SRNC_RelocationInfo_v7e0ext_IEs", HFILL }},
    { &hf_rrc_v7f0NonCriticalExtensions_17,
      { "v7f0NonCriticalExtensions", "rrc.v7f0NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v7f0NonCriticalExtensions_17", HFILL }},
    { &hf_rrc_sRNC_RelocationInfo_v7f0ext,
      { "sRNC-RelocationInfo-v7f0ext", "rrc.sRNC_RelocationInfo_v7f0ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "SRNC_RelocationInfo_v7f0ext_IEs", HFILL }},
    { &hf_rrc_nonCriticalExtensions_240,
      { "nonCriticalExtensions", "rrc.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_nonCriticalExtensions_240", HFILL }},
    { &hf_rrc_later_than_r3_28,
      { "later-than-r3", "rrc.later_than_r3",
        FT_UINT32, BASE_DEC, VALS(rrc_T_later_than_r3_28_vals), 0,
        "T_later_than_r3_28", HFILL }},
    { &hf_rrc_r4_14,
      { "r4", "rrc.r4",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_r4_14", HFILL }},
    { &hf_rrc_sRNC_RelocationInfo_r4,
      { "sRNC-RelocationInfo-r4", "rrc.sRNC_RelocationInfo_r4",
        FT_NONE, BASE_NONE, NULL, 0,
        "SRNC_RelocationInfo_r4_IEs", HFILL }},
    { &hf_rrc_v4d0NonCriticalExtensions_14,
      { "v4d0NonCriticalExtensions", "rrc.v4d0NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v4d0NonCriticalExtensions_14", HFILL }},
    { &hf_rrc_sRNC_RelocationInfo_v4d0ext,
      { "sRNC-RelocationInfo-v4d0ext", "rrc.sRNC_RelocationInfo_v4d0ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "SRNC_RelocationInfo_v4d0ext_IEs", HFILL }},
    { &hf_rrc_sRNC_RelocationInfo_r4_add_ext,
      { "sRNC-RelocationInfo-r4-add-ext", "rrc.sRNC_RelocationInfo_r4_add_ext",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING", HFILL }},
    { &hf_rrc_v590NonCriticalExtensions_32,
      { "v590NonCriticalExtensions", "rrc.v590NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v590NonCriticalExtensions_32", HFILL }},
    { &hf_rrc_v5a0NonCriticalExtensions_01,
      { "v5a0NonCriticalExtensions", "rrc.v5a0NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v5a0NonCriticalExtensions_01", HFILL }},
    { &hf_rrc_v5b0NonCriticalExtensions_04,
      { "v5b0NonCriticalExtensions", "rrc.v5b0NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v5b0NonCriticalExtensions_04", HFILL }},
    { &hf_rrc_v5c0NonCriticalExtensions_03,
      { "v5c0NonCriticalExtensions", "rrc.v5c0NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v5c0NonCriticalExtensions_03", HFILL }},
    { &hf_rrc_v690NonCriticalExtensions_54,
      { "v690NonCriticalExtensions", "rrc.v690NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v690NonCriticalExtensions_54", HFILL }},
    { &hf_rrc_v6b0NonCriticalExtensions_23,
      { "v6b0NonCriticalExtensions", "rrc.v6b0NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v6b0NonCriticalExtensions_22", HFILL }},
    { &hf_rrc_v770NonCriticalExtensions_49,
      { "v770NonCriticalExtensions", "rrc.v770NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v770NonCriticalExtensions_49", HFILL }},
    { &hf_rrc_v7e0NonCriticalExtensions_05,
      { "v7e0NonCriticalExtensions", "rrc.v7e0NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v7e0NonCriticalExtensions_05", HFILL }},
    { &hf_rrc_v7f0NonCriticalExtensions_18,
      { "v7f0NonCriticalExtensions", "rrc.v7f0NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v7f0NonCriticalExtensions_18", HFILL }},
    { &hf_rrc_nonCriticalExtensions_241,
      { "nonCriticalExtensions", "rrc.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_nonCriticalExtensions_241", HFILL }},
    { &hf_rrc_criticalExtensions_142,
      { "criticalExtensions", "rrc.criticalExtensions",
        FT_UINT32, BASE_DEC, VALS(rrc_T_criticalExtensions_141_vals), 0,
        "T_criticalExtensions_141", HFILL }},
    { &hf_rrc_r5_15,
      { "r5", "rrc.r5",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_r5_15", HFILL }},
    { &hf_rrc_sRNC_RelocationInfo_r5,
      { "sRNC-RelocationInfo-r5", "rrc.sRNC_RelocationInfo_r5",
        FT_NONE, BASE_NONE, NULL, 0,
        "SRNC_RelocationInfo_r5_IEs", HFILL }},
    { &hf_rrc_sRNC_RelocationInfo_r5_add_ext,
      { "sRNC-RelocationInfo-r5-add-ext", "rrc.sRNC_RelocationInfo_r5_add_ext",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING", HFILL }},
    { &hf_rrc_v5a0NonCriticalExtensions_02,
      { "v5a0NonCriticalExtensions", "rrc.v5a0NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v5a0NonCriticalExtensions_02", HFILL }},
    { &hf_rrc_v5b0NonCriticalExtensions_05,
      { "v5b0NonCriticalExtensions", "rrc.v5b0NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v5b0NonCriticalExtensions_05", HFILL }},
    { &hf_rrc_v5c0NonCriticalExtensions_04,
      { "v5c0NonCriticalExtensions", "rrc.v5c0NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v5c0NonCriticalExtensions_04", HFILL }},
    { &hf_rrc_v690NonCriticalExtensions_55,
      { "v690NonCriticalExtensions", "rrc.v690NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v690NonCriticalExtensions_55", HFILL }},
    { &hf_rrc_v6b0NonCriticalExtensions_24,
      { "v6b0NonCriticalExtensions", "rrc.v6b0NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v6b0NonCriticalExtensions_23", HFILL }},
    { &hf_rrc_v770NonCriticalExtensions_50,
      { "v770NonCriticalExtensions", "rrc.v770NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v770NonCriticalExtensions_50", HFILL }},
    { &hf_rrc_v7e0NonCriticalExtensions_06,
      { "v7e0NonCriticalExtensions", "rrc.v7e0NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v7e0NonCriticalExtensions_06", HFILL }},
    { &hf_rrc_v7f0NonCriticalExtensions_19,
      { "v7f0NonCriticalExtensions", "rrc.v7f0NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v7f0NonCriticalExtensions_19", HFILL }},
    { &hf_rrc_nonCriticalExtensions_242,
      { "nonCriticalExtensions", "rrc.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_nonCriticalExtensions_242", HFILL }},
    { &hf_rrc_criticalExtensions_143,
      { "criticalExtensions", "rrc.criticalExtensions",
        FT_UINT32, BASE_DEC, VALS(rrc_T_criticalExtensions_142_vals), 0,
        "T_criticalExtensions_142", HFILL }},
    { &hf_rrc_r6_12,
      { "r6", "rrc.r6",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_r6_12", HFILL }},
    { &hf_rrc_sRNC_RelocationInfo_r6,
      { "sRNC-RelocationInfo-r6", "rrc.sRNC_RelocationInfo_r6",
        FT_NONE, BASE_NONE, NULL, 0,
        "SRNC_RelocationInfo_r6_IEs", HFILL }},
    { &hf_rrc_sRNC_RelocationInfo_r6_add_ext,
      { "sRNC-RelocationInfo-r6-add-ext", "rrc.sRNC_RelocationInfo_r6_add_ext",
        FT_BYTES, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_v6b0NonCriticalExtensions_25,
      { "v6b0NonCriticalExtensions", "rrc.v6b0NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v6b0NonCriticalExtensions_24", HFILL }},
    { &hf_rrc_v770NonCriticalExtensions_51,
      { "v770NonCriticalExtensions", "rrc.v770NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v770NonCriticalExtensions_51", HFILL }},
    { &hf_rrc_v820NonCriticalExtensions_03,
      { "v820NonCriticalExtensions", "rrc.v820NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v820NonCriticalExtensions_03", HFILL }},
    { &hf_rrc_srnc_RelocationInfo_v820ext,
      { "srnc-RelocationInfo-v820ext", "rrc.srnc_RelocationInfo_v820ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "SRNC_RelocationInfo_v820ext_IEs", HFILL }},
    { &hf_rrc_v860NonCriticalExtensions_36,
      { "v860NonCriticalExtensions", "rrc.v860NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v860NonCriticalExtensions_36", HFILL }},
    { &hf_rrc_sRNC_RelocationInfo_v860ext,
      { "sRNC-RelocationInfo-v860ext", "rrc.sRNC_RelocationInfo_v860ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "SRNC_RelocationInfo_v860ext_IEs", HFILL }},
    { &hf_rrc_nonCriticalExtensions_243,
      { "nonCriticalExtensions", "rrc.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_nonCriticalExtensions_243", HFILL }},
    { &hf_rrc_criticalExtensions_144,
      { "criticalExtensions", "rrc.criticalExtensions",
        FT_UINT32, BASE_DEC, VALS(rrc_T_criticalExtensions_143_vals), 0,
        "T_criticalExtensions_143", HFILL }},
    { &hf_rrc_r7_16,
      { "r7", "rrc.r7",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_r7_16", HFILL }},
    { &hf_rrc_sRNC_RelocationInfo_r7,
      { "sRNC-RelocationInfo-r7", "rrc.sRNC_RelocationInfo_r7",
        FT_NONE, BASE_NONE, NULL, 0,
        "SRNC_RelocationInfo_r7_IEs", HFILL }},
    { &hf_rrc_sRNC_RelocationInfo_r7_add_ext,
      { "sRNC-RelocationInfo-r7-add-ext", "rrc.sRNC_RelocationInfo_r7_add_ext",
        FT_BYTES, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_v820NonCriticalExtensions_04,
      { "v820NonCriticalExtensions", "rrc.v820NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v820NonCriticalExtensions_04", HFILL }},
    { &hf_rrc_v860NonCriticalExtensions_37,
      { "v860NonCriticalExtensions", "rrc.v860NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v860NonCriticalExtensions_37", HFILL }},
    { &hf_rrc_nonCriticalExtensions_244,
      { "nonCriticalExtensions", "rrc.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_nonCriticalExtensions_244", HFILL }},
    { &hf_rrc_criticalExtensions_145,
      { "criticalExtensions", "rrc.criticalExtensions",
        FT_UINT32, BASE_DEC, VALS(rrc_T_criticalExtensions_144_vals), 0,
        "T_criticalExtensions_144", HFILL }},
    { &hf_rrc_r8_12,
      { "r8", "rrc.r8",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_r8_12", HFILL }},
    { &hf_rrc_sRNC_RelocationInfo_r8,
      { "sRNC-RelocationInfo-r8", "rrc.sRNC_RelocationInfo_r8",
        FT_NONE, BASE_NONE, NULL, 0,
        "SRNC_RelocationInfo_r8_IEs", HFILL }},
    { &hf_rrc_sRNC_RelocationInfo_r8_add_ext,
      { "sRNC-RelocationInfo-r8-add-ext", "rrc.sRNC_RelocationInfo_r8_add_ext",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING", HFILL }},
    { &hf_rrc_v7e0NonCriticalExtensions_07,
      { "v7e0NonCriticalExtensions", "rrc.v7e0NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v7e0NonCriticalExtensions_07", HFILL }},
    { &hf_rrc_v7f0NonCriticalExtensions_20,
      { "v7f0NonCriticalExtensions", "rrc.v7f0NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v7f0NonCriticalExtensions_20", HFILL }},
    { &hf_rrc_v8d0NonCriticalExtensions_01,
      { "v8d0NonCriticalExtensions", "rrc.v8d0NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v8d0NonCriticalExtensions_01", HFILL }},
    { &hf_rrc_sRNC_RelocationInfo_v8d0ext,
      { "sRNC-RelocationInfo-v8d0ext", "rrc.sRNC_RelocationInfo_v8d0ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "SRNC_RelocationInfo_v8d0ext_IEs", HFILL }},
    { &hf_rrc_nonCriticalExtensions_245,
      { "nonCriticalExtensions", "rrc.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_nonCriticalExtensions_245", HFILL }},
    { &hf_rrc_criticalExtensions_146,
      { "criticalExtensions", "rrc.criticalExtensions",
        FT_UINT32, BASE_DEC, VALS(rrc_T_criticalExtensions_145_vals), 0,
        "T_criticalExtensions_145", HFILL }},
    { &hf_rrc_r9_11,
      { "r9", "rrc.r9",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_r9_11", HFILL }},
    { &hf_rrc_sRNC_RelocationInfo_r9,
      { "sRNC-RelocationInfo-r9", "rrc.sRNC_RelocationInfo_r9",
        FT_NONE, BASE_NONE, NULL, 0,
        "SRNC_RelocationInfo_r9_IEs", HFILL }},
    { &hf_rrc_sRNC_RelocationInfo_r9_add_ext,
      { "sRNC-RelocationInfo-r9-add-ext", "rrc.sRNC_RelocationInfo_r9_add_ext",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING", HFILL }},
    { &hf_rrc_v8d0NonCriticalExtensions_02,
      { "v8d0NonCriticalExtensions", "rrc.v8d0NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v8d0NonCriticalExtensions_02", HFILL }},
    { &hf_rrc_v970NonCriticalExtensions_04,
      { "v970NonCriticalExtensions", "rrc.v970NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v970NonCriticalExtensions_04", HFILL }},
    { &hf_rrc_sRNC_RelocationInfo_v970ext,
      { "sRNC-RelocationInfo-v970ext", "rrc.sRNC_RelocationInfo_v970ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "SRNC_RelocationInfo_v970ext_IEs", HFILL }},
    { &hf_rrc_v9c0NonCriticalExtensions_13,
      { "v9c0NonCriticalExtensions", "rrc.v9c0NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v9c0NonCriticalExtensions_13", HFILL }},
    { &hf_rrc_sRNC_RelocationInfo_v9c0ext,
      { "sRNC-RelocationInfo-v9c0ext", "rrc.sRNC_RelocationInfo_v9c0ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "SRNC_RelocationInfo_v9c0ext_IEs", HFILL }},
    { &hf_rrc_nonCriticalExtensions_246,
      { "nonCriticalExtensions", "rrc.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_nonCriticalExtensions_246", HFILL }},
    { &hf_rrc_criticalExtensions_147,
      { "criticalExtensions", "rrc.criticalExtensions",
        FT_UINT32, BASE_DEC, VALS(rrc_T_criticalExtensions_146_vals), 0,
        "T_criticalExtensions_146", HFILL }},
    { &hf_rrc_r10_13,
      { "r10", "rrc.r10",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_r10_13", HFILL }},
    { &hf_rrc_sRNC_RelocationInfo_r10,
      { "sRNC-RelocationInfo-r10", "rrc.sRNC_RelocationInfo_r10",
        FT_NONE, BASE_NONE, NULL, 0,
        "SRNC_RelocationInfo_r10_IEs", HFILL }},
    { &hf_rrc_sRNC_RelocationInfo_r10_add_ext,
      { "sRNC-RelocationInfo-r10-add-ext", "rrc.sRNC_RelocationInfo_r10_add_ext",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING", HFILL }},
    { &hf_rrc_nonCriticalExtensions_247,
      { "nonCriticalExtensions", "rrc.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_nonCriticalExtensions_247", HFILL }},
    { &hf_rrc_criticalExtensions_148,
      { "criticalExtensions", "rrc.criticalExtensions",
        FT_UINT32, BASE_DEC, VALS(rrc_T_criticalExtensions_147_vals), 0,
        "T_criticalExtensions_147", HFILL }},
    { &hf_rrc_r11_14,
      { "r11", "rrc.r11",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_r11_14", HFILL }},
    { &hf_rrc_sRNC_RelocationInfo_r11,
      { "sRNC-RelocationInfo-r11", "rrc.sRNC_RelocationInfo_r11",
        FT_NONE, BASE_NONE, NULL, 0,
        "SRNC_RelocationInfo_r11_IEs", HFILL }},
    { &hf_rrc_sRNC_RelocationInfo_r11_add_ext,
      { "sRNC-RelocationInfo-r11-add-ext", "rrc.sRNC_RelocationInfo_r11_add_ext",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING", HFILL }},
    { &hf_rrc_v9c0NonCriticalExtensions_14,
      { "v9c0NonCriticalExtensions", "rrc.v9c0NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v9c0NonCriticalExtensions_14", HFILL }},
    { &hf_rrc_nonCriticalExtensions_248,
      { "nonCriticalExtensions", "rrc.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_nonCriticalExtensions_248", HFILL }},
    { &hf_rrc_criticalExtensions_149,
      { "criticalExtensions", "rrc.criticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_criticalExtensions_148", HFILL }},
    { &hf_rrc_stateOfRRC,
      { "stateOfRRC", "rrc.stateOfRRC",
        FT_UINT32, BASE_DEC, VALS(rrc_StateOfRRC_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_stateOfRRC_Procedure,
      { "stateOfRRC-Procedure", "rrc.stateOfRRC_Procedure",
        FT_UINT32, BASE_DEC, VALS(rrc_StateOfRRC_Procedure_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_cipheringStatus,
      { "cipheringStatus", "rrc.cipheringStatus",
        FT_UINT32, BASE_DEC, VALS(rrc_CipheringStatus_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_calculationTimeForCiphering,
      { "calculationTimeForCiphering", "rrc.calculationTimeForCiphering",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_cipheringInfoPerRB_List,
      { "cipheringInfoPerRB-List", "rrc.cipheringInfoPerRB_List",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_count_C_List,
      { "count-C-List", "rrc.count_C_List",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_integrityProtectionStatus,
      { "integrityProtectionStatus", "rrc.integrityProtectionStatus",
        FT_UINT32, BASE_DEC, VALS(rrc_IntegrityProtectionStatus_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_srb_SpecificIntegrityProtInfo,
      { "srb-SpecificIntegrityProtInfo", "rrc.srb_SpecificIntegrityProtInfo",
        FT_UINT32, BASE_DEC, NULL, 0,
        "SRB_SpecificIntegrityProtInfoList", HFILL }},
    { &hf_rrc_implementationSpecificParams,
      { "implementationSpecificParams", "rrc.implementationSpecificParams",
        FT_BYTES, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_c_RNTI,
      { "c-RNTI", "rrc.c_RNTI",
        FT_BYTES, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_ue_Positioning_LastKnownPos,
      { "ue-Positioning-LastKnownPos", "rrc.ue_Positioning_LastKnownPos",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_cn_CommonGSM_MAP_NAS_SysInfo_04,
      { "cn-CommonGSM-MAP-NAS-SysInfo", "rrc.cn_CommonGSM_MAP_NAS_SysInfo",
        FT_BYTES, BASE_NONE, NULL, 0,
        "NAS_SystemInformationGSM_MAP", HFILL }},
    { &hf_rrc_ongoingMeasRepList,
      { "ongoingMeasRepList", "rrc.ongoingMeasRepList",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_predefinedConfigStatusList_01,
      { "predefinedConfigStatusList", "rrc.predefinedConfigStatusList",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_rab_InformationList_01,
      { "rab-InformationList", "rrc.rab_InformationList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "RAB_InformationSetupList", HFILL }},
    { &hf_rrc_ul_TransChInfoList,
      { "ul-TransChInfoList", "rrc.ul_TransChInfoList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "UL_AddReconfTransChInfoList", HFILL }},
    { &hf_rrc_modeSpecificInfo_156,
      { "modeSpecificInfo", "rrc.modeSpecificInfo",
        FT_UINT32, BASE_DEC, VALS(rrc_T_modeSpecificInfo_156_vals), 0,
        "T_modeSpecificInfo_156", HFILL }},
    { &hf_rrc_fdd_200,
      { "fdd", "rrc.fdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_fdd_194", HFILL }},
    { &hf_rrc_dl_TransChInfoList,
      { "dl-TransChInfoList", "rrc.dl_TransChInfoList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "DL_AddReconfTransChInfoList", HFILL }},
    { &hf_rrc_cipheringStatusList,
      { "cipheringStatusList", "rrc.cipheringStatusList",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_cn_DomainInformationList_v390ext,
      { "cn-DomainInformationList-v390ext", "rrc.cn_DomainInformationList_v390ext",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_cipheringInfoForSRB1_v3a0ext,
      { "cipheringInfoForSRB1-v3a0ext", "rrc.cipheringInfoForSRB1_v3a0ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "CipheringInfoPerRB_List_v3a0ext", HFILL }},
    { &hf_rrc_startValueForCiphering_v3a0ext,
      { "startValueForCiphering-v3a0ext", "rrc.startValueForCiphering_v3a0ext",
        FT_BYTES, BASE_NONE, NULL, 0,
        "START_Value", HFILL }},
    { &hf_rrc_startValueForCiphering_v3b0ext,
      { "startValueForCiphering-v3b0ext", "rrc.startValueForCiphering_v3b0ext",
        FT_UINT32, BASE_DEC, NULL, 0,
        "STARTList2", HFILL }},
    { &hf_rrc_rb_IdentityForHOMessage,
      { "rb-IdentityForHOMessage", "rrc.rb_IdentityForHOMessage",
        FT_UINT32, BASE_DEC, NULL, 0,
        "RB_Identity", HFILL }},
    { &hf_rrc_tpc_CombinationInfoList,
      { "tpc-CombinationInfoList", "rrc.tpc_CombinationInfoList",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_nonCriticalExtension,
      { "nonCriticalExtension", "rrc.nonCriticalExtension",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_TPC_CombinationInfoList_item,
      { "TPC-Combination-Info", "rrc.TPC_Combination_Info",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_TPC_CombinationInfoList_r9_item,
      { "TPC-Combination-Info-r9", "rrc.TPC_Combination_Info_r9",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_STARTList2_item,
      { "STARTSingle", "rrc.STARTSingle",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_storedCompressedModeInfo,
      { "storedCompressedModeInfo", "rrc.storedCompressedModeInfo",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_dl_UM_SN,
      { "dl-UM-SN", "rrc.dl_UM_SN",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_7", HFILL }},
    { &hf_rrc_CipheringStatusList_item,
      { "CipheringStatusCNdomain", "rrc.CipheringStatusCNdomain",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_CodeChangeStatusList_item,
      { "CodeChangeStatus", "rrc.CodeChangeStatus",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_storedTGP_SequenceList,
      { "storedTGP-SequenceList", "rrc.storedTGP_SequenceList",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_codeChangeStatusList,
      { "codeChangeStatusList", "rrc.codeChangeStatusList",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_storedTGP_SequenceList_01,
      { "storedTGP-SequenceList", "rrc.storedTGP_SequenceList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "StoredTGP_SequenceList_r8", HFILL }},
    { &hf_rrc_StoredTGP_SequenceList_item,
      { "StoredTGP-Sequence", "rrc.StoredTGP_Sequence",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_StoredTGP_SequenceList_r8_item,
      { "StoredTGP-Sequence-r8", "rrc.StoredTGP_Sequence_r8",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_current_tgps_Status,
      { "current-tgps-Status", "rrc.current_tgps_Status",
        FT_UINT32, BASE_DEC, VALS(rrc_T_current_tgps_Status_vals), 0,
        "T_current_tgps_Status", HFILL }},
    { &hf_rrc_active,
      { "active", "rrc.active",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_inactive,
      { "inactive", "rrc.inactive",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_current_tgps_Status_01,
      { "current-tgps-Status", "rrc.current_tgps_Status",
        FT_UINT32, BASE_DEC, VALS(rrc_T_current_tgps_Status_01_vals), 0,
        "T_current_tgps_Status_01", HFILL }},
    { &hf_rrc_active_01,
      { "active", "rrc.active",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_active_01", HFILL }},
    { &hf_rrc_cipheringStatusList_01,
      { "cipheringStatusList", "rrc.cipheringStatusList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "CipheringStatusList_r4", HFILL }},
    { &hf_rrc_latestConfiguredCN_Domain,
      { "latestConfiguredCN-Domain", "rrc.latestConfiguredCN_Domain",
        FT_UINT32, BASE_DEC, VALS(rrc_CN_DomainIdentity_vals), 0,
        "CN_DomainIdentity", HFILL }},
    { &hf_rrc_cipheringInfoPerRB_List_01,
      { "cipheringInfoPerRB-List", "rrc.cipheringInfoPerRB_List",
        FT_UINT32, BASE_DEC, NULL, 0,
        "CipheringInfoPerRB_List_r4", HFILL }},
    { &hf_rrc_ue_RadioAccessCapability_03,
      { "ue-RadioAccessCapability", "rrc.ue_RadioAccessCapability",
        FT_NONE, BASE_NONE, NULL, 0,
        "UE_RadioAccessCapability_r4", HFILL }},
    { &hf_rrc_ue_RadioAccessCapability_ext,
      { "ue-RadioAccessCapability-ext", "rrc.ue_RadioAccessCapability_ext",
        FT_UINT32, BASE_DEC, NULL, 0,
        "UE_RadioAccessCapabBandFDDList", HFILL }},
    { &hf_rrc_cn_DomainInformationList_01,
      { "cn-DomainInformationList", "rrc.cn_DomainInformationList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "CN_DomainInformationListFull", HFILL }},
    { &hf_rrc_ongoingMeasRepList_01,
      { "ongoingMeasRepList", "rrc.ongoingMeasRepList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "OngoingMeasRepList_r4", HFILL }},
    { &hf_rrc_rab_InformationList_02,
      { "rab-InformationList", "rrc.rab_InformationList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "RAB_InformationSetupList_r4", HFILL }},
    { &hf_rrc_dummy_62,
      { "dummy", "rrc.dummy",
        FT_UINT32, BASE_DEC, VALS(rrc_T_dummy_17_vals), 0,
        "T_dummy_17", HFILL }},
    { &hf_rrc_fdd_201,
      { "fdd", "rrc.fdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_fdd_195", HFILL }},
    { &hf_rrc_dl_TransChInfoList_01,
      { "dl-TransChInfoList", "rrc.dl_TransChInfoList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "DL_AddReconfTransChInfoList_r4", HFILL }},
    { &hf_rrc_ue_RadioAccessCapability_04,
      { "ue-RadioAccessCapability", "rrc.ue_RadioAccessCapability",
        FT_NONE, BASE_NONE, NULL, 0,
        "UE_RadioAccessCapability_r5", HFILL }},
    { &hf_rrc_ue_RATSpecificCapability_02,
      { "ue-RATSpecificCapability", "rrc.ue_RATSpecificCapability",
        FT_NONE, BASE_NONE, NULL, 0,
        "InterRAT_UE_RadioAccessCapabilityList_r5", HFILL }},
    { &hf_rrc_ongoingMeasRepList_02,
      { "ongoingMeasRepList", "rrc.ongoingMeasRepList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "OngoingMeasRepList_r5", HFILL }},
    { &hf_rrc_srb_InformationList_01,
      { "srb-InformationList", "rrc.srb_InformationList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "SRB_InformationSetupList_r5", HFILL }},
    { &hf_rrc_rab_InformationList_03,
      { "rab-InformationList", "rrc.rab_InformationList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "RAB_InformationSetupList_r5", HFILL }},
    { &hf_rrc_dummy_63,
      { "dummy", "rrc.dummy",
        FT_UINT32, BASE_DEC, VALS(rrc_T_dummy_18_vals), 0,
        "T_dummy_18", HFILL }},
    { &hf_rrc_fdd_202,
      { "fdd", "rrc.fdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_fdd_196", HFILL }},
    { &hf_rrc_dl_TransChInfoList_02,
      { "dl-TransChInfoList", "rrc.dl_TransChInfoList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "DL_AddReconfTransChInfoList_r5", HFILL }},
    { &hf_rrc_ueCapabilityContainer_02,
      { "ueCapabilityContainer", "rrc.ueCapabilityContainer",
        FT_BYTES, BASE_NONE, NULL, 0,
        "T_ueCapabilityContainer_02", HFILL }},
    { &hf_rrc_ueCapabilityContainer_RSC,
      { "ueCapabilityContainer-RSC", "rrc.ueCapabilityContainer_RSC",
        FT_BYTES, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_ueCapabilityContainer_UCI,
      { "ueCapabilityContainer-UCI", "rrc.ueCapabilityContainer_UCI",
        FT_BYTES, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_intraFreqReportingCriteria_08,
      { "intraFreqReportingCriteria", "rrc.intraFreqReportingCriteria",
        FT_NONE, BASE_NONE, NULL, 0,
        "IntraFreqReportingCriteria_r6_ext", HFILL }},
    { &hf_rrc_ue_RadioAccessCapability_05,
      { "ue-RadioAccessCapability", "rrc.ue_RadioAccessCapability",
        FT_NONE, BASE_NONE, NULL, 0,
        "UE_RadioAccessCapability_r6", HFILL }},
    { &hf_rrc_ueCapabilityContainer_03,
      { "ueCapabilityContainer", "rrc.ueCapabilityContainer",
        FT_BYTES, BASE_NONE, NULL, 0,
        "T_ueCapabilityContainer_03", HFILL }},
    { &hf_rrc_ueCapabilityContainer_RSC_01,
      { "ueCapabilityContainer-RSC", "rrc.ueCapabilityContainer_RSC",
        FT_BYTES, BASE_NONE, NULL, 0,
        "T_ueCapabilityContainer_RSC_01", HFILL }},
    { &hf_rrc_ueCapabilityContainer_UCI_01,
      { "ueCapabilityContainer-UCI", "rrc.ueCapabilityContainer_UCI",
        FT_BYTES, BASE_NONE, NULL, 0,
        "T_ueCapabilityContainer_UCI_01", HFILL }},
    { &hf_rrc_ongoingMeasRepList_03,
      { "ongoingMeasRepList", "rrc.ongoingMeasRepList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "OngoingMeasRepList_r6", HFILL }},
    { &hf_rrc_srb_InformationList_02,
      { "srb-InformationList", "rrc.srb_InformationList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "SRB_InformationSetupList_r6", HFILL }},
    { &hf_rrc_rab_InformationList_04,
      { "rab-InformationList", "rrc.rab_InformationList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "RAB_InformationSetupList_r6", HFILL }},
    { &hf_rrc_ul_TransChInfoList_01,
      { "ul-TransChInfoList", "rrc.ul_TransChInfoList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "UL_AddReconfTransChInfoList_r6", HFILL }},
    { &hf_rrc_measurementReport_01,
      { "measurementReport", "rrc.measurementReport",
        FT_BYTES, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_v7f0NonCriticalExtensions_21,
      { "v7f0NonCriticalExtensions", "rrc.v7f0NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v7f0NonCriticalExtensions_21", HFILL }},
    { &hf_rrc_nonCriticalExtensions_249,
      { "nonCriticalExtensions", "rrc.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_nonCriticalExtensions_249", HFILL }},
    { &hf_rrc_ueCapabilityContainer_04,
      { "ueCapabilityContainer", "rrc.ueCapabilityContainer",
        FT_BYTES, BASE_NONE, NULL, 0,
        "T_ueCapabilityContainer_04", HFILL }},
    { &hf_rrc_ueCapabilityContainer_RSC_02,
      { "ueCapabilityContainer-RSC", "rrc.ueCapabilityContainer_RSC",
        FT_BYTES, BASE_NONE, NULL, 0,
        "T_ueCapabilityContainer_RSC_02", HFILL }},
    { &hf_rrc_ueCapabilityContainer_UCI_02,
      { "ueCapabilityContainer-UCI", "rrc.ueCapabilityContainer_UCI",
        FT_BYTES, BASE_NONE, NULL, 0,
        "T_ueCapabilityContainer_UCI_02", HFILL }},
    { &hf_rrc_srns_t_305,
      { "srns-t-305", "rrc.srns_t_305",
        FT_UINT32, BASE_DEC, VALS(rrc_T_305_vals), 0,
        "T_305", HFILL }},
    { &hf_rrc_ongoingMeasRepList_04,
      { "ongoingMeasRepList", "rrc.ongoingMeasRepList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "OngoingMeasRepList_r7", HFILL }},
    { &hf_rrc_srb_InformationList_03,
      { "srb-InformationList", "rrc.srb_InformationList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "SRB_InformationSetupList_r7", HFILL }},
    { &hf_rrc_rab_InformationList_05,
      { "rab-InformationList", "rrc.rab_InformationList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "RAB_InformationSetupList_r7", HFILL }},
    { &hf_rrc_ul_TransChInfoList_02,
      { "ul-TransChInfoList", "rrc.ul_TransChInfoList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "UL_AddReconfTransChInfoList_r7", HFILL }},
    { &hf_rrc_dl_TransChInfoList_03,
      { "dl-TransChInfoList", "rrc.dl_TransChInfoList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "DL_AddReconfTransChInfoList_r7", HFILL }},
    { &hf_rrc_measurementReport_02,
      { "measurementReport", "rrc.measurementReport",
        FT_BYTES, BASE_NONE, NULL, 0,
        "T_measurementReport_01", HFILL }},
    { &hf_rrc_v7f0NonCriticalExtensions_22,
      { "v7f0NonCriticalExtensions", "rrc.v7f0NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_v7f0NonCriticalExtensions_22", HFILL }},
    { &hf_rrc_nonCriticalExtensions_250,
      { "nonCriticalExtensions", "rrc.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_nonCriticalExtensions_250", HFILL }},
    { &hf_rrc_thresholdSFN_GPS_TOW_us,
      { "thresholdSFN-GPS-TOW-us", "rrc.thresholdSFN_GPS_TOW_us",
        FT_UINT32, BASE_DEC, VALS(rrc_ThresholdSFN_GPS_TOW_us_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_ueCapabilityContainer_05,
      { "ueCapabilityContainer", "rrc.ueCapabilityContainer",
        FT_BYTES, BASE_NONE, NULL, 0,
        "T_ueCapabilityContainer_05", HFILL }},
    { &hf_rrc_ueCapabilityContainer_RSC_03,
      { "ueCapabilityContainer-RSC", "rrc.ueCapabilityContainer_RSC",
        FT_BYTES, BASE_NONE, NULL, 0,
        "T_ueCapabilityContainer_RSC_03", HFILL }},
    { &hf_rrc_ueCapabilityContainer_UCI_03,
      { "ueCapabilityContainer-UCI", "rrc.ueCapabilityContainer_UCI",
        FT_BYTES, BASE_NONE, NULL, 0,
        "T_ueCapabilityContainer_UCI_03", HFILL }},
    { &hf_rrc_ongoingMeasRepList_05,
      { "ongoingMeasRepList", "rrc.ongoingMeasRepList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "OngoingMeasRepList_r8", HFILL }},
    { &hf_rrc_srb_InformationList_04,
      { "srb-InformationList", "rrc.srb_InformationList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "SRB_InformationSetupList_r8", HFILL }},
    { &hf_rrc_rab_InformationList_06,
      { "rab-InformationList", "rrc.rab_InformationList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "RAB_InformationSetupList_r8", HFILL }},
    { &hf_rrc_ul_TransChInfoList_03,
      { "ul-TransChInfoList", "rrc.ul_TransChInfoList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "UL_AddReconfTransChInfoList_r8", HFILL }},
    { &hf_rrc_e_RGCH_CombinationInfoList,
      { "e-RGCH-CombinationInfoList", "rrc.e_RGCH_CombinationInfoList",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_storedCompressedModeInfo_01,
      { "storedCompressedModeInfo", "rrc.storedCompressedModeInfo",
        FT_NONE, BASE_NONE, NULL, 0,
        "StoredCompressedModeInfo_r8", HFILL }},
    { &hf_rrc_measurementReport_03,
      { "measurementReport", "rrc.measurementReport",
        FT_BYTES, BASE_NONE, NULL, 0,
        "T_measurementReport_02", HFILL }},
    { &hf_rrc_ue_HistoryInformation,
      { "ue-HistoryInformation", "rrc.ue_HistoryInformation",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_rab_InformationList_07,
      { "rab-InformationList", "rrc.rab_InformationList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "RAB_InformationSetupList_v820ext", HFILL }},
    { &hf_rrc_ueCapabilityContainer_06,
      { "ueCapabilityContainer", "rrc.ueCapabilityContainer",
        FT_BYTES, BASE_NONE, NULL, 0,
        "T_ueCapabilityContainer_06", HFILL }},
    { &hf_rrc_ueCapabilityContainer_RSC_04,
      { "ueCapabilityContainer-RSC", "rrc.ueCapabilityContainer_RSC",
        FT_BYTES, BASE_NONE, NULL, 0,
        "T_ueCapabilityContainer_RSC_04", HFILL }},
    { &hf_rrc_ueCapabilityContainer_UCI_04,
      { "ueCapabilityContainer-UCI", "rrc.ueCapabilityContainer_UCI",
        FT_BYTES, BASE_NONE, NULL, 0,
        "T_ueCapabilityContainer_UCI_04", HFILL }},
    { &hf_rrc_ongoingMeasRepList_06,
      { "ongoingMeasRepList", "rrc.ongoingMeasRepList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "OngoingMeasRepList_r9", HFILL }},
    { &hf_rrc_dl_TransChInfoList_04,
      { "dl-TransChInfoList", "rrc.dl_TransChInfoList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "DL_AddReconfTransChInfoList_r9", HFILL }},
    { &hf_rrc_secondary_tpc_CombinationInfoList,
      { "secondary-tpc-CombinationInfoList", "rrc.secondary_tpc_CombinationInfoList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "TPC_CombinationInfoList_r9", HFILL }},
    { &hf_rrc_secondary_e_RGCH_CombinationInfoList,
      { "secondary-e-RGCH-CombinationInfoList", "rrc.secondary_e_RGCH_CombinationInfoList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "E_RGCH_CombinationInfoList_r9", HFILL }},
    { &hf_rrc_measurementReport_04,
      { "measurementReport", "rrc.measurementReport",
        FT_BYTES, BASE_NONE, NULL, 0,
        "T_measurementReport_03", HFILL }},
    { &hf_rrc_ongoingMeasRepList_07,
      { "ongoingMeasRepList", "rrc.ongoingMeasRepList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "OngoingMeasRepList_v970ext_IE", HFILL }},
    { &hf_rrc_ueCapabilityContainer_07,
      { "ueCapabilityContainer", "rrc.ueCapabilityContainer",
        FT_BYTES, BASE_NONE, NULL, 0,
        "T_ueCapabilityContainer_07", HFILL }},
    { &hf_rrc_ueCapabilityContainer_RSC_05,
      { "ueCapabilityContainer-RSC", "rrc.ueCapabilityContainer_RSC",
        FT_BYTES, BASE_NONE, NULL, 0,
        "T_ueCapabilityContainer_RSC_05", HFILL }},
    { &hf_rrc_ueCapabilityContainer_UCI_05,
      { "ueCapabilityContainer-UCI", "rrc.ueCapabilityContainer_UCI",
        FT_BYTES, BASE_NONE, NULL, 0,
        "T_ueCapabilityContainer_UCI_05", HFILL }},
    { &hf_rrc_ongoingMeasRepList_08,
      { "ongoingMeasRepList", "rrc.ongoingMeasRepList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "OngoingMeasRepList_r10", HFILL }},
    { &hf_rrc_measurementReport_05,
      { "measurementReport", "rrc.measurementReport",
        FT_BYTES, BASE_NONE, NULL, 0,
        "T_measurementReport_04", HFILL }},
    { &hf_rrc_ueCapabilityContainer_08,
      { "ueCapabilityContainer", "rrc.ueCapabilityContainer",
        FT_BYTES, BASE_NONE, NULL, 0,
        "T_ueCapabilityContainer_08", HFILL }},
    { &hf_rrc_ueCapabilityContainer_RSC_06,
      { "ueCapabilityContainer-RSC", "rrc.ueCapabilityContainer_RSC",
        FT_BYTES, BASE_NONE, NULL, 0,
        "T_ueCapabilityContainer_RSC_06", HFILL }},
    { &hf_rrc_ueCapabilityContainer_UCI_06,
      { "ueCapabilityContainer-UCI", "rrc.ueCapabilityContainer_UCI",
        FT_BYTES, BASE_NONE, NULL, 0,
        "T_ueCapabilityContainer_UCI_06", HFILL }},
    { &hf_rrc_ongoingMeasRepList_09,
      { "ongoingMeasRepList", "rrc.ongoingMeasRepList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "OngoingMeasRepList_r11", HFILL }},
    { &hf_rrc_rab_InformationList_08,
      { "rab-InformationList", "rrc.rab_InformationList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "RAB_InformationSetupList_r11", HFILL }},
    { &hf_rrc_dl_TransChInfoList_05,
      { "dl-TransChInfoList", "rrc.dl_TransChInfoList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "DL_AddReconfTransChInfoList_r11", HFILL }},
    { &hf_rrc_measurementReport_06,
      { "measurementReport", "rrc.measurementReport",
        FT_BYTES, BASE_NONE, NULL, 0,
        "T_measurementReport_05", HFILL }},
    { &hf_rrc_dl_HFN,
      { "dl-HFN", "rrc.dl_HFN",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_20_25", HFILL }},
    { &hf_rrc_ul_HFN,
      { "ul-HFN", "rrc.ul_HFN",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_20_25", HFILL }},
    { &hf_rrc_CipheringInfoPerRB_List_item,
      { "CipheringInfoPerRB", "rrc.CipheringInfoPerRB",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_CipheringInfoPerRB_List_r4_item,
      { "CipheringInfoPerRB-r4", "rrc.CipheringInfoPerRB_r4",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_CipheringStatusList_r4_item,
      { "CipheringStatusCNdomain-r4", "rrc.CipheringStatusCNdomain_r4",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_CN_DomainInformationList_v390ext_item,
      { "CN-DomainInformation-v390ext", "rrc.CN_DomainInformation_v390ext",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_tdd384_Measurements,
      { "tdd384-Measurements", "rrc.tdd384_Measurements",
        FT_BOOLEAN, BASE_NONE, NULL, 0,
        "BOOLEAN", HFILL }},
    { &hf_rrc_COUNT_C_List_item,
      { "COUNT-CSingle", "rrc.COUNT_CSingle",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_count_C,
      { "count-C", "rrc.count_C",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_32", HFILL }},
    { &hf_rrc_dummy3_05,
      { "dummy3", "rrc.dummy3",
        FT_UINT32, BASE_DEC, VALS(rrc_SupportOfDedicatedPilotsForChEstimation_vals), 0,
        "SupportOfDedicatedPilotsForChEstimation", HFILL }},
    { &hf_rrc_fdd_hspdsch_01,
      { "fdd-hspdsch", "rrc.fdd_hspdsch",
        FT_UINT32, BASE_DEC, VALS(rrc_T_fdd_hspdsch_01_vals), 0,
        "T_fdd_hspdsch_01", HFILL }},
    { &hf_rrc_supported_14,
      { "supported", "rrc.supported",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_supported_07", HFILL }},
    { &hf_rrc_tdd384_hspdsch_01,
      { "tdd384-hspdsch", "rrc.tdd384_hspdsch",
        FT_UINT32, BASE_DEC, VALS(rrc_T_tdd384_hspdsch_01_vals), 0,
        "T_tdd384_hspdsch_01", HFILL }},
    { &hf_rrc_tdd128_hspdsch_01,
      { "tdd128-hspdsch", "rrc.tdd128_hspdsch",
        FT_UINT32, BASE_DEC, VALS(rrc_T_tdd128_hspdsch_01_vals), 0,
        "T_tdd128_hspdsch_01", HFILL }},
    { &hf_rrc_rfc3095_Context_Identity,
      { "rfc3095-Context-Identity", "rrc.rfc3095_Context_Identity",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_16383", HFILL }},
    { &hf_rrc_dl_mode,
      { "dl-mode", "rrc.dl_mode",
        FT_UINT32, BASE_DEC, VALS(rrc_T_dl_mode_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_dl_ref_ir,
      { "dl-ref-ir", "rrc.dl_ref_ir",
        FT_BYTES, BASE_NONE, NULL, 0,
        "OCTET_STRING_SIZE_1_3000", HFILL }},
    { &hf_rrc_dl_ref_time,
      { "dl-ref-time", "rrc.dl_ref_time",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_4294967295", HFILL }},
    { &hf_rrc_dl_curr_time,
      { "dl-curr-time", "rrc.dl_curr_time",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_4294967295", HFILL }},
    { &hf_rrc_dl_syn_offset_id,
      { "dl-syn-offset-id", "rrc.dl_syn_offset_id",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_65535", HFILL }},
    { &hf_rrc_dl_syn_slope_ts,
      { "dl-syn-slope-ts", "rrc.dl_syn_slope_ts",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_4294967295", HFILL }},
    { &hf_rrc_dl_dyn_changed,
      { "dl-dyn-changed", "rrc.dl_dyn_changed",
        FT_BOOLEAN, BASE_NONE, NULL, 0,
        "BOOLEAN", HFILL }},
    { &hf_rrc_rg_CombinationIndex_01,
      { "rg-CombinationIndex", "rrc.rg_CombinationIndex",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_5", HFILL }},
    { &hf_rrc_E_RGCH_CombinationInfoList_item,
      { "E-RGCH-Combination-Info", "rrc.E_RGCH_Combination_Info",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_E_RGCH_CombinationInfoList_r9_item,
      { "E-RGCH-Combination-Info-r9", "rrc.E_RGCH_Combination_Info_r9",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_interRAT_UE_RadioAccessCapability,
      { "interRAT-UE-RadioAccessCapability", "rrc.interRAT_UE_RadioAccessCapability",
        FT_UINT32, BASE_DEC, NULL, 0,
        "InterRAT_UE_RadioAccessCapabilityList", HFILL }},
    { &hf_rrc_event_06,
      { "event", "rrc.event",
        FT_NONE, BASE_NONE, NULL, 0,
        "Event1j_r6", HFILL }},
    { &hf_rrc_downlinkCompressedMode_04,
      { "downlinkCompressedMode", "rrc.downlinkCompressedMode",
        FT_NONE, BASE_NONE, NULL, 0,
        "CompressedModeMeasCapability_r4", HFILL }},
    { &hf_rrc_uplinkCompressedMode_01,
      { "uplinkCompressedMode", "rrc.uplinkCompressedMode",
        FT_NONE, BASE_NONE, NULL, 0,
        "CompressedModeMeasCapability_r4", HFILL }},
    { &hf_rrc_modify_09,
      { "modify", "rrc.modify",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_measurementCommandWithType,
      { "measurementCommandWithType", "rrc.measurementCommandWithType",
        FT_UINT32, BASE_DEC, VALS(rrc_MeasurementCommandWithType_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_additionalMeasurementID_List,
      { "additionalMeasurementID-List", "rrc.additionalMeasurementID_List",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_measurementCommandWithType_01,
      { "measurementCommandWithType", "rrc.measurementCommandWithType",
        FT_UINT32, BASE_DEC, VALS(rrc_MeasurementCommandWithType_r4_vals), 0,
        "MeasurementCommandWithType_r4", HFILL }},
    { &hf_rrc_measurementCommand_v590ext_01,
      { "measurementCommand-v590ext", "rrc.measurementCommand_v590ext",
        FT_UINT32, BASE_DEC, VALS(rrc_T_measurementCommand_v590ext_01_vals), 0,
        "T_measurementCommand_v590ext_01", HFILL }},
    { &hf_rrc_measurementCommandWithType_02,
      { "measurementCommandWithType", "rrc.measurementCommandWithType",
        FT_UINT32, BASE_DEC, VALS(rrc_MeasurementCommandWithType_r6_vals), 0,
        "MeasurementCommandWithType_r6", HFILL }},
    { &hf_rrc_measurementCommandWithType_03,
      { "measurementCommandWithType", "rrc.measurementCommandWithType",
        FT_UINT32, BASE_DEC, VALS(rrc_MeasurementCommandWithType_r7_vals), 0,
        "MeasurementCommandWithType_r7", HFILL }},
    { &hf_rrc_measurementCommandWithType_04,
      { "measurementCommandWithType", "rrc.measurementCommandWithType",
        FT_UINT32, BASE_DEC, VALS(rrc_MeasurementCommandWithType_r8_vals), 0,
        "MeasurementCommandWithType_r8", HFILL }},
    { &hf_rrc_measurementCommandWithType_05,
      { "measurementCommandWithType", "rrc.measurementCommandWithType",
        FT_UINT32, BASE_DEC, VALS(rrc_MeasurementCommandWithType_r9_vals), 0,
        "MeasurementCommandWithType_r9", HFILL }},
    { &hf_rrc_additionalMeasurementID_List_01,
      { "additionalMeasurementID-List", "rrc.additionalMeasurementID_List",
        FT_UINT32, BASE_DEC, NULL, 0,
        "AdditionalMeasurementID_List_r9", HFILL }},
    { &hf_rrc_measurementCommandWithType_06,
      { "measurementCommandWithType", "rrc.measurementCommandWithType",
        FT_UINT32, BASE_DEC, VALS(rrc_MeasurementCommandWithType_r10_vals), 0,
        "MeasurementCommandWithType_r10", HFILL }},
    { &hf_rrc_measurementCommandWithType_07,
      { "measurementCommandWithType", "rrc.measurementCommandWithType",
        FT_UINT32, BASE_DEC, VALS(rrc_MeasurementCommandWithType_r11_vals), 0,
        "MeasurementCommandWithType_r11", HFILL }},
    { &hf_rrc_OngoingMeasRepList_item,
      { "OngoingMeasRep", "rrc.OngoingMeasRep",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_OngoingMeasRepList_r4_item,
      { "OngoingMeasRep-r4", "rrc.OngoingMeasRep_r4",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_OngoingMeasRepList_r5_item,
      { "OngoingMeasRep-r5", "rrc.OngoingMeasRep_r5",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_OngoingMeasRepList_r6_item,
      { "OngoingMeasRep-r6", "rrc.OngoingMeasRep_r6",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_OngoingMeasRepList_r7_item,
      { "OngoingMeasRep-r7", "rrc.OngoingMeasRep_r7",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_OngoingMeasRepList_r8_item,
      { "OngoingMeasRep-r8", "rrc.OngoingMeasRep_r8",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_OngoingMeasRepList_r9_item,
      { "OngoingMeasRep-r9", "rrc.OngoingMeasRep_r9",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_OngoingMeasRepList_v970ext_IE_item,
      { "OngoingMeasRep-v970ext-IEs", "rrc.OngoingMeasRep_v970ext_IEs",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_OngoingMeasRepList_r10_item,
      { "OngoingMeasRep-r10", "rrc.OngoingMeasRep_r10",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_OngoingMeasRepList_r11_item,
      { "OngoingMeasRep-r11", "rrc.OngoingMeasRep_r11",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_supportForRfc2507_01,
      { "supportForRfc2507", "rrc.supportForRfc2507",
        FT_UINT32, BASE_DEC, VALS(rrc_T_supportForRfc2507_01_vals), 0,
        "T_supportForRfc2507_01", HFILL }},
    { &hf_rrc_supportForRfc3095_01,
      { "supportForRfc3095", "rrc.supportForRfc3095",
        FT_UINT32, BASE_DEC, VALS(rrc_T_supportForRfc3095_01_vals), 0,
        "T_supportForRfc3095_01", HFILL }},
    { &hf_rrc_supported_15,
      { "supported", "rrc.supported",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_supported_08", HFILL }},
    { &hf_rrc_supportForRfc2507_02,
      { "supportForRfc2507", "rrc.supportForRfc2507",
        FT_UINT32, BASE_DEC, VALS(rrc_T_supportForRfc2507_02_vals), 0,
        "T_supportForRfc2507_02", HFILL }},
    { &hf_rrc_supported_16,
      { "supported", "rrc.supported",
        FT_UINT32, BASE_DEC, VALS(rrc_MaxHcContextSpace_r5_vals), 0,
        "MaxHcContextSpace_r5", HFILL }},
    { &hf_rrc_supportForRfc3095_02,
      { "supportForRfc3095", "rrc.supportForRfc3095",
        FT_UINT32, BASE_DEC, VALS(rrc_T_supportForRfc3095_02_vals), 0,
        "T_supportForRfc3095_02", HFILL }},
    { &hf_rrc_supported_17,
      { "supported", "rrc.supported",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_supported_09", HFILL }},
    { &hf_rrc_losslessDLRLC_PDUSizeChange_01,
      { "losslessDLRLC-PDUSizeChange", "rrc.losslessDLRLC_PDUSizeChange",
        FT_UINT32, BASE_DEC, VALS(rrc_T_losslessDLRLC_PDUSizeChange_01_vals), 0,
        "T_losslessDLRLC_PDUSizeChange_01", HFILL }},
    { &hf_rrc_supportForRfc2507_03,
      { "supportForRfc2507", "rrc.supportForRfc2507",
        FT_UINT32, BASE_DEC, VALS(rrc_T_supportForRfc2507_03_vals), 0,
        "T_supportForRfc2507_03", HFILL }},
    { &hf_rrc_supportForRfc3095_03,
      { "supportForRfc3095", "rrc.supportForRfc3095",
        FT_UINT32, BASE_DEC, VALS(rrc_T_supportForRfc3095_03_vals), 0,
        "T_supportForRfc3095_03", HFILL }},
    { &hf_rrc_supported_18,
      { "supported", "rrc.supported",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_supported_10", HFILL }},
    { &hf_rrc_fddPhysChCapability_06,
      { "fddPhysChCapability", "rrc.fddPhysChCapability",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_fddPhysChCapability_06", HFILL }},
    { &hf_rrc_downlinkPhysChCapability_13,
      { "downlinkPhysChCapability", "rrc.downlinkPhysChCapability",
        FT_NONE, BASE_NONE, NULL, 0,
        "DL_PhysChCapabilityFDD_r4", HFILL }},
    { &hf_rrc_tdd384_PhysChCapability,
      { "tdd384-PhysChCapability", "rrc.tdd384_PhysChCapability",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_tdd128_PhysChCapability_01,
      { "tdd128-PhysChCapability", "rrc.tdd128_PhysChCapability",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_tdd128_PhysChCapability_01", HFILL }},
    { &hf_rrc_fddPhysChCapability_07,
      { "fddPhysChCapability", "rrc.fddPhysChCapability",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_fddPhysChCapability_07", HFILL }},
    { &hf_rrc_downlinkPhysChCapability_14,
      { "downlinkPhysChCapability", "rrc.downlinkPhysChCapability",
        FT_NONE, BASE_NONE, NULL, 0,
        "DL_PhysChCapabilityFDD_r5", HFILL }},
    { &hf_rrc_tdd384_PhysChCapability_01,
      { "tdd384-PhysChCapability", "rrc.tdd384_PhysChCapability",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_tdd384_PhysChCapability_01", HFILL }},
    { &hf_rrc_downlinkPhysChCapability_15,
      { "downlinkPhysChCapability", "rrc.downlinkPhysChCapability",
        FT_NONE, BASE_NONE, NULL, 0,
        "DL_PhysChCapabilityTDD_r5", HFILL }},
    { &hf_rrc_tdd128_PhysChCapability_02,
      { "tdd128-PhysChCapability", "rrc.tdd128_PhysChCapability",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_tdd128_PhysChCapability_02", HFILL }},
    { &hf_rrc_downlinkPhysChCapability_16,
      { "downlinkPhysChCapability", "rrc.downlinkPhysChCapability",
        FT_NONE, BASE_NONE, NULL, 0,
        "DL_PhysChCapabilityTDD_LCR_r5", HFILL }},
    { &hf_rrc_fddRF_Capability_04,
      { "fddRF-Capability", "rrc.fddRF_Capability",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_fddRF_Capability_04", HFILL }},
    { &hf_rrc_tdd384_RF_Capability_02,
      { "tdd384-RF-Capability", "rrc.tdd384_RF_Capability",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_tdd384_RF_Capability_01", HFILL }},
    { &hf_rrc_tdd128_RF_Capability_03,
      { "tdd128-RF-Capability", "rrc.tdd128_RF_Capability",
        FT_NONE, BASE_NONE, NULL, 0,
        "T_tdd128_RF_Capability_01", HFILL }},
    { &hf_rrc_rfc3095_Context_List,
      { "rfc3095-Context-List", "rrc.rfc3095_Context_List",
        FT_UINT32, BASE_DEC, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_RFC3095_Context_List_item,
      { "RFC3095-Context-List item", "rrc.RFC3095_Context_List_item",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_dl_RFC3095_Context,
      { "dl-RFC3095-Context", "rrc.dl_RFC3095_Context",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_ul_RFC3095_Context,
      { "ul-RFC3095-Context", "rrc.ul_RFC3095_Context",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_totalRLC_AM_BufferSize_05,
      { "totalRLC-AM-BufferSize", "rrc.totalRLC_AM_BufferSize",
        FT_UINT32, BASE_DEC, VALS(rrc_TotalRLC_AM_BufferSize_r5_vals), 0,
        "TotalRLC_AM_BufferSize_r5", HFILL }},
    { &hf_rrc_ul_RRC_HFN,
      { "ul-RRC-HFN", "rrc.ul_RRC_HFN",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_28", HFILL }},
    { &hf_rrc_dl_RRC_HFN,
      { "dl-RRC-HFN", "rrc.dl_RRC_HFN",
        FT_BYTES, BASE_NONE, NULL, 0,
        "BIT_STRING_SIZE_28", HFILL }},
    { &hf_rrc_ul_RRC_SequenceNumber,
      { "ul-RRC-SequenceNumber", "rrc.ul_RRC_SequenceNumber",
        FT_UINT32, BASE_DEC, NULL, 0,
        "RRC_MessageSequenceNumber", HFILL }},
    { &hf_rrc_dl_RRC_SequenceNumber,
      { "dl-RRC-SequenceNumber", "rrc.dl_RRC_SequenceNumber",
        FT_UINT32, BASE_DEC, NULL, 0,
        "RRC_MessageSequenceNumber", HFILL }},
    { &hf_rrc_SRB_SpecificIntegrityProtInfoList_item,
      { "SRB-SpecificIntegrityProtInfo", "rrc.SRB_SpecificIntegrityProtInfo",
        FT_NONE, BASE_NONE, NULL, 0,
        NULL, HFILL }},
    { &hf_rrc_validity_CellPCH_UraPCH_01,
      { "validity-CellPCH-UraPCH", "rrc.validity_CellPCH_UraPCH",
        FT_UINT32, BASE_DEC, VALS(rrc_T_validity_CellPCH_UraPCH_01_vals), 0,
        "T_validity_CellPCH_UraPCH_01", HFILL }},
    { &hf_rrc_sfn_sfnType2Capability_01,
      { "sfn-sfnType2Capability", "rrc.sfn_sfnType2Capability",
        FT_UINT32, BASE_DEC, VALS(rrc_T_sfn_sfnType2Capability_01_vals), 0,
        "T_sfn_sfnType2Capability_01", HFILL }},
    { &hf_rrc_pdcp_Capability_03,
      { "pdcp-Capability", "rrc.pdcp_Capability",
        FT_NONE, BASE_NONE, NULL, 0,
        "PDCP_Capability_r4", HFILL }},
    { &hf_rrc_rf_Capability_04,
      { "rf-Capability", "rrc.rf_Capability",
        FT_NONE, BASE_NONE, NULL, 0,
        "RF_Capability_r4", HFILL }},
    { &hf_rrc_physicalChannelCapability_08,
      { "physicalChannelCapability", "rrc.physicalChannelCapability",
        FT_NONE, BASE_NONE, NULL, 0,
        "PhysicalChannelCapability_r4", HFILL }},
    { &hf_rrc_ue_positioning_Capability_01,
      { "ue-positioning-Capability", "rrc.ue_positioning_Capability",
        FT_NONE, BASE_NONE, NULL, 0,
        "UE_Positioning_Capability_r4", HFILL }},
    { &hf_rrc_measurementCapability_06,
      { "measurementCapability", "rrc.measurementCapability",
        FT_NONE, BASE_NONE, NULL, 0,
        "MeasurementCapability_r4", HFILL }},
    { &hf_rrc_pdcp_Capability_04,
      { "pdcp-Capability", "rrc.pdcp_Capability",
        FT_NONE, BASE_NONE, NULL, 0,
        "PDCP_Capability_r5", HFILL }},
    { &hf_rrc_rlc_Capability_05,
      { "rlc-Capability", "rrc.rlc_Capability",
        FT_NONE, BASE_NONE, NULL, 0,
        "RLC_Capability_r5", HFILL }},
    { &hf_rrc_physicalChannelCapability_09,
      { "physicalChannelCapability", "rrc.physicalChannelCapability",
        FT_NONE, BASE_NONE, NULL, 0,
        "PhysicalChannelCapability_r5", HFILL }},
    { &hf_rrc_ue_MultiModeRAT_Capability_01,
      { "ue-MultiModeRAT-Capability", "rrc.ue_MultiModeRAT_Capability",
        FT_NONE, BASE_NONE, NULL, 0,
        "UE_MultiModeRAT_Capability_r5", HFILL }},
    { &hf_rrc_pdcp_Capability_05,
      { "pdcp-Capability", "rrc.pdcp_Capability",
        FT_NONE, BASE_NONE, NULL, 0,
        "PDCP_Capability_r6", HFILL }},
    { &hf_rrc_ul_mode,
      { "ul-mode", "rrc.ul_mode",
        FT_UINT32, BASE_DEC, VALS(rrc_T_ul_mode_vals), 0,
        NULL, HFILL }},
    { &hf_rrc_ul_ref_ir,
      { "ul-ref-ir", "rrc.ul_ref_ir",
        FT_BYTES, BASE_NONE, NULL, 0,
        "OCTET_STRING_SIZE_1_3000", HFILL }},
    { &hf_rrc_ul_ref_time,
      { "ul-ref-time", "rrc.ul_ref_time",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_4294967295", HFILL }},
    { &hf_rrc_ul_curr_time,
      { "ul-curr-time", "rrc.ul_curr_time",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_4294967295", HFILL }},
    { &hf_rrc_ul_syn_offset_id,
      { "ul-syn-offset-id", "rrc.ul_syn_offset_id",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_65535", HFILL }},
    { &hf_rrc_ul_syn_slope_ts,
      { "ul-syn-slope-ts", "rrc.ul_syn_slope_ts",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_4294967295", HFILL }},
    { &hf_rrc_ul_ref_sn_1,
      { "ul-ref-sn-1", "rrc.ul_ref_sn_1",
        FT_UINT32, BASE_DEC, NULL, 0,
        "INTEGER_0_65535", HFILL }},
    { &hf_rrc_T_cipheringAlgorithmCap_spare15,
      { "spare15", "rrc.spare15",
        FT_BOOLEAN, 8, NULL, 0x80,
        NULL, HFILL }},
    { &hf_rrc_T_cipheringAlgorithmCap_spare14,
      { "spare14", "rrc.spare14",
        FT_BOOLEAN, 8, NULL, 0x40,
        NULL, HFILL }},
    { &hf_rrc_T_cipheringAlgorithmCap_spare13,
      { "spare13", "rrc.spare13",
        FT_BOOLEAN, 8, NULL, 0x20,
        NULL, HFILL }},
    { &hf_rrc_T_cipheringAlgorithmCap_spare12,
      { "spare12", "rrc.spare12",
        FT_BOOLEAN, 8, NULL, 0x10,
        NULL, HFILL }},
    { &hf_rrc_T_cipheringAlgorithmCap_spare11,
      { "spare11", "rrc.spare11",
        FT_BOOLEAN, 8, NULL, 0x08,
        NULL, HFILL }},
    { &hf_rrc_T_cipheringAlgorithmCap_spare10,
      { "spare10", "rrc.spare10",
        FT_BOOLEAN, 8, NULL, 0x04,
        NULL, HFILL }},
    { &hf_rrc_T_cipheringAlgorithmCap_spare9,
      { "spare9", "rrc.spare9",
        FT_BOOLEAN, 8, NULL, 0x02,
        NULL, HFILL }},
    { &hf_rrc_T_cipheringAlgorithmCap_spare8,
      { "spare8", "rrc.spare8",
        FT_BOOLEAN, 8, NULL, 0x01,
        NULL, HFILL }},
    { &hf_rrc_T_cipheringAlgorithmCap_spare7,
      { "spare7", "rrc.spare7",
        FT_BOOLEAN, 8, NULL, 0x80,
        NULL, HFILL }},
    { &hf_rrc_T_cipheringAlgorithmCap_spare6,
      { "spare6", "rrc.spare6",
        FT_BOOLEAN, 8, NULL, 0x40,
        NULL, HFILL }},
    { &hf_rrc_T_cipheringAlgorithmCap_spare5,
      { "spare5", "rrc.spare5",
        FT_BOOLEAN, 8, NULL, 0x20,
        NULL, HFILL }},
    { &hf_rrc_T_cipheringAlgorithmCap_spare4,
      { "spare4", "rrc.spare4",
        FT_BOOLEAN, 8, NULL, 0x10,
        NULL, HFILL }},
    { &hf_rrc_T_cipheringAlgorithmCap_spare3,
      { "spare3", "rrc.spare3",
        FT_BOOLEAN, 8, NULL, 0x08,
        NULL, HFILL }},
    { &hf_rrc_T_cipheringAlgorithmCap_uea2,
      { "uea2", "rrc.uea2",
        FT_BOOLEAN, 8, NULL, 0x04,
        NULL, HFILL }},
    { &hf_rrc_T_cipheringAlgorithmCap_uea1,
      { "uea1", "rrc.uea1",
        FT_BOOLEAN, 8, NULL, 0x02,
        NULL, HFILL }},
    { &hf_rrc_T_cipheringAlgorithmCap_uea0,
      { "uea0", "rrc.uea0",
        FT_BOOLEAN, 8, NULL, 0x01,
        NULL, HFILL }},
    { &hf_rrc_T_integrityProtectionAlgorithmCap_spare15,
      { "spare15", "rrc.spare15",
        FT_BOOLEAN, 8, NULL, 0x80,
        NULL, HFILL }},
    { &hf_rrc_T_integrityProtectionAlgorithmCap_spare14,
      { "spare14", "rrc.spare14",
        FT_BOOLEAN, 8, NULL, 0x40,
        NULL, HFILL }},
    { &hf_rrc_T_integrityProtectionAlgorithmCap_spare13,
      { "spare13", "rrc.spare13",
        FT_BOOLEAN, 8, NULL, 0x20,
        NULL, HFILL }},
    { &hf_rrc_T_integrityProtectionAlgorithmCap_spare12,
      { "spare12", "rrc.spare12",
        FT_BOOLEAN, 8, NULL, 0x10,
        NULL, HFILL }},
    { &hf_rrc_T_integrityProtectionAlgorithmCap_spare11,
      { "spare11", "rrc.spare11",
        FT_BOOLEAN, 8, NULL, 0x08,
        NULL, HFILL }},
    { &hf_rrc_T_integrityProtectionAlgorithmCap_spare10,
      { "spare10", "rrc.spare10",
        FT_BOOLEAN, 8, NULL, 0x04,
        NULL, HFILL }},
    { &hf_rrc_T_integrityProtectionAlgorithmCap_spare9,
      { "spare9", "rrc.spare9",
        FT_BOOLEAN, 8, NULL, 0x02,
        NULL, HFILL }},
    { &hf_rrc_T_integrityProtectionAlgorithmCap_spare8,
      { "spare8", "rrc.spare8",
        FT_BOOLEAN, 8, NULL, 0x01,
        NULL, HFILL }},
    { &hf_rrc_T_integrityProtectionAlgorithmCap_spare7,
      { "spare7", "rrc.spare7",
        FT_BOOLEAN, 8, NULL, 0x80,
        NULL, HFILL }},
    { &hf_rrc_T_integrityProtectionAlgorithmCap_spare6,
      { "spare6", "rrc.spare6",
        FT_BOOLEAN, 8, NULL, 0x40,
        NULL, HFILL }},
    { &hf_rrc_T_integrityProtectionAlgorithmCap_spare5,
      { "spare5", "rrc.spare5",
        FT_BOOLEAN, 8, NULL, 0x20,
        NULL, HFILL }},
    { &hf_rrc_T_integrityProtectionAlgorithmCap_spare4,
      { "spare4", "rrc.spare4",
        FT_BOOLEAN, 8, NULL, 0x10,
        NULL, HFILL }},
    { &hf_rrc_T_integrityProtectionAlgorithmCap_spare3,
      { "spare3", "rrc.spare3",
        FT_BOOLEAN, 8, NULL, 0x08,
        NULL, HFILL }},
    { &hf_rrc_T_integrityProtectionAlgorithmCap_uia2,
      { "uia2", "rrc.uia2",
        FT_BOOLEAN, 8, NULL, 0x04,
        NULL, HFILL }},
    { &hf_rrc_T_integrityProtectionAlgorithmCap_uia1,
      { "uia1", "rrc.uia1",
        FT_BOOLEAN, 8, NULL, 0x02,
        NULL, HFILL }},
    { &hf_rrc_T_integrityProtectionAlgorithmCap_spare0,
      { "spare0", "rrc.spare0",
        FT_BOOLEAN, 8, NULL, 0x01,
        NULL, HFILL }},
    { &hf_rrc_T_assignedSubChannelNumber_b3,
      { "b3", "rrc.b3",
        FT_BOOLEAN, 8, NULL, 0x80,
        NULL, HFILL }},
    { &hf_rrc_T_assignedSubChannelNumber_b2,
      { "b2", "rrc.b2",
        FT_BOOLEAN, 8, NULL, 0x40,
        NULL, HFILL }},
    { &hf_rrc_T_assignedSubChannelNumber_b1,
      { "b1", "rrc.b1",
        FT_BOOLEAN, 8, NULL, 0x20,
        NULL, HFILL }},
    { &hf_rrc_T_assignedSubChannelNumber_b0,
      { "b0", "rrc.b0",
        FT_BOOLEAN, 8, NULL, 0x10,
        NULL, HFILL }},
    { &hf_rrc_T_channelisationCodeIndices_chCodeIndex7,
      { "chCodeIndex7", "rrc.chCodeIndex7",
        FT_BOOLEAN, 8, NULL, 0x80,
        NULL, HFILL }},
    { &hf_rrc_T_channelisationCodeIndices_chCodeIndex6,
      { "chCodeIndex6", "rrc.chCodeIndex6",
        FT_BOOLEAN, 8, NULL, 0x40,
        NULL, HFILL }},
    { &hf_rrc_T_channelisationCodeIndices_chCodeIndex5,
      { "chCodeIndex5", "rrc.chCodeIndex5",
        FT_BOOLEAN, 8, NULL, 0x20,
        NULL, HFILL }},
    { &hf_rrc_T_channelisationCodeIndices_chCodeIndex4,
      { "chCodeIndex4", "rrc.chCodeIndex4",
        FT_BOOLEAN, 8, NULL, 0x10,
        NULL, HFILL }},
    { &hf_rrc_T_channelisationCodeIndices_chCodeIndex3,
      { "chCodeIndex3", "rrc.chCodeIndex3",
        FT_BOOLEAN, 8, NULL, 0x08,
        NULL, HFILL }},
    { &hf_rrc_T_channelisationCodeIndices_chCodeIndex2,
      { "chCodeIndex2", "rrc.chCodeIndex2",
        FT_BOOLEAN, 8, NULL, 0x04,
        NULL, HFILL }},
    { &hf_rrc_T_channelisationCodeIndices_chCodeIndex1,
      { "chCodeIndex1", "rrc.chCodeIndex1",
        FT_BOOLEAN, 8, NULL, 0x02,
        NULL, HFILL }},
    { &hf_rrc_T_channelisationCodeIndices_chCodeIndex0,
      { "chCodeIndex0", "rrc.chCodeIndex0",
        FT_BOOLEAN, 8, NULL, 0x01,
        NULL, HFILL }},
    { &hf_rrc_T_subchannels_01_subCh3,
      { "subCh3", "rrc.subCh3",
        FT_BOOLEAN, 8, NULL, 0x80,
        NULL, HFILL }},
    { &hf_rrc_T_subchannels_01_subCh2,
      { "subCh2", "rrc.subCh2",
        FT_BOOLEAN, 8, NULL, 0x40,
        NULL, HFILL }},
    { &hf_rrc_T_subchannels_01_subCh1,
      { "subCh1", "rrc.subCh1",
        FT_BOOLEAN, 8, NULL, 0x20,
        NULL, HFILL }},
    { &hf_rrc_T_subchannels_01_subCh0,
      { "subCh0", "rrc.subCh0",
        FT_BOOLEAN, 8, NULL, 0x10,
        NULL, HFILL }},
    { &hf_rrc_T_subchannels_02_subCh7,
      { "subCh7", "rrc.subCh7",
        FT_BOOLEAN, 8, NULL, 0x80,
        NULL, HFILL }},
    { &hf_rrc_T_subchannels_02_subCh6,
      { "subCh6", "rrc.subCh6",
        FT_BOOLEAN, 8, NULL, 0x40,
        NULL, HFILL }},
    { &hf_rrc_T_subchannels_02_subCh5,
      { "subCh5", "rrc.subCh5",
        FT_BOOLEAN, 8, NULL, 0x20,
        NULL, HFILL }},
    { &hf_rrc_T_subchannels_02_subCh4,
      { "subCh4", "rrc.subCh4",
        FT_BOOLEAN, 8, NULL, 0x10,
        NULL, HFILL }},
    { &hf_rrc_T_subchannels_02_subCh3,
      { "subCh3", "rrc.subCh3",
        FT_BOOLEAN, 8, NULL, 0x08,
        NULL, HFILL }},
    { &hf_rrc_T_subchannels_02_subCh2,
      { "subCh2", "rrc.subCh2",
        FT_BOOLEAN, 8, NULL, 0x04,
        NULL, HFILL }},
    { &hf_rrc_T_subchannels_02_subCh1,
      { "subCh1", "rrc.subCh1",
        FT_BOOLEAN, 8, NULL, 0x02,
        NULL, HFILL }},
    { &hf_rrc_T_subchannels_02_subCh0,
      { "subCh0", "rrc.subCh0",
        FT_BOOLEAN, 8, NULL, 0x01,
        NULL, HFILL }},
    { &hf_rrc_T_channelisationCodeIndices_01_chCodeIndex15,
      { "chCodeIndex15", "rrc.chCodeIndex15",
        FT_BOOLEAN, 8, NULL, 0x80,
        NULL, HFILL }},
    { &hf_rrc_T_channelisationCodeIndices_01_chCodeIndex14,
      { "chCodeIndex14", "rrc.chCodeIndex14",
        FT_BOOLEAN, 8, NULL, 0x40,
        NULL, HFILL }},
    { &hf_rrc_T_channelisationCodeIndices_01_chCodeIndex13,
      { "chCodeIndex13", "rrc.chCodeIndex13",
        FT_BOOLEAN, 8, NULL, 0x20,
        NULL, HFILL }},
    { &hf_rrc_T_channelisationCodeIndices_01_chCodeIndex12,
      { "chCodeIndex12", "rrc.chCodeIndex12",
        FT_BOOLEAN, 8, NULL, 0x10,
        NULL, HFILL }},
    { &hf_rrc_T_channelisationCodeIndices_01_chCodeIndex11,
      { "chCodeIndex11", "rrc.chCodeIndex11",
        FT_BOOLEAN, 8, NULL, 0x08,
        NULL, HFILL }},
    { &hf_rrc_T_channelisationCodeIndices_01_chCodeIndex10,
      { "chCodeIndex10", "rrc.chCodeIndex10",
        FT_BOOLEAN, 8, NULL, 0x04,
        NULL, HFILL }},
    { &hf_rrc_T_channelisationCodeIndices_01_chCodeIndex9,
      { "chCodeIndex9", "rrc.chCodeIndex9",
        FT_BOOLEAN, 8, NULL, 0x02,
        NULL, HFILL }},
    { &hf_rrc_T_channelisationCodeIndices_01_chCodeIndex8,
      { "chCodeIndex8", "rrc.chCodeIndex8",
        FT_BOOLEAN, 8, NULL, 0x01,
        NULL, HFILL }},
    { &hf_rrc_T_channelisationCodeIndices_01_chCodeIndex7,
      { "chCodeIndex7", "rrc.chCodeIndex7",
        FT_BOOLEAN, 8, NULL, 0x80,
        NULL, HFILL }},
    { &hf_rrc_T_channelisationCodeIndices_01_chCodeIndex6,
      { "chCodeIndex6", "rrc.chCodeIndex6",
        FT_BOOLEAN, 8, NULL, 0x40,
        NULL, HFILL }},
    { &hf_rrc_T_channelisationCodeIndices_01_chCodeIndex5,
      { "chCodeIndex5", "rrc.chCodeIndex5",
        FT_BOOLEAN, 8, NULL, 0x20,
        NULL, HFILL }},
    { &hf_rrc_T_channelisationCodeIndices_01_chCodeIndex4,
      { "chCodeIndex4", "rrc.chCodeIndex4",
        FT_BOOLEAN, 8, NULL, 0x10,
        NULL, HFILL }},
    { &hf_rrc_T_channelisationCodeIndices_01_chCodeIndex3,
      { "chCodeIndex3", "rrc.chCodeIndex3",
        FT_BOOLEAN, 8, NULL, 0x08,
        NULL, HFILL }},
    { &hf_rrc_T_channelisationCodeIndices_01_chCodeIndex2,
      { "chCodeIndex2", "rrc.chCodeIndex2",
        FT_BOOLEAN, 8, NULL, 0x04,
        NULL, HFILL }},
    { &hf_rrc_T_channelisationCodeIndices_01_chCodeIndex1,
      { "chCodeIndex1", "rrc.chCodeIndex1",
        FT_BOOLEAN, 8, NULL, 0x02,
        NULL, HFILL }},
    { &hf_rrc_T_channelisationCodeIndices_01_chCodeIndex0,
      { "chCodeIndex0", "rrc.chCodeIndex0",
        FT_BOOLEAN, 8, NULL, 0x01,
        NULL, HFILL }},
    { &hf_rrc_T_subchannels_04_subCh3,
      { "subCh3", "rrc.subCh3",
        FT_BOOLEAN, 8, NULL, 0x80,
        NULL, HFILL }},
    { &hf_rrc_T_subchannels_04_subCh2,
      { "subCh2", "rrc.subCh2",
        FT_BOOLEAN, 8, NULL, 0x40,
        NULL, HFILL }},
    { &hf_rrc_T_subchannels_04_subCh1,
      { "subCh1", "rrc.subCh1",
        FT_BOOLEAN, 8, NULL, 0x20,
        NULL, HFILL }},
    { &hf_rrc_T_subchannels_04_subCh0,
      { "subCh0", "rrc.subCh0",
        FT_BOOLEAN, 8, NULL, 0x10,
        NULL, HFILL }},
    { &hf_rrc_T_subchannels_05_subCh7,
      { "subCh7", "rrc.subCh7",
        FT_BOOLEAN, 8, NULL, 0x80,
        NULL, HFILL }},
    { &hf_rrc_T_subchannels_05_subCh6,
      { "subCh6", "rrc.subCh6",
        FT_BOOLEAN, 8, NULL, 0x40,
        NULL, HFILL }},
    { &hf_rrc_T_subchannels_05_subCh5,
      { "subCh5", "rrc.subCh5",
        FT_BOOLEAN, 8, NULL, 0x20,
        NULL, HFILL }},
    { &hf_rrc_T_subchannels_05_subCh4,
      { "subCh4", "rrc.subCh4",
        FT_BOOLEAN, 8, NULL, 0x10,
        NULL, HFILL }},
    { &hf_rrc_T_subchannels_05_subCh3,
      { "subCh3", "rrc.subCh3",
        FT_BOOLEAN, 8, NULL, 0x08,
        NULL, HFILL }},
    { &hf_rrc_T_subchannels_05_subCh2,
      { "subCh2", "rrc.subCh2",
        FT_BOOLEAN, 8, NULL, 0x04,
        NULL, HFILL }},
    { &hf_rrc_T_subchannels_05_subCh1,
      { "subCh1", "rrc.subCh1",
        FT_BOOLEAN, 8, NULL, 0x02,
        NULL, HFILL }},
    { &hf_rrc_T_subchannels_05_subCh0,
      { "subCh0", "rrc.subCh0",
        FT_BOOLEAN, 8, NULL, 0x01,
        NULL, HFILL }},
    { &hf_rrc_T_subchannels_06_subCh15,
      { "subCh15", "rrc.subCh15",
        FT_BOOLEAN, 8, NULL, 0x80,
        NULL, HFILL }},
    { &hf_rrc_T_subchannels_06_subCh14,
      { "subCh14", "rrc.subCh14",
        FT_BOOLEAN, 8, NULL, 0x40,
        NULL, HFILL }},
    { &hf_rrc_T_subchannels_06_subCh13,
      { "subCh13", "rrc.subCh13",
        FT_BOOLEAN, 8, NULL, 0x20,
        NULL, HFILL }},
    { &hf_rrc_T_subchannels_06_subCh12,
      { "subCh12", "rrc.subCh12",
        FT_BOOLEAN, 8, NULL, 0x10,
        NULL, HFILL }},
    { &hf_rrc_T_subchannels_06_subCh11,
      { "subCh11", "rrc.subCh11",
        FT_BOOLEAN, 8, NULL, 0x08,
        NULL, HFILL }},
    { &hf_rrc_T_subchannels_06_subCh10,
      { "subCh10", "rrc.subCh10",
        FT_BOOLEAN, 8, NULL, 0x04,
        NULL, HFILL }},
    { &hf_rrc_T_subchannels_06_subCh9,
      { "subCh9", "rrc.subCh9",
        FT_BOOLEAN, 8, NULL, 0x02,
        NULL, HFILL }},
    { &hf_rrc_T_subchannels_06_subCh8,
      { "subCh8", "rrc.subCh8",
        FT_BOOLEAN, 8, NULL, 0x01,
        NULL, HFILL }},
    { &hf_rrc_T_subchannels_06_subCh7,
      { "subCh7", "rrc.subCh7",
        FT_BOOLEAN, 8, NULL, 0x80,
        NULL, HFILL }},
    { &hf_rrc_T_subchannels_06_subCh6,
      { "subCh6", "rrc.subCh6",
        FT_BOOLEAN, 8, NULL, 0x40,
        NULL, HFILL }},
    { &hf_rrc_T_subchannels_06_subCh5,
      { "subCh5", "rrc.subCh5",
        FT_BOOLEAN, 8, NULL, 0x20,
        NULL, HFILL }},
    { &hf_rrc_T_subchannels_06_subCh4,
      { "subCh4", "rrc.subCh4",
        FT_BOOLEAN, 8, NULL, 0x10,
        NULL, HFILL }},
    { &hf_rrc_T_subchannels_06_subCh3,
      { "subCh3", "rrc.subCh3",
        FT_BOOLEAN, 8, NULL, 0x08,
        NULL, HFILL }},
    { &hf_rrc_T_subchannels_06_subCh2,
      { "subCh2", "rrc.subCh2",
        FT_BOOLEAN, 8, NULL, 0x04,
        NULL, HFILL }},
    { &hf_rrc_T_subchannels_06_subCh1,
      { "subCh1", "rrc.subCh1",
        FT_BOOLEAN, 8, NULL, 0x02,
        NULL, HFILL }},
    { &hf_rrc_T_subchannels_06_subCh0,
      { "subCh0", "rrc.subCh0",
        FT_BOOLEAN, 8, NULL, 0x01,
        NULL, HFILL }},
    { &hf_rrc_T_availableSYNC_UlCodesIndics_sulCodeIndex7,
      { "sulCodeIndex7", "rrc.sulCodeIndex7",
        FT_BOOLEAN, 8, NULL, 0x80,
        NULL, HFILL }},
    { &hf_rrc_T_availableSYNC_UlCodesIndics_sulCodeIndex6,
      { "sulCodeIndex6", "rrc.sulCodeIndex6",
        FT_BOOLEAN, 8, NULL, 0x40,
        NULL, HFILL }},
    { &hf_rrc_T_availableSYNC_UlCodesIndics_sulCodeIndex5,
      { "sulCodeIndex5", "rrc.sulCodeIndex5",
        FT_BOOLEAN, 8, NULL, 0x20,
        NULL, HFILL }},
    { &hf_rrc_T_availableSYNC_UlCodesIndics_sulCodeIndex4,
      { "sulCodeIndex4", "rrc.sulCodeIndex4",
        FT_BOOLEAN, 8, NULL, 0x10,
        NULL, HFILL }},
    { &hf_rrc_T_availableSYNC_UlCodesIndics_sulCodeIndex3,
      { "sulCodeIndex3", "rrc.sulCodeIndex3",
        FT_BOOLEAN, 8, NULL, 0x08,
        NULL, HFILL }},
    { &hf_rrc_T_availableSYNC_UlCodesIndics_sulCodeIndex2,
      { "sulCodeIndex2", "rrc.sulCodeIndex2",
        FT_BOOLEAN, 8, NULL, 0x04,
        NULL, HFILL }},
    { &hf_rrc_T_availableSYNC_UlCodesIndics_sulCodeIndex1,
      { "sulCodeIndex1", "rrc.sulCodeIndex1",
        FT_BOOLEAN, 8, NULL, 0x02,
        NULL, HFILL }},
    { &hf_rrc_T_availableSYNC_UlCodesIndics_sulCodeIndex0,
      { "sulCodeIndex0", "rrc.sulCodeIndex0",
        FT_BOOLEAN, 8, NULL, 0x01,
        NULL, HFILL }},
    { &hf_rrc_T_subchannels_08_subCh3,
      { "subCh3", "rrc.subCh3",
        FT_BOOLEAN, 8, NULL, 0x80,
        NULL, HFILL }},
    { &hf_rrc_T_subchannels_08_subCh2,
      { "subCh2", "rrc.subCh2",
        FT_BOOLEAN, 8, NULL, 0x40,
        NULL, HFILL }},
    { &hf_rrc_T_subchannels_08_subCh1,
      { "subCh1", "rrc.subCh1",
        FT_BOOLEAN, 8, NULL, 0x20,
        NULL, HFILL }},
    { &hf_rrc_T_subchannels_08_subCh0,
      { "subCh0", "rrc.subCh0",
        FT_BOOLEAN, 8, NULL, 0x10,
        NULL, HFILL }},
    { &hf_rrc_T_subchannels_09_subCh7,
      { "subCh7", "rrc.subCh7",
        FT_BOOLEAN, 8, NULL, 0x80,
        NULL, HFILL }},
    { &hf_rrc_T_subchannels_09_subCh6,
      { "subCh6", "rrc.subCh6",
        FT_BOOLEAN, 8, NULL, 0x40,
        NULL, HFILL }},
    { &hf_rrc_T_subchannels_09_subCh5,
      { "subCh5", "rrc.subCh5",
        FT_BOOLEAN, 8, NULL, 0x20,
        NULL, HFILL }},
    { &hf_rrc_T_subchannels_09_subCh4,
      { "subCh4", "rrc.subCh4",
        FT_BOOLEAN, 8, NULL, 0x10,
        NULL, HFILL }},
    { &hf_rrc_T_subchannels_09_subCh3,
      { "subCh3", "rrc.subCh3",
        FT_BOOLEAN, 8, NULL, 0x08,
        NULL, HFILL }},
    { &hf_rrc_T_subchannels_09_subCh2,
      { "subCh2", "rrc.subCh2",
        FT_BOOLEAN, 8, NULL, 0x04,
        NULL, HFILL }},
    { &hf_rrc_T_subchannels_09_subCh1,
      { "subCh1", "rrc.subCh1",
        FT_BOOLEAN, 8, NULL, 0x02,
        NULL, HFILL }},
    { &hf_rrc_T_subchannels_09_subCh0,
      { "subCh0", "rrc.subCh0",
        FT_BOOLEAN, 8, NULL, 0x01,
        NULL, HFILL }},
    { &hf_rrc_AvailableSignatures_signature15,
      { "signature15", "rrc.signature15",
        FT_BOOLEAN, 8, NULL, 0x80,
        NULL, HFILL }},
    { &hf_rrc_AvailableSignatures_signature14,
      { "signature14", "rrc.signature14",
        FT_BOOLEAN, 8, NULL, 0x40,
        NULL, HFILL }},
    { &hf_rrc_AvailableSignatures_signature13,
      { "signature13", "rrc.signature13",
        FT_BOOLEAN, 8, NULL, 0x20,
        NULL, HFILL }},
    { &hf_rrc_AvailableSignatures_signature12,
      { "signature12", "rrc.signature12",
        FT_BOOLEAN, 8, NULL, 0x10,
        NULL, HFILL }},
    { &hf_rrc_AvailableSignatures_signature11,
      { "signature11", "rrc.signature11",
        FT_BOOLEAN, 8, NULL, 0x08,
        NULL, HFILL }},
    { &hf_rrc_AvailableSignatures_signature10,
      { "signature10", "rrc.signature10",
        FT_BOOLEAN, 8, NULL, 0x04,
        NULL, HFILL }},
    { &hf_rrc_AvailableSignatures_signature9,
      { "signature9", "rrc.signature9",
        FT_BOOLEAN, 8, NULL, 0x02,
        NULL, HFILL }},
    { &hf_rrc_AvailableSignatures_signature8,
      { "signature8", "rrc.signature8",
        FT_BOOLEAN, 8, NULL, 0x01,
        NULL, HFILL }},
    { &hf_rrc_AvailableSignatures_signature7,
      { "signature7", "rrc.signature7",
        FT_BOOLEAN, 8, NULL, 0x80,
        NULL, HFILL }},
    { &hf_rrc_AvailableSignatures_signature6,
      { "signature6", "rrc.signature6",
        FT_BOOLEAN, 8, NULL, 0x40,
        NULL, HFILL }},
    { &hf_rrc_AvailableSignatures_signature5,
      { "signature5", "rrc.signature5",
        FT_BOOLEAN, 8, NULL, 0x20,
        NULL, HFILL }},
    { &hf_rrc_AvailableSignatures_signature4,
      { "signature4", "rrc.signature4",
        FT_BOOLEAN, 8, NULL, 0x10,
        NULL, HFILL }},
    { &hf_rrc_AvailableSignatures_signature3,
      { "signature3", "rrc.signature3",
        FT_BOOLEAN, 8, NULL, 0x08,
        NULL, HFILL }},
    { &hf_rrc_AvailableSignatures_signature2,
      { "signature2", "rrc.signature2",
        FT_BOOLEAN, 8, NULL, 0x04,
        NULL, HFILL }},
    { &hf_rrc_AvailableSignatures_signature1,
      { "signature1", "rrc.signature1",
        FT_BOOLEAN, 8, NULL, 0x02,
        NULL, HFILL }},
    { &hf_rrc_AvailableSignatures_signature0,
      { "signature0", "rrc.signature0",
        FT_BOOLEAN, 8, NULL, 0x01,
        NULL, HFILL }},
    { &hf_rrc_AvailableSubChannelNumbers_subCh11,
      { "subCh11", "rrc.subCh11",
        FT_BOOLEAN, 8, NULL, 0x80,
        NULL, HFILL }},
    { &hf_rrc_AvailableSubChannelNumbers_subCh10,
      { "subCh10", "rrc.subCh10",
        FT_BOOLEAN, 8, NULL, 0x40,
        NULL, HFILL }},
    { &hf_rrc_AvailableSubChannelNumbers_subCh9,
      { "subCh9", "rrc.subCh9",
        FT_BOOLEAN, 8, NULL, 0x20,
        NULL, HFILL }},
    { &hf_rrc_AvailableSubChannelNumbers_subCh8,
      { "subCh8", "rrc.subCh8",
        FT_BOOLEAN, 8, NULL, 0x10,
        NULL, HFILL }},
    { &hf_rrc_AvailableSubChannelNumbers_subCh7,
      { "subCh7", "rrc.subCh7",
        FT_BOOLEAN, 8, NULL, 0x08,
        NULL, HFILL }},
    { &hf_rrc_AvailableSubChannelNumbers_subCh6,
      { "subCh6", "rrc.subCh6",
        FT_BOOLEAN, 8, NULL, 0x04,
        NULL, HFILL }},
    { &hf_rrc_AvailableSubChannelNumbers_subCh5,
      { "subCh5", "rrc.subCh5",
        FT_BOOLEAN, 8, NULL, 0x02,
        NULL, HFILL }},
    { &hf_rrc_AvailableSubChannelNumbers_subCh4,
      { "subCh4", "rrc.subCh4",
        FT_BOOLEAN, 8, NULL, 0x01,
        NULL, HFILL }},
    { &hf_rrc_AvailableSubChannelNumbers_subCh3,
      { "subCh3", "rrc.subCh3",
        FT_BOOLEAN, 8, NULL, 0x80,
        NULL, HFILL }},
    { &hf_rrc_AvailableSubChannelNumbers_subCh2,
      { "subCh2", "rrc.subCh2",
        FT_BOOLEAN, 8, NULL, 0x40,
        NULL, HFILL }},
    { &hf_rrc_AvailableSubChannelNumbers_subCh1,
      { "subCh1", "rrc.subCh1",
        FT_BOOLEAN, 8, NULL, 0x20,
        NULL, HFILL }},
    { &hf_rrc_AvailableSubChannelNumbers_subCh0,
      { "subCh0", "rrc.subCh0",
        FT_BOOLEAN, 8, NULL, 0x10,
        NULL, HFILL }},
    { &hf_rrc_T_bitmap_chCode16_SF16,
      { "chCode16-SF16", "rrc.chCode16-SF16",
        FT_BOOLEAN, 8, NULL, 0x80,
        NULL, HFILL }},
    { &hf_rrc_T_bitmap_chCode15_SF16,
      { "chCode15-SF16", "rrc.chCode15-SF16",
        FT_BOOLEAN, 8, NULL, 0x40,
        NULL, HFILL }},
    { &hf_rrc_T_bitmap_chCode14_SF16,
      { "chCode14-SF16", "rrc.chCode14-SF16",
        FT_BOOLEAN, 8, NULL, 0x20,
        NULL, HFILL }},
    { &hf_rrc_T_bitmap_chCode13_SF16,
      { "chCode13-SF16", "rrc.chCode13-SF16",
        FT_BOOLEAN, 8, NULL, 0x10,
        NULL, HFILL }},
    { &hf_rrc_T_bitmap_chCode12_SF16,
      { "chCode12-SF16", "rrc.chCode12-SF16",
        FT_BOOLEAN, 8, NULL, 0x08,
        NULL, HFILL }},
    { &hf_rrc_T_bitmap_chCode11_SF16,
      { "chCode11-SF16", "rrc.chCode11-SF16",
        FT_BOOLEAN, 8, NULL, 0x04,
        NULL, HFILL }},
    { &hf_rrc_T_bitmap_chCode10_SF16,
      { "chCode10-SF16", "rrc.chCode10-SF16",
        FT_BOOLEAN, 8, NULL, 0x02,
        NULL, HFILL }},
    { &hf_rrc_T_bitmap_chCode9_SF16,
      { "chCode9-SF16", "rrc.chCode9-SF16",
        FT_BOOLEAN, 8, NULL, 0x01,
        NULL, HFILL }},
    { &hf_rrc_T_bitmap_chCode8_SF16,
      { "chCode8-SF16", "rrc.chCode8-SF16",
        FT_BOOLEAN, 8, NULL, 0x80,
        NULL, HFILL }},
    { &hf_rrc_T_bitmap_chCode7_SF16,
      { "chCode7-SF16", "rrc.chCode7-SF16",
        FT_BOOLEAN, 8, NULL, 0x40,
        NULL, HFILL }},
    { &hf_rrc_T_bitmap_chCode6_SF16,
      { "chCode6-SF16", "rrc.chCode6-SF16",
        FT_BOOLEAN, 8, NULL, 0x20,
        NULL, HFILL }},
    { &hf_rrc_T_bitmap_chCode5_SF16,
      { "chCode5-SF16", "rrc.chCode5-SF16",
        FT_BOOLEAN, 8, NULL, 0x10,
        NULL, HFILL }},
    { &hf_rrc_T_bitmap_chCode4_SF16,
      { "chCode4-SF16", "rrc.chCode4-SF16",
        FT_BOOLEAN, 8, NULL, 0x08,
        NULL, HFILL }},
    { &hf_rrc_T_bitmap_chCode3_SF16,
      { "chCode3-SF16", "rrc.chCode3-SF16",
        FT_BOOLEAN, 8, NULL, 0x04,
        NULL, HFILL }},
    { &hf_rrc_T_bitmap_chCode2_SF16,
      { "chCode2-SF16", "rrc.chCode2-SF16",
        FT_BOOLEAN, 8, NULL, 0x02,
        NULL, HFILL }},
    { &hf_rrc_T_bitmap_chCode1_SF16,
      { "chCode1-SF16", "rrc.chCode1-SF16",
        FT_BOOLEAN, 8, NULL, 0x01,
        NULL, HFILL }},
    { &hf_rrc_T_bitmap_01_chCode32_SF32,
      { "chCode32-SF32", "rrc.chCode32-SF32",
        FT_BOOLEAN, 8, NULL, 0x80,
        NULL, HFILL }},
    { &hf_rrc_T_bitmap_01_chCode31_SF32,
      { "chCode31-SF32", "rrc.chCode31-SF32",
        FT_BOOLEAN, 8, NULL, 0x40,
        NULL, HFILL }},
    { &hf_rrc_T_bitmap_01_chCode30_SF32,
      { "chCode30-SF32", "rrc.chCode30-SF32",
        FT_BOOLEAN, 8, NULL, 0x20,
        NULL, HFILL }},
    { &hf_rrc_T_bitmap_01_chCode29_SF32,
      { "chCode29-SF32", "rrc.chCode29-SF32",
        FT_BOOLEAN, 8, NULL, 0x10,
        NULL, HFILL }},
    { &hf_rrc_T_bitmap_01_chCode28_SF32,
      { "chCode28-SF32", "rrc.chCode28-SF32",
        FT_BOOLEAN, 8, NULL, 0x08,
        NULL, HFILL }},
    { &hf_rrc_T_bitmap_01_chCode27_SF32,
      { "chCode27-SF32", "rrc.chCode27-SF32",
        FT_BOOLEAN, 8, NULL, 0x04,
        NULL, HFILL }},
    { &hf_rrc_T_bitmap_01_chCode26_SF32,
      { "chCode26-SF32", "rrc.chCode26-SF32",
        FT_BOOLEAN, 8, NULL, 0x02,
        NULL, HFILL }},
    { &hf_rrc_T_bitmap_01_chCode25_SF32,
      { "chCode25-SF32", "rrc.chCode25-SF32",
        FT_BOOLEAN, 8, NULL, 0x01,
        NULL, HFILL }},
    { &hf_rrc_T_bitmap_01_chCode24_SF32,
      { "chCode24-SF32", "rrc.chCode24-SF32",
        FT_BOOLEAN, 8, NULL, 0x80,
        NULL, HFILL }},
    { &hf_rrc_T_bitmap_01_chCode23_SF32,
      { "chCode23-SF32", "rrc.chCode23-SF32",
        FT_BOOLEAN, 8, NULL, 0x40,
        NULL, HFILL }},
    { &hf_rrc_T_bitmap_01_chCode22_SF32,
      { "chCode22-SF32", "rrc.chCode22-SF32",
        FT_BOOLEAN, 8, NULL, 0x20,
        NULL, HFILL }},
    { &hf_rrc_T_bitmap_01_chCode21_SF32,
      { "chCode21-SF32", "rrc.chCode21-SF32",
        FT_BOOLEAN, 8, NULL, 0x10,
        NULL, HFILL }},
    { &hf_rrc_T_bitmap_01_chCode20_SF32,
      { "chCode20-SF32", "rrc.chCode20-SF32",
        FT_BOOLEAN, 8, NULL, 0x08,
        NULL, HFILL }},
    { &hf_rrc_T_bitmap_01_chCode19_SF32,
      { "chCode19-SF32", "rrc.chCode19-SF32",
        FT_BOOLEAN, 8, NULL, 0x04,
        NULL, HFILL }},
    { &hf_rrc_T_bitmap_01_chCode18_SF32,
      { "chCode18-SF32", "rrc.chCode18-SF32",
        FT_BOOLEAN, 8, NULL, 0x02,
        NULL, HFILL }},
    { &hf_rrc_T_bitmap_01_chCode17_SF32,
      { "chCode17-SF32", "rrc.chCode17-SF32",
        FT_BOOLEAN, 8, NULL, 0x01,
        NULL, HFILL }},
    { &hf_rrc_T_bitmap_01_chCode16_SF32,
      { "chCode16-SF32", "rrc.chCode16-SF32",
        FT_BOOLEAN, 8, NULL, 0x80,
        NULL, HFILL }},
    { &hf_rrc_T_bitmap_01_chCode15_SF32,
      { "chCode15-SF32", "rrc.chCode15-SF32",
        FT_BOOLEAN, 8, NULL, 0x40,
        NULL, HFILL }},
    { &hf_rrc_T_bitmap_01_chCode14_SF32,
      { "chCode14-SF32", "rrc.chCode14-SF32",
        FT_BOOLEAN, 8, NULL, 0x20,
        NULL, HFILL }},
    { &hf_rrc_T_bitmap_01_chCode13_SF32,
      { "chCode13-SF32", "rrc.chCode13-SF32",
        FT_BOOLEAN, 8, NULL, 0x10,
        NULL, HFILL }},
    { &hf_rrc_T_bitmap_01_chCode12_SF32,
      { "chCode12-SF32", "rrc.chCode12-SF32",
        FT_BOOLEAN, 8, NULL, 0x08,
        NULL, HFILL }},
    { &hf_rrc_T_bitmap_01_chCode11_SF32,
      { "chCode11-SF32", "rrc.chCode11-SF32",
        FT_BOOLEAN, 8, NULL, 0x04,
        NULL, HFILL }},
    { &hf_rrc_T_bitmap_01_chCode10_SF32,
      { "chCode10-SF32", "rrc.chCode10-SF32",
        FT_BOOLEAN, 8, NULL, 0x02,
        NULL, HFILL }},
    { &hf_rrc_T_bitmap_01_chCode9_SF32,
      { "chCode9-SF32", "rrc.chCode9-SF32",
        FT_BOOLEAN, 8, NULL, 0x01,
        NULL, HFILL }},
    { &hf_rrc_T_bitmap_01_chCode8_SF32,
      { "chCode8-SF32", "rrc.chCode8-SF32",
        FT_BOOLEAN, 8, NULL, 0x80,
        NULL, HFILL }},
    { &hf_rrc_T_bitmap_01_chCode7_SF32,
      { "chCode7-SF32", "rrc.chCode7-SF32",
        FT_BOOLEAN, 8, NULL, 0x40,
        NULL, HFILL }},
    { &hf_rrc_T_bitmap_01_chCode6_SF32,
      { "chCode6-SF32", "rrc.chCode6-SF32",
        FT_BOOLEAN, 8, NULL, 0x20,
        NULL, HFILL }},
    { &hf_rrc_T_bitmap_01_chCode5_SF32,
      { "chCode5-SF32", "rrc.chCode5-SF32",
        FT_BOOLEAN, 8, NULL, 0x10,
        NULL, HFILL }},
    { &hf_rrc_T_bitmap_01_chCode4_SF32,
      { "chCode4-SF32", "rrc.chCode4-SF32",
        FT_BOOLEAN, 8, NULL, 0x08,
        NULL, HFILL }},
    { &hf_rrc_T_bitmap_01_chCode3_SF32,
      { "chCode3-SF32", "rrc.chCode3-SF32",
        FT_BOOLEAN, 8, NULL, 0x04,
        NULL, HFILL }},
    { &hf_rrc_T_bitmap_01_chCode2_SF32,
      { "chCode2-SF32", "rrc.chCode2-SF32",
        FT_BOOLEAN, 8, NULL, 0x02,
        NULL, HFILL }},
    { &hf_rrc_T_bitmap_01_chCode1_SF32,
      { "chCode1-SF32", "rrc.chCode1-SF32",
        FT_BOOLEAN, 8, NULL, 0x01,
        NULL, HFILL }},
    { &hf_rrc_T_sync_UL_CodesBitmap_code7,
      { "code7", "rrc.code7",
        FT_BOOLEAN, 8, NULL, 0x80,
        NULL, HFILL }},
    { &hf_rrc_T_sync_UL_CodesBitmap_code6,
      { "code6", "rrc.code6",
        FT_BOOLEAN, 8, NULL, 0x40,
        NULL, HFILL }},
    { &hf_rrc_T_sync_UL_CodesBitmap_code5,
      { "code5", "rrc.code5",
        FT_BOOLEAN, 8, NULL, 0x20,
        NULL, HFILL }},
    { &hf_rrc_T_sync_UL_CodesBitmap_code4,
      { "code4", "rrc.code4",
        FT_BOOLEAN, 8, NULL, 0x10,
        NULL, HFILL }},
    { &hf_rrc_T_sync_UL_CodesBitmap_code3,
      { "code3", "rrc.code3",
        FT_BOOLEAN, 8, NULL, 0x08,
        NULL, HFILL }},
    { &hf_rrc_T_sync_UL_CodesBitmap_code2,
      { "code2", "rrc.code2",
        FT_BOOLEAN, 8, NULL, 0x04,
        NULL, HFILL }},
    { &hf_rrc_T_sync_UL_CodesBitmap_code1,
      { "code1", "rrc.code1",
        FT_BOOLEAN, 8, NULL, 0x02,
        NULL, HFILL }},
    { &hf_rrc_T_sync_UL_CodesBitmap_code0,
      { "code0", "rrc.code0",
        FT_BOOLEAN, 8, NULL, 0x01,
        NULL, HFILL }},
    { &hf_rrc_Sync_UL_Codes_Bitmap_code7,
      { "code7", "rrc.code7",
        FT_BOOLEAN, 8, NULL, 0x80,
        NULL, HFILL }},
    { &hf_rrc_Sync_UL_Codes_Bitmap_code6,
      { "code6", "rrc.code6",
        FT_BOOLEAN, 8, NULL, 0x40,
        NULL, HFILL }},
    { &hf_rrc_Sync_UL_Codes_Bitmap_code5,
      { "code5", "rrc.code5",
        FT_BOOLEAN, 8, NULL, 0x20,
        NULL, HFILL }},
    { &hf_rrc_Sync_UL_Codes_Bitmap_code4,
      { "code4", "rrc.code4",
        FT_BOOLEAN, 8, NULL, 0x10,
        NULL, HFILL }},
    { &hf_rrc_Sync_UL_Codes_Bitmap_code3,
      { "code3", "rrc.code3",
        FT_BOOLEAN, 8, NULL, 0x08,
        NULL, HFILL }},
    { &hf_rrc_Sync_UL_Codes_Bitmap_code2,
      { "code2", "rrc.code2",
        FT_BOOLEAN, 8, NULL, 0x04,
        NULL, HFILL }},
    { &hf_rrc_Sync_UL_Codes_Bitmap_code1,
      { "code1", "rrc.code1",
        FT_BOOLEAN, 8, NULL, 0x02,
        NULL, HFILL }},
    { &hf_rrc_Sync_UL_Codes_Bitmap_code0,
      { "code0", "rrc.code0",
        FT_BOOLEAN, 8, NULL, 0x01,
        NULL, HFILL }},
    { &hf_rrc_GsmSecurityCapability_a5_7,
      { "a5-7", "rrc.a5-7",
        FT_BOOLEAN, 8, NULL, 0x80,
        NULL, HFILL }},
    { &hf_rrc_GsmSecurityCapability_a5_6,
      { "a5-6", "rrc.a5-6",
        FT_BOOLEAN, 8, NULL, 0x40,
        NULL, HFILL }},
    { &hf_rrc_GsmSecurityCapability_a5_5,
      { "a5-5", "rrc.a5-5",
        FT_BOOLEAN, 8, NULL, 0x20,
        NULL, HFILL }},
    { &hf_rrc_GsmSecurityCapability_a5_4,
      { "a5-4", "rrc.a5-4",
        FT_BOOLEAN, 8, NULL, 0x10,
        NULL, HFILL }},
    { &hf_rrc_GsmSecurityCapability_a5_3,
      { "a5-3", "rrc.a5-3",
        FT_BOOLEAN, 8, NULL, 0x08,
        NULL, HFILL }},
    { &hf_rrc_GsmSecurityCapability_a5_2,
      { "a5-2", "rrc.a5-2",
        FT_BOOLEAN, 8, NULL, 0x04,
        NULL, HFILL }},
    { &hf_rrc_GsmSecurityCapability_a5_1,
      { "a5-1", "rrc.a5-1",
        FT_BOOLEAN, 8, NULL, 0x02,
        NULL, HFILL }},

/*--- End of included file: packet-rrc-hfarr.c ---*/
#line 261 "../../asn1/rrc/packet-rrc-template.c"
    { &hf_test,
      { "RAB Test", "rrc.RAB.test",
        FT_UINT8, BASE_DEC, NULL, 0,
        "rrc.RAB_Info_r6", HFILL }},
    { &hf_rrc_eutra_feat_group_ind_1,
      { "Indicator 1", "rrc.eutra_feat_group_ind_1",
        FT_BOOLEAN, BASE_NONE, TFS(&rrc_eutra_feat_group_ind_1_val), 0,
        "EUTRA Feature Group Indicator 1", HFILL }},
    { &hf_rrc_eutra_feat_group_ind_2,
      { "Indicator 2", "rrc.eutra_feat_group_ind_2",
        FT_BOOLEAN, BASE_NONE, TFS(&rrc_eutra_feat_group_ind_2_val), 0,
        "EUTRA Feature Group Indicator 2", HFILL }},
    { &hf_rrc_eutra_feat_group_ind_3,
      { "Indicator 3", "rrc.eutra_feat_group_ind_3",
        FT_BOOLEAN, BASE_NONE, TFS(&rrc_eutra_feat_group_ind_3_val), 0,
        "EUTRA Feature Group Indicator 3", HFILL }},
    { &hf_rrc_eutra_feat_group_ind_4,
      { "Indicator 4", "rrc.eutra_feat_group_ind_4",
        FT_BOOLEAN, BASE_NONE, TFS(&rrc_eutra_feat_group_ind_4_val), 0,
        "EUTRA Feature Group Indicator 4", HFILL }},
  };

  /* List of subtrees */
  static gint *ett[] = {
    &ett_rrc,

/*--- Included file: packet-rrc-ettarr.c ---*/
#line 1 "../../asn1/rrc/packet-rrc-ettarr.c"
    &ett_rrc_DL_DCCH_Message,
    &ett_rrc_DL_DCCH_MessageType,
    &ett_rrc_UL_DCCH_Message,
    &ett_rrc_UL_DCCH_MessageType,
    &ett_rrc_UL_DCCH_MessageType_ext,
    &ett_rrc_DL_CCCH_Message,
    &ett_rrc_DL_CCCH_MessageType,
    &ett_rrc_UL_CCCH_Message,
    &ett_rrc_UL_CCCH_MessageType,
    &ett_rrc_PCCH_Message,
    &ett_rrc_PCCH_MessageType,
    &ett_rrc_DL_SHCCH_Message,
    &ett_rrc_DL_SHCCH_MessageType,
    &ett_rrc_UL_SHCCH_Message,
    &ett_rrc_UL_SHCCH_MessageType,
    &ett_rrc_BCCH_FACH_Message,
    &ett_rrc_BCCH_FACH_MessageType,
    &ett_rrc_BCCH_BCH_Message,
    &ett_rrc_MCCH_Message,
    &ett_rrc_MCCH_MessageType,
    &ett_rrc_MSCH_Message,
    &ett_rrc_MSCH_MessageType,
    &ett_rrc_ActiveSetUpdate,
    &ett_rrc_T_r3,
    &ett_rrc_T_laterNonCriticalExtensions,
    &ett_rrc_T_v4b0NonCriticalExtensions,
    &ett_rrc_T_v590NonCriticalExtensions,
    &ett_rrc_T_v690NonCriticalExtensions,
    &ett_rrc_T_nonCriticalExtensions,
    &ett_rrc_T_later_than_r3,
    &ett_rrc_T_criticalExtensions,
    &ett_rrc_T_r6,
    &ett_rrc_T_v6b0NonCriticalExtensions,
    &ett_rrc_T_nonCriticalExtensions_01,
    &ett_rrc_T_criticalExtensions_01,
    &ett_rrc_T_r7,
    &ett_rrc_T_v780NonCriticalExtensions,
    &ett_rrc_T_v7f0NonCriticalExtensions,
    &ett_rrc_T_v7g0NonCriticalExtensions,
    &ett_rrc_T_nonCriticalExtensions_02,
    &ett_rrc_T_criticalExtensions_02,
    &ett_rrc_T_r8,
    &ett_rrc_T_v890NonCriticalExtensions,
    &ett_rrc_T_v7g0NonCriticalExtensions_01,
    &ett_rrc_T_nonCriticalExtensions_03,
    &ett_rrc_T_criticalExtensions_03,
    &ett_rrc_T_r9,
    &ett_rrc_T_v950NonCriticalExtensions,
    &ett_rrc_T_nonCriticalExtensions_04,
    &ett_rrc_T_criticalExtensions_04,
    &ett_rrc_T_r10,
    &ett_rrc_T_vbxyNonCriticalExtensions,
    &ett_rrc_T_nonCriticalExtensions_05,
    &ett_rrc_T_criticalExtensions_05,
    &ett_rrc_T_r11,
    &ett_rrc_T_nonCriticalExtensions_06,
    &ett_rrc_T_criticalExtensions_06,
    &ett_rrc_ActiveSetUpdate_r3_IEs,
    &ett_rrc_ActiveSetUpdate_v4b0ext_IEs,
    &ett_rrc_ActiveSetUpdate_v590ext_IEs,
    &ett_rrc_ActiveSetUpdate_v690ext_IEs,
    &ett_rrc_ActiveSetUpdate_r6_IEs,
    &ett_rrc_ActiveSetUpdate_v6b0ext_IEs,
    &ett_rrc_ActiveSetUpdate_r7_IEs,
    &ett_rrc_ActiveSetUpdate_v780ext_IEs,
    &ett_rrc_ActiveSetUpdate_v7f0ext_IEs,
    &ett_rrc_ActiveSetUpdate_v7g0ext_IEs,
    &ett_rrc_ActiveSetUpdate_r8_IEs,
    &ett_rrc_ActiveSetUpdate_v890ext_IEs,
    &ett_rrc_ActiveSetUpdate_r9_IEs,
    &ett_rrc_ActiveSetUpdate_v950ext_IEs,
    &ett_rrc_ActiveSetUpdate_r10_IEs,
    &ett_rrc_ActiveSetUpdate_vbxyext_IEs,
    &ett_rrc_ActiveSetUpdate_r11_IEs,
    &ett_rrc_ActiveSetUpdateComplete,
    &ett_rrc_T_laterNonCriticalExtensions_01,
    &ett_rrc_T_nonCriticalExtensions_07,
    &ett_rrc_ActiveSetUpdateFailure,
    &ett_rrc_T_laterNonCriticalExtensions_02,
    &ett_rrc_T_nonCriticalExtensions_08,
    &ett_rrc_AssistanceDataDelivery,
    &ett_rrc_T_r3_01,
    &ett_rrc_T_v3a0NonCriticalExtensions,
    &ett_rrc_T_laterNonCriticalExtensions_03,
    &ett_rrc_T_v4b0NonCriticalExtensions_01,
    &ett_rrc_T_v770NonCriticalExtension,
    &ett_rrc_T_v860NonCriticalExtension,
    &ett_rrc_T_v920NonCriticalExtension,
    &ett_rrc_T_va40NonCriticalExtension,
    &ett_rrc_T_nonCriticalExtensions_09,
    &ett_rrc_T_later_than_r3_01,
    &ett_rrc_T_criticalExtensions_07,
    &ett_rrc_AssistanceDataDelivery_r3_IEs,
    &ett_rrc_AssistanceDataDelivery_v3a0ext,
    &ett_rrc_AssistanceDataDelivery_v4b0ext_IEs,
    &ett_rrc_AssistanceDataDelivery_v770ext_IEs,
    &ett_rrc_AssistanceDataDelivery_v860ext_IEs,
    &ett_rrc_AssistanceDataDelivery_v920ext_IEs,
    &ett_rrc_AssistanceDataDelivery_va40ext_IEs,
    &ett_rrc_CellChangeOrderFromUTRAN,
    &ett_rrc_T_r3_02,
    &ett_rrc_T_laterNonCriticalExtensions_04,
    &ett_rrc_T_v590NonCriticalExtensions_01,
    &ett_rrc_T_nonCriticalExtensions_10,
    &ett_rrc_T_later_than_r3_02,
    &ett_rrc_T_criticalExtensions_08,
    &ett_rrc_CellChangeOrderFromUTRAN_r3_IEs,
    &ett_rrc_CellChangeOrderFromUTRAN_v590ext_IEs,
    &ett_rrc_T_geran_SystemInfoType,
    &ett_rrc_CellChangeOrderFromUTRANFailure,
    &ett_rrc_T_r3_03,
    &ett_rrc_T_laterNonCriticalExtensions_05,
    &ett_rrc_T_nonCriticalExtensions_11,
    &ett_rrc_T_dummy,
    &ett_rrc_T_criticalExtensions_09,
    &ett_rrc_CellChangeOrderFromUTRANFailure_r3_IEs,
    &ett_rrc_CellUpdate,
    &ett_rrc_T_laterNonCriticalExtensions_06,
    &ett_rrc_T_v590NonCriticalExtensions_02,
    &ett_rrc_T_v690NonCriticalExtensions_01,
    &ett_rrc_T_v6b0NonCriticalExtensions_01,
    &ett_rrc_T_v770NonCriticalExtensions,
    &ett_rrc_T_v860NonCriticalExtensions,
    &ett_rrc_T_va40NonCriticalExtensions,
    &ett_rrc_T_vbxyNonCriticalExtensions_01,
    &ett_rrc_T_nonCriticalExtensions_12,
    &ett_rrc_CellUpdate_r3_add_ext_IEs,
    &ett_rrc_T_v7g0NonCriticalExtensions_02,
    &ett_rrc_T_nonCriticalExtensions_13,
    &ett_rrc_CellUpdate_v590ext,
    &ett_rrc_CellUpdate_v690ext_IEs,
    &ett_rrc_CellUpdate_v6b0ext_IEs,
    &ett_rrc_CellUpdate_v770ext_IEs,
    &ett_rrc_CellUpdate_v7e0ext_IEs,
    &ett_rrc_CellUpdate_v7g0ext_IEs,
    &ett_rrc_CellUpdate_v860ext_IEs,
    &ett_rrc_CellUpdate_va40ext_IEs,
    &ett_rrc_CellUpdate_vbxyext_IEs,
    &ett_rrc_CellUpdateConfirm,
    &ett_rrc_T_r3_04,
    &ett_rrc_T_v3a0NonCriticalExtensions_01,
    &ett_rrc_T_laterNonCriticalExtensions_07,
    &ett_rrc_T_v4b0NonCriticalExtensions_02,
    &ett_rrc_T_v590NonCriticalExtenstions,
    &ett_rrc_T_v5d0NonCriticalExtenstions,
    &ett_rrc_T_v690NonCriticalExtensions_02,
    &ett_rrc_T_nonCriticalExtensions_14,
    &ett_rrc_T_later_than_r3_03,
    &ett_rrc_T_criticalExtensions_10,
    &ett_rrc_T_r4,
    &ett_rrc_T_v4d0NonCriticalExtensions,
    &ett_rrc_T_v590NonCriticalExtenstions_01,
    &ett_rrc_T_v5d0NonCriticalExtenstions_01,
    &ett_rrc_T_v690NonCriticalExtensions_03,
    &ett_rrc_T_nonCriticalExtensions_15,
    &ett_rrc_T_criticalExtensions_11,
    &ett_rrc_T_r5,
    &ett_rrc_T_v5d0NonCriticalExtenstions_02,
    &ett_rrc_T_v690NonCriticalExtensions_04,
    &ett_rrc_T_nonCriticalExtensions_16,
    &ett_rrc_T_criticalExtensions_12,
    &ett_rrc_T_r6_01,
    &ett_rrc_T_v6b0NonCriticalExtensions_02,
    &ett_rrc_T_nonCriticalExtensions_17,
    &ett_rrc_T_criticalExtensions_13,
    &ett_rrc_T_r7_01,
    &ett_rrc_T_v780NonCriticalExtensions_01,
    &ett_rrc_T_v860NonCriticalExtensions_01,
    &ett_rrc_T_nonCriticalExtensions_18,
    &ett_rrc_T_criticalExtensions_14,
    &ett_rrc_T_r8_01,
    &ett_rrc_T_v7d0NonCriticalExtensions,
    &ett_rrc_T_v7f0NonCriticalExtensions_01,
    &ett_rrc_T_v890NonCriticalExtensions_01,
    &ett_rrc_T_v7g0NonCriticalExtensions_03,
    &ett_rrc_T_v8a0NonCriticalExtensions,
    &ett_rrc_T_nonCriticalExtensions_19,
    &ett_rrc_T_criticalExtensions_15,
    &ett_rrc_T_r9_01,
    &ett_rrc_T_v950NonCriticalExtensions_01,
    &ett_rrc_T_v9c0NonCriticalExtensions,
    &ett_rrc_T_nonCriticalExtensions_20,
    &ett_rrc_T_criticalExtensions_16,
    &ett_rrc_T_r10_01,
    &ett_rrc_T_v9c0NonCriticalExtensions_01,
    &ett_rrc_T_vbxyNonCriticalExtensions_02,
    &ett_rrc_T_nonCriticalExtensions_21,
    &ett_rrc_T_criticalExtensions_17,
    &ett_rrc_T_r11_01,
    &ett_rrc_T_nonCriticalExtensions_22,
    &ett_rrc_T_criticalExtensions_18,
    &ett_rrc_CellUpdateConfirm_r3_IEs,
    &ett_rrc_T_modeSpecificTransChInfo,
    &ett_rrc_T_fdd,
    &ett_rrc_T_modeSpecificPhysChInfo,
    &ett_rrc_T_fdd_01,
    &ett_rrc_CellUpdateConfirm_v3a0ext,
    &ett_rrc_CellUpdateConfirm_v4b0ext_IEs,
    &ett_rrc_CellUpdateConfirm_v590ext_IEs,
    &ett_rrc_CellUpdateConfirm_v5d0ext_IEs,
    &ett_rrc_CellUpdateConfirm_r4_IEs,
    &ett_rrc_T_modeSpecificTransChInfo_01,
    &ett_rrc_T_fdd_02,
    &ett_rrc_T_modeSpecificPhysChInfo_01,
    &ett_rrc_T_fdd_03,
    &ett_rrc_CellUpdateConfirm_r5_IEs,
    &ett_rrc_T_modeSpecificTransChInfo_02,
    &ett_rrc_T_fdd_04,
    &ett_rrc_T_modeSpecificPhysChInfo_02,
    &ett_rrc_T_fdd_05,
    &ett_rrc_CellUpdateConfirm_r6_IEs,
    &ett_rrc_CellUpdateConfirm_v690ext_IEs,
    &ett_rrc_CellUpdateConfirm_v6b0ext_IEs,
    &ett_rrc_CellUpdateConfirm_r7_IEs,
    &ett_rrc_CellUpdateConfirm_r7_add_ext_IEs,
    &ett_rrc_T_v7f0NonCriticalExtensions_02,
    &ett_rrc_T_v7g0NonCriticalExtensions_04,
    &ett_rrc_T_nonCriticalExtensions_23,
    &ett_rrc_CellUpdateConfirm_v780ext_IEs,
    &ett_rrc_CellUpdateConfirm_v7d0ext_IEs,
    &ett_rrc_CellUpdateConfirm_v7f0ext_IEs,
    &ett_rrc_CellUpdateConfirm_v7g0ext_IEs,
    &ett_rrc_CellUpdateConfirm_r8_IEs,
    &ett_rrc_CellUpdateConfirm_v860ext_IEs,
    &ett_rrc_T_rab_InformationSetup,
    &ett_rrc_CellUpdateConfirm_v890ext_IEs,
    &ett_rrc_CellUpdateConfirm_v8a0ext_IEs,
    &ett_rrc_CellUpdateConfirm_r9_IEs,
    &ett_rrc_CellUpdateConfirm_v950ext_IEs,
    &ett_rrc_CellUpdateConfirm_v9c0ext_IEs,
    &ett_rrc_CellUpdateConfirm_r10_IEs,
    &ett_rrc_CellUpdateConfirm_vbxyext_IEs,
    &ett_rrc_CellUpdateConfirm_r11_IEs,
    &ett_rrc_CellUpdateConfirm_CCCH,
    &ett_rrc_T_r3_05,
    &ett_rrc_T_laterNonCriticalExtensions_08,
    &ett_rrc_T_v4b0NonCriticalExtensions_03,
    &ett_rrc_T_v590NonCriticalExtensions_03,
    &ett_rrc_T_v5d0NonCriticalExtenstions_03,
    &ett_rrc_T_v690NonCriticalExtensions_05,
    &ett_rrc_T_nonCriticalExtensions_24,
    &ett_rrc_T_later_than_r3_04,
    &ett_rrc_T_criticalExtensions_19,
    &ett_rrc_T_r4_01,
    &ett_rrc_T_v4d0NonCriticalExtensions_01,
    &ett_rrc_T_v590NonCriticalExtensions_04,
    &ett_rrc_T_v5d0NonCriticalExtenstions_04,
    &ett_rrc_T_v690NonCriticalExtensions_06,
    &ett_rrc_T_nonCriticalExtensions_25,
    &ett_rrc_T_criticalExtensions_20,
    &ett_rrc_T_r5_01,
    &ett_rrc_T_v5d0NonCriticalExtenstions_05,
    &ett_rrc_T_v690NonCriticalExtensions_07,
    &ett_rrc_T_nonCriticalExtensions_26,
    &ett_rrc_T_criticalExtensions_21,
    &ett_rrc_T_r6_02,
    &ett_rrc_T_v6b0NonCriticalExtensions_03,
    &ett_rrc_T_nonCriticalExtensions_27,
    &ett_rrc_T_criticalExtensions_22,
    &ett_rrc_T_r7_02,
    &ett_rrc_T_v780NonCriticalExtensions_02,
    &ett_rrc_T_v860NonCriticalExtensions_02,
    &ett_rrc_T_nonCriticalExtensions_28,
    &ett_rrc_T_criticalExtensions_23,
    &ett_rrc_T_r8_02,
    &ett_rrc_T_v7d0NonCriticalExtensions_01,
    &ett_rrc_T_v890NonCriticalExtensions_02,
    &ett_rrc_T_v8a0NonCriticalExtensions_01,
    &ett_rrc_T_v7f0NonCriticalExtensions_03,
    &ett_rrc_T_v7g0NonCriticalExtensions_05,
    &ett_rrc_T_nonCriticalExtensions_29,
    &ett_rrc_T_criticalExtensions_24,
    &ett_rrc_T_r9_02,
    &ett_rrc_T_nonCriticalExtensions_30,
    &ett_rrc_T_criticalExtensions_25,
    &ett_rrc_T_r10_02,
    &ett_rrc_T_vbxyNonCriticalExtensions_03,
    &ett_rrc_T_nonCriticalExtensions_31,
    &ett_rrc_T_criticalExtensions_26,
    &ett_rrc_T_r11_02,
    &ett_rrc_T_nonCriticalExtensions_32,
    &ett_rrc_T_criticalExtensions_27,
    &ett_rrc_CounterCheck,
    &ett_rrc_T_r3_06,
    &ett_rrc_T_laterNonCriticalExtensions_09,
    &ett_rrc_T_nonCriticalExtensions_33,
    &ett_rrc_T_later_than_r3_05,
    &ett_rrc_T_criticalExtensions_28,
    &ett_rrc_CounterCheck_r3_IEs,
    &ett_rrc_CounterCheckResponse,
    &ett_rrc_T_laterNonCriticalExtensions_10,
    &ett_rrc_T_nonCriticalExtensions_34,
    &ett_rrc_DownlinkDirectTransfer,
    &ett_rrc_T_r3_07,
    &ett_rrc_T_laterNonCriticalExtensions_11,
    &ett_rrc_T_nonCriticalExtensions_35,
    &ett_rrc_T_later_than_r3_06,
    &ett_rrc_T_criticalExtensions_29,
    &ett_rrc_DownlinkDirectTransfer_r3_IEs,
    &ett_rrc_ETWSPrimaryNotificationWithSecurity,
    &ett_rrc_T_nonCriticalExtensions_36,
    &ett_rrc_HandoverToUTRANCommand,
    &ett_rrc_T_r3_08,
    &ett_rrc_T_nonCriticalExtensions_37,
    &ett_rrc_T_criticalExtensions_30,
    &ett_rrc_T_r4_02,
    &ett_rrc_T_nonCriticalExtensions_38,
    &ett_rrc_T_criticalExtensions_31,
    &ett_rrc_T_r5_02,
    &ett_rrc_T_nonCriticalExtensions_39,
    &ett_rrc_T_criticalExtensions_32,
    &ett_rrc_T_r6_03,
    &ett_rrc_T_v6b0NonCriticalExtensions_04,
    &ett_rrc_T_nonCriticalExtensions_40,
    &ett_rrc_T_criticalExtensions_33,
    &ett_rrc_T_r7_03,
    &ett_rrc_T_v780NonCriticalExtensions_03,
    &ett_rrc_T_v820NonCriticalExtensions,
    &ett_rrc_T_v7d0NonCriticalExtensions_02,
    &ett_rrc_T_nonCriticalExtensions_41,
    &ett_rrc_T_criticalExtensions_34,
    &ett_rrc_T_r8_03,
    &ett_rrc_T_v7d0NonCriticalExtensions_03,
    &ett_rrc_T_v890NonCriticalExtensions_03,
    &ett_rrc_T_v8a0NonCriticalExtensions_02,
    &ett_rrc_T_nonCriticalExtensions_42,
    &ett_rrc_T_criticalExtensions_35,
    &ett_rrc_T_r9_03,
    &ett_rrc_T_v9c0NonCriticalExtensions_02,
    &ett_rrc_T_nonCriticalExtensions_43,
    &ett_rrc_HandoverToUTRANCommand_r3_IEs,
    &ett_rrc_T_specificationMode,
    &ett_rrc_T_complete,
    &ett_rrc_T_modeSpecificInfo,
    &ett_rrc_T_fdd_06,
    &ett_rrc_T_preconfiguration,
    &ett_rrc_T_preConfigMode,
    &ett_rrc_T_defaultConfig,
    &ett_rrc_T_modeSpecificInfo_01,
    &ett_rrc_T_fdd_07,
    &ett_rrc_T_tdd,
    &ett_rrc_HandoverToUTRANCommand_r4_IEs,
    &ett_rrc_T_specificationMode_01,
    &ett_rrc_T_complete_01,
    &ett_rrc_T_modeSpecificInfo_02,
    &ett_rrc_T_fdd_08,
    &ett_rrc_T_preconfiguration_01,
    &ett_rrc_T_preConfigMode_01,
    &ett_rrc_T_defaultConfig_01,
    &ett_rrc_T_modeSpecificInfo_03,
    &ett_rrc_T_fdd_09,
    &ett_rrc_T_tdd_01,
    &ett_rrc_T_tdd384,
    &ett_rrc_T_tdd128,
    &ett_rrc_HandoverToUTRANCommand_r5_IEs,
    &ett_rrc_T_specificationMode_02,
    &ett_rrc_T_complete_02,
    &ett_rrc_T_modeSpecificInfo_04,
    &ett_rrc_T_fdd_10,
    &ett_rrc_T_preconfiguration_02,
    &ett_rrc_T_preConfigMode_02,
    &ett_rrc_T_defaultConfig_02,
    &ett_rrc_T_modeSpecificInfo_05,
    &ett_rrc_T_fdd_11,
    &ett_rrc_T_tdd_02,
    &ett_rrc_T_tdd384_01,
    &ett_rrc_T_tdd128_01,
    &ett_rrc_HandoverToUTRANCommand_r6_IEs,
    &ett_rrc_T_specificationMode_03,
    &ett_rrc_T_complete_03,
    &ett_rrc_T_preconfiguration_03,
    &ett_rrc_T_preConfigMode_03,
    &ett_rrc_T_defaultConfig_03,
    &ett_rrc_T_modeSpecificInfo_06,
    &ett_rrc_T_fdd_12,
    &ett_rrc_T_tdd_03,
    &ett_rrc_T_tdd384_02,
    &ett_rrc_T_tdd128_02,
    &ett_rrc_HandoverToUTRANCommand_v6b0ext_IEs,
    &ett_rrc_HandoverToUTRANCommand_r7_IEs,
    &ett_rrc_T_specificationMode_04,
    &ett_rrc_T_complete_04,
    &ett_rrc_T_preconfiguration_04,
    &ett_rrc_T_preConfigMode_04,
    &ett_rrc_T_defaultConfig_04,
    &ett_rrc_T_modeSpecificInfo_07,
    &ett_rrc_T_fdd_13,
    &ett_rrc_T_tdd_04,
    &ett_rrc_T_tdd384_03,
    &ett_rrc_T_tdd128_03,
    &ett_rrc_T_tdd768,
    &ett_rrc_HandoverToUTRANCommand_v780ext_IEs,
    &ett_rrc_HandoverToUTRANCommand_v7d0ext_IEs,
    &ett_rrc_HandoverToUTRANCommand_r8_IEs,
    &ett_rrc_T_specificationMode_05,
    &ett_rrc_T_complete_05,
    &ett_rrc_T_preconfiguration_05,
    &ett_rrc_T_preConfigMode_05,
    &ett_rrc_T_defaultConfig_05,
    &ett_rrc_T_modeSpecificInfo_08,
    &ett_rrc_T_fdd_14,
    &ett_rrc_T_tdd_05,
    &ett_rrc_T_tdd384_04,
    &ett_rrc_T_tdd128_04,
    &ett_rrc_T_tdd768_01,
    &ett_rrc_HandoverToUTRANCommand_v820ext_IEs,
    &ett_rrc_HandoverToUTRANCommand_v890ext_IEs,
    &ett_rrc_HandoverToUTRANCommand_v8a0ext_IEs,
    &ett_rrc_HandoverToUTRANCommand_r9_IEs,
    &ett_rrc_T_specificationMode_06,
    &ett_rrc_T_complete_06,
    &ett_rrc_T_preconfiguration_06,
    &ett_rrc_T_preConfigMode_06,
    &ett_rrc_T_defaultConfig_06,
    &ett_rrc_T_modeSpecificInfo_09,
    &ett_rrc_T_fdd_15,
    &ett_rrc_T_tdd_06,
    &ett_rrc_T_tdd384_05,
    &ett_rrc_T_tdd128_05,
    &ett_rrc_T_tdd768_02,
    &ett_rrc_HandoverToUTRANCommand_v9c0ext_IEs,
    &ett_rrc_HandoverToUTRANCommand_LaterCriticalExtensions,
    &ett_rrc_T_r10_03,
    &ett_rrc_T_v9c0NonCriticalExtensions_03,
    &ett_rrc_T_nonCriticalExtensions_44,
    &ett_rrc_T_criticalExtensions_36,
    &ett_rrc_T_r11_03,
    &ett_rrc_T_nonCriticalExtensions_45,
    &ett_rrc_T_criticalExtensions_37,
    &ett_rrc_HandoverToUTRANCommand_r10_IEs,
    &ett_rrc_T_specificationMode_07,
    &ett_rrc_T_complete_07,
    &ett_rrc_T_preconfiguration_07,
    &ett_rrc_T_preConfigMode_07,
    &ett_rrc_T_defaultConfig_07,
    &ett_rrc_T_modeSpecificInfo_10,
    &ett_rrc_T_fdd_16,
    &ett_rrc_T_tdd_07,
    &ett_rrc_T_tdd384_06,
    &ett_rrc_T_tdd128_06,
    &ett_rrc_T_tdd768_03,
    &ett_rrc_HandoverToUTRANCommand_r11_IEs,
    &ett_rrc_T_specificationMode_08,
    &ett_rrc_T_complete_08,
    &ett_rrc_T_preconfiguration_08,
    &ett_rrc_T_preConfigMode_08,
    &ett_rrc_T_defaultConfig_08,
    &ett_rrc_T_modeSpecificInfo_11,
    &ett_rrc_T_fdd_17,
    &ett_rrc_T_tdd_08,
    &ett_rrc_T_tdd384_07,
    &ett_rrc_T_tdd128_07,
    &ett_rrc_T_tdd768_04,
    &ett_rrc_HandoverToUTRANComplete,
    &ett_rrc_T_laterNonCriticalExtensions_12,
    &ett_rrc_T_va40NonCriticalExtensions_01,
    &ett_rrc_T_vbxyNonCriticalExtensions_04,
    &ett_rrc_T_nonCriticalExtensions_46,
    &ett_rrc_HandoverToUTRANComplete_va40ext,
    &ett_rrc_HandoverToUTRANComplete_vbxyext_IEs,
    &ett_rrc_InitialDirectTransfer,
    &ett_rrc_T_v3a0NonCriticalExtensions_02,
    &ett_rrc_T_laterNonCriticalExtensions_13,
    &ett_rrc_T_v590NonCriticalExtensions_05,
    &ett_rrc_T_v690NonCriticalExtensions_08,
    &ett_rrc_T_v770NonCriticalExtensions_01,
    &ett_rrc_T_v860NonCriticalExtensions_03,
    &ett_rrc_T_vbxyNonCriticalExtensions_05,
    &ett_rrc_T_nonCriticalExtensions_47,
    &ett_rrc_InitialDirectTransfer_v3a0ext,
    &ett_rrc_InitialDirectTransfer_v590ext,
    &ett_rrc_InitialDirectTransfer_v690ext_IEs,
    &ett_rrc_InitialDirectTransfer_v770ext_IEs,
    &ett_rrc_InitialDirectTransfer_v860ext_IEs,
    &ett_rrc_InitialDirectTransfer_r3_add_ext_IEs,
    &ett_rrc_T_nonCriticalExtensions_48,
    &ett_rrc_InitialDirectTransfer_v7g0ext_IEs,
    &ett_rrc_InitialDirectTransfer_vbxyext_IEs,
    &ett_rrc_HandoverFromUTRANCommand_GSM,
    &ett_rrc_T_r3_09,
    &ett_rrc_T_laterNonCriticalExtensions_14,
    &ett_rrc_T_v690NonCriticalExtensions_09,
    &ett_rrc_T_v860NonCriticalExtensions_04,
    &ett_rrc_T_nonCriticalExtensions_49,
    &ett_rrc_T_later_than_r3_07,
    &ett_rrc_T_criticalExtensions_38,
    &ett_rrc_T_r6_04,
    &ett_rrc_T_v860NonCriticalExtensions_05,
    &ett_rrc_T_nonCriticalExtensions_50,
    &ett_rrc_T_criticalExtensions_39,
    &ett_rrc_HandoverFromUTRANCommand_GSM_r3_IEs,
    &ett_rrc_T_gsm_message,
    &ett_rrc_T_single_GSM_Message_r3,
    &ett_rrc_T_gsm_MessageList_r3,
    &ett_rrc_HandoverFromUTRANCommand_GSM_r6_IEs,
    &ett_rrc_T_gsm_message_01,
    &ett_rrc_T_single_GSM_Message_r6,
    &ett_rrc_T_gsm_MessageList_r6,
    &ett_rrc_T_geran_SystemInfoType_01,
    &ett_rrc_HandoverFromUTRANCommand_GSM_v690ext_IEs,
    &ett_rrc_T_geran_SystemInfoType_02,
    &ett_rrc_HandoverFromUTRANCommand_GSM_v860ext_IEs,
    &ett_rrc_HandoverFromUTRANCommand_GERANIu,
    &ett_rrc_T_handoverFromUTRANCommand_GERANIu,
    &ett_rrc_T_r5_03,
    &ett_rrc_T_nonCriticalExtensions_51,
    &ett_rrc_T_later_than_r5,
    &ett_rrc_T_criticalExtensions_40,
    &ett_rrc_HandoverFromUTRANCommand_GERANIu_r5_IEs,
    &ett_rrc_T_geranIu_Message,
    &ett_rrc_T_single_GERANIu_Message,
    &ett_rrc_T_geranIu_MessageList,
    &ett_rrc_HandoverFromUTRANCommand_CDMA2000,
    &ett_rrc_T_r3_10,
    &ett_rrc_T_laterNonCriticalExtensions_15,
    &ett_rrc_T_nonCriticalExtensions_52,
    &ett_rrc_T_later_than_r3_08,
    &ett_rrc_T_criticalExtensions_41,
    &ett_rrc_HandoverFromUTRANCommand_CDMA2000_r3_IEs,
    &ett_rrc_HandoverFromUTRANCommand_EUTRA,
    &ett_rrc_T_criticalExtensions_42,
    &ett_rrc_T_r8_04,
    &ett_rrc_T_nonCriticalExtensions_53,
    &ett_rrc_T_criticalExtensions_43,
    &ett_rrc_T_r11_04,
    &ett_rrc_T_nonCriticalExtensions_54,
    &ett_rrc_T_criticalExtensions_44,
    &ett_rrc_HandoverFromUTRANCommand_EUTRA_r8_IEs,
    &ett_rrc_HandoverFromUTRANCommand_EUTRA_r11_IEs,
    &ett_rrc_HandoverFromUTRANFailure,
    &ett_rrc_T_interRATMessage,
    &ett_rrc_T_gsm,
    &ett_rrc_T_cdma2000,
    &ett_rrc_T_laterNonCriticalExtensions_16,
    &ett_rrc_T_v590NonCriticalExtensions_06,
    &ett_rrc_T_v860NonCriticalExtensions_06,
    &ett_rrc_T_nonCriticalExtensions_55,
    &ett_rrc_HandoverFromUtranFailure_v590ext_IEs,
    &ett_rrc_HandoverFromUtranFailure_v860ext_IEs,
    &ett_rrc_InterRATHandoverInfo,
    &ett_rrc_T_predefinedConfigStatusList,
    &ett_rrc_T_uE_SecurityInformation,
    &ett_rrc_T_ue_CapabilityContainer,
    &ett_rrc_T_v390NonCriticalExtensions,
    &ett_rrc_T_present_01,
    &ett_rrc_T_v3a0NonCriticalExtensions_03,
    &ett_rrc_T_laterNonCriticalExtensions_17,
    &ett_rrc_T_v3g0NonCriticalExtensions,
    &ett_rrc_T_v4b0NonCriticalExtensions_04,
    &ett_rrc_T_v4d0NonCriticalExtensions_02,
    &ett_rrc_T_v590NonCriticalExtensions_07,
    &ett_rrc_T_v690NonCriticalExtensions_10,
    &ett_rrc_InterRATHandoverInfo_v390ext_IEs,
    &ett_rrc_InterRATHandoverInfo_v3a0ext_IEs,
    &ett_rrc_InterRATHandoverInfo_v3d0ext_IEs,
    &ett_rrc_InterRATHandoverInfo_v3g0ext_IEs,
    &ett_rrc_InterRATHandoverInfo_r3_add_ext_IEs,
    &ett_rrc_T_v7e0NonCriticalExtensions,
    &ett_rrc_T_v7f0NonCriticalExtensions_04,
    &ett_rrc_T_va40NonCriticalExtensions_02,
    &ett_rrc_T_nonCriticalExtensions_56,
    &ett_rrc_InterRATHandoverInfo_v4b0ext_IEs,
    &ett_rrc_InterRATHandoverInfo_v4d0ext_IEs,
    &ett_rrc_InterRATHandoverInfo_v590ext_IEs,
    &ett_rrc_InterRATHandoverInfo_v690ext1_IEs,
    &ett_rrc_InterRATHandoverInfo_v690ext_IEs,
    &ett_rrc_InterRATHandoverInfo_NonCriticalExtension_v6b0_IEs,
    &ett_rrc_T_v6e0NonCriticalExtensions,
    &ett_rrc_T_v770NonCriticalExtensions_02,
    &ett_rrc_T_v790nonCriticalExtensions,
    &ett_rrc_T_v860NonCriticalExtensions_07,
    &ett_rrc_T_v880NonCriticalExtensions,
    &ett_rrc_T_v920NonCriticalExtensions,
    &ett_rrc_T_v8b0NonCriticalExtensions,
    &ett_rrc_T_v950NonCriticalExtensions_02,
    &ett_rrc_T_va40NonCriticalExtensions_03,
    &ett_rrc_T_va80NonCriticalExtensions,
    &ett_rrc_T_vbxyNonCriticalExtensions_06,
    &ett_rrc_T_nonCriticalExtensions_57,
    &ett_rrc_InterRATHandoverInfo_v6b0ext_IEs,
    &ett_rrc_InterRATHandoverInfo_v6e0ext_IEs,
    &ett_rrc_InterRATHandoverInfo_v770ext_IEs,
    &ett_rrc_InterRATHandoverInfo_v790ext_IEs,
    &ett_rrc_InterRATHandoverInfo_v7e0ext_IEs,
    &ett_rrc_InterRATHandoverInfo_v7f0ext_IEs,
    &ett_rrc_InterRATHandoverInfo_v860ext_IEs,
    &ett_rrc_InterRATHandoverInfo_v880ext_IEs,
    &ett_rrc_InterRATHandoverInfo_v920ext_IEs,
    &ett_rrc_InterRATHandoverInfo_v8b0ext_IEs,
    &ett_rrc_InterRATHandoverInfo_v950ext_IEs,
    &ett_rrc_InterRATHandoverInfo_va40ext_IEs,
    &ett_rrc_InterRATHandoverInfoBand_va40ext_IEs,
    &ett_rrc_InterRATHandoverInfo_va80ext_IEs,
    &ett_rrc_InterRATHandoverInfo_vbxyext_IEs,
    &ett_rrc_MeasurementControl,
    &ett_rrc_T_r3_11,
    &ett_rrc_T_v390nonCriticalExtensions,
    &ett_rrc_T_v3a0NonCriticalExtensions_04,
    &ett_rrc_T_laterNonCriticalExtensions_18,
    &ett_rrc_T_v4b0NonCriticalExtensions_05,
    &ett_rrc_T_v590NonCriticalExtensions_08,
    &ett_rrc_T_v5b0NonCriticalExtensions,
    &ett_rrc_T_nonCriticalExtensions_58,
    &ett_rrc_T_later_than_r3_09,
    &ett_rrc_T_criticalExtensions_45,
    &ett_rrc_T_r4_03,
    &ett_rrc_T_v4d0NonCriticalExtensions_03,
    &ett_rrc_T_v590NonCriticalExtensions_09,
    &ett_rrc_T_v5b0NonCriticalExtensions_01,
    &ett_rrc_T_nonCriticalExtensions_59,
    &ett_rrc_T_later_than_r4,
    &ett_rrc_T_criticalExtensions_46,
    &ett_rrc_T_r6_05,
    &ett_rrc_T_v6a0NonCriticalExtensions,
    &ett_rrc_T_nonCriticalExtensions_60,
    &ett_rrc_T_criticalExtensions_47,
    &ett_rrc_T_r7_04,
    &ett_rrc_T_v7b0NonCriticalExtensions,
    &ett_rrc_T_nonCriticalExtensions_61,
    &ett_rrc_T_criticalExtensions_48,
    &ett_rrc_T_r8_05,
    &ett_rrc_T_v8a0NonCriticalExtensions_03,
    &ett_rrc_T_nonCriticalExtensions_62,
    &ett_rrc_T_criticalExtensions_49,
    &ett_rrc_T_r9_04,
    &ett_rrc_T_v970NonCriticalExtensions,
    &ett_rrc_T_nonCriticalExtensions_63,
    &ett_rrc_T_criticalExtensions_50,
    &ett_rrc_T_r10_04,
    &ett_rrc_T_vaa0NonCriticalExtensions,
    &ett_rrc_T_nonCriticalExtensions_64,
    &ett_rrc_T_criticalExtensions_51,
    &ett_rrc_T_r11_05,
    &ett_rrc_T_nonCriticalExtensions_65,
    &ett_rrc_T_criticalExtensions_52,
    &ett_rrc_MeasurementControl_r3_IEs,
    &ett_rrc_MeasurementControl_v390ext,
    &ett_rrc_MeasurementControl_v3a0ext,
    &ett_rrc_MeasurementControl_r4_IEs,
    &ett_rrc_MeasurementControl_v590ext_IEs,
    &ett_rrc_T_measurementCommand_v590ext,
    &ett_rrc_MeasurementControl_v5b0ext_IEs,
    &ett_rrc_MeasurementControl_r6_IEs,
    &ett_rrc_MeasurementControl_v6a0ext_IEs,
    &ett_rrc_MeasurementControl_r7_IEs,
    &ett_rrc_MeasurementControl_v7b0ext_IEs,
    &ett_rrc_MeasurementControl_r8_IEs,
    &ett_rrc_MeasurementControl_v8a0ext_IEs,
    &ett_rrc_MeasurementControl_r9_IEs,
    &ett_rrc_MeasurementControl_v970ext_IEs,
    &ett_rrc_MeasurementControl_r10_IEs,
    &ett_rrc_MeasurementControl_vaa0ext_IEs,
    &ett_rrc_MeasurementControl_r11_IEs,
    &ett_rrc_MeasurementControlFailure,
    &ett_rrc_T_laterNonCriticalExtensions_19,
    &ett_rrc_T_v590NonCriticalExtensions_10,
    &ett_rrc_T_nonCriticalExtensions_66,
    &ett_rrc_MeasurementControlFailure_v590ext_IEs,
    &ett_rrc_MeasurementReport,
    &ett_rrc_T_v390nonCriticalExtensions_01,
    &ett_rrc_T_laterNonCriticalExtensions_20,
    &ett_rrc_T_v4b0NonCriticalExtensions_06,
    &ett_rrc_T_v590NonCriticalExtensions_11,
    &ett_rrc_T_v5b0NonCriticalExtensions_02,
    &ett_rrc_T_v690NonCriticalExtensions_11,
    &ett_rrc_T_v770NonCriticalExtensions_03,
    &ett_rrc_T_v860NonCriticalExtensions_08,
    &ett_rrc_T_v920NonCriticalExtensions_01,
    &ett_rrc_T_v970NonCriticalExtensions_01,
    &ett_rrc_T_va40NonCriticalExtensions_04,
    &ett_rrc_T_vbxyNonCriticalExtensions_07,
    &ett_rrc_T_nonCriticalExtensions_67,
    &ett_rrc_MeasurementReport_v390ext,
    &ett_rrc_MeasurementReport_v4b0ext_IEs,
    &ett_rrc_MeasurementReport_v590ext_IEs,
    &ett_rrc_MeasurementReport_v5b0ext_IEs,
    &ett_rrc_MeasurementReport_v690ext_IEs,
    &ett_rrc_MeasurementReport_v770ext_IEs,
    &ett_rrc_MeasurementReport_v860ext_IEs,
    &ett_rrc_MeasurementReport_v920ext_IEs,
    &ett_rrc_MeasurementReport_v970ext_IEs,
    &ett_rrc_MeasurementReport_va40ext_IEs,
    &ett_rrc_MeasurementReport_vbxyext_IEs,
    &ett_rrc_PagingType1,
    &ett_rrc_T_laterNonCriticalExtensions_21,
    &ett_rrc_T_v590NonCriticalExtensions_12,
    &ett_rrc_T_v860NonCriticalExtensions_09,
    &ett_rrc_T_nonCriticalExtensions_68,
    &ett_rrc_PagingType1_v590ext_IEs,
    &ett_rrc_PagingType1_v860ext_IEs,
    &ett_rrc_PagingType2,
    &ett_rrc_T_laterNonCriticalExtensions_22,
    &ett_rrc_T_nonCriticalExtensions_69,
    &ett_rrc_PhysicalChannelReconfiguration,
    &ett_rrc_T_r3_12,
    &ett_rrc_T_v3a0NonCriticalExtensions_05,
    &ett_rrc_T_laterNonCriticalExtensions_23,
    &ett_rrc_T_v4b0NonCriticalExtenstions,
    &ett_rrc_T_v590NonCriticalExtenstions_02,
    &ett_rrc_T_v690NonCriticalExtensions_12,
    &ett_rrc_T_v770NonCriticalExtensions_04,
    &ett_rrc_T_nonCriticalExtensions_70,
    &ett_rrc_T_later_than_r3_10,
    &ett_rrc_T_criticalExtensions_53,
    &ett_rrc_T_r4_04,
    &ett_rrc_T_v4d0NonCriticalExtensions_04,
    &ett_rrc_T_v590NonCriticalExtenstions_03,
    &ett_rrc_T_v690NonCriticalExtensions_13,
    &ett_rrc_T_v770NonCriticalExtensions_05,
    &ett_rrc_T_nonCriticalExtensions_71,
    &ett_rrc_T_criticalExtensions_54,
    &ett_rrc_T_r5_04,
    &ett_rrc_T_v690NonCriticalExtensions_14,
    &ett_rrc_T_v770NonCriticalExtensions_06,
    &ett_rrc_T_nonCriticalExtensions_72,
    &ett_rrc_T_criticalExtensions_55,
    &ett_rrc_T_r6_06,
    &ett_rrc_T_v6b0NonCriticalExtensions_05,
    &ett_rrc_T_v770NonCriticalExtensions_07,
    &ett_rrc_T_nonCriticalExtensions_73,
    &ett_rrc_T_criticalExtensions_56,
    &ett_rrc_T_r7_05,
    &ett_rrc_T_v780NonCriticalExtensions_04,
    &ett_rrc_T_v7d0NonCriticalExtensions_04,
    &ett_rrc_T_v7f0NonCriticalExtensions_05,
    &ett_rrc_T_v7g0NonCriticalExtensions_06,
    &ett_rrc_T_nonCriticalExtensions_74,
    &ett_rrc_T_criticalExtensions_57,
    &ett_rrc_T_r8_06,
    &ett_rrc_T_v7d0NonCriticalExtensions_05,
    &ett_rrc_T_v7f0NonCriticalExtensions_06,
    &ett_rrc_T_v890NonCriticalExtensions_04,
    &ett_rrc_T_v7g0NonCriticalExtensions_07,
    &ett_rrc_T_v8a0NonCriticalExtensions_04,
    &ett_rrc_T_nonCriticalExtensions_75,
    &ett_rrc_T_criticalExtensions_58,
    &ett_rrc_T_r9_05,
    &ett_rrc_T_v950NonCriticalExtensions_03,
    &ett_rrc_T_nonCriticalExtensions_76,
    &ett_rrc_T_criticalExtensions_59,
    &ett_rrc_T_r10_05,
    &ett_rrc_T_vbxyNonCriticalExtensions_08,
    &ett_rrc_T_nonCriticalExtensions_77,
    &ett_rrc_T_criticalExtensions_60,
    &ett_rrc_T_r11_06,
    &ett_rrc_T_nonCriticalExtensions_78,
    &ett_rrc_T_criticalExtensions_61,
    &ett_rrc_PhysicalChannelReconfiguration_r3_IEs,
    &ett_rrc_T_modeSpecificInfo_12,
    &ett_rrc_T_fdd_18,
    &ett_rrc_PhysicalChannelReconfiguration_v3a0ext,
    &ett_rrc_PhysicalChannelReconfiguration_v4b0ext_IEs,
    &ett_rrc_PhysicalChannelReconfiguration_v590ext_IEs,
    &ett_rrc_PhysicalChannelReconfiguration_r4_IEs,
    &ett_rrc_T_modeSpecificInfo_13,
    &ett_rrc_T_fdd_19,
    &ett_rrc_PhysicalChannelReconfiguration_r5_IEs,
    &ett_rrc_T_modeSpecificInfo_14,
    &ett_rrc_T_fdd_20,
    &ett_rrc_PhysicalChannelReconfiguration_r6_IEs,
    &ett_rrc_PhysicalChannelReconfiguration_v690ext_IEs,
    &ett_rrc_PhysicalChannelReconfiguration_v6b0ext_IEs,
    &ett_rrc_PhysicalChannelReconfiguration_r7_IEs,
    &ett_rrc_PhysicalChannelReconfiguration_v770ext_IEs,
    &ett_rrc_PhysicalChannelReconfiguration_v780ext_IEs,
    &ett_rrc_PhysicalChannelReconfiguration_v7d0ext_IEs,
    &ett_rrc_PhysicalChannelReconfiguration_v7f0ext_IEs,
    &ett_rrc_PhysicalChannelReconfiguration_v7g0ext_IEs,
    &ett_rrc_PhysicalChannelReconfiguration_r8_IEs,
    &ett_rrc_PhysicalChannelReconfiguration_v890ext_IEs,
    &ett_rrc_PhysicalChannelReconfiguration_v8a0ext_IEs,
    &ett_rrc_PhysicalChannelReconfiguration_r9_IEs,
    &ett_rrc_PhysicalChannelReconfiguration_v950ext_IEs,
    &ett_rrc_PhysicalChannelReconfiguration_r10_IEs,
    &ett_rrc_PhysicalChannelReconfiguration_vbxyext_IEs,
    &ett_rrc_PhysicalChannelReconfiguration_r11_IEs,
    &ett_rrc_PhysicalChannelReconfigurationComplete,
    &ett_rrc_T_laterNonCriticalExtensions_24,
    &ett_rrc_T_v770NonCriticalExtensions_08,
    &ett_rrc_T_nonCriticalExtensions_79,
    &ett_rrc_PhysicalChannelReconfigurationComplete_v770ext_IEs,
    &ett_rrc_PhysicalChannelReconfigurationFailure,
    &ett_rrc_T_laterNonCriticalExtensions_25,
    &ett_rrc_T_nonCriticalExtensions_80,
    &ett_rrc_PhysicalSharedChannelAllocation,
    &ett_rrc_T_r3_13,
    &ett_rrc_T_laterNonCriticalExtensions_26,
    &ett_rrc_T_nonCriticalExtensions_81,
    &ett_rrc_T_later_than_r3_11,
    &ett_rrc_T_criticalExtensions_62,
    &ett_rrc_T_r4_05,
    &ett_rrc_T_v4d0NonCriticalExtensions_05,
    &ett_rrc_T_v690NonCriticalExtensions_15,
    &ett_rrc_T_v770NonCriticalExtensions_09,
    &ett_rrc_T_nonCriticalExtensions_82,
    &ett_rrc_T_criticalExtensions_63,
    &ett_rrc_PhysicalSharedChannelAllocation_r3_IEs,
    &ett_rrc_PhysicalSharedChannelAllocation_r4_IEs,
    &ett_rrc_PhysicalSharedChannelAllocation_v690ext_IEs,
    &ett_rrc_PhysicalSharedChannelAllocation_v770ext_IEs,
    &ett_rrc_PUSCHCapacityRequest,
    &ett_rrc_T_allocationConfirmation,
    &ett_rrc_T_laterNonCriticalExtensions_27,
    &ett_rrc_T_v590NonCriticalExtensions_13,
    &ett_rrc_T_nonCriticalExtensions_83,
    &ett_rrc_PUSCHCapacityRequest_v590ext,
    &ett_rrc_RadioBearerReconfiguration,
    &ett_rrc_T_r3_14,
    &ett_rrc_T_v3aoNonCriticalExtensions,
    &ett_rrc_T_laterNonCriticalExtensions_28,
    &ett_rrc_T_v4b0NonCriticalExtensions_07,
    &ett_rrc_T_v590NonCriticalExtensions_14,
    &ett_rrc_T_v5d0NonCriticalExtenstions_06,
    &ett_rrc_T_v690NonCriticalExtensions_16,
    &ett_rrc_T_v770NonCriticalExtensions_10,
    &ett_rrc_T_nonCriticalExtensions_84,
    &ett_rrc_T_later_than_r3_12,
    &ett_rrc_T_criticalExtensions_64,
    &ett_rrc_T_r4_06,
    &ett_rrc_T_v4d0NonCriticalExtensions_06,
    &ett_rrc_T_v590NonCriticalExtensions_15,
    &ett_rrc_T_v5d0NonCriticalExtenstions_07,
    &ett_rrc_T_v690NonCriticalExtensions_17,
    &ett_rrc_T_v770NonCriticalExtensions_11,
    &ett_rrc_T_nonCriticalExtensions_85,
    &ett_rrc_T_criticalExtensions_65,
    &ett_rrc_T_r5_05,
    &ett_rrc_T_v5d0NonCriticalExtenstions_08,
    &ett_rrc_T_v690NonCriticalExtensions_18,
    &ett_rrc_T_v770NonCriticalExtensions_12,
    &ett_rrc_T_nonCriticalExtensions_86,
    &ett_rrc_T_criticalExtensions_66,
    &ett_rrc_T_r6_07,
    &ett_rrc_T_v6b0NonCriticalExtensions_06,
    &ett_rrc_T_v6f0NonCriticalExtensions,
    &ett_rrc_T_v770NonCriticalExtensions_13,
    &ett_rrc_T_nonCriticalExtensions_87,
    &ett_rrc_T_criticalExtensions_67,
    &ett_rrc_T_r7_06,
    &ett_rrc_T_v780NonCriticalExtensions_05,
    &ett_rrc_T_v790NonCriticalExtensions,
    &ett_rrc_T_v7d0NonCriticalExtensions_06,
    &ett_rrc_T_v7f0NonCriticalExtensions_07,
    &ett_rrc_T_v7g0NonCriticalExtensions_08,
    &ett_rrc_T_nonCriticalExtensions_88,
    &ett_rrc_T_criticalExtensions_68,
    &ett_rrc_T_r8_07,
    &ett_rrc_T_v7d0NonCriticalExtensions_07,
    &ett_rrc_T_v7f0NonCriticalExtensions_08,
    &ett_rrc_T_v890NonCriticalExtensions_05,
    &ett_rrc_T_v7g0NonCriticalExtensions_09,
    &ett_rrc_T_v8a0NonCriticalExtensions_05,
    &ett_rrc_T_nonCriticalExtensions_89,
    &ett_rrc_T_criticalExtensions_69,
    &ett_rrc_T_r9_06,
    &ett_rrc_T_v950NonCriticalExtensions_04,
    &ett_rrc_T_v9c0NonCriticalExtensions_04,
    &ett_rrc_T_nonCriticalExtensions_90,
    &ett_rrc_T_criticalExtensions_70,
    &ett_rrc_T_r10_06,
    &ett_rrc_T_v9c0NonCriticalExtensions_05,
    &ett_rrc_T_vaa0NonCriticalExtensions_01,
    &ett_rrc_T_vbxyNonCriticalExtensions_09,
    &ett_rrc_T_nonCriticalExtensions_91,
    &ett_rrc_T_criticalExtensions_71,
    &ett_rrc_T_r11_07,
    &ett_rrc_T_nonCriticalExtensions_92,
    &ett_rrc_T_criticalExtensions_72,
    &ett_rrc_RadioBearerReconfiguration_vaa0ext_IEs,
    &ett_rrc_RadioBearerReconfiguration_r3_IEs,
    &ett_rrc_T_dummy_01,
    &ett_rrc_T_fdd_21,
    &ett_rrc_T_modeSpecificPhysChInfo_03,
    &ett_rrc_T_fdd_22,
    &ett_rrc_RadioBearerReconfiguration_v3a0ext,
    &ett_rrc_RadioBearerReconfiguration_v4b0ext_IEs,
    &ett_rrc_RadioBearerReconfiguration_v590ext_IEs,
    &ett_rrc_RadioBearerReconfiguration_v5d0ext_IEs,
    &ett_rrc_RadioBearerReconfiguration_r4_IEs,
    &ett_rrc_T_dummy_02,
    &ett_rrc_T_fdd_23,
    &ett_rrc_T_modeSpecificPhysChInfo_04,
    &ett_rrc_T_fdd_24,
    &ett_rrc_RadioBearerReconfiguration_r5_IEs,
    &ett_rrc_T_specificationMode_09,
    &ett_rrc_T_complete_09,
    &ett_rrc_T_dummy_03,
    &ett_rrc_T_fdd_25,
    &ett_rrc_T_preconfiguration_09,
    &ett_rrc_T_preConfigMode_09,
    &ett_rrc_T_defaultConfig_09,
    &ett_rrc_T_modeSpecificPhysChInfo_05,
    &ett_rrc_T_fdd_26,
    &ett_rrc_RadioBearerReconfiguration_r6_IEs,
    &ett_rrc_T_specificationMode_10,
    &ett_rrc_T_complete_10,
    &ett_rrc_T_preconfiguration_10,
    &ett_rrc_T_preConfigMode_10,
    &ett_rrc_T_defaultConfig_10,
    &ett_rrc_RadioBearerReconfiguration_v690ext_IEs,
    &ett_rrc_RadioBearerReconfiguration_v6b0ext_IEs,
    &ett_rrc_RadioBearerReconfiguration_v6f0ext_IEs,
    &ett_rrc_T_specificationMode_11,
    &ett_rrc_T_complete_11,
    &ett_rrc_RadioBearerReconfiguration_r7_IEs,
    &ett_rrc_T_specificationMode_12,
    &ett_rrc_T_complete_12,
    &ett_rrc_T_preconfiguration_11,
    &ett_rrc_T_preConfigMode_11,
    &ett_rrc_T_defaultConfig_11,
    &ett_rrc_RadioBearerReconfiguration_v770ext_IEs,
    &ett_rrc_RadioBearerReconfiguration_v780ext_IEs,
    &ett_rrc_RadioBearerReconfiguration_v790ext_IEs,
    &ett_rrc_RadioBearerReconfiguration_v7d0ext_IEs,
    &ett_rrc_RadioBearerReconfiguration_v7f0ext_IEs,
    &ett_rrc_RadioBearerReconfiguration_v7g0ext_IEs,
    &ett_rrc_RadioBearerReconfiguration_r8_IEs,
    &ett_rrc_T_specificationMode_13,
    &ett_rrc_T_complete_13,
    &ett_rrc_T_preconfiguration_12,
    &ett_rrc_T_preConfigMode_12,
    &ett_rrc_T_defaultConfig_12,
    &ett_rrc_RadioBearerReconfiguration_v890ext_IEs,
    &ett_rrc_RadioBearerReconfiguration_v8a0ext_IEs,
    &ett_rrc_RadioBearerReconfiguration_r9_IEs,
    &ett_rrc_T_specificationMode_14,
    &ett_rrc_T_complete_14,
    &ett_rrc_T_preconfiguration_13,
    &ett_rrc_T_preConfigMode_13,
    &ett_rrc_T_defaultConfig_13,
    &ett_rrc_RadioBearerReconfiguration_v950ext_IEs,
    &ett_rrc_RadioBearerReconfiguration_v9c0ext_IEs,
    &ett_rrc_RadioBearerReconfiguration_r10_IEs,
    &ett_rrc_T_specificationMode_15,
    &ett_rrc_T_complete_15,
    &ett_rrc_T_preconfiguration_14,
    &ett_rrc_T_preConfigMode_14,
    &ett_rrc_T_defaultConfig_14,
    &ett_rrc_RadioBearerReconfiguration_vbxyext_IEs,
    &ett_rrc_RadioBearerReconfiguration_r11_IEs,
    &ett_rrc_T_specificationMode_16,
    &ett_rrc_T_complete_16,
    &ett_rrc_T_preconfiguration_15,
    &ett_rrc_T_preConfigMode_15,
    &ett_rrc_T_defaultConfig_15,
    &ett_rrc_RadioBearerReconfigurationComplete,
    &ett_rrc_T_laterNonCriticalExtensions_29,
    &ett_rrc_T_v770NonCriticalExtensions_14,
    &ett_rrc_T_nonCriticalExtensions_93,
    &ett_rrc_RadioBearerReconfigurationComplete_v770ext_IEs,
    &ett_rrc_RadioBearerReconfigurationFailure,
    &ett_rrc_T_laterNonCriticalExtensions_30,
    &ett_rrc_T_nonCriticalExtensions_94,
    &ett_rrc_RadioBearerRelease,
    &ett_rrc_T_r3_15,
    &ett_rrc_T_v3a0NonCriticalExtensions_06,
    &ett_rrc_T_laterNonCriticalExtensions_31,
    &ett_rrc_T_v4b0NonCriticalExtensions_08,
    &ett_rrc_T_v590NonCriticalExtensions_16,
    &ett_rrc_T_v690NonCriticalExtensions_19,
    &ett_rrc_T_v770NonCriticalExtensions_15,
    &ett_rrc_T_nonCriticalExtensions_95,
    &ett_rrc_T_later_than_r3_13,
    &ett_rrc_T_criticalExtensions_73,
    &ett_rrc_T_r4_07,
    &ett_rrc_T_v4d0NonCriticalExtensions_07,
    &ett_rrc_T_v590NonCriticalExtensions_17,
    &ett_rrc_T_v690NonCriticalExtensions_20,
    &ett_rrc_T_v770NonCriticalExtensions_16,
    &ett_rrc_T_nonCriticalExtensions_96,
    &ett_rrc_T_criticalExtensions_74,
    &ett_rrc_T_r5_06,
    &ett_rrc_T_v690NonCriticalExtensions_21,
    &ett_rrc_T_v770NonCriticalExtensions_17,
    &ett_rrc_T_nonCriticalExtensions_97,
    &ett_rrc_T_criticalExtensions_75,
    &ett_rrc_T_r6_08,
    &ett_rrc_T_v6b0NonCriticalExtensions_07,
    &ett_rrc_T_v770NonCriticalExtensions_18,
    &ett_rrc_T_nonCriticalExtensions_98,
    &ett_rrc_T_criticalExtensions_76,
    &ett_rrc_T_r7_07,
    &ett_rrc_T_v780NonCriticalExtensions_06,
    &ett_rrc_T_v7d0NonCriticalExtensions_08,
    &ett_rrc_T_v7f0NonCriticalExtensions_09,
    &ett_rrc_T_v7g0NonCriticalExtensions_10,
    &ett_rrc_T_nonCriticalExtensions_99,
    &ett_rrc_T_criticalExtensions_77,
    &ett_rrc_T_r8_08,
    &ett_rrc_T_v7d0NonCriticalExtensions_09,
    &ett_rrc_T_v7f0NonCriticalExtensions_10,
    &ett_rrc_T_v890NonCriticalExtensions_06,
    &ett_rrc_T_v7g0NonCriticalExtensions_11,
    &ett_rrc_T_v8a0NonCriticalExtensions_06,
    &ett_rrc_T_nonCriticalExtensions_100,
    &ett_rrc_T_criticalExtensions_78,
    &ett_rrc_T_r9_07,
    &ett_rrc_T_v950NonCriticalExtensions_05,
    &ett_rrc_T_v970NonCriticalExtensions_02,
    &ett_rrc_T_v9c0NonCriticalExtensions_06,
    &ett_rrc_T_nonCriticalExtensions_101,
    &ett_rrc_T_criticalExtensions_79,
    &ett_rrc_T_r10_07,
    &ett_rrc_T_v9c0NonCriticalExtensions_07,
    &ett_rrc_T_vbxyNonCriticalExtensions_10,
    &ett_rrc_T_nonCriticalExtensions_102,
    &ett_rrc_T_criticalExtensions_80,
    &ett_rrc_T_r11_08,
    &ett_rrc_T_nonCriticalExtensions_103,
    &ett_rrc_T_criticalExtensions_81,
    &ett_rrc_RadioBearerRelease_r3_IEs,
    &ett_rrc_T_dummy_04,
    &ett_rrc_T_fdd_27,
    &ett_rrc_T_modeSpecificPhysChInfo_06,
    &ett_rrc_T_fdd_28,
    &ett_rrc_RadioBearerRelease_v3a0ext,
    &ett_rrc_RadioBearerRelease_v4b0ext_IEs,
    &ett_rrc_RadioBearerRelease_v590ext_IEs,
    &ett_rrc_RadioBearerRelease_r4_IEs,
    &ett_rrc_T_dummy_05,
    &ett_rrc_T_fdd_29,
    &ett_rrc_T_modeSpecificPhysChInfo_07,
    &ett_rrc_T_fdd_30,
    &ett_rrc_RadioBearerRelease_r5_IEs,
    &ett_rrc_T_dummy_06,
    &ett_rrc_T_fdd_31,
    &ett_rrc_T_modeSpecificPhysChInfo_08,
    &ett_rrc_T_fdd_32,
    &ett_rrc_RadioBearerRelease_v690ext_IEs,
    &ett_rrc_RadioBearerRelease_r6_IEs,
    &ett_rrc_RadioBearerRelease_v6b0ext_IEs,
    &ett_rrc_RadioBearerRelease_r7_IEs,
    &ett_rrc_RadioBearerRelease_v770ext_IEs,
    &ett_rrc_RadioBearerRelease_v780ext_IEs,
    &ett_rrc_RadioBearerRelease_v7d0ext_IEs,
    &ett_rrc_RadioBearerRelease_v7f0ext_IEs,
    &ett_rrc_RadioBearerRelease_v7g0ext_IEs,
    &ett_rrc_RadioBearerRelease_r8_IEs,
    &ett_rrc_RadioBearerRelease_v890ext_IEs,
    &ett_rrc_RadioBearerRelease_v8a0ext_IEs,
    &ett_rrc_RadioBearerRelease_r9_IEs,
    &ett_rrc_RadioBearerRelease_v950ext_IEs,
    &ett_rrc_RadioBearerRelease_v970ext_IEs,
    &ett_rrc_RadioBearerRelease_v9c0ext_IEs,
    &ett_rrc_RadioBearerRelease_r10_IEs,
    &ett_rrc_RadioBearerRelease_vbxyext_IEs,
    &ett_rrc_RadioBearerRelease_r11_IEs,
    &ett_rrc_RadioBearerReleaseComplete,
    &ett_rrc_T_laterNonCriticalExtensions_32,
    &ett_rrc_T_v770NonCriticalExtensions_19,
    &ett_rrc_T_nonCriticalExtensions_104,
    &ett_rrc_RadioBearerReleaseComplete_v770ext_IEs,
    &ett_rrc_RadioBearerReleaseFailure,
    &ett_rrc_T_laterNonCriticalExtensions_33,
    &ett_rrc_T_nonCriticalExtensions_105,
    &ett_rrc_RadioBearerSetup,
    &ett_rrc_T_r3_16,
    &ett_rrc_T_v3a0NonCriticalExtensions_07,
    &ett_rrc_T_laterNonCriticalExtensions_34,
    &ett_rrc_T_v4b0NonCriticalExtensions_09,
    &ett_rrc_T_v590NonCriticalExtensions_18,
    &ett_rrc_T_v5d0NonCriticalExtenstions_09,
    &ett_rrc_T_v690NonCriticalExtensions_22,
    &ett_rrc_T_nonCriticalExtensions_106,
    &ett_rrc_T_later_than_r3_14,
    &ett_rrc_T_criticalExtensions_82,
    &ett_rrc_T_r4_08,
    &ett_rrc_T_v4d0NonCriticalExtensions_08,
    &ett_rrc_T_v590NonCriticalExtensions_19,
    &ett_rrc_T_v5d0NonCriticalExtenstions_10,
    &ett_rrc_T_v690NonCriticalExtensions_23,
    &ett_rrc_T_nonCriticalExtensions_107,
    &ett_rrc_T_criticalExtensions_83,
    &ett_rrc_T_r5_07,
    &ett_rrc_T_v5d0NonCriticalExtenstions_11,
    &ett_rrc_T_v690NonCriticalExtensions_24,
    &ett_rrc_T_nonCriticalExtensions_108,
    &ett_rrc_T_criticalExtensions_84,
    &ett_rrc_T_r6_09,
    &ett_rrc_T_v6b0NonCriticalExtensions_08,
    &ett_rrc_T_nonCriticalExtensions_109,
    &ett_rrc_T_criticalExtensions_85,
    &ett_rrc_T_r7_08,
    &ett_rrc_T_v780NonCriticalExtensions_07,
    &ett_rrc_T_v820NonCriticalExtensions_01,
    &ett_rrc_T_nonCriticalExtensions_110,
    &ett_rrc_T_criticalExtensions_86,
    &ett_rrc_T_r8_09,
    &ett_rrc_T_v7d0NonCriticalExtensions_10,
    &ett_rrc_T_v7f0NonCriticalExtensions_11,
    &ett_rrc_T_v890NonCriticalExtensions_07,
    &ett_rrc_T_v7g0NonCriticalExtensions_12,
    &ett_rrc_T_v8a0NonCriticalExtensions_07,
    &ett_rrc_T_nonCriticalExtensions_111,
    &ett_rrc_T_criticalExtensions_87,
    &ett_rrc_T_r9_08,
    &ett_rrc_T_v950NonCriticalExtensions_06,
    &ett_rrc_T_v9c0NonCriticalExtensions_08,
    &ett_rrc_T_nonCriticalExtensions_112,
    &ett_rrc_T_criticalExtensions_88,
    &ett_rrc_T_r10_08,
    &ett_rrc_T_v9c0NonCriticalExtensions_09,
    &ett_rrc_T_vbxyNonCriticalExtensions_11,
    &ett_rrc_T_nonCriticalExtensions_113,
    &ett_rrc_T_criticalExtensions_89,
    &ett_rrc_T_r11_09,
    &ett_rrc_T_nonCriticalExtensions_114,
    &ett_rrc_T_criticalExtensions_90,
    &ett_rrc_RadioBearerSetup_r3_IEs,
    &ett_rrc_T_dummy_07,
    &ett_rrc_T_fdd_33,
    &ett_rrc_T_modeSpecificPhysChInfo_09,
    &ett_rrc_T_fdd_34,
    &ett_rrc_RadioBearerSetup_v3a0ext,
    &ett_rrc_RadioBearerSetup_v4b0ext_IEs,
    &ett_rrc_RadioBearerSetup_v590ext_IEs,
    &ett_rrc_RadioBearerSetup_v5d0ext_IEs,
    &ett_rrc_RadioBearerSetup_r4_IEs,
    &ett_rrc_T_dummy_08,
    &ett_rrc_T_fdd_35,
    &ett_rrc_T_modeSpecificPhysChInfo_10,
    &ett_rrc_T_fdd_36,
    &ett_rrc_RadioBearerSetup_r5_IEs,
    &ett_rrc_T_dummy_09,
    &ett_rrc_T_fdd_37,
    &ett_rrc_T_modeSpecificPhysChInfo_11,
    &ett_rrc_T_fdd_38,
    &ett_rrc_RadioBearerSetup_v690ext_IEs,
    &ett_rrc_RadioBearerSetup_r6_IEs,
    &ett_rrc_T_specificationMode_17,
    &ett_rrc_T_complete_17,
    &ett_rrc_T_dummy_10,
    &ett_rrc_RadioBearerSetup_v6b0ext_IEs,
    &ett_rrc_RadioBearerSetup_r7_IEs,
    &ett_rrc_T_specificationMode_18,
    &ett_rrc_T_complete_18,
    &ett_rrc_RadioBearerSetup_r7_add_ext_IEs,
    &ett_rrc_T_v7f0NonCriticalExtensions_12,
    &ett_rrc_T_v7g0NonCriticalExtensions_13,
    &ett_rrc_T_nonCriticalExtensions_115,
    &ett_rrc_RadioBearerSetup_v780ext_IEs,
    &ett_rrc_RadioBearerSetup_v7d0ext_IEs,
    &ett_rrc_RadioBearerSetup_v7f0ext_IEs,
    &ett_rrc_RadioBearerSetup_v7g0ext_IEs,
    &ett_rrc_RadioBearerSetup_r8_IEs,
    &ett_rrc_RadioBearerSetup_v820ext_IEs,
    &ett_rrc_RadioBearerSetup_v890ext_IEs,
    &ett_rrc_RadioBearerSetup_v8a0ext_IEs,
    &ett_rrc_RadioBearerSetup_r9_IEs,
    &ett_rrc_RadioBearerSetup_v950ext_IEs,
    &ett_rrc_RadioBearerSetup_v9c0ext_IEs,
    &ett_rrc_RadioBearerSetup_r10_IEs,
    &ett_rrc_RadioBearerSetup_vbxyext_IEs,
    &ett_rrc_RadioBearerSetup_r11_IEs,
    &ett_rrc_RadioBearerSetupComplete,
    &ett_rrc_T_laterNonCriticalExtensions_35,
    &ett_rrc_T_v770NonCriticalExtensions_20,
    &ett_rrc_T_nonCriticalExtensions_116,
    &ett_rrc_RadioBearerSetupComplete_v770ext_IEs,
    &ett_rrc_RadioBearerSetupFailure,
    &ett_rrc_T_laterNonCriticalExtensions_36,
    &ett_rrc_T_nonCriticalExtensions_117,
    &ett_rrc_RRCConnectionReject,
    &ett_rrc_T_r3_17,
    &ett_rrc_T_laterNonCriticalExtensions_37,
    &ett_rrc_T_v690NonCriticalExtensions_25,
    &ett_rrc_T_v6f0NonCriticalExtensions_01,
    &ett_rrc_T_v860NonCriticalExtensions_10,
    &ett_rrc_T_va40NonCriticalExtensions_05,
    &ett_rrc_T_nonCriticalExtensions_118,
    &ett_rrc_T_later_than_r3_15,
    &ett_rrc_T_criticalExtensions_91,
    &ett_rrc_RRCConnectionReject_r3_IEs,
    &ett_rrc_RRCConnectionReject_v690ext_IEs,
    &ett_rrc_RRCConnectionReject_v6f0ext_IEs,
    &ett_rrc_RRCConnectionReject_v860ext_IEs,
    &ett_rrc_RRCConnectionReject_va40ext_IEs,
    &ett_rrc_RRCConnectionRelease,
    &ett_rrc_T_r3_18,
    &ett_rrc_T_laterNonCriticalExtensions_38,
    &ett_rrc_T_v690NonCriticalExtensions_26,
    &ett_rrc_T_v770NonCriticalExtensions_21,
    &ett_rrc_T_v860NonCriticalExtensions_11,
    &ett_rrc_T_va40NonCriticalExtensions_06,
    &ett_rrc_T_nonCriticalExtensions_119,
    &ett_rrc_T_later_than_r3_16,
    &ett_rrc_T_criticalExtensions_92,
    &ett_rrc_T_r4_09,
    &ett_rrc_T_v4d0NonCriticalExtensions_09,
    &ett_rrc_T_v690NonCriticalExtensions_27,
    &ett_rrc_T_v770NonCriticalExtensions_22,
    &ett_rrc_T_v860NonCriticalExtensions_12,
    &ett_rrc_T_va40NonCriticalExtensions_07,
    &ett_rrc_T_nonCriticalExtensions_120,
    &ett_rrc_T_criticalExtensions_93,
    &ett_rrc_RRCConnectionRelease_r3_IEs,
    &ett_rrc_RRCConnectionRelease_r4_IEs,
    &ett_rrc_RRCConnectionRelease_v690ext_IEs,
    &ett_rrc_RRCConnectionRelease_v770ext_IEs,
    &ett_rrc_RRCConnectionRelease_v860ext_IEs,
    &ett_rrc_RRCConnectionRelease_va40ext_IEs,
    &ett_rrc_RRCConnectionRelease_CCCH,
    &ett_rrc_T_r3_19,
    &ett_rrc_T_laterNonCriticalExtensions_39,
    &ett_rrc_T_v690NonCriticalExtensions_28,
    &ett_rrc_T_v860NonCriticalExtensions_13,
    &ett_rrc_T_va40NonCriticalExtensions_08,
    &ett_rrc_T_nonCriticalExtensions_121,
    &ett_rrc_T_later_than_r3_17,
    &ett_rrc_T_criticalExtensions_94,
    &ett_rrc_T_r4_10,
    &ett_rrc_T_v4d0NonCriticalExtensions_10,
    &ett_rrc_T_v690NonCriticalExtensions_29,
    &ett_rrc_T_v860NonCriticalExtensions_14,
    &ett_rrc_T_va40NonCriticalExtensions_09,
    &ett_rrc_T_nonCriticalExtensions_122,
    &ett_rrc_T_criticalExtensions_95,
    &ett_rrc_SEQUENCE_SIZE_1_maxURNTI_Group_OF_GroupReleaseInformation,
    &ett_rrc_T_criticalExtensions_96,
    &ett_rrc_T_r5_08,
    &ett_rrc_T_v690NonCriticalExtensions_30,
    &ett_rrc_T_v860NonCriticalExtensions_15,
    &ett_rrc_T_va40NonCriticalExtensions_10,
    &ett_rrc_T_nonCriticalExtensions_123,
    &ett_rrc_T_criticalExtensions_97,
    &ett_rrc_RRCConnectionRelease_CCCH_r3_IEs,
    &ett_rrc_RRCConnectionRelease_CCCH_r4_IEs,
    &ett_rrc_RRCConnectionReleaseComplete,
    &ett_rrc_T_laterNonCriticalExtensions_40,
    &ett_rrc_T_nonCriticalExtensions_124,
    &ett_rrc_RRCConnectionRequest,
    &ett_rrc_T_v3d0NonCriticalExtensions,
    &ett_rrc_T_v4b0NonCriticalExtensions_10,
    &ett_rrc_T_v590NonCriticalExtensions_20,
    &ett_rrc_T_v690NonCriticalExtensions_31,
    &ett_rrc_T_v6b0NonCriticalExtensions_09,
    &ett_rrc_T_v6e0NonCriticalExtensions_01,
    &ett_rrc_T_v770NonCriticalExtensions_23,
    &ett_rrc_T_v7b0NonCriticalExtensions_01,
    &ett_rrc_T_v860NonCriticalExtensions_16,
    &ett_rrc_T_v7e0NonCriticalExtensions_01,
    &ett_rrc_T_v7g0NonCriticalExtensions_14,
    &ett_rrc_T_v920NonCriticalExtensions_02,
    &ett_rrc_T_v940NonCriticalExtensions,
    &ett_rrc_RRCConnectionRequest_v3d0ext_IEs,
    &ett_rrc_RRCConnectionRequest_v4b0ext_IEs,
    &ett_rrc_RRCConnectionRequest_v590ext_IEs,
    &ett_rrc_RRCConnectionRequest_v690ext_IEs,
    &ett_rrc_T_domainIndicator,
    &ett_rrc_T_cs_domain,
    &ett_rrc_RRCConnectionRequest_v6b0ext_IEs,
    &ett_rrc_RRCConnectionRequest_v6e0ext_IEs,
    &ett_rrc_RRCConnectionRequest_v770ext_IEs,
    &ett_rrc_RRCConnectionRequest_v7b0ext_IEs,
    &ett_rrc_RRCConnectionRequest_v7e0ext_IEs,
    &ett_rrc_RRCConnectionRequest_v7g0ext_IEs,
    &ett_rrc_RRCConnectionRequest_v860ext_IEs,
    &ett_rrc_RRCConnectionRequest_v920ext_IEs,
    &ett_rrc_RRCConnectionRequest_v940ext_IEs,
    &ett_rrc_RRCConnectionRequest_v9b0ext_IEs,
    &ett_rrc_RRCConnectionRequest_NonCriticalExts_va40_IEs,
    &ett_rrc_T_v9b0NonCriticalExtensions,
    &ett_rrc_T_vbxyNonCriticalExtensions_12,
    &ett_rrc_T_nonCriticalExtensions_125,
    &ett_rrc_RRCConnectionRequest_va40ext_IEs,
    &ett_rrc_RRCConnectionRequest_vbxyext_IEs,
    &ett_rrc_RRCConnectionSetup,
    &ett_rrc_T_r3_20,
    &ett_rrc_T_laterNonCriticalExtensions_41,
    &ett_rrc_T_v4b0NonCriticalExtensions_11,
    &ett_rrc_T_v590NonCriticalExtensions_21,
    &ett_rrc_T_v690NonCriticalExtensions_32,
    &ett_rrc_T_nonCriticalExtensions_126,
    &ett_rrc_T_later_than_r3_18,
    &ett_rrc_T_criticalExtensions_98,
    &ett_rrc_T_r4_11,
    &ett_rrc_T_v4d0NonCriticalExtensions_11,
    &ett_rrc_T_v590NonCriticalExtensions_22,
    &ett_rrc_T_v690NonCriticalExtensions_33,
    &ett_rrc_T_nonCriticalExtensions_127,
    &ett_rrc_T_criticalExtensions_99,
    &ett_rrc_T_r5_09,
    &ett_rrc_T_v690NonCriticalExtensions_34,
    &ett_rrc_T_nonCriticalExtensions_128,
    &ett_rrc_T_criticalExtensions_100,
    &ett_rrc_T_r6_10,
    &ett_rrc_T_v6b0NonCriticalExtensions_10,
    &ett_rrc_T_nonCriticalExtensions_129,
    &ett_rrc_T_criticalExtensions_101,
    &ett_rrc_T_r7_09,
    &ett_rrc_T_v780NonCriticalExtensions_08,
    &ett_rrc_T_v7d0NonCriticalExtensions_11,
    &ett_rrc_T_nonCriticalExtensions_130,
    &ett_rrc_T_criticalExtensions_102,
    &ett_rrc_T_r8_10,
    &ett_rrc_T_v7d0NonCriticalExtensions_12,
    &ett_rrc_T_v890NonCriticalExtensions_08,
    &ett_rrc_T_v8a0NonCriticalExtensions_08,
    &ett_rrc_T_nonCriticalExtensions_131,
    &ett_rrc_T_criticalExtensions_103,
    &ett_rrc_T_r9_09,
    &ett_rrc_T_v950NonCriticalExtensions_07,
    &ett_rrc_T_nonCriticalExtensions_132,
    &ett_rrc_T_criticalExtensions_104,
    &ett_rrc_T_r10_09,
    &ett_rrc_T_nonCriticalExtensions_133,
    &ett_rrc_T_criticalExtensions_105,
    &ett_rrc_T_r11_10,
    &ett_rrc_T_nonCriticalExtensions_134,
    &ett_rrc_T_criticalExtensions_106,
    &ett_rrc_RRCConnectionSetup_r3_IEs,
    &ett_rrc_RRCConnectionSetup_v4b0ext_IEs,
    &ett_rrc_RRCConnectionSetup_v590ext_IEs,
    &ett_rrc_RRCConnectionSetup_r4_IEs,
    &ett_rrc_RRCConnectionSetup_r5_IEs,
    &ett_rrc_T_specificationMode_19,
    &ett_rrc_T_complete_19,
    &ett_rrc_T_preconfiguration_16,
    &ett_rrc_T_preConfigMode_16,
    &ett_rrc_T_defaultConfig_16,
    &ett_rrc_RRCConnectionSetup_v690ext_IEs,
    &ett_rrc_RRCConnectionSetup_r6_IEs,
    &ett_rrc_T_specificationMode_20,
    &ett_rrc_T_complete_20,
    &ett_rrc_T_preconfiguration_17,
    &ett_rrc_T_preConfigMode_17,
    &ett_rrc_T_defaultConfig_17,
    &ett_rrc_RRCConnectionSetup_v6b0ext_IEs,
    &ett_rrc_RRCConnectionSetup_r7_IEs,
    &ett_rrc_T_specificationMode_21,
    &ett_rrc_T_complete_21,
    &ett_rrc_T_preconfiguration_18,
    &ett_rrc_T_preConfigMode_18,
    &ett_rrc_T_defaultConfig_18,
    &ett_rrc_RRCConnectionSetup_v780ext_IEs,
    &ett_rrc_RRCConnectionSetup_v7d0ext_IEs,
    &ett_rrc_RRCConnectionSetup_r8_IEs,
    &ett_rrc_T_specificationMode_22,
    &ett_rrc_T_complete_22,
    &ett_rrc_T_preconfiguration_19,
    &ett_rrc_T_preConfigMode_19,
    &ett_rrc_T_defaultConfig_19,
    &ett_rrc_RRCConnectionSetup_v890ext_IEs,
    &ett_rrc_RRCConnectionSetup_v8a0ext_IEs,
    &ett_rrc_RRCConnectionSetup_r9_IEs,
    &ett_rrc_T_specificationMode_23,
    &ett_rrc_T_complete_23,
    &ett_rrc_T_preconfiguration_20,
    &ett_rrc_T_preConfigMode_20,
    &ett_rrc_T_defaultConfig_20,
    &ett_rrc_RRCConnectionSetup_v950ext_IEs,
    &ett_rrc_RRCConnectionSetup_r10_IEs,
    &ett_rrc_T_specificationMode_24,
    &ett_rrc_T_complete_24,
    &ett_rrc_T_preconfiguration_21,
    &ett_rrc_T_preConfigMode_21,
    &ett_rrc_T_defaultConfig_21,
    &ett_rrc_RRCConnectionSetup_r11_IEs,
    &ett_rrc_T_specificationMode_25,
    &ett_rrc_T_complete_25,
    &ett_rrc_T_preconfiguration_22,
    &ett_rrc_T_preConfigMode_22,
    &ett_rrc_T_defaultConfig_22,
    &ett_rrc_RRCConnectionSetupComplete,
    &ett_rrc_T_v370NonCriticalExtensions,
    &ett_rrc_T_v380NonCriticalExtensions,
    &ett_rrc_T_v3a0NonCriticalExtensions_08,
    &ett_rrc_T_laterNonCriticalExtensions_42,
    &ett_rrc_T_v3g0NonCriticalExtensions_01,
    &ett_rrc_T_v4b0NonCriticalExtensions_12,
    &ett_rrc_T_v590NonCriticalExtensions_23,
    &ett_rrc_T_v5c0NonCriticalExtensions,
    &ett_rrc_T_v690NonCriticalExtensions_35,
    &ett_rrc_T_v770NonCriticalExtensions_24,
    &ett_rrc_T_va40NonCriticalExtensions_11,
    &ett_rrc_T_vbxyNonCriticalExtensions_13,
    &ett_rrc_T_nonCriticalExtensions_135,
    &ett_rrc_RRCConnectionSetupComplete_v370ext,
    &ett_rrc_RRCConnectionSetupComplete_v380ext_IEs,
    &ett_rrc_RRCConnectionSetupComplete_v3a0ext_IEs,
    &ett_rrc_RRCConnectionSetupComplete_v3g0ext_IEs,
    &ett_rrc_RRCConnectionSetupComplete_r3_add_ext_IEs,
    &ett_rrc_T_v680NonCriticalExtensions,
    &ett_rrc_T_v7e0NonCriticalExtensions_02,
    &ett_rrc_T_v7f0NonCriticalExtensions_13,
    &ett_rrc_T_va40NonCriticalExtensions_12,
    &ett_rrc_T_nonCriticalExtensions_136,
    &ett_rrc_RRCConnectionSetupComplete_v4b0ext_IEs,
    &ett_rrc_RRCConnectionSetupComplete_v590ext_IEs,
    &ett_rrc_RRCConnectionSetupComplete_v5c0ext_IEs,
    &ett_rrc_RRCConnectionSetupComplete_v650ext_IEs,
    &ett_rrc_RRCConnectionSetupComplete_v680ext_IEs,
    &ett_rrc_RRCConnectionSetupComplete_v690ext_IEs,
    &ett_rrc_RRCConnectionSetupComplete_v770ext_IEs,
    &ett_rrc_RRCConnectionSetupComplete_v7e0ext_IEs,
    &ett_rrc_RRCConnectionSetupComplete_v7f0ext_IEs,
    &ett_rrc_RRCConnectionSetupComplete_va40ext_IEs,
    &ett_rrc_RRCConnectionSetupCompleteBand_va40ext_IEs,
    &ett_rrc_RRCConnectionSetupComplete_vbxyext_IEs,
    &ett_rrc_RRC_FailureInfo,
    &ett_rrc_T_r3_21,
    &ett_rrc_T_laterNonCriticalExtensions_43,
    &ett_rrc_T_nonCriticalExtensions_137,
    &ett_rrc_T_criticalExtensions_107,
    &ett_rrc_RRC_FailureInfo_r3_IEs,
    &ett_rrc_RRCStatus,
    &ett_rrc_T_laterNonCriticalExtensions_44,
    &ett_rrc_T_nonCriticalExtensions_138,
    &ett_rrc_SecurityModeCommand,
    &ett_rrc_T_r3_22,
    &ett_rrc_T_laterNonCriticalExtensions_45,
    &ett_rrc_T_nonCriticalExtensions_139,
    &ett_rrc_T_later_than_r3_19,
    &ett_rrc_T_criticalExtensions_108,
    &ett_rrc_T_r7_10,
    &ett_rrc_T_nonCriticalExtensions_140,
    &ett_rrc_T_criticalExtensions_109,
    &ett_rrc_SecurityModeCommand_r3_IEs,
    &ett_rrc_SecurityModeCommand_r7_IEs,
    &ett_rrc_SecurityModeComplete,
    &ett_rrc_T_laterNonCriticalExtensions_46,
    &ett_rrc_T_nonCriticalExtensions_141,
    &ett_rrc_SecurityModeFailure,
    &ett_rrc_T_laterNonCriticalExtensions_47,
    &ett_rrc_T_nonCriticalExtensions_142,
    &ett_rrc_SignallingConnectionRelease,
    &ett_rrc_T_r3_23,
    &ett_rrc_T_laterNonCriticalExtensions_48,
    &ett_rrc_T_va40NonCriticalExtensions_13,
    &ett_rrc_T_nonCriticalExtensions_143,
    &ett_rrc_T_later_than_r3_20,
    &ett_rrc_T_criticalExtensions_110,
    &ett_rrc_SignallingConnectionRelease_r3_IEs,
    &ett_rrc_SignallingConnectionRelease_va40ext_IEs,
    &ett_rrc_SignallingConnectionReleaseIndication,
    &ett_rrc_T_laterNonCriticalExtensions_49,
    &ett_rrc_T_v860nonCriticalExtentions,
    &ett_rrc_T_nonCriticalExtensions_144,
    &ett_rrc_SignallingConnectionReleaseIndication_v860ext,
    &ett_rrc_SystemInformation_BCH,
    &ett_rrc_T_payload,
    &ett_rrc_T_lastAndFirst,
    &ett_rrc_T_lastAndComplete,
    &ett_rrc_T_lastAndCompleteAndFirst,
    &ett_rrc_T_completeAndFirst,
    &ett_rrc_SystemInformation_FACH,
    &ett_rrc_T_payload_01,
    &ett_rrc_T_lastAndFirst_01,
    &ett_rrc_T_lastAndComplete_01,
    &ett_rrc_T_lastAndCompleteAndFirst_01,
    &ett_rrc_T_completeAndFirst_01,
    &ett_rrc_FirstSegment,
    &ett_rrc_FirstSegmentShort,
    &ett_rrc_SubsequentSegment,
    &ett_rrc_LastSegment,
    &ett_rrc_LastSegmentShort,
    &ett_rrc_CompleteSIB_List,
    &ett_rrc_CompleteSIB,
    &ett_rrc_CompleteSIBshort,
    &ett_rrc_SystemInformationChangeIndication,
    &ett_rrc_T_laterNonCriticalExtensions_50,
    &ett_rrc_T_v860NonCriticalExtensions_17,
    &ett_rrc_T_nonCriticalExtensions_145,
    &ett_rrc_SystemInformationChangeIndication_v860ext_IEs,
    &ett_rrc_TransportChannelReconfiguration,
    &ett_rrc_T_r3_24,
    &ett_rrc_T_v3a0NonCriticalExtensions_09,
    &ett_rrc_T_laterNonCriticalExtensions_51,
    &ett_rrc_T_v4b0NonCriticalExtensions_13,
    &ett_rrc_T_v590NonCriticalExtensions_24,
    &ett_rrc_T_v690NonCriticalExtensions_36,
    &ett_rrc_T_v770NonCriticalExtensions_25,
    &ett_rrc_T_nonCriticalExtensions_146,
    &ett_rrc_T_later_than_r3_21,
    &ett_rrc_T_criticalExtensions_111,
    &ett_rrc_T_r4_12,
    &ett_rrc_T_v4d0NonCriticalExtensions_12,
    &ett_rrc_T_v590NonCriticalExtensions_25,
    &ett_rrc_T_v690NonCriticalExtensions_37,
    &ett_rrc_T_v770NonCriticalExtensions_26,
    &ett_rrc_T_nonCriticalExtensions_147,
    &ett_rrc_T_criticalExtensions_112,
    &ett_rrc_T_r5_10,
    &ett_rrc_T_v690NonCriticalExtensions_38,
    &ett_rrc_T_v770NonCriticalExtensions_27,
    &ett_rrc_T_nonCriticalExtensions_148,
    &ett_rrc_T_criticalExtensions_113,
    &ett_rrc_T_r6_11,
    &ett_rrc_T_v6b0NonCriticalExtensions_11,
    &ett_rrc_T_v770NonCriticalExtensions_28,
    &ett_rrc_T_nonCriticalExtensions_149,
    &ett_rrc_T_criticalExtensions_114,
    &ett_rrc_T_r7_11,
    &ett_rrc_T_v780NonCriticalExtensions_09,
    &ett_rrc_T_v7d0NonCriticalExtensions_13,
    &ett_rrc_T_v7f0NonCriticalExtensions_14,
    &ett_rrc_T_v7g0NonCriticalExtensions_15,
    &ett_rrc_T_nonCriticalExtensions_150,
    &ett_rrc_T_criticalExtensions_115,
    &ett_rrc_T_r8_11,
    &ett_rrc_T_v7d0NonCriticalExtensions_14,
    &ett_rrc_T_v7f0NonCriticalExtensions_15,
    &ett_rrc_T_v890NonCriticalExtensions_09,
    &ett_rrc_T_v7g0NonCriticalExtensions_16,
    &ett_rrc_T_v8a0NonCriticalExtensions_09,
    &ett_rrc_T_nonCriticalExtensions_151,
    &ett_rrc_T_criticalExtensions_116,
    &ett_rrc_T_r9_10,
    &ett_rrc_T_v950NonCriticalExtensions_08,
    &ett_rrc_T_v9c0NonCriticalExtensions_10,
    &ett_rrc_T_nonCriticalExtensions_152,
    &ett_rrc_T_criticalExtensions_117,
    &ett_rrc_T_r10_10,
    &ett_rrc_T_v9c0NonCriticalExtensions_11,
    &ett_rrc_T_vbxyNonCriticalExtensions_14,
    &ett_rrc_T_nonCriticalExtensions_153,
    &ett_rrc_T_criticalExtensions_118,
    &ett_rrc_T_r11_11,
    &ett_rrc_T_nonCriticalExtensions_154,
    &ett_rrc_T_criticalExtensions_119,
    &ett_rrc_TransportChannelReconfiguration_r3_IEs,
    &ett_rrc_T_dummy_11,
    &ett_rrc_T_fdd_39,
    &ett_rrc_T_modeSpecificPhysChInfo_12,
    &ett_rrc_T_fdd_40,
    &ett_rrc_TransportChannelReconfiguration_v3a0ext,
    &ett_rrc_TransportChannelReconfiguration_v4b0ext_IEs,
    &ett_rrc_TransportChannelReconfiguration_v590ext_IEs,
    &ett_rrc_TransportChannelReconfiguration_r4_IEs,
    &ett_rrc_T_dummy_12,
    &ett_rrc_T_fdd_41,
    &ett_rrc_T_modeSpecificPhysChInfo_13,
    &ett_rrc_T_fdd_42,
    &ett_rrc_TransportChannelReconfiguration_r5_IEs,
    &ett_rrc_T_dummy_13,
    &ett_rrc_T_fdd_43,
    &ett_rrc_T_modeSpecificPhysChInfo_14,
    &ett_rrc_T_fdd_44,
    &ett_rrc_TransportChannelReconfiguration_v690ext_IEs,
    &ett_rrc_TransportChannelReconfiguration_r6_IEs,
    &ett_rrc_TransportChannelReconfiguration_v6b0ext_IEs,
    &ett_rrc_TransportChannelReconfiguration_r7_IEs,
    &ett_rrc_TransportChannelReconfiguration_v770ext_IEs,
    &ett_rrc_TransportChannelReconfiguration_v780ext_IEs,
    &ett_rrc_TransportChannelReconfiguration_v7d0ext_IEs,
    &ett_rrc_TransportChannelReconfiguration_v7f0ext_IEs,
    &ett_rrc_TransportChannelReconfiguration_v7g0ext_IEs,
    &ett_rrc_TransportChannelReconfiguration_r8_IEs,
    &ett_rrc_TransportChannelReconfiguration_v890ext_IEs,
    &ett_rrc_TransportChannelReconfiguration_v8a0ext_IEs,
    &ett_rrc_TransportChannelReconfiguration_r9_IEs,
    &ett_rrc_TransportChannelReconfiguration_v950ext_IEs,
    &ett_rrc_TransportChannelReconfiguration_v9c0ext_IEs,
    &ett_rrc_TransportChannelReconfiguration_r10_IEs,
    &ett_rrc_TransportChannelReconfiguration_vbxyext_IEs,
    &ett_rrc_TransportChannelReconfiguration_r11_IEs,
    &ett_rrc_TransportChannelReconfigurationComplete,
    &ett_rrc_T_laterNonCriticalExtensions_52,
    &ett_rrc_T_v770NonCriticalExtensions_29,
    &ett_rrc_T_nonCriticalExtensions_155,
    &ett_rrc_TransportChannelReconfigurationComplete_v770ext_IEs,
    &ett_rrc_TransportChannelReconfigurationFailure,
    &ett_rrc_T_laterNonCriticalExtensions_53,
    &ett_rrc_T_nonCriticalExtensions_156,
    &ett_rrc_TransportFormatCombinationControl,
    &ett_rrc_T_modeSpecificInfo_15,
    &ett_rrc_T_tdd_09,
    &ett_rrc_T_laterNonCriticalExtensions_54,
    &ett_rrc_T_v820NonCriticalExtensions_02,
    &ett_rrc_T_nonCriticalExtensions_157,
    &ett_rrc_TransportFormatCombinationControl_v820ext_IEs,
    &ett_rrc_TransportFormatCombinationControlFailure,
    &ett_rrc_T_laterNonCriticalExtensions_55,
    &ett_rrc_T_nonCriticalExtensions_158,
    &ett_rrc_UECapabilityEnquiry,
    &ett_rrc_T_r3_25,
    &ett_rrc_T_laterNonCriticalExtensions_56,
    &ett_rrc_T_v4b0NonCriticalExtensions_14,
    &ett_rrc_T_v590NonCriticalExtensions_26,
    &ett_rrc_T_v770NonCriticalExtensions_30,
    &ett_rrc_T_v860NonCriticalExtensions_18,
    &ett_rrc_T_nonCriticalExtensions_159,
    &ett_rrc_T_later_than_r3_22,
    &ett_rrc_T_criticalExtensions_120,
    &ett_rrc_UECapabilityEnquiry_r3_IEs,
    &ett_rrc_UECapabilityEnquiry_v4b0ext_IEs,
    &ett_rrc_UECapabilityEnquiry_v590ext_IEs,
    &ett_rrc_UECapabilityEnquiry_v770ext_IEs,
    &ett_rrc_UECapabilityEnquiry_v860ext_IEs,
    &ett_rrc_UECapabilityInformation,
    &ett_rrc_T_v370NonCriticalExtensions_01,
    &ett_rrc_T_v380NonCriticalExtensions_01,
    &ett_rrc_T_v3a0NonCriticalExtensions_10,
    &ett_rrc_T_laterNonCriticalExtensions_57,
    &ett_rrc_T_v4b0NonCriticalExtensions_15,
    &ett_rrc_T_v590NonCriticalExtensions_27,
    &ett_rrc_T_v5c0NonCriticalExtensions_01,
    &ett_rrc_T_v690NonCriticalExtensions_39,
    &ett_rrc_T_nonCriticalExtensions_160,
    &ett_rrc_UECapabilityInformation_v370ext,
    &ett_rrc_UECapabilityInformation_v380ext_IEs,
    &ett_rrc_UECapabilityInformation_v3a0ext_IEs,
    &ett_rrc_UECapabilityInformation_r3_add_ext_IEs,
    &ett_rrc_T_v680NonCriticalExtensions_01,
    &ett_rrc_T_v7e0NonCriticalExtensions_03,
    &ett_rrc_T_v7f0NonCriticalExtensions_16,
    &ett_rrc_T_va40NonCriticalExtensions_14,
    &ett_rrc_T_nonCriticalExtensions_161,
    &ett_rrc_UECapabilityInformation_v4b0ext,
    &ett_rrc_UECapabilityInformation_v590ext,
    &ett_rrc_UECapabilityInformation_v5c0ext,
    &ett_rrc_UECapabilityInformation_v650ext_IEs,
    &ett_rrc_UECapabilityInformation_v680ext_IEs,
    &ett_rrc_UECapabilityInformation_v690ext_IEs,
    &ett_rrc_UECapabilityInformation_v7e0ext_IEs,
    &ett_rrc_UECapabilityInformation_v7f0ext_IEs,
    &ett_rrc_UECapabilityInformation_va40ext_IEs,
    &ett_rrc_UECapabilityInformationConfirm,
    &ett_rrc_T_r3_26,
    &ett_rrc_T_laterNonCriticalExtensions_58,
    &ett_rrc_T_v770NonCriticalExtensions_31,
    &ett_rrc_T_nonCriticalExtensions_162,
    &ett_rrc_T_later_than_r3_23,
    &ett_rrc_T_criticalExtensions_121,
    &ett_rrc_UECapabilityInformationConfirm_r3_IEs,
    &ett_rrc_UECapabilityInformationConfirm_v770ext_IEs,
    &ett_rrc_UplinkDirectTransfer,
    &ett_rrc_T_laterNonCriticalExtensions_59,
    &ett_rrc_T_v690NonCriticalExtensions_40,
    &ett_rrc_T_v7g0NonCriticalExtensions_17,
    &ett_rrc_T_vbxyNonCriticalExtensions_15,
    &ett_rrc_T_nonCriticalExtensions_163,
    &ett_rrc_UplinkDirectTransfer_v690ext_IEs,
    &ett_rrc_UplinkDirectTransfer_v7g0ext_IEs,
    &ett_rrc_UplinkDirectTransfer_vbxyext_IEs,
    &ett_rrc_UplinkPhysicalChannelControl,
    &ett_rrc_T_r3_27,
    &ett_rrc_T_laterNonCriticalExtensions_60,
    &ett_rrc_T_v4b0NonCriticalExtensions_16,
    &ett_rrc_T_noncriticalExtensions,
    &ett_rrc_T_later_than_r3_24,
    &ett_rrc_T_criticalExtensions_122,
    &ett_rrc_T_r4_13,
    &ett_rrc_T_v4d0NonCriticalExtensions_13,
    &ett_rrc_T_v690NonCriticalExtensions_41,
    &ett_rrc_T_nonCriticalExtensions_164,
    &ett_rrc_T_criticalExtensions_123,
    &ett_rrc_T_r5_11,
    &ett_rrc_T_v690NonCriticalExtensions_42,
    &ett_rrc_T_v6a0NonCriticalExtensions_01,
    &ett_rrc_T_nonCriticalExtensions_165,
    &ett_rrc_T_criticalExtensions_124,
    &ett_rrc_T_r7_12,
    &ett_rrc_T_nonCriticalExtensions_166,
    &ett_rrc_T_criticalExtensions_125,
    &ett_rrc_UplinkPhysicalChannelControl_r3_IEs,
    &ett_rrc_UplinkPhysicalChannelControl_v4b0ext_IEs,
    &ett_rrc_UplinkPhysicalChannelControl_r4_IEs,
    &ett_rrc_T_tddOption,
    &ett_rrc_T_tdd384_08,
    &ett_rrc_T_tdd128_08,
    &ett_rrc_UplinkPhysicalChannelControl_r5_IEs,
    &ett_rrc_T_tddOption_01,
    &ett_rrc_T_tdd384_09,
    &ett_rrc_T_tdd128_09,
    &ett_rrc_UplinkPhysicalChannelControl_v690ext_IEs,
    &ett_rrc_UplinkPhysicalChannelControl_v6a0ext_IEs,
    &ett_rrc_UplinkPhysicalChannelControl_r7_IEs,
    &ett_rrc_T_tddOption_02,
    &ett_rrc_T_tdd384_10,
    &ett_rrc_T_tdd768_05,
    &ett_rrc_T_tdd128_10,
    &ett_rrc_URAUpdate,
    &ett_rrc_T_laterNonCriticalExtensions_61,
    &ett_rrc_T_v770NonCriticalExtensions_32,
    &ett_rrc_T_v860NonCriticalExtensions_19,
    &ett_rrc_T_va40NonCriticalExtensions_15,
    &ett_rrc_T_nonCriticalExtensions_167,
    &ett_rrc_URAUpdate_r3_add_ext_IEs,
    &ett_rrc_T_nonCriticalExtensions_168,
    &ett_rrc_UraUpdate_v770ext_IEs,
    &ett_rrc_URAUpdate_v7e0ext_IEs,
    &ett_rrc_URAUpdate_v860ext_IEs,
    &ett_rrc_URAUpdate_va40ext_IEs,
    &ett_rrc_URAUpdateConfirm,
    &ett_rrc_T_r3_28,
    &ett_rrc_T_laterNonCriticalExtensions_62,
    &ett_rrc_T_v690NonCriticalExtensions_43,
    &ett_rrc_T_nonCriticalExtensions_169,
    &ett_rrc_T_later_than_r3_25,
    &ett_rrc_T_criticalExtensions_126,
    &ett_rrc_T_r5_12,
    &ett_rrc_T_v690NonCriticalExtensions_44,
    &ett_rrc_T_nonCriticalExtensions_170,
    &ett_rrc_T_criticalExtensions_127,
    &ett_rrc_T_r7_13,
    &ett_rrc_T_v860NonCriticalExtensions_20,
    &ett_rrc_T_nonCriticalExtensions_171,
    &ett_rrc_T_criticalExtensions_128,
    &ett_rrc_URAUpdateConfirm_r3_IEs,
    &ett_rrc_URAUpdateConfirm_r5_IEs,
    &ett_rrc_URAUpdateConfirm_v690ext_IEs,
    &ett_rrc_URAUpdateConfirm_r7_IEs,
    &ett_rrc_URAUpdateConfirm_v860ext_IEs,
    &ett_rrc_URAUpdateConfirm_CCCH,
    &ett_rrc_T_r3_29,
    &ett_rrc_T_laterNonCriticalExtensions_63,
    &ett_rrc_T_v690NonCriticalExtensions_45,
    &ett_rrc_T_v860NonCriticalExtensions_21,
    &ett_rrc_T_nonCriticalExtensions_172,
    &ett_rrc_T_later_than_r3_26,
    &ett_rrc_T_criticalExtensions_129,
    &ett_rrc_URAUpdateConfirm_CCCH_r3_IEs,
    &ett_rrc_UTRANMobilityInformation,
    &ett_rrc_T_r3_30,
    &ett_rrc_T_v3a0NonCriticalExtensions_11,
    &ett_rrc_T_laterNonCriticalExtensions_64,
    &ett_rrc_T_v690NonCriticalExtensions_46,
    &ett_rrc_T_v860NonCriticalExtentions,
    &ett_rrc_T_nonCriticalExtensions_173,
    &ett_rrc_T_later_than_r3_27,
    &ett_rrc_T_criticalExtensions_130,
    &ett_rrc_T_r5_13,
    &ett_rrc_T_v690NonCriticalExtensions_47,
    &ett_rrc_T_v860NonCriticalExtentions_01,
    &ett_rrc_T_nonCriticalExtensions_174,
    &ett_rrc_T_criticalExtensions_131,
    &ett_rrc_T_r7_14,
    &ett_rrc_T_v860NonCriticalExtensions_22,
    &ett_rrc_T_nonCriticalExtensions_175,
    &ett_rrc_T_criticalExtensions_132,
    &ett_rrc_UTRANMobilityInformation_r3_IEs,
    &ett_rrc_T_nonCriticalExtensions_176,
    &ett_rrc_UTRANMobilityInformation_v3a0ext_IEs,
    &ett_rrc_UTRANMobilityInformation_r5_IEs,
    &ett_rrc_UtranMobilityInformation_v690ext_IEs,
    &ett_rrc_UTRANMobilityInformation_r7_IEs,
    &ett_rrc_UTRANMobilityInformation_v860ext1_IEs,
    &ett_rrc_UTRANMobilityInformation_v860ext2_IEs,
    &ett_rrc_UTRANMobilityInformationConfirm,
    &ett_rrc_T_laterNonCriticalExtensions_65,
    &ett_rrc_T_v770NonCriticalExtension_01,
    &ett_rrc_T_va40NonCriticalExtensions_16,
    &ett_rrc_T_nonCriticalExtensions_177,
    &ett_rrc_UTRANMobilityInformationConfirm_v770ext_IEs,
    &ett_rrc_UTRANMobilityInformationConfirm_va40ext_IEs,
    &ett_rrc_UTRANMobilityInformationFailure,
    &ett_rrc_T_laterNonCriticalExtensions_66,
    &ett_rrc_T_nonCriticalExtensions_178,
    &ett_rrc_MBMSAccessInformation,
    &ett_rrc_T_nonCriticalExtensions_179,
    &ett_rrc_MBMSCommonPTMRBInformation,
    &ett_rrc_T_v770NonCriticalExtensions_33,
    &ett_rrc_T_v780NonCriticalExtensions_10,
    &ett_rrc_T_v860NonCriticalExtensions_23,
    &ett_rrc_T_nonCriticalExtensions_180,
    &ett_rrc_MBMSCommonPTMRBInformation_v770ext_IEs,
    &ett_rrc_MBMSCommonPTMRBInformation_v780ext_IEs,
    &ett_rrc_MBMSCommonPTMRBInformation_v860ext_IEs,
    &ett_rrc_MBMSCurrentCellPTMRBInformation,
    &ett_rrc_T_v770NonCriticalExtensions_34,
    &ett_rrc_T_nonCriticalExtensions_181,
    &ett_rrc_MBMSCurrentCellPTMRBInfo_v770ext_IEs,
    &ett_rrc_MBMSGeneralInformation,
    &ett_rrc_T_v6b0NonCriticalExtensions_12,
    &ett_rrc_T_v770NonCriticalExtensions_35,
    &ett_rrc_T_v860NonCriticalExtensions_24,
    &ett_rrc_T_v890NoncriticalExtensions,
    &ett_rrc_T_nonCriticalExtensions_182,
    &ett_rrc_MBMSGeneralInformation_v6b0ext_IEs,
    &ett_rrc_MBMSGeneralInformation_v770ext_IEs,
    &ett_rrc_MBMSGeneralInformation_v860ext_IEs,
    &ett_rrc_MBMSGeneralInformation_v890ext_IEs,
    &ett_rrc_MBMSModificationRequest,
    &ett_rrc_T_v6b0NonCriticalExtensions_13,
    &ett_rrc_T_v6f0NonCriticalExtensions_02,
    &ett_rrc_T_nonCriticalExtensions_183,
    &ett_rrc_MBMSModificationRequest_v6b0ext_IEs,
    &ett_rrc_MBMSModificationRequest_v6f0ext_IEs,
    &ett_rrc_MBMSModifiedServicesInformation,
    &ett_rrc_T_v770NonCriticalExtensions_36,
    &ett_rrc_T_v7c0NonCriticalExtensions,
    &ett_rrc_T_nonCriticalExtensions_184,
    &ett_rrc_MBMSModifiedServicesInformation_v770ext_IEs,
    &ett_rrc_MBMSModifiedServicesInformation_v7c0ext_IEs,
    &ett_rrc_MBMSNeighbouringCellPTMRBInformation,
    &ett_rrc_T_v770NonCriticalExtensions_37,
    &ett_rrc_T_nonCriticalExtensions_185,
    &ett_rrc_MBMSNeighbouringCellPTMRBInformation_v770ext_IEs,
    &ett_rrc_MBMSSchedulingInformation,
    &ett_rrc_T_nonCriticalExtensions_186,
    &ett_rrc_MBMSUnmodifiedServicesInformation,
    &ett_rrc_T_v770NonCriticalExtensions_38,
    &ett_rrc_T_nonCriticalExtensions_187,
    &ett_rrc_MBMSUnmodifiedServicesInformation_v770ext_IEs,
    &ett_rrc_System_Information_Container,
    &ett_rrc_T_vbxyNonCriticalExtensions_16,
    &ett_rrc_T_nonCriticalExtensions_188,
    &ett_rrc_System_Information_Container_vbxyext_IEs,
    &ett_rrc_LoggingMeasurementConfiguration,
    &ett_rrc_T_criticalExtensions_133,
    &ett_rrc_T_r10_11,
    &ett_rrc_T_nonCriticalExtensions_189,
    &ett_rrc_T_criticalExtensions_134,
    &ett_rrc_T_r11_12,
    &ett_rrc_T_nonCriticalExtensions_190,
    &ett_rrc_T_criticalExtensions_135,
    &ett_rrc_LoggingMeasurementConfiguration_r10_IEs,
    &ett_rrc_LoggingMeasurementConfiguration_r11_IEs,
    &ett_rrc_UEInformationResponse,
    &ett_rrc_T_vbxyNonCriticalExtensions_17,
    &ett_rrc_T_nonCriticalExtensions_191,
    &ett_rrc_UEInformationResponse_vbxyext_IEs,
    &ett_rrc_UEInformationRequest,
    &ett_rrc_T_criticalExtensions_136,
    &ett_rrc_T_r10_12,
    &ett_rrc_T_nonCriticalExtensions_192,
    &ett_rrc_T_criticalExtensions_137,
    &ett_rrc_T_r11_13,
    &ett_rrc_T_nonCriticalExtensions_193,
    &ett_rrc_T_criticalExtensions_138,
    &ett_rrc_UEInformationRequest_r10_IEs,
    &ett_rrc_UEInformationRequest_r11_IEs,
    &ett_rrc_CN_DomainInformation,
    &ett_rrc_CN_DomainInformationFull,
    &ett_rrc_CN_DomainInformationList,
    &ett_rrc_CN_DomainInformationListFull,
    &ett_rrc_CN_DomainSysInfo,
    &ett_rrc_T_cn_Type,
    &ett_rrc_CN_DomainSysInfoList,
    &ett_rrc_CN_InformationInfo,
    &ett_rrc_CN_InformationInfo_r6,
    &ett_rrc_CN_InformationInfoFull,
    &ett_rrc_DomainSpecificEAB_Parameters,
    &ett_rrc_EAB_AccessClassBarredList,
    &ett_rrc_EAB_Configuration,
    &ett_rrc_EAB_Parameters,
    &ett_rrc_EAB_ParametersPerPLMN_List,
    &ett_rrc_EAB_ParametersPerPLMN_List_item,
    &ett_rrc_Ec_N0forANR,
    &ett_rrc_Gsm_map_IDNNS,
    &ett_rrc_T_routingbasis,
    &ett_rrc_T_localPTMSI,
    &ett_rrc_T_tMSIofsamePLMN,
    &ett_rrc_T_tMSIofdifferentPLMN,
    &ett_rrc_T_iMSIresponsetopaging,
    &ett_rrc_T_iMSIcauseUEinitiatedEvent,
    &ett_rrc_T_iMEI,
    &ett_rrc_T_spare2,
    &ett_rrc_T_spare1,
    &ett_rrc_IMEI,
    &ett_rrc_IMSI_GSM_MAP,
    &ett_rrc_IntraDomainNasNodeSelector,
    &ett_rrc_T_version,
    &ett_rrc_T_release99,
    &ett_rrc_T_cn_Type_01,
    &ett_rrc_T_later,
    &ett_rrc_LAI,
    &ett_rrc_LocationRegistrationAccessClassBarredList,
    &ett_rrc_LocationRegistrationParameters,
    &ett_rrc_MCC,
    &ett_rrc_MNC,
    &ett_rrc_MultiplePLMN_List_r6,
    &ett_rrc_SEQUENCE_SIZE_1_5_OF_PLMN_IdentityWithOptionalMCC_r6,
    &ett_rrc_PagingPermissionWithAccessControlForSharedNetwork,
    &ett_rrc_PagingPermissionWithAccessControlList,
    &ett_rrc_PagingPermissionWithAccessControlParameters,
    &ett_rrc_PLMN_Identity,
    &ett_rrc_PLMN_IdentityWithOptionalMCC_r6,
    &ett_rrc_PLMN_Type,
    &ett_rrc_T_gsm_MAP,
    &ett_rrc_T_ansi_41,
    &ett_rrc_T_gsm_MAP_and_ANSI_41,
    &ett_rrc_RAB_Identity,
    &ett_rrc_RAI,
    &ett_rrc_AccessClassBarredList,
    &ett_rrc_CellAccessRestriction,
    &ett_rrc_CellAccessRestriction_v870ext,
    &ett_rrc_CellBarred,
    &ett_rrc_T_barred,
    &ett_rrc_CellIdentity_PerRL_List,
    &ett_rrc_CellIDListItem,
    &ett_rrc_CellSelectReselectInfoSIB_3_4,
    &ett_rrc_T_cellSelectQualityMeasure,
    &ett_rrc_T_cpich_Ec_N0,
    &ett_rrc_T_modeSpecificInfo_16,
    &ett_rrc_T_fdd_45,
    &ett_rrc_T_tdd_10,
    &ett_rrc_CSG_DedicatedFrequencyInfoList,
    &ett_rrc_CSG_PSCSplitInfo,
    &ett_rrc_DomainSpecificAccessRestrictionForSharedNetwork_v670ext,
    &ett_rrc_DomainSpecificAccessRestrictionList_v670ext,
    &ett_rrc_DomainSpecificAccessRestrictionParam_v670ext,
    &ett_rrc_DomainSpecificAccessRestriction_v670ext,
    &ett_rrc_T_restriction,
    &ett_rrc_Mapping,
    &ett_rrc_Mapping_LCR_r4,
    &ett_rrc_MappingFunctionParameter,
    &ett_rrc_MappingFunctionParameterList,
    &ett_rrc_MappingInfo,
    &ett_rrc_DedicatedPriorityInformation,
    &ett_rrc_T_action,
    &ett_rrc_T_configureDedicatedPriorities,
    &ett_rrc_PriorityLevel,
    &ett_rrc_T_radioAccessTechnology,
    &ett_rrc_T_utraFDD,
    &ett_rrc_T_utraFDD_item,
    &ett_rrc_T_utraTDD,
    &ett_rrc_T_utraTDD_item,
    &ett_rrc_T_eutra,
    &ett_rrc_T_eutra_item,
    &ett_rrc_T_gsm_01,
    &ett_rrc_PriorityLevelList,
    &ett_rrc_RAT_FDD_Info,
    &ett_rrc_RAT_FDD_InfoList,
    &ett_rrc_RAT_TDD_Info,
    &ett_rrc_RAT_TDD_InfoList,
    &ett_rrc_URA_IdentityList,
    &ett_rrc_BackoffControlParams,
    &ett_rrc_CapabilityUpdateRequirement,
    &ett_rrc_CapabilityUpdateRequirement_r4_ext,
    &ett_rrc_CapabilityUpdateRequirement_r4,
    &ett_rrc_CapabilityUpdateRequirement_r5,
    &ett_rrc_CapabilityUpdateRequirement_v770ext,
    &ett_rrc_CapabilityUpdateRequirement_r7,
    &ett_rrc_CapabilityUpdateRequirement_v860ext,
    &ett_rrc_CapabilityUpdateRequirement_r8,
    &ett_rrc_CipheringModeCommand,
    &ett_rrc_CipheringModeCommand_r7,
    &ett_rrc_CipheringModeInfo,
    &ett_rrc_CipheringModeInfo_r7,
    &ett_rrc_CN_PagedUE_Identity,
    &ett_rrc_CompressedModeMeasCapability,
    &ett_rrc_CompressedModeMeasCapability_LCR_r4,
    &ett_rrc_CompressedModeMeasCapability_v860ext,
    &ett_rrc_CompressedModeMeasCapability_v920ext,
    &ett_rrc_CompressedModeMeasCapability_va40ext,
    &ett_rrc_CompressedModeMeasCapabEUTRA,
    &ett_rrc_CompressedModeMeasCapabEUTRAList,
    &ett_rrc_CompressedModeMeasCapabFDDList,
    &ett_rrc_CompressedModeMeasCapabFDDList2,
    &ett_rrc_CompressedModeMeasCapabFDDList3,
    &ett_rrc_CompressedModeMeasCapabFDDList_ext,
    &ett_rrc_CompressedModeMeasCapabFDD,
    &ett_rrc_CompressedModeMeasCapabFDD2,
    &ett_rrc_CompressedModeMeasCapabFDD3,
    &ett_rrc_CompressedModeMeasCapabFDD_ext,
    &ett_rrc_CompressedModeMeasCapabTDDList,
    &ett_rrc_CompressedModeMeasCapabTDD,
    &ett_rrc_CompressedModeMeasCapabGSMList,
    &ett_rrc_CompressedModeMeasCapabGSM,
    &ett_rrc_CompressedModeMeasCapabMC,
    &ett_rrc_CPCH_Parameters,
    &ett_rrc_CSG_ProximityIndicationCapability,
    &ett_rrc_DL_InformationPerSecondaryRL_List,
    &ett_rrc_DL_InformationPerSecondaryRL,
    &ett_rrc_T_e_RGCH_Info,
    &ett_rrc_DL_PhysChCapabilityFDD,
    &ett_rrc_DL_PhysChCapabilityFDD_v380ext,
    &ett_rrc_DL_PhysChCapabilityFDD_v770ext,
    &ett_rrc_DL_PhysChCapabilityFDD_v860ext,
    &ett_rrc_DL_PhysChCapabilityFDD_v920ext,
    &ett_rrc_DL_PhysChCapabilityFDD_va40ext,
    &ett_rrc_DL_PhysChCapabilityFDD_vbxyext,
    &ett_rrc_DL_PhysChCapabilityTDD,
    &ett_rrc_DL_PhysChCapabilityTDD_LCR_r4,
    &ett_rrc_DL_PhysChCapabilityTDD_128_v770ext,
    &ett_rrc_DL_PhysChCapabilityTDD_128_v860ext,
    &ett_rrc_DL_PhysChCapabilityInfoTDD_128_va40ext,
    &ett_rrc_DL_PhysChCapabilityTDD_768,
    &ett_rrc_T_tdd768_hspdsch,
    &ett_rrc_DL_PhysChCapabilityInfoTDD_768,
    &ett_rrc_DL_TransChCapability,
    &ett_rrc_DRAC_SysInfo,
    &ett_rrc_DRAC_SysInfoList,
    &ett_rrc_FailureCauseWithProtErr,
    &ett_rrc_FailureCauseWithProtErrTrId,
    &ett_rrc_FrequencyBandsIndicatorSupport,
    &ett_rrc_T_modeSpecificInfo_17,
    &ett_rrc_T_fdd_46,
    &ett_rrc_T_tdd_11,
    &ett_rrc_FrequencyBandsIndicatorTDD128,
    &ett_rrc_FrequencyBandsIndicatorSupportTDD128,
    &ett_rrc_FrequencyBandsRedirectionFDD,
    &ett_rrc_FrequencyBandsIndicatorFDD,
    &ett_rrc_GroupIdentityWithReleaseInformation,
    &ett_rrc_GroupReleaseInformation,
    &ett_rrc_GSM_Measurements,
    &ett_rrc_MultiflowBandCombination,
    &ett_rrc_MultiflowBandCombList,
    &ett_rrc_MultiflowCapability,
    &ett_rrc_MultiflowCapabilityBand,
    &ett_rrc_MultiflowNonContiguousCapability,
    &ett_rrc_IdleIntervalMeasCapabEUTRA,
    &ett_rrc_IdleIntervalMeasCapabEUTRAList,
    &ett_rrc_IMSI_and_ESN_DS_41,
    &ett_rrc_InitialPriorityDelayList,
    &ett_rrc_InitialUE_Identity,
    &ett_rrc_IntegrityCheckInfo,
    &ett_rrc_IntegrityProtActivationInfo,
    &ett_rrc_IntegrityProtectionModeCommand,
    &ett_rrc_T_startIntegrityProtection,
    &ett_rrc_T_modify,
    &ett_rrc_IntegrityProtectionModeInfo,
    &ett_rrc_IntegrityProtectionModeInfo_r7,
    &ett_rrc_MeasurementCapability,
    &ett_rrc_MeasurementCapabilityExt,
    &ett_rrc_MeasurementCapabilityExt2,
    &ett_rrc_MeasurementCapabilityExt3,
    &ett_rrc_MeasurementCapabilityExt4,
    &ett_rrc_MeasurementCapability_r4_ext,
    &ett_rrc_MeasurementCapability_v860ext,
    &ett_rrc_MeasurementCapability_v920ext,
    &ett_rrc_MeasurementCapability_va40ext,
    &ett_rrc_MeasurementCapability_vbxyext,
    &ett_rrc_MeasurementCapabilityTDD,
    &ett_rrc_NonContiguousMultiCellCombinationList,
    &ett_rrc_NonContiguousMultiCell,
    &ett_rrc_MultiRAT_Capability,
    &ett_rrc_MultiModeRAT_Capability_v590ext,
    &ett_rrc_MultiModeRAT_Capability_v680ext,
    &ett_rrc_MultiModeRAT_Capability_v770ext,
    &ett_rrc_MultiModeRAT_Capability_v860ext,
    &ett_rrc_MultiModeRAT_Capability_vbxyext,
    &ett_rrc_NeighCellSI_AcquisitionCapability,
    &ett_rrc_NetworkAssistedGANSS_Supported_List,
    &ett_rrc_NetworkAssistedGANSS_Supported_List_item,
    &ett_rrc_NetworkAssistedGANSS_Supported_List_v860ext,
    &ett_rrc_NetworkAssistedGANSS_Supported_List_v860ext_item,
    &ett_rrc_P_TMSI_and_RAI_GSM_MAP,
    &ett_rrc_PagingRecord,
    &ett_rrc_T_cn_Identity,
    &ett_rrc_T_utran_Identity,
    &ett_rrc_T_cn_OriginatedPage_connectedMode_UE,
    &ett_rrc_PagingRecord2_r5,
    &ett_rrc_T_utran_SingleUE_Identity,
    &ett_rrc_T_cn_OriginatedPage_connectedMode_UE_01,
    &ett_rrc_SEQUENCE_SIZE_1_maxURNTI_Group_OF_GroupIdentityWithReleaseInformation,
    &ett_rrc_PagingRecordList,
    &ett_rrc_PagingRecord2List_r5,
    &ett_rrc_PDCP_Capability,
    &ett_rrc_T_supportForRfc2507,
    &ett_rrc_PDCP_Capability_r4_ext,
    &ett_rrc_T_supportForRfc3095,
    &ett_rrc_T_supported,
    &ett_rrc_PDCP_Capability_r5_ext,
    &ett_rrc_PDCP_Capability_r5_ext2,
    &ett_rrc_PDCP_Capability_v770ext,
    &ett_rrc_PhysicalChannelCapability,
    &ett_rrc_T_fddPhysChCapability,
    &ett_rrc_T_tddPhysChCapability,
    &ett_rrc_PhysicalChannelCapability_v770ext,
    &ett_rrc_T_fddPhysChCapability_01,
    &ett_rrc_T_tddPhysChCapability_384,
    &ett_rrc_T_tddPhysChCapability_768,
    &ett_rrc_T_tddPhysChCapability_128,
    &ett_rrc_PhysicalChannelCapability_v860ext,
    &ett_rrc_T_fddPhysChCapability_02,
    &ett_rrc_T_tddPhysChCapability_128_01,
    &ett_rrc_PhysicalChannelCapability_v920ext,
    &ett_rrc_T_fddPhysChCapability_03,
    &ett_rrc_PhysicalChannelCapability_va40ext,
    &ett_rrc_T_fddPhysChCapability_04,
    &ett_rrc_T_tddPhysChCapability_128_02,
    &ett_rrc_PhysicalChannelCapability_vbxyext,
    &ett_rrc_T_fddPhysChCapability_05,
    &ett_rrc_PhysicalChannelCapabilityInfo_v770ext,
    &ett_rrc_T_tddPhysChCapability_768_01,
    &ett_rrc_T_tddPhysChCapability_128_03,
    &ett_rrc_PhysicalChannelCapability_LCR_r4,
    &ett_rrc_T_tdd128_PhysChCapability,
    &ett_rrc_PhysicalChannelCapability_hspdsch_r5,
    &ett_rrc_T_fdd_hspdsch,
    &ett_rrc_T_supported_01,
    &ett_rrc_T_tdd384_hspdsch,
    &ett_rrc_T_tdd128_hspdsch,
    &ett_rrc_PNBSCH_Allocation_r4,
    &ett_rrc_Pre_RedirectionInfo,
    &ett_rrc_ProtocolErrorIndicatorWithMoreInfo,
    &ett_rrc_T_errorOccurred,
    &ett_rrc_ProtocolErrorMoreInformation,
    &ett_rrc_T_diagnosticsType,
    &ett_rrc_T_type1,
    &ett_rrc_RadioFrequencyBandTDDextList,
    &ett_rrc_Rb_timer_indicator,
    &ett_rrc_RedirectionInfo,
    &ett_rrc_RedirectionInfo_r6,
    &ett_rrc_RedirectionInfo_v860ext,
    &ett_rrc_RF_Capability,
    &ett_rrc_T_fddRF_Capability,
    &ett_rrc_T_tddRF_Capability,
    &ett_rrc_RF_Capability_r4_ext,
    &ett_rrc_T_tddRF_Capability_01,
    &ett_rrc_RF_Capability_v770ext,
    &ett_rrc_T_tdd768RF_Capability,
    &ett_rrc_RF_Capability_v860ext,
    &ett_rrc_RF_Capability_vaa0ext,
    &ett_rrc_RLC_Capability,
    &ett_rrc_RLC_Capability_r5_ext,
    &ett_rrc_RLC_Capability_v770ext,
    &ett_rrc_RLC_Capability_v920ext,
    &ett_rrc_RLC_Capability_va40ext,
    &ett_rrc_RLC_Capability_vbxyext,
    &ett_rrc_RRC_ConnectionReleaseInformation,
    &ett_rrc_T_release,
    &ett_rrc_RRC_MessageSequenceNumberList,
    &ett_rrc_RSR_VCC_Info,
    &ett_rrc_SecurityCapability,
    &ett_rrc_T_cipheringAlgorithmCap,
    &ett_rrc_T_integrityProtectionAlgorithmCap,
    &ett_rrc_SecondaryServingEDCHCell_Info,
    &ett_rrc_SecondaryEDCH_Info_Common,
    &ett_rrc_T_servingGrant,
    &ett_rrc_Serving_HSDSCH_CellInformation,
    &ett_rrc_Serving_HSDSCH_CellInformation_r7,
    &ett_rrc_Serving_HSDSCH_CellInformation_r8,
    &ett_rrc_Serving_HSDSCH_CellInformation_r9,
    &ett_rrc_Serving_HSDSCH_CellInformation_r11,
    &ett_rrc_SimultaneousSCCPCH_DPCH_Reception,
    &ett_rrc_T_supported_02,
    &ett_rrc_SR_VCC_Info,
    &ett_rrc_SR_VCC_SecurityRABInfo_v860ext,
    &ett_rrc_STARTList,
    &ett_rrc_STARTSingle,
    &ett_rrc_SystemSpecificCapUpdateReqList,
    &ett_rrc_SystemSpecificCapUpdateReqList_r5,
    &ett_rrc_SystemSpecificCapUpdateReqList_r8,
    &ett_rrc_TMSI_and_LAI_GSM_MAP,
    &ett_rrc_TransportChannelCapability,
    &ett_rrc_TurboSupport,
    &ett_rrc_U_RNTI,
    &ett_rrc_U_RNTI_Group,
    &ett_rrc_U_RNTI_Short,
    &ett_rrc_UE_BasedNetworkPerformanceMeasurementsParameters,
    &ett_rrc_UE_CapabilityContainer_IEs,
    &ett_rrc_T_v6b0NonCriticalExtensions_14,
    &ett_rrc_T_v6e0NonCriticalExtensions_02,
    &ett_rrc_T_v770NonCriticalExtensions_39,
    &ett_rrc_T_v790NonCriticalExtensions_01,
    &ett_rrc_T_v860NonCriticalExtensions_25,
    &ett_rrc_T_v880NonCriticalExtensions_01,
    &ett_rrc_T_v890NonCriticalExtensions_10,
    &ett_rrc_T_v920NonCriticalExtensions_03,
    &ett_rrc_T_v970NonCriticalExtensions_03,
    &ett_rrc_T_va40NonCriticalExtensions_17,
    &ett_rrc_T_va60NonCriticalExtensions,
    &ett_rrc_T_va80NonCriticalExtensions_01,
    &ett_rrc_UE_RadioAccessCapability_v6b0ext_IEs,
    &ett_rrc_UE_RadioAccessCapability_v6e0ext_IEs,
    &ett_rrc_UE_RadioAccessCapability_v770ext_IEs,
    &ett_rrc_UE_RadioAccessCapability_v790ext_IEs,
    &ett_rrc_UE_RadioAccessCapability_v860ext_IEs,
    &ett_rrc_UE_RadioAccessCapability_v880ext_IEs,
    &ett_rrc_UE_RadioAccessCapability_v890ext_IEs,
    &ett_rrc_UE_RadioAccessCapability_v920ext_IEs,
    &ett_rrc_UE_RadioAccessCapability_v970ext_IEs,
    &ett_rrc_UE_RadioAccessCapability_v9c0ext_IEs,
    &ett_rrc_UE_RadioAccessCapability_LaterNonCriticalExtensions,
    &ett_rrc_T_v9c0NonCriticalExtensions_12,
    &ett_rrc_T_vaa0NonCriticalExtensions_02,
    &ett_rrc_T_vbxyNonCriticalExtensions_18,
    &ett_rrc_T_nonCriticalExtensions_194,
    &ett_rrc_UE_RadioAccessCapability_va40ext_IEs,
    &ett_rrc_UE_RadioAccessCapability_va60ext_IEs,
    &ett_rrc_UE_RadioAccessCapability_va80ext_IEs,
    &ett_rrc_UE_RadioAccessCapability_vaa0ext_IEs,
    &ett_rrc_UE_RadioAccessCapability_vbxyext_IEs,
    &ett_rrc_UE_ConnTimersAndConstants,
    &ett_rrc_UE_ConnTimersAndConstants_v3a0ext,
    &ett_rrc_UE_ConnTimersAndConstants_r5,
    &ett_rrc_UE_ConnTimersAndConstants_v860ext,
    &ett_rrc_UE_GANSSPositioning_Capability,
    &ett_rrc_UE_GANSSPositioning_Capability_v860ext_IEs,
    &ett_rrc_UE_IdleTimersAndConstants,
    &ett_rrc_UE_HSPA_Identities_r6,
    &ett_rrc_UE_IdleTimersAndConstants_v3a0ext,
    &ett_rrc_UE_MultiModeRAT_Capability,
    &ett_rrc_UE_RadioAccessCapability,
    &ett_rrc_UE_RadioAccessCapabilityInfo,
    &ett_rrc_UE_RadioAccessCapability_v370ext,
    &ett_rrc_UE_RadioAccessCapability_v380ext,
    &ett_rrc_UE_RadioAccessCapability_v3a0ext,
    &ett_rrc_UE_RadioAccessCapability_v3g0ext,
    &ett_rrc_UE_RadioAccessCapability_v650ext,
    &ett_rrc_UE_RadioAccessCapability_v690ext,
    &ett_rrc_UE_RadioAccessCapability_v7e0ext,
    &ett_rrc_UE_RadioAccessCapability_v7f0ext,
    &ett_rrc_UE_RadioAccessCapabilityInfo_v770ext,
    &ett_rrc_UE_RadioAccessCapabilityInfo_TDD128_v8b0ext,
    &ett_rrc_UE_RadioAccessCapability_v860ext,
    &ett_rrc_UE_RadioAccessCapability_v880ext,
    &ett_rrc_UE_RadioAccessCapability_v920ext,
    &ett_rrc_UE_RadioAccessCapability_va40ext,
    &ett_rrc_UE_RadioAccessCapability_va80ext,
    &ett_rrc_UE_RadioAccessCapability_vbxyext,
    &ett_rrc_UE_RadioAccessCapabBand_va40ext,
    &ett_rrc_UE_RadioAccessCapabBandFDDList2,
    &ett_rrc_UE_RadioAccessCapabBandFDDList3,
    &ett_rrc_UE_RadioAccessCapabBandFDDList4,
    &ett_rrc_UE_RadioAccessCapabBandFDDList4_va60ext,
    &ett_rrc_UE_RadioAccessCapabBandFDDList4_va80ext,
    &ett_rrc_UE_RadioAccessCapabBandFDDList5,
    &ett_rrc_UE_RadioAccessCapabBandFDDList6,
    &ett_rrc_UE_RadioAccessCapabBandFDD2,
    &ett_rrc_T_fddRF_Capability_01,
    &ett_rrc_UE_RadioAccessCapabBandFDD3,
    &ett_rrc_UE_RadioAccessCapabBandFDD4,
    &ett_rrc_UE_RadioAccessCapabBandFDD4_va60ext,
    &ett_rrc_UE_RadioAccessCapabBandFDD4_va80ext,
    &ett_rrc_UE_RadioAccessCapabBandFDD5,
    &ett_rrc_T_fddRF_Capability_02,
    &ett_rrc_UE_RadioAccessCapabBandFDD6,
    &ett_rrc_UE_PositioningCapabilityExt_v380,
    &ett_rrc_UE_PositioningCapabilityExt_v3a0,
    &ett_rrc_UE_PositioningCapabilityExt_v3g0,
    &ett_rrc_UE_PositioningCapability_v770ext,
    &ett_rrc_UE_PositioningCapability_v860ext,
    &ett_rrc_UE_RadioAccessCapabBandCombList,
    &ett_rrc_UE_RadioAccessCapabBandFDDList,
    &ett_rrc_UE_RadioAccessCapabBandFDDList_ext,
    &ett_rrc_UE_RadioAccessCapabBandFDDList_ext2,
    &ett_rrc_UE_RadioAccessCapabBandFDD,
    &ett_rrc_T_fddRF_Capability_03,
    &ett_rrc_UE_RadioAccessCapabBandFDD_ext,
    &ett_rrc_UE_RadioAccessCapabBandFDD_ext2,
    &ett_rrc_UE_RadioAccessCapability_v4b0ext,
    &ett_rrc_T_tdd_CapabilityExt,
    &ett_rrc_UE_RadioAccessCapabilityComp,
    &ett_rrc_UE_RadioAccessCapabilityComp_ext,
    &ett_rrc_UE_RadioAccessCapabilityComp_TDD128_v7f0ext,
    &ett_rrc_UE_RadioAccessCapabilityComp_v770ext,
    &ett_rrc_UE_RadioAccessCapabilityComp2,
    &ett_rrc_T_fddPhysicalChannelCapab_hspdsch_edch,
    &ett_rrc_UE_RadioAccessCapabilityComp2_v770ext,
    &ett_rrc_UE_RadioAccessCapabilityComp2_v7f0ext,
    &ett_rrc_UE_RadioAccessCapabilityComp2_v860ext,
    &ett_rrc_UE_RadioAccessCapabilityComp2_v920ext,
    &ett_rrc_UE_RadioAccessCapabilityComp2_va40ext,
    &ett_rrc_UE_RadioAccessCapabilityComp2_vbxyext,
    &ett_rrc_UE_RadioAccessCapabilityComp_TDD128,
    &ett_rrc_RF_CapabilityComp,
    &ett_rrc_T_fdd_47,
    &ett_rrc_T_tdd384_RF_Capability,
    &ett_rrc_T_tdd128_RF_Capability,
    &ett_rrc_RF_CapabilityComp_v770ext,
    &ett_rrc_T_tdd768_RF_Capability,
    &ett_rrc_RF_CapabBandListFDDComp,
    &ett_rrc_RF_CapabBandListFDDComp_ext,
    &ett_rrc_RF_CapabBandListFDDComp_ext2,
    &ett_rrc_UE_RadioAccessCapabBandCombList_va40ext,
    &ett_rrc_UE_RadioAccessCapabBandCombList_vbxyext,
    &ett_rrc_SupportedCarrierCombination,
    &ett_rrc_SupportedCarrierCombination_ext,
    &ett_rrc_UE_RadioAccessCapability_v590ext,
    &ett_rrc_UE_RadioAccessCapability_v5c0ext,
    &ett_rrc_UE_RadioAccessCapability_v680ext,
    &ett_rrc_UL_OLTD_InfoFDD,
    &ett_rrc_UL_PhysChCapabilityFDD,
    &ett_rrc_UL_PhysChCapabilityFDD_v770ext,
    &ett_rrc_UL_PhysChCapabilityFDD_v920ext,
    &ett_rrc_UL_PhysChCapabilityTDD,
    &ett_rrc_UL_PhysChCapabilityTDD_384_v770ext,
    &ett_rrc_T_tdd384_edch,
    &ett_rrc_T_supported_03,
    &ett_rrc_UL_PhysChCapabilityTDD_LCR_r4,
    &ett_rrc_UL_PhysChCapabilityTDD_128_v770ext,
    &ett_rrc_T_tdd128_edch,
    &ett_rrc_T_supported_04,
    &ett_rrc_UL_PhysChCapabilityInfoTDD_128_v770ext,
    &ett_rrc_UL_PhysChCapabilityInfoTDD_128_va40ext,
    &ett_rrc_UL_PhysChCapabilityTDD_768,
    &ett_rrc_T_tdd384_edch_01,
    &ett_rrc_T_supported_05,
    &ett_rrc_UL_SecondaryCellInfoFDD,
    &ett_rrc_T_newConfiguration,
    &ett_rrc_PhysicalChannelCapability_edch_r6,
    &ett_rrc_T_fdd_edch,
    &ett_rrc_T_supported_06,
    &ett_rrc_UL_TransChCapability,
    &ett_rrc_T_modeSpecificInfo_18,
    &ett_rrc_T_tdd_12,
    &ett_rrc_UE_Positioning_Capability,
    &ett_rrc_UE_SecurityInformation,
    &ett_rrc_UE_SecurityInformation2,
    &ett_rrc_UTRAN_DRX_CycleLengthCoefficient_r7,
    &ett_rrc_AlgorithmSpecificInfo,
    &ett_rrc_AlgorithmSpecificInfo_r4,
    &ett_rrc_CommonRBMappingInfo,
    &ett_rrc_CS_HSPA_Information,
    &ett_rrc_DefaultConfigForCellFACH,
    &ett_rrc_DL_AM_RLC_Mode,
    &ett_rrc_DL_AM_RLC_Mode_r5,
    &ett_rrc_DL_AM_RLC_Mode_r7,
    &ett_rrc_T_dl_RLC_PDU_size,
    &ett_rrc_DL_AM_RLC_Mode_r11,
    &ett_rrc_T_dl_RLC_PDU_size_01,
    &ett_rrc_DL_CounterSynchronisationInfo,
    &ett_rrc_DL_CounterSynchronisationInfo_r5,
    &ett_rrc_DL_LogicalChannelMapping,
    &ett_rrc_DL_LogicalChannelMapping_r5,
    &ett_rrc_DL_LogicalChannelMapping_r7,
    &ett_rrc_DL_LogicalChannelMappingList,
    &ett_rrc_DL_LogicalChannelMappingList_r5,
    &ett_rrc_DL_LogicalChannelMappingList_r7,
    &ett_rrc_DL_RFC3095_r4,
    &ett_rrc_DL_RLC_Mode,
    &ett_rrc_DL_RLC_Mode_r5,
    &ett_rrc_DL_RLC_Mode_r6,
    &ett_rrc_DL_RLC_Mode_r7,
    &ett_rrc_DL_RLC_Mode_r11,
    &ett_rrc_DL_RLC_StatusInfo,
    &ett_rrc_DL_RLC_StatusInfo_r11,
    &ett_rrc_DL_TM_RLC_Mode,
    &ett_rrc_DL_TransportChannelType,
    &ett_rrc_DL_TransportChannelType_r5,
    &ett_rrc_DL_TransportChannelType_r7,
    &ett_rrc_T_hsdsch,
    &ett_rrc_T_dch_and_hsdsch,
    &ett_rrc_DL_UM_RLC_Mode_r5,
    &ett_rrc_DL_UM_RLC_Mode_r6,
    &ett_rrc_ExplicitDiscard,
    &ett_rrc_HeaderCompressionInfo,
    &ett_rrc_HeaderCompressionInfoList,
    &ett_rrc_HeaderCompressionInfo_r4,
    &ett_rrc_HeaderCompressionInfoList_r4,
    &ett_rrc_LosslessSRNS_RelocSupport,
    &ett_rrc_MaxDAT_Retransmissions,
    &ett_rrc_PDCP_Info,
    &ett_rrc_PDCP_Info_r4,
    &ett_rrc_PDCP_InfoReconfig,
    &ett_rrc_PDCP_InfoReconfig_r4,
    &ett_rrc_PollingInfo,
    &ett_rrc_PredefinedRB_Configuration,
    &ett_rrc_PreDefRadioConfiguration,
    &ett_rrc_PreDefRadioConfiguration_v770ext,
    &ett_rrc_PreDefRadioConfiguration_v920ext,
    &ett_rrc_PredefinedConfigStatusList,
    &ett_rrc_PredefinedConfigStatusInfo,
    &ett_rrc_T_other,
    &ett_rrc_PredefinedConfigStatusListComp,
    &ett_rrc_PredefinedConfigSetsWithDifferentValueTag,
    &ett_rrc_PredefinedConfigSetWithDifferentValueTag,
    &ett_rrc_PredefinedConfigValueTagList,
    &ett_rrc_PredefinedConfigStatusListVarSz,
    &ett_rrc_RAB_Info,
    &ett_rrc_RAB_Info_r6_ext,
    &ett_rrc_RAB_Info_v6b0ext,
    &ett_rrc_RAB_Info_r6,
    &ett_rrc_RAB_Info_r7,
    &ett_rrc_RAB_InfoReplace,
    &ett_rrc_RAB_InformationList,
    &ett_rrc_RAB_InformationList_r6,
    &ett_rrc_RAB_InformationReconfigList,
    &ett_rrc_RAB_InformationReconfigList_r8,
    &ett_rrc_RAB_InformationReconfig,
    &ett_rrc_RAB_InformationReconfig_r8,
    &ett_rrc_RAB_Info_Post,
    &ett_rrc_RAB_InformationMBMSPtp,
    &ett_rrc_RAB_InformationMBMSPtpList,
    &ett_rrc_RAB_InformationSetup,
    &ett_rrc_RAB_InformationSetup_r4,
    &ett_rrc_RAB_InformationSetup_r5,
    &ett_rrc_RAB_InformationSetup_r6_ext,
    &ett_rrc_RAB_InformationSetup_r6,
    &ett_rrc_RAB_InformationSetup_v6b0ext,
    &ett_rrc_RAB_InformationSetup_r7,
    &ett_rrc_RAB_InformationSetup_r8,
    &ett_rrc_RAB_InformationSetup_v820ext,
    &ett_rrc_RAB_InformationSetup_r11,
    &ett_rrc_RAB_InformationSetupList,
    &ett_rrc_RAB_InformationSetupList_r4,
    &ett_rrc_RAB_InformationSetupList_r5,
    &ett_rrc_RAB_InformationSetupList_r6,
    &ett_rrc_RAB_InformationSetupList_r6_ext,
    &ett_rrc_RAB_InformationSetupList_v6b0ext,
    &ett_rrc_RAB_InformationSetupList_r7,
    &ett_rrc_RAB_InformationSetupList_r8,
    &ett_rrc_RAB_InformationSetupList_v820ext,
    &ett_rrc_RAB_InformationSetupList_r11,
    &ett_rrc_RB_ActivationTimeInfo,
    &ett_rrc_RB_ActivationTimeInfoList,
    &ett_rrc_RB_COUNT_C_Information,
    &ett_rrc_RB_COUNT_C_InformationList,
    &ett_rrc_RB_COUNT_C_MSB_Information,
    &ett_rrc_RB_COUNT_C_MSB_InformationList,
    &ett_rrc_RB_IdentityList,
    &ett_rrc_RB_InformationAffected,
    &ett_rrc_RB_InformationAffected_r5,
    &ett_rrc_RB_InformationAffected_r6,
    &ett_rrc_RB_InformationAffected_r7,
    &ett_rrc_RB_InformationAffected_r8,
    &ett_rrc_RB_InformationAffectedList,
    &ett_rrc_RB_InformationAffectedList_r5,
    &ett_rrc_RB_InformationAffectedList_r6,
    &ett_rrc_RB_InformationAffectedList_r7,
    &ett_rrc_RB_InformationAffectedList_r8,
    &ett_rrc_RB_InformationChanged_r6,
    &ett_rrc_T_rb_Change,
    &ett_rrc_RB_InformationChangedList_r6,
    &ett_rrc_RB_InformationReconfig,
    &ett_rrc_RB_InformationReconfig_r4,
    &ett_rrc_RB_InformationReconfig_r5,
    &ett_rrc_RB_InformationReconfig_r6,
    &ett_rrc_RB_InformationReconfig_r7,
    &ett_rrc_RB_InformationReconfig_r8,
    &ett_rrc_RB_InformationReconfig_r11,
    &ett_rrc_RB_InformationReconfigList,
    &ett_rrc_RB_InformationReconfigList_r4,
    &ett_rrc_RB_InformationReconfigList_r5,
    &ett_rrc_RB_InformationReconfigList_r6,
    &ett_rrc_RB_InformationReconfigList_r7,
    &ett_rrc_RB_InformationReconfigList_r8,
    &ett_rrc_RB_InformationReconfigList_r11,
    &ett_rrc_RB_InformationReleaseList,
    &ett_rrc_RB_InformationSetup,
    &ett_rrc_RB_InformationSetup_r4,
    &ett_rrc_RB_InformationSetup_r5,
    &ett_rrc_RB_InformationSetup_r6,
    &ett_rrc_RB_InformationSetup_r7,
    &ett_rrc_RB_InformationSetup_r8,
    &ett_rrc_RB_InformationSetup_r11,
    &ett_rrc_RB_InformationSetupList,
    &ett_rrc_RB_InformationSetupList_r4,
    &ett_rrc_RB_InformationSetupList_r5,
    &ett_rrc_RB_InformationSetupList_r6,
    &ett_rrc_RB_InformationSetupList_r7,
    &ett_rrc_RB_InformationSetupList_r8,
    &ett_rrc_RB_InformationSetupList_r11,
    &ett_rrc_RB_MappingInfo,
    &ett_rrc_RB_MappingInfo_r5,
    &ett_rrc_RB_MappingInfo_r6,
    &ett_rrc_RB_MappingInfo_r7,
    &ett_rrc_RB_MappingInfo_r8,
    &ett_rrc_RB_MappingOption,
    &ett_rrc_RB_MappingOption_r5,
    &ett_rrc_RB_MappingOption_r6,
    &ett_rrc_RB_MappingOption_r7,
    &ett_rrc_RB_MappingOption_r8,
    &ett_rrc_RB_PDCPContextRelocation,
    &ett_rrc_RB_PDCPContextRelocationList,
    &ett_rrc_RB_WithPDCP_Info,
    &ett_rrc_RB_WithPDCP_InfoList,
    &ett_rrc_RFC2507_Info,
    &ett_rrc_RFC3095_Info_r4,
    &ett_rrc_RLC_Info,
    &ett_rrc_RLC_Info_r5,
    &ett_rrc_RLC_Info_r6,
    &ett_rrc_RLC_Info_r7,
    &ett_rrc_RLC_Info_r11,
    &ett_rrc_RLC_Info_MCCH_r6,
    &ett_rrc_RLC_Info_MSCH_r6,
    &ett_rrc_RLC_Info_MTCH_r6,
    &ett_rrc_RLC_InfoChoice,
    &ett_rrc_RLC_InfoChoice_r5,
    &ett_rrc_RLC_InfoChoice_r6,
    &ett_rrc_RLC_InfoChoice_r7,
    &ett_rrc_RLC_InfoChoice_r11,
    &ett_rrc_RLC_PDU_SizeConstraint,
    &ett_rrc_RLC_PDU_SizeList,
    &ett_rrc_RLC_SizeInfo,
    &ett_rrc_RLC_SizeExplicitList,
    &ett_rrc_ROHC_ProfileList_r4,
    &ett_rrc_ROHC_PacketSizeList_r4,
    &ett_rrc_SRB_InformationSetup,
    &ett_rrc_SRB_InformationSetup_r5,
    &ett_rrc_SRB_InformationSetup_r6,
    &ett_rrc_SRB_InformationSetup_r7,
    &ett_rrc_SRB_InformationSetup_r8,
    &ett_rrc_SRB_InformationSetupList,
    &ett_rrc_SRB_InformationSetupList_r5,
    &ett_rrc_SRB_InformationSetupList_r6,
    &ett_rrc_SRB_InformationSetupList_r7,
    &ett_rrc_SRB_InformationSetupList_r8,
    &ett_rrc_SRB_InformationSetupList2,
    &ett_rrc_SRB_InformationSetupList2_r6,
    &ett_rrc_SRB_InformationSetupList2_r7,
    &ett_rrc_SRB_InformationSetupList2_r8,
    &ett_rrc_TransmissionRLC_Discard,
    &ett_rrc_UL_AM_RLC_Mode,
    &ett_rrc_UL_CounterSynchronisationInfo,
    &ett_rrc_UL_LogicalChannelMapping,
    &ett_rrc_T_rlc_SizeList,
    &ett_rrc_UL_LogicalChannelMapping_r6,
    &ett_rrc_T_ul_TrCH_Type,
    &ett_rrc_T_dch_rach_usch,
    &ett_rrc_T_rlc_SizeList_01,
    &ett_rrc_T_e_dch,
    &ett_rrc_UL_LogicalChannelMapping_r8,
    &ett_rrc_T_ul_TrCH_Type_01,
    &ett_rrc_T_dch_rach_usch_01,
    &ett_rrc_T_rlc_SizeList_02,
    &ett_rrc_T_e_dch_01,
    &ett_rrc_T_rlc_PDU_Size,
    &ett_rrc_T_fixedSize,
    &ett_rrc_UL_LogicalChannelMappingList,
    &ett_rrc_SEQUENCE_SIZE_maxLoCHperRLC_OF_UL_LogicalChannelMapping,
    &ett_rrc_UL_LogicalChannelMappingList_r6,
    &ett_rrc_SEQUENCE_SIZE_maxLoCHperRLC_OF_UL_LogicalChannelMapping_r6,
    &ett_rrc_UL_LogicalChannelMappingList_r8,
    &ett_rrc_SEQUENCE_SIZE_maxLoCHperRLC_OF_UL_LogicalChannelMapping_r8,
    &ett_rrc_UL_LogicalChannelMappings,
    &ett_rrc_UL_LogicalChannelMappings_r6,
    &ett_rrc_UL_LogicalChannelMappings_r8,
    &ett_rrc_UL_RFC3095_r4,
    &ett_rrc_UL_RLC_Mode,
    &ett_rrc_UL_TM_RLC_Mode,
    &ett_rrc_UL_UM_RLC_Mode,
    &ett_rrc_UL_TransportChannelType,
    &ett_rrc_UM_RLC_DuplAvoid_Reord_Info_r6,
    &ett_rrc_UM_RLC_OutOSeqDelivery_Info_r6,
    &ett_rrc_AddOrReconfMAC_dFlow,
    &ett_rrc_AddOrReconfMAC_ehs_ReordQ,
    &ett_rrc_AddOrReconfMAC_ehs_ReordQ_r9,
    &ett_rrc_AddOrReconfMAC_ehs_ReordQ_TDD128_v9c0ext,
    &ett_rrc_AddOrReconfMAC_ehs_ReordQ_r11,
    &ett_rrc_AllowedTFC_List,
    &ett_rrc_AllowedTFI_List,
    &ett_rrc_BitModeRLC_SizeInfo,
    &ett_rrc_T_sizeType2,
    &ett_rrc_T_sizeType3,
    &ett_rrc_T_sizeType4,
    &ett_rrc_ChannelCodingType,
    &ett_rrc_Common_E_DCH_MAC_d_Flow,
    &ett_rrc_Common_E_DCH_MAC_d_FlowList,
    &ett_rrc_Common_E_DCH_MAC_d_Flow_Info_ConcurrentTTI,
    &ett_rrc_Common_E_DCH_MAC_d_Flow_Info_List_ConcurrentTTI,
    &ett_rrc_CommonDynamicTF_Info,
    &ett_rrc_T_rlc_Size,
    &ett_rrc_T_fdd_48,
    &ett_rrc_T_tdd_13,
    &ett_rrc_T_commonTDD_Choice,
    &ett_rrc_SEQUENCE_SIZE_1_maxTF_OF_NumberOfTransportBlocks,
    &ett_rrc_CommonDynamicTF_Info_DynamicTTI,
    &ett_rrc_T_commonTDD_Choice_01,
    &ett_rrc_CommonDynamicTF_InfoList,
    &ett_rrc_CommonDynamicTF_InfoList_DynamicTTI,
    &ett_rrc_CommonTransChTFS,
    &ett_rrc_T_tti,
    &ett_rrc_CommonTransChTFS_LCR,
    &ett_rrc_T_tti_01,
    &ett_rrc_Common_MAC_ehs_ReorderingQueue,
    &ett_rrc_Common_MAC_ehs_ReorderingQueueList,
    &ett_rrc_Concurrent_Deployment_2ms_10ms_TTI,
    &ett_rrc_T_tti_selection,
    &ett_rrc_T_fixed,
    &ett_rrc_T_threshold_based,
    &ett_rrc_DedicatedDynamicTF_Info,
    &ett_rrc_T_rlc_Size_01,
    &ett_rrc_DedicatedDynamicTF_Info_DynamicTTI,
    &ett_rrc_T_rlc_Size_02,
    &ett_rrc_DedicatedDynamicTF_InfoList,
    &ett_rrc_DedicatedDynamicTF_InfoList_DynamicTTI,
    &ett_rrc_DedicatedTransChTFS,
    &ett_rrc_T_tti_02,
    &ett_rrc_DL_AddReconfTransChInfo2List,
    &ett_rrc_DL_AddReconfTransChInfoList,
    &ett_rrc_DL_AddReconfTransChInfoList_r4,
    &ett_rrc_DL_AddReconfTransChInfoList_r5,
    &ett_rrc_DL_AddReconfTransChInfoList_r7,
    &ett_rrc_DL_AddReconfTransChInfoList_r9,
    &ett_rrc_DL_AddReconfTransChInfoList_TDD128_v9c0ext,
    &ett_rrc_DL_AddReconfTransChInfoList_r11,
    &ett_rrc_DL_AddReconfTransChInformation,
    &ett_rrc_T_tfs_SignallingMode,
    &ett_rrc_DL_AddReconfTransChInformation_r4,
    &ett_rrc_T_tfs_SignallingMode_01,
    &ett_rrc_DL_AddReconfTransChInformation_r5,
    &ett_rrc_T_tfs_SignallingMode_02,
    &ett_rrc_DL_AddReconfTransChInformation_r7,
    &ett_rrc_T_tfs_SignallingMode_03,
    &ett_rrc_DL_AddReconfTransChInformation_r9,
    &ett_rrc_T_tfs_SignallingMode_04,
    &ett_rrc_DL_AddReconfTransChInformation_TDD128_v9c0ext,
    &ett_rrc_DL_AddReconfTransChInformation_r11,
    &ett_rrc_T_tfs_SignallingMode_05,
    &ett_rrc_DL_AddReconfTransChInformation2,
    &ett_rrc_T_tfs_SignallingMode_06,
    &ett_rrc_DL_CommonTransChInfo,
    &ett_rrc_T_modeSpecificInfo_19,
    &ett_rrc_T_fdd_49,
    &ett_rrc_T_dl_Parameters,
    &ett_rrc_T_tdd_14,
    &ett_rrc_DL_CommonTransChInfo_r4,
    &ett_rrc_T_modeSpecificInfo_20,
    &ett_rrc_T_fdd_50,
    &ett_rrc_T_dl_Parameters_01,
    &ett_rrc_T_dl_DCH_TFCS,
    &ett_rrc_T_tdd_15,
    &ett_rrc_DL_DeletedTransChInfoList,
    &ett_rrc_DL_DeletedTransChInfoList_r5,
    &ett_rrc_DL_DeletedTransChInfoList_r7,
    &ett_rrc_DL_TransportChannelIdentity,
    &ett_rrc_DL_TransportChannelIdentity_r5,
    &ett_rrc_DL_TransportChannelIdentity_r7,
    &ett_rrc_DL_TrCH_TypeId1_r5,
    &ett_rrc_DL_TrCH_TypeId2_r5,
    &ett_rrc_DL_TrCH_TypeId2_r7,
    &ett_rrc_T_hsdsch_01,
    &ett_rrc_DRAC_StaticInformation,
    &ett_rrc_DRAC_StaticInformationList,
    &ett_rrc_E_DCH_AddReconf_MAC_d_Flow,
    &ett_rrc_T_transmissionGrantType,
    &ett_rrc_T_non_ScheduledTransGrantInfo,
    &ett_rrc_E_DCH_AddReconf_MAC_d_Flow_r7,
    &ett_rrc_T_transmissionGrantType_01,
    &ett_rrc_T_non_ScheduledTransGrantInfo_01,
    &ett_rrc_T_modeSpecificInfo_21,
    &ett_rrc_T_fdd_51,
    &ett_rrc_E_DCH_AddReconf_MAC_d_FlowList,
    &ett_rrc_E_DCH_AddReconf_MAC_d_FlowList_r7,
    &ett_rrc_ExplicitTFCS_Configuration,
    &ett_rrc_T_replacement,
    &ett_rrc_GainFactorInformation,
    &ett_rrc_HSDSCH_Info,
    &ett_rrc_HSDSCH_Info_r7,
    &ett_rrc_T_dl_MAC_HeaderType,
    &ett_rrc_HSDSCH_Info_r9,
    &ett_rrc_T_dl_MAC_HeaderType_01,
    &ett_rrc_HSDSCH_Info_TDD128_v9c0ext,
    &ett_rrc_HSDSCH_Info_r11,
    &ett_rrc_T_dl_MAC_HeaderType_02,
    &ett_rrc_HARQ_Info,
    &ett_rrc_T_memoryPartitioning,
    &ett_rrc_SEQUENCE_SIZE_1_maxHProcesses_OF_HARQMemorySize,
    &ett_rrc_HARQ_Info_r7,
    &ett_rrc_T_memoryPartitioning_01,
    &ett_rrc_T_explicit,
    &ett_rrc_HARQ_Info_r11,
    &ett_rrc_T_memoryPartitioning_02,
    &ett_rrc_T_explicit_01,
    &ett_rrc_IndividualDL_CCTrCH_Info,
    &ett_rrc_T_tfcs_SignallingMode,
    &ett_rrc_IndividualDL_CCTrCH_InfoList,
    &ett_rrc_IndividualUL_CCTrCH_Info,
    &ett_rrc_IndividualUL_CCTrCH_InfoList,
    &ett_rrc_LogicalChannelByRB,
    &ett_rrc_LogicalChannelList,
    &ett_rrc_SEQUENCE_SIZE_1_15_OF_LogicalChannelByRB,
    &ett_rrc_MAC_d_FlowIdentityDCHandHSDSCH,
    &ett_rrc_MAC_d_PDU_SizeInfo_List,
    &ett_rrc_MAC_d_PDUsizeInfo,
    &ett_rrc_MAC_hs_AddReconfQueue_List,
    &ett_rrc_MAC_hs_AddReconfQueue,
    &ett_rrc_MAC_hs_DelQueue_List,
    &ett_rrc_MAC_hs_DelQueue,
    &ett_rrc_MAC_ehs_AddReconfReordQ_List,
    &ett_rrc_MAC_ehs_AddReconfReordQ_List_r9,
    &ett_rrc_MAC_ehs_AddReconfReordQ_List_TDD128_v9c0ext,
    &ett_rrc_MAC_ehs_AddReconfReordQ_List_r11,
    &ett_rrc_MAC_ehs_AddReconfReordQ,
    &ett_rrc_MAC_ehs_AddReconfReordQ_r9,
    &ett_rrc_MAC_ehs_AddReconfReordQ_TDD128_v9c0ext,
    &ett_rrc_MAC_ehs_AddReconfReordQ_r11,
    &ett_rrc_MAC_ehs_DelReordQ_List,
    &ett_rrc_MAC_ehs_DelReordQ,
    &ett_rrc_MAC_ehs_QueueIdDCHandHSDSCH,
    &ett_rrc_NumberOfTbSizeAndTTIList,
    &ett_rrc_NumberOfTbSizeAndTTIList_item,
    &ett_rrc_Non_allowedTFC_List,
    &ett_rrc_NumberOfTransportBlocks,
    &ett_rrc_OctetModeRLC_SizeInfoType1,
    &ett_rrc_T_sizeType2_01,
    &ett_rrc_T_sizeType3_01,
    &ett_rrc_OctetModeRLC_SizeInfoType2,
    &ett_rrc_PowerOffsetInfoShort,
    &ett_rrc_T_modeSpecificInfo_22,
    &ett_rrc_T_fdd_52,
    &ett_rrc_PowerOffsetInformation,
    &ett_rrc_PreDefTransChConfiguration,
    &ett_rrc_QualityTarget,
    &ett_rrc_RestrictedTrChInfo,
    &ett_rrc_RestrictedTrChInfoList,
    &ett_rrc_SemistaticTF_Information,
    &ett_rrc_SignalledGainFactors,
    &ett_rrc_T_modeSpecificInfo_23,
    &ett_rrc_T_fdd_53,
    &ett_rrc_SplitTFCI_Signalling,
    &ett_rrc_TFC_Subset,
    &ett_rrc_TFC_SubsetList,
    &ett_rrc_TFC_SubsetList_item,
    &ett_rrc_T_modeSpecificInfo_24,
    &ett_rrc_T_tdd_16,
    &ett_rrc_TFCI_Field2_Information,
    &ett_rrc_TFCI_Range,
    &ett_rrc_TFCI_RangeList,
    &ett_rrc_TFCS,
    &ett_rrc_TFCS_Identity,
    &ett_rrc_TFCS_InfoForDSCH,
    &ett_rrc_TFCS_ReconfAdd,
    &ett_rrc_T_ctfcSize,
    &ett_rrc_T_ctfc2Bit,
    &ett_rrc_T_ctfc2Bit_item,
    &ett_rrc_T_ctfc4Bit,
    &ett_rrc_T_ctfc4Bit_item,
    &ett_rrc_T_ctfc6Bit,
    &ett_rrc_T_ctfc6Bit_item,
    &ett_rrc_T_ctfc8Bit,
    &ett_rrc_T_ctfc8Bit_item,
    &ett_rrc_T_ctfc12Bit,
    &ett_rrc_T_ctfc12Bit_item,
    &ett_rrc_T_ctfc16Bit,
    &ett_rrc_T_ctfc16Bit_item,
    &ett_rrc_T_ctfc24Bit,
    &ett_rrc_T_ctfc24Bit_item,
    &ett_rrc_TFCS_Removal,
    &ett_rrc_TFCS_RemovalList,
    &ett_rrc_TM_SignallingInfo,
    &ett_rrc_T_tm_SignallingMode,
    &ett_rrc_T_mode2,
    &ett_rrc_TransportChannelIdentityDCHandDSCH,
    &ett_rrc_TransportFormatSet,
    &ett_rrc_TransportFormatSet_LCR,
    &ett_rrc_UL_AddReconfTransChInfoList,
    &ett_rrc_UL_AddReconfTransChInfoList_r6,
    &ett_rrc_UL_AddReconfTransChInfoList_r7,
    &ett_rrc_UL_AddReconfTransChInfoList_r8,
    &ett_rrc_UL_AddReconfTransChInformation,
    &ett_rrc_UL_AddReconfTransChInformation_r6,
    &ett_rrc_T_dch_usch,
    &ett_rrc_T_e_dch_02,
    &ett_rrc_UL_AddReconfTransChInformation_r7,
    &ett_rrc_T_dch_usch_01,
    &ett_rrc_T_e_dch_03,
    &ett_rrc_T_modeSpecific,
    &ett_rrc_T_fdd_54,
    &ett_rrc_UL_AddReconfTransChInformation_r8,
    &ett_rrc_T_dch_usch_02,
    &ett_rrc_T_e_dch_04,
    &ett_rrc_T_modeSpecific_01,
    &ett_rrc_T_fdd_55,
    &ett_rrc_UL_CommonTransChInfo,
    &ett_rrc_T_modeSpecificInfo_25,
    &ett_rrc_T_fdd_56,
    &ett_rrc_T_tdd_17,
    &ett_rrc_UL_CommonTransChInfo_r4,
    &ett_rrc_T_modeSpecificInfo_26,
    &ett_rrc_T_fdd_57,
    &ett_rrc_T_tdd_18,
    &ett_rrc_UL_ControlledTrChList,
    &ett_rrc_UL_DeletedTransChInfoList,
    &ett_rrc_UL_DeletedTransChInfoList_r6,
    &ett_rrc_UL_TransportChannelIdentity,
    &ett_rrc_UL_TransportChannelIdentity_r6,
    &ett_rrc_T_dch_usch_03,
    &ett_rrc_USCH_TransportChannelsInfo,
    &ett_rrc_USCH_TransportChannelsInfo_item,
    &ett_rrc_AC_To_ASC_MappingTable,
    &ett_rrc_AccessServiceClass_FDD,
    &ett_rrc_T_assignedSubChannelNumber,
    &ett_rrc_AccessServiceClass_TDD,
    &ett_rrc_T_channelisationCodeIndices,
    &ett_rrc_T_subchannelSize,
    &ett_rrc_T_size2,
    &ett_rrc_T_size4,
    &ett_rrc_T_subchannels_01,
    &ett_rrc_T_size8,
    &ett_rrc_T_subchannels_02,
    &ett_rrc_AccessServiceClass_TDD_r7,
    &ett_rrc_T_channelisationCodeIndices_01,
    &ett_rrc_T_subchannelSize_01,
    &ett_rrc_T_size2_01,
    &ett_rrc_T_size4_01,
    &ett_rrc_T_subchannels_04,
    &ett_rrc_T_size8_01,
    &ett_rrc_T_subchannels_05,
    &ett_rrc_T_size16,
    &ett_rrc_T_subchannels_06,
    &ett_rrc_AccessServiceClass_TDD_LCR_r4,
    &ett_rrc_T_availableSYNC_UlCodesIndics,
    &ett_rrc_T_subchannelSize_02,
    &ett_rrc_T_size2_02,
    &ett_rrc_T_size4_02,
    &ett_rrc_T_subchannels_08,
    &ett_rrc_T_size8_02,
    &ett_rrc_T_subchannels_09,
    &ett_rrc_AdditionalPRACH_TF_and_TFCS_CCCH_IEs,
    &ett_rrc_AdditionalPRACH_TF_and_TFCS_CCCH,
    &ett_rrc_AdditionalPRACH_TF_and_TFCS_CCCH_List,
    &ett_rrc_AdditionalDLSecCellInfoListFDD,
    &ett_rrc_AdditionalDLSecCellInfoListFDD_r11,
    &ett_rrc_AdditionalDLSecCellInfoListFDD2_r11,
    &ett_rrc_AdditionalDLSecCellInfoFDD,
    &ett_rrc_AdditionalDLSecCellInfoFDD_r11,
    &ett_rrc_AICH_Info,
    &ett_rrc_AICH_Info_Compressed,
    &ett_rrc_AllocationPeriodInfo,
    &ett_rrc_Antenna3And4,
    &ett_rrc_Antenna3And4_S_CPICH,
    &ett_rrc_Antenna3And4_D_CPICH,
    &ett_rrc_Antenna3_D_CPICH,
    &ett_rrc_Antenna4_D_CPICH,
    &ett_rrc_Antenna3_S_CPICH,
    &ett_rrc_Antenna4_S_CPICH,
    &ett_rrc_AP_Signature_VCAM,
    &ett_rrc_ASCSetting_FDD,
    &ett_rrc_ASCSetting_TDD,
    &ett_rrc_ASCSetting_TDD_r7,
    &ett_rrc_ASCSetting_TDD_LCR_r4,
    &ett_rrc_AvailableAP_Signature_VCAMList,
    &ett_rrc_AvailableAP_SignatureList,
    &ett_rrc_AvailableAP_SubchannelList,
    &ett_rrc_AvailableMinimumSF_ListVCAM,
    &ett_rrc_AvailableMinimumSF_VCAM,
    &ett_rrc_AvailableSignatures,
    &ett_rrc_AvailableSubChannelNumbers,
    &ett_rrc_CCTrCH_PowerControlInfo,
    &ett_rrc_CCTrCH_PowerControlInfo_r4,
    &ett_rrc_CCTrCH_PowerControlInfo_r5,
    &ett_rrc_CCTrCH_PowerControlInfo_r7,
    &ett_rrc_CD_AccessSlotSubchannelList,
    &ett_rrc_CD_SignatureCodeList,
    &ett_rrc_CellAndChannelIdentity,
    &ett_rrc_ChannelAssignmentActive,
    &ett_rrc_ChannelReqParamsForUCSM,
    &ett_rrc_CodeRange,
    &ett_rrc_CodeResourceInformation_TDD128,
    &ett_rrc_Common_E_DCH_ResourceInfoList,
    &ett_rrc_Common_E_DCH_ResourceInfoListExt,
    &ett_rrc_Common_E_RNTI_Info,
    &ett_rrc_Common_E_RNTI_Info_item,
    &ett_rrc_CommonEDCHResourceConfigInfoListExt,
    &ett_rrc_CommonEDCHSystemInfo,
    &ett_rrc_T_modeSpecificInfo_27,
    &ett_rrc_T_dummy_14,
    &ett_rrc_T_tdd_19,
    &ett_rrc_T_tdd128_11,
    &ett_rrc_T_ccch_transmission_Info,
    &ett_rrc_CommonEDCHSystemInfoFDD,
    &ett_rrc_SEQUENCE_SIZE_1_maxEDCHs_OF_Common_E_DCH_ResourceInfoList,
    &ett_rrc_CommonEDCHSystemInfoParamConcurrentTTI,
    &ett_rrc_CommonERGCHChannelConfig,
    &ett_rrc_CommonERGCHInfoFDD,
    &ett_rrc_T_configurationInfo,
    &ett_rrc_T_newConfiguration_01,
    &ett_rrc_SEQUENCE_SIZE_1_maxCellMeas_OF_CommonERGCHChannelConfig,
    &ett_rrc_CommonTimeslotInfo,
    &ett_rrc_CommonTimeslotInfoMBMS,
    &ett_rrc_CommonTimeslotInfoSCCPCH,
    &ett_rrc_ControlChannelDRXInfo_TDD128_r8,
    &ett_rrc_T_controlChannelDrxOperation,
    &ett_rrc_T_continue,
    &ett_rrc_T_newOperation,
    &ett_rrc_CPCH_PersistenceLevels,
    &ett_rrc_CPCH_PersistenceLevelsList,
    &ett_rrc_CPCH_SetInfo,
    &ett_rrc_CPCH_SetInfoList,
    &ett_rrc_DL_CCTrCh,
    &ett_rrc_DL_CCTrCh_r4,
    &ett_rrc_T_tddOption_03,
    &ett_rrc_T_tdd384_11,
    &ett_rrc_T_tdd128_12,
    &ett_rrc_DL_CCTrCh_r7,
    &ett_rrc_T_tddOption_04,
    &ett_rrc_T_tdd384_12,
    &ett_rrc_T_tdd768_06,
    &ett_rrc_T_tdd128_13,
    &ett_rrc_DL_CCTrChList,
    &ett_rrc_DL_CCTrChList_r7,
    &ett_rrc_DL_CCTrChList_r4,
    &ett_rrc_DL_CCTrChListToRemove,
    &ett_rrc_DL_ChannelisationCode,
    &ett_rrc_DL_ChannelisationCodeList,
    &ett_rrc_DL_CommonInformation,
    &ett_rrc_T_modeSpecificInfo_28,
    &ett_rrc_T_fdd_58,
    &ett_rrc_T_tdd_20,
    &ett_rrc_DL_CommonInformation_r4,
    &ett_rrc_T_modeSpecificInfo_29,
    &ett_rrc_T_fdd_59,
    &ett_rrc_T_tdd_21,
    &ett_rrc_T_tddOption_05,
    &ett_rrc_T_tdd128_14,
    &ett_rrc_DL_CommonInformation_r5,
    &ett_rrc_T_modeSpecificInfo_30,
    &ett_rrc_T_fdd_60,
    &ett_rrc_T_tdd_22,
    &ett_rrc_T_tddOption_06,
    &ett_rrc_T_tdd128_15,
    &ett_rrc_DL_CommonInformation_r6,
    &ett_rrc_T_dl_dpchInfoCommon,
    &ett_rrc_T_modeSpecificInfo_31,
    &ett_rrc_T_fdd_61,
    &ett_rrc_T_tdd_23,
    &ett_rrc_T_tddOption_07,
    &ett_rrc_T_tdd128_16,
    &ett_rrc_DL_CommonInformation_r7,
    &ett_rrc_T_dl_dpchInfoCommon_01,
    &ett_rrc_T_modeSpecificInfo_32,
    &ett_rrc_T_fdd_62,
    &ett_rrc_T_tdd_24,
    &ett_rrc_T_tddOption_08,
    &ett_rrc_T_tdd128_17,
    &ett_rrc_DL_CommonInformation_r8,
    &ett_rrc_T_dl_dpchInfoCommon_02,
    &ett_rrc_T_modeSpecificInfo_33,
    &ett_rrc_T_fdd_63,
    &ett_rrc_T_tdd_25,
    &ett_rrc_T_tddOption_09,
    &ett_rrc_T_tdd128_18,
    &ett_rrc_DL_CommonInformation_r10,
    &ett_rrc_T_dl_dpchInfoCommon_03,
    &ett_rrc_T_modeSpecificInfo_34,
    &ett_rrc_T_fdd_64,
    &ett_rrc_T_tdd_26,
    &ett_rrc_T_tddOption_10,
    &ett_rrc_T_tdd128_19,
    &ett_rrc_DL_CommonInformation_r11,
    &ett_rrc_T_dl_dpchInfoCommon_04,
    &ett_rrc_T_modeSpecificInfo_35,
    &ett_rrc_T_fdd_65,
    &ett_rrc_T_tdd_27,
    &ett_rrc_T_tddOption_11,
    &ett_rrc_T_tdd128_20,
    &ett_rrc_DL_CommonInformationPost,
    &ett_rrc_DL_CommonInformationPredef,
    &ett_rrc_DL_DPCH_InfoCommon,
    &ett_rrc_T_cfnHandling,
    &ett_rrc_T_initialise,
    &ett_rrc_T_modeSpecificInfo_36,
    &ett_rrc_T_fdd_66,
    &ett_rrc_T_tdd_28,
    &ett_rrc_DL_DPCH_InfoCommon_r4,
    &ett_rrc_T_cfnHandling_01,
    &ett_rrc_T_initialise_01,
    &ett_rrc_T_modeSpecificInfo_37,
    &ett_rrc_T_fdd_67,
    &ett_rrc_T_tdd_29,
    &ett_rrc_DL_DPCH_InfoCommon_r6,
    &ett_rrc_T_cfnHandling_02,
    &ett_rrc_T_maintain,
    &ett_rrc_T_modeSpecificInfo_38,
    &ett_rrc_T_fdd_68,
    &ett_rrc_T_tdd_30,
    &ett_rrc_DL_DPCH_InfoCommonPost,
    &ett_rrc_DL_DPCH_InfoCommonPredef,
    &ett_rrc_T_modeSpecificInfo_39,
    &ett_rrc_T_fdd_69,
    &ett_rrc_T_tdd_31,
    &ett_rrc_DL_DPCH_InfoPerRL,
    &ett_rrc_T_fdd_70,
    &ett_rrc_T_tdd_32,
    &ett_rrc_DL_DPCH_InfoPerRL_r4,
    &ett_rrc_T_fdd_71,
    &ett_rrc_T_tdd_33,
    &ett_rrc_DL_DPCH_InfoPerRL_r5,
    &ett_rrc_T_fdd_72,
    &ett_rrc_T_tdd_34,
    &ett_rrc_DL_DPCH_InfoPerRL_r6,
    &ett_rrc_T_fdd_73,
    &ett_rrc_T_tdd_35,
    &ett_rrc_DL_DPCH_InfoPerRL_r7,
    &ett_rrc_T_fdd_74,
    &ett_rrc_T_tdd_36,
    &ett_rrc_DL_FDPCH_InfoPerRL_r6,
    &ett_rrc_DL_FDPCH_InfoPerRL_r7,
    &ett_rrc_DL_DPCH_InfoPerRL_PostFDD,
    &ett_rrc_DL_DPCH_InfoPerRL_PostTDD,
    &ett_rrc_DL_DPCH_InfoPerRL_PostTDD_LCR_r4,
    &ett_rrc_DL_DPCH_PowerControlInfo,
    &ett_rrc_T_modeSpecificInfo_40,
    &ett_rrc_T_fdd_75,
    &ett_rrc_T_tdd_37,
    &ett_rrc_DL_FDPCH_InfoCommon_r6,
    &ett_rrc_T_cfnHandling_03,
    &ett_rrc_T_maintain_01,
    &ett_rrc_DL_HSPDSCH_Information,
    &ett_rrc_T_modeSpecificInfo_41,
    &ett_rrc_T_tdd_38,
    &ett_rrc_T_tdd384_13,
    &ett_rrc_T_tdd128_21,
    &ett_rrc_DL_HSPDSCH_Information_r6,
    &ett_rrc_T_modeSpecificInfo_42,
    &ett_rrc_T_tdd_39,
    &ett_rrc_T_tdd384_14,
    &ett_rrc_T_tdd128_22,
    &ett_rrc_DL_HSPDSCH_Information_r7,
    &ett_rrc_T_modeSpecificInfo_43,
    &ett_rrc_T_tdd_40,
    &ett_rrc_T_tdd384_15,
    &ett_rrc_T_tdd768_07,
    &ett_rrc_T_tdd128_23,
    &ett_rrc_T_fdd_76,
    &ett_rrc_DL_HSPDSCH_Information_r8,
    &ett_rrc_T_modeSpecificInfo_44,
    &ett_rrc_T_tdd_41,
    &ett_rrc_T_tdd384_16,
    &ett_rrc_T_tdd768_08,
    &ett_rrc_T_tdd128_24,
    &ett_rrc_T_fdd_77,
    &ett_rrc_DL_HSPDSCH_Information_r8_ext,
    &ett_rrc_T_modeSpecificInfo_45,
    &ett_rrc_T_tdd_42,
    &ett_rrc_T_tdd128_25,
    &ett_rrc_DL_HSPDSCH_Information_r8_ext2,
    &ett_rrc_DL_HSPDSCH_Information_r9,
    &ett_rrc_T_modeSpecificInfo_46,
    &ett_rrc_T_tdd_43,
    &ett_rrc_T_tdd384_17,
    &ett_rrc_T_tdd768_09,
    &ett_rrc_T_tdd128_26,
    &ett_rrc_T_fdd_78,
    &ett_rrc_DL_HSPDSCH_Information_r11,
    &ett_rrc_T_modeSpecificInfo_47,
    &ett_rrc_T_tdd_44,
    &ett_rrc_T_tdd384_18,
    &ett_rrc_T_tdd768_10,
    &ett_rrc_T_tdd128_27,
    &ett_rrc_T_fdd_79,
    &ett_rrc_DL_HSPDSCH_MultiCarrier_Information,
    &ett_rrc_DL_HSPDSCH_MultiCarrier_Information_item,
    &ett_rrc_SEQUENCE_SIZE_1_maxHSSCCHs_OF_HS_SCCH_TDD128_MultiCarrier,
    &ett_rrc_DL_HSPDSCH_TS_Configuration,
    &ett_rrc_DL_HSPDSCH_TS_Configuration_item,
    &ett_rrc_DL_HSPDSCH_TS_Configuration_VHCR,
    &ett_rrc_DL_HSPDSCH_TS_Configuration_VHCR_item,
    &ett_rrc_DL_InformationPerRL,
    &ett_rrc_T_modeSpecificInfo_48,
    &ett_rrc_T_fdd_80,
    &ett_rrc_DL_InformationPerRL_r4,
    &ett_rrc_T_modeSpecificInfo_49,
    &ett_rrc_T_fdd_81,
    &ett_rrc_DL_InformationPerRL_r5,
    &ett_rrc_T_modeSpecificInfo_50,
    &ett_rrc_T_fdd_82,
    &ett_rrc_DL_InformationPerRL_r5bis,
    &ett_rrc_T_modeSpecificInfo_51,
    &ett_rrc_T_fdd_83,
    &ett_rrc_DL_InformationPerRL_r6,
    &ett_rrc_T_modeSpecificInfo_52,
    &ett_rrc_T_fdd_84,
    &ett_rrc_T_dl_dpchInfo,
    &ett_rrc_T_e_HICH_Info,
    &ett_rrc_T_e_RGCH_Info_01,
    &ett_rrc_DL_InformationPerRL_v6b0ext,
    &ett_rrc_DL_InformationPerRL_r7,
    &ett_rrc_T_modeSpecificInfo_53,
    &ett_rrc_T_fdd_85,
    &ett_rrc_T_dl_dpchInfo_01,
    &ett_rrc_T_modeSpecificInfo2,
    &ett_rrc_T_fdd_86,
    &ett_rrc_T_e_HICH_Info_01,
    &ett_rrc_T_e_RGCH_Info_02,
    &ett_rrc_T_tdd_45,
    &ett_rrc_T_tdd384_tdd768,
    &ett_rrc_T_tdd128_28,
    &ett_rrc_DL_InformationPerRL_r8,
    &ett_rrc_T_modeSpecificInfo_54,
    &ett_rrc_T_fdd_87,
    &ett_rrc_T_dl_dpchInfo_02,
    &ett_rrc_T_modeSpecificInfo2_01,
    &ett_rrc_T_fdd_88,
    &ett_rrc_T_e_HICH_Info_02,
    &ett_rrc_T_e_RGCH_Info_03,
    &ett_rrc_T_tdd_46,
    &ett_rrc_T_tdd384_tdd768_01,
    &ett_rrc_T_tdd128_29,
    &ett_rrc_DL_InformationPerRL_r11,
    &ett_rrc_T_modeSpecificInfo_55,
    &ett_rrc_T_fdd_89,
    &ett_rrc_T_dl_dpchInfo_03,
    &ett_rrc_T_modeSpecificInfo2_02,
    &ett_rrc_T_fdd_90,
    &ett_rrc_T_e_HICH_Info_03,
    &ett_rrc_T_e_RGCH_Info_04,
    &ett_rrc_T_f_TPICH_Info,
    &ett_rrc_T_tdd_47,
    &ett_rrc_T_tdd384_tdd768_02,
    &ett_rrc_T_tdd128_30,
    &ett_rrc_DL_InformationPerRL_List,
    &ett_rrc_DL_InformationPerRL_List_r4,
    &ett_rrc_DL_InformationPerRL_List_r5,
    &ett_rrc_DL_InformationPerRL_List_r6,
    &ett_rrc_DL_InformationPerRL_List_v6b0ext,
    &ett_rrc_DL_InformationPerRL_List_r5bis,
    &ett_rrc_DL_InformationPerRL_List_r7,
    &ett_rrc_DL_InformationPerRL_List_r8,
    &ett_rrc_DL_InformationPerRL_List_r11,
    &ett_rrc_DL_InformationPerRL_ListPostFDD,
    &ett_rrc_DL_InformationPerRL_PostFDD,
    &ett_rrc_DL_InformationPerRL_PostTDD,
    &ett_rrc_DL_InformationPerRL_PostTDD_LCR_r4,
    &ett_rrc_DL_MultiCarrier_Information,
    &ett_rrc_DL_PDSCH_Information,
    &ett_rrc_Dl_rate_matching_restriction,
    &ett_rrc_DL_SecondaryCellInfoFDD,
    &ett_rrc_T_newConfiguration_02,
    &ett_rrc_SEQUENCE_SIZE_1_maxHSSCCHs_OF_HS_SCCH_Codes,
    &ett_rrc_DL_SecondaryCellInfoFDD_v890ext,
    &ett_rrc_DL_SecondaryCellInfoFDD_r9,
    &ett_rrc_T_configurationInfo_01,
    &ett_rrc_T_newConfiguration_03,
    &ett_rrc_DL_SecondaryCellInfoFDD_r10,
    &ett_rrc_T_configurationInfo_02,
    &ett_rrc_T_newConfiguration_04,
    &ett_rrc_DL_SecondaryCellInfoFDD_r11,
    &ett_rrc_T_configurationInfo_03,
    &ett_rrc_T_newConfiguration_05,
    &ett_rrc_DL_TPC_PowerOffsetPerRL,
    &ett_rrc_DL_TPC_PowerOffsetPerRL_List,
    &ett_rrc_DL_TS_ChannelisationCodesShort,
    &ett_rrc_T_codesRepresentation,
    &ett_rrc_T_consecutive,
    &ett_rrc_T_bitmap,
    &ett_rrc_DL_TS_ChannelisationCodesShort_VHCR,
    &ett_rrc_T_codesRepresentation_01,
    &ett_rrc_T_consecutive_01,
    &ett_rrc_T_bitmap_01,
    &ett_rrc_DL_ChannelCodes_MBSFN_IMB384,
    &ett_rrc_DLUL_HSPA_Information_r8,
    &ett_rrc_DLUL_HSPA_Information_r9,
    &ett_rrc_DLUL_HSPA_Information_r10,
    &ett_rrc_DownlinkAdditionalTimeslots,
    &ett_rrc_T_parameters,
    &ett_rrc_T_sameAsLast,
    &ett_rrc_T_newParameters,
    &ett_rrc_DownlinkAdditionalTimeslots_VHCR,
    &ett_rrc_T_parameters_01,
    &ett_rrc_T_sameAsLast_01,
    &ett_rrc_T_newParameters_01,
    &ett_rrc_DownlinkAdditionalTimeslots_LCR_r4,
    &ett_rrc_T_parameters_02,
    &ett_rrc_T_sameAsLast_02,
    &ett_rrc_T_newParameters_02,
    &ett_rrc_DownlinkAdditionalTimeslots_r7,
    &ett_rrc_T_parameters_03,
    &ett_rrc_T_sameAsLast_03,
    &ett_rrc_T_newParameters_03,
    &ett_rrc_DownlinkTimeslotsCodes,
    &ett_rrc_T_moreTimeslots,
    &ett_rrc_T_additionalTimeslots,
    &ett_rrc_SEQUENCE_SIZE_1_maxTS_1_OF_DownlinkAdditionalTimeslots,
    &ett_rrc_DownlinkTimeslotsCodes_VHCR,
    &ett_rrc_T_moreTimeslots_01,
    &ett_rrc_T_additionalTimeslots_01,
    &ett_rrc_SEQUENCE_SIZE_1_maxTS_1_OF_DownlinkAdditionalTimeslots_VHCR,
    &ett_rrc_DownlinkTimeslotsCodes_LCR_r4,
    &ett_rrc_T_moreTimeslots_02,
    &ett_rrc_T_additionalTimeslots_02,
    &ett_rrc_SEQUENCE_SIZE_1_maxTS_LCR_1_OF_DownlinkAdditionalTimeslots_LCR_r4,
    &ett_rrc_DownlinkTimeslotsCodes_r7,
    &ett_rrc_T_moreTimeslots_03,
    &ett_rrc_T_additionalTimeslots_03,
    &ett_rrc_SEQUENCE_SIZE_1_maxTS_1_OF_DownlinkAdditionalTimeslots_r7,
    &ett_rrc_DPCH_CompressedModeInfo,
    &ett_rrc_DPCH_CompressedModeInfo_r8,
    &ett_rrc_DPCH_CompressedModeInfo_r10,
    &ett_rrc_DPCH_CompressedModeStatusInfo,
    &ett_rrc_SEQUENCE_SIZE_1_maxTGPS_OF_TGP_SequenceShort,
    &ett_rrc_DPCH_CompressedModeStatusInfo_r10,
    &ett_rrc_SEQUENCE_SIZE_1_maxTGPS_OF_TGP_SequenceShort_r10,
    &ett_rrc_DRX_Info,
    &ett_rrc_DSCH_Mapping,
    &ett_rrc_DSCH_MappingList,
    &ett_rrc_DSCH_TransportChannelsInfo,
    &ett_rrc_DSCH_TransportChannelsInfo_item,
    &ett_rrc_DTX_DRX_Info_r7,
    &ett_rrc_DTX_E_DCH_TTI_10ms,
    &ett_rrc_DTX_E_DCH_TTI_2ms,
    &ett_rrc_DTX_Info,
    &ett_rrc_T_e_dch_TTI_Length,
    &ett_rrc_DynamicPersistenceLevelList,
    &ett_rrc_DynamicPersistenceLevelTF_List,
    &ett_rrc_DynamicTFInformationCCCH,
    &ett_rrc_E_AGCH_DRX_Info_TDD128,
    &ett_rrc_T_e_AGCH_DRX_InfoType,
    &ett_rrc_T_e_AGCH_DRX_Parameters,
    &ett_rrc_E_AGCH_Individual,
    &ett_rrc_E_AGCH_Individual_VHCR,
    &ett_rrc_E_AGCH_Individual_LCR,
    &ett_rrc_T_midambleAllocationMode,
    &ett_rrc_E_AGCH_Information,
    &ett_rrc_E_AGCH_Information_r7,
    &ett_rrc_T_modeSpecific_02,
    &ett_rrc_T_fdd_91,
    &ett_rrc_T_tdd_48,
    &ett_rrc_T_tdd384_19,
    &ett_rrc_T_tdd768_11,
    &ett_rrc_T_tdd128_31,
    &ett_rrc_E_AGCH_Information_r8,
    &ett_rrc_T_modeSpecific_03,
    &ett_rrc_T_fdd_92,
    &ett_rrc_T_tdd_49,
    &ett_rrc_T_tdd384_20,
    &ett_rrc_T_tdd768_12,
    &ett_rrc_T_tdd128_32,
    &ett_rrc_E_AGCH_Information_TDD128,
    &ett_rrc_E_AGCH_Set_Config,
    &ett_rrc_E_AGCH_Set_Config_VHCR,
    &ett_rrc_E_AGCH_Set_Config_LCR,
    &ett_rrc_E_ROCH_Information,
    &ett_rrc_E_DCH_ReconfigurationInfo,
    &ett_rrc_SEQUENCE_SIZE_1_maxEDCHRL_OF_E_DCH_RL_InfoOtherCell,
    &ett_rrc_E_DCH_ReconfigurationInfo_r7,
    &ett_rrc_E_DCH_ReconfigurationInfo_r11,
    &ett_rrc_SEQUENCE_SIZE_1_maxEDCHRL_OF_E_DCH_RL_InfoOtherCell_r11,
    &ett_rrc_E_DCH_ReconfigurationInfo_SecULFrequency,
    &ett_rrc_SEQUENCE_SIZE_1_maxEDCHRL_OF_E_DCH_RL_InfoOtherCell_SecULFreq,
    &ett_rrc_E_DCH_RL_InfoNewServingCell,
    &ett_rrc_T_servingGrant_01,
    &ett_rrc_T_e_RGCH_Info_05,
    &ett_rrc_E_DCH_RL_InfoNewServingCell_r7,
    &ett_rrc_T_servingGrant_02,
    &ett_rrc_T_e_RGCH_Info_06,
    &ett_rrc_E_DCH_RL_InfoNewServingCell_r11,
    &ett_rrc_T_servingGrant_03,
    &ett_rrc_T_e_RGCH_Info_07,
    &ett_rrc_E_DCH_RL_InfoNewSecServingCell,
    &ett_rrc_T_e_RGCH_Info_08,
    &ett_rrc_E_DCH_RL_InfoOtherCell,
    &ett_rrc_T_e_HICH_Info_04,
    &ett_rrc_T_e_RGCH_Info_09,
    &ett_rrc_E_DCH_RL_InfoOtherCell_r11,
    &ett_rrc_T_e_HICH_Info_05,
    &ett_rrc_T_e_RGCH_Info_10,
    &ett_rrc_E_DCH_RL_InfoOtherCell_SecULFreq,
    &ett_rrc_T_e_RGCH_Info_11,
    &ett_rrc_E_DCH_RL_InfoSameServingCell,
    &ett_rrc_E_DCH_SPS_Information_TDD128,
    &ett_rrc_T_e_dch_SPS_Operation,
    &ett_rrc_E_DCH_SPS_NewOperation_TDD128,
    &ett_rrc_T_initialSPSInfoForEDCH,
    &ett_rrc_E_DCH_TxPatternList_TDD128,
    &ett_rrc_E_DCH_TxPatternList_TDD128_item,
    &ett_rrc_E_DPCCH_Info,
    &ett_rrc_E_DPCCH_Info_r7,
    &ett_rrc_E_DPDCH_Info,
    &ett_rrc_E_DPDCH_Info_r7,
    &ett_rrc_E_DPDCH_Info_r8,
    &ett_rrc_E_DPDCH_Reference_E_TFCI,
    &ett_rrc_E_DPDCH_Reference_E_TFCI_r7,
    &ett_rrc_E_DPDCH_Reference_E_TFCIList,
    &ett_rrc_E_DPDCH_Reference_E_TFCIList_r7,
    &ett_rrc_E_DPDCH_SchedulingInfoConfiguration,
    &ett_rrc_E_DPDCH_SchedulingTransmConfiguration,
    &ett_rrc_T_servingGrant_04,
    &ett_rrc_E_HICH_Information,
    &ett_rrc_E_HICH_Information_r11,
    &ett_rrc_E_HICH_Information_CommonEdch,
    &ett_rrc_E_HICH_Information_TDD384_768,
    &ett_rrc_T_modeSpecificInfo_56,
    &ett_rrc_E_HICH_Information_LCR,
    &ett_rrc_T_midambleAllocationMode_01,
    &ett_rrc_E_HICH_Information_TDD128,
    &ett_rrc_E_HICH_Information_LCR_List,
    &ett_rrc_E_HICH_Information_For_SPS_TDD128,
    &ett_rrc_T_configurationmode,
    &ett_rrc_T_implicit,
    &ett_rrc_T_explicit_02,
    &ett_rrc_T_midambleAllocationMode_02,
    &ett_rrc_E_PUCH_Info,
    &ett_rrc_T_modeSpecificInfo_57,
    &ett_rrc_T_tdd348_tdd768,
    &ett_rrc_SEQUENCE_SIZE_1_maxTS_2_OF_E_PUCH_TS_Slots,
    &ett_rrc_T_tdd128_33,
    &ett_rrc_SEQUENCE_SIZE_1_maxTS_LCR_1_OF_E_PUCH_TS_Slots_LCR,
    &ett_rrc_E_PUCH_Info_TDD128,
    &ett_rrc_E_PUCH_Info_MulticarrierEDCH_TDD128,
    &ett_rrc_E_PUCH_TS_Slots,
    &ett_rrc_E_PUCH_TS_Slots_LCR,
    &ett_rrc_T_midambleAllocationMode_03,
    &ett_rrc_E_RGCH_Information,
    &ett_rrc_E_RGCH_Information_CommonEdch,
    &ett_rrc_E_RUCCH_Info,
    &ett_rrc_T_modeSpecificInfo_58,
    &ett_rrc_T_tdd384_tdd768_03,
    &ett_rrc_T_modeSpecificInfo_59,
    &ett_rrc_T_tdd384_21,
    &ett_rrc_SEQUENCE_SIZE_1_8_OF_SF16Codes,
    &ett_rrc_SEQUENCE_SIZE_1_8_OF_SF8Codes,
    &ett_rrc_T_tdd768_13,
    &ett_rrc_SEQUENCE_SIZE_1_16_OF_SF32Codes,
    &ett_rrc_SEQUENCE_SIZE_1_16_OF_SF16Codes2,
    &ett_rrc_T_tdd128_34,
    &ett_rrc_E_RUCCH_Info_TDD128,
    &ett_rrc_E_TFC_Boost_Info_r7,
    &ett_rrc_E_TFCS_Info,
    &ett_rrc_SEQUENCE_SIZE_1_8_OF_Reference_Beta_QPSK,
    &ett_rrc_SEQUENCE_SIZE_1_8_OF_Reference_Beta_16QAM,
    &ett_rrc_EXT_UL_TimingAdvance,
    &ett_rrc_T_modeSpecificInfo_60,
    &ett_rrc_T_tdd384_22,
    &ett_rrc_T_tdd768_14,
    &ett_rrc_FACH_PCH_Information,
    &ett_rrc_FACH_PCH_InformationList,
    &ett_rrc_Fallback_R99_PRACH_info_IEs,
    &ett_rrc_FPACH_Info_r4,
    &ett_rrc_F_TPICH_InfoNewServingCell,
    &ett_rrc_F_TPICH_InfoOtherCell,
    &ett_rrc_T_f_tpich_Info,
    &ett_rrc_F_TPICH_Information,
    &ett_rrc_F_TPICH_ReconfigurationInfo,
    &ett_rrc_SEQUENCE_SIZE_1_maxRL_OF_F_TPICH_InfoOtherCell,
    &ett_rrc_FreqIndexListForEnhancedMeas,
    &ett_rrc_FrequencyInfo,
    &ett_rrc_T_modeSpecificInfo_61,
    &ett_rrc_FrequencyInfoFDD,
    &ett_rrc_FrequencyInfoTDD,
    &ett_rrc_HS_ChannelisationCodeSetInfo_LCR,
    &ett_rrc_HS_DSCH_CommonSystemInformation,
    &ett_rrc_SEQUENCE_SIZE_1_maxCommonHRNTI_OF_H_RNTI,
    &ett_rrc_HS_DSCH_CommonSystemInformation_TDD128,
    &ett_rrc_HS_DSCH_DrxCellfach_info,
    &ett_rrc_HS_DSCH_DrxCellfach_info_TDD128,
    &ett_rrc_HS_DSCH_DrxCellfach_SecondDrx_info,
    &ett_rrc_T_drx_level,
    &ett_rrc_T_one_level_DRX,
    &ett_rrc_T_two_level_DRX,
    &ett_rrc_HS_DSCH_PagingSystemInformation,
    &ett_rrc_SEQUENCE_SIZE_1_maxSCCPCH_OF_PICH_ForHSDPASupportedPaging,
    &ett_rrc_SEQUENCE_SIZE_1_2_OF_TransportBlockSizeIndex,
    &ett_rrc_HS_DSCH_PagingSystemInformation_TDD128,
    &ett_rrc_SEQUENCE_SIZE_1_maxSCCPCH_OF_PICH_ForHSDPASupportedPaging_TDD128,
    &ett_rrc_HS_DSCH_RxPatternList_TDD128,
    &ett_rrc_HS_DSCH_RxPatternList_TDD128_item,
    &ett_rrc_HS_DSCH_TbsList_TDD128,
    &ett_rrc_HS_DSCH_TbsList_TDD128_item,
    &ett_rrc_HS_DSCH_SPS_Information_TDD128,
    &ett_rrc_T_hs_dsch_SPS_Operation,
    &ett_rrc_HS_DSCH_SPS_NewOperation_TDD128,
    &ett_rrc_T_hARQInfoForSPS,
    &ett_rrc_T_initialSPSInfoForHSDSCH,
    &ett_rrc_HS_PDSCH_Midamble_Configuration_TDD128,
    &ett_rrc_T_midambleAllocationMode_04,
    &ett_rrc_HS_SCCH_Info,
    &ett_rrc_T_modeSpecificInfo_62,
    &ett_rrc_T_fdd_93,
    &ett_rrc_T_tdd_50,
    &ett_rrc_T_tdd384_23,
    &ett_rrc_SEQUENCE_SIZE_1_maxHSSCCHs_OF_HS_SCCH_TDD384,
    &ett_rrc_SEQUENCE_SIZE_1_maxHSSCCHs_OF_HS_SCCH_TDD128,
    &ett_rrc_HS_SCCH_Info_r6,
    &ett_rrc_T_modeSpecificInfo_63,
    &ett_rrc_T_fdd_94,
    &ett_rrc_T_tdd_51,
    &ett_rrc_T_tdd384_24,
    &ett_rrc_SEQUENCE_SIZE_1_maxHSSCCHs_OF_HS_SCCH_TDD384_r6,
    &ett_rrc_T_tdd128_35,
    &ett_rrc_SEQUENCE_SIZE_1_maxHSSCCHs_OF_HS_SCCH_TDD128_r6,
    &ett_rrc_HS_SCCH_Info_r7,
    &ett_rrc_T_modeSpecificInfo_64,
    &ett_rrc_T_fdd_95,
    &ett_rrc_T_tdd_52,
    &ett_rrc_T_tdd384_25,
    &ett_rrc_T_tdd768_15,
    &ett_rrc_SEQUENCE_SIZE_1_maxHSSCCHs_OF_HS_SCCH_TDD768,
    &ett_rrc_T_tdd128_36,
    &ett_rrc_HS_SCCH_Info_r8_ext,
    &ett_rrc_T_modeSpecificInfo_65,
    &ett_rrc_T_tdd_53,
    &ett_rrc_T_tdd128_37,
    &ett_rrc_HS_SCCH_Info_r9,
    &ett_rrc_T_modeSpecificInfo_66,
    &ett_rrc_T_fdd_96,
    &ett_rrc_T_tdd_54,
    &ett_rrc_T_tdd384_26,
    &ett_rrc_T_tdd768_16,
    &ett_rrc_T_tdd128_38,
    &ett_rrc_HS_SCCH_DRX_Info_TDD128,
    &ett_rrc_HS_SCCH_Less_NewOperation,
    &ett_rrc_HS_SCCH_LessInfo_r7,
    &ett_rrc_T_hs_scchLessOperation,
    &ett_rrc_HS_SCCH_LessTFSList,
    &ett_rrc_HS_SCCH_LessTFSList_item,
    &ett_rrc_HS_SCCH_SystemInfo,
    &ett_rrc_HS_SCCH_SystemInfo_TDD128,
    &ett_rrc_HS_SCCH_TDD128,
    &ett_rrc_T_midambleAllocationMode_05,
    &ett_rrc_HS_SCCH_TDD128_r6,
    &ett_rrc_T_midambleAllocationMode_06,
    &ett_rrc_HS_SCCH_TDD128_MultiCarrier,
    &ett_rrc_T_midambleAllocationMode_07,
    &ett_rrc_HS_SICH_Configuration_TDD128,
    &ett_rrc_T_midambleAllocationMode_08,
    &ett_rrc_T_ueSpecificMidamble,
    &ett_rrc_HS_SICH_Configuration_TDD128_r6,
    &ett_rrc_T_midambleAllocationMode_09,
    &ett_rrc_T_ueSpecificMidamble_01,
    &ett_rrc_HS_SICH_List_TDD128,
    &ett_rrc_HS_SICH_List_TDD128_item,
    &ett_rrc_T_implicit_01,
    &ett_rrc_T_explicit_03,
    &ett_rrc_HS_SICH_ReferenceSignalInfoList,
    &ett_rrc_HS_SICH_ReferenceSignalInfoList_item,
    &ett_rrc_HS_SCCH_TDD384,
    &ett_rrc_T_midambleAllocationMode_10,
    &ett_rrc_T_ueSpecificMidamble_02,
    &ett_rrc_HS_SCCH_TDD384_r6,
    &ett_rrc_T_midambleAllocationMode_11,
    &ett_rrc_T_ueSpecificMidamble_03,
    &ett_rrc_HS_SCCH_TDD768,
    &ett_rrc_T_midambleAllocationMode_12,
    &ett_rrc_T_ueSpecificMidamble_04,
    &ett_rrc_HS_SICH_Configuration_TDD384,
    &ett_rrc_T_midambleAllocationMode_13,
    &ett_rrc_T_ueSpecificMidamble_05,
    &ett_rrc_HS_SICH_Configuration_TDD768,
    &ett_rrc_T_midambleAllocationMode_14,
    &ett_rrc_T_ueSpecificMidamble_06,
    &ett_rrc_HS_SICH_Power_Control_Info_TDD384,
    &ett_rrc_HS_SICH_Power_Control_Info_TDD768,
    &ett_rrc_IndividualTimeslotInfo,
    &ett_rrc_IndividualTimeslotInfo_VHCR,
    &ett_rrc_IndividualTimeslotInfo_LCR_r4,
    &ett_rrc_IndividualTimeslotInfo_LCR_r4_ext,
    &ett_rrc_IndividualTimeslotInfo_r7,
    &ett_rrc_IndividualTS_Interference,
    &ett_rrc_IndividualTS_InterferenceList,
    &ett_rrc_Measurement_Feedback_Info,
    &ett_rrc_T_modeSpecificInfo_67,
    &ett_rrc_T_fdd_97,
    &ett_rrc_Measurement_Feedback_Info_r7,
    &ett_rrc_T_modeSpecificInfo_68,
    &ett_rrc_T_fdd_98,
    &ett_rrc_Measurement_Feedback_Info_r11,
    &ett_rrc_T_modeSpecificInfo_69,
    &ett_rrc_T_fdd_99,
    &ett_rrc_Measurement_Feedback_Info_ConcurrentTTI,
    &ett_rrc_MidambleShiftAndBurstType,
    &ett_rrc_T_burstType,
    &ett_rrc_T_type1_01,
    &ett_rrc_T_midambleAllocationMode_15,
    &ett_rrc_T_ueSpecificMidamble_07,
    &ett_rrc_T_type2,
    &ett_rrc_T_midambleAllocationMode_16,
    &ett_rrc_T_ueSpecificMidamble_08,
    &ett_rrc_T_type3,
    &ett_rrc_T_midambleAllocationMode_17,
    &ett_rrc_T_ueSpecificMidamble_09,
    &ett_rrc_MidambleShiftAndBurstType_VHCR,
    &ett_rrc_T_burstType_01,
    &ett_rrc_T_type1_02,
    &ett_rrc_T_midambleAllocationMode_18,
    &ett_rrc_T_ueSpecificMidamble_10,
    &ett_rrc_T_type2_01,
    &ett_rrc_T_midambleAllocationMode_19,
    &ett_rrc_T_ueSpecificMidamble_11,
    &ett_rrc_T_type3_01,
    &ett_rrc_T_midambleAllocationMode_20,
    &ett_rrc_T_ueSpecificMidamble_12,
    &ett_rrc_MidambleShiftAndBurstType_r7,
    &ett_rrc_T_burstType_02,
    &ett_rrc_T_type1_03,
    &ett_rrc_T_midambleAllocationMode_21,
    &ett_rrc_T_ueSpecificMidamble_13,
    &ett_rrc_T_type2_02,
    &ett_rrc_T_midambleAllocationMode_22,
    &ett_rrc_T_ueSpecificMidamble_14,
    &ett_rrc_T_type3_02,
    &ett_rrc_T_midambleAllocationMode_23,
    &ett_rrc_T_ueSpecificMidamble_15,
    &ett_rrc_MidambleShiftAndBurstType_DL,
    &ett_rrc_T_burstType_03,
    &ett_rrc_T_type1_04,
    &ett_rrc_T_midambleAllocationMode_24,
    &ett_rrc_T_ueSpecificMidamble_16,
    &ett_rrc_T_type2_03,
    &ett_rrc_T_midambleAllocationMode_25,
    &ett_rrc_T_ueSpecificMidamble_17,
    &ett_rrc_MidambleShiftAndBurstType_DL_VHCR,
    &ett_rrc_T_burstType_04,
    &ett_rrc_T_type1_05,
    &ett_rrc_T_midambleAllocationMode_26,
    &ett_rrc_T_ueSpecificMidamble_18,
    &ett_rrc_T_type2_04,
    &ett_rrc_T_midambleAllocationMode_27,
    &ett_rrc_T_ueSpecificMidamble_19,
    &ett_rrc_MidambleShiftAndBurstType_LCR_r4,
    &ett_rrc_T_midambleAllocationMode_28,
    &ett_rrc_T_ueSpecificMidamble_20,
    &ett_rrc_MidambleShiftAndBurstType_EDCH,
    &ett_rrc_T_burstType_05,
    &ett_rrc_T_type1_06,
    &ett_rrc_T_midambleAllocationMode_29,
    &ett_rrc_T_ueSpecificMidamble_21,
    &ett_rrc_T_type2_05,
    &ett_rrc_T_midambleAllocationMode_30,
    &ett_rrc_T_ueSpecificMidamble_22,
    &ett_rrc_MIMO_Parameters_r7,
    &ett_rrc_MIMO4x4_Parameters,
    &ett_rrc_T_newConfiguration_06,
    &ett_rrc_MIMO_Parameters_v7g0ext,
    &ett_rrc_MIMO_Parameters_r8,
    &ett_rrc_T_modeSpecificInfo_70,
    &ett_rrc_T_fdd_100,
    &ett_rrc_T_tdd_55,
    &ett_rrc_T_tdd128_39,
    &ett_rrc_MIMO_Parameters_r9,
    &ett_rrc_T_modeSpecificInfo_71,
    &ett_rrc_T_fdd_101,
    &ett_rrc_T_tdd_56,
    &ett_rrc_T_tdd128_40,
    &ett_rrc_MIMO_PilotConfiguration,
    &ett_rrc_T_secondCPICH_Pattern,
    &ett_rrc_T_diversityPattern,
    &ett_rrc_MIMO_PilotConfiguration_v7f0ext,
    &ett_rrc_MIMO_PilotConfiguration_r9,
    &ett_rrc_T_secondCPICH_Pattern_01,
    &ett_rrc_T_diversityPattern_01,
    &ett_rrc_MIMO4x4_PilotConfiguration,
    &ett_rrc_Multi_frequencyInfo_LCR_r7,
    &ett_rrc_MultiflowConfigurationInfo,
    &ett_rrc_MU_MIMO_Info_TDD128,
    &ett_rrc_T_newConfiguration_07,
    &ett_rrc_NewTiming,
    &ett_rrc_NodeB_Trigger_HS_DPCCH_Transmission,
    &ett_rrc_Non_ScheduledTransGrantInfoTDD,
    &ett_rrc_T_tdd384_768,
    &ett_rrc_T_tdd128_41,
    &ett_rrc_T_e_HICH_Info_06,
    &ett_rrc_T_midambleAllocationMode_31,
    &ett_rrc_Non_ScheduledTransGrantInfoTDD_ext,
    &ett_rrc_T_tdd128_42,
    &ett_rrc_Non_ScheduledTransGrantInfoTDD_r9,
    &ett_rrc_T_tdd384_768_01,
    &ett_rrc_T_tdd128_43,
    &ett_rrc_T_e_HICH_Info_07,
    &ett_rrc_T_midambleAllocationMode_32,
    &ett_rrc_OpenLoopPowerControl_TDD,
    &ett_rrc_OpenLoopPowerControl_IPDL_TDD_r4,
    &ett_rrc_PCCH_InformationList,
    &ett_rrc_T_paging_associatedHspdschInfo,
    &ett_rrc_T_paging_associatedHspdschInfo_item,
    &ett_rrc_PCPCH_ChannelInfo,
    &ett_rrc_PCPCH_ChannelInfoList,
    &ett_rrc_PDSCH_CapacityAllocationInfo,
    &ett_rrc_T_configuration,
    &ett_rrc_T_old_Configuration,
    &ett_rrc_T_new_Configuration,
    &ett_rrc_PDSCH_CapacityAllocationInfo_r4,
    &ett_rrc_T_configuration_01,
    &ett_rrc_T_old_Configuration_01,
    &ett_rrc_T_new_Configuration_01,
    &ett_rrc_PDSCH_CapacityAllocationInfo_r7,
    &ett_rrc_T_configuration_02,
    &ett_rrc_T_old_Configuration_02,
    &ett_rrc_T_new_Configuration_02,
    &ett_rrc_PDSCH_CodeInfo,
    &ett_rrc_PDSCH_CodeInfoList,
    &ett_rrc_PDSCH_CodeMap,
    &ett_rrc_PDSCH_CodeMapList,
    &ett_rrc_PDSCH_CodeMapping,
    &ett_rrc_T_signallingMethod,
    &ett_rrc_PDSCH_Info,
    &ett_rrc_PDSCH_Info_r4,
    &ett_rrc_T_tddOption_12,
    &ett_rrc_T_tdd384_27,
    &ett_rrc_T_tdd128_44,
    &ett_rrc_PDSCH_Info_r7,
    &ett_rrc_T_tddOption_13,
    &ett_rrc_T_tdd384_28,
    &ett_rrc_T_tdd768_17,
    &ett_rrc_T_tdd128_45,
    &ett_rrc_PDSCH_Info_LCR_r4,
    &ett_rrc_PDSCH_PowerControlInfo,
    &ett_rrc_PDSCH_SHO_DCH_Info,
    &ett_rrc_PDSCH_SysInfo,
    &ett_rrc_PDSCH_SysInfo_VHCR_r7,
    &ett_rrc_PDSCH_SysInfo_HCR_r5,
    &ett_rrc_PDSCH_SysInfo_LCR_r4,
    &ett_rrc_PDSCH_SysInfoList,
    &ett_rrc_PDSCH_SysInfoList_VHCR_r7,
    &ett_rrc_PDSCH_SysInfoList_HCR_r5,
    &ett_rrc_PDSCH_SysInfoList_LCR_r4,
    &ett_rrc_PDSCH_SysInfoList_SFN,
    &ett_rrc_PDSCH_SysInfoList_SFN_item,
    &ett_rrc_PDSCH_SysInfoList_SFN_HCR_r5,
    &ett_rrc_PDSCH_SysInfoList_SFN_HCR_r5_item,
    &ett_rrc_PDSCH_SysInfoList_SFN_LCR_r4,
    &ett_rrc_PDSCH_SysInfoList_SFN_LCR_r4_item,
    &ett_rrc_PersistenceScalingFactorList,
    &ett_rrc_PichChannelisationCodeList_LCR_r4,
    &ett_rrc_PICH_ForHSDPASupportedPaging,
    &ett_rrc_PICH_ForHSDPASupportedPaging_TDD128,
    &ett_rrc_T_implicit_02,
    &ett_rrc_PICH_Info,
    &ett_rrc_T_fdd_102,
    &ett_rrc_T_tdd_57,
    &ett_rrc_PICH_Info_HCR_VHCR_r7,
    &ett_rrc_T_channelisationCode,
    &ett_rrc_PICH_Info_LCR_r4,
    &ett_rrc_PLCCH_Info,
    &ett_rrc_PowerControlAlgorithm,
    &ett_rrc_PRACH_ChanCodes_LCR_r4,
    &ett_rrc_PRACH_ChanCodes_List_LCR,
    &ett_rrc_PRACH_Definition_LCR_r4,
    &ett_rrc_PRACH_Information_LCR,
    &ett_rrc_PRACH_Information_LCR_List,
    &ett_rrc_PRACH_Partitioning,
    &ett_rrc_SEQUENCE_SIZE_1_maxASC_OF_ASCSetting_FDD,
    &ett_rrc_SEQUENCE_SIZE_1_maxASC_OF_ASCSetting_TDD,
    &ett_rrc_PRACH_Partitioning_r7,
    &ett_rrc_SEQUENCE_SIZE_1_maxASC_OF_ASCSetting_TDD_r7,
    &ett_rrc_PRACH_Partitioning_LCR_r4,
    &ett_rrc_PRACH_PowerOffset,
    &ett_rrc_PRACH_PreambleForEnhancedUplink,
    &ett_rrc_PRACH_PreambleForEnhancedUplinkExt,
    &ett_rrc_PRACH_PreambleForEnhancedUplinkExtList,
    &ett_rrc_SEQUENCE_SIZE_1_maxPRACH_EUL_OF_PRACH_PreambleForEnhancedUplinkExtWithWeight,
    &ett_rrc_PRACH_PreambleForEnhancedUplinkExtWithWeight,
    &ett_rrc_PRACH_RACH_Info,
    &ett_rrc_T_modeSpecificInfo_72,
    &ett_rrc_T_fdd_103,
    &ett_rrc_T_tdd_58,
    &ett_rrc_PRACH_RACH_Info_VHCR_r7,
    &ett_rrc_PRACH_RACH_Info_LCR_v770ext,
    &ett_rrc_PRACH_RACH_Info_LCR_r4,
    &ett_rrc_SEQUENCE_SIZE_1_maxPRACH_FPACH_OF_PRACH_Definition_LCR_r4,
    &ett_rrc_PRACH_SystemInformation,
    &ett_rrc_T_modeSpecificInfo_73,
    &ett_rrc_T_fdd_104,
    &ett_rrc_PRACH_SystemInformation_VHCR_r7,
    &ett_rrc_PRACH_SystemInformation_LCR_r4,
    &ett_rrc_PRACH_SystemInformation_LCR_v770ext,
    &ett_rrc_PRACH_SystemInformationList,
    &ett_rrc_PRACH_SystemInformationList_VHCR_r7,
    &ett_rrc_PRACH_SystemInformationList_LCR_r4,
    &ett_rrc_PRACH_SystemInformationList_LCR_v770ext,
    &ett_rrc_PreDefPhyChConfiguration,
    &ett_rrc_PreDefPhyChConfiguration_v770ext,
    &ett_rrc_PrimaryCCPCH_Info,
    &ett_rrc_T_fdd_105,
    &ett_rrc_T_tdd_59,
    &ett_rrc_T_syncCase,
    &ett_rrc_T_syncCase1,
    &ett_rrc_T_syncCase2,
    &ett_rrc_PrimaryCCPCH_Info_r4,
    &ett_rrc_T_fdd_106,
    &ett_rrc_T_tdd_60,
    &ett_rrc_T_tddOption_14,
    &ett_rrc_T_tdd384_tdd768_04,
    &ett_rrc_T_syncCase_01,
    &ett_rrc_T_syncCase1_01,
    &ett_rrc_T_syncCase2_01,
    &ett_rrc_T_tdd128_46,
    &ett_rrc_PrimaryCCPCH_Info_LCR_r4,
    &ett_rrc_PrimaryCCPCH_Info_LCR_r4_ext,
    &ett_rrc_PrimaryCCPCH_InfoPost,
    &ett_rrc_T_syncCase_02,
    &ett_rrc_T_syncCase1_02,
    &ett_rrc_T_syncCase2_02,
    &ett_rrc_PrimaryCCPCH_InfoPostTDD_LCR_r4,
    &ett_rrc_PrimaryCPICH_Info,
    &ett_rrc_PUSCH_CapacityAllocationInfo,
    &ett_rrc_T_pusch_Allocation,
    &ett_rrc_T_pusch_AllocationAssignment,
    &ett_rrc_T_configuration_03,
    &ett_rrc_T_old_Configuration_03,
    &ett_rrc_T_new_Configuration_03,
    &ett_rrc_PUSCH_CapacityAllocationInfo_r4,
    &ett_rrc_T_pusch_Allocation_01,
    &ett_rrc_T_pusch_AllocationAssignment_01,
    &ett_rrc_T_configuration_04,
    &ett_rrc_T_old_Configuration_04,
    &ett_rrc_T_new_Configuration_04,
    &ett_rrc_PUSCH_CapacityAllocationInfo_r7,
    &ett_rrc_T_pusch_Allocation_02,
    &ett_rrc_T_pusch_AllocationAssignment_02,
    &ett_rrc_T_configuration_05,
    &ett_rrc_T_old_Configuration_05,
    &ett_rrc_T_new_Configuration_05,
    &ett_rrc_PUSCH_Info,
    &ett_rrc_PUSCH_Info_r4,
    &ett_rrc_T_tddOption_15,
    &ett_rrc_T_tdd384_29,
    &ett_rrc_T_tdd128_47,
    &ett_rrc_PUSCH_Info_VHCR,
    &ett_rrc_PUSCH_Info_LCR_r4,
    &ett_rrc_PUSCH_PowerControlInfo_r4,
    &ett_rrc_T_tddOption_16,
    &ett_rrc_T_tdd128_48,
    &ett_rrc_PUSCH_PowerControlInfo_r7,
    &ett_rrc_T_tddOption_17,
    &ett_rrc_T_tdd128_49,
    &ett_rrc_PUSCH_SysInfo,
    &ett_rrc_PUSCH_SysInfo_VHCR,
    &ett_rrc_PUSCH_SysInfo_HCR_r5,
    &ett_rrc_PUSCH_SysInfo_LCR_r4,
    &ett_rrc_PUSCH_SysInfoList,
    &ett_rrc_PUSCH_SysInfoList_HCR_r5,
    &ett_rrc_PUSCH_SysInfoList_LCR_r4,
    &ett_rrc_PUSCH_SysInfoList_SFN,
    &ett_rrc_PUSCH_SysInfoList_SFN_item,
    &ett_rrc_PUSCH_SysInfoList_SFN_HCR_r5,
    &ett_rrc_PUSCH_SysInfoList_SFN_HCR_r5_item,
    &ett_rrc_PUSCH_SysInfoList_SFN_LCR_r4,
    &ett_rrc_PUSCH_SysInfoList_SFN_LCR_r4_item,
    &ett_rrc_PUSCH_SysInfoList_SFN_VHCR,
    &ett_rrc_PUSCH_SysInfoList_SFN_VHCR_item,
    &ett_rrc_RACH_TransmissionParameters,
    &ett_rrc_Reference_Beta_QPSK,
    &ett_rrc_Reference_Beta_16QAM,
    &ett_rrc_RepetitionPeriodAndLength,
    &ett_rrc_RepetitionPeriodLengthAndOffset,
    &ett_rrc_T_repetitionPeriod2,
    &ett_rrc_T_repetitionPeriod4,
    &ett_rrc_T_repetitionPeriod8,
    &ett_rrc_T_repetitionPeriod16,
    &ett_rrc_T_repetitionPeriod32,
    &ett_rrc_T_repetitionPeriod64,
    &ett_rrc_RepetitionPeriodAndLengthForSPS,
    &ett_rrc_ReplacedPDSCH_CodeInfo,
    &ett_rrc_ReplacedPDSCH_CodeInfoList,
    &ett_rrc_RepPerLengthOffset_PICH,
    &ett_rrc_RepPerLengthOffset_MICH,
    &ett_rrc_RestrictedTrCH,
    &ett_rrc_RestrictedTrCH_InfoList,
    &ett_rrc_RL_AdditionInformation,
    &ett_rrc_RL_AdditionInformation_r6,
    &ett_rrc_T_dl_dpchInfo_04,
    &ett_rrc_RL_AdditionInformation_r7,
    &ett_rrc_T_dl_dpchInfo_05,
    &ett_rrc_RL_AdditionInformation_r8,
    &ett_rrc_T_dl_dpchInfo_06,
    &ett_rrc_RL_AdditionInformation_r9,
    &ett_rrc_T_dl_dpchInfo_07,
    &ett_rrc_RL_AdditionInformation_r10,
    &ett_rrc_T_dl_dpchInfo_08,
    &ett_rrc_RL_AdditionInformation_vbxyext,
    &ett_rrc_RL_AdditionInformation_r11,
    &ett_rrc_T_dl_dpchInfo_09,
    &ett_rrc_RL_AdditionInformation_v6b0ext,
    &ett_rrc_RL_AdditionInformation_v890ext,
    &ett_rrc_RL_AdditionInformation_v950ext,
    &ett_rrc_RL_AdditionInformationList,
    &ett_rrc_RL_AdditionInformationList_r6,
    &ett_rrc_RL_AdditionInformationList_r7,
    &ett_rrc_RL_AdditionInformation_list_v6b0ext,
    &ett_rrc_RL_AdditionInformationList_r8,
    &ett_rrc_RL_AdditionInformationList_v890ext,
    &ett_rrc_RL_AdditionInformationList_r9,
    &ett_rrc_RL_AdditionInformationList_v950ext,
    &ett_rrc_RL_AdditionInformationList_r10,
    &ett_rrc_RL_AdditionInformationList_vbxyext,
    &ett_rrc_RL_AdditionInformationList_r11,
    &ett_rrc_RL_AdditionInformationList_SecULFreq,
    &ett_rrc_RL_AdditionInformation_SecULFreq,
    &ett_rrc_RL_IdentifierList,
    &ett_rrc_RL_RemovalInformationList,
    &ett_rrc_RL_RemovalInformationList_SecULFreq,
    &ett_rrc_SCCPCH_ChannelisationCodeList,
    &ett_rrc_SCCPCH_ChannelisationCodeList_VHCR,
    &ett_rrc_SCCPCH_InfoForFACH,
    &ett_rrc_T_modeSpecificInfo_74,
    &ett_rrc_T_fdd_107,
    &ett_rrc_T_tdd_61,
    &ett_rrc_SCCPCH_InfoForFACH_r4,
    &ett_rrc_T_modeSpecificInfo_75,
    &ett_rrc_T_fdd_108,
    &ett_rrc_SCCPCH_SystemInformation,
    &ett_rrc_SCCPCH_SystemInformation_LCR_r4_ext,
    &ett_rrc_SCCPCH_SystemInformation_HCR_VHCR_r7,
    &ett_rrc_SCCPCH_SystemInformationList,
    &ett_rrc_SCCPCH_SystemInformationList_HCR_VHCR_r7,
    &ett_rrc_SCCPCH_SystemInformationList_LCR_r4_ext,
    &ett_rrc_SCCPCH_SystemInformation_MBMS_r6,
    &ett_rrc_T_fachCarryingMCCH,
    &ett_rrc_T_fachCarryingMSCH,
    &ett_rrc_SCCPCH_SystemInformation_MBMS_r7,
    &ett_rrc_T_fachCarryingMCCH_01,
    &ett_rrc_T_fachCarryingMSCH_01,
    &ett_rrc_ScheduledTransmissionConfiguration,
    &ett_rrc_SEQUENCE_SIZE_1_maxEDCHs_OF_Common_E_DCH_ResourceInfoListExt,
    &ett_rrc_SecondaryCCPCH_Info,
    &ett_rrc_T_modeSpecificInfo_76,
    &ett_rrc_T_fdd_109,
    &ett_rrc_T_tdd_62,
    &ett_rrc_SecondaryCCPCH_Info_r4,
    &ett_rrc_T_modeSpecificInfo_77,
    &ett_rrc_T_fdd_110,
    &ett_rrc_T_tdd_63,
    &ett_rrc_T_tddOption_18,
    &ett_rrc_T_tdd384_30,
    &ett_rrc_T_tdd128_50,
    &ett_rrc_SecondaryCCPCH_Info_HCR_VHCR_r7,
    &ett_rrc_T_modeSpecificInfo_78,
    &ett_rrc_T_tdd384_31,
    &ett_rrc_T_tdd768_18,
    &ett_rrc_SecondaryCCPCH_Info_LCR_r4_ext,
    &ett_rrc_SecondaryCCPCHFrameType2Info,
    &ett_rrc_T_modulation_03,
    &ett_rrc_T_mod16QAM,
    &ett_rrc_SecondaryCCPCHInfo_MBMS_r6,
    &ett_rrc_T_modeSpecificInfo_79,
    &ett_rrc_T_fdd_111,
    &ett_rrc_T_tdd384_32,
    &ett_rrc_T_tdd128_51,
    &ett_rrc_SecondaryCCPCHInfo_MBMS_r7,
    &ett_rrc_T_modeSpecificInfo_80,
    &ett_rrc_T_fdd_112,
    &ett_rrc_T_modulation_04,
    &ett_rrc_T_tdd384_33,
    &ett_rrc_T_tdd768_19,
    &ett_rrc_T_tdd128_52,
    &ett_rrc_SecondaryCCPCHInfoDiff_MBMS,
    &ett_rrc_SecondaryCPICH_Info,
    &ett_rrc_SecondaryCellMIMOparametersFDD,
    &ett_rrc_T_newConfiguration_08,
    &ett_rrc_SecondaryCellMIMOparametersFDD_v950ext,
    &ett_rrc_SecondaryCellMIMOparametersFDD_r10,
    &ett_rrc_T_newConfiguration_09,
    &ett_rrc_SecondaryCell4x4MIMOparametersFDD,
    &ett_rrc_T_newConfiguration_10,
    &ett_rrc_ServingCellChangeParameters,
    &ett_rrc_SF256_AndCodeNumber,
    &ett_rrc_SF512_AndCodeNumber,
    &ett_rrc_SF512_AndPilot,
    &ett_rrc_SFN_TimeInfo,
    &ett_rrc_SPS_Information_TDD128_r8,
    &ett_rrc_SSDT_Information,
    &ett_rrc_SSDT_Information_r4,
    &ett_rrc_StandaloneMidambleInfo_TDD128,
    &ett_rrc_SynchronisationParameters_r4,
    &ett_rrc_T_sync_UL_CodesBitmap,
    &ett_rrc_Sync_UL_Codes_Bitmap,
    &ett_rrc_SYNC_UL_Procedure_r4,
    &ett_rrc_SYNC_UL_Info_r4,
    &ett_rrc_SYNC_UL_InfoForE_RUCCH,
    &ett_rrc_TargetCellPreconfigInfo,
    &ett_rrc_TargetCellPreconfigInfo_r9,
    &ett_rrc_TargetCellPreconfigInfo_r10,
    &ett_rrc_TargetCellPreconfigInfo_vbxyext,
    &ett_rrc_TargetCellPreconfigInfo_r11,
    &ett_rrc_TDD_MBSFNInformation,
    &ett_rrc_TDD_MBSFNTSlotInfo,
    &ett_rrc_TDD_PRACH_CCodeList,
    &ett_rrc_SEQUENCE_SIZE_1_8_OF_TDD_PRACH_CCode8,
    &ett_rrc_SEQUENCE_SIZE_1_8_OF_TDD_PRACH_CCode16,
    &ett_rrc_TDD768_PRACH_CCodeList,
    &ett_rrc_SEQUENCE_SIZE_1_16_OF_TDD768_PRACH_CCode32,
    &ett_rrc_SEQUENCE_SIZE_1_16_OF_TDD768_PRACH_CCode16,
    &ett_rrc_TGP_Sequence,
    &ett_rrc_T_tgps_Status,
    &ett_rrc_T_activate,
    &ett_rrc_TGP_Sequence_r8,
    &ett_rrc_T_tgps_Status_01,
    &ett_rrc_T_activate_01,
    &ett_rrc_TGP_Sequence_r10,
    &ett_rrc_T_tgps_Status_02,
    &ett_rrc_T_activate_02,
    &ett_rrc_TGP_SequenceList,
    &ett_rrc_TGP_SequenceList_r8,
    &ett_rrc_TGP_SequenceList_r10,
    &ett_rrc_TGP_SequenceShort,
    &ett_rrc_T_tgps_Status_03,
    &ett_rrc_T_activate_03,
    &ett_rrc_TGP_SequenceShort_r10,
    &ett_rrc_T_tgps_Status_04,
    &ett_rrc_T_activate_04,
    &ett_rrc_TGPS_ConfigurationParams,
    &ett_rrc_TGPS_ConfigurationParams_r8,
    &ett_rrc_TGPS_ConfigurationParams_r10,
    &ett_rrc_TimeInfo,
    &ett_rrc_TimeslotList,
    &ett_rrc_TimeslotList_r4,
    &ett_rrc_SEQUENCE_SIZE_1_maxTS_OF_TimeslotNumber,
    &ett_rrc_SEQUENCE_SIZE_1_maxTS_LCR_OF_TimeslotNumber_LCR_r4,
    &ett_rrc_DTX_DRX_TimingInfo_r7,
    &ett_rrc_T_timing,
    &ett_rrc_TwoMsHarqConfiguration,
    &ett_rrc_UCSM_Info,
    &ett_rrc_UL_16QAM_Config,
    &ett_rrc_UL_16QAM_Settings,
    &ett_rrc_UL_64QAM_Config,
    &ett_rrc_UL_64QAM_Settings,
    &ett_rrc_UL_CCTrCH,
    &ett_rrc_UL_CCTrCH_r4,
    &ett_rrc_T_tddOption_19,
    &ett_rrc_T_tdd384_34,
    &ett_rrc_T_tdd128_53,
    &ett_rrc_UL_CCTrCH_r7,
    &ett_rrc_T_tddOption_20,
    &ett_rrc_T_tdd384_35,
    &ett_rrc_T_tdd768_20,
    &ett_rrc_T_tdd128_54,
    &ett_rrc_UL_CCTrCHList,
    &ett_rrc_UL_CCTrCHList_r4,
    &ett_rrc_UL_CCTrCHList_r7,
    &ett_rrc_UL_CCTrCHListToRemove,
    &ett_rrc_UL_CCTrChTPCList,
    &ett_rrc_UL_ChannelRequirement,
    &ett_rrc_UL_ChannelRequirement_r4,
    &ett_rrc_UL_ChannelRequirement_r5,
    &ett_rrc_UL_ChannelRequirementWithCPCH_SetID,
    &ett_rrc_UL_ChannelRequirementWithCPCH_SetID_r4,
    &ett_rrc_UL_ChannelRequirementWithCPCH_SetID_r5,
    &ett_rrc_UL_CLTD_InfoFDD,
    &ett_rrc_T_configurationInfo_04,
    &ett_rrc_T_newConfiguration_11,
    &ett_rrc_UL_MIMO_InfoFDD,
    &ett_rrc_T_configurationInfo_05,
    &ett_rrc_T_newConfiguration_12,
    &ett_rrc_UL_DL_Mode,
    &ett_rrc_T_ul_and_dl,
    &ett_rrc_UL_DPCH_CodeInfoForCommonEDCH,
    &ett_rrc_UL_DPCH_Info,
    &ett_rrc_T_modeSpecificInfo_81,
    &ett_rrc_T_fdd_113,
    &ett_rrc_T_tdd_64,
    &ett_rrc_UL_DPCH_Info_r4,
    &ett_rrc_T_modeSpecificInfo_82,
    &ett_rrc_T_fdd_114,
    &ett_rrc_T_tdd_65,
    &ett_rrc_UL_DPCH_Info_r5,
    &ett_rrc_T_modeSpecificInfo_83,
    &ett_rrc_T_fdd_115,
    &ett_rrc_T_tdd_66,
    &ett_rrc_UL_DPCH_Info_r6,
    &ett_rrc_T_modeSpecificInfo_84,
    &ett_rrc_T_fdd_116,
    &ett_rrc_T_dpdchPresence,
    &ett_rrc_T_present_02,
    &ett_rrc_T_notPresent,
    &ett_rrc_T_tdd_67,
    &ett_rrc_UL_DPCH_Info_r7,
    &ett_rrc_T_modeSpecificInfo_85,
    &ett_rrc_T_fdd_117,
    &ett_rrc_T_dpdchPresence_01,
    &ett_rrc_T_present_03,
    &ett_rrc_T_notPresent_01,
    &ett_rrc_T_tdd_68,
    &ett_rrc_UL_DPCH_Info_r11,
    &ett_rrc_T_modeSpecificInfo_86,
    &ett_rrc_T_fdd_118,
    &ett_rrc_T_dpdchPresence_02,
    &ett_rrc_T_present_04,
    &ett_rrc_T_notPresent_02,
    &ett_rrc_T_tdd_69,
    &ett_rrc_UL_DPCH_InfoPostFDD,
    &ett_rrc_UL_DPCH_InfoPostTDD,
    &ett_rrc_UL_DPCH_InfoPostTDD_LCR_r4,
    &ett_rrc_UL_DPCH_InfoPredef,
    &ett_rrc_T_modeSpecificInfo_87,
    &ett_rrc_T_fdd_119,
    &ett_rrc_T_tdd_70,
    &ett_rrc_UL_DPCH_InfoPredef_v770ext,
    &ett_rrc_T_modeSpecificInfo_88,
    &ett_rrc_T_fdd_120,
    &ett_rrc_UL_DPCH_PowerControlInfo,
    &ett_rrc_T_fdd_121,
    &ett_rrc_T_tdd_71,
    &ett_rrc_T_ul_OL_PC_Signalling,
    &ett_rrc_T_individuallySignalled,
    &ett_rrc_UL_DPCH_PowerControlInfo_r4,
    &ett_rrc_T_fdd_122,
    &ett_rrc_T_tdd_72,
    &ett_rrc_T_ul_OL_PC_Signalling_01,
    &ett_rrc_T_individuallySignalled_01,
    &ett_rrc_T_tddOption_21,
    &ett_rrc_T_tdd384_36,
    &ett_rrc_T_tdd128_55,
    &ett_rrc_UL_DPCH_PowerControlInfo_r5,
    &ett_rrc_T_fdd_123,
    &ett_rrc_T_tdd_73,
    &ett_rrc_T_ul_OL_PC_Signalling_02,
    &ett_rrc_T_individuallySignalled_02,
    &ett_rrc_T_tddOption_22,
    &ett_rrc_T_tdd384_37,
    &ett_rrc_T_tdd128_56,
    &ett_rrc_UL_DPCH_PowerControlInfo_r6,
    &ett_rrc_T_fdd_124,
    &ett_rrc_T_tdd_74,
    &ett_rrc_T_ul_OL_PC_Signalling_03,
    &ett_rrc_T_individuallySignalled_03,
    &ett_rrc_T_tddOption_23,
    &ett_rrc_T_tdd384_38,
    &ett_rrc_T_tdd128_57,
    &ett_rrc_UL_DPCH_PowerControlInfo_r7,
    &ett_rrc_T_fdd_125,
    &ett_rrc_T_tdd_75,
    &ett_rrc_T_ul_OL_PC_Signalling_04,
    &ett_rrc_T_individuallySignalled_04,
    &ett_rrc_T_tddOption_24,
    &ett_rrc_T_tdd384_39,
    &ett_rrc_T_tdd768_21,
    &ett_rrc_T_tdd128_58,
    &ett_rrc_UL_DPCH_PowerControlInfo_r11,
    &ett_rrc_T_fdd_126,
    &ett_rrc_T_tdd_76,
    &ett_rrc_T_ul_OL_PC_Signalling_05,
    &ett_rrc_T_individuallySignalled_05,
    &ett_rrc_T_tddOption_25,
    &ett_rrc_T_tdd384_40,
    &ett_rrc_T_tdd768_22,
    &ett_rrc_T_tdd128_59,
    &ett_rrc_UL_DPCHpowerControlInfoConcurrentTTI,
    &ett_rrc_UL_DPCHpowerControlInfoForCommonEDCH,
    &ett_rrc_UL_DPCH_PowerControlInfoPostFDD,
    &ett_rrc_UL_DPCH_PowerControlInfoPostTDD,
    &ett_rrc_UL_DPCH_PowerControlInfoPostTDD_LCR_r4,
    &ett_rrc_UL_DPCH_PowerControlInfoPredef,
    &ett_rrc_T_fdd_127,
    &ett_rrc_T_tdd_77,
    &ett_rrc_UL_EDCH_Information_ext,
    &ett_rrc_T_tdd_78,
    &ett_rrc_UL_EDCH_Information_r6,
    &ett_rrc_UL_EDCH_Information_r7,
    &ett_rrc_T_modeSpecificInfo_89,
    &ett_rrc_T_fdd_128,
    &ett_rrc_T_tdd_79,
    &ett_rrc_UL_EDCH_Information_r8,
    &ett_rrc_T_modeSpecificInfo_90,
    &ett_rrc_T_fdd_129,
    &ett_rrc_T_tdd_80,
    &ett_rrc_UL_EDCH_Information_r9,
    &ett_rrc_T_modeSpecificInfo_91,
    &ett_rrc_T_fdd_130,
    &ett_rrc_T_tdd_81,
    &ett_rrc_UL_EDCH_Information_r11,
    &ett_rrc_T_modeSpecificInfo_92,
    &ett_rrc_T_fdd_131,
    &ett_rrc_T_tdd_82,
    &ett_rrc_UL_MulticarrierEDCH_InfolistItem_TDD128,
    &ett_rrc_UL_MulticarrierEDCH_Infolist_TDD128,
    &ett_rrc_UL_MulticarrierEDCH_Deletelist_TDD128,
    &ett_rrc_UL_MulticarrierEDCHInfo_TDD128,
    &ett_rrc_T_configuration_info,
    &ett_rrc_UL_MulticarrierEDCH_NewConfigurationInfo_TDD128,
    &ett_rrc_UL_SynchronisationParameters_r4,
    &ett_rrc_UL_TimingAdvanceControl,
    &ett_rrc_T_enabled,
    &ett_rrc_UL_TimingAdvanceControl_r4,
    &ett_rrc_T_enabled_01,
    &ett_rrc_T_tddOption_26,
    &ett_rrc_T_tdd384_41,
    &ett_rrc_T_tdd128_60,
    &ett_rrc_UL_TimingAdvanceControl_r7,
    &ett_rrc_T_enabled_02,
    &ett_rrc_T_tddOption_27,
    &ett_rrc_T_tdd384_42,
    &ett_rrc_T_tdd768_23,
    &ett_rrc_T_tdd128_61,
    &ett_rrc_UL_TimingAdvanceControl_LCR_r4,
    &ett_rrc_T_enabled_03,
    &ett_rrc_UL_TS_ChannelisationCodeList,
    &ett_rrc_UL_TS_ChannelisationCodeList_r7,
    &ett_rrc_UL_TS_ChannelisationCodeList_r7_item,
    &ett_rrc_UL_TS_ChannelisationCodeList_VHCR,
    &ett_rrc_UplinkAdditionalTimeslots,
    &ett_rrc_T_parameters_04,
    &ett_rrc_T_sameAsLast_04,
    &ett_rrc_T_newParameters_04,
    &ett_rrc_UplinkAdditionalTimeslots_LCR_r4,
    &ett_rrc_T_parameters_05,
    &ett_rrc_T_sameAsLast_05,
    &ett_rrc_T_newParameters_05,
    &ett_rrc_UplinkAdditionalTimeslots_LCR_r7,
    &ett_rrc_T_parameters_06,
    &ett_rrc_T_sameAsLast_06,
    &ett_rrc_T_newParameters_06,
    &ett_rrc_UplinkAdditionalTimeslots_VHCR,
    &ett_rrc_T_parameters_07,
    &ett_rrc_T_sameAsLast_07,
    &ett_rrc_T_newParameters_07,
    &ett_rrc_UplinkTimeslotsCodes,
    &ett_rrc_T_moreTimeslots_04,
    &ett_rrc_T_additionalTimeslots_04,
    &ett_rrc_T_consecutive_02,
    &ett_rrc_SEQUENCE_SIZE_1_maxTS_1_OF_UplinkAdditionalTimeslots,
    &ett_rrc_UplinkTimeslotsCodes_VHCR,
    &ett_rrc_T_moreTimeslots_05,
    &ett_rrc_T_additionalTimeslots_05,
    &ett_rrc_T_consecutive_03,
    &ett_rrc_SEQUENCE_SIZE_1_maxTS_1_OF_UplinkAdditionalTimeslots_VHCR,
    &ett_rrc_UplinkTimeslotsCodes_LCR_r4,
    &ett_rrc_T_moreTimeslots_06,
    &ett_rrc_T_additionalTimeslots_06,
    &ett_rrc_T_consecutive_04,
    &ett_rrc_SEQUENCE_SIZE_1_maxTS_LCR_1_OF_UplinkAdditionalTimeslots_LCR_r4,
    &ett_rrc_UplinkTimeslotsCodes_LCR_r7,
    &ett_rrc_T_moreTimeslots_07,
    &ett_rrc_T_additionalTimeslots_07,
    &ett_rrc_T_consecutive_05,
    &ett_rrc_SEQUENCE_SIZE_1_maxTS_LCR_1_OF_UplinkAdditionalTimeslots_LCR_r7,
    &ett_rrc_AcquisitionSatInfo,
    &ett_rrc_AcquisitionSatInfo_va40ext,
    &ett_rrc_AcquisitionSatInfo_r10,
    &ett_rrc_AcquisitionSatInfoList,
    &ett_rrc_AcquisitionSatInfoList_va40ext,
    &ett_rrc_AcquisitionSatInfoList_r10,
    &ett_rrc_AdditionalMeasurementID_List,
    &ett_rrc_AdditionalMeasurementID_List_r9,
    &ett_rrc_AlmanacSatInfo,
    &ett_rrc_AlmanacSatInfoList,
    &ett_rrc_ALM_ECEFsbasAlmanacSet,
    &ett_rrc_ALM_GlonassAlmanacSet,
    &ett_rrc_ALM_keplerianParameters,
    &ett_rrc_ALM_MidiAlmanacSet,
    &ett_rrc_ALM_NAVKeplerianSet,
    &ett_rrc_ALM_ReducedKeplerianSet,
    &ett_rrc_AuxInfoGANSS_ID1,
    &ett_rrc_AuxInfoGANSS_ID1_element,
    &ett_rrc_AuxInfoGANSS_ID3,
    &ett_rrc_AuxInfoGANSS_ID3_element,
    &ett_rrc_AzimuthAndElevation,
    &ett_rrc_AzimuthAndElevation_va40ext,
    &ett_rrc_AzimuthAndElevation_r10,
    &ett_rrc_BadSatList,
    &ett_rrc_BLER_MeasurementResults,
    &ett_rrc_BLER_MeasurementResultsList,
    &ett_rrc_BLER_TransChIdList,
    &ett_rrc_BSICReported,
    &ett_rrc_BurstModeParameters,
    &ett_rrc_CellDCH_ReportCriteria,
    &ett_rrc_CellDCH_ReportCriteria_LCR_r4,
    &ett_rrc_CellDCHMeasOccasionPattern_LCR,
    &ett_rrc_CellDCHMeasOccasionInfo_TDD128_r9,
    &ett_rrc_SEQUENCE_SIZE_1_maxMeasOccasionPattern_OF_CellDCHMeasOccasionPattern_LCR,
    &ett_rrc_CellInfo,
    &ett_rrc_T_modeSpecificInfo_93,
    &ett_rrc_T_fdd_132,
    &ett_rrc_T_tdd_83,
    &ett_rrc_CellInfo_r4,
    &ett_rrc_T_modeSpecificInfo_94,
    &ett_rrc_T_fdd_133,
    &ett_rrc_T_tdd_84,
    &ett_rrc_CellInfo_LCR_r8_ext,
    &ett_rrc_CellInfo_r9,
    &ett_rrc_T_modeSpecificInfo_95,
    &ett_rrc_T_fdd_134,
    &ett_rrc_T_tdd_85,
    &ett_rrc_CellInfoListToBeExcluded_r11,
    &ett_rrc_CellInfoSI_RSCP,
    &ett_rrc_T_modeSpecificInfo_96,
    &ett_rrc_T_fdd_135,
    &ett_rrc_T_tdd_86,
    &ett_rrc_CellInfoSI_RSCP_LCR_r4,
    &ett_rrc_CellInfoSI_ECN0,
    &ett_rrc_T_modeSpecificInfo_97,
    &ett_rrc_T_fdd_136,
    &ett_rrc_T_tdd_87,
    &ett_rrc_CellInfoSI_ECN0_LCR_r4,
    &ett_rrc_CellInfoSI_HCS_RSCP,
    &ett_rrc_T_modeSpecificInfo_98,
    &ett_rrc_T_fdd_137,
    &ett_rrc_T_tdd_88,
    &ett_rrc_CellInfoSI_HCS_RSCP_LCR_r4,
    &ett_rrc_CellInfoSI_HCS_ECN0,
    &ett_rrc_T_modeSpecificInfo_99,
    &ett_rrc_T_fdd_138,
    &ett_rrc_T_tdd_89,
    &ett_rrc_CellInfoSI_HCS_ECN0_LCR_r4,
    &ett_rrc_CellMeasuredResults,
    &ett_rrc_T_modeSpecificInfo_100,
    &ett_rrc_T_fdd_139,
    &ett_rrc_T_tdd_90,
    &ett_rrc_CellMeasuredResults_r9,
    &ett_rrc_T_modeSpecificInfo_101,
    &ett_rrc_T_fdd_140,
    &ett_rrc_T_tdd_91,
    &ett_rrc_CellMeasuredResults_v920ext,
    &ett_rrc_CellMeasurementEventResults,
    &ett_rrc_SEQUENCE_SIZE_1_maxCellMeas_OF_PrimaryCPICH_Info,
    &ett_rrc_SEQUENCE_SIZE_1_maxCellMeas_OF_PrimaryCCPCH_Info,
    &ett_rrc_CellMeasurementEventResultsOnSecUlFreq,
    &ett_rrc_SEQUENCE_SIZE_1_maxCellMeasOnSecULFreq_OF_PrimaryCPICH_Info,
    &ett_rrc_CellMeasurementEventResults_LCR_r4,
    &ett_rrc_CellReportingQuantities,
    &ett_rrc_T_modeSpecificInfo_102,
    &ett_rrc_T_fdd_141,
    &ett_rrc_T_tdd_92,
    &ett_rrc_CellSelectReselectInfoMC_RSCP,
    &ett_rrc_T_modeSpecificInfo_103,
    &ett_rrc_T_fdd_142,
    &ett_rrc_T_tdd_93,
    &ett_rrc_T_gsm_02,
    &ett_rrc_CellSelectReselectInfoSIB_11_12,
    &ett_rrc_T_modeSpecificInfo_104,
    &ett_rrc_T_fdd_143,
    &ett_rrc_T_tdd_94,
    &ett_rrc_T_gsm_03,
    &ett_rrc_CellSelectReselectInfoSIB_11_12_RSCP,
    &ett_rrc_T_modeSpecificInfo_105,
    &ett_rrc_T_fdd_144,
    &ett_rrc_T_tdd_95,
    &ett_rrc_T_gsm_04,
    &ett_rrc_CellSelectReselectInfoSIB_11_12_ECN0,
    &ett_rrc_T_modeSpecificInfo_106,
    &ett_rrc_T_fdd_145,
    &ett_rrc_T_tdd_96,
    &ett_rrc_T_gsm_05,
    &ett_rrc_CellSelectReselectInfoSIB_11_12_HCS_RSCP,
    &ett_rrc_T_modeSpecificInfo_107,
    &ett_rrc_T_fdd_146,
    &ett_rrc_T_tdd_97,
    &ett_rrc_T_gsm_06,
    &ett_rrc_CellSelectReselectInfoSIB_11_12_HCS_ECN0,
    &ett_rrc_T_modeSpecificInfo_108,
    &ett_rrc_T_fdd_147,
    &ett_rrc_T_tdd_98,
    &ett_rrc_T_gsm_07,
    &ett_rrc_CellSelectReselectInfo_v590ext,
    &ett_rrc_CellSelectReselectInfoPCHFACH_v5b0ext,
    &ett_rrc_CellSelectReselectInfoTreselectionScaling_v5c0ext,
    &ett_rrc_CellsForInterFreqMeasList,
    &ett_rrc_CellsForInterRATMeasList,
    &ett_rrc_CellsForIntraFreqMeasList,
    &ett_rrc_CellsForIntraFreqMeasListOnSecULFreq,
    &ett_rrc_CellSynchronisationInfo,
    &ett_rrc_T_modeSpecificInfo_109,
    &ett_rrc_T_fdd_148,
    &ett_rrc_T_tdd_99,
    &ett_rrc_CellToReport,
    &ett_rrc_CellToReportList,
    &ett_rrc_CNAVclockModel,
    &ett_rrc_CountC_SFN_Frame_difference,
    &ett_rrc_CSGCellInfo,
    &ett_rrc_T_modeSpecificInfo_110,
    &ett_rrc_T_fdd_149,
    &ett_rrc_CSGCellInfoList,
    &ett_rrc_CSGInterFreqCellInfo,
    &ett_rrc_CSGInterFreqCellInfoList,
    &ett_rrc_CSGProximityDetection,
    &ett_rrc_CSGProximityIndication,
    &ett_rrc_T_radioAccessTechnology_01,
    &ett_rrc_T_uTRA,
    &ett_rrc_T_e_UTRA,
    &ett_rrc_DataBitAssistance,
    &ett_rrc_DataBitAssistanceList,
    &ett_rrc_DataBitAssistanceSat,
    &ett_rrc_DataBitAssistanceSgnList,
    &ett_rrc_DeltaRSCPPerCell,
    &ett_rrc_DeltaUT1,
    &ett_rrc_DGANSSInfo,
    &ett_rrc_DGANSSInfo_r9,
    &ett_rrc_DGANSSInfo_v920ext,
    &ett_rrc_DGANSSInfoList,
    &ett_rrc_DGANSSInfoList_r9,
    &ett_rrc_DGANSSInfoList_v920ext,
    &ett_rrc_DGANSSSignalInformation,
    &ett_rrc_DGANSSSignalInformation_r9,
    &ett_rrc_DGANSSSignalInformation_v920ext,
    &ett_rrc_DGANSSSignalInformationList,
    &ett_rrc_DGANSSSignalInformationList_r9,
    &ett_rrc_DGANSSSignalInformationList_v920ext,
    &ett_rrc_DGPS_CorrectionSatInfo,
    &ett_rrc_DGPS_CorrectionSatInfo_r9,
    &ett_rrc_DGPS_CorrectionSatInfo_v920ext,
    &ett_rrc_DGPS_CorrectionSatInfoList,
    &ett_rrc_DGPS_CorrectionSatInfoList_r9,
    &ett_rrc_DGPS_CorrectionSatInfoList_v920ext,
    &ett_rrc_EllipsoidPoint,
    &ett_rrc_EllipsoidPointAltitude,
    &ett_rrc_EllipsoidPointAltitudeEllipsoide,
    &ett_rrc_EllipsoidPointUncertCircle,
    &ett_rrc_EllipsoidPointUncertEllipse,
    &ett_rrc_Eutra_EventResult,
    &ett_rrc_SEQUENCE_SIZE_1_maxReportedEUTRACellPerFreq_OF_EUTRA_PhysicalCellIdentity,
    &ett_rrc_Eutra_EventResultList,
    &ett_rrc_EUTRA_EventResults,
    &ett_rrc_EUTRA_EventResultsForCELLFACH,
    &ett_rrc_EUTRA_FrequencyAndPriorityInfo,
    &ett_rrc_EUTRA_FrequencyAndPriorityInfo_v920ext,
    &ett_rrc_EUTRA_FrequencyAndPriorityInfoList,
    &ett_rrc_EUTRA_FrequencyAndPriorityInfoList_v920ext,
    &ett_rrc_EUTRA_FrequencyAndPriorityInfoList_vbxyext,
    &ett_rrc_EUTRA_FrequencyList,
    &ett_rrc_EUTRA_FrequencyList_r9,
    &ett_rrc_EUTRA_FrequencyList_r11,
    &ett_rrc_EUTRA_SIAcquisition,
    &ett_rrc_EUTRA_MeasuredCells,
    &ett_rrc_Eutra_MeasuredResult,
    &ett_rrc_SEQUENCE_SIZE_1_maxReportedEUTRACellPerFreq_OF_EUTRA_MeasuredCells,
    &ett_rrc_Eutra_MeasuredResultList,
    &ett_rrc_EUTRA_MeasuredResults,
    &ett_rrc_EUTRA_MeasuredCells_v920ext,
    &ett_rrc_Eutra_MeasuredResult_v920ext,
    &ett_rrc_SEQUENCE_SIZE_1_maxReportedEUTRACellPerFreq_OF_EUTRA_MeasuredCells_v920ext,
    &ett_rrc_Eutra_MeasuredResultList_v920ext,
    &ett_rrc_EUTRA_MeasuredResults_v920ext,
    &ett_rrc_EUTRA_SIacquisitionResults,
    &ett_rrc_T_cgiInfo,
    &ett_rrc_EUTRA_FrequencyRemoval,
    &ett_rrc_SEQUENCE_SIZE_1_maxNumEUTRAFreqs_OF_EARFCN,
    &ett_rrc_EUTRA_FrequencyInfoList,
    &ett_rrc_EUTRA_FrequencyInfoList_r11,
    &ett_rrc_EUTRA_FrequencyInfo,
    &ett_rrc_EUTRA_FrequencyInfo_r11,
    &ett_rrc_EUTRA_FrequencyRACHReportingInfo,
    &ett_rrc_EutraMeasurementForCELLFACH,
    &ett_rrc_SEQUENCE_SIZE_1_maxNumEUTRAFreqs_FACH_OF_EARFCN,
    &ett_rrc_Event1a,
    &ett_rrc_Event1a_r4,
    &ett_rrc_Event1a_OnSecUlFreq_r9,
    &ett_rrc_Event1a_LCR_r4,
    &ett_rrc_Event1b,
    &ett_rrc_Event1b_r4,
    &ett_rrc_Event1b_r7,
    &ett_rrc_Event1b_OnSecUlFreq_r9,
    &ett_rrc_Event1b_LCR_r4,
    &ett_rrc_Event1c,
    &ett_rrc_Event1d,
    &ett_rrc_Event1e,
    &ett_rrc_Event1e_r6,
    &ett_rrc_Event1f,
    &ett_rrc_Event1f_r6,
    &ett_rrc_Event1j_r6,
    &ett_rrc_Event2a,
    &ett_rrc_Event2a_r6,
    &ett_rrc_Event2a_r10,
    &ett_rrc_Event2a_r11,
    &ett_rrc_Event2b,
    &ett_rrc_Event2b_r6,
    &ett_rrc_Event2b_r10,
    &ett_rrc_Event2b_r11,
    &ett_rrc_Event2c,
    &ett_rrc_Event2c_r6,
    &ett_rrc_Event2c_r10,
    &ett_rrc_Event2c_r11,
    &ett_rrc_Event2d,
    &ett_rrc_Event2d_r6,
    &ett_rrc_Event2d_r10,
    &ett_rrc_Event2e,
    &ett_rrc_Event2e_r6,
    &ett_rrc_Event2e_r10,
    &ett_rrc_Event2e_r11,
    &ett_rrc_Event2f,
    &ett_rrc_Event2f_r6,
    &ett_rrc_Event2f_r10,
    &ett_rrc_Event3a,
    &ett_rrc_Event3b,
    &ett_rrc_Event3c,
    &ett_rrc_Event3d,
    &ett_rrc_EventResults,
    &ett_rrc_EventResults_v770ext,
    &ett_rrc_EventResults_v860ext,
    &ett_rrc_EventResults_va40ext,
    &ett_rrc_EventResultsOnSecUlFreq,
    &ett_rrc_ExtraDoppler,
    &ett_rrc_ExtraDopplerInfo,
    &ett_rrc_FACH_MeasurementOccasionInfo,
    &ett_rrc_SEQUENCE_SIZE_1_maxOtherRAT_OF_RAT_Type,
    &ett_rrc_FACH_MeasurementOccasionInfo_LCR_r4_ext,
    &ett_rrc_ForbiddenAffectCell,
    &ett_rrc_ForbiddenAffectCell_r4,
    &ett_rrc_ForbiddenAffectCell_LCR_r4,
    &ett_rrc_ForbiddenAffectCellList,
    &ett_rrc_ForbiddenAffectCellList_r4,
    &ett_rrc_ForbiddenAffectCellList_LCR_r4,
    &ett_rrc_ForbiddenAffectCellListOnSecULFreq,
    &ett_rrc_ForbiddenAffectCellOnSecULFreq,
    &ett_rrc_GanssDataBits,
    &ett_rrc_GANSSGenericData,
    &ett_rrc_GANSSGenericData_v860ext,
    &ett_rrc_GANSSGenericData_r8,
    &ett_rrc_GANSSGenericData_r9,
    &ett_rrc_GANSSGenericData_v920ext,
    &ett_rrc_GANSSGenericData_va40ext,
    &ett_rrc_GANSSGenericData_r10,
    &ett_rrc_GANSSGenericDataList,
    &ett_rrc_GANSSGenericDataList_v860ext,
    &ett_rrc_GANSSGenericDataList_r8,
    &ett_rrc_GANSSGenericDataList_r9,
    &ett_rrc_GANSSGenericDataList_v920ext,
    &ett_rrc_GANSSGenericDataList_va40ext,
    &ett_rrc_GANSSGenericDataList_r10,
    &ett_rrc_GANSSGenericMeasurementInfo,
    &ett_rrc_GANSSGenericMeasurementInfo_item,
    &ett_rrc_GANSSGenericMeasurementInfo_v860ext,
    &ett_rrc_GANSSGenericMeasurementInfo_v860ext_item,
    &ett_rrc_GANSSMeasurementParameters,
    &ett_rrc_GANSSMeasurementParameters_item,
    &ett_rrc_GANSSMeasurementParameters_v860ext,
    &ett_rrc_GANSSMeasurementParameters_v860ext_item,
    &ett_rrc_GANSSMeasurementSignalList,
    &ett_rrc_GANSSMeasurementSignalList_item,
    &ett_rrc_GANSSMeasurementSignalList_v860ext,
    &ett_rrc_GANSSMeasurementSignalList_v860ext_item,
    &ett_rrc_GanssNavigationModelAdditionalData,
    &ett_rrc_GANSSReferenceTimeOnly,
    &ett_rrc_GanssReqGenericData,
    &ett_rrc_GanssReqGenericData_v860ext,
    &ett_rrc_T_ganssAddADchoices,
    &ett_rrc_GanssRequestedGenericAssistanceDataList,
    &ett_rrc_GanssRequestedGenericAssistanceDataList_v860ext,
    &ett_rrc_GANSSSatelliteInformation,
    &ett_rrc_GANSSSatelliteInformation_va40ext,
    &ett_rrc_GANSSSatelliteInformation_r10,
    &ett_rrc_GANSSSatelliteInformationList,
    &ett_rrc_GANSSSatelliteInformationList_va40ext,
    &ett_rrc_GANSSSatelliteInformationList_r10,
    &ett_rrc_GANSSTimeModelsList,
    &ett_rrc_GANSSTimeModelsList_va40ext,
    &ett_rrc_GANSSTimeModelsList_r10,
    &ett_rrc_GANSS_SAT_Info_Almanac_GLOkp,
    &ett_rrc_GANSS_SAT_Info_Almanac_Kp,
    &ett_rrc_GANSS_SAT_Info_Almanac_MIDIkp,
    &ett_rrc_GANSS_SAT_Info_Almanac_NAVkp,
    &ett_rrc_GANSS_SAT_Info_Almanac_REDkp,
    &ett_rrc_GANSS_SAT_Info_Almanac_SBASecef,
    &ett_rrc_GANSS_SAT_Info_Almanac_GLOkpList,
    &ett_rrc_GANSS_SAT_Info_Almanac_KpList,
    &ett_rrc_GANSS_SAT_Info_Almanac_MIDIkpList,
    &ett_rrc_GANSS_SAT_Info_Almanac_NAVkpList,
    &ett_rrc_GANSS_SAT_Info_Almanac_REDkpList,
    &ett_rrc_GANSS_SAT_Info_Almanac_SBASecefList,
    &ett_rrc_Ganss_Sat_Info_AddNav,
    &ett_rrc_Ganss_Sat_Info_Nav,
    &ett_rrc_Ganss_Sat_Info_AddNavList,
    &ett_rrc_Ganss_Sat_Info_NavList,
    &ett_rrc_GANSS_Storm_Flag,
    &ett_rrc_GLONASSclockModel,
    &ett_rrc_GPS_MeasurementParam,
    &ett_rrc_GPS_MeasurementParamList,
    &ett_rrc_GSM_CellGroup,
    &ett_rrc_T_followingARFCNs,
    &ett_rrc_SEQUENCE_SIZE_0_31_OF_BCCH_ARFCN,
    &ett_rrc_T_equallySpacedARFCNs,
    &ett_rrc_T_continuousRangeOfARFCNs,
    &ett_rrc_GSM_MeasuredResults,
    &ett_rrc_GSM_MeasuredResultsList,
    &ett_rrc_GPS_TOW_Assist,
    &ett_rrc_GPS_TOW_AssistList,
    &ett_rrc_GSM_PriorityInfo,
    &ett_rrc_GSM_PriorityInfoList,
    &ett_rrc_GSM_PriorityInfoList_vbxyext,
    &ett_rrc_HCS_CellReselectInformation_RSCP,
    &ett_rrc_HCS_CellReselectInformation_ECN0,
    &ett_rrc_HCS_NeighbouringCellInformation_RSCP,
    &ett_rrc_HCS_NeighbouringCellInformation_ECN0,
    &ett_rrc_HCS_ServingCellInformation,
    &ett_rrc_HorizontalVelocity,
    &ett_rrc_HorizontalWithVerticalVelocity,
    &ett_rrc_HorizontalVelocityWithUncertainty,
    &ett_rrc_HorizontalWithVerticalVelocityAndUncertainty,
    &ett_rrc_IdleIntervalInfo,
    &ett_rrc_InterFreqCell,
    &ett_rrc_InterFreqCell_LCR_r4,
    &ett_rrc_InterFreqCellInfoList,
    &ett_rrc_InterFreqCellInfoList_r4,
    &ett_rrc_InterFreqCellInfoList_r8,
    &ett_rrc_InterFreqCellInfoList_r9,
    &ett_rrc_InterFreqCellInfoList_r10,
    &ett_rrc_InterFreqSIAcquisition,
    &ett_rrc_T_modeSpecificInfo_111,
    &ett_rrc_T_fdd_150,
    &ett_rrc_InterFreqCellInfoSI_List_RSCP,
    &ett_rrc_InterFreqCellInfoSI_List_ECN0,
    &ett_rrc_InterFreqCellInfoSI_List_HCS_RSCP,
    &ett_rrc_InterFreqCellInfoSI_List_HCS_ECN0,
    &ett_rrc_InterFreqCellInfoSI_List_RSCP_LCR,
    &ett_rrc_InterFreqCellInfoSI_List_ECN0_LCR,
    &ett_rrc_InterFreqCellInfoSI_List_HCS_RSCP_LCR,
    &ett_rrc_InterFreqCellInfoSI_List_HCS_ECN0_LCR,
    &ett_rrc_InterFreqCellList,
    &ett_rrc_InterFreqCellList_LCR_r4_ext,
    &ett_rrc_InterFreqCellMeasuredResultsList,
    &ett_rrc_InterFreqCellMeasuredResultsList_v920ext,
    &ett_rrc_InterFreqEvent,
    &ett_rrc_InterFreqEvent_r6,
    &ett_rrc_InterFreqEvent_r10,
    &ett_rrc_InterFreqEvent_r11,
    &ett_rrc_InterFreqEventList,
    &ett_rrc_InterFreqEventList_r6,
    &ett_rrc_InterFreqEventList_r10,
    &ett_rrc_InterFreqEventList_r11,
    &ett_rrc_InterFrequencyMeasuredResultsList_v590ext,
    &ett_rrc_Inter_FreqEventCriteria_v590ext,
    &ett_rrc_Inter_FreqEventCriteriaList_v590ext,
    &ett_rrc_Intra_FreqEventCriteriaList_v590ext,
    &ett_rrc_IntraFrequencyMeasuredResultsList_v590ext,
    &ett_rrc_InterFreqEventResults,
    &ett_rrc_InterFreqEventResults_va40ext,
    &ett_rrc_InterFreqEventResults_LCR_r4_ext,
    &ett_rrc_InterFreqMeasQuantity,
    &ett_rrc_T_reportingCriteria,
    &ett_rrc_T_intraFreqReportingCriteria,
    &ett_rrc_T_interFreqReportingCriteria,
    &ett_rrc_T_modeSpecificInfo_112,
    &ett_rrc_T_fdd_151,
    &ett_rrc_T_tdd_100,
    &ett_rrc_InterFreqMeasuredResults,
    &ett_rrc_InterFreqMeasuredResults_v920ext,
    &ett_rrc_InterFreqMeasuredResultsList,
    &ett_rrc_InterFreqMeasuredResultsList_v920ext,
    &ett_rrc_InterFreqMeasurementSysInfo_RSCP,
    &ett_rrc_InterFreqMeasurementSysInfo_ECN0,
    &ett_rrc_InterFreqMeasurementSysInfo_HCS_RSCP,
    &ett_rrc_InterFreqMeasurementSysInfo_HCS_ECN0,
    &ett_rrc_InterFreqMeasurementSysInfo_RSCP_LCR_r4,
    &ett_rrc_InterFreqMeasurementSysInfo_ECN0_LCR_r4,
    &ett_rrc_InterFreqMeasurementSysInfo_HCS_RSCP_LCR_r4,
    &ett_rrc_InterFreqMeasurementSysInfo_HCS_ECN0_LCR_r4,
    &ett_rrc_InterFreqRACHRepCellsList,
    &ett_rrc_Dummy_InterFreqRACHReportingInfo,
    &ett_rrc_T_modeSpecificInfo_113,
    &ett_rrc_T_fdd_152,
    &ett_rrc_T_tdd_101,
    &ett_rrc_InterFreqRACHReportingInfo,
    &ett_rrc_T_modeSpecificInfo_114,
    &ett_rrc_T_fdd_153,
    &ett_rrc_T_tdd_102,
    &ett_rrc_InterFreqReportCriteria,
    &ett_rrc_InterFreqReportCriteria_r4,
    &ett_rrc_InterFreqReportCriteria_r6,
    &ett_rrc_InterFreqReportCriteria_r7,
    &ett_rrc_InterFreqReportCriteria_r9,
    &ett_rrc_InterFreqReportCriteria_r10,
    &ett_rrc_InterFreqReportCriteria_r11,
    &ett_rrc_InterFreqReportingCriteria,
    &ett_rrc_InterFreqReportingCriteria_r6,
    &ett_rrc_InterFreqReportingCriteria_r10,
    &ett_rrc_InterFreqReportingCriteria_r11,
    &ett_rrc_InterFreqReportingQuantity,
    &ett_rrc_InterFreqRepQuantityRACH_TDDList,
    &ett_rrc_InterFrequencyMeasurement,
    &ett_rrc_InterFrequencyMeasurement_r4,
    &ett_rrc_InterFrequencyMeasurement_r6,
    &ett_rrc_InterFrequencyMeasurement_r7,
    &ett_rrc_InterFrequencyMeasurement_r8,
    &ett_rrc_InterFrequencyMeasurement_r9,
    &ett_rrc_InterFrequencyMeasurement_r10,
    &ett_rrc_InterFrequencyMeasurement_r11,
    &ett_rrc_InterRAT_TargetCellDescription,
    &ett_rrc_T_technologySpecificInfo,
    &ett_rrc_T_gsm_08,
    &ett_rrc_InterRATCellInfoList,
    &ett_rrc_InterRATCellInfoList_B,
    &ett_rrc_InterRATCellInfoList_r4,
    &ett_rrc_InterRATCellInfoList_r6,
    &ett_rrc_InterRATEvent,
    &ett_rrc_InterRATEventList,
    &ett_rrc_InterRATEventResults,
    &ett_rrc_InterRATInfo_r6,
    &ett_rrc_InterRATInfo_v860ext,
    &ett_rrc_T_eutra_01,
    &ett_rrc_InterRATMeasQuantity,
    &ett_rrc_T_ratSpecificInfo,
    &ett_rrc_T_gsm_09,
    &ett_rrc_T_is_2000,
    &ett_rrc_InterRATMeasQuantity_r8,
    &ett_rrc_T_ratSpecificInfo_01,
    &ett_rrc_T_gsm_10,
    &ett_rrc_T_is_2000_01,
    &ett_rrc_T_e_UTRA_01,
    &ett_rrc_InterRATMeasuredResults,
    &ett_rrc_InterRATMeasuredResultsList,
    &ett_rrc_InterRATMeasurement,
    &ett_rrc_InterRATMeasurement_r4,
    &ett_rrc_InterRATMeasurement_r6,
    &ett_rrc_InterRATMeasurement_r8,
    &ett_rrc_T_interRATMeasurementObjects,
    &ett_rrc_InterRATMeasurement_r9,
    &ett_rrc_T_interRATMeasurementObjects_01,
    &ett_rrc_InterRATMeasurement_r11,
    &ett_rrc_T_interRATMeasurementObjects_02,
    &ett_rrc_InterRATMeasurementSysInfo,
    &ett_rrc_InterRATMeasurementSysInfo_B,
    &ett_rrc_InterRATReportCriteria,
    &ett_rrc_InterRATReportingCriteria,
    &ett_rrc_InterRATReportingQuantity,
    &ett_rrc_T_ratSpecificInfo_02,
    &ett_rrc_T_gsm_11,
    &ett_rrc_InterRATReportingQuantity_r8,
    &ett_rrc_T_ratSpecificInfo_03,
    &ett_rrc_T_gsm_12,
    &ett_rrc_T_eutra_02,
    &ett_rrc_IntraFreqCellInfoList,
    &ett_rrc_IntraFreqCellInfoList_r4,
    &ett_rrc_IntraFreqCellInfoListInfoOnSecULFreq,
    &ett_rrc_IntraFreqCellInfoListOnSecULFreq,
    &ett_rrc_IntraFreqCellInfoList_r9,
    &ett_rrc_IntraFreqCellInfoList_r10,
    &ett_rrc_IntraFreqSIAcquisitionInfo,
    &ett_rrc_IntraFreqSIAcquisition,
    &ett_rrc_T_modeSpecificInfo_115,
    &ett_rrc_T_fdd_154,
    &ett_rrc_IntraFreqCellInfoSI_List_RSCP,
    &ett_rrc_IntraFreqCellInfoSI_List_ECN0,
    &ett_rrc_IntraFreqCellInfoSI_List_HCS_RSCP,
    &ett_rrc_IntraFreqCellInfoSI_List_HCS_ECN0,
    &ett_rrc_IntraFreqCellInfoSI_List_RSCP_LCR_r4,
    &ett_rrc_IntraFreqCellInfoSI_List_ECN0_LCR_r4,
    &ett_rrc_IntraFreqCellInfoSI_List_HCS_RSCP_LCR_r4,
    &ett_rrc_IntraFreqCellInfoSI_List_HCS_ECN0_LCR_r4,
    &ett_rrc_IntraFreqEvent,
    &ett_rrc_IntraFreqEvent_r4,
    &ett_rrc_IntraFreqEvent_LCR_r4,
    &ett_rrc_IntraFreqEvent_r6,
    &ett_rrc_IntraFreqEvent_r7,
    &ett_rrc_IntraFreqEventOnSecULFreq,
    &ett_rrc_IntraFreqEvent_1d_r5,
    &ett_rrc_IntraFreqEventCriteria,
    &ett_rrc_IntraFreqEventCriteria_r4,
    &ett_rrc_IntraFreqEventCriteria_LCR_r4,
    &ett_rrc_IntraFreqEventCriteria_r6,
    &ett_rrc_IntraFreqEventCriteria_r7,
    &ett_rrc_IntraFreqEventCriteriaOnSecULFreq,
    &ett_rrc_IntraFreqEventCriteriaList,
    &ett_rrc_IntraFreqEventCriteriaList_r4,
    &ett_rrc_IntraFreqEventCriteriaList_LCR_r4,
    &ett_rrc_IntraFreqEventCriteriaList_r6,
    &ett_rrc_IntraFreqEventCriteriaList_r7,
    &ett_rrc_IntraFreqEventCriteriaListOnSecULFreq,
    &ett_rrc_SEQUENCE_SIZE_1_maxMeasEventOnSecULFreq_OF_IntraFreqEventCriteriaOnSecULFreq,
    &ett_rrc_IntraFreqEventResults,
    &ett_rrc_IntraFreqEventResultsOnSecUlFreq,
    &ett_rrc_IntraFreqMeasQuantity,
    &ett_rrc_T_modeSpecificInfo_116,
    &ett_rrc_T_fdd_155,
    &ett_rrc_T_tdd_103,
    &ett_rrc_IntraFreqMeasQuantity_TDDList,
    &ett_rrc_IntraFreqMeasQuantity_TDD_sib3List,
    &ett_rrc_IntraFreqMeasuredResultsList,
    &ett_rrc_IntraFreqMeasuredResultsList_v920ext,
    &ett_rrc_IntraFreqMeasurementSysInfo_RSCP,
    &ett_rrc_IntraFreqMeasurementSysInfo_ECN0,
    &ett_rrc_IntraFreqMeasurementSysInfo_HCS_RSCP,
    &ett_rrc_IntraFreqMeasurementSysInfo_HCS_ECN0,
    &ett_rrc_IntraFreqMeasurementSysInfo_RSCP_LCR_r4,
    &ett_rrc_IntraFreqMeasurementSysInfo_ECN0_LCR_r4,
    &ett_rrc_IntraFreqMeasurementSysInfo_HCS_RSCP_LCR_r4,
    &ett_rrc_IntraFreqMeasurementSysInfo_HCS_ECN0_LCR_r4,
    &ett_rrc_IntraFreqReportCriteria,
    &ett_rrc_IntraFreqReportCriteria_r4,
    &ett_rrc_IntraFreqReportCriteria_r6,
    &ett_rrc_IntraFreqReportCriteria_r7,
    &ett_rrc_IntraFreqReportCriteria_r9,
    &ett_rrc_IntraFreqReportCriteria_r11,
    &ett_rrc_IntraFreqReportingCriteria,
    &ett_rrc_IntraFreqReportingCriteria_r4,
    &ett_rrc_IntraFreqReportingCriteria_LCR_r4,
    &ett_rrc_IntraFreqReportingCriteria_r6,
    &ett_rrc_IntraFreqReportingCriteria_r7,
    &ett_rrc_IntraFreqReportingCriteria_r9,
    &ett_rrc_IntraFreqReportingCriteria_r11,
    &ett_rrc_IntraFreqReportingCriteria_1b_r5,
    &ett_rrc_IntraFreqReportingQuantity,
    &ett_rrc_IntraFreqReportingQuantityForRACH,
    &ett_rrc_T_modeSpecificInfo_117,
    &ett_rrc_T_fdd_156,
    &ett_rrc_T_tdd_104,
    &ett_rrc_IntraFreqRepQuantityRACH_TDDList,
    &ett_rrc_IntraFrequencyMeasurement,
    &ett_rrc_IntraFrequencyMeasurement_r4,
    &ett_rrc_IntraFrequencyMeasurement_r6,
    &ett_rrc_IntraFrequencyMeasurement_r7,
    &ett_rrc_IntraFrequencyMeasurement_r9,
    &ett_rrc_IntraFrequencyMeasurement_r10,
    &ett_rrc_IntraFrequencyMeasurement_r11,
    &ett_rrc_IntraFrequencyMeasuredResultsListOnSecULFreq,
    &ett_rrc_IntraUTRAANR,
    &ett_rrc_LoggedANRConfigurationInfo,
    &ett_rrc_LoggedANRReportInfoList,
    &ett_rrc_LoggedANRReportInfo,
    &ett_rrc_LoggedCellInfo,
    &ett_rrc_LoggedConnectionEstablishmentFailureInfo_FDD,
    &ett_rrc_LoggedConnectionEstablishmentFailureInfo_TDD128,
    &ett_rrc_LoggedConnectionEstablishmentFailureAccessInfo_FDD,
    &ett_rrc_LoggedConnectionEstablishmentFailureAccessInfo_TDD,
    &ett_rrc_LoggedConnectionEstablishmentFailureLocationInfo,
    &ett_rrc_T_location,
    &ett_rrc_LoggedConnectionEstablishmentFailureReport,
    &ett_rrc_T_modeSpecificInfo_118,
    &ett_rrc_T_fdd_157,
    &ett_rrc_T_tdd_105,
    &ett_rrc_LoggedEUTRACellInfo,
    &ett_rrc_LoggedGSMCellInfo,
    &ett_rrc_LoggedMeasurementsConfigurationInfo,
    &ett_rrc_T_areaConfiguration,
    &ett_rrc_SEQUENCE_SIZE_1_32_OF_CellIdentity,
    &ett_rrc_SEQUENCE_SIZE_1_8_OF_LAI,
    &ett_rrc_SEQUENCE_SIZE_1_8_OF_RAI,
    &ett_rrc_LoggedMeasurementsConfigurationInfo_r11,
    &ett_rrc_SEQUENCE_SIZE_1_15_OF_PLMN_Identity,
    &ett_rrc_T_areaConfiguration_01,
    &ett_rrc_SEQUENCE_SIZE_1_32_OF_CellIDListItem,
    &ett_rrc_LoggedMeasInterfreqList_FDD,
    &ett_rrc_LoggedMeasInterfreqInfo_FDD,
    &ett_rrc_SEQUENCE_SIZE_1_maxnumLoggedMeas_OF_LoggedMeasNeighbourInfo_FDD,
    &ett_rrc_LoggedMeasInterfreqInfo_TDD128,
    &ett_rrc_LoggedMeasInterRATNeighbourMeas,
    &ett_rrc_LoggedMeasIntrafreqNeighbourList_FDD,
    &ett_rrc_LoggedMeasNeighbourInfo_FDD,
    &ett_rrc_LoggedMeasIntrafreqNeighbourMeasList_TDD128,
    &ett_rrc_LoggedMeasNeighbourMeas_TDD128,
    &ett_rrc_LoggedMeasGSMNeighbourCellsList,
    &ett_rrc_LoggedMeasGSMNeighbourCellsinfo,
    &ett_rrc_LoggedMeasEUTRAFreqList,
    &ett_rrc_LoggedMeasEUTRAFreqInfo,
    &ett_rrc_SEQUENCE_SIZE_1_maxnumLoggedMeas_OF_LoggedMeasEUTRAFreqNeighbourInfo,
    &ett_rrc_LoggedMeasEUTRAFreqNeighbourInfo,
    &ett_rrc_LoggedMeasLocationInfo,
    &ett_rrc_LoggedMeasLocationInfo_vbxyext,
    &ett_rrc_LoggedMeasInfo_FDD,
    &ett_rrc_LoggedMeasInfo_FDD_vbxyext,
    &ett_rrc_LoggedMeasInfo_TDD128,
    &ett_rrc_LoggedMeasInfo_TDD128_vbxyext,
    &ett_rrc_LoggedMeasInterfreqList_TDD128,
    &ett_rrc_LoggedMeasInterfreqNeighbourList_TDD128,
    &ett_rrc_LoggedMeasReport,
    &ett_rrc_LoggedMeasReport_vbxyext,
    &ett_rrc_LoggedMeasInfoList_FDD,
    &ett_rrc_LoggedMeasInfoList_FDD_vbxyext,
    &ett_rrc_LoggedMeasInfoList_TDD128,
    &ett_rrc_LoggedMeasInfoList_TDD128_vbxyext,
    &ett_rrc_LoggedMeasInfoList,
    &ett_rrc_T_modeSpecificInfo_119,
    &ett_rrc_T_fdd_158,
    &ett_rrc_T_tdd_106,
    &ett_rrc_LoggedMeasInfoList_vbxyext,
    &ett_rrc_T_modeSpecificInfo_120,
    &ett_rrc_T_fdd_159,
    &ett_rrc_T_tdd_107,
    &ett_rrc_LoggedMeasServingCellMeas_FDD,
    &ett_rrc_LoggedMeasServingCellMeas_FDD_vbxyext,
    &ett_rrc_LoggedMeasServingCellMeas_TDD128,
    &ett_rrc_LoggedMeasServingCellMeas_TDD128_vbxyext,
    &ett_rrc_LoggedUTRACellInfo,
    &ett_rrc_LoggingAbsoluteThreshold,
    &ett_rrc_KeplerianParameters,
    &ett_rrc_MeasuredResults,
    &ett_rrc_MeasuredResultsOnSecUlFreq,
    &ett_rrc_MeasuredResults_v390ext,
    &ett_rrc_MeasuredResults_v590ext,
    &ett_rrc_MeasuredResults_v770ext,
    &ett_rrc_MeasuredResults_v860ext,
    &ett_rrc_MeasuredResultsList_v920ext,
    &ett_rrc_MeasuredResultsListOnSecUlFreq,
    &ett_rrc_MeasuredResults_v920ext,
    &ett_rrc_MeasuredResults_LCR_r4,
    &ett_rrc_MeasuredResultsList,
    &ett_rrc_MeasuredResultsList_LCR_r4_ext,
    &ett_rrc_MeasuredResultsList_v770xet,
    &ett_rrc_MeasuredResultsList_v770xet_item,
    &ett_rrc_MeasuredResultsList_v860ext,
    &ett_rrc_MeasuredResultsList_v860ext_item,
    &ett_rrc_MeasuredResultsOnRACH,
    &ett_rrc_T_currentCell,
    &ett_rrc_T_modeSpecificInfo_121,
    &ett_rrc_T_fdd_160,
    &ett_rrc_T_measurementQuantity_01,
    &ett_rrc_T_tdd_108,
    &ett_rrc_MeasuredResultsOnRACH_v7g0ext,
    &ett_rrc_MonitoredCellRACH_List_v7g0ext,
    &ett_rrc_MeasuredResultsOnRACHinterFreq,
    &ett_rrc_MeasuredResultsOnRACHeutraFreq,
    &ett_rrc_MeasurementCommand,
    &ett_rrc_T_modify_01,
    &ett_rrc_MeasurementCommand_r4,
    &ett_rrc_T_modify_02,
    &ett_rrc_MeasurementCommand_r6,
    &ett_rrc_T_modify_03,
    &ett_rrc_MeasurementCommand_r7,
    &ett_rrc_T_modify_04,
    &ett_rrc_MeasurementCommand_r8,
    &ett_rrc_T_modify_05,
    &ett_rrc_MeasurementCommand_r9,
    &ett_rrc_T_modify_06,
    &ett_rrc_MeasurementCommand_r10,
    &ett_rrc_T_modify_07,
    &ett_rrc_MeasurementCommand_r11,
    &ett_rrc_T_modify_08,
    &ett_rrc_MeasurementControlSysInfo,
    &ett_rrc_T_use_of_HCS,
    &ett_rrc_T_hcs_not_used,
    &ett_rrc_T_cellSelectQualityMeasure_01,
    &ett_rrc_T_cpich_RSCP,
    &ett_rrc_T_cpich_Ec_N0_01,
    &ett_rrc_T_hcs_used,
    &ett_rrc_T_cellSelectQualityMeasure_02,
    &ett_rrc_T_cpich_RSCP_01,
    &ett_rrc_T_cpich_Ec_N0_02,
    &ett_rrc_MeasurementControlSysInfoExtension,
    &ett_rrc_T_use_of_HCS_01,
    &ett_rrc_T_hcs_not_used_01,
    &ett_rrc_T_cellSelectQualityMeasure_03,
    &ett_rrc_T_cpich_RSCP_02,
    &ett_rrc_T_cpich_Ec_N0_03,
    &ett_rrc_T_hcs_used_01,
    &ett_rrc_T_cellSelectQualityMeasure_04,
    &ett_rrc_T_cpich_RSCP_03,
    &ett_rrc_T_cpich_Ec_N0_04,
    &ett_rrc_MeasurementControlSysInfoExtensionAddon_r5,
    &ett_rrc_SEQUENCE_SIZE_1_maxCellMeas_OF_CellSelectReselectInfo_v590ext,
    &ett_rrc_MeasurementControlSysInfoExtension_LCR_r4,
    &ett_rrc_T_use_of_HCS_02,
    &ett_rrc_T_hcs_not_used_02,
    &ett_rrc_T_cellSelectQualityMeasure_05,
    &ett_rrc_T_cpich_RSCP_04,
    &ett_rrc_T_cpich_Ec_N0_05,
    &ett_rrc_T_hcs_used_02,
    &ett_rrc_T_cellSelectQualityMeasure_06,
    &ett_rrc_T_cpich_RSCP_05,
    &ett_rrc_T_cpich_Ec_N0_06,
    &ett_rrc_MeasurementControlSysInfo_LCR_r4_ext,
    &ett_rrc_T_use_of_HCS_03,
    &ett_rrc_T_hcs_not_used_03,
    &ett_rrc_T_cellSelectQualityMeasure_07,
    &ett_rrc_T_cpich_RSCP_06,
    &ett_rrc_T_cpich_Ec_N0_07,
    &ett_rrc_T_hcs_used_03,
    &ett_rrc_T_cellSelectQualityMeasure_08,
    &ett_rrc_T_cpich_RSCP_07,
    &ett_rrc_T_cpich_Ec_N0_08,
    &ett_rrc_MeasurementOccasionPatternParameter,
    &ett_rrc_MeasurementReportingMode,
    &ett_rrc_MeasurementType,
    &ett_rrc_MeasurementType_r4,
    &ett_rrc_MeasurementType_r6,
    &ett_rrc_MeasurementType_r7,
    &ett_rrc_MeasurementType_r8,
    &ett_rrc_MeasurementType_r9,
    &ett_rrc_MeasurementType_r10,
    &ett_rrc_MeasurementType_r11,
    &ett_rrc_MeasurementValidity,
    &ett_rrc_MonitoredCellRACH_List,
    &ett_rrc_MonitoredCellRACH_Result,
    &ett_rrc_T_modeSpecificInfo_122,
    &ett_rrc_T_fdd_161,
    &ett_rrc_T_measurementQuantity_02,
    &ett_rrc_T_tdd_109,
    &ett_rrc_MultipleEUTRAFrequencyBandIndicatorList,
    &ett_rrc_MultipleEUTRAFrequencyBandInfo,
    &ett_rrc_MultipleEUTRAFrequencyInfoList,
    &ett_rrc_MultipleFrequencyBandIndicatorListFDD,
    &ett_rrc_MultipleFrequencyBandInfo,
    &ett_rrc_MultipleFrequencyInfoListFDD,
    &ett_rrc_N_CR_T_CRMaxHyst,
    &ett_rrc_NAVclockModel,
    &ett_rrc_NavigationModelSatInfo,
    &ett_rrc_NavigationModelSatInfoList,
    &ett_rrc_NavModel_CNAVKeplerianSet,
    &ett_rrc_NavModel_GLONASSecef,
    &ett_rrc_NavModel_NAVKeplerianSet,
    &ett_rrc_NavModel_SBASecef,
    &ett_rrc_EphemerisParameter,
    &ett_rrc_Neighbour,
    &ett_rrc_T_modeSpecificInfo_123,
    &ett_rrc_T_fdd_162,
    &ett_rrc_T_tdd_110,
    &ett_rrc_Neighbour_v390ext,
    &ett_rrc_T_modeSpecificInfo_124,
    &ett_rrc_T_fdd_163,
    &ett_rrc_NeighbourList,
    &ett_rrc_Neighbour_TDD_r7,
    &ett_rrc_NeighbourList_TDD_r7,
    &ett_rrc_NeighbourList_v390ext,
    &ett_rrc_NeighbourQuality,
    &ett_rrc_NewInterFreqCell,
    &ett_rrc_NewInterFreqCell_r4,
    &ett_rrc_NewInterFreqCell_v7b0ext,
    &ett_rrc_NewInterFreqCell_LCR_v8a0ext,
    &ett_rrc_NewInterFreqCell_r8,
    &ett_rrc_T_modeSpecificInfo_125,
    &ett_rrc_T_tdd128_62,
    &ett_rrc_NewInterFreqCell_r9,
    &ett_rrc_T_modeSpecificInfo_126,
    &ett_rrc_T_tdd128_63,
    &ett_rrc_NewInterFreqCell_r10,
    &ett_rrc_T_modeSpecificInfo_127,
    &ett_rrc_T_tdd128_64,
    &ett_rrc_NewInterFreqCellList,
    &ett_rrc_NewInterFreqCellList_r4,
    &ett_rrc_NewInterFreqCellList_v7b0ext,
    &ett_rrc_NewInterFreqCellList_LCR_v8a0ext,
    &ett_rrc_NewInterFreqCellList_r8,
    &ett_rrc_NewInterFreqCellList_r9,
    &ett_rrc_NewInterFreqCellList_r10,
    &ett_rrc_NewInterFreqCellSI_RSCP,
    &ett_rrc_NewInterFreqCellSI_ECN0,
    &ett_rrc_NewInterFreqCellSI_HCS_RSCP,
    &ett_rrc_NewInterFreqCellSI_HCS_ECN0,
    &ett_rrc_NewInterFreqCellSI_RSCP_LCR_r4,
    &ett_rrc_NewInterFreqCellSI_ECN0_LCR_r4,
    &ett_rrc_NewInterFreqCellSI_HCS_RSCP_LCR_r4,
    &ett_rrc_NewInterFreqCellSI_HCS_ECN0_LCR_r4,
    &ett_rrc_NewInterFreqCellSI_List_ECN0,
    &ett_rrc_NewInterFreqCellSI_List_HCS_RSCP,
    &ett_rrc_NewInterFreqCellSI_List_HCS_ECN0,
    &ett_rrc_NewInterFreqCellSI_List_RSCP,
    &ett_rrc_NewInterFreqCellSI_List_ECN0_LCR_r4,
    &ett_rrc_NewInterFreqCellSI_List_HCS_RSCP_LCR_r4,
    &ett_rrc_NewInterFreqCellSI_List_HCS_ECN0_LCR_r4,
    &ett_rrc_NewInterFreqCellSI_List_RSCP_LCR_r4,
    &ett_rrc_NewInterRATCell,
    &ett_rrc_T_technologySpecificInfo_01,
    &ett_rrc_T_gsm_13,
    &ett_rrc_T_is_2000_02,
    &ett_rrc_NewInterRATCell_B,
    &ett_rrc_T_technologySpecificInfo_02,
    &ett_rrc_T_gsm_14,
    &ett_rrc_T_is_2000_03,
    &ett_rrc_NewInterRATCellList,
    &ett_rrc_NewInterRATCellList_B,
    &ett_rrc_NewIntraFreqCell,
    &ett_rrc_NewIntraFreqCell_r4,
    &ett_rrc_NewIntraFreqCell_r9,
    &ett_rrc_NewIntraFreqCell_r10,
    &ett_rrc_T_modeSpecificInfo_128,
    &ett_rrc_T_tdd128_65,
    &ett_rrc_NewIntraFreqCellOnSecULFreq,
    &ett_rrc_NewIntraFreqCellList,
    &ett_rrc_NewIntraFreqCellList_r4,
    &ett_rrc_NewIntraFreqCellList_r9,
    &ett_rrc_NewIntraFreqCellList_r10,
    &ett_rrc_NewIntraFreqCell_LCR_v8a0ext,
    &ett_rrc_NewIntraFreqCellList_LCR_v8a0ext,
    &ett_rrc_NewIntraFreqCellListOnSecULFreq,
    &ett_rrc_NewIntraFreqCellSI_RSCP,
    &ett_rrc_NewIntraFreqCellSI_ECN0,
    &ett_rrc_NewIntraFreqCellSI_HCS_RSCP,
    &ett_rrc_NewIntraFreqCellSI_HCS_ECN0,
    &ett_rrc_NewIntraFreqCellSI_RSCP_LCR_r4,
    &ett_rrc_NewIntraFreqCellSI_ECN0_LCR_r4,
    &ett_rrc_NewIntraFreqCellSI_HCS_RSCP_LCR_r4,
    &ett_rrc_NewIntraFreqCellSI_HCS_ECN0_LCR_r4,
    &ett_rrc_NewIntraFreqCellSI_List_RSCP,
    &ett_rrc_NewIntraFreqCellSI_List_ECN0,
    &ett_rrc_NewIntraFreqCellSI_List_HCS_RSCP,
    &ett_rrc_NewIntraFreqCellSI_List_HCS_ECN0,
    &ett_rrc_NewIntraFreqCellSI_List_RSCP_LCR_r4,
    &ett_rrc_NewIntraFreqCellSI_List_ECN0_LCR_r4,
    &ett_rrc_NewIntraFreqCellSI_List_HCS_RSCP_LCR_r4,
    &ett_rrc_NewIntraFreqCellSI_List_HCS_ECN0_LCR_r4,
    &ett_rrc_NonUsedFreqParameter,
    &ett_rrc_NonUsedFreqParameter_r6,
    &ett_rrc_NonUsedFreqParameter_r10,
    &ett_rrc_NonUsedFreqParameter_r11,
    &ett_rrc_NonUsedFreq2aParameter_r10,
    &ett_rrc_NonUsedFreq2aParameter_r11,
    &ett_rrc_NonUsedFreqParameterList,
    &ett_rrc_NonUsedFreqParameterList_r6,
    &ett_rrc_NonUsedFreqParameterList_r10,
    &ett_rrc_NonUsedFreqParameterList_r11,
    &ett_rrc_NonUsedFreq2aParameterList_r10,
    &ett_rrc_NonUsedFreq2aParameterList_r11,
    &ett_rrc_NonUsedFreqWList_r6,
    &ett_rrc_PenaltyTime_RSCP,
    &ett_rrc_PenaltyTime_ECN0,
    &ett_rrc_PeriodicalReportingCriteria,
    &ett_rrc_PeriodicReportingInfo_1b,
    &ett_rrc_PeriodicalWithReportingCellStatus,
    &ett_rrc_PeriodicalWithReportingCellStatus_r10,
    &ett_rrc_PeriodicalWithReportingCellStatusOnSecULFreq,
    &ett_rrc_PLMNIdentitiesOfNeighbourCells,
    &ett_rrc_PLMNIdentitiesOfNeighbourCells_v860ext,
    &ett_rrc_PLMNsOfInterFreqCellsList,
    &ett_rrc_PLMNsOfInterFreqCellsList_item,
    &ett_rrc_MultiplePLMNsOfInterFreqCellsList,
    &ett_rrc_MultiplePLMNsOfInterFreqCellsList_item,
    &ett_rrc_SEQUENCE_SIZE_1_6_OF_PLMN_IdentityWithOptionalMCC_r6,
    &ett_rrc_PLMNsOfIntraFreqCellsList,
    &ett_rrc_PLMNsOfIntraFreqCellsList_item,
    &ett_rrc_MultiplePLMNsOfIntraFreqCellsList,
    &ett_rrc_MultiplePLMNsOfIntraFreqCellsList_item,
    &ett_rrc_PLMNsOfInterRATCellsList,
    &ett_rrc_PLMNsOfInterRATCellsList_item,
    &ett_rrc_PositionEstimate,
    &ett_rrc_QualityEventResults,
    &ett_rrc_QualityMeasuredResults,
    &ett_rrc_T_modeSpecificInfo_129,
    &ett_rrc_T_tdd_111,
    &ett_rrc_QualityMeasurement,
    &ett_rrc_QualityReportCriteria,
    &ett_rrc_QualityReportingCriteria,
    &ett_rrc_QualityReportingCriteriaSingle,
    &ett_rrc_QualityReportingQuantity,
    &ett_rrc_T_modeSpecificInfo_130,
    &ett_rrc_T_tdd_112,
    &ett_rrc_ReferenceCellPosition,
    &ett_rrc_ReferenceLocation,
    &ett_rrc_ReferenceLocationGANSS,
    &ett_rrc_ReferenceTimeDifferenceToCell,
    &ett_rrc_RemovedInterFreqCellList,
    &ett_rrc_SEQUENCE_SIZE_1_maxCellMeas_OF_InterFreqCellID,
    &ett_rrc_RemovedInterRATCellList,
    &ett_rrc_SEQUENCE_SIZE_1_maxCellMeas_OF_InterRATCellID,
    &ett_rrc_RemovedIntraFreqCellList,
    &ett_rrc_SEQUENCE_SIZE_1_maxCellMeas_OF_IntraFreqCellID,
    &ett_rrc_RemovedIntraFreqCellListOnSecULFreq,
    &ett_rrc_SEQUENCE_SIZE_1_maxCellMeasOnSecULFreq_OF_IntraFreqCellIDOnSecULFreq,
    &ett_rrc_ReportingCellStatus,
    &ett_rrc_ReportingCellStatus_r10,
    &ett_rrc_ReportingCellStatusOpt,
    &ett_rrc_ReportingCellStatusOpt_r10,
    &ett_rrc_ReportingInfoForCellDCH,
    &ett_rrc_ReportingInfoForCellDCH_LCR_r4,
    &ett_rrc_ReqDataBitAssistance,
    &ett_rrc_T_ganssSatelliteInfo,
    &ett_rrc_RL_AdditionInfoList,
    &ett_rrc_RL_InformationLists,
    &ett_rrc_RSCPforANR,
    &ett_rrc_SatData,
    &ett_rrc_SatDataList,
    &ett_rrc_SatellitesListRelatedData,
    &ett_rrc_SatellitesListRelatedDataList,
    &ett_rrc_Satellite_clock_model,
    &ett_rrc_Satellite_clock_modelList,
    &ett_rrc_SBASclockModel,
    &ett_rrc_S_DPCCH_Information,
    &ett_rrc_SFN_SFN_ObsTimeDifference,
    &ett_rrc_SFN_SFN_RelTimeDifference1,
    &ett_rrc_SIR_MeasurementList,
    &ett_rrc_SIR_MeasurementResults,
    &ett_rrc_SIR_TFCS_List,
    &ett_rrc_SIR_TimeslotList,
    &ett_rrc_SubFrame1Reserved,
    &ett_rrc_T_ADVinfo,
    &ett_rrc_T_ADVinfo_ext,
    &ett_rrc_T_CRMax,
    &ett_rrc_TCE_Id,
    &ett_rrc_TemporaryOffsetList,
    &ett_rrc_ThreholdNonUsedFrequency_deltaList,
    &ett_rrc_TimeslotInfo,
    &ett_rrc_TimeslotInfo_LCR_r4,
    &ett_rrc_TimeslotInfoList,
    &ett_rrc_TimeslotInfoList_LCR_r4,
    &ett_rrc_TimeslotInfoList_r4,
    &ett_rrc_SEQUENCE_SIZE_1_maxTS_OF_TimeslotInfo,
    &ett_rrc_SEQUENCE_SIZE_1_maxTS_LCR_OF_TimeslotInfo_LCR_r4,
    &ett_rrc_TimeslotISCP_List,
    &ett_rrc_TimeslotListWithISCP,
    &ett_rrc_TimeslotWithISCP,
    &ett_rrc_TraceReference,
    &ett_rrc_TraceRecordingSession,
    &ett_rrc_TrafficVolumeEventParam,
    &ett_rrc_TrafficVolumeEventResults,
    &ett_rrc_TrafficVolumeMeasQuantity,
    &ett_rrc_TrafficVolumeMeasSysInfo,
    &ett_rrc_TrafficVolumeMeasuredResults,
    &ett_rrc_TrafficVolumeMeasuredResultsList,
    &ett_rrc_TrafficVolumeMeasurement,
    &ett_rrc_TrafficVolumeMeasurementObjectList,
    &ett_rrc_TrafficVolumeReportCriteria,
    &ett_rrc_TrafficVolumeReportCriteriaSysInfo,
    &ett_rrc_TrafficVolumeReportingCriteria,
    &ett_rrc_TrafficVolumeReportingQuantity,
    &ett_rrc_TransChCriteria,
    &ett_rrc_SEQUENCE_SIZE_1_maxMeasParEvent_OF_TrafficVolumeEventParam,
    &ett_rrc_TransChCriteriaList,
    &ett_rrc_UE_6AB_Event,
    &ett_rrc_UE_6FG_Event,
    &ett_rrc_UE_AutonomousUpdateMode,
    &ett_rrc_UE_InternalEventParam,
    &ett_rrc_UE_InternalEventParamList,
    &ett_rrc_UE_InternalEventResults,
    &ett_rrc_UE_InternalMeasQuantity,
    &ett_rrc_UE_InternalMeasuredResults,
    &ett_rrc_T_modeSpecificInfo_131,
    &ett_rrc_T_fdd_164,
    &ett_rrc_T_tdd_113,
    &ett_rrc_UE_InternalMeasuredResults_LCR_r4,
    &ett_rrc_UE_InternalMeasuredResults_v770ext,
    &ett_rrc_T_modeSpecificInfo_132,
    &ett_rrc_T_tdd384_768_02,
    &ett_rrc_T_tdd128_66,
    &ett_rrc_UE_InternalMeasurement,
    &ett_rrc_UE_InternalMeasurement_r4,
    &ett_rrc_UE_InternalMeasurementSysInfo,
    &ett_rrc_UE_InternalReportCriteria,
    &ett_rrc_UE_InternalReportingCriteria,
    &ett_rrc_UE_InternalReportingQuantity,
    &ett_rrc_T_modeSpecificInfo_133,
    &ett_rrc_T_fdd_165,
    &ett_rrc_T_tdd_114,
    &ett_rrc_UE_InternalReportingQuantity_r4,
    &ett_rrc_T_modeSpecificInfo_134,
    &ett_rrc_T_fdd_166,
    &ett_rrc_T_tdd_115,
    &ett_rrc_T_tddOption_28,
    &ett_rrc_T_tdd384_43,
    &ett_rrc_T_tdd128_67,
    &ett_rrc_UE_RX_TX_ReportEntry,
    &ett_rrc_UE_RX_TX_ReportEntryList,
    &ett_rrc_UE_RX_TX_TimeDifferenceType2Info,
    &ett_rrc_UE_TransmittedPowerTDD_List,
    &ett_rrc_UL_TrCH_Identity,
    &ett_rrc_UE_Positioning_CipherParameters,
    &ett_rrc_UE_Positioning_DGANSSCorrections,
    &ett_rrc_UE_Positioning_DGANSSCorrections_r9,
    &ett_rrc_UE_Positioning_DGANSSCorrections_v920ext,
    &ett_rrc_UE_Positioning_Error,
    &ett_rrc_UE_Positioning_Error_r7,
    &ett_rrc_UE_Positioning_Error_v860ext,
    &ett_rrc_UE_Positioning_EventParam,
    &ett_rrc_UE_Positioning_EventParam_r7,
    &ett_rrc_UE_Positioning_EventParamList,
    &ett_rrc_UE_Positioning_EventParamList_r7,
    &ett_rrc_UE_Positioning_EventSpecificInfo,
    &ett_rrc_UE_Positioning_EventSpecificInfo_r7,
    &ett_rrc_UE_Positioning_GANSS_AddUTCModels,
    &ett_rrc_UE_Positioning_GANSS_Almanac,
    &ett_rrc_UE_Positioning_GANSS_Almanac_v860ext,
    &ett_rrc_UE_Positioning_GANSS_Almanac_r8,
    &ett_rrc_UE_Positioning_GANSS_Almanac_va40ext,
    &ett_rrc_UE_Positioning_GANSS_Almanac_r10,
    &ett_rrc_UE_Positioning_GANSS_AdditionalAssistanceDataRequest,
    &ett_rrc_UE_Positioning_GANSS_AdditionalAssistanceDataRequest_v860ext,
    &ett_rrc_UE_Positioning_GANSS_AddIonoModel,
    &ett_rrc_UE_Positioning_GANSS_AddNavigationModels,
    &ett_rrc_UE_Positioning_GANSS_AssistanceData,
    &ett_rrc_UE_Positioning_GANSS_AssistanceData_v860ext,
    &ett_rrc_UE_Positioning_GANSS_AssistanceData_r8,
    &ett_rrc_UE_Positioning_GANSS_AssistanceData_r9,
    &ett_rrc_UE_Positioning_GANSS_AssistanceData_v920ext,
    &ett_rrc_UE_Positioning_GANSS_AssistanceData_va40ext,
    &ett_rrc_UE_Positioning_GANSS_AssistanceData_r10,
    &ett_rrc_UE_Positioning_GANSS_AuxiliaryInfo,
    &ett_rrc_UE_Positioning_GANSS_ClockModel,
    &ett_rrc_UE_Positioning_GANSS_AddClockModels,
    &ett_rrc_UE_Positioning_GANSS_AddOrbitModels,
    &ett_rrc_UE_Positioning_GANSS_Data_Bit_Assistance,
    &ett_rrc_UE_Positioning_GANSS_EarthOrientPara,
    &ett_rrc_UE_Positioning_GANSS_IonosphericModel,
    &ett_rrc_UE_Positioning_GANSS_MeasuredResults,
    &ett_rrc_T_referenceTime,
    &ett_rrc_UE_Positioning_GANSS_MeasuredResults_v860ext,
    &ett_rrc_UE_Positioning_GANSS_NavigationModel,
    &ett_rrc_UE_Positioning_GANSS_OrbitModel,
    &ett_rrc_UE_Positioning_GANSS_RealTimeIntegrity,
    &ett_rrc_UE_Positioning_GANSS_RealTimeIntegrity_item,
    &ett_rrc_UE_Positioning_GANSS_ReferenceMeasurementInfo,
    &ett_rrc_UE_Positioning_GANSS_ReferenceMeasurementInfo_va40ext,
    &ett_rrc_UE_Positioning_GANSS_ReferenceMeasurementInfo_r10,
    &ett_rrc_UE_Positioning_GANSS_ReferenceTime,
    &ett_rrc_T_utran_ganssreferenceTime,
    &ett_rrc_T_mode,
    &ett_rrc_T_fdd_167,
    &ett_rrc_T_tdd_116,
    &ett_rrc_UE_Positioning_GANSS_ReferenceTime_va40ext,
    &ett_rrc_UE_Positioning_GANSS_ReferenceTime_r10,
    &ett_rrc_T_utran_ganssreferenceTime_01,
    &ett_rrc_T_mode_01,
    &ett_rrc_T_fdd_168,
    &ett_rrc_T_tdd_117,
    &ett_rrc_UE_Positioning_GANSS_TimeModel,
    &ett_rrc_UE_Positioning_GANSS_TimeModel_va40ext,
    &ett_rrc_UE_Positioning_GANSS_TimeModel_r10,
    &ett_rrc_UE_Positioning_GANSS_TimeModels,
    &ett_rrc_UE_Positioning_GANSS_TimeModels_va40ext,
    &ett_rrc_UE_Positioning_GANSS_UTCModel,
    &ett_rrc_UE_Positioning_GPS_AcquisitionAssistance,
    &ett_rrc_UE_Positioning_GPS_AcquisitionAssistance_v770ext,
    &ett_rrc_UE_Positioning_GPS_AcquisitionAssistance_r7,
    &ett_rrc_UE_Positioning_GPS_AcquisitionAssistance_va40ext,
    &ett_rrc_UE_Positioning_GPS_AcquisitionAssistance_r10,
    &ett_rrc_UE_Positioning_GPS_AdditionalAssistanceDataRequest,
    &ett_rrc_UE_Positioning_GPS_Almanac,
    &ett_rrc_UE_Positioning_GPS_Almanac_va40ext,
    &ett_rrc_UE_Positioning_GPS_Almanac_r10,
    &ett_rrc_UE_Positioning_GPS_AssistanceData,
    &ett_rrc_UE_Positioning_GPS_AssistanceData_v770ext,
    &ett_rrc_UE_Positioning_GPS_AssistanceData_r7,
    &ett_rrc_UE_Positioning_GPS_AssistanceData_r9,
    &ett_rrc_UE_Positioning_GPS_AssistanceData_v920ext,
    &ett_rrc_UE_Positioning_GPS_AssistanceData_va40ext,
    &ett_rrc_UE_Positioning_GPS_AssistanceData_r10,
    &ett_rrc_UE_Positioning_GPS_DGPS_Corrections,
    &ett_rrc_UE_Positioning_GPS_DGPS_Corrections_r9,
    &ett_rrc_UE_Positioning_GPS_DGPS_Corrections_v920ext,
    &ett_rrc_UE_Positioning_GPS_IonosphericModel,
    &ett_rrc_UE_Positioning_GPS_MeasurementResults,
    &ett_rrc_T_referenceTime_01,
    &ett_rrc_UE_Positioning_GPS_MeasurementResults_v770ext,
    &ett_rrc_UE_Positioning_GPS_NavigationModel,
    &ett_rrc_UE_Positioning_GPS_NavModelAddDataReq,
    &ett_rrc_UE_Positioning_GPS_ReferenceCellInfo,
    &ett_rrc_T_modeSpecificInfo_135,
    &ett_rrc_T_fdd_169,
    &ett_rrc_T_tdd_118,
    &ett_rrc_UE_Positioning_GPS_ReferenceTime,
    &ett_rrc_UE_Positioning_GPS_ReferenceTime_v770ext,
    &ett_rrc_UE_Positioning_GPS_ReferenceTime_r7,
    &ett_rrc_UE_Positioning_GPS_ReferenceTime_va40ext,
    &ett_rrc_UE_Positioning_GPS_ReferenceTime_r10,
    &ett_rrc_UE_Positioning_GPS_UTC_Model,
    &ett_rrc_UE_Positioning_IPDL_Parameters,
    &ett_rrc_UE_Positioning_IPDL_Parameters_r4,
    &ett_rrc_T_modeSpecificInfo_136,
    &ett_rrc_T_fdd_170,
    &ett_rrc_T_tdd_119,
    &ett_rrc_UE_Positioning_IPDL_Parameters_TDD_r4_ext,
    &ett_rrc_UE_Positioning_MeasuredResults,
    &ett_rrc_UE_Positioning_MeasuredResults_v770ext,
    &ett_rrc_UE_Positioning_MeasuredResults_v390ext,
    &ett_rrc_UE_Positioning_MeasuredResults_v860ext,
    &ett_rrc_UE_Positioning_Measurement,
    &ett_rrc_UE_Positioning_Measurement_v390ext,
    &ett_rrc_UE_Positioning_Measurement_r4,
    &ett_rrc_UE_Positioning_Measurement_r7,
    &ett_rrc_UE_Positioning_Measurement_r8,
    &ett_rrc_UE_Positioning_Measurement_r9,
    &ett_rrc_UE_Positioning_Measurement_r10,
    &ett_rrc_UE_Positioning_MeasurementEventResults,
    &ett_rrc_UE_Positioning_MeasurementEventResults_v770ext,
    &ett_rrc_UE_Positioning_MeasurementEventResults_v860ext,
    &ett_rrc_UE_Positioning_OTDOA_AssistanceData,
    &ett_rrc_UE_Positioning_OTDOA_AssistanceData_r4,
    &ett_rrc_UE_Positioning_OTDOA_AssistanceData_r4ext,
    &ett_rrc_UE_Positioning_OTDOA_AssistanceData_UEB,
    &ett_rrc_UE_Positioning_OTDOA_AssistanceData_UEB_ext,
    &ett_rrc_UE_Positioning_OTDOA_AssistanceData_r7,
    &ett_rrc_UE_Positioning_IPDL_Parameters_TDDList_r4_ext,
    &ett_rrc_UE_Positioning_OTDOA_MeasuredResultsTDD_ext,
    &ett_rrc_UE_Positioning_OTDOA_Measurement,
    &ett_rrc_T_modeSpecificInfo_137,
    &ett_rrc_T_fdd_171,
    &ett_rrc_T_tdd_120,
    &ett_rrc_UE_Positioning_OTDOA_Measurement_v390ext,
    &ett_rrc_UE_Positioning_OTDOA_NeighbourCellInfo,
    &ett_rrc_T_modeSpecificInfo_138,
    &ett_rrc_T_fdd_172,
    &ett_rrc_T_tdd_121,
    &ett_rrc_T_positioningMode,
    &ett_rrc_T_ueBased,
    &ett_rrc_T_ueAssisted,
    &ett_rrc_UE_Positioning_OTDOA_NeighbourCellInfo_r4,
    &ett_rrc_T_modeSpecificInfo_139,
    &ett_rrc_T_fdd_173,
    &ett_rrc_T_tdd_122,
    &ett_rrc_T_positioningMode_01,
    &ett_rrc_T_ueBased_01,
    &ett_rrc_T_ueAssisted_01,
    &ett_rrc_UE_Positioning_OTDOA_NeighbourCellInfo_UEB,
    &ett_rrc_T_modeSpecificInfo_140,
    &ett_rrc_T_fdd_174,
    &ett_rrc_T_tdd_123,
    &ett_rrc_UE_Positioning_OTDOA_NeighbourCellInfo_UEB_ext,
    &ett_rrc_UE_Positioning_OTDOA_NeighbourCellInfo_r7,
    &ett_rrc_T_modeSpecificInfo_141,
    &ett_rrc_T_fdd_175,
    &ett_rrc_T_tdd_124,
    &ett_rrc_T_positioningMode_02,
    &ett_rrc_T_ueBased_02,
    &ett_rrc_T_ueAssisted_02,
    &ett_rrc_UE_Positioning_OTDOA_NeighbourCellList,
    &ett_rrc_UE_Positioning_OTDOA_NeighbourCellList_r4,
    &ett_rrc_UE_Positioning_OTDOA_NeighbourCellList_UEB,
    &ett_rrc_UE_Positioning_OTDOA_NeighbourCellList_UEB_ext,
    &ett_rrc_UE_Positioning_OTDOA_NeighbourCellList_r7,
    &ett_rrc_UE_Positioning_OTDOA_Quality,
    &ett_rrc_UE_Positioning_OTDOA_ReferenceCellInfo,
    &ett_rrc_T_modeSpecificInfo_142,
    &ett_rrc_T_fdd_176,
    &ett_rrc_T_tdd_125,
    &ett_rrc_T_positioningMode_03,
    &ett_rrc_T_ueBased_03,
    &ett_rrc_T_ueAssisted_03,
    &ett_rrc_UE_Positioning_OTDOA_ReferenceCellInfo_r4,
    &ett_rrc_T_modeSpecificInfo_143,
    &ett_rrc_T_fdd_177,
    &ett_rrc_T_tdd_126,
    &ett_rrc_T_positioningMode_04,
    &ett_rrc_T_ueBased_04,
    &ett_rrc_T_ueAssisted_04,
    &ett_rrc_UE_Positioning_OTDOA_ReferenceCellInfo_UEB,
    &ett_rrc_T_modeSpecificInfo_144,
    &ett_rrc_T_fdd_178,
    &ett_rrc_T_tdd_127,
    &ett_rrc_UE_Positioning_OTDOA_ReferenceCellInfo_UEB_ext,
    &ett_rrc_UE_Positioning_OTDOA_ReferenceCellInfo_r7,
    &ett_rrc_T_modeSpecificInfo_145,
    &ett_rrc_T_fdd_179,
    &ett_rrc_T_tdd_128,
    &ett_rrc_T_positioningMode_05,
    &ett_rrc_T_ueBased_05,
    &ett_rrc_T_ueAssisted_05,
    &ett_rrc_UE_Positioning_PositionEstimateInfo,
    &ett_rrc_T_referenceTime_02,
    &ett_rrc_T_cell_Timing,
    &ett_rrc_T_modeSpecificInfo_146,
    &ett_rrc_T_fdd_180,
    &ett_rrc_T_tdd_129,
    &ett_rrc_UE_Positioning_PositionEstimateInfo_v770ext,
    &ett_rrc_T_referenceTimeOptions,
    &ett_rrc_T_r7_15,
    &ett_rrc_T_referenceTime_03,
    &ett_rrc_UE_Positioning_ReportCriteria,
    &ett_rrc_UE_Positioning_ReportCriteria_r7,
    &ett_rrc_UE_Positioning_ReportingQuantity,
    &ett_rrc_UE_Positioning_ReportingQuantity_v390ext,
    &ett_rrc_UE_Positioning_ReportingQuantity_r4,
    &ett_rrc_UE_Positioning_ReportingQuantity_r7,
    &ett_rrc_UE_Positioning_ReportingQuantity_r8,
    &ett_rrc_UTCmodelSet1,
    &ett_rrc_UTCmodelSet2,
    &ett_rrc_UTCmodelSet3,
    &ett_rrc_UTRAN_GANSSReferenceTime,
    &ett_rrc_T_ue_GANSSTimingOfCellFrames,
    &ett_rrc_T_mode_02,
    &ett_rrc_T_fdd_181,
    &ett_rrc_T_tdd_130,
    &ett_rrc_UTRAN_FDD_FrequencyList,
    &ett_rrc_UTRAN_FDD_Frequency,
    &ett_rrc_UTRAN_TDD_FrequencyList,
    &ett_rrc_UTRAN_TDD_Frequency,
    &ett_rrc_UTRAN_GPSReferenceTime,
    &ett_rrc_T_utran_GPSTimingOfCell,
    &ett_rrc_T_modeSpecificInfo_147,
    &ett_rrc_T_fdd_182,
    &ett_rrc_T_tdd_131,
    &ett_rrc_UTRAN_GPSReferenceTimeResult,
    &ett_rrc_T_ue_GPSTimingOfCell,
    &ett_rrc_T_modeSpecificInfo_148,
    &ett_rrc_T_fdd_183,
    &ett_rrc_T_tdd_132,
    &ett_rrc_UTRA_PriorityInfoList,
    &ett_rrc_T_utra_ServingCell,
    &ett_rrc_UTRA_PriorityInfoList_v920ext,
    &ett_rrc_VelocityEstimate,
    &ett_rrc_BCCH_ModificationInfo,
    &ett_rrc_BSIC,
    &ett_rrc_CBS_DRX_Level1Information,
    &ett_rrc_CDMA2000_Message,
    &ett_rrc_CDMA2000_MessageList,
    &ett_rrc_CDMA2000_UMTS_Frequency_List,
    &ett_rrc_DataTransmFrequency,
    &ett_rrc_DataVolumeHistory,
    &ett_rrc_DataVolumePerRB,
    &ett_rrc_DataVolumePerRB_List,
    &ett_rrc_DeferredMeasurementControlReadingSupport,
    &ett_rrc_T_modeSpecificInfo_149,
    &ett_rrc_T_fdd_184,
    &ett_rrc_T_tdd_133,
    &ett_rrc_ETWS_Information,
    &ett_rrc_EUTRA_BlacklistedCell,
    &ett_rrc_EUTRA_BlacklistedCellPerFreqList,
    &ett_rrc_EUTRA_RadioAccessCapability,
    &ett_rrc_EUTRA_TargetFreqInfoList,
    &ett_rrc_EUTRA_TargetFreqInfo,
    &ett_rrc_ExtGANSS_SchedulingInfo,
    &ett_rrc_ExtGANSS_SIBTypeInfoSchedulingInfoList,
    &ett_rrc_ExtGANSS_SIBTypeInfoSchedulingInfo,
    &ett_rrc_SEQUENCE_SIZE_1_maxSIB_OF_ExtGANSS_SchedulingInfo,
    &ett_rrc_ExtSIBTypeInfoSchedulingInfo_List,
    &ett_rrc_ExtSIBTypeInfoSchedulingInfo,
    &ett_rrc_ExtSIBTypeInfoSchedulingInfo_List2,
    &ett_rrc_ExtSIBTypeInfoSchedulingInfo2,
    &ett_rrc_FDD_UMTS_Frequency_List,
    &ett_rrc_FrequencyInfoCDMA2000,
    &ett_rrc_GERAN_SystemInformation,
    &ett_rrc_GSM_BA_Range,
    &ett_rrc_GSM_BA_Range_List,
    &ett_rrc_GSM_MessageList,
    &ett_rrc_GsmSecurityCapability,
    &ett_rrc_GSM_TargetCellInfoList,
    &ett_rrc_GSM_TargetCellInfo,
    &ett_rrc_IdentificationOfReceivedMessage,
    &ett_rrc_InterRAT_ChangeFailureCause,
    &ett_rrc_GERANIu_MessageList,
    &ett_rrc_InterRAT_UE_RadioAccessCapability,
    &ett_rrc_T_gsm_15,
    &ett_rrc_T_cdma2000_01,
    &ett_rrc_InterRAT_UE_RadioAccessCapabilityList,
    &ett_rrc_InterRAT_UE_RadioAccessCapability_v590ext,
    &ett_rrc_InterRAT_UE_RadioAccessCapability_v690ext,
    &ett_rrc_InterRAT_UE_RadioAccessCapability_v860ext,
    &ett_rrc_InterRAT_UE_SecurityCapability,
    &ett_rrc_T_gsm_16,
    &ett_rrc_InterRAT_UE_SecurityCapList,
    &ett_rrc_InterRAT_HO_FailureCause,
    &ett_rrc_MasterInformationBlock,
    &ett_rrc_T_v690NonCriticalExtensions_48,
    &ett_rrc_T_v6b0NonCriticalExtensions_15,
    &ett_rrc_T_v860NonCriticalExtensions_26,
    &ett_rrc_T_nonCriticalExtensions_195,
    &ett_rrc_MasterInformationBlock_v690ext,
    &ett_rrc_MasterInformationBlock_v6b0ext_IEs,
    &ett_rrc_MasterInformationBlock_v860ext_IEs,
    &ett_rrc_PredefinedConfigIdentityAndValueTag,
    &ett_rrc_ProtocolErrorInformation,
    &ett_rrc_T_diagnosticsType_01,
    &ett_rrc_T_type1_07,
    &ett_rrc_Rplmn_Information,
    &ett_rrc_Rplmn_Information_r4,
    &ett_rrc_SchedulingInformation,
    &ett_rrc_T_scheduling,
    &ett_rrc_T_sib_Pos,
    &ett_rrc_SchedulingInformationSIB,
    &ett_rrc_SchedulingInformationSIBSb,
    &ett_rrc_SIBOccurrenceIdentityAndValueTag,
    &ett_rrc_SIB_ReferenceList,
    &ett_rrc_SIBSb_ReferenceList,
    &ett_rrc_SIB_ReferenceListFACH,
    &ett_rrc_SIB_TypeAndTag,
    &ett_rrc_SIBSb_TypeAndTag,
    &ett_rrc_SIB_TypeExt,
    &ett_rrc_SIB_TypeExt2,
    &ett_rrc_SIB_TypeExtGANSS,
    &ett_rrc_SibOFF_List,
    &ett_rrc_SysInfoType1,
    &ett_rrc_T_v3a0NonCriticalExtensions_12,
    &ett_rrc_T_v860NonCriticalExtentions_02,
    &ett_rrc_T_nonCriticalExtensions_196,
    &ett_rrc_SysInfoType1_v3a0ext_IEs,
    &ett_rrc_SysInfoType1_v860ext_IEs,
    &ett_rrc_SysInfoType2,
    &ett_rrc_T_nonCriticalExtensions_197,
    &ett_rrc_SysInfoType3,
    &ett_rrc_T_v4b0NonCriticalExtensions_17,
    &ett_rrc_T_v590NonCriticalExtension,
    &ett_rrc_T_v5c0NoncriticalExtension,
    &ett_rrc_T_v670NonCriticalExtension,
    &ett_rrc_T_v770NonCriticalExtension_02,
    &ett_rrc_T_v830NonCriticalExtension,
    &ett_rrc_T_v860NonCriticalExtension_01,
    &ett_rrc_T_v870NonCriticalExtension,
    &ett_rrc_T_v900NonCriticalExtension,
    &ett_rrc_T_nonCriticalExtensions_198,
    &ett_rrc_SysInfoType3_v4b0ext_IEs,
    &ett_rrc_SysInfoType3_v590ext,
    &ett_rrc_SysInfoType3_v5c0ext_IEs,
    &ett_rrc_SysInfoType3_v670ext,
    &ett_rrc_SysInfoType3_v770ext_IEs,
    &ett_rrc_SysInfoType3_v830ext_IEs,
    &ett_rrc_SysInfoType3_v860ext_IEs,
    &ett_rrc_SysInfoType3_v870ext_IEs,
    &ett_rrc_SysInfoType3_v920ext_IEs,
    &ett_rrc_SysInfoType4,
    &ett_rrc_T_v4b0NonCriticalExtensions_18,
    &ett_rrc_T_v590NonCriticalExtension_01,
    &ett_rrc_T_v5b0NonCriticalExtension,
    &ett_rrc_T_v5c0NonCriticalExtension,
    &ett_rrc_T_nonCriticalExtensions_199,
    &ett_rrc_SysInfoType4_v4b0ext_IEs,
    &ett_rrc_SysInfoType4_v590ext,
    &ett_rrc_SysInfoType4_v5b0ext_IEs,
    &ett_rrc_SysInfoType4_v5c0ext_IEs,
    &ett_rrc_SysInfoType5,
    &ett_rrc_T_modeSpecificInfo_150,
    &ett_rrc_T_fdd_185,
    &ett_rrc_T_tdd_134,
    &ett_rrc_T_v4b0NonCriticalExtensions_19,
    &ett_rrc_T_v590NonCriticalExtensions_28,
    &ett_rrc_T_v650NonCriticalExtensions,
    &ett_rrc_T_v680NonCriticalExtensions_02,
    &ett_rrc_T_v690NonCriticalExtensions_49,
    &ett_rrc_T_v770NonCriticalExtensions_40,
    &ett_rrc_T_v860NonCriticalExtensions_27,
    &ett_rrc_T_v890NonCriticalExtensions_11,
    &ett_rrc_T_v8b0NonCriticalExtensions_01,
    &ett_rrc_T_v8d0NonCriticalExtensions,
    &ett_rrc_T_va40NonCriticalExtensions_18,
    &ett_rrc_T_va80NonCriticalExtensions_02,
    &ett_rrc_SysInfoType5_v4b0ext_IEs,
    &ett_rrc_T_tdd128SpecificInfo,
    &ett_rrc_SysInfoType5_v590ext_IEs,
    &ett_rrc_T_hcr_r5_SpecificInfo,
    &ett_rrc_SysInfoType5_v650ext_IEs,
    &ett_rrc_SysInfoType5_v680ext_IEs,
    &ett_rrc_SysInfoType5_v690ext_IEs,
    &ett_rrc_T_sccpch_SystemInformation_MBMS,
    &ett_rrc_SysInfoType5_v770ext_IEs,
    &ett_rrc_T_modeSpecificInfo_151,
    &ett_rrc_T_fdd_186,
    &ett_rrc_T_tdd768_24,
    &ett_rrc_T_tdd128_68,
    &ett_rrc_T_sccpch_SystemInformation_MBMS_01,
    &ett_rrc_SysInfoType5_v860ext_IEs,
    &ett_rrc_T_dummy_15,
    &ett_rrc_T_modeSpecificInfo_152,
    &ett_rrc_T_fdd_187,
    &ett_rrc_T_tdd128_69,
    &ett_rrc_T_hs_dsch_CommonSysInfo,
    &ett_rrc_SysInfoType5_v890ext_IEs,
    &ett_rrc_SysInfoType5_v8b0ext_IEs,
    &ett_rrc_SysInfoType5_v8d0ext_IEs,
    &ett_rrc_SysInfoType5_va40ext_IEs,
    &ett_rrc_SysInfoType5_va80ext_IEs,
    &ett_rrc_SysInfoType5_NonCriticalExtensions_vbxy_IEs,
    &ett_rrc_T_nonCriticalExtensions_200,
    &ett_rrc_SysInfoType5_vbxyext_IEs,
    &ett_rrc_SysInfoType6,
    &ett_rrc_T_modeSpecificInfo_153,
    &ett_rrc_T_fdd_188,
    &ett_rrc_T_tdd_135,
    &ett_rrc_T_v4b0NonCriticalExtensions_20,
    &ett_rrc_T_v590NonCriticalExtensions_29,
    &ett_rrc_T_v650nonCriticalExtensions,
    &ett_rrc_T_v690nonCriticalExtensions,
    &ett_rrc_T_v770NonCriticalExtensions_41,
    &ett_rrc_T_va40NonCriticalExtensions_19,
    &ett_rrc_T_va80NonCriticalExtensions_03,
    &ett_rrc_T_nonCriticalExtensions_201,
    &ett_rrc_SysInfoType6_v4b0ext_IEs,
    &ett_rrc_T_tdd128SpecificInfo_01,
    &ett_rrc_SysInfoType6_v590ext_IEs,
    &ett_rrc_T_hcr_r5_SpecificInfo_01,
    &ett_rrc_SysInfoType6_v650ext_IEs,
    &ett_rrc_SysInfoType6_v690ext_IEs,
    &ett_rrc_SysInfoType6_v770ext_IEs,
    &ett_rrc_T_tdd768SpecificInfo,
    &ett_rrc_SysInfoType6_va40ext_IEs,
    &ett_rrc_SysInfoType6_va80ext_IEs,
    &ett_rrc_SysInfoType7,
    &ett_rrc_T_modeSpecificInfo_154,
    &ett_rrc_T_fdd_189,
    &ett_rrc_T_nonCriticalExtensions_202,
    &ett_rrc_SysInfoType8,
    &ett_rrc_T_nonCriticalExtensions_203,
    &ett_rrc_SysInfoType9,
    &ett_rrc_T_nonCriticalExtensions_204,
    &ett_rrc_SysInfoType10,
    &ett_rrc_T_nonCriticalExtensions_205,
    &ett_rrc_SysInfoType11,
    &ett_rrc_T_v4b0NonCriticalExtensions_21,
    &ett_rrc_T_v590NonCriticalExtension_02,
    &ett_rrc_T_v690NonCriticalExtensions_50,
    &ett_rrc_T_v6b0NonCriticalExtensions_16,
    &ett_rrc_T_v770NonCriticalExtensions_42,
    &ett_rrc_T_v7b0NonCriticalExtensions_02,
    &ett_rrc_T_v860NonCriticalExtensions_28,
    &ett_rrc_T_va80NonCriticalExtensions_04,
    &ett_rrc_T_nonCriticalExtensions_206,
    &ett_rrc_SysInfoType11_v4b0ext_IEs,
    &ett_rrc_SysInfoType11_v590ext_IEs,
    &ett_rrc_SysInfoType11_v690ext_IEs,
    &ett_rrc_SysInfoType11_v6b0ext_IEs,
    &ett_rrc_SysInfoType11_v770ext_IEs,
    &ett_rrc_SysInfoType11_v7b0ext_IEs,
    &ett_rrc_SysInfoType11_v860ext_IEs,
    &ett_rrc_SysInfoType11_va80ext_IEs,
    &ett_rrc_SysInfoType11bis,
    &ett_rrc_T_v7b0NonCriticalExtensions_03,
    &ett_rrc_T_v860NonCriticalExtensions_29,
    &ett_rrc_T_va80NonCriticalExtensions_05,
    &ett_rrc_T_nonCriticalExtensions_207,
    &ett_rrc_SysInfoType11bis_v7b0ext_IEs,
    &ett_rrc_SysInfoType11bis_v860ext_IEs,
    &ett_rrc_SysInfoType11bis_va80ext_IEs,
    &ett_rrc_SysInfoType12,
    &ett_rrc_T_v4b0NonCriticalExtensions_22,
    &ett_rrc_T_v590NonCriticalExtension_03,
    &ett_rrc_T_v690NonCriticalExtensions_51,
    &ett_rrc_T_v6b0NonCriticalExtensions_17,
    &ett_rrc_T_v7b0NonCriticalExtensions_04,
    &ett_rrc_T_va80NonCriticalExtensions_06,
    &ett_rrc_T_nonCriticalExtensions_208,
    &ett_rrc_SysInfoType12_v4b0ext_IEs,
    &ett_rrc_SysInfoType12_v590ext_IEs,
    &ett_rrc_SysInfoType12_v690ext_IEs,
    &ett_rrc_SysInfoType12_v6b0ext_IEs,
    &ett_rrc_SysInfoType12_v7b0ext_IEs,
    &ett_rrc_SysInfoType12_va80ext_IEs,
    &ett_rrc_SysInfoType13,
    &ett_rrc_T_v3a0NonCriticalExtensions_13,
    &ett_rrc_T_v4b0NonCriticalExtensions_23,
    &ett_rrc_T_v770NonCriticalExtensions_43,
    &ett_rrc_T_nonCriticalExtensions_209,
    &ett_rrc_SysInfoType13_v3a0ext_IEs,
    &ett_rrc_SysInfoType13_v4b0ext_IEs,
    &ett_rrc_SysInfoType13_v770ext_IEs,
    &ett_rrc_SysInfoType13_1,
    &ett_rrc_T_nonCriticalExtensions_210,
    &ett_rrc_SysInfoType13_2,
    &ett_rrc_T_nonCriticalExtensions_211,
    &ett_rrc_SysInfoType13_3,
    &ett_rrc_T_nonCriticalExtensions_212,
    &ett_rrc_SysInfoType13_4,
    &ett_rrc_T_nonCriticalExtensions_213,
    &ett_rrc_SysInfoType14,
    &ett_rrc_T_nonCriticalExtensions_214,
    &ett_rrc_SysInfoType15,
    &ett_rrc_T_v4b0NonCriticalExtensions_24,
    &ett_rrc_T_v770NonCriticalExtensions_44,
    &ett_rrc_T_va40NonCriticalExtensions_20,
    &ett_rrc_T_nonCriticalExtensions_215,
    &ett_rrc_SysInfoType15_v4b0ext_IEs,
    &ett_rrc_SysInfoType15_v770ext_IEs,
    &ett_rrc_SysInfoType15_va40ext_IEs,
    &ett_rrc_SysInfoType15bis,
    &ett_rrc_T_v860NonCriticalExtensions_30,
    &ett_rrc_T_va40NonCriticalExtensions_21,
    &ett_rrc_T_nonCriticalExtensions_216,
    &ett_rrc_SysInfoType15bis_v860ext_IEs,
    &ett_rrc_SysInfoType15bis_va40ext_IEs,
    &ett_rrc_SysInfoType15_1,
    &ett_rrc_T_v920NonCriticalExtensions_04,
    &ett_rrc_T_nonCriticalExtensions_217,
    &ett_rrc_SysInfoType15_1_v920ext_IEs,
    &ett_rrc_SysInfoType15_1bis,
    &ett_rrc_T_v920NonCriticalExtensions_05,
    &ett_rrc_T_nonCriticalExtensions_218,
    &ett_rrc_SysInfoType15_1bis_v920ext_IEs,
    &ett_rrc_SysInfoType15_2,
    &ett_rrc_T_nonCriticalExtensions_219,
    &ett_rrc_SysInfoType15_2bis,
    &ett_rrc_T_nonCriticalExtensions_220,
    &ett_rrc_SysInfoType15_2ter,
    &ett_rrc_T_nonCriticalExtensions_221,
    &ett_rrc_SysInfoType15_3,
    &ett_rrc_T_va40NonCriticalExtensions_22,
    &ett_rrc_T_nonCriticalExtensions_222,
    &ett_rrc_SysInfoType15_3_va40ext_IEs,
    &ett_rrc_SysInfoType15_3bis,
    &ett_rrc_T_v860NonCriticalExtensions_31,
    &ett_rrc_T_va40NonCriticalExtensions_23,
    &ett_rrc_T_nonCriticalExtensions_223,
    &ett_rrc_SysInfoType15_3bis_v860ext_IEs,
    &ett_rrc_SysInfoType15_3bis_va40ext_IEs,
    &ett_rrc_SysInfoType15_4,
    &ett_rrc_T_v3a0NonCriticalExtensions_14,
    &ett_rrc_T_v4b0NonCriticalExtensions_25,
    &ett_rrc_T_nonCriticalExtensions_224,
    &ett_rrc_SysInfoType15_4_v3a0ext,
    &ett_rrc_SysInfoType15_4_v4b0ext,
    &ett_rrc_SysInfoType15_5,
    &ett_rrc_T_v3a0NonCriticalExtensions_15,
    &ett_rrc_T_v770NonCriticalExtensions_45,
    &ett_rrc_T_nonCriticalExtensions_225,
    &ett_rrc_SysInfoType15_5_v3a0ext,
    &ett_rrc_SysInfoType15_5_v770ext_IEs,
    &ett_rrc_SysInfoType15_6,
    &ett_rrc_T_va40NonCriticalExtensions_24,
    &ett_rrc_T_nonCriticalExtensions_226,
    &ett_rrc_SysInfoType15_6_va40ext_IEs,
    &ett_rrc_SysInfoType15_7,
    &ett_rrc_T_nonCriticalExtensions_227,
    &ett_rrc_SysInfoType15_8,
    &ett_rrc_T_nonCriticalExtensions_228,
    &ett_rrc_SysInfoType16,
    &ett_rrc_T_v770NonCriticalExtensions_46,
    &ett_rrc_T_v920NonCriticalExtensions_06,
    &ett_rrc_T_nonCriticalExtensions_229,
    &ett_rrc_SysInfoType16_v770ext_IEs,
    &ett_rrc_SysInfoType16_v920ext_IEs,
    &ett_rrc_SysInfoType17,
    &ett_rrc_T_v4b0NonCriticalExtensions_26,
    &ett_rrc_T_v590NonCriticalExtensions_30,
    &ett_rrc_T_v770NonCriticalExtensions_47,
    &ett_rrc_T_nonCriticalExtensions_230,
    &ett_rrc_SysInfoType17_v4b0ext_IEs,
    &ett_rrc_T_tdd128SpecificInfo_02,
    &ett_rrc_SysInfoType17_v590ext_IEs,
    &ett_rrc_T_hcr_r5_SpecificInfo_02,
    &ett_rrc_SysInfoType17_v770ext_IEs,
    &ett_rrc_T_tdd768SpecificInfo_01,
    &ett_rrc_SysInfoType18,
    &ett_rrc_T_v6b0NonCriticalExtensions_18,
    &ett_rrc_T_v860NonCriticalExtensions_32,
    &ett_rrc_T_nonCriticalExtensions_231,
    &ett_rrc_SysInfoType18_v6b0ext,
    &ett_rrc_SysInfoType18_v860ext,
    &ett_rrc_SysInfoType19,
    &ett_rrc_T_v920NonCriticalExtensions_07,
    &ett_rrc_T_va80NonCriticalExtensions_07,
    &ett_rrc_T_vb30NonCriticalExtensions,
    &ett_rrc_T_vbxyNonCriticalExtensions_19,
    &ett_rrc_T_nonCriticalExtensions_232,
    &ett_rrc_SysInfoType19_v920ext,
    &ett_rrc_SysInfoType19_va80ext,
    &ett_rrc_SysInfoType19_vb30ext,
    &ett_rrc_SysInfoType19_vbxyext,
    &ett_rrc_SysInfoType20,
    &ett_rrc_T_nonCriticalExtensions_233,
    &ett_rrc_SysInfoType21,
    &ett_rrc_T_nonCriticalExtensions_234,
    &ett_rrc_SysInfoType22,
    &ett_rrc_T_modeSpecificInfo_155,
    &ett_rrc_T_fdd_190,
    &ett_rrc_T_nonCriticalExtensions_235,
    &ett_rrc_SysInfoTypeSB1,
    &ett_rrc_T_v6b0NonCriticalExtensions_19,
    &ett_rrc_T_v860NonCriticalExtensions_33,
    &ett_rrc_T_nonCriticalExtensions_236,
    &ett_rrc_SysInfoTypeSB1_v6b0ext,
    &ett_rrc_SysInfoTypeSB1_v860ext,
    &ett_rrc_SysInfoTypeSB2,
    &ett_rrc_T_v6b0NonCriticalExtensions_20,
    &ett_rrc_T_v860NonCriticalExtensions_34,
    &ett_rrc_T_nonCriticalExtensions_237,
    &ett_rrc_SysInfoTypeSB2_v6b0ext,
    &ett_rrc_SysInfoTypeSB2_v860ext,
    &ett_rrc_TDD_UMTS_Frequency_List,
    &ett_rrc_UE_HistoryInformation,
    &ett_rrc_ValueTagInfo,
    &ett_rrc_MBMS_CommonRBInformation_r6,
    &ett_rrc_MBMS_CommonRBInformationList_r6,
    &ett_rrc_MBMS_ConnectedModeCountingScope,
    &ett_rrc_MBMS_CurrentCell_SCCPCH_r6,
    &ett_rrc_MBMS_CurrentCell_SCCPCHList_r6,
    &ett_rrc_MBMS_FACHCarryingMTCH_List,
    &ett_rrc_MBMS_JoinedInformation_r6,
    &ett_rrc_MBMS_L1CombiningSchedule_32,
    &ett_rrc_T_mtch_L1CombiningPeriodList,
    &ett_rrc_T_mtch_L1CombiningPeriodList_item,
    &ett_rrc_MBMS_L1CombiningSchedule_64,
    &ett_rrc_T_mtch_L1CombiningPeriodList_01,
    &ett_rrc_T_mtch_L1CombiningPeriodList_item_01,
    &ett_rrc_MBMS_L1CombiningSchedule_128,
    &ett_rrc_T_mtch_L1CombiningPeriodList_02,
    &ett_rrc_T_mtch_L1CombiningPeriodList_item_02,
    &ett_rrc_MBMS_L1CombiningSchedule_256,
    &ett_rrc_T_mtch_L1CombiningPeriodList_03,
    &ett_rrc_T_mtch_L1CombiningPeriodList_item_03,
    &ett_rrc_MBMS_L1CombiningSchedule_512,
    &ett_rrc_T_mtch_L1CombiningPeriodList_04,
    &ett_rrc_T_mtch_L1CombiningPeriodList_item_04,
    &ett_rrc_MBMS_L1CombiningSchedule_1024,
    &ett_rrc_T_mtch_L1CombiningPeriodList_05,
    &ett_rrc_T_mtch_L1CombiningPeriodList_item_05,
    &ett_rrc_MBMS_L1CombiningSchedule,
    &ett_rrc_MBMS_L23Configuration,
    &ett_rrc_T_sameAsCurrent,
    &ett_rrc_T_different,
    &ett_rrc_MBMS_MCCH_ConfigurationInfo_r6,
    &ett_rrc_MBMS_MICHConfigurationInfo_r6,
    &ett_rrc_T_mode_03,
    &ett_rrc_T_fdd_191,
    &ett_rrc_T_tdd384_44,
    &ett_rrc_T_tdd128_70,
    &ett_rrc_SEQUENCE_SIZE_1_2_OF_DL_TS_ChannelisationCode,
    &ett_rrc_MBMS_MICHConfigurationInfo_v770ext,
    &ett_rrc_T_mode_04,
    &ett_rrc_T_tdd384_45,
    &ett_rrc_T_tdd768_25,
    &ett_rrc_T_tdd128_71,
    &ett_rrc_MBMS_MICHConfigurationInfo_v890ext,
    &ett_rrc_T_mode_05,
    &ett_rrc_T_imb384,
    &ett_rrc_MBMS_ModifedService_r6,
    &ett_rrc_T_mbms_PreferredFrequency,
    &ett_rrc_MBMS_ModifedServiceList_r6,
    &ett_rrc_MBMS_ModifiedService_v770ext,
    &ett_rrc_MBMS_ModifiedService_LCR_v7c0ext,
    &ett_rrc_MBMS_ModifiedServiceList_v770ext,
    &ett_rrc_MBMS_ModifiedServiceList_LCR_v7c0ext,
    &ett_rrc_MBMS_MSCH_ConfigurationInfo_r6,
    &ett_rrc_MBMS_MSCHSchedulingInfo,
    &ett_rrc_MBMS_NeighbouringCellSCCPCH_r6,
    &ett_rrc_T_layer1Combining,
    &ett_rrc_T_fdd_192,
    &ett_rrc_MBMS_NeighbouringCellSCCPCH_v770ext,
    &ett_rrc_T_layer1Combining_01,
    &ett_rrc_T_fdd_193,
    &ett_rrc_MBMS_NeighbouringCellSCCPCHList_r6,
    &ett_rrc_MBMS_NeighbouringCellSCCPCHList_v770ext,
    &ett_rrc_MBMS_NetworkStandardTimeInformation_LCR_v890ext,
    &ett_rrc_MBMS_PhyChInformation_IMB384,
    &ett_rrc_MBMS_PhyChInformationList_IMB384,
    &ett_rrc_MBMS_PhyChInformation_r6,
    &ett_rrc_MBMS_PhyChInformation_r7,
    &ett_rrc_MBMS_PhyChInformationList_r6,
    &ett_rrc_MBMS_PhyChInformationList_r7,
    &ett_rrc_MBMS_PreferredFrequencyInfo_r6,
    &ett_rrc_T_layerConvergenceInformation,
    &ett_rrc_MBMS_PreferredFrequencyList_r6,
    &ett_rrc_MBMS_PTM_RBInformation_C,
    &ett_rrc_MBMS_PTM_RBInformation_CList,
    &ett_rrc_MBMS_PTM_RBInformation_N,
    &ett_rrc_MBMS_PTM_RBInformation_NList,
    &ett_rrc_MBMS_SelectedServiceInfo,
    &ett_rrc_T_status,
    &ett_rrc_MBMS_SelectedServicesListFull,
    &ett_rrc_MBMS_SelectedServicesListShort,
    &ett_rrc_MBMS_SelectedServicesShort,
    &ett_rrc_MBMS_ServiceAccessInfo_r6,
    &ett_rrc_MBMS_ServiceAccessInfoList_r6,
    &ett_rrc_MBMS_ServiceIdentity_r6,
    &ett_rrc_T_plmn_Identity,
    &ett_rrc_T_other_01,
    &ett_rrc_MBMS_ServiceSchedulingInfo_r6,
    &ett_rrc_MBMS_ServiceSchedulingInfoList_r6,
    &ett_rrc_MBMS_ServiceTransmInfo,
    &ett_rrc_MBMS_ServiceTransmInfoList,
    &ett_rrc_MBMS_SIBType5_SCCPCH_r6,
    &ett_rrc_MBMS_SIBType5_SCCPCHList_r6,
    &ett_rrc_MBMS_TimersAndCounters_r6,
    &ett_rrc_MBMS_TransmissionIdentity,
    &ett_rrc_MBMS_TranspChInfoForCCTrCh_r6,
    &ett_rrc_MBMS_TranspChInfoForEachCCTrCh_r6,
    &ett_rrc_MBMS_TranspChInfoForEachTrCh_r6,
    &ett_rrc_MBMS_TranspChInfoForTrCh_r6,
    &ett_rrc_MBMS_TrCHInformation_Curr,
    &ett_rrc_MBMS_TrCHInformation_CurrList,
    &ett_rrc_MBMS_TrCHInformation_Neighb,
    &ett_rrc_MBMS_TrCHInformation_NeighbList,
    &ett_rrc_MBMS_TrCHInformation_SIB5,
    &ett_rrc_MBMS_TrCHInformation_SIB5List,
    &ett_rrc_MBMS_UnmodifiedService_r6,
    &ett_rrc_MBMS_UnmodifiedServiceList_r6,
    &ett_rrc_MBMS_UnmodifiedService_v770ext,
    &ett_rrc_MBMS_UnmodifiedServiceList_v770ext,
    &ett_rrc_MBSFNFrequency,
    &ett_rrc_MBSFNFrequency_v860ext,
    &ett_rrc_MBSFNFrequencyList,
    &ett_rrc_MBSFNFrequencyList_v860ext,
    &ett_rrc_MBSFNservicesNotNotified_r7,
    &ett_rrc_MBSFNInterFrequencyNeighbour_r7,
    &ett_rrc_T_mbsfnServicesNotification,
    &ett_rrc_MBSFNInterFrequencyNeighbour_v860ext,
    &ett_rrc_MBSFN_InterFrequencyNeighbourList_r7,
    &ett_rrc_MBSFN_InterFrequencyNeighbourList_v860ext,
    &ett_rrc_MBSFN_TDDInformation,
    &ett_rrc_MBSFN_TDDInformation_LCR,
    &ett_rrc_MBSFN_TDDTimeSlotInfo,
    &ett_rrc_MBSFN_TDDTimeSlotInfo_LCR,
    &ett_rrc_MBSFN_TDM_Info,
    &ett_rrc_MBSFN_TDM_Info_List,
    &ett_rrc_ToTargetRNC_Container,
    &ett_rrc_TargetRNC_ToSourceRNC_Container,
    &ett_rrc_InterRATHandoverInfoWithInterRATCapabilities_r3,
    &ett_rrc_T_r3_31,
    &ett_rrc_T_v390NonCriticalExtensions_01,
    &ett_rrc_T_v690NonCriticalExtensions_52,
    &ett_rrc_T_v860NonCriticalExtensions_35,
    &ett_rrc_T_v920NonCriticalExtensions_08,
    &ett_rrc_T_nonCriticalExtensions_238,
    &ett_rrc_T_criticalExtensions_139,
    &ett_rrc_InterRATHandoverInfoWithInterRATCapabilities_r3_IEs,
    &ett_rrc_InterRATHandoverInfoWithInterRATCapabilities_v390ext_IEs,
    &ett_rrc_InterRATHandoverInfoWithInterRATCapabilities_v690ext_IEs,
    &ett_rrc_InterRATHandoverInfoWithInterRATCapabilities_v860ext_IEs,
    &ett_rrc_InterRATHandoverInfoWithInterRATCapabilities_v920ext_IEs,
    &ett_rrc_RFC3095_ContextInfo_r5,
    &ett_rrc_T_r5_14,
    &ett_rrc_T_nonCriticalExtensions_239,
    &ett_rrc_T_criticalExtensions_140,
    &ett_rrc_RFC3095_ContextInfoList_r5,
    &ett_rrc_SRNC_RelocationInfo_r3,
    &ett_rrc_T_r3_32,
    &ett_rrc_T_v380NonCriticalExtensions_02,
    &ett_rrc_T_v390NonCriticalExtensions_02,
    &ett_rrc_T_v3a0NonCriticalExtensions_16,
    &ett_rrc_T_v3b0NonCriticalExtensions,
    &ett_rrc_T_v3c0NonCriticalExtensions,
    &ett_rrc_T_laterNonCriticalExtensions_67,
    &ett_rrc_T_v3g0NonCriticalExtensions_02,
    &ett_rrc_T_v4b0NonCriticalExtensions_27,
    &ett_rrc_T_v590NonCriticalExtensions_31,
    &ett_rrc_T_v5a0NonCriticalExtensions,
    &ett_rrc_T_v5b0NonCriticalExtensions_03,
    &ett_rrc_T_v5c0NonCriticalExtensions_02,
    &ett_rrc_T_v690NonCriticalExtensions_53,
    &ett_rrc_T_v6b0NonCriticalExtensions_21,
    &ett_rrc_T_v770NonCriticalExtensions_48,
    &ett_rrc_T_v7e0NonCriticalExtensions_04,
    &ett_rrc_T_v7f0NonCriticalExtensions_17,
    &ett_rrc_T_nonCriticalExtensions_240,
    &ett_rrc_T_later_than_r3_28,
    &ett_rrc_T_r4_14,
    &ett_rrc_T_v4d0NonCriticalExtensions_14,
    &ett_rrc_T_v590NonCriticalExtensions_32,
    &ett_rrc_T_v5a0NonCriticalExtensions_01,
    &ett_rrc_T_v5b0NonCriticalExtensions_04,
    &ett_rrc_T_v5c0NonCriticalExtensions_03,
    &ett_rrc_T_v690NonCriticalExtensions_54,
    &ett_rrc_T_v6b0NonCriticalExtensions_22,
    &ett_rrc_T_v770NonCriticalExtensions_49,
    &ett_rrc_T_v7e0NonCriticalExtensions_05,
    &ett_rrc_T_v7f0NonCriticalExtensions_18,
    &ett_rrc_T_nonCriticalExtensions_241,
    &ett_rrc_T_criticalExtensions_141,
    &ett_rrc_T_r5_15,
    &ett_rrc_T_v5a0NonCriticalExtensions_02,
    &ett_rrc_T_v5b0NonCriticalExtensions_05,
    &ett_rrc_T_v5c0NonCriticalExtensions_04,
    &ett_rrc_T_v690NonCriticalExtensions_55,
    &ett_rrc_T_v6b0NonCriticalExtensions_23,
    &ett_rrc_T_v770NonCriticalExtensions_50,
    &ett_rrc_T_v7e0NonCriticalExtensions_06,
    &ett_rrc_T_v7f0NonCriticalExtensions_19,
    &ett_rrc_T_nonCriticalExtensions_242,
    &ett_rrc_T_criticalExtensions_142,
    &ett_rrc_T_r6_12,
    &ett_rrc_T_v6b0NonCriticalExtensions_24,
    &ett_rrc_T_v770NonCriticalExtensions_51,
    &ett_rrc_T_v820NonCriticalExtensions_03,
    &ett_rrc_T_v860NonCriticalExtensions_36,
    &ett_rrc_T_nonCriticalExtensions_243,
    &ett_rrc_T_criticalExtensions_143,
    &ett_rrc_T_r7_16,
    &ett_rrc_T_v820NonCriticalExtensions_04,
    &ett_rrc_T_v860NonCriticalExtensions_37,
    &ett_rrc_T_nonCriticalExtensions_244,
    &ett_rrc_T_criticalExtensions_144,
    &ett_rrc_T_r8_12,
    &ett_rrc_T_v7e0NonCriticalExtensions_07,
    &ett_rrc_T_v7f0NonCriticalExtensions_20,
    &ett_rrc_T_v8d0NonCriticalExtensions_01,
    &ett_rrc_T_nonCriticalExtensions_245,
    &ett_rrc_T_criticalExtensions_145,
    &ett_rrc_T_r9_11,
    &ett_rrc_T_v8d0NonCriticalExtensions_02,
    &ett_rrc_T_v970NonCriticalExtensions_04,
    &ett_rrc_T_v9c0NonCriticalExtensions_13,
    &ett_rrc_T_nonCriticalExtensions_246,
    &ett_rrc_T_criticalExtensions_146,
    &ett_rrc_T_r10_13,
    &ett_rrc_T_nonCriticalExtensions_247,
    &ett_rrc_T_criticalExtensions_147,
    &ett_rrc_T_r11_14,
    &ett_rrc_T_v9c0NonCriticalExtensions_14,
    &ett_rrc_T_nonCriticalExtensions_248,
    &ett_rrc_T_criticalExtensions_148,
    &ett_rrc_SRNC_RelocationInfo_r3_IEs,
    &ett_rrc_T_modeSpecificInfo_156,
    &ett_rrc_T_fdd_194,
    &ett_rrc_SRNC_RelocationInfo_v380ext_IEs,
    &ett_rrc_SRNC_RelocationInfo_v390ext_IEs,
    &ett_rrc_SRNC_RelocationInfo_v3a0ext_IEs,
    &ett_rrc_SRNC_RelocationInfo_v3b0ext_IEs,
    &ett_rrc_SRNC_RelocationInfo_v3c0ext_IEs,
    &ett_rrc_SRNC_RelocationInfo_v3d0ext_IEs,
    &ett_rrc_SRNC_RelocationInfo_v3g0ext_IEs,
    &ett_rrc_SRNC_RelocationInfo_v3h0ext_IEs,
    &ett_rrc_T_nonCriticalExtension,
    &ett_rrc_SRNC_RelocationInfo_v4d0ext_IEs,
    &ett_rrc_TPC_CombinationInfoList,
    &ett_rrc_TPC_CombinationInfoList_r9,
    &ett_rrc_STARTList2,
    &ett_rrc_SRNC_RelocationInfo_v4b0ext_IEs,
    &ett_rrc_SRNC_RelocationInfo_v590ext_IEs,
    &ett_rrc_SRNC_RelocationInfo_v5a0ext_IEs,
    &ett_rrc_SRNC_RelocationInfo_v5b0ext_IEs,
    &ett_rrc_SRNC_RelocationInfo_v5c0ext_IEs,
    &ett_rrc_CipheringInfoPerRB_List_v3a0ext,
    &ett_rrc_CipheringStatusList,
    &ett_rrc_CipheringStatusCNdomain,
    &ett_rrc_CodeChangeStatusList,
    &ett_rrc_CodeChangeStatus,
    &ett_rrc_StoredCompressedModeInfo,
    &ett_rrc_StoredCompressedModeInfo_r8,
    &ett_rrc_StoredTGP_SequenceList,
    &ett_rrc_StoredTGP_SequenceList_r8,
    &ett_rrc_StoredTGP_Sequence,
    &ett_rrc_T_current_tgps_Status,
    &ett_rrc_T_active,
    &ett_rrc_StoredTGP_Sequence_r8,
    &ett_rrc_T_current_tgps_Status_01,
    &ett_rrc_T_active_01,
    &ett_rrc_SRNC_RelocationInfo_r4_IEs,
    &ett_rrc_T_dummy_17,
    &ett_rrc_T_fdd_195,
    &ett_rrc_SRNC_RelocationInfo_r5_IEs,
    &ett_rrc_T_dummy_18,
    &ett_rrc_T_fdd_196,
    &ett_rrc_SRNC_RelocationInfo_v690ext_IEs,
    &ett_rrc_SRNC_RelocationInfo_r6_IEs,
    &ett_rrc_SRNC_RelocationInfo_r6_add_ext_IEs,
    &ett_rrc_T_v7f0NonCriticalExtensions_21,
    &ett_rrc_T_nonCriticalExtensions_249,
    &ett_rrc_SRNC_RelocationInfo_v6b0ext_IEs,
    &ett_rrc_SRNC_RelocationInfo_r7_IEs,
    &ett_rrc_SRNC_RelocationInfo_r7_add_ext_IEs,
    &ett_rrc_T_v7f0NonCriticalExtensions_22,
    &ett_rrc_T_nonCriticalExtensions_250,
    &ett_rrc_SRNC_RelocationInfo_v770ext_IEs,
    &ett_rrc_SRNC_RelocationInfo_v7e0ext_IEs,
    &ett_rrc_SRNC_RelocationInfo_v7f0ext_IEs,
    &ett_rrc_SRNC_RelocationInfo_r8_IEs,
    &ett_rrc_SRNC_RelocationInfo_v820ext_IEs,
    &ett_rrc_SRNC_RelocationInfo_v860ext_IEs,
    &ett_rrc_SRNC_RelocationInfo_v8d0ext_IEs,
    &ett_rrc_SRNC_RelocationInfo_r9_IEs,
    &ett_rrc_SRNC_RelocationInfo_v970ext_IEs,
    &ett_rrc_SRNC_RelocationInfo_v9c0ext_IEs,
    &ett_rrc_SRNC_RelocationInfo_r10_IEs,
    &ett_rrc_SRNC_RelocationInfo_r11_IEs,
    &ett_rrc_CalculationTimeForCiphering,
    &ett_rrc_CipheringInfoPerRB,
    &ett_rrc_CipheringInfoPerRB_r4,
    &ett_rrc_CipheringInfoPerRB_List,
    &ett_rrc_CipheringInfoPerRB_List_r4,
    &ett_rrc_CipheringStatusList_r4,
    &ett_rrc_CipheringStatusCNdomain_r4,
    &ett_rrc_CN_DomainInformation_v390ext,
    &ett_rrc_CN_DomainInformationList_v390ext,
    &ett_rrc_CompressedModeMeasCapability_r4,
    &ett_rrc_COUNT_C_List,
    &ett_rrc_COUNT_CSingle,
    &ett_rrc_DL_PhysChCapabilityFDD_r4,
    &ett_rrc_DL_PhysChCapabilityFDD_r5,
    &ett_rrc_T_fdd_hspdsch_01,
    &ett_rrc_T_supported_07,
    &ett_rrc_DL_PhysChCapabilityTDD_r5,
    &ett_rrc_T_tdd384_hspdsch_01,
    &ett_rrc_DL_PhysChCapabilityTDD_LCR_r5,
    &ett_rrc_T_tdd128_hspdsch_01,
    &ett_rrc_DL_RFC3095_Context,
    &ett_rrc_E_RGCH_Combination_Info,
    &ett_rrc_E_RGCH_Combination_Info_r9,
    &ett_rrc_E_RGCH_CombinationInfoList,
    &ett_rrc_E_RGCH_CombinationInfoList_r9,
    &ett_rrc_InterRAT_UE_RadioAccessCapabilityList_r5,
    &ett_rrc_IntraFreqReportingCriteria_r6_ext,
    &ett_rrc_MeasurementCapability_r4,
    &ett_rrc_MeasurementCommandWithType,
    &ett_rrc_MeasurementCommandWithType_r4,
    &ett_rrc_MeasurementCommandWithType_r6,
    &ett_rrc_MeasurementCommandWithType_r7,
    &ett_rrc_MeasurementCommandWithType_r8,
    &ett_rrc_MeasurementCommandWithType_r9,
    &ett_rrc_MeasurementCommandWithType_r10,
    &ett_rrc_MeasurementCommandWithType_r11,
    &ett_rrc_OngoingMeasRep,
    &ett_rrc_OngoingMeasRep_r4,
    &ett_rrc_OngoingMeasRep_r5,
    &ett_rrc_T_measurementCommand_v590ext_01,
    &ett_rrc_OngoingMeasRep_r6,
    &ett_rrc_OngoingMeasRep_r7,
    &ett_rrc_OngoingMeasRep_r8,
    &ett_rrc_OngoingMeasRep_r9,
    &ett_rrc_OngoingMeasRep_v970ext_IEs,
    &ett_rrc_OngoingMeasRep_r10,
    &ett_rrc_OngoingMeasRep_r11,
    &ett_rrc_OngoingMeasRepList,
    &ett_rrc_OngoingMeasRepList_r4,
    &ett_rrc_OngoingMeasRepList_r5,
    &ett_rrc_OngoingMeasRepList_r6,
    &ett_rrc_OngoingMeasRepList_r7,
    &ett_rrc_OngoingMeasRepList_r8,
    &ett_rrc_OngoingMeasRepList_r9,
    &ett_rrc_OngoingMeasRepList_v970ext_IE,
    &ett_rrc_OngoingMeasRepList_r10,
    &ett_rrc_OngoingMeasRepList_r11,
    &ett_rrc_PDCP_Capability_r4,
    &ett_rrc_T_supportForRfc2507_01,
    &ett_rrc_T_supportForRfc3095_01,
    &ett_rrc_T_supported_08,
    &ett_rrc_PDCP_Capability_r5,
    &ett_rrc_T_supportForRfc2507_02,
    &ett_rrc_T_supportForRfc3095_02,
    &ett_rrc_T_supported_09,
    &ett_rrc_PDCP_Capability_r6,
    &ett_rrc_T_supportForRfc2507_03,
    &ett_rrc_T_supportForRfc3095_03,
    &ett_rrc_T_supported_10,
    &ett_rrc_PhysicalChannelCapability_r4,
    &ett_rrc_T_fddPhysChCapability_06,
    &ett_rrc_T_tdd384_PhysChCapability,
    &ett_rrc_T_tdd128_PhysChCapability_01,
    &ett_rrc_PhysicalChannelCapability_r5,
    &ett_rrc_T_fddPhysChCapability_07,
    &ett_rrc_T_tdd384_PhysChCapability_01,
    &ett_rrc_T_tdd128_PhysChCapability_02,
    &ett_rrc_RF_Capability_r4,
    &ett_rrc_T_fddRF_Capability_04,
    &ett_rrc_T_tdd384_RF_Capability_01,
    &ett_rrc_T_tdd128_RF_Capability_01,
    &ett_rrc_RFC3095_ContextInfo,
    &ett_rrc_RFC3095_Context_List,
    &ett_rrc_RFC3095_Context_List_item,
    &ett_rrc_RLC_Capability_r5,
    &ett_rrc_SRB_SpecificIntegrityProtInfo,
    &ett_rrc_SRB_SpecificIntegrityProtInfoList,
    &ett_rrc_TPC_Combination_Info,
    &ett_rrc_TPC_Combination_Info_r9,
    &ett_rrc_UE_MultiModeRAT_Capability_r5,
    &ett_rrc_UE_Positioning_Capability_r4,
    &ett_rrc_UE_Positioning_LastKnownPos,
    &ett_rrc_UE_RadioAccessCapability_r4,
    &ett_rrc_UE_RadioAccessCapability_r5,
    &ett_rrc_UE_RadioAccessCapability_r6,
    &ett_rrc_UL_RFC3095_Context,

/*--- End of included file: packet-rrc-ettarr.c ---*/
#line 287 "../../asn1/rrc/packet-rrc-template.c"
    &ett_rrc_eutraFeatureGroupIndicators,
    &ett_rrc_cn_CommonGSM_MAP_NAS_SysInfo,
  };


  /* Register protocol */
  proto_rrc = proto_register_protocol(PNAME, PSNAME, PFNAME);
  /* Register fields and subtrees */
  proto_register_field_array(proto_rrc, hf, array_length(hf));
  proto_register_subtree_array(ett, array_length(ett));

  register_dissector("rrc", dissect_rrc, proto_rrc);


/*--- Included file: packet-rrc-dis-reg.c ---*/
#line 1 "../../asn1/rrc/packet-rrc-dis-reg.c"
  new_register_dissector("rrc.dl.dcch", dissect_DL_DCCH_Message_PDU, proto_rrc);
  new_register_dissector("rrc.ul.dcch", dissect_UL_DCCH_Message_PDU, proto_rrc);
  new_register_dissector("rrc.dl.ccch", dissect_DL_CCCH_Message_PDU, proto_rrc);
  new_register_dissector("rrc.ul.ccch", dissect_UL_CCCH_Message_PDU, proto_rrc);
  new_register_dissector("rrc.pcch", dissect_PCCH_Message_PDU, proto_rrc);
  new_register_dissector("rrc.dl.shcch", dissect_DL_SHCCH_Message_PDU, proto_rrc);
  new_register_dissector("rrc.ul.shcch", dissect_UL_SHCCH_Message_PDU, proto_rrc);
  new_register_dissector("rrc.bcch.fach", dissect_BCCH_FACH_Message_PDU, proto_rrc);
  new_register_dissector("rrc.bcch.bch", dissect_BCCH_BCH_Message_PDU, proto_rrc);
  new_register_dissector("rrc.mcch", dissect_MCCH_Message_PDU, proto_rrc);
  new_register_dissector("rrc.msch", dissect_MSCH_Message_PDU, proto_rrc);
  new_register_dissector("rrc.irat.ho_to_utran_cmd", dissect_rrc_HandoverToUTRANCommand_PDU, proto_rrc);
  new_register_dissector("rrc.irat.irat_ho_info", dissect_rrc_InterRATHandoverInfo_PDU, proto_rrc);
  new_register_dissector("rrc.sysinfo", dissect_SystemInformation_BCH_PDU, proto_rrc);
  new_register_dissector("rrc.sysinfo.cont", dissect_System_Information_Container_PDU, proto_rrc);
  new_register_dissector("rrc.ue_radio_access_cap_info", dissect_UE_RadioAccessCapabilityInfo_PDU, proto_rrc);
  new_register_dissector("rrc.si.mib", dissect_rrc_MasterInformationBlock_PDU, proto_rrc);
  new_register_dissector("rrc.si.sib1", dissect_rrc_SysInfoType1_PDU, proto_rrc);
  new_register_dissector("rrc.si.sib2", dissect_rrc_SysInfoType2_PDU, proto_rrc);
  new_register_dissector("rrc.si.sib3", dissect_rrc_SysInfoType3_PDU, proto_rrc);
  new_register_dissector("rrc.si.sib4", dissect_SysInfoType4_PDU, proto_rrc);
  new_register_dissector("rrc.si.sib5", dissect_SysInfoType5_PDU, proto_rrc);
  new_register_dissector("rrc.si.sib5bis", dissect_SysInfoType5bis_PDU, proto_rrc);
  new_register_dissector("rrc.si.sib6", dissect_SysInfoType6_PDU, proto_rrc);
  new_register_dissector("rrc.si.sib7", dissect_rrc_SysInfoType7_PDU, proto_rrc);
  new_register_dissector("rrc.si.sib8", dissect_SysInfoType8_PDU, proto_rrc);
  new_register_dissector("rrc.si.sib9", dissect_SysInfoType9_PDU, proto_rrc);
  new_register_dissector("rrc.si.sib10", dissect_SysInfoType10_PDU, proto_rrc);
  new_register_dissector("rrc.si.sib11", dissect_SysInfoType11_PDU, proto_rrc);
  new_register_dissector("rrc.si.sib11bis", dissect_SysInfoType11bis_PDU, proto_rrc);
  new_register_dissector("rrc.si.sib12", dissect_rrc_SysInfoType12_PDU, proto_rrc);
  new_register_dissector("rrc.si.sib13", dissect_SysInfoType13_PDU, proto_rrc);
  new_register_dissector("rrc.si.sib13-1", dissect_SysInfoType13_1_PDU, proto_rrc);
  new_register_dissector("rrc.si.sib13-2", dissect_SysInfoType13_2_PDU, proto_rrc);
  new_register_dissector("rrc.si.sib13-3", dissect_SysInfoType13_3_PDU, proto_rrc);
  new_register_dissector("rrc.si.sib13-4", dissect_SysInfoType13_4_PDU, proto_rrc);
  new_register_dissector("rrc.si.sib14", dissect_SysInfoType14_PDU, proto_rrc);
  new_register_dissector("rrc.si.sib15", dissect_SysInfoType15_PDU, proto_rrc);
  new_register_dissector("rrc.si.sib15bis", dissect_SysInfoType15bis_PDU, proto_rrc);
  new_register_dissector("rrc.si.sib15-1", dissect_SysInfoType15_1_PDU, proto_rrc);
  new_register_dissector("rrc.si.sib15-1bis", dissect_SysInfoType15_1bis_PDU, proto_rrc);
  new_register_dissector("rrc.si.sib15-2", dissect_SysInfoType15_2_PDU, proto_rrc);
  new_register_dissector("rrc.si.sib15-2bis", dissect_SysInfoType15_2bis_PDU, proto_rrc);
  new_register_dissector("rrc.si.sib15-2ter", dissect_SysInfoType15_2ter_PDU, proto_rrc);
  new_register_dissector("rrc.si.sib15-3", dissect_SysInfoType15_3_PDU, proto_rrc);
  new_register_dissector("rrc.si.sib15-3bis", dissect_SysInfoType15_3bis_PDU, proto_rrc);
  new_register_dissector("rrc.si.sib15-4", dissect_SysInfoType15_4_PDU, proto_rrc);
  new_register_dissector("rrc.si.sib15-5", dissect_SysInfoType15_5_PDU, proto_rrc);
  new_register_dissector("rrc.si.sib15-6", dissect_SysInfoType15_6_PDU, proto_rrc);
  new_register_dissector("rrc.si.sib15-7", dissect_SysInfoType15_7_PDU, proto_rrc);
  new_register_dissector("rrc.si.sib15-8", dissect_SysInfoType15_8_PDU, proto_rrc);
  new_register_dissector("rrc.si.sib16", dissect_SysInfoType16_PDU, proto_rrc);
  new_register_dissector("rrc.si.sib17", dissect_SysInfoType17_PDU, proto_rrc);
  new_register_dissector("rrc.si.sib18", dissect_SysInfoType18_PDU, proto_rrc);
  new_register_dissector("rrc.si.sib19", dissect_SysInfoType19_PDU, proto_rrc);
  new_register_dissector("rrc.si.sib20", dissect_SysInfoType20_PDU, proto_rrc);
  new_register_dissector("rrc.si.sib21", dissect_SysInfoType21_PDU, proto_rrc);
  new_register_dissector("rrc.si.sib22", dissect_SysInfoType22_PDU, proto_rrc);
  new_register_dissector("rrc.si.sb1", dissect_SysInfoTypeSB1_PDU, proto_rrc);
  new_register_dissector("rrc.si.sb2", dissect_SysInfoTypeSB2_PDU, proto_rrc);
  new_register_dissector("rrc.s_to_trnc_cont", dissect_rrc_ToTargetRNC_Container_PDU, proto_rrc);
  new_register_dissector("rrc.t_to_srnc_cont", dissect_rrc_TargetRNC_ToSourceRNC_Container_PDU, proto_rrc);


/*--- End of included file: packet-rrc-dis-reg.c ---*/
#line 301 "../../asn1/rrc/packet-rrc-template.c"




    register_init_routine(rrc_init);
}


/*--- proto_reg_handoff_rrc ---------------------------------------*/
void
proto_reg_handoff_rrc(void)
{
  gsm_a_dtap_handle = find_dissector("gsm_a_dtap");
  rrc_pcch_handle = find_dissector("rrc.pcch");
  rrc_ul_ccch_handle = find_dissector("rrc.ul.ccch");
  rrc_dl_ccch_handle = find_dissector("rrc.dl.ccch");
  rrc_ul_dcch_handle = find_dissector("rrc.ul.dcch");
  rrc_dl_dcch_handle = find_dissector("rrc.dl.dcch");
  rrc_ue_radio_access_cap_info_handle = find_dissector("rrc.ue_radio_access_cap_info");
  rrc_dl_dcch_handle = find_dissector("rrc.dl.dcch");
  lte_rrc_ue_eutra_cap_handle = find_dissector("lte-rrc.ue_eutra_cap");
  lte_rrc_dl_dcch_handle = find_dissector("lte-rrc.dl.dcch");
  rrc_bcch_fach_handle = find_dissector("rrc.bcch.fach");
  gsm_rlcmac_dl_handle = find_dissector("gsm_rlcmac_dl");
}


